/**
 * @version 1.0.0.0
 * @copyright Copyright ©  2019
 * @compiler Bridge.NET 17.9.0
 */
Bridge.assembly("CSHTML5", function ($asm, globals) {
    "use strict";

    /** @namespace Windows.UI.Xaml.Markup */

    /**
     * Defines a contract for how names of elements should be accessed within a
     particular XAML namescope, and how to enforce uniqueness of names within
     that XAML namescope.
     *
     * @abstract
     * @public
     * @class Windows.UI.Xaml.Markup.INameScope
     */
    Bridge.define("Windows.UI.Xaml.Markup.INameScope", {
        $kind: "interface"
    });

    /** @namespace System.ComponentModel */

    /**
     * Specifies that this object supports a simple, transacted notification for batch
     initialization.
     *
     * @abstract
     * @public
     * @class System.ComponentModel.ISupportInitialize
     */
    Bridge.define("System.ComponentModel.ISupportInitialize", {
        $kind: "interface"
    });

    /** @namespace Windows.UI.Xaml */

    /**
     * Represents an object that participates in the dependency property system. DependencyObject
     is the immediate base class of many important UI-related classes, such as
     UIElement, Geometry, FrameworkTemplate, Style, and ResourceDictionary.
     *
     * @public
     * @class Windows.UI.Xaml.DependencyObject
     */
    Bridge.define("Windows.UI.Xaml.DependencyObject", {
        fields: {
            _inheritanceContext: null,
            _inheritedParent: null,
            _contextListeners: null,
            CanBeInheritanceContext: false,
            IsInheritanceContextSealed: false,
            INTERNAL_PropertyStorageDictionary: null,
            INTERNAL_AllInheritedProperties: null,
            INTERNAL_PropertiesForWhichToCallPropertyChangedWhenLoadedIntoVisualTree: null
        },
        events: {
            InheritedContextChanged: null
        },
        props: {
            InheritedParent: {
                get: function () {
                    return this._inheritedParent;
                }
            },
            InheritanceContext: {
                get: function () {
                    return this._inheritanceContext;
                }
            },
            /**
             * Gets the CoreDispatcher that this object is associated with.
             *
             * @instance
             * @public
             * @readonly
             * @memberof Windows.UI.Xaml.DependencyObject
             * @function Dispatcher
             * @type Windows.UI.Core.CoreDispatcher
             */
            Dispatcher: {
                get: function () {
                    return Windows.UI.Core.CoreDispatcher.INTERNAL_GetCurrentDispatcher();
                }
            }
        },
        ctors: {
            init: function () {
                this._contextListeners = new (System.Collections.Generic.HashSet$1(Windows.UI.Xaml.DependencyObject)).ctor();
            },
            ctor: function () {
                this.$initialize();
                this.CanBeInheritanceContext = true;
                this.INTERNAL_PropertyStorageDictionary = new (System.Collections.Generic.Dictionary$2(Windows.UI.Xaml.DependencyProperty,CSHTML5.Internal.INTERNAL_PropertyStorage)).ctor();
                this.INTERNAL_AllInheritedProperties = new (System.Collections.Generic.Dictionary$2(Windows.UI.Xaml.DependencyProperty,CSHTML5.Internal.INTERNAL_PropertyStorage)).ctor();
            }
        },
        methods: {
            ShouldProvideInheritanceContext: function (target, property) {
                return !(Bridge.is(target, Windows.UI.Xaml.FrameworkElement));
            },
            RemoveSelfAsInheritanceContext: function (value, dp) {
                var doValue = Bridge.as(value, Windows.UI.Xaml.DependencyObject);
                if (doValue != null) {
                    return this.RemoveSelfAsInheritanceContext$1(doValue, dp);
                } else {
                    return false;
                }
            },
            RemoveSelfAsInheritanceContext$1: function (doValue, dp) {
                if (doValue != null && this.ShouldProvideInheritanceContext(doValue, dp) && this.CanBeInheritanceContext && !doValue.IsInheritanceContextSealed) {
                    var oldInheritanceContext = doValue.InheritanceContext;
                    if (Bridge.referenceEquals(this, oldInheritanceContext)) {
                        doValue.RemoveInheritanceContext(this, dp);

                        return true;
                    } else {
                        return false;
                    }
                } else {
                    return false;
                }
            },
            RemoveInheritanceContext: function (context, property) {
                context.StopListeningToInheritanceContextChanges(this);

                this._inheritanceContext = null;

                this._inheritedParent = null;

                this.OnInheritedContextChanged(null);
            },
            ProvideSelfAsInheritanceContext: function (value, dp) {
                var doValue = Bridge.as(value, Windows.UI.Xaml.DependencyObject);
                if (doValue != null) {
                    return this.ProvideSelfAsInheritanceContext$1(doValue, dp);
                } else {
                    return false;
                }
            },
            ProvideSelfAsInheritanceContext$1: function (doValue, dp) {
                if (doValue != null && this.ShouldProvideInheritanceContext(doValue, dp) && this.CanBeInheritanceContext && !doValue.IsInheritanceContextSealed) {
                    if (doValue.InheritanceContext != null) {
                        return false;
                    } else {
                        doValue.AddInheritanceContext(this, dp);

                        return true;
                    }
                } else {
                    return false;
                }
            },
            AddInheritanceContext: function (context, property) {
                context.ListenToInheritanceContextChanges(this);

                this._inheritanceContext = context;

                this._inheritedParent = this.ComputeInheritedParent();

                this.OnInheritedContextChanged(this._inheritedParent);
            },
            OnInheritedContextChanged: function (newParent) {
                var $t;
                if (!Bridge.staticEquals(this.InheritedContextChanged, null)) {
                    this.InheritedContextChanged(this, { });
                }
                $t = Bridge.getEnumerator(this._contextListeners);
                try {
                    while ($t.moveNext()) {
                        var listener = $t.Current;
                        listener._inheritedParent = newParent;
                        listener.OnInheritedContextChanged(newParent);
                    }
                } finally {
                    if (Bridge.is($t, System.IDisposable)) {
                        $t.System$IDisposable$Dispose();
                    }
                }
            },
            ComputeInheritedParent: function () {
                var inheritedParent;
                for (var contextDO = this._inheritanceContext; contextDO != null; contextDO = contextDO._inheritanceContext) {
                    if (((inheritedParent = Bridge.as(contextDO, Windows.UI.Xaml.FrameworkElement))) != null) {
                        return inheritedParent;
                    }
                }
                return null;
            },
            ListenToInheritanceContextChanges: function (listener) {
                this._contextListeners.add(listener);
            },
            StopListeningToInheritanceContextChanges: function (listener) {
                var isListening = this._contextListeners.contains(listener);
                if (isListening) {
                    this._contextListeners.remove(listener);
                }
            },
            /**
             * Returns the current effective value of a dependency property from a DependencyObject.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.DependencyObject
             * @memberof Windows.UI.Xaml.DependencyObject
             * @param   {Windows.UI.Xaml.DependencyProperty}    dependencyProperty    The DependencyProperty identifier of the property for which to retrieve the
             value.
             * @return  {System.Object}                                               Returns the current effective value.
             */
            GetValue: function (dependencyProperty) {
                var storage = { };
                if (CSHTML5.Internal.INTERNAL_PropertyStore.TryGetStorage(this, dependencyProperty, false, storage)) {
                    return CSHTML5.Internal.INTERNAL_PropertyStore.GetEffectiveValue(storage.v);
                }
                return dependencyProperty.GetTypeMetaData(Bridge.getType(this)).DefaultValue;
            },
            /**
             * Sets the local value of a dependency property on a DependencyObject while not overriding a hypothetical Binding (example: when the user writes in a TextBox with a two way Binding on its Text property).
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.DependencyObject
             * @memberof Windows.UI.Xaml.DependencyObject
             * @param   {Windows.UI.Xaml.DependencyProperty}    dependencyProperty    The identifier of the dependency property to set.
             * @param   {System.Object}                         value                 The new local value.
             * @return  {void}
             */
            SetLocalValue: function (dependencyProperty, value) {
                this.SetCurrentValue(dependencyProperty, value);
            },
            SetCurrentValue: function (dp, value) {
                var storage = { };
                CSHTML5.Internal.INTERNAL_PropertyStore.TryGetStorage(this, dp, true, storage);
                CSHTML5.Internal.INTERNAL_PropertyStore.SetValueCommon(storage.v, value, true);
            },
            CoerceCurrentValue: function (dependencyProperty, propertyMetadata) {
                var storage = { };
                CSHTML5.Internal.INTERNAL_PropertyStore.TryGetStorage(this, dependencyProperty, true, storage);
                CSHTML5.Internal.INTERNAL_PropertyStore.CoerceValueCommon(storage.v);
            },
            /**
             * Returns the local value of a dependency property, if a local value is set.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.DependencyObject
             * @memberof Windows.UI.Xaml.DependencyObject
             * @param   {Windows.UI.Xaml.DependencyProperty}    dp    The DependencyProperty identifier of the property for which to retrieve the
             local value.
             * @return  {System.Object}                               Returns the local value, or returns the sentinel value UnsetValue if no local
             value is set.
             */
            ReadLocalValue: function (dp) {
                var storage = { };
                if (CSHTML5.Internal.INTERNAL_PropertyStore.TryGetStorage(this, dp, false, storage)) {
                    if (!Bridge.referenceEquals(storage.v.LocalValue, Windows.UI.Xaml.DependencyProperty.UnsetValue)) {
                        return storage.v.LocalValue;
                    } else if (!Bridge.referenceEquals(storage.v.LocalStyleValue, Windows.UI.Xaml.DependencyProperty.UnsetValue)) {
                        var be = Bridge.as(storage.v.LocalStyleValue, Windows.UI.Xaml.Data.BindingExpression);
                        if (be != null) {
                            return be;
                        }
                    } else {
                        var be1 = Bridge.as(storage.v.ThemeStyleValue, Windows.UI.Xaml.Data.BindingExpression);
                        if (be1 != null) {
                            return be1;
                        }
                    }
                    return storage.v.LocalValue;
                }
                return Windows.UI.Xaml.DependencyProperty.UnsetValue;
            },
            ReadLocalValueInternal: function (dp) {
                var storage = { };
                if (CSHTML5.Internal.INTERNAL_PropertyStore.TryGetStorage(this, dp, false, storage)) {
                    return storage.v.LocalValue;
                }
                return Windows.UI.Xaml.DependencyProperty.UnsetValue;
            },
            GetVisualStateValue: function (dependencyProperty) {
                if (dependencyProperty == null) {
                    throw new System.ArgumentNullException.$ctor1("No property specified");
                }
                var storage = { };
                if (CSHTML5.Internal.INTERNAL_PropertyStore.TryGetStorage(this, dependencyProperty, false, storage)) {
                    return storage.v.AnimatedValue;
                }
                return dependencyProperty.GetTypeMetaData(Bridge.getType(this)).DefaultValue;
            },
            SetVisualStateValue: function (dependencyProperty, value) {
                if (dependencyProperty == null) {
                    throw new System.ArgumentNullException.$ctor1("No property specified");
                }

                var storage = { };
                CSHTML5.Internal.INTERNAL_PropertyStore.TryGetStorage(this, dependencyProperty, true, storage);
                CSHTML5.Internal.INTERNAL_PropertyStore.SetAnimationValue(storage.v, value);
            },
            SetAnimationValue: function (dependencyProperty, value) {
                if (dependencyProperty == null) {
                    throw new System.ArgumentNullException.$ctor1("No property specified");
                }

                var storage = { };
                CSHTML5.Internal.INTERNAL_PropertyStore.TryGetStorage(this, dependencyProperty, true, storage);
                CSHTML5.Internal.INTERNAL_PropertyStore.SetAnimationValue(storage.v, value);
            },
            GetAnimationValue: function (dependencyProperty) {
                if (dependencyProperty == null) {
                    throw new System.ArgumentNullException.$ctor1("No property specified");
                }

                var storage = { };
                if (CSHTML5.Internal.INTERNAL_PropertyStore.TryGetStorage(this, dependencyProperty, false, storage)) {
                    return storage.v.AnimatedValue;
                }
                return dependencyProperty.GetTypeMetaData(Bridge.getType(this)).DefaultValue;
            },
            SetValue: function (dp, value) {
                if (dp == null) {
                    throw new System.ArgumentNullException.$ctor1("No property specified");
                }

                var storage = { };
                CSHTML5.Internal.INTERNAL_PropertyStore.TryGetStorage(this, dp, true, storage);
                CSHTML5.Internal.INTERNAL_PropertyStore.SetValueCommon(storage.v, value, false);
            },
            SetLocalStyleValue: function (dp, value) {
                var storage = { };
                if (CSHTML5.Internal.INTERNAL_PropertyStore.TryGetStorage(this, dp, !Bridge.referenceEquals(value, Windows.UI.Xaml.DependencyProperty.UnsetValue), storage)) {
                    CSHTML5.Internal.INTERNAL_PropertyStore.SetLocalStyleValue(storage.v, value);
                }
            },
            SetImplicitReferenceValue: function (dp, value) {
                var storage = { };
                if (CSHTML5.Internal.INTERNAL_PropertyStore.TryGetStorage(this, dp, !Bridge.referenceEquals(value, Windows.UI.Xaml.DependencyProperty.UnsetValue), storage)) {
                    CSHTML5.Internal.INTERNAL_PropertyStore.SetImplicitReferenceValue(storage.v, value);
                }
            },
            SetThemeStyleValue: function (dp, value) {
                var storage = { };
                if (CSHTML5.Internal.INTERNAL_PropertyStore.TryGetStorage(this, dp, !Bridge.referenceEquals(value, Windows.UI.Xaml.DependencyProperty.UnsetValue), storage)) {
                    CSHTML5.Internal.INTERNAL_PropertyStore.SetThemeStyleValue(storage.v, value);
                }
            },
            /**
             * Sets the inherited value of a dependency property on a DependencyObject. Do not use this method.
             *
             * @instance
             * @this Windows.UI.Xaml.DependencyObject
             * @memberof Windows.UI.Xaml.DependencyObject
             * @param   {Windows.UI.Xaml.DependencyProperty}    dp             The identifier of the dependency property to set.
             * @param   {System.Object}                         value          The new local value.
             * @param   {boolean}                               recursively    Specifies if the inherited value must be applied to the children of this DependencyObject.
             * @return  {void}
             */
            SetInheritedValue: function (dp, value, recursively) {
                var storage = { };
                CSHTML5.Internal.INTERNAL_PropertyStore.TryGetInheritedPropertyStorage(this, dp, true, storage);
                CSHTML5.Internal.INTERNAL_PropertyStore.SetInheritedValue(storage.v, value, recursively);
            },
            /**
             * Refreshes the value of the given DependencyProperty on this DependencyObject so that it fits the coercion that should be applied on it.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.DependencyObject
             * @memberof Windows.UI.Xaml.DependencyObject
             * @param   {Windows.UI.Xaml.DependencyProperty}    dependencyProperty    The dependencyProperty whose value we want to refresh.
             * @return  {void}
             */
            Coerce: function (dependencyProperty) {
                this.CoerceValue(dependencyProperty);
            },
            CoerceValue: function (dp) {
                if (dp == null) {
                    throw new System.ArgumentNullException.$ctor1("No property specified.");
                }

                var storage = { };
                CSHTML5.Internal.INTERNAL_PropertyStore.TryGetStorage(this, dp, true, storage);
                CSHTML5.Internal.INTERNAL_PropertyStore.CoerceValueCommon(storage.v);
            },
            ResetInheritedProperties: function () {
                var $t;
                var storages = System.Linq.Enumerable.from(this.INTERNAL_AllInheritedProperties.Values).ToArray();
                $t = Bridge.getEnumerator(storages);
                try {
                    while ($t.moveNext()) {
                        var storage = $t.Current;
                        CSHTML5.Internal.INTERNAL_PropertyStore.SetInheritedValue(storage, Windows.UI.Xaml.DependencyProperty.UnsetValue, false);
                        if (storage.BaseValueSourceInternal === CSHTML5.Internal.BaseValueSourceInternal.Default) {
                            this.INTERNAL_AllInheritedProperties.remove(storage.Property);
                            this.INTERNAL_PropertyStorageDictionary.remove(storage.Property);
                        }
                    }
                } finally {
                    if (Bridge.is($t, System.IDisposable)) {
                        $t.System$IDisposable$Dispose();
                    }
                }
            },
            ApplyBindingExpression: function (dp, expression) {
                var storage = { };
                CSHTML5.Internal.INTERNAL_PropertyStore.TryGetStorage(this, dp, true, storage);
                CSHTML5.Internal.INTERNAL_PropertyStore.RefreshBindingExpressionCommon(storage.v, expression);
            },
            INTERNAL_UpdateBindingsSource: function () {
                var $t;
                var copyOfCollection = System.Linq.Enumerable.from(System.Linq.Enumerable.from(this.INTERNAL_PropertyStorageDictionary).select(function (kp) {
                            return kp.value;
                        })).ToArray();
                $t = Bridge.getEnumerator(copyOfCollection);
                try {
                    while ($t.moveNext()) {
                        var storage = $t.Current;
                        if (storage.IsExpression) {
                            Bridge.cast(storage.LocalValue, Windows.UI.Xaml.Data.BindingExpression).OnSourceAvailable();
                        } else if (storage.IsExpressionFromStyle) {
                            Bridge.cast(storage.ModifiedValue.BaseValue, Windows.UI.Xaml.Data.BindingExpression).OnSourceAvailable();
                        }
                    }
                } finally {
                    if (Bridge.is($t, System.IDisposable)) {
                        $t.System$IDisposable$Dispose();
                    }
                }
            },
            /**
             * @instance
             * @this Windows.UI.Xaml.DependencyObject
             * @memberof Windows.UI.Xaml.DependencyObject
             * @return  {void}
             */
            INTERNAL_OnAttachedToVisualTree: function () {

            },
            /**
             * @instance
             * @this Windows.UI.Xaml.DependencyObject
             * @memberof Windows.UI.Xaml.DependencyObject
             * @return  {void}
             */
            INTERNAL_OnDetachedFromVisualTree: function () { },
            /**
             * Clears the local value of a property
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.DependencyObject
             * @memberof Windows.UI.Xaml.DependencyObject
             * @param   {Windows.UI.Xaml.DependencyProperty}    dp
             * @return  {void}
             */
            ClearValue: function (dp) {
                var storage = { };
                if (CSHTML5.Internal.INTERNAL_PropertyStore.TryGetStorage(this, dp, false, storage)) {
                    CSHTML5.Internal.INTERNAL_PropertyStore.ClearValueCommon(storage.v);
                }
            }
        }
    });

    Bridge.define("\u01c0\u01c0Cshtml5\u01c0\u01c0Component\u01c0\u01c0Defaultstyles\u01c0\u01c0Internal_Defaultautocompleteboxstyle\u01c0\u01c0Xaml\u01c0\u01c0Factory", {
        statics: {
            methods: {
                Instantiate: function () {
                    var type = ___CSHTML5__Component__Defaultstyles__INTERNAL_Defaultautocompleteboxstyle__Xaml;
                    return Bridge.global.CSHTML5.Internal.TypeInstantiationHelper.Instantiate(type);
                }
            }
        }
    });

    Bridge.define("\u01c0\u01c0Cshtml5\u01c0\u01c0Component\u01c0\u01c0Defaultstyles\u01c0\u01c0Internal_Defaultbusyindicatorstyle\u01c0\u01c0Xaml\u01c0\u01c0Factory", {
        statics: {
            methods: {
                Instantiate: function () {
                    var type = ___CSHTML5__Component__Defaultstyles__INTERNAL_Defaultbusyindicatorstyle__Xaml;
                    return Bridge.global.CSHTML5.Internal.TypeInstantiationHelper.Instantiate(type);
                }
            }
        }
    });

    Bridge.define("\u01c0\u01c0Cshtml5\u01c0\u01c0Component\u01c0\u01c0Defaultstyles\u01c0\u01c0Internal_Defaultbuttonstyle\u01c0\u01c0Xaml\u01c0\u01c0Factory", {
        statics: {
            methods: {
                Instantiate: function () {
                    var type = ___CSHTML5__Component__Defaultstyles__INTERNAL_Defaultbuttonstyle__Xaml;
                    return Bridge.global.CSHTML5.Internal.TypeInstantiationHelper.Instantiate(type);
                }
            }
        }
    });

    Bridge.define("\u01c0\u01c0Cshtml5\u01c0\u01c0Component\u01c0\u01c0Defaultstyles\u01c0\u01c0Internal_Defaultchildwindowstyle\u01c0\u01c0Xaml\u01c0\u01c0Factory", {
        statics: {
            methods: {
                Instantiate: function () {
                    var type = ___CSHTML5__Component__Defaultstyles__INTERNAL_Defaultchildwindowstyle__Xaml;
                    return Bridge.global.CSHTML5.Internal.TypeInstantiationHelper.Instantiate(type);
                }
            }
        }
    });

    Bridge.define("\u01c0\u01c0Cshtml5\u01c0\u01c0Component\u01c0\u01c0Defaultstyles\u01c0\u01c0Internal_Defaultcomboboxstyle\u01c0\u01c0Xaml\u01c0\u01c0Factory", {
        statics: {
            methods: {
                Instantiate: function () {
                    var type = ___CSHTML5__Component__Defaultstyles__INTERNAL_Defaultcomboboxstyle__Xaml;
                    return Bridge.global.CSHTML5.Internal.TypeInstantiationHelper.Instantiate(type);
                }
            }
        }
    });

    Bridge.define("\u01c0\u01c0Cshtml5\u01c0\u01c0Component\u01c0\u01c0Defaultstyles\u01c0\u01c0Internal_Defaultdatapagerstyle\u01c0\u01c0Xaml\u01c0\u01c0Factory", {
        statics: {
            methods: {
                Instantiate: function () {
                    var type = ___CSHTML5__Component__Defaultstyles__INTERNAL_Defaultdatapagerstyle__Xaml;
                    return Bridge.global.CSHTML5.Internal.TypeInstantiationHelper.Instantiate(type);
                }
            }
        }
    });

    Bridge.define("\u01c0\u01c0Cshtml5\u01c0\u01c0Component\u01c0\u01c0Defaultstyles\u01c0\u01c0Internal_Defaultdatepickerstyle\u01c0\u01c0Xaml\u01c0\u01c0Factory", {
        statics: {
            methods: {
                Instantiate: function () {
                    var type = ___CSHTML5__Component__Defaultstyles__INTERNAL_Defaultdatepickerstyle__Xaml;
                    return Bridge.global.CSHTML5.Internal.TypeInstantiationHelper.Instantiate(type);
                }
            }
        }
    });

    Bridge.define("\u01c0\u01c0Cshtml5\u01c0\u01c0Component\u01c0\u01c0Defaultstyles\u01c0\u01c0Internal_Defaultexpanderstyle\u01c0\u01c0Xaml\u01c0\u01c0Factory", {
        statics: {
            methods: {
                Instantiate: function () {
                    var type = ___CSHTML5__Component__Defaultstyles__INTERNAL_Defaultexpanderstyle__Xaml;
                    return Bridge.global.CSHTML5.Internal.TypeInstantiationHelper.Instantiate(type);
                }
            }
        }
    });

    Bridge.define("\u01c0\u01c0Cshtml5\u01c0\u01c0Component\u01c0\u01c0Defaultstyles\u01c0\u01c0Internal_Defaultgridsplitterstyle\u01c0\u01c0Xaml\u01c0\u01c0Factory", {
        statics: {
            methods: {
                Instantiate: function () {
                    var type = ___CSHTML5__Component__Defaultstyles__INTERNAL_Defaultgridsplitterstyle__Xaml;
                    return Bridge.global.CSHTML5.Internal.TypeInstantiationHelper.Instantiate(type);
                }
            }
        }
    });

    Bridge.define("\u01c0\u01c0Cshtml5\u01c0\u01c0Component\u01c0\u01c0Defaultstyles\u01c0\u01c0Internal_Defaulthyperlinkbuttonstyle\u01c0\u01c0Xaml\u01c0\u01c0Factory", {
        statics: {
            methods: {
                Instantiate: function () {
                    var type = ___CSHTML5__Component__Defaultstyles__INTERNAL_Defaulthyperlinkbuttonstyle__Xaml;
                    return Bridge.global.CSHTML5.Internal.TypeInstantiationHelper.Instantiate(type);
                }
            }
        }
    });

    Bridge.define("\u01c0\u01c0Cshtml5\u01c0\u01c0Component\u01c0\u01c0Defaultstyles\u01c0\u01c0Internal_Defaultitemscontrolstyle\u01c0\u01c0Xaml\u01c0\u01c0Factory", {
        statics: {
            methods: {
                Instantiate: function () {
                    var type = ___CSHTML5__Component__Defaultstyles__INTERNAL_Defaultitemscontrolstyle__Xaml;
                    return Bridge.global.CSHTML5.Internal.TypeInstantiationHelper.Instantiate(type);
                }
            }
        }
    });

    Bridge.define("\u01c0\u01c0Cshtml5\u01c0\u01c0Component\u01c0\u01c0Defaultstyles\u01c0\u01c0Internal_Defaultlabelstyle\u01c0\u01c0Xaml\u01c0\u01c0Factory", {
        statics: {
            methods: {
                Instantiate: function () {
                    var type = ___CSHTML5__Component__Defaultstyles__INTERNAL_Defaultlabelstyle__Xaml;
                    return Bridge.global.CSHTML5.Internal.TypeInstantiationHelper.Instantiate(type);
                }
            }
        }
    });

    Bridge.define("\u01c0\u01c0Cshtml5\u01c0\u01c0Component\u01c0\u01c0Defaultstyles\u01c0\u01c0Internal_Defaultmenuitemstyle\u01c0\u01c0Xaml\u01c0\u01c0Factory", {
        statics: {
            methods: {
                Instantiate: function () {
                    var type = ___CSHTML5__Component__Defaultstyles__INTERNAL_Defaultmenuitemstyle__Xaml;
                    return Bridge.global.CSHTML5.Internal.TypeInstantiationHelper.Instantiate(type);
                }
            }
        }
    });

    Bridge.define("\u01c0\u01c0Cshtml5\u01c0\u01c0Component\u01c0\u01c0Defaultstyles\u01c0\u01c0Internal_Defaultnumericupdownstyle\u01c0\u01c0Xaml\u01c0\u01c0Factory", {
        statics: {
            methods: {
                Instantiate: function () {
                    var type = ___CSHTML5__Component__Defaultstyles__INTERNAL_Defaultnumericupdownstyle__Xaml;
                    return Bridge.global.CSHTML5.Internal.TypeInstantiationHelper.Instantiate(type);
                }
            }
        }
    });

    Bridge.define("\u01c0\u01c0Cshtml5\u01c0\u01c0Component\u01c0\u01c0Defaultstyles\u01c0\u01c0Internal_Defaultrepeatbuttonstyle\u01c0\u01c0Xaml\u01c0\u01c0Factory", {
        statics: {
            methods: {
                Instantiate: function () {
                    var type = ___CSHTML5__Component__Defaultstyles__INTERNAL_Defaultrepeatbuttonstyle__Xaml;
                    return Bridge.global.CSHTML5.Internal.TypeInstantiationHelper.Instantiate(type);
                }
            }
        }
    });

    Bridge.define("\u01c0\u01c0Cshtml5\u01c0\u01c0Component\u01c0\u01c0Defaultstyles\u01c0\u01c0Internal_Defaultscrollbarstyle\u01c0\u01c0Xaml\u01c0\u01c0Factory", {
        statics: {
            methods: {
                Instantiate: function () {
                    var type = ___CSHTML5__Component__Defaultstyles__INTERNAL_Defaultscrollbarstyle__Xaml;
                    return Bridge.global.CSHTML5.Internal.TypeInstantiationHelper.Instantiate(type);
                }
            }
        }
    });

    Bridge.define("\u01c0\u01c0Cshtml5\u01c0\u01c0Component\u01c0\u01c0Defaultstyles\u01c0\u01c0Internal_Defaultsliderstyle\u01c0\u01c0Xaml\u01c0\u01c0Factory", {
        statics: {
            methods: {
                Instantiate: function () {
                    var type = ___CSHTML5__Component__Defaultstyles__INTERNAL_Defaultsliderstyle__Xaml;
                    return Bridge.global.CSHTML5.Internal.TypeInstantiationHelper.Instantiate(type);
                }
            }
        }
    });

    Bridge.define("\u01c0\u01c0Cshtml5\u01c0\u01c0Component\u01c0\u01c0Defaultstyles\u01c0\u01c0Internal_Defaulttabcontrolstyle\u01c0\u01c0Xaml\u01c0\u01c0Factory", {
        statics: {
            methods: {
                Instantiate: function () {
                    var type = ___CSHTML5__Component__Defaultstyles__INTERNAL_Defaulttabcontrolstyle__Xaml;
                    return Bridge.global.CSHTML5.Internal.TypeInstantiationHelper.Instantiate(type);
                }
            }
        }
    });

    Bridge.define("\u01c0\u01c0Cshtml5\u01c0\u01c0Component\u01c0\u01c0Defaultstyles\u01c0\u01c0Internal_Defaulttabitemstyle\u01c0\u01c0Xaml\u01c0\u01c0Factory", {
        statics: {
            methods: {
                Instantiate: function () {
                    var type = ___CSHTML5__Component__Defaultstyles__INTERNAL_Defaulttabitemstyle__Xaml;
                    return Bridge.global.CSHTML5.Internal.TypeInstantiationHelper.Instantiate(type);
                }
            }
        }
    });

    Bridge.define("\u01c0\u01c0Cshtml5\u01c0\u01c0Component\u01c0\u01c0Defaultstyles\u01c0\u01c0Internal_Defaultthumbstyle\u01c0\u01c0Xaml\u01c0\u01c0Factory", {
        statics: {
            methods: {
                Instantiate: function () {
                    var type = ___CSHTML5__Component__Defaultstyles__INTERNAL_Defaultthumbstyle__Xaml;
                    return Bridge.global.CSHTML5.Internal.TypeInstantiationHelper.Instantiate(type);
                }
            }
        }
    });

    Bridge.define("\u01c0\u01c0Cshtml5\u01c0\u01c0Component\u01c0\u01c0Defaultstyles\u01c0\u01c0Internal_Defaulttileviewitemstyle\u01c0\u01c0Xaml\u01c0\u01c0Factory", {
        statics: {
            methods: {
                Instantiate: function () {
                    var type = ___CSHTML5__Component__Defaultstyles__INTERNAL_Defaulttileviewitemstyle__Xaml;
                    return Bridge.global.CSHTML5.Internal.TypeInstantiationHelper.Instantiate(type);
                }
            }
        }
    });

    Bridge.define("\u01c0\u01c0Cshtml5\u01c0\u01c0Component\u01c0\u01c0Defaultstyles\u01c0\u01c0Internal_Defaulttileviewstyle\u01c0\u01c0Xaml\u01c0\u01c0Factory", {
        statics: {
            methods: {
                Instantiate: function () {
                    var type = ___CSHTML5__Component__Defaultstyles__INTERNAL_Defaulttileviewstyle__Xaml;
                    return Bridge.global.CSHTML5.Internal.TypeInstantiationHelper.Instantiate(type);
                }
            }
        }
    });

    Bridge.define("\u01c0\u01c0Cshtml5\u01c0\u01c0Component\u01c0\u01c0Defaultstyles\u01c0\u01c0Internal_Defaulttimepickerstyle\u01c0\u01c0Xaml\u01c0\u01c0Factory", {
        statics: {
            methods: {
                Instantiate: function () {
                    var type = ___CSHTML5__Component__Defaultstyles__INTERNAL_Defaulttimepickerstyle__Xaml;
                    return Bridge.global.CSHTML5.Internal.TypeInstantiationHelper.Instantiate(type);
                }
            }
        }
    });

    Bridge.define("\u01c0\u01c0Cshtml5\u01c0\u01c0Component\u01c0\u01c0Defaultstyles\u01c0\u01c0Internal_Defaulttogglebuttonstyle\u01c0\u01c0Xaml\u01c0\u01c0Factory", {
        statics: {
            methods: {
                Instantiate: function () {
                    var type = ___CSHTML5__Component__Defaultstyles__INTERNAL_Defaulttogglebuttonstyle__Xaml;
                    return Bridge.global.CSHTML5.Internal.TypeInstantiationHelper.Instantiate(type);
                }
            }
        }
    });

    Bridge.define("\u01c0\u01c0Cshtml5\u01c0\u01c0Component\u01c0\u01c0Defaultstyles\u01c0\u01c0Internal_Defaulttooltipstyle\u01c0\u01c0Xaml\u01c0\u01c0Factory", {
        statics: {
            methods: {
                Instantiate: function () {
                    var type = ___CSHTML5__Component__Defaultstyles__INTERNAL_Defaulttooltipstyle__Xaml;
                    return Bridge.global.CSHTML5.Internal.TypeInstantiationHelper.Instantiate(type);
                }
            }
        }
    });

    Bridge.define("\u01c0\u01c0Cshtml5\u01c0\u01c0Component\u01c0\u01c0Defaultstyles\u01c0\u01c0Internal_Defaultviewboxstyle\u01c0\u01c0Xaml\u01c0\u01c0Factory", {
        statics: {
            methods: {
                Instantiate: function () {
                    var type = ___CSHTML5__Component__Defaultstyles__INTERNAL_Defaultviewboxstyle__Xaml;
                    return Bridge.global.CSHTML5.Internal.TypeInstantiationHelper.Instantiate(type);
                }
            }
        }
    });

    Bridge.define("\u01c0\u01c0Cshtml5\u01c0\u01c0Component\u01c0\u01c0Themes\u01c0\u01c0Generic\u01c0\u01c0Xaml\u01c0\u01c0Factory", {
        statics: {
            methods: {
                Instantiate: function () {
                    var type = ___CSHTML5__Component__Themes__Generic__Xaml;
                    return Bridge.global.CSHTML5.Internal.TypeInstantiationHelper.Instantiate(type);
                }
            }
        }
    });

    Bridge.define("AnonymousTypes.AnonymousType0");

    Bridge.define("CSharpXamlForHtml5");

    /**
     * This class allows the management of the Dom tree without using VerbatimExpressions
     *
     * @static
     * @abstract
     * @public
     * @class CSharpXamlForHtml5.DomManagement
     */
    Bridge.define("CSharpXamlForHtml5.DomManagement", {
        $kind: "nested class",
        statics: {
            methods: {
                /**
                 * Checks if the UIElement is in the visual tree.
                 *
                 * @static
                 * @public
                 * @this CSharpXamlForHtml5.DomManagement
                 * @memberof CSharpXamlForHtml5.DomManagement
                 * @param   {Windows.UI.Xaml.UIElement}    control    The UIElement
                 * @return  {boolean}                                 True if the UIElement is currently in the visual tree, false otherwise.
                 */
                IsControlInVisualTree: function (control) {
                    return CSHTML5.Internal.INTERNAL_VisualTreeManager.IsElementInVisualTree(control);
                },
                /**
                 * Returns the DOM element corresponding to the UIElement.
                 *
                 * @static
                 * @public
                 * @this CSharpXamlForHtml5.DomManagement
                 * @memberof CSharpXamlForHtml5.DomManagement
                 * @param   {Windows.UI.Xaml.UIElement}    control    The UIElement
                 * @return  {object}                                  The DOM element corresponding to the UIElement. It is the DOM Element that contains all of the UIElement's DOM elements.
                 */
                GetDomElementFromControl: function (control) {
                    if (control.INTERNAL_OuterDomElement == null) {
                        throw new System.InvalidOperationException.$ctor1("Cannot get the DOM element because the control is not (yet?) in the visual tree. Consider waiting until the Loaded event before calling this piece of code. You can also call the 'IsControlInVisualTree' method to check if the control is in the visual tree.");
                    }

                    if (CSharpXamlForHtml5.Environment.IsRunningInJavaScript) {
                        return control.INTERNAL_OuterDomElement;
                    } else {
                        return new CSharpXamlForHtml5.DomManagement.Types.DynamicDomElement(control.INTERNAL_OuterDomElement);
                    }
                },
                /**
                 * Sets the Html representation of the UIElement.
                 *
                 * @static
                 * @public
                 * @this CSharpXamlForHtml5.DomManagement
                 * @memberof CSharpXamlForHtml5.DomManagement
                 * @param   {Windows.UI.Xaml.UIElement}    control                The UIElement
                 * @param   {string}                       htmlReprensentation    The string that defines the html representation of the UIElement.
                 * @return  {void}
                 */
                SetHtmlRepresentation: function (control, htmlReprensentation) {
                    control.INTERNAL_HtmlRepresentation = htmlReprensentation;
                }
            }
        }
    });

    Bridge.define("CSharpXamlForHtml5.DomManagement.Types", {
        $kind: "nested class"
    });

    /**
     * A class that allows to know the current Environnement (between Running in C# or in Javascript)
     *
     * @static
     * @abstract
     * @public
     * @class CSharpXamlForHtml5.Environment
     */
    Bridge.define("CSharpXamlForHtml5.Environment", {
        $kind: "nested class",
        statics: {
            props: {
                /**
                 * Gets a boolean saying if we are currently running in Javascript.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof CSharpXamlForHtml5.Environment
                 * @function IsRunningInJavaScript
                 * @type boolean
                 */
                IsRunningInJavaScript: {
                    get: function () {
                        return !false;
                    }
                }
            }
        }
    });

    /** @namespace CSHTML5.Internal.Attributes */

    /**
     * Specifies whether the project was compiled in the "SLMigration" mode.
     *
     * @public
     * @class CSHTML5.Internal.Attributes.CompilerIsSLMigrationAttribute
     * @augments System.Attribute
     */
    Bridge.define("CSHTML5.Internal.Attributes.CompilerIsSLMigrationAttribute", {
        inherits: [System.Attribute],
        fields: {
            /**
             * Gets a boolean that is true if the project compiled in the "SLMigration" mode, and false otherwise.
             *
             * @instance
             * @public
             * @memberof CSHTML5.Internal.Attributes.CompilerIsSLMigrationAttribute
             * @function IsSLMigration
             * @type boolean
             */
            IsSLMigration: false
        },
        ctors: {
            /**
             * Initializes a new instance of the CompilerIsSLMigrationAttribute class with a
             boolean specifying whether the project was compiled in the "SLMigration" mode.
             *
             * @instance
             * @public
             * @this CSHTML5.Internal.Attributes.CompilerIsSLMigrationAttribute
             * @memberof CSHTML5.Internal.Attributes.CompilerIsSLMigrationAttribute
             * @param   {boolean}    isSLMigration    A boolean that is true if the project compiled in the "SLMigration" mode, and false otherwise.
             * @return  {void}
             */
            ctor: function (isSLMigration) {
                this.$initialize();
                System.Attribute.ctor.call(this);
                this.IsSLMigration = isSLMigration;
            }
        }
    });

    /**
     * Specifies the friendly name of the version of the C#/XAML for HTML5 compiler used to compile the assembly being attributed.
     *
     * @public
     * @class CSHTML5.Internal.Attributes.CompilerVersionFriendlyNameAttribute
     * @augments System.Attribute
     */
    Bridge.define("CSHTML5.Internal.Attributes.CompilerVersionFriendlyNameAttribute", {
        inherits: [System.Attribute],
        fields: {
            /**
             * The friendly name of the version of the C#/XAML for HTML5 compiler used to compile the assembly being attributed
             *
             * @instance
             * @public
             * @memberof CSHTML5.Internal.Attributes.CompilerVersionFriendlyNameAttribute
             * @function VersionFriendlyName
             * @type string
             */
            VersionFriendlyName: null
        },
        ctors: {
            /**
             * Initializes a new instance of the CompilerVersionFriendlyNameAttribute class with the
             friendly name of the version of the C#/XAML for HTML5 compiler used to compile the assembly being attributed.
             *
             * @instance
             * @public
             * @this CSHTML5.Internal.Attributes.CompilerVersionFriendlyNameAttribute
             * @memberof CSHTML5.Internal.Attributes.CompilerVersionFriendlyNameAttribute
             * @param   {string}    versionFriendlyName    The friendly name of the version of the C#/XAML for HTML5 compiler used to compile the assembly being attributed.
             * @return  {void}
             */
            ctor: function (versionFriendlyName) {
                this.$initialize();
                System.Attribute.ctor.call(this);
                this.VersionFriendlyName = versionFriendlyName;
            }
        }
    });

    /**
     * Specifies the version number of the C#/XAML for HTML5 compiler used to compile the assembly being attributed.
     *
     * @public
     * @class CSHTML5.Internal.Attributes.CompilerVersionNumberAttribute
     * @augments System.Attribute
     */
    Bridge.define("CSHTML5.Internal.Attributes.CompilerVersionNumberAttribute", {
        inherits: [System.Attribute],
        fields: {
            /**
             * The version of the C#/XAML for HTML5 compiler used to compile the assembly being attributed
             *
             * @instance
             * @public
             * @memberof CSHTML5.Internal.Attributes.CompilerVersionNumberAttribute
             * @function Version
             * @type System.Version
             */
            Version: null
        },
        props: {
            /**
             * The version number of the C#/XAML for HTML5 compiler used to compile the assembly being attributed
             *
             * @instance
             * @public
             * @memberof CSHTML5.Internal.Attributes.CompilerVersionNumberAttribute
             * @function VersionNumber
             * @type string
             */
            VersionNumber: {
                get: function () {
                    return (System.Version.op_Inequality(this.Version, null) ? this.Version.toString() : null);
                },
                set: function (value) {
                    this.Version = new System.Version.$ctor4(value);
                }
            }
        },
        ctors: {
            /**
             * Initializes a new instance of the CompilerVersionNumberAttribute class with the
             version number of the C#/XAML for HTML5 compiler used to compile the assembly being attributed.
             *
             * @instance
             * @public
             * @this CSHTML5.Internal.Attributes.CompilerVersionNumberAttribute
             * @memberof CSHTML5.Internal.Attributes.CompilerVersionNumberAttribute
             * @param   {string}    versionNumber    The version number of the C#/XAML for HTML5 compiler used to compile the assembly being attributed.
             * @return  {void}
             */
            ctor: function (versionNumber) {
                this.$initialize();
                System.Attribute.ctor.call(this);
                this.VersionNumber = versionNumber;
            }
        }
    });

    Bridge.define("CSHTML5.Internal.Attributes.IntermediateOutputAbsolutePathAttribute", {
        inherits: [System.Attribute],
        fields: {
            IntermediateOutputAbsolutePath: null
        },
        ctors: {
            ctor: function (intermediateOutputAbsolutePath) {
                this.$initialize();
                System.Attribute.ctor.call(this);
                this.IntermediateOutputAbsolutePath = intermediateOutputAbsolutePath;
            }
        }
    });

    /**
     * Specifies the friendly name of the minimum version of the C#/XAML for HTML5 compiler that is required to process the assembly being attributed.
     *
     * @public
     * @class CSHTML5.Internal.Attributes.MinimumRequiredCompilerVersionFriendlyNameAttribute
     * @augments System.Attribute
     */
    Bridge.define("CSHTML5.Internal.Attributes.MinimumRequiredCompilerVersionFriendlyNameAttribute", {
        inherits: [System.Attribute],
        fields: {
            /**
             * The friendly name of the minimum version of the C#/XAML for HTML5 compiler that is required to process the assembly being attributed
             *
             * @instance
             * @public
             * @memberof CSHTML5.Internal.Attributes.MinimumRequiredCompilerVersionFriendlyNameAttribute
             * @function VersionFriendlyName
             * @type string
             */
            VersionFriendlyName: null
        },
        ctors: {
            /**
             * Initializes a new instance of the MinimumRequiredCompilerVersionFriendlyNameAttribute class with the
             friendly name of the minimum version of the C#/XAML for HTML5 compiler that is required to process the assembly being attributed.
             *
             * @instance
             * @public
             * @this CSHTML5.Internal.Attributes.MinimumRequiredCompilerVersionFriendlyNameAttribute
             * @memberof CSHTML5.Internal.Attributes.MinimumRequiredCompilerVersionFriendlyNameAttribute
             * @param   {string}    versionFriendlyName    The friendly name of the minimum version of the C#/XAML for HTML5 compiler that is required to process the assembly being attributed.
             * @return  {void}
             */
            ctor: function (versionFriendlyName) {
                this.$initialize();
                System.Attribute.ctor.call(this);
                this.VersionFriendlyName = versionFriendlyName;
            }
        }
    });

    /**
     * Specifies the minimum version of the C#/XAML for HTML5 compiler that is required to process the assembly being attributed.
     *
     * @public
     * @class CSHTML5.Internal.Attributes.MinimumRequiredCompilerVersionNumberAttribute
     * @augments System.Attribute
     */
    Bridge.define("CSHTML5.Internal.Attributes.MinimumRequiredCompilerVersionNumberAttribute", {
        inherits: [System.Attribute],
        fields: {
            /**
             * The minimum version of the C#/XAML for HTML5 compiler that is required to process the assembly being attributed
             *
             * @instance
             * @public
             * @memberof CSHTML5.Internal.Attributes.MinimumRequiredCompilerVersionNumberAttribute
             * @function Version
             * @type System.Version
             */
            Version: null
        },
        props: {
            /**
             * The minimum version number of the C#/XAML for HTML5 compiler that is required to process the assembly being attributed
             *
             * @instance
             * @public
             * @memberof CSHTML5.Internal.Attributes.MinimumRequiredCompilerVersionNumberAttribute
             * @function VersionNumber
             * @type string
             */
            VersionNumber: {
                get: function () {
                    return (System.Version.op_Inequality(this.Version, null) ? this.Version.toString() : null);
                },
                set: function (value) {
                    this.Version = new System.Version.$ctor4(value);
                }
            }
        },
        ctors: {
            /**
             * Initializes a new instance of the MinimumRequiredCompilerVersionNumberAttribute class with the minimum
             version number of the C#/XAML for HTML5 compiler that is required to process the assembly being attributed.
             *
             * @instance
             * @public
             * @this CSHTML5.Internal.Attributes.MinimumRequiredCompilerVersionNumberAttribute
             * @memberof CSHTML5.Internal.Attributes.MinimumRequiredCompilerVersionNumberAttribute
             * @param   {string}    versionNumber    The minimum version number of the C#/XAML for HTML5 compiler that is required to process the assembly being attributed.
             * @return  {void}
             */
            ctor: function (versionNumber) {
                this.$initialize();
                System.Attribute.ctor.call(this);
                this.VersionNumber = versionNumber;
            }
        }
    });

    Bridge.define("CSHTML5.Internal.Attributes.OutputAppFilesPathAttribute", {
        inherits: [System.Attribute],
        fields: {
            OutputAppFilesPath: null
        },
        ctors: {
            ctor: function (outputAppFilesPath) {
                this.$initialize();
                System.Attribute.ctor.call(this);
                this.OutputAppFilesPath = outputAppFilesPath;
            }
        }
    });

    Bridge.define("CSHTML5.Internal.Attributes.OutputLibrariesPathAttribute", {
        inherits: [System.Attribute],
        fields: {
            OutputLibrariesPath: null
        },
        ctors: {
            ctor: function (outputLibrariesPath) {
                this.$initialize();
                System.Attribute.ctor.call(this);
                this.OutputLibrariesPath = outputLibrariesPath;
            }
        }
    });

    Bridge.define("CSHTML5.Internal.Attributes.OutputResourcesPathAttribute", {
        inherits: [System.Attribute],
        fields: {
            OutputResourcesPath: null
        },
        ctors: {
            ctor: function (outputResourcesPath) {
                this.$initialize();
                System.Attribute.ctor.call(this);
                this.OutputResourcesPath = outputResourcesPath;
            }
        }
    });

    Bridge.define("CSHTML5.Internal.Attributes.OutputRootPathAttribute", {
        inherits: [System.Attribute],
        fields: {
            OutputRootPath: null
        },
        ctors: {
            ctor: function (outputRootPath) {
                this.$initialize();
                System.Attribute.ctor.call(this);
                this.OutputRootPath = outputRootPath;
            }
        }
    });

    Bridge.define("CSHTML5.Internal.Attributes.SettingsAttribute", {
        inherits: [System.Attribute],
        ctors: {
            ctor: function () {
                this.$initialize();
                System.Attribute.ctor.call(this);
            }
        }
    });

    Bridge.define("CSHTML5.Internal.BaseValueSourceInternal", {
        $kind: "enum",
        statics: {
            fields: {
                Default: 0,
                Inherited: 1,
                ThemeStyle: 2,
                LocalStyle: 3,
                ImplicitReference: 4,
                Animated: 5,
                Local: 6
            }
        },
        $utype: System.Int16
    });

    Bridge.define("CSHTML5.Internal.DelegateUtility");

    Bridge.define("CSHTML5.Internal.FullValueSource", {
        $kind: "enum",
        statics: {
            fields: {
                ValueSourceMask: 15,
                ModifiersMask: 112,
                IsExpression: 16,
                IsExpressionFromStyle: 32,
                IsCoerced: 64,
                IsCoercedWithCurrentValue: 512,
                IsLocalOverAnimatedValue: 1024
            }
        },
        $utype: System.Int16
    });

    /** @namespace CSHTML5.Internal */

    /**
     * @public
     * @class CSHTML5.Internal.HtmlEventProxy
     * @implements  System.IDisposable
     */
    Bridge.define("CSHTML5.Internal.HtmlEventProxy", {
        inherits: [System.IDisposable],
        fields: {
            _eventHandler: null,
            _originalEventHandler: null,
            _sender: null,
            _domElementRef: null,
            _eventName: null
        },
        alias: ["Dispose", "System$IDisposable$Dispose"],
        ctors: {
            ctor: function (eventName, domElementRef, originalEventHandler) {
                this.$initialize();
                this._eventName = eventName;
                this._domElementRef = domElementRef;
                this._sender = this;
                this._eventHandler = function (s, e) {
                    originalEventHandler(e.JSEventObject);
                };
                this._originalEventHandler = originalEventHandler;
            }
        },
        methods: {
            Dispose: function () {
                if (this._domElementRef != null) {
                    this._domElementRef.removeEventListener(this._eventName,this._originalEventHandler);

                    this._domElementRef = null;
                    this._sender = null;
                    this._eventHandler = null;
                    this._originalEventHandler = null;
                }
            }
        }
    });

    /**
     * @public
     * @class CSHTML5.Internal.HtmlEventProxy.EventArgsWithJSEventObject
     * @augments System.Object
     */
    Bridge.define("CSHTML5.Internal.HtmlEventProxy.EventArgsWithJSEventObject", {
        $kind: "nested class",
        fields: {
            JSEventObject: null
        },
        ctors: {
            ctor: function (jsEventObject) {
                this.$initialize();
                System.Object.call(this);
                this.JSEventObject = jsEventObject;
            }
        }
    });

    /** @namespace System */

    /**
     * @memberof System
     * @callback System.Action
     * @return  {void}
     */

    Bridge.define("CSHTML5.Internal.INTERNAL_DispatcherHelpers", {
        statics: {
            fields: {
                _pendingActionsExecute: null,
                _isDispatcherPending: false,
                _dispatcher: null
            },
            ctors: {
                init: function () {
                    this._pendingActionsExecute = new (System.Collections.Generic.List$1(Function)).ctor();
                    this._isDispatcherPending = false;
                }
            },
            methods: {
                /**
                 * This will add the action to a list of actions to perform at once in a Dispatcher.Invoke call.
                 This has better performance than calling Dispatcher.BeginInvoke directly, because all the
                 queued actions are performed together in a single Dispatch.BeginInvoke call.
                 *
                 * @static
                 * @public
                 * @this CSHTML5.Internal.INTERNAL_DispatcherHelpers
                 * @memberof CSHTML5.Internal.INTERNAL_DispatcherHelpers
                 * @param   {System.Action}    action    The action to queue.
                 * @return  {void}
                 */
                QueueAction: function (action) {
                    CSHTML5.Internal.INTERNAL_DispatcherHelpers._pendingActionsExecute.add(action);

                    if (!CSHTML5.Internal.INTERNAL_DispatcherHelpers._isDispatcherPending) {
                        CSHTML5.Internal.INTERNAL_DispatcherHelpers._isDispatcherPending = true;

                        if (CSHTML5.Internal.INTERNAL_DispatcherHelpers._dispatcher == null) {
                            CSHTML5.Internal.INTERNAL_DispatcherHelpers._dispatcher = Windows.UI.Core.CoreDispatcher.INTERNAL_GetCurrentDispatcher();
                        }

                        CSHTML5.Internal.INTERNAL_DispatcherHelpers._dispatcher.BeginInvoke(function () {
                            if (CSHTML5.Internal.INTERNAL_DispatcherHelpers._isDispatcherPending) {
                                CSHTML5.Internal.INTERNAL_DispatcherHelpers.ExecutePendingActions();
                            }
                        });
                    }
                },
                ExecutePendingActions: function () {
                    var $t;
                    CSHTML5.Internal.INTERNAL_DispatcherHelpers._isDispatcherPending = false;

                    var copy = new (System.Collections.Generic.List$1(Function)).$ctor1(CSHTML5.Internal.INTERNAL_DispatcherHelpers._pendingActionsExecute);

                    CSHTML5.Internal.INTERNAL_DispatcherHelpers._pendingActionsExecute.clear();

                    $t = Bridge.getEnumerator(copy);
                    try {
                        while ($t.moveNext()) {
                            var action = $t.Current;
                            action();
                        }
                    } finally {
                        if (Bridge.is($t, System.IDisposable)) {
                            $t.System$IDisposable$Dispose();
                        }
                    }
                }
            }
        }
    });

    /**
     * A class that allows to queue an Action so that it is executed in a Dispatcher.Invoke call,
     but only if another action is not already pending execution. This is useful for example to
     redraw the screen when a collection of item changes, doing so only once if the collection
     changes multiple times. Another example is redrawing an element when any of its properties
     change, while ensuring that if many properties are changed, the element is redrawn only once.
     *
     * @public
     * @class CSHTML5.Internal.INTERNAL_DispatcherQueueHandler
     */
    Bridge.define("CSHTML5.Internal.INTERNAL_DispatcherQueueHandler", {
        fields: {
            _queueIsEmpty: false
        },
        ctors: {
            init: function () {
                this._queueIsEmpty = true;
            }
        },
        methods: {
            /**
             * Call this method to perform the specified action in a Dispatcher.Invoke call, but only
             if another action is not already pending execution. This is useful for example to redraw
             the screen when a collection of item changes, doing so only once if the collection changes
             multiple times. Another example is redrawing an element when any of its properties change,
             while ensuring that if many properties are changed, the element is redrawn only once.
             *
             * @instance
             * @public
             * @this CSHTML5.Internal.INTERNAL_DispatcherQueueHandler
             * @memberof CSHTML5.Internal.INTERNAL_DispatcherQueueHandler
             * @param   {System.Action}    action    The action to queue
             * @return  {void}
             */
            QueueActionIfQueueIsEmpty: function (action) {
                if (this._queueIsEmpty) {
                    this._queueIsEmpty = false;
                    CSHTML5.Internal.INTERNAL_DispatcherHelpers.QueueAction(Bridge.fn.bind(this, function () {
                        this._queueIsEmpty = true;

                        action();
                    }));
                }
            }
        }
    });

    Bridge.define("CSHTML5.Internal.INTERNAL_EventManager$2", function (EVENT_HANDLER, EVENT_ARGS) { return {
        fields: {
            _htmlEventProxies: null,
            _domEventsNamesToListenTo: null,
            _actionOnEvent: null,
            _domElementProvider: null,
            _handlers: null,
            _handlersForHandledEventsToo: null,
            _isListeningToDomEvents: false
        },
        props: {
            Handlers: {
                get: function () {
                    return this._handlers;
                }
            },
            HandlersForHandledEventsToo: {
                get: function () {
                    return this._handlersForHandledEventsToo;
                }
            }
        },
        ctors: {
            init: function () {
                this._htmlEventProxies = new (System.Collections.Generic.List$1(CSHTML5.Internal.HtmlEventProxy)).ctor();
                this._handlers = new (System.Collections.Generic.List$1(EVENT_HANDLER)).ctor();
                this._handlersForHandledEventsToo = new (System.Collections.Generic.List$1(EVENT_HANDLER)).ctor();
                this._isListeningToDomEvents = false;
            },
            ctor: function (domElementProvider, domEventsNamesToListenTo, actionOnEvent) {
                this.$initialize();

                this._domEventsNamesToListenTo = System.Array.init([domEventsNamesToListenTo], System.String);
                this._actionOnEvent = actionOnEvent;
                this._domElementProvider = domElementProvider;
            },
            $ctor1: function (domElementProvider, domEventsNamesToListenTo, actionOnEvent) {
                this.$initialize();

                this._domEventsNamesToListenTo = domEventsNamesToListenTo;
                this._actionOnEvent = actionOnEvent;
                this._domElementProvider = domElementProvider;
            }
        },
        methods: {
            Add: function (value, handledEventsToo) {
                if (handledEventsToo === void 0) { handledEventsToo = false; }

                this.StartListeningToDomEventsIfNotAlreadyListening();

                if (handledEventsToo) {
                    this._handlersForHandledEventsToo.add(value);
                } else {
                    this._handlers.add(value);
                }
            },
            Remove: function (value) {

                this._handlers.remove(value);
                this._handlersForHandledEventsToo.remove(value);

                if (this._handlers.Count === 0 && this._handlersForHandledEventsToo.Count === 0) {
                    this.StopListeningToDomEvents();
                }
            },
            /**
             * Raises the event
             *
             * @instance
             * @private
             * @this CSHTML5.Internal.INTERNAL_EventManager$2
             * @memberof CSHTML5.Internal.INTERNAL_EventManager$2
             * @param   {System.Object}    jsEventArg    The javascript event argument.
             * @return  {void}
             */
            OnEvent: function (jsEventArg) {
                if (this._htmlEventProxies != null && this._htmlEventProxies.Count !== 0) {
                    this._actionOnEvent(jsEventArg);
                }
            },
            AttachToDomEvents: function () {
                if (this._handlers.Count > 0 || this._handlersForHandledEventsToo.Count > 0) {
                    this.StartListeningToDomEventsIfNotAlreadyListening();
                }
            },
            DetachFromDomEvents: function () {
                if (this._isListeningToDomEvents && this._htmlEventProxies != null) {
                    this.StopListeningToDomEvents();
                }
            },
            StartListeningToDomEventsIfNotAlreadyListening: function () {
                var $t;
                if (!this._isListeningToDomEvents) {
                    var domElement = this._domElementProvider();
                    if (domElement != null) {
                        if (this._htmlEventProxies == null) {
                            this._htmlEventProxies = new (System.Collections.Generic.List$1(CSHTML5.Internal.HtmlEventProxy)).ctor();
                        }

                        $t = Bridge.getEnumerator(this._domEventsNamesToListenTo);
                        try {
                            while ($t.moveNext()) {
                                var eventName = $t.Current;
                                this._htmlEventProxies.add(CSHTML5.Internal.INTERNAL_EventsHelper.AttachToDomEvents(eventName, domElement, Bridge.fn.bind(this, function (jsEventArg) {
                                    this.OnEvent(jsEventArg);
                                })));
                            }
                        } finally {
                            if (Bridge.is($t, System.IDisposable)) {
                                $t.System$IDisposable$Dispose();
                            }
                        }

                        this._isListeningToDomEvents = true;
                    }
                }
            },
            StopListeningToDomEvents: function () {
                if (this._isListeningToDomEvents && this._htmlEventProxies != null) {
                    for (var i = (this._htmlEventProxies.Count - 1) | 0; i >= 0; i = (i - 1) | 0) {
                        this._htmlEventProxies.getItem(i).Dispose();
                        this._htmlEventProxies.removeAt(i);
                    }
                }
                this._isListeningToDomEvents = false;
            }
        }
    }; });

    Bridge.define("CSHTML5.Internal.INTERNAL_EventsHelper", {
        statics: {
            methods: {
                AttachToDomEvents: function (eventName, domElementRef, eventHandlerWithJsEventArg) {
                    var newProxy = new CSHTML5.Internal.HtmlEventProxy(eventName, domElementRef, eventHandlerWithJsEventArg);
                    domElementRef.addEventListener(eventName, eventHandlerWithJsEventArg);
                    return newProxy;
                }
            }
        }
    });

    Bridge.define("CSHTML5.Internal.INTERNAL_FontsHelper", {
        statics: {
            fields: {
                _loadedFonts: null,
                _suffixesUsed: 0
            },
            ctors: {
                init: function () {
                    this._loadedFonts = new (System.Collections.Generic.Dictionary$2(System.String,System.String)).ctor();
                    this._suffixesUsed = 0;
                }
            },
            methods: {
                /**
                 * Creates a css style with @font-face for the given font, then returns the name to use as the font-family. This name is actually the relative path to the file containing the font.
                 *
                 * @static
                 * @this CSHTML5.Internal.INTERNAL_FontsHelper
                 * @memberof CSHTML5.Internal.INTERNAL_FontsHelper
                 * @param   {string}                       fontPath                               The path to the font.
                 * @param   {Windows.UI.Xaml.UIElement}    elementThatARelativeUriIsRelativeTo    This is an optional parameter that is helpful
                 in case of relative URI's. In fact, when an URI is relative, we need to transform it into an
                 absolute URI in order to locate the resources. A relative URI is usually relative to the place
                 where the .XAML file is located. For example, when using an Image control, relative image paths
                 are relative to the location of the .XAML file that contains the Image control.
                 * @return  {string}                                                              The name to use to set the font-family property, a.k.a: the relative path to the font.
                 */
                LoadFont: function (fontPath, elementThatARelativeUriIsRelativeTo) {
                    if (elementThatARelativeUriIsRelativeTo === void 0) { elementThatARelativeUriIsRelativeTo = null; }
                    fontPath = fontPath.trim();
                    if (!System.Linq.Enumerable.from(fontPath).contains(46)) {
                        return fontPath;
                    }
                    var fontPathWithoutCustomName = fontPath;
                    var indexOfHashTag = System.String.indexOf(fontPath, String.fromCharCode(35));
                    if (indexOfHashTag !== -1) {
                        fontPathWithoutCustomName = fontPath.substr(0, indexOfHashTag);
                    }

                    var lowerCasePathWithoutCustomName = fontPathWithoutCustomName.toLowerCase();

                    if (!(System.String.startsWith(lowerCasePathWithoutCustomName, "ms-appx:/") || System.String.startsWith(lowerCasePathWithoutCustomName, "http://") || System.String.startsWith(lowerCasePathWithoutCustomName, "https://") || System.String.contains(fontPathWithoutCustomName,";component/"))) {
                        fontPathWithoutCustomName = "ms-appx:/" + (fontPathWithoutCustomName || "");
                    }

                    var relativeFontPath = CSHTML5.Internal.INTERNAL_UriHelper.ConvertToHtml5Path(fontPathWithoutCustomName, elementThatARelativeUriIsRelativeTo);

                    if (CSHTML5.Internal.INTERNAL_FontsHelper._loadedFonts.containsKey(relativeFontPath)) {
                        return CSHTML5.Internal.INTERNAL_FontsHelper._loadedFonts.getItem(relativeFontPath);
                    }
                    return CSHTML5.Internal.INTERNAL_FontsHelper.CreateFontInJS(relativeFontPath);
                },
                CreateFontInJS: function (fontUri) {
                    var fontName = CSHTML5.Internal.INTERNAL_FontsHelper.GetNewFontName();
                    fontUri = System.String.replaceAll(fontUri, String.fromCharCode(92), String.fromCharCode(47));
                    if (System.String.startsWith(fontUri, "/")) {
                        fontUri = fontUri.substr(1);
                    }
var newStyle = document.createElement('style');
newStyle.appendChild(document.createTextNode("\
@font-face {\
    font-family: '" +                     fontName + "';\
    src: url('" + fontUri + "')\
}}\
"));
document.body.appendChild(newStyle);

                    CSHTML5.Internal.INTERNAL_FontsHelper._loadedFonts.add(fontUri, fontName);
                    return fontName;
                },
                GetNewFontName: function () {
                    var suffixesUsedAsString = Bridge.toString(CSHTML5.Internal.INTERNAL_FontsHelper._suffixesUsed);
                    CSHTML5.Internal.INTERNAL_FontsHelper._suffixesUsed = (CSHTML5.Internal.INTERNAL_FontsHelper._suffixesUsed + 1) | 0;
                    return "fontName" + (suffixesUsedAsString || "");
                }
            }
        }
    });

    Bridge.define("CSHTML5.Internal.INTERNAL_HtmlDomManager", {
        statics: {
            fields: {
                IdOfApplicationRootDomElement: null,
                INTERNAL_idsToUIElements: null,
                NumericTypes: null
            },
            ctors: {
                init: function () {
                    this.IdOfApplicationRootDomElement = "cshtml5-root";
                },
                ctor: function () {
                    if (!true) {
                        CSHTML5.Internal.INTERNAL_HtmlDomManager.INTERNAL_idsToUIElements = new (System.Collections.Generic.Dictionary$2(System.String,Windows.UI.Xaml.UIElement)).ctor();
                    }
                }
            },
            methods: {
                GetApplicationRootDomElement: function () {
                    var rootDomElement = document.getElementById(CSHTML5.Internal.INTERNAL_HtmlDomManager.IdOfApplicationRootDomElement);

                    if (rootDomElement == null) {
                        throw new System.Exception("The application root DOM element was not found. To fix this issue, please add a DIV with the ID 'cshtml5-root' to the HTML page.");
                    }

                    return rootDomElement;
                },
                RemoveFromDom: function (domNode, commentForDebugging) {
                    if (commentForDebugging === void 0) { commentForDebugging = null; }
                    if (true) {
                        domNode.parentNode.removeChild(domNode);
                    } else {
                        var javaScriptCodeToExecute = System.String.format("var element = document.getElementById(\"{0}\");\r\n                                                           element.parentNode.removeChild(element);", [domNode.UniqueIdentifier]);
                        CSHTML5.Internal.INTERNAL_HtmlDomManager.ExecuteJavaScript(javaScriptCodeToExecute, commentForDebugging);
                        if (domNode.Parent != null) {
                            domNode.Parent.FirstChild = null;
                        }
                    }
                },
                IsNotUndefinedOrNull: function (obj) {
                    if (true) {
                        return (typeof obj !== 'undefined' && obj !== null);
                    } else {
                        return obj != null;
                    }
                },
                SetFocus: function (element) {
                    if (CSHTML5.Internal.INTERNAL_VisualTreeManager.IsElementInVisualTree(element)) {
                        var domElementRefConcernedByFocus = element.INTERNAL_OptionalSpecifyDomElementConcernedByFocus || element.INTERNAL_OuterDomElement;
                        if (true) {
setTimeout(function() {                             domElementRefConcernedByFocus.focus(); }, 1);
                        } else {
                            CSHTML5.Internal.INTERNAL_HtmlDomManager.SetFocus_SimulatorOnly(domElementRefConcernedByFocus);
                        }
                    }
                },
                SetContentString: function (element, content, removeTextWrapping) {
                    if (removeTextWrapping === void 0) { removeTextWrapping = false; }
                    var domElement = element.GetDomElementToSetContentString();


                    if (true) {

if ('innerText' in                         Bridge.unbox(domElement) && !window.IE_VERSION)
{
    // --- CHROME & IE: ---
    Bridge.unbox(domElement).innerText = content;
    if (removeTextWrapping)
    {
        Bridge.unbox(domElement).style.whiteSpace = 'nowrap';
    }
}
else
{
    // --- FIREFOX: ---
    // First, escape the string so that if it contains some HTML, it is made harmless:
    var tempDiv = document.createElement('div');
    tempDiv.appendChild(document.createTextNode(content));
    var escapedText = tempDiv.innerHTML;
    // Then, replace all line breaks with '<br>' so that FireFox can render them properly:
    var finalHtml = escapedText.replace(/\r\n/g,'<br>');
    finalHtml = finalHtml.replace(/\n/g,'<br>');
    // Finally, display the HTML:
    Bridge.unbox(domElement).innerHTML = finalHtml;
    if (removeTextWrapping)
    {
        Bridge.unbox(domElement).style.whiteSpace = 'nowrap';
    }
};
                    } else {
                        var uniqueIdentifier = domElement.UniqueIdentifier;
                        var javaScriptCodeToExecute = System.String.format("\r\nvar element = document.getElementById(\"{0}\");\r\nif (element)\r\n{{\r\n    element.innerText = \"{1}\";\r\n    if ({2})\r\n    {{\r\n        element.style.whiteSpace = \"nowrap\";\r\n    }}\r\n}}\r\n                ", uniqueIdentifier, null, System.Boolean.toString(removeTextWrapping).toLowerCase());
                        CSHTML5.Internal.INTERNAL_SimulatorExecuteJavaScript.ExecuteJavaScriptAsync(javaScriptCodeToExecute);
                    }

                    CSHTML5.Internal.INTERNAL_WorkaroundIE11IssuesWithScrollViewerInsideGrid.RefreshLayoutIfIE();

                },
                SetUIElementContentString: function (uiElement, newText) {
                    if (true) {
                        uiElement.INTERNAL_InnerDomElement['value'] = newText;
                    } else {
                        var uniqueIdentifier = uiElement.INTERNAL_InnerDomElement.UniqueIdentifier;


                        var javaScriptCodeToExecute = System.String.format("\r\nvar element = document.getElementById(\"{0}\");\r\nif (element) \r\n{{\r\n    element.value = \"{1}\";\r\n    element.style.visibility=\"collapse\";\r\n    setTimeout(function(){{ var element2 = document.getElementById(\"{0}\"); if (element2) {{ element2.style.visibility=\"visible\"; }} }}, 0);\r\n}}\r\n                ", uniqueIdentifier, null);

                        CSHTML5.Internal.INTERNAL_SimulatorExecuteJavaScript.ExecuteJavaScriptAsync(javaScriptCodeToExecute);
                    }
                },
                GetTextBoxText: function (domElementRef) {
                    if (true) {
                        return CSHTML5.Internal.INTERNAL_HtmlDomManager.GetTextBoxText_JavaScript(domElementRef);
                    } else {
                        var uniqueIdentifier = domElementRef.UniqueIdentifier;
                        var domElement = document.getElementById(uniqueIdentifier);

                        return Bridge.toString(getTextAreaInnerText(domElement));
                    }
                },
                GetTextBoxText_JavaScript: function (domElementRef) {


                var innerText = getTextAreaInnerText(                    Bridge.unbox(domElementRef));

                return innerText;
                

                    return "";
                },
                AddOptionToNativeComboBox$1: function (nativeComboBoxDomElement, elementToAdd, index) {
                    var optionDomElement = 
(function(){
    var option = document.createElement("option");
    option.text = elementToAdd;
    nativeComboBoxDomElement.add(option, Bridge.box(index, System.Int32));
    return option;
}())
;

                    return optionDomElement;
                },
                AddOptionToNativeComboBox: function (nativeComboBoxDomElement, elementToAdd) {
                    /* 
                    var optionDomElement = CSHTML5.Interop.ExecuteJavaScript(@"
                    function(){
                    var option = htmlDocument.createElement(""option"");
                    option.text = $1;
                    $0.add(option);
                    return option;
                    }()
                    ", domNode, elementToAdd);
                    */
                    var optionDomElement = 
(function(){
    var option = document.createElement("option");
    option.text = elementToAdd;
    nativeComboBoxDomElement.add(option);
    return option;
}())
;
                    return optionDomElement;
                    /* 
                    if (IsRunningInJavaScript())
                    {
                       var option = htmlDocument.createElement("option");
                       option.text = elementToAdd;
                       domNode.add(option); //add option instead of string, it'll work better
                    }
                    else
                    {
                       string javaScriptCodeToExecute = string.Format(@"
                    var element = document.getElementById(""{0}"");
                    var option = document.createElement(""option"");
                    option.text = ""{1}"";
                    element.add(option);
                    ", ((INTERNAL_HtmlDomElementReference)domNode).UniqueIdentifier, elementToAdd);
                       ExecuteJavaScript(javaScriptCodeToExecute);
                    }
                    */
                },
                RemoveOptionFromNativeComboBox$1: function (optionToRemove, nativeComboBoxDomElement) {
                    nativeComboBoxDomElement.removeChild(optionToRemove);
                },
                RemoveOptionFromNativeComboBox: function (nativeComboBoxDomElement, index) {
                    nativeComboBoxDomElement.remove(Bridge.box(index, System.Int32));
                },
                GetFrameworkElementOuterStyleForModification: function (element) {
                    return CSHTML5.Internal.INTERNAL_HtmlDomManager.GetDomElementStyleForModification(element.INTERNAL_OuterDomElement);
                },
                GetFrameworkElementBoxSizingStyleForModification: function (element) {
                    return CSHTML5.Internal.INTERNAL_HtmlDomManager.GetDomElementStyleForModification(element.INTERNAL_AdditionalOutsideDivForMargins);
                },
                GetDomElementStyleForModification: function (domElementRef) {
                    if (true) {
                        return domElementRef.style;
                    } else {
                        return CSHTML5.Internal.INTERNAL_HtmlDomStyleReference.GetInstance(domElementRef.UniqueIdentifier);
                    }
                },
                Get2dCanvasContext: function (domElementRef) {
                    if (true) {
                        return domElementRef.getContext("2d");
                    } else {
                        return CSHTML5.Internal.INTERNAL_Html2dContextReference.GetInstance(domElementRef.UniqueIdentifier);
                    }
                },
                CreateDomElementAppendItAndGetStyle: function (domElementTag, parentRef, associatedUIElement, newElementRef) {
                    newElementRef.v = CSHTML5.Internal.INTERNAL_HtmlDomManager.CreateDomElementAndAppendIt(domElementTag, parentRef, associatedUIElement);

                    return CSHTML5.Internal.INTERNAL_HtmlDomManager.GetDomElementStyleForModification(newElementRef.v);

                },
                SetDomElementProperty: function (domElementRef, attributeName, attributeValue, forceSimulatorExecuteImmediately) {
                    if (forceSimulatorExecuteImmediately === void 0) { forceSimulatorExecuteImmediately = false; }
                    if (true) {
                        domElementRef[attributeName] = Bridge.unbox(attributeValue);
                    } else {
                        var uniqueIdentifier = domElementRef.UniqueIdentifier;
                        var javaScriptCodeToExecute = System.String.format("var element = document.getElementById(\"{0}\");if (element) {{ element[\"{1}\"] = {2} }};", uniqueIdentifier, attributeName, null);

                        if (forceSimulatorExecuteImmediately) {
                            CSHTML5.Internal.INTERNAL_HtmlDomManager.ExecuteJavaScript(javaScriptCodeToExecute);
                        } else {
                            CSHTML5.Internal.INTERNAL_SimulatorExecuteJavaScript.ExecuteJavaScriptAsync(javaScriptCodeToExecute);
                        }
                    }
                },
                SetDomElementAttribute: function (domElementRef, attributeName, attributeValue, forceSimulatorExecuteImmediately) {
                    if (forceSimulatorExecuteImmediately === void 0) { forceSimulatorExecuteImmediately = false; }
                    if (true) {
                        domElementRef.setAttribute(attributeName, Bridge.unbox(attributeValue));
                    } else {
                        var uniqueIdentifier = domElementRef.UniqueIdentifier;
                        var javaScriptCodeToExecute = System.String.format("var element = document.getElementById(\"{0}\");if (element) {{ element.setAttribute(\"{1}\", {2}) }};", uniqueIdentifier, attributeName, null);

                        if (forceSimulatorExecuteImmediately) {
                            CSHTML5.Internal.INTERNAL_HtmlDomManager.ExecuteJavaScript(javaScriptCodeToExecute);
                        } else {
                            CSHTML5.Internal.INTERNAL_SimulatorExecuteJavaScript.ExecuteJavaScriptAsync(javaScriptCodeToExecute);
                        }
                    }
                },
                SetDomElementStyleProperty: function (domElementRef, propertyCSSNames, attributeValue, forceSimulatorExecuteImmediately) {
                    var $t, $t1;
                    if (forceSimulatorExecuteImmediately === void 0) { forceSimulatorExecuteImmediately = false; }


                    if (true) {
                        $t = Bridge.getEnumerator(propertyCSSNames);
                        try {
                            while ($t.moveNext()) {
                                var propertyName = $t.Current;
                                domElementRef.style[propertyName] = Bridge.unbox(attributeValue);
                            }
                        } finally {
                            if (Bridge.is($t, System.IDisposable)) {
                                $t.System$IDisposable$Dispose();
                            }
                        }
                    } else {
                        var uniqueIdentifier = domElementRef.UniqueIdentifier;
                        var settingProperties = "";
                        var value = null;
                        $t1 = Bridge.getEnumerator(propertyCSSNames);
                        try {
                            while ($t1.moveNext()) {
                                var propertyName1 = $t1.Current;
                                settingProperties = (settingProperties || "") + ((System.String.format("element.style.{0} = {1};", propertyName1, value)) || "");
                            }
                        } finally {
                            if (Bridge.is($t1, System.IDisposable)) {
                                $t1.System$IDisposable$Dispose();
                            }
                        }
                        var javaScriptCodeToExecute = System.String.format("var element = document.getElementById(\"{0}\");if (element) {{ {1} }};", uniqueIdentifier, settingProperties);

                        if (forceSimulatorExecuteImmediately) {
                            CSHTML5.Internal.INTERNAL_HtmlDomManager.ExecuteJavaScript(javaScriptCodeToExecute);
                        } else {
                            CSHTML5.Internal.INTERNAL_SimulatorExecuteJavaScript.ExecuteJavaScriptAsync(javaScriptCodeToExecute);
                        }
                    }
                },
                SetDomElementStylePropertyUsingVelocity: function (domElement, cssPropertyNames, cssValue) {
                    var $t;
                    if (cssPropertyNames != null && cssPropertyNames.Count !== 0) {
                        if (domElement != null) {
                            var newObj = new Object();

                            $t = Bridge.getEnumerator(cssPropertyNames);
                            try {
                                while ($t.moveNext()) {
                                    var csspropertyName = $t.Current;
                                    newObj[csspropertyName] = cssValue;
                                }
                            } finally {
                                if (Bridge.is($t, System.IDisposable)) {
                                    $t.System$IDisposable$Dispose();
                                }
                            }
Velocity(                            domElement, newObj, {duration:1, queue:false});
                        }

                    } else {
                        throw new System.InvalidOperationException.$ctor1("Please set the Name property of the CSSEquivalent class.");
                    }
                },
                RemoveDomElementAttribute: function (domElementRef, attributeName, forceSimulatorExecuteImmediately) {
                    if (forceSimulatorExecuteImmediately === void 0) { forceSimulatorExecuteImmediately = false; }
                    if (true) {
                        domElementRef.removeAttribute(attributeName);
                    } else {
                        var uniqueIdentifier = domElementRef.UniqueIdentifier;
                        var javaScriptCodeToExecute = System.String.format("var element = document.getElementById(\"{0}\");if (element) {{ element.removeAttribute(\"{1}\") }};", uniqueIdentifier, attributeName);

                        if (forceSimulatorExecuteImmediately) {
                            CSHTML5.Internal.INTERNAL_HtmlDomManager.ExecuteJavaScript(javaScriptCodeToExecute);
                        } else {
                            CSHTML5.Internal.INTERNAL_SimulatorExecuteJavaScript.ExecuteJavaScriptAsync(javaScriptCodeToExecute);
                        }
                    }
                },
                GetDomElementAttribute: function (domElementRef, attributeName) {
                    return domElementRef[attributeName];

                },
                CallDomMethod: function (domElementRef, methodName, args) {
                    var $t;
                    if (args === void 0) { args = []; }
                    if (true) {
                        return domElementRef[methodName].apply(domElementRef, Array.prototype.slice.call(arguments, 2));
                    } else {
                        var uniqueIdentifier = domElementRef.UniqueIdentifier;
                        var parameters = "";
                        $t = Bridge.getEnumerator(args);
                        try {
                            while ($t.moveNext()) {
                                var obj = $t.Current;
                                parameters = (parameters || "") + ((null) || "");
                            }
                        } finally {
                            if (Bridge.is($t, System.IDisposable)) {
                                $t.System$IDisposable$Dispose();
                            }
                        }

                        var javaScriptCodeToExecute = System.String.format("var element = document.getElementById(\"{0}\");if (element) {{ element[\"{1}\"]({2}) }};", uniqueIdentifier, methodName, parameters);

                        return CSHTML5.Internal.INTERNAL_HtmlDomManager.ExecuteJavaScriptWithResult(javaScriptCodeToExecute);
                    }
                },
                CreateDomElementAndAppendIt: function (domElementTag, parentRef, associatedUIElement, index) {
                    if (index === void 0) { index = -1; }

                    if (true) {
                        var result = 
(function(){
    var newElement = document.createElement(domElementTag);
    newElement.associatedUIElement = associatedUIElement;
    if(index < 0 || index >= Bridge.unbox(parentRef).children.length)
    {
        Bridge.unbox(parentRef).appendChild(newElement);
    }
    else
    {
        var nextSibling = Bridge.unbox(parentRef).children[index];
        Bridge.unbox(parentRef).insertBefore(newElement,nextSibling);
    }
    return newElement;
}());


                        return result;
                    } else {
                        return CSHTML5.Internal.INTERNAL_HtmlDomManager.CreateDomElementAndAppendIt_ForUseByTheSimulator(domElementTag, Bridge.unbox(parentRef), associatedUIElement, index);
                    }
                },
                CreateDomElementAndInsertIt: function (domElementTag, parentRef, associatedUIElement, insertionIndex, relativePosition) {

                    if (true) {
                        var result = 
(function(){
    var newElement = document.createElement(domElementTag);
    newElement.associatedUIElement = associatedUIElement;
    Bridge.unbox(parentRef).children[insertionIndex].insertAdjacentElement(relativePosition, newElement);
    return newElement;
}());

                        return result;
                    } else {
                        return CSHTML5.Internal.INTERNAL_HtmlDomManager.CreateDomElementAndInsertIt_ForUseByTheSimulator(domElementTag, Bridge.unbox(parentRef), associatedUIElement, insertionIndex, relativePosition);
                    }
                },
                CreateDomFromStringAndAppendIt: function (domAsString, parentRef, associatedUIElement) {
                    if (true) {
                        var tempDiv = document.createElement("div");
                        tempDiv.innerHTML = domAsString;
                        var newElement = tempDiv.firstChild;
                        newElement.associatedUIElement = associatedUIElement;
                        parentRef.appendChild(newElement);
                        return newElement;
                    } else {
                        var uniqueIdentifier = CSHTML5.Internal.INTERNAL_HtmlDomUniqueIdentifiers.CreateNew();
                        var parentUniqueIdentifier = parentRef.UniqueIdentifier;
                        var javaScriptToExecute = System.String.format("\r\n// Create a temporary parent div to which we can write the innerHTML, then extract the contents:\r\nvar tempDiv = document.createElement(\"div\");\r\ntempDiv.innerHTML = \"{0}\";\r\nvar newElement = tempDiv.firstChild;\r\nnewElement.setAttribute(\"id\", \"{1}\");\r\nvar parentElement = document.getElementById(\"{2}\");\r\nparentElement.appendChild(newElement);\r\n", System.String.replaceAll(System.String.replaceAll(System.String.replaceAll(domAsString, String.fromCharCode(34), String.fromCharCode(39)), "\r", ""), "\n", ""), uniqueIdentifier, parentUniqueIdentifier);
                        CSHTML5.Internal.INTERNAL_HtmlDomManager.ExecuteJavaScript(javaScriptToExecute);
                        CSHTML5.Internal.INTERNAL_HtmlDomManager.INTERNAL_idsToUIElements.add(uniqueIdentifier, associatedUIElement);
                        return new CSHTML5.Internal.INTERNAL_HtmlDomElementReference(uniqueIdentifier, parentRef.Parent);
                    }
                },
                AppendChild_ForUseByPublicAPIOnly_SimulatorOnly: function (domElementRef, parentDomElementRef) {
                    var childUniqueIdentifier = domElementRef.UniqueIdentifier;
                    var parentUniqueIdentifier = parentDomElementRef.UniqueIdentifier;
                    var javaScriptToExecute = System.String.format("\r\nvar child = document.getElementById(\"{0}\");\r\nvar parentElement = document.getElementById(\"{1}\");\r\nparentElement.appendChild(child);\r\n", childUniqueIdentifier, parentUniqueIdentifier);
                    CSHTML5.Internal.INTERNAL_HtmlDomManager.ExecuteJavaScript(javaScriptToExecute);
                    if (CSHTML5.Internal.INTERNAL_HtmlDomManager.INTERNAL_idsToUIElements.containsKey(parentUniqueIdentifier)) {
                        CSHTML5.Internal.INTERNAL_HtmlDomManager.INTERNAL_idsToUIElements.setItem(childUniqueIdentifier, CSHTML5.Internal.INTERNAL_HtmlDomManager.INTERNAL_idsToUIElements.getItem(parentUniqueIdentifier));
                    }
                },
                /**
                 * Retrieves an object that is located within a specified point of an object's coordinate space.
                 *
                 * @static
                 * @public
                 * @this CSHTML5.Internal.INTERNAL_HtmlDomManager
                 * @memberof CSHTML5.Internal.INTERNAL_HtmlDomManager
                 * @param   {number}                       x    
                 * @param   {number}                       y
                 * @return  {Windows.UI.Xaml.UIElement}         The UIElement object that is determined to be located
                 in the visual tree composition at the specified point.
                 */
                FindElementInHostCoordinates_UsedBySimulatorToo: function (x, y) {
                    var domElementAtCoordinates = 
(function(){
    var domElementAtCoordinates = document.elementFromPoint(Bridge.box(x, System.Double, System.Double.format, System.Double.getHashCode), Bridge.box(y, System.Double, System.Double.format, System.Double.getHashCode));
    if (!domElementAtCoordinates || domElementAtCoordinates === document.documentElement)
    {
        return null;
    }
    else
    {
        return domElementAtCoordinates;
    }
}())
;

                    var result = CSHTML5.Internal.INTERNAL_HtmlDomManager.GetUIElementFromDomElement(domElementAtCoordinates);

                    return result;
                },
                GetUIElementFromDomElement: function (domElementRef) {
                    var result = null;

                    while (!CSHTML5.Internal.INTERNAL_HtmlDomManager.IsNullOrUndefined(domElementRef)) {
                        if (CSHTML5.Interop.IsRunningInTheSimulator) {

                            var jsId = domElementRef.id;
                            if (!CSHTML5.Internal.INTERNAL_HtmlDomManager.IsNullOrUndefined(jsId)) {
                                var id = System.Convert.toString(jsId);
                                if (CSHTML5.Internal.INTERNAL_HtmlDomManager.INTERNAL_idsToUIElements.containsKey(id)) {
                                    result = CSHTML5.Internal.INTERNAL_HtmlDomManager.INTERNAL_idsToUIElements.getItem(id);
                                    break;
                                }
                            }
                        } else {

                            var associatedUIElement = domElementRef.associatedUIElement;
                            if (!CSHTML5.Internal.INTERNAL_HtmlDomManager.IsNullOrUndefined(associatedUIElement)) {
                                result = Bridge.cast(associatedUIElement, Windows.UI.Xaml.UIElement);
                                break;
                            }
                        }

                        domElementRef = domElementRef.parentNode;
                    }

                    return result;
                },
                IsNullOrUndefined: function (jsObject) {
                    return (jsObject === null) || (typeof(jsObject) === 'undefined');
                }
            }
        }
    });

    Bridge.define("CSHTML5.Internal.INTERNAL_HtmlDomUniqueIdentifiers", {
        statics: {
            fields: {
                CurrentIndentifier: 0
            },
            ctors: {
                init: function () {
                    this.CurrentIndentifier = 0;
                }
            },
            methods: {
                CreateNew: function () {
                    CSHTML5.Internal.INTERNAL_HtmlDomUniqueIdentifiers.CurrentIndentifier = (CSHTML5.Internal.INTERNAL_HtmlDomUniqueIdentifiers.CurrentIndentifier + 1) | 0;
                    return "id" + (Bridge.toString(CSHTML5.Internal.INTERNAL_HtmlDomUniqueIdentifiers.CurrentIndentifier) || "");
                }
            }
        }
    });

    Bridge.define("CSHTML5.Internal.INTERNAL_ListsHelper", {
        statics: {
            methods: {
                ConvertToListOfObjectsOrNull: function (enumerable) {
                    var $t;
                    var result = null;
                    if (enumerable != null) {
                        result = new (System.Collections.Generic.List$1(System.Object)).ctor();
                        $t = Bridge.getEnumerator(enumerable);
                        try {
                            while ($t.moveNext()) {
                                var obj = $t.Current;
                                result.add(obj);
                            }
                        } finally {
                            if (Bridge.is($t, System.IDisposable)) {
                                $t.System$IDisposable$Dispose();
                            }
                        }
                    }
                    return result;
                }
            }
        }
    });

    /**
     * An instance of this class can be used wherever you might otherwise use
     "new Object()".  The name will show up in the debugger, instead of
     merely "{object}"
     *
     * @class CSHTML5.Internal.INTERNAL_NamedObject
     */
    Bridge.define("CSHTML5.Internal.INTERNAL_NamedObject", {
        fields: {
            _name: null
        },
        ctors: {
            ctor: function (name) {
                this.$initialize();
                if (System.String.isNullOrEmpty(name)) {
                    throw new System.ArgumentNullException.$ctor1("name");
                }
                this._name = name;
            }
        },
        methods: {
            toString: function () {
                if (this._name.charCodeAt(0) !== 123) {
                    this._name = System.String.formatProvider(System.Globalization.CultureInfo.invariantCulture, "{{{0}}}", [this._name]);
                }
                return this._name;
            }
        }
    });

    Bridge.define("CSHTML5.Internal.INTERNAL_ObjectTreeHelper", {
        statics: {
            methods: {
                GetObjectTreeChildren: function (dependencyObject) {
                    var $t;
                    var isFound = false;
                    var isSingleChild = false;
                    var singleChild = null;
                    var multipleChildren = new (System.Collections.Generic.List$1(Windows.UI.Xaml.DependencyObject)).ctor();
                    if (Bridge.is(dependencyObject, Windows.UI.Xaml.Controls.Border)) {
                        singleChild = Bridge.cast(dependencyObject, Windows.UI.Xaml.Controls.Border).Child;
                        isSingleChild = true;
                        isFound = true;
                    }
                    if (Bridge.is(dependencyObject, Windows.UI.Xaml.Controls.Canvas)) {
                        $t = Bridge.getEnumerator(Bridge.cast(dependencyObject, Windows.UI.Xaml.Controls.Canvas).Children);
                        try {
                            while ($t.moveNext()) {
                                var child = $t.Current;
                                multipleChildren.add(child);
                            }
                        } finally {
                            if (Bridge.is($t, System.IDisposable)) {
                                $t.System$IDisposable$Dispose();
                            }
                        }
                        isSingleChild = false;
                        isFound = true;
                    }
                    if (Bridge.is(dependencyObject, Windows.UI.Xaml.Controls.UserControl)) {
                        singleChild = Bridge.cast(dependencyObject, Windows.UI.Xaml.Controls.UserControl).Content;
                        isSingleChild = true;
                        isFound = true;
                    }
                    if (Bridge.is(dependencyObject, Windows.UI.Xaml.Window)) {
                        singleChild = Bridge.cast(Bridge.cast(dependencyObject, Windows.UI.Xaml.Window).Content, Windows.UI.Xaml.UIElement);
                        isSingleChild = true;
                        isFound = true;
                    }
                    if (isFound) {
                        if (isSingleChild) {
                            if (singleChild != null) {
                                return function (_o1) {
                                        _o1.add(singleChild);
                                        return _o1;
                                    }(new (System.Collections.Generic.List$1(Windows.UI.Xaml.DependencyObject)).ctor());
                            } else {
                                return new (System.Collections.Generic.List$1(Windows.UI.Xaml.DependencyObject)).ctor();
                            }
                        } else {
                            return multipleChildren;
                        }
                    }

                    throw new System.Exception("Could not determine the children of: " + (Bridge.getTypeName(Bridge.getType(dependencyObject)) || ""));
                }
            }
        }
    });

    Bridge.define("CSHTML5.Internal.INTERNAL_PropertyStorage", {
        fields: {
            _source: 0,
            _value: null,
            _values: null,
            Owner: null,
            Property: null,
            TypeMetadata: null,
            PropertyListeners: null
        },
        props: {
            LocalValue: {
                get: function () {
                    return this._values[System.Array.index(5, this._values)];
                },
                set: function (value) {
                    this._values[System.Array.index(5, this._values)] = value;
                    this.IsAnimatedOverLocal = (Bridge.referenceEquals(value, Windows.UI.Xaml.DependencyProperty.UnsetValue) && !Bridge.referenceEquals(this.AnimatedValue, Windows.UI.Xaml.DependencyProperty.UnsetValue));
                }
            },
            AnimatedValue: {
                get: function () {
                    return this._values[System.Array.index(4, this._values)];
                },
                set: function (value) {
                    this._values[System.Array.index(4, this._values)] = value;
                    this.IsAnimatedOverLocal = (!Bridge.referenceEquals(value, Windows.UI.Xaml.DependencyProperty.UnsetValue));
                }
            },
            ImplicitReferenceValue: {
                get: function () {
                    return this._values[System.Array.index(3, this._values)];
                },
                set: function (value) {
                    this._values[System.Array.index(3, this._values)] = value;
                }
            },
            LocalStyleValue: {
                get: function () {
                    return this._values[System.Array.index(2, this._values)];
                },
                set: function (value) {
                    this._values[System.Array.index(2, this._values)] = value;
                }
            },
            ThemeStyleValue: {
                get: function () {
                    return this._values[System.Array.index(1, this._values)];
                },
                set: function (value) {
                    this._values[System.Array.index(1, this._values)] = value;
                }
            },
            InheritedValue: {
                get: function () {
                    return this._values[System.Array.index(0, this._values)];
                },
                set: function (value) {
                    this._values[System.Array.index(0, this._values)] = value;
                }
            },
            Value: {
                get: function () {
                    return this._value;
                },
                set: function (value) {
                    this._value = value;
                }
            },
            BaseValueSourceInternal: {
                get: function () {
                    return Bridge.Int.sxs((this._source & CSHTML5.Internal.FullValueSource.ValueSourceMask) & 65535);
                },
                set: function (value) {
                    this._source = Bridge.Int.sxs(((Bridge.Int.sxs((this._source & -16) & 65535)) | value) & 65535);
                }
            },
            FullValueSource: {
                get: function () {
                    return this._source;
                }
            },
            ModifiedValue: {
                get: function () {
                    if (this._value != null) {
                        return Bridge.as(this._value, CSHTML5.Internal.ModifiedValue);
                    }
                    return null;
                }
            },
            IsExpression: {
                get: function () {
                    return this.ReadPrivateFlag(CSHTML5.Internal.FullValueSource.IsExpression);
                },
                set: function (value) {
                    this.WritePrivateFlag(CSHTML5.Internal.FullValueSource.IsExpression, value);
                }
            },
            IsExpressionFromStyle: {
                get: function () {
                    return this.ReadPrivateFlag(CSHTML5.Internal.FullValueSource.IsExpressionFromStyle);
                },
                set: function (value) {
                    this.WritePrivateFlag(CSHTML5.Internal.FullValueSource.IsExpressionFromStyle, value);
                }
            },
            IsCoerced: {
                get: function () {
                    return this.ReadPrivateFlag(CSHTML5.Internal.FullValueSource.IsCoerced);
                },
                set: function (value) {
                    this.WritePrivateFlag(CSHTML5.Internal.FullValueSource.IsCoerced, value);
                }
            },
            IsCoercedWithCurrentValue: {
                get: function () {
                    return this.ReadPrivateFlag(CSHTML5.Internal.FullValueSource.IsCoercedWithCurrentValue);
                },
                set: function (value) {
                    this.WritePrivateFlag(CSHTML5.Internal.FullValueSource.IsCoercedWithCurrentValue, value);
                }
            },
            IsAnimatedOverLocal: {
                get: function () {
                    return this.ReadPrivateFlag(CSHTML5.Internal.FullValueSource.IsLocalOverAnimatedValue);
                },
                set: function (value) {
                    this.WritePrivateFlag(CSHTML5.Internal.FullValueSource.IsLocalOverAnimatedValue, value);
                }
            },
            HasModifiers: {
                get: function () {
                    return (Bridge.Int.sxs((this._source & CSHTML5.Internal.FullValueSource.ModifiersMask) & 65535)) !== 0;
                }
            }
        },
        ctors: {
            ctor: function (owner, property, typeMetadata) {
                this.$initialize();
                this._values = System.Array.init([Windows.UI.Xaml.DependencyProperty.UnsetValue, Windows.UI.Xaml.DependencyProperty.UnsetValue, Windows.UI.Xaml.DependencyProperty.UnsetValue, Windows.UI.Xaml.DependencyProperty.UnsetValue, Windows.UI.Xaml.DependencyProperty.UnsetValue, Windows.UI.Xaml.DependencyProperty.UnsetValue], System.Object);
                this._source = CSHTML5.Internal.BaseValueSourceInternal.Default;
                this.Owner = owner;
                this.Property = property;
                this.TypeMetadata = typeMetadata;
                this.Value = typeMetadata.DefaultValue;
            }
        },
        methods: {
            WritePrivateFlag: function (bit, value) {
                if (value) {
                    this._source |= Bridge.Int.sxs(bit & 65535);
                } else {
                    this._source &= Bridge.Int.sxs((~bit) & 65535);
                }
            },
            ReadPrivateFlag: function (bit) {
                return (Bridge.Int.sxs((this._source & bit) & 65535)) !== 0;
            },
            EnsureModifiedValue: function () {
                var modifiedValue;
                if (this._value == null) {
                    this._value = (modifiedValue = new CSHTML5.Internal.ModifiedValue());
                } else {
                    modifiedValue = Bridge.as(this._value, CSHTML5.Internal.ModifiedValue);
                    if (modifiedValue == null) {
                        modifiedValue = new CSHTML5.Internal.ModifiedValue();
                        modifiedValue.BaseValue = this._value;
                        this._value = modifiedValue;
                    }
                }
                return modifiedValue;
            },
            SetCoercedValue: function (value, baseValue, coerceWithCurrentValue) {
                var modifiedValue = this.EnsureModifiedValue();
                if (this.IsCoercedWithCurrentValue) {
                    baseValue = modifiedValue.BaseValue;
                }
                modifiedValue.CoercedValue = value;
                this.IsCoerced = true;
                this.IsCoercedWithCurrentValue = coerceWithCurrentValue;
                Bridge.global.System.Diagnostics.Debug.Assert(Bridge.equals(baseValue, modifiedValue.BaseValue) || Bridge.equals(baseValue, modifiedValue.ExpressionValue));
            },
            SetExpressionValue: function (value, baseValue) {
                var modifiedValue = this.EnsureModifiedValue();
                modifiedValue.ExpressionValue = value;
                this.IsExpression = true;
                Bridge.global.System.Diagnostics.Debug.Assert(Bridge.equals(baseValue, modifiedValue.BaseValue));
            },
            SetExpressionFromStyleValue: function (value, baseValue) {
                var modifiedValue = this.EnsureModifiedValue();
                Bridge.global.System.Diagnostics.Debug.Assert$1(modifiedValue.ExpressionValue == null, "Can't set expression from style if local expression is set");
                modifiedValue.ExpressionValue = value;
                this.IsExpressionFromStyle = true;
                Bridge.global.System.Diagnostics.Debug.Assert(Bridge.equals(baseValue, modifiedValue.BaseValue));
            },
            ResetCoercedValue: function () {
                if (this.IsCoerced) {
                    var modifiedValue = this.ModifiedValue;
                    modifiedValue.CoercedValue = null;
                    this.IsCoerced = false;

                    if (!this.HasModifiers) {
                        this.Value = modifiedValue.BaseValue;
                    }
                }
            },
            ResetExpressionValue: function () {
                if (this.IsExpression) {
                    var modifiedValue = this.ModifiedValue;
                    modifiedValue.ExpressionValue = null;
                    this.IsExpression = false;

                    if (!this.HasModifiers) {
                        this.Value = modifiedValue.BaseValue;
                    }
                }
            },
            ResetExpressionFromStyleValue: function () {
                if (this.IsExpressionFromStyle) {
                    var modifiedValue = this.ModifiedValue;
                    modifiedValue.ExpressionValue = null;
                    this.IsExpressionFromStyle = false;

                    if (!this.HasModifiers) {
                        this.Value = modifiedValue.BaseValue;
                    }
                }
            },
            ResetValue: function () {
                if (this.HasModifiers) {
                    var modifiedValue = this.ModifiedValue;
                    this._source &= Bridge.Int.sxs((CSHTML5.Internal.FullValueSource.ValueSourceMask) & 65535);
                    this.Value = modifiedValue.BaseValue;
                }
            }
        }
    });

    Bridge.define("CSHTML5.Internal.INTERNAL_PropertyStore", {
        statics: {
            methods: {
                /**
                 * Attempt to get a Property Storage
                 *
                 * @static
                 * @public
                 * @this CSHTML5.Internal.INTERNAL_PropertyStore
                 * @memberof CSHTML5.Internal.INTERNAL_PropertyStore
                 * @param   {Windows.UI.Xaml.DependencyObject}             dependencyObject      
                 * @param   {Windows.UI.Xaml.DependencyProperty}           dependencyProperty    
                 * @param   {boolean}                                      createIfNotFoud       when set to true, it forces the creation of the storage if it does not exists yet.
                 * @param   {CSHTML5.Internal.INTERNAL_PropertyStorage}    storage
                 * @return  {boolean}
                 */
                TryGetStorage: function (dependencyObject, dependencyProperty, createIfNotFoud, storage) {
                    if (dependencyObject.INTERNAL_PropertyStorageDictionary.tryGetValue(dependencyProperty, storage)) {
                        return true;
                    }

                    if (createIfNotFoud) {
                        var typeMetadata = dependencyProperty.GetTypeMetaData(Bridge.getType(dependencyObject));


                        storage.v = new CSHTML5.Internal.INTERNAL_PropertyStorage(dependencyObject, dependencyProperty, typeMetadata);
                        dependencyObject.INTERNAL_PropertyStorageDictionary.add(dependencyProperty, storage.v);

                        if (typeMetadata.Inherits) {
                            dependencyObject.INTERNAL_AllInheritedProperties.add(dependencyProperty, storage.v);
                        }
                    }

                    return createIfNotFoud;
                },
                /**
                 * Attemp to get a Property Storage for an inherited property 
                 (faster than generic accessor 'TryGetStorage')
                 *
                 * @static
                 * @this CSHTML5.Internal.INTERNAL_PropertyStore
                 * @memberof CSHTML5.Internal.INTERNAL_PropertyStore
                 * @param   {Windows.UI.Xaml.DependencyObject}             dependencyObject      
                 * @param   {Windows.UI.Xaml.DependencyProperty}           dependencyProperty    
                 * @param   {boolean}                                      createIfNotFoud       when set to true, it forces the creation of the storage if it does not exists yet.
                 * @param   {CSHTML5.Internal.INTERNAL_PropertyStorage}    storage
                 * @return  {boolean}
                 */
                TryGetInheritedPropertyStorage: function (dependencyObject, dependencyProperty, createIfNotFoud, storage) {
                    if (dependencyObject.INTERNAL_AllInheritedProperties.tryGetValue(dependencyProperty, storage)) {
                        return true;
                    }

                    if (createIfNotFoud) {
                        var typeMetadata = dependencyProperty.GetTypeMetaData(Bridge.getType(dependencyObject));

                        Bridge.global.System.Diagnostics.Debug.Assert$1(typeMetadata != null && typeMetadata.Inherits, System.String.format("{0} is not an inherited property.", [dependencyProperty.Name]));

                        storage.v = new CSHTML5.Internal.INTERNAL_PropertyStorage(dependencyObject, dependencyProperty, typeMetadata);

                        if (Bridge.Reflection.isAssignableFrom(dependencyProperty.OwnerType, Bridge.getType(dependencyObject))) {
                            dependencyObject.INTERNAL_PropertyStorageDictionary.add(dependencyProperty, storage.v);
                        }
                        dependencyObject.INTERNAL_AllInheritedProperties.add(dependencyProperty, storage.v);
                    }

                    return createIfNotFoud;
                },
                SetValueCommon: function (storage, newValue, coerceWithCurrentValue) {
                    if (Bridge.referenceEquals(newValue, Windows.UI.Xaml.DependencyProperty.UnsetValue)) {
                        Bridge.global.System.Diagnostics.Debug.Assert$1(!coerceWithCurrentValue, "Don't call SetCurrentValue with UnsetValue");
                        CSHTML5.Internal.INTERNAL_PropertyStore.ClearValueCommon(storage);
                        return;
                    }

                    if (!coerceWithCurrentValue) {
                        var newLocalValue = newValue;

                        if (storage.BaseValueSourceInternal === CSHTML5.Internal.BaseValueSourceInternal.Local) {
                            var currentExpr = Bridge.as(storage.LocalValue, Windows.UI.Xaml.Data.BindingExpression);
                            if (currentExpr != null) {
                                var newExpr = Bridge.as(newValue, Windows.UI.Xaml.Data.BindingExpression);
                                if (Bridge.referenceEquals(currentExpr, newExpr)) {
                                    Bridge.global.System.Diagnostics.Debug.Assert(newExpr.IsAttached);
                                    CSHTML5.Internal.INTERNAL_PropertyStore.RefreshBindingExpressionCommon(storage, newExpr);
                                    return;
                                }

                                if (newExpr != null || currentExpr.ParentBinding.Mode !== Windows.UI.Xaml.Data.BindingMode.TwoWay) {
                                    currentExpr.OnDetached(storage.Owner);
                                } else {
                                    newLocalValue = currentExpr;
                                }
                            }
                        }

                        storage.LocalValue = newLocalValue;
                    }

                    CSHTML5.Internal.INTERNAL_PropertyStore.UpdateEffectiveValue(storage, newValue, CSHTML5.Internal.BaseValueSourceInternal.Local, coerceWithCurrentValue, false, false, true);
                },
                RefreshBindingExpressionCommon: function (storage, expression) {
                    Bridge.global.System.Diagnostics.Debug.Assert$1(expression != null, "Expression should not be null");
                    Bridge.global.System.Diagnostics.Debug.Assert$1(storage.IsExpression || storage.IsExpressionFromStyle, "Property base value is not a BindingExpression !");

                    CSHTML5.Internal.INTERNAL_PropertyStore.UpdateEffectiveValue(storage, expression, expression.ParentBinding._isInStyle ? CSHTML5.Internal.BaseValueSourceInternal.LocalStyle : CSHTML5.Internal.BaseValueSourceInternal.Local, false, false, false, true);
                },
                ClearValueCommon: function (storage) {
                    var currentExpr = storage.IsExpression ? Bridge.as(storage.LocalValue, Windows.UI.Xaml.Data.BindingExpression) : null;
                    if (currentExpr != null) {
                        currentExpr.OnDetached(storage.Owner);
                    }

                    storage.LocalValue = Windows.UI.Xaml.DependencyProperty.UnsetValue;

                    CSHTML5.Internal.INTERNAL_PropertyStore.UpdateEffectiveValue(storage, Windows.UI.Xaml.DependencyProperty.UnsetValue, CSHTML5.Internal.BaseValueSourceInternal.Local, false, false, true, true);
                },
                CoerceValueCommon: function (storage) {
                    if (storage.IsCoercedWithCurrentValue) {
                        CSHTML5.Internal.INTERNAL_PropertyStore.SetValueCommon(storage, storage.ModifiedValue.CoercedValue, true);
                        return;
                    }

                    CSHTML5.Internal.INTERNAL_PropertyStore.UpdateEffectiveValue(storage, null, CSHTML5.Internal.BaseValueSourceInternal.Local, false, true, false, true);
                },
                GetEffectiveValue: function (storage) {
                    if (storage.HasModifiers) {
                        return (storage.IsCoercedWithCurrentValue || storage.IsCoerced) ? storage.ModifiedValue.CoercedValue : storage.ModifiedValue.ExpressionValue;
                    } else {
                        return storage.Value;
                    }
                },
                SetAnimationValue: function (storage, value) {
                    if (storage.IsExpression || storage.IsExpressionFromStyle) {
                        var currentExpr = Bridge.as(storage.ModifiedValue.BaseValue, Windows.UI.Xaml.Data.BindingExpression);
                        if (currentExpr != null) {
                            currentExpr.OnDetached(storage.Owner);
                        }
                    }

                    storage.AnimatedValue = value;

                    CSHTML5.Internal.INTERNAL_PropertyStore.UpdateEffectiveValue(storage, value, CSHTML5.Internal.BaseValueSourceInternal.Animated, false, false, Bridge.referenceEquals(value, Windows.UI.Xaml.DependencyProperty.UnsetValue), true);
                },
                ComputeEffectiveValue: function (storage, effectiveValue, kind) {
                    if (!storage.IsAnimatedOverLocal && !Bridge.referenceEquals(storage.LocalValue, Windows.UI.Xaml.DependencyProperty.UnsetValue)) {
                        effectiveValue.v = storage.LocalValue;
                        kind.v = CSHTML5.Internal.BaseValueSourceInternal.Local;
                    } else if (storage.IsAnimatedOverLocal && !Bridge.referenceEquals(storage.AnimatedValue, Windows.UI.Xaml.DependencyProperty.UnsetValue)) {
                        effectiveValue.v = storage.AnimatedValue;
                        kind.v = CSHTML5.Internal.BaseValueSourceInternal.Animated;
                    } else if (!Bridge.referenceEquals(storage.ImplicitReferenceValue, Windows.UI.Xaml.DependencyProperty.UnsetValue)) {
                        effectiveValue.v = storage.ImplicitReferenceValue;
                        kind.v = CSHTML5.Internal.BaseValueSourceInternal.ImplicitReference;
                    } else if (!Bridge.referenceEquals(storage.LocalStyleValue, Windows.UI.Xaml.DependencyProperty.UnsetValue)) {
                        effectiveValue.v = storage.LocalStyleValue;
                        kind.v = CSHTML5.Internal.BaseValueSourceInternal.LocalStyle;
                    } else if (!Bridge.referenceEquals(storage.ThemeStyleValue, Windows.UI.Xaml.DependencyProperty.UnsetValue)) {
                        effectiveValue.v = storage.ThemeStyleValue;
                        kind.v = CSHTML5.Internal.BaseValueSourceInternal.ThemeStyle;
                    } else if (!Bridge.referenceEquals(storage.InheritedValue, Windows.UI.Xaml.DependencyProperty.UnsetValue)) {
                        effectiveValue.v = storage.InheritedValue;
                        kind.v = CSHTML5.Internal.BaseValueSourceInternal.Inherited;
                    } else {
                        effectiveValue.v = storage.TypeMetadata.DefaultValue;
                        kind.v = CSHTML5.Internal.BaseValueSourceInternal.Default;
                    }
                },
                UpdateEffectiveValue: function (storage, newValue, newValueSource, coerceWithCurrentValue, coerceValue, clearValue, propagateChanges) {
                    Bridge.global.System.Diagnostics.Debug.Assert((coerceWithCurrentValue === coerceValue && !coerceValue) || coerceValue !== coerceWithCurrentValue);

                    var isCoerceOperation = coerceValue || coerceWithCurrentValue;
                    var oldBaseValueSource = storage.BaseValueSourceInternal;

                    var oldValue;
                    var currentExpr = null;

                    var effectiveValue = { };
                    var effectiveValueKind = { v : new CSHTML5.Internal.BaseValueSourceInternal() };
                    if (isCoerceOperation) {
                        effectiveValue.v = newValue;
                        effectiveValueKind.v = oldBaseValueSource;

                        oldValue = CSHTML5.Internal.INTERNAL_PropertyStore.GetEffectiveValue(storage);
                    } else {
                        CSHTML5.Internal.INTERNAL_PropertyStore.ComputeEffectiveValue(storage, effectiveValue, effectiveValueKind);

                        if (effectiveValueKind.v === oldBaseValueSource && newValueSource < effectiveValueKind.v) {
                            return;
                        }

                        oldValue = CSHTML5.Internal.INTERNAL_PropertyStore.GetEffectiveValue(storage);

                        currentExpr = (storage.IsExpression || storage.IsExpressionFromStyle) ? Bridge.as(storage.ModifiedValue.BaseValue, Windows.UI.Xaml.Data.BindingExpression) : null;

                        Bridge.global.System.Diagnostics.Debug.Assert$1(currentExpr == null || !currentExpr.IsAttached || Bridge.referenceEquals(currentExpr, effectiveValue.v), "Binding expression should be detached.");

                        storage.ResetValue();

                        storage.BaseValueSourceInternal = effectiveValueKind.v;
                    }

                    var computedValue = { };

                    if (!isCoerceOperation) {
                        var newExpr = Bridge.as(effectiveValue.v, Windows.UI.Xaml.Data.BindingExpression);
                        if (newExpr == null) {
                            computedValue.v = Bridge.referenceEquals(storage.Property.PropertyType, System.String) ? effectiveValue.v != null ? Bridge.toString(effectiveValue.v) : null : effectiveValue.v;
                            storage.Value = computedValue.v;
                        } else {
                            Bridge.global.System.Diagnostics.Debug.Assert(effectiveValueKind.v === CSHTML5.Internal.BaseValueSourceInternal.Local || effectiveValueKind.v === CSHTML5.Internal.BaseValueSourceInternal.LocalStyle);

                            var isNewBinding = !Bridge.referenceEquals(currentExpr, newExpr);
                            if (isNewBinding) {
                                if (newExpr.IsAttached) {
                                    throw new System.InvalidOperationException.$ctor1(System.String.format("Cannot attach an instance of '{0}' multiple times", [Windows.UI.Xaml.Data.BindingExpression]));
                                }
                                newExpr.OnAttached(storage.Owner);
                                storage.Value = newExpr;
                            }

                            if (effectiveValueKind.v === CSHTML5.Internal.BaseValueSourceInternal.Local) {
                                storage.SetExpressionValue(storage.TypeMetadata.DefaultValue, newExpr);
                            } else {
                                storage.SetExpressionFromStyleValue(storage.TypeMetadata.DefaultValue, newExpr);
                            }

                            computedValue.v = isNewBinding || Bridge.is(newValue, Windows.UI.Xaml.Data.BindingExpression) ? newExpr.GetValue(storage.Property, Bridge.getType(storage.Owner)) : newValue;
                            computedValue.v = Bridge.referenceEquals(storage.Property.PropertyType, System.String) ? computedValue.v != null ? Bridge.toString(computedValue.v) : null : computedValue.v;
                            storage.ModifiedValue.ExpressionValue = computedValue.v;
                        }
                    } else {
                        computedValue.v = coerceWithCurrentValue ? newValue : CSHTML5.Internal.INTERNAL_PropertyStore.GetCoercionBaseValue(storage);
                        if (coerceValue) {
                            storage.ResetCoercedValue();
                        }
                    }

                    if (coerceWithCurrentValue) {
                        var baseValue = CSHTML5.Internal.INTERNAL_PropertyStore.GetCoercionBaseValue(storage);
                        CSHTML5.Internal.INTERNAL_PropertyStore.ProcessCoerceValue(storage, computedValue, oldValue, baseValue, true);
                    }

                    if (!Bridge.staticEquals(storage.TypeMetadata.CoerceValueCallback, null) && !(clearValue && storage.FullValueSource === CSHTML5.Internal.BaseValueSourceInternal.Default)) {
                        var baseValue1 = CSHTML5.Internal.INTERNAL_PropertyStore.GetCoercionBaseValue(storage);
                        CSHTML5.Internal.INTERNAL_PropertyStore.ProcessCoerceValue(storage, computedValue, oldValue, baseValue1, false);
                    }

                    if (oldBaseValueSource === CSHTML5.Internal.BaseValueSourceInternal.Local) {
                        storage.Owner.RemoveSelfAsInheritanceContext(oldValue, null);
                    }

                    if (effectiveValueKind.v === CSHTML5.Internal.BaseValueSourceInternal.Local) {
                        storage.Owner.ProvideSelfAsInheritanceContext(computedValue.v, null);
                    }

                    if (!CSHTML5.Internal.INTERNAL_PropertyStore.ArePropertiesEqual(oldValue, computedValue.v, storage.Property.PropertyType)) {
                        if (!storage.TypeMetadata.Inherits || CSHTML5.Internal.INTERNAL_PropertyStore.ShouldRaisePropertyChanged(storage)) {
                            CSHTML5.Internal.INTERNAL_PropertyStore.OnPropertyChanged(storage, oldValue, computedValue.v);
                        }

                        if (storage.TypeMetadata.Inherits && propagateChanges) {
                            CSHTML5.Internal.INTERNAL_PropertyStore.CascadeInheritedPropertyToChildren(storage, computedValue.v);
                        }
                    }

                    if (currentExpr != null && currentExpr.ParentBinding.Mode === Windows.UI.Xaml.Data.BindingMode.TwoWay) {
                        currentExpr.TryUpdateSourceObject(computedValue.v);
                    }
                },
                ProcessCoerceValue: function (storage, newValue, oldValue, baseValue, coerceWithCurrentValue) {
                    newValue.v = coerceWithCurrentValue ? newValue.v : storage.TypeMetadata.CoerceValueCallback(storage.Owner, newValue.v);

                    if (!CSHTML5.Internal.INTERNAL_PropertyStore.ArePropertiesEqual(newValue.v, baseValue, storage.Property.PropertyType)) {
                        if (Bridge.referenceEquals(newValue.v, Windows.UI.Xaml.DependencyProperty.UnsetValue)) {
                            newValue.v = oldValue;
                        }

                        storage.SetCoercedValue(newValue.v, baseValue, coerceWithCurrentValue);
                    }
                },
                GetCoercionBaseValue: function (storage) {
                    var baseValue;
                    if (!storage.HasModifiers) {
                        baseValue = storage.Value;
                    } else if (storage.IsCoerced) {
                        if (storage.IsCoercedWithCurrentValue) {
                            baseValue = storage.ModifiedValue.CoercedValue;
                        } else if (storage.IsExpression) {
                            baseValue = storage.ModifiedValue.ExpressionValue;
                        } else {
                            baseValue = storage.ModifiedValue.BaseValue;
                        }
                    } else {
                        Bridge.global.System.Diagnostics.Debug.Assert(storage.IsExpression);
                        baseValue = storage.ModifiedValue.ExpressionValue;
                    }
                    return baseValue;
                },
                OnPropertyChanged: function (storage, oldValue, newValue) {
                    var $t;
                    var sender = storage.Owner;

                    var typeMetadata = storage.TypeMetadata;


                    if (Bridge.is(oldValue, Windows.UI.Xaml.IHasAccessToPropertiesWhereItIsUsed)) {
                        Bridge.cast(oldValue, Windows.UI.Xaml.IHasAccessToPropertiesWhereItIsUsed).Windows$UI$Xaml$IHasAccessToPropertiesWhereItIsUsed$PropertiesWhereUsed.remove(new (System.Collections.Generic.KeyValuePair$2(Windows.UI.Xaml.DependencyObject,Windows.UI.Xaml.DependencyProperty)).$ctor1(sender, storage.Property));
                    }

                    if (Bridge.is(newValue, Windows.UI.Xaml.IHasAccessToPropertiesWhereItIsUsed)) {
                        var newValueAsIHasAccessToPropertiesWhereItIsUsed = Bridge.cast(newValue, Windows.UI.Xaml.IHasAccessToPropertiesWhereItIsUsed);
                        newValueAsIHasAccessToPropertiesWhereItIsUsed.Windows$UI$Xaml$IHasAccessToPropertiesWhereItIsUsed$PropertiesWhereUsed.add(new (System.Collections.Generic.KeyValuePair$2(Windows.UI.Xaml.DependencyObject,Windows.UI.Xaml.DependencyProperty)).$ctor1(sender, storage.Property));
                    }


                    if (typeMetadata != null) {
                        CSHTML5.Internal.INTERNAL_PropertyStore.ApplyCssChanges(oldValue, newValue, typeMetadata, sender);

                        if (Bridge.is(sender, Windows.UI.Xaml.UIElement) && Bridge.cast(sender, Windows.UI.Xaml.UIElement)._isLoaded) {
                            if (!Bridge.staticEquals(typeMetadata.MethodToUpdateDom, null)) {
                                typeMetadata.MethodToUpdateDom(sender, newValue);
                            }
                        }
                    }


                    if (typeMetadata != null && !Bridge.staticEquals(typeMetadata.PropertyChangedCallback, null)) {
                        typeMetadata.PropertyChangedCallback(sender, new Windows.UI.Xaml.DependencyPropertyChangedEventArgs.$ctor1(oldValue, newValue, storage.Property));
                    }


                    if (storage.PropertyListeners != null) {
                        $t = Bridge.getEnumerator(storage.PropertyListeners);
                        try {
                            while ($t.moveNext()) {
                                var listener = $t.Current;
                                listener.Windows$UI$Xaml$Data$IPropertyChangedListener$OnPropertyChanged(sender, new Windows.UI.Xaml.DependencyPropertyChangedEventArgs.$ctor1(oldValue, newValue, storage.Property));
                            }
                        } finally {
                            if (Bridge.is($t, System.IDisposable)) {
                                $t.System$IDisposable$Dispose();
                            }
                        }
                    }
                },
                CascadeInheritedPropertyToChildren: function (storage, newValue) {
                    var $t;
                    var dependencyObject = storage.Owner;
                    var parentUIE;
                    if (((parentUIE = Bridge.as(dependencyObject, Windows.UI.Xaml.UIElement))) != null) {
                        if (parentUIE.INTERNAL_VisualChildrenInformation != null) {
                            $t = Bridge.getEnumerator(parentUIE.INTERNAL_VisualChildrenInformation.Keys);
                            try {
                                while ($t.moveNext()) {
                                    var child = $t.Current;
                                    child.SetInheritedValue(storage.Property, newValue, true);
                                }
                            } finally {
                                if (Bridge.is($t, System.IDisposable)) {
                                    $t.System$IDisposable$Dispose();
                                }
                            }
                        }
                    }
                },
                ArePropertiesEqual: function (obj1, obj2, type) {
                    if (Bridge.Reflection.isValueType(type) || Bridge.referenceEquals(type, System.String)) {
                        return Bridge.equals(obj1, obj2);
                    }
                    return Bridge.referenceEquals(obj1, obj2);
                },
                ShouldRaisePropertyChanged: function (storage) {
                    if (!storage.Property.IsAttached) {
                        return Bridge.Reflection.isAssignableFrom(storage.Property.OwnerType, Bridge.getType(storage.Owner)) || Bridge.referenceEquals(storage.Property, Windows.UI.Xaml.FrameworkElement.DataContextProperty);
                    }
                    return true;
                },
                SetInheritedValue: function (storage, newValue, recursively) {
                    storage.InheritedValue = newValue;

                    CSHTML5.Internal.INTERNAL_PropertyStore.UpdateEffectiveValue(storage, newValue, CSHTML5.Internal.BaseValueSourceInternal.Inherited, false, false, Bridge.referenceEquals(newValue, Windows.UI.Xaml.DependencyProperty.UnsetValue), recursively);
                },
                SetLocalStyleValue: function (storage, newValue) {
                    if (storage.BaseValueSourceInternal === CSHTML5.Internal.BaseValueSourceInternal.LocalStyle) {
                        var oldExpr = storage.IsExpressionFromStyle ? Bridge.as(storage.LocalStyleValue, Windows.UI.Xaml.Data.BindingExpression) : null;
                        if (oldExpr != null) {
                            oldExpr.OnDetached(storage.Owner);
                        }
                    }

                    storage.LocalStyleValue = newValue;

                    CSHTML5.Internal.INTERNAL_PropertyStore.UpdateEffectiveValue(storage, newValue, CSHTML5.Internal.BaseValueSourceInternal.LocalStyle, false, false, Bridge.referenceEquals(newValue, Windows.UI.Xaml.DependencyProperty.UnsetValue), true);
                },
                SetImplicitReferenceValue: function (storage, newValue) {
                    storage.ImplicitReferenceValue = newValue;

                    CSHTML5.Internal.INTERNAL_PropertyStore.UpdateEffectiveValue(storage, newValue, CSHTML5.Internal.BaseValueSourceInternal.ImplicitReference, false, false, Bridge.referenceEquals(newValue, Windows.UI.Xaml.DependencyProperty.UnsetValue), true);
                },
                SetThemeStyleValue: function (storage, newValue) {
                    if (storage.BaseValueSourceInternal === CSHTML5.Internal.BaseValueSourceInternal.ThemeStyle) {
                        var oldExpr = storage.IsExpressionFromStyle ? Bridge.as(storage.ThemeStyleValue, Windows.UI.Xaml.Data.BindingExpression) : null;
                        if (oldExpr != null) {
                            oldExpr.OnDetached(storage.Owner);
                        }
                    }

                    storage.ThemeStyleValue = newValue;

                    CSHTML5.Internal.INTERNAL_PropertyStore.UpdateEffectiveValue(storage, newValue, CSHTML5.Internal.BaseValueSourceInternal.ThemeStyle, false, false, Bridge.referenceEquals(newValue, Windows.UI.Xaml.DependencyProperty.UnsetValue), true);
                },
                ApplyCssChanges: function (oldValue, newValue, typeMetadata, sender) {
                    var $t;
                    if (!Bridge.staticEquals(typeMetadata.GetCSSEquivalent, null)) {
                        var cssEquivalent = typeMetadata.GetCSSEquivalent(sender);
                        if (cssEquivalent != null) {
                            CSHTML5.Internal.INTERNAL_PropertyStore.ApplyPropertyChanged(sender, cssEquivalent, oldValue, newValue);
                        }
                    }

                    if (!Bridge.staticEquals(typeMetadata.GetCSSEquivalents, null)) {
                        var cssEquivalents = typeMetadata.GetCSSEquivalents(sender);
                        if (cssEquivalents != null) {
                            $t = Bridge.getEnumerator(cssEquivalents);
                            try {
                                while ($t.moveNext()) {
                                    var cssEquivalent1 = $t.Current;
                                    CSHTML5.Internal.INTERNAL_PropertyStore.ApplyPropertyChanged(sender, cssEquivalent1, oldValue, newValue);
                                }
                            } finally {
                                if (Bridge.is($t, System.IDisposable)) {
                                    $t.System$IDisposable$Dispose();
                                }
                            }
                        }
                    }
                },
                ApplyPropertyChanged: function (sender, cssEquivalent, oldValue, newValue) {
                    var $t, $t1;

                    if (cssEquivalent.Name != null && cssEquivalent.Name.Count > 0 || !Bridge.staticEquals(cssEquivalent.CallbackMethod, null)) {
                        var uiElement = cssEquivalent.UIElement || (Bridge.as(sender, Windows.UI.Xaml.UIElement));

                        var hasTemplate = (Bridge.is(uiElement, Windows.UI.Xaml.Controls.Control)) && Bridge.cast(uiElement, Windows.UI.Xaml.Controls.Control).HasTemplate;

                        if (!hasTemplate || cssEquivalent.ApplyAlsoWhenThereIsAControlTemplate) {
                            if (!Bridge.staticEquals(cssEquivalent.CallbackMethod, null)) {



                                cssEquivalent.CallbackMethod(cssEquivalent.UIElement, new Windows.UI.Xaml.DependencyPropertyChangedEventArgs.$ctor1(oldValue, newValue, cssEquivalent.DependencyProperty));
                            } else {
                                if (cssEquivalent.DomElement == null && uiElement != null) {
                                    cssEquivalent.DomElement = uiElement.INTERNAL_OuterDomElement;
                                }
                                if (cssEquivalent.DomElement != null) {
                                    if (Bridge.is(newValue, Windows.UI.Xaml.ICanConvertToCSSValue)) {
                                        cssEquivalent.Value = function (finalInstance, value) {
                                            return Bridge.cast(value, Windows.UI.Xaml.ICanConvertToCSSValue).Windows$UI$Xaml$ICanConvertToCSSValue$ConvertToCSSValue();
                                        };
                                    }
                                    if (Bridge.is(newValue, Windows.UI.Xaml.ICanConvertToCSSValues)) {
                                        cssEquivalent.Values = function (finalInstance, value) {
                                            return Bridge.cast(value, Windows.UI.Xaml.ICanConvertToCSSValues).Windows$UI$Xaml$ICanConvertToCSSValues$ConvertToCSSValues(sender);
                                        };
                                    }
                                    if (Bridge.staticEquals(cssEquivalent.Value, null)) {
                                        cssEquivalent.Value = function (finalInstance, value) {
                                            var $t;
                                            return ($t = value, $t != null ? $t : "");
                                        };
                                    }
                                    if (!Bridge.staticEquals(cssEquivalent.Values, null)) {
                                        var cssValues = cssEquivalent.Values(sender, newValue);

                                        if (cssEquivalent.OnlyUseVelocity) {
                                            $t = Bridge.getEnumerator(cssValues);
                                            try {
                                                while ($t.moveNext()) {
                                                    var cssValue = $t.Current;
                                                    CSHTML5.Internal.INTERNAL_HtmlDomManager.SetDomElementStylePropertyUsingVelocity(cssEquivalent.DomElement, cssEquivalent.Name, cssValue);
                                                }
                                            } finally {
                                                if (Bridge.is($t, System.IDisposable)) {
                                                    $t.System$IDisposable$Dispose();
                                                }
                                            }
                                        } else {
                                            $t1 = Bridge.getEnumerator(cssValues);
                                            try {
                                                while ($t1.moveNext()) {
                                                    var cssValue1 = $t1.Current;
                                                    CSHTML5.Internal.INTERNAL_HtmlDomManager.SetDomElementStyleProperty(cssEquivalent.DomElement, cssEquivalent.Name, cssValue1);
                                                }
                                            } finally {
                                                if (Bridge.is($t1, System.IDisposable)) {
                                                    $t1.System$IDisposable$Dispose();
                                                }
                                            }
                                        }
                                    } else if (!Bridge.staticEquals(cssEquivalent.Value, null)) {
                                        var cssValue2 = cssEquivalent.Value(sender, newValue);

                                        if (!(Bridge.is(cssValue2, System.Collections.Generic.Dictionary$2(System.String,System.Object)))) {
                                            if (cssEquivalent.OnlyUseVelocity) {
                                                CSHTML5.Internal.INTERNAL_HtmlDomManager.SetDomElementStylePropertyUsingVelocity(cssEquivalent.DomElement, cssEquivalent.Name, cssValue2);
                                            } else {
                                                CSHTML5.Internal.INTERNAL_HtmlDomManager.SetDomElementStyleProperty(cssEquivalent.DomElement, cssEquivalent.Name, cssValue2);
                                            }
                                        } else {
                                            if (Bridge.is(newValue, Windows.UI.Color)) {
                                                var newColor = System.Nullable.getValue(Bridge.cast(Bridge.unbox(newValue, Windows.UI.Color), Windows.UI.Color));
                                                if (cssEquivalent.OnlyUseVelocity) {
                                                    CSHTML5.Internal.INTERNAL_HtmlDomManager.SetDomElementStylePropertyUsingVelocity(cssEquivalent.DomElement, cssEquivalent.Name, newColor.INTERNAL_ToHtmlStringForVelocity());
                                                } else {
                                                    CSHTML5.Internal.INTERNAL_HtmlDomManager.SetDomElementStyleProperty(cssEquivalent.DomElement, cssEquivalent.Name, newColor.INTERNAL_ToHtmlString(1.0));
                                                }
                                            }
                                        }
                                    }
                                }
                            }
                        }
                    } else {
                        throw new System.InvalidOperationException.$ctor1("Please set the Name property of the CSSEquivalent class.");
                    }
                },
                ListenToChanged: function (target, property, updateSourceCallback) {
                    var $t;
                    var storage = { };
                    CSHTML5.Internal.INTERNAL_PropertyStore.TryGetStorage(target, property, true, storage);
                    var listeners = storage.v.PropertyListeners;
                    if (listeners == null) {
                        listeners = ($t = new (System.Collections.Generic.List$1(Windows.UI.Xaml.Data.IPropertyChangedListener)).ctor(), storage.v.PropertyListeners = $t, $t);
                    }

                    var listener = new Windows.UI.Xaml.Data.PropertyChangedListener(storage.v, updateSourceCallback);

                    listeners.add(listener);
                    return listener;
                }
            }
        }
    });

    Bridge.define("CSHTML5.Internal.INTERNAL_ShapesDrawHelpers", {
        statics: {
            methods: {
                CreateDomElementForPathAndSimilar: function (associatedUIElement, parentRef, canvasDomElement, domElementWhereToPlaceChildren) {
                    domElementWhereToPlaceChildren.v = null;
                    var div = CSHTML5.Internal.INTERNAL_HtmlDomManager.CreateDomElementAndAppendIt("div", parentRef, associatedUIElement);
                    var divStyle = CSHTML5.Internal.INTERNAL_HtmlDomManager.GetDomElementStyleForModification(div);
                    divStyle.lineHeight = "0";
                    divStyle.width = "100%";
                    divStyle.height = "100%";
                    divStyle.fontSize = "0px";
                    canvasDomElement.v = CSHTML5.Internal.INTERNAL_HtmlDomManager.CreateDomElementAndAppendIt("canvas", div, associatedUIElement);
                    var style = CSHTML5.Internal.INTERNAL_HtmlDomManager.GetDomElementStyleForModification(canvasDomElement.v);
                    style.width = "0px";
                    style.height = "0px";
                    style.marginBottom = "-4px";
                    return div;
                },
                /**
                 * Prepares the Shape so that its canvas has the size it should have, depending on its container, content and Stretch mode.
                 *
                 * @static
                 * @this CSHTML5.Internal.INTERNAL_ShapesDrawHelpers
                 * @memberof CSHTML5.Internal.INTERNAL_ShapesDrawHelpers
                 * @param   {Windows.UI.Xaml.FrameworkElement}    frameworkElement    The Shape containing the canvas.
                 * @param   {System.Object}                       canvasDomElement    The canvas in the Shape.
                 * @param   {number}                              minX                The minimum X-coordinate of the points in the Shape.
                 * @param   {number}                              maxX                The maximum X-coordinate of the points in the Shape.
                 * @param   {number}                              minY                The minimum Y-coordinate of the points in the Shape.
                 * @param   {number}                              maxY                The maximum Y-coordinate of the points in the Shape.
                 * @param   {Windows.UI.Xaml.Media.Stretch}       stretch             The Stretch mode to apply on the Shape
                 * @param   {Windows.Foundation.Size}             shapeActualSize
                 * @return  {void}
                 */
                PrepareStretch: function (frameworkElement, canvasDomElement, minX, maxX, minY, maxY, stretch, shapeActualSize) {
                    var canvasStyle = CSHTML5.Internal.INTERNAL_HtmlDomManager.GetDomElementStyleForModification(canvasDomElement);


                    var sizeX = maxX;
                    var sizeY = maxY;

                    if (stretch !== Windows.UI.Xaml.Media.Stretch.None) {
                        sizeX = maxX - minX;
                        sizeY = maxY - minY;
                    } else {
                        if (minX < 0) {
                            sizeX = maxX - minX;
                        }
                        if (minY < 0) {
                            sizeY = maxY - minY;
                        }
                    }

                    var sizeToApply = new Windows.Foundation.Size.$ctor1(Math.max(sizeX + 1, 0), Math.max(sizeY + 1, 0));

                    var style = CSHTML5.Internal.INTERNAL_HtmlDomManager.GetFrameworkElementOuterStyleForModification(frameworkElement);
                    var frameworkElementWidthWasSpecified = false;
                    var frameworkElementWidth = frameworkElement.Width;
                    var frameworkElementHeight = frameworkElement.Height;
                    if (!isNaN(frameworkElementWidth)) {
                        frameworkElementWidthWasSpecified = true;
                        sizeToApply.Width = frameworkElementWidth + 1;
                        style.width = System.Double.format(sizeToApply.Width) + "px";
                    }
                    var frameworkElementHeightWasSpecified = false;
                    if (!isNaN(frameworkElementHeight)) {
                        frameworkElementHeightWasSpecified = true;
                        sizeToApply.Height = frameworkElementHeight + 1;
                        style.height = System.Double.format(sizeToApply.Height) + "px";
                    }

                    CSHTML5.Internal.INTERNAL_HtmlDomManager.SetDomElementAttribute(canvasDomElement, "width", Bridge.box(sizeToApply.Width, System.Double, System.Double.format, System.Double.getHashCode));
                    CSHTML5.Internal.INTERNAL_HtmlDomManager.SetDomElementAttribute(canvasDomElement, "height", Bridge.box(sizeToApply.Height, System.Double, System.Double.format, System.Double.getHashCode));

                    canvasStyle.width = System.Double.format(sizeToApply.Width) + "px";
                    canvasStyle.height = System.Double.format(sizeToApply.Height) + "px";

                    if (frameworkElementWidthWasSpecified && frameworkElementHeightWasSpecified) {
                        shapeActualSize.v = new Windows.Foundation.Size.$ctor1(frameworkElementWidth, frameworkElementHeight);
                    } else {
                        shapeActualSize.v = frameworkElement.INTERNAL_GetActualWidthAndHeight();
                        if (frameworkElementWidthWasSpecified) {
                            shapeActualSize.v.Width = frameworkElementWidth;
                        }
                        if (frameworkElementHeightWasSpecified) {
                            shapeActualSize.v.Height = frameworkElementHeight;
                        }
                    }

                    if (!frameworkElementWidthWasSpecified || !frameworkElementHeightWasSpecified) {

                        if (!frameworkElementWidthWasSpecified) {
                            canvasStyle.width = System.Double.format(shapeActualSize.v.Width) + "px";
                            CSHTML5.Internal.INTERNAL_HtmlDomManager.SetDomElementAttribute(canvasDomElement, "width", Bridge.box(shapeActualSize.v.Width + 1, System.Double, System.Double.format, System.Double.getHashCode));
                        }
                        if (!frameworkElementHeightWasSpecified) {
                            canvasStyle.height = System.Double.format(shapeActualSize.v.Height) + "px";
                            CSHTML5.Internal.INTERNAL_HtmlDomManager.SetDomElementAttribute(canvasDomElement, "height", Bridge.box(shapeActualSize.v.Height + 1, System.Double, System.Double.format, System.Double.getHashCode));
                        }
                    }

                    var context = CSHTML5.Internal.INTERNAL_HtmlDomManager.Get2dCanvasContext(canvasDomElement);
                    context.translate(0.5, 0.5);
                },
                /**
                 * Gets the multiplicators and offsets to fit the Stretch value.
                 *
                 * @static
                 * @this CSHTML5.Internal.INTERNAL_ShapesDrawHelpers
                 * @memberof CSHTML5.Internal.INTERNAL_ShapesDrawHelpers
                 * @param   {Windows.UI.Xaml.FrameworkElement}    frameworkElement                      The element on which we make the stretch
                 * @param   {number}                              strokeThickness                       The thickness of the stroke
                 * @param   {number}                              minX                                  the minimal value on the horizontal axis as defined by the user
                 * @param   {number}                              maxX                                  the maximal value on the horizontal axis as defined by the user
                 * @param   {number}                              minY                                  the minimal value on the vertical axis as defined by the user
                 * @param   {number}                              maxY                                  the minimal value on the vertical axis as defined by the user
                 * @param   {Windows.UI.Xaml.Media.Stretch}       stretch                               the Stretch value to apply
                 * @param   {Windows.Foundation.Size}             shapeActualSize                       
                 * @param   {System.Double}                       horizontalMultiplicator               the resulting value by which we have to multiply the horizontal coordinates to fit the Stretch value
                 * @param   {System.Double}                       verticalMultiplicator                 the resulting value by which we have to multiply the vertical coordinates to fit the Stretch value
                 * @param   {System.Double}                       xOffsetToApplyBeforeMultiplication    the offset to apply on horizontal coordinates before applying multiplicators
                 * @param   {System.Double}                       yOffsetToApplyBeforeMultiplication    the offset to apply on vertical coordinates before applying multiplicators
                 * @param   {System.Double}                       xOffsetToApplyAfterMultiplication     the offset to apply on horizontal coordinates after applying multiplicators
                 * @param   {System.Double}                       yOffsetToApplyAfterMultiplication     the offset to apply on vertical coordinates after applying multiplicators
                 * @param   {Windows.Foundation.Point}            marginOffsets                         the offset to apply on the Left margin of the element to compensate for the negative coordinates in the path
                 * @return  {void}
                 */
                GetMultiplicatorsAndOffsetForStretch: function (frameworkElement, strokeThickness, minX, maxX, minY, maxY, stretch, shapeActualSize, horizontalMultiplicator, verticalMultiplicator, xOffsetToApplyBeforeMultiplication, yOffsetToApplyBeforeMultiplication, xOffsetToApplyAfterMultiplication, yOffsetToApplyAfterMultiplication, marginOffsets) {

                    horizontalMultiplicator.v = 1.0;
                    verticalMultiplicator.v = 1.0;
                    xOffsetToApplyBeforeMultiplication.v = 0.0;
                    yOffsetToApplyBeforeMultiplication.v = 0.0;
                    xOffsetToApplyAfterMultiplication.v = 0.0;
                    yOffsetToApplyAfterMultiplication.v = 0.0;
                    if (shapeActualSize.Width > 0 && shapeActualSize.Height > 0) {
                        var nonStretchedWidthOfThePath = maxX - minX;
                        var nonStretchedHeightOfThePath = maxY - minY;
                        if (nonStretchedWidthOfThePath === 0) {
                            nonStretchedWidthOfThePath = 1;
                        }
                        if (nonStretchedHeightOfThePath === 0) {
                            nonStretchedHeightOfThePath = 1;
                        }

                        if (stretch !== Windows.UI.Xaml.Media.Stretch.None) {
                            horizontalMultiplicator.v = (shapeActualSize.Width - strokeThickness) / nonStretchedWidthOfThePath;
                            verticalMultiplicator.v = (shapeActualSize.Height - strokeThickness) / nonStretchedHeightOfThePath;


                            if (stretch === Windows.UI.Xaml.Media.Stretch.Uniform) {
                                if (horizontalMultiplicator.v > verticalMultiplicator.v) {
                                    horizontalMultiplicator.v = verticalMultiplicator.v;
                                } else {
                                    verticalMultiplicator.v = horizontalMultiplicator.v;
                                }
                            } else if (stretch === Windows.UI.Xaml.Media.Stretch.UniformToFill) {
                                if (horizontalMultiplicator.v < verticalMultiplicator.v) {
                                    horizontalMultiplicator.v = verticalMultiplicator.v;
                                } else {
                                    verticalMultiplicator.v = horizontalMultiplicator.v;
                                }
                            }
                            if (strokeThickness > 1) {
                                xOffsetToApplyBeforeMultiplication.v = -minX;
                                yOffsetToApplyBeforeMultiplication.v = -minY;
                                xOffsetToApplyAfterMultiplication.v = strokeThickness / 2 - 0.5;
                                yOffsetToApplyAfterMultiplication.v = strokeThickness / 2 - 0.5;
                            } else {
                                xOffsetToApplyBeforeMultiplication.v = -minX;
                                yOffsetToApplyBeforeMultiplication.v = -minY;
                                xOffsetToApplyAfterMultiplication.v = 0;
                                yOffsetToApplyAfterMultiplication.v = 0;

                            }
                            marginOffsets.v = new Windows.Foundation.Point.ctor();
                        } else {
                            var Xnormalization = minX < 0 ? -minX : 0.0;
                            var Ynormalization = minY < 0 ? -minY : 0.0;
                            marginOffsets.v = new Windows.Foundation.Point.$ctor1(-Xnormalization, -Ynormalization);

                            xOffsetToApplyBeforeMultiplication.v = Xnormalization;
                            yOffsetToApplyBeforeMultiplication.v = Ynormalization;
                        }
                    } else {
                        marginOffsets.v = new Windows.Foundation.Point.ctor();
                    }
                },
                PrepareLine: function (canvasDomElement, StartPoint, EndPoint) {
                    var context = CSHTML5.Internal.INTERNAL_HtmlDomManager.Get2dCanvasContext(canvasDomElement);

                    context.beginPath();
                    context.moveTo(StartPoint.X, StartPoint.Y);
                    context.lineTo(EndPoint.X, EndPoint.Y);



                },
                PrepareEllipse: function (canvasDomElement, ellipseWidth, ellipseHeight, centerX, centerY) {
                    var context = CSHTML5.Internal.INTERNAL_HtmlDomManager.Get2dCanvasContext(canvasDomElement);








                    var kappa = 0.5522848;
                    var ox = (ellipseWidth / 2) * kappa;
                    var oy = (ellipseHeight / 2) * kappa;

                    var topPoint = new Windows.Foundation.Point.$ctor1(centerX, centerY - ellipseHeight / 2);
                    var leftPoint = new Windows.Foundation.Point.$ctor1(centerX - ellipseWidth / 2, centerY);
                    var rightPoint = new Windows.Foundation.Point.$ctor1(centerX + ellipseWidth / 2, centerY);
                    var bottomPoint = new Windows.Foundation.Point.$ctor1(centerX, centerY + ellipseHeight / 2);

                    context.beginPath();
                    context.moveTo(leftPoint.X, leftPoint.Y);
                    context.bezierCurveTo(centerX - ellipseWidth / 2, centerY - oy, centerX - ox, centerY - ellipseHeight / 2, topPoint.X, topPoint.Y);
                    context.bezierCurveTo(centerX + ox, centerY - ellipseHeight / 2, centerX + ellipseWidth / 2, centerY - oy, rightPoint.X, rightPoint.Y);
                    context.bezierCurveTo(centerX + ellipseWidth / 2, centerY + oy, centerX + ox, centerY + ellipseHeight / 2, bottomPoint.X, bottomPoint.Y);
                    context.bezierCurveTo(centerX - ox, centerY + ellipseHeight / 2, centerX - ellipseWidth / 2, centerY + oy, leftPoint.X, leftPoint.Y);





                },
                ApplyTransformToCanvas: function (transform, canvasDomElement) {
                    var matrixTransform = Bridge.as(transform, Windows.UI.Xaml.Media.MatrixTransform);
                    if (matrixTransform != null) {
                        var m = matrixTransform.Matrix.$clone();


                        var context = CSHTML5.Internal.INTERNAL_HtmlDomManager.Get2dCanvasContext(canvasDomElement);
                        context.transform(m.M11, m.M12, m.M21, m.M22, m.OffsetX, m.OffsetY);

                    } else {
                        if (transform != null) {
                            throw new System.NotSupportedException.$ctor1(System.String.format("'{0}' is not currently supported in PathGeometry.", [Bridge.getType(transform)]));
                        }
                    }
                },
                FixCoordinatesGreaterThanInt32MaxValue: function (minX, minY, maxX, maxY, actualTransform, int32FactorX, int32FactorY) {
                    var $t;
                    var maxValue = 2147483647;

                    if (Math.abs(minX) > maxValue || Math.abs(maxX) > maxValue) {
                        var max = Math.max(Math.abs(minX), Math.abs(maxX));
                        int32FactorX.v = (maxValue / max) * 0.99;
                        var invertedInt32FactorX = 1 / int32FactorX.v;
                        var scaleTransformMatrix = new Windows.UI.Xaml.Media.Matrix.$ctor1(invertedInt32FactorX, 0, 0, 1, 0, 0);
                        if (actualTransform.v != null) {
                            if (Bridge.is(actualTransform.v, Windows.UI.Xaml.Media.MatrixTransform)) {
                                actualTransform.v = ($t = new Windows.UI.Xaml.Media.MatrixTransform(), $t.Matrix = Windows.UI.Xaml.Media.Matrix.Multiply(scaleTransformMatrix.$clone(), Bridge.cast(actualTransform.v, Windows.UI.Xaml.Media.MatrixTransform).Matrix.$clone()), $t);
                            } else {
                                throw new System.NotSupportedException.$ctor1("The size of the shape exceeds Int32. This is currently supported only if the Transform is null or a MatrixTransform.");
                            }
                        } else {
                            actualTransform.v = ($t = new Windows.UI.Xaml.Media.MatrixTransform(), $t.Matrix = scaleTransformMatrix.$clone(), $t);
                        }
                    }

                    if (Math.abs(minY) > maxValue || Math.abs(maxY) > maxValue) {
                        var max1 = Math.max(Math.abs(minY), Math.abs(maxY));
                        int32FactorY.v = (maxValue / max1) * 0.99;
                        var invertedInt32FactorY = 1 / int32FactorY.v;
                        var scaleTransformMatrix1 = new Windows.UI.Xaml.Media.Matrix.$ctor1(1, 0, 0, invertedInt32FactorY, 0, 0);
                        if (actualTransform.v != null) {
                            if (Bridge.is(actualTransform.v, Windows.UI.Xaml.Media.MatrixTransform)) {
                                actualTransform.v = ($t = new Windows.UI.Xaml.Media.MatrixTransform(), $t.Matrix = Windows.UI.Xaml.Media.Matrix.Multiply(scaleTransformMatrix1.$clone(), Bridge.cast(actualTransform.v, Windows.UI.Xaml.Media.MatrixTransform).Matrix.$clone()), $t);
                            } else {
                                throw new System.NotSupportedException.$ctor1("The size of the shape exceeds Int32. This is currently supported only if the Transform is null or a MatrixTransform.");
                            }
                        } else {
                            actualTransform.v = ($t = new Windows.UI.Xaml.Media.MatrixTransform(), $t.Matrix = scaleTransformMatrix1.$clone(), $t);
                        }
                    }
                }
            }
        }
    });

    Bridge.define("CSHTML5.Internal.INTERNAL_SimulatorExecuteJavaScript", {
        statics: {
            fields: {
                EnableInteropLogging: false,
                _pendingAsyncJavaScriptToExecute: null,
                _disableAsyncJavaScriptExecution: false,
                _isDispatcherPending: false
            },
            props: {
                DisableAsyncJavaScriptExecution: {
                    get: function () {
                        return CSHTML5.Internal.INTERNAL_SimulatorExecuteJavaScript._disableAsyncJavaScriptExecution;
                    },
                    set: function (value) {
                        CSHTML5.Internal.INTERNAL_SimulatorExecuteJavaScript._disableAsyncJavaScriptExecution = value;
                    }
                }
            },
            ctors: {
                init: function () {
                    this._pendingAsyncJavaScriptToExecute = new (System.Collections.Generic.List$1(System.String)).ctor();
                    this._disableAsyncJavaScriptExecution = false;
                    this._isDispatcherPending = false;
                }
            },
            methods: {
                /**
                 * Executes JavaScript code immediately. This also forces all the pending async JS code to be executed (flush).
                 *
                 * @static
                 * @this CSHTML5.Internal.INTERNAL_SimulatorExecuteJavaScript
                 * @memberof CSHTML5.Internal.INTERNAL_SimulatorExecuteJavaScript
                 * @param   {string}           javaScriptToExecute        The JS code to execute.
                 * @param   {string}           commentForDebugging        Some optional comments to write to the log of JS calls.
                 * @param   {boolean}          noImpactOnPendingJSCode
                 * @return  {System.Object}
                 */
                ExecuteJavaScriptSync: function (javaScriptToExecute, commentForDebugging, noImpactOnPendingJSCode) {
                    if (commentForDebugging === void 0) { commentForDebugging = null; }
                    if (noImpactOnPendingJSCode === void 0) { noImpactOnPendingJSCode = false; }
                    javaScriptToExecute = {v:javaScriptToExecute};
                    if (!noImpactOnPendingJSCode) {
                        CSHTML5.Internal.INTERNAL_SimulatorExecuteJavaScript.AddCommentsForDebuggingIfAny(javaScriptToExecute, commentForDebugging);

                        var aggregatedPendingJavaScriptCode = CSHTML5.Internal.INTERNAL_SimulatorExecuteJavaScript.ReadAndClearAggregatedPendingJavaScriptCode();

                        if (!System.String.isNullOrWhiteSpace(aggregatedPendingJavaScriptCode)) {
                            javaScriptToExecute.v = "// [START OF PENDING JAVASCRIPT]" + ("\n" || "") + (aggregatedPendingJavaScriptCode || "") + ("\n" || "") + "// [END OF PENDING JAVASCRIPT]" + ("\n" || "") + ("\n" || "") + (javaScriptToExecute.v || "");
                        }
                    }

                    return CSHTML5.Internal.INTERNAL_SimulatorExecuteJavaScript.PerformActualInteropCall(javaScriptToExecute.v, "SYNC");
                },
                /**
                 * Postpone the executes of the JavaScript code to until a JavaScript execution "with return value" is made, or when the process is idle. This enables to significantly improve performance in the Simulator.
                 *
                 * @static
                 * @this CSHTML5.Internal.INTERNAL_SimulatorExecuteJavaScript
                 * @memberof CSHTML5.Internal.INTERNAL_SimulatorExecuteJavaScript
                 * @param   {string}    javaScriptToExecute    The JS code to execute.
                 * @param   {string}    commentForDebugging    Some optional comments to write to the log of JS calls.
                 * @return  {void}
                 */
                ExecuteJavaScriptAsync: function (javaScriptToExecute, commentForDebugging) {
                    if (commentForDebugging === void 0) { commentForDebugging = null; }
                    javaScriptToExecute = {v:javaScriptToExecute};
                    CSHTML5.Internal.INTERNAL_SimulatorExecuteJavaScript.AddCommentsForDebuggingIfAny(javaScriptToExecute, commentForDebugging);

                    if (!CSHTML5.Internal.INTERNAL_SimulatorExecuteJavaScript._disableAsyncJavaScriptExecution) {

                        CSHTML5.Internal.INTERNAL_SimulatorExecuteJavaScript._pendingAsyncJavaScriptToExecute;
                        {
                            CSHTML5.Internal.INTERNAL_SimulatorExecuteJavaScript._pendingAsyncJavaScriptToExecute.add(javaScriptToExecute.v);
                        }

                        if (!CSHTML5.Internal.INTERNAL_SimulatorExecuteJavaScript._isDispatcherPending) {
                            CSHTML5.Internal.INTERNAL_SimulatorExecuteJavaScript._isDispatcherPending = true;

                            DotNetForHtml5.Core.INTERNAL_Simulator.WebControlDispatcherBeginInvoke(function () {
                                if (CSHTML5.Internal.INTERNAL_SimulatorExecuteJavaScript._isDispatcherPending) {
                                    CSHTML5.Internal.INTERNAL_SimulatorExecuteJavaScript.ExecutePendingJavaScriptCode("BEGININVOKE COMPLETED");
                                }
                            });
                        }
                    } else {
                        CSHTML5.Internal.INTERNAL_SimulatorExecuteJavaScript.PerformActualInteropCall(javaScriptToExecute.v, "ASYNC DISABLED");
                    }
                },
                ExecutePendingJavaScriptCode: function (reasonForPerformingTheCallNow) {
                    var aggregatedPendingJavaScriptCode = CSHTML5.Internal.INTERNAL_SimulatorExecuteJavaScript.ReadAndClearAggregatedPendingJavaScriptCode();

                    if (!System.String.isNullOrWhiteSpace(aggregatedPendingJavaScriptCode)) {
                        CSHTML5.Internal.INTERNAL_SimulatorExecuteJavaScript.PerformActualInteropCall(aggregatedPendingJavaScriptCode, reasonForPerformingTheCallNow);
                    }
                },
                ReadAndClearAggregatedPendingJavaScriptCode: function () {
                    CSHTML5.Internal.INTERNAL_SimulatorExecuteJavaScript._isDispatcherPending = false;

                    CSHTML5.Internal.INTERNAL_SimulatorExecuteJavaScript._pendingAsyncJavaScriptToExecute;
                    {
                        var aggregatedPendingJavaScriptCode = Bridge.toArray(System.Linq.Enumerable.from(CSHTML5.Internal.INTERNAL_SimulatorExecuteJavaScript._pendingAsyncJavaScriptToExecute).toList(System.String)).join("\r\n");
                        CSHTML5.Internal.INTERNAL_SimulatorExecuteJavaScript._pendingAsyncJavaScriptToExecute.clear();
                        return aggregatedPendingJavaScriptCode;
                    }
                },
                AddCommentsForDebuggingIfAny: function (javaScriptToExecute, commentForDebugging) {
                    if (commentForDebugging != null) {
                        javaScriptToExecute.v = "//" + (commentForDebugging || "") + ("\n" || "") + (javaScriptToExecute.v || "");
                    }
                },
                PerformActualInteropCall: function (javaScriptToExecute, reasonForPerformingTheCallNow) {
                    javaScriptToExecute = "//---- START INTEROP (" + (reasonForPerformingTheCallNow || "") + ") ----" + ("\n" || "") + (javaScriptToExecute || "") + ("\n" || "") + "//---- END INTEROP (" + (reasonForPerformingTheCallNow || "") + ") ----";
                    try {
                        return DotNetForHtml5.Core.INTERNAL_Simulator.JavaScriptExecutionHandler.ExecuteJavaScriptWithResult(javaScriptToExecute);
                    } catch ($e1) {
                        $e1 = System.Exception.create($e1);
                        var ex;
                        if (Bridge.is($e1, System.InvalidOperationException)) {
                            ex = $e1;
                            throw new System.InvalidOperationException.$ctor2("Unable to execute the following JavaScript code: " + ("\n" || "") + (javaScriptToExecute || ""), ex);
                        } else {
                            throw $e1;
                        }
                    }
                }
            }
        }
    });

    Bridge.define("CSHTML5.Internal.INTERNAL_SizeComparisonHelpers", {
        statics: {
            methods: {
                AreSizesEqual: function (size1, size2) {
                    if (isNaN(size1.Width)) {
                        if (!isNaN(size2.Width)) {
                            return false;
                        }
                    } else {
                        if (isNaN(size2.Width)) {
                            return false;
                        }

                        if (size1.Width !== size2.Width) {
                            return false;
                        }
                    }

                    if (isNaN(size1.Height)) {
                        if (!isNaN(size2.Height)) {
                            return false;
                        }
                    } else {
                        if (isNaN(size2.Height)) {
                            return false;
                        }

                        if (size1.Height !== size2.Height) {
                            return false;
                        }
                    }
                    return true;
                }
            }
        }
    });

    Bridge.define("CSHTML5.Internal.INTERNAL_TypeToDependencyProperties", {
        statics: {
            fields: {
                TypeToDependencyProperties: null
            },
            ctors: {
                init: function () {
                    this.TypeToDependencyProperties = new (System.Collections.Generic.Dictionary$2(System.Type,System.Collections.Generic.List$1(Windows.UI.Xaml.DependencyProperty))).ctor();
                }
            },
            methods: {
                Add: function (ownerType, dependencyProperty) {
                    CSHTML5.Internal.INTERNAL_TypeToDependencyProperties.GetCollectionForType(ownerType).add(dependencyProperty);
                },
                GetCollectionForType: function (ownerType) {
                    var dependencyProperties;
                    if (CSHTML5.Internal.INTERNAL_TypeToDependencyProperties.TypeToDependencyProperties.containsKey(ownerType)) {
                        dependencyProperties = CSHTML5.Internal.INTERNAL_TypeToDependencyProperties.TypeToDependencyProperties.getItem(ownerType);
                        if (dependencyProperties == null) {
                            dependencyProperties = new (System.Collections.Generic.List$1(Windows.UI.Xaml.DependencyProperty)).ctor();
                            CSHTML5.Internal.INTERNAL_TypeToDependencyProperties.TypeToDependencyProperties.setItem(ownerType, dependencyProperties);
                        }
                    } else {
                        dependencyProperties = new (System.Collections.Generic.List$1(Windows.UI.Xaml.DependencyProperty)).ctor();
                        CSHTML5.Internal.INTERNAL_TypeToDependencyProperties.TypeToDependencyProperties.add(ownerType, dependencyProperties);
                    }
                    return dependencyProperties;
                }
            }
        }
    });

    Bridge.define("CSHTML5.Internal.INTERNAL_TypeToDependencyPropertiesThatRequirePropertyChanged", {
        statics: {
            fields: {
                TypeToDependencyPropertiesThatRequirePropertyChanged: null
            },
            ctors: {
                init: function () {
                    this.TypeToDependencyPropertiesThatRequirePropertyChanged = new (System.Collections.Generic.Dictionary$2(System.Type,System.Collections.Generic.List$1(Windows.UI.Xaml.DependencyProperty))).ctor();
                }
            },
            methods: {
                Add: function (ownerType, dependencyProperty) {
                    CSHTML5.Internal.INTERNAL_TypeToDependencyPropertiesThatRequirePropertyChanged.GetCollectionForType(ownerType).add(dependencyProperty);
                },
                GetCollectionForType: function (ownerType) {
                    var dependencyProperties;
                    if (CSHTML5.Internal.INTERNAL_TypeToDependencyPropertiesThatRequirePropertyChanged.TypeToDependencyPropertiesThatRequirePropertyChanged.containsKey(ownerType)) {
                        dependencyProperties = CSHTML5.Internal.INTERNAL_TypeToDependencyPropertiesThatRequirePropertyChanged.TypeToDependencyPropertiesThatRequirePropertyChanged.getItem(ownerType);
                        if (dependencyProperties == null) {
                            dependencyProperties = new (System.Collections.Generic.List$1(Windows.UI.Xaml.DependencyProperty)).ctor();
                            CSHTML5.Internal.INTERNAL_TypeToDependencyPropertiesThatRequirePropertyChanged.TypeToDependencyPropertiesThatRequirePropertyChanged.setItem(ownerType, dependencyProperties);
                        }
                    } else {
                        dependencyProperties = new (System.Collections.Generic.List$1(Windows.UI.Xaml.DependencyProperty)).ctor();
                        CSHTML5.Internal.INTERNAL_TypeToDependencyPropertiesThatRequirePropertyChanged.TypeToDependencyPropertiesThatRequirePropertyChanged.add(ownerType, dependencyProperties);
                    }
                    return dependencyProperties;
                }
            }
        }
    });

    Bridge.define("CSHTML5.Internal.INTERNAL_TypeToStringsToDependencyProperties", {
        statics: {
            fields: {
                TypeToStringsToDependencyProperties: null
            },
            ctors: {
                init: function () {
                    this.TypeToStringsToDependencyProperties = new (System.Collections.Generic.Dictionary$2(System.Type,System.Collections.Generic.Dictionary$2(System.String,Windows.UI.Xaml.DependencyProperty))).ctor();
                }
            },
            methods: {
                GetPropertyInTypeOrItsBaseTypes: function (type, propertyName) {
                    var currentType = type;
                    while (currentType != null && (!CSHTML5.Internal.INTERNAL_TypeToStringsToDependencyProperties.TypeToStringsToDependencyProperties.containsKey(currentType) || (CSHTML5.Internal.INTERNAL_TypeToStringsToDependencyProperties.TypeToStringsToDependencyProperties.getItem(currentType) == null || !CSHTML5.Internal.INTERNAL_TypeToStringsToDependencyProperties.TypeToStringsToDependencyProperties.getItem(currentType).containsKey(propertyName)))) {
                        currentType = Bridge.Reflection.getBaseType(currentType);
                    }
                    if (currentType != null) {
                        return CSHTML5.Internal.INTERNAL_TypeToStringsToDependencyProperties.TypeToStringsToDependencyProperties.getItem(currentType).getItem(propertyName);
                    }
                    return null;
                },
                GetDictionaryForType: function (ownerType) {
                    var stringsToDependencyProperties;
                    if (CSHTML5.Internal.INTERNAL_TypeToStringsToDependencyProperties.TypeToStringsToDependencyProperties.containsKey(ownerType)) {
                        stringsToDependencyProperties = CSHTML5.Internal.INTERNAL_TypeToStringsToDependencyProperties.TypeToStringsToDependencyProperties.getItem(ownerType);
                        if (stringsToDependencyProperties == null) {
                            stringsToDependencyProperties = new (System.Collections.Generic.Dictionary$2(System.String,Windows.UI.Xaml.DependencyProperty)).ctor();
                            CSHTML5.Internal.INTERNAL_TypeToStringsToDependencyProperties.TypeToStringsToDependencyProperties.setItem(ownerType, stringsToDependencyProperties);
                        }
                    } else {
                        stringsToDependencyProperties = new (System.Collections.Generic.Dictionary$2(System.String,Windows.UI.Xaml.DependencyProperty)).ctor();
                        CSHTML5.Internal.INTERNAL_TypeToStringsToDependencyProperties.TypeToStringsToDependencyProperties.add(ownerType, stringsToDependencyProperties);
                    }
                    return stringsToDependencyProperties;
                }
            }
        }
    });

    Bridge.define("CSHTML5.Internal.INTERNAL_UriHelper", {
        statics: {
            methods: {
                /**
                 * Converts to an URI suitable to use in the HTML5 "src" property.
                 *
                 * @static
                 * @public
                 * @this CSHTML5.Internal.INTERNAL_UriHelper
                 * @memberof CSHTML5.Internal.INTERNAL_UriHelper
                 * @param   {string}                       uri                                    The URI with the C#/XAML syntax.
                 * @param   {Windows.UI.Xaml.UIElement}    elementThatARelativeUriIsRelativeTo    This is an optional parameter that is helpful
                 in case of relative URI's. In fact, when an URI is relative, we need to transform it into an
                 absolute URI in order to locate the resources. A relative URI is usually relative to the place
                 where the .XAML file is located. For example, when using an Image control, relative image paths
                 are relative to the location of the .XAML file that contains the Image control.
                 * @return  {string}                                                              The URI suitable to use in the HTML5 "src" property
                 */
                ConvertToHtml5Path: function (uri, elementThatARelativeUriIsRelativeTo) {
                    if (elementThatARelativeUriIsRelativeTo === void 0) { elementThatARelativeUriIsRelativeTo = null; }
                    if (uri == null) {
                        return null;
                    }
                    uri = uri.trim();
                    var originalStringLowercase = uri.toLowerCase();

                    if (System.String.startsWith(originalStringLowercase, "ms-appx:/")) {

                        var html5Path;

                        html5Path = System.String.replaceAll(System.String.replaceAll(System.String.replaceAll(uri, "://", ":/"), "://", ":/"), "://", ":/").substr(9);

                        html5Path = System.String.replaceAll(html5Path, String.fromCharCode(92), String.fromCharCode(47));

                        var assemblyNameIncludingSlashes = { };
                        var pathAfterAssemblyName = { };
                        if (!CSHTML5.Internal.INTERNAL_UriHelper.DoesPathContainAssemblyName("/" + (html5Path || ""), assemblyNameIncludingSlashes, pathAfterAssemblyName)) {
                            var startupAssemblyShortName = CSHTML5.Internal.StartupAssemblyInfo.StartupAssemblyShortName;
                            if (!System.String.isNullOrEmpty(startupAssemblyShortName)) {
                                html5Path = (startupAssemblyShortName || "") + "/" + (html5Path.toLowerCase() || "");
                            }
                        } else {
                            html5Path = (assemblyNameIncludingSlashes.v || "") + (pathAfterAssemblyName.v.toLowerCase() || "");
                        }

                        var outputResourcesPath = System.String.replaceAll(CSHTML5.Internal.StartupAssemblyInfo.OutputResourcesPath, String.fromCharCode(92), String.fromCharCode(47));
                        if (!System.String.endsWith(outputResourcesPath, "/") && !Bridge.referenceEquals(outputResourcesPath, "")) {
                            outputResourcesPath = (outputResourcesPath || "") + String.fromCharCode(47);
                        }

                        html5Path = (outputResourcesPath || "") + (html5Path || "");

                        html5Path = html5Path;

                        return html5Path;
                    } else if (System.String.startsWith(originalStringLowercase, "http://") || System.String.startsWith(originalStringLowercase, "https://")) {

                        return uri;
                    } else if (System.String.startsWith(originalStringLowercase, "pack://application:,,,/")) {

                        var html5Path1 = uri.substr(23);

                        var assemblyNameIncludingSlashes1 = { };
                        var pathAfterAssemblyName1 = { };
                        if (!CSHTML5.Internal.INTERNAL_UriHelper.DoesPathContainAssemblyName("/" + (html5Path1 || ""), assemblyNameIncludingSlashes1, pathAfterAssemblyName1)) {
                            var startupAssemblyShortName1 = CSHTML5.Internal.StartupAssemblyInfo.StartupAssemblyShortName;
                            if (!System.String.isNullOrEmpty(startupAssemblyShortName1)) {
                                html5Path1 = (startupAssemblyShortName1 || "") + "/" + (html5Path1.toLowerCase() || "");
                            }
                        } else {
                            html5Path1 = (assemblyNameIncludingSlashes1.v || "") + (pathAfterAssemblyName1.v.toLowerCase() || "");
                        }

                        var outputResourcesPath1 = System.String.replaceAll(CSHTML5.Internal.StartupAssemblyInfo.OutputResourcesPath, String.fromCharCode(92), String.fromCharCode(47));
                        if (!System.String.endsWith(outputResourcesPath1, "/") && !Bridge.referenceEquals(outputResourcesPath1, "")) {
                            outputResourcesPath1 = (outputResourcesPath1 || "") + String.fromCharCode(47);
                        }

                        html5Path1 = (outputResourcesPath1 || "") + (html5Path1 || "");

                        html5Path1 = html5Path1;

                        return html5Path1;
                    } else if (System.String.contains(originalStringLowercase,";component/")) {

                        var componentKeyword = ";component/";
                        var indexOfComponentKeyword = System.String.indexOf(originalStringLowercase, componentKeyword);
                        var assemblyName = uri.substr(0, indexOfComponentKeyword);
                        assemblyName = CSHTML5.Internal.INTERNAL_UriHelper.TrimStartChars(CSHTML5.Internal.INTERNAL_UriHelper.TrimStartChars(assemblyName, 47), 92);
                        assemblyName = CSHTML5.Internal.INTERNAL_UriHelper.TrimEndChars(CSHTML5.Internal.INTERNAL_UriHelper.TrimEndChars(assemblyName, 47), 92);
                        var relativeFolderAndFileName = uri.substr(((indexOfComponentKeyword + componentKeyword.length) | 0));
                        relativeFolderAndFileName = CSHTML5.Internal.INTERNAL_UriHelper.TrimStartChars(CSHTML5.Internal.INTERNAL_UriHelper.TrimStartChars(relativeFolderAndFileName, 47), 92);

                        var outputResourcesPath2 = System.String.replaceAll(CSHTML5.Internal.StartupAssemblyInfo.OutputResourcesPath, String.fromCharCode(92), String.fromCharCode(47));
                        if (!System.String.endsWith(outputResourcesPath2, "/") && !Bridge.referenceEquals(outputResourcesPath2, "")) {
                            outputResourcesPath2 = (outputResourcesPath2 || "") + String.fromCharCode(47);
                        }

                        var html5Path2 = (outputResourcesPath2 || "") + (assemblyName || "") + "/" + (relativeFolderAndFileName.toLowerCase() || "");

                        html5Path2 = html5Path2;

                        return html5Path2;
                    } else {

                        var xamlSourcePath = { v : null };
                        if (elementThatARelativeUriIsRelativeTo != null && CSHTML5.Internal.INTERNAL_VisualTreeManager.IsElementInVisualTree(elementThatARelativeUriIsRelativeTo) && CSHTML5.Internal.INTERNAL_UriHelper.TryGetLocationOfXamlFile(elementThatARelativeUriIsRelativeTo, xamlSourcePath) && System.String.contains(xamlSourcePath.v,"\\")) {

                            uri = System.String.replaceAll(uri, String.fromCharCode(92), String.fromCharCode(47));
                            xamlSourcePath.v = System.String.replaceAll(xamlSourcePath.v, String.fromCharCode(92), String.fromCharCode(47));

                            var xamlSourcePathWithoutFileName = xamlSourcePath.v.substr(0, xamlSourcePath.v.lastIndexOf(String.fromCharCode(47)));

                            var assemblyName1;
                            var folderWhereXamlFileIsLoated_PossibleEmpty;
                            if (System.Linq.Enumerable.from(xamlSourcePathWithoutFileName).contains(47)) {
                                assemblyName1 = xamlSourcePathWithoutFileName.substr(0, System.String.indexOf(xamlSourcePathWithoutFileName, String.fromCharCode(47)));
                                folderWhereXamlFileIsLoated_PossibleEmpty = xamlSourcePathWithoutFileName.substr(((assemblyName1.length + 1) | 0));
                            } else {
                                assemblyName1 = xamlSourcePathWithoutFileName;
                                folderWhereXamlFileIsLoated_PossibleEmpty = "";
                            }

                            var absolutePath;

                            if (System.String.startsWith(uri, "/")) {

                                absolutePath = "ms-appx:/" + (assemblyName1 || "") + (uri || "");
                            } else {

                                while (System.String.startsWith(uri, "../")) {
                                    uri = uri.substr(3);

                                    if (System.Linq.Enumerable.from(folderWhereXamlFileIsLoated_PossibleEmpty).contains(47)) {
                                        folderWhereXamlFileIsLoated_PossibleEmpty = folderWhereXamlFileIsLoated_PossibleEmpty.substr(0, folderWhereXamlFileIsLoated_PossibleEmpty.lastIndexOf(String.fromCharCode(47)));
                                    } else {
                                        folderWhereXamlFileIsLoated_PossibleEmpty = "";
                                    }
                                }

                                absolutePath = "ms-appx:/" + (assemblyName1 || "") + ((!Bridge.referenceEquals(folderWhereXamlFileIsLoated_PossibleEmpty, "") ? "/" + (folderWhereXamlFileIsLoated_PossibleEmpty || "") : "") || "") + ((!System.String.startsWith(uri, "/") ? "/" : "") || "") + (uri || "");
                            }

                            var result = CSHTML5.Internal.INTERNAL_UriHelper.ConvertToHtml5Path(absolutePath, null);
                            return result;
                        } else {
                            throw new System.Exception("Unless you specify the URI in XAML, the current version only supports absolute URIs that start with http:// or https:// or that are in the form of \"ms-appx:///AssemblyName/Folder/FileName\" or \"/AssemblyName;component/Folder/FileName\"" + ((!System.String.contains(originalStringLowercase,":") ? "    - Try adding  ms-appx:///YourAssemblyName/  to the beginning of your path." : "") || ""));
                        }
                    }
                },
                TryGetLocationOfXamlFile: function (element, xamlSourcePath) {
                    var current = element;
                    while (current != null) {
                        if (!System.String.isNullOrEmpty(current.XamlSourcePath)) {
                            xamlSourcePath.v = current.XamlSourcePath;
                            return true;
                        } else {
                            current = Bridge.as(current.INTERNAL_VisualParent, Windows.UI.Xaml.UIElement);
                        }
                    }
                    xamlSourcePath.v = null;
                    return false;
                },
                TrimStartChars: function (value, charToRemove) {
                    var removeLength = 0;
                    for (var i = 0; i < value.length; i = (i + 1) | 0) {
                        var $let = value.charCodeAt(i);
                        if ($let === charToRemove) {
                            removeLength = (removeLength + 1) | 0;
                        } else {
                            break;
                        }
                    }
                    if (removeLength > 0) {
                        return value.substr(removeLength, ((value.length - removeLength) | 0));
                    }
                    return value;
                },
                TrimEndChars: function (value, charToRemove) {
                    var removeLength = 0;
                    for (var i = (value.length - 1) | 0; i >= 0; i = (i - 1) | 0) {
                        var $let = value.charCodeAt(i);
                        if ($let === charToRemove) {
                            removeLength = (removeLength + 1) | 0;
                        } else {
                            break;
                        }
                    }
                    if (removeLength > 0) {
                        return value.substr(0, ((value.length - removeLength) | 0));
                    }
                    return value;
                },
                DoesPathContainAssemblyName: function (path, assemblyNameIncludingSlashes, pathAfterAssemblyName) {
                    var $t;
                    var pathLowercase = path.toLowerCase();
                    var listOfAssemblies = CSHTML5.Internal.INTERNAL_UriHelper.GetListOfLoadedAssemblies();
                    $t = Bridge.getEnumerator(listOfAssemblies);
                    try {
                        while ($t.moveNext()) {
                            var assemblyShortName = $t.Current;
                            if (System.String.contains(pathLowercase,"/" + (assemblyShortName.toLowerCase() || "") + "/")) {
                                assemblyNameIncludingSlashes.v = "/" + (assemblyShortName || "") + "/";
                                pathAfterAssemblyName.v = pathLowercase.substr(assemblyNameIncludingSlashes.v.length);
                                return true;
                            }
                        }
                    } finally {
                        if (Bridge.is($t, System.IDisposable)) {
                            $t.System$IDisposable$Dispose();
                        }
                    }
                    assemblyNameIncludingSlashes.v = null;
                    pathAfterAssemblyName.v = null;
                    return false;
                },
                GetListOfLoadedAssemblies: function () {
                    var listOfAssemblies;
                    if (CSHTML5.Interop.IsRunningInTheSimulator) {
                        listOfAssemblies = System.Linq.Enumerable.from(System.Linq.Enumerable.from(System.AppDomain.getAssemblies()).select(function (a) {
                                    return INTERNAL_BridgeWorkarounds.GetAssemblyNameWithoutCallingGetNameMethod(a);
                                })).ToArray();
                    } else {
                        listOfAssemblies = System.Linq.Enumerable.from(System.Linq.Enumerable.from(System.AppDomain.getAssemblies()).select(function (a) {
                                    return CSHTML5.Internal.INTERNAL_UriHelper.GetAssemblyName(a.name);
                                })).ToArray();
                    }
                    return listOfAssemblies;
                },
                GetAssemblyName: function (fullName) {

                    var tmpIndex = System.String.indexOf(fullName, String.fromCharCode(44));

                    if (tmpIndex === -1) {
                        return fullName;
                    } else {
                        return fullName.substr(0, tmpIndex);
                    }
                },
                GetJavaScriptCallingAssembly: function () {
                    return null;

                }
            }
        }
    });

    /**
     * The goal of this class is to allow elements to subscribe
     to the Visibility_Changed event of any of its ancestors.
     It is useful for example to work around the fact that
     drawing on a hidden html canvas (in the XAML "Path"
     control for example) is not possible, so if a html
     canvas is hidden, we subscribe to the Visibility_Changed
     event of its ancestors, so as to refresh it when it becomes
     visible.
     *
     * @static
     * @abstract
     * @class CSHTML5.Internal.INTERNAL_VisibilityChangedNotifier
     */
    Bridge.define("CSHTML5.Internal.INTERNAL_VisibilityChangedNotifier", {
        statics: {
            fields: {
                ElementsToListeners: null,
                ElementsToListened: null,
                ListenersToCallbacks: null
            },
            ctors: {
                init: function () {
                    this.ElementsToListeners = new (System.Collections.Generic.Dictionary$2(Windows.UI.Xaml.DependencyObject,System.Collections.Generic.HashSet2$1(Windows.UI.Xaml.DependencyObject))).ctor();
                    this.ElementsToListened = new (System.Collections.Generic.Dictionary$2(Windows.UI.Xaml.DependencyObject,System.Collections.Generic.HashSet2$1(Windows.UI.Xaml.DependencyObject))).ctor();
                    this.ListenersToCallbacks = new (System.Collections.Generic.Dictionary$2(Windows.UI.Xaml.DependencyObject,Function)).ctor();
                }
            },
            methods: {
                StartListeningToAncestorsVisibilityChanged: function (listeningElement, onAncestorVisibilityChanged) {
                    var $t;
                    var ancestorsAndSelf = new (System.Collections.Generic.List$1(Windows.UI.Xaml.DependencyObject)).ctor();
                    var element = listeningElement;
                    while (element != null) {
                        ancestorsAndSelf.add(element);
                        element = Bridge.cast(element, Windows.UI.Xaml.UIElement).INTERNAL_VisualParent;
                    }

                    $t = Bridge.getEnumerator(ancestorsAndSelf);
                    try {
                        while ($t.moveNext()) {
                            var ancestor = $t.Current;
                            CSHTML5.Internal.INTERNAL_VisibilityChangedNotifier.AddToDictionaryIfNotAlreadyThere(CSHTML5.Internal.INTERNAL_VisibilityChangedNotifier.ElementsToListeners, ancestor, listeningElement);
                            CSHTML5.Internal.INTERNAL_VisibilityChangedNotifier.AddToDictionaryIfNotAlreadyThere(CSHTML5.Internal.INTERNAL_VisibilityChangedNotifier.ElementsToListened, listeningElement, ancestor);
                        }
                    } finally {
                        if (Bridge.is($t, System.IDisposable)) {
                            $t.System$IDisposable$Dispose();
                        }
                    }

                    CSHTML5.Internal.INTERNAL_VisibilityChangedNotifier.ListenersToCallbacks.setItem(listeningElement, onAncestorVisibilityChanged);
                },
                StopListeningToAncestorsVisibilityChanged: function (listeningElement) {
                    var $t;
                    var listenedElements = null;
                    if (CSHTML5.Internal.INTERNAL_VisibilityChangedNotifier.ElementsToListened.containsKey(listeningElement)) {
                        listenedElements = CSHTML5.Internal.INTERNAL_VisibilityChangedNotifier.ElementsToListened.getItem(listeningElement);
                    }

                    if (listenedElements != null) {
                        $t = Bridge.getEnumerator(listenedElements);
                        try {
                            while ($t.moveNext()) {
                                var listenedElement = $t.Current;
                                CSHTML5.Internal.INTERNAL_VisibilityChangedNotifier.RemoveFromDictionaryIfFound(CSHTML5.Internal.INTERNAL_VisibilityChangedNotifier.ElementsToListeners, listenedElement, listeningElement);
                            }
                        } finally {
                            if (Bridge.is($t, System.IDisposable)) {
                                $t.System$IDisposable$Dispose();
                            }
                        }
                    }

                    if (listenedElements != null) {
                        CSHTML5.Internal.INTERNAL_VisibilityChangedNotifier.ElementsToListened.remove(listeningElement);
                    }

                    if (CSHTML5.Internal.INTERNAL_VisibilityChangedNotifier.ListenersToCallbacks.containsKey(listeningElement)) {
                        CSHTML5.Internal.INTERNAL_VisibilityChangedNotifier.ListenersToCallbacks.remove(listeningElement);
                    }
                },
                NotifyListenersThatVisibilityHasChanged: function (elementThatHasChanged) {
                    var $t;
                    if (CSHTML5.Internal.INTERNAL_VisibilityChangedNotifier.ElementsToListeners.containsKey(elementThatHasChanged)) {
                        var listeners = CSHTML5.Internal.INTERNAL_VisibilityChangedNotifier.ElementsToListeners.getItem(elementThatHasChanged);

                        var listenersCopy = new (System.Collections.Generic.List$1(Windows.UI.Xaml.DependencyObject)).$ctor1(listeners);

                        $t = Bridge.getEnumerator(listenersCopy);
                        try {
                            while ($t.moveNext()) {
                                var listener = $t.Current;
                                if (CSHTML5.Internal.INTERNAL_VisibilityChangedNotifier.ListenersToCallbacks.containsKey(listener)) {
                                    var onAncestorVisibilityChanged = CSHTML5.Internal.INTERNAL_VisibilityChangedNotifier.ListenersToCallbacks.getItem(listener);
                                    if (!Bridge.staticEquals(onAncestorVisibilityChanged, null)) {
                                        onAncestorVisibilityChanged();
                                    }
                                }
                            }
                        } finally {
                            if (Bridge.is($t, System.IDisposable)) {
                                $t.System$IDisposable$Dispose();
                            }
                        }
                    }
                },
                AddToDictionaryIfNotAlreadyThere: function (dictionary, key, value) {
                    var list;
                    if (dictionary.containsKey(key)) {
                        list = dictionary.getItem(key);
                    } else {
                        list = new (System.Collections.Generic.HashSet2$1(Windows.UI.Xaml.DependencyObject)).ctor();
                        dictionary.setItem(key, list);
                    }
                    if (!list.contains(value)) {
                        list.add(value);
                    }
                },
                RemoveFromDictionaryIfFound: function (dictionary, key, value) {
                    if (dictionary.containsKey(key)) {
                        var list = dictionary.getItem(key);
                        if (list.contains(value)) {
                            list.remove(value);
                        }
                    }
                }
            }
        }
    });

    Bridge.define("CSHTML5.Internal.INTERNAL_VisualChildInformation", {
        fields: {
            INTERNAL_UIElement: null,
            INTERNAL_OptionalChildWrapper_OuterDomElement: null,
            INTERNAL_OptionalChildWrapper_ChildWrapperInnerDomElement: null
        }
    });

    Bridge.define("CSHTML5.Internal.INTERNAL_VisualTreeManager", {
        statics: {
            fields: {
                EnablePerformanceLogging: false,
                EnableOptimizationWhereCollapsedControlsAreNotRendered: false,
                EnableOptimizationWhereCollapsedControlsAreNotLoaded: false,
                EnableOptimizationWhereCollapsedControlsAreLoadedLast: false
            },
            ctors: {
                init: function () {
                    this.EnableOptimizationWhereCollapsedControlsAreNotRendered = true;
                    this.EnableOptimizationWhereCollapsedControlsAreNotLoaded = false;
                    this.EnableOptimizationWhereCollapsedControlsAreLoadedLast = false;
                }
            },
            methods: {
                DetachVisualChildIfNotNull: function (child, parent) {
                    if (child != null && CSHTML5.Internal.INTERNAL_VisualTreeManager.IsElementInVisualTree(child)) {
                        if (parent.INTERNAL_VisualChildrenInformation != null && parent.INTERNAL_VisualChildrenInformation.containsKey(child)) {
                            var stringForDebugging = !true ? "Removing " + (Bridge.getTypeName(Bridge.getType(child)) || "") : null;
                            CSHTML5.Internal.INTERNAL_HtmlDomManager.RemoveFromDom(child.INTERNAL_AdditionalOutsideDivForMargins, stringForDebugging);

                            var optionalChildWrapper_OuterDomElement = parent.INTERNAL_VisualChildrenInformation.getItem(child).INTERNAL_OptionalChildWrapper_OuterDomElement;
                            if (optionalChildWrapper_OuterDomElement != null) {
                                CSHTML5.Internal.INTERNAL_HtmlDomManager.RemoveFromDom(optionalChildWrapper_OuterDomElement);
                            }

                            parent.INTERNAL_VisualChildrenInformation.remove(child);

                            child.INTERNAL_SpanParentCell = null;

                            CSHTML5.Internal.INTERNAL_VisualTreeManager.DetachVisualChidrenRecursively(child);


                            CSHTML5.Internal.INTERNAL_WorkaroundIE11IssuesWithScrollViewerInsideGrid.RefreshLayoutIfIE();
                        } else {
                            throw new System.Exception(System.String.format("Cannot detach the element '{0}' because it is not a child of the element '{1}'.", Bridge.getTypeName(Bridge.getType(child)), (parent != null ? Bridge.getTypeName(Bridge.getType(parent)) : "null")));
                        }
                    }
                },
                DetachVisualChidrenRecursively: function (element) {
                    var $t;


                    if (element._pointerExitedEventManager != null) {
                        if (element.INTERNAL_isPointerInside) {

                            var eventArgs = new Windows.UI.Xaml.Input.PointerRoutedEventArgs();
                            element.OnPointerExited(eventArgs);
                            element.OnPointerExited_ForHandledEventsToo(eventArgs);
                        }
                        element.isAlreadySubscribedToMouseEnterAndLeave = false;
                        element.INTERNAL_isPointerInside = false;
                    }

                    element.INTERNAL_OnDetachedFromVisualTree();

                    element.INTERNAL_DetachFromDomEvents();

                    element._isLoaded = false;
                    if (Bridge.is(element, Windows.UI.Xaml.FrameworkElement)) {
                        Bridge.cast(element, Windows.UI.Xaml.FrameworkElement).INTERNAL_RaiseUnloadedEvent();
                    }

                    if (element.INTERNAL_VisualChildrenInformation != null) {
                        $t = Bridge.getEnumerator(INTERNAL_BridgeWorkarounds.GetDictionaryValues_SimulatorCompatible(Windows.UI.Xaml.UIElement, CSHTML5.Internal.INTERNAL_VisualChildInformation, element.INTERNAL_VisualChildrenInformation), CSHTML5.Internal.INTERNAL_VisualChildInformation);
                        try {
                            while ($t.moveNext()) {
                                var childInfo = $t.Current;
                                CSHTML5.Internal.INTERNAL_VisualTreeManager.DetachVisualChidrenRecursively(childInfo.INTERNAL_UIElement);
                            }
                        } finally {
                            if (Bridge.is($t, System.IDisposable)) {
                                $t.System$IDisposable$Dispose();
                            }
                        }
                    }

                    var oldParent = element.INTERNAL_VisualParent;

                    element.INTERNAL_OuterDomElement = null;
                    element.INTERNAL_InnerDomElement = null;
                    element.INTERNAL_VisualParent = null;
                    element.INTERNAL_VisualChildrenInformation = null;
                    element.INTERNAL_AdditionalOutsideDivForMargins = null;
                    element.INTERNAL_DeferredRenderingWhenControlBecomesVisible = null;
                    element.INTERNAL_DeferredLoadingWhenControlBecomesVisible = null;

                    element.ResetInheritedProperties();

                    if (oldParent != null) {
                        Windows.UI.Xaml.UIElement.SynchronizeForceInheritProperties(element, oldParent);
                    }
                },
                MoveVisualChildInSameParent$1: function (child, parent, newIndex, oldIndex) {
                    if (oldIndex < 0) {
                        CSHTML5.Internal.INTERNAL_VisualTreeManager.MoveVisualChildInSameParent(child, parent, newIndex);
                        return;
                    }

                    if (parent.INTERNAL_VisualChildrenInformation.containsKey(child)) {
                        var visualChildInformation = parent.INTERNAL_VisualChildrenInformation.getItem(child);
                        var domElementToMove = visualChildInformation.INTERNAL_OptionalChildWrapper_OuterDomElement || child.INTERNAL_OuterDomElement;

                        if (domElementToMove != null) {
                            var domElementWhereToPlaceChildStuff = (parent.GetDomElementWhereToPlaceChild(child) || parent.INTERNAL_InnerDomElement);

                            var movedChild = domElementWhereToPlaceChildStuff.children[Bridge.box(oldIndex, System.Int32)];


                            if (!System.Convert.toBoolean(movedChild == domElementToMove)) {
                                throw new System.InvalidOperationException.$ctor1(System.String.format("index '{0}' does match index of the element about to be moved.", [Bridge.box(oldIndex, System.Int32)]));
                            }

                            var nextSibling = domElementWhereToPlaceChildStuff.children[Bridge.box(newIndex, System.Int32)];

                            if (nextSibling != null) {
                                domElementWhereToPlaceChildStuff.insertBefore(domElementToMove, nextSibling);
                            } else {
                                domElementWhereToPlaceChildStuff.appendChild(domElementToMove);
                            }
                        }
                    }
                },
                MoveVisualChildInSameParent: function (child, parent, index) {
                    if (parent.INTERNAL_VisualChildrenInformation.containsKey(child)) {
                        var visualChildInformation = parent.INTERNAL_VisualChildrenInformation.getItem(child);
                        var domElementToMove = visualChildInformation.INTERNAL_OptionalChildWrapper_OuterDomElement;
                        if (domElementToMove == null) {
                            domElementToMove = child.INTERNAL_OuterDomElement;
                        }

                        if (domElementToMove != null) {
                            var domElementWhereToPlaceChildStuff = (parent.GetDomElementWhereToPlaceChild(child) || parent.INTERNAL_InnerDomElement);

var actualIndex =                             Bridge.box(index, System.Int32);
var i = 0;
while (i < actualIndex && domElementWhereToPlaceChildStuff.children[i]!=domElementToMove) { 
    ++i;
}
if(i < actualIndex) {
    ++actualIndex; //to compensate the fact that the item that will be moved was before the next sibling
}
var nextSibling = domElementWhereToPlaceChildStuff.children[Bridge.box(index, System.Int32)];
if(nextSibling != undefined) {
    domElementWhereToPlaceChildStuff.insertBefore(domElementToMove, nextSibling);
} else {
    domElementWhereToPlaceChildStuff.appendChild(domElementToMove);
};
                        }
                    }
                },
                AttachVisualChildIfNotAlreadyAttached: function (child, parent, index) {
                    if (index === void 0) { index = -1; }
                    if (child != null && CSHTML5.Internal.INTERNAL_VisualTreeManager.IsElementInVisualTree(parent)) {

                        if (child.INTERNAL_VisualParent == null) {
                            var label = "";
                            if (CSHTML5.Internal.INTERNAL_VisualTreeManager.EnablePerformanceLogging) {
                                label = "Attach - " + (Bridge.Reflection.getTypeName(Bridge.getType(child)) || "") + " - " + (Bridge.toString(Bridge.getHashCode(child)) || "");
                                console.time(label);
                            }

                            CSHTML5.Internal.INTERNAL_VisualTreeManager.AttachVisualChild_Private(child, parent, index);

                            if (CSHTML5.Internal.INTERNAL_VisualTreeManager.EnablePerformanceLogging) {
                                console.timeEnd(label);
                            }

                        } else if (!Bridge.referenceEquals(child.INTERNAL_VisualParent, parent)) {
                            throw new System.InvalidOperationException.$ctor1("The element already has a parent. An element cannot appear in multiple locations in the Visual Tree. Remove the element from the Visual Tree before adding it elsewhere.");
                        } else {
                            return;
                        }

                        CSHTML5.Internal.INTERNAL_WorkaroundIE11IssuesWithScrollViewerInsideGrid.RefreshLayoutIfIE();
                    }
                },
                AttachVisualChild_Private: function (child, parent, index) {




                    var domElementWhereToPlaceChildStuff = (parent.GetDomElementWhereToPlaceChild(child) || parent.INTERNAL_InnerDomElement);

                    var innerDivOfWrapperForChild = { };
                    var wrapperForChild = parent.CreateDomChildWrapper(domElementWhereToPlaceChildStuff, innerDivOfWrapperForChild, index);
                    var comparison1 = (wrapperForChild == null);
                    var comparison2 = (innerDivOfWrapperForChild.v == null);
                    var doesParentRequireToCreateAWrapperForEachChild = (!comparison1 && !comparison2);



                    if (CSHTML5.Internal.INTERNAL_VisualTreeManager.EnableOptimizationWhereCollapsedControlsAreNotLoaded && child.Visibility === Windows.UI.Xaml.Visibility.Collapsed) {
                        child.INTERNAL_DeferredLoadingWhenControlBecomesVisible = function () {
                            CSHTML5.Internal.INTERNAL_VisualTreeManager.AttachVisualChild_Private_MainSteps(child, parent, doesParentRequireToCreateAWrapperForEachChild, innerDivOfWrapperForChild.v, domElementWhereToPlaceChildStuff, wrapperForChild);
                        };
                    } else if (CSHTML5.Internal.INTERNAL_VisualTreeManager.EnableOptimizationWhereCollapsedControlsAreLoadedLast && child.Visibility === Windows.UI.Xaml.Visibility.Collapsed) {
                        child.INTERNAL_DeferredLoadingWhenControlBecomesVisible = function () {
                            CSHTML5.Internal.INTERNAL_VisualTreeManager.AttachVisualChild_Private_MainSteps(child, parent, doesParentRequireToCreateAWrapperForEachChild, innerDivOfWrapperForChild.v, domElementWhereToPlaceChildStuff, wrapperForChild);
                        };
                        CSHTML5.Internal.INTERNAL_DispatcherHelpers.QueueAction(function () {
                            var $step = 0,
                                $task1, 
                                $jumpFromFinally, 
                                deferredLoadingWhenControlBecomesVisible, 
                                $asyncBody = Bridge.fn.bind(this, function () {
                                    for (;;) {
                                        $step = System.Array.min([0,1], $step);
                                        switch ($step) {
                                            case 0: {
                                                $task1 = System.Threading.Tasks.Task.delay(100);
                                                $step = 1;
                                                if ($task1.isCompleted()) {
                                                    continue;
                                                }
                                                $task1.continue($asyncBody);
                                                return;
                                            }
                                            case 1: {
                                                $task1.getAwaitedResult();
                                                deferredLoadingWhenControlBecomesVisible = child.INTERNAL_DeferredLoadingWhenControlBecomesVisible;
                                                if (!Bridge.staticEquals(deferredLoadingWhenControlBecomesVisible, null)) {
                                                    child.INTERNAL_DeferredLoadingWhenControlBecomesVisible = null;
                                                    deferredLoadingWhenControlBecomesVisible();
                                                }
                                                return;
                                            }
                                            default: {
                                                return;
                                            }
                                        }
                                    }
                                }, arguments);

                            $asyncBody();
                        });
                    } else {
                        CSHTML5.Internal.INTERNAL_VisualTreeManager.AttachVisualChild_Private_MainSteps(child, parent, doesParentRequireToCreateAWrapperForEachChild, innerDivOfWrapperForChild.v, domElementWhereToPlaceChildStuff, wrapperForChild);
                    }
                },
                AttachVisualChild_Private_MainSteps: function (child, parent, doesParentRequireToCreateAWrapperForEachChild, innerDivOfWrapperForChild, domElementWhereToPlaceChildStuff, wrapperForChild) {
                    var $t, $t1;



                    var additionalOutsideDivForMargins = null;
                    var margin = Bridge.cast(child, Windows.UI.Xaml.FrameworkElement).Margin.$clone();
                    var containsNegativeMargins = (margin.Left < 0.0 || margin.Top < 0.0 || margin.Right < 0.0 || margin.Bottom < 0.0);
                    var isADivForMarginsNeeded = !(Bridge.is(parent, Windows.UI.Xaml.Controls.Canvas)) && !(Bridge.is(child, Windows.UI.Xaml.Documents.Inline));
                    if (isADivForMarginsNeeded) {
                        var whereToPlaceDivForMargins = (doesParentRequireToCreateAWrapperForEachChild ? innerDivOfWrapperForChild : domElementWhereToPlaceChildStuff);

                        additionalOutsideDivForMargins = CSHTML5.Internal.INTERNAL_HtmlDomManager.CreateDomElementAndAppendIt("div", whereToPlaceDivForMargins, parent);

                        var style = CSHTML5.Internal.INTERNAL_HtmlDomManager.GetDomElementStyleForModification(additionalOutsideDivForMargins);
                        style.boxSizing = "border-box";
                        if (Bridge.is(child, Windows.UI.Xaml.FrameworkElement) && (Bridge.cast(child, Windows.UI.Xaml.FrameworkElement).HorizontalAlignment === Windows.UI.Xaml.HorizontalAlignment.Stretch && isNaN(Bridge.cast(child, Windows.UI.Xaml.FrameworkElement).Width))) {
                            if (!containsNegativeMargins) {
                                style.width = "100%";
                            }
                        }
                        if (Bridge.is(child, Windows.UI.Xaml.FrameworkElement) && (Bridge.cast(child, Windows.UI.Xaml.FrameworkElement).VerticalAlignment === Windows.UI.Xaml.VerticalAlignment.Stretch && isNaN(Bridge.cast(child, Windows.UI.Xaml.FrameworkElement).Height))) {
                            style.height = "100%";
                        }
                    }




                    var whereToPlaceTheChild = (isADivForMarginsNeeded ? additionalOutsideDivForMargins : (doesParentRequireToCreateAWrapperForEachChild ? innerDivOfWrapperForChild : domElementWhereToPlaceChildStuff));

                    child.INTERNAL_VisualParent = parent;

                    child.INTERNAL_ParentWindow = parent.INTERNAL_ParentWindow;

                    var domElementWhereToPlaceGrandChildren = { v : null };
                    var outerDomElement;
                    var isChildAControl = Bridge.is(child, Windows.UI.Xaml.Controls.Control);
                    if (child.INTERNAL_HtmlRepresentation == null) {
                        var hasTemplate = isChildAControl && Bridge.cast(child, Windows.UI.Xaml.Controls.Control).HasTemplate;
                        if (hasTemplate) {
                            outerDomElement = Bridge.cast(child, Windows.UI.Xaml.Controls.Control).CreateDomElementForControlTemplate(whereToPlaceTheChild, domElementWhereToPlaceGrandChildren);
                        } else {
                            outerDomElement = child.CreateDomElement(whereToPlaceTheChild, domElementWhereToPlaceGrandChildren);
                        }
                    } else {
                        outerDomElement = CSHTML5.Internal.INTERNAL_HtmlDomManager.CreateDomFromStringAndAppendIt(child.INTERNAL_HtmlRepresentation, whereToPlaceTheChild, child);
                    }

                    if (Bridge.is(child, Windows.UI.Xaml.FrameworkElement)) {
                        Windows.UI.Xaml.FrameworkElement.INTERNAL_InitializeOuterDomElementWidthAndHeight(Bridge.cast(child, Windows.UI.Xaml.FrameworkElement), outerDomElement);
                    }

                    child.INTERNAL_UpdateDomStructureIfNecessary();

                    CSHTML5.Internal.INTERNAL_HtmlDomManager.SetDomElementAttribute(outerDomElement, "class", Bridge.getTypeName(Bridge.getType(child)));




                    child.INTERNAL_OuterDomElement = outerDomElement;
                    child.INTERNAL_InnerDomElement = domElementWhereToPlaceGrandChildren.v;
                    child.INTERNAL_AdditionalOutsideDivForMargins = ($t = additionalOutsideDivForMargins, $t != null ? $t : outerDomElement);
                    child.INTERNAL_InnerDivOfTheChildWrapperOfTheParentIfAny = doesParentRequireToCreateAWrapperForEachChild ? innerDivOfWrapperForChild : null;

                    if (parent.INTERNAL_VisualChildrenInformation == null) {
                        parent.INTERNAL_VisualChildrenInformation = new (System.Collections.Generic.Dictionary$2(Windows.UI.Xaml.UIElement,CSHTML5.Internal.INTERNAL_VisualChildInformation)).ctor();
                    }
                    parent.INTERNAL_VisualChildrenInformation.add(child, ($t1 = new CSHTML5.Internal.INTERNAL_VisualChildInformation(), $t1.INTERNAL_UIElement = child, $t1.INTERNAL_OptionalChildWrapper_OuterDomElement = wrapperForChild, $t1.INTERNAL_OptionalChildWrapper_ChildWrapperInnerDomElement = innerDivOfWrapperForChild, $t1));




                    if (Bridge.is(parent, Windows.UI.Xaml.Controls.Canvas)) {
                        CSHTML5.Internal.INTERNAL_HtmlDomManager.GetDomElementStyleForModification(outerDomElement).position = "absolute";
                    }

                    Windows.UI.Xaml.UIElement.SynchronizeForceInheritProperties(child, parent);

                    if (Bridge.is(parent, Windows.UI.Xaml.Controls.Grid) && Bridge.is(child, Windows.UI.Xaml.FrameworkElement)) {
                        var frameworkElement = Bridge.cast(child, Windows.UI.Xaml.FrameworkElement);
                        Windows.UI.Xaml.UIElement.INTERNAL_UpdateCssPointerEventsPropertyBasedOnIsHitTestVisibleAndIsEnabled(frameworkElement, frameworkElement.IsHitTestVisible, frameworkElement.IsEnabled);
                    }

                    child.INTERNAL_RenderTransformOriginHasBeenApplied = false;




                    child.INTERNAL_AttachToDomEvents();



                    $t1 = Bridge.getEnumerator(INTERNAL_BridgeWorkarounds.GetDictionaryKeys_SimulatorCompatible(Windows.UI.Xaml.DependencyProperty, CSHTML5.Internal.INTERNAL_PropertyStorage, parent.INTERNAL_AllInheritedProperties), Windows.UI.Xaml.DependencyProperty);
                    try {
                        while ($t1.moveNext()) {
                            var dependencyProperty = $t1.Current;
                            var recursively = false;
                            var storage = parent.INTERNAL_AllInheritedProperties.getItem(dependencyProperty);
                            child.SetInheritedValue(dependencyProperty, CSHTML5.Internal.INTERNAL_PropertyStore.GetEffectiveValue(storage), recursively);
                        }
                    } finally {
                        if (Bridge.is($t1, System.IDisposable)) {
                            $t1.System$IDisposable$Dispose();
                        }
                    }



                    child._isLoaded = true;

                    child.INTERNAL_OnAttachedToVisualTree();


                    var enableDeferredRenderingOfCollapsedControls = CSHTML5.Internal.INTERNAL_VisualTreeManager.EnableOptimizationWhereCollapsedControlsAreNotRendered || CSHTML5.Internal.INTERNAL_VisualTreeManager.EnableOptimizationWhereCollapsedControlsAreLoadedLast || CSHTML5.Internal.INTERNAL_VisualTreeManager.EnableOptimizationWhereCollapsedControlsAreNotLoaded;

                    if (enableDeferredRenderingOfCollapsedControls && !child.IsVisible) {
                        child.INTERNAL_DeferredRenderingWhenControlBecomesVisible = function () {
                            CSHTML5.Internal.INTERNAL_VisualTreeManager.RenderElementsAndRaiseChangedEventOnAllDependencyProperties(child);
                        };
                    } else {
                        CSHTML5.Internal.INTERNAL_VisualTreeManager.RenderElementsAndRaiseChangedEventOnAllDependencyProperties(child);
                    }



                    child.INTERNAL_UpdateBindingsSource();




                    if (isChildAControl) {
                        if (!(Bridge.is(child, Windows.UI.Xaml.Controls.ContentPresenter)) && !(Bridge.is(child, Windows.UI.Xaml.Controls.TextBlock))) {
                            Windows.UI.Xaml.Controls.Control.TabIndexProperty_MethodToUpdateDom(child, Bridge.box(Bridge.cast(child, Windows.UI.Xaml.Controls.Control).TabIndex, System.Int32));
                        }
                    }


                    var childVisibility = child.Visibility;
                    if (childVisibility === Windows.UI.Xaml.Visibility.Collapsed) {
                        Windows.UI.Xaml.UIElement.INTERNAL_ApplyVisibility(child, childVisibility);
                    }


                    if (Bridge.is(child, Windows.UI.Xaml.FrameworkElement)) {
                        Bridge.cast(child, Windows.UI.Xaml.FrameworkElement).INTERNAL_SizeChangedWhenAttachedToVisualTree();
                    }



                    if (Bridge.is(child, Windows.UI.Xaml.FrameworkElement)) {
                        Bridge.cast(child, Windows.UI.Xaml.FrameworkElement).INTERNAL_RaiseLoadedEvent();
                    }

                    child.StartManagingPointerPositionForPointerExitedEventIfNeeded();


                },
                IsElementInVisualTree: function (child) {
                    return (child.INTERNAL_VisualParent != null || Bridge.is(child, Windows.UI.Xaml.Window) || Bridge.is(child, Windows.UI.Xaml.Controls.Primitives.PopupRoot));
                },
                RenderElementsAndRaiseChangedEventOnAllDependencyProperties: function (dependencyObject) {
                    var $t;


                    var list = System.Linq.Enumerable.from(System.Linq.Enumerable.from(dependencyObject.INTERNAL_PropertyStorageDictionary).where(function (s) {
                                return s.value.BaseValueSourceInternal > CSHTML5.Internal.BaseValueSourceInternal.Default;
                            })).toList(System.Collections.Generic.KeyValuePair$2(Windows.UI.Xaml.DependencyProperty,CSHTML5.Internal.INTERNAL_PropertyStorage));

                    $t = Bridge.getEnumerator(list);
                    try {
                        while ($t.moveNext()) {
                            var propertiesAndTheirStorage = $t.Current;
                            var property = propertiesAndTheirStorage.key;


                            var propertyMetadata = property.GetTypeMetaData(Bridge.getType(dependencyObject));

                            if (propertyMetadata != null) {
                                var storage = propertiesAndTheirStorage.value;
                                var value = null;
                                var valueWasRetrieved = false;


                                if (!Bridge.staticEquals(propertyMetadata.GetCSSEquivalent, null) || !Bridge.staticEquals(propertyMetadata.GetCSSEquivalents, null)) {
                                    if (!valueWasRetrieved) {
                                        value = CSHTML5.Internal.INTERNAL_PropertyStore.GetEffectiveValue(storage);
                                        valueWasRetrieved = true;
                                    }

                                    CSHTML5.Internal.INTERNAL_PropertyStore.ApplyCssChanges(value, value, propertyMetadata, storage.Owner);
                                }

                                if (!Bridge.staticEquals(propertyMetadata.MethodToUpdateDom, null)) {
                                    if (!valueWasRetrieved) {
                                        value = CSHTML5.Internal.INTERNAL_PropertyStore.GetEffectiveValue(storage);
                                        valueWasRetrieved = true;
                                    }

                                    propertyMetadata.MethodToUpdateDom(storage.Owner, value);
                                }


                                if (!Bridge.staticEquals(propertyMetadata.PropertyChangedCallback, null) && propertyMetadata.CallPropertyChangedWhenLoadedIntoVisualTree !== Windows.UI.Xaml.WhenToCallPropertyChangedEnum.Never) {
                                    if (!valueWasRetrieved) {
                                        value = CSHTML5.Internal.INTERNAL_PropertyStore.GetEffectiveValue(storage);
                                        valueWasRetrieved = true;
                                    }

                                    propertyMetadata.PropertyChangedCallback(storage.Owner, new Windows.UI.Xaml.DependencyPropertyChangedEventArgs.$ctor1(value, value, property));
                                }
                            }


                        }
                    } finally {
                        if (Bridge.is($t, System.IDisposable)) {
                            $t.System$IDisposable$Dispose();
                        }
                    }
                },
                /**
                 * Returns the first child of the specified type (recursively).
                 *
                 * @static
                 * @public
                 * @this CSHTML5.Internal.INTERNAL_VisualTreeManager
                 * @memberof CSHTML5.Internal.INTERNAL_VisualTreeManager
                 * @param   {Function}                     T         The type to lookup.
                 * @param   {Windows.UI.Xaml.UIElement}    parent    The parent element.
                 * @return  {T}                                      The first child of the specified type.
                 */
                GetChildOfType: function (T, parent) {
                    var $t;
                    if (parent == null || parent.INTERNAL_VisualChildrenInformation == null) {
                        return null;
                    }

                    $t = Bridge.getEnumerator(INTERNAL_BridgeWorkarounds.GetDictionaryValues_SimulatorCompatible(Windows.UI.Xaml.UIElement, CSHTML5.Internal.INTERNAL_VisualChildInformation, parent.INTERNAL_VisualChildrenInformation), CSHTML5.Internal.INTERNAL_VisualChildInformation);
                    try {
                        while ($t.moveNext()) {
                            var visualChildInformation = $t.Current;
                            var child = visualChildInformation.INTERNAL_UIElement;
                            var result = (Bridge.as(child, T)) || CSHTML5.Internal.INTERNAL_VisualTreeManager.GetChildOfType(T, child);
                            if (result != null) {
                                return result;
                            }
                        }
                    } finally {
                        if (Bridge.is($t, System.IDisposable)) {
                            $t.System$IDisposable$Dispose();
                        }
                    }

                    return null;
                }
            }
        }
    });

    Bridge.define("CSHTML5.Internal.INTERNAL_WorkaroundIE11IssuesWithScrollViewerInsideGrid", {
        statics: {
            methods: {
                RefreshLayoutIfIE: function () {
                    if (true) {
                        
if (window.IE_VERSION)
{
    // Force refresh the layout to workaround the issues when a scrollviewer is inside a grid:
    var temp = document.createElement('div');
    document.body.appendChild(temp);
    document.body.removeChild(temp);
}
;
                    }
                }
            }
        }
    });

    Bridge.define("CSHTML5.Internal.INTERNAL_WorkaroundObservableCollectionBugWithJSIL", {
        statics: {
            methods: {
                Contains: function (collection, value) {
                    if (true) {
                        return CSHTML5.Internal.INTERNAL_WorkaroundObservableCollectionBugWithJSIL.ContainsForJavascript(collection, value);
                    }
                    return System.Array.contains(collection, value);
                },
                ContainsForJavascript: function (collection, value) {
                    return Bridge.cast(collection, System.Collections.ObjectModel.ObservableCollection$1(System.Object)).contains(value);
                },
                Remove: function (collection, value) {
                    if (true) {
                        CSHTML5.Internal.INTERNAL_WorkaroundObservableCollectionBugWithJSIL.RemoveForJavascript(collection, value);
                    } else {
                        System.Array.remove(collection, value, Object);
                    }
                },
                RemoveForJavascript: function (collection, value) {
                    Bridge.cast(collection, System.Collections.ObjectModel.ObservableCollection$1(System.Object)).remove(value);
                },
                Clear: function (collection) {
                    if (true) {
                        CSHTML5.Internal.INTERNAL_WorkaroundObservableCollectionBugWithJSIL.ClearForJavascript(collection);
                    } else {
                        System.Array.clear(collection, Object);
                    }
                },
                ClearForJavascript: function (collection) {
                    Bridge.cast(collection, System.Collections.ObjectModel.ObservableCollection$1(System.Object)).clear();
                },
                Add: function (collection, value) {
                    if (true) {
                        CSHTML5.Internal.INTERNAL_WorkaroundObservableCollectionBugWithJSIL.AddForJavascript(collection, value);
                    } else {
                        System.Array.add(collection, value, Object);
                    }
                },
                AddForJavascript: function (collection, value) {
                    Bridge.cast(collection, System.Collections.ObjectModel.ObservableCollection$1(System.Object)).add(value);
                }
            }
        }
    });

    Bridge.define("CSHTML5.Internal.INTERNAL_XamlResourcesHandler", {
        statics: {
            fields: {
                HIGH_CONSTRAST_RESOURCE_THEME_KEY: null,
                LIGHT_RESOURCE_THEME_KEY: null,
                DARK_RESOURCE_THEME_KEY: null,
                _parsing: 0
            },
            props: {
                IsSystemResourcesParsing: {
                    get: function () {
                        return CSHTML5.Internal.INTERNAL_XamlResourcesHandler._parsing > 0;
                    },
                    set: function (value) {
                        if (value) {
                            CSHTML5.Internal.INTERNAL_XamlResourcesHandler._parsing = (CSHTML5.Internal.INTERNAL_XamlResourcesHandler._parsing + 1) | 0;
                        } else {
                            CSHTML5.Internal.INTERNAL_XamlResourcesHandler._parsing = (CSHTML5.Internal.INTERNAL_XamlResourcesHandler._parsing - 1) | 0;
                        }
                    }
                }
            },
            ctors: {
                init: function () {
                    this.HIGH_CONSTRAST_RESOURCE_THEME_KEY = "HighContrast";
                    this.LIGHT_RESOURCE_THEME_KEY = "Light";
                    this.DARK_RESOURCE_THEME_KEY = "Dark";
                }
            },
            methods: {
                MakeTitleCase: function (str) {
                    var result = "";
                    var lowerStr = str.toLowerCase();
                    var length = str.length;
                    var makeUpper = true;
                    var lastCopiedIndex = -1;

                    for (var i = 0; i < length; i = (i + 1) | 0) {
                        var ch = lowerStr.charCodeAt(i);
                        if (ch >= 97 && ch <= 122 || ch >= 48 && ch <= 48) {
                            if (makeUpper && ch >= 97 && ch <= 122) {
                                if (!(lastCopiedIndex === -1 && i === 0)) {
                                    result = (result || "") + ((lowerStr.substr(((lastCopiedIndex + 1) | 0), ((((i - lastCopiedIndex) | 0) - 1) | 0))) || "");
                                }
                                result = (result || "") + String.fromCharCode((((((ch - 32) | 0)) & 65535)));
                                lastCopiedIndex = i;
                            }
                            makeUpper = false;
                        } else {
                            makeUpper = true;
                        }
                    }
                    if (lastCopiedIndex < ((length - 1) | 0)) {
                        result = (result || "") + ((str.substr(((lastCopiedIndex + 1) | 0))) || "");
                    }
                    return result;


                }
            }
        },
        fields: {
            _assemblyToResourceDictionary: null,
            _defaultResources: null,
            _defaultThemeResourcesDictionary: null,
            _charToSimpleHighContrastNames: null
        },
        ctors: {
            init: function () {
                this._assemblyToResourceDictionary = new (System.Collections.Generic.Dictionary$2(System.Reflection.Assembly,Windows.UI.Xaml.ResourceDictionary)).ctor();
            }
        },
        methods: {
            /**
             * Tries to find the resourceKey in the Generic.xaml resources of the assembly. Note: the resource currently need to be defined in Project/Themes/generic.xaml
             *
             * @instance
             * @this CSHTML5.Internal.INTERNAL_XamlResourcesHandler
             * @memberof CSHTML5.Internal.INTERNAL_XamlResourcesHandler
             * @param   {System.Reflection.Assembly}    assemblyWhereGenericXamlIsLocated    The assembly where we will look into.
             * @param   {System.Object}                 resourceKey                          The resource to find in the Assembly's resources.
             * @return  {System.Object}                                                      The resource associated with the given key in the given assembly's resources.
             */
            TryFindResourceInGenericXaml: function (assemblyWhereGenericXamlIsLocated, resourceKey) {
                var resourceDictionary = null;
                if (this._assemblyToResourceDictionary.containsKey(assemblyWhereGenericXamlIsLocated)) {
                    resourceDictionary = this._assemblyToResourceDictionary.getItem(assemblyWhereGenericXamlIsLocated);
                } else {
                    var assemblyName = INTERNAL_BridgeWorkarounds.GetAssemblyNameWithoutCallingGetNameMethod(assemblyWhereGenericXamlIsLocated);
                    assemblyName = System.String.replaceAll(System.String.replaceAll(assemblyName, " ", "\u01c0\u01c0"), ".", "\u01c0\u01c0");
                    var factoryTypeName = CSHTML5.Internal.INTERNAL_XamlResourcesHandler.MakeTitleCase("\u01c0\u01c0" + (assemblyName || "") + "\u01c0\u01c0Component\u01c0\u01c0Themes\u01c0\u01c0Generic\u01c0\u01c0Xaml\u01c0\u01c0Factory");
                    var resourceDictionaryFactoryType = Bridge.Reflection.getType(factoryTypeName, assemblyWhereGenericXamlIsLocated);

                    if (resourceDictionaryFactoryType != null) {
                        CSHTML5.Internal.INTERNAL_XamlResourcesHandler.IsSystemResourcesParsing = true;

                        try {
                            resourceDictionary = Bridge.cast(Bridge.Reflection.midel(Bridge.Reflection.getMembers(resourceDictionaryFactoryType, 8, 284, "Instantiate"), null).apply(null, null), Windows.UI.Xaml.ResourceDictionary);
                        } finally {
                            CSHTML5.Internal.INTERNAL_XamlResourcesHandler.IsSystemResourcesParsing = false;
                        }

                        this._assemblyToResourceDictionary.add(assemblyWhereGenericXamlIsLocated, resourceDictionary);
                    }
                }

                if (resourceDictionary != null && resourceDictionary.contains(resourceKey)) {
                    return resourceDictionary.getItem(resourceKey);
                }

                return null;
            },
            /**
             * Searches for the specified resource.
             *
             * @instance
             * @this CSHTML5.Internal.INTERNAL_XamlResourcesHandler
             * @memberof CSHTML5.Internal.INTERNAL_XamlResourcesHandler
             * @param   {System.Object}    resourceKey    The name of the resource to find.
             * @return  {System.Object}                   The requested resource object. If the requested resource is not found, a
             null reference is returned.
             */
            TryFindResource: function (resourceKey) {
                if (Bridge.is(resourceKey, System.Type)) {
                    var resourceKeyAsType = Bridge.as(resourceKey, System.Type);
                    var result = this.TryFindResourceInGenericXaml(Bridge.Reflection.getTypeAssembly(resourceKeyAsType), resourceKeyAsType);
                    if (result != null) {
                        return result;
                    }
                }

                if (Windows.UI.Xaml.Application.Current.Resources.contains(resourceKey)) {
                    return Windows.UI.Xaml.Application.Current.Resources.getItem(resourceKey);
                }

                CSHTML5.Internal.INTERNAL_XamlResourcesHandler.IsSystemResourcesParsing = true;

                try {
                    this.EnsureDefaultThemeResources();
                } finally {
                    CSHTML5.Internal.INTERNAL_XamlResourcesHandler.IsSystemResourcesParsing = false;
                }

                var lightResourceDictionary = this._defaultThemeResourcesDictionary.ThemeDictionaries.System$Collections$Generic$IDictionary$2$System$Object$Windows$UI$Xaml$ResourceDictionary$getItem(CSHTML5.Internal.INTERNAL_XamlResourcesHandler.LIGHT_RESOURCE_THEME_KEY);
                if (lightResourceDictionary.contains(resourceKey)) {
                    return lightResourceDictionary.getItem(resourceKey);
                }

                if (this._defaultThemeResourcesDictionary.ThemeDictionaries.System$Collections$Generic$IDictionary$2$System$Object$Windows$UI$Xaml$ResourceDictionary$getItem(CSHTML5.Internal.INTERNAL_XamlResourcesHandler.HIGH_CONSTRAST_RESOURCE_THEME_KEY).contains(resourceKey)) {
                    return this._defaultThemeResourcesDictionary.ThemeDictionaries.System$Collections$Generic$IDictionary$2$System$Object$Windows$UI$Xaml$ResourceDictionary$getItem(CSHTML5.Internal.INTERNAL_XamlResourcesHandler.HIGH_CONSTRAST_RESOURCE_THEME_KEY).getItem(resourceKey);
                }

                if (Bridge.is(resourceKey, System.String)) {
                    var result1 = null;
                    result1 = this.FindThemeDependentBrush(Bridge.as(resourceKey, System.String));
                    if (result1 != null) {
                        return result1;
                    }
                }

                CSHTML5.Internal.INTERNAL_XamlResourcesHandler.IsSystemResourcesParsing = true;

                try {
                    this.EnsureDefaultResources();
                } finally {
                    CSHTML5.Internal.INTERNAL_XamlResourcesHandler.IsSystemResourcesParsing = false;
                }

                if (this._defaultResources.contains(resourceKey)) {
                    return this._defaultResources.getItem(resourceKey);
                }
                return null;
            },
            FindThemeDependentBrush: function (resourceName) {
                var $t;

                if (!(System.String.startsWith(resourceName, "SystemControl") && System.String.endsWith(resourceName, "Brush"))) {
                    return null;
                }






                if (this._charToSimpleHighContrastNames == null) {
                    this._charToSimpleHighContrastNames = new (System.Collections.Generic.Dictionary$2(System.Char,System.Collections.Generic.List$1(System.String))).ctor();
                    this._charToSimpleHighContrastNames.add(66, function (_o1) {
                            _o1.add("Background");
                            return _o1;
                        }(new (System.Collections.Generic.List$1(System.String)).ctor()));
                    this._charToSimpleHighContrastNames.add(70, function (_o2) {
                            _o2.add("Foreground");
                            return _o2;
                        }(new (System.Collections.Generic.List$1(System.String)).ctor()));
                    this._charToSimpleHighContrastNames.add(68, function (_o3) {
                            _o3.add("Disabled");
                            return _o3;
                        }(new (System.Collections.Generic.List$1(System.String)).ctor()));
                    this._charToSimpleHighContrastNames.add(72, function (_o4) {
                            _o4.add("Highlight");
                            _o4.add("Hyperlink");
                            return _o4;
                        }(new (System.Collections.Generic.List$1(System.String)).ctor()));
                    this._charToSimpleHighContrastNames.add(80, function (_o5) {
                            _o5.add("PageBackground");
                            _o5.add("PageText");
                            return _o5;
                        }(new (System.Collections.Generic.List$1(System.String)).ctor()));
                }

                resourceName = resourceName.substr(13, ((resourceName.length - 18) | 0));

                var firstLetter = resourceName.charCodeAt(0);
                var possibleNames = this._charToSimpleHighContrastNames.containsKey(firstLetter) ? this._charToSimpleHighContrastNames.getItem(firstLetter) : null;
                if (possibleNames == null) {
                    return null;
                }
                $t = Bridge.getEnumerator(possibleNames);
                try {
                    while ($t.moveNext()) {
                        var possibleName = $t.Current;
                        if (System.String.startsWith(resourceName, possibleName)) {
                            resourceName = resourceName.substr(possibleName.length);
                            break;
                        }
                    }
                } finally {
                    if (Bridge.is($t, System.IDisposable)) {
                        $t.System$IDisposable$Dispose();
                    }
                }

                if (this._defaultThemeResourcesDictionary.ThemeDictionaries.System$Collections$Generic$IDictionary$2$System$Object$Windows$UI$Xaml$ResourceDictionary$getItem(CSHTML5.Internal.INTERNAL_XamlResourcesHandler.LIGHT_RESOURCE_THEME_KEY).contains(resourceName)) {
                    var color = System.Nullable.getValue(Bridge.cast(Bridge.unbox(this._defaultThemeResourcesDictionary.ThemeDictionaries.System$Collections$Generic$IDictionary$2$System$Object$Windows$UI$Xaml$ResourceDictionary$getItem(CSHTML5.Internal.INTERNAL_XamlResourcesHandler.LIGHT_RESOURCE_THEME_KEY).getItem(resourceName), Windows.UI.Color), Windows.UI.Color));
                    return new Windows.UI.Xaml.Media.SolidColorBrush.$ctor1(color.$clone());
                } else {
                    if (System.String.startsWith(resourceName, "Alt")) {
                        resourceName = resourceName.substr(3);
                        if (this._defaultThemeResourcesDictionary.ThemeDictionaries.System$Collections$Generic$IDictionary$2$System$Object$Windows$UI$Xaml$ResourceDictionary$getItem(CSHTML5.Internal.INTERNAL_XamlResourcesHandler.LIGHT_RESOURCE_THEME_KEY).contains(resourceName)) {
                            var color1 = System.Nullable.getValue(Bridge.cast(Bridge.unbox(this._defaultThemeResourcesDictionary.ThemeDictionaries.System$Collections$Generic$IDictionary$2$System$Object$Windows$UI$Xaml$ResourceDictionary$getItem(CSHTML5.Internal.INTERNAL_XamlResourcesHandler.LIGHT_RESOURCE_THEME_KEY).getItem(resourceName), Windows.UI.Color), Windows.UI.Color));
                            return new Windows.UI.Xaml.Media.SolidColorBrush.$ctor1(color1.$clone());
                        }
                    }
                    if (Bridge.referenceEquals(resourceName, "Transparent")) {
                        return new Windows.UI.Xaml.Media.SolidColorBrush.$ctor1(Windows.UI.Colors.Transparent.$clone());
                    }

                }
                return null;
            },
            EnsureDefaultResources: function () {
                if (this._defaultResources != null) {
                    return;
                }

                this._defaultResources = new Windows.UI.Xaml.ResourceDictionary();
                this._defaultResources.setItem("ContentControlThemeFontFamily", new Windows.UI.Xaml.Media.FontFamily("Segoe UI"));
                this._defaultResources.setItem("SymbolThemeFontFamily", new Windows.UI.Xaml.Media.FontFamily("Segoe UI Symbol"));
                this._defaultResources.setItem("ControlContentThemeFontSize", Bridge.box(14.667, System.Double, System.Double.format, System.Double.getHashCode));
                this._defaultResources.setItem("ThumbBorderThemeBrush", new Windows.UI.Xaml.Media.SolidColorBrush.$ctor1(System.Nullable.getValue(Bridge.cast(Bridge.unbox(Windows.UI.Color.INTERNAL_ConvertFromString("#3B555555"), Windows.UI.Color), Windows.UI.Color))));
                this._defaultResources.setItem("SystemAccentColor", Windows.UI.Color.INTERNAL_ConvertFromString("#FF0078D7"));
                this._defaultResources.setItem("SystemColorButtonFaceColor", Windows.UI.Color.INTERNAL_ConvertFromString("#FF000000"));
                this._defaultResources.setItem("ScrollBarThumbBackgroundColor", Windows.UI.Color.INTERNAL_ConvertFromString("#FF333333"));
                this._defaultResources.setItem("ScrollBarPanningThumbBackgroundColor", Windows.UI.Color.INTERNAL_ConvertFromString("#00333333"));
            },
            EnsureDefaultThemeResources: function () {
                if (this._defaultThemeResourcesDictionary != null) {
                    return;
                }



                this._defaultThemeResourcesDictionary = new Windows.UI.Xaml.ResourceDictionary();

                var resourceDictionary = new Windows.UI.Xaml.ResourceDictionary();

                var color = System.Nullable.getValue(Bridge.cast(Bridge.unbox(Windows.UI.Color.INTERNAL_ConvertFromString("#FFFFFFFF"), Windows.UI.Color), Windows.UI.Color));
                resourceDictionary.Add("SystemAltHighColor", color.$clone());

                color = System.Nullable.getValue(Bridge.cast(Bridge.unbox(Windows.UI.Color.INTERNAL_ConvertFromString("#33FFFFFF"), Windows.UI.Color), Windows.UI.Color));
                resourceDictionary.Add("SystemAltLowColor", color.$clone());

                color = System.Nullable.getValue(Bridge.cast(Bridge.unbox(Windows.UI.Color.INTERNAL_ConvertFromString("#99FFFFFF"), Windows.UI.Color), Windows.UI.Color));
                resourceDictionary.Add("SystemAltMediumColor", color.$clone());

                color = System.Nullable.getValue(Bridge.cast(Bridge.unbox(Windows.UI.Color.INTERNAL_ConvertFromString("#CCFFFFFF"), Windows.UI.Color), Windows.UI.Color));
                resourceDictionary.Add("SystemAltMediumHighColor", color.$clone());

                color = System.Nullable.getValue(Bridge.cast(Bridge.unbox(Windows.UI.Color.INTERNAL_ConvertFromString("#66FFFFFF"), Windows.UI.Color), Windows.UI.Color));
                resourceDictionary.Add("SystemAltMediumLowColor", color.$clone());

                color = System.Nullable.getValue(Bridge.cast(Bridge.unbox(Windows.UI.Color.INTERNAL_ConvertFromString("#FF000000"), Windows.UI.Color), Windows.UI.Color));
                resourceDictionary.Add("SystemBaseHighColor", color.$clone());

                color = System.Nullable.getValue(Bridge.cast(Bridge.unbox(Windows.UI.Color.INTERNAL_ConvertFromString("#33000000"), Windows.UI.Color), Windows.UI.Color));
                resourceDictionary.Add("SystemBaseLowColor", color.$clone());

                color = System.Nullable.getValue(Bridge.cast(Bridge.unbox(Windows.UI.Color.INTERNAL_ConvertFromString("#99000000"), Windows.UI.Color), Windows.UI.Color));
                resourceDictionary.Add("SystemBaseMediumColor", color.$clone());

                color = System.Nullable.getValue(Bridge.cast(Bridge.unbox(Windows.UI.Color.INTERNAL_ConvertFromString("#CC000000"), Windows.UI.Color), Windows.UI.Color));
                resourceDictionary.Add("SystemBaseMediumHighColor", color.$clone());

                color = System.Nullable.getValue(Bridge.cast(Bridge.unbox(Windows.UI.Color.INTERNAL_ConvertFromString("#66000000"), Windows.UI.Color), Windows.UI.Color));
                resourceDictionary.Add("SystemBaseMediumLowColor", color.$clone());

                color = System.Nullable.getValue(Bridge.cast(Bridge.unbox(Windows.UI.Color.INTERNAL_ConvertFromString("#FF171717"), Windows.UI.Color), Windows.UI.Color));
                resourceDictionary.Add("SystemChromeAltLowColor", color.$clone());

                color = System.Nullable.getValue(Bridge.cast(Bridge.unbox(Windows.UI.Color.INTERNAL_ConvertFromString("#FF000000"), Windows.UI.Color), Windows.UI.Color));
                resourceDictionary.Add("SystemChromeBlackHighColor", color.$clone());

                color = System.Nullable.getValue(Bridge.cast(Bridge.unbox(Windows.UI.Color.INTERNAL_ConvertFromString("#33000000"), Windows.UI.Color), Windows.UI.Color));
                resourceDictionary.Add("SystemChromeBlackLowColor", color.$clone());

                color = System.Nullable.getValue(Bridge.cast(Bridge.unbox(Windows.UI.Color.INTERNAL_ConvertFromString("#66000000"), Windows.UI.Color), Windows.UI.Color));
                resourceDictionary.Add("SystemChromeBlackMediumLowColor", color.$clone());

                color = System.Nullable.getValue(Bridge.cast(Bridge.unbox(Windows.UI.Color.INTERNAL_ConvertFromString("#CC000000"), Windows.UI.Color), Windows.UI.Color));
                resourceDictionary.Add("SystemChromeBlackMediumColor", color.$clone());

                color = System.Nullable.getValue(Bridge.cast(Bridge.unbox(Windows.UI.Color.INTERNAL_ConvertFromString("#FFCCCCCC"), Windows.UI.Color), Windows.UI.Color));
                resourceDictionary.Add("SystemChromeDisabledHighColor", color.$clone());

                color = System.Nullable.getValue(Bridge.cast(Bridge.unbox(Windows.UI.Color.INTERNAL_ConvertFromString("#FF7A7A7A"), Windows.UI.Color), Windows.UI.Color));
                resourceDictionary.Add("SystemChromeDisabledLowColor", color.$clone());

                color = System.Nullable.getValue(Bridge.cast(Bridge.unbox(Windows.UI.Color.INTERNAL_ConvertFromString("#FFCCCCCC"), Windows.UI.Color), Windows.UI.Color));
                resourceDictionary.Add("SystemChromeHighColor", color.$clone());

                color = System.Nullable.getValue(Bridge.cast(Bridge.unbox(Windows.UI.Color.INTERNAL_ConvertFromString("#FFF2F2F2"), Windows.UI.Color), Windows.UI.Color));
                resourceDictionary.Add("SystemChromeLowColor", color.$clone());

                color = System.Nullable.getValue(Bridge.cast(Bridge.unbox(Windows.UI.Color.INTERNAL_ConvertFromString("#FFE6E6E6"), Windows.UI.Color), Windows.UI.Color));
                resourceDictionary.Add("SystemChromeMediumColor", color.$clone());

                color = System.Nullable.getValue(Bridge.cast(Bridge.unbox(Windows.UI.Color.INTERNAL_ConvertFromString("#FFF2F2F2"), Windows.UI.Color), Windows.UI.Color));
                resourceDictionary.Add("SystemChromeMediumLowColor", color.$clone());

                color = System.Nullable.getValue(Bridge.cast(Bridge.unbox(Windows.UI.Color.INTERNAL_ConvertFromString("#FFFFFFFF"), Windows.UI.Color), Windows.UI.Color));
                resourceDictionary.Add("SystemChromeWhiteColor", color.$clone());

                color = System.Nullable.getValue(Bridge.cast(Bridge.unbox(Windows.UI.Color.INTERNAL_ConvertFromString("#19000000"), Windows.UI.Color), Windows.UI.Color));
                resourceDictionary.Add("SystemListLowColor", color.$clone());

                color = System.Nullable.getValue(Bridge.cast(Bridge.unbox(Windows.UI.Color.INTERNAL_ConvertFromString("#33000000"), Windows.UI.Color), Windows.UI.Color));
                resourceDictionary.Add("SystemListMediumColor", color.$clone());

                this._defaultThemeResourcesDictionary.ThemeDictionaries.System$Collections$Generic$IDictionary$2$System$Object$Windows$UI$Xaml$ResourceDictionary$add(CSHTML5.Internal.INTERNAL_XamlResourcesHandler.LIGHT_RESOURCE_THEME_KEY, resourceDictionary);

                resourceDictionary = new Windows.UI.Xaml.ResourceDictionary();


                color = System.Nullable.getValue(Bridge.cast(Bridge.unbox(Windows.UI.Color.INTERNAL_ConvertFromString("#FF000000"), Windows.UI.Color), Windows.UI.Color));
                resourceDictionary.Add("SystemAltHighColor", color.$clone());

                color = System.Nullable.getValue(Bridge.cast(Bridge.unbox(Windows.UI.Color.INTERNAL_ConvertFromString("#33000000"), Windows.UI.Color), Windows.UI.Color));
                resourceDictionary.Add("SystemAltLowColor", color.$clone());

                color = System.Nullable.getValue(Bridge.cast(Bridge.unbox(Windows.UI.Color.INTERNAL_ConvertFromString("#99000000"), Windows.UI.Color), Windows.UI.Color));
                resourceDictionary.Add("SystemAltMediumColor", color.$clone());

                color = System.Nullable.getValue(Bridge.cast(Bridge.unbox(Windows.UI.Color.INTERNAL_ConvertFromString("#CC000000"), Windows.UI.Color), Windows.UI.Color));
                resourceDictionary.Add("SystemAltMediumHighColor", color.$clone());

                color = System.Nullable.getValue(Bridge.cast(Bridge.unbox(Windows.UI.Color.INTERNAL_ConvertFromString("#66000000"), Windows.UI.Color), Windows.UI.Color));
                resourceDictionary.Add("SystemAltMediumLowColor", color.$clone());

                color = System.Nullable.getValue(Bridge.cast(Bridge.unbox(Windows.UI.Color.INTERNAL_ConvertFromString("#FFFFFFFF"), Windows.UI.Color), Windows.UI.Color));
                resourceDictionary.Add("SystemBaseHighColor", color.$clone());

                color = System.Nullable.getValue(Bridge.cast(Bridge.unbox(Windows.UI.Color.INTERNAL_ConvertFromString("#33FFFFFF"), Windows.UI.Color), Windows.UI.Color));
                resourceDictionary.Add("SystemBaseLowColor", color.$clone());

                color = System.Nullable.getValue(Bridge.cast(Bridge.unbox(Windows.UI.Color.INTERNAL_ConvertFromString("#99FFFFFF"), Windows.UI.Color), Windows.UI.Color));
                resourceDictionary.Add("SystemBaseMediumColor", color.$clone());

                color = System.Nullable.getValue(Bridge.cast(Bridge.unbox(Windows.UI.Color.INTERNAL_ConvertFromString("#CCFFFFFF"), Windows.UI.Color), Windows.UI.Color));
                resourceDictionary.Add("SystemBaseMediumHighColor", color.$clone());

                color = System.Nullable.getValue(Bridge.cast(Bridge.unbox(Windows.UI.Color.INTERNAL_ConvertFromString("#66FFFFFF"), Windows.UI.Color), Windows.UI.Color));
                resourceDictionary.Add("SystemBaseMediumLowColor", color.$clone());

                color = System.Nullable.getValue(Bridge.cast(Bridge.unbox(Windows.UI.Color.INTERNAL_ConvertFromString("#FFF2F2F2"), Windows.UI.Color), Windows.UI.Color));
                resourceDictionary.Add("SystemChromeAltLowColor", color.$clone());

                color = System.Nullable.getValue(Bridge.cast(Bridge.unbox(Windows.UI.Color.INTERNAL_ConvertFromString("#FF000000"), Windows.UI.Color), Windows.UI.Color));
                resourceDictionary.Add("SystemChromeBlackHighColor", color.$clone());

                color = System.Nullable.getValue(Bridge.cast(Bridge.unbox(Windows.UI.Color.INTERNAL_ConvertFromString("#33000000"), Windows.UI.Color), Windows.UI.Color));
                resourceDictionary.Add("SystemChromeBlackLowColor", color.$clone());

                color = System.Nullable.getValue(Bridge.cast(Bridge.unbox(Windows.UI.Color.INTERNAL_ConvertFromString("#66000000"), Windows.UI.Color), Windows.UI.Color));
                resourceDictionary.Add("SystemChromeBlackMediumLowColor", color.$clone());

                color = System.Nullable.getValue(Bridge.cast(Bridge.unbox(Windows.UI.Color.INTERNAL_ConvertFromString("#CC000000"), Windows.UI.Color), Windows.UI.Color));
                resourceDictionary.Add("SystemChromeBlackMediumColor", color.$clone());

                color = System.Nullable.getValue(Bridge.cast(Bridge.unbox(Windows.UI.Color.INTERNAL_ConvertFromString("#FF333333"), Windows.UI.Color), Windows.UI.Color));
                resourceDictionary.Add("SystemChromeDisabledHighColor", color.$clone());

                color = System.Nullable.getValue(Bridge.cast(Bridge.unbox(Windows.UI.Color.INTERNAL_ConvertFromString("#FF858585"), Windows.UI.Color), Windows.UI.Color));
                resourceDictionary.Add("SystemChromeDisabledLowColor", color.$clone());

                color = System.Nullable.getValue(Bridge.cast(Bridge.unbox(Windows.UI.Color.INTERNAL_ConvertFromString("#FF767676"), Windows.UI.Color), Windows.UI.Color));
                resourceDictionary.Add("SystemChromeHighColor", color.$clone());

                color = System.Nullable.getValue(Bridge.cast(Bridge.unbox(Windows.UI.Color.INTERNAL_ConvertFromString("#FF171717"), Windows.UI.Color), Windows.UI.Color));
                resourceDictionary.Add("SystemChromeLowColor", color.$clone());

                color = System.Nullable.getValue(Bridge.cast(Bridge.unbox(Windows.UI.Color.INTERNAL_ConvertFromString("#FF1F1F1F"), Windows.UI.Color), Windows.UI.Color));
                resourceDictionary.Add("SystemChromeMediumColor", color.$clone());

                color = System.Nullable.getValue(Bridge.cast(Bridge.unbox(Windows.UI.Color.INTERNAL_ConvertFromString("#FF2B2B2B"), Windows.UI.Color), Windows.UI.Color));
                resourceDictionary.Add("SystemChromeMediumLowColor", color.$clone());

                color = System.Nullable.getValue(Bridge.cast(Bridge.unbox(Windows.UI.Color.INTERNAL_ConvertFromString("#FFFFFFFF"), Windows.UI.Color), Windows.UI.Color));
                resourceDictionary.Add("SystemChromeWhiteColor", color.$clone());

                color = System.Nullable.getValue(Bridge.cast(Bridge.unbox(Windows.UI.Color.INTERNAL_ConvertFromString("#19FFFFFF"), Windows.UI.Color), Windows.UI.Color));
                resourceDictionary.Add("SystemListLowColor", color.$clone());

                color = System.Nullable.getValue(Bridge.cast(Bridge.unbox(Windows.UI.Color.INTERNAL_ConvertFromString("#33FFFFFF"), Windows.UI.Color), Windows.UI.Color));
                resourceDictionary.Add("SystemListMediumColor", color.$clone());


                this._defaultThemeResourcesDictionary.ThemeDictionaries.System$Collections$Generic$IDictionary$2$System$Object$Windows$UI$Xaml$ResourceDictionary$add(CSHTML5.Internal.INTERNAL_XamlResourcesHandler.DARK_RESOURCE_THEME_KEY, resourceDictionary);

                resourceDictionary = new Windows.UI.Xaml.ResourceDictionary();


                color = System.Nullable.getValue(Bridge.cast(Bridge.unbox(Windows.UI.Color.INTERNAL_ConvertFromString("#FFF0F0F0"), Windows.UI.Color), Windows.UI.Color));
                resourceDictionary.Add("SystemColorButtonFaceColor", color.$clone());

                color = System.Nullable.getValue(Bridge.cast(Bridge.unbox(Windows.UI.Color.INTERNAL_ConvertFromString("#FF000000"), Windows.UI.Color), Windows.UI.Color));
                resourceDictionary.Add("SystemColorButtonTextColor", color.$clone());

                color = System.Nullable.getValue(Bridge.cast(Bridge.unbox(Windows.UI.Color.INTERNAL_ConvertFromString("#FF6D6D6D"), Windows.UI.Color), Windows.UI.Color));
                resourceDictionary.Add("SystemColorGrayTextColor", color.$clone());

                color = System.Nullable.getValue(Bridge.cast(Bridge.unbox(Windows.UI.Color.INTERNAL_ConvertFromString("#FF3399FF"), Windows.UI.Color), Windows.UI.Color));
                resourceDictionary.Add("SystemColorHighlightColor", color.$clone());

                color = System.Nullable.getValue(Bridge.cast(Bridge.unbox(Windows.UI.Color.INTERNAL_ConvertFromString("#FFFFFFFF"), Windows.UI.Color), Windows.UI.Color));
                resourceDictionary.Add("SystemColorHighlightTextColor", color.$clone());

                color = System.Nullable.getValue(Bridge.cast(Bridge.unbox(Windows.UI.Color.INTERNAL_ConvertFromString("#FF0066CC"), Windows.UI.Color), Windows.UI.Color));
                resourceDictionary.Add("SystemColorHotlightColor", color.$clone());

                color = System.Nullable.getValue(Bridge.cast(Bridge.unbox(Windows.UI.Color.INTERNAL_ConvertFromString("#FFFFFFFF"), Windows.UI.Color), Windows.UI.Color));
                resourceDictionary.Add("SystemColorWindowColor", color.$clone());

                color = System.Nullable.getValue(Bridge.cast(Bridge.unbox(Windows.UI.Color.INTERNAL_ConvertFromString("#FF000000"), Windows.UI.Color), Windows.UI.Color));
                resourceDictionary.Add("SystemColorWindowTextColor", color.$clone());


                this._defaultThemeResourcesDictionary.ThemeDictionaries.System$Collections$Generic$IDictionary$2$System$Object$Windows$UI$Xaml$ResourceDictionary$add(CSHTML5.Internal.INTERNAL_XamlResourcesHandler.HIGH_CONSTRAST_RESOURCE_THEME_KEY, resourceDictionary);



            }
        }
    });

    Bridge.define("CSHTML5.Internal.KnownTypesHelper", {
        statics: {
            fields: {
                _additionalKnownTypes: null
            },
            ctors: {
                init: function () {
                    this._additionalKnownTypes = new (System.Collections.Generic.HashSet$1(System.Type)).ctor();
                }
            },
            methods: {
                AddKnownType: function (knownType) {
                    if (!CSHTML5.Internal.KnownTypesHelper._additionalKnownTypes.contains(knownType)) {
                        CSHTML5.Internal.KnownTypesHelper._additionalKnownTypes.add(knownType);
                    }
                }
            }
        }
    });

    Bridge.define("CSHTML5.Internal.ModifiedValue", {
        fields: {
            _baseValue: null,
            _expressionValue: null,
            _coercedValue: null
        },
        props: {
            BaseValue: {
                get: function () {
                    return this._baseValue;
                },
                set: function (value) {
                    this._baseValue = value;
                }
            },
            ExpressionValue: {
                get: function () {
                    return this._expressionValue;
                },
                set: function (value) {
                    this._expressionValue = value;
                }
            },
            CoercedValue: {
                get: function () {
                    return this._coercedValue;
                },
                set: function (value) {
                    this._coercedValue = value;
                }
            }
        }
    });

    /**
     * {@link } namespace.
     *
     * @class CSHTML5.Internal.NamespaceDoc
     */
    Bridge.define("CSHTML5.Internal.NamespaceDoc");

    Bridge.define("CSHTML5.Internal.Performance");

    Bridge.define("CSHTML5.Internal.StartupAssemblyInfo", {
        statics: {
            fields: {
                _startupAssembly: null,
                _startupAssemblyShortName: null,
                OutputRootPath: null,
                OutputAppFilesPath: null,
                OutputLibrariesPath: null,
                OutputResourcesPath: null
            },
            props: {
                StartupAssembly: {
                    get: function () {
                        return CSHTML5.Internal.StartupAssemblyInfo._startupAssembly;
                    },
                    set: function (value) {
                        CSHTML5.Internal.StartupAssemblyInfo._startupAssembly = value;

                        var name = CSHTML5.Internal.StartupAssemblyInfo._startupAssembly.name;
                        var i = System.String.indexOf(name, String.fromCharCode(44));
                        if (i > -1) {
                            name = name.substr(0, i);
                        }
                        CSHTML5.Internal.StartupAssemblyInfo._startupAssemblyShortName = name;
                    }
                },
                StartupAssemblyShortName: {
                    get: function () {
                        return CSHTML5.Internal.StartupAssemblyInfo._startupAssemblyShortName;
                    }
                }
            }
        }
    });

    Bridge.define("CSHTML5.Internal.TypeInstantiationHelper", {
        statics: {
            methods: {
                /**
                 * Invokes the parameterless constructor of the specified type.
                 *
                 * @static
                 * @public
                 * @this CSHTML5.Internal.TypeInstantiationHelper
                 * @memberof CSHTML5.Internal.TypeInstantiationHelper
                 * @param   {System.Type}      type    The Type to instantiate.
                 * @return  {System.Object}            A new instance of the Type, initialized through the parameterless constructor.
                 */
                Instantiate: function (type) {
                    var $constructor = Bridge.Reflection.getMembers(type, 1, 284, null, System.Array.init([], System.Type));
                    if ($constructor != null) {
                        return Bridge.Reflection.invokeCI($constructor, null);
                    } else {
                        throw new System.Exception("No parameterless constructor defined for the type: \"" + (Bridge.Reflection.getTypeFullName(type) || "") + "\".");
                    }
                }
            }
        }
    });

    Bridge.define("CSHTML5.Internal.WebRequestsHelper", {
        statics: {
            methods: {
                MakeRequest: function (address, method, body, headers, isAsync) {
                    if (headers === void 0) { headers = null; }
                    if (isAsync === void 0) { isAsync = false; }
                    return new System.INTERNAL_WebRequestHelper_JSOnly().MakeRequest(new System.Uri(address), method, null, headers, body, null, isAsync);
                }
            }
        }
    });

    Bridge.define("CSHTML5.Internal.Windows.UI.Xaml.Data.PropertyNodeType", {
        $kind: "enum",
        statics: {
            fields: {
                None: 0,
                AttachedProperty: 1,
                Indexed: 2,
                Property: 3
            }
        }
    });

    Bridge.define("CSHTML5.Internal.Windows.UI.Xaml.Data.PropertyPathParser", {
        fields: {
            Path: null
        },
        ctors: {
            ctor: function (path) {
                this.$initialize();
                this.Path = path;
            }
        },
        methods: {
            Step: function (typeName, propertyName, index) {
                var type = CSHTML5.Internal.Windows.UI.Xaml.Data.PropertyNodeType.None;
                var path = this.Path;
                if (path.length === 0) {
                    typeName.v = null;
                    propertyName.v = null;
                    index.v = null;
                    return type;
                }

                var end = 0;
                if (path.charCodeAt(0) === 40) {
                    type = CSHTML5.Internal.Windows.UI.Xaml.Data.PropertyNodeType.AttachedProperty;
                    end = System.String.indexOf(path, String.fromCharCode(41));
                    if (end === -1) {
                        throw new System.ArgumentException.$ctor1("Invalid property path. Attached property is missing the closing bracket");
                    }

                    var tickOpen = System.String.indexOf(path, String.fromCharCode(39));
                    var tickClose = 0;
                    var typeOpen;
                    var typeClose;
                    var propOpen;
                    var propClose;

                    typeOpen = System.String.indexOf(path, String.fromCharCode(39));
                    if (typeOpen > 0) {
                        typeOpen = (typeOpen + 1) | 0;

                        typeClose = System.String.indexOf(path, String.fromCharCode(39), ((typeOpen + 1) | 0));
                        if (typeClose < 0) {
                            throw new System.Exception("Invalid property path, Unclosed type name '" + (path || "") + "'.");
                        }

                        propOpen = System.String.indexOf(path, String.fromCharCode(46), typeClose);
                        if (propOpen < 0) {
                            throw new System.Exception("Invalid properth path, No property indexer found '" + (path || "") + "'.");
                        }

                        propOpen = (propOpen + 1) | 0;
                    } else {
                        typeOpen = 1;
                        typeClose = System.String.indexOf(path, String.fromCharCode(46), typeOpen);
                        if (typeClose < 0) {
                            throw new System.Exception("Invalid property path, No property indexer found on '" + (path || "") + "'.");
                        }
                        propOpen = (typeClose + 1) | 0;
                    }

                    propClose = end;

                    typeName.v = path.substr(typeOpen, ((typeClose - typeOpen) | 0));
                    propertyName.v = path.substr(propOpen, ((propClose - propOpen) | 0));

                    index.v = null;
                    if (path.length > (((end + 1) | 0)) && path.charCodeAt(((end + 1) | 0)) === 46) {
                        end = (end + 1) | 0;
                    }
                    path = path.substr(((end + 1) | 0));
                } else if (path.charCodeAt(0) === 91) {
                    type = CSHTML5.Internal.Windows.UI.Xaml.Data.PropertyNodeType.Indexed;
                    end = System.String.indexOf(path, String.fromCharCode(93));

                    typeName.v = null;
                    propertyName.v = null;
                    index.v = path.substr(1, ((end - 1) | 0));
                    path = path.substr(((end + 1) | 0));
                    if (path.length > 0 && path.charCodeAt(0) === 46) {
                        path = path.substr(1);
                    }
                } else {
                    type = CSHTML5.Internal.Windows.UI.Xaml.Data.PropertyNodeType.Property;
                    var splitters = System.Array.init([46, 91], System.Char);
                    end = System.String.indexOfAny(path, splitters);

                    if (end === -1) {
                        propertyName.v = path;
                        path = "";
                    } else {
                        propertyName.v = path.substr(0, end);
                        if (path.charCodeAt(end) === 46) {
                            path = path.substr(((end + 1) | 0));
                        } else {
                            path = path.substr(end);
                        }
                    }

                    typeName.v = null;
                    index.v = null;
                }
                this.Path = path;

                return type;
            }
        }
    });

    Bridge.define("CSHTML5.Internal.WorkaroundFixMe");

    Bridge.define("CSHTML5.INTERNAL_InteropImplementation", {
        statics: {
            fields: {
                IsJavaScriptCSharpInteropSetUp: false,
                CallbacksDictionary: null,
                RandomGenerator: null,
                UnmodifiedJavascriptCalls: null,
                IndexOfNextUnmodifiedJSCallInList: 0,
                _pendingJSFile: null,
                _loadedFiles: null
            },
            ctors: {
                init: function () {
                    this.CallbacksDictionary = new (System.Collections.Generic.Dictionary$2(System.Int32,Function)).ctor();
                    this.RandomGenerator = new System.Random.ctor();
                    this.UnmodifiedJavascriptCalls = new (System.Collections.Generic.List$1(System.String)).ctor();
                    this.IndexOfNextUnmodifiedJSCallInList = 0;
                    this._pendingJSFile = new (System.Collections.Generic.Dictionary$2(System.String,System.Collections.Generic.List$1(System.Tuple$2(Function,Function)))).ctor();
                    this._loadedFiles = new (System.Collections.Generic.HashSet$1(System.String)).ctor();
                },
                ctor: function () {
                    Windows.UI.Xaml.Application.addINTERNAL_Reloaded(function (sender, e) {
                        CSHTML5.INTERNAL_InteropImplementation.IsJavaScriptCSharpInteropSetUp = false;
                    });
                }
            },
            methods: {
                ResetLoadedFilesDictionaries: function () {
                    CSHTML5.INTERNAL_InteropImplementation._pendingJSFile.clear();
                    CSHTML5.INTERNAL_InteropImplementation._loadedFiles.clear();
                },
                ShowErrorMessage: function (errorMessage, indexOfCallInList) {
                    var str = System.Linq.Enumerable.from(CSHTML5.INTERNAL_InteropImplementation.UnmodifiedJavascriptCalls).elementAt(indexOfCallInList);
                    var message = System.String.format("Error in the following javascript code:\r\n\r\n{0}\r\n\r\n----- Error: -----\r\n\r\n{1}\r\n", str, errorMessage);
                    Windows.UI.Xaml.MessageBox.Show(message);
                },
                LoadJavaScriptFile: function (url, callerAssemblyName, callbackOnSuccess, callbackOnFailure) {
                    if (callbackOnFailure === void 0) { callbackOnFailure = null; }
                    var html5Path = CSHTML5.Internal.INTERNAL_UriHelper.ConvertToHtml5Path(url);
                    if (CSHTML5.INTERNAL_InteropImplementation._loadedFiles.contains(html5Path)) {
                        callbackOnSuccess();
                    } else if (CSHTML5.INTERNAL_InteropImplementation._pendingJSFile.containsKey(html5Path)) {
                        CSHTML5.INTERNAL_InteropImplementation._pendingJSFile.getItem(html5Path).add({ Item1: callbackOnSuccess, Item2: callbackOnFailure });
                    } else {
                        CSHTML5.INTERNAL_InteropImplementation._pendingJSFile.add(html5Path, function (_o1) {
                                _o1.add({ Item1: callbackOnSuccess, Item2: callbackOnFailure });
                                return _o1;
                            }(new (System.Collections.Generic.List$1(System.Tuple$2(Function,Function))).ctor()));
// Add the script tag to the head
var filePath =                         html5Path;
var head = document.getElementsByTagName('head')[0];
var script = document.createElement('script');
script.type = 'text/javascript';
script.src = filePath;
// Then bind the event to the callback function
// There are several events for cross browser compatibility.
if(script.onreadystatechange != undefined) {
script.onreadystatechange = CSHTML5.INTERNAL_InteropImplementation.LoadJavaScriptFileSuccess;
} else {
script.onload = function () { CSHTML5.INTERNAL_InteropImplementation.LoadJavaScriptFileSuccess(filePath) };
script.onerror = function () { CSHTML5.INTERNAL_InteropImplementation.LoadJavaScriptFileFailure(filePath) };
}

// Fire the loading
head.appendChild(script);
                    }
                },
                LoadJavaScriptFileSuccess: function (jsArgument) {
                    var $t;
                    var loadedFileName = System.Convert.toString(jsArgument);
                    $t = Bridge.getEnumerator(CSHTML5.INTERNAL_InteropImplementation._pendingJSFile.getItem(loadedFileName));
                    try {
                        while ($t.moveNext()) {
                            var actions = $t.Current;
                            actions.Item1();
                        }
                    } finally {
                        if (Bridge.is($t, System.IDisposable)) {
                            $t.System$IDisposable$Dispose();
                        }
                    }
                    CSHTML5.INTERNAL_InteropImplementation._loadedFiles.add(loadedFileName);
                    CSHTML5.INTERNAL_InteropImplementation._pendingJSFile.remove(loadedFileName);
                },
                LoadJavaScriptFileFailure: function (jsArgument) {
                    var $t;
                    var loadedFileName = System.Convert.toString(jsArgument);
                    $t = Bridge.getEnumerator(CSHTML5.INTERNAL_InteropImplementation._pendingJSFile.getItem(loadedFileName));
                    try {
                        while ($t.moveNext()) {
                            var actions = $t.Current;
                            actions.Item2();
                        }
                    } finally {
                        if (Bridge.is($t, System.IDisposable)) {
                            $t.System$IDisposable$Dispose();
                        }
                    }
                    CSHTML5.INTERNAL_InteropImplementation._pendingJSFile.remove(loadedFileName);
                },
                LoadJavaScriptFiles: function (urls, callerAssemblyName, onCompleted, onError) {
                    if (onError === void 0) { onError = null; }
                    if (urls.Count > 0) {
                        CSHTML5.INTERNAL_InteropImplementation.LoadJavaScriptFile(urls.getItem(0), callerAssemblyName, function () {
                            urls.remove(urls.getItem(0));
                            CSHTML5.INTERNAL_InteropImplementation.LoadJavaScriptFiles(urls, callerAssemblyName, onCompleted, onError);
                        }, function () {
                            if (!Bridge.staticEquals(onError, null)) {
                                onError();
                            }
                        });
                    } else {
                        if (!Bridge.staticEquals(onCompleted, null)) {
                            onCompleted();
                        }
                    }
                },
                LoadCssFile: function (url, callback) {
                    var callerAssemblyName = CSHTML5.Internal.INTERNAL_UriHelper.GetJavaScriptCallingAssembly();

                    var html5Path = CSHTML5.Internal.INTERNAL_UriHelper.ConvertToHtml5Path(url);

// Add the link tag to the head
var head = document.getElementsByTagName('head')[0];
var link = document.createElement('link');
link.rel  = 'stylesheet';
link.type = 'text/css';
link.href =                     html5Path;
link.media = 'all';

// Fire the loading
head.appendChild(link);

// Some browsers do not support the 'onload' event of the 'link' element,
// therefore we use the 'onerror' event of the 'img' tag instead, which is always triggered:
var img = document.createElement('img');
img.onerror = callback;
img.src = html5Path;
                },
                LoadCssFiles: function (urls, onCompleted) {
                    if (urls.Count > 0) {
                        CSHTML5.INTERNAL_InteropImplementation.LoadCssFile(urls.getItem(0), function () {
                            urls.remove(urls.getItem(0));
                            CSHTML5.INTERNAL_InteropImplementation.LoadCssFiles(urls, onCompleted);
                        });
                    } else {
                        onCompleted();
                    }
                }
            }
        }
    });

    Bridge.define("CSHTML5.Internals.Controls.IGeneratorHost", {
        $kind: "interface"
    });

    /** @namespace CSHTML5 */

    /**
     * Provides static methods for executing JavaScript code from within C#.
     *
     * @static
     * @abstract
     * @public
     * @class CSHTML5.Interop
     */
    Bridge.define("CSHTML5.Interop", {
        statics: {
            fields: {
                _jsScriptFileKeys: null,
                _cssFileKeys: null
            },
            props: {
                /**
                 * Returns True is the app is running in C# inside the Simulator, and False otherwise.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof CSHTML5.Interop
                 * @function IsRunningInTheSimulator
                 * @type boolean
                 */
                IsRunningInTheSimulator: {
                    get: function () {
                        return false;
                    }
                }
            },
            ctors: {
                init: function () {
                    this._jsScriptFileKeys = new (System.Collections.Generic.HashSet2$1(System.String)).ctor();
                    this._cssFileKeys = new (System.Collections.Generic.HashSet2$1(System.String)).ctor();
                }
            },
            methods: {
                /**
                 * Adds a 'script' tag to the HTML page and waits for the script to finish loading.
                 *
                 * @static
                 * @public
                 * @this CSHTML5.Interop
                 * @memberof CSHTML5.Interop
                 * @param   {string}                           url    The URL of the JavaScript file, with the syntax ms-appx:AssemblyName/Folder/FileName.js or /AssemblyName;component/Folder/FileName.js
                 * @return  {System.Threading.Tasks.Task$1}           Nothing.
                 */
                LoadJavaScriptFile$1: function (url) {
                    var callerAssemblyName = CSHTML5.Internal.INTERNAL_UriHelper.GetJavaScriptCallingAssembly();

                    var t = new System.Threading.Tasks.TaskCompletionSource();
                    CSHTML5.INTERNAL_InteropImplementation.LoadJavaScriptFile(url, callerAssemblyName, function () {
                        t.setResult(null);
                    }, function () {
                        t.setException(new System.Exception("Could not load file: \"" + (url || "") + "\"."));
                    });
                    return t.task;
                },
                LoadJavaScriptFile: function (resourceFile) {
                    if (!CSHTML5.Interop._jsScriptFileKeys.contains(resourceFile.Key)) {
                        var callerAssemblyName = CSHTML5.Internal.INTERNAL_UriHelper.GetJavaScriptCallingAssembly();

                        var t = new System.Threading.Tasks.TaskCompletionSource();
                        CSHTML5.INTERNAL_InteropImplementation.LoadJavaScriptFile(resourceFile.Url, callerAssemblyName, function () {
                            t.setResult(null);
                            CSHTML5.Interop._jsScriptFileKeys.add(resourceFile.Key);
                        }, function () {
                            t.setException(new System.Exception("Could not load file: \"" + (resourceFile.Url || "") + "\"."));
                        });

                        return t.task;
                    }

                    var task = new System.Threading.Tasks.TaskCompletionSource();
                    task.setResult(null);
                    return task.task;
                },
                /**
                 * Loads a list of JavaScript files from either an online location (http/https) or the local project. Note: This method will stop at the first script it cannot load, meaning that all subsequent scripts will not be loaded.
                 *
                 * @static
                 * @public
                 * @this CSHTML5.Interop
                 * @memberof CSHTML5.Interop
                 * @param   {System.Collections.Generic.IEnumerable$1}    urls               The URLs of the JavaScript files, with the syntax ms-appx:AssemblyName/Folder/FileName.js or /AssemblyName;component/Folder/FileName.js or https://someAddress/FileName.js
                 * @param   {System.Action}                               callback           The method that is called when all the files have successfully finished loading.
                 * @param   {System.Action}                               callbackOnError    The method that is called when one of the files could not be loaded.
                 * @return  {void}
                 */
                LoadJavaScriptFilesAsync$1: function (urls, callback, callbackOnError) {
                    if (callbackOnError === void 0) { callbackOnError = null; }
                    var callerAssemblyName = CSHTML5.Internal.INTERNAL_UriHelper.GetJavaScriptCallingAssembly();
                    var urlsAsList = (Bridge.is(urls, System.Collections.Generic.List$1(System.String)) ? Bridge.cast(urls, System.Collections.Generic.List$1(System.String)) : new (System.Collections.Generic.List$1(System.String)).$ctor1(urls));
                    CSHTML5.INTERNAL_InteropImplementation.LoadJavaScriptFiles(urlsAsList, callerAssemblyName, callback, callbackOnError);
                },
                LoadJavaScriptFilesAsync: function (resourceFiles, callback, callbackOnError) {
                    var $t;
                    if (callbackOnError === void 0) { callbackOnError = null; }
                    var callerAssemblyName = CSHTML5.Internal.INTERNAL_UriHelper.GetJavaScriptCallingAssembly();
                    var urlsAsList = new (System.Collections.Generic.List$1(System.String)).ctor();
                    $t = Bridge.getEnumerator(resourceFiles, CSHTML5.Interop.ResourceFile);
                    try {
                        while ($t.moveNext()) {
                            var resourceFile = $t.Current;
                            if (!CSHTML5.Interop._jsScriptFileKeys.contains(resourceFile.Key)) {
                                CSHTML5.Interop._jsScriptFileKeys.add(resourceFile.Key);
                                urlsAsList.add(resourceFile.Url);
                            }
                        }
                    } finally {
                        if (Bridge.is($t, System.IDisposable)) {
                            $t.System$IDisposable$Dispose();
                        }
                    }
                    CSHTML5.INTERNAL_InteropImplementation.LoadJavaScriptFiles(urlsAsList, callerAssemblyName, callback, callbackOnError);
                },
                /**
                 * Adds a 'link' tag to the HTML page and waits for the CSS file to finish loading.
                 *
                 * @static
                 * @public
                 * @this CSHTML5.Interop
                 * @memberof CSHTML5.Interop
                 * @param   {string}                           url    The URL of the CSS file, with the syntax ms-appx:AssemblyName/Folder/FileName.css or /AssemblyName;component/Folder/FileName.css
                 * @return  {System.Threading.Tasks.Task$1}           Nothing
                 */
                LoadCssFile$1: function (url) {
                    var t = new System.Threading.Tasks.TaskCompletionSource();
                    CSHTML5.INTERNAL_InteropImplementation.LoadCssFile(url, function () {
                        t.setResult(null);
                    });
                    return t.task;
                },
                LoadCssFile: function (resourceFile) {
                    if (!CSHTML5.Interop._cssFileKeys.contains(resourceFile.Key)) {
                        var t = new System.Threading.Tasks.TaskCompletionSource();
                        CSHTML5.INTERNAL_InteropImplementation.LoadCssFile(resourceFile.Url, function () {
                            t.setResult(null);
                        });
                        CSHTML5.Interop._cssFileKeys.add(resourceFile.Key);
                        return t.task;
                    }

                    var task = new System.Threading.Tasks.TaskCompletionSource();
                    task.setResult(null);
                    return task.task;
                },
                /**
                 * Adds 'link' tags to the HTML page and waits for the CSS files to finish loading.
                 *
                 * @static
                 * @public
                 * @this CSHTML5.Interop
                 * @memberof CSHTML5.Interop
                 * @param   {System.Collections.Generic.IEnumerable$1}    urls        The URL of the CSS files, with the syntax ms-appx:AssemblyName/Folder/FileName.css or /AssemblyName;component/Folder/FileName.css
                 * @param   {System.Action}                               callback    The method that is called when the CSS files have finished loading.
                 * @return  {void}
                 */
                LoadCssFilesAsync$1: function (urls, callback) {
                    var urlsAsList = (Bridge.is(urls, System.Collections.Generic.List$1(System.String)) ? Bridge.cast(urls, System.Collections.Generic.List$1(System.String)) : new (System.Collections.Generic.List$1(System.String)).$ctor1(urls));
                    CSHTML5.INTERNAL_InteropImplementation.LoadCssFiles(urlsAsList, callback);
                },
                LoadCssFilesAsync: function (resourceFiles, callback) {
                    var $t;
                    var urlsAsList = new (System.Collections.Generic.List$1(System.String)).ctor();
                    $t = Bridge.getEnumerator(resourceFiles, CSHTML5.Interop.ResourceFile);
                    try {
                        while ($t.moveNext()) {
                            var resourceFile = $t.Current;
                            if (!CSHTML5.Interop._cssFileKeys.contains(resourceFile.Key)) {
                                CSHTML5.Interop._cssFileKeys.add(resourceFile.Key);
                                urlsAsList.add(resourceFile.Url);
                            }
                        }
                    } finally {
                        if (Bridge.is($t, System.IDisposable)) {
                            $t.System$IDisposable$Dispose();
                        }
                    }
                    CSHTML5.INTERNAL_InteropImplementation.LoadCssFiles(urlsAsList, callback);
                },
                /**
                 * Returns the HTML Div that is associated to the specified FrameworkElement.
                 Note: the FrameworkElement must be in the visual tree. Consider calling this
                 method from the 'Loaded' event to ensure that the element is in the visual tree.
                 *
                 * @static
                 * @public
                 * @this CSHTML5.Interop
                 * @memberof CSHTML5.Interop
                 * @param   {Windows.UI.Xaml.FrameworkElement}    frameworkElement
                 * @return  {System.Object}
                 */
                GetDiv: function (frameworkElement) {
                    if (frameworkElement.INTERNAL_OuterDomElement == null) {
                        throw new System.InvalidOperationException.$ctor1("Cannot get the Div because the control is not in the visual tree. Consider waiting until the 'FrameworkElement.Loaded' event is fired before calling this piece of code.");
                    }

                    if (CSHTML5.Interop.IsRunningInTheSimulator) {
                        return frameworkElement.INTERNAL_OuterDomElement;
                    } else {
                        return frameworkElement.INTERNAL_OuterDomElement;
                    }
                }
            }
        }
    });

    Bridge.define("CSHTML5.Interop.ResourceFile", {
        $kind: "nested class",
        fields: {
            /**
             * The Key associated with the file. This is used to avoid loading a file multiple times or loading multiple files with the same purpose (for example JQuery).
             *
             * @instance
             * @public
             * @memberof CSHTML5.Interop.ResourceFile
             * @function Key
             * @type string
             */
            Key: null,
            /**
             * The path to the file.
             *
             * @instance
             * @public
             * @memberof CSHTML5.Interop.ResourceFile
             * @function Url
             * @type string
             */
            Url: null
        },
        ctors: {
            ctor: function (key, url) {
                this.$initialize();
                this.Key = key;
                this.Url = url;
            }
        }
    });

    /**
     * The {@link } namespace contains classes that are specific to
     C#/XAML for HTML5, such as the 'Interop' class for executing JavaScript code
     from within C#.
     *
     * @class CSHTML5.NamespaceDoc
     */
    Bridge.define("CSHTML5.NamespaceDoc");

    /** @namespace Windows.UI.Xaml.Controls */

    /**
     * @memberof Windows.UI.Xaml.Controls
     * @callback Windows.UI.Xaml.Controls.ContextMenuEventHandler
     * @param   {System.Object}                                    sender    
     * @param   {Windows.UI.Xaml.Controls.ContextMenuEventArgs}    e
     * @return  {void}
     */

    /** @namespace CSHTML5.Native.Html.Controls */

    /**
     * Defines an element which can be in a HtmlCanvas. It cannot be instantiated, but
     it provides some common properties. Every HtmlCanvas child is an HtmlCanvasElement.
     Full documentation is available at:
     http://cshtml5.com/links/how-to-use-the-html5-canvas.aspx
     *
     * @abstract
     * @public
     * @class CSHTML5.Native.Html.Controls.HtmlCanvasElement
     */
    Bridge.define("CSHTML5.Native.Html.Controls.HtmlCanvasElement", {
        statics: {
            methods: {
                ConvertColorToHtml: function (color) {
                    return "rgba(" + color.R + ", " + color.G + ", " + color.B + ", " + (System.String.replaceAll(System.Double.format((color.A / 255)), String.fromCharCode(44), String.fromCharCode(46)) || "") + ")";
                }
            }
        },
        fields: {
            IsHitTestVisible: false,
            HorizontalAlignment: 0,
            VerticalAlignment: 0,
            Opacity: 0,
            ActualWidth: 0,
            ActualHeight: 0,
            Visibility: 0,
            Cursor: null,
            Name: null,
            /**
             * X coordinate (relative to its parent)
             *
             * @instance
             * @public
             * @memberof CSHTML5.Native.Html.Controls.HtmlCanvasElement
             * @type number
             */
            X: 0,
            /**
             * Y coordinate (relative to its parent)
             *
             * @instance
             * @public
             * @memberof CSHTML5.Native.Html.Controls.HtmlCanvasElement
             * @type number
             */
            Y: 0,
            /**
             * Common style for this element
             *
             * @instance
             * @public
             * @memberof CSHTML5.Native.Html.Controls.HtmlCanvasElement
             * @type CSHTML5.Native.Html.Controls.ElementStyle
             */
            Style: null,
            _isStyleOverrided: false,
            _isVisible: null,
            _fillColor: null,
            _strokeColor: null,
            _shadowColor: null,
            _shadowBlur: null,
            _shadowOffsetX: null,
            _shadowOffsetY: null,
            _lineCap: null,
            _lineJoin: null,
            _lineWidth: null,
            _miterLimit: null,
            _fillStyleStr: null,
            _strokeStyleStr: null,
            _shadowColorStr: null,
            _toolTipContainer: null,
            _toolTip: null,
            _contextMenu: null
        },
        events: {
            PointerPressed: null,
            PointerMoved: null,
            PointerExited: null,
            PointerReleased: null,
            PointerEntered: null,
            RightTapped: null,
            /**
             * Occurs when any context menu on the element is opened.
             *
             * @instance
             * @public
             * @this CSHTML5.Native.Html.Controls.HtmlCanvasElement
             * @memberof CSHTML5.Native.Html.Controls.HtmlCanvasElement
             * @function addContextMenuOpening
             * @param   {Windows.UI.Xaml.Controls.ContextMenuEventHandler}    value
             * @return  {void}
             */
            /**
             * Occurs when any context menu on the element is opened.
             *
             * @instance
             * @public
             * @this CSHTML5.Native.Html.Controls.HtmlCanvasElement
             * @memberof CSHTML5.Native.Html.Controls.HtmlCanvasElement
             * @function removeContextMenuOpening
             * @param   {Windows.UI.Xaml.Controls.ContextMenuEventHandler}    value
             * @return  {void}
             */
            ContextMenuOpening: null
        },
        props: {
            /**
             * Visibility status
             *
             * @instance
             * @public
             * @memberof CSHTML5.Native.Html.Controls.HtmlCanvasElement
             * @function IsVisible
             * @type boolean
             */
            IsVisible: {
                get: function () {
                    return (this._isVisible == null) ? this.Style.IsVisible : System.Nullable.getValue(this._isVisible);
                },
                set: function (value) {
                    if (this._isVisible == null) {
                        this._isStyleOverrided = true;
                    }
                    this._isVisible = value;
                }
            },
            /**
             * Fill color
             *
             * @instance
             * @public
             * @memberof CSHTML5.Native.Html.Controls.HtmlCanvasElement
             * @function FillColor
             * @type Windows.UI.Color
             */
            FillColor: {
                get: function () {
                    return (System.Nullable.lifteq(Windows.UI.Color.op_Equality, System.Nullable.lift1("$clone", this._fillColor), null)) ? this.Style.FillColor : System.Nullable.getValue(this._fillColor);
                },
                set: function (value) {
                    if (System.Nullable.lifteq(Windows.UI.Color.op_Equality, System.Nullable.lift1("$clone", this._fillColor), null)) {
                        this._isStyleOverrided = true;
                    }
                    this._fillStyleStr = CSHTML5.Native.Html.Controls.HtmlCanvasElement.ConvertColorToHtml(value.$clone());
                    this._fillColor = value.$clone();
                }
            },
            /**
             * Stroke color
             *
             * @instance
             * @public
             * @memberof CSHTML5.Native.Html.Controls.HtmlCanvasElement
             * @function StrokeColor
             * @type Windows.UI.Color
             */
            StrokeColor: {
                get: function () {
                    return (System.Nullable.lifteq(Windows.UI.Color.op_Equality, System.Nullable.lift1("$clone", this._strokeColor), null)) ? this.Style.StrokeColor : System.Nullable.getValue(this._strokeColor);
                },
                set: function (value) {
                    if (System.Nullable.lifteq(Windows.UI.Color.op_Equality, System.Nullable.lift1("$clone", this._strokeColor), null)) {
                        this._isStyleOverrided = true;
                    }
                    this._strokeStyleStr = CSHTML5.Native.Html.Controls.HtmlCanvasElement.ConvertColorToHtml(value.$clone());
                    this._strokeColor = value.$clone();
                }
            },
            /**
             * Shadow color
             *
             * @instance
             * @public
             * @memberof CSHTML5.Native.Html.Controls.HtmlCanvasElement
             * @function ShadowColor
             * @type Windows.UI.Color
             */
            ShadowColor: {
                get: function () {
                    return (System.Nullable.lifteq(Windows.UI.Color.op_Equality, System.Nullable.lift1("$clone", this._shadowColor), null)) ? this.Style.ShadowColor : System.Nullable.getValue(this._shadowColor);
                },
                set: function (value) {
                    if (System.Nullable.lifteq(Windows.UI.Color.op_Equality, System.Nullable.lift1("$clone", this._shadowColor), null)) {
                        this._isStyleOverrided = true;
                    }
                    this._shadowColorStr = CSHTML5.Native.Html.Controls.HtmlCanvasElement.ConvertColorToHtml(value.$clone());
                    this._shadowColor = value.$clone();
                }
            },
            /**
             * Shadow blur
             *
             * @instance
             * @public
             * @memberof CSHTML5.Native.Html.Controls.HtmlCanvasElement
             * @function ShadowBlur
             * @type number
             */
            ShadowBlur: {
                get: function () {
                    return (this._shadowBlur == null) ? this.Style.ShadowBlur : System.Nullable.getValue(this._shadowBlur);
                },
                set: function (value) {
                    if (this._shadowBlur == null) {
                        this._isStyleOverrided = true;
                    }
                    this._shadowBlur = value;
                }
            },
            /**
             * Shadow horizontal offset
             *
             * @instance
             * @public
             * @memberof CSHTML5.Native.Html.Controls.HtmlCanvasElement
             * @function ShadowOffsetX
             * @type number
             */
            ShadowOffsetX: {
                get: function () {
                    return (this._shadowOffsetX == null) ? this.Style.ShadowOffsetX : System.Nullable.getValue(this._shadowOffsetX);
                },
                set: function (value) {
                    if (this._shadowOffsetX == null) {
                        this._isStyleOverrided = true;
                    }
                    this._shadowOffsetX = value;
                }
            },
            /**
             * Shadow vertical offset
             *
             * @instance
             * @public
             * @memberof CSHTML5.Native.Html.Controls.HtmlCanvasElement
             * @function ShadowOffsetY
             * @type number
             */
            ShadowOffsetY: {
                get: function () {
                    return (this._shadowOffsetY == null) ? this.Style.ShadowOffsetY : System.Nullable.getValue(this._shadowOffsetY);
                },
                set: function (value) {
                    if (this._shadowOffsetY == null) {
                        this._isStyleOverrided = true;
                    }
                    this._shadowOffsetY = value;
                }
            },
            /**
             * Style of the end caps for a line
             *
             * @instance
             * @public
             * @memberof CSHTML5.Native.Html.Controls.HtmlCanvasElement
             * @function LineCap
             * @type CSHTML5.Native.Html.Controls.LineCap
             */
            LineCap: {
                get: function () {
                    return (this._lineCap == null) ? this.Style.LineCap : System.Nullable.getValue(this._lineCap);
                },
                set: function (value) {
                    if (this._lineCap == null) {
                        this._isStyleOverrided = true;
                    }
                    this._lineCap = value;
                }
            },
            /**
             * Type of corner created, when two lines meet
             *
             * @instance
             * @public
             * @memberof CSHTML5.Native.Html.Controls.HtmlCanvasElement
             * @function LineJoin
             * @type CSHTML5.Native.Html.Controls.LineJoin
             */
            LineJoin: {
                get: function () {
                    return (this._lineJoin == null) ? this.Style.LineJoin : System.Nullable.getValue(this._lineJoin);
                },
                set: function (value) {
                    if (this._lineJoin == null) {
                        this._isStyleOverrided = true;
                    }
                    this._lineJoin = value;
                }
            },
            /**
             * Line Width
             *
             * @instance
             * @public
             * @memberof CSHTML5.Native.Html.Controls.HtmlCanvasElement
             * @function LineWidth
             * @type number
             */
            LineWidth: {
                get: function () {
                    return (this._lineWidth == null) ? this.Style.LineWidth : System.Nullable.getValue(this._lineWidth);
                },
                set: function (value) {
                    if (this._lineWidth == null) {
                        this._isStyleOverrided = true;
                    }
                    this._lineWidth = value;
                }
            },
            /**
             * Maximum miter length
             *
             * @instance
             * @public
             * @memberof CSHTML5.Native.Html.Controls.HtmlCanvasElement
             * @function MiterLimit
             * @type number
             */
            MiterLimit: {
                get: function () {
                    return (this._miterLimit == null) ? this.Style.MiterLimit : System.Nullable.getValue(this._miterLimit);
                },
                set: function (value) {
                    if (this._miterLimit == null) {
                        this._isStyleOverrided = true;
                    }
                    this._miterLimit = value;
                }
            },
            /**
             * (Deprecated) Access to StrokeColor with Brush instead of Color
             *
             * @instance
             * @public
             * @memberof CSHTML5.Native.Html.Controls.HtmlCanvasElement
             * @function Stroke
             * @type Windows.UI.Xaml.Media.Brush
             */
            Stroke: {
                get: function () {
                    return Windows.UI.Color.op_Explicit(this.StrokeColor);
                },
                set: function (value) {
                    if (Bridge.is(value, Windows.UI.Xaml.Media.SolidColorBrush)) {
                        this.StrokeColor = Bridge.cast(value, Windows.UI.Xaml.Media.SolidColorBrush).Color.$clone();
                    }
                }
            },
            /**
             * (Deprecated) Access to FillColor with Brush instead of Color
             *
             * @instance
             * @public
             * @memberof CSHTML5.Native.Html.Controls.HtmlCanvasElement
             * @function Background
             * @type Windows.UI.Xaml.Media.Brush
             */
            Background: {
                get: function () {
                    return Windows.UI.Color.op_Explicit(this.FillColor);
                },
                set: function (value) {
                    if (Bridge.is(value, Windows.UI.Xaml.Media.SolidColorBrush)) {
                        this.FillColor = Bridge.cast(value, Windows.UI.Xaml.Media.SolidColorBrush).Color.$clone();
                    }
                }
            },
            /**
             * (Deprecated) Access to FillColor with Brush instead of Color
             *
             * @instance
             * @public
             * @memberof CSHTML5.Native.Html.Controls.HtmlCanvasElement
             * @function Fill
             * @type Windows.UI.Xaml.Media.Brush
             */
            Fill: {
                get: function () {
                    return Windows.UI.Color.op_Explicit(this.FillColor);
                },
                set: function (value) {
                    if (Bridge.is(value, Windows.UI.Xaml.Media.SolidColorBrush)) {
                        this.FillColor = Bridge.cast(value, Windows.UI.Xaml.Media.SolidColorBrush).Color.$clone();
                    }
                }
            },
            /**
             * (Deprecated) Exactly the same as LineWidth
             *
             * @instance
             * @public
             * @memberof CSHTML5.Native.Html.Controls.HtmlCanvasElement
             * @function StrokeThickness
             * @type number
             */
            StrokeThickness: {
                get: function () {
                    return this.LineWidth;
                },
                set: function (value) {
                    this.LineWidth = value;
                }
            },
            ToolTip: {
                get: function () {
                    return this._toolTip;
                },
                set: function (value) {
                    if (!Bridge.referenceEquals(value, this._toolTip)) {
                        if (this._toolTip != null) {
                            this.removePointerEntered(Bridge.fn.cacheBind(this, this.ToolTipOwner_PointerEntered));
                            this.removePointerExited(Bridge.fn.cacheBind(this, this.ToolTipOwner_PointerExited));

                            if (this._toolTipContainer != null) {
                                this._toolTipContainer.INTERNAL_HtmlCanvasElementToWhichThisToolTipIsAssigned = null;
                            }
                        }

                        this._toolTip = value;

                        if (value != null) {
                            this._toolTipContainer = Windows.UI.Xaml.Controls.ToolTipService.ConvertToToolTip(value);
                            this._toolTipContainer.INTERNAL_HtmlCanvasElementToWhichThisToolTipIsAssigned = this;

                            this.removePointerEntered(Bridge.fn.cacheBind(this, this.ToolTipOwner_PointerEntered));
                            this.addPointerEntered(Bridge.fn.cacheBind(this, this.ToolTipOwner_PointerEntered));
                            this.removePointerExited(Bridge.fn.cacheBind(this, this.ToolTipOwner_PointerExited));
                            this.addPointerExited(Bridge.fn.cacheBind(this, this.ToolTipOwner_PointerExited));
                        }
                    }
                }
            },
            ContextMenu: {
                get: function () {
                    return this._contextMenu;
                },
                set: function (value) {
                    this._contextMenu = value;
                    Windows.UI.Xaml.Controls.INTERNAL_ContextMenuHelpers.RegisterContextMenu(this, this._contextMenu);
                }
            }
        },
        ctors: {
            ctor: function () {
                this.$initialize();
                this.Style = CSHTML5.Native.Html.Controls.ElementStyle.Default;
                this.IsHitTestVisible = true;
            },
            $ctor1: function (X, Y, Style) {
                if (Style === void 0) { Style = null; }

                this.$initialize();
                this.IsHitTestVisible = true;
                this.X = X;
                this.Y = Y;
                if (Style == null) {
                    this.Style = CSHTML5.Native.Html.Controls.ElementStyle.Default;
                } else {
                    this.Style = Style;
                }
            }
        },
        methods: {
            ToolTipOwner_PointerEntered: function (sender, e) {
                if (this._toolTipContainer != null && this._toolTipContainer.IsOpen === false) {
                    var absoluteCoordinates = e.GetCurrentPoint$1(null).Position.$clone();
                    var absoluteCoordinatesShiftedToBeBelowThePointer = new Windows.Foundation.Point.$ctor1(absoluteCoordinates.X, absoluteCoordinates.Y + 20);
                    this._toolTipContainer.INTERNAL_OpenAtCoordinates(absoluteCoordinatesShiftedToBeBelowThePointer.$clone());
                }
            },
            ToolTipOwner_PointerExited: function (sender, e) {
                if (this._toolTipContainer != null && this._toolTipContainer.IsOpen === true) {
                    this._toolTipContainer.IsOpen = false;
                }
            },
            /**
             * Moves the element relatively to its position
             *
             * @instance
             * @public
             * @this CSHTML5.Native.Html.Controls.HtmlCanvasElement
             * @memberof CSHTML5.Native.Html.Controls.HtmlCanvasElement
             * @param   {number}    deltaX    X movement
             * @param   {number}    deltaY    Y movement
             * @return  {void}
             */
            Move: function (deltaX, deltaY) {
                this.X += deltaX;
                this.Y += deltaY;
            },
            /**
             * Moves the element to an absolute position (still relative to its parent)
             *
             * @instance
             * @public
             * @this CSHTML5.Native.Html.Controls.HtmlCanvasElement
             * @memberof CSHTML5.Native.Html.Controls.HtmlCanvasElement
             * @param   {number}    x    X position
             * @param   {number}    y    Y position
             * @return  {void}
             */
            MoveTo: function (x, y) {
                this.X = x;
                this.Y = y;
            },
            /**
             * Apply the element's drawing style (if needed)
             *
             * @instance
             * @public
             * @this CSHTML5.Native.Html.Controls.HtmlCanvasElement
             * @memberof CSHTML5.Native.Html.Controls.HtmlCanvasElement
             * @param   {CSHTML5.Native.Html.Controls.ElementStyle}    oldStyle       Previous style used to draw
             * @param   {System.Object}                                jsContext2d    Canvas 2d javascript context
             * @return  {CSHTML5.Native.Html.Controls.ElementStyle}                   Current applied style (can be null)
             */
            ApplyStyle: function (oldStyle, jsContext2d) {
                if (this._isStyleOverrided) {

                    jsContext2d.fillStyle = (System.Nullable.liftne(Windows.UI.Color.op_Inequality, System.Nullable.lift1("$clone", this._fillColor), null)) ? this._fillStyleStr : this.Style.fillColorStr;
jsContext2d.strokeStyle = (System.Nullable.liftne(Windows.UI.Color.op_Inequality, System.Nullable.lift1("$clone", this._strokeColor), null)) ? this._strokeStyleStr : this.Style.strokeColorStr;
jsContext2d.shadowColor = (System.Nullable.liftne(Windows.UI.Color.op_Inequality, System.Nullable.lift1("$clone", this._shadowColor), null)) ? this._shadowColorStr : this.Style.shadowColorStr;
jsContext2d.shadowBlur = Bridge.box(this.ShadowBlur, System.Double, System.Double.format, System.Double.getHashCode);
jsContext2d.shadowOffsetX = Bridge.box(this.ShadowOffsetX, System.Double, System.Double.format, System.Double.getHashCode);
jsContext2d.shadowOffsetY = Bridge.box(this.ShadowOffsetY, System.Double, System.Double.format, System.Double.getHashCode);
jsContext2d.lineCap = System.Enum.toString(CSHTML5.Native.Html.Controls.LineCap, this.LineCap).toLowerCase();
jsContext2d.lineJoin = System.Enum.toString(CSHTML5.Native.Html.Controls.LineJoin, this.LineJoin).toLowerCase();
jsContext2d.lineWidth = Bridge.box(this.LineWidth, System.Double, System.Double.format, System.Double.getHashCode);


                    jsContext2d.miterLimit = Bridge.box(this.MiterLimit, System.Double, System.Double.format, System.Double.getHashCode);

                    return null;
                } else if (!Bridge.referenceEquals(this.Style, oldStyle)) {
                    this.Style.Apply(jsContext2d);
                    return this.Style;
                } else {
                    return oldStyle;
                }
            },
            StyleToString: function () {
                var s = "IsVisible = " + System.Boolean.toString(this.IsVisible) + "\n";
                s = (s || "") + (("fillStyle = '" + (this._fillStyleStr || "") + "'\n") || "");
                s = (s || "") + (("strokeStyle = '" + (this._strokeStyleStr || "") + "'\n") || "");
                s = (s || "") + (("shadowColor = '" + (this._shadowColorStr || "") + "'\n") || "");
                s = (s || "") + (("shadowBlur = " + System.Double.format(this.ShadowBlur) + "\n") || "");
                s = (s || "") + (("shadowOffsetX = " + System.Double.format(this.ShadowOffsetX) + "\n") || "");
                s = (s || "") + (("shadowOffsetY = " + System.Double.format(this.ShadowOffsetY) + "\n") || "");
                return s;
            },
            OnPointerMoved: function (e) {
                if (!Bridge.staticEquals(this.PointerMoved, null)) {
                    this.PointerMoved(this, e);
                }
            },
            OnPointerPressed: function (e) {
                if (!Bridge.staticEquals(this.PointerPressed, null)) {
                    this.PointerPressed(this, e);
                }
            },
            OnRightTapped: function (e) {
                if (!Bridge.staticEquals(this.RightTapped, null)) {
                    this.RightTapped(this, e);
                }
            },
            OnPointerReleased: function (e) {
                if (!Bridge.staticEquals(this.PointerReleased, null)) {
                    this.PointerReleased(this, e);
                }
            },
            OnPointerEntered: function (e) {
                if (!Bridge.staticEquals(this.PointerEntered, null)) {
                    this.PointerEntered(this, e);
                }
            },
            OnPointerExited: function (e) {
                if (!Bridge.staticEquals(this.PointerExited, null)) {
                    this.PointerExited(this, e);
                }
            },
            INTERNAL_RaiseContextMenuOpeningEvent: function (pointerLeft, pointerTop) {
                if (!Bridge.staticEquals(this.ContextMenuOpening, null)) {
                    this.ContextMenuOpening(this, new Windows.UI.Xaml.Controls.ContextMenuEventArgs(pointerLeft, pointerTop));
                }
            }
        }
    });

    /**
     * Provides a drawing style to an element
     *
     * @public
     * @class CSHTML5.Native.Html.Controls.ElementStyle
     */
    Bridge.define("CSHTML5.Native.Html.Controls.ElementStyle", {
        statics: {
            fields: {
                _Default: null
            },
            props: {
                /**
                 * Default style (can be modified)
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof CSHTML5.Native.Html.Controls.ElementStyle
                 * @function Default
                 * @type CSHTML5.Native.Html.Controls.ElementStyle
                 */
                Default: {
                    get: function () {
                        if (CSHTML5.Native.Html.Controls.ElementStyle._Default == null) {
                            CSHTML5.Native.Html.Controls.ElementStyle._Default = new CSHTML5.Native.Html.Controls.ElementStyle();
                        }
                        return CSHTML5.Native.Html.Controls.ElementStyle._Default;
                    }
                }
            },
            methods: {
                ConvertColorToHtml: function (color) {
                    return "rgba(" + color.R + ", " + color.G + ", " + color.B + ", " + (System.String.replaceAll(System.Double.format((color.A / 255)), String.fromCharCode(44), String.fromCharCode(46)) || "") + ")";
                }
            }
        },
        fields: {
            /**
             * Visibility of the element
             *
             * @instance
             * @public
             * @memberof CSHTML5.Native.Html.Controls.ElementStyle
             * @type boolean
             */
            IsVisible: false,
            fillColorStr: null,
            strokeColorStr: null,
            shadowColorStr: null,
            _fillColor: null,
            _strokeColor: null,
            _shadowColor: null,
            /**
             * Shadow blur
             *
             * @instance
             * @public
             * @memberof CSHTML5.Native.Html.Controls.ElementStyle
             * @function ShadowBlur
             * @type number
             */
            ShadowBlur: 0,
            /**
             * Shadow horizontal offset
             *
             * @instance
             * @public
             * @memberof CSHTML5.Native.Html.Controls.ElementStyle
             * @function ShadowOffsetX
             * @type number
             */
            ShadowOffsetX: 0,
            /**
             * Shadow vertical offset
             *
             * @instance
             * @public
             * @memberof CSHTML5.Native.Html.Controls.ElementStyle
             * @function ShadowOffsetY
             * @type number
             */
            ShadowOffsetY: 0,
            /**
             * Style of the end caps for a line
             *
             * @instance
             * @public
             * @memberof CSHTML5.Native.Html.Controls.ElementStyle
             * @function LineCap
             * @type CSHTML5.Native.Html.Controls.LineCap
             */
            LineCap: 0,
            /**
             * Type of corner created, when two lines meet
             *
             * @instance
             * @public
             * @memberof CSHTML5.Native.Html.Controls.ElementStyle
             * @function LineJoin
             * @type CSHTML5.Native.Html.Controls.LineJoin
             */
            LineJoin: 0,
            /**
             * Line Width
             *
             * @instance
             * @public
             * @memberof CSHTML5.Native.Html.Controls.ElementStyle
             * @function LineWidth
             * @type number
             */
            LineWidth: 0,
            /**
             * Maximum miter length
             *
             * @instance
             * @public
             * @memberof CSHTML5.Native.Html.Controls.ElementStyle
             * @function MiterLimit
             * @type number
             */
            MiterLimit: 0
        },
        props: {
            /**
             * Fill color
             *
             * @instance
             * @public
             * @memberof CSHTML5.Native.Html.Controls.ElementStyle
             * @function FillColor
             * @type Windows.UI.Color
             */
            FillColor: {
                get: function () {
                    return this._fillColor.$clone();
                },
                set: function (value) {
                    this.fillColorStr = CSHTML5.Native.Html.Controls.ElementStyle.ConvertColorToHtml(value.$clone());
                    this._fillColor = value.$clone();
                }
            },
            /**
             * Stroke color
             *
             * @instance
             * @public
             * @memberof CSHTML5.Native.Html.Controls.ElementStyle
             * @function StrokeColor
             * @type Windows.UI.Color
             */
            StrokeColor: {
                get: function () {
                    return this._strokeColor.$clone();
                },
                set: function (value) {
                    this.strokeColorStr = CSHTML5.Native.Html.Controls.ElementStyle.ConvertColorToHtml(value.$clone());
                    this._strokeColor = value.$clone();
                }
            },
            /**
             * Shadow color
             *
             * @instance
             * @public
             * @memberof CSHTML5.Native.Html.Controls.ElementStyle
             * @function ShadowColor
             * @type Windows.UI.Color
             */
            ShadowColor: {
                get: function () {
                    return this._shadowColor.$clone();
                },
                set: function (value) {
                    this.shadowColorStr = CSHTML5.Native.Html.Controls.ElementStyle.ConvertColorToHtml(value.$clone());
                    this._shadowColor = value.$clone();
                }
            }
        },
        ctors: {
            init: function () {
                this._fillColor = new Windows.UI.Color();
                this._strokeColor = new Windows.UI.Color();
                this._shadowColor = new Windows.UI.Color();
            },
            /**
             * Empty style constructor
             *
             * @instance
             * @public
             * @this CSHTML5.Native.Html.Controls.ElementStyle
             * @memberof CSHTML5.Native.Html.Controls.ElementStyle
             * @return  {void}
             */
            ctor: function () {
                this.$initialize();
                this.FillColor = Windows.UI.Color.FromArgb(0, 0, 0, 0);
                this.StrokeColor = Windows.UI.Color.FromArgb(0, 0, 0, 0);
                this.ShadowColor = Windows.UI.Color.FromArgb(0, 0, 0, 0);
                this.LineWidth = 1;
                this.MiterLimit = 10;
            }
        },
        methods: {
            /**
             * Apply this style as the current drawing style
             *
             * @instance
             * @this CSHTML5.Native.Html.Controls.ElementStyle
             * @memberof CSHTML5.Native.Html.Controls.ElementStyle
             * @param   {System.Object}    jsContext2d    Canvas 2d javascript context
             * @return  {void}
             */
            Apply: function (jsContext2d) {

                jsContext2d.fillStyle = this.fillColorStr;
jsContext2d.strokeStyle = this.strokeColorStr;
jsContext2d.shadowColor = this.shadowColorStr;
jsContext2d.shadowBlur = Bridge.box(this.ShadowBlur, System.Double, System.Double.format, System.Double.getHashCode);
jsContext2d.shadowOffsetX = Bridge.box(this.ShadowOffsetX, System.Double, System.Double.format, System.Double.getHashCode);
jsContext2d.shadowOffsetY = Bridge.box(this.ShadowOffsetY, System.Double, System.Double.format, System.Double.getHashCode);
jsContext2d.lineCap = System.Enum.toString(CSHTML5.Native.Html.Controls.LineCap, this.LineCap).toLowerCase();
jsContext2d.lineJoin = System.Enum.toString(CSHTML5.Native.Html.Controls.LineJoin, this.LineJoin).toLowerCase();
jsContext2d.lineWidth = Bridge.box(this.LineWidth, System.Double, System.Double.format, System.Double.getHashCode);


                jsContext2d.miterLimit = Bridge.box(this.MiterLimit, System.Double, System.Double.format, System.Double.getHashCode);
            }
        }
    });

    /**
     * Line cap type
     *
     * @public
     * @class CSHTML5.Native.Html.Controls.LineCap
     */
    Bridge.define("CSHTML5.Native.Html.Controls.LineCap", {
        $kind: "enum",
        statics: {
            fields: {
                Butt: 0,
                Round: 1,
                Square: 2
            }
        }
    });

    /**
     * Line join type
     *
     * @public
     * @class CSHTML5.Native.Html.Controls.LineJoin
     */
    Bridge.define("CSHTML5.Native.Html.Controls.LineJoin", {
        $kind: "enum",
        statics: {
            fields: {
                Bevel: 0,
                Round: 1,
                Miter: 2
            }
        }
    });

    Bridge.define("CSHTML5.Native.Html.Controls.NativeComboBox.SelectionInfo", {
        $kind: "nested class",
        fields: {
            Index: 0,
            Item: null
        },
        ctors: {
            ctor: function (item, index) {
                this.$initialize();
                this.Item = item;
                this.Index = index;
            }
        }
    });

    /**
     * Contains state information and event data associated with a routed event.
     *
     * @public
     * @class Windows.UI.Xaml.RoutedEventArgs
     */
    Bridge.define("Windows.UI.Xaml.RoutedEventArgs", {
        fields: {
            _originalSource: null,
            _originalJSEventArg: null
        },
        props: {
            /**
             * Gets a reference to the object that raised the event.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.RoutedEventArgs
             * @function OriginalSource
             * @type System.Object
             */
            OriginalSource: {
                get: function () {
                    if (this._originalSource != null) {
                        return this._originalSource;
                    } else if (this._originalJSEventArg != null) {
                        var jsTarget = this._originalJSEventArg.target || this._originalJSEventArg.srcElement;
                        var correspondingUiElementIfFound = CSHTML5.Internal.INTERNAL_HtmlDomManager.GetUIElementFromDomElement(jsTarget);
                        return correspondingUiElementIfFound;
                    } else {
                        return null;
                    }
                },
                set: function (value) {
                    if (value == null || !(Bridge.is(value, Windows.UI.Xaml.DependencyObject))) {
                        throw new System.ArgumentException.ctor();
                    }
                    this._originalSource = Bridge.cast(value, Windows.UI.Xaml.DependencyObject);
                }
            },
            /**
             * (Optional) Gets the original javascript event arg.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.RoutedEventArgs
             * @function INTERNAL_OriginalJSEventArg
             * @type System.Object
             */
            INTERNAL_OriginalJSEventArg: {
                get: function () {
                    return this._originalJSEventArg;
                },
                set: function (value) {
                    this._originalJSEventArg = value;
                }
            }
        },
        ctors: {
            /**
             * Initializes a new instance of the RoutedEventArgs class.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.RoutedEventArgs
             * @memberof Windows.UI.Xaml.RoutedEventArgs
             * @return  {void}
             */
            ctor: function () {
                this.$initialize();
            }
        }
    });

    /** @namespace CSHTML5.Native.Html.Printing */

    /**
     * Provides methods to print.
     *
     * @static
     * @abstract
     * @public
     * @class CSHTML5.Native.Html.Printing.PrintManager
     */
    Bridge.define("CSHTML5.Native.Html.Printing.PrintManager", {
        statics: {
            fields: {
                CurrentPrintArea: null,
                IsDefaultPrintArea: false
            },
            ctors: {
                init: function () {
                    this.IsDefaultPrintArea = true;
                }
            },
            methods: {
                /**
                 * Make the browser Print dialog appear.
                 *
                 * @static
                 * @public
                 * @this CSHTML5.Native.Html.Printing.PrintManager
                 * @memberof CSHTML5.Native.Html.Printing.PrintManager
                 * @return  {void}
                 */
                Print: function () {
                    if (!CSHTML5.Interop.IsRunningInTheSimulator) {
                        window.print();
                    } else {
                        Windows.UI.Xaml.MessageBox.Show("The application requested to show the print dialog. This feature is not implemented in the Simulator. Please run the application in the browser instead.");
                    }
                },
                /**
                 * Print a specific element.
                 *
                 * @static
                 * @public
                 * @this CSHTML5.Native.Html.Printing.PrintManager
                 * @memberof CSHTML5.Native.Html.Printing.PrintManager
                 * @param   {Windows.UI.Xaml.FrameworkElement}    element    The element to print.
                 * @return  {void}
                 */
                Print$1: function (element) {
                    var $t;
                    if (!CSHTML5.Interop.IsRunningInTheSimulator) {
                        var previousPrintArea = CSHTML5.Native.Html.Printing.PrintManager.CurrentPrintArea;

                        if (element._isLoaded) {

                            CSHTML5.Native.Html.Printing.PrintManager.SetPrintArea(element);

                            CSHTML5.Native.Html.Printing.PrintManager.Print();

                            CSHTML5.Native.Html.Printing.PrintManager.RestorePreviousPrintArea(previousPrintArea);
                        } else {

                            var temporaryPopup = ($t = new Windows.UI.Xaml.Controls.Primitives.Popup(), $t.VerticalOffset = 10000, $t);
                            temporaryPopup.IsOpen = true;

                            var container = ($t = new Windows.UI.Xaml.Controls.Border(), $t.Child = element, $t);

                            container.addLoaded(function (s2, e2) {
                                CSHTML5.Native.Html.Printing.PrintManager.Print$1(element);

                                CSHTML5.Native.Html.Printing.PrintManager.RestorePreviousPrintArea(previousPrintArea);

                                Windows.UI.Core.CoreDispatcher.INTERNAL_GetCurrentDispatcher().BeginInvoke(function () {
                                    temporaryPopup.IsOpen = false;
                                });
                            });

                            temporaryPopup.Child = container;
                            temporaryPopup.IsOpen = true;
                        }
                    } else {
                        Windows.UI.Xaml.MessageBox.Show("The application requested to print. This feature is not implemented in the Simulator. Please run the application in the browser instead.");
                    }
                },
                /**
                 * Define the element that will be printed when pressing Ctrl+P or when clicking the Print command in the browser menu.
                 *
                 * @static
                 * @public
                 * @this CSHTML5.Native.Html.Printing.PrintManager
                 * @memberof CSHTML5.Native.Html.Printing.PrintManager
                 * @param   {Windows.UI.Xaml.FrameworkElement}    element    An element in in the Visual Tree.
                 * @return  {void}
                 */
                SetPrintArea: function (element) {
                    if (element != null) {
                        if (element._isLoaded) {
                            if (!CSHTML5.Interop.IsRunningInTheSimulator) {
                                if (CSHTML5.Native.Html.Printing.PrintManager.CurrentPrintArea != null && CSHTML5.Native.Html.Printing.PrintManager.CurrentPrintArea._isLoaded) {
                                    CSHTML5.Native.Html.Printing.PrintManager.CurrentPrintArea.INTERNAL_OuterDomElement.classList.remove("section-to-print");
                                }

                                element.INTERNAL_OuterDomElement.classList.add("section-to-print");

                                CSHTML5.Native.Html.Printing.PrintManager.CurrentPrintArea = element;

                                CSHTML5.Native.Html.Printing.PrintManager.IsDefaultPrintArea = false;
                            } else {
                                Windows.UI.Xaml.MessageBox.Show("The application requested to set the print area. This feature is not implemented in the Simulator. Please run the application in the browser instead.");
                            }
                        } else {
                            throw new System.InvalidOperationException.$ctor1("You can only set the print area to an element that is visible on screen or that has been loaded in the Visual Tree.");
                        }
                    } else {
                        throw new System.ArgumentNullException.$ctor1("element");
                    }
                },
                /**
                 * Revert to the default print area, which is the area that will be printed when pressing Ctrl+P or when clicking the Print command in the browser menu.
                 *
                 * @static
                 * @public
                 * @this CSHTML5.Native.Html.Printing.PrintManager
                 * @memberof CSHTML5.Native.Html.Printing.PrintManager
                 * @return  {void}
                 */
                ResetPrintArea: function () {
                    if (!CSHTML5.Interop.IsRunningInTheSimulator) {
                        var root = Windows.UI.Xaml.Window.Current.Content;
                        if (root != null) {
                            CSHTML5.Native.Html.Printing.PrintManager.SetPrintArea(Bridge.cast(root, Windows.UI.Xaml.FrameworkElement));
                        } else {
                        }

                        CSHTML5.Native.Html.Printing.PrintManager.IsDefaultPrintArea = true;
                    } else {
                        Windows.UI.Xaml.MessageBox.Show("The application requested to reset the print area. This feature is not implemented in the Simulator. Please run the application in the browser instead.");
                    }
                },
                RestorePreviousPrintArea: function (previousPrintArea) {
                    if (previousPrintArea != null) {
                        CSHTML5.Native.Html.Printing.PrintManager.SetPrintArea(previousPrintArea);
                    } else {
                        CSHTML5.Native.Html.Printing.PrintManager.ResetPrintArea();
                    }
                }
            }
        }
    });

    Bridge.define("CSHTML5.Native.Html.UriHelper", {
        statics: {
            methods: {
                /**
                 * Converts a XAML-style URI (such as "ms-appx") into a path that can be used in HTML.
                 *
                 * @static
                 * @public
                 * @this CSHTML5.Native.Html.UriHelper
                 * @memberof CSHTML5.Native.Html.UriHelper
                 * @param   {string}    uri    A XAML-style URI such as ms-appx:AssemblyName/Folder/Image.png or /AssemblyName;component/Folder/Image.png
                 * @return  {string}           A path that can be used in HTML
                 */
                ConvertToHtmlPath: function (uri) {
                    return CSHTML5.Internal.INTERNAL_UriHelper.ConvertToHtml5Path(uri, null);
                }
            }
        }
    });

    Bridge.define("CSHTML5.Native.JavaScript.Networking.HttpMethod", {
        $kind: "enum",
        statics: {
            fields: {
                GET: 0,
                POST: 1,
                PUT: 2,
                DELETE: 3
            }
        }
    });

    Bridge.define("CSHTML5.Native.JavaScript.Networking.XhrReadyState", {
        $kind: "enum",
        statics: {
            fields: {
                UNSENT: 0,
                OPENED: 1,
                HEADERS_RECEIVED: 2,
                LOADING: 3,
                DONE: 4
            }
        }
    });

    Bridge.define("CSHTML5.Native.JavaScript.Networking.XMLHttpRequest", {
        statics: {
            fields: {
                _defaultTimeOut: 0
            },
            ctors: {
                init: function () {
                    this._defaultTimeOut = 2000;
                }
            },
            methods: {
                Send: function (xmlHttpRequest, body) {
                    var $t;
                    xmlHttpRequest.send(($t = body, $t != null ? $t : ""));
                },
                GetReadyState: function (xmlHttpRequest) {
                    return System.Convert.toInt32(xmlHttpRequest.readyState);
                },
                GetStatus: function (xmlHttpRequest) {
                    return System.Convert.toInt32(xmlHttpRequest.status);
                },
                GetResponse: function (xmlHttpRequest) {
                    return Bridge.toString(xmlHttpRequest.responseText);
                }
            }
        },
        fields: {
            _tcs: null,
            _xmlHttpRequest: null,
            Uri: null,
            Method: null,
            Headers: null,
            Body: null
        },
        ctors: {
            ctor: function () {
                this.$initialize();
                this.Method = System.Enum.toString(CSHTML5.Native.JavaScript.Networking.HttpMethod, CSHTML5.Native.JavaScript.Networking.HttpMethod.GET);
                this.Headers = new (System.Collections.Generic.Dictionary$2(System.String,System.String)).ctor();
            },
            $ctor2: function (uri) {
                CSHTML5.Native.JavaScript.Networking.XMLHttpRequest.ctor.call(this);
                this.Uri = uri;
            },
            $ctor1: function (address) {
                CSHTML5.Native.JavaScript.Networking.XMLHttpRequest.ctor.call(this);
                this.Uri = new System.Uri(address);
            }
        },
        methods: {
            Execute: function () {
                var $t;
                var xhr = this.NewHttpRequest();

                this.Open(xhr, this.Method, Bridge.toString(this.Uri), false);
                $t = Bridge.getEnumerator(this.Headers);
                try {
                    while ($t.moveNext()) {
                        var kv = $t.Current;
                        this.SetRequestHeader(xhr, kv.key, kv.value);
                    }
                } finally {
                    if (Bridge.is($t, System.IDisposable)) {
                        $t.System$IDisposable$Dispose();
                    }
                }

                CSHTML5.Native.JavaScript.Networking.XMLHttpRequest.Send(xhr, this.Body);

                this.CheckStatusAndThrow(xhr);

                var responseObject = CSHTML5.Native.JavaScript.Networking.XMLHttpRequest.GetResponse(xhr);
                return responseObject != null ? Bridge.toString(responseObject) : null;
            },
            ExecuteAsync: function (timeOut) {
                var $t;
                if (timeOut === void 0) { timeOut = 2000; }
                this._tcs = new System.Threading.Tasks.TaskCompletionSource();

                this._xmlHttpRequest = this.NewHttpRequest();

                this.OnLoad(this._xmlHttpRequest, Bridge.fn.cacheBind(this, this.InternalOnLoad));
                this.OnProgress(this._xmlHttpRequest, Bridge.fn.cacheBind(this, this.InternalOnProgress));
                this.OnError(this._xmlHttpRequest, Bridge.fn.cacheBind(this, this.InternalOnError));
                this.OnAbort(this._xmlHttpRequest, Bridge.fn.cacheBind(this, this.InternalOnAbort));
                this.OnTimeOut(this._xmlHttpRequest, Bridge.fn.cacheBind(this, this.InternalOnTimeOut));

                this.Open(this._xmlHttpRequest, this.Method, Bridge.toString(this.Uri), true);
                $t = Bridge.getEnumerator(this.Headers);
                try {
                    while ($t.moveNext()) {
                        var kv = $t.Current;
                        this.SetRequestHeader(this._xmlHttpRequest, kv.key, kv.value);
                    }
                } finally {
                    if (Bridge.is($t, System.IDisposable)) {
                        $t.System$IDisposable$Dispose();
                    }
                }

                this.SetTimout(this._xmlHttpRequest, timeOut);
                CSHTML5.Native.JavaScript.Networking.XMLHttpRequest.Send(this._xmlHttpRequest, this.Body);

                return this._tcs.task;
            },
            InternalOnProgress: function (e) { },
            InternalOnLoad: function (e) {
                var exception = this.CheckStatus(this._xmlHttpRequest);
                if (exception == null) {
                    var responseObject = CSHTML5.Native.JavaScript.Networking.XMLHttpRequest.GetResponse(this._xmlHttpRequest);
                    var response = responseObject != null ? Bridge.toString(responseObject) : null;
                    this._tcs.setResult(response);
                } else {
                    this._tcs.setException(exception);
                }
            },
            InternalOnError: function (e) {
                var exception = this.CheckStatus(this._xmlHttpRequest);
                if (exception == null) {
                    exception = new System.NotSupportedException.$ctor1("This error type is not yet supported!");
                }
                this._tcs.setException(exception);
            },
            InternalOnAbort: function (e) {
                var exception = this.CheckStatus(this._xmlHttpRequest);
                if (exception == null) {
                    exception = new System.NotSupportedException.$ctor1("This abort type is not yet supported!");
                }
                this._tcs.setException(exception);
            },
            InternalOnTimeOut: function (e) {
                var exception = new System.TimeoutException.ctor();
                this._tcs.setException(exception);
            },
            CheckStatusAndThrow: function (xhr) {
                var exception = this.CheckStatus(xhr);
                if (exception != null) {
                    throw exception;
                }
            },
            CheckStatus: function (xhr) {
                var readState = CSHTML5.Native.JavaScript.Networking.XMLHttpRequest.GetReadyState(xhr);
                var status = CSHTML5.Native.JavaScript.Networking.XMLHttpRequest.GetStatus(xhr);
                if (status === 0) {
                    return new System.Exception("no response from server or error");
                }
                if (status === 404) {
                    return new System.Exception("there was a response from the server; it was a 404 meaning resource not found");
                }
                if (status >= 500) {
                    return new System.Exception("there was a response from the server; it was a " + status + " meaning an error occured on the server");
                }
                if (status !== 200) {
                    return new System.Exception("there was a response from the server; it was a " + status);
                }
                switch (readState) {
                    case CSHTML5.Native.JavaScript.Networking.XhrReadyState.UNSENT: 
                        return new System.Exception("The request is not initialized!");
                    case CSHTML5.Native.JavaScript.Networking.XhrReadyState.OPENED: 
                        return new System.Exception("An error occured. Cross-Site Http Request might not be allowed at the target Url. If you own the domain of the Url, consider adding the header \"Access-Control-Allow-Origin\" to enable requests to be done at this Url.");
                    case CSHTML5.Native.JavaScript.Networking.XhrReadyState.HEADERS_RECEIVED: 
                    case CSHTML5.Native.JavaScript.Networking.XhrReadyState.LOADING: 
                        return new System.Exception("An error occured while submitting your request.");
                    case CSHTML5.Native.JavaScript.Networking.XhrReadyState.DONE: 
                        break;
                }
                return null;
            },
            NewHttpRequest: function () {
                return new XMLHttpRequest();
            },
            SetTimout: function (xmlHttpRequest, timeOut) {
                xmlHttpRequest.timeout = Bridge.box(timeOut, System.Int32);
            },
            SetRequestHeader: function (xmlHttpRequest, key, header) {
                xmlHttpRequest.setRequestHeader(key, header);
            },
            OnLoad: function (xmlHttpRequest, onLoad) {
                xmlHttpRequest.onload = onLoad;
            },
            OnProgress: function (xmlHttpRequest, onProgress) {
                xmlHttpRequest.onprogress = onProgress;
            },
            OnError: function (xmlHttpRequest, onError) {
                xmlHttpRequest.onerror = onError;
            },
            OnAbort: function (xmlHttpRequest, onAbort) {
                xmlHttpRequest.onabort = onAbort;
            },
            OnTimeOut: function (xmlHttpRequest, onTimeOut) {
                xmlHttpRequest.ontimeout = onTimeOut;
            },
            Open: function (xmlHttpRequest, method, address, isAsync) {
                xmlHttpRequest.open(method, address, Bridge.box(isAsync, System.Boolean, System.Boolean.toString));
            }
        }
    });

    Bridge.define("CSHTML5.Performance", {
        statics: {
            fields: {
                /**
                 * The backing store for the "EnableProgressiveLoading" attached property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof CSHTML5.Performance
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                EnableProgressiveLoadingProperty: null
            },
            ctors: {
                init: function () {
                    this.EnableProgressiveLoadingProperty = Windows.UI.Xaml.DependencyProperty.RegisterAttached("EnableProgressiveLoading", System.Boolean, CSHTML5.Performance, new Windows.UI.Xaml.PropertyMetadata.$ctor2(Bridge.box(false, System.Boolean, System.Boolean.toString), CSHTML5.Performance.EnableProgressiveLoading_Changed));
                }
            },
            methods: {
                /**
                 * @static
                 * @public
                 * @this CSHTML5.Performance
                 * @memberof CSHTML5.Performance
                 * @param   {Windows.UI.Xaml.DependencyObject}    obj
                 * @return  {boolean}
                 */
                GetEnableProgressiveLoading: function (obj) {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(obj.GetValue(CSHTML5.Performance.EnableProgressiveLoadingProperty), System.Boolean), System.Boolean));
                },
                /**
                 * Sets a value indicating whether a control should be rendered before its
                 children and whether the children should be rendered one at a time,
                 rather than loading everything at once. The default value is false.
                 *
                 * @static
                 * @public
                 * @this CSHTML5.Performance
                 * @memberof CSHTML5.Performance
                 * @param   {Windows.UI.Xaml.DependencyObject}    obj      
                 * @param   {boolean}                             value
                 * @return  {void}
                 */
                SetEnableProgressiveLoading: function (obj, value) {
                    obj.SetValue(CSHTML5.Performance.EnableProgressiveLoadingProperty, Bridge.box(value, System.Boolean, System.Boolean.toString));
                },
                EnableProgressiveLoading_Changed: function (d, e) {
                    if (Bridge.is(d, Windows.UI.Xaml.UIElement) && Bridge.is(e.NewValue, System.Boolean)) {
                        Bridge.cast(d, Windows.UI.Xaml.UIElement).INTERNAL_EnableProgressiveLoading = System.Nullable.getValue(Bridge.cast(Bridge.unbox(e.NewValue, System.Boolean), System.Boolean));
                    }
                }
            }
        }
    });

    /**
     * Provides static methods for profiling performance in the browser.
     *
     * @static
     * @abstract
     * @public
     * @class CSHTML5.Profiler
     */
    Bridge.define("CSHTML5.Profiler");

    Bridge.define("DotNetForHtml5.Core.INTERNAL_PopupsManager", {
        statics: {
            fields: {
                CurrentPopupRootIndentifier: 0,
                PopupRootIdentifierToInstance: null
            },
            ctors: {
                init: function () {
                    this.CurrentPopupRootIndentifier = 0;
                    this.PopupRootIdentifierToInstance = new (System.Collections.Generic.Dictionary$2(System.String,System.Object)).ctor();
                }
            },
            methods: {
                OnClickOnPopupOrWindow: function (sender, e) {
                    var $t, $t1, $t2;

                    var listOfPopupThatMustBeClosed = new (System.Collections.Generic.HashSet2$1(Windows.UI.Xaml.Controls.Primitives.Popup)).ctor();
                    var popupRootList = new (System.Collections.Generic.List$1(Windows.UI.Xaml.Controls.Primitives.PopupRoot)).ctor();

                    $t = Bridge.getEnumerator(DotNetForHtml5.Core.INTERNAL_PopupsManager.GetAllRootUIElements());
                    try {
                        while ($t.moveNext()) {
                            var obj = $t.Current;
                            if (Bridge.is(obj, Windows.UI.Xaml.Controls.Primitives.PopupRoot)) {
                                var root = Bridge.cast(obj, Windows.UI.Xaml.Controls.Primitives.PopupRoot);
                                popupRootList.add(root);

                                if (root.INTERNAL_LinkedPopup != null) {
                                    listOfPopupThatMustBeClosed.add(root.INTERNAL_LinkedPopup);
                                }
                            }
                        }
                    } finally {
                        if (Bridge.is($t, System.IDisposable)) {
                            $t.System$IDisposable$Dispose();
                        }
                    }

                    $t1 = Bridge.getEnumerator(popupRootList);
                    try {
                        while ($t1.moveNext()) {
                            var popupRoot = $t1.Current;
                            if (popupRoot.INTERNAL_LinkedPopup != null) {

                                var popup = popupRoot.INTERNAL_LinkedPopup;

                                if (popup.StayOpen || Bridge.referenceEquals(sender, popupRoot)) {
                                    do {
                                        if (!listOfPopupThatMustBeClosed.contains(popup)) {
                                            break;
                                        }

                                        listOfPopupThatMustBeClosed.remove(popup);

                                        popup = popup.ParentPopup;

                                    } while (popup != null);
                                }
                            }
                        }
                    } finally {
                        if (Bridge.is($t1, System.IDisposable)) {
                            $t1.System$IDisposable$Dispose();
                        }
                    }

                    $t2 = Bridge.getEnumerator(listOfPopupThatMustBeClosed);
                    try {
                        while ($t2.moveNext()) {
                            var popup1 = $t2.Current;
                            popup1.CloseFromAnOutsideClick();
                        }
                    } finally {
                        if (Bridge.is($t2, System.IDisposable)) {
                            $t2.System$IDisposable$Dispose();
                        }
                    }

                },
                CreateAndAppendNewPopupRoot: function (parentWindow) {
                    DotNetForHtml5.Core.INTERNAL_PopupsManager.CurrentPopupRootIndentifier = (DotNetForHtml5.Core.INTERNAL_PopupsManager.CurrentPopupRootIndentifier + 1) | 0;
                    var uniquePopupRootIdentifier = "INTERNAL_Cshtml5_PopupRoot_" + (Bridge.toString(DotNetForHtml5.Core.INTERNAL_PopupsManager.CurrentPopupRootIndentifier) || "");



var popupRoot = document.createElement('div');
popupRoot.setAttribute('id',                     uniquePopupRootIdentifier);
popupRoot.style.position = 'absolute';
popupRoot.style.width = '100%';
popupRoot.style.height = '100%';
popupRoot.style.overflowX = 'hidden';
popupRoot.style.overflowY = 'hidden';
popupRoot.style.pointerEvents = 'none';
parentWindow.INTERNAL_RootDomElement.appendChild(popupRoot);



                    var popupRootDiv;

                    if (CSHTML5.Interop.IsRunningInTheSimulator) {
                        popupRootDiv = new CSHTML5.Internal.INTERNAL_HtmlDomElementReference(uniquePopupRootIdentifier, null);
                    } else {
                        popupRootDiv = document.getElementById(uniquePopupRootIdentifier);
                    }


                    var popupRoot = new Windows.UI.Xaml.Controls.Primitives.PopupRoot(uniquePopupRootIdentifier, parentWindow);
                    popupRoot.INTERNAL_OuterDomElement = (popupRoot.INTERNAL_InnerDomElement = popupRootDiv, popupRootDiv);


                    popupRoot.AddHandler(Windows.UI.Xaml.UIElement.PointerPressedEvent, DotNetForHtml5.Core.INTERNAL_PopupsManager.OnClickOnPopupOrWindow, true);


                    DotNetForHtml5.Core.INTERNAL_PopupsManager.PopupRootIdentifierToInstance.add(uniquePopupRootIdentifier, popupRoot);

                    return popupRoot;
                },
                RemovePopupRoot: function (popupRoot) {
                    var uniquePopupRootIdentifier = popupRoot.INTERNAL_UniqueIndentifier;
                    if (DotNetForHtml5.Core.INTERNAL_PopupsManager.PopupRootIdentifierToInstance.containsKey(uniquePopupRootIdentifier)) {
                        var parentWindow = popupRoot.INTERNAL_ParentWindow;


                        popupRoot.RemoveHandler(Windows.UI.Xaml.UIElement.PointerPressedEvent, DotNetForHtml5.Core.INTERNAL_PopupsManager.OnClickOnPopupOrWindow);



var popupRoot = document.getElementById(                        uniquePopupRootIdentifier);
parentWindow.INTERNAL_RootDomElement.removeChild(popupRoot);



                        DotNetForHtml5.Core.INTERNAL_PopupsManager.PopupRootIdentifierToInstance.remove(uniquePopupRootIdentifier);
                    } else {
                        throw new System.Exception("No PopupRoot with the following identifier was found: " + (uniquePopupRootIdentifier || "") + ". Please contact support.");
                    }
                },
                GetAllRootUIElements: function () {
                    return new Bridge.GeneratorEnumerable(Bridge.fn.bind(this, function ()  {
                        var $step = 0,
                            $jumpFromFinally,
                            $returnValue,
                            $t,
                            popupRoot,
                            $async_e;

                        var $enumerator = new Bridge.GeneratorEnumerator(Bridge.fn.bind(this, function () {
                            try {
                                for (;;) {
                                    switch ($step) {
                                        case 0: {
                                            $enumerator.current = Windows.UI.Xaml.Window.Current;
                                                $step = 1;
                                                return true;
                                        }
                                        case 1: {
                                            $t = Bridge.getEnumerator(INTERNAL_BridgeWorkarounds.GetDictionaryValues_SimulatorCompatible(System.String, System.Object, DotNetForHtml5.Core.INTERNAL_PopupsManager.PopupRootIdentifierToInstance), System.Object);
                                                $step = 2;
                                                continue;
                                        }
                                        case 2: {
                                            if ($t.moveNext()) {
                                                    popupRoot = $t.Current;
                                                    $step = 3;
                                                    continue;
                                                }
                                            $step = 5;
                                            continue;
                                        }
                                        case 3: {
                                            $enumerator.current = popupRoot;
                                                $step = 4;
                                                return true;
                                        }
                                        case 4: {
                                            $step = 2;
                                            continue;
                                        }
                                        case 5: {

                                        }
                                        default: {
                                            return false;
                                        }
                                    }
                                }
                            } catch($async_e1) {
                                $async_e = System.Exception.create($async_e1);
                                throw $async_e;
                            }
                        }));
                        return $enumerator;
                    }));
                },
                CalculatePopupAbsolutePositionBasedOnElementPosition: function (parentElement, horizontalOffset, verticalOffset) {
                    if (CSHTML5.Internal.INTERNAL_VisualTreeManager.IsElementInVisualTree(parentElement)) {
                        var gt = parentElement.TransformToVisual(parentElement.INTERNAL_ParentWindow);
                        var parentElementAbsolutePosition = gt.Transform(new Windows.Foundation.Point.$ctor1(0.0, 0.0));

                        if (Bridge.is(parentElement, Windows.UI.Xaml.FrameworkElement) && !isNaN(Bridge.cast(parentElement, Windows.UI.Xaml.FrameworkElement).ActualHeight)) {
                            verticalOffset += Bridge.cast(parentElement, Windows.UI.Xaml.FrameworkElement).ActualHeight;
                        }

                        return new Windows.Foundation.Point.$ctor1(parentElementAbsolutePosition.X + horizontalOffset, parentElementAbsolutePosition.Y + verticalOffset);
                    } else {
                        return new Windows.Foundation.Point.ctor();
                    }
                },
                GetUIElementAbsolutePosition: function (element) {
                    if (CSHTML5.Internal.INTERNAL_VisualTreeManager.IsElementInVisualTree(element)) {
                        var gt = element.TransformToVisual(null);


                        return gt.Transform(new Windows.Foundation.Point.$ctor1(0.0, 0.0));
                    }
                    return new Windows.Foundation.Point.ctor();
                },
                EnsurePopupStaysWithinScreenBounds: function (popup) {
                    if (popup.IsOpen && popup.PopupRoot != null && Bridge.is(popup.PopupRoot.Content, Windows.UI.Xaml.FrameworkElement)) {
                        var content = Bridge.cast(popup.PopupRoot.Content, Windows.UI.Xaml.FrameworkElement);
                        var popupActualWidth = content.ActualWidth;
                        var popupActualHeight = content.ActualHeight;
                        if (!isNaN(popupActualWidth) && !isNaN(popupActualHeight) && popupActualWidth > 0 && popupActualHeight > 0) {
                            var popupPosition = new Windows.Foundation.Point.$ctor1(0, 0);
                            if (popup.INTERNAL_VisualParent != null) {
                                popupPosition = popup.TransformToVisual(Windows.UI.Xaml.Application.Current.RootVisual).Transform(popupPosition.$clone());
                            }

                            var windowBounds = Windows.UI.Xaml.Window.Current.Bounds.$clone();
                            var popupX = popup.HorizontalOffset + popupPosition.X;
                            var popupY = popup.VerticalOffset + popupPosition.Y;

                            var widthOfAreaOutsideBounds = (popupX + popupActualWidth) - windowBounds.Width;
                            var heightOfAreaOutsideBounds = (popupY + popupActualHeight) - windowBounds.Height;

                            if (widthOfAreaOutsideBounds > 0) {
                                popup.HorizontalOffset = popup.HorizontalOffset - widthOfAreaOutsideBounds;
                            }
                            if (heightOfAreaOutsideBounds > 0) {
                                popup.VerticalOffset = popup.VerticalOffset - heightOfAreaOutsideBounds;
                            }
                        }
                    }
                },
                /**
                 * Determines whether the parent of a popup is visible on screen, and if it is the foremost element (ie. not covered by another element). This is useful for example to hide the TextBox validation tooltips when they become hidden after scrolling (cf. ZenDesk 628).
                 *
                 * @static
                 * @public
                 * @this DotNetForHtml5.Core.INTERNAL_PopupsManager
                 * @memberof DotNetForHtml5.Core.INTERNAL_PopupsManager
                 * @param   {Windows.UI.Xaml.Controls.Primitives.Popup}    popup    The popup.
                 * @return  {boolean}                                               Returns True if the parent of the popup is visible on screen and it is the foremost element, False otherwise.
                 */
                IsPopupParentVisibleOnScreen: function (popup) {
                    if (popup.PlacementTarget != null) {
                        if (Bridge.is(popup.PlacementTarget, Windows.UI.Xaml.FrameworkElement)) {
                            var element = Bridge.cast(popup.PlacementTarget, Windows.UI.Xaml.FrameworkElement);

                            return DotNetForHtml5.Core.INTERNAL_PopupsManager.IsElementVisibleOnScreen(element);
                        } else {
                            throw new System.InvalidOperationException.$ctor1("The Popup.PlacementTarget property must contain a FrameworkElement");
                        }
                    } else {
                        throw new System.InvalidOperationException.$ctor1("The Popup.PlacementTarget property must be set.");
                    }
                },
                /**
                 * Determines whether an element is visible on screen, and if it is the foremost element (ie. it is not covered by another element). This is useful for example to hide the TextBox validation tooltips when they become hidden after scrolling (cf. ZenDesk 628).
                 *
                 * @static
                 * @this DotNetForHtml5.Core.INTERNAL_PopupsManager
                 * @memberof DotNetForHtml5.Core.INTERNAL_PopupsManager
                 * @param   {Windows.UI.Xaml.FrameworkElement}    element    The element to check.
                 * @return  {boolean}                                        Returns True if the element is visible on screen and it is the foremost element, False otherwise.
                 */
                IsElementVisibleOnScreen: function (element) {
                    if (!element._isLoaded) {
                        return false;
                    }

                    if (!(!(typeof element.INTERNAL_OuterDomElement === 'undefined' || element.INTERNAL_OuterDomElement === null) && !document.doesElementInheritDisplayNone(element.INTERNAL_OuterDomElement))) {
                        return false;
                    }

                    var actualSize = element.INTERNAL_GetActualWidthAndHeight();
                    if (isNaN(actualSize.Width) || isNaN(actualSize.Height)) {
                        return false;
                    }

                    var topLeftCoordinates = DotNetForHtml5.Core.INTERNAL_PopupsManager.GetUIElementAbsolutePosition(element);
                    var elementBounds = new Windows.Foundation.Rect.$ctor1(topLeftCoordinates.X, topLeftCoordinates.Y, actualSize.Width, actualSize.Height);

                    var windowBounds = Windows.UI.Xaml.Window.Current.Bounds.$clone();

                    if (elementBounds.X + elementBounds.Width < 0.0 || elementBounds.Y + elementBounds.Height < 0.0 || elementBounds.X > windowBounds.Width || elementBounds.Y > windowBounds.Height) {
                        return false;
                    }

                    var centerPosition = new Windows.Foundation.Point.$ctor1(elementBounds.X + elementBounds.Width / 2, elementBounds.Y + elementBounds.Height / 2);

                    var elementAtCenterPosition = Windows.UI.Xaml.Media.VisualTreeHelper.FindElementInHostCoordinates(centerPosition.$clone());

                    if (Bridge.referenceEquals(elementAtCenterPosition, element)) {
                        return true;
                    } else {
                        return false;
                    }
                }
            }
        }
    });

    Bridge.define("DotNetForHtml5.Core.INTERNAL_Simulator", {
        statics: {
            fields: {
                htmlDocument: null,
                domDocument: null,
                WebControlDispatcherBeginInvoke: null,
                javaScriptExecutionHandler: null,
                wpfMediaElementFactory: null,
                webClientFactory: null,
                clipboardHandler: null,
                simulatorProxy: null
            },
            props: {
                HtmlDocument: {
                    get: function () {
                        return DotNetForHtml5.Core.INTERNAL_Simulator.htmlDocument;
                    },
                    set: function (value) {
                        DotNetForHtml5.Core.INTERNAL_Simulator.htmlDocument = value;
                    }
                },
                DOMDocument: {
                    get: function () {
                        return DotNetForHtml5.Core.INTERNAL_Simulator.domDocument;
                    },
                    set: function (value) {
                        DotNetForHtml5.Core.INTERNAL_Simulator.domDocument = value;
                    }
                },
                JavaScriptExecutionHandler: {
                    get: function () {
                        return DotNetForHtml5.Core.INTERNAL_Simulator.javaScriptExecutionHandler;
                    },
                    set: function (value) {
                        DotNetForHtml5.Core.INTERNAL_Simulator.javaScriptExecutionHandler = value;
                    }
                },
                WpfMediaElementFactory: {
                    get: function () {
                        return DotNetForHtml5.Core.INTERNAL_Simulator.wpfMediaElementFactory;
                    },
                    set: function (value) {
                        DotNetForHtml5.Core.INTERNAL_Simulator.wpfMediaElementFactory = value;
                    }
                },
                WebClientFactory: {
                    get: function () {
                        return DotNetForHtml5.Core.INTERNAL_Simulator.webClientFactory;
                    },
                    set: function (value) {
                        DotNetForHtml5.Core.INTERNAL_Simulator.webClientFactory = value;
                    }
                },
                ClipboardHandler: {
                    get: function () {
                        return DotNetForHtml5.Core.INTERNAL_Simulator.clipboardHandler;
                    },
                    set: function (value) {
                        DotNetForHtml5.Core.INTERNAL_Simulator.clipboardHandler = value;
                    }
                },
                SimulatorProxy: {
                    get: function () {
                        return DotNetForHtml5.Core.INTERNAL_Simulator.simulatorProxy;
                    },
                    set: function (value) {
                        DotNetForHtml5.Core.INTERNAL_Simulator.simulatorProxy = value;
                    }
                }
            }
        }
    });

    /**
     * @memberof System
     * @callback System.Func
     * @param   {string}           arg
     * @return  {System.Object}
     */

    /** @namespace DotNetForHtml5.Core */

    /**
     * A class used to convert elements written as strings in the XAML code into the correct type for the properties setted
     *
     * @static
     * @abstract
     * @public
     * @class DotNetForHtml5.Core.TypeFromStringConverters
     */
    Bridge.define("DotNetForHtml5.Core.TypeFromStringConverters", {
        statics: {
            fields: {
                AdviseToFixTheError: null,
                IsBaseTypesConvertersRegistered: false,
                Converters: null
            },
            ctors: {
                init: function () {
                    this.AdviseToFixTheError = "To fix the issue, please locate the error by looking for the first XAML file in the Stack Trace.";
                    this.Converters = new (System.Collections.Generic.Dictionary$2(System.Type,Function)).ctor();
                }
            },
            methods: {
                RegisterBaseTypesConverters: function () {
                    DotNetForHtml5.Core.TypeFromStringConverters.RegisterConverter(System.Boolean, DotNetForHtml5.Core.TypeFromStringConverters.ConvertBoolFromString);
                    DotNetForHtml5.Core.TypeFromStringConverters.RegisterConverter(System.Nullable$1(System.Boolean), DotNetForHtml5.Core.TypeFromStringConverters.ConvertNullableBoolFromString);
                    DotNetForHtml5.Core.TypeFromStringConverters.RegisterConverter(System.Int32, DotNetForHtml5.Core.TypeFromStringConverters.ConvertIntFromString);
                    DotNetForHtml5.Core.TypeFromStringConverters.RegisterConverter(System.Double, DotNetForHtml5.Core.TypeFromStringConverters.ConvertDoubleFromString);
                    DotNetForHtml5.Core.TypeFromStringConverters.RegisterConverter(System.Uri, DotNetForHtml5.Core.TypeFromStringConverters.ConvertUriFromString);
                    DotNetForHtml5.Core.TypeFromStringConverters.RegisterConverter(System.Object, DotNetForHtml5.Core.TypeFromStringConverters.ConvertObjectFromString);
                    DotNetForHtml5.Core.TypeFromStringConverters.RegisterConverter(System.Byte, DotNetForHtml5.Core.TypeFromStringConverters.ConvertByteFromString);
                    DotNetForHtml5.Core.TypeFromStringConverters.RegisterConverter(Windows.UI.Text.TextDecorations, DotNetForHtml5.Core.TypeFromStringConverters.ConvertTextDecorationsFromString);
                    DotNetForHtml5.Core.TypeFromStringConverters.RegisterConverter(System.Nullable$1(Windows.UI.Text.TextDecorations), DotNetForHtml5.Core.TypeFromStringConverters.ConvertNullableTextDecorationsFromString);
                    DotNetForHtml5.Core.TypeFromStringConverters.RegisterConverter(System.TimeSpan, DotNetForHtml5.Core.TypeFromStringConverters.ConvertTimeSpanFromString);
                    DotNetForHtml5.Core.TypeFromStringConverters.RegisterConverter(System.Nullable$1(System.TimeSpan), DotNetForHtml5.Core.TypeFromStringConverters.ConvertTimeSpanFromString);
                },
                /**
                 * Checks if it is possible to convert from string to a given type.
                 *
                 * @static
                 * @public
                 * @this DotNetForHtml5.Core.TypeFromStringConverters
                 * @memberof DotNetForHtml5.Core.TypeFromStringConverters
                 * @param   {System.Type}    type    Type to lookup.
                 * @return  {boolean}                True if a converter exists, false otherwise.
                 */
                CanTypeBeConverted: function (type) {
                    var converter = { };
                    return DotNetForHtml5.Core.TypeFromStringConverters.TryGetConverter(type, converter);
                },
                /**
                 * Registers a type and its associated method to convert it from a string. This method takes part in allowing the definition of an object of the said type directly in the xaml.
                 *
                 * @static
                 * @public
                 * @this DotNetForHtml5.Core.TypeFromStringConverters
                 * @memberof DotNetForHtml5.Core.TypeFromStringConverters
                 * @param   {System.Type}    type         The type for which a converter will be defined
                 * @param   {System.Func}    converter    The method that will convert the XAML string into an instance of the type.
                 * @return  {void}
                 */
                RegisterConverter: function (type, converter) {
                    if (!DotNetForHtml5.Core.TypeFromStringConverters.Converters.containsKey(type)) {
                        DotNetForHtml5.Core.TypeFromStringConverters.Converters.add(type, converter);
                    }
                },
                /**
                 * Converts the given string into the specified Type using the registered converter for the Type.
                 *
                 * @static
                 * @public
                 * @this DotNetForHtml5.Core.TypeFromStringConverters
                 * @memberof DotNetForHtml5.Core.TypeFromStringConverters
                 * @param   {System.Type}      type    The Type to which the conversion method is registered.
                 * @param   {string}           s       The string to convert.
                 * @return  {System.Object}            The instance of the specified type converted from the string.
                 */
                ConvertFromInvariantString: function (type, s) {

                    if (Bridge.referenceEquals(Bridge.Reflection.getTypeName(type), "Nullable`1") && Bridge.referenceEquals(s, "x:Null")) {

                        return null;
                    } else if (Bridge.Reflection.isEnum(type)) {
                        var enumValue = System.Enum.parse(type, s, true);
                        return enumValue;
                    } else {
                        var converter = { };
                        if (DotNetForHtml5.Core.TypeFromStringConverters.TryGetConverter(type, converter)) {
                            var convertedValue = converter.v(s);

                            return convertedValue;
                        } else {
                            throw new System.Exception("Unable to find a converter from \"String\" to \"" + (Bridge.getTypeName(type) || "") + "\"");
                        }
                    }
                },
                TryGetConverter: function (type, converter) {
                    var $t;
                    if (!DotNetForHtml5.Core.TypeFromStringConverters.IsBaseTypesConvertersRegistered) {
                        DotNetForHtml5.Core.TypeFromStringConverters.RegisterBaseTypesConverters();
                        DotNetForHtml5.Core.TypeFromStringConverters.IsBaseTypesConvertersRegistered = true;
                    }

                    var nonNullableType = type;
                    if (Bridge.referenceEquals(Bridge.Reflection.getTypeName(type), "Nullable`1")) {
                        if (DotNetForHtml5.Core.TypeFromStringConverters.Converters.containsKey(type)) {
                            converter.v = DotNetForHtml5.Core.TypeFromStringConverters.Converters.getItem(type);
                            return true;
                        } else {
                            if (Bridge.Reflection.isGenericType(type) && Bridge.Reflection.getGenericArguments(type).length > 0) {
                                nonNullableType = ($t = Bridge.Reflection.getGenericArguments(type))[System.Array.index(0, $t)];
                            }
                        }
                    }

                    if (DotNetForHtml5.Core.TypeFromStringConverters.Converters.containsKey(nonNullableType)) {
                        converter.v = DotNetForHtml5.Core.TypeFromStringConverters.Converters.getItem(nonNullableType);
                        return true;
                    } else {
                        converter.v = null;
                        return false;
                    }
                },
                ConvertBoolFromString: function (str) {
                    var lowerStr = str.toLowerCase();
                    if (Bridge.referenceEquals(lowerStr, "true")) {
                        return Bridge.box(true, System.Boolean, System.Boolean.toString);
                    } else if (Bridge.referenceEquals(lowerStr, "false")) {
                        return Bridge.box(false, System.Boolean, System.Boolean.toString);
                    } else {
                        throw new System.Exception("Xaml exception: cannot convert \"" + (str || "") + "\" to bool. " + (DotNetForHtml5.Core.TypeFromStringConverters.AdviseToFixTheError || ""));
                    }
                },
                ConvertNullableBoolFromString: function (str) {
                    var lowerStr = str.toLowerCase();
                    if (Bridge.referenceEquals(lowerStr, "true")) {
                        return Bridge.box(true, System.Boolean, System.Boolean.toString);
                    } else if (Bridge.referenceEquals(lowerStr, "false")) {
                        return Bridge.box(false, System.Boolean, System.Boolean.toString);
                    } else {
                        return null;
                    }
                },
                ConvertIntFromString: function (str) {
                    var returnValue = { };
                    if (!System.Int32.tryParse(str, returnValue)) {
                        throw new System.Exception("Xaml exception: cannot convert \"" + (str || "") + "\" to int. " + (DotNetForHtml5.Core.TypeFromStringConverters.AdviseToFixTheError || ""));
                    }
                    return Bridge.box(returnValue.v, System.Int32);
                },
                ConvertDoubleFromString: function (str) {
                    var returnValue = { };
                    if (str != null && Bridge.referenceEquals(str.toLowerCase(), "auto")) {
                        returnValue.v = Number.NaN;
                    } else if (!System.Double.tryParse(str, null, returnValue)) {
                        throw new System.Exception("Xaml exception: cannot convert \"" + (str || "") + "\" to double. " + (DotNetForHtml5.Core.TypeFromStringConverters.AdviseToFixTheError || ""));
                    }
                    return Bridge.box(returnValue.v, System.Double, System.Double.format, System.Double.getHashCode);
                },
                ConvertByteFromString: function (str) {
                    var returnValue = { };
                    if (!System.Byte.tryParse(str, returnValue)) {
                        throw new System.Exception("Xaml exception: cannot convert \"" + (str || "") + "\" to byte. " + (DotNetForHtml5.Core.TypeFromStringConverters.AdviseToFixTheError || ""));
                    }
                    return Bridge.box(returnValue.v, System.Byte);
                },
                ConvertUriFromString: function (str) {
                    var uriKind = new System.UriKind();
                    if (System.String.contains(str,":")) {
                        var textBeforeColon = str.substr(0, System.String.indexOf(str, ":"));
                        if (!System.String.contains(textBeforeColon,"\\") && !System.String.contains(textBeforeColon,"/")) {
                            uriKind = System.UriKind.Absolute;
                        } else {
                            uriKind = System.UriKind.Relative;
                        }
                    } else {
                        uriKind = System.UriKind.Relative;
                    }

                    var returnValue = new System.Uri(str, uriKind);

                    return returnValue;
                },
                ConvertObjectFromString: function (str) {
                    return str;
                },
                ConvertTextDecorationsFromString: function (textDecorations) {
                    var $t;
                    var returnValue = { };
                    if (System.String.isNullOrEmpty(textDecorations) || !DotNetForHtml5.Core.TypeFromStringConverters.TryParseTextDecorations(textDecorations, returnValue)) {
                        throw new System.Exception("Xaml exception: cannot convert \"" + ((($t = textDecorations, $t != null ? $t : "")) || "") + "\" to TextDecorations enum. " + (DotNetForHtml5.Core.TypeFromStringConverters.AdviseToFixTheError || ""));
                    }
                    return Bridge.box(System.Nullable.getValue(returnValue.v), Windows.UI.Text.TextDecorations, System.Enum.toStringFn(Windows.UI.Text.TextDecorations));
                },
                ConvertNullableTextDecorationsFromString: function (textDecorations) {
                    var returnValue = { };
                    if (System.String.isNullOrEmpty(textDecorations)) {
                        return null;
                    } else {
                        if (!DotNetForHtml5.Core.TypeFromStringConverters.TryParseTextDecorations(textDecorations, returnValue)) {
                            throw new System.Exception("Xaml exception: cannot convert \"" + (textDecorations || "") + "\" to TextDecorations enum. " + (DotNetForHtml5.Core.TypeFromStringConverters.AdviseToFixTheError || ""));
                        } else {
                            return Bridge.box(returnValue.v, Windows.UI.Text.TextDecorations, System.Nullable.toStringFn(System.Enum.toStringFn(Windows.UI.Text.TextDecorations)), System.Nullable.getHashCode);
                        }
                    }
                },
                TryParseTextDecorations: function (textDecorationsString, textDecorations) {
                    var $t;
                    $t = Bridge.getEnumerator(System.Enum.getValues(Windows.UI.Text.TextDecorations));
                    try {
                        while ($t.moveNext()) {
                            var value = Bridge.cast($t.Current, Windows.UI.Text.TextDecorations);
                            if (Bridge.referenceEquals(textDecorationsString.toLowerCase(), System.Enum.toString(Windows.UI.Text.TextDecorations, value).toLowerCase())) {
                                textDecorations.v = value;
                                return true;
                            }
                        }
                    } finally {
                        if (Bridge.is($t, System.IDisposable)) {
                            $t.System$IDisposable$Dispose();
                        }
                    }
                    textDecorations.v = null;
                    return false;
                },
                ConvertTimeSpanFromString: function (str) {
                    try {
                        var returnValue = INTERNAL_BridgeWorkarounds.TimeSpanParse(str);
                        return returnValue;
                    } catch ($e1) {
                        $e1 = System.Exception.create($e1);
                        throw new System.Exception("Xaml exception: cannot convert \"" + (str || "") + "\" to TimeSpan. " + (DotNetForHtml5.Core.TypeFromStringConverters.AdviseToFixTheError || ""));
                    }
                }
            }
        }
    });

    Bridge.define("INTERNAL_BridgeWorkarounds", {
        statics: {
            methods: {
                GetAssemblyNameWithoutCallingGetNameMethod: function (assembly) {
                    var fullName = assembly.name;
                    var indexOfComma = System.String.indexOf(fullName, String.fromCharCode(44));
                    if (indexOfComma > -1) {
                        return fullName.substr(0, indexOfComma);
                    } else {
                        return fullName;
                    }
                },
                TimeSpanParse: function (timeSpanAsString, canBeNegative) {
                    if (canBeNegative === void 0) { canBeNegative = true; }

                    var timeSpan;
                    var days = 0;
                    var hours = 0;
                    var minutes = 0;
                    var seconds = 0;
                    var fractionalSeconds = 0;
                    var timeAsString = timeSpanAsString.trim();
                    var signKeeper = 1;
                    if (timeAsString.charCodeAt(0) === 45) {
                        signKeeper = -1;
                        timeAsString = timeAsString.substr(1);
                    }
                    var splittedTime = System.String.split(timeAsString, [58].map(function (i) {{ return String.fromCharCode(i); }}));
                    if (splittedTime.length > 3) {
                        throw new System.FormatException.$ctor1("Could not create KeyTime from \"" + (timeSpanAsString || "") + "\".");
                    } else if (splittedTime.length === 1) {
                        days = System.Int32.parse(splittedTime[System.Array.index(0, splittedTime)]);
                        if (days < 0) {
                            throw new System.FormatException.$ctor1("Could not create KeyTime from \"" + (timeSpanAsString || "") + "\". At least one of the numeric components is out of range or contains too many digits.");
                        }
                    } else {
                        for (var i = 0; i < splittedTime.length; i = (i + 1) | 0) {
                            if (i === 0) {
                                var daysAndHours = System.String.split(splittedTime[System.Array.index(0, splittedTime)], [46].map(function (i) {{ return String.fromCharCode(i); }}));
                                if (daysAndHours.length === 1) {
                                    hours = System.Int32.parse(daysAndHours[System.Array.index(0, daysAndHours)]);
                                } else {
                                    days = System.Int32.parse(daysAndHours[System.Array.index(0, daysAndHours)]);
                                    if (days < 0) {
                                        throw new System.FormatException.$ctor1("Could not create KeyTime from \"" + (timeSpanAsString || "") + "\". At least one of the numeric components is out of range or contains too many digits.");
                                    }
                                    hours = System.Int32.parse(daysAndHours[System.Array.index(1, daysAndHours)]);
                                }
                                if (hours < 0 || hours > 23) {
                                    throw new System.FormatException.$ctor1("Could not create KeyTime from \"" + (timeSpanAsString || "") + "\". At least one of the numeric components is out of range or contains too many digits.");
                                }
                            } else if (i === 1) {
                                minutes = System.Int32.parse(splittedTime[System.Array.index(i, splittedTime)]);
                                if (minutes < 0 || minutes > 59) {
                                    throw new System.FormatException.$ctor1("Could not create KeyTime from \"" + (timeSpanAsString || "") + "\". At least one of the numeric components is out of range or contains too many digits.");
                                }
                            } else if (i === 2) {
                                var secondsAndFractionalSeconds = System.String.split(splittedTime[System.Array.index(i, splittedTime)], [46].map(function (i) {{ return String.fromCharCode(i); }}));
                                if (secondsAndFractionalSeconds.length === 1) {
                                    seconds = System.Int32.parse(secondsAndFractionalSeconds[System.Array.index(0, secondsAndFractionalSeconds)]);
                                } else {
                                    seconds = System.Int32.parse((Bridge.referenceEquals(secondsAndFractionalSeconds[System.Array.index(0, secondsAndFractionalSeconds)], "") ? "0" : secondsAndFractionalSeconds[System.Array.index(0, secondsAndFractionalSeconds)]));
                                    fractionalSeconds = System.Int32.parse(secondsAndFractionalSeconds[System.Array.index(1, secondsAndFractionalSeconds)]);
                                    if (fractionalSeconds < 0 || fractionalSeconds > 9999999) {
                                        throw new System.FormatException.$ctor1("Could not create KeyTime from \"" + (timeSpanAsString || "") + "\". At least one of the numeric components is out of range or contains too many digits.");
                                    }
                                }
                                if (seconds < 0 || seconds > 59) {
                                    throw new System.FormatException.$ctor1("Could not create KeyTime from \"" + (timeSpanAsString || "") + "\". At least one of the numeric components is out of range or contains too many digits.");
                                }
                            }
                        }
                    }
                    var ticks = System.Int64(Bridge.Int.mul(days, 24));
                    ticks = ticks.add(System.Int64(hours));
                    ticks = ticks.mul(System.Int64(60));
                    ticks = ticks.add(System.Int64(minutes));
                    ticks = ticks.mul(System.Int64(60));
                    ticks = ticks.add(System.Int64(seconds));
                    ticks = ticks.mul(System.Int64(10000000));
                    ticks = ticks.add(System.Int64(fractionalSeconds));

                    if (signKeeper === -1) {
                        if (canBeNegative) {
                            timeSpan = new System.TimeSpan(ticks.mul(System.Int64(signKeeper)));
                        } else {
                            timeSpan = new System.TimeSpan();
                        }
                    } else {
                        timeSpan = new System.TimeSpan(ticks);
                    }
                    return timeSpan;
                },
                GetDictionaryEnumerator: function (TKey, TValue, dict) {
                    return Bridge.getEnumerator(Bridge.cast(dict, System.Collections.Generic.IEnumerable$1(System.Collections.Generic.KeyValuePair$2(TKey,TValue))), System.Collections.Generic.KeyValuePair$2(TKey,TValue));
                },
                GetDictionaryKeys_SimulatorCompatible: function (TKey, TValue, dict) {
                    if (false) {
                        return System.Linq.Enumerable.from(dict).select(function (keyValuePair) {
                                return keyValuePair.key;
                            });
                    } else {
                        return dict.Keys;
                    }
                },
                GetDictionaryValues_SimulatorCompatible: function (TKey, TValue, dict) {
                    if (false) {
                        return System.Linq.Enumerable.from(dict).select(function (keyValuePair) {
                                return keyValuePair.value;
                            });
                    } else {
                        return dict["System$Collections$Generic$IDictionary$2$" + Bridge.getTypeAlias(TKey) + "$" + Bridge.getTypeAlias(TValue) + "$Values"];
                    }
                },
                TypeGetMethods_SimulatorCompatible: function (type, bindingFlags) {
                    if (CSHTML5.Interop.IsRunningInTheSimulator) {
                        return Bridge.as(type.GetMethods(bindingFlags), System.Array.type(System.Reflection.MethodInfo));
                    } else {
                        return INTERNAL_BridgeWorkarounds.TypeGetMethods_BrowserOnly(type, bindingFlags);
                    }
                },
                TypeGetMethods_BrowserOnly: function (type, bindingFlags) {
                    return Bridge.Reflection.getMembers(type, 8, bindingFlags);
                },
                MethodInfoIsStatic_SimulatorCompatible: function (methodInfo) {
                    if (CSHTML5.Interop.IsRunningInTheSimulator) {
                        return Bridge.cast((methodInfo.IsStatic), System.Boolean);
                    } else {
                        return INTERNAL_BridgeWorkarounds.MethodInfoIsStatic_BrowserOnly(methodInfo);
                    }
                },
                MethodInfoIsStatic_BrowserOnly: function (methodInfo) {
                    return (methodInfo.is || false);
                },
                ConstructorInfoIsStatic_SimulatorCompatible: function (constructorInfo) {
                    if (CSHTML5.Interop.IsRunningInTheSimulator) {
                        return Bridge.cast((constructorInfo.IsStatic), System.Boolean);
                    } else {
                        return INTERNAL_BridgeWorkarounds.ConstructorInfoIsStatic_BrowserOnly(constructorInfo);
                    }
                },
                ConstructorInfoIsStatic_BrowserOnly: function (constructorInfo) {
                    return (constructorInfo.is || false);
                },
                ConstructorInfoIsPublic_SimulatorCompatible: function (constructorInfo) {
                    if (CSHTML5.Interop.IsRunningInTheSimulator) {
                        return Bridge.cast((constructorInfo.IsPublic), System.Boolean);
                    } else {
                        return INTERNAL_BridgeWorkarounds.ConstructorInfoIsPublic_BrowserOnly(constructorInfo);
                    }
                },
                ConstructorInfoIsPublic_BrowserOnly: function (constructorInfo) {
                    return (constructorInfo.a === 2);
                },
                FieldInfoIsStatic_SimulatorCompatible: function (fieldInfo) {
                    if (CSHTML5.Interop.IsRunningInTheSimulator) {
                        return Bridge.cast((fieldInfo.IsStatic), System.Boolean);
                    } else {
                        return INTERNAL_BridgeWorkarounds.FieldInfoIsStatic_BrowserOnly(fieldInfo);
                    }
                },
                FieldInfoIsStatic_BrowserOnly: function (fieldInfo) {
                    return (fieldInfo.is || false);
                },
                FieldInfoIsPublic_SimulatorCompatible: function (fieldInfo) {
                    if (CSHTML5.Interop.IsRunningInTheSimulator) {
                        return Bridge.cast((fieldInfo.IsPublic), System.Boolean);
                    } else {
                        return INTERNAL_BridgeWorkarounds.FieldInfoIsPublic_BrowserOnly(fieldInfo);
                    }
                },
                FieldInfoIsPublic_BrowserOnly: function (fieldInfo) {
                    return (fieldInfo.a === 2);
                },
                HexToInt_SimulatorOnly: function (hexString) {

                    if (DotNetForHtml5.Core.INTERNAL_Simulator.SimulatorProxy != null) {
                        return Bridge.cast(DotNetForHtml5.Core.INTERNAL_Simulator.SimulatorProxy.HexToInt(hexString), System.Int32);
                    } else {
                        return 0;
                    }
                }
            }
        }
    });

    Bridge.define("INTERNAL_DateTimeHelpers", {
        statics: {
            methods: {
                FromDateTime: function (value) {
                    /* 
                    DateTime dateTimeUtc = (value.Kind == DateTimeKind.Local ? value.ToUniversalTime() : value);
                    TimeSpan timeSince1970 = (dateTimeUtc - new DateTime(1970, 1, 1, 0, 0, 0, DateTimeKind.Utc));
                    double millisecondsSince1970 = timeSince1970.TotalMilliseconds;
                    var jsDate = JSIL.Verbatim.Expression("new Date($0)", millisecondsSince1970);
                    string serializedDate = JSIL.Verbatim.Expression("$0.toJSON()", jsDate);
                    return serializedDate;
                    */

                    var ts = System.DateTime.subdd(value, System.DateTime.getDate(value));
                    ts = System.TimeSpan.sub(ts, new System.TimeSpan(System.DateTime.getHour(value), System.DateTime.getMinute(value), System.DateTime.getSecond(value)));
                    var tsString = ts.getTicks().gt(System.Int64(1000000)) ? Bridge.toString(ts.getTicks()) : "0" + ts.getTicks();
                    if (System.String.endsWith(tsString, "0000")) {
                        tsString = tsString.substr(0, ((tsString.length - 4) | 0));
                    }
                    var secondString = "" + System.DateTime.getSecond(value);
                    if (secondString.length === 1) {
                        secondString = 0 + (secondString || "");
                    }
                    var minuteString = "" + System.DateTime.getMinute(value);
                    if (minuteString.length === 1) {
                        minuteString = 0 + (minuteString || "");
                    }
                    var hourString = "" + System.DateTime.getHour(value);
                    if (hourString.length === 1) {
                        hourString = 0 + (hourString || "");
                    }
                    var dayString = "" + System.DateTime.getDay(value);
                    if (dayString.length === 1) {
                        dayString = 0 + (dayString || "");
                    }
                    var monthString = "" + System.DateTime.getMonth(value);
                    if (monthString.length === 1) {
                        monthString = 0 + (monthString || "");
                    }
                    var yearString = "" + System.DateTime.getYear(value);
                    while (yearString.length < 4) {
                        yearString = 0 + (yearString || "");
                    }


                    var str = (yearString || "") + "-" + (monthString || "") + "-" + (dayString || "") + "T" + (hourString || "") + ":" + (minuteString || "") + ":" + (secondString || "");
                    if (ts.getTicks().ne(System.Int64(0))) {
                        str = (str || "") + (("." + (tsString || "")) || "");
                    }

                    if (System.DateTime.getKind(value) === 2) {
                        var changeFromUTC = System.DateTime.subdd(value, System.DateTime.toUniversalTime(value));
                        var minuteStringForTimeZone = "" + changeFromUTC.getMinutes();
                        if (minuteStringForTimeZone.length === 1) {
                            minuteStringForTimeZone = 0 + (minuteStringForTimeZone || "");
                        }
                        var hourStringForTimeZone = "" + Math.abs(changeFromUTC.getHours());
                        if (hourStringForTimeZone.length === 1) {
                            hourStringForTimeZone = 0 + (hourStringForTimeZone || "");
                        }
                        var timeZone = (hourStringForTimeZone || "") + ":" + (minuteStringForTimeZone || "");
                        if (changeFromUTC.getTicks().gte(System.Int64(0))) {
                            timeZone = "+" + (timeZone || "");
                        } else {
                            timeZone = "-" + (timeZone || "");
                        }
                        str = (str || "") + (timeZone || "");

                    } else if (System.DateTime.getKind(value) === 1) {
                        str = (str || "") + "Z";
                    }
                    return str;
                },
                ToDateTime: function (dateTimeAsString) {
                    /* 
                    var jsDate = JSIL.Verbatim.Expression("new Date($0)", dateTimeAsString);

                    int year = Convert.ToInt32(JSIL.Verbatim.Expression("$0.getFullYear()", jsDate));
                    int month = Convert.ToInt32(JSIL.Verbatim.Expression("$0.getMonth()", jsDate)) + 1; // Note: month index in JS starts at "0".
                    int day = Convert.ToInt32(JSIL.Verbatim.Expression("$0.getDate()", jsDate));
                    int hour = Convert.ToInt32(JSIL.Verbatim.Expression("$0.getHours()", jsDate));
                    int minute = Convert.ToInt32(JSIL.Verbatim.Expression("$0.getMinutes()", jsDate));
                    int second = Convert.ToInt32(JSIL.Verbatim.Expression("$0.getSeconds()", jsDate));
                    int millisecond = Convert.ToInt32(JSIL.Verbatim.Expression("$0.getMilliseconds()", jsDate));

                    DateTime dateTime = new DateTime(year, month, day, hour, minute, second, millisecond);

                    return dateTime;
                    */

                    var year, month, day, hour, minute, second, tickOrMillisecond, timezoneHour, timezoneMinute;
                    var dt;
                    var kind = 0;
                    var split = System.String.split(dateTimeAsString, [84].map(function (i) {{ return String.fromCharCode(i); }}));
                    var datePart = split[System.Array.index(0, split)];
                    var splittedDate = System.String.split(datePart, [45].map(function (i) {{ return String.fromCharCode(i); }}));
                    year = System.Int32.parse(splittedDate[System.Array.index(0, splittedDate)]);
                    month = System.Int32.parse(splittedDate[System.Array.index(1, splittedDate)]);
                    day = System.Int32.parse(splittedDate[System.Array.index(2, splittedDate)]);

                    var hourPart = split[System.Array.index(1, split)];
                    var timezoneSplitters = System.Array.init([43, 45], System.Char);
                    var hourSplittedFromTimeZone = System.String.split(hourPart, timezoneSplitters.map(function (i) {{ return String.fromCharCode(i); }}));
                    var splitters = System.Array.init([58, 46], System.Char);
                    var splittedHour = System.String.split(hourSplittedFromTimeZone[System.Array.index(0, hourSplittedFromTimeZone)], splitters.map(function (i) {{ return String.fromCharCode(i); }}));
                    hour = System.Int32.parse(splittedHour[System.Array.index(0, splittedHour)]);
                    minute = System.Int32.parse(splittedHour[System.Array.index(1, splittedHour)]);
                    var secondString = splittedHour[System.Array.index(2, splittedHour)];
                    if (System.String.endsWith(secondString, "Z")) {
                        kind = 1;
                        secondString = secondString.substr(0, ((secondString.length - 1) | 0));
                    }
                    second = System.Int32.parse(secondString);

                    if (hourSplittedFromTimeZone.length >= 2) {
                        var sign;
                        if (System.String.contains(hourPart,"+")) {
                            sign = 43;
                        } else {
                            sign = 45;
                        }
                        var splittedTimezone = System.String.split(hourSplittedFromTimeZone[System.Array.index(1, hourSplittedFromTimeZone)], [58].map(function (i) {{ return String.fromCharCode(i); }}));
                        timezoneHour = System.Int32.parse(String.fromCharCode(sign) + (splittedTimezone[System.Array.index(0, splittedTimezone)] || ""));
                        timezoneMinute = System.Int32.parse(splittedTimezone[System.Array.index(1, splittedTimezone)]);
                        hour = (hour - timezoneHour) | 0;
                        minute = (minute - timezoneMinute) | 0;
                        var datetimeForTimeZoneOffset = System.DateTime.getNow();
                        var ts = System.DateTime.subdd(datetimeForTimeZoneOffset, System.DateTime.toUniversalTime(datetimeForTimeZoneOffset));
                        hour = (hour + ts.getHours()) | 0;
                        minute = (minute + ts.getMinutes()) | 0;
                        kind = 2;
                    }
                    dt = System.DateTime.create(year, month, day, hour, minute, second, 0, kind);
                    return dt;
                }
            }
        }
    });

    Bridge.define("INTERNAL_EscapeHelpers", {
        statics: {
            methods: {
                EscapeXml: function (str) {
                    return 
            str.replace(/&/g, '&amp;')
               .replace(/</g, '&lt;')
               .replace(/>/g, '&gt;')
               .replace(/"/g, '&quot;')
               .replace(/'/g, '&apos;');
                },
                UnescapeXml: function (str) {
                    return 
            str.replace(/&apos;/g, "'")
               .replace(/&quot;/g, '"')
               .replace(/&gt;/g, '>')
               .replace(/&lt;/g, '<')
               .replace(/&amp;/g, '&');

                }
            }
        }
    });

    /** @namespace Microsoft.Win32 */

    /**
     * The {@link } namespaces provide types that handle events raised
     by the operating system, that manipulate the system registry, and that represent file
     and operating system handles.
     *
     * @class Microsoft.Win32.NamespaceDoc
     */
    Bridge.define("Microsoft.Win32.NamespaceDoc");

    /**
     * Represents the SOAP style that determines how the WSDL metadata for the service
     is formatted.
     *
     * @public
     * @class OperationFormatStyle
     */
    Bridge.define("OperationFormatStyle", {
        $kind: "enum",
        statics: {
            fields: {
                /**
                 * Causes the WSDL representation to contain a single element that represents the document that is exchanged for the operation.
                 *
                 * @static
                 * @public
                 * @memberof OperationFormatStyle
                 * @constant
                 * @default 0
                 * @type OperationFormatStyle
                 */
                Document: 0,
                /**
                 * Causes the WSDL representation of messages exchanged for an operation and
                 contains parameters as if it were a remote procedure call.
                 *
                 * @static
                 * @public
                 * @memberof OperationFormatStyle
                 * @constant
                 * @default 1
                 * @type OperationFormatStyle
                 */
                Rpc: 1
            }
        }
    });

    /** @namespace System.CodeDom.Compiler */

    /**
     * Identifies code generated by a tool. This class cannot be inherited.
     *
     * @public
     * @class System.CodeDom.Compiler.GeneratedCodeAttribute
     * @augments System.Attribute
     */
    Bridge.define("System.CodeDom.Compiler.GeneratedCodeAttribute", {
        inherits: [System.Attribute],
        fields: {
            /**
             * Gets the name of the tool that generated the code.
             *
             * @instance
             * @public
             * @memberof System.CodeDom.Compiler.GeneratedCodeAttribute
             * @function Tool
             * @type string
             */
            Tool: null,
            /**
             * Gets the version of the tool that generated the code.
             *
             * @instance
             * @public
             * @memberof System.CodeDom.Compiler.GeneratedCodeAttribute
             * @function Version
             * @type string
             */
            Version: null
        },
        ctors: {
            /**
             * Initializes a new instance of the System.CodeDom.Compiler.GeneratedCodeAttribute
             class specifying the name and version of the tool that generated the code.
             *
             * @instance
             * @public
             * @this System.CodeDom.Compiler.GeneratedCodeAttribute
             * @memberof System.CodeDom.Compiler.GeneratedCodeAttribute
             * @param   {string}    tool       The name of the tool that generated the code.
             * @param   {string}    version    The version of the tool that generated the code.
             * @return  {void}
             */
            ctor: function (tool, version) {
                this.$initialize();
                System.Attribute.ctor.call(this);
                this.Tool = tool;
                this.Version = version;
            }
        }
    });

    Bridge.define("System.Collections.Generic.HashSet2$1", function (T) { return {
        inherits: [System.Collections.Generic.HashSet$1(T)],
        ctors: {
            ctor: function () {
                this.$initialize();
                System.Collections.Generic.HashSet$1(T).ctor.call(this);
            },
            $ctor1: function (source) {
                this.$initialize();
                System.Collections.Generic.HashSet$1(T).$ctor1.call(this, source);
            }
        }
    }; });

    Bridge.definei("System.Collections.Generic.ISet2$1", function (T) { return {
        inherits: [System.Collections.Generic.ICollection$1(T)],
        $kind: "interface"
    }; });

    /** @namespace System.Collections.Generic */

    /**
     * The {@link } namespace contains interfaces
     and classes that define generic collections, which allow users to create strongly
     typed collections that provide better type safety and performance than non-generic
     strongly typed collections.
     *
     * @class System.Collections.Generic.NamespaceDoc
     */
    Bridge.define("System.Collections.Generic.NamespaceDoc");

    /** @namespace System.Collections */

    /**
     * The {@link } namespaces contain types that define various standard, specialized, and generic collection objects.
     *
     * @class System.Collections.NamespaceDoc
     */
    Bridge.define("System.Collections.NamespaceDoc");

    /** @namespace System.Collections.ObjectModel */

    /**
     * The {@link } namespace contains classes
     that can be used as collections in the object model of a reusable library. Use these
     classes when properties or methods return collections.
     *
     * @class System.Collections.ObjectModel.NamespaceDoc
     */
    Bridge.define("System.Collections.ObjectModel.NamespaceDoc");

    /** @namespace System.Collections.Specialized */

    /**
     * Notifies listeners of dynamic changes, such as when items get added and removed or the whole list is refreshed.
     *
     * @abstract
     * @public
     * @class System.Collections.Specialized.INotifyCollectionChanged
     */
    Bridge.define("System.Collections.Specialized.INotifyCollectionChanged", {
        $kind: "interface"
    });

    Bridge.define("System.Collections.ObjectModel.ObservableCollection$1.SimpleMonitor", function (T) { return {
        inherits: [System.IDisposable],
        $kind: "nested class",
        fields: {
            _busyCount: 0
        },
        props: {
            Busy: {
                get: function () {
                    return this._busyCount > 0;
                }
            }
        },
        alias: ["Dispose", "System$IDisposable$Dispose"],
        methods: {
            Enter: function () {
                this._busyCount = (this._busyCount + 1) | 0;
            },
            Dispose: function () {
                this._busyCount = (this._busyCount - 1) | 0;
            }
        }
    }; });

    Bridge.define("System.Collections.ObjectModel.ThrowHelper", {
        statics: {
            methods: {
                IfNullAndNullsAreIllegalThenThrow: function (T, value) {
                    if (value == null && !(Bridge.getDefaultValue(T) == null)) {
                        throw new System.ArgumentNullException.$ctor1((Bridge.toString(value) || "") + "is null");
                    }
                }
            }
        }
    });

    /**
     * The {@link } namespace contains specialized
     and strongly-typed collections; for example, a linked list dictionary, a bit vector,
     and collections that contain only strings.
     *
     * @class System.Collections.Specialized.NamespaceDoc
     */
    Bridge.define("System.Collections.Specialized.NamespaceDoc");

    /**
     * Describes the action that caused a System.Collections.Specialized.INotifyCollectionChanged.CollectionChanged event.
     *
     * @public
     * @class System.Collections.Specialized.NotifyCollectionChangedAction
     */
    Bridge.define("System.Collections.Specialized.NotifyCollectionChangedAction", {
        $kind: "enum",
        statics: {
            fields: {
                /**
                 * One or more items were added to the collection.
                 *
                 * @static
                 * @public
                 * @memberof System.Collections.Specialized.NotifyCollectionChangedAction
                 * @constant
                 * @default 0
                 * @type System.Collections.Specialized.NotifyCollectionChangedAction
                 */
                Add: 0,
                /**
                 * One or more items were removed from the collection.
                 *
                 * @static
                 * @public
                 * @memberof System.Collections.Specialized.NotifyCollectionChangedAction
                 * @constant
                 * @default 1
                 * @type System.Collections.Specialized.NotifyCollectionChangedAction
                 */
                Remove: 1,
                /**
                 * One or more items were replaced in the collection.
                 *
                 * @static
                 * @public
                 * @memberof System.Collections.Specialized.NotifyCollectionChangedAction
                 * @constant
                 * @default 2
                 * @type System.Collections.Specialized.NotifyCollectionChangedAction
                 */
                Replace: 2,
                /**
                 * One or more items were moved within the collection.
                 *
                 * @static
                 * @public
                 * @memberof System.Collections.Specialized.NotifyCollectionChangedAction
                 * @constant
                 * @default 3
                 * @type System.Collections.Specialized.NotifyCollectionChangedAction
                 */
                Move: 3,
                /**
                 * The content of the collection changed dramatically.
                 *
                 * @static
                 * @public
                 * @memberof System.Collections.Specialized.NotifyCollectionChangedAction
                 * @constant
                 * @default 4
                 * @type System.Collections.Specialized.NotifyCollectionChangedAction
                 */
                Reset: 4
            }
        }
    });

    /**
     * Provides data for the System.Collections.Specialized.INotifyCollectionChanged.CollectionChanged
     event.
     *
     * @public
     * @class System.Collections.Specialized.NotifyCollectionChangedEventArgs
     * @augments System.Object
     */
    Bridge.define("System.Collections.Specialized.NotifyCollectionChangedEventArgs", {
        fields: {
            _action: 0,
            _newItems: null,
            _newStartingIndex: 0,
            _oldItems: null,
            _oldStartingIndex: 0
        },
        props: {
            /**
             * Gets the action that caused the event. Returns a System.Collections.Specialized.NotifyCollectionChangedAction value that describes the action that caused the event.
             *
             * @instance
             * @public
             * @memberof System.Collections.Specialized.NotifyCollectionChangedEventArgs
             * @function Action
             * @type System.Collections.Specialized.NotifyCollectionChangedAction
             */
            Action: {
                get: function () {
                    return this._action;
                },
                set: function (value) {
                    this._action = value;
                }
            },
            /**
             * Gets the list of new items involved in the change. Returns the list of new items involved in the change.
             *
             * @instance
             * @public
             * @memberof System.Collections.Specialized.NotifyCollectionChangedEventArgs
             * @function NewItems
             * @type System.Collections.IList
             */
            NewItems: {
                get: function () {
                    return this._newItems;
                },
                set: function (value) {
                    this._newItems = value;
                }
            },
            /**
             * Gets the index at which the change occurred.
             Returns the zero-based index at which the change occurred.
             *
             * @instance
             * @public
             * @memberof System.Collections.Specialized.NotifyCollectionChangedEventArgs
             * @function NewStartingIndex
             * @type number
             */
            NewStartingIndex: {
                get: function () {
                    return this._newStartingIndex;
                },
                set: function (value) {
                    this._newStartingIndex = value;
                }
            },
            /**
             * Gets the list of items affected by a System.Collections.Specialized.NotifyCollectionChangedAction.Replace, Remove, or Move action.
             Returns he list of items affected by a System.Collections.Specialized.NotifyCollectionChangedAction.Replace, Remove, or Move action.
             *
             * @instance
             * @public
             * @memberof System.Collections.Specialized.NotifyCollectionChangedEventArgs
             * @function OldItems
             * @type System.Collections.IList
             */
            OldItems: {
                get: function () {
                    return this._oldItems;
                },
                set: function (value) {
                    this._oldItems = value;
                }
            },
            /**
             * Gets the index at which a System.Collections.Specialized.NotifyCollectionChangedAction.Move, Remove, or Replace action occurred.
             Returns the zero-based index at which a System.Collections.Specialized.NotifyCollectionChangedAction.Move, Remove, or Replace action occurred.
             *
             * @instance
             * @public
             * @memberof System.Collections.Specialized.NotifyCollectionChangedEventArgs
             * @function OldStartingIndex
             * @type number
             */
            OldStartingIndex: {
                get: function () {
                    return this._oldStartingIndex;
                },
                set: function (value) {
                    this._oldStartingIndex = value;
                }
            }
        },
        ctors: {
            /**
             * Initializes a new instance of the System.Collections.Specialized.NotifyCollectionChangedEventArgs class
             that describes a System.Collections.Specialized.NotifyCollectionChangedAction.Reset change.
             *
             * @instance
             * @public
             * @this System.Collections.Specialized.NotifyCollectionChangedEventArgs
             * @memberof System.Collections.Specialized.NotifyCollectionChangedEventArgs
             * @param   {System.Collections.Specialized.NotifyCollectionChangedAction}    action    The action that caused the event. This must be set to System.Collections.Specialized.NotifyCollectionChangedAction.Reset.
             * @return  {void}
             */
            ctor: function (action) {
                this.$initialize();
                System.Object.call(this);
                this.Action = action;
            },
            /**
             * Initializes a new instance of the System.Collections.Specialized.NotifyCollectionChangedEventArgs class that describes a multi-item change.
             *
             * @instance
             * @public
             * @this System.Collections.Specialized.NotifyCollectionChangedEventArgs
             * @memberof System.Collections.Specialized.NotifyCollectionChangedEventArgs
             * @param   {System.Collections.Specialized.NotifyCollectionChangedAction}    action          The action that caused the event. This can be set to System.Collections.Specialized.NotifyCollectionChangedAction.Reset,
             System.Collections.Specialized.NotifyCollectionChangedAction.Add, or System.Collections.Specialized.NotifyCollectionChangedAction.Remove.
             * @param   {System.Collections.IList}                                        changedItems    The items that are affected by the change.
             * @return  {void}
             */
            $ctor1: function (action, changedItems) {
                this.$initialize();
                System.Object.call(this);
                this.Action = action;
                if (action === System.Collections.Specialized.NotifyCollectionChangedAction.Reset || action === System.Collections.Specialized.NotifyCollectionChangedAction.Remove) {
                    this.OldItems = changedItems;
                } else if (action === System.Collections.Specialized.NotifyCollectionChangedAction.Add) {
                    this.NewItems = changedItems;
                }
            },
            /**
             * Initializes a new instance of the System.Collections.Specialized.NotifyCollectionChangedEventArgs class that describes a one-item change.
             Throws an exception if action is not Reset, Add, or Remove, or if action is Reset and changedItem is not null.
             *
             * @instance
             * @public
             * @this System.Collections.Specialized.NotifyCollectionChangedEventArgs
             * @memberof System.Collections.Specialized.NotifyCollectionChangedEventArgs
             * @param   {System.Collections.Specialized.NotifyCollectionChangedAction}    action         The action that caused the event. This can be set to System.Collections.Specialized.NotifyCollectionChangedAction.Reset, System.Collections.Specialized.NotifyCollectionChangedAction.Add, or System.Collections.Specialized.NotifyCollectionChangedAction.Remove.
             * @param   {System.Object}                                                   changedItem    The item that is affected by the change.
             * @return  {void}
             */
            $ctor6: function (action, changedItem) {
                this.$initialize();
                System.Object.call(this);
                if ((action !== System.Collections.Specialized.NotifyCollectionChangedAction.Add && action !== System.Collections.Specialized.NotifyCollectionChangedAction.Remove && action !== System.Collections.Specialized.NotifyCollectionChangedAction.Reset) || (action === System.Collections.Specialized.NotifyCollectionChangedAction.Reset && changedItem != null)) {
                    throw new System.ArgumentException.ctor();
                }
                if (action === System.Collections.Specialized.NotifyCollectionChangedAction.Add) {
                    this.NewItems = new (System.Collections.Generic.List$1(System.Object)).ctor();
                    System.Array.add(this.NewItems, changedItem, Object);
                }
                if (action === System.Collections.Specialized.NotifyCollectionChangedAction.Remove || action === System.Collections.Specialized.NotifyCollectionChangedAction.Reset) {
                    this.OldItems = new (System.Collections.Generic.List$1(System.Object)).ctor();
                    System.Array.add(this.OldItems, changedItem, Object);
                }

                this.Action = action;
            },
            /**
             * Initializes a new instance of the System.Collections.Specialized.NotifyCollectionChangedEventArgs class
             that describes a multi-item System.Collections.Specialized.NotifyCollectionChangedAction.Replace change.
             Exceptions:
               System.ArgumentException:
                 If action is not Replace.
               System.ArgumentNullException:
                 If oldItems or newItems is null.
             *
             * @instance
             * @public
             * @this System.Collections.Specialized.NotifyCollectionChangedEventArgs
             * @memberof System.Collections.Specialized.NotifyCollectionChangedEventArgs
             * @param   {System.Collections.Specialized.NotifyCollectionChangedAction}    action      The action that caused the event. This can only be set to System.Collections.Specialized.NotifyCollectionChangedAction.Replace.
             * @param   {System.Collections.IList}                                        newItems    The new items that are replacing the original items.
             * @param   {System.Collections.IList}                                        oldItems    The original items that are replaced.
             * @return  {void}
             */
            $ctor2: function (action, newItems, oldItems) {
                this.$initialize();
                System.Object.call(this);
                if (action !== System.Collections.Specialized.NotifyCollectionChangedAction.Replace) {
                    throw new System.ArgumentException.ctor();
                }
                if (oldItems == null || newItems == null) {
                    throw new System.ArgumentNullException.ctor();
                }
                this.NewItems = newItems;
                this.OldItems = oldItems;
                this.Action = action;
            },
            /**
             * Initializes a new instance of the System.Collections.Specialized.NotifyCollectionChangedEventArgs class
             that describes a multi-item change or a System.Collections.Specialized.NotifyCollectionChangedAction.Reset change.
             Exceptions:
               System.ArgumentException:
                 If action is not Reset, Add, or Remove, if action is Reset and either changedItems
                 is not null or startingIndex is not -1, or if action is Add or Remove and
                 startingIndex is less than -1.
               System.ArgumentNullException:
                 If action is Add or Remove and changedItems is null.
             *
             * @instance
             * @public
             * @this System.Collections.Specialized.NotifyCollectionChangedEventArgs
             * @memberof System.Collections.Specialized.NotifyCollectionChangedEventArgs
             * @param   {System.Collections.Specialized.NotifyCollectionChangedAction}    action           The action that caused the event. This can be set to System.Collections.Specialized.NotifyCollectionChangedAction.Reset,
             System.Collections.Specialized.NotifyCollectionChangedAction.Add, or System.Collections.Specialized.NotifyCollectionChangedAction.Remove.
             * @param   {System.Collections.IList}                                        changedItems     The items affected by the change.
             * @param   {number}                                                          startingIndex    The index where the change occurred.
             * @return  {void}
             */
            $ctor4: function (action, changedItems, startingIndex) {
                this.$initialize();
                System.Object.call(this);
                if ((action !== System.Collections.Specialized.NotifyCollectionChangedAction.Add && action !== System.Collections.Specialized.NotifyCollectionChangedAction.Remove && action !== System.Collections.Specialized.NotifyCollectionChangedAction.Reset) || (action === System.Collections.Specialized.NotifyCollectionChangedAction.Reset && (changedItems != null || startingIndex !== -1)) || (action === System.Collections.Specialized.NotifyCollectionChangedAction.Add || action === System.Collections.Specialized.NotifyCollectionChangedAction.Remove) && startingIndex < 0) {
                    throw new System.ArgumentException.ctor();
                }
                if ((action === System.Collections.Specialized.NotifyCollectionChangedAction.Add || action === System.Collections.Specialized.NotifyCollectionChangedAction.Remove) && changedItems == null) {
                    throw new System.ArgumentNullException.ctor();
                }
                if (action === System.Collections.Specialized.NotifyCollectionChangedAction.Add) {
                    this.NewItems = changedItems;
                    this.NewStartingIndex = startingIndex;
                } else {
                    this.OldItems = changedItems;
                    this.OldStartingIndex = startingIndex;
                }

                this.Action = action;
            },
            /**
             * Initializes a new instance of the System.Collections.Specialized.NotifyCollectionChangedEventArgs class that describes a one-item change.
             Exceptions:
               System.ArgumentException:
                 If action is not Reset, Add, or Remove, or if action is Reset and either
                 changedItems is not null or index is not -1.
             *
             * @instance
             * @public
             * @this System.Collections.Specialized.NotifyCollectionChangedEventArgs
             * @memberof System.Collections.Specialized.NotifyCollectionChangedEventArgs
             * @param   {System.Collections.Specialized.NotifyCollectionChangedAction}    action         The action that caused the event. This can be set to System.Collections.Specialized.NotifyCollectionChangedAction.Reset,
             System.Collections.Specialized.NotifyCollectionChangedAction.Add, or System.Collections.Specialized.NotifyCollectionChangedAction.Remove.
             * @param   {System.Object}                                                   changedItem    The item that is affected by the change.
             * @param   {number}                                                          index          The index where the change occurred.
             * @return  {void}
             */
            $ctor7: function (action, changedItem, index) {
                this.$initialize();
                System.Object.call(this);
                if ((action !== System.Collections.Specialized.NotifyCollectionChangedAction.Add && action !== System.Collections.Specialized.NotifyCollectionChangedAction.Remove && action !== System.Collections.Specialized.NotifyCollectionChangedAction.Reset) || (action === System.Collections.Specialized.NotifyCollectionChangedAction.Reset && (changedItem != null || index !== -1))) {
                    throw new System.ArgumentException.ctor();
                }

                if (action === System.Collections.Specialized.NotifyCollectionChangedAction.Add) {
                    this.NewItems = new (System.Collections.Generic.List$1(System.Object)).ctor();
                    System.Array.add(this.NewItems, changedItem, Object);
                    this.NewStartingIndex = index;
                }
                if (action === System.Collections.Specialized.NotifyCollectionChangedAction.Remove || action === System.Collections.Specialized.NotifyCollectionChangedAction.Reset) {
                    this.OldItems = new (System.Collections.Generic.List$1(System.Object)).ctor();
                    System.Array.add(this.OldItems, changedItem, Object);
                    this.OldStartingIndex = index;
                }

                this.Action = action;
            },
            /**
             * Initializes a new instance of the System.Collections.Specialized.NotifyCollectionChangedEventArgs class
             that describes a one-item System.Collections.Specialized.NotifyCollectionChangedAction.Replace change.
             System.ArgumentException:
               If action is not Replace.
             *
             * @instance
             * @public
             * @this System.Collections.Specialized.NotifyCollectionChangedEventArgs
             * @memberof System.Collections.Specialized.NotifyCollectionChangedEventArgs
             * @param   {System.Collections.Specialized.NotifyCollectionChangedAction}    action     The action that caused the event. This can only be set to System.Collections.Specialized.NotifyCollectionChangedAction.Replace.
             * @param   {System.Object}                                                   newItem    The new item that is replacing the original item.
             * @param   {System.Object}                                                   oldItem    The original item that is replaced.
             * @return  {void}
             */
            $ctor9: function (action, newItem, oldItem) {
                this.$initialize();
                System.Object.call(this);
                if (action !== System.Collections.Specialized.NotifyCollectionChangedAction.Replace) {
                    throw new System.ArgumentException.ctor();
                }
                this.NewItems = new (System.Collections.Generic.List$1(System.Object)).ctor();
                System.Array.add(this.NewItems, newItem, Object);

                this.OldItems = new (System.Collections.Generic.List$1(System.Object)).ctor();
                System.Array.add(this.OldItems, oldItem, Object);

                this.Action = action;
            },
            /**
             * Initializes a new instance of the System.Collections.Specialized.NotifyCollectionChangedEventArgs class
             that describes a multi-item System.Collections.Specialized.NotifyCollectionChangedAction.Replace change.
             Exceptions:
               System.ArgumentException:
                 If action is not Replace.
               System.ArgumentNullException:
                 If oldItems or newItems is null.
             *
             * @instance
             * @public
             * @this System.Collections.Specialized.NotifyCollectionChangedEventArgs
             * @memberof System.Collections.Specialized.NotifyCollectionChangedEventArgs
             * @param   {System.Collections.Specialized.NotifyCollectionChangedAction}    action           The action that caused the event. This can only be set to System.Collections.Specialized.NotifyCollectionChangedAction.Replace.
             * @param   {System.Collections.IList}                                        newItems         The new items that are replacing the original items.
             * @param   {System.Collections.IList}                                        oldItems         The original items that are replaced.
             * @param   {number}                                                          startingIndex    The index of the first item of the items that are being replaced.
             * @return  {void}
             */
            $ctor3: function (action, newItems, oldItems, startingIndex) {
                this.$initialize();
                System.Object.call(this);
                if (action !== System.Collections.Specialized.NotifyCollectionChangedAction.Replace) {
                    throw new System.ArgumentException.ctor();
                }

                this.NewItems = newItems;
                this.OldItems = oldItems;
                this.OldStartingIndex = startingIndex;
                this.Action = action;
            },
            /**
             * Initializes a new instance of the System.Collections.Specialized.NotifyCollectionChangedEventArgs
             class that describes a multi-item System.Collections.Specialized.NotifyCollectionChangedAction.Move
             change.
             Exceptions:
               System.ArgumentException:
                 If action is not Move or index is less than 0.
             *
             * @instance
             * @public
             * @this System.Collections.Specialized.NotifyCollectionChangedEventArgs
             * @memberof System.Collections.Specialized.NotifyCollectionChangedEventArgs
             * @param   {System.Collections.Specialized.NotifyCollectionChangedAction}    action          The action that caused the event. This can only be set to System.Collections.Specialized.NotifyCollectionChangedAction.Move.
             * @param   {System.Collections.IList}                                        changedItems    The items affected by the change.
             * @param   {number}                                                          index           The new index for the changed items.
             * @param   {number}                                                          oldIndex        The old index for the changed items.
             * @return  {void}
             */
            $ctor5: function (action, changedItems, index, oldIndex) {
                this.$initialize();
                System.Object.call(this);
                if (action !== System.Collections.Specialized.NotifyCollectionChangedAction.Move || index < 0) {
                    throw new System.ArgumentException.ctor();
                }
                this.OldItems = changedItems;
                this.NewStartingIndex = index;
                this.OldStartingIndex = oldIndex;
                this.Action = action;
            },
            /**
             * Initializes a new instance of the System.Collections.Specialized.NotifyCollectionChangedEventArgs
             class that describes a one-item System.Collections.Specialized.NotifyCollectionChangedAction.Move
             change.
             Exceptions:
               System.ArgumentException:
                 If action is not Move or index is less than 0.
             *
             * @instance
             * @public
             * @this System.Collections.Specialized.NotifyCollectionChangedEventArgs
             * @memberof System.Collections.Specialized.NotifyCollectionChangedEventArgs
             * @param   {System.Collections.Specialized.NotifyCollectionChangedAction}    action         The action that caused the event. This can only be set to System.Collections.Specialized.NotifyCollectionChangedAction.Move.
             * @param   {System.Object}                                                   changedItem    The item affected by the change.
             * @param   {number}                                                          index          The new index for the changed item.
             * @param   {number}                                                          oldIndex       The old index for the changed item.
             * @return  {void}
             */
            $ctor8: function (action, changedItem, index, oldIndex) {
                this.$initialize();
                System.Object.call(this);
                if (action !== System.Collections.Specialized.NotifyCollectionChangedAction.Move || index < 0) {
                    throw new System.ArgumentException.ctor();
                }
                this.OldItems = new (System.Collections.Generic.List$1(System.Object)).ctor();
                System.Array.add(this.OldItems, changedItem, Object);
                this.NewStartingIndex = index;
                this.OldStartingIndex = oldIndex;
                this.Action = action;
            },
            /**
             * Initializes a new instance of the System.Collections.Specialized.NotifyCollectionChangedEventArgs
             class that describes a one-item System.Collections.Specialized.NotifyCollectionChangedAction.Replace
             change.
             Exceptions:
               System.ArgumentException:
                 If action is not Replace.
             *
             * @instance
             * @public
             * @this System.Collections.Specialized.NotifyCollectionChangedEventArgs
             * @memberof System.Collections.Specialized.NotifyCollectionChangedEventArgs
             * @param   {System.Collections.Specialized.NotifyCollectionChangedAction}    action     The action that caused the event. This can be set to System.Collections.Specialized.NotifyCollectionChangedAction.Replace.
             * @param   {System.Object}                                                   newItem    The new item that is replacing the original item.
             * @param   {System.Object}                                                   oldItem    The original item that is replaced.
             * @param   {number}                                                          index      The index of the item being replaced.
             * @return  {void}
             */
            $ctor10: function (action, newItem, oldItem, index) {
                this.$initialize();
                System.Object.call(this);
                if (action !== System.Collections.Specialized.NotifyCollectionChangedAction.Replace) {
                    throw new System.ArgumentException.ctor();
                }

                this.NewItems = new (System.Collections.Generic.List$1(System.Object)).ctor();
                System.Array.add(this.NewItems, newItem, Object);

                this.OldItems = new (System.Collections.Generic.List$1(System.Object)).ctor();
                System.Array.add(this.OldItems, oldItem, Object);

                this.OldStartingIndex = index;
                this.Action = action;
            }
        }
    });

    Bridge.define("System.ComponentModel.AsyncCompletedEventArgs", {
        fields: {
            cancelled: false,
            error: null,
            userState: null
        },
        props: {
            Cancelled: {
                get: function () {
                    return this.cancelled;
                }
            },
            Error: {
                get: function () {
                    return this.error;
                }
            },
            UserState: {
                get: function () {
                    return this.userState;
                }
            }
        },
        ctors: {
            ctor: function (error, cancelled, userState) {
                this.$initialize();
                System.Object.call(this);
                this.error = error;
                this.cancelled = cancelled;
                this.userState = userState;
            }
        },
        methods: {
            RaiseExceptionIfNecessary: function () {
                if (this.Error != null) {
                    throw new System.Reflection.TargetInvocationException.$ctor1("Exception Occurred", this.Error);
                } else if (this.Cancelled) {
                    throw new System.InvalidOperationException.$ctor1("Operation Cancelled");
                }
            }
        }
    });

    /**
     * Provides data for a cancelable event.
     *
     * @public
     * @class System.ComponentModel.CancelEventArgs
     * @augments System.Object
     */
    Bridge.define("System.ComponentModel.CancelEventArgs", {
        fields: {
            _cancel: false
        },
        props: {
            /**
             * Gets or sets a value indicating whether the event should be canceled.
             *
             * @instance
             * @public
             * @memberof System.ComponentModel.CancelEventArgs
             * @function Cancel
             * @type boolean
             */
            Cancel: {
                get: function () {
                    return this._cancel;
                },
                set: function (value) {
                    this._cancel = value;
                }
            }
        },
        ctors: {
            /**
             * Initializes a new instance of the System.ComponentModel.CancelEventArgs class
             with the System.ComponentModel.CancelEventArgs.Cancel property set to false.
             *
             * @instance
             * @public
             * @this System.ComponentModel.CancelEventArgs
             * @memberof System.ComponentModel.CancelEventArgs
             * @return  {void}
             */
            ctor: function () {
                System.ComponentModel.CancelEventArgs.$ctor1.call(this, false);
            },
            /**
             * Initializes a new instance of the System.ComponentModel.CancelEventArgs class
             with the System.ComponentModel.CancelEventArgs.Cancel property set to the
             given value.
             *
             * @instance
             * @public
             * @this System.ComponentModel.CancelEventArgs
             * @memberof System.ComponentModel.CancelEventArgs
             * @param   {boolean}    cancel    true to cancel the event; otherwise, false.
             * @return  {void}
             */
            $ctor1: function (cancel) {
                this.$initialize();
                System.Object.call(this);
                this._cancel = cancel;
            }
        }
    });

    Bridge.define("System.ComponentModel.CurrentChangingEventArgs", {
        fields: {
            _cancel: false,
            IsCancelable: false
        },
        props: {
            Cancel: {
                get: function () {
                    return this._cancel;
                },
                set: function (value) {
                    if (!this.IsCancelable) {
                        throw new System.InvalidOperationException.$ctor1("The IsCancelable property value is false.");
                    }
                    this._cancel = value;
                }
            }
        },
        ctors: {
            ctor: function () {
                this.$initialize();
                System.Object.call(this);
                this.IsCancelable = true;
            },
            $ctor1: function (isCancelable) {
                this.$initialize();
                System.Object.call(this);
                this.IsCancelable = isCancelable;
            }
        }
    });

    Bridge.define("System.ComponentModel.DescriptionAttribute", {
        inherits: [System.Attribute],
        statics: {
            fields: {
                default: null
            },
            ctors: {
                init: function () {
                    this.default = new System.ComponentModel.DescriptionAttribute.ctor();
                }
            }
        },
        fields: {
            description: null
        },
        props: {
            Description: {
                get: function () {
                    return this.DescriptionValue;
                }
            },
            DescriptionValue: {
                get: function () {
                    return this.description;
                },
                set: function (value) {
                    this.description = value;
                }
            }
        },
        ctors: {
            ctor: function () {
                System.ComponentModel.DescriptionAttribute.$ctor1.call(this, "");
            },
            $ctor1: function (description) {
                this.$initialize();
                System.Attribute.ctor.call(this);
                this.description = description;
            }
        },
        methods: {
            equals: function (obj) {
                if (Bridge.referenceEquals(obj, this)) {
                    return true;
                }

                var other = Bridge.as(obj, System.ComponentModel.DescriptionAttribute);

                return (other != null) && Bridge.referenceEquals(other.Description, this.Description);
            },
            getHashCode: function () {
                return Bridge.getHashCode(this.Description);
            }
        }
    });

    /**
     * Provides attached properties that can be used to communicate with a designer.
     * // Gets or sets a value that indicates whether all the metadata associated with
    // an assembly or just the xmlns namespace definitions should be refreshed in
    // the designer when an assembly is recompiled.
    //
     *
     * @static
     * @abstract
     * @public
     * @class System.ComponentModel.DesignerProperties
     */
    Bridge.define("System.ComponentModel.DesignerProperties", {
        statics: {
            fields: {
                /**
                 * Identifies the System.ComponentModel.DesignerProperties.IsInDesignMode attached
                 property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof System.ComponentModel.DesignerProperties
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                IsInDesignModeProperty: null
            },
            props: {
                /**
                 * Gets a value that indicates whether the element is running in the context
                 of a designer.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof System.ComponentModel.DesignerProperties
                 * @function IsInDesignTool
                 * @type boolean
                 */
                IsInDesignTool: {
                    get: function () {
                        return false;
                    }
                }
            },
            ctors: {
                init: function () {
                    this.IsInDesignModeProperty = Windows.UI.Xaml.DependencyProperty.RegisterAttached("IsInDesignMode", System.Boolean, System.ComponentModel.DesignerProperties, new Windows.UI.Xaml.PropertyMetadata.$ctor1(Bridge.box(false, System.Boolean, System.Boolean.toString)));
                }
            },
            methods: {
                /**
                 * Gets the value of the System.ComponentModel.DesignerProperties.IsInDesignMode
                 attached property for the specified System.Windows.UIElement.
                 *
                 * @static
                 * @public
                 * @this System.ComponentModel.DesignerProperties
                 * @memberof System.ComponentModel.DesignerProperties
                 * @param   {Windows.UI.Xaml.DependencyObject}    element    The element from which the property value is read.
                 * @return  {boolean}                                        The System.ComponentModel.DesignerProperties.IsInDesignMode property value
                 for the element.
                 */
                GetIsInDesignMode: function (element) {
                    if (element == null) {
                        throw new System.ArgumentNullException.$ctor1("The element parameter cannot be null");
                    }
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(element.GetValue(System.ComponentModel.DesignerProperties.IsInDesignModeProperty), System.Boolean), System.Boolean));
                },
                /**
                 * Sets the value of the System.ComponentModel.DesignerProperties.IsInDesignMode
                 attached property to a specified element.
                 *
                 * @static
                 * @public
                 * @this System.ComponentModel.DesignerProperties
                 * @memberof System.ComponentModel.DesignerProperties
                 * @param   {Windows.UI.Xaml.DependencyObject}    element    The element to which the attached property is written.
                 * @param   {boolean}                             value      The needed System.Boolean value.
                 * @return  {void}
                 */
                SetIsInDesignMode: function (element, value) {
                    if (element == null) {
                        throw new System.ArgumentNullException.$ctor1("The element parameter cannot be null");
                    }
                    element.SetValue(System.ComponentModel.DesignerProperties.IsInDesignModeProperty, Bridge.box(value, System.Boolean, System.Boolean.toString));
                }
            }
        }
    });

    /**
     * Defines methods and properties that a collection view implements to provide paging capabilities to a collection.
     *
     * @abstract
     * @public
     * @class System.ComponentModel.IPagedCollectionView
     */
    Bridge.define("System.ComponentModel.IPagedCollectionView", {
        $kind: "interface"
    });

    Bridge.define("System.ComponentModel.ListSortDirection", {
        $kind: "enum",
        statics: {
            fields: {
                Ascending: 0,
                Descending: 1
            }
        }
    });

    /**
     * The {@link } namespaces contain types that implement
     the run-time and design-time behavior of components and controls. Child namespaces support
     the Managed Extensibility Framework (MEF), provide attribute classes that define metadata
     for ASP.NET Dynamic Data controls, and contain types that let you define the design-time
     behavior of components and their user interfaces.
     *
     * @class System.ComponentModel.NamespaceDoc
     */
    Bridge.define("System.ComponentModel.NamespaceDoc");

    /**
     * @public
     * @class System.ComponentModel.ReadOnlyAttribute
     * @augments System.Attribute
     */
    Bridge.define("System.ComponentModel.ReadOnlyAttribute", {
        inherits: [System.Attribute],
        statics: {
            fields: {
                /**
                 * @static
                 * @public
                 * @readonly
                 * @memberof System.ComponentModel.ReadOnlyAttribute
                 * @type System.ComponentModel.ReadOnlyAttribute
                 */
                yes: null,
                /**
                 * @static
                 * @public
                 * @readonly
                 * @memberof System.ComponentModel.ReadOnlyAttribute
                 * @type System.ComponentModel.ReadOnlyAttribute
                 */
                no: null,
                /**
                 * @static
                 * @public
                 * @readonly
                 * @memberof System.ComponentModel.ReadOnlyAttribute
                 * @type System.ComponentModel.ReadOnlyAttribute
                 */
                default: null
            },
            ctors: {
                init: function () {
                    this.yes = new System.ComponentModel.ReadOnlyAttribute(true);
                    this.no = new System.ComponentModel.ReadOnlyAttribute(false);
                    this.default = System.ComponentModel.ReadOnlyAttribute.no;
                }
            }
        },
        fields: {
            isReadOnly: false
        },
        props: {
            /**
             * @instance
             * @public
             * @readonly
             * @memberof System.ComponentModel.ReadOnlyAttribute
             * @function IsReadOnly
             * @type boolean
             */
            IsReadOnly: {
                get: function () {
                    return this.isReadOnly;
                }
            }
        },
        ctors: {
            init: function () {
                this.isReadOnly = false;
            },
            /**
             * @instance
             * @public
             * @this System.ComponentModel.ReadOnlyAttribute
             * @memberof System.ComponentModel.ReadOnlyAttribute
             * @param   {boolean}    isReadOnly
             * @return  {void}
             */
            ctor: function (isReadOnly) {
                this.$initialize();
                System.Attribute.ctor.call(this);
                this.isReadOnly = isReadOnly;
            }
        },
        methods: {
            /**
             * @instance
             * @public
             * @override
             * @this System.ComponentModel.ReadOnlyAttribute
             * @memberof System.ComponentModel.ReadOnlyAttribute
             * @param   {System.Object}    value
             * @return  {boolean}
             */
            equals: function (value) {

                if (Bridge.referenceEquals(this, value)) {
                    return true;
                } else {
                    var other = Bridge.as(value, System.ComponentModel.ReadOnlyAttribute);
                    return other != null && other.IsReadOnly === this.IsReadOnly;
                }

            },
            /**
             * @instance
             * @public
             * @override
             * @this System.ComponentModel.ReadOnlyAttribute
             * @memberof System.ComponentModel.ReadOnlyAttribute
             * @return  {number}
             */
            getHashCode: function () {
                return Bridge.getHashCode(this);
            }
        }
    });

    /** @namespace System.Configuration */

    /**
     * The {@link } contain types for handling configuration data,
     such as data in machine or application configuration files. Child namespaces contain
     types that are used to configure an assembly, to write custom installers for components,
     and to support a pluggable model for adding functionality to, or removing functionality
     from, both client and server applications.
     *
     * @class System.Configuration.NamespaceDoc
     */
    Bridge.define("System.Configuration.NamespaceDoc");

    /** @namespace System.Diagnostics */

    /**
     * The {@link } namespaces contain types that enable you to
     interact with system processes, event logs, and performance counters. Child namespaces
     contain types to interact with code analysis tools, to support contracts, to extend
     design-time support for application monitoring and instrumentation, to log event data
     using the Event Tracing for Windows (ETW) tracing subsystem, to read to and write from
     event logs and collect performance data, and to read and write debug symbol information.
     *
     * @class System.Diagnostics.NamespaceDoc
     */
    Bridge.define("System.Diagnostics.NamespaceDoc");

    Bridge.define("System.Diagnostics.StackFrame", {
        ctors: {
            ctor: function (skipFrames) {
                this.$initialize();
            }
        },
        methods: {
            GetMethod: function () {
                return Bridge.Reflection.getMembers(Bridge.getType(this), 8, 284, "Error");
            },
            Error: function () {
                throw new System.NotImplementedException.$ctor1("StackFrame error - Error - ");
            }
        }
    });

    /** @namespace System.Globalization */

    /**
     * The {@link } namespace contains classes that define
     culture-related information, including language, country/region, calendars in use,
     format patterns for dates, currency, and numbers, and sort order for strings. These
     classes are useful for writing globalized (internationalized) applications. Classes
     such as StringInfo and TextInfo provide advanced globalization functionalities,
     including surrogate support and text element processing.
     *
     * @class System.Globalization.NamespaceDoc
     */
    Bridge.define("System.Globalization.NamespaceDoc");

    Bridge.define("System.Globalization.NumberStyles", {
        $kind: "enum",
        statics: {
            fields: {
                None: 0,
                AllowLeadingWhite: 1,
                AllowTrailingWhite: 2,
                AllowLeadingSign: 4,
                Integer: 7,
                AllowTrailingSign: 8,
                AllowParentheses: 16,
                AllowDecimalPoint: 32,
                AllowThousands: 64,
                Number: 111,
                AllowExponent: 128,
                Float: 167,
                AllowCurrencySymbol: 256,
                Currency: 383,
                Any: 511,
                AllowHexSpecifier: 512,
                HexNumber: 515
            }
        },
        $flags: true
    });

    /** @namespace System.Net */

    /**
     * @memberof System.Net
     * @callback System.Net.INTERNAL_WebRequestHelper_JSOnly_RequestCompletedEventHandler
     * @param   {System.Object}                                                            sender    
     * @param   {System.Net.INTERNAL_WebRequestHelper_JSOnly_RequestCompletedEventArgs}    e
     * @return  {void}
     */

    Bridge.define("System.INTERNAL_WebRequestHelper_JSOnly", {
        statics: {
            fields: {
                WebServiceUrlToJsCredentialsSupported: null,
                _sender: null,
                _requester: null
            },
            ctors: {
                init: function () {
                    this.WebServiceUrlToJsCredentialsSupported = new (System.Collections.Generic.Dictionary$2(System.String,System.Boolean)).ctor();
                }
            },
            methods: {
                EscapeStringAndSurroundWithQuotes: function (str) {
                    var $t;
                    return "@\"" + (System.String.replaceAll((($t = str, $t != null ? $t : "")), "\"", "\"\"") || "") + "\"";
                },
                ResendLastUnsafeRequest: function () {
                    return new System.INTERNAL_WebRequestHelper_JSOnly().MakeRequest(System.INTERNAL_WebRequestHelper_JSOnly._requester._address, System.INTERNAL_WebRequestHelper_JSOnly._requester._Method, System.INTERNAL_WebRequestHelper_JSOnly._sender, System.INTERNAL_WebRequestHelper_JSOnly._requester._headers, System.INTERNAL_WebRequestHelper_JSOnly._requester._body, System.INTERNAL_WebRequestHelper_JSOnly._requester._callback, System.INTERNAL_WebRequestHelper_JSOnly._requester._isAsync, System.Net.CredentialsMode.Disabled);
                },
                GetHasError: function (xmlHttpRequest) {
                    var currentStatus = xmlHttpRequest.status;
                    if (currentStatus >= 400 && currentStatus < 600) {
                        return true;
                    }
                    return false;
                }
            }
        },
        fields: {
            _xmlHttpRequest: null,
            _address: null,
            _Method: null,
            _headers: null,
            _body: null,
            _isAsync: false,
            _callback: null,
            _isFirstTryAtSendingUnsafeRequest: false
        },
        events: {
            /**
             * Occurs when the string download is completed.
             *
             * @instance
             * @public
             * @this System.INTERNAL_WebRequestHelper_JSOnly
             * @memberof System.INTERNAL_WebRequestHelper_JSOnly
             * @function addDownloadStringCompleted
             * @param   {System.Net.INTERNAL_WebRequestHelper_JSOnly_RequestCompletedEventHandler}    value
             * @return  {void}
             */
            /**
             * Occurs when the string download is completed.
             *
             * @instance
             * @public
             * @this System.INTERNAL_WebRequestHelper_JSOnly
             * @memberof System.INTERNAL_WebRequestHelper_JSOnly
             * @function removeDownloadStringCompleted
             * @param   {System.Net.INTERNAL_WebRequestHelper_JSOnly_RequestCompletedEventHandler}    value
             * @return  {void}
             */
            DownloadStringCompleted: null
        },
        props: {
            IsCrashInPreflight: {
                get: function () {
                    return (this._xmlHttpRequest.readyState === 4 && this._xmlHttpRequest.status === 0);
                }
            }
        },
        ctors: {
            init: function () {
                this._isFirstTryAtSendingUnsafeRequest = false;
            },
            /**
             * Initializes a new instance of the INTERNAL_WebRequestHelper class.
             *
             * @instance
             * @public
             * @this System.INTERNAL_WebRequestHelper_JSOnly
             * @memberof System.INTERNAL_WebRequestHelper_JSOnly
             * @return  {void}
             */
            ctor: function () {
                this.$initialize();
            }
        },
        methods: {
            /**
             * Makes a synchronous or asynchronous request at the specified uri, using the specified method, with the specified headers and body, and calls the callbackMethod.
             *
             * @instance
             * @public
             * @this System.INTERNAL_WebRequestHelper_JSOnly
             * @memberof System.INTERNAL_WebRequestHelper_JSOnly
             * @param   {System.Uri}                                                                  address           the uri that identifies the Internet resource.
             * @param   {string}                                                                      Method            The method to be called after making the request.
             * @param   {System.Object}                                                               sender            
             * @param   {System.Collections.Generic.Dictionary$2}                                     headers           A dictionary containing the headers to put in the method.
             The pairs key/values in the dictionary correspond to the pairs key/value in the headers.
             * @param   {string}                                                                      body              The body of the request.
             * @param   {System.Net.INTERNAL_WebRequestHelper_JSOnly_RequestCompletedEventHandler}    callbackMethod    The method to be called after the request has been made.
             * @param   {boolean}                                                                     isAsync           A boolean that determines whether the request must be made synchronously or asynchronously.
             * @param   {System.Net.CredentialsMode}                                                  mode
             * @return  {string}                                                                                        The result of the request as a string.
             */
            MakeRequest: function (address, Method, sender, headers, body, callbackMethod, isAsync, mode) {
                var $t;
                if (mode === void 0) { mode = 0; }
                if (Windows.UI.Xaml.Application.Current.Host.Settings.EnableWebRequestsLogging) {
                    var headersCode = headers != null && headers.Count > 0 ? "new global::System.Collections.Generic.Dictionary<string, string> { " + (Bridge.toArray(System.Linq.Enumerable.from(headers).select(function (keyValuePair) {
                                return "{ " + (System.INTERNAL_WebRequestHelper_JSOnly.EscapeStringAndSurroundWithQuotes(keyValuePair.key) || "") + ", " + (System.INTERNAL_WebRequestHelper_JSOnly.EscapeStringAndSurroundWithQuotes(keyValuePair.value) || "") + " }";
                            })).join(", ") || "") + " }" : "null";

                    System.Diagnostics.Debug.WriteLine$2(System.String.format("CSHTML5.Internal.WebRequestsHelper.MakeRequest({0}, {1}, {2}, {3}, {4});", System.INTERNAL_WebRequestHelper_JSOnly.EscapeStringAndSurroundWithQuotes(Bridge.toString(address)), System.INTERNAL_WebRequestHelper_JSOnly.EscapeStringAndSurroundWithQuotes(Method), System.INTERNAL_WebRequestHelper_JSOnly.EscapeStringAndSurroundWithQuotes(body), headersCode, "false"));
                }

                var askForUnsafeRequest = false;

                this._xmlHttpRequest = new XMLHttpRequest();

                if (!Bridge.staticEquals(callbackMethod, null)) {
                    this.removeDownloadStringCompleted(callbackMethod);
                    this.addDownloadStringCompleted(callbackMethod);
                }
                this._xmlHttpRequest.onloadend = Bridge.fn.cacheBind(this, this.OnDownloadStringCompleted);

                this._xmlHttpRequest.open(Method, address.getOriginalString(), isAsync);

                if (!System.INTERNAL_WebRequestHelper_JSOnly.WebServiceUrlToJsCredentialsSupported.containsKey(address.getOriginalString())) {
                    if (mode === System.Net.CredentialsMode.Auto) {
                        System.INTERNAL_WebRequestHelper_JSOnly.WebServiceUrlToJsCredentialsSupported.add(address.getOriginalString(), true);
                        askForUnsafeRequest = true;
                    } else {
                        System.INTERNAL_WebRequestHelper_JSOnly.WebServiceUrlToJsCredentialsSupported.add(address.getOriginalString(), mode === System.Net.CredentialsMode.Disabled ? false : true);
                    }
                }

                this._xmlHttpRequest.withCredentials = System.INTERNAL_WebRequestHelper_JSOnly.WebServiceUrlToJsCredentialsSupported.getItem(address.getOriginalString());

                if (headers != null && headers.Count > 0) {
                    $t = Bridge.getEnumerator(headers.Keys);
                    try {
                        while ($t.moveNext()) {
                            var key = $t.Current;
                            this._xmlHttpRequest.setRequestHeader(key,headers.getItem(key));
                        }
                    } finally {
                        if (Bridge.is($t, System.IDisposable)) {
                            $t.System$IDisposable$Dispose();
                        }
                    }
                }

                if (askForUnsafeRequest) {
                    this._xmlHttpRequest.onerror = Bridge.fn.cacheBind(this, this.OnError);

                    this.SaveParameters(address, Method, sender, headers, callbackMethod, body, isAsync);

                    return this.SendUnsafeRequest(this._xmlHttpRequest, address.getOriginalString(), Method, isAsync, body);
                } else {
                    this._xmlHttpRequest.send(body);
                }

                var result = this._xmlHttpRequest.responseText;

                if (System.INTERNAL_WebRequestHelper_JSOnly.GetHasError(this._xmlHttpRequest)) {
                    if (System.String.indexOf(result, ":Fault>") === -1) {
                        var errorMessage = "The remote server has returned an error: (" + this._xmlHttpRequest.status + ") " + (this._xmlHttpRequest.statusText || "") + ".";
                        var exception = new System.Net.WebException.$ctor4(errorMessage, null, System.Net.WebExceptionStatus.ProtocolError, new System.Net.HttpWebResponse(this.GetXmlHttpRequest()));
                        throw exception;
                    }
                } else {
                    var currentReadyState = this._xmlHttpRequest.readyState;
                    var currentStatus = this._xmlHttpRequest.status;
                    if ((currentStatus === 0 && !(document.location.protocol === "file:")) && (currentReadyState === 4 || currentReadyState === 1)) {
                        if (!isAsync) {
                            throw new System.ServiceModel.CommunicationException.$ctor1("An error occured. Please make sure that the target URL is available: " + (Bridge.toString(address) || ""));
                        }
                    }
                }

                return result;
            },
            SendUnsafeRequest: function (xmlHttpRequest, address, method, isAsync, body) {
                console.log("CredentialsMode is set to Auto: if a preflight error appears below, please ignore it.");

                if (!isAsync) {
                    try {
                        this._xmlHttpRequest.send(body);

                        this.ResendRequestInCaseOfPreflightError(false);
                        return this._xmlHttpRequest.responseText;
                    } catch ($e1) {
                        $e1 = System.Exception.create($e1);
                        if (this.IsCrashInPreflight) {
                            return this.ResendRequestInCaseOfPreflightError(true);
                        } else {
                            this.ResendRequestInCaseOfPreflightError(false);
                            return this._xmlHttpRequest.responseText;
                        }
                    }
                } else {
                    this._isFirstTryAtSendingUnsafeRequest = true;
                    this._xmlHttpRequest.send(body);
                    return this._xmlHttpRequest.responseText;
                }
            },
            OnError: function (sender) {
                if (this.IsCrashInPreflight) {
                    this.ResendRequestInCaseOfPreflightError(true);
                }
            },
            ResendRequestInCaseOfPreflightError: function (error) {
                var newResult = "";

                if (error) {
                    System.INTERNAL_WebRequestHelper_JSOnly.WebServiceUrlToJsCredentialsSupported.setItem(this._address.getOriginalString(), false);

                    newResult = System.INTERNAL_WebRequestHelper_JSOnly.ResendLastUnsafeRequest();

                    console.log("The requested server does not seem to accept credentials. To stop getting the error above, make sure to set CredentialsMode to Disabled. To do so with REST calls, please set the property WebClientWithCredentials.CredentialsMode to Disabled. For SOAP calls, please place the following code in your application constructor: Application.Current.Host.Settings.DefaultSoapCredentialsMode = System.Net.CredentialsMode.Disabled;");
                }

                console.log("Credentials status is now confirmed: no other preflight errors are expected for this request.");

                return newResult;
            },
            SaveParameters: function (address, Method, sender, headers, callback, body, isAsync) {
                this._address = address;
                this._Method = Method;
                System.INTERNAL_WebRequestHelper_JSOnly._sender = sender;
                this._headers = headers;
                this._body = body;
                this._isAsync = isAsync;
                this._callback = callback;
                System.INTERNAL_WebRequestHelper_JSOnly._requester = this;
            },
            OnDownloadStringCompleted: function () {
                var e = new System.Net.INTERNAL_WebRequestHelper_JSOnly_RequestCompletedEventArgs.ctor();
                this.SetEventArgs(e);
                if (!this._isFirstTryAtSendingUnsafeRequest || !this.IsCrashInPreflight) {
                    if (!Bridge.staticEquals(this.DownloadStringCompleted, null)) {
                        this.DownloadStringCompleted(System.INTERNAL_WebRequestHelper_JSOnly._sender, e);
                    }
                }
                this._isFirstTryAtSendingUnsafeRequest = false;
            },
            SetEventArgs: function (e) {
                var currentReadyState = this._xmlHttpRequest.readyState;
                var currentStatus = this._xmlHttpRequest.status;
                var errorMessage = null;
                if (System.INTERNAL_WebRequestHelper_JSOnly.GetHasError(this._xmlHttpRequest)) {
                    errorMessage = System.String.format("{0} - {1}", Bridge.box(currentStatus, System.Int32), this._xmlHttpRequest.statusText);
                } else if (currentReadyState === 0 && !e.Cancelled) {
                    errorMessage = "Request not initialized";
                } else if ((currentStatus === 0 && !(document.location.protocol === "file:")) && (currentReadyState === 4 || currentReadyState === 1)) {
                    errorMessage = "An error occured. Please make sure that the target Url is available.";
                } else if (currentReadyState === 1 && !e.Cancelled) {
                    errorMessage = "An Error occured. Cross-Site Http Request might not be allowed at the target Url. If you own the domain of the Url, consider adding the header \"Access-Control-Allow-Origin\" to enable requests to be done at this Url.";
                } else if (currentReadyState !== 4) {
                    errorMessage = "An Error has occured while submitting your request.";
                }
                if (errorMessage != null) {
                    var exception = new System.Net.WebException.$ctor4(errorMessage, null, System.Net.WebExceptionStatus.ProtocolError, new System.Net.HttpWebResponse(this.GetXmlHttpRequest()));
                    e.Error = exception;
                }
                e.Result = this._xmlHttpRequest.responseText;
            },
            GetXmlHttpRequest: function () {
                return this._xmlHttpRequest;
            }
        }
    });

    Bridge.define("System.IO.CSHTML5_File", {
        statics: {
            methods: {
                Exists: function (path) {
                    throw new System.NotImplementedException.$ctor1("File error - Exists - args: " + (path || ""));
                },
                Copy: function (sourceFileName, destFileName) {
                    throw new System.NotImplementedException.$ctor1("File error - Copy - args: " + (sourceFileName || "") + " " + (sourceFileName || ""));
                },
                Copy$1: function (sourceFileName, destFileName, overwrite) {
                    throw new System.NotImplementedException.$ctor1("File error - Copy - args: " + (sourceFileName || "") + " " + (sourceFileName || "") + ((overwrite ? "   True" : "    False") || ""));
                },
                Delete: function (path) {
                    throw new System.NotImplementedException.$ctor1("File error - Delete - args: " + (path || ""));
                },
                Move: function (sourceFileName, destFileName) {
                    throw new System.NotImplementedException.$ctor1("File error - Move - args: " + (sourceFileName || "") + " " + (sourceFileName || ""));
                },
                Open: function (path, mode) {
                    return System.IO.CSHTML5_File.Open$2(path, mode, (mode === 6 ? System.IO.FileAccess.Write : System.IO.FileAccess.ReadWrite), 0);
                },
                Open$1: function (path, mode, access) {
                    return System.IO.CSHTML5_File.Open$2(path, mode, access, 0);
                },
                Open$2: function (path, mode, access, share) {
                    throw new System.NotImplementedException.$ctor1("File error - Open - args: " + (path || "") + " " + System.Enum.toString(System.IO.FileMode, mode) + " " + System.Enum.toString(System.IO.FileAccess, access) + " " + System.Enum.toString(System.IO.FileShare, share));
                },
                Create: function (path) {
                    throw new System.NotImplementedException.$ctor1("File error - Open - args: " + (path || ""));
                    /* 
                    return Create(path, FileStream.DefaultBufferSize);
                    */
                },
                Create$1: function (path, bufferSize) {
                    throw new System.NotImplementedException.$ctor1("File error - Open - args: " + (path || "") + " " + bufferSize);
                    /* 
                    return new FileStream(path, FileMode.Create, FileAccess.ReadWrite, FileShare.None, bufferSize);
                    */
                },
                Create$2: function (path, bufferSize, options) {
                    throw new System.NotImplementedException.$ctor1("File error - Open - args: " + (path || "") + " " + bufferSize + " " + System.Enum.toString(System.IO.FileOptions, options));
                    /* 
                    return new FileStream(path, FileMode.Create, FileAccess.ReadWrite,
                                         FileShare.None, bufferSize, options);
                    */
                }
            }
        }
    });

    Bridge.define("System.IO.Directory", {
        statics: {
            methods: {
                CreateDirectory: function (path) {
                    return new System.IO.DirectoryInfo(path);
                },
                Exists: function (path) {
                    throw new System.NotImplementedException.$ctor1("Directory error - Exist - args: " + (path || ""));
                },
                GetDirectories: function (path) {
                    throw new System.NotImplementedException.$ctor1("Directory error - GetDirectories - args: " + (path || ""));
                },
                GetFiles: function (path) {
                    throw new System.NotImplementedException.$ctor1("Directory error - GetFiles - args: " + (path || ""));
                }
            }
        }
    });

    Bridge.define("System.IO.DirectoryInfo", {
        fields: {
            FullPath: null,
            OriginalPath: null,
            _displayPath: null
        },
        props: {
            FullName: {
                get: function () {
                    return this.FullPath;
                }
            },
            Name: {
                get: function () {
                    return System.IO.Path.GetFileName(this.FullPath);
                }
            }
        },
        ctors: {
            init: function () {
                this._displayPath = "";
            },
            ctor: function (path) {
                this.$initialize();
                this.OriginalPath = path;
            }
        }
    });

    Bridge.define("System.IO.FileAccess", {
        $kind: "enum",
        statics: {
            fields: {
                Read: 1,
                Write: 2,
                ReadWrite: 3
            }
        },
        $flags: true
    });

    Bridge.define("System.IO.FileInfo", {
        fields: {
            FullPath: null,
            OriginalPath: null,
            _displayPath: null
        },
        props: {
            FullName: {
                get: function () {
                    return this.FullPath;
                }
            },
            Name: {
                get: function () {
                    return System.IO.Path.GetFileName(this.FullPath);
                }
            },
            Exists: {
                get: function () {
                    throw new System.NotImplementedException.$ctor1("FileInfo error - Exist - ");
                }
            },
            Extension: {
                get: function () {
                    var split = System.String.split(this.FullPath, [46].map(function (i) {{ return String.fromCharCode(i); }}));
                    if (split.length > 0) {
                        return split[System.Array.index(((split.length - 1) | 0), split)];
                    }
                    return "";
                }
            }
        },
        ctors: {
            init: function () {
                this._displayPath = "";
            },
            ctor: function (fileName) {
                this.$initialize();
                this.OriginalPath = fileName;
                this.FullPath = fileName;
            }
        },
        methods: {
            Open: function (mode, access, share) {
                return null;
            }
        }
    });

    Bridge.define("System.IO.FileNotFoundException", {
        inherits: [System.IO.IOException],
        ctors: {
            ctor: function () {
                this.$initialize();
                System.IO.IOException.ctor.call(this);
            },
            $ctor1: function (message) {
                this.$initialize();
                System.IO.IOException.$ctor1.call(this, message);
            }
        }
    });

    Bridge.define("System.IO.IsolatedStorage.IsolatedStorageFile", {
        inherits: [System.IDisposable],
        statics: {
            methods: {
                GetUserStoreForAssembly: function () {
                    var $t;
                    return ($t = new System.IO.IsolatedStorage.IsolatedStorageFile(), $t._assemblyName = $asm.name, $t);
                }
            }
        },
        fields: {
            _assemblyName: null
        },
        alias: ["Dispose", "System$IDisposable$Dispose"],
        methods: {
            CreateFile: function (fileName) {
                return new System.IO.IsolatedStorage.IsolatedStorageFileStream(("storage_" + this._assemblyName + "/" + fileName), 2);
            },
            DeleteFile: function (fileName) {
                var filePath_lower = ("storage_" + this._assemblyName + "/" + fileName).toLowerCase();
localStorage.removeItem(                (filePath_lower || "") + "\u01c0\u01c0CaseSensitivePath"); localStorage.removeItem(filePath_lower);
            },
            FileExists: function (fileName) {
                var filePath_lower = ("storage_" + this._assemblyName + "/" + fileName).toLowerCase();
                return System.Convert.toBoolean(localStorage.getItem(filePath_lower) != null);
            },
            OpenFile: function (fileName, fileMode) {
                return new System.IO.IsolatedStorage.IsolatedStorageFileStream(("storage_" + this._assemblyName + "/" + fileName), 3);
            },
            Dispose: function () { }
        }
    });

    Bridge.define("System.IO.IsolatedStorage.IsolatedStorageFileStream", {
        inherits: [System.IO.FileStream,System.IDisposable],
        fields: {
            _filePath: null,
            _fs: null
        },
        props: {
            Position: {
                get: function () {
                    return this._fs.Position;
                },
                set: function (value) {
                    this._fs.Position = value;
                }
            },
            Length: {
                get: function () {
                    return this._fs.Length;
                }
            }
        },
        ctors: {
            ctor: function (path, mode) {
                this.$initialize();
                System.IO.FileStream.$ctor1.call(this, path, mode);
                this._filePath = path;

                this._fs = new System.IO.MemoryStream.ctor();
                if (mode === 3 || mode === 4) {
                    var fileContent = System.Convert.toString(localStorage.getItem(this._filePath.toLowerCase()));
                    if (fileContent != null) {
                        var bytes = System.Convert.fromBase64String(fileContent);
                        this._fs.Write(bytes, 0, bytes.length);
                        this._fs.Position = System.Int64(0);
                    }
                }
            }
        },
        methods: {
            /**
             * Writes a block of bytes to the System.IO.IsolatedStorage.IsolatedStorageFileStream
             object using data read from a byte array.
             *
             * @instance
             * @public
             * @override
             * @this System.IO.IsolatedStorage.IsolatedStorageFileStream
             * @memberof System.IO.IsolatedStorage.IsolatedStorageFileStream
             * @param   {Array.<number>}    buffer    The buffer to write.
             * @param   {number}            offset    The byte offset in buffer from which to begin.
             * @param   {number}            count     The maximum number of bytes to write.
             * @return  {void}
             */
            Write: function (buffer, offset, count) {
                this._fs.Write(buffer, offset, count);
                this._fs.Position = System.Int64(0);

                var msLenght = this.Length;
                var bytes = System.Array.init(msLenght, 0, System.Byte);
                this._fs.Read(bytes, 0, System.Int64.clip32(msLenght));

                var filePath_lowered = this._filePath.toLowerCase();
localStorage.setItem(                (filePath_lowered || "") + "\u01c0\u01c0CaseSensitivePath", this._filePath); localStorage.setItem((filePath_lowered || "") + "\u01c0\u01c0CaseSensitivePath", System.Convert.toBase64String(bytes, null, null, null));
            },
            /**
             * Copies bytes from the current buffered System.IO.IsolatedStorage.IsolatedStorageFileStream
             object to an array.
             *
             * @instance
             * @public
             * @override
             * @this System.IO.IsolatedStorage.IsolatedStorageFileStream
             * @memberof System.IO.IsolatedStorage.IsolatedStorageFileStream
             * @param   {Array.<number>}    buffer    The buffer to read.
             * @param   {number}            offset    The offset in the buffer at which to begin writing.
             * @param   {number}            count     The maximum number of bytes to read.
             * @return  {number}                      The total number of bytes read into the buffer. This can be less than the
             number of bytes requested if that many bytes are not currently available,
             or zero if the end of the stream is reached.
             */
            Read: function (buffer, offset, count) {
                return this._fs.Read(buffer, offset, count);
            },
            Dispose$1: function (disposing) {
                if (this._fs != null) {
                    this._fs.Dispose();
                }
                System.IO.FileStream.prototype.Dispose$1.call(this, disposing);
            },
            Dispose$2: function () {
                this.Close();
            },
            Flush: function () {
                this._fs.Flush();
                System.IO.FileStream.prototype.Flush.call(this);
            },
            ReadByte: function () {
                return this._fs.ReadByte();
            },
            Seek: function (offset, origin) {
                return this._fs.Seek(offset, origin);
            },
            WriteByte: function (value) {
                this._fs.WriteByte(value);
            },
            Close: function () {
                this._fs.Close();
                System.IO.FileStream.prototype.Close.call(this);
            }
        }
    });

    /** @namespace System.IO.IsolatedStorage */

    /**
     * Provides a System.Collections.Generic.Dictionary&lt;TKey,TValue&gt; that stores
     key-value pairs in isolated storage.
     *
     * @public
     * @class System.IO.IsolatedStorage.IsolatedStorageSettings
     * @implements  System.Collections.IEnumerable
     * @implements  System.Collections.Generic.IEnumerable$1
     * @example Here is how to use the IsolatedStorageSettings:
     *
     * //Write in the IsolatedStorageSettings:
     * IsolatedStorageSettings.ApplicationSettings["someKey"] = "someValue";
     * //Read from it:
     * string value;
     * string myString = IsolatedStorageSettings.ApplicationSettings.TryGetValue("someKey", out value);
     * 
     *
     *
     */
    Bridge.define("System.IO.IsolatedStorage.IsolatedStorageSettings", {
        inherits: [System.Collections.IEnumerable,System.Collections.Generic.IEnumerable$1(System.Collections.Generic.KeyValuePair$2(System.String,System.Object))],
        statics: {
            fields: {
                _applicationSettings: null
            },
            props: {
                /**
                 * Gets an instance of System.IO.IsolatedStorage.IsolatedStorageSettings that
                 contains the contents of the application's System.IO.IsolatedStorage.IsolatedStorageFile,
                 scoped at the application level, or creates a new instance of System.IO.IsolatedStorage.IsolatedStorageSettings
                 if one does not exist.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof System.IO.IsolatedStorage.IsolatedStorageSettings
                 * @function ApplicationSettings
                 * @type System.IO.IsolatedStorage.IsolatedStorageSettings
                 */
                ApplicationSettings: {
                    get: function () {
                        if (System.IO.IsolatedStorage.IsolatedStorageSettings._applicationSettings == null) {
                            System.IO.IsolatedStorage.IsolatedStorageSettings._applicationSettings = new System.IO.IsolatedStorage.IsolatedStorageSettings();
                        }
                        return System.IO.IsolatedStorage.IsolatedStorageSettings._applicationSettings;
                    }
                }
            }
        },
        fields: {
            _fullApplicationName: null
        },
        props: {
            /**
             * Gets the number of key-value pairs that are stored in the dictionary.
             *
             * @instance
             * @public
             * @readonly
             * @memberof System.IO.IsolatedStorage.IsolatedStorageSettings
             * @function Count
             * @type number
             */
            Count: {
                get: function () {
                    if (true) {
                        var localStorage = this.GetLocalStorage();
                        var length = localStorage.length;
                        var count = 0;
                        for (var i = 0; i < length; i = (i + 1) | 0) {
                            if (localStorage.key(i).startsWith(this.GetKeysFirstPart())) {
                                count = (count + 1) | 0;
                            }
                        }
                        return count;
                    } else {
                        return System.IO.IsolatedStorage.IsolatedStorageSettingsForCSharp.Instance.Count;
                    }
                }
            },
            /**
             * Gets a collection that contains the keys in the dictionary.
             *
             * @instance
             * @public
             * @readonly
             * @memberof System.IO.IsolatedStorage.IsolatedStorageSettings
             * @function Keys
             * @type System.Collections.ICollection
             */
            Keys: {
                get: function () {
                    if (true) {
                        var localStorage = this.GetLocalStorage();
                        var keysList = new (System.Collections.Generic.List$1(System.String)).ctor();
                        var length = localStorage.length;
                        var lengthOfPartToRemoveFromKey = (this.GetKeysFirstPart()).length;
                        for (var i = 0; i < length; i = (i + 1) | 0) {
                            if (localStorage.key(i).startsWith(this.GetKeysFirstPart())) {
                                keysList.add(localStorage.key(i).substring(lengthOfPartToRemoveFromKey));
                            }
                        }
                        return keysList;
                    } else {
                        return System.Linq.Enumerable.from(System.IO.IsolatedStorage.IsolatedStorageSettingsForCSharp.Instance.Keys).toList(System.String);
                    }
                }
            },
            /**
             * Gets a collection that contains the values in the dictionary.
             *
             * @instance
             * @public
             * @readonly
             * @memberof System.IO.IsolatedStorage.IsolatedStorageSettings
             * @function Values
             * @type System.Collections.ICollection
             */
            Values: {
                get: function () {
                    if (true) {
                        var localStorage = this.GetLocalStorage();
                        var valuesList = new (System.Collections.Generic.List$1(System.Object)).ctor();
                        var length = localStorage.length;
                        for (var i = 0; i < length; i = (i + 1) | 0) {
                            var str = localStorage.key(i);
                            if (System.String.startsWith(str, this.GetKeysFirstPart())) {
                                valuesList.add(localStorage[str]);
                            }
                        }
                        return valuesList;
                    } else {
                        return Bridge.cast((System.Linq.Enumerable.from(INTERNAL_BridgeWorkarounds.GetDictionaryValues_SimulatorCompatible(System.String, System.Object, System.IO.IsolatedStorage.IsolatedStorageSettingsForCSharp.Instance)).toList(System.Object)), System.Collections.ICollection);

                    }
                }
            }
        },
        alias: ["GetEnumerator", ["System$Collections$Generic$IEnumerable$1$System$Collections$Generic$KeyValuePair$2$System$String$System$Object$GetEnumerator", "System$Collections$Generic$IEnumerable$1$GetEnumerator"]],
        ctors: {
            ctor: function () {
                this.$initialize();
                this._fullApplicationName = Bridge.toString(Windows.UI.Xaml.Application.Current);
            }
        },
        methods: {
            /**
             * Gets or sets the value associated with the specified key.
             *
             * @instance
             * @public
             * @this System.IO.IsolatedStorage.IsolatedStorageSettings
             * @memberof System.IO.IsolatedStorage.IsolatedStorageSettings
             * @param   {string}           key    The key of the item to get or set.
             * @return  {System.Object}           The value associated with the specified key. If the specified key is not
             found, a get operation throws a System.Collections.Generic.KeyNotFoundException,
             and a set operation creates a new element that has the specified key.
             */
            getItem: function (key) {
                if (true) {
                    var localStorage = this.GetLocalStorage();
                    return localStorage[(this.GetKeysFirstPart() || "") + (key || "")];
                } else {
                    return System.IO.IsolatedStorage.IsolatedStorageSettingsForCSharp.Instance.getItem(key);
                }
            },
            /**
             * Gets or sets the value associated with the specified key.
             *
             * @instance
             * @public
             * @this System.IO.IsolatedStorage.IsolatedStorageSettings
             * @memberof System.IO.IsolatedStorage.IsolatedStorageSettings
             * @param   {string}           key      The key of the item to get or set.
             * @param   {System.Object}    value
             * @return  {void}                      The value associated with the specified key. If the specified key is not
             found, a get operation throws a System.Collections.Generic.KeyNotFoundException,
             and a set operation creates a new element that has the specified key.
             */
            setItem: function (key, value) {
                if (true) {
                    var localStorage = this.GetLocalStorage();
                    localStorage[(this.GetKeysFirstPart() || "") + (key || "")] = Bridge.unbox(value);
                } else {
                    System.IO.IsolatedStorage.IsolatedStorageSettingsForCSharp.Instance.setItem(key, value);
                }
            },
            GetLocalStorage: function () {
                if (true) {
                    var localStorage = 
(function(){
    return window.localStorage;
}());

                    if (localStorage == null || Bridge.referenceEquals(localStorage, undefined)) {
                        throw new System.Exception("The local storage - used to persist data - is not available on Internet Explorer or Edge when running the website from the local file system (ie. the URL starts with 'c:' or 'file:///'). To solve the problem, please run the website from a web server instead (ie. the URL must start with 'http://' or 'https://') or test the local storage using a different browser.");
                    } else {
                        return localStorage;
                    }
                } else {
                    return CSHTML5.Internal.INTERNAL_HtmlDomManager.ExecuteJavaScriptWithResult("window.localStorage");
                }
            },
            GetKeysFirstPart: function () {
                return "storage_" + (this._fullApplicationName || "") + "_settings_";
            },
            /**
             * Adds an entry to the dictionary for the key-value pair.
             *
             * @instance
             * @public
             * @this System.IO.IsolatedStorage.IsolatedStorageSettings
             * @memberof System.IO.IsolatedStorage.IsolatedStorageSettings
             * @param   {string}           key      The key for the entry to be stored.
             * @param   {System.Object}    value    The value to be stored.
             * @return  {void}
             */
            Add: function (key, value) {
                if (true) {
                    var localStorage = this.GetLocalStorage();
                    localStorage[(this.GetKeysFirstPart() || "") + (key || "")] = Bridge.unbox(value);
                } else {
                    System.IO.IsolatedStorage.IsolatedStorageSettingsForCSharp.Instance.add$1(key, value);
                    System.IO.IsolatedStorage.IsolatedStorageSettingsForCSharp.Instance.Save();
                }
            },
            /**
             * Resets the count of items stored in System.IO.IsolatedStorage.IsolatedStorageSettings
             to zero and releases all references to elements in the collection.
             *
             * @instance
             * @public
             * @this System.IO.IsolatedStorage.IsolatedStorageSettings
             * @memberof System.IO.IsolatedStorage.IsolatedStorageSettings
             * @return  {void}
             */
            Clear: function () {
                var $t;
                if (true) {
                    var localStorage = this.GetLocalStorage();
                    var keys = Bridge.cast(this.Keys, System.Collections.Generic.List$1(System.String));
                    $t = Bridge.getEnumerator(keys);
                    try {
                        while ($t.moveNext()) {
                            var key = $t.Current;
                            localStorage.removeItem((this.GetKeysFirstPart() || "") + (key || ""));
                        }
                    } finally {
                        if (Bridge.is($t, System.IDisposable)) {
                            $t.System$IDisposable$Dispose();
                        }
                    }
                } else {
                    System.IO.IsolatedStorage.IsolatedStorageSettingsForCSharp.Instance.clear();
                }
            },
            /**
             * Determines if the application settings dictionary contains the specified
             key.
             *
             * @instance
             * @public
             * @this System.IO.IsolatedStorage.IsolatedStorageSettings
             * @memberof System.IO.IsolatedStorage.IsolatedStorageSettings
             * @param   {string}     key    The key for the entry to be located.
             * @return  {boolean}           true if the dictionary contains the specified key; otherwise, false.
             */
            Contains: function (key) {
                if (true) {
                    var localStorage = this.GetLocalStorage();
                    return (localStorage.getItem((this.GetKeysFirstPart() || "") + (key || "")) != null);
                } else {
                    return System.IO.IsolatedStorage.IsolatedStorageSettingsForCSharp.Instance.containsKey(key);
                }
            },
            /**
             * Removes the entry with the specified key.
             *
             * @instance
             * @public
             * @this System.IO.IsolatedStorage.IsolatedStorageSettings
             * @memberof System.IO.IsolatedStorage.IsolatedStorageSettings
             * @param   {string}     key    The key for the entry to be deleted.
             * @return  {boolean}           true if the specified key was removed; otherwise, false.
             */
            Remove: function (key) {
                if (true) {
                    var localStorage = this.GetLocalStorage();
                    if ((localStorage.getItem((this.GetKeysFirstPart() || "") + (key || "")) == null)) {
                        return false;
                    }
                    localStorage.removeItem((this.GetKeysFirstPart() || "") + (key || ""));
                    return true;
                } else {
                    var ret = System.IO.IsolatedStorage.IsolatedStorageSettingsForCSharp.Instance.remove$1(key);
                    System.IO.IsolatedStorage.IsolatedStorageSettingsForCSharp.Instance.Save();
                    return ret;
                }
            },
            /**
             * // Saves data written to the current System.IO.IsolatedStorage.IsolatedStorageSettings
            // object.
            //
             * Gets a value for the specified key.
             *
             * @instance
             * @public
             * @this System.IO.IsolatedStorage.IsolatedStorageSettings
             * @memberof System.IO.IsolatedStorage.IsolatedStorageSettings
             * @param   {Function}    T        The System.Type of the value parameter.
             * @param   {string}      key      The key of the value to get.
             * @param   {T}           value    When this method returns, the value associated with the specified key if
             the key is found; otherwise, the default value for the type of the value
             parameter. This parameter is passed uninitialized.
             * @return  {boolean}              true if the specified key is found; otherwise, false.
             */
            TryGetValue: function (T, key, value) {
                if (true) {
                    var localStorage = this.GetLocalStorage();
                    var temp = (localStorage.getItem((this.GetKeysFirstPart() || "") + (key || "")));
                    if (temp == null) {
                        value.v = Bridge.getDefaultValue(T);
                        return false;
                    } else {
                        value.v = temp;
                        return true;
                    }
                } else {
                    var temp1 = { };
                    var ret = System.IO.IsolatedStorage.IsolatedStorageSettingsForCSharp.Instance.tryGetValue(key, temp1);
                    value.v = Bridge.cast(Bridge.unbox(temp1.v, T), T);
                    return ret;
                }
            },
            System$Collections$IEnumerable$GetEnumerator: function () {
                var $step = 0,
                    $jumpFromFinally,
                    $returnValue,
                    $t,
                    kv,
                    $async_e;

                var $enumerator = new Bridge.GeneratorEnumerator(Bridge.fn.bind(this, function () {
                    try {
                        for (;;) {
                            switch ($step) {
                                case 0: {
                                    $t = Bridge.getEnumerator(this.EnumerateKeyValues(), System.Collections.Generic.KeyValuePair$2(System.String,System.Object));
                                        $step = 1;
                                        continue;
                                }
                                case 1: {
                                    if ($t.moveNext()) {
                                            kv = $t.Current;
                                            $step = 2;
                                            continue;
                                        }
                                    $step = 4;
                                    continue;
                                }
                                case 2: {
                                    $enumerator.current = kv;
                                        $step = 3;
                                        return true;
                                }
                                case 3: {
                                    $step = 1;
                                    continue;
                                }
                                case 4: {

                                }
                                default: {
                                    return false;
                                }
                            }
                        }
                    } catch($async_e1) {
                        $async_e = System.Exception.create($async_e1);
                        throw $async_e;
                    }
                }));
                return $enumerator;
            },
            GetEnumerator: function () {
                var $step = 0,
                    $jumpFromFinally,
                    $returnValue,
                    $t,
                    kv,
                    $async_e;

                var $enumerator = new (Bridge.GeneratorEnumerator$1(System.Collections.Generic.KeyValuePair$2(System.String,System.Object)))(Bridge.fn.bind(this, function () {
                    try {
                        for (;;) {
                            switch ($step) {
                                case 0: {
                                    $t = Bridge.getEnumerator(this.EnumerateKeyValues(), System.Collections.Generic.KeyValuePair$2(System.String,System.Object));
                                        $step = 1;
                                        continue;
                                }
                                case 1: {
                                    if ($t.moveNext()) {
                                            kv = $t.Current;
                                            $step = 2;
                                            continue;
                                        }
                                    $step = 4;
                                    continue;
                                }
                                case 2: {
                                    $enumerator.current = kv;
                                        $step = 3;
                                        return true;
                                }
                                case 3: {
                                    $step = 1;
                                    continue;
                                }
                                case 4: {

                                }
                                default: {
                                    return false;
                                }
                            }
                        }
                    } catch($async_e1) {
                        $async_e = System.Exception.create($async_e1);
                        throw $async_e;
                    }
                }));
                return $enumerator;
            },
            EnumerateKeyValues: function () {
                return new (Bridge.GeneratorEnumerable$1(System.Collections.Generic.KeyValuePair$2(System.String,System.Object)))(Bridge.fn.bind(this, function ()  {
                    var $step = 0,
                        $jumpFromFinally,
                        $returnValue,
                        localStorage,
                        keys,
                        $t,
                        key,
                        item,
                        $t1,
                        kv,
                        $async_e;

                    var $enumerator = new (Bridge.GeneratorEnumerator$1(System.Collections.Generic.KeyValuePair$2(System.String,System.Object)))(Bridge.fn.bind(this, function () {
                        try {
                            for (;;) {
                                switch ($step) {
                                    case 0: {
                                        if (true) {
                                                $step = 1;
                                                continue;
                                            } else  {
                                                $step = 6;
                                                continue;
                                            }
                                    }
                                    case 1: {
                                        localStorage = this.GetLocalStorage();
                                            keys = Bridge.cast(this.Keys, System.Collections.Generic.List$1(System.String));
                                            $t = Bridge.getEnumerator(keys);
                                            $step = 2;
                                            continue;
                                    }
                                    case 2: {
                                        if ($t.moveNext()) {
                                                key = $t.Current;
                                                $step = 3;
                                                continue;
                                            }
                                        $step = 5;
                                        continue;
                                    }
                                    case 3: {
                                        item = localStorage.getItem((this.GetKeysFirstPart() || "") + (key || ""));
                                            $enumerator.current = new (System.Collections.Generic.KeyValuePair$2(System.String,System.Object)).$ctor1(key, item);
                                            $step = 4;
                                            return true;
                                    }
                                    case 4: {
                                        $step = 2;
                                        continue;
                                    }
                                    case 5: {
                                        $step = 11;
                                        continue;
                                    }
                                    case 6: {
                                        $t1 = Bridge.getEnumerator(System.IO.IsolatedStorage.IsolatedStorageSettingsForCSharp.Instance);
                                            $step = 7;
                                            continue;
                                    }
                                    case 7: {
                                        if ($t1.moveNext()) {
                                                kv = $t1.Current;
                                                $step = 8;
                                                continue;
                                            }
                                        $step = 10;
                                        continue;
                                    }
                                    case 8: {
                                        $enumerator.current = kv;
                                            $step = 9;
                                            return true;
                                    }
                                    case 9: {
                                        $step = 7;
                                        continue;
                                    }
                                    case 10: {
                                        $step = 11;
                                        continue;
                                    }
                                    case 11: {

                                    }
                                    default: {
                                        return false;
                                    }
                                }
                            }
                        } catch($async_e1) {
                            $async_e = System.Exception.create($async_e1);
                            throw $async_e;
                        }
                    }));
                    return $enumerator;
                }));
            }
        }
    });

    Bridge.define("System.IO.IsolatedStorage.IsolatedStorageSettingsForCSharp", {
        inherits: [System.Collections.Generic.IDictionary$2(System.String,System.Object)],
        statics: {
            fields: {
                Filename: null,
                AppDictionary: null,
                StaticIsolatedStorageSettings: null
            },
            props: {
                /**
                 * Its a static singleton instance.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof System.IO.IsolatedStorage.IsolatedStorageSettingsForCSharp
                 * @function Instance
                 * @type System.IO.IsolatedStorage.IsolatedStorageSettingsForCSharp
                 */
                Instance: {
                    get: function () {
                        return System.IO.IsolatedStorage.IsolatedStorageSettingsForCSharp.StaticIsolatedStorageSettings;
                    }
                }
            },
            ctors: {
                init: function () {
                    this.Filename = "Settings.bin";
                    this.AppDictionary = new (System.Collections.Generic.Dictionary$2(System.String,System.Object)).ctor();
                    this.StaticIsolatedStorageSettings = new System.IO.IsolatedStorage.IsolatedStorageSettingsForCSharp();
                }
            }
        },
        props: {
            Keys: {
                get: function () {
                    return System.IO.IsolatedStorage.IsolatedStorageSettingsForCSharp.AppDictionary.Keys;
                }
            },
            Values: {
                get: function () {
                    return System.Linq.Enumerable.from(INTERNAL_BridgeWorkarounds.GetDictionaryValues_SimulatorCompatible(System.String, System.Object, System.IO.IsolatedStorage.IsolatedStorageSettingsForCSharp.AppDictionary)).toList(System.Object);
                }
            },
            Count: {
                get: function () {
                    return System.IO.IsolatedStorage.IsolatedStorageSettingsForCSharp.AppDictionary.Count;
                }
            },
            IsReadOnly: {
                get: function () {
                    return false;
                }
            }
        },
        alias: [
            "add$1", "System$Collections$Generic$IDictionary$2$System$String$System$Object$add",
            "containsKey", "System$Collections$Generic$IDictionary$2$System$String$System$Object$containsKey",
            "Keys", "System$Collections$Generic$IDictionary$2$System$String$System$Object$Keys",
            "remove$1", "System$Collections$Generic$IDictionary$2$System$String$System$Object$remove",
            "tryGetValue", "System$Collections$Generic$IDictionary$2$System$String$System$Object$tryGetValue",
            "Values", "System$Collections$Generic$IDictionary$2$System$String$System$Object$Values",
            "getItem", "System$Collections$Generic$IDictionary$2$System$String$System$Object$getItem",
            "setItem", "System$Collections$Generic$IDictionary$2$System$String$System$Object$setItem",
            "add", "System$Collections$Generic$ICollection$1$System$Collections$Generic$KeyValuePair$2$System$String$System$Object$add",
            "clear", "System$Collections$Generic$ICollection$1$System$Collections$Generic$KeyValuePair$2$System$String$System$Object$clear",
            "contains", "System$Collections$Generic$ICollection$1$System$Collections$Generic$KeyValuePair$2$System$String$System$Object$contains",
            "copyTo", "System$Collections$Generic$ICollection$1$System$Collections$Generic$KeyValuePair$2$System$String$System$Object$copyTo",
            "Count", "System$Collections$Generic$ICollection$1$System$Collections$Generic$KeyValuePair$2$System$String$System$Object$Count",
            "IsReadOnly", "System$Collections$Generic$ICollection$1$System$Collections$Generic$KeyValuePair$2$System$String$System$Object$IsReadOnly",
            "remove", "System$Collections$Generic$ICollection$1$System$Collections$Generic$KeyValuePair$2$System$String$System$Object$remove",
            "GetEnumerator", ["System$Collections$Generic$IEnumerable$1$System$Collections$Generic$KeyValuePair$2$System$String$System$Object$GetEnumerator", "System$Collections$Generic$IEnumerable$1$GetEnumerator"]
        ],
        ctors: {
            /**
             * Its a private constructor.
             *
             * @instance
             * @private
             * @this System.IO.IsolatedStorage.IsolatedStorageSettingsForCSharp
             * @memberof System.IO.IsolatedStorage.IsolatedStorageSettingsForCSharp
             * @return  {void}
             */
            ctor: function () {
                this.$initialize();
            }
        },
        methods: {
            /**
             * It Checks if Dictionary object has item corresponding to passed key,
                 if True then it returns that object else it returns default value.
             *
             * @instance
             * @public
             * @this System.IO.IsolatedStorage.IsolatedStorageSettingsForCSharp
             * @memberof System.IO.IsolatedStorage.IsolatedStorageSettingsForCSharp
             * @param   {string}           key             
             * @param   {System.Object}    defaultvalue
             * @return  {System.Object}
             */
            getItem$1: function (key, defaultvalue) {
                return System.IO.IsolatedStorage.IsolatedStorageSettingsForCSharp.AppDictionary.containsKey(key) ? System.IO.IsolatedStorage.IsolatedStorageSettingsForCSharp.AppDictionary.getItem(key) : defaultvalue;
            },
            /**
             * It Checks if Dictionary object has item corresponding to passed key,
                 if True then it returns that object else it returns default value.
             *
             * @instance
             * @public
             * @this System.IO.IsolatedStorage.IsolatedStorageSettingsForCSharp
             * @memberof System.IO.IsolatedStorage.IsolatedStorageSettingsForCSharp
             * @param   {string}           key             
             * @param   {System.Object}    defaultvalue    
             * @param   {System.Object}    value
             * @return  {void}
             */
            setItem$1: function (key, defaultvalue, value) {
                System.IO.IsolatedStorage.IsolatedStorageSettingsForCSharp.AppDictionary.setItem(key, value);
            },
            getItem: function (key) {
                return System.IO.IsolatedStorage.IsolatedStorageSettingsForCSharp.AppDictionary.getItem(key);
            },
            setItem: function (key, value) {
                System.IO.IsolatedStorage.IsolatedStorageSettingsForCSharp.AppDictionary.setItem(key, value);
            },
            /**
             * It serializes dictionary in binary format and stores it in a binary file.
             *
             * @instance
             * @public
             * @this System.IO.IsolatedStorage.IsolatedStorageSettingsForCSharp
             * @memberof System.IO.IsolatedStorage.IsolatedStorageSettingsForCSharp
             * @return  {void}
             */
            Save: function () { },
            add$1: function (key, value) {
                System.IO.IsolatedStorage.IsolatedStorageSettingsForCSharp.AppDictionary.add(key, value);
            },
            add: function (item) {
                System.IO.IsolatedStorage.IsolatedStorageSettingsForCSharp.AppDictionary.add(item.key, item.value);
            },
            containsKey: function (key) {
                return System.IO.IsolatedStorage.IsolatedStorageSettingsForCSharp.AppDictionary.containsKey(key);
            },
            remove$1: function (key) {
                try {
                    System.IO.IsolatedStorage.IsolatedStorageSettingsForCSharp.AppDictionary.remove(key);
                    return true;
                } catch ($e1) {
                    $e1 = System.Exception.create($e1);
                    return false;
                }
            },
            remove: function (item) {
                return System.IO.IsolatedStorage.IsolatedStorageSettingsForCSharp.AppDictionary.remove(item.key);
            },
            tryGetValue: function (key, value) {
                return System.IO.IsolatedStorage.IsolatedStorageSettingsForCSharp.AppDictionary.tryGetValue(key, value);
            },
            clear: function () {
                System.IO.IsolatedStorage.IsolatedStorageSettingsForCSharp.AppDictionary.clear();
            },
            contains: function (item) {
                return System.IO.IsolatedStorage.IsolatedStorageSettingsForCSharp.AppDictionary.containsKey(item.key);
            },
            copyTo: function (array, arrayIndex) {
                throw new System.NotImplementedException.ctor();
            },
            GetEnumerator: function () {
                return System.IO.IsolatedStorage.IsolatedStorageSettingsForCSharp.AppDictionary.GetEnumerator().$clone();
            },
            System$Collections$IEnumerable$GetEnumerator: function () {
                return System.IO.IsolatedStorage.IsolatedStorageSettingsForCSharp.AppDictionary.GetEnumerator().$clone();
            }
        }
    });

    /**
     * The {@link } namespace contains types that allow the
     creation and use of isolated stores. With these stores, you can read and write data
     that less trusted code cannot access and prevent the exposure of sensitive information
     that can be saved elsewhere on the file system. Data is stored in compartments that
     are isolated.
     *
     * @class System.IO.IsolatedStorage.NamespaceDoc
     */
    Bridge.define("System.IO.IsolatedStorage.NamespaceDoc");

    /** @namespace System.IO */

    /**
     * The {@link } namespaces contain types that support input and
     output, including the ability to read and write data to streams either synchronously
     or asynchronously, to compress data in streams, to create and use isolated stores, to
     map files to an application's logical address space, to store multiple data objects
     in a single container, to communicate using anonymous or named pipes, to implement
     custom logging, and to handle the flow of data to and from serial ports.
     *
     * @class System.IO.NamespaceDoc
     */
    Bridge.define("System.IO.NamespaceDoc");

    Bridge.define("System.IO.Path", {
        statics: {
            fields: {
                DirectorySeparatorCharAsString: null,
                DirectorySeparatorChar: 0,
                AltDirectorySeparatorChar: 0,
                VolumeSeparatorChar: 0
            },
            ctors: {
                init: function () {
                    this.DirectorySeparatorCharAsString = "\\";
                    this.DirectorySeparatorChar = 92;
                    this.AltDirectorySeparatorChar = 47;
                    this.VolumeSeparatorChar = 58;
                }
            },
            methods: {
                GetDirectoryName: function (path) {

                    var paths = System.String.split(path, [92].map(function (i) {{ return String.fromCharCode(i); }}));

                    path = "";

                    if (paths.length > 0) {
                        path = paths[System.Array.index(0, paths)];
                    }

                    for (var i = 1; i < ((paths.length - 1) | 0); i = (i + 1) | 0) {
                        path = (path || "") + ((String.fromCharCode(92) + (paths[System.Array.index(i, paths)] || "")) || "");
                    }

                    return path;
                },
                GetFileName: function (path) {
                    var paths = System.String.split(path, [92].map(function (i) {{ return String.fromCharCode(i); }}));

                    path = "";

                    if (paths.length > 0) {
                        path = paths[System.Array.index(((paths.length - 1) | 0), paths)];
                    }

                    return path;
                }
            }
        }
    });

    Bridge.define("System.Linq.GroupedEnumerable$3", function (TSource, TKey, TElement) { return {
        inherits: [System.Collections.Generic.IEnumerable$1(System.Linq.IGrouping$2)],
        fields: {
            groups: null
        },
        alias: ["GetEnumerator", ["System$Collections$Generic$IEnumerable$1$System$Linq$IGrouping$2$GetEnumerator", "System$Collections$Generic$IEnumerable$1$GetEnumerator"]],
        ctors: {
            init: function () {
                this.groups = new (System.Collections.Generic.Dictionary$2(TKey,System.Linq.Grouping$2)).ctor();
            },
            ctor: function (source, keySelector, elementSelector, comparer) {
                this.$initialize();
            }
        },
        methods: {
            FillGroups: function (source, keySelector, elementSelector, comparer) { },
            GetEnumerator: function () {
                var $step = 0,
                    $jumpFromFinally,
                    $returnValue,
                    $t,
                    key,
                    $async_e;

                var $enumerator = new (Bridge.GeneratorEnumerator$1(System.Linq.IGrouping$2))(Bridge.fn.bind(this, function () {
                    try {
                        for (;;) {
                            switch ($step) {
                                case 0: {
                                    $t = Bridge.getEnumerator(this.groups.Keys);
                                        $step = 1;
                                        continue;
                                }
                                case 1: {
                                    if ($t.moveNext()) {
                                            key = $t.Current;
                                            $step = 2;
                                            continue;
                                        }
                                    $step = 4;
                                    continue;
                                }
                                case 2: {
                                    $enumerator.current = this.groups.getItem(key);
                                        $step = 3;
                                        return true;
                                }
                                case 3: {
                                    $step = 1;
                                    continue;
                                }
                                case 4: {

                                }
                                default: {
                                    return false;
                                }
                            }
                        }
                    } catch($async_e1) {
                        $async_e = System.Exception.create($async_e1);
                        throw $async_e;
                    }
                }));
                return $enumerator;
            },
            System$Collections$IEnumerable$GetEnumerator: function () {
                return this.GetEnumerator();
            }
        }
    }; });

    Bridge.define("System.Linq.GroupedEnumerable$4", function (TSource, TKey, TElement, TResult) { return {
        inherits: [System.Collections.Generic.IEnumerable$1(TResult)],
        fields: {
            result: null,
            groups: null
        },
        alias: ["System$Collections$Generic$IEnumerable$1$GetEnumerator", "System$Collections$Generic$IEnumerable$1$" + Bridge.getTypeAlias(TResult) + "$GetEnumerator"],
        ctors: {
            init: function () {
                this.result = new (System.Collections.ObjectModel.Collection$1(TResult)).ctor();
                this.groups = new (System.Collections.Generic.Dictionary$2(TKey,System.Linq.Grouping$2)).ctor();
            },
            ctor: function (source, keySelector, elementSelector, resultSelector, comparer) {
                var $t;
                this.$initialize();
                if (source == null) {
                    throw new System.ArgumentNullException.$ctor1("source");
                }
                if (Bridge.staticEquals(keySelector, null)) {
                    throw new System.ArgumentNullException.$ctor1("keySelector");
                }
                if (Bridge.staticEquals(elementSelector, null)) {
                    throw new System.ArgumentNullException.$ctor1("elementSelector");
                }
                if (Bridge.staticEquals(resultSelector, null)) {
                    throw new System.ArgumentNullException.$ctor1("resultSelector");
                }

                this.FillGroups(source, keySelector, elementSelector, comparer);

                $t = Bridge.getEnumerator(this.groups.Keys);
                try {
                    while ($t.moveNext()) {
                        var groupKey = $t.Current;
                        this.result.add(resultSelector(groupKey, this.groups.getItem(groupKey)));
                    }
                } finally {
                    if (Bridge.is($t, System.IDisposable)) {
                        $t.System$IDisposable$Dispose();
                    }
                }
            }
        },
        methods: {
            FillGroups: function (source, keySelector, elementSelector, comparer) { },
            System$Collections$IEnumerable$GetEnumerator: function () {
                return this.result.GetEnumerator();
            },
            System$Collections$Generic$IEnumerable$1$GetEnumerator: function () {
                return this.result.GetEnumerator();
            }
        }
    }; });

    /**
     * The {@link } namespace contains fundamental classes and base classes
     that define commonly-used value and reference data types, events and event handlers,
     interfaces, attributes, and processing exceptions.
     *
     * @class System.NamespaceDoc
     */
    Bridge.define("System.NamespaceDoc");

    Bridge.define("System.Net.CredentialsMode", {
        $kind: "enum",
        statics: {
            fields: {
                /**
                 * Cookies are not passed
                 *
                 * @static
                 * @public
                 * @memberof System.Net.CredentialsMode
                 * @constant
                 * @default 0
                 * @type System.Net.CredentialsMode
                 */
                Disabled: 0,
                /**
                 * Cookies are passed
                 *
                 * @static
                 * @public
                 * @memberof System.Net.CredentialsMode
                 * @constant
                 * @default 1
                 * @type System.Net.CredentialsMode
                 */
                Enabled: 1,
                /**
                 * The first request will attempt to use cookies. If it fails, the first request and all the subsequent ones are done without cookies.
                 *
                 * @static
                 * @public
                 * @memberof System.Net.CredentialsMode
                 * @constant
                 * @default 2
                 * @type System.Net.CredentialsMode
                 */
                Auto: 2
            }
        }
    });

    Bridge.define("System.Net.CSharpHeaderToHtmlHeaderConverter", {
        statics: {
            fields: {
                _headerStringEquivalence: null
            },
            ctors: {
                init: function () {
                    this._headerStringEquivalence = function (_o1) {
                            _o1.add("CacheControl", "Cache-Control");
                            _o1.add("Connection", "Connection");
                            _o1.add("KeepAlive", "Keep-Alive");
                            _o1.add("TransferEncoding", "Transfer-Encoding");
                            _o1.add("ContentLength", "Content-Length");
                            _o1.add("ContentType", "Content-Type");
                            _o1.add("ContentEncoding", "Content-Encoding");
                            _o1.add("ContentLanguage", "Content-Language");
                            _o1.add("ContentLocation", "Content-Location");
                            _o1.add("ContentMd5", "Content-MD5");
                            _o1.add("ContentRange", "Content-Range");
                            _o1.add("LastModified", "Last-Modified");
                            _o1.add("AcceptCharset", "Accept-Charset");
                            _o1.add("AcceptEncoding", "Accept-Encoding");
                            _o1.add("AcceptLanguage", "Accept-Language");
                            _o1.add("IfMatch", "If-Match");
                            _o1.add("IfModifiedSince", "If-Modified-Since");
                            _o1.add("IfNoneMatch", "If-None-Match");
                            _o1.add("IfRange", "If-Range");
                            _o1.add("IfUnmodifiedSince", "If-Unmodified-Since");
                            _o1.add("MaxForwards", "Max-Forwards");
                            _o1.add("ProxyAuthorization", "Proxy-Authorization");
                            _o1.add("UserAgent", "User-Agent");
                            _o1.add("AcceptRanges", "Accept-Ranges");
                            _o1.add("ProxyAuthenticate", "Proxy-Authenticate");
                            _o1.add("RetryAfter", "Retry-After");
                            _o1.add("SetCookie", "Set-Cookie");
                            return _o1;
                        }(new (System.Collections.Generic.Dictionary$2(System.String,System.String)).ctor());
                }
            },
            methods: {
                Convert: function (header) {
                    var headerAsString = System.Enum.toString(System.Net.HttpRequestHeader, header);
                    return System.Net.CSharpHeaderToHtmlHeaderConverter.Convert$2(headerAsString);
                },
                Convert$1: function (header) {
                    var headerAsString = System.Enum.toString(System.Net.HttpResponseHeader, header);
                    return System.Net.CSharpHeaderToHtmlHeaderConverter.Convert$2(headerAsString);
                },
                Convert$2: function (header) {
                    var returnString = header;
                    if (System.Net.CSharpHeaderToHtmlHeaderConverter._headerStringEquivalence.containsKey(header)) {
                        returnString = System.Net.CSharpHeaderToHtmlHeaderConverter._headerStringEquivalence.getItem(header);
                    }
                    return returnString;
                }
            }
        }
    });

    /**
     * Provides data for the System.Net.WebClient.DownloadStringCompleted event.
     *
     * @public
     * @class System.Net.DownloadStringCompletedEventArgs
     */
    Bridge.define("System.Net.DownloadStringCompletedEventArgs", {
        fields: {
            /**
             * Gets the data that is downloaded by a Overload:System.Net.WebClient.DownloadStringAsync
             method.
             *
             * @instance
             * @public
             * @memberof System.Net.DownloadStringCompletedEventArgs
             * @function Result
             * @type string
             */
            Result: null,
            /**
             * Gets the unique identifier for the asynchronous task.
             *
             * @instance
             * @public
             * @memberof System.Net.DownloadStringCompletedEventArgs
             * @function UserState
             * @type System.Object
             */
            UserState: null,
            /**
             * Gets a value indicating whether an asynchronous operation has been canceled.
             *
             * @instance
             * @public
             * @memberof System.Net.DownloadStringCompletedEventArgs
             * @function Cancelled
             * @type boolean
             */
            Cancelled: false,
            /**
             * Gets a value indicating which error occurred during an asynchronous operation.
             *
             * @instance
             * @public
             * @memberof System.Net.DownloadStringCompletedEventArgs
             * @function Error
             * @type System.Exception
             */
            Error: null
        },
        ctors: {
            ctor: function () {
                this.$initialize();
            },
            $ctor1: function (e) {
                this.$initialize();
                this.Result = e.Result;
                this.UserState = e.UserState;
                this.Error = e.Error;
                this.Cancelled = e.Cancelled;
            },
            $ctor2: function (e) {
                this.$initialize();
                this.Result = e.Result;
                this.UserState = e.UserState;
                this.Error = e.Error;
                this.Cancelled = e.Cancelled;
            }
        }
    });

    Bridge.define("System.ServiceModel.INTERNAL_AsyncResult", {
        inherits: [System.IAsyncResult,System.IDisposable],
        fields: {
            _callback: null,
            _state: null,
            _isCompleted: false
        },
        props: {
            IsCompleted: {
                get: function () {
                    return this._isCompleted;
                }
            },
            AsyncWaitHandle: {
                get: function () {
                    return null;
                }
            },
            AsyncState: {
                get: function () {
                    return this._state;
                }
            },
            CompletedSynchronously: {
                get: function () {
                    return false;
                }
            }
        },
        alias: [
            "IsCompleted", "System$IAsyncResult$IsCompleted",
            "AsyncWaitHandle", "System$IAsyncResult$AsyncWaitHandle",
            "AsyncState", "System$IAsyncResult$AsyncState",
            "CompletedSynchronously", "System$IAsyncResult$CompletedSynchronously",
            "Dispose", "System$IDisposable$Dispose"
        ],
        ctors: {
            ctor: function (callback, state) {
                this.$initialize();
                this._callback = callback;
                this._state = state;
            }
        },
        methods: {
            Completed: function () {
                this._isCompleted = true;
                if (!Bridge.staticEquals(this._callback, null)) {
                    this._callback(this);
                }
            },
            Dispose: function () {
                this._state = null;
                this._callback = null;
            }
        }
    });

    /**
     * The HTTP headers that may be specified in a client request.
     *
     * @public
     * @class System.Net.HttpRequestHeader
     */
    Bridge.define("System.Net.HttpRequestHeader", {
        $kind: "enum",
        statics: {
            fields: {
                /**
                 * The Cache-Control header, which specifies directives that must be obeyed
                 by all cache control mechanisms along the request/response chain.
                 *
                 * @static
                 * @public
                 * @memberof System.Net.HttpRequestHeader
                 * @constant
                 * @default 0
                 * @type System.Net.HttpRequestHeader
                 */
                CacheControl: 0,
                /**
                 * The Connection header, which specifies options that are desired for a particular
                 connection.
                 *
                 * @static
                 * @public
                 * @memberof System.Net.HttpRequestHeader
                 * @constant
                 * @default 1
                 * @type System.Net.HttpRequestHeader
                 */
                Connection: 1,
                /**
                 * The Date header, which specifies the date and time at which the request originated.
                 *
                 * @static
                 * @public
                 * @memberof System.Net.HttpRequestHeader
                 * @constant
                 * @default 2
                 * @type System.Net.HttpRequestHeader
                 */
                Date: 2,
                /**
                 * The Keep-Alive header, which specifies a parameter used into order to maintain
                 a persistent connection.
                 *
                 * @static
                 * @public
                 * @memberof System.Net.HttpRequestHeader
                 * @constant
                 * @default 3
                 * @type System.Net.HttpRequestHeader
                 */
                KeepAlive: 3,
                /**
                 * The Pragma header, which specifies implementation-specific directives that
                 might apply to any agent along the request/response chain.
                 *
                 * @static
                 * @public
                 * @memberof System.Net.HttpRequestHeader
                 * @constant
                 * @default 4
                 * @type System.Net.HttpRequestHeader
                 */
                Pragma: 4,
                /**
                 * The Trailer header, which specifies the header fields present in the trailer
                 of a message encoded with chunked transfer-coding.
                 *
                 * @static
                 * @public
                 * @memberof System.Net.HttpRequestHeader
                 * @constant
                 * @default 5
                 * @type System.Net.HttpRequestHeader
                 */
                Trailer: 5,
                /**
                 * The Transfer-Encoding header, which specifies what (if any) type of transformation
                 that has been applied to the message body.
                 *
                 * @static
                 * @public
                 * @memberof System.Net.HttpRequestHeader
                 * @constant
                 * @default 6
                 * @type System.Net.HttpRequestHeader
                 */
                TransferEncoding: 6,
                /**
                 * The Upgrade header, which specifies additional communications protocols that
                 the client supports.
                 *
                 * @static
                 * @public
                 * @memberof System.Net.HttpRequestHeader
                 * @constant
                 * @default 7
                 * @type System.Net.HttpRequestHeader
                 */
                Upgrade: 7,
                /**
                 * The Via header, which specifies intermediate protocols to be used by gateway
                 and proxy agents.
                 *
                 * @static
                 * @public
                 * @memberof System.Net.HttpRequestHeader
                 * @constant
                 * @default 8
                 * @type System.Net.HttpRequestHeader
                 */
                Via: 8,
                /**
                 * The Warning header, which specifies additional information about that status
                 or transformation of a message that might not be reflected in the message.
                 *
                 * @static
                 * @public
                 * @memberof System.Net.HttpRequestHeader
                 * @constant
                 * @default 9
                 * @type System.Net.HttpRequestHeader
                 */
                Warning: 9,
                /**
                 * The Allow header, which specifies the set of HTTP methods supported.
                 *
                 * @static
                 * @public
                 * @memberof System.Net.HttpRequestHeader
                 * @constant
                 * @default 10
                 * @type System.Net.HttpRequestHeader
                 */
                Allow: 10,
                /**
                 * The Content-Length header, which specifies the length, in bytes, of the accompanying
                 body data.
                 *
                 * @static
                 * @public
                 * @memberof System.Net.HttpRequestHeader
                 * @constant
                 * @default 11
                 * @type System.Net.HttpRequestHeader
                 */
                ContentLength: 11,
                /**
                 * The Content-Type header, which specifies the MIME type of the accompanying
                 body data.
                 *
                 * @static
                 * @public
                 * @memberof System.Net.HttpRequestHeader
                 * @constant
                 * @default 12
                 * @type System.Net.HttpRequestHeader
                 */
                ContentType: 12,
                /**
                 * The Content-Encoding header, which specifies the encodings that have been
                 applied to the accompanying body data.
                 *
                 * @static
                 * @public
                 * @memberof System.Net.HttpRequestHeader
                 * @constant
                 * @default 13
                 * @type System.Net.HttpRequestHeader
                 */
                ContentEncoding: 13,
                /**
                 * The Content-Langauge header, which specifies the natural language(s) of the
                 accompanying body data.
                 *
                 * @static
                 * @public
                 * @memberof System.Net.HttpRequestHeader
                 * @constant
                 * @default 14
                 * @type System.Net.HttpRequestHeader
                 */
                ContentLanguage: 14,
                /**
                 * The Content-Location header, which specifies a URI from which the accompanying
                 body may be obtained.
                 *
                 * @static
                 * @public
                 * @memberof System.Net.HttpRequestHeader
                 * @constant
                 * @default 15
                 * @type System.Net.HttpRequestHeader
                 */
                ContentLocation: 15,
                /**
                 * The Content-MD5 header, which specifies the MD5 digest of the accompanying
                 body data, for the purpose of providing an end-to-end message integrity check.
                 *
                 * @static
                 * @public
                 * @memberof System.Net.HttpRequestHeader
                 * @constant
                 * @default 16
                 * @type System.Net.HttpRequestHeader
                 */
                ContentMd5: 16,
                /**
                 * The Content-Range header, which specifies where in the full body the accompanying
                 partial body data should be applied.
                 *
                 * @static
                 * @public
                 * @memberof System.Net.HttpRequestHeader
                 * @constant
                 * @default 17
                 * @type System.Net.HttpRequestHeader
                 */
                ContentRange: 17,
                /**
                 * The Expires header, which specifies the date and time after which the accompanying
                 body data should be considered stale.
                 *
                 * @static
                 * @public
                 * @memberof System.Net.HttpRequestHeader
                 * @constant
                 * @default 18
                 * @type System.Net.HttpRequestHeader
                 */
                Expires: 18,
                /**
                 * The Last-Modified header, which specifies the date and time at which the
                 accompanying body data was last modified.
                 *
                 * @static
                 * @public
                 * @memberof System.Net.HttpRequestHeader
                 * @constant
                 * @default 19
                 * @type System.Net.HttpRequestHeader
                 */
                LastModified: 19,
                /**
                 * The Accept header, which specifies the MIME types that are acceptable for
                 the response.
                 *
                 * @static
                 * @public
                 * @memberof System.Net.HttpRequestHeader
                 * @constant
                 * @default 20
                 * @type System.Net.HttpRequestHeader
                 */
                Accept: 20,
                /**
                 * The Accept-Charset header, which specifies the character sets that are acceptable
                 for the response.
                 *
                 * @static
                 * @public
                 * @memberof System.Net.HttpRequestHeader
                 * @constant
                 * @default 21
                 * @type System.Net.HttpRequestHeader
                 */
                AcceptCharset: 21,
                /**
                 * The Accept-Encoding header, which specifies the content encodings that are
                 acceptable for the response.
                 *
                 * @static
                 * @public
                 * @memberof System.Net.HttpRequestHeader
                 * @constant
                 * @default 22
                 * @type System.Net.HttpRequestHeader
                 */
                AcceptEncoding: 22,
                /**
                 * The Accept-Langauge header, which specifies that natural languages that are
                 preferred for the response.
                 *
                 * @static
                 * @public
                 * @memberof System.Net.HttpRequestHeader
                 * @constant
                 * @default 23
                 * @type System.Net.HttpRequestHeader
                 */
                AcceptLanguage: 23,
                /**
                 * The Authorization header, which specifies the credentials that the client
                 presents in order to authenticate itself to the server.
                 *
                 * @static
                 * @public
                 * @memberof System.Net.HttpRequestHeader
                 * @constant
                 * @default 24
                 * @type System.Net.HttpRequestHeader
                 */
                Authorization: 24,
                /**
                 * The Cookie header, which specifies cookie data presented to the server.
                 *
                 * @static
                 * @public
                 * @memberof System.Net.HttpRequestHeader
                 * @constant
                 * @default 25
                 * @type System.Net.HttpRequestHeader
                 */
                Cookie: 25,
                /**
                 * The Expect header, which specifies particular server behaviors that are required
                 by the client.
                 *
                 * @static
                 * @public
                 * @memberof System.Net.HttpRequestHeader
                 * @constant
                 * @default 26
                 * @type System.Net.HttpRequestHeader
                 */
                Expect: 26,
                /**
                 * The From header, which specifies an Internet E-mail address for the human
                 user who controls the requesting user agent.
                 *
                 * @static
                 * @public
                 * @memberof System.Net.HttpRequestHeader
                 * @constant
                 * @default 27
                 * @type System.Net.HttpRequestHeader
                 */
                From: 27,
                /**
                 * The Host header, which specifies the host name and port number of the resource
                 being requested.
                 *
                 * @static
                 * @public
                 * @memberof System.Net.HttpRequestHeader
                 * @constant
                 * @default 28
                 * @type System.Net.HttpRequestHeader
                 */
                Host: 28,
                /**
                 * The If-Match header, which specifies that the requested operation should
                 be performed only if the client's cached copy of the indicated resource is
                 current.
                 *
                 * @static
                 * @public
                 * @memberof System.Net.HttpRequestHeader
                 * @constant
                 * @default 29
                 * @type System.Net.HttpRequestHeader
                 */
                IfMatch: 29,
                /**
                 * The If-Modified-Since header, which specifies that the requested operation
                 should be performed only if the requested resource has been modified since
                 the indicated data and time.
                 *
                 * @static
                 * @public
                 * @memberof System.Net.HttpRequestHeader
                 * @constant
                 * @default 30
                 * @type System.Net.HttpRequestHeader
                 */
                IfModifiedSince: 30,
                /**
                 * The If-None-Match header, which specifies that the requested operation should
                 be performed only if none of client's cached copies of the indicated resources
                 are current.
                 *
                 * @static
                 * @public
                 * @memberof System.Net.HttpRequestHeader
                 * @constant
                 * @default 31
                 * @type System.Net.HttpRequestHeader
                 */
                IfNoneMatch: 31,
                /**
                 * The If-Range header, which specifies that only the specified range of the
                 requested resource should be sent, if the client's cached copy is current.
                 *
                 * @static
                 * @public
                 * @memberof System.Net.HttpRequestHeader
                 * @constant
                 * @default 32
                 * @type System.Net.HttpRequestHeader
                 */
                IfRange: 32,
                /**
                 * The If-Unmodified-Since header, which specifies that the requested operation
                 should be performed only if the requested resource has not been modified
                 since the indicated date and time.
                 *
                 * @static
                 * @public
                 * @memberof System.Net.HttpRequestHeader
                 * @constant
                 * @default 33
                 * @type System.Net.HttpRequestHeader
                 */
                IfUnmodifiedSince: 33,
                /**
                 * The Max-Forwards header, which specifies an integer indicating the remaining
                 number of times that this request may be forwarded.
                 *
                 * @static
                 * @public
                 * @memberof System.Net.HttpRequestHeader
                 * @constant
                 * @default 34
                 * @type System.Net.HttpRequestHeader
                 */
                MaxForwards: 34,
                /**
                 * The Proxy-Authorization header, which specifies the credentials that the
                 client presents in order to authenticate itself to a proxy.
                 *
                 * @static
                 * @public
                 * @memberof System.Net.HttpRequestHeader
                 * @constant
                 * @default 35
                 * @type System.Net.HttpRequestHeader
                 */
                ProxyAuthorization: 35,
                /**
                 * The Referer header, which specifies the URI of the resource from which the
                 request URI was obtained.
                 *
                 * @static
                 * @public
                 * @memberof System.Net.HttpRequestHeader
                 * @constant
                 * @default 36
                 * @type System.Net.HttpRequestHeader
                 */
                Referer: 36,
                /**
                 * The Range header, which specifies the the sub-range(s) of the response that
                 the client requests be returned in lieu of the entire response.
                 *
                 * @static
                 * @public
                 * @memberof System.Net.HttpRequestHeader
                 * @constant
                 * @default 37
                 * @type System.Net.HttpRequestHeader
                 */
                Range: 37,
                /**
                 * The TE header, which specifies the transfer encodings that are acceptable
                 for the response.
                 *
                 * @static
                 * @public
                 * @memberof System.Net.HttpRequestHeader
                 * @constant
                 * @default 38
                 * @type System.Net.HttpRequestHeader
                 */
                Te: 38,
                /**
                 * The Translate header, a Microsoft extension to the HTTP specification used
                 in conjunction with WebDAV functionality.
                 *
                 * @static
                 * @public
                 * @memberof System.Net.HttpRequestHeader
                 * @constant
                 * @default 39
                 * @type System.Net.HttpRequestHeader
                 */
                Translate: 39,
                /**
                 * The User-Agent header, which specifies information about the client agent.
                 *
                 * @static
                 * @public
                 * @memberof System.Net.HttpRequestHeader
                 * @constant
                 * @default 40
                 * @type System.Net.HttpRequestHeader
                 */
                UserAgent: 40
            }
        }
    });

    Bridge.define("System.Net.HttpResponseHeader", {
        $kind: "enum",
        statics: {
            fields: {
                /**
                 * The Cache-Control header, which specifies caching directives that must be
                 obeyed by all caching mechanisms along the request/response chain.
                 *
                 * @static
                 * @public
                 * @memberof System.Net.HttpResponseHeader
                 * @constant
                 * @default 0
                 * @type System.Net.HttpResponseHeader
                 */
                CacheControl: 0,
                /**
                 * The Connection header, which specifies options that are desired for a particular
                 connection.
                 *
                 * @static
                 * @public
                 * @memberof System.Net.HttpResponseHeader
                 * @constant
                 * @default 1
                 * @type System.Net.HttpResponseHeader
                 */
                Connection: 1,
                /**
                 * The Date header, which specifies the date and time at which the response
                 originated.
                 *
                 * @static
                 * @public
                 * @memberof System.Net.HttpResponseHeader
                 * @constant
                 * @default 2
                 * @type System.Net.HttpResponseHeader
                 */
                Date: 2,
                /**
                 * The Keep-Alive header, which specifies a parameter to be used to maintain
                 a persistent connection.
                 *
                 * @static
                 * @public
                 * @memberof System.Net.HttpResponseHeader
                 * @constant
                 * @default 3
                 * @type System.Net.HttpResponseHeader
                 */
                KeepAlive: 3,
                /**
                 * The Pragma header, which specifies implementation-specific directives that
                 might apply to any agent along the request/response chain.
                 *
                 * @static
                 * @public
                 * @memberof System.Net.HttpResponseHeader
                 * @constant
                 * @default 4
                 * @type System.Net.HttpResponseHeader
                 */
                Pragma: 4,
                /**
                 * The Trailer header, which specifies that the indicated header fields are
                 present in the trailer of a message that is encoded with chunked transfer-coding.
                 *
                 * @static
                 * @public
                 * @memberof System.Net.HttpResponseHeader
                 * @constant
                 * @default 5
                 * @type System.Net.HttpResponseHeader
                 */
                Trailer: 5,
                /**
                 * The Transfer-Encoding header, which specifies what (if any) type of transformation
                 has been applied to the message body.
                 *
                 * @static
                 * @public
                 * @memberof System.Net.HttpResponseHeader
                 * @constant
                 * @default 6
                 * @type System.Net.HttpResponseHeader
                 */
                TransferEncoding: 6,
                /**
                 * The Upgrade header, which specifies additional communications protocols that
                 the client supports.
                 *
                 * @static
                 * @public
                 * @memberof System.Net.HttpResponseHeader
                 * @constant
                 * @default 7
                 * @type System.Net.HttpResponseHeader
                 */
                Upgrade: 7,
                /**
                 * The Via header, which specifies intermediate protocols to be used by gateway
                 and proxy agents.
                 *
                 * @static
                 * @public
                 * @memberof System.Net.HttpResponseHeader
                 * @constant
                 * @default 8
                 * @type System.Net.HttpResponseHeader
                 */
                Via: 8,
                /**
                 * The Warning header, which specifies additional information about that status
                 or transformation of a message that might not be reflected in the message.
                 *
                 * @static
                 * @public
                 * @memberof System.Net.HttpResponseHeader
                 * @constant
                 * @default 9
                 * @type System.Net.HttpResponseHeader
                 */
                Warning: 9,
                /**
                 * The Allow header, which specifies the set of HTTP methods that are supported.
                 *
                 * @static
                 * @public
                 * @memberof System.Net.HttpResponseHeader
                 * @constant
                 * @default 10
                 * @type System.Net.HttpResponseHeader
                 */
                Allow: 10,
                /**
                 * The Content-Length header, which specifies the length, in bytes, of the accompanying
                 body data.
                 *
                 * @static
                 * @public
                 * @memberof System.Net.HttpResponseHeader
                 * @constant
                 * @default 11
                 * @type System.Net.HttpResponseHeader
                 */
                ContentLength: 11,
                /**
                 * The Content-Type header, which specifies the MIME type of the accompanying
                 body data.
                 *
                 * @static
                 * @public
                 * @memberof System.Net.HttpResponseHeader
                 * @constant
                 * @default 12
                 * @type System.Net.HttpResponseHeader
                 */
                ContentType: 12,
                /**
                 * The Content-Encoding header, which specifies the encodings that have been
                 applied to the accompanying body data.
                 *
                 * @static
                 * @public
                 * @memberof System.Net.HttpResponseHeader
                 * @constant
                 * @default 13
                 * @type System.Net.HttpResponseHeader
                 */
                ContentEncoding: 13,
                /**
                 * The Content-Langauge header, which specifies the natural language or languages
                 of the accompanying body data.
                 *
                 * @static
                 * @public
                 * @memberof System.Net.HttpResponseHeader
                 * @constant
                 * @default 14
                 * @type System.Net.HttpResponseHeader
                 */
                ContentLanguage: 14,
                /**
                 * The Content-Location header, which specifies a URI from which the accompanying
                 body can be obtained.
                 *
                 * @static
                 * @public
                 * @memberof System.Net.HttpResponseHeader
                 * @constant
                 * @default 15
                 * @type System.Net.HttpResponseHeader
                 */
                ContentLocation: 15,
                /**
                 * The Content-MD5 header, which specifies the MD5 digest of the accompanying
                 body data, for the purpose of providing an end-to-end message integrity check.
                 *
                 * @static
                 * @public
                 * @memberof System.Net.HttpResponseHeader
                 * @constant
                 * @default 16
                 * @type System.Net.HttpResponseHeader
                 */
                ContentMd5: 16,
                /**
                 * The Range header, which specifies the subrange or subranges of the response
                 that the client requests be returned in lieu of the entire response.
                 *
                 * @static
                 * @public
                 * @memberof System.Net.HttpResponseHeader
                 * @constant
                 * @default 17
                 * @type System.Net.HttpResponseHeader
                 */
                ContentRange: 17,
                /**
                 * The Expires header, which specifies the date and time after which the accompanying
                 body data should be considered stale.
                 *
                 * @static
                 * @public
                 * @memberof System.Net.HttpResponseHeader
                 * @constant
                 * @default 18
                 * @type System.Net.HttpResponseHeader
                 */
                Expires: 18,
                /**
                 * The Last-Modified header, which specifies the date and time at which the
                 accompanying body data was last modified.
                 *
                 * @static
                 * @public
                 * @memberof System.Net.HttpResponseHeader
                 * @constant
                 * @default 19
                 * @type System.Net.HttpResponseHeader
                 */
                LastModified: 19,
                /**
                 * The Accept-Ranges header, which specifies the range that is accepted by the
                 server.
                 *
                 * @static
                 * @public
                 * @memberof System.Net.HttpResponseHeader
                 * @constant
                 * @default 20
                 * @type System.Net.HttpResponseHeader
                 */
                AcceptRanges: 20,
                /**
                 * The Age header, which specifies the time, in seconds, since the response
                 was generated by the originating server.
                 *
                 * @static
                 * @public
                 * @memberof System.Net.HttpResponseHeader
                 * @constant
                 * @default 21
                 * @type System.Net.HttpResponseHeader
                 */
                Age: 21,
                /**
                 * The Etag header, which specifies the current value for the requested variant.
                 *
                 * @static
                 * @public
                 * @memberof System.Net.HttpResponseHeader
                 * @constant
                 * @default 22
                 * @type System.Net.HttpResponseHeader
                 */
                ETag: 22,
                /**
                 * The Location header, which specifies a URI to which the client is redirected
                 to obtain the requested resource.
                 *
                 * @static
                 * @public
                 * @memberof System.Net.HttpResponseHeader
                 * @constant
                 * @default 23
                 * @type System.Net.HttpResponseHeader
                 */
                Location: 23,
                /**
                 * The Proxy-Authenticate header, which specifies that the client must authenticate
                 itself to a proxy.
                 *
                 * @static
                 * @public
                 * @memberof System.Net.HttpResponseHeader
                 * @constant
                 * @default 24
                 * @type System.Net.HttpResponseHeader
                 */
                ProxyAuthenticate: 24,
                /**
                 * The Retry-After header, which specifies a time (in seconds), or a date and
                 time, after which the client can retry its request.
                 *
                 * @static
                 * @public
                 * @memberof System.Net.HttpResponseHeader
                 * @constant
                 * @default 25
                 * @type System.Net.HttpResponseHeader
                 */
                RetryAfter: 25,
                /**
                 * The Server header, which specifies information about the originating server
                 agent.
                 *
                 * @static
                 * @public
                 * @memberof System.Net.HttpResponseHeader
                 * @constant
                 * @default 26
                 * @type System.Net.HttpResponseHeader
                 */
                Server: 26,
                /**
                 * The Set-Cookie header, which specifies cookie data that is presented to the
                 client.
                 *
                 * @static
                 * @public
                 * @memberof System.Net.HttpResponseHeader
                 * @constant
                 * @default 27
                 * @type System.Net.HttpResponseHeader
                 */
                SetCookie: 27,
                /**
                 * The Vary header, which specifies the request headers that are used to determine
                 whether a cached response is fresh.
                 *
                 * @static
                 * @public
                 * @memberof System.Net.HttpResponseHeader
                 * @constant
                 * @default 28
                 * @type System.Net.HttpResponseHeader
                 */
                Vary: 28,
                /**
                 * The WWW-Authenticate header, which specifies that the client must authenticate
                 itself to the server.
                 *
                 * @static
                 * @public
                 * @memberof System.Net.HttpResponseHeader
                 * @constant
                 * @default 29
                 * @type System.Net.HttpResponseHeader
                 */
                WwwAuthenticate: 29
            }
        }
    });

    /**
     * Contains the values of status codes defined for HTTP.
     *
     * @public
     * @class System.Net.HttpStatusCode
     */
    Bridge.define("System.Net.HttpStatusCode", {
        $kind: "enum",
        statics: {
            fields: {
                /**
                 * Equivalent to HTTP status 100. System.Net.HttpStatusCode.Continue indicates
                 that the client can continue with its request.
                 *
                 * @static
                 * @public
                 * @memberof System.Net.HttpStatusCode
                 * @constant
                 * @default 100
                 * @type System.Net.HttpStatusCode
                 */
                Continue: 100,
                /**
                 * Equivalent to HTTP status 101. System.Net.HttpStatusCode.SwitchingProtocols
                 indicates that the protocol version or protocol is being changed.
                 *
                 * @static
                 * @public
                 * @memberof System.Net.HttpStatusCode
                 * @constant
                 * @default 101
                 * @type System.Net.HttpStatusCode
                 */
                SwitchingProtocols: 101,
                /**
                 * Equivalent to HTTP status 200. System.Net.HttpStatusCode.OK indicates that
                 the request succeeded and that the requested information is in the response.
                 This is the most common status code to receive.
                 *
                 * @static
                 * @public
                 * @memberof System.Net.HttpStatusCode
                 * @constant
                 * @default 200
                 * @type System.Net.HttpStatusCode
                 */
                OK: 200,
                /**
                 * Equivalent to HTTP status 201. System.Net.HttpStatusCode.Created indicates
                 that the request resulted in a new resource created before the response was
                 sent.
                 *
                 * @static
                 * @public
                 * @memberof System.Net.HttpStatusCode
                 * @constant
                 * @default 201
                 * @type System.Net.HttpStatusCode
                 */
                Created: 201,
                /**
                 * Equivalent to HTTP status 202. System.Net.HttpStatusCode.Accepted indicates
                 that the request has been accepted for further processing.
                 *
                 * @static
                 * @public
                 * @memberof System.Net.HttpStatusCode
                 * @constant
                 * @default 202
                 * @type System.Net.HttpStatusCode
                 */
                Accepted: 202,
                /**
                 * Equivalent to HTTP status 203. System.Net.HttpStatusCode.NonAuthoritativeInformation
                 indicates that the returned metainformation is from a cached copy instead
                 of the origin server and therefore may be incorrect.
                 *
                 * @static
                 * @public
                 * @memberof System.Net.HttpStatusCode
                 * @constant
                 * @default 203
                 * @type System.Net.HttpStatusCode
                 */
                NonAuthoritativeInformation: 203,
                /**
                 * Equivalent to HTTP status 204. System.Net.HttpStatusCode.NoContent indicates
                 that the request has been successfully processed and that the response is
                 intentionally blank.
                 *
                 * @static
                 * @public
                 * @memberof System.Net.HttpStatusCode
                 * @constant
                 * @default 204
                 * @type System.Net.HttpStatusCode
                 */
                NoContent: 204,
                /**
                 * Equivalent to HTTP status 205. System.Net.HttpStatusCode.ResetContent indicates
                 that the client should reset (not reload) the current resource.
                 *
                 * @static
                 * @public
                 * @memberof System.Net.HttpStatusCode
                 * @constant
                 * @default 205
                 * @type System.Net.HttpStatusCode
                 */
                ResetContent: 205,
                /**
                 * Equivalent to HTTP status 206. System.Net.HttpStatusCode.PartialContent indicates
                 that the response is a partial response as requested by a GET request that
                 includes a byte range.
                 *
                 * @static
                 * @public
                 * @memberof System.Net.HttpStatusCode
                 * @constant
                 * @default 206
                 * @type System.Net.HttpStatusCode
                 */
                PartialContent: 206,
                /**
                 * Equivalent to HTTP status 300. System.Net.HttpStatusCode.MultipleChoices
                 indicates that the requested information has multiple representations. The
                 default action is to treat this status as a redirect and follow the contents
                 of the Location header associated with this response.
                 *
                 * @static
                 * @public
                 * @memberof System.Net.HttpStatusCode
                 * @constant
                 * @default 300
                 * @type System.Net.HttpStatusCode
                 */
                MultipleChoices: 300,
                /**
                 * Equivalent to HTTP status 300. System.Net.HttpStatusCode.Ambiguous indicates
                 that the requested information has multiple representations. The default
                 action is to treat this status as a redirect and follow the contents of the
                 Location header associated with this response.
                 *
                 * @static
                 * @public
                 * @memberof System.Net.HttpStatusCode
                 * @constant
                 * @default 300
                 * @type System.Net.HttpStatusCode
                 */
                Ambiguous: 300,
                /**
                 * Equivalent to HTTP status 301. System.Net.HttpStatusCode.MovedPermanently
                 indicates that the requested information has been moved to the URI specified
                 in the Location header. The default action when this status is received is
                 to follow the Location header associated with the response.
                 *
                 * @static
                 * @public
                 * @memberof System.Net.HttpStatusCode
                 * @constant
                 * @default 301
                 * @type System.Net.HttpStatusCode
                 */
                MovedPermanently: 301,
                /**
                 * Equivalent to HTTP status 301. System.Net.HttpStatusCode.Moved indicates
                 that the requested information has been moved to the URI specified in the
                 Location header. The default action when this status is received is to follow
                 the Location header associated with the response. When the original request
                 method was POST, the redirected request will use the GET method.
                 *
                 * @static
                 * @public
                 * @memberof System.Net.HttpStatusCode
                 * @constant
                 * @default 301
                 * @type System.Net.HttpStatusCode
                 */
                Moved: 301,
                /**
                 * Equivalent to HTTP status 302. System.Net.HttpStatusCode.Found indicates
                 that the requested information is located at the URI specified in the Location
                 header. The default action when this status is received is to follow the
                 Location header associated with the response. When the original request method
                 was POST, the redirected request will use the GET method.
                 *
                 * @static
                 * @public
                 * @memberof System.Net.HttpStatusCode
                 * @constant
                 * @default 302
                 * @type System.Net.HttpStatusCode
                 */
                Found: 302,
                /**
                 * Equivalent to HTTP status 302. System.Net.HttpStatusCode.Redirect indicates
                 that the requested information is located at the URI specified in the Location
                 header. The default action when this status is received is to follow the
                 Location header associated with the response. When the original request method
                 was POST, the redirected request will use the GET method.
                 *
                 * @static
                 * @public
                 * @memberof System.Net.HttpStatusCode
                 * @constant
                 * @default 302
                 * @type System.Net.HttpStatusCode
                 */
                Redirect: 302,
                /**
                 * Equivalent to HTTP status 303. System.Net.HttpStatusCode.SeeOther automatically
                 redirects the client to the URI specified in the Location header as the result
                 of a POST. The request to the resource specified by the Location header will
                 be made with a GET.
                 *
                 * @static
                 * @public
                 * @memberof System.Net.HttpStatusCode
                 * @constant
                 * @default 303
                 * @type System.Net.HttpStatusCode
                 */
                SeeOther: 303,
                /**
                 * Equivalent to HTTP status 303. System.Net.HttpStatusCode.RedirectMethod automatically
                 redirects the client to the URI specified in the Location header as the result
                 of a POST. The request to the resource specified by the Location header will
                 be made with a GET.
                 *
                 * @static
                 * @public
                 * @memberof System.Net.HttpStatusCode
                 * @constant
                 * @default 303
                 * @type System.Net.HttpStatusCode
                 */
                RedirectMethod: 303,
                /**
                 * Equivalent to HTTP status 304. System.Net.HttpStatusCode.NotModified indicates
                 that the client's cached copy is up to date. The contents of the resource
                 are not transferred.
                 *
                 * @static
                 * @public
                 * @memberof System.Net.HttpStatusCode
                 * @constant
                 * @default 304
                 * @type System.Net.HttpStatusCode
                 */
                NotModified: 304,
                /**
                 * Equivalent to HTTP status 305. System.Net.HttpStatusCode.UseProxy indicates
                 that the request should use the proxy server at the URI specified in the
                 Location header.
                 *
                 * @static
                 * @public
                 * @memberof System.Net.HttpStatusCode
                 * @constant
                 * @default 305
                 * @type System.Net.HttpStatusCode
                 */
                UseProxy: 305,
                /**
                 * Equivalent to HTTP status 306. System.Net.HttpStatusCode.Unused is a proposed
                 extension to the HTTP/1.1 specification that is not fully specified.
                 *
                 * @static
                 * @public
                 * @memberof System.Net.HttpStatusCode
                 * @constant
                 * @default 306
                 * @type System.Net.HttpStatusCode
                 */
                Unused: 306,
                /**
                 * Equivalent to HTTP status 307. System.Net.HttpStatusCode.RedirectKeepVerb
                 indicates that the request information is located at the URI specified in
                 the Location header. The default action when this status is received is to
                 follow the Location header associated with the response. When the original
                 request method was POST, the redirected request will also use the POST method.
                 *
                 * @static
                 * @public
                 * @memberof System.Net.HttpStatusCode
                 * @constant
                 * @default 307
                 * @type System.Net.HttpStatusCode
                 */
                RedirectKeepVerb: 307,
                /**
                 * Equivalent to HTTP status 307. System.Net.HttpStatusCode.TemporaryRedirect
                 indicates that the request information is located at the URI specified in
                 the Location header. The default action when this status is received is to
                 follow the Location header associated with the response. When the original
                 request method was POST, the redirected request will also use the POST method.
                 *
                 * @static
                 * @public
                 * @memberof System.Net.HttpStatusCode
                 * @constant
                 * @default 307
                 * @type System.Net.HttpStatusCode
                 */
                TemporaryRedirect: 307,
                /**
                 * Equivalent to HTTP status 400. System.Net.HttpStatusCode.BadRequest indicates
                 that the request could not be understood by the server. System.Net.HttpStatusCode.BadRequest
                 is sent when no other error is applicable, or if the exact error is unknown
                 or does not have its own error code.
                 *
                 * @static
                 * @public
                 * @memberof System.Net.HttpStatusCode
                 * @constant
                 * @default 400
                 * @type System.Net.HttpStatusCode
                 */
                BadRequest: 400,
                /**
                 * Equivalent to HTTP status 401. System.Net.HttpStatusCode.Unauthorized indicates
                 that the requested resource requires authentication. The WWW-Authenticate
                 header contains the details of how to perform the authentication.
                 *
                 * @static
                 * @public
                 * @memberof System.Net.HttpStatusCode
                 * @constant
                 * @default 401
                 * @type System.Net.HttpStatusCode
                 */
                Unauthorized: 401,
                /**
                 * Equivalent to HTTP status 402. System.Net.HttpStatusCode.PaymentRequired
                 is reserved for future use.
                 *
                 * @static
                 * @public
                 * @memberof System.Net.HttpStatusCode
                 * @constant
                 * @default 402
                 * @type System.Net.HttpStatusCode
                 */
                PaymentRequired: 402,
                /**
                 * Equivalent to HTTP status 403. System.Net.HttpStatusCode.Forbidden indicates
                 that the server refuses to fulfill the request.
                 *
                 * @static
                 * @public
                 * @memberof System.Net.HttpStatusCode
                 * @constant
                 * @default 403
                 * @type System.Net.HttpStatusCode
                 */
                Forbidden: 403,
                /**
                 * Equivalent to HTTP status 404. System.Net.HttpStatusCode.NotFound indicates
                 that the requested resource does not exist on the server.
                 *
                 * @static
                 * @public
                 * @memberof System.Net.HttpStatusCode
                 * @constant
                 * @default 404
                 * @type System.Net.HttpStatusCode
                 */
                NotFound: 404,
                /**
                 * Equivalent to HTTP status 405. System.Net.HttpStatusCode.MethodNotAllowed
                 indicates that the request method (POST or GET) is not allowed on the requested
                 resource.
                 *
                 * @static
                 * @public
                 * @memberof System.Net.HttpStatusCode
                 * @constant
                 * @default 405
                 * @type System.Net.HttpStatusCode
                 */
                MethodNotAllowed: 405,
                /**
                 * Equivalent to HTTP status 406. System.Net.HttpStatusCode.NotAcceptable indicates
                 that the client has indicated with Accept headers that it will not accept
                 any of the available representations of the resource.
                 *
                 * @static
                 * @public
                 * @memberof System.Net.HttpStatusCode
                 * @constant
                 * @default 406
                 * @type System.Net.HttpStatusCode
                 */
                NotAcceptable: 406,
                /**
                 * Equivalent to HTTP status 407. System.Net.HttpStatusCode.ProxyAuthenticationRequired
                 indicates that the requested proxy requires authentication. The Proxy-authenticate
                 header contains the details of how to perform the authentication.
                 *
                 * @static
                 * @public
                 * @memberof System.Net.HttpStatusCode
                 * @constant
                 * @default 407
                 * @type System.Net.HttpStatusCode
                 */
                ProxyAuthenticationRequired: 407,
                /**
                 * Equivalent to HTTP status 408. System.Net.HttpStatusCode.RequestTimeout indicates
                 that the client did not send a request within the time the server was expecting
                 the request.
                 *
                 * @static
                 * @public
                 * @memberof System.Net.HttpStatusCode
                 * @constant
                 * @default 408
                 * @type System.Net.HttpStatusCode
                 */
                RequestTimeout: 408,
                /**
                 * Equivalent to HTTP status 409. System.Net.HttpStatusCode.Conflict indicates
                 that the request could not be carried out because of a conflict on the server.
                 *
                 * @static
                 * @public
                 * @memberof System.Net.HttpStatusCode
                 * @constant
                 * @default 409
                 * @type System.Net.HttpStatusCode
                 */
                Conflict: 409,
                /**
                 * Equivalent to HTTP status 410. System.Net.HttpStatusCode.Gone indicates that
                 the requested resource is no longer available.
                 *
                 * @static
                 * @public
                 * @memberof System.Net.HttpStatusCode
                 * @constant
                 * @default 410
                 * @type System.Net.HttpStatusCode
                 */
                Gone: 410,
                /**
                 * Equivalent to HTTP status 411. System.Net.HttpStatusCode.LengthRequired indicates
                 that the required Content-length header is missing.
                 *
                 * @static
                 * @public
                 * @memberof System.Net.HttpStatusCode
                 * @constant
                 * @default 411
                 * @type System.Net.HttpStatusCode
                 */
                LengthRequired: 411,
                /**
                 * Equivalent to HTTP status 412. System.Net.HttpStatusCode.PreconditionFailed
                 indicates that a condition set for this request failed, and the request cannot
                 be carried out. Conditions are set with conditional request headers like
                 If-Match, If-None-Match, or If-Unmodified-Since.
                 *
                 * @static
                 * @public
                 * @memberof System.Net.HttpStatusCode
                 * @constant
                 * @default 412
                 * @type System.Net.HttpStatusCode
                 */
                PreconditionFailed: 412,
                /**
                 * Equivalent to HTTP status 413. System.Net.HttpStatusCode.RequestEntityTooLarge
                 indicates that the request is too large for the server to process.
                 *
                 * @static
                 * @public
                 * @memberof System.Net.HttpStatusCode
                 * @constant
                 * @default 413
                 * @type System.Net.HttpStatusCode
                 */
                RequestEntityTooLarge: 413,
                /**
                 * Equivalent to HTTP status 414. System.Net.HttpStatusCode.RequestUriTooLong
                 indicates that the URI is too long.
                 *
                 * @static
                 * @public
                 * @memberof System.Net.HttpStatusCode
                 * @constant
                 * @default 414
                 * @type System.Net.HttpStatusCode
                 */
                RequestUriTooLong: 414,
                /**
                 * Equivalent to HTTP status 415. System.Net.HttpStatusCode.UnsupportedMediaType
                 indicates that the request is an unsupported type.
                 *
                 * @static
                 * @public
                 * @memberof System.Net.HttpStatusCode
                 * @constant
                 * @default 415
                 * @type System.Net.HttpStatusCode
                 */
                UnsupportedMediaType: 415,
                /**
                 * Equivalent to HTTP status 416. System.Net.HttpStatusCode.RequestedRangeNotSatisfiable
                 indicates that the range of data requested from the resource cannot be returned,
                 either because the beginning of the range is before the beginning of the
                 resource, or the end of the range is after the end of the resource.
                 *
                 * @static
                 * @public
                 * @memberof System.Net.HttpStatusCode
                 * @constant
                 * @default 416
                 * @type System.Net.HttpStatusCode
                 */
                RequestedRangeNotSatisfiable: 416,
                /**
                 * Equivalent to HTTP status 417. System.Net.HttpStatusCode.ExpectationFailed
                 indicates that an expectation given in an Expect header could not be met
                 by the server.
                 *
                 * @static
                 * @public
                 * @memberof System.Net.HttpStatusCode
                 * @constant
                 * @default 417
                 * @type System.Net.HttpStatusCode
                 */
                ExpectationFailed: 417,
                /**
                 * Equivalent to HTTP status 426. System.Net.HttpStatusCode.UpgradeRequired
                 indicates that the client should switch to a different protocol such as TLS/1.0.
                 *
                 * @static
                 * @public
                 * @memberof System.Net.HttpStatusCode
                 * @constant
                 * @default 426
                 * @type System.Net.HttpStatusCode
                 */
                UpgradeRequired: 426,
                /**
                 * Equivalent to HTTP status 500. System.Net.HttpStatusCode.InternalServerError
                 indicates that a generic error has occurred on the server.
                 *
                 * @static
                 * @public
                 * @memberof System.Net.HttpStatusCode
                 * @constant
                 * @default 500
                 * @type System.Net.HttpStatusCode
                 */
                InternalServerError: 500,
                /**
                 * Equivalent to HTTP status 501. System.Net.HttpStatusCode.NotImplemented indicates
                 that the server does not support the requested function.
                 *
                 * @static
                 * @public
                 * @memberof System.Net.HttpStatusCode
                 * @constant
                 * @default 501
                 * @type System.Net.HttpStatusCode
                 */
                NotImplemented: 501,
                /**
                 * Equivalent to HTTP status 502. System.Net.HttpStatusCode.BadGateway indicates
                 that an intermediate proxy server received a bad response from another proxy
                 or the origin server.
                 *
                 * @static
                 * @public
                 * @memberof System.Net.HttpStatusCode
                 * @constant
                 * @default 502
                 * @type System.Net.HttpStatusCode
                 */
                BadGateway: 502,
                /**
                 * Equivalent to HTTP status 503. System.Net.HttpStatusCode.ServiceUnavailable
                 indicates that the server is temporarily unavailable, usually due to high
                 load or maintenance.
                 *
                 * @static
                 * @public
                 * @memberof System.Net.HttpStatusCode
                 * @constant
                 * @default 503
                 * @type System.Net.HttpStatusCode
                 */
                ServiceUnavailable: 503,
                /**
                 * Equivalent to HTTP status 504. System.Net.HttpStatusCode.GatewayTimeout indicates
                 that an intermediate proxy server timed out while waiting for a response
                 from another proxy or the origin server.
                 *
                 * @static
                 * @public
                 * @memberof System.Net.HttpStatusCode
                 * @constant
                 * @default 504
                 * @type System.Net.HttpStatusCode
                 */
                GatewayTimeout: 504,
                /**
                 * Equivalent to HTTP status 505. System.Net.HttpStatusCode.HttpVersionNotSupported
                 indicates that the requested HTTP version is not supported by the server.
                 *
                 * @static
                 * @public
                 * @memberof System.Net.HttpStatusCode
                 * @constant
                 * @default 505
                 * @type System.Net.HttpStatusCode
                 */
                HttpVersionNotSupported: 505
            }
        }
    });

    Bridge.define("System.Net.WebRequest", {
        statics: {
            methods: {
                Create: function (requestUriString) {
                    if (requestUriString == null) {
                        throw new System.ArgumentNullException.$ctor1("requestUriString");
                    }
                    return System.Net.WebRequest.CreateHttp$1(new System.Uri(requestUriString));
                },
                Create$1: function (requestUri) {
                    if (System.Uri.equals(requestUri, null)) {
                        throw new System.ArgumentNullException.$ctor1("requestUri");
                    }
                    return System.Net.WebRequest.CreateHttp$1(requestUri);
                },
                CreateHttp: function (requestUriString) {
                    if (requestUriString == null) {
                        throw new System.ArgumentNullException.$ctor1("requestUriString");
                    }
                    return System.Net.WebRequest.CreateHttp$1(new System.Uri(requestUriString));
                },
                CreateHttp$1: function (requestUri) {
                    if (System.Uri.equals(requestUri, null)) {
                        throw new System.ArgumentNullException.$ctor1("requestUri");
                    }
                    return Bridge.cast(System.Net.WebRequest.CreateDefault(requestUri), System.Net.HttpWebRequest);
                },
                CreateDefault: function (requestUri) {
                    if (System.Uri.equals(requestUri, null)) {
                        throw new System.ArgumentNullException.$ctor1("requestUri");
                    }
                    var request = new System.Net.HttpWebRequest.$ctor1(requestUri);
                    return request;
                }
            }
        },
        fields: {
            _webRequestHelper: null
        },
        props: {
            RequestUri: {
                get: function () {
                    throw new System.NotImplementedException.ctor();
                }
            },
            Method: {
                get: function () {
                    throw new System.NotImplementedException.ctor();
                },
                set: function (value) {
                    throw new System.NotImplementedException.ctor();
                }
            },
            Headers: {
                get: function () {
                    throw new System.NotImplementedException.ctor();
                },
                set: function (value) {
                    throw new System.NotImplementedException.ctor();
                }
            }
        },
        ctors: {
            ctor: function () {
                this.$initialize();
                this._webRequestHelper = new System.INTERNAL_WebRequestHelper_JSOnly();
            }
        },
        methods: {
            GetResponse: function () {
                throw new System.NotImplementedException.ctor();
            },
            BeginGetResponse: function (callback, state) {
                throw new System.NotImplementedException.ctor();
            },
            EndGetResponse: function (asyncResult) {
                throw new System.NotImplementedException.ctor();
            }
        }
    });

    /**
     * Provides a response from a Uniform Resource Identifier (URI). This is an
     abstract class.
     *
     * @abstract
     * @public
     * @class System.Net.WebResponse
     */
    Bridge.define("System.Net.WebResponse", {
        methods: {
            /**
             * @instance
             * @public
             * @this System.Net.WebResponse
             * @memberof System.Net.WebResponse
             * @return  {System.IO.Stream}
             */
            GetResponseStream: function () {

                throw new System.NotImplementedException.ctor();
            }
        }
    });

    /**
     * Provides data for the System.Net.WebClient.DownloadStringCompleted event.
     *
     * @class System.Net.INTERNAL_WebRequestHelper_JSOnly_RequestCompletedEventArgs
     */
    Bridge.define("System.Net.INTERNAL_WebRequestHelper_JSOnly_RequestCompletedEventArgs", {
        fields: {
            /**
             * Gets the data that is downloaded by a Overload:System.Net.WebClient.DownloadStringAsync
             method.
             *
             * @instance
             * @public
             * @memberof System.Net.INTERNAL_WebRequestHelper_JSOnly_RequestCompletedEventArgs
             * @function Result
             * @type string
             */
            Result: null,
            /**
             * Gets the unique identifier for the asynchronous task.
             *
             * @instance
             * @public
             * @memberof System.Net.INTERNAL_WebRequestHelper_JSOnly_RequestCompletedEventArgs
             * @function UserState
             * @type System.Object
             */
            UserState: null,
            /**
             * Gets a value indicating whether an asynchronous operation has been canceled.
             *
             * @instance
             * @public
             * @memberof System.Net.INTERNAL_WebRequestHelper_JSOnly_RequestCompletedEventArgs
             * @function Cancelled
             * @type boolean
             */
            Cancelled: false,
            /**
             * Gets a value indicating which error occurred during an asynchronous operation.
             *
             * @instance
             * @public
             * @memberof System.Net.INTERNAL_WebRequestHelper_JSOnly_RequestCompletedEventArgs
             * @function Error
             * @type System.Exception
             */
            Error: null
        },
        ctors: {
            ctor: function () {
                this.$initialize();
            },
            $ctor1: function (e) {
                this.$initialize();
                this.Result = e.Result;
                this.UserState = e.UserState;
                this.Error = e.Error;
                this.Cancelled = e.Cancelled;
            }
        }
    });

    /**
     * The {@link } namespaces contain classes that provide a simple
     programming interface for a number of network protocols, programmatically access and
     update configuration settings for the System.Net namespaces, define cache policies for
     web resources, compose and send e-mail, represent Multipurpose Internet Mail Exchange (MIME)
     headers, access network traffic data and network address information, and access
     peer-to-peer networking functionality. Additional child namespaces provide a managed
     implementation of the Windows Sockets (Winsock) interface and provide access to network
     streams for secure communications between hosts.
     *
     * @class System.Net.NamespaceDoc
     */
    Bridge.define("System.Net.NamespaceDoc");

    /** @namespace System.Net.Security */

    /**
     * The {@link } namespace provides network streams for secure
     communications between hosts.
     *
     * @class System.Net.Security.NamespaceDoc
     */
    Bridge.define("System.Net.Security.NamespaceDoc");

    /**
     * Indicates the security services requested for an authenticated stream.
     *
     * @public
     * @class System.Net.Security.ProtectionLevel
     */
    Bridge.define("System.Net.Security.ProtectionLevel", {
        $kind: "enum",
        statics: {
            fields: {
                /**
                 * Authentication only.
                 *
                 * @static
                 * @public
                 * @memberof System.Net.Security.ProtectionLevel
                 * @constant
                 * @default 0
                 * @type System.Net.Security.ProtectionLevel
                 */
                None: 0,
                /**
                 * Sign data to help ensure the integrity of transmitted data.
                 *
                 * @static
                 * @public
                 * @memberof System.Net.Security.ProtectionLevel
                 * @constant
                 * @default 1
                 * @type System.Net.Security.ProtectionLevel
                 */
                Sign: 1,
                /**
                 * Encrypt and sign data to help ensure the confidentiality and integrity of
                 transmitted data.
                 *
                 * @static
                 * @public
                 * @memberof System.Net.Security.ProtectionLevel
                 * @constant
                 * @default 2
                 * @type System.Net.Security.ProtectionLevel
                 */
                EncryptAndSign: 2
            }
        }
    });

    /**
     * Specifies the security protocols that are supported by the Schannel security
     package.
     *
     * @public
     * @class System.Net.SecurityProtocolType
     */
    Bridge.define("System.Net.SecurityProtocolType", {
        $kind: "enum",
        statics: {
            fields: {
                /**
                 * Specifies the system default security protocol as defined by Schannel.
                 *
                 * @static
                 * @public
                 * @memberof System.Net.SecurityProtocolType
                 * @constant
                 * @default 0
                 * @type System.Net.SecurityProtocolType
                 */
                SystemDefault: 0,
                /**
                 * Specifies the Secure Socket Layer (SSL) 3.0 security protocol.
                 *
                 * @static
                 * @public
                 * @memberof System.Net.SecurityProtocolType
                 * @constant
                 * @default 48
                 * @type System.Net.SecurityProtocolType
                 */
                Ssl3: 48,
                /**
                 * Specifies the Transport Layer Security (TLS) 1.0 security protocol.
                 *
                 * @static
                 * @public
                 * @memberof System.Net.SecurityProtocolType
                 * @constant
                 * @default 192
                 * @type System.Net.SecurityProtocolType
                 */
                Tls: 192,
                /**
                 * Specifies the Transport Layer Security (TLS) 1.1 security protocol.
                 *
                 * @static
                 * @public
                 * @memberof System.Net.SecurityProtocolType
                 * @constant
                 * @default 768
                 * @type System.Net.SecurityProtocolType
                 */
                Tls11: 768,
                /**
                 * Specifies the Transport Layer Security (TLS) 1.2 security protocol.
                 *
                 * @static
                 * @public
                 * @memberof System.Net.SecurityProtocolType
                 * @constant
                 * @default 3072
                 * @type System.Net.SecurityProtocolType
                 */
                Tls12: 3072
            }
        },
        $flags: true
    });

    /**
     * Manages the collection of System.Net.ServicePoint objects.
     * // The default number of non-persistent connections (4) allowed on a System.Net.ServicePoint
    // object connected to an HTTP/1.0 or later server. This field is constant but is
    // no longer used in the .NET Framework 2.0.
    //
     * // The default number of persistent connections (2) allowed on a System.Net.ServicePoint
    // object connected to an HTTP/1.1 or later server. This field is constant and is
    // used to initialize the System.Net.ServicePointManager.DefaultConnectionLimit
    // property if the value of the System.Net.ServicePointManager.DefaultConnectionLimit
    // property has not been set either directly or through configuration.
    //
     * // Setting this property value to true causes all outbound TCP connections from
    // HttpWebRequest to use the native socket option SO_REUSE_UNICASTPORT on the socket.
    // This causes the underlying outgoing ports to be shared. This is useful for scenarios
    // where a large number of outgoing connections are made in a short time, and the
    // app risks running out of ports.
    //
     * // Gets or sets the callback to validate a server certificate.
    //
     * // Gets or sets policy for server certificates.
    //
     * // Gets or sets a value that indicates how long a Domain Name Service (DNS) resolution
    // is considered valid.
    //
     * // Gets or sets a value that indicates whether a Domain Name Service (DNS) resolution
    // rotates among the applicable Internet Protocol (IP) addresses.
    //
     * // Gets or sets a System.Boolean value that determines whether 100-Continue behavior
    // is used.
    //
     * // Determines whether the Nagle algorithm is used by the service points managed
    // by this System.Net.ServicePointManager object.
    //
     * // Gets or sets the maximum idle time of a System.Net.ServicePoint object.
    //
     * // Gets or sets the maximum number of concurrent connections allowed by a System.Net.ServicePoint
    // object.
    //
     * // Gets or sets the maximum number of System.Net.ServicePoint objects to maintain
    // at any time.
    //
     * // Gets or sets a System.Boolean value that indicates whether the certificate is
    // checked against the certificate authority revocation list.
    //
     * // Gets the System.Net.Security.EncryptionPolicy for this System.Net.ServicePointManager
    // instance.
    //
     * // Finds an existing System.Net.ServicePoint object or creates a new System.Net.ServicePoint
    // object to manage communications with the specified System.Uri object.
    //
     * // Finds an existing System.Net.ServicePoint object or creates a new System.Net.ServicePoint
    // object to manage communications with the specified Uniform Resource Identifier
    // (URI).
    //
     * // Finds an existing System.Net.ServicePoint object or creates a new System.Net.ServicePoint
    // object to manage communications with the specified System.Uri object.
    //
     * // Enables or disables the keep-alive option on a TCP connection.
    //
     *
     * @public
     * @class System.Net.ServicePointManager
     * @return  {[type]}        The System.Net.ServicePoint object that manages communications for the request.
     */
    Bridge.define("System.Net.ServicePointManager", {
        statics: {
            fields: {
                /**
                 * Gets or sets the security protocol used by the System.Net.ServicePoint objects
                 managed by the System.Net.ServicePointManager object.
                 *
                 * @static
                 * @public
                 * @memberof System.Net.ServicePointManager
                 * @function SecurityProtocol
                 * @type System.Net.SecurityProtocolType
                 */
                SecurityProtocol: 0
            }
        }
    });

    /**
     * Provides data for the System.Net.WebClient.UploadStringCompleted event.
     *
     * @public
     * @class System.Net.UploadStringCompletedEventArgs
     */
    Bridge.define("System.Net.UploadStringCompletedEventArgs", {
        fields: {
            /**
             * Gets a value indicating whether an asynchronous operation has been canceled.
             *
             * @instance
             * @public
             * @memberof System.Net.UploadStringCompletedEventArgs
             * @function Cancelled
             * @type boolean
             */
            Cancelled: false,
            /**
             * Gets a value indicating which error occurred during an asynchronous operation.
             *
             * @instance
             * @public
             * @memberof System.Net.UploadStringCompletedEventArgs
             * @function Error
             * @type System.Exception
             */
            Error: null,
            /**
             * Gets the unique identifier for the asynchronous task.
             *
             * @instance
             * @public
             * @memberof System.Net.UploadStringCompletedEventArgs
             * @function UserState
             * @type System.Object
             */
            UserState: null,
            /**
             * Gets the server reply to a string upload operation that is started by calling
             an Overload:System.Net.WebClient.UploadStringAsync method.
             *
             * @instance
             * @public
             * @memberof System.Net.UploadStringCompletedEventArgs
             * @function Result
             * @type string
             */
            Result: null
        },
        ctors: {
            /**
             * Initializes a new instance of the System.ComponentModel.AsyncCompletedEventArgs
             class.
             *
             * @instance
             * @public
             * @this System.Net.UploadStringCompletedEventArgs
             * @memberof System.Net.UploadStringCompletedEventArgs
             * @return  {void}
             */
            ctor: function () {
                this.$initialize();
            },
            /**
             * Initializes a new instance of the System.ComponentModel.AsyncCompletedEventArgs
             class.
             *
             * @instance
             * @public
             * @this System.Net.UploadStringCompletedEventArgs
             * @memberof System.Net.UploadStringCompletedEventArgs
             * @param   {System.Exception}    error        Any error that occurred during the asynchronous operation.
             * @param   {boolean}             cancelled    A value indicating whether the asynchronous operation was canceled.
             * @param   {System.Object}       userState    The optional user-supplied state object passed to the System.ComponentModel.BackgroundWorker.RunWorkerAsync(System.Object)
             method.
             * @return  {void}
             */
            $ctor1: function (error, cancelled, userState) {
                this.$initialize();
                this.Error = error;
                this.Cancelled = cancelled;
                this.UserState = userState;
            },
            $ctor2: function (e) {
                this.$initialize();
                this.Result = e.Result;
                this.UserState = e.UserState;
                this.Error = e.Error;
                this.Cancelled = e.Cancelled;
            },
            $ctor3: function (e) {
                this.$initialize();
                this.Result = e.Result;
                this.UserState = e.UserState;
                this.Error = e.Error;
                this.Cancelled = e.Cancelled;
            }
        }
    });

    /**
     * @memberof System.Net
     * @callback System.Net.UploadStringCompletedEventHandler
     * @param   {System.Object}                                sender    
     * @param   {System.Net.UploadStringCompletedEventArgs}    e
     * @return  {void}
     */

    /**
     * @memberof System.Net
     * @callback System.Net.DownloadStringCompletedEventHandler
     * @param   {System.Object}                                  sender    
     * @param   {System.Net.DownloadStringCompletedEventArgs}    e
     * @return  {void}
     */

    
    Bridge.define("System.Net.WebClient", {
        fields: {
            /**
             * Gets or sets the System.Text.Encoding used to upload and download strings.
             *
             * @instance
             * @public
             * @memberof System.Net.WebClient
             * @function Encoding
             * @type System.Text.Encoding
             */
            Encoding: null,
            _headers: null,
            ResponseHeaders: null
        },
        events: {
            /**
             * Occurs when an asynchronous resource-download operation completes.
             *
             * @instance
             * @public
             * @this System.Net.WebClient
             * @memberof System.Net.WebClient
             * @function addDownloadStringCompleted
             * @param   {System.Net.DownloadStringCompletedEventHandler}    value
             * @return  {void}
             */
            /**
             * Occurs when an asynchronous resource-download operation completes.
             *
             * @instance
             * @public
             * @this System.Net.WebClient
             * @memberof System.Net.WebClient
             * @function removeDownloadStringCompleted
             * @param   {System.Net.DownloadStringCompletedEventHandler}    value
             * @return  {void}
             */
            DownloadStringCompleted: null,
            /**
             * / Uploads the specified string to the specified resource. This method does
            / not block the calling thread.
            /
             * Occurs when an asynchronous string-upload operation completes.
             *
             * @instance
             * @public
             * @this System.Net.WebClient
             * @memberof System.Net.WebClient
             * @function addUploadStringCompleted
             * @param   {System.Net.UploadStringCompletedEventHandler}    value
             * @return  {void}
             */
            /**
             * / Uploads the specified string to the specified resource. This method does
            / not block the calling thread.
            /
             * Occurs when an asynchronous string-upload operation completes.
             *
             * @instance
             * @public
             * @this System.Net.WebClient
             * @memberof System.Net.WebClient
             * @function removeUploadStringCompleted
             * @param   {System.Net.UploadStringCompletedEventHandler}    value
             * @return  {void}
             */
            UploadStringCompleted: null
        },
        props: {
            /**
             * Gets or sets a collection of header name/value pairs associated with the
             request.
             *
             * @instance
             * @public
             * @memberof System.Net.WebClient
             * @function Headers
             * @type System.Net.WebHeaderCollection
             */
            Headers: {
                get: function () {
                    return this._headers;
                },
                set: function (value) {
                    this._headers = value;
                }
            }
        },
        ctors: {
            init: function () {
                this._headers = new System.Net.WebHeaderCollection();
            },
            /**
             * Initializes a new instance of the System.Net.WebClient class.
             *
             * @instance
             * @public
             * @this System.Net.WebClient
             * @memberof System.Net.WebClient
             * @return  {void}
             */
            ctor: function () {
                this.$initialize();
            }
        },
        methods: {
            OnDownloadStringCompleted: function (sender, e) {
                if (!Bridge.staticEquals(this.DownloadStringCompleted, null)) {
                    this.DownloadStringCompleted(sender, new System.Net.DownloadStringCompletedEventArgs.$ctor2(e));
                }
            },
            /**
             * Downloads the requested resource as a System.String. The resource to download
             is specified as a System.String containing the URI.
             *
             * @instance
             * @public
             * @this System.Net.WebClient
             * @memberof System.Net.WebClient
             * @param   {string}    address    A System.String containing the URI to download.
             * @return  {string}               A System.String containing the requested resource.
             */
            DownloadString: function (address) {
                return this.DownloadString$1(new System.Uri(address));
            },
            /**
             * Downloads the requested resource as a System.String. The resource to download
             is specified as a System.Uri.
             *
             * @instance
             * @public
             * @this System.Net.WebClient
             * @memberof System.Net.WebClient
             * @param   {System.Uri}    address    A System.Uri object containing the URI to download.
             * @return  {string}                   A System.String containing the requested resource.
             */
            DownloadString$1: function (address) {
                var $t;
                if (CSHTML5.Interop.IsRunningInTheSimulator) {
                    var wcFactory = DotNetForHtml5.Core.INTERNAL_Simulator.WebClientFactory;
                    return wcFactory.DownloadString(this.Headers._headers, address);
                }
                if (System.Uri.equals(address, null)) {
                    throw new System.ArgumentNullException.$ctor1("The address parameter in DownloadString cannot be null");
                }
                var guid = System.Guid.NewGuid();

                var webRequestHelper = new System.INTERNAL_WebRequestHelper_JSOnly();

                webRequestHelper.removeDownloadStringCompleted(Bridge.fn.cacheBind(this, this.OnDownloadStringCompleted));
                webRequestHelper.addDownloadStringCompleted(Bridge.fn.cacheBind(this, this.OnDownloadStringCompleted));

                var headers = new (System.Collections.Generic.Dictionary$2(System.String,System.String)).ctor();
                $t = Bridge.getEnumerator(this._headers.AllKeys);
                try {
                    while ($t.moveNext()) {
                        var key = $t.Current;
                        headers.add(key, this._headers.Get(key));
                    }
                } finally {
                    if (Bridge.is($t, System.IDisposable)) {
                        $t.System$IDisposable$Dispose();
                    }
                }
                return webRequestHelper.MakeRequest(address, "GET", this, headers, null, Bridge.fn.cacheBind(this, this.OnDownloadStringCompleted), false, this.GetCredentialsMode());
            },
            /**
             * Downloads the resource specified as a System.Uri. This method does not block
             the calling thread.
             *
             * @instance
             * @public
             * @this System.Net.WebClient
             * @memberof System.Net.WebClient
             * @param   {System.Uri}    address    A System.Uri containing the URI to download.
             * @return  {void}
             */
            DownloadStringAsync: function (address) {
                this.DownloadStringAsync$1(address, null);
            },
            /**
             * Downloads the specified string to the specified resource. This method does
             not block the calling thread.
             *
             * @instance
             * @private
             * @this System.Net.WebClient
             * @memberof System.Net.WebClient
             * @param   {System.Uri}       address      A System.Uri containing the URI to download.
             * @param   {System.Object}    userToken    A user-defined object that is passed to the method invoked when the asynchronous
             operation completes.
             * @return  {void}
             */
            DownloadStringAsync$1: function (address, userToken) {
                var $t;
                if (CSHTML5.Interop.IsRunningInTheSimulator) {
                    var wcFactory = DotNetForHtml5.Core.INTERNAL_Simulator.WebClientFactory;
                    var del = Bridge.fn.bind(this, function (s, e) {
                        var $t;
                        this.OnDownloadStringCompleted(this, ($t = new System.Net.INTERNAL_WebRequestHelper_JSOnly_RequestCompletedEventArgs.ctor(), $t.Result = e.Result, $t.Cancelled = e.Cancelled, $t.Error = e.Error, $t.UserState = e.UserState, $t));
                    });
                    wcFactory.DownloadStringAsync(this.Headers._headers, del, address, Bridge.unbox(userToken));
                    return;
                }
                if (System.Uri.equals(address, null)) {
                    throw new System.ArgumentNullException.$ctor1("The address parameter in DownloadStringAsync cannot be null");
                }
                var webRequestHelper = new System.INTERNAL_WebRequestHelper_JSOnly();

                var headers = new (System.Collections.Generic.Dictionary$2(System.String,System.String)).ctor();
                $t = Bridge.getEnumerator(this._headers.AllKeys);
                try {
                    while ($t.moveNext()) {
                        var key = $t.Current;
                        headers.add(key, this._headers.Get(key));
                    }
                } finally {
                    if (Bridge.is($t, System.IDisposable)) {
                        $t.System$IDisposable$Dispose();
                    }
                }

                webRequestHelper.MakeRequest(address, "GET", this, headers, null, Bridge.fn.cacheBind(this, this.OnDownloadStringCompleted), true, this.GetCredentialsMode());





            },
            /**
             * Downloads the resource as a String from the URI specified as an asynchronous operation using a task object.
             *
             * @instance
             * @public
             * @this System.Net.WebClient
             * @memberof System.Net.WebClient
             * @param   {string}                           address    A System.Uri containing the URI to download.
             * @return  {System.Threading.Tasks.Task$1}               Returns the resource as {@link }.
             */
            DownloadStringTaskAsync: function (address) {
                return this.DownloadStringTaskAsync$1(new System.Uri(address));
            },
            /**
             * Downloads the resource as a String from the URI specified as an asynchronous operation using a task object.
             *
             * @instance
             * @public
             * @this System.Net.WebClient
             * @memberof System.Net.WebClient
             * @param   {System.Uri}                       address    A System.Uri containing the URI to download.
             * @return  {System.Threading.Tasks.Task$1}               Returns the resource as {@link }.
             */
            DownloadStringTaskAsync$1: function (address) {
                var $t;
                if (CSHTML5.Interop.IsRunningInTheSimulator) {
                    var wcFactory = DotNetForHtml5.Core.INTERNAL_Simulator.WebClientFactory;
                    return wcFactory.DownloadStringTaskAsync(this.Headers._headers, address);
                }

                var webRequestHelper = new System.INTERNAL_WebRequestHelper_JSOnly();

                var headers = new (System.Collections.Generic.Dictionary$2(System.String,System.String)).ctor();
                $t = Bridge.getEnumerator(this._headers.AllKeys);
                try {
                    while ($t.moveNext()) {
                        var key = $t.Current;
                        headers.add(key, this._headers.Get(key));
                    }
                } finally {
                    if (Bridge.is($t, System.IDisposable)) {
                        $t.System$IDisposable$Dispose();
                    }
                }

                var taskCompletionSource = new System.Threading.Tasks.TaskCompletionSource();
                webRequestHelper.MakeRequest(address, "GET", this, headers, null, Bridge.fn.bind(this, function (sender, args) {
                    this.TriggerDownloadStringTaskCompleted(taskCompletionSource, args);
                }), true, this.GetCredentialsMode());

                return taskCompletionSource.task;
            },
            TriggerDownloadStringTaskCompleted: function (taskCompletionSource, e) {
                if (e.Error == null) {
                    taskCompletionSource.setResult(e.Result);
                } else {
                    taskCompletionSource.trySetException(e.Error);
                }
            },
            GetCredentialsMode: function () {

                var credentialMode = Bridge.Reflection.getMembers(Bridge.getType(this), 16, 284, "CredentialsMode");

                if (credentialMode != null) {
                    var mode = Bridge.Reflection.midel(credentialMode.g, this)();

                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(mode, System.Net.CredentialsMode), System.Int32));
                }

                return System.Net.CredentialsMode.Disabled;
            },
            /**
             * Uploads the specified string to the specified resource, using the POST method.
             *
             * @instance
             * @public
             * @this System.Net.WebClient
             * @memberof System.Net.WebClient
             * @param   {string}    address    The URI of the resource to receive the string. For Http resources, this URI
             must identify a resource that can accept a request sent with the POST method,
             such as a script or ASP page.
             * @param   {string}    data       The string to be uploaded.
             * @return  {string}               A System.String containing the response sent by the server.
             */
            UploadString: function (address, data) {
                return this.UploadString$3(new System.Uri(address), "POST", data);

            },
            /**
             * Uploads the specified string to the specified resource, using the POST method.
             *
             * @instance
             * @public
             * @this System.Net.WebClient
             * @memberof System.Net.WebClient
             * @param   {System.Uri}    address    The URI of the resource to receive the string. For Http resources, this URI
             must identify a resource that can accept a request sent with the POST method,
             such as a script or ASP page.
             * @param   {string}        data       The string to be uploaded.
             * @return  {string}                   A System.String containing the response sent by the server.
             */
            UploadString$2: function (address, data) {
                return this.UploadString$3(address, "POST", data);
            },
            /**
             * Uploads the specified string to the specified resource, using the specified
             method.
             *
             * @instance
             * @public
             * @this System.Net.WebClient
             * @memberof System.Net.WebClient
             * @param   {string}    address    The URI of the resource to receive the file. This URI must identify a resource
             that can accept a request sent with the method method.
             * @param   {string}    method     The HTTP method used to send the string to the resource. If null, the default
             is POST for http and STOR for ftp.
             * @param   {string}    data       The string to be uploaded.
             * @return  {string}               A System.String containing the response sent by the server.
             */
            UploadString$1: function (address, method, data) {
                return this.UploadString$3(new System.Uri(address), method, data);
            },
            /**
             * Uploads the specified string to the specified resource, using the specified
             method.
             *
             * @instance
             * @public
             * @this System.Net.WebClient
             * @memberof System.Net.WebClient
             * @param   {System.Uri}    address    The URI of the resource to receive the file. This URI must identify a resource
             that can accept a request sent with the method method.
             * @param   {string}        method     The HTTP method used to send the string to the resource. If null, the default
             is POST for http and STOR for ftp.
             * @param   {string}        data       The string to be uploaded.
             * @return  {string}                   A System.String containing the response sent by the server.
             */
            UploadString$3: function (address, method, data) {
                return this.UploadString$4(address, method, data, null, false);
            },
            UploadString$4: function (address, method, data, onCompleted, isAsync) {
                var $t;
                if (CSHTML5.Interop.IsRunningInTheSimulator) {
                    var wcFactory = DotNetForHtml5.Core.INTERNAL_Simulator.WebClientFactory;
                    if (isAsync) {
                        var del = Bridge.fn.bind(this, function (s, e) {
                            var $t;
                            onCompleted(this, ($t = new System.Net.INTERNAL_WebRequestHelper_JSOnly_RequestCompletedEventArgs.ctor(), $t.Result = e.Result, $t.Cancelled = e.Cancelled, $t.Error = e.Error, $t.UserState = e.UserState, $t));
                        });
                        wcFactory.UploadStringAsync(this.Headers._headers, del, address, method, data);
                        return "";
                    } else {
                        return wcFactory.UploadString(this.Headers._headers, address, method, data);
                    }
                }
                var webRequestHelper = new System.INTERNAL_WebRequestHelper_JSOnly();

                var headers = new (System.Collections.Generic.Dictionary$2(System.String,System.String)).ctor();
                $t = Bridge.getEnumerator(this._headers.AllKeys);
                try {
                    while ($t.moveNext()) {
                        var key = $t.Current;
                        headers.add(key, this._headers.Get(key));
                    }
                } finally {
                    if (Bridge.is($t, System.IDisposable)) {
                        $t.System$IDisposable$Dispose();
                    }
                }
                return webRequestHelper.MakeRequest(address, method, this, headers, data, onCompleted, isAsync, this.GetCredentialsMode());
            },
            /**
             * Uploads the specified string to the specified resource. This method does
             not block the calling thread.
             *
             * @instance
             * @public
             * @this System.Net.WebClient
             * @memberof System.Net.WebClient
             * @param   {System.Uri}    address    The URI of the resource to receive the file. For HTTP resources, this URI
             must identify a resource that can accept a request sent with the POST method,
             such as a script or ASP page.
             * @param   {string}        data       The string to be uploaded.
             * @return  {void}
             */
            UploadStringAsync: function (address, data) {
                this.UploadStringAsync$1(address, "POST", data);
            },
            /**
             * Uploads the specified string to the specified resource. This method does
             not block the calling thread.
             *
             * @instance
             * @public
             * @this System.Net.WebClient
             * @memberof System.Net.WebClient
             * @param   {System.Uri}    address    The URI of the resource to receive the file. For HTTP resources, this URI
             must identify a resource that can accept a request sent with the POST method,
             such as a script or ASP page.
             * @param   {string}        method     The HTTP method used to send the file to the resource. If null, the default
             is POST for http and STOR for ftp.
             * @param   {string}        data       The string to be uploaded.
             * @return  {void}
             */
            UploadStringAsync$1: function (address, method, data) {
                this.UploadString$4(address, method, data, Bridge.fn.cacheBind(this, this.INTERNAL_OnUploadStringCompleted), true);
            },
            INTERNAL_OnUploadStringCompleted: function (sender, e) {
                this.OnUploadStringCompleted(sender, new System.Net.UploadStringCompletedEventArgs.$ctor3(e));
            },
            /**
             * Uploads the specified string to the specified resource as an asynchronous operation using a task object.
             *
             * @instance
             * @public
             * @this System.Net.WebClient
             * @memberof System.Net.WebClient
             * @param   {string}                           address    The URI of the resource to receive the string. For HTTP resources, this URI
             must identify a resource that can accept a request sent with the POST method,
             such as a script or ASP page.
             * @param   {string}                           data       The string to be uploaded.
             * @return  {System.Threading.Tasks.Task$1}               Returns {@link }.
             The task object representing the asynchronous operation. The Result property
             on the task object returns a String containing the response sent by the server.
             */
            UploadStringTaskAsync: function (address, data) {
                return this.UploadStringTaskAsync$3(new System.Uri(address), "POST", data);
            },
            /**
             * Uploads the specified string to the specified resource as an asynchronous operation using a task object.
             *
             * @instance
             * @public
             * @this System.Net.WebClient
             * @memberof System.Net.WebClient
             * @param   {System.Uri}                       address    The URI of the resource to receive the string. For HTTP resources, this URI
             must identify a resource that can accept a request sent with the POST method,
             such as a script or ASP page.
             * @param   {string}                           data       The string to be uploaded.
             * @return  {System.Threading.Tasks.Task$1}               Returns {@link }.
             The task object representing the asynchronous operation. The Result property
             on the task object returns a String containing the response sent by the server.
             */
            UploadStringTaskAsync$2: function (address, data) {
                return this.UploadStringTaskAsync$3(address, "POST", data);
            },
            /**
             * Uploads the specified string to the specified resource as an asynchronous operation using a task object.
             *
             * @instance
             * @public
             * @this System.Net.WebClient
             * @memberof System.Net.WebClient
             * @param   {string}                           address    The URI of the resource to receive the string. For HTTP resources, this URI
             must identify a resource that can accept a request sent with the POST method,
             such as a script or ASP page.
             * @param   {string}                           method     The HTTP method used to send the file to the resource. If null, the default
             is POST for http and STOR for ftp.
             * @param   {string}                           data       The string to be uploaded.
             * @return  {System.Threading.Tasks.Task$1}               Returns {@link }.
             The task object representing the asynchronous operation. The Result property
             on the task object returns a String containing the response sent by the server.
             */
            UploadStringTaskAsync$1: function (address, method, data) {
                return this.UploadStringTaskAsync$3(new System.Uri(address), method, data);
            },
            /**
             * Uploads the specified string to the specified resource as an asynchronous operation using a task object.
             *
             * @instance
             * @public
             * @this System.Net.WebClient
             * @memberof System.Net.WebClient
             * @param   {System.Uri}                       address    The URI of the resource to receive the string. For HTTP resources, this URI
             must identify a resource that can accept a request sent with the POST method,
             such as a script or ASP page.
             * @param   {string}                           method     The HTTP method used to send the file to the resource. If null, the default
             is POST for http and STOR for ftp.
             * @param   {string}                           data       The string to be uploaded.
             * @return  {System.Threading.Tasks.Task$1}               Returns {@link }.
             The task object representing the asynchronous operation. The Result property
             on the task object returns a String containing the response sent by the server.
             */
            UploadStringTaskAsync$3: function (address, method, data) {
                var $t;
                if (CSHTML5.Interop.IsRunningInTheSimulator) {
                    var wcFactory = DotNetForHtml5.Core.INTERNAL_Simulator.WebClientFactory;
                    return wcFactory.UploadStringTaskAsync(this.Headers._headers, address, method, data);
                }

                var webRequestHelper = new System.INTERNAL_WebRequestHelper_JSOnly();

                var headers = new (System.Collections.Generic.Dictionary$2(System.String,System.String)).ctor();
                $t = Bridge.getEnumerator(this._headers.AllKeys);
                try {
                    while ($t.moveNext()) {
                        var key = $t.Current;
                        headers.add(key, this._headers.Get(key));
                    }
                } finally {
                    if (Bridge.is($t, System.IDisposable)) {
                        $t.System$IDisposable$Dispose();
                    }
                }

                var taskCompletionSource = new System.Threading.Tasks.TaskCompletionSource();
                webRequestHelper.MakeRequest(address, method, this, headers, data, Bridge.fn.bind(this, function (sender, args) {
                    this.TriggerUploadStringTaskCompleted(taskCompletionSource, args);
                }), true, this.GetCredentialsMode());

                return taskCompletionSource.task;
            },
            TriggerUploadStringTaskCompleted: function (taskCompletionSource, e) {
                if (e.Error == null) {
                    taskCompletionSource.setResult(e.Result);
                } else {
                    taskCompletionSource.trySetException(e.Error);
                }
            },
            OnUploadStringCompleted: function (sender, e) {
                if (!Bridge.staticEquals(this.UploadStringCompleted, null)) {
                    this.UploadStringCompleted(sender, e);
                }
            }
        }
    });

    Bridge.define("System.Net.WebClientWithCredentials.CookiesContainer", {
        $kind: "nested class",
        fields: {
            LastSessionId: null,
            _cookies: null
        },
        props: {
            Cookies: {
                get: function () {
                    return this._cookies;
                }
            }
        },
        ctors: {
            init: function () {
                this._cookies = new (System.Collections.Generic.HashSet2$1(System.String)).ctor();
            }
        },
        methods: {
            INTERNAL_OnAsyncResponse: function (sender, e) {
                var wc = Bridge.cast(sender, System.Net.WebClientWithCredentials);

                wc.INTERNAL_AddNewCookiesInContainer_SimulatorOnly(this);
            }
        }
    });

    Bridge.define("System.Net.WebException", {
        inherits: [System.InvalidOperationException],
        fields: {
            m_Status: 0,
            m_Response: null,
            m_InternalStatus: 0
        },
        props: {
            Status: {
                get: function () {
                    return this.m_Status;
                }
            },
            Response: {
                get: function () {
                    return this.m_Response;
                }
            },
            InternalStatus: {
                get: function () {
                    return this.m_InternalStatus;
                }
            }
        },
        ctors: {
            init: function () {
                this.m_Status = System.Net.WebExceptionStatus.UnknownError;
                this.m_InternalStatus = System.Net.WebExceptionInternalStatus.RequestFatal;
            },
            ctor: function () {
                this.$initialize();
                System.InvalidOperationException.ctor.call(this);

            },
            $ctor2: function (message) {
                System.Net.WebException.$ctor3.call(this, message, null);
            },
            $ctor3: function (message, innerException) {
                this.$initialize();
                System.InvalidOperationException.$ctor2.call(this, message, innerException);
            },
            $ctor6: function (message, status) {
                System.Net.WebException.$ctor4.call(this, message, null, status, null);
            },
            $ctor7: function (message, status, internalStatus, innerException) {
                System.Net.WebException.$ctor5.call(this, message, innerException, status, null, internalStatus);
            },
            $ctor4: function (message, innerException, status, response) {
                System.Net.WebException.$ctor8.call(this, message, null, innerException, status, response);
            },
            $ctor8: function (message, data, innerException, status, response) {
                this.$initialize();
                System.InvalidOperationException.$ctor2.call(this, (message || "") + ((data != null ? ": '" + (data || "") + "'" : "") || ""), innerException);
                this.m_Status = status;
                this.m_Response = response;
            },
            $ctor5: function (message, innerException, status, response, internalStatus) {
                System.Net.WebException.$ctor9.call(this, message, null, innerException, status, response, internalStatus);
            },
            $ctor9: function (message, data, innerException, status, response, internalStatus) {
                this.$initialize();
                System.InvalidOperationException.$ctor2.call(this, (message || "") + ((data != null ? ": '" + (data || "") + "'" : "") || ""), innerException);
                this.m_Status = status;
                this.m_Response = response;
                this.m_InternalStatus = internalStatus;
            },
            $ctor1: function (serializationInfo, streamingContext) {
                this.$initialize();
                System.InvalidOperationException.ctor.call(this);
            }
        }
    });

    Bridge.define("System.Net.WebExceptionInternalStatus", {
        $kind: "enum",
        statics: {
            fields: {
                RequestFatal: 0,
                ServicePointFatal: 1,
                Recoverable: 2,
                Isolated: 3
            }
        }
    });

    /**
     * Defines status codes for the System.Net.WebException class.
     *
     * @public
     * @class System.Net.WebExceptionStatus
     */
    Bridge.define("System.Net.WebExceptionStatus", {
        $kind: "enum",
        statics: {
            fields: {
                /**
                 * No error was encountered.
                 *
                 * @static
                 * @public
                 * @memberof System.Net.WebExceptionStatus
                 * @constant
                 * @default 0
                 * @type System.Net.WebExceptionStatus
                 */
                Success: 0,
                /**
                 * The name resolver service could not resolve the host name.
                 *
                 * @static
                 * @public
                 * @memberof System.Net.WebExceptionStatus
                 * @constant
                 * @default 1
                 * @type System.Net.WebExceptionStatus
                 */
                NameResolutionFailure: 1,
                /**
                 * The remote service point could not be contacted at the transport level.
                 *
                 * @static
                 * @public
                 * @memberof System.Net.WebExceptionStatus
                 * @constant
                 * @default 2
                 * @type System.Net.WebExceptionStatus
                 */
                ConnectFailure: 2,
                /**
                 * A complete response was not received from the remote server.
                 *
                 * @static
                 * @public
                 * @memberof System.Net.WebExceptionStatus
                 * @constant
                 * @default 3
                 * @type System.Net.WebExceptionStatus
                 */
                ReceiveFailure: 3,
                /**
                 * A complete request could not be sent to the remote server.
                 *
                 * @static
                 * @public
                 * @memberof System.Net.WebExceptionStatus
                 * @constant
                 * @default 4
                 * @type System.Net.WebExceptionStatus
                 */
                SendFailure: 4,
                /**
                 * The request was a piplined request and the connection was closed before the
                 response was received.
                 *
                 * @static
                 * @public
                 * @memberof System.Net.WebExceptionStatus
                 * @constant
                 * @default 5
                 * @type System.Net.WebExceptionStatus
                 */
                PipelineFailure: 5,
                /**
                 * The request was canceled, the System.Net.WebRequest.Abort() method was called,
                 or an unclassifiable error occurred. This is the default value for System.Net.WebException.Status.
                 *
                 * @static
                 * @public
                 * @memberof System.Net.WebExceptionStatus
                 * @constant
                 * @default 6
                 * @type System.Net.WebExceptionStatus
                 */
                RequestCanceled: 6,
                /**
                 * The response received from the server was complete but indicated a protocol-level
                 error. For example, an HTTP protocol error such as 401 Access Denied would
                 use this status.
                 *
                 * @static
                 * @public
                 * @memberof System.Net.WebExceptionStatus
                 * @constant
                 * @default 7
                 * @type System.Net.WebExceptionStatus
                 */
                ProtocolError: 7,
                /**
                 * The connection was prematurely closed.
                 *
                 * @static
                 * @public
                 * @memberof System.Net.WebExceptionStatus
                 * @constant
                 * @default 8
                 * @type System.Net.WebExceptionStatus
                 */
                ConnectionClosed: 8,
                /**
                 * A server certificate could not be validated.
                 *
                 * @static
                 * @public
                 * @memberof System.Net.WebExceptionStatus
                 * @constant
                 * @default 9
                 * @type System.Net.WebExceptionStatus
                 */
                TrustFailure: 9,
                /**
                 * An error occurred while establishing a connection using SSL.
                 *
                 * @static
                 * @public
                 * @memberof System.Net.WebExceptionStatus
                 * @constant
                 * @default 10
                 * @type System.Net.WebExceptionStatus
                 */
                SecureChannelFailure: 10,
                /**
                 * The server response was not a valid HTTP response.
                 *
                 * @static
                 * @public
                 * @memberof System.Net.WebExceptionStatus
                 * @constant
                 * @default 11
                 * @type System.Net.WebExceptionStatus
                 */
                ServerProtocolViolation: 11,
                /**
                 * The connection for a request that specifies the Keep-alive header was closed
                 unexpectedly.
                 *
                 * @static
                 * @public
                 * @memberof System.Net.WebExceptionStatus
                 * @constant
                 * @default 12
                 * @type System.Net.WebExceptionStatus
                 */
                KeepAliveFailure: 12,
                /**
                 * An internal asynchronous request is pending.
                 *
                 * @static
                 * @public
                 * @memberof System.Net.WebExceptionStatus
                 * @constant
                 * @default 13
                 * @type System.Net.WebExceptionStatus
                 */
                Pending: 13,
                /**
                 * No response was received during the time-out period for a request.
                 *
                 * @static
                 * @public
                 * @memberof System.Net.WebExceptionStatus
                 * @constant
                 * @default 14
                 * @type System.Net.WebExceptionStatus
                 */
                Timeout: 14,
                /**
                 * The name resolver service could not resolve the proxy host name.
                 *
                 * @static
                 * @public
                 * @memberof System.Net.WebExceptionStatus
                 * @constant
                 * @default 15
                 * @type System.Net.WebExceptionStatus
                 */
                ProxyNameResolutionFailure: 15,
                /**
                 * An exception of unknown type has occurred.
                 *
                 * @static
                 * @public
                 * @memberof System.Net.WebExceptionStatus
                 * @constant
                 * @default 16
                 * @type System.Net.WebExceptionStatus
                 */
                UnknownError: 16,
                /**
                 * A message was received that exceeded the specified limit when sending a request
                 or receiving a response from the server.
                 *
                 * @static
                 * @public
                 * @memberof System.Net.WebExceptionStatus
                 * @constant
                 * @default 17
                 * @type System.Net.WebExceptionStatus
                 */
                MessageLengthLimitExceeded: 17,
                /**
                 * The specified cache entry was not found.
                 *
                 * @static
                 * @public
                 * @memberof System.Net.WebExceptionStatus
                 * @constant
                 * @default 18
                 * @type System.Net.WebExceptionStatus
                 */
                CacheEntryNotFound: 18,
                /**
                 * The request was not permitted by the cache policy. In general, this occurs
                 when a request is not cacheable and the effective policy prohibits sending
                 the request to the server. You might receive this status if a request method
                 implies the presence of a request body, a request method requires direct
                 interaction with the server, or a request contains a conditional header.
                 *
                 * @static
                 * @public
                 * @memberof System.Net.WebExceptionStatus
                 * @constant
                 * @default 19
                 * @type System.Net.WebExceptionStatus
                 */
                RequestProhibitedByCachePolicy: 19,
                /**
                 * This request was not permitted by the proxy.
                 *
                 * @static
                 * @public
                 * @memberof System.Net.WebExceptionStatus
                 * @constant
                 * @default 20
                 * @type System.Net.WebExceptionStatus
                 */
                RequestProhibitedByProxy: 20
            }
        }
    });

    /**
     * Contains protocol headers associated with a request or response.
     *
     * @public
     * @class System.Net.WebHeaderCollection
     */
    Bridge.define("System.Net.WebHeaderCollection", {
        fields: {
            _headers: null
        },
        props: {
            /**
             * Gets all header names (keys) in the collection.
             *
             * @instance
             * @public
             * @readonly
             * @memberof System.Net.WebHeaderCollection
             * @function AllKeys
             * @type Array.<string>
             */
            AllKeys: {
                get: function () {
                    return System.Linq.Enumerable.from(this._headers.Keys).ToArray();
                }
            },
            /**
             * Gets the number of headers in the collection.
             *
             * @instance
             * @public
             * @readonly
             * @memberof System.Net.WebHeaderCollection
             * @function Count
             * @type number
             */
            Count: {
                get: function () {
                    return this._headers.Count;
                }
            }
        },
        ctors: {
            /**
             * Initializes a new instance of the System.Net.WebHeaderCollection class.
             *
             * @instance
             * @public
             * @this System.Net.WebHeaderCollection
             * @memberof System.Net.WebHeaderCollection
             * @return  {void}
             */
            ctor: function () {
                this.$initialize();
                this._headers = new (System.Collections.Generic.Dictionary$2(System.String,System.String)).ctor();
            }
        },
        methods: {
            /**
             * Gets or sets the specified request header.
             *
             * @instance
             * @public
             * @this System.Net.WebHeaderCollection
             * @memberof System.Net.WebHeaderCollection
             * @param   {System.Net.HttpRequestHeader}    header    The request header value.
             * @return  {string}                                    A System.String instance containing the specified header value.
             */
            getItem: function (header) {
                return this._headers.getItem(System.Net.CSharpHeaderToHtmlHeaderConverter.Convert(header));
            },
            /**
             * Gets or sets the specified request header.
             *
             * @instance
             * @public
             * @this System.Net.WebHeaderCollection
             * @memberof System.Net.WebHeaderCollection
             * @param   {System.Net.HttpRequestHeader}    header    The request header value.
             * @param   {string}                          value
             * @return  {void}                                      A System.String instance containing the specified header value.
             */
            setItem: function (header, value) {
                this._headers.setItem(System.Net.CSharpHeaderToHtmlHeaderConverter.Convert(header), value);
            },
            /**
             * Gets or sets the specified response header.
             *
             * @instance
             * @public
             * @this System.Net.WebHeaderCollection
             * @memberof System.Net.WebHeaderCollection
             * @param   {System.Net.HttpResponseHeader}    header    The response header value.
             * @return  {string}                                     A System.String instance containing the specified header.
             */
            getItem$1: function (header) {
                return this._headers.getItem(System.Net.CSharpHeaderToHtmlHeaderConverter.Convert$1(header));
            },
            /**
             * Gets or sets the specified response header.
             *
             * @instance
             * @public
             * @this System.Net.WebHeaderCollection
             * @memberof System.Net.WebHeaderCollection
             * @param   {System.Net.HttpResponseHeader}    header    The response header value.
             * @param   {string}                           value
             * @return  {void}                                       A System.String instance containing the specified header.
             */
            setItem$1: function (header, value) {
                this._headers.setItem(System.Net.CSharpHeaderToHtmlHeaderConverter.Convert$1(header), value);
            },
            /**
             * Removes all headers from the collection.
             *
             * @instance
             * @public
             * @this System.Net.WebHeaderCollection
             * @memberof System.Net.WebHeaderCollection
             * @return  {void}
             */
            Clear: function () {
                this._headers.clear();
            },
            /**
             * Returns an enumerator that can iterate through the System.Net.WebHeaderCollection instance
             *
             * @instance
             * @public
             * @this System.Net.WebHeaderCollection
             * @memberof System.Net.WebHeaderCollection
             * @return  {System.Collections.IEnumerator}        An enumerator that can iterate through the System.Net.WebHeaderCollection instance
             */
            GetEnumerator: function () {
                return this._headers.GetEnumerator().$clone();
            },
            /**
             * Inserts the specified header into the collection.
             *
             * @instance
             * @public
             * @this System.Net.WebHeaderCollection
             * @memberof System.Net.WebHeaderCollection
             * @param   {string}    header    The header to add, with the name and value separated by a colon.
             * @return  {void}
             */
            Add$2: function (header) {
                var splitIndex = System.String.indexOf(header, String.fromCharCode(58));
                if (splitIndex <= 0) {
                    throw new System.ArgumentException.$ctor1("header must be of the form: \"header:value\".");
                }
                var headerPart = header.substr(0, splitIndex);
                var valuePart = header.substr(((splitIndex + 1) | 0));
                headerPart = headerPart.trim();
                valuePart = valuePart.trim();
                this.Add$3(headerPart, valuePart);
            },
            /**
             * Inserts the specified header with the specified value into the collection.
             *
             * @instance
             * @public
             * @this System.Net.WebHeaderCollection
             * @memberof System.Net.WebHeaderCollection
             * @param   {System.Net.HttpRequestHeader}    header    The header to add to the collection.
             * @param   {string}                          value     The content of the header.
             * @return  {void}
             */
            Add: function (header, value) {
                this.Add$3(System.Net.CSharpHeaderToHtmlHeaderConverter.Convert(header), value);
            },
            /**
             * Inserts the specified header with the specified value into the collection.
             *
             * @instance
             * @public
             * @this System.Net.WebHeaderCollection
             * @memberof System.Net.WebHeaderCollection
             * @param   {System.Net.HttpResponseHeader}    header    The header to add to the collection.
             * @param   {string}                           value     The content of the header.
             * @return  {void}
             */
            Add$1: function (header, value) {
                this.Add$3(System.Net.CSharpHeaderToHtmlHeaderConverter.Convert$1(header), value);
            },
            /**
             * Inserts a header with the specified name and value into the collection.
             *
             * @instance
             * @public
             * @this System.Net.WebHeaderCollection
             * @memberof System.Net.WebHeaderCollection
             * @param   {string}    name     The header to add to the collection.
             * @param   {string}    value    The content of the header.
             * @return  {void}
             */
            Add$3: function (name, value) {
                this._headers.add(name, value);
            },
            /**
             * Get the value of a particular header in the collection, specified by the
             name of the header.
             *
             * @instance
             * @public
             * @this System.Net.WebHeaderCollection
             * @memberof System.Net.WebHeaderCollection
             * @param   {string}    name    The name of the Web header.
             * @return  {string}            A System.String holding the value of the specified header.
             */
            Get: function (name) {
                return this._headers.getItem(name);
            },
            /**
             * Removes the specified header from the collection.
             *
             * @instance
             * @public
             * @this System.Net.WebHeaderCollection
             * @memberof System.Net.WebHeaderCollection
             * @param   {System.Net.HttpRequestHeader}    header    The System.Net.HttpRequestHeader instance to remove from the collection.
             * @return  {void}
             */
            Remove: function (header) {
                this.Remove$2(System.Net.CSharpHeaderToHtmlHeaderConverter.Convert(header));
            },
            /**
             * Removes the specified header from the collection.
             *
             * @instance
             * @public
             * @this System.Net.WebHeaderCollection
             * @memberof System.Net.WebHeaderCollection
             * @param   {System.Net.HttpResponseHeader}    header    The System.Net.HttpResponseHeader instance to remove from the collection.
             * @return  {void}
             */
            Remove$1: function (header) {
                this.Remove$2(System.Net.CSharpHeaderToHtmlHeaderConverter.Convert$1(header));
            },
            /**
             * Removes the specified header from the collection.
             *
             * @instance
             * @public
             * @this System.Net.WebHeaderCollection
             * @memberof System.Net.WebHeaderCollection
             * @param   {string}    name    The name of the header to remove from the collection.
             * @return  {void}
             */
            Remove$2: function (name) {
                this._headers.remove(name);
            },
            /**
             * Sets the specified header to the specified value.
             *
             * @instance
             * @public
             * @this System.Net.WebHeaderCollection
             * @memberof System.Net.WebHeaderCollection
             * @param   {string}    name     The header to set.
             * @param   {string}    value    The content of the header to set.
             * @return  {void}
             */
            Set: function (name, value) {
                if (this._headers.containsKey(name)) {
                    this._headers.setItem(name, value);
                } else {
                    this._headers.add(name, value);
                }
            },
            GetHeadersAsDictionaryStringString: function () {
                return this._headers;
            }
        }
    });

    /** @namespace System.Reflection */

    /**
     * The {@link } namespaces contain types that provide a managed
     view of loaded types, methods, and fields, and that can dynamically create and invoke
     types. A child namespace contains types that enable a compiler or other tool to
     emit metadata and Microsoft intermediate language (MSIL).
     *
     * @class System.Reflection.NamespaceDoc
     */
    Bridge.define("System.Reflection.NamespaceDoc");

    Bridge.define("System.Reflection.TargetInvocationException", {
        inherits: [System.ApplicationException],
        ctors: {
            ctor: function (inner) {
                this.$initialize();
                System.ApplicationException.$ctor1.call(this, inner.Message);
            },
            $ctor1: function (message, inner) {
                this.$initialize();
                System.ApplicationException.$ctor2.call(this, message, inner);
            }
        }
    });

    /** @namespace System.Resources */

    /**
     * The {@link } namespaces contain types that enable developers to
     create, store, and manage an application's culture-specific resources.
     *
     * @class System.Resources.NamespaceDoc
     */
    Bridge.define("System.Resources.NamespaceDoc");

    /**
     * Provides convenient access to culture-specific resources at run time.
     * // Creates a System.Resources.ResourceManager that looks up resources in satellite
    // assemblies based on information from the specified System.Type.
    //
     *
     * @public
     * @class System.Resources.ResourceManager
     */
    Bridge.define("System.Resources.ResourceManager", {
        fields: {
            /**
             * Indicates the root name of the resource files that the System.Resources.ResourceManager
             searches for resources.
             *
             * @instance
             * @protected
             * @memberof System.Resources.ResourceManager
             * @type string
             */
            BaseNameField: null,
            /**
             * Indicates the main System.Reflection.Assembly that contains the resources.
             *
             * @instance
             * @protected
             * @memberof System.Resources.ResourceManager
             * @type System.Reflection.Assembly
             */
            MainAssembly: null,
            /**
             * Gets or sets a Boolean value indicating whether the current instance of ResourceManager
             allows case-insensitive resource lookups in the System.Resources.ResourceManager.GetString(System.String)
             and System.Resources.ResourceManager.GetObject(System.String) methods.
             *
             * @instance
             * @public
             * @memberof System.Resources.ResourceManager
             * @function IgnoreCase
             * @type boolean
             */
            IgnoreCase: false
        },
        props: {
            /**
             * Gets the root name of the resource files that the System.Resources.ResourceManager
             searches for resources.
             *
             * @instance
             * @public
             * @readonly
             * @memberof System.Resources.ResourceManager
             * @function BaseName
             * @type string
             */
            BaseName: {
                get: function () {
                    return this.BaseNameField;
                }
            }
        },
        ctors: {
            /**
             * Initializes a new instance of the System.Resources.ResourceManager class that
             looks up resources contained in files derived from the specified root name using
             the given System.Reflection.Assembly.
             *
             * @instance
             * @public
             * @this System.Resources.ResourceManager
             * @memberof System.Resources.ResourceManager
             * @throws {System.ArgumentNullException} The baseName or assembly parameter is null.
             * @param   {string}                        baseName    The root name of the resource file without its extension but including a fully
             qualified namespace name. For example, the root name for the resource file named
             "MyApplication.MyResource.en-US.resources" is "MyApplication.MyResource".
             * @param   {System.Reflection.Assembly}    assembly    The main assembly for the resources.
             * @return  {void}
             */
            $ctor1: function (baseName, assembly) {
                this.$initialize();
                if (baseName == null) {
                    throw new System.ArgumentNullException.$ctor1("baseName");
                }
                if (assembly == null) {
                    throw new System.ArgumentNullException.$ctor1("assembly");
                }

                this.BaseNameField = baseName;
                this.MainAssembly = assembly;
            },
            /**
             * Initializes a new instance of the System.Resources.ResourceManager class with
             default values.
             *
             * @instance
             * @protected
             * @this System.Resources.ResourceManager
             * @memberof System.Resources.ResourceManager
             * @return  {void}
             */
            ctor: function () {
                this.$initialize();
            }
        },
        methods: {
            /**
             * Gets the value of the System.String resource localized for the specified culture.
             *
             * @instance
             * @public
             * @this System.Resources.ResourceManager
             * @memberof System.Resources.ResourceManager
             * @throws {System.ArgumentNullException} The name parameter is null
             * @param   {string}                              name       The name of the resource to get.
             * @param   {System.Globalization.CultureInfo}    culture    The culture for which the resource is localized. Note that if the resource is
             not localized for this culture, the lookup will fall back using the culture's
             System.Globalization.CultureInfo.Parent property, stopping after looking in the
             neutral culture.If this value is null, the System.Globalization.CultureInfo is
             obtained using the culture's System.Globalization.CultureInfo.CurrentUICulture
             property.
             * @return  {string}                                         The value of the resource localized for the specified culture. If a best match
             is not possible, null is returned.
             */
            GetString$1: function (name, culture) {
                var $t;
                if (name == null) {
                    throw new System.ArgumentNullException.$ctor1("name");
                }

                if (culture == null) {
                    culture = System.Globalization.CultureInfo.getCurrentCulture();
                }

                if (CSHTML5.Interop.IsRunningInTheSimulator) {
                    return name;
                } else {
                    var pathInJSThroughBaseNameField = "";
                    var currentElementInJSPath = document.ResXFiles;
                    var splittedPath = System.String.split(this.BaseNameField, [46].map(function (i) {{ return String.fromCharCode(i); }}));
                    var result = null;
                    try {
                        $t = Bridge.getEnumerator(splittedPath);
                        try {
                            while ($t.moveNext()) {
                                var pathPart = $t.Current;
                                currentElementInJSPath = currentElementInJSPath[pathPart];
                            }
                        } finally {
                            if (Bridge.is($t, System.IDisposable)) {
                                $t.System$IDisposable$Dispose();
                            }
                        }
                        result = System.Convert.toString(currentElementInJSPath[name]);
                        return result;
                    } catch ($e1) {
                        $e1 = System.Exception.create($e1);
                        return name;
                    }
                }
            },
            /**
             * Returns the value of the specified System.String resource.
             *
             * @instance
             * @public
             * @this System.Resources.ResourceManager
             * @memberof System.Resources.ResourceManager
             * @throws {System.ArgumentNullException} The name parameter is null
             * @param   {string}    name    The name of the resource to get.
             * @return  {string}            The value of the resource localized for the caller's current culture settings.
             If a match is not possible, null is returned.
             */
            GetString: function (name) {
                return this.GetString$1(name, null);
            }
        }
    });

    /** @namespace System.Runtime.CompilerServices */

    /**
     * The {@link } namespace provides functionality for
     compiler writers who use managed code to specify attributes in metadata that affect the
     run-time behavior of the common language runtime.
     *
     * @class System.Runtime.CompilerServices.NamespaceDoc
     */
    Bridge.define("System.Runtime.CompilerServices.NamespaceDoc");

    /** @namespace System.Runtime.InteropServices */

    /**
     * The {@link } namespace provides a wide variety of
     members that support COM interop and platform invoke services. If you are unfamiliar
     with these services, see Interoperating with Unmanaged Code.
     *
     * @class System.Runtime.InteropServices.NamespaceDoc
     */
    Bridge.define("System.Runtime.InteropServices.NamespaceDoc");

    /** @namespace System.Runtime.Serialization */

    /**
     * When applied to the member of a type, specifies that the member is part of
     a data contract and is serializable by the DataContractSerializer.
     *
     * @public
     * @class System.Runtime.Serialization.CollectionDataContract2Attribute
     * @augments System.Attribute
     */
    Bridge.define("System.Runtime.Serialization.CollectionDataContract2Attribute", {
        inherits: [System.Attribute],
        fields: {
            /**
             * Gets or sets a value that indicates whether to preserve object reference data.
             *
             * @instance
             * @public
             * @memberof System.Runtime.Serialization.CollectionDataContract2Attribute
             * @function IsReference
             * @type boolean
             */
            IsReference: false,
            /**
             * Gets or sets a custom name for a collection element.
             *
             * @instance
             * @public
             * @memberof System.Runtime.Serialization.CollectionDataContract2Attribute
             * @function ItemName
             * @type string
             */
            ItemName: null,
            /**
             * Gets or sets the custom name for a dictionary key name.
             *
             * @instance
             * @public
             * @memberof System.Runtime.Serialization.CollectionDataContract2Attribute
             * @function KeyName
             * @type string
             */
            KeyName: null,
            /**
             * Gets or sets the data contract name for the collection type.
             *
             * @instance
             * @public
             * @memberof System.Runtime.Serialization.CollectionDataContract2Attribute
             * @function Name
             * @type string
             */
            Name: null,
            /**
             * Gets or sets the namespace for the data contract.
             *
             * @instance
             * @public
             * @memberof System.Runtime.Serialization.CollectionDataContract2Attribute
             * @function Namespace
             * @type string
             */
            Namespace: null,
            /**
             * Gets or sets the custom name for a dictionary value name.
             *
             * @instance
             * @public
             * @memberof System.Runtime.Serialization.CollectionDataContract2Attribute
             * @function ValueName
             * @type string
             */
            ValueName: null
        },
        ctors: {
            ctor: function () {
                this.$initialize();
                System.Attribute.ctor.call(this);
            },
            $ctor1: function (IsReference, ItemName, KeyName, Name, Namespace, ValueName) {
                if (IsReference === void 0) { IsReference = false; }
                if (ItemName === void 0) { ItemName = null; }
                if (KeyName === void 0) { KeyName = null; }
                if (Name === void 0) { Name = null; }
                if (Namespace === void 0) { Namespace = null; }
                if (ValueName === void 0) { ValueName = null; }

                this.$initialize();
                System.Attribute.ctor.call(this);
                this.IsReference = IsReference;
                this.ItemName = ItemName;
                this.KeyName = KeyName;
                this.Name = Name;
                this.Namespace = Namespace;
                this.ValueName = ValueName;
            }
        }
    });

    /**
     * Serializes and deserializes an instance of a type into an XML stream or document
     using a supplied data contract. This class cannot be inherited.
     *
     * @public
     * @class System.Runtime.Serialization.DataContractSerializer
     */
    Bridge.define("System.Runtime.Serialization.DataContractSerializer", {
        fields: {
            _type: null,
            _useXmlSerializerFormat: false,
            _knownTypes: null
        },
        props: {
            /**
             * Gets a collection of types that may be present in the object graph serialized
             using this instance of the System.Runtime.Serialization.DataContractSerializer.
             *
             * @instance
             * @public
             * @readonly
             * @memberof System.Runtime.Serialization.DataContractSerializer
             * @function KnownTypes
             * @type System.Collections.Generic.IReadOnlyList$1
             */
            KnownTypes: {
                get: function () {
                    return this._knownTypes;
                }
            }
        },
        ctors: {
            init: function () {
                this._knownTypes = new (System.Collections.Generic.List$1(System.Type)).ctor();
            },
            /**
             * Initializes a new instance of the System.Runtime.Serialization.DataContractSerializer
             class to serialize or deserialize an object of the specified type.
             *
             * @instance
             * @public
             * @this System.Runtime.Serialization.DataContractSerializer
             * @memberof System.Runtime.Serialization.DataContractSerializer
             * @param   {System.Type}    type                      The type of the instances that are serialized or deserialized.
             * @param   {boolean}        useXmlSerializerFormat
             * @return  {void}
             */
            ctor: function (type, useXmlSerializerFormat) {
                if (useXmlSerializerFormat === void 0) { useXmlSerializerFormat = false; }

                this.$initialize();
                System.Runtime.Serialization.DataContractSerializer_ValueTypesHandler.EnsureInitialized();

                this._type = type;
                this._useXmlSerializerFormat = useXmlSerializerFormat;
            },
            /**
             * Initializes a new instance of the System.Runtime.Serialization.DataContractSerializer
             class to serialize or deserialize an object of the specified type, and a
             collection of known types that may be present in the object graph.
             *
             * @instance
             * @public
             * @this System.Runtime.Serialization.DataContractSerializer
             * @memberof System.Runtime.Serialization.DataContractSerializer
             * @param   {System.Type}                                 type                      The type of the instances that are serialized or deserialized.
             * @param   {System.Collections.Generic.IEnumerable$1}    knownTypes                An System.Collections.Generic.IEnumerable`1 of System.Type that contains
             the types that may be present in the object graph.
             * @param   {boolean}                                     useXmlSerializerFormat
             * @return  {void}
             */
            $ctor1: function (type, knownTypes, useXmlSerializerFormat) {
                if (useXmlSerializerFormat === void 0) { useXmlSerializerFormat = false; }

                this.$initialize();
                System.Runtime.Serialization.DataContractSerializer_ValueTypesHandler.EnsureInitialized();

                this._type = type;
                this._useXmlSerializerFormat = useXmlSerializerFormat;

                if (knownTypes != null) {
                    Bridge.cast(this._knownTypes, System.Collections.Generic.List$1(System.Type)).AddRange(knownTypes);
                }
                if (CSHTML5.Internal.KnownTypesHelper._additionalKnownTypes != null) {
                    Bridge.cast(this._knownTypes, System.Collections.Generic.List$1(System.Type)).AddRange(CSHTML5.Internal.KnownTypesHelper._additionalKnownTypes);
                }
            }
        },
        methods: {
            SerializeToString: function (obj, indentXml, omitXmlDeclaration) {
                if (indentXml === void 0) { indentXml = false; }
                if (omitXmlDeclaration === void 0) { omitXmlDeclaration = false; }
                var xdoc = new System.Xml.Linq.XDocument();

                var defaultNamespace = System.Runtime.Serialization.DataContractSerializer_Helpers.GetDefaultNamespace(Bridge.Reflection.getTypeNamespace(this._type), this._useXmlSerializerFormat);

                var objectType = Bridge.getType(obj);
                if (Bridge.is(obj, System.Char)) {
                    objectType = System.Char;
                }

                var typeInformation = null;
                if (!Bridge.referenceEquals(objectType, this._type)) {
                    typeInformation = System.Runtime.Serialization.DataContractSerializer_Helpers.GetTypeInformationByReadingAttributes(this._type, defaultNamespace, this._useXmlSerializerFormat);
                }

                var xnodesForRoot = System.Runtime.Serialization.DataContractSerializer_Serialization.SerializeToXObjects(obj, this._type, this._knownTypes, this._useXmlSerializerFormat, true, false, typeInformation, null);
                xdoc.Add(System.Linq.Enumerable.from(xnodesForRoot).first());
                var xml = xdoc.ToString(indentXml);
                if (!omitXmlDeclaration) {
                    xml = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>" + ("\n" || "") + (xml || "");
                }

                return xml;
            },
            SerializeToXDocument: function (obj) {
                var xdoc = new System.Xml.Linq.XDocument();

                var defaultNamespace = System.Runtime.Serialization.DataContractSerializer_Helpers.GetDefaultNamespace(Bridge.Reflection.getTypeNamespace(this._type), this._useXmlSerializerFormat);

                var objectType = Bridge.getType(obj);
                if (Bridge.is(obj, System.Char)) {
                    objectType = System.Char;
                }

                var typeInformation = null;
                if (!Bridge.referenceEquals(objectType, this._type)) {
                    typeInformation = System.Runtime.Serialization.DataContractSerializer_Helpers.GetTypeInformationByReadingAttributes(this._type, defaultNamespace, this._useXmlSerializerFormat);
                }

                var xnodesForRoot = System.Runtime.Serialization.DataContractSerializer_Serialization.SerializeToXObjects(obj, this._type, this._knownTypes, this._useXmlSerializerFormat, true, false, typeInformation, null);
                xdoc.Add(System.Linq.Enumerable.from(xnodesForRoot).first());
                return xdoc;
            },
            DeserializeFromString: function (xml) {
                var xdoc = System.Xml.Linq.XDocument.Parse(xml);
                var root = xdoc.Root;
                var expectedType = this._type;
                var actuallyExpectedType = System.Runtime.Serialization.DataContractSerializer_KnownTypes.GetCSharpTypeForNode(root, this._type, expectedType, this._knownTypes, null, this._useXmlSerializerFormat);
                var result = System.Runtime.Serialization.DataContractSerializer_Deserialization.DeserializeToCSharpObject(root.Nodes(), actuallyExpectedType, root, this._knownTypes, false, this._useXmlSerializerFormat);
                return result;
            },
            DeserializeFromXElement: function (xElement) {
                var expectedType = this._type;
                var actuallyExpectedType = System.Runtime.Serialization.DataContractSerializer_KnownTypes.GetCSharpTypeForNode(xElement, this._type, expectedType, this._knownTypes, null, this._useXmlSerializerFormat);
                var result = System.Runtime.Serialization.DataContractSerializer_Deserialization.DeserializeToCSharpObject(xElement.Nodes(), actuallyExpectedType, xElement, this._knownTypes, false, this._useXmlSerializerFormat);
                return result;
            }
        }
    });

    Bridge.define("System.Runtime.Serialization.DataContractSerializer_Deserialization", {
        statics: {
            fields: {
                TypeToOnDeserializingMethodCache: null,
                TypeToOnDeserializedMethodCache: null
            },
            ctors: {
                init: function () {
                    this.TypeToOnDeserializingMethodCache = new (System.Collections.Generic.Dictionary$2(System.Type,System.Reflection.MethodInfo)).ctor();
                    this.TypeToOnDeserializedMethodCache = new (System.Collections.Generic.Dictionary$2(System.Type,System.Reflection.MethodInfo)).ctor();
                }
            },
            methods: {
                DeserializeToCSharpObject: function (content, resultType, parentElement, knownTypes, ignoreErrors, useXmlSerializerFormat) {
                    var $t;
                    if (Bridge.Reflection.isGenericType(resultType) && Bridge.referenceEquals(Bridge.Reflection.getGenericTypeDefinition(resultType), System.Nullable$1)) {
                        resultType = ($t = Bridge.Reflection.getGenericArguments(resultType))[System.Array.index(0, $t)];
                    }

                    var itemsType = { };

                    if (Bridge.Reflection.isEnum(resultType) || System.Runtime.Serialization.DataContractSerializer_ValueTypesHandler.TypesToNames.containsKey(resultType)) {

                        return System.Runtime.Serialization.DataContractSerializer_Deserialization.DeserializeToCSharpObject_ValueTypeOrString(content, resultType);

                    } else if (Bridge.isArray(null, resultType) && Bridge.referenceEquals((resultType.$elementType || null), System.Byte)) {

                        return System.Runtime.Serialization.DataContractSerializer_Deserialization.DeserializeToCSharpObject_ByteArray(content);
                    } else if (System.Runtime.Serialization.DataContractSerializer_Helpers.IsAssignableToGenericEnumerableOrArray(resultType, itemsType)) {

                        return System.Runtime.Serialization.DataContractSerializer_Deserialization.DeserializeToCSharpObject_Enumerable_WithRecursion(content, resultType, knownTypes, ignoreErrors, itemsType.v, useXmlSerializerFormat);

                    } else {

                        return System.Runtime.Serialization.DataContractSerializer_Deserialization.DeserializeToCSharpObject_Object_WithRecursion(content, resultType, parentElement, knownTypes, ignoreErrors, useXmlSerializerFormat);
                    }

                },
                DeserializeToCSharpObject_ValueTypeOrString: function (content, resultType) {
                    var $t;
                    if (System.Linq.Enumerable.from(content).any()) {
                        $t = Bridge.getEnumerator(content, System.Xml.Linq.XNode);
                        try {
                            while ($t.moveNext()) {
                                var node = $t.Current;
                                if (Bridge.is(node, System.Xml.Linq.XText)) {
                                    var text = Bridge.cast(node, System.Xml.Linq.XText);
                                    var str = text.Value;

                                    return System.Runtime.Serialization.DataContractSerializer_ValueTypesHandler.ConvertStringToValueType(str, resultType);
                                }
                            }
                        } finally {
                            if (Bridge.is($t, System.IDisposable)) {
                                $t.System$IDisposable$Dispose();
                            }
                        }
                    } else {
                        if (Bridge.referenceEquals(resultType, System.String)) {
                            return "";
                        }
                    }
                    throw new System.Exception("Unable to create the type '" + (Bridge.getTypeName(resultType) || "") + "'.");
                },
                DeserializeToCSharpObject_ByteArray: function (content) {
                    var $t;
                    $t = Bridge.getEnumerator(content, System.Xml.Linq.XNode);
                    try {
                        while ($t.moveNext()) {
                            var node = $t.Current;
                            if (Bridge.is(node, System.Xml.Linq.XText)) {
                                return System.Convert.fromBase64String(Bridge.cast(node, System.Xml.Linq.XText).Value);
                            }
                        }
                    } finally {
                        if (Bridge.is($t, System.IDisposable)) {
                            $t.System$IDisposable$Dispose();
                        }
                    }
                    return System.Array.init([], System.Byte);
                },
                DeserializeToCSharpObject_Enumerable_WithRecursion: function (content, resultType, knownTypes, ignoreErrors, itemsType, useXmlSerializerFormat) {
                    var $t, $t1, $t2, $t3, $t4, $t5;



                    var arg = System.Array.init([itemsType], System.Type);
                    var list;

                    if (CSHTML5.Interop.IsRunningInTheSimulator) {
                        list = DotNetForHtml5.Core.INTERNAL_Simulator.SimulatorProxy.MakeInstanceOfGenericType(System.Collections.Generic.List$1, arg);
                    } else {
                        list = Bridge.createInstance(System.Collections.Generic.List$1.apply(null, arg));
                    }


                    var listAddMethod = Bridge.Reflection.getMembers(Bridge.getType(list), 8, 284, "Add");

                    var count = 0;

                    $t = Bridge.getEnumerator(content, System.Xml.Linq.XNode);
                    try {
                        while ($t.moveNext()) {
                            var node = $t.Current;
                            if (Bridge.is(node, System.Xml.Linq.XElement)) {
                                var xElement = Bridge.cast(node, System.Xml.Linq.XElement);
                                var childObject;
                                if (System.Runtime.Serialization.DataContractSerializer_Helpers.IsElementNil(xElement)) {
                                    if (Bridge.Reflection.isValueType(itemsType) && !System.String.startsWith(Bridge.Reflection.getTypeFullName(itemsType), "System.Nullable`1")) {
                                        childObject = Bridge.createInstance(itemsType);
                                    } else {
                                        childObject = null;
                                    }
                                } else {
                                    var elementChildNodes = xElement.Nodes();

                                    var childObjectActualType = System.Runtime.Serialization.DataContractSerializer_KnownTypes.GetCSharpTypeForNode(xElement, itemsType, itemsType, knownTypes, null, useXmlSerializerFormat);


                                    childObject = System.Runtime.Serialization.DataContractSerializer_Deserialization.DeserializeToCSharpObject(elementChildNodes, childObjectActualType, xElement, knownTypes, ignoreErrors, useXmlSerializerFormat);
                                }

                                Bridge.Reflection.midel(listAddMethod, Bridge.unbox(list)).apply(null, Bridge.unbox(System.Array.init([childObject], System.Object)));
                            }
                            count = (count + 1) | 0;
                        }
                    } finally {
                        if (Bridge.is($t, System.IDisposable)) {
                            $t.System$IDisposable$Dispose();
                        }
                    }


                    var genericTypeDefinition;
                    if (Bridge.isArray(null, resultType) || Bridge.referenceEquals(resultType, System.Collections.IEnumerable)) {

                        var result = System.Array.init(count, Bridge.getDefaultValue(itemsType), itemsType);

                        System.Array.copyTo(Bridge.cast(list, System.Collections.IList), result, 0);

                        return result;
                    } else if (Bridge.Reflection.isGenericType(resultType) && ((genericTypeDefinition = Bridge.Reflection.getGenericTypeDefinition(resultType))) != null && (Bridge.referenceEquals(genericTypeDefinition, System.Collections.Generic.List$1) || Bridge.referenceEquals(genericTypeDefinition, System.Collections.Generic.IList$1) || Bridge.referenceEquals(genericTypeDefinition, System.Collections.Generic.IEnumerable$1) || Bridge.referenceEquals(genericTypeDefinition, System.Collections.Generic.ICollection$1))) {

                        return list;
                    } else {

                        try {
                            var result1 = Bridge.Reflection.applyConstructor(resultType, System.Array.init([list], System.Object));


                            var isOk = false;
                            if (count === 0) {
                                isOk = true;
                            } else {
                                if (Bridge.isArray(null, resultType)) {
                                    if (Bridge.cast(result1, Array).length === count) {
                                        isOk = true;
                                    }
                                } else {
                                    $t1 = Bridge.getEnumerator(Bridge.cast(result1, System.Collections.IEnumerable));
                                    try {
                                        while ($t1.moveNext()) {
                                            var item = $t1.Current;
                                            isOk = true;
                                            break;
                                        }
                                    } finally {
                                        if (Bridge.is($t1, System.IDisposable)) {
                                            $t1.System$IDisposable$Dispose();
                                        }
                                    }
                                }
                            }
                            if (isOk) {
                                return result1;
                            }
                        } catch ($e1) {
                            $e1 = System.Exception.create($e1);
                        }


                        var addMethod = Bridge.Reflection.getMembers(resultType, 8, 284, "Add", System.Array.init([itemsType], System.Type));
                        var isDictionary = false;
                        if (addMethod == null) {


                            var currentType = resultType;
                            while (currentType != null && !(System.String.startsWith(Bridge.Reflection.getTypeFullName(currentType), "System.Collections.Generic.Dictionary"))) {
                                currentType = Bridge.Reflection.getBaseType(currentType);
                            }
                            if (currentType != null) {
                                currentType = itemsType;
                                while (currentType != null && !(System.String.startsWith(Bridge.Reflection.getTypeFullName(currentType), "System.Collections.Generic.KeyValuePair"))) {
                                    currentType = Bridge.Reflection.getBaseType(currentType);
                                }

                                if (currentType != null) {
                                    isDictionary = true;
                                    var keyType = ($t2 = Bridge.Reflection.getGenericArguments(itemsType))[System.Array.index(0, $t2)];
                                    var valueType = ($t3 = Bridge.Reflection.getGenericArguments(itemsType))[System.Array.index(1, $t3)];
                                    addMethod = Bridge.Reflection.getMembers(resultType, 8, 284, "Add", System.Array.init([keyType, valueType], System.Type));
                                }
                            }
                            if (!isDictionary) {
                                if (!ignoreErrors) {
                                    throw new System.Runtime.Serialization.InvalidDataContractException.$ctor1(System.String.format("Type '{0}' is an invalid collection type since it does not have a valid Add method with parameter of type '{1}'.", Bridge.getTypeName(resultType), Bridge.getTypeName(itemsType)));
                                } else {
                                    return null;
                                }
                            }
                        }

                        var result2 = null;
                        try {
                            result2 = Bridge.createInstance(resultType);
                        } catch ($e2) {
                            $e2 = System.Exception.create($e2);
                        }
                        if (result2 == null) {
                            if (!ignoreErrors) {
                                throw new System.Runtime.Serialization.SerializationException.$ctor1(System.String.format("Cannot create a new instance of the type '{0}'. Please verify that the type has a public parameterless constructor.", [Bridge.getTypeName(resultType)]));
                            } else {
                                return null;
                            }
                        }

                        if (!isDictionary) {
                            $t4 = Bridge.getEnumerator(Bridge.cast(list, System.Collections.IList));
                            try {
                                while ($t4.moveNext()) {
                                    var item1 = $t4.Current;
                                    Bridge.Reflection.midel(addMethod, Bridge.unbox(result2)).apply(null, Bridge.unbox(System.Array.init([item1], System.Object)));
                                }
                            } finally {
                                if (Bridge.is($t4, System.IDisposable)) {
                                    $t4.System$IDisposable$Dispose();
                                }
                            }
                        } else {
                            $t5 = Bridge.getEnumerator(Bridge.cast(list, System.Collections.IList));
                            try {
                                while ($t5.moveNext()) {
                                    var item2 = $t5.Current;
                                    var dynamicItem = item2;
                                    if (CSHTML5.Interop.IsRunningInTheSimulator) {
                                        Bridge.Reflection.midel(addMethod, Bridge.unbox(result2)).apply(null, Bridge.unbox(System.Array.init([dynamicItem.Key, dynamicItem.Value], System.Object)));
                                    } else {
                                        Bridge.Reflection.midel(addMethod, Bridge.unbox(result2)).apply(null, Bridge.unbox(System.Array.init([dynamicItem.key, dynamicItem.value], System.Object)));
                                    }
                                }
                            } finally {
                                if (Bridge.is($t5, System.IDisposable)) {
                                    $t5.System$IDisposable$Dispose();
                                }
                            }
                        }

                        return result2;
                    }
                },
                DeserializeToCSharpObject_Enumerable_WithRecursion_SpecialCase: function (name, content, resultType, knownTypes, ignoreErrors, itemsType, useXmlSerializerFormat) {
                    var $t;
                    var nodesThatCorrespondToTheEnumerableContent = new (System.Collections.Generic.List$1(System.Xml.Linq.XNode)).ctor();

                    $t = Bridge.getEnumerator(content, System.Xml.Linq.XNode);
                    try {
                        while ($t.moveNext()) {
                            var node = $t.Current;
                            if (Bridge.is(node, System.Xml.Linq.XElement)) {
                                var xElement = Bridge.cast(node, System.Xml.Linq.XElement);
                                var elementName = xElement.Name;
                                var elementNameWithoutNamespace = elementName.LocalName;

                                if (Bridge.referenceEquals(elementNameWithoutNamespace, name)) {
                                    nodesThatCorrespondToTheEnumerableContent.add(node);
                                }
                            }
                        }
                    } finally {
                        if (Bridge.is($t, System.IDisposable)) {
                            $t.System$IDisposable$Dispose();
                        }
                    }

                    var result = System.Runtime.Serialization.DataContractSerializer_Deserialization.DeserializeToCSharpObject_Enumerable_WithRecursion(nodesThatCorrespondToTheEnumerableContent, resultType, knownTypes, ignoreErrors, itemsType, useXmlSerializerFormat);

                    return result;
                },
                DeserializeToCSharpObject_Object_WithRecursion: function (content, resultType, parentElement, knownTypes, ignoreErrors, useXmlSerializerFormat) {
                    var $t, $t1, $t2, $t3, $t4;
                    if (!Bridge.Reflection.isInterface(resultType)) {
                        var resultTypeFullName = Bridge.Reflection.getTypeFullName(resultType);

                        var resultInstance = Bridge.createInstance(resultType);

                        System.Runtime.Serialization.DataContractSerializer_Deserialization.CallOnDeserializingMethod(resultInstance, resultType);

                        var typeInformation = System.Runtime.Serialization.DataContractSerializer_Helpers.GetTypeInformationByReadingAttributes(resultType, null, useXmlSerializerFormat);

                        var membersInformation = System.Runtime.Serialization.DataContractSerializer_Helpers.GetDataContractMembers(resultType, typeInformation.serializationType, useXmlSerializerFormat);

                        var memberNameToMemberInformation = new (System.Collections.Generic.Dictionary$2(System.String,System.Runtime.Serialization.MemberInformation)).ctor();
                        $t = Bridge.getEnumerator(membersInformation, System.Runtime.Serialization.MemberInformation);
                        try {
                            while ($t.moveNext()) {
                                var memberInformation = $t.Current;
                                var memberName = memberInformation.Name;
                                if (System.String.startsWith(Bridge.Reflection.getTypeFullName(resultType), "System.Collections.Generic.KeyValuePair")) {
                                    if (Bridge.referenceEquals(memberName, "key")) {
                                        memberName = "Key";
                                    } else if (Bridge.referenceEquals(memberName, "value")) {
                                        memberName = "Value";
                                    }
                                }
                                if (!memberNameToMemberInformation.containsKey(memberName)) {
                                    memberNameToMemberInformation.add(memberName, memberInformation);
                                } else {
                                    var collidingMemberInformation = memberNameToMemberInformation.getItem(memberName);
                                    throw new System.Runtime.Serialization.InvalidDataContractException.$ctor1(System.String.format("Type '{0}' contains two members '{1}' 'and '{2}' with the same data member name '{3}'. Multiple members with the same name in one type are not supported. Consider changing one of the member names using DataMemberAttribute attribute.", Bridge.getTypeName(resultType), memberInformation.MemberInfo.n, collidingMemberInformation.MemberInfo.n, memberName));
                                }
                            }
                        } finally {
                            if (Bridge.is($t, System.IDisposable)) {
                                $t.System$IDisposable$Dispose();
                            }
                        }

                        var membersForWhichWeSuccessfullSetTheValue = new (System.Collections.Generic.HashSet2$1(System.String)).ctor();
                        $t1 = Bridge.getEnumerator(content, System.Xml.Linq.XNode);
                        try {
                            while ($t1.moveNext()) {
                                var node = $t1.Current;
                                if (Bridge.is(node, System.Xml.Linq.XElement)) {
                                    var xElement = Bridge.cast(node, System.Xml.Linq.XElement);
                                    var elementName = xElement.Name;
                                    var elementNameWithoutNamespace = elementName.LocalName;

                                    var memberInformation1 = { };
                                    if (memberNameToMemberInformation.tryGetValue(elementNameWithoutNamespace, memberInformation1)) {
                                        if (!membersForWhichWeSuccessfullSetTheValue.contains(memberInformation1.v.Name)) {
                                            var memberValue = null;
                                            var itemsType = { v : null };
                                            var memberActualType = memberInformation1.v.MemberType;
                                            if (System.Runtime.Serialization.DataContractSerializer_Helpers.IsElementNil(xElement)) {

                                                memberValue = System.Runtime.Serialization.DataContractSerializer_Helpers.GetDefault(memberInformation1.v.MemberType);
                                            } else if (useXmlSerializerFormat && memberInformation1.v.HasXmlElementAttribute && System.Runtime.Serialization.DataContractSerializer_Helpers.IsAssignableToGenericEnumerableOrArray(memberActualType, itemsType)) {

                                                var deserializedEnumerable = System.Runtime.Serialization.DataContractSerializer_Deserialization.DeserializeToCSharpObject_Enumerable_WithRecursion_SpecialCase(memberInformation1.v.Name, content, memberActualType, knownTypes, ignoreErrors, itemsType.v, useXmlSerializerFormat);
                                                memberValue = deserializedEnumerable;
                                            } else {

                                                memberActualType = System.Runtime.Serialization.DataContractSerializer_KnownTypes.GetCSharpTypeForNode(xElement, memberInformation1.v.MemberInfo.td, memberActualType, knownTypes, memberInformation1.v, useXmlSerializerFormat);

                                                var nonNullableMemberType = memberActualType;
                                                if (System.String.startsWith(Bridge.Reflection.getTypeFullName(memberActualType), "System.Nullable`1")) {
                                                    nonNullableMemberType = System.Nullable.getUnderlyingType(memberActualType);
                                                }

                                                var propertyChildNodes = xElement.Nodes();

                                                memberValue = System.Runtime.Serialization.DataContractSerializer_Deserialization.DeserializeToCSharpObject(propertyChildNodes, nonNullableMemberType, xElement, knownTypes, ignoreErrors, useXmlSerializerFormat);
                                            }


                                            System.Runtime.Serialization.DataContractSerializer_Helpers.SetMemberValue(resultInstance, memberInformation1.v, memberValue);
                                            membersForWhichWeSuccessfullSetTheValue.add(memberInformation1.v.Name);
                                        }
                                    } else {
                                    }
                                }
                            }
                        } finally {
                            if (Bridge.is($t1, System.IDisposable)) {
                                $t1.System$IDisposable$Dispose();
                            }
                        }

                        if (useXmlSerializerFormat) {
                            $t2 = Bridge.getEnumerator(parentElement.Attributes(), System.Xml.Linq.XAttribute);
                            try {
                                while ($t2.moveNext()) {
                                    var attribute = $t2.Current;
                                    var attributeName = attribute.Name;
                                    if (System.String.isNullOrEmpty(attributeName.NamespaceName)) {
                                        var attributeNameWithoutNamespace = attributeName.LocalName;

                                        var memberInformation2 = { };
                                        if (memberNameToMemberInformation.tryGetValue(attributeNameWithoutNamespace, memberInformation2) && memberInformation2.v.HasXmlAttributeAttribute) {
                                            if (!membersForWhichWeSuccessfullSetTheValue.contains(memberInformation2.v.Name)) {
                                                var attributeValue = attribute.Value;

                                                if (System.Runtime.Serialization.DataContractSerializer_ValueTypesHandler.TypesToNames.containsKey(memberInformation2.v.MemberType)) {
                                                    var memberValue1 = System.Runtime.Serialization.DataContractSerializer_ValueTypesHandler.ConvertStringToValueType(attributeValue, memberInformation2.v.MemberType);

                                                    System.Runtime.Serialization.DataContractSerializer_Helpers.SetMemberValue(resultInstance, memberInformation2.v, memberValue1);
                                                    membersForWhichWeSuccessfullSetTheValue.add(memberInformation2.v.Name);
                                                } else {
                                                    if (Bridge.referenceEquals(memberInformation2.v.MemberType, System.Collections.Generic.List$1(System.Int32))) {
                                                        var splittedElements = System.String.split(attributeValue, [32].map(function (i) {{ return String.fromCharCode(i); }}));

                                                        var listint = new (System.Collections.Generic.List$1(System.Int32)).ctor();

                                                        $t3 = Bridge.getEnumerator(splittedElements);
                                                        try {
                                                            while ($t3.moveNext()) {
                                                                var str = $t3.Current;
                                                                listint.add(System.Int32.parse(str));
                                                            }
                                                        } finally {
                                                            if (Bridge.is($t3, System.IDisposable)) {
                                                                $t3.System$IDisposable$Dispose();
                                                            }
                                                        }


                                                        System.Runtime.Serialization.DataContractSerializer_Helpers.SetMemberValue(resultInstance, memberInformation2.v, listint);
                                                        membersForWhichWeSuccessfullSetTheValue.add(memberInformation2.v.Name);
                                                    }
                                                }
                                            } else {
                                            }
                                        } else {
                                        }
                                    }
                                }
                            } finally {
                                if (Bridge.is($t2, System.IDisposable)) {
                                    $t2.System$IDisposable$Dispose();
                                }
                            }
                        }

                        $t4 = Bridge.getEnumerator(membersInformation, System.Runtime.Serialization.MemberInformation);
                        try {
                            while ($t4.moveNext()) {
                                var memberInformation3 = $t4.Current;
                                if (memberInformation3.IsRequired && !membersForWhichWeSuccessfullSetTheValue.contains(memberInformation3.Name)) {
                                    throw new System.Runtime.Serialization.SerializationException.$ctor1(System.String.format("The member '{0}' is required but it was not found in the document being deserialized.", [memberInformation3.Name]));
                                }
                            }
                        } finally {
                            if (Bridge.is($t4, System.IDisposable)) {
                                $t4.System$IDisposable$Dispose();
                            }
                        }

                        System.Runtime.Serialization.DataContractSerializer_Deserialization.CallOnDeserializedMethod(resultInstance, resultType);

                        return resultInstance;
                    } else {
                        return null;
                    }
                },
                CallOnDeserializingMethod: function (obj, objType) {
                    System.Runtime.Serialization.DataContractSerializer_Helpers.CallMethod(obj, objType, System.Runtime.Serialization.OnDeserializingAttribute, System.Runtime.Serialization.DataContractSerializer_Deserialization.TypeToOnDeserializingMethodCache);
                },
                CallOnDeserializedMethod: function (obj, objType) {
                    System.Runtime.Serialization.DataContractSerializer_Helpers.CallMethod(obj, objType, System.Runtime.Serialization.OnDeserializedAttribute, System.Runtime.Serialization.DataContractSerializer_Deserialization.TypeToOnDeserializedMethodCache);
                },
                CreateNewInstanceOfGenericList: function (T) {

                    return new (System.Collections.Generic.List$1(T)).ctor();
                }
            }
        }
    });

    Bridge.define("System.Runtime.Serialization.DataContractSerializer_Helpers", {
        statics: {
            fields: {
                DATACONTRACTSERIALIZER_OBJECT_DEFAULT_NAMESPACE: null,
                XMLSCHEMA_NAMESPACE: null,
                XMLSCHEMA_NAMESPACE_XSD: null,
                XMLSCHEMA_NAMESPACE_TYPES: null,
                XMLNS_NAMESPACE: null,
                ATTRIBUTE_TO_REMOVE_FROM_XMLSERIALIZATION_NAME: null,
                ATTRIBUTE_TO_REMOVE_FROM_XMLSERIALIZATION_VALUE: null,
                _typesWithOmittedSerializableAttribute: null
            },
            props: {
                INTERNAL_TypesWithOmittedSerializableAttribute: {
                    get: function () {
                        if (System.Runtime.Serialization.DataContractSerializer_Helpers._typesWithOmittedSerializableAttribute == null) {
                            System.Runtime.Serialization.DataContractSerializer_Helpers._typesWithOmittedSerializableAttribute = function (_o1) {
                                    _o1.add(System.Collections.Generic.KeyValuePair$2);
                                    _o1.add(System.Tuple$1);
                                    _o1.add(System.Tuple$2);
                                    _o1.add(System.Tuple$3);
                                    _o1.add(System.Tuple$4);
                                    _o1.add(System.Tuple$5);
                                    _o1.add(System.Tuple$7);
                                    return _o1;
                                }(new (System.Collections.Generic.HashSet2$1(System.Type)).ctor());
                        }
                        return System.Runtime.Serialization.DataContractSerializer_Helpers._typesWithOmittedSerializableAttribute;
                    }
                }
            },
            ctors: {
                init: function () {
                    this.DATACONTRACTSERIALIZER_OBJECT_DEFAULT_NAMESPACE = "http://schemas.datacontract.org/2004/07/";
                    this.XMLSCHEMA_NAMESPACE = "http://www.w3.org/2001/XMLSchema-instance";
                    this.XMLSCHEMA_NAMESPACE_XSD = "http://www.w3.org/2001/XMLSchema";
                    this.XMLSCHEMA_NAMESPACE_TYPES = "http://microsoft.com/wsdl/types/";
                    this.XMLNS_NAMESPACE = "http://www.w3.org/2000/xmlns/";
                    this.ATTRIBUTE_TO_REMOVE_FROM_XMLSERIALIZATION_NAME = "AttributeToRemoveFromXMLSerialization";
                    this.ATTRIBUTE_TO_REMOVE_FROM_XMLSERIALIZATION_VALUE = "Remove this from the XMLSerialization";
                }
            },
            methods: {
                GetDefaultNamespace: function (typeNamespace, useXmlSerializerFormat) {
                    if (useXmlSerializerFormat) {
                        return null;
                    } else {
                        return (System.Runtime.Serialization.DataContractSerializer_Helpers.DATACONTRACTSERIALIZER_OBJECT_DEFAULT_NAMESPACE || "") + (typeNamespace || "");
                    }
                },
                IsAssignableToGenericEnumerableOrArray: function (type, itemsType) {
                    if (Bridge.isArray(null, type)) {
                        itemsType.v = (type.$elementType || null);
                        if (System.Runtime.Serialization.DataContractSerializer_Helpers.IsNullOrUndefined(itemsType.v)) {
                            itemsType.v = System.Object;
                        }
                        return true;
                    } else if (Bridge.referenceEquals(type, System.String)) {
                        itemsType.v = null;
                        return false;
                    } else {
                        var genericEnumerable = null;

                        if (Bridge.Reflection.isGenericType(type) && Bridge.referenceEquals(Bridge.Reflection.getGenericTypeDefinition(type), System.Collections.Generic.IEnumerable$1)) {
                            genericEnumerable = type;
                        } else {
                            if (CSHTML5.Interop.IsRunningInTheSimulator) {
                                genericEnumerable = DotNetForHtml5.Core.INTERNAL_Simulator.SimulatorProxy.GetInterface(Bridge.unbox(type), "IEnumerable`1");
                            } else {
                                genericEnumerable = System.Runtime.Serialization.DataContractSerializer_Helpers.GetInterface(type, Bridge.Reflection.getTypeName(System.Collections.Generic.IEnumerable$1));
                            }
                        }

                        if (genericEnumerable != null) {
                            var genericArguments = Bridge.Reflection.getGenericArguments(genericEnumerable);
                            if (genericArguments.length > 0) {
                                itemsType.v = genericArguments[System.Array.index(0, genericArguments)];
                                return true;
                            }
                        }
                    }

                    itemsType.v = null;
                    return false;
                },
                CheckIfObjectIsNullNullable: function (obj) {
                    var type = Bridge.getType(obj);
                    if (System.String.startsWith(Bridge.Reflection.getTypeFullName(type), "System.Nullable`1")) {
                        return !obj.hasValue;
                    } else {
                        return false;
                    }
                },
                /**
                 * If the given namespace already has an assigned prefix, returns that prefix,
                 otherwise, generates a unique, unused prefix, defines it in the element, then returns the prefix.
                 *
                 * @static
                 * @this System.Runtime.Serialization.DataContractSerializer_Helpers
                 * @memberof System.Runtime.Serialization.DataContractSerializer_Helpers
                 * @param   {System.Xml.Linq.XElement}    element          The XElement in which to define the prefix.
                 * @param   {string}                      namespaceName    The namespace name for which to define the prefix.
                 * @return  {string}
                 */
                GenerateUniqueNamespacePrefixIfNeeded: function (element, namespaceName) {
                    var prefix = element.GetPrefixOfNamespace(System.Xml.Linq.XNamespace.op_Implicit(namespaceName));

                    if (!System.String.isNullOrWhiteSpace(prefix)) {
                        return prefix;
                    } else {
                        var index = 0;
                        while (true) {
                            prefix = "Prefix" + (Bridge.toString(Bridge.getHashCode(namespaceName)) || "") + (Bridge.toString(index) || "");

                            var namespaceOfPrefix = element.GetNamespaceOfPrefix(prefix);

                            if (System.String.isNullOrWhiteSpace(namespaceOfPrefix.NamespaceName)) {
                                element.SetAttributeValue(System.Xml.Linq.XNamespace.Get(System.Runtime.Serialization.DataContractSerializer_Helpers.XMLNS_NAMESPACE).GetName(prefix), namespaceName);
                                if ((window.IE_VERSION)) {
                                    element.SetAttributeValue(System.Xml.Linq.XNamespace.Get(namespaceName).GetName((prefix || "") + ":" + (System.Runtime.Serialization.DataContractSerializer_Helpers.ATTRIBUTE_TO_REMOVE_FROM_XMLSERIALIZATION_NAME || "")), System.Runtime.Serialization.DataContractSerializer_Helpers.ATTRIBUTE_TO_REMOVE_FROM_XMLSERIALIZATION_VALUE);
                                }
                                return prefix;
                            }
                            index = (index + 1) | 0;
                        }
                    }
                },
                /**
                 * Produces a string from an input System.DateTime.
                 *
                 * @static
                 * @this System.Runtime.Serialization.DataContractSerializer_Helpers
                 * @memberof System.Runtime.Serialization.DataContractSerializer_Helpers
                 * @param   {System.DateTime}    value    A System.DateTime to translate to a string.
                 * @return  {string}                      A string representation of the System.DateTime that shows the date and time.
                 */
                ConvertDateTimeToSerializationString: function (value) {
                    /* 
                    DateTime dateTimeUtc = (value.Kind == DateTimeKind.Local ? value.ToUniversalTime() : value);
                    TimeSpan timeSince1970 = (dateTimeUtc - new DateTime(1970, 1, 1, 0, 0, 0, DateTimeKind.Utc));
                    double millisecondsSince1970 = timeSince1970.TotalMilliseconds;
                    var jsDate = JSIL.Verbatim.Expression("new Date($0)", millisecondsSince1970);
                    string serializedDate = JSIL.Verbatim.Expression("$0.toJSON()", jsDate);
                    return serializedDate;
                    */

                    var ts = new System.TimeSpan(0, 0, 0, 0, (System.DateTime.subdd(value, System.DateTime.getDate(value))).getMilliseconds());
                    var tsString = ts.getTicks().gte(System.Int64(1000000)) ? Bridge.toString(ts.getTicks()) : "0" + ts.getTicks();
                    if (System.String.endsWith(tsString, "0000")) {
                        tsString = tsString.substr(0, ((tsString.length - 4) | 0));
                    }
                    var secondString = "" + System.DateTime.getSecond(value);
                    if (secondString.length === 1) {
                        secondString = 0 + (secondString || "");
                    }
                    var minuteString = "" + System.DateTime.getMinute(value);
                    if (minuteString.length === 1) {
                        minuteString = 0 + (minuteString || "");
                    }
                    var hourString = "" + System.DateTime.getHour(value);
                    if (hourString.length === 1) {
                        hourString = 0 + (hourString || "");
                    }
                    var dayString = "" + System.DateTime.getDay(value);
                    if (dayString.length === 1) {
                        dayString = 0 + (dayString || "");
                    }
                    var monthString = "" + System.DateTime.getMonth(value);
                    if (monthString.length === 1) {
                        monthString = 0 + (monthString || "");
                    }
                    var yearString = "" + System.DateTime.getYear(value);
                    while (yearString.length < 4) {
                        yearString = 0 + (yearString || "");
                    }


                    var str = (yearString || "") + "-" + (monthString || "") + "-" + (dayString || "") + "T" + (hourString || "") + ":" + (minuteString || "") + ":" + (secondString || "");
                    if (ts.getTicks().ne(System.Int64(0))) {
                        str = (str || "") + (("." + (tsString || "")) || "");
                    }

                    if (System.DateTime.getKind(value) === 2) {
                        var changeFromUTC = System.DateTime.subdd(value, System.DateTime.toUniversalTime(value));
                        var minuteStringForTimeZone = "" + changeFromUTC.getMinutes();
                        if (minuteStringForTimeZone.length === 1) {
                            minuteStringForTimeZone = 0 + (minuteStringForTimeZone || "");
                        }
                        var hourStringForTimeZone = "" + Math.abs(changeFromUTC.getHours());
                        if (hourStringForTimeZone.length === 1) {
                            hourStringForTimeZone = 0 + (hourStringForTimeZone || "");
                        }
                        var timeZone = (hourStringForTimeZone || "") + ":" + (minuteStringForTimeZone || "");
                        if (changeFromUTC.getTicks().gte(System.Int64(0))) {
                            timeZone = "+" + (timeZone || "");
                        } else {
                            timeZone = "-" + (timeZone || "");
                        }
                        str = (str || "") + (timeZone || "");

                    } else if (System.DateTime.getKind(value) === 1) {
                        str = (str || "") + "Z";
                    }
                    return str;
                },
                GetDataContractMembers: function (type, serializationType, useXmlSerializerFormat) {
                    var $t, $t1;
                    var bindingFlags = 52;


                    var properties = Bridge.Reflection.getMembers(type, 16, bindingFlags);

                    var fields = Bridge.Reflection.getMembers(type, 4, bindingFlags);

                    var typesToMembersInformations = new (System.Collections.Generic.Dictionary$2(System.Type,System.Collections.Generic.List$1(System.Runtime.Serialization.MemberInformation))).ctor();
                    $t = Bridge.getEnumerator(properties);
                    try {
                        while ($t.moveNext()) {
                            var propertyInfo = $t.Current;
                            var memberInformation = System.Runtime.Serialization.DataContractSerializer_Helpers.GetDataMemberInformation(propertyInfo);
                            memberInformation.MemberType = propertyInfo.rt;
                            if (!System.Runtime.Serialization.DataContractSerializer_Helpers.IsNullOrUndefined(memberInformation.MemberType)) {
                                var memberInfo = memberInformation.MemberInfo;
                                var memberInfoBase = memberInformation.MemberInfoBase;
                                if (System.Runtime.Serialization.DataContractSerializer_Helpers.ShouldMemberBeConsideredForSerialization(memberInfo, serializationType, useXmlSerializerFormat)) {
                                    var propertyDeclaringType = memberInfoBase.td;
                                    if (!typesToMembersInformations.containsKey(propertyDeclaringType)) {
                                        typesToMembersInformations.add(propertyDeclaringType, new (System.Collections.Generic.List$1(System.Runtime.Serialization.MemberInformation)).ctor());
                                    }
                                    typesToMembersInformations.getItem(propertyDeclaringType).add(memberInformation);
                                }
                            } else {
                                System.Diagnostics.Debug.WriteLine$2("Cannot read 'PropertyType' from the property: " + (Bridge.getTypeName(propertyInfo.td) || "") + "." + (propertyInfo.n || ""));
                            }
                        }
                    } finally {
                        if (Bridge.is($t, System.IDisposable)) {
                            $t.System$IDisposable$Dispose();
                        }
                    }
                    $t1 = Bridge.getEnumerator(fields);
                    try {
                        while ($t1.moveNext()) {
                            var fieldInfo = $t1.Current;
                            var memberInformation1 = System.Runtime.Serialization.DataContractSerializer_Helpers.GetDataMemberInformation(fieldInfo);
                            memberInformation1.MemberType = fieldInfo.rt;
                            if (!System.Runtime.Serialization.DataContractSerializer_Helpers.IsNullOrUndefined(memberInformation1.MemberType)) {
                                var memberInfo1 = memberInformation1.MemberInfo;
                                if (System.Runtime.Serialization.DataContractSerializer_Helpers.ShouldMemberBeConsideredForSerialization(memberInfo1, serializationType, useXmlSerializerFormat)) {
                                    var fieldDeclaringType = fieldInfo.td;
                                    if (!typesToMembersInformations.containsKey(fieldDeclaringType)) {
                                        typesToMembersInformations.add(fieldDeclaringType, new (System.Collections.Generic.List$1(System.Runtime.Serialization.MemberInformation)).ctor());
                                    }
                                    typesToMembersInformations.getItem(fieldDeclaringType).add(memberInformation1);
                                }
                            } else {
                                System.Diagnostics.Debug.WriteLine$2("Cannot read 'FieldType' from the field: " + (Bridge.getTypeName(fieldInfo.td) || "") + "." + (fieldInfo.n || ""));
                            }
                        }
                    } finally {
                        if (Bridge.is($t1, System.IDisposable)) {
                            $t1.System$IDisposable$Dispose();
                        }
                    }

                    return System.Runtime.Serialization.DataContractSerializer_Helpers.SortMembers(typesToMembersInformations);
                },
                /**
                 * returns a List&lt;MemberInformation&gt; with the memberInformations ordered to fit the requirements of Microsoft's DataContractSerializer (see notes below DataMember2Attribute.Order).
                 *
                 * @static
                 * @private
                 * @this System.Runtime.Serialization.DataContractSerializer_Helpers
                 * @memberof System.Runtime.Serialization.DataContractSerializer_Helpers
                 * @param   {System.Collections.Generic.Dictionary$2}     membersUnitedByDeclaringType    A dictionary containing the memberInformations with their DeclaringType as Keys.
                 * @return  {System.Collections.Generic.IEnumerable$1}                                    A List&lt;MemberInformation&gt; with the memberInformations ordered to fit the requirements of Microsoft's DataContractSerializer.
                 */
                SortMembers: function (membersUnitedByDeclaringType) {
                    var $t;
                    var orderedTypes = System.Linq.Enumerable.from(membersUnitedByDeclaringType.Keys).toList(System.Type);
                    orderedTypes.Sort$2(function (t1, t2) {
                        return Bridge.Reflection.isAssignableFrom(t1, t2) ? -1 : 1;
                    });

                    var resultList = new (System.Collections.Generic.List$1(System.Runtime.Serialization.MemberInformation)).ctor();
                    $t = Bridge.getEnumerator(orderedTypes);
                    try {
                        while ($t.moveNext()) {
                            var currentType = $t.Current;
                            var sortedResultForType = System.Linq.Enumerable.from(membersUnitedByDeclaringType.getItem(currentType)).toList(System.Runtime.Serialization.MemberInformation);
                            sortedResultForType.Sort$2(System.Runtime.Serialization.DataContractSerializer_Helpers.CompareMembers);
                            resultList.AddRange(sortedResultForType);
                        }
                    } finally {
                        if (Bridge.is($t, System.IDisposable)) {
                            $t.System$IDisposable$Dispose();
                        }
                    }
                    return resultList;
                },
                CompareMembers: function (member1, member2) {
                    if (member1.Order > member2.Order) {
                        return 1;
                    } else if (member1.Order < member2.Order) {
                        return -1;
                    } else {
                        return System.String.compare(member1.Name, member2.Name);
                    }
                },
                GetDataContractMembersAndValues: function (instance, serializationType, useXmlSerializerFormat) {
                    var $t;
                    var instanceType = Bridge.getType(instance);
                    var membersInformation = System.Runtime.Serialization.DataContractSerializer_Helpers.GetDataContractMembers(instanceType, serializationType, useXmlSerializerFormat);

                    var resultList = new (System.Collections.Generic.List$1(System.Runtime.Serialization.MemberInformationAndValue)).ctor();
                    $t = Bridge.getEnumerator(membersInformation, System.Runtime.Serialization.MemberInformation);
                    try {
                        while ($t.moveNext()) {
                            var memberInformation = $t.Current;
                            var value = null;
                            var successGettingTheValue = false;
                            if (Bridge.is(memberInformation.MemberInfo, System.Reflection.PropertyInfo)) {
                                var propertyInfo = Bridge.cast(memberInformation.MemberInfo, System.Reflection.PropertyInfo);
                                if ((!!propertyInfo.g)) {
                                    value = Bridge.Reflection.midel(propertyInfo.g, Bridge.unbox(instance)).apply(null, null);
                                    successGettingTheValue = true;
                                }
                            } else if (Bridge.is(memberInformation.MemberInfo, System.Reflection.FieldInfo)) {
                                value = Bridge.Reflection.fieldAccess(Bridge.cast(memberInformation.MemberInfo, System.Reflection.FieldInfo), Bridge.unbox(instance));
                                successGettingTheValue = true;
                            } else if (memberInformation.MemberInfo == null) {
                                throw new System.Exception("MemberInfo is null.");
                            } else {
                                throw new System.Exception("The MemberInfo type is not supported.");
                            }
                            if (successGettingTheValue) {
                                if (memberInformation.EmitDefaultValue || !System.Runtime.Serialization.DataContractSerializer_Helpers.IsValueTheDefault(value, memberInformation.MemberType)) {
                                    var memberInformationAndValue = new System.Runtime.Serialization.MemberInformationAndValue(memberInformation, value);
                                    resultList.add(memberInformationAndValue);
                                }
                            }
                        }
                    } finally {
                        if (Bridge.is($t, System.IDisposable)) {
                            $t.System$IDisposable$Dispose();
                        }
                    }

                    return resultList;
                },
                SetMemberValue: function (instance, memberInformation, memberValue) {
                    if (Bridge.is(memberInformation.MemberInfo, System.Reflection.PropertyInfo)) {
                        var propertyInfo = Bridge.cast(memberInformation.MemberInfo, System.Reflection.PropertyInfo);
                        if ((!!propertyInfo.s)) {
                            Bridge.Reflection.midel(propertyInfo.s, instance)(memberValue);
                        }
                    } else if (Bridge.is(memberInformation.MemberInfo, System.Reflection.FieldInfo)) {
                        Bridge.Reflection.fieldAccess(Bridge.cast(memberInformation.MemberInfo, System.Reflection.FieldInfo), Bridge.unbox(instance), Bridge.unbox(memberValue));
                    } else if (memberInformation.MemberInfo == null) {
                        throw new System.Exception("MemberInfo is null.");
                    } else {
                        throw new System.Exception("The MemberInfo type is not supported.");
                    }
                },
                GetMemberValue: function (instance, memberInformation) {
                    if (Bridge.is(memberInformation.MemberInfo, System.Reflection.PropertyInfo)) {
                        var propertyInfo = Bridge.cast(memberInformation.MemberInfo, System.Reflection.PropertyInfo);
                        if ((!!propertyInfo.g)) {
                            return Bridge.Reflection.midel(propertyInfo.g, Bridge.unbox(instance))();
                        }
                    } else if (Bridge.is(memberInformation.MemberInfo, System.Reflection.FieldInfo)) {
                        return Bridge.Reflection.fieldAccess(Bridge.cast(memberInformation.MemberInfo, System.Reflection.FieldInfo), Bridge.unbox(instance));
                    } else if (memberInformation.MemberInfo == null) {
                        throw new System.Exception("MemberInfo is null.");
                    } else {
                        throw new System.Exception("The MemberInfo type is not supported.");
                    }

                    return null;
                },
                GetDataMemberInformation: function (memberInfo1) {
                    var $t;
                    var emitDefaultValue = true;
                    var isRequired = false;
                    var name = null;
                    var order = -1;
                    var hasXmlElementAttribute = false;
                    var hasXmlAttributeAttribute = false;

                    var memberInfoBase = System.Runtime.Serialization.DataContractSerializer_Helpers.WalkUpInheritanceChainToFindWhereMemberIsFirstDefined(memberInfo1);

                    $t = Bridge.getEnumerator(System.Attribute.getCustomAttributes(memberInfoBase, false, false));
                    try {
                        while ($t.moveNext()) {
                            var attr = Bridge.cast($t.Current, System.Attribute);
                            if (Bridge.is(attr, System.Runtime.Serialization.DataMember2Attribute)) {
                                var attribute = Bridge.cast(attr, System.Runtime.Serialization.DataMember2Attribute);
                                emitDefaultValue = attribute.EmitDefaultValue;
                                isRequired = attribute.IsRequired;
                                if (!System.String.isNullOrWhiteSpace(attribute.Name)) {
                                    name = attribute.Name;
                                }
                                order = attribute.Order;
                            } else if (Bridge.is(attr, System.Runtime.Serialization.DataMemberAttribute)) {
                                var attribute1 = Bridge.cast(attr, System.Runtime.Serialization.DataMemberAttribute);
                                emitDefaultValue = attribute1.EmitDefaultValue;
                                isRequired = attribute1.IsRequired;
                                if (!System.String.isNullOrWhiteSpace(attribute1.Name)) {
                                    name = attribute1.Name;
                                }
                                order = attribute1.Order;
                            } else if (Bridge.is(attr, System.Xml.Serialization.XmlElementAttribute)) {
                                var attribute2 = Bridge.cast(attr, System.Xml.Serialization.XmlElementAttribute);
                                hasXmlElementAttribute = true;
                                if (!System.String.isNullOrWhiteSpace(attribute2.ElementName)) {
                                    name = attribute2.ElementName;
                                }
                                order = attribute2.Order;
                            } else if (Bridge.is(attr, System.Xml.Serialization.XmlAttributeAttribute)) {
                                var attribute3 = Bridge.cast(attr, System.Xml.Serialization.XmlAttributeAttribute);
                                hasXmlAttributeAttribute = true;
                                if (!System.String.isNullOrWhiteSpace(attribute3.AttributeName)) {
                                    name = attribute3.AttributeName;
                                }
                            }
                        }
                    } finally {
                        if (Bridge.is($t, System.IDisposable)) {
                            $t.System$IDisposable$Dispose();
                        }
                    }
                    if (name == null) {
                        name = memberInfoBase.n;
                    }
                    return new System.Runtime.Serialization.MemberInformation(memberInfo1, memberInfoBase, emitDefaultValue, isRequired, name, order, hasXmlElementAttribute, hasXmlAttributeAttribute);
                },
                /**
                 * Same as "MethodInfo.GetBaseDefinition()" (which is not available at the time of writing)
                 *
                 * @static
                 * @private
                 * @this System.Runtime.Serialization.DataContractSerializer_Helpers
                 * @memberof System.Runtime.Serialization.DataContractSerializer_Helpers
                 * @param   {System.Object}    memberInfo
                 * @return  {System.Object}
                 */
                WalkUpInheritanceChainToFindWhereMemberIsFirstDefined: function (memberInfo) {


                    var type = memberInfo.td;
                    var objectType = System.Object;
                    var memberName = memberInfo.n;
                    var isProperty = (Bridge.is(memberInfo, System.Reflection.PropertyInfo));
                    var isField = (Bridge.is(memberInfo, System.Reflection.FieldInfo));
                    var result = memberInfo;

                    while (((type = Bridge.Reflection.getBaseType(type))) != null && (!Bridge.referenceEquals(type, objectType))) {
                        var baseMemberInfo;
                        if (isProperty) {
                            baseMemberInfo = Bridge.Reflection.getMembers(type, 16, 284, memberName);
                        } else if (isField) {
                            baseMemberInfo = Bridge.Reflection.getMembers(type, 4, 284, memberName);
                        } else {
                            break;
                        }
                        if (!Bridge.equals(baseMemberInfo, null)) {
                            result = baseMemberInfo;
                        }
                    }

                    return result;
                },
                IsElementNil: function (element) {
                    var $t;
                    $t = Bridge.getEnumerator(element.Attributes(), System.Xml.Linq.XAttribute);
                    try {
                        while ($t.moveNext()) {
                            var attribute = $t.Current;
                            if (Bridge.referenceEquals(attribute.Name.NamespaceName, System.Runtime.Serialization.DataContractSerializer_Helpers.XMLSCHEMA_NAMESPACE) && Bridge.referenceEquals(attribute.Name.LocalName, "nil") && Bridge.referenceEquals(attribute.Value.toLowerCase(), "true")) {
                                return true;
                            }
                        }
                    } finally {
                        if (Bridge.is($t, System.IDisposable)) {
                            $t.System$IDisposable$Dispose();
                        }
                    }
                    return false;
                },
                /**
                 * Useful to call the methods "OnDeserializing, OnDeserialized, OnSerializing and OnSerialized.
                 *
                 * @static
                 * @this System.Runtime.Serialization.DataContractSerializer_Helpers
                 * @memberof System.Runtime.Serialization.DataContractSerializer_Helpers
                 * @param   {System.Object}                              obj              
                 * @param   {System.Type}                                objType          
                 * @param   {System.Type}                                attributeType    
                 * @param   {System.Collections.Generic.Dictionary$2}    methodsCache
                 * @return  {void}
                 */
                CallMethod: function (obj, objType, attributeType, methodsCache) {
                    var methodIfAny;

                    if (methodsCache.containsKey(objType)) {
                        methodIfAny = methodsCache.getItem(objType);
                    } else {
                        var bindingFlags = 52;
                        var instanceMethods = INTERNAL_BridgeWorkarounds.TypeGetMethods_SimulatorCompatible(objType, bindingFlags);
                        methodIfAny = System.Linq.Enumerable.from(System.Linq.Enumerable.from(instanceMethods).where(function (m) {
                                    return System.Attribute.getCustomAttributes(m, attributeType, false).length > 0;
                                })).firstOrDefault(null, null);
                        methodsCache.add(objType, methodIfAny);
                    }

                    if (methodIfAny != null) {
                        Bridge.Reflection.midel(methodIfAny, Bridge.unbox(obj)).apply(null, Bridge.unbox(System.Array.init([null], System.Object)));
                    }
                },
                GetTypeNameSafeForSerialization: function (type) {
                    var isRunningUnderJSIL = false;

                    if (isRunningUnderJSIL) {
                        return System.Runtime.Serialization.DataContractSerializer_Helpers.JSIL_Workaround_GetTypeNameSafeForSerialization(type);
                    } else {
                        var typeName = System.Runtime.Serialization.DataContractSerializer_Helpers.MakeGenericTypeReadyForSerialization(type);
                        var currentType = type;

                        while ((Bridge.Reflection.getMetaValue(currentType, "td", null) != null)) {
                            currentType = Bridge.Reflection.getMetaValue(currentType, "td", null);

                            typeName = (System.Runtime.Serialization.DataContractSerializer_Helpers.MakeGenericTypeReadyForSerialization(currentType) || "") + "." + (typeName || "");
                        }

                        return typeName;
                    }
                },
                MakeGenericTypeReadyForSerialization: function (type) {
                    var $t;
                    var readableTypeName;
                    var typeName = Bridge.Reflection.getTypeName(type);

                    var lastIndexOfPlusSign = typeName.lastIndexOf(String.fromCharCode(43));
                    if (lastIndexOfPlusSign > -1) {
                        typeName = typeName.substr(((lastIndexOfPlusSign + 1) | 0));
                    }

                    if (Bridge.Reflection.isGenericType(type)) {
                        readableTypeName = (typeName.substr(0, System.String.indexOf(typeName, String.fromCharCode(96))) || "") + "Of";

                        $t = Bridge.getEnumerator(Bridge.Reflection.getGenericArguments(type));
                        try {
                            while ($t.moveNext()) {
                                var typeInGenericTypeArguments = $t.Current;
                                if (System.Runtime.Serialization.DataContractSerializer_ValueTypesHandler.TypesToNames.containsKey(typeInGenericTypeArguments)) {
                                    readableTypeName = (readableTypeName || "") + (System.Runtime.Serialization.DataContractSerializer_ValueTypesHandler.TypesToNames.getItem(typeInGenericTypeArguments) || "");
                                } else {
                                    readableTypeName = (readableTypeName || "") + ((System.Runtime.Serialization.DataContractSerializer_Helpers.MakeGenericTypeReadyForSerialization(typeInGenericTypeArguments)) || "");
                                }
                            }
                        } finally {
                            if (Bridge.is($t, System.IDisposable)) {
                                $t.System$IDisposable$Dispose();
                            }
                        }
                    } else {
                        readableTypeName = Bridge.Reflection.getTypeName(type);
                    }
                    return readableTypeName;
                },
                JSIL_Workaround_GetTypeNameSafeForSerialization: function (type) {
                    var $t;
                    var readableTypeName = "";
                    var typeName = Bridge.Reflection.getTypeName(type);
                    if (!System.Linq.Enumerable.from(typeName).contains(96)) {
                        readableTypeName = System.String.replaceAll(typeName, String.fromCharCode(43), String.fromCharCode(46));
                    } else {
                        var currentType = type;
                        var currentTypeFullName = Bridge.Reflection.getTypeFullName(type);

                        var assembly = type.Assembly;
                        var separatorToAdd = "";
                        while (!System.String.isNullOrWhiteSpace(currentTypeFullName)) {
                            var currentTypeName = currentType.Name;
                            if (currentType.IsGenericType) {
                                currentTypeName = currentTypeName.substr(((System.String.indexOf(currentTypeName, String.fromCharCode(43)) + 1) | 0));
                                currentTypeName = (currentTypeName.substr(0, System.String.indexOf(currentTypeName, String.fromCharCode(96))) || "") + "Of";

                                $t = Bridge.getEnumerator(Bridge.Reflection.getGenericArguments(type));
                                try {
                                    while ($t.moveNext()) {
                                        var typeInGenericTypeArguments = $t.Current;
                                        if (System.Runtime.Serialization.DataContractSerializer_ValueTypesHandler.TypesToNames.containsKey(typeInGenericTypeArguments)) {
                                            currentTypeName = (currentTypeName || "") + (System.Runtime.Serialization.DataContractSerializer_ValueTypesHandler.TypesToNames.getItem(typeInGenericTypeArguments) || "");
                                        } else {

                                            currentTypeName = (currentTypeName || "") + ((System.Runtime.Serialization.DataContractSerializer_Helpers.JSIL_Workaround_GetTypeNameSafeForSerialization(typeInGenericTypeArguments)) || "");
                                        }
                                    }
                                } finally {
                                    if (Bridge.is($t, System.IDisposable)) {
                                        $t.System$IDisposable$Dispose();
                                    }
                                }
                                readableTypeName = (currentTypeName || "") + (separatorToAdd || "") + (readableTypeName || "");
                            } else {
                                readableTypeName = (currentTypeName.substr(((System.String.indexOf(currentTypeName, String.fromCharCode(43)) + 1) | 0)) || "") + (separatorToAdd || "") + (readableTypeName || "");
                            }

                            currentTypeFullName = currentTypeFullName.substr(0, System.String.indexOf(currentTypeFullName, String.fromCharCode(91)));

                            var indexOfLastPlusSign = currentTypeFullName.lastIndexOf(String.fromCharCode(43));

                            currentTypeFullName = currentTypeFullName.substr(0, indexOfLastPlusSign);

                            if (System.String.isNullOrWhiteSpace(currentTypeFullName)) {
                                break;
                            }
                            currentType = assembly.GetType(currentTypeFullName);
                            separatorToAdd = ".";
                        }
                    }
                    return readableTypeName;
                },
                GetTypeInformationByReadingAttributes: function (objectType, nodeDefaultNamespaceIfAny, useXmlSerializerFormat) {
                    var $t;

                    var namespaceName;
                    if (Bridge.referenceEquals(Bridge.Reflection.getTypeNamespace(objectType), "System.Collections.Generic") || Bridge.referenceEquals(Bridge.Reflection.getTypeNamespace(objectType), "")) {
                        namespaceName = nodeDefaultNamespaceIfAny;
                    } else {
                        namespaceName = System.Runtime.Serialization.DataContractSerializer_Helpers.GetDefaultNamespace(Bridge.Reflection.getTypeNamespace(objectType), useXmlSerializerFormat);
                    }
                    var name = System.Runtime.Serialization.DataContractSerializer_Helpers.GetTypeNameSafeForSerialization(objectType);
                    var itemName = null;
                    var keyName = null;
                    var valueName = null;
                    var serializationType = System.Runtime.Serialization.SerializationType.Default;

                    var objectTypeForComparison = Bridge.Reflection.isGenericType(objectType) ? Bridge.Reflection.getGenericTypeDefinition(objectType) : objectType;
                    if (System.Runtime.Serialization.DataContractSerializer_Helpers.INTERNAL_TypesWithOmittedSerializableAttribute.contains(objectTypeForComparison)) {
                        serializationType = System.Runtime.Serialization.SerializationType.Serializable;
                    } else {

                        $t = Bridge.getEnumerator(Bridge.Reflection.getAttributes(objectType, null, true));
                        try {
                            while ($t.moveNext()) {
                                var attribute = Bridge.cast($t.Current, System.Attribute);
                                if (Bridge.is(attribute, System.Runtime.Serialization.DataContractAttribute)) {
                                    serializationType = System.Runtime.Serialization.SerializationType.DataContract;

                                    var attributeAsDataContractAttribute = Bridge.cast(attribute, System.Runtime.Serialization.DataContractAttribute);
                                    if (!System.String.isNullOrWhiteSpace(attributeAsDataContractAttribute.Namespace)) {
                                        namespaceName = attributeAsDataContractAttribute.Namespace;
                                    }
                                    if (!System.String.isNullOrWhiteSpace(attributeAsDataContractAttribute.Name)) {
                                        name = attributeAsDataContractAttribute.Name;
                                    }
                                } else if (Bridge.is(attribute, System.Runtime.Serialization.CollectionDataContractAttribute)) {
                                    serializationType = System.Runtime.Serialization.SerializationType.DataContract;

                                    var collectionDataContractAttribute = Bridge.cast(attribute, System.Runtime.Serialization.CollectionDataContractAttribute);
                                    if (!System.String.isNullOrWhiteSpace(collectionDataContractAttribute.Namespace)) {
                                        namespaceName = collectionDataContractAttribute.Namespace;
                                    }
                                    if (!System.String.isNullOrWhiteSpace(collectionDataContractAttribute.Name)) {
                                        name = collectionDataContractAttribute.Name;
                                    }
                                    if (!System.String.isNullOrWhiteSpace(collectionDataContractAttribute.ItemName)) {
                                        itemName = collectionDataContractAttribute.ItemName;
                                    }
                                    if (!System.String.isNullOrWhiteSpace(collectionDataContractAttribute.KeyName)) {
                                        itemName = collectionDataContractAttribute.KeyName;
                                    }
                                    if (!System.String.isNullOrWhiteSpace(collectionDataContractAttribute.ValueName)) {
                                        itemName = collectionDataContractAttribute.ValueName;
                                    }
                                } else if (Bridge.is(attribute, System.Runtime.Serialization.CollectionDataContract2Attribute)) {
                                    serializationType = System.Runtime.Serialization.SerializationType.DataContract;

                                    var collectionDataContractAttribute1 = Bridge.cast(attribute, System.Runtime.Serialization.CollectionDataContract2Attribute);
                                    if (!System.String.isNullOrWhiteSpace(collectionDataContractAttribute1.Namespace)) {
                                        namespaceName = collectionDataContractAttribute1.Namespace;
                                    }
                                    if (!System.String.isNullOrWhiteSpace(collectionDataContractAttribute1.Name)) {
                                        name = collectionDataContractAttribute1.Name;
                                    }
                                    if (!System.String.isNullOrWhiteSpace(collectionDataContractAttribute1.ItemName)) {
                                        itemName = collectionDataContractAttribute1.ItemName;
                                    }
                                    if (!System.String.isNullOrWhiteSpace(collectionDataContractAttribute1.KeyName)) {
                                        itemName = collectionDataContractAttribute1.KeyName;
                                    }
                                    if (!System.String.isNullOrWhiteSpace(collectionDataContractAttribute1.ValueName)) {
                                        itemName = collectionDataContractAttribute1.ValueName;
                                    }
                                    break;
                                } else if (Bridge.is(attribute, System.SerializableAttribute)) {
                                    if (serializationType === System.Runtime.Serialization.SerializationType.Default) {
                                        serializationType = System.Runtime.Serialization.SerializationType.Serializable;
                                    }
                                } else if (Bridge.is(attribute, System.Serializable2Attribute)) {
                                    if (serializationType === System.Runtime.Serialization.SerializationType.Default) {
                                        serializationType = System.Runtime.Serialization.SerializationType.Serializable;
                                    }
                                }
                            }
                        } finally {
                            if (Bridge.is($t, System.IDisposable)) {
                                $t.System$IDisposable$Dispose();
                            }
                        }
                    }

                    return new System.Runtime.Serialization.TypeInformation(objectType, name, namespaceName, itemName, keyName, valueName, serializationType);
                },
                GetDefault: function (type) {
                    if (Bridge.Reflection.isValueType(type) && !System.Runtime.Serialization.DataContractSerializer_Helpers.IsNullable(type)) {
                        return Bridge.createInstance(type);
                    }
                    return null;
                },
                DoesTypeHaveConstructorWithParameter: function (type, typeOfTheParameter) {
                    var $t, $t1;



                    var constructors = Bridge.Reflection.getMembers(type, 1, 28);

                    var constructorsList = new (System.Collections.Generic.List$1(System.Reflection.ConstructorInfo)).ctor();

                    $t = Bridge.getEnumerator(constructors);
                    try {
                        while ($t.moveNext()) {
                            var constructorInfo = $t.Current;
                            if (INTERNAL_BridgeWorkarounds.ConstructorInfoIsPublic_SimulatorCompatible(constructorInfo) && !INTERNAL_BridgeWorkarounds.ConstructorInfoIsStatic_SimulatorCompatible(constructorInfo)) {
                                constructorsList.add(constructorInfo);
                            }
                        }
                    } finally {
                        if (Bridge.is($t, System.IDisposable)) {
                            $t.System$IDisposable$Dispose();
                        }
                    }

                    constructors = constructorsList.ToArray();

                    $t1 = Bridge.getEnumerator(constructors);
                    try {
                        while ($t1.moveNext()) {
                            var constructorInfo1 = $t1.Current;
                            var parameters = (constructorInfo1.pi || []);
                            if (parameters.length === 1) {
                                var parameterInfo = parameters[System.Array.index(0, parameters)];
                                if (Bridge.Reflection.isAssignableFrom(parameterInfo.pt, typeOfTheParameter)) {
                                    return true;
                                }
                            }
                        }
                    } finally {
                        if (Bridge.is($t1, System.IDisposable)) {
                            $t1.System$IDisposable$Dispose();
                        }
                    }
                    return false;
                },
                GetInterface: function (type, name) {
                    var $t;

                    $t = Bridge.getEnumerator(Bridge.Reflection.getInterfaces(type));
                    try {
                        while ($t.moveNext()) {
                            var theInterface = $t.Current;
                            if (Bridge.referenceEquals(Bridge.Reflection.getTypeName(theInterface), name)) {
                                return theInterface;
                            }
                        }
                    } finally {
                        if (Bridge.is($t, System.IDisposable)) {
                            $t.System$IDisposable$Dispose();
                        }
                    }
                    return null;
                },
                ShouldMemberBeConsideredForSerialization: function (memberInfo, serializationType, useXmlSerializerFormat) {
                    var $t;

                    var shouldSerialize;

                    if (serializationType === System.Runtime.Serialization.SerializationType.Serializable) {
                        if (Bridge.is(memberInfo, System.Reflection.FieldInfo)) {
                            shouldSerialize = true;
                            var fieldInfo = Bridge.cast(memberInfo, System.Reflection.FieldInfo);
                            if (INTERNAL_BridgeWorkarounds.FieldInfoIsStatic_SimulatorCompatible(fieldInfo)) {
                                shouldSerialize = false;
                            }
                            if (shouldSerialize) {
                                $t = Bridge.getEnumerator(System.Attribute.getCustomAttributes(fieldInfo, false, false));
                                try {
                                    while ($t.moveNext()) {
                                        var attribute = Bridge.cast($t.Current, System.Attribute);

                                        if (Bridge.referenceEquals(Bridge.Reflection.getTypeName(Bridge.getType(attribute)), "NonSerializedAttribute")) {
                                            shouldSerialize = false;
                                            break;
                                        }

                                    }
                                } finally {
                                    if (Bridge.is($t, System.IDisposable)) {
                                        $t.System$IDisposable$Dispose();
                                    }
                                }
                            }
                        } else {
                            shouldSerialize = false;
                        }
                    } else if (serializationType === System.Runtime.Serialization.SerializationType.Default || useXmlSerializerFormat) {

                        if (Bridge.is(memberInfo, System.Reflection.PropertyInfo)) {
                            var propertyInfo = Bridge.cast(memberInfo, System.Reflection.PropertyInfo);

                            var getter = propertyInfo.g;
                            var setter = propertyInfo.s;

                            shouldSerialize = getter != null && !INTERNAL_BridgeWorkarounds.MethodInfoIsStatic_SimulatorCompatible(getter) && setter != null && !INTERNAL_BridgeWorkarounds.MethodInfoIsStatic_SimulatorCompatible(setter);
                        } else if (Bridge.is(memberInfo, System.Reflection.FieldInfo)) {
                            var fieldInfo1 = Bridge.cast(memberInfo, System.Reflection.FieldInfo);
                            shouldSerialize = INTERNAL_BridgeWorkarounds.FieldInfoIsPublic_SimulatorCompatible(fieldInfo1) && !INTERNAL_BridgeWorkarounds.FieldInfoIsStatic_SimulatorCompatible(fieldInfo1);
                        } else {
                            throw new System.Exception("Unsupported member type.");
                        }
                    } else {

                        shouldSerialize = System.Runtime.Serialization.DataContractSerializer_Helpers.DoesMemberHaveTheDataMemberAttribute(memberInfo);
                    }


                    return shouldSerialize;
                },
                DoesMemberHaveTheDataMemberAttribute: function (memberInfo1) {
                    var $t;
                    var memberInfoBase = System.Runtime.Serialization.DataContractSerializer_Helpers.WalkUpInheritanceChainToFindWhereMemberIsFirstDefined(memberInfo1);
                    $t = Bridge.getEnumerator(System.Attribute.getCustomAttributes(memberInfoBase, false, false));
                    try {
                        while ($t.moveNext()) {
                            var attribute = Bridge.cast($t.Current, System.Attribute);
                            if (Bridge.is(attribute, System.Runtime.Serialization.DataMemberAttribute) || Bridge.is(attribute, System.Runtime.Serialization.DataMember2Attribute)) {
                                return true;
                            }
                        }
                    } finally {
                        if (Bridge.is($t, System.IDisposable)) {
                            $t.System$IDisposable$Dispose();
                        }
                    }
                    return false;

                    /* 
                    // ======= OLD CODE BEFORE June 26, 2019 ========
                    //todo: when supported by JSIL, replace the code with the method "IsDefined", such as: "memberInfo.IsDefined(typeof(DataMemberAttribute), false))"  IMPORTANT: verify that it still works fine with abstract inherited properties though.
                    foreach (MemberInfo memberInfo in GetMemberInheritanceChain(memberInfo1))
                    {
                       foreach (Attribute attribute in memberInfo.GetCustomAttributes(false))
                       {
                           if (attribute is DataMemberAttribute
                               || attribute is DataMember2Attribute)
                           {
                               return true;
                           }
                       }
                    }
                    return false;
                    */
                },
                IsValueTheDefault: function (value, type) {
                    if (Bridge.Reflection.isValueType(type) && !System.Runtime.Serialization.DataContractSerializer_Helpers.IsNullable(type)) {
                        if (value == null) {
                            return false;
                        } else {
                            return Bridge.equals(value, Bridge.createInstance(type));
                        }
                    } else {
                        return (value == null);
                    }
                },
                IsNullable: function (type) {
                    return System.Nullable.getUnderlyingType(type) != null;
                },
                IsNullOrUndefined: function (obj) {
                    if (CSHTML5.Interop.IsRunningInTheSimulator) {
                        if (obj == null) {
                            return true;
                        }
                        if (!(Bridge.is(obj, DotNetBrowser.JSValue))) {
                            return false;
                        }
                        var value = Bridge.cast(obj, DotNetBrowser.JSValue);
                        return value.IsNull() || value.IsUndefined();
                    } else {
                        return System.Convert.toBoolean((typeof obj === 'undefined' || obj === null));
                    }
                }
            }
        }
    });

    Bridge.define("System.Runtime.Serialization.DataContractSerializer_KnownTypes", {
        statics: {
            methods: {
                CheckIfItIsAKnownType: function (obj, objectType, knownTypes, memberType) {
                    var $t;
                    if (System.Linq.Enumerable.from(knownTypes).contains(memberType)) {
                        return true;
                    }

                    $t = Bridge.getEnumerator(System.Runtime.Serialization.DataContractSerializer_KnownTypes.GetKnownTypesByReadingKnownTypeAttributes(objectType), System.Type);
                    try {
                        while ($t.moveNext()) {
                            var knownType = $t.Current;
                            if (Bridge.referenceEquals(knownType, memberType)) {
                                return true;
                            }
                        }
                    } finally {
                        if (Bridge.is($t, System.IDisposable)) {
                            $t.System$IDisposable$Dispose();
                        }
                    }

                    return false;
                },
                GetCSharpTypeForNode: function (xElement, parentType, memberType, knownTypes, parentMemberInformationIfNotRoot, useXmlSerializerFormat) {
                    var $t, $t1, $t2;
                    var xmlTypeAttribute = null;
                    $t = Bridge.getEnumerator(xElement.Attributes$1(System.Xml.Linq.XNamespace.Get(System.Runtime.Serialization.DataContractSerializer_Helpers.XMLSCHEMA_NAMESPACE).GetName("type")), System.Xml.Linq.XAttribute);
                    try {
                        while ($t.moveNext()) {
                            var attribute = $t.Current;
                            xmlTypeAttribute = attribute;
                            break;
                        }
                    } finally {
                        if (Bridge.is($t, System.IDisposable)) {
                            $t.System$IDisposable$Dispose();
                        }
                    }


                    var typeNameInXmlTypeAttribute = { v : null };
                    var typeNamespaceInXmlTypeAttribute = { v : null };
                    if (xmlTypeAttribute != null) {
                        System.Runtime.Serialization.DataContractSerializer_KnownTypes.GetNameAndNamespaceFromXmlTypeAttribute(xmlTypeAttribute, xElement, typeNameInXmlTypeAttribute, typeNamespaceInXmlTypeAttribute);
                    }

                    if (System.String.isNullOrWhiteSpace(typeNameInXmlTypeAttribute.v) && Bridge.referenceEquals(memberType, System.Object)) {
                        typeNameInXmlTypeAttribute.v = xElement.Name.LocalName;
                        typeNamespaceInXmlTypeAttribute.v = (System.Xml.Linq.XNamespace.op_Inequality(xElement.Name.Namespace, null) ? xElement.Name.Namespace.NamespaceName : null);
                        if (System.String.isNullOrEmpty(typeNamespaceInXmlTypeAttribute.v)) {
                            typeNamespaceInXmlTypeAttribute.v = xElement.GetDefaultNamespace().NamespaceName;
                        }
                    }

                    if (!System.String.isNullOrWhiteSpace(typeNamespaceInXmlTypeAttribute.v) && !System.String.isNullOrWhiteSpace(typeNameInXmlTypeAttribute.v)) {
                        if (System.Runtime.Serialization.DataContractSerializer_KnownTypes.IsTypeSameAsTheOneSpecifiedInXmlTypeAttribute(memberType, typeNameInXmlTypeAttribute.v, typeNamespaceInXmlTypeAttribute.v, useXmlSerializerFormat)) {

                        } else if ((Bridge.referenceEquals(typeNamespaceInXmlTypeAttribute.v, System.Runtime.Serialization.DataContractSerializer_Helpers.XMLSCHEMA_NAMESPACE) || Bridge.referenceEquals(typeNamespaceInXmlTypeAttribute.v, System.Runtime.Serialization.DataContractSerializer_Helpers.XMLSCHEMA_NAMESPACE_XSD) || Bridge.referenceEquals(typeNamespaceInXmlTypeAttribute.v, System.Runtime.Serialization.DataContractSerializer_Helpers.XMLSCHEMA_NAMESPACE_TYPES)) && System.Runtime.Serialization.DataContractSerializer_ValueTypesHandler.NamesToTypes.containsKey(typeNameInXmlTypeAttribute.v)) {

                            memberType = System.Runtime.Serialization.DataContractSerializer_ValueTypesHandler.NamesToTypes.getItem(typeNameInXmlTypeAttribute.v);
                        } else {

                            var typeWasFound = false;
                            $t1 = Bridge.getEnumerator(knownTypes, System.Type);
                            try {
                                while ($t1.moveNext()) {
                                    var knownType = $t1.Current;
                                    if (System.Runtime.Serialization.DataContractSerializer_KnownTypes.IsTypeSameAsTheOneSpecifiedInXmlTypeAttribute(knownType, typeNameInXmlTypeAttribute.v, typeNamespaceInXmlTypeAttribute.v, useXmlSerializerFormat)) {
                                        memberType = knownType;
                                        typeWasFound = true;
                                        break;
                                    }
                                }
                            } finally {
                                if (Bridge.is($t1, System.IDisposable)) {
                                    $t1.System$IDisposable$Dispose();
                                }
                            }

                            if (!typeWasFound) {
                                if (parentType != null) {
                                    $t2 = Bridge.getEnumerator(System.Runtime.Serialization.DataContractSerializer_KnownTypes.GetKnownTypesByReadingKnownTypeAttributes(parentType), System.Type);
                                    try {
                                        while ($t2.moveNext()) {
                                            var knownType1 = $t2.Current;
                                            if (System.Runtime.Serialization.DataContractSerializer_KnownTypes.IsTypeSameAsTheOneSpecifiedInXmlTypeAttribute(knownType1, typeNameInXmlTypeAttribute.v, typeNamespaceInXmlTypeAttribute.v, useXmlSerializerFormat)) {
                                                memberType = knownType1;
                                                typeWasFound = true;
                                                break;
                                            }
                                        }
                                    } finally {
                                        if (Bridge.is($t2, System.IDisposable)) {
                                            $t2.System$IDisposable$Dispose();
                                        }
                                    }
                                }
                            }

                            if (!typeWasFound) {
                                throw new System.Runtime.Serialization.SerializationException.$ctor1(System.String.format("Element '{0}' contains data of the '{1}' data contract. The deserializer has no knowledge of any type that maps to this contract. Add the type corresponding to '{2}' to the list of known types - for example, by using the KnownTypeAttribute attribute or by adding it to the list of known types passed to DataContractSerializer.", parentMemberInformationIfNotRoot != null ? parentMemberInformationIfNotRoot.Name : "", (typeNamespaceInXmlTypeAttribute.v || "") + ":" + (typeNameInXmlTypeAttribute.v || ""), typeNameInXmlTypeAttribute.v));
                            }

                        }
                    }

                    return memberType;
                },
                IsTypeSameAsTheOneSpecifiedInXmlTypeAttribute: function (typeToCompare, typeNameInXmlTypeAttribute, typeNamespaceInXmlTypeAttribute, useXmlSerializerFormat) {
                    var referenceTypeInfo = System.Runtime.Serialization.DataContractSerializer_Helpers.GetTypeInformationByReadingAttributes(typeToCompare, null, useXmlSerializerFormat);
                    var typeNamespaceName = referenceTypeInfo.NamespaceName;
                    var typeName = referenceTypeInfo.Name;

                    if (Bridge.referenceEquals(typeName, typeNameInXmlTypeAttribute) && Bridge.referenceEquals(typeNamespaceName, typeNamespaceInXmlTypeAttribute)) {
                        return true;
                    } else {
                        return false;
                    }
                },
                GetKnownTypesByReadingKnownTypeAttributes: function (typeThatHasTheKnownTypeAttributes) {
                    return new (Bridge.GeneratorEnumerable$1(System.Type))(Bridge.fn.bind(this, function (typeThatHasTheKnownTypeAttributes) {
                        var $step = 0,
                            $jumpFromFinally,
                            $returnValue,
                            $t,
                            attr,
                            knownTypeAttribute,
                            methodName,
                            methodInfo,
                            types,
                            $t1,
                            type,
                            $async_e;

                        var $enumerator = new (Bridge.GeneratorEnumerator$1(System.Type))(Bridge.fn.bind(this, function () {
                            try {
                                for (;;) {
                                    switch ($step) {
                                        case 0: {
                                            $t = Bridge.getEnumerator(Bridge.Reflection.getAttributes(typeThatHasTheKnownTypeAttributes, System.Runtime.Serialization.KnownTypeAttribute, true));
                                                $step = 1;
                                                continue;
                                        }
                                        case 1: {
                                            if ($t.moveNext()) {
                                                    attr = Bridge.cast($t.Current, System.Attribute);
                                                    $step = 2;
                                                    continue;
                                                }
                                            $step = 19;
                                            continue;
                                        }
                                        case 2: {
                                            if (Bridge.is(attr, System.Runtime.Serialization.KnownTypeAttribute)) {
                                                    $step = 3;
                                                    continue;
                                                } 
                                                $step = 18;
                                                continue;
                                        }
                                        case 3: {
                                            knownTypeAttribute = Bridge.cast(attr, System.Runtime.Serialization.KnownTypeAttribute);

                                                if (knownTypeAttribute.Type != null) {
                                                    $step = 4;
                                                    continue;
                                                } 
                                                $step = 6;
                                                continue;
                                        }
                                        case 4: {
                                            $enumerator.current = knownTypeAttribute.Type;
                                                $step = 5;
                                                return true;
                                        }
                                        case 5: {
                                            $step = 6;
                                            continue;
                                        }
                                        case 6: {
                                            if (!System.String.isNullOrWhiteSpace(knownTypeAttribute.MethodName)) {
                                                    $step = 7;
                                                    continue;
                                                } 
                                                $step = 17;
                                                continue;
                                        }
                                        case 7: {
                                            methodName = knownTypeAttribute.MethodName;
                                                methodInfo = Bridge.Reflection.getMembers(typeThatHasTheKnownTypeAttributes, 8, 56 | 256, methodName);
                                                if (methodInfo != null) {
                                                    $step = 8;
                                                    continue;
                                                } else  {
                                                    $step = 15;
                                                    continue;
                                                }
                                        }
                                        case 8: {
                                            types = Bridge.Reflection.midel(methodInfo, null).apply(null, null);
                                                if (Bridge.is(types, System.Collections.Generic.IEnumerable$1(System.Type))) {
                                                    $step = 9;
                                                    continue;
                                                } 
                                                $step = 14;
                                                continue;
                                        }
                                        case 9: {
                                            $t1 = Bridge.getEnumerator(Bridge.cast(types, System.Collections.Generic.IEnumerable$1(System.Type)), System.Type);
                                                $step = 10;
                                                continue;
                                        }
                                        case 10: {
                                            if ($t1.moveNext()) {
                                                    type = $t1.Current;
                                                    $step = 11;
                                                    continue;
                                                }
                                            $step = 13;
                                            continue;
                                        }
                                        case 11: {
                                            $enumerator.current = type;
                                                $step = 12;
                                                return true;
                                        }
                                        case 12: {
                                            $step = 10;
                                            continue;
                                        }
                                        case 13: {
                                            $step = 14;
                                            continue;
                                        }
                                        case 14: {
                                            $step = 16;
                                            continue;
                                        }
                                        case 15: {
                                            throw new System.Runtime.Serialization.InvalidDataContractException.$ctor1(System.String.format("KnownTypeAttribute attribute on type '{0}' specifies a method named '{1}' to provide known types. Static method '{1}()' was not found on this type. Ensure that the method exists and is marked as static.", Bridge.getTypeName(typeThatHasTheKnownTypeAttributes), methodName));
                                            $step = 16;
                                            continue;
                                        }
                                        case 16: {
                                            $step = 17;
                                            continue;
                                        }
                                        case 17: {
                                            $step = 18;
                                            continue;
                                        }
                                        case 18: {
                                            $step = 1;
                                            continue;
                                        }
                                        case 19: {

                                        }
                                        default: {
                                            return false;
                                        }
                                    }
                                }
                            } catch($async_e1) {
                                $async_e = System.Exception.create($async_e1);
                                throw $async_e;
                            }
                        }));
                        return $enumerator;
                    }, arguments));
                },
                GetNameAndNamespaceFromXmlTypeAttribute: function (xmlTypeAttribute, xElement, typeNameInXmlTypeAttribute, typeNamespaceInXmlTypeAttribute) {
                    typeNameInXmlTypeAttribute.v = null;
                    typeNamespaceInXmlTypeAttribute.v = null;
                    if (xmlTypeAttribute != null) {
                        typeNameInXmlTypeAttribute.v = xmlTypeAttribute.Value;
                        var indexOfColon = System.String.indexOf(typeNameInXmlTypeAttribute.v, String.fromCharCode(58));

                        typeNamespaceInXmlTypeAttribute.v = xElement.GetDefaultNamespace().NamespaceName;
                        if (indexOfColon !== -1) {
                            typeNamespaceInXmlTypeAttribute.v = xElement.GetNamespaceOfPrefix(typeNameInXmlTypeAttribute.v.substr(0, indexOfColon)).NamespaceName;
                        }
                        typeNameInXmlTypeAttribute.v = typeNameInXmlTypeAttribute.v.substr(((indexOfColon + 1) | 0));
                    }
                }
            }
        }
    });

    Bridge.define("System.Runtime.Serialization.DataContractSerializer_Serialization", {
        statics: {
            fields: {
                TypeToOnSerializingMethodCache: null,
                TypeToOnSerializedMethodCache: null
            },
            ctors: {
                init: function () {
                    this.TypeToOnSerializingMethodCache = new (System.Collections.Generic.Dictionary$2(System.Type,System.Reflection.MethodInfo)).ctor();
                    this.TypeToOnSerializedMethodCache = new (System.Collections.Generic.Dictionary$2(System.Type,System.Reflection.MethodInfo)).ctor();
                }
            },
            methods: {
                /**
                 * Serializes the given object.
                 *
                 * @static
                 * @this System.Runtime.Serialization.DataContractSerializer_Serialization
                 * @memberof System.Runtime.Serialization.DataContractSerializer_Serialization
                 * @param   {System.Object}                                   obj                            The instance of the object to serialize
                 * @param   {System.Type}                                     objExpectedType                The type that the serialization should expect (it can be different from the type of the instance if the instance is from a child class of this type)
                 * @param   {System.Collections.Generic.IReadOnlyList$1}      knownTypes                     The types that should be known by the DataContractSerializer to allow serialization.
                 * @param   {boolean}                                         useXmlSerializerFormat         True if the DataContractSerializer should use the format normally used by the XmlSerializer.
                 * @param   {boolean}                                         isRoot                         True if the current node we want to Serialize is the Root (it is the object that is put as the parameter when calling DataContractSerializer.SerializeToXXX.
                 * @param   {boolean}                                         isContainedInsideEnumerable    True if the current element is contained inside an Enumerable. It means that we should add a Node that contains the name of the class expected by the Enumerable (I think, this is old stuff for me)
                 * @param   {System.Runtime.Serialization.TypeInformation}    parentTypeInformation          This contains informations on the type that contains the current object; or in the case of the root, on the type expected by the DataContractSerializer
                 * @param   {string}                                          nodeDefaultNamespaceIfAny      The default namespace that is applied on the node (not sure exactly since it is old stuff for me).
                 * @return  {System.Collections.Generic.List$1}                                              A List&lt;XNode&gt; of the nodes for the current element (a list in case of an Enumerable I think).
                 */
                SerializeToXObjects: function (obj, objExpectedType, knownTypes, useXmlSerializerFormat, isRoot, isContainedInsideEnumerable, parentTypeInformation, nodeDefaultNamespaceIfAny) {


                    var objectType = Bridge.getType(obj);

                    if (Bridge.Reflection.isEnum(objectType) || System.Runtime.Serialization.DataContractSerializer_ValueTypesHandler.TypesToNames.containsKey(objectType)) {

                        return System.Runtime.Serialization.DataContractSerializer_Serialization.SerializeToXObjects_ValueTypeOrString(obj, objExpectedType, objectType, knownTypes, useXmlSerializerFormat, nodeDefaultNamespaceIfAny, isRoot, isContainedInsideEnumerable, parentTypeInformation);
                    } else if (Bridge.referenceEquals(Bridge.getType(obj), System.Array.type(System.Byte))) {

                        return System.Runtime.Serialization.DataContractSerializer_Serialization.SerializeToXObjects_ByteArray(obj, isRoot, isContainedInsideEnumerable);
                    } else if (Bridge.is(obj, System.Collections.IEnumerable)) {

                        return System.Runtime.Serialization.DataContractSerializer_Serialization.SerializeToXObjects_Enumerable_WithRecursion(obj, objectType, knownTypes, useXmlSerializerFormat, nodeDefaultNamespaceIfAny, isRoot, isContainedInsideEnumerable);
                    } else {

                        return System.Runtime.Serialization.DataContractSerializer_Serialization.SerializeToXObjects_Object_WithRecursion(obj, objectType, objExpectedType, knownTypes, useXmlSerializerFormat, nodeDefaultNamespaceIfAny, isRoot, isContainedInsideEnumerable, parentTypeInformation);
                    }
                },
                SerializeToXObjects_ValueTypeOrString: function (obj, objExpectedType, objectType, knownTypes, useXmlSerializerFormat, nodeDefaultNamespaceIfAny, isRoot, isContainedInsideEnumerable, parentTypeInformation) {
                    var str;
                    if (Bridge.is(obj, System.Double)) {
                        str = System.Double.format(System.Nullable.getValue(Bridge.cast(Bridge.unbox(obj, System.Double), System.Double)));
                    } else if (Bridge.is(obj, System.Char) && !Bridge.referenceEquals(objExpectedType, System.String)) {
                        str = Bridge.toString(System.Nullable.getValue(Bridge.cast(Bridge.unbox(obj, System.Char), System.Char)));
                    } else if (Bridge.is(obj, System.DateTime)) {
                        var dt = System.Nullable.getValue(Bridge.cast(Bridge.unbox(obj, System.DateTime), System.DateTime));
                        str = INTERNAL_DateTimeHelpers.FromDateTime(dt);
                    } else if (Bridge.is(obj, System.Boolean)) {
                        str = Bridge.toString(obj).toLowerCase();
                    } else {
                        str = Bridge.toString(obj);
                    }

                    var xtext = new System.Xml.Linq.XText.$ctor1(str);

                    if (isRoot || isContainedInsideEnumerable) {
                        var objType = Bridge.getType(obj);
                        var typeName;
                        var typeNamespace;
                        if (System.Runtime.Serialization.DataContractSerializer_ValueTypesHandler.TypesToNames.containsKey(objType)) {
                            typeName = System.Runtime.Serialization.DataContractSerializer_ValueTypesHandler.TypesToNames.getItem(objType);
                            typeNamespace = "http://schemas.microsoft.com/2003/10/Serialization/";
                        } else {
                            typeName = Bridge.Reflection.getTypeName(objType);
                            typeNamespace = System.Runtime.Serialization.DataContractSerializer_Helpers.GetDefaultNamespace(Bridge.Reflection.getTypeNamespace(objType), useXmlSerializerFormat);
                        }

                        return System.Runtime.Serialization.DataContractSerializer_Serialization.AddSurroundingXElement(xtext, typeName, typeNamespace, isRoot, isContainedInsideEnumerable);
                    } else {
                        return function (_o1) {
                                _o1.add(xtext);
                                return _o1;
                            }(new (System.Collections.Generic.List$1(System.Xml.Linq.XObject)).ctor());
                    }
                },
                SerializeToXObjects_ByteArray: function (obj, isRoot, isContainedInsideEnumerable) {
                    var xtext = new System.Xml.Linq.XText.$ctor1(System.Convert.toBase64String(Bridge.cast(obj, System.Array.type(System.Byte)), null, null, null));

                    if (isRoot || isContainedInsideEnumerable) {
                        return System.Runtime.Serialization.DataContractSerializer_Serialization.AddSurroundingXElement(xtext, "base64Binary", "http://schemas.microsoft.com/2003/10/Serialization/", isRoot, isContainedInsideEnumerable);
                    } else {
                        return function (_o1) {
                                _o1.add(xtext);
                                return _o1;
                            }(new (System.Collections.Generic.List$1(System.Xml.Linq.XObject)).ctor());
                    }
                },
                SerializeToXObjects_Enumerable_WithRecursion: function (obj, objectType, knownTypes, useXmlSerializerFormat, nodeDefaultNamespaceIfAny, isRoot, isContainedInsideEnumerable) {
                    var $t;
                    var result = new (System.Collections.Generic.List$1(System.Xml.Linq.XObject)).ctor();

                    var typeInformation = System.Runtime.Serialization.DataContractSerializer_Helpers.GetTypeInformationByReadingAttributes(objectType, nodeDefaultNamespaceIfAny, useXmlSerializerFormat);

                    $t = Bridge.getEnumerator(Bridge.cast(obj, System.Collections.IEnumerable));
                    try {
                        while ($t.moveNext()) {
                            var item = $t.Current;
                            var itemType = Bridge.getType(item);
                            if (Bridge.is(item, System.Char)) {
                                itemType = System.Char;
                            }

                            var xnodesForItem = System.Runtime.Serialization.DataContractSerializer_Serialization.SerializeToXObjects(item, itemType, knownTypes, useXmlSerializerFormat, false, true, typeInformation, typeInformation.NamespaceName);

                            if (xnodesForItem.Count > 1) {
                                throw new System.Exception("When serializing an IEnumerable, we do not expect an item of the enumerable to be serialized as multiple XNodes.");
                            }
                            var nodeForItem = System.Linq.Enumerable.from(xnodesForItem).first();

                            if (Bridge.is(nodeForItem, System.Xml.Linq.XText)) {


                                var elementName = System.Xml.Linq.XNamespace.Get("http://schemas.microsoft.com/2003/10/Serialization/Arrays").GetName(System.Runtime.Serialization.DataContractSerializer_ValueTypesHandler.TypesToNames.getItem(Bridge.getType(item)));
                                var element = new System.Xml.Linq.XElement.$ctor2(elementName);
                                element.Add(nodeForItem);
                                nodeForItem = element;
                            }

                            result.add(nodeForItem);
                        }
                    } finally {
                        if (Bridge.is($t, System.IDisposable)) {
                            $t.System$IDisposable$Dispose();
                        }
                    }

                    if (isRoot) {
                        var itemsType = { };
                        if (System.Runtime.Serialization.DataContractSerializer_Helpers.IsAssignableToGenericEnumerableOrArray(objectType, itemsType)) {
                            var elementName1 = "Array";

                            if (System.Runtime.Serialization.DataContractSerializer_ValueTypesHandler.TypesToNames.containsKey(itemsType.v)) {
                                elementName1 = "ArrayOf" + (System.Runtime.Serialization.DataContractSerializer_ValueTypesHandler.TypesToNames.getItem(itemsType.v) || "");
                            } else {
                                var itemsTypeName = System.Runtime.Serialization.DataContractSerializer_Helpers.GetTypeNameSafeForSerialization(itemsType.v);

                                elementName1 = "ArrayOf" + (itemsTypeName || "");
                            }

                            var xElement;
                            if (Bridge.Reflection.isPrimitive(itemsType.v)) {
                                xElement = new System.Xml.Linq.XElement.$ctor3(System.Xml.Linq.XNamespace.Get("http://schemas.microsoft.com/2003/10/Serialization/Arrays").GetName(elementName1), result);
                            } else {
                                xElement = new System.Xml.Linq.XElement.$ctor3(System.Xml.Linq.XNamespace.Get(System.Runtime.Serialization.DataContractSerializer_Helpers.GetDefaultNamespace(Bridge.Reflection.getTypeNamespace(itemsType.v), useXmlSerializerFormat)).GetName(elementName1), result);
                            }
                            xElement.Add(new System.Xml.Linq.XAttribute.$ctor1(System.Xml.Linq.XNamespace.Get(System.Runtime.Serialization.DataContractSerializer_Helpers.XMLNS_NAMESPACE).GetName("xmlns:i"), System.Runtime.Serialization.DataContractSerializer_Helpers.XMLSCHEMA_NAMESPACE));
                            return function (_o1) {
                                    _o1.add(xElement);
                                    return _o1;
                                }(new (System.Collections.Generic.List$1(System.Xml.Linq.XObject)).ctor());
                        } else {
                            throw new System.Runtime.Serialization.SerializationException.$ctor1(System.String.format("The type '{0}' cannot be serialized because it does implement IEnumerable<T> nor is it an Array.", [Bridge.getTypeName(objectType)]));
                        }
                    }

                    return result;
                },
                SerializeToXObjects_Object_WithRecursion: function (obj, objectType, resultExpectedType, knownTypes, useXmlSerializerFormat, nodeDefaultNamespaceIfAny, isRoot, isContainedInsideEnumerable, parentTypeInformation) {
                    var $t, $t1, $t2, $t3;
                    System.Runtime.Serialization.DataContractSerializer_Serialization.CallOnSerializingMethod(obj, objectType);

                    var typeInformation = System.Runtime.Serialization.DataContractSerializer_Helpers.GetTypeInformationByReadingAttributes(objectType, nodeDefaultNamespaceIfAny, useXmlSerializerFormat);

                    var childrenObjects = new (System.Collections.Generic.List$1(System.Xml.Linq.XObject)).ctor();
                    var membersAndValues = System.Runtime.Serialization.DataContractSerializer_Helpers.GetDataContractMembersAndValues(obj, typeInformation.serializationType, useXmlSerializerFormat);
                    $t = Bridge.getEnumerator(membersAndValues, System.Runtime.Serialization.MemberInformationAndValue);
                    try {
                        while ($t.moveNext()) {
                            var memberInfoAndValue = $t.Current;

                            var memberInfo = memberInfoAndValue.MemberInformation.MemberInfo;
                            var memberValue = memberInfoAndValue.MemberValue;
                            var memberName = { v : memberInfoAndValue.MemberInformation.Name };
                            var hasXmlAttributeAttribute = memberInfoAndValue.MemberInformation.HasXmlAttributeAttribute;

                            if (hasXmlAttributeAttribute && useXmlSerializerFormat) {
                                if (memberValue != null) {
                                    var memberValueAsString = Bridge.toString(memberValue);
                                    var xnameForMember = System.Runtime.Serialization.DataContractSerializer_Serialization.GetXNameForMember(objectType, typeInformation, memberName);
                                    var xAtttribute = new System.Xml.Linq.XAttribute.$ctor1(xnameForMember, memberValueAsString);
                                    childrenObjects.add(xAtttribute);
                                }
                            } else {
                                var xnameForMember1 = System.Runtime.Serialization.DataContractSerializer_Serialization.GetXNameForMember(objectType, typeInformation, memberName);
                                var xElementForMember = new System.Xml.Linq.XElement.$ctor2(xnameForMember1);

                                var isNull = ((memberValue == null) || System.Runtime.Serialization.DataContractSerializer_Helpers.CheckIfObjectIsNullNullable(memberValue));

                                if (!isNull) {
                                    var expectedType = memberInfoAndValue.MemberInformation.MemberType;
                                    var memberType = Bridge.getType(memberValue);

                                    if (Bridge.is(memberValue, System.Char) && Bridge.referenceEquals(expectedType, System.Char)) {
                                        memberType = System.Char;
                                    } else if (Bridge.referenceEquals(expectedType, System.Double)) {
                                        memberType = System.Double;
                                    } else if (Bridge.referenceEquals(expectedType, System.Byte)) {
                                        memberType = System.Byte;
                                    } else if (Bridge.referenceEquals(expectedType, System.Single)) {
                                        memberType = System.Single;
                                    }

                                    var isValueEnumerableDifferentThanString = (Bridge.is(memberValue, System.Collections.IEnumerable)) && !(Bridge.is(memberValue, System.String));

                                    var nonNullableMemberType = memberType;
                                    if (System.String.startsWith(Bridge.Reflection.getTypeFullName(memberType), "System.Nullable`1")) {
                                        nonNullableMemberType = System.Nullable.getUnderlyingType(memberType);
                                    }
                                    var nonNullableExpectedType = expectedType;
                                    if (System.String.startsWith(Bridge.Reflection.getTypeFullName(expectedType), "System.Nullable`1")) {
                                        nonNullableExpectedType = System.Nullable.getUnderlyingType(expectedType);
                                    }

                                    if (!Bridge.referenceEquals(nonNullableMemberType, nonNullableExpectedType) && !isValueEnumerableDifferentThanString) {
                                        if (System.Runtime.Serialization.DataContractSerializer_ValueTypesHandler.TypesToNames.containsKey(nonNullableMemberType)) {
                                            var prefixForType = xElementForMember.GetPrefixOfNamespace(System.Xml.Linq.XNamespace.op_Implicit("http://www.w3.org/2001/XMLSchema"));
                                            if (System.String.isNullOrWhiteSpace(prefixForType)) {

                                                prefixForType = System.Runtime.Serialization.DataContractSerializer_Helpers.GenerateUniqueNamespacePrefixIfNeeded(xElementForMember, "http://www.w3.org/2001/XMLSchema");

                                                xElementForMember.Add(new System.Xml.Linq.XAttribute.$ctor1(System.Xml.Linq.XNamespace.Get(System.Runtime.Serialization.DataContractSerializer_Helpers.XMLSCHEMA_NAMESPACE).GetName("type"), (prefixForType || "") + ":" + (System.Runtime.Serialization.DataContractSerializer_ValueTypesHandler.TypesToNames.getItem(nonNullableMemberType) || "")));
                                            } else {
                                                xElementForMember.Add(new System.Xml.Linq.XAttribute.$ctor1(System.Xml.Linq.XNamespace.Get(System.Runtime.Serialization.DataContractSerializer_Helpers.XMLSCHEMA_NAMESPACE).GetName("type"), (prefixForType || "") + ":" + (System.Runtime.Serialization.DataContractSerializer_ValueTypesHandler.TypesToNames.getItem(nonNullableMemberType) || "")));
                                            }
                                        } else {
                                            var isTypeOk = System.Runtime.Serialization.DataContractSerializer_KnownTypes.CheckIfItIsAKnownType(obj, objectType, knownTypes, memberType);
                                            if (isTypeOk) {
                                                if (Bridge.Reflection.isEnum(memberType)) {
                                                    xElementForMember.Add(new System.Xml.Linq.XAttribute.$ctor1(System.Xml.Linq.XNamespace.Get(System.Runtime.Serialization.DataContractSerializer_Helpers.XMLSCHEMA_NAMESPACE).GetName("type"), Bridge.Reflection.getTypeName(memberType)));
                                                } else {
                                                    var defaultNamespace = System.Runtime.Serialization.DataContractSerializer_Helpers.GetDefaultNamespace(Bridge.Reflection.getTypeNamespace(memberType), useXmlSerializerFormat);

                                                    var childTypeInformation = System.Runtime.Serialization.DataContractSerializer_Helpers.GetTypeInformationByReadingAttributes(memberType, defaultNamespace, useXmlSerializerFormat);

                                                    var prefixForTypeName = "";
                                                    if (childTypeInformation.NamespaceName != null) {
                                                        prefixForTypeName = (System.Runtime.Serialization.DataContractSerializer_Helpers.GenerateUniqueNamespacePrefixIfNeeded(xElementForMember, childTypeInformation.NamespaceName) || "") + ":";
                                                    }

                                                    xElementForMember.Add(new System.Xml.Linq.XAttribute.$ctor1(System.Xml.Linq.XNamespace.Get(System.Runtime.Serialization.DataContractSerializer_Helpers.XMLSCHEMA_NAMESPACE).GetName("type"), (prefixForTypeName || "") + (childTypeInformation.Name || "")));
                                                }
                                            } else {
                                                var namespaceNameToPutInException = "";
                                                if (typeInformation.NamespaceName != null) {
                                                    namespaceNameToPutInException = ":" + (typeInformation.NamespaceName || "");
                                                }
                                                throw new System.Runtime.Serialization.SerializationException.$ctor1("Type \"" + (Bridge.Reflection.getTypeFullName(memberType) || "") + "\" with data contract name \"" + (Bridge.Reflection.getTypeName(memberType) || "") + (namespaceNameToPutInException || "") + "\" is not expected. Add any types not known statically to the list of known types - for example, by using the KnownTypeAttribute attribute or by adding them to the list of known types passed to DataContractSerializer.");
                                            }
                                        }
                                    }

                                    var xobjectsForMemberValue = System.Runtime.Serialization.DataContractSerializer_Serialization.SerializeToXObjects(memberValue, memberType, knownTypes, useXmlSerializerFormat, false, false, typeInformation, typeInformation.NamespaceName);
                                    $t1 = Bridge.getEnumerator(xobjectsForMemberValue);
                                    try {
                                        while ($t1.moveNext()) {
                                            var xobjectForMemberValue = $t1.Current;
                                            xElementForMember.Add(xobjectForMemberValue);
                                        }
                                    } finally {
                                        if (Bridge.is($t1, System.IDisposable)) {
                                            $t1.System$IDisposable$Dispose();
                                        }
                                    }
                                } else {

                                    var nilName = System.Xml.Linq.XNamespace.Get(System.Runtime.Serialization.DataContractSerializer_Helpers.XMLSCHEMA_NAMESPACE).GetName("nil");
                                    xElementForMember.Add(new System.Xml.Linq.XAttribute.$ctor1(nilName, "true"));
                                }
                                childrenObjects.add(xElementForMember);
                            }
                        }
                    } finally {
                        if (Bridge.is($t, System.IDisposable)) {
                            $t.System$IDisposable$Dispose();
                        }
                    }
                    if (isRoot || isContainedInsideEnumerable) {


                        var typeExpectedByParentEnumerable = isRoot ? resultExpectedType : ($t2 = Bridge.Reflection.getGenericArguments(System.Runtime.Serialization.DataContractSerializer_Helpers.GetInterface(parentTypeInformation.Type, "IEnumerable`1")))[System.Array.index(0, $t2)];

                        var objectTypeName;


                        if (parentTypeInformation != null && !System.String.isNullOrEmpty(parentTypeInformation.ItemName)) {
                            objectTypeName = parentTypeInformation.ItemName;
                        } else {
                            objectTypeName = typeInformation.Name;
                        }

                        var elementName;
                        if (typeInformation.NamespaceName == null) {
                            elementName = System.Xml.Linq.XName.op_Implicit(objectTypeName);
                        } else {
                            elementName = System.Xml.Linq.XNamespace.Get(typeInformation.NamespaceName).GetName(objectTypeName);
                        }

                        var xelement = new System.Xml.Linq.XElement.$ctor2(elementName);

                        if (Bridge.Reflection.isEnum(objectType)) {
                            xelement.Add(new System.Xml.Linq.XAttribute.$ctor1(System.Xml.Linq.XNamespace.Get(System.Runtime.Serialization.DataContractSerializer_Helpers.XMLSCHEMA_NAMESPACE).GetName("type"), objectTypeName));
                        } else if (typeInformation.NamespaceName != null && !Bridge.Reflection.isGenericType(objectType) && !Bridge.referenceEquals(typeExpectedByParentEnumerable, Bridge.getType(obj))) {
                            var prefixForTypeName1 = System.Runtime.Serialization.DataContractSerializer_Helpers.GenerateUniqueNamespacePrefixIfNeeded(xelement, typeInformation.NamespaceName);

                            xelement.Add(new System.Xml.Linq.XAttribute.$ctor1(System.Xml.Linq.XNamespace.Get(System.Runtime.Serialization.DataContractSerializer_Helpers.XMLSCHEMA_NAMESPACE).GetName("type"), (prefixForTypeName1 || "") + ":" + (typeInformation.Name || "")));
                        }

                        $t3 = Bridge.getEnumerator(childrenObjects);
                        try {
                            while ($t3.moveNext()) {
                                var child = $t3.Current;
                                xelement.Add(child);
                            }
                        } finally {
                            if (Bridge.is($t3, System.IDisposable)) {
                                $t3.System$IDisposable$Dispose();
                            }
                        }
                        childrenObjects = function (_o1) {
                                _o1.add(xelement);
                                return _o1;
                            }(new (System.Collections.Generic.List$1(System.Xml.Linq.XObject)).ctor());
                    }

                    System.Runtime.Serialization.DataContractSerializer_Serialization.CallOnSerializedMethod(obj, objectType);

                    return childrenObjects;
                },
                GetXNameForMember: function (objectType, typeInformation, memberName) {
                    var xnameForMember;
                    if (System.String.startsWith(Bridge.Reflection.getTypeFullName(objectType), "System.Collections.Generic.KeyValuePair")) {
                        if (Bridge.referenceEquals(memberName.v, "key")) {
                            memberName.v = "Key";
                        } else if (Bridge.referenceEquals(memberName.v, "value")) {
                            memberName.v = "Value";
                        }
                        xnameForMember = System.Xml.Linq.XName.op_Implicit(memberName.v);
                    } else {
                        if (typeInformation.NamespaceName == null) {
                            xnameForMember = System.Xml.Linq.XName.op_Implicit(memberName.v);
                        } else {
                            xnameForMember = System.Xml.Linq.XNamespace.Get(typeInformation.NamespaceName).GetName(memberName.v);
                        }
                    }

                    return xnameForMember;
                },
                AddSurroundingXElement: function (xtext, surroundingElementName, surroundingElementNamespace, isRoot, isContainedInsideEnumerable) {
                    var elementNamespace = isContainedInsideEnumerable ? "http://schemas.microsoft.com/2003/10/Serialization/Arrays" : surroundingElementNamespace;
                    var xElement = new System.Xml.Linq.XElement.$ctor3(System.Xml.Linq.XNamespace.Get(elementNamespace).GetName(surroundingElementName), xtext);
                    return function (_o1) {
                            _o1.add(xElement);
                            return _o1;
                        }(new (System.Collections.Generic.List$1(System.Xml.Linq.XObject)).ctor());
                },
                CallOnSerializingMethod: function (obj, objType) {
                    System.Runtime.Serialization.DataContractSerializer_Helpers.CallMethod(obj, objType, System.Runtime.Serialization.OnSerializingAttribute, System.Runtime.Serialization.DataContractSerializer_Serialization.TypeToOnSerializingMethodCache);
                },
                CallOnSerializedMethod: function (obj, objType) {
                    System.Runtime.Serialization.DataContractSerializer_Helpers.CallMethod(obj, objType, System.Runtime.Serialization.OnSerializedAttribute, System.Runtime.Serialization.DataContractSerializer_Serialization.TypeToOnSerializedMethodCache);
                }
            }
        }
    });

    Bridge.define("System.Runtime.Serialization.DataContractSerializer_ValueTypesHandler", {
        statics: {
            fields: {
                TypesToNames: null,
                NamesToTypes: null
            },
            ctors: {
                ctor: function () {
                    System.Runtime.Serialization.DataContractSerializer_ValueTypesHandler.FillDictionaries();
                }
            },
            methods: {
                EnsureInitialized: function () {
                    if (System.Runtime.Serialization.DataContractSerializer_ValueTypesHandler.TypesToNames == null) {
                        System.Runtime.Serialization.DataContractSerializer_ValueTypesHandler.FillDictionaries();
                    }
                },
                ConvertStringToValueType: function (str, resultType) {

                    if (Bridge.referenceEquals(resultType, System.String)) {
                        return str;
                    } else if (Bridge.referenceEquals(resultType, System.Char) || Bridge.referenceEquals(resultType, System.Nullable$1(System.Char))) {
                        return Bridge.box(System.Convert.toChar(System.Int32.parse(str), null, 9), System.Char, String.fromCharCode, System.Char.getHashCode);
                    } else if (Bridge.referenceEquals(resultType, System.Boolean) || Bridge.referenceEquals(resultType, System.Nullable$1(System.Boolean))) {
                        return Bridge.box(System.Boolean.parse(str), System.Boolean, System.Boolean.toString);
                    } else if (Bridge.referenceEquals(resultType, System.Int16) || Bridge.referenceEquals(resultType, System.Nullable$1(System.Int16))) {
                        return Bridge.box(System.Int16.parse(str), System.Int16);
                    } else if (Bridge.referenceEquals(resultType, System.Int32) || Bridge.referenceEquals(resultType, System.Nullable$1(System.Int32))) {
                        return Bridge.box(System.Int32.parse(str), System.Int32);
                    } else if (Bridge.referenceEquals(resultType, System.Int64) || Bridge.referenceEquals(resultType, System.Nullable$1(System.Int64))) {
                        return System.Int64.parse(str);
                    } else if (Bridge.referenceEquals(resultType, System.UInt16) || Bridge.referenceEquals(resultType, System.Nullable$1(System.UInt16))) {
                        return Bridge.box(System.UInt16.parse(str), System.UInt16);
                    } else if (Bridge.referenceEquals(resultType, System.UInt32) || Bridge.referenceEquals(resultType, System.Nullable$1(System.UInt32))) {
                        return Bridge.box(System.UInt32.parse(str), System.UInt32);
                    } else if (Bridge.referenceEquals(resultType, System.UInt64) || Bridge.referenceEquals(resultType, System.Nullable$1(System.UInt64))) {
                        return System.UInt64.parse(str);
                    } else if (Bridge.referenceEquals(resultType, System.Decimal) || Bridge.referenceEquals(resultType, System.Nullable$1(System.Decimal))) {
                        return System.Decimal(str);
                    } else if (Bridge.referenceEquals(resultType, System.Single) || Bridge.referenceEquals(resultType, System.Nullable$1(System.Single))) {
                        return Bridge.box(System.Single.parse(str), System.Single, System.Single.format, System.Single.getHashCode);
                    } else if (Bridge.referenceEquals(resultType, System.Double) || Bridge.referenceEquals(resultType, System.Nullable$1(System.Double))) {
                        return Bridge.box(System.Double.parse(str), System.Double, System.Double.format, System.Double.getHashCode);
                    } else if (Bridge.referenceEquals(resultType, System.DateTime) || Bridge.referenceEquals(resultType, System.Nullable$1(System.DateTime))) {
                        return Bridge.box(System.DateTime.parse(str), System.DateTime, System.DateTime.format);
                    } else if (Bridge.referenceEquals(resultType, System.Guid) || Bridge.referenceEquals(resultType, System.Nullable$1(System.Guid))) {
                        return System.Guid.Parse(str);
                    } else if (Bridge.referenceEquals(resultType, System.Byte) || Bridge.referenceEquals(resultType, System.Nullable$1(System.Byte))) {
                        return Bridge.box(System.Byte.parse(str), System.Byte);
                    } else if (Bridge.referenceEquals(resultType, System.SByte) || Bridge.referenceEquals(resultType, System.Nullable$1(System.SByte))) {
                        return Bridge.box(System.SByte.parse(str), System.SByte);
                    } else if (Bridge.Reflection.isEnum(resultType)) {
                        var splitted = System.String.split(str, System.Array.init([32], System.Char).map(function (i) {{ return String.fromCharCode(i); }}), null, 1);
                        var reformattedEnumValue = splitted.join(", ");

                        var enumValue = System.Enum.parse(resultType, reformattedEnumValue, true);

                        return enumValue;
                    }

                    throw new System.Exception("Unable to create the type '" + (Bridge.getTypeName(resultType) || "") + "' from the string '" + (str || "") + "'.");
                },
                FillDictionaries: function () {
                    System.Runtime.Serialization.DataContractSerializer_ValueTypesHandler.TypesToNames = new (System.Collections.Generic.Dictionary$2(System.Type,System.String)).ctor();
                    System.Runtime.Serialization.DataContractSerializer_ValueTypesHandler.NamesToTypes = new (System.Collections.Generic.Dictionary$2(System.String,System.Type)).ctor();


                    System.Runtime.Serialization.DataContractSerializer_ValueTypesHandler.TypesToNames.add(System.Boolean, "boolean");
                    System.Runtime.Serialization.DataContractSerializer_ValueTypesHandler.NamesToTypes.add("boolean", System.Boolean);

                    System.Runtime.Serialization.DataContractSerializer_ValueTypesHandler.TypesToNames.add(System.Char, "char");
                    System.Runtime.Serialization.DataContractSerializer_ValueTypesHandler.NamesToTypes.add("char", System.Char);

                    System.Runtime.Serialization.DataContractSerializer_ValueTypesHandler.TypesToNames.add(System.String, "string");
                    System.Runtime.Serialization.DataContractSerializer_ValueTypesHandler.NamesToTypes.add("string", System.String);

                    System.Runtime.Serialization.DataContractSerializer_ValueTypesHandler.TypesToNames.add(System.Byte, "byte");
                    System.Runtime.Serialization.DataContractSerializer_ValueTypesHandler.NamesToTypes.add("byte", System.Byte);

                    System.Runtime.Serialization.DataContractSerializer_ValueTypesHandler.TypesToNames.add(System.Int16, "short");
                    System.Runtime.Serialization.DataContractSerializer_ValueTypesHandler.NamesToTypes.add("short", System.Int16);

                    System.Runtime.Serialization.DataContractSerializer_ValueTypesHandler.TypesToNames.add(System.Int32, "int");
                    System.Runtime.Serialization.DataContractSerializer_ValueTypesHandler.NamesToTypes.add("int", System.Int32);

                    System.Runtime.Serialization.DataContractSerializer_ValueTypesHandler.TypesToNames.add(System.Int64, "long");
                    System.Runtime.Serialization.DataContractSerializer_ValueTypesHandler.NamesToTypes.add("long", System.Int64);

                    System.Runtime.Serialization.DataContractSerializer_ValueTypesHandler.TypesToNames.add(System.UInt16, "unsignedShort");
                    System.Runtime.Serialization.DataContractSerializer_ValueTypesHandler.NamesToTypes.add("unsignedShort", System.UInt16);

                    System.Runtime.Serialization.DataContractSerializer_ValueTypesHandler.TypesToNames.add(System.UInt32, "unsignedInt");
                    System.Runtime.Serialization.DataContractSerializer_ValueTypesHandler.NamesToTypes.add("unsignedInt", System.UInt32);

                    System.Runtime.Serialization.DataContractSerializer_ValueTypesHandler.TypesToNames.add(System.UInt64, "unsignedLong");
                    System.Runtime.Serialization.DataContractSerializer_ValueTypesHandler.NamesToTypes.add("unsignedLong", System.UInt64);

                    System.Runtime.Serialization.DataContractSerializer_ValueTypesHandler.TypesToNames.add(System.Decimal, "decimal");
                    System.Runtime.Serialization.DataContractSerializer_ValueTypesHandler.NamesToTypes.add("decimal", System.Decimal);

                    System.Runtime.Serialization.DataContractSerializer_ValueTypesHandler.TypesToNames.add(System.Double, "double");
                    System.Runtime.Serialization.DataContractSerializer_ValueTypesHandler.NamesToTypes.add("double", System.Double);

                    System.Runtime.Serialization.DataContractSerializer_ValueTypesHandler.TypesToNames.add(System.Single, "float");
                    System.Runtime.Serialization.DataContractSerializer_ValueTypesHandler.NamesToTypes.add("float", System.Single);

                    System.Runtime.Serialization.DataContractSerializer_ValueTypesHandler.TypesToNames.add(System.DateTime, "dateTime");
                    System.Runtime.Serialization.DataContractSerializer_ValueTypesHandler.NamesToTypes.add("dateTime", System.DateTime);

                    System.Runtime.Serialization.DataContractSerializer_ValueTypesHandler.TypesToNames.add(System.Guid, "guid");
                    System.Runtime.Serialization.DataContractSerializer_ValueTypesHandler.NamesToTypes.add("guid", System.Guid);

                }
            }
        }
    });

    /**
     * When applied to the member of a type, specifies that the member is part of
     a data contract and is serializable by the DataContractSerializer.
     *
     * @public
     * @class System.Runtime.Serialization.DataMember2Attribute
     * @augments System.Attribute
     */
    Bridge.define("System.Runtime.Serialization.DataMember2Attribute", {
        inherits: [System.Attribute],
        fields: {
            /**
             * Gets or sets a value that specifies whether to serialize the default value
             for a field or property being serialized.
             *
             * @instance
             * @public
             * @memberof System.Runtime.Serialization.DataMember2Attribute
             * @function EmitDefaultValue
             * @type boolean
             */
            EmitDefaultValue: false,
            /**
             * Gets or sets a value that instructs the serialization engine that the member
             must be present when reading or deserializing.
             *
             * @instance
             * @public
             * @memberof System.Runtime.Serialization.DataMember2Attribute
             * @function IsRequired
             * @type boolean
             */
            IsRequired: false,
            /**
             * Gets or sets a data member name.
             *
             * @instance
             * @public
             * @memberof System.Runtime.Serialization.DataMember2Attribute
             * @function Name
             * @type string
             */
            Name: null,
            _order: 0
        },
        props: {
            /**
             * Gets or sets the order of serialization and deserialization of a member.
             *
             * @instance
             * @public
             * @memberof System.Runtime.Serialization.DataMember2Attribute
             * @function Order
             * @type number
             */
            Order: {
                get: function () {
                    return this._order;
                },
                set: function (value) {
                    if (value < 0) {
                        throw new System.Runtime.Serialization.InvalidDataContractException.$ctor1("Order cannot be negative.");
                    }
                    this._order = value;
                }
            }
        },
        ctors: {
            init: function () {
                this._order = -1;
            },
            /**
             * Initializes a new instance of the DataMemberAttribute
             class.
             *
             * @instance
             * @public
             * @this System.Runtime.Serialization.DataMember2Attribute
             * @memberof System.Runtime.Serialization.DataMember2Attribute
             * @return  {void}
             */
            ctor: function () {
                this.$initialize();
                System.Attribute.ctor.call(this);
                this.EmitDefaultValue = true;
            },
            $ctor1: function (EmitDefaultValue, IsRequired, Name, Order) {
                if (EmitDefaultValue === void 0) { EmitDefaultValue = true; }
                if (IsRequired === void 0) { IsRequired = false; }
                if (Name === void 0) { Name = null; }
                if (Order === void 0) { Order = -1; }

                this.$initialize();
                System.Attribute.ctor.call(this);
                this.EmitDefaultValue = EmitDefaultValue;
                this.IsRequired = IsRequired;
                this.Name = Name;
                this.Order = Order;
            }
        }
    });

    /**
     * Stores data from a versioned data contract that has been extended by adding
     new members.
     *
     * @public
     * @class System.Runtime.Serialization.ExtensionDataObject
     */
    Bridge.define("System.Runtime.Serialization.ExtensionDataObject");

    /**
     * Provides a data structure to store extra data encountered by the System.Runtime.Serialization.XmlObjectSerializer
     during deserialization of a type marked with the System.Runtime.Serialization.DataContractAttribute
     attribute.
     *
     * @abstract
     * @public
     * @class System.Runtime.Serialization.IExtensibleDataObject
     */
    Bridge.define("System.Runtime.Serialization.IExtensibleDataObject", {
        $kind: "interface"
    });

    Bridge.define("System.Runtime.Serialization.IFormatter", {
        $kind: "interface"
    });

    Bridge.define("System.Runtime.Serialization.MemberInformation", {
        fields: {
            MemberInfo: null,
            /**
             * If the member is "override", this property contains the corresponding "virtual" or "abstract" declaration in the base classes.
             *
             * @instance
             * @public
             * @readonly
             * @memberof System.Runtime.Serialization.MemberInformation
             * @type System.Object
             */
            MemberInfoBase: null,
            EmitDefaultValue: false,
            IsRequired: false,
            Name: null,
            Order: 0,
            HasXmlElementAttribute: false,
            HasXmlAttributeAttribute: false,
            MemberType: null
        },
        ctors: {
            ctor: function (memberInfo, memberInfoBase, emitDefaultValue, isRequired, name, order, hasXmlElementAttribute, hasXmlAttributeAttribute) {
                this.$initialize();
                this.MemberInfo = memberInfo;
                this.MemberInfoBase = memberInfoBase;
                this.EmitDefaultValue = emitDefaultValue;
                this.IsRequired = isRequired;
                this.Name = name;
                this.Order = order;
                this.HasXmlElementAttribute = hasXmlElementAttribute;
                this.HasXmlAttributeAttribute = hasXmlAttributeAttribute;
            }
        }
    });

    Bridge.define("System.Runtime.Serialization.MemberInformationAndValue", {
        fields: {
            MemberInformation: null,
            MemberValue: null
        },
        ctors: {
            ctor: function (memberInformation, memberValue) {
                this.$initialize();
                this.MemberInformation = memberInformation;
                this.MemberValue = memberValue;
            }
        }
    });

    /**
     * The {@link } namespace contains classes that can
     be used for serializing and deserializing objects. Serialization is the process of
     converting an object or a graph of objects into a linear sequence of bytes for
     either storage or transmission to another location. Deserialization is the process
     of taking in stored information and recreating objects from it.
     *
     * @class System.Runtime.Serialization.NamespaceDoc
     */
    Bridge.define("System.Runtime.Serialization.NamespaceDoc");

    Bridge.define("System.Runtime.Serialization.SerializationBinder", {
        methods: {
            BindToName: function (serializedType, assemblyName, typeName) {
                assemblyName.v = null;
                typeName.v = null;
            }
        }
    });

    Bridge.define("System.Runtime.Serialization.SerializationInfo", {
        ctors: {
            ctor: function (type, converter) {
                this.$initialize();
            },
            $ctor1: function (type, converter, requireSameTokenInPartialTrust) {
                this.$initialize();
            }
        }
    });

    Bridge.define("System.Runtime.Serialization.SerializationType", {
        $kind: "enum",
        statics: {
            fields: {
                Default: 1,
                DataContract: 2,
                Serializable: 4
            }
        }
    });

    Bridge.define("System.Runtime.Serialization.TypeInformation", {
        fields: {
            Type: null,
            Name: null,
            NamespaceName: null,
            ItemName: null,
            KeyName: null,
            ValueName: null,
            serializationType: 0
        },
        ctors: {
            ctor: function (type, name, namespaceName, itemName, keyName, valueName, isAttributeSuchAsDataContractPresent) {
                this.$initialize();
                this.Type = type;
                this.Name = name;
                this.NamespaceName = namespaceName;
                this.ItemName = itemName;
                this.KeyName = keyName;
                this.ValueName = valueName;
                this.serializationType = isAttributeSuchAsDataContractPresent;
            }
        }
    });

    /** @namespace System.Security.AccessControl */

    /**
     * The {@link } namespace provides programming elements
     that control access to and audit security-related actions on securable objects
     *
     * @class System.Security.AccessControl.NamespaceDoc
     */
    Bridge.define("System.Security.AccessControl.NamespaceDoc");

    /** @namespace System.Security.Cryptography */

    /**
     * The {@link } namespace provides cryptographic services,
     including secure encoding and decoding of data, as well as many other operations, such
     as hashing, random number generation, and message authentication.
     *
     * @class System.Security.Cryptography.NamespaceDoc
     */
    Bridge.define("System.Security.Cryptography.NamespaceDoc");

    /** @namespace System.Security.Principal */

    /**
     * The {@link } namespace defines a principal object that
     represents the security context under which code is running.
     *
     * @class System.Security.Principal.NamespaceDoc
     */
    Bridge.define("System.Security.Principal.NamespaceDoc");

    /**
     * Indicates that a class can be serialized. This class cannot be inherited.
     *
     * @public
     * @class System.Serializable2Attribute
     * @augments System.Attribute
     */
    Bridge.define("System.Serializable2Attribute", {
        inherits: [System.Attribute],
        ctors: {
            /**
             * Initializes a new instance of the System.SerializableAttribute class.
             *
             * @instance
             * @public
             * @this System.Serializable2Attribute
             * @memberof System.Serializable2Attribute
             * @return  {void}
             */
            ctor: function () {
                this.$initialize();
                System.Attribute.ctor.call(this);
            }
        }
    });

    /** @namespace System.ServiceModel.Channels */

    /**
     * Contains the binding elements that specify the protocols, transports, and
     message encoders used for communication between clients and services.
     *
     * @public
     * @class System.ServiceModel.Channels.Binding
     */
    Bridge.define("System.ServiceModel.Channels.Binding");

    /** @namespace System.ServiceModel */

    /**
     * Configures the security settings of a basicHttpBinding binding.
     *
     * @public
     * @class System.ServiceModel.BasicHttpSecurity
     */
    Bridge.define("System.ServiceModel.BasicHttpSecurity", {
        fields: {
            /**
             * Gets or sets the security mode for a basicHttpBinding binding.
             *
             * @instance
             * @public
             * @memberof System.ServiceModel.BasicHttpSecurity
             * @function Mode
             * @type System.ServiceModel.BasicHttpSecurityMode
             */
            Mode: 0
        },
        ctors: {
            /**
             * Initializes a new instance of the System.ServiceModel.BasicHttpSecurity class.
             *
             * @instance
             * @public
             * @this System.ServiceModel.BasicHttpSecurity
             * @memberof System.ServiceModel.BasicHttpSecurity
             * @return  {void}
             */
            ctor: function () {
                this.$initialize();
            }
        }
    });

    /**
     * Specifies the types of security that can be used with the system-provided
     System.ServiceModel.BasicHttpBinding.
     *
     * @public
     * @class System.ServiceModel.BasicHttpSecurityMode
     */
    Bridge.define("System.ServiceModel.BasicHttpSecurityMode", {
        $kind: "enum",
        statics: {
            fields: {
                /**
                 * The SOAP message is not secured during transfer. This is the default behavior.
                 *
                 * @static
                 * @public
                 * @memberof System.ServiceModel.BasicHttpSecurityMode
                 * @constant
                 * @default 0
                 * @type System.ServiceModel.BasicHttpSecurityMode
                 */
                None: 0,
                /**
                 * Security is provided using HTTPS. The service must be configured with SSL
                 certificates. The SOAP message is protected as a whole using HTTPS. The service
                 is authenticated by the client using the service’s SSL certificate. The client
                 authentication is controlled through the System.ServiceModel.HttpTransportSecurity.ClientCredentialType.
                 *
                 * @static
                 * @public
                 * @memberof System.ServiceModel.BasicHttpSecurityMode
                 * @constant
                 * @default 1
                 * @type System.ServiceModel.BasicHttpSecurityMode
                 */
                Transport: 1
            }
        }
    });

    /**
     * Represents a header that encapsulates an address information item used to
     identify or interact with an endpoint.
     *
     * @abstract
     * @public
     * @class System.ServiceModel.Channels.AddressHeader
     */
    Bridge.define("System.ServiceModel.Channels.AddressHeader");

    /**
     * The {@link } provides classes related to service model channels.
     *
     * @class System.ServiceModel.Channels.NamespaceDoc
     */
    Bridge.define("System.ServiceModel.Channels.NamespaceDoc");

    /**
     * Represents a communication error in either the service or client application.
     *
     * @public
     * @class System.ServiceModel.CommunicationException
     * @augments System.SystemException
     */
    Bridge.define("System.ServiceModel.CommunicationException", {
        inherits: [System.SystemException],
        ctors: {
            /**
             * Initializes a new instance of the System.ServiceModel.CommunicationException
             class.
             *
             * @instance
             * @public
             * @this System.ServiceModel.CommunicationException
             * @memberof System.ServiceModel.CommunicationException
             * @return  {void}
             */
            ctor: function () {
                this.$initialize();
                System.SystemException.ctor.call(this);
            },
            /**
             * Initializes a new instance of the System.ServiceModel.CommunicationException
             class, using the specified message.
             *
             * @instance
             * @public
             * @this System.ServiceModel.CommunicationException
             * @memberof System.ServiceModel.CommunicationException
             * @param   {string}    message    The description of the error condition.
             * @return  {void}
             */
            $ctor1: function (message) {
                this.$initialize();
                System.SystemException.$ctor1.call(this, message);
            },
            /**
             * Initializes a new instance of the System.ServiceModel.CommunicationException
             class, using the specified message and the inner exception.
             *
             * @instance
             * @public
             * @this System.ServiceModel.CommunicationException
             * @memberof System.ServiceModel.CommunicationException
             * @param   {string}              message           The description of the error condition.
             * @param   {System.Exception}    innerException    The inner exception to be used.
             * @return  {void}
             */
            $ctor2: function (message, innerException) {
                this.$initialize();
                System.SystemException.$ctor2.call(this, message, innerException);
            }
        }
    });

    /** @namespace System.Threading */

    /**
     * @memberof System.Threading
     * @callback System.Threading.SendOrPostCallback
     * @param   {System.Object}    state
     * @return  {void}
     */

    /**
     * @memberof System.ServiceModel
     * @callback System.ServiceModel.CSHTML5_ClientBase.EndOperationDelegate
     * @param   {System.IAsyncResult}      result
     * @return  {Array.<System.Object>}
     */

    /**
     * @memberof System.ServiceModel
     * @callback System.ServiceModel.CSHTML5_ClientBase.BeginOperationDelegate
     * @param   {Array.<System.Object>}    inValues         
     * @param   {System.AsyncCallback}     asyncCallback    
     * @param   {System.Object}            state
     * @return  {System.IAsyncResult}
     */

    /**
     * @memberof System
     * @callback System.AsyncCallback
     * @param   {System.IAsyncResult}    ar
     * @return  {void}
     */

    /**
     * Provides the base implementation used to create Windows Communication Foundation
     (WCF) client objects that can call services.
     *
     * @abstract
     * @public
     * @class System.ServiceModel.CSHTML5_ClientBase$1
     * @example Here is an example on how you can use a WebService to receive data:
 
     Here is another example that shows how you can send data to a WebService:
     *
     * //We create a new instance of the ServiceClient
     * MyServiceClient soapClient =
     *     new MyServiceClient(
     *         new BasicHttpBinding(),
     *         new EndpointAddress(
     *             new Uri("http://MyServiceAddress.com/MyService.svc")));
     * //We call the method that will give us the data we want:
     * var result = await soapClient.GetToDosAsync(_ownerId);
     * //We get the data from the response:
     * ToDoItem[] todos = result.Body.GetToDosResult;
     * 
     *
     * //We create an item to send to the WebService:
     * ToDoItem todo = new ToDoItem()
     * {
     *     Description = MyTextBox.Text,
     *     Id = Guid.NewGuid(),
     * };
     * //We create a new instance of the ServiceClient
     * MyServiceClient soapClient =
     *     new MyServiceClient(
     *         new BasicHttpBinding(),
     *         new EndpointAddress(
     *             new Uri("http://MyServiceAddress.com/MyService.svc")));
     * //We send the data by calling a method implemented for that purpose in the WebService:
     * await soapClient.AddOrUpdateToDoAsync(todo);
     * 
     *
     *
     * @param   {Function}    [name]    The channel to be used to connect to the service.
     */
    Bridge.define("System.ServiceModel.CSHTML5_ClientBase$1", function (TChannel) { return {
        statics: {
            methods: {
                OnAsyncCallCompleted: function (result) {
                    var context = Bridge.cast(result.System$IAsyncResult$AsyncState, System.ServiceModel.CSHTML5_ClientBase$1.AsyncOperationContext(TChannel));
                    var error = null;
                    var results = null;
                    try {
                        results = context.EndDelegate(result);
                    } catch (e) {
                        e = System.Exception.create(e);
                        error = e;
                    }
                    System.ServiceModel.CSHTML5_ClientBase$1(TChannel).CompleteAsyncCall(context, results, error);
                },
                CompleteAsyncCall: function (context, results, error) {
                    if (!Bridge.staticEquals(context.CompletionCallback, null)) {
                        var e = new (System.ServiceModel.CSHTML5_ClientBase$1.InvokeAsyncCompletedEventArgs(TChannel))(results, error, false, null);
                        context.CompletionCallback(e);
                    }
                },
                TryReadEndpoint: function (configFileContent, fileName, contractConfigurationName, throwIfFileNotFound, endpointAddress) {
                    var $t;
                    var isNullOrUndefined = System.Convert.toBoolean(configFileContent == undefined || configFileContent == null);
                    if (!isNullOrUndefined) {
                        var fileContentAsString = System.Convert.toString(configFileContent);

                        var doc = System.Xml.Linq.XDocument.Parse(fileContentAsString);

                        $t = Bridge.getEnumerator(doc.Descendants$1(System.Xml.Linq.XName.op_Implicit("endpoint")), System.Xml.Linq.XElement);
                        try {
                            while ($t.moveNext()) {
                                var endpointXElement = $t.Current;
                                var contractXAttribute = System.Linq.Enumerable.from(endpointXElement.Attributes$1(System.Xml.Linq.XName.op_Implicit("contract"))).firstOrDefault(null, null);
                                if (contractXAttribute != null && Bridge.referenceEquals(contractXAttribute.Value, contractConfigurationName)) {
                                    var addressXAttribute = System.Linq.Enumerable.from(endpointXElement.Attributes$1(System.Xml.Linq.XName.op_Implicit("address"))).firstOrDefault(null, null);
                                    if (addressXAttribute != null) {
                                        endpointAddress.v = addressXAttribute.Value;
                                        return true;
                                    }
                                }
                            }
                        } finally {
                            if (Bridge.is($t, System.IDisposable)) {
                                $t.System$IDisposable$Dispose();
                            }
                        }

                        endpointAddress.v = null;
                        return false;
                    } else {

                        if (throwIfFileNotFound) {
                            throw new System.IO.FileNotFoundException.$ctor1(System.String.format("Could not find the '{0}' file to get the default endpoint. Please make sure that you have added your service reference through 'Add service reference' or use the constructor that allows you to set the Binding and Endpoint programmatically.\nSee http://www.cshtml5.com/links/wcf-limitations-and-tutorials.aspx for details.", [fileName]));
                        } else {
                            endpointAddress.v = null;
                            return false;
                        }
                    }
                }
            }
        },
        fields: {
            _remoteAddressAsString: null,
            channel: Bridge.getDefaultValue(TChannel)
        },
        props: {
            Channel: {
                get: function () {
                    return this.channel;
                }
            },
            INTERNAL_RemoteAddressAsString: {
                get: function () {
                    return this._remoteAddressAsString;
                }
            }
        },
        ctors: {
            /**
             * Initializes a new instance of the System.ServiceModel.ClientBase`1
             class using the default target endpoint from the application configuration
             file.
             *
             * @instance
             * @protected
             * @this System.ServiceModel.CSHTML5_ClientBase$1
             * @memberof System.ServiceModel.CSHTML5_ClientBase$1
             * @throws Either there is no default endpoint information in the configuration file,
             more than one endpoint in the file, or no configuration file.
             * @return  {void}
             */
            ctor: function () {
                var $t, $t1;
                this.$initialize();
                var contractConfigurationName = null;
                var interfacetype = ($t = Bridge.Reflection.getGenericArguments(Bridge.Reflection.getBaseType(Bridge.getType(this))))[System.Array.index(0, $t)];
                $t1 = Bridge.getEnumerator(Bridge.Reflection.getAttributes(interfacetype, System.ServiceModel.ServiceContractAttribute, false));
                try {
                    while ($t1.moveNext()) {
                        var attr = Bridge.cast($t1.Current, System.ServiceModel.ServiceContractAttribute);
                        if (attr.ConfigurationName != null) {
                            contractConfigurationName = attr.ConfigurationName;
                            break;
                        }
                    }
                } finally {
                    if (Bridge.is($t1, System.IDisposable)) {
                        $t1.System$IDisposable$Dispose();
                    }
                }
                if (contractConfigurationName == null) {
                    throw new System.Exception(System.String.format("Could not find a suitable ServiceContractAttribute to get the default endpoint in the type: '{0}'.", [Bridge.Reflection.getTypeFullName(interfacetype)]));
                }

                var endpointAddress = { };
                var serviceReferencesClientConfig = window.ServiceReferencesClientConfig;
                if (System.ServiceModel.CSHTML5_ClientBase$1(TChannel).TryReadEndpoint(serviceReferencesClientConfig, "ServiceReferences.ClientConfig", contractConfigurationName, false, endpointAddress)) {
                    this._remoteAddressAsString = endpointAddress.v;
                } else {
                    var appConfig = window.AppConfig;
                    if (System.ServiceModel.CSHTML5_ClientBase$1(TChannel).TryReadEndpoint(appConfig, "App.Config", contractConfigurationName, true, endpointAddress)) {
                        this._remoteAddressAsString = endpointAddress.v;
                    } else {
                        throw new System.Exception(System.String.format("Could not find the default WCF endpoint element that references the contract '{0}' in the ServiceModel client configuration section.", [contractConfigurationName]));
                    }
                }
            },
            /**
             * Initializes a new instance of the System.ServiceModel.ClientBase`1
             class using the configuration information specified in the application configuration
             file by endpointConfigurationName.
             *
             * @instance
             * @protected
             * @this System.ServiceModel.CSHTML5_ClientBase$1
             * @memberof System.ServiceModel.CSHTML5_ClientBase$1
             * @throws The specified endpoint information is null.
             * @throws The endpoint cannot be found or the endpoint contract is not valid.
             * @param   {string}    endpointConfigurationName    The name of the endpoint in the application configuration file.
             * @return  {void}
             */
            $ctor2: function (endpointConfigurationName) {
                this.$initialize();
                throw new System.NotSupportedException.$ctor1("Please specify the Binding and Endpoint programmatically. See http://www.cshtml5.com/links/wcf-limitations-and-tutorials.aspx for details.");
            },
            /**
             * Initializes a new instance of the System.ServiceModel.ClientBase`1
             class using the specified binding and target address.
             *
             * @instance
             * @protected
             * @this System.ServiceModel.CSHTML5_ClientBase$1
             * @memberof System.ServiceModel.CSHTML5_ClientBase$1
             * @throws The binding is null.
             * @throws The remote address is null.
             * @param   {System.ServiceModel.Channels.Binding}    binding          The binding with which to make calls to the service.
             * @param   {System.ServiceModel.EndpointAddress}     remoteAddress    The address of the service endpoint.
             * @return  {void}
             */
            $ctor1: function (binding, remoteAddress) {
                this.$initialize();
                if (remoteAddress == null) {
                    throw new System.ArgumentNullException.$ctor1("remoteAddress");
                }

                this._remoteAddressAsString = remoteAddress.Uri.getOriginalString();

            },
            /**
             * Initializes a new instance of the System.ServiceModel.ClientBase`1
             class using the specified target address and endpoint information.
             *
             * @instance
             * @protected
             * @this System.ServiceModel.CSHTML5_ClientBase$1
             * @memberof System.ServiceModel.CSHTML5_ClientBase$1
             * @throws The endpoint is null.
             * @throws The remote address is null.
             * @throws The endpoint cannot be found or the endpoint contract is not valid.
             * @param   {string}                                 endpointConfigurationName    The name of the endpoint in the application configuration file.
             * @param   {System.ServiceModel.EndpointAddress}    remoteAddress                The address of the service.
             * @return  {void}
             */
            $ctor3: function (endpointConfigurationName, remoteAddress) {
                this.$initialize();
                throw new System.NotSupportedException.$ctor1("Please specify the Binding and Endpoint programmatically. See http://www.cshtml5.com/links/wcf-limitations-and-tutorials.aspx for details.");
            },
            /**
             * Initializes a new instance of the System.ServiceModel.ClientBase`1
             class.
             *
             * @instance
             * @protected
             * @this System.ServiceModel.CSHTML5_ClientBase$1
             * @memberof System.ServiceModel.CSHTML5_ClientBase$1
             * @throws The endpoint is null.
             * @throws The remote address is null.
             * @throws The endpoint cannot be found or the endpoint contract is not valid.
             * @param   {string}    endpointConfigurationName    The name of the endpoint in the application configuration file.
             * @param   {string}    remoteAddress                The address of the service.
             * @return  {void}
             */
            $ctor4: function (endpointConfigurationName, remoteAddress) {
                this.$initialize();
                throw new System.NotSupportedException.$ctor1("Please specify the Binding and Endpoint programmatically. See http://www.cshtml5.com/links/wcf-limitations-and-tutorials.aspx for details.");
            }
        },
        methods: {
            /**
             * Provides support for implementing the event-based asynchronous pattern.
             *
             * @instance
             * @protected
             * @this System.ServiceModel.CSHTML5_ClientBase$1
             * @memberof System.ServiceModel.CSHTML5_ClientBase$1
             * @param   {System.ServiceModel.CSHTML5_ClientBase.BeginOperationDelegate}    beginOperationDelegate        A delegate that is used for calling the asynchronous operation.
             * @param   {Array.<System.Object>}                                            inValues                      The input values to the asynchronous call.
             * @param   {System.ServiceModel.CSHTML5_ClientBase.EndOperationDelegate}      endOperationDelegate          A delegate that is used to end the asynchronous call after it has completed.
             * @param   {System.Threading.SendOrPostCallback}                              operationCompletedCallback    A client-supplied callback that is invoked when the asynchronous method is
             complete. The callback is passed to the BeginOperationDelegate.
             * @param   {System.Object}                                                    userState                     The userState object to associate with the asynchronous call.
             * @return  {void}
             */
            InvokeAsync: function (beginOperationDelegate, inValues, endOperationDelegate, operationCompletedCallback, userState) {
                var context = new (System.ServiceModel.CSHTML5_ClientBase$1.AsyncOperationContext(TChannel))(endOperationDelegate, operationCompletedCallback);
                var result = beginOperationDelegate(inValues, System.ServiceModel.CSHTML5_ClientBase$1(TChannel).OnAsyncCallCompleted, context);
            }
        }
    }; });

    Bridge.define("System.ServiceModel.CSHTML5_ClientBase$1.AsyncOperationContext", function (TChannel) { return {
        $kind: "nested class",
        fields: {
            endDelegate: null,
            completionCallback: null
        },
        props: {
            EndDelegate: {
                get: function () {
                    return this.endDelegate;
                }
            },
            CompletionCallback: {
                get: function () {
                    return this.completionCallback;
                }
            }
        },
        ctors: {
            ctor: function (endDelegate, completionCallback) {
                this.$initialize();
                this.endDelegate = endDelegate;
                this.completionCallback = completionCallback;
            }
        }
    }; });

    /**
     * Provides an API to call web methods defined in a WebService
     *
     * @public
     * @class System.ServiceModel.CSHTML5_ClientBase$1.WebMethodsCaller
     */
    Bridge.define("System.ServiceModel.CSHTML5_ClientBase$1.WebMethodsCaller", function (TChannel) { return {
        $kind: "nested class",
        statics: {
            methods: {
                IsXmlSerializer: function (webMethodName, methodReturnType, method) {
                    if (methodReturnType != null) {
                        if (Bridge.referenceEquals(Bridge.Reflection.getTypeName(methodReturnType), (webMethodName || "") + "Response") && Bridge.Reflection.getMembers(methodReturnType, 4, 284, "Body") != null) {
                            return true;
                        }
                    }

                    if (method != null) {
                        var parameterInfos = (method.pi || []);
                        if (Bridge.referenceEquals(methodReturnType, System.IAsyncResult) && (parameterInfos != null && parameterInfos.length > 0) && Bridge.referenceEquals(Bridge.Reflection.getTypeName(parameterInfos[System.Array.index(0, parameterInfos)].pt), (webMethodName || "") + "Request")) {
                            return true;
                        }
                    }

                    return false;
                },
                VerifyThatResponseIsNotNullOrEmpty: function (responseAsString) {
                    if (System.String.isNullOrEmpty(responseAsString)) {
                        throw new System.ServiceModel.CommunicationException.$ctor1("The remote server returned an error. To debug, look at the browser Console output, or use a tool such as Fiddler.");
                    }
                }
            }
        },
        fields: {
            _addressOfService: null,
            _webRequestHelper_JSVersion: null
        },
        ctors: {
            init: function () {
                this._webRequestHelper_JSVersion = new System.INTERNAL_WebRequestHelper_JSOnly();
            },
            /**
             * Constructor for the WebMethodsCaller's class
             *
             * @instance
             * @public
             * @this System.ServiceModel.CSHTML5_ClientBase$1.WebMethodsCaller
             * @memberof System.ServiceModel.CSHTML5_ClientBase$1.WebMethodsCaller
             * @param   {string}    addressOfService    The address of the WebService
             * @return  {void}
             */
            ctor: function (addressOfService) {
                this.$initialize();
                this._addressOfService = addressOfService;
            }
        },
        methods: {
            BeginCallWebMethod: function (webMethodName, interfaceType, methodReturnType, originalRequestObject, callback, soapVersion) {
                var isXmlSerializer = { };
                var headers = { };
                var request = { };
                this.PrepareRequest(webMethodName, "Begin" + (webMethodName || ""), interfaceType, methodReturnType, originalRequestObject, soapVersion, isXmlSerializer, headers, request);

                var address = new System.Uri(this._addressOfService);

                this._webRequestHelper_JSVersion.MakeRequest(address, "POST", this, headers.v, request.v, function (sender, e) {
                    var xmlReturnedFromTheServer = e.Result;
                    callback(xmlReturnedFromTheServer);
                }, true, Windows.UI.Xaml.Application.Current.Host.Settings.DefaultSoapCredentialsMode);
            },
            EndCallWebMethod$1: function (webMethodName, interfaceType, methodReturnType, xmlReturnedFromTheServer, soapVersion) {
                var beginMethod = Bridge.Reflection.getMembers(interfaceType, 8, 284, "Begin" + (webMethodName || ""));

                var isXmlSerializer = System.ServiceModel.CSHTML5_ClientBase$1.WebMethodsCaller(TChannel).IsXmlSerializer(webMethodName, methodReturnType, beginMethod);

                var requestResponse = this.ReadAndPrepareResponse(xmlReturnedFromTheServer, interfaceType, methodReturnType, function (faultException) {
                    throw faultException;
                }, isXmlSerializer, soapVersion);

                return requestResponse;
            },
            EndCallWebMethod: function (RETURN_TYPE, webMethodName, interfaceType, xmlReturnedFromTheServer, soapVersion) {
                return Bridge.cast(Bridge.unbox(this.EndCallWebMethod$1(webMethodName, interfaceType, RETURN_TYPE, xmlReturnedFromTheServer, soapVersion), RETURN_TYPE), RETURN_TYPE);
            },
            CallWebMethodAsyncBeginEnd: function (T, webMethodName, interfaceType, methodReturnType, originalRequestObject, soapVersion) {
                var tcs = new System.Threading.Tasks.TaskCompletionSource();

                var callback = Bridge.fn.bind(this, function (asyncResponseResult) {
                    try {
                        var result = this.EndCallWebMethod(T, webMethodName, interfaceType, Bridge.cast(asyncResponseResult, System.ServiceModel.INTERNAL_WebMethodsCaller.WebMethodAsyncResult).XmlReturnedFromTheServer, soapVersion);
                        tcs.setResult(result);
                    } catch (ex) {
                        ex = System.Exception.create(ex);
                        tcs.trySetException(ex);
                    }
                });
                var asyncState = null;

                var webMethodAsyncResult = new System.ServiceModel.INTERNAL_WebMethodsCaller.WebMethodAsyncResult(callback, asyncState);

                this.BeginCallWebMethod(webMethodName, interfaceType, methodReturnType, originalRequestObject, function (xmlReturnedFromTheServer) {
                    webMethodAsyncResult.XmlReturnedFromTheServer = xmlReturnedFromTheServer;

                    webMethodAsyncResult.Completed();
                }, soapVersion);

                return tcs.task;
            },
            /**
             * Asynchronously calls a WebMethod.
             *
             * @instance
             * @public
             * @this System.ServiceModel.CSHTML5_ClientBase$1.WebMethodsCaller
             * @memberof System.ServiceModel.CSHTML5_ClientBase$1.WebMethodsCaller
             * @param   {Function}                                    T                        The return type of the WebMethod
             * @param   {string}                                      webMethodName            The name of the WebMethod
             * @param   {System.Type}                                 interfaceType            The Type of the interface
             * @param   {System.Type}                                 methodReturnType         The return Type of the method
             * @param   {System.Collections.Generic.IDictionary$2}    originalRequestObject    The additional arguments of the method
             * @param   {string}                                      soapVersion
             * @return  {System.Threading.Tasks.Task$1}                                        The result of the call of the method.
             */
            CallWebMethodAsync: function (T, webMethodName, interfaceType, methodReturnType, originalRequestObject, soapVersion) {
                var isXmlSerializer = { };
                var headers = { };
                var request = { };

                this.PrepareRequest(webMethodName, (webMethodName || "") + "Async", interfaceType, methodReturnType, originalRequestObject, soapVersion, isXmlSerializer, headers, request);

                var tcs = new System.Threading.Tasks.TaskCompletionSource();

                var response = this._webRequestHelper_JSVersion.MakeRequest(new System.Uri(this._addressOfService), "POST", this, headers.v, request.v, Bridge.fn.bind(this, function (sender, args2) {
                    this.ReadAndPrepareResponseGeneric_JSVersion(T, tcs, args2, interfaceType, methodReturnType, isXmlSerializer.v, soapVersion);
                }), true, Windows.UI.Xaml.Application.Current.Host.Settings.DefaultSoapCredentialsMode);

                return tcs.task;
            },
            /**
             * Calls a WebMethod
             *
             * @instance
             * @public
             * @this System.ServiceModel.CSHTML5_ClientBase$1.WebMethodsCaller
             * @memberof System.ServiceModel.CSHTML5_ClientBase$1.WebMethodsCaller
             * @param   {string}                                      webMethodName            The name of the Method
             * @param   {System.Type}                                 interfaceType            The Type of the interface
             * @param   {System.Type}                                 methodReturnType         
             * @param   {System.Collections.Generic.IDictionary$2}    originalRequestObject    
             * @param   {string}                                      soapVersion
             * @return  {System.Object}                                                        The result of the call of the method.
             */
            CallWebMethod: function (webMethodName, interfaceType, methodReturnType, originalRequestObject, soapVersion) {



                var isXmlSerializer = { };
                var headers = { };
                var request = { };

                this.PrepareRequest(webMethodName, webMethodName, interfaceType, methodReturnType, originalRequestObject, soapVersion, isXmlSerializer, headers, request);

                var response = this._webRequestHelper_JSVersion.MakeRequest(new System.Uri(this._addressOfService), "POST", this, headers.v, request.v, null, false, Windows.UI.Xaml.Application.Current.Host.Settings.DefaultSoapCredentialsMode);

                return this.ReadAndPrepareResponse(response, interfaceType, methodReturnType, function (faultException) {
                    throw faultException;
                }, isXmlSerializer.v, soapVersion);
            },
            PrepareRequest: function (webMethodName, methodName, interfaceType, methodReturnType, requestParameters, soapVersion, isXmlSerializer, headers, request) {
                var $t, $t1, $t2, $t3;
                headers.v = new (System.Collections.Generic.Dictionary$2(System.String,System.String)).ctor();
                var requestFormat = null;

                var interfaceTypeName = Bridge.Reflection.getTypeName(interfaceType);
                var interfaceTypeNamespace = "http://tempuri.org/";
                var soapAction = "";

                var method = Bridge.Reflection.getMembers(interfaceType, 8, 284, methodName);
                if (method == null) {
                    throw new System.MissingMethodException.$ctor1(System.String.format("Method '{0}' was not in found in interface '{1}'.", methodName, Bridge.Reflection.getTypeName(interfaceType)));
                }
                isXmlSerializer.v = System.ServiceModel.CSHTML5_ClientBase$1.WebMethodsCaller(TChannel).IsXmlSerializer(webMethodName, methodReturnType, method);

                var serviceContractAttr = Bridge.cast(System.Linq.Enumerable.from(Bridge.Reflection.getAttributes(interfaceType, System.ServiceModel.ServiceContractAttribute, false)).firstOrDefault(null, null), System.ServiceModel.ServiceContractAttribute);

                if (serviceContractAttr != null) {
                    if (!System.String.isNullOrEmpty(serviceContractAttr.Namespace) && !Bridge.referenceEquals(serviceContractAttr.Namespace, "http://tempuri.org")) {
                        interfaceTypeNamespace = serviceContractAttr.Namespace;
                    }
                    if (!System.String.isNullOrEmpty(serviceContractAttr.Name)) {
                        interfaceTypeName = serviceContractAttr.Name;
                    }
                }

                switch (soapVersion) {
                    case "1.1": 
                        var operationContractAttr = Bridge.cast(System.Linq.Enumerable.from(System.Attribute.getCustomAttributes(method, System.ServiceModel.OperationContractAttribute, false)).firstOrDefault(null, null), System.ServiceModel.OperationContractAttribute);
                        if (operationContractAttr != null) {
                            soapAction = operationContractAttr.Action;
                        }
                        if (System.String.isNullOrEmpty(soapAction)) {
                            soapAction = System.String.format("{0}/{1}/{2}", System.String.trim(interfaceTypeNamespace, [47]), System.String.trim(interfaceTypeName, [47]), webMethodName);
                        }
                        headers.v.add("Content-Type", "text/xml; charset=utf-8");
                        headers.v.add("SOAPAction", soapAction);
                        requestFormat = "<s:Envelope xmlns:s=\"http://schemas.xmlsoap.org/soap/envelope/\"><s:Body>{0}</s:Body></s:Envelope>";
                        break;
                    case "1.2": 
                        headers.v.add("Content-Type", "application/soap+xml; charset=utf-8");
                        soapAction = System.String.format("http://tempuri.org/ServiceHost/{0}", [webMethodName]);
                        requestFormat = System.String.format("<s:Envelope xmlns:a=\"http://www.w3.org/2005/08/addressing\" xmlns:s=\"http://www.w3.org/2003/05/soap-envelope\"><s:Header><a:Action>{0}</a:Action></s:Header><s:Body>{{0}}</s:Body></s:Envelope>", [soapAction]);
                        break;
                    default: 
                        throw new System.InvalidOperationException.$ctor1(System.String.format("SOAP version not supported: {0}", [soapVersion]));
                }

                var methodNameElement = new System.Xml.Linq.XElement.$ctor2(System.Xml.Linq.XNamespace.Get(interfaceTypeNamespace).GetName(webMethodName));

                if (requestParameters != null) {
                    var parameterInfos = (method.pi || []);
                    var parametersCount = requestParameters != null ? System.Array.getCount(requestParameters, System.Collections.Generic.KeyValuePair$2(System.String,System.Object)) : 0;

                    for (var i = 0; i < parametersCount; i = (i + 1) | 0) {
                        var requestBody = requestParameters.System$Collections$Generic$IDictionary$2$System$String$System$Object$getItem(parameterInfos[System.Array.index(i, parameterInfos)].n);
                        if (requestBody != null) {
                            var knownTypes = System.Linq.Enumerable.from(Bridge.Reflection.getAttributes(interfaceType, System.ServiceModel.ServiceKnownTypeAttribute, true)).select(function (o) {
                                    return Bridge.cast(o, System.ServiceModel.ServiceKnownTypeAttribute).Type;
                                });

                            var dataContractSerializer = new System.Runtime.Serialization.DataContractSerializer.$ctor1(Bridge.getType(requestBody), knownTypes, isXmlSerializer.v);

                            var xdoc = dataContractSerializer.SerializeToXDocument(requestBody);

                            var paramNameElement = new System.Xml.Linq.XElement.$ctor2(System.Xml.Linq.XNamespace.Get(interfaceTypeNamespace).GetName(parameterInfos[System.Array.index(i, parameterInfos)].n));
                            if (!isXmlSerializer.v) {
                                methodNameElement.Add(paramNameElement);
                                $t = Bridge.getEnumerator(xdoc.Root.Nodes(), System.Xml.Linq.XNode);
                                try {
                                    while ($t.moveNext()) {
                                        var currentNode = $t.Current;
                                        paramNameElement.Add(currentNode);
                                    }
                                } finally {
                                    if (Bridge.is($t, System.IDisposable)) {
                                        $t.System$IDisposable$Dispose();
                                    }
                                }
                                $t1 = Bridge.getEnumerator(xdoc.Root.Attributes(), System.Xml.Linq.XAttribute);
                                try {
                                    while ($t1.moveNext()) {
                                        var currentAttribute = $t1.Current;
                                        if (!Bridge.referenceEquals(currentAttribute.Name.LocalName, "xmlns")) {
                                            paramNameElement.Add(currentAttribute);
                                        }
                                    }
                                } finally {
                                    if (Bridge.is($t1, System.IDisposable)) {
                                        $t1.System$IDisposable$Dispose();
                                    }
                                }
                            } else {
                                $t2 = Bridge.getEnumerator(xdoc.Root.Nodes(), System.Xml.Linq.XNode);
                                try {
                                    while ($t2.moveNext()) {
                                        var currentNode1 = $t2.Current;
                                        var xElement = Bridge.as(currentNode1, System.Xml.Linq.XElement);
                                        if (xElement != null) {
                                            $t3 = Bridge.getEnumerator(xElement.Nodes(), System.Xml.Linq.XNode);
                                            try {
                                                while ($t3.moveNext()) {
                                                    var node = $t3.Current;
                                                    methodNameElement.Add(node);
                                                }
                                            } finally {
                                                if (Bridge.is($t3, System.IDisposable)) {
                                                    $t3.System$IDisposable$Dispose();
                                                }
                                            }
                                        }
                                    }
                                } finally {
                                    if (Bridge.is($t2, System.IDisposable)) {
                                        $t2.System$IDisposable$Dispose();
                                    }
                                }
                            }
                        } else {
                            var paramNameElement1 = new System.Xml.Linq.XElement.$ctor2(System.Xml.Linq.XNamespace.Get(interfaceTypeNamespace).GetName(parameterInfos[System.Array.index(i, parameterInfos)].n));
                            var attribute = new System.Xml.Linq.XAttribute.$ctor1(System.Xml.Linq.XNamespace.Get(System.Runtime.Serialization.DataContractSerializer_Helpers.XMLSCHEMA_NAMESPACE).GetName("nil"), "true");
                            paramNameElement1.Add(attribute);
                            methodNameElement.Add(paramNameElement1);
                        }
                    }
                }

                request.v = System.String.format(requestFormat, [methodNameElement.ToString(false)]);
            },
            ReadAndPrepareResponseGeneric_JSVersion: function (T, taskCompletionSource, e, interfaceType, requestResponseType, isXmlSerializer, soapVersion) {
                if (e.Error == null) {
                    var requestResponse = Bridge.cast(Bridge.unbox(this.ReadAndPrepareResponse(e.Result, interfaceType, requestResponseType, function (faultException) {
                        taskCompletionSource.trySetException(faultException);
                    }, isXmlSerializer, soapVersion), T), T);

                    if (!taskCompletionSource.task.isCompleted()) {
                        taskCompletionSource.setResult(requestResponse);
                    }
                } else {
                    taskCompletionSource.trySetException(e.Error);
                }
            },
            ReadAndPrepareResponse: function (responseAsString, interfaceType, requestResponseType, raiseFaultException, isXmlSerializer, soapVersion) {

                var m = System.String.indexOf(responseAsString, ":Fault>");
                if (m === -1) {
                    m = System.String.indexOf(responseAsString, "<Fault>");
                }
                if (m !== -1) {
                    m = System.String.indexOf(responseAsString, "<faultstring", m);
                    if (m !== -1) {
                        m = System.String.indexOf(responseAsString, String.fromCharCode(62), m);
                        responseAsString = System.String.remove(responseAsString, 0, ((m + 1) | 0));
                        m = System.String.indexOf(responseAsString, "</faultstring");
                        responseAsString = System.String.remove(responseAsString, m);
                        var fe = new System.ServiceModel.FaultException.$ctor1(responseAsString);
                        raiseFaultException(fe);
                        return null;
                    }
                }

                var requestResponse = null;

                if (!Bridge.Reflection.isValueType(requestResponseType)) {
                    var bodyFieldInfo = null;

                    if (Bridge.referenceEquals(requestResponseType, System.Object)) {
                        System.ServiceModel.CSHTML5_ClientBase$1.WebMethodsCaller(TChannel).VerifyThatResponseIsNotNullOrEmpty(responseAsString);
                        var doc = System.Xml.Linq.XDocument.Parse(responseAsString);
                        var currentElement = doc.Root;
                        currentElement = Bridge.cast(currentElement.FirstNode, System.Xml.Linq.XElement);
                        currentElement = Bridge.cast(currentElement.FirstNode, System.Xml.Linq.XElement);
                        if (System.Linq.Enumerable.from(currentElement.Nodes()).count() === 0) {
                            return null;
                        }
                    }

                    var typeToDeserialize = requestResponseType;
                    if (isXmlSerializer) {
                        bodyFieldInfo = Bridge.Reflection.getMembers(requestResponseType, 4, 284, "Body");
                        typeToDeserialize = bodyFieldInfo.rt;
                    }

                    var knownTypes = System.Linq.Enumerable.from(Bridge.Reflection.getAttributes(interfaceType, System.ServiceModel.ServiceKnownTypeAttribute, true)).select(function (o) {
                            return Bridge.cast(o, System.ServiceModel.ServiceKnownTypeAttribute).Type;
                        });

                    var deSerializer = new System.Runtime.Serialization.DataContractSerializer.$ctor1(typeToDeserialize, knownTypes);
                    System.ServiceModel.CSHTML5_ClientBase$1.WebMethodsCaller(TChannel).VerifyThatResponseIsNotNullOrEmpty(responseAsString);
                    var xDoc = System.Xml.Linq.XDocument.Parse(responseAsString);
                    responseAsString = this.RemoveUnparsableStrings(responseAsString);
                    var xElement = xDoc.Root;

                    if (Bridge.referenceEquals(soapVersion, "1.1")) {
                        xElement = System.Linq.Enumerable.from(xElement.Elements()).firstOrDefault(null, null) || xElement;
                    } else {
                        System.Diagnostics.Debug.Assert$1(Bridge.referenceEquals(soapVersion, "1.2"), System.String.format("Unexpected soap version ({0}) !", [soapVersion]));
                        xElement = xElement.Element(System.Xml.Linq.XName.Get$1("Body", "http://www.w3.org/2003/05/soap-envelope"));
                    }

                    var isTypeSpecified = System.Linq.Enumerable.from(xElement.Attributes$1(System.Xml.Linq.XNamespace.Get("http://www.w3.org/2001/XMLSchema-instance").GetName("type"))).any();
                    if (!isXmlSerializer || !isTypeSpecified) {
                        xElement = System.Linq.Enumerable.from(xElement.Elements()).firstOrDefault(null, null) || xElement;
                    }

                    if (!isXmlSerializer) {
                        xElement = System.Linq.Enumerable.from(xElement.Elements()).firstOrDefault(null, null) || xElement;
                        requestResponse = deSerializer.DeserializeFromXElement(xElement);
                    } else {
                        requestResponse = Bridge.createInstance(requestResponseType);
                        var requestResponseBody = deSerializer.DeserializeFromXElement(xElement);
                        Bridge.Reflection.fieldAccess(bodyFieldInfo, Bridge.unbox(requestResponse), Bridge.unbox(requestResponseBody));
                    }
                } else {
                    var keyWord = ":Body";
                    var i = System.String.indexOf(responseAsString, keyWord);
                    var k = System.String.indexOf(responseAsString, String.fromCharCode(62), i);
                    responseAsString = System.String.remove(responseAsString, 0, ((k + 1) | 0));
                    i = responseAsString.lastIndexOf(keyWord);
                    k = responseAsString.substr(0, i).lastIndexOf(String.fromCharCode(60));
                    responseAsString = System.String.remove(responseAsString, k);



                    var indexOfNil = System.String.indexOf(responseAsString, ":nil=\"true\"");
                    if (indexOfNil === -1) {
                        var ii = System.String.indexOf(responseAsString, String.fromCharCode(62), 0);
                        var jj = System.String.indexOf(responseAsString, String.fromCharCode(62), ((ii + 1) | 0));
                        var kk = System.String.indexOf(responseAsString, String.fromCharCode(60), ((jj + 1) | 0));
                        responseAsString = responseAsString.substr(((jj + 1) | 0), (((((kk - jj) | 0) - 1) | 0)));

                        if (Bridge.referenceEquals(requestResponseType, System.Int32) || Bridge.referenceEquals(requestResponseType, System.Nullable$1(System.Int32))) {
                            requestResponse = Bridge.box(System.Int32.parse(responseAsString), System.Int32);
                        } else {
                            if (Bridge.referenceEquals(requestResponseType, System.Int64) || Bridge.referenceEquals(requestResponseType, System.Nullable$1(System.Int64))) {
                                requestResponse = System.Int64.parse(responseAsString);
                            } else {
                                if (Bridge.referenceEquals(requestResponseType, System.Boolean) || Bridge.referenceEquals(requestResponseType, System.Nullable$1(System.Boolean))) {
                                    requestResponse = Bridge.box(System.Boolean.parse(responseAsString), System.Boolean, System.Boolean.toString);
                                } else {
                                    if (Bridge.referenceEquals(requestResponseType, System.Single) || Bridge.referenceEquals(requestResponseType, System.Nullable$1(System.Single))) {
                                        requestResponse = Bridge.box(System.Single.parse(responseAsString), System.Single, System.Single.format, System.Single.getHashCode);
                                    } else {
                                        if (Bridge.referenceEquals(requestResponseType, System.Double) || Bridge.referenceEquals(requestResponseType, System.Nullable$1(System.Double))) {
                                            requestResponse = Bridge.box(System.Double.parse(responseAsString), System.Double, System.Double.format, System.Double.getHashCode);
                                        } else {
                                            if (Bridge.referenceEquals(requestResponseType, System.Char) || Bridge.referenceEquals(requestResponseType, System.Nullable$1(System.Char))) {
                                                requestResponse = Bridge.box((System.Int32.parse(responseAsString)) & 65535, System.Char, String.fromCharCode, System.Char.getHashCode);
                                            } else {
                                                if (Bridge.referenceEquals(requestResponseType, System.DateTime) || Bridge.referenceEquals(requestResponseType, System.Nullable$1(System.DateTime))) {
                                                    requestResponse = Bridge.box(INTERNAL_DateTimeHelpers.ToDateTime(responseAsString), System.DateTime, System.DateTime.format);
                                                } else {
                                                    throw new System.NotSupportedException.$ctor1(System.String.format("The following type is not supported in the current WCF implementation: '{0}'. \nPlease report this issue to support@cshtml5.com", [requestResponseType]));
                                                }
                                            }
                                        }
                                    }
                                }
                            }
                        }
                    } else {
                        if (System.String.startsWith(Bridge.Reflection.getTypeFullName(requestResponseType), "System.Nullable`1")) {
                            return null;
                        } else {
                            return Bridge.createInstance(requestResponseType);
                        }
                    }
                }

                return requestResponse;
            },
            RemoveUnparsableStrings: function (str) {
                var $t;
                var unparsableStrings = System.Array.init(["&#x1A;"], System.String);

                $t = Bridge.getEnumerator(unparsableStrings);
                try {
                    while ($t.moveNext()) {
                        var unparsableString = $t.Current;
                        str = System.String.replaceAll(str, unparsableString, "");
                    }
                } finally {
                    if (Bridge.is($t, System.IDisposable)) {
                        $t.System$IDisposable$Dispose();
                    }
                }
                return str;
            }
        }
    }; });

    /**
     * Provides a unique network address that a client uses to communicate with
     a service endpoint.
     *
     * @public
     * @class System.ServiceModel.EndpointAddress
     */
    Bridge.define("System.ServiceModel.EndpointAddress", {
        fields: {
            _uri: null
        },
        props: {
            Uri: {
                get: function () {
                    return this._uri;
                },
                set: function (value) {
                    this._uri = value;
                }
            }
        },
        ctors: {
            /**
             * Initializes a new instance of the System.ServiceModel.EndpointAddress class
             with a specified URI and headers.
             *
             * @instance
             * @public
             * @this System.ServiceModel.EndpointAddress
             * @memberof System.ServiceModel.EndpointAddress
             * @param   {System.Uri}                                            uri               The System.Uri that identifies the endpoint location.
             * @param   {Array.<System.ServiceModel.Channels.AddressHeader>}    addressHeaders    The System.Array of type System.ServiceModel.Channels.AddressHeader that
             contains address information used to interact with the endpoint.
             * @return  {void}
             */
            $ctor1: function (uri, addressHeaders) {
                if (addressHeaders === void 0) { addressHeaders = []; }

                this.$initialize();
                this._uri = uri;
            },
            ctor: function (uri) {
                this.$initialize();
                var u = new System.Uri(uri);
                this._uri = u;
            }
        }
    });

    Bridge.define("System.ServiceModel.FaultContractAttribute", {
        inherits: [System.Attribute],
        statics: {
            fields: {
                ProtectionLevelPropertyName: null
            },
            ctors: {
                init: function () {
                    this.ProtectionLevelPropertyName = "ProtectionLevel";
                }
            }
        },
        fields: {
            action: null,
            name: null,
            ns: null,
            type: null,
            protectionLevel: 0,
            hasProtectionLevel: false
        },
        props: {
            DetailType: {
                get: function () {
                    return this.type;
                }
            },
            Action: {
                get: function () {
                    return this.action;
                },
                set: function (value) {
                    if (value == null) {
                        throw new System.ArgumentNullException.$ctor1("value");
                    }
                    this.action = value;
                }
            },
            Name: {
                get: function () {
                    return this.name;
                },
                set: function (value) {
                    if (value == null) {
                        throw new System.ArgumentNullException.$ctor1("value");
                    }
                    if (Bridge.referenceEquals(value, "")) {
                        throw new System.ArgumentOutOfRangeException.$ctor1("value");
                    }
                    this.name = value;
                }
            },
            Namespace: {
                get: function () {
                    return this.ns;
                },
                set: function (value) {
                    this.ns = value;
                }
            },
            ProtectionLevel: {
                get: function () {
                    return this.protectionLevel;
                },
                set: function (value) {
                    this.protectionLevel = value;
                    this.hasProtectionLevel = true;
                }
            },
            HasProtectionLevel: {
                get: function () {
                    return this.hasProtectionLevel;
                }
            }
        },
        ctors: {
            init: function () {
                this.protectionLevel = System.Net.Security.ProtectionLevel.None;
                this.hasProtectionLevel = false;
            },
            ctor: function (detailType) {
                this.$initialize();
                System.Attribute.ctor.call(this);
                if (detailType == null) {
                    throw new System.ArgumentNullException.$ctor1("detailType");
                }

                this.type = detailType;
            }
        }
    });

    /**
     * Represents a SOAP fault.
     *
     * @public
     * @class System.ServiceModel.FaultException
     * @augments System.Exception
     */
    Bridge.define("System.ServiceModel.FaultException", {
        inherits: [System.Exception],
        fields: {
            /**
             * @instance
             * @private
             * @memberof System.ServiceModel.FaultException
             * @type string
             */
            _message: null
        },
        props: {
            /**
             * Gets the message for the exception.
             *
             * @instance
             * @public
             * @override
             * @readonly
             * @memberof System.ServiceModel.FaultException
             * @function Message
             * @type string
             */
            Message: {
                get: function () {
                    return this._message;
                }
            }
        },
        ctors: {
            /**
             * Initializes a new instance of the System.ServiceModel.FaultException class.
             *
             * @instance
             * @public
             * @this System.ServiceModel.FaultException
             * @memberof System.ServiceModel.FaultException
             * @return  {void}
             */
            ctor: function () {
                this.$initialize();
                System.Exception.ctor.call(this);
            },
            /**
             * Initializes a new instance of the System.ServiceModel.FaultException class
             with the specified fault reason.
             *
             * @instance
             * @public
             * @this System.ServiceModel.FaultException
             * @memberof System.ServiceModel.FaultException
             * @param   {string}    reason    The reason for the fault.
             * @return  {void}
             */
            $ctor1: function (reason) {
                this.$initialize();
                System.Exception.ctor.call(this);
                this._message = reason;
            }
        }
    });

    /**
     * Defines the behavior of outbound request and request/reply channels used
     by client applications.
     *
     * @abstract
     * @public
     * @class System.ServiceModel.IClientChannel
     * @implements  System.IDisposable
     */
    Bridge.define("System.ServiceModel.IClientChannel", {
        inherits: [System.IDisposable],
        $kind: "interface"
    });

    Bridge.define("System.ServiceModel.INTERNAL_WebMethodsCaller", {
        statics: {
            methods: {
                CallWebMethodAsync: function (RETURN_TYPE, INTERFACE_TYPE, endpointAddress, webMethodName, requestParameters, soapVersion) {
                    var webMethodsCaller = new (System.ServiceModel.CSHTML5_ClientBase$1.WebMethodsCaller(INTERFACE_TYPE))(endpointAddress);

                    var task = null;
                    try {
                        task = webMethodsCaller.CallWebMethodAsync(RETURN_TYPE, webMethodName, INTERFACE_TYPE, RETURN_TYPE, requestParameters, soapVersion);
                    } catch ($e1) {
                        $e1 = System.Exception.create($e1);
                        if (Bridge.is($e1, System.MissingMethodException)) {
                            task = webMethodsCaller.CallWebMethodAsyncBeginEnd(RETURN_TYPE, webMethodName, INTERFACE_TYPE, RETURN_TYPE, requestParameters, soapVersion);
                        } else {
                            throw $e1;
                        }
                    }

                    return task;
                },
                CallWebMethod: function (RETURN_TYPE, INTERFACE_TYPE, endpointAddress, webMethodName, requestParameters, soapVersion) {
                    var webMethodsCaller = new (System.ServiceModel.CSHTML5_ClientBase$1.WebMethodsCaller(INTERFACE_TYPE))(endpointAddress);

                    return Bridge.cast(Bridge.unbox(webMethodsCaller.CallWebMethod(webMethodName, INTERFACE_TYPE, RETURN_TYPE, requestParameters, soapVersion), RETURN_TYPE), RETURN_TYPE);
                },
                BeginCallWebMethod: function (INTERFACE_TYPE, endpointAddress, webMethodName, methodReturnType, requestParameters, soapVersion) {
                    var callback = requestParameters.System$Collections$Generic$IDictionary$2$System$String$System$Object$getItem("callback");
                    var asyncState = requestParameters.System$Collections$Generic$IDictionary$2$System$String$System$Object$getItem("asyncState");

                    requestParameters.System$Collections$Generic$IDictionary$2$System$String$System$Object$remove("callback");
                    requestParameters.System$Collections$Generic$IDictionary$2$System$String$System$Object$remove("asyncState");

                    var webMethodsCaller = new (System.ServiceModel.CSHTML5_ClientBase$1.WebMethodsCaller(INTERFACE_TYPE))(endpointAddress);
                    var webMethodAsyncResult = new System.ServiceModel.INTERNAL_WebMethodsCaller.WebMethodAsyncResult(callback, asyncState);

                    webMethodsCaller.BeginCallWebMethod(webMethodName, INTERFACE_TYPE, methodReturnType, requestParameters, function (xmlReturnedFromTheServer) {
                        webMethodAsyncResult.XmlReturnedFromTheServer = xmlReturnedFromTheServer;

                        webMethodAsyncResult.Completed();
                    }, soapVersion);

                    return webMethodAsyncResult;
                },
                BeginCallWebMethod$1: function (RETURN_TYPE, INTERFACE_TYPE, endpointAddress, webMethodName, requestParameters, soapVersion) {
                    return System.ServiceModel.INTERNAL_WebMethodsCaller.BeginCallWebMethod(INTERFACE_TYPE, endpointAddress, webMethodName, RETURN_TYPE, requestParameters, soapVersion);
                },
                EndCallWebMethod$1: function (INTERFACE_TYPE, endpointAddress, webMethodName, methodReturnType, requestParameters, soapVersion) {
                    var asyncResult = Bridge.cast(requestParameters.System$Collections$Generic$IDictionary$2$System$String$System$Object$getItem("result"), System.IAsyncResult);
                    var webMethodAsyncResult = Bridge.cast(asyncResult, System.ServiceModel.INTERNAL_WebMethodsCaller.WebMethodAsyncResult);
                    var xmlReturnedFromTheServer = webMethodAsyncResult.XmlReturnedFromTheServer;

                    var webMethodsCaller = new (System.ServiceModel.CSHTML5_ClientBase$1.WebMethodsCaller(INTERFACE_TYPE))(endpointAddress);

                    var result = webMethodsCaller.EndCallWebMethod$1(webMethodName, INTERFACE_TYPE, methodReturnType, xmlReturnedFromTheServer, soapVersion);

                    return result;
                },
                EndCallWebMethod: function (RETURN_TYPE, INTERFACE_TYPE, endpointAddress, webMethodName, requestParameters, soapVersion) {
                    return Bridge.cast(Bridge.unbox(System.ServiceModel.INTERNAL_WebMethodsCaller.EndCallWebMethod$1(INTERFACE_TYPE, endpointAddress, webMethodName, RETURN_TYPE, requestParameters, soapVersion), RETURN_TYPE), RETURN_TYPE);
                },
                CallWebMethodAsync_WithoutReturnValue: function (INTERFACE_TYPE, endpointAddress, webMethodName, requestParameters, soapVersion) {
                    return System.ServiceModel.INTERNAL_WebMethodsCaller.CallWebMethodAsync(System.Object, INTERFACE_TYPE, endpointAddress, webMethodName, requestParameters, soapVersion);
                },
                CallWebMethod_WithoutReturnValue: function (INTERFACE_TYPE, endpointAddress, webMethodName, requestParameters, soapVersion) {
                    System.ServiceModel.INTERNAL_WebMethodsCaller.CallWebMethod(System.Object, INTERFACE_TYPE, endpointAddress, webMethodName, requestParameters, soapVersion);
                },
                EndCallWebMethod_WithoutReturnValue: function (INTERFACE_TYPE, endpointAddress, webMethodName, requestParameters, soapVersion) {
                    System.ServiceModel.INTERNAL_WebMethodsCaller.EndCallWebMethod(System.Object, INTERFACE_TYPE, endpointAddress, webMethodName, requestParameters, soapVersion);
                }
            }
        }
    });

    /**
     * Declares the base members for System.ServiceModel.MessageBodyMemberAttribute
     and System.ServiceModel.MessageHeaderAttribute.
     *
     * @abstract
     * @public
     * @class System.ServiceModel.MessageContractMemberAttribute
     * @augments System.Attribute
     */
    Bridge.define("System.ServiceModel.MessageContractMemberAttribute", {
        inherits: [System.Attribute],
        fields: {
            /**
             * When overridden in a derived class, gets a value that indicates whether the
             member has a protection level assigned.
             *
             * @instance
             * @public
             * @memberof System.ServiceModel.MessageContractMemberAttribute
             * @function HasProtectionLevel
             * @type boolean
             */
            HasProtectionLevel: false,
            /**
             * Specifies the name of the element that corresponds to this member.
             *
             * @instance
             * @public
             * @memberof System.ServiceModel.MessageContractMemberAttribute
             * @function Name
             * @type string
             */
            Name: null,
            /**
             * Specifies the namespace of the element that corresponds to this member.
             *
             * @instance
             * @public
             * @memberof System.ServiceModel.MessageContractMemberAttribute
             * @function Namespace
             * @type string
             */
            Namespace: null,
            /**
             * Specifies whether the member is to be transmitted as-is, signed, or signed
             and encrypted.
             *
             * @instance
             * @public
             * @memberof System.ServiceModel.MessageContractMemberAttribute
             * @function ProtectionLevel
             * @type System.Net.Security.ProtectionLevel
             */
            ProtectionLevel: 0
        },
        ctors: {
            /**
             * Initializes a new instance of the System.ServiceModel.MessageContractMemberAttribute
             class.
             *
             * @instance
             * @protected
             * @this System.ServiceModel.MessageContractMemberAttribute
             * @memberof System.ServiceModel.MessageContractMemberAttribute
             * @return  {void}
             */
            ctor: function () {
                this.$initialize();
                System.Attribute.ctor.call(this);
            }
        }
    });

    /**
     * Defines a strongly-typed class that corresponds to a SOAP message.
     *
     * @public
     * @class System.ServiceModel.MessageContractAttribute
     * @augments System.Attribute
     */
    Bridge.define("System.ServiceModel.MessageContractAttribute", {
        inherits: [System.Attribute],
        fields: {
            /**
             * Gets a value that indicates whether the message has a protection level.
             *
             * @instance
             * @public
             * @memberof System.ServiceModel.MessageContractAttribute
             * @function HasProtectionLevel
             * @type boolean
             */
            HasProtectionLevel: false,
            /**
             * Gets or sets a value that specifies whether the message body has a wrapper
             element.
             *
             * @instance
             * @public
             * @memberof System.ServiceModel.MessageContractAttribute
             * @function IsWrapped
             * @type boolean
             */
            IsWrapped: false,
            /**
             * Gets or sets a value that specified whether the message must be encrypted,
             signed, or both.
             *
             * @instance
             * @public
             * @memberof System.ServiceModel.MessageContractAttribute
             * @function ProtectionLevel
             * @type System.Net.Security.ProtectionLevel
             */
            ProtectionLevel: 0,
            /**
             * Gets or sets the name of the wrapper element of the message body.
             *
             * @instance
             * @public
             * @memberof System.ServiceModel.MessageContractAttribute
             * @function WrapperName
             * @type string
             */
            WrapperName: null,
            /**
             * Gets or sets the namespace of the message body wrapper element.
             *
             * @instance
             * @public
             * @memberof System.ServiceModel.MessageContractAttribute
             * @function WrapperNamespace
             * @type string
             */
            WrapperNamespace: null
        },
        ctors: {
            /**
             * Initializes a new instance of the System.ServiceModel.MessageContractAttribute
             class.
             *
             * @instance
             * @public
             * @this System.ServiceModel.MessageContractAttribute
             * @memberof System.ServiceModel.MessageContractAttribute
             * @return  {void}
             */
            ctor: function () {
                this.$initialize();
                System.Attribute.ctor.call(this);
            }
        }
    });

    /**
     * The {@link } namespaces contain the types necessary to
     build Windows Communication Foundation (WCF) service and client applications.
     *
     * @class System.ServiceModel.NamespaceDoc
     */
    Bridge.define("System.ServiceModel.NamespaceDoc");

    /**
     * Indicates that a method defines an operation that is part of a service contract
     in a Windows Communication Foundation (WCF) application.
     *
     * @public
     * @class System.ServiceModel.OperationContractAttribute
     * @augments System.Attribute
     */
    Bridge.define("System.ServiceModel.OperationContractAttribute", {
        inherits: [System.Attribute],
        fields: {
            /**
             * Gets or sets the WS-Addressing action of the request message.
             *
             * @instance
             * @public
             * @memberof System.ServiceModel.OperationContractAttribute
             * @function Action
             * @type string
             */
            Action: null,
            /**
             * Indicates that an operation is implemented asynchronously using a Begin+methodName
             and End+methodName method pair in a service contract.
             *
             * @instance
             * @public
             * @memberof System.ServiceModel.OperationContractAttribute
             * @function AsyncPattern
             * @type boolean
             */
            AsyncPattern: false,
            /**
             * Gets or sets a value that indicates whether an operation returns a reply
             message.
             *
             * @instance
             * @public
             * @memberof System.ServiceModel.OperationContractAttribute
             * @function IsOneWay
             * @type boolean
             */
            IsOneWay: false,
            /**
             * Gets or sets the name of the operation.
             *
             * @instance
             * @public
             * @memberof System.ServiceModel.OperationContractAttribute
             * @function Name
             * @type string
             */
            Name: null,
            /**
             * Gets or sets the value of the SOAP action for the reply message of the operation.
             *
             * @instance
             * @public
             * @memberof System.ServiceModel.OperationContractAttribute
             * @function ReplyAction
             * @type string
             */
            ReplyAction: null
        },
        ctors: {
            /**
             * Initializes a new instance of the System.ServiceModel.OperationContractAttribute
             class.
             *
             * @instance
             * @public
             * @this System.ServiceModel.OperationContractAttribute
             * @memberof System.ServiceModel.OperationContractAttribute
             * @return  {void}
             */
            ctor: function () {
                this.$initialize();
                System.Attribute.ctor.call(this);
            }
        }
    });

    /**
     * Determines the format of the message.
     *
     * @public
     * @class System.ServiceModel.OperationFormatUse
     */
    Bridge.define("System.ServiceModel.OperationFormatUse", {
        $kind: "enum",
        statics: {
            fields: {
                /**
                 * Implies that the message is a literal instance of the schema in the WSDL.
                 *
                 * @static
                 * @public
                 * @memberof System.ServiceModel.OperationFormatUse
                 * @constant
                 * @default 0
                 * @type System.ServiceModel.OperationFormatUse
                 */
                Literal: 0,
                /**
                 * Implies that the schemas in the WSDL are abstract specifications that are
                 encoded according to the rules found in SOAP 1.1 section 5.
                 *
                 * @static
                 * @public
                 * @memberof System.ServiceModel.OperationFormatUse
                 * @constant
                 * @default 1
                 * @type System.ServiceModel.OperationFormatUse
                 */
                Encoded: 1
            }
        }
    });

    /**
     * Indicates that an interface or a class defines a service contract in a Windows
     Communication Foundation (WCF) application.
     *
     * @public
     * @class System.ServiceModel.ServiceContractAttribute
     * @augments System.Attribute
     */
    Bridge.define("System.ServiceModel.ServiceContractAttribute", {
        inherits: [System.Attribute],
        fields: {
            /**
             * Gets or sets the type of callback contract when the contract is a duplex
             contract.
             *
             * @instance
             * @public
             * @memberof System.ServiceModel.ServiceContractAttribute
             * @function CallbackContract
             * @type System.Type
             */
            CallbackContract: null,
            /**
             * The name used to locate the service element in an application configuration
             file. The default is the name of the service implementation class.
             *
             * @instance
             * @public
             * @memberof System.ServiceModel.ServiceContractAttribute
             * @function ConfigurationName
             * @type string
             */
            ConfigurationName: null,
            /**
             * Gets or sets the name for the portType element in Web Services Description
             Language (WSDL).
             *
             * @instance
             * @public
             * @memberof System.ServiceModel.ServiceContractAttribute
             * @function Name
             * @type string
             */
            Name: null,
            /**
             * Gets or sets the namespace of the portType element in Web Services Description
             Language (WSDL).
             *
             * @instance
             * @public
             * @memberof System.ServiceModel.ServiceContractAttribute
             * @function Namespace
             * @type string
             */
            Namespace: null
        },
        ctors: {
            /**
             * Initializes a new instance of the System.ServiceModel.ServiceContractAttribute
             class.
             *
             * @instance
             * @public
             * @this System.ServiceModel.ServiceContractAttribute
             * @memberof System.ServiceModel.ServiceContractAttribute
             * @return  {void}
             */
            ctor: function () {
                this.$initialize();
                System.Attribute.ctor.call(this);
                this.Namespace = "http://tempuri.org";
            }
        }
    });

    Bridge.define("System.ServiceModel.ServiceKnownTypeAttribute", {
        inherits: [System.Attribute],
        fields: {
            declaringType: null,
            methodName: null,
            type: null
        },
        props: {
            DeclaringType: {
                get: function () {
                    return this.declaringType;
                }
            },
            MethodName: {
                get: function () {
                    return this.methodName;
                }
            },
            Type: {
                get: function () {
                    return this.type;
                }
            }
        },
        ctors: {
            ctor: function () {
                this.$initialize();
                System.Attribute.ctor.call(this);
            },
            $ctor3: function (type) {
                this.$initialize();
                System.Attribute.ctor.call(this);
                this.type = type;
            },
            $ctor1: function (methodName) {
                this.$initialize();
                System.Attribute.ctor.call(this);
                this.methodName = methodName;
            },
            $ctor2: function (methodName, declaringType) {
                this.$initialize();
                System.Attribute.ctor.call(this);
                this.methodName = methodName;
                this.declaringType = declaringType;
            }
        }
    });

    Bridge.define("System.ServiceModel.XmlSerializerFormatAttribute", {
        inherits: [System.Attribute],
        fields: {
            Style: 0,
            SupportFaults: false,
            /**
             * Gets or sets a value that specifies the format to use. The default is
             System.ServiceModel.OperationFormatUse.Literal.
             *
             * @instance
             * @public
             * @memberof System.ServiceModel.XmlSerializerFormatAttribute
             * @function Use
             * @type System.ServiceModel.OperationFormatUse
             */
            Use: 0
        }
    });

    /** @namespace System.Windows.Markup */

    /**
     * Represents a service that reports situational object-property relationships
     for markup extension evaluation.
     *
     * @abstract
     * @public
     * @class System.Windows.Markup.IProvideValueTarget
     */
    Bridge.define("System.Windows.Markup.IProvideValueTarget", {
        $kind: "interface"
    });

    Bridge.define("System.Settings", {
        fields: {
            DefaultSoapCredentialsMode: 0,
            EnableBindingErrorsLogging: false,
            EnableBindingErrorsThrowing: false,
            EnableWebRequestsLogging: false,
            EnableInvalidPropertyMetadataDefaultValueExceptions: false
        },
        props: {
            EnableInteropLogging: {
                get: function () {
                    return CSHTML5.Internal.INTERNAL_SimulatorExecuteJavaScript.EnableInteropLogging;
                },
                set: function (value) {
                    CSHTML5.Internal.INTERNAL_SimulatorExecuteJavaScript.EnableInteropLogging = value;
                }
            },
            EnablePerformanceLogging: {
                get: function () {
                    return CSHTML5.Internal.INTERNAL_VisualTreeManager.EnablePerformanceLogging;
                },
                set: function (value) {
                    CSHTML5.Internal.INTERNAL_VisualTreeManager.EnablePerformanceLogging = value;
                }
            },
            /**
             * When True, do not apply the CSS properties of the UI elements that are not visible.
             Those property are applied later, when the control becomes visible.
             Enabling this option results in improved performance.
             *
             * @instance
             * @public
             * @memberof System.Settings
             * @function EnableOptimizationWhereCollapsedControlsAreNotRendered
             * @type boolean
             */
            EnableOptimizationWhereCollapsedControlsAreNotRendered: {
                get: function () {
                    return CSHTML5.Internal.INTERNAL_VisualTreeManager.EnableOptimizationWhereCollapsedControlsAreNotRendered;
                },
                set: function (value) {
                    CSHTML5.Internal.INTERNAL_VisualTreeManager.EnableOptimizationWhereCollapsedControlsAreNotRendered = value;
                }
            },
            /**
             * When True, do not create the DOM elements for the UI elements that are not visible.
             The DOM elements are created later, when the control becomes visible.
             Enabling this option results in significantly improved performance if there are many hidden elements.
             *
             * @instance
             * @public
             * @memberof System.Settings
             * @function EnableOptimizationWhereCollapsedControlsAreNotLoaded
             * @type boolean
             */
            EnableOptimizationWhereCollapsedControlsAreNotLoaded: {
                get: function () {
                    return CSHTML5.Internal.INTERNAL_VisualTreeManager.EnableOptimizationWhereCollapsedControlsAreNotLoaded;
                },
                set: function (value) {
                    CSHTML5.Internal.INTERNAL_VisualTreeManager.EnableOptimizationWhereCollapsedControlsAreNotLoaded = value;
                }
            },
            /**
             * Wait a few moments before creating the DOM elements for the UI elements that are not visible.
             The goal is to give the time to the browser engine to draw the visible ones, in order to improve the perceived performance.
             The DOM elements are created immediately after the browser engine has finished drawing (when the UI thread is available).
             *
             * @instance
             * @public
             * @memberof System.Settings
             * @function EnableOptimizationWhereCollapsedControlsAreLoadedLast
             * @type boolean
             */
            EnableOptimizationWhereCollapsedControlsAreLoadedLast: {
                get: function () {
                    return CSHTML5.Internal.INTERNAL_VisualTreeManager.EnableOptimizationWhereCollapsedControlsAreLoadedLast;
                },
                set: function (value) {
                    CSHTML5.Internal.INTERNAL_VisualTreeManager.EnableOptimizationWhereCollapsedControlsAreLoadedLast = value;
                }
            },
            EnableProgressiveRendering: {
                get: function () {
                    return Windows.UI.Xaml.Controls.Panel.INTERNAL_ApplicationWideEnableProgressiveRendering;
                },
                set: function (value) {
                    Windows.UI.Xaml.Controls.Panel.INTERNAL_ApplicationWideEnableProgressiveRendering = value;
                }
            }
        },
        ctors: {
            ctor: function () {
                this.$initialize();
                this.DefaultSoapCredentialsMode = System.Net.CredentialsMode.Disabled;
                this.EnableBindingErrorsLogging = false;
                this.EnableBindingErrorsThrowing = false;
                this.EnableInvalidPropertyMetadataDefaultValueExceptions = true;
            }
        }
    });

    /** @namespace System.Text */

    /**
     * The {@link } namespaces contain types for character encoding and
     string manipulation. A child namespace enables you to process text using regular expressions.
     *
     * @class System.Text.NamespaceDoc
     */
    Bridge.define("System.Text.NamespaceDoc");

    /**
     * The {@link } namespaces contain types that enable multithreaded
     programming. A child namespace provides types that simplify the work of writing
     concurrent and asynchronous code.
     *
     * @class System.Threading.NamespaceDoc
     */
    Bridge.define("System.Threading.NamespaceDoc");

    /**
     * Utility class that simplifies cration of tuples by using
     method calls instead of constructor calls
     *
     * @static
     * @abstract
     * @public
     * @class System.Tuple2
     */
    Bridge.define("System.Tuple2", {
        statics: {
            methods: {
                /**
                 * Creates a new tuple value with the specified elements. The method
                 can be used without specifying the generic parameters, because C#
                 compiler can usually infer the actual types.
                 *
                 * @static
                 * @public
                 * @this System.Tuple2
                 * @memberof System.Tuple2
                 * @param   {Function}           T1        
                 * @param   {Function}           T2        
                 * @param   {T1}                 item1     First element of the tuple
                 * @param   {T2}                 second    Second element of the tuple
                 * @return  {System.Tuple2$2}              A newly created tuple
                 */
                Create: function (T1, T2, item1, second) {
                    return new (System.Tuple2$2(T1,T2))(item1, second);
                },
                /**
                 * Creates a new tuple value with the specified elements. The method
                 can be used without specifying the generic parameters, because C#
                 compiler can usually infer the actual types.
                 *
                 * @static
                 * @public
                 * @this System.Tuple2
                 * @memberof System.Tuple2
                 * @param   {Function}           T1        
                 * @param   {Function}           T2        
                 * @param   {Function}           T3        
                 * @param   {T1}                 item1     First element of the tuple
                 * @param   {T2}                 second    Second element of the tuple
                 * @param   {T3}                 third     Third element of the tuple
                 * @return  {System.Tuple2$3}              A newly created tuple
                 */
                Create$1: function (T1, T2, T3, item1, second, third) {
                    return new (System.Tuple2$3(T1,T2,T3))(item1, second, third);
                },
                /**
                 * Creates a new tuple value with the specified elements. The method
                 can be used without specifying the generic parameters, because C#
                 compiler can usually infer the actual types.
                 *
                 * @static
                 * @public
                 * @this System.Tuple2
                 * @memberof System.Tuple2
                 * @param   {Function}           T1        
                 * @param   {Function}           T2        
                 * @param   {Function}           T3        
                 * @param   {Function}           T4        
                 * @param   {T1}                 item1     First element of the tuple
                 * @param   {T2}                 second    Second element of the tuple
                 * @param   {T3}                 third     Third element of the tuple
                 * @param   {T4}                 fourth    Fourth element of the tuple
                 * @return  {System.Tuple2$4}              A newly created tuple
                 */
                Create$2: function (T1, T2, T3, T4, item1, second, third, fourth) {
                    return new (System.Tuple2$4(T1,T2,T3,T4))(item1, second, third, fourth);
                },
                /**
                 * Creates a new tuple value with the specified elements. The method
                 can be used without specifying the generic parameters, because C#
                 compiler can usually infer the actual types.
                 *
                 * @static
                 * @public
                 * @this System.Tuple2
                 * @memberof System.Tuple2
                 * @param   {Function}           T1        
                 * @param   {Function}           T2        
                 * @param   {Function}           T3        
                 * @param   {Function}           T4        
                 * @param   {Function}           T5        
                 * @param   {T1}                 item1     First element of the tuple
                 * @param   {T2}                 second    Second element of the tuple
                 * @param   {T3}                 third     Third element of the tuple
                 * @param   {T4}                 fourth    Fourth element of the tuple
                 * @param   {T5}                 fifth     Fifth element of the tuple
                 * @return  {System.Tuple2$5}              A newly created tuple
                 */
                Create$3: function (T1, T2, T3, T4, T5, item1, second, third, fourth, fifth) {
                    return new (System.Tuple2$5(T1,T2,T3,T4,T5))(item1, second, third, fourth, fifth);
                },
                /**
                 * Creates a new tuple value with the specified elements. The method
                 can be used without specifying the generic parameters, because C#
                 compiler can usually infer the actual types.
                 *
                 * @static
                 * @public
                 * @this System.Tuple2
                 * @memberof System.Tuple2
                 * @param   {Function}           T1        
                 * @param   {Function}           T2        
                 * @param   {Function}           T3        
                 * @param   {Function}           T4        
                 * @param   {Function}           T5        
                 * @param   {Function}           T6        
                 * @param   {T1}                 item1     First element of the tuple
                 * @param   {T2}                 second    Second element of the tuple
                 * @param   {T3}                 third     Third element of the tuple
                 * @param   {T4}                 fourth    Fourth element of the tuple
                 * @param   {T5}                 fifth     Fifth element of the tuple
                 * @param   {T6}                 sixth     Sixth element of the tuple
                 * @return  {System.Tuple2$6}              A newly created tuple
                 */
                Create$4: function (T1, T2, T3, T4, T5, T6, item1, second, third, fourth, fifth, sixth) {
                    return new (System.Tuple2$6(T1,T2,T3,T4,T5,T6))(item1, second, third, fourth, fifth, sixth);
                }
            }
        }
    });

    /**
     * Represents a functional tuple that can be used to store
     two values of different types inside one object.
     *
     * @public
     * @class System.Tuple2$2
     * @param   {Function}    [name]    The type of the first element
     * @param   {Function}    [name]    The type of the second element
     */
    Bridge.define("System.Tuple2$2", function (T1, T2) { return {
        statics: {
            methods: {
                op_Equality: function (a, b) {
                    if (Bridge.referenceEquals(a, null)) {
                        return Bridge.referenceEquals(b, null);
                    }
                    if (a.item1 == null && b.item1 != null) {
                        return false;
                    }
                    if (a.item2 == null && b.item2 != null) {
                        return false;
                    }
                    return Bridge.equals(a.item1, b.item1) && Bridge.equals(a.item2, b.item2);
                },
                op_Inequality: function (a, b) {
                    return !(System.Tuple2$2(T1,T2).op_Equality(a, b));
                }
            }
        },
        fields: {
            item1: Bridge.getDefaultValue(T1),
            item2: Bridge.getDefaultValue(T2)
        },
        props: {
            /**
             * Retyurns the first element of the tuple
             *
             * @instance
             * @public
             * @readonly
             * @memberof System.Tuple2$2
             * @function Item1
             * @type T1
             */
            Item1: {
                get: function () {
                    return this.item1;
                }
            },
            /**
             * Returns the second element of the tuple
             *
             * @instance
             * @public
             * @readonly
             * @memberof System.Tuple2$2
             * @function Item2
             * @type T2
             */
            Item2: {
                get: function () {
                    return this.item2;
                }
            }
        },
        ctors: {
            /**
             * Create a new tuple value
             *
             * @instance
             * @public
             * @this System.Tuple2$2
             * @memberof System.Tuple2$2
             * @param   {T1}      item1    First element of the tuple
             * @param   {T2}      item2    Second element of the tuple
             * @return  {void}
             */
            ctor: function (item1, item2) {
                this.$initialize();
                this.item1 = item1;
                this.item2 = item2;
            }
        },
        methods: {
            toString: function () {
                return System.String.format("Tuple({0}, {1})", this.Item1, this.Item2);
            },
            getHashCode: function () {
                var hash = 17;
                hash = (Bridge.Int.mul(hash, 23) + (this.item1 == null ? 0 : Bridge.getHashCode(this.item1))) | 0;
                hash = (Bridge.Int.mul(hash, 23) + (this.item2 == null ? 0 : Bridge.getHashCode(this.item2))) | 0;
                return hash;
            },
            equals: function (o) {
                if (!(Bridge.is(o, System.Tuple2$2(T1,T2)))) {
                    return false;
                }

                var other = Bridge.cast(o, System.Tuple2$2(T1,T2));

                return System.Tuple2$2(T1,T2).op_Equality(this, other);
            },
            Equals: function (other) {
                return System.Tuple2$2(T1,T2).op_Equality(this, other);
            }
        }
    }; });

    /**
     * Represents a functional tuple that can be used to store
     two values of different types inside one object.
     *
     * @public
     * @class System.Tuple2$3
     * @param   {Function}    [name]    The type of the first element
     * @param   {Function}    [name]    The type of the second element
     * @param   {Function}    [name]    The type of the third element
     */
    Bridge.define("System.Tuple2$3", function (T1, T2, T3) { return {
        statics: {
            methods: {
                op_Equality: function (a, b) {
                    if (Bridge.referenceEquals(a, null)) {
                        return Bridge.referenceEquals(b, null);
                    }
                    if (a.item1 == null && b.item1 != null) {
                        return false;
                    }
                    if (a.item2 == null && b.item2 != null) {
                        return false;
                    }
                    if (a.item3 == null && b.item3 != null) {
                        return false;
                    }
                    return Bridge.equals(a.item1, b.item1) && Bridge.equals(a.item2, b.item2) && Bridge.equals(a.item3, b.item3);
                },
                op_Inequality: function (a, b) {
                    return !(System.Tuple2$3(T1,T2,T3).op_Equality(a, b));
                }
            }
        },
        fields: {
            item1: Bridge.getDefaultValue(T1),
            item2: Bridge.getDefaultValue(T2),
            item3: Bridge.getDefaultValue(T3)
        },
        props: {
            /**
             * Retyurns the first element of the tuple
             *
             * @instance
             * @public
             * @readonly
             * @memberof System.Tuple2$3
             * @function Item1
             * @type T1
             */
            Item1: {
                get: function () {
                    return this.item1;
                }
            },
            /**
             * Returns the second element of the tuple
             *
             * @instance
             * @public
             * @readonly
             * @memberof System.Tuple2$3
             * @function Item2
             * @type T2
             */
            Item2: {
                get: function () {
                    return this.item2;
                }
            },
            /**
             * Returns the second element of the tuple
             *
             * @instance
             * @public
             * @readonly
             * @memberof System.Tuple2$3
             * @function Item3
             * @type T3
             */
            Item3: {
                get: function () {
                    return this.item3;
                }
            }
        },
        ctors: {
            /**
             * Create a new tuple value
             *
             * @instance
             * @public
             * @this System.Tuple2$3
             * @memberof System.Tuple2$3
             * @param   {T1}      item1    First element of the tuple
             * @param   {T2}      item2    Second element of the tuple
             * @param   {T3}      item3    Third element of the tuple
             * @return  {void}
             */
            ctor: function (item1, item2, item3) {
                this.$initialize();
                this.item1 = item1;
                this.item2 = item2;
                this.item3 = item3;
            }
        },
        methods: {
            getHashCode: function () {
                var hash = 17;
                hash = (Bridge.Int.mul(hash, 23) + (this.item1 == null ? 0 : Bridge.getHashCode(this.item1))) | 0;
                hash = (Bridge.Int.mul(hash, 23) + (this.item2 == null ? 0 : Bridge.getHashCode(this.item2))) | 0;
                hash = (Bridge.Int.mul(hash, 23) + (this.item3 == null ? 0 : Bridge.getHashCode(this.item3))) | 0;
                return hash;
            },
            equals: function (o) {
                if (!(Bridge.is(o, System.Tuple2$3(T1,T2,T3)))) {
                    return false;
                }

                var other = Bridge.cast(o, System.Tuple2$3(T1,T2,T3));

                return System.Tuple2$3(T1,T2,T3).op_Equality(this, other);
            }
        }
    }; });

    /**
     * Represents a functional tuple that can be used to store
     two values of different types inside one object.
     *
     * @public
     * @class System.Tuple2$4
     * @param   {Function}    [name]    The type of the first element
     * @param   {Function}    [name]    The type of the second element
     * @param   {Function}    [name]    The type of the third element
     * @param   {Function}    [name]    The type of the fourth element
     */
    Bridge.define("System.Tuple2$4", function (T1, T2, T3, T4) { return {
        statics: {
            methods: {
                op_Equality: function (a, b) {
                    if (Bridge.referenceEquals(a, null)) {
                        return Bridge.referenceEquals(b, null);
                    }
                    if (a.item1 == null && b.item1 != null) {
                        return false;
                    }
                    if (a.item2 == null && b.item2 != null) {
                        return false;
                    }
                    if (a.item3 == null && b.item3 != null) {
                        return false;
                    }
                    if (a.item4 == null && b.item4 != null) {
                        return false;
                    }
                    return Bridge.equals(a.item1, b.item1) && Bridge.equals(a.item2, b.item2) && Bridge.equals(a.item3, b.item3) && Bridge.equals(a.item4, b.item4);
                },
                op_Inequality: function (a, b) {
                    return !(System.Tuple2$4(T1,T2,T3,T4).op_Equality(a, b));
                }
            }
        },
        fields: {
            item1: Bridge.getDefaultValue(T1),
            item2: Bridge.getDefaultValue(T2),
            item3: Bridge.getDefaultValue(T3),
            item4: Bridge.getDefaultValue(T4)
        },
        props: {
            /**
             * Retyurns the first element of the tuple
             *
             * @instance
             * @public
             * @readonly
             * @memberof System.Tuple2$4
             * @function Item1
             * @type T1
             */
            Item1: {
                get: function () {
                    return this.item1;
                }
            },
            /**
             * Returns the second element of the tuple
             *
             * @instance
             * @public
             * @readonly
             * @memberof System.Tuple2$4
             * @function Item2
             * @type T2
             */
            Item2: {
                get: function () {
                    return this.item2;
                }
            },
            /**
             * Returns the second element of the tuple
             *
             * @instance
             * @public
             * @readonly
             * @memberof System.Tuple2$4
             * @function Item3
             * @type T3
             */
            Item3: {
                get: function () {
                    return this.item3;
                }
            },
            /**
             * Returns the second element of the tuple
             *
             * @instance
             * @public
             * @readonly
             * @memberof System.Tuple2$4
             * @function Item4
             * @type T4
             */
            Item4: {
                get: function () {
                    return this.item4;
                }
            }
        },
        ctors: {
            /**
             * Create a new tuple value
             *
             * @instance
             * @public
             * @this System.Tuple2$4
             * @memberof System.Tuple2$4
             * @param   {T1}      item1    First element of the tuple
             * @param   {T2}      item2    Second element of the tuple
             * @param   {T3}      item3    Third element of the tuple
             * @param   {T4}      item4    Fourth element of the tuple
             * @return  {void}
             */
            ctor: function (item1, item2, item3, item4) {
                this.$initialize();
                this.item1 = item1;
                this.item2 = item2;
                this.item3 = item3;
                this.item4 = item4;
            }
        },
        methods: {
            getHashCode: function () {
                var hash = 17;
                hash = (Bridge.Int.mul(hash, 23) + (this.item1 == null ? 0 : Bridge.getHashCode(this.item1))) | 0;
                hash = (Bridge.Int.mul(hash, 23) + (this.item2 == null ? 0 : Bridge.getHashCode(this.item2))) | 0;
                hash = (Bridge.Int.mul(hash, 23) + (this.item3 == null ? 0 : Bridge.getHashCode(this.item3))) | 0;
                hash = (Bridge.Int.mul(hash, 23) + (this.item4 == null ? 0 : Bridge.getHashCode(this.item4))) | 0;
                return hash;
            },
            equals: function (o) {
                if (!Bridge.referenceEquals(Bridge.getType(o), System.Tuple2$4(T1,T2,T3,T4))) {
                    return false;
                }

                var other = Bridge.cast(o, System.Tuple2$4(T1,T2,T3,T4));

                return System.Tuple2$4(T1,T2,T3,T4).op_Equality(this, other);
            }
        }
    }; });

    /**
     * Represents a functional tuple that can be used to store
     two values of different types inside one object.
     *
     * @public
     * @class System.Tuple2$5
     * @param   {Function}    [name]    The type of the first element
     * @param   {Function}    [name]    The type of the second element
     * @param   {Function}    [name]    The type of the third element
     * @param   {Function}    [name]    The type of the fourth element
     * @param   {Function}    [name]    The type of the fifth element
     */
    Bridge.define("System.Tuple2$5", function (T1, T2, T3, T4, T5) { return {
        statics: {
            methods: {
                op_Equality: function (a, b) {
                    if (Bridge.referenceEquals(a, null)) {
                        return Bridge.referenceEquals(b, null);
                    }
                    if (a.item1 == null && b.item1 != null) {
                        return false;
                    }
                    if (a.item2 == null && b.item2 != null) {
                        return false;
                    }
                    if (a.item3 == null && b.item3 != null) {
                        return false;
                    }
                    if (a.item4 == null && b.item4 != null) {
                        return false;
                    }
                    if (a.item5 == null && b.item5 != null) {
                        return false;
                    }
                    return Bridge.equals(a.item1, b.item1) && Bridge.equals(a.item2, b.item2) && Bridge.equals(a.item3, b.item3) && Bridge.equals(a.item4, b.item4) && Bridge.equals(a.item5, b.item5);
                },
                op_Inequality: function (a, b) {
                    return !(System.Tuple2$5(T1,T2,T3,T4,T5).op_Equality(a, b));
                }
            }
        },
        fields: {
            item1: Bridge.getDefaultValue(T1),
            item2: Bridge.getDefaultValue(T2),
            item3: Bridge.getDefaultValue(T3),
            item4: Bridge.getDefaultValue(T4),
            item5: Bridge.getDefaultValue(T5)
        },
        props: {
            /**
             * Retyurns the first element of the tuple
             *
             * @instance
             * @public
             * @readonly
             * @memberof System.Tuple2$5
             * @function Item1
             * @type T1
             */
            Item1: {
                get: function () {
                    return this.item1;
                }
            },
            /**
             * Returns the second element of the tuple
             *
             * @instance
             * @public
             * @readonly
             * @memberof System.Tuple2$5
             * @function Item2
             * @type T2
             */
            Item2: {
                get: function () {
                    return this.item2;
                }
            },
            /**
             * Returns the third element of the tuple
             *
             * @instance
             * @public
             * @readonly
             * @memberof System.Tuple2$5
             * @function Item3
             * @type T3
             */
            Item3: {
                get: function () {
                    return this.item3;
                }
            },
            /**
             * Returns the fourth element of the tuple
             *
             * @instance
             * @public
             * @readonly
             * @memberof System.Tuple2$5
             * @function Item4
             * @type T4
             */
            Item4: {
                get: function () {
                    return this.item4;
                }
            },
            /**
             * Returns the fifth element of the tuple
             *
             * @instance
             * @public
             * @readonly
             * @memberof System.Tuple2$5
             * @function Item5
             * @type T5
             */
            Item5: {
                get: function () {
                    return this.item5;
                }
            }
        },
        ctors: {
            /**
             * Create a new tuple value
             *
             * @instance
             * @public
             * @this System.Tuple2$5
             * @memberof System.Tuple2$5
             * @param   {T1}      item1    First element of the tuple
             * @param   {T2}      item2    Second element of the tuple
             * @param   {T3}      item3    Third element of the tuple
             * @param   {T4}      item4    Fourth element of the tuple
             * @param   {T5}      item5    Fifth element of the tuple
             * @return  {void}
             */
            ctor: function (item1, item2, item3, item4, item5) {
                this.$initialize();
                this.item1 = item1;
                this.item2 = item2;
                this.item3 = item3;
                this.item4 = item4;
                this.item5 = item5;
            }
        },
        methods: {
            getHashCode: function () {
                var hash = 17;
                hash = (Bridge.Int.mul(hash, 23) + (this.item1 == null ? 0 : Bridge.getHashCode(this.item1))) | 0;
                hash = (Bridge.Int.mul(hash, 23) + (this.item2 == null ? 0 : Bridge.getHashCode(this.item2))) | 0;
                hash = (Bridge.Int.mul(hash, 23) + (this.item3 == null ? 0 : Bridge.getHashCode(this.item3))) | 0;
                hash = (Bridge.Int.mul(hash, 23) + (this.item4 == null ? 0 : Bridge.getHashCode(this.item4))) | 0;
                hash = (Bridge.Int.mul(hash, 23) + (this.item5 == null ? 0 : Bridge.getHashCode(this.item5))) | 0;
                return hash;
            },
            equals: function (o) {
                if (!Bridge.referenceEquals(Bridge.getType(o), System.Tuple2$5(T1,T2,T3,T4,T5))) {
                    return false;
                }

                var other = Bridge.cast(o, System.Tuple2$5(T1,T2,T3,T4,T5));

                return System.Tuple2$5(T1,T2,T3,T4,T5).op_Equality(this, other);
            }
        }
    }; });

    /**
     * Represents a functional tuple that can be used to store
     two values of different types inside one object.
     *
     * @public
     * @class System.Tuple2$6
     * @param   {Function}    [name]    The type of the first element
     * @param   {Function}    [name]    The type of the second element
     * @param   {Function}    [name]    The type of the third element
     * @param   {Function}    [name]    The type of the fourth element
     * @param   {Function}    [name]    The type of the fifth element
     * @param   {Function}    [name]    The type of the sixth element
     */
    Bridge.define("System.Tuple2$6", function (T1, T2, T3, T4, T5, T6) { return {
        statics: {
            methods: {
                op_Equality: function (a, b) {
                    if (Bridge.referenceEquals(a, null)) {
                        return Bridge.referenceEquals(b, null);
                    }
                    if (a.item1 == null && b.item1 != null) {
                        return false;
                    }
                    if (a.item2 == null && b.item2 != null) {
                        return false;
                    }
                    if (a.item3 == null && b.item3 != null) {
                        return false;
                    }
                    if (a.item4 == null && b.item4 != null) {
                        return false;
                    }
                    if (a.item5 == null && b.item5 != null) {
                        return false;
                    }
                    if (a.item6 == null && b.item6 != null) {
                        return false;
                    }
                    return Bridge.equals(a.item1, b.item1) && Bridge.equals(a.item2, b.item2) && Bridge.equals(a.item3, b.item3) && Bridge.equals(a.item4, b.item4) && Bridge.equals(a.item5, b.item5) && Bridge.equals(a.item6, b.item6);
                },
                op_Inequality: function (a, b) {
                    return !(System.Tuple2$6(T1,T2,T3,T4,T5,T6).op_Equality(a, b));
                }
            }
        },
        fields: {
            item1: Bridge.getDefaultValue(T1),
            item2: Bridge.getDefaultValue(T2),
            item3: Bridge.getDefaultValue(T3),
            item4: Bridge.getDefaultValue(T4),
            item5: Bridge.getDefaultValue(T5),
            item6: Bridge.getDefaultValue(T6)
        },
        props: {
            /**
             * Retyurns the first element of the tuple
             *
             * @instance
             * @public
             * @readonly
             * @memberof System.Tuple2$6
             * @function Item1
             * @type T1
             */
            Item1: {
                get: function () {
                    return this.item1;
                }
            },
            /**
             * Returns the second element of the tuple
             *
             * @instance
             * @public
             * @readonly
             * @memberof System.Tuple2$6
             * @function Item2
             * @type T2
             */
            Item2: {
                get: function () {
                    return this.item2;
                }
            },
            /**
             * Returns the third element of the tuple
             *
             * @instance
             * @public
             * @readonly
             * @memberof System.Tuple2$6
             * @function Item3
             * @type T3
             */
            Item3: {
                get: function () {
                    return this.item3;
                }
            },
            /**
             * Returns the fourth element of the tuple
             *
             * @instance
             * @public
             * @readonly
             * @memberof System.Tuple2$6
             * @function Item4
             * @type T4
             */
            Item4: {
                get: function () {
                    return this.item4;
                }
            },
            /**
             * Returns the fifth element of the tuple
             *
             * @instance
             * @public
             * @readonly
             * @memberof System.Tuple2$6
             * @function Item5
             * @type T5
             */
            Item5: {
                get: function () {
                    return this.item5;
                }
            },
            /**
             * Returns the fifth element of the tuple
             *
             * @instance
             * @public
             * @readonly
             * @memberof System.Tuple2$6
             * @function Item6
             * @type T6
             */
            Item6: {
                get: function () {
                    return this.item6;
                }
            }
        },
        ctors: {
            /**
             * Create a new tuple value
             *
             * @instance
             * @public
             * @this System.Tuple2$6
             * @memberof System.Tuple2$6
             * @param   {T1}      item1    First element of the tuple
             * @param   {T2}      item2    Second element of the tuple
             * @param   {T3}      item3    Third element of the tuple
             * @param   {T4}      item4    Fourth element of the tuple
             * @param   {T5}      item5    Fifth element of the tuple
             * @param   {T6}      item6    Sixth element of the tuple
             * @return  {void}
             */
            ctor: function (item1, item2, item3, item4, item5, item6) {
                this.$initialize();
                this.item1 = item1;
                this.item2 = item2;
                this.item3 = item3;
                this.item4 = item4;
                this.item5 = item5;
                this.item6 = item6;
            }
        },
        methods: {
            getHashCode: function () {
                var hash = 17;
                hash = (Bridge.Int.mul(hash, 23) + (this.item1 == null ? 0 : Bridge.getHashCode(this.item1))) | 0;
                hash = (Bridge.Int.mul(hash, 23) + (this.item2 == null ? 0 : Bridge.getHashCode(this.item2))) | 0;
                hash = (Bridge.Int.mul(hash, 23) + (this.item3 == null ? 0 : Bridge.getHashCode(this.item3))) | 0;
                hash = (Bridge.Int.mul(hash, 23) + (this.item4 == null ? 0 : Bridge.getHashCode(this.item4))) | 0;
                hash = (Bridge.Int.mul(hash, 23) + (this.item5 == null ? 0 : Bridge.getHashCode(this.item5))) | 0;
                hash = (Bridge.Int.mul(hash, 23) + (this.item6 == null ? 0 : Bridge.getHashCode(this.item6))) | 0;
                return hash;
            },
            equals: function (o) {
                if (!Bridge.referenceEquals(Bridge.getType(o), System.Tuple2$6(T1,T2,T3,T4,T5,T6))) {
                    return false;
                }

                var other = Bridge.cast(o, System.Tuple2$6(T1,T2,T3,T4,T5,T6));

                return System.Tuple2$6(T1,T2,T3,T4,T5,T6).op_Equality(this, other);
            }
        }
    }; });

    Bridge.define("System.Windows.Browser.HtmlDocument", {
        props: {
            /**
             * Gets a Uniform Resource Identifier (URI) object that represents the current HTML
             document.
             *
             * @instance
             * @public
             * @readonly
             * @memberof System.Windows.Browser.HtmlDocument
             * @function DocumentUri
             * @type System.Uri
             */
            DocumentUri: {
                get: function () {
                    return new System.Uri(Bridge.toString(window.location.href));
                }
            },
            /**
             * Gets a navigable, read-only collection of name/value pairs that represent
             the query string parameters on the current page's URL.
             *
             * @instance
             * @public
             * @readonly
             * @memberof System.Windows.Browser.HtmlDocument
             * @function QueryString
             * @type System.Collections.Generic.IDictionary$2
             */
            QueryString: {
                get: function () {
                    var $t;
                    var returnValue = new (System.Collections.Generic.Dictionary$2(System.String,System.String)).ctor();
                    var url = Bridge.toString(document.URL);
                    var index = System.String.indexOf(url, String.fromCharCode(63));
                    if (index !== -1) {
                        url = url.substr(((index + 1) | 0));
                        var splittedParameters = System.String.split(url, [38].map(function (i) {{ return String.fromCharCode(i); }}));
                        $t = Bridge.getEnumerator(splittedParameters);
                        try {
                            while ($t.moveNext()) {
                                var str = $t.Current;
                                var splittedParameter = System.String.split(str, [61].map(function (i) {{ return String.fromCharCode(i); }}));
                                returnValue.add(splittedParameter[System.Array.index(0, splittedParameter)], splittedParameter[System.Array.index(1, splittedParameter)]);
                            }
                        } finally {
                            if (Bridge.is($t, System.IDisposable)) {
                                $t.System$IDisposable$Dispose();
                            }
                        }
                    }
                    return returnValue;
                }
            }
        },
        ctors: {
            ctor: function () {
                this.$initialize();

            }
        }
    });

    Bridge.define("System.Windows.Browser.HtmlPage", {
        statics: {
            fields: {
                _initWindow: null,
                _initDocument: null
            },
            props: {
                /**
                 * Gets the browser's window object.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof System.Windows.Browser.HtmlPage
                 * @function Window
                 * @type System.Windows.Browser.HtmlWindow
                 */
                Window: {
                    get: function () {
                        if (System.Windows.Browser.HtmlPage._initWindow == null) {
                            System.Windows.Browser.HtmlPage._initWindow = new System.Windows.Browser.HtmlWindow();
                        }
                        return System.Windows.Browser.HtmlPage._initWindow;
                    }
                },
                /**
                 * Gets the browser's document object.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof System.Windows.Browser.HtmlPage
                 * @function Document
                 * @type System.Windows.Browser.HtmlDocument
                 */
                Document: {
                    get: function () {
                        if (System.Windows.Browser.HtmlPage._initDocument == null) {
                            System.Windows.Browser.HtmlPage._initDocument = new System.Windows.Browser.HtmlDocument();
                        }
                        return System.Windows.Browser.HtmlPage._initDocument;
                    }
                }
            }
        }
    });

    Bridge.define("System.Windows.Browser.HtmlWindow", {
        ctors: {
            ctor: function () {
                this.$initialize();

            }
        },
        methods: {
            /**
             * Opens the specified page in the specified browser instance, with the indicated user interface features.
             *
             * @instance
             * @public
             * @this System.Windows.Browser.HtmlWindow
             * @memberof System.Windows.Browser.HtmlWindow
             * @param   {System.Uri}    navigateToUri     
             * @param   {string}        target            
             * @param   {string}        targetFeatures
             * @return  {void}
             */
            Navigate: function (navigateToUri, target, targetFeatures) {
                if (target === void 0) { target = "_self"; }
                if (targetFeatures === void 0) { targetFeatures = ""; }
                if (System.Uri.notEquals(navigateToUri, null) && target != null && targetFeatures != null) {
                    if (!Bridge.referenceEquals(target, "_search")) {
                        if (Bridge.referenceEquals(target, "_blank") && CSHTML5.Interop.IsRunningInTheSimulator) {
                            DotNetForHtml5.Core.INTERNAL_Simulator.SimulatorProxy.NavigateToUrlInNewBrowserWindow(Bridge.toString(navigateToUri));
                        } else {
                            if (Bridge.referenceEquals(target, "")) {
                                target = "_self";
                            }

window.open(                            Bridge.toString(navigateToUri), target, targetFeatures);
                        }
                    } else {
                        throw new System.NotImplementedException.$ctor1("The search target is not implemented.");
                    }
                } else {
                    throw new System.ArgumentNullException.ctor();
                }

            },
            /**
             * Evaluates a string that contains arbitrary JavaScript code.
             *
             * @instance
             * @public
             * @this System.Windows.Browser.HtmlWindow
             * @memberof System.Windows.Browser.HtmlWindow
             * @param   {string}           code    Javascript code.
             * @return  {System.Object}            The results of the JavaScript engine's evaluation of the string in the code parameter.
             */
            Eval: function (code) {
                eval(code)
            }
        }
    });

    /** @namespace System.Windows.Browser */

    /**
     * Provides methods for encoding and decoding HTML and URL strings.
     *
     * @static
     * @abstract
     * @public
     * @class System.Windows.Browser.HttpUtility
     */
    Bridge.define("System.Windows.Browser.HttpUtility", {
        statics: {
            fields: {
                codeNamesToChars: null
            },
            ctors: {
                ctor: function () {
                    System.Windows.Browser.HttpUtility.codeNamesToChars = function (_o1) {
                            _o1.add("&nbsp;", " ");
                            _o1.add("&quot;", "\"");
                            _o1.add("&amp;", "&");
                            _o1.add("&apos;", "'");
                            _o1.add("&lt;", "<");
                            _o1.add("&gt;", ">");
                            _o1.add("&iexcl;", "\u00a1");
                            _o1.add("&cent;", "\u00a2");
                            _o1.add("&pound;", "\u00a3");
                            _o1.add("&curren;", "\u00a4");
                            _o1.add("&yen;", "\u00a5");
                            _o1.add("&brvbar;", "\u00a6");
                            _o1.add("&sect;", "\u00a7");
                            _o1.add("&uml;", "\u00a8");
                            _o1.add("&copy;", "\u00a9");
                            _o1.add("&ordf;", "\u00aa");
                            _o1.add("&laquo;", "\u00ab");
                            _o1.add("&not;", "\u00ac");
                            _o1.add("&shy;", "\u00ad");
                            _o1.add("&reg;", "\u00ae");
                            _o1.add("&macr;", "\u00af");
                            _o1.add("&deg;", "\u00b0");
                            _o1.add("&plusmn;", "\u00b1");
                            _o1.add("&sup2;", "\u00b2");
                            _o1.add("&sup3;", "\u00b3");
                            _o1.add("&acute;", "\u00b4");
                            _o1.add("&micro;", "\u00b5\u00ad");
                            _o1.add("&para;", "\u00b6");
                            _o1.add("&middot;", "\u00b7");
                            _o1.add("&cedil;", "\u00b8");
                            _o1.add("&sup1;", "\u00b9");
                            _o1.add("&ordm;", "\u00ba");
                            _o1.add("&raquo;", "\u00bb");
                            _o1.add("&frac14;", "\u00bc");
                            _o1.add("&frac12;", "\u00bd");
                            _o1.add("&frac34;", "\u00be");
                            _o1.add("&iquest;", "\u00bf");
                            _o1.add("&Agrave;", "\u00c0");
                            _o1.add("&Aacute;", "\u00c1");
                            _o1.add("&Acirc;", "\u00c2");
                            _o1.add("&Atilde;", "\u00c3");
                            _o1.add("&Auml;", "\u00c4");
                            _o1.add("&Aring;", "\u00c5");
                            _o1.add("&AElig;", "\u00c6");
                            _o1.add("&Ccedil;", "\u00c7");
                            _o1.add("&Egrave;", "\u00c8");
                            _o1.add("&Eacute;", "\u00c9");
                            _o1.add("&Ecirc;", "\u00ca");
                            _o1.add("&Euml;", "\u00cb");
                            _o1.add("&Igrave;", "\u00cc");
                            _o1.add("&Iacute;", "\u00cd");
                            _o1.add("&Icirc;", "\u00ce");
                            _o1.add("&Iuml;", "\u00cf");
                            _o1.add("&ETH;", "\u00d0");
                            _o1.add("&Ntilde;", "\u00d1");
                            _o1.add("&Ograve;", "\u00d2");
                            _o1.add("&Oacute;", "\u00d3");
                            _o1.add("&Ocirc;", "\u00d4");
                            _o1.add("&Otilde;", "\u00d5");
                            _o1.add("&Ouml;", "\u00d6");
                            _o1.add("&times;", "\u00d7");
                            _o1.add("&Oslash;", "\u00d8");
                            _o1.add("&Ugrave;", "\u00d9");
                            _o1.add("&Uacute;", "\u00da");
                            _o1.add("&Ucirc;", "\u00db");
                            _o1.add("&Uuml;", "\u00dc");
                            _o1.add("&Yacute;", "\u00dd");
                            _o1.add("&THORN;", "\u00de");
                            _o1.add("&szlig;", "\u00df");
                            _o1.add("&agrave;", "\u00e0");
                            _o1.add("&aacute;", "\u00e1");
                            _o1.add("&acirc;", "\u00e2");
                            _o1.add("&atilde;", "\u00e3");
                            _o1.add("&auml;", "\u00e4");
                            _o1.add("&aring;", "\u00e5");
                            _o1.add("&aelig;", "\u00e6");
                            _o1.add("&ccedil;", "\u00e7");
                            _o1.add("&egrave;", "\u00e8");
                            _o1.add("&eacute;", "\u00e9");
                            _o1.add("&ecirc;", "\u00ea");
                            _o1.add("&euml;", "\u00eb");
                            _o1.add("&igrave;", "\u00ec");
                            _o1.add("&iacute;", "\u00ed");
                            _o1.add("&icirc;", "\u00ee");
                            _o1.add("&iuml;", "\u00ef");
                            _o1.add("&eth;", "\u00f0");
                            _o1.add("&ntilde;", "\u00f1");
                            _o1.add("&ograve;", "\u00f2");
                            _o1.add("&oacute;", "\u00f3");
                            _o1.add("&ocirc;", "\u00f4");
                            _o1.add("&otilde;", "\u00f5");
                            _o1.add("&ouml;", "\u00f6");
                            _o1.add("&divide;", "\u00f7");
                            _o1.add("&oslash;", "\u00f8");
                            _o1.add("&ugrave;", "\u00f9");
                            _o1.add("&uacute;", "\u00fa");
                            _o1.add("&ucirc;", "\u00fb");
                            _o1.add("&uuml;", "\u00fc");
                            _o1.add("&yacute;", "\u00fd");
                            _o1.add("&thorn;", "\u00fe");
                            _o1.add("&yuml;", "\u00ff");
                            _o1.add("&bull;", "\u2022");
                            _o1.add("&infin;", "\u221e");
                            _o1.add("&permil;", "\u2030");
                            _o1.add("&sdot;", "\u22c5");
                            _o1.add("&dagger;", "\u2020");
                            _o1.add("&mdash;", "\u2014");
                            _o1.add("&perp;", "\u22a5");
                            _o1.add("&par;", "\u2225");
                            _o1.add("&euro;", "\u20ac");
                            _o1.add("&trade;", "\u2122");
                            _o1.add("&alpha;", "\u03b1");
                            _o1.add("&beta;", "\u03b2");
                            _o1.add("&gamma;", "\u03b3");
                            _o1.add("&delta;", "\u03b4");
                            _o1.add("&epsilon;", "\u03b5");
                            _o1.add("&zeta;", "\u03b6");
                            _o1.add("&eta;", "\u03b7");
                            _o1.add("&theta;", "\u03b8");
                            _o1.add("&iota;", "\u03b9");
                            _o1.add("&kappa;", "\u03ba");
                            _o1.add("&lambda;", "\u03bb");
                            _o1.add("&mu;", "\u03bc");
                            _o1.add("&nu;", "\u03bd");
                            _o1.add("&xi;", "\u03be");
                            _o1.add("&omicron;", "\u03bf");
                            _o1.add("&pi;", "\u03c0");
                            _o1.add("&rho;", "\u03c1");
                            _o1.add("&sigma;", "\u03c3");
                            _o1.add("&tau;", "\u03c4");
                            _o1.add("&upsilon;", "\u03c5");
                            _o1.add("&phi;", "\u03c6");
                            _o1.add("&chi;", "\u03c7");
                            _o1.add("&psi;", "\u03c8");
                            _o1.add("&omega;", "\u03c9");
                            _o1.add("&Alpha;", "\u0391");
                            _o1.add("&Beta;", "\u0392");
                            _o1.add("&Gamma;", "\u0393");
                            _o1.add("&Delta;", "\u0394");
                            _o1.add("&Epsilon;", "\u0395");
                            _o1.add("&Zeta;", "\u0396");
                            _o1.add("&Eta;", "\u0397");
                            _o1.add("&Theta;", "\u0398");
                            _o1.add("&Iota;", "\u0399");
                            _o1.add("&Kappa;", "\u039a");
                            _o1.add("&Lambda;", "\u039b");
                            _o1.add("&Mu;", "\u039c");
                            _o1.add("&Nu;", "\u039d");
                            _o1.add("&Xi;", "\u039e");
                            _o1.add("&Omicron;", "\u039f");
                            _o1.add("&Pi;", "\u03a0");
                            _o1.add("&Rho;", "\u03a1");
                            _o1.add("&Sigma;", "\u03a3");
                            _o1.add("&Tau;", "\u03a4");
                            _o1.add("&Upsilon;", "\u03a5");
                            _o1.add("&Phi;", "\u03a6");
                            _o1.add("&Chi;", "\u03a7");
                            _o1.add("&Psi;", "\u03a8");
                            _o1.add("&Omega;", "\u03a9");
                            return _o1;
                        }(new (System.Collections.Generic.Dictionary$2(System.String,System.String)).ctor());
                }
            },
            methods: {
                /**
                 * Converts a string that has been HTML-encoded (for HTTP transmission) into
                 a decoded string.
                 *
                 * @static
                 * @public
                 * @this System.Windows.Browser.HttpUtility
                 * @memberof System.Windows.Browser.HttpUtility
                 * @param   {string}    html    An HTML-encoded string to decode.
                 * @return  {string}            A decoded string.
                 */
                HtmlDecode: function (html) {
                    if (System.String.contains(html,"&>#38;")) {
                        throw new System.FormatException.$ctor1("The html string to decode is in an incorrect format.");
                    }

                    var decodedHtml = System.String.replaceAll(html, "&#38;", "&>#38;");

                    if (System.String.contains(decodedHtml,"&>#x26;")) {
                        throw new System.FormatException.$ctor1("The html string to decode is in an incorrect format.");
                    }

                    decodedHtml = System.String.replaceAll(decodedHtml, "&#x26;", "&>#x26;");


                    decodedHtml = System.Convert.toString(decodedHtml.replace(/&#\d+;/gim, function(i) { return String.fromCharCode(i.substring(2,i.length-1));} ));

                    decodedHtml = System.Convert.toString(decodedHtml.replace(/&#x[a-fA-F0-9]+;/gim, function(i) { return String.fromCharCode(parseInt(i.substring(3,i.length-1), 16));} ));

                    var i = 0;
                    while (i !== -1 && i < decodedHtml.length) {
                        i = System.String.indexOf(decodedHtml, String.fromCharCode(38), i);
                        if (i !== -1) {
                            var j = System.String.indexOf(decodedHtml, String.fromCharCode(59), ((i + 1) | 0));
                            if (j !== -1) {
                                var sub = decodedHtml.substr(i, ((((j - i) | 0) + 1) | 0));
                                if (System.Windows.Browser.HttpUtility.codeNamesToChars.containsKey(sub)) {
                                    decodedHtml = System.String.replaceAll(decodedHtml, sub, System.Windows.Browser.HttpUtility.codeNamesToChars.getItem(sub));
                                }
                                i = (i + 1) | 0;
                            } else {
                                i = -1;
                            }
                        }
                    }


                    decodedHtml = System.String.replaceAll(decodedHtml, "&>#38;", "&");

                    return decodedHtml;
                },
                /**
                 * Converts a text string into an HTML-encoded string.
                 *
                 * @static
                 * @public
                 * @this System.Windows.Browser.HttpUtility
                 * @memberof System.Windows.Browser.HttpUtility
                 * @param   {string}    html    The text to HTML-encode.
                 * @return  {string}            An HTML-encoded string.
                 */
                HtmlEncode: function (html) {
                    var encodedHtml = System.Convert.toString(html.replace(/[^a-zA-Z0-9]/gim, function(i) { return '&#' + i.charCodeAt(0) + ';'} ));
                    return encodedHtml;

                }
            }
        }
    });

    /** @namespace System.Windows */

    /**
     * Provides static methods that facilitate transferring data to and from the
     system clipboard. In Silverlight 5, this access is limited to Unicode text
     strings.
     *
     * @static
     * @abstract
     * @public
     * @class System.Windows.Clipboard
     */
    Bridge.define("System.Windows.Clipboard", {
        statics: {
            methods: {
                /**
                 * Sets text data to store on the clipboard.
                 *
                 * @static
                 * @public
                 * @this System.Windows.Clipboard
                 * @memberof System.Windows.Clipboard
                 * @param   {string}    text    A string that contains the Unicode text data to store on the clipboard.
                 * @return  {void}
                 */
                SetText: function (text) {
                    if (CSHTML5.Interop.IsRunningInTheSimulator) {
                        DotNetForHtml5.Core.INTERNAL_Simulator.ClipboardHandler.SetText(text);
                    } else {

                  // IE specific
                  if (window.clipboardData && window.clipboardData.setData) {
                    return clipboardData.setData("Text",                         text);
                  }

                  // all other modern
                  var target = document.createElement("textarea");
                  target.style.position = "absolute";
                  target.style.left = "-9999px";
                  target.style.top = "0";
                  target.textContent = text;
                  document.body.appendChild(target);
                  target.focus();
                  target.setSelectionRange(0, target.value.length);

                  // copy the selection of fall back to prompt
                  try {
                    document.execCommand("copy");
                    target.remove();
                  } catch(e) {
                    window.prompt("Please confirm that you would like to copy to the clipboard by pressing Ctrl+C now. The data below will be copied to the clipboard.", text);
                  }
                    }
                }
            }
        }
    });

    /** @namespace System.Windows.Controls */

    /**
     * The {@link } namespace provides classes to create
     elements, known as controls, that enable a user to interact with an application.
     The control classes are at the core of the user's experience with any application
     because they allow a user to view, select, or enter data or other information.
     *
     * @class System.Windows.Controls.NamespaceDoc
     */
    Bridge.define("System.Windows.Controls.NamespaceDoc");

    /** @namespace System.Windows.Controls.Primitives */

    /**
     * The {@link } namespace contains base classes
     and controls that are intended to be used as part of other more complex controls.
     *
     * @class System.Windows.Controls.Primitives.NamespaceDoc
     */
    Bridge.define("System.Windows.Controls.Primitives.NamespaceDoc");

    /**
     * Provides a format-independent mechanism for transferring data.
     *
     * @abstract
     * @public
     * @class System.Windows.IDataObject
     */
    Bridge.define("System.Windows.IDataObject", {
        $kind: "interface"
    });

    /**
     * Provides helper methods and fields for drag-and-drop operations.
     *
     * @static
     * @abstract
     * @public
     * @class System.Windows.DragDrop
     */
    Bridge.define("System.Windows.DragDrop", {
        statics: {
            props: {
                /**
                 * Gets a value indicating whether a drag is in progress.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof System.Windows.DragDrop
                 * @function IsDragInProgress
                 * @type boolean
                 */
                IsDragInProgress: {
                    get: function () {
                        return (Windows.UI.Xaml.Input.Pointer.INTERNAL_captured != null);
                    }
                }
            }
        }
    });

    /**
     * Specifies the effects of a drag-and-drop operation.
     *
     * @public
     * @class System.Windows.DragDropEffects
     */
    Bridge.define("System.Windows.DragDropEffects", {
        $kind: "enum",
        statics: {
            fields: {
                /**
                 * The drop target does not accept the data.
                 *
                 * @static
                 * @public
                 * @memberof System.Windows.DragDropEffects
                 * @constant
                 * @default 0
                 * @type System.Windows.DragDropEffects
                 */
                None: 0
            }
        },
        $flags: true
    });

    Bridge.define("System.Windows.ExtendedRoutedEventArgs", {
        fields: {
            /**
             * Gets or sets a value indicating whether the present state of the 
             event handling for a routed event as it travels the route.
             *
             * @instance
             * @public
             * @memberof System.Windows.ExtendedRoutedEventArgs
             * @function Handled
             * @type boolean
             */
            Handled: false
        }
    });

    /** @namespace System.Windows.Input */

    /**
     * Represents the image used for the mouse pointer.
     *
     * @public
     * @class System.Windows.Input.Cursor
     * @implements  System.IDisposable
     */
    Bridge.define("System.Windows.Input.Cursor", {
        inherits: [System.IDisposable],
        statics: {
            ctors: {
                ctor: function () {
                    DotNetForHtml5.Core.TypeFromStringConverters.RegisterConverter(System.Windows.Input.Cursor, System.Windows.Input.Cursor.INTERNAL_ConvertFromString);
                    System.Windows.Input.Cursors.FillCursorTypeToStringDictionary();
                }
            },
            methods: {
                /**
                 * // Returns the string representation of the System.Windows.Input.Cursor.
                //
                 *
                 * @static
                 * @this System.Windows.Input.Cursor
                 * @memberof System.Windows.Input.Cursor
                 * @param   {string}           cursorTypeString
                 * @return  {System.Object}                        The name of the cursor.
                 */
                INTERNAL_ConvertFromString: function (cursorTypeString) {
                    try {
                        var cursorType = System.Nullable.getValue(Bridge.cast(Bridge.unbox(System.Enum.parse(System.Windows.Input.Cursors.INTERNAL_CursorsEnum, cursorTypeString), System.Windows.Input.Cursors.INTERNAL_CursorsEnum), System.Int32));
                        var cursor = new System.Windows.Input.Cursor();
                        cursor._cursorHtmlString = System.Windows.Input.Cursors.INTERNAL_cursorEnumToCursorString.getItem(cursorType);
                        return cursor;
                    } catch (ex) {
                        ex = System.Exception.create(ex);
                        throw new System.Exception("Invalid cursor: " + (cursorTypeString || ""), ex);
                    }
                }
            }
        },
        fields: {
            _isCustom: false,
            _cursorHtmlString: null
        },
        alias: ["Dispose", "System$IDisposable$Dispose"],
        ctors: {
            init: function () {
                this._isCustom = false;
            },
            ctor: function () {
                this.$initialize();

            }
        },
        methods: {
            /**
             * // Initializes a new instance of the System.Windows.Input.Cursor class from
            // the specified System.IO.Stream.
            //
             * // Initializes a new instance of the System.Windows.Input.Cursor class from
            // the specified .ani or a .cur file.
            //
             * Releases the resources used by the System.Windows.Input.Cursor class.
             *
             * @instance
             * @public
             * @this System.Windows.Input.Cursor
             * @memberof System.Windows.Input.Cursor
             * @return  {void}
             */
            Dispose: function () { }
        }
    });

    /**
     * Defines a set of default cursors.
     *
     * @static
     * @abstract
     * @public
     * @class System.Windows.Input.Cursors
     */
    Bridge.define("System.Windows.Input.Cursors", {
        statics: {
            fields: {
                INTERNAL_cursorEnumToCursorString: null
            },
            props: {
                /**
                 * Gets the System.Windows.Input.Cursor that appears when an application is
                 starting.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof System.Windows.Input.Cursors
                 * @function AppStarting
                 * @type System.Windows.Input.Cursor
                 */
                AppStarting: {
                    get: function () {
                        var cursor = new System.Windows.Input.Cursor();
                        cursor._cursorHtmlString = System.Windows.Input.Cursors.INTERNAL_cursorEnumToCursorString.getItem(System.Windows.Input.Cursors.INTERNAL_CursorsEnum.AppStarting);
                        return cursor;
                    }
                },
                /**
                 * Gets the Arrow System.Windows.Input.Cursor.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof System.Windows.Input.Cursors
                 * @function Arrow
                 * @type System.Windows.Input.Cursor
                 */
                Arrow: {
                    get: function () {
                        var cursor = new System.Windows.Input.Cursor();
                        cursor._cursorHtmlString = System.Windows.Input.Cursors.INTERNAL_cursorEnumToCursorString.getItem(System.Windows.Input.Cursors.INTERNAL_CursorsEnum.Arrow);
                        return cursor;
                    }
                },
                /**
                 * // Gets the arrow with a compact disk System.Windows.Input.Cursor.
                //
                 * Gets the crosshair System.Windows.Input.Cursor.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof System.Windows.Input.Cursors
                 * @function Cross
                 * @type System.Windows.Input.Cursor
                 */
                Cross: {
                    get: function () {
                        var cursor = new System.Windows.Input.Cursor();
                        cursor._cursorHtmlString = System.Windows.Input.Cursors.INTERNAL_cursorEnumToCursorString.getItem(System.Windows.Input.Cursors.INTERNAL_CursorsEnum.Cross);
                        return cursor;
                    }
                },
                /**
                 * Gets a hand System.Windows.Input.Cursor.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof System.Windows.Input.Cursors
                 * @function Hand
                 * @type System.Windows.Input.Cursor
                 */
                Hand: {
                    get: function () {
                        var cursor = new System.Windows.Input.Cursor();
                        cursor._cursorHtmlString = System.Windows.Input.Cursors.INTERNAL_cursorEnumToCursorString.getItem(System.Windows.Input.Cursors.INTERNAL_CursorsEnum.Hand);
                        return cursor;
                    }
                },
                /**
                 * Gets a help System.Windows.Input.Cursor which is a combination of an arrow
                  and a question mark.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof System.Windows.Input.Cursors
                 * @function Help
                 * @type System.Windows.Input.Cursor
                 */
                Help: {
                    get: function () {
                        var cursor = new System.Windows.Input.Cursor();
                        cursor._cursorHtmlString = System.Windows.Input.Cursors.INTERNAL_cursorEnumToCursorString.getItem(System.Windows.Input.Cursors.INTERNAL_CursorsEnum.Help);
                        return cursor;
                    }
                },
                /**
                 * Gets an I-beam System.Windows.Input.Cursor, which is used to show where the
                 text cursor appears when the mouse is clicked.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof System.Windows.Input.Cursors
                 * @function IBeam
                 * @type System.Windows.Input.Cursor
                 */
                IBeam: {
                    get: function () {
                        var cursor = new System.Windows.Input.Cursor();
                        cursor._cursorHtmlString = System.Windows.Input.Cursors.INTERNAL_cursorEnumToCursorString.getItem(System.Windows.Input.Cursors.INTERNAL_CursorsEnum.IBeam);
                        return cursor;
                    }
                },
                /**
                 * Gets a System.Windows.Input.Cursor with which indicates that a particular
                 region is invalid for a given operation.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof System.Windows.Input.Cursors
                 * @function No
                 * @type System.Windows.Input.Cursor
                 */
                No: {
                    get: function () {
                        var cursor = new System.Windows.Input.Cursor();
                        cursor._cursorHtmlString = System.Windows.Input.Cursors.INTERNAL_cursorEnumToCursorString.getItem(System.Windows.Input.Cursors.INTERNAL_CursorsEnum.No);
                        return cursor;
                    }
                },
                /**
                 * Gets a special cursor that is invisible.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof System.Windows.Input.Cursors
                 * @function None
                 * @type System.Windows.Input.Cursor
                 */
                None: {
                    get: function () {
                        var cursor = new System.Windows.Input.Cursor();
                        cursor._cursorHtmlString = System.Windows.Input.Cursors.INTERNAL_cursorEnumToCursorString.getItem(System.Windows.Input.Cursors.INTERNAL_CursorsEnum.None);
                        return cursor;
                    }
                },
                /**
                 * // Gets a pen System.Windows.Input.Cursor.
                //
                 * // Gets the scroll all System.Windows.Input.Cursor.
                //
                 * // Gets the scroll east System.Windows.Input.Cursor.
                //
                 * // Gets the scroll north System.Windows.Input.Cursor.
                //
                 * // Gets the scroll northeast cursor.
                //
                 * // Gets the scroll north/south cursor.
                //
                 * // Gets a scroll northwest cursor.
                //
                 * // Gets the scroll south System.Windows.Input.Cursor.
                //
                 * // Gets a south/east scrolling System.Windows.Input.Cursor.
                //
                 * // Gets the scroll southwest System.Windows.Input.Cursor.
                //
                 * // Gets the scroll west System.Windows.Input.Cursor.
                //
                 * // Gets a west/east scrolling System.Windows.Input.Cursor.
                //
                 * Gets a four-headed sizing System.Windows.Input.Cursor, which consists of
                 four joined arrows that point north, south, east, and west.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof System.Windows.Input.Cursors
                 * @function SizeAll
                 * @type System.Windows.Input.Cursor
                 */
                SizeAll: {
                    get: function () {
                        var cursor = new System.Windows.Input.Cursor();
                        cursor._cursorHtmlString = System.Windows.Input.Cursors.INTERNAL_cursorEnumToCursorString.getItem(System.Windows.Input.Cursors.INTERNAL_CursorsEnum.SizeAll);
                        return cursor;
                    }
                },
                /**
                 * Gets a two-headed northeast/southwest sizing System.Windows.Input.Cursor.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof System.Windows.Input.Cursors
                 * @function SizeNESW
                 * @type System.Windows.Input.Cursor
                 */
                SizeNESW: {
                    get: function () {
                        var cursor = new System.Windows.Input.Cursor();
                        cursor._cursorHtmlString = System.Windows.Input.Cursors.INTERNAL_cursorEnumToCursorString.getItem(System.Windows.Input.Cursors.INTERNAL_CursorsEnum.SizeNESW);
                        return cursor;
                    }
                },
                /**
                 * Gets a two-headed north/south sizing System.Windows.Input.Cursor.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof System.Windows.Input.Cursors
                 * @function SizeNS
                 * @type System.Windows.Input.Cursor
                 */
                SizeNS: {
                    get: function () {
                        var cursor = new System.Windows.Input.Cursor();
                        cursor._cursorHtmlString = System.Windows.Input.Cursors.INTERNAL_cursorEnumToCursorString.getItem(System.Windows.Input.Cursors.INTERNAL_CursorsEnum.SizeNS);
                        return cursor;
                    }
                },
                /**
                 * Gets a two-headed northwest/southeast sizing System.Windows.Input.Cursor.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof System.Windows.Input.Cursors
                 * @function SizeNWSE
                 * @type System.Windows.Input.Cursor
                 */
                SizeNWSE: {
                    get: function () {
                        var cursor = new System.Windows.Input.Cursor();
                        cursor._cursorHtmlString = System.Windows.Input.Cursors.INTERNAL_cursorEnumToCursorString.getItem(System.Windows.Input.Cursors.INTERNAL_CursorsEnum.SizeNWSE);
                        return cursor;
                    }
                },
                /**
                 * Gets a two-headed west/east sizing System.Windows.Input.Cursor.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof System.Windows.Input.Cursors
                 * @function SizeWE
                 * @type System.Windows.Input.Cursor
                 */
                SizeWE: {
                    get: function () {
                        var cursor = new System.Windows.Input.Cursor();
                        cursor._cursorHtmlString = System.Windows.Input.Cursors.INTERNAL_cursorEnumToCursorString.getItem(System.Windows.Input.Cursors.INTERNAL_CursorsEnum.SizeWE);
                        return cursor;
                    }
                },
                /**
                 * // Gets an up arrow System.Windows.Input.Cursor, which is typically used to
                // identify an insertion point.
                //
                 * Specifies a wait (or hourglass) System.Windows.Input.Cursor.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof System.Windows.Input.Cursors
                 * @function Wait
                 * @type System.Windows.Input.Cursor
                 */
                Wait: {
                    get: function () {
                        var cursor = new System.Windows.Input.Cursor();
                        cursor._cursorHtmlString = System.Windows.Input.Cursors.INTERNAL_cursorEnumToCursorString.getItem(System.Windows.Input.Cursors.INTERNAL_CursorsEnum.Wait);
                        return cursor;
                    }
                }
            },
            methods: {
                FillCursorTypeToStringDictionary: function () {
                    if (System.Windows.Input.Cursors.INTERNAL_cursorEnumToCursorString == null) {
                        System.Windows.Input.Cursors.INTERNAL_cursorEnumToCursorString = new (System.Collections.Generic.Dictionary$2(System.Windows.Input.Cursors.INTERNAL_CursorsEnum,System.String)).ctor();
                    }
                    System.Windows.Input.Cursors.INTERNAL_cursorEnumToCursorString.add(System.Windows.Input.Cursors.INTERNAL_CursorsEnum.AppStarting, "progress");
                    System.Windows.Input.Cursors.INTERNAL_cursorEnumToCursorString.add(System.Windows.Input.Cursors.INTERNAL_CursorsEnum.Arrow, "default");
                    System.Windows.Input.Cursors.INTERNAL_cursorEnumToCursorString.add(System.Windows.Input.Cursors.INTERNAL_CursorsEnum.Cross, "crosshair");
                    System.Windows.Input.Cursors.INTERNAL_cursorEnumToCursorString.add(System.Windows.Input.Cursors.INTERNAL_CursorsEnum.Hand, "pointer");
                    System.Windows.Input.Cursors.INTERNAL_cursorEnumToCursorString.add(System.Windows.Input.Cursors.INTERNAL_CursorsEnum.Help, "help");
                    System.Windows.Input.Cursors.INTERNAL_cursorEnumToCursorString.add(System.Windows.Input.Cursors.INTERNAL_CursorsEnum.IBeam, "text");
                    System.Windows.Input.Cursors.INTERNAL_cursorEnumToCursorString.add(System.Windows.Input.Cursors.INTERNAL_CursorsEnum.No, "not-allowed");
                    System.Windows.Input.Cursors.INTERNAL_cursorEnumToCursorString.add(System.Windows.Input.Cursors.INTERNAL_CursorsEnum.None, "none");
                    System.Windows.Input.Cursors.INTERNAL_cursorEnumToCursorString.add(System.Windows.Input.Cursors.INTERNAL_CursorsEnum.SizeAll, "move");
                    System.Windows.Input.Cursors.INTERNAL_cursorEnumToCursorString.add(System.Windows.Input.Cursors.INTERNAL_CursorsEnum.SizeNESW, "nesw-resize");
                    System.Windows.Input.Cursors.INTERNAL_cursorEnumToCursorString.add(System.Windows.Input.Cursors.INTERNAL_CursorsEnum.SizeNS, "ns-resize");
                    System.Windows.Input.Cursors.INTERNAL_cursorEnumToCursorString.add(System.Windows.Input.Cursors.INTERNAL_CursorsEnum.SizeNWSE, "nwse-resize");
                    System.Windows.Input.Cursors.INTERNAL_cursorEnumToCursorString.add(System.Windows.Input.Cursors.INTERNAL_CursorsEnum.SizeWE, "ew-resize");
                    System.Windows.Input.Cursors.INTERNAL_cursorEnumToCursorString.add(System.Windows.Input.Cursors.INTERNAL_CursorsEnum.Wait, "wait");
                }
            }
        }
    });

    Bridge.define("System.Windows.Input.Cursors.INTERNAL_CursorsEnum", {
        $kind: "nested enum",
        statics: {
            fields: {
                AppStarting: 0,
                Arrow: 1,
                Cross: 2,
                Hand: 3,
                Help: 4,
                IBeam: 5,
                No: 6,
                None: 7,
                SizeAll: 8,
                SizeNESW: 9,
                SizeNS: 10,
                SizeNWSE: 11,
                SizeWE: 12,
                Wait: 13
            }
        }
    });

    /**
     * Defines a command.
     *
     * @abstract
     * @public
     * @class System.Windows.Input.ICommand
     */
    Bridge.define("System.Windows.Input.ICommand", {
        $kind: "interface"
    });

    /**
     * Represents the keyboard device.
     *
     * @static
     * @abstract
     * @public
     * @class System.Windows.Input.Keyboard
     */
    Bridge.define("System.Windows.Input.Keyboard", {
        statics: {
            props: {
                /**
                 * Gets the set of System.Windows.Input.ModifierKeys that are currently pressed.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof System.Windows.Input.Keyboard
                 * @function Modifiers
                 * @type Windows.System.VirtualKeyModifiers
                 */
                Modifiers: {
                    get: function () {
                        var value = System.Convert.toInt32(document.modifiersPressed);
                        return value;
                    }
                }
            }
        }
    });

    /** @namespace System.Windows.Interactivity */

    /**
     * An interface for an object that can be attached to another object.
     *
     * @abstract
     * @public
     * @class System.Windows.Interactivity.IAttachedObject
     */
    Bridge.define("System.Windows.Interactivity.IAttachedObject", {
        $kind: "interface"
    });

    /**
     * Static class that owns the Triggers and Behaviors attached properties. Handles
     propagation of AssociatedObject change notifications.
     *
     * @static
     * @abstract
     * @public
     * @class System.Windows.Interactivity.Interaction
     */
    Bridge.define("System.Windows.Interactivity.Interaction", {
        statics: {
            fields: {
                /**
                 * This property is used as the internal backing store for the public Behaviors
                 attached property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof System.Windows.Interactivity.Interaction
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                BehaviorsProperty: null,
                /**
                 * This property is used as the internal backing store for the public Triggers attached property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof System.Windows.Interactivity.Interaction
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                TriggersProperty: null
            },
            ctors: {
                init: function () {
                    var $t;
                    this.BehaviorsProperty = Windows.UI.Xaml.DependencyProperty.RegisterAttached("Behaviors", System.Windows.Interactivity.BehaviorCollection, System.Windows.Interactivity.Interaction, new Windows.UI.Xaml.PropertyMetadata.$ctor4(null));
                    this.TriggersProperty = Windows.UI.Xaml.DependencyProperty.RegisterAttached("Triggers", System.Windows.Interactivity.TriggerCollection, System.Windows.Interactivity.Interaction, ($t = new Windows.UI.Xaml.PropertyMetadata.$ctor2(null, System.Windows.Interactivity.Interaction.OnTriggersChanged), $t.CallPropertyChangedWhenLoadedIntoVisualTree = Windows.UI.Xaml.WhenToCallPropertyChangedEnum.IfPropertyIsSet, $t));
                }
            },
            methods: {
                /**
                 * Gets the System.Windows.Interactivity.BehaviorCollection associated with
                 a specified object.
                 *
                 * @static
                 * @public
                 * @this System.Windows.Interactivity.Interaction
                 * @memberof System.Windows.Interactivity.Interaction
                 * @param   {Windows.UI.Xaml.DependencyObject}                   obj    The object from which to retrieve the System.Windows.Interactivity.BehaviorCollection.
                 * @return  {System.Windows.Interactivity.BehaviorCollection}           A System.Windows.Interactivity.BehaviorCollection containing the behaviors
                 associated with the specified object.
                 */
                GetBehaviors: function (obj) {
                    if (obj.GetValue(System.Windows.Interactivity.Interaction.BehaviorsProperty) == null) {
                        var col = new System.Windows.Interactivity.BehaviorCollection();
                        col.Attach(obj);
                        obj.SetValue(System.Windows.Interactivity.Interaction.BehaviorsProperty, col);
                    }
                    return Bridge.cast(obj.GetValue(System.Windows.Interactivity.Interaction.BehaviorsProperty), System.Windows.Interactivity.BehaviorCollection);
                },
                /**
                 * Gets the TriggerCollection containing the triggers associated with the specified object.
                 *
                 * @static
                 * @public
                 * @this System.Windows.Interactivity.Interaction
                 * @memberof System.Windows.Interactivity.Interaction
                 * @param   {Windows.UI.Xaml.DependencyObject}                  obj    The object from which to retrieve the triggers.
                 * @return  {System.Windows.Interactivity.TriggerCollection}           A TriggerCollection containing the triggers associated with the specified object.
                 */
                GetTriggers: function (obj) {
                    var triggerCollection = Bridge.cast(obj.GetValue(System.Windows.Interactivity.Interaction.TriggersProperty), System.Windows.Interactivity.TriggerCollection);
                    if (triggerCollection == null) {
                        triggerCollection = new System.Windows.Interactivity.TriggerCollection();
                        obj.SetValue(System.Windows.Interactivity.Interaction.TriggersProperty, triggerCollection);
                    }
                    return triggerCollection;
                },
                /**
                 * @static
                 * @private
                 * @this System.Windows.Interactivity.Interaction
                 * @memberof System.Windows.Interactivity.Interaction
                 * @throws Cannot host the same TriggerCollection on more than one object at a time.
                 * @param   {Windows.UI.Xaml.DependencyObject}                      obj     
                 * @param   {Windows.UI.Xaml.DependencyPropertyChangedEventArgs}    args
                 * @return  {void}
                 */
                OnTriggersChanged: function (obj, args) {
                    var triggerCollection1 = Bridge.as(args.OldValue, System.Windows.Interactivity.TriggerCollection);
                    var triggerCollection2 = Bridge.as(args.NewValue, System.Windows.Interactivity.TriggerCollection);
                    if (Bridge.referenceEquals(triggerCollection1, triggerCollection2)) {
                        return;
                    }
                    if (triggerCollection1 != null && triggerCollection1.AssociatedObject != null) {
                        triggerCollection1.Detach();
                    }
                    if (triggerCollection2 == null || obj == null) {
                        return;
                    }
                    if (triggerCollection2.AssociatedObject != null) {
                        throw new System.InvalidOperationException.$ctor1("Cannot Host TriggerCollection Multiple Times");
                    }
                    var fElement = Bridge.as(obj, Windows.UI.Xaml.FrameworkElement);
                    if (fElement == null) {
                        throw new System.InvalidOperationException.$ctor1("Can only host BehaviorCollection on FrameworkElement");
                    }
                    triggerCollection2.Attach(fElement);
                }
            }
        }
    });

    /**
     * Indicates which property of a type is the XAML content property. A XAML processor
     uses this information when processing XAML child elements of XAML representations
     of the attributed type.
     *
     * @public
     * @class System.Windows.Markup.ContentPropertyAttribute
     * @augments System.Attribute
     */
    Bridge.define("System.Windows.Markup.ContentPropertyAttribute", {
        inherits: [System.Attribute],
        fields: {
            Name: null
        },
        ctors: {
            /**
             * Initializes a new instance of the ContentPropertyAttribute class.
             *
             * @instance
             * @public
             * @this System.Windows.Markup.ContentPropertyAttribute
             * @memberof System.Windows.Markup.ContentPropertyAttribute
             * @return  {void}
             */
            ctor: function () {
                this.$initialize();
                System.Attribute.ctor.call(this);
            },
            /**
             * Initializes a new instance of the ContentPropertyAttribute class with the specified name.
             *
             * @instance
             * @public
             * @this System.Windows.Markup.ContentPropertyAttribute
             * @memberof System.Windows.Markup.ContentPropertyAttribute
             * @param   {string}    name    The name of the ContentPropertyAttribute
             * @return  {void}
             */
            $ctor1: function (name) {
                this.$initialize();
                System.Attribute.ctor.call(this);
                this.Name = name;
            }
        }
    });

    /**
     * Specifies one or more types on the associated collection type that will be
     used to wrap foreign content.
     *
     * @public
     * @class System.Windows.Markup.ContentWrapperAttribute
     * @augments System.Attribute
     */
    Bridge.define("System.Windows.Markup.ContentWrapperAttribute", {
        inherits: [System.Attribute],
        fields: {
            _contentWrapper: null
        },
        props: {
            /**
             * Gets the type that is declared as a content wrapper for the collection type
             associated with this attribute.
             *
             * @instance
             * @public
             * @readonly
             * @memberof System.Windows.Markup.ContentWrapperAttribute
             * @function ContentWrapper
             * @type System.Type
             */
            ContentWrapper: {
                get: function () {
                    return this._contentWrapper;
                }
            },
            /**
             * Gets a unique identifier for this attribute.
             *
             * @instance
             * @public
             * @readonly
             * @memberof System.Windows.Markup.ContentWrapperAttribute
             * @function TypeId
             * @type System.Object
             */
            TypeId: {
                get: function () {
                    return this;
                }
            }
        },
        ctors: {
            /**
             * Initializes a new instance of the ContentWrapperAttribute
             class.
             *
             * @instance
             * @public
             * @this System.Windows.Markup.ContentWrapperAttribute
             * @memberof System.Windows.Markup.ContentWrapperAttribute
             * @param   {System.Type}    contentWrapper    The System.Type that is declared as a content wrapper for the collection
             type.
             * @return  {void}
             */
            ctor: function (contentWrapper) {
                this.$initialize();
                System.Attribute.ctor.call(this);
                this._contentWrapper = contentWrapper;

            }
        },
        methods: {
            /**
             * Determines whether the specified ContentWrapperAttribute
             is equivalent this ContentWrapperAttribute by comparing
             the ContentWrapperAttribute.ContentWrapper properties.
             *
             * @instance
             * @public
             * @override
             * @this System.Windows.Markup.ContentWrapperAttribute
             * @memberof System.Windows.Markup.ContentWrapperAttribute
             * @param   {System.Object}    obj
             * @return  {boolean}
             */
            equals: function (obj) {
                var wrapperAttribute = Bridge.as(obj, System.Windows.Markup.ContentWrapperAttribute);
                if (wrapperAttribute == null) {
                    return false;
                } else {
                    return Bridge.referenceEquals(this._contentWrapper, wrapperAttribute._contentWrapper);
                }
            },
            /**
             * Gets a hash code for this instance.
             *
             * @instance
             * @public
             * @override
             * @this System.Windows.Markup.ContentWrapperAttribute
             * @memberof System.Windows.Markup.ContentWrapperAttribute
             * @return  {number}        An integer hash code
             */
            getHashCode: function () {
                return Bridge.getHashCode(this._contentWrapper);
            }
        }
    });

    /**
     * Provides a base class for XAML markup extension implementations that can
     be supported by .NET Framework XAML Services and other XAML readers and XAML
     writers.
     *
     * @abstract
     * @public
     * @class System.Windows.Markup.MarkupExtension
     */
    Bridge.define("System.Windows.Markup.MarkupExtension");

    Bridge.define("System.Windows.Markup.MethodToTranslateXamlValueToCSharpAttribute", {
        inherits: [System.Attribute],
        fields: {
            MethodName: null
        },
        ctors: {
            ctor: function () {
                this.$initialize();
                System.Attribute.ctor.call(this);
            },
            $ctor1: function (methodName) {
                this.$initialize();
                System.Attribute.ctor.call(this);
                this.MethodName = methodName;
            }
        }
    });

    /**
     * The {@link } namespace provides types to support XAML.
     Some of these types are located in WPF assemblies and are specific to WPF scenarios
     that involve XAML. Other types in this namespace provide support for .NET Framework
     XAML Services in general, and do not require referencing WPF assemblies.
     *
     * @class System.Windows.Markup.NamespaceDoc
     */
    Bridge.define("System.Windows.Markup.NamespaceDoc");

    /**
     * Indicates that the type supports direct content when used in XAML. A XAML processor
     uses this information when processing XAML child elements of XAML representations
     of the attributed type. The direct content is converted using the
     TypeFromStringConverter class.
     *
     * @public
     * @class System.Windows.Markup.SupportsDirectContentViaTypeFromStringConvertersAttribute
     * @augments System.Attribute
     */
    Bridge.define("System.Windows.Markup.SupportsDirectContentViaTypeFromStringConvertersAttribute", {
        inherits: [System.Attribute],
        ctors: {
            /**
             * Initializes a new instance of the SupportsDirectContentViaTypeFromStringConvertersAttribute class.
             *
             * @instance
             * @public
             * @this System.Windows.Markup.SupportsDirectContentViaTypeFromStringConvertersAttribute
             * @memberof System.Windows.Markup.SupportsDirectContentViaTypeFromStringConvertersAttribute
             * @return  {void}
             */
            ctor: function () {
                this.$initialize();
                System.Attribute.ctor.call(this);
            }
        }
    });

    Bridge.define("System.Windows.Markup.XamlParseException", {
        inherits: [System.SystemException],
        ctors: {
            /**
             * Initializes a new instance of the System.Windows.Markup.XamlParseException
             class.
             *
             * @instance
             * @public
             * @this System.Windows.Markup.XamlParseException
             * @memberof System.Windows.Markup.XamlParseException
             * @return  {void}
             */
            ctor: function () {
                this.$initialize();
                System.SystemException.ctor.call(this);
            },
            /**
             * Initializes a new instance of the System.Windows.Markup.XamlParseException
             class, using the specified exception message string.
             *
             * @instance
             * @public
             * @this System.Windows.Markup.XamlParseException
             * @memberof System.Windows.Markup.XamlParseException
             * @param   {string}    message    The exception message.
             * @return  {void}
             */
            $ctor1: function (message) {
                this.$initialize();
                System.SystemException.$ctor1.call(this, message);
            },
            /**
             * Initializes a new instance of the System.Windows.Markup.XamlParseException
             class, using the specified exception message string and inner exception.
             *
             * @instance
             * @public
             * @this System.Windows.Markup.XamlParseException
             * @memberof System.Windows.Markup.XamlParseException
             * @param   {string}              message           The exception message.
             * @param   {System.Exception}    innerException    The initial exception that occurred.
             * @return  {void}
             */
            $ctor2: function (message, innerException) {
                this.$initialize();
                System.SystemException.$ctor2.call(this, message, innerException);
            }
        }
    });

    /**
     * Specifies a mapping on a per-assembly basis between a XAML namespace and
     a CLR namespace, which is then used for type resolution by a XAML object
     writer or XAML schema context.
     *
     * @public
     * @class System.Windows.Markup.XmlnsDefinitionAttribute
     * @augments System.Attribute
     */
    Bridge.define("System.Windows.Markup.XmlnsDefinitionAttribute", {
        inherits: [System.Attribute],
        fields: {
            /**
             * Gets or sets the name of the assembly associated with the attribute.
             *
             * @instance
             * @public
             * @memberof System.Windows.Markup.XmlnsDefinitionAttribute
             * @function AssemblyName
             * @type string
             */
            AssemblyName: null,
            /**
             * Gets the XAML namespace identifier specified in this attribute.
             *
             * @instance
             * @public
             * @memberof System.Windows.Markup.XmlnsDefinitionAttribute
             * @function XmlNamespace
             * @type string
             */
            XmlNamespace: null,
            /**
             * Gets the string name of the CLR namespace specified in this attribute
             *
             * @instance
             * @public
             * @memberof System.Windows.Markup.XmlnsDefinitionAttribute
             * @function ClrNamespace
             * @type string
             */
            ClrNamespace: null
        },
        ctors: {
            /**
             * Initializes a new instance of the System.Windows.Markup.XmlnsDefinitionAttribute
             class.
             *
             * @instance
             * @public
             * @this System.Windows.Markup.XmlnsDefinitionAttribute
             * @memberof System.Windows.Markup.XmlnsDefinitionAttribute
             * @param   {string}    xmlNamespace    The XAML namespace identifier.
             * @param   {string}    clrNamespace    A string that references a CLR namespace name.
             * @return  {void}
             */
            ctor: function (xmlNamespace, clrNamespace) {
                this.$initialize();
                System.Attribute.ctor.call(this);
                this.XmlNamespace = xmlNamespace;
                this.ClrNamespace = clrNamespace;
            }
        }
    });

    /**
     * The {@link } namespaces contain types used in Windows Presentation
     Foundation (WPF) applications, including animation clients, user interface controls,
     data binding, and type conversion. System.Windows.Forms and its child namespaces are
     used for developing Windows Forms applications.
     *
     * @class System.Windows.NamespaceDoc
     */
    Bridge.define("System.Windows.NamespaceDoc");

    /** @namespace System.Windows.Resources */

    /**
     * Provides resource stream information for application resources or other packages
     obtained through the System.Windows.Application.GetResourceStream(System.Windows.Resources.StreamResourceInfo,System.Uri)
     method.
     *
     * @public
     * @class System.Windows.Resources.StreamResourceInfo
     */
    Bridge.define("System.Windows.Resources.StreamResourceInfo", {
        fields: {
            _contentType: null,
            _stream: null
        },
        props: {
            /**
             * Gets the content type of a stream.
             *
             * @instance
             * @public
             * @memberof System.Windows.Resources.StreamResourceInfo
             * @function ContentType
             * @type string
             */
            ContentType: {
                get: function () {
                    return this._contentType;
                },
                set: function (value) {
                    this._contentType = value;
                }
            },
            /**
             * Gets the actual stream of the resource.
             *
             * @instance
             * @public
             * @memberof System.Windows.Resources.StreamResourceInfo
             * @function Stream
             * @type System.IO.Stream
             */
            Stream: {
                get: function () {
                    return this._stream;
                },
                set: function (value) {
                    this._stream = value;
                }
            }
        },
        ctors: {
            /**
             * Initializes a new instance of the System.Windows.Resources.StreamResourceInfo
             class.
             *
             * @instance
             * @public
             * @this System.Windows.Resources.StreamResourceInfo
             * @memberof System.Windows.Resources.StreamResourceInfo
             * @return  {void}
             */
            ctor: function () {
                this.$initialize();
            },
            /**
             * Initializes a new instance of the System.Windows.Resources.StreamResourceInfo
             class based on a provided stream.
             *
             * @instance
             * @public
             * @this System.Windows.Resources.StreamResourceInfo
             * @memberof System.Windows.Resources.StreamResourceInfo
             * @param   {System.IO.Stream}    stream         The reference stream.
             * @param   {string}              contentType    The Multipurpose Internet Mail Extensions (MIME) content type of the stream.
             * @return  {void}
             */
            $ctor1: function (stream, contentType) {
                this.$initialize();
                this.Stream = stream;
                this.ContentType = contentType;
            }
        }
    });

    /** @namespace System.Xml */

    /**
     * Represents a reader that provides fast, noncached, forward-only access to
     XML data.
     *
     * @abstract
     * @public
     * @class System.Xml.XmlReader
     * @implements  System.IDisposable
     */
    Bridge.define("System.Xml.XmlReader", {
        inherits: [System.IDisposable],
        statics: {
            methods: {
                /**
                 * Creates a new System.Xml.XmlReader instance using the specified stream.
                 *
                 * @static
                 * @public
                 * @this System.Xml.XmlReader
                 * @memberof System.Xml.XmlReader
                 * @param   {System.IO.Stream}        input    The stream containing the XML data.The System.Xml.XmlReader scans the first
                 bytes of the stream looking for a byte order mark or other sign of encoding.
                 When encoding is determined, the encoding is used to continue reading the
                 stream, and processing continues parsing the input as a stream of (Unicode)
                 characters.
                 * @return  {System.Xml.XmlReader}             An System.Xml.XmlReader object used to read the data contained in the stream.
                 */
                Create: function (input) {
                    return System.Xml.Cshtml5_XmlReader.Create(input);
                },
                /**
                 * Creates a new System.Xml.XmlReader instance with specified URI.
                 *
                 * @static
                 * @public
                 * @this System.Xml.XmlReader
                 * @memberof System.Xml.XmlReader
                 * @param   {string}                  inputUri    The URI for the file containing the XML data. The System.Xml.XmlUrlResolver
                 class is used to convert the path to a canonical data representation.
                 * @return  {System.Xml.XmlReader}                An System.Xml.XmlReader object to read the XML data.
                 */
                Create$8: function (inputUri) {
                    throw new System.NotImplementedException.ctor();
                },
                /**
                 * Creates a new System.Xml.XmlReader instance with the specified System.IO.TextReader.
                 *
                 * @static
                 * @public
                 * @this System.Xml.XmlReader
                 * @memberof System.Xml.XmlReader
                 * @param   {System.IO.TextReader}    input    The System.IO.TextReader from which to read the XML data. Because a System.IO.TextReader
                 returns a stream of Unicode characters, the encoding specified in the XML
                 declaration is not used by the System.Xml.XmlReader to decode the data stream.
                 * @return  {System.Xml.XmlReader}             An System.Xml.XmlReader object to read the XML data.
                 */
                Create$4: function (input) {
                    throw new System.NotImplementedException.ctor();
                },
                /**
                 * Creates a new System.Xml.XmlReader instance with the specified stream and
                 System.Xml.XmlReaderSettings object.
                 *
                 * @static
                 * @public
                 * @this System.Xml.XmlReader
                 * @memberof System.Xml.XmlReader
                 * @param   {System.IO.Stream}                input       The stream containing the XML data.The System.Xml.XmlReader scans the first
                 bytes of the stream looking for a byte order mark or other sign of encoding.
                 When encoding is determined, the encoding is used to continue reading the
                 stream, and processing continues parsing the input as a stream of (Unicode)
                 characters.
                 * @param   {System.Xml.XmlReaderSettings}    settings    The System.Xml.XmlReaderSettings object used to configure the new System.Xml.XmlReader
                 instance. This value can be null.
                 * @return  {System.Xml.XmlReader}                        An System.Xml.XmlReader object to read the XML data.
                 */
                Create$1: function (input, settings) {
                    throw new System.NotImplementedException.ctor();
                },
                /**
                 * Creates a new instance with the specified URI and System.Xml.XmlReaderSettings.
                 *
                 * @static
                 * @public
                 * @this System.Xml.XmlReader
                 * @memberof System.Xml.XmlReader
                 * @param   {string}                          inputUri    The URI for the file containing the XML data. The System.Xml.XmlResolver
                 object on the System.Xml.XmlReaderSettings object is used to convert the
                 path to a canonical data representation. If System.Xml.XmlReaderSettings.XmlResolver
                 is null, a new System.Xml.XmlUrlResolver object is used.
                 * @param   {System.Xml.XmlReaderSettings}    settings    The System.Xml.XmlReaderSettings object used to configure the new System.Xml.XmlReader
                 instance. This value can be null.
                 * @return  {System.Xml.XmlReader}                        An System.Xml.XmlReader object to read XML data.
                 */
                Create$9: function (inputUri, settings) {
                    throw new System.NotImplementedException.ctor();
                },
                Create$5: function (input, settings) {
                    throw new System.NotImplementedException.ctor();
                },
                Create$11: function (reader, settings) {
                    throw new System.NotImplementedException.ctor();
                },
                Create$2: function (input, settings, baseUri) {
                    throw new System.NotImplementedException.ctor();
                },
                Create$3: function (input, settings, inputContext) {
                    throw new System.NotImplementedException.ctor();
                },
                Create$10: function (inputUri, settings, inputContext) {
                    throw new System.NotImplementedException.ctor();
                },
                Create$6: function (input, settings, baseUri) {
                    throw new System.NotImplementedException.ctor();
                },
                Create$7: function (input, settings, inputContext) {
                    throw new System.NotImplementedException.ctor();
                }
            }
        },
        props: {
            /**
             * When overridden in a derived class, gets the qualified name of the current
             node.
             *
             * @instance
             * @public
             * @readonly
             * @memberof System.Xml.XmlReader
             * @function Name
             * @type string
             */
            Name: {
                get: function () {
                    throw new System.NotSupportedException.ctor();
                }
            }
        },
        alias: ["Dispose", "System$IDisposable$Dispose"],
        ctors: {
            /**
             * Initializes a new instance of the XmlReader class.
             *
             * @instance
             * @protected
             * @this System.Xml.XmlReader
             * @memberof System.Xml.XmlReader
             * @return  {void}
             */
            ctor: function () {
                this.$initialize();

            }
        },
        methods: {
            Dispose: function () {
                throw new System.NotImplementedException.ctor();
            },
            Dispose$1: function (disposing) {
                throw new System.NotImplementedException.ctor();
            },
            /**
             * @instance
             * @public
             * @this System.Xml.XmlReader
             * @memberof System.Xml.XmlReader
             * @return  {boolean}
             */
            IsStartElement: function () {
                throw new System.NotImplementedException.ctor();
            },
            IsStartElement$1: function (name) {
                throw new System.NotImplementedException.ctor();
            },
            IsStartElement$2: function (localname, ns) {
                throw new System.NotImplementedException.ctor();
            },
            /**
             * @instance
             * @public
             * @this System.Xml.XmlReader
             * @memberof System.Xml.XmlReader
             * @return  {System.Xml.XmlNodeType}
             */
            MoveToContent: function () {
                throw new System.NotImplementedException.ctor();
            },
            /**
             * @instance
             * @public
             * @this System.Xml.XmlReader
             * @memberof System.Xml.XmlReader
             * @return  {string}
             */
            ReadContentAsString: function () {
                throw new System.NotImplementedException.ctor();
            },
            /**
             * @instance
             * @public
             * @this System.Xml.XmlReader
             * @memberof System.Xml.XmlReader
             * @return  {string}
             */
            ReadElementContentAsString: function () {
                throw new System.NotImplementedException.ctor();
            },
            ReadElementContentAsString$1: function (localName, namespaceURI) {
                throw new System.NotImplementedException.ctor();
            },
            /**
             * @instance
             * @public
             * @this System.Xml.XmlReader
             * @memberof System.Xml.XmlReader
             * @return  {string}
             */
            ReadElementString: function () {
                throw new System.NotImplementedException.ctor();
            },
            ReadElementString$1: function (name) {
                throw new System.NotImplementedException.ctor();
            },
            ReadElementString$2: function (localname, ns) {
                throw new System.NotImplementedException.ctor();
            },
            ReadEndElement: function () {
                throw new System.NotImplementedException.ctor();
            },
            /**
             * @instance
             * @public
             * @this System.Xml.XmlReader
             * @memberof System.Xml.XmlReader
             * @return  {void}
             */
            ReadStartElement: function () {
                throw new System.NotImplementedException.ctor();
            },
            ReadStartElement$1: function (name) {
                throw new System.NotImplementedException.ctor();
            },
            ReadStartElement$2: function (localname, ns) {
                throw new System.NotImplementedException.ctor();
            },
            ReadString: function () {
                throw new System.NotImplementedException.ctor();
            },
            /**
             * @instance
             * @public
             * @this System.Xml.XmlReader
             * @memberof System.Xml.XmlReader
             * @param   {string}     name
             * @return  {boolean}
             */
            ReadToFollowing: function (name) {
                throw new System.NotImplementedException.ctor();
            },
            ReadToFollowing$1: function (localName, namespaceURI) {
                throw new System.NotImplementedException.ctor();
            },
            ReadToNextSibling: function (name) {
                throw new System.NotImplementedException.ctor();
            },
            ReadToNextSibling$1: function (localName, namespaceURI) {
                throw new System.NotImplementedException.ctor();
            },
            /**
             * @instance
             * @public
             * @this System.Xml.XmlReader
             * @memberof System.Xml.XmlReader
             * @return  {void}
             */
            Skip: function () {
                throw new System.NotImplementedException.ctor();
            }
        }
    });

    
    Bridge.define("System.Xml.XmlWriter", {
        inherits: [System.IDisposable],
        statics: {
            methods: {
                /**
                 * Creates a new System.Xml.XmlWriter instance using the specified stream.
                 *
                 * @static
                 * @public
                 * @this System.Xml.XmlWriter
                 * @memberof System.Xml.XmlWriter
                 * @param   {System.IO.Stream}        output    The stream to which you want to write. The System.Xml.XmlWriter writes XML
                 1.0 text syntax and appends it to the specified stream.
                 * @return  {System.Xml.XmlWriter}              An System.Xml.XmlWriter object.
                 */
                Create: function (output) {
                    return System.Xml.Cshtml5_XmlWriter.Create(output);

                }
            }
        },
        alias: ["Dispose", "System$IDisposable$Dispose"],
        ctors: {
            /**
             * Initializes a new instance of the System.Xml.XmlWriter class.
             *
             * @instance
             * @protected
             * @this System.Xml.XmlWriter
             * @memberof System.Xml.XmlWriter
             * @return  {void}
             */
            ctor: function () {
                this.$initialize();
            }
        },
        methods: {
            /**
             * When overridden in a derived class, closes this stream and the underlying
             stream.
             *
             * @instance
             * @public
             * @this System.Xml.XmlWriter
             * @memberof System.Xml.XmlWriter
             * @return  {void}
             */
            Close: function () {
                throw new System.NotImplementedException.ctor();
            },
            /**
             * When overridden in a derived class, writes out the attribute with the specified
             local name and value.
             *
             * @instance
             * @public
             * @this System.Xml.XmlWriter
             * @memberof System.Xml.XmlWriter
             * @param   {string}    localName    The local name of the attribute.
             * @param   {string}    value        The value of the attribute.
             * @return  {void}
             */
            WriteAttributeString: function (localName, value) {
                if (Bridge.is(this, System.Xml.Cshtml5_XmlWriter)) {
                    Bridge.cast(this, System.Xml.Cshtml5_XmlWriter).WriteAttributeString$1(localName, value);
                }
            },
            /**
             * Writes an element with the specified local name and value.
             *
             * @instance
             * @public
             * @this System.Xml.XmlWriter
             * @memberof System.Xml.XmlWriter
             * @param   {string}    localName    The local name of the element.
             * @param   {string}    value        The value of the element.
             * @return  {void}
             */
            WriteElementString: function (localName, value) {
                if (Bridge.is(this, System.Xml.Cshtml5_XmlWriter)) {
                    Bridge.cast(this, System.Xml.Cshtml5_XmlWriter).WriteElementString$1(localName, value);
                }
            },
            WriteStartElement: function (localName) {
                if (Bridge.is(this, System.Xml.Cshtml5_XmlWriter)) {
                    Bridge.cast(this, System.Xml.Cshtml5_XmlWriter).WriteStartElement$3(localName);
                }
            },
            /**
             * When overridden in a derived class, writes the specified start tag and associates
             it with the given namespace.
             *
             * @instance
             * @public
             * @this System.Xml.XmlWriter
             * @memberof System.Xml.XmlWriter
             * @param   {string}    localName    The local name of the element.
             * @param   {string}    ns           The namespace URI to associate with the element. If this namespace is already
             in scope and has an associated prefix, the writer automatically writes that
             prefix also.
             * @return  {void}
             */
            WriteStartElement$1: function (localName, ns) {
                if (Bridge.is(this, System.Xml.Cshtml5_XmlWriter)) {
                    Bridge.cast(this, System.Xml.Cshtml5_XmlWriter).WriteStartElement$2("", localName, ns);
                }
            },
            /**
             * Releases all resources used by the current instance of the System.Xml.XmlWriter
             class.
             *
             * @instance
             * @public
             * @this System.Xml.XmlWriter
             * @memberof System.Xml.XmlWriter
             * @return  {void}
             */
            Dispose: function () { },
            /**
             * Releases the unmanaged resources used by the System.Xml.XmlWriter and optionally
             releases the managed resources.
             *
             * @instance
             * @protected
             * @this System.Xml.XmlWriter
             * @memberof System.Xml.XmlWriter
             * @param   {boolean}    disposing    true to release both managed and unmanaged resources; false to release only
             unmanaged resources.
             * @return  {void}
             */
            Dispose$1: function (disposing) {
                throw new System.NotImplementedException.ctor();
            }
        }
    });

    Bridge.define("System.Xml.INTERNAL_NodeType", {
        statics: {
            fields: {
                Default: null
            },
            ctors: {
                init: function () {
                    this.Default = new System.Xml.INTERNAL_NodeType(null, System.Xml.XmlNodeType.None);
                }
            }
        },
        fields: {
            _nodeEnumerator: null,
            _attributsEnumerator: null,
            _type: 0,
            _node: null,
            MoreChild: false
        },
        props: {
            Type: {
                get: function () {
                    return this._type;
                }
            },
            RawNode: {
                get: function () {
                    return this._node;
                }
            },
            ElementOrNull: {
                get: function () {
                    return this._type === System.Xml.XmlNodeType.Element ? Bridge.cast(this._node, System.Xml.Linq.XElement) : null;
                }
            },
            AttributEnumerator: {
                get: function () {
                    return this._attributsEnumerator;
                }
            },
            NodeEnumerator: {
                get: function () {
                    return this._nodeEnumerator;
                }
            }
        },
        ctors: {
            ctor: function (node, type) {
                this.$initialize();
                this._node = node;
                this._type = type;

                if (Bridge.is(this._node, System.Xml.Linq.XContainer)) {
                    var nodeEnumerable = Bridge.cast(this._node, System.Xml.Linq.XContainer).Nodes();

                    if (System.Linq.Enumerable.from(nodeEnumerable).any()) {
                        this._nodeEnumerator = Bridge.getEnumerator(nodeEnumerable, System.Xml.Linq.XNode);
                        this.MoreChild = this._nodeEnumerator.System$Collections$IEnumerator$moveNext();
                    } else {
                        this.MoreChild = false;
                    }
                } else {
                    this.MoreChild = false;
                }


                if (Bridge.is(this._node, System.Xml.Linq.XElement)) {
                    this._attributsEnumerator = Bridge.getEnumerator(Bridge.cast(this._node, System.Xml.Linq.XElement).Attributes(), System.Xml.Linq.XAttribute);

                    this._attributsEnumerator.System$Collections$IEnumerator$moveNext();
                } else {
                    this._attributsEnumerator = new (System.Collections.Generic.List$1(System.Xml.Linq.XNode)).ctor().GetEnumerator().$clone();
                }

            }
        }
    });

    Bridge.define("System.Xml.IXmlSchemaInfo", {
        $kind: "interface"
    });

    /** @namespace System.Xml.Linq */

    /**
     * Represents a node or an attribute in an XML tree.
     *
     * @abstract
     * @public
     * @class System.Xml.Linq.XObject
     */
    Bridge.define("System.Xml.Linq.XObject");

    /**
     * Represents a name of an XML element or attribute.
     *
     * @public
     * @class System.Xml.Linq.XName
     * @implements  System.IEquatable$1
     */
    Bridge.define("System.Xml.Linq.XName", {
        inherits: function () { return [System.IEquatable$1(System.Xml.Linq.XName)]; },
        statics: {
            methods: {
                /**
                 * Gets an {@link } object from a local name and a namespace.
                 *
                 * @static
                 * @public
                 * @this System.Xml.Linq.XName
                 * @memberof System.Xml.Linq.XName
                 * @param   {string}                   localName        A local (unqualified) name.
                 * @param   {string}                   namespaceName    An XML namespace.
                 * @return  {System.Xml.Linq.XName}                     An {@link } object created from the specified local name and
                 namespace.
                 */
                Get$1: function (localName, namespaceName) {
                    return System.Xml.Linq.XNamespace.Get(namespaceName).GetName(localName);
                },
                /**
                 * Gets an {@link } object from an expanded name.
                 *
                 * @static
                 * @public
                 * @this System.Xml.Linq.XName
                 * @memberof System.Xml.Linq.XName
                 * @param   {string}                   expandedName    A {@link } that contains an expanded XML name in the format {namespace}localname.
                 * @return  {System.Xml.Linq.XName}                    An {@link } object constructed from the expanded name.
                 */
                Get: function (expandedName) {
                    if (expandedName == null) {
                        throw new System.ArgumentNullException.$ctor1("expandedName");
                    }
                    if (expandedName.length === 0) {
                        throw new System.ArgumentException.$ctor1(System.String.format("'{0}' is an invalid expanded name.", [expandedName]));
                    }
                    if (expandedName.charCodeAt(0) === 123) {
                        var i = expandedName.lastIndexOf(String.fromCharCode(125));
                        if (i <= 1 || i === ((expandedName.length - 1) | 0)) {
                            throw new System.ArgumentException.$ctor1(System.String.format("'{0}' is an invalid expanded name.", [expandedName]));
                        }
                        return System.Xml.Linq.XNamespace.Get$1(expandedName, 1, ((i - 1) | 0)).GetName$1(expandedName, ((i + 1) | 0), ((((expandedName.length - i) | 0) - 1) | 0));
                    } else {
                        return System.Xml.Linq.XNamespace.None.GetName(expandedName);
                    }
                }/**
                 * Converts a string formatted as an expanded XML name (that is,{namespace}localname)
                 to an System.Xml.Linq.XName object.
                 *
                 * @static
                 * @public
                 * @this System.Xml.Linq.XName
                 * @memberof System.Xml.Linq.XName
                 * @param   {string}                   expandedName    A string that contains an expanded XML name in the format {namespace}localname.
                 * @return  {System.Xml.Linq.XName}                    An System.Xml.Linq.XName object constructed from the expanded name.
                 */
                ,
                op_Implicit: function (expandedName) {
                    var xName = new System.Xml.Linq.XName();
                    var splittedName = System.String.split(expandedName, [125].map(function (i) {{ return String.fromCharCode(i); }}));
                    if (splittedName.length === 1) {
                        xName._localName = splittedName[System.Array.index(0, splittedName)];
                        return xName;
                    } else if (splittedName.length === 2) {
                        xName._namespace = System.Xml.Linq.XNamespace.Get(splittedName[System.Array.index(0, splittedName)].substr(1));
                        xName._localName = splittedName[System.Array.index(1, splittedName)];
                        return xName;
                    }
                    throw new System.InvalidCastException.$ctor1("the string \"" + (expandedName || "") + "\" could not be casted into a XName.");
                }/**
                 * Returns a value indicating whether two instances of System.Xml.Linq.XName
                 are not equal.
                 *
                 * @static
                 * @public
                 * @this System.Xml.Linq.XName
                 * @memberof System.Xml.Linq.XName
                 * @param   {System.Xml.Linq.XName}    left     The first System.Xml.Linq.XName to compare.
                 * @param   {System.Xml.Linq.XName}    right    The second System.Xml.Linq.XName to compare.
                 * @return  {boolean}                           true if left and right are not equal; otherwise false.
                 */
                ,
                op_Inequality: function (left, right) {
                    if (Bridge.referenceEquals(left, null)) {
                        if (Bridge.referenceEquals(right, null)) {
                            return false;
                        } else {
                            return true;
                        }
                    } else if (Bridge.referenceEquals(right, null)) {
                        return true;
                    }

                    if (System.Xml.Linq.XNamespace.op_Inequality(left.Namespace, right.Namespace)) {
                        return true;
                    }

                    return !Bridge.referenceEquals(left._localName, right._localName);
                }/**
                 * Returns a value indicating whether two instances of System.Xml.Linq.XName
                 are equal.
                 *
                 * @static
                 * @public
                 * @this System.Xml.Linq.XName
                 * @memberof System.Xml.Linq.XName
                 * @param   {System.Xml.Linq.XName}    left     The first System.Xml.Linq.XName to compare.
                 * @param   {System.Xml.Linq.XName}    right    The second System.Xml.Linq.XName to compare.
                 * @return  {boolean}                           true if left and right are equal; otherwise false.
                 */
                ,
                op_Equality: function (left, right) {
                    return !(System.Xml.Linq.XName.op_Inequality(left, right));
                }
            }
        },
        fields: {
            _localName: null,
            _namespace: null
        },
        props: {
            /**
             * Gets the local (unqualified) part of the name.
             *
             * @instance
             * @public
             * @memberof System.Xml.Linq.XName
             * @function LocalName
             * @type string
             */
            LocalName: {
                get: function () {
                    return this._localName;
                },
                set: function (value) {
                    this._localName = value;
                }
            },
            /**
             * Gets the namespace part of the fully qualified name.
             *
             * @instance
             * @public
             * @memberof System.Xml.Linq.XName
             * @function Namespace
             * @type System.Xml.Linq.XNamespace
             */
            Namespace: {
                get: function () {
                    if (System.Xml.Linq.XNamespace.op_Equality(this._namespace, null)) {
                        this._namespace = System.Xml.Linq.XNamespace.op_Implicit("");
                    }
                    return this._namespace;
                },
                set: function (value) {
                    this._namespace = value;
                }
            },
            /**
             * Returns the URI of the System.Xml.Linq.XNamespace for this System.Xml.Linq.XName.
             *
             * @instance
             * @public
             * @readonly
             * @memberof System.Xml.Linq.XName
             * @function NamespaceName
             * @type string
             */
            NamespaceName: {
                get: function () {
                    if (System.Xml.Linq.XNamespace.op_Equality(this._namespace, null)) {
                        this._namespace = System.Xml.Linq.XNamespace.op_Implicit("");
                    }
                    return this._namespace.NamespaceName;
                }
            }
        },
        alias: ["equalsT", "System$IEquatable$1$System$Xml$Linq$XName$equalsT"],
        methods: {
            /**
             * Determines whether the specified System.Xml.Linq.XName is equal to this System.Xml.Linq.XName.
             *
             * @instance
             * @public
             * @override
             * @this System.Xml.Linq.XName
             * @memberof System.Xml.Linq.XName
             * @param   {System.Object}    obj    The System.Xml.Linq.XName to compare to the current System.Xml.Linq.XName.
             * @return  {boolean}                 true if the specified System.Xml.Linq.XName is equal to the current System.Xml.Linq.XName;
             otherwise false.
             */
            equals: function (obj) {
                if (Bridge.is(obj, System.Xml.Linq.XName)) {
                    return System.Xml.Linq.XName.op_Equality(this, Bridge.cast(obj, System.Xml.Linq.XName));
                } else {
                    return false;
                }
            },
            equalsT: function (other) {
                return System.Xml.Linq.XName.op_Equality(this, other);
            },
            /**
             * Gets a hash code for this System.Xml.Linq.XName.
             *
             * @instance
             * @public
             * @override
             * @this System.Xml.Linq.XName
             * @memberof System.Xml.Linq.XName
             * @return  {number}        An System.Int32 that contains the hash code for the System.Xml.Linq.XName.
             */
            getHashCode: function () {
                return this.Namespace.getHashCode() ^ Bridge.getHashCode(this.LocalName);
            },
            /**
             * Returns the expanded XML name in the format {namespace}localname.
             *
             * @instance
             * @public
             * @override
             * @this System.Xml.Linq.XName
             * @memberof System.Xml.Linq.XName
             * @return  {string}        A System.String that contains the expanded XML name in the format {namespace}localname.
             */
            toString: function () {
                var result = "";
                if (System.Xml.Linq.XNamespace.op_Inequality(this._namespace, null)) {
                    result = (result || "") + (("{" + (this._namespace.NamespaceName || "") + "}") || "");
                }
                result = (result || "") + (this._localName || "");
                return result;
            }
        }
    });

    /**
     * Represents an XML namespace. This class cannot be inherited.
     * // Gets the System.Xml.Linq.XNamespace object that corresponds to the XML URI
    // (http://www.w3.org/XML/1998/namespace).
    //
     * // Gets the System.Xml.Linq.XNamespace object that corresponds to the xmlns
    // URI (http://www.w3.org/2000/xmlns/).
    //
     *
     * @public
     * @class System.Xml.Linq.XNamespace
     */
    Bridge.define("System.Xml.Linq.XNamespace", {
        statics: {
            fields: {
                xmlPrefixNamespace: null,
                xmlnsPrefixNamespace: null,
                _none: null
            },
            props: {
                /**
                 * Gets the XNamespace object that corresponds to the xml uri (http://www.w3.org/XML/1998/namespace).
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof System.Xml.Linq.XNamespace
                 * @function Xml
                 * @type System.Xml.Linq.XNamespace
                 */
                Xml: {
                    get: function () {
                        return System.Xml.Linq.XNamespace.Get(System.Xml.Linq.XNamespace.xmlPrefixNamespace);
                    }
                },
                /**
                 * Gets the XNamespace object that corresponds to the xmlns uri (http://www.w3.org/2000/xmlns/).
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof System.Xml.Linq.XNamespace
                 * @function Xmlns
                 * @type System.Xml.Linq.XNamespace
                 */
                Xmlns: {
                    get: function () {
                        return System.Xml.Linq.XNamespace.Get(System.Xml.Linq.XNamespace.xmlnsPrefixNamespace);
                    }
                },
                /**
                 * Gets the {@link } object that corresponds to no namespace.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof System.Xml.Linq.XNamespace
                 * @function None
                 * @type System.Xml.Linq.XNamespace
                 */
                None: {
                    get: function () {
                        if (System.Xml.Linq.XNamespace.op_Equality(System.Xml.Linq.XNamespace._none, null)) {
                            System.Xml.Linq.XNamespace._none = new System.Xml.Linq.XNamespace();
                            System.Xml.Linq.XNamespace._none.NamespaceName = "";
                        }
                        return System.Xml.Linq.XNamespace._none;
                    }
                }
            },
            ctors: {
                init: function () {
                    this.xmlPrefixNamespace = "http://www.w3.org/XML/1998/namespace";
                    this.xmlnsPrefixNamespace = "http://www.w3.org/2000/xmlns/";
                }
            },
            methods: {
                /**
                 * Gets an System.Xml.Linq.XNamespace for the specified Uniform Resource Identifier
                 (URI).
                 *
                 * @static
                 * @public
                 * @this System.Xml.Linq.XNamespace
                 * @memberof System.Xml.Linq.XNamespace
                 * @param   {string}                        namespaceName    A System.String that contains a namespace URI.
                 * @return  {System.Xml.Linq.XNamespace}                     An System.Xml.Linq.XNamespace created from the specified URI.
                 */
                Get: function (namespaceName) {
                    var xNamespace = new System.Xml.Linq.XNamespace();
                    xNamespace.NamespaceName = namespaceName;
                    return xNamespace;
                },
                Get$1: function (namespaceName, index, count) {
                    return System.Xml.Linq.XNamespace.Get(namespaceName.substr(index, count));
                }/**
                 * Combines an System.Xml.Linq.XNamespace object with a local name to create
                 an System.Xml.Linq.XName.
                 *
                 * @static
                 * @public
                 * @this System.Xml.Linq.XNamespace
                 * @memberof System.Xml.Linq.XNamespace
                 * @param   {System.Xml.Linq.XNamespace}    ns           An System.Xml.Linq.XNamespace that contains the namespace.
                 * @param   {string}                        localName    A System.String that contains the local name.
                 * @return  {System.Xml.Linq.XName}                      The new System.Xml.Linq.XName constructed from the namespace and local name.
                 */
                ,
                op_Addition: function (ns, localName) {
                    return ns.GetName(localName);
                }/**
                 * Converts a string containing a Uniform Resource Identifier (URI) to an System.Xml.Linq.XNamespace.
                 *
                 * @static
                 * @public
                 * @this System.Xml.Linq.XNamespace
                 * @memberof System.Xml.Linq.XNamespace
                 * @param   {string}                        namespaceName    A System.String that contains the namespace URI.
                 * @return  {System.Xml.Linq.XNamespace}                     An System.Xml.Linq.XNamespace constructed from the URI string.
                 */
                ,
                op_Implicit: function (namespaceName) {
                    return System.Xml.Linq.XNamespace.Get(namespaceName);
                }/**
                 * Returns a value indicating whether two instances of System.Xml.Linq.XNamespace
                 are not equal.
                 *
                 * @static
                 * @public
                 * @this System.Xml.Linq.XNamespace
                 * @memberof System.Xml.Linq.XNamespace
                 * @param   {System.Xml.Linq.XNamespace}    left     The first System.Xml.Linq.XNamespace to compare.
                 * @param   {System.Xml.Linq.XNamespace}    right    The second System.Xml.Linq.XNamespace to compare.
                 * @return  {boolean}                                A System.Boolean that indicates whether left and right are not equal.
                 */
                ,
                op_Inequality: function (left, right) {
                    if (Bridge.referenceEquals(left, null)) {
                        if (Bridge.referenceEquals(right, null)) {
                            return false;
                        } else {
                            return true;
                        }
                    } else if (Bridge.referenceEquals(right, null)) {
                        return true;
                    }
                    return !Bridge.referenceEquals(left._namespaceName, right._namespaceName);
                }/**
                 * Returns a value indicating whether two instances of System.Xml.Linq.XNamespace
                 are equal.
                 *
                 * @static
                 * @public
                 * @this System.Xml.Linq.XNamespace
                 * @memberof System.Xml.Linq.XNamespace
                 * @param   {System.Xml.Linq.XNamespace}    left     The first System.Xml.Linq.XNamespace to compare.
                 * @param   {System.Xml.Linq.XNamespace}    right    The second System.Xml.Linq.XNamespace to compare.
                 * @return  {boolean}                                A System.Boolean that indicates whether left and right are equal.
                 */
                ,
                op_Equality: function (left, right) {

                    return !(System.Xml.Linq.XNamespace.op_Inequality(left, right));
                }
            }
        },
        fields: {
            _namespaceName: null
        },
        props: {
            /**
             * Gets the Uniform Resource Identifier (URI) of this namespace.
             *
             * @instance
             * @public
             * @memberof System.Xml.Linq.XNamespace
             * @function NamespaceName
             * @type string
             */
            NamespaceName: {
                get: function () {
                    return this._namespaceName;
                },
                set: function (value) {
                    this._namespaceName = value;
                }
            }
        },
        methods: {
            /**
             * Returns an System.Xml.Linq.XName object created from this System.Xml.Linq.XNamespace
             and the specified local name.
             *
             * @instance
             * @public
             * @this System.Xml.Linq.XNamespace
             * @memberof System.Xml.Linq.XNamespace
             * @param   {string}                   localName    A System.String that contains a local name.
             * @return  {System.Xml.Linq.XName}                 An System.Xml.Linq.XName created from this System.Xml.Linq.XNamespace and
             the specified local name.
             */
            GetName: function (localName) {
                var xName = new System.Xml.Linq.XName();
                xName.LocalName = localName;
                xName.Namespace = this;
                return xName;
            },
            GetName$1: function (localName, index, count) {
                return this.GetName(localName.substr(index, count));
            },
            /**
             * Returns the URI of this System.Xml.Linq.XNamespace.
             *
             * @instance
             * @public
             * @override
             * @this System.Xml.Linq.XNamespace
             * @memberof System.Xml.Linq.XNamespace
             * @return  {string}        The URI of this System.Xml.Linq.XNamespace.
             */
            toString: function () {
                return this.NamespaceName;
            },
            /**
             * Gets a hash code for this {@link }.
             *
             * @instance
             * @public
             * @override
             * @this System.Xml.Linq.XNamespace
             * @memberof System.Xml.Linq.XNamespace
             * @return  {number}        An {@link } that contains the hash code for the {@link }.
             */
            getHashCode: function () {
                var $t;
                return Bridge.getHashCode((($t = this.NamespaceName, $t != null ? $t : "")));
            },
            /**
             * Determines whether the specified {@link } is equal to the
             current {@link }.
             *
             * @instance
             * @public
             * @override
             * @this System.Xml.Linq.XNamespace
             * @memberof System.Xml.Linq.XNamespace
             * @param   {System.Object}    obj    The {@link } to compare to the current {@link }.
             * @return  {boolean}                 A {@link } that indicates whether the specified {@link }
             is equal to the current {@link }.
             */
            equals: function (obj) {
                return System.Xml.Linq.XNamespace.op_Equality(this, Bridge.as(obj, System.Xml.Linq.XNamespace));
            }
        }
    });

    /**
     * The {@link } namespaces contain types for processing XML. Child
     namespaces support serialization of XML documents or streams, XSD schemas, XQuery
     1.0 and XPath 2.0, and LINQ to XML, which is an in-memory XML programming interface
     that enables easy modification of XML documents.
     *
     * @class System.Xml.NamespaceDoc
     */
    Bridge.define("System.Xml.NamespaceDoc");

    Bridge.define("System.Xml.ReadState", {
        $kind: "enum"
    });

    /** @namespace System.Xml.Schema */

    /**
     * The {@link } namespace contains the XML classes that provide
     standards-based support for XML schema definition language (XSD) schemas.
     *
     * @class System.Xml.Schema.NamespaceDoc
     */
    Bridge.define("System.Xml.Schema.NamespaceDoc");

    /**
     * Indicates if attributes or elements need to be qualified with a namespace
     prefix.
     *
     * @public
     * @class System.Xml.Schema.XmlSchemaForm
     */
    Bridge.define("System.Xml.Schema.XmlSchemaForm", {
        $kind: "enum",
        statics: {
            fields: {
                /**
                 * Element and attribute form is not specified in the schema.
                 *
                 * @static
                 * @public
                 * @memberof System.Xml.Schema.XmlSchemaForm
                 * @constant
                 * @default 0
                 * @type System.Xml.Schema.XmlSchemaForm
                 */
                None: 0,
                /**
                 * Elements and attributes must be qualified with a namespace prefix.
                 *
                 * @static
                 * @public
                 * @memberof System.Xml.Schema.XmlSchemaForm
                 * @constant
                 * @default 1
                 * @type System.Xml.Schema.XmlSchemaForm
                 */
                Qualified: 1,
                /**
                 * Elements and attributes are not required to be qualified with a namespace
                 prefix.
                 *
                 * @static
                 * @public
                 * @memberof System.Xml.Schema.XmlSchemaForm
                 * @constant
                 * @default 2
                 * @type System.Xml.Schema.XmlSchemaForm
                 */
                Unqualified: 2
            }
        }
    });

    /** @namespace System.Xml.Serialization */

    /**
     * Provides custom formatting for XML serialization and deserialization.
     *
     * @abstract
     * @public
     * @class System.Xml.Serialization.IXmlSerializable
     */
    Bridge.define("System.Xml.Serialization.IXmlSerializable", {
        $kind: "interface"
    });

    /**
     * The {@link } namespace contains classes that are used
     to serialize objects into XML format documents or streams.
     *
     * @class System.Xml.Serialization.NamespaceDoc
     */
    Bridge.define("System.Xml.Serialization.NamespaceDoc");

    /**
     * Specifies that the System.Xml.Serialization.XmlSerializer must serialize
     a particular class member as an array of XML elements.
     *
     * @public
     * @class System.Xml.Serialization.XmlArrayAttribute
     * @augments System.Attribute
     */
    Bridge.define("System.Xml.Serialization.XmlArrayAttribute", {
        inherits: [System.Attribute],
        fields: {
            elementName: null,
            ns: null,
            nullable: false,
            form: 0,
            order: 0
        },
        props: {
            /**
             * Gets or sets the XML element name given to the serialized array.
             *
             * @instance
             * @public
             * @memberof System.Xml.Serialization.XmlArrayAttribute
             * @function ElementName
             * @type string
             */
            ElementName: {
                get: function () {
                    return this.elementName == null ? "" : this.elementName;
                },
                set: function (value) {
                    this.elementName = value;
                }
            },
            /**
             * Gets or sets the namespace of the XML element.
             *
             * @instance
             * @public
             * @memberof System.Xml.Serialization.XmlArrayAttribute
             * @function Namespace
             * @type string
             */
            Namespace: {
                get: function () {
                    return this.ns;
                },
                set: function (value) {
                    this.ns = value;
                }
            },
            /**
             * Gets or sets a value that indicates whether the System.Xml.Serialization.XmlSerializer
             must serialize a member as an empty XML tag with the xsi:nil attribute set
             to true.
             *
             * @instance
             * @public
             * @memberof System.Xml.Serialization.XmlArrayAttribute
             * @function IsNullable
             * @type boolean
             */
            IsNullable: {
                get: function () {
                    return this.nullable;
                },
                set: function (value) {
                    this.nullable = value;
                }
            },
            /**
             * Gets or sets a value that indicates whether the XML element name generated
             by the System.Xml.Serialization.XmlSerializer is qualified or unqualified.
             *
             * @instance
             * @public
             * @memberof System.Xml.Serialization.XmlArrayAttribute
             * @function Form
             * @type System.Xml.Schema.XmlSchemaForm
             */
            Form: {
                get: function () {
                    return this.form;
                },
                set: function (value) {
                    this.form = value;
                }
            },
            /**
             * Gets or sets the explicit order in which the elements are serialized or deserialized.
             *
             * @instance
             * @public
             * @memberof System.Xml.Serialization.XmlArrayAttribute
             * @function Order
             * @type number
             */
            Order: {
                get: function () {
                    return this.order;
                },
                set: function (value) {
                    if (value < 0) {
                        throw new System.ArgumentException.$ctor1("Order cannot be negative.");
                    }
                    this.order = value;
                }
            }
        },
        ctors: {
            init: function () {
                this.form = System.Xml.Schema.XmlSchemaForm.None;
                this.order = -1;
            },
            /**
             * Initializes a new instance of the System.Xml.Serialization.XmlArrayAttribute
             class.
             *
             * @instance
             * @public
             * @this System.Xml.Serialization.XmlArrayAttribute
             * @memberof System.Xml.Serialization.XmlArrayAttribute
             * @return  {void}
             */
            ctor: function () {
                this.$initialize();
                System.Attribute.ctor.call(this);
            },
            /**
             * Initializes a new instance of the System.Xml.Serialization.XmlArrayAttribute
             class and specifies the XML element name generated in the XML document instance.
             *
             * @instance
             * @public
             * @this System.Xml.Serialization.XmlArrayAttribute
             * @memberof System.Xml.Serialization.XmlArrayAttribute
             * @param   {string}    elementName    The name of the XML element that the System.Xml.Serialization.XmlSerializer
             generates.
             * @return  {void}
             */
            $ctor1: function (elementName) {
                this.$initialize();
                System.Attribute.ctor.call(this);
                this.elementName = elementName;
            }
        }
    });

    /**
     * Represents an attribute that specifies the derived types that the System.Xml.Serialization.XmlSerializer
     can place in a serialized array.
     *
     * @public
     * @class System.Xml.Serialization.XmlArrayItemAttribute
     * @augments System.Attribute
     */
    Bridge.define("System.Xml.Serialization.XmlArrayItemAttribute", {
        inherits: [System.Attribute],
        fields: {
            elementName: null,
            type: null,
            ns: null,
            dataType: null,
            nullable: false,
            nullableSpecified: false,
            form: 0,
            nestingLevel: 0
        },
        props: {
            /**
             * Gets or sets the type allowed in an array.
             *
             * @instance
             * @public
             * @memberof System.Xml.Serialization.XmlArrayItemAttribute
             * @function Type
             * @type System.Type
             */
            Type: {
                get: function () {
                    return this.type;
                },
                set: function (value) {
                    this.type = value;
                }
            },
            /**
             * Gets or sets the name of the generated XML element.
             *
             * @instance
             * @public
             * @memberof System.Xml.Serialization.XmlArrayItemAttribute
             * @function ElementName
             * @type string
             */
            ElementName: {
                get: function () {
                    return this.elementName == null ? "" : this.elementName;
                },
                set: function (value) {
                    this.elementName = value;
                }
            },
            /**
             * Gets or sets the namespace of the generated XML element.
             *
             * @instance
             * @public
             * @memberof System.Xml.Serialization.XmlArrayItemAttribute
             * @function Namespace
             * @type string
             */
            Namespace: {
                get: function () {
                    return this.ns;
                },
                set: function (value) {
                    this.ns = value;
                }
            },
            /**
             * Gets or sets the level in a hierarchy of XML elements that the System.Xml.Serialization.XmlArrayItemAttribute
             affects.
             *
             * @instance
             * @public
             * @memberof System.Xml.Serialization.XmlArrayItemAttribute
             * @function NestingLevel
             * @type number
             */
            NestingLevel: {
                get: function () {
                    return this.nestingLevel;
                },
                set: function (value) {
                    this.nestingLevel = value;
                }
            },
            /**
             * Gets or sets the XML data type of the generated XML element.
             *
             * @instance
             * @public
             * @memberof System.Xml.Serialization.XmlArrayItemAttribute
             * @function DataType
             * @type string
             */
            DataType: {
                get: function () {
                    return this.dataType == null ? "" : this.dataType;
                },
                set: function (value) {
                    this.dataType = value;
                }
            },
            /**
             * Gets or sets a value that indicates whether the System.Xml.Serialization.XmlSerializer
             must serialize a member as an empty XML tag with the xsi:nil attribute set
             to true.
             *
             * @instance
             * @public
             * @memberof System.Xml.Serialization.XmlArrayItemAttribute
             * @function IsNullable
             * @type boolean
             */
            IsNullable: {
                get: function () {
                    return this.nullable;
                },
                set: function (value) {
                    this.nullable = value;
                    this.nullableSpecified = true;
                }
            },
            IsNullableSpecified: {
                get: function () {
                    return this.nullableSpecified;
                }
            },
            /**
             * Gets or sets a value that indicates whether the name of the generated XML
             element is qualified.
             *
             * @instance
             * @public
             * @memberof System.Xml.Serialization.XmlArrayItemAttribute
             * @function Form
             * @type System.Xml.Schema.XmlSchemaForm
             */
            Form: {
                get: function () {
                    return this.form;
                },
                set: function (value) {
                    this.form = value;
                }
            }
        },
        ctors: {
            init: function () {
                this.nullableSpecified = false;
                this.form = System.Xml.Schema.XmlSchemaForm.None;
            },
            /**
             * Initializes a new instance of the System.Xml.Serialization.XmlArrayItemAttribute
             class.
             *
             * @instance
             * @public
             * @this System.Xml.Serialization.XmlArrayItemAttribute
             * @memberof System.Xml.Serialization.XmlArrayItemAttribute
             * @return  {void}
             */
            ctor: function () {
                this.$initialize();
                System.Attribute.ctor.call(this);
            },
            /**
             * Initializes a new instance of the System.Xml.Serialization.XmlArrayItemAttribute
             class and specifies the name of the XML element generated in the XML document.
             *
             * @instance
             * @public
             * @this System.Xml.Serialization.XmlArrayItemAttribute
             * @memberof System.Xml.Serialization.XmlArrayItemAttribute
             * @param   {string}    elementName    The name of the XML element.
             * @return  {void}
             */
            $ctor1: function (elementName) {
                this.$initialize();
                System.Attribute.ctor.call(this);
                this.elementName = elementName;
            },
            /**
             * Initializes a new instance of the System.Xml.Serialization.XmlArrayItemAttribute
             class and specifies the System.Type that can be inserted into the serialized
             array.
             *
             * @instance
             * @public
             * @this System.Xml.Serialization.XmlArrayItemAttribute
             * @memberof System.Xml.Serialization.XmlArrayItemAttribute
             * @param   {System.Type}    type    The System.Type of the object to serialize.
             * @return  {void}
             */
            $ctor3: function (type) {
                this.$initialize();
                System.Attribute.ctor.call(this);
                this.type = type;
            },
            /**
             * Initializes a new instance of the System.Xml.Serialization.XmlArrayItemAttribute
             class and specifies the name of the XML element generated in the XML document
             and the System.Type that can be inserted into the generated XML document.
             *
             * @instance
             * @public
             * @this System.Xml.Serialization.XmlArrayItemAttribute
             * @memberof System.Xml.Serialization.XmlArrayItemAttribute
             * @param   {string}         elementName    The name of the XML element.
             * @param   {System.Type}    type           The System.Type of the object to serialize.
             * @return  {void}
             */
            $ctor2: function (elementName, type) {
                this.$initialize();
                System.Attribute.ctor.call(this);
                this.elementName = elementName;
                this.type = type;
            }
        }
    });

    /**
     * Specifies that the System.Xml.Serialization.XmlSerializer must serialize
     the class member as an XML attribute.
     *
     * @public
     * @class System.Xml.Serialization.XmlAttributeAttribute
     * @augments System.Attribute
     */
    Bridge.define("System.Xml.Serialization.XmlAttributeAttribute", {
        inherits: [System.Attribute],
        fields: {
            attributeName: null,
            type: null,
            ns: null,
            dataType: null,
            form: 0
        },
        props: {
            /**
             * Gets or sets the complex type of the XML attribute.
             *
             * @instance
             * @public
             * @memberof System.Xml.Serialization.XmlAttributeAttribute
             * @function Type
             * @type System.Type
             */
            Type: {
                get: function () {
                    return this.type;
                },
                set: function (value) {
                    this.type = value;
                }
            },
            /**
             * Gets or sets the name of the XML attribute.
             *
             * @instance
             * @public
             * @memberof System.Xml.Serialization.XmlAttributeAttribute
             * @function AttributeName
             * @type string
             */
            AttributeName: {
                get: function () {
                    return this.attributeName == null ? "" : this.attributeName;
                },
                set: function (value) {
                    this.attributeName = value;
                }
            },
            /**
             * Gets or sets the XML namespace of the XML attribute.
             *
             * @instance
             * @public
             * @memberof System.Xml.Serialization.XmlAttributeAttribute
             * @function Namespace
             * @type string
             */
            Namespace: {
                get: function () {
                    return this.ns;
                },
                set: function (value) {
                    this.ns = value;
                }
            },
            /**
             * Gets or sets the XSD data type of the XML attribute generated by the System.Xml.Serialization.XmlSerializer.
             *
             * @instance
             * @public
             * @memberof System.Xml.Serialization.XmlAttributeAttribute
             * @function DataType
             * @type string
             */
            DataType: {
                get: function () {
                    return this.dataType == null ? "" : this.dataType;
                },
                set: function (value) {
                    this.dataType = value;
                }
            },
            /**
             * Gets or sets a value that indicates whether the XML attribute name generated
             by the System.Xml.Serialization.XmlSerializer is qualified.
             *
             * @instance
             * @public
             * @memberof System.Xml.Serialization.XmlAttributeAttribute
             * @function Form
             * @type System.Xml.Schema.XmlSchemaForm
             */
            Form: {
                get: function () {
                    return this.form;
                },
                set: function (value) {
                    this.form = value;
                }
            }
        },
        ctors: {
            init: function () {
                this.form = System.Xml.Schema.XmlSchemaForm.None;
            },
            /**
             * Initializes a new instance of the System.Xml.Serialization.XmlAttributeAttribute
             class.
             *
             * @instance
             * @public
             * @this System.Xml.Serialization.XmlAttributeAttribute
             * @memberof System.Xml.Serialization.XmlAttributeAttribute
             * @return  {void}
             */
            ctor: function () {
                this.$initialize();
                System.Attribute.ctor.call(this);
            },
            /**
             * Initializes a new instance of the System.Xml.Serialization.XmlAttributeAttribute
             class and specifies the name of the generated XML attribute.
             *
             * @instance
             * @public
             * @this System.Xml.Serialization.XmlAttributeAttribute
             * @memberof System.Xml.Serialization.XmlAttributeAttribute
             * @param   {string}    attributeName    The name of the XML attribute that the System.Xml.Serialization.XmlSerializer
             generates.
             * @return  {void}
             */
            $ctor1: function (attributeName) {
                this.$initialize();
                System.Attribute.ctor.call(this);
                this.attributeName = attributeName;
            },
            /**
             * Initializes a new instance of the System.Xml.Serialization.XmlAttributeAttribute
             class.
             *
             * @instance
             * @public
             * @this System.Xml.Serialization.XmlAttributeAttribute
             * @memberof System.Xml.Serialization.XmlAttributeAttribute
             * @param   {System.Type}    type    The System.Type used to store the attribute.
             * @return  {void}
             */
            $ctor3: function (type) {
                this.$initialize();
                System.Attribute.ctor.call(this);
                this.type = type;
            },
            /**
             * Initializes a new instance of the System.Xml.Serialization.XmlAttributeAttribute
             class.
             *
             * @instance
             * @public
             * @this System.Xml.Serialization.XmlAttributeAttribute
             * @memberof System.Xml.Serialization.XmlAttributeAttribute
             * @param   {string}         attributeName    The name of the XML attribute that is generated.
             * @param   {System.Type}    type             The System.Type used to store the attribute.
             * @return  {void}
             */
            $ctor2: function (attributeName, type) {
                this.$initialize();
                System.Attribute.ctor.call(this);
                this.attributeName = attributeName;
                this.type = type;
            }
        }
    });

    /**
     * Indicates that a public field or property represents an XML element when
     the System.Xml.Serialization.XmlSerializer serializes or deserializes the
     object that contains it.
     *
     * @public
     * @class System.Xml.Serialization.XmlElementAttribute
     * @augments System.Attribute
     */
    Bridge.define("System.Xml.Serialization.XmlElementAttribute", {
        inherits: [System.Attribute],
        fields: {
            elementName: null,
            type: null,
            ns: null,
            dataType: null,
            nullable: false,
            nullableSpecified: false,
            form: 0,
            order: 0
        },
        props: {
            /**
             * Gets or sets the object type used to represent the XML element.
             *
             * @instance
             * @public
             * @memberof System.Xml.Serialization.XmlElementAttribute
             * @function Type
             * @type System.Type
             */
            Type: {
                get: function () {
                    return this.type;
                },
                set: function (value) {
                    this.type = value;
                }
            },
            /**
             * Gets or sets the name of the generated XML element.
             *
             * @instance
             * @public
             * @memberof System.Xml.Serialization.XmlElementAttribute
             * @function ElementName
             * @type string
             */
            ElementName: {
                get: function () {
                    return this.elementName == null ? "" : this.elementName;
                },
                set: function (value) {
                    this.elementName = value;
                }
            },
            /**
             * Gets or sets the namespace assigned to the XML element that results when
             the class is serialized.
             *
             * @instance
             * @public
             * @memberof System.Xml.Serialization.XmlElementAttribute
             * @function Namespace
             * @type string
             */
            Namespace: {
                get: function () {
                    return this.ns;
                },
                set: function (value) {
                    this.ns = value;
                }
            },
            /**
             * Gets or sets the XML Schema definition (XSD) data type of the XML element
             generated by the System.Xml.Serialization.XmlSerializer.
             *
             * @instance
             * @public
             * @memberof System.Xml.Serialization.XmlElementAttribute
             * @function DataType
             * @type string
             */
            DataType: {
                get: function () {
                    return this.dataType == null ? "" : this.dataType;
                },
                set: function (value) {
                    this.dataType = value;
                }
            },
            /**
             * Gets or sets a value that indicates whether the System.Xml.Serialization.XmlSerializer
             must serialize a member that is set to null as an empty tag with the xsi:nil
             attribute set to true.
             *
             * @instance
             * @public
             * @memberof System.Xml.Serialization.XmlElementAttribute
             * @function IsNullable
             * @type boolean
             */
            IsNullable: {
                get: function () {
                    return this.nullable;
                },
                set: function (value) {
                    this.nullable = value;
                    this.nullableSpecified = true;
                }
            },
            IsNullableSpecified: {
                get: function () {
                    return this.nullableSpecified;
                }
            },
            /**
             * Gets or sets a value that indicates whether the element is qualified.
             *
             * @instance
             * @public
             * @memberof System.Xml.Serialization.XmlElementAttribute
             * @function Form
             * @type System.Xml.Schema.XmlSchemaForm
             */
            Form: {
                get: function () {
                    return this.form;
                },
                set: function (value) {
                    this.form = value;
                }
            },
            /**
             * Gets or sets the explicit order in which the elements are serialized or deserialized.
             *
             * @instance
             * @public
             * @memberof System.Xml.Serialization.XmlElementAttribute
             * @function Order
             * @type number
             */
            Order: {
                get: function () {
                    return this.order;
                },
                set: function (value) {
                    if (value < 0) {
                        throw new System.ArgumentException.$ctor1("Order cannot be negative.");
                    }
                    this.order = value;
                }
            }
        },
        ctors: {
            init: function () {
                this.form = System.Xml.Schema.XmlSchemaForm.None;
                this.order = -1;
            },
            /**
             * Initializes a new instance of the System.Xml.Serialization.XmlElementAttribute
             class.
             *
             * @instance
             * @public
             * @this System.Xml.Serialization.XmlElementAttribute
             * @memberof System.Xml.Serialization.XmlElementAttribute
             * @return  {void}
             */
            ctor: function () {
                this.$initialize();
                System.Attribute.ctor.call(this);
            },
            /**
             * Initializes a new instance of the System.Xml.Serialization.XmlElementAttribute
             class and specifies the name of the XML element.
             *
             * @instance
             * @public
             * @this System.Xml.Serialization.XmlElementAttribute
             * @memberof System.Xml.Serialization.XmlElementAttribute
             * @param   {string}    elementName    The XML element name of the serialized member.
             * @return  {void}
             */
            $ctor1: function (elementName) {
                this.$initialize();
                System.Attribute.ctor.call(this);
                this.elementName = elementName;
            },
            /**
             * Initializes a new instance of the System.Xml.Serialization.XmlElementAttribute
             class and specifies a type for the member to which the System.Xml.Serialization.XmlElementAttribute
             is applied. This type is used by the System.Xml.Serialization.XmlSerializer
             when serializing or deserializing object that contains it.
             *
             * @instance
             * @public
             * @this System.Xml.Serialization.XmlElementAttribute
             * @memberof System.Xml.Serialization.XmlElementAttribute
             * @param   {System.Type}    type    The System.Type of an object derived from the member's type.
             * @return  {void}
             */
            $ctor3: function (type) {
                this.$initialize();
                System.Attribute.ctor.call(this);
                this.type = type;
            },
            /**
             * Initializes a new instance of the System.Xml.Serialization.XmlElementAttribute
             and specifies the name of the XML element and a derived type for the member
             to which the System.Xml.Serialization.XmlElementAttribute is applied. This
             member type is used when the System.Xml.Serialization.XmlSerializer serializes
             the object that contains it.
             *
             * @instance
             * @public
             * @this System.Xml.Serialization.XmlElementAttribute
             * @memberof System.Xml.Serialization.XmlElementAttribute
             * @param   {string}         elementName    The XML element name of the serialized member.
             * @param   {System.Type}    type           The System.Type of an object derived from the member's type.
             * @return  {void}
             */
            $ctor2: function (elementName, type) {
                this.$initialize();
                System.Attribute.ctor.call(this);
                this.elementName = elementName;
                this.type = type;
            }
        }
    });

    /**
     * Controls how the System.Xml.Serialization.XmlSerializer serializes an enumeration
     member.
     *
     * @public
     * @class System.Xml.Serialization.XmlEnumAttribute
     * @augments System.Attribute
     */
    Bridge.define("System.Xml.Serialization.XmlEnumAttribute", {
        inherits: [System.Attribute],
        fields: {
            name: null
        },
        props: {
            /**
             * Gets or sets the value generated in an XML-document instance when the System.Xml.Serialization.XmlSerializer
             serializes an enumeration, or the value recognized when it deserializes the
             enumeration member.
             *
             * @instance
             * @public
             * @memberof System.Xml.Serialization.XmlEnumAttribute
             * @function Name
             * @type string
             */
            Name: {
                get: function () {
                    return this.name;
                },
                set: function (value) {
                    this.name = value;
                }
            }
        },
        ctors: {
            /**
             * Initializes a new instance of the System.Xml.Serialization.XmlEnumAttribute
             class.
             *
             * @instance
             * @public
             * @this System.Xml.Serialization.XmlEnumAttribute
             * @memberof System.Xml.Serialization.XmlEnumAttribute
             * @return  {void}
             */
            ctor: function () {
                this.$initialize();
                System.Attribute.ctor.call(this);
            },
            /**
             * Initializes a new instance of the System.Xml.Serialization.XmlEnumAttribute
             class, and specifies the XML value that the System.Xml.Serialization.XmlSerializer
             generates or recognizes (when it serializes or deserializes the enumeration,
             respectively).
             *
             * @instance
             * @public
             * @this System.Xml.Serialization.XmlEnumAttribute
             * @memberof System.Xml.Serialization.XmlEnumAttribute
             * @param   {string}    name    The overriding name of the enumeration member.
             * @return  {void}
             */
            $ctor1: function (name) {
                this.$initialize();
                System.Attribute.ctor.call(this);
                this.name = name;
            }
        }
    });

    /**
     * Instructs the System.Xml.Serialization.XmlSerializer.Serialize(System.IO.TextWriter,System.Object)
     method of the System.Xml.Serialization.XmlSerializer not to serialize the
     public field or public read/write property value.
     *
     * @public
     * @class System.Xml.Serialization.XmlIgnoreAttribute
     * @augments System.Attribute
     */
    Bridge.define("System.Xml.Serialization.XmlIgnoreAttribute", {
        inherits: [System.Attribute],
        ctors: {
            /**
             * Initializes a new instance of the System.Xml.Serialization.XmlIgnoreAttribute
             class.
             *
             * @instance
             * @public
             * @this System.Xml.Serialization.XmlIgnoreAttribute
             * @memberof System.Xml.Serialization.XmlIgnoreAttribute
             * @return  {void}
             */
            ctor: function () {
                this.$initialize();
                System.Attribute.ctor.call(this);
            }
        }
    });

    /**
     * Allows the System.Xml.Serialization.XmlSerializer to recognize a type when
     it serializes or deserializes an object.
     *
     * @public
     * @class System.Xml.Serialization.XmlIncludeAttribute
     * @augments System.Attribute
     */
    Bridge.define("System.Xml.Serialization.XmlIncludeAttribute", {
        inherits: [System.Attribute],
        fields: {
            type: null
        },
        props: {
            /**
             * Gets or sets the type of the object to include.
             *
             * @instance
             * @public
             * @memberof System.Xml.Serialization.XmlIncludeAttribute
             * @function Type
             * @type System.Type
             */
            Type: {
                get: function () {
                    return this.type;
                },
                set: function (value) {
                    this.type = value;
                }
            }
        },
        ctors: {
            /**
             * Initializes a new instance of the System.Xml.Serialization.XmlIncludeAttribute
             class.
             *
             * @instance
             * @public
             * @this System.Xml.Serialization.XmlIncludeAttribute
             * @memberof System.Xml.Serialization.XmlIncludeAttribute
             * @param   {System.Type}    type    The System.Type of the object to include.
             * @return  {void}
             */
            ctor: function (type) {
                this.$initialize();
                System.Attribute.ctor.call(this);
                this.type = type;
            }
        }
    });

    /**
     * Controls XML serialization of the attribute target as an XML root element.
     *
     * @public
     * @class System.Xml.Serialization.XmlRootAttribute
     * @augments System.Attribute
     */
    Bridge.define("System.Xml.Serialization.XmlRootAttribute", {
        inherits: [System.Attribute],
        fields: {
            elementName: null,
            ns: null,
            dataType: null,
            nullable: false,
            nullableSpecified: false
        },
        props: {
            /**
             * Gets or sets the name of the XML element that is generated and recognized
             by the System.Xml.Serialization.XmlSerializer class's System.Xml.Serialization.XmlSerializer.Serialize(System.IO.TextWriter,System.Object)
             and System.Xml.Serialization.XmlSerializer.Deserialize(System.IO.Stream)
             methods, respectively.
             *
             * @instance
             * @public
             * @memberof System.Xml.Serialization.XmlRootAttribute
             * @function ElementName
             * @type string
             */
            ElementName: {
                get: function () {
                    return this.elementName == null ? "" : this.elementName;
                },
                set: function (value) {
                    this.elementName = value;
                }
            },
            /**
             * Gets or sets the namespace for the XML root element.
             *
             * @instance
             * @public
             * @memberof System.Xml.Serialization.XmlRootAttribute
             * @function Namespace
             * @type string
             */
            Namespace: {
                get: function () {
                    return this.ns;
                },
                set: function (value) {
                    this.ns = value;
                }
            },
            /**
             * Gets or sets the XSD data type of the XML root element.
             *
             * @instance
             * @public
             * @memberof System.Xml.Serialization.XmlRootAttribute
             * @function DataType
             * @type string
             */
            DataType: {
                get: function () {
                    return this.dataType == null ? "" : this.dataType;
                },
                set: function (value) {
                    this.dataType = value;
                }
            },
            /**
             * Gets or sets a value that indicates whether the System.Xml.Serialization.XmlSerializer
             must serialize a member that is set to null into the xsi:nil attribute set
             to true.
             *
             * @instance
             * @public
             * @memberof System.Xml.Serialization.XmlRootAttribute
             * @function IsNullable
             * @type boolean
             */
            IsNullable: {
                get: function () {
                    return this.nullable;
                },
                set: function (value) {
                    this.nullable = value;
                    this.nullableSpecified = true;
                }
            },
            IsNullableSpecified: {
                get: function () {
                    return this.nullableSpecified;
                }
            },
            Key: {
                get: function () {
                    return ((this.ns == null ? "" : this.ns) || "") + ":" + (this.ElementName || "") + ":" + (System.Boolean.toString(this.nullable) || "");
                }
            }
        },
        ctors: {
            init: function () {
                this.nullable = true;
            },
            /**
             * Initializes a new instance of the System.Xml.Serialization.XmlRootAttribute
             class.
             *
             * @instance
             * @public
             * @this System.Xml.Serialization.XmlRootAttribute
             * @memberof System.Xml.Serialization.XmlRootAttribute
             * @return  {void}
             */
            ctor: function () {
                this.$initialize();
                System.Attribute.ctor.call(this);
            },
            /**
             * Initializes a new instance of the System.Xml.Serialization.XmlRootAttribute
             class and specifies the name of the XML root element.
             *
             * @instance
             * @public
             * @this System.Xml.Serialization.XmlRootAttribute
             * @memberof System.Xml.Serialization.XmlRootAttribute
             * @param   {string}    elementName    The name of the XML root element.
             * @return  {void}
             */
            $ctor1: function (elementName) {
                this.$initialize();
                System.Attribute.ctor.call(this);
                this.elementName = elementName;
            }
        }
    });

    /**
     * An abstract class that is the base class for System.Xml.Serialization.XmlSerializationReader
     and System.Xml.Serialization.XmlSerializationWriter and that contains methods
     common to both of these types.
     *
     * @abstract
     * @public
     * @class System.Xml.Serialization.XmlSerializationGeneratedCode
     */
    Bridge.define("System.Xml.Serialization.XmlSerializationGeneratedCode", {
        ctors: {
            /**
             * Initializes an instance of the System.Xml.Serialization.XmlSerializationGeneratedCode
             class.
             *
             * @instance
             * @protected
             * @this System.Xml.Serialization.XmlSerializationGeneratedCode
             * @memberof System.Xml.Serialization.XmlSerializationGeneratedCode
             * @return  {void}
             */
            ctor: function () {
                this.$initialize();
                throw new System.NotImplementedException.ctor();
            }
        }
    });

    /**
     * Serializes and deserializes objects into and from XML documents. The System.Xml.Serialization.XmlSerializer
     enables you to control how objects are encoded into XML.
     * / Serializes the specified System.Object and writes the XML document to a file
    / using the specified System.Xml.XmlWriter.
    /
     * / Serializes the specified System.Object and writes the XML document to a file
    / using the specified System.Xml.XmlWriter and references the specified namespaces.
    /
     * / Serializes the specified System.Object and writes the XML document to a file
    / using the specified System.IO.Streamthat references the specified namespaces.
    /
     *
     * @public
     * @class System.Xml.Serialization.XmlSerializer
     */
    Bridge.define("System.Xml.Serialization.XmlSerializer", {
        statics: {
            fields: {
                MethodToSerializeUsingDataContractSerializer: null,
                MethodToDeserializeUsingDataContractSerializer: null
            }
        },
        fields: {
            _type: null
        },
        ctors: {
            /**
             * Initializes a new instance of the System.Xml.Serialization.XmlSerializer
             class.
             *
             * @instance
             * @protected
             * @this System.Xml.Serialization.XmlSerializer
             * @memberof System.Xml.Serialization.XmlSerializer
             * @return  {void}
             */
            ctor: function () {
                this.$initialize();
            },
            /**
             * Initializes a new instance of the System.Xml.Serialization.XmlSerializer
             class that can serialize objects of the specified type into XML documents,
             and deserialize XML documents into objects of the specified type.
             *
             * @instance
             * @public
             * @this System.Xml.Serialization.XmlSerializer
             * @memberof System.Xml.Serialization.XmlSerializer
             * @param   {System.Type}    type    The type of the object that this System.Xml.Serialization.XmlSerializer can
             serialize.
             * @return  {void}
             */
            $ctor1: function (type) {
                this.$initialize();
                this._type = type;
            }
        },
        methods: {
            /**
             * Gets a value that indicates whether this System.Xml.Serialization.XmlSerializer
             can deserialize a specified XML document.
             *
             * @instance
             * @public
             * @this System.Xml.Serialization.XmlSerializer
             * @memberof System.Xml.Serialization.XmlSerializer
             * @param   {System.Xml.XmlReader}    xmlReader    An System.Xml.XmlReader that points to the document to deserialize.
             * @return  {boolean}                              true if this System.Xml.Serialization.XmlSerializer can deserialize the object
             that the System.Xml.XmlReader points to; otherwise, false.
             */
            CanDeserialize: function (xmlReader) {
                throw new System.NotImplementedException.ctor();
            },
            /**
             * Deserializes the XML document contained by the specified System.IO.Stream.
             *
             * @instance
             * @public
             * @this System.Xml.Serialization.XmlSerializer
             * @memberof System.Xml.Serialization.XmlSerializer
             * @param   {System.IO.Stream}    stream    The System.IO.Stream that contains the XML document to deserialize.
             * @return  {System.Object}                 The System.Object being deserialized.
             */
            Deserialize: function (stream) {

                var reader = new System.IO.StreamReader.$ctor1(stream);
                var serializedXml = reader.ReadToEnd();
                return this.Deserialize$3(serializedXml);
            },
            /**
             * Deserializes the XML document contained by the specified System.xml.XmlReader.
             *
             * @instance
             * @public
             * @this System.Xml.Serialization.XmlSerializer
             * @memberof System.Xml.Serialization.XmlSerializer
             * @param   {System.Xml.XmlReader}    xmlReader    The System.xml.XmlReader that contains the XML document to deserialize.
             * @return  {System.Object}                        The System.Object being deserialized.
             */
            Deserialize$1: function (xmlReader) {

                var domNode = xmlReader._domNode;
                var serializedXml = (new XMLSerializer()).serializeToString(Bridge.unbox(domNode));
                var bytes = System.Text.Encoding.UTF8.GetBytes$2(serializedXml);
                var memoryStream = new System.IO.MemoryStream.$ctor1(bytes);
                return this.Deserialize$3(serializedXml);
            },
            Deserialize$3: function (serializedXml) {

                return System.Xml.Serialization.XmlSerializer.MethodToDeserializeUsingDataContractSerializer(serializedXml, this._type);
            },
            /**
             * Deserializes the XML document contained by the specified System.Xml.Serialization.XmlSerializationReader.
             *
             * @instance
             * @protected
             * @this System.Xml.Serialization.XmlSerializer
             * @memberof System.Xml.Serialization.XmlSerializer
             * @param   {System.Xml.Serialization.XmlSerializationReader}    reader    The System.Xml.Serialization.XmlSerializationReader that contains the XML
             document to deserialize.
             * @return  {System.Object}                                                The deserialized object.
             */
            Deserialize$2: function (reader) {
                throw new System.NotImplementedException.ctor();
            },
            /**
             * Serializes the specified System.Object and writes the XML document to a file
             using the specified System.IO.Stream.
             *
             * @instance
             * @public
             * @this System.Xml.Serialization.XmlSerializer
             * @memberof System.Xml.Serialization.XmlSerializer
             * @param   {System.IO.Stream}    stream    The System.IO.Stream used to write the XML document.
             * @param   {System.Object}       o         The System.Object to serialize.
             * @return  {void}
             */
            Serialize: function (stream, o) {

                alert('The XmlSerializer has been replaced by the DataContractSerializer.');

                var serializedObject = System.Xml.Serialization.XmlSerializer.MethodToSerializeUsingDataContractSerializer(o, this._type);
                var writer = new System.IO.StreamWriter.$ctor1(stream);
                try {
                    writer.Write$10(serializedObject);
                    writer.Flush();
                }
                finally {
                    if (Bridge.hasValue(writer)) {
                        writer.System$IDisposable$Dispose();
                    }
                }
            },
            /**
             * Serializes the specified System.Object and writes the XML document to a file
             using the specified System.Xml.Serialization.XmlSerializationWriter.
             *
             * @instance
             * @protected
             * @this System.Xml.Serialization.XmlSerializer
             * @memberof System.Xml.Serialization.XmlSerializer
             * @param   {System.Object}                                      o         The System.Object to serialize.
             * @param   {System.Xml.Serialization.XmlSerializationWriter}    writer    The System.Xml.Serialization.XmlSerializationWriter used to write the XML
             document.
             * @return  {void}
             */
            Serialize$1: function (o, writer) {
                throw new System.NotImplementedException.$ctor1("Please use the DataContractSerializer instead of the XmlSerializer.");
            },
            /**
             * Returns an object used to read the XML document to be serialized.
             *
             * @instance
             * @protected
             * @this System.Xml.Serialization.XmlSerializer
             * @memberof System.Xml.Serialization.XmlSerializer
             * @return  {System.Xml.Serialization.XmlSerializationReader}        An System.Xml.Serialization.XmlSerializationReader used to read the XML document.
             */
            CreateReader: function () {
                throw new System.NotImplementedException.ctor();
            },
            /**
             * When overridden in a derived class, returns a writer used to serialize the
             object.
             *
             * @instance
             * @protected
             * @this System.Xml.Serialization.XmlSerializer
             * @memberof System.Xml.Serialization.XmlSerializer
             * @return  {System.Xml.Serialization.XmlSerializationWriter}        An instance that implements the System.Xml.Serialization.XmlSerializationWriter
             class.
             */
            CreateWriter: function () {
                throw new System.NotImplementedException.ctor();
            }
        }
    });

    /**
     * Defines the reader, writer, and methods for pre-generated, typed serializers.
     *
     * @abstract
     * @public
     * @class System.Xml.Serialization.XmlSerializerImplementation
     */
    Bridge.define("System.Xml.Serialization.XmlSerializerImplementation", {
        props: {
            /**
             * Gets the XML reader object that is used by the serializer.
             *
             * @instance
             * @public
             * @readonly
             * @memberof System.Xml.Serialization.XmlSerializerImplementation
             * @function Reader
             * @type System.Xml.Serialization.XmlSerializationReader
             */
            Reader: {
                get: function () {
                    throw new System.NotImplementedException.ctor();
                }
            },
            /**
             * Gets the XML writer object for the serializer.
             *
             * @instance
             * @public
             * @readonly
             * @memberof System.Xml.Serialization.XmlSerializerImplementation
             * @function Writer
             * @type System.Xml.Serialization.XmlSerializationWriter
             */
            Writer: {
                get: function () {
                    throw new System.NotImplementedException.ctor();
                }
            }
        },
        ctors: {
            /**
             * Initializes a new instance of the System.Xml.Serialization.XmlSerializerImplementation
             class.
             *
             * @instance
             * @protected
             * @this System.Xml.Serialization.XmlSerializerImplementation
             * @memberof System.Xml.Serialization.XmlSerializerImplementation
             * @return  {void}
             */
            ctor: function () {
                this.$initialize();
            }
        },
        methods: {
            /**
             * Gets a value that determines whether a type can be serialized.
             *
             * @instance
             * @public
             * @this System.Xml.Serialization.XmlSerializerImplementation
             * @memberof System.Xml.Serialization.XmlSerializerImplementation
             * @param   {System.Type}    type    The System.Type to be serialized.
             * @return  {boolean}                true if the type can be serialized; otherwise, false.
             */
            CanSerialize: function (type) {
                throw new System.NotImplementedException.ctor();
            },
            /**
             * Returns a serializer for the specified type.
             *
             * @instance
             * @public
             * @this System.Xml.Serialization.XmlSerializerImplementation
             * @memberof System.Xml.Serialization.XmlSerializerImplementation
             * @param   {System.Type}                               type    The System.Type to be serialized.
             * @return  {System.Xml.Serialization.XmlSerializer}            An instance of a type derived from the System.Xml.Serialization.XmlSerializer
             class.
             */
            GetSerializer: function (type) {
                throw new System.NotImplementedException.ctor();
            }
        }
    });

    /**
     * Contains the XML namespaces and prefixes that the System.Xml.Serialization.XmlSerializer
     uses to generate qualified names in an XML-document instance.
     *
     * @public
     * @class System.Xml.Serialization.XmlSerializerNamespaces
     */
    Bridge.define("System.Xml.Serialization.XmlSerializerNamespaces");

    /**
     * Signifies that the code was generated by the serialization infrastructure
     and can be reused for increased performance, when this attribute is applied
     to an assembly.
     *
     * @public
     * @class System.Xml.Serialization.XmlSerializerVersionAttribute
     * @augments System.Attribute
     */
    Bridge.define("System.Xml.Serialization.XmlSerializerVersionAttribute", {
        inherits: [System.Attribute],
        fields: {
            /**
             * Gets or sets the common language runtime (CLR) namespace of the assembly.
             *
             * @instance
             * @public
             * @memberof System.Xml.Serialization.XmlSerializerVersionAttribute
             * @function Namespace
             * @type string
             */
            Namespace: null,
            /**
             * Gets or sets the identity of the parent assembly.
             *
             * @instance
             * @public
             * @memberof System.Xml.Serialization.XmlSerializerVersionAttribute
             * @function ParentAssemblyId
             * @type string
             */
            ParentAssemblyId: null,
            /**
             * Gets or sets the type that the serializer operates on.
             *
             * @instance
             * @public
             * @memberof System.Xml.Serialization.XmlSerializerVersionAttribute
             * @function Type
             * @type System.Type
             */
            Type: null,
            /**
             * Gets or sets the assembly's version number.
             *
             * @instance
             * @public
             * @memberof System.Xml.Serialization.XmlSerializerVersionAttribute
             * @function Version
             * @type string
             */
            Version: null
        },
        ctors: {
            /**
             * Initializes a new instance of the System.Xml.Serialization.XmlSerializerVersionAttribute
             class.
             *
             * @instance
             * @public
             * @this System.Xml.Serialization.XmlSerializerVersionAttribute
             * @memberof System.Xml.Serialization.XmlSerializerVersionAttribute
             * @return  {void}
             */
            ctor: function () {
                this.$initialize();
                System.Attribute.ctor.call(this);
            },
            /**
             * Initializes a new instance of the System.Xml.Serialization.XmlSerializerVersionAttribute
             class for the specified type.
             *
             * @instance
             * @public
             * @this System.Xml.Serialization.XmlSerializerVersionAttribute
             * @memberof System.Xml.Serialization.XmlSerializerVersionAttribute
             * @param   {System.Type}    type    The type that is being serialized.
             * @return  {void}
             */
            $ctor1: function (type) {
                this.$initialize();
                System.Attribute.ctor.call(this);
            }
        }
    });

    /**
     * Controls the XML schema that is generated when the attribute target is serialized
     by the System.Xml.Serialization.XmlSerializer.
     *
     * @public
     * @class System.Xml.Serialization.XmlTypeAttribute
     * @augments System.Attribute
     */
    Bridge.define("System.Xml.Serialization.XmlTypeAttribute", {
        inherits: [System.Attribute],
        fields: {
            includeInSchema: false,
            anonymousType: false,
            ns: null,
            typeName: null
        },
        props: {
            /**
             * Gets or sets a value that determines whether the resulting schema type is
             an XSD anonymous type.
             *
             * @instance
             * @public
             * @memberof System.Xml.Serialization.XmlTypeAttribute
             * @function AnonymousType
             * @type boolean
             */
            AnonymousType: {
                get: function () {
                    return this.anonymousType;
                },
                set: function (value) {
                    this.anonymousType = value;
                }
            },
            /**
             * Gets or sets a value that indicates whether to include the type in XML schema
             documents.
             *
             * @instance
             * @public
             * @memberof System.Xml.Serialization.XmlTypeAttribute
             * @function IncludeInSchema
             * @type boolean
             */
            IncludeInSchema: {
                get: function () {
                    return this.includeInSchema;
                },
                set: function (value) {
                    this.includeInSchema = value;
                }
            },
            /**
             * Gets or sets the name of the XML type.
             *
             * @instance
             * @public
             * @memberof System.Xml.Serialization.XmlTypeAttribute
             * @function TypeName
             * @type string
             */
            TypeName: {
                get: function () {
                    return this.typeName == null ? "" : this.typeName;
                },
                set: function (value) {
                    this.typeName = value;
                }
            },
            /**
             * Gets or sets the namespace of the XML type.
             *
             * @instance
             * @public
             * @memberof System.Xml.Serialization.XmlTypeAttribute
             * @function Namespace
             * @type string
             */
            Namespace: {
                get: function () {
                    return this.ns;
                },
                set: function (value) {
                    this.ns = value;
                }
            }
        },
        ctors: {
            init: function () {
                this.includeInSchema = true;
            },
            /**
             * Initializes a new instance of the System.Xml.Serialization.XmlTypeAttribute
             class.
             *
             * @instance
             * @public
             * @this System.Xml.Serialization.XmlTypeAttribute
             * @memberof System.Xml.Serialization.XmlTypeAttribute
             * @return  {void}
             */
            ctor: function () {
                this.$initialize();
                System.Attribute.ctor.call(this);
            },
            /**
             * Initializes a new instance of the System.Xml.Serialization.XmlTypeAttribute
             class and specifies the name of the XML type.
             *
             * @instance
             * @public
             * @this System.Xml.Serialization.XmlTypeAttribute
             * @memberof System.Xml.Serialization.XmlTypeAttribute
             * @param   {string}    typeName    The name of the XML type that the System.Xml.Serialization.XmlSerializer
             generates when it serializes the class instance (and recognizes when it deserializes
             the class instance).
             * @return  {void}
             */
            $ctor1: function (typeName) {
                this.$initialize();
                System.Attribute.ctor.call(this);
                this.typeName = typeName;
            }
        }
    });

    Bridge.define("System.Xml.WriteState", {
        $kind: "enum",
        statics: {
            fields: {
                Nothing: 0
            }
        }
    });

    Bridge.define("System.Xml.XmlConvert", {
        statics: {
            methods: {
                /**
                 * Converts the System.String to a System.Boolean equivalent.
                 *
                 * @static
                 * @public
                 * @this System.Xml.XmlConvert
                 * @memberof System.Xml.XmlConvert
                 * @param   {string}     s    The string to convert.
                 * @return  {boolean}         A Boolean value, that is, true or false.
                 */
                ToBoolean: function (s) {
                    if (Bridge.referenceEquals(s.toLowerCase(), "true")) {
                        return true;
                    }
                    return false;
                },
                /**
                 * Converts the System.String to a System.Byte equivalent.
                 *
                 * @static
                 * @public
                 * @this System.Xml.XmlConvert
                 * @memberof System.Xml.XmlConvert
                 * @param   {string}    s    The string to convert.
                 * @return  {number}         A Byte equivalent of the string.
                 */
                ToByte: function (s) {
                    return Bridge.Int.clipu8(System.Xml.XmlConvert.Parse(s));
                },
                /**
                 * Converts the System.String to a System.Decimal equivalent.
                 *
                 * @static
                 * @public
                 * @this System.Xml.XmlConvert
                 * @memberof System.Xml.XmlConvert
                 * @param   {string}            s    The string to convert.
                 * @return  {System.Decimal}         A Decimal equivalent of the string.
                 */
                ToDecimal: function (s) {
                    return System.Decimal(System.Xml.XmlConvert.Parse(s), null, System.Double);
                },
                /**
                 * Converts the System.String to a System.Double equivalent.
                 *
                 * @static
                 * @public
                 * @this System.Xml.XmlConvert
                 * @memberof System.Xml.XmlConvert
                 * @param   {string}    s    The string to convert.
                 * @return  {number}         A Double equivalent of the string.
                 */
                ToDouble: function (s) {
                    return System.Xml.XmlConvert.Parse(s);
                },
                /**
                 * Converts the System.String to a System.Guid equivalent.
                 *
                 * @static
                 * @public
                 * @this System.Xml.XmlConvert
                 * @memberof System.Xml.XmlConvert
                 * @param   {string}         s    The string to convert.
                 * @return  {System.Guid}         A Guid equivalent of the string.
                 */
                ToGuid: function (s) {
                    var guid = { v : new System.Guid() };

                    if (System.Guid.TryParse(s, guid)) {
                        return guid.v;
                    }
                    return System.Guid.NewGuid();
                },
                /**
                 * Converts the System.String to a System.Int16 equivalent.
                 *
                 * @static
                 * @public
                 * @this System.Xml.XmlConvert
                 * @memberof System.Xml.XmlConvert
                 * @param   {string}    s    The string to convert.
                 * @return  {number}         An Int16 equivalent of the string.
                 */
                ToInt16: function (s) {
                    return Bridge.Int.clip16(System.Xml.XmlConvert.Parse(s));
                },
                /**
                 * Converts the System.String to a System.Int32 equivalent.
                 *
                 * @static
                 * @public
                 * @this System.Xml.XmlConvert
                 * @memberof System.Xml.XmlConvert
                 * @param   {string}    s    The string to convert.
                 * @return  {number}         An Int32 equivalent of the string.
                 */
                ToInt32: function (s) {
                    return Bridge.Int.clip32(System.Xml.XmlConvert.Parse(s));
                },
                /**
                 * Converts the System.String to a System.Int64 equivalent.
                 *
                 * @static
                 * @public
                 * @this System.Xml.XmlConvert
                 * @memberof System.Xml.XmlConvert
                 * @param   {string}          s    The string to convert.
                 * @return  {System.Int64}         An Int64 equivalent of the string.
                 */
                ToInt64: function (s) {
                    return Bridge.Int.clip64(System.Xml.XmlConvert.Parse(s));
                },
                /**
                 * Converts the System.String to a System.SByte equivalent.
                 *
                 * @static
                 * @public
                 * @this System.Xml.XmlConvert
                 * @memberof System.Xml.XmlConvert
                 * @param   {string}    s    The string to convert.
                 * @return  {number}         An SByte equivalent of the string.
                 */
                ToSByte: function (s) {
                    return Bridge.Int.clip8(System.Xml.XmlConvert.Parse(s));
                },
                /**
                 * Converts the System.String to a System.Single equivalent.
                 *
                 * @static
                 * @public
                 * @this System.Xml.XmlConvert
                 * @memberof System.Xml.XmlConvert
                 * @param   {string}    s    The string to convert.
                 * @return  {number}         A Single equivalent of the string.
                 */
                ToSingle: function (s) {
                    return System.Xml.XmlConvert.Parse(s);
                },
                /**
                 * Converts the System.Boolean to a System.String.
                 *
                 * @static
                 * @public
                 * @this System.Xml.XmlConvert
                 * @memberof System.Xml.XmlConvert
                 * @param   {boolean}    value    The value to convert.
                 * @return  {string}              A string representation of the Boolean, that is, "true" or "false".
                 */
                ToString: function (value) {
                    if (value) {
                        return "true";
                    }
                    return "false";
                },
                /**
                 * Converts the System.Byte to a System.String.
                 *
                 * @static
                 * @public
                 * @this System.Xml.XmlConvert
                 * @memberof System.Xml.XmlConvert
                 * @param   {number}    value    The value to convert.
                 * @return  {string}             A string representation of the Byte.
                 */
                ToString$1: function (value) {
                    return Bridge.toString(value);
                },
                /**
                 * Converts the System.Char to a System.String.
                 *
                 * @static
                 * @public
                 * @this System.Xml.XmlConvert
                 * @memberof System.Xml.XmlConvert
                 * @param   {number}    value    The value to convert.
                 * @return  {string}             A string representation of the Char.
                 */
                ToString$2: function (value) {
                    return String.fromCharCode(value);
                },
                /**
                 * Converts the System.Double to a System.String.
                 *
                 * @static
                 * @public
                 * @this System.Xml.XmlConvert
                 * @memberof System.Xml.XmlConvert
                 * @param   {number}    value    The value to convert.
                 * @return  {string}             A string representation of the Double.
                 */
                ToString$3: function (value) {
                    return System.Double.format(value);
                },
                /**
                 * Converts the System.Single to a System.String.
                 *
                 * @static
                 * @public
                 * @this System.Xml.XmlConvert
                 * @memberof System.Xml.XmlConvert
                 * @param   {number}    value    The value to convert.
                 * @return  {string}             A string representation of the Single.
                 */
                ToString$9: function (value) {
                    return System.Single.format(value);
                },
                /**
                 * Converts the System.Guid to a System.String.
                 *
                 * @static
                 * @public
                 * @this System.Xml.XmlConvert
                 * @memberof System.Xml.XmlConvert
                 * @param   {System.Guid}    value    The value to convert.
                 * @return  {string}                  A string representation of the Guid.
                 */
                ToString$4: function (value) {
                    return value.toString();
                },
                /**
                 * Converts the System.Int32 to a System.String.
                 *
                 * @static
                 * @public
                 * @this System.Xml.XmlConvert
                 * @memberof System.Xml.XmlConvert
                 * @param   {number}    value    The value to convert.
                 * @return  {string}             A string representation of the Int32.
                 */
                ToString$6: function (value) {
                    return Bridge.toString(value);
                },
                /**
                 * Converts the System.Int64 to a System.String.
                 *
                 * @static
                 * @public
                 * @this System.Xml.XmlConvert
                 * @memberof System.Xml.XmlConvert
                 * @param   {System.Int64}    value    The value to convert.
                 * @return  {string}                   A string representation of the Int64.
                 */
                ToString$7: function (value) {
                    return Bridge.toString(value);
                },
                /**
                 * Converts the System.SByte to a System.String.
                 *
                 * @static
                 * @public
                 * @this System.Xml.XmlConvert
                 * @memberof System.Xml.XmlConvert
                 * @param   {number}    value    The value to convert.
                 * @return  {string}             A string representation of the SByte.
                 */
                ToString$8: function (value) {
                    return Bridge.toString(value);
                },
                /**
                 * Converts the System.Int16 to a System.String.
                 *
                 * @static
                 * @public
                 * @this System.Xml.XmlConvert
                 * @memberof System.Xml.XmlConvert
                 * @param   {number}    value    The value to convert.
                 * @return  {string}             A string representation of the Int16.
                 */
                ToString$5: function (value) {
                    return Bridge.toString(value);
                },
                /**
                 * Converts the System.UInt32 to a System.String.
                 *
                 * @static
                 * @public
                 * @this System.Xml.XmlConvert
                 * @memberof System.Xml.XmlConvert
                 * @param   {number}    value    The value to convert.
                 * @return  {string}             A string representation of the UInt32.
                 */
                ToString$11: function (value) {
                    return Bridge.toString(value);
                },
                /**
                 * Converts the System.UInt64 to a System.String.
                 *
                 * @static
                 * @public
                 * @this System.Xml.XmlConvert
                 * @memberof System.Xml.XmlConvert
                 * @param   {System.UInt64}    value    The value to convert.
                 * @return  {string}                    A string representation of the UInt64.
                 */
                ToString$12: function (value) {
                    return Bridge.toString(value);
                },
                /**
                 * Converts the System.UInt16 to a System.String.
                 *
                 * @static
                 * @public
                 * @this System.Xml.XmlConvert
                 * @memberof System.Xml.XmlConvert
                 * @param   {number}    value    The value to convert.
                 * @return  {string}             A string representation of the UInt16.
                 */
                ToString$10: function (value) {
                    return Bridge.toString(value);
                },
                /**
                 * Converts the System.String to a System.UInt16 equivalent.
                 *
                 * @static
                 * @public
                 * @this System.Xml.XmlConvert
                 * @memberof System.Xml.XmlConvert
                 * @param   {string}    s    The string to convert.
                 * @return  {number}         A UInt16 equivalent of the string.
                 */
                ToUInt16: function (s) {
                    return Bridge.Int.clipu16(System.Xml.XmlConvert.Parse(s));
                },
                /**
                 * Converts the System.String to a System.UInt32 equivalent.
                 *
                 * @static
                 * @public
                 * @this System.Xml.XmlConvert
                 * @memberof System.Xml.XmlConvert
                 * @param   {string}    s    The string to convert.
                 * @return  {number}         A UInt32 equivalent of the string.
                 */
                ToUInt32: function (s) {
                    return Bridge.Int.clipu32(System.Xml.XmlConvert.Parse(s));
                },
                /**
                 * Converts the System.String to a System.UInt64 equivalent.
                 *
                 * @static
                 * @public
                 * @this System.Xml.XmlConvert
                 * @memberof System.Xml.XmlConvert
                 * @param   {string}           s    The string to convert.
                 * @return  {System.UInt64}         A UInt64 equivalent of the string.
                 */
                ToUInt64: function (s) {
                    return Bridge.Int.clipu64(System.Xml.XmlConvert.Parse(s));
                },
                /**
                 * @static
                 * @private
                 * @this System.Xml.XmlConvert
                 * @memberof System.Xml.XmlConvert
                 * @param   {string}    s
                 * @return  {number}
                 */
                Parse: function (s) {
                    var number = { };

                    if (System.Double.tryParse(s, null, number)) {
                        return number.v;
                    }
                    return 0;
                }
            }
        },
        ctors: {
            /**
             * Initializes a new instance of the System.Xml.XmlConvert class.
             *
             * @instance
             * @public
             * @this System.Xml.XmlConvert
             * @memberof System.Xml.XmlConvert
             * @return  {void}
             */
            ctor: function () {
                this.$initialize();
            }
        }
    });

    /**
     * Specifies how to treat the time value when converting between string and
     System.DateTime.
     *
     * @public
     * @class System.Xml.XmlDateTimeSerializationMode
     */
    Bridge.define("System.Xml.XmlDateTimeSerializationMode", {
        $kind: "enum",
        statics: {
            fields: {
                /**
                 * Treat as local time. If the System.DateTime object represents a Coordinated
                 Universal Time (UTC), it is converted to the local time.
                 *
                 * @static
                 * @public
                 * @memberof System.Xml.XmlDateTimeSerializationMode
                 * @constant
                 * @default 0
                 * @type System.Xml.XmlDateTimeSerializationMode
                 */
                Local: 0,
                /**
                 * Treat as a UTC. If the System.DateTime object represents a local time, it
                 is converted to a UTC.
                 *
                 * @static
                 * @public
                 * @memberof System.Xml.XmlDateTimeSerializationMode
                 * @constant
                 * @default 1
                 * @type System.Xml.XmlDateTimeSerializationMode
                 */
                Utc: 1,
                /**
                 * Treat as a local time if a System.DateTime is being converted to a string.
                 *
                 * @static
                 * @public
                 * @memberof System.Xml.XmlDateTimeSerializationMode
                 * @constant
                 * @default 2
                 * @type System.Xml.XmlDateTimeSerializationMode
                 */
                Unspecified: 2,
                /**
                 * Time zone information should be preserved when converting.
                 *
                 * @static
                 * @public
                 * @memberof System.Xml.XmlDateTimeSerializationMode
                 * @constant
                 * @default 3
                 * @type System.Xml.XmlDateTimeSerializationMode
                 */
                RoundtripKind: 3
            }
        }
    });

    /**
     * Table of atomized string objects.
     *
     * @abstract
     * @public
     * @class System.Xml.XmlNameTable
     */
    Bridge.define("System.Xml.XmlNameTable", {
        ctors: {
            /**
             * Initializes a new instance of the System.Xml.XmlNameTable class.
             *
             * @instance
             * @protected
             * @this System.Xml.XmlNameTable
             * @memberof System.Xml.XmlNameTable
             * @return  {void}
             */
            ctor: function () {
                this.$initialize();
                throw new System.NotImplementedException.ctor();
            }
        }
    });

    /**
     * Specifies the type of node.
     *
     * @public
     * @class System.Xml.XmlNodeType
     */
    Bridge.define("System.Xml.XmlNodeType", {
        $kind: "enum",
        statics: {
            fields: {
                /**
                 * This is returned by the System.Xml.XmlReader if a Read method has not been
                 called.
                 *
                 * @static
                 * @public
                 * @memberof System.Xml.XmlNodeType
                 * @constant
                 * @default 0
                 * @type System.Xml.XmlNodeType
                 */
                None: 0,
                /**
                 * An element.
                 *
                 * @static
                 * @public
                 * @memberof System.Xml.XmlNodeType
                 * @constant
                 * @default 1
                 * @type System.Xml.XmlNodeType
                 */
                Element: 1,
                /**
                 * An attribute (for example, id='123').
                 *
                 * @static
                 * @public
                 * @memberof System.Xml.XmlNodeType
                 * @constant
                 * @default 2
                 * @type System.Xml.XmlNodeType
                 */
                Attribute: 2,
                /**
                 * The text content of a node.
                 *
                 * @static
                 * @public
                 * @memberof System.Xml.XmlNodeType
                 * @constant
                 * @default 3
                 * @type System.Xml.XmlNodeType
                 */
                Text: 3,
                /**
                 * A CDATA section.
                 *
                 * @static
                 * @public
                 * @memberof System.Xml.XmlNodeType
                 * @constant
                 * @default 4
                 * @type System.Xml.XmlNodeType
                 */
                CDATA: 4,
                /**
                 * A reference to an entity.
                 *
                 * @static
                 * @public
                 * @memberof System.Xml.XmlNodeType
                 * @constant
                 * @default 5
                 * @type System.Xml.XmlNodeType
                 */
                EntityReference: 5,
                /**
                 * An entity declaration.
                 *
                 * @static
                 * @public
                 * @memberof System.Xml.XmlNodeType
                 * @constant
                 * @default 6
                 * @type System.Xml.XmlNodeType
                 */
                Entity: 6,
                /**
                 * A processing instruction.
                 *
                 * @static
                 * @public
                 * @memberof System.Xml.XmlNodeType
                 * @constant
                 * @default 7
                 * @type System.Xml.XmlNodeType
                 */
                ProcessingInstruction: 7,
                /**
                 * A comment.
                 *
                 * @static
                 * @public
                 * @memberof System.Xml.XmlNodeType
                 * @constant
                 * @default 8
                 * @type System.Xml.XmlNodeType
                 */
                Comment: 8,
                /**
                 * A document object that, as the root of the document tree, provides access
                 to the entire XML document.
                 *
                 * @static
                 * @public
                 * @memberof System.Xml.XmlNodeType
                 * @constant
                 * @default 9
                 * @type System.Xml.XmlNodeType
                 */
                Document: 9,
                /**
                 * The document type declaration, indicated by the following tag.
                 *
                 * @static
                 * @public
                 * @memberof System.Xml.XmlNodeType
                 * @constant
                 * @default 10
                 * @type System.Xml.XmlNodeType
                 */
                DocumentType: 10,
                /**
                 * A document fragment.
                 *
                 * @static
                 * @public
                 * @memberof System.Xml.XmlNodeType
                 * @constant
                 * @default 11
                 * @type System.Xml.XmlNodeType
                 */
                DocumentFragment: 11,
                /**
                 * A notation in the document type declaration.
                 *
                 * @static
                 * @public
                 * @memberof System.Xml.XmlNodeType
                 * @constant
                 * @default 12
                 * @type System.Xml.XmlNodeType
                 */
                Notation: 12,
                /**
                 * White space between markup.
                 *
                 * @static
                 * @public
                 * @memberof System.Xml.XmlNodeType
                 * @constant
                 * @default 13
                 * @type System.Xml.XmlNodeType
                 */
                Whitespace: 13,
                /**
                 * White space between markup in a mixed content model or white space within
                 the xml:space="preserve" scope.
                 *
                 * @static
                 * @public
                 * @memberof System.Xml.XmlNodeType
                 * @constant
                 * @default 14
                 * @type System.Xml.XmlNodeType
                 */
                SignificantWhitespace: 14,
                /**
                 * An end element tag.
                 *
                 * @static
                 * @public
                 * @memberof System.Xml.XmlNodeType
                 * @constant
                 * @default 15
                 * @type System.Xml.XmlNodeType
                 */
                EndElement: 15,
                /**
                 * Returned when XmlReader gets to the end of the entity replacement as a result
                 of a call to System.Xml.XmlReader.ResolveEntity().
                 *
                 * @static
                 * @public
                 * @memberof System.Xml.XmlNodeType
                 * @constant
                 * @default 16
                 * @type System.Xml.XmlNodeType
                 */
                EndEntity: 16,
                /**
                 * The XML declaration.
                 *
                 * @static
                 * @public
                 * @memberof System.Xml.XmlNodeType
                 * @constant
                 * @default 17
                 * @type System.Xml.XmlNodeType
                 */
                XmlDeclaration: 17
            }
        }
    });

    /**
     * Provides all the context information required by the System.Xml.XmlReader
     to parse an XML fragment.
     *
     * @public
     * @class System.Xml.XmlParserContext
     */
    Bridge.define("System.Xml.XmlParserContext");

    /**
     * Represents an XML qualified name.
     *
     * @public
     * @class System.Xml.XmlQualifiedName
     */
    Bridge.define("System.Xml.XmlQualifiedName", {
        statics: {
            fields: {
                /**
                 * Provides an empty System.Xml.XmlQualifiedName.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof System.Xml.XmlQualifiedName
                 * @type System.Xml.XmlQualifiedName
                 */
                Empty: null
            },
            methods: {
                /**
                 * Returns the string value of the System.Xml.XmlQualifiedName.
                 *
                 * @static
                 * @public
                 * @this System.Xml.XmlQualifiedName
                 * @memberof System.Xml.XmlQualifiedName
                 * @param   {string}    name    The name of the object.
                 * @param   {string}    ns      The namespace of the object.
                 * @return  {string}            The string value of the System.Xml.XmlQualifiedName in the format of namespace:localname.
                 If the object does not have a namespace defined, this method returns just
                 the local name.
                 */
                ToString: function (name, ns) {
                    throw new System.NotImplementedException.ctor();
                }/**
                 * Compares two System.Xml.XmlQualifiedName objects.
                 *
                 * @static
                 * @public
                 * @this System.Xml.XmlQualifiedName
                 * @memberof System.Xml.XmlQualifiedName
                 * @param   {System.Xml.XmlQualifiedName}    a    An System.Xml.XmlQualifiedName to compare.
                 * @param   {System.Xml.XmlQualifiedName}    b    An System.Xml.XmlQualifiedName to compare.
                 * @return  {boolean}                             true if the name and namespace values for the two objects differ; otherwise,
                 false.
                 */
                ,
                op_Inequality: function (a, b) {
                    throw new System.NotImplementedException.ctor();
                }/**
                 * Compares two System.Xml.XmlQualifiedName objects.
                 *
                 * @static
                 * @public
                 * @this System.Xml.XmlQualifiedName
                 * @memberof System.Xml.XmlQualifiedName
                 * @param   {System.Xml.XmlQualifiedName}    a    An System.Xml.XmlQualifiedName to compare.
                 * @param   {System.Xml.XmlQualifiedName}    b    An System.Xml.XmlQualifiedName to compare.
                 * @return  {boolean}                             true if the two objects have the same name and namespace values; otherwise,
                 false.
                 */
                ,
                op_Equality: function (a, b) {
                    throw new System.NotImplementedException.ctor();
                }
            }
        },
        props: {
            /**
             * Gets a value indicating whether the System.Xml.XmlQualifiedName is empty.
             *
             * @instance
             * @public
             * @readonly
             * @memberof System.Xml.XmlQualifiedName
             * @function IsEmpty
             * @type boolean
             */
            IsEmpty: {
                get: function () {
                    throw new System.NotImplementedException.ctor();
                }
            },
            /**
             * Gets a string representation of the qualified name of the System.Xml.XmlQualifiedName.
             *
             * @instance
             * @public
             * @readonly
             * @memberof System.Xml.XmlQualifiedName
             * @function Name
             * @type string
             */
            Name: {
                get: function () {
                    throw new System.NotImplementedException.ctor();
                }
            },
            /**
             * Gets a string representation of the namespace of the System.Xml.XmlQualifiedName.
             *
             * @instance
             * @public
             * @readonly
             * @memberof System.Xml.XmlQualifiedName
             * @function Namespace
             * @type string
             */
            Namespace: {
                get: function () {
                    throw new System.NotImplementedException.ctor();
                }
            }
        },
        ctors: {
            /**
             * Initializes a new instance of the System.Xml.XmlQualifiedName class.
             *
             * @instance
             * @public
             * @this System.Xml.XmlQualifiedName
             * @memberof System.Xml.XmlQualifiedName
             * @return  {void}
             */
            ctor: function () {
                this.$initialize();
                throw new System.NotImplementedException.ctor();
            },
            /**
             * Initializes a new instance of the System.Xml.XmlQualifiedName class with
             the specified name.
             *
             * @instance
             * @public
             * @this System.Xml.XmlQualifiedName
             * @memberof System.Xml.XmlQualifiedName
             * @param   {string}    name    The local name to use as the name of the System.Xml.XmlQualifiedName object.
             * @return  {void}
             */
            $ctor1: function (name) {
                this.$initialize();
                throw new System.NotImplementedException.ctor();
            },
            /**
             * Initializes a new instance of the System.Xml.XmlQualifiedName class with
             the specified name and namespace.
             *
             * @instance
             * @public
             * @this System.Xml.XmlQualifiedName
             * @memberof System.Xml.XmlQualifiedName
             * @param   {string}    name    The local name to use as the name of the System.Xml.XmlQualifiedName object.
             * @param   {string}    ns      The namespace for the System.Xml.XmlQualifiedName object.
             * @return  {void}
             */
            $ctor2: function (name, ns) {
                this.$initialize();
                throw new System.NotImplementedException.ctor();
            }
        },
        methods: {
            /**
             * Determines whether the specified System.Xml.XmlQualifiedName object is equal
             to the current System.Xml.XmlQualifiedName object.
             *
             * @instance
             * @public
             * @override
             * @this System.Xml.XmlQualifiedName
             * @memberof System.Xml.XmlQualifiedName
             * @param   {System.Object}    other    The System.Xml.XmlQualifiedName to compare.
             * @return  {boolean}                   true if the two are the same instance object; otherwise, false.
             */
            equals: function (other) {
                throw new System.NotImplementedException.ctor();
            },
            /**
             * Returns the hash code for the System.Xml.XmlQualifiedName.
             *
             * @instance
             * @public
             * @override
             * @this System.Xml.XmlQualifiedName
             * @memberof System.Xml.XmlQualifiedName
             * @return  {number}        A hash code for this object.
             */
            getHashCode: function () {
                throw new System.NotImplementedException.ctor();
            },
            /**
             * Returns the string value of the System.Xml.XmlQualifiedName.
             *
             * @instance
             * @public
             * @override
             * @this System.Xml.XmlQualifiedName
             * @memberof System.Xml.XmlQualifiedName
             * @return  {string}        The string value of the System.Xml.XmlQualifiedName in the format of namespace:localname.
             If the object does not have a namespace defined, this method returns just
             the local name.
             */
            toString: function () {
                throw new System.NotImplementedException.ctor();
            }
        }
    });

    /**
     * Specifies a set of features to support on the System.Xml.XmlReader object
     created by the Overload:System.Xml.XmlReader.Create method.
     *
     * @public
     * @class System.Xml.XmlReaderSettings
     */
    Bridge.define("System.Xml.XmlReaderSettings", {
        ctors: {
            /**
             * Initializes a new instance of the System.Xml.XmlReaderSettings class.
             *
             * @instance
             * @public
             * @this System.Xml.XmlReaderSettings
             * @memberof System.Xml.XmlReaderSettings
             * @return  {void}
             */
            ctor: function () {
                this.$initialize();
                throw new System.NotImplementedException.ctor();
            }
        }
    });

    Bridge.define("System.Xml.XmlSpace", {
        $kind: "enum"
    });

    Bridge.define("ToJavaScriptObjectExtender.ToJavaScriptObjectExtender", {
        statics: {
            methods: {
                ToJavaScriptObject$3: function (o) {
                    return o;
                },
                ToJavaScriptObject$5: function (o) {
                    return o.TypeScriptDefinitionsSupport$IJSObject$UnderlyingJSInstance;
                },
                ToJavaScriptObject$4: function (S) {
                    return S;
                },
                ToJavaScriptObject$2: function (D) {
                    return Bridge.box(D, System.Double, System.Double.format, System.Double.getHashCode);
                },
                ToJavaScriptObject$1: function (B) {
                    return Bridge.box(B, System.Boolean, System.Boolean.toString);
                },
                ToJavaScriptObject: function (a) {
                    return a;
                },
                ToJavaScriptObject$6: function (a) {
                    return a;
                }
            }
        }
    });

    Bridge.define("TypeScriptDefinitionsSupport.IJSObject", {
        $kind: "interface"
    });

    /** @namespace Windows.ApplicationModel.Activation */

    /**
     * Provides event information when an app is launched
     *
     * @public
     * @class Windows.ApplicationModel.Activation.LaunchActivatedEventArgs
     */
    Bridge.define("Windows.ApplicationModel.Activation.LaunchActivatedEventArgs");

    /**
     * The {@link } namespace enables an app to
     handle the activation parameters for the activation contracts, appointments, and
     extensions supported by Windows.
     *
     * @class Windows.ApplicationModel.Activation.NamespaceDoc
     */
    Bridge.define("Windows.ApplicationModel.Activation.NamespaceDoc");

    /** @namespace Windows.Devices.Input */

    /**
     * The {@link } namespace provides support for identifying
     the input devices available (pointer, touch, mouse, and keyboard) and retrieving
     information about those devices.
     *
     * @class Windows.Devices.Input.NamespaceDoc
     */
    Bridge.define("Windows.Devices.Input.NamespaceDoc");

    /**
     * Enumerates pointer device types.
     *
     * @public
     * @class Windows.Devices.Input.PointerDeviceType
     */
    Bridge.define("Windows.Devices.Input.PointerDeviceType", {
        $kind: "enum",
        statics: {
            fields: {
                /**
                 * A touch-enabled device
                 *
                 * @static
                 * @public
                 * @memberof Windows.Devices.Input.PointerDeviceType
                 * @constant
                 * @default 0
                 * @type Windows.Devices.Input.PointerDeviceType
                 */
                Touch: 0,
                /**
                 * Pen
                 *
                 * @static
                 * @public
                 * @memberof Windows.Devices.Input.PointerDeviceType
                 * @constant
                 * @default 1
                 * @type Windows.Devices.Input.PointerDeviceType
                 */
                Pen: 1,
                /**
                 * Mouse
                 *
                 * @static
                 * @public
                 * @memberof Windows.Devices.Input.PointerDeviceType
                 * @constant
                 * @default 2
                 * @type Windows.Devices.Input.PointerDeviceType
                 */
                Mouse: 2
            }
        }
    });

    /** @namespace Windows.Foundation */

    /**
     * The {@link } namespace enables fundamental Windows Runtime
     functionality, including managing asynchronous operations and accessing property
     stores. This namespace also defines common value types that represent Uniform
     Resource Identifiers (URIs), dates and times, 2-D measurements, and other basic values.
     *
     * @class Windows.Foundation.NamespaceDoc
     */
    Bridge.define("Windows.Foundation.NamespaceDoc");

    /**
     * Represents an x- and y-coordinate pair in two-dimensional
     space. Can also represent a logical point for certain property usages.
     *
     * @public
     * @class Windows.Foundation.Point
     */
    Bridge.define("Windows.Foundation.Point", {
        $kind: "struct",
        statics: {
            ctors: {
                ctor: function () {
                    DotNetForHtml5.Core.TypeFromStringConverters.RegisterConverter(Windows.Foundation.Point, function (s) {
                        return Windows.Foundation.Point.Parse(s).$clone();
                    });
                }
            },
            methods: {
                Parse: function (pointAsString) {
                    var splittedString = System.String.split(pointAsString, System.Array.init([44, 32], System.Char).map(function (i) {{ return String.fromCharCode(i); }}), null, 1);

                    if (splittedString.length === 2) {
                        var x = { }, y = { };
                        if (System.Double.tryParse(splittedString[System.Array.index(0, splittedString)], null, x) && System.Double.tryParse(splittedString[System.Array.index(1, splittedString)], null, y)) {
                            return new Windows.Foundation.Point.$ctor1(x.v, y.v);
                        }
                    }

                    throw new System.FormatException.$ctor1((pointAsString || "") + " is not an eligible value for a Point");
                }/**
                 * Compares two Windows.Foundation.Point structures for inequality
                 *
                 * @static
                 * @public
                 * @this Windows.Foundation.Point
                 * @memberof Windows.Foundation.Point
                 * @param   {Windows.Foundation.Point}    point1    The first point to compare.
                 * @param   {Windows.Foundation.Point}    point2    The second point to compare.
                 * @return  {boolean}                               true if point1 and point2 have different Windows.Foundation.Point.X or Windows.Foundation.Point.Y
                 values; false if point1 and point2 have the same Windows.Foundation.Point.X
                 and Windows.Foundation.Point.Y values.
                 */
                ,
                op_Inequality: function (point1, point2) {
                    return (point1.X !== point2.X || point1.Y !== point2.Y);
                }/**
                 * Compares two Windows.Foundation.Point structures for equality
                 *
                 * @static
                 * @public
                 * @this Windows.Foundation.Point
                 * @memberof Windows.Foundation.Point
                 * @param   {Windows.Foundation.Point}    point1    The first point to compare.
                 * @param   {Windows.Foundation.Point}    point2    The second point to compare.
                 * @return  {boolean}                               true if both the Windows.Foundation.Point.X and Windows.Foundation.Point.Y
                 values of point1 and point2 are equal; otherwise, false.
                 */
                ,
                op_Equality: function (point1, point2) {
                    return (point1.X === point2.X && point1.Y === point2.Y);

                },
                getDefaultValue: function () { return new Windows.Foundation.Point(); }
            }
        },
        fields: {
            _x: 0,
            _y: 0
        },
        props: {
            /**
             * Gets or sets the Windows.Foundation.Point.X-coordinate
             value of this Windows.Foundation.Point structure.
             *
             * @instance
             * @public
             * @memberof Windows.Foundation.Point
             * @function X
             * @type number
             */
            X: {
                get: function () {
                    return this._x;
                },
                set: function (value) {
                    this._x = value;
                }
            },
            /**
             * Gets or sets the Windows.Foundation.Point.Y-coordinate
             value of this Windows.Foundation.Point.
             *
             * @instance
             * @public
             * @memberof Windows.Foundation.Point
             * @function Y
             * @type number
             */
            Y: {
                get: function () {
                    return this._y;
                },
                set: function (value) {
                    this._y = value;
                }
            }
        },
        ctors: {
            /**
             * Initializes a Windows.Foundation.Point structure that
             contains the specified values.
             *
             * @instance
             * @public
             * @this Windows.Foundation.Point
             * @memberof Windows.Foundation.Point
             * @param   {number}    x    The x-coordinate value of the Windows.Foundation.Point structure.
             * @param   {number}    y    The y-coordinate value of the Windows.Foundation.Point structure.
             * @return  {void}
             */
            $ctor1: function (x, y) {
                this.$initialize();
                this._x = x;
                this._y = y;
            },
            ctor: function () {
                this.$initialize();
            }
        },
        methods: {
            /**
             * Determines whether the specified object is a Windows.Foundation.Point
             and whether it contains the same values as this Windows.Foundation.Point.
             *
             * @instance
             * @public
             * @override
             * @this Windows.Foundation.Point
             * @memberof Windows.Foundation.Point
             * @param   {System.Object}    o    The object to compare.
             * @return  {boolean}               true if obj is a Windows.Foundation.Point and contains the same Windows.Foundation.Point.X
             and Windows.Foundation.Point.Y values as this Windows.Foundation.Point; otherwise,
             false.
             */
            equals: function (o) {
                return (Bridge.is(o, Windows.Foundation.Point) && Windows.Foundation.Point.op_Equality(System.Nullable.getValue(Bridge.cast(Bridge.unbox(o, Windows.Foundation.Point), Windows.Foundation.Point)), this));
            },
            /**
             * Compares two Windows.Foundation.Point structures for
             equality.
             *
             * @instance
             * @public
             * @this Windows.Foundation.Point
             * @memberof Windows.Foundation.Point
             * @param   {Windows.Foundation.Point}    value    The point to compare to this instance.
             * @return  {boolean}                              true if both Windows.Foundation.Point structures contain the same Windows.Foundation.Point.X
             and Windows.Foundation.Point.Y values; otherwise, false.
             */
            Equals: function (value) {
                return Windows.Foundation.Point.op_Equality(value.$clone(), this);
            },
            /**
             * Returns the hash code for this Windows.Foundation.Point.
             *
             * @instance
             * @public
             * @override
             * @this Windows.Foundation.Point
             * @memberof Windows.Foundation.Point
             * @return  {number}        The hash code for this Windows.Foundation.Point.
             */
            getHashCode: function () {
                return (System.Double.getHashCode(this.X) ^ System.Double.getHashCode(this.Y));
            },
            /**
             * Creates a System.String representation of this Windows.Foundation.Point.
             *
             * @instance
             * @public
             * @override
             * @this Windows.Foundation.Point
             * @memberof Windows.Foundation.Point
             * @return  {string}        A System.String containing the Windows.Foundation.Point.X and Windows.Foundation.Point.Y
             values of this Windows.Foundation.Point structure.
             */
            toString: function () {
                return System.Double.format(this.X) + "," + System.Double.format(this.Y);
            },
            ToString: function (format, formatProvider) {
                return null;
            },
            $clone: function (to) {
                var s = to || new Windows.Foundation.Point();
                s._x = this._x;
                s._y = this._y;
                return s;
            }
        }
    });

    /**
     * Describes the width, height, and point origin of a rectangle.
     *
     * @public
     * @class Windows.Foundation.Rect
     */
    Bridge.define("Windows.Foundation.Rect", {
        $kind: "struct",
        statics: {
            fields: {
                /**
                 * Gets a special value that represents a rectangle with
                 no position or area.
                 *
                 * @static
                 * @public
                 * @memberof Windows.Foundation.Rect
                 * @function Empty
                 * @type Windows.Foundation.Rect
                 */
                Empty: null
            },
            ctors: {
                init: function () {
                    this.Empty = new Windows.Foundation.Rect();
                },
                ctor: function () {
                    var $t;
                    Windows.Foundation.Rect.Empty = ($t = new Windows.Foundation.Rect.ctor(), $t._width = Number.NEGATIVE_INFINITY, $t._height = Number.NEGATIVE_INFINITY, $t._x = Number.POSITIVE_INFINITY, $t._y = Number.POSITIVE_INFINITY, $t);
                    DotNetForHtml5.Core.TypeFromStringConverters.RegisterConverter(Windows.Foundation.Rect, function (s) {
                        return Windows.Foundation.Rect.Parse(s).$clone();
                    });
                }
            },
            methods: {
                Parse: function (rectAsString) {
                    var splittedString = System.String.split(rectAsString, System.Array.init([44, 32], System.Char).map(function (i) {{ return String.fromCharCode(i); }}), null, 1);

                    if (splittedString.length === 4) {
                        var x = { }, y = { }, width = { }, height = { };
                        if (System.Double.tryParse(splittedString[System.Array.index(0, splittedString)], null, x) && System.Double.tryParse(splittedString[System.Array.index(1, splittedString)], null, y) && System.Double.tryParse(splittedString[System.Array.index(2, splittedString)], null, width) && System.Double.tryParse(splittedString[System.Array.index(3, splittedString)], null, height)) {
                            return new Windows.Foundation.Rect.$ctor1(x.v, y.v, width.v, height.v);
                        }
                    }

                    throw new System.FormatException.$ctor1((rectAsString || "") + "is not an eligible value for Rect");
                }/**
                 * Compares two Windows.Foundation.Rect structures for inequality.
                 *
                 * @static
                 * @public
                 * @this Windows.Foundation.Rect
                 * @memberof Windows.Foundation.Rect
                 * @param   {Windows.Foundation.Rect}    rect1    The first rectangle to compare.
                 * @param   {Windows.Foundation.Rect}    rect2    The second rectangle to compare.
                 * @return  {boolean}                             true if the Windows.Foundation.Rect structures do not have the same x,y,width,height
                 property values; otherwise, false.
                 */
                ,
                op_Inequality: function (rect1, rect2) {
                    return (rect1.X !== rect2.X || rect1.Y !== rect2.Y || rect1.Width !== rect2.Width || rect1.Height !== rect2.Height);
                }/**
                 * Compares two Windows.Foundation.Rect structures for equality.
                 *
                 * @static
                 * @public
                 * @this Windows.Foundation.Rect
                 * @memberof Windows.Foundation.Rect
                 * @param   {Windows.Foundation.Rect}    rect1    The first rectangle to compare.
                 * @param   {Windows.Foundation.Rect}    rect2    The second rectangle to compare.
                 * @return  {boolean}                             true if the Windows.Foundation.Rect structures have the same x,y,width,height
                 property values; otherwise, false.
                 */
                ,
                op_Equality: function (rect1, rect2) {
                    return (rect1.X === rect2.X && rect1.Y === rect2.Y && rect1.Width === rect2.Width && rect1.Height === rect2.Height);
                },
                getDefaultValue: function () { return new Windows.Foundation.Rect(); }
            }
        },
        fields: {
            _x: 0,
            _y: 0,
            _width: 0,
            _height: 0
        },
        props: {
            /**
             * Gets the y-axis value of the bottom of the rectangle.
             *
             * @instance
             * @public
             * @readonly
             * @memberof Windows.Foundation.Rect
             * @function Bottom
             * @type number
             */
            Bottom: {
                get: function () {
                    if (this.IsEmpty) {
                        return Number.NEGATIVE_INFINITY;
                    }
                    return this.Y + this.Height;
                }
            },
            /**
             * Gets the position of the bottom-left corner of the rectangle
             *
             * @instance
             * @public
             * @readonly
             * @memberof Windows.Foundation.Rect
             * @function BottomLeft
             * @type Windows.Foundation.Point
             */
            BottomLeft: {
                get: function () {
                    return new Windows.Foundation.Point.$ctor1(this.Left, this.Bottom);
                }
            },
            /**
             * Gets the position of the bottom-right corner of the rectangle.
             *
             * @instance
             * @public
             * @readonly
             * @memberof Windows.Foundation.Rect
             * @function BottomRight
             * @type Windows.Foundation.Point
             */
            BottomRight: {
                get: function () {
                    return new Windows.Foundation.Point.$ctor1(this.Right, this.Bottom);
                }
            },
            /**
             * Gets or sets the height of the rectangle.
             *
             * @instance
             * @public
             * @memberof Windows.Foundation.Rect
             * @function Height
             * @type number
             */
            Height: {
                get: function () {
                    return this._height;
                },
                set: function (value) {
                    if (value < 0) {
                        throw new System.ArgumentException.$ctor1("Height cannot be lower than 0");
                    }
                    if (this.IsEmpty) {
                        throw new System.InvalidOperationException.$ctor1("Cannot modify empty Rect.");
                    }

                    this._height = value;
                }
            },
            /**
             * Gets a value that indicates whether the rectangle is
             the Windows.Foundation.Rect.Empty rectangle.
             *
             * @instance
             * @public
             * @readonly
             * @memberof Windows.Foundation.Rect
             * @function IsEmpty
             * @type boolean
             */
            IsEmpty: {
                get: function () {
                    return this.Width < 0 && this.Height < 0;
                }
            },
            /**
             * Gets the x-axis value of the left side of the rectangle.
             *
             * @instance
             * @public
             * @readonly
             * @memberof Windows.Foundation.Rect
             * @function Left
             * @type number
             */
            Left: {
                get: function () {
                    if (this.IsEmpty) {
                        return Number.NEGATIVE_INFINITY;
                    }
                    return this.X;
                }
            },
            /**
             * Gets or sets the position of the top-left corner of the rectangle.
             *
             * @instance
             * @public
             * @memberof Windows.Foundation.Rect
             * @function Location
             * @type Windows.Foundation.Point
             */
            Location: {
                get: function () {
                    return new Windows.Foundation.Point.$ctor1(this.X, this.Y);
                },
                set: function (value) {
                    if (this.IsEmpty) {
                        throw new System.InvalidOperationException.$ctor1("Cannot modify empty Rect.");
                    }
                    this.X = value.X;
                    this.Y = value.Y;
                }
            },
            /**
             * Gets the x-axis value of the right side of the rectangle.
             *
             * @instance
             * @public
             * @readonly
             * @memberof Windows.Foundation.Rect
             * @function Right
             * @type number
             */
            Right: {
                get: function () {
                    if (this.IsEmpty) {
                        return Number.NEGATIVE_INFINITY;
                    }
                    return this.X + this.Width;
                }
            },
            /**
             * Gets or sets the width and height of the rectangle.
             *
             * @instance
             * @public
             * @memberof Windows.Foundation.Rect
             * @function Size
             * @type Windows.Foundation.Size
             */
            Size: {
                get: function () {
                    return new Windows.Foundation.Size.$ctor1(this.Width, this.Height);
                },
                set: function (value) {
                    if (this.IsEmpty) {
                        throw new System.InvalidOperationException.$ctor1("Cannot modify empty Rect.");
                    }
                    this.Width = value.Width;
                    this.Height = value.Height;
                }
            },
            /**
             * Gets the y-axis position of the top of the rectangle.
             *
             * @instance
             * @public
             * @readonly
             * @memberof Windows.Foundation.Rect
             * @function Top
             * @type number
             */
            Top: {
                get: function () {
                    if (this.IsEmpty) {
                        return Number.NEGATIVE_INFINITY;
                    }
                    return this.Y;
                }
            },
            /**
             * Gets the position of the top-left corner of the rectangle
             *
             * @instance
             * @public
             * @readonly
             * @memberof Windows.Foundation.Rect
             * @function TopLeft
             * @type Windows.Foundation.Point
             */
            TopLeft: {
                get: function () {
                    return new Windows.Foundation.Point.$ctor1(this.Left, this.Top);
                }
            },
            /**
             * Gets the position of the top-right corner of the rectangle.
             *
             * @instance
             * @public
             * @readonly
             * @memberof Windows.Foundation.Rect
             * @function TopRight
             * @type Windows.Foundation.Point
             */
            TopRight: {
                get: function () {
                    return new Windows.Foundation.Point.$ctor1(this.Right, this.Top);
                }
            },
            /**
             * Gets or sets the width of the rectangle.
             *
             * @instance
             * @public
             * @memberof Windows.Foundation.Rect
             * @function Width
             * @type number
             */
            Width: {
                get: function () {
                    return this._width;
                },
                set: function (value) {
                    if (!this.IsEmpty && value < 0) {
                        throw new System.ArgumentException.$ctor1("Width cannot be lower than 0");
                    } else {
                        this._width = value;
                    }
                }
            },
            /**
             * Gets or sets the x-axis value of the left side of the
             rectangle.
             *
             * @instance
             * @public
             * @memberof Windows.Foundation.Rect
             * @function X
             * @type number
             */
            X: {
                get: function () {
                    return this._x;
                },
                set: function (value) {
                    if (this.IsEmpty) {
                        throw new System.InvalidOperationException.$ctor1("Cannot modify empty Rect.");
                    }

                    this._x = value;
                }
            },
            /**
             * Gets or sets the y-axis value of the top side of the
             rectangle.
             *
             * @instance
             * @public
             * @memberof Windows.Foundation.Rect
             * @function Y
             * @type number
             */
            Y: {
                get: function () {
                    return this._y;
                },
                set: function (value) {
                    if (this.IsEmpty) {
                        throw new System.InvalidOperationException.$ctor1("Cannot modify empty Rect.");
                    }

                    this._y = value;
                }
            }
        },
        ctors: {
            /**
             * Initializes a Windows.Foundation.Rect structure that
             is exactly large enough to contain the two specified points.
             *
             * @instance
             * @public
             * @this Windows.Foundation.Rect
             * @memberof Windows.Foundation.Rect
             * @param   {Windows.Foundation.Point}    point1    The first point that the new rectangle must contain.
             * @param   {Windows.Foundation.Point}    point2    The second point that the new rectangle must contain.
             * @return  {void}
             */
            $ctor2: function (point1, point2) {
                this.$initialize();
                this._x = 0;
                this._y = 0;
                this._width = 0;
                this._height = 0;


                var x1 = (point1.X < point2.X) ? point1.X : point2.X;
                var x2 = (point1.X >= point2.X) ? point1.X : point2.X;

                this.X = point1.X;
                this.Width = point2.X - point1.X;


                var y1 = (point1.Y < point2.Y) ? point1.Y : point2.Y;
                var y2 = (point1.Y >= point2.Y) ? point1.Y : point2.Y;

                this.Y = point1.Y;
                this.Height = point2.Y - point1.Y;
            },
            /**
             * Initializes a Windows.Foundation.Rect structure based
             on an origin and size.
             *
             * @instance
             * @public
             * @this Windows.Foundation.Rect
             * @memberof Windows.Foundation.Rect
             * @param   {Windows.Foundation.Point}    location    The origin of the new Windows.Foundation.Rect.
             * @param   {Windows.Foundation.Size}     size        The size of the new Windows.Foundation.Rect.
             * @return  {void}
             */
            $ctor3: function (location, size) {
                this.$initialize();
                this._x = 0;
                this._y = 0;
                this._width = 0;
                this._height = 0;

                this.X = location.X;
                this.Y = location.Y;
                this.Width = size.Width;
                this.Height = size.Height;
            },
            /**
             * Initializes a Windows.Foundation.Rect structure that
             has the specified x-coordinate, y-coordinate, width, and height.
             *
             * @instance
             * @public
             * @this Windows.Foundation.Rect
             * @memberof Windows.Foundation.Rect
             * @param   {number}    x         The x-coordinate of the top-left corner of the rectangle.
             * @param   {number}    y         The y-coordinate of the top-left corner of the rectangle.
             * @param   {number}    width     The width of the rectangle.
             * @param   {number}    height    The height of the rectangle.
             * @return  {void}
             */
            $ctor1: function (x, y, width, height) {
                this.$initialize();
                this._x = 0;
                this._y = 0;
                this._width = 0;
                this._height = 0;

                this.X = x;
                this.Y = y;
                this.Width = width;
                this.Height = height;
            },
            /**
             * Initializes a new instance of the System.Windows.Rect structure that is of the
             specified size and is located at (0,0).
             *
             * @instance
             * @public
             * @this Windows.Foundation.Rect
             * @memberof Windows.Foundation.Rect
             * @param   {Windows.Foundation.Size}    size    The size of the new Windows.Foundation.Rect.
             * @return  {void}
             */
            $ctor4: function (size) {
                this.$initialize();
                this._x = 0;
                this._y = 0;
                this._width = 0;
                this._height = 0;

                this.Width = size.Width;
                this.Height = size.Height;
            },
            ctor: function () {
                this.$initialize();
            }
        },
        methods: {
            /**
             * Indicates whether the rectangle described by the Windows.Foundation.Rect
             contains the specified point.
             *
             * @instance
             * @public
             * @this Windows.Foundation.Rect
             * @memberof Windows.Foundation.Rect
             * @param   {Windows.Foundation.Point}    point    The point to check.
             * @return  {boolean}                              true if the rectangle described by the Windows.Foundation.Rect contains the
             specified point; otherwise, false.
             */
            Contains: function (point) {
                var ret = this.X <= point.X && this.X + this.Width >= point.X;
                ret = ret && this.Y <= point.Y && this.Y + this.Height >= point.Y;
                return ret;
            },
            /**
             * Indicates whether the specified object is equal to the
             current Windows.Foundation.Rect.
             *
             * @instance
             * @public
             * @override
             * @this Windows.Foundation.Rect
             * @memberof Windows.Foundation.Rect
             * @param   {System.Object}    o    The object to compare to the current rectangle.
             * @return  {boolean}               true if o is a Windows.Foundation.Rect and has the same x,y,width,height
             values as the current Windows.Foundation.Rect; otherwise, false.
             */
            equals: function (o) {
                if (Bridge.is(o, Windows.Foundation.Rect)) {
                    return Windows.Foundation.Rect.op_Equality(System.Nullable.getValue(Bridge.cast(Bridge.unbox(o, Windows.Foundation.Rect), Windows.Foundation.Rect)), this);
                } else {
                    return false;
                }
            },
            /**
             * Indicates whether the specified Windows.Foundation.Rect
             is equal to the current Windows.Foundation.Rect.
             *
             * @instance
             * @public
             * @this Windows.Foundation.Rect
             * @memberof Windows.Foundation.Rect
             * @param   {Windows.Foundation.Rect}    value    The rectangle to compare to the current rectangle.
             * @return  {boolean}                             true if the specified Windows.Foundation.Rect has the same x,y,width,height
             property values as the current Windows.Foundation.Rect; otherwise, false.
             */
            Equals: function (value) {
                return Windows.Foundation.Rect.op_Equality(value.$clone(), this);
            },
            /**
             * Creates a hash code for the Windows.Foundation.Rect.
             *
             * @instance
             * @public
             * @override
             * @this Windows.Foundation.Rect
             * @memberof Windows.Foundation.Rect
             * @return  {number}        A hash code for the current Windows.Foundation.Rect structure.
             */
            getHashCode: function () {
                return System.Double.getHashCode(this.X) ^ System.Double.getHashCode(this.Y) ^ System.Double.getHashCode(this.Width) ^ System.Double.getHashCode(this.Height);
            },
            /**
             * Returns a string representation of the Windows.Foundation.Rect
             structure.
             *
             * @instance
             * @public
             * @override
             * @this Windows.Foundation.Rect
             * @memberof Windows.Foundation.Rect
             * @return  {string}        A string representation of the current Windows.Foundation.Rect structure.
             The string has the following form: "Windows.Foundation.Rect.X,Windows.Foundation.Rect.Y,Windows.Foundation.Rect.Width,Windows.Foundation.Rect.Height".
             */
            toString: function () {
                return System.Double.format(this.X) + "," + System.Double.format(this.Y) + "," + System.Double.format(this.Width) + "," + System.Double.format(this.Height);
            },
            /**
             * Expands the rectangle represented by the current Windows.Foundation.Rect
             exactly enough to contain the specified point.
             *
             * @instance
             * @public
             * @this Windows.Foundation.Rect
             * @memberof Windows.Foundation.Rect
             * @param   {Windows.Foundation.Point}    point    The point to include.
             * @return  {void}
             */
            Union: function (point) {
                if (point.X < this.X) {
                    this.Width = this.Width + this.X - point.X;
                    this.X = point.X;
                } else if (point.X > this.X + this.Width) {
                    this.Width = this.Width + point.X - this.X;
                }

                if (point.Y < this.Y) {
                    this.Height = this.Height + this.Y - point.Y;
                    this.Y = point.Y;
                } else if (point.Y > this.Y + this.Height) {
                    this.Height = this.Height + point.Y - this.Y;
                }
            },
            /**
             * Expands the rectangle represented by the current Windows.Foundation.Rect
             exactly enough to contain the specified rectangle.
             *
             * @instance
             * @public
             * @this Windows.Foundation.Rect
             * @memberof Windows.Foundation.Rect
             * @param   {Windows.Foundation.Rect}    rect    The rectangle to include.
             * @return  {void}
             */
            Union$1: function (rect) {
                if (rect.X < this.X) {
                    this.Width = this.Width + this.X - rect.X;
                    this.X = rect.X;
                } else if (rect.X + rect.Width > this.X + this.Width) {
                    this.Width = this.Width + rect.X + rect.Width - this.X;
                }

                if (rect.Y < this.Y) {
                    this.Height = this.Height + this.Y - rect.Y;
                    this.Y = rect.Y;
                } else if (rect.Y + rect.Height > this.Y + this.Height) {
                    this.Height = this.Height + rect.Y + rect.Height - this.Y;
                }
            },
            $clone: function (to) {
                var s = to || new Windows.Foundation.Rect();
                s._x = this._x;
                s._y = this._y;
                s._width = this._width;
                s._height = this._height;
                return s;
            }
        }
    });

    /**
     * Describes the width and height of an object.
     *
     * @public
     * @class Windows.Foundation.Size
     */
    Bridge.define("Windows.Foundation.Size", {
        $kind: "struct",
        statics: {
            fields: {
                emptySize: null
            },
            props: {
                /**
                 * Gets a value that represents a static empty Windows.Foundation.Size.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.Foundation.Size
                 * @function Empty
                 * @type Windows.Foundation.Size
                 */
                Empty: {
                    get: function () {
                        return Windows.Foundation.Size.emptySize.$clone();
                    }
                }
            },
            ctors: {
                init: function () {
                    this.emptySize = new Windows.Foundation.Size();
                },
                ctor: function () {
                    var $t;
                    Windows.Foundation.Size.emptySize = ($t = new Windows.Foundation.Size.ctor(), $t._width = Number.NEGATIVE_INFINITY, $t._height = Number.NEGATIVE_INFINITY, $t);
                    DotNetForHtml5.Core.TypeFromStringConverters.RegisterConverter(Windows.Foundation.Size, function (s) {
                        return Windows.Foundation.Size.Parse(s).$clone();
                    });
                }
            },
            methods: {
                Equals: function (size1, size2) {
                    if (size1.IsEmpty) {
                        return size2.IsEmpty;
                    } else {
                        return size1.Width === size2.Width && size1.Height === size2.Height;
                    }
                },
                Parse: function (sizeAsString) {
                    var splittedString = System.String.split(sizeAsString, System.Array.init([44, 32], System.Char).map(function (i) {{ return String.fromCharCode(i); }}), null, 1);

                    if (splittedString.length === 2) {
                        var width = { }, height = { };
                        if (System.Double.tryParse(splittedString[System.Array.index(0, splittedString)], null, width) && System.Double.tryParse(splittedString[System.Array.index(1, splittedString)], null, height)) {
                            return new Windows.Foundation.Size.$ctor1(width.v, height.v);
                        }
                    }

                    throw new System.FormatException.$ctor1((sizeAsString || "") + " is not an eligible value for a Size");
                }/**
                 * Compares two instances of Windows.Foundation.Size for
                 inequality.
                 *
                 * @static
                 * @public
                 * @this Windows.Foundation.Size
                 * @memberof Windows.Foundation.Size
                 * @param   {Windows.Foundation.Size}    size1    The first instance of Windows.Foundation.Size to compare.
                 * @param   {Windows.Foundation.Size}    size2    The second instance of Windows.Foundation.Size to compare.
                 * @return  {boolean}                             true if the instances of Windows.Foundation.Size are not equal; otherwise
                 false.
                 */
                ,
                op_Inequality: function (size1, size2) {
                    return size1.Height !== size2.Height || size1.Width !== size2.Width;
                }/**
                 * Compares two instances of Windows.Foundation.Size for
                 equality.
                 *
                 * @static
                 * @public
                 * @this Windows.Foundation.Size
                 * @memberof Windows.Foundation.Size
                 * @param   {Windows.Foundation.Size}    size1    The first instance of Windows.Foundation.Size to compare.
                 * @param   {Windows.Foundation.Size}    size2    The second instance of Windows.Foundation.Size to compare.
                 * @return  {boolean}                             true if the two instances of Windows.Foundation.Size are equal; otherwise
                 false.
                 */
                ,
                op_Equality: function (size1, size2) {
                    return size1.Height === size2.Height && size1.Width === size2.Width;
                },
                getDefaultValue: function () { return new Windows.Foundation.Size(); }
            }
        },
        fields: {
            _width: 0,
            _height: 0
        },
        props: {
            /**
             * Gets or sets the height of this instance of Windows.Foundation.Size in pixels. The default is 0. The value cannot be negative.
             *
             * @instance
             * @public
             * @memberof Windows.Foundation.Size
             * @function Height
             * @type number
             */
            Height: {
                get: function () {
                    return this._height;
                },
                set: function (value) {
                    if (this.IsEmpty) {
                        throw new System.InvalidOperationException.$ctor1("Cannot modify Empty size.");
                    }
                    if (value < 0) {
                        throw new System.ArgumentException.$ctor1("Height cannot be negative.");
                    }
                    this._height = value;
                }
            },
            /**
             * Gets a value that indicates whether this instance of
             Windows.Foundation.Size is Windows.Foundation.Size.Empty.
             *
             * @instance
             * @public
             * @readonly
             * @memberof Windows.Foundation.Size
             * @function IsEmpty
             * @type boolean
             */
            IsEmpty: {
                get: function () {
                    return this._width < 0;
                }
            },
            /**
             * Gets or sets the width of this instance of Windows.Foundation.Size.
             *
             * @instance
             * @public
             * @memberof Windows.Foundation.Size
             * @function Width
             * @type number
             */
            Width: {
                get: function () {
                    return this._width;
                },
                set: function (value) {
                    if (this.IsEmpty) {
                        throw new System.InvalidOperationException.$ctor1("Cannot modify Empty size.");
                    }
                    if (value < 0) {
                        throw new System.ArgumentException.$ctor1("Width cannot be negative.");
                    }
                    this._width = value;
                }
            }
        },
        ctors: {
            /**
             * Initializes a new instance of the Windows.Foundation.Size
             structure and assigns it an initial width and height.
             *
             * @instance
             * @public
             * @this Windows.Foundation.Size
             * @memberof Windows.Foundation.Size
             * @param   {number}    width     The initial width of the instance of Windows.Foundation.Size.
             * @param   {number}    height    The initial height of the instance of Windows.Foundation.Size.
             * @return  {void}
             */
            $ctor1: function (width, height) {
                this.$initialize();
                if (width < 0 || height < 0) {
                    throw new System.ArgumentException.$ctor1("Width and Height cannot be negative.");
                }
                this._width = width;
                this._height = height;
            },
            ctor: function () {
                this.$initialize();
            }
        },
        methods: {
            /**
             * Compares an object to an instance of Windows.Foundation.Size
             for equality.
             *
             * @instance
             * @public
             * @override
             * @this Windows.Foundation.Size
             * @memberof Windows.Foundation.Size
             * @param   {System.Object}    o    The System.Object to compare.
             * @return  {boolean}               true if the sizes are equal; otherwise, false.
             */
            equals: function (o) {
                var value = new Windows.Foundation.Size();
                if (System.Nullable.liftne(Windows.Foundation.Size.op_Inequality, ((value = Bridge.is(o, Windows.Foundation.Size) ? System.Nullable.getValue(Bridge.cast(Bridge.unbox(o, Windows.Foundation.Size), Windows.Foundation.Size)) : null)), null)) {
                    return Windows.Foundation.Size.Equals(this, value.$clone());
                }
                return false;
            },
            /**
             * Compares a value to an instance of Windows.Foundation.Size
             for equality.
             *
             * @instance
             * @public
             * @this Windows.Foundation.Size
             * @memberof Windows.Foundation.Size
             * @param   {Windows.Foundation.Size}    value    The size to compare to this current instance of Windows.Foundation.Size.
             * @return  {boolean}                             true if the instances of Windows.Foundation.Size are equal; otherwise, false.
             */
            Equals: function (value) {
                return Windows.Foundation.Size.Equals(this, value.$clone());
            },
            /**
             * Gets the hash code for this instance of Windows.Foundation.Size.
             *
             * @instance
             * @public
             * @override
             * @this Windows.Foundation.Size
             * @memberof Windows.Foundation.Size
             * @return  {number}        The hash code for this instance of Windows.Foundation.Size.
             */
            getHashCode: function () {
                if (this.IsEmpty) {
                    return 0;
                } else {
                    return System.Double.getHashCode(this.Width) ^ System.Double.getHashCode(this.Height);
                }
            },
            /**
             * Returns a string representation of this Windows.Foundation.Size.
             *
             * @instance
             * @public
             * @override
             * @this Windows.Foundation.Size
             * @memberof Windows.Foundation.Size
             * @return  {string}        A string representation of this Windows.Foundation.Size.
             */
            toString: function () {
                if (this.IsEmpty) {
                    return "Empty";
                }
                return System.Double.format(this.Width) + "," + System.Double.format(this.Height);
            },
            $clone: function (to) {
                var s = to || new Windows.Foundation.Size();
                s._width = this._width;
                s._height = this._height;
                return s;
            }
        }
    });

    Bridge.define("Windows.System.INTERNAL_VirtualKeysHelpers", {
        statics: {
            methods: {
                IsUnknownKey: function (intValue) {
                    if (intValue < 0 || intValue > 165) {
                        return true;
                    }
                    var unknownKeys = function (_o1) {
                            _o1.add(7);
                            _o1.add(10);
                            _o1.add(11);
                            _o1.add(14);
                            _o1.add(15);
                            _o1.add(22);
                            _o1.add(26);
                            _o1.add(58);
                            _o1.add(59);
                            _o1.add(60);
                            _o1.add(61);
                            _o1.add(62);
                            _o1.add(63);
                            _o1.add(64);
                            _o1.add(94);
                            _o1.add(136);
                            _o1.add(137);
                            _o1.add(138);
                            _o1.add(139);
                            _o1.add(140);
                            _o1.add(141);
                            _o1.add(142);
                            _o1.add(143);
                            _o1.add(146);
                            _o1.add(147);
                            _o1.add(148);
                            _o1.add(149);
                            _o1.add(150);
                            _o1.add(151);
                            _o1.add(152);
                            _o1.add(153);
                            _o1.add(154);
                            _o1.add(155);
                            _o1.add(156);
                            _o1.add(157);
                            _o1.add(158);
                            _o1.add(159);
                            return _o1;
                        }(new (System.Collections.Generic.List$1(System.Int32)).ctor());
                    if (unknownKeys.contains(intValue)) {
                        return true;
                    }
                    return false;
                }
            }
        }
    });

    /** @namespace Windows.System */

    /**
     * Specifies the values for each virtual key.
     *
     * @public
     * @class Windows.System.VirtualKey
     * @augments number
     */
    Bridge.define("Windows.System.VirtualKey", {
        $kind: "enum",
        statics: {
            fields: {
                /**
                 * No virtual key value.
                 *
                 * @static
                 * @public
                 * @memberof Windows.System.VirtualKey
                 * @constant
                 * @default 0
                 * @type Windows.System.VirtualKey
                 */
                None: 0,
                /**
                 * The left mouse button.
                 *
                 * @static
                 * @public
                 * @memberof Windows.System.VirtualKey
                 * @constant
                 * @default 1
                 * @type Windows.System.VirtualKey
                 */
                LeftButton: 1,
                /**
                 * The right mouse button.
                 *
                 * @static
                 * @public
                 * @memberof Windows.System.VirtualKey
                 * @constant
                 * @default 2
                 * @type Windows.System.VirtualKey
                 */
                RightButton: 2,
                Cancel: 3,
                MiddleButton: 4,
                XButton1: 5,
                XButton2: 6,
                Back: 8,
                Tab: 9,
                Clear: 12,
                Enter: 13,
                Shift: 16,
                Control: 17,
                Menu: 18,
                Pause: 19,
                CapitalLock: 20,
                Kana: 21,
                Hangul: 21,
                Junja: 23,
                Final: 24,
                Kanji: 25,
                Hanja: 25,
                Escape: 27,
                Convert: 28,
                NonConvert: 29,
                Accept: 30,
                ModeChange: 31,
                Space: 32,
                PageUp: 33,
                PageDown: 34,
                End: 35,
                Home: 36,
                Left: 37,
                Up: 38,
                Right: 39,
                Down: 40,
                Select: 41,
                Print: 42,
                Execute: 43,
                Snapshot: 44,
                Insert: 45,
                Delete: 46,
                Help: 47,
                Number0: 48,
                Number1: 49,
                Number2: 50,
                Number3: 51,
                Number4: 52,
                Number5: 53,
                Number6: 54,
                Number7: 55,
                Number8: 56,
                Number9: 57,
                A: 65,
                B: 66,
                C: 67,
                D: 68,
                E: 69,
                F: 70,
                G: 71,
                H: 72,
                I: 73,
                J: 74,
                K: 75,
                L: 76,
                M: 77,
                N: 78,
                O: 79,
                P: 80,
                Q: 81,
                R: 82,
                S: 83,
                T: 84,
                U: 85,
                V: 86,
                W: 87,
                X: 88,
                Y: 89,
                Z: 90,
                LeftWindows: 91,
                RightWindows: 92,
                Application: 93,
                Sleep: 95,
                NumberPad0: 96,
                NumberPad1: 97,
                NumberPad2: 98,
                NumberPad3: 99,
                NumberPad4: 100,
                NumberPad5: 101,
                NumberPad6: 102,
                NumberPad7: 103,
                NumberPad8: 104,
                NumberPad9: 105,
                Multiply: 106,
                Add: 107,
                Separator: 108,
                Subtract: 109,
                Decimal: 110,
                Divide: 111,
                F1: 112,
                F2: 113,
                F3: 114,
                F4: 115,
                F5: 116,
                F6: 117,
                F7: 118,
                F8: 119,
                F9: 120,
                F10: 121,
                F11: 122,
                F12: 123,
                F13: 124,
                F14: 125,
                F15: 126,
                F16: 127,
                F17: 128,
                F18: 129,
                F19: 130,
                F20: 131,
                F21: 132,
                F22: 133,
                F23: 134,
                F24: 135,
                NumberKeyLock: 144,
                Scroll: 145,
                LeftShift: 160,
                RightShift: 161,
                LeftControl: 162,
                RightControl: 163,
                LeftMenu: 164,
                RightMenu: 165,
                Unknown: 0
            },
            ctors: {
                init: function () {
                    this.Unknown = 2147483647;
                }
            }
        }
    });

    /**
     * Specifies the virtual key used to modify another keypress.
     *
     * @public
     * @class Windows.System.VirtualKeyModifiers
     */
    Bridge.define("Windows.System.VirtualKeyModifiers", {
        $kind: "enum",
        statics: {
            fields: {
                /**
                 * No virtual key modifier.
                 *
                 * @static
                 * @public
                 * @memberof Windows.System.VirtualKeyModifiers
                 * @constant
                 * @default 0
                 * @type Windows.System.VirtualKeyModifiers
                 */
                None: 0,
                /**
                 * The Ctrl (control) virtual key.
                 *
                 * @static
                 * @public
                 * @memberof Windows.System.VirtualKeyModifiers
                 * @constant
                 * @default 1
                 * @type Windows.System.VirtualKeyModifiers
                 */
                Control: 1,
                /**
                 * The Menu (Alt) virtual key.
                 *
                 * @static
                 * @public
                 * @memberof Windows.System.VirtualKeyModifiers
                 * @constant
                 * @default 2
                 * @type Windows.System.VirtualKeyModifiers
                 */
                Menu: 2,
                /**
                 * The Shift virtual key.
                 *
                 * @static
                 * @public
                 * @memberof Windows.System.VirtualKeyModifiers
                 * @constant
                 * @default 4
                 * @type Windows.System.VirtualKeyModifiers
                 */
                Shift: 4,
                /**
                 * The Windows virtual key.
                 *
                 * @static
                 * @public
                 * @memberof Windows.System.VirtualKeyModifiers
                 * @constant
                 * @default 8
                 * @type Windows.System.VirtualKeyModifiers
                 */
                Windows: 8,
                /**
                 * The Apple key (also known as the Open Apple key) is pressed.
                 *
                 * @static
                 * @public
                 * @memberof Windows.System.VirtualKeyModifiers
                 * @constant
                 * @default 8
                 * @type Windows.System.VirtualKeyModifiers
                 */
                Apple: 8
            }
        },
        $flags: true
    });

    /** @namespace Windows.UI */

    /**
     * Describes a color in terms of alpha, red, green, and blue channels.
     *
     * @public
     * @class Windows.UI.Color
     */
    Bridge.define("Windows.UI.Color", {
        $kind: "struct",
        statics: {
            ctors: {
                ctor: function () {
                    DotNetForHtml5.Core.TypeFromStringConverters.RegisterConverter(Windows.UI.Color, Windows.UI.Color.INTERNAL_ConvertFromString);
                }
            },
            methods: {
                /**
                 * Creates a new Windows.UI.Color structure by using the specified sRGB alpha
                 channel and color channel values.
                 *
                 * @static
                 * @public
                 * @this Windows.UI.Color
                 * @memberof Windows.UI.Color
                 * @param   {number}              a    The alpha channel, Windows.UI.Color.A, of the new color. The value must be
                 between 0 and 255.
                 * @param   {number}              r    The red channel, Windows.UI.Color.R, of the new color. The value must be
                 between 0 and 255.
                 * @param   {number}              g    The green channel, Windows.UI.Color.G, of the new color. The value must be
                 between 0 and 255.
                 * @param   {number}              b    The blue channel, Windows.UI.Color.B, of the new color. The value must be
                 between 0 and 255.
                 * @return  {Windows.UI.Color}         A Windows.UI.Color structure with the specified values.
                 */
                FromArgb: function (a, r, g, b) {
                    var $t;
                    return ($t = new Windows.UI.Color(), $t.A = a, $t.R = r, $t.G = g, $t.B = b, $t);
                },
                INTERNAL_ConvertFromString: function (colorcode) {
                    try {
                        if (!System.String.startsWith(colorcode, "#")) {
                            var namedColor = System.Nullable.getValue(Bridge.cast(Bridge.unbox(System.Enum.parse(Windows.UI.Colors.INTERNAL_ColorsEnum, colorcode, true), Windows.UI.Colors.INTERNAL_ColorsEnum), System.Int32));
                            return Windows.UI.Color.INTERNAL_ConvertFromInt32(namedColor).$clone();
                        } else {
                            colorcode = System.String.replaceAll(colorcode, "#", "");
                            if (colorcode.length === 6) {
                                colorcode = "FF" + (colorcode || "");
                            }
                            var colorAsInt;
                            if (CSHTML5.Interop.IsRunningInTheSimulator) {
                                colorcode = System.String.replaceAll(colorcode, "#", "");
                                colorAsInt = INTERNAL_BridgeWorkarounds.HexToInt_SimulatorOnly(colorcode);
                            } else {
                                colorAsInt = parseInt(System.String.replaceAll(colorcode, "#", ""), 16);
                            }
                            return Windows.UI.Color.INTERNAL_ConvertFromInt32(colorAsInt).$clone();
                        }
                    } catch (ex) {
                        ex = System.Exception.create(ex);
                        throw new System.Exception("Invalid color: " + (colorcode || ""), ex);
                    }
                },
                INTERNAL_ConvertFromInt32: function (colorAsInt32) {
                    var $t;
                    return ($t = new Windows.UI.Color(), $t.A = (((colorAsInt32 >> 24) & 255) & 255), $t.R = (((colorAsInt32 >> 16) & 255) & 255), $t.G = (((colorAsInt32 >> 8) & 255) & 255), $t.B = ((colorAsInt32 & 255) & 255), $t);
                },
                op_Explicit: function (color) {
                    /* ******************************************************
                    This method is here to support the following use:

                           <ObjectAnimationUsingKeyFrames Duration="0:0:0.3" Storyboard.TargetProperty="Fill" Storyboard.TargetName="path">
                               <ObjectAnimationUsingKeyFrames.KeyFrames>
                                   <DiscreteObjectKeyFrame KeyTime="0" Value="{StaticResource ThemeHighlightedColor}"/>
                               </ObjectAnimationUsingKeyFrames.KeyFrames>
                           </ObjectAnimationUsingKeyFrames>

                    If we didn't have this method, we would get an invalid cast exception at the "Getter" of the following property in the class "Shape":

                           public Brush Fill
                           {
                               get { return (Brush)GetValue(FillProperty); }
                               set { SetValue(FillProperty, value); }
                           }
                    *******************************************************/

                    return new Windows.UI.Xaml.Media.SolidColorBrush.$ctor1(color.$clone());
                },
                op_Equality: function (color1, color2) {
                    if (color1.R !== color2.R) {
                        return false;
                    }

                    if (color1.G !== color2.G) {
                        return false;
                    }

                    if (color1.B !== color2.B) {
                        return false;
                    }

                    if (color1.A !== color2.A) {
                        return false;
                    }

                    return true;
                },
                op_Inequality: function (color1, color2) {
                    return (!(Windows.UI.Color.op_Equality(color1.$clone(), color2.$clone())));
                },
                getDefaultValue: function () { return new Windows.UI.Color(); }
            }
        },
        fields: {
            /**
             * Gets or sets the sRGB alpha channel value of the color.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Color
             * @function A
             * @type number
             */
            A: 0,
            /**
             * Gets or sets the sRGB blue channel value of the color.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Color
             * @function B
             * @type number
             */
            B: 0,
            /**
             * Gets or sets the sRGB green channel value of the color.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Color
             * @function G
             * @type number
             */
            G: 0,
            /**
             * Gets or sets the sRGB red channel value of the color.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Color
             * @function R
             * @type number
             */
            R: 0
        },
        ctors: {
            ctor: function () {
                this.$initialize();
            }
        },
        methods: {
            INTERNAL_ToHtmlString: function (opacity) {
                return "rgba(" + (Bridge.toString(this.R) || "") + ", " + (Bridge.toString(this.G) || "") + ", " + (Bridge.toString(this.B) || "") + ", " + (System.String.replaceAll(System.Double.format(((this.A / 255) * opacity)), String.fromCharCode(44), String.fromCharCode(46)) || "") + ")";
            },
            INTERNAL_ToHtmlStringForVelocity: function () {
                return "#" + (System.Byte.format(this.R, "X2") || "") + (System.Byte.format(this.G, "X2") || "") + (System.Byte.format(this.B, "X2") || "");

            },
            toString: function () {
                return "#" + (System.Byte.format(this.A, "X2") || "") + (System.Byte.format(this.R, "X2") || "") + (System.Byte.format(this.G, "X2") || "") + (System.Byte.format(this.B, "X2") || "");
            },
            getHashCode: function () {
                return System.BitConverter.toInt32(System.Array.init([this.A, this.R, this.G, this.B], System.Byte), 0);
            },
            Equals: function (color) {
                return Windows.UI.Color.op_Equality(this, color.$clone());
            },
            equals: function (o) {
                if (Bridge.is(o, Windows.UI.Color)) {
                    var color = System.Nullable.getValue(Bridge.cast(Bridge.unbox(o, Windows.UI.Color), Windows.UI.Color));

                    return (Windows.UI.Color.op_Equality(this, color.$clone()));
                } else {
                    return false;
                }
            },
            $clone: function (to) {
                var s = to || new Windows.UI.Color();
                s.A = this.A;
                s.B = this.B;
                s.G = this.G;
                s.R = this.R;
                return s;
            }
        }
    });

    /**
     * Implements a set of predefined colors. See Color for usage information.
     *
     * @static
     * @abstract
     * @public
     * @class Windows.UI.Colors
     */
    Bridge.define("Windows.UI.Colors", {
        statics: {
            props: {
                /**
                 * Gets the color value that represents the AliceBlue named color.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Colors
                 * @function AliceBlue
                 * @type Windows.UI.Color
                 */
                AliceBlue: {
                    get: function () {
                        return Windows.UI.Color.INTERNAL_ConvertFromInt32(Windows.UI.Colors.INTERNAL_ColorsEnum.AliceBlue);
                    }
                },
                /**
                 * Gets the color value that represents the AntiqueWhite named color.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Colors
                 * @function AntiqueWhite
                 * @type Windows.UI.Color
                 */
                AntiqueWhite: {
                    get: function () {
                        return Windows.UI.Color.INTERNAL_ConvertFromInt32(Windows.UI.Colors.INTERNAL_ColorsEnum.AntiqueWhite);
                    }
                },
                /**
                 * Gets the color value that represents the Aqua named color.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Colors
                 * @function Aqua
                 * @type Windows.UI.Color
                 */
                Aqua: {
                    get: function () {
                        return Windows.UI.Color.INTERNAL_ConvertFromInt32(Windows.UI.Colors.INTERNAL_ColorsEnum.Aqua);
                    }
                },
                /**
                 * Gets the color value that represents the Aquamarine named color.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Colors
                 * @function Aquamarine
                 * @type Windows.UI.Color
                 */
                Aquamarine: {
                    get: function () {
                        return Windows.UI.Color.INTERNAL_ConvertFromInt32(Windows.UI.Colors.INTERNAL_ColorsEnum.Aquamarine);
                    }
                },
                /**
                 * Gets the color value that represents the Azure named color.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Colors
                 * @function Azure
                 * @type Windows.UI.Color
                 */
                Azure: {
                    get: function () {
                        return Windows.UI.Color.INTERNAL_ConvertFromInt32(Windows.UI.Colors.INTERNAL_ColorsEnum.Azure);
                    }
                },
                /**
                 * Gets the color value that represents the Beige named color.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Colors
                 * @function Beige
                 * @type Windows.UI.Color
                 */
                Beige: {
                    get: function () {
                        return Windows.UI.Color.INTERNAL_ConvertFromInt32(Windows.UI.Colors.INTERNAL_ColorsEnum.Beige);
                    }
                },
                /**
                 * Gets the color value that represents the Bisque named color.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Colors
                 * @function Bisque
                 * @type Windows.UI.Color
                 */
                Bisque: {
                    get: function () {
                        return Windows.UI.Color.INTERNAL_ConvertFromInt32(Windows.UI.Colors.INTERNAL_ColorsEnum.Bisque);
                    }
                },
                /**
                 * Gets the color value that represents the Black named color.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Colors
                 * @function Black
                 * @type Windows.UI.Color
                 */
                Black: {
                    get: function () {
                        return Windows.UI.Color.INTERNAL_ConvertFromInt32(Windows.UI.Colors.INTERNAL_ColorsEnum.Black);
                    }
                },
                /**
                 * Gets the color value that represents the BlanchedAlmond named color.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Colors
                 * @function BlanchedAlmond
                 * @type Windows.UI.Color
                 */
                BlanchedAlmond: {
                    get: function () {
                        return Windows.UI.Color.INTERNAL_ConvertFromInt32(Windows.UI.Colors.INTERNAL_ColorsEnum.BlanchedAlmond);
                    }
                },
                /**
                 * Gets the color value that represents the Blue named color.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Colors
                 * @function Blue
                 * @type Windows.UI.Color
                 */
                Blue: {
                    get: function () {
                        return Windows.UI.Color.INTERNAL_ConvertFromInt32(Windows.UI.Colors.INTERNAL_ColorsEnum.Blue);
                    }
                },
                /**
                 * Gets the color value that represents the BlueViolet named color.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Colors
                 * @function BlueViolet
                 * @type Windows.UI.Color
                 */
                BlueViolet: {
                    get: function () {
                        return Windows.UI.Color.INTERNAL_ConvertFromInt32(Windows.UI.Colors.INTERNAL_ColorsEnum.BlueViolet);
                    }
                },
                /**
                 * Gets the color value that represents the Brown named color.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Colors
                 * @function Brown
                 * @type Windows.UI.Color
                 */
                Brown: {
                    get: function () {
                        return Windows.UI.Color.INTERNAL_ConvertFromInt32(Windows.UI.Colors.INTERNAL_ColorsEnum.Brown);
                    }
                },
                /**
                 * Gets the color value that represents the BurlyWood named color.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Colors
                 * @function BurlyWood
                 * @type Windows.UI.Color
                 */
                BurlyWood: {
                    get: function () {
                        return Windows.UI.Color.INTERNAL_ConvertFromInt32(Windows.UI.Colors.INTERNAL_ColorsEnum.BurlyWood);
                    }
                },
                /**
                 * Gets the color value that represents the CadetBlue named color.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Colors
                 * @function CadetBlue
                 * @type Windows.UI.Color
                 */
                CadetBlue: {
                    get: function () {
                        return Windows.UI.Color.INTERNAL_ConvertFromInt32(Windows.UI.Colors.INTERNAL_ColorsEnum.CadetBlue);
                    }
                },
                /**
                 * Gets the color value that represents the Chartreuse named color.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Colors
                 * @function Chartreuse
                 * @type Windows.UI.Color
                 */
                Chartreuse: {
                    get: function () {
                        return Windows.UI.Color.INTERNAL_ConvertFromInt32(Windows.UI.Colors.INTERNAL_ColorsEnum.Chartreuse);
                    }
                },
                /**
                 * Gets the color value that represents the Chocolate named color.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Colors
                 * @function Chocolate
                 * @type Windows.UI.Color
                 */
                Chocolate: {
                    get: function () {
                        return Windows.UI.Color.INTERNAL_ConvertFromInt32(Windows.UI.Colors.INTERNAL_ColorsEnum.Chocolate);
                    }
                },
                /**
                 * Gets the color value that represents the Coral named color.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Colors
                 * @function Coral
                 * @type Windows.UI.Color
                 */
                Coral: {
                    get: function () {
                        return Windows.UI.Color.INTERNAL_ConvertFromInt32(Windows.UI.Colors.INTERNAL_ColorsEnum.Coral);
                    }
                },
                /**
                 * Gets the color value that represents the CornflowerBlue named color.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Colors
                 * @function CornflowerBlue
                 * @type Windows.UI.Color
                 */
                CornflowerBlue: {
                    get: function () {
                        return Windows.UI.Color.INTERNAL_ConvertFromInt32(Windows.UI.Colors.INTERNAL_ColorsEnum.CornflowerBlue);
                    }
                },
                /**
                 * Gets the color value that represents the Cornsilk named color.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Colors
                 * @function Cornsilk
                 * @type Windows.UI.Color
                 */
                Cornsilk: {
                    get: function () {
                        return Windows.UI.Color.INTERNAL_ConvertFromInt32(Windows.UI.Colors.INTERNAL_ColorsEnum.Cornsilk);
                    }
                },
                /**
                 * Gets the color value that represents the Crimson named color.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Colors
                 * @function Crimson
                 * @type Windows.UI.Color
                 */
                Crimson: {
                    get: function () {
                        return Windows.UI.Color.INTERNAL_ConvertFromInt32(Windows.UI.Colors.INTERNAL_ColorsEnum.Crimson);
                    }
                },
                /**
                 * Gets the color value that represents the Cyan named color.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Colors
                 * @function Cyan
                 * @type Windows.UI.Color
                 */
                Cyan: {
                    get: function () {
                        return Windows.UI.Color.INTERNAL_ConvertFromInt32(Windows.UI.Colors.INTERNAL_ColorsEnum.Cyan);
                    }
                },
                /**
                 * Gets the color value that represents the DarkBlue named color.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Colors
                 * @function DarkBlue
                 * @type Windows.UI.Color
                 */
                DarkBlue: {
                    get: function () {
                        return Windows.UI.Color.INTERNAL_ConvertFromInt32(Windows.UI.Colors.INTERNAL_ColorsEnum.DarkBlue);
                    }
                },
                /**
                 * Gets the color value that represents the DarkCyan named color.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Colors
                 * @function DarkCyan
                 * @type Windows.UI.Color
                 */
                DarkCyan: {
                    get: function () {
                        return Windows.UI.Color.INTERNAL_ConvertFromInt32(Windows.UI.Colors.INTERNAL_ColorsEnum.DarkCyan);
                    }
                },
                /**
                 * Gets the color value that represents the DarkGoldenrod named color.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Colors
                 * @function DarkGoldenrod
                 * @type Windows.UI.Color
                 */
                DarkGoldenrod: {
                    get: function () {
                        return Windows.UI.Color.INTERNAL_ConvertFromInt32(Windows.UI.Colors.INTERNAL_ColorsEnum.DarkGoldenrod);
                    }
                },
                /**
                 * Gets the color value that represents the DarkGray named color.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Colors
                 * @function DarkGray
                 * @type Windows.UI.Color
                 */
                DarkGray: {
                    get: function () {
                        return Windows.UI.Color.INTERNAL_ConvertFromInt32(Windows.UI.Colors.INTERNAL_ColorsEnum.DarkGray);
                    }
                },
                /**
                 * Gets the color value that represents the DarkGreen named color.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Colors
                 * @function DarkGreen
                 * @type Windows.UI.Color
                 */
                DarkGreen: {
                    get: function () {
                        return Windows.UI.Color.INTERNAL_ConvertFromInt32(Windows.UI.Colors.INTERNAL_ColorsEnum.DarkGreen);
                    }
                },
                /**
                 * Gets the color value that represents the DarkKhaki named color.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Colors
                 * @function DarkKhaki
                 * @type Windows.UI.Color
                 */
                DarkKhaki: {
                    get: function () {
                        return Windows.UI.Color.INTERNAL_ConvertFromInt32(Windows.UI.Colors.INTERNAL_ColorsEnum.DarkKhaki);
                    }
                },
                /**
                 * Gets the color value that represents the DarkMagenta named color.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Colors
                 * @function DarkMagenta
                 * @type Windows.UI.Color
                 */
                DarkMagenta: {
                    get: function () {
                        return Windows.UI.Color.INTERNAL_ConvertFromInt32(Windows.UI.Colors.INTERNAL_ColorsEnum.DarkMagenta);
                    }
                },
                /**
                 * Gets the color value that represents the DarkOliveGreen named color.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Colors
                 * @function DarkOliveGreen
                 * @type Windows.UI.Color
                 */
                DarkOliveGreen: {
                    get: function () {
                        return Windows.UI.Color.INTERNAL_ConvertFromInt32(Windows.UI.Colors.INTERNAL_ColorsEnum.DarkOliveGreen);
                    }
                },
                /**
                 * Gets the color value that represents the DarkOrange named color.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Colors
                 * @function DarkOrange
                 * @type Windows.UI.Color
                 */
                DarkOrange: {
                    get: function () {
                        return Windows.UI.Color.INTERNAL_ConvertFromInt32(Windows.UI.Colors.INTERNAL_ColorsEnum.DarkOrange);
                    }
                },
                /**
                 * Gets the color value that represents the DarkOrchid named color.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Colors
                 * @function DarkOrchid
                 * @type Windows.UI.Color
                 */
                DarkOrchid: {
                    get: function () {
                        return Windows.UI.Color.INTERNAL_ConvertFromInt32(Windows.UI.Colors.INTERNAL_ColorsEnum.DarkOrchid);
                    }
                },
                /**
                 * Gets the color value that represents the DarkRed named color.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Colors
                 * @function DarkRed
                 * @type Windows.UI.Color
                 */
                DarkRed: {
                    get: function () {
                        return Windows.UI.Color.INTERNAL_ConvertFromInt32(Windows.UI.Colors.INTERNAL_ColorsEnum.DarkRed);
                    }
                },
                /**
                 * Gets the color value that represents the DarkSalmon named color.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Colors
                 * @function DarkSalmon
                 * @type Windows.UI.Color
                 */
                DarkSalmon: {
                    get: function () {
                        return Windows.UI.Color.INTERNAL_ConvertFromInt32(Windows.UI.Colors.INTERNAL_ColorsEnum.DarkSalmon);
                    }
                },
                /**
                 * Gets the color value that represents the DarkSeaGreen named color.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Colors
                 * @function DarkSeaGreen
                 * @type Windows.UI.Color
                 */
                DarkSeaGreen: {
                    get: function () {
                        return Windows.UI.Color.INTERNAL_ConvertFromInt32(Windows.UI.Colors.INTERNAL_ColorsEnum.DarkSeaGreen);
                    }
                },
                /**
                 * Gets the color value that represents the DarkSlateBlue named color.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Colors
                 * @function DarkSlateBlue
                 * @type Windows.UI.Color
                 */
                DarkSlateBlue: {
                    get: function () {
                        return Windows.UI.Color.INTERNAL_ConvertFromInt32(Windows.UI.Colors.INTERNAL_ColorsEnum.DarkSlateBlue);
                    }
                },
                /**
                 * Gets the color value that represents the DarkSlateGray named color.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Colors
                 * @function DarkSlateGray
                 * @type Windows.UI.Color
                 */
                DarkSlateGray: {
                    get: function () {
                        return Windows.UI.Color.INTERNAL_ConvertFromInt32(Windows.UI.Colors.INTERNAL_ColorsEnum.DarkSlateGray);
                    }
                },
                /**
                 * Gets the color value that represents the DarkTurquoise named color.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Colors
                 * @function DarkTurquoise
                 * @type Windows.UI.Color
                 */
                DarkTurquoise: {
                    get: function () {
                        return Windows.UI.Color.INTERNAL_ConvertFromInt32(Windows.UI.Colors.INTERNAL_ColorsEnum.DarkTurquoise);
                    }
                },
                /**
                 * Gets the color value that represents the DarkViolet named color.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Colors
                 * @function DarkViolet
                 * @type Windows.UI.Color
                 */
                DarkViolet: {
                    get: function () {
                        return Windows.UI.Color.INTERNAL_ConvertFromInt32(Windows.UI.Colors.INTERNAL_ColorsEnum.DarkViolet);
                    }
                },
                /**
                 * Gets the color value that represents the DeepPink named color.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Colors
                 * @function DeepPink
                 * @type Windows.UI.Color
                 */
                DeepPink: {
                    get: function () {
                        return Windows.UI.Color.INTERNAL_ConvertFromInt32(Windows.UI.Colors.INTERNAL_ColorsEnum.DeepPink);
                    }
                },
                /**
                 * Gets the color value that represents the DeepSkyBlue named color.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Colors
                 * @function DeepSkyBlue
                 * @type Windows.UI.Color
                 */
                DeepSkyBlue: {
                    get: function () {
                        return Windows.UI.Color.INTERNAL_ConvertFromInt32(Windows.UI.Colors.INTERNAL_ColorsEnum.DeepSkyBlue);
                    }
                },
                /**
                 * Gets the color value that represents the DimGray named color.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Colors
                 * @function DimGray
                 * @type Windows.UI.Color
                 */
                DimGray: {
                    get: function () {
                        return Windows.UI.Color.INTERNAL_ConvertFromInt32(Windows.UI.Colors.INTERNAL_ColorsEnum.DimGray);
                    }
                },
                /**
                 * Gets the color value that represents the DodgerBlue named color.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Colors
                 * @function DodgerBlue
                 * @type Windows.UI.Color
                 */
                DodgerBlue: {
                    get: function () {
                        return Windows.UI.Color.INTERNAL_ConvertFromInt32(Windows.UI.Colors.INTERNAL_ColorsEnum.DodgerBlue);
                    }
                },
                /**
                 * Gets the color value that represents the Firebrick named color.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Colors
                 * @function Firebrick
                 * @type Windows.UI.Color
                 */
                Firebrick: {
                    get: function () {
                        return Windows.UI.Color.INTERNAL_ConvertFromInt32(Windows.UI.Colors.INTERNAL_ColorsEnum.Firebrick);
                    }
                },
                /**
                 * Gets the color value that represents the FloralWhite named color.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Colors
                 * @function FloralWhite
                 * @type Windows.UI.Color
                 */
                FloralWhite: {
                    get: function () {
                        return Windows.UI.Color.INTERNAL_ConvertFromInt32(Windows.UI.Colors.INTERNAL_ColorsEnum.FloralWhite);
                    }
                },
                /**
                 * Gets the color value that represents the ForestGreen named color.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Colors
                 * @function ForestGreen
                 * @type Windows.UI.Color
                 */
                ForestGreen: {
                    get: function () {
                        return Windows.UI.Color.INTERNAL_ConvertFromInt32(Windows.UI.Colors.INTERNAL_ColorsEnum.ForestGreen);
                    }
                },
                /**
                 * Gets the color value that represents the Fuchsia named color.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Colors
                 * @function Fuchsia
                 * @type Windows.UI.Color
                 */
                Fuchsia: {
                    get: function () {
                        return Windows.UI.Color.INTERNAL_ConvertFromInt32(Windows.UI.Colors.INTERNAL_ColorsEnum.Fuchsia);
                    }
                },
                /**
                 * Gets the color value that represents the Gainsboro named color.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Colors
                 * @function Gainsboro
                 * @type Windows.UI.Color
                 */
                Gainsboro: {
                    get: function () {
                        return Windows.UI.Color.INTERNAL_ConvertFromInt32(Windows.UI.Colors.INTERNAL_ColorsEnum.Gainsboro);
                    }
                },
                /**
                 * Gets the color value that represents the GhostWhite named color.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Colors
                 * @function GhostWhite
                 * @type Windows.UI.Color
                 */
                GhostWhite: {
                    get: function () {
                        return Windows.UI.Color.INTERNAL_ConvertFromInt32(Windows.UI.Colors.INTERNAL_ColorsEnum.GhostWhite);
                    }
                },
                /**
                 * Gets the color value that represents the Gold named color.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Colors
                 * @function Gold
                 * @type Windows.UI.Color
                 */
                Gold: {
                    get: function () {
                        return Windows.UI.Color.INTERNAL_ConvertFromInt32(Windows.UI.Colors.INTERNAL_ColorsEnum.Gold);
                    }
                },
                /**
                 * Gets the color value that represents the Goldenrod named color.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Colors
                 * @function Goldenrod
                 * @type Windows.UI.Color
                 */
                Goldenrod: {
                    get: function () {
                        return Windows.UI.Color.INTERNAL_ConvertFromInt32(Windows.UI.Colors.INTERNAL_ColorsEnum.Goldenrod);
                    }
                },
                /**
                 * Gets the color value that represents the Gray named color.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Colors
                 * @function Gray
                 * @type Windows.UI.Color
                 */
                Gray: {
                    get: function () {
                        return Windows.UI.Color.INTERNAL_ConvertFromInt32(Windows.UI.Colors.INTERNAL_ColorsEnum.Gray);
                    }
                },
                /**
                 * Gets the color value that represents the Green named color.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Colors
                 * @function Green
                 * @type Windows.UI.Color
                 */
                Green: {
                    get: function () {
                        return Windows.UI.Color.INTERNAL_ConvertFromInt32(Windows.UI.Colors.INTERNAL_ColorsEnum.Green);
                    }
                },
                /**
                 * Gets the color value that represents the GreenYellow named color.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Colors
                 * @function GreenYellow
                 * @type Windows.UI.Color
                 */
                GreenYellow: {
                    get: function () {
                        return Windows.UI.Color.INTERNAL_ConvertFromInt32(Windows.UI.Colors.INTERNAL_ColorsEnum.GreenYellow);
                    }
                },
                /**
                 * Gets the color value that represents the Honeydew named color.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Colors
                 * @function Honeydew
                 * @type Windows.UI.Color
                 */
                Honeydew: {
                    get: function () {
                        return Windows.UI.Color.INTERNAL_ConvertFromInt32(Windows.UI.Colors.INTERNAL_ColorsEnum.Honeydew);
                    }
                },
                /**
                 * Gets the color value that represents the HotPink named color.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Colors
                 * @function HotPink
                 * @type Windows.UI.Color
                 */
                HotPink: {
                    get: function () {
                        return Windows.UI.Color.INTERNAL_ConvertFromInt32(Windows.UI.Colors.INTERNAL_ColorsEnum.HotPink);
                    }
                },
                /**
                 * Gets the color value that represents the IndianRed named color.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Colors
                 * @function IndianRed
                 * @type Windows.UI.Color
                 */
                IndianRed: {
                    get: function () {
                        return Windows.UI.Color.INTERNAL_ConvertFromInt32(Windows.UI.Colors.INTERNAL_ColorsEnum.IndianRed);
                    }
                },
                /**
                 * Gets the color value that represents the Indigo named color.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Colors
                 * @function Indigo
                 * @type Windows.UI.Color
                 */
                Indigo: {
                    get: function () {
                        return Windows.UI.Color.INTERNAL_ConvertFromInt32(Windows.UI.Colors.INTERNAL_ColorsEnum.Indigo);
                    }
                },
                /**
                 * Gets the color value that represents the Ivory named color.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Colors
                 * @function Ivory
                 * @type Windows.UI.Color
                 */
                Ivory: {
                    get: function () {
                        return Windows.UI.Color.INTERNAL_ConvertFromInt32(Windows.UI.Colors.INTERNAL_ColorsEnum.Ivory);
                    }
                },
                /**
                 * Gets the color value that represents the Khaki named color.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Colors
                 * @function Khaki
                 * @type Windows.UI.Color
                 */
                Khaki: {
                    get: function () {
                        return Windows.UI.Color.INTERNAL_ConvertFromInt32(Windows.UI.Colors.INTERNAL_ColorsEnum.Khaki);
                    }
                },
                /**
                 * Gets the color value that represents the Lavender named color.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Colors
                 * @function Lavender
                 * @type Windows.UI.Color
                 */
                Lavender: {
                    get: function () {
                        return Windows.UI.Color.INTERNAL_ConvertFromInt32(Windows.UI.Colors.INTERNAL_ColorsEnum.Lavender);
                    }
                },
                /**
                 * Gets the color value that represents the LavenderBlush named color.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Colors
                 * @function LavenderBlush
                 * @type Windows.UI.Color
                 */
                LavenderBlush: {
                    get: function () {
                        return Windows.UI.Color.INTERNAL_ConvertFromInt32(Windows.UI.Colors.INTERNAL_ColorsEnum.LavenderBlush);
                    }
                },
                /**
                 * Gets the color value that represents the LawnGreen named color.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Colors
                 * @function LawnGreen
                 * @type Windows.UI.Color
                 */
                LawnGreen: {
                    get: function () {
                        return Windows.UI.Color.INTERNAL_ConvertFromInt32(Windows.UI.Colors.INTERNAL_ColorsEnum.LawnGreen);
                    }
                },
                /**
                 * Gets the color value that represents the LemonChiffon named color.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Colors
                 * @function LemonChiffon
                 * @type Windows.UI.Color
                 */
                LemonChiffon: {
                    get: function () {
                        return Windows.UI.Color.INTERNAL_ConvertFromInt32(Windows.UI.Colors.INTERNAL_ColorsEnum.LemonChiffon);
                    }
                },
                /**
                 * Gets the color value that represents the LightBlue named color.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Colors
                 * @function LightBlue
                 * @type Windows.UI.Color
                 */
                LightBlue: {
                    get: function () {
                        return Windows.UI.Color.INTERNAL_ConvertFromInt32(Windows.UI.Colors.INTERNAL_ColorsEnum.LightBlue);
                    }
                },
                /**
                 * Gets the color value that represents the LightCoral named color.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Colors
                 * @function LightCoral
                 * @type Windows.UI.Color
                 */
                LightCoral: {
                    get: function () {
                        return Windows.UI.Color.INTERNAL_ConvertFromInt32(Windows.UI.Colors.INTERNAL_ColorsEnum.LightCoral);
                    }
                },
                /**
                 * Gets the color value that represents the LightCyan named color.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Colors
                 * @function LightCyan
                 * @type Windows.UI.Color
                 */
                LightCyan: {
                    get: function () {
                        return Windows.UI.Color.INTERNAL_ConvertFromInt32(Windows.UI.Colors.INTERNAL_ColorsEnum.LightCyan);
                    }
                },
                /**
                 * Gets the color value that represents the LightGoldenrodYellow named color.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Colors
                 * @function LightGoldenrodYellow
                 * @type Windows.UI.Color
                 */
                LightGoldenrodYellow: {
                    get: function () {
                        return Windows.UI.Color.INTERNAL_ConvertFromInt32(Windows.UI.Colors.INTERNAL_ColorsEnum.LightGoldenrodYellow);
                    }
                },
                /**
                 * Gets the color value that represents the LightGray named color.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Colors
                 * @function LightGray
                 * @type Windows.UI.Color
                 */
                LightGray: {
                    get: function () {
                        return Windows.UI.Color.INTERNAL_ConvertFromInt32(Windows.UI.Colors.INTERNAL_ColorsEnum.LightGray);
                    }
                },
                /**
                 * Gets the color value that represents the LightGreen named color.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Colors
                 * @function LightGreen
                 * @type Windows.UI.Color
                 */
                LightGreen: {
                    get: function () {
                        return Windows.UI.Color.INTERNAL_ConvertFromInt32(Windows.UI.Colors.INTERNAL_ColorsEnum.LightGreen);
                    }
                },
                /**
                 * Gets the color value that represents the LightPink named color.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Colors
                 * @function LightPink
                 * @type Windows.UI.Color
                 */
                LightPink: {
                    get: function () {
                        return Windows.UI.Color.INTERNAL_ConvertFromInt32(Windows.UI.Colors.INTERNAL_ColorsEnum.LightPink);
                    }
                },
                /**
                 * Gets the color value that represents the LightSalmon named color.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Colors
                 * @function LightSalmon
                 * @type Windows.UI.Color
                 */
                LightSalmon: {
                    get: function () {
                        return Windows.UI.Color.INTERNAL_ConvertFromInt32(Windows.UI.Colors.INTERNAL_ColorsEnum.LightSalmon);
                    }
                },
                /**
                 * Gets the color value that represents the LightSeaGreen named color.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Colors
                 * @function LightSeaGreen
                 * @type Windows.UI.Color
                 */
                LightSeaGreen: {
                    get: function () {
                        return Windows.UI.Color.INTERNAL_ConvertFromInt32(Windows.UI.Colors.INTERNAL_ColorsEnum.LightSeaGreen);
                    }
                },
                /**
                 * Gets the color value that represents the LightSkyBlue named color.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Colors
                 * @function LightSkyBlue
                 * @type Windows.UI.Color
                 */
                LightSkyBlue: {
                    get: function () {
                        return Windows.UI.Color.INTERNAL_ConvertFromInt32(Windows.UI.Colors.INTERNAL_ColorsEnum.LightSkyBlue);
                    }
                },
                /**
                 * Gets the color value that represents the LightSlateGray named color.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Colors
                 * @function LightSlateGray
                 * @type Windows.UI.Color
                 */
                LightSlateGray: {
                    get: function () {
                        return Windows.UI.Color.INTERNAL_ConvertFromInt32(Windows.UI.Colors.INTERNAL_ColorsEnum.LightSlateGray);
                    }
                },
                /**
                 * Gets the color value that represents the LightSteelBlue named color.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Colors
                 * @function LightSteelBlue
                 * @type Windows.UI.Color
                 */
                LightSteelBlue: {
                    get: function () {
                        return Windows.UI.Color.INTERNAL_ConvertFromInt32(Windows.UI.Colors.INTERNAL_ColorsEnum.LightSteelBlue);
                    }
                },
                /**
                 * Gets the color value that represents the LightYellow named color.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Colors
                 * @function LightYellow
                 * @type Windows.UI.Color
                 */
                LightYellow: {
                    get: function () {
                        return Windows.UI.Color.INTERNAL_ConvertFromInt32(Windows.UI.Colors.INTERNAL_ColorsEnum.LightYellow);
                    }
                },
                /**
                 * Gets the color value that represents the Lime named color.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Colors
                 * @function Lime
                 * @type Windows.UI.Color
                 */
                Lime: {
                    get: function () {
                        return Windows.UI.Color.INTERNAL_ConvertFromInt32(Windows.UI.Colors.INTERNAL_ColorsEnum.Lime);
                    }
                },
                /**
                 * Gets the color value that represents the LimeGreen named color.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Colors
                 * @function LimeGreen
                 * @type Windows.UI.Color
                 */
                LimeGreen: {
                    get: function () {
                        return Windows.UI.Color.INTERNAL_ConvertFromInt32(Windows.UI.Colors.INTERNAL_ColorsEnum.LimeGreen);
                    }
                },
                /**
                 * Gets the color value that represents the Linen named color.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Colors
                 * @function Linen
                 * @type Windows.UI.Color
                 */
                Linen: {
                    get: function () {
                        return Windows.UI.Color.INTERNAL_ConvertFromInt32(Windows.UI.Colors.INTERNAL_ColorsEnum.Linen);
                    }
                },
                /**
                 * Gets the color value that represents the Magenta named color.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Colors
                 * @function Magenta
                 * @type Windows.UI.Color
                 */
                Magenta: {
                    get: function () {
                        return Windows.UI.Color.INTERNAL_ConvertFromInt32(Windows.UI.Colors.INTERNAL_ColorsEnum.Magenta);
                    }
                },
                /**
                 * Gets the color value that represents the Maroon named color.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Colors
                 * @function Maroon
                 * @type Windows.UI.Color
                 */
                Maroon: {
                    get: function () {
                        return Windows.UI.Color.INTERNAL_ConvertFromInt32(Windows.UI.Colors.INTERNAL_ColorsEnum.Maroon);
                    }
                },
                /**
                 * Gets the color value that represents the MediumAquamarine named color.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Colors
                 * @function MediumAquamarine
                 * @type Windows.UI.Color
                 */
                MediumAquamarine: {
                    get: function () {
                        return Windows.UI.Color.INTERNAL_ConvertFromInt32(Windows.UI.Colors.INTERNAL_ColorsEnum.MediumAquamarine);
                    }
                },
                /**
                 * Gets the color value that represents the MediumBlue named color.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Colors
                 * @function MediumBlue
                 * @type Windows.UI.Color
                 */
                MediumBlue: {
                    get: function () {
                        return Windows.UI.Color.INTERNAL_ConvertFromInt32(Windows.UI.Colors.INTERNAL_ColorsEnum.MediumBlue);
                    }
                },
                /**
                 * Gets the color value that represents the MediumOrchid named color.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Colors
                 * @function MediumOrchid
                 * @type Windows.UI.Color
                 */
                MediumOrchid: {
                    get: function () {
                        return Windows.UI.Color.INTERNAL_ConvertFromInt32(Windows.UI.Colors.INTERNAL_ColorsEnum.MediumOrchid);
                    }
                },
                /**
                 * Gets the color value that represents the MediumPurple named color.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Colors
                 * @function MediumPurple
                 * @type Windows.UI.Color
                 */
                MediumPurple: {
                    get: function () {
                        return Windows.UI.Color.INTERNAL_ConvertFromInt32(Windows.UI.Colors.INTERNAL_ColorsEnum.MediumPurple);
                    }
                },
                /**
                 * Gets the color value that represents the MediumSeaGreen named color.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Colors
                 * @function MediumSeaGreen
                 * @type Windows.UI.Color
                 */
                MediumSeaGreen: {
                    get: function () {
                        return Windows.UI.Color.INTERNAL_ConvertFromInt32(Windows.UI.Colors.INTERNAL_ColorsEnum.MediumSeaGreen);
                    }
                },
                /**
                 * Gets the color value that represents the MediumSlateBlue named color.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Colors
                 * @function MediumSlateBlue
                 * @type Windows.UI.Color
                 */
                MediumSlateBlue: {
                    get: function () {
                        return Windows.UI.Color.INTERNAL_ConvertFromInt32(Windows.UI.Colors.INTERNAL_ColorsEnum.MediumSlateBlue);
                    }
                },
                /**
                 * Gets the color value that represents the MediumSpringGreen named color.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Colors
                 * @function MediumSpringGreen
                 * @type Windows.UI.Color
                 */
                MediumSpringGreen: {
                    get: function () {
                        return Windows.UI.Color.INTERNAL_ConvertFromInt32(Windows.UI.Colors.INTERNAL_ColorsEnum.MediumSpringGreen);
                    }
                },
                /**
                 * Gets the color value that represents the MediumTurquoise named color.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Colors
                 * @function MediumTurquoise
                 * @type Windows.UI.Color
                 */
                MediumTurquoise: {
                    get: function () {
                        return Windows.UI.Color.INTERNAL_ConvertFromInt32(Windows.UI.Colors.INTERNAL_ColorsEnum.MediumTurquoise);
                    }
                },
                /**
                 * Gets the color value that represents the MediumVioletRed named color.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Colors
                 * @function MediumVioletRed
                 * @type Windows.UI.Color
                 */
                MediumVioletRed: {
                    get: function () {
                        return Windows.UI.Color.INTERNAL_ConvertFromInt32(Windows.UI.Colors.INTERNAL_ColorsEnum.MediumVioletRed);
                    }
                },
                /**
                 * Gets the color value that represents the MidnightBlue named color.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Colors
                 * @function MidnightBlue
                 * @type Windows.UI.Color
                 */
                MidnightBlue: {
                    get: function () {
                        return Windows.UI.Color.INTERNAL_ConvertFromInt32(Windows.UI.Colors.INTERNAL_ColorsEnum.MidnightBlue);
                    }
                },
                /**
                 * Gets the color value that represents the MintCream named color.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Colors
                 * @function MintCream
                 * @type Windows.UI.Color
                 */
                MintCream: {
                    get: function () {
                        return Windows.UI.Color.INTERNAL_ConvertFromInt32(Windows.UI.Colors.INTERNAL_ColorsEnum.MintCream);
                    }
                },
                /**
                 * Gets the color value that represents the MistyRose named color.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Colors
                 * @function MistyRose
                 * @type Windows.UI.Color
                 */
                MistyRose: {
                    get: function () {
                        return Windows.UI.Color.INTERNAL_ConvertFromInt32(Windows.UI.Colors.INTERNAL_ColorsEnum.MistyRose);
                    }
                },
                /**
                 * Gets the color value that represents the Moccasin named color.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Colors
                 * @function Moccasin
                 * @type Windows.UI.Color
                 */
                Moccasin: {
                    get: function () {
                        return Windows.UI.Color.INTERNAL_ConvertFromInt32(Windows.UI.Colors.INTERNAL_ColorsEnum.Moccasin);
                    }
                },
                /**
                 * Gets the color value that represents the NavajoWhite named color.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Colors
                 * @function NavajoWhite
                 * @type Windows.UI.Color
                 */
                NavajoWhite: {
                    get: function () {
                        return Windows.UI.Color.INTERNAL_ConvertFromInt32(Windows.UI.Colors.INTERNAL_ColorsEnum.NavajoWhite);
                    }
                },
                /**
                 * Gets the color value that represents the Navy named color.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Colors
                 * @function Navy
                 * @type Windows.UI.Color
                 */
                Navy: {
                    get: function () {
                        return Windows.UI.Color.INTERNAL_ConvertFromInt32(Windows.UI.Colors.INTERNAL_ColorsEnum.Navy);
                    }
                },
                /**
                 * Gets the color value that represents the OldLace named color.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Colors
                 * @function OldLace
                 * @type Windows.UI.Color
                 */
                OldLace: {
                    get: function () {
                        return Windows.UI.Color.INTERNAL_ConvertFromInt32(Windows.UI.Colors.INTERNAL_ColorsEnum.OldLace);
                    }
                },
                /**
                 * Gets the color value that represents the Olive named color.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Colors
                 * @function Olive
                 * @type Windows.UI.Color
                 */
                Olive: {
                    get: function () {
                        return Windows.UI.Color.INTERNAL_ConvertFromInt32(Windows.UI.Colors.INTERNAL_ColorsEnum.Olive);
                    }
                },
                /**
                 * Gets the color value that represents the OliveDrab named color.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Colors
                 * @function OliveDrab
                 * @type Windows.UI.Color
                 */
                OliveDrab: {
                    get: function () {
                        return Windows.UI.Color.INTERNAL_ConvertFromInt32(Windows.UI.Colors.INTERNAL_ColorsEnum.OliveDrab);
                    }
                },
                /**
                 * Gets the color value that represents the Orange named color.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Colors
                 * @function Orange
                 * @type Windows.UI.Color
                 */
                Orange: {
                    get: function () {
                        return Windows.UI.Color.INTERNAL_ConvertFromInt32(Windows.UI.Colors.INTERNAL_ColorsEnum.Orange);
                    }
                },
                /**
                 * Gets the color value that represents the OrangeRed named color.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Colors
                 * @function OrangeRed
                 * @type Windows.UI.Color
                 */
                OrangeRed: {
                    get: function () {
                        return Windows.UI.Color.INTERNAL_ConvertFromInt32(Windows.UI.Colors.INTERNAL_ColorsEnum.OrangeRed);
                    }
                },
                /**
                 * Gets the color value that represents the Orchid named color.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Colors
                 * @function Orchid
                 * @type Windows.UI.Color
                 */
                Orchid: {
                    get: function () {
                        return Windows.UI.Color.INTERNAL_ConvertFromInt32(Windows.UI.Colors.INTERNAL_ColorsEnum.Orchid);
                    }
                },
                /**
                 * Gets the color value that represents the PaleGoldenrod named color.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Colors
                 * @function PaleGoldenrod
                 * @type Windows.UI.Color
                 */
                PaleGoldenrod: {
                    get: function () {
                        return Windows.UI.Color.INTERNAL_ConvertFromInt32(Windows.UI.Colors.INTERNAL_ColorsEnum.PaleGoldenrod);
                    }
                },
                /**
                 * Gets the color value that represents the PaleGreen named color.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Colors
                 * @function PaleGreen
                 * @type Windows.UI.Color
                 */
                PaleGreen: {
                    get: function () {
                        return Windows.UI.Color.INTERNAL_ConvertFromInt32(Windows.UI.Colors.INTERNAL_ColorsEnum.PaleGreen);
                    }
                },
                /**
                 * Gets the color value that represents the PaleTurquoise named color.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Colors
                 * @function PaleTurquoise
                 * @type Windows.UI.Color
                 */
                PaleTurquoise: {
                    get: function () {
                        return Windows.UI.Color.INTERNAL_ConvertFromInt32(Windows.UI.Colors.INTERNAL_ColorsEnum.PaleTurquoise);
                    }
                },
                /**
                 * Gets the color value that represents the PaleVioletRed named color.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Colors
                 * @function PaleVioletRed
                 * @type Windows.UI.Color
                 */
                PaleVioletRed: {
                    get: function () {
                        return Windows.UI.Color.INTERNAL_ConvertFromInt32(Windows.UI.Colors.INTERNAL_ColorsEnum.PaleVioletRed);
                    }
                },
                /**
                 * Gets the color value that represents the PapayaWhip named color.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Colors
                 * @function PapayaWhip
                 * @type Windows.UI.Color
                 */
                PapayaWhip: {
                    get: function () {
                        return Windows.UI.Color.INTERNAL_ConvertFromInt32(Windows.UI.Colors.INTERNAL_ColorsEnum.PapayaWhip);
                    }
                },
                /**
                 * Gets the color value that represents the PeachPuff named color.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Colors
                 * @function PeachPuff
                 * @type Windows.UI.Color
                 */
                PeachPuff: {
                    get: function () {
                        return Windows.UI.Color.INTERNAL_ConvertFromInt32(Windows.UI.Colors.INTERNAL_ColorsEnum.PeachPuff);
                    }
                },
                /**
                 * Gets the color value that represents the Peru named color.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Colors
                 * @function Peru
                 * @type Windows.UI.Color
                 */
                Peru: {
                    get: function () {
                        return Windows.UI.Color.INTERNAL_ConvertFromInt32(Windows.UI.Colors.INTERNAL_ColorsEnum.Peru);
                    }
                },
                /**
                 * Gets the color value that represents the Pink named color.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Colors
                 * @function Pink
                 * @type Windows.UI.Color
                 */
                Pink: {
                    get: function () {
                        return Windows.UI.Color.INTERNAL_ConvertFromInt32(Windows.UI.Colors.INTERNAL_ColorsEnum.Pink);
                    }
                },
                /**
                 * Gets the color value that represents the Plum named color.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Colors
                 * @function Plum
                 * @type Windows.UI.Color
                 */
                Plum: {
                    get: function () {
                        return Windows.UI.Color.INTERNAL_ConvertFromInt32(Windows.UI.Colors.INTERNAL_ColorsEnum.Plum);
                    }
                },
                /**
                 * Gets the color value that represents the PowderBlue named color.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Colors
                 * @function PowderBlue
                 * @type Windows.UI.Color
                 */
                PowderBlue: {
                    get: function () {
                        return Windows.UI.Color.INTERNAL_ConvertFromInt32(Windows.UI.Colors.INTERNAL_ColorsEnum.PowderBlue);
                    }
                },
                /**
                 * Gets the color value that represents the Purple named color.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Colors
                 * @function Purple
                 * @type Windows.UI.Color
                 */
                Purple: {
                    get: function () {
                        return Windows.UI.Color.INTERNAL_ConvertFromInt32(Windows.UI.Colors.INTERNAL_ColorsEnum.Purple);
                    }
                },
                /**
                 * Gets the color value that represents the Red named color.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Colors
                 * @function Red
                 * @type Windows.UI.Color
                 */
                Red: {
                    get: function () {
                        return Windows.UI.Color.INTERNAL_ConvertFromInt32(Windows.UI.Colors.INTERNAL_ColorsEnum.Red);
                    }
                },
                /**
                 * Gets the color value that represents the RosyBrown named color.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Colors
                 * @function RosyBrown
                 * @type Windows.UI.Color
                 */
                RosyBrown: {
                    get: function () {
                        return Windows.UI.Color.INTERNAL_ConvertFromInt32(Windows.UI.Colors.INTERNAL_ColorsEnum.RosyBrown);
                    }
                },
                /**
                 * Gets the color value that represents the RoyalBlue named color.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Colors
                 * @function RoyalBlue
                 * @type Windows.UI.Color
                 */
                RoyalBlue: {
                    get: function () {
                        return Windows.UI.Color.INTERNAL_ConvertFromInt32(Windows.UI.Colors.INTERNAL_ColorsEnum.RoyalBlue);
                    }
                },
                /**
                 * Gets the color value that represents the SaddleBrown named color.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Colors
                 * @function SaddleBrown
                 * @type Windows.UI.Color
                 */
                SaddleBrown: {
                    get: function () {
                        return Windows.UI.Color.INTERNAL_ConvertFromInt32(Windows.UI.Colors.INTERNAL_ColorsEnum.SaddleBrown);
                    }
                },
                /**
                 * Gets the color value that represents the Salmon named color.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Colors
                 * @function Salmon
                 * @type Windows.UI.Color
                 */
                Salmon: {
                    get: function () {
                        return Windows.UI.Color.INTERNAL_ConvertFromInt32(Windows.UI.Colors.INTERNAL_ColorsEnum.Salmon);
                    }
                },
                /**
                 * Gets the color value that represents the SandyBrown named color.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Colors
                 * @function SandyBrown
                 * @type Windows.UI.Color
                 */
                SandyBrown: {
                    get: function () {
                        return Windows.UI.Color.INTERNAL_ConvertFromInt32(Windows.UI.Colors.INTERNAL_ColorsEnum.SandyBrown);
                    }
                },
                /**
                 * Gets the color value that represents the SeaGreen named color.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Colors
                 * @function SeaGreen
                 * @type Windows.UI.Color
                 */
                SeaGreen: {
                    get: function () {
                        return Windows.UI.Color.INTERNAL_ConvertFromInt32(Windows.UI.Colors.INTERNAL_ColorsEnum.SeaGreen);
                    }
                },
                /**
                 * Gets the color value that represents the SeaShell named color.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Colors
                 * @function SeaShell
                 * @type Windows.UI.Color
                 */
                SeaShell: {
                    get: function () {
                        return Windows.UI.Color.INTERNAL_ConvertFromInt32(Windows.UI.Colors.INTERNAL_ColorsEnum.SeaShell);
                    }
                },
                /**
                 * Gets the color value that represents the Sienna named color.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Colors
                 * @function Sienna
                 * @type Windows.UI.Color
                 */
                Sienna: {
                    get: function () {
                        return Windows.UI.Color.INTERNAL_ConvertFromInt32(Windows.UI.Colors.INTERNAL_ColorsEnum.Sienna);
                    }
                },
                /**
                 * Gets the color value that represents the Silver named color.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Colors
                 * @function Silver
                 * @type Windows.UI.Color
                 */
                Silver: {
                    get: function () {
                        return Windows.UI.Color.INTERNAL_ConvertFromInt32(Windows.UI.Colors.INTERNAL_ColorsEnum.Silver);
                    }
                },
                /**
                 * Gets the color value that represents the SkyBlue named color.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Colors
                 * @function SkyBlue
                 * @type Windows.UI.Color
                 */
                SkyBlue: {
                    get: function () {
                        return Windows.UI.Color.INTERNAL_ConvertFromInt32(Windows.UI.Colors.INTERNAL_ColorsEnum.SkyBlue);
                    }
                },
                /**
                 * Gets the color value that represents the SlateBlue named color.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Colors
                 * @function SlateBlue
                 * @type Windows.UI.Color
                 */
                SlateBlue: {
                    get: function () {
                        return Windows.UI.Color.INTERNAL_ConvertFromInt32(Windows.UI.Colors.INTERNAL_ColorsEnum.SlateBlue);
                    }
                },
                /**
                 * Gets the color value that represents the SlateGray named color.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Colors
                 * @function SlateGray
                 * @type Windows.UI.Color
                 */
                SlateGray: {
                    get: function () {
                        return Windows.UI.Color.INTERNAL_ConvertFromInt32(Windows.UI.Colors.INTERNAL_ColorsEnum.SlateGray);
                    }
                },
                /**
                 * Gets the color value that represents the Snow named color.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Colors
                 * @function Snow
                 * @type Windows.UI.Color
                 */
                Snow: {
                    get: function () {
                        return Windows.UI.Color.INTERNAL_ConvertFromInt32(Windows.UI.Colors.INTERNAL_ColorsEnum.Snow);
                    }
                },
                /**
                 * Gets the color value that represents the SpringGreen named color.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Colors
                 * @function SpringGreen
                 * @type Windows.UI.Color
                 */
                SpringGreen: {
                    get: function () {
                        return Windows.UI.Color.INTERNAL_ConvertFromInt32(Windows.UI.Colors.INTERNAL_ColorsEnum.SpringGreen);
                    }
                },
                /**
                 * Gets the color value that represents the SteelBlue named color.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Colors
                 * @function SteelBlue
                 * @type Windows.UI.Color
                 */
                SteelBlue: {
                    get: function () {
                        return Windows.UI.Color.INTERNAL_ConvertFromInt32(Windows.UI.Colors.INTERNAL_ColorsEnum.SteelBlue);
                    }
                },
                /**
                 * Gets the color value that represents the Tan named color.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Colors
                 * @function Tan
                 * @type Windows.UI.Color
                 */
                Tan: {
                    get: function () {
                        return Windows.UI.Color.INTERNAL_ConvertFromInt32(Windows.UI.Colors.INTERNAL_ColorsEnum.Tan);
                    }
                },
                /**
                 * Gets the color value that represents the Teal named color.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Colors
                 * @function Teal
                 * @type Windows.UI.Color
                 */
                Teal: {
                    get: function () {
                        return Windows.UI.Color.INTERNAL_ConvertFromInt32(Windows.UI.Colors.INTERNAL_ColorsEnum.Teal);
                    }
                },
                /**
                 * Gets the color value that represents the Thistle named color.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Colors
                 * @function Thistle
                 * @type Windows.UI.Color
                 */
                Thistle: {
                    get: function () {
                        return Windows.UI.Color.INTERNAL_ConvertFromInt32(Windows.UI.Colors.INTERNAL_ColorsEnum.Thistle);
                    }
                },
                /**
                 * Gets the color value that represents the Tomato named color.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Colors
                 * @function Tomato
                 * @type Windows.UI.Color
                 */
                Tomato: {
                    get: function () {
                        return Windows.UI.Color.INTERNAL_ConvertFromInt32(Windows.UI.Colors.INTERNAL_ColorsEnum.Tomato);
                    }
                },
                /**
                 * Gets the color value that represents the Transparent named color.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Colors
                 * @function Transparent
                 * @type Windows.UI.Color
                 */
                Transparent: {
                    get: function () {
                        return Windows.UI.Color.INTERNAL_ConvertFromInt32(Windows.UI.Colors.INTERNAL_ColorsEnum.Transparent);
                    }
                },
                /**
                 * Gets the color value that represents the Turquoise named color.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Colors
                 * @function Turquoise
                 * @type Windows.UI.Color
                 */
                Turquoise: {
                    get: function () {
                        return Windows.UI.Color.INTERNAL_ConvertFromInt32(Windows.UI.Colors.INTERNAL_ColorsEnum.Turquoise);
                    }
                },
                /**
                 * Gets the color value that represents the Violet named color.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Colors
                 * @function Violet
                 * @type Windows.UI.Color
                 */
                Violet: {
                    get: function () {
                        return Windows.UI.Color.INTERNAL_ConvertFromInt32(Windows.UI.Colors.INTERNAL_ColorsEnum.Violet);
                    }
                },
                /**
                 * Gets the color value that represents the Wheat named color.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Colors
                 * @function Wheat
                 * @type Windows.UI.Color
                 */
                Wheat: {
                    get: function () {
                        return Windows.UI.Color.INTERNAL_ConvertFromInt32(Windows.UI.Colors.INTERNAL_ColorsEnum.Wheat);
                    }
                },
                /**
                 * Gets the color value that represents the White named color.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Colors
                 * @function White
                 * @type Windows.UI.Color
                 */
                White: {
                    get: function () {
                        return Windows.UI.Color.INTERNAL_ConvertFromInt32(Windows.UI.Colors.INTERNAL_ColorsEnum.White);
                    }
                },
                /**
                 * Gets the color value that represents the WhiteSmoke named color.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Colors
                 * @function WhiteSmoke
                 * @type Windows.UI.Color
                 */
                WhiteSmoke: {
                    get: function () {
                        return Windows.UI.Color.INTERNAL_ConvertFromInt32(Windows.UI.Colors.INTERNAL_ColorsEnum.WhiteSmoke);
                    }
                },
                /**
                 * Gets the color value that represents the Yellow named color.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Colors
                 * @function Yellow
                 * @type Windows.UI.Color
                 */
                Yellow: {
                    get: function () {
                        return Windows.UI.Color.INTERNAL_ConvertFromInt32(Windows.UI.Colors.INTERNAL_ColorsEnum.Yellow);
                    }
                },
                /**
                 * Gets the color value that represents the YellowGreen named color.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Colors
                 * @function YellowGreen
                 * @type Windows.UI.Color
                 */
                YellowGreen: {
                    get: function () {
                        return Windows.UI.Color.INTERNAL_ConvertFromInt32(Windows.UI.Colors.INTERNAL_ColorsEnum.YellowGreen);
                    }
                }
            }
        }
    });

    Bridge.define("Windows.UI.Colors.INTERNAL_ColorsEnum", {
        $kind: "nested enum",
        statics: {
            fields: {
                AliceBlue: -984833,
                AntiqueWhite: -332841,
                Aqua: -16711681,
                Aquamarine: -8388652,
                Azure: -983041,
                Beige: -657956,
                Bisque: -6972,
                Black: -16777216,
                BlanchedAlmond: -5171,
                Blue: -16776961,
                BlueViolet: -7722014,
                Brown: -5952982,
                BurlyWood: -2180985,
                CadetBlue: -10510688,
                Chartreuse: -8388864,
                Chocolate: -2987746,
                Coral: -32944,
                CornflowerBlue: -10185235,
                Cornsilk: -1828,
                Crimson: -2354116,
                Cyan: -16711681,
                DarkBlue: -16777077,
                DarkCyan: -16741493,
                DarkGoldenrod: -4684277,
                DarkGray: -5658199,
                DarkGreen: -16751616,
                DarkKhaki: -4343957,
                DarkMagenta: -7667573,
                DarkOliveGreen: -11179217,
                DarkOrange: -29696,
                DarkOrchid: -6737204,
                DarkRed: -7667712,
                DarkSalmon: -1468806,
                DarkSeaGreen: -7357297,
                DarkSlateBlue: -12042869,
                DarkSlateGray: -13676721,
                DarkTurquoise: -16724271,
                DarkViolet: -7077677,
                DeepPink: -60269,
                DeepSkyBlue: -16728065,
                DimGray: -9868951,
                DodgerBlue: -14774017,
                Firebrick: -5103070,
                FloralWhite: -1296,
                ForestGreen: -14513374,
                Fuchsia: -65281,
                Gainsboro: -2302756,
                GhostWhite: -460545,
                Gold: -10496,
                Goldenrod: -2448096,
                Gray: -8355712,
                Green: -16744448,
                GreenYellow: -5374161,
                Honeydew: -983056,
                HotPink: -38476,
                IndianRed: -3318692,
                Indigo: -11861886,
                Ivory: -16,
                Khaki: -989556,
                Lavender: -1644806,
                LavenderBlush: -3851,
                LawnGreen: -8586240,
                LemonChiffon: -1331,
                LightBlue: -5383962,
                LightCoral: -1015680,
                LightCyan: -2031617,
                LightGoldenrodYellow: -329006,
                LightGreen: -7278960,
                LightGray: -2894893,
                LightPink: -18751,
                LightSalmon: -24454,
                LightSeaGreen: -14634326,
                LightSkyBlue: -7876870,
                LightSlateGray: -8943463,
                LightSteelBlue: -5192482,
                LightYellow: -32,
                Lime: -16711936,
                LimeGreen: -13447886,
                Linen: -331546,
                Magenta: -65281,
                Maroon: -8388608,
                MediumAquamarine: -10039894,
                MediumBlue: -16777011,
                MediumOrchid: -4565549,
                MediumPurple: -7114533,
                MediumSeaGreen: -12799119,
                MediumSlateBlue: -8689426,
                MediumSpringGreen: -16713062,
                MediumTurquoise: -12004916,
                MediumVioletRed: -3730043,
                MidnightBlue: -15132304,
                MintCream: -655366,
                MistyRose: -6943,
                Moccasin: -6987,
                NavajoWhite: -8531,
                Navy: -16777088,
                OldLace: -133658,
                Olive: -8355840,
                OliveDrab: -9728477,
                Orange: -23296,
                OrangeRed: -47872,
                Orchid: -2461482,
                PaleGoldenrod: -1120086,
                PaleGreen: -6751336,
                PaleTurquoise: -5247250,
                PaleVioletRed: -2396013,
                PapayaWhip: -4139,
                PeachPuff: -9543,
                Peru: -3308225,
                Pink: -16181,
                Plum: -2252579,
                PowderBlue: -5185306,
                Purple: -8388480,
                Red: -65536,
                RosyBrown: -4419697,
                RoyalBlue: -12490271,
                SaddleBrown: -7650029,
                Salmon: -360334,
                SandyBrown: -744352,
                SeaGreen: -13726889,
                SeaShell: -2578,
                Sienna: -6270419,
                Silver: -4144960,
                SkyBlue: -7876885,
                SlateBlue: -9807155,
                SlateGray: -9404272,
                Snow: -1286,
                SpringGreen: -16711809,
                SteelBlue: -12156236,
                Tan: -2968436,
                Teal: -16744320,
                Thistle: -2572328,
                Tomato: -40121,
                Transparent: 16777215,
                Turquoise: -12525360,
                Violet: -1146130,
                Wheat: -663885,
                White: -1,
                WhiteSmoke: -657931,
                Yellow: -256,
                YellowGreen: -6632142
            }
        }
    });

    /** @namespace Windows.UI.Core */

    /**
     * Provides the core event message dispatcher. Instances of this type are responsible
     for processing the window messages and dispatching the events to the client.
     *
     * @public
     * @class Windows.UI.Core.CoreDispatcher
     */
    Bridge.define("Windows.UI.Core.CoreDispatcher", {
        statics: {
            fields: {
                _currentDispatcher: null
            },
            methods: {
                INTERNAL_GetCurrentDispatcher: function () {
                    if (Windows.UI.Core.CoreDispatcher._currentDispatcher == null) {
                        Windows.UI.Core.CoreDispatcher._currentDispatcher = new Windows.UI.Core.CoreDispatcher();
                    }
                    return Windows.UI.Core.CoreDispatcher._currentDispatcher;
                }
            }
        },
        methods: {
            /**
             * Executes the specified delegate asynchronously on the thread that the System.Windows.Threading.Dispatcher was created on.
             *
             * @instance
             * @public
             * @this Windows.UI.Core.CoreDispatcher
             * @memberof Windows.UI.Core.CoreDispatcher
             * @param   {System.Action}    method    The delegate to a method, which is
             pushed onto the System.Windows.Threading.Dispatcher event queue.
             * @return  {void}
             */
            BeginInvoke: function (method) {
                if (Bridge.staticEquals(method, null)) {
                    throw new System.ArgumentNullException.$ctor1("method");
                }
                setTimeout(function(){ method(); },1);
            }
        }
    });

    /**
     * The {@link } namespace provides the basic user interface window
     components, including core input events and event dispatch.
     *
     * @class Windows.UI.Core.NamespaceDoc
     */
    Bridge.define("Windows.UI.Core.NamespaceDoc");

    /**
     * Contains the argument returned by a window size change event.
     *
     * @public
     * @class Windows.UI.Core.WindowSizeChangedEventArgs
     */
    Bridge.define("Windows.UI.Core.WindowSizeChangedEventArgs", {
        fields: {
            /**
             * Gets or sets whether the window size event was handled.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Core.WindowSizeChangedEventArgs
             * @function Handled
             * @type boolean
             */
            Handled: false,
            /**
             * Gets the new size of the window.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Core.WindowSizeChangedEventArgs
             * @function Size
             * @type Windows.Foundation.Size
             */
            Size: null
        },
        ctors: {
            init: function () {
                this.Size = new Windows.Foundation.Size();
            }
        }
    });

    /** @namespace Windows.UI.Input */

    /**
     * The {@link } namespace provides support for the Windows input system.
     *
     * @class Windows.UI.Input.NamespaceDoc
     */
    Bridge.define("Windows.UI.Input.NamespaceDoc");

    /**
     * Provides basic properties for the input pointer associated with a single
     mouse, pen/stylus, or touch contact.
     * // Gets the raw location of the pointer input in client coordinates.
    //
     * // Gets the time when the input occurred.
    //
     *
     * @public
     * @class Windows.UI.Input.PointerPoint
     */
    Bridge.define("Windows.UI.Input.PointerPoint", {
        fields: {
            /**
             * // Gets the ID of an input frame.
            //
             * // Gets a value that indicates whether the physical entity (touch, pen/stylus,
            // or mouse button) is pressed down.
            //
             * // Gets a unique identifier for the input pointer.
            //
             *
             * @instance
             * @private
             * @memberof Windows.UI.Input.PointerPoint
             * @type Windows.Foundation.Point
             */
            _position: null
        },
        props: {
            /**
             * Gets the location of the pointer input in client coordinates.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Input.PointerPoint
             * @function Position
             * @type Windows.Foundation.Point
             */
            Position: {
                get: function () {
                    return this._position.$clone();
                },
                set: function (value) {
                    this._position = value.$clone();
                }
            }
        },
        ctors: {
            init: function () {
                this._position = new Windows.Foundation.Point();
            }
        }
    });

    /**
     * The {@link } namespace provides an app with access to core system
     functionality and run-time information about its UI.
     *
     * @class Windows.UI.NamespaceDoc
     */
    Bridge.define("Windows.UI.NamespaceDoc");

    /** @namespace Windows.UI.Text */

    /**
     * Represents the style of a font face (for example, normal or italic).
     *
     * @public
     * @class Windows.UI.Text.FontStyle
     */
    Bridge.define("Windows.UI.Text.FontStyle", {
        $kind: "enum",
        statics: {
            fields: {
                /**
                 * Represents a normal font style.
                 *
                 * @static
                 * @public
                 * @memberof Windows.UI.Text.FontStyle
                 * @constant
                 * @default 0
                 * @type Windows.UI.Text.FontStyle
                 */
                Normal: 0,
                /**
                 * Represents an oblique font style.
                 *
                 * @static
                 * @public
                 * @memberof Windows.UI.Text.FontStyle
                 * @constant
                 * @default 1
                 * @type Windows.UI.Text.FontStyle
                 */
                Oblique: 1,
                /**
                 * Represents an italic font style.
                 *
                 * @static
                 * @public
                 * @memberof Windows.UI.Text.FontStyle
                 * @constant
                 * @default 2
                 * @type Windows.UI.Text.FontStyle
                 */
                Italic: 2
            }
        }
    });

    /**
     * Refers to the density of a typeface, in terms of the lightness or heaviness
     of the strokes.
     *
     * @public
     * @class Windows.UI.Text.FontWeight
     */
    Bridge.define("Windows.UI.Text.FontWeight", {
        $kind: "struct",
        statics: {
            ctors: {
                ctor: function () {
                    DotNetForHtml5.Core.TypeFromStringConverters.RegisterConverter(Windows.UI.Text.FontWeight, Windows.UI.Text.FontWeight.INTERNAL_ConvertFromString);
                }
            },
            methods: {
                INTERNAL_ConvertFromString: function (fontCode) {
                    try {
                        var result = { };
                        if (!System.UInt16.tryParse(fontCode, result)) {
                            var namedFont = System.Nullable.getValue(Bridge.cast(Bridge.unbox(System.Enum.parse(Windows.UI.Text.FontWeights.INTERNAL_FontweightsEnum, fontCode), Windows.UI.Text.FontWeights.INTERNAL_FontweightsEnum), System.UInt16));
                            result.v = namedFont;
                        }
                        return Windows.UI.Text.FontWeight.INTERNAL_ConvertFromUshort(result.v).$clone();
                    } catch (ex) {
                        ex = System.Exception.create(ex);
                        throw new System.Exception("Invalid font: " + (fontCode || ""), ex);
                    }
                },
                INTERNAL_ConvertFromUshort: function (fontWeightAsUshort) {
                    var $t;
                    return ($t = new Windows.UI.Text.FontWeight(), $t.Weight = fontWeightAsUshort, $t);
                },
                /**
                 * Compares two font weight values and returns an indication of their relative values.
                 *
                 * @static
                 * @public
                 * @this Windows.UI.Text.FontWeight
                 * @memberof Windows.UI.Text.FontWeight
                 * @param   {Windows.UI.Text.FontWeight}    left     First object to compare.
                 * @param   {Windows.UI.Text.FontWeight}    right    Second object to compare.
                 * @return  {number}                                 A 32-bit signed integer indicating the lexical relationship between the two comparands.
                 When the return value is less than zero this means that left is less than right.
                 When the return value is zero this means that left is equal to right.
                 When the return value is greater than zero this means that left is greater than right.
                 */
                Compare: function (left, right) {
                    return ((left.Weight - right.Weight) | 0);
                },
                op_Equality: function (left, right) {
                    return Windows.UI.Text.FontWeight.Compare(left.$clone(), right.$clone()) === 0;
                },
                op_Inequality: function (left, right) {
                    return Windows.UI.Text.FontWeight.Compare(left.$clone(), right.$clone()) !== 0;
                },
                op_GreaterThan: function (left, right) {
                    return Windows.UI.Text.FontWeight.Compare(left.$clone(), right.$clone()) > 0;
                },
                op_LessThan: function (left, right) {
                    return Windows.UI.Text.FontWeight.Compare(left.$clone(), right.$clone()) < 0;
                },
                op_GreaterThanOrEqual: function (left, right) {
                    return Windows.UI.Text.FontWeight.Compare(left.$clone(), right.$clone()) >= 0;
                },
                op_LessThanOrEqual: function (left, right) {
                    return Windows.UI.Text.FontWeight.Compare(left.$clone(), right.$clone()) <= 0;
                },
                getDefaultValue: function () { return new Windows.UI.Text.FontWeight(); }
            }
        },
        fields: {
            /**
             * The font weight expressed as a numeric value. See Remarks.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Text.FontWeight
             * @type number
             */
            Weight: 0
        },
        ctors: {
            ctor: function () {
                this.$initialize();
            }
        },
        methods: {
            INTERNAL_ToHtmlString: function () {
                return Bridge.toString(this.Weight);
            },
            toString: function () {
                return Bridge.toString(this.Weight);
            },
            Equals: function (fontWeight) {
                return Windows.UI.Text.FontWeight.op_Equality(this, fontWeight.$clone());
            },
            equals: function (o) {
                if (Bridge.is(o, Windows.UI.Text.FontWeight)) {
                    var fw = System.Nullable.getValue(Bridge.cast(Bridge.unbox(o, Windows.UI.Text.FontWeight), Windows.UI.Text.FontWeight));
                    return Windows.UI.Text.FontWeight.op_Equality(this, fw.$clone());
                }
                return false;
            },
            getHashCode: function () {
                return this.Weight;
            },
            $clone: function (to) {
                var s = to || new Windows.UI.Text.FontWeight();
                s.Weight = this.Weight;
                return s;
            }
        }
    });

    /**
     * Provides a set of predefined font weights as static property values.
     *
     * @public
     * @class Windows.UI.Text.FontWeights
     */
    Bridge.define("Windows.UI.Text.FontWeights", {
        statics: {
            props: {
                /**
                 * Specifies a "Black" font weight.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Text.FontWeights
                 * @function Black
                 * @type Windows.UI.Text.FontWeight
                 */
                Black: {
                    get: function () {
                        return Windows.UI.Text.FontWeight.INTERNAL_ConvertFromUshort(Windows.UI.Text.FontWeights.INTERNAL_FontweightsEnum.Black);
                    }
                },
                /**
                 * Specifies a "Bold" font weight.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Text.FontWeights
                 * @function Bold
                 * @type Windows.UI.Text.FontWeight
                 */
                Bold: {
                    get: function () {
                        return Windows.UI.Text.FontWeight.INTERNAL_ConvertFromUshort(Windows.UI.Text.FontWeights.INTERNAL_FontweightsEnum.Bold);
                    }
                },
                /**
                 * Specifies a "Demi-bold" font weight.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Text.FontWeights
                 * @function DemiBold
                 * @type Windows.UI.Text.FontWeight
                 */
                DemiBold: {
                    get: function () {
                        return Windows.UI.Text.FontWeight.INTERNAL_ConvertFromUshort(Windows.UI.Text.FontWeights.INTERNAL_FontweightsEnum.DemiBold);
                    }
                },
                /**
                 * Specifies an "ExtraBlack" font weight.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Text.FontWeights
                 * @function ExtraBlack
                 * @type Windows.UI.Text.FontWeight
                 */
                ExtraBlack: {
                    get: function () {
                        return Windows.UI.Text.FontWeight.INTERNAL_ConvertFromUshort(Windows.UI.Text.FontWeights.INTERNAL_FontweightsEnum.ExtraBlack);
                    }
                },
                /**
                 * Specifies an "ExtraBold" font weight.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Text.FontWeights
                 * @function ExtraBold
                 * @type Windows.UI.Text.FontWeight
                 */
                ExtraBold: {
                    get: function () {
                        return Windows.UI.Text.FontWeight.INTERNAL_ConvertFromUshort(Windows.UI.Text.FontWeights.INTERNAL_FontweightsEnum.ExtraBold);
                    }
                },
                /**
                 * Specifies an "ExtraLight" font weight.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Text.FontWeights
                 * @function ExtraLight
                 * @type Windows.UI.Text.FontWeight
                 */
                ExtraLight: {
                    get: function () {
                        return Windows.UI.Text.FontWeight.INTERNAL_ConvertFromUshort(Windows.UI.Text.FontWeights.INTERNAL_FontweightsEnum.ExtraLight);
                    }
                },
                /**
                 * Specifies a "Heavy" font weight.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Text.FontWeights
                 * @function Heavy
                 * @type Windows.UI.Text.FontWeight
                 */
                Heavy: {
                    get: function () {
                        return Windows.UI.Text.FontWeight.INTERNAL_ConvertFromUshort(Windows.UI.Text.FontWeights.INTERNAL_FontweightsEnum.Heavy);
                    }
                },
                /**
                 * Specifies a "Light" font weight.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Text.FontWeights
                 * @function Light
                 * @type Windows.UI.Text.FontWeight
                 */
                Light: {
                    get: function () {
                        return Windows.UI.Text.FontWeight.INTERNAL_ConvertFromUshort(Windows.UI.Text.FontWeights.INTERNAL_FontweightsEnum.Light);
                    }
                },
                /**
                 * Specifies a "Medium" font weight.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Text.FontWeights
                 * @function Medium
                 * @type Windows.UI.Text.FontWeight
                 */
                Medium: {
                    get: function () {
                        return Windows.UI.Text.FontWeight.INTERNAL_ConvertFromUshort(Windows.UI.Text.FontWeights.INTERNAL_FontweightsEnum.Medium);
                    }
                },
                /**
                 * Specifies a "Normal" font weight.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Text.FontWeights
                 * @function Normal
                 * @type Windows.UI.Text.FontWeight
                 */
                Normal: {
                    get: function () {
                        return Windows.UI.Text.FontWeight.INTERNAL_ConvertFromUshort(Windows.UI.Text.FontWeights.INTERNAL_FontweightsEnum.Normal);
                    }
                },
                /**
                 * Specifies a "Regular" font weight.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Text.FontWeights
                 * @function Regular
                 * @type Windows.UI.Text.FontWeight
                 */
                Regular: {
                    get: function () {
                        return Windows.UI.Text.FontWeight.INTERNAL_ConvertFromUshort(Windows.UI.Text.FontWeights.INTERNAL_FontweightsEnum.Regular);
                    }
                },
                /**
                 * Specifies a "SemiBold" font weight.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Text.FontWeights
                 * @function SemiBold
                 * @type Windows.UI.Text.FontWeight
                 */
                SemiBold: {
                    get: function () {
                        return Windows.UI.Text.FontWeight.INTERNAL_ConvertFromUshort(Windows.UI.Text.FontWeights.INTERNAL_FontweightsEnum.SemiBold);
                    }
                },
                /**
                 * Specifies a "SemiLight" font weight.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Text.FontWeights
                 * @function SemiLight
                 * @type Windows.UI.Text.FontWeight
                 */
                SemiLight: {
                    get: function () {
                        return Windows.UI.Text.FontWeight.INTERNAL_ConvertFromUshort(Windows.UI.Text.FontWeights.INTERNAL_FontweightsEnum.SemiLight);
                    }
                },
                /**
                 * Specifies a "Thin" font weight.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Text.FontWeights
                 * @function Thin
                 * @type Windows.UI.Text.FontWeight
                 */
                Thin: {
                    get: function () {
                        return Windows.UI.Text.FontWeight.INTERNAL_ConvertFromUshort(Windows.UI.Text.FontWeights.INTERNAL_FontweightsEnum.Thin);
                    }
                },
                /**
                 * Specifies an "Ultra-black" font weight.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Text.FontWeights
                 * @function UltraBlack
                 * @type Windows.UI.Text.FontWeight
                 */
                UltraBlack: {
                    get: function () {
                        return Windows.UI.Text.FontWeight.INTERNAL_ConvertFromUshort(Windows.UI.Text.FontWeights.INTERNAL_FontweightsEnum.UltraBlack);
                    }
                },
                /**
                 * Specifies an "Ultra-bold" font weight.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Text.FontWeights
                 * @function UltraBold
                 * @type Windows.UI.Text.FontWeight
                 */
                UltraBold: {
                    get: function () {
                        return Windows.UI.Text.FontWeight.INTERNAL_ConvertFromUshort(Windows.UI.Text.FontWeights.INTERNAL_FontweightsEnum.UltraBold);
                    }
                },
                /**
                 * Specifies an "Ultra-light" font weight.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Text.FontWeights
                 * @function UltraLight
                 * @type Windows.UI.Text.FontWeight
                 */
                UltraLight: {
                    get: function () {
                        return Windows.UI.Text.FontWeight.INTERNAL_ConvertFromUshort(Windows.UI.Text.FontWeights.INTERNAL_FontweightsEnum.UltraLight);
                    }
                }
            }
        }
    });

    Bridge.define("Windows.UI.Text.FontWeights.INTERNAL_FontweightsEnum", {
        $kind: "nested enum",
        statics: {
            fields: {
                Black: 900,
                Bold: 700,
                DemiBold: 600,
                ExtraBlack: 900,
                ExtraBold: 800,
                ExtraLight: 200,
                Heavy: 900,
                Light: 300,
                Medium: 500,
                Normal: 400,
                Regular: 400,
                SemiBold: 600,
                SemiLight: 300,
                Thin: 100,
                UltraBlack: 900,
                UltraBold: 800,
                UltraLight: 200
            }
        },
        $utype: System.UInt16
    });

    /**
     * The {@link } namespace provides support for working with textual content.
     *
     * @class Windows.UI.Text.NamespaceDoc
     */
    Bridge.define("Windows.UI.Text.NamespaceDoc");

    /**
     * Provides a set of predefined text decorations.
     *
     * @public
     * @class Windows.UI.Text.TextDecorations
     */
    Bridge.define("Windows.UI.Text.TextDecorations", {
        $kind: "enum",
        statics: {
            fields: {
                /**
                 * Defines a line above the text
                 *
                 * @static
                 * @public
                 * @memberof Windows.UI.Text.TextDecorations
                 * @constant
                 * @default 0
                 * @type Windows.UI.Text.TextDecorations
                 */
                None: 0,
                /**
                 * // Defines a line below the text
                //
                 * Defines a line above the text
                 *
                 * @static
                 * @public
                 * @memberof Windows.UI.Text.TextDecorations
                 * @constant
                 * @default 2
                 * @type Windows.UI.Text.TextDecorations
                 */
                OverLine: 2,
                /**
                 * Defines a line through the text
                 *
                 * @static
                 * @public
                 * @memberof Windows.UI.Text.TextDecorations
                 * @constant
                 * @default 3
                 * @type Windows.UI.Text.TextDecorations
                 */
                Strikethrough: 3,
                /**
                 * Defines a line below the text
                 *
                 * @static
                 * @public
                 * @memberof Windows.UI.Text.TextDecorations
                 * @constant
                 * @default 4
                 * @type Windows.UI.Text.TextDecorations
                 */
                Underline: 4
            }
        }
    });

    /**
     * @memberof System
     * @callback System.EventHandler
     * @param   {System.Object}    sender    
     * @param   {System.Object}    e
     * @return  {void}
     */

    /**
     * @memberof Windows.UI.Xaml
     * @callback Windows.UI.Xaml.StartupEventHandler
     * @param   {System.Object}                       sender    
     * @param   {Windows.UI.Xaml.StartupEventArgs}    e
     * @return  {void}
     */

    /**
     * Encapsulates the app and its available services.
     *
     * @public
     * @class Windows.UI.Xaml.Application
     */
    Bridge.define("Windows.UI.Xaml.Application", {
        statics: {
            fields: {
                _resourcesCache: null,
                /**
                 * Gets the Application object for the current application.
                 *
                 * @static
                 * @public
                 * @memberof Windows.UI.Xaml.Application
                 * @function Current
                 * @type Windows.UI.Xaml.Application
                 */
                Current: null,
                _host: null
            },
            events: {
                INTERNAL_Reloaded: null
            },
            methods: {
                /**
                 * Injects the "DataContractSerializer" into the "XmlSerializer" (read note in the "XmlSerializer" implementation to understand why).
                 *
                 * @static
                 * @private
                 * @this Windows.UI.Xaml.Application
                 * @memberof Windows.UI.Xaml.Application
                 * @return  {void}
                 */
                InjectDataContractSerializerIntoXmlSerializer: function () {

                },
                /**
                 * Returns a string that contains the content of the file that is located at the
                 specified System.Uri.
                 *
                 * @static
                 * @public
                 * @this Windows.UI.Xaml.Application
                 * @memberof Windows.UI.Xaml.Application
                 * @param   {System.Uri}                       uriResource    The System.Uri that maps to an embedded resource.
                 * @return  {System.Threading.Tasks.Task$1}                   A string that contains the content of the file that is located at the specified System.Uri.
                 */
                GetResourceString: function (uriResource) {
                    if (Windows.UI.Xaml.Application._resourcesCache == null) {
                        Windows.UI.Xaml.Application._resourcesCache = new (System.Collections.Generic.Dictionary$2(System.String,System.String)).ctor();
                    }
                    var tcs = new System.Threading.Tasks.TaskCompletionSource();
                    if (Windows.UI.Xaml.Application._resourcesCache.containsKey(uriResource.getOriginalString().toLowerCase())) {
                        tcs.setResult(Windows.UI.Xaml.Application._resourcesCache.getItem(uriResource.getOriginalString().toLowerCase()));
                        return tcs.task;
                    }
                    var supportedExtensions = new (System.Collections.Generic.HashSet2$1(System.String)).$ctor1(System.Array.init([".txt", ".xml", ".config", ".json", ".clientconfig"], System.String));

                    var uriAsString = uriResource.getOriginalString();
                    var extension = uriAsString.substr(uriAsString.lastIndexOf(String.fromCharCode(46)));
                    if (!supportedExtensions.contains(extension.toLowerCase())) {
                        throw new System.NotSupportedException.$ctor1("Application.GetResourceString is currently not supported for files with an extension different than .txt, .xml, .json, .config, or .clientconfig.");
                    }
                    var uris = new (System.Collections.Generic.List$1(System.String)).ctor();
                    uris.add((uriAsString || "") + ".g.js");
                    if (Bridge.referenceEquals(uriResource.getOriginalString().toLowerCase(), "ms-appx://app.config")) {
                        CSHTML5.Interop.LoadJavaScriptFilesAsync$1(uris, function () {
                            tcs.setResult(System.Convert.toString(window.AppConfig));
                        });
                    } else if (Bridge.referenceEquals(uriResource.getOriginalString().toLowerCase(), "ms-appx://servicereferences.clientconfig")) {
                        CSHTML5.Interop.LoadJavaScriptFilesAsync$1(uris, function () {
                            tcs.setResult(System.Convert.toString(window.ServiceReferencesClientConfig));
                        });
                    } else {
                        CSHTML5.Interop.LoadJavaScriptFilesAsync$1(uris, function () {
                            var result = System.Convert.toString(window.FileContent);
                            Windows.UI.Xaml.Application._resourcesCache.add(uriResource.getOriginalString().toLowerCase(), result);
                            tcs.setResult(result);
                        });
                    }
                    return tcs.task;
                },
                /**
                 * Returns a resource stream for a resource data file that is located at the
                 specified System.Uri (see WPF Application Resource, Content, and Data Files).
                 *
                 * @static
                 * @public
                 * @this Windows.UI.Xaml.Application
                 * @memberof Windows.UI.Xaml.Application
                 * @param   {System.Uri}                       uriResource    The System.Uri that maps to an embedded resource.
                 * @return  {System.Threading.Tasks.Task$1}                   A System.Windows.Resources.StreamResourceInfo that contains a resource stream
                 for resource data file that is located at the specified System.Uri.
                 */
                GetResourceStream: function (uriResource) {
                    var $step = 0,
                        $task1, 
                        $taskResult1, 
                        $jumpFromFinally, 
                        $tcs = new System.Threading.Tasks.TaskCompletionSource(), 
                        $returnValue, 
                        resourceString, 
                        uriAsString, 
                        extensionLowercase, 
                        byteArray, 
                        stream, 
                        mimeType, 
                        resourceInfo, 
                        $async_e, 
                        $asyncBody = Bridge.fn.bind(this, function () {
                            try {
                                for (;;) {
                                    $step = System.Array.min([0,1], $step);
                                    switch ($step) {
                                        case 0: {
                                            $task1 = Windows.UI.Xaml.Application.GetResourceString(uriResource);
                                            $step = 1;
                                            if ($task1.isCompleted()) {
                                                continue;
                                            }
                                            $task1.continue($asyncBody);
                                            return;
                                        }
                                        case 1: {
                                            $taskResult1 = $task1.getAwaitedResult();
                                            resourceString = $taskResult1;
                                            uriAsString = uriResource.getOriginalString();
                                            extensionLowercase = uriAsString.substr(uriAsString.lastIndexOf(String.fromCharCode(46))).toLowerCase();

                                            byteArray = System.Text.Encoding.ASCII.GetBytes$2(resourceString);
                                            stream = new System.IO.MemoryStream.$ctor1(byteArray);

                                            mimeType = "text/plain";
                                            if (Bridge.referenceEquals(extensionLowercase, ".xml") || Bridge.referenceEquals(extensionLowercase, ".config") || Bridge.referenceEquals(extensionLowercase, ".clientconfig")) {
                                                mimeType = "application/xml";
                                            } else if (Bridge.referenceEquals(extensionLowercase, ".json")) {
                                                mimeType = "application/json";
                                            }

                                            resourceInfo = new System.Windows.Resources.StreamResourceInfo.$ctor1(stream, mimeType);
                                            $tcs.setResult(resourceInfo);
                                            return;
                                        }
                                        default: {
                                            $tcs.setResult(null);
                                            return;
                                        }
                                    }
                                }
                            } catch($async_e1) {
                                $async_e = System.Exception.create($async_e1);
                                $tcs.setException($async_e);
                            }
                        }, arguments);

                    $asyncBody();
                    return $tcs.task;
                },
                /**
                 * Intended to be called by the Simulator when navigating back from an external page.
                 *
                 * @static
                 * @public
                 * @this Windows.UI.Xaml.Application
                 * @memberof Windows.UI.Xaml.Application
                 * @return  {void}
                 */
                INTERNAL_RaiseReloadedEvent: function () {
                    var handler = Windows.UI.Xaml.Application.INTERNAL_Reloaded;
                    if (!Bridge.staticEquals(handler, null)) {
                        handler(null, null);
                    }
                }
            }
        },
        fields: {
            _xamlResourcesHandler: null,
            HasImplicitStylesInResources: false,
            _mainWindow: null,
            _resources: null,
            _timerForWorkaroundFireFoxIssue: null,
            _childWindowForWorkaroundFireFoxIssue: null,
            _ExitEventManager: null
        },
        events: {
            /**
             * Occurs when an application is started.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Application
             * @memberof Windows.UI.Xaml.Application
             * @function addStartup
             * @param   {Windows.UI.Xaml.StartupEventHandler}    value
             * @return  {void}
             */
            /**
             * Occurs when an application is started.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Application
             * @memberof Windows.UI.Xaml.Application
             * @function removeStartup
             * @param   {Windows.UI.Xaml.StartupEventHandler}    value
             * @return  {void}
             */
            Startup: null
        },
        props: {
            XamlResourcesHandler: {
                get: function () {
                    return this._xamlResourcesHandler;
                }
            },
            /**
             * Gets a collection of application-scoped resources, such as styles, templates,
             and brushes.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Application
             * @function Resources
             * @type Windows.UI.Xaml.ResourceDictionary
             */
            Resources: {
                get: function () {
                    if (this._resources == null) {
                        this._resources = new Windows.UI.Xaml.ResourceDictionary();
                        this._resources.AddOwner(this);
                    }
                    return this._resources;
                },
                set: function (value) {
                    var oldValue = this._resources;
                    this._resources = value;

                    if (oldValue != null) {
                        oldValue.RemoveOwner(this);
                    }

                    if (value != null) {
                        if (!value.ContainsOwner(this)) {
                            value.AddOwner(this);
                        }
                    }

                }
            },
            /**
             * Gets or sets the main application UI. This is an alias for: Window.Current.Content
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Application
             * @function RootVisual
             * @type Windows.UI.Xaml.UIElement
             */
            RootVisual: {
                get: function () {
                    return (Bridge.as(Windows.UI.Xaml.Window.Current.Content, Windows.UI.Xaml.UIElement));
                },
                set: function (value) {
                    Windows.UI.Xaml.Window.Current.Content = value;
                }
            },
            /**
             * Gets the application main window.
             *
             * @instance
             * @public
             * @readonly
             * @memberof Windows.UI.Xaml.Application
             * @function MainWindow
             * @type Windows.UI.Xaml.Window
             */
            MainWindow: {
                get: function () {
                    return Windows.UI.Xaml.Window.Current;
                }
            },
            ExitEventManager: {
                get: function () {
                    if (this._ExitEventManager == null) {
                        this._ExitEventManager = new (CSHTML5.Internal.INTERNAL_EventManager$2(Function,System.Object)).ctor(Bridge.fn.cacheBind(this, this.GetWindow), "unload", Bridge.fn.cacheBind(this, this.ProcessOnExit));
                    }
                    return this._ExitEventManager;
                }
            },
            /**
             * Gets various details about the application's host.
             *
             * @instance
             * @public
             * @readonly
             * @memberof Windows.UI.Xaml.Application
             * @function Host
             * @type Windows.UI.Xaml.Host
             */
            Host: {
                get: function () {
                    if (Windows.UI.Xaml.Application._host == null) {
                        Windows.UI.Xaml.Application._host = new Windows.UI.Xaml.Host();
                    }
                    return Windows.UI.Xaml.Application._host;
                }
            }
        },
        ctors: {
            init: function () {
                this._xamlResourcesHandler = new CSHTML5.Internal.INTERNAL_XamlResourcesHandler();
                this._timerForWorkaroundFireFoxIssue = new Windows.UI.Xaml.DispatcherTimer();
            },
            ctor: function () {
                this.$initialize();
                CSHTML5.INTERNAL_InteropImplementation.ResetLoadedFilesDictionaries();

                CSHTML5.Internal.StartupAssemblyInfo.StartupAssembly = Bridge.Reflection.getTypeAssembly(Bridge.getType(this));

                document.isSLMigration = false;



                if (!CSHTML5.Interop.IsRunningInTheSimulator) {
                    Windows.UI.Xaml.Application.InjectDataContractSerializerIntoXmlSerializer();
                }

                if (CSHTML5.Interop.IsRunningInTheSimulator) {
                    Windows.UI.Xaml.Application.RedirectAlertToMessageBox_SimulatorOnly();
                }

                Windows.UI.Xaml.Application.Current = this;

                if (this._mainWindow == null) {
                    this._mainWindow = new Windows.UI.Xaml.Window();
                    Windows.UI.Xaml.Window.Current = this._mainWindow;
                    var applicationRootDomElement = CSHTML5.Internal.INTERNAL_HtmlDomManager.GetApplicationRootDomElement();
                    this._mainWindow.AttachToDomElement(applicationRootDomElement);

                    this._mainWindow.AddHandler(Windows.UI.Xaml.UIElement.PointerPressedEvent, DotNetForHtml5.Core.INTERNAL_PopupsManager.OnClickOnPopupOrWindow, true);

                    if ((window.FIREFOX_VERSION)) {
                        this._mainWindow.addSizeChanged$1(Bridge.fn.cacheBind(this, this.MainWindow_SizeChanged));
                        this._timerForWorkaroundFireFoxIssue.Interval = new System.TimeSpan(0, 0, 2);
                        this._timerForWorkaroundFireFoxIssue.addTick(Bridge.fn.cacheBind(this, this.TimerForWorkaroundFireFoxIssue_Tick));
                    }
                }

                Windows.UI.Core.CoreDispatcher.INTERNAL_GetCurrentDispatcher().BeginInvoke(Bridge.fn.bind(this, function () {
                    if (!Bridge.staticEquals(this.Startup, null)) {
                        this.Startup(this, new Windows.UI.Xaml.StartupEventArgs());
                    }

                    this.OnLaunched(new Windows.ApplicationModel.Activation.LaunchActivatedEventArgs());
                }));
            }
        },
        methods: {
            /**
             * Occurs when an otherwise unhandled Tap interaction occurs over the hit test
             area of this element.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Application
             * @event Windows.UI.Xaml.Application#Exit
             * @return  {System.EventHandler}
             */
            addExit: function (value) {
                this.ExitEventManager.Add(value);
            },
            /**
             * Occurs when an otherwise unhandled Tap interaction occurs over the hit test
             area of this element.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Application
             * @event Windows.UI.Xaml.Application#Exit
             * @return  {System.EventHandler}
             */
            removeExit: function (value) {
                this.ExitEventManager.Remove(value);
            },
            MainWindow_SizeChanged: function (sender, e) {
                this.INTERNAL_WorkAroundFirefoxDisappearingUI();
            },
            INTERNAL_WorkAroundFirefoxDisappearingUI: function () {
                var $t;
                if (this._childWindowForWorkaroundFireFoxIssue == null) {
                    this._childWindowForWorkaroundFireFoxIssue = ($t = new Windows.UI.Xaml.Controls.ChildWindow(), $t.Opacity = 0, $t);
                    this._childWindowForWorkaroundFireFoxIssue.Show();
                    this._timerForWorkaroundFireFoxIssue.Start();
                }
            },
            TimerForWorkaroundFireFoxIssue_Tick: function (sender, e) {
                this._childWindowForWorkaroundFireFoxIssue.Dispatcher.BeginInvoke(Bridge.fn.bind(this, function () {
                    this._timerForWorkaroundFireFoxIssue.Stop();
                    this._childWindowForWorkaroundFireFoxIssue.Close();
                    this._childWindowForWorkaroundFireFoxIssue = null;
                }));
            },
            /**
             * Invoked when the application is launched. Override this method to perform
             application initialization and to display initial content in the associated
             Window.
             *
             * @instance
             * @protected
             * @this Windows.UI.Xaml.Application
             * @memberof Windows.UI.Xaml.Application
             * @param   {Windows.ApplicationModel.Activation.LaunchActivatedEventArgs}    args    Event data for the event.
             * @return  {void}
             */
            OnLaunched: function (args) { },
            /**
             * Occurs just before an application shuts down and cannot be canceled.
             *
             * @instance
             * @this Windows.UI.Xaml.Application
             * @memberof Windows.UI.Xaml.Application
             * @return  {System.Object}
             */
            GetWindow: function () {
                return window;
            },
            /**
             * Searches for the specified resource.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Application
             * @memberof Windows.UI.Xaml.Application
             * @param   {System.Object}    resourceKey    The name of the resource to find.
             * @return  {System.Object}                   The requested resource object. If the requested resource is not found, a
             null reference is returned.
             */
            TryFindResource: function (resourceKey) {
                return this._xamlResourcesHandler.TryFindResource(resourceKey);
            },
            /**
             * Raises the Exit event
             *
             * @instance
             * @private
             * @this Windows.UI.Xaml.Application
             * @memberof Windows.UI.Xaml.Application
             * @param   {System.Object}    jsEventArg
             * @return  {void}
             */
            ProcessOnExit: function (jsEventArg) {
                var eventArgs = { };
                this.OnExit(eventArgs);
            },
            /**
             * Raises the Exit event
             *
             * @instance
             * @protected
             * @this Windows.UI.Xaml.Application
             * @memberof Windows.UI.Xaml.Application
             * @param   {System.Object}    eventArgs    The arguments for the event.
             * @return  {void}
             */
            OnExit: function (eventArgs) {
                var $t;
                $t = Bridge.getEnumerator(this._ExitEventManager.Handlers);
                try {
                    while ($t.moveNext()) {
                        var handler = $t.Current;
                        handler(this, eventArgs);
                    }
                } finally {
                    if (Bridge.is($t, System.IDisposable)) {
                        $t.System$IDisposable$Dispose();
                    }
                }
            }
        }
    });

    Bridge.define("Windows.UI.Xaml.Content", {
        props: {
            /**
             * Gets the browser-determined height of the content area.
             *
             * @instance
             * @public
             * @readonly
             * @memberof Windows.UI.Xaml.Content
             * @function ActualHeight
             * @type number
             */
            ActualHeight: {
                get: function () {
                    return Windows.UI.Xaml.Application.Current.MainWindow.ActualHeight;
                }
            },
            /**
             * Gets the browser-determined width of the Silverlight content area.
             *
             * @instance
             * @public
             * @readonly
             * @memberof Windows.UI.Xaml.Content
             * @function ActualWidth
             * @type number
             */
            ActualWidth: {
                get: function () {
                    return Windows.UI.Xaml.Application.Current.MainWindow.ActualWidth;
                }
            },
            /**
             * Gets or sets a value that indicates whether the page is displaying in full-screen mode.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Content
             * @function IsFullScreen
             * @type boolean
             */
            IsFullScreen: {
                get: function () {
                    return System.Convert.toBoolean(
                    (function(){
                        if (window.IE_VERSION)
                        {
                            // Internet Explorer:
                            return (window.screenTop == 0);
                        }
                        else
                        {
                            // Other browsers:
                            return (window.innerHeight == screen.height);
                        }
                    }()));
                },
                set: function (value) {
                    if (value) {
                        if (!CSHTML5.Interop.IsRunningInTheSimulator) {
                            
var element = document.body;
var requestMethod = element.requestFullScreen || element.webkitRequestFullScreen || element.mozRequestFullScreen || element.msRequestFullScreen;
if (requestMethod) { // Native full screen.
    requestMethod.call(element);
} else if (typeof window.ActiveXObject !== 'undefined') { // Older IE (< 11)
    var isFullScreen = (window.screenTop == 0);
    if (!isFullScreen)
    {
        var wscript = new ActiveXObject('WScript.Shell');
        if (wscript !== null) {
            wscript.SendKeys('{F11}');
        }
    }
};
                        } else {
                            Windows.UI.Xaml.MessageBox.Show("Full-screen mode is not supported when running inside the Simulator. Please launch the application in the browser instead.");
                        }
                    } else {
                        
var requestMethod = document.exitFullScreen || document.webkitExitFullScreen || document.webkitCancelFullScreen || document.mozCancelFullScreen || document.msExitFullScreen || document.msCancelFullScreen;
if (requestMethod) { // Native exit full screen.
    requestMethod.call(document);
} else if (typeof window.ActiveXObject !== 'undefined') { // Older IE (< 11)
    var isFullScreen = (window.screenTop == 0);
    if (isFullScreen)
    {
        var wscript = new ActiveXObject('WScript.Shell');
        if (wscript !== null) {
            wscript.SendKeys('{F11}');
        }
    }
};
                    }
                }
            }
        }
    });

    /**
     * Specifies how text in the text box portion of the AutoCompleteBox
     control is used to filter items specified by the AutoCompleteBox.ItemsSource
     property for display in the drop-down.
     *
     * @public
     * @class Windows.UI.Xaml.Controls.AutoCompleteFilterMode
     */
    Bridge.define("Windows.UI.Xaml.Controls.AutoCompleteFilterMode", {
        $kind: "enum",
        statics: {
            fields: {
                /**
                 * Specifies that no filter is used. All items are returned.
                 *
                 * @static
                 * @public
                 * @memberof Windows.UI.Xaml.Controls.AutoCompleteFilterMode
                 * @constant
                 * @default 0
                 * @type Windows.UI.Xaml.Controls.AutoCompleteFilterMode
                 */
                None: 0,
                /**
                 * Specifies a culture-sensitive, case-insensitive filter where the returned items
                 start with the specified text. The filter uses the {@link }
                 method.
                 *
                 * @static
                 * @public
                 * @memberof Windows.UI.Xaml.Controls.AutoCompleteFilterMode
                 * @constant
                 * @default 1
                 * @type Windows.UI.Xaml.Controls.AutoCompleteFilterMode
                 */
                StartsWith: 1,
                /**
                 * Specifies a culture-sensitive, case-sensitive filter where the returned items
                 start with the specified text. The filter uses the {@link }
                 method.
                 *
                 * @static
                 * @public
                 * @memberof Windows.UI.Xaml.Controls.AutoCompleteFilterMode
                 * @constant
                 * @default 2
                 * @type Windows.UI.Xaml.Controls.AutoCompleteFilterMode
                 */
                StartsWithCaseSensitive: 2,
                /**
                 * Specifies an ordinal, case-insensitive filter where the returned items start
                 with the specified text. The filter uses the {@link }
                 method.
                 *
                 * @static
                 * @public
                 * @memberof Windows.UI.Xaml.Controls.AutoCompleteFilterMode
                 * @constant
                 * @default 3
                 * @type Windows.UI.Xaml.Controls.AutoCompleteFilterMode
                 */
                StartsWithOrdinal: 3,
                /**
                 * Specifies an ordinal, case-sensitive filter where the returned items start with
                 the specified text. The filter uses the {@link }
                 method.
                 *
                 * @static
                 * @public
                 * @memberof Windows.UI.Xaml.Controls.AutoCompleteFilterMode
                 * @constant
                 * @default 4
                 * @type Windows.UI.Xaml.Controls.AutoCompleteFilterMode
                 */
                StartsWithOrdinalCaseSensitive: 4,
                /**
                 * Specifies a culture-sensitive, case-insensitive filter where the returned items
                 contain the specified text.
                 *
                 * @static
                 * @public
                 * @memberof Windows.UI.Xaml.Controls.AutoCompleteFilterMode
                 * @constant
                 * @default 5
                 * @type Windows.UI.Xaml.Controls.AutoCompleteFilterMode
                 */
                Contains: 5,
                /**
                 * Specifies a culture-sensitive, case-sensitive filter where the returned items
                 contain the specified text.
                 *
                 * @static
                 * @public
                 * @memberof Windows.UI.Xaml.Controls.AutoCompleteFilterMode
                 * @constant
                 * @default 6
                 * @type Windows.UI.Xaml.Controls.AutoCompleteFilterMode
                 */
                ContainsCaseSensitive: 6,
                /**
                 * Specifies an ordinal, case-insensitive filter where the returned items contain
                 the specified text.
                 *
                 * @static
                 * @public
                 * @memberof Windows.UI.Xaml.Controls.AutoCompleteFilterMode
                 * @constant
                 * @default 7
                 * @type Windows.UI.Xaml.Controls.AutoCompleteFilterMode
                 */
                ContainsOrdinal: 7,
                /**
                 * Specifies an ordinal, case-sensitive filter where the returned items contain
                 the specified text.
                 *
                 * @static
                 * @public
                 * @memberof Windows.UI.Xaml.Controls.AutoCompleteFilterMode
                 * @constant
                 * @default 8
                 * @type Windows.UI.Xaml.Controls.AutoCompleteFilterMode
                 */
                ContainsOrdinalCaseSensitive: 8,
                /**
                 * Specifies a culture-sensitive, case-insensitive filter where the returned items
                 equal the specified text. The filter uses the {@link }
                 method.
                 *
                 * @static
                 * @public
                 * @memberof Windows.UI.Xaml.Controls.AutoCompleteFilterMode
                 * @constant
                 * @default 9
                 * @type Windows.UI.Xaml.Controls.AutoCompleteFilterMode
                 */
                Equals: 9,
                /**
                 * Specifies a culture-sensitive, case-sensitive filter where the returned items
                 equal the specified text. The filter uses the {@link }
                 method.
                 *
                 * @static
                 * @public
                 * @memberof Windows.UI.Xaml.Controls.AutoCompleteFilterMode
                 * @constant
                 * @default 10
                 * @type Windows.UI.Xaml.Controls.AutoCompleteFilterMode
                 */
                EqualsCaseSensitive: 10,
                /**
                 * Specifies an ordinal, case-insensitive filter where the returned items equal
                 the specified text. The filter uses the {@link }
                 method.
                 *
                 * @static
                 * @public
                 * @memberof Windows.UI.Xaml.Controls.AutoCompleteFilterMode
                 * @constant
                 * @default 11
                 * @type Windows.UI.Xaml.Controls.AutoCompleteFilterMode
                 */
                EqualsOrdinal: 11,
                /**
                 * Specifies an ordinal, case-sensitive filter where the returned items equal the
                 specified text. The filter uses the {@link }
                 method.
                 *
                 * @static
                 * @public
                 * @memberof Windows.UI.Xaml.Controls.AutoCompleteFilterMode
                 * @constant
                 * @default 12
                 * @type Windows.UI.Xaml.Controls.AutoCompleteFilterMode
                 */
                EqualsOrdinalCaseSensitive: 12,
                /**
                 * Specifies that a custom filter is used. This mode is used when the {@link }
                 or {@link } properties are set.
                 *
                 * @static
                 * @public
                 * @memberof Windows.UI.Xaml.Controls.AutoCompleteFilterMode
                 * @constant
                 * @default 13
                 * @type Windows.UI.Xaml.Controls.AutoCompleteFilterMode
                 */
                Custom: 13
            }
        }
    });

    /** @namespace Windows.UI.Xaml.Data */

    /**
     * Exposes methods that allow the data to be modified as it passes through the binding engine.
     *
     * @abstract
     * @public
     * @class Windows.UI.Xaml.Data.IValueConverter
     */
    Bridge.define("Windows.UI.Xaml.Data.IValueConverter", {
        $kind: "interface"
    });

    /**
     * Specifies when the Click event should be raised for a control.
     *
     * @public
     * @class Windows.UI.Xaml.Controls.ClickMode
     */
    Bridge.define("Windows.UI.Xaml.Controls.ClickMode", {
        $kind: "enum",
        statics: {
            fields: {
                /**
                 * Specifies that the Click event
                 should be raised when the left mouse button is pressed and released, and
                 the mouse pointer is over the control.
                 *
                 * @static
                 * @public
                 * @memberof Windows.UI.Xaml.Controls.ClickMode
                 * @constant
                 * @default 0
                 * @type Windows.UI.Xaml.Controls.ClickMode
                 */
                Release: 0,
                /**
                 * Specifies that the Click event
                 should be raised when the mouse button is pressed and the mouse pointer is
                 over the control.
                 *
                 * @static
                 * @public
                 * @memberof Windows.UI.Xaml.Controls.ClickMode
                 * @constant
                 * @default 1
                 * @type Windows.UI.Xaml.Controls.ClickMode
                 */
                Press: 1
            }
        }
    });

    /**
     * A set of extension methods for manipulating collections.
     *
     * @static
     * @abstract
     * @class Windows.UI.Xaml.Controls.CollectionHelper
     */
    Bridge.define("Windows.UI.Xaml.Controls.CollectionHelper", {
        statics: {
            methods: {
                /**
                 * Returns a value indicating whether a collection is read-only.
                 *
                 * @static
                 * @public
                 * @this Windows.UI.Xaml.Controls.CollectionHelper
                 * @memberof Windows.UI.Xaml.Controls.CollectionHelper
                 * @param   {System.Collections.IEnumerable}    collection    The collection to examine.
                 * @return  {boolean}                                         A value indicating whether a collection is read-only.
                 */
                IsReadOnly: function (collection) {
                    return Bridge.isArray(null, Bridge.getType(collection)) || System.Linq.Enumerable.from(System.Linq.Enumerable.from(Windows.UI.Xaml.Controls.EnumerableExtensions.Iterate(System.Type, Bridge.getType(collection), function (type) {
                                return Bridge.Reflection.getBaseType(type);
                            })).takeWhile(function (type) {
                                return type != null;
                            })).any(function (type) {
                            return System.String.startsWith(Bridge.Reflection.getTypeFullName(type), "System.Collections.ObjectModel.ReadOnlyCollection`1", 4);
                        });
                },
                /**
                 * Returns a value Indicating whether an item can be inserted in a 
                 collection.
                 *
                 * @static
                 * @public
                 * @this Windows.UI.Xaml.Controls.CollectionHelper
                 * @memberof Windows.UI.Xaml.Controls.CollectionHelper
                 * @param   {System.Collections.IEnumerable}    collection    The collection.
                 * @param   {System.Object}                     item          The item to be inserted.
                 * @return  {boolean}                                         A value Indicating whether an item can be inserted in a 
                 collection.
                 */
                CanInsert: function (collection, item) {
                    var $t;

                    if (Windows.UI.Xaml.Controls.CollectionHelper.IsReadOnly(collection)) {
                        return false;
                    }

                    var genericListType = System.Linq.Enumerable.from(System.Linq.Enumerable.from(Bridge.Reflection.getInterfaces(Bridge.getType(collection))).where(function (interfaceType) {
                                return System.String.startsWith(Bridge.Reflection.getTypeFullName(interfaceType), "System.Collections.Generic.IList`1", 4);
                            })).firstOrDefault(null, null);
                    if (genericListType != null) {
                        return Bridge.referenceEquals(($t = Bridge.Reflection.getGenericArguments(genericListType))[System.Array.index(0, $t)], Bridge.getType(item));
                    }

                    if (Bridge.is(collection, System.Collections.IList)) {
                        return true;
                    }
                    return false;
                },
                /**
                 * Inserts an item into the collection at an index.
                 *
                 * @static
                 * @public
                 * @this Windows.UI.Xaml.Controls.CollectionHelper
                 * @memberof Windows.UI.Xaml.Controls.CollectionHelper
                 * @param   {System.Collections.IEnumerable}    collection    The collection.
                 * @param   {number}                            index         The index at which to insert the item.
                 * @param   {System.Object}                     item          The item to be inserted.
                 * @return  {void}
                 */
                Insert: function (collection, index, item) {
                    var genericListType = System.Linq.Enumerable.from(System.Linq.Enumerable.from(Bridge.Reflection.getInterfaces(Bridge.getType(collection))).where(function (interfaceType) {
                                return System.String.startsWith(Bridge.Reflection.getTypeFullName(interfaceType), "System.Collections.Generic.IList`1", 4);
                            })).firstOrDefault(null, null);
                    if (genericListType != null) {
                        var method = Bridge.Reflection.getMembers(genericListType, 8, 284, "Insert");
                        Bridge.Reflection.midel(method, collection).apply(null, Bridge.unbox(System.Array.init([Bridge.box(index, System.Int32), item], System.Object)));
                    } else {
                        var list = Bridge.as(collection, System.Collections.IList);
                        System.Array.insert(list, index, item, Object);
                    }
                },
                /**
                 * Gets the number of items in the collection.
                 *
                 * @static
                 * @public
                 * @this Windows.UI.Xaml.Controls.CollectionHelper
                 * @memberof Windows.UI.Xaml.Controls.CollectionHelper
                 * @param   {System.Collections.IEnumerable}    collection    The collection.
                 * @return  {number}                                          The number of items in the collection.
                 */
                Count: function (collection) {
                    var genericListType = System.Linq.Enumerable.from(System.Linq.Enumerable.from(Bridge.Reflection.getInterfaces(Bridge.getType(collection))).where(function (interfaceType) {
                                return System.String.startsWith(Bridge.Reflection.getTypeFullName(interfaceType), "System.Collections.Generic.ICollection`1", 4);
                            })).firstOrDefault(null, null);
                    if (genericListType != null) {
                        return System.Nullable.getValue(Bridge.cast(Bridge.unbox(Bridge.Reflection.midel(Bridge.Reflection.getMembers(genericListType, 16, 284, "Count").g, collection).apply(null, Bridge.unbox(System.Array.init([], System.Object))), System.Int32), System.Int32));
                    }

                    var list = Bridge.as(collection, System.Collections.IList);
                    if (list != null) {
                        return System.Array.getCount(list);
                    }

                    return System.Linq.Enumerable.from(System.Linq.Enumerable.from(collection).ofType(System.Object)).count();
                },
                /**
                 * Adds an item to the collection.
                 *
                 * @static
                 * @public
                 * @this Windows.UI.Xaml.Controls.CollectionHelper
                 * @memberof Windows.UI.Xaml.Controls.CollectionHelper
                 * @param   {System.Collections.IEnumerable}    collection    The collection.
                 * @param   {System.Object}                     item          The item to be added.
                 * @return  {void}
                 */
                Add: function (collection, item) {
                    var countProperty = Bridge.Reflection.getMembers(Bridge.getType(collection), 16, 284, "Count");
                    var count = System.Nullable.getValue(Bridge.cast(Bridge.unbox(Bridge.Reflection.midel(countProperty.g, collection).apply(null, Bridge.unbox(System.Array.init([], System.Object))), System.Int32), System.Int32));
                    Windows.UI.Xaml.Controls.CollectionHelper.Insert(collection, count, item);
                },
                /**
                 * Removes an item from the collection.
                 *
                 * @static
                 * @public
                 * @this Windows.UI.Xaml.Controls.CollectionHelper
                 * @memberof Windows.UI.Xaml.Controls.CollectionHelper
                 * @param   {System.Collections.IEnumerable}    collection    The collection.
                 * @param   {System.Object}                     item          The item to be removed.
                 * @return  {void}
                 */
                Remove: function (collection, item) {
                    var genericListType = System.Linq.Enumerable.from(System.Linq.Enumerable.from(Bridge.Reflection.getInterfaces(Bridge.getType(collection))).where(function (interfaceType) {
                                return System.String.startsWith(Bridge.Reflection.getTypeFullName(interfaceType), "System.Collections.Generic.IList`1", 4);
                            })).firstOrDefault(null, null);
                    if (genericListType != null) {
                        var index = System.Nullable.getValue(Bridge.cast(Bridge.unbox(Bridge.Reflection.midel(Bridge.Reflection.getMembers(genericListType, 8, 284, "IndexOf"), collection).apply(null, Bridge.unbox(System.Array.init([item], System.Object))), System.Int32), System.Int32));
                        if (index !== -1) {
                            Bridge.Reflection.midel(Bridge.Reflection.getMembers(genericListType, 8, 284, "RemoveAt"), collection).apply(null, Bridge.unbox(System.Array.init([Bridge.box(index, System.Int32)], System.Object)));
                        }
                    } else {
                        var list = Bridge.as(collection, System.Collections.IList);
                        System.Array.remove(list, item, Object);
                    }
                },
                /**
                 * Removes an item at a given index from the collection.
                 *
                 * @static
                 * @public
                 * @this Windows.UI.Xaml.Controls.CollectionHelper
                 * @memberof Windows.UI.Xaml.Controls.CollectionHelper
                 * @param   {System.Collections.IEnumerable}    collection    The collection.
                 * @param   {number}                            index         The index of the item to be removed.
                 * @return  {void}
                 */
                RemoveAt: function (collection, index) {
                    var genericListType = System.Linq.Enumerable.from(System.Linq.Enumerable.from(Bridge.Reflection.getInterfaces(Bridge.getType(collection))).where(function (interfaceType) {
                                return System.String.startsWith(Bridge.Reflection.getTypeFullName(interfaceType), "System.Collections.Generic.IList`1", 4);
                            })).firstOrDefault(null, null);
                    if (genericListType != null) {
                        Bridge.Reflection.midel(Bridge.Reflection.getMembers(genericListType, 8, 284, "RemoveAt"), collection).apply(null, Bridge.unbox(System.Array.init([Bridge.box(index, System.Int32)], System.Object)));
                    } else {
                        var list = Bridge.as(collection, System.Collections.IList);
                        System.Array.removeAt(list, index, Object);
                    }
                }
            }
        }
    });

    Bridge.define("Windows.UI.Xaml.Controls.ContextMenuService", {
        statics: {
            fields: {
                /**
                 * The DependencyProperty for the ContextMenu property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Controls.ContextMenuService
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                ContextMenuProperty: null
            },
            ctors: {
                init: function () {
                    this.ContextMenuProperty = Windows.UI.Xaml.DependencyProperty.Register("ContextMenu", Windows.UI.Xaml.Controls.ContextMenu, Windows.UI.Xaml.Controls.ContextMenuService, new Windows.UI.Xaml.PropertyMetadata.$ctor4(null));
                }
            },
            methods: {
                /**
                 * Gets the value of the ContextMenu property on the specified object.
                 *
                 * @static
                 * @public
                 * @this Windows.UI.Xaml.Controls.ContextMenuService
                 * @memberof Windows.UI.Xaml.Controls.ContextMenuService
                 * @param   {Windows.UI.Xaml.DependencyObject}        element    The object on which to query the ContextMenu property.
                 * @return  {Windows.UI.Xaml.Controls.ContextMenu}               The value of the ContextMenu property.
                 */
                GetContextMenu: function (element) {
                    return Bridge.cast(element, Windows.UI.Xaml.FrameworkElement).ContextMenu;
                },
                /**
                 * Sets the ContextMenu property on the specified object.
                 *
                 * @static
                 * @public
                 * @this Windows.UI.Xaml.Controls.ContextMenuService
                 * @memberof Windows.UI.Xaml.Controls.ContextMenuService
                 * @param   {Windows.UI.Xaml.DependencyObject}        element    The object on which to set the ContextMenu property.
                 * @param   {Windows.UI.Xaml.Controls.ContextMenu}    value      The value of the ContextMenu property. If the value is of type ContextMenu, then
                     that is the ContextMenu that will be used (without any modification). If the value
                     is of any other type, then that value will be used as the content for a ContextMenu
                     provided by this service, and the other attached properties of this service
                     will be used to configure the ContextMenu.
                 * @return  {void}
                 */
                SetContextMenu: function (element, value) {
                    Bridge.cast(element, Windows.UI.Xaml.FrameworkElement).ContextMenu = value;
                }
            }
        }
    });

    /**
     * Represents the lengths of elements within the System.Windows.Controls.DataGrid
     control.
     *
     * @public
     * @class Windows.UI.Xaml.Controls.DataGridLength
     * @implements  System.IEquatable$1
     */
    Bridge.define("Windows.UI.Xaml.Controls.DataGridLength", {
        inherits: function () { return [System.IEquatable$1(Windows.UI.Xaml.Controls.DataGridLength)]; },
        $kind: "struct",
        statics: {
            props: {
                /**
                 * Gets a System.Windows.Controls.DataGridLength structure that represents the
                 standard automatic sizing mode.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Controls.DataGridLength
                 * @function Auto
                 * @type Windows.UI.Xaml.Controls.DataGridLength
                 */
                Auto: {
                    get: function () {
                        return new Windows.UI.Xaml.Controls.DataGridLength.$ctor2(1.0, Windows.UI.Xaml.Controls.DataGridLengthUnitType.Auto);
                    }
                }
            },
            ctors: {
                ctor: function () {
                    DotNetForHtml5.Core.TypeFromStringConverters.RegisterConverter(Windows.UI.Xaml.Controls.DataGridLength, Windows.UI.Xaml.Controls.DataGridLength.INTERNAL_ConvertFromString);
                }
            },
            methods: {
                INTERNAL_ConvertFromString: function (gridLengthAsString) {
                    var trimmedLowercase = gridLengthAsString.trim().toLowerCase();
                    if (System.String.endsWith(trimmedLowercase, "*")) {
                        var valueAsString = trimmedLowercase.substr(0, ((trimmedLowercase.length - 1) | 0));
                        var value = { };
                        if (Bridge.referenceEquals(valueAsString, "")) {
                            return new Windows.UI.Xaml.Controls.DataGridLength.$ctor2(1.0, Windows.UI.Xaml.Controls.DataGridLengthUnitType.Star).$clone();
                        }
                        if (System.Double.tryParse(valueAsString, null, value)) {
                            return new Windows.UI.Xaml.Controls.DataGridLength.$ctor2(value.v, Windows.UI.Xaml.Controls.DataGridLengthUnitType.Star).$clone();
                        } else {
                            throw new System.Exception("Invalid GridLength: " + (gridLengthAsString || ""));
                        }
                    } else if (Bridge.referenceEquals(trimmedLowercase, "auto")) {
                        return new Windows.UI.Xaml.Controls.DataGridLength.$ctor2(1.0, Windows.UI.Xaml.Controls.DataGridLengthUnitType.Auto).$clone();
                    } else {
                        if (Bridge.referenceEquals(trimmedLowercase, "sizetocells")) {
                            return new Windows.UI.Xaml.Controls.DataGridLength.$ctor2(1.0, Windows.UI.Xaml.Controls.DataGridLengthUnitType.Auto).$clone();
                        } else {
                            if (Bridge.referenceEquals(trimmedLowercase, "sizetoheader")) {
                                return new Windows.UI.Xaml.Controls.DataGridLength.$ctor2(1.0, Windows.UI.Xaml.Controls.DataGridLengthUnitType.Auto).$clone();
                            } else {
                                var value1 = { };
                                if (System.Double.tryParse(trimmedLowercase, null, value1)) {
                                    return new Windows.UI.Xaml.Controls.DataGridLength.$ctor2(value1.v, Windows.UI.Xaml.Controls.DataGridLengthUnitType.Pixel).$clone();
                                } else {
                                    throw new System.Exception("Invalid GridLength: " + (gridLengthAsString || ""));
                                }
                            }
                        }
                    }
                }/**
                 * Compares two System.Windows.Controls.DataGridLength structures to determine
                 whether they are not equal.
                 *
                 * @static
                 * @public
                 * @this Windows.UI.Xaml.Controls.DataGridLength
                 * @memberof Windows.UI.Xaml.Controls.DataGridLength
                 * @param   {Windows.UI.Xaml.Controls.DataGridLength}    gl1    The first System.Windows.Controls.DataGridLength instance to compare.
                 * @param   {Windows.UI.Xaml.Controls.DataGridLength}    gl2    The second System.Windows.Controls.DataGridLength instance to compare.
                 * @return  {boolean}                                           true if the two System.Windows.Controls.DataGridLength instances do not have
                 the same value or sizing mode; otherwise, false.
                 */
                ,
                op_Inequality: function (gl1, gl2) {
                    return (gl1.UnitType !== gl2.UnitType || gl1.Value !== gl2.Value);
                }/**
                 * Compares two System.Windows.Controls.DataGridLength structures for equality.
                 *
                 * @static
                 * @public
                 * @this Windows.UI.Xaml.Controls.DataGridLength
                 * @memberof Windows.UI.Xaml.Controls.DataGridLength
                 * @param   {Windows.UI.Xaml.Controls.DataGridLength}    gl1    The first System.Windows.Controls.DataGridLength instance to compare.
                 * @param   {Windows.UI.Xaml.Controls.DataGridLength}    gl2    The second System.Windows.Controls.DataGridLength instance to compare.
                 * @return  {boolean}                                           true if the two System.Windows.Controls.DataGridLength instances have the
                 same value or sizing mode; otherwise, false.
                 */
                ,
                op_Equality: function (gl1, gl2) {
                    return (gl1.UnitType === gl2.UnitType && gl1.Value === gl2.Value);

                }/**
                 * Converts a System.Double to an instance of the System.Windows.Controls.DataGridLength
                 class.
                 *
                 * @static
                 * @public
                 * @this Windows.UI.Xaml.Controls.DataGridLength
                 * @memberof Windows.UI.Xaml.Controls.DataGridLength
                 * @param   {number}                                     value    The absolute pixel value (96 pixels-per-inch) to initialize the length to.
                 * @return  {Windows.UI.Xaml.Controls.DataGridLength}             An object that represents the specified length.
                 */
                ,
                op_Implicit: function (value) {
                    return new Windows.UI.Xaml.Controls.DataGridLength.$ctor1(value);
                },
                getDefaultValue: function () { return new Windows.UI.Xaml.Controls.DataGridLength(); }
            }
        },
        fields: {
            _value: 0,
            _type: 0
        },
        props: {
            /**
             * Gets a value that indicates whether this instance sizes elements based on
             a fixed pixel value.
             *
             * @instance
             * @public
             * @readonly
             * @memberof Windows.UI.Xaml.Controls.DataGridLength
             * @function IsAbsolute
             * @type boolean
             */
            IsAbsolute: {
                get: function () {
                    return this._type === Windows.UI.Xaml.Controls.DataGridLengthUnitType.Pixel;
                }
            },
            /**
             * Gets a value that indicates whether this instance automatically sizes elements
             based on both the content of cells and the column headers.
             *
             * @instance
             * @public
             * @readonly
             * @memberof Windows.UI.Xaml.Controls.DataGridLength
             * @function IsAuto
             * @type boolean
             */
            IsAuto: {
                get: function () {
                    return this._type === Windows.UI.Xaml.Controls.DataGridLengthUnitType.Auto;
                }
            },
            /**
             * Gets a value that indicates whether this instance automatically sizes elements
             based on a weighted proportion of available space.
             *
             * @instance
             * @public
             * @readonly
             * @memberof Windows.UI.Xaml.Controls.DataGridLength
             * @function IsStar
             * @type boolean
             */
            IsStar: {
                get: function () {
                    return this._type === Windows.UI.Xaml.Controls.DataGridLengthUnitType.Star;
                }
            },
            /**
             * Gets the type that is used to determine how the size of the element is calculated.
             *
             * @instance
             * @public
             * @readonly
             * @memberof Windows.UI.Xaml.Controls.DataGridLength
             * @function UnitType
             * @type Windows.UI.Xaml.Controls.DataGridLengthUnitType
             */
            UnitType: {
                get: function () {
                    return this._type;
                }
            },
            /**
             * Gets the absolute value of the System.Windows.Controls.DataGridLength in
             pixels, or 1.0 if the System.Windows.Controls.DataGridLength.UnitType property is
             set to System.Windows.Controls.DataGridLengthUnitType.Auto.
             *
             * @instance
             * @public
             * @readonly
             * @memberof Windows.UI.Xaml.Controls.DataGridLength
             * @function Value
             * @type number
             */
            Value: {
                get: function () {
                    return this._value;
                }
            }
        },
        alias: ["equalsT", "System$IEquatable$1$Windows$UI$Xaml$Controls$DataGridLength$equalsT"],
        ctors: {
            /**
             * Initializes a new instance of the System.Windows.Controls.DataGridLength
             class with an absolute value in pixels.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Controls.DataGridLength
             * @memberof Windows.UI.Xaml.Controls.DataGridLength
             * @param   {number}    pixels    The absolute pixel value (96 pixels-per-inch) to initialize the length to.
             * @return  {void}
             */
            $ctor1: function (pixels) {
                this.$initialize();
                this._value = pixels;
                this._type = Windows.UI.Xaml.Controls.DataGridLengthUnitType.Pixel;
            },
            /**
             * Initializes a new instance of the System.Windows.Controls.DataGridLength
             class with a specified value and unit.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Controls.DataGridLength
             * @memberof Windows.UI.Xaml.Controls.DataGridLength
             * @param   {number}                                             value    The requested size of the element.
             * @param   {Windows.UI.Xaml.Controls.DataGridLengthUnitType}    type     The type that is used to determine how the size of the element is calculated.
             * @return  {void}
             */
            $ctor2: function (value, type) {
                this.$initialize();
                this._value = (type === Windows.UI.Xaml.Controls.DataGridLengthUnitType.Auto ? 0.0 : value);
                this._type = type;
            },
            ctor: function () {
                this.$initialize();
            }
        },
        methods: {
            /**
             * Determines whether the specified System.Windows.Controls.DataGridLength is
             equal to the current System.Windows.Controls.DataGridLength.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Controls.DataGridLength
             * @memberof Windows.UI.Xaml.Controls.DataGridLength
             * @param   {Windows.UI.Xaml.Controls.DataGridLength}    other    The System.Windows.Controls.DataGridLength to compare to the current instance.
             * @return  {boolean}                                             true if the specified object is a System.Windows.Controls.DataGridLength
             with the same value or sizing mode as the current System.Windows.Controls.DataGridLength;
             otherwise, false.
             */
            equalsT: function (other) {
                return Windows.UI.Xaml.Controls.DataGridLength.op_Equality(this, other.$clone());
            },
            /**
             * Determines whether the specified object is equal to the current System.Windows.Controls.DataGridLength.
             *
             * @instance
             * @public
             * @override
             * @this Windows.UI.Xaml.Controls.DataGridLength
             * @memberof Windows.UI.Xaml.Controls.DataGridLength
             * @param   {System.Object}    obj    The object to compare to the current instance.
             * @return  {boolean}                 true if the specified object is a System.Windows.Controls.DataGridLength
             with the same value or sizing mode as the current System.Windows.Controls.DataGridLength;
             otherwise, false.
             */
            equals: function (obj) {
                return ((Bridge.is(obj, Windows.UI.Xaml.Controls.DataGridLength)) && (Windows.UI.Xaml.Controls.DataGridLength.op_Equality(this, System.Nullable.getValue(Bridge.cast(Bridge.unbox(obj, Windows.UI.Xaml.Controls.DataGridLength), Windows.UI.Xaml.Controls.DataGridLength)))));
            },
            /**
             * Gets a hash code for the System.Windows.Controls.DataGridLength.
             *
             * @instance
             * @public
             * @override
             * @this Windows.UI.Xaml.Controls.DataGridLength
             * @memberof Windows.UI.Xaml.Controls.DataGridLength
             * @return  {number}        A hash code for the current System.Windows.Controls.DataGridLength.
             */
            getHashCode: function () {
                return ((Bridge.Int.mul(Bridge.Int.clip32(this._value), 7) + this._type) | 0);
            },
            /**
             * Returns a string that represents the current object.
             *
             * @instance
             * @public
             * @override
             * @this Windows.UI.Xaml.Controls.DataGridLength
             * @memberof Windows.UI.Xaml.Controls.DataGridLength
             * @return  {string}        A string that represent the current object.
             */
            toString: function () {
                if (this._type === Windows.UI.Xaml.Controls.DataGridLengthUnitType.Auto) {
                    return "Auto";
                } else if (this._type === Windows.UI.Xaml.Controls.DataGridLengthUnitType.Star) {
                    return (this._value === 1 ? "*" : (System.Double.format(this._value) || "") + "*");
                } else {
                    return System.Double.format(this._value);
                }
            },
            $clone: function (to) {
                var s = to || new Windows.UI.Xaml.Controls.DataGridLength();
                s._value = this._value;
                s._type = this._type;
                return s;
            }
        }
    });

    /**
     * Defines constants that specify how elements in a System.Windows.Controls.DataGrid
     are sized. Note: SizeToCells and SizeToHeader are not available yet.
     *
     * @public
     * @class Windows.UI.Xaml.Controls.DataGridLengthUnitType
     */
    Bridge.define("Windows.UI.Xaml.Controls.DataGridLengthUnitType", {
        $kind: "enum",
        statics: {
            fields: {
                /**
                 * The size is based on the contents of both the cells and the column header.
                 *
                 * @static
                 * @public
                 * @memberof Windows.UI.Xaml.Controls.DataGridLengthUnitType
                 * @constant
                 * @default 0
                 * @type Windows.UI.Xaml.Controls.DataGridLengthUnitType
                 */
                Auto: 0,
                /**
                 * The size is a fixed value expressed in pixels.
                 *
                 * @static
                 * @public
                 * @memberof Windows.UI.Xaml.Controls.DataGridLengthUnitType
                 * @constant
                 * @default 1
                 * @type Windows.UI.Xaml.Controls.DataGridLengthUnitType
                 */
                Pixel: 1,
                /**
                 * @static
                 * @public
                 * @memberof Windows.UI.Xaml.Controls.DataGridLengthUnitType
                 * @constant
                 * @default 4
                 * @type Windows.UI.Xaml.Controls.DataGridLengthUnitType
                 */
                Star: 4
            }
        }
    });

    Bridge.define("Windows.UI.Xaml.Controls.DataGridRow.ObjectRepresentationInRow", {
        $kind: "nested class",
        fields: {
            ElementsInRow: null,
            RowDefinition: null
        },
        ctors: {
            init: function () {
                this.ElementsInRow = new (System.Collections.Generic.List$1(Windows.UI.Xaml.Controls.DataGridCell)).ctor();
            }
        }
    });

    Bridge.define("Windows.UI.Xaml.Controls.DataGridRowEventArgs", {
        fields: {
            Row: null
        },
        ctors: {
            ctor: function (row) {
                this.$initialize();
                System.Object.call(this);
                this.Row = row;
            }
        }
    });

    /**
     * Defines constants that specify whether single or multiple item selections
     are supported by a System.Windows.Controls.DataGrid control.
     *
     * @public
     * @class Windows.UI.Xaml.Controls.DataGridSelectionMode
     */
    Bridge.define("Windows.UI.Xaml.Controls.DataGridSelectionMode", {
        $kind: "enum",
        statics: {
            fields: {
                /**
                 * Only one item in the System.Windows.Controls.DataGrid can be selected at
                 a time.
                 *
                 * @static
                 * @public
                 * @memberof Windows.UI.Xaml.Controls.DataGridSelectionMode
                 * @constant
                 * @default 0
                 * @type Windows.UI.Xaml.Controls.DataGridSelectionMode
                 */
                Single: 0,
                /**
                 * Multiple items in the System.Windows.Controls.DataGrid can be selected at
                 the same time.
                 *
                 * @static
                 * @public
                 * @memberof Windows.UI.Xaml.Controls.DataGridSelectionMode
                 * @constant
                 * @default 1
                 * @type Windows.UI.Xaml.Controls.DataGridSelectionMode
                 */
                Extended: 1
            }
        }
    });

    Bridge.define("Windows.UI.Xaml.Controls.DataPager_DisplayMode", {
        $kind: "enum",
        statics: {
            fields: {
                FirstLastPreviousNextNumeric: 0,
                FirstLastNumeric: 1,
                FirstLastPreviousNext: 2,
                Numeric: 3,
                PreviousNext: 4,
                PreviousNextNumeric: 5
            }
        }
    });

    /**
     * Specifies the Dock position of a child element that
     is inside a DockPanel.
     *
     * @public
     * @class Windows.UI.Xaml.Controls.Dock
     */
    Bridge.define("Windows.UI.Xaml.Controls.Dock", {
        $kind: "enum",
        statics: {
            fields: {
                /**
                 * A child element that is positioned on the left side of the DockPanel.
                 *
                 * @static
                 * @public
                 * @memberof Windows.UI.Xaml.Controls.Dock
                 * @constant
                 * @default 0
                 * @type Windows.UI.Xaml.Controls.Dock
                 */
                Left: 0,
                /**
                 * A child element that is positioned at the top of the DockPanel.
                 *
                 * @static
                 * @public
                 * @memberof Windows.UI.Xaml.Controls.Dock
                 * @constant
                 * @default 1
                 * @type Windows.UI.Xaml.Controls.Dock
                 */
                Top: 1,
                /**
                 * A child element that is positioned on the right side of the DockPanel.
                 *
                 * @static
                 * @public
                 * @memberof Windows.UI.Xaml.Controls.Dock
                 * @constant
                 * @default 2
                 * @type Windows.UI.Xaml.Controls.Dock
                 */
                Right: 2,
                /**
                 * A child element that is positioned at the bottom of the DockPanel.
                 *
                 * @static
                 * @public
                 * @memberof Windows.UI.Xaml.Controls.Dock
                 * @constant
                 * @default 3
                 * @type Windows.UI.Xaml.Controls.Dock
                 */
                Bottom: 3
            }
        }
    });

    /**
     * Extension methods for Enumerable.
     *
     * @static
     * @abstract
     * @class Windows.UI.Xaml.Controls.EnumerableExtensions
     */
    Bridge.define("Windows.UI.Xaml.Controls.EnumerableExtensions", {
        statics: {
            methods: {
                /**
                 * Creates a sequence from an integer, to an integer, skipping 
                 a defined amount of integers.
                 *
                 * @static
                 * @public
                 * @this Windows.UI.Xaml.Controls.EnumerableExtensions
                 * @memberof Windows.UI.Xaml.Controls.EnumerableExtensions
                 * @param   {number}                                      from    The value of the first integer in the sequence.
                 * @param   {number}                                      to      The inclusive end of the sequence.
                 * @param   {number}                                      by      The amount of integers to skip.
                 * @return  {System.Collections.Generic.IEnumerable$1}            A sequence of integers.
                 */
                Range: function (from, to, by) {
                    return new (Bridge.GeneratorEnumerable$1(System.Int32))(Bridge.fn.bind(this, function (from, to, by) {
                        var $step = 0,
                            $jumpFromFinally,
                            $returnValue,
                            i,
                            i1,
                            $async_e;

                        var $enumerator = new (Bridge.GeneratorEnumerator$1(System.Int32))(Bridge.fn.bind(this, function () {
                            try {
                                for (;;) {
                                    switch ($step) {
                                        case 0: {
                                            if (by <= 0) {
                                                    throw new System.ArgumentOutOfRangeException.$ctor4("by", "Parameter by is expected to be larger than 0");
                                                }

                                                if (from < to) {
                                                    $step = 1;
                                                    continue;
                                                } else  {
                                                    $step = 7;
                                                    continue;
                                                }
                                        }
                                        case 1: {
                                            i = from;
                                                $step = 2;
                                                continue;
                                        }
                                        case 2: {
                                            if ( i <= to ) {
                                                    $step = 3;
                                                    continue;
                                                }
                                            $step = 6;
                                            continue;
                                        }
                                        case 3: {
                                            $enumerator.current = i;
                                                $step = 4;
                                                return true;
                                        }
                                        case 4: {
                                            $step = 5;
                                            continue;
                                        }
                                        case 5: {
                                            i = (i + by) | 0;
                                            $step = 2;
                                            continue;
                                        }
                                        case 6: {
                                            $step = 13;
                                            continue;
                                        }
                                        case 7: {
                                            i1 = from;
                                                $step = 8;
                                                continue;
                                        }
                                        case 8: {
                                            if ( i1 >= to ) {
                                                    $step = 9;
                                                    continue;
                                                }
                                            $step = 12;
                                            continue;
                                        }
                                        case 9: {
                                            $enumerator.current = i1;
                                                $step = 10;
                                                return true;
                                        }
                                        case 10: {
                                            $step = 11;
                                            continue;
                                        }
                                        case 11: {
                                            i1 = (i1 - by) | 0;
                                            $step = 8;
                                            continue;
                                        }
                                        case 12: {
                                            $step = 13;
                                            continue;
                                        }
                                        case 13: {

                                        }
                                        default: {
                                            return false;
                                        }
                                    }
                                }
                            } catch($async_e1) {
                                $async_e = System.Exception.create($async_e1);
                                throw $async_e;
                            }
                        }));
                        return $enumerator;
                    }, arguments));
                },
                /**
                 * Applies an action to each element in an IEnumerable.
                 *
                 * @static
                 * @public
                 * @this Windows.UI.Xaml.Controls.EnumerableExtensions
                 * @memberof Windows.UI.Xaml.Controls.EnumerableExtensions
                 * @param   {Function}                                    T         The Type of the elements.
                 * @param   {System.Collections.Generic.IEnumerable$1}    source    The source sequence.
                 * @param   {System.Action}                               action    The action applied to all the elements.
                 * @return  {void}
                 */
                ForEach: function (T, source, action) {
                    var $t;
                    if (source == null) {
                        return;
                    }

                    $t = Bridge.getEnumerator(source, T);
                    try {
                        while ($t.moveNext()) {
                            var i = $t.Current;
                            action(i);
                        }
                    } finally {
                        if (Bridge.is($t, System.IDisposable)) {
                            $t.System$IDisposable$Dispose();
                        }
                    }
                },
                /**
                 * Produces a sequence of items using a seed value and iteration 
                 method.
                 *
                 * @static
                 * @public
                 * @this Windows.UI.Xaml.Controls.EnumerableExtensions
                 * @memberof Windows.UI.Xaml.Controls.EnumerableExtensions
                 * @param   {Function}                                    T        The type of the sequence.
                 * @param   {T}                                           value    The initial value.
                 * @param   {System.Func}                                 next     The iteration function.
                 * @return  {System.Collections.Generic.IEnumerable$1}             A sequence of items using a seed value and iteration 
                 method.
                 */
                Iterate: function (T, value, next) {
                    return new (Bridge.GeneratorEnumerable$1(T))(Bridge.fn.bind(this, function (T, value, next) {
                        var $step = 0,
                            $jumpFromFinally,
                            $returnValue,
                            $async_e;

                        var $enumerator = new (Bridge.GeneratorEnumerator$1(T))(Bridge.fn.bind(this, function () {
                            try {
                                for (;;) {
                                    switch ($step) {
                                        case 0: {
                                            $enumerator.current = value;
                                                $step = 1;
                                                return true;
                                        }
                                        case 1: {
                                            value = next(value);
                                            $step = 2;
                                            continue;
                                        }
                                        case 2: {
                                            if ( true ) {

                                                    $step = 0;
                                                    continue;
                                                }
                                            $step = 3;
                                            continue;
                                        }
                                        case 3: {

                                        }
                                        default: {
                                            return false;
                                        }
                                    }
                                }
                            } catch($async_e1) {
                                $async_e = System.Exception.create($async_e1);
                                throw $async_e;
                            }
                        }));
                        return $enumerator;
                    }, arguments));
                }
            }
        }
    });

    /**
     * Defines a method for internal navigation in an application.
     *
     * @abstract
     * @public
     * @class Windows.UI.Xaml.Controls.INavigate
     */
    Bridge.define("Windows.UI.Xaml.Controls.INavigate", {
        $kind: "interface"
    });

    /**
     * // Gets or sets the number of pages that can be cached for the frame.
    //
     * // Identifies the System.Windows.Controls.Frame.CacheSize dependency property.
    //
     * // Gets or sets the object responsible for providing the content that corresponds
    // to a requested URI.
    //
     * // Identifies the System.Windows.Controls.Frame.ContentLoader dependency property.
    //
     * // Gets the uniform resource identifier (URI) of the content that is currently
    // displayed.
    //
     * // Identifies the System.Windows.Controls.Frame.CurrentSource dependency property.
    //
     * // Occurs when navigation to a content fragment begins.
    //
     * // Occurs when a new navigation is requested.
    //
     * // Occurs when a navigation is terminated by either calling the System.Windows.Controls.Frame.StopLoading()
    // method, or when a new navigation is requested while the current navigation
    // is in progress.
    //
     * // Builds the visual tree for the System.Windows.Controls.Frame when a new template
    // is applied.
    //
     * // Returns a System.Windows.Automation.Peers.FrameAutomationPeer for use by
    // the Silverlight automation infrastructure.
    //
     * // Reloads the current page.
    //
     * // Stops asynchronous navigations that have not yet been processed.
    //
     * // Gets or sets the maximum amount of objects the cache can remember.
    //
     * // Adds the object right after the current position, and forgets the possible elements that were after it.
    //
     *
     * @class Windows.UI.Xaml.Controls.Frame.FrameCache
     * @return  {[type]}        // A System.Windows.Automation.Peers.FrameAutomationPeer for the System.Windows.Controls.Frame
    // object.
    //
     */
    Bridge.define("Windows.UI.Xaml.Controls.Frame.FrameCache", {
        $kind: "nested class",
        fields: {
            _currentAmount: 0,
            _currentItem: null,
            firstItem: null
        },
        props: {
            CanGoBack: {
                get: function () {
                    if (this._currentItem == null) {
                        return false;
                    } else {
                        return this._currentItem._previous != null;
                    }
                }
            },
            CanGoForward: {
                get: function () {
                    if (this._currentItem == null) {
                        return false;
                    } else {
                        return this._currentItem._next != null;
                    }
                }
            }
        },
        ctors: {
            init: function () {
                this._currentAmount = 0;
            }
        },
        methods: {
            /**
             * Adds the item to the cache, and handles the cases where:
             - it is the first item,
             - we add it to the end
             - we add it somewhere else.
             *
             * @instance
             * @this Windows.UI.Xaml.Controls.Frame.FrameCache
             * @memberof Windows.UI.Xaml.Controls.Frame.FrameCache
             * @param   {System.Object}    next    
             * @param   {System.Uri}       uri
             * @return  {void}
             */
            Add: function (next, uri) {
                var nextFrameCacheItem = new Windows.UI.Xaml.Controls.Frame.FrameCacheItem(next, uri);
                if (this._currentItem != null) {
                    this._currentItem.RemoveNext();
                    this._currentItem._next = nextFrameCacheItem;
                    nextFrameCacheItem._previous = this._currentItem;
                } else {
                    this.firstItem = nextFrameCacheItem;
                }
                this._currentItem = nextFrameCacheItem;
            },
            /**
             * // Adds the item to the cache, and handles the cases where:
            // - it is the first item,
            // - we add it to the end
            // - we add it somewhere else.
            //
             *
             * @instance
             * @this Windows.UI.Xaml.Controls.Frame.FrameCache
             * @memberof Windows.UI.Xaml.Controls.Frame.FrameCache
             * @return  {Windows.UI.Xaml.Controls.Frame.FrameCacheItem}
             */
            GoBack: function () {
                if (this.CanGoBack) {
                    this._currentItem = this._currentItem._previous;
                    return this._currentItem;
                } else {
                    throw new System.InvalidOperationException.$ctor1("There are no entries in the back navigation history.");
                }
            },
            GoForward: function () {
                if (this.CanGoForward) {
                    this._currentItem = this._currentItem._next;
                    return this._currentItem;
                } else {
                    throw new System.InvalidOperationException.$ctor1("There are no entries in the forward navigation history.");
                }
            },
            Clear: function () {
                this.firstItem.RemoveNext();
                this.firstItem = null;
            }
        }
    });

    Bridge.define("Windows.UI.Xaml.Controls.Frame.FrameCacheItem", {
        $kind: "nested class",
        fields: {
            _previous: null,
            _next: null,
            _uri: null,
            _value: null
        },
        ctors: {
            ctor: function (value, uri) {
                this.$initialize();
                this._value = value;
                this._uri = uri;
            }
        },
        methods: {
            /**
             * removes the next element(s) if any.
             *
             * @instance
             * @this Windows.UI.Xaml.Controls.Frame.FrameCacheItem
             * @memberof Windows.UI.Xaml.Controls.Frame.FrameCacheItem
             * @return  {void}
             */
            RemoveNext: function () {
                if (this._next != null) {
                    var next = this._next;
                    this._next = null;
                    next._previous = null;
                    next._value = null;
                    next._uri = null;
                    next.RemoveNext();
                }
            }
        }
    });

    Bridge.define("Windows.UI.Xaml.Controls.Grid_InternalHelpers", {
        statics: {
            fields: {
                INTERNAL_CSSGRID_MS_PREFIX: null,
                _isGridSupported: null,
                _isMSGrid: null
            },
            ctors: {
                init: function () {
                    this.INTERNAL_CSSGRID_MS_PREFIX = "-ms-";
                }
            },
            methods: {
                CalculateNewCellsStructure: function (columnDefinitionsOrNull, rowDefinitionsOrNull) {
                    var $t, $t1, $t2;
                    var output = new (System.Collections.Generic.List$1(System.Collections.Generic.List$1(Windows.UI.Xaml.Controls.INTERNAL_CellDefinition))).ctor();
                    var rowIndex = 0;
                    $t = Bridge.getEnumerator(Windows.UI.Xaml.Controls.Grid_InternalHelpers.IterateThroughCollectionOrCreateNewElementIfCollectionIsNullOrEmpty$1(rowDefinitionsOrNull), Windows.UI.Xaml.Controls.RowDefinition);
                    try {
                        while ($t.moveNext()) {
                            var row = $t.Current;
                            var columnIndex = 0;
                            var currentRow = new (System.Collections.Generic.List$1(Windows.UI.Xaml.Controls.INTERNAL_CellDefinition)).ctor();
                            output.add(currentRow);
                            $t1 = Bridge.getEnumerator(Windows.UI.Xaml.Controls.Grid_InternalHelpers.IterateThroughCollectionOrCreateNewElementIfCollectionIsNullOrEmpty(columnDefinitionsOrNull), Windows.UI.Xaml.Controls.ColumnDefinition);
                            try {
                                while ($t1.moveNext()) {
                                    var column = $t1.Current;
                                    currentRow.add(($t2 = new Windows.UI.Xaml.Controls.INTERNAL_CellDefinition(), $t2.Row = rowIndex, $t2.Column = columnIndex, $t2));
                                    columnIndex = (columnIndex + 1) | 0;
                                }
                            } finally {
                                if (Bridge.is($t1, System.IDisposable)) {
                                    $t1.System$IDisposable$Dispose();
                                }
                            }
                            rowIndex = (rowIndex + 1) | 0;
                        }
                    } finally {
                        if (Bridge.is($t, System.IDisposable)) {
                            $t.System$IDisposable$Dispose();
                        }
                    }

                    return output;
                },
                IterateThroughCollectionOrCreateNewElementIfCollectionIsNullOrEmpty$1: function (rowDefinitionsOrNull) {
                    return new (Bridge.GeneratorEnumerable$1(Windows.UI.Xaml.Controls.RowDefinition))(Bridge.fn.bind(this, function (rowDefinitionsOrNull) {
                        var $step = 0,
                            $jumpFromFinally,
                            $returnValue,
                            $t,
                            row,
                            $async_e;

                        var $enumerator = new (Bridge.GeneratorEnumerator$1(Windows.UI.Xaml.Controls.RowDefinition))(Bridge.fn.bind(this, function () {
                            try {
                                for (;;) {
                                    switch ($step) {
                                        case 0: {
                                            if (rowDefinitionsOrNull != null && rowDefinitionsOrNull.Count > 0) {
                                                    $step = 1;
                                                    continue;
                                                } else  {
                                                    $step = 6;
                                                    continue;
                                                }
                                        }
                                        case 1: {
                                            $t = Bridge.getEnumerator(rowDefinitionsOrNull);
                                                $step = 2;
                                                continue;
                                        }
                                        case 2: {
                                            if ($t.moveNext()) {
                                                    row = $t.Current;
                                                    $step = 3;
                                                    continue;
                                                }
                                            $step = 5;
                                            continue;
                                        }
                                        case 3: {
                                            $enumerator.current = row;
                                                $step = 4;
                                                return true;
                                        }
                                        case 4: {
                                            $step = 2;
                                            continue;
                                        }
                                        case 5: {
                                            $step = 8;
                                            continue;
                                        }
                                        case 6: {
                                            $enumerator.current = new Windows.UI.Xaml.Controls.RowDefinition();
                                                $step = 7;
                                                return true;
                                        }
                                        case 7: {
                                            $step = 8;
                                            continue;
                                        }
                                        case 8: {

                                        }
                                        default: {
                                            return false;
                                        }
                                    }
                                }
                            } catch($async_e1) {
                                $async_e = System.Exception.create($async_e1);
                                throw $async_e;
                            }
                        }));
                        return $enumerator;
                    }, arguments));
                },
                IterateThroughCollectionOrCreateNewElementIfCollectionIsNullOrEmpty: function (columnDefinitionsOrNull) {
                    return new (Bridge.GeneratorEnumerable$1(Windows.UI.Xaml.Controls.ColumnDefinition))(Bridge.fn.bind(this, function (columnDefinitionsOrNull) {
                        var $step = 0,
                            $jumpFromFinally,
                            $returnValue,
                            $t,
                            column,
                            $async_e;

                        var $enumerator = new (Bridge.GeneratorEnumerator$1(Windows.UI.Xaml.Controls.ColumnDefinition))(Bridge.fn.bind(this, function () {
                            try {
                                for (;;) {
                                    switch ($step) {
                                        case 0: {
                                            if (columnDefinitionsOrNull != null && columnDefinitionsOrNull.Count > 0) {
                                                    $step = 1;
                                                    continue;
                                                } else  {
                                                    $step = 6;
                                                    continue;
                                                }
                                        }
                                        case 1: {
                                            $t = Bridge.getEnumerator(columnDefinitionsOrNull);
                                                $step = 2;
                                                continue;
                                        }
                                        case 2: {
                                            if ($t.moveNext()) {
                                                    column = $t.Current;
                                                    $step = 3;
                                                    continue;
                                                }
                                            $step = 5;
                                            continue;
                                        }
                                        case 3: {
                                            $enumerator.current = column;
                                                $step = 4;
                                                return true;
                                        }
                                        case 4: {
                                            $step = 2;
                                            continue;
                                        }
                                        case 5: {
                                            $step = 8;
                                            continue;
                                        }
                                        case 6: {
                                            $enumerator.current = new Windows.UI.Xaml.Controls.ColumnDefinition();
                                                $step = 7;
                                                return true;
                                        }
                                        case 7: {
                                            $step = 8;
                                            continue;
                                        }
                                        case 8: {

                                        }
                                        default: {
                                            return false;
                                        }
                                    }
                                }
                            } catch($async_e1) {
                                $async_e = System.Exception.create($async_e1);
                                throw $async_e;
                            }
                        }));
                        return $enumerator;
                    }, arguments));
                },
                ConvertGridLengthToCssString: function (gridLength, minSize, signUsedForPercentage) {
                    if (signUsedForPercentage === void 0) { signUsedForPercentage = "%"; }
                    if (gridLength.IsAuto) {
                        return "auto";
                    } else if (gridLength.IsStar) {
                        var isCSSGrid = document.isGridSupported;
                        if (isCSSGrid) {
                            var minWidthString = (isNaN(minSize) || (Math.abs(minSize) === Number.POSITIVE_INFINITY) ? "0px" : (System.String.replaceAll(System.Double.format(minSize), String.fromCharCode(44), String.fromCharCode(46)) || "") + "px");
                            return "minmax(" + (minWidthString || "") + ", " + (System.String.replaceAll(System.Double.format(gridLength.Value), String.fromCharCode(44), String.fromCharCode(46)) || "") + (signUsedForPercentage || "") + ")";
                        } else {
                            return (System.String.replaceAll(System.Double.format(gridLength.Value), String.fromCharCode(44), String.fromCharCode(46)) || "") + (signUsedForPercentage || "");
                        }
                    } else {
                        return (System.String.replaceAll(System.Double.format(gridLength.Value), String.fromCharCode(44), String.fromCharCode(46)) || "") + "px";
                    }
                },
                GenerateDomElementsForGrid_NonCSSVersion: function (grid, cellsStructure, columnDefinitionsOrNull, rowDefinitionsOrNull, parentRef, normalizedColumnDefinitions, normalizedRowDefinitions) {
                    var $t, $t1;

                    var table = CSHTML5.Internal.INTERNAL_HtmlDomManager.CreateDomElementAndAppendIt("table", parentRef, grid);
                    var tableStyle = CSHTML5.Internal.INTERNAL_HtmlDomManager.GetDomElementStyleForModification(table);
                    tableStyle.borderCollapse = "collapse";
                    tableStyle.position = "relative";
                    tableStyle.height = "100%";
                    tableStyle.width = "100%";

                    Windows.UI.Xaml.Controls.Grid_InternalHelpers.NormalizeWidthAndHeightPercentages(grid, columnDefinitionsOrNull, rowDefinitionsOrNull, normalizedColumnDefinitions, normalizedRowDefinitions);

                    var currentRowIndex = 0;
                    $t = Bridge.getEnumerator(normalizedRowDefinitions.v);
                    try {
                        while ($t.moveNext()) {
                            var rowDefinition = $t.Current;
                            var currentRow = cellsStructure.getItem(currentRowIndex);

                            var tr = CSHTML5.Internal.INTERNAL_HtmlDomManager.CreateDomElementAndAppendIt("tr", table, grid);

                            var currentColumnIndex = 0;
                            $t1 = Bridge.getEnumerator(normalizedColumnDefinitions.v);
                            try {
                                while ($t1.moveNext()) {
                                    var columnDefinition = $t1.Current;
                                    var currentCell = currentRow.getItem(currentColumnIndex);

                                    if (!currentCell.IsOverlapped) {
                                        var td = CSHTML5.Internal.INTERNAL_HtmlDomManager.CreateDomElementAndAppendIt("td", tr, grid);
                                        var tdStyle = CSHTML5.Internal.INTERNAL_HtmlDomManager.GetDomElementStyleForModification(td);
                                        if (columnDefinition.Visibility === Windows.UI.Xaml.Visibility.Visible) {
                                            tdStyle.display = "table-cell";
                                        } else {
                                            tdStyle.display = "none";
                                        }

                                        CSHTML5.Internal.INTERNAL_HtmlDomManager.SetDomElementAttribute(td, "rowspan", Bridge.box(currentCell.RowSpan, System.Int32));
                                        CSHTML5.Internal.INTERNAL_HtmlDomManager.SetDomElementAttribute(td, "colspan", Bridge.box(currentCell.ColumnSpan, System.Int32));

                                        Windows.UI.Xaml.Controls.Grid_InternalHelpers.ApplyGridLinesValues(grid, currentRowIndex, currentColumnIndex, tdStyle);

                                        var div = CSHTML5.Internal.INTERNAL_HtmlDomManager.CreateDomElementAndAppendIt("div", td, grid);
                                        var divStyle = CSHTML5.Internal.INTERNAL_HtmlDomManager.GetDomElementStyleForModification(div);
                                        divStyle.position = "relative";



                                        currentCell.DomElement = div;
                                        currentCell.ColumnDomElement = td;
                                    }
                                    currentCell.RowDomElement = tr;
                                    currentColumnIndex = (currentColumnIndex + 1) | 0;
                                }
                            } finally {
                                if (Bridge.is($t1, System.IDisposable)) {
                                    $t1.System$IDisposable$Dispose();
                                }
                            }
                            currentRowIndex = (currentRowIndex + 1) | 0;
                        }
                    } finally {
                        if (Bridge.is($t, System.IDisposable)) {
                            $t.System$IDisposable$Dispose();
                        }
                    }


                    return table;
                },
                RefreshGridLines: function (grid, cellsStructure, columnDefinitions, rowDefinitions) {
                    var $t, $t1;
                    var isCSSGrid = document.isGridSupported;
                    if (!isCSSGrid) {
                        var currentRowIndex = 0;
                        $t = Bridge.getEnumerator(rowDefinitions);
                        try {
                            while ($t.moveNext()) {
                                var rowDefinition = $t.Current;
                                var currentRow = cellsStructure.getItem(currentRowIndex);
                                var currentColumnIndex = 0;
                                $t1 = Bridge.getEnumerator(columnDefinitions);
                                try {
                                    while ($t1.moveNext()) {
                                        var columnDefinition = $t1.Current;
                                        var currentCell = currentRow.getItem(currentColumnIndex);

                                        if (!currentCell.IsOverlapped) {
                                            var td = currentCell.ColumnDomElement;
                                            var tdStyle = CSHTML5.Internal.INTERNAL_HtmlDomManager.GetDomElementStyleForModification(td);
                                            Windows.UI.Xaml.Controls.Grid_InternalHelpers.ApplyGridLinesValues(grid, currentRowIndex, currentColumnIndex, tdStyle);
                                        }
                                        currentColumnIndex = (currentColumnIndex + 1) | 0;
                                    }
                                } finally {
                                    if (Bridge.is($t1, System.IDisposable)) {
                                        $t1.System$IDisposable$Dispose();
                                    }
                                }
                                currentRowIndex = (currentRowIndex + 1) | 0;
                            }
                        } finally {
                            if (Bridge.is($t, System.IDisposable)) {
                                $t.System$IDisposable$Dispose();
                            }
                        }
                    }
                },
                ApplyGridLinesValues: function (grid, rowIndex, columnIndex, tdStyle) {
                    if (grid.INTERNAL_StringToSetVerticalGridLinesInCss != null) {
                        tdStyle.borderRight = grid.INTERNAL_StringToSetVerticalGridLinesInCss;
                        if (columnIndex === 0) {
                            tdStyle.borderLeft = grid.INTERNAL_StringToSetVerticalGridLinesInCss;
                        }
                    }
                    if (grid.INTERNAL_StringToSetHorizontalGridLinesInCss != null) {
                        tdStyle.borderBottom = grid.INTERNAL_StringToSetHorizontalGridLinesInCss;
                        if (rowIndex === 0) {
                            tdStyle.borderTop = grid.INTERNAL_StringToSetHorizontalGridLinesInCss;
                        }
                    }
                },
                NormalizeWidthAndHeightPercentages: function (grid, columnDefinitionsOrNull, rowDefinitionsOrNull, normalizedColumnDefinitions, normalizedRowDefinitions) {
                    if (document.isGridSupported) {
                        Windows.UI.Xaml.Controls.Grid_InternalHelpers.NormalizeWidthAndHeightPercentages_CSSVersion(grid, columnDefinitionsOrNull, rowDefinitionsOrNull, normalizedColumnDefinitions, normalizedRowDefinitions);
                    } else {
                        Windows.UI.Xaml.Controls.Grid_InternalHelpers.NormalizeWidthAndHeightPercentages_NonCSSVersion(grid, columnDefinitionsOrNull, rowDefinitionsOrNull, normalizedColumnDefinitions, normalizedRowDefinitions);
                    }
                },
                NormalizeWidthAndHeightPercentages_NonCSSVersion: function (grid, columnDefinitionsOrNull, rowDefinitionsOrNull, normalizedColumnDefinitions, normalizedRowDefinitions) {
                    var $t, $t1, $t2, $t3, $t4;
                    normalizedColumnDefinitions.v = new (System.Collections.Generic.List$1(Windows.UI.Xaml.Controls.ColumnDefinition)).ctor();
                    normalizedRowDefinitions.v = new (System.Collections.Generic.List$1(Windows.UI.Xaml.Controls.RowDefinition)).ctor();
                    var sumOfWidthPercentages = 0;
                    var sumOfHeightPercentages = 0;
                    if (columnDefinitionsOrNull != null) {
                        $t = Bridge.getEnumerator(columnDefinitionsOrNull);
                        try {
                            while ($t.moveNext()) {
                                var columnDefinition = $t.Current;
                                if (columnDefinition.Width.IsStar) {
                                    sumOfWidthPercentages += columnDefinition.Width.Value;
                                }
                            }
                        } finally {
                            if (Bridge.is($t, System.IDisposable)) {
                                $t.System$IDisposable$Dispose();
                            }
                        }
                    }
                    if (rowDefinitionsOrNull != null) {
                        $t1 = Bridge.getEnumerator(rowDefinitionsOrNull);
                        try {
                            while ($t1.moveNext()) {
                                var rowDefinition = $t1.Current;
                                if (rowDefinition.Height.IsStar) {
                                    sumOfHeightPercentages += rowDefinition.Height.Value;
                                }
                            }
                        } finally {
                            if (Bridge.is($t1, System.IDisposable)) {
                                $t1.System$IDisposable$Dispose();
                            }
                        }
                    }
                    if (columnDefinitionsOrNull != null) {
                        $t2 = Bridge.getEnumerator(columnDefinitionsOrNull);
                        try {
                            while ($t2.moveNext()) {
                                var columnDefinition1 = $t2.Current;
                                var normalizedColumnDefinition = columnDefinition1;
                                if (columnDefinition1.Width.IsStar) {
                                    normalizedColumnDefinition = columnDefinition1.Clone();
                                    normalizedColumnDefinition.Width = new Windows.UI.Xaml.GridLength.$ctor2(normalizedColumnDefinition.Width.Value * 100.0 / sumOfWidthPercentages, Windows.UI.Xaml.GridUnitType.Star);
                                }
                                normalizedColumnDefinitions.v.add(normalizedColumnDefinition);
                            }
                        } finally {
                            if (Bridge.is($t2, System.IDisposable)) {
                                $t2.System$IDisposable$Dispose();
                            }
                        }
                    }
                    if (rowDefinitionsOrNull != null) {
                        $t3 = Bridge.getEnumerator(rowDefinitionsOrNull);
                        try {
                            while ($t3.moveNext()) {
                                var rowDefinition1 = $t3.Current;
                                var normalizedRowDefinition = rowDefinition1;
                                if (rowDefinition1.Height.IsStar) {
                                    normalizedRowDefinition = rowDefinition1.Clone();
                                    normalizedRowDefinition.Height = new Windows.UI.Xaml.GridLength.$ctor2(normalizedRowDefinition.Height.Value * 100.0 / sumOfHeightPercentages, Windows.UI.Xaml.GridUnitType.Star);
                                }
                                normalizedRowDefinitions.v.add(normalizedRowDefinition);
                            }
                        } finally {
                            if (Bridge.is($t3, System.IDisposable)) {
                                $t3.System$IDisposable$Dispose();
                            }
                        }
                    }

                    if (normalizedColumnDefinitions.v.Count === 0) {
                        normalizedColumnDefinitions.v.add(($t4 = new Windows.UI.Xaml.Controls.ColumnDefinition(), $t4.Width = new Windows.UI.Xaml.GridLength.$ctor2(100.0, Windows.UI.Xaml.GridUnitType.Star), $t4.Parent = grid, $t4));
                    }
                    if (normalizedRowDefinitions.v.Count === 0) {
                        normalizedRowDefinitions.v.add(($t4 = new Windows.UI.Xaml.Controls.RowDefinition(), $t4.Height = new Windows.UI.Xaml.GridLength.$ctor2(100.0, Windows.UI.Xaml.GridUnitType.Star), $t4.Parent = grid, $t4));
                    }
                },
                NormalizeWidthAndHeightPercentages_CSSVersion: function (grid, columnDefinitionsOrNull, rowDefinitionsOrNull, normalizedColumnDefinitions, normalizedRowDefinitions) {
                    var $t, $t1, $t2, $t3, $t4;




                    var smallestRowStarValue = System.Double.max;
                    var smallestColumnStarValue = System.Double.max;
                    normalizedColumnDefinitions.v = new (System.Collections.Generic.List$1(Windows.UI.Xaml.Controls.ColumnDefinition)).ctor();
                    normalizedRowDefinitions.v = new (System.Collections.Generic.List$1(Windows.UI.Xaml.Controls.RowDefinition)).ctor();

                    if (columnDefinitionsOrNull != null) {
                        $t = Bridge.getEnumerator(columnDefinitionsOrNull);
                        try {
                            while ($t.moveNext()) {
                                var col = $t.Current;
                                if (col.Width.IsStar && col.Width.Value < smallestColumnStarValue) {
                                    smallestColumnStarValue = col.Width.Value;
                                }
                            }
                        } finally {
                            if (Bridge.is($t, System.IDisposable)) {
                                $t.System$IDisposable$Dispose();
                            }
                        }

                        $t1 = Bridge.getEnumerator(columnDefinitionsOrNull);
                        try {
                            while ($t1.moveNext()) {
                                var col1 = $t1.Current;
                                if (col1.Width.IsStar) {
                                    normalizedColumnDefinitions.v.add(($t2 = new Windows.UI.Xaml.Controls.ColumnDefinition(), $t2.Width = new Windows.UI.Xaml.GridLength.$ctor2(col1.Width.Value / smallestColumnStarValue, Windows.UI.Xaml.GridUnitType.Star), $t2.MinWidth = col1.MinWidth, $t2.MaxWidth = col1.MaxWidth, $t2));
                                } else {
                                    normalizedColumnDefinitions.v.add(col1);
                                }
                            }
                        } finally {
                            if (Bridge.is($t1, System.IDisposable)) {
                                $t1.System$IDisposable$Dispose();
                            }
                        }
                    }

                    if (rowDefinitionsOrNull != null) {
                        $t2 = Bridge.getEnumerator(rowDefinitionsOrNull);
                        try {
                            while ($t2.moveNext()) {
                                var row = $t2.Current;
                                if (row.Height.IsStar && row.Height.Value < smallestRowStarValue) {
                                    smallestRowStarValue = row.Height.Value;
                                }
                            }
                        } finally {
                            if (Bridge.is($t2, System.IDisposable)) {
                                $t2.System$IDisposable$Dispose();
                            }
                        }

                        $t3 = Bridge.getEnumerator(rowDefinitionsOrNull);
                        try {
                            while ($t3.moveNext()) {
                                var row1 = $t3.Current;
                                if (row1.Height.IsStar) {
                                    normalizedRowDefinitions.v.add(($t4 = new Windows.UI.Xaml.Controls.RowDefinition(), $t4.Height = new Windows.UI.Xaml.GridLength.$ctor2(row1.Height.Value / smallestRowStarValue, Windows.UI.Xaml.GridUnitType.Star), $t4.MinHeight = row1.MinHeight, $t4.MaxHeight = row1.MaxHeight, $t4));
                                } else {
                                    normalizedRowDefinitions.v.add(row1);
                                }
                            }
                        } finally {
                            if (Bridge.is($t3, System.IDisposable)) {
                                $t3.System$IDisposable$Dispose();
                            }
                        }
                    }



                    if (normalizedColumnDefinitions.v.Count === 0) {
                        normalizedColumnDefinitions.v.add(($t4 = new Windows.UI.Xaml.Controls.ColumnDefinition(), $t4.Width = new Windows.UI.Xaml.GridLength.$ctor2(1.0, Windows.UI.Xaml.GridUnitType.Star), $t4.Parent = grid, $t4));
                    }
                    if (normalizedRowDefinitions.v.Count === 0) {
                        normalizedRowDefinitions.v.add(($t4 = new Windows.UI.Xaml.Controls.RowDefinition(), $t4.Height = new Windows.UI.Xaml.GridLength.$ctor2(1.0, Windows.UI.Xaml.GridUnitType.Star), $t4.Parent = grid, $t4));
                    }
                },
                /**
                 * Refreshes the height of all the rows (and their columns) of the given grid
                 *
                 * @static
                 * @this Windows.UI.Xaml.Controls.Grid_InternalHelpers
                 * @memberof Windows.UI.Xaml.Controls.Grid_InternalHelpers
                 * @param   {Windows.UI.Xaml.Controls.Grid}        grid                        the grid of which rows' heights we want to refresh
                 * @param   {System.Collections.Generic.List$1}    normalizedRowDefinitions    the normalized(aka. their stars add up to 100%) version of the rows' definitions, null value is allowed, it will be calculated in this method.
                 * @return  {void}
                 */
                RefreshAllRowsHeight_NonCSSVersion: function (grid, normalizedRowDefinitions) {
                    if (normalizedRowDefinitions === void 0) { normalizedRowDefinitions = null; }
                    normalizedRowDefinitions = {v:normalizedRowDefinitions};
                    var normalizedColumnDefinitions = { };
                    if (normalizedRowDefinitions.v == null) {
                        Windows.UI.Xaml.Controls.Grid_InternalHelpers.NormalizeWidthAndHeightPercentages(grid, grid._columnDefinitionsOrNull, grid._rowDefinitionsOrNull, normalizedColumnDefinitions, normalizedRowDefinitions);
                    }
                    var hasOnlyOneRow = normalizedRowDefinitions.v.Count === 1;
                    var hasStar = false;
                    for (var i = 0; i < normalizedRowDefinitions.v.Count; i = (i + 1) | 0) {
                        Windows.UI.Xaml.Controls.Grid_InternalHelpers.RefreshRowHeight_NonCSSVersion(grid, i, hasOnlyOneRow, normalizedRowDefinitions.v);
                        if (normalizedRowDefinitions.v.getItem(i).Height.IsStar) {
                            hasStar = true;
                        }
                    }
                    if (!hasStar) {
                        Windows.UI.Xaml.Controls.Grid_InternalHelpers.AddFillingStarRow_NonCSSVersion(grid);
                    }
                },
                RefreshAllRowsHeight_CSSVersion: function (grid, normalizedColumnDefinitions, normalizedRowDefinitions) {
                    var $t;
                    if (normalizedColumnDefinitions === void 0) { normalizedColumnDefinitions = null; }
                    if (normalizedRowDefinitions === void 0) { normalizedRowDefinitions = null; }
                    normalizedColumnDefinitions = {v:normalizedColumnDefinitions};
                    normalizedRowDefinitions = {v:normalizedRowDefinitions};

                    var rowsAsString = "";
                    var isFirstRow = true;

                    if (grid._rowDefinitionsOrNull != null) {
                        if (normalizedColumnDefinitions.v == null || normalizedRowDefinitions.v == null) {
                            Windows.UI.Xaml.Controls.Grid_InternalHelpers.NormalizeWidthAndHeightPercentages(grid, null, grid._rowDefinitionsOrNull, normalizedColumnDefinitions, normalizedRowDefinitions);
                        }

                        var hadStarRow = false;

                        $t = Bridge.getEnumerator(normalizedRowDefinitions.v);
                        try {
                            while ($t.moveNext()) {
                                var rowDefinition = $t.Current;
                                if (!hadStarRow && rowDefinition.Height.IsStar) {
                                    hadStarRow = true;
                                }

                                rowsAsString = (rowsAsString || "") + ((!isFirstRow ? " " : "") || "") + (Windows.UI.Xaml.Controls.Grid_InternalHelpers.ConvertGridLengthToCssString(rowDefinition.Height.$clone(), rowDefinition.MinHeight, "fr") || "");
                                isFirstRow = false;
                            }
                        } finally {
                            if (Bridge.is($t, System.IDisposable)) {
                                $t.System$IDisposable$Dispose();
                            }
                        }
                        if (!hadStarRow) {
                            rowsAsString = (rowsAsString || "") + ((!isFirstRow ? " minmax(0px, 1fr)" : "minmax(0px, 1fr)") || "");
                        }
                    }

                    if (isFirstRow) {
                        rowsAsString = "minmax(0px, 1fr)";
                    }

                    var domElementStyle = CSHTML5.Internal.INTERNAL_HtmlDomManager.GetDomElementStyleForModification(grid._innerDiv);
                    var isMsGrid = document.isMSGrid;
                    if (!isMsGrid) {
                        domElementStyle.gridTemplateRows = rowsAsString;
                    } else {
                        domElementStyle.msGridRows = rowsAsString;
                    }
                },
                RefreshRowHeight_NonCSSVersion: function (grid, rowIndex, isTheOnlyRow, normalizedRowsDefinitionsIfNeeded) {
                    var $t;
                    if (normalizedRowsDefinitionsIfNeeded === void 0) { normalizedRowsDefinitionsIfNeeded = null; }
                    var clipToBounds = grid.ClipToBounds;
                    var row = grid._currentCellsStructure.getItem(rowIndex);
                    var rowDefinition;
                    if (normalizedRowsDefinitionsIfNeeded != null) {
                        rowDefinition = normalizedRowsDefinitionsIfNeeded.getItem(rowIndex);
                    } else {
                        rowDefinition = grid._rowDefinitionsOrNull.getItem(rowIndex);
                    }
                    var rowHeight = Windows.UI.Xaml.Controls.Grid_InternalHelpers.ConvertGridLengthToCssString(rowDefinition.Height.$clone(), rowDefinition.MinHeight);
                    var internalElementForRowHeight = "100%";
                    if (System.String.endsWith(rowHeight, "px")) {
                        internalElementForRowHeight = rowHeight;
                        rowHeight = "auto";
                    } else if (isTheOnlyRow && !isNaN(grid.Height)) {
                        internalElementForRowHeight = (System.String.replaceAll(System.Double.format(grid.Height), String.fromCharCode(44), String.fromCharCode(46)) || "") + "px";
                        rowHeight = "auto";
                    }
                    var rowDomElementStyle = CSHTML5.Internal.INTERNAL_HtmlDomManager.GetDomElementStyleForModification(row.getItem(0).RowDomElement);
                    rowDomElementStyle.height = rowHeight;

                    $t = Bridge.getEnumerator(row);
                    try {
                        while ($t.moveNext()) {
                            var cell = $t.Current;
                            if (!cell.IsOverlapped) {
                                var domElementStyle = CSHTML5.Internal.INTERNAL_HtmlDomManager.GetDomElementStyleForModification(cell.DomElement);

                                var domElementStyleAppliedValue = { v : internalElementForRowHeight };

                                if (System.String.endsWith(internalElementForRowHeight, "px")) {
                                    if (clipToBounds) {
                                        domElementStyle.overflowX = "hidden";
                                    }

                                }

                                var columnDomElementStyle = CSHTML5.Internal.INTERNAL_HtmlDomManager.GetDomElementStyleForModification(cell.ColumnDomElement);
                                columnDomElementStyle.height = "inherit";
                                if (Bridge.referenceEquals(rowHeight, "auto")) {
                                    if (!System.String.endsWith(internalElementForRowHeight, "px")) {
                                        domElementStyleAppliedValue.v = "auto";
                                    }
                                } else {
                                    domElementStyleAppliedValue.v = "100%";
                                }

                                if (cell.RowSpan > 1 && (System.String.endsWith(domElementStyleAppliedValue.v, "px") || Bridge.referenceEquals(domElementStyleAppliedValue.v, "auto"))) {
                                    Windows.UI.Xaml.Controls.Grid_InternalHelpers.RefreshCellWithSpanHeight(grid, cell, domElementStyleAppliedValue);
                                }

                                domElementStyle.height = domElementStyleAppliedValue.v;
                            }
                        }
                    } finally {
                        if (Bridge.is($t, System.IDisposable)) {
                            $t.System$IDisposable$Dispose();
                        }
                    }
                },
                /**
                 * Sets the size of the dom element of the cell ONLY IN THE CASE WHERE THE CELL HAS A ROW SPAN.
                 If any involved row has a star value, it will set it to 100%,
                 otherwise, if they all have an absolute size, it will set it to the sum of those sizes,
                 otherwise (if there is any auto sized row), it won't do anything.
                 *
                 * @static
                 * @this Windows.UI.Xaml.Controls.Grid_InternalHelpers
                 * @memberof Windows.UI.Xaml.Controls.Grid_InternalHelpers
                 * @param   {Windows.UI.Xaml.Controls.Grid}                       grid                             
                 * @param   {Windows.UI.Xaml.Controls.INTERNAL_CellDefinition}    cell                             
                 * @param   {System.String}                                       valueToApplyToDomElementStyle
                 * @return  {void}
                 */
                RefreshCellWithSpanHeight: function (grid, cell, valueToApplyToDomElementStyle) {
                    if (cell.RowSpan > 1) {

                        var rowIndex = cell.Row;
                        var heightInPixels = 0;
                        var isAbsoluteHeight = true;
                        for (var i = 0; i < cell.RowSpan; i = (i + 1) | 0) {
                            var currentRowDefinition = grid._rowDefinitionsOrNull.getItem(rowIndex);

                            if (currentRowDefinition.Height.IsAbsolute) {
                                heightInPixels += currentRowDefinition.Height.Value;
                            } else {
                                isAbsoluteHeight = false;
                                break;
                            }
                        }
                        if (isAbsoluteHeight) {
                            valueToApplyToDomElementStyle.v = System.Double.format(heightInPixels) + "px";
                        } else {
                            valueToApplyToDomElementStyle.v = "100%";
                        }
                    }
                },
                RefreshCellWidthAndHeight: function (grid, cell, normalizedColumnDefinition, isTheOnlyColumn, normalizedRowDefinition, isTheOnlyRow) {
                    var clipToBounds = grid.ClipToBounds;

                    var rowHeight = Windows.UI.Xaml.Controls.Grid_InternalHelpers.ConvertGridLengthToCssString(normalizedRowDefinition.Height.$clone(), normalizedRowDefinition.MinHeight);
                    var internalElementForRowHeight = "100%";
                    if (System.String.endsWith(rowHeight, "px")) {
                        internalElementForRowHeight = rowHeight;
                        rowHeight = "auto";
                    } else if (isTheOnlyRow && !isNaN(grid.Height)) {
                        internalElementForRowHeight = (System.String.replaceAll(System.Double.format(grid.Height), String.fromCharCode(44), String.fromCharCode(46)) || "") + "px";
                        rowHeight = "auto";
                    }

                    if (!cell.IsOverlapped) {
                        var domElementStyle = CSHTML5.Internal.INTERNAL_HtmlDomManager.GetDomElementStyleForModification(cell.DomElement);

                        var domElementStyleAppliedValue = { v : internalElementForRowHeight };


                        if (System.String.endsWith(internalElementForRowHeight, "px")) {
                            if (clipToBounds) {
                                domElementStyle.overflowX = "hidden";
                            }

                        }

                        var columnDomElementStyle = CSHTML5.Internal.INTERNAL_HtmlDomManager.GetDomElementStyleForModification(cell.ColumnDomElement);
                        columnDomElementStyle.height = "inherit";
                        if (Bridge.referenceEquals(rowHeight, "auto")) {
                            if (!System.String.endsWith(internalElementForRowHeight, "px")) {
                                domElementStyleAppliedValue.v = "auto";
                            }
                        } else {
                            domElementStyleAppliedValue.v = "100%";
                        }

                        if (cell.RowSpan > 1 && (System.String.endsWith(internalElementForRowHeight, "px") || Bridge.referenceEquals(internalElementForRowHeight, "auto"))) {
                            Windows.UI.Xaml.Controls.Grid_InternalHelpers.RefreshCellWithSpanHeight(grid, cell, domElementStyleAppliedValue);
                            domElementStyle.height = domElementStyleAppliedValue.v;
                        }
                    }







                    if (!cell.IsOverlapped) {
                        var tdStyle = CSHTML5.Internal.INTERNAL_HtmlDomManager.GetDomElementStyleForModification(cell.ColumnDomElement);
                        var columnWidth = Windows.UI.Xaml.Controls.Grid_InternalHelpers.ConvertGridLengthToCssString(normalizedColumnDefinition.Width.$clone(), normalizedColumnDefinition.MinWidth);
                        var internalElementForColumnWidth = "100%";
                        if (System.String.endsWith(columnWidth, "px")) {
                            internalElementForColumnWidth = columnWidth;
                            columnWidth = "auto";
                        } else if (isTheOnlyColumn && !isNaN(grid.Width)) {
                            internalElementForColumnWidth = (System.String.replaceAll(System.Double.format(grid.Width), String.fromCharCode(44), String.fromCharCode(46)) || "") + "px";
                            columnWidth = "auto";
                        }

                        tdStyle.width = columnWidth;
                        tdStyle.position = "relative";
                        tdStyle.padding = "0px";

                        var domElementStyle1 = CSHTML5.Internal.INTERNAL_HtmlDomManager.GetDomElementStyleForModification(cell.DomElement);
                        domElementStyle1.width = internalElementForColumnWidth;

                        if (System.String.endsWith(internalElementForColumnWidth, "px")) {
                            if (clipToBounds) {
                                domElementStyle1.overflowX = "hidden";
                            }
                        }
                    }

                },
                RefreshAllColumnsWidth_NonCSSVersion: function (grid, normalizedColumnDefinitions) {
                    if (normalizedColumnDefinitions === void 0) { normalizedColumnDefinitions = null; }
                    normalizedColumnDefinitions = {v:normalizedColumnDefinitions};
                    var normalizedRowDefinitions = { };
                    if (normalizedColumnDefinitions.v == null) {
                        Windows.UI.Xaml.Controls.Grid_InternalHelpers.NormalizeWidthAndHeightPercentages(grid, grid._columnDefinitionsOrNull, grid._rowDefinitionsOrNull, normalizedColumnDefinitions, normalizedRowDefinitions);
                    }
                    var isTheOnlyColumn = normalizedColumnDefinitions.v.Count === 1;
                    var hasStar = false;
                    for (var i = 0; i < normalizedColumnDefinitions.v.Count; i = (i + 1) | 0) {
                        Windows.UI.Xaml.Controls.Grid_InternalHelpers.RefreshColumnWidth_NonCSSVersion(grid, i, isTheOnlyColumn, normalizedColumnDefinitions.v);
                        if (normalizedColumnDefinitions.v.getItem(i).Width.IsStar) {
                            hasStar = true;
                        }
                    }
                    if (!hasStar) {
                        Windows.UI.Xaml.Controls.Grid_InternalHelpers.AddFillingStarColumn_NonCSSVersion(grid);
                    }
                },
                RefreshAllColumnsWidth_CSSVersion: function (grid, normalizedColumnDefinitions, normalizedRowDefinitions) {
                    var $t, $t1;
                    if (normalizedColumnDefinitions === void 0) { normalizedColumnDefinitions = null; }
                    if (normalizedRowDefinitions === void 0) { normalizedRowDefinitions = null; }
                    normalizedColumnDefinitions = {v:normalizedColumnDefinitions};
                    normalizedRowDefinitions = {v:normalizedRowDefinitions};

                    var columnsAsString = "";
                    var isFirstColumn = true;

                    if (grid._columnDefinitionsOrNull != null) {
                        if (normalizedColumnDefinitions.v == null || normalizedRowDefinitions.v == null) {
                            Windows.UI.Xaml.Controls.Grid_InternalHelpers.NormalizeWidthAndHeightPercentages(grid, grid._columnDefinitionsOrNull, null, normalizedColumnDefinitions, normalizedRowDefinitions);
                        }

                        var hasStarColumn = false;
                        var collapsedColumns = new (System.Collections.Generic.List$1(System.Int32)).ctor();
                        var currentIndex = 0;
                        $t = Bridge.getEnumerator(normalizedColumnDefinitions.v);
                        try {
                            while ($t.moveNext()) {
                                var columnDefinition = $t.Current;
                                if (!hasStarColumn && columnDefinition.Width.IsStar) {
                                    hasStarColumn = true;
                                }

                                if (columnDefinition.Visibility === Windows.UI.Xaml.Visibility.Visible) {
                                    columnsAsString = (columnsAsString || "") + ((!isFirstColumn ? " " : "") || "") + (Windows.UI.Xaml.Controls.Grid_InternalHelpers.ConvertGridLengthToCssString(columnDefinition.Width.$clone(), columnDefinition.MinWidth, "fr") || "");
                                } else {
                                    collapsedColumns.add(currentIndex);
                                    columnsAsString = (columnsAsString || "") + ((!isFirstColumn ? " 0px" : "0px") || "");
                                }
                                currentIndex = (currentIndex + 1) | 0;

                                isFirstColumn = false;
                            }
                        } finally {
                            if (Bridge.is($t, System.IDisposable)) {
                                $t.System$IDisposable$Dispose();
                            }
                        }
                        if (!hasStarColumn) {
                            columnsAsString = (columnsAsString || "") + ((!isFirstColumn ? " minmax(0px, 1fr)" : "minmax(0px, 1fr)") || "");
                        }

                        $t1 = Bridge.getEnumerator(grid.Children);
                        try {
                            while ($t1.moveNext()) {
                                var child = $t1.Current;
                                if (CSHTML5.Internal.INTERNAL_VisualTreeManager.IsElementInVisualTree(child)) {
                                    var startColumn = Windows.UI.Xaml.Controls.Grid.GetColumn(child);
                                    var endColumn = (((startColumn + Windows.UI.Xaml.Controls.Grid.GetColumnSpan(child)) | 0) - 1) | 0;
                                    if (Windows.UI.Xaml.Controls.Grid_InternalHelpers.IsAllCollapsedColumns(grid, startColumn, endColumn)) {
                                        child.INTERNAL_OuterDomElement.style.overflow = 'hidden';
                                        child.INTERNAL_OuterDomElement.setAttribute('data-isCollapsedDueToHiddenColumn', true);
                                    } else {

if (                                        child.INTERNAL_OuterDomElement.getAttribute('data-isCollapsedDueToHiddenColumn' == true)){
    child.INTERNAL_OuterDomElement.style.overflow = 'visible';
    child.INTERNAL_OuterDomElement.setAttribute('data-isCollapsedDueToHiddenColumn', false);
};
                                    }
                                }
                            }
                        } finally {
                            if (Bridge.is($t1, System.IDisposable)) {
                                $t1.System$IDisposable$Dispose();
                            }
                        }
                    }


                    if (isFirstColumn) {
                        columnsAsString = "minmax(0px, 1fr)";
                    }

                    var domElementStyle = CSHTML5.Internal.INTERNAL_HtmlDomManager.GetDomElementStyleForModification(grid._innerDiv);
                    var isMsGrid = document.isMSGrid;
                    if (!isMsGrid) {
                        domElementStyle.gridTemplateColumns = columnsAsString;
                    } else {
                        domElementStyle.msGridColumns = columnsAsString;
                    }
                },
                IsAllCollapsedColumns: function (grid, startColumn, endColumn) {
                    var maxColumnIndex = (grid.ColumnDefinitions.Count - 1) | 0;
                    if (maxColumnIndex > -1) {
                        if (startColumn >= maxColumnIndex) {
                            startColumn = maxColumnIndex;
                        }
                        if (endColumn > maxColumnIndex) {
                            endColumn = maxColumnIndex;
                        }
                        for (var i = startColumn; i <= endColumn; i = (i + 1) | 0) {
                            if (grid.ColumnDefinitions.getItem(i).Visibility === Windows.UI.Xaml.Visibility.Visible) {
                                return false;
                            }
                        }
                        return true;
                    }
                    return false;
                },
                RefreshColumnWidth_NonCSSVersion: function (grid, ColumnIndex, isTheOnlyColumn, normalizedColumnsDefinitionsIfNeeded) {
                    var $t, $t1;
                    if (normalizedColumnsDefinitionsIfNeeded === void 0) { normalizedColumnsDefinitionsIfNeeded = null; }
                    var clipToBounds = grid.ClipToBounds;
                    var column = new (System.Collections.Generic.List$1(Windows.UI.Xaml.Controls.INTERNAL_CellDefinition)).ctor();

                    $t = Bridge.getEnumerator(grid._currentCellsStructure);
                    try {
                        while ($t.moveNext()) {
                            var row = $t.Current;
                            column.add(row.getItem(ColumnIndex));
                        }
                    } finally {
                        if (Bridge.is($t, System.IDisposable)) {
                            $t.System$IDisposable$Dispose();
                        }
                    }

                    var columnDefinition;
                    if (normalizedColumnsDefinitionsIfNeeded != null) {
                        columnDefinition = normalizedColumnsDefinitionsIfNeeded.getItem(ColumnIndex);
                    } else {
                        columnDefinition = grid._columnDefinitionsOrNull.getItem(ColumnIndex);
                    }

                    $t1 = Bridge.getEnumerator(column);
                    try {
                        while ($t1.moveNext()) {
                            var cell = $t1.Current;
                            if (!cell.IsOverlapped) {
                                var tdStyle = CSHTML5.Internal.INTERNAL_HtmlDomManager.GetDomElementStyleForModification(cell.ColumnDomElement);
                                var columnWidth = Windows.UI.Xaml.Controls.Grid_InternalHelpers.ConvertGridLengthToCssString(columnDefinition.Width.$clone(), columnDefinition.MinWidth);
                                var internalElementForColumnWidth = "100%";
                                if (System.String.endsWith(columnWidth, "px")) {
                                    internalElementForColumnWidth = columnWidth;
                                    columnWidth = "auto";
                                } else if (isTheOnlyColumn && !isNaN(grid.Width)) {
                                    internalElementForColumnWidth = (System.String.replaceAll(System.Double.format(grid.Width), String.fromCharCode(44), String.fromCharCode(46)) || "") + "px";
                                    columnWidth = "auto";
                                }

                                tdStyle.width = columnWidth;
                                tdStyle.position = "relative";
                                tdStyle.padding = "0px";

                                var domElementStyle = CSHTML5.Internal.INTERNAL_HtmlDomManager.GetDomElementStyleForModification(cell.DomElement);
                                domElementStyle.width = internalElementForColumnWidth;

                                if (System.String.endsWith(internalElementForColumnWidth, "px")) {
                                    if (clipToBounds) {
                                        domElementStyle.overflowX = "hidden";
                                    }

                                }
                            }
                        }
                    } finally {
                        if (Bridge.is($t1, System.IDisposable)) {
                            $t1.System$IDisposable$Dispose();
                        }
                    }
                },
                AddFillingStarColumn_NonCSSVersion: function (grid) {
                    var $t;
                    if (grid._currentCellsStructure != null) {
                        $t = Bridge.getEnumerator(grid._currentCellsStructure);
                        try {
                            while ($t.moveNext()) {
                                var row = $t.Current;
                                if (row != null && row.getItem(0) != null) {
                                    var cell = row.getItem(0);
                                    if (cell.RowDomElement != null) {
                                        var td = CSHTML5.Internal.INTERNAL_HtmlDomManager.CreateDomElementAndAppendIt("td", cell.RowDomElement, grid);
                                        var tdStyle = CSHTML5.Internal.INTERNAL_HtmlDomManager.GetDomElementStyleForModification(td);

                                        tdStyle.display = "table-cell";

                                        tdStyle.width = "100%";

                                    }
                                }
                            }
                        } finally {
                            if (Bridge.is($t, System.IDisposable)) {
                                $t.System$IDisposable$Dispose();
                            }
                        }
                    }
                },
                AddFillingStarRow_NonCSSVersion: function (grid) {
                    if (grid._currentDomTable != null) {
                        var tr = CSHTML5.Internal.INTERNAL_HtmlDomManager.CreateDomElementAndAppendIt("tr", grid._currentDomTable, grid);
                        var trStyle = CSHTML5.Internal.INTERNAL_HtmlDomManager.GetDomElementStyleForModification(tr);
                        trStyle.height = "100%";
                    }
                },
                RefreshAllColumnsVisibility_CSSVersion: function (grid) {
                    if (grid != null && CSHTML5.Internal.INTERNAL_VisualTreeManager.IsElementInVisualTree(grid)) {
                        if (grid.ColumnDefinitions != null) {
                            Windows.UI.Xaml.Controls.Grid_InternalHelpers.RefreshAllColumnsWidth_CSSVersion(grid);

                        }
                    }
                },
                RefreshColumnVisibility: function (grid, columnDefinition, newVisibility) {
                    var $t, $t1;
                    if (document.isGridSupported) {
                        Windows.UI.Xaml.Controls.Grid_InternalHelpers.RefreshColumnVisibility_CSSVersion(grid, columnDefinition, newVisibility);
                    } else {
                        if (CSHTML5.Internal.INTERNAL_VisualTreeManager.IsElementInVisualTree(grid)) {
                            var column = new (System.Collections.Generic.List$1(Windows.UI.Xaml.Controls.INTERNAL_CellDefinition)).ctor();
                            var columnIndex = grid._columnDefinitionsOrNull.indexOf(columnDefinition);

                            $t = Bridge.getEnumerator(grid._currentCellsStructure);
                            try {
                                while ($t.moveNext()) {
                                    var row = $t.Current;
                                    column.add(row.getItem(columnIndex));
                                }
                            } finally {
                                if (Bridge.is($t, System.IDisposable)) {
                                    $t.System$IDisposable$Dispose();
                                }
                            }

                            $t1 = Bridge.getEnumerator(column);
                            try {
                                while ($t1.moveNext()) {
                                    var cell = $t1.Current;
                                    if (!cell.IsOverlapped) {
                                        var tdStyle = CSHTML5.Internal.INTERNAL_HtmlDomManager.GetDomElementStyleForModification(cell.ColumnDomElement);
                                        if (newVisibility === Windows.UI.Xaml.Visibility.Collapsed) {
                                            if ((!Bridge.referenceEquals(Bridge.cast(tdStyle.display, System.String), "none"))) {
                                                cell.INTERNAL_previousValueOfDisplayCssProperty = Bridge.cast(tdStyle.display, System.String);
                                                tdStyle.display = "none";
                                            }
                                        } else {
                                            tdStyle.display = cell.INTERNAL_previousValueOfDisplayCssProperty;
                                        }
                                    }
                                }
                            } finally {
                                if (Bridge.is($t1, System.IDisposable)) {
                                    $t1.System$IDisposable$Dispose();
                                }
                            }
                        }
                    }
                },
                RefreshColumnVisibility_CSSVersion: function (grid, columnDefinition, newVisibility) {
                    if (CSHTML5.Internal.INTERNAL_VisualTreeManager.IsElementInVisualTree(grid)) {
                        if (grid.INTERNAL_InnerDomElement != null) {
                            Windows.UI.Xaml.Controls.Grid_InternalHelpers.RefreshAllColumnsWidth_CSSVersion(grid);
                        }
                    }
                },
                HandleSpansInCellsStructureAndReturnRedirectedElements: function (newCellsStructure, Children) {
                    var $t;
                    var redirectedElements = new (System.Collections.Generic.Dictionary$2(Windows.UI.Xaml.UIElement,Windows.UI.Xaml.Controls.INTERNAL_CellDefinition)).ctor();

                    var maxRowIndex = (newCellsStructure.Count - 1) | 0;
                    var maxColumnIndex = (newCellsStructure.getItem(0).Count - 1) | 0;

                    $t = Bridge.getEnumerator(Children);
                    try {
                        while ($t.moveNext()) {
                            var uielement = $t.Current;

                            var rowIndex = Windows.UI.Xaml.Controls.Grid.GetRow(uielement);
                            var columnIndex = Windows.UI.Xaml.Controls.Grid.GetColumn(uielement);

                            rowIndex = Windows.UI.Xaml.Controls.Grid_InternalHelpers.EnsureValueIsBetweenMinAndMax(rowIndex, 0, maxRowIndex);
                            columnIndex = Windows.UI.Xaml.Controls.Grid_InternalHelpers.EnsureValueIsBetweenMinAndMax(columnIndex, 0, maxColumnIndex);

                            var cellDefinition = newCellsStructure.getItem(rowIndex).getItem(columnIndex);

                            if (cellDefinition.IsOverlapped) {

                                redirectedElements.add(uielement, cellDefinition);
                            } else {

                                var spanLastRowIndex = (((rowIndex + Windows.UI.Xaml.Controls.Grid.GetRowSpan(uielement)) | 0) - 1) | 0;
                                var spanLastColumnIndex = (((columnIndex + Windows.UI.Xaml.Controls.Grid.GetColumnSpan(uielement)) | 0) - 1) | 0;

                                spanLastRowIndex = Windows.UI.Xaml.Controls.Grid_InternalHelpers.EnsureValueIsBetweenMinAndMax(spanLastRowIndex, rowIndex, maxRowIndex);
                                spanLastColumnIndex = Windows.UI.Xaml.Controls.Grid_InternalHelpers.EnsureValueIsBetweenMinAndMax(spanLastColumnIndex, columnIndex, maxColumnIndex);

                                var parentCell = null;
                                var isTopLeftCellOfSpannedArea = true;
                                var exitForLoop = false;

                                for (var row = rowIndex; row <= spanLastRowIndex; row = (row + 1) | 0) {
                                    for (var column = columnIndex; column <= spanLastColumnIndex; column = (column + 1) | 0) {
                                        var cellStructure2 = newCellsStructure.getItem(row).getItem(column);

                                        if (cellStructure2.IsOverlapped === true || (cellStructure2.IsOccupied && !isTopLeftCellOfSpannedArea)) {
                                            if (cellStructure2.ParentCell != null) {
                                                parentCell = cellStructure2.ParentCell;
                                            } else {
                                                parentCell = cellStructure2;
                                            }
                                            redirectedElements.add(uielement, parentCell);

                                            exitForLoop = true;

                                            if (exitForLoop) {
                                                break;
                                            }
                                        }
                                        isTopLeftCellOfSpannedArea = false;
                                    }
                                    if (exitForLoop) {
                                        break;
                                    }
                                }

                                if (parentCell == null) {
                                    var isTopLeftCellOfSpannedArea2 = true;
                                    for (var row1 = rowIndex; row1 <= spanLastRowIndex; row1 = (row1 + 1) | 0) {
                                        for (var column1 = columnIndex; column1 <= spanLastColumnIndex; column1 = (column1 + 1) | 0) {
                                            var cellDefinition2 = newCellsStructure.getItem(row1).getItem(column1);
                                            if (isTopLeftCellOfSpannedArea2) {

                                                parentCell = cellDefinition2;
                                                cellDefinition2.IsOccupied = true;
                                                cellDefinition2.RowSpan = (((spanLastRowIndex - rowIndex) | 0) + 1) | 0;
                                                cellDefinition2.ColumnSpan = (((spanLastColumnIndex - columnIndex) | 0) + 1) | 0;
                                                isTopLeftCellOfSpannedArea2 = false;
                                            } else {

                                                cellDefinition2.IsOverlapped = true;
                                                cellDefinition2.ParentCell = parentCell;
                                            }
                                        }
                                    }
                                }
                            }
                        }
                    } finally {
                        if (Bridge.is($t, System.IDisposable)) {
                            $t.System$IDisposable$Dispose();
                        }
                    }
                    return redirectedElements;
                },
                /**
                 * Constraints the value to be between the specified min and max/
                 *
                 * @static
                 * @private
                 * @this Windows.UI.Xaml.Controls.Grid_InternalHelpers
                 * @memberof Windows.UI.Xaml.Controls.Grid_InternalHelpers
                 * @param   {number}    value    The value to constraint.
                 * @param   {number}    min      The minimum allowed.
                 * @param   {number}    max      The maximum allowed.
                 * @return  {number}             The value constrained between min and max.
                 */
                EnsureValueIsBetweenMinAndMax: function (value, min, max) {
                    if (value > max) {
                        value = max;
                    }

                    if (value < 0) {
                        value = 0;
                    }

                    return value;
                }
            }
        }
    });

    /**
     * Enum to indicate what Columns or Rows the GridSplitter resizes
     *
     * @public
     * @class number
     */
    Bridge.define("Windows.UI.Xaml.Controls.GridSplitter.GridResizeBehavior", {
        $kind: "nested enum",
        statics: {
            fields: {
                /**
                 * Determine which columns or rows to resize based on its Alignment.
                 *
                 * @static
                 * @public
                 * @memberof number
                 * @constant
                 * @default 0
                 * @type number
                 */
                BasedOnAlignment: 0,
                /**
                 * Resize the current and next Columns or Rows.
                 *
                 * @static
                 * @public
                 * @memberof number
                 * @constant
                 * @default 1
                 * @type number
                 */
                CurrentAndNext: 1,
                /**
                 * Resize the previous and current Columns or Rows.
                 *
                 * @static
                 * @public
                 * @memberof number
                 * @constant
                 * @default 2
                 * @type number
                 */
                PreviousAndCurrent: 2,
                /**
                 * Resize the previous and next Columns or Rows.
                 *
                 * @static
                 * @public
                 * @memberof number
                 * @constant
                 * @default 3
                 * @type number
                 */
                PreviousAndNext: 3
            }
        }
    });

    /**
     * Enum to indicate whether GridSplitter resizes Columns or Rows
     *
     * @public
     * @class number
     */
    Bridge.define("Windows.UI.Xaml.Controls.GridSplitter.GridResizeDirection", {
        $kind: "nested enum",
        statics: {
            fields: {
                /**
                 * Determines whether to resize rows or columns based on its Alignment and
                 width compared to height
                 *
                 * @static
                 * @public
                 * @memberof number
                 * @constant
                 * @default 0
                 * @type number
                 */
                Auto: 0,
                /**
                 * Resize columns when dragging Splitter.
                 *
                 * @static
                 * @public
                 * @memberof number
                 * @constant
                 * @default 1
                 * @type number
                 */
                Columns: 1,
                /**
                 * Resize rows when dragging Splitter.
                 *
                 * @static
                 * @public
                 * @memberof number
                 * @constant
                 * @default 2
                 * @type number
                 */
                Rows: 2
            }
        }
    });

    /**
     * Enum to indicate the supported gripper cursor types.
     *
     * @public
     * @class number
     */
    Bridge.define("Windows.UI.Xaml.Controls.GridSplitter.GripperCursorType", {
        $kind: "nested enum",
        statics: {
            fields: {
                /**
                 * Change the cursor based on the splitter direction
                 *
                 * @static
                 * @public
                 * @memberof number
                 * @constant
                 * @default -1
                 * @type number
                 */
                Default: -1,
                /**
                 * Standard Arrow cursor
                 *
                 * @static
                 * @public
                 * @memberof number
                 * @constant
                 * @default 0
                 * @type number
                 */
                Arrow: 0,
                /**
                 * Standard Cross cursor
                 *
                 * @static
                 * @public
                 * @memberof number
                 * @constant
                 * @default 1
                 * @type number
                 */
                Cross: 1,
                /**
                 * Standard Custom cursor
                 *
                 * @static
                 * @public
                 * @memberof number
                 * @constant
                 * @default 2
                 * @type number
                 */
                Custom: 2,
                /**
                 * Standard Hand cursor
                 *
                 * @static
                 * @public
                 * @memberof number
                 * @constant
                 * @default 3
                 * @type number
                 */
                Hand: 3,
                /**
                 * Standard Help cursor
                 *
                 * @static
                 * @public
                 * @memberof number
                 * @constant
                 * @default 4
                 * @type number
                 */
                Help: 4,
                /**
                 * Standard IBeam cursor
                 *
                 * @static
                 * @public
                 * @memberof number
                 * @constant
                 * @default 5
                 * @type number
                 */
                IBeam: 5,
                /**
                 * Standard SizeAll cursor
                 *
                 * @static
                 * @public
                 * @memberof number
                 * @constant
                 * @default 6
                 * @type number
                 */
                SizeAll: 6,
                /**
                 * Standard SizeNortheastSouthwest cursor
                 *
                 * @static
                 * @public
                 * @memberof number
                 * @constant
                 * @default 7
                 * @type number
                 */
                SizeNortheastSouthwest: 7,
                /**
                 * Standard SizeNorthSouth cursor
                 *
                 * @static
                 * @public
                 * @memberof number
                 * @constant
                 * @default 8
                 * @type number
                 */
                SizeNorthSouth: 8,
                /**
                 * Standard SizeNorthwestSoutheast cursor
                 *
                 * @static
                 * @public
                 * @memberof number
                 * @constant
                 * @default 9
                 * @type number
                 */
                SizeNorthwestSoutheast: 9,
                /**
                 * Standard SizeWestEast cursor
                 *
                 * @static
                 * @public
                 * @memberof number
                 * @constant
                 * @default 10
                 * @type number
                 */
                SizeWestEast: 10,
                /**
                 * Standard UniversalNo cursor
                 *
                 * @static
                 * @public
                 * @memberof number
                 * @constant
                 * @default 11
                 * @type number
                 */
                UniversalNo: 11,
                /**
                 * Standard UpArrow cursor
                 *
                 * @static
                 * @public
                 * @memberof number
                 * @constant
                 * @default 12
                 * @type number
                 */
                UpArrow: 12,
                /**
                 * Standard Wait cursor
                 *
                 * @static
                 * @public
                 * @memberof number
                 * @constant
                 * @default 13
                 * @type number
                 */
                Wait: 13
            }
        }
    });

    /**
     * Enum to indicate the behavior of window cursor on grid splitter hover
     *
     * @public
     * @class number
     */
    Bridge.define("Windows.UI.Xaml.Controls.GridSplitter.SplitterCursorBehavior", {
        $kind: "nested enum",
        statics: {
            fields: {
                /**
                 * Update window cursor on Grid Splitter hover
                 *
                 * @static
                 * @public
                 * @memberof number
                 * @constant
                 * @default 0
                 * @type number
                 */
                ChangeOnSplitterHover: 0,
                /**
                 * Update window cursor on Grid Splitter Gripper hover
                 *
                 * @static
                 * @public
                 * @memberof number
                 * @constant
                 * @default 1
                 * @type number
                 */
                ChangeOnGripperHover: 1
            }
        }
    });

    Bridge.define("Windows.UI.Xaml.Controls.GripperHoverWrapper", {
        fields: {
            _gridSplitterDirection: 0,
            _gripperCursor: 0,
            _gripperCustomCursorResource: 0,
            _isDragging: false,
            _element: null
        },
        props: {
            GripperCursor: {
                get: function () {
                    return this._gripperCursor;
                },
                set: function (value) {
                    this._gripperCursor = value;
                }
            },
            GripperCustomCursorResource: {
                get: function () {
                    return this._gripperCustomCursorResource;
                },
                set: function (value) {
                    this._gripperCustomCursorResource = value;
                }
            }
        },
        ctors: {
            /**
             * Initializes a new instance of the {@link } class that add cursor change on hover functionality for GridSplitter.
             *
             * @instance
             * @this Windows.UI.Xaml.Controls.GripperHoverWrapper
             * @memberof Windows.UI.Xaml.Controls.GripperHoverWrapper
             * @param   {Windows.UI.Xaml.UIElement}    element                        UI element to apply cursor change on hover
             * @param   {number}                       gridSplitterDirection          GridSplitter resize direction
             * @param   {number}                       gripperCursor                  GridSplitter gripper on hover cursor type
             * @param   {number}                       gripperCustomCursorResource    GridSplitter gripper custom cursor resource number
             * @return  {void}
             */
            ctor: function (element, gridSplitterDirection, gripperCursor, gripperCustomCursorResource) {
                this.$initialize();
                this._gridSplitterDirection = gridSplitterDirection;
                this._gripperCursor = gripperCursor;
                this._gripperCustomCursorResource = gripperCustomCursorResource;
                this._element = element;
                this.UnhookEvents();
            }
        },
        methods: {
            UpdateHoverElement: function (element) {
                this.UnhookEvents();
                this._element = element;
            },
            UnhookEvents: function () {
                if (this._element == null) {
                    return;
                }

            }
        }
    });

    Bridge.define("Windows.UI.Xaml.Controls.INTERNAL_CellDefinition", {
        fields: {
            RowSpan: 0,
            ColumnSpan: 0,
            Row: 0,
            Column: 0,
            IsOverlapped: false,
            IsOccupied: false,
            ParentCell: null,
            DomElement: null,
            RowDomElement: null,
            ColumnDomElement: null,
            INTERNAL_previousValueOfDisplayCssProperty: null,
            numberOfChildren: 0
        },
        ctors: {
            init: function () {
                this.RowSpan = 1;
                this.ColumnSpan = 1;
                this.Row = 0;
                this.Column = 0;
                this.IsOverlapped = false;
                this.IsOccupied = false;
                this.INTERNAL_previousValueOfDisplayCssProperty = "table-cell";
                this.numberOfChildren = 0;
            }
        }
    });

    Bridge.define("Windows.UI.Xaml.Controls.INTERNAL_CheckBoxAndRadioButtonHelpers", {
        statics: {
            methods: {
                CreateDomElement: function (checkBoxOrRadioButton, domInputType, parentRef, domElementWhereToPlaceChildren) {
                    var outerDiv = { };
                    var outerDivStyle = CSHTML5.Internal.INTERNAL_HtmlDomManager.CreateDomElementAppendItAndGetStyle("div", parentRef, checkBoxOrRadioButton, outerDiv);

                    var divInBetween = { };
                    var divInBetweenStyle = CSHTML5.Internal.INTERNAL_HtmlDomManager.CreateDomElementAppendItAndGetStyle("div", outerDiv.v, checkBoxOrRadioButton, divInBetween);

                    var innerElement = { };
                    var innerElementStyle = CSHTML5.Internal.INTERNAL_HtmlDomManager.CreateDomElementAppendItAndGetStyle("input", divInBetween.v, checkBoxOrRadioButton, innerElement);

                    CSHTML5.Internal.INTERNAL_HtmlDomManager.SetDomElementAttribute(innerElement.v, "type", domInputType, true);
                    divInBetweenStyle.position = "relative";
                    divInBetweenStyle.display = "table-cell";
                    divInBetweenStyle.height = "100%";

                    checkBoxOrRadioButton.INTERNAL_OptionalSpecifyDomElementConcernedByFocus = innerElement.v;
                    checkBoxOrRadioButton.INTERNAL_OptionalSpecifyDomElementConcernedByIsEnabled = innerElement.v;

                    var divWhereToPlaceChild = { };
                    var divWhereToPlaceChildStyle = CSHTML5.Internal.INTERNAL_HtmlDomManager.CreateDomElementAppendItAndGetStyle("div", outerDiv.v, checkBoxOrRadioButton, divWhereToPlaceChild);
                    divWhereToPlaceChildStyle.position = "relative";
                    divWhereToPlaceChildStyle.display = "table-cell";
                    divWhereToPlaceChildStyle.height = "100%";


                    if ((window.IE_VERSION) && Bridge.referenceEquals(domInputType, "checkbox")) {
                        CSHTML5.Internal.INTERNAL_EventsHelper.AttachToDomEvents("click", innerElement.v, function (e) {
                            Windows.UI.Xaml.Controls.INTERNAL_CheckBoxAndRadioButtonHelpers.IsCheckedValueChanged(checkBoxOrRadioButton);
                        });
                        CSHTML5.Internal.INTERNAL_EventsHelper.AttachToDomEvents("click", divWhereToPlaceChild.v, function (e) {
                            Windows.UI.Xaml.Controls.INTERNAL_CheckBoxAndRadioButtonHelpers.IsCheckedValueChanged(checkBoxOrRadioButton);
                        });
                    } else {
                        Windows.UI.Xaml.Controls.INTERNAL_CheckBoxAndRadioButtonHelpers.SubscribeToBasicEventsForRadioButton(checkBoxOrRadioButton, innerElement.v, divWhereToPlaceChild.v);
                    }

                    domElementWhereToPlaceChildren.v = divWhereToPlaceChild.v;
                    return outerDiv.v;
                },
                SubscribeToBasicEventsForRadioButton: function (checkBoxOrRadioButton, innerElement, divWhereToPlaceChild) {
                    checkBoxOrRadioButton.SubscribeToClickEventForChildContainerDiv(divWhereToPlaceChild, innerElement);
                    if (Bridge.is(checkBoxOrRadioButton, Windows.UI.Xaml.Controls.RadioButton)) {
                        CSHTML5.Internal.INTERNAL_EventsHelper.AttachToDomEvents("change", innerElement, function (e) {
                            Bridge.cast(checkBoxOrRadioButton, Windows.UI.Xaml.Controls.RadioButton).ChangedEventReceived();
                        });
                    } else {
                        CSHTML5.Internal.INTERNAL_EventsHelper.AttachToDomEvents("change", innerElement, function (e) {
                            Windows.UI.Xaml.Controls.INTERNAL_CheckBoxAndRadioButtonHelpers.IsCheckedValueChanged(checkBoxOrRadioButton);
                        });
                    }
                },
                IsCheckedValueChanged: function (checkBoxOrRadioButton) {

                    if (!checkBoxOrRadioButton.INTERNAL_IsCodeProgrammaticallyChangingIsChecked) {
                        if (CSHTML5.Internal.INTERNAL_VisualTreeManager.IsElementInVisualTree(checkBoxOrRadioButton)) {

                            var formerIsChecked = checkBoxOrRadioButton.IsChecked;
                            checkBoxOrRadioButton.INTERNAL_IsCodeProgrammaticallyChangingIsChecked = true;
                            if (formerIsChecked == null) {
                                checkBoxOrRadioButton.SetCurrentValue(Windows.UI.Xaml.Controls.Primitives.ToggleButton.IsCheckedProperty, Bridge.box(false, System.Boolean, System.Boolean.toString));
                            } else {
                                if (checkBoxOrRadioButton.IsThreeState && System.Nullable.eq(formerIsChecked, true)) {
                                    checkBoxOrRadioButton.SetCurrentValue(Windows.UI.Xaml.Controls.Primitives.ToggleButton.IsCheckedProperty, null);
                                } else {
                                    checkBoxOrRadioButton.SetCurrentValue(Windows.UI.Xaml.Controls.Primitives.ToggleButton.IsCheckedProperty, Bridge.box(!System.Nullable.getValue(formerIsChecked), System.Boolean, System.Boolean.toString));
                                }
                            }
                            checkBoxOrRadioButton.INTERNAL_IsCodeProgrammaticallyChangingIsChecked = false;
                        }
                    }
                },
                UpdateDomBasedOnCheckedState: function (checkBoxOrRadioButton, isChecked) {
                    if (isChecked == null) {
                        CSHTML5.Internal.INTERNAL_HtmlDomManager.SetDomElementProperty(checkBoxOrRadioButton.INTERNAL_OptionalSpecifyDomElementConcernedByFocus, "indeterminate", Bridge.box(true, System.Boolean, System.Boolean.toString), true);
                    } else if (System.Nullable.eq(isChecked, false)) {
                        CSHTML5.Internal.INTERNAL_HtmlDomManager.SetDomElementProperty(checkBoxOrRadioButton.INTERNAL_OptionalSpecifyDomElementConcernedByFocus, "indeterminate", Bridge.box(false, System.Boolean, System.Boolean.toString), true);

                        CSHTML5.Internal.INTERNAL_HtmlDomManager.SetDomElementProperty(checkBoxOrRadioButton.INTERNAL_OptionalSpecifyDomElementConcernedByFocus, "checked", Bridge.box(false, System.Boolean, System.Boolean.toString), true);
                    } else {
                        CSHTML5.Internal.INTERNAL_HtmlDomManager.SetDomElementProperty(checkBoxOrRadioButton.INTERNAL_OptionalSpecifyDomElementConcernedByFocus, "indeterminate", Bridge.box(false, System.Boolean, System.Boolean.toString), true);
                        CSHTML5.Internal.INTERNAL_HtmlDomManager.SetDomElementProperty(checkBoxOrRadioButton.INTERNAL_OptionalSpecifyDomElementConcernedByFocus, "checked", Bridge.box(true, System.Boolean, System.Boolean.toString), true);
                    }
                }
            }
        }
    });

    Bridge.define("Windows.UI.Xaml.Controls.INTERNAL_ContextMenuHelpers", {
        statics: {
            methods: {
                RegisterContextMenu$1: function (frameworkElement, contextMenu) {
                    if (contextMenu != null) {
                        contextMenu.INTERNAL_ElementToWhichThisMenuIsAssigned = frameworkElement;

                        frameworkElement.removeRightTapped(Windows.UI.Xaml.Controls.INTERNAL_ContextMenuHelpers.FrameworkElement_RightTapped);
                        frameworkElement.addRightTapped(Windows.UI.Xaml.Controls.INTERNAL_ContextMenuHelpers.FrameworkElement_RightTapped);
                    } else {
                        Windows.UI.Xaml.Controls.INTERNAL_ContextMenuHelpers.UnregisterContextMenu$1(frameworkElement);
                    }
                },
                RegisterContextMenu: function (htmlCanvasElement, contextMenu) {
                    if (contextMenu != null) {
                        contextMenu.INTERNAL_HtmlCanvasElementToWhichThisMenuIsAssigned = htmlCanvasElement;

                        htmlCanvasElement.removeRightTapped(Windows.UI.Xaml.Controls.INTERNAL_ContextMenuHelpers.HtmlCanvasElement_RightTapped);
                        htmlCanvasElement.addRightTapped(Windows.UI.Xaml.Controls.INTERNAL_ContextMenuHelpers.HtmlCanvasElement_RightTapped);
                    } else {
                        Windows.UI.Xaml.Controls.INTERNAL_ContextMenuHelpers.UnregisterContextMenu(htmlCanvasElement);
                    }
                },
                htmlCanvasElement_RightTapped: function (sender, e) {

                },
                UnregisterContextMenu$1: function (frameworkElement) {
                    frameworkElement.removeRightTapped(Windows.UI.Xaml.Controls.INTERNAL_ContextMenuHelpers.FrameworkElement_RightTapped);
                },
                UnregisterContextMenu: function (htmlCanvasElement) {
                    htmlCanvasElement.removeRightTapped(Windows.UI.Xaml.Controls.INTERNAL_ContextMenuHelpers.HtmlCanvasElement_RightTapped);
                },
                FrameworkElement_RightTapped: function (sender, e) {
                    var frameworkElement = Bridge.cast(sender, Windows.UI.Xaml.FrameworkElement);
                    var contextMenu = frameworkElement.ContextMenu;
                    if (contextMenu != null) {
                        if (contextMenu.IsOpen === false) {
                            var pointerPosition = e.GetCurrentPoint(null).Position.$clone();
                            frameworkElement.INTERNAL_RaiseContextMenuOpeningEvent(pointerPosition.X, pointerPosition.Y);
                            contextMenu.INTERNAL_OpenAtCoordinates(pointerPosition.$clone());
                        }
                    } else {
                        Windows.UI.Xaml.Controls.INTERNAL_ContextMenuHelpers.UnregisterContextMenu$1(frameworkElement);
                    }
                },
                HtmlCanvasElement_RightTapped: function (sender, e) {
                    var htmlCanvasElement = Bridge.cast(sender, CSHTML5.Native.Html.Controls.HtmlCanvasElement);
                    var contextMenu = htmlCanvasElement.ContextMenu;
                    if (contextMenu != null) {
                        if (contextMenu.IsOpen === false) {
                            var pointerPosition = e.GetCurrentPoint$1(null).Position.$clone();
                            htmlCanvasElement.INTERNAL_RaiseContextMenuOpeningEvent(pointerPosition.X, pointerPosition.Y);
                            contextMenu.INTERNAL_OpenAtCoordinates(pointerPosition.$clone());
                        }
                    } else {
                        Windows.UI.Xaml.Controls.INTERNAL_ContextMenuHelpers.UnregisterContextMenu(htmlCanvasElement);
                    }
                }
            }
        }
    });

    Bridge.define("Windows.UI.Xaml.Controls.ItemCollection.EnumerableWrapper", {
        inherits: [System.Collections.Generic.List$1(System.Object)],
        $kind: "nested class",
        fields: {
            _sourceCollection: null,
            _owner: null
        },
        ctors: {
            ctor: function (source, owner) {
                this.$initialize();
                System.Collections.Generic.List$1(System.Object).ctor.call(this);
                System.Diagnostics.Debug.Assert(source != null);
                System.Diagnostics.Debug.Assert(owner != null);
                this._sourceCollection = source;
                this._owner = owner;

                this.Refresh();
            }
        },
        methods: {
            Refresh: function () {
                this.clear();

                var enumerator = Bridge.getEnumerator(this._sourceCollection);
                while (enumerator.System$Collections$IEnumerator$moveNext()) {
                    this.add(enumerator.System$Collections$IEnumerator$Current);
                }
            }
        }
    });

    Bridge.define("Windows.UI.Xaml.Controls.ItemCollection.SimpleMonitor", {
        inherits: [System.IDisposable],
        $kind: "nested class",
        fields: {
            _busyCount: 0
        },
        props: {
            Busy: {
                get: function () {
                    return this._busyCount > 0;
                }
            }
        },
        alias: ["Dispose", "System$IDisposable$Dispose"],
        methods: {
            Enter: function () {
                this._busyCount = (this._busyCount + 1) | 0;
            },
            Dispose: function () {
                this._busyCount = (this._busyCount - 1) | 0;
            }
        }
    });

    Bridge.define("Windows.UI.Xaml.Controls.ItemContainerGenerator", {
        fields: {
            _itemsToContainers: null,
            _containerToItem: null,
            _containers: null
        },
        props: {
            /**
             * Gets all the containers associated to all the items.
             *
             * @instance
             * @public
             * @readonly
             * @memberof Windows.UI.Xaml.Controls.ItemContainerGenerator
             * @function INTERNAL_AllContainers
             * @type System.Collections.Generic.IEnumerable$1
             */
            INTERNAL_AllContainers: {
                get: function () {
                    return new (Bridge.GeneratorEnumerable$1(System.Object))(Bridge.fn.bind(this, function ()  {
                        var $step = 0,
                            $jumpFromFinally,
                            $returnValue,
                            $t,
                            containers,
                            $t1,
                            container,
                            $async_e;

                        var $enumerator = new (Bridge.GeneratorEnumerator$1(System.Object))(Bridge.fn.bind(this, function () {
                            try {
                                for (;;) {
                                    switch ($step) {
                                        case 0: {
                                            $t = Bridge.getEnumerator(INTERNAL_BridgeWorkarounds.GetDictionaryValues_SimulatorCompatible(System.Object, System.Collections.Generic.List$1(System.Object), this._itemsToContainers), System.Collections.Generic.List$1(System.Object));
                                                $step = 1;
                                                continue;
                                        }
                                        case 1: {
                                            if ($t.moveNext()) {
                                                    containers = $t.Current;
                                                    $step = 2;
                                                    continue;
                                                }
                                            $step = 7;
                                            continue;
                                        }
                                        case 2: {
                                            $t1 = Bridge.getEnumerator(containers);
                                                $step = 3;
                                                continue;
                                        }
                                        case 3: {
                                            if ($t1.moveNext()) {
                                                    container = $t1.Current;
                                                    $step = 4;
                                                    continue;
                                                }
                                            $step = 6;
                                            continue;
                                        }
                                        case 4: {
                                            $enumerator.current = container;
                                                $step = 5;
                                                return true;
                                        }
                                        case 5: {
                                            $step = 3;
                                            continue;
                                        }
                                        case 6: {
                                            $step = 1;
                                            continue;
                                        }
                                        case 7: {

                                        }
                                        default: {
                                            return false;
                                        }
                                    }
                                }
                            } catch($async_e1) {
                                $async_e = System.Exception.create($async_e1);
                                throw $async_e;
                            }
                        }));
                        return $enumerator;
                    }));
                }
            }
        },
        ctors: {
            init: function () {
                this._itemsToContainers = new (System.Collections.Generic.Dictionary$2(System.Object,System.Collections.Generic.List$1(System.Object))).ctor();
                this._containerToItem = new (System.Collections.Generic.Dictionary$2(System.Object,System.Object)).ctor();
                this._containers = new (System.Collections.Generic.List$1(System.Object)).ctor();
            }
        },
        methods: {
            /**
             * Returns the container corresponding to the specified item, or null if no container was found.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Controls.ItemContainerGenerator
             * @memberof Windows.UI.Xaml.Controls.ItemContainerGenerator
             * @param   {System.Object}                       item    The item to retrieve the container for.
             * @return  {Windows.UI.Xaml.DependencyObject}            A container that corresponds to the specified item, if the item has a container
             and exists in the collection; otherwise, null.
             */
            ContainerFromItem: function (item) {
                return Bridge.cast(this.INTERNAL_ContainerFromItem(item), Windows.UI.Xaml.DependencyObject);
            },
            INTERNAL_ContainerFromItem: function (item) {
                if (item != null && this._itemsToContainers.containsKey(item)) {
                    var containersAssociatedToTheItem = this._itemsToContainers.getItem(item);
                    if (containersAssociatedToTheItem != null && containersAssociatedToTheItem.Count > 0) {
                        return containersAssociatedToTheItem.getItem(0);
                    } else {
                        return null;
                    }
                } else {
                    return null;
                }
            },
            /**
             * Returns the container at the given index, or null if the index is negative or above the amount of containers.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Controls.ItemContainerGenerator
             * @memberof Windows.UI.Xaml.Controls.ItemContainerGenerator
             * @param   {number}                              index    The index at which the container is located.
             * @return  {Windows.UI.Xaml.DependencyObject}             The container at the given index, or null if the index is negative or above the amount of containers.
             */
            ContainerFromIndex: function (index) {
                if (index > -1 && this._containers.Count > index) {
                    return Bridge.cast(this._containers.getItem(index), Windows.UI.Xaml.DependencyObject);
                } else {
                    return null;
                }
            },
            IndexFromContainer: function (container) {
                return this._containers.indexOf(container);
            },
            /**
             * Removes the container associated to the item.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Controls.ItemContainerGenerator
             * @memberof Windows.UI.Xaml.Controls.ItemContainerGenerator
             * @param   {System.Object}    container            The container to remove.
             * @param   {System.Object}    correspondingItem    The item that corresponds to the container to remove.
             * @return  {boolean}                               True if found and removed, false otherwise.
             */
            INTERNAL_TryUnregisterContainer: function (container, correspondingItem) {
                var indexOfContainerInContainerList = this._containers.indexOf(container);
                if (indexOfContainerInContainerList !== -1) {
                    this._containers.remove(container);
                }
                if (this._itemsToContainers.containsKey(correspondingItem)) {
                    var containersAssociatedToTheItem = this._itemsToContainers.getItem(correspondingItem);
                    if (containersAssociatedToTheItem != null) {
                        if (containersAssociatedToTheItem.remove(container)) {

                            if (containersAssociatedToTheItem.Count === 0) {
                                this._itemsToContainers.remove(correspondingItem);
                            }

                            this._containerToItem.remove(container);

                            return true;
                        }
                    }
                }

                return false;
            },
            /**
             * // Removes the container at the given position. IMPORTANT NOTE: Contrary to expectations, this method is not as efficient as INTERNAL_TryUnregisterContainer so if given the choice, use that one instead.
            //
             * Resets the ItemContainerGenerator.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Controls.ItemContainerGenerator
             * @memberof Windows.UI.Xaml.Controls.ItemContainerGenerator
             * @return  {void}        True if found and removed, false otherwise
             */
            INTERNAL_Clear: function () {
                this._itemsToContainers.clear();
                this._containerToItem.clear();
                this._containers.clear();
            },
            /**
             * Adds a container to the collection of containers.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Controls.ItemContainerGenerator
             * @memberof Windows.UI.Xaml.Controls.ItemContainerGenerator
             * @param   {System.Object}    container            The container to add.
             * @param   {System.Object}    correspondingItem    The item that corresponds to the container to add.
             * @return  {void}
             */
            INTERNAL_RegisterContainer: function (container, correspondingItem) {
                var containersAssociatedToTheItem;
                if (this._itemsToContainers.containsKey(correspondingItem)) {
                    containersAssociatedToTheItem = this._itemsToContainers.getItem(correspondingItem);
                } else {
                    containersAssociatedToTheItem = new (System.Collections.Generic.List$1(System.Object)).ctor();
                    this._itemsToContainers.add(correspondingItem, containersAssociatedToTheItem);
                }
                containersAssociatedToTheItem.add(container);
                this._containerToItem.add(container, correspondingItem);
                this._containers.add(container);
            }
        }
    });

    /**
     * Information describing a drag event on a UIElement.
     *
     * @public
     * @class Windows.UI.Xaml.Controls.ItemDragEventArgs
     * @augments System.Object
     */
    Bridge.define("Windows.UI.Xaml.Controls.ItemDragEventArgs", {
        fields: {
            /**
             * Gets or sets a value indicating whether an item drag
             operation was handled.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.ItemDragEventArgs
             * @function Handled
             * @type boolean
             */
            Handled: false,
            /**
             * Gets or sets the effects of the completed drag operation.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.ItemDragEventArgs
             * @function Effects
             * @type System.Windows.DragDropEffects
             */
            Effects: 0,
            /**
             * Gets or sets the data associated with the item container being dragged.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.ItemDragEventArgs
             * @function Data
             * @type System.Object
             */
            Data: null,
            /**
             * Gets or sets a value indicating whether to cancel the action.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.ItemDragEventArgs
             * @function Cancel
             * @type boolean
             */
            Cancel: false
        },
        ctors: {
            /**
             * Initializes a new instance of the ItemDragEventArgs class.
             *
             * @instance
             * @this Windows.UI.Xaml.Controls.ItemDragEventArgs
             * @memberof Windows.UI.Xaml.Controls.ItemDragEventArgs
             * @param   {Windows.UI.Xaml.Controls.SelectionCollection}    data
             * @return  {void}
             */
            ctor: function (data) {
                this.$initialize();
                System.Object.call(this);
                this.Data = data;
            }
        }
    });

    /**
     * Provides useful extensions to ItemsControl instances.
     *
     * @static
     * @abstract
     * @public
     * @class Windows.UI.Xaml.Controls.ItemsControlExtensions
     */
    Bridge.define("Windows.UI.Xaml.Controls.ItemsControlExtensions", {
        statics: {
            methods: {
                /**
                 * // Gets the Panel that contains the containers of an ItemsControl.
                //
                 * // Gets the ScrollViewer that contains the containers of an
                // ItemsControl.
                //
                 * Get the item containers of an ItemsControl.
                 *
                 * @static
                 * @public
                 * @this Windows.UI.Xaml.Controls.ItemsControlExtensions
                 * @memberof Windows.UI.Xaml.Controls.ItemsControlExtensions
                 * @throws // <b /> is null.
                //
                 * @throws // <b /> is null.
                //
                 * @throws <b /> is null.
                 * @param   {Windows.UI.Xaml.Controls.ItemsControl}       control    The ItemsControl.
                 * @return  {System.Collections.Generic.IEnumerable$1}               The item containers of an ItemsControl.
                 */
                GetContainers$1: function (control) {
                    if (control == null) {
                        throw new System.ArgumentNullException.$ctor1("control");
                    }

                    return Windows.UI.Xaml.Controls.ItemsControlExtensions.GetContainersIterator(Windows.UI.Xaml.DependencyObject, control);
                },
                /**
                 * Get the item containers of an ItemsControl.
                 *
                 * @static
                 * @public
                 * @this Windows.UI.Xaml.Controls.ItemsControlExtensions
                 * @memberof Windows.UI.Xaml.Controls.ItemsControlExtensions
                 * @throws <b /> is null.
                 * @param   {Function}                                    TContainer    The type of the item containers.
                 * @param   {Windows.UI.Xaml.Controls.ItemsControl}       control       The ItemsControl.
                 * @return  {System.Collections.Generic.IEnumerable$1}                  The item containers of an ItemsControl.
                 */
                GetContainers: function (TContainer, control) {
                    if (control == null) {
                        throw new System.ArgumentNullException.$ctor1("control");
                    }

                    return Windows.UI.Xaml.Controls.ItemsControlExtensions.GetContainersIterator(TContainer, control);
                },
                /**
                 * Get the item containers of an ItemsControl.
                 *
                 * @static
                 * @private
                 * @this Windows.UI.Xaml.Controls.ItemsControlExtensions
                 * @memberof Windows.UI.Xaml.Controls.ItemsControlExtensions
                 * @param   {Function}                                    TContainer    The type of the item containers.
                 * @param   {Windows.UI.Xaml.Controls.ItemsControl}       control       The ItemsControl.
                 * @return  {System.Collections.Generic.IEnumerable$1}                  The item containers of an ItemsControl.
                 */
                GetContainersIterator: function (TContainer, control) {
                    return System.Linq.Enumerable.from(Windows.UI.Xaml.Controls.ItemsControlExtensions.GetItemsAndContainers(TContainer, control)).select(function (p) {
                            return p.value;
                        });
                },
                /**
                 * Get the items and item containers of an ItemsControl.
                 *
                 * @static
                 * @public
                 * @this Windows.UI.Xaml.Controls.ItemsControlExtensions
                 * @memberof Windows.UI.Xaml.Controls.ItemsControlExtensions
                 * @throws <b /> is null.
                 * @param   {Windows.UI.Xaml.Controls.ItemsControl}       control    The ItemsControl.
                 * @return  {System.Collections.Generic.IEnumerable$1}               The items and item containers of an ItemsControl.
                 */
                GetItemsAndContainers$1: function (control) {
                    if (control == null) {
                        throw new System.ArgumentNullException.$ctor1("control");
                    }

                    return Windows.UI.Xaml.Controls.ItemsControlExtensions.GetItemsAndContainersIterator(Windows.UI.Xaml.DependencyObject, control);
                },
                /**
                 * Get the items and item containers of an ItemsControl.
                 *
                 * @static
                 * @public
                 * @this Windows.UI.Xaml.Controls.ItemsControlExtensions
                 * @memberof Windows.UI.Xaml.Controls.ItemsControlExtensions
                 * @throws <b /> is null.
                 * @param   {Function}                                    TContainer    The type of the item containers.
                 * @param   {Windows.UI.Xaml.Controls.ItemsControl}       control       The ItemsControl.
                 * @return  {System.Collections.Generic.IEnumerable$1}                  The items and item containers of an ItemsControl.
                 */
                GetItemsAndContainers: function (TContainer, control) {
                    if (control == null) {
                        throw new System.ArgumentNullException.$ctor1("control");
                    }

                    return Windows.UI.Xaml.Controls.ItemsControlExtensions.GetItemsAndContainersIterator(TContainer, control);
                },
                /**
                 * Get the items and item containers of an ItemsControl.
                 *
                 * @static
                 * @private
                 * @this Windows.UI.Xaml.Controls.ItemsControlExtensions
                 * @memberof Windows.UI.Xaml.Controls.ItemsControlExtensions
                 * @throws <b /> is null.
                 * @param   {Function}                                    TContainer    The type of the item containers.
                 * @param   {Windows.UI.Xaml.Controls.ItemsControl}       control       The ItemsControl.
                 * @return  {System.Collections.Generic.IEnumerable$1}                  The items and item containers of an ItemsControl.
                 */
                GetItemsAndContainersIterator: function (TContainer, control) {
                    return new (Bridge.GeneratorEnumerable$1(System.Collections.Generic.KeyValuePair$2(System.Object,TContainer)))(Bridge.fn.bind(this, function (TContainer, control) {
                        var $step = 0,
                            $jumpFromFinally,
                            $returnValue,
                            count,
                            i,
                            container,
                            $async_e;

                        var $enumerator = new (Bridge.GeneratorEnumerator$1(System.Collections.Generic.KeyValuePair$2(System.Object,TContainer)))(Bridge.fn.bind(this, function () {
                            try {
                                for (;;) {
                                    switch ($step) {
                                        case 0: {
                                            count = control.Items.Count;
                                                i = 0;
                                                $step = 1;
                                                continue;
                                        }
                                        case 1: {
                                            if ( i < count ) {
                                                    $step = 2;
                                                    continue;
                                                }
                                            $step = 5;
                                            continue;
                                        }
                                        case 2: {
                                            container = control.ItemContainerGenerator.ContainerFromIndex(i);
                                                if (container == null) {
                                                    $step = 4;
                                                    continue;
                                                }

                                                $enumerator.current = new (System.Collections.Generic.KeyValuePair$2(System.Object,TContainer)).$ctor1(control.Items.getItem(i), Bridge.as(container, TContainer));
                                                $step = 3;
                                                return true;
                                        }
                                        case 3: {
                                            $step = 4;
                                            continue;
                                        }
                                        case 4: {
                                            i = (i + 1) | 0;
                                            $step = 1;
                                            continue;
                                        }
                                        case 5: {

                                        }
                                        default: {
                                            return false;
                                        }
                                    }
                                }
                            } catch($async_e1) {
                                $async_e = System.Exception.create($async_e1);
                                throw $async_e;
                            }
                        }));
                        return $enumerator;
                    }, arguments));
                },
                /**
                 * Returns a value indicating whether an item can be added to an
                 ItemsControl.
                 *
                 * @static
                 * @this Windows.UI.Xaml.Controls.ItemsControlExtensions
                 * @memberof Windows.UI.Xaml.Controls.ItemsControlExtensions
                 * @param   {Windows.UI.Xaml.Controls.ItemsControl}    that    The ItemsControl instance.
                 * @param   {System.Object}                            item    The item to be added.
                 * @return  {boolean}                                          A value Indicating whether an item can be added to an
                 ItemsControl.
                 */
                CanAddItem: function (that, item) {
                    if (that.ItemsSource == null) {
                        return true;
                    } else {
                        return Windows.UI.Xaml.Controls.CollectionHelper.CanInsert(that.ItemsSource, item);
                    }
                },
                /**
                 * Returns a value indicating whether an item can be removed from the
                 ItemsControl.
                 *
                 * @static
                 * @this Windows.UI.Xaml.Controls.ItemsControlExtensions
                 * @memberof Windows.UI.Xaml.Controls.ItemsControlExtensions
                 * @param   {Windows.UI.Xaml.Controls.ItemsControl}    that    The items control.
                 * @return  {boolean}                                          A value indicating whether an item can be removed from the
                 ItemsControl.
                 */
                CanRemoveItem: function (that) {
                    if (that.ItemsSource != null) {
                        return !Windows.UI.Xaml.Controls.CollectionHelper.IsReadOnly(that.ItemsSource) && Bridge.is(that.ItemsSource, System.Collections.Specialized.INotifyCollectionChanged);
                    }
                    return true;
                },
                /**
                 * Inserts an item into an ItemsControl.
                 *
                 * @static
                 * @this Windows.UI.Xaml.Controls.ItemsControlExtensions
                 * @memberof Windows.UI.Xaml.Controls.ItemsControlExtensions
                 * @param   {Windows.UI.Xaml.Controls.ItemsControl}    that     The ItemsControl instance.
                 * @param   {number}                                   index    The index at which to insert the item.
                 * @param   {System.Object}                            item     The item to be inserted.
                 * @return  {void}
                 */
                InsertItem: function (that, index, item) {
                    if (that.ItemsSource == null) {
                        that.Items.insert(index, item);
                    } else {
                        Windows.UI.Xaml.Controls.CollectionHelper.Insert(that.ItemsSource, index, item);
                    }
                },
                /**
                 * Adds an item to an ItemsControl.
                 *
                 * @static
                 * @this Windows.UI.Xaml.Controls.ItemsControlExtensions
                 * @memberof Windows.UI.Xaml.Controls.ItemsControlExtensions
                 * @param   {Windows.UI.Xaml.Controls.ItemsControl}    that    The ItemsControl instance.
                 * @param   {System.Object}                            item    The item to be inserted.
                 * @return  {void}
                 */
                AddItem: function (that, item) {
                    if (that.ItemsSource == null) {
                        Windows.UI.Xaml.Controls.ItemsControlExtensions.InsertItem(that, that.Items.Count, item);
                    } else {
                        Windows.UI.Xaml.Controls.CollectionHelper.Add(that.ItemsSource, item);
                    }
                },
                /**
                 * Removes an item from an ItemsControl.
                 *
                 * @static
                 * @this Windows.UI.Xaml.Controls.ItemsControlExtensions
                 * @memberof Windows.UI.Xaml.Controls.ItemsControlExtensions
                 * @param   {Windows.UI.Xaml.Controls.ItemsControl}    that    The ItemsControl instance.
                 * @param   {System.Object}                            item    The item to be removed.
                 * @return  {void}
                 */
                RemoveItem: function (that, item) {
                    if (that.ItemsSource == null) {
                        that.Items.remove(item);
                    } else {
                        Windows.UI.Xaml.Controls.CollectionHelper.Remove(that.ItemsSource, item);
                    }
                },
                /**
                 * Removes an item from an ItemsControl.
                 *
                 * @static
                 * @this Windows.UI.Xaml.Controls.ItemsControlExtensions
                 * @memberof Windows.UI.Xaml.Controls.ItemsControlExtensions
                 * @param   {Windows.UI.Xaml.Controls.ItemsControl}    that     The ItemsControl instance.
                 * @param   {number}                                   index    The index of the item to be removed.
                 * @return  {void}
                 */
                RemoveItemAtIndex: function (that, index) {
                    if (that.ItemsSource == null) {
                        that.Items.removeAt(index);
                    } else {
                        Windows.UI.Xaml.Controls.CollectionHelper.RemoveAt(that.ItemsSource, index);
                    }
                },
                /**
                 * Gets the number of items in an ItemsControl.
                 *
                 * @static
                 * @this Windows.UI.Xaml.Controls.ItemsControlExtensions
                 * @memberof Windows.UI.Xaml.Controls.ItemsControlExtensions
                 * @param   {Windows.UI.Xaml.Controls.ItemsControl}    that    The ItemsControl instance.
                 * @return  {number}                                           The number of items in the ItemsControl.
                 */
                GetItemCount: function (that) {
                    if (that.ItemsSource == null) {
                        return that.Items.Count;
                    } else {
                        return Windows.UI.Xaml.Controls.CollectionHelper.Count(that.ItemsSource);
                    }
                }
            }
        }
    });

    /**
     * The {@link } namespace provides UI controls and classes
     that support existing and custom controls.
     *
     * @class Windows.UI.Xaml.Controls.NamespaceDoc
     */
    Bridge.define("Windows.UI.Xaml.Controls.NamespaceDoc");

    /**
     * Determins whether the value bar behind the value text should be visible
     *
     * @public
     * @class Windows.UI.Xaml.Controls.NumericUpDownValueBarVisibility
     */
    Bridge.define("Windows.UI.Xaml.Controls.NumericUpDownValueBarVisibility", {
        $kind: "enum",
        statics: {
            fields: {
                /**
                 * Visible
                 *
                 * @static
                 * @public
                 * @memberof Windows.UI.Xaml.Controls.NumericUpDownValueBarVisibility
                 * @constant
                 * @default 0
                 * @type Windows.UI.Xaml.Controls.NumericUpDownValueBarVisibility
                 */
                Visible: 0,
                /**
                 * Collapsed
                 *
                 * @static
                 * @public
                 * @memberof Windows.UI.Xaml.Controls.NumericUpDownValueBarVisibility
                 * @constant
                 * @default 1
                 * @type Windows.UI.Xaml.Controls.NumericUpDownValueBarVisibility
                 */
                Collapsed: 1
            }
        }
    });

    /**
     * Defines constants that specify the different orientations that a control or layout can have.
     *
     * @public
     * @class Windows.UI.Xaml.Controls.Orientation
     */
    Bridge.define("Windows.UI.Xaml.Controls.Orientation", {
        $kind: "enum",
        statics: {
            fields: {
                /**
                 * The control or layout should be vertically oriented.
                 *
                 * @static
                 * @public
                 * @memberof Windows.UI.Xaml.Controls.Orientation
                 * @constant
                 * @default 0
                 * @type Windows.UI.Xaml.Controls.Orientation
                 */
                Vertical: 0,
                /**
                 * The control or layout should be horizontally oriented.
                 *
                 * @static
                 * @public
                 * @memberof Windows.UI.Xaml.Controls.Orientation
                 * @constant
                 * @default 1
                 * @type Windows.UI.Xaml.Controls.Orientation
                 */
                Horizontal: 1
            }
        }
    });

    /** @namespace Windows.UI.Xaml.Controls.Primitives */

    /**
     * The {@link } namespace defines classes
     that represent the component parts of UI controls, or otherwise support the control
     composition model. Also defines interfaces for control patterns such as snapping and selection.
     *
     * @class Windows.UI.Xaml.Controls.Primitives.NamespaceDoc
     */
    Bridge.define("Windows.UI.Xaml.Controls.Primitives.NamespaceDoc");

    /**
     * Specifies the preferred location for positioning a ToolTip relative to a visual element.
     *
     * @public
     * @class Windows.UI.Xaml.Controls.Primitives.PlacementMode
     */
    Bridge.define("Windows.UI.Xaml.Controls.Primitives.PlacementMode", {
        $kind: "enum",
        statics: {
            fields: {
                /**
                 * Indicates that the preferred location of the tooltip is at the bottom of the target element.
                 *
                 * @static
                 * @public
                 * @memberof Windows.UI.Xaml.Controls.Primitives.PlacementMode
                 * @constant
                 * @default 2
                 * @type Windows.UI.Xaml.Controls.Primitives.PlacementMode
                 */
                Bottom: 2,
                /**
                 * Indicates that the preferred location of the tooltip is at the right of the target element.
                 *
                 * @static
                 * @public
                 * @memberof Windows.UI.Xaml.Controls.Primitives.PlacementMode
                 * @constant
                 * @default 4
                 * @type Windows.UI.Xaml.Controls.Primitives.PlacementMode
                 */
                Right: 4,
                /**
                 * Indicates that the preferred location of the tooltip is at the mouse pointer location.
                 *
                 * @static
                 * @public
                 * @memberof Windows.UI.Xaml.Controls.Primitives.PlacementMode
                 * @constant
                 * @default 7
                 * @type Windows.UI.Xaml.Controls.Primitives.PlacementMode
                 */
                Mouse: 7,
                /**
                 * Indicates that the preferred location of the tooltip is at the left of the target element.
                 *
                 * @static
                 * @public
                 * @memberof Windows.UI.Xaml.Controls.Primitives.PlacementMode
                 * @constant
                 * @default 9
                 * @type Windows.UI.Xaml.Controls.Primitives.PlacementMode
                 */
                Left: 9,
                /**
                 * Indicates that the preferred location of the tooltip is at the top of the target element.
                 *
                 * @static
                 * @public
                 * @memberof Windows.UI.Xaml.Controls.Primitives.PlacementMode
                 * @constant
                 * @default 10
                 * @type Windows.UI.Xaml.Controls.Primitives.PlacementMode
                 */
                Top: 10
            }
        }
    });

    /**
     * Specifies the type of Scroll event that occurred.
     *
     * @public
     * @class Windows.UI.Xaml.Controls.Primitives.ScrollEventType
     */
    Bridge.define("Windows.UI.Xaml.Controls.Primitives.ScrollEventType", {
        $kind: "enum",
        statics: {
            fields: {
                /**
                 * Specifies that the Thumb moved a specified distance, as determined by the
                 value of SmallChange. The Thumb moved to the left for a horizontal ScrollBar
                 or upward for a vertical ScrollBar.
                 *
                 * @static
                 * @public
                 * @memberof Windows.UI.Xaml.Controls.Primitives.ScrollEventType
                 * @constant
                 * @default 0
                 * @type Windows.UI.Xaml.Controls.Primitives.ScrollEventType
                 */
                SmallDecrement: 0,
                /**
                 * Specifies that the Thumb moved a specified distance, as determined by the
                 value of SmallChange. The Thumb moved to the right for a horizontal ScrollBar
                 or downward for a vertical ScrollBar.
                 *
                 * @static
                 * @public
                 * @memberof Windows.UI.Xaml.Controls.Primitives.ScrollEventType
                 * @constant
                 * @default 1
                 * @type Windows.UI.Xaml.Controls.Primitives.ScrollEventType
                 */
                SmallIncrement: 1,
                /**
                 * Specifies that the Thumb moved a specified distance, as determined by the
                 value of LargeChange. The Thumb moved to the left for a horizontal ScrollBar
                 or upward for a vertical ScrollBar.
                 *
                 * @static
                 * @public
                 * @memberof Windows.UI.Xaml.Controls.Primitives.ScrollEventType
                 * @constant
                 * @default 2
                 * @type Windows.UI.Xaml.Controls.Primitives.ScrollEventType
                 */
                LargeDecrement: 2,
                /**
                 * Specifies that the Thumb moved a specified distance, as determined by the
                 value of LargeChange. The Thumb moved to the right for a horizontal ScrollBar
                 or downward for a vertical ScrollBar.
                 *
                 * @static
                 * @public
                 * @memberof Windows.UI.Xaml.Controls.Primitives.ScrollEventType
                 * @constant
                 * @default 3
                 * @type Windows.UI.Xaml.Controls.Primitives.ScrollEventType
                 */
                LargeIncrement: 3,
                /**
                 * // Specifies that the Thumb moved to a new position because the user selected
                // Scroll Here in the shortcut menu of the ScrollBar.
                //
                 * The Thumb was dragged and caused a PointerMoved event. A Scroll event of
                 this ScrollEventType may occur more than one time when the Thumb is dragged
                 in the ScrollBar.
                 *
                 * @static
                 * @public
                 * @memberof Windows.UI.Xaml.Controls.Primitives.ScrollEventType
                 * @constant
                 * @default 5
                 * @type Windows.UI.Xaml.Controls.Primitives.ScrollEventType
                 */
                ThumbTrack: 5,
                /**
                 * // Specifies that the Thumb moved to the Minimum position of the ScrollBar.
                //
                 * // Specifies that the Thumb moved to the Minimum position of the ScrollBar.
                //
                 * Specifies that the Thumb was dragged to a new position and is now no longer
                 being dragged by the user.
                 *
                 * @static
                 * @public
                 * @memberof Windows.UI.Xaml.Controls.Primitives.ScrollEventType
                 * @constant
                 * @default 8
                 * @type Windows.UI.Xaml.Controls.Primitives.ScrollEventType
                 */
                EndScroll: 8
            }
        }
    });

    Bridge.define("Windows.UI.Xaml.Controls.Primitives.Selector.SelectionInfo", {
        $kind: "nested class",
        fields: {
            Index: 0,
            Item: null
        },
        ctors: {
            ctor: function (item, index) {
                this.$initialize();
                this.Item = item;
                this.Index = index;
            }
        }
    });

    /**
     * Specifies the visibility of a scrollbar within a ScrollViewer control.
     *
     * @public
     * @class Windows.UI.Xaml.Controls.ScrollBarVisibility
     */
    Bridge.define("Windows.UI.Xaml.Controls.ScrollBarVisibility", {
        $kind: "enum",
        statics: {
            fields: {
                /**
                 * [DOESN'T WORK YET, BEHAVES LIKE Hidden]
                 A ScrollBar does not appear even when the viewport cannot display all of
                 the content. The dimension of the content is set to the corresponding dimension
                 of the ScrollViewer parent. For a horizontal ScrollBar, the width of the
                 content is set to the ViewportWidth of the ScrollViewer. For a vertical ScrollBar,
                 the height of the content is set to the ViewportHeight of the ScrollViewer.
                 *
                 * @static
                 * @public
                 * @memberof Windows.UI.Xaml.Controls.ScrollBarVisibility
                 * @constant
                 * @default 0
                 * @type Windows.UI.Xaml.Controls.ScrollBarVisibility
                 */
                Disabled: 0,
                /**
                 * A ScrollBar appears and the dimension of the ScrollViewer is applied to the
                 content when the viewport cannot display all of the content. For a horizontal
                 ScrollBar, the width of the content is set to the ViewportWidth of the ScrollViewer.
                 For a vertical ScrollBar, the height of the content is set to the ViewportHeight
                 of the ScrollViewer.
                 *
                 * @static
                 * @public
                 * @memberof Windows.UI.Xaml.Controls.ScrollBarVisibility
                 * @constant
                 * @default 1
                 * @type Windows.UI.Xaml.Controls.ScrollBarVisibility
                 */
                Auto: 1,
                /**
                 * A ScrollBar does not appear even when the viewport cannot display all of
                 the content. The dimension of the ScrollViewer is not applied to the content.
                 *
                 * @static
                 * @public
                 * @memberof Windows.UI.Xaml.Controls.ScrollBarVisibility
                 * @constant
                 * @default 2
                 * @type Windows.UI.Xaml.Controls.ScrollBarVisibility
                 */
                Hidden: 2,
                /**
                 * A ScrollBar always appears. The dimension of the ScrollViewer is applied
                 to the content. For a horizontal ScrollBar, the width of the content is set
                 to the ViewportWidth of the ScrollViewer. For a vertical ScrollBar, the height
                 of the content is set to the ViewportHeight of the ScrollViewer.
                 *
                 * @static
                 * @public
                 * @memberof Windows.UI.Xaml.Controls.ScrollBarVisibility
                 * @constant
                 * @default 3
                 * @type Windows.UI.Xaml.Controls.ScrollBarVisibility
                 */
                Visible: 3
            }
        }
    });

    /**
     * Defines constants that specify scrolling behavior for ScrollViewer and other
     parts involved in scrolling scenarios.
     *
     * @public
     * @class Windows.UI.Xaml.Controls.ScrollMode
     */
    Bridge.define("Windows.UI.Xaml.Controls.ScrollMode", {
        $kind: "enum",
        statics: {
            fields: {
                /**
                 * Scrolling is disabled.
                 *
                 * @static
                 * @public
                 * @memberof Windows.UI.Xaml.Controls.ScrollMode
                 * @constant
                 * @default 0
                 * @type Windows.UI.Xaml.Controls.ScrollMode
                 */
                Disabled: 0,
                /**
                 * Scrolling is enabled.
                 *
                 * @static
                 * @public
                 * @memberof Windows.UI.Xaml.Controls.ScrollMode
                 * @constant
                 * @default 1
                 * @type Windows.UI.Xaml.Controls.ScrollMode
                 */
                Enabled: 1,
                /**
                 * Scrolling is enabled but behavior uses a "rails" manipulation mode.
                 *
                 * @static
                 * @public
                 * @memberof Windows.UI.Xaml.Controls.ScrollMode
                 * @constant
                 * @default 2
                 * @type Windows.UI.Xaml.Controls.ScrollMode
                 */
                Auto: 2
            }
        }
    });

    /**
     * A class that holds a selected item.
     *
     * @public
     * @class Windows.UI.Xaml.Controls.Selection
     */
    Bridge.define("Windows.UI.Xaml.Controls.Selection", {
        fields: {
            /**
             * Gets the index of the selection within the source collection.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.Selection
             * @function Index
             * @type ?number
             */
            Index: null,
            /**
             * Gets the selected item.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.Selection
             * @function Item
             * @type System.Object
             */
            Item: null
        },
        ctors: {
            /**
             * Initializes an instance of the Selection class.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Controls.Selection
             * @memberof Windows.UI.Xaml.Controls.Selection
             * @param   {?number}          index    The index of the selected item within the 
             source collection.
             * @param   {System.Object}    item     The selected item.
             * @return  {void}
             */
            ctor: function (index, item) {
                this.$initialize();
                this.Index = index;
                this.Item = item;
            },
            /**
             * Initializes an instance of the Selection class.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Controls.Selection
             * @memberof Windows.UI.Xaml.Controls.Selection
             * @param   {System.Object}    item    The selected item.
             * @return  {void}
             */
            $ctor1: function (item) {
                this.$initialize();
                this.Item = item;
            }
        }
    });

    /**
     * Defines the selection behavior for a ListBox.
     *
     * @public
     * @class Windows.UI.Xaml.Controls.SelectionMode
     */
    Bridge.define("Windows.UI.Xaml.Controls.SelectionMode", {
        $kind: "enum",
        statics: {
            fields: {
                /**
                 * The user can select only one item at a time.
                 *
                 * @static
                 * @public
                 * @memberof Windows.UI.Xaml.Controls.SelectionMode
                 * @constant
                 * @default 0
                 * @type Windows.UI.Xaml.Controls.SelectionMode
                 */
                Single: 0,
                /**
                 * The user can select multiple items without entering a special mode.
                 *
                 * @static
                 * @public
                 * @memberof Windows.UI.Xaml.Controls.SelectionMode
                 * @constant
                 * @default 1
                 * @type Windows.UI.Xaml.Controls.SelectionMode
                 */
                Multiple: 1,
                /**
                 * // The user can select multiple items by entering a special mode, for example
                // when pressing a modifier key.
                //
                 *
                 * @static
                 * @public
                 * @memberof Windows.UI.Xaml.Controls.SelectionMode
                 * @constant
                 * @default 2
                 * @type Windows.UI.Xaml.Controls.SelectionMode
                 */
                Extended: 2
            }
        }
    });

    /**
     * Describes the direction that content is scaled.
     *
     * @public
     * @class Windows.UI.Xaml.Controls.StretchDirection
     */
    Bridge.define("Windows.UI.Xaml.Controls.StretchDirection", {
        $kind: "enum",
        statics: {
            fields: {
                /**
                 * The content scales upward only when it is smaller than the parent. If the
                 content is larger, no scaling downward is performed.
                 *
                 * @static
                 * @public
                 * @memberof Windows.UI.Xaml.Controls.StretchDirection
                 * @constant
                 * @default 0
                 * @type Windows.UI.Xaml.Controls.StretchDirection
                 */
                UpOnly: 0,
                /**
                 * The content scales downward only when it is larger than the parent. If the
                 content is smaller, no scaling upward is performed.
                 *
                 * @static
                 * @public
                 * @memberof Windows.UI.Xaml.Controls.StretchDirection
                 * @constant
                 * @default 1
                 * @type Windows.UI.Xaml.Controls.StretchDirection
                 */
                DownOnly: 1,
                /**
                 * The content stretches to fit the parent according to the Stretch property.
                 *
                 * @static
                 * @public
                 * @memberof Windows.UI.Xaml.Controls.StretchDirection
                 * @constant
                 * @default 2
                 * @type Windows.UI.Xaml.Controls.StretchDirection
                 */
                Both: 2
            }
        }
    });

    /**
     * Represents a service that provides static methods to display a tooltip.
     *
     * @static
     * @abstract
     * @public
     * @class Windows.UI.Xaml.Controls.ToolTipService
     */
    Bridge.define("Windows.UI.Xaml.Controls.ToolTipService", {
        statics: {
            fields: {
                /**
                 * Identifies the ToolTipService.Placement XAML attached property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Controls.ToolTipService
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                PlacementProperty: null,
                /**
                 * Identifies the ToolTipService.PlacementTarget XAML attached property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Controls.ToolTipService
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                PlacementTargetProperty: null,
                /**
                 * Identifies the ToolTipService.ToolTip XAML attached property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Controls.ToolTipService
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                ToolTipProperty: null
            },
            ctors: {
                init: function () {
                    var $t;
                    this.PlacementProperty = Windows.UI.Xaml.DependencyProperty.RegisterAttached("Placement", Windows.UI.Xaml.Controls.Primitives.PlacementMode, Windows.UI.Xaml.Controls.ToolTipService, new Windows.UI.Xaml.PropertyMetadata.$ctor1(Bridge.box(Windows.UI.Xaml.Controls.Primitives.PlacementMode.Bottom, Windows.UI.Xaml.Controls.Primitives.PlacementMode, System.Enum.toStringFn(Windows.UI.Xaml.Controls.Primitives.PlacementMode))));
                    this.PlacementTargetProperty = Windows.UI.Xaml.DependencyProperty.RegisterAttached("PlacementTarget", Windows.UI.Xaml.UIElement, Windows.UI.Xaml.Controls.ToolTipService, new Windows.UI.Xaml.PropertyMetadata.$ctor4(null));
                    this.ToolTipProperty = Windows.UI.Xaml.DependencyProperty.RegisterAttached("ToolTip", System.Object, Windows.UI.Xaml.Controls.ToolTipService, ($t = new Windows.UI.Xaml.PropertyMetadata.$ctor2(null, Windows.UI.Xaml.Controls.ToolTipService.ToolTip_Changed), $t.CallPropertyChangedWhenLoadedIntoVisualTree = Windows.UI.Xaml.WhenToCallPropertyChangedEnum.IfPropertyIsSet, $t));
                }
            },
            methods: {
                ToolTip_Changed: function (d, e) {
                    var element = d;
                    if (Bridge.is(element, Windows.UI.Xaml.UIElement)) {
                        var uiElement = Bridge.cast(element, Windows.UI.Xaml.UIElement);
                        if (CSHTML5.Internal.INTERNAL_VisualTreeManager.IsElementInVisualTree(uiElement)) {
                            var openNewTooltip = false;
                            var position = null;
                            var oldTooltip = uiElement.INTERNAL_AssignedToolTip;
                            if (oldTooltip != null) {
                                openNewTooltip = oldTooltip.IsOpen;
                                position = System.Nullable.lift1("$clone", oldTooltip._forceSpecifyAbsoluteCoordinates);
                                oldTooltip.IsOpen = false;
                            }

                            var newTooltip = e.NewValue;

                            if (newTooltip != null) {
                                var toolTip = Windows.UI.Xaml.Controls.ToolTipService.ConvertToToolTip(newTooltip);
                                uiElement.INTERNAL_AssignedToolTip = toolTip;
                                toolTip.INTERNAL_ElementToWhichThisToolTipIsAssigned = uiElement;

                                if (openNewTooltip) {
                                    uiElement.INTERNAL_AssignedToolTip._forceSpecifyAbsoluteCoordinates = System.Nullable.lift1("$clone", position);
                                    uiElement.INTERNAL_AssignedToolTip.IsOpen = true;
                                }


                                uiElement.removePointerEntered(Windows.UI.Xaml.Controls.ToolTipService.UIElement_PointerEntered);
                                uiElement.addPointerEntered(Windows.UI.Xaml.Controls.ToolTipService.UIElement_PointerEntered);
                                uiElement.removePointerExited(Windows.UI.Xaml.Controls.ToolTipService.UIElement_PointerExited);
                                uiElement.addPointerExited(Windows.UI.Xaml.Controls.ToolTipService.UIElement_PointerExited);
                            } else {
                                uiElement.removePointerEntered(Windows.UI.Xaml.Controls.ToolTipService.UIElement_PointerEntered);
                                uiElement.removePointerExited(Windows.UI.Xaml.Controls.ToolTipService.UIElement_PointerExited);

                                uiElement.INTERNAL_AssignedToolTip = null;
                            }
                        }
                    }
                },
                ConvertToToolTip: function (obj) {
                    var $t;
                    if (Bridge.is(obj, Windows.UI.Xaml.Controls.ToolTip)) {
                        return Bridge.cast(obj, Windows.UI.Xaml.Controls.ToolTip);
                    } else {
                        if (Bridge.is(obj, Windows.UI.Xaml.UIElement) && Bridge.is(Bridge.cast(obj, Windows.UI.Xaml.UIElement).INTERNAL_VisualParent, Windows.UI.Xaml.Controls.ToolTip)) {
                            return Bridge.cast(Bridge.cast(obj, Windows.UI.Xaml.UIElement).INTERNAL_VisualParent, Windows.UI.Xaml.Controls.ToolTip);
                        } else {
                            return ($t = new Windows.UI.Xaml.Controls.ToolTip(), $t.Content = obj, $t);
                        }
                    }
                },
                UIElement_PointerExited: function (sender, e) {
                    var uielement = Bridge.cast(sender, Windows.UI.Xaml.UIElement);

                    if (uielement.INTERNAL_AssignedToolTip != null && uielement.INTERNAL_AssignedToolTip.IsOpen === true) {
                        uielement.INTERNAL_AssignedToolTip.IsOpen = false;
                    }
                },
                UIElement_PointerEntered: function (sender, e) {
                    var uielement = Bridge.cast(sender, Windows.UI.Xaml.UIElement);

                    if (uielement.INTERNAL_AssignedToolTip != null && uielement.INTERNAL_AssignedToolTip.IsOpen === false) {
                        var absoluteCoordinates = e.GetCurrentPoint(null).Position.$clone();
                        var absoluteCoordinatesShiftedToBeBelowThePointer = new Windows.Foundation.Point.$ctor1(absoluteCoordinates.X, absoluteCoordinates.Y + 20);
                        uielement.INTERNAL_AssignedToolTip.INTERNAL_OpenAtCoordinates(absoluteCoordinatesShiftedToBeBelowThePointer.$clone());
                    }
                },
                /**
                 * Gets the visual element that the tooltip is positioned relative to.
                 *
                 * @static
                 * @public
                 * @this Windows.UI.Xaml.Controls.ToolTipService
                 * @memberof Windows.UI.Xaml.Controls.ToolTipService
                 * @param   {Windows.UI.Xaml.DependencyObject}    element    The tooltip to retrieve the visual element of.
                 * @return  {Windows.UI.Xaml.UIElement}                      The visual element that the tooltip is positioned relative to.
                 */
                GetPlacementTarget: function (element) {
                    return Bridge.cast(element.GetValue(Windows.UI.Xaml.Controls.ToolTipService.PlacementTargetProperty), Windows.UI.Xaml.UIElement);
                },
                /**
                 * Sets the position of the specified {@link } relative to the specified value element.
                 *
                 * @static
                 * @public
                 * @this Windows.UI.Xaml.Controls.ToolTipService
                 * @memberof Windows.UI.Xaml.Controls.ToolTipService
                 * @param   {Windows.UI.Xaml.DependencyObject}    element    The tooltip to set the position of.
                 * @param   {Windows.UI.Xaml.UIElement}           value      The visual element to set the tooltip for.
                 * @return  {void}
                 */
                SetPlacementTarget: function (element, value) {
                    element.SetValue(Windows.UI.Xaml.Controls.ToolTipService.PlacementTargetProperty, Bridge.cast(value, Windows.UI.Xaml.DependencyObject));
                },
                /**
                 * Gets the value of the ToolTipService.ToolTip XAML attached property for an object.
                 *
                 * @static
                 * @public
                 * @this Windows.UI.Xaml.Controls.ToolTipService
                 * @memberof Windows.UI.Xaml.Controls.ToolTipService
                 * @param   {Windows.UI.Xaml.DependencyObject}    element    The object from which the property value is read.
                 * @return  {System.Object}                                  The object's tooltip content.
                 */
                GetToolTip: function (element) {
                    if (element == null) {
                        throw new System.ArgumentNullException.$ctor1("element");
                    }

                    return element.GetValue(Windows.UI.Xaml.Controls.ToolTipService.ToolTipProperty);
                },
                /**
                 * Sets the value of the ToolTipService.ToolTip XAML attached property.
                 *
                 * @static
                 * @public
                 * @this Windows.UI.Xaml.Controls.ToolTipService
                 * @memberof Windows.UI.Xaml.Controls.ToolTipService
                 * @param   {Windows.UI.Xaml.DependencyObject}    element    The object to set tooltip content on.
                 * @param   {System.Object}                       value      The value to set for tooltip content.
                 * @return  {void}
                 */
                SetToolTip: function (element, value) {
                    if (element == null) {
                        throw new System.ArgumentNullException.$ctor1("element");
                    }

                    element.SetValue(Windows.UI.Xaml.Controls.ToolTipService.ToolTipProperty, value);
                }
            }
        }
    });

    /**
     * Provides methods and attached properties that support data validation.
     * // Identifies the System.Windows.Controls.Validation.Error attached event.
    //
     * // Adds an event handler for the System.Windows.Controls.Validation.Error attached
    // event to the specified object.
    //
     * // Adds an event handler for the System.Windows.Controls.Validation.Error attached
    // event from the specified object.
    //
     * // Gets the value of the System.Windows.Controls.Validation.ErrorTemplate attached
    // property of the specified element.
    //
     * // Sets the value of the System.Windows.Controls.Validation.ErrorTemplate attached
    // property to the specified element.
    //
     * // Identifies the System.Windows.Controls.Validation.ErrorTemplate attached
    // property.
    //
     * // Identifies the System.Windows.Controls.Validation.ValidationAdornerSiteFor attached
    // property.
    //
     * // Gets the value of the System.Windows.Controls.Validation.ValidationAdornerSiteFor
    // attached property for the specified element.
    //
     * // Sets the System.Windows.Controls.Validation.ValidationAdornerSiteFor attached
    // property to the specified value on the specified element.
    //
     * // Identifies the System.Windows.Controls.Validation.ValidationAdornerSite attached
    // property.
    //
     * // Gets the value of the System.Windows.Controls.Validation.ValidationAdornerSite
    // attached property for the specified element.
    //
     * // Sets the System.Windows.Controls.Validation.ValidationAdornerSite attached
    // property to the specified value on the specified element.
    //
     *
     * @static
     * @abstract
     * @public
     * @class Windows.UI.Xaml.Controls.Validation
     * @return  {[type]}        The value of the System.Windows.Controls.Validation.ValidationAdornerSite.
     */
    Bridge.define("Windows.UI.Xaml.Controls.Validation", {
        statics: {
            fields: {
                /**
                 * Identifies the System.Windows.Controls.Validation.Errors attached property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Controls.Validation
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                ErrorsProperty: null,
                /**
                 * Identifies the System.Windows.Controls.Validation.HasError attached property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Controls.Validation
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                HasErrorProperty: null
            },
            ctors: {
                init: function () {
                    this.ErrorsProperty = Windows.UI.Xaml.DependencyProperty.RegisterAttached("Errors", System.Collections.ObjectModel.ObservableCollection$1(Windows.UI.Xaml.Controls.ValidationError), Windows.UI.Xaml.Controls.Validation, new Windows.UI.Xaml.PropertyMetadata.$ctor4(null));
                    this.HasErrorProperty = Windows.UI.Xaml.DependencyProperty.RegisterAttached("HasError", System.Boolean, Windows.UI.Xaml.Controls.Validation, new Windows.UI.Xaml.PropertyMetadata.$ctor1(Bridge.box(false, System.Boolean, System.Boolean.toString)));
                }
            },
            methods: {
                /**
                 * Gets the value of the System.Windows.Controls.Validation.Errors attached
                 property of the specified element.
                 *
                 * @static
                 * @public
                 * @this Windows.UI.Xaml.Controls.Validation
                 * @memberof Windows.UI.Xaml.Controls.Validation
                 * @param   {Windows.UI.Xaml.DependencyObject}                         element    The System.Windows.UIElement or System.Windows.ContentElement object to read
                 the value from.
                 * @return  {System.Collections.ObjectModel.ObservableCollection$1}               A System.Collections.ObjectModel.ReadOnlyObservableCollection`1 of System.Windows.Controls.ValidationError
                 objects.
                 */
                GetErrors: function (element) {
                    var returnValue = Bridge.cast(element.GetValue(Windows.UI.Xaml.Controls.Validation.ErrorsProperty), System.Collections.ObjectModel.ObservableCollection$1(Windows.UI.Xaml.Controls.ValidationError));
                    if (returnValue == null) {
                        returnValue = new (System.Collections.ObjectModel.ObservableCollection$1(Windows.UI.Xaml.Controls.ValidationError)).ctor();
                        element.SetValue(Windows.UI.Xaml.Controls.Validation.ErrorsProperty, returnValue);
                    }
                    return returnValue;
                },
                /**
                 * Gets the value of the System.Windows.Controls.Validation.HasError attached
                 property of the specified element.
                 *
                 * @static
                 * @public
                 * @this Windows.UI.Xaml.Controls.Validation
                 * @memberof Windows.UI.Xaml.Controls.Validation
                 * @param   {Windows.UI.Xaml.DependencyObject}    obj    The System.Windows.UIElement or System.Windows.ContentElement object to read
                 the value from.
                 * @return  {boolean}                                    The value of the System.Windows.Controls.Validation.HasError attached property
                 of the specified element.
                 */
                GetHasError: function (obj) {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(obj.GetValue(Windows.UI.Xaml.Controls.Validation.HasErrorProperty), System.Boolean), System.Boolean));
                },
                SetHasError: function (obj, value) {
                    obj.SetValue(Windows.UI.Xaml.Controls.Validation.HasErrorProperty, value);
                },
                /**
                 * Removes all System.Windows.Controls.ValidationError objects from the specified
                 System.Windows.Data.BindingExpressionBase object.
                 *
                 * @static
                 * @public
                 * @this Windows.UI.Xaml.Controls.Validation
                 * @memberof Windows.UI.Xaml.Controls.Validation
                 * @param   {Windows.UI.Xaml.Data.BindingExpression}    bindingExpression    The object to turn valid.
                 * @return  {void}
                 */
                ClearInvalid: function (bindingExpression) {
                    var $t;
                    if (bindingExpression == null) {
                        throw new System.ArgumentNullException.$ctor1("bindingExpression");
                    }
                    if (Bridge.is(bindingExpression.Target, Windows.UI.Xaml.UIElement)) {
                        var targetAsUIElement = Bridge.cast(bindingExpression.Target, Windows.UI.Xaml.UIElement);
                        if (targetAsUIElement.INTERNAL_ValidationErrorsDictionary != null && targetAsUIElement.INTERNAL_ValidationErrorsDictionary.containsKey(bindingExpression)) {
                            var error = targetAsUIElement.INTERNAL_ValidationErrorsDictionary.getItem(bindingExpression);
                            var errors = Windows.UI.Xaml.Controls.Validation.GetErrors(targetAsUIElement);
                            errors.remove(error);
                            targetAsUIElement.INTERNAL_ValidationErrorsDictionary.remove(bindingExpression);

                            if (errors.Count === 0) {
                                Windows.UI.Xaml.Controls.Validation.SetHasError(targetAsUIElement, Bridge.box(false, System.Boolean, System.Boolean.toString));
                            }

                            if (bindingExpression.ParentBinding.NotifyOnValidationError) {
                                var element = targetAsUIElement;
                                while (Bridge.is(element, Windows.UI.Xaml.FrameworkElement)) {
                                    Bridge.cast(element, Windows.UI.Xaml.FrameworkElement).INTERNAL_RaiseBindingValidationErrorEvent(($t = new Windows.UI.Xaml.Controls.ValidationErrorEventArgs(), $t.Action = Windows.UI.Xaml.Controls.ValidationErrorEventAction.Removed, $t.Error = error, $t.OriginalSource = targetAsUIElement, $t));

                                    element = Bridge.cast(element, Windows.UI.Xaml.FrameworkElement).Parent;
                                }
                            }

                            Windows.UI.Xaml.Controls.Validation.RefreshPopup(targetAsUIElement, errors);
                        }
                    }
                },
                /**
                 * Marks the specified System.Windows.Data.BindingExpression object as invalid
                 with the specified System.Windows.Controls.ValidationError object.
                 *
                 * @static
                 * @public
                 * @this Windows.UI.Xaml.Controls.Validation
                 * @memberof Windows.UI.Xaml.Controls.Validation
                 * @param   {Windows.UI.Xaml.Data.BindingExpression}      bindingExpression    The System.Windows.Data.BindingExpression object to mark as invalid.
                 * @param   {Windows.UI.Xaml.Controls.ValidationError}    validationError      The System.Windows.Controls.ValidationError object to use.
                 * @return  {void}
                 */
                MarkInvalid: function (bindingExpression, validationError) {
                    var $t;
                    if (bindingExpression == null) {
                        throw new System.ArgumentNullException.$ctor1("bindingExpression");
                    }
                    if (validationError == null) {
                        throw new System.ArgumentNullException.$ctor1("validationError");
                    }
                    var target = bindingExpression.Target;
                    if (Bridge.is(target, Windows.UI.Xaml.UIElement)) {
                        var targetAsUIElement = Bridge.cast(target, Windows.UI.Xaml.UIElement);

                        Windows.UI.Xaml.Controls.Validation.ClearInvalid(bindingExpression);

                        var errors = Windows.UI.Xaml.Controls.Validation.GetErrors(targetAsUIElement);
                        var setHasError = errors.Count === 0;

                        if (targetAsUIElement.INTERNAL_ValidationErrorsDictionary == null) {
                            targetAsUIElement.INTERNAL_ValidationErrorsDictionary = new (System.Collections.Generic.Dictionary$2(Windows.UI.Xaml.Data.BindingExpression,Windows.UI.Xaml.Controls.ValidationError)).ctor();
                        }
                        targetAsUIElement.INTERNAL_ValidationErrorsDictionary.add(bindingExpression, validationError);
                        errors.add(validationError);


                        if (setHasError) {
                            Windows.UI.Xaml.Controls.Validation.SetHasError(targetAsUIElement, Bridge.box(true, System.Boolean, System.Boolean.toString));
                        }

                        if (bindingExpression.ParentBinding.NotifyOnValidationError) {
                            var element = target;
                            while (Bridge.is(element, Windows.UI.Xaml.FrameworkElement)) {
                                Bridge.cast(element, Windows.UI.Xaml.FrameworkElement).INTERNAL_RaiseBindingValidationErrorEvent(($t = new Windows.UI.Xaml.Controls.ValidationErrorEventArgs(), $t.Action = Windows.UI.Xaml.Controls.ValidationErrorEventAction.Added, $t.Error = validationError, $t.OriginalSource = targetAsUIElement, $t));
                                element = Bridge.cast(element, Windows.UI.Xaml.FrameworkElement).Parent;
                            }
                        }


                        Windows.UI.Xaml.Controls.Validation.RefreshPopup(targetAsUIElement, errors);
                    }

                },
                RefreshPopup: function (target, errors) {
                    var $t;
                    if (CSHTML5.Internal.INTERNAL_VisualTreeManager.IsElementInVisualTree(target)) {
                        var popup = target.INTERNAL_ValidationErrorPopup;
                        if (errors.Count === 0) {
                            if (popup != null) {
                                popup.IsOpen = false;
                            }
                        } else {
                            if (popup == null) {
                                popup = new Windows.UI.Xaml.Controls.Primitives.Popup();
                                popup.addINTERNAL_PopupMoved(Windows.UI.Xaml.Controls.Validation.Popup_PopupMoved);
                                target.INTERNAL_ValidationErrorPopup = popup;

                                var border = ($t = new Windows.UI.Xaml.Controls.Border(), $t.Background = new Windows.UI.Xaml.Media.SolidColorBrush.$ctor1(Windows.UI.Color.FromArgb(255, 219, 2, 12)), $t.CornerRadius = new Windows.UI.Xaml.CornerRadius.$ctor1(2), $t.Margin = new Windows.UI.Xaml.Thickness.$ctor2(5, 0, 0, 0), $t);
                                var textBlock = ($t = new Windows.UI.Xaml.Controls.TextBlock(), $t.Foreground = new Windows.UI.Xaml.Media.SolidColorBrush.$ctor1(Windows.UI.Colors.White.$clone()), $t.FontSize = 11.0, $t.Margin = new Windows.UI.Xaml.Thickness.$ctor2(5, 3, 5, 3), $t.TextWrapping = Windows.UI.Xaml.TextWrapping.Wrap, $t.MaxWidth = 250, $t);
                                border.Child = textBlock;
                                popup.Child = border;
                                popup.IsHitTestVisible = false;
                                popup.PlacementTarget = target;
                                popup.Placement = Windows.UI.Xaml.Controls.Primitives.PlacementMode.Right;
                            }

                            var errorsInString = "";
                            var isFirst = true;
                            $t = Bridge.getEnumerator(errors);
                            try {
                                while ($t.moveNext()) {
                                    var validationError = $t.Current;
                                    errorsInString = (errorsInString || "") + ("\n" || "");
                                    if (isFirst) {
                                        errorsInString = "";
                                        isFirst = false;
                                    }
                                    errorsInString = System.String.concat(errorsInString, validationError.ErrorContent);
                                }
                            } finally {
                                if (Bridge.is($t, System.IDisposable)) {
                                    $t.System$IDisposable$Dispose();
                                }
                            }

                            Bridge.cast(Bridge.cast(popup.Child, Windows.UI.Xaml.Controls.Border).Child, Windows.UI.Xaml.Controls.TextBlock).Text = errorsInString;
                            popup.IsOpen = true;
                        }

                    }

                },
                Popup_PopupMoved: function (sender, e) {
                    var popup = Bridge.cast(sender, Windows.UI.Xaml.Controls.Primitives.Popup);
                    var popupRoot = popup.PopupRoot;

                    if (Bridge.is(popup.PlacementTarget, Windows.UI.Xaml.FrameworkElement) && popupRoot != null) {
                        var isParentVisible = DotNetForHtml5.Core.INTERNAL_PopupsManager.IsPopupParentVisibleOnScreen(popup);

                        popupRoot.Visibility = (isParentVisible ? Windows.UI.Xaml.Visibility.Visible : Windows.UI.Xaml.Visibility.Collapsed);

                    }
                }
            }
        }
    });

    /**
     * Represents a validation error that is created either by the binding engine
     when a System.Windows.Controls.ValidationRule reports a validation error,
     or through the System.Windows.Controls.Validation.MarkInvalid(System.Windows.Data.BindingExpressionBase,System.Windows.Controls.ValidationError)
     method explicitly.
     * // Initializes a new instance of the System.Windows.Controls.ValidationError
    // class with the specified parameters.
    //
     * // Initializes a new instance of the System.Windows.Controls.ValidationError
    // class with the specified parameters.
    //
     * // Gets or sets the System.Windows.Controls.ValidationRule object that was the
    // cause of this System.Windows.Controls.ValidationError, if the error is the
    // result of a validation rule.
    //
     *
     * @public
     * @class Windows.UI.Xaml.Controls.ValidationError
     */
    Bridge.define("Windows.UI.Xaml.Controls.ValidationError", {
        fields: {
            /**
             * Gets the System.Windows.Data.BindingExpression or System.Windows.Data.MultiBindingExpression
             object of this System.Windows.Controls.ValidationError. The object is either
             marked invalid explicitly or has a failed validation rule.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.ValidationError
             * @function BindingInError
             * @type System.Object
             */
            BindingInError: null,
            /**
             * Gets or sets an object that provides additional context for this System.Windows.Controls.ValidationError,
             such as a string describing the error.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.ValidationError
             * @function ErrorContent
             * @type System.Object
             */
            ErrorContent: null,
            /**
             * Gets or sets the System.Exception object that was the cause of this System.Windows.Controls.ValidationError,
             if the error is the result of an exception.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.ValidationError
             * @function Exception
             * @type System.Exception
             */
            Exception: null
        },
        ctors: {
            /**
             * Initializes a new instance of the System.Windows.Controls.ValidationError
             class with the specified parameters.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Controls.ValidationError
             * @memberof Windows.UI.Xaml.Controls.ValidationError
             * @param   {System.Object}    bindingInError    The System.Windows.Data.BindingExpression or System.Windows.Data.MultiBindingExpression
             object with the validation error.
             * @return  {void}
             */
            ctor: function (bindingInError) {
                this.$initialize();
                this.BindingInError = bindingInError;
            }
        }
    });

    /**
     * Describes the reason a System.Windows.FrameworkElement.BindingValidationError
     event has occurred.
     *
     * @public
     * @class Windows.UI.Xaml.Controls.ValidationErrorEventAction
     */
    Bridge.define("Windows.UI.Xaml.Controls.ValidationErrorEventAction", {
        $kind: "enum",
        statics: {
            fields: {
                /**
                 * A new System.Windows.Controls.ValidationError has occurred.
                 *
                 * @static
                 * @public
                 * @memberof Windows.UI.Xaml.Controls.ValidationErrorEventAction
                 * @constant
                 * @default 0
                 * @type Windows.UI.Xaml.Controls.ValidationErrorEventAction
                 */
                Added: 0,
                /**
                 * An existing System.Windows.Controls.ValidationError has been removed.
                 *
                 * @static
                 * @public
                 * @memberof Windows.UI.Xaml.Controls.ValidationErrorEventAction
                 * @constant
                 * @default 1
                 * @type Windows.UI.Xaml.Controls.ValidationErrorEventAction
                 */
                Removed: 1
            }
        }
    });

    /**
     * Names and helpers for visual states in the controls.
     *
     * @static
     * @abstract
     * @class Windows.UI.Xaml.Controls.VisualStates
     */
    Bridge.define("Windows.UI.Xaml.Controls.VisualStates", {
        statics: {
            fields: {
                /**
                 * Common state group.
                 *
                 * @static
                 * @public
                 * @memberof Windows.UI.Xaml.Controls.VisualStates
                 * @constant
                 * @default "CommonStates"
                 * @type string
                 */
                GroupCommon: null,
                /**
                 * Normal state of the Common state group.
                 *
                 * @static
                 * @public
                 * @memberof Windows.UI.Xaml.Controls.VisualStates
                 * @constant
                 * @default "Normal"
                 * @type string
                 */
                StateNormal: null,
                /**
                 * Normal state of the Common state group.
                 *
                 * @static
                 * @public
                 * @memberof Windows.UI.Xaml.Controls.VisualStates
                 * @constant
                 * @default "ReadOnly"
                 * @type string
                 */
                StateReadOnly: null,
                /**
                 * MouseOver state of the Common state group.
                 *
                 * @static
                 * @public
                 * @memberof Windows.UI.Xaml.Controls.VisualStates
                 * @constant
                 * @default "MouseOver"
                 * @type string
                 */
                StateMouseOver: null,
                /**
                 * Pressed state of the Common state group.
                 *
                 * @static
                 * @public
                 * @memberof Windows.UI.Xaml.Controls.VisualStates
                 * @constant
                 * @default "Pressed"
                 * @type string
                 */
                StatePressed: null,
                /**
                 * Disabled state of the Common state group.
                 *
                 * @static
                 * @public
                 * @memberof Windows.UI.Xaml.Controls.VisualStates
                 * @constant
                 * @default "Disabled"
                 * @type string
                 */
                StateDisabled: null,
                /**
                 * Focus state group.
                 *
                 * @static
                 * @public
                 * @memberof Windows.UI.Xaml.Controls.VisualStates
                 * @constant
                 * @default "FocusStates"
                 * @type string
                 */
                GroupFocus: null,
                /**
                 * Unfocused state of the Focus state group.
                 *
                 * @static
                 * @public
                 * @memberof Windows.UI.Xaml.Controls.VisualStates
                 * @constant
                 * @default "Unfocused"
                 * @type string
                 */
                StateUnfocused: null,
                /**
                 * Focused state of the Focus state group.
                 *
                 * @static
                 * @public
                 * @memberof Windows.UI.Xaml.Controls.VisualStates
                 * @constant
                 * @default "Focused"
                 * @type string
                 */
                StateFocused: null,
                /**
                 * Selection state group.
                 *
                 * @static
                 * @public
                 * @memberof Windows.UI.Xaml.Controls.VisualStates
                 * @constant
                 * @default "SelectionStates"
                 * @type string
                 */
                GroupSelection: null,
                /**
                 * Selected state of the Selection state group.
                 *
                 * @static
                 * @public
                 * @memberof Windows.UI.Xaml.Controls.VisualStates
                 * @constant
                 * @default "Selected"
                 * @type string
                 */
                StateSelected: null,
                /**
                 * Unselected state of the Selection state group.
                 *
                 * @static
                 * @public
                 * @memberof Windows.UI.Xaml.Controls.VisualStates
                 * @constant
                 * @default "Unselected"
                 * @type string
                 */
                StateUnselected: null,
                /**
                 * Selected inactive state of the Selection state group.
                 *
                 * @static
                 * @public
                 * @memberof Windows.UI.Xaml.Controls.VisualStates
                 * @constant
                 * @default "SelectedInactive"
                 * @type string
                 */
                StateSelectedInactive: null,
                /**
                 * Expansion state group.
                 *
                 * @static
                 * @public
                 * @memberof Windows.UI.Xaml.Controls.VisualStates
                 * @constant
                 * @default "ExpansionStates"
                 * @type string
                 */
                GroupExpansion: null,
                /**
                 * Expanded state of the Expansion state group.
                 *
                 * @static
                 * @public
                 * @memberof Windows.UI.Xaml.Controls.VisualStates
                 * @constant
                 * @default "Expanded"
                 * @type string
                 */
                StateExpanded: null,
                /**
                 * Collapsed state of the Expansion state group.
                 *
                 * @static
                 * @public
                 * @memberof Windows.UI.Xaml.Controls.VisualStates
                 * @constant
                 * @default "Collapsed"
                 * @type string
                 */
                StateCollapsed: null,
                /**
                 * Popup state group.
                 *
                 * @static
                 * @public
                 * @memberof Windows.UI.Xaml.Controls.VisualStates
                 * @constant
                 * @default "PopupStates"
                 * @type string
                 */
                GroupPopup: null,
                /**
                 * Opened state of the Popup state group.
                 *
                 * @static
                 * @public
                 * @memberof Windows.UI.Xaml.Controls.VisualStates
                 * @constant
                 * @default "PopupOpened"
                 * @type string
                 */
                StatePopupOpened: null,
                /**
                 * Closed state of the Popup state group.
                 *
                 * @static
                 * @public
                 * @memberof Windows.UI.Xaml.Controls.VisualStates
                 * @constant
                 * @default "PopupClosed"
                 * @type string
                 */
                StatePopupClosed: null,
                /**
                 * ValidationStates state group.
                 *
                 * @static
                 * @public
                 * @memberof Windows.UI.Xaml.Controls.VisualStates
                 * @constant
                 * @default "ValidationStates"
                 * @type string
                 */
                GroupValidation: null,
                /**
                 * The valid state for the ValidationStates group.
                 *
                 * @static
                 * @public
                 * @memberof Windows.UI.Xaml.Controls.VisualStates
                 * @constant
                 * @default "Valid"
                 * @type string
                 */
                StateValid: null,
                /**
                 * Invalid, focused state for the ValidationStates group.
                 *
                 * @static
                 * @public
                 * @memberof Windows.UI.Xaml.Controls.VisualStates
                 * @constant
                 * @default "InvalidFocused"
                 * @type string
                 */
                StateInvalidFocused: null,
                /**
                 * Invalid, unfocused state for the ValidationStates group.
                 *
                 * @static
                 * @public
                 * @memberof Windows.UI.Xaml.Controls.VisualStates
                 * @constant
                 * @default "InvalidUnfocused"
                 * @type string
                 */
                StateInvalidUnfocused: null,
                /**
                 * ExpandDirection state group.
                 *
                 * @static
                 * @public
                 * @memberof Windows.UI.Xaml.Controls.VisualStates
                 * @constant
                 * @default "ExpandDirectionStates"
                 * @type string
                 */
                GroupExpandDirection: null,
                /**
                 * Down expand direction state of ExpandDirection state group.
                 *
                 * @static
                 * @public
                 * @memberof Windows.UI.Xaml.Controls.VisualStates
                 * @constant
                 * @default "ExpandDown"
                 * @type string
                 */
                StateExpandDown: null,
                /**
                 * Up expand direction state of ExpandDirection state group.
                 *
                 * @static
                 * @public
                 * @memberof Windows.UI.Xaml.Controls.VisualStates
                 * @constant
                 * @default "ExpandUp"
                 * @type string
                 */
                StateExpandUp: null,
                /**
                 * Left expand direction state of ExpandDirection state group.
                 *
                 * @static
                 * @public
                 * @memberof Windows.UI.Xaml.Controls.VisualStates
                 * @constant
                 * @default "ExpandLeft"
                 * @type string
                 */
                StateExpandLeft: null,
                /**
                 * Right expand direction state of ExpandDirection state group.
                 *
                 * @static
                 * @public
                 * @memberof Windows.UI.Xaml.Controls.VisualStates
                 * @constant
                 * @default "ExpandRight"
                 * @type string
                 */
                StateExpandRight: null,
                /**
                 * HasItems state group.
                 *
                 * @static
                 * @public
                 * @memberof Windows.UI.Xaml.Controls.VisualStates
                 * @constant
                 * @default "HasItemsStates"
                 * @type string
                 */
                GroupHasItems: null,
                /**
                 * HasItems state of the HasItems state group.
                 *
                 * @static
                 * @public
                 * @memberof Windows.UI.Xaml.Controls.VisualStates
                 * @constant
                 * @default "HasItems"
                 * @type string
                 */
                StateHasItems: null,
                /**
                 * NoItems state of the HasItems state group.
                 *
                 * @static
                 * @public
                 * @memberof Windows.UI.Xaml.Controls.VisualStates
                 * @constant
                 * @default "NoItems"
                 * @type string
                 */
                StateNoItems: null,
                /**
                 * Increment state group.
                 *
                 * @static
                 * @public
                 * @memberof Windows.UI.Xaml.Controls.VisualStates
                 * @constant
                 * @default "IncreaseStates"
                 * @type string
                 */
                GroupIncrease: null,
                /**
                 * State enabled for increment group.
                 *
                 * @static
                 * @public
                 * @memberof Windows.UI.Xaml.Controls.VisualStates
                 * @constant
                 * @default "IncreaseEnabled"
                 * @type string
                 */
                StateIncreaseEnabled: null,
                /**
                 * State disabled for increment group.
                 *
                 * @static
                 * @public
                 * @memberof Windows.UI.Xaml.Controls.VisualStates
                 * @constant
                 * @default "IncreaseDisabled"
                 * @type string
                 */
                StateIncreaseDisabled: null,
                /**
                 * Decrement state group.
                 *
                 * @static
                 * @public
                 * @memberof Windows.UI.Xaml.Controls.VisualStates
                 * @constant
                 * @default "DecreaseStates"
                 * @type string
                 */
                GroupDecrease: null,
                /**
                 * State enabled for decrement group.
                 *
                 * @static
                 * @public
                 * @memberof Windows.UI.Xaml.Controls.VisualStates
                 * @constant
                 * @default "DecreaseEnabled"
                 * @type string
                 */
                StateDecreaseEnabled: null,
                /**
                 * State disabled for decrement group.
                 *
                 * @static
                 * @public
                 * @memberof Windows.UI.Xaml.Controls.VisualStates
                 * @constant
                 * @default "DecreaseDisabled"
                 * @type string
                 */
                StateDecreaseDisabled: null,
                /**
                 * InteractionMode state group.
                 *
                 * @static
                 * @public
                 * @memberof Windows.UI.Xaml.Controls.VisualStates
                 * @constant
                 * @default "InteractionModeStates"
                 * @type string
                 */
                GroupInteractionMode: null,
                /**
                 * Edit of the DisplayMode state group.
                 *
                 * @static
                 * @public
                 * @memberof Windows.UI.Xaml.Controls.VisualStates
                 * @constant
                 * @default "Edit"
                 * @type string
                 */
                StateEdit: null,
                /**
                 * Display of the DisplayMode state group.
                 *
                 * @static
                 * @public
                 * @memberof Windows.UI.Xaml.Controls.VisualStates
                 * @constant
                 * @default "Display"
                 * @type string
                 */
                StateDisplay: null,
                /**
                 * DisplayMode state group.
                 *
                 * @static
                 * @public
                 * @memberof Windows.UI.Xaml.Controls.VisualStates
                 * @constant
                 * @default "LockedStates"
                 * @type string
                 */
                GroupLocked: null,
                /**
                 * Edit of the DisplayMode state group.
                 *
                 * @static
                 * @public
                 * @memberof Windows.UI.Xaml.Controls.VisualStates
                 * @constant
                 * @default "Locked"
                 * @type string
                 */
                StateLocked: null,
                /**
                 * Display of the DisplayMode state group.
                 *
                 * @static
                 * @public
                 * @memberof Windows.UI.Xaml.Controls.VisualStates
                 * @constant
                 * @default "Unlocked"
                 * @type string
                 */
                StateUnlocked: null,
                /**
                 * Active state.
                 *
                 * @static
                 * @public
                 * @memberof Windows.UI.Xaml.Controls.VisualStates
                 * @constant
                 * @default "Active"
                 * @type string
                 */
                StateActive: null,
                /**
                 * Inactive state.
                 *
                 * @static
                 * @public
                 * @memberof Windows.UI.Xaml.Controls.VisualStates
                 * @constant
                 * @default "Inactive"
                 * @type string
                 */
                StateInactive: null,
                /**
                 * Active state group.
                 *
                 * @static
                 * @public
                 * @memberof Windows.UI.Xaml.Controls.VisualStates
                 * @constant
                 * @default "ActiveStates"
                 * @type string
                 */
                GroupActive: null,
                /**
                 * Non-watermarked state.
                 *
                 * @static
                 * @public
                 * @memberof Windows.UI.Xaml.Controls.VisualStates
                 * @constant
                 * @default "Unwatermarked"
                 * @type string
                 */
                StateUnwatermarked: null,
                /**
                 * Watermarked state.
                 *
                 * @static
                 * @public
                 * @memberof Windows.UI.Xaml.Controls.VisualStates
                 * @constant
                 * @default "Watermarked"
                 * @type string
                 */
                StateWatermarked: null,
                /**
                 * Watermark state group.
                 *
                 * @static
                 * @public
                 * @memberof Windows.UI.Xaml.Controls.VisualStates
                 * @constant
                 * @default "WatermarkStates"
                 * @type string
                 */
                GroupWatermark: null,
                /**
                 * Unfocused state for Calendar Buttons.
                 *
                 * @static
                 * @public
                 * @memberof Windows.UI.Xaml.Controls.VisualStates
                 * @constant
                 * @default "CalendarButtonUnfocused"
                 * @type string
                 */
                StateCalendarButtonUnfocused: null,
                /**
                 * Focused state for Calendar Buttons.
                 *
                 * @static
                 * @public
                 * @memberof Windows.UI.Xaml.Controls.VisualStates
                 * @constant
                 * @default "CalendarButtonFocused"
                 * @type string
                 */
                StateCalendarButtonFocused: null,
                /**
                 * CalendarButtons Focus state group.
                 *
                 * @static
                 * @public
                 * @memberof Windows.UI.Xaml.Controls.VisualStates
                 * @constant
                 * @default "CalendarButtonFocusStates"
                 * @type string
                 */
                GroupCalendarButtonFocus: null,
                /**
                 * Busy state for BusyIndicator.
                 *
                 * @static
                 * @public
                 * @memberof Windows.UI.Xaml.Controls.VisualStates
                 * @constant
                 * @default "Busy"
                 * @type string
                 */
                StateBusy: null,
                /**
                 * Idle state for BusyIndicator.
                 *
                 * @static
                 * @public
                 * @memberof Windows.UI.Xaml.Controls.VisualStates
                 * @constant
                 * @default "Idle"
                 * @type string
                 */
                StateIdle: null,
                /**
                 * Busyness group name.
                 *
                 * @static
                 * @public
                 * @memberof Windows.UI.Xaml.Controls.VisualStates
                 * @constant
                 * @default "BusyStatusStates"
                 * @type string
                 */
                GroupBusyStatus: null,
                /**
                 * Visible state name for BusyIndicator.
                 *
                 * @static
                 * @public
                 * @memberof Windows.UI.Xaml.Controls.VisualStates
                 * @constant
                 * @default "Visible"
                 * @type string
                 */
                StateVisible: null,
                /**
                 * Hidden state name for BusyIndicator.
                 *
                 * @static
                 * @public
                 * @memberof Windows.UI.Xaml.Controls.VisualStates
                 * @constant
                 * @default "Hidden"
                 * @type string
                 */
                StateHidden: null,
                /**
                 * BusyDisplay group.
                 *
                 * @static
                 * @public
                 * @memberof Windows.UI.Xaml.Controls.VisualStates
                 * @constant
                 * @default "VisibilityStates"
                 * @type string
                 */
                GroupVisibility: null
            },
            ctors: {
                init: function () {
                    this.GroupCommon = "CommonStates";
                    this.StateNormal = "Normal";
                    this.StateReadOnly = "ReadOnly";
                    this.StateMouseOver = "MouseOver";
                    this.StatePressed = "Pressed";
                    this.StateDisabled = "Disabled";
                    this.GroupFocus = "FocusStates";
                    this.StateUnfocused = "Unfocused";
                    this.StateFocused = "Focused";
                    this.GroupSelection = "SelectionStates";
                    this.StateSelected = "Selected";
                    this.StateUnselected = "Unselected";
                    this.StateSelectedInactive = "SelectedInactive";
                    this.GroupExpansion = "ExpansionStates";
                    this.StateExpanded = "Expanded";
                    this.StateCollapsed = "Collapsed";
                    this.GroupPopup = "PopupStates";
                    this.StatePopupOpened = "PopupOpened";
                    this.StatePopupClosed = "PopupClosed";
                    this.GroupValidation = "ValidationStates";
                    this.StateValid = "Valid";
                    this.StateInvalidFocused = "InvalidFocused";
                    this.StateInvalidUnfocused = "InvalidUnfocused";
                    this.GroupExpandDirection = "ExpandDirectionStates";
                    this.StateExpandDown = "ExpandDown";
                    this.StateExpandUp = "ExpandUp";
                    this.StateExpandLeft = "ExpandLeft";
                    this.StateExpandRight = "ExpandRight";
                    this.GroupHasItems = "HasItemsStates";
                    this.StateHasItems = "HasItems";
                    this.StateNoItems = "NoItems";
                    this.GroupIncrease = "IncreaseStates";
                    this.StateIncreaseEnabled = "IncreaseEnabled";
                    this.StateIncreaseDisabled = "IncreaseDisabled";
                    this.GroupDecrease = "DecreaseStates";
                    this.StateDecreaseEnabled = "DecreaseEnabled";
                    this.StateDecreaseDisabled = "DecreaseDisabled";
                    this.GroupInteractionMode = "InteractionModeStates";
                    this.StateEdit = "Edit";
                    this.StateDisplay = "Display";
                    this.GroupLocked = "LockedStates";
                    this.StateLocked = "Locked";
                    this.StateUnlocked = "Unlocked";
                    this.StateActive = "Active";
                    this.StateInactive = "Inactive";
                    this.GroupActive = "ActiveStates";
                    this.StateUnwatermarked = "Unwatermarked";
                    this.StateWatermarked = "Watermarked";
                    this.GroupWatermark = "WatermarkStates";
                    this.StateCalendarButtonUnfocused = "CalendarButtonUnfocused";
                    this.StateCalendarButtonFocused = "CalendarButtonFocused";
                    this.GroupCalendarButtonFocus = "CalendarButtonFocusStates";
                    this.StateBusy = "Busy";
                    this.StateIdle = "Idle";
                    this.GroupBusyStatus = "BusyStatusStates";
                    this.StateVisible = "Visible";
                    this.StateHidden = "Hidden";
                    this.GroupVisibility = "VisibilityStates";
                }
            },
            methods: {
                /**
                 * Use VisualStateManager to change the visual state of the control.
                 *
                 * @static
                 * @public
                 * @this Windows.UI.Xaml.Controls.VisualStates
                 * @memberof Windows.UI.Xaml.Controls.VisualStates
                 * @param   {Windows.UI.Xaml.Controls.Control}    control           Control whose visual state is being changed.
                 * @param   {boolean}                             useTransitions    A value indicating whether to use transitions when updating the
                 visual state, or to snap directly to the new visual state.
                 * @param   {Array.<string>}                      stateNames        Ordered list of state names and fallback states to transition into.
                 Only the first state to be found will be used.
                 * @return  {void}
                 */
                GoToState: function (control, useTransitions, stateNames) {
                    var $t;
                    if (stateNames === void 0) { stateNames = []; }

                    $t = Bridge.getEnumerator(stateNames);
                    try {
                        while ($t.moveNext()) {
                            var name = $t.Current;
                            if (Windows.UI.Xaml.VisualStateManager.GoToState(control, name, useTransitions)) {
                                break;
                            }
                        }
                    } finally {
                        if (Bridge.is($t, System.IDisposable)) {
                            $t.System$IDisposable$Dispose();
                        }
                    }
                }
            }
        }
    });

    Bridge.define("Windows.UI.Xaml.ControlToWatch", {
        fields: {
            ControltoWatch: null,
            PreviousSize: null,
            PreviousPosition: null,
            OnPositionOrSizeChanged: null
        },
        ctors: {
            init: function () {
                this.PreviousSize = new Windows.Foundation.Size();
                this.PreviousPosition = new Windows.Foundation.Point();
            },
            ctor: function (controlToWatch, OnPositionOrSizeChangedCallback) {
                this.$initialize();
                this.ControltoWatch = controlToWatch;
                this.OnPositionOrSizeChanged = OnPositionOrSizeChangedCallback;
            }
        }
    });

    /**
     * [SECURITY CRITICAL] Describes the characteristics of a rounded corner, such
     as can be applied to a Windows.UI.Xaml.Controls.Border.
     *
     * @public
     * @class Windows.UI.Xaml.CornerRadius
     */
    Bridge.define("Windows.UI.Xaml.CornerRadius", {
        $kind: "struct",
        statics: {
            ctors: {
                ctor: function () {
                    DotNetForHtml5.Core.TypeFromStringConverters.RegisterConverter(Windows.UI.Xaml.CornerRadius, Windows.UI.Xaml.CornerRadius.INTERNAL_ConvertFromString);
                }
            },
            methods: {
                INTERNAL_ConvertFromString: function (cornerRadiusAsString) {
                    var separator;
                    if (System.String.contains(cornerRadiusAsString,",")) {
                        separator = 44;
                    } else {
                        separator = 32;
                    }
                    var splittedString = System.String.split(cornerRadiusAsString.trim(), [separator].map(function (i) {{ return String.fromCharCode(i); }}));
                    if (splittedString.length === 1) {
                        var radius = { v : 0.0 };
                        if (System.Double.tryParse(splittedString[System.Array.index(0, splittedString)], null, radius)) {
                            return new Windows.UI.Xaml.CornerRadius.$ctor1(radius.v).$clone();
                        }
                    } else if (splittedString.length === 4) {
                        var topLeft = { v : 0.0 };
                        var topRight = { v : 0.0 };
                        var bottomRight = { v : 0.0 };
                        var bottomLeft = { v : 0.0 };

                        var isParseOK = System.Double.tryParse(splittedString[System.Array.index(0, splittedString)], null, topLeft);
                        isParseOK = isParseOK && System.Double.tryParse(splittedString[System.Array.index(1, splittedString)], null, topRight);
                        isParseOK = isParseOK && System.Double.tryParse(splittedString[System.Array.index(2, splittedString)], null, bottomRight);
                        isParseOK = isParseOK && System.Double.tryParse(splittedString[System.Array.index(3, splittedString)], null, bottomLeft);

                        if (isParseOK) {
                            return new Windows.UI.Xaml.CornerRadius.$ctor2(topLeft.v, topRight.v, bottomRight.v, bottomLeft.v).$clone();
                        }
                    }
                    throw new System.FormatException.$ctor1((cornerRadiusAsString || "") + "is not an eligible value for CornerRadius");
                }/**
                 * [SECURITY CRITICAL] Compares two Windows.UI.Xaml.CornerRadius structures
                 for inequality.
                 *
                 * @static
                 * @public
                 * @this Windows.UI.Xaml.CornerRadius
                 * @memberof Windows.UI.Xaml.CornerRadius
                 * @param   {Windows.UI.Xaml.CornerRadius}    cr1    The first structure to compare.
                 * @param   {Windows.UI.Xaml.CornerRadius}    cr2    The other structure to compare.
                 * @return  {boolean}                                true if the two instances of Windows.UI.Xaml.CornerRadius are not equal; otherwise, false.
                 */
                ,
                op_Inequality: function (cr1, cr2) {
                    return (cr1.TopLeft !== cr2.TopLeft || cr1.TopRight !== cr2.TopRight || cr1.BottomRight !== cr2.BottomRight || cr1.BottomLeft !== cr2.BottomLeft);
                }/**
                 * [SECURITY CRITICAL] Compares the value of two Windows.UI.Xaml.CornerRadius
                 structures for equality.
                 *
                 * @static
                 * @public
                 * @this Windows.UI.Xaml.CornerRadius
                 * @memberof Windows.UI.Xaml.CornerRadius
                 * @param   {Windows.UI.Xaml.CornerRadius}    cr1    The first structure to compare.
                 * @param   {Windows.UI.Xaml.CornerRadius}    cr2    The other structure to compare.
                 * @return  {boolean}                                true if the two instances of Windows.UI.Xaml.CornerRadius are equal; otherwise,
                 false.
                 */
                ,
                op_Equality: function (cr1, cr2) {
                    return (cr1.TopLeft === cr2.TopLeft && cr1.TopRight === cr2.TopRight && cr1.BottomRight === cr2.BottomRight && cr1.BottomLeft === cr2.BottomLeft);
                },
                getDefaultValue: function () { return new Windows.UI.Xaml.CornerRadius(); }
            }
        },
        fields: {
            _topLeftRadius: 0,
            _bottomLeftRadius: 0,
            _topRightRadius: 0,
            _bottomRightRadius: 0
        },
        props: {
            /**
             * [SECURITY CRITICAL] Gets or sets the radius of rounding, in pixels, of the
             bottom left corner of the object where a Windows.UI.Xaml.CornerRadius is
             applied.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.CornerRadius
             * @function BottomLeft
             * @type number
             */
            BottomLeft: {
                get: function () {
                    return this._bottomLeftRadius;
                },
                set: function (value) {
                    this._bottomLeftRadius = value;
                }
            },
            /**
             * [SECURITY CRITICAL] Gets or sets the radius of rounding, in pixels, of the
             bottom right corner of the object where a Windows.UI.Xaml.CornerRadius is
             applied.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.CornerRadius
             * @function BottomRight
             * @type number
             */
            BottomRight: {
                get: function () {
                    return this._bottomRightRadius;
                },
                set: function (value) {
                    this._bottomRightRadius = value;
                }
            },
            /**
             * [SECURITY CRITICAL] Gets or sets the radius of rounding, in pixels, of the
             top left corner of the object where a Windows.UI.Xaml.CornerRadius is applied.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.CornerRadius
             * @function TopLeft
             * @type number
             */
            TopLeft: {
                get: function () {
                    return this._topLeftRadius;
                },
                set: function (value) {
                    this._topLeftRadius = value;
                }
            },
            /**
             * [SECURITY CRITICAL] Gets or sets the radius of rounding, in pixels, of the
             top right corner of the object where a Windows.UI.Xaml.CornerRadius is applied.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.CornerRadius
             * @function TopRight
             * @type number
             */
            TopRight: {
                get: function () {
                    return this._topRightRadius;
                },
                set: function (value) {
                    this._topRightRadius = value;
                }
            }
        },
        ctors: {
            /**
             * [SECURITY CRITICAL] Initializes a new Windows.UI.Xaml.CornerRadius structure,
             applying the same uniform radius to all its corners.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.CornerRadius
             * @memberof Windows.UI.Xaml.CornerRadius
             * @param   {number}    uniformRadius    A uniform radius applied to all four Windows.UI.Xaml.CornerRadius properties
             (Windows.UI.Xaml.CornerRadius.TopLeft, Windows.UI.Xaml.CornerRadius.TopRight,
             Windows.UI.Xaml.CornerRadius.BottomRight, Windows.UI.Xaml.CornerRadius.BottomLeft).
             * @return  {void}
             */
            $ctor1: function (uniformRadius) {
                this.$initialize();
                this._topLeftRadius = uniformRadius;
                this._bottomLeftRadius = uniformRadius;
                this._topRightRadius = uniformRadius;
                this._bottomRightRadius = uniformRadius;
            },
            /**
             * [SECURITY CRITICAL] Initializes a new instance of the Windows.UI.Xaml.CornerRadius
             structure, applying specific radius values to its corners.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.CornerRadius
             * @memberof Windows.UI.Xaml.CornerRadius
             * @param   {number}    topLeft        Sets the initial Windows.UI.Xaml.CornerRadius.TopLeft.
             * @param   {number}    topRight       Sets the initial Windows.UI.Xaml.CornerRadius.TopRight.
             * @param   {number}    bottomRight    Sets the initial Windows.UI.Xaml.CornerRadius.BottomLeft.
             * @param   {number}    bottomLeft     Sets the initial Windows.UI.Xaml.CornerRadius.BottomRight.
             * @return  {void}
             */
            $ctor2: function (topLeft, topRight, bottomRight, bottomLeft) {
                this.$initialize();
                this._topLeftRadius = topLeft;
                this._bottomLeftRadius = bottomLeft;
                this._topRightRadius = topRight;
                this._bottomRightRadius = bottomRight;
            },
            ctor: function () {
                this.$initialize();
            }
        },
        methods: {
            /**
             * [SECURITY CRITICAL] Compares this Windows.UI.Xaml.CornerRadius structure
             to another Windows.UI.Xaml.CornerRadius structure for equality.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.CornerRadius
             * @memberof Windows.UI.Xaml.CornerRadius
             * @param   {Windows.UI.Xaml.CornerRadius}    cornerRadius    An instance of Windows.UI.Xaml.CornerRadius to compare for equality.
             * @return  {boolean}                                         true if the two instances of Windows.UI.Xaml.CornerRadius are equal; otherwise,
             false.
             */
            Equals: function (cornerRadius) {
                return Windows.UI.Xaml.CornerRadius.op_Equality(this, cornerRadius.$clone());
            },
            /**
             * [SECURITY CRITICAL] Compares this Windows.UI.Xaml.CornerRadius structure
             to another object for equality.
             *
             * @instance
             * @public
             * @override
             * @this Windows.UI.Xaml.CornerRadius
             * @memberof Windows.UI.Xaml.CornerRadius
             * @param   {System.Object}    obj    The object to compare.
             * @return  {boolean}                 true if the two objects are equal; otherwise, false.
             */
            equals: function (obj) {
                if (Bridge.is(obj, Windows.UI.Xaml.CornerRadius)) {
                    return Windows.UI.Xaml.CornerRadius.op_Equality(System.Nullable.getValue(Bridge.cast(Bridge.unbox(obj, Windows.UI.Xaml.CornerRadius), Windows.UI.Xaml.CornerRadius)), this);
                }
                return false;
            },
            /**
             * [SECURITY CRITICAL] Returns the hash code of the structure.
             *
             * @instance
             * @public
             * @override
             * @this Windows.UI.Xaml.CornerRadius
             * @memberof Windows.UI.Xaml.CornerRadius
             * @return  {number}        A hash code for this Windows.UI.Xaml.CornerRadius.
             */
            getHashCode: function () {
                return System.Double.getHashCode(this.TopLeft) ^ System.Double.getHashCode(this.TopRight) ^ System.Double.getHashCode(this.BottomRight) ^ System.Double.getHashCode(this.BottomLeft);
            },
            /**
             * [SECURITY CRITICAL] Returns the string representation of the Windows.UI.Xaml.CornerRadius
             structure.
             *
             * @instance
             * @public
             * @override
             * @this Windows.UI.Xaml.CornerRadius
             * @memberof Windows.UI.Xaml.CornerRadius
             * @return  {string}        A System.String that represents the Windows.UI.Xaml.CornerRadius value.
             */
            toString: function () {
                return System.Double.format(this.TopLeft) + "," + System.Double.format(this.TopRight) + "," + System.Double.format(this.BottomRight) + "," + System.Double.format(this.BottomLeft);
            },
            $clone: function (to) {
                var s = to || new Windows.UI.Xaml.CornerRadius();
                s._topLeftRadius = this._topLeftRadius;
                s._bottomLeftRadius = this._bottomLeftRadius;
                s._topRightRadius = this._topRightRadius;
                s._bottomRightRadius = this._bottomRightRadius;
                return s;
            }
        }
    });

    Bridge.define("Windows.UI.Xaml.CSSEquivalent", {
        fields: {
            Name: null,
            /**
             * true means that all we will always use Velocity to apply the value to the dom Element, even when there is no animation. This is useful when the name used in Velocity is different than the name we would normally use and when Velocity ignores changes that happen outside of Velocity. See ScaleTransform.ScaleX
             *
             * @instance
             * @memberof Windows.UI.Xaml.CSSEquivalent
             * @default false
             * @type boolean
             */
            OnlyUseVelocity: false,
            DomElement: null,
            ApplyAlsoWhenThereIsAControlTemplate: false,
            Value: null,
            Values: null,
            UIElement: null,
            CallbackMethod: null,
            DependencyProperty: null
        },
        ctors: {
            init: function () {
                this.OnlyUseVelocity = false;
                this.ApplyAlsoWhenThereIsAControlTemplate = false;
            }
        }
    });

    Bridge.define("Windows.UI.Xaml.Data.IPropertyPathWalkerListener", {
        $kind: "interface"
    });

    /**
     * Describes how the data propagates in a binding.
     *
     * @public
     * @class Windows.UI.Xaml.Data.BindingMode
     */
    Bridge.define("Windows.UI.Xaml.Data.BindingMode", {
        $kind: "enum",
        statics: {
            fields: {
                /**
                 * Updates the target property when the binding is created. Changes to the source object can also propagate to the target.
                 *
                 * @static
                 * @public
                 * @memberof Windows.UI.Xaml.Data.BindingMode
                 * @constant
                 * @default 1
                 * @type Windows.UI.Xaml.Data.BindingMode
                 */
                OneWay: 1,
                /**
                 * Updates the target property when the binding is created.
                 *
                 * @static
                 * @public
                 * @memberof Windows.UI.Xaml.Data.BindingMode
                 * @constant
                 * @default 2
                 * @type Windows.UI.Xaml.Data.BindingMode
                 */
                OneTime: 2,
                /**
                 * Updates either the target or the source object when either changes. When the binding is created, the target property is updated from the source.
                 *
                 * @static
                 * @public
                 * @memberof Windows.UI.Xaml.Data.BindingMode
                 * @constant
                 * @default 3
                 * @type Windows.UI.Xaml.Data.BindingMode
                 */
                TwoWay: 3
            }
        }
    });

    /**
     * Provides the static BindingOperations.SetBinding(DependencyObject,DependencyProperty,BindingBase) method.
     *
     * @static
     * @abstract
     * @public
     * @class Windows.UI.Xaml.Data.BindingOperations
     */
    Bridge.define("Windows.UI.Xaml.Data.BindingOperations", {
        statics: {
            methods: {
                /**
                 * Creates and associates a new BindingExpression with the specified binding target property.
                 *
                 * @static
                 * @public
                 * @this Windows.UI.Xaml.Data.BindingOperations
                 * @memberof Windows.UI.Xaml.Data.BindingOperations
                 * @param   {Windows.UI.Xaml.DependencyObject}          target     The target to set the binding to.
                 * @param   {Windows.UI.Xaml.DependencyProperty}        dp         The property on the target to bind.
                 * @param   {Windows.UI.Xaml.Data.Binding}              binding    The binding to assign to the target property.
                 * @return  {Windows.UI.Xaml.Data.BindingExpression}               An object that contains information about the binding.
                 */
                SetBinding: function (target, dp, binding) {
                    if (target == null) {
                        throw new System.ArgumentNullException.$ctor1("target");
                    }
                    if (dp == null) {
                        throw new System.ArgumentNullException.$ctor1("dp");
                    }
                    if (binding == null) {
                        throw new System.ArgumentNullException.$ctor1("binding");
                    }

                    var expr;

                    if (Bridge.referenceEquals(dp, Windows.UI.Xaml.Setter.ValueProperty)) {
                        binding._isInStyle = true;
                        expr = new Windows.UI.Xaml.Data.BindingExpression.$ctor1(binding, Bridge.cast(target, Windows.UI.Xaml.Setter).Property);
                    } else {
                        expr = new Windows.UI.Xaml.Data.BindingExpression.$ctor1(binding, dp);
                    }
                    target.SetValue(dp, expr);

                    return expr;
                },
                GetBindingExpression: function (target, dp) {
                    if (target == null) {
                        throw new System.ArgumentNullException.$ctor1("target");
                    }
                    if (dp == null) {
                        throw new System.ArgumentNullException.$ctor1("dp");
                    }
                    return Bridge.as(target.ReadLocalValue(dp), Windows.UI.Xaml.Data.BindingExpression);
                }
            }
        }
    });

    /**
     * The class that creates a tree results based on the operations requested by a PagedCollectionView.
     This is the class that can filter, sort and group (not paged, because pages are handled PagedCollectionView).
     *
     * @class Windows.UI.Xaml.Data.CollectionViewGroupInternal
     */
    Bridge.define("Windows.UI.Xaml.Data.CollectionViewGroupInternal", {
        fields: {
            _parentView: null,
            _operations: null,
            _level: 0,
            IsLeaf: false,
            Items: null
        },
        ctors: {
            ctor: function (source, parent, operations, level) {
                if (level === void 0) { level = 0; }

                this.$initialize();
                this.Items = source;
                this._level = level;
                this._parentView = parent;
                this._operations = operations;

                var operationToDo = this._operations.HandleNextOperation(this._level);

                this.ExecuteOperation(operationToDo);
            }
        },
        methods: {
            ExecuteOperation: function (operation) {
                if (operation != null) {
                    this.IsLeaf = false;

                    if (Bridge.is(operation, Windows.UI.Xaml.Data.FilterDescription)) {
                        var filterOperation = Bridge.cast(operation, Windows.UI.Xaml.Data.FilterDescription);

                        if (!Bridge.staticEquals(filterOperation.FilterUsingAnEvent, null)) {
                            this.Filter$1(filterOperation.FilterUsingAnEvent);
                        } else {
                            this.Filter(filterOperation.FilterUsingAPredicate);
                        }
                    } else if (Bridge.is(operation, Windows.UI.Xaml.Data.PropertySortDescription)) {
                        var sortOperation = Bridge.cast(operation, Windows.UI.Xaml.Data.PropertySortDescription);

                        if (sortOperation.Comparer == null) {
                            this.Sort(sortOperation);
                        } else {
                            this.CustomSort(sortOperation);
                        }

                    } else if (Bridge.is(operation, Windows.UI.Xaml.Data.PropertyGroupDescription)) {
                        var groupOperation = Bridge.cast(operation, Windows.UI.Xaml.Data.PropertyGroupDescription);

                        this.GroupBy(groupOperation);
                    } else {
                        throw new System.InvalidOperationException.$ctor1("unknow operation type");
                    }
                } else {
                    this.IsLeaf = true;
                }
            },
            Filter: function (predicate) {
                var $t;
                var childItems = new (System.Collections.ObjectModel.Collection$1(System.Object)).ctor();

                $t = Bridge.getEnumerator(this.Items, System.Object);
                try {
                    while ($t.moveNext()) {
                        var obj = $t.Current;
                        if (predicate(obj)) {
                            childItems.add(obj);
                        }
                    }
                } finally {
                    if (Bridge.is($t, System.IDisposable)) {
                        $t.System$IDisposable$Dispose();
                    }
                }

                var newView = new Windows.UI.Xaml.Data.CollectionViewGroupInternal(childItems, this, this._operations);
                this._operations.Requester.AddView(newView);
            },
            Filter$1: function (logicMethod) {
                var $t;
                var childItems = new (System.Collections.ObjectModel.Collection$1(System.Object)).ctor();

                $t = Bridge.getEnumerator(this.Items, System.Object);
                try {
                    while ($t.moveNext()) {
                        var obj = $t.Current;
                        var args = new Windows.UI.Xaml.Data.FilterEventArgs(obj);

                        logicMethod(this, args);

                        if (args.Accepted) {
                            childItems.add(obj);
                        }
                    }
                } finally {
                    if (Bridge.is($t, System.IDisposable)) {
                        $t.System$IDisposable$Dispose();
                    }
                }

                var newView = new Windows.UI.Xaml.Data.CollectionViewGroupInternal(childItems, this, this._operations);
                this._operations.Requester.AddView(newView);
            },
            VerifyComparable: function (operation) {
                if (operation.Comparer == null) {
                    if (System.Array.getCount(this.Items, System.Object) !== 0) {
                        if (CSHTML5.Interop.IsRunningInTheSimulator) {
                            var type = Bridge.getType(this.GetValue$1(System.Linq.Enumerable.from(this.Items).elementAt(0), operation));

                            if (Bridge.Reflection.isAssignableFrom(System.IComparable, type)) {
                                return true;
                            }
                        } else {

                            try {
                                var value1ToCompare = this.GetValue$1(System.Linq.Enumerable.from(this.Items).elementAt(0), operation);
                                var value2ToCompare = value1ToCompare;

                                if (value1ToCompare > value2ToCompare) {
                                }

                                return true;
                            } catch ($e1) {
                                $e1 = System.Exception.create($e1);
                                /* 
                                // custom types (see also __comparer__ in jsil)
                                Type type = GetValue(Items.ElementAt(0), operation).GetType();

                                foreach (MethodInfo method in type.GetMethods())
                                {
                                   if (method.Name == "CompareTo")
                                       return true;
                                }
                                */

                                return false;
                            }
                        }
                    }
                    return false;
                }

                throw new System.NotImplementedException.$ctor1("Custom Sorting is not supported yet");
            },
            CustomSort: function (operation) {
                throw new System.NotImplementedException.$ctor1("Custom Sorting is not supported yet");
            },
            Sort: function (operation) {
                var $t;
                if (this.VerifyComparable(operation)) {
                    var directionBackup = operation.Direction;

                    operation.Direction = this._operations.GetFixedDirection(operation);

                    var childItems = new (System.Collections.ObjectModel.Collection$1(System.Object)).ctor();


                    $t = Bridge.getEnumerator(this.Items, System.Object);
                    try {
                        while ($t.moveNext()) {
                            var item = $t.Current;
                            var index;

                            if (CSHTML5.Interop.IsRunningInTheSimulator) {
                                var value = Bridge.cast(this.GetValue$1(item, operation), System.IComparable);

                                index = this.GetPosIndex_SimulatorOnly(operation, value, 0, childItems.Count, childItems);
                            } else {
                                var value1 = this.GetValue$1(item, operation);

                                index = this.GetPosIndex_JSOnly(operation, value1, 0, childItems.Count, childItems);
                            }

                            childItems.insert(index, item);
                        }
                    } finally {
                        if (Bridge.is($t, System.IDisposable)) {
                            $t.System$IDisposable$Dispose();
                        }
                    }

                    operation.Direction = directionBackup;

                    var newView = new Windows.UI.Xaml.Data.CollectionViewGroupInternal(childItems, this, this._operations);
                    this._operations.Requester.AddView(newView);
                } else {
                    var newView1 = new Windows.UI.Xaml.Data.CollectionViewGroupInternal(this.Items, this, this._operations);
                    this._operations.Requester.AddView(newView1);
                }
            },
            GetPosIndex_SimulatorOnly: function (operation, x, min, max, childItems) {
                if (min === max) {
                    return min;
                } else {
                    var middle = ((((Bridge.Int.div((((max - min) | 0)), 2)) | 0)) + min) | 0;

                    var childValue = this.GetValue$1(childItems.getItem(middle), operation);

                    if (Bridge.Int.mul((operation.Direction === System.ComponentModel.ListSortDirection.Ascending ? 1 : -1), Bridge.compare(x, childValue)) > 0) {
                        return this.GetPosIndex_SimulatorOnly(operation, x, ((middle + 1) | 0), max, childItems);
                    } else {
                        return this.GetPosIndex_SimulatorOnly(operation, x, min, middle, childItems);
                    }
                }
            },
            GetPosIndex_JSOnly: function (operation, x, min, max, childItems) {
                if (min === max) {
                    return min;
                } else {
                    var middle = ((((Bridge.Int.div((((max - min) | 0)), 2)) | 0)) + min) | 0;

                    var childValue = this.GetValue$1(childItems.getItem(middle), operation);

                    if (operation.Direction === System.ComponentModel.ListSortDirection.Ascending ? (x > childValue) : (x < childValue)) {
                        return this.GetPosIndex_JSOnly(operation, x, ((middle + 1) | 0), max, childItems);
                    } else {
                        return this.GetPosIndex_JSOnly(operation, x, min, middle, childItems);
                    }
                }
            },
            GroupBy: function (operation) {
                var $t, $t1, $t2;
                var childGroups = new (System.Collections.Generic.List$1(System.Collections.ObjectModel.Collection$1(System.Object))).ctor();

                $t = Bridge.getEnumerator(this.Items, System.Object);
                try {
                    while ($t.moveNext()) {
                        var item = $t.Current;
                        var find = false;

                        $t1 = Bridge.getEnumerator(childGroups);
                        try {
                            while ($t1.moveNext()) {
                                var child = $t1.Current;
                                if (Bridge.equals(this.GetValue(child.getItem(0), operation), this.GetValue(item, operation))) {
                                    child.add(item);
                                    find = true;
                                    break;
                                }
                            }
                        } finally {
                            if (Bridge.is($t1, System.IDisposable)) {
                                $t1.System$IDisposable$Dispose();
                            }
                        }

                        if (!find) {
                            childGroups.add(new (System.Collections.ObjectModel.Collection$1(System.Object)).ctor());
                            childGroups.getItem(((childGroups.Count - 1) | 0)).add(item);
                        }
                    }
                } finally {
                    if (Bridge.is($t, System.IDisposable)) {
                        $t.System$IDisposable$Dispose();
                    }
                }

                $t2 = Bridge.getEnumerator(childGroups);
                try {
                    while ($t2.moveNext()) {
                        var child1 = $t2.Current;
                        var newView = new Windows.UI.Xaml.Data.CollectionViewGroupInternal(child1, this, this._operations, ((this._level + 1) | 0));
                        this._operations.Requester.AddView(newView);
                    }
                } finally {
                    if (Bridge.is($t2, System.IDisposable)) {
                        $t2.System$IDisposable$Dispose();
                    }
                }
            },
            GetValue$1: function (obj, operation) {
                return Bridge.Reflection.midel(Bridge.Reflection.getMembers(Bridge.getType(obj), 16, 284, operation.PropertyName).g, Bridge.unbox(obj)).apply(null, null);
            },
            GetValue: function (obj, operation) {
                return Bridge.Reflection.midel(Bridge.Reflection.getMembers(Bridge.getType(obj), 16, 284, operation.PropertyName).g, Bridge.unbox(obj)).apply(null, null);
            }
        }
    });

    Bridge.define("Windows.UI.Xaml.Data.FilterDescription", {
        fields: {
            FilterUsingAnEvent: null,
            FilterUsingAPredicate: null
        },
        ctors: {
            $ctor1: function (_filterUsingAndEvent) {
                this.$initialize();
                this.FilterUsingAnEvent = _filterUsingAndEvent;
            },
            ctor: function (_filterUsingAPredicate) {
                this.$initialize();
                this.FilterUsingAPredicate = _filterUsingAPredicate;
            }
        }
    });

    /**
     * Arguments for the Filter event.
     *
     * @public
     * @class Windows.UI.Xaml.Data.FilterEventArgs
     * @augments System.Object
     */
    Bridge.define("Windows.UI.Xaml.Data.FilterEventArgs", {
        fields: {
            Accepted: false,
            Item: null
        },
        ctors: {
            ctor: function (item) {
                this.$initialize();
                System.Object.call(this);
                this.Item = item;
                this.Accepted = true;
            }
        }
    });

    Bridge.define("Windows.UI.Xaml.Data.IDependencyPropertyChangedEventArgs", {
        $kind: "interface"
    });

    Bridge.define("Windows.UI.Xaml.Data.IIsAttachedMonitor", {
        $kind: "interface"
    });

    Bridge.define("Windows.UI.Xaml.Data.IPropertyPathNode", {
        $kind: "interface"
    });

    Bridge.define("Windows.UI.Xaml.Data.INTERNAL_Operations", {
        fields: {
            _hasFilteringBeenDone: false,
            _sortOperationIndex: 0,
            _collectionViewer: null
        },
        props: {
            Requester: {
                get: function () {
                    return this._collectionViewer;
                }
            }
        },
        ctors: {
            ctor: function (collectionViewer) {
                this.$initialize();
                this._hasFilteringBeenDone = false;
                this._sortOperationIndex = -1;
                this._collectionViewer = collectionViewer;
            }
        },
        methods: {
            GetFixedDirection: function (operation) {
                var index = this._collectionViewer.SortDescriptions.indexOf(operation);

                if (index % 2 === 0 && this._collectionViewer.SortDescriptions.Count > 1) {
                    return operation.Direction === System.ComponentModel.ListSortDirection.Ascending ? System.ComponentModel.ListSortDirection.Descending : System.ComponentModel.ListSortDirection.Ascending;
                }

                return operation.Direction;
            },
            HasFilteringBeenDone: function () {
                return this._hasFilteringBeenDone;
            },
            HandleNextOperation: function (level) {
                if (level === void 0) { level = 0; }
                if (this._hasFilteringBeenDone) {
                    if (this._sortOperationIndex >= ((this._collectionViewer.SortDescriptions.Count - 1) | 0)) {
                        if (level < this._collectionViewer.GroupDescriptions.Count) {
                            return this._collectionViewer.GroupDescriptions.getItem(level);
                        } else {
                            return null;
                        }
                    } else {
                        this._sortOperationIndex = (this._sortOperationIndex + 1) | 0;
                        return this._collectionViewer.SortDescriptions.getItem(this._sortOperationIndex);
                    }
                } else {
                    this._hasFilteringBeenDone = true;
                    if (this._collectionViewer._filter != null) {
                        return this._collectionViewer._filter;
                    } else {
                        return this.HandleNextOperation();
                    }
                }
            }
        }
    });

    Bridge.define("Windows.UI.Xaml.Data.IPropertyChangedListener", {
        $kind: "interface"
    });

    Bridge.define("Windows.UI.Xaml.Data.IPropertyPathNodeListener", {
        $kind: "interface"
    });

    /**
     * The {@link } namespace defines the data binding infrastructure for apps.
     *
     * @class Windows.UI.Xaml.Data.NamespaceDoc
     */
    Bridge.define("Windows.UI.Xaml.Data.NamespaceDoc");

    /**
     * Description of grouping based on a property value.
     *
     * @public
     * @class Windows.UI.Xaml.Data.PropertyGroupDescription
     */
    Bridge.define("Windows.UI.Xaml.Data.PropertyGroupDescription", {
        fields: {
            PropertyName: null
        },
        ctors: {
            ctor: function () {
                this.$initialize();

            },
            $ctor1: function (propertyName) {
                this.$initialize();
                this.PropertyName = propertyName;
            }
        }
    });

    /**
     * Defines a property and direction to sort a list by.
     *
     * @public
     * @class Windows.UI.Xaml.Data.PropertySortDescription
     */
    Bridge.define("Windows.UI.Xaml.Data.PropertySortDescription", {
        fields: {
            PropertyName: null,
            Direction: 0,
            Comparer: null
        },
        ctors: {
            ctor: function (propertyName, direction, comparer) {
                if (comparer === void 0) { comparer = null; }

                this.$initialize();
                this.PropertyName = propertyName;
                this.Direction = direction;
                this.Comparer = comparer;
            }
        }
    });

    /**
     * Defines constants that describe the location of the binding source relative to the position of the binding target.
     *
     * @public
     * @class Windows.UI.Xaml.Data.RelativeSourceMode
     */
    Bridge.define("Windows.UI.Xaml.Data.RelativeSourceMode", {
        $kind: "enum",
        statics: {
            fields: {
                /**
                 * Don't use this value of RelativeSourceMode; always use either Self or TemplatedParent.
                 *
                 * @static
                 * @public
                 * @memberof Windows.UI.Xaml.Data.RelativeSourceMode
                 * @constant
                 * @default 0
                 * @type Windows.UI.Xaml.Data.RelativeSourceMode
                 */
                None: 0,
                /**
                 * Refers to the element to which the template (in which the data-bound element
                 exists) is applied. This is similar to setting a TemplateBinding Markup Extension
                 and is only applicable if the Binding is within a template.
                 *
                 * @static
                 * @public
                 * @memberof Windows.UI.Xaml.Data.RelativeSourceMode
                 * @constant
                 * @default 1
                 * @type Windows.UI.Xaml.Data.RelativeSourceMode
                 */
                TemplatedParent: 1,
                /**
                 * Refers to the element on which you are setting the binding and allows you
                 to bind one property of that element to another property on the same element.
                 *
                 * @static
                 * @public
                 * @memberof Windows.UI.Xaml.Data.RelativeSourceMode
                 * @constant
                 * @default 2
                 * @type Windows.UI.Xaml.Data.RelativeSourceMode
                 */
                Self: 2,
                /**
                 * Refers to the ancestor in the parent chain of the data-bound element. You
                 can use this to bind to an ancestor of a specific type or its subclasses.
                 This is the mode you use if you want to specify System.Windows.Data.RelativeSource.AncestorType
                 and/or System.Windows.Data.RelativeSource.AncestorLevel.
                 *
                 * @static
                 * @public
                 * @memberof Windows.UI.Xaml.Data.RelativeSourceMode
                 * @constant
                 * @default 3
                 * @type Windows.UI.Xaml.Data.RelativeSourceMode
                 */
                FindAncestor: 3
            }
        }
    });

    /**
     * This enum describes when updates (target-to-source data flow)
     happen in a given Binding.
     *
     * @public
     * @class Windows.UI.Xaml.Data.UpdateSourceTrigger
     */
    Bridge.define("Windows.UI.Xaml.Data.UpdateSourceTrigger", {
        $kind: "enum",
        statics: {
            fields: {
                /**
                 * Obtain trigger from target property default
                 *
                 * @static
                 * @public
                 * @memberof Windows.UI.Xaml.Data.UpdateSourceTrigger
                 * @constant
                 * @default 0
                 * @type Windows.UI.Xaml.Data.UpdateSourceTrigger
                 */
                Default: 0,
                /**
                 * Update whenever the target property changes
                 *
                 * @static
                 * @public
                 * @memberof Windows.UI.Xaml.Data.UpdateSourceTrigger
                 * @constant
                 * @default 1
                 * @type Windows.UI.Xaml.Data.UpdateSourceTrigger
                 */
                PropertyChanged: 1,
                /**
                 * / Update only when target element loses focus, or when Binding deactivates
                /
                 * Update only by explicit call to BindingExpression.UpdateSource()
                 *
                 * @static
                 * @public
                 * @memberof Windows.UI.Xaml.Data.UpdateSourceTrigger
                 * @constant
                 * @default 2
                 * @type Windows.UI.Xaml.Data.UpdateSourceTrigger
                 */
                Explicit: 2
            }
        }
    });

    /**
     * Represents a dependency property that is registered with the dependency property
     system. Dependency properties provide support for value expressions, data
     binding, animation, and property change notification.
     *
     * @public
     * @class Windows.UI.Xaml.DependencyProperty
     */
    Bridge.define("Windows.UI.Xaml.DependencyProperty", {
        statics: {
            fields: {
                nullableType: null,
                UnsetValue: null
            },
            ctors: {
                init: function () {
                    this.UnsetValue = new CSHTML5.Internal.INTERNAL_NamedObject("DependencyProperty.UnsetValue");
                },
                ctor: function () {
                    Windows.UI.Xaml.DependencyProperty.nullableType = System.Nullable$1;
                }
            },
            methods: {
                RegisterAttached: function (name, propertyType, ownerType, typeMetadata) {
                    var property = Windows.UI.Xaml.DependencyProperty.Register(name, propertyType, ownerType, typeMetadata);
                    property._isAttached = true;
                    return property;
                },
                EnsureDefaultValue: function (typeMetadata, propertyType, name, ownerType) {
                    if (typeMetadata.IsDefaultValueModified) {
                        if (!Windows.UI.Xaml.DependencyProperty.IsValueTypeValid(typeMetadata.DefaultValue, propertyType)) {
                            var message = System.String.format("Default value type ({0}) does not match property type ({1}). To fix this issue, please change the default value of the dependency property named '{2}' in the type '{3}' so that it matches the type of the property.", (typeMetadata.DefaultValue == null ? "null" : Bridge.Reflection.getTypeFullName(Bridge.getType(typeMetadata.DefaultValue))), Bridge.Reflection.getTypeFullName(propertyType), name, ownerType);
                            if (Windows.UI.Xaml.Application.Current.Host.Settings.EnableInvalidPropertyMetadataDefaultValueExceptions) {
                                throw new System.ArgumentException.$ctor1(message);
                            } else {
                                var defaultValue = Windows.UI.Xaml.DependencyProperty.CreateDefaultValue(propertyType);
                                typeMetadata.DefaultValue = defaultValue;
                                System.Console.WriteLine((message || "") + ("\n" || "") + (System.String.format("The default value has been automatically set to '{0}'.", [defaultValue]) || ""));
                            }
                        }
                    } else {
                        typeMetadata.DefaultValue = Windows.UI.Xaml.DependencyProperty.CreateDefaultValue(propertyType);
                    }
                },
                IsValueTypeValid: function (value, type) {
                    if (Bridge.referenceEquals(value, Windows.UI.Xaml.DependencyProperty.UnsetValue)) {
                        return false;
                    } else {
                        if (value == null) {
                            if (Bridge.Reflection.isValueType(type) && !(Bridge.Reflection.isGenericType(type) && Bridge.referenceEquals(Bridge.Reflection.getGenericTypeDefinition(type), Windows.UI.Xaml.DependencyProperty.nullableType))) {
                                return false;
                            }
                        } else {
                            if (!Bridge.Reflection.isInstanceOfType(value, type)) {
                                return false;
                            }
                        }
                        return true;
                    }
                },
                /**
                 * Registers a dependency property with the specified property name, property type, owner type, and property metadata.
                 *
                 * @static
                 * @public
                 * @this Windows.UI.Xaml.DependencyProperty
                 * @memberof Windows.UI.Xaml.DependencyProperty
                 * @param   {string}                                name            The property name.
                 * @param   {System.Type}                           propertyType    The type of the property.
                 * @param   {System.Type}                           ownerType       The type of the property's owner.
                 * @param   {Windows.UI.Xaml.PropertyMetadata}      typeMetadata    The property metadata.
                 * @return  {Windows.UI.Xaml.DependencyProperty}                    The DependencyProperty.
                 */
                Register: function (name, propertyType, ownerType, typeMetadata) {
                    var $t;
                    var defaultMetadata = typeMetadata;
                    if (defaultMetadata == null) {
                        defaultMetadata = new Windows.UI.Xaml.PropertyMetadata.ctor();
                    }
                    Windows.UI.Xaml.DependencyProperty.EnsureDefaultValue(defaultMetadata, propertyType, name, ownerType);

                    defaultMetadata.Seal();

                    var newDependencyProperty = ($t = new Windows.UI.Xaml.DependencyProperty(), $t.Name = name, $t.PropertyType = propertyType, $t.OwnerType = ownerType, $t._defaultMetadata = defaultMetadata, $t);

                    CSHTML5.Internal.INTERNAL_TypeToDependencyProperties.Add(ownerType, newDependencyProperty);

                    if (typeMetadata != null && typeMetadata.CallPropertyChangedWhenLoadedIntoVisualTree === Windows.UI.Xaml.WhenToCallPropertyChangedEnum.Always) {
                        CSHTML5.Internal.INTERNAL_TypeToDependencyPropertiesThatRequirePropertyChanged.Add(ownerType, newDependencyProperty);
                    }

                    var stringsToDependencyProperties = CSHTML5.Internal.INTERNAL_TypeToStringsToDependencyProperties.GetDictionaryForType(ownerType);
                    if (stringsToDependencyProperties.containsKey(name)) {
                        if (stringsToDependencyProperties.getItem(name) != null) {
                            throw new System.Exception("Cannot register multiple properties with the same PropertyName");
                        }
                        stringsToDependencyProperties.setItem(name, newDependencyProperty);
                    } else {
                        stringsToDependencyProperties.add(name, newDependencyProperty);
                    }
                    return newDependencyProperty;
                },
                CreateDefaultValue: function (type) {
                    return Bridge.Reflection.isValueType(type) ? Bridge.createInstance(type) : null;
                }
            }
        },
        fields: {
            Name: null,
            PropertyType: null,
            OwnerType: null,
            _isAttached: false,
            _defaultMetadata: null,
            _typesToOverridenMetadatas: null,
            Optimization_typesWithoutOverride: null,
            Optimization_typesToOverrides: null
        },
        props: {
            IsAttached: {
                get: function () {
                    return this._isAttached;
                }
            }
        },
        ctors: {
            init: function () {
                this._isAttached = false;
            }
        },
        methods: {
            GetMetadata: function (type) {
                return this.GetTypeMetaData(type);
            },
            GetTypeMetaData: function (typeOfOwner) {
                var $t, $t1;
                var typesWentThrough = null;
                if (this.Optimization_typesToOverrides != null) {
                    if (this.Optimization_typesToOverrides.containsKey(typeOfOwner)) {
                        return this.Optimization_typesToOverrides.getItem(typeOfOwner);
                    } else if (this.Optimization_typesWithoutOverride != null && this.Optimization_typesWithoutOverride.containsKey(typeOfOwner)) {
                        return this._defaultMetadata;
                    }
                    var currentType = typeOfOwner;
                    typesWentThrough = new (System.Collections.Generic.List$1(System.Type)).ctor();
                    while (!Bridge.referenceEquals(currentType, this.OwnerType) && !this.Optimization_typesToOverrides.containsKey(currentType)) {
                        typesWentThrough.add(currentType);
                        currentType = Bridge.Reflection.getBaseType(currentType);
                    }
                    if (!Bridge.referenceEquals(currentType, this.OwnerType)) {
                        $t = Bridge.getEnumerator(typesWentThrough);
                        try {
                            while ($t.moveNext()) {
                                var type = $t.Current;
                                this.Optimization_typesToOverrides.add(type, this.Optimization_typesToOverrides.getItem(currentType));
                            }
                        } finally {
                            if (Bridge.is($t, System.IDisposable)) {
                                $t.System$IDisposable$Dispose();
                            }
                        }
                        return this.Optimization_typesToOverrides.getItem(currentType);
                    }
                }
                if (typesWentThrough != null) {
                    if (this.Optimization_typesWithoutOverride == null) {
                        this.Optimization_typesWithoutOverride = new (System.Collections.Generic.Dictionary$2(System.Type,System.Boolean)).ctor();
                    }
                    $t1 = Bridge.getEnumerator(typesWentThrough);
                    try {
                        while ($t1.moveNext()) {
                            var type1 = $t1.Current;
                            if (!this.Optimization_typesWithoutOverride.containsKey(type1)) {
                                this.Optimization_typesWithoutOverride.add(type1, false);
                            }
                        }
                    } finally {
                        if (Bridge.is($t1, System.IDisposable)) {
                            $t1.System$IDisposable$Dispose();
                        }
                    }
                }
                return this._defaultMetadata;
            },
            /**
             * Returns the DependencyProperty as a string.
             *
             * @instance
             * @public
             * @override
             * @this Windows.UI.Xaml.DependencyProperty
             * @memberof Windows.UI.Xaml.DependencyProperty
             * @return  {string}        The DependencyProperty as a string.
             */
            toString: function () {
                if (this.OwnerType != null && this.Name != null) {
                    return (Bridge.Reflection.getTypeName(this.OwnerType) || "") + "." + (this.Name || "") + "Property";
                } else {
                    return Bridge.toString(this);
                }
            },
            PrepareOverrideMetadata: function (newOwnerType, typeMetadata) {
                if (newOwnerType == null) {
                    throw new System.ArgumentNullException.$ctor1("newOwnerType");
                }
                if (typeMetadata == null) {
                    throw new System.ArgumentNullException.$ctor1("typeMetadata");
                }
                if (typeMetadata.IsDefaultValueModified) {
                    if (!Windows.UI.Xaml.DependencyProperty.IsValueTypeValid(typeMetadata.DefaultValue, this.PropertyType)) {
                        throw new System.ArgumentException.$ctor1(System.String.format("Default value type does not match type of property. To fix this issue, please change the default value of the dependency property named '{0}' in the type '{1}' so that it matches the type of the property.", this.Name, Bridge.getTypeName(this.OwnerType)));
                    }
                }
            },
            OverrideMetadata: function (newOwnerType, typeMetadata) {
                this.PrepareOverrideMetadata(newOwnerType, typeMetadata);

                if (!typeMetadata.IsDefaultValueModified) {
                    typeMetadata.DefaultValue = Windows.UI.Xaml.DependencyProperty.CreateDefaultValue(this.PropertyType);
                }

                typeMetadata.Seal();

                if (this._typesToOverridenMetadatas == null) {
                    this._typesToOverridenMetadatas = new (System.Collections.Generic.Dictionary$2(System.Type,Windows.UI.Xaml.PropertyMetadata)).ctor();
                }
                this._typesToOverridenMetadatas.add(newOwnerType, typeMetadata);

                if (this.Optimization_typesToOverrides == null) {
                    this.Optimization_typesToOverrides = new (System.Collections.Generic.Dictionary$2(System.Type,Windows.UI.Xaml.PropertyMetadata)).ctor();
                }
                this.Optimization_typesToOverrides.add(newOwnerType, typeMetadata);
            }
        }
    });

    /**
     * Provides a timer that is integrated into the Dispatcher queue, which is processed
     at a specified interval of time and at a specified priority.  One scenario
     for this is to run code on the UI thread.
     *
     * @public
     * @class Windows.UI.Xaml.DispatcherTimer
     * @example Here is how you can create and use a DispatcherTimer:
 
     When you want to stop the DispatcherTimer, you can use the following code:
     *
     * DispatcherTimer _dispatcherTimer;
     * 
     *
     * //We create a new instance of DispatcherTimer:
     * _dispatcherTimer = new Dispatchertimer();
     * //we set the time between each tick of the DispatcherTimer to 100 milliseconds:
     * timer.Interval = new TimeSpan(0, 0, 0, 0, 100);
     * _dispatcherTimer.Tick += DispatcherTimer_Tick;
     * _dispatcherTimer.Start();
     * 
     *
     * void DispatcherTimer_Tick(object sender, object e)
     * {
     *     //Some code to execute at each Tick of the DispatcherTimer.
     * }
     * 
     *
     *     _dispatcherTimer.Stop();
     * 
     *
     *
     */
    Bridge.define("Windows.UI.Xaml.DispatcherTimer", {
        fields: {
            _timer: null,
            _interval: null
        },
        events: {
            /**
             * Occurs when the timer interval has elapsed.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.DispatcherTimer
             * @memberof Windows.UI.Xaml.DispatcherTimer
             * @function addTick
             * @param   {System.EventHandler}    value
             * @return  {void}
             */
            /**
             * Occurs when the timer interval has elapsed.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.DispatcherTimer
             * @memberof Windows.UI.Xaml.DispatcherTimer
             * @function removeTick
             * @param   {System.EventHandler}    value
             * @return  {void}
             */
            Tick: null
        },
        props: {
            /**
             * Gets or sets the amount of time between timer ticks.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.DispatcherTimer
             * @function Interval
             * @type System.TimeSpan
             */
            Interval: {
                get: function () {
                    return this._interval;
                },
                set: function (value) {
                    this._interval = value;

                    if (this._timer != null) {
                        this.Stop();
                        this.Start();
                    }
                }
            },
            /**
             * Gets a value that indicates whether the timer is running.
             *
             * @instance
             * @public
             * @readonly
             * @memberof Windows.UI.Xaml.DispatcherTimer
             * @function IsEnabled
             * @type boolean
             */
            IsEnabled: {
                get: function () {
                    return this._timer != null;
                }
            }
        },
        ctors: {
            init: function () {
                this._interval = new System.TimeSpan();
            },
            /**
             * Initializes a new instance of the DispatcherTimer class.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.DispatcherTimer
             * @memberof Windows.UI.Xaml.DispatcherTimer
             * @return  {void}
             */
            ctor: function () {
                this.$initialize();
            }
        },
        methods: {
            /**
             * Raises the Tick event.
             *
             * @instance
             * @protected
             * @this Windows.UI.Xaml.DispatcherTimer
             * @memberof Windows.UI.Xaml.DispatcherTimer
             * @return  {void}
             */
            OnTick: function () {
                if (!Bridge.staticEquals(this.Tick, null)) {
                    this.Tick(this, { });
                }
            },
            /**
             * Starts the DispatcherTimer.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.DispatcherTimer
             * @memberof Windows.UI.Xaml.DispatcherTimer
             * @return  {void}
             */
            Start: function () {
                if (this._timer == null) {
                    var intervalInMilliseconds = Bridge.Int.clip64(this._interval.getTotalMilliseconds());
                    if (intervalInMilliseconds.equals(System.Int64(0))) {
                        intervalInMilliseconds = System.Int64(1);
                    }
                    this._timer = setInterval(Bridge.fn.cacheBind(this, this.OnTick),intervalInMilliseconds);
                }
            },
            /**
             * Stops the DispatcherTimer.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.DispatcherTimer
             * @memberof Windows.UI.Xaml.DispatcherTimer
             * @return  {void}
             */
            Stop: function () {
                if (this._timer != null) {
                    clearInterval(this._timer);
                    this._timer = null;
                }
            }
        }
    });

    Bridge.define("Windows.UI.Xaml.Documents.INTERNAL_ITextContainer", {
        $kind: "interface"
    });

    Bridge.define("Windows.UI.Xaml.Documents.INTERNAL_TextContainerHelper", {
        statics: {
            methods: {
                FromOwner: function (parent) {
                    var container = null;
                    if (Bridge.is(parent, Windows.UI.Xaml.Controls.TextBlock)) {
                        container = new Windows.UI.Xaml.Documents.INTERNAL_TextContainerTextBlock(Bridge.cast(parent, Windows.UI.Xaml.Controls.TextBlock));
                    } else if (Bridge.is(parent, Windows.UI.Xaml.Documents.Span)) {
                        container = new Windows.UI.Xaml.Documents.INTERNAL_TextContainerSpan(Bridge.cast(parent, Windows.UI.Xaml.Documents.Span));
                    }
                    return container;
                }
            }
        }
    });

    /**
     * Represents the duration of time that a Windows.UI.Xaml.Media.Animation.Timeline
     is active.
     *
     * @public
     * @class Windows.UI.Xaml.Duration
     */
    Bridge.define("Windows.UI.Xaml.Duration", {
        $kind: "struct",
        statics: {
            props: {
                /**
                 * Gets a Windows.UI.Xaml.Duration value that is automatically determined.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Duration
                 * @function Automatic
                 * @type Windows.UI.Xaml.Duration
                 */
                Automatic: {
                    get: function () {
                        var duration = new Windows.UI.Xaml.Duration.$ctor1(new System.TimeSpan());
                        duration._durationType = Windows.UI.Xaml.Duration.DurationType.Automatic;
                        return duration.$clone();
                    }
                },
                /**
                 * Gets a Windows.UI.Xaml.Duration value that represents an infinite interval.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Duration
                 * @function Forever
                 * @type Windows.UI.Xaml.Duration
                 */
                Forever: {
                    get: function () {
                        var duration = new Windows.UI.Xaml.Duration.$ctor1(new System.TimeSpan(9999, 0, 0, 0));
                        duration._durationType = Windows.UI.Xaml.Duration.DurationType.Forever;
                        return duration.$clone();
                    }
                }
            },
            ctors: {
                ctor: function () {
                    DotNetForHtml5.Core.TypeFromStringConverters.RegisterConverter(Windows.UI.Xaml.Duration, Windows.UI.Xaml.Duration.INTERNAL_ConvertFromString);
                }
            },
            methods: {
                /**
                 * Determines whether two Windows.UI.Xaml.Duration values are equal.
                 *
                 * @static
                 * @public
                 * @this Windows.UI.Xaml.Duration
                 * @memberof Windows.UI.Xaml.Duration
                 * @param   {Windows.UI.Xaml.Duration}    t1    First Windows.UI.Xaml.Duration to compare.
                 * @param   {Windows.UI.Xaml.Duration}    t2    Second Windows.UI.Xaml.Duration to compare.
                 * @return  {boolean}                           true if t1 is equal to t2; otherwise, false.
                 */
                Equals: function (t1, t2) {
                    return Windows.UI.Xaml.Duration.op_Equality(t1.$clone(), t2.$clone());
                },
                /**
                 * // Subtracts the specified Windows.UI.Xaml.Duration from this Windows.UI.Xaml.Duration.
                //
                 * // Converts a Windows.UI.Xaml.Duration to a System.String representation.
                //
                 *
                 * @static
                 * @this Windows.UI.Xaml.Duration
                 * @memberof Windows.UI.Xaml.Duration
                 * @param   {string}           p
                 * @return  {System.Object}         A System.String representation of this Windows.UI.Xaml.Duration.
                 */
                INTERNAL_ConvertFromString: function (p) {
                    var timeSpan = INTERNAL_BridgeWorkarounds.TimeSpanParse(p);
                    return new Windows.UI.Xaml.Duration.$ctor1(timeSpan).$clone();
                }/**
                 * Subtracts the value of one Windows.UI.Xaml.Duration from another.
                 *
                 * @static
                 * @public
                 * @this Windows.UI.Xaml.Duration
                 * @memberof Windows.UI.Xaml.Duration
                 * @param   {Windows.UI.Xaml.Duration}    t1    The first Windows.UI.Xaml.Duration.
                 * @param   {Windows.UI.Xaml.Duration}    t2    The Windows.UI.Xaml.Duration to subtract.
                 * @return  {Windows.UI.Xaml.Duration}
                 */
                ,
                op_Subtraction: function (t1, t2) {
                    if (t1.HasTimeSpan && t2.HasTimeSpan) {
                        return new Windows.UI.Xaml.Duration.$ctor1(System.TimeSpan.sub(t1.TimeSpan, t2.TimeSpan));
                    }

                    if (t1._durationType === Windows.UI.Xaml.Duration.DurationType.Forever && t2.HasTimeSpan) {
                        return Windows.UI.Xaml.Duration.Forever.$clone();
                    } else {
                        return Windows.UI.Xaml.Duration.Automatic.$clone();
                    }
                }/**
                 * Determines if two Windows.UI.Xaml.Duration cases are not equal.
                 *
                 * @static
                 * @public
                 * @this Windows.UI.Xaml.Duration
                 * @memberof Windows.UI.Xaml.Duration
                 * @param   {Windows.UI.Xaml.Duration}    t1    The first Windows.UI.Xaml.Duration to compare.
                 * @param   {Windows.UI.Xaml.Duration}    t2    The second Windows.UI.Xaml.Duration to compare.
                 * @return  {boolean}                           true if exactly one of t1 or t2 represent a value, or if they both represent
                 values that are not equal; otherwise, false.
                 */
                ,
                op_Inequality: function (t1, t2) {
                    return !(t1.Equals(t2.$clone()));
                }/**
                 * Returns the specified Windows.UI.Xaml.Duration.
                 *
                 * @static
                 * @public
                 * @this Windows.UI.Xaml.Duration
                 * @memberof Windows.UI.Xaml.Duration
                 * @param   {Windows.UI.Xaml.Duration}    duration    The Windows.UI.Xaml.Duration to get.
                 * @return  {Windows.UI.Xaml.Duration}                The Windows.UI.Xaml.Duration operation result.
                 */
                ,
                op_UnaryPlus: function (duration) {
                    return duration.$clone();
                }/**
                 * // Adds two Windows.UI.Xaml.Duration values together.
                //
                 * // Determines if a Windows.UI.Xaml.Duration is less than the value of another
                // instance.
                //
                 * // Determines if a Windows.UI.Xaml.Duration is less than or equal to another.
                //
                 * Determines whether two Windows.UI.Xaml.Duration cases are equal.
                 *
                 * @static
                 * @public
                 * @this Windows.UI.Xaml.Duration
                 * @memberof Windows.UI.Xaml.Duration
                 * @param   {Windows.UI.Xaml.Duration}    t1    The first Windows.UI.Xaml.Duration to compare.
                 * @param   {Windows.UI.Xaml.Duration}    t2    The second Windows.UI.Xaml.Duration to compare.
                 * @return  {boolean}                           true if both Windows.UI.Xaml.Duration values have equal property values,
                 or if all Windows.UI.Xaml.Duration values are null. Otherwise, this method
                 returns false.
                 */
                ,
                op_Equality: function (t1, t2) {
                    return t1.Equals(t2.$clone());
                }/**
                 * // Determines if one Windows.UI.Xaml.Duration is greater than another.
                //
                 * // Determines whether a Windows.UI.Xaml.Duration is greater than or equal to
                // another.
                //
                 * Implicitly creates a Duration from a given System.TimeSpan.
                 *
                 * @static
                 * @public
                 * @this Windows.UI.Xaml.Duration
                 * @memberof Windows.UI.Xaml.Duration
                 * @param   {System.TimeSpan}             timeSpan    System.TimeSpan from which a Duration is implicitly created.
                 * @return  {Windows.UI.Xaml.Duration}                A created Duration.
                 */
                ,
                op_Implicit: function (timeSpan) {
                    if (System.TimeSpan.lt(timeSpan, System.TimeSpan.zero)) {
                        throw new System.ArgumentException.$ctor3("The TimeSpan cannot be a negative TimeSpan.", "timeSpan");
                    }
                    return new Windows.UI.Xaml.Duration.$ctor1(timeSpan);
                },
                getDefaultValue: function () { return new Windows.UI.Xaml.Duration(); }
            }
        },
        fields: {
            _durationType: 0,
            _timeSpan: null
        },
        props: {
            /**
             * Gets a value that indicates if this Windows.UI.Xaml.Duration represents a
             System.TimeSpan value.
             *
             * @instance
             * @public
             * @readonly
             * @memberof Windows.UI.Xaml.Duration
             * @function HasTimeSpan
             * @type boolean
             */
            HasTimeSpan: {
                get: function () {
                    return this._durationType === Windows.UI.Xaml.Duration.DurationType.TimeSpan;
                }
            },
            /**
             * Gets the System.TimeSpan value that this Windows.UI.Xaml.Duration represents.
             *
             * @instance
             * @public
             * @readonly
             * @memberof Windows.UI.Xaml.Duration
             * @function TimeSpan
             * @type System.TimeSpan
             */
            TimeSpan: {
                get: function () {
                    if (this.HasTimeSpan) {
                        return this._timeSpan;
                    } else {
                        throw new System.InvalidOperationException.$ctor1("The Duration does no represent a System.TimeSpan.");
                    }
                }
            }
        },
        ctors: {
            init: function () {
                this._timeSpan = new System.TimeSpan();
            },
            /**
             * Initializes a new instance of the Windows.UI.Xaml.Duration structure with
             the supplied System.TimeSpan value.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Duration
             * @memberof Windows.UI.Xaml.Duration
             * @param   {System.TimeSpan}    timeSpan    Represents the initial time interval of this duration.
             * @return  {void}
             */
            $ctor1: function (timeSpan) {
                this.$initialize();
                if (System.TimeSpan.gte(timeSpan, System.TimeSpan.zero)) {
                    this._timeSpan = timeSpan;
                    this._durationType = Windows.UI.Xaml.Duration.DurationType.TimeSpan;
                } else {
                    throw new System.ArgumentException.$ctor1("The TimeSpan cannot be a negative TimeSpan.");
                }
            },
            ctor: function () {
                this.$initialize();
            }
        },
        methods: {
            /**
             * // Adds the value of the specified Windows.UI.Xaml.Duration to this Windows.UI.Xaml.Duration.
            //
             * // Compares one Windows.UI.Xaml.Duration value to another.
            //
             * Determines whether a specified Windows.UI.Xaml.Duration is equal to this
             Windows.UI.Xaml.Duration.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Duration
             * @memberof Windows.UI.Xaml.Duration
             * @param   {Windows.UI.Xaml.Duration}    duration    The Windows.UI.Xaml.Duration to check for equality.
             * @return  {boolean}                                 true if duration is equal to this Windows.UI.Xaml.Duration; otherwise, false.
             */
            Equals: function (duration) {
                if (this.HasTimeSpan) {
                    if (duration.HasTimeSpan) {
                        return System.TimeSpan.eq(this._timeSpan, duration._timeSpan);
                    } else {
                        return false;
                    }
                } else {
                    return this._durationType === duration._durationType;
                }
            },
            /**
             * Determines whether a specified object is equal to a Windows.UI.Xaml.Duration.
             *
             * @instance
             * @public
             * @override
             * @this Windows.UI.Xaml.Duration
             * @memberof Windows.UI.Xaml.Duration
             * @param   {System.Object}    value    Object to check for equality.
             * @return  {boolean}                   true if value is equal to this Windows.UI.Xaml.Duration; otherwise, false.
             */
            equals: function (value) {
                if (Bridge.is(value, Windows.UI.Xaml.Duration)) {
                    return Windows.UI.Xaml.Duration.op_Equality(this, System.Nullable.getValue(Bridge.cast(Bridge.unbox(value, Windows.UI.Xaml.Duration), Windows.UI.Xaml.Duration)));
                }
                return false;
            },
            /**
             * Gets a hash code for this object.
             *
             * @instance
             * @public
             * @override
             * @this Windows.UI.Xaml.Duration
             * @memberof Windows.UI.Xaml.Duration
             * @return  {number}        The hash code identifier.
             */
            getHashCode: function () {
                return Bridge.getHashCode(this.TimeSpan);
            },
            $clone: function (to) {
                var s = to || new Windows.UI.Xaml.Duration();
                s._durationType = this._durationType;
                s._timeSpan = this._timeSpan;
                return s;
            }
        }
    });

    /**
     * An enumeration of the different types of Duration behaviors.
     *
     * @private
     * @class number
     */
    Bridge.define("Windows.UI.Xaml.Duration.DurationType", {
        $kind: "nested enum",
        statics: {
            fields: {
                Automatic: 0,
                TimeSpan: 1,
                Forever: 2
            }
        }
    });

    /**
     * Represents the length of elements that explicitly support Windows.UI.Xaml.GridUnitType.Star
     unit types.
     *
     * @public
     * @class Windows.UI.Xaml.GridLength
     */
    Bridge.define("Windows.UI.Xaml.GridLength", {
        $kind: "struct",
        statics: {
            props: {
                /**
                 * Gets an instance of Windows.UI.Xaml.GridLength that holds
                 a value whose size is determined by the size properties of the content object.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.GridLength
                 * @function Auto
                 * @type Windows.UI.Xaml.GridLength
                 */
                Auto: {
                    get: function () {
                        return new Windows.UI.Xaml.GridLength.$ctor2(1.0, Windows.UI.Xaml.GridUnitType.Auto);
                    }
                }
            },
            ctors: {
                ctor: function () {
                    DotNetForHtml5.Core.TypeFromStringConverters.RegisterConverter(Windows.UI.Xaml.GridLength, Windows.UI.Xaml.GridLength.INTERNAL_ConvertFromString);
                }
            },
            methods: {
                INTERNAL_ConvertFromString: function (gridLengthAsString) {
                    var trimmedLowercase = gridLengthAsString.trim().toLowerCase();
                    if (System.String.endsWith(trimmedLowercase, "*")) {
                        var valueAsString = trimmedLowercase.substr(0, ((trimmedLowercase.length - 1) | 0));
                        var value = { };
                        if (Bridge.referenceEquals(valueAsString, "")) {
                            return new Windows.UI.Xaml.GridLength.$ctor2(1.0, Windows.UI.Xaml.GridUnitType.Star).$clone();
                        }
                        if (System.Double.tryParse(valueAsString, null, value)) {
                            return new Windows.UI.Xaml.GridLength.$ctor2(value.v, Windows.UI.Xaml.GridUnitType.Star).$clone();
                        } else {
                            throw new System.Exception("Invalid GridLength: " + (gridLengthAsString || ""));
                        }
                    } else if (Bridge.referenceEquals(trimmedLowercase, "auto")) {
                        return new Windows.UI.Xaml.GridLength.$ctor2(1.0, Windows.UI.Xaml.GridUnitType.Auto).$clone();
                    } else {
                        var value1 = { };
                        if (System.Double.tryParse(trimmedLowercase, null, value1)) {
                            return new Windows.UI.Xaml.GridLength.$ctor2(value1.v, Windows.UI.Xaml.GridUnitType.Pixel).$clone();
                        } else {
                            throw new System.Exception("Invalid GridLength: " + (gridLengthAsString || ""));
                        }
                    }
                }/**
                 * Compares two Windows.UI.Xaml.GridLength structures to
                 determine if they are not equal.
                 *
                 * @static
                 * @public
                 * @this Windows.UI.Xaml.GridLength
                 * @memberof Windows.UI.Xaml.GridLength
                 * @param   {Windows.UI.Xaml.GridLength}    gl1    The first instance of Windows.UI.Xaml.GridLength to compare.
                 * @param   {Windows.UI.Xaml.GridLength}    gl2    The second instance of Windows.UI.Xaml.GridLength to compare.
                 * @return  {boolean}                              true if the two instances of Windows.UI.Xaml.GridLength do not have the same
                 value and Windows.UI.Xaml.GridUnitType; otherwise, false.
                 */
                ,
                op_Inequality: function (gl1, gl2) {
                    return (gl1.GridUnitType !== gl2.GridUnitType || gl1.Value !== gl2.Value);
                }/**
                 * Compares two Windows.UI.Xaml.GridLength structures for
                 equality.
                 *
                 * @static
                 * @public
                 * @this Windows.UI.Xaml.GridLength
                 * @memberof Windows.UI.Xaml.GridLength
                 * @param   {Windows.UI.Xaml.GridLength}    gl1    The first instance of Windows.UI.Xaml.GridLength to compare.
                 * @param   {Windows.UI.Xaml.GridLength}    gl2    The second instance of Windows.UI.Xaml.GridLength to compare.
                 * @return  {boolean}                              true if the two instances of Windows.UI.Xaml.GridLength have the same value
                 and Windows.UI.Xaml.GridUnitType; otherwise, false.
                 */
                ,
                op_Equality: function (gl1, gl2) {
                    return (gl1.GridUnitType === gl2.GridUnitType && gl1.Value === gl2.Value);
                },
                getDefaultValue: function () { return new Windows.UI.Xaml.GridLength(); }
            }
        },
        fields: {
            _value: 0,
            _type: 0
        },
        props: {
            /**
             * Gets the associated Windows.UI.Xaml.GridUnitType for
             the Windows.UI.Xaml.GridLength.
             *
             * @instance
             * @public
             * @readonly
             * @memberof Windows.UI.Xaml.GridLength
             * @function GridUnitType
             * @type Windows.UI.Xaml.GridUnitType
             */
            GridUnitType: {
                get: function () {
                    return this._type;
                }
            },
            /**
             * Gets a value that indicates whether the Windows.UI.Xaml.GridLength
             holds a value that is expressed in pixels.
             *
             * @instance
             * @public
             * @readonly
             * @memberof Windows.UI.Xaml.GridLength
             * @function IsAbsolute
             * @type boolean
             */
            IsAbsolute: {
                get: function () {
                    return this._type === Windows.UI.Xaml.GridUnitType.Pixel;
                }
            },
            /**
             * Gets a value that indicates whether the Windows.UI.Xaml.GridLength
             holds a value whose size is determined by the size properties of the content
             object.
             *
             * @instance
             * @public
             * @readonly
             * @memberof Windows.UI.Xaml.GridLength
             * @function IsAuto
             * @type boolean
             */
            IsAuto: {
                get: function () {
                    return this._type === Windows.UI.Xaml.GridUnitType.Auto;
                }
            },
            /**
             * Gets a value that indicates whether the Windows.UI.Xaml.GridLength
             holds a value that is expressed as a weighted proportion of available space.
             *
             * @instance
             * @public
             * @readonly
             * @memberof Windows.UI.Xaml.GridLength
             * @function IsStar
             * @type boolean
             */
            IsStar: {
                get: function () {
                    return this._type === Windows.UI.Xaml.GridUnitType.Star;
                }
            },
            /**
             * Gets a System.Double that represents the value of the
             Windows.UI.Xaml.GridLength.
             *
             * @instance
             * @public
             * @readonly
             * @memberof Windows.UI.Xaml.GridLength
             * @function Value
             * @type number
             */
            Value: {
                get: function () {
                    return this._value;
                }
            }
        },
        ctors: {
            /**
             * Initializes a new instance of the Windows.UI.Xaml.GridLength
             structure using the specified absolute value in pixels.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.GridLength
             * @memberof Windows.UI.Xaml.GridLength
             * @param   {number}    pixels    The absolute count of pixels to establish as the value.
             * @return  {void}
             */
            $ctor1: function (pixels) {
                this.$initialize();
                this._value = pixels;
                this._type = Windows.UI.Xaml.GridUnitType.Pixel;
            },
            /**
             * Initializes a new instance of the Windows.UI.Xaml.GridLength
             structure and specifies what kind of value it holds.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.GridLength
             * @memberof Windows.UI.Xaml.GridLength
             * @param   {number}                          value    The initial value of this instance of Windows.UI.Xaml.GridLength.
             * @param   {Windows.UI.Xaml.GridUnitType}    type     The Windows.UI.Xaml.GridUnitType held by this instance of Windows.UI.Xaml.GridLength.
             * @return  {void}
             */
            $ctor2: function (value, type) {
                this.$initialize();
                this._value = (type === Windows.UI.Xaml.GridUnitType.Auto ? 0.0 : value);
                this._type = type;
            },
            ctor: function () {
                this.$initialize();
            }
        },
        methods: {
            /**
             * Returns a new instance of Gridlength with the same properties values.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.GridLength
             * @memberof Windows.UI.Xaml.GridLength
             * @return  {Windows.UI.Xaml.GridLength}        A new instance of Gridlength with the same properties values.
             */
            Clone: function () {
                var $t;
                return ($t = new Windows.UI.Xaml.GridLength.ctor(), $t._type = this._type, $t._value = this._value, $t);
            },
            /**
             * Determines whether the specified Windows.UI.Xaml.GridLength
             is equal to the current Windows.UI.Xaml.GridLength.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.GridLength
             * @memberof Windows.UI.Xaml.GridLength
             * @param   {Windows.UI.Xaml.GridLength}    gridLength    The Windows.UI.Xaml.GridLength structure to compare with the current instance.
             * @return  {boolean}                                     true if the specified Windows.UI.Xaml.GridLength has the same value and Windows.UI.Xaml.GridLength.GridUnitType
             as the current instance; otherwise, false.
             */
            Equals: function (gridLength) {
                return (Windows.UI.Xaml.GridLength.op_Equality(this, gridLength.$clone()));
            },
            /**
             * Determines whether the specified object is equal to the
             current Windows.UI.Xaml.GridLength instance.
             *
             * @instance
             * @public
             * @override
             * @this Windows.UI.Xaml.GridLength
             * @memberof Windows.UI.Xaml.GridLength
             * @param   {System.Object}    oCompare    The object to compare with the current instance.
             * @return  {boolean}                      true if the specified object has the same value and Windows.UI.Xaml.GridUnitType
             as the current instance; otherwise, false.
             */
            equals: function (oCompare) {
                if (Bridge.is(oCompare, Windows.UI.Xaml.GridLength)) {
                    return (Windows.UI.Xaml.GridLength.op_Equality(this, System.Nullable.getValue(Bridge.cast(Bridge.unbox(oCompare, Windows.UI.Xaml.GridLength), Windows.UI.Xaml.GridLength))));
                } else {
                    return false;
                }
            },
            /**
             * Gets a hash code for the Windows.UI.Xaml.GridLength.
             *
             * @instance
             * @public
             * @override
             * @this Windows.UI.Xaml.GridLength
             * @memberof Windows.UI.Xaml.GridLength
             * @return  {number}        A hash code for the Windows.UI.Xaml.GridLength.
             */
            getHashCode: function () {
                return ((Bridge.Int.mul(Bridge.Int.clip32(this._value), 7) + this._type) | 0);
            },
            /**
             * Returns a System.String representation of the Windows.UI.Xaml.GridLength.
             *
             * @instance
             * @public
             * @override
             * @this Windows.UI.Xaml.GridLength
             * @memberof Windows.UI.Xaml.GridLength
             * @return  {string}        A System.String representation of the current Windows.UI.Xaml.GridLength
             structure.
             */
            toString: function () {
                if (this._type === Windows.UI.Xaml.GridUnitType.Auto) {
                    return "Auto";
                } else if (this._type === Windows.UI.Xaml.GridUnitType.Star) {
                    return (this._value === 1 ? "*" : (System.Double.format(this._value) || "") + "*");
                } else {
                    return System.Double.format(this._value);
                }
            },
            $clone: function (to) {
                var s = to || new Windows.UI.Xaml.GridLength();
                s._value = this._value;
                s._type = this._type;
                return s;
            }
        }
    });

    /**
     * Describes the kind of value that a Windows.UI.Xaml.GridLength
     object is holding.
     *
     * @public
     * @class Windows.UI.Xaml.GridUnitType
     */
    Bridge.define("Windows.UI.Xaml.GridUnitType", {
        $kind: "enum",
        statics: {
            fields: {
                /**
                 * [SECURITY CRITICAL] The size is determined by the size properties of the
                 content object.
                 *
                 * @static
                 * @public
                 * @memberof Windows.UI.Xaml.GridUnitType
                 * @constant
                 * @default 0
                 * @type Windows.UI.Xaml.GridUnitType
                 */
                Auto: 0,
                /**
                 * [SECURITY CRITICAL] The value is expressed in pixels.
                 *
                 * @static
                 * @public
                 * @memberof Windows.UI.Xaml.GridUnitType
                 * @constant
                 * @default 1
                 * @type Windows.UI.Xaml.GridUnitType
                 */
                Pixel: 1,
                /**
                 * [SECURITY CRITICAL] The value is expressed as a weighted proportion of available
                 *
                 * @static
                 * @public
                 * @memberof Windows.UI.Xaml.GridUnitType
                 * @constant
                 * @default 2
                 * @type Windows.UI.Xaml.GridUnitType
                 */
                Star: 2
            }
        }
    });

    /**
     * Describes how a child element is vertically positioned or stretched within
     a parent's layout slot.
     *
     * @public
     * @class Windows.UI.Xaml.HorizontalAlignment
     */
    Bridge.define("Windows.UI.Xaml.HorizontalAlignment", {
        $kind: "enum",
        statics: {
            fields: {
                /**
                 * The element is aligned to the top of the parent's layout slot.
                 *
                 * @static
                 * @public
                 * @memberof Windows.UI.Xaml.HorizontalAlignment
                 * @constant
                 * @default 0
                 * @type Windows.UI.Xaml.HorizontalAlignment
                 */
                Left: 0,
                /**
                 * The element is aligned to the center of the parent's layout slot.
                 *
                 * @static
                 * @public
                 * @memberof Windows.UI.Xaml.HorizontalAlignment
                 * @constant
                 * @default 1
                 * @type Windows.UI.Xaml.HorizontalAlignment
                 */
                Center: 1,
                /**
                 * The element is aligned to the right of the parent's layout slot.
                 *
                 * @static
                 * @public
                 * @memberof Windows.UI.Xaml.HorizontalAlignment
                 * @constant
                 * @default 2
                 * @type Windows.UI.Xaml.HorizontalAlignment
                 */
                Right: 2,
                /**
                 * The element is stretched to fill the entire layout slot of the parent element.
                 *
                 * @static
                 * @public
                 * @memberof Windows.UI.Xaml.HorizontalAlignment
                 * @constant
                 * @default 3
                 * @type Windows.UI.Xaml.HorizontalAlignment
                 */
                Stretch: 3
            }
        }
    });

    Bridge.define("Windows.UI.Xaml.Host", {
        fields: {
            _content: null,
            _settings: null
        },
        props: {
            /**
             * Gets the "Content" sub-object of this Host.
             *
             * @instance
             * @public
             * @readonly
             * @memberof Windows.UI.Xaml.Host
             * @function Content
             * @type Windows.UI.Xaml.Content
             */
            Content: {
                get: function () {
                    if (this._content == null) {
                        this._content = new Windows.UI.Xaml.Content();
                    }
                    return this._content;
                }
            },
            /**
             * Gets the "Settings" sub-object of this tHost.
             *
             * @instance
             * @public
             * @readonly
             * @memberof Windows.UI.Xaml.Host
             * @function Settings
             * @type System.Settings
             */
            Settings: {
                get: function () {
                    if (this._settings == null) {
                        this._settings = new System.Settings();
                    }
                    return this._settings;
                }
            }
        }
    });

    Bridge.define("Windows.UI.Xaml.ICanConvertToCSSValue", {
        $kind: "interface"
    });

    Bridge.define("Windows.UI.Xaml.ICanConvertToCSSValues", {
        $kind: "interface"
    });

    Bridge.define("Windows.UI.Xaml.ICloneOnAnimation", {
        $kind: "interface"
    });

    Bridge.define("Windows.UI.Xaml.IHasAccessToPropertiesWhereItIsUsed", {
        $kind: "interface"
    });

    /** @namespace Windows.UI.Xaml.Input */

    /**
     * The {@link } namespace defines the input and input event
     infrastructure for apps and UI elements.
     *
     * @class Windows.UI.Xaml.Input.NamespaceDoc
     */
    Bridge.define("Windows.UI.Xaml.Input.NamespaceDoc");

    /**
     * Represents a pointer device.
     *
     * @public
     * @class Windows.UI.Xaml.Input.Pointer
     */
    Bridge.define("Windows.UI.Xaml.Input.Pointer", {
        statics: {
            fields: {
                INTERNAL_captured: null
            }
        },
        fields: {
            _isInContact: false,
            _isInRange: false,
            _pointerDeviceType: 0,
            _pointerId: 0
        },
        props: {
            /**
             * Gets a value that determines whether the pointer device was in contact with
             a sensor or digitizer at the time that the event was reported.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Input.Pointer
             * @function IsInContact
             * @type boolean
             */
            IsInContact: {
                get: function () {
                    return this._isInContact;
                },
                set: function (value) {
                    this._isInContact = value;
                }
            },
            /**
             * Gets a value that determines whether the pointer device was in detection
             range of a sensor or digitizer at the time that the event was reported.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Input.Pointer
             * @function IsInRange
             * @type boolean
             */
            IsInRange: {
                get: function () {
                    return this._isInRange;
                },
                set: function (value) {
                    this._isInRange = value;
                }
            },
            /**
             * Gets the PointerDeviceType for the pointer device.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Input.Pointer
             * @function PointerDeviceType
             * @type Windows.Devices.Input.PointerDeviceType
             */
            PointerDeviceType: {
                get: function () {
                    return this._pointerDeviceType;
                },
                set: function (value) {
                    this._pointerDeviceType = value;
                }
            },
            /**
             * Gets the system-generated identifier for this pointer reference.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Input.Pointer
             * @function PointerId
             * @type number
             */
            PointerId: {
                get: function () {
                    return this._pointerId;
                },
                set: function (value) {
                    this._pointerId = value;
                }
            }
        },
        ctors: {
            init: function () {
                this._isInContact = false;
                this._isInRange = false;
            }
        }
    });

    Bridge.define("Windows.UI.Xaml.InternalFlags", {
        $kind: "enum",
        statics: {
            fields: {
                /**
                 * @static
                 * @public
                 * @memberof Windows.UI.Xaml.InternalFlags
                 * @constant
                 * @default 4
                 * @type Windows.UI.Xaml.InternalFlags
                 */
                HasImplicitStyleFromResources: 4,
                IsStyleUpdateInProgress: 64,
                IsThemeStyleUpdateInProgress: 128,
                HasThemeStyleEverBeenFetched: 4194304,
                HasLocalStyle: 8388608,
                HasStyleInvalidated: 33554432,
                ShouldLookupImplicitStyles: 1073741824
            }
        },
        $utype: System.UInt32
    });

    /**
     * The {@link } namespace provides XAML language support
     API for Windows Store apps and the Windows Runtime.
     *
     * @class Windows.UI.Xaml.Markup.NamespaceDoc
     */
    Bridge.define("Windows.UI.Xaml.Markup.NamespaceDoc");

    /** @namespace Windows.UI.Xaml.Media */

    /**
     * Parser for XAML abbreviated geometry.
     SVG path spec is closely followed http://www.w3.org/TR/SVG11/paths.html
     3/23/2006, new parser for performance (fyuan)
     *
     * @class Windows.UI.Xaml.Media.AbbreviatedGeometryParser
     */
    Bridge.define("Windows.UI.Xaml.Media.AbbreviatedGeometryParser", {
        statics: {
            fields: {
                AllowSign: false,
                AllowComma: false,
                IsFilled: false,
                IsClosed: false,
                IsStroked: false,
                IsSmoothJoin: false
            },
            ctors: {
                init: function () {
                    this.AllowSign = true;
                    this.AllowComma = true;
                    this.IsFilled = true;
                    this.IsClosed = true;
                    this.IsStroked = true;
                    this.IsSmoothJoin = true;
                }
            }
        },
        fields: {
            _formatProvider: null,
            _pathString: null,
            _pathLength: 0,
            _curIndex: 0,
            _figureStarted: false,
            _lastStart: null,
            _lastPoint: null,
            _secondLastPoint: null,
            _token: 0,
            _context: null
        },
        ctors: {
            init: function () {
                this._lastStart = new Windows.Foundation.Point();
                this._lastPoint = new Windows.Foundation.Point();
                this._secondLastPoint = new Windows.Foundation.Point();
            }
        },
        methods: {
            /**
             * Throw unexpected token exception
             *
             * @instance
             * @private
             * @this Windows.UI.Xaml.Media.AbbreviatedGeometryParser
             * @memberof Windows.UI.Xaml.Media.AbbreviatedGeometryParser
             * @return  {void}
             */
            ThrowBadToken: function () {
                throw new System.FormatException.$ctor1(System.String.format("Parser Unexpected Token in string \"{0}\" at position {1}.", this._pathString, Bridge.box(((this._curIndex - 1) | 0), System.Int32)));
            },
            More: function () {
                return this._curIndex < this._pathLength;
            },
            SkipWhiteSpace: function (allowComma) {
                var commaMet = false;

                while (this.More()) {
                    var ch = this._pathString.charCodeAt(this._curIndex);

                    switch (ch) {
                        case 32: 
                        case 10: 
                        case 13: 
                        case 9: 
                            break;
                        case 44: 
                            if (allowComma) {
                                commaMet = true;
                                allowComma = false;
                            } else {
                                this.ThrowBadToken();
                            }
                            break;
                        default: 
                            if (((ch > 32) && (ch <= 122)) || !System.Char.isWhiteSpace(String.fromCharCode(ch))) {
                                return commaMet;
                            }
                            break;
                    }

                    this._curIndex = (this._curIndex + 1) | 0;
                }

                return commaMet;
            },
            /**
             * Read the next non whitespace character
             *
             * @instance
             * @private
             * @this Windows.UI.Xaml.Media.AbbreviatedGeometryParser
             * @memberof Windows.UI.Xaml.Media.AbbreviatedGeometryParser
             * @return  {boolean}        True if not end of string
             */
            ReadToken: function () {
                this.SkipWhiteSpace(false);

                if (this.More()) {
                    this._token = this._pathString.charCodeAt(Bridge.identity(this._curIndex, ((this._curIndex = (this._curIndex + 1) | 0))));

                    return true;
                } else {
                    return false;
                }
            },
            IsNumber: function (allowComma) {
                var commaMet = this.SkipWhiteSpace(allowComma);

                if (this.More()) {
                    this._token = this._pathString.charCodeAt(this._curIndex);

                    if ((this._token === 46) || (this._token === 45) || (this._token === 43) || ((this._token >= 48) && (this._token <= 57)) || (this._token === 73) || (this._token === 78)) {
                        return true;
                    }
                }

                if (commaMet) {
                    this.ThrowBadToken();
                }

                return false;
            },
            SkipDigits: function (signAllowed) {
                if (signAllowed && this.More() && ((this._pathString.charCodeAt(this._curIndex) === 45) || this._pathString.charCodeAt(this._curIndex) === 43)) {
                    this._curIndex = (this._curIndex + 1) | 0;
                }

                while (this.More() && (this._pathString.charCodeAt(this._curIndex) >= 48) && (this._pathString.charCodeAt(this._curIndex) <= 57)) {
                    this._curIndex = (this._curIndex + 1) | 0;
                }
            },
            /**
             * Read a floating point number
             *
             * @instance
             * @private
             * @this Windows.UI.Xaml.Media.AbbreviatedGeometryParser
             * @memberof Windows.UI.Xaml.Media.AbbreviatedGeometryParser
             * @param   {boolean}    allowComma
             * @return  {number}
             */
            ReadNumber: function (allowComma) {
                if (!this.IsNumber(allowComma)) {
                    this.ThrowBadToken();
                }

                var simple = true;
                var start = this._curIndex;

                if (this.More() && ((this._pathString.charCodeAt(this._curIndex) === 45) || this._pathString.charCodeAt(this._curIndex) === 43)) {
                    this._curIndex = (this._curIndex + 1) | 0;
                }

                if (this.More() && (this._pathString.charCodeAt(this._curIndex) === 73)) {
                    this._curIndex = Math.min(((this._curIndex + 8) | 0), this._pathLength);
                    simple = false;
                } else if (this.More() && (this._pathString.charCodeAt(this._curIndex) === 78)) {
                    this._curIndex = Math.min(((this._curIndex + 3) | 0), this._pathLength);
                    simple = false;
                } else {
                    this.SkipDigits(false);

                    if (this.More() && (this._pathString.charCodeAt(this._curIndex) === 46)) {
                        simple = false;
                        this._curIndex = (this._curIndex + 1) | 0;
                        this.SkipDigits(false);
                    }

                    if (this.More() && ((this._pathString.charCodeAt(this._curIndex) === 69) || (this._pathString.charCodeAt(this._curIndex) === 101))) {
                        simple = false;
                        this._curIndex = (this._curIndex + 1) | 0;
                        this.SkipDigits(Windows.UI.Xaml.Media.AbbreviatedGeometryParser.AllowSign);
                    }
                }

                if (simple && (this._curIndex <= (((start + 8) | 0)))) {
                    var sign = 1;

                    if (this._pathString.charCodeAt(start) === 43) {
                        start = (start + 1) | 0;
                    } else if (this._pathString.charCodeAt(start) === 45) {
                        start = (start + 1) | 0;
                        sign = -1;
                    }

                    var value = 0;

                    while (start < this._curIndex) {
                        value = (Bridge.Int.mul(value, 10) + (((this._pathString.charCodeAt(start) - 48) | 0))) | 0;
                        start = (start + 1) | 0;
                    }

                    return Bridge.Int.mul(value, sign);
                } else {
                    var subString = this._pathString.substr(start, ((this._curIndex - start) | 0));

                    try {
                        return System.Convert.toDouble(subString, this._formatProvider);
                    } catch ($e1) {
                        $e1 = System.Exception.create($e1);
                        var except;
                        if (Bridge.is($e1, System.FormatException)) {
                            except = $e1;
                            throw new System.FormatException.$ctor2(System.String.format("Parser Unexpected Token in string \"{0}\" at position {1}.", this._pathString, Bridge.box(start, System.Int32)), except);
                        } else {
                            throw $e1;
                        }
                    }
                }
            },
            /**
             * Read a bool: 1 or 0
             *
             * @instance
             * @private
             * @this Windows.UI.Xaml.Media.AbbreviatedGeometryParser
             * @memberof Windows.UI.Xaml.Media.AbbreviatedGeometryParser
             * @return  {boolean}
             */
            ReadBool: function () {
                this.SkipWhiteSpace(Windows.UI.Xaml.Media.AbbreviatedGeometryParser.AllowComma);

                if (this.More()) {
                    this._token = this._pathString.charCodeAt(Bridge.identity(this._curIndex, ((this._curIndex = (this._curIndex + 1) | 0))));

                    if (this._token === 48) {
                        return false;
                    } else if (this._token === 49) {
                        return true;
                    }
                }

                this.ThrowBadToken();

                return false;
            },
            /**
             * Read a relative point
             *
             * @instance
             * @private
             * @this Windows.UI.Xaml.Media.AbbreviatedGeometryParser
             * @memberof Windows.UI.Xaml.Media.AbbreviatedGeometryParser
             * @param   {number}                      cmd           
             * @param   {boolean}                     allowcomma
             * @return  {Windows.Foundation.Point}
             */
            ReadPoint: function (cmd, allowcomma) {
                var x = this.ReadNumber(allowcomma);
                var y = this.ReadNumber(Windows.UI.Xaml.Media.AbbreviatedGeometryParser.AllowComma);

                if (cmd >= 97) {
                    x += this._lastPoint.X;
                    y += this._lastPoint.Y;
                }

                return new Windows.Foundation.Point.$ctor1(x, y);
            },
            /**
             * Reflect _secondLastPoint over _lastPoint to get a new point for smooth curve
             *
             * @instance
             * @private
             * @this Windows.UI.Xaml.Media.AbbreviatedGeometryParser
             * @memberof Windows.UI.Xaml.Media.AbbreviatedGeometryParser
             * @return  {Windows.Foundation.Point}
             */
            Reflect: function () {
                return new Windows.Foundation.Point.$ctor1(2 * this._lastPoint.X - this._secondLastPoint.X, 2 * this._lastPoint.Y - this._secondLastPoint.Y);
            },
            EnsureFigure: function () {
                if (!this._figureStarted) {
                    this._context.BeginFigure(this._lastStart.$clone(), Windows.UI.Xaml.Media.AbbreviatedGeometryParser.IsFilled, false);
                    this._figureStarted = true;
                }
            },
            /**
             * Parse a PathFigureCollection string
             *
             * @instance
             * @this Windows.UI.Xaml.Media.AbbreviatedGeometryParser
             * @memberof Windows.UI.Xaml.Media.AbbreviatedGeometryParser
             * @param   {Windows.UI.Xaml.Media.PathStreamGeometryContext}    context       
             * @param   {string}                                             pathString    
             * @param   {number}                                             startIndex
             * @return  {void}
             */
            ParseToGeometryContext: function (context, pathString, startIndex) {

                this._formatProvider = Bridge.global.System.Globalization.CultureInfo.invariantCulture;

                this._context = context;
                this._pathString = pathString;
                this._pathLength = pathString.length;
                this._curIndex = startIndex;

                this._secondLastPoint = new Windows.Foundation.Point.$ctor1(0, 0);
                this._lastPoint = new Windows.Foundation.Point.$ctor1(0, 0);
                this._lastStart = new Windows.Foundation.Point.$ctor1(0, 0);

                this._figureStarted = false;

                var first = true;

                var last_cmd = 32;

                while (this.ReadToken()) {
                    var cmd = this._token;

                    if (first) {
                        if ((cmd !== 77) && (cmd !== 109)) {
                            this.ThrowBadToken();
                        }

                        first = false;
                    }

                    switch (cmd) {
                        case 109: 
                        case 77: 
                            this._lastPoint = this.ReadPoint(cmd, false);
                            context.BeginFigure(this._lastPoint.$clone(), Windows.UI.Xaml.Media.AbbreviatedGeometryParser.IsFilled, false);
                            this._figureStarted = true;
                            this._lastStart = this._lastPoint.$clone();
                            last_cmd = 77;
                            while (this.IsNumber(Windows.UI.Xaml.Media.AbbreviatedGeometryParser.AllowComma)) {
                                this._lastPoint = this.ReadPoint(cmd, false);

                                context.LineTo(this._lastPoint.$clone(), Windows.UI.Xaml.Media.AbbreviatedGeometryParser.IsStroked, false);
                                last_cmd = 76;
                            }
                            break;
                        case 108: 
                        case 76: 
                        case 104: 
                        case 72: 
                        case 118: 
                        case 86: 
                            this.EnsureFigure();
                            do {
                                switch (cmd) {
                                    case 108: 
                                        this._lastPoint = this.ReadPoint(cmd, false);
                                        break;
                                    case 76: 
                                        this._lastPoint = this.ReadPoint(cmd, false);
                                        break;
                                    case 104: 
                                        this._lastPoint.X += this.ReadNumber(false);
                                        break;
                                    case 72: 
                                        this._lastPoint.X = this.ReadNumber(false);
                                        break;
                                    case 118: 
                                        this._lastPoint.Y += this.ReadNumber(false);
                                        break;
                                    case 86: 
                                        this._lastPoint.Y = this.ReadNumber(false);
                                        break;
                                }

                                context.LineTo(this._lastPoint.$clone(), Windows.UI.Xaml.Media.AbbreviatedGeometryParser.IsStroked, false);
                            } while (this.IsNumber(Windows.UI.Xaml.Media.AbbreviatedGeometryParser.AllowComma));
                            last_cmd = 76;
                            break;
                        case 99: 
                        case 67: 
                        case 115: 
                        case 83: 
                            this.EnsureFigure();
                            do {
                                var p = new Windows.Foundation.Point();

                                if ((cmd === 115) || (cmd === 83)) {
                                    if (last_cmd === 67) {
                                        p = this.Reflect();
                                    } else {
                                        p = this._lastPoint.$clone();
                                    }

                                    this._secondLastPoint = this.ReadPoint(cmd, false);
                                } else {
                                    p = this.ReadPoint(cmd, false);

                                    this._secondLastPoint = this.ReadPoint(cmd, Windows.UI.Xaml.Media.AbbreviatedGeometryParser.AllowComma);
                                }

                                this._lastPoint = this.ReadPoint(cmd, Windows.UI.Xaml.Media.AbbreviatedGeometryParser.AllowComma);

                                context.BezierTo(p.$clone(), this._secondLastPoint.$clone(), this._lastPoint.$clone(), Windows.UI.Xaml.Media.AbbreviatedGeometryParser.IsStroked, false);

                                last_cmd = 67;
                            } while (this.IsNumber(Windows.UI.Xaml.Media.AbbreviatedGeometryParser.AllowComma));
                            break;
                        case 113: 
                        case 81: 
                        case 116: 
                        case 84: 
                            this.EnsureFigure();
                            do {
                                if ((cmd === 116) || (cmd === 84)) {
                                    if (last_cmd === 81) {
                                        this._secondLastPoint = this.Reflect();
                                    } else {
                                        this._secondLastPoint = this._lastPoint.$clone();
                                    }

                                    this._lastPoint = this.ReadPoint(cmd, false);
                                } else {
                                    this._secondLastPoint = this.ReadPoint(cmd, false);
                                    this._lastPoint = this.ReadPoint(cmd, Windows.UI.Xaml.Media.AbbreviatedGeometryParser.AllowComma);
                                }

                                context.QuadraticBezierTo(this._secondLastPoint.$clone(), this._lastPoint.$clone(), Windows.UI.Xaml.Media.AbbreviatedGeometryParser.IsStroked, false);

                                last_cmd = 81;
                            } while (this.IsNumber(Windows.UI.Xaml.Media.AbbreviatedGeometryParser.AllowComma));
                            break;
                        case 97: 
                        case 65: 
                            this.EnsureFigure();
                            do {
                                var w = this.ReadNumber(false);
                                var h = this.ReadNumber(Windows.UI.Xaml.Media.AbbreviatedGeometryParser.AllowComma);
                                var rotation = this.ReadNumber(Windows.UI.Xaml.Media.AbbreviatedGeometryParser.AllowComma);
                                var large = this.ReadBool();
                                var sweep = this.ReadBool();

                                this._lastPoint = this.ReadPoint(cmd, Windows.UI.Xaml.Media.AbbreviatedGeometryParser.AllowComma);

                                context.ArcTo(this._lastPoint.$clone(), new Windows.Foundation.Size.$ctor1(w, h), rotation, large, sweep ? Windows.UI.Xaml.Media.SweepDirection.Clockwise : Windows.UI.Xaml.Media.SweepDirection.Counterclockwise, Windows.UI.Xaml.Media.AbbreviatedGeometryParser.IsStroked, false);
                            } while (this.IsNumber(Windows.UI.Xaml.Media.AbbreviatedGeometryParser.AllowComma));
                            last_cmd = 65;
                            break;
                        case 122: 
                        case 90: 
                            this.EnsureFigure();
                            context.SetClosedState(Windows.UI.Xaml.Media.AbbreviatedGeometryParser.IsClosed);
                            this._figureStarted = false;
                            last_cmd = 90;
                            this._lastPoint = this._lastStart.$clone();
                            break;
                        default: 
                            this.ThrowBadToken();
                            break;
                    }
                }
            }
        }
    });

    Bridge.define("Windows.UI.Xaml.Media.Animation.AnimationHelpers", {
        statics: {
            methods: {
                CallVelocity: function (domElement, Duration, easingFunction, visualStateGroupName, callbackForWhenfinished, jsFromToValues) {
                    var $t;
                    var easingFunctionAsString = "linear";
                    if (easingFunction != null) {
                        easingFunctionAsString = easingFunction.GetFunctionAsString();
                    }

                    var duration = Duration.TimeSpan.getTotalMilliseconds();
                    if (duration === 0) {
                        ++duration;
                    }

                    var options = new Object();
                    if (Bridge.staticEquals(callbackForWhenfinished, null)) {

                        options.easing = easingFunctionAsString;
options.duration = Bridge.box(duration, System.Double, System.Double.format, System.Double.getHashCode);
options.queue = false;
options.queue = visualStateGroupName;

                    } else {

                        options.easing = easingFunctionAsString;
options.duration = Bridge.box(duration, System.Double, System.Double.format, System.Double.getHashCode);
options.queue = false;
options.queue = visualStateGroupName;
options.complete = callbackForWhenfinished;

                    }

                    if (easingFunction != null) {
                        var additionalOptions = easingFunction.GetAdditionalOptionsForVelocityCall();
                        if (additionalOptions != null) {
                            $t = Bridge.getEnumerator(additionalOptions.Keys);
                            try {
                                while ($t.moveNext()) {
                                    var key = $t.Current;
                                    options[key] = additionalOptions.getItem(key);
                                }
                            } finally {
                                if (Bridge.is($t, System.IDisposable)) {
                                    $t.System$IDisposable$Dispose();
                                }
                            }
                        }
                    }

Velocity(                    domElement, jsFromToValues, options);
                                                     Velocity.Utilities.dequeue(domElement, visualStateGroupName);
                },
                ApplyValue: function (target, propertyPath, value, isVisualStateChange) {
                    if (isVisualStateChange) {
                        propertyPath.INTERNAL_PropertySetVisualState(target, value);
                    } else {
                        propertyPath.INTERNAL_PropertySetAnimationValue(target, value);
                    }
                },
                IsValueNull: function (from) {
                    return from == null || Windows.UI.Xaml.Media.Animation.AnimationHelpers.CheckIfObjectIsNullNullable(from);
                },
                CheckIfObjectIsNullNullable: function (obj) {
                    var type = Bridge.getType(obj);
                    if (System.String.startsWith(Bridge.Reflection.getTypeFullName(type), "System.Nullable`1")) {
                        return !obj.hasValue;
                    } else {
                        return false;
                    }
                }
            }
        }
    });

    Bridge.define("Windows.UI.Xaml.Media.Animation.IKeyFrame", {
        $kind: "interface"
    });

    /** @namespace Windows.UI.Xaml.Media.Animation */

    /**
     * Specifies how the animation associated with an easing function interpolates.
     *
     * @public
     * @class Windows.UI.Xaml.Media.Animation.EasingMode
     */
    Bridge.define("Windows.UI.Xaml.Media.Animation.EasingMode", {
        $kind: "enum",
        statics: {
            fields: {
                /**
                 * Interpolation follows 100% interpolation minus the output of the formula
                 associated with the easing function.
                 *
                 * @static
                 * @public
                 * @memberof Windows.UI.Xaml.Media.Animation.EasingMode
                 * @constant
                 * @default 0
                 * @type Windows.UI.Xaml.Media.Animation.EasingMode
                 */
                EaseOut: 0,
                /**
                 * Interpolation follows the mathematical formula associated with the easing
                 function.
                 *
                 * @static
                 * @public
                 * @memberof Windows.UI.Xaml.Media.Animation.EasingMode
                 * @constant
                 * @default 1
                 * @type Windows.UI.Xaml.Media.Animation.EasingMode
                 */
                EaseIn: 1,
                /**
                 * Interpolation uses EaseIn for the first half of the animation and EaseOut
                 for the second half.
                 *
                 * @static
                 * @public
                 * @memberof Windows.UI.Xaml.Media.Animation.EasingMode
                 * @constant
                 * @default 2
                 * @type Windows.UI.Xaml.Media.Animation.EasingMode
                 */
                EaseInOut: 2
            }
        }
    });

    Bridge.define("Windows.UI.Xaml.Media.Animation.IEasingFunction", {
        $kind: "interface"
    });

    /**
     * this class has been added to make passing the parameters through the iterations easier.
     *
     * @class Windows.UI.Xaml.Media.Animation.IterationParameters
     */
    Bridge.define("Windows.UI.Xaml.Media.Animation.IterationParameters", {
        fields: {
            Target: null,
            Guid: null,
            UseTransitions: false,
            VisualStateGroupName: null,
            IsTargetParentTheTarget: false,
            IsVisualStateChange: false
        },
        ctors: {
            init: function () {
                this.Guid = new System.Guid();
            }
        },
        methods: {
            Clone: function () {
                var $t;
                return ($t = new Windows.UI.Xaml.Media.Animation.IterationParameters(), $t.Target = this.Target, $t.Guid = this.Guid, $t.UseTransitions = this.UseTransitions, $t.VisualStateGroupName = this.VisualStateGroupName, $t.IsTargetParentTheTarget = this.IsTargetParentTheTarget, $t.IsVisualStateChange = this.IsVisualStateChange, $t);
            }
        }
    });

    /**
     * Specifies when a particular key frame should take place during an animation.
     *
     * @public
     * @class Windows.UI.Xaml.Media.Animation.KeyTime
     */
    Bridge.define("Windows.UI.Xaml.Media.Animation.KeyTime", {
        $kind: "struct",
        statics: {
            ctors: {
                ctor: function () {
                    DotNetForHtml5.Core.TypeFromStringConverters.RegisterConverter(Windows.UI.Xaml.Media.Animation.KeyTime, Windows.UI.Xaml.Media.Animation.KeyTime.INTERNAL_ConvertFromString);
                }
            },
            methods: {
                /**
                 * Creates a new System.Windows.Media.Animation.KeyTime instance, with the System.Windows.Media.Animation.KeyTimeType
                 property initialized to the value of the specified parameter.
                 *
                 * @static
                 * @public
                 * @this Windows.UI.Xaml.Media.Animation.KeyTime
                 * @memberof Windows.UI.Xaml.Media.Animation.KeyTime
                 * @param   {System.TimeSpan}                            timeSpan    The value of the new System.Windows.Media.Animation.KeyTime.
                 * @return  {Windows.UI.Xaml.Media.Animation.KeyTime}                A new System.Windows.Media.Animation.KeyTime instance, initialized to the value
                 of timeSpan.
                 */
                FromTimeSpan: function (timeSpan) {
                    return new Windows.UI.Xaml.Media.Animation.KeyTime.$ctor1(timeSpan);
                },
                INTERNAL_ConvertFromString: function (keyTimeCode) {
                    try {
                        if (Bridge.referenceEquals(keyTimeCode, "Uniform")) {
                            throw new System.NotImplementedException.$ctor1("The Value \"Uniform\" for keyTime is not supported yet.");
                        } else if (Bridge.referenceEquals(keyTimeCode, "Paced")) {
                            throw new System.NotImplementedException.$ctor1("The Value \"Paced\" for keyTime is not supported yet.");
                        } else if (System.String.endsWith(keyTimeCode, "%")) {
                            throw new System.NotImplementedException.$ctor1("The percentage values for keyTime are not supported yet.");
                        } else {
                            var timeSpan = INTERNAL_BridgeWorkarounds.TimeSpanParse(keyTimeCode, false);
                            return new Windows.UI.Xaml.Media.Animation.KeyTime.$ctor1(timeSpan).$clone();
                        }
                    } catch (ex) {
                        ex = System.Exception.create(ex);
                        throw new System.Exception("Invalid KeyTime: " + (keyTimeCode || ""), ex);
                    }
                }/**
                 * // Compares two Windows.UI.Xaml.Media.Animation.KeyTime values for inequality.
                //
                 * // Compares two Windows.UI.Xaml.Media.Animation.KeyTime values for equality.
                //
                 * // Implicitly converts a Windows.UI.Xaml.Media.Animation.KeyTime.TimeSpan to
                // a Windows.UI.Xaml.Media.Animation.KeyTime.
                //
                 * // Indicates whether a specified Windows.UI.Xaml.Media.Animation.KeyTime is
                // equal to this Windows.UI.Xaml.Media.Animation.KeyTime.
                //
                 * // Indicates whether a Windows.UI.Xaml.Media.Animation.KeyTime is equal to this
                // Windows.UI.Xaml.Media.Animation.KeyTime.
                //
                 * // Indicates whether two Windows.UI.Xaml.Media.Animation.KeyTime values are
                // equal.
                //
                 * // Creates a new Windows.UI.Xaml.Media.Animation.KeyTime, using the supplied
                // System.TimeSpan.
                //
                 * // Returns a hash code representing this Windows.UI.Xaml.Media.Animation.KeyTime.
                //
                 * // Returns a string representation of this Windows.UI.Xaml.Media.Animation.KeyTime.
                //
                 * Implicitly creates a KeyTime value from a Time value.
                 *
                 * @static
                 * @public
                 * @this Windows.UI.Xaml.Media.Animation.KeyTime
                 * @memberof Windows.UI.Xaml.Media.Animation.KeyTime
                 * @param   {System.TimeSpan}                            timeSpan    The Time value.
                 * @return  {Windows.UI.Xaml.Media.Animation.KeyTime}                A new KeyTime.
                 */
                ,
                op_Implicit: function (timeSpan) {
                    return Windows.UI.Xaml.Media.Animation.KeyTime.FromTimeSpan(timeSpan);
                },
                getDefaultValue: function () { return new Windows.UI.Xaml.Media.Animation.KeyTime(); }
            }
        },
        fields: {
            _timeSpan: null
        },
        props: {
            /**
             * Gets the time when the key frame ends, expressed as a time relative to the
             beginning of the animation.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Media.Animation.KeyTime
             * @function TimeSpan
             * @type System.TimeSpan
             */
            TimeSpan: {
                get: function () {
                    return this._timeSpan;
                },
                set: function (value) {
                    this._timeSpan = value;
                }
            }
        },
        ctors: {
            init: function () {
                this._timeSpan = new System.TimeSpan();
            },
            $ctor1: function (timeSpan) {
                this.$initialize();
                this._timeSpan = timeSpan;
            },
            ctor: function () {
                this.$initialize();
            }
        },
        methods: {
            getHashCode: function () {
                var h = Bridge.addHash([1423889076, this._timeSpan]);
                return h;
            },
            equals: function (o) {
                if (!Bridge.is(o, Windows.UI.Xaml.Media.Animation.KeyTime)) {
                    return false;
                }
                return Bridge.equals(this._timeSpan, o._timeSpan);
            },
            $clone: function (to) {
                var s = to || new Windows.UI.Xaml.Media.Animation.KeyTime();
                s._timeSpan = this._timeSpan;
                return s;
            }
        }
    });

    Bridge.define("Windows.UI.Xaml.Media.Animation.ObjectAnimationUsingKeyFrames.NullableTimer", {
        $kind: "nested class",
        fields: {
            _timer: null
        },
        events: {
            Completed: null
        },
        props: {
            HasTimer: {
                get: function () {
                    return this._timer != null;
                }
            }
        },
        ctors: {
            ctor: function (interval) {
                this.$initialize();
                if (System.TimeSpan.gt(interval, System.TimeSpan.zero)) {
                    this._timer = this.NewTimer(interval);
                } else {
                    this._timer = null;
                }
            }
        },
        methods: {
            NewTimer: function (interval) {
                var $t;
                var timer = ($t = new Windows.UI.Xaml.DispatcherTimer(), $t.Interval = interval, $t);
                timer.addTick(Bridge.fn.cacheBind(this, this.Timer_Tick));
                return timer;
            },
            Timer_Tick: function (sender, e) {
                this.INTERNAL_RaiseCompletedEvent();
                this.Stop();
            },
            INTERNAL_RaiseCompletedEvent: function () {
                if (!Bridge.staticEquals(this.Completed, null)) {
                    this.Completed(this, { });
                }
            },
            Start: function () {
                if (this.HasTimer) {
                    this._timer.Start();
                } else {
                    this.INTERNAL_RaiseCompletedEvent();
                }
            },
            Stop: function () {
                if (this.HasTimer) {
                    this._timer.Stop();
                }
            },
            Reset: function () {
                if (this.HasTimer) {
                    this._timer.Stop();
                    this._timer = this.NewTimer(this._timer.Interval);
                }
            }
        }
    });

    /**
     * Describes how a Windows.UI.Xaml.Media.Animation.Timeline repeats its simple
     duration.
     * // Returns the hash code of this instance.
    //
     * // Returns a string representation of this Windows.UI.Xaml.Media.Animation.RepeatBehavior.
    //
     * // Returns a string representation of this Windows.UI.Xaml.Media.Animation.RepeatBehavior
    // with the specified format.
    //
     *
     * @public
     * @class Windows.UI.Xaml.Media.Animation.RepeatBehavior
     * @return  {[type]}        A string representation of this Windows.UI.Xaml.Media.Animation.RepeatBehavior.
     */
    Bridge.define("Windows.UI.Xaml.Media.Animation.RepeatBehavior", {
        $kind: "struct",
        statics: {
            props: {
                /**
                 * // Gets the total length of time a Windows.UI.Xaml.Media.Animation.Timeline
                // should play.
                //
                 * Gets a Windows.UI.Xaml.Media.Animation.RepeatBehavior that specifies an infinite
                 number of repetitions.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Media.Animation.RepeatBehavior
                 * @function Forever
                 * @type Windows.UI.Xaml.Media.Animation.RepeatBehavior
                 */
                Forever: {
                    get: function () {
                        var forever = new Windows.UI.Xaml.Media.Animation.RepeatBehavior.ctor();
                        forever._type = Windows.UI.Xaml.Media.Animation.RepeatBehaviorType.Forever;
                        return forever.$clone();
                    }
                }
            },
            ctors: {
                ctor: function () {
                    DotNetForHtml5.Core.TypeFromStringConverters.RegisterConverter(Windows.UI.Xaml.Media.Animation.RepeatBehavior, Windows.UI.Xaml.Media.Animation.RepeatBehavior.INTERNAL_ConvertFromString);
                }
            },
            methods: {
                INTERNAL_ConvertFromString: function (arg) {
                    var loweredArg = arg.toLowerCase();
                    if (Bridge.referenceEquals(loweredArg, "forever")) {
                        return Windows.UI.Xaml.Media.Animation.RepeatBehavior.Forever.$clone();
                    } else if (System.String.endsWith(loweredArg, "x")) {
                        var repeatCount = System.Double.parse(loweredArg.substr(0, ((loweredArg.length - 1) | 0)));
                        return new Windows.UI.Xaml.Media.Animation.RepeatBehavior.$ctor1(repeatCount).$clone();
                    } else {
                        throw new System.FormatException.$ctor1("The string: \"" + (arg || "") + "\" could not be parsed into a RepeatBehavior. Note: The duration is not supported yet as a RepeatBehavior.");
                    }
                },
                /**
                 * Indicates whether the two specified Windows.UI.Xaml.Media.Animation.RepeatBehavior
                 values are equal.
                 *
                 * @static
                 * @public
                 * @this Windows.UI.Xaml.Media.Animation.RepeatBehavior
                 * @memberof Windows.UI.Xaml.Media.Animation.RepeatBehavior
                 * @param   {Windows.UI.Xaml.Media.Animation.RepeatBehavior}    repeatBehavior1    The first value to compare.
                 * @param   {Windows.UI.Xaml.Media.Animation.RepeatBehavior}    repeatBehavior2    The second value to compare.
                 * @return  {boolean}                                                              true if both the type and repeat behavior of repeatBehavior1 are equal to
                 that of repeatBehavior2; otherwise, false.
                 */
                Equals: function (repeatBehavior1, repeatBehavior2) {
                    return Windows.UI.Xaml.Media.Animation.RepeatBehavior.op_Equality(repeatBehavior1.$clone(), repeatBehavior2.$clone());

                }/**
                 * // Initializes a new instance of the Windows.UI.Xaml.Media.Animation.RepeatBehavior
                // structure with the specified repeat duration.
                //
                 * Indicates whether the two Windows.UI.Xaml.Media.Animation.RepeatBehavior
                 values are not equal.
                 *
                 * @static
                 * @public
                 * @this Windows.UI.Xaml.Media.Animation.RepeatBehavior
                 * @memberof Windows.UI.Xaml.Media.Animation.RepeatBehavior
                 * @param   {Windows.UI.Xaml.Media.Animation.RepeatBehavior}    repeatBehavior1    The first value to compare.
                 * @param   {Windows.UI.Xaml.Media.Animation.RepeatBehavior}    repeatBehavior2    The second value to compare.
                 * @return  {boolean}                                                              true if repeatBehavior1 and repeatBehavior2 are different types or the repeat
                 behavior properties are not equal; otherwise, false.
                 */
                ,
                op_Inequality: function (repeatBehavior1, repeatBehavior2) {
                    if (repeatBehavior1._hasCount) {
                        if ((!repeatBehavior2._hasCount) || repeatBehavior1._count !== repeatBehavior2._count) {
                            return true;
                        }
                    } else if (repeatBehavior2._hasCount) {
                        return true;
                    }

                    return false;
                }/**
                 * Indicates whether the two specified Windows.UI.Xaml.Media.Animation.RepeatBehavior
                 values are equal.
                 *
                 * @static
                 * @public
                 * @this Windows.UI.Xaml.Media.Animation.RepeatBehavior
                 * @memberof Windows.UI.Xaml.Media.Animation.RepeatBehavior
                 * @param   {Windows.UI.Xaml.Media.Animation.RepeatBehavior}    repeatBehavior1    The first value to compare.
                 * @param   {Windows.UI.Xaml.Media.Animation.RepeatBehavior}    repeatBehavior2    The second value to compare.
                 * @return  {boolean}                                                              true if both the type and repeat behavior of repeatBehavior1 are equal to
                 that of repeatBehavior2; otherwise, false.
                 */
                ,
                op_Equality: function (repeatBehavior1, repeatBehavior2) {
                    return !(Windows.UI.Xaml.Media.Animation.RepeatBehavior.op_Inequality(repeatBehavior1.$clone(), repeatBehavior2.$clone()));
                },
                getDefaultValue: function () { return new Windows.UI.Xaml.Media.Animation.RepeatBehavior(); }
            }
        },
        fields: {
            _count: 0,
            _hasCount: false,
            /**
             * // Gets a value that indicates whether the repeat behavior has a specified repeat
            // duration.
            //
             *
             * @instance
             * @private
             * @memberof Windows.UI.Xaml.Media.Animation.RepeatBehavior
             * @type Windows.UI.Xaml.Media.Animation.RepeatBehaviorType
             */
            _type: 0
        },
        props: {
            /**
             * Gets the number of times a Windows.UI.Xaml.Media.Animation.Timeline should
             repeat.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Media.Animation.RepeatBehavior
             * @function Count
             * @type number
             */
            Count: {
                get: function () {
                    return this._count;
                },
                set: function (value) {
                    this._count = value;
                    this._hasCount = true;
                    this._type = Windows.UI.Xaml.Media.Animation.RepeatBehaviorType.Count;
                }
            },
            /**
             * Gets a value that indicates whether the repeat behavior has a specified iteration
             count.
             *
             * @instance
             * @public
             * @readonly
             * @memberof Windows.UI.Xaml.Media.Animation.RepeatBehavior
             * @function HasCount
             * @type boolean
             */
            HasCount: {
                get: function () {
                    return this._hasCount;
                }
            },
            /**
             * Gets or sets one of the Windows.UI.Xaml.Media.Animation.RepeatBehaviorType
             values that describes the way behavior repeats.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Media.Animation.RepeatBehavior
             * @function Type
             * @type Windows.UI.Xaml.Media.Animation.RepeatBehaviorType
             */
            Type: {
                get: function () {
                    return this._type;
                },
                set: function (value) {
                    this._type = value;
                    if (value === Windows.UI.Xaml.Media.Animation.RepeatBehaviorType.Count) {
                        if (this._count > 0) {
                            this._hasCount = true;
                        }
                    } else if (value === Windows.UI.Xaml.Media.Animation.RepeatBehaviorType.Forever) {
                        this._hasCount = false;
                    }
                }
            }
        },
        ctors: {
            /**
             * Initializes a new instance of the Windows.UI.Xaml.Media.Animation.RepeatBehavior
             structure with the specified iteration count.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Media.Animation.RepeatBehavior
             * @memberof Windows.UI.Xaml.Media.Animation.RepeatBehavior
             * @param   {number}    count    A number greater than or equal to 0 that specifies the number of iterations
             for an animation.
             * @return  {void}
             */
            $ctor1: function (count) {
                this.$initialize();
                this._type = Windows.UI.Xaml.Media.Animation.RepeatBehaviorType.Count;
                this._count = count;
                this._hasCount = true;
            },
            ctor: function () {
                this.$initialize();
            }
        },
        methods: {
            /**
             * Indicates whether the specified object is equal to this Windows.UI.Xaml.Media.Animation.RepeatBehavior.
             *
             * @instance
             * @public
             * @override
             * @this Windows.UI.Xaml.Media.Animation.RepeatBehavior
             * @memberof Windows.UI.Xaml.Media.Animation.RepeatBehavior
             * @param   {System.Object}    value    The object to compare with this Windows.UI.Xaml.Media.Animation.RepeatBehavior.
             * @return  {boolean}                   true if value is a Windows.UI.Xaml.Media.Animation.RepeatBehavior that represents
             the same repeat behavior as this Windows.UI.Xaml.Media.Animation.RepeatBehavior;
             otherwise, false.
             */
            equals: function (value) {
                if (!(Bridge.is(value, Windows.UI.Xaml.Media.Animation.RepeatBehavior))) {
                    return false;
                }
                return Windows.UI.Xaml.Media.Animation.RepeatBehavior.op_Equality(this, System.Nullable.getValue(Bridge.cast(Bridge.unbox(value, Windows.UI.Xaml.Media.Animation.RepeatBehavior), Windows.UI.Xaml.Media.Animation.RepeatBehavior)));
            },
            /**
             * Returns a value that indicates whether the specified Windows.UI.Xaml.Media.Animation.RepeatBehavior
             is equal to this Windows.UI.Xaml.Media.Animation.RepeatBehavior.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Media.Animation.RepeatBehavior
             * @memberof Windows.UI.Xaml.Media.Animation.RepeatBehavior
             * @param   {Windows.UI.Xaml.Media.Animation.RepeatBehavior}    repeatBehavior    The value to compare to this Windows.UI.Xaml.Media.Animation.RepeatBehavior.
             * @return  {boolean}                                                             true if both the type and repeat behavior of repeatBehavior are equal to
             this Windows.UI.Xaml.Media.Animation.RepeatBehavior; otherwise, false.
             */
            Equals: function (repeatBehavior) {
                return Windows.UI.Xaml.Media.Animation.RepeatBehavior.op_Equality(this, repeatBehavior.$clone());
            },
            getHashCode: function () {
                var h = Bridge.addHash([5170113930, this._count, this._hasCount, this._type]);
                return h;
            },
            $clone: function (to) {
                var s = to || new Windows.UI.Xaml.Media.Animation.RepeatBehavior();
                s._count = this._count;
                s._hasCount = this._hasCount;
                s._type = this._type;
                return s;
            }
        }
    });

    /**
     * Specifies the repeat mode that a Windows.UI.Xaml.Media.Animation.RepeatBehavior
     raw value represents.
     *
     * @public
     * @class Windows.UI.Xaml.Media.Animation.RepeatBehaviorType
     */
    Bridge.define("Windows.UI.Xaml.Media.Animation.RepeatBehaviorType", {
        $kind: "enum",
        statics: {
            fields: {
                /**
                 * The Windows.UI.Xaml.Media.Animation.RepeatBehavior represents a case where
                 the timeline should repeat for a fixed number of complete runs.
                 *
                 * @static
                 * @public
                 * @memberof Windows.UI.Xaml.Media.Animation.RepeatBehaviorType
                 * @constant
                 * @default 0
                 * @type Windows.UI.Xaml.Media.Animation.RepeatBehaviorType
                 */
                Count: 0,
                /**
                 * / The Windows.UI.Xaml.Media.Animation.RepeatBehavior represents a case where
                / the timeline should repeat for a time duration, which might result in an
                / animation terminating part way through.
                /
                 * The Windows.UI.Xaml.Media.Animation.RepeatBehavior represents a case where
                 the timeline should repeat indefinitely.
                 *
                 * @static
                 * @public
                 * @memberof Windows.UI.Xaml.Media.Animation.RepeatBehaviorType
                 * @constant
                 * @default 2
                 * @type Windows.UI.Xaml.Media.Animation.RepeatBehaviorType
                 */
                Forever: 2
            }
        }
    });

    /**
     * Specifies the coordinate system used by a System.Windows.Media.Brush.
     *
     * @public
     * @class Windows.UI.Xaml.Media.BrushMappingMode
     */
    Bridge.define("Windows.UI.Xaml.Media.BrushMappingMode", {
        $kind: "enum",
        statics: {
            fields: {
                /**
                 * The coordinate system is not relative to a bounding box. Values are interpreted
                 directly in local space.
                 *
                 * @static
                 * @public
                 * @memberof Windows.UI.Xaml.Media.BrushMappingMode
                 * @constant
                 * @default 0
                 * @type Windows.UI.Xaml.Media.BrushMappingMode
                 */
                Absolute: 0,
                /**
                 * The coordinate system is relative to a bounding box: 0 indicates 0 percent
                 of the bounding box, and 1 indicates 100 percent of the bounding box. For
                 example, (0.5, 0.5) describes a point in the middle of the bounding box,
                 and (1, 1) describes a point at the bottom right of the bounding box.
                 *
                 * @static
                 * @public
                 * @memberof Windows.UI.Xaml.Media.BrushMappingMode
                 * @constant
                 * @default 1
                 * @type Windows.UI.Xaml.Media.BrushMappingMode
                 */
                RelativeToBoundingBox: 1
            }
        }
    });

    /**
     * Specifies how the intersecting areas of PathFigure objects contained in a
     Geometry are combined to form the area of the Geometry.
     *
     * @public
     * @class Windows.UI.Xaml.Media.FillRule
     */
    Bridge.define("Windows.UI.Xaml.Media.FillRule", {
        $kind: "enum",
        statics: {
            fields: {
                /**
                 * Rule that determines whether a point is in the fill region by drawing a ray
                 from that point to infinity in any direction and counting the number of path
                 segments within the given shape that the ray crosses. If this number is odd,
                 the point is inside; if even, the point is outside.
                 *
                 * @static
                 * @public
                 * @memberof Windows.UI.Xaml.Media.FillRule
                 * @constant
                 * @default 0
                 * @type Windows.UI.Xaml.Media.FillRule
                 */
                EvenOdd: 0,
                /**
                 * Rule that determines whether a point is in the fill region of the path by
                 drawing a ray from that point to infinity in any direction and then examining
                 the places where a segment of the shape crosses the ray. Starting with a
                 count of zero, add one each time a segment crosses the ray from left to right
                 and subtract one each time a path segment crosses the ray from right to left.
                 After counting the crossings, if the result is zero then the point is outside
                 the path. Otherwise, it is inside.
                 *
                 * @static
                 * @public
                 * @memberof Windows.UI.Xaml.Media.FillRule
                 * @constant
                 * @default 1
                 * @type Windows.UI.Xaml.Media.FillRule
                 */
                Nonzero: 1
            }
        }
    });

    /**
     * Represents a family of related fonts.
     *
     * @public
     * @class Windows.UI.Xaml.Media.FontFamily
     */
    Bridge.define("Windows.UI.Xaml.Media.FontFamily", {
        statics: {
            ctors: {
                ctor: function () {
                    DotNetForHtml5.Core.TypeFromStringConverters.RegisterConverter(Windows.UI.Xaml.Media.FontFamily, Windows.UI.Xaml.Media.FontFamily.INTERNAL_ConvertFromString);
                }
            },
            methods: {
                INTERNAL_ConvertFromString: function (fontCode) {
                    return new Windows.UI.Xaml.Media.FontFamily(fontCode);
                }
            }
        },
        fields: {
            _source: null
        },
        props: {
            /**
             * Gets the font family name that is used to construct the FontFamily object.
             *
             * @instance
             * @public
             * @readonly
             * @memberof Windows.UI.Xaml.Media.FontFamily
             * @function Source
             * @type string
             */
            Source: {
                get: function () {
                    return this._source;
                }
            }
        },
        ctors: {
            /**
             * Initializes a new instance of the FontFamily class from the specified font
             family string.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Media.FontFamily
             * @memberof Windows.UI.Xaml.Media.FontFamily
             * @param   {string}    familyName    The family name of the font to represent. This can include a hashed suffix.
             * @return  {void}
             */
            ctor: function (familyName) {
                this.$initialize();
                this._source = familyName;
            }
        },
        methods: {
            INTERNAL_ToHtmlString: function () {
                return this.Source;
            },
            toString: function () {
                return this._source;
            }
        }
    });

    Bridge.define("Windows.UI.Xaml.Media.GeometryParser", {
        statics: {
            methods: {
                ParseGeometry: function (pathString) {
                    var fillRule = { v : Windows.UI.Xaml.Media.FillRule.EvenOdd };
                    var context = new Windows.UI.Xaml.Media.PathStreamGeometryContext.ctor();
                    Windows.UI.Xaml.Media.GeometryParser.ParseStringToStreamGeometryContext(context, pathString, fillRule);
                    var geometry = context.GetPathGeometry();
                    geometry.FillRule = fillRule.v;
                    return geometry;
                },
                ParseStringToStreamGeometryContext: function (geometry, pathString, fillRule) {
                    if (pathString != null) {
                        var curIndex = 0;

                        while ((curIndex < pathString.length) && System.Char.isWhiteSpace((pathString).charAt(curIndex))) {
                            curIndex = (curIndex + 1) | 0;
                        }

                        if (curIndex < pathString.length) {
                            if (pathString.charCodeAt(curIndex) === 70) {
                                curIndex = (curIndex + 1) | 0;

                                while ((curIndex < pathString.length) && System.Char.isWhiteSpace((pathString).charAt(curIndex))) {
                                    curIndex = (curIndex + 1) | 0;
                                }

                                if ((curIndex === pathString.length) || ((pathString.charCodeAt(curIndex) !== 48) && (pathString.charCodeAt(curIndex) !== 49))) {
                                    throw new System.FormatException.$ctor1("Parsers Illegal Token");
                                }

                                fillRule.v = pathString.charCodeAt(curIndex) === 48 ? Windows.UI.Xaml.Media.FillRule.EvenOdd : Windows.UI.Xaml.Media.FillRule.Nonzero;

                                curIndex = (curIndex + 1) | 0;
                            }
                        }

                        var parser = new Windows.UI.Xaml.Media.AbbreviatedGeometryParser();

                        parser.ParseToGeometryContext(geometry, pathString, curIndex);
                    }
                }
            }
        }
    });

    /**
     * Specifies how to draw the gradient outside a gradient brush's gradient vector
     or space.
     *
     * @public
     * @class Windows.UI.Xaml.Media.GradientSpreadMethod
     */
    Bridge.define("Windows.UI.Xaml.Media.GradientSpreadMethod", {
        $kind: "enum",
        statics: {
            fields: {
                /**
                 * The color values at the ends of the gradient vector fill the remaining space.
                 *
                 * @static
                 * @public
                 * @memberof Windows.UI.Xaml.Media.GradientSpreadMethod
                 * @constant
                 * @default 0
                 * @type Windows.UI.Xaml.Media.GradientSpreadMethod
                 */
                Pad: 0,
                /**
                 * // The gradient is repeated in the reverse direction until the space is filled.
                //
                 * The gradient is repeated in the original direction until the space is filled.
                 *
                 * @static
                 * @public
                 * @memberof Windows.UI.Xaml.Media.GradientSpreadMethod
                 * @constant
                 * @default 2
                 * @type Windows.UI.Xaml.Media.GradientSpreadMethod
                 */
                Repeat: 2
            }
        }
    });

    /** @namespace Windows.UI.Xaml.Media.Imaging */

    /**
     * The {@link } namespace provides types related to
     imaging and obtaining resources and source files for bitmap images.
     *
     * @class Windows.UI.Xaml.Media.Imaging.NamespaceDoc
     */
    Bridge.define("Windows.UI.Xaml.Media.Imaging.NamespaceDoc");

    /**
     * Represents a 3x3 affine transformation matrix used for transformations in
     two-dimensional space.
     *
     * @public
     * @class Windows.UI.Xaml.Media.Matrix
     */
    Bridge.define("Windows.UI.Xaml.Media.Matrix", {
        $kind: "struct",
        statics: {
            props: {
                /**
                 * Gets an identity Matrix.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Media.Matrix
                 * @function Identity
                 * @type Windows.UI.Xaml.Media.Matrix
                 */
                Identity: {
                    get: function () {
                        return new Windows.UI.Xaml.Media.Matrix.ctor();
                    }
                }
            },
            ctors: {
                ctor: function () {
                    DotNetForHtml5.Core.TypeFromStringConverters.RegisterConverter(Windows.UI.Xaml.Media.Matrix, function (s) {
                        return Windows.UI.Xaml.Media.Matrix.Parse(s).$clone();
                    });
                }
            },
            methods: {
                Parse: function (matrixAsString) {
                    if (Bridge.referenceEquals(matrixAsString, "Identity")) {
                        return Windows.UI.Xaml.Media.Matrix.Identity.$clone();
                    }

                    var splittedString = System.String.split(matrixAsString, System.Array.init([44, 32], System.Char).map(function (i) {{ return String.fromCharCode(i); }}), null, 1);

                    if (splittedString.length === 6) {
                        var m11 = { }, m12 = { }, m21 = { }, m22 = { }, offsetX = { }, offsetY = { };
                        if (System.Double.tryParse(splittedString[System.Array.index(0, splittedString)], null, m11) && System.Double.tryParse(splittedString[System.Array.index(1, splittedString)], null, m12) && System.Double.tryParse(splittedString[System.Array.index(2, splittedString)], null, m21) && System.Double.tryParse(splittedString[System.Array.index(3, splittedString)], null, m22) && System.Double.tryParse(splittedString[System.Array.index(4, splittedString)], null, offsetX) && System.Double.tryParse(splittedString[System.Array.index(5, splittedString)], null, offsetY)) {
                            return new Windows.UI.Xaml.Media.Matrix.$ctor1(m11.v, m12.v, m21.v, m22.v, offsetX.v, offsetY.v);
                        }
                    }

                    throw new System.FormatException.$ctor1((matrixAsString || "") + " is not an eligible value for a Matrix");
                },
                Multiply: function (matrix1, matrix2) {
                    return new Windows.UI.Xaml.Media.Matrix.$ctor1((matrix1.M11 * matrix2.M11) + (matrix1.M12 * matrix2.M21), (matrix1.M11 * matrix2.M12) + (matrix1.M12 * matrix2.M22), (matrix1.M21 * matrix2.M11) + (matrix1.M22 * matrix2.M21), (matrix1.M21 * matrix2.M12) + (matrix1.M22 * matrix2.M22), ((matrix1.OffsetX * matrix2.M11) + (matrix1.OffsetY * matrix2.M21)) + matrix2.OffsetX, ((matrix1.OffsetX * matrix2.M12) + (matrix1.OffsetY * matrix2.M22)) + matrix2.OffsetY);
                }/**
                 * Determines whether the two specified Matrix structures are not identical.
                 *
                 * @static
                 * @public
                 * @this Windows.UI.Xaml.Media.Matrix
                 * @memberof Windows.UI.Xaml.Media.Matrix
                 * @param   {Windows.UI.Xaml.Media.Matrix}    matrix1    The first Matrix structure to compare.
                 * @param   {Windows.UI.Xaml.Media.Matrix}    matrix2    The second Matrix structure to compare.
                 * @return  {boolean}                                    true if matrix1 and matrix2 are not identical; otherwise, false.
                 */
                ,
                op_Inequality: function (matrix1, matrix2) {
                    return !matrix1.Equals(matrix2.$clone());
                }/**
                 * Determines whether the two specified Matrix structures are identical.
                 *
                 * @static
                 * @public
                 * @this Windows.UI.Xaml.Media.Matrix
                 * @memberof Windows.UI.Xaml.Media.Matrix
                 * @param   {Windows.UI.Xaml.Media.Matrix}    matrix1    The first Matrix structure to compare.
                 * @param   {Windows.UI.Xaml.Media.Matrix}    matrix2    The second Matrix structure to compare.
                 * @return  {boolean}                                    true if matrix1 and matrix2 are identical; otherwise, false.
                 */
                ,
                op_Equality: function (matrix1, matrix2) {
                    return matrix1.Equals(matrix2.$clone());
                },
                getDefaultValue: function () { return new Windows.UI.Xaml.Media.Matrix(); }
            }
        },
        fields: {
            _isInitialized: false,
            _m11: 0,
            _m12: 0,
            _m21: 0,
            _m22: 0,
            _offsetX: 0,
            _offsetY: 0
        },
        props: {
            /**
             * Gets a value that indicates whether this Matrix structure is an identity matrix.
             *
             * @instance
             * @public
             * @readonly
             * @memberof Windows.UI.Xaml.Media.Matrix
             * @function IsIdentity
             * @type boolean
             */
            IsIdentity: {
                get: function () {
                    return this.Equals(Windows.UI.Xaml.Media.Matrix.Identity.$clone());
                }
            },
            /**
             * Gets or sets the value of the first row and first column of this Matrix structure. The default value is 1.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Media.Matrix
             * @function M11
             * @type number
             */
            M11: {
                get: function () {
                    if (!this._isInitialized) {
                        this.Initialize();
                    }
                    return this._m11;
                },
                set: function (value) {
                    if (!this._isInitialized) {
                        this.Initialize();
                    }
                    this._m11 = value;
                }
            },
            /**
             * Gets or sets the value of the first row and second column of this Matrix structure. The default value is 0.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Media.Matrix
             * @function M12
             * @type number
             */
            M12: {
                get: function () {
                    if (!this._isInitialized) {
                        this.Initialize();
                    }
                    return this._m12;
                },
                set: function (value) {
                    if (!this._isInitialized) {
                        this.Initialize();
                    }
                    this._m12 = value;
                }
            },
            /**
             * Gets or sets the value of the second row and first column of this Matrix structure. The default value is 0.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Media.Matrix
             * @function M21
             * @type number
             */
            M21: {
                get: function () {
                    if (!this._isInitialized) {
                        this.Initialize();
                    }
                    return this._m21;
                },
                set: function (value) {
                    if (!this._isInitialized) {
                        this.Initialize();
                    }
                    this._m21 = value;
                }
            },
            /**
             * Gets or sets the value of the second row and second column of this Matrix structure. The default value is 1.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Media.Matrix
             * @function M22
             * @type number
             */
            M22: {
                get: function () {
                    if (!this._isInitialized) {
                        this.Initialize();
                    }
                    return this._m22;
                },
                set: function (value) {
                    if (!this._isInitialized) {
                        this.Initialize();
                    }
                    this._m22 = value;
                }
            },
            /**
             * Gets or sets the value of the third row and first column of this Matrix structure. The default value is 0.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Media.Matrix
             * @function OffsetX
             * @type number
             */
            OffsetX: {
                get: function () {
                    if (!this._isInitialized) {
                        this.Initialize();
                    }
                    return this._offsetX;
                },
                set: function (value) {
                    if (!this._isInitialized) {
                        this.Initialize();
                    }
                    this._offsetX = value;
                }
            },
            /**
             * Gets or sets the value of the third row and second column of this Matrix structure. The default value is 0.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Media.Matrix
             * @function OffsetY
             * @type number
             */
            OffsetY: {
                get: function () {
                    if (!this._isInitialized) {
                        this.Initialize();
                    }
                    return this._offsetY;
                },
                set: function (value) {
                    if (!this._isInitialized) {
                        this.Initialize();
                    }
                    this._offsetY = value;
                }
            }
        },
        ctors: {
            /**
             * Initializes a Matrix structure.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Media.Matrix
             * @memberof Windows.UI.Xaml.Media.Matrix
             * @param   {number}    m11        The Matrix structure's Matrix.M11 coefficient.
             * @param   {number}    m12        The Matrix structure's Matrix.M12 coefficient.
             * @param   {number}    m21        The Matrix structure's Matrix.M21 coefficient.
             * @param   {number}    m22        The Matrix structure's Matrix.M22 coefficient.
             * @param   {number}    offsetX    The Matrix structure's Matrix.OffsetX coefficient.
             * @param   {number}    offsetY    The Matrix structure's Matrix.OffsetY coefficient.
             * @return  {void}
             */
            $ctor1: function (m11, m12, m21, m22, offsetX, offsetY) {
                this.$initialize();
                this._m11 = m11;
                this._m21 = m21;
                this._m12 = m12;
                this._m22 = m22;
                this._offsetX = offsetX;
                this._offsetY = offsetY;
                this._isInitialized = true;
            },
            ctor: function () {
                this.$initialize();
            }
        },
        methods: {
            Initialize: function () {
                this._m11 = 1;
                this._m21 = 0;
                this._m12 = 0;
                this._m22 = 1;
                this._offsetX = 0;
                this._offsetY = 0;
                this._isInitialized = true;
            },
            /**
             * Determines whether the specified Matrix structure is identical to this instance.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Media.Matrix
             * @memberof Windows.UI.Xaml.Media.Matrix
             * @param   {Windows.UI.Xaml.Media.Matrix}    value    The instance of Matrix to compare to this instance.
             * @return  {boolean}                                  true if instances are equal; otherwise, false.
             */
            Equals: function (value) {
                if (!this._isInitialized) {
                    this.Initialize();
                }

                if (!value._isInitialized) {
                    value.Initialize();
                }

                return (value._m11 === this._m11 && value._m12 === this._m12 && value._m21 === this._m21 && value._m22 === this._m22 && value._offsetX === this._offsetX && value.OffsetY === this.OffsetY);
            },
            /**
             * Determines whether the specified System.Object is a Matrix structure that is identical to this Matrix.
             *
             * @instance
             * @public
             * @override
             * @this Windows.UI.Xaml.Media.Matrix
             * @memberof Windows.UI.Xaml.Media.Matrix
             * @param   {System.Object}    o    The System.Object to compare.
             * @return  {boolean}               true if o is a Matrix structure that is identical to this Matrix structure; otherwise, false.
             */
            equals: function (o) {
                if (!this._isInitialized) {
                    this.Initialize();
                }

                if (Bridge.is(o, Windows.UI.Xaml.Media.Matrix) && !System.Nullable.getValue(Bridge.cast(Bridge.unbox(o, Windows.UI.Xaml.Media.Matrix), Windows.UI.Xaml.Media.Matrix))._isInitialized) {
                    System.Nullable.getValue(Bridge.cast(Bridge.unbox(o, Windows.UI.Xaml.Media.Matrix), Windows.UI.Xaml.Media.Matrix)).Initialize();
                }

                return (Bridge.is(o, Windows.UI.Xaml.Media.Matrix) && System.Nullable.getValue(Bridge.cast(Bridge.unbox(o, Windows.UI.Xaml.Media.Matrix), Windows.UI.Xaml.Media.Matrix))._m11 === this._m11 && System.Nullable.getValue(Bridge.cast(Bridge.unbox(o, Windows.UI.Xaml.Media.Matrix), Windows.UI.Xaml.Media.Matrix))._m12 === this._m12 && System.Nullable.getValue(Bridge.cast(Bridge.unbox(o, Windows.UI.Xaml.Media.Matrix), Windows.UI.Xaml.Media.Matrix))._m21 === this._m21 && System.Nullable.getValue(Bridge.cast(Bridge.unbox(o, Windows.UI.Xaml.Media.Matrix), Windows.UI.Xaml.Media.Matrix))._m22 === this._m22 && System.Nullable.getValue(Bridge.cast(Bridge.unbox(o, Windows.UI.Xaml.Media.Matrix), Windows.UI.Xaml.Media.Matrix))._offsetX === this._offsetX && System.Nullable.getValue(Bridge.cast(Bridge.unbox(o, Windows.UI.Xaml.Media.Matrix), Windows.UI.Xaml.Media.Matrix)).OffsetY === this.OffsetY);
            },
            /**
             * Returns the hash code for this Matrix structure.
             *
             * @instance
             * @public
             * @override
             * @this Windows.UI.Xaml.Media.Matrix
             * @memberof Windows.UI.Xaml.Media.Matrix
             * @return  {number}
             */
            getHashCode: function () {
                if (!this._isInitialized) {
                    this.Initialize();
                }

                return System.Double.getHashCode(this._m11) ^ System.Double.getHashCode(this._m12) ^ System.Double.getHashCode(this._m21) ^ System.Double.getHashCode(this._m22) ^ System.Double.getHashCode(this._offsetX) ^ System.Double.getHashCode(this._offsetY);
            },
            /**
             * Creates a System.String representation of this Matrix
             structure.
             *
             * @instance
             * @public
             * @override
             * @this Windows.UI.Xaml.Media.Matrix
             * @memberof Windows.UI.Xaml.Media.Matrix
             * @return  {string}        A System.String containing the Matrix.M11, Matrix.M12,
             Matrix.M21, Matrix.M22, Matrix.OffsetX,
             and Matrix.OffsetY values of this Matrix.
             */
            toString: function () {
                if (!this._isInitialized) {
                    this.Initialize();
                }

                if (this.IsIdentity) {
                    return "Identity";
                } else {
                    return (System.Double.format(this._m11) || "") + ";" + (System.Double.format(this._m12) || "") + ";" + (System.Double.format(this._m21) || "") + ";" + (System.Double.format(this._m22) || "") + ";" + (System.Double.format(this._offsetX) || "") + ";" + (System.Double.format(this._offsetY) || "");
                }
            },
            MultiplyPoint: function (x, y) {
                var xadd = y.v * this._m21 + this._offsetX;
                var yadd = x.v * this._m12 + this._offsetY;
                x.v *= this._m11;
                x.v += xadd;
                y.v *= this._m22;
                y.v += yadd;
            },
            Transform: function (point) {
                var newPoint = point.$clone();
                this.MultiplyPoint(Bridge.ref(newPoint, "_x"), Bridge.ref(newPoint, "_y"));
                return newPoint.$clone();
            },
            $clone: function (to) {
                var s = to || new Windows.UI.Xaml.Media.Matrix();
                s._isInitialized = this._isInitialized;
                s._m11 = this._m11;
                s._m12 = this._m12;
                s._m21 = this._m21;
                s._m22 = this._m22;
                s._offsetX = this._offsetX;
                s._offsetY = this._offsetY;
                return s;
            }
        }
    });

    /**
     * Describes the likelihood that the media engine can play a media source based
     on its file type and characteristics.
     *
     * @public
     * @class Windows.UI.Xaml.Media.MediaCanPlayResponse
     */
    Bridge.define("Windows.UI.Xaml.Media.MediaCanPlayResponse", {
        $kind: "enum",
        statics: {
            fields: {
                /**
                 * Media engine cannot support the media source.
                 *
                 * @static
                 * @public
                 * @memberof Windows.UI.Xaml.Media.MediaCanPlayResponse
                 * @constant
                 * @default 0
                 * @type Windows.UI.Xaml.Media.MediaCanPlayResponse
                 */
                NotSupported: 0,
                /**
                 * Media engine might support the media source.
                 *
                 * @static
                 * @public
                 * @memberof Windows.UI.Xaml.Media.MediaCanPlayResponse
                 * @constant
                 * @default 1
                 * @type Windows.UI.Xaml.Media.MediaCanPlayResponse
                 */
                Maybe: 1,
                /**
                 * Media engine can probably support the media source.
                 *
                 * @static
                 * @public
                 * @memberof Windows.UI.Xaml.Media.MediaCanPlayResponse
                 * @constant
                 * @default 2
                 * @type Windows.UI.Xaml.Media.MediaCanPlayResponse
                 */
                Probably: 2
            }
        }
    });

    Bridge.define("Windows.UI.Xaml.Media.MIL_SEGMENT_TYPE", {
        $kind: "enum",
        statics: {
            fields: {
                MilSegmentNone: 0,
                MilSegmentLine: 1,
                MilSegmentBezier: 2,
                MilSegmentQuadraticBezier: 3,
                MilSegmentArc: 4,
                MilSegmentPolyLine: 5,
                MilSegmentPolyBezier: 6,
                MilSegmentPolyQuadraticBezier: 7,
                MIL_SEGMENT_TYPE_FORCE_DWORD: -1
            }
        }
    });

    /**
     * The {@link } namespace provides basic media support,
     graphics primitives, and brush-drawing APIs.
     *
     * @class Windows.UI.Xaml.Media.NamespaceDoc
     */
    Bridge.define("Windows.UI.Xaml.Media.NamespaceDoc");

    /**
     * PathStreamGeometryContext
     *
     * @class Windows.UI.Xaml.Media.PathStreamGeometryContext
     */
    Bridge.define("Windows.UI.Xaml.Media.PathStreamGeometryContext", {
        statics: {
            fields: {
                s_defaultFillRule: 0,
                s_defaultValueForPathFigureIsClosed: false,
                s_defaultValueForPathFigureIsFilled: false,
                s_defaultValueForPathFigureStartPoint: null,
                s_defaultValueForPathSegmentIsStroked: false,
                s_defaultValueForPathSegmentIsSmoothJoin: false,
                s_defaultValueForArcSegmentIsLargeArc: false,
                s_defaultValueForArcSegmentSweepDirection: 0,
                s_defaultValueForArcSegmentRotationAngle: 0
            },
            ctors: {
                init: function () {
                    this.s_defaultValueForPathFigureStartPoint = new Windows.Foundation.Point();
                },
                ctor: function () {

                    Windows.UI.Xaml.Media.PathStreamGeometryContext.s_defaultFillRule = Windows.UI.Xaml.Media.FillRule.EvenOdd;

                    Windows.UI.Xaml.Media.PathStreamGeometryContext.s_defaultValueForPathFigureIsClosed = false;

                    Windows.UI.Xaml.Media.PathStreamGeometryContext.s_defaultValueForPathFigureIsFilled = false;

                    Windows.UI.Xaml.Media.PathStreamGeometryContext.s_defaultValueForPathFigureStartPoint = new Windows.Foundation.Point.ctor();


                    Windows.UI.Xaml.Media.PathStreamGeometryContext.s_defaultValueForArcSegmentIsLargeArc = false;

                    Windows.UI.Xaml.Media.PathStreamGeometryContext.s_defaultValueForArcSegmentSweepDirection = Windows.UI.Xaml.Media.SweepDirection.Counterclockwise;

                    Windows.UI.Xaml.Media.PathStreamGeometryContext.s_defaultValueForArcSegmentRotationAngle = 0.0;
                }
            }
        },
        fields: {
            _pathGeometry: null,
            _figures: null,
            _currentFigure: null,
            _segments: null,
            _currentIsClosed: false,
            _currentSegmentType: 0,
            _currentSegmentPoints: null,
            _currentSegmentIsStroked: false,
            _currentSegmentIsSmoothJoin: false
        },
        ctors: {
            ctor: function () {
                this.$initialize();
                this._pathGeometry = new Windows.UI.Xaml.Media.PathGeometry.ctor();
            },
            $ctor1: function (fillRule, transform) {
                this.$initialize();
                this._pathGeometry = new Windows.UI.Xaml.Media.PathGeometry.ctor();

                if (fillRule !== Windows.UI.Xaml.Media.PathStreamGeometryContext.s_defaultFillRule) {
                    this._pathGeometry.FillRule = fillRule;
                }

            }
        },
        methods: {
            SetFigureCount: function (figureCount) {
                System.Diagnostics.Debug.Assert$1(this._figures == null, "It is illegal to call SetFigureCount multiple times or after BeginFigure.");
                System.Diagnostics.Debug.Assert(figureCount > 0);

                this._figures = new Windows.UI.Xaml.Media.PathFigureCollection.$ctor2(figureCount);
                this._pathGeometry.Figures = this._figures;
            },
            SetSegmentCount: function (segmentCount) {
                System.Diagnostics.Debug.Assert$1(this._figures != null, "It is illegal to call SetSegmentCount before BeginFigure.");
                System.Diagnostics.Debug.Assert$1(this._currentFigure != null, "It is illegal to call SetSegmentCount before BeginFigure.");
                System.Diagnostics.Debug.Assert$1(this._segments == null, "It is illegal to call SetSegmentCount multiple times per BeginFigure or after a *To method.");
                System.Diagnostics.Debug.Assert(segmentCount > 0);

                this._segments = new Windows.UI.Xaml.Media.PathSegmentCollection.$ctor2(segmentCount);
                this._currentFigure.Segments = this._segments;
            },
            /**
             * SetClosed - Sets the current closed state of the figure.
             *
             * @instance
             * @this Windows.UI.Xaml.Media.PathStreamGeometryContext
             * @memberof Windows.UI.Xaml.Media.PathStreamGeometryContext
             * @param   {boolean}    isClosed
             * @return  {void}
             */
            SetClosedState: function (isClosed) {
                System.Diagnostics.Debug.Assert(this._currentFigure != null);

                if (isClosed !== this._currentIsClosed) {
                    this._currentFigure.IsClosed = isClosed;
                    this._currentIsClosed = isClosed;
                }
            },
            /**
             * BeginFigure - Start a new figure.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Media.PathStreamGeometryContext
             * @memberof Windows.UI.Xaml.Media.PathStreamGeometryContext
             * @param   {Windows.Foundation.Point}    startPoint    
             * @param   {boolean}                     isFilled      
             * @param   {boolean}                     isClosed
             * @return  {void}
             */
            BeginFigure: function (startPoint, isFilled, isClosed) {
                System.Diagnostics.Debug.Assert(this._currentFigure == null || this._figures != null);

                if (this._currentFigure == null) {
                    if (this._figures == null) {

                        this._figures = new Windows.UI.Xaml.Media.PathFigureCollection.ctor();
                        this._pathGeometry.Figures = this._figures;
                    }
                }

                this.FinishSegment();

                this._segments = null;

                this._currentFigure = new Windows.UI.Xaml.Media.PathFigure();
                this._currentIsClosed = isClosed;

                if (Windows.Foundation.Point.op_Inequality(startPoint.$clone(), Windows.UI.Xaml.Media.PathStreamGeometryContext.s_defaultValueForPathFigureStartPoint.$clone())) {
                    this._currentFigure.StartPoint = startPoint.$clone();
                }

                if (isClosed !== Windows.UI.Xaml.Media.PathStreamGeometryContext.s_defaultValueForPathFigureIsClosed) {
                    this._currentFigure.IsClosed = isClosed;
                }

                if (isFilled !== Windows.UI.Xaml.Media.PathStreamGeometryContext.s_defaultValueForPathFigureIsFilled) {
                    this._currentFigure.IsFilled = isFilled;
                }

                this._figures.add(this._currentFigure);

                this._currentSegmentType = Windows.UI.Xaml.Media.MIL_SEGMENT_TYPE.MilSegmentNone;
            },
            /**
             * LineTo - append a LineTo to the current figure.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Media.PathStreamGeometryContext
             * @memberof Windows.UI.Xaml.Media.PathStreamGeometryContext
             * @param   {Windows.Foundation.Point}    point           
             * @param   {boolean}                     isStroked       
             * @param   {boolean}                     isSmoothJoin
             * @return  {void}
             */
            LineTo: function (point, isStroked, isSmoothJoin) {
                this.PrepareToAddPoints(1, isStroked, isSmoothJoin, Windows.UI.Xaml.Media.MIL_SEGMENT_TYPE.MilSegmentPolyLine);

                this._currentSegmentPoints.add(point.$clone());
            },
            /**
             * QuadraticBezierTo - append a QuadraticBezierTo to the current figure.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Media.PathStreamGeometryContext
             * @memberof Windows.UI.Xaml.Media.PathStreamGeometryContext
             * @param   {Windows.Foundation.Point}    point1          
             * @param   {Windows.Foundation.Point}    point2          
             * @param   {boolean}                     isStroked       
             * @param   {boolean}                     isSmoothJoin
             * @return  {void}
             */
            QuadraticBezierTo: function (point1, point2, isStroked, isSmoothJoin) {
                this.PrepareToAddPoints(2, isStroked, isSmoothJoin, Windows.UI.Xaml.Media.MIL_SEGMENT_TYPE.MilSegmentPolyQuadraticBezier);

                this._currentSegmentPoints.add(point1.$clone());
                this._currentSegmentPoints.add(point2.$clone());
            },
            /**
             * BezierTo - apply a BezierTo to the current figure.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Media.PathStreamGeometryContext
             * @memberof Windows.UI.Xaml.Media.PathStreamGeometryContext
             * @param   {Windows.Foundation.Point}    point1          
             * @param   {Windows.Foundation.Point}    point2          
             * @param   {Windows.Foundation.Point}    point3          
             * @param   {boolean}                     isStroked       
             * @param   {boolean}                     isSmoothJoin
             * @return  {void}
             */
            BezierTo: function (point1, point2, point3, isStroked, isSmoothJoin) {
                this.PrepareToAddPoints(3, isStroked, isSmoothJoin, Windows.UI.Xaml.Media.MIL_SEGMENT_TYPE.MilSegmentPolyBezier);

                this._currentSegmentPoints.add(point1.$clone());
                this._currentSegmentPoints.add(point2.$clone());
                this._currentSegmentPoints.add(point3.$clone());
            },
            /**
             * PolyLineTo - append a PolyLineTo to the current figure.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Media.PathStreamGeometryContext
             * @memberof Windows.UI.Xaml.Media.PathStreamGeometryContext
             * @param   {System.Collections.Generic.IList$1}    points          
             * @param   {boolean}                               isStroked       
             * @param   {boolean}                               isSmoothJoin
             * @return  {void}
             */
            PolyLineTo: function (points, isStroked, isSmoothJoin) {
                this.GenericPolyTo(points, isStroked, isSmoothJoin, Windows.UI.Xaml.Media.MIL_SEGMENT_TYPE.MilSegmentPolyLine);
            },
            /**
             * PolyQuadraticBezierTo - append a PolyQuadraticBezierTo to the current figure.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Media.PathStreamGeometryContext
             * @memberof Windows.UI.Xaml.Media.PathStreamGeometryContext
             * @param   {System.Collections.Generic.IList$1}    points          
             * @param   {boolean}                               isStroked       
             * @param   {boolean}                               isSmoothJoin
             * @return  {void}
             */
            PolyQuadraticBezierTo: function (points, isStroked, isSmoothJoin) {
                this.GenericPolyTo(points, isStroked, isSmoothJoin, Windows.UI.Xaml.Media.MIL_SEGMENT_TYPE.MilSegmentPolyQuadraticBezier);
            },
            /**
             * PolyBezierTo - append a PolyBezierTo to the current figure.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Media.PathStreamGeometryContext
             * @memberof Windows.UI.Xaml.Media.PathStreamGeometryContext
             * @param   {System.Collections.Generic.IList$1}    points          
             * @param   {boolean}                               isStroked       
             * @param   {boolean}                               isSmoothJoin
             * @return  {void}
             */
            PolyBezierTo: function (points, isStroked, isSmoothJoin) {
                this.GenericPolyTo(points, isStroked, isSmoothJoin, Windows.UI.Xaml.Media.MIL_SEGMENT_TYPE.MilSegmentPolyBezier);
            },
            /**
             * ArcTo - append an ArcTo to the current figure.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Media.PathStreamGeometryContext
             * @memberof Windows.UI.Xaml.Media.PathStreamGeometryContext
             * @param   {Windows.Foundation.Point}                point             
             * @param   {Windows.Foundation.Size}                 size              
             * @param   {number}                                  rotationAngle     
             * @param   {boolean}                                 isLargeArc        
             * @param   {Windows.UI.Xaml.Media.SweepDirection}    sweepDirection    
             * @param   {boolean}                                 isStroked         
             * @param   {boolean}                                 isSmoothJoin
             * @return  {void}
             */
            ArcTo: function (point, size, rotationAngle, isLargeArc, sweepDirection, isStroked, isSmoothJoin) {
                System.Diagnostics.Debug.Assert(this._figures != null);
                System.Diagnostics.Debug.Assert(this._currentFigure != null);

                this.FinishSegment();

                if (this._segments == null) {

                    this._segments = new Windows.UI.Xaml.Media.PathSegmentCollection.ctor();
                    this._currentFigure.Segments = this._segments;
                }

                var segment = new Windows.UI.Xaml.Media.ArcSegment();

                segment.Point = point.$clone();
                segment.Size = size.$clone();

                if (isLargeArc !== Windows.UI.Xaml.Media.PathStreamGeometryContext.s_defaultValueForArcSegmentIsLargeArc) {
                    segment.IsLargeArc = isLargeArc;
                }

                if (sweepDirection !== Windows.UI.Xaml.Media.PathStreamGeometryContext.s_defaultValueForArcSegmentSweepDirection) {
                    segment.SweepDirection = sweepDirection;
                }

                if (rotationAngle !== Windows.UI.Xaml.Media.PathStreamGeometryContext.s_defaultValueForArcSegmentRotationAngle) {
                    segment.RotationAngle = rotationAngle;
                }



                this._segments.add(segment);

                this._currentSegmentType = Windows.UI.Xaml.Media.MIL_SEGMENT_TYPE.MilSegmentArc;
            },
            /**
             * PathStreamGeometryContext is never opened, so it shouldn't be closed.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Media.PathStreamGeometryContext
             * @memberof Windows.UI.Xaml.Media.PathStreamGeometryContext
             * @return  {void}
             */
            Close: function () {
                System.Diagnostics.Debug.Assert(false);
            },
            /**
             * GetPathGeometry - Retrieves the PathGeometry built by this Context.
             *
             * @instance
             * @this Windows.UI.Xaml.Media.PathStreamGeometryContext
             * @memberof Windows.UI.Xaml.Media.PathStreamGeometryContext
             * @return  {Windows.UI.Xaml.Media.PathGeometry}
             */
            GetPathGeometry: function () {
                this.FinishSegment();

                System.Diagnostics.Debug.Assert(this._currentSegmentPoints == null);

                return this._pathGeometry;
            },
            GenericPolyTo: function (points, isStroked, isSmoothJoin, segmentType) {
                System.Diagnostics.Debug.Assert(points != null);

                var count = System.Array.getCount(points, Windows.Foundation.Point);
                this.PrepareToAddPoints(count, isStroked, isSmoothJoin, segmentType);

                for (var i = 0; i < count; i = (i + 1) | 0) {
                    this._currentSegmentPoints.add(System.Array.getItem(points, i, Windows.Foundation.Point).$clone());
                }
            },
            PrepareToAddPoints: function (count, isStroked, isSmoothJoin, segmentType) {
                System.Diagnostics.Debug.Assert(this._figures != null);
                System.Diagnostics.Debug.Assert(this._currentFigure != null);

                System.Diagnostics.Debug.Assert(count !== 0);

                if (this._currentSegmentType !== segmentType || this._currentSegmentIsStroked !== isStroked || this._currentSegmentIsSmoothJoin !== isSmoothJoin) {
                    this.FinishSegment();

                    this._currentSegmentType = segmentType;
                    this._currentSegmentIsStroked = isStroked;
                    this._currentSegmentIsSmoothJoin = isSmoothJoin;
                }

                if (this._currentSegmentPoints == null) {
                    this._currentSegmentPoints = new Windows.UI.Xaml.Media.PointCollection.ctor();
                }
            },
            /**
             * FinishSegment - called to completed any outstanding Segment which may be present.
             *
             * @instance
             * @private
             * @this Windows.UI.Xaml.Media.PathStreamGeometryContext
             * @memberof Windows.UI.Xaml.Media.PathStreamGeometryContext
             * @return  {void}
             */
            FinishSegment: function () {
                if (this._currentSegmentPoints != null) {
                    System.Diagnostics.Debug.Assert(this._currentFigure != null);

                    var count = this._currentSegmentPoints.Count;

                    System.Diagnostics.Debug.Assert(count > 0);

                    if (this._segments == null) {

                        this._segments = new Windows.UI.Xaml.Media.PathSegmentCollection.ctor();
                        this._currentFigure.Segments = this._segments;
                    }

                    var segment;

                    switch (this._currentSegmentType) {
                        case Windows.UI.Xaml.Media.MIL_SEGMENT_TYPE.MilSegmentPolyLine: 
                            if (count === 1) {
                                var lSegment = new Windows.UI.Xaml.Media.LineSegment();
                                lSegment.Point = this._currentSegmentPoints.getItem(0).$clone();
                                segment = lSegment;
                            } else {
                                var pSegment = new Windows.UI.Xaml.Media.PolyLineSegment();
                                pSegment.Points = this._currentSegmentPoints;
                                segment = pSegment;
                            }
                            break;
                        case Windows.UI.Xaml.Media.MIL_SEGMENT_TYPE.MilSegmentPolyBezier: 
                            if (count === 3) {
                                var bSegment = new Windows.UI.Xaml.Media.BezierSegment();
                                bSegment.Point1 = this._currentSegmentPoints.getItem(0).$clone();
                                bSegment.Point2 = this._currentSegmentPoints.getItem(1).$clone();
                                bSegment.Point3 = this._currentSegmentPoints.getItem(2).$clone();
                                segment = bSegment;
                            } else {
                                System.Diagnostics.Debug.Assert(count % 3 === 0);

                                var pSegment1 = new Windows.UI.Xaml.Media.PolyBezierSegment();
                                pSegment1.Points = this._currentSegmentPoints;
                                segment = pSegment1;
                            }
                            break;
                        case Windows.UI.Xaml.Media.MIL_SEGMENT_TYPE.MilSegmentPolyQuadraticBezier: 
                            if (count === 2) {
                                var qSegment = new Windows.UI.Xaml.Media.QuadraticBezierSegment();
                                qSegment.Point1 = this._currentSegmentPoints.getItem(0).$clone();
                                qSegment.Point2 = this._currentSegmentPoints.getItem(1).$clone();
                                segment = qSegment;
                            } else {
                                System.Diagnostics.Debug.Assert(count % 2 === 0);

                                var pSegment2 = new Windows.UI.Xaml.Media.PolyQuadraticBezierSegment();
                                pSegment2.Points = this._currentSegmentPoints;
                                segment = pSegment2;
                            }
                            break;
                        default: 
                            segment = null;
                            System.Diagnostics.Debug.Assert(false);
                            break;
                    }



                    this._segments.add(segment);

                    this._currentSegmentPoints = null;
                    this._currentSegmentType = Windows.UI.Xaml.Media.MIL_SEGMENT_TYPE.MilSegmentNone;
                }
            }
        }
    });

    /**
     * Describes the shape at the end of a line or segment.
     *
     * @public
     * @class Windows.UI.Xaml.Media.PenLineCap
     */
    Bridge.define("Windows.UI.Xaml.Media.PenLineCap", {
        $kind: "enum",
        statics: {
            fields: {
                /**
                 * A cap that does not extend past the last point of the line. Comparable to
                 no line cap.
                 *
                 * @static
                 * @public
                 * @memberof Windows.UI.Xaml.Media.PenLineCap
                 * @constant
                 * @default 0
                 * @type Windows.UI.Xaml.Media.PenLineCap
                 */
                Flat: 0,
                /**
                 * A rectangle that has a height equal to the line thickness and a length equal
                 to half the line thickness.
                 *
                 * @static
                 * @public
                 * @memberof Windows.UI.Xaml.Media.PenLineCap
                 * @constant
                 * @default 1
                 * @type Windows.UI.Xaml.Media.PenLineCap
                 */
                Square: 1,
                /**
                 * A semicircle that has a diameter equal to the line thickness.
                 *
                 * @static
                 * @public
                 * @memberof Windows.UI.Xaml.Media.PenLineCap
                 * @constant
                 * @default 2
                 * @type Windows.UI.Xaml.Media.PenLineCap
                 */
                Round: 2
            }
        }
    });

    /**
     * Describes the shape that joins two lines or segments.
     *
     * @public
     * @class Windows.UI.Xaml.Media.PenLineJoin
     */
    Bridge.define("Windows.UI.Xaml.Media.PenLineJoin", {
        $kind: "enum",
        statics: {
            fields: {
                /**
                 * Line joins use regular angular vertices.
                 *
                 * @static
                 * @public
                 * @memberof Windows.UI.Xaml.Media.PenLineJoin
                 * @constant
                 * @default 0
                 * @type Windows.UI.Xaml.Media.PenLineJoin
                 */
                Miter: 0,
                /**
                 * Line joins use beveled vertices.
                 *
                 * @static
                 * @public
                 * @memberof Windows.UI.Xaml.Media.PenLineJoin
                 * @constant
                 * @default 1
                 * @type Windows.UI.Xaml.Media.PenLineJoin
                 */
                Bevel: 1,
                /**
                 * Line joins use rounded vertices.
                 *
                 * @static
                 * @public
                 * @memberof Windows.UI.Xaml.Media.PenLineJoin
                 * @constant
                 * @default 2
                 * @type Windows.UI.Xaml.Media.PenLineJoin
                 */
                Round: 2
            }
        }
    });

    /**
     * Describes how content is resized to fill its allocated space.
     *
     * @public
     * @class Windows.UI.Xaml.Media.Stretch
     */
    Bridge.define("Windows.UI.Xaml.Media.Stretch", {
        $kind: "enum",
        statics: {
            fields: {
                /**
                 * The content preserves its original size.
                 *
                 * @static
                 * @public
                 * @memberof Windows.UI.Xaml.Media.Stretch
                 * @constant
                 * @default 0
                 * @type Windows.UI.Xaml.Media.Stretch
                 */
                None: 0,
                /**
                 * The content is resized to fill the destination dimensions. The aspect ratio is not preserved.
                 *
                 * @static
                 * @public
                 * @memberof Windows.UI.Xaml.Media.Stretch
                 * @constant
                 * @default 1
                 * @type Windows.UI.Xaml.Media.Stretch
                 */
                Fill: 1,
                /**
                 * The content is resized to fit in the destination dimensions while it preserves its native aspect ratio.
                 *
                 * @static
                 * @public
                 * @memberof Windows.UI.Xaml.Media.Stretch
                 * @constant
                 * @default 2
                 * @type Windows.UI.Xaml.Media.Stretch
                 */
                Uniform: 2,
                /**
                 * The content is resized to fill the destination dimensions while it preserves
                 its native aspect ratio. If the aspect ratio of the destination rectangle
                 differs from the source, the source content is clipped to fit in the destination
                 dimensions.
                 *
                 * @static
                 * @public
                 * @memberof Windows.UI.Xaml.Media.Stretch
                 * @constant
                 * @default 3
                 * @type Windows.UI.Xaml.Media.Stretch
                 */
                UniformToFill: 3
            }
        }
    });

    /**
     * Specifies the direction in which an elliptical arc is drawn.
     *
     * @public
     * @class Windows.UI.Xaml.Media.SweepDirection
     */
    Bridge.define("Windows.UI.Xaml.Media.SweepDirection", {
        $kind: "enum",
        statics: {
            fields: {
                /**
                 * Arcs are drawn in a counterclockwise (negative-angle) direction.
                 *
                 * @static
                 * @public
                 * @memberof Windows.UI.Xaml.Media.SweepDirection
                 * @constant
                 * @default 0
                 * @type Windows.UI.Xaml.Media.SweepDirection
                 */
                Counterclockwise: 0,
                /**
                 * Arcs are drawn in a clockwise (positive-angle) direction.
                 *
                 * @static
                 * @public
                 * @memberof Windows.UI.Xaml.Media.SweepDirection
                 * @constant
                 * @default 1
                 * @type Windows.UI.Xaml.Media.SweepDirection
                 */
                Clockwise: 1
            }
        }
    });

    /**
     * Provides utility methods that can used to traverse object relationships (along
     child object or parent object axes) in the visual tree.
     *
     * @public
     * @class Windows.UI.Xaml.Media.VisualTreeHelper
     */
    Bridge.define("Windows.UI.Xaml.Media.VisualTreeHelper", {
        statics: {
            methods: {
                /**
                 * Returns an object's parent object in the visual tree.
                 *
                 * @static
                 * @public
                 * @this Windows.UI.Xaml.Media.VisualTreeHelper
                 * @memberof Windows.UI.Xaml.Media.VisualTreeHelper
                 * @param   {Windows.UI.Xaml.DependencyObject}    reference    The object for which to get the parent object.
                 * @return  {Windows.UI.Xaml.DependencyObject}                 The parent object of the reference object in the visual tree.
                 */
                GetParent: function (reference) {
                    if (Bridge.is(reference, Windows.UI.Xaml.UIElement)) {
                        return Bridge.cast(reference, Windows.UI.Xaml.UIElement).INTERNAL_VisualParent;
                    } else {
                        return null;
                    }
                },
                /**
                 * Returns the number of children that exist in an object's child collection in the visual tree.
                 *
                 * @static
                 * @public
                 * @this Windows.UI.Xaml.Media.VisualTreeHelper
                 * @memberof Windows.UI.Xaml.Media.VisualTreeHelper
                 * @param   {Windows.UI.Xaml.DependencyObject}    reference    The source visual.
                 * @return  {number}                                           The number of visual children for the provided source visual.
                 */
                GetChildrenCount: function (reference) {
                    var children = { };

                    if (Bridge.is(reference, Windows.UI.Xaml.UIElement)) {

                        if (Bridge.cast(reference, Windows.UI.Xaml.UIElement)._isLoaded) {
                            var visualChildrenInformation = Bridge.cast(reference, Windows.UI.Xaml.UIElement).INTERNAL_VisualChildrenInformation;
                            if (visualChildrenInformation != null) {
                                return visualChildrenInformation.Count;
                            } else {
                                return 0;
                            }
                        } else if (Windows.UI.Xaml.Media.VisualTreeHelper.TryGetChildrenUsingContentProperty(reference, children)) {
                            return children.v.Count;
                        } else {
                            return 0;
                        }
                    } else {
                        throw new System.InvalidOperationException.$ctor1("The argument is null, or is not a valid UIElement.");
                    }
                },
                /**
                 * Using the provided index, obtains a specific child object of the provided object by examining the visual tree. Please note that the current implementation does not guarantee that the index corresponds to the visual order of the controls.
                 *
                 * @static
                 * @public
                 * @this Windows.UI.Xaml.Media.VisualTreeHelper
                 * @memberof Windows.UI.Xaml.Media.VisualTreeHelper
                 * @param   {Windows.UI.Xaml.DependencyObject}    reference     The object that holds the child collection.
                 * @param   {number}                              childIndex    The index of the requested child object in the reference child collection. Please note that the current implementation does not guarantee that the index corresponds to the visual order of the controls.
                 * @return  {Windows.UI.Xaml.DependencyObject}                  The child object as referenced by childIndex.
                 */
                GetChild: function (reference, childIndex) {


                    var children = { };

                    if (Bridge.is(reference, Windows.UI.Xaml.UIElement)) {

                        if (Bridge.cast(reference, Windows.UI.Xaml.UIElement)._isLoaded) {
                            var visualChildrenInformation = Bridge.cast(reference, Windows.UI.Xaml.UIElement).INTERNAL_VisualChildrenInformation;
                            if (visualChildrenInformation != null) {
                                var keys = System.Linq.Enumerable.from(visualChildrenInformation.Keys).ToArray();


                                if (childIndex >= keys.length) {
                                    throw new System.Exception(System.String.format("The index '{0}' is out of bounds.", [Bridge.toString(childIndex)]));
                                }

                                return visualChildrenInformation.getItem(keys[System.Array.index(childIndex, keys)]).INTERNAL_UIElement;
                            } else {
                                throw new System.Exception(System.String.format("The index '{0}' is out of bounds.", [Bridge.toString(childIndex)]));
                            }
                        } else if (Windows.UI.Xaml.Media.VisualTreeHelper.TryGetChildrenUsingContentProperty(reference, children)) {
                            if (childIndex >= children.v.Count) {
                                throw new System.Exception(System.String.format("The index '{0}' is out of bounds.", [Bridge.toString(childIndex)]));
                            }

                            return children.v.getItem(childIndex);
                        } else {
                            throw new System.Exception(System.String.format("The index '{0}' is out of bounds.", [Bridge.toString(childIndex)]));
                        }
                    } else {
                        throw new System.InvalidOperationException.$ctor1("The argument is null, or is not a valid UIElement.");
                    }
                },
                /**
                 * Retrieves an object that is located within a specified point of an object's coordinate space.
                 *
                 * @static
                 * @this Windows.UI.Xaml.Media.VisualTreeHelper
                 * @memberof Windows.UI.Xaml.Media.VisualTreeHelper
                 * @param   {Windows.Foundation.Point}     intersectingPoint    The point to use as the determination point.
                 * @return  {Windows.UI.Xaml.UIElement}                         The UIElement object that is determined to be located
                 in the visual tree composition at the specified point.
                 */
                FindElementInHostCoordinates: function (intersectingPoint) {
                    return CSHTML5.Internal.INTERNAL_HtmlDomManager.FindElementInHostCoordinates_UsedBySimulatorToo(intersectingPoint.X, intersectingPoint.Y);
                },
                /**
                 * Retrieves a set of objects that are located within a specified point of an object's coordinate space. Note: the current implementation only returns the top-most element.
                 *
                 * @static
                 * @public
                 * @this Windows.UI.Xaml.Media.VisualTreeHelper
                 * @memberof Windows.UI.Xaml.Media.VisualTreeHelper
                 * @param   {Windows.Foundation.Point}                    intersectingPoint    The point to use as the determination point.
                 * @param   {Windows.UI.Xaml.UIElement}                   subtree              The object to search within.
                 * @return  {System.Collections.Generic.IEnumerable$1}                         An enumerable set of UIElement objects that are determined to be located
                 in the visual tree composition at the specified point and within the specified
                 subtree.  Note: the current implementation only returns the top-most element.
                 */
                FindElementsInHostCoordinates: function (intersectingPoint, subtree) {
                    var elementAtCoordinates = Windows.UI.Xaml.Media.VisualTreeHelper.FindElementInHostCoordinates(intersectingPoint.$clone());

                    if (elementAtCoordinates != null) {
                        if (subtree == null) {
                            return function (_o1) {
                                    _o1.add(elementAtCoordinates);
                                    return _o1;
                                }(new (System.Collections.Generic.List$1(Windows.UI.Xaml.UIElement)).ctor());
                        } else {

                            var current = elementAtCoordinates;
                            while (current != null) {
                                if (Bridge.referenceEquals(current, subtree)) {
                                    break;
                                } else {
                                    current = Bridge.as(current.INTERNAL_VisualParent, Windows.UI.Xaml.UIElement);
                                }
                            }

                            var elementIsInSubtree = (current != null);

                            if (elementIsInSubtree) {
                                return function (_o2) {
                                        _o2.add(elementAtCoordinates);
                                        return _o2;
                                    }(new (System.Collections.Generic.List$1(Windows.UI.Xaml.UIElement)).ctor());
                            } else {
                                return new (System.Collections.Generic.List$1(Windows.UI.Xaml.UIElement)).ctor();
                            }
                        }
                    } else {
                        return new (System.Collections.Generic.List$1(Windows.UI.Xaml.UIElement)).ctor();
                    }
                },
                TryGetChildrenUsingContentProperty: function (depObj, children) {
                    var $t;
                    var depObjType = Bridge.getType(depObj);


                    var contentPropertyAttribute = Bridge.cast(Windows.UI.Xaml.Media.VisualTreeHelper.GetCustomAttribute(Bridge.Reflection.getAttributes(depObjType, null, true), System.Windows.Markup.ContentPropertyAttribute), System.Windows.Markup.ContentPropertyAttribute);

                    if (contentPropertyAttribute != null) {
                        var contentPropertyName = contentPropertyAttribute.Name;

                        var propertyInfo = Bridge.Reflection.getMembers(depObjType, 16, 284, contentPropertyName);

                        if (propertyInfo != null && (!!propertyInfo.g) && propertyInfo.g != null) {
                            var value = Bridge.Reflection.midel(propertyInfo.g, depObj)();

                            children.v = new (System.Collections.Generic.List$1(Windows.UI.Xaml.DependencyObject)).ctor();

                            if (Bridge.is(value, Windows.UI.Xaml.DependencyObject)) {
                                children.v.add(Bridge.cast(value, Windows.UI.Xaml.DependencyObject));
                            } else if (Bridge.is(value, System.Collections.IEnumerable) && (!(Bridge.is(value, System.String)))) {
                                $t = Bridge.getEnumerator(Bridge.cast(value, System.Collections.IEnumerable));
                                try {
                                    while ($t.moveNext()) {
                                        var item = $t.Current;
                                        if (!(Bridge.is(item, Windows.UI.Xaml.DependencyObject))) {
                                            children.v = null;

                                            return false;
                                        } else {
                                            children.v.add(Bridge.cast(item, Windows.UI.Xaml.DependencyObject));
                                        }
                                    }
                                } finally {
                                    if (Bridge.is($t, System.IDisposable)) {
                                        $t.System$IDisposable$Dispose();
                                    }
                                }
                            }

                            return true;
                        } else {
                            children.v = null;
                            return false;
                        }
                    } else {
                        children.v = null;
                        return false;
                    }
                },
                GetCustomAttribute: function (arrayOfAttribute, typeWanted) {
                    var k = 0;

                    while (k < arrayOfAttribute.length) {
                        if (Bridge.referenceEquals(Bridge.getType(arrayOfAttribute[System.Array.index(k, arrayOfAttribute)]), typeWanted)) {
                            return Bridge.cast(arrayOfAttribute[System.Array.index(k, arrayOfAttribute)], System.Attribute);
                        }
                        k = (k + 1) | 0;
                    }
                    return null;
                }
            }
        }
    });

    /**
     * Displays a message box.
     *
     * @static
     * @abstract
     * @public
     * @class Windows.UI.Xaml.MessageBox
     */
    Bridge.define("Windows.UI.Xaml.MessageBox", {
        statics: {
            fields: {
                _codeToShowTheMessageBoxWithTitleAndButtons: null
            },
            props: {
                /**
                 * Intended to be called by the Simulator to inject the code to display a MessageBox with a title and button.
                 *
                 * @static
                 * @public
                 * @memberof Windows.UI.Xaml.MessageBox
                 * @function INTERNAL_CodeToShowTheMessageBoxWithTitleAndButtons
                 * @type System.Func
                 */
                INTERNAL_CodeToShowTheMessageBoxWithTitleAndButtons: {
                    set: function (value) {
                        Windows.UI.Xaml.MessageBox._codeToShowTheMessageBoxWithTitleAndButtons = value;
                    }
                }
            },
            methods: {
                /**
                 * Displays a message box that contains the specified text and an OK button.
                 *
                 * @static
                 * @public
                 * @this Windows.UI.Xaml.MessageBox
                 * @memberof Windows.UI.Xaml.MessageBox
                 * @param   {string}                              msg    A System.String that specifies the text to display.
                 * @return  {Windows.UI.Xaml.MessageBoxResult}           System.Windows.MessageBoxResult.OK in all cases.
                 */
                Show: function (msg) {
                    return Windows.UI.Xaml.MessageBox.Show$2(msg, "", Windows.UI.Xaml.MessageBoxButton.OK);
                },
                /**
                 * Displays a message box that contains the specified text, title bar caption, and response buttons.
                 *
                 * @static
                 * @public
                 * @this Windows.UI.Xaml.MessageBox
                 * @memberof Windows.UI.Xaml.MessageBox
                 * @param   {string}                              messageBoxText    The message to display.
                 * @param   {Windows.UI.Xaml.MessageBoxButton}    button            A value that indicates the button or buttons to display.
                 * @return  {Windows.UI.Xaml.MessageBoxResult}                      A value that indicates the user's response to the message.
                 */
                Show$3: function (messageBoxText, button) {
                    return Windows.UI.Xaml.MessageBox.Show$2(messageBoxText, "", button);
                },
                /**
                 * Displays a message box that contains the specified text, title bar caption, and response buttons.
                 *
                 * @static
                 * @public
                 * @this Windows.UI.Xaml.MessageBox
                 * @memberof Windows.UI.Xaml.MessageBox
                 * @param   {string}                              messageBoxText    The message to display.
                 * @param   {string}                              caption           The title of the message box.
                 * @return  {Windows.UI.Xaml.MessageBoxResult}                      System.Windows.MessageBoxResult.OK in all cases.
                 */
                Show$1: function (messageBoxText, caption) {
                    return Windows.UI.Xaml.MessageBox.Show$2(messageBoxText, caption, Windows.UI.Xaml.MessageBoxButton.OK);
                },
                /**
                 * Displays a message box that contains the specified text, title bar caption, and response buttons.
                 *
                 * @static
                 * @public
                 * @this Windows.UI.Xaml.MessageBox
                 * @memberof Windows.UI.Xaml.MessageBox
                 * @param   {string}                              messageBoxText    The message to display.
                 * @param   {string}                              caption           The title of the message box.
                 * @param   {Windows.UI.Xaml.MessageBoxButton}    button            A value that indicates the button or buttons to display.
                 * @return  {Windows.UI.Xaml.MessageBoxResult}                      A value that indicates the user's response to the message.
                 */
                Show$2: function (messageBoxText, caption, button) {
                    var $t;


                    if (Windows.UI.Xaml.Input.Pointer.INTERNAL_captured != null) {
                        Windows.UI.Xaml.Input.Pointer.INTERNAL_captured.ReleasePointerCapture();
                    }




                    if (!Bridge.staticEquals(Windows.UI.Xaml.MessageBox._codeToShowTheMessageBoxWithTitleAndButtons, null)) {

                        var showCancelButton = button === Windows.UI.Xaml.MessageBoxButton.OKCancel;
                        var isOK = Windows.UI.Xaml.MessageBox._codeToShowTheMessageBoxWithTitleAndButtons(messageBoxText, ($t = caption, $t != null ? $t : ""), showCancelButton);
                        return (isOK ? Windows.UI.Xaml.MessageBoxResult.OK : Windows.UI.Xaml.MessageBoxResult.Cancel);
                    } else {

                        var messageIncludingTitleIfAny;

                        if (!System.String.isNullOrEmpty(caption)) {
                            messageIncludingTitleIfAny = (caption.toUpperCase() || "") + ("\n" || "") + ("\n" || "") + (messageBoxText || "") + ("\n" || "");
                        } else {
                            messageIncludingTitleIfAny = ("\n" || "") + (messageBoxText || "") + ("\n" || "");
                        }

                        var isOK1 = true;
                        if (button === Windows.UI.Xaml.MessageBoxButton.OK) {
alert(                            messageIncludingTitleIfAny);
                        } else if (button === Windows.UI.Xaml.MessageBoxButton.OKCancel) {
                            isOK1 = System.Nullable.getValue(Bridge.cast(Bridge.unbox(confirm(messageIncludingTitleIfAny), System.Boolean), System.Boolean));
                        } else {
                            throw new System.NotSupportedException.$ctor1("Buttons other than OK and Cancel are not supported in the MessageBox.");
                        }
                        return (isOK1 ? Windows.UI.Xaml.MessageBoxResult.OK : Windows.UI.Xaml.MessageBoxResult.Cancel);
                    }
                }
            }
        }
    });

    /**
     * Specifies the buttons to include when you display a message box.
     *
     * @public
     * @class Windows.UI.Xaml.MessageBoxButton
     */
    Bridge.define("Windows.UI.Xaml.MessageBoxButton", {
        $kind: "enum",
        statics: {
            fields: {
                /**
                 * Displays only the OK button.
                 *
                 * @static
                 * @public
                 * @memberof Windows.UI.Xaml.MessageBoxButton
                 * @constant
                 * @default 0
                 * @type Windows.UI.Xaml.MessageBoxButton
                 */
                OK: 0,
                /**
                 * Displays both the OK and Cancel buttons.
                 *
                 * @static
                 * @public
                 * @memberof Windows.UI.Xaml.MessageBoxButton
                 * @constant
                 * @default 1
                 * @type Windows.UI.Xaml.MessageBoxButton
                 */
                OKCancel: 1
            }
        }
    });

    /**
     * Represents a user's response to a message box.
     *
     * @public
     * @class Windows.UI.Xaml.MessageBoxResult
     */
    Bridge.define("Windows.UI.Xaml.MessageBoxResult", {
        $kind: "enum",
        statics: {
            fields: {
                /**
                 * This value is not currently used.
                 *
                 * @static
                 * @public
                 * @memberof Windows.UI.Xaml.MessageBoxResult
                 * @constant
                 * @default 0
                 * @type Windows.UI.Xaml.MessageBoxResult
                 */
                None: 0,
                /**
                 * The user clicked the OK button.
                 *
                 * @static
                 * @public
                 * @memberof Windows.UI.Xaml.MessageBoxResult
                 * @constant
                 * @default 1
                 * @type Windows.UI.Xaml.MessageBoxResult
                 */
                OK: 1,
                /**
                 * The user clicked the Cancel button or pressed ESC.
                 *
                 * @static
                 * @public
                 * @memberof Windows.UI.Xaml.MessageBoxResult
                 * @constant
                 * @default 2
                 * @type Windows.UI.Xaml.MessageBoxResult
                 */
                Cancel: 2
            }
        }
    });

    /**
     * The {@link } namespace provides general framework APIs and
     application model APIs, and a variety of support classes that are commonly used
     by many different feature areas.
     *
     * @class Windows.UI.Xaml.NamespaceDoc
     */
    Bridge.define("Windows.UI.Xaml.NamespaceDoc");

    /** @namespace Windows.UI.Xaml.Navigation */

    /**
     * Specifies the type of journal used by the frame.
     *
     * @public
     * @class Windows.UI.Xaml.Navigation.JournalOwnership
     */
    Bridge.define("Windows.UI.Xaml.Navigation.JournalOwnership", {
        $kind: "enum",
        statics: {
            fields: {
                /**
                 * If the System.Windows.Controls.Frame control is a top-level frame, it integrates
                 with the browser journal; otherwise, it maintains its own journal.
                 *
                 * @static
                 * @public
                 * @memberof Windows.UI.Xaml.Navigation.JournalOwnership
                 * @constant
                 * @default 0
                 * @type Windows.UI.Xaml.Navigation.JournalOwnership
                 */
                Automatic: 0,
                /**
                 * The System.Windows.Controls.Frame maintains its own journal. This option
                 can be used with any System.Windows.Controls.Frame.
                 *
                 * @static
                 * @public
                 * @memberof Windows.UI.Xaml.Navigation.JournalOwnership
                 * @constant
                 * @default 1
                 * @type Windows.UI.Xaml.Navigation.JournalOwnership
                 */
                OwnsJournal: 1,
                /**
                 * The System.Windows.Controls.Frame integrates with the browser journal. This
                 option can be used only with a top-level System.Windows.Controls.Frame; otherwise,
                 an exception is thrown.
                 *
                 * @static
                 * @public
                 * @memberof Windows.UI.Xaml.Navigation.JournalOwnership
                 * @constant
                 * @default 2
                 * @type Windows.UI.Xaml.Navigation.JournalOwnership
                 */
                UsesParentJournal: 2
            }
        }
    });

    /**
     * The {@link } namespace provides types that support navigation events as initiated by the Page and Frame classes.
     *
     * @class Windows.UI.Xaml.Navigation.NamespaceDoc
     */
    Bridge.define("Windows.UI.Xaml.Navigation.NamespaceDoc");

    /**
     * Provides data for navigation methods and event handlers that cannot cancel
     the navigation request.
     * // Initializes a new instance of the System.Windows.Navigation.NavigationEventArgs
    // class based on content, URI, navigation type, and whether the navigation
    // is internal to the current application.
    //
     * // Gets a value that indicates whether the current application is the origin
    // and destination of the navigation.
    //
     * // Gets a value that indicates whether the navigation is forward, back, or a
    // new navigation.
    //
     *
     * @public
     * @class Windows.UI.Xaml.Navigation.NavigationEventArgs
     * @augments System.Object
     */
    Bridge.define("Windows.UI.Xaml.Navigation.NavigationEventArgs", {
        fields: {
            /**
             * Gets the content of the target being navigated to.
             *
             * @instance
             * @private
             * @memberof Windows.UI.Xaml.Navigation.NavigationEventArgs
             * @type System.Object
             */
            _content: null,
            /**
             * Gets the uniform resource identifier (URI) of the target.
             *
             * @instance
             * @private
             * @memberof Windows.UI.Xaml.Navigation.NavigationEventArgs
             * @type System.Uri
             */
            _uri: null
        },
        props: {
            Content: {
                get: function () {
                    return this._content;
                }
            },
            Uri: {
                get: function () {
                    return this._uri;
                }
            }
        },
        ctors: {
            /**
             * Initializes a new instance of the NavigationEventArgs class, based on content and URI.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Navigation.NavigationEventArgs
             * @memberof Windows.UI.Xaml.Navigation.NavigationEventArgs
             * @param   {System.Object}    content    Initializes the NavigationEventArgs.Content property.
             * @param   {System.Uri}       uri        Initializes the NavigationEventArgs.Uri property.
             * @return  {void}
             */
            ctor: function (content, uri) {
                this.$initialize();
                System.Object.call(this);
                this._content = content;
                this._uri = uri;
            }
        }
    });

    /**
     * Provides data for the NavigationFailed event of the NavigationService class and the NavigationFailed event of the Frame class.
     *
     * @public
     * @class Windows.UI.Xaml.Navigation.NavigationFailedEventArgs
     * @augments System.Object
     */
    Bridge.define("Windows.UI.Xaml.Navigation.NavigationFailedEventArgs", {
        fields: {
            _exception: null,
            _handled: false,
            _uri: null
        },
        props: {
            /**
             * Gets the error from the failed navigation.
             *
             * @instance
             * @public
             * @readonly
             * @memberof Windows.UI.Xaml.Navigation.NavigationFailedEventArgs
             * @function Exception
             * @type System.Exception
             */
            Exception: {
                get: function () {
                    return this._exception;
                }
            },
            /**
             * Gets or sets a value that indicates whether the failure event has been handled.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Navigation.NavigationFailedEventArgs
             * @function Handled
             * @type boolean
             */
            Handled: {
                get: function () {
                    return this._handled;
                },
                set: function (value) {
                    this._handled = value;
                }
            },
            /**
             * Gets the uniform resource identifier (URI) for the content that could not be navigated to.
             *
             * @instance
             * @public
             * @readonly
             * @memberof Windows.UI.Xaml.Navigation.NavigationFailedEventArgs
             * @function Uri
             * @type System.Uri
             */
            Uri: {
                get: function () {
                    return this._uri;
                }
            }
        },
        ctors: {
            /**
             * Initializes a new instance of the NavigationFailedEventArgs.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Navigation.NavigationFailedEventArgs
             * @memberof Windows.UI.Xaml.Navigation.NavigationFailedEventArgs
             * @param   {System.Exception}    exception    
             * @param   {boolean}             handled      
             * @param   {System.Uri}          uri
             * @return  {void}
             */
            ctor: function (exception, handled, uri) {
                this.$initialize();
                System.Object.call(this);
                this._exception = exception;
                this._handled = handled;
                this._uri = uri;
            }
        }
    });

    /**
     * Represents the base class for classes that convert a requested uniform resource
     identifier (URI) into a new URI based on mapping rules.
     *
     * @abstract
     * @public
     * @class Windows.UI.Xaml.Navigation.UriMapperBase
     */
    Bridge.define("Windows.UI.Xaml.Navigation.UriMapperBase");

    Bridge.define("Windows.UI.Xaml.PositionsWatcher", {
        fields: {
            _controlsToWatch: null,
            _timer: null
        },
        ctors: {
            init: function () {
                this._controlsToWatch = new (System.Collections.Generic.List$1(Windows.UI.Xaml.ControlToWatch)).ctor();
                this._timer = new Windows.UI.Xaml.DispatcherTimer();
            },
            ctor: function () {
                this.$initialize();
                this._timer.Interval = new System.TimeSpan(0, 0, 0, 0, 500);
                this._timer.addTick(Bridge.fn.cacheBind(this, this._timer_Tick));
            }
        },
        methods: {
            _timer_Tick: function (sender, e) {
                var $t;
                var snapshot = this._controlsToWatch.ToArray();
                $t = Bridge.getEnumerator(snapshot);
                try {
                    while ($t.moveNext()) {
                        var controlToWatch = $t.Current;
                        var elementCurrentPosition = DotNetForHtml5.Core.INTERNAL_PopupsManager.GetUIElementAbsolutePosition(controlToWatch.ControltoWatch);
                        var elementCurrentSize = new Windows.Foundation.Size();
                        var control = Bridge.as(controlToWatch.ControltoWatch, Windows.UI.Xaml.FrameworkElement);
                        if (control != null) {
                            elementCurrentSize = control.INTERNAL_GetActualWidthAndHeight();
                        } else {
                            elementCurrentSize = new Windows.Foundation.Size.ctor();
                        }

                        if (Windows.Foundation.Point.op_Inequality(elementCurrentPosition.$clone(), controlToWatch.PreviousPosition.$clone()) || !CSHTML5.Internal.INTERNAL_SizeComparisonHelpers.AreSizesEqual(elementCurrentSize.$clone(), controlToWatch.PreviousSize.$clone())) {
                            controlToWatch.OnPositionOrSizeChanged(elementCurrentPosition.$clone(), elementCurrentSize.$clone());
                            controlToWatch.PreviousPosition = elementCurrentPosition.$clone();
                            controlToWatch.PreviousSize = elementCurrentSize.$clone();
                        }
                    }
                } finally {
                    if (Bridge.is($t, System.IDisposable)) {
                        $t.System$IDisposable$Dispose();
                    }
                }
            },
            AddControlToWatch: function (elementToWatch, callback) {
                var returnvalue = new Windows.UI.Xaml.ControlToWatch(elementToWatch, callback);

                var elementCurrentSize = new Windows.Foundation.Size();
                var control = Bridge.as(elementToWatch, Windows.UI.Xaml.FrameworkElement);
                if (control != null) {
                    elementCurrentSize = control.INTERNAL_GetActualWidthAndHeight();
                } else {
                    elementCurrentSize = new Windows.Foundation.Size.ctor();
                }

                var elementCurrentPosition = DotNetForHtml5.Core.INTERNAL_PopupsManager.GetUIElementAbsolutePosition(elementToWatch);
                returnvalue.PreviousPosition = elementCurrentPosition.$clone();
                returnvalue.PreviousSize = elementCurrentSize.$clone();

                this._controlsToWatch.add(returnvalue);
                if (!this._timer.IsEnabled) {
                    this._timer.Start();
                }
                return returnvalue;
            },
            RemoveControlToWatch: function (controlToStopWatching) {
                this._controlsToWatch.remove(controlToStopWatching);
                if (this._controlsToWatch.Count === 0) {
                    this._timer.Stop();
                }
            }
        }
    });

    /**
     * @memberof Windows.UI.Xaml
     * @callback Windows.UI.Xaml.CoerceValueCallback
     * @param   {Windows.UI.Xaml.DependencyObject}    d            
     * @param   {System.Object}                       baseValue
     * @return  {System.Object}
     */

    /**
     * @memberof Windows.UI.Xaml
     * @callback Windows.UI.Xaml.MethodToUpdateDom
     * @param   {Windows.UI.Xaml.DependencyObject}    d           
     * @param   {System.Object}                       newValue
     * @return  {void}
     */

    /**
     * @memberof Windows.UI.Xaml
     * @callback Windows.UI.Xaml.PropertyChangedCallback
     * @param   {Windows.UI.Xaml.DependencyObject}                      d    
     * @param   {Windows.UI.Xaml.DependencyPropertyChangedEventArgs}    e
     * @return  {void}
     */

    /**
     * Defines behavior aspects of a dependency property, including conditions it
     was registered with.
     *
     * @public
     * @class Windows.UI.Xaml.PropertyMetadata
     */
    Bridge.define("Windows.UI.Xaml.PropertyMetadata", {
        fields: {
            _defaultValue: null,
            _inherits: false,
            _callPropertyChangedWhenLoadedIntoVisualTree: 0,
            _propertyChangedCallback: null,
            _methodToUpdateDom: null,
            _coerceValueCallback: null,
            IsDefaultValueModified: false,
            Sealed: false,
            GetCSSEquivalent: null,
            GetCSSEquivalents: null
        },
        props: {
            IsSealed: {
                get: function () {
                    return this.Sealed;
                }
            },
            /**
             * Gets the default value for the dependency property.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.PropertyMetadata
             * @function DefaultValue
             * @type System.Object
             */
            DefaultValue: {
                get: function () {
                    return this._defaultValue;
                },
                set: function (value) {
                    if (Bridge.referenceEquals(value, Windows.UI.Xaml.DependencyProperty.UnsetValue)) {
                        throw new System.ArgumentException.$ctor1("Default value cannot be 'Unset'.");
                    }

                    this.CheckSealed();

                    if (!Bridge.referenceEquals(this._defaultValue, value)) {
                        this._defaultValue = value;
                        this.IsDefaultValueModified = true;
                    }
                }
            },
            /**
             * Gets the method that is called when the property value changes.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.PropertyMetadata
             * @function PropertyChangedCallback
             * @type Windows.UI.Xaml.PropertyChangedCallback
             */
            PropertyChangedCallback: {
                get: function () {
                    return this._propertyChangedCallback;
                },
                set: function (value) {
                    this.CheckSealed();
                    this._propertyChangedCallback = value;
                }
            },
            /**
             * Gets the method that is called when the object is added to the visual tree, and when the property value changes while the object is already in the visual tree.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.PropertyMetadata
             * @function MethodToUpdateDom
             * @type Windows.UI.Xaml.MethodToUpdateDom
             */
            MethodToUpdateDom: {
                get: function () {
                    return this._methodToUpdateDom;
                },
                set: function (value) {
                    this.CheckSealed();
                    this._methodToUpdateDom = value;
                }
            },
            /**
             * Gets the method that will be called on update of value.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.PropertyMetadata
             * @function CoerceValueCallback
             * @type Windows.UI.Xaml.CoerceValueCallback
             */
            CoerceValueCallback: {
                get: function () {
                    return this._coerceValueCallback;
                },
                set: function (value) {
                    this.CheckSealed();
                    this._coerceValueCallback = value;
                }
            },
            /**
             * Determines if the property's value can be inherited from a parent element to a child element.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.PropertyMetadata
             * @function Inherits
             * @type boolean
             */
            Inherits: {
                get: function () {
                    return this._inherits;
                },
                set: function (value) {
                    this.CheckSealed();
                    this._inherits = value;
                }
            },
            /**
             * Determines if the callback method should be called when the element is added to the visual tree.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.PropertyMetadata
             * @function CallPropertyChangedWhenLoadedIntoVisualTree
             * @type Windows.UI.Xaml.WhenToCallPropertyChangedEnum
             */
            CallPropertyChangedWhenLoadedIntoVisualTree: {
                get: function () {
                    return this._callPropertyChangedWhenLoadedIntoVisualTree;
                },
                set: function (value) {
                    this.CheckSealed();
                    this._callPropertyChangedWhenLoadedIntoVisualTree = value;
                }
            }
        },
        ctors: {
            /**
             * Initializes a new instance of Windows.UI.Xaml.PropertyMetadata.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.PropertyMetadata
             * @memberof Windows.UI.Xaml.PropertyMetadata
             * @return  {void}
             */
            ctor: function () {
                this.$initialize();
                this.CallPropertyChangedWhenLoadedIntoVisualTree = Windows.UI.Xaml.WhenToCallPropertyChangedEnum.Never;
            },
            /**
             * Initializes a new instance of the PropertyMetadata class, using a property
             default value.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.PropertyMetadata
             * @memberof Windows.UI.Xaml.PropertyMetadata
             * @param   {System.Object}    defaultValue    A default value for the property where this PropertyMetadata is applied.
             * @return  {void}
             */
            $ctor1: function (defaultValue) {
                Windows.UI.Xaml.PropertyMetadata.ctor.call(this);
                this.DefaultValue = defaultValue;
            },
            /**
             * Initializes a new instance of the PropertyMetadata class, using a callback reference.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.PropertyMetadata
             * @memberof Windows.UI.Xaml.PropertyMetadata
             * @param   {Windows.UI.Xaml.PropertyChangedCallback}    propertyChangedCallback    A reference to the callback to call for property changed behavior.
             * @return  {void}
             */
            $ctor4: function (propertyChangedCallback) {
                Windows.UI.Xaml.PropertyMetadata.ctor.call(this);
                this.PropertyChangedCallback = propertyChangedCallback;
            },
            /**
             * Initializes a new instance of the PropertyMetadata class, using a property
             default value and callback reference.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.PropertyMetadata
             * @memberof Windows.UI.Xaml.PropertyMetadata
             * @param   {System.Object}                              defaultValue               A default value for the property where this PropertyMetadata is applied.
             * @param   {Windows.UI.Xaml.PropertyChangedCallback}    propertyChangedCallback    A reference to the callback to call for property changed behavior.
             * @return  {void}
             */
            $ctor2: function (defaultValue, propertyChangedCallback) {
                Windows.UI.Xaml.PropertyMetadata.ctor.call(this);
                this.DefaultValue = defaultValue;
                this.PropertyChangedCallback = propertyChangedCallback;
            },
            /**
             * Type meta construction
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.PropertyMetadata
             * @memberof Windows.UI.Xaml.PropertyMetadata
             * @param   {System.Object}                              defaultValue               Default value of property
             * @param   {Windows.UI.Xaml.PropertyChangedCallback}    propertyChangedCallback    Called when the property has been changed
             * @param   {Windows.UI.Xaml.CoerceValueCallback}        coerceValueCallback        Called on update of value
             * @return  {void}
             */
            $ctor3: function (defaultValue, propertyChangedCallback, coerceValueCallback) {
                Windows.UI.Xaml.PropertyMetadata.ctor.call(this);
                this.DefaultValue = defaultValue;
                this.PropertyChangedCallback = propertyChangedCallback;
                this.CoerceValueCallback = coerceValueCallback;
            }
        },
        methods: {
            Seal: function () {
                this.Sealed = true;
            },
            CheckSealed: function () {
                if (this.Sealed) {
                    throw new System.InvalidOperationException.$ctor1("Cannot change property metadata after it has been associated with a property.");
                }
            }
        }
    });

    Bridge.define("Windows.UI.Xaml.PropertyPathHelper", {
        statics: {
            methods: {
                AccessValueByApplyingPropertyPathIfAny: function (item, path) {
                    if (!System.String.isNullOrWhiteSpace(path)) {
                        var splittedPath = System.String.split(path, [46].map(function (i) {{ return String.fromCharCode(i); }}));
                        var currentItem = item;
                        for (var i = 0; i < splittedPath.length; i = (i + 1) | 0) {
                            var type = Bridge.getType(currentItem);
                            var memberArray = Bridge.Reflection.getMembers(type, 31, 28, splittedPath[System.Array.index(i, splittedPath)]);
                            if (memberArray != null && memberArray.length > 0) {
                                if (Bridge.is(memberArray[System.Array.index(0, memberArray)], System.Reflection.FieldInfo)) {
                                    var info = Bridge.cast(memberArray[System.Array.index(0, memberArray)], System.Reflection.FieldInfo);
                                    currentItem = Bridge.Reflection.fieldAccess(info, Bridge.unbox(currentItem));
                                } else if (Bridge.is(memberArray[System.Array.index(0, memberArray)], System.Reflection.PropertyInfo)) {
                                    var info1 = Bridge.cast(memberArray[System.Array.index(0, memberArray)], System.Reflection.PropertyInfo);
                                    currentItem = Bridge.Reflection.midel(info1.g, Bridge.unbox(currentItem))();
                                } else {
                                    return "";
                                }
                            } else {
                                if (!CSHTML5.Interop.IsRunningInTheSimulator) {
                                    var obj = 
                            (function(){
    var propname = "$$" + splittedPath[System.Array.index(i, splittedPath)];
    var temp = currentItem[propname];
    if(!temp){
        propname = currentItem.GetType().Name + "$$" + propname;
        temp = currentItem[propname];
        if(!temp){
            propname = propname + "$";
            temp = currentItem[propname];
            if(!temp) temp = null;
        }
    }
    return temp;
}());
                                    if (obj == null) {
                                        return "";
                                    }
                                    currentItem = obj;
                                } else {
                                    return "";
                                }
                            }
                        }
                        return currentItem;
                    } else {
                        return item;
                    }
                }
            }
        }
    });

    Bridge.define("Windows.UI.Xaml.PropertyValue", {
        $kind: "struct",
        statics: {
            methods: {
                getDefaultValue: function () { return new Windows.UI.Xaml.PropertyValue(); }
            }
        },
        fields: {
            Property: null,
            ValueInternal: null
        },
        ctors: {
            ctor: function () {
                this.$initialize();
            }
        },
        methods: {
            getHashCode: function () {
                var h = Bridge.addHash([5894063728, this.Property, this.ValueInternal]);
                return h;
            },
            equals: function (o) {
                if (!Bridge.is(o, Windows.UI.Xaml.PropertyValue)) {
                    return false;
                }
                return Bridge.equals(this.Property, o.Property) && Bridge.equals(this.ValueInternal, o.ValueInternal);
            },
            $clone: function (to) {
                var s = to || new Windows.UI.Xaml.PropertyValue();
                s.Property = this.Property;
                s.ValueInternal = this.ValueInternal;
                return s;
            }
        }
    });

    Bridge.define("Windows.UI.Xaml.ResourceDictionary.PrivateFlags", {
        $kind: "nested enum",
        statics: {
            fields: {
                IsInitialized: 1,
                IsInitializePending: 2,
                IsReadOnly: 4,
                IsThemeDictionary: 8,
                HasImplicitStyles: 16,
                CanBeAccessedAcrossThreads: 32,
                InvalidatesImplicitDataTemplateResources: 64,
                HasImplicitDataTemplates: 128
            }
        },
        $utype: System.Byte
    });

    Bridge.define("Windows.UI.Xaml.ResourceDictionaryCollection.SimpleMonitor", {
        inherits: [System.IDisposable],
        $kind: "nested class",
        fields: {
            _busyCount: 0
        },
        props: {
            Busy: {
                get: function () {
                    return this._busyCount > 0;
                }
            }
        },
        alias: ["Dispose", "System$IDisposable$Dispose"],
        methods: {
            Enter: function () {
                this._busyCount = (this._busyCount + 1) | 0;
            },
            Dispose: function () {
                this._busyCount = (this._busyCount - 1) | 0;
            }
        }
    });

    /**
     * This is the data that is passed through the DescendentsWalker
         during a resources change tree-walk.
     *
     * @class Windows.UI.Xaml.ResourcesChangeInfo
     */
    Bridge.define("Windows.UI.Xaml.ResourcesChangeInfo", {
        $kind: "struct",
        statics: {
            props: {
                /**
                 * This is a static accessor for a ResourcesChangeInfo that is used
                     for theme change notifications
                 *
                 * @static
                 * @readonly
                 * @memberof Windows.UI.Xaml.ResourcesChangeInfo
                 * @function ThemeChangeInfo
                 * @type Windows.UI.Xaml.ResourcesChangeInfo
                 */
                ThemeChangeInfo: {
                    get: function () {
                        var info = new Windows.UI.Xaml.ResourcesChangeInfo.ctor();
                        info.IsThemeChange = true;
                        return info.$clone();
                    }
                },
                /**
                 * This is a static accessor for a ResourcesChangeInfo that is used
                     for tree change notifications
                 *
                 * @static
                 * @readonly
                 * @memberof Windows.UI.Xaml.ResourcesChangeInfo
                 * @function TreeChangeInfo
                 * @type Windows.UI.Xaml.ResourcesChangeInfo
                 */
                TreeChangeInfo: {
                    get: function () {
                        var info = new Windows.UI.Xaml.ResourcesChangeInfo.ctor();
                        info.IsTreeChange = true;
                        return info.$clone();
                    }
                },
                /**
                 * This is a static accessor for a ResourcesChangeInfo that is used
                     for system colors or settings change notifications
                 *
                 * @static
                 * @readonly
                 * @memberof Windows.UI.Xaml.ResourcesChangeInfo
                 * @function SysColorsOrSettingsChangeInfo
                 * @type Windows.UI.Xaml.ResourcesChangeInfo
                 */
                SysColorsOrSettingsChangeInfo: {
                    get: function () {
                        var info = new Windows.UI.Xaml.ResourcesChangeInfo.ctor();
                        info.IsSysColorsOrSettingsChange = true;
                        return info.$clone();
                    }
                },
                /**
                 * This is a static accessor for a ResourcesChangeInfo that is used
                     for any ResourceDictionary operations that we aren't able to provide
                     the precise 'key that changed' information
                 *
                 * @static
                 * @readonly
                 * @memberof Windows.UI.Xaml.ResourcesChangeInfo
                 * @function CatastrophicDictionaryChangeInfo
                 * @type Windows.UI.Xaml.ResourcesChangeInfo
                 */
                CatastrophicDictionaryChangeInfo: {
                    get: function () {
                        var info = new Windows.UI.Xaml.ResourcesChangeInfo.ctor();
                        info.IsCatastrophicDictionaryChange = true;
                        return info.$clone();
                    }
                }
            },
            methods: {
                getDefaultValue: function () { return new Windows.UI.Xaml.ResourcesChangeInfo(); }
            }
        },
        fields: {
            _oldDictionaries: null,
            _newDictionaries: null,
            _key: null,
            _container: null,
            _flags: 0
        },
        props: {
            IsThemeChange: {
                get: function () {
                    return this.ReadPrivateFlag(Windows.UI.Xaml.ResourcesChangeInfo.PrivateFlags.IsThemeChange);
                },
                set: function (value) {
                    this.WritePrivateFlag(Windows.UI.Xaml.ResourcesChangeInfo.PrivateFlags.IsThemeChange, value);
                }
            },
            IsTreeChange: {
                get: function () {
                    return this.ReadPrivateFlag(Windows.UI.Xaml.ResourcesChangeInfo.PrivateFlags.IsTreeChange);
                },
                set: function (value) {
                    this.WritePrivateFlag(Windows.UI.Xaml.ResourcesChangeInfo.PrivateFlags.IsTreeChange, value);
                }
            },
            IsStyleResourcesChange: {
                get: function () {
                    return this.ReadPrivateFlag(Windows.UI.Xaml.ResourcesChangeInfo.PrivateFlags.IsStyleResourceChange);
                },
                set: function (value) {
                    this.WritePrivateFlag(Windows.UI.Xaml.ResourcesChangeInfo.PrivateFlags.IsStyleResourceChange, value);
                }
            },
            IsTemplateResourcesChange: {
                get: function () {
                    return this.ReadPrivateFlag(Windows.UI.Xaml.ResourcesChangeInfo.PrivateFlags.IsTemplateResourceChange);
                },
                set: function (value) {
                    this.WritePrivateFlag(Windows.UI.Xaml.ResourcesChangeInfo.PrivateFlags.IsTemplateResourceChange, value);
                }
            },
            IsSysColorsOrSettingsChange: {
                get: function () {
                    return this.ReadPrivateFlag(Windows.UI.Xaml.ResourcesChangeInfo.PrivateFlags.IsSysColorsOrSettingsChange);
                },
                set: function (value) {
                    this.WritePrivateFlag(Windows.UI.Xaml.ResourcesChangeInfo.PrivateFlags.IsSysColorsOrSettingsChange, value);
                }
            },
            IsCatastrophicDictionaryChange: {
                get: function () {
                    return this.ReadPrivateFlag(Windows.UI.Xaml.ResourcesChangeInfo.PrivateFlags.IsCatastrophicDictionaryChange);
                },
                set: function (value) {
                    this.WritePrivateFlag(Windows.UI.Xaml.ResourcesChangeInfo.PrivateFlags.IsCatastrophicDictionaryChange, value);
                }
            },
            IsImplicitDataTemplateChange: {
                get: function () {
                    return this.ReadPrivateFlag(Windows.UI.Xaml.ResourcesChangeInfo.PrivateFlags.IsImplicitDataTemplateChange);
                },
                set: function (value) {
                    this.WritePrivateFlag(Windows.UI.Xaml.ResourcesChangeInfo.PrivateFlags.IsImplicitDataTemplateChange, value);
                }
            },
            IsResourceAddOperation: {
                get: function () {
                    return this._key != null || (this._newDictionaries != null && this._newDictionaries.Count > 0);
                }
            },
            Container: {
                get: function () {
                    return this._container;
                }
            }
        },
        ctors: {
            /**
             * This constructor is used for notifying changes to individual
                 entries in a ResourceDictionary
             *
             * @instance
             * @this Windows.UI.Xaml.ResourcesChangeInfo
             * @memberof Windows.UI.Xaml.ResourcesChangeInfo
             * @param   {System.Object}    key
             * @return  {void}
             */
            $ctor2: function (key) {
                this.$initialize();
                this._oldDictionaries = null;
                this._newDictionaries = null;
                this._key = key;
                this._container = null;
                this._flags = 0;
            },
            /**
             * This constructor is used for notifying changes in Application.Resources,
                 [FE/FCE].Resources, ResourceDictionary.EndInit
             *
             * @instance
             * @this Windows.UI.Xaml.ResourcesChangeInfo
             * @memberof Windows.UI.Xaml.ResourcesChangeInfo
             * @param   {Windows.UI.Xaml.ResourceDictionary}    oldDictionary    
             * @param   {Windows.UI.Xaml.ResourceDictionary}    newDictionary
             * @return  {void}
             */
            $ctor3: function (oldDictionary, newDictionary) {
                this.$initialize();
                this._oldDictionaries = null;
                if (oldDictionary != null) {
                    this._oldDictionaries = new (System.Collections.Generic.List$1(Windows.UI.Xaml.ResourceDictionary)).$ctor2(1);
                    this._oldDictionaries.add(oldDictionary);
                }

                this._newDictionaries = null;
                if (newDictionary != null) {
                    this._newDictionaries = new (System.Collections.Generic.List$1(Windows.UI.Xaml.ResourceDictionary)).$ctor2(1);
                    this._newDictionaries.add(newDictionary);
                }

                this._key = null;
                this._container = null;
                this._flags = 0;
            },
            /**
             * This constructor is used for notifying changes in Style.Resources,
                 Template.Resources, ThemeStyle.Resources
             *
             * @instance
             * @this Windows.UI.Xaml.ResourcesChangeInfo
             * @memberof Windows.UI.Xaml.ResourcesChangeInfo
             * @param   {System.Collections.Generic.List$1}    oldDictionaries              
             * @param   {System.Collections.Generic.List$1}    newDictionaries              
             * @param   {boolean}                              isStyleResourcesChange       
             * @param   {boolean}                              isTemplateResourcesChange    
             * @param   {Windows.UI.Xaml.DependencyObject}     container
             * @return  {void}
             */
            $ctor1: function (oldDictionaries, newDictionaries, isStyleResourcesChange, isTemplateResourcesChange, container) {
                this.$initialize();
                this._oldDictionaries = oldDictionaries;
                this._newDictionaries = newDictionaries;
                this._key = null;
                this._container = container;
                this._flags = 0;
                this.IsStyleResourcesChange = isStyleResourcesChange;
                this.IsTemplateResourcesChange = isTemplateResourcesChange;
            },
            ctor: function () {
                this.$initialize();
            }
        },
        methods: {
            Contains: function (key, isImplicitStyleKey) {
                if (this.IsTreeChange || this.IsCatastrophicDictionaryChange) {
                    return true;
                } else if (this.IsThemeChange || this.IsSysColorsOrSettingsChange) {

                    return !isImplicitStyleKey;
                }

                System.Diagnostics.Debug.Assert$1(this._oldDictionaries != null || this._newDictionaries != null || this._key != null, "Must have a dictionary or a key that has changed");

                if (this._key != null) {
                    if (Bridge.equals(this._key, key)) {
                        return true;
                    }
                }

                if (this._oldDictionaries != null) {
                    for (var i = 0; i < this._oldDictionaries.Count; i = (i + 1) | 0) {
                        if (this._oldDictionaries.getItem(i).contains(key)) {
                            return true;
                        }
                    }
                }

                if (this._newDictionaries != null) {
                    for (var i1 = 0; i1 < this._newDictionaries.Count; i1 = (i1 + 1) | 0) {
                        if (this._newDictionaries.getItem(i1).contains(key)) {
                            return true;
                        }
                    }
                }

                return false;
            },
            SetIsImplicitDataTemplateChange: function () {




            },
            WritePrivateFlag: function (bit, value) {
                if (value) {
                    this._flags |= bit & 255;
                } else {
                    this._flags &= (~bit) & 255;
                }
            },
            ReadPrivateFlag: function (bit) {
                return (((this._flags & bit) & 255)) !== 0;
            },
            getHashCode: function () {
                var h = Bridge.addHash([6443755798, this._oldDictionaries, this._newDictionaries, this._key, this._container, this._flags]);
                return h;
            },
            equals: function (o) {
                if (!Bridge.is(o, Windows.UI.Xaml.ResourcesChangeInfo)) {
                    return false;
                }
                return Bridge.equals(this._oldDictionaries, o._oldDictionaries) && Bridge.equals(this._newDictionaries, o._newDictionaries) && Bridge.equals(this._key, o._key) && Bridge.equals(this._container, o._container) && Bridge.equals(this._flags, o._flags);
            },
            $clone: function (to) {
                var s = to || new Windows.UI.Xaml.ResourcesChangeInfo();
                s._oldDictionaries = this._oldDictionaries;
                s._newDictionaries = this._newDictionaries;
                s._key = this._key;
                s._container = this._container;
                s._flags = this._flags;
                return s;
            }
        }
    });

    Bridge.define("Windows.UI.Xaml.ResourcesChangeInfo.PrivateFlags", {
        $kind: "nested enum",
        statics: {
            fields: {
                IsThemeChange: 1,
                IsTreeChange: 2,
                IsStyleResourceChange: 4,
                IsTemplateResourceChange: 8,
                IsSysColorsOrSettingsChange: 16,
                IsCatastrophicDictionaryChange: 32,
                IsImplicitDataTemplateChange: 64
            }
        },
        $utype: System.Byte
    });

    Bridge.define("Windows.UI.Xaml.RoutedEvent", {
        fields: {
            _eventName: null
        },
        ctors: {
            ctor: function (eventName) {
                this.$initialize();
                this._eventName = eventName;
            }
        },
        methods: {
            /**
             * Returns the string representation of the routed event.
             *
             * @instance
             * @public
             * @override
             * @this Windows.UI.Xaml.RoutedEvent
             * @memberof Windows.UI.Xaml.RoutedEvent
             * @return  {string}        The name of the routed event.
             */
            toString: function () {
                return this._eventName;
            }
        }
    });

    /** @namespace Windows.UI.Xaml.Shapes */

    /**
     * The {@link } namespace defines basic shapes that are intended for decorative rendering or for compositing non-interactive parts of controls.
     *
     * @class Windows.UI.Xaml.Shapes.NamespaceDoc
     */
    Bridge.define("Windows.UI.Xaml.Shapes.NamespaceDoc");

    /**
     * Contains the event data for the Application.Startup event.
     *
     * @public
     * @class Windows.UI.Xaml.StartupEventArgs
     * @augments System.Object
     */
    Bridge.define("Windows.UI.Xaml.StartupEventArgs");

    /**
     * Defines a generated instance of a FrameworkTemplate.
     *
     * @public
     * @class Windows.UI.Xaml.TemplateInstance
     */
    Bridge.define("Windows.UI.Xaml.TemplateInstance", {
        fields: {
            /**
             * The element that contains the FrameworkTemplate.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.TemplateInstance
             * @type Windows.UI.Xaml.Controls.Control
             */
            TemplateOwner: null,
            /**
             * The visual subtree that has been generated for the FrameworkTemplate.
             Note: this should only be used inside the methods put as parameter in FrameworkTemplate.SetMethodToInstantiateFrameworkTemplate, to define the root of the Template's generated subtree.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.TemplateInstance
             * @type Windows.UI.Xaml.FrameworkElement
             */
            TemplateContent: null
        }
    });

    /**
     * Style authors should be able to identify the part type used for styling the specific class.
     The part is usually required in the style and should have a specific predefined name.
     *
     * @public
     * @class Windows.UI.Xaml.TemplatePartAttribute
     * @augments System.Attribute
     */
    Bridge.define("Windows.UI.Xaml.TemplatePartAttribute", {
        inherits: [System.Attribute],
        fields: {
            _name: null,
            _type: null
        },
        props: {
            /**
             * Part name used by the class to indentify required element in the style
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.TemplatePartAttribute
             * @function Name
             * @type string
             */
            Name: {
                get: function () {
                    return this._name;
                },
                set: function (value) {
                    this._name = value;
                }
            },
            /**
             * Type of the element that should be used as a part with name specified in TemplatePartAttribute.Name
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.TemplatePartAttribute
             * @function Type
             * @type System.Type
             */
            Type: {
                get: function () {
                    return this._type;
                },
                set: function (value) {
                    this._type = value;
                }
            }
        },
        ctors: {
            /**
             * Default constructor
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.TemplatePartAttribute
             * @memberof Windows.UI.Xaml.TemplatePartAttribute
             * @return  {void}
             */
            ctor: function () {
                this.$initialize();
                System.Attribute.ctor.call(this);
            }
        }
    });

    /**
     * Define an expected VisualState in the contract between a Control and its
     ControlTemplate for use with the VisualStateManager.
     *
     * @public
     * @class Windows.UI.Xaml.TemplateVisualStateAttribute
     * @augments System.Attribute
     */
    Bridge.define("Windows.UI.Xaml.TemplateVisualStateAttribute", {
        inherits: [System.Attribute],
        fields: {
            /**
             * Name of the VisualState.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.TemplateVisualStateAttribute
             * @function Name
             * @type string
             */
            Name: null,
            /**
             * Name of the VisualStateGroup containing this state.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.TemplateVisualStateAttribute
             * @function GroupName
             * @type string
             */
            GroupName: null
        }
    });

    /**
     * Specifies whether text is centered, left-aligned, or right-aligned.
     *
     * @public
     * @class Windows.UI.Xaml.TextAlignment
     */
    Bridge.define("Windows.UI.Xaml.TextAlignment", {
        $kind: "enum",
        statics: {
            fields: {
                /**
                 * Text is centered within the container.
                 *
                 * @static
                 * @public
                 * @memberof Windows.UI.Xaml.TextAlignment
                 * @constant
                 * @default 0
                 * @type Windows.UI.Xaml.TextAlignment
                 */
                Center: 0,
                /**
                 * Text is aligned to the left edge of the container.
                 *
                 * @static
                 * @public
                 * @memberof Windows.UI.Xaml.TextAlignment
                 * @constant
                 * @default 1
                 * @type Windows.UI.Xaml.TextAlignment
                 */
                Left: 1,
                /**
                 * Text is aligned to the right edge of the container.
                 *
                 * @static
                 * @public
                 * @memberof Windows.UI.Xaml.TextAlignment
                 * @constant
                 * @default 2
                 * @type Windows.UI.Xaml.TextAlignment
                 */
                Right: 2,
                /**
                 * Text is justified within the container.
                 *
                 * @static
                 * @public
                 * @memberof Windows.UI.Xaml.TextAlignment
                 * @constant
                 * @default 3
                 * @type Windows.UI.Xaml.TextAlignment
                 */
                Justify: 3
            }
        }
    });

    /**
     * Specifies whether text wraps when it reaches the edge of its container.
     *
     * @public
     * @class Windows.UI.Xaml.TextWrapping
     */
    Bridge.define("Windows.UI.Xaml.TextWrapping", {
        $kind: "enum",
        statics: {
            fields: {
                /**
                 * No line wrapping is performed.
                 *
                 * @static
                 * @public
                 * @memberof Windows.UI.Xaml.TextWrapping
                 * @constant
                 * @default 1
                 * @type Windows.UI.Xaml.TextWrapping
                 */
                NoWrap: 1,
                /**
                 * Line breaking occurs if a line of text overflows beyond the available width
                 of its container. Line breaking occurs even if the standard line-breaking
                 algorithm cannot determine any line break opportunity, such as when a line
                 of text includes a long word that is constrained by a fixed-width container
                 without scrolling.
                 *
                 * @static
                 * @public
                 * @memberof Windows.UI.Xaml.TextWrapping
                 * @constant
                 * @default 2
                 * @type Windows.UI.Xaml.TextWrapping
                 */
                Wrap: 2
            }
        }
    });

    /**
     * [SECURITY CRITICAL] Describes the thickness of a frame around a rectangle.
     Four System.Double values describe the Windows.UI.Xaml.Thickness.Left, Windows.UI.Xaml.Thickness.Top,
     Windows.UI.Xaml.Thickness.Right, and Windows.UI.Xaml.Thickness.Bottom sides
     of the rectangle, respectively.
     *
     * @public
     * @class Windows.UI.Xaml.Thickness
     */
    Bridge.define("Windows.UI.Xaml.Thickness", {
        $kind: "struct",
        statics: {
            ctors: {
                ctor: function () {
                    DotNetForHtml5.Core.TypeFromStringConverters.RegisterConverter(Windows.UI.Xaml.Thickness, Windows.UI.Xaml.Thickness.INTERNAL_ConvertFromString);
                }
            },
            methods: {
                INTERNAL_ConvertFromString: function (thicknessAsString) {
                    var splitter = 44;
                    var trimmedThicknessAsString = thicknessAsString.trim();
                    if (!System.Linq.Enumerable.from(trimmedThicknessAsString).contains(44)) {
                        splitter = 32;
                    }
                    var splittedString = System.String.split(trimmedThicknessAsString, [splitter].map(function (i) {{ return String.fromCharCode(i); }}));
                    if (splittedString.length === 1) {
                        var thickness = { v : 0.0 };
                        if (System.Double.tryParse(splittedString[System.Array.index(0, splittedString)], null, thickness)) {
                            return new Windows.UI.Xaml.Thickness.$ctor1(thickness.v).$clone();
                        }
                    } else if (splittedString.length === 2) {
                        var leftAndRight = { v : 0.0 };
                        var topAndBottom = { v : 0.0 };

                        var isParseOK = System.Double.tryParse(splittedString[System.Array.index(0, splittedString)], null, leftAndRight);
                        isParseOK = isParseOK && System.Double.tryParse(splittedString[System.Array.index(1, splittedString)], null, topAndBottom);

                        if (isParseOK) {
                            return new Windows.UI.Xaml.Thickness.$ctor2(leftAndRight.v, topAndBottom.v, leftAndRight.v, topAndBottom.v).$clone();
                        }
                    } else if (splittedString.length === 4) {
                        var left = { v : 0.0 };
                        var top = { v : 0.0 };
                        var right = { v : 0.0 };
                        var bottom = { v : 0.0 };

                        var isParseOK1 = System.Double.tryParse(splittedString[System.Array.index(0, splittedString)], null, left);
                        isParseOK1 = isParseOK1 && System.Double.tryParse(splittedString[System.Array.index(1, splittedString)], null, top);
                        isParseOK1 = isParseOK1 && System.Double.tryParse(splittedString[System.Array.index(2, splittedString)], null, right);
                        isParseOK1 = isParseOK1 && System.Double.tryParse(splittedString[System.Array.index(3, splittedString)], null, bottom);

                        if (isParseOK1) {
                            return new Windows.UI.Xaml.Thickness.$ctor2(left.v, top.v, right.v, bottom.v).$clone();
                        }
                    }
                    throw new System.FormatException.$ctor1((thicknessAsString || "") + " is not an eligible value for Thickness");
                }/**
                 * [SECURITY CRITICAL] Compares two Windows.UI.Xaml.Thickness structures for
                 inequality.
                 *
                 * @static
                 * @public
                 * @this Windows.UI.Xaml.Thickness
                 * @memberof Windows.UI.Xaml.Thickness
                 * @param   {Windows.UI.Xaml.Thickness}    t1    The first structure to compare.
                 * @param   {Windows.UI.Xaml.Thickness}    t2    The other structure to compare.
                 * @return  {boolean}                            true if the two instances of Windows.UI.Xaml.Thickness are not equal; otherwise,
                 false.
                 */
                ,
                op_Inequality: function (t1, t2) {
                    return (t1.Left !== t2.Left || t1.Top !== t2.Top || t1.Bottom !== t2.Bottom || t1.Right !== t2.Right);
                }/**
                 * [SECURITY CRITICAL] Compares the value of two Windows.UI.Xaml.Thickness structures
                 for equality.
                 *
                 * @static
                 * @public
                 * @this Windows.UI.Xaml.Thickness
                 * @memberof Windows.UI.Xaml.Thickness
                 * @param   {Windows.UI.Xaml.Thickness}    t1    The first structure to compare.
                 * @param   {Windows.UI.Xaml.Thickness}    t2    The other structure to compare.
                 * @return  {boolean}                            true if the two instances of Windows.UI.Xaml.Thickness are equal; otherwise,
                 false.
                 */
                ,
                op_Equality: function (t1, t2) {
                    return (t1.Left === t2.Left && t1.Top === t2.Top && t1.Bottom === t2.Bottom && t1.Right === t2.Right);
                },
                getDefaultValue: function () { return new Windows.UI.Xaml.Thickness(); }
            }
        },
        fields: {
            _left: 0,
            _top: 0,
            _right: 0,
            _bottom: 0
        },
        props: {
            /**
             * [SECURITY CRITICAL] Gets or sets the width, in pixels, of the lower side
             of the bounding rectangle.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Thickness
             * @function Bottom
             * @type number
             */
            Bottom: {
                get: function () {
                    return this._bottom;
                },
                set: function (value) {
                    this._bottom = value;
                }
            },
            /**
             * [SECURITY CRITICAL] Gets or sets the width, in pixels, of the left side of
             the bounding rectangle.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Thickness
             * @function Left
             * @type number
             */
            Left: {
                get: function () {
                    return this._left;
                },
                set: function (value) {
                    this._left = value;
                }
            },
            /**
             * [SECURITY CRITICAL] Gets or sets the width, in pixels, of the right side
             of the bounding rectangle.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Thickness
             * @function Right
             * @type number
             */
            Right: {
                get: function () {
                    return this._right;
                },
                set: function (value) {
                    this._right = value;
                }
            },
            /**
             * [SECURITY CRITICAL] Gets or sets the width, in pixels, of the upper side
             of the bounding rectangle.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Thickness
             * @function Top
             * @type number
             */
            Top: {
                get: function () {
                    return this._top;
                },
                set: function (value) {
                    this._top = value;
                }
            }
        },
        ctors: {
            /**
             * [SECURITY CRITICAL] Initializes a Windows.UI.Xaml.Thickness structure that
             has the specified uniform length on each side.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Thickness
             * @memberof Windows.UI.Xaml.Thickness
             * @param   {number}    uniformLength    The uniform length applied to all four sides of the bounding rectangle.
             * @return  {void}
             */
            $ctor1: function (uniformLength) {
                this.$initialize();
                this._left = uniformLength;
                this._top = uniformLength;
                this._right = uniformLength;
                this._bottom = uniformLength;
            },
            /**
             * [SECURITY CRITICAL] Initializes a Windows.UI.Xaml.Thickness structure that
             has specific lengths (supplied as a System.Double) applied to each side of
             the rectangle.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Thickness
             * @memberof Windows.UI.Xaml.Thickness
             * @param   {number}    left      The thickness for the left side of the rectangle.
             * @param   {number}    top       The thickness for the upper side of the rectangle.
             * @param   {number}    right     The thickness for the right side of the rectangle
             * @param   {number}    bottom    The thickness for the lower side of the rectangle.
             * @return  {void}
             */
            $ctor2: function (left, top, right, bottom) {
                this.$initialize();
                this._left = left;
                this._top = top;
                this._right = right;
                this._bottom = bottom;
            },
            ctor: function () {
                this.$initialize();
            }
        },
        methods: {
            /**
             * [SECURITY CRITICAL] Compares this Windows.UI.Xaml.Thickness structure to
             another System.Object for equality.
             *
             * @instance
             * @public
             * @override
             * @this Windows.UI.Xaml.Thickness
             * @memberof Windows.UI.Xaml.Thickness
             * @param   {System.Object}    obj    The object to compare.
             * @return  {boolean}                 true if the two objects are equal; otherwise, false.
             */
            equals: function (obj) {
                return (Bridge.is(obj, Windows.UI.Xaml.Thickness) && Windows.UI.Xaml.Thickness.op_Equality(System.Nullable.getValue(Bridge.cast(Bridge.unbox(obj, Windows.UI.Xaml.Thickness), Windows.UI.Xaml.Thickness)), this));
            },
            /**
             * [SECURITY CRITICAL] Compares this Windows.UI.Xaml.Thickness structure to
             another Windows.UI.Xaml.Thickness structure for equality.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Thickness
             * @memberof Windows.UI.Xaml.Thickness
             * @param   {Windows.UI.Xaml.Thickness}    thickness    An instance of Windows.UI.Xaml.Thickness to compare for equality.
             * @return  {boolean}                                   true if the two instances of Windows.UI.Xaml.Thickness are equal; otherwise,
             false.
             */
            Equals: function (thickness) {
                return Windows.UI.Xaml.Thickness.op_Equality(thickness.$clone(), this);
            },
            /**
             * [SECURITY CRITICAL] Returns the hash code of the structure.
             *
             * @instance
             * @public
             * @override
             * @this Windows.UI.Xaml.Thickness
             * @memberof Windows.UI.Xaml.Thickness
             * @return  {number}        A hash code for this instance of Windows.UI.Xaml.Thickness.
             */
            getHashCode: function () {
                return System.Double.getHashCode(this.Left) ^ System.Double.getHashCode(this.Top) ^ System.Double.getHashCode(this.Right) ^ System.Double.getHashCode(this.Bottom);
            },
            /**
             * [SECURITY CRITICAL] Returns the string representation of the Windows.UI.Xaml.Thickness
             structure.
             *
             * @instance
             * @public
             * @override
             * @this Windows.UI.Xaml.Thickness
             * @memberof Windows.UI.Xaml.Thickness
             * @return  {string}        A System.String that represents the Windows.UI.Xaml.Thickness value.
             */
            toString: function () {
                return System.Double.format(this.Left) + "," + System.Double.format(this.Top) + "," + System.Double.format(this.Right) + "," + System.Double.format(this.Bottom);
            },
            $clone: function (to) {
                var s = to || new Windows.UI.Xaml.Thickness();
                s._left = this._left;
                s._top = this._top;
                s._right = this._right;
                s._bottom = this._bottom;
                return s;
            }
        }
    });

    /**
     * Describes how a child element is vertically positioned or stretched within
     a parent's layout slot.
     *
     * @public
     * @class Windows.UI.Xaml.VerticalAlignment
     */
    Bridge.define("Windows.UI.Xaml.VerticalAlignment", {
        $kind: "enum",
        statics: {
            fields: {
                /**
                 * The element is aligned to the top of the parent's layout slot.
                 *
                 * @static
                 * @public
                 * @memberof Windows.UI.Xaml.VerticalAlignment
                 * @constant
                 * @default 0
                 * @type Windows.UI.Xaml.VerticalAlignment
                 */
                Top: 0,
                /**
                 * The element is aligned to the center of the parent's layout slot.
                 *
                 * @static
                 * @public
                 * @memberof Windows.UI.Xaml.VerticalAlignment
                 * @constant
                 * @default 1
                 * @type Windows.UI.Xaml.VerticalAlignment
                 */
                Center: 1,
                /**
                 * The element is aligned to the bottom of the parent's layout slot.
                 *
                 * @static
                 * @public
                 * @memberof Windows.UI.Xaml.VerticalAlignment
                 * @constant
                 * @default 2
                 * @type Windows.UI.Xaml.VerticalAlignment
                 */
                Bottom: 2,
                /**
                 * The element is stretched to fill the entire layout slot of the parent element.
                 *
                 * @static
                 * @public
                 * @memberof Windows.UI.Xaml.VerticalAlignment
                 * @constant
                 * @default 3
                 * @type Windows.UI.Xaml.VerticalAlignment
                 */
                Stretch: 3
            }
        }
    });

    /**
     * Specifies the display state of an element.
     *
     * @public
     * @class Windows.UI.Xaml.Visibility
     */
    Bridge.define("Windows.UI.Xaml.Visibility", {
        $kind: "enum",
        statics: {
            fields: {
                /**
                 * Display the element.
                 *
                 * @static
                 * @public
                 * @memberof Windows.UI.Xaml.Visibility
                 * @constant
                 * @default 0
                 * @type Windows.UI.Xaml.Visibility
                 */
                Visible: 0,
                /**
                 * Do not display the element, and do not reserve space for it in layout.
                 *
                 * @static
                 * @public
                 * @memberof Windows.UI.Xaml.Visibility
                 * @constant
                 * @default 1
                 * @type Windows.UI.Xaml.Visibility
                 */
                Collapsed: 1
            }
        }
    });

    Bridge.define("Windows.UI.Xaml.WhenToCallPropertyChangedEnum", {
        $kind: "enum",
        statics: {
            fields: {
                Always: 0,
                IfPropertyIsSet: 1,
                Never: 2
            }
        }
    });

    /**
     * Defines a dictionary that contains resources used by components of the app.
     This dictionary is oriented toward defining the resources in XAML, and then
     retrieving them through XAML references via the StaticResource markup extension.
     Alternatively you can access resources by traversing the dictionary at run
     time.
     *
     * @public
     * @class Windows.UI.Xaml.ResourceDictionary
     * @augments Windows.UI.Xaml.DependencyObject
     * @implements  System.Collections.Generic.IDictionary$2
     * @implements  System.Collections.IDictionary
     * @implements  System.Collections.ICollection
     * @implements  System.ComponentModel.ISupportInitialize
     * @implements  Windows.UI.Xaml.Markup.INameScope
     */
    Bridge.define("Windows.UI.Xaml.ResourceDictionary", {
        inherits: [Windows.UI.Xaml.DependencyObject,System.Collections.Generic.IDictionary$2(System.Object,System.Object),System.Collections.IDictionary,System.Collections.ICollection,System.ComponentModel.ISupportInitialize,Windows.UI.Xaml.Markup.INameScope],
        statics: {
            fields: {
                DummyInheritanceContext: null
            },
            ctors: {
                init: function () {
                    this.DummyInheritanceContext = new Windows.UI.Xaml.DependencyObject();
                }
            }
        },
        fields: {
            INTERNAL_HasImplicitStyles: false,
            _flags: 0,
            _baseDictionary: null,
            _mergedDictionaries: null,
            _themeDictionaries: null,
            _parentDictionary: null,
            _ownerFEs: null,
            _ownerApps: null,
            _inheritanceContext$1: null,
            /**
             * Gets or sets a Uniform Resource Identifier (URI) that provides the source
             location of a merged resource dictionary.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.ResourceDictionary
             * @function Source
             * @type System.Uri
             */
            Source: null,
            _nameScopeDictionary: null
        },
        props: {
            /**
             * Gets a collection of merged resource dictionaries that are 
             specifically keyed and composed to address theme scenarios
             *
             * @instance
             * @public
             * @readonly
             * @memberof Windows.UI.Xaml.ResourceDictionary
             * @function ThemeDictionaries
             * @type System.Collections.Generic.IDictionary$2
             */
            ThemeDictionaries: {
                get: function () {
                    if (this._themeDictionaries == null) {
                        this._themeDictionaries = new (System.Collections.Generic.Dictionary$2(System.Object,Windows.UI.Xaml.ResourceDictionary)).ctor();
                    }
                    return this._themeDictionaries;
                }
            },
            /**
             * Gets the number of elements contained in the collection.
             *
             * @instance
             * @public
             * @readonly
             * @memberof Windows.UI.Xaml.ResourceDictionary
             * @function Count
             * @type number
             * @return  {[type]}        The number of elements contained in the collection.
             */
            Count: {
                get: function () {
                    return this._baseDictionary.Count;
                }
            },
            /**
             * Gets a value indicating whether the {@link } has a fixed size.
             *
             * @instance
             * @public
             * @readonly
             * @memberof Windows.UI.Xaml.ResourceDictionary
             * @function IsFixedSize
             * @type boolean
             * @return  {[type]}        Always returns false.
             */
            IsFixedSize: {
                get: function () {
                    return false;
                }
            },
            /**
             * Gets a value indicating whether the collection is read-only.
             *
             * @instance
             * @public
             * @readonly
             * @memberof Windows.UI.Xaml.ResourceDictionary
             * @function IsReadOnly
             * @type boolean
             * @return  {[type]}        Always returns false
             */
            IsReadOnly: {
                get: function () {
                    return false;
                }
            },
            /**
             * Gets an {@link } object containing the keys of the {@link }.
             *
             * @instance
             * @public
             * @readonly
             * @memberof Windows.UI.Xaml.ResourceDictionary
             * @function Keys
             * @type System.Collections.ICollection
             */
            Keys: {
                get: function () {
                    return this._baseDictionary.Keys;
                }
            },
            /**
             * List of ResourceDictionaries merged into this Resource Dictionary
             *
             * @instance
             * @public
             * @readonly
             * @memberof Windows.UI.Xaml.ResourceDictionary
             * @function MergedDictionaries
             * @type Windows.UI.Xaml.PresentationFrameworkCollection$1
             */
            MergedDictionaries: {
                get: function () {
                    if (this._mergedDictionaries == null) {
                        this._mergedDictionaries = new Windows.UI.Xaml.ResourceDictionaryCollection(this);
                        this._mergedDictionaries.addCollectionChanged(Bridge.fn.cacheBind(this, this.OnMergedDictionariesChanged));
                    }
                    return this._mergedDictionaries;
                }
            },
            /**
             * Gets an {@link } object containing the values of the {@link }.
             *
             * @instance
             * @public
             * @readonly
             * @memberof Windows.UI.Xaml.ResourceDictionary
             * @function Values
             * @type System.Collections.ICollection
             */
            Values: {
                get: function () {
                    return this._baseDictionary.Values;
                }
            },
            System$Collections$ICollection$SyncRoot: {
                get: function () {
                    throw new System.NotImplementedException.$ctor1("The method or operation is not implemented.");
                }
            },
            System$Collections$ICollection$IsSynchronized: {
                get: function () {
                    return false;
                }
            },
            System$Collections$Generic$ICollection$1$System$Collections$Generic$KeyValuePair$2$System$Object$System$Object$Count: {
                get: function () {
                    throw new System.NotImplementedException.$ctor1("The method or operation is not implemented.");
                }
            },
            System$Collections$Generic$IDictionary$2$System$Object$System$Object$Keys: {
                get: function () {
                    throw new System.NotImplementedException.$ctor1("The method or operation is not implemented.");
                }
            },
            System$Collections$Generic$IDictionary$2$System$Object$System$Object$Values: {
                get: function () {
                    throw new System.NotImplementedException.$ctor1("The method or operation is not implemented.");
                }
            },
            InheritanceContext$1: {
                get: function () {
                    return this._inheritanceContext$1;
                }
            },
            IsInitialized: {
                get: function () {
                    return this.ReadPrivateFlag(Windows.UI.Xaml.ResourceDictionary.PrivateFlags.IsInitialized);
                },
                set: function (value) {
                    this.WritePrivateFlag(Windows.UI.Xaml.ResourceDictionary.PrivateFlags.IsInitialized, value);
                }
            },
            IsInitializePending: {
                get: function () {
                    return this.ReadPrivateFlag(Windows.UI.Xaml.ResourceDictionary.PrivateFlags.IsInitializePending);
                },
                set: function (value) {
                    this.WritePrivateFlag(Windows.UI.Xaml.ResourceDictionary.PrivateFlags.IsInitializePending, value);
                }
            },
            IsThemeDictionary: {
                get: function () {
                    return this.ReadPrivateFlag(Windows.UI.Xaml.ResourceDictionary.PrivateFlags.IsThemeDictionary);
                },
                set: function (value) {
                    if (this.IsThemeDictionary !== value) {
                        this.WritePrivateFlag(Windows.UI.Xaml.ResourceDictionary.PrivateFlags.IsThemeDictionary, value);
                        if (this._mergedDictionaries != null) {
                            for (var i = 0; i < this._mergedDictionaries.Count; i = (i + 1) | 0) {
                                this._mergedDictionaries.getItem(i).IsThemeDictionary = value;
                            }
                        }
                    }
                }
            },
            HasImplicitStyles: {
                get: function () {
                    return this.ReadPrivateFlag(Windows.UI.Xaml.ResourceDictionary.PrivateFlags.HasImplicitStyles);
                },
                set: function (value) {
                    this.WritePrivateFlag(Windows.UI.Xaml.ResourceDictionary.PrivateFlags.HasImplicitStyles, value);
                }
            },
            HasImplicitDataTemplates: {
                get: function () {
                    return this.ReadPrivateFlag(Windows.UI.Xaml.ResourceDictionary.PrivateFlags.HasImplicitDataTemplates);
                },
                set: function (value) {
                    this.WritePrivateFlag(Windows.UI.Xaml.ResourceDictionary.PrivateFlags.HasImplicitDataTemplates, value);
                }
            }
        },
        alias: [
            "getItem", "System$Collections$IDictionary$getItem",
            "setItem", "System$Collections$IDictionary$setItem",
            "getItem", "System$Collections$Generic$IDictionary$2$System$Object$System$Object$getItem",
            "setItem", "System$Collections$Generic$IDictionary$2$System$Object$System$Object$setItem",
            "Count", "System$Collections$ICollection$Count",
            "IsFixedSize", "System$Collections$IDictionary$IsFixedSize",
            "IsReadOnly", "System$Collections$IDictionary$IsReadOnly",
            "IsReadOnly", "System$Collections$Generic$ICollection$1$System$Collections$Generic$KeyValuePair$2$System$Object$System$Object$IsReadOnly",
            "Keys", "System$Collections$IDictionary$Keys",
            "Values", "System$Collections$IDictionary$Values",
            "add", "System$Collections$IDictionary$add",
            "add", "System$Collections$Generic$IDictionary$2$System$Object$System$Object$add",
            "clear", "System$Collections$IDictionary$clear",
            "clear", "System$Collections$Generic$ICollection$1$System$Collections$Generic$KeyValuePair$2$System$Object$System$Object$clear",
            "contains", "System$Collections$IDictionary$contains",
            "containsKey", "System$Collections$Generic$IDictionary$2$System$Object$System$Object$containsKey",
            "copyTo", "System$Collections$ICollection$copyTo",
            "GetEnumerator", "System$Collections$IDictionary$GetEnumerator",
            "remove", "System$Collections$IDictionary$remove",
            "BeginInit", "System$ComponentModel$ISupportInitialize$BeginInit",
            "EndInit", "System$ComponentModel$ISupportInitialize$EndInit",
            "System$Collections$Generic$IEnumerable$1$System$Collections$Generic$KeyValuePair$2$System$Object$System$Object$GetEnumerator", "System$Collections$Generic$IEnumerable$1$GetEnumerator",
            "FindName", "Windows$UI$Xaml$Markup$INameScope$FindName",
            "RegisterName", "Windows$UI$Xaml$Markup$INameScope$RegisterName",
            "UnregisterName", "Windows$UI$Xaml$Markup$INameScope$UnregisterName"
        ],
        ctors: {
            init: function () {
                this.INTERNAL_HasImplicitStyles = false;
                this._flags = 0;
                this._nameScopeDictionary = new (System.Collections.Generic.Dictionary$2(System.String,System.Object)).ctor();
            },
            /**
             * Constructor for ResourceDictionary
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.ResourceDictionary
             * @memberof Windows.UI.Xaml.ResourceDictionary
             * @return  {void}
             */
            ctor: function () {
                this.$initialize();
                Windows.UI.Xaml.DependencyObject.ctor.call(this);
                this._baseDictionary = new (System.Collections.Generic.Dictionary$2(System.Object,System.Object)).ctor();
                this.IsThemeDictionary = CSHTML5.Internal.INTERNAL_XamlResourcesHandler.IsSystemResourcesParsing;

                this.CanBeInheritanceContext = false;
                this.IsInheritanceContextSealed = true;
            }
        },
        methods: {
            /**
             * Gets or sets the value associated with the specified key.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.ResourceDictionary
             * @memberof Windows.UI.Xaml.ResourceDictionary
             * @throws key is null.
             * @param   {System.Object}    key
             * @return  {System.Object}
             */
            getItem: function (key) {
                return this.GetItem(key);
            },
            /**
             * Gets or sets the value associated with the specified key.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.ResourceDictionary
             * @memberof Windows.UI.Xaml.ResourceDictionary
             * @throws key is null.
             * @param   {System.Object}    key      
             * @param   {System.Object}    value
             * @return  {void}
             */
            setItem: function (key, value) {
                this.SealValue(value);

                this.SetItem(key, value);
            },
            /**
             * Adds an item to the {@link }.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.ResourceDictionary
             * @memberof Windows.UI.Xaml.ResourceDictionary
             * @throws Attempted to add null as a value.
             * @throws Attempted to add an item with a key that already exists in this {@link ArgumentException}.-or-Attempted
             to use a key that is not a string.
             * @param   {System.Object}    key      The string key of the item to add.
             * @param   {System.Object}    value    The item value to add.
             * @return  {void}
             */
            add: function (key, value) {
                this.AddInternal(key, value, true);
            },
            Add: function (key, value) {
                this.AddInternal(key, value, false);
            },
            System$Collections$Generic$ICollection$1$System$Collections$Generic$KeyValuePair$2$System$Object$System$Object$add: function (item) {
                this.AddInternal(item.key, item.value, true);
            },
            /**
             * Removes all items from this {@link }.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.ResourceDictionary
             * @memberof Windows.UI.Xaml.ResourceDictionary
             * @return  {void}
             */
            clear: function () {
                if (this.Count > 0) {
                    this.RemoveInheritanceContextFromValues();

                    this._baseDictionary.clear();

                    this.NotifyOwners(Windows.UI.Xaml.ResourcesChangeInfo.CatastrophicDictionaryChangeInfo.$clone());
                }
            },
            /**
             * Returns a value that indicates whether a specified key exists in the {@link }.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.ResourceDictionary
             * @memberof Windows.UI.Xaml.ResourceDictionary
             * @throws key is null.
             * @param   {System.Object}    key    The key to check for in the {@link }.
             * @return  {boolean}                 true if an item with that key exists in the {@link };
             otherwise, false.
             */
            contains: function (key) {
                var result = this._baseDictionary.containsKey(key);

                if (this._mergedDictionaries != null) {
                    for (var i = (this._mergedDictionaries.Count - 1) | 0; (i > -1) && !result; i = (i - 1) | 0) {
                        result = this._mergedDictionaries.getItem(i).contains(key);
                    }
                }

                return result;
            },
            System$Collections$Generic$ICollection$1$System$Collections$Generic$KeyValuePair$2$System$Object$System$Object$contains: function (item) {
                return this.contains(item.key);
            },
            /**
             * {@link }
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.ResourceDictionary
             * @memberof Windows.UI.Xaml.ResourceDictionary
             * @param   {System.Object}    key
             * @return  {boolean}
             */
            containsKey: function (key) {
                return this.contains(key);
            },
            /**
             * Copies the elements of the {@link } to an {@link },
             starting at a particular {@link } index.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.ResourceDictionary
             * @memberof Windows.UI.Xaml.ResourceDictionary
             * @param   {Array}     array    The one-dimensional {@link } that is the destination of the elements copied
             from {@link }. The System.Array must have zero-based
             indexing.
             * @param   {number}    index    The zero-based index in array at which copying begins.
             * @return  {void}
             */
            copyTo: function (array, index) {
                throw new System.NotImplementedException.ctor();
            },
            System$Collections$Generic$ICollection$1$System$Collections$Generic$KeyValuePair$2$System$Object$System$Object$copyTo: function (array, arrayIndex) {
                throw new System.NotImplementedException.$ctor1("The method or operation is not implemented.");
            },
            /**
             * Exposes the enumerator, which supports a simple iteration over a non-generic
             collection.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.ResourceDictionary
             * @memberof Windows.UI.Xaml.ResourceDictionary
             * @return  {System.Collections.IDictionaryEnumerator}        An enumerator that can be used to iterate through the collection.
             */
            GetEnumerator: function () {
                return this._baseDictionary.GetEnumerator().$clone();
            },
            System$Collections$Generic$IEnumerable$1$System$Collections$Generic$KeyValuePair$2$System$Object$System$Object$GetEnumerator: function () {
                return Bridge.getEnumerator(Bridge.cast(this._baseDictionary, System.Collections.Generic.IEnumerable$1(System.Collections.Generic.KeyValuePair$2(System.Object,System.Object))), System.Collections.Generic.KeyValuePair$2(System.Object,System.Object));
            },
            System$Collections$IEnumerable$GetEnumerator: function () {
                return this.GetEnumerator();
            },
            /**
             * Removes a specific item from the {@link }.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.ResourceDictionary
             * @memberof Windows.UI.Xaml.ResourceDictionary
             * @throws key is null.
             * @param   {System.Object}    key    The key of the item to remove.
             * @return  {void}
             */
            remove: function (key) {
                this.RemoveInheritanceContext$1(this._baseDictionary.getItem(key));

                this._baseDictionary.remove(key);

                this.NotifyOwners(new Windows.UI.Xaml.ResourcesChangeInfo.$ctor2(key));
            },
            /**
             * Removes a specific item from the {@link }.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.ResourceDictionary
             * @memberof Windows.UI.Xaml.ResourceDictionary
             * @throws key is null.
             * @param   {string}    key    The string key of the item to remove.
             * @return  {void}
             */
            Remove: function (key) {
                this.remove(key);
            },
            System$Collections$Generic$IDictionary$2$System$Object$System$Object$remove: function (key) {
                var count = this.Count;
                this.remove(key);
                return this.Count < count;
            },
            System$Collections$Generic$ICollection$1$System$Collections$Generic$KeyValuePair$2$System$Object$System$Object$remove: function (item) {
                var count = this.Count;
                this.remove(item.key);
                return this.Count < count;
            },
            /**
             * Mark the begining of the Init phase
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.ResourceDictionary
             * @memberof Windows.UI.Xaml.ResourceDictionary
             * @return  {void}
             */
            BeginInit: function () {
                if (this.IsInitializePending) {
                    throw new System.InvalidOperationException.$ctor1("Cannot have nested BeginInit calls on the same instance.");
                }

                this.IsInitializePending = true;
                this.IsInitialized = false;
            },
            /**
             * Fire Invalidation at the end of Init phase
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.ResourceDictionary
             * @memberof Windows.UI.Xaml.ResourceDictionary
             * @return  {void}
             */
            EndInit: function () {
                if (!this.IsInitializePending) {
                    throw new System.InvalidOperationException.$ctor1("Must call BeginInit before EndInit.");
                }
                System.Diagnostics.Debug.Assert$1(this.IsInitialized === false, "Dictionary should not be initialized when EndInit is called");

                this.IsInitializePending = false;
                this.IsInitialized = true;

                this.NotifyOwners(new Windows.UI.Xaml.ResourcesChangeInfo.$ctor3(null, this));
            },
            System$Collections$Generic$IDictionary$2$System$Object$System$Object$tryGetValue: function (key, value) {
                return ((value.v = this.GetItem(key))) != null;
            },
            AddOwner: function (owner) {
                if (this._inheritanceContext$1 == null) {
                    var inheritanceContext = Bridge.as(owner, Windows.UI.Xaml.DependencyObject);

                    if (inheritanceContext != null) {
                        this._inheritanceContext$1 = inheritanceContext;

                        this.AddInheritanceContextToValues();
                    } else {
                        this._inheritanceContext$1 = Windows.UI.Xaml.ResourceDictionary.DummyInheritanceContext;

                        this.AddInheritanceContextToValues();

                    }
                }

                var fe = Bridge.as(owner, Windows.UI.Xaml.FrameworkElement);
                if (fe != null) {
                    if (this._ownerFEs == null) {
                        this._ownerFEs = new (System.Collections.Generic.List$1(Windows.UI.Xaml.FrameworkElement)).$ctor2(1);
                    } else if (this._ownerFEs.contains(fe) && this.ContainsCycle(this)) {
                        throw new System.InvalidOperationException.$ctor1("The merged dictionary is invalid. Either a ResourceDictionary is being placed into its own MergedDictionaries collection or a it is being added to the same MergedDictionary collection twice.");
                    }

                    if (this.HasImplicitStyles) {
                        fe.ShouldLookupImplicitStyles = true;
                    }

                    this._ownerFEs.add(fe);
                } else {
                    var app = Bridge.as(owner, Windows.UI.Xaml.Application);
                    if (app != null) {
                        if (this._ownerApps == null) {
                            this._ownerApps = new (System.Collections.Generic.List$1(Windows.UI.Xaml.Application)).$ctor2(1);
                        } else if (this._ownerApps.contains(app) && this.ContainsCycle(this)) {
                            throw new System.InvalidOperationException.$ctor1("The merged dictionary is invalid. Either a ResourceDictionary is being placed into its own MergedDictionaries collection or a it is being added to the same MergedDictionary collection twice.");
                        }

                        if (this.HasImplicitStyles) {
                            app.HasImplicitStylesInResources = true;
                        }

                        this._ownerApps.add(app);
                    }
                }

                this.AddOwnerToAllMergedDictionaries(owner);

                this.TryInitialize();
            },
            RemoveOwner: function (owner) {
                var fe = Bridge.as(owner, Windows.UI.Xaml.FrameworkElement);
                if (fe != null) {
                    if (this._ownerFEs != null) {
                        this._ownerFEs.remove(fe);

                        if (this._ownerFEs.Count === 0) {
                            this._ownerFEs = null;
                        }
                    }
                } else {
                    var app = Bridge.as(owner, Windows.UI.Xaml.Application);
                    if (app != null) {
                        if (this._ownerApps != null) {
                            this._ownerApps.remove(app);

                            if (this._ownerApps.Count === 0) {
                                this._ownerApps = null;
                            }
                        }
                    }
                }

                if (Bridge.referenceEquals(owner, this.InheritanceContext$1)) {
                    this.RemoveInheritanceContextFromValues();
                    this._inheritanceContext$1 = null;
                }

                this.RemoveOwnerFromAllMergedDictionaries(owner);
            },
            ContainsOwner: function (owner) {
                var fe = Bridge.as(owner, Windows.UI.Xaml.FrameworkElement);
                if (fe != null) {
                    return (this._ownerFEs != null && this._ownerFEs.contains(fe));
                } else {
                    var app = Bridge.as(owner, Windows.UI.Xaml.Application);
                    if (app != null) {
                        return (this._ownerApps != null && this._ownerApps.contains(app));
                    }
                }

                return false;
            },
            TryInitialize: function () {
                if (!this.IsInitializePending && !this.IsInitialized) {
                    this.IsInitialized = true;
                }
            },
            NotifyOwners: function (info) {
                var $t, $t1;
                var shouldInvalidate = this.IsInitialized;
                var hasImplicitStyles = info.IsResourceAddOperation && this.HasImplicitStyles;


                if (shouldInvalidate || hasImplicitStyles) {
                    if (this._ownerFEs != null) {
                        $t = Bridge.getEnumerator(this._ownerFEs);
                        try {
                            while ($t.moveNext()) {
                                var fe = $t.Current;
                                if (fe != null) {
                                    if (hasImplicitStyles) {
                                        fe.ShouldLookupImplicitStyles = true;
                                    }

                                }
                            }
                        } finally {
                            if (Bridge.is($t, System.IDisposable)) {
                                $t.System$IDisposable$Dispose();
                            }
                        }
                    }

                    if (this._ownerApps != null) {
                        $t1 = Bridge.getEnumerator(this._ownerApps);
                        try {
                            while ($t1.moveNext()) {
                                var app = $t1.Current;
                                if (app != null) {
                                    if (hasImplicitStyles) {
                                        app.HasImplicitStylesInResources = true;
                                    }

                                }
                            }
                        } finally {
                            if (Bridge.is($t1, System.IDisposable)) {
                                $t1.System$IDisposable$Dispose();
                            }
                        }
                    }
                }
            },
            OnMergedDictionariesChanged: function (sender, e) {
                var oldDictionaries = null;
                var newDictionaries = null;
                var mergedDictionary;
                var info = new Windows.UI.Xaml.ResourcesChangeInfo();

                if (e.Action !== System.Collections.Specialized.NotifyCollectionChangedAction.Reset) {
                    System.Diagnostics.Debug.Assert$1((e.NewItems != null && System.Array.getCount(e.NewItems) > 0) || (e.OldItems != null && System.Array.getCount(e.OldItems) > 0), "The NotifyCollectionChanged event fired when no dictionaries were added or removed");


                    if (e.Action === System.Collections.Specialized.NotifyCollectionChangedAction.Remove || e.Action === System.Collections.Specialized.NotifyCollectionChangedAction.Replace) {
                        oldDictionaries = new (System.Collections.Generic.List$1(Windows.UI.Xaml.ResourceDictionary)).$ctor2(System.Array.getCount(e.OldItems));

                        for (var i = 0; i < System.Array.getCount(e.OldItems); i = (i + 1) | 0) {
                            mergedDictionary = Bridge.cast(System.Array.getItem(e.OldItems, i), Windows.UI.Xaml.ResourceDictionary);
                            oldDictionaries.add(mergedDictionary);

                            this.RemoveParentOwners(mergedDictionary);
                        }
                    }

                    if (e.Action === System.Collections.Specialized.NotifyCollectionChangedAction.Add || e.Action === System.Collections.Specialized.NotifyCollectionChangedAction.Replace) {
                        newDictionaries = new (System.Collections.Generic.List$1(Windows.UI.Xaml.ResourceDictionary)).$ctor2(System.Array.getCount(e.NewItems));

                        for (var i1 = 0; i1 < System.Array.getCount(e.NewItems); i1 = (i1 + 1) | 0) {
                            mergedDictionary = Bridge.cast(System.Array.getItem(e.NewItems, i1), Windows.UI.Xaml.ResourceDictionary);
                            newDictionaries.add(mergedDictionary);

                            if (!this.HasImplicitStyles && mergedDictionary.HasImplicitStyles) {
                                this.HasImplicitStyles = true;
                            }

                            if (!this.HasImplicitDataTemplates && mergedDictionary.HasImplicitDataTemplates) {
                                this.HasImplicitDataTemplates = true;
                            }

                            if (this.IsThemeDictionary) {
                                mergedDictionary.IsThemeDictionary = true;
                            }

                            this.PropagateParentOwners(mergedDictionary);
                        }
                    }

                    info = new Windows.UI.Xaml.ResourcesChangeInfo.$ctor1(oldDictionaries, newDictionaries, false, false, null);
                } else {
                    info = Windows.UI.Xaml.ResourcesChangeInfo.CatastrophicDictionaryChangeInfo.$clone();
                }


                this.NotifyOwners(info.$clone());
            },
            /**
             * Adds the given owner to all merged dictionaries of this ResourceDictionary
             *
             * @instance
             * @private
             * @this Windows.UI.Xaml.ResourceDictionary
             * @memberof Windows.UI.Xaml.ResourceDictionary
             * @param   {System.Object}    owner
             * @return  {void}
             */
            AddOwnerToAllMergedDictionaries: function (owner) {
                if (this._mergedDictionaries != null) {
                    for (var i = 0; i < this._mergedDictionaries.Count; i = (i + 1) | 0) {
                        this._mergedDictionaries.getItem(i).AddOwner(owner);
                    }
                }
            },
            /**
             * Removes the given owner to all merged dictionaries of this ResourceDictionary
             *
             * @instance
             * @private
             * @this Windows.UI.Xaml.ResourceDictionary
             * @memberof Windows.UI.Xaml.ResourceDictionary
             * @param   {System.Object}    owner
             * @return  {void}
             */
            RemoveOwnerFromAllMergedDictionaries: function (owner) {
                if (this._mergedDictionaries != null) {
                    for (var i = 0; i < this._mergedDictionaries.Count; i = (i + 1) | 0) {
                        this._mergedDictionaries.getItem(i).RemoveOwner(owner);
                    }
                }
            },
            /**
             * This sends down the owners of this ResourceDictionary into the given
             merged dictionary.  We do this because whenever a merged dictionary
             changes it should invalidate all owners of its parent ResourceDictionary.
             Note that AddOwners throw if the merged dictionary already has one of the
             parent's owners.  This implies that either we're putting a dictionary
             into its own MergedDictionaries collection or we're putting the same
             dictionary into the collection twice, neither of which are legal.
             *
             * @instance
             * @private
             * @this Windows.UI.Xaml.ResourceDictionary
             * @memberof Windows.UI.Xaml.ResourceDictionary
             * @param   {Windows.UI.Xaml.ResourceDictionary}    mergedDictionary
             * @return  {void}
             */
            PropagateParentOwners: function (mergedDictionary) {
                var $t, $t1;
                if (this._ownerFEs != null) {
                    System.Diagnostics.Debug.Assert(this._ownerFEs.Count > 0);

                    if (mergedDictionary._ownerFEs == null) {
                        mergedDictionary._ownerFEs = new (System.Collections.Generic.List$1(Windows.UI.Xaml.FrameworkElement)).$ctor2(this._ownerFEs.Count);
                    }

                    $t = Bridge.getEnumerator(this._ownerFEs);
                    try {
                        while ($t.moveNext()) {
                            var fe = $t.Current;
                            if (fe != null) {
                                mergedDictionary.AddOwner(fe);
                            }
                        }
                    } finally {
                        if (Bridge.is($t, System.IDisposable)) {
                            $t.System$IDisposable$Dispose();
                        }
                    }
                }

                if (this._ownerApps != null) {
                    System.Diagnostics.Debug.Assert(this._ownerApps.Count > 0);

                    if (mergedDictionary._ownerApps == null) {
                        mergedDictionary._ownerApps = new (System.Collections.Generic.List$1(Windows.UI.Xaml.Application)).$ctor2(this._ownerApps.Count);
                    }

                    $t1 = Bridge.getEnumerator(this._ownerApps);
                    try {
                        while ($t1.moveNext()) {
                            var app = $t1.Current;
                            if (app != null) {
                                mergedDictionary.AddOwner(app);
                            }
                        }
                    } finally {
                        if (Bridge.is($t1, System.IDisposable)) {
                            $t1.System$IDisposable$Dispose();
                        }
                    }
                }
            },
            /**
             * Removes the owners of this ResourceDictionary from the given
             merged dictionary.  The merged dictionary will be left with
             whatever owners it had before being merged.
             *
             * @instance
             * @this Windows.UI.Xaml.ResourceDictionary
             * @memberof Windows.UI.Xaml.ResourceDictionary
             * @param   {Windows.UI.Xaml.ResourceDictionary}    mergedDictionary
             * @return  {void}
             */
            RemoveParentOwners: function (mergedDictionary) {
                var $t, $t1;
                if (this._ownerFEs != null) {
                    $t = Bridge.getEnumerator(this._ownerFEs);
                    try {
                        while ($t.moveNext()) {
                            var fe = $t.Current;
                            mergedDictionary.RemoveOwner(fe);
                        }
                    } finally {
                        if (Bridge.is($t, System.IDisposable)) {
                            $t.System$IDisposable$Dispose();
                        }
                    }
                }

                if (this._ownerApps != null) {
                    System.Diagnostics.Debug.Assert(this._ownerApps.Count > 0);

                    $t1 = Bridge.getEnumerator(this._ownerApps);
                    try {
                        while ($t1.moveNext()) {
                            var app = $t1.Current;
                            mergedDictionary.RemoveOwner(app);
                        }
                    } finally {
                        if (Bridge.is($t1, System.IDisposable)) {
                            $t1.System$IDisposable$Dispose();
                        }
                    }
                }
            },
            ContainsCycle: function (origin) {
                for (var i = 0; i < this.MergedDictionaries.Count; i = (i + 1) | 0) {
                    var mergedDictionary = this.MergedDictionaries.getItem(i);
                    if (Bridge.referenceEquals(mergedDictionary, origin) || mergedDictionary.ContainsCycle(origin)) {
                        return true;
                    }
                }

                return false;
            },
            SealValue: function (value) {
                var inheritanceContext = this.InheritanceContext$1;
                if (inheritanceContext != null) {
                    this.AddInheritanceContext$1(inheritanceContext, value);
                }

            },
            AddInheritanceContext$1: function (inheritanceContext, value) {
                if (inheritanceContext.ProvideSelfAsInheritanceContext(value, null)) {
                    var doValue = Bridge.as(value, Windows.UI.Xaml.DependencyObject);
                    if (doValue != null) {
                        doValue.IsInheritanceContextSealed = true;
                    }
                }
            },
            AddInheritanceContextToValues: function () {
                var $t;
                var inheritanceContext = this.InheritanceContext$1;

                var count = this._baseDictionary.Count;
                if (count > 0) {
                    var values = System.Array.init(count, null, System.Object);
                    this._baseDictionary.Values.copyTo(values, 0);

                    $t = Bridge.getEnumerator(values);
                    try {
                        while ($t.moveNext()) {
                            var value = $t.Current;
                            this.AddInheritanceContext$1(inheritanceContext, value);
                        }
                    } finally {
                        if (Bridge.is($t, System.IDisposable)) {
                            $t.System$IDisposable$Dispose();
                        }
                    }
                }
            },
            RemoveInheritanceContext$1: function (value) {
                var doValue = Bridge.as(value, Windows.UI.Xaml.DependencyObject);
                var inheritanceContext = this.InheritanceContext$1;

                if (doValue != null && inheritanceContext != null && doValue.IsInheritanceContextSealed && Bridge.referenceEquals(doValue.InheritanceContext, inheritanceContext)) {
                    doValue.IsInheritanceContextSealed = false;
                    inheritanceContext.RemoveSelfAsInheritanceContext$1(doValue, null);
                }
            },
            RemoveInheritanceContextFromValues: function () {
                var $t;
                $t = Bridge.getEnumerator(this._baseDictionary.Values);
                try {
                    while ($t.moveNext()) {
                        var value = $t.Current;
                        this.RemoveInheritanceContext$1(value);
                    }
                } finally {
                    if (Bridge.is($t, System.IDisposable)) {
                        $t.System$IDisposable$Dispose();
                    }
                }
            },
            GetItem: function (key) {
                var value = { };
                if (this._baseDictionary.tryGetValue(key, value)) {
                    return value.v;
                } else {
                    if (this._mergedDictionaries != null) {
                        for (var i = (this._mergedDictionaries.Count - 1) | 0; (i > -1); i = (i - 1) | 0) {
                            value.v = this._mergedDictionaries.getItem(i).GetItem(key);
                            if (value.v != null) {
                                break;
                            }
                        }
                    }
                }
                return value.v;
            },
            SetItem: function (key, value) {
                if (value == null) {
                    throw new System.NotSupportedException.$ctor1("Null value not supported in a ResourceDictionary.");
                }

                var oldItem = { };
                if (!this._baseDictionary.tryGetValue(key, oldItem) || !Bridge.referenceEquals(oldItem.v, value)) {
                    this._baseDictionary.setItem(key, value);

                    this.UpdateHasImplicitStyles(key);

                    this.UpdateHasImplicitDataTemplates(key);

                    this.NotifyOwners(new Windows.UI.Xaml.ResourcesChangeInfo.$ctor2(key));
                }
            },
            AddInternal: function (key, value, updateFlags) {
                this.SealValue(value);

                this._baseDictionary.add(key, value);

                if (updateFlags) {
                    this.UpdateHasImplicitStyles(key);

                    this.UpdateHasImplicitDataTemplates(key);
                }

                this.NotifyOwners(new Windows.UI.Xaml.ResourcesChangeInfo.$ctor2(key));
            },
            UpdateHasImplicitStyles: function (key) {
                if (!this.HasImplicitStyles) {
                    this.HasImplicitStyles = ((Bridge.as(key, System.Type)) != null);
                }
            },
            UpdateHasImplicitDataTemplates: function (key) { },
            WritePrivateFlag: function (bit, value) {
                if (value) {
                    this._flags |= bit & 255;
                } else {
                    this._flags &= (~bit) & 255;
                }
            },
            ReadPrivateFlag: function (bit) {
                return (((this._flags & bit) & 255)) !== 0;
            },
            /**
             * Finds the UIElement with the specified name.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.ResourceDictionary
             * @memberof Windows.UI.Xaml.ResourceDictionary
             * @param   {string}           name    The name to look for.
             * @return  {System.Object}            The object with the specified name if any; otherwise null.
             */
            FindName: function (name) {
                if (this._nameScopeDictionary.containsKey(name)) {
                    return this._nameScopeDictionary.getItem(name);
                } else {
                    return null;
                }
            },
            RegisterName: function (name, scopedElement) {
                if (this._nameScopeDictionary.containsKey(name) && !Bridge.referenceEquals(this._nameScopeDictionary.getItem(name), scopedElement)) {
                    throw new System.ArgumentException.$ctor1(System.String.format("Cannot register duplicate name '{0}' in this scope.", [name]));
                }

                this._nameScopeDictionary.setItem(name, scopedElement);
            },
            UnregisterName: function (name) {
                if (!this._nameScopeDictionary.containsKey(name)) {
                    throw new System.ArgumentException.$ctor1(System.String.format("Name '{0}' was not found.", [name]));
                }

                this._nameScopeDictionary.remove(name);
            }
        }
    });

    
    Bridge.define("CSHTML5.Native.Html.Controls.CircleElement", {
        inherits: [CSHTML5.Native.Html.Controls.HtmlCanvasElement],
        fields: {
            /**
             * Diameter of the circle == Width == Height
             *
             * @instance
             * @public
             * @memberof CSHTML5.Native.Html.Controls.CircleElement
             * @type number
             */
            Diameter: 0
        },
        props: {
            /**
             * Radius of the circle
             *
             * @instance
             * @public
             * @memberof CSHTML5.Native.Html.Controls.CircleElement
             * @function Radius
             * @type number
             */
            Radius: {
                get: function () {
                    return this.Diameter / 2;
                },
                set: function (value) {
                    this.Diameter = 2.0 * value;
                }
            }
        },
        ctors: {
            /**
             * Default circle constructor
             *
             * @instance
             * @public
             * @this CSHTML5.Native.Html.Controls.CircleElement
             * @memberof CSHTML5.Native.Html.Controls.CircleElement
             * @return  {void}
             */
            ctor: function () {
                this.$initialize();
                CSHTML5.Native.Html.Controls.HtmlCanvasElement.ctor.call(this);
                this.Diameter = 0;
            },
            /**
             * Constructor with specified position and size
             *
             * @instance
             * @public
             * @this CSHTML5.Native.Html.Controls.CircleElement
             * @memberof CSHTML5.Native.Html.Controls.CircleElement
             * @param   {number}    X         X position of the centre
             * @param   {number}    Y         Y position of the centre
             * @param   {number}    Radius    Radius of the circle
             * @return  {void}
             */
            $ctor1: function (X, Y, Radius) {
                this.$initialize();
                CSHTML5.Native.Html.Controls.HtmlCanvasElement.$ctor1.call(this, X, Y);
                this.Radius = Radius;
            }
        },
        methods: {
            /**
             * Draws the circle
             *
             * @instance
             * @public
             * @override
             * @this CSHTML5.Native.Html.Controls.CircleElement
             * @memberof CSHTML5.Native.Html.Controls.CircleElement
             * @param   {CSHTML5.Native.Html.Controls.ElementStyle}    currentDrawingStyle    Draw style used for last element (can allow optimizations, null if unknown)
             * @param   {System.Object}                                jsContext2d            Canvas 2d javascript context
             * @param   {number}                                       xParent                X position of the parent element
             * @param   {number}                                       yParent                Y position of the parent element
             * @return  {CSHTML5.Native.Html.Controls.ElementStyle}                           Draw style used for this element (can be null)
             */
            Draw: function (currentDrawingStyle, jsContext2d, xParent, yParent) {
                if (xParent === void 0) { xParent = 0.0; }
                if (yParent === void 0) { yParent = 0.0; }
                if (this.Visibility === Windows.UI.Xaml.Visibility.Visible) {
                    currentDrawingStyle = this.ApplyStyle(currentDrawingStyle, jsContext2d);
                    jsContext2d.beginPath();
                    jsContext2d.arc(Bridge.box(this.X + xParent, System.Double, System.Double.format, System.Double.getHashCode), Bridge.box(this.Y + yParent, System.Double, System.Double.format, System.Double.getHashCode), Bridge.box(this.Radius, System.Double, System.Double.format, System.Double.getHashCode), Bridge.box(0.0, System.Double, System.Double.format, System.Double.getHashCode), Bridge.box(6.2831853071795862, System.Double, System.Double.format, System.Double.getHashCode), Bridge.box(0, System.Int32));
                    jsContext2d.fill();
                }

                return currentDrawingStyle;
            },
            IsPointed: function (x, y) {
                return (x - this.X) * (x - this.X) + (y - this.Y) * (y - this.Y) <= this.Radius * this.Radius;
            }
        }
    });

    
    Bridge.define("CSHTML5.Native.Html.Controls.ContainerElement", {
        inherits: [CSHTML5.Native.Html.Controls.HtmlCanvasElement],
        fields: {
            /**
             * Gets the collection of child elements of the container.
             *
             * @instance
             * @public
             * @memberof CSHTML5.Native.Html.Controls.ContainerElement
             * @type System.Collections.Generic.List$1
             */
            Children: null,
            /**
             * Width of the container
             *
             * @instance
             * @public
             * @memberof CSHTML5.Native.Html.Controls.ContainerElement
             * @type number
             */
            Width: 0,
            /**
             * Height of the container
             *
             * @instance
             * @public
             * @memberof CSHTML5.Native.Html.Controls.ContainerElement
             * @type number
             */
            Height: 0
        },
        ctors: {
            /**
             * Default constructor
             *
             * @instance
             * @public
             * @this CSHTML5.Native.Html.Controls.ContainerElement
             * @memberof CSHTML5.Native.Html.Controls.ContainerElement
             * @return  {void}
             */
            ctor: function () {
                this.$initialize();
                CSHTML5.Native.Html.Controls.HtmlCanvasElement.ctor.call(this);
                this.Children = new (System.Collections.Generic.List$1(CSHTML5.Native.Html.Controls.HtmlCanvasElement)).ctor();
            }
        },
        methods: {
            /**
             * Draws the container and its children only if the container is visible.
             *
             * @instance
             * @public
             * @override
             * @this CSHTML5.Native.Html.Controls.ContainerElement
             * @memberof CSHTML5.Native.Html.Controls.ContainerElement
             * @param   {CSHTML5.Native.Html.Controls.ElementStyle}    currentDrawingStyle    Draw style used for last element (can allow optimizations, null if unknown)
             * @param   {System.Object}                                jsContext2d            Canvas 2d javascript context
             * @param   {number}                                       xParent                X position of the parent element
             * @param   {number}                                       yParent                Y position of the parent element
             * @return  {CSHTML5.Native.Html.Controls.ElementStyle}                           Draw style used for this element (can be null)
             */
            Draw: function (currentDrawingStyle, jsContext2d, xParent, yParent) {
                var $t;
                if (xParent === void 0) { xParent = 0.0; }
                if (yParent === void 0) { yParent = 0.0; }
                if (this.Visibility === Windows.UI.Xaml.Visibility.Visible) {
                    currentDrawingStyle = this.ApplyStyle(currentDrawingStyle, jsContext2d);
                    jsContext2d.fillRect(Bridge.box(this.X + xParent, System.Double, System.Double.format, System.Double.getHashCode), Bridge.box(this.Y + yParent, System.Double, System.Double.format, System.Double.getHashCode), Bridge.box(this.Width, System.Double, System.Double.format, System.Double.getHashCode), Bridge.box(this.Height, System.Double, System.Double.format, System.Double.getHashCode));
                    jsContext2d.strokeRect(Bridge.box(this.X + xParent, System.Double, System.Double.format, System.Double.getHashCode), Bridge.box(this.Y + yParent, System.Double, System.Double.format, System.Double.getHashCode), Bridge.box(this.Width, System.Double, System.Double.format, System.Double.getHashCode), Bridge.box(this.Height, System.Double, System.Double.format, System.Double.getHashCode));

                    $t = Bridge.getEnumerator(this.Children);
                    try {
                        while ($t.moveNext()) {
                            var elem = $t.Current;
                            currentDrawingStyle = elem.Draw(currentDrawingStyle, jsContext2d, this.X + xParent, this.Y + yParent);
                        }
                    } finally {
                        if (Bridge.is($t, System.IDisposable)) {
                            $t.System$IDisposable$Dispose();
                        }
                    }
                }
                return currentDrawingStyle;
            },
            IsPointed: function (x, y) {
                return x >= this.X && x < this.X + this.Width && y >= this.Y && y < this.Y + this.Height;
            }
        }
    });

    /**
     * @memberof Windows.UI.Xaml
     * @callback Windows.UI.Xaml.RoutedEventHandler
     * @param   {System.Object}                      sender    
     * @param   {Windows.UI.Xaml.RoutedEventArgs}    e
     * @return  {void}
     */

    /**
     * @memberof Windows.UI.Xaml.Input
     * @callback Windows.UI.Xaml.Input.KeyEventHandler
     * @param   {System.Object}                               sender    
     * @param   {Windows.UI.Xaml.Input.KeyRoutedEventArgs}    e
     * @return  {void}
     */

    /**
     * @memberof Windows.UI.Xaml.Input
     * @callback Windows.UI.Xaml.Input.RightTappedEventHandler
     * @param   {System.Object}                                       sender    
     * @param   {Windows.UI.Xaml.Input.RightTappedRoutedEventArgs}    e
     * @return  {void}
     */

    /**
     * @memberof Windows.UI.Xaml.Input
     * @callback Windows.UI.Xaml.Input.TappedEventHandler
     * @param   {System.Object}                                  sender    
     * @param   {Windows.UI.Xaml.Input.TappedRoutedEventArgs}    e
     * @return  {void}
     */

    /**
     * @memberof Windows.UI.Xaml.Input
     * @callback Windows.UI.Xaml.Input.PointerEventHandler
     * @param   {System.Object}                                   sender    
     * @param   {Windows.UI.Xaml.Input.PointerRoutedEventArgs}    e
     * @return  {void}
     */

    /**
     * UIElement is a base class for most of the objects that have visual appearance
     and can process basic input in a user interface.
     *
     * @abstract
     * @public
     * @class Windows.UI.Xaml.UIElement
     * @augments Windows.UI.Xaml.DependencyObject
     */
    Bridge.define("Windows.UI.Xaml.UIElement", {
        inherits: [Windows.UI.Xaml.DependencyObject],
        statics: {
            fields: {
                _random: null,
                ClipToBoundsProperty: null,
                EffectProperty: null,
                /**
                 * Identifies the RenderTransform dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.UIElement
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                RenderTransformProperty: null,
                RenderTransformOriginProperty: null,
                /**
                 * Identifies the UseLayoutRounding dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.UIElement
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                UseLayoutRoundingProperty: null,
                /**
                 * Identifies the Visibility dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.UIElement
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                VisibilityProperty: null,
                /**
                 * Identifies the IsVisible dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.UIElement
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                IsVisibleProperty: null,
                /**
                 * Identifies the Opacity dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.UIElement
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                OpacityProperty: null,
                /**
                 * Identifies the IsHitTestVisible property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.UIElement
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                IsHitTestVisibleProperty: null,
                /**
                 * Identifies the AllowDrop dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.UIElement
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                AllowDropProperty: null,
                /**
                 * Identifies the AllowScrollOnTouchMove dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.UIElement
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                AllowScrollOnTouchMoveProperty: null,
                ignoreMouseEvents: false,
                _ignoreMouseEventsTimer: null,
                PointerMovedEvent: null,
                PointerPressedEvent: null,
                PointerReleasedEvent: null,
                PointerEnteredEvent: null,
                PointerExitedEvent: null,
                TappedEvent: null,
                RightTappedEvent: null,
                KeyDownEvent: null,
                KeyUpEvent: null,
                GotFocusEvent: null,
                LostFocusEvent: null,
                GotFocusForIsTabStopEvent: null
            },
            ctors: {
                init: function () {
                    var $t;
                    this.ClipToBoundsProperty = Windows.UI.Xaml.DependencyProperty.Register("ClipToBounds", System.Boolean, Windows.UI.Xaml.UIElement, ($t = new Windows.UI.Xaml.PropertyMetadata.$ctor1(Bridge.box(false, System.Boolean, System.Boolean.toString)), $t.MethodToUpdateDom = Windows.UI.Xaml.UIElement.ClipToBounds_MethodToUpdateDom, $t));
                    this.EffectProperty = Windows.UI.Xaml.DependencyProperty.Register("Effect", System.Windows.Media.Effects.Effect, Windows.UI.Xaml.UIElement, ($t = new Windows.UI.Xaml.PropertyMetadata.$ctor2(null, Windows.UI.Xaml.UIElement.Effect_Changed), $t.CallPropertyChangedWhenLoadedIntoVisualTree = Windows.UI.Xaml.WhenToCallPropertyChangedEnum.IfPropertyIsSet, $t));
                    this.RenderTransformProperty = Windows.UI.Xaml.DependencyProperty.Register("RenderTransform", Windows.UI.Xaml.Media.Transform, Windows.UI.Xaml.UIElement, ($t = new Windows.UI.Xaml.PropertyMetadata.$ctor2(null, Windows.UI.Xaml.UIElement.RenderTransform_Changed), $t.CallPropertyChangedWhenLoadedIntoVisualTree = Windows.UI.Xaml.WhenToCallPropertyChangedEnum.IfPropertyIsSet, $t));
                    this.RenderTransformOriginProperty = Windows.UI.Xaml.DependencyProperty.Register("RenderTransformOrigin", Windows.Foundation.Point, Windows.UI.Xaml.UIElement, ($t = new Windows.UI.Xaml.PropertyMetadata.$ctor2(new Windows.Foundation.Point.$ctor1(0.0, 0.0).$clone(), Windows.UI.Xaml.UIElement.RenderTransformOrigin_Changed), $t.CallPropertyChangedWhenLoadedIntoVisualTree = Windows.UI.Xaml.WhenToCallPropertyChangedEnum.IfPropertyIsSet, $t));
                    this.UseLayoutRoundingProperty = Windows.UI.Xaml.DependencyProperty.Register("UseLayoutRounding", System.Boolean, Windows.UI.Xaml.UIElement, new Windows.UI.Xaml.PropertyMetadata.$ctor1(Bridge.box(false, System.Boolean, System.Boolean.toString)));
                    this.VisibilityProperty = Windows.UI.Xaml.DependencyProperty.Register("Visibility", Windows.UI.Xaml.Visibility, Windows.UI.Xaml.UIElement, new Windows.UI.Xaml.PropertyMetadata.$ctor2(Bridge.box(Windows.UI.Xaml.Visibility.Visible, Windows.UI.Xaml.Visibility, System.Enum.toStringFn(Windows.UI.Xaml.Visibility)), Windows.UI.Xaml.UIElement.Visibility_Changed));
                    this.IsVisibleProperty = Windows.UI.Xaml.DependencyProperty.Register("IsVisible", System.Boolean, Windows.UI.Xaml.UIElement, new Windows.UI.Xaml.PropertyMetadata.$ctor3(Bridge.box(true, System.Boolean, System.Boolean.toString), Windows.UI.Xaml.UIElement.OnIsVisiblePropertyChanged, Windows.UI.Xaml.UIElement.CoerceIsVisibleProperty));
                    this.OpacityProperty = Windows.UI.Xaml.DependencyProperty.Register("Opacity", System.Double, Windows.UI.Xaml.UIElement, ($t = new Windows.UI.Xaml.PropertyMetadata.$ctor1(Bridge.box(1.0, System.Double, System.Double.format, System.Double.getHashCode)), $t.GetCSSEquivalent = function (instance) {
                        var $t1;
                        return ($t1 = new Windows.UI.Xaml.CSSEquivalent(), $t1.Value = function (inst, value) {
                            return System.Double.format((Math.floor(System.Nullable.getValue(Bridge.cast(Bridge.unbox(value, System.Double), System.Double)) * 1000) / 1000));
                        }, $t1.Name = function (_o1) {
                                _o1.add("opacity");
                                return _o1;
                            }(new (System.Collections.Generic.List$1(System.String)).ctor()), $t1.ApplyAlsoWhenThereIsAControlTemplate = true, $t1);
                    }, $t));
                    this.IsHitTestVisibleProperty = Windows.UI.Xaml.DependencyProperty.Register("IsHitTestVisible", System.Boolean, Windows.UI.Xaml.UIElement, ($t = new Windows.UI.Xaml.PropertyMetadata.$ctor3(Bridge.box(true, System.Boolean, System.Boolean.toString), Windows.UI.Xaml.UIElement.OnIsHitTestVisiblePropertyChanged, Windows.UI.Xaml.UIElement.CoerceIsHitTestVisibleProperty), $t.MethodToUpdateDom = Windows.UI.Xaml.UIElement.IsHitTestVisible_MethodToUpdateDom, $t));
                    this.AllowDropProperty = Windows.UI.Xaml.DependencyProperty.Register("AllowDrop", System.Boolean, Windows.UI.Xaml.UIElement, new Windows.UI.Xaml.PropertyMetadata.$ctor1(Bridge.box(false, System.Boolean, System.Boolean.toString)));
                    this.AllowScrollOnTouchMoveProperty = Windows.UI.Xaml.DependencyProperty.Register("AllowScrollOnTouchMove", System.Boolean, Windows.UI.Xaml.UIElement, ($t = new Windows.UI.Xaml.PropertyMetadata.$ctor2(Bridge.box(true, System.Boolean, System.Boolean.toString), Windows.UI.Xaml.UIElement.AllowScrollOnTouchMove_Changed), $t.CallPropertyChangedWhenLoadedIntoVisualTree = Windows.UI.Xaml.WhenToCallPropertyChangedEnum.IfPropertyIsSet, $t));
                    this.ignoreMouseEvents = false;
                    this.PointerMovedEvent = new Windows.UI.Xaml.RoutedEvent("PointerMovedEvent");
                    this.PointerPressedEvent = new Windows.UI.Xaml.RoutedEvent("PointerPressedEvent");
                    this.PointerReleasedEvent = new Windows.UI.Xaml.RoutedEvent("PointerReleasedEvent");
                    this.PointerEnteredEvent = new Windows.UI.Xaml.RoutedEvent("PointerEnteredEvent");
                    this.PointerExitedEvent = new Windows.UI.Xaml.RoutedEvent("PointerExitedEvent");
                    this.TappedEvent = new Windows.UI.Xaml.RoutedEvent("TappedEvent");
                    this.RightTappedEvent = new Windows.UI.Xaml.RoutedEvent("RightTappedEvent");
                    this.KeyDownEvent = new Windows.UI.Xaml.RoutedEvent("KeyDownEvent");
                    this.KeyUpEvent = new Windows.UI.Xaml.RoutedEvent("KeyUpEvent");
                    this.GotFocusEvent = new Windows.UI.Xaml.RoutedEvent("GotFocusEvent");
                    this.LostFocusEvent = new Windows.UI.Xaml.RoutedEvent("LostFocusEvent");
                    this.GotFocusForIsTabStopEvent = new Windows.UI.Xaml.RoutedEvent("GotFocusForIsTabStopEvent");
                }
            },
            methods: {
                GenerateRadioButtonsDefaultName: function () {
                    if (Windows.UI.Xaml.UIElement._random == null) {
                        Windows.UI.Xaml.UIElement._random = new System.Random.ctor();
                    }
                    var i = Windows.UI.Xaml.UIElement._random.Next();
                    return "RadioButtonDefaultGroupName" + (Bridge.toString(i) || "");
                },
                ClipToBounds_MethodToUpdateDom: function (d, newValue) {
                    var uiElement = Bridge.cast(d, Windows.UI.Xaml.UIElement);
                    var outerDomElement = uiElement.INTERNAL_OuterDomElement;
                    var style = CSHTML5.Internal.INTERNAL_HtmlDomManager.GetDomElementStyleForModification(outerDomElement);
                    if (System.Nullable.getValue(Bridge.cast(Bridge.unbox(newValue, System.Boolean), System.Boolean))) {
                        style.overflow = "hidden";
                        style.tableLayout = "fixed";
                    } else {
                        style.overflow = "auto";
                        style.tableLayout = "auto";
                    }
                },
                Effect_Changed: function (d, e) {
                    var element = Bridge.cast(d, Windows.UI.Xaml.UIElement);
                    if (e.OldValue != null) {
                        Bridge.cast(e.OldValue, System.Windows.Media.Effects.Effect).SetParentUIElement(null);
                    }

                    if (e.NewValue != null) {
                        Bridge.cast(e.NewValue, System.Windows.Media.Effects.Effect).SetParentUIElement(element);
                    }
                },
                RenderTransform_Changed: function (d, e) {
                    var uiElement = Bridge.cast(d, Windows.UI.Xaml.UIElement);
                    var newValue = Bridge.cast(e.NewValue, Windows.UI.Xaml.Media.Transform);
                    if (e.OldValue != null) {
                        Bridge.cast(e.OldValue, Windows.UI.Xaml.Media.Transform).INTERNAL_UnapplyTransform();
                        Bridge.cast(e.OldValue, Windows.UI.Xaml.Media.Transform).INTERNAL_parent = null;
                    }
                    if (CSHTML5.Internal.INTERNAL_VisualTreeManager.IsElementInVisualTree(uiElement)) {
                        if (newValue != null) {
                            newValue.INTERNAL_parent = uiElement;
                            newValue.INTERNAL_ApplyTransform();

                            if (!uiElement.INTERNAL_RenderTransformOriginHasBeenApplied) {
                                Windows.UI.Xaml.UIElement.ApplyRenderTransformOrigin(uiElement, new Windows.Foundation.Point.$ctor1(0.0, 0.0));
                            }
                        } else {
                            var domStyle = CSHTML5.Internal.INTERNAL_HtmlDomManager.GetFrameworkElementOuterStyleForModification(uiElement);

                            try {
                                domStyle.transform = "";
                            } catch ($e1) {
                                $e1 = System.Exception.create($e1);
                            }
                            try {
                                domStyle.msTransform = "";
                            } catch ($e2) {
                                $e2 = System.Exception.create($e2);
                            }
                            try {
                                domStyle.WebkitTransform = "";
                            } catch ($e3) {
                                $e3 = System.Exception.create($e3);
                            }
                        }
                    }
                },
                RenderTransformOrigin_Changed: function (d, e) {
                    var uiElement = Bridge.cast(d, Windows.UI.Xaml.UIElement);
                    if (CSHTML5.Internal.INTERNAL_VisualTreeManager.IsElementInVisualTree(uiElement)) {
                        var newValue = new Windows.Foundation.Point();
                        if (Bridge.is(e.NewValue, Windows.Foundation.Point)) {
                            newValue = System.Nullable.getValue(Bridge.cast(Bridge.unbox(e.NewValue, Windows.Foundation.Point), Windows.Foundation.Point));
                        } else {
                            newValue = new Windows.Foundation.Point.$ctor1(0.0, 0.0);
                        }
                        Windows.UI.Xaml.UIElement.ApplyRenderTransformOrigin(uiElement, newValue.$clone());
                    }
                },
                ApplyRenderTransformOrigin: function (uiElement, newValue) {
                    var domStyle = CSHTML5.Internal.INTERNAL_HtmlDomManager.GetFrameworkElementOuterStyleForModification(uiElement);
                    var transformOriginValue = (System.Double.format((newValue.X * 100)) || "") + "% " + (System.Double.format((newValue.Y * 100)) || "") + "%";
                    try {
                        domStyle.transformOrigin = transformOriginValue;
                    } catch ($e1) {
                        $e1 = System.Exception.create($e1);
                    }
                    try {
                        domStyle.webkitTransformOrigin = transformOriginValue;
                    } catch ($e2) {
                        $e2 = System.Exception.create($e2);
                    }
                    try {
                        domStyle.msTransformOrigin = transformOriginValue;
                    } catch ($e3) {
                        $e3 = System.Exception.create($e3);
                    }
                    uiElement.INTERNAL_RenderTransformOriginHasBeenApplied = true;
                },
                Visibility_Changed: function (d, e) {
                    var uiElement = Bridge.cast(d, Windows.UI.Xaml.UIElement);
                    var newValue = System.Nullable.getValue(Bridge.cast(Bridge.unbox(e.NewValue, Windows.UI.Xaml.Visibility), System.Int32));

                    if (!Bridge.staticEquals(uiElement.INTERNAL_DeferredLoadingWhenControlBecomesVisible, null) && newValue !== Windows.UI.Xaml.Visibility.Collapsed) {
                        var deferredLoadingWhenControlBecomesVisible = uiElement.INTERNAL_DeferredLoadingWhenControlBecomesVisible;
                        uiElement.INTERNAL_DeferredLoadingWhenControlBecomesVisible = null;
                        deferredLoadingWhenControlBecomesVisible();
                    }

                    Windows.UI.Xaml.UIElement.INTERNAL_ApplyVisibility(uiElement, newValue);

                    d.SetValue(Windows.UI.Xaml.UIElement.IsVisibleProperty, Bridge.box(newValue !== Windows.UI.Xaml.Visibility.Collapsed, System.Boolean, System.Boolean.toString));
                },
                INTERNAL_ApplyVisibility: function (uiElement, newValue) {
                    var $t;
                    if (CSHTML5.Internal.INTERNAL_VisualTreeManager.IsElementInVisualTree(uiElement)) {
                        var mostOuterDomElement = null;
                        if (Bridge.is(uiElement.INTERNAL_VisualParent, Windows.UI.Xaml.UIElement)) {
                            mostOuterDomElement = Bridge.cast(uiElement.INTERNAL_VisualParent, Windows.UI.Xaml.UIElement).INTERNAL_VisualChildrenInformation.getItem(uiElement).INTERNAL_OptionalChildWrapper_OuterDomElement;
                        }
                        if (mostOuterDomElement == null) {
                            mostOuterDomElement = ($t = uiElement.INTERNAL_AdditionalOutsideDivForMargins, $t != null ? $t : uiElement.INTERNAL_OuterDomElement);
                        }
                        var style = CSHTML5.Internal.INTERNAL_HtmlDomManager.GetDomElementStyleForModification(mostOuterDomElement);

                        if (newValue === Windows.UI.Xaml.Visibility.Collapsed) {
                            var previousValueOfDisplayCssProperty = style.display;
                            if (!Bridge.referenceEquals(previousValueOfDisplayCssProperty, "none")) {
                                uiElement._previousValueOfDisplayCssProperty = previousValueOfDisplayCssProperty;
                            }

                            style.display = "none";
                        } else {
                            if (Bridge.referenceEquals(style.display, "none")) {
                                style.display = uiElement._previousValueOfDisplayCssProperty;
                            }

                            if (Bridge.is(uiElement, Windows.UI.Xaml.FrameworkElement) && uiElement.INTERNAL_VisualParent != null) {
                                Windows.UI.Xaml.FrameworkElement.INTERNAL_ApplyHorizontalAlignmentAndWidth(Bridge.cast(uiElement, Windows.UI.Xaml.FrameworkElement), Bridge.cast(uiElement, Windows.UI.Xaml.FrameworkElement).HorizontalAlignment);
                                Windows.UI.Xaml.FrameworkElement.INTERNAL_ApplyVerticalAlignmentAndHeight(Bridge.cast(uiElement, Windows.UI.Xaml.FrameworkElement), Bridge.cast(uiElement, Windows.UI.Xaml.FrameworkElement).VerticalAlignment);
                            }
                        }
                        CSHTML5.Internal.INTERNAL_WorkaroundIE11IssuesWithScrollViewerInsideGrid.RefreshLayoutIfIE();

                        CSHTML5.Internal.INTERNAL_VisibilityChangedNotifier.NotifyListenersThatVisibilityHasChanged(uiElement);
                    }
                },
                OnIsVisiblePropertyChanged: function (d, e) {
                    var uiElement = Bridge.cast(d, Windows.UI.Xaml.UIElement);
                    var newValue = System.Nullable.getValue(Bridge.cast(Bridge.unbox(e.NewValue, System.Boolean), System.Boolean));

                    if (!Bridge.staticEquals(uiElement.INTERNAL_DeferredRenderingWhenControlBecomesVisible, null) && newValue === true) {
                        var deferredLoadingWhenControlBecomesVisible = uiElement.INTERNAL_DeferredRenderingWhenControlBecomesVisible;
                        uiElement.INTERNAL_DeferredRenderingWhenControlBecomesVisible = null;
                        deferredLoadingWhenControlBecomesVisible();
                    }

                    Windows.UI.Xaml.UIElement.InvalidateForceInheritPropertyOnChildren(Bridge.cast(d, Windows.UI.Xaml.UIElement), e.Property);

                    if (!Bridge.staticEquals(uiElement.IsVisibleChanged, null)) {
                        uiElement.IsVisibleChanged(d, e);
                    }
                },
                CoerceIsVisibleProperty: function (d, baseValue) {
                    var $this = Bridge.cast(d, Windows.UI.Xaml.UIElement);

                    if (!(Bridge.is(baseValue, System.Boolean))) {
                        return Bridge.box(true, System.Boolean, System.Boolean.toString);
                    }

                    if (System.Nullable.getValue(Bridge.cast(Bridge.unbox(baseValue, System.Boolean), System.Boolean))) {
                        var parent = $this.INTERNAL_VisualParent;
                        if (parent == null || System.Nullable.getValue(Bridge.cast(Bridge.unbox(parent.GetValue(Windows.UI.Xaml.UIElement.IsVisibleProperty), System.Boolean), System.Boolean))) {
                            return Bridge.box(true, System.Boolean, System.Boolean.toString);
                        } else {
                            return Bridge.box(false, System.Boolean, System.Boolean.toString);
                        }
                    } else {
                        return Bridge.box(false, System.Boolean, System.Boolean.toString);
                    }
                },
                OnIsHitTestVisiblePropertyChanged: function (d, e) {
                    Windows.UI.Xaml.UIElement.InvalidateForceInheritPropertyOnChildren(Bridge.cast(d, Windows.UI.Xaml.UIElement), e.Property);
                },
                IsHitTestVisible_MethodToUpdateDom: function (d, newValue) {
                    var element = Bridge.cast(d, Windows.UI.Xaml.UIElement);
                    Windows.UI.Xaml.UIElement.INTERNAL_UpdateCssPointerEventsPropertyBasedOnIsHitTestVisibleAndIsEnabled(element, System.Nullable.getValue(Bridge.cast(Bridge.unbox(newValue, System.Boolean), System.Boolean)), Bridge.is(element, Windows.UI.Xaml.FrameworkElement) ? Bridge.cast(element, Windows.UI.Xaml.FrameworkElement).IsEnabled : true);

                },
                CoerceIsHitTestVisibleProperty: function (d, baseValue) {
                    var $this = Bridge.cast(d, Windows.UI.Xaml.UIElement);


                    if (!(Bridge.is(baseValue, System.Boolean))) {
                        return Bridge.box(true, System.Boolean, System.Boolean.toString);
                    }

                    if (System.Nullable.getValue(Bridge.cast(Bridge.unbox(baseValue, System.Boolean), System.Boolean))) {
                        var parent = $this.INTERNAL_VisualParent;
                        if (parent == null || System.Nullable.getValue(Bridge.cast(Bridge.unbox(parent.GetValue(Windows.UI.Xaml.UIElement.IsHitTestVisibleProperty), System.Boolean), System.Boolean))) {
                            return Bridge.box(true, System.Boolean, System.Boolean.toString);
                        } else {
                            return Bridge.box(false, System.Boolean, System.Boolean.toString);
                        }
                    } else {
                        return Bridge.box(false, System.Boolean, System.Boolean.toString);
                    }
                },
                INTERNAL_UpdateCssPointerEventsPropertyBasedOnIsHitTestVisibleAndIsEnabled: function (element, isHitTestVisible, isEnabled) {

                    if (CSHTML5.Internal.INTERNAL_VisualTreeManager.IsElementInVisualTree(element)) {
                        var style = CSHTML5.Internal.INTERNAL_HtmlDomManager.GetDomElementStyleForModification(element.INTERNAL_OuterDomElement);
                        if (isHitTestVisible && isEnabled) {
                            style.pointerEvents = "auto";
                        } else {
                            style.pointerEvents = "none";
                        }
                    }
                },
                AllowScrollOnTouchMove_Changed: function (d, e) {
                    var element = Bridge.cast(d, Windows.UI.Xaml.UIElement);
                    if (CSHTML5.Internal.INTERNAL_VisualTreeManager.IsElementInVisualTree(element)) {
                        var value;
                        if (System.Nullable.getValue(Bridge.cast(Bridge.unbox(e.NewValue, System.Boolean), System.Boolean))) {
                            value = "auto";
                        } else {
                            value = "none";
                        }
                        element.INTERNAL_OuterDomElement.style.touchAction = value;
                    }
                },
                SynchronizeForceInheritProperties: function (uiE, parent) {
                    if (!System.Nullable.getValue(Bridge.cast(Bridge.unbox(parent.GetValue(Windows.UI.Xaml.FrameworkElement.IsEnabledProperty), System.Boolean), System.Boolean))) {
                        uiE.CoerceValue(Windows.UI.Xaml.FrameworkElement.IsEnabledProperty);
                    }

                    if (!System.Nullable.getValue(Bridge.cast(Bridge.unbox(parent.GetValue(Windows.UI.Xaml.UIElement.IsHitTestVisibleProperty), System.Boolean), System.Boolean))) {
                        uiE.CoerceValue(Windows.UI.Xaml.UIElement.IsHitTestVisibleProperty);
                    }

                    if (!System.Nullable.getValue(Bridge.cast(Bridge.unbox(parent.GetValue(Windows.UI.Xaml.UIElement.IsVisibleProperty), System.Boolean), System.Boolean))) {
                        uiE.CoerceValue(Windows.UI.Xaml.UIElement.IsVisibleProperty);
                    }
                },
                InvalidateForceInheritPropertyOnChildren: function (uiE, property) {
                    var $t;
                    if (uiE.INTERNAL_VisualChildrenInformation == null) {
                        return;
                    }

                    $t = Bridge.getEnumerator(System.Linq.Enumerable.from(uiE.INTERNAL_VisualChildrenInformation).select(function (kp) {
                            return kp.key;
                        }), Windows.UI.Xaml.UIElement);
                    try {
                        while ($t.moveNext()) {
                            var child = $t.Current;
                            child.CoerceValue(property);
                        }
                    } finally {
                        if (Bridge.is($t, System.IDisposable)) {
                            $t.System$IDisposable$Dispose();
                        }
                    }
                }
            }
        },
        fields: {
            INTERNAL_VisualParent: null,
            INTERNAL_ParentWindow: null,
            INTERNAL_OuterDomElement: null,
            INTERNAL_InnerDomElement: null,
            INTERNAL_AdditionalOutsideDivForMargins: null,
            INTERNAL_InnerDivOfTheChildWrapperOfTheParentIfAny: null,
            INTERNAL_OptionalSpecifyDomElementConcernedByFocus: null,
            INTERNAL_OptionalSpecifyDomElementConcernedByIsEnabled: null,
            INTERNAL_OptionalSpecifyDomElementConcernedByMinMaxHeightAndWidth: null,
            INTERNAL_SpanParentCell: null,
            INTERNAL_HtmlRepresentation: null,
            INTERNAL_VisualChildrenInformation: null,
            INTERNAL_RenderTransformOriginHasBeenApplied: false,
            INTERNAL_AssignedToolTip: null,
            INTERNAL_ValidationErrorPopup: null,
            INTERNAL_clickCount: 0,
            INTERNAL_lastClickDate: 0,
            XamlSourcePath: null,
            _isLoaded: false,
            INTERNAL_EnableProgressiveLoading: false,
            INTERNAL_DeferredRenderingWhenControlBecomesVisible: null,
            INTERNAL_DeferredLoadingWhenControlBecomesVisible: null,
            /**
             * Dictionary that helps link the validationErrors to the BindingExpressions for managing the errors.
             *
             * @instance
             * @memberof Windows.UI.Xaml.UIElement
             * @type System.Collections.Generic.Dictionary$2
             */
            INTERNAL_ValidationErrorsDictionary: null,
            _childrenRadioButtonDefaultName: null,
            _previousValueOfDisplayCssProperty: null,
            _pointerMovedEventManager: null,
            _pointerPressedEventManager: null,
            _pointerReleasedEventManager: null,
            _pointerEnteredEventManager: null,
            _pointerExitedEventManager: null,
            isAlreadySubscribedToMouseEnterAndLeave: false,
            /**
             * IMPORTANT NOTE: This boolean is only updated in the case the PointerExited/MouseLeave event is used on this element.
             It states whether the pointer is currently inside the UIElement.
             *
             * @instance
             * @memberof Windows.UI.Xaml.UIElement
             * @default false
             * @type boolean
             */
            INTERNAL_isPointerInside: false,
            _tappedEventManager: null,
            _rightTappedEventManager: null,
            _keyDownEventManager: null,
            _keyUpEventManager: null,
            _gotFocusEventManager: null,
            _lostFocusEventManager: null,
            /**
             * Set this boolean to true for the UIElements that have to react to keyboard events when they are focused.
             It activates the mechanic used to handle those events.
             It is used with the method OnKeyDownWhenFocused which is overriden and does the specific handling of the events.
             *
             * @instance
             * @memberof Windows.UI.Xaml.UIElement
             * @default false
             * @type boolean
             */
            _reactsToKeyboardEventsWhenFocused: false,
            _gotFocusForIsTabStopEventManager: null,
            INTERNAL_AreFocusEventsAllowed: false
        },
        events: {
            IsVisibleChanged: null,
            PointerCaptureLost: null
        },
        props: {
            INTERNAL_ChildrenRadioButtonDefaultName: {
                get: function () {
                    if (System.String.isNullOrWhiteSpace(this._childrenRadioButtonDefaultName)) {
                        this._childrenRadioButtonDefaultName = Windows.UI.Xaml.UIElement.GenerateRadioButtonsDefaultName();
                    }
                    return this._childrenRadioButtonDefaultName;
                }
            },
            /**
             * Gets or sets a value indicating whether to clip the content of this element
             (or content coming from the child elements of this element) to fit into the
             size of the containing element. This is a dependency property.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.UIElement
             * @function ClipToBounds
             * @type boolean
             */
            ClipToBounds: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.UIElement.ClipToBoundsProperty), System.Boolean), System.Boolean));
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.UIElement.ClipToBoundsProperty, Bridge.box(value, System.Boolean, System.Boolean.toString));
                }
            },
            Effect: {
                get: function () {
                    return Bridge.cast(this.GetValue(Windows.UI.Xaml.UIElement.EffectProperty), System.Windows.Media.Effects.Effect);
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.UIElement.EffectProperty, value);
                }
            },
            /**
             * Gets or sets transform information that affects the rendering position of
             a UIElement.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.UIElement
             * @function RenderTransform
             * @type Windows.UI.Xaml.Media.Transform
             */
            RenderTransform: {
                get: function () {
                    return Bridge.cast(this.GetValue(Windows.UI.Xaml.UIElement.RenderTransformProperty), Windows.UI.Xaml.Media.Transform);
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.UIElement.RenderTransformProperty, value);
                }
            },
            RenderTransformOrigin: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.UIElement.RenderTransformOriginProperty), Windows.Foundation.Point), Windows.Foundation.Point));
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.UIElement.RenderTransformOriginProperty, value.$clone());
                }
            },
            /**
             * Gets or sets a value that determines whether rendering for the object and
             its visual subtree should use rounding behavior that aligns rendering to
             whole pixels.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.UIElement
             * @function UseLayoutRounding
             * @type boolean
             */
            UseLayoutRounding: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.UIElement.UseLayoutRoundingProperty), System.Boolean), System.Boolean));
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.UIElement.UseLayoutRoundingProperty, Bridge.box(value, System.Boolean, System.Boolean.toString));
                }
            },
            /**
             * Gets or sets the visibility of a UIElement. A UIElement that is not visible
             is not rendered and does not communicate its desired size to layout.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.UIElement
             * @function Visibility
             * @type Windows.UI.Xaml.Visibility
             */
            Visibility: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.UIElement.VisibilityProperty), Windows.UI.Xaml.Visibility), System.Int32));
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.UIElement.VisibilityProperty, Bridge.box(value, Windows.UI.Xaml.Visibility, System.Enum.toStringFn(Windows.UI.Xaml.Visibility)));
                }
            },
            IsVisible: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.UIElement.IsVisibleProperty), System.Boolean), System.Boolean));
                }
            },
            /**
             * Gets or sets the degree of the object's opacity.
             A value between 0 and 1.0 that declares the opacity factor, with 1.0 meaning
             full opacity and 0 meaning transparent. The default value is 1.0.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.UIElement
             * @function Opacity
             * @type number
             */
            Opacity: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.UIElement.OpacityProperty), System.Double), System.Double));
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.UIElement.OpacityProperty, Bridge.box(value, System.Double, System.Double.format, System.Double.getHashCode));
                }
            },
            /**
             * Gets or sets whether the contained area of this UIElement can return true
             values for hit testing.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.UIElement
             * @function IsHitTestVisible
             * @type boolean
             */
            IsHitTestVisible: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.UIElement.IsHitTestVisibleProperty), System.Boolean), System.Boolean));
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.UIElement.IsHitTestVisibleProperty, Bridge.box(value, System.Boolean, System.Boolean.toString));
                }
            },
            /**
             * Gets or sets a value that determines whether this UIElement
             can be a drop target for purposes of drag-and-drop operations.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.UIElement
             * @function AllowDrop
             * @type boolean
             */
            AllowDrop: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.UIElement.AllowDropProperty), System.Boolean), System.Boolean));
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.UIElement.AllowDropProperty, Bridge.box(value, System.Boolean, System.Boolean.toString));
                }
            },
            /**
             * Gets a value indicating whether the pointer is captured to this element.
             *
             * @instance
             * @public
             * @readonly
             * @memberof Windows.UI.Xaml.UIElement
             * @function IsPointerCaptured
             * @type boolean
             */
            IsPointerCaptured: {
                get: function () {
                    return (Bridge.referenceEquals(Windows.UI.Xaml.Input.Pointer.INTERNAL_captured, this));
                }
            },
            /**
             * Gets or sets whether pressing (touchscreen devices) on this UIElement then moving should allow scrolling or not. The default value is True.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.UIElement
             * @function AllowScrollOnTouchMove
             * @type boolean
             */
            AllowScrollOnTouchMove: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.UIElement.AllowScrollOnTouchMoveProperty), System.Boolean), System.Boolean));
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.UIElement.AllowScrollOnTouchMoveProperty, Bridge.box(value, System.Boolean, System.Boolean.toString));
                }
            },
            PointerMovedEventManager: {
                get: function () {
                    if (this._pointerMovedEventManager == null) {
                        var eventsNames = System.Array.init(["mousemove", "touchmove"], System.String);
                        this._pointerMovedEventManager = new (CSHTML5.Internal.INTERNAL_EventManager$2(Function,Windows.UI.Xaml.Input.PointerRoutedEventArgs)).$ctor1(Bridge.fn.bind(this, function () {
                            return this.INTERNAL_OuterDomElement;
                        }), eventsNames, Bridge.fn.bind(this, function (jsEventArg) {
                            this.ProcessPointerEvent(jsEventArg, Bridge.fn.cacheBind(this, this.OnPointerMoved), Bridge.fn.cacheBind(this, this.OnPointerMoved_ForHandledEventsToo), true, false, false);
                        }));
                    }
                    return this._pointerMovedEventManager;
                }
            },
            PointerPressedEventManager: {
                get: function () {
                    if (this._pointerPressedEventManager == null) {
                        var eventsNames = System.Array.init(["mousedown", "touchstart"], System.String);
                        this._pointerPressedEventManager = new (CSHTML5.Internal.INTERNAL_EventManager$2(Function,Windows.UI.Xaml.Input.PointerRoutedEventArgs)).$ctor1(Bridge.fn.bind(this, function () {
                            return this.INTERNAL_OuterDomElement;
                        }), eventsNames, Bridge.fn.bind(this, function (jsEventArg) {
                            this.ProcessPointerEvent(jsEventArg, Bridge.fn.cacheBind(this, this.OnPointerPressed), Bridge.fn.cacheBind(this, this.OnPointerPressed_ForHandledEventsToo), true, true, true);
                        }));
                    }
                    return this._pointerPressedEventManager;
                }
            },
            PointerReleasedEventManager: {
                get: function () {
                    if (this._pointerReleasedEventManager == null) {
                        var eventsNames = System.Array.init(["mouseup", "touchend"], System.String);

                        this._pointerReleasedEventManager = new (CSHTML5.Internal.INTERNAL_EventManager$2(Function,Windows.UI.Xaml.Input.PointerRoutedEventArgs)).$ctor1(Bridge.fn.bind(this, function () {
                            return this.INTERNAL_OuterDomElement;
                        }), eventsNames, Bridge.fn.bind(this, function (jsEventArg) {
                            this.ProcessPointerEvent(jsEventArg, Bridge.fn.cacheBind(this, this.OnPointerReleased), Bridge.fn.cacheBind(this, this.OnPointerReleased_ForHandledEventsToo), false, true, false);
                        }));
                    }
                    return this._pointerReleasedEventManager;
                }
            },
            PointerEnteredEventManager: {
                get: function () {
                    if (this._pointerEnteredEventManager == null) {
                        var eventsNames = System.Array.init(["mouseenter"], System.String);

                        this._pointerEnteredEventManager = new (CSHTML5.Internal.INTERNAL_EventManager$2(Function,Windows.UI.Xaml.Input.PointerRoutedEventArgs)).$ctor1(Bridge.fn.bind(this, function () {
                            return this.INTERNAL_OuterDomElement;
                        }), eventsNames, Bridge.fn.cacheBind(this, this.ProcessOnPointerEntered));
                    }
                    return this._pointerEnteredEventManager;
                }
            },
            PointerExitedEventManager: {
                get: function () {
                    if (this._pointerExitedEventManager == null) {
                        var eventsNames = System.Array.init(["mouseleave"], System.String);
                        this._pointerExitedEventManager = new (CSHTML5.Internal.INTERNAL_EventManager$2(Function,Windows.UI.Xaml.Input.PointerRoutedEventArgs)).$ctor1(Bridge.fn.bind(this, function () {
                            return this.INTERNAL_OuterDomElement;
                        }), eventsNames, Bridge.fn.cacheBind(this, this.ProcessOnPointerExited));
                    }
                    return this._pointerExitedEventManager;
                }
            },
            TappedEventManager: {
                get: function () {
                    if (this._tappedEventManager == null) {
                        this._tappedEventManager = new (CSHTML5.Internal.INTERNAL_EventManager$2(Function,Windows.UI.Xaml.Input.TappedRoutedEventArgs)).ctor(Bridge.fn.bind(this, function () {
                            return this.INTERNAL_OuterDomElement;
                        }), "mouseup", Bridge.fn.cacheBind(this, this.ProcessOnTapped));
                    }
                    return this._tappedEventManager;
                }
            },
            RightTappedEventManager: {
                get: function () {
                    if (this._rightTappedEventManager == null) {
                        this._rightTappedEventManager = new (CSHTML5.Internal.INTERNAL_EventManager$2(Function,Windows.UI.Xaml.Input.RightTappedRoutedEventArgs)).ctor(Bridge.fn.bind(this, function () {
                            return this.INTERNAL_OuterDomElement;
                        }), "contextmenu", Bridge.fn.cacheBind(this, this.ProcessOnRightTapped));
                    }
                    return this._rightTappedEventManager;
                }
            },
            KeyDownEventManager: {
                get: function () {
                    if (this._keyDownEventManager == null) {
                        this._keyDownEventManager = new (CSHTML5.Internal.INTERNAL_EventManager$2(Function,Windows.UI.Xaml.Input.KeyRoutedEventArgs)).ctor(Bridge.fn.bind(this, function () {
                            return this.INTERNAL_OptionalSpecifyDomElementConcernedByFocus || this.INTERNAL_OuterDomElement;
                        }), "keydown", Bridge.fn.cacheBind(this, this.ProcessOnKeyDown));
                    }
                    return this._keyDownEventManager;
                }
            },
            KeyUpEventManager: {
                get: function () {
                    if (this._keyUpEventManager == null) {
                        this._keyUpEventManager = new (CSHTML5.Internal.INTERNAL_EventManager$2(Function,Windows.UI.Xaml.Input.KeyRoutedEventArgs)).ctor(Bridge.fn.bind(this, function () {
                            return this.INTERNAL_OptionalSpecifyDomElementConcernedByFocus || this.INTERNAL_OuterDomElement;
                        }), "keyup", Bridge.fn.cacheBind(this, this.ProcessOnKeyUp));
                    }
                    return this._keyUpEventManager;
                }
            },
            GotFocusEventManager: {
                get: function () {
                    if (this._gotFocusEventManager == null) {
                        this._gotFocusEventManager = new (CSHTML5.Internal.INTERNAL_EventManager$2(Function,Windows.UI.Xaml.RoutedEventArgs)).ctor(Bridge.fn.bind(this, function () {
                            return (this.INTERNAL_OptionalSpecifyDomElementConcernedByFocus || this.INTERNAL_OuterDomElement);
                        }), "focusin", Bridge.fn.cacheBind(this, this.ProcessOnGotFocus));
                    }
                    return this._gotFocusEventManager;
                }
            },
            LostFocusEventManager: {
                get: function () {
                    if (this._lostFocusEventManager == null) {
                        this._lostFocusEventManager = new (CSHTML5.Internal.INTERNAL_EventManager$2(Function,Windows.UI.Xaml.RoutedEventArgs)).ctor(Bridge.fn.bind(this, function () {
                            return (this.INTERNAL_OptionalSpecifyDomElementConcernedByFocus || this.INTERNAL_OuterDomElement);
                        }), "focusout", Bridge.fn.cacheBind(this, this.ProcessOnLostFocus));
                    }
                    return this._lostFocusEventManager;
                }
            },
            GotFocusForIsTabStopEventManager: {
                get: function () {
                    if (this._gotFocusForIsTabStopEventManager == null) {
                        this._gotFocusForIsTabStopEventManager = new (CSHTML5.Internal.INTERNAL_EventManager$2(Function,Windows.UI.Xaml.RoutedEventArgs)).ctor(Bridge.fn.bind(this, function () {
                            return (this.INTERNAL_OptionalSpecifyDomElementConcernedByFocus || this.INTERNAL_OuterDomElement);
                        }), "focusin", Bridge.fn.cacheBind(this, this.ProcessOnGotFocusForIsTabStop));
                    }
                    return this._gotFocusForIsTabStopEventManager;
                }
            }
        },
        ctors: {
            init: function () {
                this.INTERNAL_RenderTransformOriginHasBeenApplied = false;
                this._previousValueOfDisplayCssProperty = "block";
                this.isAlreadySubscribedToMouseEnterAndLeave = false;
                this.INTERNAL_isPointerInside = false;
                this._reactsToKeyboardEventsWhenFocused = false;
                this.INTERNAL_AreFocusEventsAllowed = true;
            }
        },
        methods: {
            /**
             * Occurs when the pointer device that previously initiated a Press action is
             moved, while within this element.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.UIElement
             * @event Windows.UI.Xaml.UIElement#PointerMoved
             * @return  {Windows.UI.Xaml.Input.PointerEventHandler}
             */
            addPointerMoved: function (value) {
                this.PointerMovedEventManager.Add(value);
            },
            /**
             * Occurs when the pointer device that previously initiated a Press action is
             moved, while within this element.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.UIElement
             * @event Windows.UI.Xaml.UIElement#PointerMoved
             * @return  {Windows.UI.Xaml.Input.PointerEventHandler}
             */
            removePointerMoved: function (value) {
                this.PointerMovedEventManager.Remove(value);
            },
            /**
             * Occurs when the pointer device that previously initiated a Press action is
             pressed, while within this element.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.UIElement
             * @event Windows.UI.Xaml.UIElement#PointerPressed
             * @return  {Windows.UI.Xaml.Input.PointerEventHandler}
             */
            addPointerPressed: function (value) {
                this.PointerPressedEventManager.Add(value);
            },
            /**
             * Occurs when the pointer device that previously initiated a Press action is
             pressed, while within this element.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.UIElement
             * @event Windows.UI.Xaml.UIElement#PointerPressed
             * @return  {Windows.UI.Xaml.Input.PointerEventHandler}
             */
            removePointerPressed: function (value) {
                this.PointerPressedEventManager.Remove(value);
            },
            /**
             * Occurs when the pointer device that previously initiated a Press action is
             released, while within this element.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.UIElement
             * @event Windows.UI.Xaml.UIElement#PointerReleased
             * @return  {Windows.UI.Xaml.Input.PointerEventHandler}
             */
            addPointerReleased: function (value) {
                this.PointerReleasedEventManager.Add(value);
            },
            /**
             * Occurs when the pointer device that previously initiated a Press action is
             released, while within this element.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.UIElement
             * @event Windows.UI.Xaml.UIElement#PointerReleased
             * @return  {Windows.UI.Xaml.Input.PointerEventHandler}
             */
            removePointerReleased: function (value) {
                this.PointerReleasedEventManager.Remove(value);
            },
            /**
             * Occurs when a pointer enters the hit test area of this element.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.UIElement
             * @event Windows.UI.Xaml.UIElement#PointerEntered
             * @return  {Windows.UI.Xaml.Input.PointerEventHandler}
             */
            addPointerEntered: function (value) {
                this.PointerEnteredEventManager.Add(value);

            },
            /**
             * Occurs when a pointer enters the hit test area of this element.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.UIElement
             * @event Windows.UI.Xaml.UIElement#PointerEntered
             * @return  {Windows.UI.Xaml.Input.PointerEventHandler}
             */
            removePointerEntered: function (value) {
                this.PointerEnteredEventManager.Remove(value);
            },
            /**
             * Occurs when a pointer leaves the hit test area of this element.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.UIElement
             * @event Windows.UI.Xaml.UIElement#PointerExited
             * @return  {Windows.UI.Xaml.Input.PointerEventHandler}
             */
            addPointerExited: function (value) {
                this.StartManagingPointerPositionForPointerExitedEvent();
                this.PointerExitedEventManager.Add(value);
            },
            /**
             * Occurs when a pointer leaves the hit test area of this element.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.UIElement
             * @event Windows.UI.Xaml.UIElement#PointerExited
             * @return  {Windows.UI.Xaml.Input.PointerEventHandler}
             */
            removePointerExited: function (value) {
                this.PointerExitedEventManager.Remove(value);
                if (this.PointerExitedEventManager.Handlers.Count === 0) {
                    this.StopManagingPointerPositionForPointerExitedEvent();
                }
            },
            /**
             * Occurs when an otherwise unhandled Tap interaction occurs over the hit test
             area of this element.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.UIElement
             * @event Windows.UI.Xaml.UIElement#Tapped
             * @return  {Windows.UI.Xaml.Input.TappedEventHandler}
             */
            addTapped: function (value) {
                this.TappedEventManager.Add(value);
            },
            /**
             * Occurs when an otherwise unhandled Tap interaction occurs over the hit test
             area of this element.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.UIElement
             * @event Windows.UI.Xaml.UIElement#Tapped
             * @return  {Windows.UI.Xaml.Input.TappedEventHandler}
             */
            removeTapped: function (value) {
                this.TappedEventManager.Remove(value);
            },
            /**
             * Occurs when a right-tap input stimulus happens while the pointer is over
             the element.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.UIElement
             * @event Windows.UI.Xaml.UIElement#RightTapped
             * @return  {Windows.UI.Xaml.Input.RightTappedEventHandler}
             */
            addRightTapped: function (value) {
                this.RightTappedEventManager.Add(value);
            },
            /**
             * Occurs when a right-tap input stimulus happens while the pointer is over
             the element.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.UIElement
             * @event Windows.UI.Xaml.UIElement#RightTapped
             * @return  {Windows.UI.Xaml.Input.RightTappedEventHandler}
             */
            removeRightTapped: function (value) {
                this.RightTappedEventManager.Remove(value);
            },
            /**
             * Occurs when a keyboard key is pressed while the UIElement has focus.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.UIElement
             * @event Windows.UI.Xaml.UIElement#KeyDown
             * @return  {Windows.UI.Xaml.Input.KeyEventHandler}
             */
            addKeyDown: function (value) {
                this.KeyDownEventManager.Add(value);
            },
            /**
             * Occurs when a keyboard key is pressed while the UIElement has focus.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.UIElement
             * @event Windows.UI.Xaml.UIElement#KeyDown
             * @return  {Windows.UI.Xaml.Input.KeyEventHandler}
             */
            removeKeyDown: function (value) {
                this.KeyDownEventManager.Remove(value);
            },
            /**
             * Occurs when a keyboard key is released while the UIElement has focus.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.UIElement
             * @event Windows.UI.Xaml.UIElement#KeyUp
             * @return  {Windows.UI.Xaml.Input.KeyEventHandler}
             */
            addKeyUp: function (value) {
                this.KeyUpEventManager.Add(value);
            },
            /**
             * Occurs when a keyboard key is released while the UIElement has focus.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.UIElement
             * @event Windows.UI.Xaml.UIElement#KeyUp
             * @return  {Windows.UI.Xaml.Input.KeyEventHandler}
             */
            removeKeyUp: function (value) {
                this.KeyUpEventManager.Remove(value);
            },
            /**
             * Occurs when the pointer device that previously initiated a Press action is
             pressed, while within this element.
             Note that ONLY sender's informations are currently filled (not pointer's)
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.UIElement
             * @event Windows.UI.Xaml.UIElement#GotFocus
             * @return  {Windows.UI.Xaml.RoutedEventHandler}
             */
            addGotFocus: function (value) {
                this.GotFocusEventManager.Add(value);
            },
            /**
             * Occurs when the pointer device that previously initiated a Press action is
             pressed, while within this element.
             Note that ONLY sender's informations are currently filled (not pointer's)
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.UIElement
             * @event Windows.UI.Xaml.UIElement#GotFocus
             * @return  {Windows.UI.Xaml.RoutedEventHandler}
             */
            removeGotFocus: function (value) {
                this.GotFocusEventManager.Remove(value);
            },
            /**
             * Occurs when a UIElement loses focus.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.UIElement
             * @event Windows.UI.Xaml.UIElement#LostFocus
             * @return  {Windows.UI.Xaml.RoutedEventHandler}
             */
            addLostFocus: function (value) {
                this.LostFocusEventManager.Add(value);
            },
            /**
             * Occurs when a UIElement loses focus.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.UIElement
             * @event Windows.UI.Xaml.UIElement#LostFocus
             * @return  {Windows.UI.Xaml.RoutedEventHandler}
             */
            removeLostFocus: function (value) {
                this.LostFocusEventManager.Remove(value);
            },
            /**
             * Occurs when the pointer device that previously initiated a Press action is
             pressed, while within this element.
             Note that ONLY sender's informations are currently filled (not pointer's)
             *
             * @instance
             * @private
             * @memberof Windows.UI.Xaml.UIElement
             * @event Windows.UI.Xaml.UIElement#GotFocusForIsTabStop
             * @return  {Windows.UI.Xaml.RoutedEventHandler}
             */
            addGotFocusForIsTabStop: function (value) {
                this.GotFocusForIsTabStopEventManager.Add(value);
            },
            /**
             * Occurs when the pointer device that previously initiated a Press action is
             pressed, while within this element.
             Note that ONLY sender's informations are currently filled (not pointer's)
             *
             * @instance
             * @private
             * @memberof Windows.UI.Xaml.UIElement
             * @event Windows.UI.Xaml.UIElement#GotFocusForIsTabStop
             * @return  {Windows.UI.Xaml.RoutedEventHandler}
             */
            removeGotFocusForIsTabStop: function (value) {
                this.GotFocusForIsTabStopEventManager.Remove(value);
            },
            GetDomElementToSetContentString: function () {
                return this.INTERNAL_InnerDomElement;
            },
            /**
             * When overriden, creates a dom wrapper for each child that is added to the UIElement.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.UIElement
             * @memberof Windows.UI.Xaml.UIElement
             * @param   {System.Object}    parentRef                      
             * @param   {System.Object}    domElementWhereToPlaceChild    
             * @param   {number}           index                          The index for the position in which to add the child.
             * @return  {System.Object}
             */
            CreateDomChildWrapper: function (parentRef, domElementWhereToPlaceChild, index) {
                if (index === void 0) { index = -1; }
                domElementWhereToPlaceChild.v = null;
                return null;
            },
            GetDomElementWhereToPlaceChild: function (child) {
                return null;
            },
            GetChildsWrapper: function (child) {
                if (CSHTML5.Internal.INTERNAL_VisualTreeManager.IsElementInVisualTree(this) && CSHTML5.Internal.INTERNAL_VisualTreeManager.IsElementInVisualTree(child)) {
                    return this.INTERNAL_VisualChildrenInformation.getItem(child).INTERNAL_OptionalChildWrapper_OuterDomElement;
                } else {
                    return null;
                }
            },
            INTERNAL_IsChildOf: function (element) {
                var currentParent = this;
                while (currentParent.INTERNAL_VisualParent != null) {
                    if (Bridge.referenceEquals(currentParent.INTERNAL_VisualParent, element)) {
                        return true;
                    }
                    currentParent = Bridge.cast(currentParent.INTERNAL_VisualParent, Windows.UI.Xaml.UIElement);
                }
                return false;
            },
            INTERNAL_UpdateDomStructureIfNecessary: function () { },
            /**
             * Sets pointer capture to a UIElement.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.UIElement
             * @memberof Windows.UI.Xaml.UIElement
             * @param   {Windows.UI.Xaml.Input.Pointer}    value    The pointer object reference.
             * @return  {boolean}                                   True if the object has pointer capture; otherwise, false.
             */
            CapturePointer: function (value) {
                if (value === void 0) { value = null; }
                return this.CapturePointer$1(value, this.INTERNAL_OuterDomElement);
            },
            CapturePointer$1: function (value, element) {
                if (Windows.UI.Xaml.Input.Pointer.INTERNAL_captured == null) {
                    Windows.UI.Xaml.Input.Pointer.INTERNAL_captured = this;

                    if (true) {

     document.onmouseup = function(e) {
        if(e.doNotReroute == undefined)
        {
               document.reroute(e,                         element);
        }
    }
     document.onmouseover = function(e) {
       if(e.doNotReroute == undefined)
        {
               document.reroute(e, element);
        }
    }       
    document.onmousedown = function(e) {
       if(e.doNotReroute == undefined)
        {
               document.reroute(e, element);
        }
    }                               
     document.onmouseout = function(e) {   
       if(e.doNotReroute == undefined)
        {
               document.reroute(e, element);
        }
    }                                      
     document.onmousemove = function(e) {  
       if(e.doNotReroute == undefined)
        {
               document.reroute(e, element);
        }
    }                                      
     document.onclick = function(e) {      
       if(e.doNotReroute == undefined)
        {
               document.reroute(e, element);
        }
    }                                      
     document.oncontextmenu = function(e) {
       if(e.doNotReroute == undefined)
        {
               document.reroute(e, element);
        }
    }                                      
     document.ondblclick = function(e) {   
       if(e.doNotReroute == undefined)
        {
               document.reroute(e, element);
        }
    }
;


                    } else {
                        var javaScriptCodeToExecute = System.String.format("\r\n     document.onmouseup = function(e) {{\r\n        if(e.doNotReroute == undefined)\r\n        {{\r\n               var element = document.getElementById(\"{0}\");\r\n               document.reroute(e, element);\r\n        }}\r\n    }}\r\n     document.onmouseover = function(e) {{\r\n       if(e.doNotReroute == undefined)\r\n        {{\r\n               var element = document.getElementById(\"{0}\");\r\n               document.reroute(e, element);\r\n        }}\r\n    }} \r\n    document.onmousedown = function(e) {{\r\n       if(e.doNotReroute == undefined)\r\n        {{\r\n               var element = document.getElementById(\"{0}\");\r\n               document.reroute(e, element);\r\n        }}\r\n    }}                       \r\n     document.onmouseout = function(e) {{   \r\n       if(e.doNotReroute == undefined)\r\n        {{\r\n               var element = document.getElementById(\"{0}\");\r\n               document.reroute(e, element);\r\n        }}\r\n    }}                            \r\n     document.onmousemove = function(e) {{\r\n       if(e.doNotReroute == undefined)\r\n        {{\r\n               var element = document.getElementById(\"{0}\");\r\n               document.reroute(e, element);\r\n        }}\r\n    }}                                    \r\n     document.onclick = function(e) {{   \r\n       if(e.doNotReroute == undefined)\r\n        {{\r\n               var element = document.getElementById(\"{0}\");\r\n               document.reroute(e, element);\r\n        }}\r\n    }}                                     \r\n     document.oncontextmenu = function(e) {{\r\n       if(e.doNotReroute == undefined)\r\n        {{\r\n               var element = document.getElementById(\"{0}\");\r\n               document.reroute(e, element);\r\n        }}\r\n    }}                                      \r\n     document.ondblclick = function(e) {{   \r\n       if(e.doNotReroute == undefined)\r\n        {{\r\n               var element = document.getElementById(\"{0}\");\r\n               document.reroute(e, element);\r\n        }}\r\n    }}\r\n\r\n", [element.UniqueIdentifier]);
                        CSHTML5.Internal.INTERNAL_HtmlDomManager.ExecuteJavaScriptWithResult(javaScriptCodeToExecute);
                    }
                    return true;
                } else if (Bridge.referenceEquals(Windows.UI.Xaml.Input.Pointer.INTERNAL_captured, this)) {
                    return true;
                } else {
                    return false;
                }
            },
            /**
             * Releases pointer captures for capture of one specific pointer by this UIElement.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.UIElement
             * @memberof Windows.UI.Xaml.UIElement
             * @param   {Windows.UI.Xaml.Input.Pointer}    value    The pointer reference. Use either saved references from previous captures,
             or pointer event data, to obtain this reference.
             * @return  {void}
             */
            ReleasePointerCapture: function (value) {
                if (value === void 0) { value = null; }
                if (true) {
                    if (Windows.UI.Xaml.Input.Pointer.INTERNAL_captured != null) {
                        
 document.onmousedown = null;
 document.onmouseup = null;
 document.onmouseover = null;
 document.onmouseout = null;
 document.onmousemove = null;
 document.onclick = null;
 document.oncontextmenu = null;
 document.ondblclick = null;



                        Windows.UI.Xaml.Input.Pointer.INTERNAL_captured = null;
                        this.OnPointerCaptureLost(new Windows.UI.Xaml.Input.PointerRoutedEventArgs());
                    }
                } else {
                    if (Windows.UI.Xaml.Input.Pointer.INTERNAL_captured != null) {
                        var javaScriptCodeToExecute = System.String.format("\r\n document.onmousedown = null;\r\n document.onmouseup = null;\r\n document.onmouseover = null;\r\n document.onmouseout = null;\r\n document.onmousemove = null;\r\n document.onclick = null;\r\n document.oncontextmenu = null;\r\n document.ondblclick = null;\r\n", null);
                        CSHTML5.Internal.INTERNAL_HtmlDomManager.ExecuteJavaScript(javaScriptCodeToExecute);
                        Windows.UI.Xaml.Input.Pointer.INTERNAL_captured = null;
                        this.OnPointerCaptureLost(new Windows.UI.Xaml.Input.PointerRoutedEventArgs());
                    }
                }
            },
            OnPointerCaptureLost: function (e) {
                if (!Bridge.staticEquals(this.PointerCaptureLost, null)) {
                    this.PointerCaptureLost(this, e);
                }
            },
            INTERNAL_OnDetachedFromVisualTree: function () {
                Windows.UI.Xaml.DependencyObject.prototype.INTERNAL_OnDetachedFromVisualTree.call(this);

                if (Bridge.referenceEquals(Windows.UI.Xaml.Input.Pointer.INTERNAL_captured, this)) {
                    this.ReleasePointerCapture();
                }
            },
            /**
             * Returns a transform object that can be used to transform coordinates from
             the UIElement to the specified object.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.UIElement
             * @memberof Windows.UI.Xaml.UIElement
             * @param   {Windows.UI.Xaml.UIElement}                 visual    The object to compare to the current object for purposes of obtaining the
             transform.
             * @return  {Windows.UI.Xaml.Media.GeneralTransform}              The transform information as an object. Call methods on this object to get
             a practical transform.
             */
            TransformToVisual: function (visual) {
                var $t;
                var outerDivOfThisControl = this.INTERNAL_OuterDomElement;

                var outerDivOfReferenceVisual = (visual != null) ? visual.INTERNAL_OuterDomElement : this.INTERNAL_ParentWindow.INTERNAL_OuterDomElement;

                var offsetLeft, offsetTop;
                if (CSharpXamlForHtml5.Environment.IsRunningInJavaScript) {
                    var rectOfThisControl = outerDivOfThisControl.getBoundingClientRect();
                    var rectOfReferenceVisual = outerDivOfReferenceVisual.getBoundingClientRect();

                    offsetLeft = rectOfThisControl.left - rectOfReferenceVisual.left;
                    offsetTop = rectOfThisControl.top - rectOfReferenceVisual.top;
                } else {

                    var concatenated = System.Convert.toString((outerDivOfThisControl.getBoundingClientRect().left - outerDivOfReferenceVisual.getBoundingClientRect().left) + '|' + (outerDivOfThisControl.getBoundingClientRect().top - outerDivOfReferenceVisual.getBoundingClientRect().top));
                    var sepIndex = System.String.indexOf(concatenated, String.fromCharCode(124));
                    var offsetLeftAsString = concatenated.substr(0, sepIndex);
                    var offsetTopAsString = concatenated.substr(((sepIndex + 1) | 0));
                    offsetLeft = System.Convert.toDouble(offsetLeftAsString, System.Globalization.CultureInfo.invariantCulture);
                    offsetTop = System.Convert.toDouble(offsetTopAsString, System.Globalization.CultureInfo.invariantCulture);
                }

                return ($t = new Windows.UI.Xaml.Media.TranslateTransform(), $t.X = offsetLeft, $t.Y = offsetTop, $t);
            },
            _ignoreMouseEventsTimer_Tick: function (sender, e) {
                Windows.UI.Xaml.UIElement.ignoreMouseEvents = false;
                Windows.UI.Xaml.UIElement._ignoreMouseEventsTimer.Stop();
            },
            /**
             * Raises the PointerMoved event
             *
             * @instance
             * @protected
             * @this Windows.UI.Xaml.UIElement
             * @memberof Windows.UI.Xaml.UIElement
             * @param   {Windows.UI.Xaml.Input.PointerRoutedEventArgs}    eventArgs    The arguments for the event.
             * @return  {void}
             */
            OnPointerMoved: function (eventArgs) {
                var $t;
                $t = Bridge.getEnumerator(System.Linq.Enumerable.from(this._pointerMovedEventManager.Handlers).toList(Function));
                try {
                    while ($t.moveNext()) {
                        var handler = $t.Current;
                        if (eventArgs.Handled) {
                            break;
                        }
                        handler(this, eventArgs);
                    }
                } finally {
                    if (Bridge.is($t, System.IDisposable)) {
                        $t.System$IDisposable$Dispose();
                    }
                }
            },
            OnPointerMoved_ForHandledEventsToo: function (eventArgs) {
                var $t;
                $t = Bridge.getEnumerator(System.Linq.Enumerable.from(this._pointerMovedEventManager.HandlersForHandledEventsToo).toList(Function));
                try {
                    while ($t.moveNext()) {
                        var handler = $t.Current;
                        handler(this, eventArgs);
                    }
                } finally {
                    if (Bridge.is($t, System.IDisposable)) {
                        $t.System$IDisposable$Dispose();
                    }
                }
            },
            /**
             * Raises the PointerPressed event
             *
             * @instance
             * @protected
             * @this Windows.UI.Xaml.UIElement
             * @memberof Windows.UI.Xaml.UIElement
             * @param   {Windows.UI.Xaml.Input.PointerRoutedEventArgs}    eventArgs    The arguments for the event.
             * @return  {void}
             */
            OnPointerPressed: function (eventArgs) {
                var $t;
                $t = Bridge.getEnumerator(System.Linq.Enumerable.from(this._pointerPressedEventManager.Handlers).toList(Function));
                try {
                    while ($t.moveNext()) {
                        var handler = $t.Current;
                        if (eventArgs.Handled) {
                            break;
                        }
                        handler(this, eventArgs);
                    }
                } finally {
                    if (Bridge.is($t, System.IDisposable)) {
                        $t.System$IDisposable$Dispose();
                    }
                }

                if (Bridge.is(this, Windows.UI.Xaml.Controls.Control)) {
                    var thisAsControl = Bridge.cast(this, Windows.UI.Xaml.Controls.Control);
                    if (thisAsControl.IsTabStop) {
                        thisAsControl.Focus();
                    }
                }
            },
            OnPointerPressed_ForHandledEventsToo: function (eventArgs) {
                var $t;
                $t = Bridge.getEnumerator(System.Linq.Enumerable.from(this._pointerPressedEventManager.HandlersForHandledEventsToo).toList(Function));
                try {
                    while ($t.moveNext()) {
                        var handler = $t.Current;
                        handler(this, eventArgs);
                    }
                } finally {
                    if (Bridge.is($t, System.IDisposable)) {
                        $t.System$IDisposable$Dispose();
                    }
                }
            },
            /**
             * Raises the PointerReleased event
             *
             * @instance
             * @protected
             * @this Windows.UI.Xaml.UIElement
             * @memberof Windows.UI.Xaml.UIElement
             * @param   {Windows.UI.Xaml.Input.PointerRoutedEventArgs}    eventArgs    The arguments for the event.
             * @return  {void}
             */
            OnPointerReleased: function (eventArgs) {
                var $t;
                $t = Bridge.getEnumerator(System.Linq.Enumerable.from(this._pointerReleasedEventManager.Handlers).toList(Function));
                try {
                    while ($t.moveNext()) {
                        var handler = $t.Current;
                        if (eventArgs.Handled) {
                            break;
                        }
                        handler(this, eventArgs);
                    }
                } finally {
                    if (Bridge.is($t, System.IDisposable)) {
                        $t.System$IDisposable$Dispose();
                    }
                }
            },
            OnPointerReleased_ForHandledEventsToo: function (eventArgs) {
                var $t;
                $t = Bridge.getEnumerator(System.Linq.Enumerable.from(this._pointerReleasedEventManager.HandlersForHandledEventsToo).toList(Function));
                try {
                    while ($t.moveNext()) {
                        var handler = $t.Current;
                        handler(this, eventArgs);
                    }
                } finally {
                    if (Bridge.is($t, System.IDisposable)) {
                        $t.System$IDisposable$Dispose();
                    }
                }
            },
            /**
             * Creates the eventArgs from the infos in the javascript's version of the event then raises the PointerEntered event
             *
             * @instance
             * @private
             * @this Windows.UI.Xaml.UIElement
             * @memberof Windows.UI.Xaml.UIElement
             * @param   {System.Object}    jsEventArg
             * @return  {void}
             */
            ProcessOnPointerEntered: function (jsEventArg) {

                this.ProcessPointerEvent(jsEventArg, Bridge.fn.cacheBind(this, this.OnPointerEntered), Bridge.fn.cacheBind(this, this.OnPointerEntered_ForHandledEventsToo), true, false, false);
            },
            /**
             * Raises the PointerEntered event
             *
             * @instance
             * @protected
             * @this Windows.UI.Xaml.UIElement
             * @memberof Windows.UI.Xaml.UIElement
             * @param   {Windows.UI.Xaml.Input.PointerRoutedEventArgs}    eventArgs    The arguments for the event.
             * @return  {void}
             */
            OnPointerEntered: function (eventArgs) {
                var $t;
                $t = Bridge.getEnumerator(System.Linq.Enumerable.from(this._pointerEnteredEventManager.Handlers).toList(Function));
                try {
                    while ($t.moveNext()) {
                        var handler = $t.Current;
                        if (eventArgs.Handled) {
                            break;
                        }
                        handler(this, eventArgs);
                    }
                } finally {
                    if (Bridge.is($t, System.IDisposable)) {
                        $t.System$IDisposable$Dispose();
                    }
                }
            },
            OnPointerEntered_ForHandledEventsToo: function (eventArgs) {
                var $t;
                $t = Bridge.getEnumerator(System.Linq.Enumerable.from(this._pointerEnteredEventManager.HandlersForHandledEventsToo).toList(Function));
                try {
                    while ($t.moveNext()) {
                        var handler = $t.Current;
                        handler(this, eventArgs);
                    }
                } finally {
                    if (Bridge.is($t, System.IDisposable)) {
                        $t.System$IDisposable$Dispose();
                    }
                }
            },
            SetIsPointerInsideToTrue: function () {
                this.INTERNAL_isPointerInside = true;
            },
            StartManagingPointerPositionForPointerExitedEvent: function () {
                if (!this.isAlreadySubscribedToMouseEnterAndLeave && this.INTERNAL_OuterDomElement != null) {
                    this.INTERNAL_OuterDomElement.addEventListener("mouseenter", Bridge.fn.cacheBind(this, this.SetIsPointerInsideToTrue), false);


                    this.isAlreadySubscribedToMouseEnterAndLeave = true;
                }
            },
            StopManagingPointerPositionForPointerExitedEvent: function () {
                if (this.isAlreadySubscribedToMouseEnterAndLeave && this.INTERNAL_OuterDomElement != null) {
                    this.INTERNAL_OuterDomElement.removeEventListener("mouseenter", Bridge.fn.cacheBind(this, this.SetIsPointerInsideToTrue));

                    this.INTERNAL_isPointerInside = false;
                    this.isAlreadySubscribedToMouseEnterAndLeave = false;
                }
            },
            StartManagingPointerPositionForPointerExitedEventIfNeeded: function () {
                if (this._pointerExitedEventManager != null) {
                    this.StartManagingPointerPositionForPointerExitedEvent();
                }
            },
            /**
             * Creates the eventArgs from the infos in the javascript's version of the event then raises the PointerExited event
             *
             * @instance
             * @this Windows.UI.Xaml.UIElement
             * @memberof Windows.UI.Xaml.UIElement
             * @param   {System.Object}    jsEventArg
             * @return  {void}
             */
            ProcessOnPointerExited: function (jsEventArg) {
                this.ProcessPointerEvent(jsEventArg, Bridge.fn.cacheBind(this, this.OnPointerExited), Bridge.fn.cacheBind(this, this.OnPointerExited_ForHandledEventsToo), true, false, false);
            },
            /**
             * Raises the PointerExited event
             *
             * @instance
             * @this Windows.UI.Xaml.UIElement
             * @memberof Windows.UI.Xaml.UIElement
             * @param   {Windows.UI.Xaml.Input.PointerRoutedEventArgs}    eventArgs    The arguments for the event.
             * @return  {void}
             */
            OnPointerExited: function (eventArgs) {
                var $t;
                this.INTERNAL_isPointerInside = false;
                $t = Bridge.getEnumerator(System.Linq.Enumerable.from(this._pointerExitedEventManager.Handlers).toList(Function));
                try {
                    while ($t.moveNext()) {
                        var handler = $t.Current;
                        if (eventArgs.Handled) {
                            break;
                        }
                        handler(this, eventArgs);
                    }
                } finally {
                    if (Bridge.is($t, System.IDisposable)) {
                        $t.System$IDisposable$Dispose();
                    }
                }
            },
            OnPointerExited_ForHandledEventsToo: function (eventArgs) {
                var $t;
                this.INTERNAL_isPointerInside = false;
                $t = Bridge.getEnumerator(System.Linq.Enumerable.from(this._pointerExitedEventManager.HandlersForHandledEventsToo).toList(Function));
                try {
                    while ($t.moveNext()) {
                        var handler = $t.Current;
                        handler(this, eventArgs);
                    }
                } finally {
                    if (Bridge.is($t, System.IDisposable)) {
                        $t.System$IDisposable$Dispose();
                    }
                }
            },
            /**
             * Raises the Tapped event
             *
             * @instance
             * @private
             * @this Windows.UI.Xaml.UIElement
             * @memberof Windows.UI.Xaml.UIElement
             * @param   {System.Object}    jsEventArg
             * @return  {void}
             */
            ProcessOnTapped: function (jsEventArg) {
                var $t, $t1;
                var eventArgs = ($t = new Windows.UI.Xaml.Input.TappedRoutedEventArgs(), $t.INTERNAL_OriginalJSEventArg = jsEventArg, $t.Handled = (Bridge.referenceEquals(Bridge.toString((($t1 = jsEventArg.data, $t1 != null ? $t1 : ""))), "handled")), $t);

                if (eventArgs.CheckIfEventShouldBeTreated(this, jsEventArg)) {
                    eventArgs.FillEventArgs(this, jsEventArg);

                    if (!eventArgs.Handled) {
                        this.OnTapped(eventArgs);
                    }
                    this.OnTapped_ForHandledEventsToo(eventArgs);

                    if (eventArgs.Handled) {
                        jsEventArg.data = 'handled';
                    }
                }
            },
            /**
             * Raises the Tapped event
             *
             * @instance
             * @protected
             * @this Windows.UI.Xaml.UIElement
             * @memberof Windows.UI.Xaml.UIElement
             * @param   {Windows.UI.Xaml.Input.TappedRoutedEventArgs}    eventArgs    The arguments for the event.
             * @return  {void}
             */
            OnTapped: function (eventArgs) {
                var $t;
                $t = Bridge.getEnumerator(System.Linq.Enumerable.from(this._tappedEventManager.Handlers).toList(Function));
                try {
                    while ($t.moveNext()) {
                        var handler = $t.Current;
                        if (eventArgs.Handled) {
                            break;
                        }
                        handler(this, eventArgs);
                    }
                } finally {
                    if (Bridge.is($t, System.IDisposable)) {
                        $t.System$IDisposable$Dispose();
                    }
                }
            },
            OnTapped_ForHandledEventsToo: function (eventArgs) {
                var $t;
                $t = Bridge.getEnumerator(System.Linq.Enumerable.from(this._tappedEventManager.HandlersForHandledEventsToo).toList(Function));
                try {
                    while ($t.moveNext()) {
                        var handler = $t.Current;
                        handler(this, eventArgs);
                    }
                } finally {
                    if (Bridge.is($t, System.IDisposable)) {
                        $t.System$IDisposable$Dispose();
                    }
                }
            },
            /**
             * Raises the RightTapped event
             *
             * @instance
             * @private
             * @this Windows.UI.Xaml.UIElement
             * @memberof Windows.UI.Xaml.UIElement
             * @param   {System.Object}    jsEventArg
             * @return  {void}
             */
            ProcessOnRightTapped: function (jsEventArg) {
                var $t, $t1;
                var eventArgs = ($t = new Windows.UI.Xaml.Input.RightTappedRoutedEventArgs(), $t.INTERNAL_OriginalJSEventArg = jsEventArg, $t.Handled = (Bridge.referenceEquals(Bridge.toString((($t1 = jsEventArg.data, $t1 != null ? $t1 : ""))), "handled")), $t);

                if (eventArgs.CheckIfEventShouldBeTreated(this, jsEventArg)) {
                    eventArgs.FillEventArgs(this, jsEventArg);


                    if (                    jsEventArg.preventDefault)
                        jsEventArg.preventDefault();

                    if (!eventArgs.Handled) {
                        this.OnRightTapped(eventArgs);
                    }
                    this.OnRightTapped_ForHandledEventsToo(eventArgs);

                    if (eventArgs.Handled) {
                        jsEventArg.data = 'handled';
                    }
                }
            },
            /**
             * Raises the RightTapped event
             *
             * @instance
             * @protected
             * @this Windows.UI.Xaml.UIElement
             * @memberof Windows.UI.Xaml.UIElement
             * @param   {Windows.UI.Xaml.Input.RightTappedRoutedEventArgs}    eventArgs    The arguments for the event.
             * @return  {void}
             */
            OnRightTapped: function (eventArgs) {
                var $t;
                $t = Bridge.getEnumerator(System.Linq.Enumerable.from(this._rightTappedEventManager.Handlers).toList(Function));
                try {
                    while ($t.moveNext()) {
                        var handler = $t.Current;
                        if (eventArgs.Handled) {
                            break;
                        }
                        handler(this, eventArgs);
                    }
                } finally {
                    if (Bridge.is($t, System.IDisposable)) {
                        $t.System$IDisposable$Dispose();
                    }
                }
            },
            OnRightTapped_ForHandledEventsToo: function (eventArgs) {
                var $t;
                $t = Bridge.getEnumerator(System.Linq.Enumerable.from(this._rightTappedEventManager.HandlersForHandledEventsToo).toList(Function));
                try {
                    while ($t.moveNext()) {
                        var handler = $t.Current;
                        handler(this, eventArgs);
                    }
                } finally {
                    if (Bridge.is($t, System.IDisposable)) {
                        $t.System$IDisposable$Dispose();
                    }
                }
            },
            /**
             * Raises the OnKeyDown event
             *
             * @instance
             * @private
             * @this Windows.UI.Xaml.UIElement
             * @memberof Windows.UI.Xaml.UIElement
             * @param   {System.Object}    jsEventArg
             * @return  {void}
             */
            ProcessOnKeyDown: function (jsEventArg) {
                var $t, $t1;
                var keyCode = System.Convert.toInt32(jsEventArg.keyCode);

                var eventArgs = ($t = new Windows.UI.Xaml.Input.KeyRoutedEventArgs(), $t.INTERNAL_OriginalJSEventArg = jsEventArg, $t.PlatformKeyCode = keyCode, $t.Key = (Windows.System.INTERNAL_VirtualKeysHelpers.IsUnknownKey(keyCode) ? Windows.System.VirtualKey.Unknown : keyCode), $t.Handled = (Bridge.referenceEquals(Bridge.toString((($t1 = jsEventArg.data, $t1 != null ? $t1 : ""))), "handled")), $t);

                eventArgs.AddKeyModifiersAndUpdateDocumentValue(jsEventArg);

                if (!eventArgs.Handled) {
                    this.OnKeyDown(eventArgs);
                }
                this.OnKeyDown_ForHandledEventsToo(eventArgs);

                if (eventArgs.Handled) {
                    jsEventArg.data = 'handled';
                }
            },
            /**
             * Raises the KeyDown event
             *
             * @instance
             * @protected
             * @this Windows.UI.Xaml.UIElement
             * @memberof Windows.UI.Xaml.UIElement
             * @param   {Windows.UI.Xaml.Input.KeyRoutedEventArgs}    eventArgs    The arguments for the event.
             * @return  {void}
             */
            OnKeyDown: function (eventArgs) {
                var $t;
                $t = Bridge.getEnumerator(System.Linq.Enumerable.from(this._keyDownEventManager.Handlers).toList(Function));
                try {
                    while ($t.moveNext()) {
                        var handler = $t.Current;
                        if (eventArgs.Handled) {
                            break;
                        }
                        handler(this, eventArgs);
                    }
                } finally {
                    if (Bridge.is($t, System.IDisposable)) {
                        $t.System$IDisposable$Dispose();
                    }
                }
            },
            OnKeyDown_ForHandledEventsToo: function (eventArgs) {
                var $t;
                $t = Bridge.getEnumerator(System.Linq.Enumerable.from(this._keyDownEventManager.HandlersForHandledEventsToo).toList(Function));
                try {
                    while ($t.moveNext()) {
                        var handler = $t.Current;
                        handler(this, eventArgs);
                    }
                } finally {
                    if (Bridge.is($t, System.IDisposable)) {
                        $t.System$IDisposable$Dispose();
                    }
                }
            },
            /**
             * Raises the OnKeyUp event
             *
             * @instance
             * @private
             * @this Windows.UI.Xaml.UIElement
             * @memberof Windows.UI.Xaml.UIElement
             * @param   {System.Object}    jsEventArg
             * @return  {void}
             */
            ProcessOnKeyUp: function (jsEventArg) {
                var $t, $t1;
                var keyCode = System.Convert.toInt32(jsEventArg.keyCode);

                var eventArgs = ($t = new Windows.UI.Xaml.Input.KeyRoutedEventArgs(), $t.INTERNAL_OriginalJSEventArg = jsEventArg, $t.PlatformKeyCode = keyCode, $t.Key = (Windows.System.INTERNAL_VirtualKeysHelpers.IsUnknownKey(keyCode) ? Windows.System.VirtualKey.Unknown : keyCode), $t.Handled = (Bridge.referenceEquals(Bridge.toString((($t1 = jsEventArg.data, $t1 != null ? $t1 : ""))), "handled")), $t);

                eventArgs.AddKeyModifiersAndUpdateDocumentValue(jsEventArg);

                if (!eventArgs.Handled) {
                    this.OnKeyUp(eventArgs);
                }
                this.OnKeyUp_ForHandledEventsToo(eventArgs);

                if (eventArgs.Handled) {
                    jsEventArg.data = 'handled';
                }
            },
            /**
             * Raises the KeyUp event
             *
             * @instance
             * @protected
             * @this Windows.UI.Xaml.UIElement
             * @memberof Windows.UI.Xaml.UIElement
             * @param   {Windows.UI.Xaml.Input.KeyRoutedEventArgs}    eventArgs    The arguments for the event.
             * @return  {void}
             */
            OnKeyUp: function (eventArgs) {
                var $t;
                $t = Bridge.getEnumerator(System.Linq.Enumerable.from(this._keyUpEventManager.Handlers).toList(Function));
                try {
                    while ($t.moveNext()) {
                        var handler = $t.Current;
                        if (eventArgs.Handled) {
                            break;
                        }
                        handler(this, eventArgs);
                    }
                } finally {
                    if (Bridge.is($t, System.IDisposable)) {
                        $t.System$IDisposable$Dispose();
                    }
                }
            },
            OnKeyUp_ForHandledEventsToo: function (eventArgs) {
                var $t;
                $t = Bridge.getEnumerator(System.Linq.Enumerable.from(this._keyUpEventManager.HandlersForHandledEventsToo).toList(Function));
                try {
                    while ($t.moveNext()) {
                        var handler = $t.Current;
                        handler(this, eventArgs);
                    }
                } finally {
                    if (Bridge.is($t, System.IDisposable)) {
                        $t.System$IDisposable$Dispose();
                    }
                }
            },
            /**
             * Raises the GotFocus event
             *
             * @instance
             * @private
             * @this Windows.UI.Xaml.UIElement
             * @memberof Windows.UI.Xaml.UIElement
             * @param   {System.Object}    jsEventArg
             * @return  {void}
             */
            ProcessOnGotFocus: function (jsEventArg) {
                var $t;
                var eventArgs = ($t = new Windows.UI.Xaml.RoutedEventArgs(), $t.INTERNAL_OriginalJSEventArg = jsEventArg, $t);
                this.OnGotFocus(eventArgs);
            },
            /**
             * Raises the GotFocus event
             *
             * @instance
             * @protected
             * @this Windows.UI.Xaml.UIElement
             * @memberof Windows.UI.Xaml.UIElement
             * @param   {Windows.UI.Xaml.RoutedEventArgs}    eventArgs    The arguments for the event.
             * @return  {void}
             */
            OnGotFocus: function (eventArgs) {
                var $t;
                $t = Bridge.getEnumerator(System.Linq.Enumerable.from(this._gotFocusEventManager.Handlers).toList(Function));
                try {
                    while ($t.moveNext()) {
                        var handler = $t.Current;
                        handler(this, eventArgs);
                    }
                } finally {
                    if (Bridge.is($t, System.IDisposable)) {
                        $t.System$IDisposable$Dispose();
                    }
                }
            },
            /**
             * Raises the LostFocus event
             *
             * @instance
             * @private
             * @this Windows.UI.Xaml.UIElement
             * @memberof Windows.UI.Xaml.UIElement
             * @param   {System.Object}    jsEventArg
             * @return  {void}
             */
            ProcessOnLostFocus: function (jsEventArg) {
                var $t;
                var eventArgs = ($t = new Windows.UI.Xaml.RoutedEventArgs(), $t.INTERNAL_OriginalJSEventArg = jsEventArg, $t);
                this.OnLostFocus(eventArgs);
            },
            /**
             * Raises the LostFocus event
             *
             * @instance
             * @protected
             * @this Windows.UI.Xaml.UIElement
             * @memberof Windows.UI.Xaml.UIElement
             * @param   {Windows.UI.Xaml.RoutedEventArgs}    eventArgs    The arguments for the event.
             * @return  {void}
             */
            OnLostFocus: function (eventArgs) {
                var $t;
                $t = Bridge.getEnumerator(System.Linq.Enumerable.from(this._lostFocusEventManager.Handlers).toList(Function));
                try {
                    while ($t.moveNext()) {
                        var handler = $t.Current;
                        handler(this, eventArgs);
                    }
                } finally {
                    if (Bridge.is($t, System.IDisposable)) {
                        $t.System$IDisposable$Dispose();
                    }
                }
            },
            StartListeningToKeyboardEvents: function (sender, e) {
                if (!System.Convert.toBoolean(document.checkForDivsThatAbsorbEvents(e.INTERNAL_OriginalJSEventArg))) {
                    this.removeKeyDown(Bridge.fn.cacheBind(this, this.OnKeyDownWhenFocused));
                    this.addKeyDown(Bridge.fn.cacheBind(this, this.OnKeyDownWhenFocused));
                }
            },
            StopListeningToKeyboardEvents: function (sender, e) {
                this.removeKeyDown(Bridge.fn.cacheBind(this, this.OnKeyDownWhenFocused));
            },
            /**
             * This method is to be overriden by UIElements that react to keyboard events when they are focused, to handle those events properly.
             *
             * @instance
             * @this Windows.UI.Xaml.UIElement
             * @memberof Windows.UI.Xaml.UIElement
             * @param   {System.Object}                               sender    
             * @param   {Windows.UI.Xaml.Input.KeyRoutedEventArgs}    e
             * @return  {void}
             */
            OnKeyDownWhenFocused: function (sender, e) { },
            /**
             * Raises the GotFocus event
             *
             * @instance
             * @private
             * @this Windows.UI.Xaml.UIElement
             * @memberof Windows.UI.Xaml.UIElement
             * @param   {System.Object}    jsEventArg
             * @return  {void}
             */
            ProcessOnGotFocusForIsTabStop: function (jsEventArg) {
                var $t;
                var eventArgs = ($t = new Windows.UI.Xaml.RoutedEventArgs(), $t.INTERNAL_OriginalJSEventArg = jsEventArg, $t);
                this.OnGotFocusForIsTabStop(eventArgs);
            },
            /**
             * Raises the GotFocus event
             *
             * @instance
             * @private
             * @this Windows.UI.Xaml.UIElement
             * @memberof Windows.UI.Xaml.UIElement
             * @param   {Windows.UI.Xaml.RoutedEventArgs}    eventArgs    The arguments for the event.
             * @return  {void}
             */
            OnGotFocusForIsTabStop: function (eventArgs) {
                var $t;
                $t = Bridge.getEnumerator(System.Linq.Enumerable.from(this._gotFocusForIsTabStopEventManager.Handlers).toList(Function));
                try {
                    while ($t.moveNext()) {
                        var handler = $t.Current;
                        handler(this, eventArgs);
                    }
                } finally {
                    if (Bridge.is($t, System.IDisposable)) {
                        $t.System$IDisposable$Dispose();
                    }
                }
            },
            PreventFocusEvents: function () {
                if (this.INTERNAL_AreFocusEventsAllowed) {
                    this.INTERNAL_AreFocusEventsAllowed = false;

                    this.INTERNAL_DetachFromFocusEvents();

                    this.removeGotFocusForIsTabStop(Bridge.fn.cacheBind(this, this.UIElement_GotFocusForIsTabStop));
                    this.addGotFocusForIsTabStop(Bridge.fn.cacheBind(this, this.UIElement_GotFocusForIsTabStop));

                }
            },
            UIElement_GotFocusForIsTabStop: function (sender, e) {
                var element = Bridge.cast(sender, Windows.UI.Xaml.UIElement);
                var elementToBlur = element.INTERNAL_OptionalSpecifyDomElementConcernedByFocus || element.INTERNAL_OuterDomElement;
                if (elementToBlur != null) {
                    elementToBlur.blur();
                }
            },
            AllowFocusEvents: function () {
                if (!this.INTERNAL_AreFocusEventsAllowed) {
                    this.INTERNAL_AreFocusEventsAllowed = true;

                    this.INTERNAL_AttachToFocusEvents();
                    this.removeGotFocusForIsTabStop(Bridge.fn.cacheBind(this, this.UIElement_GotFocusForIsTabStop));



                }
            },
            /**
             * DO NOT USE THIS, IT IS ONLY FOR THE ALLOWFOCUSEVENTS METHOD
             *
             * @instance
             * @private
             * @this Windows.UI.Xaml.UIElement
             * @memberof Windows.UI.Xaml.UIElement
             * @return  {void}
             */
            INTERNAL_AttachToFocusEvents: function () {
                if (this._gotFocusEventManager != null) {
                    this._gotFocusEventManager.AttachToDomEvents();
                }
                if (this._lostFocusEventManager != null) {
                    this._lostFocusEventManager.AttachToDomEvents();
                }
            },
            /**
             * DO NOT USE THIS, IT IS ONLY FOR THE PREVENTFOCUSEVENTS METHOD
             *
             * @instance
             * @private
             * @this Windows.UI.Xaml.UIElement
             * @memberof Windows.UI.Xaml.UIElement
             * @return  {void}
             */
            INTERNAL_DetachFromFocusEvents: function () {
                if (this._gotFocusEventManager != null) {
                    this._gotFocusEventManager.DetachFromDomEvents();
                }
                if (this._lostFocusEventManager != null) {
                    this._lostFocusEventManager.DetachFromDomEvents();
                }
            },
            ProcessPointerEvent: function (jsEventArg, onEvent, onEvent_ForHandledEventsToo, preventTextSelectionWhenPointerIsCaptured, checkForDivsThatAbsorbEvents, refreshClickCount) {
                var $t, $t1;
                if (preventTextSelectionWhenPointerIsCaptured === void 0) { preventTextSelectionWhenPointerIsCaptured = false; }
                if (checkForDivsThatAbsorbEvents === void 0) { checkForDivsThatAbsorbEvents = false; }
                if (refreshClickCount === void 0) { refreshClickCount = false; }
                var isMouseEvent = System.Convert.toBoolean(jsEventArg.type.startsWith('mouse'));
                if (!(Windows.UI.Xaml.UIElement.ignoreMouseEvents && isMouseEvent)) {
                    var eventArgs = ($t = new Windows.UI.Xaml.Input.PointerRoutedEventArgs(), $t.INTERNAL_OriginalJSEventArg = jsEventArg, $t.Handled = (Bridge.referenceEquals(Bridge.toString((($t1 = jsEventArg.data, $t1 != null ? $t1 : ""))), "handled")), $t);

                    if (!eventArgs.Handled && checkForDivsThatAbsorbEvents) {
                        eventArgs.Handled = System.Convert.toBoolean(document.checkForDivsThatAbsorbEvents(jsEventArg));
                    }

                    if (refreshClickCount) {
                        eventArgs.RefreshClickCount(this);
                    }

                    if (eventArgs.CheckIfEventShouldBeTreated(this, jsEventArg)) {
                        eventArgs.FillEventArgs(this, jsEventArg);

                        if (!eventArgs.Handled) {
                            onEvent(eventArgs);
                        }
                        onEvent_ForHandledEventsToo(eventArgs);

                        if (eventArgs.Handled) {
                            jsEventArg.data = 'handled';
                        }
                    }

                    if (preventTextSelectionWhenPointerIsCaptured && Windows.UI.Xaml.Input.Pointer.INTERNAL_captured != null) {
                        window.getSelection().removeAllRanges();
                    }
                    var isTouchEndEvent = System.Convert.toBoolean(jsEventArg.type == 'touchend');
                    if (isTouchEndEvent) {
                        Windows.UI.Xaml.UIElement.ignoreMouseEvents = true;
                        if (Windows.UI.Xaml.UIElement._ignoreMouseEventsTimer == null) {
                            Windows.UI.Xaml.UIElement._ignoreMouseEventsTimer = ($t = new Windows.UI.Xaml.DispatcherTimer(), $t.Interval = new System.TimeSpan(0, 0, 0, 0, 100), $t);
                            Windows.UI.Xaml.UIElement._ignoreMouseEventsTimer.addTick(Bridge.fn.cacheBind(this, this._ignoreMouseEventsTimer_Tick));

                        }
                        Windows.UI.Xaml.UIElement._ignoreMouseEventsTimer.Stop();
                        Windows.UI.Xaml.UIElement._ignoreMouseEventsTimer.Start();
                    }
                }
            },
            INTERNAL_AttachToDomEvents: function () {
                if (this._reactsToKeyboardEventsWhenFocused) {
                    this.removeGotFocus(Bridge.fn.cacheBind(this, this.StartListeningToKeyboardEvents));
                    this.addGotFocus(Bridge.fn.cacheBind(this, this.StartListeningToKeyboardEvents));
                    this.removeLostFocus(Bridge.fn.cacheBind(this, this.StopListeningToKeyboardEvents));
                    this.addLostFocus(Bridge.fn.cacheBind(this, this.StopListeningToKeyboardEvents));
                }

                if (this._pointerMovedEventManager != null) {
                    this._pointerMovedEventManager.AttachToDomEvents();
                }
                if (this._pointerPressedEventManager != null) {
                    this._pointerPressedEventManager.AttachToDomEvents();
                }
                if (this._pointerReleasedEventManager != null) {
                    this._pointerReleasedEventManager.AttachToDomEvents();
                }
                if (this._pointerEnteredEventManager != null) {
                    this._pointerEnteredEventManager.AttachToDomEvents();
                }
                if (this._pointerExitedEventManager != null) {
                    this._pointerExitedEventManager.AttachToDomEvents();
                }
                if (this._tappedEventManager != null) {
                    this._tappedEventManager.AttachToDomEvents();
                }
                if (this._rightTappedEventManager != null) {
                    this._rightTappedEventManager.AttachToDomEvents();
                }
                if (this._keyDownEventManager != null) {
                    this._keyDownEventManager.AttachToDomEvents();
                }
                if (this._keyUpEventManager != null) {
                    this._keyUpEventManager.AttachToDomEvents();
                }
                if (this._gotFocusEventManager != null) {
                    this._gotFocusEventManager.AttachToDomEvents();
                }
                if (this._lostFocusEventManager != null) {
                    this._lostFocusEventManager.AttachToDomEvents();
                }
            },
            INTERNAL_DetachFromDomEvents: function () {
                if (this._pointerMovedEventManager != null) {
                    this._pointerMovedEventManager.DetachFromDomEvents();
                }
                if (this._pointerPressedEventManager != null) {
                    this._pointerPressedEventManager.DetachFromDomEvents();
                }
                if (this._pointerReleasedEventManager != null) {
                    this._pointerReleasedEventManager.DetachFromDomEvents();
                }
                if (this._pointerEnteredEventManager != null) {
                    this._pointerEnteredEventManager.DetachFromDomEvents();
                }
                if (this._pointerExitedEventManager != null) {
                    this._pointerExitedEventManager.DetachFromDomEvents();
                }
                if (this._tappedEventManager != null) {
                    this._tappedEventManager.DetachFromDomEvents();
                }
                if (this._rightTappedEventManager != null) {
                    this._rightTappedEventManager.DetachFromDomEvents();
                }
                if (this._keyDownEventManager != null) {
                    this._keyDownEventManager.DetachFromDomEvents();
                }
                if (this._keyUpEventManager != null) {
                    this._keyUpEventManager.DetachFromDomEvents();
                }
                if (this._gotFocusEventManager != null) {
                    this._gotFocusEventManager.DetachFromDomEvents();
                }
                if (this._lostFocusEventManager != null) {
                    this._lostFocusEventManager.DetachFromDomEvents();
                }
            },
            /**
             * Adds a routed event handler for a specified routed event, adding the handler
             to the handler collection on the current element. Specify handledEventsToo
             as true to have the provided handler be invoked for a routed event case that
             had already been marked as handled by another element along the event route.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.UIElement
             * @memberof Windows.UI.Xaml.UIElement
             * @param   {Windows.UI.Xaml.RoutedEvent}    routedEvent         An identifier for the routed event to be handled.
             * @param   {System.Object}                  handler             A reference to the handler implementation.
             * @param   {boolean}                        handledEventsToo    True to register the handler such that it is invoked even when the routed
             event is marked handled in its event data. False to register the handler
             with the default condition that it will not be invoked if the routed event
             is already marked handled. The default is false.
             * @return  {void}
             */
            AddHandler: function (routedEvent, handler, handledEventsToo) {
                if (Bridge.referenceEquals(routedEvent, Windows.UI.Xaml.UIElement.PointerMovedEvent)) {
                    this.PointerMovedEventManager.Add(handler, handledEventsToo);
                } else if (Bridge.referenceEquals(routedEvent, Windows.UI.Xaml.UIElement.PointerPressedEvent)) {
                    this.PointerPressedEventManager.Add(handler, handledEventsToo);
                } else if (Bridge.referenceEquals(routedEvent, Windows.UI.Xaml.UIElement.PointerReleasedEvent)) {
                    this.PointerReleasedEventManager.Add(handler, handledEventsToo);
                } else if (Bridge.referenceEquals(routedEvent, Windows.UI.Xaml.UIElement.PointerEnteredEvent)) {
                    this.PointerEnteredEventManager.Add(handler, handledEventsToo);
                } else if (Bridge.referenceEquals(routedEvent, Windows.UI.Xaml.UIElement.PointerExitedEvent)) {
                    this.PointerExitedEventManager.Add(handler, handledEventsToo);
                } else if (Bridge.referenceEquals(routedEvent, Windows.UI.Xaml.UIElement.TappedEvent)) {
                    this.TappedEventManager.Add(handler, handledEventsToo);
                } else if (Bridge.referenceEquals(routedEvent, Windows.UI.Xaml.UIElement.RightTappedEvent)) {
                    this.RightTappedEventManager.Add(handler, handledEventsToo);
                } else if (Bridge.referenceEquals(routedEvent, Windows.UI.Xaml.UIElement.KeyDownEvent)) {
                    this.KeyDownEventManager.Add(handler, handledEventsToo);
                } else if (Bridge.referenceEquals(routedEvent, Windows.UI.Xaml.UIElement.KeyUpEvent)) {
                    this.KeyUpEventManager.Add(handler, handledEventsToo);
                } else if (Bridge.referenceEquals(routedEvent, Windows.UI.Xaml.UIElement.GotFocusEvent)) {
                    this.GotFocusEventManager.Add(handler, handledEventsToo);
                } else if (Bridge.referenceEquals(routedEvent, Windows.UI.Xaml.UIElement.LostFocusEvent)) {
                    this.LostFocusEventManager.Add(handler, handledEventsToo);
                } else {
                    throw new System.NotSupportedException.$ctor1("The following routed event cannot be used in the AddHandler method: " + (routedEvent.toString() || "") + " - Please contact support.");
                }
            },
            /**
             * Removes the specified routed event handler from this UIElement. Typically
             the handler in question was added by AddHandler.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.UIElement
             * @memberof Windows.UI.Xaml.UIElement
             * @param   {Windows.UI.Xaml.RoutedEvent}    routedEvent    The identifier of the routed event for which the handler is attached.
             * @param   {System.Object}                  handler        The specific handler implementation to remove from the event handler collection
             on this UIElement.
             * @return  {void}
             */
            RemoveHandler: function (routedEvent, handler) {
                if (Bridge.referenceEquals(routedEvent, Windows.UI.Xaml.UIElement.PointerMovedEvent)) {
                    this.PointerMovedEventManager.Remove(handler);
                } else if (Bridge.referenceEquals(routedEvent, Windows.UI.Xaml.UIElement.PointerPressedEvent)) {
                    this.PointerPressedEventManager.Remove(handler);
                } else if (Bridge.referenceEquals(routedEvent, Windows.UI.Xaml.UIElement.PointerReleasedEvent)) {
                    this.PointerReleasedEventManager.Remove(handler);
                } else if (Bridge.referenceEquals(routedEvent, Windows.UI.Xaml.UIElement.PointerEnteredEvent)) {
                    this.PointerEnteredEventManager.Remove(handler);
                } else if (Bridge.referenceEquals(routedEvent, Windows.UI.Xaml.UIElement.PointerExitedEvent)) {
                    this.PointerExitedEventManager.Remove(handler);
                } else if (Bridge.referenceEquals(routedEvent, Windows.UI.Xaml.UIElement.TappedEvent)) {
                    this.TappedEventManager.Remove(handler);
                } else if (Bridge.referenceEquals(routedEvent, Windows.UI.Xaml.UIElement.RightTappedEvent)) {
                    this.RightTappedEventManager.Remove(handler);
                } else if (Bridge.referenceEquals(routedEvent, Windows.UI.Xaml.UIElement.KeyDownEvent)) {
                    this.KeyDownEventManager.Remove(handler);
                } else if (Bridge.referenceEquals(routedEvent, Windows.UI.Xaml.UIElement.KeyUpEvent)) {
                    this.KeyUpEventManager.Remove(handler);
                } else if (Bridge.referenceEquals(routedEvent, Windows.UI.Xaml.UIElement.GotFocusEvent)) {
                    this.GotFocusEventManager.Remove(handler);
                } else if (Bridge.referenceEquals(routedEvent, Windows.UI.Xaml.UIElement.LostFocusEvent)) {
                    this.LostFocusEventManager.Remove(handler);
                } else {
                    throw new System.NotSupportedException.$ctor1("The following routed event cannot be used in the RemoveHandler method: " + (routedEvent.toString() || "") + " - Please contact support.");
                }
            }
        }
    });

    
    Bridge.define("CSHTML5.Native.Html.Controls.ImageElement", {
        inherits: [CSHTML5.Native.Html.Controls.HtmlCanvasElement],
        fields: {
            _jsImage: null,
            _sourceCache: null,
            _source: null,
            _lastDrawnSource: null,
            _lastDrawnWidth: 0,
            _lastDrawnHeight: 0,
            /**
             * Width of the container
             *
             * @instance
             * @public
             * @memberof CSHTML5.Native.Html.Controls.ImageElement
             * @type number
             */
            Width: 0,
            /**
             * Height of the container
             *
             * @instance
             * @public
             * @memberof CSHTML5.Native.Html.Controls.ImageElement
             * @type number
             */
            Height: 0
        },
        props: {
            /**
             * Gets or sets the source for the image. It is possible to use either the
             syntax "ms-appx:AssemblyName/FolderName/FileName.png" or the syntax
             "/AssemblyName;component/FolderName/FileName.png".
             *
             * @instance
             * @public
             * @memberof CSHTML5.Native.Html.Controls.ImageElement
             * @function Source
             * @type string
             */
            Source: {
                get: function () {
                    return this._source;
                },
                set: function (value) {
                    this._source = value;
                    this._sourceCache = CSHTML5.Internal.INTERNAL_UriHelper.ConvertToHtml5Path(value, null);
                }
            }
        },
        ctors: {
            /**
             * Default image constructor
             *
             * @instance
             * @public
             * @this CSHTML5.Native.Html.Controls.ImageElement
             * @memberof CSHTML5.Native.Html.Controls.ImageElement
             * @return  {void}
             */
            ctor: function () {
                this.$initialize();
                CSHTML5.Native.Html.Controls.HtmlCanvasElement.ctor.call(this);
                this._jsImage = document.createElement('img');
            }
        },
        methods: {
            /**
             * Draws the image
             *
             * @instance
             * @public
             * @override
             * @this CSHTML5.Native.Html.Controls.ImageElement
             * @memberof CSHTML5.Native.Html.Controls.ImageElement
             * @param   {CSHTML5.Native.Html.Controls.ElementStyle}    currentDrawingStyle    Draw style used for last element (can allow optimizations, null if unknown)
             * @param   {System.Object}                                jsContext2d            Canvas 2d javascript context
             * @param   {number}                                       xParent                X position of the parent element
             * @param   {number}                                       yParent                Y position of the parent element
             * @return  {CSHTML5.Native.Html.Controls.ElementStyle}                           Draw style used for this element (can be null)
             */
            Draw: function (currentDrawingStyle, jsContext2d, xParent, yParent) {
                if (xParent === void 0) { xParent = 0.0; }
                if (yParent === void 0) { yParent = 0.0; }
                if (this.Visibility === Windows.UI.Xaml.Visibility.Visible) {
                    currentDrawingStyle = this.ApplyStyle(currentDrawingStyle, jsContext2d);

                    if (!Bridge.referenceEquals(this._lastDrawnSource, this._sourceCache) || this._lastDrawnWidth !== this.Width || this._lastDrawnHeight !== this.Height) {

                        this._jsImage.width = Bridge.box(this.Width, System.Double, System.Double.format, System.Double.getHashCode);
this._jsImage.height = Bridge.box(this.Height, System.Double, System.Double.format, System.Double.getHashCode);
this._jsImage.src = this._sourceCache;
                        this._lastDrawnSource = this._sourceCache;
                        this._lastDrawnWidth = this.Width;
                        this._lastDrawnHeight = this.Height;
                    }

                    jsContext2d.drawImage(this._jsImage, Bridge.box(this.X + xParent, System.Double, System.Double.format, System.Double.getHashCode), Bridge.box(this.Y + yParent, System.Double, System.Double.format, System.Double.getHashCode));
                }

                return currentDrawingStyle;
            },
            IsPointed: function (x, y) {
                return x >= this.X && x < this.X + this.Width && y >= this.Y && y < this.Y + this.Height;
            }
        }
    });

    /**
     * Html5 canvas line.
     Full documentation is available at:
     http://cshtml5.com/links/how-to-use-the-html5-canvas.aspx
     *
     * @public
     * @class CSHTML5.Native.Html.Controls.LineElement
     * @augments CSHTML5.Native.Html.Controls.HtmlCanvasElement
     * @example You can add a line to the XAML as follows:
 
     Or in C#:
     *
     * 
     *     
     * 
     * 
     *
     * HtmlCanvas myCanvas = new HtmlCanvas() { Width = 1000, Height = 500 };
     * LineElement myLine = new LineElement()
     * {
     *     X1 = 200,
     *     Y1 = 42,
     *     X2 = 100,
     *     Y2 = 50,
     *     StrokeColor = Colors.Red,
     *     LineWidth = 2
     * };
     * myCanvas.Children.Add(myLine);
     * 
     *
     *
     */
    Bridge.define("CSHTML5.Native.Html.Controls.LineElement", {
        inherits: [CSHTML5.Native.Html.Controls.HtmlCanvasElement],
        fields: {
            /**
             * X coordinate of the start point
             *
             * @instance
             * @public
             * @memberof CSHTML5.Native.Html.Controls.LineElement
             * @type number
             */
            X1: 0,
            /**
             * Y coordinate of the start point
             *
             * @instance
             * @public
             * @memberof CSHTML5.Native.Html.Controls.LineElement
             * @type number
             */
            Y1: 0,
            /**
             * X coordinate of the end point
             *
             * @instance
             * @public
             * @memberof CSHTML5.Native.Html.Controls.LineElement
             * @type number
             */
            X2: 0,
            /**
             * Y coordinate of the end point
             *
             * @instance
             * @public
             * @memberof CSHTML5.Native.Html.Controls.LineElement
             * @type number
             */
            Y2: 0
        },
        ctors: {
            /**
             * Default line constructor
             *
             * @instance
             * @public
             * @this CSHTML5.Native.Html.Controls.LineElement
             * @memberof CSHTML5.Native.Html.Controls.LineElement
             * @return  {void}
             */
            ctor: function () {
                this.$initialize();
                CSHTML5.Native.Html.Controls.HtmlCanvasElement.ctor.call(this);

            },
            /**
             * Create line with specified start and end points
             *
             * @instance
             * @public
             * @this CSHTML5.Native.Html.Controls.LineElement
             * @memberof CSHTML5.Native.Html.Controls.LineElement
             * @param   {number}    X1    X coordinate of the start point
             * @param   {number}    Y1    Y coordinate of the start point
             * @param   {number}    X2    X coordinate of the end point
             * @param   {number}    Y2    Y coordinate of the end point
             * @return  {void}
             */
            $ctor1: function (X1, Y1, X2, Y2) {
                this.$initialize();
                CSHTML5.Native.Html.Controls.HtmlCanvasElement.ctor.call(this);
                this.X1 = X1;
                this.Y1 = Y1;
                this.X2 = X2;
                this.Y2 = Y2;
            }
        },
        methods: {
            /**
             * Draws the line
             *
             * @instance
             * @public
             * @override
             * @this CSHTML5.Native.Html.Controls.LineElement
             * @memberof CSHTML5.Native.Html.Controls.LineElement
             * @param   {CSHTML5.Native.Html.Controls.ElementStyle}    currentDrawingStyle    Draw style used for last element (can allow optimizations, null if unknown)
             * @param   {System.Object}                                jsContext2d            Canvas 2d javascript context
             * @param   {number}                                       xParent                X position of the parent element
             * @param   {number}                                       yParent                Y position of the parent element
             * @return  {CSHTML5.Native.Html.Controls.ElementStyle}                           Draw style used for this element (can be null)
             */
            Draw: function (currentDrawingStyle, jsContext2d, xParent, yParent) {
                if (xParent === void 0) { xParent = 0.0; }
                if (yParent === void 0) { yParent = 0.0; }
                if (this.Visibility === Windows.UI.Xaml.Visibility.Visible) {
                    currentDrawingStyle = this.ApplyStyle(currentDrawingStyle, jsContext2d);
                    jsContext2d.beginPath();
                    jsContext2d.moveTo(Bridge.box(this.X1 + xParent, System.Double, System.Double.format, System.Double.getHashCode), Bridge.box(this.Y1 + yParent, System.Double, System.Double.format, System.Double.getHashCode));
                    jsContext2d.lineTo(Bridge.box(this.X2 + xParent, System.Double, System.Double.format, System.Double.getHashCode), Bridge.box(this.Y2 + yParent, System.Double, System.Double.format, System.Double.getHashCode));
                    jsContext2d.stroke();
                }

                return currentDrawingStyle;
            },
            IsPointed: function (x, y) {
                return false;
            }
        }
    });

    
    Bridge.define("CSHTML5.Native.Html.Controls.RectangleElement", {
        inherits: [CSHTML5.Native.Html.Controls.HtmlCanvasElement],
        fields: {
            /**
             * Width of the rectangle
             *
             * @instance
             * @public
             * @memberof CSHTML5.Native.Html.Controls.RectangleElement
             * @type number
             */
            Width: 0,
            /**
             * Height of the rectangle
             *
             * @instance
             * @public
             * @memberof CSHTML5.Native.Html.Controls.RectangleElement
             * @type number
             */
            Height: 0
        },
        ctors: {
            /**
             * Default rectangle constructor
             *
             * @instance
             * @public
             * @this CSHTML5.Native.Html.Controls.RectangleElement
             * @memberof CSHTML5.Native.Html.Controls.RectangleElement
             * @return  {void}
             */
            ctor: function () {
                this.$initialize();
                CSHTML5.Native.Html.Controls.HtmlCanvasElement.ctor.call(this);
                this.Width = 0;
                this.Height = 0;
            },
            /**
             * Constructor with specified position and size
             *
             * @instance
             * @public
             * @this CSHTML5.Native.Html.Controls.RectangleElement
             * @memberof CSHTML5.Native.Html.Controls.RectangleElement
             * @param   {number}    X         X position
             * @param   {number}    Y         Y position
             * @param   {number}    Width     Width of the rectangle
             * @param   {number}    Height    Height of the rectangle
             * @return  {void}
             */
            $ctor1: function (X, Y, Width, Height) {
                this.$initialize();
                CSHTML5.Native.Html.Controls.HtmlCanvasElement.$ctor1.call(this, X, Y);
                this.Width = Width;
                this.Height = Height;
            }
        },
        methods: {
            /**
             * Draws the rectangle
             *
             * @instance
             * @public
             * @override
             * @this CSHTML5.Native.Html.Controls.RectangleElement
             * @memberof CSHTML5.Native.Html.Controls.RectangleElement
             * @param   {CSHTML5.Native.Html.Controls.ElementStyle}    currentDrawingStyle    Draw style used for last element (can allow optimizations, null if unknown)
             * @param   {System.Object}                                jsContext2d            Canvas 2d javascript context
             * @param   {number}                                       xParent                X position of the parent element
             * @param   {number}                                       yParent                Y position of the parent element
             * @return  {CSHTML5.Native.Html.Controls.ElementStyle}                           Draw style used for this element (can be null)
             */
            Draw: function (currentDrawingStyle, jsContext2d, xParent, yParent) {
                if (xParent === void 0) { xParent = 0.0; }
                if (yParent === void 0) { yParent = 0.0; }
                if (this.Visibility === Windows.UI.Xaml.Visibility.Visible) {
                    currentDrawingStyle = this.ApplyStyle(currentDrawingStyle, jsContext2d);
                    jsContext2d.fillRect(Bridge.box(this.X + xParent, System.Double, System.Double.format, System.Double.getHashCode), Bridge.box(this.Y + yParent, System.Double, System.Double.format, System.Double.getHashCode), Bridge.box(this.Width, System.Double, System.Double.format, System.Double.getHashCode), Bridge.box(this.Height, System.Double, System.Double.format, System.Double.getHashCode));
                    jsContext2d.strokeRect(Bridge.box(this.X + xParent, System.Double, System.Double.format, System.Double.getHashCode), Bridge.box(this.Y + yParent, System.Double, System.Double.format, System.Double.getHashCode), Bridge.box(this.Width, System.Double, System.Double.format, System.Double.getHashCode), Bridge.box(this.Height, System.Double, System.Double.format, System.Double.getHashCode));
                }

                return currentDrawingStyle;
            },
            IsPointed: function (x, y) {
                return x >= this.X && x < this.X + this.Width && y >= this.Y && y < this.Y + this.Height;
            }
        }
    });

    
    Bridge.define("CSHTML5.Native.Html.Controls.TextElement", {
        inherits: [CSHTML5.Native.Html.Controls.HtmlCanvasElement],
        statics: {
            fields: {
                DEFAULT_FONT_HEIGHT: 0,
                _jsCanvasForMeasuringTextWidth: null
            },
            ctors: {
                init: function () {
                    this.DEFAULT_FONT_HEIGHT = 12;
                }
            }
        },
        fields: {
            /**
             * Text value
             *
             * @instance
             * @public
             * @memberof CSHTML5.Native.Html.Controls.TextElement
             * @default ""
             * @type string
             */
            Text: null,
            /**
             * Font of the text, in javascript format (eg. "20px Arial")
             *
             * @instance
             * @public
             * @memberof CSHTML5.Native.Html.Controls.TextElement
             * @type string
             */
            Font: null,
            FontHeight: 0,
            FontWeight: null
        },
        props: {
            ActualWidth$1: {
                get: function () {
                    if (CSHTML5.Native.Html.Controls.TextElement._jsCanvasForMeasuringTextWidth == null) {
                        CSHTML5.Native.Html.Controls.TextElement._jsCanvasForMeasuringTextWidth = document.createElement('canvas').getContext('2d');
                    }
                    return System.Convert.toDouble(CSHTML5.Native.Html.Controls.TextElement._jsCanvasForMeasuringTextWidth.measureText(this.Text).width) * this.FontHeight / CSHTML5.Native.Html.Controls.TextElement.DEFAULT_FONT_HEIGHT;
                }
            }
        },
        ctors: {
            init: function () {
                this.FontWeight = new Windows.UI.Text.FontWeight();
                this.Text = "";
            },
            /**
             * Default text constructor (Black, 12px, Georgia)
             *
             * @instance
             * @public
             * @this CSHTML5.Native.Html.Controls.TextElement
             * @memberof CSHTML5.Native.Html.Controls.TextElement
             * @return  {void}
             */
            ctor: function () {
                this.$initialize();
                CSHTML5.Native.Html.Controls.HtmlCanvasElement.ctor.call(this);
                this.FillColor = Windows.UI.Color.FromArgb(255, 0, 0, 0);
                this.FontWeight = Windows.UI.Text.FontWeights.Normal.$clone();
                this.FontHeight = CSHTML5.Native.Html.Controls.TextElement.DEFAULT_FONT_HEIGHT;
                this.Font = "Georgia";
            }
        },
        methods: {
            /**
             * Draws the text
             *
             * @instance
             * @public
             * @override
             * @this CSHTML5.Native.Html.Controls.TextElement
             * @memberof CSHTML5.Native.Html.Controls.TextElement
             * @param   {CSHTML5.Native.Html.Controls.ElementStyle}    currentDrawingStyle    Draw style used for last element (can allow optimizations, null if unknown)
             * @param   {System.Object}                                jsContext2d            Canvas 2d javascript context
             * @param   {number}                                       xParent                X position of the parent element
             * @param   {number}                                       yParent                Y position of the parent element
             * @return  {CSHTML5.Native.Html.Controls.ElementStyle}                           Draw style used for this element (can be null)
             */
            Draw: function (currentDrawingStyle, jsContext2d, xParent, yParent) {
                if (xParent === void 0) { xParent = 0.0; }
                if (yParent === void 0) { yParent = 0.0; }
                if (this.Visibility === Windows.UI.Xaml.Visibility.Visible) {
                    currentDrawingStyle = this.ApplyStyle(currentDrawingStyle, jsContext2d);

                    jsContext2d.font = (this.FontWeight.toString() || "") + " " + (System.Double.format(this.FontHeight) || "") + "px " + (this.Font || "");
                    jsContext2d.fillText(this.Text, Bridge.box(this.X + xParent, System.Double, System.Double.format, System.Double.getHashCode), Bridge.box(this.Y + yParent + this.FontHeight, System.Double, System.Double.format, System.Double.getHashCode));
                }

                return currentDrawingStyle;
            },
            IsPointed: function (x, y) {
                return x >= this.X && x < this.X + this.ActualWidth$1 && y >= this.Y && y < this.Y + this.ActualHeight;
            }
        }
    });

    /**
     * Provides event data for pointer message events related to specific user interface
     elements, such as PointerPressed.
     *
     * @public
     * @class Windows.UI.Xaml.Input.PointerRoutedEventArgs
     * @augments Windows.UI.Xaml.RoutedEventArgs
     */
    Bridge.define("Windows.UI.Xaml.Input.PointerRoutedEventArgs", {
        inherits: [Windows.UI.Xaml.RoutedEventArgs],
        fields: {
            _pointerAbsoluteX: 0,
            _pointerAbsoluteY: 0,
            _handled: false,
            _keyModifiers: 0,
            _pointer: null,
            _clickCount: 0
        },
        props: {
            /**
             * Gets or sets a value that marks the routed event as handled, and prevents
             most handlers along the event route from handling the same event again.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Input.PointerRoutedEventArgs
             * @function Handled
             * @type boolean
             */
            Handled: {
                get: function () {
                    return this._handled;
                },
                set: function (value) {
                    this._handled = value;
                }
            },
            /**
             * Gets a value that indicates which key modifiers were active at the time that
             the pointer event was initiated.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Input.PointerRoutedEventArgs
             * @function KeyModifiers
             * @type Windows.System.VirtualKeyModifiers
             */
            KeyModifiers: {
                get: function () {
                    return this._keyModifiers;
                },
                set: function (value) {
                    this._keyModifiers = value;
                }
            },
            /**
             * Gets a reference to a pointer token.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Input.PointerRoutedEventArgs
             * @function Pointer
             * @type Windows.UI.Xaml.Input.Pointer
             */
            Pointer: {
                get: function () {
                    return this._pointer;
                },
                set: function (value) {
                    this._pointer = value;
                }
            },
            /**
             * Gets the number of times the button was clicked.
             *
             * @instance
             * @public
             * @readonly
             * @memberof Windows.UI.Xaml.Input.PointerRoutedEventArgs
             * @function ClickCount
             * @type number
             */
            ClickCount: {
                get: function () {
                    return this._clickCount;
                }
            }
        },
        ctors: {
            init: function () {
                this._pointerAbsoluteX = 0.0;
                this._pointerAbsoluteY = 0.0;
                this._handled = false;
                this._clickCount = 1;
            }
        },
        methods: {
            CheckIfEventShouldBeTreated: function (element, jsEventArg) {

                if (Windows.UI.Xaml.Input.Pointer.INTERNAL_captured == null) {
                    jsEventArg.doNotReroute = true;
                    return true;
                } else {
                    if (Bridge.referenceEquals(Windows.UI.Xaml.Input.Pointer.INTERNAL_captured, element)) {
                        jsEventArg.doNotReroute = true;
                        return true;
                    } else {
                        if (System.Convert.toBoolean(jsEventArg.doNotReroute == true)) {
                            return true;
                        }
                        return false;
                    }
                }
            },
            FillEventArgs: function (element, jsEventArg) {
                if (Windows.UI.Xaml.Input.Pointer.INTERNAL_captured == null || Bridge.referenceEquals(Windows.UI.Xaml.Input.Pointer.INTERNAL_captured, element)) {
                    jsEventArg.doNotReroute = true;
                }

                this.AddKeyModifiers(jsEventArg);
                this.SetPointerAbsolutePosition(jsEventArg, element.INTERNAL_ParentWindow);
            },
            AddKeyModifiers: function (jsEventArg) {
                var keyModifiers = Windows.System.VirtualKeyModifiers.None;
                if (System.Convert.toBoolean(jsEventArg.shiftKey || false)) {
                    keyModifiers = keyModifiers | Windows.System.VirtualKeyModifiers.Shift;
                }
                if (System.Convert.toBoolean(jsEventArg.altKey || false)) {
                    keyModifiers = keyModifiers | Windows.System.VirtualKeyModifiers.Menu;
                }
                if (System.Convert.toBoolean(jsEventArg.ctrlKey || false)) {
                    keyModifiers = keyModifiers | Windows.System.VirtualKeyModifiers.Control;
                }
                this.KeyModifiers = keyModifiers;
            },
            SetPointerAbsolutePosition: function (jsEventArg, $window) {
                if (CSHTML5.Interop.IsRunningInTheSimulator) {
                    var concatenated = System.Convert.toString(jsEventArg.pageX + '|' + jsEventArg.pageY);
                    var sepIndex = System.String.indexOf(concatenated, String.fromCharCode(124));
                    var pointerAbsoluteXAsString = concatenated.substr(0, sepIndex);
                    var pointerAbsoluteYAsString = concatenated.substr(((sepIndex + 1) | 0));
                    this._pointerAbsoluteX = Bridge.Int.parseFloat(pointerAbsoluteXAsString, System.Globalization.CultureInfo.invariantCulture);
                    this._pointerAbsoluteY = Bridge.Int.parseFloat(pointerAbsoluteYAsString, System.Globalization.CultureInfo.invariantCulture);
                } else {
                    var jsEventArgDynamic = jsEventArg;
                    var isArgsPageXDefined = CSHTML5.Internal.INTERNAL_HtmlDomManager.IsNotUndefinedOrNull(jsEventArgDynamic.pageX);
                    var isArgsPageXNotNull = isArgsPageXDefined;
                    if (isArgsPageXDefined) {
                        isArgsPageXNotNull = !Bridge.referenceEquals(jsEventArgDynamic.pageX, 0);
                    }
                    if (isArgsPageXNotNull) {
                        this._pointerAbsoluteX = Bridge.cast(jsEventArgDynamic.pageX, System.Double);
                        this._pointerAbsoluteY = Bridge.cast(jsEventArgDynamic.pageY, System.Double);
                    } else {
                        var isArgsTouchesDefined = CSHTML5.Internal.INTERNAL_HtmlDomManager.IsNotUndefinedOrNull(jsEventArgDynamic.touches);
                        var isArgsTouchesNotEmpty = isArgsTouchesDefined;
                        if (isArgsTouchesDefined) {
                            isArgsTouchesNotEmpty = !Bridge.referenceEquals(jsEventArgDynamic.touches.length, 0);
                        }
                        if (isArgsTouchesNotEmpty) {
                            this._pointerAbsoluteX = Bridge.cast(jsEventArgDynamic.touches[0].pageX, System.Double);
                            this._pointerAbsoluteY = Bridge.cast(jsEventArgDynamic.touches[0].pageY, System.Double);
                        } else {
                            var isArgsChangedTouchesDefined = CSHTML5.Internal.INTERNAL_HtmlDomManager.IsNotUndefinedOrNull(jsEventArgDynamic.changedTouches);
                            var isArgsChangedTouchesNotEmpty = isArgsChangedTouchesDefined;
                            if (isArgsChangedTouchesDefined) {
                                isArgsChangedTouchesNotEmpty = !Bridge.referenceEquals(jsEventArgDynamic.changedTouches.length, 0);
                            }
                            if (isArgsChangedTouchesNotEmpty) {
                                this._pointerAbsoluteX = Bridge.cast(jsEventArgDynamic.changedTouches[0].pageX, System.Double);
                                this._pointerAbsoluteY = Bridge.cast(jsEventArgDynamic.changedTouches[0].pageY, System.Double);
                            } else {
                                this._pointerAbsoluteX = 0.0;
                                this._pointerAbsoluteY = 0.0;
                            }
                        }
                    }
                }

                if ($window != null) {
                    var windowRootDomElement = $window.INTERNAL_OuterDomElement;
                    var windowBoundingClientRect = windowRootDomElement.getBoundingClientRect();
                    var pageBodyBoundingClientRect = document.body.getBoundingClientRect();

                    var windowRootLeft;
                    var windowRootTop;

                    var concatenated1 = Bridge.toString((windowBoundingClientRect.left - pageBodyBoundingClientRect.left) + '|' + (windowBoundingClientRect.top - pageBodyBoundingClientRect.top));
                    var sepIndex1 = System.String.indexOf(concatenated1, String.fromCharCode(124));
                    if (sepIndex1 > -1) {
                        var windowRootLeftAsString = concatenated1.substr(0, sepIndex1);
                        var windowRootTopAsString = concatenated1.substr(((sepIndex1 + 1) | 0));
                        windowRootLeft = Bridge.Int.parseFloat(windowRootLeftAsString, Bridge.global.System.Globalization.CultureInfo.invariantCulture);
                        windowRootTop = Bridge.Int.parseFloat(windowRootTopAsString, Bridge.global.System.Globalization.CultureInfo.invariantCulture);
                    } else {
                        windowRootLeft = Number.NaN;
                        windowRootTop = Number.NaN;
                    }

                    this._pointerAbsoluteX = this._pointerAbsoluteX - windowRootLeft;
                    this._pointerAbsoluteY = this._pointerAbsoluteY - windowRootTop;
                }
            },
            RefreshClickCount: function (sender) {
                var currentDate = Date.now();
                if (((currentDate - sender.INTERNAL_lastClickDate) | 0) > 400) {
                    sender.INTERNAL_clickCount = 1;
                } else {
                    sender.INTERNAL_clickCount = (sender.INTERNAL_clickCount + 1) | 0;
                }
                sender.INTERNAL_lastClickDate = currentDate;
                this._clickCount = sender.INTERNAL_clickCount;
            },
            /**
             * Returns the pointer position for this event occurrence, optionally evaluated
             against a coordinate origin of a supplied UIElement.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Input.PointerRoutedEventArgs
             * @memberof Windows.UI.Xaml.Input.PointerRoutedEventArgs
             * @param   {Windows.UI.Xaml.UIElement}        relativeTo    Any UIElement-derived object that is connected to the same object tree. To
             specify the object relative to the overall coordinate system, use a relativeTo value
             of null.
             * @return  {Windows.UI.Input.PointerPoint}                  A PointerPoint value that represents the pointer point associated with this
             event. If null was passed as relativeTo, the coordinates are in the frame
             of reference of the overall window. If a non-null relativeTo was passed,
             the coordinates are relative to the object referenced by relativeTo.
             */
            GetCurrentPoint: function (relativeTo) {
                var $t;
                if (relativeTo == null) {
                    var pointerPoint = ($t = new Windows.UI.Input.PointerPoint(), $t.Position = new Windows.Foundation.Point.$ctor1(this._pointerAbsoluteX, this._pointerAbsoluteY), $t);
                    return pointerPoint;
                } else {

                    var generalTransform = Windows.UI.Xaml.Window.Current.TransformToVisual(relativeTo);

                    var pointerPoint1 = ($t = new Windows.UI.Input.PointerPoint(), $t.Position = generalTransform.TransformPoint(new Windows.Foundation.Point.$ctor1(this._pointerAbsoluteX, this._pointerAbsoluteY)), $t);
                    return pointerPoint1;
                }
            }
        }
    });

    /**
     * @memberof System.ComponentModel
     * @callback System.ComponentModel.PropertyChangedEventHandler
     * @param   {System.Object}                                     sender    
     * @param   {System.ComponentModel.PropertyChangedEventArgs}    e
     * @return  {void}
     */

    /**
     * @memberof System.Collections.Specialized
     * @callback System.Collections.Specialized.NotifyCollectionChangedEventHandler
     * @param   {System.Object}                                                      sender    
     * @param   {System.Collections.Specialized.NotifyCollectionChangedEventArgs}    e
     * @return  {void}
     */

    /**
     * Implementation of a dynamic data collection based on generic Collection&lt;T&gt;,
     implementing INotifyCollectionChanged to notify listeners
     when items get added, removed or the whole list is refreshed.
     *
     * @public
     * @class System.Collections.ObjectModel.ObservableCollection$1
     * @augments System.Collections.ObjectModel.Collection$1
     * @implements  System.Collections.Specialized.INotifyCollectionChanged
     * @implements  System.ComponentModel.INotifyPropertyChanged
     */
    Bridge.define("System.Collections.ObjectModel.ObservableCollection$1", function (T) { return {
        inherits: [System.Collections.ObjectModel.Collection$1(T),System.Collections.Specialized.INotifyCollectionChanged,System.ComponentModel.INotifyPropertyChanged],
        statics: {
            fields: {
                CountString: null,
                IndexerName: null
            },
            ctors: {
                init: function () {
                    this.CountString = "Count";
                    this.IndexerName = "Item[]";
                }
            }
        },
        fields: {
            _monitor: null
        },
        events: {
            /**
             * Occurs when the collection changes, either by adding or removing an item.
             *
             * @instance
             * @public
             * @this System.Collections.ObjectModel.ObservableCollection$1
             * @memberof System.Collections.ObjectModel.ObservableCollection$1
             * @function addCollectionChanged
             * @param   {System.Collections.Specialized.NotifyCollectionChangedEventHandler}    value
             * @return  {void}
             * @see {@link INotifyCollectionChanged}
             */
            /**
             * Occurs when the collection changes, either by adding or removing an item.
             *
             * @instance
             * @public
             * @this System.Collections.ObjectModel.ObservableCollection$1
             * @memberof System.Collections.ObjectModel.ObservableCollection$1
             * @function removeCollectionChanged
             * @param   {System.Collections.Specialized.NotifyCollectionChangedEventHandler}    value
             * @return  {void}
             * @see {@link INotifyCollectionChanged}
             */
            CollectionChanged: null,
            /**
             * PropertyChanged event (per {@link }).
             *
             * @instance
             * @protected
             * @this System.Collections.ObjectModel.ObservableCollection$1
             * @memberof System.Collections.ObjectModel.ObservableCollection$1
             * @function addPropertyChanged
             * @param   {System.ComponentModel.PropertyChangedEventHandler}    value
             * @return  {void}
             */
            /**
             * PropertyChanged event (per {@link }).
             *
             * @instance
             * @protected
             * @this System.Collections.ObjectModel.ObservableCollection$1
             * @memberof System.Collections.ObjectModel.ObservableCollection$1
             * @function removePropertyChanged
             * @param   {System.ComponentModel.PropertyChangedEventHandler}    value
             * @return  {void}
             */
            PropertyChanged: null
        },
        alias: ["addCollectionChanged", "System$Collections$Specialized$INotifyCollectionChanged$addCollectionChanged",
        "removeCollectionChanged", "System$Collections$Specialized$INotifyCollectionChanged$removeCollectionChanged"],
        ctors: {
            init: function () {
                this._monitor = new (System.Collections.ObjectModel.ObservableCollection$1.SimpleMonitor(T))();
            },
            /**
             * Initializes a new instance of ObservableCollection that is empty and has default initial capacity.
             *
             * @instance
             * @public
             * @this System.Collections.ObjectModel.ObservableCollection$1
             * @memberof System.Collections.ObjectModel.ObservableCollection$1
             * @return  {void}
             */
            ctor: function () {
                this.$initialize();
                System.Collections.ObjectModel.Collection$1(T).ctor.call(this);
            },
            /**
             * Initializes a new instance of the ObservableCollection class
             that contains elements copied from the specified list
             *
             * @instance
             * @public
             * @this System.Collections.ObjectModel.ObservableCollection$1
             * @memberof System.Collections.ObjectModel.ObservableCollection$1
             * @throws list is a null reference
             * @param   {System.Collections.Generic.List$1}    list    The list whose elements are copied to the new list.
             * @return  {void}
             */
            $ctor2: function (list) {
                this.$initialize();
                System.Collections.ObjectModel.Collection$1(T).$ctor1.call(this, (list != null) ? new (System.Collections.Generic.List$1(T)).$ctor2(list.Count) : list);
                this.CopyFrom(list);
            },
            /**
             * Initializes a new instance of the ObservableCollection class that contains
             elements copied from the specified collection and has sufficient capacity
             to accommodate the number of elements copied.
             *
             * @instance
             * @public
             * @this System.Collections.ObjectModel.ObservableCollection$1
             * @memberof System.Collections.ObjectModel.ObservableCollection$1
             * @throws collection is a null reference
             * @param   {System.Collections.Generic.IEnumerable$1}    collection    The collection whose elements are copied to the new list.
             * @return  {void}
             */
            $ctor1: function (collection) {
                this.$initialize();
                System.Collections.ObjectModel.Collection$1(T).ctor.call(this);
                if (collection == null) {
                    throw new System.ArgumentNullException.$ctor1("collection");
                }

                this.CopyFrom(collection);
            }
        },
        methods: {
            /**
             * PropertyChanged event (per {@link }).
             *
             * @instance
             * @memberof System.Collections.ObjectModel.ObservableCollection$1
             * @event System.Collections.ObjectModel.ObservableCollection$1#PropertyChanged
             * @return  {System.ComponentModel.PropertyChangedEventHandler}
             */
            System$ComponentModel$INotifyPropertyChanged$addPropertyChanged: function (value) {
                this.addPropertyChanged(value);
            },
            /**
             * PropertyChanged event (per {@link }).
             *
             * @instance
             * @memberof System.Collections.ObjectModel.ObservableCollection$1
             * @event System.Collections.ObjectModel.ObservableCollection$1#PropertyChanged
             * @return  {System.ComponentModel.PropertyChangedEventHandler}
             */
            System$ComponentModel$INotifyPropertyChanged$removePropertyChanged: function (value) {
                this.removePropertyChanged(value);
            },
            CopyFrom: function (collection) {
                var items = this.Items;
                if (collection != null && items != null) {
                    var enumerator = Bridge.getEnumerator(collection, T);
                    try {
                        while (enumerator.System$Collections$IEnumerator$moveNext()) {
                            System.Array.add(items, enumerator[Bridge.geti(enumerator, "System$Collections$Generic$IEnumerator$1$" + Bridge.getTypeAlias(T) + "$Current$1", "System$Collections$Generic$IEnumerator$1$Current$1")], T);
                        }
                    }
                    finally {
                        if (Bridge.hasValue(enumerator)) {
                            enumerator.System$IDisposable$Dispose();
                        }
                    }
                }
            },
            /**
             * Move item at oldIndex to newIndex.
             *
             * @instance
             * @public
             * @this System.Collections.ObjectModel.ObservableCollection$1
             * @memberof System.Collections.ObjectModel.ObservableCollection$1
             * @param   {number}    oldIndex    
             * @param   {number}    newIndex
             * @return  {void}
             */
            Move: function (oldIndex, newIndex) {
                this.MoveItem(oldIndex, newIndex);
            },
            /**
             * Called by base class Collection&lt;T&gt; when the list is being cleared;
             raises a CollectionChanged event to any listeners.
             *
             * @instance
             * @protected
             * @override
             * @this System.Collections.ObjectModel.ObservableCollection$1
             * @memberof System.Collections.ObjectModel.ObservableCollection$1
             * @return  {void}
             */
            ClearItems: function () {
                this.CheckReentrancy();
                System.Collections.ObjectModel.Collection$1(T).prototype.ClearItems.call(this);
                this.OnPropertyChanged$1(System.Collections.ObjectModel.ObservableCollection$1(T).CountString);
                this.OnPropertyChanged$1(System.Collections.ObjectModel.ObservableCollection$1(T).IndexerName);
                this.OnCollectionReset();
            },
            /**
             * Called by base class Collection&lt;T&gt; when an item is removed from list;
             raises a CollectionChanged event to any listeners.
             *
             * @instance
             * @protected
             * @override
             * @this System.Collections.ObjectModel.ObservableCollection$1
             * @memberof System.Collections.ObjectModel.ObservableCollection$1
             * @param   {number}    index
             * @return  {void}
             */
            RemoveItem: function (index) {
                this.CheckReentrancy();
                var removedItem = this.getItem(index);

                System.Collections.ObjectModel.Collection$1(T).prototype.RemoveItem.call(this, index);

                this.OnPropertyChanged$1(System.Collections.ObjectModel.ObservableCollection$1(T).CountString);
                this.OnPropertyChanged$1(System.Collections.ObjectModel.ObservableCollection$1(T).IndexerName);
                this.OnCollectionChanged$1(System.Collections.Specialized.NotifyCollectionChangedAction.Remove, removedItem, index);
            },
            /**
             * Called by base class Collection&lt;T&gt; when an item is added to list;
             raises a CollectionChanged event to any listeners.
             *
             * @instance
             * @protected
             * @override
             * @this System.Collections.ObjectModel.ObservableCollection$1
             * @memberof System.Collections.ObjectModel.ObservableCollection$1
             * @param   {number}    index    
             * @param   {T}         item
             * @return  {void}
             */
            InsertItem: function (index, item) {
                this.CheckReentrancy();
                System.Collections.ObjectModel.Collection$1(T).prototype.InsertItem.call(this, index, item);

                this.OnPropertyChanged$1(System.Collections.ObjectModel.ObservableCollection$1(T).CountString);
                this.OnPropertyChanged$1(System.Collections.ObjectModel.ObservableCollection$1(T).IndexerName);
                this.OnCollectionChanged$1(System.Collections.Specialized.NotifyCollectionChangedAction.Add, item, index);
            },
            /**
             * Called by base class Collection&lt;T&gt; when an item is set in list;
             raises a CollectionChanged event to any listeners.
             *
             * @instance
             * @protected
             * @override
             * @this System.Collections.ObjectModel.ObservableCollection$1
             * @memberof System.Collections.ObjectModel.ObservableCollection$1
             * @param   {number}    index    
             * @param   {T}         item
             * @return  {void}
             */
            SetItem: function (index, item) {
                this.CheckReentrancy();
                var originalItem = this.getItem(index);
                System.Collections.ObjectModel.Collection$1(T).prototype.SetItem.call(this, index, item);

                this.OnPropertyChanged$1(System.Collections.ObjectModel.ObservableCollection$1(T).IndexerName);
                this.OnCollectionChanged$3(System.Collections.Specialized.NotifyCollectionChangedAction.Replace, originalItem, item, index);
            },
            /**
             * Called by base class ObservableCollection&lt;T&gt; when an item is to be moved within the list;
             raises a CollectionChanged event to any listeners.
             *
             * @instance
             * @protected
             * @this System.Collections.ObjectModel.ObservableCollection$1
             * @memberof System.Collections.ObjectModel.ObservableCollection$1
             * @param   {number}    oldIndex    
             * @param   {number}    newIndex
             * @return  {void}
             */
            MoveItem: function (oldIndex, newIndex) {
                this.CheckReentrancy();

                var removedItem = this.getItem(oldIndex);

                System.Collections.ObjectModel.Collection$1(T).prototype.RemoveItem.call(this, oldIndex);
                System.Collections.ObjectModel.Collection$1(T).prototype.InsertItem.call(this, newIndex, removedItem);

                this.OnPropertyChanged$1(System.Collections.ObjectModel.ObservableCollection$1(T).IndexerName);
                this.OnCollectionChanged$2(System.Collections.Specialized.NotifyCollectionChangedAction.Move, removedItem, newIndex, oldIndex);
            },
            /**
             * Raises a PropertyChanged event (per {@link }).
             *
             * @instance
             * @protected
             * @this System.Collections.ObjectModel.ObservableCollection$1
             * @memberof System.Collections.ObjectModel.ObservableCollection$1
             * @param   {System.ComponentModel.PropertyChangedEventArgs}    e
             * @return  {void}
             */
            OnPropertyChanged: function (e) {
                if (!Bridge.staticEquals(this.PropertyChanged, null)) {
                    this.PropertyChanged(this, e);
                }
            },
            /**
             * Helper to raise a PropertyChanged event  /&gt;).
             *
             * @instance
             * @private
             * @this System.Collections.ObjectModel.ObservableCollection$1
             * @memberof System.Collections.ObjectModel.ObservableCollection$1
             * @param   {string}    propertyName
             * @return  {void}
             */
            OnPropertyChanged$1: function (propertyName) {
                this.OnPropertyChanged(new System.ComponentModel.PropertyChangedEventArgs(propertyName));
            },
            /**
             * Raise CollectionChanged event to any listeners.
             Properties/methods modifying this ObservableCollection will raise
             a collection changed event through this virtual method.
             *
             * @instance
             * @protected
             * @this System.Collections.ObjectModel.ObservableCollection$1
             * @memberof System.Collections.ObjectModel.ObservableCollection$1
             * @param   {System.Collections.Specialized.NotifyCollectionChangedEventArgs}    e
             * @return  {void}
             */
            OnCollectionChanged: function (e) {
                var $t;
                if (!Bridge.staticEquals(this.CollectionChanged, null)) {
                    $t = this.BlockReentrancy();
                    try {
                        this.CollectionChanged(this, e);
                    }
                    finally {
                        if (Bridge.hasValue($t)) {
                            $t.System$IDisposable$Dispose();
                        }
                    }
                }
            },
            /**
             * Helper to raise CollectionChanged event to any listeners
             *
             * @instance
             * @private
             * @this System.Collections.ObjectModel.ObservableCollection$1
             * @memberof System.Collections.ObjectModel.ObservableCollection$1
             * @param   {System.Collections.Specialized.NotifyCollectionChangedAction}    action    
             * @param   {System.Object}                                                   item      
             * @param   {number}                                                          index
             * @return  {void}
             */
            OnCollectionChanged$1: function (action, item, index) {
                this.OnCollectionChanged(new System.Collections.Specialized.NotifyCollectionChangedEventArgs.$ctor7(action, item, index));
            },
            /**
             * Helper to raise CollectionChanged event to any listeners
             *
             * @instance
             * @private
             * @this System.Collections.ObjectModel.ObservableCollection$1
             * @memberof System.Collections.ObjectModel.ObservableCollection$1
             * @param   {System.Collections.Specialized.NotifyCollectionChangedAction}    action      
             * @param   {System.Object}                                                   item        
             * @param   {number}                                                          index       
             * @param   {number}                                                          oldIndex
             * @return  {void}
             */
            OnCollectionChanged$2: function (action, item, index, oldIndex) {
                this.OnCollectionChanged(new System.Collections.Specialized.NotifyCollectionChangedEventArgs.$ctor8(action, item, index, oldIndex));
            },
            /**
             * Helper to raise CollectionChanged event to any listeners
             *
             * @instance
             * @private
             * @this System.Collections.ObjectModel.ObservableCollection$1
             * @memberof System.Collections.ObjectModel.ObservableCollection$1
             * @param   {System.Collections.Specialized.NotifyCollectionChangedAction}    action     
             * @param   {System.Object}                                                   oldItem    
             * @param   {System.Object}                                                   newItem    
             * @param   {number}                                                          index
             * @return  {void}
             */
            OnCollectionChanged$3: function (action, oldItem, newItem, index) {
                this.OnCollectionChanged(new System.Collections.Specialized.NotifyCollectionChangedEventArgs.$ctor10(action, newItem, oldItem, index));
            },
            /**
             * Disallow reentrant attempts to change this collection. E.g. a event handler
             of the CollectionChanged event is not allowed to make changes to this collection.
             *
             * @instance
             * @protected
             * @this System.Collections.ObjectModel.ObservableCollection$1
             * @memberof System.Collections.ObjectModel.ObservableCollection$1
             * @return  {System.IDisposable}
             */
            BlockReentrancy: function () {
                this._monitor.Enter();
                return this._monitor;
            },
            /**
             * Check and assert for reentrant attempts to change this collection.
             *
             * @instance
             * @protected
             * @this System.Collections.ObjectModel.ObservableCollection$1
             * @memberof System.Collections.ObjectModel.ObservableCollection$1
             * @throws raised when changing the collection
             while another collection change is still being notified to other listeners
             * @return  {void}
             */
            CheckReentrancy: function () {
                if (this._monitor.Busy) {
                    if ((!Bridge.staticEquals(this.CollectionChanged, null)) && (Bridge.fn.getInvocationList(this.CollectionChanged).length > 1)) {
                        throw new System.InvalidOperationException.$ctor1("Observable collection Reentrancy not allowed");
                    }
                }
            },
            /**
             * Helper to raise CollectionChanged event with action == Reset to any listeners
             *
             * @instance
             * @private
             * @this System.Collections.ObjectModel.ObservableCollection$1
             * @memberof System.Collections.ObjectModel.ObservableCollection$1
             * @return  {void}
             */
            OnCollectionReset: function () {
                this.OnCollectionChanged(new System.Collections.Specialized.NotifyCollectionChangedEventArgs.ctor(System.Collections.Specialized.NotifyCollectionChangedAction.Reset));
            }
        }
    }; });

    /**
     * Provides data for the System.Windows.Window.Closing event.
     *
     * @public
     * @class System.ComponentModel.ClosingEventArgs
     * @augments System.ComponentModel.CancelEventArgs
     */
    Bridge.define("System.ComponentModel.ClosingEventArgs", {
        inherits: [System.ComponentModel.CancelEventArgs],
        fields: {
            INTERNAL_JSArgs: null,
            _isCancelable: false,
            _showCloseConfirmationDialog: false
        },
        props: {
            /**
             * Gets a value that indicates whether you can cancel the Window.Closing event.
             *
             * @instance
             * @public
             * @readonly
             * @memberof System.ComponentModel.ClosingEventArgs
             * @function IsCancelable
             * @type boolean
             */
            IsCancelable: {
                get: function () {
                    return this._isCancelable;
                }
            },
            ShowCloseConfirmationDialog: {
                get: function () {
                    return this._showCloseConfirmationDialog;
                },
                set: function (value) {
                    this._showCloseConfirmationDialog = value;
                    if (this.INTERNAL_JSArgs != null) {
                        if (value) {
                            this.INTERNAL_JSArgs.returnValue = "The modifications you made may not be saved.";
                        } else {
                            this.INTERNAL_JSArgs.returnValue = "";
                        }
                    }
                }
            }
        },
        ctors: {
            init: function () {
                this._isCancelable = true;
            },
            /**
             * Initializes a new instance of the System.ComponentModel.ClosingEventArgs
             class.
             *
             * @instance
             * @public
             * @this System.ComponentModel.ClosingEventArgs
             * @memberof System.ComponentModel.ClosingEventArgs
             * @param   {boolean}    isCancelable    Initializes the System.ComponentModel.ClosingEventArgs.IsCancelable property.
             * @return  {void}
             */
            ctor: function (isCancelable) {
                this.$initialize();
                System.ComponentModel.CancelEventArgs.ctor.call(this);
                this._isCancelable = isCancelable;
            }
        }
    });

    /**
     * Provides data for the {@link } event.
     *
     * @public
     * @class System.ComponentModel.PageChangingEventArgs
     * @augments System.ComponentModel.CancelEventArgs
     */
    Bridge.define("System.ComponentModel.PageChangingEventArgs", {
        inherits: [System.ComponentModel.CancelEventArgs],
        fields: {
            /**
             * Gets the index of the requested page.
             *
             * @instance
             * @public
             * @memberof System.ComponentModel.PageChangingEventArgs
             * @function NewPageIndex
             * @type number
             * @return  {[type]}        The index of the requested page.
             */
            NewPageIndex: 0
        },
        ctors: {
            /**
             * Initializes a new instance of the {@link } class.
             *
             * @instance
             * @public
             * @this System.ComponentModel.PageChangingEventArgs
             * @memberof System.ComponentModel.PageChangingEventArgs
             * @param   {number}    newPageIndex    The index of the requested page.
             * @return  {void}
             */
            ctor: function (newPageIndex) {
                this.$initialize();
                System.ComponentModel.CancelEventArgs.ctor.call(this);
                this.NewPageIndex = newPageIndex;
            }
        }
    });

    Bridge.define("System.Net.HttpRequestAsyncResult", {
        inherits: [System.ServiceModel.INTERNAL_AsyncResult],
        ctors: {
            ctor: function (callBack, state) {
                this.$initialize();
                System.ServiceModel.INTERNAL_AsyncResult.ctor.call(this, callBack, state);

            }
        }
    });

    Bridge.define("System.Net.HttpWebRequest", {
        inherits: [System.Net.WebRequest],
        fields: {
            _requestUri: null,
            _methodName: null,
            _headers: null
        },
        props: {
            RequestUri: {
                get: function () {
                    return this._requestUri;
                }
            },
            Method: {
                get: function () {
                    return this._methodName;
                },
                set: function (value) {
                    this._methodName = value;
                }
            },
            Headers: {
                get: function () {
                    return this._headers;
                },
                set: function (value) {
                    this._headers = value;
                }
            }
        },
        ctors: {
            ctor: function () {
                this.$initialize();
                System.Net.WebRequest.ctor.call(this);
                this._methodName = "GET";
                this._headers = new System.Net.WebHeaderCollection();
            },
            $ctor1: function (requestUri) {
                System.Net.HttpWebRequest.ctor.call(this);
                this._requestUri = requestUri;
            }
        },
        methods: {
            GetResponse: function () {
                throw new System.NotImplementedException.ctor();
            },
            BeginGetResponse: function (callback, state) {
                var $t;
                var result = new System.Net.HttpRequestAsyncResult(callback, state);
                var headers = new (System.Collections.Generic.Dictionary$2(System.String,System.String)).ctor();
                $t = Bridge.getEnumerator(this.Headers.AllKeys);
                try {
                    while ($t.moveNext()) {
                        var key = $t.Current;
                        headers.add(key, this.Headers.Get(key));
                    }
                } finally {
                    if (Bridge.is($t, System.IDisposable)) {
                        $t.System$IDisposable$Dispose();
                    }
                }
                this._webRequestHelper.MakeRequest(this.RequestUri, this.Method, this, headers, null, function (o, e) {
                    callback(result);
                }, true, System.Net.CredentialsMode.Disabled);
                return result;
            },
            EndGetResponse: function (asyncResult) {
                var castedAsyncResult = Bridge.as(asyncResult, System.Net.HttpRequestAsyncResult);
                if (castedAsyncResult == null) {
                    throw new System.ArgumentException.$ctor1("asyncResult");
                }

                var response = new System.Net.HttpWebResponse(this._webRequestHelper.GetXmlHttpRequest());
                return response;
            }
        }
    });

    /**
     * Provides an HTTP-specific implementation of the System.Net.WebResponse class.
     *
     * @public
     * @class System.Net.HttpWebResponse
     * @augments System.Net.WebResponse
     */
    Bridge.define("System.Net.HttpWebResponse", {
        inherits: [System.Net.WebResponse],
        fields: {
            _xmlHttpRequest: null
        },
        props: {
            /**
             * Gets the status description returned with the response.
             *
             * @instance
             * @public
             * @readonly
             * @memberof System.Net.HttpWebResponse
             * @function StatusDescription
             * @type string
             */
            StatusDescription: {
                get: function () {
                    return System.Convert.toString(this._xmlHttpRequest.statusText);
                }
            }
        },
        ctors: {
            ctor: function (xmlHttpRequest) {
                this.$initialize();
                System.Net.WebResponse.ctor.call(this);
                this._xmlHttpRequest = xmlHttpRequest;
            }
        },
        methods: {
            /**
             * Gets the status of the response.
             *
             * @instance
             * @public
             * @this System.Net.HttpWebResponse
             * @memberof System.Net.HttpWebResponse
             * @return  {System.Net.HttpStatusCode}
             */
            StatusCode: function () {
                var statusCode = System.Convert.toInt32(this._xmlHttpRequest.status);
                return statusCode;
            },
            /**
             * Gets the stream that is used to read the body of the response from the server.
             *
             * @instance
             * @public
             * @override
             * @this System.Net.HttpWebResponse
             * @memberof System.Net.HttpWebResponse
             * @return  {System.IO.Stream}        A System.IO.Stream containing the body of the response.
             */
            GetResponseStream: function () {
                var stream = new System.IO.MemoryStream.ctor();
                var writer = new System.IO.StreamWriter.$ctor1(stream);
                writer.Write$10(this.GetResponseAsString(this._xmlHttpRequest));
                writer.Flush();
                stream.Position = System.Int64(0);
                return stream;
            },
            GetResponseAsString: function (xmlHttpRequest) {
                return System.Convert.toString(xmlHttpRequest.responseText);
            }
        }
    });

    Bridge.define("System.Net.WebClientWithCredentials", {
        inherits: [System.Net.WebClient],
        statics: {
            fields: {
                containerDictionary_SimulatorOnly: null
            },
            ctors: {
                init: function () {
                    this.containerDictionary_SimulatorOnly = new (System.Collections.Generic.Dictionary$2(System.String,System.Net.WebClientWithCredentials.CookiesContainer)).ctor();
                }
            },
            methods: {
                GetContainer: function (address) {
                    if (!System.Net.WebClientWithCredentials.containerDictionary_SimulatorOnly.containsKey(address)) {
                        System.Net.WebClientWithCredentials.containerDictionary_SimulatorOnly.add(address, new System.Net.WebClientWithCredentials.CookiesContainer());
                    }

                    return System.Net.WebClientWithCredentials.containerDictionary_SimulatorOnly.getItem(address);
                }
            }
        },
        fields: {
            _credentialsMode: 0
        },
        props: {
            CredentialsMode: {
                get: function () {
                    return this._credentialsMode;
                },
                set: function (value) {
                    this._credentialsMode = value;
                }
            }
        },
        ctors: {
            ctor: function (credentialsMode) {
                if (credentialsMode === void 0) { credentialsMode = 2; }

                this.$initialize();
                System.Net.WebClient.ctor.call(this);
                this.CredentialsMode = credentialsMode;
            }
        },
        methods: {
            INTERNAL_AddNewCookiesInContainer_SimulatorOnly: function (container) {
                var $t;
                if (CSHTML5.Interop.IsRunningInTheSimulator) {

                    var myWebHeaderCollection = this.ResponseHeaders;

                    if (myWebHeaderCollection != null) {
                        var allowOrigin = myWebHeaderCollection.Get("Access-Control-Allow-Origin");
                        var wrongAccessOrigin = (Bridge.referenceEquals(allowOrigin, "*") || Bridge.referenceEquals(allowOrigin, "") || allowOrigin == null);

                        if (!wrongAccessOrigin) {
                            var Header = myWebHeaderCollection.Get("Set-Cookie");

                            if (Header != null) {
                                var cookies = System.String.split(Header, [44].map(function (i) {{ return String.fromCharCode(i); }}));

                                $t = Bridge.getEnumerator(cookies);
                                try {
                                    while ($t.moveNext()) {
                                        var cookie = $t.Current;
                                        if (System.String.contains(cookie,"ASP.NET")) {
                                            container.LastSessionId = cookie;
                                        } else {
                                            container.Cookies.add(cookie);
                                        }
                                    }
                                } finally {
                                    if (Bridge.is($t, System.IDisposable)) {
                                        $t.System$IDisposable$Dispose();
                                    }
                                }
                            }
                        }
                    }
                }
            },
            INTERNAL_addFakeOrigin_SimulatorOnly: function () {
                if (CSHTML5.Interop.IsRunningInTheSimulator) {
                    this.Headers.Add$3("Origin", "http://cshtml5-Simulator");
                }
            },
            INTERNAL_AddCookiesInRequest_SimulatorOnly: function (address) {
                var $t;
                var container = System.Net.WebClientWithCredentials.GetContainer(address);

                $t = Bridge.getEnumerator(container.Cookies);
                try {
                    while ($t.moveNext()) {
                        var cookie = $t.Current;
                        this.Headers.Add$3("cookie", cookie);
                    }
                } finally {
                    if (Bridge.is($t, System.IDisposable)) {
                        $t.System$IDisposable$Dispose();
                    }
                }

                if (container.Cookies.Count > 0 && container.LastSessionId != null && !Bridge.referenceEquals(container.LastSessionId, "")) {
                    this.Headers.Add$3("cookie", container.LastSessionId);
                }

            },
            INTERNAL_AddHeaders_SimulatorOnly: function (address) {
                this.INTERNAL_addFakeOrigin_SimulatorOnly();
                this.INTERNAL_AddCookiesInRequest_SimulatorOnly(address);
            },
            UploadStringAsync$2: function (address, data) {
                this.UploadStringAsync$4(address, null, data, null);
            },
            UploadStringAsync$3: function (address, method, data) {
                this.UploadStringAsync$4(address, method, data, null);
            },
            UploadStringAsync$4: function (address, method, data, userToken) {
                if (this.CredentialsMode !== System.Net.CredentialsMode.Disabled) {
                    this.INTERNAL_AddHeaders_SimulatorOnly(address.getOriginalString());
                }

                if (CSHTML5.Interop.IsRunningInTheSimulator) {
                    this.addUploadStringCompleted(Bridge.fn.cacheBind(System.Net.WebClientWithCredentials.GetContainer(address.getOriginalString()), System.Net.WebClientWithCredentials.GetContainer(address.getOriginalString()).INTERNAL_OnAsyncResponse));
                }

                this.UploadStringAsync$1(address, method, data);
            },
            UploadString$5: function (address, data) {
                return this.UploadString$8(new System.Uri(address), "POST", data);
            },
            UploadString$6: function (address, method, data) {
                return this.UploadString$8(new System.Uri(address), method, data);
            },
            UploadString$7: function (address, data) {
                return this.UploadString$8(address, "POST", data);
            },
            UploadString$8: function (address, method, data) {
                if (this.CredentialsMode !== System.Net.CredentialsMode.Disabled) {
                    this.INTERNAL_AddHeaders_SimulatorOnly(address.getOriginalString());
                }

                var response = this.UploadString$3(address, method, data);

                if (this.CredentialsMode !== System.Net.CredentialsMode.Disabled) {
                    this.INTERNAL_AddNewCookiesInContainer_SimulatorOnly(System.Net.WebClientWithCredentials.GetContainer(address.getOriginalString()));
                }

                return response;
            }
        }
    });

    /**
     * @abstract
     * @public
     * @class System.ServiceModel.HttpBindingBase
     * @augments System.ServiceModel.Channels.Binding
     */
    Bridge.define("System.ServiceModel.HttpBindingBase", {
        inherits: [System.ServiceModel.Channels.Binding],
        fields: {
            /**
             * Gets or sets the maximum amount of memory, in bytes, that is allocated for
             use by the manager of the message buffers that receive messages from the
             channel.
             *
             * @instance
             * @public
             * @memberof System.ServiceModel.HttpBindingBase
             * @function MaxBufferPoolSize
             * @type System.Int64
             */
            MaxBufferPoolSize: System.Int64(0),
            /**
             * Gets or sets the maximum size, in bytes, for a buffer that receives messages
             from the channel.
             *
             * @instance
             * @public
             * @memberof System.ServiceModel.HttpBindingBase
             * @function MaxBufferSize
             * @type number
             */
            MaxBufferSize: 0,
            /**
             * Gets or sets the maximum size, in bytes, for a message that can be received
             on a channel configured with this binding.
             *
             * @instance
             * @public
             * @memberof System.ServiceModel.HttpBindingBase
             * @function MaxReceivedMessageSize
             * @type System.Int64
             */
            MaxReceivedMessageSize: System.Int64(0),
            /**
             * Gets or sets a value that indicates whether the client accepts cookies and
             propagates them on future requests.
             *
             * @instance
             * @public
             * @memberof System.ServiceModel.HttpBindingBase
             * @function AllowCookies
             * @type boolean
             */
            AllowCookies: false
        }
    });

    /**
     * Stores the results from an asynchronous call made by the client.
     *
     * @protected
     * @class System.ServiceModel.CSHTML5_ClientBase$1.InvokeAsyncCompletedEventArgs
     * @augments System.ComponentModel.AsyncCompletedEventArgs
     */
    Bridge.define("System.ServiceModel.CSHTML5_ClientBase$1.InvokeAsyncCompletedEventArgs", function (TChannel) { return {
        inherits: [System.ComponentModel.AsyncCompletedEventArgs],
        $kind: "nested class",
        fields: {
            results: null
        },
        props: {
            Results: {
                get: function () {
                    return this.results;
                }
            }
        },
        ctors: {
            ctor: function (results, error, cancelled, userState) {
                this.$initialize();
                System.ComponentModel.AsyncCompletedEventArgs.ctor.call(this, error, cancelled, userState);
                this.results = results;
            }
        }
    }; });

    Bridge.define("System.ServiceModel.INTERNAL_WebMethodsCaller.WebMethodAsyncResult", {
        inherits: [System.ServiceModel.INTERNAL_AsyncResult],
        $kind: "nested class",
        fields: {
            XmlReturnedFromTheServer: null
        },
        ctors: {
            ctor: function (callback, state) {
                this.$initialize();
                System.ServiceModel.INTERNAL_AsyncResult.ctor.call(this, callback, state);
            }
        }
    });

    /**
     * Specifies that a member is serialized as an element inside the SOAP body.
     *
     * @public
     * @class System.ServiceModel.MessageBodyMemberAttribute
     * @augments System.ServiceModel.MessageContractMemberAttribute
     */
    Bridge.define("System.ServiceModel.MessageBodyMemberAttribute", {
        inherits: [System.ServiceModel.MessageContractMemberAttribute],
        fields: {
            /**
             * Gets or sets a value that indicates the position in which the member is serialized
             into the SOAP body.
             *
             * @instance
             * @public
             * @memberof System.ServiceModel.MessageBodyMemberAttribute
             * @function Order
             * @type number
             */
            Order: 0
        },
        ctors: {
            /**
             * Initializes a new instance of the System.ServiceModel.MessageBodyMemberAttribute
             class.
             *
             * @instance
             * @public
             * @this System.ServiceModel.MessageBodyMemberAttribute
             * @memberof System.ServiceModel.MessageBodyMemberAttribute
             * @return  {void}
             */
            ctor: function () {
                this.$initialize();
                System.ServiceModel.MessageContractMemberAttribute.ctor.call(this);
            }
        }
    });

    /**
     * Defines a mechanism for retrieving a service object; that is, an object that
     provides custom support to other objects.
     *
     * @public
     * @class System.ServiceProvider
     * @implements  System.Windows.Markup.IProvideValueTarget
     */
    Bridge.define("System.ServiceProvider", {
        inherits: [System.Windows.Markup.IProvideValueTarget],
        fields: {
            _parents: null,
            _targetObject: null,
            _targetProperty: null
        },
        props: {
            /**
             * Gets or sets the List of UIElement that are parents of the element.
             *
             * @instance
             * @public
             * @memberof System.ServiceProvider
             * @function Parents
             * @type System.Collections.Generic.List$1
             */
            Parents: {
                get: function () {
                    return this._parents;
                },
                set: function (value) {
                    this._parents = value;
                }
            },
            TargetObject: {
                get: function () {
                    return this._targetObject;
                },
                set: function (value) {
                    this._targetObject = value;
                }
            },
            TargetProperty: {
                get: function () {
                    return this._targetProperty;
                },
                set: function (value) {
                    this._targetProperty = value;
                }
            }
        },
        alias: [
            "TargetObject", "System$Windows$Markup$IProvideValueTarget$TargetObject",
            "TargetProperty", "System$Windows$Markup$IProvideValueTarget$TargetProperty"
        ],
        ctors: {
            /**
             * Constructor for the  ServiceProvider class
             *
             * @instance
             * @public
             * @this System.ServiceProvider
             * @memberof System.ServiceProvider
             * @param   {System.Object}                         element     
             * @param   {Windows.UI.Xaml.DependencyProperty}    property    
             * @param   {System.Collections.Generic.List$1}     parents
             * @return  {void}
             */
            $ctor1: function (element, property, parents) {
                this.$initialize();
                this.TargetObject = element;
                this.TargetProperty = property;
                this.Parents = parents;
            },
            ctor: function (element, property) {
                System.ServiceProvider.$ctor1.call(this, element, property, new (System.Collections.Generic.List$1(System.Object)).ctor());

            }
        },
        methods: {
            /**
             * Gets the service object of the specified type.
             *
             * @instance
             * @public
             * @this System.ServiceProvider
             * @memberof System.ServiceProvider
             * @param   {System.Type}      serviceType    An object that specifies the type of service object to get.
             * @return  {System.Object}                   A service object of type serviceType.-or- null if there is no service object
             of type serviceType.
             */
            GetService: function (serviceType) {
                return this;
            }
        }
    });

    /**
     * Provides a basic implementation of the IDataObject interface,
     which defines a format-independent mechanism for transferring data.
     *
     * @public
     * @class System.Windows.DataObject
     * @implements  System.Windows.IDataObject
     */
    Bridge.define("System.Windows.DataObject", {
        inherits: [System.Windows.IDataObject],
        fields: {
            _data: null
        },
        alias: [
            "GetData", "System$Windows$IDataObject$GetData",
            "GetDataPresent", "System$Windows$IDataObject$GetDataPresent",
            "GetFormats", "System$Windows$IDataObject$GetFormats",
            "SetData", "System$Windows$IDataObject$SetData",
            "SetData$1", "System$Windows$IDataObject$SetData$1"
        ],
        ctors: {
            init: function () {
                this._data = new (System.Collections.Generic.Dictionary$2(System.String,System.Object)).ctor();
            },
            ctor: function () {
                this.$initialize();
            }
        },
        methods: {
            /**
             * Retrieves a data object in a specified format; the data format is specified 
             by a string.
             *
             * @instance
             * @public
             * @this System.Windows.DataObject
             * @memberof System.Windows.DataObject
             * @param   {string}           format    A string that specifies what format to retrieve the data as.
             * @return  {System.Object}              A data object that has the data in the specified format, or null if the data
             is not available in the specified format.
             */
            GetData: function (format) {
                if (this._data.containsKey(format)) {
                    return this._data.getItem(format);
                } else {
                    return null;
                }
            },
            /**
             * Checks to see whether the data is available in, or can be converted to, a
             specified format; the data format is specified by a string.
             *
             * @instance
             * @public
             * @this System.Windows.DataObject
             * @memberof System.Windows.DataObject
             * @param   {string}     format    A string that specifies the format to check for.
             * @return  {boolean}              True if the data is in, or can be converted to, the specified format; otherwise,
             false.
             */
            GetDataPresent: function (format) {
                return this._data.containsKey(format);
            },
            /**
             * Returns a list of all formats that the data in this data object is stored
             in, or can be converted to.
             *
             * @instance
             * @public
             * @this System.Windows.DataObject
             * @memberof System.Windows.DataObject
             * @return  {Array.<string>}        An array of strings, with each string specifying the name of a format supported
             by this data object.
             */
            GetFormats: function () {
                return System.Linq.Enumerable.from(this._data.Keys).ToArray();
            },
            /**
             * Stores the specified data in this data object, automatically determining
             the data format from the source object type.
             *
             * @instance
             * @public
             * @this System.Windows.DataObject
             * @memberof System.Windows.DataObject
             * @param   {System.Object}    data    The data to store in this data object.
             * @return  {void}
             */
            SetData: function (data) {
                this._data.setItem(Bridge.getTypeName(Bridge.getType(data)), data);
            },
            /**
             * Stores the specified data in this data object. The data format is specified by a string.
             *
             * @instance
             * @public
             * @this System.Windows.DataObject
             * @memberof System.Windows.DataObject
             * @param   {string}           format    A string that specifies what format to store the data in.
             * @param   {System.Object}    data      The data to store in this data object.
             * @return  {void}
             */
            SetData$1: function (format, data) {
                this._data.setItem(format, data);
            }
        }
    });

    /**
     * Provides application part and localization information in the application
     manifest when deploying a Silverlight-based application.
     *
     * @public
     * @class System.Windows.Deployment
     * @augments Windows.UI.Xaml.DependencyObject
     */
    Bridge.define("System.Windows.Deployment", {
        inherits: [Windows.UI.Xaml.DependencyObject],
        statics: {
            fields: {
                _deployment: null
            },
            props: {
                /**
                 * Gets the current System.Windows.Deployment object.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof System.Windows.Deployment
                 * @function Current
                 * @type System.Windows.Deployment
                 */
                Current: {
                    get: function () {
                        if (System.Windows.Deployment._deployment == null) {
                            System.Windows.Deployment._deployment = new System.Windows.Deployment();
                        }
                        return System.Windows.Deployment._deployment;
                    }
                }
            }
        },
        ctors: {
            ctor: function () {
                this.$initialize();
                Windows.UI.Xaml.DependencyObject.ctor.call(this);
            }
        }
    });

    /**
     * Provides data for drag-and-drop events in Silverlight.
     *
     * @public
     * @class System.Windows.DragEventArgs
     * @augments System.Windows.ExtendedRoutedEventArgs
     */
    Bridge.define("System.Windows.DragEventArgs", {
        inherits: [System.Windows.ExtendedRoutedEventArgs],
        fields: {
            PointerRoutedEventArgs: null,
            /**
             * Gets or sets the target drop-and-drop operation.
             *
             * @instance
             * @public
             * @memberof System.Windows.DragEventArgs
             * @function Effects
             * @type System.Windows.DragDropEffects
             */
            Effects: 0,
            /**
             * Gets a data object (implements IDataObject) that contains
             the data associated with the corresponding drag event. This value is not
             useful in all event cases; see Remarks.
             *
             * @instance
             * @public
             * @memberof System.Windows.DragEventArgs
             * @function Data
             * @type System.Windows.IDataObject
             */
            Data: null,
            /**
             * Gets or sets a value that indicates the present state of the event handling
             for a routed event as it travels the route.
             *
             * @instance
             * @public
             * @memberof System.Windows.DragEventArgs
             * @function Handled$1
             * @type boolean
             */
            Handled$1: false
        },
        ctors: {
            /**
             * Initializes a new instance of the DragEventArgs class.
             *
             * @instance
             * @this System.Windows.DragEventArgs
             * @memberof System.Windows.DragEventArgs
             * @param   {System.Windows.IDataObject}    data
             * @return  {void}
             */
            ctor: function (data) {
                this.$initialize();
                System.Windows.ExtendedRoutedEventArgs.ctor.call(this);
                this.Data = data;
            },
            /**
             * Initializes a new instance of the DragEventArgs class.
             *
             * @instance
             * @this System.Windows.DragEventArgs
             * @memberof System.Windows.DragEventArgs
             * @param   {System.Windows.IDataObject}                      data    
             * @param   {Windows.UI.Xaml.Input.PointerRoutedEventArgs}    e
             * @return  {void}
             */
            $ctor1: function (data, e) {
                this.$initialize();
                System.Windows.ExtendedRoutedEventArgs.ctor.call(this);
                this.Data = data;
                this.PointerRoutedEventArgs = e;
            }
        },
        methods: {
            /**
             * Returns a drop point that is relative to a specified UIElement
             *
             * @instance
             * @public
             * @this System.Windows.DragEventArgs
             * @memberof System.Windows.DragEventArgs
             * @param   {Windows.UI.Xaml.UIElement}        relativeTo    The UIElement for which to get a relative drop point.
             * @return  {Windows.UI.Input.PointerPoint}                  A drop point that is relative to the element specified in relativeTo.
             */
            GetPosition: function (relativeTo) {
                return this.PointerRoutedEventArgs.GetCurrentPoint(relativeTo);
            }
        }
    });

    /**
     * Represents a collection of System.Windows.DependencyObject instances of a specified
     type.
     *
     * @public
     * @class Windows.UI.Xaml.DependencyObjectCollection$1
     * @augments Windows.UI.Xaml.DependencyObject
     * @implements  System.Collections.Generic.IList$1
     * @implements  System.Collections.Generic.ICollection$1
     * @implements  System.Collections.Generic.IEnumerable$1
     * @implements  System.Collections.IList
     * @implements  System.Collections.ICollection
     * @implements  System.Collections.IEnumerable
     * @implements  System.Collections.Specialized.INotifyCollectionChanged
     * @param   {Function}    [name]    The type of items in the collection.
     */
    Bridge.define("Windows.UI.Xaml.DependencyObjectCollection$1", function (T) { return {
        inherits: [Windows.UI.Xaml.DependencyObject,System.Collections.Generic.IList$1(T),System.Collections.Generic.ICollection$1(T),System.Collections.Generic.IEnumerable$1(T),System.Collections.IList,System.Collections.ICollection,System.Collections.IEnumerable,System.Collections.Specialized.INotifyCollectionChanged],
        fields: {
            _monitor: null,
            _collection: null
        },
        events: {
            /**
             * Occurs when items in the collection are added, removed, or replaced.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.DependencyObjectCollection$1
             * @memberof Windows.UI.Xaml.DependencyObjectCollection$1
             * @function addCollectionChanged
             * @param   {System.Collections.Specialized.NotifyCollectionChangedEventHandler}    value
             * @return  {void}
             */
            /**
             * Occurs when items in the collection are added, removed, or replaced.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.DependencyObjectCollection$1
             * @memberof Windows.UI.Xaml.DependencyObjectCollection$1
             * @function removeCollectionChanged
             * @param   {System.Collections.Specialized.NotifyCollectionChangedEventHandler}    value
             * @return  {void}
             */
            CollectionChanged: null
        },
        props: {
            /**
             * Gets the number of objects in the collection.
             *
             * @instance
             * @public
             * @readonly
             * @memberof Windows.UI.Xaml.DependencyObjectCollection$1
             * @function Count
             * @type number
             */
            Count: {
                get: function () {
                    return this._collection.CountInternal;
                }
            },
            /**
             * Gets a value that indicates whether the collection can be modified.
             *
             * @instance
             * @public
             * @readonly
             * @memberof Windows.UI.Xaml.DependencyObjectCollection$1
             * @function IsReadOnly
             * @type boolean
             */
            IsReadOnly: {
                get: function () {
                    return false;
                }
            },
            System$Collections$IList$IsFixedSize: {
                get: function () {
                    return this.IsReadOnly;
                }
            },
            System$Collections$ICollection$SyncRoot: {
                get: function () {
                    return this._collection;
                }
            },
            System$Collections$ICollection$IsSynchronized: {
                get: function () {
                    return false;
                }
            }
        },
        alias: [
            "addCollectionChanged", "System$Collections$Specialized$INotifyCollectionChanged$addCollectionChanged",
            "removeCollectionChanged", "System$Collections$Specialized$INotifyCollectionChanged$removeCollectionChanged",
            "getItem", "System$Collections$Generic$IList$1$" + Bridge.getTypeAlias(T) + "$getItem",
            "setItem", "System$Collections$Generic$IList$1$" + Bridge.getTypeAlias(T) + "$setItem",
            "Count", "System$Collections$ICollection$Count",
            "Count", "System$Collections$Generic$ICollection$1$" + Bridge.getTypeAlias(T) + "$Count",
            "IsReadOnly", "System$Collections$IList$IsReadOnly",
            "IsReadOnly", "System$Collections$Generic$ICollection$1$" + Bridge.getTypeAlias(T) + "$IsReadOnly",
            "add", "System$Collections$Generic$ICollection$1$" + Bridge.getTypeAlias(T) + "$add",
            "clear", "System$Collections$IList$clear",
            "clear", "System$Collections$Generic$ICollection$1$" + Bridge.getTypeAlias(T) + "$clear",
            "contains", "System$Collections$Generic$ICollection$1$" + Bridge.getTypeAlias(T) + "$contains",
            "GetEnumerator", ["System$Collections$Generic$IEnumerable$1$" + Bridge.getTypeAlias(T) + "$GetEnumerator", "System$Collections$Generic$IEnumerable$1$GetEnumerator"],
            "indexOf", "System$Collections$Generic$IList$1$" + Bridge.getTypeAlias(T) + "$indexOf",
            "insert", "System$Collections$Generic$IList$1$" + Bridge.getTypeAlias(T) + "$insert",
            "remove", "System$Collections$Generic$ICollection$1$" + Bridge.getTypeAlias(T) + "$remove",
            "removeAt", "System$Collections$IList$removeAt",
            "removeAt", "System$Collections$Generic$IList$1$" + Bridge.getTypeAlias(T) + "$removeAt",
            "System$Collections$Generic$ICollection$1$copyTo", "System$Collections$Generic$ICollection$1$" + Bridge.getTypeAlias(T) + "$copyTo"
        ],
        ctors: {
            init: function () {
                this._monitor = new (Windows.UI.Xaml.DependencyObjectCollection$1.SimpleMonitor(T))();
            },
            /**
             * Initializes a new instance of the {@link }
             class.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.DependencyObjectCollection$1
             * @memberof Windows.UI.Xaml.DependencyObjectCollection$1
             * @return  {void}
             */
            ctor: function () {
                this.$initialize();
                Windows.UI.Xaml.DependencyObject.ctor.call(this);
                this._collection = new Windows.UI.Xaml.DependencyObjectCollectionInternal(this);
            }
        },
        methods: {
            getItem: function (index) {
                return Bridge.cast(this._collection.getItem(index), T);
            },
            setItem: function (index, value) {
                this.CheckReentrancy();
                var originalItem = this._collection.getItem(index);
                this._collection.setItem(index, value);
                this.OnCollectionChanged(new System.Collections.Specialized.NotifyCollectionChangedEventArgs.$ctor10(System.Collections.Specialized.NotifyCollectionChangedAction.Replace, value, originalItem, index));
            },
            System$Collections$IList$getItem: function (index) {
                return this.getItem(index);
            },
            System$Collections$IList$setItem: function (index, value) {
                this.setItem(index, this.Cast(value));
            },
            /**
             * Adds the specified object to the end of the collection.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.DependencyObjectCollection$1
             * @memberof Windows.UI.Xaml.DependencyObjectCollection$1
             * @throws item is not a {@link ArgumentNullException}.
             * @throws The method is being called in a {@link InvalidOperationException} 
             event handler.
             * @param   {T}       item    The object to add to the collection.
             * @return  {void}
             */
            add: function (item) {
                this.CheckReentrancy();
                this._collection.add(item);
                this.OnCollectionChanged(new System.Collections.Specialized.NotifyCollectionChangedEventArgs.$ctor7(System.Collections.Specialized.NotifyCollectionChangedAction.Add, item, this.Count));
            },
            System$Collections$IList$add: function (item) {
                this.add(this.Cast(item));
                return this.Count;
            },
            /**
             * Removes all objects from the collection.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.DependencyObjectCollection$1
             * @memberof Windows.UI.Xaml.DependencyObjectCollection$1
             * @throws The method is being called in a {@link InvalidOperationException} 
             event handler.
             * @return  {void}
             */
            clear: function () {
                this.CheckReentrancy();
                this._collection.clear();
                this.OnCollectionChanged(new System.Collections.Specialized.NotifyCollectionChangedEventArgs.ctor(System.Collections.Specialized.NotifyCollectionChangedAction.Reset));
            },
            /**
             * Gets a value that indicates whether the specified object is in the collection.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.DependencyObjectCollection$1
             * @memberof Windows.UI.Xaml.DependencyObjectCollection$1
             * @throws item is not a {@link ArgumentException}.
             * @param   {T}          item    The object to check for in the collection.
             * @return  {boolean}            true if the object is in the collection; otherwise, false.
             */
            contains: function (item) {
                if (!(Bridge.is(item, Windows.UI.Xaml.DependencyObject))) {
                    throw new System.ArgumentException.$ctor1("item is not a DependencyObject !");
                }
                return this._collection.contains(item);
            },
            System$Collections$IList$contains: function (item) {
                return this.contains(this.Cast(item));
            },
            /**
             * Gets an enumerator that iterates through the collection.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.DependencyObjectCollection$1
             * @memberof Windows.UI.Xaml.DependencyObjectCollection$1
             * @return  {System.Collections.Generic.IEnumerator$1}        An enumerator for the collection.
             */
            GetEnumerator: function () {
                return Bridge.getEnumerator(System.Linq.Enumerable.from(this._collection).select(function (d) {
                        return Bridge.cast(d, T);
                    }), T);
            },
            System$Collections$IEnumerable$GetEnumerator: function () {
                return this.GetEnumerator();
            },
            /**
             * Gets the index of the specified object within the collection, or -1 if the object
             is not in the collection.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.DependencyObjectCollection$1
             * @memberof Windows.UI.Xaml.DependencyObjectCollection$1
             * @throws item is not a {@link ArgumentException}.
             * @param   {T}         item    The object to locate in the collection.
             * @return  {number}            The index of the first occurrence of item within the collection, or -1 if item
             is not in the collection.
             */
            indexOf: function (item) {
                if (!(Bridge.is(item, Windows.UI.Xaml.DependencyObject))) {
                    throw new System.ArgumentException.$ctor1("item is not a DependencyObject !");
                }
                return this._collection.indexOf(item);
            },
            System$Collections$IList$indexOf: function (value) {
                return this.indexOf(this.Cast(value));
            },
            /**
             * Adds the specified object to the collection at the specified index.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.DependencyObjectCollection$1
             * @memberof Windows.UI.Xaml.DependencyObjectCollection$1
             * @throws index is less than zero or greater than the number of items in the collection.
             * @throws item is not a {@link ArgumentException}.
             * @throws The method is being called in a {@link InvalidOperationException} event handler.
             * @param   {number}    index    The index at which to add the object.
             * @param   {T}         item     The object to add.
             * @return  {void}
             */
            insert: function (index, item) {
                this.CheckReentrancy();
                this._collection.insert(index, item);
                this.OnCollectionChanged(new System.Collections.Specialized.NotifyCollectionChangedEventArgs.$ctor7(System.Collections.Specialized.NotifyCollectionChangedAction.Add, item, index));
            },
            System$Collections$IList$insert: function (index, value) {
                this.insert(index, this.Cast(value));
            },
            /**
             * Removes the specified object from the collection.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.DependencyObjectCollection$1
             * @memberof Windows.UI.Xaml.DependencyObjectCollection$1
             * @throws item is not a {@link ArgumentException}.
             * @throws The method is being called in a {@link InvalidOperationException} event handler.
             * @param   {T}          item    The object to remove.
             * @return  {boolean}            true if the object was removed; false if the object was not found in the collection.
             */
            remove: function (item) {
                this.CheckReentrancy();
                var index = this.indexOf(item);
                if (index > -1) {
                    var removedItem = this._collection.getItem(index);
                    this._collection.removeAt(index);
                    this.OnCollectionChanged(new System.Collections.Specialized.NotifyCollectionChangedEventArgs.$ctor7(System.Collections.Specialized.NotifyCollectionChangedAction.Remove, removedItem, index));
                    return true;
                }
                return false;
            },
            System$Collections$IList$remove: function (value) {
                this.remove(this.Cast(value));
            },
            /**
             * Removes the object at the specified index from the collection.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.DependencyObjectCollection$1
             * @memberof Windows.UI.Xaml.DependencyObjectCollection$1
             * @throws index is less than zero or greater than the highest index in the collection.
             * @throws The method is being called in a {@link InvalidOperationException} event handler.
             * @param   {number}    index    The index of the object to remove.
             * @return  {void}
             */
            removeAt: function (index) {
                this.CheckReentrancy();
                var removedItem = this.getItem(index);
                this._collection.removeAt(index);
                this.OnCollectionChanged(new System.Collections.Specialized.NotifyCollectionChangedEventArgs.$ctor7(System.Collections.Specialized.NotifyCollectionChangedAction.Remove, removedItem, index));
            },
            System$Collections$Generic$ICollection$1$copyTo: function (array, arrayIndex) {
                this._collection.copyTo(array, arrayIndex);
            },
            System$Collections$ICollection$copyTo: function (array, arrayIndex) {
                this._collection.copyTo$1(array, arrayIndex);
            },
            Cast: function (item) {
                var castedItem;
                if (!(((castedItem = Bridge.as(item, T))) != null)) {
                    throw new System.ArgumentException.$ctor1(System.String.format("item is not a {0}.", [Windows.UI.Xaml.DependencyObject]));
                }
                return castedItem;
            },
            OnCollectionChanged: function (e) {
                var $t;
                if (!Bridge.staticEquals(this.CollectionChanged, null)) {
                    $t = this.BlockReentrancy();
                    try {
                        this.CollectionChanged(this, e);
                    }
                    finally {
                        if (Bridge.hasValue($t)) {
                            $t.System$IDisposable$Dispose();
                        }
                    }
                }
            },
            /**
             * Disallow reentrant attempts to change this collection. E.g. a event handler
             of the CollectionChanged event is not allowed to make changes to this collection.
             *
             * @instance
             * @private
             * @this Windows.UI.Xaml.DependencyObjectCollection$1
             * @memberof Windows.UI.Xaml.DependencyObjectCollection$1
             * @return  {System.IDisposable}
             */
            BlockReentrancy: function () {
                this._monitor.Enter();
                return this._monitor;
            },
            /**
             * Check and assert for reentrant attempts to change this collection.
             *
             * @instance
             * @private
             * @this Windows.UI.Xaml.DependencyObjectCollection$1
             * @memberof Windows.UI.Xaml.DependencyObjectCollection$1
             * @throws raised when changing the collection
             while another collection change is still being notified to other listeners
             * @return  {void}
             */
            CheckReentrancy: function () {
                if (this._monitor.Busy) {
                    if ((!Bridge.staticEquals(this.CollectionChanged, null)) && (Bridge.fn.getInvocationList(this.CollectionChanged).length > 1)) {
                        throw new System.InvalidOperationException.$ctor1("DependencyObjectCollection Reentrancy not allowed");
                    }
                }
            }
        }
    }; });

    /**
     * Encapsulates state information and zero or more ICommands into an attachable
     object.
     *
     * @abstract
     * @public
     * @class System.Windows.Interactivity.Behavior
     * @augments Windows.UI.Xaml.DependencyObject
     * @implements  System.Windows.Interactivity.IAttachedObject
     */
    Bridge.define("System.Windows.Interactivity.Behavior", {
        inherits: [Windows.UI.Xaml.DependencyObject,System.Windows.Interactivity.IAttachedObject],
        fields: {
            _associatedObject: null,
            _associatedType: null
        },
        props: {
            /**
             * Gets the object to which this behavior is attached.
             *
             * @instance
             * @public
             * @readonly
             * @memberof System.Windows.Interactivity.Behavior
             * @function AssociatedObject
             * @type Windows.UI.Xaml.DependencyObject
             */
            AssociatedObject: {
                get: function () {
                    return this._associatedObject;
                }
            },
            /**
             * The type to which this behavior can be attached.
             *
             * @instance
             * @protected
             * @readonly
             * @memberof System.Windows.Interactivity.Behavior
             * @function AssociatedType
             * @type System.Type
             */
            AssociatedType: {
                get: function () {
                    return this._associatedType;
                }
            }
        },
        alias: [
            "AssociatedObject", "System$Windows$Interactivity$IAttachedObject$AssociatedObject",
            "Attach", "System$Windows$Interactivity$IAttachedObject$Attach",
            "Detach", "System$Windows$Interactivity$IAttachedObject$Detach"
        ],
        methods: {
            /**
             * Attaches to the specified object.
             *
             * @instance
             * @public
             * @this System.Windows.Interactivity.Behavior
             * @memberof System.Windows.Interactivity.Behavior
             * @param   {Windows.UI.Xaml.DependencyObject}    dependencyObject    The object to attach to.
             * @return  {void}
             */
            Attach: function (dependencyObject) {
                if (this._associatedObject != null) {
                    throw new System.InvalidOperationException.$ctor1("The Behavior is already hosted on a different element.");
                }
                if (this.AssociatedType != null && !Bridge.Reflection.isAssignableFrom(this.AssociatedType, Bridge.getType(dependencyObject))) {
                    throw new System.InvalidOperationException.$ctor1("dependencyObject does not satisfy the Behavior type constraint.");
                } else {
                    this._associatedObject = dependencyObject;
                    this.OnAttached();
                }
            },
            /**
             * Detaches this instance from its associated object.
             *
             * @instance
             * @public
             * @this System.Windows.Interactivity.Behavior
             * @memberof System.Windows.Interactivity.Behavior
             * @return  {void}
             */
            Detach: function () {
                this.OnDetaching();
                this._associatedObject = null;
            },
            /**
             * Called after the behavior is attached to an AssociatedObject.
             *
             * @instance
             * @protected
             * @this System.Windows.Interactivity.Behavior
             * @memberof System.Windows.Interactivity.Behavior
             * @return  {void}
             */
            OnAttached: function () { },
            /**
             * Called when the behavior is being detached from its AssociatedObject, but
             before it has actually occurred.
             *
             * @instance
             * @protected
             * @this System.Windows.Interactivity.Behavior
             * @memberof System.Windows.Interactivity.Behavior
             * @return  {void}
             */
            OnDetaching: function () { }
        }
    });

    /**
     * Represents an object that can invoke Actions conditionally.
     *
     * @abstract
     * @public
     * @class System.Windows.Interactivity.TriggerBase
     * @augments Windows.UI.Xaml.DependencyObject
     * @implements  System.Windows.Interactivity.IAttachedObject
     */
    Bridge.define("System.Windows.Interactivity.TriggerBase", {
        inherits: [Windows.UI.Xaml.DependencyObject,System.Windows.Interactivity.IAttachedObject],
        statics: {
            fields: {
                ActionsProperty: null
            },
            ctors: {
                init: function () {
                    this.ActionsProperty = Windows.UI.Xaml.DependencyProperty.Register("Actions", System.Windows.Interactivity.TriggerActionCollection, System.Windows.Interactivity.TriggerBase, null);
                }
            }
        },
        fields: {
            _associatedObject: null
        },
        props: {
            /**
             * Gets the object to which this behavior is attached.
             *
             * @instance
             * @public
             * @readonly
             * @memberof System.Windows.Interactivity.TriggerBase
             * @function AssociatedObject
             * @type Windows.UI.Xaml.DependencyObject
             */
            AssociatedObject: {
                get: function () {
                    return this._associatedObject;
                }
            },
            /**
             * Gets the actions associated with this trigger.
             *
             * @instance
             * @public
             * @readonly
             * @memberof System.Windows.Interactivity.TriggerBase
             * @function Actions
             * @type System.Windows.Interactivity.TriggerActionCollection
             */
            Actions: {
                get: function () {
                    var actions = Bridge.cast(this.GetValue(System.Windows.Interactivity.TriggerBase.ActionsProperty), System.Windows.Interactivity.TriggerActionCollection);
                    if (actions == null) {
                        actions = new System.Windows.Interactivity.TriggerActionCollection();
                        this.SetValue(System.Windows.Interactivity.TriggerBase.ActionsProperty, actions);
                    }
                    return actions;
                }
            }
        },
        alias: [
            "AssociatedObject", "System$Windows$Interactivity$IAttachedObject$AssociatedObject",
            "Attach", "System$Windows$Interactivity$IAttachedObject$Attach",
            "Detach", "System$Windows$Interactivity$IAttachedObject$Detach"
        ],
        methods: {
            /**
             * Attaches to the specified object.
             *
             * @instance
             * @public
             * @this System.Windows.Interactivity.TriggerBase
             * @memberof System.Windows.Interactivity.TriggerBase
             * @param   {Windows.UI.Xaml.DependencyObject}    dependencyObject    The object to attach to.
             * @return  {void}
             */
            Attach: function (dependencyObject) {
                if (Bridge.referenceEquals(dependencyObject, this.AssociatedObject)) {
                    return;
                }
                if (this._associatedObject != null) {
                    throw new System.InvalidOperationException.$ctor1("The Behavior is already hosted on a different element.");
                }
                this._associatedObject = dependencyObject;
                this.OnAttached();
            },
            /**
             * Attaches to the specified object.
             *
             * @instance
             * @public
             * @this System.Windows.Interactivity.TriggerBase
             * @memberof System.Windows.Interactivity.TriggerBase
             * @throws Cannot host the same trigger on more than one object at a time.
             * @throws dependencyObject does not satisfy the trigger type constraint.
             * @param   {Windows.UI.Xaml.FrameworkElement}    frameworkElement    The object to attach to.
             * @return  {void}
             */
            Attach$1: function (frameworkElement) {

                if (Bridge.referenceEquals(frameworkElement, this.AssociatedObject)) {
                    return;
                }
                if (this.AssociatedObject != null) {
                    throw new System.InvalidOperationException.$ctor1("Cannot Host Trigger Multiple Times");
                }

                this._associatedObject = frameworkElement;
                this.Attach(Bridge.cast(frameworkElement, Windows.UI.Xaml.DependencyObject));
                this.Actions.Attach(frameworkElement);
                this.OnAttached();
            },
            OnAttached: function () {
                this.Actions.Attach(this._associatedObject);

            },
            /**
             * Invoke all actions associated with this trigger.
             *
             * @instance
             * @protected
             * @this System.Windows.Interactivity.TriggerBase
             * @memberof System.Windows.Interactivity.TriggerBase
             * @param   {System.Object}    parameter
             * @return  {void}
             */
            InvokeActions: function (parameter) {
                var $t;
                $t = Bridge.getEnumerator(this.Actions);
                try {
                    while ($t.moveNext()) {
                        var triggerAction = $t.Current;
                        triggerAction.CallInvoke(parameter);
                    }
                } finally {
                    if (Bridge.is($t, System.IDisposable)) {
                        $t.System$IDisposable$Dispose();
                    }
                }
            },
            /**
             * Detaches this instance from its associated object.
             *
             * @instance
             * @public
             * @this System.Windows.Interactivity.TriggerBase
             * @memberof System.Windows.Interactivity.TriggerBase
             * @return  {void}
             */
            Detach: function () {
                this._associatedObject = null;
                this.Actions.Detach();
            }
        }
    });

    /**
     * Represents an attachable object that encapsulates a unit of functionality.
     *
     * @abstract
     * @public
     * @class System.Windows.Interactivity.TriggerAction
     * @augments Windows.UI.Xaml.DependencyObject
     * @implements  System.Windows.Interactivity.IAttachedObject
     */
    Bridge.define("System.Windows.Interactivity.TriggerAction", {
        inherits: [Windows.UI.Xaml.DependencyObject,System.Windows.Interactivity.IAttachedObject],
        statics: {
            fields: {
                IsEnabledProperty: null
            },
            ctors: {
                init: function () {
                    this.IsEnabledProperty = Windows.UI.Xaml.DependencyProperty.Register("IsEnabled", System.Boolean, System.Windows.Interactivity.TriggerAction, new Windows.UI.Xaml.PropertyMetadata.$ctor1(Bridge.box(true, System.Boolean, System.Boolean.toString)));
                }
            }
        },
        fields: {
            _associatedObject: null,
            isHosted: false
        },
        props: {
            /**
             * Gets the object to which this behavior is attached.
             *
             * @instance
             * @public
             * @readonly
             * @memberof System.Windows.Interactivity.TriggerAction
             * @function AssociatedObject
             * @type Windows.UI.Xaml.DependencyObject
             */
            AssociatedObject: {
                get: function () {
                    return this._associatedObject;
                }
            },
            /**
             * Gets or sets a value indicating whether this action will run when invoked. This is a dependency property.
             *
             * @instance
             * @public
             * @memberof System.Windows.Interactivity.TriggerAction
             * @function IsEnabled
             * @type boolean
             */
            IsEnabled: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(System.Windows.Interactivity.TriggerAction.IsEnabledProperty), System.Boolean), System.Boolean));
                },
                set: function (value) {
                    this.SetValue(System.Windows.Interactivity.TriggerAction.IsEnabledProperty, Bridge.box(value, System.Boolean, System.Boolean.toString));
                }
            },
            /**
             * Gets or sets a value indicating whether this instance is attached.
             *
             * @instance
             * @memberof System.Windows.Interactivity.TriggerAction
             * @function IsHosted
             * @type boolean
             */
            IsHosted: {
                get: function () {
                    return this.isHosted;
                },
                set: function (value) {
                    this.isHosted = value;
                }
            }
        },
        alias: [
            "AssociatedObject", "System$Windows$Interactivity$IAttachedObject$AssociatedObject",
            "Attach", "System$Windows$Interactivity$IAttachedObject$Attach",
            "Detach", "System$Windows$Interactivity$IAttachedObject$Detach"
        ],
        ctors: {
            ctor: function () {
                this.$initialize();
                Windows.UI.Xaml.DependencyObject.ctor.call(this);
            }
        },
        methods: {
            /**
             * Attaches to the specified object.
             *
             * @instance
             * @public
             * @this System.Windows.Interactivity.TriggerAction
             * @memberof System.Windows.Interactivity.TriggerAction
             * @param   {Windows.UI.Xaml.DependencyObject}    dependencyObject    The object to attach to.
             * @return  {void}
             */
            Attach: function (dependencyObject) {
                if (this._associatedObject != null) {
                    throw new System.InvalidOperationException.$ctor1("The Behavior is already hosted on a different element.");
                }
                this._associatedObject = dependencyObject;
            },
            /**
             * Attaches to the specified object.
             *
             * @instance
             * @public
             * @this System.Windows.Interactivity.TriggerAction
             * @memberof System.Windows.Interactivity.TriggerAction
             * @throws Cannot host the same TriggerAction on more than one object at a time.
             * @throws dependencyObject does not satisfy the TriggerAction type constraint.
             * @param   {Windows.UI.Xaml.FrameworkElement}    frameworkElement    The object to attach to.
             * @return  {void}
             */
            Attach$1: function (frameworkElement) {
                if (Bridge.referenceEquals(frameworkElement, this.AssociatedObject)) {
                    return;
                }
                if (this.AssociatedObject != null) {
                    throw new System.InvalidOperationException.$ctor1("Cannot Host TriggerAction Multiple Times");
                }
                this._associatedObject = frameworkElement;
                this.Attach(Bridge.cast(frameworkElement, Windows.UI.Xaml.DependencyObject));
            },
            /**
             * Attempts to invoke the action.
             *
             * @instance
             * @this System.Windows.Interactivity.TriggerAction
             * @memberof System.Windows.Interactivity.TriggerAction
             * @param   {System.Object}    parameter    The parameter to the action. If the action does not require a parameter, the parameter may be set to a null reference.
             * @return  {void}
             */
            CallInvoke: function (parameter) {
                if (!this.IsEnabled) {
                    return;
                }
                this.Invoke(parameter);
            },
            /**
             * Detaches this instance from its associated object.
             *
             * @instance
             * @public
             * @this System.Windows.Interactivity.TriggerAction
             * @memberof System.Windows.Interactivity.TriggerAction
             * @return  {void}
             */
            Detach: function () {
                this._associatedObject = null;
            }
        }
    });

    /**
     * In XAML markup, specifies a null value for a property.
     *
     * @public
     * @class System.Windows.Markup.NullExtension
     * @augments System.Windows.Markup.MarkupExtension
     */
    Bridge.define("System.Windows.Markup.NullExtension", {
        inherits: [System.Windows.Markup.MarkupExtension],
        ctors: {
            /**
             * Initializes a new instance of NullExtension.
             *
             * @instance
             * @public
             * @this System.Windows.Markup.NullExtension
             * @memberof System.Windows.Markup.NullExtension
             * @return  {void}
             */
            ctor: function () {
                this.$initialize();
                System.Windows.Markup.MarkupExtension.ctor.call(this);
            }
        },
        methods: {
            ProvideValue: function (serviceProvider) {
                return null;
            }
        }
    });

    /**
     * Class used to access elements inside the XAML code
     *
     * @public
     * @class System.Windows.Markup.StaticResourceExtension
     * @augments System.Windows.Markup.MarkupExtension
     */
    Bridge.define("System.Windows.Markup.StaticResourceExtension", {
        inherits: [System.Windows.Markup.MarkupExtension],
        fields: {
            /**
             * The key of the StaticResource.
             *
             * @instance
             * @public
             * @memberof System.Windows.Markup.StaticResourceExtension
             * @function ResourceKey
             * @type string
             */
            ResourceKey: null
        },
        ctors: {
            /**
             * Initializes a new instance of StaticResource.
             *
             * @instance
             * @public
             * @this System.Windows.Markup.StaticResourceExtension
             * @memberof System.Windows.Markup.StaticResourceExtension
             * @return  {void}
             */
            ctor: function () {
                this.$initialize();
                System.Windows.Markup.MarkupExtension.ctor.call(this);
            },
            /**
             * Initializes a new instance of StaticResource with the given ResourceKey.
             *
             * @instance
             * @public
             * @this System.Windows.Markup.StaticResourceExtension
             * @memberof System.Windows.Markup.StaticResourceExtension
             * @param   {string}    resourceKey    The ResourceKey for the StaticResource
             * @return  {void}
             */
            $ctor1: function (resourceKey) {
                this.$initialize();
                System.Windows.Markup.MarkupExtension.ctor.call(this);
                this.ResourceKey = resourceKey;
            }
        },
        methods: {
            /**
             * returns an object that is provided as the value of the target property for this StaticResource.
             *
             * @instance
             * @public
             * @override
             * @this System.Windows.Markup.StaticResourceExtension
             * @memberof System.Windows.Markup.StaticResourceExtension
             * @param   {System.ServiceProvider}    serviceProvider    A service provider helper that can provide services for the StaticResource.
             * @return  {System.Object}                                An object that is provided as the value of the target property for this StaticResource.
             */
            ProvideValue: function (serviceProvider) {
                var $t;
                var serviceProviderAsServiceProvider = serviceProvider;
                var targetType;
                if (Bridge.as(serviceProviderAsServiceProvider.TargetProperty, Windows.UI.Xaml.DependencyProperty) != null) {
                    targetType = Bridge.cast(serviceProviderAsServiceProvider.TargetProperty, Windows.UI.Xaml.DependencyProperty).PropertyType;
                } else {
                    targetType = null;
                }
                var isFirst = true;
                var elementItself = null;
                $t = Bridge.getEnumerator(serviceProviderAsServiceProvider.Parents);
                try {
                    while ($t.moveNext()) {
                        var parentElement = $t.Current;
                        if (isFirst) {
                            elementItself = parentElement;
                        } else {
                            var resourceDictionary = null;
                            if (((resourceDictionary = Bridge.as(parentElement, Windows.UI.Xaml.ResourceDictionary))) == null) {
                                var parentFE;
                                if (((parentFE = Bridge.as(parentElement, Windows.UI.Xaml.FrameworkElement))) != null) {
                                    resourceDictionary = parentFE.HasResources ? parentFE.Resources : null;
                                }
                            }
                            if (resourceDictionary != null && resourceDictionary.contains(this.ResourceKey)) {
                                var returnElement = resourceDictionary.getItem(this.ResourceKey);
                                if (!Bridge.equals(returnElement, elementItself)) {
                                    return this.EnsurePropertyType(returnElement, targetType);
                                }
                            }
                        }
                        isFirst = false;
                    }
                } finally {
                    if (Bridge.is($t, System.IDisposable)) {
                        $t.System$IDisposable$Dispose();
                    }
                }
                if (Windows.UI.Xaml.Application.Current.Resources.contains(this.ResourceKey)) {
                    return this.EnsurePropertyType(Windows.UI.Xaml.Application.Current.Resources.getItem(this.ResourceKey), targetType);
                } else {
                    var result = Windows.UI.Xaml.Application.Current.TryFindResource(this.ResourceKey);
                    if (result == null) {
                        throw new System.Windows.Markup.XamlParseException.$ctor1(System.String.format("StaticResource resolve failed: cannot find resource named '{0}' (Note: resource names are case sensitive)", [this.ResourceKey]));
                    }
                    return this.EnsurePropertyType(result, targetType);
                }
            },
            EnsurePropertyType: function (item, targetType) {
                if (targetType == null || item == null) {
                    return item;
                }

                var itemType = Bridge.getType(item);
                if (Bridge.Reflection.isAssignableFrom(targetType, itemType)) {
                    return item;
                } else if (Bridge.referenceEquals(itemType, System.String) && DotNetForHtml5.Core.TypeFromStringConverters.CanTypeBeConverted(targetType)) {
                    return DotNetForHtml5.Core.TypeFromStringConverters.ConvertFromInvariantString(targetType, Bridge.cast(item, System.String));
                } else {
                    return System.Convert.changeType(item, targetType);
                }
            }
        }
    });

    /**
     * Class used to access elements inside the XAML code
     *
     * @public
     * @class System.Windows.Markup.ThemeResourceExtension
     * @augments System.Windows.Markup.MarkupExtension
     */
    Bridge.define("System.Windows.Markup.ThemeResourceExtension", {
        inherits: [System.Windows.Markup.MarkupExtension],
        fields: {
            /**
             * The key of the StaticResource.
             *
             * @instance
             * @public
             * @memberof System.Windows.Markup.ThemeResourceExtension
             * @function ResourceKey
             * @type string
             */
            ResourceKey: null
        },
        ctors: {
            /**
             * Initializes a new instance of StaticResource.
             *
             * @instance
             * @public
             * @this System.Windows.Markup.ThemeResourceExtension
             * @memberof System.Windows.Markup.ThemeResourceExtension
             * @return  {void}
             */
            ctor: function () {
                this.$initialize();
                System.Windows.Markup.MarkupExtension.ctor.call(this);
            },
            /**
             * Initializes a new instance of StaticResource with the given ResourceKey.
             *
             * @instance
             * @public
             * @this System.Windows.Markup.ThemeResourceExtension
             * @memberof System.Windows.Markup.ThemeResourceExtension
             * @param   {string}    resourceKey    The ResourceKey for the StaticResource
             * @return  {void}
             */
            $ctor1: function (resourceKey) {
                this.$initialize();
                System.Windows.Markup.MarkupExtension.ctor.call(this);
                this.ResourceKey = resourceKey;
            }
        },
        methods: {
            /**
             * returns an object that is provided as the value of the target property for this StaticResource.
             *
             * @instance
             * @public
             * @override
             * @this System.Windows.Markup.ThemeResourceExtension
             * @memberof System.Windows.Markup.ThemeResourceExtension
             * @param   {System.ServiceProvider}    serviceProvider    A service provider helper that can provide services for the StaticResource.
             * @return  {System.Object}                                An object that is provided as the value of the target property for this StaticResource.
             */
            ProvideValue: function (serviceProvider) {
                var $t;

                if (Bridge.hasValue(serviceProvider)) {
                    var serviceProviderAsServiceProvider = serviceProvider;
                    var isFirst = true;
                    var elementItself = null;
                    $t = Bridge.getEnumerator(serviceProviderAsServiceProvider.Parents);
                    try {
                        while ($t.moveNext()) {
                            var parentElement = $t.Current;
                            if (isFirst) {
                                elementItself = parentElement;
                            } else {
                                var resourceDictionary = null;
                                if (Bridge.is(parentElement, Windows.UI.Xaml.ResourceDictionary)) {
                                    resourceDictionary = Bridge.cast(parentElement, Windows.UI.Xaml.ResourceDictionary);
                                } else {
                                    var parentFE;
                                    if (((parentFE = Bridge.as(parentElement, Windows.UI.Xaml.FrameworkElement))) != null) {
                                        resourceDictionary = parentFE.HasResources ? parentFE.Resources : null;
                                    }
                                }
                                if (resourceDictionary != null && resourceDictionary.contains(this.ResourceKey)) {
                                    var returnElement = resourceDictionary.getItem(this.ResourceKey);
                                    if (!Bridge.referenceEquals(returnElement, elementItself)) {
                                        return returnElement;
                                    }
                                }
                            }
                            isFirst = false;
                        }
                    } finally {
                        if (Bridge.is($t, System.IDisposable)) {
                            $t.System$IDisposable$Dispose();
                        }
                    }
                    if (Windows.UI.Xaml.Application.Current.Resources != null && Windows.UI.Xaml.Application.Current.Resources.contains(this.ResourceKey)) {
                        return Windows.UI.Xaml.Application.Current.Resources.getItem(this.ResourceKey);
                    }
                    throw new System.Windows.Markup.XamlParseException.$ctor1(System.String.format("StaticResource resolve failed: cannot find resource named '{0}' (Note: resource names are case sensitive)", [this.ResourceKey]));
                } else {
                    throw new System.SystemException.$ctor1("StaticResourceExtension.ProvideValue failed: the service provider is not of the expected type. Please contact support.");
                }
            }
        }
    });

    /** @namespace System.Windows.Media.Effects */

    /**
     * Provides a custom bitmap effect.
     * // When overridden in a derived class, transforms mouse input and coordinate
    // systems through the effect.
    //
     * // Gets a System.Windows.Media.Brush that, when it is used as an input for an
    // System.Windows.Media.Effects.Effect, causes the bitmap of the System.Windows.UIElement
    // that the System.Windows.Media.Effects.Effect is applied to be that input.
    //
     * // Creates a modifiable clone of this System.Windows.Media.Effects.Effect object,
    // making deep copies of this object's values. When copying this object's dependency
    // properties, this method copies resource references and data bindings (which
    // may no longer resolve), but not animations or their current values.
    //
     * // Creates a modifiable clone of this System.Windows.Media.Effects.Effect object,
    // making deep copies of this object's current values. Resource references,
    // data bindings, and animations are not copied, but their current values are
    // copied.
    //
     *
     * @abstract
     * @public
     * @class System.Windows.Media.Effects.Effect
     * @augments Windows.UI.Xaml.DependencyObject
     * @return  {[type]}
     */
    Bridge.define("System.Windows.Media.Effects.Effect", {
        inherits: [Windows.UI.Xaml.DependencyObject],
        fields: {
            _parentUIElement: null
        },
        ctors: {
            /**
             * Initializes a new instance of the System.Windows.Media.Effects.Effect class.
             *
             * @instance
             * @protected
             * @this System.Windows.Media.Effects.Effect
             * @memberof System.Windows.Media.Effects.Effect
             * @return  {void}
             */
            ctor: function () {
                this.$initialize();
                Windows.UI.Xaml.DependencyObject.ctor.call(this);
            }
        },
        methods: {
            SetParentUIElement: function (newParent) {
                this._parentUIElement = newParent;
            }
        }
    });

    Bridge.define("System.Xml.Cshtml5_XmlReader", {
        inherits: [System.Xml.XmlReader],
        statics: {
            methods: {
                Create: function (input) {
                    var streamReader = new System.IO.StreamReader.$ctor1(input);

                    var xmlString = streamReader.ReadToEnd();

                    var xDocument = System.Xml.Linq.XDocument.Parse(xmlString);

                    return xDocument.CreateReader();
                }
            }
        },
        fields: {
            _xmlDoc: null,
            _reader: null,
            _nodeStack: null,
            _text: null
        },
        props: {
            LastNode: {
                get: function () {
                    return this._nodeStack.Count > 0 ? this._nodeStack.getItem(((this._nodeStack.Count - 1) | 0)) : System.Xml.INTERNAL_NodeType.Default;
                }
            },
            LastChildList: {
                get: function () {
                    return this._nodeStack.Count > 0 ? this.LastNode.NodeEnumerator : null;
                }
            },
            NodeType: {
                get: function () {
                    return this.LastNode.Type;
                }
            },
            IsEmptyElement: {
                get: function () {
                    return this.LastNode.NodeEnumerator == null ? true : false;
                }
            },
            Name: {
                get: function () {
                    var node = this.LastNode.RawNode;

                    if (Bridge.is(node, System.Xml.Linq.XElement)) {
                        return this.BuildName(Bridge.cast(node, System.Xml.Linq.XElement).Name);
                    } else {
                        if (Bridge.is(node, System.Xml.Linq.XAttribute)) {
                            return this.BuildName(Bridge.cast(node, System.Xml.Linq.XAttribute).Name);
                        }
                    }

                    return "";
                }
            },
            LocalName: {
                get: function () {
                    var node = this.LastNode.RawNode;

                    if (Bridge.is(node, System.Xml.Linq.XElement)) {
                        return Bridge.cast(node, System.Xml.Linq.XElement).Name.LocalName;
                    } else {
                        if (Bridge.is(node, System.Xml.Linq.XAttribute)) {
                            return Bridge.cast(node, System.Xml.Linq.XAttribute).Name.LocalName;
                        }
                    }

                    return "";
                }
            },
            Value: {
                get: function () {
                    var node = this.LastNode.RawNode;

                    if (Bridge.is(node, System.Xml.Linq.XAttribute)) {
                        return Bridge.cast(node, System.Xml.Linq.XAttribute).Value;
                    } else {
                        if (Bridge.is(node, System.Xml.Linq.XText)) {
                            return Bridge.cast(node, System.Xml.Linq.XText).Value;
                        }
                    }

                    return "";
                }
            },
            Settings: {
                get: function () {
                    return this._reader.Settings;
                }
            },
            NamespaceURI: {
                get: function () {
                    return this._reader.NamespaceURI;
                }
            },
            Prefix: {
                get: function () {
                    return this._reader.Prefix;
                }
            },
            HasValue: {
                get: function () {
                    return this._reader.HasValue;
                }
            },
            Depth: {
                get: function () {
                    return this._reader.Depth;
                }
            },
            BaseURI: {
                get: function () {
                    return this._reader.BaseURI;
                }
            },
            IsDefault: {
                get: function () {
                    return this._reader.IsDefault;
                }
            },
            QuoteChar: {
                get: function () {
                    return this._reader.QuoteChar;
                }
            },
            XmlLang: {
                get: function () {
                    return this._reader.XmlLang;
                }
            },
            SchemaInfo: {
                get: function () {
                    return this._reader.SchemaInfo;
                }
            },
            ValueType: {
                get: function () {
                    return this._reader.ValueType;
                }
            },
            AttributeCount: {
                get: function () {
                    return this._reader.AttributeCount;
                }
            },
            CanResolveEntity: {
                get: function () {
                    return this._reader.CanResolveEntity;
                }
            },
            EOF: {
                get: function () {
                    return this._reader.EOF;
                }
            },
            ReadState: {
                get: function () {
                    return this._reader.ReadState;
                }
            },
            HasAttributes: {
                get: function () {
                    return this._reader.HasAttributes;
                }
            },
            NameTable: {
                get: function () {
                    return this._reader.NameTable;
                }
            }
        },
        alias: ["Dispose", "System$IDisposable$Dispose"],
        ctors: {
            ctor: function (xmlDoc, text) {
                this.$initialize();
                System.Xml.XmlReader.ctor.call(this);
                this._text = text;
                this._xmlDoc = xmlDoc;
                this._reader = this;

                this._nodeStack = new (System.Collections.Generic.List$1(System.Xml.INTERNAL_NodeType)).ctor();

                this.AddNewNode(new System.Xml.INTERNAL_NodeType(xmlDoc, System.Xml.XmlNodeType.Document));
            }
        },
        methods: {
            getItem: function (i) {
                return this._reader.getItem(i);
            },
            getItem$1: function (name) {
                return this._reader.getItem$1(name);
            },
            getItem$2: function (name, namespaceURI) {
                return this._reader.getItem$2(name, namespaceURI);
            },
            toString: function () {
                return this._xmlDoc.toString();
            },
            RemoveLastNode: function () {
                this._nodeStack.removeAt(((this._nodeStack.Count - 1) | 0));
            },
            AddNewNode: function (node) {
                this._nodeStack.add(node);
            },
            GetAttribute$1: function (name) {
                var element = this.LastNode.ElementOrNull;

                if (element != null) {
                    if (System.Linq.Enumerable.from(element.Attributes$1(System.Xml.Linq.XName.op_Implicit(name))).count() > 0) {
                        var attribute = System.Linq.Enumerable.from(element.Attributes$1(System.Xml.Linq.XName.op_Implicit(name))).elementAt(0);

                        if (attribute != null) {
                            if (!Bridge.referenceEquals(attribute.Value, "")) {
                                return attribute.Value;
                            }
                        }
                    }
                }
                return null;
            },
            GetAttribute: function (i) {
                throw new System.NotImplementedException.ctor();
            },
            GetAttribute$2: function (name, namespaceURI) {
                throw new System.NotImplementedException.ctor();
            },
            IsWhitespaceNode: function (xNode) {
                if (Bridge.is(xNode, System.Xml.Linq.XText)) {
                    var value = Bridge.cast(xNode, System.Xml.Linq.XText).Value;

                    if (System.String.isNullOrEmpty(value) || System.String.isNullOrWhiteSpace(value) || value == null || Bridge.referenceEquals(value, "")) {
                        return true;
                    }
                }

                return false;
            },
            ReadUntilNotNull: function () {
                do {
                    if (!this.Read()) {
                        return;
                    }

                } while (this.LastNode.Type === System.Xml.XmlNodeType.Whitespace);
            },
            Read: function () {
                this.ReadNext();

                if (this.LastNode.Type === System.Xml.XmlNodeType.None) {
                    return false;
                }

                return true;
            },
            MoveToNextNode: function () {
                if (this._nodeStack.Count > 0) {

                    this.LastNode.MoreChild = this.LastNode.NodeEnumerator.System$Collections$IEnumerator$moveNext();
                    return true;
                }
                return false;
            },
            BuildName: function (Name) {
                if (!System.String.isNullOrWhiteSpace(Name.NamespaceName)) {
                    return (Name.NamespaceName || "") + ":" + (Name.LocalName || "");
                }

                return Name.LocalName;
            },
            ReadElementContentAsString$1: function (localName, namespaceURI) {
                var fullName = localName;

                if (!Bridge.referenceEquals(namespaceURI, "")) {
                    fullName = (namespaceURI || "") + ":" + (localName || "");
                }

                if (!Bridge.referenceEquals(fullName, this.Name)) {
                    throw new System.InvalidOperationException.$ctor1("XmlException : " + (this.Name || "") + " is different of " + (fullName || ""));
                }

                var value = "";

                if (!this.IsEmptyElement) {
                    this.ReadUntilNotNull();

                    if (this.LastNode.Type === System.Xml.XmlNodeType.Text) {
                        value = Bridge.cast(this.LastNode.RawNode, System.Xml.Linq.XText).Value;
                    } else {
                        throw new System.InvalidOperationException.$ctor1("XmlException : the current element content can not be read !");
                    }

                    this.ReadUntilNotNull();

                    if (this.LastNode.Type === System.Xml.XmlNodeType.EndElement) {
                        this.ReadUntilNotNull();
                    }
                } else {
                    this.ReadUntilNotNull();
                }

                return value;
            },
            ReadStartElement$1: function (name) {
                if (this.LastNode.Type === System.Xml.XmlNodeType.Whitespace) {
                    this.ReadUntilNotNull();
                }

                if (!Bridge.referenceEquals(this.Name, name)) {
                    throw new System.InvalidOperationException.$ctor1("XmlException : " + (this.Name || "") + " is different of " + (name || ""));
                }

                this.ReadStartElement();
            },
            ReadStartElement: function () {
                if (this.LastNode.Type === System.Xml.XmlNodeType.Whitespace) {
                    this.ReadUntilNotNull();
                }

                if (this.LastNode.Type === System.Xml.XmlNodeType.Element) {
                    this.ReadUntilNotNull();
                }
            },
            GetNode: function (enumerator) {
                if (enumerator == null) {
                    return System.Xml.INTERNAL_NodeType.Default;
                }

                if (!this.LastNode.MoreChild) {
                    return System.Xml.INTERNAL_NodeType.Default;
                }

                var node = enumerator.System$Collections$IEnumerator$Current;

                if (node == null) {
                    if (this.MoveToNextNode()) {
                        return this.GetNode(enumerator);
                    }

                    return System.Xml.INTERNAL_NodeType.Default;
                }

                if (Bridge.is(node, System.Xml.Linq.XNode)) {
                    var xNode = Bridge.cast(node, System.Xml.Linq.XNode);

                    if (Bridge.is(xNode, System.Xml.Linq.XElement)) {
                        return new System.Xml.INTERNAL_NodeType(xNode, System.Xml.XmlNodeType.Element);
                    } else {
                        if (this.IsWhitespaceNode(xNode)) {
                            return new System.Xml.INTERNAL_NodeType(xNode, System.Xml.XmlNodeType.Whitespace);
                        } else {
                            if (Bridge.is(xNode, System.Xml.Linq.XText)) {
                                return new System.Xml.INTERNAL_NodeType(xNode, System.Xml.XmlNodeType.Text);
                            }
                        }
                    }

                    return System.Xml.INTERNAL_NodeType.Default;
                }

                throw new System.Exception((Bridge.toString(node) || "") + " is not an XNode");
            },
            ToNextSiblingOrPreviousNode: function () {
                this.RemoveLastNode();
                if (this.MoveToNextNode()) {
                    var node = this.GetNode(this.LastChildList);

                    if (node.Type !== System.Xml.XmlNodeType.None) {
                        this.AddNewNode(node);
                        return true;
                    } else if (this.LastNode.Type === System.Xml.XmlNodeType.Element) {
                        this.AddNewNode(new System.Xml.INTERNAL_NodeType(this.LastNode, System.Xml.XmlNodeType.EndElement));
                        return true;
                    }
                }

                return false;
            },
            ReadNext: function () {
                if (this.LastNode.Type === System.Xml.XmlNodeType.Attribute) {
                    this.RemoveLastNode();
                }

                if (this.LastNode.Type === System.Xml.XmlNodeType.EndElement) {
                    this.RemoveLastNode();
                    this.ToNextSiblingOrPreviousNode();
                } else {
                    var node = this.GetNode(this.LastChildList);

                    if (node.Type === System.Xml.XmlNodeType.None) {
                        if (this._nodeStack.Count > 0) {
                            if (this.LastNode.Type === System.Xml.XmlNodeType.Element && !this.IsEmptyElement) {
                                this.AddNewNode(new System.Xml.INTERNAL_NodeType(this.LastNode, System.Xml.XmlNodeType.EndElement));
                            } else {
                                this.ToNextSiblingOrPreviousNode();
                            }
                        } else {
                            return;
                        }
                    } else {
                        this.AddNewNode(node);
                    }
                }
            },
            ReadEndElement: function () {
                if (this.LastNode.Type === System.Xml.XmlNodeType.Whitespace) {
                    this.ReadUntilNotNull();
                }

                if (this.LastNode.Type === System.Xml.XmlNodeType.EndElement) {
                    this.ReadUntilNotNull();
                }
            },
            Dispose: function () { },
            MoveToNextAttribute: function () {
                if (this.LastNode.Type === System.Xml.XmlNodeType.Element) {
                    return this.MoveToFirstAttribute();
                } else if (this.LastNode.Type === System.Xml.XmlNodeType.Attribute) {
                    var attribute = Bridge.cast(this.LastNode.RawNode, System.Xml.Linq.XAttribute);

                    this.RemoveLastNode();

                    if (this.LastNode.AttributEnumerator.System$Collections$IEnumerator$moveNext()) {
                        attribute = Bridge.cast(this.LastNode.AttributEnumerator.System$Collections$IEnumerator$Current, System.Xml.Linq.XAttribute);

                        this.AddNewNode(new System.Xml.INTERNAL_NodeType(attribute, System.Xml.XmlNodeType.Attribute));

                        return true;
                    }

                    this.AddNewNode(new System.Xml.INTERNAL_NodeType(attribute, System.Xml.XmlNodeType.Attribute));
                }

                return false;
            },
            MoveToElement: function () {
                if (this.LastNode.Type !== System.Xml.XmlNodeType.Element) {
                    if (this.LastNode.Type === System.Xml.XmlNodeType.Attribute) {
                        this.RemoveLastNode();
                    } else {
                        do {
                            if (!this.ToNextSiblingOrPreviousNode()) {
                                return false;
                            }

                        } while (this.LastNode.Type !== System.Xml.XmlNodeType.Element);
                    }
                }

                return true;
            },
            MoveToContent: function () {
                if (this.LastNode.Type === System.Xml.XmlNodeType.Document) {
                    this.Read();
                }

                return this.NodeType;
            },
            MoveToFirstAttribute: function () {
                if (this.LastNode.Type === System.Xml.XmlNodeType.Attribute) {
                    this.RemoveLastNode();
                }

                if (this.LastNode.Type === System.Xml.XmlNodeType.Element) {
                    var element = this.LastNode.ElementOrNull;

                    if (element != null) {
                        if (System.Linq.Enumerable.from(element.Attributes()).count() > 0) {
                            this.AddNewNode(new System.Xml.INTERNAL_NodeType(System.Linq.Enumerable.from(element.Attributes()).elementAt(0), System.Xml.XmlNodeType.Attribute));

                            return true;
                        }
                    }
                }

                return false;
            },
            MoveToAttribute: function (name) {
                throw new System.NotImplementedException.ctor();
            },
            MoveToAttribute$1: function (name, ns) {
                throw new System.NotImplementedException.ctor();
            },
            ReadAttributeValue: function () {
                throw new System.NotImplementedException.ctor();
            },
            LookupNamespace: function (prefix) {
                throw new System.NotImplementedException.ctor();
            },
            ResolveEntity: function () {
                throw new System.NotImplementedException.ctor();
            }
        }
    });

    Bridge.define("System.Xml.Cshtml5_XmlWriter", {
        inherits: [System.Xml.XmlWriter],
        statics: {
            methods: {
                Create: function (output) {
                    return new System.Xml.Cshtml5_XmlWriter.ctor(output);
                }
            }
        },
        fields: {
            _data: null,
            _header: null,
            _elementsName: null,
            _open: false,
            _openTag: false,
            _isInAttributeValue: false,
            _xmlDoc: null,
            _stream: null,
            _writter: null
        },
        props: {
            WriteState: {
                get: function () {
                    throw new System.NotImplementedException.ctor();
                }
            }
        },
        ctors: {
            $ctor1: function (xmlDoc) {
                this.$initialize();
                System.Xml.XmlWriter.ctor.call(this);
                this._xmlDoc = xmlDoc;
                this._stream = null;
                this.Init();
            },
            ctor: function (stream) {
                this.$initialize();
                System.Xml.XmlWriter.ctor.call(this);
                this._xmlDoc = null;
                this._stream = stream;

                this._writter = new System.IO.StreamWriter.$ctor1(this._stream);

                this.Init();
            }
        },
        methods: {
            Init: function () {
                this._header = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>" + ("\n" || "");
                this._data = "";
                this._elementsName = new (System.Collections.Generic.List$1(System.String)).ctor();
                this._open = true;
                this._openTag = false;
                this._isInAttributeValue = false;
            },
            WriteElementString$1: function (localName, value) {
                this.WriteStartElement$2(null, localName, null);
                this.WriteString(value);
                this.WriteEndElement();
            },
            WriteAttributeString$1: function (localName, value) {
                this.WriteStartAttribute(null, localName, null);
                this.WriteString(value);
                this.WriteEndAttribute();
            },
            WriteString: function (text) {
                if (!this._isInAttributeValue && this._openTag) {
                    this.AddData(">");
                    this._openTag = false;
                }

                this.AddData(text);
            },
            WriteStartAttribute: function (prefix, localName, ns) {
                this.CheckOpen();

                this._isInAttributeValue = true;

                this.AddData(" " + (localName || "") + "=" + "\"");
            },
            WriteEndAttribute: function () {
                this.AddData("\" ");
                this._isInAttributeValue = false;
            },
            WriteEndElement: function () {
                this.CheckOpen();

                this.CloseTagIfOpen(false);

                var Element;

                if (this._elementsName.Count > 0) {
                    Element = this._elementsName.getItem(((this._elementsName.Count - 1) | 0));

                    this._elementsName.removeAt(((this._elementsName.Count - 1) | 0));

                    this.AddData("</" + (Element || "") + ">" + ("\n" || ""));
                }
            },
            Close: function () {
                if (this._open) {
                    this.CloseAllTag();

                    this._open = false;

                    if (this._xmlDoc != null) {
                        this._xmlDoc = System.Xml.Linq.XDocument.Parse(this._data);
                    }
                } else {
                    throw new System.InvalidOperationException.$ctor1("XmlWriter is already close !");
                }
            },
            WriteStartElement$2: function (prefix, localName, ns) {
                this.CheckOpen();

                this.CloseTagIfOpen(true);

                this.AddData("<" + (localName || ""));

                this._elementsName.add(localName);

                this._openTag = true;
            },
            WriteStartElement$3: function (localName) {
                this.WriteStartElement$2(null, localName, null);
            },
            toString: function () {
                return this._data;
            },
            CheckOpen: function () {
                if (!this._open) {
                    throw new System.InvalidOperationException.$ctor1("Error to write in XmlWriter: The document is close !");
                }
            },
            CloseTagIfOpen: function (LineReturn) {
                if (this._openTag) {
                    this.AddData(">");

                    if (LineReturn) {
                        this.AddData("\n");
                    }
                }

                this._isInAttributeValue = false;
                this._openTag = false;
            },
            CloseAllTag: function () {
                while (this._elementsName.Count > 0) {
                    this.WriteEndElement();
                }
            },
            AddData: function (newData) {
                this._data = (this._data || "") + (newData || "");

                if (this._stream != null) {
                    this._writter.Write$10(newData);
                    this._writter.Flush();
                }
            },
            WriteStartDocument: function () {
                throw new System.NotImplementedException.ctor();
            },
            WriteStartDocument$1: function (standalone) { },
            WriteEndDocument: function () {
                throw new System.NotImplementedException.ctor();
            },
            WriteDocType: function (name, pubid, sysid, subset) { },
            WriteFullEndElement: function () { },
            WriteCData: function (text) { },
            WriteComment: function (text) { },
            WriteProcessingInstruction: function (name, text) { },
            WriteEntityRef: function (name) { },
            WriteCharEntity: function (ch) { },
            WriteWhitespace: function (ws) { },
            WriteSurrogateCharEntity: function (lowChar, highChar) { },
            WriteChars: function (buffer, index, count) { },
            WriteRaw: function (buffer, index, count) { },
            WriteRaw$1: function (data) { },
            WriteBase64: function (buffer, index, count) { },
            Flush: function () { },
            LookupPrefix: function (ns) {
                throw new System.NotImplementedException.ctor();
            }
        }
    });

    /**
     * Represents an XML attribute.
     * // Removes this attribute from its parent element.
    //
     * // Sets the value of this attribute.
    //
     * // Converts the current System.Xml.Linq.XAttribute object to a string representation.
    //
     * // Gets an empty collection of attributes.
    //
     * // Determines if this attribute is a namespace declaration.
    //
     *
     * @public
     * @class System.Xml.Linq.XAttribute
     * @augments System.Xml.Linq.XObject
     * @return  {[type]}        // A System.String containing the XML text representation of an attribute and
    // its value.
    //
     */
    Bridge.define("System.Xml.Linq.XAttribute", {
        inherits: [System.Xml.Linq.XObject],
        statics: {
            methods: {
                /**
                 * Cast the value of this System.Xml.Linq.XAttribute to a System.Boolean.
                 *
                 * @static
                 * @public
                 * @this System.Xml.Linq.XAttribute
                 * @memberof System.Xml.Linq.XAttribute
                 * @param   {System.Xml.Linq.XAttribute}    attribute    The System.Xml.Linq.XAttribute to cast to System.Boolean.
                 * @return  {boolean}                                    A System.Boolean that contains the content of this System.Xml.Linq.XAttribute.
                 */
                op_Explicit: function (attribute) {
                    if (attribute == null) {
                        throw new System.ArgumentNullException.$ctor1("The parameter cannot be null.");
                    }
                    if (System.String.isNullOrWhiteSpace(attribute.Value)) {
                        throw new System.FormatException.$ctor1("The attribute does not contain a valid System.Boolean value.");
                    } else {
                        return System.Boolean.parse(attribute.Value);
                    }
                },
                op_Explicit$6: function (attribute) {
                    if (attribute == null) {
                        throw new System.ArgumentNullException.$ctor1("The parameter cannot be null.");
                    }
                    if (System.String.isNullOrWhiteSpace(attribute.Value)) {
                        return null;
                    } else {
                        return System.Boolean.parse(attribute.Value);
                    }
                }/**
                 * Cast the value of this System.Xml.Linq.XAttribute to an System.Int32.
                 *
                 * @static
                 * @public
                 * @this System.Xml.Linq.XAttribute
                 * @memberof System.Xml.Linq.XAttribute
                 * @param   {System.Xml.Linq.XAttribute}    attribute    The System.Xml.Linq.XAttribute to cast to System.Int32.
                 * @return  {number}                                     A System.Int32 that contains the content of this System.Xml.Linq.XAttribute.
                 */
                ,
                op_Explicit$4: function (attribute) {
                    if (attribute == null) {
                        throw new System.ArgumentNullException.$ctor1("The parameter cannot be null.");
                    }
                    if (System.String.isNullOrWhiteSpace(attribute.Value)) {
                        throw new System.FormatException.$ctor1("The attribute does not contain a valid System.int value.");
                    } else {
                        return System.Int32.parse(attribute.Value);
                    }
                },
                op_Explicit$10: function (attribute) {
                    if (attribute == null) {
                        throw new System.ArgumentNullException.$ctor1("The parameter cannot be null.");
                    }
                    if (System.String.isNullOrWhiteSpace(attribute.Value)) {
                        return null;
                    } else {
                        return System.Int32.parse(attribute.Value);
                    }
                }/**
                 * Cast the value of this System.Xml.Linq.XAttribute to a System.UInt32.
                 *
                 * @static
                 * @public
                 * @this System.Xml.Linq.XAttribute
                 * @memberof System.Xml.Linq.XAttribute
                 * @param   {System.Xml.Linq.XAttribute}    attribute    The System.Xml.Linq.XAttribute to cast to System.UInt32.
                 * @return  {number}                                     A System.UInt32 that contains the content of this System.Xml.Linq.XAttribute.
                 */
                ,
                op_Explicit$19: function (attribute) {
                    if (attribute == null) {
                        throw new System.ArgumentNullException.$ctor1("The parameter cannot be null.");
                    }
                    if (System.String.isNullOrWhiteSpace(attribute.Value)) {
                        throw new System.FormatException.$ctor1("The attribute does not contain a valid System.uint value.");
                    } else {
                        return System.UInt32.parse(attribute.Value);
                    }
                },
                op_Explicit$14: function (attribute) {
                    if (attribute == null) {
                        throw new System.ArgumentNullException.$ctor1("The parameter cannot be null.");
                    }
                    if (System.String.isNullOrWhiteSpace(attribute.Value)) {
                        return null;
                    } else {
                        return System.UInt32.parse(attribute.Value);
                    }
                }/**
                 * Cast the value of this System.Xml.Linq.XAttribute to an System.Int64.
                 *
                 * @static
                 * @public
                 * @this System.Xml.Linq.XAttribute
                 * @memberof System.Xml.Linq.XAttribute
                 * @param   {System.Xml.Linq.XAttribute}    attribute    The System.Xml.Linq.XAttribute to cast to System.Int64.
                 * @return  {System.Int64}                               A System.Int64 that contains the content of this System.Xml.Linq.XAttribute.
                 */
                ,
                op_Explicit$5: function (attribute) {
                    if (attribute == null) {
                        throw new System.ArgumentNullException.$ctor1("The parameter cannot be null.");
                    }
                    if (System.String.isNullOrWhiteSpace(attribute.Value)) {
                        throw new System.FormatException.$ctor1("The attribute does not contain a valid System.long value.");
                    } else {
                        return System.Int64.parse(attribute.Value);
                    }
                },
                op_Explicit$11: function (attribute) {
                    if (attribute == null) {
                        throw new System.ArgumentNullException.$ctor1("The parameter cannot be null.");
                    }
                    if (System.String.isNullOrWhiteSpace(attribute.Value)) {
                        return System.Int64.lift(null);
                    } else {
                        return System.Int64.parse(attribute.Value);
                    }
                }/**
                 * Cast the value of this System.Xml.Linq.XAttribute to a System.UInt64.
                 *
                 * @static
                 * @public
                 * @this System.Xml.Linq.XAttribute
                 * @memberof System.Xml.Linq.XAttribute
                 * @param   {System.Xml.Linq.XAttribute}    attribute    The System.Xml.Linq.XAttribute to cast to System.UInt64.
                 * @return  {System.UInt64}                              A System.UInt64 that contains the content of this System.Xml.Linq.XAttribute.
                 */
                ,
                op_Explicit$20: function (attribute) {
                    if (attribute == null) {
                        throw new System.ArgumentNullException.$ctor1("The parameter cannot be null.");
                    }
                    if (System.String.isNullOrWhiteSpace(attribute.Value)) {
                        throw new System.FormatException.$ctor1("The attribute does not contain a valid System.ulong value.");
                    } else {
                        return System.UInt64.parse(attribute.Value);
                    }
                },
                op_Explicit$15: function (attribute) {
                    if (attribute == null) {
                        throw new System.ArgumentNullException.$ctor1("The parameter cannot be null.");
                    }
                    if (System.String.isNullOrWhiteSpace(attribute.Value)) {
                        return System.UInt64.lift(null);
                    } else {
                        return System.UInt64.parse(attribute.Value);
                    }
                }/**
                 * Cast the value of this System.Xml.Linq.XAttribute to a System.DateTime.
                 *
                 * @static
                 * @public
                 * @this System.Xml.Linq.XAttribute
                 * @memberof System.Xml.Linq.XAttribute
                 * @param   {System.Xml.Linq.XAttribute}    attribute    The System.Xml.Linq.XAttribute to cast to System.DateTime.
                 * @return  {System.DateTime}                            A System.DateTime that contains the content of this System.Xml.Linq.XAttribute.
                 */
                ,
                op_Explicit$1: function (attribute) {
                    if (attribute == null) {
                        throw new System.ArgumentNullException.$ctor1("The parameter cannot be null.");
                    }
                    if (System.String.isNullOrWhiteSpace(attribute.Value)) {
                        throw new System.FormatException.$ctor1("The attribute does not contain a valid System.DateTime value.");
                    } else {
                        return System.DateTime.parse(attribute.Value);
                    }
                },
                op_Explicit$7: function (attribute) {
                    if (attribute == null) {
                        throw new System.ArgumentNullException.$ctor1("The parameter cannot be null.");
                    }
                    if (System.String.isNullOrWhiteSpace(attribute.Value)) {
                        return null;
                    } else {
                        return System.DateTime.parse(attribute.Value);
                    }
                }/**
                 * Cast the value of this System.Xml.Linq.XAttribute to a System.Guid.
                 *
                 * @static
                 * @public
                 * @this System.Xml.Linq.XAttribute
                 * @memberof System.Xml.Linq.XAttribute
                 * @param   {System.Xml.Linq.XAttribute}    attribute    The System.Xml.Linq.XAttribute to cast to System.Guid.
                 * @return  {System.Guid}                                A System.Guid that contains the content of this System.Xml.Linq.XAttribute.
                 */
                ,
                op_Explicit$3: function (attribute) {
                    if (attribute == null) {
                        throw new System.ArgumentNullException.$ctor1("The parameter cannot be null.");
                    }
                    if (System.String.isNullOrWhiteSpace(attribute.Value)) {
                        throw new System.FormatException.$ctor1("The attribute does not contain a valid System.Guid value.");
                    } else {
                        return System.Guid.Parse(attribute.Value);
                    }
                },
                op_Explicit$9: function (attribute) {
                    if (attribute == null) {
                        throw new System.ArgumentNullException.$ctor1("The parameter cannot be null.");
                    }
                    if (System.String.isNullOrWhiteSpace(attribute.Value)) {
                        return null;
                    } else {
                        return System.Guid.Parse(attribute.Value);
                    }
                }/**
                 * Cast the value of this System.Xml.Linq.XAttribute to a System.TimeSpan.
                 *
                 * @static
                 * @public
                 * @this System.Xml.Linq.XAttribute
                 * @memberof System.Xml.Linq.XAttribute
                 * @param   {System.Xml.Linq.XAttribute}    attribute    The System.Xml.Linq.XAttribute to cast to System.TimeSpan.
                 * @return  {System.TimeSpan}                            A System.TimeSpan that contains the content of this System.Xml.Linq.XAttribute.
                 */
                ,
                op_Explicit$18: function (attribute) {
                    if (attribute == null) {
                        throw new System.ArgumentNullException.$ctor1("The parameter cannot be null.");
                    }
                    if (System.String.isNullOrWhiteSpace(attribute.Value)) {
                        throw new System.FormatException.$ctor1("The attribute does not contain a valid System.TimeSpan value.");
                    } else {
                        return INTERNAL_BridgeWorkarounds.TimeSpanParse(attribute.Value);
                    }
                },
                op_Explicit$13: function (attribute) {
                    if (attribute == null) {
                        throw new System.ArgumentNullException.$ctor1("The parameter cannot be null.");
                    }
                    if (System.String.isNullOrWhiteSpace(attribute.Value)) {
                        return null;
                    } else {
                        return INTERNAL_BridgeWorkarounds.TimeSpanParse(attribute.Value);
                    }
                }/**
                 * // Cast the value of this System.Xml.Linq.XAttribute to a System.DateTimeOffset.
                //
                 * Cast the value of this System.Xml.Linq.XAttribute to a System.String.
                 *
                 * @static
                 * @public
                 * @this System.Xml.Linq.XAttribute
                 * @memberof System.Xml.Linq.XAttribute
                 * @param   {System.Xml.Linq.XAttribute}    attribute    The System.Xml.Linq.XAttribute to cast to System.String.
                 * @return  {string}                                     A System.String that contains the content of this System.Xml.Linq.XAttribute.
                 */
                ,
                op_Explicit$17: function (attribute) {
                    return attribute.Value;
                }/**
                 * // Cast the value of this System.Xml.Linq.XAttribute to a System.Decimal.
                //
                 * Cast the value of this System.Xml.Linq.XAttribute to a System.Double.
                 *
                 * @static
                 * @public
                 * @this System.Xml.Linq.XAttribute
                 * @memberof System.Xml.Linq.XAttribute
                 * @param   {System.Xml.Linq.XAttribute}    attribute    The System.Xml.Linq.XAttribute to cast to System.Double.
                 * @return  {number}                                     A System.Double that contains the content of this System.Xml.Linq.XAttribute.
                 */
                ,
                op_Explicit$2: function (attribute) {
                    if (attribute == null) {
                        throw new System.ArgumentNullException.$ctor1("The parameter cannot be null.");
                    }
                    if (System.String.isNullOrWhiteSpace(attribute.Value)) {
                        throw new System.FormatException.$ctor1("The attribute does not contain a valid System.double value.");
                    } else {
                        return System.Double.parse(attribute.Value);
                    }
                },
                op_Explicit$8: function (attribute) {
                    if (attribute == null) {
                        throw new System.ArgumentNullException.$ctor1("The parameter cannot be null.");
                    }
                    if (System.String.isNullOrWhiteSpace(attribute.Value)) {
                        return null;
                    } else {
                        return System.Double.parse(attribute.Value);
                    }
                }/**
                 * Cast the value of this System.Xml.Linq.XAttribute to a System.Single.
                 *
                 * @static
                 * @public
                 * @this System.Xml.Linq.XAttribute
                 * @memberof System.Xml.Linq.XAttribute
                 * @param   {System.Xml.Linq.XAttribute}    attribute    The System.Xml.Linq.XAttribute to cast to System.Single.
                 * @return  {number}                                     A System.Single that contains the content of this System.Xml.Linq.XAttribute.
                 */
                ,
                op_Explicit$16: function (attribute) {
                    if (attribute == null) {
                        throw new System.ArgumentNullException.$ctor1("The parameter cannot be null.");
                    }
                    if (System.String.isNullOrWhiteSpace(attribute.Value)) {
                        throw new System.FormatException.$ctor1("The attribute does not contain a valid System.float value.");
                    } else {
                        return System.Single.parse(attribute.Value);
                    }
                },
                op_Explicit$12: function (attribute) {
                    if (attribute == null) {
                        throw new System.ArgumentNullException.$ctor1("The parameter cannot be null.");
                    }
                    if (System.String.isNullOrWhiteSpace(attribute.Value)) {
                        return null;
                    } else {
                        return System.Single.parse(attribute.Value);
                    }
                }
            }
        },
        fields: {
            isJsValueUpToDate: false,
            /**
             * Note: it is up to the element that sets this to handle the corresponding actions (adding the attribute and setting the value when setting it, removing the attribute when unsetting it).
             *
             * @instance
             * @memberof System.Xml.Linq.XAttribute
             * @type System.Object
             */
            INTERNAL_containerJSNode: null,
            _nameWithPrefix: null,
            _name: null,
            _value: null
        },
        props: {
            /**
             * Gets the expanded name of this attribute.
             *
             * @instance
             * @public
             * @readonly
             * @memberof System.Xml.Linq.XAttribute
             * @function Name
             * @type System.Xml.Linq.XName
             */
            Name: {
                get: function () {
                    return this._name;
                }
            },
            /**
             * // Gets the next attribute of the parent element.
            //
             * // Gets the node type for this node.
            //
             * // Gets the previous attribute of the parent element.
            //
             * Gets or sets the value of this attribute.
             *
             * @instance
             * @public
             * @memberof System.Xml.Linq.XAttribute
             * @function Value
             * @type string
             */
            Value: {
                get: function () {
                    if (this.INTERNAL_containerJSNode != null && this.isJsValueUpToDate) {
                        var attributejs;
                        if (System.String.isNullOrWhiteSpace(this.Name.NamespaceName)) {
                            attributejs = this.INTERNAL_containerJSNode.getAttribute(this.Name.LocalName);
                        } else {
                            attributejs = this.INTERNAL_containerJSNode.getAttributeNS(this.Name.NamespaceName,this.Name.LocalName);
                        }
                        if (System.Convert.toBoolean(attributejs != undefined && attributejs != null)) {
                            this._value = System.Convert.toString(attributejs);
                        }
                    }
                    return this._value;
                },
                set: function (value) {
                    if (value == null) {
                        throw new System.ArgumentNullException.ctor();
                    }
                    this.isJsValueUpToDate = false;
                    if (this.INTERNAL_containerJSNode != null) {
                        if (System.String.isNullOrWhiteSpace(this.Name.NamespaceName)) {
                            this.INTERNAL_containerJSNode.setAttribute(this.Name.LocalName, value);
                        } else {
                            var str = System.Convert.toString(this.INTERNAL_containerJSNode.lookupPrefix(this.Name.NamespaceName));
                            if (!System.String.isNullOrWhiteSpace(str)) {
                                this.INTERNAL_containerJSNode.setAttribute(this.Name.LocalName, value);
                                var jsAttribute = this.INTERNAL_containerJSNode.getAttribute(this.Name.LocalName).prefix = str;
                            }

                            this.INTERNAL_containerJSNode.setAttributeNS(this.Name.NamespaceName, this.Name.LocalName, value);
                        }
                        this.isJsValueUpToDate = true;
                    }

                    this._value = value;
                }
            }
        },
        ctors: {
            init: function () {
                this.isJsValueUpToDate = false;
            },
            /**
             * Initializes a new instance of the System.Xml.Linq.XAttribute class from another
             System.Xml.Linq.XAttribute object.
             *
             * @instance
             * @public
             * @this System.Xml.Linq.XAttribute
             * @memberof System.Xml.Linq.XAttribute
             * @param   {System.Xml.Linq.XAttribute}    other    An System.Xml.Linq.XAttribute object to copy from.
             * @return  {void}
             */
            ctor: function (other) {
                this.$initialize();
                System.Xml.Linq.XObject.ctor.call(this);
                if (other == null) {
                    throw new System.ArgumentNullException.ctor();
                }
                this._name = other.Name;
                this._value = other.Value;
                this._nameWithPrefix = other._nameWithPrefix;
            },
            /**
             * Initializes a new instance of the System.Xml.Linq.XAttribute class from the
             specified name and value.
             *
             * @instance
             * @public
             * @this System.Xml.Linq.XAttribute
             * @memberof System.Xml.Linq.XAttribute
             * @param   {System.Xml.Linq.XName}    name     The System.Xml.Linq.XName of the attribute.
             * @param   {System.Object}            value    An System.Object containing the value of the attribute.
             * @return  {void}
             */
            $ctor1: function (name, value) {
                this.$initialize();
                System.Xml.Linq.XObject.ctor.call(this);
                if (System.Xml.Linq.XName.op_Equality(name, null)) {
                    throw new System.ArgumentNullException.$ctor1("The argument \"name\" cannot be null.");
                }
                if (value == null) {
                    throw new System.ArgumentNullException.$ctor1("The argument \"value\" cannot be null.");
                }
                this._name = name;
                this._value = Bridge.toString(value);
                this._nameWithPrefix = name;
            },
            $ctor2: function (name, value, nameWithPrefix) {
                System.Xml.Linq.XAttribute.$ctor1.call(this, name, value);
                this._nameWithPrefix = nameWithPrefix;
            }
        }
    });

    /**
     * Represents the abstract concept of a node (element, comment, document type,
     processing instruction, or text node) in the XML tree.
     *
     * @abstract
     * @public
     * @class System.Xml.Linq.XNode
     * @augments System.Xml.Linq.XObject
     */
    Bridge.define("System.Xml.Linq.XNode", {
        inherits: [System.Xml.Linq.XObject],
        fields: {
            INTERNAL_jsnode: null
        },
        props: {
            /**
             * Gets the System.Xml.Linq.XDocument for this System.Xml.Linq.XObject.
             *
             * @instance
             * @public
             * @readonly
             * @memberof System.Xml.Linq.XNode
             * @function Document
             * @type System.Xml.Linq.XDocument
             */
            Document: {
                get: function () {
                    var documentJSNode = this.INTERNAL_jsnode.ownerDocument;
                    if (documentJSNode != null) {
                        return Bridge.cast(System.Xml.Linq.XDocument.GetXNodeFromJSNode(documentJSNode), System.Xml.Linq.XDocument);
                    }
                    return null;
                }
            }
        },
        methods: {
            /**
             * Returns the indented XML for this node.
             *
             * @instance
             * @public
             * @override
             * @this System.Xml.Linq.XNode
             * @memberof System.Xml.Linq.XNode
             * @return  {string}        A System.String containing the indented XML.
             */
            toString: function () {
                return this.ToString(true);
            },
            ToString: function (indentXml) {
                var nonIndentedXML = null;
                if ((window.IE_VERSION) || CSHTML5.Interop.IsRunningInTheSimulator) {
                    nonIndentedXML = System.Convert.toString((new XMLSerializer()).serializeToString(this.INTERNAL_jsnode).replace(/Prefix\d+:AttributeToRemoveFromXMLSerialization=\System.Runtime.Serialization.DataContractSerializer_Helpers.ATTRIBUTE_TO_REMOVE_FROM_XMLSERIALIZATION_VALUE/g, ""));
                } else {
                    if (System.Convert.toBoolean(this.INTERNAL_jsnode.outerHTML != undefined)) {
                        nonIndentedXML = System.Convert.toString(this.INTERNAL_jsnode.outerHTML);
                    } else {
                        nonIndentedXML = System.Convert.toString(this.INTERNAL_jsnode.firstchild.outerHTML);
                    }
                }

                if (indentXml) {

                    var indentedXML = Bridge.toString(window.vkbeautify.xml(nonIndentedXML, 2));
                    return indentedXML;
                } else {
                    return nonIndentedXML;
                }
            },
            CascadeDocumentToChildren: function () { },
            
            Clone: function () {
                throw new System.NotImplementedException.ctor();
            },
            HasParent: function () {
                return System.Convert.toBoolean(this.INTERNAL_jsnode.parentNode != undefined && this.INTERNAL_jsnode.parentNode != null);
            }
        }
    });

    /**
     * Controls deserialization by the System.Xml.Serialization.XmlSerializer class.
     *
     * @abstract
     * @public
     * @class System.Xml.Serialization.XmlSerializationReader
     * @augments System.Xml.Serialization.XmlSerializationGeneratedCode
     */
    Bridge.define("System.Xml.Serialization.XmlSerializationReader", {
        inherits: [System.Xml.Serialization.XmlSerializationGeneratedCode],
        statics: {
            methods: {
                /**
                 * Produces a System.Char object from an input string.
                 *
                 * @static
                 * @protected
                 * @this System.Xml.Serialization.XmlSerializationReader
                 * @memberof System.Xml.Serialization.XmlSerializationReader
                 * @param   {string}    value    A string to translate into a System.Char object.
                 * @return  {number}             A System.Char object.
                 */
                ToChar: function (value) {
                    throw new System.NotImplementedException.ctor();
                },
                /**
                 * Produces a System.DateTime object from an input string.
                 *
                 * @static
                 * @protected
                 * @this System.Xml.Serialization.XmlSerializationReader
                 * @memberof System.Xml.Serialization.XmlSerializationReader
                 * @param   {string}             value    A string to translate into a System.DateTime object.
                 * @return  {System.DateTime}             A System.DateTime object.
                 */
                ToDateTime: function (value) {
                    throw new System.NotImplementedException.ctor();
                },
                /**
                 * Produces a base-64 byte array from an input string.
                 *
                 * @static
                 * @protected
                 * @this System.Xml.Serialization.XmlSerializationReader
                 * @memberof System.Xml.Serialization.XmlSerializationReader
                 * @param   {string}            value    A string to translate into a base-64 byte array.
                 * @return  {Array.<number>}             A base-64 byte array.
                 */
                ToByteArrayBase64: function (value) {
                    if (value == null) {
                        return null;
                    }
                    value = value.trim();
                    if (value.length === 0) {
                        return System.Array.init(0, 0, System.Byte);
                    }
                    return System.Convert.fromBase64String(value);
                }
            }
        },
        props: {
            /**
             * Gets the System.Xml.XmlReader object that is being used by System.Xml.Serialization.XmlSerializationReader.
             *
             * @instance
             * @protected
             * @readonly
             * @memberof System.Xml.Serialization.XmlSerializationReader
             * @function Reader
             * @type System.Xml.XmlReader
             */
            Reader: {
                get: function () {
                    throw new System.NotImplementedException.ctor();
                }
            },
            /**
             * Gets the current count of the System.Xml.XmlReader.
             *
             * @instance
             * @protected
             * @readonly
             * @memberof System.Xml.Serialization.XmlSerializationReader
             * @function ReaderCount
             * @type number
             */
            ReaderCount: {
                get: function () {
                    throw new System.NotImplementedException.ctor();
                }
            }
        },
        ctors: {
            /**
             * Initializes a new instance of the System.Xml.Serialization.XmlSerializationReader
             class.
             *
             * @instance
             * @protected
             * @this System.Xml.Serialization.XmlSerializationReader
             * @memberof System.Xml.Serialization.XmlSerializationReader
             * @return  {void}
             */
            ctor: function () {
                this.$initialize();
                System.Xml.Serialization.XmlSerializationGeneratedCode.ctor.call(this);
                throw new System.NotImplementedException.ctor();
            }
        },
        methods: {
            /**
             * Checks whether the deserializer has advanced.
             *
             * @instance
             * @protected
             * @this System.Xml.Serialization.XmlSerializationReader
             * @memberof System.Xml.Serialization.XmlSerializationReader
             * @param   {System.Int32}    whileIterations    The current count in a while loop.
             * @param   {System.Int32}    readerCount        The current System.Xml.Serialization.XmlSerializationReader.ReaderCount.
             * @return  {void}
             */
            CheckReaderCount: function (whileIterations, readerCount) {
                throw new System.NotImplementedException.ctor();
            },
            /**
             * Creates an System.InvalidOperationException that indicates that an object
             being deserialized should be abstract.
             *
             * @instance
             * @protected
             * @this System.Xml.Serialization.XmlSerializationReader
             * @memberof System.Xml.Serialization.XmlSerializationReader
             * @param   {string}              name    The name of the abstract type.
             * @param   {string}              ns      The .NET Framework namespace of the abstract type.
             * @return  {System.Exception}            An System.InvalidOperationException exception.
             */
            CreateAbstractTypeException: function (name, ns) {
                throw new System.NotImplementedException.ctor();
            },
            /**
             * Populates an object from its XML representation at the current location of
             the System.Xml.XmlReader, with an option to read the inner element.
             *
             * @instance
             * @protected
             * @this System.Xml.Serialization.XmlSerializationReader
             * @memberof System.Xml.Serialization.XmlSerializationReader
             * @param   {string}              xsdDerived    The local name of the derived XML Schema data type.
             * @param   {string}              nsDerived     The namespace of the derived XML Schema data type.
             * @param   {string}              xsdBase       The local name of the base XML Schema data type.
             * @param   {string}              nsBase        The namespace of the base XML Schema data type.
             * @param   {string}              clrDerived    The namespace of the derived .NET Framework type.
             * @param   {string}              clrBase       The name of the base .NET Framework type.
             * @return  {System.Exception}                  An System.InvalidOperationException exception.
             */
            CreateBadDerivationException: function (xsdDerived, nsDerived, xsdBase, nsBase, clrDerived, clrBase) {
                throw new System.NotImplementedException.ctor();
            },
            /**
             * Creates an System.InvalidOperationException that indicates that an object
             being deserialized cannot be instantiated because the constructor throws
             a security exception.
             *
             * @instance
             * @protected
             * @this System.Xml.Serialization.XmlSerializationReader
             * @memberof System.Xml.Serialization.XmlSerializationReader
             * @param   {string}              typeName    The name of the type.
             * @return  {System.Exception}                An System.InvalidOperationException exception.
             */
            CreateCtorHasSecurityException: function (typeName) {
                throw new System.NotImplementedException.ctor();
            },
            /**
             * Creates an System.InvalidOperationException that indicates that an object
             being deserialized cannot be instantiated because there is no constructor
             available.
             *
             * @instance
             * @protected
             * @this System.Xml.Serialization.XmlSerializationReader
             * @memberof System.Xml.Serialization.XmlSerializationReader
             * @param   {string}              typeName    The name of the type.
             * @return  {System.Exception}                An System.InvalidOperationException exception.
             */
            CreateInaccessibleConstructorException: function (typeName) {
                throw new System.NotImplementedException.ctor();
            },
            /**
             * Creates an System.InvalidCastException that indicates that an explicit reference
             conversion failed.
             *
             * @instance
             * @protected
             * @this System.Xml.Serialization.XmlSerializationReader
             * @memberof System.Xml.Serialization.XmlSerializationReader
             * @param   {System.Type}         type     The System.Type that an object cannot be cast to. This type is incorporated
             into the exception message.
             * @param   {System.Object}       value    The object that cannot be cast. This object is incorporated into the exception
             message.
             * @return  {System.Exception}             An System.InvalidCastException exception.
             */
            CreateInvalidCastException: function (type, value) {
                throw new System.NotImplementedException.ctor();
            },
            /**
             * Creates an System.InvalidCastException that indicates that an explicit reference
             conversion failed.
             *
             * @instance
             * @protected
             * @this System.Xml.Serialization.XmlSerializationReader
             * @memberof System.Xml.Serialization.XmlSerializationReader
             * @param   {System.Type}         type     The System.Type that an object cannot be cast to. This type is incorporated
             into the exception message.
             * @param   {System.Object}       value    The object that cannot be cast. This object is incorporated into the exception
             message.
             * @param   {string}              id       A string identifier.
             * @return  {System.Exception}             An System.InvalidCastException exception.
             */
            CreateInvalidCastException$1: function (type, value, id) {
                throw new System.NotImplementedException.ctor();
            },
            /**
             * Creates an System.InvalidOperationException that indicates that a derived
             type that is mapped to an XML Schema data type cannot be located.
             *
             * @instance
             * @protected
             * @this System.Xml.Serialization.XmlSerializationReader
             * @memberof System.Xml.Serialization.XmlSerializationReader
             * @param   {string}              name       The local name of the XML Schema data type that is mapped to the unavailable
             derived type.
             * @param   {string}              ns         The namespace of the XML Schema data type that is mapped to the unavailable
             derived type.
             * @param   {string}              clrType    The full name of the .NET Framework base type for which a derived type cannot
             be located.
             * @return  {System.Exception}               An System.InvalidOperationException exception.
             */
            CreateMissingIXmlSerializableType: function (name, ns, clrType) {
                throw new System.NotImplementedException.ctor();
            },
            /**
             * Creates an System.InvalidOperationException that indicates that a SOAP-encoded
             collection type cannot be modified and its values cannot be filled in.
             *
             * @instance
             * @protected
             * @this System.Xml.Serialization.XmlSerializationReader
             * @memberof System.Xml.Serialization.XmlSerializationReader
             * @param   {string}              name    The fully qualified name of the .NET Framework type for which there is a
             mapping.
             * @return  {System.Exception}            An System.InvalidOperationException exception.
             */
            CreateReadOnlyCollectionException: function (name) {
                throw new System.NotImplementedException.ctor();
            },
            /**
             * Creates an System.InvalidOperationException that indicates that an enumeration
             value is not valid.
             *
             * @instance
             * @protected
             * @this System.Xml.Serialization.XmlSerializationReader
             * @memberof System.Xml.Serialization.XmlSerializationReader
             * @param   {string}              value       The enumeration value that is not valid.
             * @param   {System.Type}         enumType    The enumeration type.
             * @return  {System.Exception}                An System.InvalidOperationException exception.
             */
            CreateUnknownConstantException: function (value, enumType) {
                throw new System.NotImplementedException.ctor();
            },
            /**
             * Creates an System.InvalidOperationException that indicates that the current
             position of System.Xml.XmlReader represents an unknown XML node.
             *
             * @instance
             * @protected
             * @this System.Xml.Serialization.XmlSerializationReader
             * @memberof System.Xml.Serialization.XmlSerializationReader
             * @return  {System.Exception}        An System.InvalidOperationException exception.
             */
            CreateUnknownNodeException: function () {
                throw new System.NotImplementedException.ctor();
            },
            /**
             * Creates an System.InvalidOperationException that indicates that a type is
             unknown.
             *
             * @instance
             * @protected
             * @this System.Xml.Serialization.XmlSerializationReader
             * @memberof System.Xml.Serialization.XmlSerializationReader
             * @param   {System.Xml.XmlQualifiedName}    type    An System.Xml.XmlQualifiedName that represents the name of the unknown type.
             * @return  {System.Exception}                       An System.InvalidOperationException exception.
             */
            CreateUnknownTypeException: function (type) {
                throw new System.NotImplementedException.ctor();
            },
            /**
             * Ensures that a given array, or a copy, is large enough to contain a specified
             index.
             *
             * @instance
             * @protected
             * @this System.Xml.Serialization.XmlSerializationReader
             * @memberof System.Xml.Serialization.XmlSerializationReader
             * @param   {Array}          a              The System.Array that is being checked.
             * @param   {number}         index          The required index.
             * @param   {System.Type}    elementType    The System.Type of the array's elements.
             * @return  {Array}                         The existing System.Array, if it is already large enough; otherwise, a new,
             larger array that contains the original array's elements.
             */
            EnsureArrayIndex: function (a, index, elementType) {
                throw new System.NotImplementedException.ctor();
            },
            /**
             * Determines whether the XML element where the System.Xml.XmlReader is currently
             positioned has a null attribute set to the value true.
             *
             * @instance
             * @protected
             * @this System.Xml.Serialization.XmlSerializationReader
             * @memberof System.Xml.Serialization.XmlSerializationReader
             * @return  {boolean}        true if System.Xml.XmlReader is currently positioned over a null attribute
             with the value true; otherwise, false.
             */
            GetNullAttr: function () {
                throw new System.NotImplementedException.ctor();
            },
            /**
             * Gets the value of the xsi:type attribute for the XML element at the current
             location of the System.Xml.XmlReader.
             *
             * @instance
             * @protected
             * @this System.Xml.Serialization.XmlSerializationReader
             * @memberof System.Xml.Serialization.XmlSerializationReader
             * @return  {System.Xml.XmlQualifiedName}        An XML qualified name that indicates the data type of an XML element.
             */
            GetXsiType: function () {
                throw new System.NotImplementedException.ctor();
            },
            /**
             * Determines whether an XML attribute name indicates an XML namespace.
             *
             * @instance
             * @protected
             * @this System.Xml.Serialization.XmlSerializationReader
             * @memberof System.Xml.Serialization.XmlSerializationReader
             * @param   {string}     name    The name of an XML attribute.
             * @return  {boolean}            true if the XML attribute name indicates an XML namespace; otherwise, false.
             */
            IsXmlnsAttribute: function (name) {
                throw new System.NotImplementedException.ctor();
            },
            /**
             * Makes the System.Xml.XmlReader read an XML end tag.
             *
             * @instance
             * @protected
             * @this System.Xml.Serialization.XmlSerializationReader
             * @memberof System.Xml.Serialization.XmlSerializationReader
             * @return  {void}
             */
            ReadEndElement: function () {
                throw new System.NotImplementedException.ctor();
            },
            /**
             * Instructs the System.Xml.XmlReader to read the current XML element if the
             element has a null attribute with the value true.
             *
             * @instance
             * @protected
             * @this System.Xml.Serialization.XmlSerializationReader
             * @memberof System.Xml.Serialization.XmlSerializationReader
             * @return  {boolean}        true if the element has a null="true" attribute value and has been read;
             otherwise, false.
             */
            ReadNull: function () {
                throw new System.NotImplementedException.ctor();
            },
            /**
             * Populates an object from its XML representation at the current location of
             the System.Xml.XmlReader.
             *
             * @instance
             * @protected
             * @this System.Xml.Serialization.XmlSerializationReader
             * @memberof System.Xml.Serialization.XmlSerializationReader
             * @param   {System.Xml.Serialization.IXmlSerializable}    serializable    An System.Xml.Serialization.IXmlSerializable that corresponds to the current
             position of the System.Xml.XmlReader.
             * @return  {System.Xml.Serialization.IXmlSerializable}                    An object that implements the System.Xml.Serialization.IXmlSerializable interface
             with its members populated from the location of the System.Xml.XmlReader.
             */
            ReadSerializable: function (serializable) {
                throw new System.NotImplementedException.ctor();
            },
            /**
             * This method supports the .NET Framework infrastructure and is not intended
             to be used directly from your code.
             *
             * @instance
             * @protected
             * @this System.Xml.Serialization.XmlSerializationReader
             * @memberof System.Xml.Serialization.XmlSerializationReader
             * @param   {System.Xml.Serialization.IXmlSerializable}    serializable    An IXmlSerializable object that corresponds to the current position of the
             XMLReader.
             * @param   {boolean}                                      wrappedAny      Specifies whether the serializable object is wrapped.
             * @return  {System.Xml.Serialization.IXmlSerializable}                    An object that implements the IXmlSerializable interface with its members
             populated from the location of the XmlReader.
             */
            ReadSerializable$1: function (serializable, wrappedAny) {
                throw new System.NotImplementedException.ctor();
            },
            /**
             * Produces the result of a call to the System.Xml.XmlReader.ReadString() method
             appended to the input value.
             *
             * @instance
             * @protected
             * @this System.Xml.Serialization.XmlSerializationReader
             * @memberof System.Xml.Serialization.XmlSerializationReader
             * @param   {string}    value    A string to prefix to the result of a call to the System.Xml.XmlReader.ReadString()
             method.
             * @return  {string}             The result of call to the System.Xml.XmlReader.ReadString() method appended
             to the input value.
             */
            ReadString: function (value) {
                throw new System.NotImplementedException.ctor();
            },
            /**
             * Returns the result of a call to the System.Xml.XmlReader.ReadString() method
             of the System.Xml.XmlReader class, trimmed of white space if needed, and
             appended to the input value.
             *
             * @instance
             * @protected
             * @this System.Xml.Serialization.XmlSerializationReader
             * @memberof System.Xml.Serialization.XmlSerializationReader
             * @param   {string}     value    A string that will be appended to.
             * @param   {boolean}    trim     true if the result of the read operation should be trimmed; otherwise, false.
             * @return  {string}              The result of the read operation appended to the input value.
             */
            ReadString$1: function (value, trim) {
                throw new System.NotImplementedException.ctor();
            },
            /**
             * Ensures that a given array, or a copy, is no larger than a specified length.
             *
             * @instance
             * @protected
             * @this System.Xml.Serialization.XmlSerializationReader
             * @memberof System.Xml.Serialization.XmlSerializationReader
             * @param   {Array}          a              The array that is being checked.
             * @param   {number}         length         The maximum length of the array.
             * @param   {System.Type}    elementType    The System.Type of the array's elements.
             * @param   {boolean}        isNullable     true if null for the array, if present for the input array, can be returned;
             otherwise, a new, smaller array.
             * @return  {Array}                         The existing System.Array, if it is already small enough; otherwise, a new,
             smaller array that contains the original array's elements up to the size
             of length.
             */
            ShrinkArray: function (a, length, elementType, isNullable) {
                return null;
            },
            /**
             * Instructs the System.Xml.XmlReader to read the string value at its current
             position and return it as a base-64 byte array.
             *
             * @instance
             * @protected
             * @this System.Xml.Serialization.XmlSerializationReader
             * @memberof System.Xml.Serialization.XmlSerializationReader
             * @param   {boolean}           isNull    true to return null; false to return a base-64 byte array.
             * @return  {Array.<number>}              A base-64 byte array; otherwise, null if the value of the isNull parameter
             is true.
             */
            ToByteArrayBase64: function (isNull) {
                if (isNull) {
                    return null;
                }
                return null;
            },
            /**
             * Raises an System.Xml.Serialization.XmlSerializer.UnknownNode event for the
             current position of the System.Xml.XmlReader.
             *
             * @instance
             * @protected
             * @this System.Xml.Serialization.XmlSerializationReader
             * @memberof System.Xml.Serialization.XmlSerializationReader
             * @param   {System.Object}    o    The object that is being deserialized.
             * @return  {void}
             */
            UnknownNode: function (o) {
                throw new System.NotImplementedException.ctor();
            },
            /**
             * Raises an System.Xml.Serialization.XmlSerializer.UnknownNode event for the
             current position of the System.Xml.XmlReader.
             *
             * @instance
             * @protected
             * @this System.Xml.Serialization.XmlSerializationReader
             * @memberof System.Xml.Serialization.XmlSerializationReader
             * @param   {System.Object}    o         The object being deserialized.
             * @param   {string}           qnames    A comma-delimited list of XML qualified names.
             * @return  {void}
             */
            UnknownNode$1: function (o, qnames) {
                throw new System.NotImplementedException.ctor();
            }
        }
    });

    
    Bridge.define("System.Xml.Serialization.XmlSerializationWriter", {
        inherits: [System.Xml.Serialization.XmlSerializationGeneratedCode],
        statics: {
            methods: {
                /**
                 * Produces a string from an input System.Char.
                 *
                 * @static
                 * @protected
                 * @this System.Xml.Serialization.XmlSerializationWriter
                 * @memberof System.Xml.Serialization.XmlSerializationWriter
                 * @param   {number}    value    A System.Char to translate to a string.
                 * @return  {string}             The System.Char value converted to a string.
                 */
                FromChar: function (value) {
                    return Bridge.toString(value);
                },
                /**
                 * Produces a string from an input System.DateTime.
                 *
                 * @static
                 * @protected
                 * @this System.Xml.Serialization.XmlSerializationWriter
                 * @memberof System.Xml.Serialization.XmlSerializationWriter
                 * @param   {System.DateTime}    value    A System.DateTime to translate to a string.
                 * @return  {string}                      A string representation of the System.DateTime that shows the date and time.
                 */
                FromDateTime: function (value) {
                    return INTERNAL_DateTimeHelpers.FromDateTime(value);
                },
                /**
                 * @static
                 * @protected
                 * @this System.Xml.Serialization.XmlSerializationWriter
                 * @memberof System.Xml.Serialization.XmlSerializationWriter
                 * @param   {Array.<number>}    value
                 * @return  {Array.<number>}
                 */
                FromByteArrayBase64: function (value) {
                    return value;
                }
            }
        },
        fields: {
            _xmlWriter: null
        },
        ctors: {
            /**
             * Initializes a new instance of the System.Xml.Serialization.XmlSerializationWriter
             class.
             *
             * @instance
             * @protected
             * @this System.Xml.Serialization.XmlSerializationWriter
             * @memberof System.Xml.Serialization.XmlSerializationWriter
             * @return  {void}
             */
            ctor: function () {
                this.$initialize();
                System.Xml.Serialization.XmlSerializationGeneratedCode.ctor.call(this);
            }
        },
        methods: {
            Init: function (xmlWriter, namespaces, encodingStyle, id) {
                this._xmlWriter = xmlWriter;




            },
            /**
             * Creates an System.InvalidOperationException that indicates that a type being
             serialized is not being used in a valid manner or is unexpectedly encountered.
             *
             * @instance
             * @protected
             * @this System.Xml.Serialization.XmlSerializationWriter
             * @memberof System.Xml.Serialization.XmlSerializationWriter
             * @param   {System.Object}       o    The object whose type cannot be serialized.
             * @return  {System.Exception}         The newly created exception.
             */
            CreateUnknownTypeException: function (o) {
                return this.CreateUnknownTypeException$1(Bridge.getType(o));
            },
            /**
             * Creates an System.InvalidOperationException that indicates that a type being
             serialized is not being used in a valid manner or is unexpectedly encountered.
             *
             * @instance
             * @protected
             * @this System.Xml.Serialization.XmlSerializationWriter
             * @memberof System.Xml.Serialization.XmlSerializationWriter
             * @param   {System.Type}         type    The type that cannot be serialized.
             * @return  {System.Exception}            The newly created exception.
             */
            CreateUnknownTypeException$1: function (type) {
                return new System.InvalidOperationException.$ctor1("Unknown type for serialization: " + (Bridge.Reflection.getTypeFullName(type) || ""));
            },
            /**
             * Initializes object references only while serializing a SOAP-encoded SOAP
             message.
             *
             * @instance
             * @protected
             * @this System.Xml.Serialization.XmlSerializationWriter
             * @memberof System.Xml.Serialization.XmlSerializationWriter
             * @return  {void}
             */
            TopLevelElement: function () { },
            /**
             * Writes an XML element with a specified value in its body.
             *
             * @instance
             * @protected
             * @this System.Xml.Serialization.XmlSerializationWriter
             * @memberof System.Xml.Serialization.XmlSerializationWriter
             * @param   {string}    localName    The local name of the XML element.
             * @param   {string}    ns           The namespace of the XML element.
             * @param   {string}    value        The text value of the XML element.
             * @return  {void}
             */
            WriteElementString: function (localName, ns, value) {
                this._xmlWriter.WriteStartElement(localName);
                if (value != null) {
                    this._xmlWriter.WriteString(INTERNAL_EscapeHelpers.EscapeXml(value));
                }
                this._xmlWriter.WriteEndElement();
            },
            /**
             * Writes an XML element with a specified value in its body.
             *
             * @instance
             * @protected
             * @this System.Xml.Serialization.XmlSerializationWriter
             * @memberof System.Xml.Serialization.XmlSerializationWriter
             * @param   {string}    localName    The local name of the XML element.
             * @param   {string}    ns           The namespace of the XML element.
             * @param   {string}    value        The text value of the XML element.
             * @return  {void}
             */
            WriteElementStringRaw$1: function (localName, ns, value) {
                if (value == null) {
                    return;
                }
                this._xmlWriter.WriteStartElement(localName);
                this._xmlWriter.WriteString(INTERNAL_EscapeHelpers.EscapeXml(value));
                this._xmlWriter.WriteEndElement();
            },
            /**
             * Writes an XML element with a specified value in its body.
             *
             * @instance
             * @protected
             * @this System.Xml.Serialization.XmlSerializationWriter
             * @memberof System.Xml.Serialization.XmlSerializationWriter
             * @param   {string}            localName    The local name of the XML element.
             * @param   {string}            ns           The namespace of the XML element.
             * @param   {Array.<number>}    value        The text value of the XML element.
             * @return  {void}
             */
            WriteElementStringRaw: function (localName, ns, value) {
                if (value == null) {
                    return;
                }
                this._xmlWriter.WriteStartElement(localName);
                this._xmlWriter.WriteString(INTERNAL_EscapeHelpers.EscapeXml(System.Convert.toBase64String(value, null, null, null)));
                this._xmlWriter.WriteEndElement();
            },
            /**
             * Writes an XML element whose body is empty.
             *
             * @instance
             * @protected
             * @this System.Xml.Serialization.XmlSerializationWriter
             * @memberof System.Xml.Serialization.XmlSerializationWriter
             * @param   {string}    name    The local name of the XML element to write.
             * @param   {string}    ns      The namespace of the XML element to write.
             * @return  {void}
             */
            WriteEmptyTag: function (name, ns) { },
            
            WriteEndElement: function () {
                this._xmlWriter.WriteEndElement();
            },
            
            WriteEndElement$1: function (o) {
                this.WriteEndElement();
            },
            /**
             * Writes an XML element that contains a string as the body. System.Xml.XmlWriter
             inserts an xsi:nil='true' attribute if the string's value is null.
             *
             * @instance
             * @protected
             * @this System.Xml.Serialization.XmlSerializationWriter
             * @memberof System.Xml.Serialization.XmlSerializationWriter
             * @param   {string}    name     The local name of the XML element to write.
             * @param   {string}    ns       The namespace of the XML element to write.
             * @param   {string}    value    The string to write in the body of the XML element.
             * @return  {void}
             */
            WriteNullableStringLiteral: function (name, ns, value) {
                if (value == null) {
                    this.WriteNullTagLiteral(name, ns);
                } else {
                    this.WriteElementString(name, ns, value);
                }
            },
            /**
             * Writes an XML element that contains a string as the body. System.Xml.XmlWriter
             inserts a xsi:nil='true' attribute if the string's value is null.
             *
             * @instance
             * @protected
             * @this System.Xml.Serialization.XmlSerializationWriter
             * @memberof System.Xml.Serialization.XmlSerializationWriter
             * @param   {string}    name     The local name of the XML element to write.
             * @param   {string}    ns       The namespace of the XML element to write.
             * @param   {string}    value    The string to write in the body of the XML element.
             * @return  {void}
             */
            WriteNullableStringLiteralRaw: function (name, ns, value) {
                if (value == null) {
                    this.WriteNullTagLiteral(name, ns);
                } else {
                    this.WriteElementStringRaw$1(name, ns, value);
                }
            },
            /**
             * Writes an XML element with an xsi:nil='true' attribute.
             *
             * @instance
             * @protected
             * @this System.Xml.Serialization.XmlSerializationWriter
             * @memberof System.Xml.Serialization.XmlSerializationWriter
             * @param   {string}    name    The local name of the XML element to write.
             * @param   {string}    ns      The namespace of the XML element to write.
             * @return  {void}
             */
            WriteNullTagLiteral: function (name, ns) {
                if (name == null || name.length === 0) {
                    return;
                }
                this.WriteStartElement$1(name, ns, null, false, null);
                this._xmlWriter.WriteAttributeString("xmlns:i", "http://www.w3.org/2001/XMLSchema-instance");
                this._xmlWriter.WriteAttributeString("i:nil", "true");
                this._xmlWriter.WriteEndElement();
            },
            /**
             * Writes the XML declaration if the writer is positioned at the start of an
             XML document.
             *
             * @instance
             * @protected
             * @this System.Xml.Serialization.XmlSerializationWriter
             * @memberof System.Xml.Serialization.XmlSerializationWriter
             * @return  {void}
             */
            WriteStartDocument: function () {
                this._xmlWriter.WriteStartDocument();
            },
            /**
             * Writes an opening element tag, including any attributes.
             *
             * @instance
             * @protected
             * @this System.Xml.Serialization.XmlSerializationWriter
             * @memberof System.Xml.Serialization.XmlSerializationWriter
             * @param   {string}           name             The local name of the XML element to write.
             * @param   {string}           ns               The namespace of the XML element to write.
             * @param   {System.Object}    o                The object being serialized as an XML element.
             * @param   {boolean}          writePrefixed    true to write the element name with a prefix if none is available for the
             specified namespace; otherwise, false.
             * @return  {void}
             */
            WriteStartElement: function (name, ns, o, writePrefixed) {
                this.WriteStartElement$1(name, ns, o, writePrefixed, null);
            },
            /**
             * Writes an opening element tag, including any attributes.
             *
             * @instance
             * @protected
             * @this System.Xml.Serialization.XmlSerializationWriter
             * @memberof System.Xml.Serialization.XmlSerializationWriter
             * @param   {string}                                              name             The local name of the XML element to write.
             * @param   {string}                                              ns               The namespace of the XML element to write.
             * @param   {System.Object}                                       o                The object being serialized as an XML element.
             * @param   {boolean}                                             writePrefixed    true to write the element name with a prefix if none is available for the
             specified namespace; otherwise, false.
             * @param   {System.Xml.Serialization.XmlSerializerNamespaces}    xmlns            An instance of the System.Xml.Serialization.XmlSerializerNamespaces class
             that contains prefix and namespace pairs to be used in the generated XML.
             * @return  {void}
             */
            WriteStartElement$1: function (name, ns, o, writePrefixed, xmlns) {
                if (System.String.isNullOrWhiteSpace(ns)) {

                    this._xmlWriter.WriteStartElement(name);
                } else {
                    this._xmlWriter.WriteStartElement$1(name, ns);
                }
            },
            /**
             * Writes an xsi:type attribute for an XML element that is being serialized
             into a document.
             *
             * @instance
             * @protected
             * @this System.Xml.Serialization.XmlSerializationWriter
             * @memberof System.Xml.Serialization.XmlSerializationWriter
             * @param   {string}    name    The local name of an XML Schema data type.
             * @param   {string}    ns      The namespace of an XML Schema data type.
             * @return  {void}
             */
            WriteXsiType: function (name, ns) {
                throw new System.NotImplementedException.ctor();
            },
            /**
             * Creates an System.InvalidOperationException for an invalid enumeration value.
             *
             * @instance
             * @protected
             * @this System.Xml.Serialization.XmlSerializationWriter
             * @memberof System.Xml.Serialization.XmlSerializationWriter
             * @param   {System.Object}       value       An object that represents the invalid enumeration.
             * @param   {string}              typeName    The XML type name.
             * @return  {System.Exception}                The newly created exception.
             */
            CreateInvalidEnumValueException: function (value, typeName) {
                return new System.InvalidOperationException.$ctor1(System.String.concat("Invalid enum value '", value) + "' for type '" + (typeName || "") + "'.");
            }
        }
    });

    Bridge.define("TypeScriptDefinitionsSupport.JSObject", {
        inherits: [TypeScriptDefinitionsSupport.IJSObject],
        statics: {
            fields: {
                Undefined: null,
                Null: null
            },
            ctors: {
                init: function () {
                    this.Undefined = new TypeScriptDefinitionsSupport.JSObject.$ctor1(undefined);
                    this.Null = new TypeScriptDefinitionsSupport.JSObject.$ctor1(null);
                }
            },
            methods: {
                FromJavaScriptInstance: function (T, jsInstance) {
                    var t = Bridge.createInstance(T);
                    t.TypeScriptDefinitionsSupport$IJSObject$UnderlyingJSInstance = jsInstance;
                    return t;
                },
                CreateFrom: function (o) {
                    if (Bridge.is(o, TypeScriptDefinitionsSupport.JSObject)) {
                        return Bridge.cast(o, TypeScriptDefinitionsSupport.JSObject);
                    } else {
                        if (Bridge.is(o, TypeScriptDefinitionsSupport.IJSObject)) {
                            return new TypeScriptDefinitionsSupport.JSObject.$ctor1(Bridge.cast(o, TypeScriptDefinitionsSupport.IJSObject).TypeScriptDefinitionsSupport$IJSObject$UnderlyingJSInstance);
                        } else {
                            return new TypeScriptDefinitionsSupport.JSObject.$ctor1(o);
                        }
                    }
                },
                Helper_IsBuiltInType: function (T) {
                    return Bridge.referenceEquals(T, System.Int32) || Bridge.referenceEquals(T, System.Single) || Bridge.referenceEquals(T, System.Double) || Bridge.referenceEquals(T, System.String) || Bridge.referenceEquals(T, System.Boolean) || Bridge.referenceEquals(T, System.UInt32) || Bridge.referenceEquals(T, System.Int16) || Bridge.referenceEquals(T, System.UInt16) || Bridge.referenceEquals(T, System.Int64) || Bridge.referenceEquals(T, System.UInt64) || Bridge.referenceEquals(T, System.Decimal) || Bridge.referenceEquals(T, System.Byte) || Bridge.referenceEquals(T, System.SByte);
                },
                Helper_ConvertTo: function (T, jsObj) {
                    if (Bridge.referenceEquals(T, System.Int32)) {
                        return Bridge.cast(Bridge.unbox(Bridge.box(System.Convert.toInt32(jsObj), System.Int32), T), T);
                    } else {
                        if (Bridge.referenceEquals(T, System.Single)) {
                            return Bridge.cast(Bridge.unbox(Bridge.box(System.Convert.toSingle(jsObj), System.Single, System.Single.format, System.Single.getHashCode), T), T);
                        } else {
                            if (Bridge.referenceEquals(T, System.Double)) {
                                return Bridge.cast(Bridge.unbox(Bridge.box(System.Convert.toDouble(jsObj), System.Double, System.Double.format, System.Double.getHashCode), T), T);
                            } else {
                                if (Bridge.referenceEquals(T, System.String)) {
                                    return Bridge.cast(Bridge.unbox(System.Convert.toString(jsObj), T), T);
                                } else {
                                    if (Bridge.referenceEquals(T, System.Boolean)) {
                                        return Bridge.cast(Bridge.unbox(Bridge.box(System.Convert.toBoolean(jsObj), System.Boolean, System.Boolean.toString), T), T);
                                    } else {
                                        if (Bridge.referenceEquals(T, System.UInt32)) {
                                            return Bridge.cast(Bridge.unbox(Bridge.box(System.Convert.toUInt32(jsObj), System.UInt32), T), T);
                                        } else {
                                            if (Bridge.referenceEquals(T, System.Int16)) {
                                                return Bridge.cast(Bridge.unbox(Bridge.box(System.Convert.toInt16(jsObj), System.Int16), T), T);
                                            } else {
                                                if (Bridge.referenceEquals(T, System.UInt16)) {
                                                    return Bridge.cast(Bridge.unbox(Bridge.box(System.Convert.toUInt16(jsObj), System.UInt16), T), T);
                                                } else {
                                                    if (Bridge.referenceEquals(T, System.Int64)) {
                                                        return Bridge.cast(Bridge.unbox(System.Convert.toInt64(jsObj), T), T);
                                                    } else {
                                                        if (Bridge.referenceEquals(T, System.UInt64)) {
                                                            return Bridge.cast(Bridge.unbox(System.Convert.toUInt64(jsObj), T), T);
                                                        } else {
                                                            if (Bridge.referenceEquals(T, System.Decimal)) {
                                                                return Bridge.cast(Bridge.unbox(System.Convert.toDecimal(jsObj), T), T);
                                                            } else {
                                                                if (Bridge.referenceEquals(T, System.Byte)) {
                                                                    return Bridge.cast(Bridge.unbox(Bridge.box(System.Convert.toByte(jsObj), System.Byte), T), T);
                                                                } else {
                                                                    if (Bridge.referenceEquals(T, System.SByte)) {
                                                                        return Bridge.cast(Bridge.unbox(Bridge.box(System.Convert.toSByte(jsObj), System.SByte), T), T);
                                                                    } else {
                                                                        throw new System.Exception("The type '" + (Bridge.Reflection.getTypeName(T) || "") + "' is not a built-in type");
                                                                    }
                                                                }
                                                            }
                                                        }
                                                    }
                                                }
                                            }
                                        }
                                    }
                                }
                            }
                        }
                    }
                },
                op_Implicit: function (b) {
                    return new TypeScriptDefinitionsSupport.JSObject.$ctor1(Bridge.box(b, System.Boolean, System.Boolean.toString));
                },
                op_Implicit$1: function (b) {
                    return new TypeScriptDefinitionsSupport.JSObject.$ctor1(Bridge.box(b, System.Byte));
                },
                op_Implicit$8: function (s) {
                    return new TypeScriptDefinitionsSupport.JSObject.$ctor1(Bridge.box(s, System.SByte));
                },
                op_Implicit$2: function (c) {
                    return new TypeScriptDefinitionsSupport.JSObject.$ctor1(Bridge.box(c, System.Char, String.fromCharCode, System.Char.getHashCode));
                },
                op_Implicit$3: function (d) {
                    return new TypeScriptDefinitionsSupport.JSObject.$ctor1(d);
                },
                op_Implicit$4: function (d) {
                    return new TypeScriptDefinitionsSupport.JSObject.$ctor1(Bridge.box(d, System.Double, System.Double.format, System.Double.getHashCode));
                },
                op_Implicit$9: function (f) {
                    return new TypeScriptDefinitionsSupport.JSObject.$ctor1(Bridge.box(f, System.Single, System.Single.format, System.Single.getHashCode));
                },
                op_Implicit$6: function (i) {
                    return new TypeScriptDefinitionsSupport.JSObject.$ctor1(Bridge.box(i, System.Int32));
                },
                op_Implicit$12: function (u) {
                    return new TypeScriptDefinitionsSupport.JSObject.$ctor1(Bridge.box(u, System.UInt32));
                },
                op_Implicit$7: function (l) {
                    return new TypeScriptDefinitionsSupport.JSObject.$ctor1(l);
                },
                op_Implicit$13: function (u) {
                    return new TypeScriptDefinitionsSupport.JSObject.$ctor1(u);
                },
                op_Implicit$5: function (s) {
                    return new TypeScriptDefinitionsSupport.JSObject.$ctor1(Bridge.box(s, System.Int16));
                },
                op_Implicit$11: function (u) {
                    return new TypeScriptDefinitionsSupport.JSObject.$ctor1(Bridge.box(u, System.UInt16));
                },
                op_Implicit$10: function (s) {
                    return new TypeScriptDefinitionsSupport.JSObject.$ctor1(s);
                }
            }
        },
        fields: {
            UnderlyingJSInstance: null
        },
        alias: ["UnderlyingJSInstance", "TypeScriptDefinitionsSupport$IJSObject$UnderlyingJSInstance"],
        ctors: {
            ctor: function () {
                this.$initialize();
                this.UnderlyingJSInstance = new Object();
            },
            $ctor1: function (jsObj) {
                this.$initialize();
                this.UnderlyingJSInstance = jsObj;
            }
        },
        methods: {
            equals: function (obj) {
                if (Bridge.is(obj, TypeScriptDefinitionsSupport.IJSObject)) {
                    return Bridge.referenceEquals(Bridge.cast(obj, TypeScriptDefinitionsSupport.IJSObject).TypeScriptDefinitionsSupport$IJSObject$UnderlyingJSInstance, this.UnderlyingJSInstance);
                }
                return false;
            }
        }
    });

    /**
     * Convert between boolean and visibility
     *
     * @public
     * @class Windows.UI.Xaml.Controls.BooleanToVisibilityConverter
     * @implements  Windows.UI.Xaml.Data.IValueConverter
     */
    Bridge.define("Windows.UI.Xaml.Controls.BooleanToVisibilityConverter", {
        inherits: [Windows.UI.Xaml.Data.IValueConverter],
        alias: [
            "Convert", "Windows$UI$Xaml$Data$IValueConverter$Convert",
            "ConvertBack", "Windows$UI$Xaml$Data$IValueConverter$ConvertBack"
        ],
        methods: {
            /**
             * Convert bool or Nullable&lt;bool&gt; to Visibility
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Controls.BooleanToVisibilityConverter
             * @memberof Windows.UI.Xaml.Controls.BooleanToVisibilityConverter
             * @param   {System.Object}    value         
             * @param   {System.Type}      targetType    
             * @param   {System.Object}    parameter     
             * @param   {string}           language
             * @return  {System.Object}
             */
            Convert: function (value, targetType, parameter, language) {
                var booleanValue = false;
                if (Bridge.is(value, System.Boolean)) {
                    booleanValue = System.Nullable.getValue(Bridge.cast(Bridge.unbox(value, System.Boolean), System.Boolean));
                } else if (Bridge.is(value, System.Boolean)) {
                    var nullableValue = Bridge.cast(Bridge.unbox(value, System.Boolean), System.Boolean, true);
                    booleanValue = System.Nullable.hasValue(nullableValue) ? System.Nullable.getValue(nullableValue) : false;
                }
                return Bridge.box((booleanValue) ? Windows.UI.Xaml.Visibility.Visible : Windows.UI.Xaml.Visibility.Collapsed, Windows.UI.Xaml.Visibility, System.Enum.toStringFn(Windows.UI.Xaml.Visibility));
            },
            /**
             * Convert Visibility to boolean
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Controls.BooleanToVisibilityConverter
             * @memberof Windows.UI.Xaml.Controls.BooleanToVisibilityConverter
             * @param   {System.Object}    value         
             * @param   {System.Type}      targetType    
             * @param   {System.Object}    parameter     
             * @param   {string}           language
             * @return  {System.Object}
             */
            ConvertBack: function (value, targetType, parameter, language) {
                if (Bridge.is(value, System.Int32)) {
                    return Bridge.box(System.Nullable.getValue(Bridge.cast(Bridge.unbox(value, Windows.UI.Xaml.Visibility), System.Int32)) === Windows.UI.Xaml.Visibility.Visible, System.Boolean, System.Boolean.toString);
                } else {
                    return Bridge.box(false, System.Boolean, System.Boolean.toString);
                }
            }
        }
    });

    /**
     * Defines column-specific properties that apply to Grid objects.
     *
     * @public
     * @class Windows.UI.Xaml.Controls.ColumnDefinition
     * @augments Windows.UI.Xaml.DependencyObject
     */
    Bridge.define("Windows.UI.Xaml.Controls.ColumnDefinition", {
        inherits: [Windows.UI.Xaml.DependencyObject],
        statics: {
            fields: {
                /**
                 * Identifies the MaxWidth dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Controls.ColumnDefinition
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                MaxWidthProperty: null,
                /**
                 * Identifies the MinWidth dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Controls.ColumnDefinition
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                MinWidthProperty: null,
                /**
                 * Identifies the Width dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Controls.ColumnDefinition
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                WidthProperty: null,
                VisibilityProperty: null
            },
            ctors: {
                init: function () {
                    var $t;
                    this.MaxWidthProperty = Windows.UI.Xaml.DependencyProperty.Register("MaxWidth", System.Double, Windows.UI.Xaml.Controls.ColumnDefinition, ($t = new Windows.UI.Xaml.PropertyMetadata.$ctor2(Bridge.box(Number.POSITIVE_INFINITY, System.Double, System.Double.format, System.Double.getHashCode), Windows.UI.Xaml.Controls.ColumnDefinition.MaxWidth_Changed), $t.CallPropertyChangedWhenLoadedIntoVisualTree = Windows.UI.Xaml.WhenToCallPropertyChangedEnum.IfPropertyIsSet, $t));
                    this.MinWidthProperty = Windows.UI.Xaml.DependencyProperty.Register("MinWidth", System.Double, Windows.UI.Xaml.Controls.ColumnDefinition, ($t = new Windows.UI.Xaml.PropertyMetadata.$ctor2(Bridge.box(Number.POSITIVE_INFINITY, System.Double, System.Double.format, System.Double.getHashCode), Windows.UI.Xaml.Controls.ColumnDefinition.MinWidth_Changed), $t.CallPropertyChangedWhenLoadedIntoVisualTree = Windows.UI.Xaml.WhenToCallPropertyChangedEnum.IfPropertyIsSet, $t));
                    this.WidthProperty = Windows.UI.Xaml.DependencyProperty.Register("Width", Windows.UI.Xaml.GridLength, Windows.UI.Xaml.Controls.ColumnDefinition, ($t = new Windows.UI.Xaml.PropertyMetadata.$ctor2(new Windows.UI.Xaml.GridLength.$ctor2(1.0, Windows.UI.Xaml.GridUnitType.Star).$clone(), Windows.UI.Xaml.Controls.ColumnDefinition.Width_Changed), $t.CallPropertyChangedWhenLoadedIntoVisualTree = Windows.UI.Xaml.WhenToCallPropertyChangedEnum.IfPropertyIsSet, $t));
                    this.VisibilityProperty = Windows.UI.Xaml.DependencyProperty.Register("Visibility", Windows.UI.Xaml.Visibility, Windows.UI.Xaml.Controls.ColumnDefinition, new Windows.UI.Xaml.PropertyMetadata.$ctor2(Bridge.box(Windows.UI.Xaml.Visibility.Visible, Windows.UI.Xaml.Visibility, System.Enum.toStringFn(Windows.UI.Xaml.Visibility)), Windows.UI.Xaml.Controls.ColumnDefinition.Visibility_Changed));
                }
            },
            methods: {
                MaxWidth_Changed: function (d, e) {
                    var rowDefinition = Bridge.cast(d, Windows.UI.Xaml.Controls.ColumnDefinition);
                    var newValue = System.Nullable.getValue(Bridge.cast(Bridge.unbox(e.NewValue, System.Double), System.Double));
                },
                MinWidth_Changed: function (d, e) {
                    var rowDefinition = Bridge.cast(d, Windows.UI.Xaml.Controls.ColumnDefinition);
                    var newValue = System.Nullable.getValue(Bridge.cast(Bridge.unbox(e.NewValue, System.Double), System.Double));
                },
                Width_Changed: function (d, e) {
                    var columnDefinition = Bridge.cast(d, Windows.UI.Xaml.Controls.ColumnDefinition);
                    var grid = columnDefinition.Parent;

                    if (grid != null && CSHTML5.Internal.INTERNAL_VisualTreeManager.IsElementInVisualTree(grid)) {
                        if (document.isGridSupported) {

                            Windows.UI.Xaml.Controls.Grid_InternalHelpers.RefreshAllColumnsWidth_CSSVersion(grid);
                        } else {

                            var isStar = (e.OldValue != null && System.Nullable.getValue(Bridge.cast(Bridge.unbox(e.OldValue, Windows.UI.Xaml.GridLength), Windows.UI.Xaml.GridLength)).IsStar) || (e.NewValue != null && System.Nullable.getValue(Bridge.cast(Bridge.unbox(e.NewValue, Windows.UI.Xaml.GridLength), Windows.UI.Xaml.GridLength)).IsStar);
                            if (isStar) {
                                Windows.UI.Xaml.Controls.Grid_InternalHelpers.RefreshAllColumnsWidth_NonCSSVersion(grid);
                            } else {
                                if (grid._columnDefinitionsOrNull != null) {
                                    var columnIndex = grid._columnDefinitionsOrNull.indexOf(columnDefinition);
                                    var isTheOnlyColumn = grid._columnDefinitionsOrNull == null || grid._columnDefinitionsOrNull.Count < 2;
                                    Windows.UI.Xaml.Controls.Grid_InternalHelpers.RefreshColumnWidth_NonCSSVersion(grid, columnIndex, isTheOnlyColumn);
                                }
                            }
                        }
                    }
                },
                Visibility_Changed: function (d, e) {
                    var columnDefinition = Bridge.cast(d, Windows.UI.Xaml.Controls.ColumnDefinition);
                    if (columnDefinition.Parent != null) {
                        Windows.UI.Xaml.Controls.Grid_InternalHelpers.RefreshColumnVisibility(columnDefinition.Parent, columnDefinition, System.Nullable.getValue(Bridge.cast(Bridge.unbox(e.NewValue, Windows.UI.Xaml.Visibility), System.Int32)));
                    }
                }
            }
        },
        fields: {
            Parent: null
        },
        props: {
            /**
             * Gets or sets a value that represents the maximum width of a ColumnDefinition. Returns a Double that represents the maximum width in pixels. The default is PositiveInfinity.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.ColumnDefinition
             * @function MaxWidth
             * @type number
             */
            MaxWidth: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.Controls.ColumnDefinition.MaxWidthProperty), System.Double), System.Double));
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Controls.ColumnDefinition.MaxWidthProperty, Bridge.box(value, System.Double, System.Double.format, System.Double.getHashCode));
                }
            },
            /**
             * Gets or sets a value that represents the minimum width of a ColumnDefinition. Returns a Double that represents the minimum width in pixels. The default is 0.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.ColumnDefinition
             * @function MinWidth
             * @type number
             */
            MinWidth: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.Controls.ColumnDefinition.MinWidthProperty), System.Double), System.Double));
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Controls.ColumnDefinition.MinWidthProperty, Bridge.box(value, System.Double, System.Double.format, System.Double.getHashCode));
                }
            },
            /**
             * Gets the calculated width of a ColumnDefinition element, or sets the GridLength value of a row that is defined by the ColumnDefinition.
             Returns the GridLength that represents the width of the row. The default value is 1.0
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.ColumnDefinition
             * @function Width
             * @type Windows.UI.Xaml.GridLength
             */
            Width: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.Controls.ColumnDefinition.WidthProperty), Windows.UI.Xaml.GridLength), Windows.UI.Xaml.GridLength));
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Controls.ColumnDefinition.WidthProperty, value.$clone());
                }
            },
            /**
             * Gets the rendered width of a FrameworkElement. The FrameworkElement must be in the visual tree,
             otherwise this property will return double.NaN.
             *
             * @instance
             * @public
             * @readonly
             * @memberof Windows.UI.Xaml.Controls.ColumnDefinition
             * @function ActualWidth
             * @type number
             */
            ActualWidth: {
                get: function () {
                    return this.Parent.GetColumnActualWidth(this);
                }
            },
            /**
             * // Gets the rendered height of a FrameworkElement. The FrameworkElement must be in the visual tree,
            // otherwise this property will return double.NaN.
            //
             *
             * @instance
             * @memberof Windows.UI.Xaml.Controls.ColumnDefinition
             * @function Visibility
             * @type Windows.UI.Xaml.Visibility
             */
            Visibility: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.Controls.ColumnDefinition.VisibilityProperty), Windows.UI.Xaml.Visibility), System.Int32));
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Controls.ColumnDefinition.VisibilityProperty, Bridge.box(value, Windows.UI.Xaml.Visibility, System.Enum.toStringFn(Windows.UI.Xaml.Visibility)));
                }
            }
        },
        methods: {
            /**
             * Returns a copy of the current ColumnDefinition.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Controls.ColumnDefinition
             * @memberof Windows.UI.Xaml.Controls.ColumnDefinition
             * @return  {Windows.UI.Xaml.Controls.ColumnDefinition}        A copy of the current ColumnDefinition.
             */
            Clone: function () {
                var $t;
                return ($t = new Windows.UI.Xaml.Controls.ColumnDefinition(), $t.MinWidth = this.MinWidth, $t.MaxWidth = this.MaxWidth, $t.Width = this.Width.Clone(), $t);
            }
        }
    });

    /**
     * Provides data for the context menu event.
     *
     * @public
     * @class Windows.UI.Xaml.Controls.ContextMenuEventArgs
     * @augments Windows.UI.Xaml.RoutedEventArgs
     */
    Bridge.define("Windows.UI.Xaml.Controls.ContextMenuEventArgs", {
        inherits: [Windows.UI.Xaml.RoutedEventArgs],
        fields: {
            _pointerLeft: 0,
            _pointerTop: 0
        },
        props: {
            /**
             * Gets the horizontal position of the mouse.
             *
             * @instance
             * @public
             * @readonly
             * @memberof Windows.UI.Xaml.Controls.ContextMenuEventArgs
             * @function PointerLeft
             * @type number
             */
            PointerLeft: {
                get: function () {
                    return this._pointerLeft;
                }
            },
            /**
             * Gets the vertical position of the mouse.
             *
             * @instance
             * @public
             * @readonly
             * @memberof Windows.UI.Xaml.Controls.ContextMenuEventArgs
             * @function PointerTop
             * @type number
             */
            PointerTop: {
                get: function () {
                    return this._pointerTop;
                }
            }
        },
        ctors: {
            ctor: function (pointerLeft, pointerTop) {
                this.$initialize();
                Windows.UI.Xaml.RoutedEventArgs.ctor.call(this);
                this._pointerLeft = pointerLeft;
                this._pointerTop = pointerTop;
            }
        }
    });

    /**
     * Creates an element tree of elements.
     *
     * @abstract
     * @public
     * @class Windows.UI.Xaml.FrameworkTemplate
     * @augments Windows.UI.Xaml.DependencyObject
     */
    Bridge.define("Windows.UI.Xaml.FrameworkTemplate", {
        inherits: [Windows.UI.Xaml.DependencyObject],
        statics: {
            fields: {
                /**
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.FrameworkTemplate
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                ContentPropertyUsefulOnlyDuringTheCompilationProperty: null
            },
            ctors: {
                init: function () {
                    this.ContentPropertyUsefulOnlyDuringTheCompilationProperty = Windows.UI.Xaml.DependencyProperty.Register("ContentPropertyUsefulOnlyDuringTheCompilation", Windows.UI.Xaml.UIElement, Windows.UI.Xaml.FrameworkTemplate, new Windows.UI.Xaml.PropertyMetadata.$ctor4(null));
                }
            }
        },
        fields: {
            _methodToInstantiateFrameworkTemplate: null,
            _isSealed: false
        },
        props: {
            /**
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.FrameworkTemplate
             * @function ContentPropertyUsefulOnlyDuringTheCompilation
             * @type Windows.UI.Xaml.UIElement
             */
            ContentPropertyUsefulOnlyDuringTheCompilation: {
                get: function () {
                    return Bridge.cast(this.GetValue(Windows.UI.Xaml.FrameworkTemplate.ContentPropertyUsefulOnlyDuringTheCompilationProperty), Windows.UI.Xaml.UIElement);
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.FrameworkTemplate.ContentPropertyUsefulOnlyDuringTheCompilationProperty, value);
                }
            }
        },
        ctors: {
            init: function () {
                this._isSealed = false;
            },
            ctor: function () {
                this.$initialize();
                Windows.UI.Xaml.DependencyObject.ctor.call(this);
                this.CanBeInheritanceContext = false;
            }
        },
        methods: {
            /**
             * Creates an instance of the Template. Intented to be called for templates that have no owner, such as DataTemplates (not ControlTemplates).
             *
             * @instance
             * @this Windows.UI.Xaml.FrameworkTemplate
             * @memberof Windows.UI.Xaml.FrameworkTemplate
             * @return  {Windows.UI.Xaml.FrameworkElement}        The instantiated template.
             */
            INTERNAL_InstantiateFrameworkTemplate: function () {
                return this.INTERNAL_InstantiateFrameworkTemplate$1(null);
            },
            /**
             * Creates an instance of the Template.
             *
             * @instance
             * @this Windows.UI.Xaml.FrameworkTemplate
             * @memberof Windows.UI.Xaml.FrameworkTemplate
             * @param   {Windows.UI.Xaml.Controls.Control}    templateOwner    Owner of the template.
             Should be null if the template has no owner (for example,
             DataTemplate)
             * @return  {Windows.UI.Xaml.FrameworkElement}
             */
            INTERNAL_InstantiateFrameworkTemplate$1: function (templateOwner) {
                if (!Bridge.staticEquals(this._methodToInstantiateFrameworkTemplate, null)) {
                    var templateInstance = this._methodToInstantiateFrameworkTemplate(templateOwner);
                    return templateInstance.TemplateContent;
                } else {
                    return null;
                }
            },
            /**
             * Creates an instance of the Template, attaches it to the Visual Tree, and calls "OnApplyTemplate". This method is intented to be called for ControlTemplates only (not DataTemplates).
             *
             * @instance
             * @this Windows.UI.Xaml.FrameworkTemplate
             * @memberof Windows.UI.Xaml.FrameworkTemplate
             * @param   {Windows.UI.Xaml.Controls.Control}    templateOwner    The owner of the template is the control to which the template is applied.
             * @return  {Windows.UI.Xaml.FrameworkElement}                     The instantiated control template.
             */
            INTERNAL_InstantiateAndAttachControlTemplate: function (templateOwner) {
                if (templateOwner == null) {
                    throw new System.ArgumentNullException.$ctor1("templateOwner");
                }

                templateOwner.INTERNAL_IsTemplated = true;

                if (!Bridge.staticEquals(this._methodToInstantiateFrameworkTemplate, null)) {

                    var templateInstance = this._methodToInstantiateFrameworkTemplate(templateOwner);

                    CSHTML5.Internal.INTERNAL_VisualTreeManager.AttachVisualChildIfNotAlreadyAttached(templateInstance.TemplateContent, templateOwner);

                    if (templateOwner != null) {
                        templateOwner.RaiseOnApplyTemplate();
                    }
                    return templateInstance.TemplateContent;
                } else {
                    return null;
                }
            },
            /**
             * Sets the method that will create the tree of elements.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.FrameworkTemplate
             * @memberof Windows.UI.Xaml.FrameworkTemplate
             * @param   {System.Func}    methodToInstantiateFrameworkTemplate    The method that will create the tree of elements.
             * @return  {void}
             */
            SetMethodToInstantiateFrameworkTemplate: function (methodToInstantiateFrameworkTemplate) {
                if (this._isSealed) {
                    throw new System.InvalidOperationException.$ctor1("Cannot modify a sealed " + (Bridge.Reflection.getTypeName(Bridge.getType(this)) || "") + ". Please create a new one instead.");
                }
                this._methodToInstantiateFrameworkTemplate = methodToInstantiateFrameworkTemplate;
            },
            /**
             * Locks the template so it cannot be changed.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.FrameworkTemplate
             * @memberof Windows.UI.Xaml.FrameworkTemplate
             * @return  {void}
             */
            Seal: function () {
                this._isSealed = true;
            },
            /**
             * Gets a value that indicates whether this object is in an immutable state
             so it cannot be changed.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.FrameworkTemplate
             * @memberof Windows.UI.Xaml.FrameworkTemplate
             * @return  {boolean}        true if this object is in an immutable state; otherwise, false.
             */
            IsSealed: function () {
                return this._isSealed;
            }
        }
    });

    /**
     * Represents a System.Windows.Controls.DataGrid column.
     *
     * @abstract
     * @public
     * @class Windows.UI.Xaml.Controls.DataGridColumn
     * @augments Windows.UI.Xaml.DependencyObject
     */
    Bridge.define("Windows.UI.Xaml.Controls.DataGridColumn", {
        inherits: [Windows.UI.Xaml.DependencyObject],
        statics: {
            fields: {
                HeaderProperty: null,
                /**
                 * Identifies the System.Windows.Controls.DataGridColumn.HeaderStyle dependency
                 property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Controls.DataGridColumn
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                HeaderStyleProperty: null,
                /**
                 * Identifies the System.Windows.Controls.DataGridColumn.HeaderTemplate dependency
                 property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Controls.DataGridColumn
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                HeaderTemplateProperty: null,
                /**
                 * Identifies the System.Windows.Controls.DataGridColumn.CellStyle dependency
                 property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Controls.DataGridColumn
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                CellStyleProperty: null,
                /**
                 * Identifies the System.Windows.Controls.DataGridColumn.IsAutoGenerated dependency
                 property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Controls.DataGridColumn
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                IsAutoGeneratedProperty: null,
                /**
                 * Identifies the System.Windows.Controls.DataGridColumn.IsAutoGenerated dependency
                 property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Controls.DataGridColumn
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                IsReadOnlyProperty: null,
                /**
                 * Identifies the System.Windows.Controls.DataGridColumn.Width dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Controls.DataGridColumn
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                WidthProperty: null,
                VisibilityProperty: null
            },
            ctors: {
                init: function () {
                    var $t;
                    this.HeaderProperty = Windows.UI.Xaml.DependencyProperty.Register("Header", System.Object, Windows.UI.Xaml.Controls.DataGridColumn, ($t = new Windows.UI.Xaml.PropertyMetadata.$ctor2(null, Windows.UI.Xaml.Controls.DataGridColumn.Header_Changed), $t.CallPropertyChangedWhenLoadedIntoVisualTree = Windows.UI.Xaml.WhenToCallPropertyChangedEnum.IfPropertyIsSet, $t));
                    this.HeaderStyleProperty = Windows.UI.Xaml.DependencyProperty.Register("HeaderStyle", Windows.UI.Xaml.Style, Windows.UI.Xaml.Controls.DataGridColumn, ($t = new Windows.UI.Xaml.PropertyMetadata.$ctor2(null, Windows.UI.Xaml.Controls.DataGridColumn.HeaderStyle_Changed), $t.CallPropertyChangedWhenLoadedIntoVisualTree = Windows.UI.Xaml.WhenToCallPropertyChangedEnum.IfPropertyIsSet, $t));
                    this.HeaderTemplateProperty = Windows.UI.Xaml.DependencyProperty.Register("HeaderTemplate", Windows.UI.Xaml.DataTemplate, Windows.UI.Xaml.Controls.DataGridColumn, ($t = new Windows.UI.Xaml.PropertyMetadata.$ctor2(null, Windows.UI.Xaml.Controls.DataGridColumn.HeaderTemplate_Changed), $t.CallPropertyChangedWhenLoadedIntoVisualTree = Windows.UI.Xaml.WhenToCallPropertyChangedEnum.IfPropertyIsSet, $t));
                    this.CellStyleProperty = Windows.UI.Xaml.DependencyProperty.Register("CellStyle", Windows.UI.Xaml.Style, Windows.UI.Xaml.Controls.DataGridColumn, ($t = new Windows.UI.Xaml.PropertyMetadata.$ctor2(null, Windows.UI.Xaml.Controls.DataGridColumn.CellStyle_Changed), $t.CallPropertyChangedWhenLoadedIntoVisualTree = Windows.UI.Xaml.WhenToCallPropertyChangedEnum.IfPropertyIsSet, $t));
                    this.IsAutoGeneratedProperty = Windows.UI.Xaml.DependencyProperty.Register("IsAutoGenerated", System.Boolean, Windows.UI.Xaml.Controls.DataGridColumn, new Windows.UI.Xaml.PropertyMetadata.$ctor1(Bridge.box(false, System.Boolean, System.Boolean.toString)));
                    this.IsReadOnlyProperty = Windows.UI.Xaml.DependencyProperty.Register("IsReadOnly", System.Boolean, Windows.UI.Xaml.Controls.DataGridColumn, new Windows.UI.Xaml.PropertyMetadata.$ctor1(Bridge.box(false, System.Boolean, System.Boolean.toString)));
                    this.WidthProperty = Windows.UI.Xaml.DependencyProperty.Register("Width", Windows.UI.Xaml.Controls.DataGridLength, Windows.UI.Xaml.Controls.DataGridColumn, ($t = new Windows.UI.Xaml.PropertyMetadata.$ctor2(Windows.UI.Xaml.Controls.DataGridLength.Auto.$clone(), Windows.UI.Xaml.Controls.DataGridColumn.Width_Changed), $t.CallPropertyChangedWhenLoadedIntoVisualTree = Windows.UI.Xaml.WhenToCallPropertyChangedEnum.IfPropertyIsSet, $t));
                    this.VisibilityProperty = Windows.UI.Xaml.DependencyProperty.Register("Visibility", Windows.UI.Xaml.Visibility, Windows.UI.Xaml.Controls.DataGridColumn, ($t = new Windows.UI.Xaml.PropertyMetadata.$ctor2(Bridge.box(Windows.UI.Xaml.Visibility.Visible, Windows.UI.Xaml.Visibility, System.Enum.toStringFn(Windows.UI.Xaml.Visibility)), Windows.UI.Xaml.Controls.DataGridColumn.Visibility_Changed), $t.CallPropertyChangedWhenLoadedIntoVisualTree = Windows.UI.Xaml.WhenToCallPropertyChangedEnum.IfPropertyIsSet, $t));
                }
            },
            methods: {
                Header_Changed: function (d, e) { },
                HeaderStyle_Changed: function (d, e) {
                    if (Bridge.is(d, Windows.UI.Xaml.Controls.DataGridColumn)) {
                        var column = Bridge.cast(d, Windows.UI.Xaml.Controls.DataGridColumn);
                        if (column._header != null) {
                            Bridge.cast(d, Windows.UI.Xaml.Controls.DataGridColumn)._header.Style = Bridge.cast(e.NewValue, Windows.UI.Xaml.Style);
                        }
                    }
                },
                HeaderTemplate_Changed: function (d, e) { },
                CellStyle_Changed: function (d, e) {
                    if (Bridge.is(d, Windows.UI.Xaml.Controls.DataGridColumn)) {
                        var column = Bridge.cast(d, Windows.UI.Xaml.Controls.DataGridColumn);
                    }
                },
                Width_Changed: function (d, e) {
                    if (Bridge.is(d, Windows.UI.Xaml.Controls.DataGridColumn)) {
                        var column = Bridge.cast(d, Windows.UI.Xaml.Controls.DataGridColumn);
                        Windows.UI.Xaml.Controls.DataGridColumn.UpdateGridColumnWidth(column);
                    }
                },
                UpdateGridColumnWidth: function (column) {
                    var newValue = column.Width.$clone();

                    if (column._parent != null && CSHTML5.Internal.INTERNAL_VisualTreeManager.IsElementInVisualTree(column._parent)) {
                        var newGridUnitType = new Windows.UI.Xaml.GridUnitType();
                        switch (newValue.UnitType) {
                            case Windows.UI.Xaml.Controls.DataGridLengthUnitType.Auto: 
                                newGridUnitType = Windows.UI.Xaml.GridUnitType.Auto;
                                break;
                            case Windows.UI.Xaml.Controls.DataGridLengthUnitType.Pixel: 
                                newGridUnitType = Windows.UI.Xaml.GridUnitType.Pixel;
                                break;
                            case Windows.UI.Xaml.Controls.DataGridLengthUnitType.Star: 
                                newGridUnitType = Windows.UI.Xaml.GridUnitType.Star;
                                break;
                            default: 
                                newGridUnitType = Windows.UI.Xaml.GridUnitType.Auto;
                                break;
                        }
                        column._gridColumn.Width = new Windows.UI.Xaml.GridLength.$ctor2(newValue.Value, newGridUnitType);
                    }
                },
                Visibility_Changed: function (d, e) {
                    var column = Bridge.cast(d, Windows.UI.Xaml.Controls.DataGridColumn);
                    if (column._gridColumn != null) {
                        column._gridColumn.Visibility = System.Nullable.getValue(Bridge.cast(Bridge.unbox(e.NewValue, Windows.UI.Xaml.Visibility), System.Int32));
                    }
                }
            }
        },
        fields: {
            _gridColumn: null,
            _header: null,
            _parent: null
        },
        props: {
            /**
             * Gets or sets the content of the column header.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.DataGridColumn
             * @function Header
             * @type System.Object
             */
            Header: {
                get: function () {
                    return this.GetValue(Windows.UI.Xaml.Controls.DataGridColumn.HeaderProperty);
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Controls.DataGridColumn.HeaderProperty, value);
                }
            },
            /**
             * Gets or sets the style that is used when rendering the column header.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.DataGridColumn
             * @function HeaderStyle
             * @type Windows.UI.Xaml.Style
             */
            HeaderStyle: {
                get: function () {
                    return Bridge.cast(this.GetValue(Windows.UI.Xaml.Controls.DataGridColumn.HeaderStyleProperty), Windows.UI.Xaml.Style);
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Controls.DataGridColumn.HeaderStyleProperty, value);
                }
            },
            /**
             * Gets or sets the template that defines the visual representation of the column
             header.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.DataGridColumn
             * @function HeaderTemplate
             * @type Windows.UI.Xaml.DataTemplate
             */
            HeaderTemplate: {
                get: function () {
                    return Bridge.cast(this.GetValue(Windows.UI.Xaml.Controls.DataGridColumn.HeaderTemplateProperty), Windows.UI.Xaml.DataTemplate);
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Controls.DataGridColumn.HeaderTemplateProperty, value);
                }
            },
            CellStyle: {
                get: function () {
                    return Bridge.cast(this.GetValue(Windows.UI.Xaml.Controls.DataGridColumn.CellStyleProperty), Windows.UI.Xaml.Style);
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Controls.DataGridColumn.CellStyleProperty, value);
                }
            },
            /**
             * Gets a value that indicates whether the column is auto-generated.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.DataGridColumn
             * @function IsAutoGenerated
             * @type boolean
             */
            IsAutoGenerated: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.Controls.DataGridColumn.IsAutoGeneratedProperty), System.Boolean), System.Boolean));
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Controls.DataGridColumn.IsAutoGeneratedProperty, Bridge.box(value, System.Boolean, System.Boolean.toString));
                }
            },
            IsReadOnly: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.Controls.DataGridColumn.IsReadOnlyProperty), System.Boolean), System.Boolean));
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Controls.DataGridColumn.IsReadOnlyProperty, Bridge.box(value, System.Boolean, System.Boolean.toString));
                }
            },
            /**
             * Gets or sets the column width or automatic sizing mode.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.DataGridColumn
             * @function Width
             * @type Windows.UI.Xaml.Controls.DataGridLength
             */
            Width: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.Controls.DataGridColumn.WidthProperty), Windows.UI.Xaml.Controls.DataGridLength), Windows.UI.Xaml.Controls.DataGridLength));
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Controls.DataGridColumn.WidthProperty, value.$clone());
                }
            },
            Visibility: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.Controls.DataGridColumn.VisibilityProperty), Windows.UI.Xaml.Visibility), System.Int32));
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Controls.DataGridColumn.VisibilityProperty, Bridge.box(value, Windows.UI.Xaml.Visibility, System.Enum.toStringFn(Windows.UI.Xaml.Visibility)));
                }
            }
        },
        ctors: {
            ctor: function () {
                this.$initialize();
                Windows.UI.Xaml.DependencyObject.ctor.call(this);
                this.CanBeInheritanceContext = false;
                this.IsInheritanceContextSealed = true;
            }
        },
        methods: {
            /**
             * This method allows to set the header's style without changing the DataGridColumn's HeaderStyle Property. This way, we will still be able to set a new header via DataGrid's ColumnHeaderStyle property.
             *
             * @instance
             * @this Windows.UI.Xaml.Controls.DataGridColumn
             * @memberof Windows.UI.Xaml.Controls.DataGridColumn
             * @param   {Windows.UI.Xaml.Style}    style    The new style to apply to the header
             * @return  {void}
             */
            SetHeaderStyleIfColumnsStyleNotSet: function (style) {
                if (this.Header == null) {
                    this._header.Style = style;
                }
            },
            /**
             * Retrieves the visual tree that was generated for a particular row and column.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Controls.DataGridColumn
             * @memberof Windows.UI.Xaml.Controls.DataGridColumn
             * @param   {System.Object}                       dataItem    The row that corresponds to the desired cell.
             * @return  {Windows.UI.Xaml.FrameworkElement}                The element if found, null otherwise.
             */
            GetCellContent: function (dataItem) {
                if (this._parent != null && dataItem != null) {
                    var row = Bridge.as(this._parent.ItemContainerGenerator.ContainerFromItem(dataItem), Windows.UI.Xaml.Controls.DataGridRow);
                    if (row != null) {
                        return this.GetCellContent$1(row);
                    }
                }

                return null;
            },
            /**
             * Retrieves the visual tree that was generated for a particular row and column.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Controls.DataGridColumn
             * @memberof Windows.UI.Xaml.Controls.DataGridColumn
             * @param   {Windows.UI.Xaml.Controls.DataGridRow}    dataGridRow    The row that corresponds to the desired cell.
             * @return  {Windows.UI.Xaml.FrameworkElement}                       The element if found, null otherwise.
             */
            GetCellContent$1: function (dataGridRow) {
                if (this._parent != null && dataGridRow != null) {
                    var columnIndex = this._parent.Columns.indexOf(this);
                    if (columnIndex >= 0) {
                        var cell = dataGridRow.TryGetCell(columnIndex);
                        if (cell != null) {
                            return Bridge.as(cell.Content, Windows.UI.Xaml.FrameworkElement);
                        }
                    }
                }

                return null;
            },
            Header_OnClick: function (sender, e) { },
            GetHeader: function () {
                var header = new Windows.UI.Xaml.Controls.Primitives.DataGridColumnHeader();
                header.Column = this;

                header.Content = this.Header;
                if (this.HeaderTemplate != null) {
                    header.ContentTemplate = this.HeaderTemplate;
                }
                if (this.HeaderStyle != null) {
                    header.Style = this.HeaderStyle;
                } else if (this._parent.ColumnHeaderStyle != null) {
                    header.Style = this._parent.ColumnHeaderStyle;
                }

                header.addClick(Bridge.fn.cacheBind(this, this.Header_OnClick));

                this._header = header;
                return header;
            },
            EnterEditionMode: function (dataGridCell) { },
            LeaveEditionMode: function (dataGridCell) { }
        }
    });

    Bridge.define("Windows.UI.Xaml.Controls.DataGridComboBoxColumn.MyConverter", {
        inherits: [Windows.UI.Xaml.Data.IValueConverter],
        $kind: "nested class",
        alias: [
            "Convert", "Windows$UI$Xaml$Data$IValueConverter$Convert",
            "ConvertBack", "Windows$UI$Xaml$Data$IValueConverter$ConvertBack"
        ],
        methods: {
            Convert: function (value, targetType, parameter, language) {
                if (value == null) {
                    return null;
                }
                return System.Enum.getName(Bridge.cast(parameter, System.Type), value);
            },
            ConvertBack: function (value, targetType, parameter, language) {
                if (value == null) {
                    return null;
                }
                return System.Enum.parse(Bridge.cast(parameter, System.Type), Bridge.cast(value, System.String));
            }
        }
    });

    /**
     * Represents a System.Windows.Controls.DataGrid row.
     *
     * @public
     * @class Windows.UI.Xaml.Controls.DataGridRow
     * @augments Windows.UI.Xaml.DependencyObject
     */
    Bridge.define("Windows.UI.Xaml.Controls.DataGridRow", {
        inherits: [Windows.UI.Xaml.DependencyObject],
        statics: {
            fields: {
                DefaultTemplateForExtendedSelectionMode: null,
                HeaderTemplateProperty: null,
                IsSelectedProperty: null,
                /**
                 * Identifies the DataContext dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Controls.DataGridRow
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                DataContextProperty: null
            },
            ctors: {
                init: function () {
                    var $t;
                    this.DefaultTemplateForExtendedSelectionMode = ($t = new Windows.UI.Xaml.DataTemplate(), $t._methodToInstantiateFrameworkTemplate = Windows.UI.Xaml.Controls.DataGridRow.GenerateDefaultHeaderTemplateForExtendedSelectionMode, $t);
                    this.HeaderTemplateProperty = Windows.UI.Xaml.DependencyProperty.Register("HeaderTemplate", Windows.UI.Xaml.DataTemplate, Windows.UI.Xaml.Controls.DataGridRow, ($t = new Windows.UI.Xaml.PropertyMetadata.$ctor2(null, Windows.UI.Xaml.Controls.DataGridRow.HeaderTemplate_Changed), $t.CallPropertyChangedWhenLoadedIntoVisualTree = Windows.UI.Xaml.WhenToCallPropertyChangedEnum.IfPropertyIsSet, $t));
                    this.IsSelectedProperty = Windows.UI.Xaml.DependencyProperty.Register("IsSelected", System.Boolean, Windows.UI.Xaml.Controls.DataGridRow, ($t = new Windows.UI.Xaml.PropertyMetadata.$ctor2(Bridge.box(false, System.Boolean, System.Boolean.toString), Windows.UI.Xaml.Controls.DataGridRow.IsSelected_Changed), $t.CallPropertyChangedWhenLoadedIntoVisualTree = Windows.UI.Xaml.WhenToCallPropertyChangedEnum.IfPropertyIsSet, $t));
                    this.DataContextProperty = Windows.UI.Xaml.DependencyProperty.Register("DataContext", System.Object, Windows.UI.Xaml.Controls.DataGridRow, ($t = new Windows.UI.Xaml.PropertyMetadata.ctor(), $t.Inherits = true, $t));
                }
            },
            methods: {
                GenerateDefaultHeaderTemplateForExtendedSelectionMode: function (templateOwner) {
                    if (templateOwner === void 0) { templateOwner = null; }
                    var templateInstance = new Windows.UI.Xaml.TemplateInstance();
                    templateInstance.TemplateOwner = templateOwner;
                    var border = new Windows.UI.Xaml.Controls.Border();
                    border.HorizontalAlignment = Windows.UI.Xaml.HorizontalAlignment.Stretch;
                    border.VerticalAlignment = Windows.UI.Xaml.VerticalAlignment.Stretch;
                    border.Background = new Windows.UI.Xaml.Media.SolidColorBrush.$ctor1(Windows.UI.Colors.Gray.$clone());
                    var checkbox = new Windows.UI.Xaml.Controls.CheckBox();
                    checkbox.HorizontalAlignment = Windows.UI.Xaml.HorizontalAlignment.Center;
                    checkbox.VerticalAlignment = Windows.UI.Xaml.VerticalAlignment.Center;
                    var b = new Windows.UI.Xaml.Data.Binding.$ctor1("IsSelected");
                    b.Mode = Windows.UI.Xaml.Data.BindingMode.TwoWay;
                    checkbox.SetBinding(Windows.UI.Xaml.Controls.Primitives.ToggleButton.IsCheckedProperty, b);
                    border.Child = checkbox;
                    templateInstance.TemplateContent = border;
                    return templateInstance;
                },
                HeaderTemplate_Changed: function (d, e) {
                    var dataGridRow = Bridge.cast(d, Windows.UI.Xaml.Controls.DataGridRow);
                    if (dataGridRow != null) {
                        if (CSHTML5.Internal.INTERNAL_VisualTreeManager.IsElementInVisualTree(dataGridRow._datagrid)) {
                            if (!Bridge.referenceEquals(e.NewValue, e.OldValue)) {
                                if (dataGridRow._headerUIElement != null && CSHTML5.Internal.INTERNAL_VisualTreeManager.IsElementInVisualTree(dataGridRow._headerUIElement)) {
                                    dataGridRow._datagrid.RemoveElementFromGrid(dataGridRow._headerUIElement);
                                }
                                var template = Bridge.cast(e.NewValue, Windows.UI.Xaml.DataTemplate);
                                if (template != null) {
                                    dataGridRow._headerUIElement = template.INTERNAL_InstantiateFrameworkTemplate();
                                    Windows.UI.Xaml.Controls.Grid.SetRow(dataGridRow._headerUIElement, dataGridRow._rowIndex);
                                    Windows.UI.Xaml.Controls.Grid.SetColumn(dataGridRow._headerUIElement, 0);
                                    if (Bridge.is(dataGridRow._headerUIElement, Windows.UI.Xaml.FrameworkElement)) {
                                        Bridge.cast(dataGridRow._headerUIElement, Windows.UI.Xaml.FrameworkElement).DataContext = dataGridRow;
                                    }
                                    var isCSSGrid = document.isGridSupported;
                                    if (isCSSGrid) {
                                        if (Bridge.is(dataGridRow._headerUIElement, Windows.UI.Xaml.Controls.Control)) {
                                            Bridge.cast(dataGridRow._headerUIElement, Windows.UI.Xaml.Controls.Control).BorderBrush = dataGridRow._datagrid.HorizontalGridLinesBrush;
                                            Bridge.cast(dataGridRow._headerUIElement, Windows.UI.Xaml.Controls.Control).BorderThickness = new Windows.UI.Xaml.Thickness.$ctor2(1, 0, 1, 1);
                                        } else if (Bridge.is(dataGridRow._headerUIElement, Windows.UI.Xaml.Controls.Border)) {
                                            Bridge.cast(dataGridRow._headerUIElement, Windows.UI.Xaml.Controls.Border).BorderBrush = dataGridRow._datagrid.HorizontalGridLinesBrush;
                                            Bridge.cast(dataGridRow._headerUIElement, Windows.UI.Xaml.Controls.Border).BorderThickness = new Windows.UI.Xaml.Thickness.$ctor2(1, 0, 1, 1);
                                        }
                                    }
                                    dataGridRow._datagrid.AddElementToGrid(dataGridRow._headerUIElement);
                                }
                            }
                        }
                    }
                },
                IsSelected_Changed: function (d, e) {
                    var row = Bridge.cast(d, Windows.UI.Xaml.Controls.DataGridRow);

                    if (row._representationInRow != null && row._representationInRow.ElementsInRow != null && row._representationInRow.ElementsInRow.Count > 0) {
                        var newValue = System.Nullable.getValue(Bridge.cast(Bridge.unbox(e.NewValue, System.Boolean), System.Boolean));
                        if (newValue !== System.Nullable.getValue(Bridge.cast(Bridge.unbox(e.OldValue, System.Boolean), System.Boolean))) {
                            if (newValue) {
                                row._datagrid.SelectItem(row._representationInRow.ElementsInRow.getItem(0).Item);
                            } else {
                                row._datagrid.UnselectItem(row._representationInRow.ElementsInRow.getItem(0).Item);
                            }
                        }
                    }
                }
            }
        },
        fields: {
            _headerUIElement: null,
            _representationInRow: null,
            _datagrid: null,
            _rowIndex: 0,
            _currentSelectionMode: 0
        },
        events: {
            MouseLeftButtonUp: null
        },
        props: {
            HeaderTemplate: {
                get: function () {
                    return Bridge.cast(this.GetValue(Windows.UI.Xaml.Controls.DataGridRow.HeaderTemplateProperty), Windows.UI.Xaml.DataTemplate);
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Controls.DataGridRow.HeaderTemplateProperty, value);
                }
            },
            IsSelected: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.Controls.DataGridRow.IsSelectedProperty), System.Boolean), System.Boolean));
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Controls.DataGridRow.IsSelectedProperty, Bridge.box(value, System.Boolean, System.Boolean.toString));
                }
            },
            /**
             * Gets or sets the data context for a DataGridRow when it participates
             in data binding.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.DataGridRow
             * @function DataContext
             * @type System.Object
             */
            DataContext: {
                get: function () {
                    return this.GetValue(Windows.UI.Xaml.Controls.DataGridRow.DataContextProperty);
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Controls.DataGridRow.DataContextProperty, value);
                }
            }
        },
        ctors: {
            init: function () {
                this._representationInRow = new Windows.UI.Xaml.Controls.DataGridRow.ObjectRepresentationInRow();
            },
            ctor: function () {
                this.$initialize();
                Windows.UI.Xaml.DependencyObject.ctor.call(this);
                this.CanBeInheritanceContext = false;
                this.IsInheritanceContextSealed = true;
            }
        },
        methods: {
            OnMouseLeftButtonUp: function (sender, e) {
                if (!Bridge.staticEquals(this.MouseLeftButtonUp, null)) {
                    this.MouseLeftButtonUp(this, new Windows.UI.Xaml.Input.PointerRoutedEventArgs());
                }
            },
            ChangeIntoSelectionMode: function (newSelectionMode) {
                if (CSHTML5.Internal.INTERNAL_VisualTreeManager.IsElementInVisualTree(this._datagrid)) {
                    if (newSelectionMode === Windows.UI.Xaml.Controls.DataGridSelectionMode.Extended) {
                        if (this._currentSelectionMode !== Windows.UI.Xaml.Controls.DataGridSelectionMode.Extended) {
                            this._currentSelectionMode = Windows.UI.Xaml.Controls.DataGridSelectionMode.Extended;
                        }
                    } else {
                        if (this._headerUIElement != null && CSHTML5.Internal.INTERNAL_VisualTreeManager.IsElementInVisualTree(this._headerUIElement)) {
                            this._datagrid.RemoveElementFromGrid(this._headerUIElement);
                        }
                    }
                }
            },
            INTERNAL_SetRowIndex: function (rowIndex) {
                var $t;
                this._rowIndex = rowIndex;
                if (this._headerUIElement != null) {
                    Windows.UI.Xaml.Controls.Grid.SetRow(this._headerUIElement, rowIndex);
                }
                $t = Bridge.getEnumerator(this._representationInRow.ElementsInRow);
                try {
                    while ($t.moveNext()) {
                        var element = Bridge.cast($t.Current, Windows.UI.Xaml.UIElement);
                        Windows.UI.Xaml.Controls.Grid.SetRow(element, rowIndex);
                    }
                } finally {
                    if (Bridge.is($t, System.IDisposable)) {
                        $t.System$IDisposable$Dispose();
                    }
                }

            },
            INTERNAL_SetRowEvents: function () {
                var $t;
                $t = Bridge.getEnumerator(this._representationInRow.ElementsInRow);
                try {
                    while ($t.moveNext()) {
                        var element = $t.Current;

                        element.addPointerReleased(Bridge.fn.cacheBind(this, this.OnMouseLeftButtonUp));
                    }
                } finally {
                    if (Bridge.is($t, System.IDisposable)) {
                        $t.System$IDisposable$Dispose();
                    }
                }
            },
            VisuallyRefreshItemSelection: function (newSelectionState) {
                var $t;
                $t = Bridge.getEnumerator(this._representationInRow.ElementsInRow);
                try {
                    while ($t.moveNext()) {
                        var currentCell = $t.Current;
                        currentCell.IsSelected = newSelectionState;
                    }
                } finally {
                    if (Bridge.is($t, System.IDisposable)) {
                        $t.System$IDisposable$Dispose();
                    }
                }
            },
            /**
             * attempts to get the cell at the given index.
             *
             * @instance
             * @this Windows.UI.Xaml.Controls.DataGridRow
             * @memberof Windows.UI.Xaml.Controls.DataGridRow
             * @param   {number}                                   index
             * @return  {Windows.UI.Xaml.Controls.DataGridCell}
             */
            TryGetCell: function (index) {
                if (index < this._representationInRow.ElementsInRow.Count && index >= 0) {
                    var cell = this._representationInRow.ElementsInRow.getItem(index);

                    if (cell != null) {
                        return cell;
                    }
                }

                return null;
            },
            INTERNAL_SetDataContext: function () {
                if (this._representationInRow.ElementsInRow.Count > 0) {
                    this.DataContext = this._representationInRow.ElementsInRow.getItem(0).Item;
                }
            }
        }
    });

    /**
     * Provides a common collection class for Silverlight collections.
     *
     * @abstract
     * @public
     * @class Windows.UI.Xaml.PresentationFrameworkCollection$1
     * @augments Windows.UI.Xaml.DependencyObject
     * @implements  System.Collections.Generic.IList$1
     * @implements  System.Collections.Generic.ICollection$1
     * @implements  System.Collections.Generic.IEnumerable$1
     * @implements  System.Collections.IList
     * @implements  System.Collections.ICollection
     * @implements  System.Collections.IEnumerable
     * @param   {Function}    [name]    Type constraint for type safety of the constrained collection implementation.
     */
    Bridge.define("Windows.UI.Xaml.PresentationFrameworkCollection$1", function (T) { return {
        inherits: [Windows.UI.Xaml.DependencyObject,System.Collections.Generic.IList$1(T),System.Collections.Generic.ICollection$1(T),System.Collections.Generic.IEnumerable$1(T),System.Collections.IList,System.Collections.ICollection,System.Collections.IEnumerable],
        statics: {
            fields: {
                CountProperty: null
            },
            ctors: {
                init: function () {
                    this.CountProperty = Windows.UI.Xaml.DependencyProperty.Register("Count", System.Int32, Windows.UI.Xaml.PresentationFrameworkCollection$1(T), new Windows.UI.Xaml.PropertyMetadata.$ctor1(Bridge.box(0, System.Int32)));
                }
            }
        },
        fields: {
            _collection: null
        },
        props: {
            Count: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.PresentationFrameworkCollection$1(T).CountProperty), System.Int32), System.Int32));
                }
            },
            /**
             * Gets a value indicating whether the {@link }
             has a fixed size.
             *
             * @instance
             * @public
             * @readonly
             * @memberof Windows.UI.Xaml.PresentationFrameworkCollection$1
             * @function IsFixedSize
             * @type boolean
             * @return  {[type]}        true if the {@link } has a fixed size;
             otherwise, false.
             */
            IsFixedSize: {
                get: function () {
                    return this.IsFixedSizeImpl;
                }
            },
            /**
             * Gets a value indicating whether the {@link }
             is read-only.
             *
             * @instance
             * @public
             * @readonly
             * @memberof Windows.UI.Xaml.PresentationFrameworkCollection$1
             * @function IsReadOnly
             * @type boolean
             * @return  {[type]}        true if the {@link } is read-only; otherwise,
             false.
             */
            IsReadOnly: {
                get: function () {
                    return this.IsReadOnlyImpl;
                }
            },
            /**
             * Gets a value indicating whether access to the {@link }
             is synchronized (thread safe).
             *
             * @instance
             * @public
             * @readonly
             * @memberof Windows.UI.Xaml.PresentationFrameworkCollection$1
             * @function IsSynchronized
             * @type boolean
             * @return  {[type]}        true if access to the {@link } is synchronized
             (thread safe); otherwise, false.
             */
            IsSynchronized: {
                get: function () {
                    return true;
                }
            },
            /**
             * Gets an object that can be used to synchronize access to the {@link }.
             *
             * @instance
             * @public
             * @readonly
             * @memberof Windows.UI.Xaml.PresentationFrameworkCollection$1
             * @function SyncRoot
             * @type System.Object
             * @return  {[type]}        An object that can be used to synchronize access to the {@link }.
             */
            SyncRoot: {
                get: function () {
                    return this;
                }
            },
            IsFixedSizeImpl: {
                get: function () {
                    return false;
                }
            },
            IsReadOnlyImpl: {
                get: function () {
                    return false;
                }
            },
            /**
             * Get the Count of the underlying {@link } collection.
             This property returns the same value as the Count property and is only here
             for performances.
             *
             * @instance
             * @readonly
             * @memberof Windows.UI.Xaml.PresentationFrameworkCollection$1
             * @function CountInternal
             * @type number
             */
            CountInternal: {
                get: function () {
                    return this._collection.Count;
                }
            }
        },
        alias: [
            "Count", "System$Collections$ICollection$Count",
            "Count", "System$Collections$Generic$ICollection$1$" + Bridge.getTypeAlias(T) + "$Count",
            "getItem", "System$Collections$Generic$IList$1$" + Bridge.getTypeAlias(T) + "$getItem",
            "setItem", "System$Collections$Generic$IList$1$" + Bridge.getTypeAlias(T) + "$setItem",
            "IsFixedSize", "System$Collections$IList$IsFixedSize",
            "IsReadOnly", "System$Collections$IList$IsReadOnly",
            "IsReadOnly", "System$Collections$Generic$ICollection$1$" + Bridge.getTypeAlias(T) + "$IsReadOnly",
            "IsSynchronized", "System$Collections$ICollection$IsSynchronized",
            "SyncRoot", "System$Collections$ICollection$SyncRoot",
            "GetEnumerator", ["System$Collections$Generic$IEnumerable$1$" + Bridge.getTypeAlias(T) + "$GetEnumerator", "System$Collections$Generic$IEnumerable$1$GetEnumerator"],
            "add", "System$Collections$Generic$ICollection$1$" + Bridge.getTypeAlias(T) + "$add",
            "clear", "System$Collections$IList$clear",
            "clear", "System$Collections$Generic$ICollection$1$" + Bridge.getTypeAlias(T) + "$clear",
            "contains", "System$Collections$Generic$ICollection$1$" + Bridge.getTypeAlias(T) + "$contains",
            "copyTo$1", "System$Collections$ICollection$copyTo",
            "copyTo", "System$Collections$Generic$ICollection$1$" + Bridge.getTypeAlias(T) + "$copyTo",
            "indexOf", "System$Collections$Generic$IList$1$" + Bridge.getTypeAlias(T) + "$indexOf",
            "insert", "System$Collections$Generic$IList$1$" + Bridge.getTypeAlias(T) + "$insert",
            "remove", "System$Collections$Generic$ICollection$1$" + Bridge.getTypeAlias(T) + "$remove",
            "removeAt", "System$Collections$Generic$IList$1$" + Bridge.getTypeAlias(T) + "$removeAt"
        ],
        ctors: {
            ctor: function () {
                this.$initialize();
                Windows.UI.Xaml.DependencyObject.ctor.call(this);
                this._collection = new (System.Collections.Generic.List$1(T)).ctor();
                this.UpdateCountProperty(this.CountInternal);
            },
            $ctor2: function (capacity) {
                this.$initialize();
                Windows.UI.Xaml.DependencyObject.ctor.call(this);
                this._collection = new (System.Collections.Generic.List$1(T)).$ctor2(capacity);
                this.UpdateCountProperty(this.CountInternal);
            },
            $ctor1: function (source) {
                this.$initialize();
                Windows.UI.Xaml.DependencyObject.ctor.call(this);
                this._collection = new (System.Collections.Generic.List$1(T)).$ctor1(source);
                this.UpdateCountProperty(this.CountInternal);
            }
        },
        methods: {
            getItem: function (index) {
                return this.GetItemOverride(index);
            },
            setItem: function (index, value) {
                this.SetItemOverride(index, value);
            },
            System$Collections$IList$getItem: function (index) {
                return this.getItem(index);
            },
            System$Collections$IList$setItem: function (index, value) {
                this.setItem(index, Bridge.cast(Bridge.unbox(value, T), T));
            },
            /**
             * Returns an enumerator that iterates through a collection.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.PresentationFrameworkCollection$1
             * @memberof Windows.UI.Xaml.PresentationFrameworkCollection$1
             * @return  {System.Collections.Generic.IEnumerator$1}        An {@link } object that can be used to iterate through
             the collection.
             */
            GetEnumerator: function () {
                return this.GetEnumeratorImpl();
            },
            System$Collections$IEnumerable$GetEnumerator: function () {
                return this.GetEnumerator();
            },
            /**
             * Adds an item to the {@link }.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.PresentationFrameworkCollection$1
             * @memberof Windows.UI.Xaml.PresentationFrameworkCollection$1
             * @param   {T}       value    The object to add.
             * @return  {void}
             */
            add: function (value) {
                this.AddOverride(value);
            },
            System$Collections$IList$add: function (value) {
                this.add(Bridge.cast(Bridge.unbox(value, T), T));
                return this.Count;
            },
            /**
             * Removes all items from the {@link }.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.PresentationFrameworkCollection$1
             * @memberof Windows.UI.Xaml.PresentationFrameworkCollection$1
             * @return  {void}
             */
            clear: function () {
                this.ClearOverride();
            },
            /**
             * Determines whether the {@link } contains
             a specific value.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.PresentationFrameworkCollection$1
             * @memberof Windows.UI.Xaml.PresentationFrameworkCollection$1
             * @param   {T}          value    The object to locate in the {@link }.
             * @return  {boolean}             true if the object is found in the {@link };
             otherwise, false.
             */
            contains: function (value) {
                return this.ContainsImpl(value);
            },
            System$Collections$IList$contains: function (value) {
                return this.contains(Bridge.cast(Bridge.unbox(value, T), T));
            },
            /**
             * Copies the elements of the {@link } to
             an {@link }, starting at a particular {@link } index.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.PresentationFrameworkCollection$1
             * @memberof Windows.UI.Xaml.PresentationFrameworkCollection$1
             * @param   {Array}     array    The one-dimensional {@link } that is the destination of the elements copied
             from the {@link }. The {@link } must
             have zero-based indexing.
             * @param   {number}    index    The zero-based index in array at which copying begins.
             * @return  {void}
             */
            copyTo$1: function (array, index) {
                this.CopyToImpl(array, index);
            },
            /**
             * Copies the elements of the {@link } to
             an {@link }, starting at a particular {@link } index.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.PresentationFrameworkCollection$1
             * @memberof Windows.UI.Xaml.PresentationFrameworkCollection$1
             * @param   {Array.<T>}    array    The one-dimensional {@link } that is the destination of the elements copied
             from the {@link }. The {@link } must
             have zero-based indexing.
             * @param   {number}       index    The zero-based index in array at which copying begins.
             * @return  {void}
             */
            copyTo: function (array, index) {
                this.CopyToImpl(array, index);
            },
            /**
             * Determines the index of a specific item in the {@link }.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.PresentationFrameworkCollection$1
             * @memberof Windows.UI.Xaml.PresentationFrameworkCollection$1
             * @throws The object was not found in the list.
             * @param   {T}         value    The object to locate in the {@link }.
             * @return  {number}             The index of value if found in the list; otherwise, an exception.
             */
            indexOf: function (value) {
                return this.IndexOfImpl(value);
            },
            System$Collections$IList$indexOf: function (value) {
                return this.indexOf(Bridge.cast(Bridge.unbox(value, T), T));
            },
            /**
             * Inserts an item to the {@link } at the
             specified index.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.PresentationFrameworkCollection$1
             * @memberof Windows.UI.Xaml.PresentationFrameworkCollection$1
             * @param   {number}    index    The zero-based index at which value should be inserted.
             * @param   {T}         value    The object to insert into the {@link }.
             * @return  {void}
             */
            insert: function (index, value) {
                this.InsertOverride(index, value);
            },
            System$Collections$IList$insert: function (index, value) {
                this.insert(index, Bridge.cast(Bridge.unbox(value, T), T));
            },
            /**
             * Removes the first occurrence of a specific object from the {@link }.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.PresentationFrameworkCollection$1
             * @memberof Windows.UI.Xaml.PresentationFrameworkCollection$1
             * @param   {T}          value    The object to remove from the {@link }.
             * @return  {boolean}             true if an object was removed; otherwise, false.
             */
            remove: function (value) {
                return this.RemoveOverride(value);
            },
            System$Collections$IList$remove: function (value) {
                this.remove(Bridge.cast(Bridge.unbox(value, T), T));
            },
            /**
             * Removes the item at the specified index.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.PresentationFrameworkCollection$1
             * @memberof Windows.UI.Xaml.PresentationFrameworkCollection$1
             * @param   {number}    index    The zero-based index of the item to remove.
             * @return  {void}
             */
            removeAt: function (index) {
                this.RemoveAtOverride(index);
            },
            System$Collections$IList$removeAt: function (index) {
                this.removeAt(index);
            },
            CopyToImpl: function (array, index) {
                for (var i = 0; i < this.CountInternal; i = (i + 1) | 0) {
                    System.Array.set(array, this.getItem(i), ((index + i) | 0));
                }
            },
            ContainsImpl: function (value) {
                return this._collection.contains(value);
            },
            IndexOfImpl: function (value) {
                return this._collection.indexOf(value);
            },
            GetEnumeratorImpl: function () {
                return this._collection.GetEnumerator().$clone();
            },
            /**
             * Call the Add method of underlying {@link } collection.
             *
             * @instance
             * @this Windows.UI.Xaml.PresentationFrameworkCollection$1
             * @memberof Windows.UI.Xaml.PresentationFrameworkCollection$1
             * @param   {T}       value
             * @return  {void}
             */
            AddInternal: function (value) {
                this._collection.add(value);
                this.UpdateCountProperty(this.CountInternal);
            },
            /**
             * Call the Clear method of underlying {@link } collection.
             *
             * @instance
             * @this Windows.UI.Xaml.PresentationFrameworkCollection$1
             * @memberof Windows.UI.Xaml.PresentationFrameworkCollection$1
             * @return  {void}
             */
            ClearInternal: function () {
                this._collection.clear();
                this.UpdateCountProperty(this.CountInternal);
            },
            /**
             * Call the Insert method of underlying {@link } collection.
             *
             * @instance
             * @this Windows.UI.Xaml.PresentationFrameworkCollection$1
             * @memberof Windows.UI.Xaml.PresentationFrameworkCollection$1
             * @param   {number}    index    
             * @param   {T}         value
             * @return  {void}
             */
            InsertInternal: function (index, value) {
                this._collection.insert(index, value);
                this.UpdateCountProperty(this.CountInternal);
            },
            /**
             * Call the RemoveAt method of underlying {@link } collection.
             *
             * @instance
             * @this Windows.UI.Xaml.PresentationFrameworkCollection$1
             * @memberof Windows.UI.Xaml.PresentationFrameworkCollection$1
             * @param   {number}    index
             * @return  {void}
             */
            RemoveAtInternal: function (index) {
                this._collection.removeAt(index);
                this.UpdateCountProperty(this.CountInternal);
            },
            /**
             * Call the Remove method of underlying {@link } collection.
             *
             * @instance
             * @this Windows.UI.Xaml.PresentationFrameworkCollection$1
             * @memberof Windows.UI.Xaml.PresentationFrameworkCollection$1
             * @param   {T}          value
             * @return  {boolean}
             */
            RemoveInternal: function (value) {
                if (this._collection.remove(value)) {
                    this.UpdateCountProperty(this.CountInternal);
                    return true;
                }
                return false;
            },
            /**
             * Call the Indexer (getter) property of underlying {@link } collection.
             *
             * @instance
             * @this Windows.UI.Xaml.PresentationFrameworkCollection$1
             * @memberof Windows.UI.Xaml.PresentationFrameworkCollection$1
             * @param   {number}    index
             * @return  {T}
             */
            GetItemInternal: function (index) {
                return this._collection.getItem(index);
            },
            /**
             * Call the Indexer (setter) property of underlying {@link } collection.
             *
             * @instance
             * @this Windows.UI.Xaml.PresentationFrameworkCollection$1
             * @memberof Windows.UI.Xaml.PresentationFrameworkCollection$1
             * @param   {number}    index    
             * @param   {T}         value
             * @return  {void}
             */
            SetItemInternal: function (index, value) {
                this._collection.setItem(index, value);
            },
            CastDO: function (value) {
                var valueDO;
                if (!(((valueDO = Bridge.as(value, Windows.UI.Xaml.DependencyObject))) != null)) {
                    throw new System.ArgumentException.$ctor1(System.String.format("'{0}' does not derive from {1}.", Bridge.Reflection.getTypeName(T), Bridge.Reflection.getTypeName(Windows.UI.Xaml.DependencyObject)));
                }
                return valueDO;
            },
            /**
             * Call the Add method of underlying {@link } collection
             and update inheritance context to match this object.
             *
             * @instance
             * @this Windows.UI.Xaml.PresentationFrameworkCollection$1
             * @memberof Windows.UI.Xaml.PresentationFrameworkCollection$1
             * @param   {T}       value
             * @return  {void}
             */
            AddDependencyObjectInternal: function (value) {
                var valueDO = this.CastDO(value);
                this.ProvideSelfAsInheritanceContext$1(valueDO, null);
                this._collection.add(value);
                this.UpdateCountProperty(this.CountInternal);
            },
            /**
             * Call the Clear method of underlying {@link } collection
             and update inheritance context to match this object.
             *
             * @instance
             * @this Windows.UI.Xaml.PresentationFrameworkCollection$1
             * @memberof Windows.UI.Xaml.PresentationFrameworkCollection$1
             * @return  {void}
             */
            ClearDependencyObjectInternal: function () {
                var $t;
                $t = Bridge.getEnumerator(System.Linq.Enumerable.from(this).select(Bridge.fn.bind(this, function (v) {
                        return this.CastDO(v);
                    })), Windows.UI.Xaml.DependencyObject);
                try {
                    while ($t.moveNext()) {
                        var valueDO = $t.Current;
                        this.RemoveSelfAsInheritanceContext$1(valueDO, null);
                    }
                } finally {
                    if (Bridge.is($t, System.IDisposable)) {
                        $t.System$IDisposable$Dispose();
                    }
                }
                this._collection.clear();
                this.UpdateCountProperty(this.CountInternal);
            },
            /**
             * Call the Insert method of underlying {@link } collection
             and update inheritance context to match this object.
             *
             * @instance
             * @this Windows.UI.Xaml.PresentationFrameworkCollection$1
             * @memberof Windows.UI.Xaml.PresentationFrameworkCollection$1
             * @param   {number}    index    
             * @param   {T}         value
             * @return  {void}
             */
            InsertDependencyObjectInternal: function (index, value) {
                var valueDO = this.CastDO(value);
                this.ProvideSelfAsInheritanceContext$1(valueDO, null);
                this._collection.insert(index, value);
                this.UpdateCountProperty(this.CountInternal);
            },
            /**
             * Call the RemoveAt method of underlying {@link } collection
             and update inheritance context to match this object.
             *
             * @instance
             * @this Windows.UI.Xaml.PresentationFrameworkCollection$1
             * @memberof Windows.UI.Xaml.PresentationFrameworkCollection$1
             * @param   {number}    index
             * @return  {void}
             */
            RemoveAtDependencyObjectInternal: function (index) {
                var removedDO = this.CastDO(this._collection.getItem(index));
                this.RemoveSelfAsInheritanceContext$1(removedDO, null);
                this._collection.removeAt(index);
                this.UpdateCountProperty(this.CountInternal);
            },
            /**
             * Call the Remove method of underlying {@link } collection
             and update inheritance context to match this object.
             *
             * @instance
             * @this Windows.UI.Xaml.PresentationFrameworkCollection$1
             * @memberof Windows.UI.Xaml.PresentationFrameworkCollection$1
             * @param   {T}          value
             * @return  {boolean}
             */
            RemoveDependencyObjectInternal: function (value) {
                var index = this._collection.indexOf(value);
                if (index > -1) {
                    var removedDO = this.CastDO(this._collection.getItem(index));
                    this.RemoveSelfAsInheritanceContext$1(removedDO, null);
                    this._collection.removeAt(index);
                    this.UpdateCountProperty(this.CountInternal);
                    return true;
                }
                return false;
            },
            /**
             * Call the Indexer (setter) property of underlying {@link } collection
             and update inheritance context to match this object.
             *
             * @instance
             * @this Windows.UI.Xaml.PresentationFrameworkCollection$1
             * @memberof Windows.UI.Xaml.PresentationFrameworkCollection$1
             * @param   {number}    index    
             * @param   {T}         value
             * @return  {void}
             */
            SetItemDependencyObjectInternal: function (index, value) {
                var originalDO = this.CastDO(this._collection.getItem(index));
                var newDO = this.CastDO(value);
                this.RemoveSelfAsInheritanceContext$1(originalDO, null);
                this.ProvideSelfAsInheritanceContext$1(newDO, null);
                this._collection.setItem(index, value);
            },
            UpdateCountProperty: function (value) {
                this.SetValue(Windows.UI.Xaml.PresentationFrameworkCollection$1(T).CountProperty, Bridge.box(value, System.Int32));
            }
        }
    }; });

    /**
     * Provides data for the DragCompleted event that occurs when a user completes
     a drag operation with the mouse of a Thumb control.
     *
     * @public
     * @class Windows.UI.Xaml.Controls.Primitives.DragCompletedEventArgs
     * @augments Windows.UI.Xaml.RoutedEventArgs
     */
    Bridge.define("Windows.UI.Xaml.Controls.Primitives.DragCompletedEventArgs", {
        inherits: [Windows.UI.Xaml.RoutedEventArgs],
        fields: {
            /**
             * Gets a value that indicates whether the drag operation was canceled.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.Primitives.DragCompletedEventArgs
             * @function Canceled
             * @type boolean
             */
            Canceled: false,
            /**
             * Gets the horizontal distance between the current mouse position and the thumb
             coordinates.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.Primitives.DragCompletedEventArgs
             * @function HorizontalChange
             * @type number
             */
            HorizontalChange: 0,
            /**
             * Gets the vertical distance between the current mouse position and the thumb coordinates.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.Primitives.DragCompletedEventArgs
             * @function VerticalChange
             * @type number
             */
            VerticalChange: 0
        },
        ctors: {
            /**
             * Initializes a new instance of the DragCompletedEventArgs class.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Controls.Primitives.DragCompletedEventArgs
             * @memberof Windows.UI.Xaml.Controls.Primitives.DragCompletedEventArgs
             * @param   {number}     horizontalChange    The horizontal change in position of the Thumb control, resulting from the drag operation.
             * @param   {number}     verticalChange      The vertical change in position of the Thumb control, resulting from the drag operation.
             * @param   {boolean}    canceled            A value that indicates whether the drag operation was canceled by a call to the CancelDrag method.
             * @return  {void}
             */
            ctor: function (horizontalChange, verticalChange, canceled) {
                this.$initialize();
                Windows.UI.Xaml.RoutedEventArgs.ctor.call(this);
                this.HorizontalChange = horizontalChange;
                this.VerticalChange = verticalChange;
                this.Canceled = canceled;
            }
        }
    });

    /**
     * Provides data for the DragDelta event that occurs one or more times when
     a user drags a Thumb control with the mouse.
     *
     * @public
     * @class Windows.UI.Xaml.Controls.Primitives.DragDeltaEventArgs
     * @augments Windows.UI.Xaml.RoutedEventArgs
     */
    Bridge.define("Windows.UI.Xaml.Controls.Primitives.DragDeltaEventArgs", {
        inherits: [Windows.UI.Xaml.RoutedEventArgs],
        fields: {
            /**
             * Gets the horizontal change in the Thumb position since the last DragDelta event.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.Primitives.DragDeltaEventArgs
             * @function HorizontalChange
             * @type number
             */
            HorizontalChange: 0,
            /**
             * Gets the vertical change in the Thumb position since the last DragDelta event.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.Primitives.DragDeltaEventArgs
             * @function VerticalChange
             * @type number
             */
            VerticalChange: 0
        },
        ctors: {
            /**
             * Initializes a new instance of the DragDeltaEventArgs class.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Controls.Primitives.DragDeltaEventArgs
             * @memberof Windows.UI.Xaml.Controls.Primitives.DragDeltaEventArgs
             * @param   {number}    horizontalChange    The horizontal change in the Thumb position since the last DragDelta event.
             * @param   {number}    verticalChange      The vertical change in the Thumb position since the last DragDelta event.
             * @return  {void}
             */
            ctor: function (horizontalChange, verticalChange) {
                this.$initialize();
                Windows.UI.Xaml.RoutedEventArgs.ctor.call(this);
                this.HorizontalChange = horizontalChange;
                this.VerticalChange = verticalChange;
            }
        }
    });

    /**
     * Provides data for the DragStarted event that occurs when a user drags a Thumb control with the mouse.
     *
     * @public
     * @class Windows.UI.Xaml.Controls.Primitives.DragStartedEventArgs
     * @augments Windows.UI.Xaml.RoutedEventArgs
     */
    Bridge.define("Windows.UI.Xaml.Controls.Primitives.DragStartedEventArgs", {
        inherits: [Windows.UI.Xaml.RoutedEventArgs],
        fields: {
            /**
             * Gets the horizontal distance between the current mouse position and the thumb
             coordinates.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.Primitives.DragStartedEventArgs
             * @function HorizontalOffset
             * @type number
             */
            HorizontalOffset: 0,
            /**
             * Gets the vertical distance between the current mouse position and the thumb
             coordinates.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.Primitives.DragStartedEventArgs
             * @function VerticalOffset
             * @type number
             */
            VerticalOffset: 0
        },
        ctors: {
            /**
             * Initializes a new instance of the DragStartedEventArgs class.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Controls.Primitives.DragStartedEventArgs
             * @memberof Windows.UI.Xaml.Controls.Primitives.DragStartedEventArgs
             * @param   {number}    horizontalOffset    The horizontal distance between the current mouse position and the thumb coordinates.
             * @param   {number}    verticalOffset      The vertical distance between the current mouse position and the thumb coordinates.
             * @return  {void}
             */
            ctor: function (horizontalOffset, verticalOffset) {
                this.$initialize();
                Windows.UI.Xaml.RoutedEventArgs.ctor.call(this);
                this.HorizontalOffset = horizontalOffset;
                this.VerticalOffset = verticalOffset;
            }
        }
    });

    /**
     * Provides data about a change in range value for the ValueChanged event.
     *
     * @public
     * @class Windows.UI.Xaml.Controls.Primitives.RangeBaseValueChangedEventArgs
     * @augments Windows.UI.Xaml.RoutedEventArgs
     */
    Bridge.define("Windows.UI.Xaml.Controls.Primitives.RangeBaseValueChangedEventArgs", {
        inherits: [Windows.UI.Xaml.RoutedEventArgs],
        fields: {
            /**
             * Gets the new value of a range value property.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.Primitives.RangeBaseValueChangedEventArgs
             * @function NewValue
             * @type number
             */
            NewValue: 0,
            /**
             * Gets the previous value of a range value property.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.Primitives.RangeBaseValueChangedEventArgs
             * @function OldValue
             * @type number
             */
            OldValue: 0
        },
        ctors: {
            /**
             * Initializes a new instance of the RangeBaseValueChangedEventArgs class.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Controls.Primitives.RangeBaseValueChangedEventArgs
             * @memberof Windows.UI.Xaml.Controls.Primitives.RangeBaseValueChangedEventArgs
             * @param   {number}    newValue    The new value of a range value property.
             * @param   {number}    oldValue    The previous value of a range value property.
             * @return  {void}
             */
            ctor: function (newValue, oldValue) {
                this.$initialize();
                Windows.UI.Xaml.RoutedEventArgs.ctor.call(this);
                this.NewValue = newValue;
                this.OldValue = oldValue;
            }
        }
    });

    /**
     * Provides data for the Scroll event.
     *
     * @public
     * @class Windows.UI.Xaml.Controls.Primitives.ScrollEventArgs
     * @augments Windows.UI.Xaml.RoutedEventArgs
     */
    Bridge.define("Windows.UI.Xaml.Controls.Primitives.ScrollEventArgs", {
        inherits: [Windows.UI.Xaml.RoutedEventArgs],
        fields: {
            /**
             * Gets the new Value of the ScrollBar.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.Primitives.ScrollEventArgs
             * @function NewValue
             * @type number
             */
            NewValue: 0,
            /**
             * Gets a ScrollEventType describing the event.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.Primitives.ScrollEventArgs
             * @function ScrollEventType
             * @type Windows.UI.Xaml.Controls.Primitives.ScrollEventType
             */
            ScrollEventType: 0
        },
        ctors: {
            /**
             * Initializes a new instance of the ScrollEventArgs class.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Controls.Primitives.ScrollEventArgs
             * @memberof Windows.UI.Xaml.Controls.Primitives.ScrollEventArgs
             * @param   {number}                                                 newValue           The new Value of the ScrollBar.
             * @param   {Windows.UI.Xaml.Controls.Primitives.ScrollEventType}    scrollEventType    A ScrollEventType describing the event.
             * @return  {void}
             */
            ctor: function (newValue, scrollEventType) {
                this.$initialize();
                Windows.UI.Xaml.RoutedEventArgs.ctor.call(this);
                this.NewValue = newValue;
                this.ScrollEventType = scrollEventType;
            }
        }
    });

    /**
     * Defines row-specific properties that apply to Grid elements.
     *
     * @public
     * @class Windows.UI.Xaml.Controls.RowDefinition
     * @augments Windows.UI.Xaml.DependencyObject
     */
    Bridge.define("Windows.UI.Xaml.Controls.RowDefinition", {
        inherits: [Windows.UI.Xaml.DependencyObject],
        statics: {
            fields: {
                /**
                 * Identifies the MaxHeight dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Controls.RowDefinition
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                MaxHeightProperty: null,
                /**
                 * Identifies the MinHeight dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Controls.RowDefinition
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                MinHeightProperty: null,
                /**
                 * Identifies the Height dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Controls.RowDefinition
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                HeightProperty: null
            },
            ctors: {
                init: function () {
                    var $t;
                    this.MaxHeightProperty = Windows.UI.Xaml.DependencyProperty.Register("MaxHeight", System.Double, Windows.UI.Xaml.Controls.RowDefinition, ($t = new Windows.UI.Xaml.PropertyMetadata.$ctor2(Bridge.box(Number.POSITIVE_INFINITY, System.Double, System.Double.format, System.Double.getHashCode), Windows.UI.Xaml.Controls.RowDefinition.MaxHeight_Changed), $t.CallPropertyChangedWhenLoadedIntoVisualTree = Windows.UI.Xaml.WhenToCallPropertyChangedEnum.IfPropertyIsSet, $t));
                    this.MinHeightProperty = Windows.UI.Xaml.DependencyProperty.Register("MinHeight", System.Double, Windows.UI.Xaml.Controls.RowDefinition, ($t = new Windows.UI.Xaml.PropertyMetadata.$ctor2(Bridge.box(Number.POSITIVE_INFINITY, System.Double, System.Double.format, System.Double.getHashCode), Windows.UI.Xaml.Controls.RowDefinition.MinHeight_Changed), $t.CallPropertyChangedWhenLoadedIntoVisualTree = Windows.UI.Xaml.WhenToCallPropertyChangedEnum.IfPropertyIsSet, $t));
                    this.HeightProperty = Windows.UI.Xaml.DependencyProperty.Register("Height", Windows.UI.Xaml.GridLength, Windows.UI.Xaml.Controls.RowDefinition, ($t = new Windows.UI.Xaml.PropertyMetadata.$ctor2(new Windows.UI.Xaml.GridLength.$ctor2(1.0, Windows.UI.Xaml.GridUnitType.Star).$clone(), Windows.UI.Xaml.Controls.RowDefinition.Height_Changed), $t.CallPropertyChangedWhenLoadedIntoVisualTree = Windows.UI.Xaml.WhenToCallPropertyChangedEnum.IfPropertyIsSet, $t));
                }
            },
            methods: {
                MaxHeight_Changed: function (d, e) {
                    var rowDefinition = Bridge.cast(d, Windows.UI.Xaml.Controls.RowDefinition);
                    var newValue = System.Nullable.getValue(Bridge.cast(Bridge.unbox(e.NewValue, System.Double), System.Double));
                },
                MinHeight_Changed: function (d, e) {
                    var rowDefinition = Bridge.cast(d, Windows.UI.Xaml.Controls.RowDefinition);
                    var newValue = System.Nullable.getValue(Bridge.cast(Bridge.unbox(e.NewValue, System.Double), System.Double));
                },
                Height_Changed: function (d, e) {
                    var rowDefinition = Bridge.cast(d, Windows.UI.Xaml.Controls.RowDefinition);
                    var grid = rowDefinition.Parent;

                    if (grid != null && CSHTML5.Internal.INTERNAL_VisualTreeManager.IsElementInVisualTree(grid)) {
                        if (document.isGridSupported) {

                            Windows.UI.Xaml.Controls.Grid_InternalHelpers.RefreshAllRowsHeight_CSSVersion(grid);
                        } else {

                            var isStar = (e.OldValue != null && System.Nullable.getValue(Bridge.cast(Bridge.unbox(e.OldValue, Windows.UI.Xaml.GridLength), Windows.UI.Xaml.GridLength)).IsStar) || (e.NewValue != null && System.Nullable.getValue(Bridge.cast(Bridge.unbox(e.NewValue, Windows.UI.Xaml.GridLength), Windows.UI.Xaml.GridLength)).IsStar);
                            if (isStar) {
                                Windows.UI.Xaml.Controls.Grid_InternalHelpers.RefreshAllRowsHeight_NonCSSVersion(grid);
                            } else {
                                if (grid._columnDefinitionsOrNull != null) {
                                    var rowIndex = grid._rowDefinitionsOrNull.indexOf(rowDefinition);
                                    var isTheOnlyRow = grid._columnDefinitionsOrNull == null || grid._columnDefinitionsOrNull.Count < 2;
                                    Windows.UI.Xaml.Controls.Grid_InternalHelpers.RefreshRowHeight_NonCSSVersion(grid, rowIndex, isTheOnlyRow);
                                }
                            }

                        }
                    }
                }
            }
        },
        fields: {
            Parent: null
        },
        props: {
            /**
             * Gets or sets a value that represents the maximum height of a RowDefinition. Returns a Double that represents the maximum height in pixels. The default is PositiveInfinity.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.RowDefinition
             * @function MaxHeight
             * @type number
             */
            MaxHeight: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.Controls.RowDefinition.MaxHeightProperty), System.Double), System.Double));
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Controls.RowDefinition.MaxHeightProperty, Bridge.box(value, System.Double, System.Double.format, System.Double.getHashCode));
                }
            },
            /**
             * Gets or sets a value that represents the minimum height of a RowDefinition. Returns a Double that represents the minimum height in pixels. The default is 0.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.RowDefinition
             * @function MinHeight
             * @type number
             */
            MinHeight: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.Controls.RowDefinition.MinHeightProperty), System.Double), System.Double));
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Controls.RowDefinition.MinHeightProperty, Bridge.box(value, System.Double, System.Double.format, System.Double.getHashCode));
                }
            },
            /**
             * Gets the calculated height of a RowDefinition element, or sets the GridLength value of a row that is defined by the RowDefinition.
             Returns the GridLength that represents the height of the row. The default value is 1.0
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.RowDefinition
             * @function Height
             * @type Windows.UI.Xaml.GridLength
             */
            Height: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.Controls.RowDefinition.HeightProperty), Windows.UI.Xaml.GridLength), Windows.UI.Xaml.GridLength));
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Controls.RowDefinition.HeightProperty, value.$clone());
                }
            },
            /**
             * // Gets the rendered width of a FrameworkElement. The FrameworkElement must be in the visual tree,
            // otherwise this property will return double.NaN.
            //
             * Gets the rendered height of a FrameworkElement. The FrameworkElement must be in the visual tree,
             otherwise this property will return double.NaN.
             *
             * @instance
             * @public
             * @readonly
             * @memberof Windows.UI.Xaml.Controls.RowDefinition
             * @function ActualHeight
             * @type number
             */
            ActualHeight: {
                get: function () {
                    return this.Parent.GetRowActualHeight(this);
                }
            }
        },
        methods: {
            /**
             * Returns a copy of this RowDefinition.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Controls.RowDefinition
             * @memberof Windows.UI.Xaml.Controls.RowDefinition
             * @return  {Windows.UI.Xaml.Controls.RowDefinition}        A copy of this RowDefinition.
             */
            Clone: function () {
                var $t;
                return ($t = new Windows.UI.Xaml.Controls.RowDefinition(), $t.MinHeight = this.MinHeight, $t.MaxHeight = this.MaxHeight, $t.Height = this.Height.Clone(), $t);
            }
        }
    });

    /**
     * Provides data for the SelectionChanged event.
     *
     * @public
     * @class Windows.UI.Xaml.Controls.SelectionChangedEventArgs
     * @augments Windows.UI.Xaml.RoutedEventArgs
     */
    Bridge.define("Windows.UI.Xaml.Controls.SelectionChangedEventArgs", {
        inherits: [Windows.UI.Xaml.RoutedEventArgs],
        fields: {
            _addedItems: null,
            _removedItems: null
        },
        props: {
            /**
             * Gets a list that contains the items that were selected.
             *
             * @instance
             * @public
             * @readonly
             * @memberof Windows.UI.Xaml.Controls.SelectionChangedEventArgs
             * @function AddedItems
             * @type System.Collections.IList
             */
            AddedItems: {
                get: function () {
                    return this._addedItems;
                }
            },
            /**
             * Gets a list that contains the items that were unselected.
             *
             * @instance
             * @public
             * @readonly
             * @memberof Windows.UI.Xaml.Controls.SelectionChangedEventArgs
             * @function RemovedItems
             * @type System.Collections.IList
             */
            RemovedItems: {
                get: function () {
                    return this._removedItems;
                }
            }
        },
        ctors: {
            /**
             * Initializes a new instance of the SelectionChangedEventArgs class with the specified removed and added items.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Controls.SelectionChangedEventArgs
             * @memberof Windows.UI.Xaml.Controls.SelectionChangedEventArgs
             * @param   {System.Collections.IList}    removedItems    A list of the elements that have been removed from the list.
             * @param   {System.Collections.IList}    addedItems      A list of the elements that have been added to the list.
             * @return  {void}
             */
            ctor: function (removedItems, addedItems) {
                this.$initialize();
                Windows.UI.Xaml.RoutedEventArgs.ctor.call(this);
                this._addedItems = addedItems;
                this._removedItems = removedItems;
            }
        }
    });

    /**
     * A collection of selected items.
     *
     * @public
     * @class Windows.UI.Xaml.Controls.SelectionCollection
     * @augments System.Collections.ObjectModel.Collection$1
     */
    Bridge.define("Windows.UI.Xaml.Controls.SelectionCollection", {
        inherits: [System.Collections.ObjectModel.Collection$1(Windows.UI.Xaml.Controls.Selection)],
        statics: {
            methods: {
                /**
                 * Returns data as a SelectionCollection.
                 *
                 * @static
                 * @this Windows.UI.Xaml.Controls.SelectionCollection
                 * @memberof Windows.UI.Xaml.Controls.SelectionCollection
                 * @param   {System.Object}                                   data    The data object.
                 * @return  {Windows.UI.Xaml.Controls.SelectionCollection}            A selection collection.
                 */
                ToSelectionCollection: function (data) {

                    if (data == null) {
                        return new Windows.UI.Xaml.Controls.SelectionCollection.ctor();
                    }

                    var selectionCollection = Bridge.as(data, Windows.UI.Xaml.Controls.SelectionCollection);
                    if (selectionCollection == null) {
                        selectionCollection = new Windows.UI.Xaml.Controls.SelectionCollection.ctor();
                        var selection = Bridge.as(data, Windows.UI.Xaml.Controls.Selection);
                        if (selection == null) {
                            selection = new Windows.UI.Xaml.Controls.Selection.$ctor1(data);
                        }
                        selectionCollection.add(selection);
                    }

                    return selectionCollection;
                }
            }
        },
        props: {
            /**
             * Gets a sequence of the items in the selection collection.
             *
             * @instance
             * @readonly
             * @memberof Windows.UI.Xaml.Controls.SelectionCollection
             * @function SelectedItems
             * @type System.Collections.Generic.IEnumerable$1
             */
            SelectedItems: {
                get: function () {
                    return System.Linq.Enumerable.from(this).select(function (selection) {
                            return selection.item;
                        });
                }
            }
        },
        ctors: {
            /**
             * Initializes a new instance of a SelectionCollection.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Controls.SelectionCollection
             * @memberof Windows.UI.Xaml.Controls.SelectionCollection
             * @return  {void}
             */
            ctor: function () {
                this.$initialize();
                System.Collections.ObjectModel.Collection$1(Windows.UI.Xaml.Controls.Selection).ctor.call(this);
            },
            /**
             * Initializes a new instance of a SelectionCollection.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Controls.SelectionCollection
             * @memberof Windows.UI.Xaml.Controls.SelectionCollection
             * @param   {System.Collections.Generic.IEnumerable$1}    items    The items to include in the selection 
             collection.
             * @return  {void}
             */
            $ctor1: function (items) {
                var $t;
                this.$initialize();
                System.Collections.ObjectModel.Collection$1(Windows.UI.Xaml.Controls.Selection).ctor.call(this);
                $t = Bridge.getEnumerator(items, System.Object);
                try {
                    while ($t.moveNext()) {
                        var item = $t.Current;
                        this.add(new Windows.UI.Xaml.Controls.Selection.$ctor1(item));
                    }
                } finally {
                    if (Bridge.is($t, System.IDisposable)) {
                        $t.System$IDisposable$Dispose();
                    }
                }
            }
        }
    });

    /**
     * Provides data for the TextChanged event.
     *
     * @public
     * @class Windows.UI.Xaml.Controls.TextChangedEventArgs
     * @augments Windows.UI.Xaml.RoutedEventArgs
     */
    Bridge.define("Windows.UI.Xaml.Controls.TextChangedEventArgs", {
        inherits: [Windows.UI.Xaml.RoutedEventArgs],
        ctors: {
            /**
             * Initializes a new instance of the TextChangedEventArgs class.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Controls.TextChangedEventArgs
             * @memberof Windows.UI.Xaml.Controls.TextChangedEventArgs
             * @return  {void}
             */
            ctor: function () {
                this.$initialize();
                Windows.UI.Xaml.RoutedEventArgs.ctor.call(this);
            }
        }
    });

    /**
     * Provides information for the System.Windows.Controls.Validation.Error attached
     event.
     * // Invokes the specified handler in a type-specific way on the specified object.
    //
     *
     * @public
     * @class Windows.UI.Xaml.Controls.ValidationErrorEventArgs
     * @augments Windows.UI.Xaml.RoutedEventArgs
     */
    Bridge.define("Windows.UI.Xaml.Controls.ValidationErrorEventArgs", {
        inherits: [Windows.UI.Xaml.RoutedEventArgs],
        fields: {
            _action: 0,
            _error: null
        },
        props: {
            /**
             * Gets a value that indicates whether the error is a new error or an existing
             error that has now been cleared.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.ValidationErrorEventArgs
             * @function Action
             * @type Windows.UI.Xaml.Controls.ValidationErrorEventAction
             */
            Action: {
                get: function () {
                    return this._action;
                },
                set: function (value) {
                    this._action = value;
                }
            },
            /**
             * Gets the error that caused this System.Windows.Controls.Validation.Error
             event.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.ValidationErrorEventArgs
             * @function Error
             * @type Windows.UI.Xaml.Controls.ValidationError
             */
            Error: {
                get: function () {
                    return this._error;
                },
                set: function (value) {
                    this._error = value;
                }
            }
        }
    });

    /**
     * Provides an abstract base class for the Binding class.
     *
     * @public
     * @class Windows.UI.Xaml.Data.BindingBase
     * @augments System.Windows.Markup.MarkupExtension
     */
    Bridge.define("Windows.UI.Xaml.Data.BindingBase", {
        inherits: [System.Windows.Markup.MarkupExtension],
        fields: {
            _fallbackValue: null,
            _targetNullValue: null,
            _stringFormat: null
        },
        props: {
            /**
             * Gets or sets the value to use when the binding is unable to return a value.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Data.BindingBase
             * @function FallbackValue
             * @type System.Object
             */
            FallbackValue: {
                get: function () {
                    return this._fallbackValue;
                },
                set: function (value) {
                    this._fallbackValue = value;
                }
            },
            /**
             * Gets or sets the value that is used in the target when the value of the source
             is null.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Data.BindingBase
             * @function TargetNullValue
             * @type System.Object
             */
            TargetNullValue: {
                get: function () {
                    return this._targetNullValue;
                },
                set: function (value) {
                    this._targetNullValue = value;
                }
            },
            /**
             * Gets or sets a string that specifies how to format the binding if it displays 
             the bound value as a string.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Data.BindingBase
             * @function StringFormat
             * @type string
             */
            StringFormat: {
                get: function () {
                    return this._stringFormat;
                },
                set: function (value) {
                    this._stringFormat = value;
                }
            }
        },
        ctors: {
            /**
             * Initializes a new instance of the BindingBase class.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Data.BindingBase
             * @memberof Windows.UI.Xaml.Data.BindingBase
             * @return  {void}
             */
            ctor: function () {
                this.$initialize();
                System.Windows.Markup.MarkupExtension.ctor.call(this);
            }
        },
        methods: {
            /**
             * @instance
             * @public
             * @override
             * @this Windows.UI.Xaml.Data.BindingBase
             * @memberof Windows.UI.Xaml.Data.BindingBase
             * @param   {System.ServiceProvider}    serviceProvider
             * @return  {System.Object}
             */
            ProvideValue: function (serviceProvider) {
                return this;
            }
        }
    });

    /**
     * Contains information about a single instance of a System.Windows.Data.Binding.
     *
     * @public
     * @class Windows.UI.Xaml.Data.BindingExpression
     * @implements  Windows.UI.Xaml.Data.IPropertyPathWalkerListener
     */
    Bridge.define("Windows.UI.Xaml.Data.BindingExpression", {
        inherits: [Windows.UI.Xaml.Data.IPropertyPathWalkerListener],
        statics: {
            fields: {
                NumericTypes: null
            },
            methods: {
                AreNumericTypes: function (type, obj) {
                    if (Windows.UI.Xaml.Data.BindingExpression.NumericTypes == null) {
                        Windows.UI.Xaml.Data.BindingExpression.NumericTypes = function (_o1) {
                                _o1.add(System.Byte);
                                _o1.add(System.SByte);
                                _o1.add(System.UInt16);
                                _o1.add(System.UInt32);
                                _o1.add(System.UInt64);
                                _o1.add(System.Int16);
                                _o1.add(System.Int32);
                                _o1.add(System.Int64);
                                _o1.add(System.Decimal);
                                _o1.add(System.Double);
                                _o1.add(System.Single);
                                _o1.add(System.Nullable$1(System.Byte));
                                _o1.add(System.Nullable$1(System.SByte));
                                _o1.add(System.Nullable$1(System.UInt16));
                                _o1.add(System.Nullable$1(System.UInt32));
                                _o1.add(System.Nullable$1(System.UInt64));
                                _o1.add(System.Nullable$1(System.Int16));
                                _o1.add(System.Nullable$1(System.Int32));
                                _o1.add(System.Nullable$1(System.Int64));
                                _o1.add(System.Nullable$1(System.Decimal));
                                _o1.add(System.Nullable$1(System.Double));
                                _o1.add(System.Nullable$1(System.Single));
                                return _o1;
                            }(new (System.Collections.Generic.HashSet2$1(System.Type)).ctor());
                    }
                    if (type != null && Windows.UI.Xaml.Data.BindingExpression.NumericTypes.contains(type) && obj != null && Windows.UI.Xaml.Data.BindingExpression.NumericTypes.contains(Bridge.getType(obj))) {
                        return true;
                    } else {
                        return false;
                    }
                },
                ConvertValueIfNecessary: function (value, targetType) {
                    if (Bridge.is(value, System.String) && !Bridge.referenceEquals(targetType, System.String)) {
                        try {
                            value = Bridge.global.DotNetForHtml5.Core.TypeFromStringConverters.ConvertFromInvariantString(targetType, Bridge.cast(value, System.String));
                        } catch (ex) {
                            ex = System.Exception.create(ex);
                            if (Windows.UI.Xaml.Application.Current.Host.Settings.EnableBindingErrorsLogging) {
                                System.Diagnostics.Debug.WriteLine$2(Bridge.toString(ex));
                            }
                            if (Windows.UI.Xaml.Application.Current.Host.Settings.EnableBindingErrorsThrowing) {
                                throw ex;
                            }
                        }
                    }

                    if (Bridge.is(value, Windows.UI.Xaml.Media.SolidColorBrush) && Bridge.referenceEquals(targetType, Windows.UI.Color)) {
                        value = Bridge.cast(value, Windows.UI.Xaml.Media.SolidColorBrush).Color.$clone();
                    }

                    return value;
                }
            }
        },
        fields: {
            IsUpdating: false,
            IsAttached: false,
            /**
             * The binding target property of this binding expression.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Data.BindingExpression
             * @function TargetProperty
             * @type Windows.UI.Xaml.DependencyProperty
             */
            TargetProperty: null,
            /**
             * The System.Windows.Data.Binding object of the current System.Windows.Data.BindingExpression.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Data.BindingExpression
             * @function ParentBinding
             * @type Windows.UI.Xaml.Data.Binding
             */
            ParentBinding: null,
            /**
             * The element that is the binding target object of this binding expression.
             *
             * @instance
             * @memberof Windows.UI.Xaml.Data.BindingExpression
             * @function Target
             * @type Windows.UI.Xaml.DependencyObject
             */
            Target: null,
            propertyPathWalker: null,
            _propertyListener: null,
            computedPath: null,
            isDataContextBound: false,
            _bindingSource: null,
            INTERNAL_ForceValidateOnNextSetValue: false,
            _isAttaching: false
        },
        props: {
            /**
             * Gets the binding source object that this System.Windows.Data.BindingExpression
             uses.
             *
             * @instance
             * @public
             * @readonly
             * @memberof Windows.UI.Xaml.Data.BindingExpression
             * @function DataItem
             * @type System.Object
             */
            DataItem: {
                get: function () {
                    var sourceFE;
                    if (this.ParentBinding.ElementName == null && this.ParentBinding.Source == null && this.ParentBinding.RelativeSource == null && ((sourceFE = Bridge.as(this._bindingSource, Windows.UI.Xaml.FrameworkElement))) != null) {
                        return sourceFE.DataContext;
                    }
                    return this._bindingSource;
                }
            }
        },
        ctors: {
            init: function () {
                this.INTERNAL_ForceValidateOnNextSetValue = false;
            },
            ctor: function (binding, target, property) {
                Windows.UI.Xaml.Data.BindingExpression.$ctor1.call(this, binding, property);
                this.Target = target;
            },
            $ctor1: function (binding, property) {
                this.$initialize();
                this.ParentBinding = binding;
                this.TargetProperty = property;

                if (binding.ElementName == null && binding.Source == null && binding.RelativeSource == null) {
                    this.isDataContextBound = true;

                    var str = this.ParentBinding.Path.Path;
                    if (!System.String.isNullOrWhiteSpace(str) && !(Bridge.referenceEquals(((str = str.trim())), "."))) {
                        this.computedPath = "DataContext." + (str || "");
                    } else {
                        this.computedPath = "DataContext";
                    }
                } else {
                    this.computedPath = this.ParentBinding.Path != null ? this.ParentBinding.Path.Path : "";
                }

                var walker = (this.propertyPathWalker = new Windows.UI.Xaml.Data.PropertyPathWalker(this.computedPath, false));
                if (binding.Mode !== Windows.UI.Xaml.Data.BindingMode.OneTime) {
                    walker.Listen(this);
                }

            }
        },
        methods: {
            IsBrokenChanged: function () {
                this.Refresh();
            },
            Windows$UI$Xaml$Data$IPropertyPathWalkerListener$IsBrokenChanged: function () {
                this.IsBrokenChanged();
            },
            ValueChanged: function () {
                this.Refresh();
            },
            Windows$UI$Xaml$Data$IPropertyPathWalkerListener$ValueChanged: function () {
                this.ValueChanged();
            },
            GetValue: function (property, OwnerType) {
                if (OwnerType === void 0) { OwnerType = null; }
                var value;

                if (this.propertyPathWalker.IsPathBroken) {

                    var typeMetadata = null;
                    if (OwnerType == null) {
                        typeMetadata = property.GetTypeMetaData(property.OwnerType);
                    } else {
                        typeMetadata = property.GetTypeMetaData(OwnerType);
                    }

                    value = this.ParentBinding.FallbackValue;

                    if (value != null) {
                        value = Windows.UI.Xaml.Data.BindingExpression.ConvertValueIfNecessary(value, property.PropertyType);
                    } else {
                        if (typeMetadata != null) {
                            value = typeMetadata.DefaultValue;
                        }
                    }

                    return value;
                } else {

                    value = this.propertyPathWalker.ValueInternal;


                    if (this.ParentBinding.Converter != null) {
                        value = this.ParentBinding.Converter.Windows$UI$Xaml$Data$IValueConverter$Convert(value, property.PropertyType, this.ParentBinding.ConverterParameter, this.ParentBinding.ConverterLanguage);
                    }

                    value = Windows.UI.Xaml.Data.BindingExpression.ConvertValueIfNecessary(value, property.PropertyType);

                    if (this.ParentBinding.StringFormat != null) {
                        value = System.String.format(this.ParentBinding.StringFormat, [value]);
                    }

                    if (value == null) {
                        value = this.ParentBinding.TargetNullValue;
                    }

                    var propertyType = property.PropertyType;
                    if (!Bridge.referenceEquals(propertyType, System.String)) {
                        if (value == null) {
                            if (Bridge.Reflection.isValueType(propertyType) && !System.String.startsWith(Bridge.Reflection.getTypeFullName(propertyType), "System.Nullable`1")) {
                                value = property.GetTypeMetaData(OwnerType).DefaultValue;
                            }
                        } else {
                            var nonNullableMemberType = propertyType;
                            if (System.String.startsWith(Bridge.Reflection.getTypeFullName(propertyType), "System.Nullable`1")) {
                                nonNullableMemberType = System.Nullable.getUnderlyingType(propertyType);
                            }
                            if (!Windows.UI.Xaml.Data.BindingExpression.AreNumericTypes(nonNullableMemberType, value)) {
                                var valueType = Bridge.getType(value);
                                if (!(Bridge.referenceEquals(Bridge.Reflection.getTypeName(valueType), "Array") && Bridge.Reflection.isAssignableFrom(System.Collections.IEnumerable, nonNullableMemberType))) {
                                    if (!Bridge.Reflection.isAssignableFrom(nonNullableMemberType, valueType)) {
                                        value = property.GetTypeMetaData(OwnerType).DefaultValue;
                                    }
                                }
                            }
                        }
                    }

                }

                return value;
            },
            OnAttached: function (target) {
                if (this.IsAttached) {
                    return;
                }

                this._isAttaching = (this.IsAttached = true);

                this.Target = target;

                this.FindSource();

                this.propertyPathWalker.Update(this._bindingSource);

                if (this.ParentBinding.Mode === Windows.UI.Xaml.Data.BindingMode.TwoWay) {
                    this._propertyListener = CSHTML5.Internal.INTERNAL_PropertyStore.ListenToChanged(this.Target, this.TargetProperty, Bridge.fn.cacheBind(this, this.UpdateSourceCallback));

                    if (this.ParentBinding.ValidatesOnExceptions && this.ParentBinding.ValidatesOnLoad) {
                        this.INTERNAL_ForceValidateOnNextSetValue = true;
                    }
                }

                this._isAttaching = false;
            },
            /**
             * This method is used to check whether the value is Valid if needed.
             *
             * @instance
             * @this Windows.UI.Xaml.Data.BindingExpression
             * @memberof Windows.UI.Xaml.Data.BindingExpression
             * @param   {System.Object}    initialValue
             * @return  {void}
             */
            CheckInitialValueValidity: function (initialValue) {
                var $t;
                if (this.ParentBinding.ValidatesOnExceptions && this.ParentBinding.ValidatesOnLoad) {
                    if (!this.propertyPathWalker.IsPathBroken) {
                        this.INTERNAL_ForceValidateOnNextSetValue = false;
                        try {
                            var node = Bridge.cast(this.propertyPathWalker.FinalNode, Windows.UI.Xaml.Data.PropertyPathNode);
                            node.SetValue(node.Value);
                        } catch (e) {
                            e = System.Exception.create(e);
                            var currentException = e;

                            if (CSHTML5.Interop.IsRunningInTheSimulator) {
                                while (currentException.InnerException != null) {
                                    currentException = currentException.InnerException;
                                }
                            }

                            Windows.UI.Xaml.Controls.Validation.MarkInvalid(this, ($t = new Windows.UI.Xaml.Controls.ValidationError(this), $t.Exception = currentException, $t.ErrorContent = currentException.Message, $t));
                        }
                    }
                }
            },
            OnDetached: function (element) {
                if (this.IsAttached) {
                    this.IsAttached = false;

                    if (this._propertyListener != null) {
                        this._propertyListener.Windows$UI$Xaml$Data$IPropertyChangedListener$Detach();
                        this._propertyListener = null;
                    }

                    this.propertyPathWalker.Update(null);

                    this.Target.removeInheritedContextChanged(Bridge.fn.cacheBind(this, this.OnTargetInheritedContextChanged));
                    this.Target = null;
                }
            },
            OnSourceAvailable: function () {
                this.FindSource();
                if (this._bindingSource != null) {
                    this.propertyPathWalker.Update(this._bindingSource);
                }



            },
            OnTargetInheritedContextChanged: function (sender, e) {
                this.Target.removeInheritedContextChanged(Bridge.fn.cacheBind(this, this.OnTargetInheritedContextChanged));
                this.OnSourceAvailable();
            },
            FindSource: function () {
                if (this.isDataContextBound) {
                    var targetFE;
                    if (((targetFE = Bridge.as(this.Target, Windows.UI.Xaml.FrameworkElement))) != null) {
                        var contextElement = targetFE;

                        this._bindingSource = targetFE;
                    } else {
                        this.Target.addInheritedContextChanged(Bridge.fn.cacheBind(this, this.OnTargetInheritedContextChanged));
                        this._bindingSource = this.Target.InheritedParent;
                    }
                } else if (this.ParentBinding.Source != null) {
                    this._bindingSource = this.ParentBinding.Source;
                } else if (this.ParentBinding.ElementName != null) {
                    var targetFE1;
                    if (((targetFE1 = Bridge.as(this.Target, Windows.UI.Xaml.FrameworkElement))) != null) {
                        this._bindingSource = targetFE1.FindName(this.ParentBinding.ElementName);
                    } else {
                        this._bindingSource = null;
                    }
                } else if (this.ParentBinding.RelativeSource != null) {
                    var relativeSource = this.ParentBinding.RelativeSource;
                    switch (relativeSource.Mode) {
                        case Windows.UI.Xaml.Data.RelativeSourceMode.Self: 
                            this._bindingSource = this.Target;
                            break;
                        case Windows.UI.Xaml.Data.RelativeSourceMode.TemplatedParent: 
                            if (this.ParentBinding.TemplateOwner != null) {
                                this._bindingSource = this.ParentBinding.TemplateOwner.TemplateOwner;
                            } else {
                                System.Diagnostics.Debug.WriteLine$2("ERROR: ParentBinding.TemplateOwner is null.");
                                this._bindingSource = null;
                            }
                            break;
                        case Windows.UI.Xaml.Data.RelativeSourceMode.FindAncestor: 
                            this._bindingSource = this.FindAncestor(this.Target, relativeSource);
                            break;
                    }
                }
            },
            FindAncestor: function (target, relativeSource) {
                var uiE;
                if (!(((uiE = Bridge.as(target, Windows.UI.Xaml.UIElement))) != null)) {
                    return null;
                }

                if (!CSHTML5.Internal.INTERNAL_VisualTreeManager.IsElementInVisualTree(uiE)) {
                    return null;
                }

                var ancestorLevel = relativeSource.AncestorLevel;
                var ancestorType = relativeSource.AncestorType;
                if (ancestorLevel < 1 || ancestorType == null) {
                    return null;
                }

                var currentParent = Bridge.cast(uiE.INTERNAL_VisualParent, Windows.UI.Xaml.UIElement);
                while (!Bridge.Reflection.isAssignableFrom(ancestorType, Bridge.getType(currentParent)) || ((ancestorLevel = (ancestorLevel - 1) | 0)) > 0) {
                    currentParent = Bridge.cast(currentParent.INTERNAL_VisualParent, Windows.UI.Xaml.UIElement);
                    if (currentParent == null) {
                        return null;
                    }
                }
                if (ancestorLevel === 0) {
                    return currentParent;
                }
                return null;
            },
            UpdateSourceCallback: function (sender, args) {
                try {
                    if (!this.IsUpdating && this.ParentBinding.UpdateSourceTrigger !== Windows.UI.Xaml.Data.UpdateSourceTrigger.Explicit) {
                        this.UpdateSourceObject(this.Target.GetValue(this.TargetProperty));
                    }
                } catch (err) {
                    err = System.Exception.create(err);
                    System.Console.WriteLine("[BINDING] UpdateSource: " + (Bridge.toString(err) || ""));
                }
            },
            TryUpdateSourceObject: function (value) {
                if (!this.IsUpdating && this.ParentBinding.UpdateSourceTrigger !== Windows.UI.Xaml.Data.UpdateSourceTrigger.Explicit) {
                    this.UpdateSourceObject(value);
                }
            },
            UpdateSourceObject: function (value) {
                var $t;
                if (!this.propertyPathWalker.IsPathBroken) {
                    var node = Bridge.cast(this.propertyPathWalker.FinalNode, Windows.UI.Xaml.Data.PropertyPathNode);
                    var oldIsUpdating = this.IsUpdating;

                    try {
                        var expectedType = null;
                        var convertedValue = value;
                        if (this.ParentBinding.Converter != null) {
                            if (node.DependencyProperty != null) {
                                convertedValue = this.ParentBinding.Converter.Windows$UI$Xaml$Data$IValueConverter$ConvertBack(value, node.DependencyProperty.PropertyType, this.ParentBinding.ConverterParameter, this.ParentBinding.ConverterLanguage);
                                expectedType = node.DependencyProperty.PropertyType;
                            } else if (node.PropertyInfo != null) {
                                convertedValue = this.ParentBinding.Converter.Windows$UI$Xaml$Data$IValueConverter$ConvertBack(value, node.PropertyInfo.rt, this.ParentBinding.ConverterParameter, this.ParentBinding.ConverterLanguage);
                                expectedType = node.PropertyInfo.rt;
                            } else if (node.FieldInfo != null) {
                                convertedValue = this.ParentBinding.Converter.Windows$UI$Xaml$Data$IValueConverter$ConvertBack(value, node.FieldInfo.rt, this.ParentBinding.ConverterParameter, this.ParentBinding.ConverterLanguage);
                                expectedType = node.FieldInfo.rt;
                            }
                        } else {
                            if (node.DependencyProperty != null) {
                                expectedType = node.DependencyProperty.PropertyType;
                            } else if (node.PropertyInfo != null) {
                                expectedType = node.PropertyInfo.rt;
                            } else if (node.FieldInfo != null) {
                                expectedType = node.FieldInfo.rt;
                            }
                        }

                        var typeAcceptsNullAsValue = !Bridge.Reflection.isValueType(expectedType) || System.String.startsWith(Bridge.Reflection.getTypeFullName(expectedType), "System.Nullable`1");
                        var isNotNullOrIsNullAccepted = ((convertedValue != null) || typeAcceptsNullAsValue);
                        if (isNotNullOrIsNullAccepted) {
                            this.IsUpdating = true;
                            var AutoConvertTypes = System.Array.init([
                                System.Int16, 
                                System.Int32, 
                                System.Int64, 
                                System.Double, 
                                System.Uri
                            ], System.Type);
                            var typeFound = false;
                            if ((System.Array.contains(AutoConvertTypes, expectedType, System.Type)) && Bridge.is(convertedValue, System.String)) {
                                typeFound = true;
                                if (Bridge.referenceEquals(expectedType, System.Int16)) {
                                    convertedValue = Bridge.box(System.Int16.parse(Bridge.cast(convertedValue, System.String)), System.Int16);
                                } else if (Bridge.referenceEquals(expectedType, System.Int32)) {
                                    convertedValue = Bridge.box(System.Int32.parse(Bridge.cast(convertedValue, System.String)), System.Int32);
                                } else if (Bridge.referenceEquals(expectedType, System.Int64)) {
                                    convertedValue = System.Int64.parse(Bridge.cast(convertedValue, System.String));
                                } else if (Bridge.referenceEquals(expectedType, System.Double)) {
                                    convertedValue = Bridge.box(System.Double.parse(Bridge.cast(convertedValue, System.String)), System.Double, System.Double.format, System.Double.getHashCode);
                                } else if (Bridge.referenceEquals(expectedType, System.Uri)) {
                                    convertedValue = new System.Uri(Bridge.cast(convertedValue, System.String));
                                }
                            }

                            if ((!typeFound && DotNetForHtml5.Core.TypeFromStringConverters.CanTypeBeConverted(expectedType)) && Bridge.is(convertedValue, System.String)) {
                                typeFound = true;
                                convertedValue = DotNetForHtml5.Core.TypeFromStringConverters.ConvertFromInvariantString(expectedType, Bridge.cast(convertedValue, System.String));
                            }

                            if (!typeFound && convertedValue != null && (Bridge.referenceEquals(expectedType, System.String))) {
                                convertedValue = Bridge.toString(convertedValue);
                            }

                            node.SetValue(convertedValue);
                            Windows.UI.Xaml.Controls.Validation.ClearInvalid(this);
                        }
                    } catch (e) {
                        e = System.Exception.create(e);
                        if (this.ParentBinding.ValidatesOnExceptions) {
                            var currentException = e;

                            if (CSHTML5.Interop.IsRunningInTheSimulator) {
                                while (currentException.InnerException != null) {
                                    currentException = currentException.InnerException;
                                }
                            }

                            Windows.UI.Xaml.Controls.Validation.MarkInvalid(this, ($t = new Windows.UI.Xaml.Controls.ValidationError(this), $t.Exception = currentException, $t.ErrorContent = currentException.Message, $t));
                        }
                    } finally {
                        this.IsUpdating = oldIsUpdating;
                    }
                }
            },
            /**
             * Sends the current binding target value to the binding source property in
             System.Windows.Data.BindingMode.TwoWay or System.Windows.Data.BindingMode.OneWayToSource
             bindings.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Data.BindingExpression
             * @memberof Windows.UI.Xaml.Data.BindingExpression
             * @return  {void}
             */
            UpdateSource: function () {
                if (!this.IsAttached) {
                    throw new System.InvalidOperationException.$ctor1("The Binding has been detached from its target.");
                }
                if (!this.IsUpdating && this.ParentBinding.Mode === Windows.UI.Xaml.Data.BindingMode.TwoWay) {
                    this.UpdateSourceObject(this.Target.GetValue(this.TargetProperty));
                }
            },
            Refresh: function () {
                if (this._isAttaching) {
                    return;
                }
                if (this.IsAttached) {
                    var oldIsUpdating = this.IsUpdating;
                    this.IsUpdating = true;
                    this.Target.ApplyBindingExpression(this.TargetProperty, this);

                    this.IsUpdating = oldIsUpdating;
                }
            }
        }
    });

    Bridge.define("Windows.UI.Xaml.Data.PropertyPathNode", {
        inherits: [Windows.UI.Xaml.Data.IPropertyPathNode],
        fields: {
            _next: null,
            _isBroken: false,
            _source: null,
            _value: null,
            DependencyProperty: null,
            PropertyInfo: null,
            FieldInfo: null,
            _nodeListener: null
        },
        props: {
            IsBroken: {
                get: function () {
                    return this._isBroken;
                }
            },
            Source: {
                get: function () {
                    return this._source;
                }
            },
            Value: {
                get: function () {
                    return this._value;
                }
            },
            Next: {
                get: function () {
                    return this._next;
                },
                set: function (value) {
                    this._next = value;
                }
            }
        },
        alias: [
            "IsBroken", "Windows$UI$Xaml$Data$IPropertyPathNode$IsBroken",
            "Value", "Windows$UI$Xaml$Data$IPropertyPathNode$Value",
            "Next", "Windows$UI$Xaml$Data$IPropertyPathNode$Next"
        ],
        methods: {
            Listen: function (listener) {
                this._nodeListener = listener;
            },
            Windows$UI$Xaml$Data$IPropertyPathNode$Listen: function (listener) {
                this.Listen(listener);
            },
            Unlisten: function (listener) {
                if (Bridge.referenceEquals(this._nodeListener, listener)) {
                    this._nodeListener = null;
                }
            },
            Windows$UI$Xaml$Data$IPropertyPathNode$Unlisten: function (listener) {
                this.Unlisten(listener);
            },
            OnSourceChanged: function (oldSource, newSource) { },
            OnSourcePropertyChanged: function (o, e) { },
            SetSource: function (value) {
                if (value == null || !Bridge.referenceEquals(value, this._source)) {
                    var oldSource = this._source;
                    var oldSourceAsINotifyPropertyChanged = Bridge.as(oldSource, System.ComponentModel.INotifyPropertyChanged);
                    if (oldSourceAsINotifyPropertyChanged != null) {
                        oldSourceAsINotifyPropertyChanged.System$ComponentModel$INotifyPropertyChanged$removePropertyChanged(Bridge.fn.cacheBind(this, this.OnSourcePropertyChanged));
                    }

                    this._source = value;
                    var newSourceAsINotifyPropertyChanged = Bridge.as(this._source, System.ComponentModel.INotifyPropertyChanged);
                    if (newSourceAsINotifyPropertyChanged != null) {
                        newSourceAsINotifyPropertyChanged.System$ComponentModel$INotifyPropertyChanged$addPropertyChanged(Bridge.fn.cacheBind(this, this.OnSourcePropertyChanged));
                    }

                    this.OnSourceChanged(oldSource, this._source);
                    this.UpdateValue();
                    if (this.Next != null) {
                        this.Next.Windows$UI$Xaml$Data$IPropertyPathNode$SetSource(this._value);
                    }
                }
            },
            Windows$UI$Xaml$Data$IPropertyPathNode$SetSource: function (source) {
                this.SetSource(source);
            },
            UpdateValueAndIsBroken: function (newValue, isBroken) {
                var emitBrokenChanged = this._isBroken !== isBroken;
                var emitValueChanged = !Bridge.referenceEquals(this.Value, newValue);

                this._isBroken = isBroken;
                this._value = newValue;

                if (emitValueChanged) {
                    var listener = this._nodeListener;
                    if (listener != null) {
                        listener.Windows$UI$Xaml$Data$IPropertyPathNodeListener$ValueChanged(this);
                    }
                } else if (emitBrokenChanged) {
                    var listener1 = this._nodeListener;
                    if (listener1 != null) {
                        listener1.Windows$UI$Xaml$Data$IPropertyPathNodeListener$IsBrokenChanged(this);
                    }
                }
            },
            CheckIsBroken: function (bindsDirectlyToSource) {
                if (bindsDirectlyToSource === void 0) { bindsDirectlyToSource = false; }
                return (this.Source == null || (!bindsDirectlyToSource && (this.PropertyInfo == null && this.FieldInfo == null && this.DependencyProperty == null)));
            },
            Windows$UI$Xaml$Data$IPropertyPathNode$SetValue: function (value) {
                this.SetValue(value);
            }
        }
    });

    /**
     * Allows to split a data source into multiple paged sources and to expose only the current page
     *
     * @class Windows.UI.Xaml.Data.INTERNAL_PagedCollectionView
     * @implements  System.Collections.IEnumerable
     * @implements  System.Collections.Specialized.INotifyCollectionChanged
     */
    Bridge.define("Windows.UI.Xaml.Data.INTERNAL_PagedCollectionView", {
        inherits: [System.Collections.IEnumerable,System.Collections.Specialized.INotifyCollectionChanged],
        fields: {
            _views: null,
            _pages: null,
            _collectionInteface: null,
            _originalDataSource: null,
            _originalDataSourceWithoutCopy: null,
            _operations: null,
            _avoidCollectionChangedEvent: false,
            _index: 0,
            _refreshPending: false,
            _filter: null,
            _PageIndex: 0,
            _pageSize: 0,
            GroupDescriptions: null,
            SortDescriptions: null,
            _isPageChanging: false
        },
        events: {
            CollectionChanged: null,
            PageChanged: null
        },
        props: {
            Index: {
                get: function () {
                    return this._index;
                },
                set: function (value) {
                    this._index = value;
                    this.ConstrainIndexToRemainInsideAllowedRange();

                    this.CreatePages();
                }
            },
            Filter: {
                get: function () {
                    return this._filter != null ? this._filter.FilterUsingAPredicate : null;
                },
                set: function (value) {
                    this._filter = new Windows.UI.Xaml.Data.FilterDescription.ctor(value);
                }
            },
            Groups: {
                get: function () {
                    var $t;
                    var Leaves = new (System.Collections.ObjectModel.Collection$1(System.Collections.IEnumerable)).ctor();

                    $t = Bridge.getEnumerator(this.GetLeaves());
                    try {
                        while ($t.moveNext()) {
                            var group = $t.Current;
                            Leaves.add(group.Items);
                        }
                    } finally {
                        if (Bridge.is($t, System.IDisposable)) {
                            $t.System$IDisposable$Dispose();
                        }
                    }

                    return Leaves;
                }
            },
            Count: {
                get: function () {
                    return System.Array.getCount(this._pages.getItem(this._PageIndex), System.Object);
                }
            },
            TotalItemCount: {
                get: function () {
                    if (this.Index !== -1 && this._originalDataSource != null) {
                        var leaves = this.GetLeaves();
                        if (leaves.Count > this.Index) {
                            return System.Array.getCount(leaves.getItem(this.Index).Items, System.Object);
                        } else {
                            return 0;
                        }
                    } else {
                        return 0;
                    }
                }
            },
            PageIndex: {
                get: function () {
                    return this._PageIndex;
                },
                set: function (value) {
                    var newPageIndex = this.VerifyPageIndex(value);
                    if (newPageIndex !== this._PageIndex) {
                        var oldCount = this.Count;
                        this._PageIndex = newPageIndex;
                        this.OnPageChanged();

                    }
                }
            },
            PageSize: {
                get: function () {
                    return this._pageSize;
                },
                set: function (value) {
                    var oldCount = this.Count;
                    this._pageSize = value;
                    if (this._views.Count !== 0) {
                        this.CreatePages();
                    }
                }
            },
            SourceCollection: {
                get: function () {
                    return this._originalDataSource;
                }
            },
            CanChangePage: {
                get: function () {
                    return true;
                }
            },
            IsPageChanging: {
                get: function () {
                    return this._isPageChanging;
                }
            },
            ItemCount: {
                get: function () {
                    return this.TotalItemCount;
                }
            }
        },
        alias: [
            "addCollectionChanged", "System$Collections$Specialized$INotifyCollectionChanged$addCollectionChanged",
            "removeCollectionChanged", "System$Collections$Specialized$INotifyCollectionChanged$removeCollectionChanged",
            "GetEnumerator", "System$Collections$IEnumerable$GetEnumerator"
        ],
        ctors: {
            init: function () {
                this._views = new (System.Collections.Generic.List$1(Windows.UI.Xaml.Data.CollectionViewGroupInternal)).ctor();
                this._pages = new (System.Collections.Generic.List$1(System.Collections.Generic.ICollection$1(System.Object))).ctor();
                this._collectionInteface = new (System.Collections.ObjectModel.Collection$1(System.Object)).ctor();
                this._index = 0;
                this._refreshPending = false;
                this._PageIndex = 0;
                this._pageSize = -1;
            },
            ctor: function (source) {
                this.$initialize();
                this._originalDataSource = source != null ? source : new (System.Collections.ObjectModel.Collection$1(System.Object)).ctor();

                this.Init();

                this.Refresh();
            }
        },
        methods: {
            AddView: function (view) {
                this._views.add(view);
            },
            GetViews: function () {
                return this._views.GetEnumerator().$clone();
            },
            SetIEnumerableAsCollectionSource: function (newSource, copySource) {
                if (copySource === void 0) { copySource = false; }
                this._originalDataSourceWithoutCopy = newSource;

                if (!(Bridge.is(newSource, Windows.UI.Xaml.Data.INTERNAL_PagedCollectionView))) {
                    this._avoidCollectionChangedEvent = true;

                    if (Bridge.is(this._originalDataSource, System.Collections.Specialized.INotifyCollectionChanged)) {
                        Bridge.cast(this._originalDataSource, System.Collections.Specialized.INotifyCollectionChanged).System$Collections$Specialized$INotifyCollectionChanged$removeCollectionChanged(Bridge.fn.cacheBind(this, this.Source_CollectionChanged));
                    }

                    this._originalDataSource = copySource ? this.CopySource(newSource) : newSource;

                    if (this._originalDataSource != null) {
                        if (Bridge.is(this._originalDataSource, System.Collections.Specialized.INotifyCollectionChanged)) {
                            Bridge.cast(this._originalDataSource, System.Collections.Specialized.INotifyCollectionChanged).System$Collections$Specialized$INotifyCollectionChanged$addCollectionChanged(Bridge.fn.cacheBind(this, this.Source_CollectionChanged));
                        }

                        this.Refresh();
                    } else {
                        this._views.clear();
                        this._pages.clear();
                    }

                    this._avoidCollectionChangedEvent = false;

                }
            },
            GetLeaves: function () {
                var $t;
                var leaves = new (System.Collections.Generic.List$1(Windows.UI.Xaml.Data.CollectionViewGroupInternal)).ctor();

                $t = Bridge.getEnumerator(this._views);
                try {
                    while ($t.moveNext()) {
                        var view = $t.Current;
                        if (view.IsLeaf) {
                            leaves.add(view);
                        }
                    }
                } finally {
                    if (Bridge.is($t, System.IDisposable)) {
                        $t.System$IDisposable$Dispose();
                    }
                }

                return leaves;
            },
            CopySource: function (source) {
                var $t;
                if (source != null) {
                    var newSource;

                    if (Bridge.is(source, System.Collections.Specialized.INotifyCollectionChanged)) {
                        newSource = new (System.Collections.ObjectModel.ObservableCollection$1(System.Object)).ctor();
                    } else {
                        newSource = new (System.Collections.ObjectModel.Collection$1(System.Object)).ctor();
                    }

                    $t = Bridge.getEnumerator(source);
                    try {
                        while ($t.moveNext()) {
                            var item = $t.Current;
                            newSource.add(item);
                        }
                    } finally {
                        if (Bridge.is($t, System.IDisposable)) {
                            $t.System$IDisposable$Dispose();
                        }
                    }

                    return newSource;
                }

                return null;
            },
            GetDefaultView: function () {
                return this._views.Count === 0 ? null : this._views.getItem(((this._views.Count - 1) | 0));
            },
            GetFilterView: function () {
                if ((!Bridge.staticEquals(this._filter.FilterUsingAnEvent, null) || !Bridge.staticEquals(this._filter.FilterUsingAPredicate, null)) && this._operations.HasFilteringBeenDone() && this._views.Count >= 2) {
                    return this._views.getItem(((this._views.Count - 2) | 0));
                }

                return null;
            },
            Init: function () {
                this.GroupDescriptions = new (System.Collections.ObjectModel.ObservableCollection$1(Windows.UI.Xaml.Data.PropertyGroupDescription)).ctor();
                this.SortDescriptions = new (System.Collections.ObjectModel.ObservableCollection$1(Windows.UI.Xaml.Data.PropertySortDescription)).ctor();

                this.GroupDescriptions.addCollectionChanged(Bridge.fn.cacheBind(this, this.OnGroupingChanged));
                this.SortDescriptions.addCollectionChanged(Bridge.fn.cacheBind(this, this.OnSortingChanged));

                if (Bridge.is(this._originalDataSource, System.Collections.Specialized.INotifyCollectionChanged)) {
                    Bridge.cast(this._originalDataSource, System.Collections.Specialized.INotifyCollectionChanged).System$Collections$Specialized$INotifyCollectionChanged$addCollectionChanged(Bridge.fn.cacheBind(this, this.Source_CollectionChanged));
                }
            },
            ConstrainIndexToRemainInsideAllowedRange: function () {
                var leaves = this.GetLeaves();
                if (leaves.Count === 0) {
                    this._index = -1;
                }

                if (this._index >= leaves.Count) {
                    this._index = (leaves.Count - 1) | 0;
                }
            },
            CreateTree: function () {
                var $t;
                this._views.clear();

                this._operations = new Windows.UI.Xaml.Data.INTERNAL_Operations(this);

                var newCollection = new (System.Collections.ObjectModel.Collection$1(System.Object)).ctor();

                $t = Bridge.getEnumerator(this._originalDataSource);
                try {
                    while ($t.moveNext()) {
                        var obj = $t.Current;
                        System.Array.add(newCollection, obj, System.Object);
                    }
                } finally {
                    if (Bridge.is($t, System.IDisposable)) {
                        $t.System$IDisposable$Dispose();
                    }
                }

                var treeRoot = new Windows.UI.Xaml.Data.CollectionViewGroupInternal(System.Linq.Enumerable.from(newCollection).toList(System.Object), null, this._operations);

                this._views.add(treeRoot);

                this.ConstrainIndexToRemainInsideAllowedRange();
            },
            CreatePages: function () {
                var $t;
                this._avoidCollectionChangedEvent = true;

                this._pages.clear();

                if (this.Index !== -1) {
                    var numberOfElementsInPage = 0;

                    var page = new (System.Collections.ObjectModel.Collection$1(System.Object)).ctor();

                    $t = Bridge.getEnumerator(this.GetLeaves().getItem(this.Index).Items, System.Object);
                    try {
                        while ($t.moveNext()) {
                            var item = $t.Current;
                            if (numberOfElementsInPage === 0) {
                                page = new (System.Collections.ObjectModel.Collection$1(System.Object)).ctor();
                            }

                            numberOfElementsInPage = (numberOfElementsInPage + 1) | 0;
                            page.add(item);

                            if (numberOfElementsInPage >= this.PageSize && this.PageSize !== -1) {
                                numberOfElementsInPage = 0;
                                this._pages.add(page);
                            }
                        }
                    } finally {
                        if (Bridge.is($t, System.IDisposable)) {
                            $t.System$IDisposable$Dispose();
                        }
                    }

                    if (this._pages.Count === 0 || !Bridge.referenceEquals(this._pages.getItem(((this._pages.Count - 1) | 0)), page)) {
                        this._pages.add(page);
                    }
                }

                this.PageIndex = this.VerifyPageIndex(this.PageIndex);

                this._avoidCollectionChangedEvent = false;

                this.ChangeOutputColletion();

            },
            VerifyPageIndex: function (indexRequested) {
                if (indexRequested >= this._pages.Count) {
                    return ((this._pages.Count - 1) | 0);
                }

                if (indexRequested < 0) {
                    return 0;
                }

                return indexRequested;
            },
            ChangeOutputColletion: function () {
                var $t;
                if (!this._avoidCollectionChangedEvent) {
                    var oldItems = this._collectionInteface;

                    this._collectionInteface = new (System.Collections.ObjectModel.Collection$1(System.Object)).ctor();

                    if (this._pages.Count !== 0) {
                        $t = Bridge.getEnumerator(this._pages.getItem(this._PageIndex), System.Object);
                        try {
                            while ($t.moveNext()) {
                                var item = $t.Current;
                                this._collectionInteface.add(item);
                            }
                        } finally {
                            if (Bridge.is($t, System.IDisposable)) {
                                $t.System$IDisposable$Dispose();
                            }
                        }
                    }

                    if (!Bridge.staticEquals(this.CollectionChanged, null)) {
                        this.CollectionChanged(this, new System.Collections.Specialized.NotifyCollectionChangedEventArgs.ctor(System.Collections.Specialized.NotifyCollectionChangedAction.Reset));
                    }
                }
            },
            Source_CollectionChanged: function (sender, e) {

                this.ScheduleRefresh();
            },
            OnGroupingChanged: function (sender, e) {
                this.ScheduleRefresh();
            },
            OnSortingChanged: function (sender, e) {
                this.ScheduleRefresh();
            },
            Refresh: function () {
                this.CreateTree();

                this.CreatePages();
            },
            ScheduleRefresh: function () {
                if (!this._refreshPending) {
                    this._refreshPending = true;
                    CSHTML5.Internal.INTERNAL_DispatcherHelpers.QueueAction(Bridge.fn.bind(this, function () {
                        this._refreshPending = false;

                        this.Refresh();
                    }));
                }
            },
            GetEnumerator: function () {
                return this._collectionInteface.GetEnumerator();
            },
            MoveToFirstPage: function () {
                return this.MoveToPage(0);
            },
            MoveToLastPage: function () {
                return this.MoveToPage(((this._pages.Count - 1) | 0));
            },
            MoveToNextPage: function () {
                return this.MoveToPage(((this.PageIndex + 1) | 0));
            },
            MoveToPreviousPage: function () {
                return this.MoveToPage(((this.PageIndex - 1) | 0));
            },
            MoveToPage: function (index) {
                if (this.PageIndex >= this._pages.Count) {
                    return false;
                }
                if (this.PageIndex < 0) {
                    return false;
                }

                this.PageIndex = index;

                return true;
            },
            PassesFilter: function (item) {
                var filterView = this.GetFilterView();
                if (filterView != null) {
                    return System.Array.contains(filterView.Items, item, System.Object);
                }
                return true;
            },
            OnPageChanged: function () {
                if (!Bridge.staticEquals(this.PageChanged, null)) {
                    this.PageChanged(this, { });
                }
                this.ChangeOutputColletion();
                this._isPageChanging = false;
            }
        }
    });

    Bridge.define("Windows.UI.Xaml.Data.PropertyChangedListener", {
        inherits: [Windows.UI.Xaml.Data.IPropertyChangedListener],
        fields: {
            Property: null,
            _storage: null,
            _sourceCallBack: null
        },
        alias: [
            "Property", "Windows$UI$Xaml$Data$IPropertyChangedListener$Property",
            "Detach", "Windows$UI$Xaml$Data$IPropertyChangedListener$Detach",
            "OnPropertyChanged", "Windows$UI$Xaml$Data$IPropertyChangedListener$OnPropertyChanged"
        ],
        ctors: {
            ctor: function (storage, sourceCallBack) {
                this.$initialize();
                this._storage = storage;
                this._sourceCallBack = sourceCallBack;
            }
        },
        methods: {
            Detach: function () {
                this._storage.PropertyListeners.remove(this);
            },
            OnPropertyChanged: function (sender, args) {
                this._sourceCallBack(sender, args);
            }
        }
    });

    Bridge.define("Windows.UI.Xaml.Data.PropertyPathWalker", {
        inherits: [Windows.UI.Xaml.Data.IPropertyPathNodeListener],
        fields: {
            Path: null,
            IsDataContextBound: false,
            Source: null,
            ValueInternal: null,
            FirstNode: null,
            FinalNode: null,
            _listener: null
        },
        props: {
            IsPathBroken: {
                get: function () {
                    var path = this.Path;
                    if (this.IsDataContextBound && (path == null || path.length < 1)) {
                        return false;
                    }

                    var node = this.FirstNode;
                    while (node != null) {
                        if (node.Windows$UI$Xaml$Data$IPropertyPathNode$IsBroken) {
                            return true;
                        }
                        node = node.Windows$UI$Xaml$Data$IPropertyPathNode$Next;
                    }
                    return false;
                }
            }
        },
        ctors: {
            ctor: function (path, isDatacontextBound) {
                this.$initialize();
                this.Path = path;
                var lastNode;
                this.IsDataContextBound = isDatacontextBound;

                if (System.String.isNullOrEmpty(path) || Bridge.referenceEquals(path, ".")) {
                    lastNode = new Windows.UI.Xaml.Data.StandardPropertyPathNode.ctor();
                    this.FirstNode = lastNode;
                    this.FinalNode = lastNode;
                } else {
                    var type = new CSHTML5.Internal.Windows.UI.Xaml.Data.PropertyNodeType();
                    var parser = new CSHTML5.Internal.Windows.UI.Xaml.Data.PropertyPathParser(path);
                    var typeName = { };
                    var propertyName = { };
                    var index = { };
                    while (((type = parser.Step(typeName, propertyName, index))) !== CSHTML5.Internal.Windows.UI.Xaml.Data.PropertyNodeType.None) {


                        switch (type) {
                            case CSHTML5.Internal.Windows.UI.Xaml.Data.PropertyNodeType.AttachedProperty: 
                            case CSHTML5.Internal.Windows.UI.Xaml.Data.PropertyNodeType.Property: 
                                if (this.FinalNode == null) {
                                    this.FinalNode = new Windows.UI.Xaml.Data.StandardPropertyPathNode.$ctor1(typeName.v, propertyName.v);
                                } else {
                                    this.FinalNode.Windows$UI$Xaml$Data$IPropertyPathNode$Next = new Windows.UI.Xaml.Data.StandardPropertyPathNode.$ctor1(typeName.v, propertyName.v);
                                }
                                break;
                            case CSHTML5.Internal.Windows.UI.Xaml.Data.PropertyNodeType.Indexed: 
                                this.FinalNode.Windows$UI$Xaml$Data$IPropertyPathNode$Next = new Windows.UI.Xaml.Data.IndexedPropertyPathNode(index.v);
                                break;
                            default: 
                                break;
                        }

                        if (this.FirstNode == null) {
                            this.FirstNode = this.FinalNode;
                        }

                        if (this.FinalNode.Windows$UI$Xaml$Data$IPropertyPathNode$Next != null) {
                            this.FinalNode = this.FinalNode.Windows$UI$Xaml$Data$IPropertyPathNode$Next;
                        }
                    }
                }

                this.FinalNode.Windows$UI$Xaml$Data$IPropertyPathNode$Listen(this);
            }
        },
        methods: {
            Listen: function (listener) {
                this._listener = listener;
            },
            Unlisten: function (listener) {
                if (Bridge.referenceEquals(this._listener, listener)) {
                    this._listener = null;
                }
            },
            Update: function (source) {
                this.Source = source;
                this.FirstNode.Windows$UI$Xaml$Data$IPropertyPathNode$SetSource(source);
            },
            Windows$UI$Xaml$Data$IPropertyPathNodeListener$IsBrokenChanged: function (node) {
                this.ValueInternal = node.Windows$UI$Xaml$Data$IPropertyPathNode$Value;
                var listener = this._listener;
                if (listener != null) {
                    listener.Windows$UI$Xaml$Data$IPropertyPathWalkerListener$IsBrokenChanged();
                }
            },
            Windows$UI$Xaml$Data$IPropertyPathNodeListener$ValueChanged: function (node) {
                this.ValueInternal = node.Windows$UI$Xaml$Data$IPropertyPathNode$Value;
                var listener = this._listener;
                if (listener != null) {
                    listener.Windows$UI$Xaml$Data$IPropertyPathWalkerListener$ValueChanged();
                }
            }
        }
    });

    /**
     * Implements a markup extension that describes the location of the binding source relative to the position of the binding target.
     *
     * @public
     * @class Windows.UI.Xaml.Data.RelativeSource
     * @augments System.Windows.Markup.MarkupExtension
     */
    Bridge.define("Windows.UI.Xaml.Data.RelativeSource", {
        inherits: [System.Windows.Markup.MarkupExtension],
        statics: {
            ctors: {
                ctor: function () {
                    DotNetForHtml5.Core.TypeFromStringConverters.RegisterConverter(Windows.UI.Xaml.Data.RelativeSourceMode, Windows.UI.Xaml.Data.RelativeSource.INTERNAL_ConvertFromString);
                }
            },
            methods: {
                INTERNAL_ConvertFromString: function (relativeSourceAsString) {
                    var $t;
                    switch (relativeSourceAsString) {
                        case "None": 
                            return (($t = new Windows.UI.Xaml.Data.RelativeSource.ctor(), $t.Mode = Windows.UI.Xaml.Data.RelativeSourceMode.None, $t));
                        case "Self": 
                            return (($t = new Windows.UI.Xaml.Data.RelativeSource.ctor(), $t.Mode = Windows.UI.Xaml.Data.RelativeSourceMode.Self, $t));
                        case "TemplatedParent": 
                            return (($t = new Windows.UI.Xaml.Data.RelativeSource.ctor(), $t.Mode = Windows.UI.Xaml.Data.RelativeSourceMode.TemplatedParent, $t));
                        default: 
                            throw new System.FormatException.$ctor1((relativeSourceAsString || "") + " is not an eligible value for a RelativeSource");
                    }
                }
            }
        },
        fields: {
            _mode: 0,
            _ancestorLevel: 0,
            _ancestorType: null
        },
        props: {
            /**
             * Gets or sets a value that describes the location of the binding source relative to the position of the binding target.
             Returns a value of the enumeration.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Data.RelativeSource
             * @function Mode
             * @type Windows.UI.Xaml.Data.RelativeSourceMode
             */
            Mode: {
                get: function () {
                    return this._mode;
                },
                set: function (value) {
                    this._mode = value;
                }
            },
            /**
             * Gets or sets the level of ancestor to look for, in System.Windows.Data.RelativeSourceMode.FindAncestor
             mode. Use 1 to indicate the one nearest to the binding target element.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Data.RelativeSource
             * @function AncestorLevel
             * @type number
             */
            AncestorLevel: {
                get: function () {
                    return this._ancestorLevel;
                },
                set: function (value) {
                    if (value < 1) {
                        throw new System.ArgumentOutOfRangeException.$ctor1("The ancestor level cannot be less than one.");
                    }
                    this._ancestorLevel = value;
                }
            },
            /**
             * Gets or sets the type of ancestor to look for.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Data.RelativeSource
             * @function AncestorType
             * @type System.Type
             */
            AncestorType: {
                get: function () {
                    return this._ancestorType;
                },
                set: function (value) {
                    if (this._mode === Windows.UI.Xaml.Data.RelativeSourceMode.None) {
                        this._mode = Windows.UI.Xaml.Data.RelativeSourceMode.FindAncestor;
                    }
                    this._ancestorType = value;
                }
            }
        },
        ctors: {
            init: function () {
                this._mode = Windows.UI.Xaml.Data.RelativeSourceMode.None;
                this._ancestorLevel = 1;
            },
            /**
             * Initializes a new instance of the RelativeSource class by using default relative source mode.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Data.RelativeSource
             * @memberof Windows.UI.Xaml.Data.RelativeSource
             * @return  {void}
             */
            ctor: function () {
                this.$initialize();
                System.Windows.Markup.MarkupExtension.ctor.call(this);
            },
            /**
             * Initializes a new instance of the RelativeSource class by using provided relative source mode.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Data.RelativeSource
             * @memberof Windows.UI.Xaml.Data.RelativeSource
             * @param   {Windows.UI.Xaml.Data.RelativeSourceMode}    relativeSourceMode    The relative source mode
             * @return  {void}
             */
            $ctor1: function (relativeSourceMode) {
                this.$initialize();
                System.Windows.Markup.MarkupExtension.ctor.call(this);
                this._mode = relativeSourceMode;
            }
        },
        methods: {
            ProvideValue: function (serviceProvider) {
                return null;
            }
        }
    });

    Bridge.define("Windows.UI.Xaml.DependencyObjectCollection$1.SimpleMonitor", function (T) { return {
        inherits: [System.IDisposable],
        $kind: "nested class",
        fields: {
            _busyCount: 0
        },
        props: {
            Busy: {
                get: function () {
                    return this._busyCount > 0;
                }
            }
        },
        alias: ["Dispose", "System$IDisposable$Dispose"],
        methods: {
            Enter: function () {
                this._busyCount = (this._busyCount + 1) | 0;
            },
            Dispose: function () {
                this._busyCount = (this._busyCount - 1) | 0;
            }
        }
    }; });

    /**
     * Provides data for a PropertyChangedCallback implementation.
     *
     * @public
     * @class Windows.UI.Xaml.DependencyPropertyChangedEventArgs
     * @implements  Windows.UI.Xaml.Data.IDependencyPropertyChangedEventArgs
     */
    Bridge.define("Windows.UI.Xaml.DependencyPropertyChangedEventArgs", {
        inherits: [Windows.UI.Xaml.Data.IDependencyPropertyChangedEventArgs],
        fields: {
            /**
             * Gets the value of the property before the change.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.DependencyPropertyChangedEventArgs
             * @function OldValue
             * @type System.Object
             */
            OldValue: null,
            /**
             * Gets the value of the property after the change.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.DependencyPropertyChangedEventArgs
             * @function NewValue
             * @type System.Object
             */
            NewValue: null,
            /**
             * Gets the identifier for the dependency property where the value change occurred.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.DependencyPropertyChangedEventArgs
             * @function Property
             * @type Windows.UI.Xaml.DependencyProperty
             */
            Property: null
        },
        alias: [
            "OldValue", "Windows$UI$Xaml$Data$IDependencyPropertyChangedEventArgs$OldValue",
            "NewValue", "Windows$UI$Xaml$Data$IDependencyPropertyChangedEventArgs$NewValue",
            "Property", "Windows$UI$Xaml$Data$IDependencyPropertyChangedEventArgs$Property"
        ],
        ctors: {
            $ctor1: function (oldValue, newValue, property) {
                this.$initialize();
                this.OldValue = oldValue;
                this.NewValue = newValue;
                this.Property = property;
            },
            ctor: function () {
                this.$initialize();

            }
        }
    });

    Bridge.define("Windows.UI.Xaml.Documents.INTERNAL_TextContainer", {
        inherits: [Windows.UI.Xaml.Documents.INTERNAL_ITextContainer],
        fields: {
            _parent: null
        },
        props: {
            Parent: {
                get: function () {
                    return this._parent;
                }
            }
        },
        alias: [
            "BeginChange", "Windows$UI$Xaml$Documents$INTERNAL_ITextContainer$BeginChange",
            "EndChange", "Windows$UI$Xaml$Documents$INTERNAL_ITextContainer$EndChange",
            "OnTextAdded", "Windows$UI$Xaml$Documents$INTERNAL_ITextContainer$OnTextAdded",
            "OnTextRemoved", "Windows$UI$Xaml$Documents$INTERNAL_ITextContainer$OnTextRemoved"
        ],
        ctors: {
            ctor: function (parent) {
                this.$initialize();
                if (parent == null) {
                    throw new System.ArgumentNullException.$ctor1("parent");
                }
                this._parent = parent;
            }
        },
        methods: {
            BeginChange: function () { },
            EndChange: function () { },
            OnTextAdded: function (textElement) {
                this.OnTextAddedOverride(textElement);
            },
            OnTextRemoved: function (textElement) {
                this.OnTextRemovedOverride(textElement);
            }
        }
    });

    /**
     * Provides event data for exceptions that are raised as events by asynchronous operations, such as MediaFailed or ImageFailed.
     *
     * @public
     * @class Windows.UI.Xaml.ExceptionRoutedEventArgs
     * @augments Windows.UI.Xaml.RoutedEventArgs
     */
    Bridge.define("Windows.UI.Xaml.ExceptionRoutedEventArgs", {
        inherits: [Windows.UI.Xaml.RoutedEventArgs],
        fields: {
            _errorMessage: null
        },
        props: {
            /**
             * Gets the message component of the exception, as a string.
             *
             * @instance
             * @public
             * @readonly
             * @memberof Windows.UI.Xaml.ExceptionRoutedEventArgs
             * @function ErrorMessage
             * @type string
             */
            ErrorMessage: {
                get: function () {
                    return this._errorMessage;
                }
            }
        },
        ctors: {
            init: function () {
                this._errorMessage = "";
            },
            /**
             * Initializes a new instance of Windows.UI.Xaml.ExceptionRoutedEventArgs with the specified error message.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.ExceptionRoutedEventArgs
             * @memberof Windows.UI.Xaml.ExceptionRoutedEventArgs
             * @param   {string}    errorMessage    The error message.
             * @return  {void}
             */
            $ctor1: function (errorMessage) {
                this.$initialize();
                Windows.UI.Xaml.RoutedEventArgs.ctor.call(this);
                this._errorMessage = errorMessage;
            },
            /**
             * Initializes a new instance of Windows.UI.Xaml.ExceptionRoutedEventArgs.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.ExceptionRoutedEventArgs
             * @memberof Windows.UI.Xaml.ExceptionRoutedEventArgs
             * @return  {void}
             */
            ctor: function () {
                this.$initialize();
                Windows.UI.Xaml.RoutedEventArgs.ctor.call(this);
            }
        }
    });

    /**
     * Provides data for the KeyUp and KeyDown routed events, as well as related
     attached and Preview events.
     *
     * @public
     * @class Windows.UI.Xaml.Input.KeyRoutedEventArgs
     * @augments Windows.UI.Xaml.RoutedEventArgs
     */
    Bridge.define("Windows.UI.Xaml.Input.KeyRoutedEventArgs", {
        inherits: [Windows.UI.Xaml.RoutedEventArgs],
        fields: {
            _handled: false,
            _key: 0,
            _platformKeyCode: 0,
            _keyModifiers: 0
        },
        props: {
            /**
             * Gets or sets a value that marks the routed event as handled. A true value
             for Handled prevents most handlers along the event route from handling the
             same event again.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Input.KeyRoutedEventArgs
             * @function Handled
             * @type boolean
             */
            Handled: {
                get: function () {
                    return this._handled;
                },
                set: function (value) {
                    this._handled = value;
                }
            },
            /**
             * Gets the keyboard key associated with the event.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Input.KeyRoutedEventArgs
             * @function Key
             * @type Windows.System.VirtualKey
             */
            Key: {
                get: function () {
                    return this._key;
                },
                set: function (value) {
                    this._key = value;
                }
            },
            /**
             * Gets an integer value that represents the key that is pressed or released
             (depending on which event is raised). This value is browser-specific.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Input.KeyRoutedEventArgs
             * @function PlatformKeyCode
             * @type number
             */
            PlatformKeyCode: {
                get: function () {
                    return this._platformKeyCode;
                },
                set: function (value) {
                    this._platformKeyCode = value;
                }
            },
            /**
             * Gets a value that indicates which key modifiers were active at the time that
             the pointer event was initiated.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Input.KeyRoutedEventArgs
             * @function KeyModifiers
             * @type Windows.System.VirtualKeyModifiers
             */
            KeyModifiers: {
                get: function () {
                    return this._keyModifiers;
                },
                set: function (value) {
                    this._keyModifiers = value;
                }
            }
        },
        ctors: {
            init: function () {
                this._handled = false;
            }
        },
        methods: {
            AddKeyModifiersAndUpdateDocumentValue: function (jsEventArg) {
                var keyModifiers = Windows.System.VirtualKeyModifiers.None;
                if (System.Convert.toBoolean(jsEventArg.shiftKey || false)) {
                    keyModifiers = keyModifiers | Windows.System.VirtualKeyModifiers.Shift;
                }
                if (System.Convert.toBoolean(jsEventArg.altKey || false)) {
                    keyModifiers = keyModifiers | Windows.System.VirtualKeyModifiers.Menu;
                }
                if (System.Convert.toBoolean(jsEventArg.ctrlKey || false)) {
                    keyModifiers = keyModifiers | Windows.System.VirtualKeyModifiers.Control;
                }
                this.KeyModifiers = keyModifiers;

document.refreshKeyModifiers(                jsEventArg);

                /* 
                if (!CSHTML5.Interop.IsRunningInTheSimulator)
                {
                   CSHTML5.Interop.ExecuteJavaScript(@"document.refreshKeyModifiers($0);", jsEventArg);
                }
                else
                {
                   dynamic document = INTERNAL_Simulator.HtmlDocument;
                   document.Invoke("refreshKeyModifiers", jsEventArg);
                }
                */
            }
        }
    });

    /**
     * Contains mutually exclusive VisualState objects and VisualTransition objects
     that are used to go from one state to another.
     * // Occurs after a control changes into a different state.
    //
     * // Occurs when a control begins changing into a different state.
    //
     *
     * @public
     * @class Windows.UI.Xaml.VisualStateGroup
     * @augments Windows.UI.Xaml.DependencyObject
     */
    Bridge.define("Windows.UI.Xaml.VisualStateGroup", {
        inherits: [Windows.UI.Xaml.DependencyObject],
        fields: {
            /**
             * // Initializes a new instance of the VisualStateGroup class.
            //
             *
             * @instance
             * @private
             * @memberof Windows.UI.Xaml.VisualStateGroup
             * @type Windows.UI.Xaml.VisualState
             */
            _currentState: null,
            _name: null,
            _states: null
        },
        props: {
            /**
             * Gets the most recently set VisualState from a successful call to the GoToState
             method.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.VisualStateGroup
             * @function CurrentState
             * @type Windows.UI.Xaml.VisualState
             */
            CurrentState: {
                get: function () {
                    return this._currentState;
                },
                set: function (value) {
                    this._currentState = value;
                }
            },
            /**
             * Gets the name of the VisualStateGroup.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.VisualStateGroup
             * @function Name
             * @type string
             */
            Name: {
                get: function () {
                    return this._name;
                },
                set: function (value) {
                    this._name = value;
                }
            },
            /**
             * Gets the collection of mutually exclusive VisualState objects.
             *
             * @instance
             * @public
             * @readonly
             * @memberof Windows.UI.Xaml.VisualStateGroup
             * @function States
             * @type System.Collections.IList
             */
            States: {
                get: function () {
                    return this._states;
                }
            }
        },
        ctors: {
            init: function () {
                this._states = new (System.Collections.Generic.List$1(Windows.UI.Xaml.VisualState)).ctor();
            }
        }
    });

    /**
     * Defines a segment of time.
     *
     * @abstract
     * @public
     * @class Windows.UI.Xaml.Media.Animation.Timeline
     * @augments Windows.UI.Xaml.DependencyObject
     */
    Bridge.define("Windows.UI.Xaml.Media.Animation.Timeline", {
        inherits: [Windows.UI.Xaml.DependencyObject],
        statics: {
            fields: {
                defaultTypesPaths: null,
                /**
                 * Identifies the Duration dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Media.Animation.Timeline
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                DurationProperty: null,
                /**
                 * Identifies the RepeatBehavior dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Media.Animation.Timeline
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                RepeatBehaviorProperty: null,
                /**
                 * Identifies the BeginTime dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Media.Animation.Timeline
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                BeginTimeProperty: null
            },
            ctors: {
                init: function () {
                    this.defaultTypesPaths = System.Array.init([
                        "", 
                        "Windows.UI.Xaml.", 
                        "Windows.UI.Xaml.Controls.", 
                        "Windows.UI.Xaml.Controls.Primitives.", 
                        "Windows.UI.Xaml.Media.", 
                        "Windows.UI.Xaml.Media.Animation.", 
                        "Windows.UI.Xaml.Media.Imaging.", 
                        "Windows.UI.Xaml.Shapes."
                    ], System.String);
                    this.DurationProperty = Windows.UI.Xaml.DependencyProperty.Register("Duration", Windows.UI.Xaml.Duration, Windows.UI.Xaml.Media.Animation.Timeline, new Windows.UI.Xaml.PropertyMetadata.$ctor1(Windows.UI.Xaml.Duration.Automatic.$clone()));
                    this.RepeatBehaviorProperty = Windows.UI.Xaml.DependencyProperty.Register("RepeatBehavior", Windows.UI.Xaml.Media.Animation.RepeatBehavior, Windows.UI.Xaml.Media.Animation.Timeline, new Windows.UI.Xaml.PropertyMetadata.$ctor1(new Windows.UI.Xaml.Media.Animation.RepeatBehavior.$ctor1(1).$clone()));
                    this.BeginTimeProperty = Windows.UI.Xaml.DependencyProperty.Register("BeginTime", System.Nullable$1(System.TimeSpan), Windows.UI.Xaml.Media.Animation.Timeline, new Windows.UI.Xaml.PropertyMetadata.$ctor1(new System.TimeSpan()));
                }
            },
            methods: {
                DynamicCast: function (source, destType) {
                    if (source == null) {
                        return null;
                    }

                    var nonNullableDestType = destType;
                    if (System.String.startsWith(Bridge.Reflection.getTypeFullName(destType), "System.Nullable`1")) {
                        nonNullableDestType = System.Nullable.getUnderlyingType(destType);
                    }

                    var srcType = Bridge.getType(source);
                    if (Bridge.referenceEquals(srcType, nonNullableDestType) || Bridge.Reflection.isAssignableFrom(nonNullableDestType, srcType) || (Bridge.referenceEquals(nonNullableDestType, System.Double) && Bridge.referenceEquals(srcType, System.Int32))) {
                        return source;
                    }

                    var paramTypes = System.Array.init([srcType], System.Type);
                    var cast = Bridge.Reflection.getMembers(nonNullableDestType, 8, 284, "op_Implicit", paramTypes);

                    if (cast == null) {
                        cast = Bridge.Reflection.getMembers(nonNullableDestType, 8, 284, "op_Explicit", paramTypes);

                        if (cast == null) {
                            cast = Bridge.Reflection.getMembers(srcType, 8, 284, "op_Implicit", paramTypes);

                            if (cast == null) {
                                cast = Bridge.Reflection.getMembers(srcType, 8, 284, "op_Explicit", paramTypes);
                            }
                        }
                    }

                    if (cast != null) {
                        return Bridge.Reflection.midel(cast, null).apply(null, Bridge.unbox(System.Array.init([source], System.Object)));
                    }

                    throw new System.NotImplementedException.$ctor1("Enum.ToObject not implemented in Bridge");

                }
            }
        },
        fields: {
            CompletedGuids: null,
            /**
             * @instance
             * @memberof Windows.UI.Xaml.Media.Animation.Timeline
             * @type System.Guid
             */
            animationInstanceSpecificName: null,
            _beginTimeTimer: null,
            remainingIterations: 0,
            _animationTimer: null,
            _isAnimationDurationReached: false,
            _parameters: null
        },
        events: {
            /**
             * // Gets or sets a value that specifies how the animation behaves after it reaches
            // the end of its active period.
            //
             * // Identifies the FillBehavior dependency property.
            //
             * // Gets or sets the rate, relative to its parent, at which time progresses for
            // this Timeline.
            //
             * // Identifies for the SpeedRatio dependency property.
            //
             * Occurs when this timeline has completely finished playing: it will no longer
             enter its active period.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Media.Animation.Timeline
             * @memberof Windows.UI.Xaml.Media.Animation.Timeline
             * @function addCompleted
             * @param   {System.EventHandler}    value
             * @return  {void}
             */
            /**
             * // Gets or sets a value that specifies how the animation behaves after it reaches
            // the end of its active period.
            //
             * // Identifies the FillBehavior dependency property.
            //
             * // Gets or sets the rate, relative to its parent, at which time progresses for
            // this Timeline.
            //
             * // Identifies for the SpeedRatio dependency property.
            //
             * Occurs when this timeline has completely finished playing: it will no longer
             enter its active period.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Media.Animation.Timeline
             * @memberof Windows.UI.Xaml.Media.Animation.Timeline
             * @function removeCompleted
             * @param   {System.EventHandler}    value
             * @return  {void}
             */
            Completed: null
        },
        props: {
            /**
             * // Provides base class initialization behavior for Timeline-derived classes.
            //
             * // Gets or sets a value that determines whether the timeline should be permitted
            // to run on properties where the animation is considered a dependent animation.
            //
             * // Gets or sets a value that indicates whether the timeline plays in reverse
            // after it completes a forward iteration.
            //
             * // Identifies the AutoReverse dependency property.
            //
             * // Gets or sets the time at which this Timeline should begin.
            //
             * // Identifies the BeginTime dependency property.
            //
             * Gets or sets the length of time for which this timeline plays, not counting
             repetitions.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Media.Animation.Timeline
             * @function Duration
             * @type Windows.UI.Xaml.Duration
             */
            Duration: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.Media.Animation.Timeline.DurationProperty), Windows.UI.Xaml.Duration), Windows.UI.Xaml.Duration));
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Media.Animation.Timeline.DurationProperty, value.$clone());
                }
            },
            /**
             * Gets or sets the repeating behavior of this timeline.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Media.Animation.Timeline
             * @function RepeatBehavior
             * @type Windows.UI.Xaml.Media.Animation.RepeatBehavior
             */
            RepeatBehavior: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.Media.Animation.Timeline.RepeatBehaviorProperty), Windows.UI.Xaml.Media.Animation.RepeatBehavior), Windows.UI.Xaml.Media.Animation.RepeatBehavior));
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Media.Animation.Timeline.RepeatBehaviorProperty, value.$clone());
                }
            },
            /**
             * Gets or sets the time at which this System.Windows.Media.Animation.Timeline
             should begin.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Media.Animation.Timeline
             * @function BeginTime
             * @type ?System.TimeSpan
             */
            BeginTime: {
                get: function () {
                    return Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.Media.Animation.Timeline.BeginTimeProperty), System.TimeSpan), System.TimeSpan, true);
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Media.Animation.Timeline.BeginTimeProperty, value);
                }
            }
        },
        ctors: {
            init: function () {
                this.animationInstanceSpecificName = new System.Guid();
                this.CompletedGuids = new (System.Collections.Generic.HashSet2$1(System.Guid)).ctor();
                this.animationInstanceSpecificName = System.Guid.NewGuid();
                this.remainingIterations = Number.NaN;
                this._animationTimer = new Windows.UI.Xaml.DispatcherTimer();
                this._isAnimationDurationReached = false;
            }
        },
        methods: {
            GetProperty: function (target, propertyPath) {
                if (propertyPath.INTERNAL_IsDirectlyDependencyPropertyPath) {
                    return propertyPath.INTERNAL_DependencyProperty;
                }
                var lastElementType = Bridge.getType(target);
                var dependencyPropertyContainerType = null;
                var propertyInfo = Bridge.Reflection.getMembers(lastElementType, 16, 284, propertyPath.INTERNAL_DependencyPropertyName);
                if (propertyInfo != null) {
                    dependencyPropertyContainerType = propertyInfo.td;
                } else {
                    var splittedPath = System.String.split(propertyPath.Path, [40].map(function (i) {{ return String.fromCharCode(i); }}));
                    var end = splittedPath[System.Array.index(((splittedPath.length - 1) | 0), splittedPath)];
                    var splittedEnd = System.String.split(end, [46].map(function (i) {{ return String.fromCharCode(i); }}));
                    var parentTypeString = end.substr(0, ((((end.length - splittedEnd[System.Array.index(((splittedEnd.length - 1) | 0), splittedEnd)].length) | 0) - 1) | 0));
                    parentTypeString = System.String.startsWith(parentTypeString, "global::") ? parentTypeString.substr(8) : parentTypeString;
                    var i = 0;
                    while (i < Windows.UI.Xaml.Media.Animation.Timeline.defaultTypesPaths.length && dependencyPropertyContainerType == null) {
                        dependencyPropertyContainerType = Bridge.Reflection.getType((Windows.UI.Xaml.Media.Animation.Timeline.defaultTypesPaths[System.Array.index(i, Windows.UI.Xaml.Media.Animation.Timeline.defaultTypesPaths)] || "") + (parentTypeString || ""));
                        i = (i + 1) | 0;
                    }
                    if (dependencyPropertyContainerType == null) {
                        throw new System.InvalidOperationException.$ctor1("Type \"" + (parentTypeString || "") + "\"could not be found.");
                    }
                }
                var dependencyPropertyField = Bridge.Reflection.getMembers(dependencyPropertyContainerType, 4, 284, (propertyPath.INTERNAL_DependencyPropertyName || "") + "Property");

                var dp = Bridge.cast(Bridge.Reflection.fieldAccess(dependencyPropertyField, null), Windows.UI.Xaml.DependencyProperty);
                return dp;
            },
            INTERNAL_RaiseCompletedEvent: function () {
                if (!Bridge.staticEquals(this.Completed, null)) {
                    this.Completed(this, { });
                }
            },
            INTERNAL_RaiseCompletedEvent$1: function (guid) {
                if (this.CompletedGuids == null) {
                    this.CompletedGuids = new (System.Collections.Generic.HashSet2$1(System.Guid)).ctor();
                }
                if (!this.CompletedGuids.contains(guid)) {
                    this.CompletedGuids.add(guid);
                }
                if (!Bridge.staticEquals(this.Completed, null)) {
                    this.Completed(this, { });
                }
            },
            Stop$1: function (frameworkElement, groupName, revertToFormerValue) {
                if (revertToFormerValue === void 0) { revertToFormerValue = false; }
                this._animationTimer.Stop();
                if (this._beginTimeTimer != null) {
                    this._beginTimeTimer.Stop();
                }
            },
            Stop: function (frameworkElement, revertToFormerValue) {
                this.Stop$1(frameworkElement, "visualStateGroupName", revertToFormerValue);
            },
            /**
             * Removes the value set for the VisualState on the specified frameworkElement.
             *
             * @instance
             * @this Windows.UI.Xaml.Media.Animation.Timeline
             * @memberof Windows.UI.Xaml.Media.Animation.Timeline
             * @param   {Windows.UI.Xaml.FrameworkElement}    frameworkElement
             * @return  {void}
             */
            UnApply: function (frameworkElement) {
                var frameworkElementAsControl = Bridge.as(frameworkElement, Windows.UI.Xaml.Controls.Control);
                if (frameworkElementAsControl != null) {
                    var target = { };
                    var propertyPath = { };
                    this.GetTargetElementAndPropertyInfo(frameworkElement, target, propertyPath);
                    propertyPath.v.INTERNAL_PropertySetVisualState(target.v, Windows.UI.Xaml.DependencyProperty.UnsetValue);
                }
            },
            GetTargetElementAndPropertyInfo: function (targetParent, target, propertyPath, isTargetParentTheTarget) {
                var $t;
                if (isTargetParentTheTarget === void 0) { isTargetParentTheTarget = false; }
                propertyPath.v = Windows.UI.Xaml.Media.Animation.Storyboard.GetTargetProperty(this);

                var targetBeforePath;
                if (!isTargetParentTheTarget) {
                    targetBeforePath = Windows.UI.Xaml.Media.Animation.Storyboard.GetTarget(this);

                    if (targetBeforePath == null) {
                        var targetName = Windows.UI.Xaml.Media.Animation.Storyboard.GetTargetName(this);
                        if (Bridge.is(targetParent, Windows.UI.Xaml.Controls.Control)) {
                            targetBeforePath = Bridge.cast(targetParent, Windows.UI.Xaml.Controls.Control).GetTemplateChild(targetName);
                        } else {
                            targetBeforePath = Bridge.cast(targetParent.FindName(targetName), Windows.UI.Xaml.DependencyObject);
                        }
                    }
                } else {
                    targetBeforePath = targetParent;
                }

                target.v = targetBeforePath;
                $t = Bridge.getEnumerator(propertyPath.v.INTERNAL_AccessPropertyContainer(targetBeforePath), System.Tuple$3(Windows.UI.Xaml.DependencyObject,Windows.UI.Xaml.DependencyProperty,System.Nullable$1(System.Int32)));
                try {
                    while ($t.moveNext()) {
                        var element = $t.Current;
                        target.v = element.Item1;
                    }
                } finally {
                    if (Bridge.is($t, System.IDisposable)) {
                        $t.System$IDisposable$Dispose();
                    }
                }
            },
            GetPropertyPathAndTargetBeforePath: function (targetParent, targetBeforePath, propertyPath, isTargetParentTheTarget) {
                if (isTargetParentTheTarget === void 0) { isTargetParentTheTarget = false; }
                var targetName = Windows.UI.Xaml.Media.Animation.Storyboard.GetTargetName(this);
                propertyPath.v = Windows.UI.Xaml.Media.Animation.Storyboard.GetTargetProperty(this);
                if (!isTargetParentTheTarget && targetName != null) {
                    if (Bridge.is(targetParent, Windows.UI.Xaml.Controls.Control)) {
                        targetBeforePath.v = Bridge.cast(targetParent, Windows.UI.Xaml.Controls.Control).GetTemplateChild(targetName);
                        if (targetBeforePath.v == null && Bridge.referenceEquals(targetParent.Name, targetName)) {
                            targetBeforePath.v = targetParent;
                        }
                    } else {
                        targetBeforePath.v = Bridge.cast(targetParent.FindName(targetName), Windows.UI.Xaml.DependencyObject);
                    }
                } else {
                    targetBeforePath.v = targetParent;
                }
            },
            GoThroughElementsToAccessProperty: function (propertyPath, targetBeforePath) {
                return new (Bridge.GeneratorEnumerable$1(System.Tuple$3(Windows.UI.Xaml.DependencyObject,Windows.UI.Xaml.DependencyProperty,System.Nullable$1(System.Int32))))(Bridge.fn.bind(this, function (propertyPath, targetBeforePath) {
                    var $step = 0,
                        $jumpFromFinally,
                        $returnValue,
                        $t,
                        element,
                        $async_e;

                    var $enumerator = new (Bridge.GeneratorEnumerator$1(System.Tuple$3(Windows.UI.Xaml.DependencyObject,Windows.UI.Xaml.DependencyProperty,System.Nullable$1(System.Int32))))(Bridge.fn.bind(this, function () {
                        try {
                            for (;;) {
                                switch ($step) {
                                    case 0: {
                                        $t = Bridge.getEnumerator(propertyPath.INTERNAL_AccessPropertyContainer(targetBeforePath), System.Tuple$3(Windows.UI.Xaml.DependencyObject,Windows.UI.Xaml.DependencyProperty,System.Nullable$1(System.Int32)));
                                            $step = 1;
                                            continue;
                                    }
                                    case 1: {
                                        if ($t.moveNext()) {
                                                element = $t.Current;
                                                $step = 2;
                                                continue;
                                            }
                                        $step = 4;
                                        continue;
                                    }
                                    case 2: {
                                        $enumerator.current = element;
                                            $step = 3;
                                            return true;
                                    }
                                    case 3: {
                                        $step = 1;
                                        continue;
                                    }
                                    case 4: {
                                        return false;
                                    }
                                    default: {
                                        return false;
                                    }
                                }
                            }
                        } catch($async_e1) {
                            $async_e = System.Exception.create($async_e1);
                            throw $async_e;
                        }
                    }));
                    return $enumerator;
                }, arguments));
            },
            GetCSSEquivalents: function (frameworkElement, cssEquivalent, cssEquivalents) {
                var target = { };
                var propertyPath = { };
                cssEquivalent.v = null;
                cssEquivalents.v = null;
                var frameworkElementAsControl = Bridge.as(frameworkElement, Windows.UI.Xaml.Controls.Control);
                if (frameworkElementAsControl != null) {
                    this.GetTargetElementAndPropertyInfo(frameworkElement, target, propertyPath);
                    var lastElementType = Bridge.getType(target.v);
                    var propertyInfo = Bridge.Reflection.getMembers(lastElementType, 16, 284, propertyPath.v.INTERNAL_DependencyPropertyName);



                    var dependencyPropertyContainerType = propertyInfo.td;
                    var dependencyPropertyField = Bridge.Reflection.getMembers(dependencyPropertyContainerType, 4, 284, (propertyPath.v.INTERNAL_DependencyPropertyName || "") + "Property");
                    var dp = Bridge.cast(Bridge.Reflection.fieldAccess(dependencyPropertyField, null), Windows.UI.Xaml.DependencyProperty);
                    var propertyMetadata = dp.GetTypeMetaData(Bridge.getType(target.v));

                    if (!Bridge.staticEquals(propertyMetadata.GetCSSEquivalent, null)) {
                        cssEquivalent.v = propertyMetadata.GetCSSEquivalent(target.v);
                    }
                    if (!Bridge.staticEquals(propertyMetadata.GetCSSEquivalents, null)) {
                        cssEquivalents.v = propertyMetadata.GetCSSEquivalents(target.v);
                    }
                }
            },
            /**
             * Starts the first iteration of this timeline, while managing the BeginTime property.
             *
             * @instance
             * @this Windows.UI.Xaml.Media.Animation.Timeline
             * @memberof Windows.UI.Xaml.Media.Animation.Timeline
             * @param   {Windows.UI.Xaml.Media.Animation.IterationParameters}    parameters     the parameters required for the iteration
             * @param   {boolean}                                                isLastLoop     A boolean that says if it is the last loop
             * @param   {?System.TimeSpan}                                       parentDelay    The Delay due to the BeginTime of the parent Timeline
             * @return  {void}
             */
            StartFirstIteration: function (parameters, isLastLoop, parentDelay) {
                if (System.TimeSpan.neq(this.BeginTime, null)) {
                    if (System.Nullable.getValue(this.BeginTime).getTotalMilliseconds() > 0) {
                        this._beginTimeTimer = new Windows.UI.Xaml.DispatcherTimer();
                        this._beginTimeTimer.Interval = System.Nullable.getValue(this.BeginTime);

                        this._beginTimeTimer.addTick(Bridge.fn.bind(this, function (sender, args) {
                            this._beginTimeTimer.Stop();
                            this.IterateOnce(parameters, isLastLoop);
                        }));
                        this._beginTimeTimer.Start();
                    } else {
                        this.IterateOnce(parameters, isLastLoop);
                    }
                }
            },
            IterateOnce: function (parameters, isLastLoop) {
                if (this.Duration.HasTimeSpan) {
                    if (this.Duration.TimeSpan.getTotalMilliseconds() > 0) {
                        this._parameters = parameters;
                        this._animationTimer.Interval = this.Duration.TimeSpan;
                        this._animationTimer.removeTick(Bridge.fn.cacheBind(this, this._animationTimer_Tick));
                        this._animationTimer.addTick(Bridge.fn.cacheBind(this, this._animationTimer_Tick));
                        this._isAnimationDurationReached = false;
                        this._animationTimer.Start();
                    } else {
                        this._isAnimationDurationReached = true;
                    }
                }
            },
            InitializeIteration: function () {
                if (System.Nullable.lifteq(Windows.UI.Xaml.Media.Animation.RepeatBehavior.op_Equality, this.RepeatBehavior.$clone(), null)) {
                    this.remainingIterations = 1;
                } else if (Windows.UI.Xaml.Media.Animation.RepeatBehavior.op_Inequality(this.RepeatBehavior.$clone(), Windows.UI.Xaml.Media.Animation.RepeatBehavior.Forever.$clone())) {
                    this.remainingIterations = this.RepeatBehavior.Count;
                } else {
                    this.remainingIterations = Number.POSITIVE_INFINITY;
                }
            },
            OnIterationCompleted: function (parameters) {
                if (this._isAnimationDurationReached) {
                    --this.remainingIterations;
                    if (this.remainingIterations <= 0) {
                        if (Bridge.is(this, Windows.UI.Xaml.Media.Animation.Storyboard)) {
                            Bridge.cast(this, Windows.UI.Xaml.Media.Animation.Storyboard).Stop$3(parameters.Target);
                        } else {
                            this.Stop(parameters.Target, false);
                        }

                        this.INTERNAL_RaiseCompletedEvent();
                    } else {
                        if (Bridge.is(this, Windows.UI.Xaml.Media.Animation.Storyboard)) {
                            Bridge.cast(this, Windows.UI.Xaml.Media.Animation.Storyboard).Stop$3(parameters.Target);
                        } else {
                            this.Stop(parameters.Target, true);
                        }

                        this.IterateOnce(parameters, this.remainingIterations === 1);
                    }
                }
            },
            _animationTimer_Tick: function (sender, e) {
                this._animationTimer.Stop();
                this._isAnimationDurationReached = true;
                this.OnIterationCompleted(this._parameters);
            },
            /**
             * Implemented by the class author to provide a custom natural Duration
             in the case that the Duration property is set to Automatic.  If the author
             cannot determine the Duration, this method should return Automatic.
             *
             * @instance
             * @this Windows.UI.Xaml.Media.Animation.Timeline
             * @memberof Windows.UI.Xaml.Media.Animation.Timeline
             * @return  {void}        A Duration quantity representing the natural duration.
             */
            ComputeDuration: function () {
                if (Windows.UI.Xaml.Duration.op_Equality(this.Duration.$clone(), Windows.UI.Xaml.Duration.Automatic.$clone())) {
                    this.Duration = this.GetNaturalDurationCore();
                }
            },
            GetNaturalDurationCore: function () {
                return Windows.UI.Xaml.Duration.Automatic.$clone();
            }
        }
    });

    /**
     * Provides the base class for all the easing functions.
     *
     * @public
     * @class Windows.UI.Xaml.Media.Animation.EasingFunctionBase
     * @augments Windows.UI.Xaml.DependencyObject
     */
    Bridge.define("Windows.UI.Xaml.Media.Animation.EasingFunctionBase", {
        inherits: [Windows.UI.Xaml.DependencyObject],
        statics: {
            fields: {
                /**
                 * Identifies the EasingMode dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Media.Animation.EasingFunctionBase
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                EasingModeProperty: null
            },
            ctors: {
                init: function () {
                    this.EasingModeProperty = Windows.UI.Xaml.DependencyProperty.Register("EasingMode", Windows.UI.Xaml.Media.Animation.EasingMode, Windows.UI.Xaml.Media.Animation.EasingFunctionBase, new Windows.UI.Xaml.PropertyMetadata.$ctor1(Bridge.box(Windows.UI.Xaml.Media.Animation.EasingMode.EaseOut, Windows.UI.Xaml.Media.Animation.EasingMode, System.Enum.toStringFn(Windows.UI.Xaml.Media.Animation.EasingMode))));
                }
            }
        },
        props: {
            /**
             * Gets or sets a value that specifies how the animation interpolates.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Media.Animation.EasingFunctionBase
             * @function EasingMode
             * @type Windows.UI.Xaml.Media.Animation.EasingMode
             */
            EasingMode: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.Media.Animation.EasingFunctionBase.EasingModeProperty), Windows.UI.Xaml.Media.Animation.EasingMode), System.Int32));
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Media.Animation.EasingFunctionBase.EasingModeProperty, Bridge.box(value, Windows.UI.Xaml.Media.Animation.EasingMode, System.Enum.toStringFn(Windows.UI.Xaml.Media.Animation.EasingMode)));
                }
            }
        },
        methods: {
            /**
             * // Transforms normalized time to control the pace of an animation.
            //
             *
             * @instance
             * @this Windows.UI.Xaml.Media.Animation.EasingFunctionBase
             * @memberof Windows.UI.Xaml.Media.Animation.EasingFunctionBase
             * @return  {string}        A double that represents the transformed progress.
             */
            GetFunctionAsString: function () {
                return "linear";
            },
            GetEasingModeAsString: function () {
                switch (this.EasingMode) {
                    case Windows.UI.Xaml.Media.Animation.EasingMode.EaseOut: 
                        return "easeOut";
                    case Windows.UI.Xaml.Media.Animation.EasingMode.EaseIn: 
                        return "easeIn";
                    case Windows.UI.Xaml.Media.Animation.EasingMode.EaseInOut: 
                        return "easeInOut";
                    default: 
                        return "easeOut";
                }
            },
            /**
             * Returns a Dictionary of the values to add in the options section of the Velocity Call. Implemented in inheriting classes that define their own easing function (see ExponentialEase)
             *
             * @instance
             * @this Windows.UI.Xaml.Media.Animation.EasingFunctionBase
             * @memberof Windows.UI.Xaml.Media.Animation.EasingFunctionBase
             * @return  {System.Collections.Generic.Dictionary$2}
             */
            GetAdditionalOptionsForVelocityCall: function () {
                return null;
            }
        }
    });

    /**
     * Defines an animation segment with its own target value and interpolation
     method for an ObjectAnimationUsingKeyFrames.
     *
     * @public
     * @class Windows.UI.Xaml.Media.Animation.ObjectKeyFrame
     * @augments Windows.UI.Xaml.DependencyObject
     * @implements  Windows.UI.Xaml.Media.Animation.IKeyFrame
     */
    Bridge.define("Windows.UI.Xaml.Media.Animation.ObjectKeyFrame", {
        inherits: [Windows.UI.Xaml.DependencyObject,Windows.UI.Xaml.Media.Animation.IKeyFrame],
        statics: {
            fields: {
                /**
                 * Identifies the KeyTime dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Media.Animation.ObjectKeyFrame
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                KeyTimeProperty: null,
                /**
                 * Identifies the Value dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Media.Animation.ObjectKeyFrame
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                ValueProperty: null
            },
            ctors: {
                init: function () {
                    this.KeyTimeProperty = Windows.UI.Xaml.DependencyProperty.Register("KeyTime", Windows.UI.Xaml.Media.Animation.KeyTime, Windows.UI.Xaml.Media.Animation.ObjectKeyFrame, new Windows.UI.Xaml.PropertyMetadata.$ctor1(new Windows.UI.Xaml.Media.Animation.KeyTime.$ctor1(new System.TimeSpan()).$clone()));
                    this.ValueProperty = Windows.UI.Xaml.DependencyProperty.Register("Value", System.Object, Windows.UI.Xaml.Media.Animation.ObjectKeyFrame, new Windows.UI.Xaml.PropertyMetadata.$ctor4(null));
                }
            }
        },
        props: {
            /**
             * // Provides base class initialization behavior for ObjectKeyFrame-derived classes.
            //
             * Gets or sets the time at which the key frame's target Value should be reached.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Media.Animation.ObjectKeyFrame
             * @function KeyTime
             * @type Windows.UI.Xaml.Media.Animation.KeyTime
             */
            KeyTime: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.Media.Animation.ObjectKeyFrame.KeyTimeProperty), Windows.UI.Xaml.Media.Animation.KeyTime), Windows.UI.Xaml.Media.Animation.KeyTime));
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Media.Animation.ObjectKeyFrame.KeyTimeProperty, value.$clone());
                }
            },
            /**
             * Gets or sets the key frame's target value.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Media.Animation.ObjectKeyFrame
             * @function Value
             * @type System.Object
             */
            Value: {
                get: function () {
                    return this.GetValue(Windows.UI.Xaml.Media.Animation.ObjectKeyFrame.ValueProperty);
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Media.Animation.ObjectKeyFrame.ValueProperty, value);
                }
            }
        },
        alias: [
            "KeyTime", "Windows$UI$Xaml$Media$Animation$IKeyFrame$KeyTime",
            "Value", "Windows$UI$Xaml$Media$Animation$IKeyFrame$Value"
        ]
    });

    /**
     * An abstract class that defines an animation segment with its own target value
     and interpolation method for a DoubleAnimationUsingKeyFrames.
     *
     * @abstract
     * @public
     * @class Windows.UI.Xaml.Media.Animation.DoubleKeyFrame
     * @augments Windows.UI.Xaml.DependencyObject
     * @implements  Windows.UI.Xaml.Media.Animation.IKeyFrame
     */
    Bridge.define("Windows.UI.Xaml.Media.Animation.DoubleKeyFrame", {
        inherits: [Windows.UI.Xaml.DependencyObject,Windows.UI.Xaml.Media.Animation.IKeyFrame],
        statics: {
            fields: {
                KeyTimeProperty: null,
                ValueProperty: null
            },
            ctors: {
                init: function () {
                    this.KeyTimeProperty = Windows.UI.Xaml.DependencyProperty.Register("KeyTime", Windows.UI.Xaml.Media.Animation.KeyTime, Windows.UI.Xaml.Media.Animation.DoubleKeyFrame, new Windows.UI.Xaml.PropertyMetadata.$ctor1(new Windows.UI.Xaml.Media.Animation.KeyTime.ctor().$clone()));
                    this.ValueProperty = Windows.UI.Xaml.DependencyProperty.Register("Value", System.Double, Windows.UI.Xaml.Media.Animation.DoubleKeyFrame, new Windows.UI.Xaml.PropertyMetadata.$ctor1(Bridge.box(0.0, System.Double, System.Double.format, System.Double.getHashCode)));
                }
            }
        },
        props: {
            /**
             * Gets or sets the time at which the key frame's target {@link } should be reached.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Media.Animation.DoubleKeyFrame
             * @function KeyTime
             * @type Windows.UI.Xaml.Media.Animation.KeyTime
             * @return  {[type]}        The time at which the key frame's current value should be equal to its {@link } property. The default is null.
             */
            KeyTime: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.Media.Animation.DoubleKeyFrame.KeyTimeProperty), Windows.UI.Xaml.Media.Animation.KeyTime), Windows.UI.Xaml.Media.Animation.KeyTime));
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Media.Animation.DoubleKeyFrame.KeyTimeProperty, value.$clone());
                }
            },
            /**
             * Gets or sets the key frame's target value.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Media.Animation.DoubleKeyFrame
             * @function Value
             * @type number
             * @return  {[type]}        The key frame's target value, which is the value of this key frame at its specified {@link }. The default is 0.
             */
            Value: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.Media.Animation.DoubleKeyFrame.ValueProperty), System.Double), System.Double));
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Media.Animation.DoubleKeyFrame.ValueProperty, Bridge.box(value, System.Double, System.Double.format, System.Double.getHashCode));
                }
            },
            /**
             * The value of this key frame at the KeyTime specified.
             *
             * @instance
             * @memberof Windows.UI.Xaml.Media.Animation.DoubleKeyFrame
             * @function Windows$UI$Xaml$Media$Animation$IKeyFrame$Value
             * @type System.Object
             */
            Windows$UI$Xaml$Media$Animation$IKeyFrame$Value: {
                get: function () {
                    return Bridge.box(this.Value, System.Double, System.Double.format, System.Double.getHashCode);
                },
                set: function (value) {
                    this.Value = System.Nullable.getValue(Bridge.cast(Bridge.unbox(value, System.Double), System.Double));
                }
            }
        },
        alias: ["KeyTime", "Windows$UI$Xaml$Media$Animation$IKeyFrame$KeyTime"],
        methods: {
            INTERNAL_GetEasingFunction: function () {
                return null;
            }
        }
    });

    Bridge.define("Windows.UI.Xaml.Media.Animation.INTERNAL_ResolvedKeyFramesEntries$1", function (T) { return {
        inherits: [System.Collections.Generic.IEnumerable$1(System.Int32)],
        fields: {
            _resolvedKeyFramesIndexes: null
        },
        alias: ["GetEnumerator", ["System$Collections$Generic$IEnumerable$1$System$Int32$GetEnumerator", "System$Collections$Generic$IEnumerable$1$GetEnumerator"]],
        ctors: {
            ctor: function () {
                this.$initialize();
                this._resolvedKeyFramesIndexes = new (System.Collections.Generic.List$1(System.Int32)).ctor();
            },
            $ctor1: function (keyFrames) {
                var $t;
                Windows.UI.Xaml.Media.Animation.INTERNAL_ResolvedKeyFramesEntries$1(T).ctor.call(this);
                var index = 0;
                $t = Bridge.getEnumerator(keyFrames, T);
                try {
                    while ($t.moveNext()) {
                        var keyFrame = $t.Current;
                        this.ResolveKeyFrame(Bridge.identity(index, ((index = (index + 1) | 0))), keyFrames);
                    }
                } finally {
                    if (Bridge.is($t, System.IDisposable)) {
                        $t.System$IDisposable$Dispose();
                    }
                }
            }
        },
        methods: {
            ResolveKeyFrame: function (index, keyFrames) {
                var isAdded = false;
                var currentIndex = 0;
                var keyFrame = System.Array.getItem(keyFrames, index, T);
                while (!isAdded && currentIndex < this._resolvedKeyFramesIndexes.Count) {
                    if (System.TimeSpan.lt(keyFrame.Windows$UI$Xaml$Media$Animation$IKeyFrame$KeyTime.TimeSpan, System.Array.getItem(keyFrames, this._resolvedKeyFramesIndexes.getItem(currentIndex), T).Windows$UI$Xaml$Media$Animation$IKeyFrame$KeyTime.TimeSpan)) {
                        this._resolvedKeyFramesIndexes.insert(currentIndex, index);
                        isAdded = true;
                    } else {
                        currentIndex = (currentIndex + 1) | 0;
                    }
                }
                if (!isAdded) {
                    this._resolvedKeyFramesIndexes.add(index);
                }
            },
            GetNextKeyFrameIndex: function (resolvedKeyFrameCount) {
                if (resolvedKeyFrameCount >= this._resolvedKeyFramesIndexes.Count || resolvedKeyFrameCount < 0) {
                    return -1;
                } else {
                    return this._resolvedKeyFramesIndexes.getItem(resolvedKeyFrameCount);
                }
            },
            GetEnumerator: function () {
                return this._resolvedKeyFramesIndexes.GetEnumerator().$clone();
            },
            System$Collections$IEnumerable$GetEnumerator: function () {
                return this.GetEnumerator();
            }
        }
    }; });

    /**
     * Represents a visual behavior that occurs when a control is affected by a
     predefined action or state change. These may be applied to individual elements
     using the Transitions property, or applied to the children of a Panel using
     the ChildrenTransitions property.
     *
     * @public
     * @class Windows.UI.Xaml.Media.Animation.Transition
     * @augments Windows.UI.Xaml.DependencyObject
     */
    Bridge.define("Windows.UI.Xaml.Media.Animation.Transition", {
        inherits: [Windows.UI.Xaml.DependencyObject]
    });

    /**
     * Represents a segment of a PathFigure object.
     *
     * @abstract
     * @public
     * @class Windows.UI.Xaml.Media.PathSegment
     * @augments Windows.UI.Xaml.DependencyObject
     */
    Bridge.define("Windows.UI.Xaml.Media.PathSegment", {
        inherits: [Windows.UI.Xaml.DependencyObject],
        fields: {
            ParentPath: null
        },
        ctors: {
            ctor: function () {
                this.$initialize();
                Windows.UI.Xaml.DependencyObject.ctor.call(this);

            }
        },
        methods: {
            SetParentPath: function (path) {
                this.ParentPath = path;
            }
        }
    });

    /**
     * Defines objects used to paint graphical objects. Classes that derive from
     Brush describe how the area is painted.
     *
     * @public
     * @class Windows.UI.Xaml.Media.Brush
     * @augments Windows.UI.Xaml.DependencyObject
     * @implements  Windows.UI.Xaml.IHasAccessToPropertiesWhereItIsUsed
     */
    Bridge.define("Windows.UI.Xaml.Media.Brush", {
        inherits: [Windows.UI.Xaml.DependencyObject,Windows.UI.Xaml.IHasAccessToPropertiesWhereItIsUsed],
        statics: {
            fields: {
                /**
                 * Identifies the Opacity dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Media.Brush
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                OpacityProperty: null
            },
            ctors: {
                init: function () {
                    this.OpacityProperty = Windows.UI.Xaml.DependencyProperty.Register("Opacity", System.Double, Windows.UI.Xaml.Media.Brush, new Windows.UI.Xaml.PropertyMetadata.$ctor1(Bridge.box(1.0, System.Double, System.Double.format, System.Double.getHashCode)));
                },
                ctor: function () {
                    DotNetForHtml5.Core.TypeFromStringConverters.RegisterConverter(Windows.UI.Xaml.Media.Brush, Windows.UI.Xaml.Media.Brush.INTERNAL_ConvertFromString);
                }
            },
            methods: {
                INTERNAL_ConvertFromString: function (colorcode) {
                    return new Windows.UI.Xaml.Media.SolidColorBrush.$ctor1(System.Nullable.getValue(Bridge.cast(Bridge.unbox(Windows.UI.Color.INTERNAL_ConvertFromString(colorcode), Windows.UI.Color), Windows.UI.Color)));
                },
                MergeCSSEquivalentsOfTheParentsProperties: function (brush, parentPropertyToValueToHtmlConverter) {
                    var $t, $t1, $t2;
                    var result = new (System.Collections.Generic.List$1(Windows.UI.Xaml.CSSEquivalent)).ctor();
                    var propertiesWhereUsed = new (System.Collections.Generic.HashSet2$1(System.Collections.Generic.KeyValuePair$2(Windows.UI.Xaml.DependencyObject,Windows.UI.Xaml.DependencyProperty))).ctor();
                    $t = Bridge.getEnumerator(brush.PropertiesWhereUsed);
                    try {
                        while ($t.moveNext()) {
                            var tuple = $t.Current;
                            propertiesWhereUsed.add(tuple);
                        }
                    } finally {
                        if (Bridge.is($t, System.IDisposable)) {
                            $t.System$IDisposable$Dispose();
                        }
                    }

                    $t1 = Bridge.getEnumerator(propertiesWhereUsed);
                    try {
                        while ($t1.moveNext()) {
                            var tuple1 = $t1.Current;
                            var uiElement = Bridge.as(tuple1.key, Windows.UI.Xaml.UIElement);
                            var dependencyProperty = tuple1.value;
                            if (uiElement != null) {
                                if (!CSHTML5.Internal.INTERNAL_VisualTreeManager.IsElementInVisualTree(uiElement)) {
                                    brush.PropertiesWhereUsed.remove(tuple1);
                                } else {
                                    var propertyMetadata = dependencyProperty.GetTypeMetaData(Bridge.getType(uiElement));
                                    if (!Bridge.staticEquals(propertyMetadata.GetCSSEquivalent, null)) {
                                        var parentPropertyCSSEquivalent = propertyMetadata.GetCSSEquivalent(uiElement);
                                        if (parentPropertyCSSEquivalent != null) {
                                            var newCSSEquivalent = ($t2 = new Windows.UI.Xaml.CSSEquivalent(), $t2.Name = parentPropertyCSSEquivalent.Name, $t2.ApplyAlsoWhenThereIsAControlTemplate = parentPropertyCSSEquivalent.ApplyAlsoWhenThereIsAControlTemplate, $t2.Value = parentPropertyToValueToHtmlConverter(parentPropertyCSSEquivalent), $t2.DomElement = parentPropertyCSSEquivalent.DomElement, $t2.UIElement = uiElement, $t2);
                                            if (newCSSEquivalent.DomElement == null) {
                                                newCSSEquivalent.DomElement = uiElement.INTERNAL_OuterDomElement;
                                            }
                                            result.add(newCSSEquivalent);
                                        }
                                    } else {
                                        if (!Bridge.staticEquals(propertyMetadata.PropertyChangedCallback, null)) {
                                            result.add(($t2 = new Windows.UI.Xaml.CSSEquivalent(), $t2.UIElement = uiElement, $t2.CallbackMethod = propertyMetadata.PropertyChangedCallback, $t2.DependencyProperty = dependencyProperty, $t2));
                                        }
                                    }
                                }
                            } else {
                            }
                        }
                    } finally {
                        if (Bridge.is($t1, System.IDisposable)) {
                            $t1.System$IDisposable$Dispose();
                        }
                    }
                    return result;
                }
            }
        },
        fields: {
            _propertiesWhereUsed: null
        },
        props: {
            /**
             * Gets or sets the degree of opacity of a Brush.
             The value of the Opacity property is expressed as a value between 0 and 1.0.
             The default value is 1.0, which is full opacity. 0 is transparent opacity.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Media.Brush
             * @function Opacity
             * @type number
             */
            Opacity: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.Media.Brush.OpacityProperty), System.Double), System.Double));
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Media.Brush.OpacityProperty, Bridge.box(value, System.Double, System.Double.format, System.Double.getHashCode));
                }
            },
            PropertiesWhereUsed: {
                get: function () {
                    if (this._propertiesWhereUsed == null) {
                        this._propertiesWhereUsed = new (System.Collections.Generic.HashSet2$1(System.Collections.Generic.KeyValuePair$2(Windows.UI.Xaml.DependencyObject,Windows.UI.Xaml.DependencyProperty))).ctor();
                    }
                    return this._propertiesWhereUsed;
                }
            }
        },
        alias: ["PropertiesWhereUsed", "Windows$UI$Xaml$IHasAccessToPropertiesWhereItIsUsed$PropertiesWhereUsed"]
    });

    /**
     * Provides generalized transformation support for objects, such as points and
     rectangles.
     * / Gets the inverse transformation of this GeneralTransform, if possible.
    /
     * / Implements the behavior for return value of Inverse in a derived or custom
    / GeneralTransform.
    /
     *
     * @abstract
     * @public
     * @class Windows.UI.Xaml.Media.GeneralTransform
     * @augments Windows.UI.Xaml.DependencyObject
     */
    Bridge.define("Windows.UI.Xaml.Media.GeneralTransform", {
        inherits: [Windows.UI.Xaml.DependencyObject],
        fields: {
            INTERNAL_parent: null
        },
        ctors: {
            /**
             * Provides base class initialization behavior for GeneralTransform-derived
             classes.
             *
             * @instance
             * @protected
             * @this Windows.UI.Xaml.Media.GeneralTransform
             * @memberof Windows.UI.Xaml.Media.GeneralTransform
             * @return  {void}
             */
            ctor: function () {
                this.$initialize();
                Windows.UI.Xaml.DependencyObject.ctor.call(this);

            }
        },
        methods: {
            /**
             * Transforms the specified point and returns the result.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Media.GeneralTransform
             * @memberof Windows.UI.Xaml.Media.GeneralTransform
             * @param   {Windows.Foundation.Point}    point    The point to transform.
             * @return  {Windows.Foundation.Point}             The result of transforming point.
             */
            TransformPoint: function (point) {
                return this.INTERNAL_TransformPoint(point.$clone());
            },
            /**
             * Transforms the specified point and returns the result.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Media.GeneralTransform
             * @memberof Windows.UI.Xaml.Media.GeneralTransform
             * @param   {Windows.Foundation.Point}    point    The point to transform.
             * @return  {Windows.Foundation.Point}             The result of transforming point.
             */
            Transform: function (point) {

                return this.TransformPoint(point.$clone());
            }
        }
    });

    /**
     * Provides a base class for objects that define geometric shapes. Geometry
     objects can be used for clipping regions and as geometry definitions for
     rendering two-dimensional graphical data as a Path.
     *
     * @abstract
     * @public
     * @class Windows.UI.Xaml.Media.Geometry
     * @augments Windows.UI.Xaml.DependencyObject
     */
    Bridge.define("Windows.UI.Xaml.Media.Geometry", {
        inherits: [Windows.UI.Xaml.DependencyObject],
        statics: {
            fields: {
                TransformProperty: null
            },
            ctors: {
                init: function () {
                    this.TransformProperty = Windows.UI.Xaml.DependencyProperty.Register("Transform", Windows.UI.Xaml.Media.Transform, Windows.UI.Xaml.Media.Geometry, new Windows.UI.Xaml.PropertyMetadata.$ctor2(null, Windows.UI.Xaml.Media.Geometry.OnTransformPropertyChanged));
                },
                ctor: function () {
                    DotNetForHtml5.Core.TypeFromStringConverters.RegisterConverter(Windows.UI.Xaml.Media.Geometry, Windows.UI.Xaml.Media.Geometry.INTERNAL_ConvertFromString);
                }
            },
            methods: {
                INTERNAL_ConvertFromString: function (pathAsString) {
                    return Windows.UI.Xaml.Media.PathGeometry.INTERNAL_ConvertFromString(pathAsString);
                },
                OnTransformPropertyChanged: function (d, e) {
                    var geometry = Bridge.cast(d, Windows.UI.Xaml.Media.Geometry);
                    if (geometry.ParentPath != null) {
                        geometry.ParentPath.ScheduleRedraw();
                    }
                }
            }
        },
        fields: {
            ParentPath: null
        },
        props: {
            Transform: {
                get: function () {
                    return Bridge.cast(this.GetValue(Windows.UI.Xaml.Media.Geometry.TransformProperty), Windows.UI.Xaml.Media.Transform);
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Media.Geometry.TransformProperty, value);
                }
            }
        },
        ctors: {
            ctor: function () {
                this.$initialize();
                Windows.UI.Xaml.DependencyObject.ctor.call(this);

            }
        },
        methods: {
            SetParentPath: function (path) {
                this.ParentPath = path;
            },
            GetFillRuleAsString: function () {
                return "evenodd";
            }
        }
    });

    /**
     * Describes the location and color of a transition point in a gradient.
     *
     * @public
     * @class Windows.UI.Xaml.Media.GradientStop
     * @augments Windows.UI.Xaml.DependencyObject
     * @implements  Windows.UI.Xaml.ICloneOnAnimation
     */
    Bridge.define("Windows.UI.Xaml.Media.GradientStop", {
        inherits: [Windows.UI.Xaml.DependencyObject,Windows.UI.Xaml.ICloneOnAnimation],
        statics: {
            fields: {
                /**
                 * Identifies the Color dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Media.GradientStop
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                ColorProperty: null,
                /**
                 * Identifies the Offset dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Media.GradientStop
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                OffsetProperty: null
            },
            ctors: {
                init: function () {
                    var $t;
                    this.ColorProperty = Windows.UI.Xaml.DependencyProperty.Register("Color", Windows.UI.Color, Windows.UI.Xaml.Media.GradientStop, ($t = new Windows.UI.Xaml.PropertyMetadata.$ctor1(Windows.UI.Color.FromArgb(0, 0, 0, 0).$clone()), $t.GetCSSEquivalents = function (instance) {
                        var gradientStop = Bridge.as(instance, Windows.UI.Xaml.Media.GradientStop);
                        if (gradientStop != null) {
                            var brush = gradientStop.INTERNAL_ParentBrush;
                            if (brush != null) {
                                var parentPropertyToValueToHtmlConverter = function (parentPropertyCSSEquivalent) {
                                    return (function (inst, value) {
                                        return brush;

                                    });
                                };

                                return Windows.UI.Xaml.Media.Brush.MergeCSSEquivalentsOfTheParentsProperties(brush, parentPropertyToValueToHtmlConverter);
                            } else {
                                return new (System.Collections.Generic.List$1(Windows.UI.Xaml.CSSEquivalent)).ctor();
                            }
                        } else {
                            throw new System.ArgumentException.ctor();
                        }
                    }, $t));
                    this.OffsetProperty = Windows.UI.Xaml.DependencyProperty.Register("Offset", System.Double, Windows.UI.Xaml.Media.GradientStop, new Windows.UI.Xaml.PropertyMetadata.$ctor1(Bridge.box(0.0, System.Double, System.Double.format, System.Double.getHashCode)));
                }
            }
        },
        fields: {
            INTERNAL_ParentBrush: null,
            _isAlreadyAClone: false
        },
        props: {
            /**
             * Gets or sets the color of the gradient stop.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Media.GradientStop
             * @function Color
             * @type Windows.UI.Color
             */
            Color: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.Media.GradientStop.ColorProperty), Windows.UI.Color), Windows.UI.Color));
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Media.GradientStop.ColorProperty, value.$clone());
                }
            },
            /**
             * Gets the location of the gradient stop within the gradient vector.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Media.GradientStop
             * @function Offset
             * @type number
             */
            Offset: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.Media.GradientStop.OffsetProperty), System.Double), System.Double));
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Media.GradientStop.OffsetProperty, Bridge.box(value, System.Double, System.Double.format, System.Double.getHashCode));
                }
            }
        },
        alias: [
            "Clone", "Windows$UI$Xaml$ICloneOnAnimation$Clone",
            "IsAlreadyAClone", "Windows$UI$Xaml$ICloneOnAnimation$IsAlreadyAClone"
        ],
        ctors: {
            init: function () {
                this._isAlreadyAClone = false;
            }
        },
        methods: {
            Clone: function () {
                var $t;
                return ($t = new Windows.UI.Xaml.Media.GradientStop(), $t.Color = this.Color.$clone(), $t.Offset = this.Offset, $t._isAlreadyAClone = true, $t);
            },
            IsAlreadyAClone: function () {
                return this._isAlreadyAClone;
            }
        }
    });

    /**
     * Provides an object source type for Source and ImageSource.
     *
     * @public
     * @class Windows.UI.Xaml.Media.ImageSource
     * @augments Windows.UI.Xaml.DependencyObject
     */
    Bridge.define("Windows.UI.Xaml.Media.ImageSource", {
        inherits: [Windows.UI.Xaml.DependencyObject],
        statics: {
            ctors: {
                ctor: function () {
                    DotNetForHtml5.Core.TypeFromStringConverters.RegisterConverter(Windows.UI.Xaml.Media.ImageSource, Windows.UI.Xaml.Media.ImageSource.INTERNAL_ConvertFromString);
                }
            },
            methods: {
                INTERNAL_ConvertFromString: function (str) {
                    var uriKind = new System.UriKind();
                    if (System.String.contains(str,":/")) {
                        uriKind = System.UriKind.Absolute;
                    } else {
                        uriKind = System.UriKind.Relative;
                    }
                    var returnValue = new Windows.UI.Xaml.Media.Imaging.BitmapImage.$ctor1(new System.Uri(str, uriKind));
                    return returnValue;
                }
            }
        }
    });

    /**
     * Represents a subsection of a geometry, a single connected series of two-dimensional
     geometric segments.
     *
     * @public
     * @class Windows.UI.Xaml.Media.PathFigure
     * @augments Windows.UI.Xaml.DependencyObject
     */
    Bridge.define("Windows.UI.Xaml.Media.PathFigure", {
        inherits: [Windows.UI.Xaml.DependencyObject],
        statics: {
            fields: {
                /**
                 * Identifies the IsClosed dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Media.PathFigure
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                IsClosedProperty: null,
                /**
                 * Identifies the IsFilled dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Media.PathFigure
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                IsFilledProperty: null,
                /**
                 * Identifies the Segments dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Media.PathFigure
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                SegmentsProperty: null,
                /**
                 * Identifies the StartPoint dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Media.PathFigure
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                StartPointProperty: null
            },
            ctors: {
                init: function () {
                    this.IsClosedProperty = Windows.UI.Xaml.DependencyProperty.Register("IsClosed", System.Boolean, Windows.UI.Xaml.Media.PathFigure, new Windows.UI.Xaml.PropertyMetadata.$ctor2(Bridge.box(false, System.Boolean, System.Boolean.toString), Windows.UI.Xaml.Media.PathFigure.IsClosed_Changed));
                    this.IsFilledProperty = Windows.UI.Xaml.DependencyProperty.Register("IsFilled", System.Boolean, Windows.UI.Xaml.Media.PathFigure, new Windows.UI.Xaml.PropertyMetadata.$ctor2(Bridge.box(false, System.Boolean, System.Boolean.toString), Windows.UI.Xaml.Media.PathFigure.IsFilled_Changed));
                    this.SegmentsProperty = Windows.UI.Xaml.DependencyProperty.Register("Segments", Windows.UI.Xaml.Media.PathSegmentCollection, Windows.UI.Xaml.Media.PathFigure, new Windows.UI.Xaml.PropertyMetadata.$ctor2(null, Windows.UI.Xaml.Media.PathFigure.Segments_Changed));
                    this.StartPointProperty = Windows.UI.Xaml.DependencyProperty.Register("StartPoint", Windows.Foundation.Point, Windows.UI.Xaml.Media.PathFigure, new Windows.UI.Xaml.PropertyMetadata.$ctor2(new Windows.Foundation.Point.ctor().$clone(), Windows.UI.Xaml.Media.PathFigure.StartPoint_Changed));
                }
            },
            methods: {
                IsClosed_Changed: function (d, e) {
                    var figure = Bridge.cast(d, Windows.UI.Xaml.Media.PathFigure);
                    if (figure._parentPath != null) {
                        figure._parentPath.ScheduleRedraw();
                    }
                },
                IsFilled_Changed: function (d, e) {
                    var figure = Bridge.cast(d, Windows.UI.Xaml.Media.PathFigure);
                    if (figure._parentPath != null) {
                        figure._parentPath.ScheduleRedraw();
                    }
                },
                Segments_Changed: function (d, e) {
                    var figure = Bridge.cast(d, Windows.UI.Xaml.Media.PathFigure);
                    if (null != e.OldValue) {
                        Bridge.cast(e.OldValue, Windows.UI.Xaml.Media.PathSegmentCollection).SetParentPath(null);
                    }
                    if (figure._parentPath != null) {
                        if (null != e.NewValue) {
                            Bridge.cast(e.NewValue, Windows.UI.Xaml.Media.PathSegmentCollection).SetParentPath(figure._parentPath);
                        }

                        figure._parentPath.ScheduleRedraw();
                    }
                },
                StartPoint_Changed: function (d, e) {
                    var figure = Bridge.cast(d, Windows.UI.Xaml.Media.PathFigure);
                    if (figure._parentPath != null) {
                        figure._parentPath.ScheduleRedraw();
                    }
                }
            }
        },
        fields: {
            _parentPath: null
        },
        props: {
            /**
             * Gets or sets a value that indicates whether this figure's first and last
             segments are connected.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Media.PathFigure
             * @function IsClosed
             * @type boolean
             */
            IsClosed: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.Media.PathFigure.IsClosedProperty), System.Boolean), System.Boolean));
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Media.PathFigure.IsClosedProperty, Bridge.box(value, System.Boolean, System.Boolean.toString));
                }
            },
            /**
             * Gets or sets a value that indicates whether the contained area of this PathFigure
             is to be used for hit-testing, rendering, and clipping.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Media.PathFigure
             * @function IsFilled
             * @type boolean
             */
            IsFilled: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.Media.PathFigure.IsFilledProperty), System.Boolean), System.Boolean));
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Media.PathFigure.IsFilledProperty, Bridge.box(value, System.Boolean, System.Boolean.toString));
                }
            },
            /**
             * Gets or sets the collection of segments that define the shape of this PathFigure
             object.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Media.PathFigure
             * @function Segments
             * @type Windows.UI.Xaml.Media.PathSegmentCollection
             */
            Segments: {
                get: function () {
                    var collection = Bridge.cast(this.GetValue(Windows.UI.Xaml.Media.PathFigure.SegmentsProperty), Windows.UI.Xaml.Media.PathSegmentCollection);
                    if (collection == null) {
                        collection = new Windows.UI.Xaml.Media.PathSegmentCollection.ctor();
                        this.SetValue(Windows.UI.Xaml.Media.PathFigure.SegmentsProperty, collection);
                    }
                    return collection;
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Media.PathFigure.SegmentsProperty, value);
                }
            },
            /**
             * Gets or sets the Point where the PathFigure begins.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Media.PathFigure
             * @function StartPoint
             * @type Windows.Foundation.Point
             */
            StartPoint: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.Media.PathFigure.StartPointProperty), Windows.Foundation.Point), Windows.Foundation.Point));
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Media.PathFigure.StartPointProperty, value.$clone());
                }
            }
        },
        ctors: {
            /**
             * Initializes a new instance of the PathFigure class.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Media.PathFigure
             * @memberof Windows.UI.Xaml.Media.PathFigure
             * @return  {void}
             */
            ctor: function () {
                this.$initialize();
                Windows.UI.Xaml.DependencyObject.ctor.call(this);

            }
        },
        methods: {
            SetParentPath: function (path) {
                if (!Bridge.referenceEquals(this._parentPath, path)) {
                    this._parentPath = path;
                    this.Segments.SetParentPath(path);
                }
            },
            DefineInCanvas: function (xOffsetToApplyBeforeMultiplication, yOffsetToApplyBeforeMultiplication, xOffsetToApplyAfterMultiplication, yOffsetToApplyAfterMultiplication, horizontalMultiplicator, verticalMultiplicator, canvasDomElement, strokeThickness, shapeActualSize) {
                var $t;
                var context = CSHTML5.Internal.INTERNAL_HtmlDomManager.Get2dCanvasContext(canvasDomElement);


                var segmentStartingPosition = new Windows.Foundation.Point.$ctor1(this.StartPoint.X, this.StartPoint.Y);

                context.moveTo(this.StartPoint.X * horizontalMultiplicator + xOffsetToApplyBeforeMultiplication * horizontalMultiplicator + xOffsetToApplyAfterMultiplication, this.StartPoint.Y * verticalMultiplicator + yOffsetToApplyBeforeMultiplication * verticalMultiplicator + yOffsetToApplyAfterMultiplication);

                $t = Bridge.getEnumerator(this.Segments);
                try {
                    while ($t.moveNext()) {
                        var segment = $t.Current;
                        segmentStartingPosition = segment.DefineInCanvas(xOffsetToApplyBeforeMultiplication, yOffsetToApplyBeforeMultiplication, xOffsetToApplyAfterMultiplication, yOffsetToApplyAfterMultiplication, horizontalMultiplicator, verticalMultiplicator, canvasDomElement, segmentStartingPosition.$clone());
                    }
                } finally {
                    if (Bridge.is($t, System.IDisposable)) {
                        $t.System$IDisposable$Dispose();
                    }
                }
                if (this.IsClosed) {
                    context.closePath();
                }
            },
            GetMinMaxXY: function (minX, maxX, minY, maxY) {
                var $t;
                minX.v = Math.min(minX.v, this.StartPoint.X);
                maxX.v = Math.max(maxX.v, this.StartPoint.X);
                minY.v = Math.min(minY.v, this.StartPoint.Y);
                maxY.v = Math.max(maxY.v, this.StartPoint.Y);

                var segmentStartingPosition = new Windows.Foundation.Point.$ctor1(this.StartPoint.X, this.StartPoint.Y);
                $t = Bridge.getEnumerator(this.Segments);
                try {
                    while ($t.moveNext()) {
                        var segment = $t.Current;
                        segmentStartingPosition = segment.GetMinMaxXY(minX, maxX, minY, maxY, segmentStartingPosition.$clone());
                    }
                } finally {
                    if (Bridge.is($t, System.IDisposable)) {
                        $t.System$IDisposable$Dispose();
                    }
                }
            },
            GetMaxXY: function () {
                var $t;
                var globalMax = new Windows.Foundation.Point.ctor();
                $t = Bridge.getEnumerator(this.Segments);
                try {
                    while ($t.moveNext()) {
                        var segment = $t.Current;
                        var segmentMaxXY = segment.GetMaxXY();
                        if (segmentMaxXY.X > globalMax.X) {
                            globalMax.X = segmentMaxXY.X;
                        }
                        if (segmentMaxXY.Y > globalMax.Y) {
                            globalMax.Y = segmentMaxXY.Y;
                        }
                    }
                } finally {
                    if (Bridge.is($t, System.IDisposable)) {
                        $t.System$IDisposable$Dispose();
                    }
                }
                return globalMax.$clone();
            }
        }
    });

    /**
     * Converts a uniform resource identifier (URI) into a new URI based on the
     rules of a matching object specified in a collection of mapping objects.
     *
     * @public
     * @class Windows.UI.Xaml.Navigation.UriMapper
     * @augments Windows.UI.Xaml.Navigation.UriMapperBase
     */
    Bridge.define("Windows.UI.Xaml.Navigation.UriMapper", {
        inherits: [Windows.UI.Xaml.Navigation.UriMapperBase],
        fields: {
            /**
             * // Initializes a new instance of the System.Windows.Navigation.UriMapper class.
            //
             *
             * @instance
             * @private
             * @memberof Windows.UI.Xaml.Navigation.UriMapper
             * @type System.Collections.Generic.List$1
             */
            _uriMappings: null
        },
        props: {
            /**
             * Gets a collection of objects that are used to convert a uniform resource
             identifier (URI) into a new URI.
             *
             * @instance
             * @public
             * @readonly
             * @memberof Windows.UI.Xaml.Navigation.UriMapper
             * @function UriMappings
             * @type System.Collections.Generic.List$1
             */
            UriMappings: {
                get: function () {
                    return this._uriMappings;
                }
            }
        },
        ctors: {
            init: function () {
                this._uriMappings = new (System.Collections.Generic.List$1(Windows.UI.Xaml.Navigation.UriMapping)).ctor();
            }
        },
        methods: {
            /**
             * Converts a specified uniform resource identifier (URI) into a new URI based
             on the rules of a matching object in the System.Windows.Navigation.UriMapper.UriMappings
             collection.
             *
             * @instance
             * @public
             * @override
             * @this Windows.UI.Xaml.Navigation.UriMapper
             * @memberof Windows.UI.Xaml.Navigation.UriMapper
             * @param   {System.Uri}    uri    Original URI value to be converted to a new URI.
             * @return  {System.Uri}           A URI to use for handling the request instead of the value of the uri parameter.
             If no object in the System.Windows.Navigation.UriMapper.UriMappings collection
             matches uri, the original value for uri is returned.
             */
            MapUri: function (uri) {
                var $t;
                $t = Bridge.getEnumerator(this._uriMappings);
                try {
                    while ($t.moveNext()) {
                        var mapping = $t.Current;
                        var mappedUri = mapping.MapUri(uri);
                        if (System.Uri.notEquals(mappedUri, null)) {
                            return mappedUri;
                        }
                    }
                } finally {
                    if (Bridge.is($t, System.IDisposable)) {
                        $t.System$IDisposable$Dispose();
                    }
                }
                return uri;
            }
        }
    });

    /**
     * Defines the pattern for converting a requested uniform resource identifier
     (URI) into a new URI.
     *
     * @public
     * @class Windows.UI.Xaml.Navigation.UriMapping
     * @augments Windows.UI.Xaml.DependencyObject
     */
    Bridge.define("Windows.UI.Xaml.Navigation.UriMapping", {
        inherits: [Windows.UI.Xaml.DependencyObject],
        statics: {
            fields: {
                /**
                 * Identifies the System.Windows.Navigation.UriMapping.MappedUri dependency property
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Navigation.UriMapping
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                MappedUriProperty: null,
                /**
                 * Identifies the System.Windows.Navigation.UriMapping.Uri dependency property
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Navigation.UriMapping
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                UriProperty: null
            },
            ctors: {
                init: function () {
                    var $t;
                    this.MappedUriProperty = Windows.UI.Xaml.DependencyProperty.Register("MappedUri", System.Uri, Windows.UI.Xaml.Navigation.UriMapping, new Windows.UI.Xaml.PropertyMetadata.$ctor4(null));
                    this.UriProperty = Windows.UI.Xaml.DependencyProperty.Register("Uri", System.Uri, Windows.UI.Xaml.Navigation.UriMapping, ($t = new Windows.UI.Xaml.PropertyMetadata.$ctor2(null, Windows.UI.Xaml.Navigation.UriMapping.Uri_Changed), $t.CallPropertyChangedWhenLoadedIntoVisualTree = Windows.UI.Xaml.WhenToCallPropertyChangedEnum.IfPropertyIsSet, $t));
                }
            },
            methods: {
                Uri_Changed: function (d, e) {
                    var $t;
                    var mapping = Bridge.cast(d, Windows.UI.Xaml.Navigation.UriMapping);
                    var newValue = Bridge.cast(e.NewValue, System.Uri);
                    var r = new System.Text.RegularExpressions.Regex.ctor("{[^}]*}");
                    mapping._groupNames = new (System.Collections.Generic.List$1(System.String)).ctor();
                    $t = Bridge.getEnumerator(r.matches(newValue.getOriginalString()));
                    try {
                        while ($t.moveNext()) {
                            var match = Bridge.cast($t.Current, System.Text.RegularExpressions.Match);
                            mapping._groupNames.add(match.getValue());
                        }
                    } finally {
                        if (Bridge.is($t, System.IDisposable)) {
                            $t.System$IDisposable$Dispose();
                        }
                    }
                    mapping._regularExpressionToApplyOnUriToMap = r.replace(newValue.getOriginalString(), "(.*)");
                }
            }
        },
        fields: {
            _regularExpressionToApplyOnUriToMap: null,
            _groupNames: null
        },
        props: {
            /**
             * // Initializes a new instance of the System.Windows.Navigation.UriMapping class.
            //
             * // Gets or sets the uniform resource identifier (URI) that is navigated to instead
            // of the originally requested URI.
            //
             * Gets or sets the uniform resource identifier (URI) that is navigated to instead
             of the originally requested URI.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Navigation.UriMapping
             * @function MappedUri
             * @type System.Uri
             */
            MappedUri: {
                get: function () {
                    return Bridge.cast(this.GetValue(Windows.UI.Xaml.Navigation.UriMapping.MappedUriProperty), System.Uri);
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Navigation.UriMapping.MappedUriProperty, value);
                }
            },
            /**
             * // Gets or sets the pattern to match when determining if the requested uniform
            // resource identifier (URI) is converted to a mapped URI.
            //
             * Gets or sets the pattern to match when determining if the requested uniform
             resource identifier (URI) is converted to a mapped URI.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Navigation.UriMapping
             * @function Uri
             * @type System.Uri
             */
            Uri: {
                get: function () {
                    return Bridge.cast(this.GetValue(Windows.UI.Xaml.Navigation.UriMapping.UriProperty), System.Uri);
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Navigation.UriMapping.UriProperty, value);
                }
            }
        },
        ctors: {
            init: function () {
                this._regularExpressionToApplyOnUriToMap = "";
            }
        },
        methods: {
            /**
             * Converts the specified uniform resource identifier (URI) to a new URI, if
             the specified URI matches the defined template for converting.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Navigation.UriMapping
             * @memberof Windows.UI.Xaml.Navigation.UriMapping
             * @param   {System.Uri}    uri    The URI to convert.
             * @return  {System.Uri}           The URI that has been converted or null if the URI cannot be converted.
             */
            MapUri: function (uri) {
                var $t, $t1;
                if (System.Uri.equals(this.Uri, null) || System.String.isNullOrWhiteSpace(this.Uri.getOriginalString())) {
                    throw new System.InvalidOperationException.$ctor1("this UriMapping's Uri is null");
                }
                if (System.Uri.equals(this.MappedUri, null) || System.String.isNullOrWhiteSpace(this.MappedUri.getOriginalString())) {
                    throw new System.InvalidOperationException.$ctor1("this UriMapping's MappedUri is null");
                }

                var reg = new System.Text.RegularExpressions.Regex.ctor(this._regularExpressionToApplyOnUriToMap);
                var matches = reg.matches(uri.getOriginalString());
                var groupNameToReplacement = new (System.Collections.Generic.Dictionary$2(System.String,System.String)).ctor();
                var result = this.MappedUri.getOriginalString();

                $t = Bridge.getEnumerator(matches);
                try {
                    while ($t.moveNext()) {
                        var match = Bridge.cast($t.Current, System.Text.RegularExpressions.Match);
                        var i = 0;
                        var groups = match.getGroups();
                        if (groups.getCount() === ((this._groupNames.Count + 1) | 0)) {
                            var isFirst = true;
                            $t1 = Bridge.getEnumerator(match.getGroups());
                            try {
                                while ($t1.moveNext()) {
                                    var group = Bridge.cast($t1.Current, System.Text.RegularExpressions.Group);
                                    if (!isFirst) {
                                        result = System.String.replaceAll(result, System.Linq.Enumerable.from(this._groupNames).elementAt(i), group.getValue());
                                        i = (i + 1) | 0;
                                    } else {
                                        isFirst = false;
                                    }
                                }
                            } finally {
                                if (Bridge.is($t1, System.IDisposable)) {
                                    $t1.System$IDisposable$Dispose();
                                }
                            }

                            return new System.Uri(result, System.UriKind.RelativeOrAbsolute);
                        } else {
                            return null;
                        }
                    }
                } finally {
                    if (Bridge.is($t, System.IDisposable)) {
                        $t.System$IDisposable$Dispose();
                    }
                }
                return null;




            }
        }
    });

    /**
     * Implements a data structure for describing a property as a path below another
     property, or below an owning type. Property paths are used in data binding to objects.
     *
     * @public
     * @class Windows.UI.Xaml.PropertyPath
     * @augments Windows.UI.Xaml.DependencyObject
     */
    Bridge.define("Windows.UI.Xaml.PropertyPath", {
        inherits: [Windows.UI.Xaml.DependencyObject],
        statics: {
            ctors: {
                ctor: function () {
                    DotNetForHtml5.Core.TypeFromStringConverters.RegisterConverter(Windows.UI.Xaml.PropertyPath, Windows.UI.Xaml.PropertyPath.INTERNAL_ConvertFromString);
                }
            },
            methods: {
                defaulAccessVisualStateProperty: function (rootTargetObjectInstance) {
                    return new (Bridge.GeneratorEnumerable$1(System.Tuple$3(Windows.UI.Xaml.DependencyObject,Windows.UI.Xaml.DependencyProperty,System.Nullable$1(System.Int32))))(Bridge.fn.bind(this, function (rootTargetObjectInstance) {
                        var $step = 0,
                            $jumpFromFinally,
                            $returnValue,
                            $async_e;

                        var $enumerator = new (Bridge.GeneratorEnumerator$1(System.Tuple$3(Windows.UI.Xaml.DependencyObject,Windows.UI.Xaml.DependencyProperty,System.Nullable$1(System.Int32))))(Bridge.fn.bind(this, function () {
                            try {
                                for (;;) {
                                    switch ($step) {
                                        case 0: {
                                            return false;
                                        }
                                        default: {
                                            return false;
                                        }
                                    }
                                }
                            } catch($async_e1) {
                                $async_e = System.Exception.create($async_e1);
                                throw $async_e;
                            }
                        }));
                        return $enumerator;
                    }, arguments));
                },
                INTERNAL_ConvertFromString: function (path) {
                    return new Windows.UI.Xaml.PropertyPath.ctor(path);
                }
            }
        },
        fields: {
            INTERNAL_DependencyProperty: null,
            INTERNAL_IsDirectlyDependencyPropertyPath: false,
            INTERNAL_DependencyPropertyName: null,
            /**
             * When set, defines a method designed to access the element that contains the property whose Value will be accessed.
             *
             * @instance
             * @memberof Windows.UI.Xaml.PropertyPath
             * @type System.Func
             */
            INTERNAL_AccessPropertyContainer: null,
            /**
             * Sets the VisualState value (value is second parameter) of the previously defined property on the DependencyObject (first parameter).
             *
             * @instance
             * @memberof Windows.UI.Xaml.PropertyPath
             * @type System.Action
             */
            INTERNAL_PropertySetVisualState: null,
            /**
             * Sets the Animation value (value is second parameter) of the previously defined property on the DependencyObject (first parameter).
             *
             * @instance
             * @memberof Windows.UI.Xaml.PropertyPath
             * @type System.Action
             */
            INTERNAL_PropertySetAnimationValue: null,
            /**
             * Sets the Local value (value is second parameter) of the previously defined property on the DependencyObject (first parameter).
             *
             * @instance
             * @memberof Windows.UI.Xaml.PropertyPath
             * @type System.Action
             */
            INTERNAL_PropertySetLocalValue: null,
            /**
             * Gets the VisualState value of the previously defined property on the DependencyObject set as parameter.
             *
             * @instance
             * @memberof Windows.UI.Xaml.PropertyPath
             * @type System.Func
             */
            INTERNAL_PropertyGetVisualState: null,
            _path: null
        },
        props: {
            /**
             * Gets the path value held by this PropertyPath. Returns the path value held by this PropertyPath.
             *
             * @instance
             * @public
             * @readonly
             * @memberof Windows.UI.Xaml.PropertyPath
             * @function Path
             * @type string
             */
            Path: {
                get: function () {
                    return this._path;
                }
            }
        },
        ctors: {
            init: function () {
                this.INTERNAL_IsDirectlyDependencyPropertyPath = false;
            },
            /**
             * Initializes a new instance of the PropertyPath class based on a path string.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.PropertyPath
             * @memberof Windows.UI.Xaml.PropertyPath
             * @param   {string}    path    The path string to construct with.
             * @return  {void}
             */
            ctor: function (path) {
                this.$initialize();
                Windows.UI.Xaml.DependencyObject.ctor.call(this);
                this._path = path;
            },
            $ctor2: function (dependencyProperty) {
                this.$initialize();
                Windows.UI.Xaml.DependencyObject.ctor.call(this);
                this.INTERNAL_IsDirectlyDependencyPropertyPath = true;
                this.INTERNAL_DependencyProperty = dependencyProperty;
                this._path = dependencyProperty.Name;
                this.INTERNAL_DependencyPropertyName = dependencyProperty.Name;
                this.INTERNAL_AccessPropertyContainer = Windows.UI.Xaml.PropertyPath.defaulAccessVisualStateProperty;
                this.INTERNAL_PropertySetVisualState = Bridge.fn.cacheBind(this, this.defaultSetVisualStateProperty);
                this.INTERNAL_PropertySetAnimationValue = Bridge.fn.cacheBind(this, this.defaultSetAnimationVisualStateProperty);
                this.INTERNAL_PropertySetLocalValue = Bridge.fn.cacheBind(this, this.defaultSetLocalVisualStateProperty);
                this.INTERNAL_PropertyGetVisualState = Bridge.fn.cacheBind(this, this.defaultGetVisualStateProperty);
            },
            /**
             * Initializes a new Instance of the PropertyPath class based on methods to access the property from a DependencyObject.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.PropertyPath
             * @memberof Windows.UI.Xaml.PropertyPath
             * @param   {string}           path                         
             * @param   {string}           dependencyPropertyName       The name of the property (consists only of the last part of the path we would put in the XAML).
             * @param   {System.Func}      accessPropertyContainer      The function that will access the object on which the property is from the DependencyObject put in its parameters(corresponds to the path in XAML without the last property).
             * @param   {System.Action}    propertySetVisualState       The function that sets the VisualState value on the given element.
             * @param   {System.Action}    propertySetAnimationValue    
             * @param   {System.Action}    propertySetLocalValue        The function that sets the Local value on the given element.
             * @param   {System.Func}      propertyGetVisualState       The function that gets the VisualState value on the given element.
             * @return  {void}
             */
            $ctor1: function (path, dependencyPropertyName, accessPropertyContainer, propertySetVisualState, propertySetAnimationValue, propertySetLocalValue, propertyGetVisualState) {
                this.$initialize();
                Windows.UI.Xaml.DependencyObject.ctor.call(this);
                this._path = path;
                this.INTERNAL_DependencyPropertyName = dependencyPropertyName;
                this.INTERNAL_AccessPropertyContainer = accessPropertyContainer;
                this.INTERNAL_PropertySetVisualState = propertySetVisualState;
                this.INTERNAL_PropertySetAnimationValue = propertySetAnimationValue;
                this.INTERNAL_PropertySetLocalValue = propertySetLocalValue;
                this.INTERNAL_PropertyGetVisualState = propertyGetVisualState;
            }
        },
        methods: {
            defaultSetVisualStateProperty: function (finalTargetInstance, value) {
                if (this.INTERNAL_IsDirectlyDependencyPropertyPath) {
                    (finalTargetInstance).SetVisualStateValue(this.INTERNAL_DependencyProperty, value);
                } else {
                    throw new System.InvalidOperationException.$ctor1("The constructor: PropertyPath(string path) for storyboards is not supported yet. Please use PropertyPath(DependencyProperty dependencyProperty) or define your storyboard in the XAML.");
                }
            },
            defaultSetAnimationVisualStateProperty: function (finalTargetInstance, value) {
                if (this.INTERNAL_IsDirectlyDependencyPropertyPath) {
                    (finalTargetInstance).SetAnimationValue(this.INTERNAL_DependencyProperty, value);
                } else {
                    throw new System.InvalidOperationException.$ctor1("The constructor: PropertyPath(string path) for storyboards is not supported yet. Please use PropertyPath(DependencyProperty dependencyProperty) or define your storyboard in the XAML.");
                }
            },
            defaultSetLocalVisualStateProperty: function (finalTargetInstance, value) {
                if (this.INTERNAL_IsDirectlyDependencyPropertyPath) {
                    (finalTargetInstance).SetCurrentValue(this.INTERNAL_DependencyProperty, value);
                } else {
                    throw new System.InvalidOperationException.$ctor1("The constructor: PropertyPath(string path) for storyboards is not supported yet. Please use PropertyPath(DependencyProperty dependencyProperty) or define your storyboard in the XAML.");
                }
            },
            defaultGetVisualStateProperty: function (finalTargetInstance) {
                if (this.INTERNAL_IsDirectlyDependencyPropertyPath) {
                    return finalTargetInstance.GetVisualStateValue(this.INTERNAL_DependencyProperty);
                } else {
                    throw new System.InvalidOperationException.$ctor1("The constructor: PropertyPath(string path) for storyboards is not supported yet. Please use PropertyPath(DependencyProperty dependencyProperty) or define your storyboard in the XAML.");
                }
            }
        }
    });

    /**
     * Represents the base class for value setters.
     *
     * @abstract
     * @public
     * @class Windows.UI.Xaml.SetterBase
     * @augments Windows.UI.Xaml.DependencyObject
     */
    Bridge.define("Windows.UI.Xaml.SetterBase", {
        inherits: [Windows.UI.Xaml.DependencyObject],
        fields: {
            _sealed: false
        },
        props: {
            /**
             * Returns the sealed state of this object.  If true, any attempt
             at modifying the state of this object will trigger an exception.
             *
             * @instance
             * @public
             * @readonly
             * @memberof Windows.UI.Xaml.SetterBase
             * @function IsSealed
             * @type boolean
             */
            IsSealed: {
                get: function () {
                    return this._sealed;
                }
            }
        },
        ctors: {
            /**
             * SetterBase construction
             *
             * @instance
             * @this Windows.UI.Xaml.SetterBase
             * @memberof Windows.UI.Xaml.SetterBase
             * @return  {void}
             */
            ctor: function () {
                this.$initialize();
                Windows.UI.Xaml.DependencyObject.ctor.call(this);
            }
        },
        methods: {
            Seal: function () {
                this._sealed = true;
            },
            /**
             * Subclasses need to call this method before any changes to their state.
             *
             * @instance
             * @protected
             * @this Windows.UI.Xaml.SetterBase
             * @memberof Windows.UI.Xaml.SetterBase
             * @return  {void}
             */
            CheckSealed: function () {
                if (this._sealed) {
                    throw new System.InvalidOperationException.$ctor1(System.String.format("Cannot modify a '{0}' after it is sealed.", ["SetterBase"]));
                }
            }
        }
    });

    /**
     * Provides data related to the SizeChanged event.
     *
     * @public
     * @class Windows.UI.Xaml.SizeChangedEventArgs
     * @augments Windows.UI.Xaml.RoutedEventArgs
     */
    Bridge.define("Windows.UI.Xaml.SizeChangedEventArgs", {
        inherits: [Windows.UI.Xaml.RoutedEventArgs],
        fields: {
            _newSize: null
        },
        props: {
            /**
             * Gets the new size of the object reporting the size change.
             *
             * @instance
             * @public
             * @readonly
             * @memberof Windows.UI.Xaml.SizeChangedEventArgs
             * @function NewSize
             * @type Windows.Foundation.Size
             */
            NewSize: {
                get: function () {
                    return this._newSize.$clone();
                }
            }
        },
        ctors: {
            init: function () {
                this._newSize = new Windows.Foundation.Size();
            },
            ctor: function (newSize) {
                this.$initialize();
                Windows.UI.Xaml.RoutedEventArgs.ctor.call(this);
                this._newSize = newSize.$clone();
            }
        }
    });

    /**
     * Contains property setters that can be shared between instances of a type.
     *
     * @public
     * @class Windows.UI.Xaml.Style
     * @augments Windows.UI.Xaml.DependencyObject
     */
    Bridge.define("Windows.UI.Xaml.Style", {
        inherits: [Windows.UI.Xaml.DependencyObject],
        statics: {
            fields: {
                TargetTypeID: 0,
                BasedOnID: 0,
                DefaultTargetType: null
            },
            ctors: {
                init: function () {
                    this.TargetTypeID = 1;
                    this.BasedOnID = 2;
                    this.DefaultTargetType = Windows.UI.Xaml.FrameworkElement;
                }
            }
        },
        fields: {
            _sealed: false,
            _setters: null,
            _targetType: null,
            _basedOn: null,
            _modified: 0,
            PropertyValues: null,
            EffectiveValues: null
        },
        props: {
            /**
             * Style mutability state
             *
             * @instance
             * @public
             * @readonly
             * @memberof Windows.UI.Xaml.Style
             * @function IsSealed
             * @type boolean
             */
            IsSealed: {
                get: function () {
                    return this._sealed;
                }
            },
            /**
             * Type that this style is intended
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Style
             * @function TargetType
             * @type System.Type
             */
            TargetType: {
                get: function () {
                    return this._targetType;
                },
                set: function (value) {
                    if (this._sealed) {
                        throw new System.InvalidOperationException.$ctor1(System.String.format("Cannot modify a '{0}' after it is sealed.", ["Style"]));
                    }

                    if (value == null) {
                        throw new System.ArgumentNullException.$ctor1("value");
                    }

                    if (!Bridge.Reflection.isAssignableFrom(Windows.UI.Xaml.FrameworkElement, value) && !(Bridge.referenceEquals(Windows.UI.Xaml.Style.DefaultTargetType, value))) {
                        throw new System.ArgumentException.$ctor1(System.String.format("'{0}' type must derive from FrameworkElement", [Bridge.Reflection.getTypeName(value)]));
                    }

                    this._targetType = value;

                    this.SetModified(Windows.UI.Xaml.Style.TargetTypeID);
                }
            },
            /**
             * Style to base on
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Style
             * @function BasedOn
             * @type Windows.UI.Xaml.Style
             */
            BasedOn: {
                get: function () {
                    return this._basedOn;
                },
                set: function (value) {
                    if (this._sealed) {
                        throw new System.InvalidOperationException.$ctor1(System.String.format("Cannot modify a '{0}' after it is sealed.", ["Style"]));
                    }

                    if (Bridge.referenceEquals(value, this)) {
                        throw new System.ArgumentException.$ctor1("A Style cannot be based on itself.");
                    }

                    this._basedOn = value;

                    this.SetModified(Windows.UI.Xaml.Style.BasedOnID);
                }
            },
            /**
             * The collection of property setters for the target type
             *
             * @instance
             * @public
             * @readonly
             * @memberof Windows.UI.Xaml.Style
             * @function Setters
             * @type Windows.UI.Xaml.SetterBaseCollection
             */
            Setters: {
                get: function () {
                    if (this._setters == null) {
                        this._setters = new Windows.UI.Xaml.SetterBaseCollection();

                        if (this._sealed) {
                            this._setters.Seal();
                        }
                    }
                    return this._setters;
                }
            }
        },
        ctors: {
            init: function () {
                this._modified = 0;
                this.EffectiveValues = new (System.Collections.Generic.Dictionary$2(Windows.UI.Xaml.DependencyProperty,System.Object)).ctor();
            },
            /**
             * Initializes a new instance of the Style class, with no initial TargetType and an empty Setters collection.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Style
             * @memberof Windows.UI.Xaml.Style
             * @return  {void}
             */
            ctor: function () {
                this.$initialize();
                Windows.UI.Xaml.DependencyObject.ctor.call(this);
                this.CanBeInheritanceContext = false;
                this.IsInheritanceContextSealed = true;
            },
            /**
             * Initializes a new instance of the Style class, with an initial TargetType and an empty Setters collection.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Style
             * @memberof Windows.UI.Xaml.Style
             * @param   {System.Type}    targetType    The type on which the Style will be used.
             * @return  {void}
             */
            $ctor1: function (targetType) {
                Windows.UI.Xaml.Style.ctor.call(this);
                this.TargetType = targetType;
            }
        },
        methods: {
            /**
             * This Style is now immutable
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Style
             * @memberof Windows.UI.Xaml.Style
             * @return  {void}
             */
            Seal: function () {
                if (this._sealed) {
                    return;
                }

                if (this._targetType == null) {
                    throw new System.InvalidOperationException.$ctor1(System.String.format("Must have non-null value for '{0}'.", ["TargetType"]));
                }

                if (this._basedOn != null) {
                    if (!Bridge.referenceEquals(Windows.UI.Xaml.Style.DefaultTargetType, this._basedOn.TargetType) && !Bridge.Reflection.isAssignableFrom(this._basedOn.TargetType, this._targetType)) {
                        throw new System.InvalidOperationException.$ctor1(System.String.format("Can only base on a Style with target type that is base type '{0}'.", [Bridge.Reflection.getTypeName(this._targetType)]));
                    }
                }

                if (this._setters != null) {
                    this._setters.Seal();
                }

                this.CheckForCircularBasedOnReferences();

                if (this._basedOn != null) {
                    this._basedOn.Seal();
                }


                this.ProcessSetters(this);

                this.ProcessSelfStyles(this);

                this._sealed = true;
            },
            /**
             * Given a set of values for the PropertyValue struct, put that in
             to the PropertyValueList, overwriting any existing entry.
             *
             * @instance
             * @private
             * @this Windows.UI.Xaml.Style
             * @memberof Windows.UI.Xaml.Style
             * @param   {Windows.UI.Xaml.DependencyProperty}    dp       
             * @param   {System.Object}                         value
             * @return  {void}
             */
            UpdatePropertyValueList: function (dp, value) {
                var existingIndex = -1;
                for (var i = 0; i < this.PropertyValues.Count; i = (i + 1) | 0) {
                    if (Bridge.referenceEquals(this.PropertyValues.getItem(i).$clone().Property, dp)) {
                        existingIndex = i;
                        break;
                    }
                }

                if (existingIndex >= 0) {
                    var propertyValue = this.PropertyValues.getItem(existingIndex).$clone();
                    propertyValue.ValueInternal = value;
                    this.PropertyValues.setItem(existingIndex, propertyValue.$clone());
                } else {
                    var propertyValue1 = new Windows.UI.Xaml.PropertyValue();
                    propertyValue1.Property = dp;
                    propertyValue1.ValueInternal = value;

                    this.PropertyValues.add(propertyValue1.$clone());
                }
            },
            CheckTargetType: function (element) { },
            /**
             * This method checks to see if the BasedOn hierarchy contains
             a loop in the chain of references.
             *
             * @instance
             * @private
             * @this Windows.UI.Xaml.Style
             * @memberof Windows.UI.Xaml.Style
             * @return  {void}
             */
            CheckForCircularBasedOnReferences: function () {
                var basedOnHierarchy = new (System.Collections.Generic.Stack$1(Windows.UI.Xaml.Style)).$ctor2(10);
                var latestBasedOn = this;

                while (latestBasedOn != null) {
                    if (basedOnHierarchy.Contains(latestBasedOn)) {
                        throw new System.InvalidOperationException.$ctor1("This Style's hierarchy of BasedOn references contains a loop.");

                    }

                    basedOnHierarchy.Push(latestBasedOn);
                    latestBasedOn = latestBasedOn.BasedOn;
                }

                return;
            },
            ProcessSetters: function (style) {
                if (style == null) {
                    return;
                }

                style.Setters.Seal();


                if (this.PropertyValues == null || this.PropertyValues.Count === 0) {
                    this.PropertyValues = new (System.Collections.Generic.List$1(Windows.UI.Xaml.PropertyValue)).$ctor2(style.Setters.Count);
                }

                for (var i = 0; i < style.Setters.Count; i = (i + 1) | 0) {
                    var setterBase = style.Setters.getItem(i);
                    System.Diagnostics.Debug.Assert$1(setterBase != null, "Setter collection must contain non-null instances of SetterBase");

                    var setter = Bridge.as(setterBase, Windows.UI.Xaml.Setter);
                    if (setter != null) {
                        if (Bridge.referenceEquals(style, this)) {
                            var expr = Windows.UI.Xaml.Data.BindingOperations.GetBindingExpression(setter, Windows.UI.Xaml.Setter.ValueProperty);
                            if (expr != null) {
                                this.UpdatePropertyValueList(setter.Property, expr);
                            } else {
                                this.UpdatePropertyValueList(setter.Property, setter.Value);
                            }
                        }
                    }
                }

                this.ProcessSetters(style._basedOn);
            },
            ProcessSelfStyles: function (style) {
                if (style == null) {
                    return;
                }

                this.ProcessSelfStyles(style._basedOn);

                for (var i = 0; i < style.PropertyValues.Count; i = (i + 1) | 0) {
                    var propertyValue = style.PropertyValues.getItem(i).$clone();
                    this.EffectiveValues.setItem(propertyValue.Property, propertyValue.ValueInternal);
                }
            },
            SetModified: function (id) {
                this._modified = this._modified | id;
            },
            IsModified: function (id) {
                return (id & this._modified) !== 0;
            }
        }
    });

    /**
     * Represents the visual appearance of the control when it is in a specific
     state.
     *
     * @public
     * @class Windows.UI.Xaml.VisualState
     * @augments Windows.UI.Xaml.DependencyObject
     */
    Bridge.define("Windows.UI.Xaml.VisualState", {
        inherits: [Windows.UI.Xaml.DependencyObject],
        fields: {
            /**
             * // Initializes a new instance of the VisualState class.
            //
             *
             * @instance
             * @private
             * @memberof Windows.UI.Xaml.VisualState
             * @type string
             */
            _name: null,
            _storyboard: null,
            INTERNAL_Group: null
        },
        props: {
            /**
             * Gets the name of the VisualState.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.VisualState
             * @function Name
             * @type string
             */
            Name: {
                get: function () {
                    return this._name;
                },
                set: function (value) {
                    this._name = value;
                }
            },
            /**
             * Gets or sets a Storyboard that defines the appearance of the control when
             it is the state that is represented by the VisualState.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.VisualState
             * @function Storyboard
             * @type Windows.UI.Xaml.Media.Animation.Storyboard
             */
            Storyboard: {
                get: function () {
                    return this._storyboard;
                },
                set: function (value) {
                    this._storyboard = value;
                }
            }
        },
        methods: {
            GetPropertiesChangedByStoryboard: function () {
                if (this.Storyboard != null) {
                    return this.Storyboard.GetPropertiesChanged();
                }
                return null;
            },
            ApplyStoryboard: function (frameworkElement, useTransitions) {
                var propertiesChanged = null;
                if (this.Storyboard != null) {
                    this.Storyboard.Begin$1(frameworkElement, useTransitions, this.INTERNAL_Group.Name, true);
                    propertiesChanged = this.Storyboard.GetPropertiesChanged();

                }
                return propertiesChanged;
            },
            StopStoryBoard: function (frameworkElement) {
                if (this.Storyboard != null) {
                    this.Storyboard.Stop$4(frameworkElement, this.INTERNAL_Group.Name);
                }
            }
        }
    });

    /**
     * Manages states and the logic for transitioning between states for controls.
     *
     * @public
     * @class Windows.UI.Xaml.VisualStateManager
     * @augments Windows.UI.Xaml.DependencyObject
     */
    Bridge.define("Windows.UI.Xaml.VisualStateManager", {
        inherits: [Windows.UI.Xaml.DependencyObject],
        statics: {
            fields: {
                /**
                 * Identifies the VisualStateManager.VisualStateGroup XAML attached property
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.VisualStateManager
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                VisualStateGroupsProperty: null
            },
            ctors: {
                init: function () {
                    this.VisualStateGroupsProperty = Windows.UI.Xaml.DependencyProperty.RegisterAttached("VisualStateGroups", System.Collections.IList, Windows.UI.Xaml.UIElement, new Windows.UI.Xaml.PropertyMetadata.$ctor4(null));
                }
            },
            methods: {
                /**
                 * // Initializes a new instance of the VisualStateManager class.
                //
                 * // Identifies the VisualStateManager.CustomVisualStateManager dependency property.
                //
                 * // Gets the value of the VisualStateManager.CustomVisualStateManager attached
                // property.
                //
                 * Gets the value of the VisualStateManager.VisualStateGroups attached property.
                 *
                 * @static
                 * @public
                 * @this Windows.UI.Xaml.VisualStateManager
                 * @memberof Windows.UI.Xaml.VisualStateManager
                 * @param   {Windows.UI.Xaml.DependencyObject}    obj    The object to get the value from.
                 * @return  {System.Collections.IList}                   The collection of VisualStateGroup objects that is associated with the specified
                 object.
                 */
                GetVisualStateGroups: function (obj) {
                    throw new System.NotSupportedException.$ctor1("Method not supported.");


                },
                /**
                 * Sets the value of the Grid.Column XAML attached property on the specified FrameworkElement.
                 *
                 * @static
                 * @public
                 * @this Windows.UI.Xaml.VisualStateManager
                 * @memberof Windows.UI.Xaml.VisualStateManager
                 * @param   {Windows.UI.Xaml.DependencyObject}    obj      The target element on which to set the Grid.Row XAML attached property.
                 * @param   {System.Collections.IList}            value    The property value to set.
                 * @return  {void}
                 */
                SetVisualStateGroups: function (obj, value) {
                    obj.SetValue(Windows.UI.Xaml.VisualStateManager.VisualStateGroupsProperty, value);
                },
                /**
                 * Transitions the control between two states.
                 *
                 * @static
                 * @public
                 * @this Windows.UI.Xaml.VisualStateManager
                 * @memberof Windows.UI.Xaml.VisualStateManager
                 * @param   {Windows.UI.Xaml.Controls.Control}    control           The control to transition between states.
                 * @param   {string}                              stateName         The state to transition to.
                 * @param   {boolean}                             useTransitions    True to use a VisualTransition to transition between states; otherwise,
                 false.
                 * @return  {boolean}                                               True if the control successfully transitioned to the new state; otherwise,
                 false.
                 */
                GoToState: function (control, stateName, useTransitions) {
                    var $t;
                    try {
                        var state = control.INTERNAL_GetVisualStateGroups().GetVisualState(stateName);
                        if (state != null) {
                            var formerState = state.INTERNAL_Group.CurrentState;
                            if (formerState != null) {
                                formerState.StopStoryBoard(control);
                            }

                            if (!Bridge.referenceEquals(state, formerState)) {

                                var newPropertiesChanged = state.ApplyStoryboard(control, useTransitions);


                                if (formerState != null) {
                                    var formerPropertiesChanged = formerState.GetPropertiesChangedByStoryboard();
                                    if (formerPropertiesChanged != null) {
                                        $t = Bridge.getEnumerator(formerPropertiesChanged.Keys);
                                        try {
                                            while ($t.moveNext()) {
                                                var formerProp = $t.Current;
                                                if (newPropertiesChanged == null || !newPropertiesChanged.containsKey(formerProp)) {
                                                    formerPropertiesChanged.getItem(formerProp).UnApply(control);

                                                }
                                            }
                                        } finally {
                                            if (Bridge.is($t, System.IDisposable)) {
                                                $t.System$IDisposable$Dispose();
                                            }
                                        }
                                        formerState.Storyboard.isUnApplied = true;
                                    }
                                }

                                state.INTERNAL_Group.CurrentState = state;
                            }

                            return true;
                        } else {
                            return false;
                        }
                    } catch ($e1) {
                        $e1 = System.Exception.create($e1);
                        return false;
                    }
                }
            }
        }
    });

    Bridge.define("___CSHTML5__Component__Defaultstyles__INTERNAL_Defaultautocompleteboxstyle__Xaml", {
        inherits: [Bridge.global.Windows.UI.Xaml.ResourceDictionary],
        fields: {
            _contentLoaded: false
        },
        ctors: {
            ctor: function () {
                this.$initialize();
                Bridge.global.Windows.UI.Xaml.ResourceDictionary.ctor.call(this);
                this.InitializeComponent();

                if (Bridge.is(this, Windows.UI.Xaml.UIElement)) {
                    Bridge.cast(this, Windows.UI.Xaml.UIElement).XamlSourcePath = "CSHTML5\\DefaultStyles\\INTERNAL_DefaultAutoCompleteBoxStyle.xaml";
                }
            }
        },
        methods: {
            InitializeComponent: function () {
                var $t;
                if (this._contentLoaded) {
                    return;
                }
                this._contentLoaded = true;


                if (Bridge.is(this, Windows.UI.Xaml.UIElement)) {
                    Bridge.cast(this, Windows.UI.Xaml.UIElement).XamlSourcePath = "CSHTML5\\DefaultStyles\\INTERNAL_DefaultAutoCompleteBoxStyle.xaml";
                }



                var Style_10f3c8b40c9c4e6fbdce18e58ac79320 = new Bridge.global.Windows.UI.Xaml.Style.ctor();
                Style_10f3c8b40c9c4e6fbdce18e58ac79320.TargetType = Bridge.global.Windows.UI.Xaml.Controls.AutoCompleteBox;
                var Setter_ed2e6b47ea654cfeab8bf65bd031b861 = new Bridge.global.Windows.UI.Xaml.Setter.ctor();
                Setter_ed2e6b47ea654cfeab8bf65bd031b861.Property = Bridge.global.Windows.UI.Xaml.Controls.AutoCompleteBox.MinimumPrefixLengthProperty;
                Setter_ed2e6b47ea654cfeab8bf65bd031b861.Value = Bridge.box(1, System.Int32);

                var Setter_1182998bb76e4b0c82046416ad9643e0 = new Bridge.global.Windows.UI.Xaml.Setter.ctor();
                Setter_1182998bb76e4b0c82046416ad9643e0.Property = Bridge.global.Windows.UI.Xaml.Controls.AutoCompleteBox.MinimumPopulateDelayProperty;
                Setter_1182998bb76e4b0c82046416ad9643e0.Value = Bridge.box(0, System.Int32);

                var Setter_77d9c0ed8147407eb057b06487dbdf09 = new Bridge.global.Windows.UI.Xaml.Setter.ctor();
                Setter_77d9c0ed8147407eb057b06487dbdf09.Property = Bridge.global.Windows.UI.Xaml.Controls.Control.BackgroundProperty;
                Setter_77d9c0ed8147407eb057b06487dbdf09.Value = new Bridge.global.Windows.UI.Xaml.Media.SolidColorBrush.$ctor1(($t = new Bridge.global.Windows.UI.Color(), $t.A = 255, $t.R = 255, $t.G = 255, $t.B = 255, $t));

                var Setter_6f4751ded86445eda716b5f8461ac09f = new Bridge.global.Windows.UI.Xaml.Setter.ctor();
                Setter_6f4751ded86445eda716b5f8461ac09f.Property = Bridge.global.Windows.UI.Xaml.Controls.Control.ForegroundProperty;
                Setter_6f4751ded86445eda716b5f8461ac09f.Value = new Bridge.global.Windows.UI.Xaml.Media.SolidColorBrush.$ctor1(($t = new Bridge.global.Windows.UI.Color(), $t.A = 255, $t.R = 0, $t.G = 0, $t.B = 0, $t));

                var Setter_64227e707adc443f87bd42fb9d8bbb55 = new Bridge.global.Windows.UI.Xaml.Setter.ctor();
                Setter_64227e707adc443f87bd42fb9d8bbb55.Property = Bridge.global.Windows.UI.Xaml.Controls.Control.PaddingProperty;
                Setter_64227e707adc443f87bd42fb9d8bbb55.Value = new Bridge.global.Windows.UI.Xaml.Thickness.$ctor2(6, 2, 30, 2).$clone();

                var Setter_be3bd349c5444fb0bb04a58fd6b73618 = new Bridge.global.Windows.UI.Xaml.Setter.ctor();
                Setter_be3bd349c5444fb0bb04a58fd6b73618.Property = Bridge.global.Windows.UI.Xaml.FrameworkElement.CursorProperty;
                Setter_be3bd349c5444fb0bb04a58fd6b73618.Value = Bridge.global.System.Windows.Input.Cursors.Hand;

                var Setter_511cdcbdb6fd4f3997faf52c12834b89 = new Bridge.global.Windows.UI.Xaml.Setter.ctor();
                Setter_511cdcbdb6fd4f3997faf52c12834b89.Property = Bridge.global.Windows.UI.Xaml.Controls.Control.TemplateProperty;
                var ControlTemplate_10181bd2e13a4ee4946cfdff43383bd0 = new Bridge.global.Windows.UI.Xaml.Controls.ControlTemplate();
                ControlTemplate_10181bd2e13a4ee4946cfdff43383bd0.TargetType = Bridge.global.Windows.UI.Xaml.Controls.AutoCompleteBox;
                ControlTemplate_10181bd2e13a4ee4946cfdff43383bd0.SetMethodToInstantiateFrameworkTemplate(function (templateOwner_ControlTemplate_10181bd2e13a4ee4946cfdff43383bd0) {
                    var $t1;
                    var templateInstance_e6a83aa5da40435e81ed8cacdb815c75 = new Bridge.global.Windows.UI.Xaml.TemplateInstance();
                    templateInstance_e6a83aa5da40435e81ed8cacdb815c75.TemplateOwner = templateOwner_ControlTemplate_10181bd2e13a4ee4946cfdff43383bd0;
                    var Path_45cb4f2e2d1949ebaf69312114cdbe82 = new Bridge.global.Windows.UI.Xaml.Shapes.Path();
                    var ToggleButton_1ac9765a8c9c4fcfb148b62cbf4e5251 = new Bridge.global.Windows.UI.Xaml.Controls.Primitives.ToggleButton();
                    var TextBox_ac373d61539840d99804c2faa2a4a242 = new Bridge.global.Windows.UI.Xaml.Controls.TextBox();
                    var Border_50fb6c67d4dc4628a41e45ddf2b30e98 = new Bridge.global.Windows.UI.Xaml.Controls.Border();
                    var ItemsPresenter_5b5b17ff863849f09391166ae4abb9c1 = new Bridge.global.Windows.UI.Xaml.Controls.ItemsPresenter();
                    var Popup_43f53276b6f2499599c4450d6cec4ddc = new Bridge.global.Windows.UI.Xaml.Controls.Primitives.Popup();
                    var StackPanel_28bbf809e43c434eb089a7fe562c1799 = new Bridge.global.Windows.UI.Xaml.Controls.StackPanel();

                    templateOwner_ControlTemplate_10181bd2e13a4ee4946cfdff43383bd0.RegisterName("OuterBorder", Border_50fb6c67d4dc4628a41e45ddf2b30e98);
                    Border_50fb6c67d4dc4628a41e45ddf2b30e98.Name = "OuterBorder";
                    var Grid_e93b189432304cc4b85a62faac9bcc1a = new Bridge.global.Windows.UI.Xaml.Controls.Grid();
                    templateOwner_ControlTemplate_10181bd2e13a4ee4946cfdff43383bd0.RegisterName("DropDownToggle", ToggleButton_1ac9765a8c9c4fcfb148b62cbf4e5251);
                    ToggleButton_1ac9765a8c9c4fcfb148b62cbf4e5251.Name = "DropDownToggle";
                    ToggleButton_1ac9765a8c9c4fcfb148b62cbf4e5251.HorizontalAlignment = Bridge.global.Windows.UI.Xaml.HorizontalAlignment.Stretch;
                    ToggleButton_1ac9765a8c9c4fcfb148b62cbf4e5251.VerticalAlignment = Bridge.global.Windows.UI.Xaml.VerticalAlignment.Stretch;
                    ToggleButton_1ac9765a8c9c4fcfb148b62cbf4e5251.HorizontalContentAlignment = Bridge.global.Windows.UI.Xaml.HorizontalAlignment.Right;
                    ToggleButton_1ac9765a8c9c4fcfb148b62cbf4e5251.VerticalContentAlignment = Bridge.global.Windows.UI.Xaml.VerticalAlignment.Center;
                    ToggleButton_1ac9765a8c9c4fcfb148b62cbf4e5251.Visibility = Bridge.global.Windows.UI.Xaml.Visibility.Collapsed;
                    templateOwner_ControlTemplate_10181bd2e13a4ee4946cfdff43383bd0.RegisterName("arrow", Path_45cb4f2e2d1949ebaf69312114cdbe82);
                    Path_45cb4f2e2d1949ebaf69312114cdbe82.Name = "arrow";
                    Path_45cb4f2e2d1949ebaf69312114cdbe82.Visibility = Bridge.global.Windows.UI.Xaml.Visibility.Visible;
                    Path_45cb4f2e2d1949ebaf69312114cdbe82.Stroke = new Bridge.global.Windows.UI.Xaml.Media.SolidColorBrush.$ctor1(($t1 = new Bridge.global.Windows.UI.Color(), $t1.A = 255, $t1.R = 0, $t1.G = 0, $t1.B = 0, $t1));
                    Path_45cb4f2e2d1949ebaf69312114cdbe82.Width = 13.0;
                    Path_45cb4f2e2d1949ebaf69312114cdbe82.Height = 9.0;
                    Path_45cb4f2e2d1949ebaf69312114cdbe82.StrokeThickness = 3.0;
                    Path_45cb4f2e2d1949ebaf69312114cdbe82.HorizontalAlignment = Bridge.global.Windows.UI.Xaml.HorizontalAlignment.Center;
                    Path_45cb4f2e2d1949ebaf69312114cdbe82.VerticalAlignment = Bridge.global.Windows.UI.Xaml.VerticalAlignment.Center;
                    Path_45cb4f2e2d1949ebaf69312114cdbe82.Stretch = Bridge.global.Windows.UI.Xaml.Media.Stretch.Fill;
                    Path_45cb4f2e2d1949ebaf69312114cdbe82.Data = Bridge.cast(Bridge.global.DotNetForHtml5.Core.TypeFromStringConverters.ConvertFromInvariantString(Bridge.global.Windows.UI.Xaml.Media.Geometry, "M 1,1.5 L 4.5,5 L 8,1.5"), Windows.UI.Xaml.Media.Geometry);

                    ToggleButton_1ac9765a8c9c4fcfb148b62cbf4e5251.Content = Path_45cb4f2e2d1949ebaf69312114cdbe82;

                    var Binding_fba67ec1c7a54d5795f3b76efbb52b26 = new Bridge.global.Windows.UI.Xaml.Data.Binding.ctor();
                    Binding_fba67ec1c7a54d5795f3b76efbb52b26.Path = new Bridge.global.Windows.UI.Xaml.PropertyPath.ctor("IsDropDownOpen");
                    Binding_fba67ec1c7a54d5795f3b76efbb52b26.Mode = Bridge.global.Windows.UI.Xaml.Data.BindingMode.TwoWay;
                    var RelativeSource_a5a7320416a5402f8d4403fcbe61c1f8 = new Bridge.global.Windows.UI.Xaml.Data.RelativeSource.ctor();
                    RelativeSource_a5a7320416a5402f8d4403fcbe61c1f8.Mode = Bridge.global.Windows.UI.Xaml.Data.RelativeSourceMode.TemplatedParent;

                    Binding_fba67ec1c7a54d5795f3b76efbb52b26.RelativeSource = RelativeSource_a5a7320416a5402f8d4403fcbe61c1f8;


                    Binding_fba67ec1c7a54d5795f3b76efbb52b26.TemplateOwner = templateInstance_e6a83aa5da40435e81ed8cacdb815c75;


                    templateOwner_ControlTemplate_10181bd2e13a4ee4946cfdff43383bd0.RegisterName("PART_TextBox", TextBox_ac373d61539840d99804c2faa2a4a242);
                    TextBox_ac373d61539840d99804c2faa2a4a242.Name = "PART_TextBox";
                    TextBox_ac373d61539840d99804c2faa2a4a242.TextWrapping = Bridge.global.Windows.UI.Xaml.TextWrapping.NoWrap;
                    var Binding_4cc70488424f465eb8a5c9fe18429192 = new Bridge.global.Windows.UI.Xaml.Data.Binding.ctor();
                    Binding_4cc70488424f465eb8a5c9fe18429192.Path = new Bridge.global.Windows.UI.Xaml.PropertyPath.ctor("Text");
                    Binding_4cc70488424f465eb8a5c9fe18429192.Mode = Bridge.global.Windows.UI.Xaml.Data.BindingMode.TwoWay;
                    var RelativeSource_9c7708ab3d144d9994adbda1025c99fe = new Bridge.global.Windows.UI.Xaml.Data.RelativeSource.ctor();
                    RelativeSource_9c7708ab3d144d9994adbda1025c99fe.Mode = Bridge.global.Windows.UI.Xaml.Data.RelativeSourceMode.TemplatedParent;

                    Binding_4cc70488424f465eb8a5c9fe18429192.RelativeSource = RelativeSource_9c7708ab3d144d9994adbda1025c99fe;


                    Binding_4cc70488424f465eb8a5c9fe18429192.TemplateOwner = templateInstance_e6a83aa5da40435e81ed8cacdb815c75;


                    Grid_e93b189432304cc4b85a62faac9bcc1a.Children.add(ToggleButton_1ac9765a8c9c4fcfb148b62cbf4e5251);
                    Grid_e93b189432304cc4b85a62faac9bcc1a.Children.add(TextBox_ac373d61539840d99804c2faa2a4a242);


                    Border_50fb6c67d4dc4628a41e45ddf2b30e98.Child = Grid_e93b189432304cc4b85a62faac9bcc1a;

                    var Binding_e9586d75e1c84b069e2464d2199f5f78 = new Bridge.global.Windows.UI.Xaml.Data.Binding.ctor();
                    Binding_e9586d75e1c84b069e2464d2199f5f78.Path = new Bridge.global.Windows.UI.Xaml.PropertyPath.ctor("Background");
                    var RelativeSource_db5fe48947b240019b2acb1ed3dee36e = new Bridge.global.Windows.UI.Xaml.Data.RelativeSource.ctor();
                    RelativeSource_db5fe48947b240019b2acb1ed3dee36e.Mode = Bridge.global.Windows.UI.Xaml.Data.RelativeSourceMode.TemplatedParent;

                    Binding_e9586d75e1c84b069e2464d2199f5f78.RelativeSource = RelativeSource_db5fe48947b240019b2acb1ed3dee36e;


                    Binding_e9586d75e1c84b069e2464d2199f5f78.TemplateOwner = templateInstance_e6a83aa5da40435e81ed8cacdb815c75;

                    var Binding_fd69f95034fb471aa3fa8aae6ef12c19 = new Bridge.global.Windows.UI.Xaml.Data.Binding.ctor();
                    Binding_fd69f95034fb471aa3fa8aae6ef12c19.Path = new Bridge.global.Windows.UI.Xaml.PropertyPath.ctor("BorderBrush");
                    var RelativeSource_0d9efca248f84e8f97f98c2aab3313ae = new Bridge.global.Windows.UI.Xaml.Data.RelativeSource.ctor();
                    RelativeSource_0d9efca248f84e8f97f98c2aab3313ae.Mode = Bridge.global.Windows.UI.Xaml.Data.RelativeSourceMode.TemplatedParent;

                    Binding_fd69f95034fb471aa3fa8aae6ef12c19.RelativeSource = RelativeSource_0d9efca248f84e8f97f98c2aab3313ae;


                    Binding_fd69f95034fb471aa3fa8aae6ef12c19.TemplateOwner = templateInstance_e6a83aa5da40435e81ed8cacdb815c75;

                    var Binding_7efac791c8d141ddbb14574b2cc1b68a = new Bridge.global.Windows.UI.Xaml.Data.Binding.ctor();
                    Binding_7efac791c8d141ddbb14574b2cc1b68a.Path = new Bridge.global.Windows.UI.Xaml.PropertyPath.ctor("BorderThickness");
                    var RelativeSource_6beabd3701c64d6a816e965ebd7e8355 = new Bridge.global.Windows.UI.Xaml.Data.RelativeSource.ctor();
                    RelativeSource_6beabd3701c64d6a816e965ebd7e8355.Mode = Bridge.global.Windows.UI.Xaml.Data.RelativeSourceMode.TemplatedParent;

                    Binding_7efac791c8d141ddbb14574b2cc1b68a.RelativeSource = RelativeSource_6beabd3701c64d6a816e965ebd7e8355;


                    Binding_7efac791c8d141ddbb14574b2cc1b68a.TemplateOwner = templateInstance_e6a83aa5da40435e81ed8cacdb815c75;


                    templateOwner_ControlTemplate_10181bd2e13a4ee4946cfdff43383bd0.RegisterName("Popup", Popup_43f53276b6f2499599c4450d6cec4ddc);
                    Popup_43f53276b6f2499599c4450d6cec4ddc.Name = "Popup";
                    Popup_43f53276b6f2499599c4450d6cec4ddc.IsOpen = false;
                    var Border_ef5d4a69c4d04b6e8b00bb11c5be5ec4 = new Bridge.global.Windows.UI.Xaml.Controls.Border();
                    Border_ef5d4a69c4d04b6e8b00bb11c5be5ec4.Background = new Bridge.global.Windows.UI.Xaml.Media.SolidColorBrush.$ctor1(($t1 = new Bridge.global.Windows.UI.Color(), $t1.A = 255, $t1.R = 255, $t1.G = 255, $t1.B = 255, $t1));
                    Border_ef5d4a69c4d04b6e8b00bb11c5be5ec4.BorderBrush = new Bridge.global.Windows.UI.Xaml.Media.SolidColorBrush.$ctor1(($t1 = new Bridge.global.Windows.UI.Color(), $t1.A = 255, $t1.R = 0, $t1.G = 0, $t1.B = 0, $t1));
                    Border_ef5d4a69c4d04b6e8b00bb11c5be5ec4.BorderThickness = new Bridge.global.Windows.UI.Xaml.Thickness.$ctor1(1);
                    var ScrollViewer_14f14c368fc746a4a05eaead3b77caae = new Bridge.global.Windows.UI.Xaml.Controls.ScrollViewer();
                    ScrollViewer_14f14c368fc746a4a05eaead3b77caae.VerticalScrollBarVisibility = Bridge.global.Windows.UI.Xaml.Controls.ScrollBarVisibility.Auto;
                    ScrollViewer_14f14c368fc746a4a05eaead3b77caae.HorizontalScrollBarVisibility = Bridge.global.Windows.UI.Xaml.Controls.ScrollBarVisibility.Disabled;
                    templateOwner_ControlTemplate_10181bd2e13a4ee4946cfdff43383bd0.RegisterName("ItemsHost", ItemsPresenter_5b5b17ff863849f09391166ae4abb9c1);
                    ItemsPresenter_5b5b17ff863849f09391166ae4abb9c1.Name = "ItemsHost";

                    ScrollViewer_14f14c368fc746a4a05eaead3b77caae.Content = ItemsPresenter_5b5b17ff863849f09391166ae4abb9c1;

                    var Binding_97eaffcb13b3430e9ba69e9c84ee5eb0 = new Bridge.global.Windows.UI.Xaml.Data.Binding.ctor();
                    Binding_97eaffcb13b3430e9ba69e9c84ee5eb0.Path = new Bridge.global.Windows.UI.Xaml.PropertyPath.ctor("MaxDropDownHeight");
                    var RelativeSource_4ee24fc40d164ef08b03f009f95f5d19 = new Bridge.global.Windows.UI.Xaml.Data.RelativeSource.ctor();
                    RelativeSource_4ee24fc40d164ef08b03f009f95f5d19.Mode = Bridge.global.Windows.UI.Xaml.Data.RelativeSourceMode.TemplatedParent;

                    Binding_97eaffcb13b3430e9ba69e9c84ee5eb0.RelativeSource = RelativeSource_4ee24fc40d164ef08b03f009f95f5d19;


                    Binding_97eaffcb13b3430e9ba69e9c84ee5eb0.TemplateOwner = templateInstance_e6a83aa5da40435e81ed8cacdb815c75;


                    Border_ef5d4a69c4d04b6e8b00bb11c5be5ec4.Child = ScrollViewer_14f14c368fc746a4a05eaead3b77caae;


                    Popup_43f53276b6f2499599c4450d6cec4ddc.Child = Border_ef5d4a69c4d04b6e8b00bb11c5be5ec4;


                    StackPanel_28bbf809e43c434eb089a7fe562c1799.Children.add(Border_50fb6c67d4dc4628a41e45ddf2b30e98);
                    StackPanel_28bbf809e43c434eb089a7fe562c1799.Children.add(Popup_43f53276b6f2499599c4450d6cec4ddc);


                    Bridge.global.Windows.UI.Xaml.Data.BindingOperations.SetBinding(ToggleButton_1ac9765a8c9c4fcfb148b62cbf4e5251, Bridge.global.Windows.UI.Xaml.Controls.Primitives.ToggleButton.IsCheckedProperty, Binding_fba67ec1c7a54d5795f3b76efbb52b26);
                    Bridge.global.Windows.UI.Xaml.Data.BindingOperations.SetBinding(TextBox_ac373d61539840d99804c2faa2a4a242, Bridge.global.Windows.UI.Xaml.Controls.TextBox.TextProperty, Binding_4cc70488424f465eb8a5c9fe18429192);
                    Bridge.global.Windows.UI.Xaml.Data.BindingOperations.SetBinding(Border_50fb6c67d4dc4628a41e45ddf2b30e98, Bridge.global.Windows.UI.Xaml.Controls.Border.BackgroundProperty, Binding_e9586d75e1c84b069e2464d2199f5f78);
                    Bridge.global.Windows.UI.Xaml.Data.BindingOperations.SetBinding(Border_50fb6c67d4dc4628a41e45ddf2b30e98, Bridge.global.Windows.UI.Xaml.Controls.Border.BorderBrushProperty, Binding_fd69f95034fb471aa3fa8aae6ef12c19);
                    Bridge.global.Windows.UI.Xaml.Data.BindingOperations.SetBinding(Border_50fb6c67d4dc4628a41e45ddf2b30e98, Bridge.global.Windows.UI.Xaml.Controls.Border.BorderThicknessProperty, Binding_7efac791c8d141ddbb14574b2cc1b68a);
                    Bridge.global.Windows.UI.Xaml.Data.BindingOperations.SetBinding(ScrollViewer_14f14c368fc746a4a05eaead3b77caae, Bridge.global.Windows.UI.Xaml.FrameworkElement.MaxHeightProperty, Binding_97eaffcb13b3430e9ba69e9c84ee5eb0);

                    templateInstance_e6a83aa5da40435e81ed8cacdb815c75.TemplateContent = StackPanel_28bbf809e43c434eb089a7fe562c1799;
                    return templateInstance_e6a83aa5da40435e81ed8cacdb815c75;
                });

                Setter_511cdcbdb6fd4f3997faf52c12834b89.Value = ControlTemplate_10181bd2e13a4ee4946cfdff43383bd0;


                Style_10f3c8b40c9c4e6fbdce18e58ac79320.Setters.add(Setter_ed2e6b47ea654cfeab8bf65bd031b861);
                Style_10f3c8b40c9c4e6fbdce18e58ac79320.Setters.add(Setter_1182998bb76e4b0c82046416ad9643e0);
                Style_10f3c8b40c9c4e6fbdce18e58ac79320.Setters.add(Setter_77d9c0ed8147407eb057b06487dbdf09);
                Style_10f3c8b40c9c4e6fbdce18e58ac79320.Setters.add(Setter_6f4751ded86445eda716b5f8461ac09f);
                Style_10f3c8b40c9c4e6fbdce18e58ac79320.Setters.add(Setter_64227e707adc443f87bd42fb9d8bbb55);
                Style_10f3c8b40c9c4e6fbdce18e58ac79320.Setters.add(Setter_be3bd349c5444fb0bb04a58fd6b73618);
                Style_10f3c8b40c9c4e6fbdce18e58ac79320.Setters.add(Setter_511cdcbdb6fd4f3997faf52c12834b89);


                this.setItem(Bridge.global.Windows.UI.Xaml.Controls.AutoCompleteBox, Style_10f3c8b40c9c4e6fbdce18e58ac79320);






            }
        }
    });

    Bridge.define("___CSHTML5__Component__Defaultstyles__INTERNAL_Defaultbusyindicatorstyle__Xaml", {
        inherits: [Bridge.global.Windows.UI.Xaml.ResourceDictionary],
        fields: {
            _contentLoaded: false
        },
        ctors: {
            ctor: function () {
                this.$initialize();
                Bridge.global.Windows.UI.Xaml.ResourceDictionary.ctor.call(this);
                this.InitializeComponent();

                if (Bridge.is(this, Windows.UI.Xaml.UIElement)) {
                    Bridge.cast(this, Windows.UI.Xaml.UIElement).XamlSourcePath = "CSHTML5\\DefaultStyles\\INTERNAL_DefaultBusyIndicatorStyle.xaml";
                }
            }
        },
        methods: {
            InitializeComponent: function () {
                var $t;
                if (this._contentLoaded) {
                    return;
                }
                this._contentLoaded = true;


                if (Bridge.is(this, Windows.UI.Xaml.UIElement)) {
                    Bridge.cast(this, Windows.UI.Xaml.UIElement).XamlSourcePath = "CSHTML5\\DefaultStyles\\INTERNAL_DefaultBusyIndicatorStyle.xaml";
                }



                var Style_c785d4ad336b4a6e8a1a94b703b18591 = new Bridge.global.Windows.UI.Xaml.Style.ctor();
                Style_c785d4ad336b4a6e8a1a94b703b18591.TargetType = Bridge.global.Windows.UI.Xaml.Controls.BusyIndicator;
                var Setter_42621b6c23cf4c769d42ae5c45e7ab12 = new Bridge.global.Windows.UI.Xaml.Setter.ctor();
                Setter_42621b6c23cf4c769d42ae5c45e7ab12.Property = Bridge.global.Windows.UI.Xaml.Controls.BusyIndicator.BusyContentProperty;
                Setter_42621b6c23cf4c769d42ae5c45e7ab12.Value = "Please wait...";

                var Setter_0e2e3f369d074584ba01ddc5b578fa10 = new Bridge.global.Windows.UI.Xaml.Setter.ctor();
                Setter_0e2e3f369d074584ba01ddc5b578fa10.Property = Bridge.global.Windows.UI.Xaml.Controls.Control.IsTabStopProperty;
                Setter_0e2e3f369d074584ba01ddc5b578fa10.Value = Bridge.box(false, System.Boolean, System.Boolean.toString);

                var Setter_c3b542abae894b7eaf5889467dec072b = new Bridge.global.Windows.UI.Xaml.Setter.ctor();
                Setter_c3b542abae894b7eaf5889467dec072b.Property = Bridge.global.Windows.UI.Xaml.Controls.BusyIndicator.OverlayStyleProperty;
                var Style_ed5d06d72eb34b81af72571febf72e67 = new Bridge.global.Windows.UI.Xaml.Style.ctor();
                Style_ed5d06d72eb34b81af72571febf72e67.TargetType = Bridge.global.Windows.UI.Xaml.Shapes.Rectangle;
                var Setter_c7bb2c02c23a4398b6e4304a91fcf4f1 = new Bridge.global.Windows.UI.Xaml.Setter.ctor();
                Setter_c7bb2c02c23a4398b6e4304a91fcf4f1.Property = Bridge.global.Windows.UI.Xaml.Shapes.Shape.FillProperty;
                Setter_c7bb2c02c23a4398b6e4304a91fcf4f1.Value = new Bridge.global.Windows.UI.Xaml.Media.SolidColorBrush.$ctor1(($t = new Bridge.global.Windows.UI.Color(), $t.A = 255, $t.R = 255, $t.G = 255, $t.B = 255, $t));

                var Setter_c5353f71cbb34017822c88d0db31465b = new Bridge.global.Windows.UI.Xaml.Setter.ctor();
                Setter_c5353f71cbb34017822c88d0db31465b.Property = Bridge.global.Windows.UI.Xaml.UIElement.OpacityProperty;
                Setter_c5353f71cbb34017822c88d0db31465b.Value = Bridge.box(0.5, System.Double, System.Double.format, System.Double.getHashCode);

                Style_ed5d06d72eb34b81af72571febf72e67.Setters.add(Setter_c7bb2c02c23a4398b6e4304a91fcf4f1);
                Style_ed5d06d72eb34b81af72571febf72e67.Setters.add(Setter_c5353f71cbb34017822c88d0db31465b);


                Setter_c3b542abae894b7eaf5889467dec072b.Value = Style_ed5d06d72eb34b81af72571febf72e67;


                var Setter_fdb52f4419b5430ca03f8673d5c3fc02 = new Bridge.global.Windows.UI.Xaml.Setter.ctor();
                Setter_fdb52f4419b5430ca03f8673d5c3fc02.Property = Bridge.global.Windows.UI.Xaml.Controls.BusyIndicator.DisplayAfterProperty;
                Setter_fdb52f4419b5430ca03f8673d5c3fc02.Value = System.Nullable.getValue(Bridge.cast(Bridge.unbox(Bridge.global.DotNetForHtml5.Core.TypeFromStringConverters.ConvertFromInvariantString(Bridge.global.System.TimeSpan, "00:00:00.1"), System.TimeSpan), System.TimeSpan));

                var Setter_e9df22b7b52f4a0f999863e5d6708857 = new Bridge.global.Windows.UI.Xaml.Setter.ctor();
                Setter_e9df22b7b52f4a0f999863e5d6708857.Property = Bridge.global.Windows.UI.Xaml.FrameworkElement.HorizontalAlignmentProperty;
                Setter_e9df22b7b52f4a0f999863e5d6708857.Value = Bridge.box(Bridge.global.Windows.UI.Xaml.HorizontalAlignment.Stretch, Windows.UI.Xaml.HorizontalAlignment, System.Enum.toStringFn(Windows.UI.Xaml.HorizontalAlignment));

                var Setter_5d89abb97ed448aca49de902a2e279e4 = new Bridge.global.Windows.UI.Xaml.Setter.ctor();
                Setter_5d89abb97ed448aca49de902a2e279e4.Property = Bridge.global.Windows.UI.Xaml.FrameworkElement.VerticalAlignmentProperty;
                Setter_5d89abb97ed448aca49de902a2e279e4.Value = Bridge.box(Bridge.global.Windows.UI.Xaml.VerticalAlignment.Stretch, Windows.UI.Xaml.VerticalAlignment, System.Enum.toStringFn(Windows.UI.Xaml.VerticalAlignment));

                var Setter_4cf2a2102851473a9a61ee5591252721 = new Bridge.global.Windows.UI.Xaml.Setter.ctor();
                Setter_4cf2a2102851473a9a61ee5591252721.Property = Bridge.global.Windows.UI.Xaml.Controls.Control.HorizontalContentAlignmentProperty;
                Setter_4cf2a2102851473a9a61ee5591252721.Value = Bridge.box(Bridge.global.Windows.UI.Xaml.HorizontalAlignment.Stretch, Windows.UI.Xaml.HorizontalAlignment, System.Enum.toStringFn(Windows.UI.Xaml.HorizontalAlignment));

                var Setter_ebd3297f0f184d82b0d2c5931ac63411 = new Bridge.global.Windows.UI.Xaml.Setter.ctor();
                Setter_ebd3297f0f184d82b0d2c5931ac63411.Property = Bridge.global.Windows.UI.Xaml.Controls.Control.VerticalContentAlignmentProperty;
                Setter_ebd3297f0f184d82b0d2c5931ac63411.Value = Bridge.box(Bridge.global.Windows.UI.Xaml.VerticalAlignment.Stretch, Windows.UI.Xaml.VerticalAlignment, System.Enum.toStringFn(Windows.UI.Xaml.VerticalAlignment));

                var Setter_018b41aa20ee4908a7987d7a2d7c7b4e = new Bridge.global.Windows.UI.Xaml.Setter.ctor();
                Setter_018b41aa20ee4908a7987d7a2d7c7b4e.Property = Bridge.global.Windows.UI.Xaml.Controls.Control.TemplateProperty;
                var ControlTemplate_5dcd43438f6a4c0b8ded1af5dd32feb5 = new Bridge.global.Windows.UI.Xaml.Controls.ControlTemplate();
                ControlTemplate_5dcd43438f6a4c0b8ded1af5dd32feb5.TargetType = Bridge.global.Windows.UI.Xaml.Controls.BusyIndicator;
                ControlTemplate_5dcd43438f6a4c0b8ded1af5dd32feb5.SetMethodToInstantiateFrameworkTemplate(Bridge.fn.bind(this, function (templateOwner_ControlTemplate_5dcd43438f6a4c0b8ded1af5dd32feb5) {
                    var $t1;
                    var templateInstance_e22fb51283ef45f3bf8645186d1ff5ca = new Bridge.global.Windows.UI.Xaml.TemplateInstance();
                    templateInstance_e22fb51283ef45f3bf8645186d1ff5ca.TemplateOwner = templateOwner_ControlTemplate_5dcd43438f6a4c0b8ded1af5dd32feb5;
                    var VisualState_0bd6fde8a5bd40129284bf7f0e5e1497 = new Bridge.global.Windows.UI.Xaml.VisualState();
                    var VisualState_b945c5f97a05407fad00d01c129afc6f = new Bridge.global.Windows.UI.Xaml.VisualState();
                    var VisualStateGroup_befe61b704664f4da8dbf8c0d178aaf5 = new Bridge.global.Windows.UI.Xaml.VisualStateGroup();
                    var VisualState_80fac42af14d422aa0ad230d0e03e558 = new Bridge.global.Windows.UI.Xaml.VisualState();
                    var VisualState_c84ca299ab9640919a9b1f36527a31e2 = new Bridge.global.Windows.UI.Xaml.VisualState();
                    var VisualStateGroup_b97532da7afc4e9b857fc170581185ff = new Bridge.global.Windows.UI.Xaml.VisualStateGroup();
                    var ContentControl_3c046ef32b824f8286f3818729f1f1da = new Bridge.global.Windows.UI.Xaml.Controls.ContentControl();
                    var Rectangle_f1538946a28447e4844d211e5d84bb09 = new Bridge.global.Windows.UI.Xaml.Shapes.Rectangle();
                    var Border_cfd20111f0ec4a0cbbadd7e775cc0069 = new Bridge.global.Windows.UI.Xaml.Controls.Border();
                    var Grid_60ad010f75754f95a256032db5e3c722 = new Bridge.global.Windows.UI.Xaml.Controls.Grid();
                    templateOwner_ControlTemplate_5dcd43438f6a4c0b8ded1af5dd32feb5.RegisterName("VisibilityStates", VisualStateGroup_befe61b704664f4da8dbf8c0d178aaf5);
                    VisualStateGroup_befe61b704664f4da8dbf8c0d178aaf5.Name = "VisibilityStates";
                    templateOwner_ControlTemplate_5dcd43438f6a4c0b8ded1af5dd32feb5.RegisterName("Hidden", VisualState_0bd6fde8a5bd40129284bf7f0e5e1497);
                    VisualState_0bd6fde8a5bd40129284bf7f0e5e1497.Name = "Hidden";
                    var Storyboard_46cf4aadc5ba4ffb8518f382e945a5b7 = new Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard();
                    var ObjectAnimationUsingKeyFrames_e7eea553b9d346b2af3ee0c2847d7370 = new Bridge.global.Windows.UI.Xaml.Media.Animation.ObjectAnimationUsingKeyFrames();
                    ObjectAnimationUsingKeyFrames_e7eea553b9d346b2af3ee0c2847d7370.BeginTime = System.Nullable.getValue(Bridge.cast(Bridge.unbox(Bridge.global.DotNetForHtml5.Core.TypeFromStringConverters.ConvertFromInvariantString(Bridge.global.System.TimeSpan, "00:00:00"), System.TimeSpan), System.TimeSpan));
                    ObjectAnimationUsingKeyFrames_e7eea553b9d346b2af3ee0c2847d7370.Duration = new Bridge.global.Windows.UI.Xaml.Duration.$ctor1(new Bridge.global.System.TimeSpan(System.Int64(10000)));
                    Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard.SetTargetName(ObjectAnimationUsingKeyFrames_e7eea553b9d346b2af3ee0c2847d7370, "busycontent");
                    var DiscreteObjectKeyFrame_e65ad26b23944dacb04e98ece0fdbc7f = new Bridge.global.Windows.UI.Xaml.Media.Animation.DiscreteObjectKeyFrame();
                    DiscreteObjectKeyFrame_e65ad26b23944dacb04e98ece0fdbc7f.KeyTime = Bridge.global.Windows.UI.Xaml.Media.Animation.KeyTime.FromTimeSpan(new Bridge.global.System.TimeSpan(System.Int64(0)));
                    var Visibility_8f2269db21884443a7f803f70c17edad = System.Nullable.getValue(Bridge.cast(Bridge.unbox(Bridge.global.DotNetForHtml5.Core.TypeFromStringConverters.ConvertFromInvariantString(Bridge.global.Windows.UI.Xaml.Visibility, "Collapsed"), Windows.UI.Xaml.Visibility), System.Int32));

                    DiscreteObjectKeyFrame_e65ad26b23944dacb04e98ece0fdbc7f.Value = Bridge.box(Visibility_8f2269db21884443a7f803f70c17edad, Windows.UI.Xaml.Visibility, System.Enum.toStringFn(Windows.UI.Xaml.Visibility));


                    ObjectAnimationUsingKeyFrames_e7eea553b9d346b2af3ee0c2847d7370.KeyFrames.add(DiscreteObjectKeyFrame_e65ad26b23944dacb04e98ece0fdbc7f);


                    var ObjectAnimationUsingKeyFrames_7a7d6195a8684434b30581b42eeb7f13 = new Bridge.global.Windows.UI.Xaml.Media.Animation.ObjectAnimationUsingKeyFrames();
                    ObjectAnimationUsingKeyFrames_7a7d6195a8684434b30581b42eeb7f13.BeginTime = System.Nullable.getValue(Bridge.cast(Bridge.unbox(Bridge.global.DotNetForHtml5.Core.TypeFromStringConverters.ConvertFromInvariantString(Bridge.global.System.TimeSpan, "00:00:00"), System.TimeSpan), System.TimeSpan));
                    ObjectAnimationUsingKeyFrames_7a7d6195a8684434b30581b42eeb7f13.Duration = new Bridge.global.Windows.UI.Xaml.Duration.$ctor1(new Bridge.global.System.TimeSpan(System.Int64(10000)));
                    Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard.SetTargetName(ObjectAnimationUsingKeyFrames_7a7d6195a8684434b30581b42eeb7f13, "overlay");
                    var DiscreteObjectKeyFrame_b2d11773cdcc48beb416a622bb9b2340 = new Bridge.global.Windows.UI.Xaml.Media.Animation.DiscreteObjectKeyFrame();
                    DiscreteObjectKeyFrame_b2d11773cdcc48beb416a622bb9b2340.KeyTime = Bridge.global.Windows.UI.Xaml.Media.Animation.KeyTime.FromTimeSpan(new Bridge.global.System.TimeSpan(System.Int64(0)));
                    var Visibility_77d699e64e7f42609362a0da888f838f = System.Nullable.getValue(Bridge.cast(Bridge.unbox(Bridge.global.DotNetForHtml5.Core.TypeFromStringConverters.ConvertFromInvariantString(Bridge.global.Windows.UI.Xaml.Visibility, "Collapsed"), Windows.UI.Xaml.Visibility), System.Int32));

                    DiscreteObjectKeyFrame_b2d11773cdcc48beb416a622bb9b2340.Value = Bridge.box(Visibility_77d699e64e7f42609362a0da888f838f, Windows.UI.Xaml.Visibility, System.Enum.toStringFn(Windows.UI.Xaml.Visibility));


                    ObjectAnimationUsingKeyFrames_7a7d6195a8684434b30581b42eeb7f13.KeyFrames.add(DiscreteObjectKeyFrame_b2d11773cdcc48beb416a622bb9b2340);


                    Storyboard_46cf4aadc5ba4ffb8518f382e945a5b7.Children.add(ObjectAnimationUsingKeyFrames_e7eea553b9d346b2af3ee0c2847d7370);
                    Storyboard_46cf4aadc5ba4ffb8518f382e945a5b7.Children.add(ObjectAnimationUsingKeyFrames_7a7d6195a8684434b30581b42eeb7f13);


                    VisualState_0bd6fde8a5bd40129284bf7f0e5e1497.Storyboard = Storyboard_46cf4aadc5ba4ffb8518f382e945a5b7;


                    templateOwner_ControlTemplate_5dcd43438f6a4c0b8ded1af5dd32feb5.RegisterName("Visible", VisualState_b945c5f97a05407fad00d01c129afc6f);
                    VisualState_b945c5f97a05407fad00d01c129afc6f.Name = "Visible";
                    var Storyboard_59d6ab187a5144279a4541cd5a2a6aa0 = new Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard();
                    var ObjectAnimationUsingKeyFrames_14a5e3c6738b4ab8833b60934c75beb0 = new Bridge.global.Windows.UI.Xaml.Media.Animation.ObjectAnimationUsingKeyFrames();
                    ObjectAnimationUsingKeyFrames_14a5e3c6738b4ab8833b60934c75beb0.BeginTime = System.Nullable.getValue(Bridge.cast(Bridge.unbox(Bridge.global.DotNetForHtml5.Core.TypeFromStringConverters.ConvertFromInvariantString(Bridge.global.System.TimeSpan, "00:00:00"), System.TimeSpan), System.TimeSpan));
                    ObjectAnimationUsingKeyFrames_14a5e3c6738b4ab8833b60934c75beb0.Duration = new Bridge.global.Windows.UI.Xaml.Duration.$ctor1(new Bridge.global.System.TimeSpan(System.Int64(10000)));
                    Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard.SetTargetName(ObjectAnimationUsingKeyFrames_14a5e3c6738b4ab8833b60934c75beb0, "busycontent");
                    var DiscreteObjectKeyFrame_7a8568ea107449699245b080a4d89272 = new Bridge.global.Windows.UI.Xaml.Media.Animation.DiscreteObjectKeyFrame();
                    DiscreteObjectKeyFrame_7a8568ea107449699245b080a4d89272.KeyTime = Bridge.global.Windows.UI.Xaml.Media.Animation.KeyTime.FromTimeSpan(new Bridge.global.System.TimeSpan(System.Int64(0)));
                    var Visibility_cf59bea1beec435a847b34f220db861b = System.Nullable.getValue(Bridge.cast(Bridge.unbox(Bridge.global.DotNetForHtml5.Core.TypeFromStringConverters.ConvertFromInvariantString(Bridge.global.Windows.UI.Xaml.Visibility, "Visible"), Windows.UI.Xaml.Visibility), System.Int32));

                    DiscreteObjectKeyFrame_7a8568ea107449699245b080a4d89272.Value = Bridge.box(Visibility_cf59bea1beec435a847b34f220db861b, Windows.UI.Xaml.Visibility, System.Enum.toStringFn(Windows.UI.Xaml.Visibility));


                    ObjectAnimationUsingKeyFrames_14a5e3c6738b4ab8833b60934c75beb0.KeyFrames.add(DiscreteObjectKeyFrame_7a8568ea107449699245b080a4d89272);


                    var ObjectAnimationUsingKeyFrames_a6a96913d284479694d8bfbfc76f293e = new Bridge.global.Windows.UI.Xaml.Media.Animation.ObjectAnimationUsingKeyFrames();
                    ObjectAnimationUsingKeyFrames_a6a96913d284479694d8bfbfc76f293e.BeginTime = System.Nullable.getValue(Bridge.cast(Bridge.unbox(Bridge.global.DotNetForHtml5.Core.TypeFromStringConverters.ConvertFromInvariantString(Bridge.global.System.TimeSpan, "00:00:00"), System.TimeSpan), System.TimeSpan));
                    ObjectAnimationUsingKeyFrames_a6a96913d284479694d8bfbfc76f293e.Duration = new Bridge.global.Windows.UI.Xaml.Duration.$ctor1(new Bridge.global.System.TimeSpan(System.Int64(10000)));
                    Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard.SetTargetName(ObjectAnimationUsingKeyFrames_a6a96913d284479694d8bfbfc76f293e, "overlay");
                    var DiscreteObjectKeyFrame_35d44de5df9b428a96eebacbd4230d3f = new Bridge.global.Windows.UI.Xaml.Media.Animation.DiscreteObjectKeyFrame();
                    DiscreteObjectKeyFrame_35d44de5df9b428a96eebacbd4230d3f.KeyTime = Bridge.global.Windows.UI.Xaml.Media.Animation.KeyTime.FromTimeSpan(new Bridge.global.System.TimeSpan(System.Int64(0)));
                    var Visibility_bd9025c69b6840a28b21be8790846489 = System.Nullable.getValue(Bridge.cast(Bridge.unbox(Bridge.global.DotNetForHtml5.Core.TypeFromStringConverters.ConvertFromInvariantString(Bridge.global.Windows.UI.Xaml.Visibility, "Visible"), Windows.UI.Xaml.Visibility), System.Int32));

                    DiscreteObjectKeyFrame_35d44de5df9b428a96eebacbd4230d3f.Value = Bridge.box(Visibility_bd9025c69b6840a28b21be8790846489, Windows.UI.Xaml.Visibility, System.Enum.toStringFn(Windows.UI.Xaml.Visibility));


                    ObjectAnimationUsingKeyFrames_a6a96913d284479694d8bfbfc76f293e.KeyFrames.add(DiscreteObjectKeyFrame_35d44de5df9b428a96eebacbd4230d3f);


                    Storyboard_59d6ab187a5144279a4541cd5a2a6aa0.Children.add(ObjectAnimationUsingKeyFrames_14a5e3c6738b4ab8833b60934c75beb0);
                    Storyboard_59d6ab187a5144279a4541cd5a2a6aa0.Children.add(ObjectAnimationUsingKeyFrames_a6a96913d284479694d8bfbfc76f293e);


                    VisualState_b945c5f97a05407fad00d01c129afc6f.Storyboard = Storyboard_59d6ab187a5144279a4541cd5a2a6aa0;


                    System.Array.add(VisualStateGroup_befe61b704664f4da8dbf8c0d178aaf5.States, VisualState_0bd6fde8a5bd40129284bf7f0e5e1497, Object);
                    System.Array.add(VisualStateGroup_befe61b704664f4da8dbf8c0d178aaf5.States, VisualState_b945c5f97a05407fad00d01c129afc6f, Object);


                    templateOwner_ControlTemplate_5dcd43438f6a4c0b8ded1af5dd32feb5.RegisterName("BusyStatusStates", VisualStateGroup_b97532da7afc4e9b857fc170581185ff);
                    VisualStateGroup_b97532da7afc4e9b857fc170581185ff.Name = "BusyStatusStates";
                    templateOwner_ControlTemplate_5dcd43438f6a4c0b8ded1af5dd32feb5.RegisterName("Idle", VisualState_80fac42af14d422aa0ad230d0e03e558);
                    VisualState_80fac42af14d422aa0ad230d0e03e558.Name = "Idle";
                    var Storyboard_0b190bde9e744719bcc75b25d8bc72da = new Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard();
                    var ObjectAnimationUsingKeyFrames_aee92a37d81a402792b89286802eb1bc = new Bridge.global.Windows.UI.Xaml.Media.Animation.ObjectAnimationUsingKeyFrames();
                    ObjectAnimationUsingKeyFrames_aee92a37d81a402792b89286802eb1bc.BeginTime = System.Nullable.getValue(Bridge.cast(Bridge.unbox(Bridge.global.DotNetForHtml5.Core.TypeFromStringConverters.ConvertFromInvariantString(Bridge.global.System.TimeSpan, "00:00:00"), System.TimeSpan), System.TimeSpan));
                    ObjectAnimationUsingKeyFrames_aee92a37d81a402792b89286802eb1bc.Duration = new Bridge.global.Windows.UI.Xaml.Duration.$ctor1(new Bridge.global.System.TimeSpan(System.Int64(10000)));
                    Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard.SetTargetName(ObjectAnimationUsingKeyFrames_aee92a37d81a402792b89286802eb1bc, "content");
                    var DiscreteObjectKeyFrame_05fa398ce7e14d1ca18dece35fb1df40 = new Bridge.global.Windows.UI.Xaml.Media.Animation.DiscreteObjectKeyFrame();
                    DiscreteObjectKeyFrame_05fa398ce7e14d1ca18dece35fb1df40.KeyTime = Bridge.global.Windows.UI.Xaml.Media.Animation.KeyTime.FromTimeSpan(new Bridge.global.System.TimeSpan(System.Int64(0)));
                    var Boolean_aa341828caf742a6a8c8f69c7457e38e = true;

                    DiscreteObjectKeyFrame_05fa398ce7e14d1ca18dece35fb1df40.Value = Bridge.box(Boolean_aa341828caf742a6a8c8f69c7457e38e, System.Boolean, System.Boolean.toString);


                    ObjectAnimationUsingKeyFrames_aee92a37d81a402792b89286802eb1bc.KeyFrames.add(DiscreteObjectKeyFrame_05fa398ce7e14d1ca18dece35fb1df40);


                    Storyboard_0b190bde9e744719bcc75b25d8bc72da.Children.add(ObjectAnimationUsingKeyFrames_aee92a37d81a402792b89286802eb1bc);


                    VisualState_80fac42af14d422aa0ad230d0e03e558.Storyboard = Storyboard_0b190bde9e744719bcc75b25d8bc72da;


                    templateOwner_ControlTemplate_5dcd43438f6a4c0b8ded1af5dd32feb5.RegisterName("Busy", VisualState_c84ca299ab9640919a9b1f36527a31e2);
                    VisualState_c84ca299ab9640919a9b1f36527a31e2.Name = "Busy";
                    var Storyboard_df7790c92c2b43d8a35270fbddc7ca75 = new Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard();
                    var ObjectAnimationUsingKeyFrames_4e6282ff78bd4848851a773edb69e363 = new Bridge.global.Windows.UI.Xaml.Media.Animation.ObjectAnimationUsingKeyFrames();
                    ObjectAnimationUsingKeyFrames_4e6282ff78bd4848851a773edb69e363.BeginTime = System.Nullable.getValue(Bridge.cast(Bridge.unbox(Bridge.global.DotNetForHtml5.Core.TypeFromStringConverters.ConvertFromInvariantString(Bridge.global.System.TimeSpan, "00:00:00"), System.TimeSpan), System.TimeSpan));
                    ObjectAnimationUsingKeyFrames_4e6282ff78bd4848851a773edb69e363.Duration = new Bridge.global.Windows.UI.Xaml.Duration.$ctor1(new Bridge.global.System.TimeSpan(System.Int64(10000)));
                    Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard.SetTargetName(ObjectAnimationUsingKeyFrames_4e6282ff78bd4848851a773edb69e363, "content");
                    var DiscreteObjectKeyFrame_53e0988ff4a74d1dba797d7de37584a9 = new Bridge.global.Windows.UI.Xaml.Media.Animation.DiscreteObjectKeyFrame();
                    DiscreteObjectKeyFrame_53e0988ff4a74d1dba797d7de37584a9.KeyTime = Bridge.global.Windows.UI.Xaml.Media.Animation.KeyTime.FromTimeSpan(new Bridge.global.System.TimeSpan(System.Int64(0)));
                    var Boolean_11a322d9f00549babb16545d4683e84a = false;

                    DiscreteObjectKeyFrame_53e0988ff4a74d1dba797d7de37584a9.Value = Bridge.box(Boolean_11a322d9f00549babb16545d4683e84a, System.Boolean, System.Boolean.toString);


                    ObjectAnimationUsingKeyFrames_4e6282ff78bd4848851a773edb69e363.KeyFrames.add(DiscreteObjectKeyFrame_53e0988ff4a74d1dba797d7de37584a9);


                    Storyboard_df7790c92c2b43d8a35270fbddc7ca75.Children.add(ObjectAnimationUsingKeyFrames_4e6282ff78bd4848851a773edb69e363);


                    VisualState_c84ca299ab9640919a9b1f36527a31e2.Storyboard = Storyboard_df7790c92c2b43d8a35270fbddc7ca75;


                    System.Array.add(VisualStateGroup_b97532da7afc4e9b857fc170581185ff.States, VisualState_80fac42af14d422aa0ad230d0e03e558, Object);
                    System.Array.add(VisualStateGroup_b97532da7afc4e9b857fc170581185ff.States, VisualState_c84ca299ab9640919a9b1f36527a31e2, Object);


                    templateOwner_ControlTemplate_5dcd43438f6a4c0b8ded1af5dd32feb5.INTERNAL_GetVisualStateGroups().add(VisualStateGroup_befe61b704664f4da8dbf8c0d178aaf5);
                    templateOwner_ControlTemplate_5dcd43438f6a4c0b8ded1af5dd32feb5.INTERNAL_GetVisualStateGroups().add(VisualStateGroup_b97532da7afc4e9b857fc170581185ff);

                    templateOwner_ControlTemplate_5dcd43438f6a4c0b8ded1af5dd32feb5.RegisterName("content", ContentControl_3c046ef32b824f8286f3818729f1f1da);
                    ContentControl_3c046ef32b824f8286f3818729f1f1da.Name = "content";
                    var Binding_8452b51aad694d51b96a49f1b9ec17ea = new Bridge.global.Windows.UI.Xaml.Data.Binding.ctor();
                    Binding_8452b51aad694d51b96a49f1b9ec17ea.Path = new Bridge.global.Windows.UI.Xaml.PropertyPath.ctor("Content");
                    var RelativeSource_2aefc3ac8cd949a98e20ab264194d4e8 = new Bridge.global.Windows.UI.Xaml.Data.RelativeSource.ctor();
                    RelativeSource_2aefc3ac8cd949a98e20ab264194d4e8.Mode = Bridge.global.Windows.UI.Xaml.Data.RelativeSourceMode.TemplatedParent;

                    Binding_8452b51aad694d51b96a49f1b9ec17ea.RelativeSource = RelativeSource_2aefc3ac8cd949a98e20ab264194d4e8;


                    Binding_8452b51aad694d51b96a49f1b9ec17ea.TemplateOwner = templateInstance_e22fb51283ef45f3bf8645186d1ff5ca;

                    var Binding_4802382e828244218b42a7de32642a36 = new Bridge.global.Windows.UI.Xaml.Data.Binding.ctor();
                    Binding_4802382e828244218b42a7de32642a36.Path = new Bridge.global.Windows.UI.Xaml.PropertyPath.ctor("ContentTemplate");
                    var RelativeSource_d1690b24a17e41f3bbf9c6e2f1ee7c88 = new Bridge.global.Windows.UI.Xaml.Data.RelativeSource.ctor();
                    RelativeSource_d1690b24a17e41f3bbf9c6e2f1ee7c88.Mode = Bridge.global.Windows.UI.Xaml.Data.RelativeSourceMode.TemplatedParent;

                    Binding_4802382e828244218b42a7de32642a36.RelativeSource = RelativeSource_d1690b24a17e41f3bbf9c6e2f1ee7c88;


                    Binding_4802382e828244218b42a7de32642a36.TemplateOwner = templateInstance_e22fb51283ef45f3bf8645186d1ff5ca;

                    var Binding_6e6798158daa48a1a1d678318a360a16 = new Bridge.global.Windows.UI.Xaml.Data.Binding.ctor();
                    Binding_6e6798158daa48a1a1d678318a360a16.Path = new Bridge.global.Windows.UI.Xaml.PropertyPath.ctor("HorizontalContentAlignment");
                    var RelativeSource_34d0d09049d14dae981124c414a6287c = new Bridge.global.Windows.UI.Xaml.Data.RelativeSource.ctor();
                    RelativeSource_34d0d09049d14dae981124c414a6287c.Mode = Bridge.global.Windows.UI.Xaml.Data.RelativeSourceMode.TemplatedParent;

                    Binding_6e6798158daa48a1a1d678318a360a16.RelativeSource = RelativeSource_34d0d09049d14dae981124c414a6287c;


                    Binding_6e6798158daa48a1a1d678318a360a16.TemplateOwner = templateInstance_e22fb51283ef45f3bf8645186d1ff5ca;

                    var Binding_e1ce276d45014ce697aa448ddfa6ac1f = new Bridge.global.Windows.UI.Xaml.Data.Binding.ctor();
                    Binding_e1ce276d45014ce697aa448ddfa6ac1f.Path = new Bridge.global.Windows.UI.Xaml.PropertyPath.ctor("VerticalContentAlignment");
                    var RelativeSource_bda9479eadf34971b676bedb90f3dcfe = new Bridge.global.Windows.UI.Xaml.Data.RelativeSource.ctor();
                    RelativeSource_bda9479eadf34971b676bedb90f3dcfe.Mode = Bridge.global.Windows.UI.Xaml.Data.RelativeSourceMode.TemplatedParent;

                    Binding_e1ce276d45014ce697aa448ddfa6ac1f.RelativeSource = RelativeSource_bda9479eadf34971b676bedb90f3dcfe;


                    Binding_e1ce276d45014ce697aa448ddfa6ac1f.TemplateOwner = templateInstance_e22fb51283ef45f3bf8645186d1ff5ca;


                    templateOwner_ControlTemplate_5dcd43438f6a4c0b8ded1af5dd32feb5.RegisterName("overlay", Rectangle_f1538946a28447e4844d211e5d84bb09);
                    Rectangle_f1538946a28447e4844d211e5d84bb09.Name = "overlay";
                    var Binding_e1709a15f6d747dab3d81d4d9fe0097e = new Bridge.global.Windows.UI.Xaml.Data.Binding.ctor();
                    Binding_e1709a15f6d747dab3d81d4d9fe0097e.Path = new Bridge.global.Windows.UI.Xaml.PropertyPath.ctor("OverlayStyle");
                    var RelativeSource_bc2694fbc8ec412db0f902d14dbe502d = new Bridge.global.Windows.UI.Xaml.Data.RelativeSource.ctor();
                    RelativeSource_bc2694fbc8ec412db0f902d14dbe502d.Mode = Bridge.global.Windows.UI.Xaml.Data.RelativeSourceMode.TemplatedParent;

                    Binding_e1709a15f6d747dab3d81d4d9fe0097e.RelativeSource = RelativeSource_bc2694fbc8ec412db0f902d14dbe502d;


                    Binding_e1709a15f6d747dab3d81d4d9fe0097e.TemplateOwner = templateInstance_e22fb51283ef45f3bf8645186d1ff5ca;


                    templateOwner_ControlTemplate_5dcd43438f6a4c0b8ded1af5dd32feb5.RegisterName("busycontent", Border_cfd20111f0ec4a0cbbadd7e775cc0069);
                    Border_cfd20111f0ec4a0cbbadd7e775cc0069.Name = "busycontent";
                    var Grid_284c766a08c047318b1d2957abe6e899 = new Bridge.global.Windows.UI.Xaml.Controls.Grid();
                    Grid_284c766a08c047318b1d2957abe6e899.HorizontalAlignment = Bridge.global.Windows.UI.Xaml.HorizontalAlignment.Center;
                    Grid_284c766a08c047318b1d2957abe6e899.VerticalAlignment = Bridge.global.Windows.UI.Xaml.VerticalAlignment.Center;
                    var Border_66cb806c4d2f4419abf40baf0bc86a01 = new Bridge.global.Windows.UI.Xaml.Controls.Border();
                    Border_66cb806c4d2f4419abf40baf0bc86a01.Background = new Bridge.global.Windows.UI.Xaml.Media.SolidColorBrush.$ctor1(($t1 = new Bridge.global.Windows.UI.Color(), $t1.A = 255, $t1.R = 255, $t1.G = 255, $t1.B = 255, $t1));
                    Border_66cb806c4d2f4419abf40baf0bc86a01.BorderThickness = new Bridge.global.Windows.UI.Xaml.Thickness.$ctor1(1);
                    Border_66cb806c4d2f4419abf40baf0bc86a01.CornerRadius = new Bridge.global.Windows.UI.Xaml.CornerRadius.$ctor1(2);
                    var LinearGradientBrush_79887bc574c14bd3beacc4fa0c704346 = new Bridge.global.Windows.UI.Xaml.Media.LinearGradientBrush.ctor();
                    LinearGradientBrush_79887bc574c14bd3beacc4fa0c704346.StartPoint = new Bridge.global.Windows.Foundation.Point.$ctor1(0.5, 0);
                    LinearGradientBrush_79887bc574c14bd3beacc4fa0c704346.EndPoint = new Bridge.global.Windows.Foundation.Point.$ctor1(0.5, 1);
                    var GradientStop_f7ba0eb88ab540c89fab9a190f5b9e45 = new Bridge.global.Windows.UI.Xaml.Media.GradientStop();
                    GradientStop_f7ba0eb88ab540c89fab9a190f5b9e45.Color = ($t1 = new Bridge.global.Windows.UI.Color(), $t1.A = 255, $t1.R = 163, $t1.G = 174, $t1.B = 185, $t1);
                    GradientStop_f7ba0eb88ab540c89fab9a190f5b9e45.Offset = 0.0;

                    var GradientStop_386e7e7f19b84773bda706a384b621d9 = new Bridge.global.Windows.UI.Xaml.Media.GradientStop();
                    GradientStop_386e7e7f19b84773bda706a384b621d9.Color = ($t1 = new Bridge.global.Windows.UI.Color(), $t1.A = 255, $t1.R = 131, $t1.G = 153, $t1.B = 169, $t1);
                    GradientStop_386e7e7f19b84773bda706a384b621d9.Offset = 0.375;

                    var GradientStop_370e89cdad2346d8a0baadcd2fbe93c8 = new Bridge.global.Windows.UI.Xaml.Media.GradientStop();
                    GradientStop_370e89cdad2346d8a0baadcd2fbe93c8.Color = ($t1 = new Bridge.global.Windows.UI.Color(), $t1.A = 255, $t1.R = 113, $t1.G = 133, $t1.B = 151, $t1);
                    GradientStop_370e89cdad2346d8a0baadcd2fbe93c8.Offset = 0.375;

                    var GradientStop_4dc9ae09413b46afb097bb2a666416d5 = new Bridge.global.Windows.UI.Xaml.Media.GradientStop();
                    GradientStop_4dc9ae09413b46afb097bb2a666416d5.Color = ($t1 = new Bridge.global.Windows.UI.Color(), $t1.A = 255, $t1.R = 97, $t1.G = 117, $t1.B = 132, $t1);
                    GradientStop_4dc9ae09413b46afb097bb2a666416d5.Offset = 1.0;

                    LinearGradientBrush_79887bc574c14bd3beacc4fa0c704346.GradientStops.add(GradientStop_f7ba0eb88ab540c89fab9a190f5b9e45);
                    LinearGradientBrush_79887bc574c14bd3beacc4fa0c704346.GradientStops.add(GradientStop_386e7e7f19b84773bda706a384b621d9);
                    LinearGradientBrush_79887bc574c14bd3beacc4fa0c704346.GradientStops.add(GradientStop_370e89cdad2346d8a0baadcd2fbe93c8);
                    LinearGradientBrush_79887bc574c14bd3beacc4fa0c704346.GradientStops.add(GradientStop_4dc9ae09413b46afb097bb2a666416d5);


                    Border_66cb806c4d2f4419abf40baf0bc86a01.BorderBrush = LinearGradientBrush_79887bc574c14bd3beacc4fa0c704346;

                    var Border_84246bc83c5941ccbac2cfb0ccfffe2c = new Bridge.global.Windows.UI.Xaml.Controls.Border();
                    Border_84246bc83c5941ccbac2cfb0ccfffe2c.CornerRadius = new Bridge.global.Windows.UI.Xaml.CornerRadius.$ctor1(1.5);
                    Border_84246bc83c5941ccbac2cfb0ccfffe2c.Margin = new Bridge.global.Windows.UI.Xaml.Thickness.$ctor1(1);
                    var LinearGradientBrush_d65d5de24e6c4248b5010ff229d32b15 = new Bridge.global.Windows.UI.Xaml.Media.LinearGradientBrush.ctor();
                    LinearGradientBrush_d65d5de24e6c4248b5010ff229d32b15.StartPoint = new Bridge.global.Windows.Foundation.Point.$ctor1(0.5, 0);
                    LinearGradientBrush_d65d5de24e6c4248b5010ff229d32b15.EndPoint = new Bridge.global.Windows.Foundation.Point.$ctor1(0.5, 1);
                    var GradientStop_720e67c082bf484ba94c2a421e28b001 = new Bridge.global.Windows.UI.Xaml.Media.GradientStop();
                    GradientStop_720e67c082bf484ba94c2a421e28b001.Color = ($t1 = new Bridge.global.Windows.UI.Color(), $t1.A = 255, $t1.R = 246, $t1.G = 248, $t1.B = 249, $t1);
                    GradientStop_720e67c082bf484ba94c2a421e28b001.Offset = 0.02;

                    var GradientStop_7c0b609b440c4a389462602cac5207f3 = new Bridge.global.Windows.UI.Xaml.Media.GradientStop();
                    GradientStop_7c0b609b440c4a389462602cac5207f3.Color = ($t1 = new Bridge.global.Windows.UI.Color(), $t1.A = 255, $t1.R = 184, $t1.G = 184, $t1.B = 184, $t1);
                    GradientStop_7c0b609b440c4a389462602cac5207f3.Offset = 0.996;

                    LinearGradientBrush_d65d5de24e6c4248b5010ff229d32b15.GradientStops.add(GradientStop_720e67c082bf484ba94c2a421e28b001);
                    LinearGradientBrush_d65d5de24e6c4248b5010ff229d32b15.GradientStops.add(GradientStop_7c0b609b440c4a389462602cac5207f3);


                    Border_84246bc83c5941ccbac2cfb0ccfffe2c.Background = LinearGradientBrush_d65d5de24e6c4248b5010ff229d32b15;

                    var Grid_91a404b02cee4017bfd41413ce77237c = new Bridge.global.Windows.UI.Xaml.Controls.Grid();
                    Grid_91a404b02cee4017bfd41413ce77237c.MinWidth = 150.0;
                    var RowDefinition_f1bd3b92104e4b5ea16e1ebe1e65999b = new Bridge.global.Windows.UI.Xaml.Controls.RowDefinition();
                    RowDefinition_f1bd3b92104e4b5ea16e1ebe1e65999b.Height = new Bridge.global.Windows.UI.Xaml.GridLength.$ctor2(1.0, Bridge.global.Windows.UI.Xaml.GridUnitType.Star);

                    var RowDefinition_81243c3b78ba49d984fa209d74db9e9c = new Bridge.global.Windows.UI.Xaml.Controls.RowDefinition();
                    RowDefinition_81243c3b78ba49d984fa209d74db9e9c.Height = new Bridge.global.Windows.UI.Xaml.GridLength.$ctor2(1.0, Bridge.global.Windows.UI.Xaml.GridUnitType.Auto);

                    Grid_91a404b02cee4017bfd41413ce77237c.RowDefinitions.add(RowDefinition_f1bd3b92104e4b5ea16e1ebe1e65999b);
                    Grid_91a404b02cee4017bfd41413ce77237c.RowDefinitions.add(RowDefinition_81243c3b78ba49d984fa209d74db9e9c);

                    var ContentPresenter_ee1dda51439949ec869b58a101639faf = new Bridge.global.Windows.UI.Xaml.Controls.ContentPresenter();
                    ContentPresenter_ee1dda51439949ec869b58a101639faf.Margin = new Bridge.global.Windows.UI.Xaml.Thickness.$ctor1(8);
                    var Binding_a8d13b21f2274a9c95efd484ad3701d4 = new Bridge.global.Windows.UI.Xaml.Data.Binding.ctor();
                    Binding_a8d13b21f2274a9c95efd484ad3701d4.Path = new Bridge.global.Windows.UI.Xaml.PropertyPath.ctor("BusyContent");
                    var RelativeSource_a08b8c4d16f34ae4bd5edbaf7a6bb730 = new Bridge.global.Windows.UI.Xaml.Data.RelativeSource.ctor();
                    RelativeSource_a08b8c4d16f34ae4bd5edbaf7a6bb730.Mode = Bridge.global.Windows.UI.Xaml.Data.RelativeSourceMode.TemplatedParent;

                    Binding_a8d13b21f2274a9c95efd484ad3701d4.RelativeSource = RelativeSource_a08b8c4d16f34ae4bd5edbaf7a6bb730;


                    Binding_a8d13b21f2274a9c95efd484ad3701d4.TemplateOwner = templateInstance_e22fb51283ef45f3bf8645186d1ff5ca;

                    var Binding_3bb56b17990e413db6405c4cd6ad99bb = new Bridge.global.Windows.UI.Xaml.Data.Binding.ctor();
                    Binding_3bb56b17990e413db6405c4cd6ad99bb.Path = new Bridge.global.Windows.UI.Xaml.PropertyPath.ctor("BusyContentTemplate");
                    var RelativeSource_1ad41592b8c84f6c8b3181357112744a = new Bridge.global.Windows.UI.Xaml.Data.RelativeSource.ctor();
                    RelativeSource_1ad41592b8c84f6c8b3181357112744a.Mode = Bridge.global.Windows.UI.Xaml.Data.RelativeSourceMode.TemplatedParent;

                    Binding_3bb56b17990e413db6405c4cd6ad99bb.RelativeSource = RelativeSource_1ad41592b8c84f6c8b3181357112744a;


                    Binding_3bb56b17990e413db6405c4cd6ad99bb.TemplateOwner = templateInstance_e22fb51283ef45f3bf8645186d1ff5ca;


                    Grid_91a404b02cee4017bfd41413ce77237c.Children.add(ContentPresenter_ee1dda51439949ec869b58a101639faf);


                    Border_84246bc83c5941ccbac2cfb0ccfffe2c.Child = Grid_91a404b02cee4017bfd41413ce77237c;


                    Border_66cb806c4d2f4419abf40baf0bc86a01.Child = Border_84246bc83c5941ccbac2cfb0ccfffe2c;


                    Grid_284c766a08c047318b1d2957abe6e899.Children.add(Border_66cb806c4d2f4419abf40baf0bc86a01);


                    Border_cfd20111f0ec4a0cbbadd7e775cc0069.Child = Grid_284c766a08c047318b1d2957abe6e899;


                    Grid_60ad010f75754f95a256032db5e3c722.Children.add(ContentControl_3c046ef32b824f8286f3818729f1f1da);
                    Grid_60ad010f75754f95a256032db5e3c722.Children.add(Rectangle_f1538946a28447e4844d211e5d84bb09);
                    Grid_60ad010f75754f95a256032db5e3c722.Children.add(Border_cfd20111f0ec4a0cbbadd7e775cc0069);


                    Bridge.global.Windows.UI.Xaml.Data.BindingOperations.SetBinding(ContentControl_3c046ef32b824f8286f3818729f1f1da, Bridge.global.Windows.UI.Xaml.Controls.ContentControl.ContentProperty, Binding_8452b51aad694d51b96a49f1b9ec17ea);
                    Bridge.global.Windows.UI.Xaml.Data.BindingOperations.SetBinding(ContentControl_3c046ef32b824f8286f3818729f1f1da, Bridge.global.Windows.UI.Xaml.Controls.ContentControl.ContentTemplateProperty, Binding_4802382e828244218b42a7de32642a36);
                    Bridge.global.Windows.UI.Xaml.Data.BindingOperations.SetBinding(ContentControl_3c046ef32b824f8286f3818729f1f1da, Bridge.global.Windows.UI.Xaml.Controls.Control.HorizontalContentAlignmentProperty, Binding_6e6798158daa48a1a1d678318a360a16);
                    Bridge.global.Windows.UI.Xaml.Data.BindingOperations.SetBinding(ContentControl_3c046ef32b824f8286f3818729f1f1da, Bridge.global.Windows.UI.Xaml.Controls.Control.VerticalContentAlignmentProperty, Binding_e1ce276d45014ce697aa448ddfa6ac1f);
                    Bridge.global.Windows.UI.Xaml.Data.BindingOperations.SetBinding(Rectangle_f1538946a28447e4844d211e5d84bb09, Bridge.global.Windows.UI.Xaml.FrameworkElement.StyleProperty, Binding_e1709a15f6d747dab3d81d4d9fe0097e);
                    Bridge.global.Windows.UI.Xaml.Data.BindingOperations.SetBinding(ContentPresenter_ee1dda51439949ec869b58a101639faf, Bridge.global.Windows.UI.Xaml.Controls.ContentControl.ContentProperty, Binding_a8d13b21f2274a9c95efd484ad3701d4);
                    Bridge.global.Windows.UI.Xaml.Data.BindingOperations.SetBinding(ContentPresenter_ee1dda51439949ec869b58a101639faf, Bridge.global.Windows.UI.Xaml.Controls.ContentControl.ContentTemplateProperty, Binding_3bb56b17990e413db6405c4cd6ad99bb);

                    Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard.SetTargetProperty(ObjectAnimationUsingKeyFrames_e7eea553b9d346b2af3ee0c2847d7370, new Bridge.global.Windows.UI.Xaml.PropertyPath.$ctor1("(global::Windows.UI.Xaml.UIElement.Visibility)", "Visibility", Bridge.fn.cacheBind(this, this.accessVisualStateProperty_d675651e41704eeab99300614d3e2a64), Bridge.fn.cacheBind(this, this.setVisualStateProperty_d675651e41704eeab99300614d3e2a64), Bridge.fn.cacheBind(this, this.setAnimationVisualStateProperty_d675651e41704eeab99300614d3e2a64), Bridge.fn.cacheBind(this, this.setLocalVisualStateProperty_d675651e41704eeab99300614d3e2a64), Bridge.fn.cacheBind(this, this.getVisualStateProperty_d675651e41704eeab99300614d3e2a64)));
                    Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard.SetTarget(ObjectAnimationUsingKeyFrames_e7eea553b9d346b2af3ee0c2847d7370, Border_cfd20111f0ec4a0cbbadd7e775cc0069);


                    Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard.SetTargetProperty(ObjectAnimationUsingKeyFrames_7a7d6195a8684434b30581b42eeb7f13, new Bridge.global.Windows.UI.Xaml.PropertyPath.$ctor1("(global::Windows.UI.Xaml.UIElement.Visibility)", "Visibility", Bridge.fn.cacheBind(this, this.accessVisualStateProperty_15cacf49f20440f38fae07c39ddcc07b), Bridge.fn.cacheBind(this, this.setVisualStateProperty_15cacf49f20440f38fae07c39ddcc07b), Bridge.fn.cacheBind(this, this.setAnimationVisualStateProperty_15cacf49f20440f38fae07c39ddcc07b), Bridge.fn.cacheBind(this, this.setLocalVisualStateProperty_15cacf49f20440f38fae07c39ddcc07b), Bridge.fn.cacheBind(this, this.getVisualStateProperty_15cacf49f20440f38fae07c39ddcc07b)));
                    Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard.SetTarget(ObjectAnimationUsingKeyFrames_7a7d6195a8684434b30581b42eeb7f13, Rectangle_f1538946a28447e4844d211e5d84bb09);


                    Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard.SetTargetProperty(ObjectAnimationUsingKeyFrames_14a5e3c6738b4ab8833b60934c75beb0, new Bridge.global.Windows.UI.Xaml.PropertyPath.$ctor1("(global::Windows.UI.Xaml.UIElement.Visibility)", "Visibility", Bridge.fn.cacheBind(this, this.accessVisualStateProperty_435543e9a20846fa997207ac962f5c0a), Bridge.fn.cacheBind(this, this.setVisualStateProperty_435543e9a20846fa997207ac962f5c0a), Bridge.fn.cacheBind(this, this.setAnimationVisualStateProperty_435543e9a20846fa997207ac962f5c0a), Bridge.fn.cacheBind(this, this.setLocalVisualStateProperty_435543e9a20846fa997207ac962f5c0a), Bridge.fn.cacheBind(this, this.getVisualStateProperty_435543e9a20846fa997207ac962f5c0a)));
                    Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard.SetTarget(ObjectAnimationUsingKeyFrames_14a5e3c6738b4ab8833b60934c75beb0, Border_cfd20111f0ec4a0cbbadd7e775cc0069);


                    Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard.SetTargetProperty(ObjectAnimationUsingKeyFrames_a6a96913d284479694d8bfbfc76f293e, new Bridge.global.Windows.UI.Xaml.PropertyPath.$ctor1("(global::Windows.UI.Xaml.UIElement.Visibility)", "Visibility", Bridge.fn.cacheBind(this, this.accessVisualStateProperty_f36e67853b584ba99189ac7007c30807), Bridge.fn.cacheBind(this, this.setVisualStateProperty_f36e67853b584ba99189ac7007c30807), Bridge.fn.cacheBind(this, this.setAnimationVisualStateProperty_f36e67853b584ba99189ac7007c30807), Bridge.fn.cacheBind(this, this.setLocalVisualStateProperty_f36e67853b584ba99189ac7007c30807), Bridge.fn.cacheBind(this, this.getVisualStateProperty_f36e67853b584ba99189ac7007c30807)));
                    Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard.SetTarget(ObjectAnimationUsingKeyFrames_a6a96913d284479694d8bfbfc76f293e, Rectangle_f1538946a28447e4844d211e5d84bb09);


                    Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard.SetTargetProperty(ObjectAnimationUsingKeyFrames_aee92a37d81a402792b89286802eb1bc, new Bridge.global.Windows.UI.Xaml.PropertyPath.$ctor1("(global::Windows.UI.Xaml.Controls.Control.IsEnabled)", "IsEnabled", Bridge.fn.cacheBind(this, this.accessVisualStateProperty_334c13e0716840c6b70584a69dcc8564), Bridge.fn.cacheBind(this, this.setVisualStateProperty_334c13e0716840c6b70584a69dcc8564), Bridge.fn.cacheBind(this, this.setAnimationVisualStateProperty_334c13e0716840c6b70584a69dcc8564), Bridge.fn.cacheBind(this, this.setLocalVisualStateProperty_334c13e0716840c6b70584a69dcc8564), Bridge.fn.cacheBind(this, this.getVisualStateProperty_334c13e0716840c6b70584a69dcc8564)));
                    Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard.SetTarget(ObjectAnimationUsingKeyFrames_aee92a37d81a402792b89286802eb1bc, ContentControl_3c046ef32b824f8286f3818729f1f1da);


                    Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard.SetTargetProperty(ObjectAnimationUsingKeyFrames_4e6282ff78bd4848851a773edb69e363, new Bridge.global.Windows.UI.Xaml.PropertyPath.$ctor1("(global::Windows.UI.Xaml.Controls.Control.IsEnabled)", "IsEnabled", Bridge.fn.cacheBind(this, this.accessVisualStateProperty_6f112160e7264bba8347b16c0027f8ad), Bridge.fn.cacheBind(this, this.setVisualStateProperty_6f112160e7264bba8347b16c0027f8ad), Bridge.fn.cacheBind(this, this.setAnimationVisualStateProperty_6f112160e7264bba8347b16c0027f8ad), Bridge.fn.cacheBind(this, this.setLocalVisualStateProperty_6f112160e7264bba8347b16c0027f8ad), Bridge.fn.cacheBind(this, this.getVisualStateProperty_6f112160e7264bba8347b16c0027f8ad)));
                    Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard.SetTarget(ObjectAnimationUsingKeyFrames_4e6282ff78bd4848851a773edb69e363, ContentControl_3c046ef32b824f8286f3818729f1f1da);

                    templateInstance_e22fb51283ef45f3bf8645186d1ff5ca.TemplateContent = Grid_60ad010f75754f95a256032db5e3c722;
                    return templateInstance_e22fb51283ef45f3bf8645186d1ff5ca;
                }));

                Setter_018b41aa20ee4908a7987d7a2d7c7b4e.Value = ControlTemplate_5dcd43438f6a4c0b8ded1af5dd32feb5;


                Style_c785d4ad336b4a6e8a1a94b703b18591.Setters.add(Setter_42621b6c23cf4c769d42ae5c45e7ab12);
                Style_c785d4ad336b4a6e8a1a94b703b18591.Setters.add(Setter_0e2e3f369d074584ba01ddc5b578fa10);
                Style_c785d4ad336b4a6e8a1a94b703b18591.Setters.add(Setter_c3b542abae894b7eaf5889467dec072b);
                Style_c785d4ad336b4a6e8a1a94b703b18591.Setters.add(Setter_fdb52f4419b5430ca03f8673d5c3fc02);
                Style_c785d4ad336b4a6e8a1a94b703b18591.Setters.add(Setter_e9df22b7b52f4a0f999863e5d6708857);
                Style_c785d4ad336b4a6e8a1a94b703b18591.Setters.add(Setter_5d89abb97ed448aca49de902a2e279e4);
                Style_c785d4ad336b4a6e8a1a94b703b18591.Setters.add(Setter_4cf2a2102851473a9a61ee5591252721);
                Style_c785d4ad336b4a6e8a1a94b703b18591.Setters.add(Setter_ebd3297f0f184d82b0d2c5931ac63411);
                Style_c785d4ad336b4a6e8a1a94b703b18591.Setters.add(Setter_018b41aa20ee4908a7987d7a2d7c7b4e);


                this.setItem(Bridge.global.Windows.UI.Xaml.Controls.BusyIndicator, Style_c785d4ad336b4a6e8a1a94b703b18591);






            },
            accessVisualStateProperty_d675651e41704eeab99300614d3e2a64: function (rootTargetObjectInstance) {
                return new (Bridge.GeneratorEnumerable$1(System.Tuple$3(Windows.UI.Xaml.DependencyObject,Windows.UI.Xaml.DependencyProperty,System.Nullable$1(System.Int32))))(Bridge.fn.bind(this, function (rootTargetObjectInstance) {
                    var $step = 0,
                        $jumpFromFinally,
                        $returnValue,
                        $async_e;

                    var $enumerator = new (Bridge.GeneratorEnumerator$1(System.Tuple$3(Windows.UI.Xaml.DependencyObject,Windows.UI.Xaml.DependencyProperty,System.Nullable$1(System.Int32))))(Bridge.fn.bind(this, function () {
                        try {
                            for (;;) {
                                switch ($step) {
                                    case 0: {
                                        return false;
                                    }
                                    default: {
                                        return false;
                                    }
                                }
                            }
                        } catch($async_e1) {
                            $async_e = System.Exception.create($async_e1);
                            throw $async_e;
                        }
                    }));
                    return $enumerator;
                }, arguments));
            },
            setVisualStateProperty_d675651e41704eeab99300614d3e2a64: function (finalTargetInstance, value) {

                (finalTargetInstance).SetVisualStateValue(Bridge.global.Windows.UI.Xaml.UIElement.VisibilityProperty, value);
            },
            setAnimationVisualStateProperty_d675651e41704eeab99300614d3e2a64: function (finalTargetInstance, value) {

                (finalTargetInstance).SetAnimationValue(Bridge.global.Windows.UI.Xaml.UIElement.VisibilityProperty, value);
            },
            setLocalVisualStateProperty_d675651e41704eeab99300614d3e2a64: function (finalTargetInstance, value) {

                (finalTargetInstance).SetLocalValue(Bridge.global.Windows.UI.Xaml.UIElement.VisibilityProperty, value);
            },
            getVisualStateProperty_d675651e41704eeab99300614d3e2a64: function (finalTargetInstance) {

                return finalTargetInstance.GetVisualStateValue(Bridge.global.Windows.UI.Xaml.UIElement.VisibilityProperty);
            },
            accessVisualStateProperty_15cacf49f20440f38fae07c39ddcc07b: function (rootTargetObjectInstance) {
                return new (Bridge.GeneratorEnumerable$1(System.Tuple$3(Windows.UI.Xaml.DependencyObject,Windows.UI.Xaml.DependencyProperty,System.Nullable$1(System.Int32))))(Bridge.fn.bind(this, function (rootTargetObjectInstance) {
                    var $step = 0,
                        $jumpFromFinally,
                        $returnValue,
                        $async_e;

                    var $enumerator = new (Bridge.GeneratorEnumerator$1(System.Tuple$3(Windows.UI.Xaml.DependencyObject,Windows.UI.Xaml.DependencyProperty,System.Nullable$1(System.Int32))))(Bridge.fn.bind(this, function () {
                        try {
                            for (;;) {
                                switch ($step) {
                                    case 0: {
                                        return false;
                                    }
                                    default: {
                                        return false;
                                    }
                                }
                            }
                        } catch($async_e1) {
                            $async_e = System.Exception.create($async_e1);
                            throw $async_e;
                        }
                    }));
                    return $enumerator;
                }, arguments));
            },
            setVisualStateProperty_15cacf49f20440f38fae07c39ddcc07b: function (finalTargetInstance, value) {

                (finalTargetInstance).SetVisualStateValue(Bridge.global.Windows.UI.Xaml.UIElement.VisibilityProperty, value);
            },
            setAnimationVisualStateProperty_15cacf49f20440f38fae07c39ddcc07b: function (finalTargetInstance, value) {

                (finalTargetInstance).SetAnimationValue(Bridge.global.Windows.UI.Xaml.UIElement.VisibilityProperty, value);
            },
            setLocalVisualStateProperty_15cacf49f20440f38fae07c39ddcc07b: function (finalTargetInstance, value) {

                (finalTargetInstance).SetLocalValue(Bridge.global.Windows.UI.Xaml.UIElement.VisibilityProperty, value);
            },
            getVisualStateProperty_15cacf49f20440f38fae07c39ddcc07b: function (finalTargetInstance) {

                return finalTargetInstance.GetVisualStateValue(Bridge.global.Windows.UI.Xaml.UIElement.VisibilityProperty);
            },
            accessVisualStateProperty_435543e9a20846fa997207ac962f5c0a: function (rootTargetObjectInstance) {
                return new (Bridge.GeneratorEnumerable$1(System.Tuple$3(Windows.UI.Xaml.DependencyObject,Windows.UI.Xaml.DependencyProperty,System.Nullable$1(System.Int32))))(Bridge.fn.bind(this, function (rootTargetObjectInstance) {
                    var $step = 0,
                        $jumpFromFinally,
                        $returnValue,
                        $async_e;

                    var $enumerator = new (Bridge.GeneratorEnumerator$1(System.Tuple$3(Windows.UI.Xaml.DependencyObject,Windows.UI.Xaml.DependencyProperty,System.Nullable$1(System.Int32))))(Bridge.fn.bind(this, function () {
                        try {
                            for (;;) {
                                switch ($step) {
                                    case 0: {
                                        return false;
                                    }
                                    default: {
                                        return false;
                                    }
                                }
                            }
                        } catch($async_e1) {
                            $async_e = System.Exception.create($async_e1);
                            throw $async_e;
                        }
                    }));
                    return $enumerator;
                }, arguments));
            },
            setVisualStateProperty_435543e9a20846fa997207ac962f5c0a: function (finalTargetInstance, value) {

                (finalTargetInstance).SetVisualStateValue(Bridge.global.Windows.UI.Xaml.UIElement.VisibilityProperty, value);
            },
            setAnimationVisualStateProperty_435543e9a20846fa997207ac962f5c0a: function (finalTargetInstance, value) {

                (finalTargetInstance).SetAnimationValue(Bridge.global.Windows.UI.Xaml.UIElement.VisibilityProperty, value);
            },
            setLocalVisualStateProperty_435543e9a20846fa997207ac962f5c0a: function (finalTargetInstance, value) {

                (finalTargetInstance).SetLocalValue(Bridge.global.Windows.UI.Xaml.UIElement.VisibilityProperty, value);
            },
            getVisualStateProperty_435543e9a20846fa997207ac962f5c0a: function (finalTargetInstance) {

                return finalTargetInstance.GetVisualStateValue(Bridge.global.Windows.UI.Xaml.UIElement.VisibilityProperty);
            },
            accessVisualStateProperty_f36e67853b584ba99189ac7007c30807: function (rootTargetObjectInstance) {
                return new (Bridge.GeneratorEnumerable$1(System.Tuple$3(Windows.UI.Xaml.DependencyObject,Windows.UI.Xaml.DependencyProperty,System.Nullable$1(System.Int32))))(Bridge.fn.bind(this, function (rootTargetObjectInstance) {
                    var $step = 0,
                        $jumpFromFinally,
                        $returnValue,
                        $async_e;

                    var $enumerator = new (Bridge.GeneratorEnumerator$1(System.Tuple$3(Windows.UI.Xaml.DependencyObject,Windows.UI.Xaml.DependencyProperty,System.Nullable$1(System.Int32))))(Bridge.fn.bind(this, function () {
                        try {
                            for (;;) {
                                switch ($step) {
                                    case 0: {
                                        return false;
                                    }
                                    default: {
                                        return false;
                                    }
                                }
                            }
                        } catch($async_e1) {
                            $async_e = System.Exception.create($async_e1);
                            throw $async_e;
                        }
                    }));
                    return $enumerator;
                }, arguments));
            },
            setVisualStateProperty_f36e67853b584ba99189ac7007c30807: function (finalTargetInstance, value) {

                (finalTargetInstance).SetVisualStateValue(Bridge.global.Windows.UI.Xaml.UIElement.VisibilityProperty, value);
            },
            setAnimationVisualStateProperty_f36e67853b584ba99189ac7007c30807: function (finalTargetInstance, value) {

                (finalTargetInstance).SetAnimationValue(Bridge.global.Windows.UI.Xaml.UIElement.VisibilityProperty, value);
            },
            setLocalVisualStateProperty_f36e67853b584ba99189ac7007c30807: function (finalTargetInstance, value) {

                (finalTargetInstance).SetLocalValue(Bridge.global.Windows.UI.Xaml.UIElement.VisibilityProperty, value);
            },
            getVisualStateProperty_f36e67853b584ba99189ac7007c30807: function (finalTargetInstance) {

                return finalTargetInstance.GetVisualStateValue(Bridge.global.Windows.UI.Xaml.UIElement.VisibilityProperty);
            },
            accessVisualStateProperty_334c13e0716840c6b70584a69dcc8564: function (rootTargetObjectInstance) {
                return new (Bridge.GeneratorEnumerable$1(System.Tuple$3(Windows.UI.Xaml.DependencyObject,Windows.UI.Xaml.DependencyProperty,System.Nullable$1(System.Int32))))(Bridge.fn.bind(this, function (rootTargetObjectInstance) {
                    var $step = 0,
                        $jumpFromFinally,
                        $returnValue,
                        $async_e;

                    var $enumerator = new (Bridge.GeneratorEnumerator$1(System.Tuple$3(Windows.UI.Xaml.DependencyObject,Windows.UI.Xaml.DependencyProperty,System.Nullable$1(System.Int32))))(Bridge.fn.bind(this, function () {
                        try {
                            for (;;) {
                                switch ($step) {
                                    case 0: {
                                        return false;
                                    }
                                    default: {
                                        return false;
                                    }
                                }
                            }
                        } catch($async_e1) {
                            $async_e = System.Exception.create($async_e1);
                            throw $async_e;
                        }
                    }));
                    return $enumerator;
                }, arguments));
            },
            setVisualStateProperty_334c13e0716840c6b70584a69dcc8564: function (finalTargetInstance, value) {

                (finalTargetInstance).SetVisualStateValue(Bridge.global.Windows.UI.Xaml.FrameworkElement.IsEnabledProperty, value);
            },
            setAnimationVisualStateProperty_334c13e0716840c6b70584a69dcc8564: function (finalTargetInstance, value) {

                (finalTargetInstance).SetAnimationValue(Bridge.global.Windows.UI.Xaml.FrameworkElement.IsEnabledProperty, value);
            },
            setLocalVisualStateProperty_334c13e0716840c6b70584a69dcc8564: function (finalTargetInstance, value) {

                (finalTargetInstance).SetLocalValue(Bridge.global.Windows.UI.Xaml.FrameworkElement.IsEnabledProperty, value);
            },
            getVisualStateProperty_334c13e0716840c6b70584a69dcc8564: function (finalTargetInstance) {

                return finalTargetInstance.GetVisualStateValue(Bridge.global.Windows.UI.Xaml.FrameworkElement.IsEnabledProperty);
            },
            accessVisualStateProperty_6f112160e7264bba8347b16c0027f8ad: function (rootTargetObjectInstance) {
                return new (Bridge.GeneratorEnumerable$1(System.Tuple$3(Windows.UI.Xaml.DependencyObject,Windows.UI.Xaml.DependencyProperty,System.Nullable$1(System.Int32))))(Bridge.fn.bind(this, function (rootTargetObjectInstance) {
                    var $step = 0,
                        $jumpFromFinally,
                        $returnValue,
                        $async_e;

                    var $enumerator = new (Bridge.GeneratorEnumerator$1(System.Tuple$3(Windows.UI.Xaml.DependencyObject,Windows.UI.Xaml.DependencyProperty,System.Nullable$1(System.Int32))))(Bridge.fn.bind(this, function () {
                        try {
                            for (;;) {
                                switch ($step) {
                                    case 0: {
                                        return false;
                                    }
                                    default: {
                                        return false;
                                    }
                                }
                            }
                        } catch($async_e1) {
                            $async_e = System.Exception.create($async_e1);
                            throw $async_e;
                        }
                    }));
                    return $enumerator;
                }, arguments));
            },
            setVisualStateProperty_6f112160e7264bba8347b16c0027f8ad: function (finalTargetInstance, value) {

                (finalTargetInstance).SetVisualStateValue(Bridge.global.Windows.UI.Xaml.FrameworkElement.IsEnabledProperty, value);
            },
            setAnimationVisualStateProperty_6f112160e7264bba8347b16c0027f8ad: function (finalTargetInstance, value) {

                (finalTargetInstance).SetAnimationValue(Bridge.global.Windows.UI.Xaml.FrameworkElement.IsEnabledProperty, value);
            },
            setLocalVisualStateProperty_6f112160e7264bba8347b16c0027f8ad: function (finalTargetInstance, value) {

                (finalTargetInstance).SetLocalValue(Bridge.global.Windows.UI.Xaml.FrameworkElement.IsEnabledProperty, value);
            },
            getVisualStateProperty_6f112160e7264bba8347b16c0027f8ad: function (finalTargetInstance) {

                return finalTargetInstance.GetVisualStateValue(Bridge.global.Windows.UI.Xaml.FrameworkElement.IsEnabledProperty);
            }
        }
    });

    Bridge.define("___CSHTML5__Component__Defaultstyles__INTERNAL_Defaultbuttonstyle__Xaml", {
        inherits: [Bridge.global.Windows.UI.Xaml.ResourceDictionary],
        fields: {
            _contentLoaded: false
        },
        ctors: {
            ctor: function () {
                this.$initialize();
                Bridge.global.Windows.UI.Xaml.ResourceDictionary.ctor.call(this);
                this.InitializeComponent();

                if (Bridge.is(this, Windows.UI.Xaml.UIElement)) {
                    Bridge.cast(this, Windows.UI.Xaml.UIElement).XamlSourcePath = "CSHTML5\\DefaultStyles\\INTERNAL_DefaultButtonStyle.xaml";
                }
            }
        },
        methods: {
            InitializeComponent: function () {
                var $t;
                if (this._contentLoaded) {
                    return;
                }
                this._contentLoaded = true;


                if (Bridge.is(this, Windows.UI.Xaml.UIElement)) {
                    Bridge.cast(this, Windows.UI.Xaml.UIElement).XamlSourcePath = "CSHTML5\\DefaultStyles\\INTERNAL_DefaultButtonStyle.xaml";
                }



                var Style_e80aef6cbbfe4fd59ae809d5756634c8 = new Bridge.global.Windows.UI.Xaml.Style.ctor();
                Style_e80aef6cbbfe4fd59ae809d5756634c8.TargetType = Bridge.global.Windows.UI.Xaml.Controls.Button;
                var Setter_2ccc35a3c40940adbf35838d4ab1a89c = new Bridge.global.Windows.UI.Xaml.Setter.ctor();
                Setter_2ccc35a3c40940adbf35838d4ab1a89c.Property = Bridge.global.Windows.UI.Xaml.Controls.Control.BackgroundProperty;
                Setter_2ccc35a3c40940adbf35838d4ab1a89c.Value = new Bridge.global.Windows.UI.Xaml.Media.SolidColorBrush.$ctor1(($t = new Bridge.global.Windows.UI.Color(), $t.A = 255, $t.R = 226, $t.G = 226, $t.B = 226, $t));

                var Setter_7f77824685d54080b034aa136dd02dcb = new Bridge.global.Windows.UI.Xaml.Setter.ctor();
                Setter_7f77824685d54080b034aa136dd02dcb.Property = Bridge.global.Windows.UI.Xaml.Controls.Control.ForegroundProperty;
                Setter_7f77824685d54080b034aa136dd02dcb.Value = new Bridge.global.Windows.UI.Xaml.Media.SolidColorBrush.$ctor1(($t = new Bridge.global.Windows.UI.Color(), $t.A = 255, $t.R = 0, $t.G = 0, $t.B = 0, $t));

                var Setter_7585f7ad790e4cd5872b2455dd31e89a = new Bridge.global.Windows.UI.Xaml.Setter.ctor();
                Setter_7585f7ad790e4cd5872b2455dd31e89a.Property = Bridge.global.Windows.UI.Xaml.Controls.Control.BorderThicknessProperty;
                Setter_7585f7ad790e4cd5872b2455dd31e89a.Value = new Bridge.global.Windows.UI.Xaml.Thickness.$ctor1(0).$clone();

                var Setter_559a5db552204b83910071d4b1ec6cfe = new Bridge.global.Windows.UI.Xaml.Setter.ctor();
                Setter_559a5db552204b83910071d4b1ec6cfe.Property = Bridge.global.Windows.UI.Xaml.Controls.Control.PaddingProperty;
                Setter_559a5db552204b83910071d4b1ec6cfe.Value = new Bridge.global.Windows.UI.Xaml.Thickness.$ctor1(3).$clone();

                var Setter_2ab92eaf7d344a4c9e7e7817f5348be2 = new Bridge.global.Windows.UI.Xaml.Setter.ctor();
                Setter_2ab92eaf7d344a4c9e7e7817f5348be2.Property = Bridge.global.Windows.UI.Xaml.FrameworkElement.CursorProperty;
                Setter_2ab92eaf7d344a4c9e7e7817f5348be2.Value = Bridge.global.System.Windows.Input.Cursors.Hand;

                var Setter_b558d56538234a7d98cc902f4a48835d = new Bridge.global.Windows.UI.Xaml.Setter.ctor();
                Setter_b558d56538234a7d98cc902f4a48835d.Property = Bridge.global.Windows.UI.Xaml.Controls.Control.HorizontalContentAlignmentProperty;
                Setter_b558d56538234a7d98cc902f4a48835d.Value = Bridge.box(Bridge.global.Windows.UI.Xaml.HorizontalAlignment.Center, Windows.UI.Xaml.HorizontalAlignment, System.Enum.toStringFn(Windows.UI.Xaml.HorizontalAlignment));

                var Setter_b04f134c5580444a98029099b35ebe0f = new Bridge.global.Windows.UI.Xaml.Setter.ctor();
                Setter_b04f134c5580444a98029099b35ebe0f.Property = Bridge.global.Windows.UI.Xaml.Controls.Control.VerticalContentAlignmentProperty;
                Setter_b04f134c5580444a98029099b35ebe0f.Value = Bridge.box(Bridge.global.Windows.UI.Xaml.VerticalAlignment.Center, Windows.UI.Xaml.VerticalAlignment, System.Enum.toStringFn(Windows.UI.Xaml.VerticalAlignment));

                var Setter_ab26d55926394ee68b354e7426fbf1fc = new Bridge.global.Windows.UI.Xaml.Setter.ctor();
                Setter_ab26d55926394ee68b354e7426fbf1fc.Property = Bridge.global.Windows.UI.Xaml.Controls.Control.TemplateProperty;
                var ControlTemplate_7d2711f0622c49cc84869d61a5953773 = new Bridge.global.Windows.UI.Xaml.Controls.ControlTemplate();
                ControlTemplate_7d2711f0622c49cc84869d61a5953773.TargetType = Bridge.global.Windows.UI.Xaml.Controls.Button;
                ControlTemplate_7d2711f0622c49cc84869d61a5953773.SetMethodToInstantiateFrameworkTemplate(Bridge.fn.bind(this, function (templateOwner_ControlTemplate_7d2711f0622c49cc84869d61a5953773) {
                    var templateInstance_fd4981d633404162b8c64b19044a50f4 = new Bridge.global.Windows.UI.Xaml.TemplateInstance();
                    templateInstance_fd4981d633404162b8c64b19044a50f4.TemplateOwner = templateOwner_ControlTemplate_7d2711f0622c49cc84869d61a5953773;
                    var VisualState_010de1e691474fb2914eec3751e37dab = new Bridge.global.Windows.UI.Xaml.VisualState();
                    var VisualState_a8cd8403a19a45b3b84018a1a8e9c26c = new Bridge.global.Windows.UI.Xaml.VisualState();
                    var VisualState_f0c88bc5f86a4f0482f1c24dd08d228d = new Bridge.global.Windows.UI.Xaml.VisualState();
                    var VisualState_e8d8e095a8c845378e907cf66e27655f = new Bridge.global.Windows.UI.Xaml.VisualState();
                    var VisualStateGroup_db30482cf0ef4a27a0342d21beecb748 = new Bridge.global.Windows.UI.Xaml.VisualStateGroup();
                    var ContentPresenter_ec38531e4fdd40f897dd558287d87ea5 = new Bridge.global.Windows.UI.Xaml.Controls.ContentPresenter();
                    var Border_953ebd6aacbd4a06bfb4e042e79b9243 = new Bridge.global.Windows.UI.Xaml.Controls.Border();
                    var Border_8a600a143efe448e926edfdf50bac42d = new Bridge.global.Windows.UI.Xaml.Controls.Border();
                    templateOwner_ControlTemplate_7d2711f0622c49cc84869d61a5953773.RegisterName("OuterBorder", Border_8a600a143efe448e926edfdf50bac42d);
                    Border_8a600a143efe448e926edfdf50bac42d.Name = "OuterBorder";
                    templateOwner_ControlTemplate_7d2711f0622c49cc84869d61a5953773.RegisterName("CommonStates", VisualStateGroup_db30482cf0ef4a27a0342d21beecb748);
                    VisualStateGroup_db30482cf0ef4a27a0342d21beecb748.Name = "CommonStates";
                    templateOwner_ControlTemplate_7d2711f0622c49cc84869d61a5953773.RegisterName("Normal", VisualState_010de1e691474fb2914eec3751e37dab);
                    VisualState_010de1e691474fb2914eec3751e37dab.Name = "Normal";

                    templateOwner_ControlTemplate_7d2711f0622c49cc84869d61a5953773.RegisterName("PointerOver", VisualState_a8cd8403a19a45b3b84018a1a8e9c26c);
                    VisualState_a8cd8403a19a45b3b84018a1a8e9c26c.Name = "PointerOver";
                    var Storyboard_0b57915709a84b1bb7b31f807a801882 = new Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard();
                    var ObjectAnimationUsingKeyFrames_617bbce67f9844ecae92ff34cdd56cba = new Bridge.global.Windows.UI.Xaml.Media.Animation.ObjectAnimationUsingKeyFrames();
                    Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard.SetTargetName(ObjectAnimationUsingKeyFrames_617bbce67f9844ecae92ff34cdd56cba, "InnerBorder");
                    var DiscreteObjectKeyFrame_f68d6da630a94851812b7a6b5c7f5e69 = new Bridge.global.Windows.UI.Xaml.Media.Animation.DiscreteObjectKeyFrame();
                    DiscreteObjectKeyFrame_f68d6da630a94851812b7a6b5c7f5e69.KeyTime = Bridge.global.Windows.UI.Xaml.Media.Animation.KeyTime.FromTimeSpan(new Bridge.global.System.TimeSpan(System.Int64(0)));
                    DiscreteObjectKeyFrame_f68d6da630a94851812b7a6b5c7f5e69.Value = "#11000000";

                    ObjectAnimationUsingKeyFrames_617bbce67f9844ecae92ff34cdd56cba.KeyFrames.add(DiscreteObjectKeyFrame_f68d6da630a94851812b7a6b5c7f5e69);


                    Storyboard_0b57915709a84b1bb7b31f807a801882.Children.add(ObjectAnimationUsingKeyFrames_617bbce67f9844ecae92ff34cdd56cba);


                    VisualState_a8cd8403a19a45b3b84018a1a8e9c26c.Storyboard = Storyboard_0b57915709a84b1bb7b31f807a801882;


                    templateOwner_ControlTemplate_7d2711f0622c49cc84869d61a5953773.RegisterName("Pressed", VisualState_f0c88bc5f86a4f0482f1c24dd08d228d);
                    VisualState_f0c88bc5f86a4f0482f1c24dd08d228d.Name = "Pressed";
                    var Storyboard_1e6a9140343b44c5b98bebf62aa07b03 = new Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard();
                    var ObjectAnimationUsingKeyFrames_44ef8479ad214c1c944cbf82800b05a3 = new Bridge.global.Windows.UI.Xaml.Media.Animation.ObjectAnimationUsingKeyFrames();
                    Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard.SetTargetName(ObjectAnimationUsingKeyFrames_44ef8479ad214c1c944cbf82800b05a3, "InnerBorder");
                    var DiscreteObjectKeyFrame_092f676d1a564352a500570ef1566738 = new Bridge.global.Windows.UI.Xaml.Media.Animation.DiscreteObjectKeyFrame();
                    DiscreteObjectKeyFrame_092f676d1a564352a500570ef1566738.KeyTime = Bridge.global.Windows.UI.Xaml.Media.Animation.KeyTime.FromTimeSpan(new Bridge.global.System.TimeSpan(System.Int64(0)));
                    DiscreteObjectKeyFrame_092f676d1a564352a500570ef1566738.Value = "#22000000";

                    ObjectAnimationUsingKeyFrames_44ef8479ad214c1c944cbf82800b05a3.KeyFrames.add(DiscreteObjectKeyFrame_092f676d1a564352a500570ef1566738);


                    Storyboard_1e6a9140343b44c5b98bebf62aa07b03.Children.add(ObjectAnimationUsingKeyFrames_44ef8479ad214c1c944cbf82800b05a3);


                    VisualState_f0c88bc5f86a4f0482f1c24dd08d228d.Storyboard = Storyboard_1e6a9140343b44c5b98bebf62aa07b03;


                    templateOwner_ControlTemplate_7d2711f0622c49cc84869d61a5953773.RegisterName("Disabled", VisualState_e8d8e095a8c845378e907cf66e27655f);
                    VisualState_e8d8e095a8c845378e907cf66e27655f.Name = "Disabled";
                    var Storyboard_a12c87b98ae0439c8f506b3e073554a4 = new Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard();
                    var ObjectAnimationUsingKeyFrames_db8299a8672c4ce2831ff169752f94f8 = new Bridge.global.Windows.UI.Xaml.Media.Animation.ObjectAnimationUsingKeyFrames();
                    Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard.SetTargetName(ObjectAnimationUsingKeyFrames_db8299a8672c4ce2831ff169752f94f8, "InnerBorder");
                    var DiscreteObjectKeyFrame_4ce6f71b6f644ec5ae9f805072a71344 = new Bridge.global.Windows.UI.Xaml.Media.Animation.DiscreteObjectKeyFrame();
                    DiscreteObjectKeyFrame_4ce6f71b6f644ec5ae9f805072a71344.KeyTime = Bridge.global.Windows.UI.Xaml.Media.Animation.KeyTime.FromTimeSpan(new Bridge.global.System.TimeSpan(System.Int64(0)));
                    DiscreteObjectKeyFrame_4ce6f71b6f644ec5ae9f805072a71344.Value = "#33FFFFFF";

                    ObjectAnimationUsingKeyFrames_db8299a8672c4ce2831ff169752f94f8.KeyFrames.add(DiscreteObjectKeyFrame_4ce6f71b6f644ec5ae9f805072a71344);


                    Storyboard_a12c87b98ae0439c8f506b3e073554a4.Children.add(ObjectAnimationUsingKeyFrames_db8299a8672c4ce2831ff169752f94f8);


                    VisualState_e8d8e095a8c845378e907cf66e27655f.Storyboard = Storyboard_a12c87b98ae0439c8f506b3e073554a4;


                    System.Array.add(VisualStateGroup_db30482cf0ef4a27a0342d21beecb748.States, VisualState_010de1e691474fb2914eec3751e37dab, Object);
                    System.Array.add(VisualStateGroup_db30482cf0ef4a27a0342d21beecb748.States, VisualState_a8cd8403a19a45b3b84018a1a8e9c26c, Object);
                    System.Array.add(VisualStateGroup_db30482cf0ef4a27a0342d21beecb748.States, VisualState_f0c88bc5f86a4f0482f1c24dd08d228d, Object);
                    System.Array.add(VisualStateGroup_db30482cf0ef4a27a0342d21beecb748.States, VisualState_e8d8e095a8c845378e907cf66e27655f, Object);


                    templateOwner_ControlTemplate_7d2711f0622c49cc84869d61a5953773.INTERNAL_GetVisualStateGroups().add(VisualStateGroup_db30482cf0ef4a27a0342d21beecb748);

                    templateOwner_ControlTemplate_7d2711f0622c49cc84869d61a5953773.RegisterName("InnerBorder", Border_953ebd6aacbd4a06bfb4e042e79b9243);
                    Border_953ebd6aacbd4a06bfb4e042e79b9243.Name = "InnerBorder";
                    templateOwner_ControlTemplate_7d2711f0622c49cc84869d61a5953773.RegisterName("ContentPresenter", ContentPresenter_ec38531e4fdd40f897dd558287d87ea5);
                    ContentPresenter_ec38531e4fdd40f897dd558287d87ea5.Name = "ContentPresenter";
                    var Binding_8321a3bd6a1e46f9864eb1785c3d1dd5 = new Bridge.global.Windows.UI.Xaml.Data.Binding.ctor();
                    Binding_8321a3bd6a1e46f9864eb1785c3d1dd5.Path = new Bridge.global.Windows.UI.Xaml.PropertyPath.ctor("ContentTemplate");
                    var RelativeSource_f85f7855c4ab469981ef8ed92b1acf34 = new Bridge.global.Windows.UI.Xaml.Data.RelativeSource.ctor();
                    RelativeSource_f85f7855c4ab469981ef8ed92b1acf34.Mode = Bridge.global.Windows.UI.Xaml.Data.RelativeSourceMode.TemplatedParent;

                    Binding_8321a3bd6a1e46f9864eb1785c3d1dd5.RelativeSource = RelativeSource_f85f7855c4ab469981ef8ed92b1acf34;


                    Binding_8321a3bd6a1e46f9864eb1785c3d1dd5.TemplateOwner = templateInstance_fd4981d633404162b8c64b19044a50f4;

                    var Binding_a38f5c6d36a041d7a1aa1ebf497a69bf = new Bridge.global.Windows.UI.Xaml.Data.Binding.ctor();
                    Binding_a38f5c6d36a041d7a1aa1ebf497a69bf.Path = new Bridge.global.Windows.UI.Xaml.PropertyPath.ctor("Content");
                    var RelativeSource_96ccfc9d72824808bbc7d331f72be190 = new Bridge.global.Windows.UI.Xaml.Data.RelativeSource.ctor();
                    RelativeSource_96ccfc9d72824808bbc7d331f72be190.Mode = Bridge.global.Windows.UI.Xaml.Data.RelativeSourceMode.TemplatedParent;

                    Binding_a38f5c6d36a041d7a1aa1ebf497a69bf.RelativeSource = RelativeSource_96ccfc9d72824808bbc7d331f72be190;


                    Binding_a38f5c6d36a041d7a1aa1ebf497a69bf.TemplateOwner = templateInstance_fd4981d633404162b8c64b19044a50f4;

                    var Binding_809e267f2c9846979eeff86db2301129 = new Bridge.global.Windows.UI.Xaml.Data.Binding.ctor();
                    Binding_809e267f2c9846979eeff86db2301129.Path = new Bridge.global.Windows.UI.Xaml.PropertyPath.ctor("Padding");
                    var RelativeSource_20bffe061e764c4aa3b172093bd09b0c = new Bridge.global.Windows.UI.Xaml.Data.RelativeSource.ctor();
                    RelativeSource_20bffe061e764c4aa3b172093bd09b0c.Mode = Bridge.global.Windows.UI.Xaml.Data.RelativeSourceMode.TemplatedParent;

                    Binding_809e267f2c9846979eeff86db2301129.RelativeSource = RelativeSource_20bffe061e764c4aa3b172093bd09b0c;


                    Binding_809e267f2c9846979eeff86db2301129.TemplateOwner = templateInstance_fd4981d633404162b8c64b19044a50f4;

                    var Binding_406f86543ecd42e5aa6dced46c64179c = new Bridge.global.Windows.UI.Xaml.Data.Binding.ctor();
                    Binding_406f86543ecd42e5aa6dced46c64179c.Path = new Bridge.global.Windows.UI.Xaml.PropertyPath.ctor("HorizontalContentAlignment");
                    var RelativeSource_fa3a4484a8b246268845b5a8054ee85a = new Bridge.global.Windows.UI.Xaml.Data.RelativeSource.ctor();
                    RelativeSource_fa3a4484a8b246268845b5a8054ee85a.Mode = Bridge.global.Windows.UI.Xaml.Data.RelativeSourceMode.TemplatedParent;

                    Binding_406f86543ecd42e5aa6dced46c64179c.RelativeSource = RelativeSource_fa3a4484a8b246268845b5a8054ee85a;


                    Binding_406f86543ecd42e5aa6dced46c64179c.TemplateOwner = templateInstance_fd4981d633404162b8c64b19044a50f4;

                    var Binding_270cd8635439493682cbbe557fbe8b50 = new Bridge.global.Windows.UI.Xaml.Data.Binding.ctor();
                    Binding_270cd8635439493682cbbe557fbe8b50.Path = new Bridge.global.Windows.UI.Xaml.PropertyPath.ctor("VerticalContentAlignment");
                    var RelativeSource_5bfc00804632465999a412f2552abcdc = new Bridge.global.Windows.UI.Xaml.Data.RelativeSource.ctor();
                    RelativeSource_5bfc00804632465999a412f2552abcdc.Mode = Bridge.global.Windows.UI.Xaml.Data.RelativeSourceMode.TemplatedParent;

                    Binding_270cd8635439493682cbbe557fbe8b50.RelativeSource = RelativeSource_5bfc00804632465999a412f2552abcdc;


                    Binding_270cd8635439493682cbbe557fbe8b50.TemplateOwner = templateInstance_fd4981d633404162b8c64b19044a50f4;


                    Border_953ebd6aacbd4a06bfb4e042e79b9243.Child = ContentPresenter_ec38531e4fdd40f897dd558287d87ea5;

                    var Binding_4f7c9a632e9a4b0a95910fea92d92dd4 = new Bridge.global.Windows.UI.Xaml.Data.Binding.ctor();
                    Binding_4f7c9a632e9a4b0a95910fea92d92dd4.Path = new Bridge.global.Windows.UI.Xaml.PropertyPath.ctor("Background");
                    var RelativeSource_792aa2f028304b8aa7f781e9a415496e = new Bridge.global.Windows.UI.Xaml.Data.RelativeSource.ctor();
                    RelativeSource_792aa2f028304b8aa7f781e9a415496e.Mode = Bridge.global.Windows.UI.Xaml.Data.RelativeSourceMode.TemplatedParent;

                    Binding_4f7c9a632e9a4b0a95910fea92d92dd4.RelativeSource = RelativeSource_792aa2f028304b8aa7f781e9a415496e;


                    Binding_4f7c9a632e9a4b0a95910fea92d92dd4.TemplateOwner = templateInstance_fd4981d633404162b8c64b19044a50f4;


                    Border_8a600a143efe448e926edfdf50bac42d.Child = Border_953ebd6aacbd4a06bfb4e042e79b9243;

                    var Binding_f417655bd7414d5781f6d04ace469051 = new Bridge.global.Windows.UI.Xaml.Data.Binding.ctor();
                    Binding_f417655bd7414d5781f6d04ace469051.Path = new Bridge.global.Windows.UI.Xaml.PropertyPath.ctor("Background");
                    var RelativeSource_b52f5dc9d6554232b321259c7570f556 = new Bridge.global.Windows.UI.Xaml.Data.RelativeSource.ctor();
                    RelativeSource_b52f5dc9d6554232b321259c7570f556.Mode = Bridge.global.Windows.UI.Xaml.Data.RelativeSourceMode.TemplatedParent;

                    Binding_f417655bd7414d5781f6d04ace469051.RelativeSource = RelativeSource_b52f5dc9d6554232b321259c7570f556;


                    Binding_f417655bd7414d5781f6d04ace469051.TemplateOwner = templateInstance_fd4981d633404162b8c64b19044a50f4;

                    var Binding_52808fccb5f34f26b7920c33d7a9a3df = new Bridge.global.Windows.UI.Xaml.Data.Binding.ctor();
                    Binding_52808fccb5f34f26b7920c33d7a9a3df.Path = new Bridge.global.Windows.UI.Xaml.PropertyPath.ctor("BorderBrush");
                    var RelativeSource_f97d5e76469c4960a3c5e4eb33907993 = new Bridge.global.Windows.UI.Xaml.Data.RelativeSource.ctor();
                    RelativeSource_f97d5e76469c4960a3c5e4eb33907993.Mode = Bridge.global.Windows.UI.Xaml.Data.RelativeSourceMode.TemplatedParent;

                    Binding_52808fccb5f34f26b7920c33d7a9a3df.RelativeSource = RelativeSource_f97d5e76469c4960a3c5e4eb33907993;


                    Binding_52808fccb5f34f26b7920c33d7a9a3df.TemplateOwner = templateInstance_fd4981d633404162b8c64b19044a50f4;

                    var Binding_8db4443b19ce4374a3fc8e8f50685cdf = new Bridge.global.Windows.UI.Xaml.Data.Binding.ctor();
                    Binding_8db4443b19ce4374a3fc8e8f50685cdf.Path = new Bridge.global.Windows.UI.Xaml.PropertyPath.ctor("BorderThickness");
                    var RelativeSource_6bf0803337a749d7ad10ef98069e4cd5 = new Bridge.global.Windows.UI.Xaml.Data.RelativeSource.ctor();
                    RelativeSource_6bf0803337a749d7ad10ef98069e4cd5.Mode = Bridge.global.Windows.UI.Xaml.Data.RelativeSourceMode.TemplatedParent;

                    Binding_8db4443b19ce4374a3fc8e8f50685cdf.RelativeSource = RelativeSource_6bf0803337a749d7ad10ef98069e4cd5;


                    Binding_8db4443b19ce4374a3fc8e8f50685cdf.TemplateOwner = templateInstance_fd4981d633404162b8c64b19044a50f4;


                    Bridge.global.Windows.UI.Xaml.Data.BindingOperations.SetBinding(ContentPresenter_ec38531e4fdd40f897dd558287d87ea5, Bridge.global.Windows.UI.Xaml.Controls.ContentControl.ContentTemplateProperty, Binding_8321a3bd6a1e46f9864eb1785c3d1dd5);
                    Bridge.global.Windows.UI.Xaml.Data.BindingOperations.SetBinding(ContentPresenter_ec38531e4fdd40f897dd558287d87ea5, Bridge.global.Windows.UI.Xaml.Controls.ContentControl.ContentProperty, Binding_a38f5c6d36a041d7a1aa1ebf497a69bf);
                    Bridge.global.Windows.UI.Xaml.Data.BindingOperations.SetBinding(ContentPresenter_ec38531e4fdd40f897dd558287d87ea5, Bridge.global.Windows.UI.Xaml.FrameworkElement.MarginProperty, Binding_809e267f2c9846979eeff86db2301129);
                    Bridge.global.Windows.UI.Xaml.Data.BindingOperations.SetBinding(ContentPresenter_ec38531e4fdd40f897dd558287d87ea5, Bridge.global.Windows.UI.Xaml.FrameworkElement.HorizontalAlignmentProperty, Binding_406f86543ecd42e5aa6dced46c64179c);
                    Bridge.global.Windows.UI.Xaml.Data.BindingOperations.SetBinding(ContentPresenter_ec38531e4fdd40f897dd558287d87ea5, Bridge.global.Windows.UI.Xaml.FrameworkElement.VerticalAlignmentProperty, Binding_270cd8635439493682cbbe557fbe8b50);
                    Bridge.global.Windows.UI.Xaml.Data.BindingOperations.SetBinding(Border_953ebd6aacbd4a06bfb4e042e79b9243, Bridge.global.Windows.UI.Xaml.Controls.Border.BackgroundProperty, Binding_4f7c9a632e9a4b0a95910fea92d92dd4);
                    Bridge.global.Windows.UI.Xaml.Data.BindingOperations.SetBinding(Border_8a600a143efe448e926edfdf50bac42d, Bridge.global.Windows.UI.Xaml.Controls.Border.BackgroundProperty, Binding_f417655bd7414d5781f6d04ace469051);
                    Bridge.global.Windows.UI.Xaml.Data.BindingOperations.SetBinding(Border_8a600a143efe448e926edfdf50bac42d, Bridge.global.Windows.UI.Xaml.Controls.Border.BorderBrushProperty, Binding_52808fccb5f34f26b7920c33d7a9a3df);
                    Bridge.global.Windows.UI.Xaml.Data.BindingOperations.SetBinding(Border_8a600a143efe448e926edfdf50bac42d, Bridge.global.Windows.UI.Xaml.Controls.Border.BorderThicknessProperty, Binding_8db4443b19ce4374a3fc8e8f50685cdf);

                    Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard.SetTargetProperty(ObjectAnimationUsingKeyFrames_617bbce67f9844ecae92ff34cdd56cba, new Bridge.global.Windows.UI.Xaml.PropertyPath.$ctor1("Background", "Background", Bridge.fn.cacheBind(this, this.accessVisualStateProperty_231e001084c24382890adb2218a4c80e), Bridge.fn.cacheBind(this, this.setVisualStateProperty_231e001084c24382890adb2218a4c80e), Bridge.fn.cacheBind(this, this.setAnimationVisualStateProperty_231e001084c24382890adb2218a4c80e), Bridge.fn.cacheBind(this, this.setLocalVisualStateProperty_231e001084c24382890adb2218a4c80e), Bridge.fn.cacheBind(this, this.getVisualStateProperty_231e001084c24382890adb2218a4c80e)));
                    Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard.SetTarget(ObjectAnimationUsingKeyFrames_617bbce67f9844ecae92ff34cdd56cba, Border_953ebd6aacbd4a06bfb4e042e79b9243);


                    Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard.SetTargetProperty(ObjectAnimationUsingKeyFrames_44ef8479ad214c1c944cbf82800b05a3, new Bridge.global.Windows.UI.Xaml.PropertyPath.$ctor1("Background", "Background", Bridge.fn.cacheBind(this, this.accessVisualStateProperty_b45f666b433c4faa807f8e5a4ac0a015), Bridge.fn.cacheBind(this, this.setVisualStateProperty_b45f666b433c4faa807f8e5a4ac0a015), Bridge.fn.cacheBind(this, this.setAnimationVisualStateProperty_b45f666b433c4faa807f8e5a4ac0a015), Bridge.fn.cacheBind(this, this.setLocalVisualStateProperty_b45f666b433c4faa807f8e5a4ac0a015), Bridge.fn.cacheBind(this, this.getVisualStateProperty_b45f666b433c4faa807f8e5a4ac0a015)));
                    Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard.SetTarget(ObjectAnimationUsingKeyFrames_44ef8479ad214c1c944cbf82800b05a3, Border_953ebd6aacbd4a06bfb4e042e79b9243);


                    Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard.SetTargetProperty(ObjectAnimationUsingKeyFrames_db8299a8672c4ce2831ff169752f94f8, new Bridge.global.Windows.UI.Xaml.PropertyPath.$ctor1("Background", "Background", Bridge.fn.cacheBind(this, this.accessVisualStateProperty_90a4b1405b3a4fd5b8970fdf07454897), Bridge.fn.cacheBind(this, this.setVisualStateProperty_90a4b1405b3a4fd5b8970fdf07454897), Bridge.fn.cacheBind(this, this.setAnimationVisualStateProperty_90a4b1405b3a4fd5b8970fdf07454897), Bridge.fn.cacheBind(this, this.setLocalVisualStateProperty_90a4b1405b3a4fd5b8970fdf07454897), Bridge.fn.cacheBind(this, this.getVisualStateProperty_90a4b1405b3a4fd5b8970fdf07454897)));
                    Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard.SetTarget(ObjectAnimationUsingKeyFrames_db8299a8672c4ce2831ff169752f94f8, Border_953ebd6aacbd4a06bfb4e042e79b9243);

                    templateInstance_fd4981d633404162b8c64b19044a50f4.TemplateContent = Border_8a600a143efe448e926edfdf50bac42d;
                    return templateInstance_fd4981d633404162b8c64b19044a50f4;
                }));

                Setter_ab26d55926394ee68b354e7426fbf1fc.Value = ControlTemplate_7d2711f0622c49cc84869d61a5953773;


                Style_e80aef6cbbfe4fd59ae809d5756634c8.Setters.add(Setter_2ccc35a3c40940adbf35838d4ab1a89c);
                Style_e80aef6cbbfe4fd59ae809d5756634c8.Setters.add(Setter_7f77824685d54080b034aa136dd02dcb);
                Style_e80aef6cbbfe4fd59ae809d5756634c8.Setters.add(Setter_7585f7ad790e4cd5872b2455dd31e89a);
                Style_e80aef6cbbfe4fd59ae809d5756634c8.Setters.add(Setter_559a5db552204b83910071d4b1ec6cfe);
                Style_e80aef6cbbfe4fd59ae809d5756634c8.Setters.add(Setter_2ab92eaf7d344a4c9e7e7817f5348be2);
                Style_e80aef6cbbfe4fd59ae809d5756634c8.Setters.add(Setter_b558d56538234a7d98cc902f4a48835d);
                Style_e80aef6cbbfe4fd59ae809d5756634c8.Setters.add(Setter_b04f134c5580444a98029099b35ebe0f);
                Style_e80aef6cbbfe4fd59ae809d5756634c8.Setters.add(Setter_ab26d55926394ee68b354e7426fbf1fc);


                this.setItem(Bridge.global.Windows.UI.Xaml.Controls.Button, Style_e80aef6cbbfe4fd59ae809d5756634c8);






            },
            accessVisualStateProperty_231e001084c24382890adb2218a4c80e: function (rootTargetObjectInstance) {
                return new (Bridge.GeneratorEnumerable$1(System.Tuple$3(Windows.UI.Xaml.DependencyObject,Windows.UI.Xaml.DependencyProperty,System.Nullable$1(System.Int32))))(Bridge.fn.bind(this, function (rootTargetObjectInstance) {
                    var $step = 0,
                        $jumpFromFinally,
                        $returnValue,
                        $async_e;

                    var $enumerator = new (Bridge.GeneratorEnumerator$1(System.Tuple$3(Windows.UI.Xaml.DependencyObject,Windows.UI.Xaml.DependencyProperty,System.Nullable$1(System.Int32))))(Bridge.fn.bind(this, function () {
                        try {
                            for (;;) {
                                switch ($step) {
                                    case 0: {
                                        return false;
                                    }
                                    default: {
                                        return false;
                                    }
                                }
                            }
                        } catch($async_e1) {
                            $async_e = System.Exception.create($async_e1);
                            throw $async_e;
                        }
                    }));
                    return $enumerator;
                }, arguments));
            },
            setVisualStateProperty_231e001084c24382890adb2218a4c80e: function (finalTargetInstance, value) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Background").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "BackgroundProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                (finalTargetInstance).SetVisualStateValue(property, value);
            },
            setAnimationVisualStateProperty_231e001084c24382890adb2218a4c80e: function (finalTargetInstance, value) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Background").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "BackgroundProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                (finalTargetInstance).SetAnimationValue(property, value);
            },
            setLocalVisualStateProperty_231e001084c24382890adb2218a4c80e: function (finalTargetInstance, value) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Background").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "BackgroundProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                (finalTargetInstance).SetLocalValue(property, value);
            },
            getVisualStateProperty_231e001084c24382890adb2218a4c80e: function (finalTargetInstance) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Background").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "BackgroundProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                return finalTargetInstance.GetVisualStateValue(property);
            },
            accessVisualStateProperty_b45f666b433c4faa807f8e5a4ac0a015: function (rootTargetObjectInstance) {
                return new (Bridge.GeneratorEnumerable$1(System.Tuple$3(Windows.UI.Xaml.DependencyObject,Windows.UI.Xaml.DependencyProperty,System.Nullable$1(System.Int32))))(Bridge.fn.bind(this, function (rootTargetObjectInstance) {
                    var $step = 0,
                        $jumpFromFinally,
                        $returnValue,
                        $async_e;

                    var $enumerator = new (Bridge.GeneratorEnumerator$1(System.Tuple$3(Windows.UI.Xaml.DependencyObject,Windows.UI.Xaml.DependencyProperty,System.Nullable$1(System.Int32))))(Bridge.fn.bind(this, function () {
                        try {
                            for (;;) {
                                switch ($step) {
                                    case 0: {
                                        return false;
                                    }
                                    default: {
                                        return false;
                                    }
                                }
                            }
                        } catch($async_e1) {
                            $async_e = System.Exception.create($async_e1);
                            throw $async_e;
                        }
                    }));
                    return $enumerator;
                }, arguments));
            },
            setVisualStateProperty_b45f666b433c4faa807f8e5a4ac0a015: function (finalTargetInstance, value) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Background").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "BackgroundProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                (finalTargetInstance).SetVisualStateValue(property, value);
            },
            setAnimationVisualStateProperty_b45f666b433c4faa807f8e5a4ac0a015: function (finalTargetInstance, value) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Background").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "BackgroundProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                (finalTargetInstance).SetAnimationValue(property, value);
            },
            setLocalVisualStateProperty_b45f666b433c4faa807f8e5a4ac0a015: function (finalTargetInstance, value) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Background").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "BackgroundProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                (finalTargetInstance).SetLocalValue(property, value);
            },
            getVisualStateProperty_b45f666b433c4faa807f8e5a4ac0a015: function (finalTargetInstance) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Background").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "BackgroundProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                return finalTargetInstance.GetVisualStateValue(property);
            },
            accessVisualStateProperty_90a4b1405b3a4fd5b8970fdf07454897: function (rootTargetObjectInstance) {
                return new (Bridge.GeneratorEnumerable$1(System.Tuple$3(Windows.UI.Xaml.DependencyObject,Windows.UI.Xaml.DependencyProperty,System.Nullable$1(System.Int32))))(Bridge.fn.bind(this, function (rootTargetObjectInstance) {
                    var $step = 0,
                        $jumpFromFinally,
                        $returnValue,
                        $async_e;

                    var $enumerator = new (Bridge.GeneratorEnumerator$1(System.Tuple$3(Windows.UI.Xaml.DependencyObject,Windows.UI.Xaml.DependencyProperty,System.Nullable$1(System.Int32))))(Bridge.fn.bind(this, function () {
                        try {
                            for (;;) {
                                switch ($step) {
                                    case 0: {
                                        return false;
                                    }
                                    default: {
                                        return false;
                                    }
                                }
                            }
                        } catch($async_e1) {
                            $async_e = System.Exception.create($async_e1);
                            throw $async_e;
                        }
                    }));
                    return $enumerator;
                }, arguments));
            },
            setVisualStateProperty_90a4b1405b3a4fd5b8970fdf07454897: function (finalTargetInstance, value) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Background").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "BackgroundProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                (finalTargetInstance).SetVisualStateValue(property, value);
            },
            setAnimationVisualStateProperty_90a4b1405b3a4fd5b8970fdf07454897: function (finalTargetInstance, value) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Background").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "BackgroundProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                (finalTargetInstance).SetAnimationValue(property, value);
            },
            setLocalVisualStateProperty_90a4b1405b3a4fd5b8970fdf07454897: function (finalTargetInstance, value) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Background").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "BackgroundProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                (finalTargetInstance).SetLocalValue(property, value);
            },
            getVisualStateProperty_90a4b1405b3a4fd5b8970fdf07454897: function (finalTargetInstance) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Background").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "BackgroundProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                return finalTargetInstance.GetVisualStateValue(property);
            }
        }
    });

    Bridge.define("___CSHTML5__Component__Defaultstyles__INTERNAL_Defaultchildwindowstyle__Xaml", {
        inherits: [Bridge.global.Windows.UI.Xaml.ResourceDictionary],
        fields: {
            _contentLoaded: false
        },
        ctors: {
            ctor: function () {
                this.$initialize();
                Bridge.global.Windows.UI.Xaml.ResourceDictionary.ctor.call(this);
                this.InitializeComponent();

                if (Bridge.is(this, Windows.UI.Xaml.UIElement)) {
                    Bridge.cast(this, Windows.UI.Xaml.UIElement).XamlSourcePath = "CSHTML5\\DefaultStyles\\INTERNAL_DefaultChildWindowStyle.xaml";
                }
            }
        },
        methods: {
            InitializeComponent: function () {
                var $t;
                if (this._contentLoaded) {
                    return;
                }
                this._contentLoaded = true;


                if (Bridge.is(this, Windows.UI.Xaml.UIElement)) {
                    Bridge.cast(this, Windows.UI.Xaml.UIElement).XamlSourcePath = "CSHTML5\\DefaultStyles\\INTERNAL_DefaultChildWindowStyle.xaml";
                }



                var Style_0853948daf174da58444bd0f43976de8 = new Bridge.global.Windows.UI.Xaml.Style.ctor();
                Style_0853948daf174da58444bd0f43976de8.TargetType = Bridge.global.Windows.UI.Xaml.Controls.ChildWindow;
                var Setter_f145e206c3a74d3fb108360a1736af3e = new Bridge.global.Windows.UI.Xaml.Setter.ctor();
                Setter_f145e206c3a74d3fb108360a1736af3e.Property = Bridge.global.Windows.UI.Xaml.FrameworkElement.HorizontalAlignmentProperty;
                Setter_f145e206c3a74d3fb108360a1736af3e.Value = Bridge.box(Bridge.global.Windows.UI.Xaml.HorizontalAlignment.Center, Windows.UI.Xaml.HorizontalAlignment, System.Enum.toStringFn(Windows.UI.Xaml.HorizontalAlignment));

                var Setter_750a022c06d84ff0bedcc09d25c2b355 = new Bridge.global.Windows.UI.Xaml.Setter.ctor();
                Setter_750a022c06d84ff0bedcc09d25c2b355.Property = Bridge.global.Windows.UI.Xaml.FrameworkElement.VerticalAlignmentProperty;
                Setter_750a022c06d84ff0bedcc09d25c2b355.Value = Bridge.box(Bridge.global.Windows.UI.Xaml.VerticalAlignment.Center, Windows.UI.Xaml.VerticalAlignment, System.Enum.toStringFn(Windows.UI.Xaml.VerticalAlignment));

                var Setter_526d1399d29746c0b4d4c10dbc094ac5 = new Bridge.global.Windows.UI.Xaml.Setter.ctor();
                Setter_526d1399d29746c0b4d4c10dbc094ac5.Property = Bridge.global.Windows.UI.Xaml.Controls.Control.HorizontalContentAlignmentProperty;
                Setter_526d1399d29746c0b4d4c10dbc094ac5.Value = Bridge.box(Bridge.global.Windows.UI.Xaml.HorizontalAlignment.Stretch, Windows.UI.Xaml.HorizontalAlignment, System.Enum.toStringFn(Windows.UI.Xaml.HorizontalAlignment));

                var Setter_62ca582ffdf24365a58db0d7bb6bfa61 = new Bridge.global.Windows.UI.Xaml.Setter.ctor();
                Setter_62ca582ffdf24365a58db0d7bb6bfa61.Property = Bridge.global.Windows.UI.Xaml.Controls.Control.VerticalContentAlignmentProperty;
                Setter_62ca582ffdf24365a58db0d7bb6bfa61.Value = Bridge.box(Bridge.global.Windows.UI.Xaml.VerticalAlignment.Stretch, Windows.UI.Xaml.VerticalAlignment, System.Enum.toStringFn(Windows.UI.Xaml.VerticalAlignment));

                var Setter_1a2aaf87a9fd46c18ddee47324f20ece = new Bridge.global.Windows.UI.Xaml.Setter.ctor();
                Setter_1a2aaf87a9fd46c18ddee47324f20ece.Property = Bridge.global.Windows.UI.Xaml.Controls.Control.BorderThicknessProperty;
                Setter_1a2aaf87a9fd46c18ddee47324f20ece.Value = new Bridge.global.Windows.UI.Xaml.Thickness.$ctor1(1).$clone();

                var Setter_88865afccfcb423ba1558aaa9b729da5 = new Bridge.global.Windows.UI.Xaml.Setter.ctor();
                Setter_88865afccfcb423ba1558aaa9b729da5.Property = Bridge.global.Windows.UI.Xaml.Controls.Control.BorderBrushProperty;
                Setter_88865afccfcb423ba1558aaa9b729da5.Value = new Bridge.global.Windows.UI.Xaml.Media.SolidColorBrush.$ctor1(($t = new Bridge.global.Windows.UI.Color(), $t.A = 255, $t.R = 226, $t.G = 226, $t.B = 226, $t));

                var Setter_e8eb86f0608b498eb60e437df0643531 = new Bridge.global.Windows.UI.Xaml.Setter.ctor();
                Setter_e8eb86f0608b498eb60e437df0643531.Property = Bridge.global.Windows.UI.Xaml.Controls.ChildWindow.OverlayBrushProperty;
                Setter_e8eb86f0608b498eb60e437df0643531.Value = new Bridge.global.Windows.UI.Xaml.Media.SolidColorBrush.$ctor1(($t = new Bridge.global.Windows.UI.Color(), $t.A = 127, $t.R = 0, $t.G = 0, $t.B = 0, $t));

                var Setter_aed6f53901e849d29415a40834a387d4 = new Bridge.global.Windows.UI.Xaml.Setter.ctor();
                Setter_aed6f53901e849d29415a40834a387d4.Property = Bridge.global.Windows.UI.Xaml.Controls.ChildWindow.OverlayOpacityProperty;
                Setter_aed6f53901e849d29415a40834a387d4.Value = Bridge.box(1.0, System.Double, System.Double.format, System.Double.getHashCode);

                var Setter_544f565953a34feaa0be15f0e96de89f = new Bridge.global.Windows.UI.Xaml.Setter.ctor();
                Setter_544f565953a34feaa0be15f0e96de89f.Property = Bridge.global.Windows.UI.Xaml.Controls.Control.TemplateProperty;
                var ControlTemplate_47209f627dd648bd92dc64ec9ad5e158 = new Bridge.global.Windows.UI.Xaml.Controls.ControlTemplate();
                ControlTemplate_47209f627dd648bd92dc64ec9ad5e158.TargetType = Bridge.global.Windows.UI.Xaml.Controls.ChildWindow;
                ControlTemplate_47209f627dd648bd92dc64ec9ad5e158.SetMethodToInstantiateFrameworkTemplate(function (templateOwner_ControlTemplate_47209f627dd648bd92dc64ec9ad5e158) {
                    var $t1;
                    var templateInstance_1e65d36b27fc47d6a46643364cc7d8ca = new Bridge.global.Windows.UI.Xaml.TemplateInstance();
                    templateInstance_1e65d36b27fc47d6a46643364cc7d8ca.TemplateOwner = templateOwner_ControlTemplate_47209f627dd648bd92dc64ec9ad5e158;
                    var Grid_fc6483f21ec149318260dbab3e348b5c = new Bridge.global.Windows.UI.Xaml.Controls.Grid();
                    var Button_fbee96f2daca4760983c6a761bf4fe4e = new Bridge.global.Windows.UI.Xaml.Controls.Button();
                    var Border_b2dc8ee6bca6497796b29b0b77b89dbf = new Bridge.global.Windows.UI.Xaml.Controls.Border();
                    var ContentPresenter_dccc3538b14a4f10b33e74e7708e94e6 = new Bridge.global.Windows.UI.Xaml.Controls.ContentPresenter();
                    var Border_46b4592f744d4bf8842609a57855801e = new Bridge.global.Windows.UI.Xaml.Controls.Border();
                    var Border_b18d9efc9f794aa6a9ccfdecbdbafdbf = new Bridge.global.Windows.UI.Xaml.Controls.Border();
                    var Grid_259d3e21cb824aa2bdea39bcc545610a = new Bridge.global.Windows.UI.Xaml.Controls.Grid();
                    templateOwner_ControlTemplate_47209f627dd648bd92dc64ec9ad5e158.RegisterName("Root", Grid_259d3e21cb824aa2bdea39bcc545610a);
                    Grid_259d3e21cb824aa2bdea39bcc545610a.Name = "Root";
                    templateOwner_ControlTemplate_47209f627dd648bd92dc64ec9ad5e158.RegisterName("Overlay", Grid_fc6483f21ec149318260dbab3e348b5c);
                    Grid_fc6483f21ec149318260dbab3e348b5c.Name = "Overlay";
                    Grid_fc6483f21ec149318260dbab3e348b5c.HorizontalAlignment = Bridge.global.Windows.UI.Xaml.HorizontalAlignment.Stretch;
                    Grid_fc6483f21ec149318260dbab3e348b5c.VerticalAlignment = Bridge.global.Windows.UI.Xaml.VerticalAlignment.Stretch;
                    Grid_fc6483f21ec149318260dbab3e348b5c.Margin = new Bridge.global.Windows.UI.Xaml.Thickness.$ctor1(0);
                    var Binding_80efdcb88ccc40578550711dac39de91 = new Bridge.global.Windows.UI.Xaml.Data.Binding.ctor();
                    Binding_80efdcb88ccc40578550711dac39de91.Path = new Bridge.global.Windows.UI.Xaml.PropertyPath.ctor("OverlayBrush");
                    var RelativeSource_8d152577014d487285e75b289e2ff1a8 = new Bridge.global.Windows.UI.Xaml.Data.RelativeSource.ctor();
                    RelativeSource_8d152577014d487285e75b289e2ff1a8.Mode = Bridge.global.Windows.UI.Xaml.Data.RelativeSourceMode.TemplatedParent;

                    Binding_80efdcb88ccc40578550711dac39de91.RelativeSource = RelativeSource_8d152577014d487285e75b289e2ff1a8;


                    Binding_80efdcb88ccc40578550711dac39de91.TemplateOwner = templateInstance_1e65d36b27fc47d6a46643364cc7d8ca;

                    var Binding_4d6bc8165902407da6132a87bdc088b2 = new Bridge.global.Windows.UI.Xaml.Data.Binding.ctor();
                    Binding_4d6bc8165902407da6132a87bdc088b2.Path = new Bridge.global.Windows.UI.Xaml.PropertyPath.ctor("OverlayOpacity");
                    var RelativeSource_1745bc942ece4e24965639b9a6fab850 = new Bridge.global.Windows.UI.Xaml.Data.RelativeSource.ctor();
                    RelativeSource_1745bc942ece4e24965639b9a6fab850.Mode = Bridge.global.Windows.UI.Xaml.Data.RelativeSourceMode.TemplatedParent;

                    Binding_4d6bc8165902407da6132a87bdc088b2.RelativeSource = RelativeSource_1745bc942ece4e24965639b9a6fab850;


                    Binding_4d6bc8165902407da6132a87bdc088b2.TemplateOwner = templateInstance_1e65d36b27fc47d6a46643364cc7d8ca;


                    templateOwner_ControlTemplate_47209f627dd648bd92dc64ec9ad5e158.RegisterName("ContentRoot", Border_b18d9efc9f794aa6a9ccfdecbdbafdbf);
                    Border_b18d9efc9f794aa6a9ccfdecbdbafdbf.Name = "ContentRoot";
                    Border_b18d9efc9f794aa6a9ccfdecbdbafdbf.RenderTransformOrigin = new Bridge.global.Windows.Foundation.Point.$ctor1(0.5, 0.5);
                    templateOwner_ControlTemplate_47209f627dd648bd92dc64ec9ad5e158.RegisterName("ContentContainer", Border_46b4592f744d4bf8842609a57855801e);
                    Border_46b4592f744d4bf8842609a57855801e.Name = "ContentContainer";
                    Border_46b4592f744d4bf8842609a57855801e.Background = new Bridge.global.Windows.UI.Xaml.Media.SolidColorBrush.$ctor1(($t1 = new Bridge.global.Windows.UI.Color(), $t1.A = 255, $t1.R = 246, $t1.G = 246, $t1.B = 246, $t1));
                    Border_46b4592f744d4bf8842609a57855801e.CornerRadius = new Bridge.global.Windows.UI.Xaml.CornerRadius.$ctor1(2);
                    var Grid_c0dc8dc891c2412f90847430474db8a6 = new Bridge.global.Windows.UI.Xaml.Controls.Grid();
                    var RowDefinition_b41fd886326b452288d70020dbc41e9b = new Bridge.global.Windows.UI.Xaml.Controls.RowDefinition();
                    RowDefinition_b41fd886326b452288d70020dbc41e9b.Height = new Bridge.global.Windows.UI.Xaml.GridLength.$ctor2(1.0, Bridge.global.Windows.UI.Xaml.GridUnitType.Auto);

                    var RowDefinition_61595449c0c94c91ac9f1bd0e56d37ac = new Bridge.global.Windows.UI.Xaml.Controls.RowDefinition();

                    Grid_c0dc8dc891c2412f90847430474db8a6.RowDefinitions.add(RowDefinition_b41fd886326b452288d70020dbc41e9b);
                    Grid_c0dc8dc891c2412f90847430474db8a6.RowDefinitions.add(RowDefinition_61595449c0c94c91ac9f1bd0e56d37ac);

                    templateOwner_ControlTemplate_47209f627dd648bd92dc64ec9ad5e158.RegisterName("Chrome", Border_b2dc8ee6bca6497796b29b0b77b89dbf);
                    Border_b2dc8ee6bca6497796b29b0b77b89dbf.Name = "Chrome";
                    Border_b2dc8ee6bca6497796b29b0b77b89dbf.Width = Number.NaN;
                    Border_b2dc8ee6bca6497796b29b0b77b89dbf.BorderBrush = new Bridge.global.Windows.UI.Xaml.Media.SolidColorBrush.$ctor1(($t1 = new Bridge.global.Windows.UI.Color(), $t1.A = 255, $t1.R = 255, $t1.G = 255, $t1.B = 255, $t1));
                    Border_b2dc8ee6bca6497796b29b0b77b89dbf.BorderThickness = new Bridge.global.Windows.UI.Xaml.Thickness.$ctor2(0, 0, 0, 1);
                    Border_b2dc8ee6bca6497796b29b0b77b89dbf.Background = new Bridge.global.Windows.UI.Xaml.Media.SolidColorBrush.$ctor1(($t1 = new Bridge.global.Windows.UI.Color(), $t1.A = 255, $t1.R = 238, $t1.G = 238, $t1.B = 238, $t1));
                    var Grid_6f82d44ccbd34c2e845984cc7e9963f1 = new Bridge.global.Windows.UI.Xaml.Controls.Grid();
                    Grid_6f82d44ccbd34c2e845984cc7e9963f1.Height = Number.NaN;
                    Grid_6f82d44ccbd34c2e845984cc7e9963f1.Width = Number.NaN;
                    var ColumnDefinition_c0caffb708a64eb1bf7439b2358667b7 = new Bridge.global.Windows.UI.Xaml.Controls.ColumnDefinition();

                    var ColumnDefinition_0d19d81408a1461d958b453b6df6efe8 = new Bridge.global.Windows.UI.Xaml.Controls.ColumnDefinition();
                    ColumnDefinition_0d19d81408a1461d958b453b6df6efe8.Width = new Bridge.global.Windows.UI.Xaml.GridLength.$ctor2(30, Bridge.global.Windows.UI.Xaml.GridUnitType.Pixel);

                    Grid_6f82d44ccbd34c2e845984cc7e9963f1.ColumnDefinitions.add(ColumnDefinition_c0caffb708a64eb1bf7439b2358667b7);
                    Grid_6f82d44ccbd34c2e845984cc7e9963f1.ColumnDefinitions.add(ColumnDefinition_0d19d81408a1461d958b453b6df6efe8);

                    var ContentControl_bbe5957d7ebf40e8a5259762cf86018d = new Bridge.global.Windows.UI.Xaml.Controls.ContentControl();
                    ContentControl_bbe5957d7ebf40e8a5259762cf86018d.FontWeight = System.Nullable.getValue(Bridge.cast(Bridge.unbox(Bridge.global.DotNetForHtml5.Core.TypeFromStringConverters.ConvertFromInvariantString(Bridge.global.Windows.UI.Text.FontWeight, "Bold"), Windows.UI.Text.FontWeight), Windows.UI.Text.FontWeight));
                    ContentControl_bbe5957d7ebf40e8a5259762cf86018d.HorizontalAlignment = Bridge.global.Windows.UI.Xaml.HorizontalAlignment.Stretch;
                    ContentControl_bbe5957d7ebf40e8a5259762cf86018d.VerticalAlignment = Bridge.global.Windows.UI.Xaml.VerticalAlignment.Center;
                    ContentControl_bbe5957d7ebf40e8a5259762cf86018d.Margin = new Bridge.global.Windows.UI.Xaml.Thickness.$ctor2(6, 0, 6, 0);
                    var Binding_bbb83b1a08b445aab16d72beee377126 = new Bridge.global.Windows.UI.Xaml.Data.Binding.ctor();
                    Binding_bbb83b1a08b445aab16d72beee377126.Path = new Bridge.global.Windows.UI.Xaml.PropertyPath.ctor("Title");
                    var RelativeSource_b8913dcf11be460a9ca2b38da9c34258 = new Bridge.global.Windows.UI.Xaml.Data.RelativeSource.ctor();
                    RelativeSource_b8913dcf11be460a9ca2b38da9c34258.Mode = Bridge.global.Windows.UI.Xaml.Data.RelativeSourceMode.TemplatedParent;

                    Binding_bbb83b1a08b445aab16d72beee377126.RelativeSource = RelativeSource_b8913dcf11be460a9ca2b38da9c34258;


                    Binding_bbb83b1a08b445aab16d72beee377126.TemplateOwner = templateInstance_1e65d36b27fc47d6a46643364cc7d8ca;


                    templateOwner_ControlTemplate_47209f627dd648bd92dc64ec9ad5e158.RegisterName("CloseButton", Button_fbee96f2daca4760983c6a761bf4fe4e);
                    Button_fbee96f2daca4760983c6a761bf4fe4e.Name = "CloseButton";
                    Button_fbee96f2daca4760983c6a761bf4fe4e.Content = "X";
                    Bridge.global.Windows.UI.Xaml.Controls.Grid.SetColumn(Button_fbee96f2daca4760983c6a761bf4fe4e, 1);
                    Button_fbee96f2daca4760983c6a761bf4fe4e.HorizontalAlignment = Bridge.global.Windows.UI.Xaml.HorizontalAlignment.Center;
                    Button_fbee96f2daca4760983c6a761bf4fe4e.VerticalAlignment = Bridge.global.Windows.UI.Xaml.VerticalAlignment.Center;
                    Button_fbee96f2daca4760983c6a761bf4fe4e.HorizontalContentAlignment = Bridge.global.Windows.UI.Xaml.HorizontalAlignment.Center;
                    Button_fbee96f2daca4760983c6a761bf4fe4e.VerticalContentAlignment = Bridge.global.Windows.UI.Xaml.VerticalAlignment.Center;
                    Button_fbee96f2daca4760983c6a761bf4fe4e.Width = 15.0;
                    Button_fbee96f2daca4760983c6a761bf4fe4e.Padding = new Bridge.global.Windows.UI.Xaml.Thickness.$ctor1(0);

                    Grid_6f82d44ccbd34c2e845984cc7e9963f1.Children.add(ContentControl_bbe5957d7ebf40e8a5259762cf86018d);
                    Grid_6f82d44ccbd34c2e845984cc7e9963f1.Children.add(Button_fbee96f2daca4760983c6a761bf4fe4e);


                    Border_b2dc8ee6bca6497796b29b0b77b89dbf.Child = Grid_6f82d44ccbd34c2e845984cc7e9963f1;


                    var Border_17172c187fa645559e54ed748675a4d9 = new Bridge.global.Windows.UI.Xaml.Controls.Border();
                    Border_17172c187fa645559e54ed748675a4d9.Margin = new Bridge.global.Windows.UI.Xaml.Thickness.$ctor1(7);
                    Bridge.global.Windows.UI.Xaml.Controls.Grid.SetRow(Border_17172c187fa645559e54ed748675a4d9, 1);
                    templateOwner_ControlTemplate_47209f627dd648bd92dc64ec9ad5e158.RegisterName("ContentPresenter", ContentPresenter_dccc3538b14a4f10b33e74e7708e94e6);
                    ContentPresenter_dccc3538b14a4f10b33e74e7708e94e6.Name = "ContentPresenter";
                    var Binding_ae2b7420a4d740a697a0a05b76e511d1 = new Bridge.global.Windows.UI.Xaml.Data.Binding.ctor();
                    Binding_ae2b7420a4d740a697a0a05b76e511d1.Path = new Bridge.global.Windows.UI.Xaml.PropertyPath.ctor("Content");
                    var RelativeSource_2b7a86d1114e46d78d969f8b13c58d29 = new Bridge.global.Windows.UI.Xaml.Data.RelativeSource.ctor();
                    RelativeSource_2b7a86d1114e46d78d969f8b13c58d29.Mode = Bridge.global.Windows.UI.Xaml.Data.RelativeSourceMode.TemplatedParent;

                    Binding_ae2b7420a4d740a697a0a05b76e511d1.RelativeSource = RelativeSource_2b7a86d1114e46d78d969f8b13c58d29;


                    Binding_ae2b7420a4d740a697a0a05b76e511d1.TemplateOwner = templateInstance_1e65d36b27fc47d6a46643364cc7d8ca;

                    var Binding_8de6bdd9cf1143b2a69febbc8f5f0f9c = new Bridge.global.Windows.UI.Xaml.Data.Binding.ctor();
                    Binding_8de6bdd9cf1143b2a69febbc8f5f0f9c.Path = new Bridge.global.Windows.UI.Xaml.PropertyPath.ctor("ContentTemplate");
                    var RelativeSource_b75a62c358994fc0b52fc539df1e2b2a = new Bridge.global.Windows.UI.Xaml.Data.RelativeSource.ctor();
                    RelativeSource_b75a62c358994fc0b52fc539df1e2b2a.Mode = Bridge.global.Windows.UI.Xaml.Data.RelativeSourceMode.TemplatedParent;

                    Binding_8de6bdd9cf1143b2a69febbc8f5f0f9c.RelativeSource = RelativeSource_b75a62c358994fc0b52fc539df1e2b2a;


                    Binding_8de6bdd9cf1143b2a69febbc8f5f0f9c.TemplateOwner = templateInstance_1e65d36b27fc47d6a46643364cc7d8ca;

                    var Binding_46528764844a44369d1ff2496f6bb865 = new Bridge.global.Windows.UI.Xaml.Data.Binding.ctor();
                    Binding_46528764844a44369d1ff2496f6bb865.Path = new Bridge.global.Windows.UI.Xaml.PropertyPath.ctor("HorizontalContentAlignment");
                    var RelativeSource_af523cff471c4a2991cc56e8f30c519a = new Bridge.global.Windows.UI.Xaml.Data.RelativeSource.ctor();
                    RelativeSource_af523cff471c4a2991cc56e8f30c519a.Mode = Bridge.global.Windows.UI.Xaml.Data.RelativeSourceMode.TemplatedParent;

                    Binding_46528764844a44369d1ff2496f6bb865.RelativeSource = RelativeSource_af523cff471c4a2991cc56e8f30c519a;


                    Binding_46528764844a44369d1ff2496f6bb865.TemplateOwner = templateInstance_1e65d36b27fc47d6a46643364cc7d8ca;

                    var Binding_021ac3797eea41cbbaeff10868652a99 = new Bridge.global.Windows.UI.Xaml.Data.Binding.ctor();
                    Binding_021ac3797eea41cbbaeff10868652a99.Path = new Bridge.global.Windows.UI.Xaml.PropertyPath.ctor("VerticalContentAlignment");
                    var RelativeSource_00646614e3dc4d4c98b9890c062503f1 = new Bridge.global.Windows.UI.Xaml.Data.RelativeSource.ctor();
                    RelativeSource_00646614e3dc4d4c98b9890c062503f1.Mode = Bridge.global.Windows.UI.Xaml.Data.RelativeSourceMode.TemplatedParent;

                    Binding_021ac3797eea41cbbaeff10868652a99.RelativeSource = RelativeSource_00646614e3dc4d4c98b9890c062503f1;


                    Binding_021ac3797eea41cbbaeff10868652a99.TemplateOwner = templateInstance_1e65d36b27fc47d6a46643364cc7d8ca;


                    Border_17172c187fa645559e54ed748675a4d9.Child = ContentPresenter_dccc3538b14a4f10b33e74e7708e94e6;

                    var Binding_2de51e22f90a4e2398fe15d99c334724 = new Bridge.global.Windows.UI.Xaml.Data.Binding.ctor();
                    Binding_2de51e22f90a4e2398fe15d99c334724.Path = new Bridge.global.Windows.UI.Xaml.PropertyPath.ctor("Background");
                    var RelativeSource_3d22eb52fb7f4e2cab27c8637d808339 = new Bridge.global.Windows.UI.Xaml.Data.RelativeSource.ctor();
                    RelativeSource_3d22eb52fb7f4e2cab27c8637d808339.Mode = Bridge.global.Windows.UI.Xaml.Data.RelativeSourceMode.TemplatedParent;

                    Binding_2de51e22f90a4e2398fe15d99c334724.RelativeSource = RelativeSource_3d22eb52fb7f4e2cab27c8637d808339;


                    Binding_2de51e22f90a4e2398fe15d99c334724.TemplateOwner = templateInstance_1e65d36b27fc47d6a46643364cc7d8ca;


                    Grid_c0dc8dc891c2412f90847430474db8a6.Children.add(Border_b2dc8ee6bca6497796b29b0b77b89dbf);
                    Grid_c0dc8dc891c2412f90847430474db8a6.Children.add(Border_17172c187fa645559e54ed748675a4d9);


                    Border_46b4592f744d4bf8842609a57855801e.Child = Grid_c0dc8dc891c2412f90847430474db8a6;

                    var Binding_5c0dc2fb9207478f81bc8b7f26cec48b = new Bridge.global.Windows.UI.Xaml.Data.Binding.ctor();
                    Binding_5c0dc2fb9207478f81bc8b7f26cec48b.Path = new Bridge.global.Windows.UI.Xaml.PropertyPath.ctor("BorderThickness");
                    var RelativeSource_fe524dffab044a7fbd5bf4ef25c568f5 = new Bridge.global.Windows.UI.Xaml.Data.RelativeSource.ctor();
                    RelativeSource_fe524dffab044a7fbd5bf4ef25c568f5.Mode = Bridge.global.Windows.UI.Xaml.Data.RelativeSourceMode.TemplatedParent;

                    Binding_5c0dc2fb9207478f81bc8b7f26cec48b.RelativeSource = RelativeSource_fe524dffab044a7fbd5bf4ef25c568f5;


                    Binding_5c0dc2fb9207478f81bc8b7f26cec48b.TemplateOwner = templateInstance_1e65d36b27fc47d6a46643364cc7d8ca;

                    var Binding_ee2e70df603c4b7b88b7f1eb4ab1c02e = new Bridge.global.Windows.UI.Xaml.Data.Binding.ctor();
                    Binding_ee2e70df603c4b7b88b7f1eb4ab1c02e.Path = new Bridge.global.Windows.UI.Xaml.PropertyPath.ctor("BorderBrush");
                    var RelativeSource_69adf19e34a941529e6616be3ea5497d = new Bridge.global.Windows.UI.Xaml.Data.RelativeSource.ctor();
                    RelativeSource_69adf19e34a941529e6616be3ea5497d.Mode = Bridge.global.Windows.UI.Xaml.Data.RelativeSourceMode.TemplatedParent;

                    Binding_ee2e70df603c4b7b88b7f1eb4ab1c02e.RelativeSource = RelativeSource_69adf19e34a941529e6616be3ea5497d;


                    Binding_ee2e70df603c4b7b88b7f1eb4ab1c02e.TemplateOwner = templateInstance_1e65d36b27fc47d6a46643364cc7d8ca;


                    Border_b18d9efc9f794aa6a9ccfdecbdbafdbf.Child = Border_46b4592f744d4bf8842609a57855801e;

                    var Binding_2aee30da23cb4161bf96ec4dc3bfa5c0 = new Bridge.global.Windows.UI.Xaml.Data.Binding.ctor();
                    Binding_2aee30da23cb4161bf96ec4dc3bfa5c0.Path = new Bridge.global.Windows.UI.Xaml.PropertyPath.ctor("HorizontalAlignment");
                    var RelativeSource_bbe234ff935f4d3a8b25151a33d40b5f = new Bridge.global.Windows.UI.Xaml.Data.RelativeSource.ctor();
                    RelativeSource_bbe234ff935f4d3a8b25151a33d40b5f.Mode = Bridge.global.Windows.UI.Xaml.Data.RelativeSourceMode.TemplatedParent;

                    Binding_2aee30da23cb4161bf96ec4dc3bfa5c0.RelativeSource = RelativeSource_bbe234ff935f4d3a8b25151a33d40b5f;


                    Binding_2aee30da23cb4161bf96ec4dc3bfa5c0.TemplateOwner = templateInstance_1e65d36b27fc47d6a46643364cc7d8ca;

                    var Binding_c371756b6a57474ea8c8eff4cdd90e13 = new Bridge.global.Windows.UI.Xaml.Data.Binding.ctor();
                    Binding_c371756b6a57474ea8c8eff4cdd90e13.Path = new Bridge.global.Windows.UI.Xaml.PropertyPath.ctor("VerticalAlignment");
                    var RelativeSource_6d8797d434314c68ba0e0e6c58ecf0f6 = new Bridge.global.Windows.UI.Xaml.Data.RelativeSource.ctor();
                    RelativeSource_6d8797d434314c68ba0e0e6c58ecf0f6.Mode = Bridge.global.Windows.UI.Xaml.Data.RelativeSourceMode.TemplatedParent;

                    Binding_c371756b6a57474ea8c8eff4cdd90e13.RelativeSource = RelativeSource_6d8797d434314c68ba0e0e6c58ecf0f6;


                    Binding_c371756b6a57474ea8c8eff4cdd90e13.TemplateOwner = templateInstance_1e65d36b27fc47d6a46643364cc7d8ca;

                    var Binding_185230fbde66433e8cfd51860201d293 = new Bridge.global.Windows.UI.Xaml.Data.Binding.ctor();
                    Binding_185230fbde66433e8cfd51860201d293.Path = new Bridge.global.Windows.UI.Xaml.PropertyPath.ctor("Height");
                    var RelativeSource_5300912f623e49f8badf8914e5dc9986 = new Bridge.global.Windows.UI.Xaml.Data.RelativeSource.ctor();
                    RelativeSource_5300912f623e49f8badf8914e5dc9986.Mode = Bridge.global.Windows.UI.Xaml.Data.RelativeSourceMode.TemplatedParent;

                    Binding_185230fbde66433e8cfd51860201d293.RelativeSource = RelativeSource_5300912f623e49f8badf8914e5dc9986;


                    Binding_185230fbde66433e8cfd51860201d293.TemplateOwner = templateInstance_1e65d36b27fc47d6a46643364cc7d8ca;

                    var Binding_a84e0b0484454a94b41ce1f71b9d6ab3 = new Bridge.global.Windows.UI.Xaml.Data.Binding.ctor();
                    Binding_a84e0b0484454a94b41ce1f71b9d6ab3.Path = new Bridge.global.Windows.UI.Xaml.PropertyPath.ctor("Width");
                    var RelativeSource_635adca4fdb4468086f747ef3b989fe1 = new Bridge.global.Windows.UI.Xaml.Data.RelativeSource.ctor();
                    RelativeSource_635adca4fdb4468086f747ef3b989fe1.Mode = Bridge.global.Windows.UI.Xaml.Data.RelativeSourceMode.TemplatedParent;

                    Binding_a84e0b0484454a94b41ce1f71b9d6ab3.RelativeSource = RelativeSource_635adca4fdb4468086f747ef3b989fe1;


                    Binding_a84e0b0484454a94b41ce1f71b9d6ab3.TemplateOwner = templateInstance_1e65d36b27fc47d6a46643364cc7d8ca;


                    Grid_259d3e21cb824aa2bdea39bcc545610a.Children.add(Grid_fc6483f21ec149318260dbab3e348b5c);
                    Grid_259d3e21cb824aa2bdea39bcc545610a.Children.add(Border_b18d9efc9f794aa6a9ccfdecbdbafdbf);


                    Bridge.global.Windows.UI.Xaml.Data.BindingOperations.SetBinding(Grid_fc6483f21ec149318260dbab3e348b5c, Bridge.global.Windows.UI.Xaml.Controls.Panel.BackgroundProperty, Binding_80efdcb88ccc40578550711dac39de91);
                    Bridge.global.Windows.UI.Xaml.Data.BindingOperations.SetBinding(Grid_fc6483f21ec149318260dbab3e348b5c, Bridge.global.Windows.UI.Xaml.UIElement.OpacityProperty, Binding_4d6bc8165902407da6132a87bdc088b2);
                    Bridge.global.Windows.UI.Xaml.Data.BindingOperations.SetBinding(ContentControl_bbe5957d7ebf40e8a5259762cf86018d, Bridge.global.Windows.UI.Xaml.Controls.ContentControl.ContentProperty, Binding_bbb83b1a08b445aab16d72beee377126);
                    Bridge.global.Windows.UI.Xaml.Data.BindingOperations.SetBinding(ContentPresenter_dccc3538b14a4f10b33e74e7708e94e6, Bridge.global.Windows.UI.Xaml.Controls.ContentControl.ContentProperty, Binding_ae2b7420a4d740a697a0a05b76e511d1);
                    Bridge.global.Windows.UI.Xaml.Data.BindingOperations.SetBinding(ContentPresenter_dccc3538b14a4f10b33e74e7708e94e6, Bridge.global.Windows.UI.Xaml.Controls.ContentControl.ContentTemplateProperty, Binding_8de6bdd9cf1143b2a69febbc8f5f0f9c);
                    Bridge.global.Windows.UI.Xaml.Data.BindingOperations.SetBinding(ContentPresenter_dccc3538b14a4f10b33e74e7708e94e6, Bridge.global.Windows.UI.Xaml.FrameworkElement.HorizontalAlignmentProperty, Binding_46528764844a44369d1ff2496f6bb865);
                    Bridge.global.Windows.UI.Xaml.Data.BindingOperations.SetBinding(ContentPresenter_dccc3538b14a4f10b33e74e7708e94e6, Bridge.global.Windows.UI.Xaml.FrameworkElement.VerticalAlignmentProperty, Binding_021ac3797eea41cbbaeff10868652a99);
                    Bridge.global.Windows.UI.Xaml.Data.BindingOperations.SetBinding(Border_17172c187fa645559e54ed748675a4d9, Bridge.global.Windows.UI.Xaml.Controls.Border.BackgroundProperty, Binding_2de51e22f90a4e2398fe15d99c334724);
                    Bridge.global.Windows.UI.Xaml.Data.BindingOperations.SetBinding(Border_46b4592f744d4bf8842609a57855801e, Bridge.global.Windows.UI.Xaml.Controls.Border.BorderThicknessProperty, Binding_5c0dc2fb9207478f81bc8b7f26cec48b);
                    Bridge.global.Windows.UI.Xaml.Data.BindingOperations.SetBinding(Border_46b4592f744d4bf8842609a57855801e, Bridge.global.Windows.UI.Xaml.Controls.Border.BorderBrushProperty, Binding_ee2e70df603c4b7b88b7f1eb4ab1c02e);
                    Bridge.global.Windows.UI.Xaml.Data.BindingOperations.SetBinding(Border_b18d9efc9f794aa6a9ccfdecbdbafdbf, Bridge.global.Windows.UI.Xaml.FrameworkElement.HorizontalAlignmentProperty, Binding_2aee30da23cb4161bf96ec4dc3bfa5c0);
                    Bridge.global.Windows.UI.Xaml.Data.BindingOperations.SetBinding(Border_b18d9efc9f794aa6a9ccfdecbdbafdbf, Bridge.global.Windows.UI.Xaml.FrameworkElement.VerticalAlignmentProperty, Binding_c371756b6a57474ea8c8eff4cdd90e13);
                    Bridge.global.Windows.UI.Xaml.Data.BindingOperations.SetBinding(Border_b18d9efc9f794aa6a9ccfdecbdbafdbf, Bridge.global.Windows.UI.Xaml.FrameworkElement.HeightProperty, Binding_185230fbde66433e8cfd51860201d293);
                    Bridge.global.Windows.UI.Xaml.Data.BindingOperations.SetBinding(Border_b18d9efc9f794aa6a9ccfdecbdbafdbf, Bridge.global.Windows.UI.Xaml.FrameworkElement.WidthProperty, Binding_a84e0b0484454a94b41ce1f71b9d6ab3);

                    templateInstance_1e65d36b27fc47d6a46643364cc7d8ca.TemplateContent = Grid_259d3e21cb824aa2bdea39bcc545610a;
                    return templateInstance_1e65d36b27fc47d6a46643364cc7d8ca;
                });

                Setter_544f565953a34feaa0be15f0e96de89f.Value = ControlTemplate_47209f627dd648bd92dc64ec9ad5e158;


                Style_0853948daf174da58444bd0f43976de8.Setters.add(Setter_f145e206c3a74d3fb108360a1736af3e);
                Style_0853948daf174da58444bd0f43976de8.Setters.add(Setter_750a022c06d84ff0bedcc09d25c2b355);
                Style_0853948daf174da58444bd0f43976de8.Setters.add(Setter_526d1399d29746c0b4d4c10dbc094ac5);
                Style_0853948daf174da58444bd0f43976de8.Setters.add(Setter_62ca582ffdf24365a58db0d7bb6bfa61);
                Style_0853948daf174da58444bd0f43976de8.Setters.add(Setter_1a2aaf87a9fd46c18ddee47324f20ece);
                Style_0853948daf174da58444bd0f43976de8.Setters.add(Setter_88865afccfcb423ba1558aaa9b729da5);
                Style_0853948daf174da58444bd0f43976de8.Setters.add(Setter_e8eb86f0608b498eb60e437df0643531);
                Style_0853948daf174da58444bd0f43976de8.Setters.add(Setter_aed6f53901e849d29415a40834a387d4);
                Style_0853948daf174da58444bd0f43976de8.Setters.add(Setter_544f565953a34feaa0be15f0e96de89f);


                this.setItem(Bridge.global.Windows.UI.Xaml.Controls.ChildWindow, Style_0853948daf174da58444bd0f43976de8);






            }
        }
    });

    Bridge.define("___CSHTML5__Component__Defaultstyles__INTERNAL_Defaultcomboboxstyle__Xaml", {
        inherits: [Bridge.global.Windows.UI.Xaml.ResourceDictionary],
        fields: {
            _contentLoaded: false
        },
        ctors: {
            ctor: function () {
                this.$initialize();
                Bridge.global.Windows.UI.Xaml.ResourceDictionary.ctor.call(this);
                this.InitializeComponent();

                if (Bridge.is(this, Windows.UI.Xaml.UIElement)) {
                    Bridge.cast(this, Windows.UI.Xaml.UIElement).XamlSourcePath = "CSHTML5\\DefaultStyles\\INTERNAL_DefaultComboBoxStyle.xaml";
                }
            }
        },
        methods: {
            InitializeComponent: function () {
                var $t;
                if (this._contentLoaded) {
                    return;
                }
                this._contentLoaded = true;


                if (Bridge.is(this, Windows.UI.Xaml.UIElement)) {
                    Bridge.cast(this, Windows.UI.Xaml.UIElement).XamlSourcePath = "CSHTML5\\DefaultStyles\\INTERNAL_DefaultComboBoxStyle.xaml";
                }



                var Style_f75636f4dd4e454596d67cf59db9106f = new Bridge.global.Windows.UI.Xaml.Style.ctor();
                Style_f75636f4dd4e454596d67cf59db9106f.TargetType = Bridge.global.Windows.UI.Xaml.Controls.ComboBox;
                var Setter_38aa5f51ea2b4c9dbde9887bbe3cf8e8 = new Bridge.global.Windows.UI.Xaml.Setter.ctor();
                Setter_38aa5f51ea2b4c9dbde9887bbe3cf8e8.Property = Bridge.global.Windows.UI.Xaml.Controls.Control.BackgroundProperty;
                var LinearGradientBrush_84adbaad0a3e4af495b249ceec2b7651 = new Bridge.global.Windows.UI.Xaml.Media.LinearGradientBrush.ctor();
                LinearGradientBrush_84adbaad0a3e4af495b249ceec2b7651.StartPoint = new Bridge.global.Windows.Foundation.Point.$ctor1(0.5, 0);
                LinearGradientBrush_84adbaad0a3e4af495b249ceec2b7651.EndPoint = new Bridge.global.Windows.Foundation.Point.$ctor1(0.5, 1);
                var GradientStopCollection_b6837d2c618d410d93e75fe9861b7430 = new Bridge.global.Windows.UI.Xaml.Media.GradientStopCollection();
                var GradientStop_18c1e85c41b242b59f98ecc935d091b4 = new Bridge.global.Windows.UI.Xaml.Media.GradientStop();
                GradientStop_18c1e85c41b242b59f98ecc935d091b4.Color = ($t = new Bridge.global.Windows.UI.Color(), $t.A = 255, $t.R = 255, $t.G = 255, $t.B = 255, $t);
                GradientStop_18c1e85c41b242b59f98ecc935d091b4.Offset = 0.0;

                var GradientStop_b94fe6a4ca1e464f9c90abdf48e3423f = new Bridge.global.Windows.UI.Xaml.Media.GradientStop();
                GradientStop_b94fe6a4ca1e464f9c90abdf48e3423f.Color = ($t = new Bridge.global.Windows.UI.Color(), $t.A = 255, $t.R = 255, $t.G = 255, $t.B = 255, $t);
                GradientStop_b94fe6a4ca1e464f9c90abdf48e3423f.Offset = 0.3;

                var GradientStop_abcfbafc5ad24557879a5728a00659cb = new Bridge.global.Windows.UI.Xaml.Media.GradientStop();
                GradientStop_abcfbafc5ad24557879a5728a00659cb.Color = ($t = new Bridge.global.Windows.UI.Color(), $t.A = 255, $t.R = 208, $t.G = 208, $t.B = 208, $t);
                GradientStop_abcfbafc5ad24557879a5728a00659cb.Offset = 1.0;

                GradientStopCollection_b6837d2c618d410d93e75fe9861b7430.add(GradientStop_18c1e85c41b242b59f98ecc935d091b4);
                GradientStopCollection_b6837d2c618d410d93e75fe9861b7430.add(GradientStop_b94fe6a4ca1e464f9c90abdf48e3423f);
                GradientStopCollection_b6837d2c618d410d93e75fe9861b7430.add(GradientStop_abcfbafc5ad24557879a5728a00659cb);

                LinearGradientBrush_84adbaad0a3e4af495b249ceec2b7651.GradientStops = GradientStopCollection_b6837d2c618d410d93e75fe9861b7430;


                Setter_38aa5f51ea2b4c9dbde9887bbe3cf8e8.Value = LinearGradientBrush_84adbaad0a3e4af495b249ceec2b7651;


                var Setter_b4157067effc4be8af02495f0636a5c8 = new Bridge.global.Windows.UI.Xaml.Setter.ctor();
                Setter_b4157067effc4be8af02495f0636a5c8.Property = Bridge.global.Windows.UI.Xaml.Controls.Control.ForegroundProperty;
                Setter_b4157067effc4be8af02495f0636a5c8.Value = new Bridge.global.Windows.UI.Xaml.Media.SolidColorBrush.$ctor1(($t = new Bridge.global.Windows.UI.Color(), $t.A = 255, $t.R = 0, $t.G = 0, $t.B = 0, $t));

                var Setter_8232d746614d4953958c2d7e6bf2af0a = new Bridge.global.Windows.UI.Xaml.Setter.ctor();
                Setter_8232d746614d4953958c2d7e6bf2af0a.Property = Bridge.global.Windows.UI.Xaml.Controls.Control.BorderBrushProperty;
                Setter_8232d746614d4953958c2d7e6bf2af0a.Value = new Bridge.global.Windows.UI.Xaml.Media.SolidColorBrush.$ctor1(($t = new Bridge.global.Windows.UI.Color(), $t.A = 255, $t.R = 169, $t.G = 169, $t.B = 169, $t));

                var Setter_19f07d53ddce4340b8c78991aaf17bdd = new Bridge.global.Windows.UI.Xaml.Setter.ctor();
                Setter_19f07d53ddce4340b8c78991aaf17bdd.Property = Bridge.global.Windows.UI.Xaml.Controls.Control.BorderThicknessProperty;
                Setter_19f07d53ddce4340b8c78991aaf17bdd.Value = new Bridge.global.Windows.UI.Xaml.Thickness.$ctor1(1).$clone();

                var Setter_df8c1119824c4b36bf8742fe68ca910b = new Bridge.global.Windows.UI.Xaml.Setter.ctor();
                Setter_df8c1119824c4b36bf8742fe68ca910b.Property = Bridge.global.Windows.UI.Xaml.Controls.Control.PaddingProperty;
                Setter_df8c1119824c4b36bf8742fe68ca910b.Value = new Bridge.global.Windows.UI.Xaml.Thickness.$ctor2(6, 2, 30, 2).$clone();

                var Setter_77cea48e5fb0412a80158a52bbb3d960 = new Bridge.global.Windows.UI.Xaml.Setter.ctor();
                Setter_77cea48e5fb0412a80158a52bbb3d960.Property = Bridge.global.Windows.UI.Xaml.FrameworkElement.CursorProperty;
                Setter_77cea48e5fb0412a80158a52bbb3d960.Value = Bridge.global.System.Windows.Input.Cursors.Hand;

                var Setter_b6edd81e4c3b4ecb9ec963734cd350de = new Bridge.global.Windows.UI.Xaml.Setter.ctor();
                Setter_b6edd81e4c3b4ecb9ec963734cd350de.Property = Bridge.global.Windows.UI.Xaml.Controls.Control.TemplateProperty;
                var ControlTemplate_1a37421c1c5541ecacf1545c77bae938 = new Bridge.global.Windows.UI.Xaml.Controls.ControlTemplate();
                ControlTemplate_1a37421c1c5541ecacf1545c77bae938.TargetType = Bridge.global.Windows.UI.Xaml.Controls.ComboBox;
                ControlTemplate_1a37421c1c5541ecacf1545c77bae938.SetMethodToInstantiateFrameworkTemplate(Bridge.fn.bind(this, function (templateOwner_ControlTemplate_1a37421c1c5541ecacf1545c77bae938) {
                    var $t1;
                    var templateInstance_62a38a9ec1ae4cda9d77ecdcbbe19338 = new Bridge.global.Windows.UI.Xaml.TemplateInstance();
                    templateInstance_62a38a9ec1ae4cda9d77ecdcbbe19338.TemplateOwner = templateOwner_ControlTemplate_1a37421c1c5541ecacf1545c77bae938;
                    var VisualState_85859d6132f84fbf9c1d144d84e7ee9d = new Bridge.global.Windows.UI.Xaml.VisualState();
                    var VisualStateGroup_b59c210b11f543c798aab3467c2e88c9 = new Bridge.global.Windows.UI.Xaml.VisualStateGroup();
                    var Path_8043679a10e84b5a90f813c78067a0eb = new Bridge.global.Windows.UI.Xaml.Shapes.Path();
                    var ToggleButton_005d97cb1c95459ba0d5f715f61f5766 = new Bridge.global.Windows.UI.Xaml.Controls.Primitives.ToggleButton();
                    var ContentPresenter_8fc18ee5839546a5a169464651b80a75 = new Bridge.global.Windows.UI.Xaml.Controls.ContentPresenter();
                    var Border_7bd1f3118578417e8d8eaf776dfb297a = new Bridge.global.Windows.UI.Xaml.Controls.Border();
                    var ItemsPresenter_f2a0f563b28a406ea185625be92fc1f4 = new Bridge.global.Windows.UI.Xaml.Controls.ItemsPresenter();
                    var Popup_98d938a5487a4f4b86ba286971f81524 = new Bridge.global.Windows.UI.Xaml.Controls.Primitives.Popup();
                    var StackPanel_6ab77e1279df40a2b7d86f90bf3819ae = new Bridge.global.Windows.UI.Xaml.Controls.StackPanel();
                    templateOwner_ControlTemplate_1a37421c1c5541ecacf1545c77bae938.RegisterName("CommonStates", VisualStateGroup_b59c210b11f543c798aab3467c2e88c9);
                    VisualStateGroup_b59c210b11f543c798aab3467c2e88c9.Name = "CommonStates";
                    templateOwner_ControlTemplate_1a37421c1c5541ecacf1545c77bae938.RegisterName("Disabled", VisualState_85859d6132f84fbf9c1d144d84e7ee9d);
                    VisualState_85859d6132f84fbf9c1d144d84e7ee9d.Name = "Disabled";
                    var Storyboard_e08c0692c01c410887467b1c822e9c6d = new Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard();
                    var ObjectAnimationUsingKeyFrames_ed7d3b98e9944453ad3b7c99ad3dd7a5 = new Bridge.global.Windows.UI.Xaml.Media.Animation.ObjectAnimationUsingKeyFrames();
                    Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard.SetTargetName(ObjectAnimationUsingKeyFrames_ed7d3b98e9944453ad3b7c99ad3dd7a5, "OuterBorder");
                    var DiscreteObjectKeyFrame_132e19c1dc8347009e30ebdc73c2808f = new Bridge.global.Windows.UI.Xaml.Media.Animation.DiscreteObjectKeyFrame();
                    DiscreteObjectKeyFrame_132e19c1dc8347009e30ebdc73c2808f.KeyTime = Bridge.global.Windows.UI.Xaml.Media.Animation.KeyTime.FromTimeSpan(new Bridge.global.System.TimeSpan(System.Int64(0)));
                    DiscreteObjectKeyFrame_132e19c1dc8347009e30ebdc73c2808f.Value = "#FFD0D0D0";

                    ObjectAnimationUsingKeyFrames_ed7d3b98e9944453ad3b7c99ad3dd7a5.KeyFrames.add(DiscreteObjectKeyFrame_132e19c1dc8347009e30ebdc73c2808f);


                    var ObjectAnimationUsingKeyFrames_668e7b058e1948b79d98023885523c1e = new Bridge.global.Windows.UI.Xaml.Media.Animation.ObjectAnimationUsingKeyFrames();
                    Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard.SetTargetName(ObjectAnimationUsingKeyFrames_668e7b058e1948b79d98023885523c1e, "arrow");
                    var DiscreteObjectKeyFrame_a48e419423b84c359676fb317a5527c6 = new Bridge.global.Windows.UI.Xaml.Media.Animation.DiscreteObjectKeyFrame();
                    DiscreteObjectKeyFrame_a48e419423b84c359676fb317a5527c6.KeyTime = Bridge.global.Windows.UI.Xaml.Media.Animation.KeyTime.FromTimeSpan(new Bridge.global.System.TimeSpan(System.Int64(0)));
                    DiscreteObjectKeyFrame_a48e419423b84c359676fb317a5527c6.Value = "#FFD0D0D0";

                    ObjectAnimationUsingKeyFrames_668e7b058e1948b79d98023885523c1e.KeyFrames.add(DiscreteObjectKeyFrame_a48e419423b84c359676fb317a5527c6);


                    var ObjectAnimationUsingKeyFrames_82358246f9a54d81bed4cff34f877ce2 = new Bridge.global.Windows.UI.Xaml.Media.Animation.ObjectAnimationUsingKeyFrames();
                    Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard.SetTargetName(ObjectAnimationUsingKeyFrames_82358246f9a54d81bed4cff34f877ce2, "arrow");
                    var DiscreteObjectKeyFrame_1676f77d3b624311a1a163f29b80e893 = new Bridge.global.Windows.UI.Xaml.Media.Animation.DiscreteObjectKeyFrame();
                    DiscreteObjectKeyFrame_1676f77d3b624311a1a163f29b80e893.KeyTime = Bridge.global.Windows.UI.Xaml.Media.Animation.KeyTime.FromTimeSpan(new Bridge.global.System.TimeSpan(System.Int64(0)));
                    DiscreteObjectKeyFrame_1676f77d3b624311a1a163f29b80e893.Value = "#FFD0D0D0";

                    ObjectAnimationUsingKeyFrames_82358246f9a54d81bed4cff34f877ce2.KeyFrames.add(DiscreteObjectKeyFrame_1676f77d3b624311a1a163f29b80e893);


                    Storyboard_e08c0692c01c410887467b1c822e9c6d.Children.add(ObjectAnimationUsingKeyFrames_ed7d3b98e9944453ad3b7c99ad3dd7a5);
                    Storyboard_e08c0692c01c410887467b1c822e9c6d.Children.add(ObjectAnimationUsingKeyFrames_668e7b058e1948b79d98023885523c1e);
                    Storyboard_e08c0692c01c410887467b1c822e9c6d.Children.add(ObjectAnimationUsingKeyFrames_82358246f9a54d81bed4cff34f877ce2);


                    VisualState_85859d6132f84fbf9c1d144d84e7ee9d.Storyboard = Storyboard_e08c0692c01c410887467b1c822e9c6d;


                    System.Array.add(VisualStateGroup_b59c210b11f543c798aab3467c2e88c9.States, VisualState_85859d6132f84fbf9c1d144d84e7ee9d, Object);


                    templateOwner_ControlTemplate_1a37421c1c5541ecacf1545c77bae938.INTERNAL_GetVisualStateGroups().add(VisualStateGroup_b59c210b11f543c798aab3467c2e88c9);

                    templateOwner_ControlTemplate_1a37421c1c5541ecacf1545c77bae938.RegisterName("OuterBorder", Border_7bd1f3118578417e8d8eaf776dfb297a);
                    Border_7bd1f3118578417e8d8eaf776dfb297a.Name = "OuterBorder";
                    Border_7bd1f3118578417e8d8eaf776dfb297a.CornerRadius = new Bridge.global.Windows.UI.Xaml.CornerRadius.$ctor1(5);
                    var Grid_ed8d7fc27eb646eeafd64f0793359736 = new Bridge.global.Windows.UI.Xaml.Controls.Grid();
                    templateOwner_ControlTemplate_1a37421c1c5541ecacf1545c77bae938.RegisterName("DropDownToggle", ToggleButton_005d97cb1c95459ba0d5f715f61f5766);
                    ToggleButton_005d97cb1c95459ba0d5f715f61f5766.Name = "DropDownToggle";
                    ToggleButton_005d97cb1c95459ba0d5f715f61f5766.HorizontalAlignment = Bridge.global.Windows.UI.Xaml.HorizontalAlignment.Stretch;
                    ToggleButton_005d97cb1c95459ba0d5f715f61f5766.VerticalAlignment = Bridge.global.Windows.UI.Xaml.VerticalAlignment.Stretch;
                    ToggleButton_005d97cb1c95459ba0d5f715f61f5766.HorizontalContentAlignment = Bridge.global.Windows.UI.Xaml.HorizontalAlignment.Right;
                    ToggleButton_005d97cb1c95459ba0d5f715f61f5766.VerticalContentAlignment = Bridge.global.Windows.UI.Xaml.VerticalAlignment.Center;
                    ToggleButton_005d97cb1c95459ba0d5f715f61f5766.BorderThickness = new Bridge.global.Windows.UI.Xaml.Thickness.$ctor1(0);
                    ToggleButton_005d97cb1c95459ba0d5f715f61f5766.Background = new Bridge.global.Windows.UI.Xaml.Media.SolidColorBrush.$ctor1(($t1 = new Bridge.global.Windows.UI.Color(), $t1.A = 0, $t1.R = 255, $t1.G = 255, $t1.B = 255, $t1));
                    templateOwner_ControlTemplate_1a37421c1c5541ecacf1545c77bae938.RegisterName("arrow", Path_8043679a10e84b5a90f813c78067a0eb);
                    Path_8043679a10e84b5a90f813c78067a0eb.Name = "arrow";
                    Path_8043679a10e84b5a90f813c78067a0eb.Visibility = Bridge.global.Windows.UI.Xaml.Visibility.Visible;
                    Path_8043679a10e84b5a90f813c78067a0eb.Stroke = new Bridge.global.Windows.UI.Xaml.Media.SolidColorBrush.$ctor1(($t1 = new Bridge.global.Windows.UI.Color(), $t1.A = 255, $t1.R = 0, $t1.G = 0, $t1.B = 0, $t1));
                    Path_8043679a10e84b5a90f813c78067a0eb.Fill = new Bridge.global.Windows.UI.Xaml.Media.SolidColorBrush.$ctor1(($t1 = new Bridge.global.Windows.UI.Color(), $t1.A = 255, $t1.R = 0, $t1.G = 0, $t1.B = 0, $t1));
                    Path_8043679a10e84b5a90f813c78067a0eb.Width = 5.0;
                    Path_8043679a10e84b5a90f813c78067a0eb.Height = 3.0;
                    Path_8043679a10e84b5a90f813c78067a0eb.StrokeThickness = 1.0;
                    Path_8043679a10e84b5a90f813c78067a0eb.HorizontalAlignment = Bridge.global.Windows.UI.Xaml.HorizontalAlignment.Center;
                    Path_8043679a10e84b5a90f813c78067a0eb.VerticalAlignment = Bridge.global.Windows.UI.Xaml.VerticalAlignment.Center;
                    Path_8043679a10e84b5a90f813c78067a0eb.Stretch = Bridge.global.Windows.UI.Xaml.Media.Stretch.None;
                    Path_8043679a10e84b5a90f813c78067a0eb.Data = Bridge.cast(Bridge.global.DotNetForHtml5.Core.TypeFromStringConverters.ConvertFromInvariantString(Bridge.global.Windows.UI.Xaml.Media.Geometry, "M 0,0 L 2.5,3 L 5,0 z"), Windows.UI.Xaml.Media.Geometry);

                    ToggleButton_005d97cb1c95459ba0d5f715f61f5766.Content = Path_8043679a10e84b5a90f813c78067a0eb;


                    templateOwner_ControlTemplate_1a37421c1c5541ecacf1545c77bae938.RegisterName("ContentPresenter", ContentPresenter_8fc18ee5839546a5a169464651b80a75);
                    ContentPresenter_8fc18ee5839546a5a169464651b80a75.Name = "ContentPresenter";
                    ContentPresenter_8fc18ee5839546a5a169464651b80a75.IsHitTestVisible = false;
                    var Binding_10db90641b8c41c2be7c35c98ac2bd41 = new Bridge.global.Windows.UI.Xaml.Data.Binding.ctor();
                    Binding_10db90641b8c41c2be7c35c98ac2bd41.Path = new Bridge.global.Windows.UI.Xaml.PropertyPath.ctor("Padding");
                    var RelativeSource_61b0c83857ab4c52b3ec6080b1cffe9b = new Bridge.global.Windows.UI.Xaml.Data.RelativeSource.ctor();
                    RelativeSource_61b0c83857ab4c52b3ec6080b1cffe9b.Mode = Bridge.global.Windows.UI.Xaml.Data.RelativeSourceMode.TemplatedParent;

                    Binding_10db90641b8c41c2be7c35c98ac2bd41.RelativeSource = RelativeSource_61b0c83857ab4c52b3ec6080b1cffe9b;


                    Binding_10db90641b8c41c2be7c35c98ac2bd41.TemplateOwner = templateInstance_62a38a9ec1ae4cda9d77ecdcbbe19338;

                    var Binding_97d4d0df3ca64ba8bddabb0df4129ec1 = new Bridge.global.Windows.UI.Xaml.Data.Binding.ctor();
                    Binding_97d4d0df3ca64ba8bddabb0df4129ec1.Path = new Bridge.global.Windows.UI.Xaml.PropertyPath.ctor("Content");
                    var RelativeSource_76d58cf2893945d78e2f51bc1d82c136 = new Bridge.global.Windows.UI.Xaml.Data.RelativeSource.ctor();
                    RelativeSource_76d58cf2893945d78e2f51bc1d82c136.Mode = Bridge.global.Windows.UI.Xaml.Data.RelativeSourceMode.TemplatedParent;

                    Binding_97d4d0df3ca64ba8bddabb0df4129ec1.RelativeSource = RelativeSource_76d58cf2893945d78e2f51bc1d82c136;


                    Binding_97d4d0df3ca64ba8bddabb0df4129ec1.TemplateOwner = templateInstance_62a38a9ec1ae4cda9d77ecdcbbe19338;

                    var Binding_1624ac9b5c4544e8a97505c07494b4a7 = new Bridge.global.Windows.UI.Xaml.Data.Binding.ctor();
                    Binding_1624ac9b5c4544e8a97505c07494b4a7.Path = new Bridge.global.Windows.UI.Xaml.PropertyPath.ctor("ContentTemplate");
                    var RelativeSource_26f07e36569943068fdf5733e82920a7 = new Bridge.global.Windows.UI.Xaml.Data.RelativeSource.ctor();
                    RelativeSource_26f07e36569943068fdf5733e82920a7.Mode = Bridge.global.Windows.UI.Xaml.Data.RelativeSourceMode.TemplatedParent;

                    Binding_1624ac9b5c4544e8a97505c07494b4a7.RelativeSource = RelativeSource_26f07e36569943068fdf5733e82920a7;


                    Binding_1624ac9b5c4544e8a97505c07494b4a7.TemplateOwner = templateInstance_62a38a9ec1ae4cda9d77ecdcbbe19338;


                    var TextBlock_781d1af18d01481e8843fd349b125c30 = new Bridge.global.Windows.UI.Xaml.Controls.TextBlock();
                    TextBlock_781d1af18d01481e8843fd349b125c30.Text = " ";
                    TextBlock_781d1af18d01481e8843fd349b125c30.Opacity = 0.0;
                    TextBlock_781d1af18d01481e8843fd349b125c30.IsHitTestVisible = false;
                    TextBlock_781d1af18d01481e8843fd349b125c30.IsTabStop = false;
                    var Binding_5911a30085a24f48aba174af64f6710e = new Bridge.global.Windows.UI.Xaml.Data.Binding.ctor();
                    Binding_5911a30085a24f48aba174af64f6710e.Path = new Bridge.global.Windows.UI.Xaml.PropertyPath.ctor("Padding");
                    var RelativeSource_3db4e17b1f544ad9855438bf3b5984d3 = new Bridge.global.Windows.UI.Xaml.Data.RelativeSource.ctor();
                    RelativeSource_3db4e17b1f544ad9855438bf3b5984d3.Mode = Bridge.global.Windows.UI.Xaml.Data.RelativeSourceMode.TemplatedParent;

                    Binding_5911a30085a24f48aba174af64f6710e.RelativeSource = RelativeSource_3db4e17b1f544ad9855438bf3b5984d3;


                    Binding_5911a30085a24f48aba174af64f6710e.TemplateOwner = templateInstance_62a38a9ec1ae4cda9d77ecdcbbe19338;


                    Grid_ed8d7fc27eb646eeafd64f0793359736.Children.add(ToggleButton_005d97cb1c95459ba0d5f715f61f5766);
                    Grid_ed8d7fc27eb646eeafd64f0793359736.Children.add(ContentPresenter_8fc18ee5839546a5a169464651b80a75);
                    Grid_ed8d7fc27eb646eeafd64f0793359736.Children.add(TextBlock_781d1af18d01481e8843fd349b125c30);


                    Border_7bd1f3118578417e8d8eaf776dfb297a.Child = Grid_ed8d7fc27eb646eeafd64f0793359736;

                    var Binding_6f65ee9383494e29a2e6c7f29f1e9ba2 = new Bridge.global.Windows.UI.Xaml.Data.Binding.ctor();
                    Binding_6f65ee9383494e29a2e6c7f29f1e9ba2.Path = new Bridge.global.Windows.UI.Xaml.PropertyPath.ctor("Background");
                    var RelativeSource_7704cbab9cd84c84a5d76c7494095f12 = new Bridge.global.Windows.UI.Xaml.Data.RelativeSource.ctor();
                    RelativeSource_7704cbab9cd84c84a5d76c7494095f12.Mode = Bridge.global.Windows.UI.Xaml.Data.RelativeSourceMode.TemplatedParent;

                    Binding_6f65ee9383494e29a2e6c7f29f1e9ba2.RelativeSource = RelativeSource_7704cbab9cd84c84a5d76c7494095f12;


                    Binding_6f65ee9383494e29a2e6c7f29f1e9ba2.TemplateOwner = templateInstance_62a38a9ec1ae4cda9d77ecdcbbe19338;

                    var Binding_00c9e678deef42138fc55d9c5879412e = new Bridge.global.Windows.UI.Xaml.Data.Binding.ctor();
                    Binding_00c9e678deef42138fc55d9c5879412e.Path = new Bridge.global.Windows.UI.Xaml.PropertyPath.ctor("BorderBrush");
                    var RelativeSource_b3dec130fdcf47f2aba18d8319e12078 = new Bridge.global.Windows.UI.Xaml.Data.RelativeSource.ctor();
                    RelativeSource_b3dec130fdcf47f2aba18d8319e12078.Mode = Bridge.global.Windows.UI.Xaml.Data.RelativeSourceMode.TemplatedParent;

                    Binding_00c9e678deef42138fc55d9c5879412e.RelativeSource = RelativeSource_b3dec130fdcf47f2aba18d8319e12078;


                    Binding_00c9e678deef42138fc55d9c5879412e.TemplateOwner = templateInstance_62a38a9ec1ae4cda9d77ecdcbbe19338;

                    var Binding_05acfb8c1b2a4a94bbe79f11136935c8 = new Bridge.global.Windows.UI.Xaml.Data.Binding.ctor();
                    Binding_05acfb8c1b2a4a94bbe79f11136935c8.Path = new Bridge.global.Windows.UI.Xaml.PropertyPath.ctor("BorderThickness");
                    var RelativeSource_d9622dff25c94f93970a30126e6b161a = new Bridge.global.Windows.UI.Xaml.Data.RelativeSource.ctor();
                    RelativeSource_d9622dff25c94f93970a30126e6b161a.Mode = Bridge.global.Windows.UI.Xaml.Data.RelativeSourceMode.TemplatedParent;

                    Binding_05acfb8c1b2a4a94bbe79f11136935c8.RelativeSource = RelativeSource_d9622dff25c94f93970a30126e6b161a;


                    Binding_05acfb8c1b2a4a94bbe79f11136935c8.TemplateOwner = templateInstance_62a38a9ec1ae4cda9d77ecdcbbe19338;


                    templateOwner_ControlTemplate_1a37421c1c5541ecacf1545c77bae938.RegisterName("Popup", Popup_98d938a5487a4f4b86ba286971f81524);
                    Popup_98d938a5487a4f4b86ba286971f81524.Name = "Popup";
                    Popup_98d938a5487a4f4b86ba286971f81524.IsOpen = false;
                    var Border_1fead6ee46434eb8b4c8c4a11f9225c4 = new Bridge.global.Windows.UI.Xaml.Controls.Border();
                    Border_1fead6ee46434eb8b4c8c4a11f9225c4.Background = new Bridge.global.Windows.UI.Xaml.Media.SolidColorBrush.$ctor1(($t1 = new Bridge.global.Windows.UI.Color(), $t1.A = 255, $t1.R = 255, $t1.G = 255, $t1.B = 255, $t1));
                    Border_1fead6ee46434eb8b4c8c4a11f9225c4.BorderBrush = new Bridge.global.Windows.UI.Xaml.Media.SolidColorBrush.$ctor1(($t1 = new Bridge.global.Windows.UI.Color(), $t1.A = 255, $t1.R = 128, $t1.G = 128, $t1.B = 128, $t1));
                    Border_1fead6ee46434eb8b4c8c4a11f9225c4.BorderThickness = new Bridge.global.Windows.UI.Xaml.Thickness.$ctor1(1);
                    Border_1fead6ee46434eb8b4c8c4a11f9225c4.CornerRadius = new Bridge.global.Windows.UI.Xaml.CornerRadius.$ctor1(3);
                    Border_1fead6ee46434eb8b4c8c4a11f9225c4.Padding = new Bridge.global.Windows.UI.Xaml.Thickness.$ctor1(3);
                    var ScrollViewer_cc386dcfea5a463dba31f3efd102159c = new Bridge.global.Windows.UI.Xaml.Controls.ScrollViewer();
                    ScrollViewer_cc386dcfea5a463dba31f3efd102159c.VerticalScrollBarVisibility = Bridge.global.Windows.UI.Xaml.Controls.ScrollBarVisibility.Auto;
                    ScrollViewer_cc386dcfea5a463dba31f3efd102159c.HorizontalScrollBarVisibility = Bridge.global.Windows.UI.Xaml.Controls.ScrollBarVisibility.Disabled;
                    templateOwner_ControlTemplate_1a37421c1c5541ecacf1545c77bae938.RegisterName("ItemsHost", ItemsPresenter_f2a0f563b28a406ea185625be92fc1f4);
                    ItemsPresenter_f2a0f563b28a406ea185625be92fc1f4.Name = "ItemsHost";

                    ScrollViewer_cc386dcfea5a463dba31f3efd102159c.Content = ItemsPresenter_f2a0f563b28a406ea185625be92fc1f4;

                    var Binding_27edd6bfc61c4b948825817e46f4e69e = new Bridge.global.Windows.UI.Xaml.Data.Binding.ctor();
                    Binding_27edd6bfc61c4b948825817e46f4e69e.Path = new Bridge.global.Windows.UI.Xaml.PropertyPath.ctor("MaxDropDownHeight");
                    var RelativeSource_8f984f2ba2ab4a95a736161655d9a56c = new Bridge.global.Windows.UI.Xaml.Data.RelativeSource.ctor();
                    RelativeSource_8f984f2ba2ab4a95a736161655d9a56c.Mode = Bridge.global.Windows.UI.Xaml.Data.RelativeSourceMode.TemplatedParent;

                    Binding_27edd6bfc61c4b948825817e46f4e69e.RelativeSource = RelativeSource_8f984f2ba2ab4a95a736161655d9a56c;


                    Binding_27edd6bfc61c4b948825817e46f4e69e.TemplateOwner = templateInstance_62a38a9ec1ae4cda9d77ecdcbbe19338;


                    Border_1fead6ee46434eb8b4c8c4a11f9225c4.Child = ScrollViewer_cc386dcfea5a463dba31f3efd102159c;


                    Popup_98d938a5487a4f4b86ba286971f81524.Child = Border_1fead6ee46434eb8b4c8c4a11f9225c4;


                    StackPanel_6ab77e1279df40a2b7d86f90bf3819ae.Children.add(Border_7bd1f3118578417e8d8eaf776dfb297a);
                    StackPanel_6ab77e1279df40a2b7d86f90bf3819ae.Children.add(Popup_98d938a5487a4f4b86ba286971f81524);


                    Bridge.global.Windows.UI.Xaml.Data.BindingOperations.SetBinding(ContentPresenter_8fc18ee5839546a5a169464651b80a75, Bridge.global.Windows.UI.Xaml.FrameworkElement.MarginProperty, Binding_10db90641b8c41c2be7c35c98ac2bd41);
                    Bridge.global.Windows.UI.Xaml.Data.BindingOperations.SetBinding(ContentPresenter_8fc18ee5839546a5a169464651b80a75, Bridge.global.Windows.UI.Xaml.Controls.ContentControl.ContentProperty, Binding_97d4d0df3ca64ba8bddabb0df4129ec1);
                    Bridge.global.Windows.UI.Xaml.Data.BindingOperations.SetBinding(ContentPresenter_8fc18ee5839546a5a169464651b80a75, Bridge.global.Windows.UI.Xaml.Controls.ContentControl.ContentTemplateProperty, Binding_1624ac9b5c4544e8a97505c07494b4a7);
                    Bridge.global.Windows.UI.Xaml.Data.BindingOperations.SetBinding(TextBlock_781d1af18d01481e8843fd349b125c30, Bridge.global.Windows.UI.Xaml.FrameworkElement.MarginProperty, Binding_5911a30085a24f48aba174af64f6710e);
                    Bridge.global.Windows.UI.Xaml.Data.BindingOperations.SetBinding(Border_7bd1f3118578417e8d8eaf776dfb297a, Bridge.global.Windows.UI.Xaml.Controls.Border.BackgroundProperty, Binding_6f65ee9383494e29a2e6c7f29f1e9ba2);
                    Bridge.global.Windows.UI.Xaml.Data.BindingOperations.SetBinding(Border_7bd1f3118578417e8d8eaf776dfb297a, Bridge.global.Windows.UI.Xaml.Controls.Border.BorderBrushProperty, Binding_00c9e678deef42138fc55d9c5879412e);
                    Bridge.global.Windows.UI.Xaml.Data.BindingOperations.SetBinding(Border_7bd1f3118578417e8d8eaf776dfb297a, Bridge.global.Windows.UI.Xaml.Controls.Border.BorderThicknessProperty, Binding_05acfb8c1b2a4a94bbe79f11136935c8);
                    Bridge.global.Windows.UI.Xaml.Data.BindingOperations.SetBinding(ScrollViewer_cc386dcfea5a463dba31f3efd102159c, Bridge.global.Windows.UI.Xaml.FrameworkElement.MaxHeightProperty, Binding_27edd6bfc61c4b948825817e46f4e69e);

                    Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard.SetTargetProperty(ObjectAnimationUsingKeyFrames_ed7d3b98e9944453ad3b7c99ad3dd7a5, new Bridge.global.Windows.UI.Xaml.PropertyPath.$ctor1("BorderBrush", "BorderBrush", Bridge.fn.cacheBind(this, this.accessVisualStateProperty_2581cbad60654032aa56f186ab455972), Bridge.fn.cacheBind(this, this.setVisualStateProperty_2581cbad60654032aa56f186ab455972), Bridge.fn.cacheBind(this, this.setAnimationVisualStateProperty_2581cbad60654032aa56f186ab455972), Bridge.fn.cacheBind(this, this.setLocalVisualStateProperty_2581cbad60654032aa56f186ab455972), Bridge.fn.cacheBind(this, this.getVisualStateProperty_2581cbad60654032aa56f186ab455972)));
                    Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard.SetTarget(ObjectAnimationUsingKeyFrames_ed7d3b98e9944453ad3b7c99ad3dd7a5, Border_7bd1f3118578417e8d8eaf776dfb297a);


                    Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard.SetTargetProperty(ObjectAnimationUsingKeyFrames_668e7b058e1948b79d98023885523c1e, new Bridge.global.Windows.UI.Xaml.PropertyPath.$ctor1("Fill", "Fill", Bridge.fn.cacheBind(this, this.accessVisualStateProperty_f8a65fad71f945568ac05a5110b0f80e), Bridge.fn.cacheBind(this, this.setVisualStateProperty_f8a65fad71f945568ac05a5110b0f80e), Bridge.fn.cacheBind(this, this.setAnimationVisualStateProperty_f8a65fad71f945568ac05a5110b0f80e), Bridge.fn.cacheBind(this, this.setLocalVisualStateProperty_f8a65fad71f945568ac05a5110b0f80e), Bridge.fn.cacheBind(this, this.getVisualStateProperty_f8a65fad71f945568ac05a5110b0f80e)));
                    Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard.SetTarget(ObjectAnimationUsingKeyFrames_668e7b058e1948b79d98023885523c1e, Path_8043679a10e84b5a90f813c78067a0eb);


                    Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard.SetTargetProperty(ObjectAnimationUsingKeyFrames_82358246f9a54d81bed4cff34f877ce2, new Bridge.global.Windows.UI.Xaml.PropertyPath.$ctor1("Stroke", "Stroke", Bridge.fn.cacheBind(this, this.accessVisualStateProperty_86a4af463674401d8d028ce1b7024c7c), Bridge.fn.cacheBind(this, this.setVisualStateProperty_86a4af463674401d8d028ce1b7024c7c), Bridge.fn.cacheBind(this, this.setAnimationVisualStateProperty_86a4af463674401d8d028ce1b7024c7c), Bridge.fn.cacheBind(this, this.setLocalVisualStateProperty_86a4af463674401d8d028ce1b7024c7c), Bridge.fn.cacheBind(this, this.getVisualStateProperty_86a4af463674401d8d028ce1b7024c7c)));
                    Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard.SetTarget(ObjectAnimationUsingKeyFrames_82358246f9a54d81bed4cff34f877ce2, Path_8043679a10e84b5a90f813c78067a0eb);

                    templateInstance_62a38a9ec1ae4cda9d77ecdcbbe19338.TemplateContent = StackPanel_6ab77e1279df40a2b7d86f90bf3819ae;
                    return templateInstance_62a38a9ec1ae4cda9d77ecdcbbe19338;
                }));

                Setter_b6edd81e4c3b4ecb9ec963734cd350de.Value = ControlTemplate_1a37421c1c5541ecacf1545c77bae938;


                Style_f75636f4dd4e454596d67cf59db9106f.Setters.add(Setter_38aa5f51ea2b4c9dbde9887bbe3cf8e8);
                Style_f75636f4dd4e454596d67cf59db9106f.Setters.add(Setter_b4157067effc4be8af02495f0636a5c8);
                Style_f75636f4dd4e454596d67cf59db9106f.Setters.add(Setter_8232d746614d4953958c2d7e6bf2af0a);
                Style_f75636f4dd4e454596d67cf59db9106f.Setters.add(Setter_19f07d53ddce4340b8c78991aaf17bdd);
                Style_f75636f4dd4e454596d67cf59db9106f.Setters.add(Setter_df8c1119824c4b36bf8742fe68ca910b);
                Style_f75636f4dd4e454596d67cf59db9106f.Setters.add(Setter_77cea48e5fb0412a80158a52bbb3d960);
                Style_f75636f4dd4e454596d67cf59db9106f.Setters.add(Setter_b6edd81e4c3b4ecb9ec963734cd350de);


                this.setItem(Bridge.global.Windows.UI.Xaml.Controls.ComboBox, Style_f75636f4dd4e454596d67cf59db9106f);






            },
            accessVisualStateProperty_2581cbad60654032aa56f186ab455972: function (rootTargetObjectInstance) {
                return new (Bridge.GeneratorEnumerable$1(System.Tuple$3(Windows.UI.Xaml.DependencyObject,Windows.UI.Xaml.DependencyProperty,System.Nullable$1(System.Int32))))(Bridge.fn.bind(this, function (rootTargetObjectInstance) {
                    var $step = 0,
                        $jumpFromFinally,
                        $returnValue,
                        $async_e;

                    var $enumerator = new (Bridge.GeneratorEnumerator$1(System.Tuple$3(Windows.UI.Xaml.DependencyObject,Windows.UI.Xaml.DependencyProperty,System.Nullable$1(System.Int32))))(Bridge.fn.bind(this, function () {
                        try {
                            for (;;) {
                                switch ($step) {
                                    case 0: {
                                        return false;
                                    }
                                    default: {
                                        return false;
                                    }
                                }
                            }
                        } catch($async_e1) {
                            $async_e = System.Exception.create($async_e1);
                            throw $async_e;
                        }
                    }));
                    return $enumerator;
                }, arguments));
            },
            setVisualStateProperty_2581cbad60654032aa56f186ab455972: function (finalTargetInstance, value) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "BorderBrush").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "BorderBrushProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                (finalTargetInstance).SetVisualStateValue(property, value);
            },
            setAnimationVisualStateProperty_2581cbad60654032aa56f186ab455972: function (finalTargetInstance, value) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "BorderBrush").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "BorderBrushProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                (finalTargetInstance).SetAnimationValue(property, value);
            },
            setLocalVisualStateProperty_2581cbad60654032aa56f186ab455972: function (finalTargetInstance, value) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "BorderBrush").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "BorderBrushProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                (finalTargetInstance).SetLocalValue(property, value);
            },
            getVisualStateProperty_2581cbad60654032aa56f186ab455972: function (finalTargetInstance) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "BorderBrush").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "BorderBrushProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                return finalTargetInstance.GetVisualStateValue(property);
            },
            accessVisualStateProperty_f8a65fad71f945568ac05a5110b0f80e: function (rootTargetObjectInstance) {
                return new (Bridge.GeneratorEnumerable$1(System.Tuple$3(Windows.UI.Xaml.DependencyObject,Windows.UI.Xaml.DependencyProperty,System.Nullable$1(System.Int32))))(Bridge.fn.bind(this, function (rootTargetObjectInstance) {
                    var $step = 0,
                        $jumpFromFinally,
                        $returnValue,
                        $async_e;

                    var $enumerator = new (Bridge.GeneratorEnumerator$1(System.Tuple$3(Windows.UI.Xaml.DependencyObject,Windows.UI.Xaml.DependencyProperty,System.Nullable$1(System.Int32))))(Bridge.fn.bind(this, function () {
                        try {
                            for (;;) {
                                switch ($step) {
                                    case 0: {
                                        return false;
                                    }
                                    default: {
                                        return false;
                                    }
                                }
                            }
                        } catch($async_e1) {
                            $async_e = System.Exception.create($async_e1);
                            throw $async_e;
                        }
                    }));
                    return $enumerator;
                }, arguments));
            },
            setVisualStateProperty_f8a65fad71f945568ac05a5110b0f80e: function (finalTargetInstance, value) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Fill").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "FillProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                (finalTargetInstance).SetVisualStateValue(property, value);
            },
            setAnimationVisualStateProperty_f8a65fad71f945568ac05a5110b0f80e: function (finalTargetInstance, value) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Fill").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "FillProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                (finalTargetInstance).SetAnimationValue(property, value);
            },
            setLocalVisualStateProperty_f8a65fad71f945568ac05a5110b0f80e: function (finalTargetInstance, value) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Fill").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "FillProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                (finalTargetInstance).SetLocalValue(property, value);
            },
            getVisualStateProperty_f8a65fad71f945568ac05a5110b0f80e: function (finalTargetInstance) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Fill").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "FillProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                return finalTargetInstance.GetVisualStateValue(property);
            },
            accessVisualStateProperty_86a4af463674401d8d028ce1b7024c7c: function (rootTargetObjectInstance) {
                return new (Bridge.GeneratorEnumerable$1(System.Tuple$3(Windows.UI.Xaml.DependencyObject,Windows.UI.Xaml.DependencyProperty,System.Nullable$1(System.Int32))))(Bridge.fn.bind(this, function (rootTargetObjectInstance) {
                    var $step = 0,
                        $jumpFromFinally,
                        $returnValue,
                        $async_e;

                    var $enumerator = new (Bridge.GeneratorEnumerator$1(System.Tuple$3(Windows.UI.Xaml.DependencyObject,Windows.UI.Xaml.DependencyProperty,System.Nullable$1(System.Int32))))(Bridge.fn.bind(this, function () {
                        try {
                            for (;;) {
                                switch ($step) {
                                    case 0: {
                                        return false;
                                    }
                                    default: {
                                        return false;
                                    }
                                }
                            }
                        } catch($async_e1) {
                            $async_e = System.Exception.create($async_e1);
                            throw $async_e;
                        }
                    }));
                    return $enumerator;
                }, arguments));
            },
            setVisualStateProperty_86a4af463674401d8d028ce1b7024c7c: function (finalTargetInstance, value) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Stroke").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "StrokeProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                (finalTargetInstance).SetVisualStateValue(property, value);
            },
            setAnimationVisualStateProperty_86a4af463674401d8d028ce1b7024c7c: function (finalTargetInstance, value) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Stroke").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "StrokeProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                (finalTargetInstance).SetAnimationValue(property, value);
            },
            setLocalVisualStateProperty_86a4af463674401d8d028ce1b7024c7c: function (finalTargetInstance, value) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Stroke").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "StrokeProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                (finalTargetInstance).SetLocalValue(property, value);
            },
            getVisualStateProperty_86a4af463674401d8d028ce1b7024c7c: function (finalTargetInstance) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Stroke").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "StrokeProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                return finalTargetInstance.GetVisualStateValue(property);
            }
        }
    });

    Bridge.define("___CSHTML5__Component__Defaultstyles__INTERNAL_Defaultdatapagerstyle__Xaml", {
        inherits: [Bridge.global.Windows.UI.Xaml.ResourceDictionary],
        fields: {
            _contentLoaded: false
        },
        ctors: {
            ctor: function () {
                this.$initialize();
                Bridge.global.Windows.UI.Xaml.ResourceDictionary.ctor.call(this);
                this.InitializeComponent();

                if (Bridge.is(this, Windows.UI.Xaml.UIElement)) {
                    Bridge.cast(this, Windows.UI.Xaml.UIElement).XamlSourcePath = "CSHTML5\\DefaultStyles\\INTERNAL_DefaultDataPagerStyle.xaml";
                }
            }
        },
        methods: {
            InitializeComponent: function () {
                var $t;
                if (this._contentLoaded) {
                    return;
                }
                this._contentLoaded = true;


                if (Bridge.is(this, Windows.UI.Xaml.UIElement)) {
                    Bridge.cast(this, Windows.UI.Xaml.UIElement).XamlSourcePath = "CSHTML5\\DefaultStyles\\INTERNAL_DefaultDataPagerStyle.xaml";
                }



                var Style_25cdadba73404c8fb24aeb2c26876f7a = new Bridge.global.Windows.UI.Xaml.Style.ctor();
                Style_25cdadba73404c8fb24aeb2c26876f7a.TargetType = Bridge.global.Windows.UI.Xaml.Controls.DataPager;
                var Setter_a87c494e48b446d191bc0f8136798c79 = new Bridge.global.Windows.UI.Xaml.Setter.ctor();
                Setter_a87c494e48b446d191bc0f8136798c79.Property = Bridge.global.Windows.UI.Xaml.Controls.Control.BackgroundProperty;
                Setter_a87c494e48b446d191bc0f8136798c79.Value = new Bridge.global.Windows.UI.Xaml.Media.SolidColorBrush.$ctor1(($t = new Bridge.global.Windows.UI.Color(), $t.A = 255, $t.R = 242, $t.G = 243, $t.B = 244, $t));

                var Setter_1fa275885388465a98377d2632ab76b2 = new Bridge.global.Windows.UI.Xaml.Setter.ctor();
                Setter_1fa275885388465a98377d2632ab76b2.Property = Bridge.global.Windows.UI.Xaml.Controls.Control.BorderBrushProperty;
                Setter_1fa275885388465a98377d2632ab76b2.Value = new Bridge.global.Windows.UI.Xaml.Media.SolidColorBrush.$ctor1(($t = new Bridge.global.Windows.UI.Color(), $t.A = 255, $t.R = 128, $t.G = 128, $t.B = 128, $t));

                var Setter_076f159d43f54685a055f2a8d8c1735f = new Bridge.global.Windows.UI.Xaml.Setter.ctor();
                Setter_076f159d43f54685a055f2a8d8c1735f.Property = Bridge.global.Windows.UI.Xaml.Controls.Control.BorderThicknessProperty;
                Setter_076f159d43f54685a055f2a8d8c1735f.Value = new Bridge.global.Windows.UI.Xaml.Thickness.$ctor1(1).$clone();

                var Setter_cbeeb17c02374bc48b5bf783eb71db18 = new Bridge.global.Windows.UI.Xaml.Setter.ctor();
                Setter_cbeeb17c02374bc48b5bf783eb71db18.Property = Bridge.global.Windows.UI.Xaml.Controls.Control.HorizontalContentAlignmentProperty;
                Setter_cbeeb17c02374bc48b5bf783eb71db18.Value = Bridge.box(Bridge.global.Windows.UI.Xaml.HorizontalAlignment.Right, Windows.UI.Xaml.HorizontalAlignment, System.Enum.toStringFn(Windows.UI.Xaml.HorizontalAlignment));

                var Setter_b95adf35db3948e0a62da7b6da83f060 = new Bridge.global.Windows.UI.Xaml.Setter.ctor();
                Setter_b95adf35db3948e0a62da7b6da83f060.Property = Bridge.global.Windows.UI.Xaml.Controls.DataPager.NumericButtonStyleProperty;
                var Style_ec3987d1679d46aa87884a9777809240 = new Bridge.global.Windows.UI.Xaml.Style.ctor();
                Style_ec3987d1679d46aa87884a9777809240.TargetType = Bridge.global.Windows.UI.Xaml.Controls.Primitives.ToggleButton;
                var Setter_86c5f28dc82f45eea5cdcc39c99f6709 = new Bridge.global.Windows.UI.Xaml.Setter.ctor();
                Setter_86c5f28dc82f45eea5cdcc39c99f6709.Property = Bridge.global.Windows.UI.Xaml.FrameworkElement.MinHeightProperty;
                Setter_86c5f28dc82f45eea5cdcc39c99f6709.Value = Bridge.box(20.0, System.Double, System.Double.format, System.Double.getHashCode);

                var Setter_13d4434ba0b948839c372982e62bdbcb = new Bridge.global.Windows.UI.Xaml.Setter.ctor();
                Setter_13d4434ba0b948839c372982e62bdbcb.Property = Bridge.global.Windows.UI.Xaml.FrameworkElement.MinWidthProperty;
                Setter_13d4434ba0b948839c372982e62bdbcb.Value = Bridge.box(20.0, System.Double, System.Double.format, System.Double.getHashCode);

                var Setter_c58ca3e32284418793e7fc58ef457f38 = new Bridge.global.Windows.UI.Xaml.Setter.ctor();
                Setter_c58ca3e32284418793e7fc58ef457f38.Property = Bridge.global.Windows.UI.Xaml.FrameworkElement.HorizontalAlignmentProperty;
                Setter_c58ca3e32284418793e7fc58ef457f38.Value = Bridge.box(Bridge.global.Windows.UI.Xaml.HorizontalAlignment.Right, Windows.UI.Xaml.HorizontalAlignment, System.Enum.toStringFn(Windows.UI.Xaml.HorizontalAlignment));

                var Setter_6d3f805a22004b3383c4bac1edbda1d8 = new Bridge.global.Windows.UI.Xaml.Setter.ctor();
                Setter_6d3f805a22004b3383c4bac1edbda1d8.Property = Bridge.global.Windows.UI.Xaml.FrameworkElement.VerticalAlignmentProperty;
                Setter_6d3f805a22004b3383c4bac1edbda1d8.Value = Bridge.box(Bridge.global.Windows.UI.Xaml.VerticalAlignment.Center, Windows.UI.Xaml.VerticalAlignment, System.Enum.toStringFn(Windows.UI.Xaml.VerticalAlignment));

                var Setter_9dc0fc71d3954575aa97f6229c3f165c = new Bridge.global.Windows.UI.Xaml.Setter.ctor();
                Setter_9dc0fc71d3954575aa97f6229c3f165c.Property = Bridge.global.Windows.UI.Xaml.Controls.Control.BackgroundProperty;
                Setter_9dc0fc71d3954575aa97f6229c3f165c.Value = new Bridge.global.Windows.UI.Xaml.Media.SolidColorBrush.$ctor1(($t = new Bridge.global.Windows.UI.Color(), $t.A = 0, $t.R = 0, $t.G = 0, $t.B = 0, $t));

                var Setter_8f9de49fd2624b88a9add9742e2cf2a1 = new Bridge.global.Windows.UI.Xaml.Setter.ctor();
                Setter_8f9de49fd2624b88a9add9742e2cf2a1.Property = Bridge.global.Windows.UI.Xaml.Controls.Control.BorderThicknessProperty;
                Setter_8f9de49fd2624b88a9add9742e2cf2a1.Value = new Bridge.global.Windows.UI.Xaml.Thickness.$ctor1(1).$clone();

                var Setter_1db26c81dff6444eaa9f156f64951466 = new Bridge.global.Windows.UI.Xaml.Setter.ctor();
                Setter_1db26c81dff6444eaa9f156f64951466.Property = Bridge.global.Windows.UI.Xaml.Controls.Control.PaddingProperty;
                Setter_1db26c81dff6444eaa9f156f64951466.Value = new Bridge.global.Windows.UI.Xaml.Thickness.$ctor1(1).$clone();

                var Setter_0240cac5e087458a9bbde5904a5ad6c0 = new Bridge.global.Windows.UI.Xaml.Setter.ctor();
                Setter_0240cac5e087458a9bbde5904a5ad6c0.Property = Bridge.global.Windows.UI.Xaml.Controls.Control.TemplateProperty;
                var ControlTemplate_b38967e81ef84765a2ad46f8d4a59bb0 = new Bridge.global.Windows.UI.Xaml.Controls.ControlTemplate();
                ControlTemplate_b38967e81ef84765a2ad46f8d4a59bb0.TargetType = Bridge.global.Windows.UI.Xaml.Controls.Primitives.ToggleButton;
                ControlTemplate_b38967e81ef84765a2ad46f8d4a59bb0.SetMethodToInstantiateFrameworkTemplate(Bridge.fn.bind(this, function (templateOwner_ControlTemplate_b38967e81ef84765a2ad46f8d4a59bb0) {
                    var $t1;
                    var templateInstance_8c66612326ba41e787ad16a0b6378454 = new Bridge.global.Windows.UI.Xaml.TemplateInstance();
                    templateInstance_8c66612326ba41e787ad16a0b6378454.TemplateOwner = templateOwner_ControlTemplate_b38967e81ef84765a2ad46f8d4a59bb0;
                    var VisualState_6ff7ebc620744d35b860458e4b90e14a = new Bridge.global.Windows.UI.Xaml.VisualState();
                    var VisualState_cfc12c2ae48347cd9ab00391a297ab6d = new Bridge.global.Windows.UI.Xaml.VisualState();
                    var VisualState_f3e0d8b7eed84e9b98cbfb4815718716 = new Bridge.global.Windows.UI.Xaml.VisualState();
                    var VisualState_3f9fc818aae3452ca2910d9b09a04aca = new Bridge.global.Windows.UI.Xaml.VisualState();
                    var VisualStateGroup_0d8b6bf1b3ec40d686b0ef1eba13448f = new Bridge.global.Windows.UI.Xaml.VisualStateGroup();
                    var VisualState_5b42c3886d5340d398d28c3d693c5ad0 = new Bridge.global.Windows.UI.Xaml.VisualState();
                    var VisualState_8369440d623c431d95120d6344ff9d8e = new Bridge.global.Windows.UI.Xaml.VisualState();
                    var VisualStateGroup_985cb445890049bd8cd03e1f0ea4aa47 = new Bridge.global.Windows.UI.Xaml.VisualStateGroup();
                    var VisualState_ee9296852392448b96420200c36af877 = new Bridge.global.Windows.UI.Xaml.VisualState();
                    var VisualState_ca57051dfcd442be85fa04922bcb86bf = new Bridge.global.Windows.UI.Xaml.VisualState();
                    var VisualStateGroup_9fe9f9e6560e446880673d0352e8fa64 = new Bridge.global.Windows.UI.Xaml.VisualStateGroup();
                    var Border_6bed044959504ed9a8c7a22c6f47b5cf = new Bridge.global.Windows.UI.Xaml.Controls.Border();
                    var ContentPresenter_1da2347a1e974f6ba82278fc6173d324 = new Bridge.global.Windows.UI.Xaml.Controls.ContentPresenter();
                    var Border_dd25e2f9bd8649b38f33d7ee44e18f38 = new Bridge.global.Windows.UI.Xaml.Controls.Border();
                    var Border_55d676e8ad224a4eb2b898c941e967b5 = new Bridge.global.Windows.UI.Xaml.Controls.Border();
                    var Border_b6c574fc2120413a80a4d4be109014c9 = new Bridge.global.Windows.UI.Xaml.Controls.Border();
                    var Grid_4af9b314fab64a0a89dbc4ede6491914 = new Bridge.global.Windows.UI.Xaml.Controls.Grid();
                    templateOwner_ControlTemplate_b38967e81ef84765a2ad46f8d4a59bb0.RegisterName("CommonStates", VisualStateGroup_0d8b6bf1b3ec40d686b0ef1eba13448f);
                    VisualStateGroup_0d8b6bf1b3ec40d686b0ef1eba13448f.Name = "CommonStates";
                    templateOwner_ControlTemplate_b38967e81ef84765a2ad46f8d4a59bb0.RegisterName("Normal", VisualState_6ff7ebc620744d35b860458e4b90e14a);
                    VisualState_6ff7ebc620744d35b860458e4b90e14a.Name = "Normal";

                    templateOwner_ControlTemplate_b38967e81ef84765a2ad46f8d4a59bb0.RegisterName("MouseOver", VisualState_cfc12c2ae48347cd9ab00391a297ab6d);
                    VisualState_cfc12c2ae48347cd9ab00391a297ab6d.Name = "MouseOver";

                    templateOwner_ControlTemplate_b38967e81ef84765a2ad46f8d4a59bb0.RegisterName("Pressed", VisualState_f3e0d8b7eed84e9b98cbfb4815718716);
                    VisualState_f3e0d8b7eed84e9b98cbfb4815718716.Name = "Pressed";

                    templateOwner_ControlTemplate_b38967e81ef84765a2ad46f8d4a59bb0.RegisterName("Disabled", VisualState_3f9fc818aae3452ca2910d9b09a04aca);
                    VisualState_3f9fc818aae3452ca2910d9b09a04aca.Name = "Disabled";
                    var Storyboard_0bcb73fbe7694b16bacae03a2b6ac8ff = new Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard();
                    var ObjectAnimationUsingKeyFrames_29fe99d3ff2d45159bd756f9ac08633d = new Bridge.global.Windows.UI.Xaml.Media.Animation.ObjectAnimationUsingKeyFrames();
                    Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard.SetTargetName(ObjectAnimationUsingKeyFrames_29fe99d3ff2d45159bd756f9ac08633d, "contentPresenter");
                    var DiscreteObjectKeyFrame_5988096e09e047ceb493eea0481ae498 = new Bridge.global.Windows.UI.Xaml.Media.Animation.DiscreteObjectKeyFrame();
                    DiscreteObjectKeyFrame_5988096e09e047ceb493eea0481ae498.KeyTime = Bridge.global.Windows.UI.Xaml.Media.Animation.KeyTime.FromTimeSpan(new Bridge.global.System.TimeSpan(System.Int64(0)));
                    DiscreteObjectKeyFrame_5988096e09e047ceb493eea0481ae498.Value = "0.5";

                    ObjectAnimationUsingKeyFrames_29fe99d3ff2d45159bd756f9ac08633d.KeyFrames.add(DiscreteObjectKeyFrame_5988096e09e047ceb493eea0481ae498);


                    Storyboard_0bcb73fbe7694b16bacae03a2b6ac8ff.Children.add(ObjectAnimationUsingKeyFrames_29fe99d3ff2d45159bd756f9ac08633d);


                    VisualState_3f9fc818aae3452ca2910d9b09a04aca.Storyboard = Storyboard_0bcb73fbe7694b16bacae03a2b6ac8ff;


                    System.Array.add(VisualStateGroup_0d8b6bf1b3ec40d686b0ef1eba13448f.States, VisualState_6ff7ebc620744d35b860458e4b90e14a, Object);
                    System.Array.add(VisualStateGroup_0d8b6bf1b3ec40d686b0ef1eba13448f.States, VisualState_cfc12c2ae48347cd9ab00391a297ab6d, Object);
                    System.Array.add(VisualStateGroup_0d8b6bf1b3ec40d686b0ef1eba13448f.States, VisualState_f3e0d8b7eed84e9b98cbfb4815718716, Object);
                    System.Array.add(VisualStateGroup_0d8b6bf1b3ec40d686b0ef1eba13448f.States, VisualState_3f9fc818aae3452ca2910d9b09a04aca, Object);


                    templateOwner_ControlTemplate_b38967e81ef84765a2ad46f8d4a59bb0.RegisterName("CheckStates", VisualStateGroup_985cb445890049bd8cd03e1f0ea4aa47);
                    VisualStateGroup_985cb445890049bd8cd03e1f0ea4aa47.Name = "CheckStates";
                    templateOwner_ControlTemplate_b38967e81ef84765a2ad46f8d4a59bb0.RegisterName("Checked", VisualState_5b42c3886d5340d398d28c3d693c5ad0);
                    VisualState_5b42c3886d5340d398d28c3d693c5ad0.Name = "Checked";
                    var Storyboard_d5ec92047eec4cc0af416673dab85f4f = new Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard();
                    var ObjectAnimationUsingKeyFrames_5882435b970b4536b2b4ecb680ef8074 = new Bridge.global.Windows.UI.Xaml.Media.Animation.ObjectAnimationUsingKeyFrames();
                    Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard.SetTargetName(ObjectAnimationUsingKeyFrames_5882435b970b4536b2b4ecb680ef8074, "CheckedStateOuterBorder");
                    var DiscreteObjectKeyFrame_70ade21dba694a73b1f0733530782349 = new Bridge.global.Windows.UI.Xaml.Media.Animation.DiscreteObjectKeyFrame();
                    DiscreteObjectKeyFrame_70ade21dba694a73b1f0733530782349.KeyTime = Bridge.global.Windows.UI.Xaml.Media.Animation.KeyTime.FromTimeSpan(new Bridge.global.System.TimeSpan(System.Int64(0)));
                    DiscreteObjectKeyFrame_70ade21dba694a73b1f0733530782349.Value = "1";

                    ObjectAnimationUsingKeyFrames_5882435b970b4536b2b4ecb680ef8074.KeyFrames.add(DiscreteObjectKeyFrame_70ade21dba694a73b1f0733530782349);


                    Storyboard_d5ec92047eec4cc0af416673dab85f4f.Children.add(ObjectAnimationUsingKeyFrames_5882435b970b4536b2b4ecb680ef8074);


                    VisualState_5b42c3886d5340d398d28c3d693c5ad0.Storyboard = Storyboard_d5ec92047eec4cc0af416673dab85f4f;


                    templateOwner_ControlTemplate_b38967e81ef84765a2ad46f8d4a59bb0.RegisterName("Unchecked", VisualState_8369440d623c431d95120d6344ff9d8e);
                    VisualState_8369440d623c431d95120d6344ff9d8e.Name = "Unchecked";

                    System.Array.add(VisualStateGroup_985cb445890049bd8cd03e1f0ea4aa47.States, VisualState_5b42c3886d5340d398d28c3d693c5ad0, Object);
                    System.Array.add(VisualStateGroup_985cb445890049bd8cd03e1f0ea4aa47.States, VisualState_8369440d623c431d95120d6344ff9d8e, Object);


                    templateOwner_ControlTemplate_b38967e81ef84765a2ad46f8d4a59bb0.RegisterName("FocusStates", VisualStateGroup_9fe9f9e6560e446880673d0352e8fa64);
                    VisualStateGroup_9fe9f9e6560e446880673d0352e8fa64.Name = "FocusStates";
                    templateOwner_ControlTemplate_b38967e81ef84765a2ad46f8d4a59bb0.RegisterName("Focused", VisualState_ee9296852392448b96420200c36af877);
                    VisualState_ee9296852392448b96420200c36af877.Name = "Focused";
                    var Storyboard_e6ea90d212fd4d44afa8242d5264b63b = new Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard();
                    var ObjectAnimationUsingKeyFrames_8f8318e65156461e8f8d13c098a2c800 = new Bridge.global.Windows.UI.Xaml.Media.Animation.ObjectAnimationUsingKeyFrames();
                    Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard.SetTargetName(ObjectAnimationUsingKeyFrames_8f8318e65156461e8f8d13c098a2c800, "FocusVisualElement");
                    var DiscreteObjectKeyFrame_a79d05b8cce84341b46ae1e7c5687858 = new Bridge.global.Windows.UI.Xaml.Media.Animation.DiscreteObjectKeyFrame();
                    DiscreteObjectKeyFrame_a79d05b8cce84341b46ae1e7c5687858.KeyTime = Bridge.global.Windows.UI.Xaml.Media.Animation.KeyTime.FromTimeSpan(new Bridge.global.System.TimeSpan(System.Int64(0)));
                    DiscreteObjectKeyFrame_a79d05b8cce84341b46ae1e7c5687858.Value = "1";

                    ObjectAnimationUsingKeyFrames_8f8318e65156461e8f8d13c098a2c800.KeyFrames.add(DiscreteObjectKeyFrame_a79d05b8cce84341b46ae1e7c5687858);


                    Storyboard_e6ea90d212fd4d44afa8242d5264b63b.Children.add(ObjectAnimationUsingKeyFrames_8f8318e65156461e8f8d13c098a2c800);


                    VisualState_ee9296852392448b96420200c36af877.Storyboard = Storyboard_e6ea90d212fd4d44afa8242d5264b63b;


                    templateOwner_ControlTemplate_b38967e81ef84765a2ad46f8d4a59bb0.RegisterName("Unfocused", VisualState_ca57051dfcd442be85fa04922bcb86bf);
                    VisualState_ca57051dfcd442be85fa04922bcb86bf.Name = "Unfocused";

                    System.Array.add(VisualStateGroup_9fe9f9e6560e446880673d0352e8fa64.States, VisualState_ee9296852392448b96420200c36af877, Object);
                    System.Array.add(VisualStateGroup_9fe9f9e6560e446880673d0352e8fa64.States, VisualState_ca57051dfcd442be85fa04922bcb86bf, Object);


                    templateOwner_ControlTemplate_b38967e81ef84765a2ad46f8d4a59bb0.INTERNAL_GetVisualStateGroups().add(VisualStateGroup_0d8b6bf1b3ec40d686b0ef1eba13448f);
                    templateOwner_ControlTemplate_b38967e81ef84765a2ad46f8d4a59bb0.INTERNAL_GetVisualStateGroups().add(VisualStateGroup_985cb445890049bd8cd03e1f0ea4aa47);
                    templateOwner_ControlTemplate_b38967e81ef84765a2ad46f8d4a59bb0.INTERNAL_GetVisualStateGroups().add(VisualStateGroup_9fe9f9e6560e446880673d0352e8fa64);

                    templateOwner_ControlTemplate_b38967e81ef84765a2ad46f8d4a59bb0.RegisterName("CheckedStateOuterBorder", Border_6bed044959504ed9a8c7a22c6f47b5cf);
                    Border_6bed044959504ed9a8c7a22c6f47b5cf.Name = "CheckedStateOuterBorder";
                    Border_6bed044959504ed9a8c7a22c6f47b5cf.Background = new Bridge.global.Windows.UI.Xaml.Media.SolidColorBrush.$ctor1(($t1 = new Bridge.global.Windows.UI.Color(), $t1.A = 127, $t1.R = 169, $t1.G = 169, $t1.B = 169, $t1));
                    Border_6bed044959504ed9a8c7a22c6f47b5cf.BorderBrush = new Bridge.global.Windows.UI.Xaml.Media.SolidColorBrush.$ctor1(($t1 = new Bridge.global.Windows.UI.Color(), $t1.A = 0, $t1.R = 255, $t1.G = 255, $t1.B = 255, $t1));
                    Border_6bed044959504ed9a8c7a22c6f47b5cf.CornerRadius = new Bridge.global.Windows.UI.Xaml.CornerRadius.$ctor1(3);
                    Border_6bed044959504ed9a8c7a22c6f47b5cf.Opacity = 0.0;
                    var Binding_59b00ad4e70347c0802540b14930bc78 = new Bridge.global.Windows.UI.Xaml.Data.Binding.ctor();
                    Binding_59b00ad4e70347c0802540b14930bc78.Path = new Bridge.global.Windows.UI.Xaml.PropertyPath.ctor("BorderThickness");
                    var RelativeSource_ee924d3f387e4a0ca703d7690cacb0cf = new Bridge.global.Windows.UI.Xaml.Data.RelativeSource.ctor();
                    RelativeSource_ee924d3f387e4a0ca703d7690cacb0cf.Mode = Bridge.global.Windows.UI.Xaml.Data.RelativeSourceMode.TemplatedParent;

                    Binding_59b00ad4e70347c0802540b14930bc78.RelativeSource = RelativeSource_ee924d3f387e4a0ca703d7690cacb0cf;


                    Binding_59b00ad4e70347c0802540b14930bc78.TemplateOwner = templateInstance_8c66612326ba41e787ad16a0b6378454;


                    templateOwner_ControlTemplate_b38967e81ef84765a2ad46f8d4a59bb0.RegisterName("OuterBtnBorder", Border_55d676e8ad224a4eb2b898c941e967b5);
                    Border_55d676e8ad224a4eb2b898c941e967b5.Name = "OuterBtnBorder";
                    Border_55d676e8ad224a4eb2b898c941e967b5.BorderBrush = new Bridge.global.Windows.UI.Xaml.Media.SolidColorBrush.$ctor1(($t1 = new Bridge.global.Windows.UI.Color(), $t1.A = 0, $t1.R = 255, $t1.G = 255, $t1.B = 255, $t1));
                    Border_55d676e8ad224a4eb2b898c941e967b5.CornerRadius = new Bridge.global.Windows.UI.Xaml.CornerRadius.$ctor1(3);
                    templateOwner_ControlTemplate_b38967e81ef84765a2ad46f8d4a59bb0.RegisterName("InnerBtnBorder", Border_dd25e2f9bd8649b38f33d7ee44e18f38);
                    Border_dd25e2f9bd8649b38f33d7ee44e18f38.Name = "InnerBtnBorder";
                    Border_dd25e2f9bd8649b38f33d7ee44e18f38.BorderBrush = new Bridge.global.Windows.UI.Xaml.Media.SolidColorBrush.$ctor1(($t1 = new Bridge.global.Windows.UI.Color(), $t1.A = 0, $t1.R = 204, $t1.G = 209, $t1.B = 214, $t1));
                    Border_dd25e2f9bd8649b38f33d7ee44e18f38.CornerRadius = new Bridge.global.Windows.UI.Xaml.CornerRadius.$ctor1(2);
                    templateOwner_ControlTemplate_b38967e81ef84765a2ad46f8d4a59bb0.RegisterName("contentPresenter", ContentPresenter_1da2347a1e974f6ba82278fc6173d324);
                    ContentPresenter_1da2347a1e974f6ba82278fc6173d324.Name = "contentPresenter";
                    ContentPresenter_1da2347a1e974f6ba82278fc6173d324.HorizontalAlignment = Bridge.global.Windows.UI.Xaml.HorizontalAlignment.Center;
                    ContentPresenter_1da2347a1e974f6ba82278fc6173d324.VerticalAlignment = Bridge.global.Windows.UI.Xaml.VerticalAlignment.Center;
                    var Binding_4acfaac162fa4267b76d6946bd284369 = new Bridge.global.Windows.UI.Xaml.Data.Binding.ctor();
                    Binding_4acfaac162fa4267b76d6946bd284369.Path = new Bridge.global.Windows.UI.Xaml.PropertyPath.ctor("Content");
                    var RelativeSource_db63bfb7d89d46ca975bb13221e0ae81 = new Bridge.global.Windows.UI.Xaml.Data.RelativeSource.ctor();
                    RelativeSource_db63bfb7d89d46ca975bb13221e0ae81.Mode = Bridge.global.Windows.UI.Xaml.Data.RelativeSourceMode.TemplatedParent;

                    Binding_4acfaac162fa4267b76d6946bd284369.RelativeSource = RelativeSource_db63bfb7d89d46ca975bb13221e0ae81;


                    Binding_4acfaac162fa4267b76d6946bd284369.TemplateOwner = templateInstance_8c66612326ba41e787ad16a0b6378454;

                    var Binding_d868e95eb4b2411e919461016267d984 = new Bridge.global.Windows.UI.Xaml.Data.Binding.ctor();
                    Binding_d868e95eb4b2411e919461016267d984.Path = new Bridge.global.Windows.UI.Xaml.PropertyPath.ctor("ContentTemplate");
                    var RelativeSource_0cd669e5a2d3484389f5b4dfa43c5961 = new Bridge.global.Windows.UI.Xaml.Data.RelativeSource.ctor();
                    RelativeSource_0cd669e5a2d3484389f5b4dfa43c5961.Mode = Bridge.global.Windows.UI.Xaml.Data.RelativeSourceMode.TemplatedParent;

                    Binding_d868e95eb4b2411e919461016267d984.RelativeSource = RelativeSource_0cd669e5a2d3484389f5b4dfa43c5961;


                    Binding_d868e95eb4b2411e919461016267d984.TemplateOwner = templateInstance_8c66612326ba41e787ad16a0b6378454;


                    Border_dd25e2f9bd8649b38f33d7ee44e18f38.Child = ContentPresenter_1da2347a1e974f6ba82278fc6173d324;

                    var Binding_2043b4c6bd26448493fc0932117d4d09 = new Bridge.global.Windows.UI.Xaml.Data.Binding.ctor();
                    Binding_2043b4c6bd26448493fc0932117d4d09.Path = new Bridge.global.Windows.UI.Xaml.PropertyPath.ctor("BorderThickness");
                    var RelativeSource_b8dae3f66cfe407c9dc5fcce9838c604 = new Bridge.global.Windows.UI.Xaml.Data.RelativeSource.ctor();
                    RelativeSource_b8dae3f66cfe407c9dc5fcce9838c604.Mode = Bridge.global.Windows.UI.Xaml.Data.RelativeSourceMode.TemplatedParent;

                    Binding_2043b4c6bd26448493fc0932117d4d09.RelativeSource = RelativeSource_b8dae3f66cfe407c9dc5fcce9838c604;


                    Binding_2043b4c6bd26448493fc0932117d4d09.TemplateOwner = templateInstance_8c66612326ba41e787ad16a0b6378454;


                    Border_55d676e8ad224a4eb2b898c941e967b5.Child = Border_dd25e2f9bd8649b38f33d7ee44e18f38;

                    var Binding_e9f943a484ef4a7e8852f179f4f491a3 = new Bridge.global.Windows.UI.Xaml.Data.Binding.ctor();
                    Binding_e9f943a484ef4a7e8852f179f4f491a3.Path = new Bridge.global.Windows.UI.Xaml.PropertyPath.ctor("Background");
                    var RelativeSource_85284820f7e74fbe85f9782b791172eb = new Bridge.global.Windows.UI.Xaml.Data.RelativeSource.ctor();
                    RelativeSource_85284820f7e74fbe85f9782b791172eb.Mode = Bridge.global.Windows.UI.Xaml.Data.RelativeSourceMode.TemplatedParent;

                    Binding_e9f943a484ef4a7e8852f179f4f491a3.RelativeSource = RelativeSource_85284820f7e74fbe85f9782b791172eb;


                    Binding_e9f943a484ef4a7e8852f179f4f491a3.TemplateOwner = templateInstance_8c66612326ba41e787ad16a0b6378454;

                    var Binding_94ff2796b494484e809a830ee39f7c42 = new Bridge.global.Windows.UI.Xaml.Data.Binding.ctor();
                    Binding_94ff2796b494484e809a830ee39f7c42.Path = new Bridge.global.Windows.UI.Xaml.PropertyPath.ctor("BorderThickness");
                    var RelativeSource_34bfc0351a994c4692795be843bdb83a = new Bridge.global.Windows.UI.Xaml.Data.RelativeSource.ctor();
                    RelativeSource_34bfc0351a994c4692795be843bdb83a.Mode = Bridge.global.Windows.UI.Xaml.Data.RelativeSourceMode.TemplatedParent;

                    Binding_94ff2796b494484e809a830ee39f7c42.RelativeSource = RelativeSource_34bfc0351a994c4692795be843bdb83a;


                    Binding_94ff2796b494484e809a830ee39f7c42.TemplateOwner = templateInstance_8c66612326ba41e787ad16a0b6378454;


                    templateOwner_ControlTemplate_b38967e81ef84765a2ad46f8d4a59bb0.RegisterName("FocusVisualElement", Border_b6c574fc2120413a80a4d4be109014c9);
                    Border_b6c574fc2120413a80a4d4be109014c9.Name = "FocusVisualElement";
                    Border_b6c574fc2120413a80a4d4be109014c9.BorderBrush = new Bridge.global.Windows.UI.Xaml.Media.SolidColorBrush.$ctor1(($t1 = new Bridge.global.Windows.UI.Color(), $t1.A = 255, $t1.R = 109, $t1.G = 189, $t1.B = 209, $t1));
                    Border_b6c574fc2120413a80a4d4be109014c9.CornerRadius = new Bridge.global.Windows.UI.Xaml.CornerRadius.$ctor1(2);
                    Border_b6c574fc2120413a80a4d4be109014c9.Margin = new Bridge.global.Windows.UI.Xaml.Thickness.$ctor1(1);
                    Border_b6c574fc2120413a80a4d4be109014c9.Opacity = 0.0;
                    var Binding_a687c56487af48e8aeb25085c37986cf = new Bridge.global.Windows.UI.Xaml.Data.Binding.ctor();
                    Binding_a687c56487af48e8aeb25085c37986cf.Path = new Bridge.global.Windows.UI.Xaml.PropertyPath.ctor("Background");
                    var RelativeSource_f6862717624f45fb84ba5db88ebf597d = new Bridge.global.Windows.UI.Xaml.Data.RelativeSource.ctor();
                    RelativeSource_f6862717624f45fb84ba5db88ebf597d.Mode = Bridge.global.Windows.UI.Xaml.Data.RelativeSourceMode.TemplatedParent;

                    Binding_a687c56487af48e8aeb25085c37986cf.RelativeSource = RelativeSource_f6862717624f45fb84ba5db88ebf597d;


                    Binding_a687c56487af48e8aeb25085c37986cf.TemplateOwner = templateInstance_8c66612326ba41e787ad16a0b6378454;

                    var Binding_4dfe6f4279cd4fb7b0d334169f6dd647 = new Bridge.global.Windows.UI.Xaml.Data.Binding.ctor();
                    Binding_4dfe6f4279cd4fb7b0d334169f6dd647.Path = new Bridge.global.Windows.UI.Xaml.PropertyPath.ctor("BorderThickness");
                    var RelativeSource_a9da98f361084a488bc471df8b0f0f73 = new Bridge.global.Windows.UI.Xaml.Data.RelativeSource.ctor();
                    RelativeSource_a9da98f361084a488bc471df8b0f0f73.Mode = Bridge.global.Windows.UI.Xaml.Data.RelativeSourceMode.TemplatedParent;

                    Binding_4dfe6f4279cd4fb7b0d334169f6dd647.RelativeSource = RelativeSource_a9da98f361084a488bc471df8b0f0f73;


                    Binding_4dfe6f4279cd4fb7b0d334169f6dd647.TemplateOwner = templateInstance_8c66612326ba41e787ad16a0b6378454;


                    Grid_4af9b314fab64a0a89dbc4ede6491914.Children.add(Border_6bed044959504ed9a8c7a22c6f47b5cf);
                    Grid_4af9b314fab64a0a89dbc4ede6491914.Children.add(Border_55d676e8ad224a4eb2b898c941e967b5);
                    Grid_4af9b314fab64a0a89dbc4ede6491914.Children.add(Border_b6c574fc2120413a80a4d4be109014c9);


                    Bridge.global.Windows.UI.Xaml.Data.BindingOperations.SetBinding(Border_6bed044959504ed9a8c7a22c6f47b5cf, Bridge.global.Windows.UI.Xaml.Controls.Border.BorderThicknessProperty, Binding_59b00ad4e70347c0802540b14930bc78);
                    Bridge.global.Windows.UI.Xaml.Data.BindingOperations.SetBinding(ContentPresenter_1da2347a1e974f6ba82278fc6173d324, Bridge.global.Windows.UI.Xaml.Controls.ContentControl.ContentProperty, Binding_4acfaac162fa4267b76d6946bd284369);
                    Bridge.global.Windows.UI.Xaml.Data.BindingOperations.SetBinding(ContentPresenter_1da2347a1e974f6ba82278fc6173d324, Bridge.global.Windows.UI.Xaml.Controls.ContentControl.ContentTemplateProperty, Binding_d868e95eb4b2411e919461016267d984);
                    Bridge.global.Windows.UI.Xaml.Data.BindingOperations.SetBinding(Border_dd25e2f9bd8649b38f33d7ee44e18f38, Bridge.global.Windows.UI.Xaml.Controls.Border.BorderThicknessProperty, Binding_2043b4c6bd26448493fc0932117d4d09);
                    Bridge.global.Windows.UI.Xaml.Data.BindingOperations.SetBinding(Border_55d676e8ad224a4eb2b898c941e967b5, Bridge.global.Windows.UI.Xaml.Controls.Border.BackgroundProperty, Binding_e9f943a484ef4a7e8852f179f4f491a3);
                    Bridge.global.Windows.UI.Xaml.Data.BindingOperations.SetBinding(Border_55d676e8ad224a4eb2b898c941e967b5, Bridge.global.Windows.UI.Xaml.Controls.Border.BorderThicknessProperty, Binding_94ff2796b494484e809a830ee39f7c42);
                    Bridge.global.Windows.UI.Xaml.Data.BindingOperations.SetBinding(Border_b6c574fc2120413a80a4d4be109014c9, Bridge.global.Windows.UI.Xaml.Controls.Border.BackgroundProperty, Binding_a687c56487af48e8aeb25085c37986cf);
                    Bridge.global.Windows.UI.Xaml.Data.BindingOperations.SetBinding(Border_b6c574fc2120413a80a4d4be109014c9, Bridge.global.Windows.UI.Xaml.Controls.Border.BorderThicknessProperty, Binding_4dfe6f4279cd4fb7b0d334169f6dd647);

                    Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard.SetTargetProperty(ObjectAnimationUsingKeyFrames_29fe99d3ff2d45159bd756f9ac08633d, new Bridge.global.Windows.UI.Xaml.PropertyPath.$ctor1("Opacity", "Opacity", Bridge.fn.cacheBind(this, this.accessVisualStateProperty_4958834675ea458b83a1dab0fe8a4335), Bridge.fn.cacheBind(this, this.setVisualStateProperty_4958834675ea458b83a1dab0fe8a4335), Bridge.fn.cacheBind(this, this.setAnimationVisualStateProperty_4958834675ea458b83a1dab0fe8a4335), Bridge.fn.cacheBind(this, this.setLocalVisualStateProperty_4958834675ea458b83a1dab0fe8a4335), Bridge.fn.cacheBind(this, this.getVisualStateProperty_4958834675ea458b83a1dab0fe8a4335)));
                    Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard.SetTarget(ObjectAnimationUsingKeyFrames_29fe99d3ff2d45159bd756f9ac08633d, ContentPresenter_1da2347a1e974f6ba82278fc6173d324);


                    Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard.SetTargetProperty(ObjectAnimationUsingKeyFrames_5882435b970b4536b2b4ecb680ef8074, new Bridge.global.Windows.UI.Xaml.PropertyPath.$ctor1("Opacity", "Opacity", Bridge.fn.cacheBind(this, this.accessVisualStateProperty_424ca16da0bf47cf821542d0abab5b94), Bridge.fn.cacheBind(this, this.setVisualStateProperty_424ca16da0bf47cf821542d0abab5b94), Bridge.fn.cacheBind(this, this.setAnimationVisualStateProperty_424ca16da0bf47cf821542d0abab5b94), Bridge.fn.cacheBind(this, this.setLocalVisualStateProperty_424ca16da0bf47cf821542d0abab5b94), Bridge.fn.cacheBind(this, this.getVisualStateProperty_424ca16da0bf47cf821542d0abab5b94)));
                    Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard.SetTarget(ObjectAnimationUsingKeyFrames_5882435b970b4536b2b4ecb680ef8074, Border_6bed044959504ed9a8c7a22c6f47b5cf);


                    Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard.SetTargetProperty(ObjectAnimationUsingKeyFrames_8f8318e65156461e8f8d13c098a2c800, new Bridge.global.Windows.UI.Xaml.PropertyPath.$ctor1("Opacity", "Opacity", Bridge.fn.cacheBind(this, this.accessVisualStateProperty_100421434fa543818d109d13a477cfc0), Bridge.fn.cacheBind(this, this.setVisualStateProperty_100421434fa543818d109d13a477cfc0), Bridge.fn.cacheBind(this, this.setAnimationVisualStateProperty_100421434fa543818d109d13a477cfc0), Bridge.fn.cacheBind(this, this.setLocalVisualStateProperty_100421434fa543818d109d13a477cfc0), Bridge.fn.cacheBind(this, this.getVisualStateProperty_100421434fa543818d109d13a477cfc0)));
                    Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard.SetTarget(ObjectAnimationUsingKeyFrames_8f8318e65156461e8f8d13c098a2c800, Border_b6c574fc2120413a80a4d4be109014c9);

                    templateInstance_8c66612326ba41e787ad16a0b6378454.TemplateContent = Grid_4af9b314fab64a0a89dbc4ede6491914;
                    return templateInstance_8c66612326ba41e787ad16a0b6378454;
                }));

                Setter_0240cac5e087458a9bbde5904a5ad6c0.Value = ControlTemplate_b38967e81ef84765a2ad46f8d4a59bb0;


                Style_ec3987d1679d46aa87884a9777809240.Setters.add(Setter_86c5f28dc82f45eea5cdcc39c99f6709);
                Style_ec3987d1679d46aa87884a9777809240.Setters.add(Setter_13d4434ba0b948839c372982e62bdbcb);
                Style_ec3987d1679d46aa87884a9777809240.Setters.add(Setter_c58ca3e32284418793e7fc58ef457f38);
                Style_ec3987d1679d46aa87884a9777809240.Setters.add(Setter_6d3f805a22004b3383c4bac1edbda1d8);
                Style_ec3987d1679d46aa87884a9777809240.Setters.add(Setter_9dc0fc71d3954575aa97f6229c3f165c);
                Style_ec3987d1679d46aa87884a9777809240.Setters.add(Setter_8f9de49fd2624b88a9add9742e2cf2a1);
                Style_ec3987d1679d46aa87884a9777809240.Setters.add(Setter_1db26c81dff6444eaa9f156f64951466);
                Style_ec3987d1679d46aa87884a9777809240.Setters.add(Setter_0240cac5e087458a9bbde5904a5ad6c0);


                Setter_b95adf35db3948e0a62da7b6da83f060.Value = Style_ec3987d1679d46aa87884a9777809240;


                var Setter_1ea1cc8b6c3c4f1ba30f65b75ef17a91 = new Bridge.global.Windows.UI.Xaml.Setter.ctor();
                Setter_1ea1cc8b6c3c4f1ba30f65b75ef17a91.Property = Bridge.global.Windows.UI.Xaml.Controls.Control.TemplateProperty;
                var ControlTemplate_c2c1fafddd0e45348f9697b3bfd04e92 = new Bridge.global.Windows.UI.Xaml.Controls.ControlTemplate();
                ControlTemplate_c2c1fafddd0e45348f9697b3bfd04e92.TargetType = Bridge.global.Windows.UI.Xaml.Controls.DataPager;
                ControlTemplate_c2c1fafddd0e45348f9697b3bfd04e92.SetMethodToInstantiateFrameworkTemplate(Bridge.fn.bind(this, function (templateOwner_ControlTemplate_c2c1fafddd0e45348f9697b3bfd04e92) {
                    var $t1;
                    var templateInstance_37f7df09e03649e7b9c6ab503fce8242 = new Bridge.global.Windows.UI.Xaml.TemplateInstance();
                    templateInstance_37f7df09e03649e7b9c6ab503fce8242.TemplateOwner = templateOwner_ControlTemplate_c2c1fafddd0e45348f9697b3bfd04e92;
                    var VisualState_0e1826d4ce224a8d86bfce7bd4614ef8 = new Bridge.global.Windows.UI.Xaml.VisualState();
                    var VisualState_a784fa3a663747d0964aa099f949c856 = new Bridge.global.Windows.UI.Xaml.VisualState();
                    var VisualState_1a6286817dd34f6e851dc74750f0ce98 = new Bridge.global.Windows.UI.Xaml.VisualState();
                    var VisualState_836618b99af240b2b37a67c2c976e259 = new Bridge.global.Windows.UI.Xaml.VisualState();
                    var VisualState_8647c530d5054e62aec9e9720e09c1ce = new Bridge.global.Windows.UI.Xaml.VisualState();
                    var VisualState_861d8139c4cd4f8bbb7b27068a143154 = new Bridge.global.Windows.UI.Xaml.VisualState();
                    var VisualStateGroup_e44f95216530493888aedcd213f18b83 = new Bridge.global.Windows.UI.Xaml.VisualStateGroup();
                    var Button_2ecf43a729774feda5635ec33b78e87b = new Bridge.global.Windows.UI.Xaml.Controls.Button();
                    var Button_7db9715dc6224087bb4e526aee5c7542 = new Bridge.global.Windows.UI.Xaml.Controls.Button();
                    var Border_583805b6b32e43a78d3bf62fba7f9f9b = new Bridge.global.Windows.UI.Xaml.Controls.Border();
                    var StackPanel_b58f85e120db44f6bd5370a6d599972d = new Bridge.global.Windows.UI.Xaml.Controls.StackPanel();
                    var TextBlock_47ec22267bb2413c95c0a2aa0c044f69 = new Bridge.global.Windows.UI.Xaml.Controls.TextBlock();
                    var TextBox_5e5b54f2d3ac42db9b91623d8ae9b49a = new Bridge.global.Windows.UI.Xaml.Controls.TextBox();
                    var TextBlock_5c303ae12396405f98da5911db622613 = new Bridge.global.Windows.UI.Xaml.Controls.TextBlock();
                    var StackPanel_5039ccb47ff04b0f926057007c69e386 = new Bridge.global.Windows.UI.Xaml.Controls.StackPanel();
                    var Border_d484f9589f234b718b2106b85893789a = new Bridge.global.Windows.UI.Xaml.Controls.Border();
                    var Button_975225bcbd324697a58acaa112f593d7 = new Bridge.global.Windows.UI.Xaml.Controls.Button();
                    var Button_b6cb766b3f6f419eb6a55dd4be55411e = new Bridge.global.Windows.UI.Xaml.Controls.Button();
                    var Grid_0ff886fd41ef49cb84afb3a4b3d1dc0a = new Bridge.global.Windows.UI.Xaml.Controls.Grid();
                    templateOwner_ControlTemplate_c2c1fafddd0e45348f9697b3bfd04e92.RegisterName("Root", Grid_0ff886fd41ef49cb84afb3a4b3d1dc0a);
                    Grid_0ff886fd41ef49cb84afb3a4b3d1dc0a.Name = "Root";
                    Grid_0ff886fd41ef49cb84afb3a4b3d1dc0a.Background = new Bridge.global.Windows.UI.Xaml.Media.SolidColorBrush.$ctor1(($t1 = new Bridge.global.Windows.UI.Color(), $t1.A = 0, $t1.R = 255, $t1.G = 255, $t1.B = 255, $t1));

                    templateOwner_ControlTemplate_c2c1fafddd0e45348f9697b3bfd04e92.RegisterName("DisplayModeStates", VisualStateGroup_e44f95216530493888aedcd213f18b83);
                    VisualStateGroup_e44f95216530493888aedcd213f18b83.Name = "DisplayModeStates";
                    templateOwner_ControlTemplate_c2c1fafddd0e45348f9697b3bfd04e92.RegisterName("FirstLastNumeric", VisualState_0e1826d4ce224a8d86bfce7bd4614ef8);
                    VisualState_0e1826d4ce224a8d86bfce7bd4614ef8.Name = "FirstLastNumeric";
                    var Storyboard_f4afeffb25ee4f2ca4b267603a19b97a = new Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard();
                    var ObjectAnimationUsingKeyFrames_eb9ea2a1cd5e4a49861eb516294d9bf7 = new Bridge.global.Windows.UI.Xaml.Media.Animation.ObjectAnimationUsingKeyFrames();
                    Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard.SetTargetName(ObjectAnimationUsingKeyFrames_eb9ea2a1cd5e4a49861eb516294d9bf7, "NextPageButton");
                    var DiscreteObjectKeyFrame_2e7d3172ee394c60968efcbe2f38593f = new Bridge.global.Windows.UI.Xaml.Media.Animation.DiscreteObjectKeyFrame();
                    DiscreteObjectKeyFrame_2e7d3172ee394c60968efcbe2f38593f.KeyTime = Bridge.global.Windows.UI.Xaml.Media.Animation.KeyTime.FromTimeSpan(new Bridge.global.System.TimeSpan(System.Int64(0)));
                    DiscreteObjectKeyFrame_2e7d3172ee394c60968efcbe2f38593f.Value = "Collapsed";

                    ObjectAnimationUsingKeyFrames_eb9ea2a1cd5e4a49861eb516294d9bf7.KeyFrames.add(DiscreteObjectKeyFrame_2e7d3172ee394c60968efcbe2f38593f);


                    var ObjectAnimationUsingKeyFrames_c20a3d4ab21e4d6782b38ede1a0b7439 = new Bridge.global.Windows.UI.Xaml.Media.Animation.ObjectAnimationUsingKeyFrames();
                    Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard.SetTargetName(ObjectAnimationUsingKeyFrames_c20a3d4ab21e4d6782b38ede1a0b7439, "PreviousPageButton");
                    var DiscreteObjectKeyFrame_a0c6b3b069e84baaa90ccb7936227912 = new Bridge.global.Windows.UI.Xaml.Media.Animation.DiscreteObjectKeyFrame();
                    DiscreteObjectKeyFrame_a0c6b3b069e84baaa90ccb7936227912.KeyTime = Bridge.global.Windows.UI.Xaml.Media.Animation.KeyTime.FromTimeSpan(new Bridge.global.System.TimeSpan(System.Int64(0)));
                    DiscreteObjectKeyFrame_a0c6b3b069e84baaa90ccb7936227912.Value = "Collapsed";

                    ObjectAnimationUsingKeyFrames_c20a3d4ab21e4d6782b38ede1a0b7439.KeyFrames.add(DiscreteObjectKeyFrame_a0c6b3b069e84baaa90ccb7936227912);


                    var ObjectAnimationUsingKeyFrames_87912696dc714f39a0ca3db9c11e2a5c = new Bridge.global.Windows.UI.Xaml.Media.Animation.ObjectAnimationUsingKeyFrames();
                    Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard.SetTargetName(ObjectAnimationUsingKeyFrames_87912696dc714f39a0ca3db9c11e2a5c, "CurrentPageTextBox");
                    var DiscreteObjectKeyFrame_21575be2ca5d42838b14ee2814942ca8 = new Bridge.global.Windows.UI.Xaml.Media.Animation.DiscreteObjectKeyFrame();
                    DiscreteObjectKeyFrame_21575be2ca5d42838b14ee2814942ca8.KeyTime = Bridge.global.Windows.UI.Xaml.Media.Animation.KeyTime.FromTimeSpan(new Bridge.global.System.TimeSpan(System.Int64(0)));
                    DiscreteObjectKeyFrame_21575be2ca5d42838b14ee2814942ca8.Value = "Collapsed";

                    ObjectAnimationUsingKeyFrames_87912696dc714f39a0ca3db9c11e2a5c.KeyFrames.add(DiscreteObjectKeyFrame_21575be2ca5d42838b14ee2814942ca8);


                    var ObjectAnimationUsingKeyFrames_133978e8021d4bb6ae0387fa94692ae5 = new Bridge.global.Windows.UI.Xaml.Media.Animation.ObjectAnimationUsingKeyFrames();
                    Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard.SetTargetName(ObjectAnimationUsingKeyFrames_133978e8021d4bb6ae0387fa94692ae5, "PageDisplay");
                    var DiscreteObjectKeyFrame_8c50138388364bfcbfb62594ae46ab3f = new Bridge.global.Windows.UI.Xaml.Media.Animation.DiscreteObjectKeyFrame();
                    DiscreteObjectKeyFrame_8c50138388364bfcbfb62594ae46ab3f.KeyTime = Bridge.global.Windows.UI.Xaml.Media.Animation.KeyTime.FromTimeSpan(new Bridge.global.System.TimeSpan(System.Int64(0)));
                    DiscreteObjectKeyFrame_8c50138388364bfcbfb62594ae46ab3f.Value = "Collapsed";

                    ObjectAnimationUsingKeyFrames_133978e8021d4bb6ae0387fa94692ae5.KeyFrames.add(DiscreteObjectKeyFrame_8c50138388364bfcbfb62594ae46ab3f);


                    Storyboard_f4afeffb25ee4f2ca4b267603a19b97a.Children.add(ObjectAnimationUsingKeyFrames_eb9ea2a1cd5e4a49861eb516294d9bf7);
                    Storyboard_f4afeffb25ee4f2ca4b267603a19b97a.Children.add(ObjectAnimationUsingKeyFrames_c20a3d4ab21e4d6782b38ede1a0b7439);
                    Storyboard_f4afeffb25ee4f2ca4b267603a19b97a.Children.add(ObjectAnimationUsingKeyFrames_87912696dc714f39a0ca3db9c11e2a5c);
                    Storyboard_f4afeffb25ee4f2ca4b267603a19b97a.Children.add(ObjectAnimationUsingKeyFrames_133978e8021d4bb6ae0387fa94692ae5);


                    VisualState_0e1826d4ce224a8d86bfce7bd4614ef8.Storyboard = Storyboard_f4afeffb25ee4f2ca4b267603a19b97a;


                    templateOwner_ControlTemplate_c2c1fafddd0e45348f9697b3bfd04e92.RegisterName("FirstLastPreviousNext", VisualState_a784fa3a663747d0964aa099f949c856);
                    VisualState_a784fa3a663747d0964aa099f949c856.Name = "FirstLastPreviousNext";
                    var Storyboard_cca6574649c84962a6c2afa8ab1f7741 = new Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard();
                    var ObjectAnimationUsingKeyFrames_4bec98a8f10a401fba166d082acc7e9b = new Bridge.global.Windows.UI.Xaml.Media.Animation.ObjectAnimationUsingKeyFrames();
                    Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard.SetTargetName(ObjectAnimationUsingKeyFrames_4bec98a8f10a401fba166d082acc7e9b, "NumericButtonPanel");
                    var DiscreteObjectKeyFrame_7a4f85cae4cd417da3434d9de7891c39 = new Bridge.global.Windows.UI.Xaml.Media.Animation.DiscreteObjectKeyFrame();
                    DiscreteObjectKeyFrame_7a4f85cae4cd417da3434d9de7891c39.KeyTime = Bridge.global.Windows.UI.Xaml.Media.Animation.KeyTime.FromTimeSpan(new Bridge.global.System.TimeSpan(System.Int64(0)));
                    DiscreteObjectKeyFrame_7a4f85cae4cd417da3434d9de7891c39.Value = "Collapsed";

                    ObjectAnimationUsingKeyFrames_4bec98a8f10a401fba166d082acc7e9b.KeyFrames.add(DiscreteObjectKeyFrame_7a4f85cae4cd417da3434d9de7891c39);


                    Storyboard_cca6574649c84962a6c2afa8ab1f7741.Children.add(ObjectAnimationUsingKeyFrames_4bec98a8f10a401fba166d082acc7e9b);


                    VisualState_a784fa3a663747d0964aa099f949c856.Storyboard = Storyboard_cca6574649c84962a6c2afa8ab1f7741;


                    templateOwner_ControlTemplate_c2c1fafddd0e45348f9697b3bfd04e92.RegisterName("FirstLastPreviousNextNumeric", VisualState_1a6286817dd34f6e851dc74750f0ce98);
                    VisualState_1a6286817dd34f6e851dc74750f0ce98.Name = "FirstLastPreviousNextNumeric";
                    var Storyboard_61ad870aafd84ab1aca60db15e3b6e8b = new Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard();
                    var ObjectAnimationUsingKeyFrames_4fa98f75e27c428291587964b706140b = new Bridge.global.Windows.UI.Xaml.Media.Animation.ObjectAnimationUsingKeyFrames();
                    Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard.SetTargetName(ObjectAnimationUsingKeyFrames_4fa98f75e27c428291587964b706140b, "CurrentPageTextBox");
                    var DiscreteObjectKeyFrame_d50a5e292fd54c92926780fb431a3950 = new Bridge.global.Windows.UI.Xaml.Media.Animation.DiscreteObjectKeyFrame();
                    DiscreteObjectKeyFrame_d50a5e292fd54c92926780fb431a3950.KeyTime = Bridge.global.Windows.UI.Xaml.Media.Animation.KeyTime.FromTimeSpan(new Bridge.global.System.TimeSpan(System.Int64(0)));
                    DiscreteObjectKeyFrame_d50a5e292fd54c92926780fb431a3950.Value = "Collapsed";

                    ObjectAnimationUsingKeyFrames_4fa98f75e27c428291587964b706140b.KeyFrames.add(DiscreteObjectKeyFrame_d50a5e292fd54c92926780fb431a3950);


                    var ObjectAnimationUsingKeyFrames_6768ae1bf3ca42cab77d8fd3e8123b2c = new Bridge.global.Windows.UI.Xaml.Media.Animation.ObjectAnimationUsingKeyFrames();
                    Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard.SetTargetName(ObjectAnimationUsingKeyFrames_6768ae1bf3ca42cab77d8fd3e8123b2c, "PageDisplay");
                    var DiscreteObjectKeyFrame_4c7ffaa3907b4c0c8f5969a80380d6b7 = new Bridge.global.Windows.UI.Xaml.Media.Animation.DiscreteObjectKeyFrame();
                    DiscreteObjectKeyFrame_4c7ffaa3907b4c0c8f5969a80380d6b7.KeyTime = Bridge.global.Windows.UI.Xaml.Media.Animation.KeyTime.FromTimeSpan(new Bridge.global.System.TimeSpan(System.Int64(0)));
                    DiscreteObjectKeyFrame_4c7ffaa3907b4c0c8f5969a80380d6b7.Value = "Collapsed";

                    ObjectAnimationUsingKeyFrames_6768ae1bf3ca42cab77d8fd3e8123b2c.KeyFrames.add(DiscreteObjectKeyFrame_4c7ffaa3907b4c0c8f5969a80380d6b7);


                    Storyboard_61ad870aafd84ab1aca60db15e3b6e8b.Children.add(ObjectAnimationUsingKeyFrames_4fa98f75e27c428291587964b706140b);
                    Storyboard_61ad870aafd84ab1aca60db15e3b6e8b.Children.add(ObjectAnimationUsingKeyFrames_6768ae1bf3ca42cab77d8fd3e8123b2c);


                    VisualState_1a6286817dd34f6e851dc74750f0ce98.Storyboard = Storyboard_61ad870aafd84ab1aca60db15e3b6e8b;


                    templateOwner_ControlTemplate_c2c1fafddd0e45348f9697b3bfd04e92.RegisterName("Numeric", VisualState_836618b99af240b2b37a67c2c976e259);
                    VisualState_836618b99af240b2b37a67c2c976e259.Name = "Numeric";
                    var Storyboard_075834babcf74edc8463fc23dfa9dfe3 = new Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard();
                    var ObjectAnimationUsingKeyFrames_e94ee94732ff4eb98b923f8db5461f7e = new Bridge.global.Windows.UI.Xaml.Media.Animation.ObjectAnimationUsingKeyFrames();
                    Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard.SetTargetName(ObjectAnimationUsingKeyFrames_e94ee94732ff4eb98b923f8db5461f7e, "FirstPageButton");
                    var DiscreteObjectKeyFrame_55dec801352348da8ddd7b7e77af3a40 = new Bridge.global.Windows.UI.Xaml.Media.Animation.DiscreteObjectKeyFrame();
                    DiscreteObjectKeyFrame_55dec801352348da8ddd7b7e77af3a40.KeyTime = Bridge.global.Windows.UI.Xaml.Media.Animation.KeyTime.FromTimeSpan(new Bridge.global.System.TimeSpan(System.Int64(0)));
                    DiscreteObjectKeyFrame_55dec801352348da8ddd7b7e77af3a40.Value = "Collapsed";

                    ObjectAnimationUsingKeyFrames_e94ee94732ff4eb98b923f8db5461f7e.KeyFrames.add(DiscreteObjectKeyFrame_55dec801352348da8ddd7b7e77af3a40);


                    var ObjectAnimationUsingKeyFrames_a4b0030d8424491784bffaf44d14bc16 = new Bridge.global.Windows.UI.Xaml.Media.Animation.ObjectAnimationUsingKeyFrames();
                    Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard.SetTargetName(ObjectAnimationUsingKeyFrames_a4b0030d8424491784bffaf44d14bc16, "LastPageButton");
                    var DiscreteObjectKeyFrame_e7dbb11ac18d4320a2dac742b692107e = new Bridge.global.Windows.UI.Xaml.Media.Animation.DiscreteObjectKeyFrame();
                    DiscreteObjectKeyFrame_e7dbb11ac18d4320a2dac742b692107e.KeyTime = Bridge.global.Windows.UI.Xaml.Media.Animation.KeyTime.FromTimeSpan(new Bridge.global.System.TimeSpan(System.Int64(0)));
                    DiscreteObjectKeyFrame_e7dbb11ac18d4320a2dac742b692107e.Value = "Collapsed";

                    ObjectAnimationUsingKeyFrames_a4b0030d8424491784bffaf44d14bc16.KeyFrames.add(DiscreteObjectKeyFrame_e7dbb11ac18d4320a2dac742b692107e);


                    var ObjectAnimationUsingKeyFrames_22fca1516c9046078761f69304986397 = new Bridge.global.Windows.UI.Xaml.Media.Animation.ObjectAnimationUsingKeyFrames();
                    Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard.SetTargetName(ObjectAnimationUsingKeyFrames_22fca1516c9046078761f69304986397, "NextPageButton");
                    var DiscreteObjectKeyFrame_41a6b2e5065f40d89d4a8b8f61bf6d13 = new Bridge.global.Windows.UI.Xaml.Media.Animation.DiscreteObjectKeyFrame();
                    DiscreteObjectKeyFrame_41a6b2e5065f40d89d4a8b8f61bf6d13.KeyTime = Bridge.global.Windows.UI.Xaml.Media.Animation.KeyTime.FromTimeSpan(new Bridge.global.System.TimeSpan(System.Int64(0)));
                    DiscreteObjectKeyFrame_41a6b2e5065f40d89d4a8b8f61bf6d13.Value = "Collapsed";

                    ObjectAnimationUsingKeyFrames_22fca1516c9046078761f69304986397.KeyFrames.add(DiscreteObjectKeyFrame_41a6b2e5065f40d89d4a8b8f61bf6d13);


                    var ObjectAnimationUsingKeyFrames_f685fdf829134a2b9a63059c4d8a935d = new Bridge.global.Windows.UI.Xaml.Media.Animation.ObjectAnimationUsingKeyFrames();
                    Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard.SetTargetName(ObjectAnimationUsingKeyFrames_f685fdf829134a2b9a63059c4d8a935d, "PreviousPageButton");
                    var DiscreteObjectKeyFrame_31ebcd76ce544efab3150547f83a81e0 = new Bridge.global.Windows.UI.Xaml.Media.Animation.DiscreteObjectKeyFrame();
                    DiscreteObjectKeyFrame_31ebcd76ce544efab3150547f83a81e0.KeyTime = Bridge.global.Windows.UI.Xaml.Media.Animation.KeyTime.FromTimeSpan(new Bridge.global.System.TimeSpan(System.Int64(0)));
                    DiscreteObjectKeyFrame_31ebcd76ce544efab3150547f83a81e0.Value = "Collapsed";

                    ObjectAnimationUsingKeyFrames_f685fdf829134a2b9a63059c4d8a935d.KeyFrames.add(DiscreteObjectKeyFrame_31ebcd76ce544efab3150547f83a81e0);


                    var ObjectAnimationUsingKeyFrames_1d01829e8165479d8a08f769eb0f2ba2 = new Bridge.global.Windows.UI.Xaml.Media.Animation.ObjectAnimationUsingKeyFrames();
                    Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard.SetTargetName(ObjectAnimationUsingKeyFrames_1d01829e8165479d8a08f769eb0f2ba2, "CurrentPageTextBox");
                    var DiscreteObjectKeyFrame_20d724dc1ba74357a462d8567bcd735c = new Bridge.global.Windows.UI.Xaml.Media.Animation.DiscreteObjectKeyFrame();
                    DiscreteObjectKeyFrame_20d724dc1ba74357a462d8567bcd735c.KeyTime = Bridge.global.Windows.UI.Xaml.Media.Animation.KeyTime.FromTimeSpan(new Bridge.global.System.TimeSpan(System.Int64(0)));
                    DiscreteObjectKeyFrame_20d724dc1ba74357a462d8567bcd735c.Value = "Collapsed";

                    ObjectAnimationUsingKeyFrames_1d01829e8165479d8a08f769eb0f2ba2.KeyFrames.add(DiscreteObjectKeyFrame_20d724dc1ba74357a462d8567bcd735c);


                    var ObjectAnimationUsingKeyFrames_aeed67ea75404c42958956a2f19f2f04 = new Bridge.global.Windows.UI.Xaml.Media.Animation.ObjectAnimationUsingKeyFrames();
                    Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard.SetTargetName(ObjectAnimationUsingKeyFrames_aeed67ea75404c42958956a2f19f2f04, "PageDisplay");
                    var DiscreteObjectKeyFrame_57e737c9d14d4aef86b2d4cfe052fdc1 = new Bridge.global.Windows.UI.Xaml.Media.Animation.DiscreteObjectKeyFrame();
                    DiscreteObjectKeyFrame_57e737c9d14d4aef86b2d4cfe052fdc1.KeyTime = Bridge.global.Windows.UI.Xaml.Media.Animation.KeyTime.FromTimeSpan(new Bridge.global.System.TimeSpan(System.Int64(0)));
                    DiscreteObjectKeyFrame_57e737c9d14d4aef86b2d4cfe052fdc1.Value = "Collapsed";

                    ObjectAnimationUsingKeyFrames_aeed67ea75404c42958956a2f19f2f04.KeyFrames.add(DiscreteObjectKeyFrame_57e737c9d14d4aef86b2d4cfe052fdc1);


                    var ObjectAnimationUsingKeyFrames_dced361ca485481ebd1109769c95762c = new Bridge.global.Windows.UI.Xaml.Media.Animation.ObjectAnimationUsingKeyFrames();
                    Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard.SetTargetName(ObjectAnimationUsingKeyFrames_dced361ca485481ebd1109769c95762c, "Separator1");
                    var DiscreteObjectKeyFrame_36cc7aeefde94b50a85d15e6b50b9c97 = new Bridge.global.Windows.UI.Xaml.Media.Animation.DiscreteObjectKeyFrame();
                    DiscreteObjectKeyFrame_36cc7aeefde94b50a85d15e6b50b9c97.KeyTime = Bridge.global.Windows.UI.Xaml.Media.Animation.KeyTime.FromTimeSpan(new Bridge.global.System.TimeSpan(System.Int64(0)));
                    DiscreteObjectKeyFrame_36cc7aeefde94b50a85d15e6b50b9c97.Value = "Collapsed";

                    ObjectAnimationUsingKeyFrames_dced361ca485481ebd1109769c95762c.KeyFrames.add(DiscreteObjectKeyFrame_36cc7aeefde94b50a85d15e6b50b9c97);


                    var ObjectAnimationUsingKeyFrames_a11aa0067f84477eb3a75d20d42f891a = new Bridge.global.Windows.UI.Xaml.Media.Animation.ObjectAnimationUsingKeyFrames();
                    Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard.SetTargetName(ObjectAnimationUsingKeyFrames_a11aa0067f84477eb3a75d20d42f891a, "Separator2");
                    var DiscreteObjectKeyFrame_e0ed3eb5aa424aea93f523eb0aed6947 = new Bridge.global.Windows.UI.Xaml.Media.Animation.DiscreteObjectKeyFrame();
                    DiscreteObjectKeyFrame_e0ed3eb5aa424aea93f523eb0aed6947.KeyTime = Bridge.global.Windows.UI.Xaml.Media.Animation.KeyTime.FromTimeSpan(new Bridge.global.System.TimeSpan(System.Int64(0)));
                    DiscreteObjectKeyFrame_e0ed3eb5aa424aea93f523eb0aed6947.Value = "Collapsed";

                    ObjectAnimationUsingKeyFrames_a11aa0067f84477eb3a75d20d42f891a.KeyFrames.add(DiscreteObjectKeyFrame_e0ed3eb5aa424aea93f523eb0aed6947);


                    Storyboard_075834babcf74edc8463fc23dfa9dfe3.Children.add(ObjectAnimationUsingKeyFrames_e94ee94732ff4eb98b923f8db5461f7e);
                    Storyboard_075834babcf74edc8463fc23dfa9dfe3.Children.add(ObjectAnimationUsingKeyFrames_a4b0030d8424491784bffaf44d14bc16);
                    Storyboard_075834babcf74edc8463fc23dfa9dfe3.Children.add(ObjectAnimationUsingKeyFrames_22fca1516c9046078761f69304986397);
                    Storyboard_075834babcf74edc8463fc23dfa9dfe3.Children.add(ObjectAnimationUsingKeyFrames_f685fdf829134a2b9a63059c4d8a935d);
                    Storyboard_075834babcf74edc8463fc23dfa9dfe3.Children.add(ObjectAnimationUsingKeyFrames_1d01829e8165479d8a08f769eb0f2ba2);
                    Storyboard_075834babcf74edc8463fc23dfa9dfe3.Children.add(ObjectAnimationUsingKeyFrames_aeed67ea75404c42958956a2f19f2f04);
                    Storyboard_075834babcf74edc8463fc23dfa9dfe3.Children.add(ObjectAnimationUsingKeyFrames_dced361ca485481ebd1109769c95762c);
                    Storyboard_075834babcf74edc8463fc23dfa9dfe3.Children.add(ObjectAnimationUsingKeyFrames_a11aa0067f84477eb3a75d20d42f891a);


                    VisualState_836618b99af240b2b37a67c2c976e259.Storyboard = Storyboard_075834babcf74edc8463fc23dfa9dfe3;


                    templateOwner_ControlTemplate_c2c1fafddd0e45348f9697b3bfd04e92.RegisterName("PreviousNext", VisualState_8647c530d5054e62aec9e9720e09c1ce);
                    VisualState_8647c530d5054e62aec9e9720e09c1ce.Name = "PreviousNext";
                    var Storyboard_648e951965aa4d08879213ad7826a234 = new Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard();
                    var ObjectAnimationUsingKeyFrames_8637b34401534c1e9170431ea815aacb = new Bridge.global.Windows.UI.Xaml.Media.Animation.ObjectAnimationUsingKeyFrames();
                    Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard.SetTargetName(ObjectAnimationUsingKeyFrames_8637b34401534c1e9170431ea815aacb, "FirstPageButton");
                    var DiscreteObjectKeyFrame_8a6f235f511f4e349ed5cabb420434ed = new Bridge.global.Windows.UI.Xaml.Media.Animation.DiscreteObjectKeyFrame();
                    DiscreteObjectKeyFrame_8a6f235f511f4e349ed5cabb420434ed.KeyTime = Bridge.global.Windows.UI.Xaml.Media.Animation.KeyTime.FromTimeSpan(new Bridge.global.System.TimeSpan(System.Int64(0)));
                    DiscreteObjectKeyFrame_8a6f235f511f4e349ed5cabb420434ed.Value = "Collapsed";

                    ObjectAnimationUsingKeyFrames_8637b34401534c1e9170431ea815aacb.KeyFrames.add(DiscreteObjectKeyFrame_8a6f235f511f4e349ed5cabb420434ed);


                    var ObjectAnimationUsingKeyFrames_641fa6d27fd345a696234fb33d4c7e93 = new Bridge.global.Windows.UI.Xaml.Media.Animation.ObjectAnimationUsingKeyFrames();
                    Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard.SetTargetName(ObjectAnimationUsingKeyFrames_641fa6d27fd345a696234fb33d4c7e93, "LastPageButton");
                    var DiscreteObjectKeyFrame_b610aa996898407c986960bd811fdc64 = new Bridge.global.Windows.UI.Xaml.Media.Animation.DiscreteObjectKeyFrame();
                    DiscreteObjectKeyFrame_b610aa996898407c986960bd811fdc64.KeyTime = Bridge.global.Windows.UI.Xaml.Media.Animation.KeyTime.FromTimeSpan(new Bridge.global.System.TimeSpan(System.Int64(0)));
                    DiscreteObjectKeyFrame_b610aa996898407c986960bd811fdc64.Value = "Collapsed";

                    ObjectAnimationUsingKeyFrames_641fa6d27fd345a696234fb33d4c7e93.KeyFrames.add(DiscreteObjectKeyFrame_b610aa996898407c986960bd811fdc64);


                    var ObjectAnimationUsingKeyFrames_5be106d8da9e4e7a9a3d9adb126e9efe = new Bridge.global.Windows.UI.Xaml.Media.Animation.ObjectAnimationUsingKeyFrames();
                    Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard.SetTargetName(ObjectAnimationUsingKeyFrames_5be106d8da9e4e7a9a3d9adb126e9efe, "NumericButtonPanel");
                    var DiscreteObjectKeyFrame_d616fac7ee2448458f449812755c40dd = new Bridge.global.Windows.UI.Xaml.Media.Animation.DiscreteObjectKeyFrame();
                    DiscreteObjectKeyFrame_d616fac7ee2448458f449812755c40dd.KeyTime = Bridge.global.Windows.UI.Xaml.Media.Animation.KeyTime.FromTimeSpan(new Bridge.global.System.TimeSpan(System.Int64(0)));
                    DiscreteObjectKeyFrame_d616fac7ee2448458f449812755c40dd.Value = "Collapsed";

                    ObjectAnimationUsingKeyFrames_5be106d8da9e4e7a9a3d9adb126e9efe.KeyFrames.add(DiscreteObjectKeyFrame_d616fac7ee2448458f449812755c40dd);


                    Storyboard_648e951965aa4d08879213ad7826a234.Children.add(ObjectAnimationUsingKeyFrames_8637b34401534c1e9170431ea815aacb);
                    Storyboard_648e951965aa4d08879213ad7826a234.Children.add(ObjectAnimationUsingKeyFrames_641fa6d27fd345a696234fb33d4c7e93);
                    Storyboard_648e951965aa4d08879213ad7826a234.Children.add(ObjectAnimationUsingKeyFrames_5be106d8da9e4e7a9a3d9adb126e9efe);


                    VisualState_8647c530d5054e62aec9e9720e09c1ce.Storyboard = Storyboard_648e951965aa4d08879213ad7826a234;


                    templateOwner_ControlTemplate_c2c1fafddd0e45348f9697b3bfd04e92.RegisterName("PreviousNextNumeric", VisualState_861d8139c4cd4f8bbb7b27068a143154);
                    VisualState_861d8139c4cd4f8bbb7b27068a143154.Name = "PreviousNextNumeric";
                    var Storyboard_ff50932ae5f744f199a6011e8495ee02 = new Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard();
                    var ObjectAnimationUsingKeyFrames_5f1ca52e977b47249b73462998e26e0e = new Bridge.global.Windows.UI.Xaml.Media.Animation.ObjectAnimationUsingKeyFrames();
                    Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard.SetTargetName(ObjectAnimationUsingKeyFrames_5f1ca52e977b47249b73462998e26e0e, "FirstPageButton");
                    var DiscreteObjectKeyFrame_5ec99764c91741dea201f0658743b8f3 = new Bridge.global.Windows.UI.Xaml.Media.Animation.DiscreteObjectKeyFrame();
                    DiscreteObjectKeyFrame_5ec99764c91741dea201f0658743b8f3.KeyTime = Bridge.global.Windows.UI.Xaml.Media.Animation.KeyTime.FromTimeSpan(new Bridge.global.System.TimeSpan(System.Int64(0)));
                    DiscreteObjectKeyFrame_5ec99764c91741dea201f0658743b8f3.Value = "Collapsed";

                    ObjectAnimationUsingKeyFrames_5f1ca52e977b47249b73462998e26e0e.KeyFrames.add(DiscreteObjectKeyFrame_5ec99764c91741dea201f0658743b8f3);


                    var ObjectAnimationUsingKeyFrames_18d0a4948c3642678329aa6f2e92700b = new Bridge.global.Windows.UI.Xaml.Media.Animation.ObjectAnimationUsingKeyFrames();
                    Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard.SetTargetName(ObjectAnimationUsingKeyFrames_18d0a4948c3642678329aa6f2e92700b, "LastPageButton");
                    var DiscreteObjectKeyFrame_4895f36a04e5445f9c866ec8ce005e49 = new Bridge.global.Windows.UI.Xaml.Media.Animation.DiscreteObjectKeyFrame();
                    DiscreteObjectKeyFrame_4895f36a04e5445f9c866ec8ce005e49.KeyTime = Bridge.global.Windows.UI.Xaml.Media.Animation.KeyTime.FromTimeSpan(new Bridge.global.System.TimeSpan(System.Int64(0)));
                    DiscreteObjectKeyFrame_4895f36a04e5445f9c866ec8ce005e49.Value = "Collapsed";

                    ObjectAnimationUsingKeyFrames_18d0a4948c3642678329aa6f2e92700b.KeyFrames.add(DiscreteObjectKeyFrame_4895f36a04e5445f9c866ec8ce005e49);


                    var ObjectAnimationUsingKeyFrames_86559b4733e94ef293cbc18965addb21 = new Bridge.global.Windows.UI.Xaml.Media.Animation.ObjectAnimationUsingKeyFrames();
                    Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard.SetTargetName(ObjectAnimationUsingKeyFrames_86559b4733e94ef293cbc18965addb21, "CurrentPageTextBox");
                    var DiscreteObjectKeyFrame_169919cee20b4bfda134fc3944816690 = new Bridge.global.Windows.UI.Xaml.Media.Animation.DiscreteObjectKeyFrame();
                    DiscreteObjectKeyFrame_169919cee20b4bfda134fc3944816690.KeyTime = Bridge.global.Windows.UI.Xaml.Media.Animation.KeyTime.FromTimeSpan(new Bridge.global.System.TimeSpan(System.Int64(0)));
                    DiscreteObjectKeyFrame_169919cee20b4bfda134fc3944816690.Value = "Collapsed";

                    ObjectAnimationUsingKeyFrames_86559b4733e94ef293cbc18965addb21.KeyFrames.add(DiscreteObjectKeyFrame_169919cee20b4bfda134fc3944816690);


                    var ObjectAnimationUsingKeyFrames_19d7f1f355fd4f47897f2d081e137ff7 = new Bridge.global.Windows.UI.Xaml.Media.Animation.ObjectAnimationUsingKeyFrames();
                    Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard.SetTargetName(ObjectAnimationUsingKeyFrames_19d7f1f355fd4f47897f2d081e137ff7, "PageDisplay");
                    var DiscreteObjectKeyFrame_a44bebfc0ba041c9b5d2122e403338dd = new Bridge.global.Windows.UI.Xaml.Media.Animation.DiscreteObjectKeyFrame();
                    DiscreteObjectKeyFrame_a44bebfc0ba041c9b5d2122e403338dd.KeyTime = Bridge.global.Windows.UI.Xaml.Media.Animation.KeyTime.FromTimeSpan(new Bridge.global.System.TimeSpan(System.Int64(0)));
                    DiscreteObjectKeyFrame_a44bebfc0ba041c9b5d2122e403338dd.Value = "Collapsed";

                    ObjectAnimationUsingKeyFrames_19d7f1f355fd4f47897f2d081e137ff7.KeyFrames.add(DiscreteObjectKeyFrame_a44bebfc0ba041c9b5d2122e403338dd);


                    Storyboard_ff50932ae5f744f199a6011e8495ee02.Children.add(ObjectAnimationUsingKeyFrames_5f1ca52e977b47249b73462998e26e0e);
                    Storyboard_ff50932ae5f744f199a6011e8495ee02.Children.add(ObjectAnimationUsingKeyFrames_18d0a4948c3642678329aa6f2e92700b);
                    Storyboard_ff50932ae5f744f199a6011e8495ee02.Children.add(ObjectAnimationUsingKeyFrames_86559b4733e94ef293cbc18965addb21);
                    Storyboard_ff50932ae5f744f199a6011e8495ee02.Children.add(ObjectAnimationUsingKeyFrames_19d7f1f355fd4f47897f2d081e137ff7);


                    VisualState_861d8139c4cd4f8bbb7b27068a143154.Storyboard = Storyboard_ff50932ae5f744f199a6011e8495ee02;


                    System.Array.add(VisualStateGroup_e44f95216530493888aedcd213f18b83.States, VisualState_0e1826d4ce224a8d86bfce7bd4614ef8, Object);
                    System.Array.add(VisualStateGroup_e44f95216530493888aedcd213f18b83.States, VisualState_a784fa3a663747d0964aa099f949c856, Object);
                    System.Array.add(VisualStateGroup_e44f95216530493888aedcd213f18b83.States, VisualState_1a6286817dd34f6e851dc74750f0ce98, Object);
                    System.Array.add(VisualStateGroup_e44f95216530493888aedcd213f18b83.States, VisualState_836618b99af240b2b37a67c2c976e259, Object);
                    System.Array.add(VisualStateGroup_e44f95216530493888aedcd213f18b83.States, VisualState_8647c530d5054e62aec9e9720e09c1ce, Object);
                    System.Array.add(VisualStateGroup_e44f95216530493888aedcd213f18b83.States, VisualState_861d8139c4cd4f8bbb7b27068a143154, Object);


                    templateOwner_ControlTemplate_c2c1fafddd0e45348f9697b3bfd04e92.INTERNAL_GetVisualStateGroups().add(VisualStateGroup_e44f95216530493888aedcd213f18b83);

                    var Border_04e7c6b3b6174d76998bf8782e87ce55 = new Bridge.global.Windows.UI.Xaml.Controls.Border();
                    Border_04e7c6b3b6174d76998bf8782e87ce55.MinHeight = 24.0;
                    Border_04e7c6b3b6174d76998bf8782e87ce55.VerticalAlignment = Bridge.global.Windows.UI.Xaml.VerticalAlignment.Bottom;
                    Border_04e7c6b3b6174d76998bf8782e87ce55.CornerRadius = new Bridge.global.Windows.UI.Xaml.CornerRadius.$ctor1(2);
                    var StackPanel_ace6805c89424815b7ba34f79da3e8d5 = new Bridge.global.Windows.UI.Xaml.Controls.StackPanel();
                    StackPanel_ace6805c89424815b7ba34f79da3e8d5.Orientation = Bridge.global.Windows.UI.Xaml.Controls.Orientation.Horizontal;
                    StackPanel_ace6805c89424815b7ba34f79da3e8d5.VerticalAlignment = Bridge.global.Windows.UI.Xaml.VerticalAlignment.Stretch;
                    templateOwner_ControlTemplate_c2c1fafddd0e45348f9697b3bfd04e92.RegisterName("FirstPageButton", Button_2ecf43a729774feda5635ec33b78e87b);
                    Button_2ecf43a729774feda5635ec33b78e87b.Name = "FirstPageButton";
                    Button_2ecf43a729774feda5635ec33b78e87b.Height = 20.0;
                    Button_2ecf43a729774feda5635ec33b78e87b.Width = 20.0;
                    Button_2ecf43a729774feda5635ec33b78e87b.Background = new Bridge.global.Windows.UI.Xaml.Media.SolidColorBrush.$ctor1(($t1 = new Bridge.global.Windows.UI.Color(), $t1.A = 0, $t1.R = 0, $t1.G = 0, $t1.B = 0, $t1));
                    Button_2ecf43a729774feda5635ec33b78e87b.Foreground = new Bridge.global.Windows.UI.Xaml.Media.SolidColorBrush.$ctor1(($t1 = new Bridge.global.Windows.UI.Color(), $t1.A = 255, $t1.R = 0, $t1.G = 0, $t1.B = 0, $t1));
                    Button_2ecf43a729774feda5635ec33b78e87b.BorderBrush = new Bridge.global.Windows.UI.Xaml.Media.SolidColorBrush.$ctor1(($t1 = new Bridge.global.Windows.UI.Color(), $t1.A = 255, $t1.R = 255, $t1.G = 255, $t1.B = 255, $t1));
                    Button_2ecf43a729774feda5635ec33b78e87b.BorderThickness = new Bridge.global.Windows.UI.Xaml.Thickness.$ctor1(1);
                    Button_2ecf43a729774feda5635ec33b78e87b.Padding = new Bridge.global.Windows.UI.Xaml.Thickness.$ctor1(1);
                    Button_2ecf43a729774feda5635ec33b78e87b.HorizontalAlignment = Bridge.global.Windows.UI.Xaml.HorizontalAlignment.Right;
                    Button_2ecf43a729774feda5635ec33b78e87b.VerticalAlignment = Bridge.global.Windows.UI.Xaml.VerticalAlignment.Center;
                    var Grid_ae4bfbeaec084a00bf82c4c06e07bb74 = new Bridge.global.Windows.UI.Xaml.Controls.Grid();
                    Grid_ae4bfbeaec084a00bf82c4c06e07bb74.Height = 9.0;
                    Grid_ae4bfbeaec084a00bf82c4c06e07bb74.Width = 8.0;
                    var Path_86eb17b87f0e4e7f83ee94401d82967b = new Bridge.global.Windows.UI.Xaml.Shapes.Path();
                    Path_86eb17b87f0e4e7f83ee94401d82967b.Stretch = Bridge.global.Windows.UI.Xaml.Media.Stretch.Fill;
                    Path_86eb17b87f0e4e7f83ee94401d82967b.Data = Bridge.cast(Bridge.global.DotNetForHtml5.Core.TypeFromStringConverters.ConvertFromInvariantString(Bridge.global.Windows.UI.Xaml.Media.Geometry, "M0,1 L1,0 L1,2 Z"), Windows.UI.Xaml.Media.Geometry);
                    Path_86eb17b87f0e4e7f83ee94401d82967b.Width = 5.0;
                    Path_86eb17b87f0e4e7f83ee94401d82967b.Height = 9.0;
                    Path_86eb17b87f0e4e7f83ee94401d82967b.HorizontalAlignment = Bridge.global.Windows.UI.Xaml.HorizontalAlignment.Right;
                    var Binding_29c2b1ba6a324836885eeae97294176e = new Bridge.global.Windows.UI.Xaml.Data.Binding.ctor();
                    Binding_29c2b1ba6a324836885eeae97294176e.Path = new Bridge.global.Windows.UI.Xaml.PropertyPath.ctor("Foreground");
                    var RelativeSource_861ae02e50ac47b7a97d7c69f667d0c1 = new Bridge.global.Windows.UI.Xaml.Data.RelativeSource.ctor();
                    RelativeSource_861ae02e50ac47b7a97d7c69f667d0c1.Mode = Bridge.global.Windows.UI.Xaml.Data.RelativeSourceMode.TemplatedParent;

                    Binding_29c2b1ba6a324836885eeae97294176e.RelativeSource = RelativeSource_861ae02e50ac47b7a97d7c69f667d0c1;


                    Binding_29c2b1ba6a324836885eeae97294176e.TemplateOwner = templateInstance_37f7df09e03649e7b9c6ab503fce8242;


                    var Rectangle_b4268b8cddb8426ab5fdbe08c7d86eff = new Bridge.global.Windows.UI.Xaml.Shapes.Rectangle();
                    Rectangle_b4268b8cddb8426ab5fdbe08c7d86eff.Width = 2.0;
                    Rectangle_b4268b8cddb8426ab5fdbe08c7d86eff.Height = 9.0;
                    Rectangle_b4268b8cddb8426ab5fdbe08c7d86eff.HorizontalAlignment = Bridge.global.Windows.UI.Xaml.HorizontalAlignment.Left;
                    var Binding_7021ab67c78f4bd2b2904d92bcfd640c = new Bridge.global.Windows.UI.Xaml.Data.Binding.ctor();
                    Binding_7021ab67c78f4bd2b2904d92bcfd640c.Path = new Bridge.global.Windows.UI.Xaml.PropertyPath.ctor("Foreground");
                    var RelativeSource_3120536d9eab4659848271cd160c81f8 = new Bridge.global.Windows.UI.Xaml.Data.RelativeSource.ctor();
                    RelativeSource_3120536d9eab4659848271cd160c81f8.Mode = Bridge.global.Windows.UI.Xaml.Data.RelativeSourceMode.TemplatedParent;

                    Binding_7021ab67c78f4bd2b2904d92bcfd640c.RelativeSource = RelativeSource_3120536d9eab4659848271cd160c81f8;


                    Binding_7021ab67c78f4bd2b2904d92bcfd640c.TemplateOwner = templateInstance_37f7df09e03649e7b9c6ab503fce8242;


                    Grid_ae4bfbeaec084a00bf82c4c06e07bb74.Children.add(Path_86eb17b87f0e4e7f83ee94401d82967b);
                    Grid_ae4bfbeaec084a00bf82c4c06e07bb74.Children.add(Rectangle_b4268b8cddb8426ab5fdbe08c7d86eff);


                    Button_2ecf43a729774feda5635ec33b78e87b.Content = Grid_ae4bfbeaec084a00bf82c4c06e07bb74;


                    templateOwner_ControlTemplate_c2c1fafddd0e45348f9697b3bfd04e92.RegisterName("PreviousPageButton", Button_7db9715dc6224087bb4e526aee5c7542);
                    Button_7db9715dc6224087bb4e526aee5c7542.Name = "PreviousPageButton";
                    Button_7db9715dc6224087bb4e526aee5c7542.Background = new Bridge.global.Windows.UI.Xaml.Media.SolidColorBrush.$ctor1(($t1 = new Bridge.global.Windows.UI.Color(), $t1.A = 0, $t1.R = 0, $t1.G = 0, $t1.B = 0, $t1));
                    Button_7db9715dc6224087bb4e526aee5c7542.Foreground = new Bridge.global.Windows.UI.Xaml.Media.SolidColorBrush.$ctor1(($t1 = new Bridge.global.Windows.UI.Color(), $t1.A = 255, $t1.R = 0, $t1.G = 0, $t1.B = 0, $t1));
                    Button_7db9715dc6224087bb4e526aee5c7542.BorderBrush = new Bridge.global.Windows.UI.Xaml.Media.SolidColorBrush.$ctor1(($t1 = new Bridge.global.Windows.UI.Color(), $t1.A = 255, $t1.R = 255, $t1.G = 255, $t1.B = 255, $t1));
                    Button_7db9715dc6224087bb4e526aee5c7542.BorderThickness = new Bridge.global.Windows.UI.Xaml.Thickness.$ctor1(1);
                    Button_7db9715dc6224087bb4e526aee5c7542.Padding = new Bridge.global.Windows.UI.Xaml.Thickness.$ctor1(1);
                    Button_7db9715dc6224087bb4e526aee5c7542.Height = 20.0;
                    Button_7db9715dc6224087bb4e526aee5c7542.Width = 20.0;
                    Button_7db9715dc6224087bb4e526aee5c7542.HorizontalAlignment = Bridge.global.Windows.UI.Xaml.HorizontalAlignment.Right;
                    Button_7db9715dc6224087bb4e526aee5c7542.VerticalAlignment = Bridge.global.Windows.UI.Xaml.VerticalAlignment.Center;
                    var Path_9602a2e31f994960b169fd1025d688d9 = new Bridge.global.Windows.UI.Xaml.Shapes.Path();
                    Path_9602a2e31f994960b169fd1025d688d9.Stretch = Bridge.global.Windows.UI.Xaml.Media.Stretch.Fill;
                    Path_9602a2e31f994960b169fd1025d688d9.Data = Bridge.cast(Bridge.global.DotNetForHtml5.Core.TypeFromStringConverters.ConvertFromInvariantString(Bridge.global.Windows.UI.Xaml.Media.Geometry, "M0,1 L1,0 L1,2 Z"), Windows.UI.Xaml.Media.Geometry);
                    Path_9602a2e31f994960b169fd1025d688d9.Width = 5.0;
                    Path_9602a2e31f994960b169fd1025d688d9.Height = 9.0;
                    Path_9602a2e31f994960b169fd1025d688d9.HorizontalAlignment = Bridge.global.Windows.UI.Xaml.HorizontalAlignment.Center;
                    Path_9602a2e31f994960b169fd1025d688d9.VerticalAlignment = Bridge.global.Windows.UI.Xaml.VerticalAlignment.Center;
                    var Binding_f60b82ce8f8b4c87a9668e972f1af2fa = new Bridge.global.Windows.UI.Xaml.Data.Binding.ctor();
                    Binding_f60b82ce8f8b4c87a9668e972f1af2fa.Path = new Bridge.global.Windows.UI.Xaml.PropertyPath.ctor("Foreground");
                    var RelativeSource_631feb93dce443fca5ad9e4e8fa6361c = new Bridge.global.Windows.UI.Xaml.Data.RelativeSource.ctor();
                    RelativeSource_631feb93dce443fca5ad9e4e8fa6361c.Mode = Bridge.global.Windows.UI.Xaml.Data.RelativeSourceMode.TemplatedParent;

                    Binding_f60b82ce8f8b4c87a9668e972f1af2fa.RelativeSource = RelativeSource_631feb93dce443fca5ad9e4e8fa6361c;


                    Binding_f60b82ce8f8b4c87a9668e972f1af2fa.TemplateOwner = templateInstance_37f7df09e03649e7b9c6ab503fce8242;


                    Button_7db9715dc6224087bb4e526aee5c7542.Content = Path_9602a2e31f994960b169fd1025d688d9;


                    templateOwner_ControlTemplate_c2c1fafddd0e45348f9697b3bfd04e92.RegisterName("Separator1", Border_583805b6b32e43a78d3bf62fba7f9f9b);
                    Border_583805b6b32e43a78d3bf62fba7f9f9b.Name = "Separator1";
                    Border_583805b6b32e43a78d3bf62fba7f9f9b.Width = 2.0;
                    Border_583805b6b32e43a78d3bf62fba7f9f9b.Background = new Bridge.global.Windows.UI.Xaml.Media.SolidColorBrush.$ctor1(($t1 = new Bridge.global.Windows.UI.Color(), $t1.A = 255, $t1.R = 204, $t1.G = 209, $t1.B = 214, $t1));
                    Border_583805b6b32e43a78d3bf62fba7f9f9b.Margin = new Bridge.global.Windows.UI.Xaml.Thickness.$ctor2(0, 3, 0, 3);
                    var Binding_2e331a4c2df844e5a552400cbe011e19 = new Bridge.global.Windows.UI.Xaml.Data.Binding.ctor();
                    Binding_2e331a4c2df844e5a552400cbe011e19.Path = new Bridge.global.Windows.UI.Xaml.PropertyPath.ctor("BorderBrush");
                    var RelativeSource_2e1db9e8b71249a7aa6826282a58fcfd = new Bridge.global.Windows.UI.Xaml.Data.RelativeSource.ctor();
                    RelativeSource_2e1db9e8b71249a7aa6826282a58fcfd.Mode = Bridge.global.Windows.UI.Xaml.Data.RelativeSourceMode.TemplatedParent;

                    Binding_2e331a4c2df844e5a552400cbe011e19.RelativeSource = RelativeSource_2e1db9e8b71249a7aa6826282a58fcfd;


                    Binding_2e331a4c2df844e5a552400cbe011e19.TemplateOwner = templateInstance_37f7df09e03649e7b9c6ab503fce8242;


                    templateOwner_ControlTemplate_c2c1fafddd0e45348f9697b3bfd04e92.RegisterName("NumericButtonPanel", StackPanel_b58f85e120db44f6bd5370a6d599972d);
                    StackPanel_b58f85e120db44f6bd5370a6d599972d.Name = "NumericButtonPanel";
                    StackPanel_b58f85e120db44f6bd5370a6d599972d.Orientation = Bridge.global.Windows.UI.Xaml.Controls.Orientation.Horizontal;
                    StackPanel_b58f85e120db44f6bd5370a6d599972d.Margin = new Bridge.global.Windows.UI.Xaml.Thickness.$ctor1(1);

                    templateOwner_ControlTemplate_c2c1fafddd0e45348f9697b3bfd04e92.RegisterName("PageDisplay", StackPanel_5039ccb47ff04b0f926057007c69e386);
                    StackPanel_5039ccb47ff04b0f926057007c69e386.Name = "PageDisplay";
                    StackPanel_5039ccb47ff04b0f926057007c69e386.Orientation = Bridge.global.Windows.UI.Xaml.Controls.Orientation.Horizontal;
                    templateOwner_ControlTemplate_c2c1fafddd0e45348f9697b3bfd04e92.RegisterName("CurrentPagePrefixTextBlock", TextBlock_47ec22267bb2413c95c0a2aa0c044f69);
                    TextBlock_47ec22267bb2413c95c0a2aa0c044f69.Name = "CurrentPagePrefixTextBlock";
                    TextBlock_47ec22267bb2413c95c0a2aa0c044f69.Width = Number.NaN;
                    TextBlock_47ec22267bb2413c95c0a2aa0c044f69.VerticalAlignment = Bridge.global.Windows.UI.Xaml.VerticalAlignment.Center;
                    TextBlock_47ec22267bb2413c95c0a2aa0c044f69.Margin = new Bridge.global.Windows.UI.Xaml.Thickness.$ctor2(4, 0, 0, 0);
                    var Binding_8df3b5f8414b4a85ac26bb1dbc3f07a1 = new Bridge.global.Windows.UI.Xaml.Data.Binding.ctor();
                    Binding_8df3b5f8414b4a85ac26bb1dbc3f07a1.Path = new Bridge.global.Windows.UI.Xaml.PropertyPath.ctor("Foreground");
                    var RelativeSource_d9a1e9bdf9264c80b905a125948afccc = new Bridge.global.Windows.UI.Xaml.Data.RelativeSource.ctor();
                    RelativeSource_d9a1e9bdf9264c80b905a125948afccc.Mode = Bridge.global.Windows.UI.Xaml.Data.RelativeSourceMode.TemplatedParent;

                    Binding_8df3b5f8414b4a85ac26bb1dbc3f07a1.RelativeSource = RelativeSource_d9a1e9bdf9264c80b905a125948afccc;


                    Binding_8df3b5f8414b4a85ac26bb1dbc3f07a1.TemplateOwner = templateInstance_37f7df09e03649e7b9c6ab503fce8242;


                    templateOwner_ControlTemplate_c2c1fafddd0e45348f9697b3bfd04e92.RegisterName("CurrentPageTextBox", TextBox_5e5b54f2d3ac42db9b91623d8ae9b49a);
                    TextBox_5e5b54f2d3ac42db9b91623d8ae9b49a.Name = "CurrentPageTextBox";
                    TextBox_5e5b54f2d3ac42db9b91623d8ae9b49a.TextWrapping = Bridge.global.Windows.UI.Xaml.TextWrapping.Wrap;
                    TextBox_5e5b54f2d3ac42db9b91623d8ae9b49a.Width = 40.0;
                    TextBox_5e5b54f2d3ac42db9b91623d8ae9b49a.Height = Number.NaN;
                    TextBox_5e5b54f2d3ac42db9b91623d8ae9b49a.VerticalAlignment = Bridge.global.Windows.UI.Xaml.VerticalAlignment.Center;
                    TextBox_5e5b54f2d3ac42db9b91623d8ae9b49a.Margin = new Bridge.global.Windows.UI.Xaml.Thickness.$ctor2(4, 2, 4, 2);
                    var Binding_5e573e7a7a77401ebb7a99dd88735aec = new Bridge.global.Windows.UI.Xaml.Data.Binding.ctor();
                    Binding_5e573e7a7a77401ebb7a99dd88735aec.Path = new Bridge.global.Windows.UI.Xaml.PropertyPath.ctor("Foreground");
                    var RelativeSource_044de882abe841e187da2ea0ea47c8ec = new Bridge.global.Windows.UI.Xaml.Data.RelativeSource.ctor();
                    RelativeSource_044de882abe841e187da2ea0ea47c8ec.Mode = Bridge.global.Windows.UI.Xaml.Data.RelativeSourceMode.TemplatedParent;

                    Binding_5e573e7a7a77401ebb7a99dd88735aec.RelativeSource = RelativeSource_044de882abe841e187da2ea0ea47c8ec;


                    Binding_5e573e7a7a77401ebb7a99dd88735aec.TemplateOwner = templateInstance_37f7df09e03649e7b9c6ab503fce8242;

                    var Binding_c8d1f0eb91ad4922b4b5bcef922ea0a1 = new Bridge.global.Windows.UI.Xaml.Data.Binding.ctor();
                    Binding_c8d1f0eb91ad4922b4b5bcef922ea0a1.Path = new Bridge.global.Windows.UI.Xaml.PropertyPath.ctor("BorderBrush");
                    var RelativeSource_8e95215db7154a8ba14ab0928707f6c7 = new Bridge.global.Windows.UI.Xaml.Data.RelativeSource.ctor();
                    RelativeSource_8e95215db7154a8ba14ab0928707f6c7.Mode = Bridge.global.Windows.UI.Xaml.Data.RelativeSourceMode.TemplatedParent;

                    Binding_c8d1f0eb91ad4922b4b5bcef922ea0a1.RelativeSource = RelativeSource_8e95215db7154a8ba14ab0928707f6c7;


                    Binding_c8d1f0eb91ad4922b4b5bcef922ea0a1.TemplateOwner = templateInstance_37f7df09e03649e7b9c6ab503fce8242;


                    templateOwner_ControlTemplate_c2c1fafddd0e45348f9697b3bfd04e92.RegisterName("CurrentPageSuffixTextBlock", TextBlock_5c303ae12396405f98da5911db622613);
                    TextBlock_5c303ae12396405f98da5911db622613.Name = "CurrentPageSuffixTextBlock";
                    TextBlock_5c303ae12396405f98da5911db622613.Width = Number.NaN;
                    TextBlock_5c303ae12396405f98da5911db622613.VerticalAlignment = Bridge.global.Windows.UI.Xaml.VerticalAlignment.Center;
                    TextBlock_5c303ae12396405f98da5911db622613.Margin = new Bridge.global.Windows.UI.Xaml.Thickness.$ctor2(0, 0, 4, 0);
                    var Binding_5c8aa652d7b84c3194ddf1579a40ac67 = new Bridge.global.Windows.UI.Xaml.Data.Binding.ctor();
                    Binding_5c8aa652d7b84c3194ddf1579a40ac67.Path = new Bridge.global.Windows.UI.Xaml.PropertyPath.ctor("Foreground");
                    var RelativeSource_1fde550e8e6a4024bab1a8f208d0f4c6 = new Bridge.global.Windows.UI.Xaml.Data.RelativeSource.ctor();
                    RelativeSource_1fde550e8e6a4024bab1a8f208d0f4c6.Mode = Bridge.global.Windows.UI.Xaml.Data.RelativeSourceMode.TemplatedParent;

                    Binding_5c8aa652d7b84c3194ddf1579a40ac67.RelativeSource = RelativeSource_1fde550e8e6a4024bab1a8f208d0f4c6;


                    Binding_5c8aa652d7b84c3194ddf1579a40ac67.TemplateOwner = templateInstance_37f7df09e03649e7b9c6ab503fce8242;


                    StackPanel_5039ccb47ff04b0f926057007c69e386.Children.add(TextBlock_47ec22267bb2413c95c0a2aa0c044f69);
                    StackPanel_5039ccb47ff04b0f926057007c69e386.Children.add(TextBox_5e5b54f2d3ac42db9b91623d8ae9b49a);
                    StackPanel_5039ccb47ff04b0f926057007c69e386.Children.add(TextBlock_5c303ae12396405f98da5911db622613);


                    templateOwner_ControlTemplate_c2c1fafddd0e45348f9697b3bfd04e92.RegisterName("Separator2", Border_d484f9589f234b718b2106b85893789a);
                    Border_d484f9589f234b718b2106b85893789a.Name = "Separator2";
                    Border_d484f9589f234b718b2106b85893789a.Width = 2.0;
                    Border_d484f9589f234b718b2106b85893789a.Background = new Bridge.global.Windows.UI.Xaml.Media.SolidColorBrush.$ctor1(($t1 = new Bridge.global.Windows.UI.Color(), $t1.A = 255, $t1.R = 204, $t1.G = 209, $t1.B = 214, $t1));
                    Border_d484f9589f234b718b2106b85893789a.Margin = new Bridge.global.Windows.UI.Xaml.Thickness.$ctor2(0, 3, 0, 3);
                    var Binding_8dc6922019cb46d796cab3c0df645583 = new Bridge.global.Windows.UI.Xaml.Data.Binding.ctor();
                    Binding_8dc6922019cb46d796cab3c0df645583.Path = new Bridge.global.Windows.UI.Xaml.PropertyPath.ctor("BorderBrush");
                    var RelativeSource_68013a7088f54b6dac4f7685b9af499c = new Bridge.global.Windows.UI.Xaml.Data.RelativeSource.ctor();
                    RelativeSource_68013a7088f54b6dac4f7685b9af499c.Mode = Bridge.global.Windows.UI.Xaml.Data.RelativeSourceMode.TemplatedParent;

                    Binding_8dc6922019cb46d796cab3c0df645583.RelativeSource = RelativeSource_68013a7088f54b6dac4f7685b9af499c;


                    Binding_8dc6922019cb46d796cab3c0df645583.TemplateOwner = templateInstance_37f7df09e03649e7b9c6ab503fce8242;


                    templateOwner_ControlTemplate_c2c1fafddd0e45348f9697b3bfd04e92.RegisterName("NextPageButton", Button_975225bcbd324697a58acaa112f593d7);
                    Button_975225bcbd324697a58acaa112f593d7.Name = "NextPageButton";
                    Button_975225bcbd324697a58acaa112f593d7.Background = new Bridge.global.Windows.UI.Xaml.Media.SolidColorBrush.$ctor1(($t1 = new Bridge.global.Windows.UI.Color(), $t1.A = 0, $t1.R = 0, $t1.G = 0, $t1.B = 0, $t1));
                    Button_975225bcbd324697a58acaa112f593d7.Foreground = new Bridge.global.Windows.UI.Xaml.Media.SolidColorBrush.$ctor1(($t1 = new Bridge.global.Windows.UI.Color(), $t1.A = 255, $t1.R = 0, $t1.G = 0, $t1.B = 0, $t1));
                    Button_975225bcbd324697a58acaa112f593d7.BorderBrush = new Bridge.global.Windows.UI.Xaml.Media.SolidColorBrush.$ctor1(($t1 = new Bridge.global.Windows.UI.Color(), $t1.A = 255, $t1.R = 255, $t1.G = 255, $t1.B = 255, $t1));
                    Button_975225bcbd324697a58acaa112f593d7.BorderThickness = new Bridge.global.Windows.UI.Xaml.Thickness.$ctor1(1);
                    Button_975225bcbd324697a58acaa112f593d7.Padding = new Bridge.global.Windows.UI.Xaml.Thickness.$ctor1(1);
                    Button_975225bcbd324697a58acaa112f593d7.Height = 20.0;
                    Button_975225bcbd324697a58acaa112f593d7.Width = 20.0;
                    Button_975225bcbd324697a58acaa112f593d7.HorizontalAlignment = Bridge.global.Windows.UI.Xaml.HorizontalAlignment.Right;
                    Button_975225bcbd324697a58acaa112f593d7.VerticalAlignment = Bridge.global.Windows.UI.Xaml.VerticalAlignment.Center;
                    var Path_6b437ad3cd1547bca7785187719e832b = new Bridge.global.Windows.UI.Xaml.Shapes.Path();
                    Path_6b437ad3cd1547bca7785187719e832b.Stretch = Bridge.global.Windows.UI.Xaml.Media.Stretch.Fill;
                    Path_6b437ad3cd1547bca7785187719e832b.Data = Bridge.cast(Bridge.global.DotNetForHtml5.Core.TypeFromStringConverters.ConvertFromInvariantString(Bridge.global.Windows.UI.Xaml.Media.Geometry, "M0,0 L1,1 L0,2 Z"), Windows.UI.Xaml.Media.Geometry);
                    Path_6b437ad3cd1547bca7785187719e832b.Width = 5.0;
                    Path_6b437ad3cd1547bca7785187719e832b.Height = 9.0;
                    Path_6b437ad3cd1547bca7785187719e832b.HorizontalAlignment = Bridge.global.Windows.UI.Xaml.HorizontalAlignment.Center;
                    Path_6b437ad3cd1547bca7785187719e832b.VerticalAlignment = Bridge.global.Windows.UI.Xaml.VerticalAlignment.Center;
                    var Binding_f36d9e36f454460ea33d9a10078a54ff = new Bridge.global.Windows.UI.Xaml.Data.Binding.ctor();
                    Binding_f36d9e36f454460ea33d9a10078a54ff.Path = new Bridge.global.Windows.UI.Xaml.PropertyPath.ctor("Foreground");
                    var RelativeSource_5383e4c17e494b3bad11ae80406562c6 = new Bridge.global.Windows.UI.Xaml.Data.RelativeSource.ctor();
                    RelativeSource_5383e4c17e494b3bad11ae80406562c6.Mode = Bridge.global.Windows.UI.Xaml.Data.RelativeSourceMode.TemplatedParent;

                    Binding_f36d9e36f454460ea33d9a10078a54ff.RelativeSource = RelativeSource_5383e4c17e494b3bad11ae80406562c6;


                    Binding_f36d9e36f454460ea33d9a10078a54ff.TemplateOwner = templateInstance_37f7df09e03649e7b9c6ab503fce8242;


                    Button_975225bcbd324697a58acaa112f593d7.Content = Path_6b437ad3cd1547bca7785187719e832b;


                    templateOwner_ControlTemplate_c2c1fafddd0e45348f9697b3bfd04e92.RegisterName("LastPageButton", Button_b6cb766b3f6f419eb6a55dd4be55411e);
                    Button_b6cb766b3f6f419eb6a55dd4be55411e.Name = "LastPageButton";
                    Button_b6cb766b3f6f419eb6a55dd4be55411e.Background = new Bridge.global.Windows.UI.Xaml.Media.SolidColorBrush.$ctor1(($t1 = new Bridge.global.Windows.UI.Color(), $t1.A = 0, $t1.R = 0, $t1.G = 0, $t1.B = 0, $t1));
                    Button_b6cb766b3f6f419eb6a55dd4be55411e.Foreground = new Bridge.global.Windows.UI.Xaml.Media.SolidColorBrush.$ctor1(($t1 = new Bridge.global.Windows.UI.Color(), $t1.A = 255, $t1.R = 0, $t1.G = 0, $t1.B = 0, $t1));
                    Button_b6cb766b3f6f419eb6a55dd4be55411e.BorderBrush = new Bridge.global.Windows.UI.Xaml.Media.SolidColorBrush.$ctor1(($t1 = new Bridge.global.Windows.UI.Color(), $t1.A = 255, $t1.R = 255, $t1.G = 255, $t1.B = 255, $t1));
                    Button_b6cb766b3f6f419eb6a55dd4be55411e.BorderThickness = new Bridge.global.Windows.UI.Xaml.Thickness.$ctor1(1);
                    Button_b6cb766b3f6f419eb6a55dd4be55411e.Padding = new Bridge.global.Windows.UI.Xaml.Thickness.$ctor1(1);
                    Button_b6cb766b3f6f419eb6a55dd4be55411e.Height = 20.0;
                    Button_b6cb766b3f6f419eb6a55dd4be55411e.Width = 20.0;
                    Button_b6cb766b3f6f419eb6a55dd4be55411e.HorizontalAlignment = Bridge.global.Windows.UI.Xaml.HorizontalAlignment.Right;
                    Button_b6cb766b3f6f419eb6a55dd4be55411e.VerticalAlignment = Bridge.global.Windows.UI.Xaml.VerticalAlignment.Center;
                    var Grid_67f41f8fc5cf4b8087971d35b618a3d8 = new Bridge.global.Windows.UI.Xaml.Controls.Grid();
                    Grid_67f41f8fc5cf4b8087971d35b618a3d8.Height = 9.0;
                    Grid_67f41f8fc5cf4b8087971d35b618a3d8.Width = 8.0;
                    var Path_b5815901a10f4038b74ee60df3d6f50c = new Bridge.global.Windows.UI.Xaml.Shapes.Path();
                    Path_b5815901a10f4038b74ee60df3d6f50c.Stretch = Bridge.global.Windows.UI.Xaml.Media.Stretch.Fill;
                    Path_b5815901a10f4038b74ee60df3d6f50c.Data = Bridge.cast(Bridge.global.DotNetForHtml5.Core.TypeFromStringConverters.ConvertFromInvariantString(Bridge.global.Windows.UI.Xaml.Media.Geometry, "M0,0 L1,1 L0,2 Z"), Windows.UI.Xaml.Media.Geometry);
                    Path_b5815901a10f4038b74ee60df3d6f50c.Width = 5.0;
                    Path_b5815901a10f4038b74ee60df3d6f50c.Height = 9.0;
                    Path_b5815901a10f4038b74ee60df3d6f50c.HorizontalAlignment = Bridge.global.Windows.UI.Xaml.HorizontalAlignment.Left;
                    var Binding_ec8a3deda786475d89d16a7114909091 = new Bridge.global.Windows.UI.Xaml.Data.Binding.ctor();
                    Binding_ec8a3deda786475d89d16a7114909091.Path = new Bridge.global.Windows.UI.Xaml.PropertyPath.ctor("Foreground");
                    var RelativeSource_9bc5d6d265174031b531373dd39b550b = new Bridge.global.Windows.UI.Xaml.Data.RelativeSource.ctor();
                    RelativeSource_9bc5d6d265174031b531373dd39b550b.Mode = Bridge.global.Windows.UI.Xaml.Data.RelativeSourceMode.TemplatedParent;

                    Binding_ec8a3deda786475d89d16a7114909091.RelativeSource = RelativeSource_9bc5d6d265174031b531373dd39b550b;


                    Binding_ec8a3deda786475d89d16a7114909091.TemplateOwner = templateInstance_37f7df09e03649e7b9c6ab503fce8242;


                    var Rectangle_bb2b8e80d0a3484cb498318318d94a30 = new Bridge.global.Windows.UI.Xaml.Shapes.Rectangle();
                    Rectangle_bb2b8e80d0a3484cb498318318d94a30.Width = 2.0;
                    Rectangle_bb2b8e80d0a3484cb498318318d94a30.Height = 9.0;
                    Rectangle_bb2b8e80d0a3484cb498318318d94a30.HorizontalAlignment = Bridge.global.Windows.UI.Xaml.HorizontalAlignment.Right;
                    var Binding_1f89daa71e684deaa593e958c84bf463 = new Bridge.global.Windows.UI.Xaml.Data.Binding.ctor();
                    Binding_1f89daa71e684deaa593e958c84bf463.Path = new Bridge.global.Windows.UI.Xaml.PropertyPath.ctor("Foreground");
                    var RelativeSource_780df8a1318442ac8bdec4a6c8ef20a2 = new Bridge.global.Windows.UI.Xaml.Data.RelativeSource.ctor();
                    RelativeSource_780df8a1318442ac8bdec4a6c8ef20a2.Mode = Bridge.global.Windows.UI.Xaml.Data.RelativeSourceMode.TemplatedParent;

                    Binding_1f89daa71e684deaa593e958c84bf463.RelativeSource = RelativeSource_780df8a1318442ac8bdec4a6c8ef20a2;


                    Binding_1f89daa71e684deaa593e958c84bf463.TemplateOwner = templateInstance_37f7df09e03649e7b9c6ab503fce8242;


                    Grid_67f41f8fc5cf4b8087971d35b618a3d8.Children.add(Path_b5815901a10f4038b74ee60df3d6f50c);
                    Grid_67f41f8fc5cf4b8087971d35b618a3d8.Children.add(Rectangle_bb2b8e80d0a3484cb498318318d94a30);


                    Button_b6cb766b3f6f419eb6a55dd4be55411e.Content = Grid_67f41f8fc5cf4b8087971d35b618a3d8;


                    StackPanel_ace6805c89424815b7ba34f79da3e8d5.Children.add(Button_2ecf43a729774feda5635ec33b78e87b);
                    StackPanel_ace6805c89424815b7ba34f79da3e8d5.Children.add(Button_7db9715dc6224087bb4e526aee5c7542);
                    StackPanel_ace6805c89424815b7ba34f79da3e8d5.Children.add(Border_583805b6b32e43a78d3bf62fba7f9f9b);
                    StackPanel_ace6805c89424815b7ba34f79da3e8d5.Children.add(StackPanel_b58f85e120db44f6bd5370a6d599972d);
                    StackPanel_ace6805c89424815b7ba34f79da3e8d5.Children.add(StackPanel_5039ccb47ff04b0f926057007c69e386);
                    StackPanel_ace6805c89424815b7ba34f79da3e8d5.Children.add(Border_d484f9589f234b718b2106b85893789a);
                    StackPanel_ace6805c89424815b7ba34f79da3e8d5.Children.add(Button_975225bcbd324697a58acaa112f593d7);
                    StackPanel_ace6805c89424815b7ba34f79da3e8d5.Children.add(Button_b6cb766b3f6f419eb6a55dd4be55411e);

                    var Binding_83ecd1791dff4956951dcae948bd1fa3 = new Bridge.global.Windows.UI.Xaml.Data.Binding.ctor();
                    Binding_83ecd1791dff4956951dcae948bd1fa3.Path = new Bridge.global.Windows.UI.Xaml.PropertyPath.ctor("HorizontalContentAlignment");
                    var RelativeSource_51bd04d2633245bc9b6bd9b44e40886c = new Bridge.global.Windows.UI.Xaml.Data.RelativeSource.ctor();
                    RelativeSource_51bd04d2633245bc9b6bd9b44e40886c.Mode = Bridge.global.Windows.UI.Xaml.Data.RelativeSourceMode.TemplatedParent;

                    Binding_83ecd1791dff4956951dcae948bd1fa3.RelativeSource = RelativeSource_51bd04d2633245bc9b6bd9b44e40886c;


                    Binding_83ecd1791dff4956951dcae948bd1fa3.TemplateOwner = templateInstance_37f7df09e03649e7b9c6ab503fce8242;


                    Border_04e7c6b3b6174d76998bf8782e87ce55.Child = StackPanel_ace6805c89424815b7ba34f79da3e8d5;

                    var Binding_adabfe606e104ecca00a25355d504122 = new Bridge.global.Windows.UI.Xaml.Data.Binding.ctor();
                    Binding_adabfe606e104ecca00a25355d504122.Path = new Bridge.global.Windows.UI.Xaml.PropertyPath.ctor("Background");
                    var RelativeSource_cae6d436a0764983931676c30a9c8f29 = new Bridge.global.Windows.UI.Xaml.Data.RelativeSource.ctor();
                    RelativeSource_cae6d436a0764983931676c30a9c8f29.Mode = Bridge.global.Windows.UI.Xaml.Data.RelativeSourceMode.TemplatedParent;

                    Binding_adabfe606e104ecca00a25355d504122.RelativeSource = RelativeSource_cae6d436a0764983931676c30a9c8f29;


                    Binding_adabfe606e104ecca00a25355d504122.TemplateOwner = templateInstance_37f7df09e03649e7b9c6ab503fce8242;

                    var Binding_65f2f242c720415294313944ff7882db = new Bridge.global.Windows.UI.Xaml.Data.Binding.ctor();
                    Binding_65f2f242c720415294313944ff7882db.Path = new Bridge.global.Windows.UI.Xaml.PropertyPath.ctor("BorderBrush");
                    var RelativeSource_c3e094f62d0a44c2aa9ec42d07aebcf8 = new Bridge.global.Windows.UI.Xaml.Data.RelativeSource.ctor();
                    RelativeSource_c3e094f62d0a44c2aa9ec42d07aebcf8.Mode = Bridge.global.Windows.UI.Xaml.Data.RelativeSourceMode.TemplatedParent;

                    Binding_65f2f242c720415294313944ff7882db.RelativeSource = RelativeSource_c3e094f62d0a44c2aa9ec42d07aebcf8;


                    Binding_65f2f242c720415294313944ff7882db.TemplateOwner = templateInstance_37f7df09e03649e7b9c6ab503fce8242;

                    var Binding_1f660acdcbfb438888225417b64adb8c = new Bridge.global.Windows.UI.Xaml.Data.Binding.ctor();
                    Binding_1f660acdcbfb438888225417b64adb8c.Path = new Bridge.global.Windows.UI.Xaml.PropertyPath.ctor("BorderThickness");
                    var RelativeSource_e785501809da4585a58e1ad8d43dc517 = new Bridge.global.Windows.UI.Xaml.Data.RelativeSource.ctor();
                    RelativeSource_e785501809da4585a58e1ad8d43dc517.Mode = Bridge.global.Windows.UI.Xaml.Data.RelativeSourceMode.TemplatedParent;

                    Binding_1f660acdcbfb438888225417b64adb8c.RelativeSource = RelativeSource_e785501809da4585a58e1ad8d43dc517;


                    Binding_1f660acdcbfb438888225417b64adb8c.TemplateOwner = templateInstance_37f7df09e03649e7b9c6ab503fce8242;

                    var Binding_4123296ec67a4eeda090d835e6de0ab3 = new Bridge.global.Windows.UI.Xaml.Data.Binding.ctor();
                    Binding_4123296ec67a4eeda090d835e6de0ab3.Path = new Bridge.global.Windows.UI.Xaml.PropertyPath.ctor("Padding");
                    var RelativeSource_e0fb0df27ee9416986b4fc69d1c9be0f = new Bridge.global.Windows.UI.Xaml.Data.RelativeSource.ctor();
                    RelativeSource_e0fb0df27ee9416986b4fc69d1c9be0f.Mode = Bridge.global.Windows.UI.Xaml.Data.RelativeSourceMode.TemplatedParent;

                    Binding_4123296ec67a4eeda090d835e6de0ab3.RelativeSource = RelativeSource_e0fb0df27ee9416986b4fc69d1c9be0f;


                    Binding_4123296ec67a4eeda090d835e6de0ab3.TemplateOwner = templateInstance_37f7df09e03649e7b9c6ab503fce8242;


                    Grid_0ff886fd41ef49cb84afb3a4b3d1dc0a.Children.add(Border_04e7c6b3b6174d76998bf8782e87ce55);


                    Bridge.global.Windows.UI.Xaml.Data.BindingOperations.SetBinding(Path_86eb17b87f0e4e7f83ee94401d82967b, Bridge.global.Windows.UI.Xaml.Shapes.Shape.FillProperty, Binding_29c2b1ba6a324836885eeae97294176e);
                    Bridge.global.Windows.UI.Xaml.Data.BindingOperations.SetBinding(Rectangle_b4268b8cddb8426ab5fdbe08c7d86eff, Bridge.global.Windows.UI.Xaml.Shapes.Shape.FillProperty, Binding_7021ab67c78f4bd2b2904d92bcfd640c);
                    Bridge.global.Windows.UI.Xaml.Data.BindingOperations.SetBinding(Path_9602a2e31f994960b169fd1025d688d9, Bridge.global.Windows.UI.Xaml.Shapes.Shape.FillProperty, Binding_f60b82ce8f8b4c87a9668e972f1af2fa);
                    Bridge.global.Windows.UI.Xaml.Data.BindingOperations.SetBinding(Border_583805b6b32e43a78d3bf62fba7f9f9b, Bridge.global.Windows.UI.Xaml.Controls.Border.BorderBrushProperty, Binding_2e331a4c2df844e5a552400cbe011e19);
                    Bridge.global.Windows.UI.Xaml.Data.BindingOperations.SetBinding(TextBlock_47ec22267bb2413c95c0a2aa0c044f69, Bridge.global.Windows.UI.Xaml.Controls.Control.ForegroundProperty, Binding_8df3b5f8414b4a85ac26bb1dbc3f07a1);
                    Bridge.global.Windows.UI.Xaml.Data.BindingOperations.SetBinding(TextBox_5e5b54f2d3ac42db9b91623d8ae9b49a, Bridge.global.Windows.UI.Xaml.Controls.Control.ForegroundProperty, Binding_5e573e7a7a77401ebb7a99dd88735aec);
                    Bridge.global.Windows.UI.Xaml.Data.BindingOperations.SetBinding(TextBox_5e5b54f2d3ac42db9b91623d8ae9b49a, Bridge.global.Windows.UI.Xaml.Controls.Control.BorderBrushProperty, Binding_c8d1f0eb91ad4922b4b5bcef922ea0a1);
                    Bridge.global.Windows.UI.Xaml.Data.BindingOperations.SetBinding(TextBlock_5c303ae12396405f98da5911db622613, Bridge.global.Windows.UI.Xaml.Controls.Control.ForegroundProperty, Binding_5c8aa652d7b84c3194ddf1579a40ac67);
                    Bridge.global.Windows.UI.Xaml.Data.BindingOperations.SetBinding(Border_d484f9589f234b718b2106b85893789a, Bridge.global.Windows.UI.Xaml.Controls.Border.BorderBrushProperty, Binding_8dc6922019cb46d796cab3c0df645583);
                    Bridge.global.Windows.UI.Xaml.Data.BindingOperations.SetBinding(Path_6b437ad3cd1547bca7785187719e832b, Bridge.global.Windows.UI.Xaml.Shapes.Shape.FillProperty, Binding_f36d9e36f454460ea33d9a10078a54ff);
                    Bridge.global.Windows.UI.Xaml.Data.BindingOperations.SetBinding(Path_b5815901a10f4038b74ee60df3d6f50c, Bridge.global.Windows.UI.Xaml.Shapes.Shape.FillProperty, Binding_ec8a3deda786475d89d16a7114909091);
                    Bridge.global.Windows.UI.Xaml.Data.BindingOperations.SetBinding(Rectangle_bb2b8e80d0a3484cb498318318d94a30, Bridge.global.Windows.UI.Xaml.Shapes.Shape.FillProperty, Binding_1f89daa71e684deaa593e958c84bf463);
                    Bridge.global.Windows.UI.Xaml.Data.BindingOperations.SetBinding(StackPanel_ace6805c89424815b7ba34f79da3e8d5, Bridge.global.Windows.UI.Xaml.FrameworkElement.HorizontalAlignmentProperty, Binding_83ecd1791dff4956951dcae948bd1fa3);
                    Bridge.global.Windows.UI.Xaml.Data.BindingOperations.SetBinding(Border_04e7c6b3b6174d76998bf8782e87ce55, Bridge.global.Windows.UI.Xaml.Controls.Border.BackgroundProperty, Binding_adabfe606e104ecca00a25355d504122);
                    Bridge.global.Windows.UI.Xaml.Data.BindingOperations.SetBinding(Border_04e7c6b3b6174d76998bf8782e87ce55, Bridge.global.Windows.UI.Xaml.Controls.Border.BorderBrushProperty, Binding_65f2f242c720415294313944ff7882db);
                    Bridge.global.Windows.UI.Xaml.Data.BindingOperations.SetBinding(Border_04e7c6b3b6174d76998bf8782e87ce55, Bridge.global.Windows.UI.Xaml.Controls.Border.BorderThicknessProperty, Binding_1f660acdcbfb438888225417b64adb8c);
                    Bridge.global.Windows.UI.Xaml.Data.BindingOperations.SetBinding(Border_04e7c6b3b6174d76998bf8782e87ce55, Bridge.global.Windows.UI.Xaml.Controls.Border.PaddingProperty, Binding_4123296ec67a4eeda090d835e6de0ab3);

                    Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard.SetTargetProperty(ObjectAnimationUsingKeyFrames_eb9ea2a1cd5e4a49861eb516294d9bf7, new Bridge.global.Windows.UI.Xaml.PropertyPath.$ctor1("Visibility", "Visibility", Bridge.fn.cacheBind(this, this.accessVisualStateProperty_4420ecb444054b3da1afee49e54e96eb), Bridge.fn.cacheBind(this, this.setVisualStateProperty_4420ecb444054b3da1afee49e54e96eb), Bridge.fn.cacheBind(this, this.setAnimationVisualStateProperty_4420ecb444054b3da1afee49e54e96eb), Bridge.fn.cacheBind(this, this.setLocalVisualStateProperty_4420ecb444054b3da1afee49e54e96eb), Bridge.fn.cacheBind(this, this.getVisualStateProperty_4420ecb444054b3da1afee49e54e96eb)));
                    Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard.SetTarget(ObjectAnimationUsingKeyFrames_eb9ea2a1cd5e4a49861eb516294d9bf7, Button_975225bcbd324697a58acaa112f593d7);


                    Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard.SetTargetProperty(ObjectAnimationUsingKeyFrames_c20a3d4ab21e4d6782b38ede1a0b7439, new Bridge.global.Windows.UI.Xaml.PropertyPath.$ctor1("Visibility", "Visibility", Bridge.fn.cacheBind(this, this.accessVisualStateProperty_d832fca3fe274fcd87daa20ae6a95599), Bridge.fn.cacheBind(this, this.setVisualStateProperty_d832fca3fe274fcd87daa20ae6a95599), Bridge.fn.cacheBind(this, this.setAnimationVisualStateProperty_d832fca3fe274fcd87daa20ae6a95599), Bridge.fn.cacheBind(this, this.setLocalVisualStateProperty_d832fca3fe274fcd87daa20ae6a95599), Bridge.fn.cacheBind(this, this.getVisualStateProperty_d832fca3fe274fcd87daa20ae6a95599)));
                    Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard.SetTarget(ObjectAnimationUsingKeyFrames_c20a3d4ab21e4d6782b38ede1a0b7439, Button_7db9715dc6224087bb4e526aee5c7542);


                    Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard.SetTargetProperty(ObjectAnimationUsingKeyFrames_87912696dc714f39a0ca3db9c11e2a5c, new Bridge.global.Windows.UI.Xaml.PropertyPath.$ctor1("Visibility", "Visibility", Bridge.fn.cacheBind(this, this.accessVisualStateProperty_cb9f7c626bce456b98ba561d4c36a7ca), Bridge.fn.cacheBind(this, this.setVisualStateProperty_cb9f7c626bce456b98ba561d4c36a7ca), Bridge.fn.cacheBind(this, this.setAnimationVisualStateProperty_cb9f7c626bce456b98ba561d4c36a7ca), Bridge.fn.cacheBind(this, this.setLocalVisualStateProperty_cb9f7c626bce456b98ba561d4c36a7ca), Bridge.fn.cacheBind(this, this.getVisualStateProperty_cb9f7c626bce456b98ba561d4c36a7ca)));
                    Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard.SetTarget(ObjectAnimationUsingKeyFrames_87912696dc714f39a0ca3db9c11e2a5c, TextBox_5e5b54f2d3ac42db9b91623d8ae9b49a);


                    Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard.SetTargetProperty(ObjectAnimationUsingKeyFrames_133978e8021d4bb6ae0387fa94692ae5, new Bridge.global.Windows.UI.Xaml.PropertyPath.$ctor1("Visibility", "Visibility", Bridge.fn.cacheBind(this, this.accessVisualStateProperty_6eaea0a4d2ea46a9908c0d174d9fdda3), Bridge.fn.cacheBind(this, this.setVisualStateProperty_6eaea0a4d2ea46a9908c0d174d9fdda3), Bridge.fn.cacheBind(this, this.setAnimationVisualStateProperty_6eaea0a4d2ea46a9908c0d174d9fdda3), Bridge.fn.cacheBind(this, this.setLocalVisualStateProperty_6eaea0a4d2ea46a9908c0d174d9fdda3), Bridge.fn.cacheBind(this, this.getVisualStateProperty_6eaea0a4d2ea46a9908c0d174d9fdda3)));
                    Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard.SetTarget(ObjectAnimationUsingKeyFrames_133978e8021d4bb6ae0387fa94692ae5, StackPanel_5039ccb47ff04b0f926057007c69e386);


                    Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard.SetTargetProperty(ObjectAnimationUsingKeyFrames_4bec98a8f10a401fba166d082acc7e9b, new Bridge.global.Windows.UI.Xaml.PropertyPath.$ctor1("Visibility", "Visibility", Bridge.fn.cacheBind(this, this.accessVisualStateProperty_6310e2fec0c649929cb380e044563709), Bridge.fn.cacheBind(this, this.setVisualStateProperty_6310e2fec0c649929cb380e044563709), Bridge.fn.cacheBind(this, this.setAnimationVisualStateProperty_6310e2fec0c649929cb380e044563709), Bridge.fn.cacheBind(this, this.setLocalVisualStateProperty_6310e2fec0c649929cb380e044563709), Bridge.fn.cacheBind(this, this.getVisualStateProperty_6310e2fec0c649929cb380e044563709)));
                    Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard.SetTarget(ObjectAnimationUsingKeyFrames_4bec98a8f10a401fba166d082acc7e9b, StackPanel_b58f85e120db44f6bd5370a6d599972d);


                    Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard.SetTargetProperty(ObjectAnimationUsingKeyFrames_4fa98f75e27c428291587964b706140b, new Bridge.global.Windows.UI.Xaml.PropertyPath.$ctor1("Visibility", "Visibility", Bridge.fn.cacheBind(this, this.accessVisualStateProperty_88c831b324e94e5780c711b6e3b1220c), Bridge.fn.cacheBind(this, this.setVisualStateProperty_88c831b324e94e5780c711b6e3b1220c), Bridge.fn.cacheBind(this, this.setAnimationVisualStateProperty_88c831b324e94e5780c711b6e3b1220c), Bridge.fn.cacheBind(this, this.setLocalVisualStateProperty_88c831b324e94e5780c711b6e3b1220c), Bridge.fn.cacheBind(this, this.getVisualStateProperty_88c831b324e94e5780c711b6e3b1220c)));
                    Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard.SetTarget(ObjectAnimationUsingKeyFrames_4fa98f75e27c428291587964b706140b, TextBox_5e5b54f2d3ac42db9b91623d8ae9b49a);


                    Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard.SetTargetProperty(ObjectAnimationUsingKeyFrames_6768ae1bf3ca42cab77d8fd3e8123b2c, new Bridge.global.Windows.UI.Xaml.PropertyPath.$ctor1("Visibility", "Visibility", Bridge.fn.cacheBind(this, this.accessVisualStateProperty_aa6a73ec6a5f4110a48e0d8b6dd06a40), Bridge.fn.cacheBind(this, this.setVisualStateProperty_aa6a73ec6a5f4110a48e0d8b6dd06a40), Bridge.fn.cacheBind(this, this.setAnimationVisualStateProperty_aa6a73ec6a5f4110a48e0d8b6dd06a40), Bridge.fn.cacheBind(this, this.setLocalVisualStateProperty_aa6a73ec6a5f4110a48e0d8b6dd06a40), Bridge.fn.cacheBind(this, this.getVisualStateProperty_aa6a73ec6a5f4110a48e0d8b6dd06a40)));
                    Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard.SetTarget(ObjectAnimationUsingKeyFrames_6768ae1bf3ca42cab77d8fd3e8123b2c, StackPanel_5039ccb47ff04b0f926057007c69e386);


                    Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard.SetTargetProperty(ObjectAnimationUsingKeyFrames_e94ee94732ff4eb98b923f8db5461f7e, new Bridge.global.Windows.UI.Xaml.PropertyPath.$ctor1("Visibility", "Visibility", Bridge.fn.cacheBind(this, this.accessVisualStateProperty_812789311c2c439b8ce47e204c88991c), Bridge.fn.cacheBind(this, this.setVisualStateProperty_812789311c2c439b8ce47e204c88991c), Bridge.fn.cacheBind(this, this.setAnimationVisualStateProperty_812789311c2c439b8ce47e204c88991c), Bridge.fn.cacheBind(this, this.setLocalVisualStateProperty_812789311c2c439b8ce47e204c88991c), Bridge.fn.cacheBind(this, this.getVisualStateProperty_812789311c2c439b8ce47e204c88991c)));
                    Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard.SetTarget(ObjectAnimationUsingKeyFrames_e94ee94732ff4eb98b923f8db5461f7e, Button_2ecf43a729774feda5635ec33b78e87b);


                    Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard.SetTargetProperty(ObjectAnimationUsingKeyFrames_a4b0030d8424491784bffaf44d14bc16, new Bridge.global.Windows.UI.Xaml.PropertyPath.$ctor1("Visibility", "Visibility", Bridge.fn.cacheBind(this, this.accessVisualStateProperty_bccbaa1396a94c1e8cbcaa39179f00ae), Bridge.fn.cacheBind(this, this.setVisualStateProperty_bccbaa1396a94c1e8cbcaa39179f00ae), Bridge.fn.cacheBind(this, this.setAnimationVisualStateProperty_bccbaa1396a94c1e8cbcaa39179f00ae), Bridge.fn.cacheBind(this, this.setLocalVisualStateProperty_bccbaa1396a94c1e8cbcaa39179f00ae), Bridge.fn.cacheBind(this, this.getVisualStateProperty_bccbaa1396a94c1e8cbcaa39179f00ae)));
                    Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard.SetTarget(ObjectAnimationUsingKeyFrames_a4b0030d8424491784bffaf44d14bc16, Button_b6cb766b3f6f419eb6a55dd4be55411e);


                    Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard.SetTargetProperty(ObjectAnimationUsingKeyFrames_22fca1516c9046078761f69304986397, new Bridge.global.Windows.UI.Xaml.PropertyPath.$ctor1("Visibility", "Visibility", Bridge.fn.cacheBind(this, this.accessVisualStateProperty_bce85cf0d74f412c9b2377e632e3c3b4), Bridge.fn.cacheBind(this, this.setVisualStateProperty_bce85cf0d74f412c9b2377e632e3c3b4), Bridge.fn.cacheBind(this, this.setAnimationVisualStateProperty_bce85cf0d74f412c9b2377e632e3c3b4), Bridge.fn.cacheBind(this, this.setLocalVisualStateProperty_bce85cf0d74f412c9b2377e632e3c3b4), Bridge.fn.cacheBind(this, this.getVisualStateProperty_bce85cf0d74f412c9b2377e632e3c3b4)));
                    Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard.SetTarget(ObjectAnimationUsingKeyFrames_22fca1516c9046078761f69304986397, Button_975225bcbd324697a58acaa112f593d7);


                    Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard.SetTargetProperty(ObjectAnimationUsingKeyFrames_f685fdf829134a2b9a63059c4d8a935d, new Bridge.global.Windows.UI.Xaml.PropertyPath.$ctor1("Visibility", "Visibility", Bridge.fn.cacheBind(this, this.accessVisualStateProperty_378bec20782849efb0795076d23f32e5), Bridge.fn.cacheBind(this, this.setVisualStateProperty_378bec20782849efb0795076d23f32e5), Bridge.fn.cacheBind(this, this.setAnimationVisualStateProperty_378bec20782849efb0795076d23f32e5), Bridge.fn.cacheBind(this, this.setLocalVisualStateProperty_378bec20782849efb0795076d23f32e5), Bridge.fn.cacheBind(this, this.getVisualStateProperty_378bec20782849efb0795076d23f32e5)));
                    Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard.SetTarget(ObjectAnimationUsingKeyFrames_f685fdf829134a2b9a63059c4d8a935d, Button_7db9715dc6224087bb4e526aee5c7542);


                    Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard.SetTargetProperty(ObjectAnimationUsingKeyFrames_1d01829e8165479d8a08f769eb0f2ba2, new Bridge.global.Windows.UI.Xaml.PropertyPath.$ctor1("Visibility", "Visibility", Bridge.fn.cacheBind(this, this.accessVisualStateProperty_87dc037efe91499ab223b8a6688f815c), Bridge.fn.cacheBind(this, this.setVisualStateProperty_87dc037efe91499ab223b8a6688f815c), Bridge.fn.cacheBind(this, this.setAnimationVisualStateProperty_87dc037efe91499ab223b8a6688f815c), Bridge.fn.cacheBind(this, this.setLocalVisualStateProperty_87dc037efe91499ab223b8a6688f815c), Bridge.fn.cacheBind(this, this.getVisualStateProperty_87dc037efe91499ab223b8a6688f815c)));
                    Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard.SetTarget(ObjectAnimationUsingKeyFrames_1d01829e8165479d8a08f769eb0f2ba2, TextBox_5e5b54f2d3ac42db9b91623d8ae9b49a);


                    Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard.SetTargetProperty(ObjectAnimationUsingKeyFrames_aeed67ea75404c42958956a2f19f2f04, new Bridge.global.Windows.UI.Xaml.PropertyPath.$ctor1("Visibility", "Visibility", Bridge.fn.cacheBind(this, this.accessVisualStateProperty_ebf628272dcf4b65a353427065109841), Bridge.fn.cacheBind(this, this.setVisualStateProperty_ebf628272dcf4b65a353427065109841), Bridge.fn.cacheBind(this, this.setAnimationVisualStateProperty_ebf628272dcf4b65a353427065109841), Bridge.fn.cacheBind(this, this.setLocalVisualStateProperty_ebf628272dcf4b65a353427065109841), Bridge.fn.cacheBind(this, this.getVisualStateProperty_ebf628272dcf4b65a353427065109841)));
                    Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard.SetTarget(ObjectAnimationUsingKeyFrames_aeed67ea75404c42958956a2f19f2f04, StackPanel_5039ccb47ff04b0f926057007c69e386);


                    Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard.SetTargetProperty(ObjectAnimationUsingKeyFrames_dced361ca485481ebd1109769c95762c, new Bridge.global.Windows.UI.Xaml.PropertyPath.$ctor1("Visibility", "Visibility", Bridge.fn.cacheBind(this, this.accessVisualStateProperty_76ef4532488f4885aa150dbed9e1f51d), Bridge.fn.cacheBind(this, this.setVisualStateProperty_76ef4532488f4885aa150dbed9e1f51d), Bridge.fn.cacheBind(this, this.setAnimationVisualStateProperty_76ef4532488f4885aa150dbed9e1f51d), Bridge.fn.cacheBind(this, this.setLocalVisualStateProperty_76ef4532488f4885aa150dbed9e1f51d), Bridge.fn.cacheBind(this, this.getVisualStateProperty_76ef4532488f4885aa150dbed9e1f51d)));
                    Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard.SetTarget(ObjectAnimationUsingKeyFrames_dced361ca485481ebd1109769c95762c, Border_583805b6b32e43a78d3bf62fba7f9f9b);


                    Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard.SetTargetProperty(ObjectAnimationUsingKeyFrames_a11aa0067f84477eb3a75d20d42f891a, new Bridge.global.Windows.UI.Xaml.PropertyPath.$ctor1("Visibility", "Visibility", Bridge.fn.cacheBind(this, this.accessVisualStateProperty_40769f64a88e4748b437e3c63f7feb36), Bridge.fn.cacheBind(this, this.setVisualStateProperty_40769f64a88e4748b437e3c63f7feb36), Bridge.fn.cacheBind(this, this.setAnimationVisualStateProperty_40769f64a88e4748b437e3c63f7feb36), Bridge.fn.cacheBind(this, this.setLocalVisualStateProperty_40769f64a88e4748b437e3c63f7feb36), Bridge.fn.cacheBind(this, this.getVisualStateProperty_40769f64a88e4748b437e3c63f7feb36)));
                    Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard.SetTarget(ObjectAnimationUsingKeyFrames_a11aa0067f84477eb3a75d20d42f891a, Border_d484f9589f234b718b2106b85893789a);


                    Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard.SetTargetProperty(ObjectAnimationUsingKeyFrames_8637b34401534c1e9170431ea815aacb, new Bridge.global.Windows.UI.Xaml.PropertyPath.$ctor1("Visibility", "Visibility", Bridge.fn.cacheBind(this, this.accessVisualStateProperty_6614f195e873451594de7a80c986a982), Bridge.fn.cacheBind(this, this.setVisualStateProperty_6614f195e873451594de7a80c986a982), Bridge.fn.cacheBind(this, this.setAnimationVisualStateProperty_6614f195e873451594de7a80c986a982), Bridge.fn.cacheBind(this, this.setLocalVisualStateProperty_6614f195e873451594de7a80c986a982), Bridge.fn.cacheBind(this, this.getVisualStateProperty_6614f195e873451594de7a80c986a982)));
                    Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard.SetTarget(ObjectAnimationUsingKeyFrames_8637b34401534c1e9170431ea815aacb, Button_2ecf43a729774feda5635ec33b78e87b);


                    Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard.SetTargetProperty(ObjectAnimationUsingKeyFrames_641fa6d27fd345a696234fb33d4c7e93, new Bridge.global.Windows.UI.Xaml.PropertyPath.$ctor1("Visibility", "Visibility", Bridge.fn.cacheBind(this, this.accessVisualStateProperty_d7801f67623d4cf4bd836a6ad7a94581), Bridge.fn.cacheBind(this, this.setVisualStateProperty_d7801f67623d4cf4bd836a6ad7a94581), Bridge.fn.cacheBind(this, this.setAnimationVisualStateProperty_d7801f67623d4cf4bd836a6ad7a94581), Bridge.fn.cacheBind(this, this.setLocalVisualStateProperty_d7801f67623d4cf4bd836a6ad7a94581), Bridge.fn.cacheBind(this, this.getVisualStateProperty_d7801f67623d4cf4bd836a6ad7a94581)));
                    Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard.SetTarget(ObjectAnimationUsingKeyFrames_641fa6d27fd345a696234fb33d4c7e93, Button_b6cb766b3f6f419eb6a55dd4be55411e);


                    Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard.SetTargetProperty(ObjectAnimationUsingKeyFrames_5be106d8da9e4e7a9a3d9adb126e9efe, new Bridge.global.Windows.UI.Xaml.PropertyPath.$ctor1("Visibility", "Visibility", Bridge.fn.cacheBind(this, this.accessVisualStateProperty_4eef6f93d3e749eeae7e857390e0167f), Bridge.fn.cacheBind(this, this.setVisualStateProperty_4eef6f93d3e749eeae7e857390e0167f), Bridge.fn.cacheBind(this, this.setAnimationVisualStateProperty_4eef6f93d3e749eeae7e857390e0167f), Bridge.fn.cacheBind(this, this.setLocalVisualStateProperty_4eef6f93d3e749eeae7e857390e0167f), Bridge.fn.cacheBind(this, this.getVisualStateProperty_4eef6f93d3e749eeae7e857390e0167f)));
                    Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard.SetTarget(ObjectAnimationUsingKeyFrames_5be106d8da9e4e7a9a3d9adb126e9efe, StackPanel_b58f85e120db44f6bd5370a6d599972d);


                    Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard.SetTargetProperty(ObjectAnimationUsingKeyFrames_5f1ca52e977b47249b73462998e26e0e, new Bridge.global.Windows.UI.Xaml.PropertyPath.$ctor1("Visibility", "Visibility", Bridge.fn.cacheBind(this, this.accessVisualStateProperty_8eee3238ec6d43b09ac4408fedebf9f9), Bridge.fn.cacheBind(this, this.setVisualStateProperty_8eee3238ec6d43b09ac4408fedebf9f9), Bridge.fn.cacheBind(this, this.setAnimationVisualStateProperty_8eee3238ec6d43b09ac4408fedebf9f9), Bridge.fn.cacheBind(this, this.setLocalVisualStateProperty_8eee3238ec6d43b09ac4408fedebf9f9), Bridge.fn.cacheBind(this, this.getVisualStateProperty_8eee3238ec6d43b09ac4408fedebf9f9)));
                    Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard.SetTarget(ObjectAnimationUsingKeyFrames_5f1ca52e977b47249b73462998e26e0e, Button_2ecf43a729774feda5635ec33b78e87b);


                    Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard.SetTargetProperty(ObjectAnimationUsingKeyFrames_18d0a4948c3642678329aa6f2e92700b, new Bridge.global.Windows.UI.Xaml.PropertyPath.$ctor1("Visibility", "Visibility", Bridge.fn.cacheBind(this, this.accessVisualStateProperty_3ca0493c2beb46c1ade814496c7ab2b2), Bridge.fn.cacheBind(this, this.setVisualStateProperty_3ca0493c2beb46c1ade814496c7ab2b2), Bridge.fn.cacheBind(this, this.setAnimationVisualStateProperty_3ca0493c2beb46c1ade814496c7ab2b2), Bridge.fn.cacheBind(this, this.setLocalVisualStateProperty_3ca0493c2beb46c1ade814496c7ab2b2), Bridge.fn.cacheBind(this, this.getVisualStateProperty_3ca0493c2beb46c1ade814496c7ab2b2)));
                    Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard.SetTarget(ObjectAnimationUsingKeyFrames_18d0a4948c3642678329aa6f2e92700b, Button_b6cb766b3f6f419eb6a55dd4be55411e);


                    Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard.SetTargetProperty(ObjectAnimationUsingKeyFrames_86559b4733e94ef293cbc18965addb21, new Bridge.global.Windows.UI.Xaml.PropertyPath.$ctor1("Visibility", "Visibility", Bridge.fn.cacheBind(this, this.accessVisualStateProperty_857ad27fff214fe799a6935f64633a23), Bridge.fn.cacheBind(this, this.setVisualStateProperty_857ad27fff214fe799a6935f64633a23), Bridge.fn.cacheBind(this, this.setAnimationVisualStateProperty_857ad27fff214fe799a6935f64633a23), Bridge.fn.cacheBind(this, this.setLocalVisualStateProperty_857ad27fff214fe799a6935f64633a23), Bridge.fn.cacheBind(this, this.getVisualStateProperty_857ad27fff214fe799a6935f64633a23)));
                    Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard.SetTarget(ObjectAnimationUsingKeyFrames_86559b4733e94ef293cbc18965addb21, TextBox_5e5b54f2d3ac42db9b91623d8ae9b49a);


                    Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard.SetTargetProperty(ObjectAnimationUsingKeyFrames_19d7f1f355fd4f47897f2d081e137ff7, new Bridge.global.Windows.UI.Xaml.PropertyPath.$ctor1("Visibility", "Visibility", Bridge.fn.cacheBind(this, this.accessVisualStateProperty_462951debfc84f228ddcd015b8011ed3), Bridge.fn.cacheBind(this, this.setVisualStateProperty_462951debfc84f228ddcd015b8011ed3), Bridge.fn.cacheBind(this, this.setAnimationVisualStateProperty_462951debfc84f228ddcd015b8011ed3), Bridge.fn.cacheBind(this, this.setLocalVisualStateProperty_462951debfc84f228ddcd015b8011ed3), Bridge.fn.cacheBind(this, this.getVisualStateProperty_462951debfc84f228ddcd015b8011ed3)));
                    Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard.SetTarget(ObjectAnimationUsingKeyFrames_19d7f1f355fd4f47897f2d081e137ff7, StackPanel_5039ccb47ff04b0f926057007c69e386);

                    templateInstance_37f7df09e03649e7b9c6ab503fce8242.TemplateContent = Grid_0ff886fd41ef49cb84afb3a4b3d1dc0a;
                    return templateInstance_37f7df09e03649e7b9c6ab503fce8242;
                }));

                Setter_1ea1cc8b6c3c4f1ba30f65b75ef17a91.Value = ControlTemplate_c2c1fafddd0e45348f9697b3bfd04e92;


                Style_25cdadba73404c8fb24aeb2c26876f7a.Setters.add(Setter_a87c494e48b446d191bc0f8136798c79);
                Style_25cdadba73404c8fb24aeb2c26876f7a.Setters.add(Setter_1fa275885388465a98377d2632ab76b2);
                Style_25cdadba73404c8fb24aeb2c26876f7a.Setters.add(Setter_076f159d43f54685a055f2a8d8c1735f);
                Style_25cdadba73404c8fb24aeb2c26876f7a.Setters.add(Setter_cbeeb17c02374bc48b5bf783eb71db18);
                Style_25cdadba73404c8fb24aeb2c26876f7a.Setters.add(Setter_b95adf35db3948e0a62da7b6da83f060);
                Style_25cdadba73404c8fb24aeb2c26876f7a.Setters.add(Setter_1ea1cc8b6c3c4f1ba30f65b75ef17a91);


                this.setItem(Bridge.global.Windows.UI.Xaml.Controls.DataPager, Style_25cdadba73404c8fb24aeb2c26876f7a);






            },
            accessVisualStateProperty_4958834675ea458b83a1dab0fe8a4335: function (rootTargetObjectInstance) {
                return new (Bridge.GeneratorEnumerable$1(System.Tuple$3(Windows.UI.Xaml.DependencyObject,Windows.UI.Xaml.DependencyProperty,System.Nullable$1(System.Int32))))(Bridge.fn.bind(this, function (rootTargetObjectInstance) {
                    var $step = 0,
                        $jumpFromFinally,
                        $returnValue,
                        $async_e;

                    var $enumerator = new (Bridge.GeneratorEnumerator$1(System.Tuple$3(Windows.UI.Xaml.DependencyObject,Windows.UI.Xaml.DependencyProperty,System.Nullable$1(System.Int32))))(Bridge.fn.bind(this, function () {
                        try {
                            for (;;) {
                                switch ($step) {
                                    case 0: {
                                        return false;
                                    }
                                    default: {
                                        return false;
                                    }
                                }
                            }
                        } catch($async_e1) {
                            $async_e = System.Exception.create($async_e1);
                            throw $async_e;
                        }
                    }));
                    return $enumerator;
                }, arguments));
            },
            setVisualStateProperty_4958834675ea458b83a1dab0fe8a4335: function (finalTargetInstance, value) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Opacity").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "OpacityProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                (finalTargetInstance).SetVisualStateValue(property, value);
            },
            setAnimationVisualStateProperty_4958834675ea458b83a1dab0fe8a4335: function (finalTargetInstance, value) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Opacity").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "OpacityProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                (finalTargetInstance).SetAnimationValue(property, value);
            },
            setLocalVisualStateProperty_4958834675ea458b83a1dab0fe8a4335: function (finalTargetInstance, value) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Opacity").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "OpacityProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                (finalTargetInstance).SetLocalValue(property, value);
            },
            getVisualStateProperty_4958834675ea458b83a1dab0fe8a4335: function (finalTargetInstance) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Opacity").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "OpacityProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                return finalTargetInstance.GetVisualStateValue(property);
            },
            accessVisualStateProperty_424ca16da0bf47cf821542d0abab5b94: function (rootTargetObjectInstance) {
                return new (Bridge.GeneratorEnumerable$1(System.Tuple$3(Windows.UI.Xaml.DependencyObject,Windows.UI.Xaml.DependencyProperty,System.Nullable$1(System.Int32))))(Bridge.fn.bind(this, function (rootTargetObjectInstance) {
                    var $step = 0,
                        $jumpFromFinally,
                        $returnValue,
                        $async_e;

                    var $enumerator = new (Bridge.GeneratorEnumerator$1(System.Tuple$3(Windows.UI.Xaml.DependencyObject,Windows.UI.Xaml.DependencyProperty,System.Nullable$1(System.Int32))))(Bridge.fn.bind(this, function () {
                        try {
                            for (;;) {
                                switch ($step) {
                                    case 0: {
                                        return false;
                                    }
                                    default: {
                                        return false;
                                    }
                                }
                            }
                        } catch($async_e1) {
                            $async_e = System.Exception.create($async_e1);
                            throw $async_e;
                        }
                    }));
                    return $enumerator;
                }, arguments));
            },
            setVisualStateProperty_424ca16da0bf47cf821542d0abab5b94: function (finalTargetInstance, value) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Opacity").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "OpacityProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                (finalTargetInstance).SetVisualStateValue(property, value);
            },
            setAnimationVisualStateProperty_424ca16da0bf47cf821542d0abab5b94: function (finalTargetInstance, value) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Opacity").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "OpacityProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                (finalTargetInstance).SetAnimationValue(property, value);
            },
            setLocalVisualStateProperty_424ca16da0bf47cf821542d0abab5b94: function (finalTargetInstance, value) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Opacity").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "OpacityProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                (finalTargetInstance).SetLocalValue(property, value);
            },
            getVisualStateProperty_424ca16da0bf47cf821542d0abab5b94: function (finalTargetInstance) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Opacity").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "OpacityProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                return finalTargetInstance.GetVisualStateValue(property);
            },
            accessVisualStateProperty_100421434fa543818d109d13a477cfc0: function (rootTargetObjectInstance) {
                return new (Bridge.GeneratorEnumerable$1(System.Tuple$3(Windows.UI.Xaml.DependencyObject,Windows.UI.Xaml.DependencyProperty,System.Nullable$1(System.Int32))))(Bridge.fn.bind(this, function (rootTargetObjectInstance) {
                    var $step = 0,
                        $jumpFromFinally,
                        $returnValue,
                        $async_e;

                    var $enumerator = new (Bridge.GeneratorEnumerator$1(System.Tuple$3(Windows.UI.Xaml.DependencyObject,Windows.UI.Xaml.DependencyProperty,System.Nullable$1(System.Int32))))(Bridge.fn.bind(this, function () {
                        try {
                            for (;;) {
                                switch ($step) {
                                    case 0: {
                                        return false;
                                    }
                                    default: {
                                        return false;
                                    }
                                }
                            }
                        } catch($async_e1) {
                            $async_e = System.Exception.create($async_e1);
                            throw $async_e;
                        }
                    }));
                    return $enumerator;
                }, arguments));
            },
            setVisualStateProperty_100421434fa543818d109d13a477cfc0: function (finalTargetInstance, value) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Opacity").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "OpacityProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                (finalTargetInstance).SetVisualStateValue(property, value);
            },
            setAnimationVisualStateProperty_100421434fa543818d109d13a477cfc0: function (finalTargetInstance, value) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Opacity").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "OpacityProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                (finalTargetInstance).SetAnimationValue(property, value);
            },
            setLocalVisualStateProperty_100421434fa543818d109d13a477cfc0: function (finalTargetInstance, value) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Opacity").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "OpacityProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                (finalTargetInstance).SetLocalValue(property, value);
            },
            getVisualStateProperty_100421434fa543818d109d13a477cfc0: function (finalTargetInstance) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Opacity").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "OpacityProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                return finalTargetInstance.GetVisualStateValue(property);
            },
            accessVisualStateProperty_4420ecb444054b3da1afee49e54e96eb: function (rootTargetObjectInstance) {
                return new (Bridge.GeneratorEnumerable$1(System.Tuple$3(Windows.UI.Xaml.DependencyObject,Windows.UI.Xaml.DependencyProperty,System.Nullable$1(System.Int32))))(Bridge.fn.bind(this, function (rootTargetObjectInstance) {
                    var $step = 0,
                        $jumpFromFinally,
                        $returnValue,
                        $async_e;

                    var $enumerator = new (Bridge.GeneratorEnumerator$1(System.Tuple$3(Windows.UI.Xaml.DependencyObject,Windows.UI.Xaml.DependencyProperty,System.Nullable$1(System.Int32))))(Bridge.fn.bind(this, function () {
                        try {
                            for (;;) {
                                switch ($step) {
                                    case 0: {
                                        return false;
                                    }
                                    default: {
                                        return false;
                                    }
                                }
                            }
                        } catch($async_e1) {
                            $async_e = System.Exception.create($async_e1);
                            throw $async_e;
                        }
                    }));
                    return $enumerator;
                }, arguments));
            },
            setVisualStateProperty_4420ecb444054b3da1afee49e54e96eb: function (finalTargetInstance, value) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Visibility").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "VisibilityProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                (finalTargetInstance).SetVisualStateValue(property, value);
            },
            setAnimationVisualStateProperty_4420ecb444054b3da1afee49e54e96eb: function (finalTargetInstance, value) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Visibility").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "VisibilityProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                (finalTargetInstance).SetAnimationValue(property, value);
            },
            setLocalVisualStateProperty_4420ecb444054b3da1afee49e54e96eb: function (finalTargetInstance, value) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Visibility").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "VisibilityProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                (finalTargetInstance).SetLocalValue(property, value);
            },
            getVisualStateProperty_4420ecb444054b3da1afee49e54e96eb: function (finalTargetInstance) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Visibility").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "VisibilityProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                return finalTargetInstance.GetVisualStateValue(property);
            },
            accessVisualStateProperty_d832fca3fe274fcd87daa20ae6a95599: function (rootTargetObjectInstance) {
                return new (Bridge.GeneratorEnumerable$1(System.Tuple$3(Windows.UI.Xaml.DependencyObject,Windows.UI.Xaml.DependencyProperty,System.Nullable$1(System.Int32))))(Bridge.fn.bind(this, function (rootTargetObjectInstance) {
                    var $step = 0,
                        $jumpFromFinally,
                        $returnValue,
                        $async_e;

                    var $enumerator = new (Bridge.GeneratorEnumerator$1(System.Tuple$3(Windows.UI.Xaml.DependencyObject,Windows.UI.Xaml.DependencyProperty,System.Nullable$1(System.Int32))))(Bridge.fn.bind(this, function () {
                        try {
                            for (;;) {
                                switch ($step) {
                                    case 0: {
                                        return false;
                                    }
                                    default: {
                                        return false;
                                    }
                                }
                            }
                        } catch($async_e1) {
                            $async_e = System.Exception.create($async_e1);
                            throw $async_e;
                        }
                    }));
                    return $enumerator;
                }, arguments));
            },
            setVisualStateProperty_d832fca3fe274fcd87daa20ae6a95599: function (finalTargetInstance, value) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Visibility").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "VisibilityProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                (finalTargetInstance).SetVisualStateValue(property, value);
            },
            setAnimationVisualStateProperty_d832fca3fe274fcd87daa20ae6a95599: function (finalTargetInstance, value) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Visibility").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "VisibilityProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                (finalTargetInstance).SetAnimationValue(property, value);
            },
            setLocalVisualStateProperty_d832fca3fe274fcd87daa20ae6a95599: function (finalTargetInstance, value) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Visibility").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "VisibilityProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                (finalTargetInstance).SetLocalValue(property, value);
            },
            getVisualStateProperty_d832fca3fe274fcd87daa20ae6a95599: function (finalTargetInstance) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Visibility").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "VisibilityProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                return finalTargetInstance.GetVisualStateValue(property);
            },
            accessVisualStateProperty_cb9f7c626bce456b98ba561d4c36a7ca: function (rootTargetObjectInstance) {
                return new (Bridge.GeneratorEnumerable$1(System.Tuple$3(Windows.UI.Xaml.DependencyObject,Windows.UI.Xaml.DependencyProperty,System.Nullable$1(System.Int32))))(Bridge.fn.bind(this, function (rootTargetObjectInstance) {
                    var $step = 0,
                        $jumpFromFinally,
                        $returnValue,
                        $async_e;

                    var $enumerator = new (Bridge.GeneratorEnumerator$1(System.Tuple$3(Windows.UI.Xaml.DependencyObject,Windows.UI.Xaml.DependencyProperty,System.Nullable$1(System.Int32))))(Bridge.fn.bind(this, function () {
                        try {
                            for (;;) {
                                switch ($step) {
                                    case 0: {
                                        return false;
                                    }
                                    default: {
                                        return false;
                                    }
                                }
                            }
                        } catch($async_e1) {
                            $async_e = System.Exception.create($async_e1);
                            throw $async_e;
                        }
                    }));
                    return $enumerator;
                }, arguments));
            },
            setVisualStateProperty_cb9f7c626bce456b98ba561d4c36a7ca: function (finalTargetInstance, value) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Visibility").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "VisibilityProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                (finalTargetInstance).SetVisualStateValue(property, value);
            },
            setAnimationVisualStateProperty_cb9f7c626bce456b98ba561d4c36a7ca: function (finalTargetInstance, value) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Visibility").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "VisibilityProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                (finalTargetInstance).SetAnimationValue(property, value);
            },
            setLocalVisualStateProperty_cb9f7c626bce456b98ba561d4c36a7ca: function (finalTargetInstance, value) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Visibility").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "VisibilityProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                (finalTargetInstance).SetLocalValue(property, value);
            },
            getVisualStateProperty_cb9f7c626bce456b98ba561d4c36a7ca: function (finalTargetInstance) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Visibility").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "VisibilityProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                return finalTargetInstance.GetVisualStateValue(property);
            },
            accessVisualStateProperty_6eaea0a4d2ea46a9908c0d174d9fdda3: function (rootTargetObjectInstance) {
                return new (Bridge.GeneratorEnumerable$1(System.Tuple$3(Windows.UI.Xaml.DependencyObject,Windows.UI.Xaml.DependencyProperty,System.Nullable$1(System.Int32))))(Bridge.fn.bind(this, function (rootTargetObjectInstance) {
                    var $step = 0,
                        $jumpFromFinally,
                        $returnValue,
                        $async_e;

                    var $enumerator = new (Bridge.GeneratorEnumerator$1(System.Tuple$3(Windows.UI.Xaml.DependencyObject,Windows.UI.Xaml.DependencyProperty,System.Nullable$1(System.Int32))))(Bridge.fn.bind(this, function () {
                        try {
                            for (;;) {
                                switch ($step) {
                                    case 0: {
                                        return false;
                                    }
                                    default: {
                                        return false;
                                    }
                                }
                            }
                        } catch($async_e1) {
                            $async_e = System.Exception.create($async_e1);
                            throw $async_e;
                        }
                    }));
                    return $enumerator;
                }, arguments));
            },
            setVisualStateProperty_6eaea0a4d2ea46a9908c0d174d9fdda3: function (finalTargetInstance, value) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Visibility").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "VisibilityProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                (finalTargetInstance).SetVisualStateValue(property, value);
            },
            setAnimationVisualStateProperty_6eaea0a4d2ea46a9908c0d174d9fdda3: function (finalTargetInstance, value) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Visibility").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "VisibilityProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                (finalTargetInstance).SetAnimationValue(property, value);
            },
            setLocalVisualStateProperty_6eaea0a4d2ea46a9908c0d174d9fdda3: function (finalTargetInstance, value) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Visibility").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "VisibilityProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                (finalTargetInstance).SetLocalValue(property, value);
            },
            getVisualStateProperty_6eaea0a4d2ea46a9908c0d174d9fdda3: function (finalTargetInstance) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Visibility").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "VisibilityProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                return finalTargetInstance.GetVisualStateValue(property);
            },
            accessVisualStateProperty_6310e2fec0c649929cb380e044563709: function (rootTargetObjectInstance) {
                return new (Bridge.GeneratorEnumerable$1(System.Tuple$3(Windows.UI.Xaml.DependencyObject,Windows.UI.Xaml.DependencyProperty,System.Nullable$1(System.Int32))))(Bridge.fn.bind(this, function (rootTargetObjectInstance) {
                    var $step = 0,
                        $jumpFromFinally,
                        $returnValue,
                        $async_e;

                    var $enumerator = new (Bridge.GeneratorEnumerator$1(System.Tuple$3(Windows.UI.Xaml.DependencyObject,Windows.UI.Xaml.DependencyProperty,System.Nullable$1(System.Int32))))(Bridge.fn.bind(this, function () {
                        try {
                            for (;;) {
                                switch ($step) {
                                    case 0: {
                                        return false;
                                    }
                                    default: {
                                        return false;
                                    }
                                }
                            }
                        } catch($async_e1) {
                            $async_e = System.Exception.create($async_e1);
                            throw $async_e;
                        }
                    }));
                    return $enumerator;
                }, arguments));
            },
            setVisualStateProperty_6310e2fec0c649929cb380e044563709: function (finalTargetInstance, value) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Visibility").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "VisibilityProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                (finalTargetInstance).SetVisualStateValue(property, value);
            },
            setAnimationVisualStateProperty_6310e2fec0c649929cb380e044563709: function (finalTargetInstance, value) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Visibility").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "VisibilityProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                (finalTargetInstance).SetAnimationValue(property, value);
            },
            setLocalVisualStateProperty_6310e2fec0c649929cb380e044563709: function (finalTargetInstance, value) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Visibility").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "VisibilityProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                (finalTargetInstance).SetLocalValue(property, value);
            },
            getVisualStateProperty_6310e2fec0c649929cb380e044563709: function (finalTargetInstance) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Visibility").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "VisibilityProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                return finalTargetInstance.GetVisualStateValue(property);
            },
            accessVisualStateProperty_88c831b324e94e5780c711b6e3b1220c: function (rootTargetObjectInstance) {
                return new (Bridge.GeneratorEnumerable$1(System.Tuple$3(Windows.UI.Xaml.DependencyObject,Windows.UI.Xaml.DependencyProperty,System.Nullable$1(System.Int32))))(Bridge.fn.bind(this, function (rootTargetObjectInstance) {
                    var $step = 0,
                        $jumpFromFinally,
                        $returnValue,
                        $async_e;

                    var $enumerator = new (Bridge.GeneratorEnumerator$1(System.Tuple$3(Windows.UI.Xaml.DependencyObject,Windows.UI.Xaml.DependencyProperty,System.Nullable$1(System.Int32))))(Bridge.fn.bind(this, function () {
                        try {
                            for (;;) {
                                switch ($step) {
                                    case 0: {
                                        return false;
                                    }
                                    default: {
                                        return false;
                                    }
                                }
                            }
                        } catch($async_e1) {
                            $async_e = System.Exception.create($async_e1);
                            throw $async_e;
                        }
                    }));
                    return $enumerator;
                }, arguments));
            },
            setVisualStateProperty_88c831b324e94e5780c711b6e3b1220c: function (finalTargetInstance, value) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Visibility").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "VisibilityProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                (finalTargetInstance).SetVisualStateValue(property, value);
            },
            setAnimationVisualStateProperty_88c831b324e94e5780c711b6e3b1220c: function (finalTargetInstance, value) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Visibility").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "VisibilityProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                (finalTargetInstance).SetAnimationValue(property, value);
            },
            setLocalVisualStateProperty_88c831b324e94e5780c711b6e3b1220c: function (finalTargetInstance, value) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Visibility").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "VisibilityProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                (finalTargetInstance).SetLocalValue(property, value);
            },
            getVisualStateProperty_88c831b324e94e5780c711b6e3b1220c: function (finalTargetInstance) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Visibility").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "VisibilityProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                return finalTargetInstance.GetVisualStateValue(property);
            },
            accessVisualStateProperty_aa6a73ec6a5f4110a48e0d8b6dd06a40: function (rootTargetObjectInstance) {
                return new (Bridge.GeneratorEnumerable$1(System.Tuple$3(Windows.UI.Xaml.DependencyObject,Windows.UI.Xaml.DependencyProperty,System.Nullable$1(System.Int32))))(Bridge.fn.bind(this, function (rootTargetObjectInstance) {
                    var $step = 0,
                        $jumpFromFinally,
                        $returnValue,
                        $async_e;

                    var $enumerator = new (Bridge.GeneratorEnumerator$1(System.Tuple$3(Windows.UI.Xaml.DependencyObject,Windows.UI.Xaml.DependencyProperty,System.Nullable$1(System.Int32))))(Bridge.fn.bind(this, function () {
                        try {
                            for (;;) {
                                switch ($step) {
                                    case 0: {
                                        return false;
                                    }
                                    default: {
                                        return false;
                                    }
                                }
                            }
                        } catch($async_e1) {
                            $async_e = System.Exception.create($async_e1);
                            throw $async_e;
                        }
                    }));
                    return $enumerator;
                }, arguments));
            },
            setVisualStateProperty_aa6a73ec6a5f4110a48e0d8b6dd06a40: function (finalTargetInstance, value) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Visibility").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "VisibilityProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                (finalTargetInstance).SetVisualStateValue(property, value);
            },
            setAnimationVisualStateProperty_aa6a73ec6a5f4110a48e0d8b6dd06a40: function (finalTargetInstance, value) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Visibility").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "VisibilityProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                (finalTargetInstance).SetAnimationValue(property, value);
            },
            setLocalVisualStateProperty_aa6a73ec6a5f4110a48e0d8b6dd06a40: function (finalTargetInstance, value) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Visibility").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "VisibilityProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                (finalTargetInstance).SetLocalValue(property, value);
            },
            getVisualStateProperty_aa6a73ec6a5f4110a48e0d8b6dd06a40: function (finalTargetInstance) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Visibility").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "VisibilityProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                return finalTargetInstance.GetVisualStateValue(property);
            },
            accessVisualStateProperty_812789311c2c439b8ce47e204c88991c: function (rootTargetObjectInstance) {
                return new (Bridge.GeneratorEnumerable$1(System.Tuple$3(Windows.UI.Xaml.DependencyObject,Windows.UI.Xaml.DependencyProperty,System.Nullable$1(System.Int32))))(Bridge.fn.bind(this, function (rootTargetObjectInstance) {
                    var $step = 0,
                        $jumpFromFinally,
                        $returnValue,
                        $async_e;

                    var $enumerator = new (Bridge.GeneratorEnumerator$1(System.Tuple$3(Windows.UI.Xaml.DependencyObject,Windows.UI.Xaml.DependencyProperty,System.Nullable$1(System.Int32))))(Bridge.fn.bind(this, function () {
                        try {
                            for (;;) {
                                switch ($step) {
                                    case 0: {
                                        return false;
                                    }
                                    default: {
                                        return false;
                                    }
                                }
                            }
                        } catch($async_e1) {
                            $async_e = System.Exception.create($async_e1);
                            throw $async_e;
                        }
                    }));
                    return $enumerator;
                }, arguments));
            },
            setVisualStateProperty_812789311c2c439b8ce47e204c88991c: function (finalTargetInstance, value) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Visibility").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "VisibilityProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                (finalTargetInstance).SetVisualStateValue(property, value);
            },
            setAnimationVisualStateProperty_812789311c2c439b8ce47e204c88991c: function (finalTargetInstance, value) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Visibility").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "VisibilityProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                (finalTargetInstance).SetAnimationValue(property, value);
            },
            setLocalVisualStateProperty_812789311c2c439b8ce47e204c88991c: function (finalTargetInstance, value) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Visibility").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "VisibilityProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                (finalTargetInstance).SetLocalValue(property, value);
            },
            getVisualStateProperty_812789311c2c439b8ce47e204c88991c: function (finalTargetInstance) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Visibility").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "VisibilityProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                return finalTargetInstance.GetVisualStateValue(property);
            },
            accessVisualStateProperty_bccbaa1396a94c1e8cbcaa39179f00ae: function (rootTargetObjectInstance) {
                return new (Bridge.GeneratorEnumerable$1(System.Tuple$3(Windows.UI.Xaml.DependencyObject,Windows.UI.Xaml.DependencyProperty,System.Nullable$1(System.Int32))))(Bridge.fn.bind(this, function (rootTargetObjectInstance) {
                    var $step = 0,
                        $jumpFromFinally,
                        $returnValue,
                        $async_e;

                    var $enumerator = new (Bridge.GeneratorEnumerator$1(System.Tuple$3(Windows.UI.Xaml.DependencyObject,Windows.UI.Xaml.DependencyProperty,System.Nullable$1(System.Int32))))(Bridge.fn.bind(this, function () {
                        try {
                            for (;;) {
                                switch ($step) {
                                    case 0: {
                                        return false;
                                    }
                                    default: {
                                        return false;
                                    }
                                }
                            }
                        } catch($async_e1) {
                            $async_e = System.Exception.create($async_e1);
                            throw $async_e;
                        }
                    }));
                    return $enumerator;
                }, arguments));
            },
            setVisualStateProperty_bccbaa1396a94c1e8cbcaa39179f00ae: function (finalTargetInstance, value) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Visibility").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "VisibilityProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                (finalTargetInstance).SetVisualStateValue(property, value);
            },
            setAnimationVisualStateProperty_bccbaa1396a94c1e8cbcaa39179f00ae: function (finalTargetInstance, value) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Visibility").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "VisibilityProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                (finalTargetInstance).SetAnimationValue(property, value);
            },
            setLocalVisualStateProperty_bccbaa1396a94c1e8cbcaa39179f00ae: function (finalTargetInstance, value) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Visibility").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "VisibilityProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                (finalTargetInstance).SetLocalValue(property, value);
            },
            getVisualStateProperty_bccbaa1396a94c1e8cbcaa39179f00ae: function (finalTargetInstance) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Visibility").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "VisibilityProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                return finalTargetInstance.GetVisualStateValue(property);
            },
            accessVisualStateProperty_bce85cf0d74f412c9b2377e632e3c3b4: function (rootTargetObjectInstance) {
                return new (Bridge.GeneratorEnumerable$1(System.Tuple$3(Windows.UI.Xaml.DependencyObject,Windows.UI.Xaml.DependencyProperty,System.Nullable$1(System.Int32))))(Bridge.fn.bind(this, function (rootTargetObjectInstance) {
                    var $step = 0,
                        $jumpFromFinally,
                        $returnValue,
                        $async_e;

                    var $enumerator = new (Bridge.GeneratorEnumerator$1(System.Tuple$3(Windows.UI.Xaml.DependencyObject,Windows.UI.Xaml.DependencyProperty,System.Nullable$1(System.Int32))))(Bridge.fn.bind(this, function () {
                        try {
                            for (;;) {
                                switch ($step) {
                                    case 0: {
                                        return false;
                                    }
                                    default: {
                                        return false;
                                    }
                                }
                            }
                        } catch($async_e1) {
                            $async_e = System.Exception.create($async_e1);
                            throw $async_e;
                        }
                    }));
                    return $enumerator;
                }, arguments));
            },
            setVisualStateProperty_bce85cf0d74f412c9b2377e632e3c3b4: function (finalTargetInstance, value) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Visibility").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "VisibilityProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                (finalTargetInstance).SetVisualStateValue(property, value);
            },
            setAnimationVisualStateProperty_bce85cf0d74f412c9b2377e632e3c3b4: function (finalTargetInstance, value) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Visibility").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "VisibilityProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                (finalTargetInstance).SetAnimationValue(property, value);
            },
            setLocalVisualStateProperty_bce85cf0d74f412c9b2377e632e3c3b4: function (finalTargetInstance, value) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Visibility").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "VisibilityProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                (finalTargetInstance).SetLocalValue(property, value);
            },
            getVisualStateProperty_bce85cf0d74f412c9b2377e632e3c3b4: function (finalTargetInstance) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Visibility").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "VisibilityProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                return finalTargetInstance.GetVisualStateValue(property);
            },
            accessVisualStateProperty_378bec20782849efb0795076d23f32e5: function (rootTargetObjectInstance) {
                return new (Bridge.GeneratorEnumerable$1(System.Tuple$3(Windows.UI.Xaml.DependencyObject,Windows.UI.Xaml.DependencyProperty,System.Nullable$1(System.Int32))))(Bridge.fn.bind(this, function (rootTargetObjectInstance) {
                    var $step = 0,
                        $jumpFromFinally,
                        $returnValue,
                        $async_e;

                    var $enumerator = new (Bridge.GeneratorEnumerator$1(System.Tuple$3(Windows.UI.Xaml.DependencyObject,Windows.UI.Xaml.DependencyProperty,System.Nullable$1(System.Int32))))(Bridge.fn.bind(this, function () {
                        try {
                            for (;;) {
                                switch ($step) {
                                    case 0: {
                                        return false;
                                    }
                                    default: {
                                        return false;
                                    }
                                }
                            }
                        } catch($async_e1) {
                            $async_e = System.Exception.create($async_e1);
                            throw $async_e;
                        }
                    }));
                    return $enumerator;
                }, arguments));
            },
            setVisualStateProperty_378bec20782849efb0795076d23f32e5: function (finalTargetInstance, value) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Visibility").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "VisibilityProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                (finalTargetInstance).SetVisualStateValue(property, value);
            },
            setAnimationVisualStateProperty_378bec20782849efb0795076d23f32e5: function (finalTargetInstance, value) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Visibility").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "VisibilityProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                (finalTargetInstance).SetAnimationValue(property, value);
            },
            setLocalVisualStateProperty_378bec20782849efb0795076d23f32e5: function (finalTargetInstance, value) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Visibility").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "VisibilityProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                (finalTargetInstance).SetLocalValue(property, value);
            },
            getVisualStateProperty_378bec20782849efb0795076d23f32e5: function (finalTargetInstance) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Visibility").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "VisibilityProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                return finalTargetInstance.GetVisualStateValue(property);
            },
            accessVisualStateProperty_87dc037efe91499ab223b8a6688f815c: function (rootTargetObjectInstance) {
                return new (Bridge.GeneratorEnumerable$1(System.Tuple$3(Windows.UI.Xaml.DependencyObject,Windows.UI.Xaml.DependencyProperty,System.Nullable$1(System.Int32))))(Bridge.fn.bind(this, function (rootTargetObjectInstance) {
                    var $step = 0,
                        $jumpFromFinally,
                        $returnValue,
                        $async_e;

                    var $enumerator = new (Bridge.GeneratorEnumerator$1(System.Tuple$3(Windows.UI.Xaml.DependencyObject,Windows.UI.Xaml.DependencyProperty,System.Nullable$1(System.Int32))))(Bridge.fn.bind(this, function () {
                        try {
                            for (;;) {
                                switch ($step) {
                                    case 0: {
                                        return false;
                                    }
                                    default: {
                                        return false;
                                    }
                                }
                            }
                        } catch($async_e1) {
                            $async_e = System.Exception.create($async_e1);
                            throw $async_e;
                        }
                    }));
                    return $enumerator;
                }, arguments));
            },
            setVisualStateProperty_87dc037efe91499ab223b8a6688f815c: function (finalTargetInstance, value) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Visibility").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "VisibilityProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                (finalTargetInstance).SetVisualStateValue(property, value);
            },
            setAnimationVisualStateProperty_87dc037efe91499ab223b8a6688f815c: function (finalTargetInstance, value) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Visibility").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "VisibilityProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                (finalTargetInstance).SetAnimationValue(property, value);
            },
            setLocalVisualStateProperty_87dc037efe91499ab223b8a6688f815c: function (finalTargetInstance, value) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Visibility").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "VisibilityProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                (finalTargetInstance).SetLocalValue(property, value);
            },
            getVisualStateProperty_87dc037efe91499ab223b8a6688f815c: function (finalTargetInstance) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Visibility").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "VisibilityProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                return finalTargetInstance.GetVisualStateValue(property);
            },
            accessVisualStateProperty_ebf628272dcf4b65a353427065109841: function (rootTargetObjectInstance) {
                return new (Bridge.GeneratorEnumerable$1(System.Tuple$3(Windows.UI.Xaml.DependencyObject,Windows.UI.Xaml.DependencyProperty,System.Nullable$1(System.Int32))))(Bridge.fn.bind(this, function (rootTargetObjectInstance) {
                    var $step = 0,
                        $jumpFromFinally,
                        $returnValue,
                        $async_e;

                    var $enumerator = new (Bridge.GeneratorEnumerator$1(System.Tuple$3(Windows.UI.Xaml.DependencyObject,Windows.UI.Xaml.DependencyProperty,System.Nullable$1(System.Int32))))(Bridge.fn.bind(this, function () {
                        try {
                            for (;;) {
                                switch ($step) {
                                    case 0: {
                                        return false;
                                    }
                                    default: {
                                        return false;
                                    }
                                }
                            }
                        } catch($async_e1) {
                            $async_e = System.Exception.create($async_e1);
                            throw $async_e;
                        }
                    }));
                    return $enumerator;
                }, arguments));
            },
            setVisualStateProperty_ebf628272dcf4b65a353427065109841: function (finalTargetInstance, value) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Visibility").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "VisibilityProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                (finalTargetInstance).SetVisualStateValue(property, value);
            },
            setAnimationVisualStateProperty_ebf628272dcf4b65a353427065109841: function (finalTargetInstance, value) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Visibility").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "VisibilityProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                (finalTargetInstance).SetAnimationValue(property, value);
            },
            setLocalVisualStateProperty_ebf628272dcf4b65a353427065109841: function (finalTargetInstance, value) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Visibility").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "VisibilityProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                (finalTargetInstance).SetLocalValue(property, value);
            },
            getVisualStateProperty_ebf628272dcf4b65a353427065109841: function (finalTargetInstance) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Visibility").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "VisibilityProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                return finalTargetInstance.GetVisualStateValue(property);
            },
            accessVisualStateProperty_76ef4532488f4885aa150dbed9e1f51d: function (rootTargetObjectInstance) {
                return new (Bridge.GeneratorEnumerable$1(System.Tuple$3(Windows.UI.Xaml.DependencyObject,Windows.UI.Xaml.DependencyProperty,System.Nullable$1(System.Int32))))(Bridge.fn.bind(this, function (rootTargetObjectInstance) {
                    var $step = 0,
                        $jumpFromFinally,
                        $returnValue,
                        $async_e;

                    var $enumerator = new (Bridge.GeneratorEnumerator$1(System.Tuple$3(Windows.UI.Xaml.DependencyObject,Windows.UI.Xaml.DependencyProperty,System.Nullable$1(System.Int32))))(Bridge.fn.bind(this, function () {
                        try {
                            for (;;) {
                                switch ($step) {
                                    case 0: {
                                        return false;
                                    }
                                    default: {
                                        return false;
                                    }
                                }
                            }
                        } catch($async_e1) {
                            $async_e = System.Exception.create($async_e1);
                            throw $async_e;
                        }
                    }));
                    return $enumerator;
                }, arguments));
            },
            setVisualStateProperty_76ef4532488f4885aa150dbed9e1f51d: function (finalTargetInstance, value) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Visibility").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "VisibilityProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                (finalTargetInstance).SetVisualStateValue(property, value);
            },
            setAnimationVisualStateProperty_76ef4532488f4885aa150dbed9e1f51d: function (finalTargetInstance, value) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Visibility").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "VisibilityProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                (finalTargetInstance).SetAnimationValue(property, value);
            },
            setLocalVisualStateProperty_76ef4532488f4885aa150dbed9e1f51d: function (finalTargetInstance, value) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Visibility").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "VisibilityProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                (finalTargetInstance).SetLocalValue(property, value);
            },
            getVisualStateProperty_76ef4532488f4885aa150dbed9e1f51d: function (finalTargetInstance) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Visibility").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "VisibilityProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                return finalTargetInstance.GetVisualStateValue(property);
            },
            accessVisualStateProperty_40769f64a88e4748b437e3c63f7feb36: function (rootTargetObjectInstance) {
                return new (Bridge.GeneratorEnumerable$1(System.Tuple$3(Windows.UI.Xaml.DependencyObject,Windows.UI.Xaml.DependencyProperty,System.Nullable$1(System.Int32))))(Bridge.fn.bind(this, function (rootTargetObjectInstance) {
                    var $step = 0,
                        $jumpFromFinally,
                        $returnValue,
                        $async_e;

                    var $enumerator = new (Bridge.GeneratorEnumerator$1(System.Tuple$3(Windows.UI.Xaml.DependencyObject,Windows.UI.Xaml.DependencyProperty,System.Nullable$1(System.Int32))))(Bridge.fn.bind(this, function () {
                        try {
                            for (;;) {
                                switch ($step) {
                                    case 0: {
                                        return false;
                                    }
                                    default: {
                                        return false;
                                    }
                                }
                            }
                        } catch($async_e1) {
                            $async_e = System.Exception.create($async_e1);
                            throw $async_e;
                        }
                    }));
                    return $enumerator;
                }, arguments));
            },
            setVisualStateProperty_40769f64a88e4748b437e3c63f7feb36: function (finalTargetInstance, value) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Visibility").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "VisibilityProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                (finalTargetInstance).SetVisualStateValue(property, value);
            },
            setAnimationVisualStateProperty_40769f64a88e4748b437e3c63f7feb36: function (finalTargetInstance, value) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Visibility").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "VisibilityProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                (finalTargetInstance).SetAnimationValue(property, value);
            },
            setLocalVisualStateProperty_40769f64a88e4748b437e3c63f7feb36: function (finalTargetInstance, value) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Visibility").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "VisibilityProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                (finalTargetInstance).SetLocalValue(property, value);
            },
            getVisualStateProperty_40769f64a88e4748b437e3c63f7feb36: function (finalTargetInstance) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Visibility").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "VisibilityProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                return finalTargetInstance.GetVisualStateValue(property);
            },
            accessVisualStateProperty_6614f195e873451594de7a80c986a982: function (rootTargetObjectInstance) {
                return new (Bridge.GeneratorEnumerable$1(System.Tuple$3(Windows.UI.Xaml.DependencyObject,Windows.UI.Xaml.DependencyProperty,System.Nullable$1(System.Int32))))(Bridge.fn.bind(this, function (rootTargetObjectInstance) {
                    var $step = 0,
                        $jumpFromFinally,
                        $returnValue,
                        $async_e;

                    var $enumerator = new (Bridge.GeneratorEnumerator$1(System.Tuple$3(Windows.UI.Xaml.DependencyObject,Windows.UI.Xaml.DependencyProperty,System.Nullable$1(System.Int32))))(Bridge.fn.bind(this, function () {
                        try {
                            for (;;) {
                                switch ($step) {
                                    case 0: {
                                        return false;
                                    }
                                    default: {
                                        return false;
                                    }
                                }
                            }
                        } catch($async_e1) {
                            $async_e = System.Exception.create($async_e1);
                            throw $async_e;
                        }
                    }));
                    return $enumerator;
                }, arguments));
            },
            setVisualStateProperty_6614f195e873451594de7a80c986a982: function (finalTargetInstance, value) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Visibility").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "VisibilityProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                (finalTargetInstance).SetVisualStateValue(property, value);
            },
            setAnimationVisualStateProperty_6614f195e873451594de7a80c986a982: function (finalTargetInstance, value) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Visibility").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "VisibilityProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                (finalTargetInstance).SetAnimationValue(property, value);
            },
            setLocalVisualStateProperty_6614f195e873451594de7a80c986a982: function (finalTargetInstance, value) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Visibility").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "VisibilityProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                (finalTargetInstance).SetLocalValue(property, value);
            },
            getVisualStateProperty_6614f195e873451594de7a80c986a982: function (finalTargetInstance) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Visibility").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "VisibilityProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                return finalTargetInstance.GetVisualStateValue(property);
            },
            accessVisualStateProperty_d7801f67623d4cf4bd836a6ad7a94581: function (rootTargetObjectInstance) {
                return new (Bridge.GeneratorEnumerable$1(System.Tuple$3(Windows.UI.Xaml.DependencyObject,Windows.UI.Xaml.DependencyProperty,System.Nullable$1(System.Int32))))(Bridge.fn.bind(this, function (rootTargetObjectInstance) {
                    var $step = 0,
                        $jumpFromFinally,
                        $returnValue,
                        $async_e;

                    var $enumerator = new (Bridge.GeneratorEnumerator$1(System.Tuple$3(Windows.UI.Xaml.DependencyObject,Windows.UI.Xaml.DependencyProperty,System.Nullable$1(System.Int32))))(Bridge.fn.bind(this, function () {
                        try {
                            for (;;) {
                                switch ($step) {
                                    case 0: {
                                        return false;
                                    }
                                    default: {
                                        return false;
                                    }
                                }
                            }
                        } catch($async_e1) {
                            $async_e = System.Exception.create($async_e1);
                            throw $async_e;
                        }
                    }));
                    return $enumerator;
                }, arguments));
            },
            setVisualStateProperty_d7801f67623d4cf4bd836a6ad7a94581: function (finalTargetInstance, value) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Visibility").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "VisibilityProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                (finalTargetInstance).SetVisualStateValue(property, value);
            },
            setAnimationVisualStateProperty_d7801f67623d4cf4bd836a6ad7a94581: function (finalTargetInstance, value) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Visibility").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "VisibilityProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                (finalTargetInstance).SetAnimationValue(property, value);
            },
            setLocalVisualStateProperty_d7801f67623d4cf4bd836a6ad7a94581: function (finalTargetInstance, value) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Visibility").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "VisibilityProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                (finalTargetInstance).SetLocalValue(property, value);
            },
            getVisualStateProperty_d7801f67623d4cf4bd836a6ad7a94581: function (finalTargetInstance) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Visibility").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "VisibilityProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                return finalTargetInstance.GetVisualStateValue(property);
            },
            accessVisualStateProperty_4eef6f93d3e749eeae7e857390e0167f: function (rootTargetObjectInstance) {
                return new (Bridge.GeneratorEnumerable$1(System.Tuple$3(Windows.UI.Xaml.DependencyObject,Windows.UI.Xaml.DependencyProperty,System.Nullable$1(System.Int32))))(Bridge.fn.bind(this, function (rootTargetObjectInstance) {
                    var $step = 0,
                        $jumpFromFinally,
                        $returnValue,
                        $async_e;

                    var $enumerator = new (Bridge.GeneratorEnumerator$1(System.Tuple$3(Windows.UI.Xaml.DependencyObject,Windows.UI.Xaml.DependencyProperty,System.Nullable$1(System.Int32))))(Bridge.fn.bind(this, function () {
                        try {
                            for (;;) {
                                switch ($step) {
                                    case 0: {
                                        return false;
                                    }
                                    default: {
                                        return false;
                                    }
                                }
                            }
                        } catch($async_e1) {
                            $async_e = System.Exception.create($async_e1);
                            throw $async_e;
                        }
                    }));
                    return $enumerator;
                }, arguments));
            },
            setVisualStateProperty_4eef6f93d3e749eeae7e857390e0167f: function (finalTargetInstance, value) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Visibility").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "VisibilityProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                (finalTargetInstance).SetVisualStateValue(property, value);
            },
            setAnimationVisualStateProperty_4eef6f93d3e749eeae7e857390e0167f: function (finalTargetInstance, value) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Visibility").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "VisibilityProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                (finalTargetInstance).SetAnimationValue(property, value);
            },
            setLocalVisualStateProperty_4eef6f93d3e749eeae7e857390e0167f: function (finalTargetInstance, value) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Visibility").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "VisibilityProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                (finalTargetInstance).SetLocalValue(property, value);
            },
            getVisualStateProperty_4eef6f93d3e749eeae7e857390e0167f: function (finalTargetInstance) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Visibility").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "VisibilityProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                return finalTargetInstance.GetVisualStateValue(property);
            },
            accessVisualStateProperty_8eee3238ec6d43b09ac4408fedebf9f9: function (rootTargetObjectInstance) {
                return new (Bridge.GeneratorEnumerable$1(System.Tuple$3(Windows.UI.Xaml.DependencyObject,Windows.UI.Xaml.DependencyProperty,System.Nullable$1(System.Int32))))(Bridge.fn.bind(this, function (rootTargetObjectInstance) {
                    var $step = 0,
                        $jumpFromFinally,
                        $returnValue,
                        $async_e;

                    var $enumerator = new (Bridge.GeneratorEnumerator$1(System.Tuple$3(Windows.UI.Xaml.DependencyObject,Windows.UI.Xaml.DependencyProperty,System.Nullable$1(System.Int32))))(Bridge.fn.bind(this, function () {
                        try {
                            for (;;) {
                                switch ($step) {
                                    case 0: {
                                        return false;
                                    }
                                    default: {
                                        return false;
                                    }
                                }
                            }
                        } catch($async_e1) {
                            $async_e = System.Exception.create($async_e1);
                            throw $async_e;
                        }
                    }));
                    return $enumerator;
                }, arguments));
            },
            setVisualStateProperty_8eee3238ec6d43b09ac4408fedebf9f9: function (finalTargetInstance, value) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Visibility").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "VisibilityProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                (finalTargetInstance).SetVisualStateValue(property, value);
            },
            setAnimationVisualStateProperty_8eee3238ec6d43b09ac4408fedebf9f9: function (finalTargetInstance, value) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Visibility").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "VisibilityProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                (finalTargetInstance).SetAnimationValue(property, value);
            },
            setLocalVisualStateProperty_8eee3238ec6d43b09ac4408fedebf9f9: function (finalTargetInstance, value) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Visibility").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "VisibilityProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                (finalTargetInstance).SetLocalValue(property, value);
            },
            getVisualStateProperty_8eee3238ec6d43b09ac4408fedebf9f9: function (finalTargetInstance) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Visibility").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "VisibilityProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                return finalTargetInstance.GetVisualStateValue(property);
            },
            accessVisualStateProperty_3ca0493c2beb46c1ade814496c7ab2b2: function (rootTargetObjectInstance) {
                return new (Bridge.GeneratorEnumerable$1(System.Tuple$3(Windows.UI.Xaml.DependencyObject,Windows.UI.Xaml.DependencyProperty,System.Nullable$1(System.Int32))))(Bridge.fn.bind(this, function (rootTargetObjectInstance) {
                    var $step = 0,
                        $jumpFromFinally,
                        $returnValue,
                        $async_e;

                    var $enumerator = new (Bridge.GeneratorEnumerator$1(System.Tuple$3(Windows.UI.Xaml.DependencyObject,Windows.UI.Xaml.DependencyProperty,System.Nullable$1(System.Int32))))(Bridge.fn.bind(this, function () {
                        try {
                            for (;;) {
                                switch ($step) {
                                    case 0: {
                                        return false;
                                    }
                                    default: {
                                        return false;
                                    }
                                }
                            }
                        } catch($async_e1) {
                            $async_e = System.Exception.create($async_e1);
                            throw $async_e;
                        }
                    }));
                    return $enumerator;
                }, arguments));
            },
            setVisualStateProperty_3ca0493c2beb46c1ade814496c7ab2b2: function (finalTargetInstance, value) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Visibility").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "VisibilityProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                (finalTargetInstance).SetVisualStateValue(property, value);
            },
            setAnimationVisualStateProperty_3ca0493c2beb46c1ade814496c7ab2b2: function (finalTargetInstance, value) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Visibility").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "VisibilityProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                (finalTargetInstance).SetAnimationValue(property, value);
            },
            setLocalVisualStateProperty_3ca0493c2beb46c1ade814496c7ab2b2: function (finalTargetInstance, value) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Visibility").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "VisibilityProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                (finalTargetInstance).SetLocalValue(property, value);
            },
            getVisualStateProperty_3ca0493c2beb46c1ade814496c7ab2b2: function (finalTargetInstance) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Visibility").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "VisibilityProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                return finalTargetInstance.GetVisualStateValue(property);
            },
            accessVisualStateProperty_857ad27fff214fe799a6935f64633a23: function (rootTargetObjectInstance) {
                return new (Bridge.GeneratorEnumerable$1(System.Tuple$3(Windows.UI.Xaml.DependencyObject,Windows.UI.Xaml.DependencyProperty,System.Nullable$1(System.Int32))))(Bridge.fn.bind(this, function (rootTargetObjectInstance) {
                    var $step = 0,
                        $jumpFromFinally,
                        $returnValue,
                        $async_e;

                    var $enumerator = new (Bridge.GeneratorEnumerator$1(System.Tuple$3(Windows.UI.Xaml.DependencyObject,Windows.UI.Xaml.DependencyProperty,System.Nullable$1(System.Int32))))(Bridge.fn.bind(this, function () {
                        try {
                            for (;;) {
                                switch ($step) {
                                    case 0: {
                                        return false;
                                    }
                                    default: {
                                        return false;
                                    }
                                }
                            }
                        } catch($async_e1) {
                            $async_e = System.Exception.create($async_e1);
                            throw $async_e;
                        }
                    }));
                    return $enumerator;
                }, arguments));
            },
            setVisualStateProperty_857ad27fff214fe799a6935f64633a23: function (finalTargetInstance, value) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Visibility").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "VisibilityProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                (finalTargetInstance).SetVisualStateValue(property, value);
            },
            setAnimationVisualStateProperty_857ad27fff214fe799a6935f64633a23: function (finalTargetInstance, value) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Visibility").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "VisibilityProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                (finalTargetInstance).SetAnimationValue(property, value);
            },
            setLocalVisualStateProperty_857ad27fff214fe799a6935f64633a23: function (finalTargetInstance, value) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Visibility").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "VisibilityProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                (finalTargetInstance).SetLocalValue(property, value);
            },
            getVisualStateProperty_857ad27fff214fe799a6935f64633a23: function (finalTargetInstance) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Visibility").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "VisibilityProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                return finalTargetInstance.GetVisualStateValue(property);
            },
            accessVisualStateProperty_462951debfc84f228ddcd015b8011ed3: function (rootTargetObjectInstance) {
                return new (Bridge.GeneratorEnumerable$1(System.Tuple$3(Windows.UI.Xaml.DependencyObject,Windows.UI.Xaml.DependencyProperty,System.Nullable$1(System.Int32))))(Bridge.fn.bind(this, function (rootTargetObjectInstance) {
                    var $step = 0,
                        $jumpFromFinally,
                        $returnValue,
                        $async_e;

                    var $enumerator = new (Bridge.GeneratorEnumerator$1(System.Tuple$3(Windows.UI.Xaml.DependencyObject,Windows.UI.Xaml.DependencyProperty,System.Nullable$1(System.Int32))))(Bridge.fn.bind(this, function () {
                        try {
                            for (;;) {
                                switch ($step) {
                                    case 0: {
                                        return false;
                                    }
                                    default: {
                                        return false;
                                    }
                                }
                            }
                        } catch($async_e1) {
                            $async_e = System.Exception.create($async_e1);
                            throw $async_e;
                        }
                    }));
                    return $enumerator;
                }, arguments));
            },
            setVisualStateProperty_462951debfc84f228ddcd015b8011ed3: function (finalTargetInstance, value) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Visibility").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "VisibilityProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                (finalTargetInstance).SetVisualStateValue(property, value);
            },
            setAnimationVisualStateProperty_462951debfc84f228ddcd015b8011ed3: function (finalTargetInstance, value) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Visibility").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "VisibilityProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                (finalTargetInstance).SetAnimationValue(property, value);
            },
            setLocalVisualStateProperty_462951debfc84f228ddcd015b8011ed3: function (finalTargetInstance, value) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Visibility").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "VisibilityProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                (finalTargetInstance).SetLocalValue(property, value);
            },
            getVisualStateProperty_462951debfc84f228ddcd015b8011ed3: function (finalTargetInstance) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Visibility").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "VisibilityProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                return finalTargetInstance.GetVisualStateValue(property);
            }
        }
    });

    Bridge.define("___CSHTML5__Component__Defaultstyles__INTERNAL_Defaultdatepickerstyle__Xaml", {
        inherits: [Bridge.global.Windows.UI.Xaml.ResourceDictionary],
        fields: {
            _contentLoaded: false
        },
        ctors: {
            ctor: function () {
                this.$initialize();
                Bridge.global.Windows.UI.Xaml.ResourceDictionary.ctor.call(this);
                this.InitializeComponent();

                if (Bridge.is(this, Windows.UI.Xaml.UIElement)) {
                    Bridge.cast(this, Windows.UI.Xaml.UIElement).XamlSourcePath = "CSHTML5\\DefaultStyles\\INTERNAL_DefaultDatePickerStyle.xaml";
                }
            }
        },
        methods: {
            InitializeComponent: function () {
                var $t;
                if (this._contentLoaded) {
                    return;
                }
                this._contentLoaded = true;


                if (Bridge.is(this, Windows.UI.Xaml.UIElement)) {
                    Bridge.cast(this, Windows.UI.Xaml.UIElement).XamlSourcePath = "CSHTML5\\DefaultStyles\\INTERNAL_DefaultDatePickerStyle.xaml";
                }



                var Style_8df5e6e9b01448979a365694a74e9554 = new Bridge.global.Windows.UI.Xaml.Style.ctor();
                Style_8df5e6e9b01448979a365694a74e9554.TargetType = Bridge.global.Windows.UI.Xaml.Controls.DatePicker;
                var Setter_23a9a5235b4d43818b373f3126e2df02 = new Bridge.global.Windows.UI.Xaml.Setter.ctor();
                Setter_23a9a5235b4d43818b373f3126e2df02.Property = Bridge.global.Windows.UI.Xaml.Controls.Control.BorderBrushProperty;
                Setter_23a9a5235b4d43818b373f3126e2df02.Value = new Bridge.global.Windows.UI.Xaml.Media.SolidColorBrush.$ctor1(($t = new Bridge.global.Windows.UI.Color(), $t.A = 255, $t.R = 128, $t.G = 128, $t.B = 128, $t));

                var Setter_2df583cb0df04b5a8af06259a76b7e88 = new Bridge.global.Windows.UI.Xaml.Setter.ctor();
                Setter_2df583cb0df04b5a8af06259a76b7e88.Property = Bridge.global.Windows.UI.Xaml.Controls.Control.BorderThicknessProperty;
                Setter_2df583cb0df04b5a8af06259a76b7e88.Value = new Bridge.global.Windows.UI.Xaml.Thickness.$ctor1(1).$clone();

                var Setter_d2fea39a155b4a85b402da8dc96d3dea = new Bridge.global.Windows.UI.Xaml.Setter.ctor();
                Setter_d2fea39a155b4a85b402da8dc96d3dea.Property = Bridge.global.Windows.UI.Xaml.Controls.Control.TemplateProperty;
                var ControlTemplate_b3b40eee606e45d3a5be8b3deaa15a1d = new Bridge.global.Windows.UI.Xaml.Controls.ControlTemplate();
                ControlTemplate_b3b40eee606e45d3a5be8b3deaa15a1d.TargetType = Bridge.global.Windows.UI.Xaml.Controls.DatePicker;
                ControlTemplate_b3b40eee606e45d3a5be8b3deaa15a1d.SetMethodToInstantiateFrameworkTemplate(function (templateOwner_ControlTemplate_b3b40eee606e45d3a5be8b3deaa15a1d) {
                    var templateInstance_608b8ac3c954440595974389057c6100 = new Bridge.global.Windows.UI.Xaml.TemplateInstance();
                    templateInstance_608b8ac3c954440595974389057c6100.TemplateOwner = templateOwner_ControlTemplate_b3b40eee606e45d3a5be8b3deaa15a1d;
                    var TextBox_54add5fd283540f6bb364426c19d459a = new Bridge.global.Windows.UI.Xaml.Controls.TextBox();
                    var Button_a0d375099fc54fa899e738e97592b851 = new Bridge.global.Windows.UI.Xaml.Controls.Button();
                    var Popup_6e14d2f2c5144a5d820f8c0044943ff3 = new Bridge.global.Windows.UI.Xaml.Controls.Primitives.Popup();
                    var Grid_89f4361e45f147c88ef7f714cffd6c5d = new Bridge.global.Windows.UI.Xaml.Controls.Grid();
                    templateOwner_ControlTemplate_b3b40eee606e45d3a5be8b3deaa15a1d.RegisterName("Root", Grid_89f4361e45f147c88ef7f714cffd6c5d);
                    Grid_89f4361e45f147c88ef7f714cffd6c5d.Name = "Root";
                    var ColumnDefinition_db8f1e6dd6184a55b3f0033470648804 = new Bridge.global.Windows.UI.Xaml.Controls.ColumnDefinition();
                    ColumnDefinition_db8f1e6dd6184a55b3f0033470648804.Width = new Bridge.global.Windows.UI.Xaml.GridLength.$ctor2(1.0, Bridge.global.Windows.UI.Xaml.GridUnitType.Star);

                    var ColumnDefinition_fd0a387fb7c548499b26df57d9e435aa = new Bridge.global.Windows.UI.Xaml.Controls.ColumnDefinition();
                    ColumnDefinition_fd0a387fb7c548499b26df57d9e435aa.Width = new Bridge.global.Windows.UI.Xaml.GridLength.$ctor2(1.0, Bridge.global.Windows.UI.Xaml.GridUnitType.Auto);

                    Grid_89f4361e45f147c88ef7f714cffd6c5d.ColumnDefinitions.add(ColumnDefinition_db8f1e6dd6184a55b3f0033470648804);
                    Grid_89f4361e45f147c88ef7f714cffd6c5d.ColumnDefinitions.add(ColumnDefinition_fd0a387fb7c548499b26df57d9e435aa);

                    templateOwner_ControlTemplate_b3b40eee606e45d3a5be8b3deaa15a1d.RegisterName("TextBox", TextBox_54add5fd283540f6bb364426c19d459a);
                    TextBox_54add5fd283540f6bb364426c19d459a.Name = "TextBox";
                    Bridge.global.Windows.UI.Xaml.Controls.Grid.SetColumn(TextBox_54add5fd283540f6bb364426c19d459a, 0);
                    var Binding_94bb20338ceb4a70a3f36de9b5666df8 = new Bridge.global.Windows.UI.Xaml.Data.Binding.ctor();
                    Binding_94bb20338ceb4a70a3f36de9b5666df8.Path = new Bridge.global.Windows.UI.Xaml.PropertyPath.ctor("Background");
                    var RelativeSource_3b5fc7ceda9c41f590a3898b2cd0bfc5 = new Bridge.global.Windows.UI.Xaml.Data.RelativeSource.ctor();
                    RelativeSource_3b5fc7ceda9c41f590a3898b2cd0bfc5.Mode = Bridge.global.Windows.UI.Xaml.Data.RelativeSourceMode.TemplatedParent;

                    Binding_94bb20338ceb4a70a3f36de9b5666df8.RelativeSource = RelativeSource_3b5fc7ceda9c41f590a3898b2cd0bfc5;


                    Binding_94bb20338ceb4a70a3f36de9b5666df8.TemplateOwner = templateInstance_608b8ac3c954440595974389057c6100;

                    var Binding_ad179499f83f46369fcf49ea350cc826 = new Bridge.global.Windows.UI.Xaml.Data.Binding.ctor();
                    Binding_ad179499f83f46369fcf49ea350cc826.Path = new Bridge.global.Windows.UI.Xaml.PropertyPath.ctor("BorderBrush");
                    var RelativeSource_16c435c90fd84f84bd576996a672b524 = new Bridge.global.Windows.UI.Xaml.Data.RelativeSource.ctor();
                    RelativeSource_16c435c90fd84f84bd576996a672b524.Mode = Bridge.global.Windows.UI.Xaml.Data.RelativeSourceMode.TemplatedParent;

                    Binding_ad179499f83f46369fcf49ea350cc826.RelativeSource = RelativeSource_16c435c90fd84f84bd576996a672b524;


                    Binding_ad179499f83f46369fcf49ea350cc826.TemplateOwner = templateInstance_608b8ac3c954440595974389057c6100;

                    var Binding_cbafd2fb43ac4b04825a527e097dfeff = new Bridge.global.Windows.UI.Xaml.Data.Binding.ctor();
                    Binding_cbafd2fb43ac4b04825a527e097dfeff.Path = new Bridge.global.Windows.UI.Xaml.PropertyPath.ctor("BorderThickness");
                    var RelativeSource_f337ad3ab51c415d8851fe825aea876b = new Bridge.global.Windows.UI.Xaml.Data.RelativeSource.ctor();
                    RelativeSource_f337ad3ab51c415d8851fe825aea876b.Mode = Bridge.global.Windows.UI.Xaml.Data.RelativeSourceMode.TemplatedParent;

                    Binding_cbafd2fb43ac4b04825a527e097dfeff.RelativeSource = RelativeSource_f337ad3ab51c415d8851fe825aea876b;


                    Binding_cbafd2fb43ac4b04825a527e097dfeff.TemplateOwner = templateInstance_608b8ac3c954440595974389057c6100;

                    var Binding_fb69dc64bc4840a2b13c42d3ff4da8d5 = new Bridge.global.Windows.UI.Xaml.Data.Binding.ctor();
                    Binding_fb69dc64bc4840a2b13c42d3ff4da8d5.Path = new Bridge.global.Windows.UI.Xaml.PropertyPath.ctor("Padding");
                    var RelativeSource_cfa0593ddeb14122895c0189aa48635b = new Bridge.global.Windows.UI.Xaml.Data.RelativeSource.ctor();
                    RelativeSource_cfa0593ddeb14122895c0189aa48635b.Mode = Bridge.global.Windows.UI.Xaml.Data.RelativeSourceMode.TemplatedParent;

                    Binding_fb69dc64bc4840a2b13c42d3ff4da8d5.RelativeSource = RelativeSource_cfa0593ddeb14122895c0189aa48635b;


                    Binding_fb69dc64bc4840a2b13c42d3ff4da8d5.TemplateOwner = templateInstance_608b8ac3c954440595974389057c6100;


                    templateOwner_ControlTemplate_b3b40eee606e45d3a5be8b3deaa15a1d.RegisterName("Button", Button_a0d375099fc54fa899e738e97592b851);
                    Button_a0d375099fc54fa899e738e97592b851.Name = "Button";
                    Bridge.global.Windows.UI.Xaml.Controls.Grid.SetColumn(Button_a0d375099fc54fa899e738e97592b851, 1);
                    Button_a0d375099fc54fa899e738e97592b851.Content = "\u25bc";
                    Button_a0d375099fc54fa899e738e97592b851.Width = 30.0;
                    Button_a0d375099fc54fa899e738e97592b851.Margin = new Bridge.global.Windows.UI.Xaml.Thickness.$ctor2(2, 0, 2, 0);
                    var Binding_19cd9f5cb1dc421fa53086e59a0d15db = new Bridge.global.Windows.UI.Xaml.Data.Binding.ctor();
                    Binding_19cd9f5cb1dc421fa53086e59a0d15db.Path = new Bridge.global.Windows.UI.Xaml.PropertyPath.ctor("Foreground");
                    var RelativeSource_cdd8f71dc6b04dfdb38650b128f7f82e = new Bridge.global.Windows.UI.Xaml.Data.RelativeSource.ctor();
                    RelativeSource_cdd8f71dc6b04dfdb38650b128f7f82e.Mode = Bridge.global.Windows.UI.Xaml.Data.RelativeSourceMode.TemplatedParent;

                    Binding_19cd9f5cb1dc421fa53086e59a0d15db.RelativeSource = RelativeSource_cdd8f71dc6b04dfdb38650b128f7f82e;


                    Binding_19cd9f5cb1dc421fa53086e59a0d15db.TemplateOwner = templateInstance_608b8ac3c954440595974389057c6100;

                    var Binding_053f8de18cc14e05bfe1cff6addbbeaf = new Bridge.global.Windows.UI.Xaml.Data.Binding.ctor();
                    Binding_053f8de18cc14e05bfe1cff6addbbeaf.Path = new Bridge.global.Windows.UI.Xaml.PropertyPath.ctor("BorderBrush");
                    var RelativeSource_70bf3425bd13448eaab3140072f195d6 = new Bridge.global.Windows.UI.Xaml.Data.RelativeSource.ctor();
                    RelativeSource_70bf3425bd13448eaab3140072f195d6.Mode = Bridge.global.Windows.UI.Xaml.Data.RelativeSourceMode.TemplatedParent;

                    Binding_053f8de18cc14e05bfe1cff6addbbeaf.RelativeSource = RelativeSource_70bf3425bd13448eaab3140072f195d6;


                    Binding_053f8de18cc14e05bfe1cff6addbbeaf.TemplateOwner = templateInstance_608b8ac3c954440595974389057c6100;

                    var Binding_3283994e0e1c4b0984f9c5430b2130b2 = new Bridge.global.Windows.UI.Xaml.Data.Binding.ctor();
                    Binding_3283994e0e1c4b0984f9c5430b2130b2.Path = new Bridge.global.Windows.UI.Xaml.PropertyPath.ctor("BorderThickness");
                    var RelativeSource_eb11ae1b681e456bb105203984a5cd17 = new Bridge.global.Windows.UI.Xaml.Data.RelativeSource.ctor();
                    RelativeSource_eb11ae1b681e456bb105203984a5cd17.Mode = Bridge.global.Windows.UI.Xaml.Data.RelativeSourceMode.TemplatedParent;

                    Binding_3283994e0e1c4b0984f9c5430b2130b2.RelativeSource = RelativeSource_eb11ae1b681e456bb105203984a5cd17;


                    Binding_3283994e0e1c4b0984f9c5430b2130b2.TemplateOwner = templateInstance_608b8ac3c954440595974389057c6100;


                    templateOwner_ControlTemplate_b3b40eee606e45d3a5be8b3deaa15a1d.RegisterName("Popup", Popup_6e14d2f2c5144a5d820f8c0044943ff3);
                    Popup_6e14d2f2c5144a5d820f8c0044943ff3.Name = "Popup";
                    Popup_6e14d2f2c5144a5d820f8c0044943ff3.VerticalAlignment = Bridge.global.Windows.UI.Xaml.VerticalAlignment.Bottom;

                    Grid_89f4361e45f147c88ef7f714cffd6c5d.Children.add(TextBox_54add5fd283540f6bb364426c19d459a);
                    Grid_89f4361e45f147c88ef7f714cffd6c5d.Children.add(Button_a0d375099fc54fa899e738e97592b851);
                    Grid_89f4361e45f147c88ef7f714cffd6c5d.Children.add(Popup_6e14d2f2c5144a5d820f8c0044943ff3);


                    Bridge.global.Windows.UI.Xaml.Data.BindingOperations.SetBinding(TextBox_54add5fd283540f6bb364426c19d459a, Bridge.global.Windows.UI.Xaml.Controls.Control.BackgroundProperty, Binding_94bb20338ceb4a70a3f36de9b5666df8);
                    Bridge.global.Windows.UI.Xaml.Data.BindingOperations.SetBinding(TextBox_54add5fd283540f6bb364426c19d459a, Bridge.global.Windows.UI.Xaml.Controls.Control.BorderBrushProperty, Binding_ad179499f83f46369fcf49ea350cc826);
                    Bridge.global.Windows.UI.Xaml.Data.BindingOperations.SetBinding(TextBox_54add5fd283540f6bb364426c19d459a, Bridge.global.Windows.UI.Xaml.Controls.Control.BorderThicknessProperty, Binding_cbafd2fb43ac4b04825a527e097dfeff);
                    Bridge.global.Windows.UI.Xaml.Data.BindingOperations.SetBinding(TextBox_54add5fd283540f6bb364426c19d459a, Bridge.global.Windows.UI.Xaml.Controls.Control.PaddingProperty, Binding_fb69dc64bc4840a2b13c42d3ff4da8d5);
                    Bridge.global.Windows.UI.Xaml.Data.BindingOperations.SetBinding(Button_a0d375099fc54fa899e738e97592b851, Bridge.global.Windows.UI.Xaml.Controls.Control.ForegroundProperty, Binding_19cd9f5cb1dc421fa53086e59a0d15db);
                    Bridge.global.Windows.UI.Xaml.Data.BindingOperations.SetBinding(Button_a0d375099fc54fa899e738e97592b851, Bridge.global.Windows.UI.Xaml.Controls.Control.BorderBrushProperty, Binding_053f8de18cc14e05bfe1cff6addbbeaf);
                    Bridge.global.Windows.UI.Xaml.Data.BindingOperations.SetBinding(Button_a0d375099fc54fa899e738e97592b851, Bridge.global.Windows.UI.Xaml.Controls.Control.BorderThicknessProperty, Binding_3283994e0e1c4b0984f9c5430b2130b2);

                    templateInstance_608b8ac3c954440595974389057c6100.TemplateContent = Grid_89f4361e45f147c88ef7f714cffd6c5d;
                    return templateInstance_608b8ac3c954440595974389057c6100;
                });

                Setter_d2fea39a155b4a85b402da8dc96d3dea.Value = ControlTemplate_b3b40eee606e45d3a5be8b3deaa15a1d;


                Style_8df5e6e9b01448979a365694a74e9554.Setters.add(Setter_23a9a5235b4d43818b373f3126e2df02);
                Style_8df5e6e9b01448979a365694a74e9554.Setters.add(Setter_2df583cb0df04b5a8af06259a76b7e88);
                Style_8df5e6e9b01448979a365694a74e9554.Setters.add(Setter_d2fea39a155b4a85b402da8dc96d3dea);


                this.setItem(Bridge.global.Windows.UI.Xaml.Controls.DatePicker, Style_8df5e6e9b01448979a365694a74e9554);






            }
        }
    });

    Bridge.define("___CSHTML5__Component__Defaultstyles__INTERNAL_Defaultexpanderstyle__Xaml", {
        inherits: [Bridge.global.Windows.UI.Xaml.ResourceDictionary],
        fields: {
            _contentLoaded: false
        },
        ctors: {
            ctor: function () {
                this.$initialize();
                Bridge.global.Windows.UI.Xaml.ResourceDictionary.ctor.call(this);
                this.InitializeComponent();

                if (Bridge.is(this, Windows.UI.Xaml.UIElement)) {
                    Bridge.cast(this, Windows.UI.Xaml.UIElement).XamlSourcePath = "CSHTML5\\DefaultStyles\\INTERNAL_DefaultExpanderStyle.xaml";
                }
            }
        },
        methods: {
            InitializeComponent: function () {
                if (this._contentLoaded) {
                    return;
                }
                this._contentLoaded = true;


                if (Bridge.is(this, Windows.UI.Xaml.UIElement)) {
                    Bridge.cast(this, Windows.UI.Xaml.UIElement).XamlSourcePath = "CSHTML5\\DefaultStyles\\INTERNAL_DefaultExpanderStyle.xaml";
                }



                var Style_273df2acc5ad457bb6acff0ffc3451af = new Bridge.global.Windows.UI.Xaml.Style.ctor();
                Style_273df2acc5ad457bb6acff0ffc3451af.TargetType = Bridge.global.Windows.UI.Xaml.Controls.Expander;
                var Setter_3bfd0120357f4ab9bd97c4d130a356f1 = new Bridge.global.Windows.UI.Xaml.Setter.ctor();
                Setter_3bfd0120357f4ab9bd97c4d130a356f1.Property = Bridge.global.Windows.UI.Xaml.Controls.Control.TemplateProperty;
                var ControlTemplate_8e5a2e0f304f44c78d3e4d0e9f702e67 = new Bridge.global.Windows.UI.Xaml.Controls.ControlTemplate();
                ControlTemplate_8e5a2e0f304f44c78d3e4d0e9f702e67.TargetType = Bridge.global.Windows.UI.Xaml.Controls.Expander;
                ControlTemplate_8e5a2e0f304f44c78d3e4d0e9f702e67.SetMethodToInstantiateFrameworkTemplate(Bridge.fn.bind(this, function (templateOwner_ControlTemplate_8e5a2e0f304f44c78d3e4d0e9f702e67) {
                    var templateInstance_4ce613c5064441719ef7751f39e59669 = new Bridge.global.Windows.UI.Xaml.TemplateInstance();
                    templateInstance_4ce613c5064441719ef7751f39e59669.TemplateOwner = templateOwner_ControlTemplate_8e5a2e0f304f44c78d3e4d0e9f702e67;
                    var VisualState_779d15e494894b2a943a00d70c46a2f3 = new Bridge.global.Windows.UI.Xaml.VisualState();
                    var VisualState_cb5f42653baf429fa395228f7527c859 = new Bridge.global.Windows.UI.Xaml.VisualState();
                    var VisualStateGroup_62b120cfe5eb4279a5d07850196e66e4 = new Bridge.global.Windows.UI.Xaml.VisualStateGroup();
                    var VisualState_cfdd23e5ce33499795f3522878d29b09 = new Bridge.global.Windows.UI.Xaml.VisualState();
                    var VisualState_fe7161e4ee3441df9d97bfa422642242 = new Bridge.global.Windows.UI.Xaml.VisualState();
                    var VisualStateGroup_9cb1ffb24a94463a9551daa629bcefdc = new Bridge.global.Windows.UI.Xaml.VisualStateGroup();
                    var VisualState_bc40572bf69d4f09b171db333a6f313d = new Bridge.global.Windows.UI.Xaml.VisualState();
                    var VisualState_c53b02fb3cfb46e387fe68ad8abe1cc7 = new Bridge.global.Windows.UI.Xaml.VisualState();
                    var VisualStateGroup_8d253317bdd24ea9bce0d44472739e24 = new Bridge.global.Windows.UI.Xaml.VisualStateGroup();
                    var ToggleButton_cd82c53a2f304e96acfc810363b3c0d2 = new Bridge.global.Windows.UI.Xaml.Controls.Primitives.ToggleButton();
                    var ContentControl_25ff6e1b99b942038f744702dffc35d8 = new Bridge.global.Windows.UI.Xaml.Controls.ContentControl();
                    var Border_6b8182905bfc445f85f99811fa802081 = new Bridge.global.Windows.UI.Xaml.Controls.Border();
                    Border_6b8182905bfc445f85f99811fa802081.CornerRadius = new Bridge.global.Windows.UI.Xaml.CornerRadius.$ctor1(3);
                    templateOwner_ControlTemplate_8e5a2e0f304f44c78d3e4d0e9f702e67.RegisterName("CommonStates", VisualStateGroup_62b120cfe5eb4279a5d07850196e66e4);
                    VisualStateGroup_62b120cfe5eb4279a5d07850196e66e4.Name = "CommonStates";
                    templateOwner_ControlTemplate_8e5a2e0f304f44c78d3e4d0e9f702e67.RegisterName("Normal", VisualState_779d15e494894b2a943a00d70c46a2f3);
                    VisualState_779d15e494894b2a943a00d70c46a2f3.Name = "Normal";

                    templateOwner_ControlTemplate_8e5a2e0f304f44c78d3e4d0e9f702e67.RegisterName("Disabled", VisualState_cb5f42653baf429fa395228f7527c859);
                    VisualState_cb5f42653baf429fa395228f7527c859.Name = "Disabled";

                    System.Array.add(VisualStateGroup_62b120cfe5eb4279a5d07850196e66e4.States, VisualState_779d15e494894b2a943a00d70c46a2f3, Object);
                    System.Array.add(VisualStateGroup_62b120cfe5eb4279a5d07850196e66e4.States, VisualState_cb5f42653baf429fa395228f7527c859, Object);


                    templateOwner_ControlTemplate_8e5a2e0f304f44c78d3e4d0e9f702e67.RegisterName("FocusStates", VisualStateGroup_9cb1ffb24a94463a9551daa629bcefdc);
                    VisualStateGroup_9cb1ffb24a94463a9551daa629bcefdc.Name = "FocusStates";
                    templateOwner_ControlTemplate_8e5a2e0f304f44c78d3e4d0e9f702e67.RegisterName("Focused", VisualState_cfdd23e5ce33499795f3522878d29b09);
                    VisualState_cfdd23e5ce33499795f3522878d29b09.Name = "Focused";

                    templateOwner_ControlTemplate_8e5a2e0f304f44c78d3e4d0e9f702e67.RegisterName("Unfocused", VisualState_fe7161e4ee3441df9d97bfa422642242);
                    VisualState_fe7161e4ee3441df9d97bfa422642242.Name = "Unfocused";

                    System.Array.add(VisualStateGroup_9cb1ffb24a94463a9551daa629bcefdc.States, VisualState_cfdd23e5ce33499795f3522878d29b09, Object);
                    System.Array.add(VisualStateGroup_9cb1ffb24a94463a9551daa629bcefdc.States, VisualState_fe7161e4ee3441df9d97bfa422642242, Object);


                    templateOwner_ControlTemplate_8e5a2e0f304f44c78d3e4d0e9f702e67.RegisterName("ExpansionStates", VisualStateGroup_8d253317bdd24ea9bce0d44472739e24);
                    VisualStateGroup_8d253317bdd24ea9bce0d44472739e24.Name = "ExpansionStates";
                    templateOwner_ControlTemplate_8e5a2e0f304f44c78d3e4d0e9f702e67.RegisterName("Collapsed", VisualState_bc40572bf69d4f09b171db333a6f313d);
                    VisualState_bc40572bf69d4f09b171db333a6f313d.Name = "Collapsed";

                    templateOwner_ControlTemplate_8e5a2e0f304f44c78d3e4d0e9f702e67.RegisterName("Expanded", VisualState_c53b02fb3cfb46e387fe68ad8abe1cc7);
                    VisualState_c53b02fb3cfb46e387fe68ad8abe1cc7.Name = "Expanded";
                    var Storyboard_58dd292c4e3443639c8ead5c44be407a = new Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard();
                    var ObjectAnimationUsingKeyFrames_543375fc60be4607b451c8895e1122d8 = new Bridge.global.Windows.UI.Xaml.Media.Animation.ObjectAnimationUsingKeyFrames();
                    ObjectAnimationUsingKeyFrames_543375fc60be4607b451c8895e1122d8.Duration = new Bridge.global.Windows.UI.Xaml.Duration.$ctor1(new Bridge.global.System.TimeSpan(System.Int64(0)));
                    Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard.SetTargetName(ObjectAnimationUsingKeyFrames_543375fc60be4607b451c8895e1122d8, "ExpandSite");
                    var DiscreteObjectKeyFrame_e897f70a1b444000a04dd2bc83f0801b = new Bridge.global.Windows.UI.Xaml.Media.Animation.DiscreteObjectKeyFrame();
                    DiscreteObjectKeyFrame_e897f70a1b444000a04dd2bc83f0801b.KeyTime = Bridge.global.Windows.UI.Xaml.Media.Animation.KeyTime.FromTimeSpan(new Bridge.global.System.TimeSpan(System.Int64(0)));
                    DiscreteObjectKeyFrame_e897f70a1b444000a04dd2bc83f0801b.Value = "Visible";

                    ObjectAnimationUsingKeyFrames_543375fc60be4607b451c8895e1122d8.KeyFrames.add(DiscreteObjectKeyFrame_e897f70a1b444000a04dd2bc83f0801b);


                    Storyboard_58dd292c4e3443639c8ead5c44be407a.Children.add(ObjectAnimationUsingKeyFrames_543375fc60be4607b451c8895e1122d8);


                    VisualState_c53b02fb3cfb46e387fe68ad8abe1cc7.Storyboard = Storyboard_58dd292c4e3443639c8ead5c44be407a;


                    System.Array.add(VisualStateGroup_8d253317bdd24ea9bce0d44472739e24.States, VisualState_bc40572bf69d4f09b171db333a6f313d, Object);
                    System.Array.add(VisualStateGroup_8d253317bdd24ea9bce0d44472739e24.States, VisualState_c53b02fb3cfb46e387fe68ad8abe1cc7, Object);


                    templateOwner_ControlTemplate_8e5a2e0f304f44c78d3e4d0e9f702e67.INTERNAL_GetVisualStateGroups().add(VisualStateGroup_62b120cfe5eb4279a5d07850196e66e4);
                    templateOwner_ControlTemplate_8e5a2e0f304f44c78d3e4d0e9f702e67.INTERNAL_GetVisualStateGroups().add(VisualStateGroup_9cb1ffb24a94463a9551daa629bcefdc);
                    templateOwner_ControlTemplate_8e5a2e0f304f44c78d3e4d0e9f702e67.INTERNAL_GetVisualStateGroups().add(VisualStateGroup_8d253317bdd24ea9bce0d44472739e24);

                    var StackPanel_d1d4cb82cbf941d2b0fdcb7d5d06d73d = new Bridge.global.Windows.UI.Xaml.Controls.StackPanel();
                    templateOwner_ControlTemplate_8e5a2e0f304f44c78d3e4d0e9f702e67.RegisterName("ExpanderButton", ToggleButton_cd82c53a2f304e96acfc810363b3c0d2);
                    ToggleButton_cd82c53a2f304e96acfc810363b3c0d2.Name = "ExpanderButton";
                    ToggleButton_cd82c53a2f304e96acfc810363b3c0d2.Margin = new Bridge.global.Windows.UI.Xaml.Thickness.$ctor1(1);
                    ToggleButton_cd82c53a2f304e96acfc810363b3c0d2.Cursor = Bridge.global.System.Windows.Input.Cursors.Hand;
                    var ControlTemplate_74d0bfe542b14de0982f91b468fb4444 = new Bridge.global.Windows.UI.Xaml.Controls.ControlTemplate();
                    ControlTemplate_74d0bfe542b14de0982f91b468fb4444.TargetType = Bridge.global.Windows.UI.Xaml.Controls.Primitives.ToggleButton;
                    ControlTemplate_74d0bfe542b14de0982f91b468fb4444.SetMethodToInstantiateFrameworkTemplate(Bridge.fn.bind(this, function (templateOwner_ControlTemplate_74d0bfe542b14de0982f91b468fb4444) {
                        var $t;
                        var templateInstance_5f98feaff44a42899c4132fd0c577843 = new Bridge.global.Windows.UI.Xaml.TemplateInstance();
                        templateInstance_5f98feaff44a42899c4132fd0c577843.TemplateOwner = templateOwner_ControlTemplate_74d0bfe542b14de0982f91b468fb4444;
                        var VisualState_e2e8adcb928f46f3adb8230f6983372c = new Bridge.global.Windows.UI.Xaml.VisualState();
                        var VisualState_2cd293526a5644cd895412ea677d0317 = new Bridge.global.Windows.UI.Xaml.VisualState();
                        var VisualStateGroup_4c7615d0a31f4371ac1e9140cab21f46 = new Bridge.global.Windows.UI.Xaml.VisualStateGroup();
                        var VisualState_79b3b8d12a5e4a1aaf39d1e31e550828 = new Bridge.global.Windows.UI.Xaml.VisualState();
                        var VisualState_e6dfdb7200d04b5da73f40e14a039753 = new Bridge.global.Windows.UI.Xaml.VisualState();
                        var VisualStateGroup_20489e587ff64c8cb7c72e2cea36ca78 = new Bridge.global.Windows.UI.Xaml.VisualStateGroup();
                        var Path_3f1f87859443473ab36fb03bf503fa7c = new Bridge.global.Windows.UI.Xaml.Shapes.Path();
                        var ContentPresenter_0da27da157864e67bac9a5de551c1b54 = new Bridge.global.Windows.UI.Xaml.Controls.ContentPresenter();
                        var Border_48c1db6326ee4d579d48466e676c3d3b = new Bridge.global.Windows.UI.Xaml.Controls.Border();
                        templateOwner_ControlTemplate_74d0bfe542b14de0982f91b468fb4444.RegisterName("CommonStates", VisualStateGroup_4c7615d0a31f4371ac1e9140cab21f46);
                        VisualStateGroup_4c7615d0a31f4371ac1e9140cab21f46.Name = "CommonStates";
                        templateOwner_ControlTemplate_74d0bfe542b14de0982f91b468fb4444.RegisterName("Normal", VisualState_e2e8adcb928f46f3adb8230f6983372c);
                        VisualState_e2e8adcb928f46f3adb8230f6983372c.Name = "Normal";

                        templateOwner_ControlTemplate_74d0bfe542b14de0982f91b468fb4444.RegisterName("Checked", VisualState_2cd293526a5644cd895412ea677d0317);
                        VisualState_2cd293526a5644cd895412ea677d0317.Name = "Checked";
                        var Storyboard_60637eba800547828cdb7ba5154f6321 = new Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard();
                        var ObjectAnimationUsingKeyFrames_45fa06dc53b9462ba7e4d6754b892f69 = new Bridge.global.Windows.UI.Xaml.Media.Animation.ObjectAnimationUsingKeyFrames();
                        ObjectAnimationUsingKeyFrames_45fa06dc53b9462ba7e4d6754b892f69.Duration = new Bridge.global.Windows.UI.Xaml.Duration.$ctor1(new Bridge.global.System.TimeSpan(System.Int64(0)));
                        Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard.SetTargetName(ObjectAnimationUsingKeyFrames_45fa06dc53b9462ba7e4d6754b892f69, "arrow");
                        var DiscreteObjectKeyFrame_e57108c124fd4e4887903de90ca102ef = new Bridge.global.Windows.UI.Xaml.Media.Animation.DiscreteObjectKeyFrame();
                        DiscreteObjectKeyFrame_e57108c124fd4e4887903de90ca102ef.KeyTime = Bridge.global.Windows.UI.Xaml.Media.Animation.KeyTime.FromTimeSpan(new Bridge.global.System.TimeSpan(System.Int64(0)));
                        DiscreteObjectKeyFrame_e57108c124fd4e4887903de90ca102ef.Value = "M 1,1.5 L 4.5,5 L 8,1.5";

                        ObjectAnimationUsingKeyFrames_45fa06dc53b9462ba7e4d6754b892f69.KeyFrames.add(DiscreteObjectKeyFrame_e57108c124fd4e4887903de90ca102ef);


                        Storyboard_60637eba800547828cdb7ba5154f6321.Children.add(ObjectAnimationUsingKeyFrames_45fa06dc53b9462ba7e4d6754b892f69);


                        VisualState_2cd293526a5644cd895412ea677d0317.Storyboard = Storyboard_60637eba800547828cdb7ba5154f6321;


                        System.Array.add(VisualStateGroup_4c7615d0a31f4371ac1e9140cab21f46.States, VisualState_e2e8adcb928f46f3adb8230f6983372c, Object);
                        System.Array.add(VisualStateGroup_4c7615d0a31f4371ac1e9140cab21f46.States, VisualState_2cd293526a5644cd895412ea677d0317, Object);


                        templateOwner_ControlTemplate_74d0bfe542b14de0982f91b468fb4444.RegisterName("FocusStates", VisualStateGroup_20489e587ff64c8cb7c72e2cea36ca78);
                        VisualStateGroup_20489e587ff64c8cb7c72e2cea36ca78.Name = "FocusStates";
                        templateOwner_ControlTemplate_74d0bfe542b14de0982f91b468fb4444.RegisterName("Focused", VisualState_79b3b8d12a5e4a1aaf39d1e31e550828);
                        VisualState_79b3b8d12a5e4a1aaf39d1e31e550828.Name = "Focused";

                        templateOwner_ControlTemplate_74d0bfe542b14de0982f91b468fb4444.RegisterName("Unfocused", VisualState_e6dfdb7200d04b5da73f40e14a039753);
                        VisualState_e6dfdb7200d04b5da73f40e14a039753.Name = "Unfocused";

                        System.Array.add(VisualStateGroup_20489e587ff64c8cb7c72e2cea36ca78.States, VisualState_79b3b8d12a5e4a1aaf39d1e31e550828, Object);
                        System.Array.add(VisualStateGroup_20489e587ff64c8cb7c72e2cea36ca78.States, VisualState_e6dfdb7200d04b5da73f40e14a039753, Object);


                        templateOwner_ControlTemplate_74d0bfe542b14de0982f91b468fb4444.INTERNAL_GetVisualStateGroups().add(VisualStateGroup_4c7615d0a31f4371ac1e9140cab21f46);
                        templateOwner_ControlTemplate_74d0bfe542b14de0982f91b468fb4444.INTERNAL_GetVisualStateGroups().add(VisualStateGroup_20489e587ff64c8cb7c72e2cea36ca78);

                        var StackPanel_fc1be953dfb146f9b4f33eab3d15c2b1 = new Bridge.global.Windows.UI.Xaml.Controls.StackPanel();
                        StackPanel_fc1be953dfb146f9b4f33eab3d15c2b1.Orientation = Bridge.global.Windows.UI.Xaml.Controls.Orientation.Horizontal;
                        StackPanel_fc1be953dfb146f9b4f33eab3d15c2b1.Margin = new Bridge.global.Windows.UI.Xaml.Thickness.$ctor2(5, 0, 0, 0);
                        templateOwner_ControlTemplate_74d0bfe542b14de0982f91b468fb4444.RegisterName("arrow", Path_3f1f87859443473ab36fb03bf503fa7c);
                        Path_3f1f87859443473ab36fb03bf503fa7c.Name = "arrow";
                        Path_3f1f87859443473ab36fb03bf503fa7c.Visibility = Bridge.global.Windows.UI.Xaml.Visibility.Visible;
                        Path_3f1f87859443473ab36fb03bf503fa7c.Stroke = new Bridge.global.Windows.UI.Xaml.Media.SolidColorBrush.$ctor1(($t = new Bridge.global.Windows.UI.Color(), $t.A = 255, $t.R = 85, $t.G = 85, $t.B = 85, $t));
                        Path_3f1f87859443473ab36fb03bf503fa7c.Width = 9.0;
                        Path_3f1f87859443473ab36fb03bf503fa7c.Height = 9.0;
                        Path_3f1f87859443473ab36fb03bf503fa7c.Margin = new Bridge.global.Windows.UI.Xaml.Thickness.$ctor2(0, 0, 3, 0);
                        Path_3f1f87859443473ab36fb03bf503fa7c.StrokeThickness = 3.0;
                        Path_3f1f87859443473ab36fb03bf503fa7c.HorizontalAlignment = Bridge.global.Windows.UI.Xaml.HorizontalAlignment.Center;
                        Path_3f1f87859443473ab36fb03bf503fa7c.VerticalAlignment = Bridge.global.Windows.UI.Xaml.VerticalAlignment.Center;
                        Path_3f1f87859443473ab36fb03bf503fa7c.Stretch = Bridge.global.Windows.UI.Xaml.Media.Stretch.Fill;
                        Path_3f1f87859443473ab36fb03bf503fa7c.Data = Bridge.cast(Bridge.global.DotNetForHtml5.Core.TypeFromStringConverters.ConvertFromInvariantString(Bridge.global.Windows.UI.Xaml.Media.Geometry, "M 2,1 L 5.5,4.5 L 2,8"), Windows.UI.Xaml.Media.Geometry);

                        templateOwner_ControlTemplate_74d0bfe542b14de0982f91b468fb4444.RegisterName("header", ContentPresenter_0da27da157864e67bac9a5de551c1b54);
                        ContentPresenter_0da27da157864e67bac9a5de551c1b54.Name = "header";
                        ContentPresenter_0da27da157864e67bac9a5de551c1b54.Margin = new Bridge.global.Windows.UI.Xaml.Thickness.$ctor2(4, 0, 0, 0);
                        ContentPresenter_0da27da157864e67bac9a5de551c1b54.HorizontalAlignment = Bridge.global.Windows.UI.Xaml.HorizontalAlignment.Stretch;
                        ContentPresenter_0da27da157864e67bac9a5de551c1b54.VerticalAlignment = Bridge.global.Windows.UI.Xaml.VerticalAlignment.Center;
                        var Binding_efdcbbcb839849dca072930ab7a5482c = new Bridge.global.Windows.UI.Xaml.Data.Binding.ctor();
                        Binding_efdcbbcb839849dca072930ab7a5482c.Path = new Bridge.global.Windows.UI.Xaml.PropertyPath.ctor("Content");
                        var RelativeSource_e6728e6561ee418eb521c525647aa075 = new Bridge.global.Windows.UI.Xaml.Data.RelativeSource.ctor();
                        RelativeSource_e6728e6561ee418eb521c525647aa075.Mode = Bridge.global.Windows.UI.Xaml.Data.RelativeSourceMode.TemplatedParent;

                        Binding_efdcbbcb839849dca072930ab7a5482c.RelativeSource = RelativeSource_e6728e6561ee418eb521c525647aa075;


                        Binding_efdcbbcb839849dca072930ab7a5482c.TemplateOwner = templateInstance_5f98feaff44a42899c4132fd0c577843;

                        var Binding_9d033170bf054a69b3dc5b3e7a28fb57 = new Bridge.global.Windows.UI.Xaml.Data.Binding.ctor();
                        Binding_9d033170bf054a69b3dc5b3e7a28fb57.Path = new Bridge.global.Windows.UI.Xaml.PropertyPath.ctor("ContentTemplate");
                        var RelativeSource_af5fc3538a15435795c0a0dfe490eb96 = new Bridge.global.Windows.UI.Xaml.Data.RelativeSource.ctor();
                        RelativeSource_af5fc3538a15435795c0a0dfe490eb96.Mode = Bridge.global.Windows.UI.Xaml.Data.RelativeSourceMode.TemplatedParent;

                        Binding_9d033170bf054a69b3dc5b3e7a28fb57.RelativeSource = RelativeSource_af5fc3538a15435795c0a0dfe490eb96;


                        Binding_9d033170bf054a69b3dc5b3e7a28fb57.TemplateOwner = templateInstance_5f98feaff44a42899c4132fd0c577843;


                        StackPanel_fc1be953dfb146f9b4f33eab3d15c2b1.Children.add(Path_3f1f87859443473ab36fb03bf503fa7c);
                        StackPanel_fc1be953dfb146f9b4f33eab3d15c2b1.Children.add(ContentPresenter_0da27da157864e67bac9a5de551c1b54);


                        Border_48c1db6326ee4d579d48466e676c3d3b.Child = StackPanel_fc1be953dfb146f9b4f33eab3d15c2b1;

                        var Binding_b21117c002ee48e087af78644104d6f2 = new Bridge.global.Windows.UI.Xaml.Data.Binding.ctor();
                        Binding_b21117c002ee48e087af78644104d6f2.Path = new Bridge.global.Windows.UI.Xaml.PropertyPath.ctor("Padding");
                        var RelativeSource_f5f415a1902b42f2a436c44621d6de61 = new Bridge.global.Windows.UI.Xaml.Data.RelativeSource.ctor();
                        RelativeSource_f5f415a1902b42f2a436c44621d6de61.Mode = Bridge.global.Windows.UI.Xaml.Data.RelativeSourceMode.TemplatedParent;

                        Binding_b21117c002ee48e087af78644104d6f2.RelativeSource = RelativeSource_f5f415a1902b42f2a436c44621d6de61;


                        Binding_b21117c002ee48e087af78644104d6f2.TemplateOwner = templateInstance_5f98feaff44a42899c4132fd0c577843;


                        Bridge.global.Windows.UI.Xaml.Data.BindingOperations.SetBinding(ContentPresenter_0da27da157864e67bac9a5de551c1b54, Bridge.global.Windows.UI.Xaml.Controls.ContentControl.ContentProperty, Binding_efdcbbcb839849dca072930ab7a5482c);
                        Bridge.global.Windows.UI.Xaml.Data.BindingOperations.SetBinding(ContentPresenter_0da27da157864e67bac9a5de551c1b54, Bridge.global.Windows.UI.Xaml.Controls.ContentControl.ContentTemplateProperty, Binding_9d033170bf054a69b3dc5b3e7a28fb57);
                        Bridge.global.Windows.UI.Xaml.Data.BindingOperations.SetBinding(Border_48c1db6326ee4d579d48466e676c3d3b, Bridge.global.Windows.UI.Xaml.Controls.Border.PaddingProperty, Binding_b21117c002ee48e087af78644104d6f2);

                        Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard.SetTargetProperty(ObjectAnimationUsingKeyFrames_45fa06dc53b9462ba7e4d6754b892f69, new Bridge.global.Windows.UI.Xaml.PropertyPath.$ctor1("Data", "Data", Bridge.fn.cacheBind(this, this.accessVisualStateProperty_8a7e70f8598d4e83bdfb4904a19f31aa), Bridge.fn.cacheBind(this, this.setVisualStateProperty_8a7e70f8598d4e83bdfb4904a19f31aa), Bridge.fn.cacheBind(this, this.setAnimationVisualStateProperty_8a7e70f8598d4e83bdfb4904a19f31aa), Bridge.fn.cacheBind(this, this.setLocalVisualStateProperty_8a7e70f8598d4e83bdfb4904a19f31aa), Bridge.fn.cacheBind(this, this.getVisualStateProperty_8a7e70f8598d4e83bdfb4904a19f31aa)));
                        Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard.SetTarget(ObjectAnimationUsingKeyFrames_45fa06dc53b9462ba7e4d6754b892f69, Path_3f1f87859443473ab36fb03bf503fa7c);

                        templateInstance_5f98feaff44a42899c4132fd0c577843.TemplateContent = Border_48c1db6326ee4d579d48466e676c3d3b;
                        return templateInstance_5f98feaff44a42899c4132fd0c577843;
                    }));

                    ToggleButton_cd82c53a2f304e96acfc810363b3c0d2.Template = ControlTemplate_74d0bfe542b14de0982f91b468fb4444;

                    var Binding_e09991ba43794fa08da2fd486112538e = new Bridge.global.Windows.UI.Xaml.Data.Binding.ctor();
                    Binding_e09991ba43794fa08da2fd486112538e.Path = new Bridge.global.Windows.UI.Xaml.PropertyPath.ctor("Padding");
                    var RelativeSource_83712df7116a4459bcdcc51baee683c9 = new Bridge.global.Windows.UI.Xaml.Data.RelativeSource.ctor();
                    RelativeSource_83712df7116a4459bcdcc51baee683c9.Mode = Bridge.global.Windows.UI.Xaml.Data.RelativeSourceMode.TemplatedParent;

                    Binding_e09991ba43794fa08da2fd486112538e.RelativeSource = RelativeSource_83712df7116a4459bcdcc51baee683c9;


                    Binding_e09991ba43794fa08da2fd486112538e.TemplateOwner = templateInstance_4ce613c5064441719ef7751f39e59669;

                    var Binding_bd9378f1fb4f46819b0da55c1196a4e6 = new Bridge.global.Windows.UI.Xaml.Data.Binding.ctor();
                    Binding_bd9378f1fb4f46819b0da55c1196a4e6.Path = new Bridge.global.Windows.UI.Xaml.PropertyPath.ctor("Header");
                    var RelativeSource_b2afa80118524eb38a6e2c3022d28355 = new Bridge.global.Windows.UI.Xaml.Data.RelativeSource.ctor();
                    RelativeSource_b2afa80118524eb38a6e2c3022d28355.Mode = Bridge.global.Windows.UI.Xaml.Data.RelativeSourceMode.TemplatedParent;

                    Binding_bd9378f1fb4f46819b0da55c1196a4e6.RelativeSource = RelativeSource_b2afa80118524eb38a6e2c3022d28355;


                    Binding_bd9378f1fb4f46819b0da55c1196a4e6.TemplateOwner = templateInstance_4ce613c5064441719ef7751f39e59669;

                    var Binding_4f9863363fbd43f5a415d42961efd03a = new Bridge.global.Windows.UI.Xaml.Data.Binding.ctor();
                    Binding_4f9863363fbd43f5a415d42961efd03a.Path = new Bridge.global.Windows.UI.Xaml.PropertyPath.ctor("HeaderTemplate");
                    var RelativeSource_19f405f4d81444fdb89e32c6ca7c2905 = new Bridge.global.Windows.UI.Xaml.Data.RelativeSource.ctor();
                    RelativeSource_19f405f4d81444fdb89e32c6ca7c2905.Mode = Bridge.global.Windows.UI.Xaml.Data.RelativeSourceMode.TemplatedParent;

                    Binding_4f9863363fbd43f5a415d42961efd03a.RelativeSource = RelativeSource_19f405f4d81444fdb89e32c6ca7c2905;


                    Binding_4f9863363fbd43f5a415d42961efd03a.TemplateOwner = templateInstance_4ce613c5064441719ef7751f39e59669;


                    templateOwner_ControlTemplate_8e5a2e0f304f44c78d3e4d0e9f702e67.RegisterName("ExpandSite", ContentControl_25ff6e1b99b942038f744702dffc35d8);
                    ContentControl_25ff6e1b99b942038f744702dffc35d8.Name = "ExpandSite";
                    ContentControl_25ff6e1b99b942038f744702dffc35d8.Visibility = Bridge.global.Windows.UI.Xaml.Visibility.Collapsed;
                    var Binding_6b2142ff10614437b4aea5c486b51040 = new Bridge.global.Windows.UI.Xaml.Data.Binding.ctor();
                    Binding_6b2142ff10614437b4aea5c486b51040.Path = new Bridge.global.Windows.UI.Xaml.PropertyPath.ctor("Padding");
                    var RelativeSource_d7ec7dc0e54e4ecebca9db7d2ae8469e = new Bridge.global.Windows.UI.Xaml.Data.RelativeSource.ctor();
                    RelativeSource_d7ec7dc0e54e4ecebca9db7d2ae8469e.Mode = Bridge.global.Windows.UI.Xaml.Data.RelativeSourceMode.TemplatedParent;

                    Binding_6b2142ff10614437b4aea5c486b51040.RelativeSource = RelativeSource_d7ec7dc0e54e4ecebca9db7d2ae8469e;


                    Binding_6b2142ff10614437b4aea5c486b51040.TemplateOwner = templateInstance_4ce613c5064441719ef7751f39e59669;

                    var Binding_a47341d50ef24feab1c34404a5463257 = new Bridge.global.Windows.UI.Xaml.Data.Binding.ctor();
                    Binding_a47341d50ef24feab1c34404a5463257.Path = new Bridge.global.Windows.UI.Xaml.PropertyPath.ctor("Content");
                    var RelativeSource_8ec46d75846d49f69ccd1f8b969c51c1 = new Bridge.global.Windows.UI.Xaml.Data.RelativeSource.ctor();
                    RelativeSource_8ec46d75846d49f69ccd1f8b969c51c1.Mode = Bridge.global.Windows.UI.Xaml.Data.RelativeSourceMode.TemplatedParent;

                    Binding_a47341d50ef24feab1c34404a5463257.RelativeSource = RelativeSource_8ec46d75846d49f69ccd1f8b969c51c1;


                    Binding_a47341d50ef24feab1c34404a5463257.TemplateOwner = templateInstance_4ce613c5064441719ef7751f39e59669;

                    var Binding_f2037e332d2c4ef886c3647fc4dcd6e2 = new Bridge.global.Windows.UI.Xaml.Data.Binding.ctor();
                    Binding_f2037e332d2c4ef886c3647fc4dcd6e2.Path = new Bridge.global.Windows.UI.Xaml.PropertyPath.ctor("ContentTemplate");
                    var RelativeSource_fe3cd6f6b37c48e1a0d7906c4879a32a = new Bridge.global.Windows.UI.Xaml.Data.RelativeSource.ctor();
                    RelativeSource_fe3cd6f6b37c48e1a0d7906c4879a32a.Mode = Bridge.global.Windows.UI.Xaml.Data.RelativeSourceMode.TemplatedParent;

                    Binding_f2037e332d2c4ef886c3647fc4dcd6e2.RelativeSource = RelativeSource_fe3cd6f6b37c48e1a0d7906c4879a32a;


                    Binding_f2037e332d2c4ef886c3647fc4dcd6e2.TemplateOwner = templateInstance_4ce613c5064441719ef7751f39e59669;


                    StackPanel_d1d4cb82cbf941d2b0fdcb7d5d06d73d.Children.add(ToggleButton_cd82c53a2f304e96acfc810363b3c0d2);
                    StackPanel_d1d4cb82cbf941d2b0fdcb7d5d06d73d.Children.add(ContentControl_25ff6e1b99b942038f744702dffc35d8);


                    Border_6b8182905bfc445f85f99811fa802081.Child = StackPanel_d1d4cb82cbf941d2b0fdcb7d5d06d73d;

                    var Binding_62031db73a404fcaa8c1b56764444c4e = new Bridge.global.Windows.UI.Xaml.Data.Binding.ctor();
                    Binding_62031db73a404fcaa8c1b56764444c4e.Path = new Bridge.global.Windows.UI.Xaml.PropertyPath.ctor("Background");
                    var RelativeSource_44a40516660843b090ecb69ce10b50aa = new Bridge.global.Windows.UI.Xaml.Data.RelativeSource.ctor();
                    RelativeSource_44a40516660843b090ecb69ce10b50aa.Mode = Bridge.global.Windows.UI.Xaml.Data.RelativeSourceMode.TemplatedParent;

                    Binding_62031db73a404fcaa8c1b56764444c4e.RelativeSource = RelativeSource_44a40516660843b090ecb69ce10b50aa;


                    Binding_62031db73a404fcaa8c1b56764444c4e.TemplateOwner = templateInstance_4ce613c5064441719ef7751f39e59669;

                    var Binding_f293d7e0640447f7a23bdee812a00c21 = new Bridge.global.Windows.UI.Xaml.Data.Binding.ctor();
                    Binding_f293d7e0640447f7a23bdee812a00c21.Path = new Bridge.global.Windows.UI.Xaml.PropertyPath.ctor("BorderBrush");
                    var RelativeSource_242121b73a134b05ab1e76c38b97ae28 = new Bridge.global.Windows.UI.Xaml.Data.RelativeSource.ctor();
                    RelativeSource_242121b73a134b05ab1e76c38b97ae28.Mode = Bridge.global.Windows.UI.Xaml.Data.RelativeSourceMode.TemplatedParent;

                    Binding_f293d7e0640447f7a23bdee812a00c21.RelativeSource = RelativeSource_242121b73a134b05ab1e76c38b97ae28;


                    Binding_f293d7e0640447f7a23bdee812a00c21.TemplateOwner = templateInstance_4ce613c5064441719ef7751f39e59669;

                    var Binding_92808c1a384240648799643e38859ca4 = new Bridge.global.Windows.UI.Xaml.Data.Binding.ctor();
                    Binding_92808c1a384240648799643e38859ca4.Path = new Bridge.global.Windows.UI.Xaml.PropertyPath.ctor("BorderThickness");
                    var RelativeSource_60786d6d2812438cbceb035341b23ef7 = new Bridge.global.Windows.UI.Xaml.Data.RelativeSource.ctor();
                    RelativeSource_60786d6d2812438cbceb035341b23ef7.Mode = Bridge.global.Windows.UI.Xaml.Data.RelativeSourceMode.TemplatedParent;

                    Binding_92808c1a384240648799643e38859ca4.RelativeSource = RelativeSource_60786d6d2812438cbceb035341b23ef7;


                    Binding_92808c1a384240648799643e38859ca4.TemplateOwner = templateInstance_4ce613c5064441719ef7751f39e59669;


                    Bridge.global.Windows.UI.Xaml.Data.BindingOperations.SetBinding(ToggleButton_cd82c53a2f304e96acfc810363b3c0d2, Bridge.global.Windows.UI.Xaml.Controls.Control.PaddingProperty, Binding_e09991ba43794fa08da2fd486112538e);
                    Bridge.global.Windows.UI.Xaml.Data.BindingOperations.SetBinding(ToggleButton_cd82c53a2f304e96acfc810363b3c0d2, Bridge.global.Windows.UI.Xaml.Controls.ContentControl.ContentProperty, Binding_bd9378f1fb4f46819b0da55c1196a4e6);
                    Bridge.global.Windows.UI.Xaml.Data.BindingOperations.SetBinding(ToggleButton_cd82c53a2f304e96acfc810363b3c0d2, Bridge.global.Windows.UI.Xaml.Controls.ContentControl.ContentTemplateProperty, Binding_4f9863363fbd43f5a415d42961efd03a);
                    Bridge.global.Windows.UI.Xaml.Data.BindingOperations.SetBinding(ContentControl_25ff6e1b99b942038f744702dffc35d8, Bridge.global.Windows.UI.Xaml.FrameworkElement.MarginProperty, Binding_6b2142ff10614437b4aea5c486b51040);
                    Bridge.global.Windows.UI.Xaml.Data.BindingOperations.SetBinding(ContentControl_25ff6e1b99b942038f744702dffc35d8, Bridge.global.Windows.UI.Xaml.Controls.ContentControl.ContentProperty, Binding_a47341d50ef24feab1c34404a5463257);
                    Bridge.global.Windows.UI.Xaml.Data.BindingOperations.SetBinding(ContentControl_25ff6e1b99b942038f744702dffc35d8, Bridge.global.Windows.UI.Xaml.Controls.ContentControl.ContentTemplateProperty, Binding_f2037e332d2c4ef886c3647fc4dcd6e2);
                    Bridge.global.Windows.UI.Xaml.Data.BindingOperations.SetBinding(Border_6b8182905bfc445f85f99811fa802081, Bridge.global.Windows.UI.Xaml.Controls.Border.BackgroundProperty, Binding_62031db73a404fcaa8c1b56764444c4e);
                    Bridge.global.Windows.UI.Xaml.Data.BindingOperations.SetBinding(Border_6b8182905bfc445f85f99811fa802081, Bridge.global.Windows.UI.Xaml.Controls.Border.BorderBrushProperty, Binding_f293d7e0640447f7a23bdee812a00c21);
                    Bridge.global.Windows.UI.Xaml.Data.BindingOperations.SetBinding(Border_6b8182905bfc445f85f99811fa802081, Bridge.global.Windows.UI.Xaml.Controls.Border.BorderThicknessProperty, Binding_92808c1a384240648799643e38859ca4);

                    Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard.SetTargetProperty(ObjectAnimationUsingKeyFrames_543375fc60be4607b451c8895e1122d8, new Bridge.global.Windows.UI.Xaml.PropertyPath.$ctor1("Visibility", "Visibility", Bridge.fn.cacheBind(this, this.accessVisualStateProperty_529feb51c23a457fa9e9535afd7cc5c4), Bridge.fn.cacheBind(this, this.setVisualStateProperty_529feb51c23a457fa9e9535afd7cc5c4), Bridge.fn.cacheBind(this, this.setAnimationVisualStateProperty_529feb51c23a457fa9e9535afd7cc5c4), Bridge.fn.cacheBind(this, this.setLocalVisualStateProperty_529feb51c23a457fa9e9535afd7cc5c4), Bridge.fn.cacheBind(this, this.getVisualStateProperty_529feb51c23a457fa9e9535afd7cc5c4)));
                    Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard.SetTarget(ObjectAnimationUsingKeyFrames_543375fc60be4607b451c8895e1122d8, ContentControl_25ff6e1b99b942038f744702dffc35d8);

                    templateInstance_4ce613c5064441719ef7751f39e59669.TemplateContent = Border_6b8182905bfc445f85f99811fa802081;
                    return templateInstance_4ce613c5064441719ef7751f39e59669;
                }));

                Setter_3bfd0120357f4ab9bd97c4d130a356f1.Value = ControlTemplate_8e5a2e0f304f44c78d3e4d0e9f702e67;


                Style_273df2acc5ad457bb6acff0ffc3451af.Setters.add(Setter_3bfd0120357f4ab9bd97c4d130a356f1);


                this.setItem(Bridge.global.Windows.UI.Xaml.Controls.Expander, Style_273df2acc5ad457bb6acff0ffc3451af);






            },
            accessVisualStateProperty_529feb51c23a457fa9e9535afd7cc5c4: function (rootTargetObjectInstance) {
                return new (Bridge.GeneratorEnumerable$1(System.Tuple$3(Windows.UI.Xaml.DependencyObject,Windows.UI.Xaml.DependencyProperty,System.Nullable$1(System.Int32))))(Bridge.fn.bind(this, function (rootTargetObjectInstance) {
                    var $step = 0,
                        $jumpFromFinally,
                        $returnValue,
                        $async_e;

                    var $enumerator = new (Bridge.GeneratorEnumerator$1(System.Tuple$3(Windows.UI.Xaml.DependencyObject,Windows.UI.Xaml.DependencyProperty,System.Nullable$1(System.Int32))))(Bridge.fn.bind(this, function () {
                        try {
                            for (;;) {
                                switch ($step) {
                                    case 0: {
                                        return false;
                                    }
                                    default: {
                                        return false;
                                    }
                                }
                            }
                        } catch($async_e1) {
                            $async_e = System.Exception.create($async_e1);
                            throw $async_e;
                        }
                    }));
                    return $enumerator;
                }, arguments));
            },
            setVisualStateProperty_529feb51c23a457fa9e9535afd7cc5c4: function (finalTargetInstance, value) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Visibility").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "VisibilityProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                (finalTargetInstance).SetVisualStateValue(property, value);
            },
            setAnimationVisualStateProperty_529feb51c23a457fa9e9535afd7cc5c4: function (finalTargetInstance, value) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Visibility").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "VisibilityProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                (finalTargetInstance).SetAnimationValue(property, value);
            },
            setLocalVisualStateProperty_529feb51c23a457fa9e9535afd7cc5c4: function (finalTargetInstance, value) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Visibility").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "VisibilityProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                (finalTargetInstance).SetLocalValue(property, value);
            },
            getVisualStateProperty_529feb51c23a457fa9e9535afd7cc5c4: function (finalTargetInstance) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Visibility").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "VisibilityProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                return finalTargetInstance.GetVisualStateValue(property);
            },
            accessVisualStateProperty_8a7e70f8598d4e83bdfb4904a19f31aa: function (rootTargetObjectInstance) {
                return new (Bridge.GeneratorEnumerable$1(System.Tuple$3(Windows.UI.Xaml.DependencyObject,Windows.UI.Xaml.DependencyProperty,System.Nullable$1(System.Int32))))(Bridge.fn.bind(this, function (rootTargetObjectInstance) {
                    var $step = 0,
                        $jumpFromFinally,
                        $returnValue,
                        $async_e;

                    var $enumerator = new (Bridge.GeneratorEnumerator$1(System.Tuple$3(Windows.UI.Xaml.DependencyObject,Windows.UI.Xaml.DependencyProperty,System.Nullable$1(System.Int32))))(Bridge.fn.bind(this, function () {
                        try {
                            for (;;) {
                                switch ($step) {
                                    case 0: {
                                        return false;
                                    }
                                    default: {
                                        return false;
                                    }
                                }
                            }
                        } catch($async_e1) {
                            $async_e = System.Exception.create($async_e1);
                            throw $async_e;
                        }
                    }));
                    return $enumerator;
                }, arguments));
            },
            setVisualStateProperty_8a7e70f8598d4e83bdfb4904a19f31aa: function (finalTargetInstance, value) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Data").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "DataProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                (finalTargetInstance).SetVisualStateValue(property, value);
            },
            setAnimationVisualStateProperty_8a7e70f8598d4e83bdfb4904a19f31aa: function (finalTargetInstance, value) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Data").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "DataProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                (finalTargetInstance).SetAnimationValue(property, value);
            },
            setLocalVisualStateProperty_8a7e70f8598d4e83bdfb4904a19f31aa: function (finalTargetInstance, value) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Data").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "DataProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                (finalTargetInstance).SetLocalValue(property, value);
            },
            getVisualStateProperty_8a7e70f8598d4e83bdfb4904a19f31aa: function (finalTargetInstance) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Data").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "DataProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                return finalTargetInstance.GetVisualStateValue(property);
            }
        }
    });

    Bridge.define("___CSHTML5__Component__Defaultstyles__INTERNAL_Defaultgridsplitterstyle__Xaml", {
        inherits: [Bridge.global.Windows.UI.Xaml.ResourceDictionary],
        fields: {
            _contentLoaded: false
        },
        ctors: {
            ctor: function () {
                this.$initialize();
                Bridge.global.Windows.UI.Xaml.ResourceDictionary.ctor.call(this);
                this.InitializeComponent();

                if (Bridge.is(this, Windows.UI.Xaml.UIElement)) {
                    Bridge.cast(this, Windows.UI.Xaml.UIElement).XamlSourcePath = "CSHTML5\\DefaultStyles\\INTERNAL_DefaultGridSplitterStyle.xaml";
                }
            }
        },
        methods: {
            InitializeComponent: function () {
                var $t;
                if (this._contentLoaded) {
                    return;
                }
                this._contentLoaded = true;


                if (Bridge.is(this, Windows.UI.Xaml.UIElement)) {
                    Bridge.cast(this, Windows.UI.Xaml.UIElement).XamlSourcePath = "CSHTML5\\DefaultStyles\\INTERNAL_DefaultGridSplitterStyle.xaml";
                }



                var Style_d148988ce9be42bc96299bf5d9405215 = new Bridge.global.Windows.UI.Xaml.Style.ctor();
                Style_d148988ce9be42bc96299bf5d9405215.TargetType = Bridge.global.Windows.UI.Xaml.Controls.GridSplitter;
                var Setter_585cf2a461f143c8bc65173148f20327 = new Bridge.global.Windows.UI.Xaml.Setter.ctor();
                Setter_585cf2a461f143c8bc65173148f20327.Property = Bridge.global.Windows.UI.Xaml.Controls.Control.IsTabStopProperty;
                Setter_585cf2a461f143c8bc65173148f20327.Value = Bridge.box(false, System.Boolean, System.Boolean.toString);

                var Setter_8ba669fb794042febb7943e7ea8c2e8a = new Bridge.global.Windows.UI.Xaml.Setter.ctor();
                Setter_8ba669fb794042febb7943e7ea8c2e8a.Property = Bridge.global.Windows.UI.Xaml.Controls.Control.BackgroundProperty;
                Setter_8ba669fb794042febb7943e7ea8c2e8a.Value = new Bridge.global.Windows.UI.Xaml.Media.SolidColorBrush.$ctor1(($t = new Bridge.global.Windows.UI.Color(), $t.A = 255, $t.R = 240, $t.G = 240, $t.B = 240, $t));

                var Setter_4eca8cd4f02849008721e99c921cebab = new Bridge.global.Windows.UI.Xaml.Setter.ctor();
                Setter_4eca8cd4f02849008721e99c921cebab.Property = Bridge.global.Windows.UI.Xaml.Controls.Control.TemplateProperty;
                var ControlTemplate_2c4b6a65810a494e9ce80bd10b07d0ff = new Bridge.global.Windows.UI.Xaml.Controls.ControlTemplate();
                ControlTemplate_2c4b6a65810a494e9ce80bd10b07d0ff.TargetType = Bridge.global.Windows.UI.Xaml.Controls.GridSplitter;
                ControlTemplate_2c4b6a65810a494e9ce80bd10b07d0ff.SetMethodToInstantiateFrameworkTemplate(function (templateOwner_ControlTemplate_2c4b6a65810a494e9ce80bd10b07d0ff) {
                    var templateInstance_348c36a3aed04ae18de1341826907319 = new Bridge.global.Windows.UI.Xaml.TemplateInstance();
                    templateInstance_348c36a3aed04ae18de1341826907319.TemplateOwner = templateOwner_ControlTemplate_2c4b6a65810a494e9ce80bd10b07d0ff;
                    var Thumb_fc4e35efdb0b4dd5ab91f30d42a03e1f = new Bridge.global.Windows.UI.Xaml.Controls.Primitives.Thumb();
                    var Grid_07199e7e3abe4ef686392f37923c7727 = new Bridge.global.Windows.UI.Xaml.Controls.Grid();
                    templateOwner_ControlTemplate_2c4b6a65810a494e9ce80bd10b07d0ff.RegisterName("PART_Thumb", Thumb_fc4e35efdb0b4dd5ab91f30d42a03e1f);
                    Thumb_fc4e35efdb0b4dd5ab91f30d42a03e1f.Name = "PART_Thumb";
                    Thumb_fc4e35efdb0b4dd5ab91f30d42a03e1f.Opacity = 0.0;

                    var ContentPresenter_1b8a621312ff48b69cff8dd415cb6c61 = new Bridge.global.Windows.UI.Xaml.Controls.ContentPresenter();
                    ContentPresenter_1b8a621312ff48b69cff8dd415cb6c61.IsHitTestVisible = false;
                    var Binding_2f7a8026325b45bcabbafcab0a9a4556 = new Bridge.global.Windows.UI.Xaml.Data.Binding.ctor();
                    Binding_2f7a8026325b45bcabbafcab0a9a4556.Path = new Bridge.global.Windows.UI.Xaml.PropertyPath.ctor("Element");
                    var RelativeSource_5ec0957784f54f3b8711d5331426705f = new Bridge.global.Windows.UI.Xaml.Data.RelativeSource.ctor();
                    RelativeSource_5ec0957784f54f3b8711d5331426705f.Mode = Bridge.global.Windows.UI.Xaml.Data.RelativeSourceMode.TemplatedParent;

                    Binding_2f7a8026325b45bcabbafcab0a9a4556.RelativeSource = RelativeSource_5ec0957784f54f3b8711d5331426705f;


                    Binding_2f7a8026325b45bcabbafcab0a9a4556.TemplateOwner = templateInstance_348c36a3aed04ae18de1341826907319;

                    var Binding_75a031f869474eed864ca8b1525f766c = new Bridge.global.Windows.UI.Xaml.Data.Binding.ctor();
                    Binding_75a031f869474eed864ca8b1525f766c.Path = new Bridge.global.Windows.UI.Xaml.PropertyPath.ctor("ContentTemplate");
                    var RelativeSource_c2d08beb8e144e15bdc4a89bb75b97c8 = new Bridge.global.Windows.UI.Xaml.Data.RelativeSource.ctor();
                    RelativeSource_c2d08beb8e144e15bdc4a89bb75b97c8.Mode = Bridge.global.Windows.UI.Xaml.Data.RelativeSourceMode.TemplatedParent;

                    Binding_75a031f869474eed864ca8b1525f766c.RelativeSource = RelativeSource_c2d08beb8e144e15bdc4a89bb75b97c8;


                    Binding_75a031f869474eed864ca8b1525f766c.TemplateOwner = templateInstance_348c36a3aed04ae18de1341826907319;


                    Grid_07199e7e3abe4ef686392f37923c7727.Children.add(Thumb_fc4e35efdb0b4dd5ab91f30d42a03e1f);
                    Grid_07199e7e3abe4ef686392f37923c7727.Children.add(ContentPresenter_1b8a621312ff48b69cff8dd415cb6c61);

                    var Binding_9e352ffbbb5047388c87df4f64c143b0 = new Bridge.global.Windows.UI.Xaml.Data.Binding.ctor();
                    Binding_9e352ffbbb5047388c87df4f64c143b0.Path = new Bridge.global.Windows.UI.Xaml.PropertyPath.ctor("Background");
                    var RelativeSource_4beba1b6b8e84c24a53dda820653c7ec = new Bridge.global.Windows.UI.Xaml.Data.RelativeSource.ctor();
                    RelativeSource_4beba1b6b8e84c24a53dda820653c7ec.Mode = Bridge.global.Windows.UI.Xaml.Data.RelativeSourceMode.TemplatedParent;

                    Binding_9e352ffbbb5047388c87df4f64c143b0.RelativeSource = RelativeSource_4beba1b6b8e84c24a53dda820653c7ec;


                    Binding_9e352ffbbb5047388c87df4f64c143b0.TemplateOwner = templateInstance_348c36a3aed04ae18de1341826907319;


                    Bridge.global.Windows.UI.Xaml.Data.BindingOperations.SetBinding(ContentPresenter_1b8a621312ff48b69cff8dd415cb6c61, Bridge.global.Windows.UI.Xaml.Controls.ContentControl.ContentProperty, Binding_2f7a8026325b45bcabbafcab0a9a4556);
                    Bridge.global.Windows.UI.Xaml.Data.BindingOperations.SetBinding(ContentPresenter_1b8a621312ff48b69cff8dd415cb6c61, Bridge.global.Windows.UI.Xaml.Controls.ContentControl.ContentTemplateProperty, Binding_75a031f869474eed864ca8b1525f766c);
                    Bridge.global.Windows.UI.Xaml.Data.BindingOperations.SetBinding(Grid_07199e7e3abe4ef686392f37923c7727, Bridge.global.Windows.UI.Xaml.Controls.Panel.BackgroundProperty, Binding_9e352ffbbb5047388c87df4f64c143b0);

                    templateInstance_348c36a3aed04ae18de1341826907319.TemplateContent = Grid_07199e7e3abe4ef686392f37923c7727;
                    return templateInstance_348c36a3aed04ae18de1341826907319;
                });

                Setter_4eca8cd4f02849008721e99c921cebab.Value = ControlTemplate_2c4b6a65810a494e9ce80bd10b07d0ff;


                Style_d148988ce9be42bc96299bf5d9405215.Setters.add(Setter_585cf2a461f143c8bc65173148f20327);
                Style_d148988ce9be42bc96299bf5d9405215.Setters.add(Setter_8ba669fb794042febb7943e7ea8c2e8a);
                Style_d148988ce9be42bc96299bf5d9405215.Setters.add(Setter_4eca8cd4f02849008721e99c921cebab);


                this.setItem(Bridge.global.Windows.UI.Xaml.Controls.GridSplitter, Style_d148988ce9be42bc96299bf5d9405215);






            }
        }
    });

    Bridge.define("___CSHTML5__Component__Defaultstyles__INTERNAL_Defaulthyperlinkbuttonstyle__Xaml", {
        inherits: [Bridge.global.Windows.UI.Xaml.ResourceDictionary],
        fields: {
            _contentLoaded: false
        },
        ctors: {
            ctor: function () {
                this.$initialize();
                Bridge.global.Windows.UI.Xaml.ResourceDictionary.ctor.call(this);
                this.InitializeComponent();

                if (Bridge.is(this, Windows.UI.Xaml.UIElement)) {
                    Bridge.cast(this, Windows.UI.Xaml.UIElement).XamlSourcePath = "CSHTML5\\DefaultStyles\\INTERNAL_DefaultHyperlinkButtonStyle.xaml";
                }
            }
        },
        methods: {
            InitializeComponent: function () {
                var $t;
                if (this._contentLoaded) {
                    return;
                }
                this._contentLoaded = true;


                if (Bridge.is(this, Windows.UI.Xaml.UIElement)) {
                    Bridge.cast(this, Windows.UI.Xaml.UIElement).XamlSourcePath = "CSHTML5\\DefaultStyles\\INTERNAL_DefaultHyperlinkButtonStyle.xaml";
                }



                var Style_e5fa8d55aa354923a7af3273e8391b13 = new Bridge.global.Windows.UI.Xaml.Style.ctor();
                Style_e5fa8d55aa354923a7af3273e8391b13.TargetType = Bridge.global.Windows.UI.Xaml.Controls.HyperlinkButton;
                var Setter_790aaa22fb9449ca9149824b49d306ba = new Bridge.global.Windows.UI.Xaml.Setter.ctor();
                Setter_790aaa22fb9449ca9149824b49d306ba.Property = Bridge.global.Windows.UI.Xaml.Controls.Control.ForegroundProperty;
                Setter_790aaa22fb9449ca9149824b49d306ba.Value = new Bridge.global.Windows.UI.Xaml.Media.SolidColorBrush.$ctor1(($t = new Bridge.global.Windows.UI.Color(), $t.A = 255, $t.R = 0, $t.G = 0, $t.B = 238, $t));

                var Setter_bc41f9e1bf564fd3849bea4b31c03fea = new Bridge.global.Windows.UI.Xaml.Setter.ctor();
                Setter_bc41f9e1bf564fd3849bea4b31c03fea.Property = Bridge.global.Windows.UI.Xaml.FrameworkElement.CursorProperty;
                Setter_bc41f9e1bf564fd3849bea4b31c03fea.Value = Bridge.global.System.Windows.Input.Cursors.Hand;

                var Setter_10a0be012a1b4828bf6fa471f2baa4c6 = new Bridge.global.Windows.UI.Xaml.Setter.ctor();
                Setter_10a0be012a1b4828bf6fa471f2baa4c6.Property = Bridge.global.Windows.UI.Xaml.Controls.Control.HorizontalContentAlignmentProperty;
                Setter_10a0be012a1b4828bf6fa471f2baa4c6.Value = Bridge.box(Bridge.global.Windows.UI.Xaml.HorizontalAlignment.Center, Windows.UI.Xaml.HorizontalAlignment, System.Enum.toStringFn(Windows.UI.Xaml.HorizontalAlignment));

                var Setter_53361baa6dbb4f09b6e0d44dd750f1a3 = new Bridge.global.Windows.UI.Xaml.Setter.ctor();
                Setter_53361baa6dbb4f09b6e0d44dd750f1a3.Property = Bridge.global.Windows.UI.Xaml.Controls.Control.VerticalContentAlignmentProperty;
                Setter_53361baa6dbb4f09b6e0d44dd750f1a3.Value = Bridge.box(Bridge.global.Windows.UI.Xaml.VerticalAlignment.Center, Windows.UI.Xaml.VerticalAlignment, System.Enum.toStringFn(Windows.UI.Xaml.VerticalAlignment));

                var Setter_344a2e7b5a1b4d87b6a623fa7e6ace62 = new Bridge.global.Windows.UI.Xaml.Setter.ctor();
                Setter_344a2e7b5a1b4d87b6a623fa7e6ace62.Property = Bridge.global.Windows.UI.Xaml.Controls.Control.TemplateProperty;
                var ControlTemplate_ed55b8fc6aa0484a82b1c84a60debd29 = new Bridge.global.Windows.UI.Xaml.Controls.ControlTemplate();
                ControlTemplate_ed55b8fc6aa0484a82b1c84a60debd29.TargetType = Bridge.global.Windows.UI.Xaml.Controls.HyperlinkButton;
                ControlTemplate_ed55b8fc6aa0484a82b1c84a60debd29.SetMethodToInstantiateFrameworkTemplate(Bridge.fn.bind(this, function (templateOwner_ControlTemplate_ed55b8fc6aa0484a82b1c84a60debd29) {
                    var templateInstance_541931248954448784187f890fb98436 = new Bridge.global.Windows.UI.Xaml.TemplateInstance();
                    templateInstance_541931248954448784187f890fb98436.TemplateOwner = templateOwner_ControlTemplate_ed55b8fc6aa0484a82b1c84a60debd29;
                    var VisualState_1fb96ef410f04df5878340ed580e270e = new Bridge.global.Windows.UI.Xaml.VisualState();
                    var VisualState_9d1bdd31c5d4459b880f3a8752b36668 = new Bridge.global.Windows.UI.Xaml.VisualState();
                    var VisualStateGroup_259836095a9e47988d3896eeaefe6cb8 = new Bridge.global.Windows.UI.Xaml.VisualStateGroup();
                    var ContentPresenter_6c7ce388471d4e1da0eb70631fadcee9 = new Bridge.global.Windows.UI.Xaml.Controls.ContentPresenter();
                    var Border_fbc7d09498e54a019f0021720e818cea = new Bridge.global.Windows.UI.Xaml.Controls.Border();
                    templateOwner_ControlTemplate_ed55b8fc6aa0484a82b1c84a60debd29.RegisterName("OuterBorder", Border_fbc7d09498e54a019f0021720e818cea);
                    Border_fbc7d09498e54a019f0021720e818cea.Name = "OuterBorder";
                    templateOwner_ControlTemplate_ed55b8fc6aa0484a82b1c84a60debd29.RegisterName("CommonStates", VisualStateGroup_259836095a9e47988d3896eeaefe6cb8);
                    VisualStateGroup_259836095a9e47988d3896eeaefe6cb8.Name = "CommonStates";
                    templateOwner_ControlTemplate_ed55b8fc6aa0484a82b1c84a60debd29.RegisterName("Normal", VisualState_1fb96ef410f04df5878340ed580e270e);
                    VisualState_1fb96ef410f04df5878340ed580e270e.Name = "Normal";

                    templateOwner_ControlTemplate_ed55b8fc6aa0484a82b1c84a60debd29.RegisterName("PointerOver", VisualState_9d1bdd31c5d4459b880f3a8752b36668);
                    VisualState_9d1bdd31c5d4459b880f3a8752b36668.Name = "PointerOver";
                    var Storyboard_1f8dc0e072f54965a369f19676307abf = new Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard();
                    var ObjectAnimationUsingKeyFrames_5c426a598a6b425c8059dc51c13c9b37 = new Bridge.global.Windows.UI.Xaml.Media.Animation.ObjectAnimationUsingKeyFrames();
                    Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard.SetTargetName(ObjectAnimationUsingKeyFrames_5c426a598a6b425c8059dc51c13c9b37, "ContentPresenter");
                    var DiscreteObjectKeyFrame_0288826bd162441a8f6fd342bc376214 = new Bridge.global.Windows.UI.Xaml.Media.Animation.DiscreteObjectKeyFrame();
                    DiscreteObjectKeyFrame_0288826bd162441a8f6fd342bc376214.KeyTime = Bridge.global.Windows.UI.Xaml.Media.Animation.KeyTime.FromTimeSpan(new Bridge.global.System.TimeSpan(System.Int64(0)));
                    DiscreteObjectKeyFrame_0288826bd162441a8f6fd342bc376214.Value = "Bold";

                    ObjectAnimationUsingKeyFrames_5c426a598a6b425c8059dc51c13c9b37.KeyFrames.add(DiscreteObjectKeyFrame_0288826bd162441a8f6fd342bc376214);


                    Storyboard_1f8dc0e072f54965a369f19676307abf.Children.add(ObjectAnimationUsingKeyFrames_5c426a598a6b425c8059dc51c13c9b37);


                    VisualState_9d1bdd31c5d4459b880f3a8752b36668.Storyboard = Storyboard_1f8dc0e072f54965a369f19676307abf;


                    System.Array.add(VisualStateGroup_259836095a9e47988d3896eeaefe6cb8.States, VisualState_1fb96ef410f04df5878340ed580e270e, Object);
                    System.Array.add(VisualStateGroup_259836095a9e47988d3896eeaefe6cb8.States, VisualState_9d1bdd31c5d4459b880f3a8752b36668, Object);


                    templateOwner_ControlTemplate_ed55b8fc6aa0484a82b1c84a60debd29.INTERNAL_GetVisualStateGroups().add(VisualStateGroup_259836095a9e47988d3896eeaefe6cb8);

                    templateOwner_ControlTemplate_ed55b8fc6aa0484a82b1c84a60debd29.RegisterName("ContentPresenter", ContentPresenter_6c7ce388471d4e1da0eb70631fadcee9);
                    ContentPresenter_6c7ce388471d4e1da0eb70631fadcee9.Name = "ContentPresenter";
                    var Binding_31b8337b3a5040c48e412a81e819c268 = new Bridge.global.Windows.UI.Xaml.Data.Binding.ctor();
                    Binding_31b8337b3a5040c48e412a81e819c268.Path = new Bridge.global.Windows.UI.Xaml.PropertyPath.ctor("ContentTemplate");
                    var RelativeSource_1eb5232262ac4c95b1c6ed84fe08cc98 = new Bridge.global.Windows.UI.Xaml.Data.RelativeSource.ctor();
                    RelativeSource_1eb5232262ac4c95b1c6ed84fe08cc98.Mode = Bridge.global.Windows.UI.Xaml.Data.RelativeSourceMode.TemplatedParent;

                    Binding_31b8337b3a5040c48e412a81e819c268.RelativeSource = RelativeSource_1eb5232262ac4c95b1c6ed84fe08cc98;


                    Binding_31b8337b3a5040c48e412a81e819c268.TemplateOwner = templateInstance_541931248954448784187f890fb98436;

                    var Binding_f4beda79dc7d42ccbf0bc66f7e597944 = new Bridge.global.Windows.UI.Xaml.Data.Binding.ctor();
                    Binding_f4beda79dc7d42ccbf0bc66f7e597944.Path = new Bridge.global.Windows.UI.Xaml.PropertyPath.ctor("Content");
                    var RelativeSource_78dd9b35e3dd47b5bcac51db9eedb8a2 = new Bridge.global.Windows.UI.Xaml.Data.RelativeSource.ctor();
                    RelativeSource_78dd9b35e3dd47b5bcac51db9eedb8a2.Mode = Bridge.global.Windows.UI.Xaml.Data.RelativeSourceMode.TemplatedParent;

                    Binding_f4beda79dc7d42ccbf0bc66f7e597944.RelativeSource = RelativeSource_78dd9b35e3dd47b5bcac51db9eedb8a2;


                    Binding_f4beda79dc7d42ccbf0bc66f7e597944.TemplateOwner = templateInstance_541931248954448784187f890fb98436;

                    var Binding_73167db2beaa4abfaf3ca6273a4440a1 = new Bridge.global.Windows.UI.Xaml.Data.Binding.ctor();
                    Binding_73167db2beaa4abfaf3ca6273a4440a1.Path = new Bridge.global.Windows.UI.Xaml.PropertyPath.ctor("Padding");
                    var RelativeSource_2f29d7cb3752484a850af08f4f1730b4 = new Bridge.global.Windows.UI.Xaml.Data.RelativeSource.ctor();
                    RelativeSource_2f29d7cb3752484a850af08f4f1730b4.Mode = Bridge.global.Windows.UI.Xaml.Data.RelativeSourceMode.TemplatedParent;

                    Binding_73167db2beaa4abfaf3ca6273a4440a1.RelativeSource = RelativeSource_2f29d7cb3752484a850af08f4f1730b4;


                    Binding_73167db2beaa4abfaf3ca6273a4440a1.TemplateOwner = templateInstance_541931248954448784187f890fb98436;

                    var Binding_2a77951280bf4f628921e4bd4735ea05 = new Bridge.global.Windows.UI.Xaml.Data.Binding.ctor();
                    Binding_2a77951280bf4f628921e4bd4735ea05.Path = new Bridge.global.Windows.UI.Xaml.PropertyPath.ctor("HorizontalContentAlignment");
                    var RelativeSource_0c0020b1a3b34e1eaea7ad235b709c4b = new Bridge.global.Windows.UI.Xaml.Data.RelativeSource.ctor();
                    RelativeSource_0c0020b1a3b34e1eaea7ad235b709c4b.Mode = Bridge.global.Windows.UI.Xaml.Data.RelativeSourceMode.TemplatedParent;

                    Binding_2a77951280bf4f628921e4bd4735ea05.RelativeSource = RelativeSource_0c0020b1a3b34e1eaea7ad235b709c4b;


                    Binding_2a77951280bf4f628921e4bd4735ea05.TemplateOwner = templateInstance_541931248954448784187f890fb98436;

                    var Binding_9a1297789dd64ee9bb1f94bc512ce543 = new Bridge.global.Windows.UI.Xaml.Data.Binding.ctor();
                    Binding_9a1297789dd64ee9bb1f94bc512ce543.Path = new Bridge.global.Windows.UI.Xaml.PropertyPath.ctor("VerticalContentAlignment");
                    var RelativeSource_d2b1e8e0846e424eba590a5901a897dc = new Bridge.global.Windows.UI.Xaml.Data.RelativeSource.ctor();
                    RelativeSource_d2b1e8e0846e424eba590a5901a897dc.Mode = Bridge.global.Windows.UI.Xaml.Data.RelativeSourceMode.TemplatedParent;

                    Binding_9a1297789dd64ee9bb1f94bc512ce543.RelativeSource = RelativeSource_d2b1e8e0846e424eba590a5901a897dc;


                    Binding_9a1297789dd64ee9bb1f94bc512ce543.TemplateOwner = templateInstance_541931248954448784187f890fb98436;


                    Border_fbc7d09498e54a019f0021720e818cea.Child = ContentPresenter_6c7ce388471d4e1da0eb70631fadcee9;

                    var Binding_392cf867b33147beaaaebb9289a9c4ba = new Bridge.global.Windows.UI.Xaml.Data.Binding.ctor();
                    Binding_392cf867b33147beaaaebb9289a9c4ba.Path = new Bridge.global.Windows.UI.Xaml.PropertyPath.ctor("Background");
                    var RelativeSource_b5dd98f776514970b61ce74cf5773dd2 = new Bridge.global.Windows.UI.Xaml.Data.RelativeSource.ctor();
                    RelativeSource_b5dd98f776514970b61ce74cf5773dd2.Mode = Bridge.global.Windows.UI.Xaml.Data.RelativeSourceMode.TemplatedParent;

                    Binding_392cf867b33147beaaaebb9289a9c4ba.RelativeSource = RelativeSource_b5dd98f776514970b61ce74cf5773dd2;


                    Binding_392cf867b33147beaaaebb9289a9c4ba.TemplateOwner = templateInstance_541931248954448784187f890fb98436;

                    var Binding_baf06f284b6f4a5dad3e85c6089eb6d3 = new Bridge.global.Windows.UI.Xaml.Data.Binding.ctor();
                    Binding_baf06f284b6f4a5dad3e85c6089eb6d3.Path = new Bridge.global.Windows.UI.Xaml.PropertyPath.ctor("BorderBrush");
                    var RelativeSource_dfb16204768046a98bf6a96cfd202897 = new Bridge.global.Windows.UI.Xaml.Data.RelativeSource.ctor();
                    RelativeSource_dfb16204768046a98bf6a96cfd202897.Mode = Bridge.global.Windows.UI.Xaml.Data.RelativeSourceMode.TemplatedParent;

                    Binding_baf06f284b6f4a5dad3e85c6089eb6d3.RelativeSource = RelativeSource_dfb16204768046a98bf6a96cfd202897;


                    Binding_baf06f284b6f4a5dad3e85c6089eb6d3.TemplateOwner = templateInstance_541931248954448784187f890fb98436;

                    var Binding_baa2f95875804407bdd3ae2592a0a399 = new Bridge.global.Windows.UI.Xaml.Data.Binding.ctor();
                    Binding_baa2f95875804407bdd3ae2592a0a399.Path = new Bridge.global.Windows.UI.Xaml.PropertyPath.ctor("BorderThickness");
                    var RelativeSource_c3dfe2bb09d5485fa30f6151ea40a822 = new Bridge.global.Windows.UI.Xaml.Data.RelativeSource.ctor();
                    RelativeSource_c3dfe2bb09d5485fa30f6151ea40a822.Mode = Bridge.global.Windows.UI.Xaml.Data.RelativeSourceMode.TemplatedParent;

                    Binding_baa2f95875804407bdd3ae2592a0a399.RelativeSource = RelativeSource_c3dfe2bb09d5485fa30f6151ea40a822;


                    Binding_baa2f95875804407bdd3ae2592a0a399.TemplateOwner = templateInstance_541931248954448784187f890fb98436;


                    Bridge.global.Windows.UI.Xaml.Data.BindingOperations.SetBinding(ContentPresenter_6c7ce388471d4e1da0eb70631fadcee9, Bridge.global.Windows.UI.Xaml.Controls.ContentControl.ContentTemplateProperty, Binding_31b8337b3a5040c48e412a81e819c268);
                    Bridge.global.Windows.UI.Xaml.Data.BindingOperations.SetBinding(ContentPresenter_6c7ce388471d4e1da0eb70631fadcee9, Bridge.global.Windows.UI.Xaml.Controls.ContentControl.ContentProperty, Binding_f4beda79dc7d42ccbf0bc66f7e597944);
                    Bridge.global.Windows.UI.Xaml.Data.BindingOperations.SetBinding(ContentPresenter_6c7ce388471d4e1da0eb70631fadcee9, Bridge.global.Windows.UI.Xaml.FrameworkElement.MarginProperty, Binding_73167db2beaa4abfaf3ca6273a4440a1);
                    Bridge.global.Windows.UI.Xaml.Data.BindingOperations.SetBinding(ContentPresenter_6c7ce388471d4e1da0eb70631fadcee9, Bridge.global.Windows.UI.Xaml.FrameworkElement.HorizontalAlignmentProperty, Binding_2a77951280bf4f628921e4bd4735ea05);
                    Bridge.global.Windows.UI.Xaml.Data.BindingOperations.SetBinding(ContentPresenter_6c7ce388471d4e1da0eb70631fadcee9, Bridge.global.Windows.UI.Xaml.FrameworkElement.VerticalAlignmentProperty, Binding_9a1297789dd64ee9bb1f94bc512ce543);
                    Bridge.global.Windows.UI.Xaml.Data.BindingOperations.SetBinding(Border_fbc7d09498e54a019f0021720e818cea, Bridge.global.Windows.UI.Xaml.Controls.Border.BackgroundProperty, Binding_392cf867b33147beaaaebb9289a9c4ba);
                    Bridge.global.Windows.UI.Xaml.Data.BindingOperations.SetBinding(Border_fbc7d09498e54a019f0021720e818cea, Bridge.global.Windows.UI.Xaml.Controls.Border.BorderBrushProperty, Binding_baf06f284b6f4a5dad3e85c6089eb6d3);
                    Bridge.global.Windows.UI.Xaml.Data.BindingOperations.SetBinding(Border_fbc7d09498e54a019f0021720e818cea, Bridge.global.Windows.UI.Xaml.Controls.Border.BorderThicknessProperty, Binding_baa2f95875804407bdd3ae2592a0a399);

                    Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard.SetTargetProperty(ObjectAnimationUsingKeyFrames_5c426a598a6b425c8059dc51c13c9b37, new Bridge.global.Windows.UI.Xaml.PropertyPath.$ctor1("FontWeight", "FontWeight", Bridge.fn.cacheBind(this, this.accessVisualStateProperty_0cfa0c0b465544baa0077553c50162e9), Bridge.fn.cacheBind(this, this.setVisualStateProperty_0cfa0c0b465544baa0077553c50162e9), Bridge.fn.cacheBind(this, this.setAnimationVisualStateProperty_0cfa0c0b465544baa0077553c50162e9), Bridge.fn.cacheBind(this, this.setLocalVisualStateProperty_0cfa0c0b465544baa0077553c50162e9), Bridge.fn.cacheBind(this, this.getVisualStateProperty_0cfa0c0b465544baa0077553c50162e9)));
                    Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard.SetTarget(ObjectAnimationUsingKeyFrames_5c426a598a6b425c8059dc51c13c9b37, ContentPresenter_6c7ce388471d4e1da0eb70631fadcee9);

                    templateInstance_541931248954448784187f890fb98436.TemplateContent = Border_fbc7d09498e54a019f0021720e818cea;
                    return templateInstance_541931248954448784187f890fb98436;
                }));

                Setter_344a2e7b5a1b4d87b6a623fa7e6ace62.Value = ControlTemplate_ed55b8fc6aa0484a82b1c84a60debd29;


                Style_e5fa8d55aa354923a7af3273e8391b13.Setters.add(Setter_790aaa22fb9449ca9149824b49d306ba);
                Style_e5fa8d55aa354923a7af3273e8391b13.Setters.add(Setter_bc41f9e1bf564fd3849bea4b31c03fea);
                Style_e5fa8d55aa354923a7af3273e8391b13.Setters.add(Setter_10a0be012a1b4828bf6fa471f2baa4c6);
                Style_e5fa8d55aa354923a7af3273e8391b13.Setters.add(Setter_53361baa6dbb4f09b6e0d44dd750f1a3);
                Style_e5fa8d55aa354923a7af3273e8391b13.Setters.add(Setter_344a2e7b5a1b4d87b6a623fa7e6ace62);


                this.setItem(Bridge.global.Windows.UI.Xaml.Controls.HyperlinkButton, Style_e5fa8d55aa354923a7af3273e8391b13);






            },
            accessVisualStateProperty_0cfa0c0b465544baa0077553c50162e9: function (rootTargetObjectInstance) {
                return new (Bridge.GeneratorEnumerable$1(System.Tuple$3(Windows.UI.Xaml.DependencyObject,Windows.UI.Xaml.DependencyProperty,System.Nullable$1(System.Int32))))(Bridge.fn.bind(this, function (rootTargetObjectInstance) {
                    var $step = 0,
                        $jumpFromFinally,
                        $returnValue,
                        $async_e;

                    var $enumerator = new (Bridge.GeneratorEnumerator$1(System.Tuple$3(Windows.UI.Xaml.DependencyObject,Windows.UI.Xaml.DependencyProperty,System.Nullable$1(System.Int32))))(Bridge.fn.bind(this, function () {
                        try {
                            for (;;) {
                                switch ($step) {
                                    case 0: {
                                        return false;
                                    }
                                    default: {
                                        return false;
                                    }
                                }
                            }
                        } catch($async_e1) {
                            $async_e = System.Exception.create($async_e1);
                            throw $async_e;
                        }
                    }));
                    return $enumerator;
                }, arguments));
            },
            setVisualStateProperty_0cfa0c0b465544baa0077553c50162e9: function (finalTargetInstance, value) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "FontWeight").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "FontWeightProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                (finalTargetInstance).SetVisualStateValue(property, value);
            },
            setAnimationVisualStateProperty_0cfa0c0b465544baa0077553c50162e9: function (finalTargetInstance, value) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "FontWeight").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "FontWeightProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                (finalTargetInstance).SetAnimationValue(property, value);
            },
            setLocalVisualStateProperty_0cfa0c0b465544baa0077553c50162e9: function (finalTargetInstance, value) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "FontWeight").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "FontWeightProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                (finalTargetInstance).SetLocalValue(property, value);
            },
            getVisualStateProperty_0cfa0c0b465544baa0077553c50162e9: function (finalTargetInstance) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "FontWeight").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "FontWeightProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                return finalTargetInstance.GetVisualStateValue(property);
            }
        }
    });

    Bridge.define("___CSHTML5__Component__Defaultstyles__INTERNAL_Defaultitemscontrolstyle__Xaml", {
        inherits: [Bridge.global.Windows.UI.Xaml.ResourceDictionary],
        fields: {
            _contentLoaded: false
        },
        ctors: {
            ctor: function () {
                this.$initialize();
                Bridge.global.Windows.UI.Xaml.ResourceDictionary.ctor.call(this);
                this.InitializeComponent();

                if (Bridge.is(this, Windows.UI.Xaml.UIElement)) {
                    Bridge.cast(this, Windows.UI.Xaml.UIElement).XamlSourcePath = "CSHTML5\\DefaultStyles\\INTERNAL_DefaultItemsControlStyle.xaml";
                }
            }
        },
        methods: {
            InitializeComponent: function () {
                if (this._contentLoaded) {
                    return;
                }
                this._contentLoaded = true;


                if (Bridge.is(this, Windows.UI.Xaml.UIElement)) {
                    Bridge.cast(this, Windows.UI.Xaml.UIElement).XamlSourcePath = "CSHTML5\\DefaultStyles\\INTERNAL_DefaultItemsControlStyle.xaml";
                }



                var Style_dbba48719f784015ab999553c77be67a = new Bridge.global.Windows.UI.Xaml.Style.ctor();
                Style_dbba48719f784015ab999553c77be67a.TargetType = Bridge.global.Windows.UI.Xaml.Controls.ItemsControl;
                var Setter_b2dcab054164423399c0baab23ac1236 = new Bridge.global.Windows.UI.Xaml.Setter.ctor();
                Setter_b2dcab054164423399c0baab23ac1236.Property = Bridge.global.Windows.UI.Xaml.Controls.Control.TemplateProperty;
                var ControlTemplate_0c9f19f0aab04419a57e20a208b42334 = new Bridge.global.Windows.UI.Xaml.Controls.ControlTemplate();
                ControlTemplate_0c9f19f0aab04419a57e20a208b42334.TargetType = Bridge.global.Windows.UI.Xaml.Controls.ItemsControl;
                ControlTemplate_0c9f19f0aab04419a57e20a208b42334.SetMethodToInstantiateFrameworkTemplate(function (templateOwner_ControlTemplate_0c9f19f0aab04419a57e20a208b42334) {
                    var templateInstance_709ddeabeef04d6099622382dbceddd1 = new Bridge.global.Windows.UI.Xaml.TemplateInstance();
                    templateInstance_709ddeabeef04d6099622382dbceddd1.TemplateOwner = templateOwner_ControlTemplate_0c9f19f0aab04419a57e20a208b42334;
                    var ItemsPresenter_5c832c8edd764d34936554e731645052 = new Bridge.global.Windows.UI.Xaml.Controls.ItemsPresenter();
                    templateOwner_ControlTemplate_0c9f19f0aab04419a57e20a208b42334.RegisterName("ItemsHost", ItemsPresenter_5c832c8edd764d34936554e731645052);
                    ItemsPresenter_5c832c8edd764d34936554e731645052.Name = "ItemsHost";



                    templateInstance_709ddeabeef04d6099622382dbceddd1.TemplateContent = ItemsPresenter_5c832c8edd764d34936554e731645052;
                    return templateInstance_709ddeabeef04d6099622382dbceddd1;
                });

                Setter_b2dcab054164423399c0baab23ac1236.Value = ControlTemplate_0c9f19f0aab04419a57e20a208b42334;


                Style_dbba48719f784015ab999553c77be67a.Setters.add(Setter_b2dcab054164423399c0baab23ac1236);


                this.setItem(Bridge.global.Windows.UI.Xaml.Controls.ItemsControl, Style_dbba48719f784015ab999553c77be67a);






            }
        }
    });

    Bridge.define("___CSHTML5__Component__Defaultstyles__INTERNAL_Defaultlabelstyle__Xaml", {
        inherits: [Bridge.global.Windows.UI.Xaml.ResourceDictionary],
        fields: {
            _contentLoaded: false
        },
        ctors: {
            ctor: function () {
                this.$initialize();
                Bridge.global.Windows.UI.Xaml.ResourceDictionary.ctor.call(this);
                this.InitializeComponent();

                if (Bridge.is(this, Windows.UI.Xaml.UIElement)) {
                    Bridge.cast(this, Windows.UI.Xaml.UIElement).XamlSourcePath = "CSHTML5\\DefaultStyles\\INTERNAL_DefaultLabelStyle.xaml";
                }
            }
        },
        methods: {
            InitializeComponent: function () {
                if (this._contentLoaded) {
                    return;
                }
                this._contentLoaded = true;


                if (Bridge.is(this, Windows.UI.Xaml.UIElement)) {
                    Bridge.cast(this, Windows.UI.Xaml.UIElement).XamlSourcePath = "CSHTML5\\DefaultStyles\\INTERNAL_DefaultLabelStyle.xaml";
                }



                var Style_fca34627be0640509e3b1bb27db4c83c = new Bridge.global.Windows.UI.Xaml.Style.ctor();
                Style_fca34627be0640509e3b1bb27db4c83c.TargetType = Bridge.global.Windows.UI.Xaml.Controls.Label;
                var Setter_cc6ac8fc6a10428b8109cdd240d5c099 = new Bridge.global.Windows.UI.Xaml.Setter.ctor();
                Setter_cc6ac8fc6a10428b8109cdd240d5c099.Property = Bridge.global.Windows.UI.Xaml.Controls.Control.HorizontalContentAlignmentProperty;
                Setter_cc6ac8fc6a10428b8109cdd240d5c099.Value = Bridge.box(Bridge.global.Windows.UI.Xaml.HorizontalAlignment.Left, Windows.UI.Xaml.HorizontalAlignment, System.Enum.toStringFn(Windows.UI.Xaml.HorizontalAlignment));

                var Setter_18803788558a4411be7356f2ad185eae = new Bridge.global.Windows.UI.Xaml.Setter.ctor();
                Setter_18803788558a4411be7356f2ad185eae.Property = Bridge.global.Windows.UI.Xaml.Controls.Control.VerticalContentAlignmentProperty;
                Setter_18803788558a4411be7356f2ad185eae.Value = Bridge.box(Bridge.global.Windows.UI.Xaml.VerticalAlignment.Center, Windows.UI.Xaml.VerticalAlignment, System.Enum.toStringFn(Windows.UI.Xaml.VerticalAlignment));

                var Setter_b0168de59de94c31b59e42d848f8fb09 = new Bridge.global.Windows.UI.Xaml.Setter.ctor();
                Setter_b0168de59de94c31b59e42d848f8fb09.Property = Bridge.global.Windows.UI.Xaml.Controls.Control.TemplateProperty;
                var ControlTemplate_648a01dee1ad463ba3937225d0254238 = new Bridge.global.Windows.UI.Xaml.Controls.ControlTemplate();
                ControlTemplate_648a01dee1ad463ba3937225d0254238.TargetType = Bridge.global.Windows.UI.Xaml.Controls.Label;
                ControlTemplate_648a01dee1ad463ba3937225d0254238.SetMethodToInstantiateFrameworkTemplate(function (templateOwner_ControlTemplate_648a01dee1ad463ba3937225d0254238) {
                    var templateInstance_f6abaeabe1784e2fbd3b4dc8f6a33c04 = new Bridge.global.Windows.UI.Xaml.TemplateInstance();
                    templateInstance_f6abaeabe1784e2fbd3b4dc8f6a33c04.TemplateOwner = templateOwner_ControlTemplate_648a01dee1ad463ba3937225d0254238;

                    var Border_fb10dc7cc34d49478056b97a99891c7b = new Bridge.global.Windows.UI.Xaml.Controls.Border();
                    var ContentPresenter_dfcc4f30065b4fcb8924baf263e525b3 = new Bridge.global.Windows.UI.Xaml.Controls.ContentPresenter();
                    var Binding_f15a5c1df2f84ec4984cc929bdf813ae = new Bridge.global.Windows.UI.Xaml.Data.Binding.ctor();
                    Binding_f15a5c1df2f84ec4984cc929bdf813ae.Path = new Bridge.global.Windows.UI.Xaml.PropertyPath.ctor("Cursor");
                    var RelativeSource_1b806a40ec9d46b6a2c6ba8f8126d334 = new Bridge.global.Windows.UI.Xaml.Data.RelativeSource.ctor();
                    RelativeSource_1b806a40ec9d46b6a2c6ba8f8126d334.Mode = Bridge.global.Windows.UI.Xaml.Data.RelativeSourceMode.TemplatedParent;

                    Binding_f15a5c1df2f84ec4984cc929bdf813ae.RelativeSource = RelativeSource_1b806a40ec9d46b6a2c6ba8f8126d334;


                    Binding_f15a5c1df2f84ec4984cc929bdf813ae.TemplateOwner = templateInstance_f6abaeabe1784e2fbd3b4dc8f6a33c04;

                    var Binding_2bcb741af308445e916e7372ee485349 = new Bridge.global.Windows.UI.Xaml.Data.Binding.ctor();
                    Binding_2bcb741af308445e916e7372ee485349.Path = new Bridge.global.Windows.UI.Xaml.PropertyPath.ctor("ContentTemplate");
                    var RelativeSource_67fdf46fe759431c96ea9f45b413e4e2 = new Bridge.global.Windows.UI.Xaml.Data.RelativeSource.ctor();
                    RelativeSource_67fdf46fe759431c96ea9f45b413e4e2.Mode = Bridge.global.Windows.UI.Xaml.Data.RelativeSourceMode.TemplatedParent;

                    Binding_2bcb741af308445e916e7372ee485349.RelativeSource = RelativeSource_67fdf46fe759431c96ea9f45b413e4e2;


                    Binding_2bcb741af308445e916e7372ee485349.TemplateOwner = templateInstance_f6abaeabe1784e2fbd3b4dc8f6a33c04;

                    var Binding_a73e38c275d74102a5ad551e0f1d5026 = new Bridge.global.Windows.UI.Xaml.Data.Binding.ctor();
                    Binding_a73e38c275d74102a5ad551e0f1d5026.Path = new Bridge.global.Windows.UI.Xaml.PropertyPath.ctor("Content");
                    var RelativeSource_aa8fa91bd8a3452a95167a46ee84f23d = new Bridge.global.Windows.UI.Xaml.Data.RelativeSource.ctor();
                    RelativeSource_aa8fa91bd8a3452a95167a46ee84f23d.Mode = Bridge.global.Windows.UI.Xaml.Data.RelativeSourceMode.TemplatedParent;

                    Binding_a73e38c275d74102a5ad551e0f1d5026.RelativeSource = RelativeSource_aa8fa91bd8a3452a95167a46ee84f23d;


                    Binding_a73e38c275d74102a5ad551e0f1d5026.TemplateOwner = templateInstance_f6abaeabe1784e2fbd3b4dc8f6a33c04;

                    var Binding_e79bc5a2b9aa464cbb42968928a7f705 = new Bridge.global.Windows.UI.Xaml.Data.Binding.ctor();
                    Binding_e79bc5a2b9aa464cbb42968928a7f705.Path = new Bridge.global.Windows.UI.Xaml.PropertyPath.ctor("HorizontalContentAlignment");
                    var RelativeSource_36bc5db20f664203ab697eab4d0bd8f9 = new Bridge.global.Windows.UI.Xaml.Data.RelativeSource.ctor();
                    RelativeSource_36bc5db20f664203ab697eab4d0bd8f9.Mode = Bridge.global.Windows.UI.Xaml.Data.RelativeSourceMode.TemplatedParent;

                    Binding_e79bc5a2b9aa464cbb42968928a7f705.RelativeSource = RelativeSource_36bc5db20f664203ab697eab4d0bd8f9;


                    Binding_e79bc5a2b9aa464cbb42968928a7f705.TemplateOwner = templateInstance_f6abaeabe1784e2fbd3b4dc8f6a33c04;

                    var Binding_4eab3b53d326409eb3e9b1e441a5f73c = new Bridge.global.Windows.UI.Xaml.Data.Binding.ctor();
                    Binding_4eab3b53d326409eb3e9b1e441a5f73c.Path = new Bridge.global.Windows.UI.Xaml.PropertyPath.ctor("Padding");
                    var RelativeSource_88b200f41cd44e2eba7fe73c38f0ffb3 = new Bridge.global.Windows.UI.Xaml.Data.RelativeSource.ctor();
                    RelativeSource_88b200f41cd44e2eba7fe73c38f0ffb3.Mode = Bridge.global.Windows.UI.Xaml.Data.RelativeSourceMode.TemplatedParent;

                    Binding_4eab3b53d326409eb3e9b1e441a5f73c.RelativeSource = RelativeSource_88b200f41cd44e2eba7fe73c38f0ffb3;


                    Binding_4eab3b53d326409eb3e9b1e441a5f73c.TemplateOwner = templateInstance_f6abaeabe1784e2fbd3b4dc8f6a33c04;

                    var Binding_7f44cae972f7413cbadb446d23dd880a = new Bridge.global.Windows.UI.Xaml.Data.Binding.ctor();
                    Binding_7f44cae972f7413cbadb446d23dd880a.Path = new Bridge.global.Windows.UI.Xaml.PropertyPath.ctor("VerticalContentAlignment");
                    var RelativeSource_16ee575976f047f29c4b71def7c155af = new Bridge.global.Windows.UI.Xaml.Data.RelativeSource.ctor();
                    RelativeSource_16ee575976f047f29c4b71def7c155af.Mode = Bridge.global.Windows.UI.Xaml.Data.RelativeSourceMode.TemplatedParent;

                    Binding_7f44cae972f7413cbadb446d23dd880a.RelativeSource = RelativeSource_16ee575976f047f29c4b71def7c155af;


                    Binding_7f44cae972f7413cbadb446d23dd880a.TemplateOwner = templateInstance_f6abaeabe1784e2fbd3b4dc8f6a33c04;


                    Border_fb10dc7cc34d49478056b97a99891c7b.Child = ContentPresenter_dfcc4f30065b4fcb8924baf263e525b3;

                    var Binding_7dc4d5cd90cb49038005f27ddc8b6d9d = new Bridge.global.Windows.UI.Xaml.Data.Binding.ctor();
                    Binding_7dc4d5cd90cb49038005f27ddc8b6d9d.Path = new Bridge.global.Windows.UI.Xaml.PropertyPath.ctor("BorderBrush");
                    var RelativeSource_8e6616c9c5e34fcb8dedfb942347c14c = new Bridge.global.Windows.UI.Xaml.Data.RelativeSource.ctor();
                    RelativeSource_8e6616c9c5e34fcb8dedfb942347c14c.Mode = Bridge.global.Windows.UI.Xaml.Data.RelativeSourceMode.TemplatedParent;

                    Binding_7dc4d5cd90cb49038005f27ddc8b6d9d.RelativeSource = RelativeSource_8e6616c9c5e34fcb8dedfb942347c14c;


                    Binding_7dc4d5cd90cb49038005f27ddc8b6d9d.TemplateOwner = templateInstance_f6abaeabe1784e2fbd3b4dc8f6a33c04;

                    var Binding_c88943557e604f428c950bce367c59e9 = new Bridge.global.Windows.UI.Xaml.Data.Binding.ctor();
                    Binding_c88943557e604f428c950bce367c59e9.Path = new Bridge.global.Windows.UI.Xaml.PropertyPath.ctor("BorderThickness");
                    var RelativeSource_937f360740504bfeac4eb67d272f4ad4 = new Bridge.global.Windows.UI.Xaml.Data.RelativeSource.ctor();
                    RelativeSource_937f360740504bfeac4eb67d272f4ad4.Mode = Bridge.global.Windows.UI.Xaml.Data.RelativeSourceMode.TemplatedParent;

                    Binding_c88943557e604f428c950bce367c59e9.RelativeSource = RelativeSource_937f360740504bfeac4eb67d272f4ad4;


                    Binding_c88943557e604f428c950bce367c59e9.TemplateOwner = templateInstance_f6abaeabe1784e2fbd3b4dc8f6a33c04;

                    var Binding_e88cba2a2010438e98f0bf5890e11d71 = new Bridge.global.Windows.UI.Xaml.Data.Binding.ctor();
                    Binding_e88cba2a2010438e98f0bf5890e11d71.Path = new Bridge.global.Windows.UI.Xaml.PropertyPath.ctor("Background");
                    var RelativeSource_251daa8fe17148c28f6d873ec7ee6a21 = new Bridge.global.Windows.UI.Xaml.Data.RelativeSource.ctor();
                    RelativeSource_251daa8fe17148c28f6d873ec7ee6a21.Mode = Bridge.global.Windows.UI.Xaml.Data.RelativeSourceMode.TemplatedParent;

                    Binding_e88cba2a2010438e98f0bf5890e11d71.RelativeSource = RelativeSource_251daa8fe17148c28f6d873ec7ee6a21;


                    Binding_e88cba2a2010438e98f0bf5890e11d71.TemplateOwner = templateInstance_f6abaeabe1784e2fbd3b4dc8f6a33c04;

                    var Binding_bca36c9008a24d63930c98c103cb60a0 = new Bridge.global.Windows.UI.Xaml.Data.Binding.ctor();
                    Binding_bca36c9008a24d63930c98c103cb60a0.Path = new Bridge.global.Windows.UI.Xaml.PropertyPath.ctor("Padding");
                    var RelativeSource_4b91751adc96410dbe15e81f947c1521 = new Bridge.global.Windows.UI.Xaml.Data.RelativeSource.ctor();
                    RelativeSource_4b91751adc96410dbe15e81f947c1521.Mode = Bridge.global.Windows.UI.Xaml.Data.RelativeSourceMode.TemplatedParent;

                    Binding_bca36c9008a24d63930c98c103cb60a0.RelativeSource = RelativeSource_4b91751adc96410dbe15e81f947c1521;


                    Binding_bca36c9008a24d63930c98c103cb60a0.TemplateOwner = templateInstance_f6abaeabe1784e2fbd3b4dc8f6a33c04;


                    Bridge.global.Windows.UI.Xaml.Data.BindingOperations.SetBinding(ContentPresenter_dfcc4f30065b4fcb8924baf263e525b3, Bridge.global.Windows.UI.Xaml.FrameworkElement.CursorProperty, Binding_f15a5c1df2f84ec4984cc929bdf813ae);
                    Bridge.global.Windows.UI.Xaml.Data.BindingOperations.SetBinding(ContentPresenter_dfcc4f30065b4fcb8924baf263e525b3, Bridge.global.Windows.UI.Xaml.Controls.ContentControl.ContentTemplateProperty, Binding_2bcb741af308445e916e7372ee485349);
                    Bridge.global.Windows.UI.Xaml.Data.BindingOperations.SetBinding(ContentPresenter_dfcc4f30065b4fcb8924baf263e525b3, Bridge.global.Windows.UI.Xaml.Controls.ContentControl.ContentProperty, Binding_a73e38c275d74102a5ad551e0f1d5026);
                    Bridge.global.Windows.UI.Xaml.Data.BindingOperations.SetBinding(ContentPresenter_dfcc4f30065b4fcb8924baf263e525b3, Bridge.global.Windows.UI.Xaml.FrameworkElement.HorizontalAlignmentProperty, Binding_e79bc5a2b9aa464cbb42968928a7f705);
                    Bridge.global.Windows.UI.Xaml.Data.BindingOperations.SetBinding(ContentPresenter_dfcc4f30065b4fcb8924baf263e525b3, Bridge.global.Windows.UI.Xaml.FrameworkElement.MarginProperty, Binding_4eab3b53d326409eb3e9b1e441a5f73c);
                    Bridge.global.Windows.UI.Xaml.Data.BindingOperations.SetBinding(ContentPresenter_dfcc4f30065b4fcb8924baf263e525b3, Bridge.global.Windows.UI.Xaml.FrameworkElement.VerticalAlignmentProperty, Binding_7f44cae972f7413cbadb446d23dd880a);
                    Bridge.global.Windows.UI.Xaml.Data.BindingOperations.SetBinding(Border_fb10dc7cc34d49478056b97a99891c7b, Bridge.global.Windows.UI.Xaml.Controls.Border.BorderBrushProperty, Binding_7dc4d5cd90cb49038005f27ddc8b6d9d);
                    Bridge.global.Windows.UI.Xaml.Data.BindingOperations.SetBinding(Border_fb10dc7cc34d49478056b97a99891c7b, Bridge.global.Windows.UI.Xaml.Controls.Border.BorderThicknessProperty, Binding_c88943557e604f428c950bce367c59e9);
                    Bridge.global.Windows.UI.Xaml.Data.BindingOperations.SetBinding(Border_fb10dc7cc34d49478056b97a99891c7b, Bridge.global.Windows.UI.Xaml.Controls.Border.BackgroundProperty, Binding_e88cba2a2010438e98f0bf5890e11d71);
                    Bridge.global.Windows.UI.Xaml.Data.BindingOperations.SetBinding(Border_fb10dc7cc34d49478056b97a99891c7b, Bridge.global.Windows.UI.Xaml.Controls.Border.PaddingProperty, Binding_bca36c9008a24d63930c98c103cb60a0);

                    templateInstance_f6abaeabe1784e2fbd3b4dc8f6a33c04.TemplateContent = Border_fb10dc7cc34d49478056b97a99891c7b;
                    return templateInstance_f6abaeabe1784e2fbd3b4dc8f6a33c04;
                });

                Setter_b0168de59de94c31b59e42d848f8fb09.Value = ControlTemplate_648a01dee1ad463ba3937225d0254238;


                Style_fca34627be0640509e3b1bb27db4c83c.Setters.add(Setter_cc6ac8fc6a10428b8109cdd240d5c099);
                Style_fca34627be0640509e3b1bb27db4c83c.Setters.add(Setter_18803788558a4411be7356f2ad185eae);
                Style_fca34627be0640509e3b1bb27db4c83c.Setters.add(Setter_b0168de59de94c31b59e42d848f8fb09);


                this.setItem(Bridge.global.Windows.UI.Xaml.Controls.Label, Style_fca34627be0640509e3b1bb27db4c83c);






            }
        }
    });

    Bridge.define("___CSHTML5__Component__Defaultstyles__INTERNAL_Defaultmenuitemstyle__Xaml", {
        inherits: [Bridge.global.Windows.UI.Xaml.ResourceDictionary],
        fields: {
            _contentLoaded: false
        },
        ctors: {
            ctor: function () {
                this.$initialize();
                Bridge.global.Windows.UI.Xaml.ResourceDictionary.ctor.call(this);
                this.InitializeComponent();

                if (Bridge.is(this, Windows.UI.Xaml.UIElement)) {
                    Bridge.cast(this, Windows.UI.Xaml.UIElement).XamlSourcePath = "CSHTML5\\DefaultStyles\\INTERNAL_DefaultMenuItemStyle.xaml";
                }
            }
        },
        methods: {
            InitializeComponent: function () {
                var $t;
                if (this._contentLoaded) {
                    return;
                }
                this._contentLoaded = true;


                if (Bridge.is(this, Windows.UI.Xaml.UIElement)) {
                    Bridge.cast(this, Windows.UI.Xaml.UIElement).XamlSourcePath = "CSHTML5\\DefaultStyles\\INTERNAL_DefaultMenuItemStyle.xaml";
                }



                var Style_a31d976f43ce40da94e520b9ebec8f87 = new Bridge.global.Windows.UI.Xaml.Style.ctor();
                Style_a31d976f43ce40da94e520b9ebec8f87.TargetType = Bridge.global.Windows.UI.Xaml.Controls.MenuItem;
                var Setter_681fdcc9824d4cb49307eb6193a377b7 = new Bridge.global.Windows.UI.Xaml.Setter.ctor();
                Setter_681fdcc9824d4cb49307eb6193a377b7.Property = Bridge.global.Windows.UI.Xaml.Controls.Control.BackgroundProperty;
                Setter_681fdcc9824d4cb49307eb6193a377b7.Value = new Bridge.global.Windows.UI.Xaml.Media.SolidColorBrush.$ctor1(($t = new Bridge.global.Windows.UI.Color(), $t.A = 255, $t.R = 226, $t.G = 226, $t.B = 226, $t));

                var Setter_31be95160bff4369a7abf0d417eb689a = new Bridge.global.Windows.UI.Xaml.Setter.ctor();
                Setter_31be95160bff4369a7abf0d417eb689a.Property = Bridge.global.Windows.UI.Xaml.Controls.Control.ForegroundProperty;
                Setter_31be95160bff4369a7abf0d417eb689a.Value = new Bridge.global.Windows.UI.Xaml.Media.SolidColorBrush.$ctor1(($t = new Bridge.global.Windows.UI.Color(), $t.A = 255, $t.R = 0, $t.G = 0, $t.B = 0, $t));

                var Setter_e9002444d2524dd8b90a7a731b61fdbb = new Bridge.global.Windows.UI.Xaml.Setter.ctor();
                Setter_e9002444d2524dd8b90a7a731b61fdbb.Property = Bridge.global.Windows.UI.Xaml.Controls.Control.BorderThicknessProperty;
                Setter_e9002444d2524dd8b90a7a731b61fdbb.Value = new Bridge.global.Windows.UI.Xaml.Thickness.$ctor1(0).$clone();

                var Setter_50b2f8b84e2d44cbb4b9ff543a178379 = new Bridge.global.Windows.UI.Xaml.Setter.ctor();
                Setter_50b2f8b84e2d44cbb4b9ff543a178379.Property = Bridge.global.Windows.UI.Xaml.Controls.Control.PaddingProperty;
                Setter_50b2f8b84e2d44cbb4b9ff543a178379.Value = new Bridge.global.Windows.UI.Xaml.Thickness.$ctor2(12, 4, 12, 4).$clone();

                var Setter_4b049b27b85542eab7b119afdf6dc902 = new Bridge.global.Windows.UI.Xaml.Setter.ctor();
                Setter_4b049b27b85542eab7b119afdf6dc902.Property = Bridge.global.Windows.UI.Xaml.FrameworkElement.CursorProperty;
                Setter_4b049b27b85542eab7b119afdf6dc902.Value = Bridge.global.System.Windows.Input.Cursors.Hand;

                var Setter_c1b8ce71bd8440c1ba8582c734400a4c = new Bridge.global.Windows.UI.Xaml.Setter.ctor();
                Setter_c1b8ce71bd8440c1ba8582c734400a4c.Property = Bridge.global.Windows.UI.Xaml.Controls.Control.HorizontalContentAlignmentProperty;
                Setter_c1b8ce71bd8440c1ba8582c734400a4c.Value = Bridge.box(Bridge.global.Windows.UI.Xaml.HorizontalAlignment.Center, Windows.UI.Xaml.HorizontalAlignment, System.Enum.toStringFn(Windows.UI.Xaml.HorizontalAlignment));

                var Setter_7f852a1e5e684b55a435e9e3e1e773fa = new Bridge.global.Windows.UI.Xaml.Setter.ctor();
                Setter_7f852a1e5e684b55a435e9e3e1e773fa.Property = Bridge.global.Windows.UI.Xaml.Controls.Control.VerticalContentAlignmentProperty;
                Setter_7f852a1e5e684b55a435e9e3e1e773fa.Value = Bridge.box(Bridge.global.Windows.UI.Xaml.VerticalAlignment.Center, Windows.UI.Xaml.VerticalAlignment, System.Enum.toStringFn(Windows.UI.Xaml.VerticalAlignment));

                var Setter_aabb875321eb4cda97067c47dd1a800f = new Bridge.global.Windows.UI.Xaml.Setter.ctor();
                Setter_aabb875321eb4cda97067c47dd1a800f.Property = Bridge.global.Windows.UI.Xaml.Controls.Control.TemplateProperty;
                var ControlTemplate_6c61f3dccfe14a2ebaa9332846c0ed0b = new Bridge.global.Windows.UI.Xaml.Controls.ControlTemplate();
                ControlTemplate_6c61f3dccfe14a2ebaa9332846c0ed0b.TargetType = Bridge.global.Windows.UI.Xaml.Controls.MenuItem;
                ControlTemplate_6c61f3dccfe14a2ebaa9332846c0ed0b.SetMethodToInstantiateFrameworkTemplate(Bridge.fn.bind(this, function (templateOwner_ControlTemplate_6c61f3dccfe14a2ebaa9332846c0ed0b) {
                    var templateInstance_2e2dd1b3065d4c95850d6d8a39e64cf3 = new Bridge.global.Windows.UI.Xaml.TemplateInstance();
                    templateInstance_2e2dd1b3065d4c95850d6d8a39e64cf3.TemplateOwner = templateOwner_ControlTemplate_6c61f3dccfe14a2ebaa9332846c0ed0b;
                    var VisualState_a5e5e1801ad34d4abb6790f5f5e785ba = new Bridge.global.Windows.UI.Xaml.VisualState();
                    var VisualState_ae4cc0d35758445fa280c25c8d4e8238 = new Bridge.global.Windows.UI.Xaml.VisualState();
                    var VisualState_4d81e74e52084b3292c580d08996d5bd = new Bridge.global.Windows.UI.Xaml.VisualState();
                    var VisualState_270f28d71bef4ebaa3985432411bbec9 = new Bridge.global.Windows.UI.Xaml.VisualState();
                    var VisualStateGroup_9afb9aa2451f4a609c745b211972c191 = new Bridge.global.Windows.UI.Xaml.VisualStateGroup();
                    var ContentPresenter_ba420b13f8974e748c2b4ce5e3be83e3 = new Bridge.global.Windows.UI.Xaml.Controls.ContentPresenter();
                    var ContentPresenter_56b3bf4dd9b74f1ab0af6485c6ddfe4e = new Bridge.global.Windows.UI.Xaml.Controls.ContentPresenter();
                    var Border_e3243ef889b74918a702bc390f0b2d2a = new Bridge.global.Windows.UI.Xaml.Controls.Border();
                    var Border_356f2d2e61b7432c98996b422e364014 = new Bridge.global.Windows.UI.Xaml.Controls.Border();
                    templateOwner_ControlTemplate_6c61f3dccfe14a2ebaa9332846c0ed0b.RegisterName("OuterBorder", Border_356f2d2e61b7432c98996b422e364014);
                    Border_356f2d2e61b7432c98996b422e364014.Name = "OuterBorder";
                    templateOwner_ControlTemplate_6c61f3dccfe14a2ebaa9332846c0ed0b.RegisterName("CommonStates", VisualStateGroup_9afb9aa2451f4a609c745b211972c191);
                    VisualStateGroup_9afb9aa2451f4a609c745b211972c191.Name = "CommonStates";
                    templateOwner_ControlTemplate_6c61f3dccfe14a2ebaa9332846c0ed0b.RegisterName("Normal", VisualState_a5e5e1801ad34d4abb6790f5f5e785ba);
                    VisualState_a5e5e1801ad34d4abb6790f5f5e785ba.Name = "Normal";

                    templateOwner_ControlTemplate_6c61f3dccfe14a2ebaa9332846c0ed0b.RegisterName("PointerOver", VisualState_ae4cc0d35758445fa280c25c8d4e8238);
                    VisualState_ae4cc0d35758445fa280c25c8d4e8238.Name = "PointerOver";
                    var Storyboard_0604e4ba80b241a78abc5f40947bdf0c = new Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard();
                    var ObjectAnimationUsingKeyFrames_b2059a57eb11401f87a31e388593e77d = new Bridge.global.Windows.UI.Xaml.Media.Animation.ObjectAnimationUsingKeyFrames();
                    Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard.SetTargetName(ObjectAnimationUsingKeyFrames_b2059a57eb11401f87a31e388593e77d, "InnerBorder");
                    var DiscreteObjectKeyFrame_6c8900a3d0504928812400475feb02f6 = new Bridge.global.Windows.UI.Xaml.Media.Animation.DiscreteObjectKeyFrame();
                    DiscreteObjectKeyFrame_6c8900a3d0504928812400475feb02f6.KeyTime = Bridge.global.Windows.UI.Xaml.Media.Animation.KeyTime.FromTimeSpan(new Bridge.global.System.TimeSpan(System.Int64(0)));
                    DiscreteObjectKeyFrame_6c8900a3d0504928812400475feb02f6.Value = "#11000000";

                    ObjectAnimationUsingKeyFrames_b2059a57eb11401f87a31e388593e77d.KeyFrames.add(DiscreteObjectKeyFrame_6c8900a3d0504928812400475feb02f6);


                    Storyboard_0604e4ba80b241a78abc5f40947bdf0c.Children.add(ObjectAnimationUsingKeyFrames_b2059a57eb11401f87a31e388593e77d);


                    VisualState_ae4cc0d35758445fa280c25c8d4e8238.Storyboard = Storyboard_0604e4ba80b241a78abc5f40947bdf0c;


                    templateOwner_ControlTemplate_6c61f3dccfe14a2ebaa9332846c0ed0b.RegisterName("Pressed", VisualState_4d81e74e52084b3292c580d08996d5bd);
                    VisualState_4d81e74e52084b3292c580d08996d5bd.Name = "Pressed";
                    var Storyboard_762dd1873fcd41b69294f574db4e3601 = new Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard();
                    var ObjectAnimationUsingKeyFrames_33069e8de01b475485896ae000e8667a = new Bridge.global.Windows.UI.Xaml.Media.Animation.ObjectAnimationUsingKeyFrames();
                    Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard.SetTargetName(ObjectAnimationUsingKeyFrames_33069e8de01b475485896ae000e8667a, "InnerBorder");
                    var DiscreteObjectKeyFrame_431456709fc941368076eccc644c9165 = new Bridge.global.Windows.UI.Xaml.Media.Animation.DiscreteObjectKeyFrame();
                    DiscreteObjectKeyFrame_431456709fc941368076eccc644c9165.KeyTime = Bridge.global.Windows.UI.Xaml.Media.Animation.KeyTime.FromTimeSpan(new Bridge.global.System.TimeSpan(System.Int64(0)));
                    DiscreteObjectKeyFrame_431456709fc941368076eccc644c9165.Value = "#22000000";

                    ObjectAnimationUsingKeyFrames_33069e8de01b475485896ae000e8667a.KeyFrames.add(DiscreteObjectKeyFrame_431456709fc941368076eccc644c9165);


                    Storyboard_762dd1873fcd41b69294f574db4e3601.Children.add(ObjectAnimationUsingKeyFrames_33069e8de01b475485896ae000e8667a);


                    VisualState_4d81e74e52084b3292c580d08996d5bd.Storyboard = Storyboard_762dd1873fcd41b69294f574db4e3601;


                    templateOwner_ControlTemplate_6c61f3dccfe14a2ebaa9332846c0ed0b.RegisterName("Disabled", VisualState_270f28d71bef4ebaa3985432411bbec9);
                    VisualState_270f28d71bef4ebaa3985432411bbec9.Name = "Disabled";
                    var Storyboard_066a577c16db4b3e95360ad7b9a5a1fa = new Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard();
                    var ObjectAnimationUsingKeyFrames_8b8f396b23194966a53e21664dd21c21 = new Bridge.global.Windows.UI.Xaml.Media.Animation.ObjectAnimationUsingKeyFrames();
                    Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard.SetTargetName(ObjectAnimationUsingKeyFrames_8b8f396b23194966a53e21664dd21c21, "InnerBorder");
                    var DiscreteObjectKeyFrame_4807a29568634c7580764af211b0b974 = new Bridge.global.Windows.UI.Xaml.Media.Animation.DiscreteObjectKeyFrame();
                    DiscreteObjectKeyFrame_4807a29568634c7580764af211b0b974.KeyTime = Bridge.global.Windows.UI.Xaml.Media.Animation.KeyTime.FromTimeSpan(new Bridge.global.System.TimeSpan(System.Int64(0)));
                    DiscreteObjectKeyFrame_4807a29568634c7580764af211b0b974.Value = "#33FFFFFF";

                    ObjectAnimationUsingKeyFrames_8b8f396b23194966a53e21664dd21c21.KeyFrames.add(DiscreteObjectKeyFrame_4807a29568634c7580764af211b0b974);


                    Storyboard_066a577c16db4b3e95360ad7b9a5a1fa.Children.add(ObjectAnimationUsingKeyFrames_8b8f396b23194966a53e21664dd21c21);


                    VisualState_270f28d71bef4ebaa3985432411bbec9.Storyboard = Storyboard_066a577c16db4b3e95360ad7b9a5a1fa;


                    System.Array.add(VisualStateGroup_9afb9aa2451f4a609c745b211972c191.States, VisualState_a5e5e1801ad34d4abb6790f5f5e785ba, Object);
                    System.Array.add(VisualStateGroup_9afb9aa2451f4a609c745b211972c191.States, VisualState_ae4cc0d35758445fa280c25c8d4e8238, Object);
                    System.Array.add(VisualStateGroup_9afb9aa2451f4a609c745b211972c191.States, VisualState_4d81e74e52084b3292c580d08996d5bd, Object);
                    System.Array.add(VisualStateGroup_9afb9aa2451f4a609c745b211972c191.States, VisualState_270f28d71bef4ebaa3985432411bbec9, Object);


                    templateOwner_ControlTemplate_6c61f3dccfe14a2ebaa9332846c0ed0b.INTERNAL_GetVisualStateGroups().add(VisualStateGroup_9afb9aa2451f4a609c745b211972c191);

                    templateOwner_ControlTemplate_6c61f3dccfe14a2ebaa9332846c0ed0b.RegisterName("InnerBorder", Border_e3243ef889b74918a702bc390f0b2d2a);
                    Border_e3243ef889b74918a702bc390f0b2d2a.Name = "InnerBorder";
                    var StackPanel_beb34c2d142d49e8806b40370deb2f9a = new Bridge.global.Windows.UI.Xaml.Controls.StackPanel();
                    StackPanel_beb34c2d142d49e8806b40370deb2f9a.Orientation = Bridge.global.Windows.UI.Xaml.Controls.Orientation.Horizontal;
                    templateOwner_ControlTemplate_6c61f3dccfe14a2ebaa9332846c0ed0b.RegisterName("IconPresenter", ContentPresenter_ba420b13f8974e748c2b4ce5e3be83e3);
                    ContentPresenter_ba420b13f8974e748c2b4ce5e3be83e3.Name = "IconPresenter";
                    ContentPresenter_ba420b13f8974e748c2b4ce5e3be83e3.Margin = new Bridge.global.Windows.UI.Xaml.Thickness.$ctor2(0, 0, 10, 0);
                    var Binding_d0ce5c6cab0542069808096225e6647c = new Bridge.global.Windows.UI.Xaml.Data.Binding.ctor();
                    Binding_d0ce5c6cab0542069808096225e6647c.Path = new Bridge.global.Windows.UI.Xaml.PropertyPath.ctor("Icon");
                    var RelativeSource_82984d6398e14ccfbbef0894dee21bec = new Bridge.global.Windows.UI.Xaml.Data.RelativeSource.ctor();
                    RelativeSource_82984d6398e14ccfbbef0894dee21bec.Mode = Bridge.global.Windows.UI.Xaml.Data.RelativeSourceMode.TemplatedParent;

                    Binding_d0ce5c6cab0542069808096225e6647c.RelativeSource = RelativeSource_82984d6398e14ccfbbef0894dee21bec;


                    Binding_d0ce5c6cab0542069808096225e6647c.TemplateOwner = templateInstance_2e2dd1b3065d4c95850d6d8a39e64cf3;

                    var Binding_5ab0da2c8c554be0bc7c9aa2106609ce = new Bridge.global.Windows.UI.Xaml.Data.Binding.ctor();
                    Binding_5ab0da2c8c554be0bc7c9aa2106609ce.Path = new Bridge.global.Windows.UI.Xaml.PropertyPath.ctor("VerticalContentAlignment");
                    var RelativeSource_337c857c9509417787f260ffbd7ea0ef = new Bridge.global.Windows.UI.Xaml.Data.RelativeSource.ctor();
                    RelativeSource_337c857c9509417787f260ffbd7ea0ef.Mode = Bridge.global.Windows.UI.Xaml.Data.RelativeSourceMode.TemplatedParent;

                    Binding_5ab0da2c8c554be0bc7c9aa2106609ce.RelativeSource = RelativeSource_337c857c9509417787f260ffbd7ea0ef;


                    Binding_5ab0da2c8c554be0bc7c9aa2106609ce.TemplateOwner = templateInstance_2e2dd1b3065d4c95850d6d8a39e64cf3;

                    var Binding_76357668fbaa4649ab1281752c89e363 = new Bridge.global.Windows.UI.Xaml.Data.Binding.ctor();
                    Binding_76357668fbaa4649ab1281752c89e363.Path = new Bridge.global.Windows.UI.Xaml.PropertyPath.ctor("INTERNAL_IconVisibility");
                    var RelativeSource_8ec866aa764d4c2dae70c1650d0bfa3b = new Bridge.global.Windows.UI.Xaml.Data.RelativeSource.ctor();
                    RelativeSource_8ec866aa764d4c2dae70c1650d0bfa3b.Mode = Bridge.global.Windows.UI.Xaml.Data.RelativeSourceMode.TemplatedParent;

                    Binding_76357668fbaa4649ab1281752c89e363.RelativeSource = RelativeSource_8ec866aa764d4c2dae70c1650d0bfa3b;


                    Binding_76357668fbaa4649ab1281752c89e363.TemplateOwner = templateInstance_2e2dd1b3065d4c95850d6d8a39e64cf3;

                    var Binding_761bfaf66c9f4b84ae3240ee3638a458 = new Bridge.global.Windows.UI.Xaml.Data.Binding.ctor();
                    Binding_761bfaf66c9f4b84ae3240ee3638a458.Path = new Bridge.global.Windows.UI.Xaml.PropertyPath.ctor("ContentTemplate");
                    var RelativeSource_ad5d84ce01704bfcb3f76a5a72bd9ee5 = new Bridge.global.Windows.UI.Xaml.Data.RelativeSource.ctor();
                    RelativeSource_ad5d84ce01704bfcb3f76a5a72bd9ee5.Mode = Bridge.global.Windows.UI.Xaml.Data.RelativeSourceMode.TemplatedParent;

                    Binding_761bfaf66c9f4b84ae3240ee3638a458.RelativeSource = RelativeSource_ad5d84ce01704bfcb3f76a5a72bd9ee5;


                    Binding_761bfaf66c9f4b84ae3240ee3638a458.TemplateOwner = templateInstance_2e2dd1b3065d4c95850d6d8a39e64cf3;


                    templateOwner_ControlTemplate_6c61f3dccfe14a2ebaa9332846c0ed0b.RegisterName("ContentPresenter", ContentPresenter_56b3bf4dd9b74f1ab0af6485c6ddfe4e);
                    ContentPresenter_56b3bf4dd9b74f1ab0af6485c6ddfe4e.Name = "ContentPresenter";
                    var Binding_0495fcba67b8443bba5cf98e6677cdab = new Bridge.global.Windows.UI.Xaml.Data.Binding.ctor();
                    Binding_0495fcba67b8443bba5cf98e6677cdab.Path = new Bridge.global.Windows.UI.Xaml.PropertyPath.ctor("ContentTemplate");
                    var RelativeSource_8c5a92c83afb43a091dc1b0ffa676f11 = new Bridge.global.Windows.UI.Xaml.Data.RelativeSource.ctor();
                    RelativeSource_8c5a92c83afb43a091dc1b0ffa676f11.Mode = Bridge.global.Windows.UI.Xaml.Data.RelativeSourceMode.TemplatedParent;

                    Binding_0495fcba67b8443bba5cf98e6677cdab.RelativeSource = RelativeSource_8c5a92c83afb43a091dc1b0ffa676f11;


                    Binding_0495fcba67b8443bba5cf98e6677cdab.TemplateOwner = templateInstance_2e2dd1b3065d4c95850d6d8a39e64cf3;

                    var Binding_68da6504818a40bc94571bb0dc21666c = new Bridge.global.Windows.UI.Xaml.Data.Binding.ctor();
                    Binding_68da6504818a40bc94571bb0dc21666c.Path = new Bridge.global.Windows.UI.Xaml.PropertyPath.ctor("Content");
                    var RelativeSource_2625fa8f887546009f916c7b2978d36f = new Bridge.global.Windows.UI.Xaml.Data.RelativeSource.ctor();
                    RelativeSource_2625fa8f887546009f916c7b2978d36f.Mode = Bridge.global.Windows.UI.Xaml.Data.RelativeSourceMode.TemplatedParent;

                    Binding_68da6504818a40bc94571bb0dc21666c.RelativeSource = RelativeSource_2625fa8f887546009f916c7b2978d36f;


                    Binding_68da6504818a40bc94571bb0dc21666c.TemplateOwner = templateInstance_2e2dd1b3065d4c95850d6d8a39e64cf3;

                    var Binding_f6a16942ed6a4322b3fc0f7e7cd7b71b = new Bridge.global.Windows.UI.Xaml.Data.Binding.ctor();
                    Binding_f6a16942ed6a4322b3fc0f7e7cd7b71b.Path = new Bridge.global.Windows.UI.Xaml.PropertyPath.ctor("HorizontalContentAlignment");
                    var RelativeSource_852d312e14524054a67be4726b05d6db = new Bridge.global.Windows.UI.Xaml.Data.RelativeSource.ctor();
                    RelativeSource_852d312e14524054a67be4726b05d6db.Mode = Bridge.global.Windows.UI.Xaml.Data.RelativeSourceMode.TemplatedParent;

                    Binding_f6a16942ed6a4322b3fc0f7e7cd7b71b.RelativeSource = RelativeSource_852d312e14524054a67be4726b05d6db;


                    Binding_f6a16942ed6a4322b3fc0f7e7cd7b71b.TemplateOwner = templateInstance_2e2dd1b3065d4c95850d6d8a39e64cf3;

                    var Binding_c5b24e14a9ae4b47b02301a45d7ba3c7 = new Bridge.global.Windows.UI.Xaml.Data.Binding.ctor();
                    Binding_c5b24e14a9ae4b47b02301a45d7ba3c7.Path = new Bridge.global.Windows.UI.Xaml.PropertyPath.ctor("VerticalContentAlignment");
                    var RelativeSource_4686cbb67c4e4a7ead1d7b2cb9a594ec = new Bridge.global.Windows.UI.Xaml.Data.RelativeSource.ctor();
                    RelativeSource_4686cbb67c4e4a7ead1d7b2cb9a594ec.Mode = Bridge.global.Windows.UI.Xaml.Data.RelativeSourceMode.TemplatedParent;

                    Binding_c5b24e14a9ae4b47b02301a45d7ba3c7.RelativeSource = RelativeSource_4686cbb67c4e4a7ead1d7b2cb9a594ec;


                    Binding_c5b24e14a9ae4b47b02301a45d7ba3c7.TemplateOwner = templateInstance_2e2dd1b3065d4c95850d6d8a39e64cf3;


                    StackPanel_beb34c2d142d49e8806b40370deb2f9a.Children.add(ContentPresenter_ba420b13f8974e748c2b4ce5e3be83e3);
                    StackPanel_beb34c2d142d49e8806b40370deb2f9a.Children.add(ContentPresenter_56b3bf4dd9b74f1ab0af6485c6ddfe4e);

                    var Binding_20e5e96b10f54f149098a37c9762da0d = new Bridge.global.Windows.UI.Xaml.Data.Binding.ctor();
                    Binding_20e5e96b10f54f149098a37c9762da0d.Path = new Bridge.global.Windows.UI.Xaml.PropertyPath.ctor("Padding");
                    var RelativeSource_9f47f46b384841cbaf6f2386101f59e8 = new Bridge.global.Windows.UI.Xaml.Data.RelativeSource.ctor();
                    RelativeSource_9f47f46b384841cbaf6f2386101f59e8.Mode = Bridge.global.Windows.UI.Xaml.Data.RelativeSourceMode.TemplatedParent;

                    Binding_20e5e96b10f54f149098a37c9762da0d.RelativeSource = RelativeSource_9f47f46b384841cbaf6f2386101f59e8;


                    Binding_20e5e96b10f54f149098a37c9762da0d.TemplateOwner = templateInstance_2e2dd1b3065d4c95850d6d8a39e64cf3;


                    Border_e3243ef889b74918a702bc390f0b2d2a.Child = StackPanel_beb34c2d142d49e8806b40370deb2f9a;

                    var Binding_50b2c01bd3f34574b6e14e70d6444987 = new Bridge.global.Windows.UI.Xaml.Data.Binding.ctor();
                    Binding_50b2c01bd3f34574b6e14e70d6444987.Path = new Bridge.global.Windows.UI.Xaml.PropertyPath.ctor("Background");
                    var RelativeSource_742415504fe64cdba09818fbc8fb7cae = new Bridge.global.Windows.UI.Xaml.Data.RelativeSource.ctor();
                    RelativeSource_742415504fe64cdba09818fbc8fb7cae.Mode = Bridge.global.Windows.UI.Xaml.Data.RelativeSourceMode.TemplatedParent;

                    Binding_50b2c01bd3f34574b6e14e70d6444987.RelativeSource = RelativeSource_742415504fe64cdba09818fbc8fb7cae;


                    Binding_50b2c01bd3f34574b6e14e70d6444987.TemplateOwner = templateInstance_2e2dd1b3065d4c95850d6d8a39e64cf3;


                    Border_356f2d2e61b7432c98996b422e364014.Child = Border_e3243ef889b74918a702bc390f0b2d2a;

                    var Binding_031f12f291f845d1a00d850b46d00708 = new Bridge.global.Windows.UI.Xaml.Data.Binding.ctor();
                    Binding_031f12f291f845d1a00d850b46d00708.Path = new Bridge.global.Windows.UI.Xaml.PropertyPath.ctor("Background");
                    var RelativeSource_2e36744464af4119af65c7e12c8bd53f = new Bridge.global.Windows.UI.Xaml.Data.RelativeSource.ctor();
                    RelativeSource_2e36744464af4119af65c7e12c8bd53f.Mode = Bridge.global.Windows.UI.Xaml.Data.RelativeSourceMode.TemplatedParent;

                    Binding_031f12f291f845d1a00d850b46d00708.RelativeSource = RelativeSource_2e36744464af4119af65c7e12c8bd53f;


                    Binding_031f12f291f845d1a00d850b46d00708.TemplateOwner = templateInstance_2e2dd1b3065d4c95850d6d8a39e64cf3;

                    var Binding_1c49682602c142a8b0578d25660e7037 = new Bridge.global.Windows.UI.Xaml.Data.Binding.ctor();
                    Binding_1c49682602c142a8b0578d25660e7037.Path = new Bridge.global.Windows.UI.Xaml.PropertyPath.ctor("BorderBrush");
                    var RelativeSource_c955631b8ce6435daea6a3d53af24ee0 = new Bridge.global.Windows.UI.Xaml.Data.RelativeSource.ctor();
                    RelativeSource_c955631b8ce6435daea6a3d53af24ee0.Mode = Bridge.global.Windows.UI.Xaml.Data.RelativeSourceMode.TemplatedParent;

                    Binding_1c49682602c142a8b0578d25660e7037.RelativeSource = RelativeSource_c955631b8ce6435daea6a3d53af24ee0;


                    Binding_1c49682602c142a8b0578d25660e7037.TemplateOwner = templateInstance_2e2dd1b3065d4c95850d6d8a39e64cf3;

                    var Binding_87d5e0b1bfdb45af852392f218558c90 = new Bridge.global.Windows.UI.Xaml.Data.Binding.ctor();
                    Binding_87d5e0b1bfdb45af852392f218558c90.Path = new Bridge.global.Windows.UI.Xaml.PropertyPath.ctor("BorderThickness");
                    var RelativeSource_c546e5d973c74d1b8f7388850402ad36 = new Bridge.global.Windows.UI.Xaml.Data.RelativeSource.ctor();
                    RelativeSource_c546e5d973c74d1b8f7388850402ad36.Mode = Bridge.global.Windows.UI.Xaml.Data.RelativeSourceMode.TemplatedParent;

                    Binding_87d5e0b1bfdb45af852392f218558c90.RelativeSource = RelativeSource_c546e5d973c74d1b8f7388850402ad36;


                    Binding_87d5e0b1bfdb45af852392f218558c90.TemplateOwner = templateInstance_2e2dd1b3065d4c95850d6d8a39e64cf3;


                    Bridge.global.Windows.UI.Xaml.Data.BindingOperations.SetBinding(ContentPresenter_ba420b13f8974e748c2b4ce5e3be83e3, Bridge.global.Windows.UI.Xaml.Controls.ContentControl.ContentProperty, Binding_d0ce5c6cab0542069808096225e6647c);
                    Bridge.global.Windows.UI.Xaml.Data.BindingOperations.SetBinding(ContentPresenter_ba420b13f8974e748c2b4ce5e3be83e3, Bridge.global.Windows.UI.Xaml.FrameworkElement.VerticalAlignmentProperty, Binding_5ab0da2c8c554be0bc7c9aa2106609ce);
                    Bridge.global.Windows.UI.Xaml.Data.BindingOperations.SetBinding(ContentPresenter_ba420b13f8974e748c2b4ce5e3be83e3, Bridge.global.Windows.UI.Xaml.UIElement.VisibilityProperty, Binding_76357668fbaa4649ab1281752c89e363);
                    Bridge.global.Windows.UI.Xaml.Data.BindingOperations.SetBinding(ContentPresenter_ba420b13f8974e748c2b4ce5e3be83e3, Bridge.global.Windows.UI.Xaml.Controls.ContentControl.ContentTemplateProperty, Binding_761bfaf66c9f4b84ae3240ee3638a458);
                    Bridge.global.Windows.UI.Xaml.Data.BindingOperations.SetBinding(ContentPresenter_56b3bf4dd9b74f1ab0af6485c6ddfe4e, Bridge.global.Windows.UI.Xaml.Controls.ContentControl.ContentTemplateProperty, Binding_0495fcba67b8443bba5cf98e6677cdab);
                    Bridge.global.Windows.UI.Xaml.Data.BindingOperations.SetBinding(ContentPresenter_56b3bf4dd9b74f1ab0af6485c6ddfe4e, Bridge.global.Windows.UI.Xaml.Controls.ContentControl.ContentProperty, Binding_68da6504818a40bc94571bb0dc21666c);
                    Bridge.global.Windows.UI.Xaml.Data.BindingOperations.SetBinding(ContentPresenter_56b3bf4dd9b74f1ab0af6485c6ddfe4e, Bridge.global.Windows.UI.Xaml.FrameworkElement.HorizontalAlignmentProperty, Binding_f6a16942ed6a4322b3fc0f7e7cd7b71b);
                    Bridge.global.Windows.UI.Xaml.Data.BindingOperations.SetBinding(ContentPresenter_56b3bf4dd9b74f1ab0af6485c6ddfe4e, Bridge.global.Windows.UI.Xaml.FrameworkElement.VerticalAlignmentProperty, Binding_c5b24e14a9ae4b47b02301a45d7ba3c7);
                    Bridge.global.Windows.UI.Xaml.Data.BindingOperations.SetBinding(StackPanel_beb34c2d142d49e8806b40370deb2f9a, Bridge.global.Windows.UI.Xaml.FrameworkElement.MarginProperty, Binding_20e5e96b10f54f149098a37c9762da0d);
                    Bridge.global.Windows.UI.Xaml.Data.BindingOperations.SetBinding(Border_e3243ef889b74918a702bc390f0b2d2a, Bridge.global.Windows.UI.Xaml.Controls.Border.BackgroundProperty, Binding_50b2c01bd3f34574b6e14e70d6444987);
                    Bridge.global.Windows.UI.Xaml.Data.BindingOperations.SetBinding(Border_356f2d2e61b7432c98996b422e364014, Bridge.global.Windows.UI.Xaml.Controls.Border.BackgroundProperty, Binding_031f12f291f845d1a00d850b46d00708);
                    Bridge.global.Windows.UI.Xaml.Data.BindingOperations.SetBinding(Border_356f2d2e61b7432c98996b422e364014, Bridge.global.Windows.UI.Xaml.Controls.Border.BorderBrushProperty, Binding_1c49682602c142a8b0578d25660e7037);
                    Bridge.global.Windows.UI.Xaml.Data.BindingOperations.SetBinding(Border_356f2d2e61b7432c98996b422e364014, Bridge.global.Windows.UI.Xaml.Controls.Border.BorderThicknessProperty, Binding_87d5e0b1bfdb45af852392f218558c90);

                    Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard.SetTargetProperty(ObjectAnimationUsingKeyFrames_b2059a57eb11401f87a31e388593e77d, new Bridge.global.Windows.UI.Xaml.PropertyPath.$ctor1("Background", "Background", Bridge.fn.cacheBind(this, this.accessVisualStateProperty_58694013b8a54742a8876294c4fa761e), Bridge.fn.cacheBind(this, this.setVisualStateProperty_58694013b8a54742a8876294c4fa761e), Bridge.fn.cacheBind(this, this.setAnimationVisualStateProperty_58694013b8a54742a8876294c4fa761e), Bridge.fn.cacheBind(this, this.setLocalVisualStateProperty_58694013b8a54742a8876294c4fa761e), Bridge.fn.cacheBind(this, this.getVisualStateProperty_58694013b8a54742a8876294c4fa761e)));
                    Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard.SetTarget(ObjectAnimationUsingKeyFrames_b2059a57eb11401f87a31e388593e77d, Border_e3243ef889b74918a702bc390f0b2d2a);


                    Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard.SetTargetProperty(ObjectAnimationUsingKeyFrames_33069e8de01b475485896ae000e8667a, new Bridge.global.Windows.UI.Xaml.PropertyPath.$ctor1("Background", "Background", Bridge.fn.cacheBind(this, this.accessVisualStateProperty_7d5f557ccd1f430e9b2aab017d9766c1), Bridge.fn.cacheBind(this, this.setVisualStateProperty_7d5f557ccd1f430e9b2aab017d9766c1), Bridge.fn.cacheBind(this, this.setAnimationVisualStateProperty_7d5f557ccd1f430e9b2aab017d9766c1), Bridge.fn.cacheBind(this, this.setLocalVisualStateProperty_7d5f557ccd1f430e9b2aab017d9766c1), Bridge.fn.cacheBind(this, this.getVisualStateProperty_7d5f557ccd1f430e9b2aab017d9766c1)));
                    Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard.SetTarget(ObjectAnimationUsingKeyFrames_33069e8de01b475485896ae000e8667a, Border_e3243ef889b74918a702bc390f0b2d2a);


                    Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard.SetTargetProperty(ObjectAnimationUsingKeyFrames_8b8f396b23194966a53e21664dd21c21, new Bridge.global.Windows.UI.Xaml.PropertyPath.$ctor1("Background", "Background", Bridge.fn.cacheBind(this, this.accessVisualStateProperty_32d96703b7434397b8672281acfbe3b1), Bridge.fn.cacheBind(this, this.setVisualStateProperty_32d96703b7434397b8672281acfbe3b1), Bridge.fn.cacheBind(this, this.setAnimationVisualStateProperty_32d96703b7434397b8672281acfbe3b1), Bridge.fn.cacheBind(this, this.setLocalVisualStateProperty_32d96703b7434397b8672281acfbe3b1), Bridge.fn.cacheBind(this, this.getVisualStateProperty_32d96703b7434397b8672281acfbe3b1)));
                    Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard.SetTarget(ObjectAnimationUsingKeyFrames_8b8f396b23194966a53e21664dd21c21, Border_e3243ef889b74918a702bc390f0b2d2a);

                    templateInstance_2e2dd1b3065d4c95850d6d8a39e64cf3.TemplateContent = Border_356f2d2e61b7432c98996b422e364014;
                    return templateInstance_2e2dd1b3065d4c95850d6d8a39e64cf3;
                }));

                Setter_aabb875321eb4cda97067c47dd1a800f.Value = ControlTemplate_6c61f3dccfe14a2ebaa9332846c0ed0b;


                Style_a31d976f43ce40da94e520b9ebec8f87.Setters.add(Setter_681fdcc9824d4cb49307eb6193a377b7);
                Style_a31d976f43ce40da94e520b9ebec8f87.Setters.add(Setter_31be95160bff4369a7abf0d417eb689a);
                Style_a31d976f43ce40da94e520b9ebec8f87.Setters.add(Setter_e9002444d2524dd8b90a7a731b61fdbb);
                Style_a31d976f43ce40da94e520b9ebec8f87.Setters.add(Setter_50b2f8b84e2d44cbb4b9ff543a178379);
                Style_a31d976f43ce40da94e520b9ebec8f87.Setters.add(Setter_4b049b27b85542eab7b119afdf6dc902);
                Style_a31d976f43ce40da94e520b9ebec8f87.Setters.add(Setter_c1b8ce71bd8440c1ba8582c734400a4c);
                Style_a31d976f43ce40da94e520b9ebec8f87.Setters.add(Setter_7f852a1e5e684b55a435e9e3e1e773fa);
                Style_a31d976f43ce40da94e520b9ebec8f87.Setters.add(Setter_aabb875321eb4cda97067c47dd1a800f);


                this.setItem(Bridge.global.Windows.UI.Xaml.Controls.MenuItem, Style_a31d976f43ce40da94e520b9ebec8f87);






            },
            accessVisualStateProperty_58694013b8a54742a8876294c4fa761e: function (rootTargetObjectInstance) {
                return new (Bridge.GeneratorEnumerable$1(System.Tuple$3(Windows.UI.Xaml.DependencyObject,Windows.UI.Xaml.DependencyProperty,System.Nullable$1(System.Int32))))(Bridge.fn.bind(this, function (rootTargetObjectInstance) {
                    var $step = 0,
                        $jumpFromFinally,
                        $returnValue,
                        $async_e;

                    var $enumerator = new (Bridge.GeneratorEnumerator$1(System.Tuple$3(Windows.UI.Xaml.DependencyObject,Windows.UI.Xaml.DependencyProperty,System.Nullable$1(System.Int32))))(Bridge.fn.bind(this, function () {
                        try {
                            for (;;) {
                                switch ($step) {
                                    case 0: {
                                        return false;
                                    }
                                    default: {
                                        return false;
                                    }
                                }
                            }
                        } catch($async_e1) {
                            $async_e = System.Exception.create($async_e1);
                            throw $async_e;
                        }
                    }));
                    return $enumerator;
                }, arguments));
            },
            setVisualStateProperty_58694013b8a54742a8876294c4fa761e: function (finalTargetInstance, value) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Background").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "BackgroundProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                (finalTargetInstance).SetVisualStateValue(property, value);
            },
            setAnimationVisualStateProperty_58694013b8a54742a8876294c4fa761e: function (finalTargetInstance, value) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Background").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "BackgroundProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                (finalTargetInstance).SetAnimationValue(property, value);
            },
            setLocalVisualStateProperty_58694013b8a54742a8876294c4fa761e: function (finalTargetInstance, value) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Background").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "BackgroundProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                (finalTargetInstance).SetLocalValue(property, value);
            },
            getVisualStateProperty_58694013b8a54742a8876294c4fa761e: function (finalTargetInstance) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Background").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "BackgroundProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                return finalTargetInstance.GetVisualStateValue(property);
            },
            accessVisualStateProperty_7d5f557ccd1f430e9b2aab017d9766c1: function (rootTargetObjectInstance) {
                return new (Bridge.GeneratorEnumerable$1(System.Tuple$3(Windows.UI.Xaml.DependencyObject,Windows.UI.Xaml.DependencyProperty,System.Nullable$1(System.Int32))))(Bridge.fn.bind(this, function (rootTargetObjectInstance) {
                    var $step = 0,
                        $jumpFromFinally,
                        $returnValue,
                        $async_e;

                    var $enumerator = new (Bridge.GeneratorEnumerator$1(System.Tuple$3(Windows.UI.Xaml.DependencyObject,Windows.UI.Xaml.DependencyProperty,System.Nullable$1(System.Int32))))(Bridge.fn.bind(this, function () {
                        try {
                            for (;;) {
                                switch ($step) {
                                    case 0: {
                                        return false;
                                    }
                                    default: {
                                        return false;
                                    }
                                }
                            }
                        } catch($async_e1) {
                            $async_e = System.Exception.create($async_e1);
                            throw $async_e;
                        }
                    }));
                    return $enumerator;
                }, arguments));
            },
            setVisualStateProperty_7d5f557ccd1f430e9b2aab017d9766c1: function (finalTargetInstance, value) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Background").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "BackgroundProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                (finalTargetInstance).SetVisualStateValue(property, value);
            },
            setAnimationVisualStateProperty_7d5f557ccd1f430e9b2aab017d9766c1: function (finalTargetInstance, value) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Background").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "BackgroundProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                (finalTargetInstance).SetAnimationValue(property, value);
            },
            setLocalVisualStateProperty_7d5f557ccd1f430e9b2aab017d9766c1: function (finalTargetInstance, value) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Background").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "BackgroundProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                (finalTargetInstance).SetLocalValue(property, value);
            },
            getVisualStateProperty_7d5f557ccd1f430e9b2aab017d9766c1: function (finalTargetInstance) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Background").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "BackgroundProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                return finalTargetInstance.GetVisualStateValue(property);
            },
            accessVisualStateProperty_32d96703b7434397b8672281acfbe3b1: function (rootTargetObjectInstance) {
                return new (Bridge.GeneratorEnumerable$1(System.Tuple$3(Windows.UI.Xaml.DependencyObject,Windows.UI.Xaml.DependencyProperty,System.Nullable$1(System.Int32))))(Bridge.fn.bind(this, function (rootTargetObjectInstance) {
                    var $step = 0,
                        $jumpFromFinally,
                        $returnValue,
                        $async_e;

                    var $enumerator = new (Bridge.GeneratorEnumerator$1(System.Tuple$3(Windows.UI.Xaml.DependencyObject,Windows.UI.Xaml.DependencyProperty,System.Nullable$1(System.Int32))))(Bridge.fn.bind(this, function () {
                        try {
                            for (;;) {
                                switch ($step) {
                                    case 0: {
                                        return false;
                                    }
                                    default: {
                                        return false;
                                    }
                                }
                            }
                        } catch($async_e1) {
                            $async_e = System.Exception.create($async_e1);
                            throw $async_e;
                        }
                    }));
                    return $enumerator;
                }, arguments));
            },
            setVisualStateProperty_32d96703b7434397b8672281acfbe3b1: function (finalTargetInstance, value) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Background").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "BackgroundProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                (finalTargetInstance).SetVisualStateValue(property, value);
            },
            setAnimationVisualStateProperty_32d96703b7434397b8672281acfbe3b1: function (finalTargetInstance, value) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Background").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "BackgroundProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                (finalTargetInstance).SetAnimationValue(property, value);
            },
            setLocalVisualStateProperty_32d96703b7434397b8672281acfbe3b1: function (finalTargetInstance, value) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Background").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "BackgroundProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                (finalTargetInstance).SetLocalValue(property, value);
            },
            getVisualStateProperty_32d96703b7434397b8672281acfbe3b1: function (finalTargetInstance) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Background").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "BackgroundProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                return finalTargetInstance.GetVisualStateValue(property);
            }
        }
    });

    Bridge.define("___CSHTML5__Component__Defaultstyles__INTERNAL_Defaultnumericupdownstyle__Xaml", {
        inherits: [Bridge.global.Windows.UI.Xaml.ResourceDictionary],
        fields: {
            _contentLoaded: false
        },
        ctors: {
            ctor: function () {
                this.$initialize();
                Bridge.global.Windows.UI.Xaml.ResourceDictionary.ctor.call(this);
                this.InitializeComponent();

                if (Bridge.is(this, Windows.UI.Xaml.UIElement)) {
                    Bridge.cast(this, Windows.UI.Xaml.UIElement).XamlSourcePath = "CSHTML5\\DefaultStyles\\INTERNAL_DefaultNumericUpDownStyle.xaml";
                }
            }
        },
        methods: {
            InitializeComponent: function () {
                if (this._contentLoaded) {
                    return;
                }
                this._contentLoaded = true;


                if (Bridge.is(this, Windows.UI.Xaml.UIElement)) {
                    Bridge.cast(this, Windows.UI.Xaml.UIElement).XamlSourcePath = "CSHTML5\\DefaultStyles\\INTERNAL_DefaultNumericUpDownStyle.xaml";
                }



                var Style_aedee4f6fc634e1d9f1f384a98dba26c = new Bridge.global.Windows.UI.Xaml.Style.ctor();
                Style_aedee4f6fc634e1d9f1f384a98dba26c.TargetType = Bridge.global.Windows.UI.Xaml.Controls.NumericUpDown;
                var Setter_3ede9bc5c173412db974718e02367a20 = new Bridge.global.Windows.UI.Xaml.Setter.ctor();
                Setter_3ede9bc5c173412db974718e02367a20.Property = Bridge.global.Windows.UI.Xaml.Controls.Control.TemplateProperty;
                var ControlTemplate_4379dc6ffb4d451f82b09f4eebba4581 = new Bridge.global.Windows.UI.Xaml.Controls.ControlTemplate();
                ControlTemplate_4379dc6ffb4d451f82b09f4eebba4581.TargetType = Bridge.global.Windows.UI.Xaml.Controls.NumericUpDown;
                ControlTemplate_4379dc6ffb4d451f82b09f4eebba4581.SetMethodToInstantiateFrameworkTemplate(function (templateOwner_ControlTemplate_4379dc6ffb4d451f82b09f4eebba4581) {
                    var $t;
                    var templateInstance_2666bcb8193347f0beb30936cc86e5ba = new Bridge.global.Windows.UI.Xaml.TemplateInstance();
                    templateInstance_2666bcb8193347f0beb30936cc86e5ba.TemplateOwner = templateOwner_ControlTemplate_4379dc6ffb4d451f82b09f4eebba4581;
                    var Rectangle_f99d9b6824f74184802f45ff924576b5 = new Bridge.global.Windows.UI.Xaml.Shapes.Rectangle();
                    var UpDownTextBox_c809c594964d4c518c4d10ee3f002cad = new Bridge.global.Windows.UI.Xaml.Controls.Primitives.UpDownTextBox();
                    var RepeatButton_74c3e65675de462d9106360d7b1e8bc0 = new Bridge.global.Windows.UI.Xaml.Controls.Primitives.RepeatButton();
                    var RepeatButton_a045d7335aba443c969cbe370ffe0610 = new Bridge.global.Windows.UI.Xaml.Controls.Primitives.RepeatButton();
                    var Border_0ee801ff809d4019bfdce331dc95434a = new Bridge.global.Windows.UI.Xaml.Controls.Border();
                    Border_0ee801ff809d4019bfdce331dc95434a.VerticalAlignment = Bridge.global.Windows.UI.Xaml.VerticalAlignment.Top;
                    var Grid_a9e8ba44a9764b878ddf05a34118db30 = new Bridge.global.Windows.UI.Xaml.Controls.Grid();
                    var ColumnDefinition_dc3c675b4fd04fe08315ffae1a1e9b96 = new Bridge.global.Windows.UI.Xaml.Controls.ColumnDefinition();

                    var ColumnDefinition_42071bcf51744f25924ff61077b0566c = new Bridge.global.Windows.UI.Xaml.Controls.ColumnDefinition();
                    ColumnDefinition_42071bcf51744f25924ff61077b0566c.Width = new Bridge.global.Windows.UI.Xaml.GridLength.$ctor2(1.0, Bridge.global.Windows.UI.Xaml.GridUnitType.Auto);

                    var ColumnDefinition_b55d61bdab2a4489a98ac64724f6934e = new Bridge.global.Windows.UI.Xaml.Controls.ColumnDefinition();
                    ColumnDefinition_b55d61bdab2a4489a98ac64724f6934e.Width = new Bridge.global.Windows.UI.Xaml.GridLength.$ctor2(1.0, Bridge.global.Windows.UI.Xaml.GridUnitType.Auto);

                    Grid_a9e8ba44a9764b878ddf05a34118db30.ColumnDefinitions.add(ColumnDefinition_dc3c675b4fd04fe08315ffae1a1e9b96);
                    Grid_a9e8ba44a9764b878ddf05a34118db30.ColumnDefinitions.add(ColumnDefinition_42071bcf51744f25924ff61077b0566c);
                    Grid_a9e8ba44a9764b878ddf05a34118db30.ColumnDefinitions.add(ColumnDefinition_b55d61bdab2a4489a98ac64724f6934e);

                    var Grid_e70a7e1b69944ce5b0b846955906a189 = new Bridge.global.Windows.UI.Xaml.Controls.Grid();
                    Bridge.global.Windows.UI.Xaml.Controls.Grid.SetColumn(Grid_e70a7e1b69944ce5b0b846955906a189, 0);
                    templateOwner_ControlTemplate_4379dc6ffb4d451f82b09f4eebba4581.RegisterName("PART_ValueBar", Rectangle_f99d9b6824f74184802f45ff924576b5);
                    Rectangle_f99d9b6824f74184802f45ff924576b5.Name = "PART_ValueBar";
                    Rectangle_f99d9b6824f74184802f45ff924576b5.IsHitTestVisible = false;
                    Rectangle_f99d9b6824f74184802f45ff924576b5.Fill = new Bridge.global.Windows.UI.Xaml.Media.SolidColorBrush.$ctor1(($t = new Bridge.global.Windows.UI.Color(), $t.A = 255, $t.R = 255, $t.G = 255, $t.B = 255, $t));

                    templateOwner_ControlTemplate_4379dc6ffb4d451f82b09f4eebba4581.RegisterName("PART_ValueTextBox", UpDownTextBox_c809c594964d4c518c4d10ee3f002cad);
                    UpDownTextBox_c809c594964d4c518c4d10ee3f002cad.Name = "PART_ValueTextBox";
                    UpDownTextBox_c809c594964d4c518c4d10ee3f002cad.HorizontalAlignment = Bridge.global.Windows.UI.Xaml.HorizontalAlignment.Stretch;
                    var Binding_9bd182672f61488d8898e093595aeca4 = new Bridge.global.Windows.UI.Xaml.Data.Binding.ctor();
                    Binding_9bd182672f61488d8898e093595aeca4.Path = new Bridge.global.Windows.UI.Xaml.PropertyPath.ctor("FontFamily");
                    var RelativeSource_c48a62abde28452e901173620e71bc26 = new Bridge.global.Windows.UI.Xaml.Data.RelativeSource.ctor();
                    RelativeSource_c48a62abde28452e901173620e71bc26.Mode = Bridge.global.Windows.UI.Xaml.Data.RelativeSourceMode.TemplatedParent;

                    Binding_9bd182672f61488d8898e093595aeca4.RelativeSource = RelativeSource_c48a62abde28452e901173620e71bc26;


                    Binding_9bd182672f61488d8898e093595aeca4.TemplateOwner = templateInstance_2666bcb8193347f0beb30936cc86e5ba;

                    var Binding_d532ec8efbde458eadd8497aa5579c9a = new Bridge.global.Windows.UI.Xaml.Data.Binding.ctor();
                    Binding_d532ec8efbde458eadd8497aa5579c9a.Path = new Bridge.global.Windows.UI.Xaml.PropertyPath.ctor("FontSize");
                    var RelativeSource_2cd1faee41844423b4087684a2edcea6 = new Bridge.global.Windows.UI.Xaml.Data.RelativeSource.ctor();
                    RelativeSource_2cd1faee41844423b4087684a2edcea6.Mode = Bridge.global.Windows.UI.Xaml.Data.RelativeSourceMode.TemplatedParent;

                    Binding_d532ec8efbde458eadd8497aa5579c9a.RelativeSource = RelativeSource_2cd1faee41844423b4087684a2edcea6;


                    Binding_d532ec8efbde458eadd8497aa5579c9a.TemplateOwner = templateInstance_2666bcb8193347f0beb30936cc86e5ba;

                    var Binding_4fbcb50bfa434547a69f34e80e6b1661 = new Bridge.global.Windows.UI.Xaml.Data.Binding.ctor();
                    Binding_4fbcb50bfa434547a69f34e80e6b1661.Path = new Bridge.global.Windows.UI.Xaml.PropertyPath.ctor("Foreground");
                    var RelativeSource_70b2a6d173b54204890de752531c3dca = new Bridge.global.Windows.UI.Xaml.Data.RelativeSource.ctor();
                    RelativeSource_70b2a6d173b54204890de752531c3dca.Mode = Bridge.global.Windows.UI.Xaml.Data.RelativeSourceMode.TemplatedParent;

                    Binding_4fbcb50bfa434547a69f34e80e6b1661.RelativeSource = RelativeSource_70b2a6d173b54204890de752531c3dca;


                    Binding_4fbcb50bfa434547a69f34e80e6b1661.TemplateOwner = templateInstance_2666bcb8193347f0beb30936cc86e5ba;


                    Grid_e70a7e1b69944ce5b0b846955906a189.Children.add(Rectangle_f99d9b6824f74184802f45ff924576b5);
                    Grid_e70a7e1b69944ce5b0b846955906a189.Children.add(UpDownTextBox_c809c594964d4c518c4d10ee3f002cad);


                    Bridge.global.Windows.UI.Xaml.Controls.Grid.SetColumn(RepeatButton_74c3e65675de462d9106360d7b1e8bc0, 1);
                    templateOwner_ControlTemplate_4379dc6ffb4d451f82b09f4eebba4581.RegisterName("PART_DecrementButton", RepeatButton_74c3e65675de462d9106360d7b1e8bc0);
                    RepeatButton_74c3e65675de462d9106360d7b1e8bc0.Name = "PART_DecrementButton";
                    RepeatButton_74c3e65675de462d9106360d7b1e8bc0.Margin = new Bridge.global.Windows.UI.Xaml.Thickness.$ctor2(4, 0, 0, 0);
                    RepeatButton_74c3e65675de462d9106360d7b1e8bc0.Content = "\u2796";

                    Bridge.global.Windows.UI.Xaml.Controls.Grid.SetColumn(RepeatButton_a045d7335aba443c969cbe370ffe0610, 2);
                    templateOwner_ControlTemplate_4379dc6ffb4d451f82b09f4eebba4581.RegisterName("PART_IncrementButton", RepeatButton_a045d7335aba443c969cbe370ffe0610);
                    RepeatButton_a045d7335aba443c969cbe370ffe0610.Name = "PART_IncrementButton";
                    RepeatButton_a045d7335aba443c969cbe370ffe0610.Margin = new Bridge.global.Windows.UI.Xaml.Thickness.$ctor2(3, 0, 0, 0);
                    RepeatButton_a045d7335aba443c969cbe370ffe0610.Content = "\u2795";

                    Grid_a9e8ba44a9764b878ddf05a34118db30.Children.add(Grid_e70a7e1b69944ce5b0b846955906a189);
                    Grid_a9e8ba44a9764b878ddf05a34118db30.Children.add(RepeatButton_74c3e65675de462d9106360d7b1e8bc0);
                    Grid_a9e8ba44a9764b878ddf05a34118db30.Children.add(RepeatButton_a045d7335aba443c969cbe370ffe0610);


                    Border_0ee801ff809d4019bfdce331dc95434a.Child = Grid_a9e8ba44a9764b878ddf05a34118db30;

                    var Binding_34c2da6dcde5444281cbffd805785e18 = new Bridge.global.Windows.UI.Xaml.Data.Binding.ctor();
                    Binding_34c2da6dcde5444281cbffd805785e18.Path = new Bridge.global.Windows.UI.Xaml.PropertyPath.ctor("Background");
                    var RelativeSource_37d1bbed0abb4ec99b85eae89b2a5ee8 = new Bridge.global.Windows.UI.Xaml.Data.RelativeSource.ctor();
                    RelativeSource_37d1bbed0abb4ec99b85eae89b2a5ee8.Mode = Bridge.global.Windows.UI.Xaml.Data.RelativeSourceMode.TemplatedParent;

                    Binding_34c2da6dcde5444281cbffd805785e18.RelativeSource = RelativeSource_37d1bbed0abb4ec99b85eae89b2a5ee8;


                    Binding_34c2da6dcde5444281cbffd805785e18.TemplateOwner = templateInstance_2666bcb8193347f0beb30936cc86e5ba;

                    var Binding_9a23483b3b274507837a490a9e181f2a = new Bridge.global.Windows.UI.Xaml.Data.Binding.ctor();
                    Binding_9a23483b3b274507837a490a9e181f2a.Path = new Bridge.global.Windows.UI.Xaml.PropertyPath.ctor("BorderBrush");
                    var RelativeSource_76d752ff219c49e2987581bef86e9918 = new Bridge.global.Windows.UI.Xaml.Data.RelativeSource.ctor();
                    RelativeSource_76d752ff219c49e2987581bef86e9918.Mode = Bridge.global.Windows.UI.Xaml.Data.RelativeSourceMode.TemplatedParent;

                    Binding_9a23483b3b274507837a490a9e181f2a.RelativeSource = RelativeSource_76d752ff219c49e2987581bef86e9918;


                    Binding_9a23483b3b274507837a490a9e181f2a.TemplateOwner = templateInstance_2666bcb8193347f0beb30936cc86e5ba;

                    var Binding_38d272d2d893446084ae2c20bb16ac86 = new Bridge.global.Windows.UI.Xaml.Data.Binding.ctor();
                    Binding_38d272d2d893446084ae2c20bb16ac86.Path = new Bridge.global.Windows.UI.Xaml.PropertyPath.ctor("BorderThickness");
                    var RelativeSource_f1a2ead006bc497d83f046d3ad6dead9 = new Bridge.global.Windows.UI.Xaml.Data.RelativeSource.ctor();
                    RelativeSource_f1a2ead006bc497d83f046d3ad6dead9.Mode = Bridge.global.Windows.UI.Xaml.Data.RelativeSourceMode.TemplatedParent;

                    Binding_38d272d2d893446084ae2c20bb16ac86.RelativeSource = RelativeSource_f1a2ead006bc497d83f046d3ad6dead9;


                    Binding_38d272d2d893446084ae2c20bb16ac86.TemplateOwner = templateInstance_2666bcb8193347f0beb30936cc86e5ba;


                    Bridge.global.Windows.UI.Xaml.Data.BindingOperations.SetBinding(UpDownTextBox_c809c594964d4c518c4d10ee3f002cad, Bridge.global.Windows.UI.Xaml.Controls.Control.FontFamilyProperty, Binding_9bd182672f61488d8898e093595aeca4);
                    Bridge.global.Windows.UI.Xaml.Data.BindingOperations.SetBinding(UpDownTextBox_c809c594964d4c518c4d10ee3f002cad, Bridge.global.Windows.UI.Xaml.Controls.Control.FontSizeProperty, Binding_d532ec8efbde458eadd8497aa5579c9a);
                    Bridge.global.Windows.UI.Xaml.Data.BindingOperations.SetBinding(UpDownTextBox_c809c594964d4c518c4d10ee3f002cad, Bridge.global.Windows.UI.Xaml.Controls.Control.ForegroundProperty, Binding_4fbcb50bfa434547a69f34e80e6b1661);
                    Bridge.global.Windows.UI.Xaml.Data.BindingOperations.SetBinding(Border_0ee801ff809d4019bfdce331dc95434a, Bridge.global.Windows.UI.Xaml.Controls.Border.BackgroundProperty, Binding_34c2da6dcde5444281cbffd805785e18);
                    Bridge.global.Windows.UI.Xaml.Data.BindingOperations.SetBinding(Border_0ee801ff809d4019bfdce331dc95434a, Bridge.global.Windows.UI.Xaml.Controls.Border.BorderBrushProperty, Binding_9a23483b3b274507837a490a9e181f2a);
                    Bridge.global.Windows.UI.Xaml.Data.BindingOperations.SetBinding(Border_0ee801ff809d4019bfdce331dc95434a, Bridge.global.Windows.UI.Xaml.Controls.Border.BorderThicknessProperty, Binding_38d272d2d893446084ae2c20bb16ac86);

                    templateInstance_2666bcb8193347f0beb30936cc86e5ba.TemplateContent = Border_0ee801ff809d4019bfdce331dc95434a;
                    return templateInstance_2666bcb8193347f0beb30936cc86e5ba;
                });

                Setter_3ede9bc5c173412db974718e02367a20.Value = ControlTemplate_4379dc6ffb4d451f82b09f4eebba4581;


                Style_aedee4f6fc634e1d9f1f384a98dba26c.Setters.add(Setter_3ede9bc5c173412db974718e02367a20);


                this.setItem(Bridge.global.Windows.UI.Xaml.Controls.NumericUpDown, Style_aedee4f6fc634e1d9f1f384a98dba26c);






            }
        }
    });

    Bridge.define("___CSHTML5__Component__Defaultstyles__INTERNAL_Defaultrepeatbuttonstyle__Xaml", {
        inherits: [Bridge.global.Windows.UI.Xaml.ResourceDictionary],
        fields: {
            _contentLoaded: false
        },
        ctors: {
            ctor: function () {
                this.$initialize();
                Bridge.global.Windows.UI.Xaml.ResourceDictionary.ctor.call(this);
                this.InitializeComponent();

                if (Bridge.is(this, Windows.UI.Xaml.UIElement)) {
                    Bridge.cast(this, Windows.UI.Xaml.UIElement).XamlSourcePath = "CSHTML5\\DefaultStyles\\INTERNAL_DefaultRepeatButtonStyle.xaml";
                }
            }
        },
        methods: {
            InitializeComponent: function () {
                var $t;
                if (this._contentLoaded) {
                    return;
                }
                this._contentLoaded = true;


                if (Bridge.is(this, Windows.UI.Xaml.UIElement)) {
                    Bridge.cast(this, Windows.UI.Xaml.UIElement).XamlSourcePath = "CSHTML5\\DefaultStyles\\INTERNAL_DefaultRepeatButtonStyle.xaml";
                }



                var Style_5fba16df9cf44216833b02d461e665de = new Bridge.global.Windows.UI.Xaml.Style.ctor();
                Style_5fba16df9cf44216833b02d461e665de.TargetType = Bridge.global.Windows.UI.Xaml.Controls.Primitives.RepeatButton;
                var Setter_8c806b9891c54c1dbc9f8e08b14645b5 = new Bridge.global.Windows.UI.Xaml.Setter.ctor();
                Setter_8c806b9891c54c1dbc9f8e08b14645b5.Property = Bridge.global.Windows.UI.Xaml.Controls.Control.BackgroundProperty;
                Setter_8c806b9891c54c1dbc9f8e08b14645b5.Value = new Bridge.global.Windows.UI.Xaml.Media.SolidColorBrush.$ctor1(($t = new Bridge.global.Windows.UI.Color(), $t.A = 255, $t.R = 226, $t.G = 226, $t.B = 226, $t));

                var Setter_da1b61cfa19d40e39d3a4bb23fb6d5fc = new Bridge.global.Windows.UI.Xaml.Setter.ctor();
                Setter_da1b61cfa19d40e39d3a4bb23fb6d5fc.Property = Bridge.global.Windows.UI.Xaml.Controls.Control.ForegroundProperty;
                Setter_da1b61cfa19d40e39d3a4bb23fb6d5fc.Value = new Bridge.global.Windows.UI.Xaml.Media.SolidColorBrush.$ctor1(($t = new Bridge.global.Windows.UI.Color(), $t.A = 255, $t.R = 0, $t.G = 0, $t.B = 0, $t));

                var Setter_839d7b5b20bd4490b4fdcd4b45249f74 = new Bridge.global.Windows.UI.Xaml.Setter.ctor();
                Setter_839d7b5b20bd4490b4fdcd4b45249f74.Property = Bridge.global.Windows.UI.Xaml.Controls.Control.BorderThicknessProperty;
                Setter_839d7b5b20bd4490b4fdcd4b45249f74.Value = new Bridge.global.Windows.UI.Xaml.Thickness.$ctor1(0).$clone();

                var Setter_6988e9fc910743e3a54f772cf4751100 = new Bridge.global.Windows.UI.Xaml.Setter.ctor();
                Setter_6988e9fc910743e3a54f772cf4751100.Property = Bridge.global.Windows.UI.Xaml.Controls.Control.PaddingProperty;
                Setter_6988e9fc910743e3a54f772cf4751100.Value = new Bridge.global.Windows.UI.Xaml.Thickness.$ctor1(3).$clone();

                var Setter_ae62fcad7b2b4a0ab1a0621c0c7e1a00 = new Bridge.global.Windows.UI.Xaml.Setter.ctor();
                Setter_ae62fcad7b2b4a0ab1a0621c0c7e1a00.Property = Bridge.global.Windows.UI.Xaml.FrameworkElement.CursorProperty;
                Setter_ae62fcad7b2b4a0ab1a0621c0c7e1a00.Value = Bridge.global.System.Windows.Input.Cursors.Hand;

                var Setter_35ad014628614a6fbd52a53cdf97290d = new Bridge.global.Windows.UI.Xaml.Setter.ctor();
                Setter_35ad014628614a6fbd52a53cdf97290d.Property = Bridge.global.Windows.UI.Xaml.Controls.Control.HorizontalContentAlignmentProperty;
                Setter_35ad014628614a6fbd52a53cdf97290d.Value = Bridge.box(Bridge.global.Windows.UI.Xaml.HorizontalAlignment.Center, Windows.UI.Xaml.HorizontalAlignment, System.Enum.toStringFn(Windows.UI.Xaml.HorizontalAlignment));

                var Setter_c928309aee7f4a86809462b668e6bdca = new Bridge.global.Windows.UI.Xaml.Setter.ctor();
                Setter_c928309aee7f4a86809462b668e6bdca.Property = Bridge.global.Windows.UI.Xaml.Controls.Control.VerticalContentAlignmentProperty;
                Setter_c928309aee7f4a86809462b668e6bdca.Value = Bridge.box(Bridge.global.Windows.UI.Xaml.VerticalAlignment.Center, Windows.UI.Xaml.VerticalAlignment, System.Enum.toStringFn(Windows.UI.Xaml.VerticalAlignment));

                var Setter_32ce28170c914d5cb56b135320a0e97c = new Bridge.global.Windows.UI.Xaml.Setter.ctor();
                Setter_32ce28170c914d5cb56b135320a0e97c.Property = Bridge.global.Windows.UI.Xaml.Controls.Control.TemplateProperty;
                var ControlTemplate_438ea801e3104db1b5e2ac151a376f3e = new Bridge.global.Windows.UI.Xaml.Controls.ControlTemplate();
                ControlTemplate_438ea801e3104db1b5e2ac151a376f3e.TargetType = Bridge.global.Windows.UI.Xaml.Controls.Primitives.RepeatButton;
                ControlTemplate_438ea801e3104db1b5e2ac151a376f3e.SetMethodToInstantiateFrameworkTemplate(Bridge.fn.bind(this, function (templateOwner_ControlTemplate_438ea801e3104db1b5e2ac151a376f3e) {
                    var templateInstance_35b70b30834241589105b16974c38832 = new Bridge.global.Windows.UI.Xaml.TemplateInstance();
                    templateInstance_35b70b30834241589105b16974c38832.TemplateOwner = templateOwner_ControlTemplate_438ea801e3104db1b5e2ac151a376f3e;
                    var VisualState_3aef25f7e13144aab1d4758aa85d7a4e = new Bridge.global.Windows.UI.Xaml.VisualState();
                    var VisualState_09ae383480aa423eaa3052e380cf9423 = new Bridge.global.Windows.UI.Xaml.VisualState();
                    var VisualState_365b034cb4654db1ab676695fcb32209 = new Bridge.global.Windows.UI.Xaml.VisualState();
                    var VisualState_5a964f4113b34c5f8dd932c790fbcc95 = new Bridge.global.Windows.UI.Xaml.VisualState();
                    var VisualStateGroup_dd3e49f01b9a45899886f8c892b5bdfb = new Bridge.global.Windows.UI.Xaml.VisualStateGroup();
                    var ContentPresenter_abde627c733d4559a499bbb2f9fe5bfb = new Bridge.global.Windows.UI.Xaml.Controls.ContentPresenter();
                    var Border_967f74e52304472393daedbfbfb1118a = new Bridge.global.Windows.UI.Xaml.Controls.Border();
                    var Border_b86c19a3e06f4afc86e3b5592eea67f6 = new Bridge.global.Windows.UI.Xaml.Controls.Border();
                    templateOwner_ControlTemplate_438ea801e3104db1b5e2ac151a376f3e.RegisterName("OuterBorder", Border_b86c19a3e06f4afc86e3b5592eea67f6);
                    Border_b86c19a3e06f4afc86e3b5592eea67f6.Name = "OuterBorder";
                    templateOwner_ControlTemplate_438ea801e3104db1b5e2ac151a376f3e.RegisterName("CommonStates", VisualStateGroup_dd3e49f01b9a45899886f8c892b5bdfb);
                    VisualStateGroup_dd3e49f01b9a45899886f8c892b5bdfb.Name = "CommonStates";
                    templateOwner_ControlTemplate_438ea801e3104db1b5e2ac151a376f3e.RegisterName("Normal", VisualState_3aef25f7e13144aab1d4758aa85d7a4e);
                    VisualState_3aef25f7e13144aab1d4758aa85d7a4e.Name = "Normal";

                    templateOwner_ControlTemplate_438ea801e3104db1b5e2ac151a376f3e.RegisterName("PointerOver", VisualState_09ae383480aa423eaa3052e380cf9423);
                    VisualState_09ae383480aa423eaa3052e380cf9423.Name = "PointerOver";
                    var Storyboard_c71b5f75e2194357a9430ccbe7b7eb9a = new Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard();
                    var ObjectAnimationUsingKeyFrames_834707980bfe46718d328b6bdf002712 = new Bridge.global.Windows.UI.Xaml.Media.Animation.ObjectAnimationUsingKeyFrames();
                    Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard.SetTargetName(ObjectAnimationUsingKeyFrames_834707980bfe46718d328b6bdf002712, "InnerBorder");
                    var DiscreteObjectKeyFrame_d5aec4f2ae604c1dbd372ec73b9e301b = new Bridge.global.Windows.UI.Xaml.Media.Animation.DiscreteObjectKeyFrame();
                    DiscreteObjectKeyFrame_d5aec4f2ae604c1dbd372ec73b9e301b.KeyTime = Bridge.global.Windows.UI.Xaml.Media.Animation.KeyTime.FromTimeSpan(new Bridge.global.System.TimeSpan(System.Int64(0)));
                    DiscreteObjectKeyFrame_d5aec4f2ae604c1dbd372ec73b9e301b.Value = "#11000000";

                    ObjectAnimationUsingKeyFrames_834707980bfe46718d328b6bdf002712.KeyFrames.add(DiscreteObjectKeyFrame_d5aec4f2ae604c1dbd372ec73b9e301b);


                    Storyboard_c71b5f75e2194357a9430ccbe7b7eb9a.Children.add(ObjectAnimationUsingKeyFrames_834707980bfe46718d328b6bdf002712);


                    VisualState_09ae383480aa423eaa3052e380cf9423.Storyboard = Storyboard_c71b5f75e2194357a9430ccbe7b7eb9a;


                    templateOwner_ControlTemplate_438ea801e3104db1b5e2ac151a376f3e.RegisterName("Pressed", VisualState_365b034cb4654db1ab676695fcb32209);
                    VisualState_365b034cb4654db1ab676695fcb32209.Name = "Pressed";
                    var Storyboard_1498c86b2772401f9ff3e64b4ec946b0 = new Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard();
                    var ObjectAnimationUsingKeyFrames_008da28ac33a44b0967a65f5e9f9b6c2 = new Bridge.global.Windows.UI.Xaml.Media.Animation.ObjectAnimationUsingKeyFrames();
                    Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard.SetTargetName(ObjectAnimationUsingKeyFrames_008da28ac33a44b0967a65f5e9f9b6c2, "InnerBorder");
                    var DiscreteObjectKeyFrame_8c0c381be9e64d5c8861a0a5bca8e808 = new Bridge.global.Windows.UI.Xaml.Media.Animation.DiscreteObjectKeyFrame();
                    DiscreteObjectKeyFrame_8c0c381be9e64d5c8861a0a5bca8e808.KeyTime = Bridge.global.Windows.UI.Xaml.Media.Animation.KeyTime.FromTimeSpan(new Bridge.global.System.TimeSpan(System.Int64(0)));
                    DiscreteObjectKeyFrame_8c0c381be9e64d5c8861a0a5bca8e808.Value = "#22000000";

                    ObjectAnimationUsingKeyFrames_008da28ac33a44b0967a65f5e9f9b6c2.KeyFrames.add(DiscreteObjectKeyFrame_8c0c381be9e64d5c8861a0a5bca8e808);


                    Storyboard_1498c86b2772401f9ff3e64b4ec946b0.Children.add(ObjectAnimationUsingKeyFrames_008da28ac33a44b0967a65f5e9f9b6c2);


                    VisualState_365b034cb4654db1ab676695fcb32209.Storyboard = Storyboard_1498c86b2772401f9ff3e64b4ec946b0;


                    templateOwner_ControlTemplate_438ea801e3104db1b5e2ac151a376f3e.RegisterName("Disabled", VisualState_5a964f4113b34c5f8dd932c790fbcc95);
                    VisualState_5a964f4113b34c5f8dd932c790fbcc95.Name = "Disabled";
                    var Storyboard_ac66994df4eb4f5b9e8f6fe66ff17deb = new Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard();
                    var ObjectAnimationUsingKeyFrames_55916996ca8641eba7d2345da9041381 = new Bridge.global.Windows.UI.Xaml.Media.Animation.ObjectAnimationUsingKeyFrames();
                    Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard.SetTargetName(ObjectAnimationUsingKeyFrames_55916996ca8641eba7d2345da9041381, "InnerBorder");
                    var DiscreteObjectKeyFrame_8200796ca4d14cf19ab13531be623f71 = new Bridge.global.Windows.UI.Xaml.Media.Animation.DiscreteObjectKeyFrame();
                    DiscreteObjectKeyFrame_8200796ca4d14cf19ab13531be623f71.KeyTime = Bridge.global.Windows.UI.Xaml.Media.Animation.KeyTime.FromTimeSpan(new Bridge.global.System.TimeSpan(System.Int64(0)));
                    DiscreteObjectKeyFrame_8200796ca4d14cf19ab13531be623f71.Value = "#33FFFFFF";

                    ObjectAnimationUsingKeyFrames_55916996ca8641eba7d2345da9041381.KeyFrames.add(DiscreteObjectKeyFrame_8200796ca4d14cf19ab13531be623f71);


                    Storyboard_ac66994df4eb4f5b9e8f6fe66ff17deb.Children.add(ObjectAnimationUsingKeyFrames_55916996ca8641eba7d2345da9041381);


                    VisualState_5a964f4113b34c5f8dd932c790fbcc95.Storyboard = Storyboard_ac66994df4eb4f5b9e8f6fe66ff17deb;


                    System.Array.add(VisualStateGroup_dd3e49f01b9a45899886f8c892b5bdfb.States, VisualState_3aef25f7e13144aab1d4758aa85d7a4e, Object);
                    System.Array.add(VisualStateGroup_dd3e49f01b9a45899886f8c892b5bdfb.States, VisualState_09ae383480aa423eaa3052e380cf9423, Object);
                    System.Array.add(VisualStateGroup_dd3e49f01b9a45899886f8c892b5bdfb.States, VisualState_365b034cb4654db1ab676695fcb32209, Object);
                    System.Array.add(VisualStateGroup_dd3e49f01b9a45899886f8c892b5bdfb.States, VisualState_5a964f4113b34c5f8dd932c790fbcc95, Object);


                    templateOwner_ControlTemplate_438ea801e3104db1b5e2ac151a376f3e.INTERNAL_GetVisualStateGroups().add(VisualStateGroup_dd3e49f01b9a45899886f8c892b5bdfb);

                    templateOwner_ControlTemplate_438ea801e3104db1b5e2ac151a376f3e.RegisterName("InnerBorder", Border_967f74e52304472393daedbfbfb1118a);
                    Border_967f74e52304472393daedbfbfb1118a.Name = "InnerBorder";
                    templateOwner_ControlTemplate_438ea801e3104db1b5e2ac151a376f3e.RegisterName("ContentPresenter", ContentPresenter_abde627c733d4559a499bbb2f9fe5bfb);
                    ContentPresenter_abde627c733d4559a499bbb2f9fe5bfb.Name = "ContentPresenter";
                    var Binding_47bc598ecfa64d6d99919b5d054e3035 = new Bridge.global.Windows.UI.Xaml.Data.Binding.ctor();
                    Binding_47bc598ecfa64d6d99919b5d054e3035.Path = new Bridge.global.Windows.UI.Xaml.PropertyPath.ctor("ContentTemplate");
                    var RelativeSource_c9e30d98dc8749778b1a4554ad24686b = new Bridge.global.Windows.UI.Xaml.Data.RelativeSource.ctor();
                    RelativeSource_c9e30d98dc8749778b1a4554ad24686b.Mode = Bridge.global.Windows.UI.Xaml.Data.RelativeSourceMode.TemplatedParent;

                    Binding_47bc598ecfa64d6d99919b5d054e3035.RelativeSource = RelativeSource_c9e30d98dc8749778b1a4554ad24686b;


                    Binding_47bc598ecfa64d6d99919b5d054e3035.TemplateOwner = templateInstance_35b70b30834241589105b16974c38832;

                    var Binding_2097c3896c1f46319aded95d4b2e166b = new Bridge.global.Windows.UI.Xaml.Data.Binding.ctor();
                    Binding_2097c3896c1f46319aded95d4b2e166b.Path = new Bridge.global.Windows.UI.Xaml.PropertyPath.ctor("Content");
                    var RelativeSource_def10460dea24a5a966a9d195fa03ce2 = new Bridge.global.Windows.UI.Xaml.Data.RelativeSource.ctor();
                    RelativeSource_def10460dea24a5a966a9d195fa03ce2.Mode = Bridge.global.Windows.UI.Xaml.Data.RelativeSourceMode.TemplatedParent;

                    Binding_2097c3896c1f46319aded95d4b2e166b.RelativeSource = RelativeSource_def10460dea24a5a966a9d195fa03ce2;


                    Binding_2097c3896c1f46319aded95d4b2e166b.TemplateOwner = templateInstance_35b70b30834241589105b16974c38832;

                    var Binding_8ca68454f7094a89a0bde297ee214fbe = new Bridge.global.Windows.UI.Xaml.Data.Binding.ctor();
                    Binding_8ca68454f7094a89a0bde297ee214fbe.Path = new Bridge.global.Windows.UI.Xaml.PropertyPath.ctor("Padding");
                    var RelativeSource_a933c60862c948f9a3d03b350d9c269c = new Bridge.global.Windows.UI.Xaml.Data.RelativeSource.ctor();
                    RelativeSource_a933c60862c948f9a3d03b350d9c269c.Mode = Bridge.global.Windows.UI.Xaml.Data.RelativeSourceMode.TemplatedParent;

                    Binding_8ca68454f7094a89a0bde297ee214fbe.RelativeSource = RelativeSource_a933c60862c948f9a3d03b350d9c269c;


                    Binding_8ca68454f7094a89a0bde297ee214fbe.TemplateOwner = templateInstance_35b70b30834241589105b16974c38832;

                    var Binding_8735b43944d640a6b1761a5e186c8fd7 = new Bridge.global.Windows.UI.Xaml.Data.Binding.ctor();
                    Binding_8735b43944d640a6b1761a5e186c8fd7.Path = new Bridge.global.Windows.UI.Xaml.PropertyPath.ctor("HorizontalContentAlignment");
                    var RelativeSource_521b155e961d41d19aa08896baab4b55 = new Bridge.global.Windows.UI.Xaml.Data.RelativeSource.ctor();
                    RelativeSource_521b155e961d41d19aa08896baab4b55.Mode = Bridge.global.Windows.UI.Xaml.Data.RelativeSourceMode.TemplatedParent;

                    Binding_8735b43944d640a6b1761a5e186c8fd7.RelativeSource = RelativeSource_521b155e961d41d19aa08896baab4b55;


                    Binding_8735b43944d640a6b1761a5e186c8fd7.TemplateOwner = templateInstance_35b70b30834241589105b16974c38832;

                    var Binding_85ac23d82b134648ab6d38281351d2f8 = new Bridge.global.Windows.UI.Xaml.Data.Binding.ctor();
                    Binding_85ac23d82b134648ab6d38281351d2f8.Path = new Bridge.global.Windows.UI.Xaml.PropertyPath.ctor("VerticalContentAlignment");
                    var RelativeSource_4243b38c783c4074a9664a4a8e2ecb52 = new Bridge.global.Windows.UI.Xaml.Data.RelativeSource.ctor();
                    RelativeSource_4243b38c783c4074a9664a4a8e2ecb52.Mode = Bridge.global.Windows.UI.Xaml.Data.RelativeSourceMode.TemplatedParent;

                    Binding_85ac23d82b134648ab6d38281351d2f8.RelativeSource = RelativeSource_4243b38c783c4074a9664a4a8e2ecb52;


                    Binding_85ac23d82b134648ab6d38281351d2f8.TemplateOwner = templateInstance_35b70b30834241589105b16974c38832;


                    Border_967f74e52304472393daedbfbfb1118a.Child = ContentPresenter_abde627c733d4559a499bbb2f9fe5bfb;

                    var Binding_7c8102fb479a4a2c8f7ac05109a17b11 = new Bridge.global.Windows.UI.Xaml.Data.Binding.ctor();
                    Binding_7c8102fb479a4a2c8f7ac05109a17b11.Path = new Bridge.global.Windows.UI.Xaml.PropertyPath.ctor("Background");
                    var RelativeSource_63b2bcc1999c4a7f81faa3d9d8dcedbc = new Bridge.global.Windows.UI.Xaml.Data.RelativeSource.ctor();
                    RelativeSource_63b2bcc1999c4a7f81faa3d9d8dcedbc.Mode = Bridge.global.Windows.UI.Xaml.Data.RelativeSourceMode.TemplatedParent;

                    Binding_7c8102fb479a4a2c8f7ac05109a17b11.RelativeSource = RelativeSource_63b2bcc1999c4a7f81faa3d9d8dcedbc;


                    Binding_7c8102fb479a4a2c8f7ac05109a17b11.TemplateOwner = templateInstance_35b70b30834241589105b16974c38832;


                    Border_b86c19a3e06f4afc86e3b5592eea67f6.Child = Border_967f74e52304472393daedbfbfb1118a;

                    var Binding_6d921c4f05a749f59f64fa1b26053b1b = new Bridge.global.Windows.UI.Xaml.Data.Binding.ctor();
                    Binding_6d921c4f05a749f59f64fa1b26053b1b.Path = new Bridge.global.Windows.UI.Xaml.PropertyPath.ctor("Background");
                    var RelativeSource_de414a258d83494ab0aeb4fe5a866895 = new Bridge.global.Windows.UI.Xaml.Data.RelativeSource.ctor();
                    RelativeSource_de414a258d83494ab0aeb4fe5a866895.Mode = Bridge.global.Windows.UI.Xaml.Data.RelativeSourceMode.TemplatedParent;

                    Binding_6d921c4f05a749f59f64fa1b26053b1b.RelativeSource = RelativeSource_de414a258d83494ab0aeb4fe5a866895;


                    Binding_6d921c4f05a749f59f64fa1b26053b1b.TemplateOwner = templateInstance_35b70b30834241589105b16974c38832;

                    var Binding_4709dc3134a44e50a79ae7837a48ca35 = new Bridge.global.Windows.UI.Xaml.Data.Binding.ctor();
                    Binding_4709dc3134a44e50a79ae7837a48ca35.Path = new Bridge.global.Windows.UI.Xaml.PropertyPath.ctor("BorderBrush");
                    var RelativeSource_62a5864154564e73bff07cf6a649c530 = new Bridge.global.Windows.UI.Xaml.Data.RelativeSource.ctor();
                    RelativeSource_62a5864154564e73bff07cf6a649c530.Mode = Bridge.global.Windows.UI.Xaml.Data.RelativeSourceMode.TemplatedParent;

                    Binding_4709dc3134a44e50a79ae7837a48ca35.RelativeSource = RelativeSource_62a5864154564e73bff07cf6a649c530;


                    Binding_4709dc3134a44e50a79ae7837a48ca35.TemplateOwner = templateInstance_35b70b30834241589105b16974c38832;

                    var Binding_5747c514f37545be98693c885433a732 = new Bridge.global.Windows.UI.Xaml.Data.Binding.ctor();
                    Binding_5747c514f37545be98693c885433a732.Path = new Bridge.global.Windows.UI.Xaml.PropertyPath.ctor("BorderThickness");
                    var RelativeSource_168876d1edf24af3a7949fe94e273ba4 = new Bridge.global.Windows.UI.Xaml.Data.RelativeSource.ctor();
                    RelativeSource_168876d1edf24af3a7949fe94e273ba4.Mode = Bridge.global.Windows.UI.Xaml.Data.RelativeSourceMode.TemplatedParent;

                    Binding_5747c514f37545be98693c885433a732.RelativeSource = RelativeSource_168876d1edf24af3a7949fe94e273ba4;


                    Binding_5747c514f37545be98693c885433a732.TemplateOwner = templateInstance_35b70b30834241589105b16974c38832;


                    Bridge.global.Windows.UI.Xaml.Data.BindingOperations.SetBinding(ContentPresenter_abde627c733d4559a499bbb2f9fe5bfb, Bridge.global.Windows.UI.Xaml.Controls.ContentControl.ContentTemplateProperty, Binding_47bc598ecfa64d6d99919b5d054e3035);
                    Bridge.global.Windows.UI.Xaml.Data.BindingOperations.SetBinding(ContentPresenter_abde627c733d4559a499bbb2f9fe5bfb, Bridge.global.Windows.UI.Xaml.Controls.ContentControl.ContentProperty, Binding_2097c3896c1f46319aded95d4b2e166b);
                    Bridge.global.Windows.UI.Xaml.Data.BindingOperations.SetBinding(ContentPresenter_abde627c733d4559a499bbb2f9fe5bfb, Bridge.global.Windows.UI.Xaml.FrameworkElement.MarginProperty, Binding_8ca68454f7094a89a0bde297ee214fbe);
                    Bridge.global.Windows.UI.Xaml.Data.BindingOperations.SetBinding(ContentPresenter_abde627c733d4559a499bbb2f9fe5bfb, Bridge.global.Windows.UI.Xaml.FrameworkElement.HorizontalAlignmentProperty, Binding_8735b43944d640a6b1761a5e186c8fd7);
                    Bridge.global.Windows.UI.Xaml.Data.BindingOperations.SetBinding(ContentPresenter_abde627c733d4559a499bbb2f9fe5bfb, Bridge.global.Windows.UI.Xaml.FrameworkElement.VerticalAlignmentProperty, Binding_85ac23d82b134648ab6d38281351d2f8);
                    Bridge.global.Windows.UI.Xaml.Data.BindingOperations.SetBinding(Border_967f74e52304472393daedbfbfb1118a, Bridge.global.Windows.UI.Xaml.Controls.Border.BackgroundProperty, Binding_7c8102fb479a4a2c8f7ac05109a17b11);
                    Bridge.global.Windows.UI.Xaml.Data.BindingOperations.SetBinding(Border_b86c19a3e06f4afc86e3b5592eea67f6, Bridge.global.Windows.UI.Xaml.Controls.Border.BackgroundProperty, Binding_6d921c4f05a749f59f64fa1b26053b1b);
                    Bridge.global.Windows.UI.Xaml.Data.BindingOperations.SetBinding(Border_b86c19a3e06f4afc86e3b5592eea67f6, Bridge.global.Windows.UI.Xaml.Controls.Border.BorderBrushProperty, Binding_4709dc3134a44e50a79ae7837a48ca35);
                    Bridge.global.Windows.UI.Xaml.Data.BindingOperations.SetBinding(Border_b86c19a3e06f4afc86e3b5592eea67f6, Bridge.global.Windows.UI.Xaml.Controls.Border.BorderThicknessProperty, Binding_5747c514f37545be98693c885433a732);

                    Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard.SetTargetProperty(ObjectAnimationUsingKeyFrames_834707980bfe46718d328b6bdf002712, new Bridge.global.Windows.UI.Xaml.PropertyPath.$ctor1("Background", "Background", Bridge.fn.cacheBind(this, this.accessVisualStateProperty_6b5bcd9fc9ca4753ba0fe99431ea817e), Bridge.fn.cacheBind(this, this.setVisualStateProperty_6b5bcd9fc9ca4753ba0fe99431ea817e), Bridge.fn.cacheBind(this, this.setAnimationVisualStateProperty_6b5bcd9fc9ca4753ba0fe99431ea817e), Bridge.fn.cacheBind(this, this.setLocalVisualStateProperty_6b5bcd9fc9ca4753ba0fe99431ea817e), Bridge.fn.cacheBind(this, this.getVisualStateProperty_6b5bcd9fc9ca4753ba0fe99431ea817e)));
                    Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard.SetTarget(ObjectAnimationUsingKeyFrames_834707980bfe46718d328b6bdf002712, Border_967f74e52304472393daedbfbfb1118a);


                    Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard.SetTargetProperty(ObjectAnimationUsingKeyFrames_008da28ac33a44b0967a65f5e9f9b6c2, new Bridge.global.Windows.UI.Xaml.PropertyPath.$ctor1("Background", "Background", Bridge.fn.cacheBind(this, this.accessVisualStateProperty_f033dabd1c774827bdbf0a52f3c363fc), Bridge.fn.cacheBind(this, this.setVisualStateProperty_f033dabd1c774827bdbf0a52f3c363fc), Bridge.fn.cacheBind(this, this.setAnimationVisualStateProperty_f033dabd1c774827bdbf0a52f3c363fc), Bridge.fn.cacheBind(this, this.setLocalVisualStateProperty_f033dabd1c774827bdbf0a52f3c363fc), Bridge.fn.cacheBind(this, this.getVisualStateProperty_f033dabd1c774827bdbf0a52f3c363fc)));
                    Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard.SetTarget(ObjectAnimationUsingKeyFrames_008da28ac33a44b0967a65f5e9f9b6c2, Border_967f74e52304472393daedbfbfb1118a);


                    Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard.SetTargetProperty(ObjectAnimationUsingKeyFrames_55916996ca8641eba7d2345da9041381, new Bridge.global.Windows.UI.Xaml.PropertyPath.$ctor1("Background", "Background", Bridge.fn.cacheBind(this, this.accessVisualStateProperty_1a6ff13a71b640608280d8a1e796bdde), Bridge.fn.cacheBind(this, this.setVisualStateProperty_1a6ff13a71b640608280d8a1e796bdde), Bridge.fn.cacheBind(this, this.setAnimationVisualStateProperty_1a6ff13a71b640608280d8a1e796bdde), Bridge.fn.cacheBind(this, this.setLocalVisualStateProperty_1a6ff13a71b640608280d8a1e796bdde), Bridge.fn.cacheBind(this, this.getVisualStateProperty_1a6ff13a71b640608280d8a1e796bdde)));
                    Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard.SetTarget(ObjectAnimationUsingKeyFrames_55916996ca8641eba7d2345da9041381, Border_967f74e52304472393daedbfbfb1118a);

                    templateInstance_35b70b30834241589105b16974c38832.TemplateContent = Border_b86c19a3e06f4afc86e3b5592eea67f6;
                    return templateInstance_35b70b30834241589105b16974c38832;
                }));

                Setter_32ce28170c914d5cb56b135320a0e97c.Value = ControlTemplate_438ea801e3104db1b5e2ac151a376f3e;


                Style_5fba16df9cf44216833b02d461e665de.Setters.add(Setter_8c806b9891c54c1dbc9f8e08b14645b5);
                Style_5fba16df9cf44216833b02d461e665de.Setters.add(Setter_da1b61cfa19d40e39d3a4bb23fb6d5fc);
                Style_5fba16df9cf44216833b02d461e665de.Setters.add(Setter_839d7b5b20bd4490b4fdcd4b45249f74);
                Style_5fba16df9cf44216833b02d461e665de.Setters.add(Setter_6988e9fc910743e3a54f772cf4751100);
                Style_5fba16df9cf44216833b02d461e665de.Setters.add(Setter_ae62fcad7b2b4a0ab1a0621c0c7e1a00);
                Style_5fba16df9cf44216833b02d461e665de.Setters.add(Setter_35ad014628614a6fbd52a53cdf97290d);
                Style_5fba16df9cf44216833b02d461e665de.Setters.add(Setter_c928309aee7f4a86809462b668e6bdca);
                Style_5fba16df9cf44216833b02d461e665de.Setters.add(Setter_32ce28170c914d5cb56b135320a0e97c);


                this.setItem(Bridge.global.Windows.UI.Xaml.Controls.Primitives.RepeatButton, Style_5fba16df9cf44216833b02d461e665de);






            },
            accessVisualStateProperty_6b5bcd9fc9ca4753ba0fe99431ea817e: function (rootTargetObjectInstance) {
                return new (Bridge.GeneratorEnumerable$1(System.Tuple$3(Windows.UI.Xaml.DependencyObject,Windows.UI.Xaml.DependencyProperty,System.Nullable$1(System.Int32))))(Bridge.fn.bind(this, function (rootTargetObjectInstance) {
                    var $step = 0,
                        $jumpFromFinally,
                        $returnValue,
                        $async_e;

                    var $enumerator = new (Bridge.GeneratorEnumerator$1(System.Tuple$3(Windows.UI.Xaml.DependencyObject,Windows.UI.Xaml.DependencyProperty,System.Nullable$1(System.Int32))))(Bridge.fn.bind(this, function () {
                        try {
                            for (;;) {
                                switch ($step) {
                                    case 0: {
                                        return false;
                                    }
                                    default: {
                                        return false;
                                    }
                                }
                            }
                        } catch($async_e1) {
                            $async_e = System.Exception.create($async_e1);
                            throw $async_e;
                        }
                    }));
                    return $enumerator;
                }, arguments));
            },
            setVisualStateProperty_6b5bcd9fc9ca4753ba0fe99431ea817e: function (finalTargetInstance, value) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Background").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "BackgroundProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                (finalTargetInstance).SetVisualStateValue(property, value);
            },
            setAnimationVisualStateProperty_6b5bcd9fc9ca4753ba0fe99431ea817e: function (finalTargetInstance, value) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Background").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "BackgroundProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                (finalTargetInstance).SetAnimationValue(property, value);
            },
            setLocalVisualStateProperty_6b5bcd9fc9ca4753ba0fe99431ea817e: function (finalTargetInstance, value) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Background").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "BackgroundProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                (finalTargetInstance).SetLocalValue(property, value);
            },
            getVisualStateProperty_6b5bcd9fc9ca4753ba0fe99431ea817e: function (finalTargetInstance) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Background").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "BackgroundProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                return finalTargetInstance.GetVisualStateValue(property);
            },
            accessVisualStateProperty_f033dabd1c774827bdbf0a52f3c363fc: function (rootTargetObjectInstance) {
                return new (Bridge.GeneratorEnumerable$1(System.Tuple$3(Windows.UI.Xaml.DependencyObject,Windows.UI.Xaml.DependencyProperty,System.Nullable$1(System.Int32))))(Bridge.fn.bind(this, function (rootTargetObjectInstance) {
                    var $step = 0,
                        $jumpFromFinally,
                        $returnValue,
                        $async_e;

                    var $enumerator = new (Bridge.GeneratorEnumerator$1(System.Tuple$3(Windows.UI.Xaml.DependencyObject,Windows.UI.Xaml.DependencyProperty,System.Nullable$1(System.Int32))))(Bridge.fn.bind(this, function () {
                        try {
                            for (;;) {
                                switch ($step) {
                                    case 0: {
                                        return false;
                                    }
                                    default: {
                                        return false;
                                    }
                                }
                            }
                        } catch($async_e1) {
                            $async_e = System.Exception.create($async_e1);
                            throw $async_e;
                        }
                    }));
                    return $enumerator;
                }, arguments));
            },
            setVisualStateProperty_f033dabd1c774827bdbf0a52f3c363fc: function (finalTargetInstance, value) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Background").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "BackgroundProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                (finalTargetInstance).SetVisualStateValue(property, value);
            },
            setAnimationVisualStateProperty_f033dabd1c774827bdbf0a52f3c363fc: function (finalTargetInstance, value) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Background").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "BackgroundProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                (finalTargetInstance).SetAnimationValue(property, value);
            },
            setLocalVisualStateProperty_f033dabd1c774827bdbf0a52f3c363fc: function (finalTargetInstance, value) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Background").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "BackgroundProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                (finalTargetInstance).SetLocalValue(property, value);
            },
            getVisualStateProperty_f033dabd1c774827bdbf0a52f3c363fc: function (finalTargetInstance) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Background").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "BackgroundProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                return finalTargetInstance.GetVisualStateValue(property);
            },
            accessVisualStateProperty_1a6ff13a71b640608280d8a1e796bdde: function (rootTargetObjectInstance) {
                return new (Bridge.GeneratorEnumerable$1(System.Tuple$3(Windows.UI.Xaml.DependencyObject,Windows.UI.Xaml.DependencyProperty,System.Nullable$1(System.Int32))))(Bridge.fn.bind(this, function (rootTargetObjectInstance) {
                    var $step = 0,
                        $jumpFromFinally,
                        $returnValue,
                        $async_e;

                    var $enumerator = new (Bridge.GeneratorEnumerator$1(System.Tuple$3(Windows.UI.Xaml.DependencyObject,Windows.UI.Xaml.DependencyProperty,System.Nullable$1(System.Int32))))(Bridge.fn.bind(this, function () {
                        try {
                            for (;;) {
                                switch ($step) {
                                    case 0: {
                                        return false;
                                    }
                                    default: {
                                        return false;
                                    }
                                }
                            }
                        } catch($async_e1) {
                            $async_e = System.Exception.create($async_e1);
                            throw $async_e;
                        }
                    }));
                    return $enumerator;
                }, arguments));
            },
            setVisualStateProperty_1a6ff13a71b640608280d8a1e796bdde: function (finalTargetInstance, value) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Background").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "BackgroundProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                (finalTargetInstance).SetVisualStateValue(property, value);
            },
            setAnimationVisualStateProperty_1a6ff13a71b640608280d8a1e796bdde: function (finalTargetInstance, value) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Background").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "BackgroundProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                (finalTargetInstance).SetAnimationValue(property, value);
            },
            setLocalVisualStateProperty_1a6ff13a71b640608280d8a1e796bdde: function (finalTargetInstance, value) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Background").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "BackgroundProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                (finalTargetInstance).SetLocalValue(property, value);
            },
            getVisualStateProperty_1a6ff13a71b640608280d8a1e796bdde: function (finalTargetInstance) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Background").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "BackgroundProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                return finalTargetInstance.GetVisualStateValue(property);
            }
        }
    });

    Bridge.define("___CSHTML5__Component__Defaultstyles__INTERNAL_Defaultscrollbarstyle__Xaml", {
        inherits: [Bridge.global.Windows.UI.Xaml.ResourceDictionary],
        fields: {
            _contentLoaded: false
        },
        ctors: {
            ctor: function () {
                this.$initialize();
                Bridge.global.Windows.UI.Xaml.ResourceDictionary.ctor.call(this);
                this.InitializeComponent();

                if (Bridge.is(this, Windows.UI.Xaml.UIElement)) {
                    Bridge.cast(this, Windows.UI.Xaml.UIElement).XamlSourcePath = "CSHTML5\\DefaultStyles\\INTERNAL_DefaultScrollBarStyle.xaml";
                }
            }
        },
        methods: {
            InitializeComponent: function () {
                var $t;
                if (this._contentLoaded) {
                    return;
                }
                this._contentLoaded = true;


                if (Bridge.is(this, Windows.UI.Xaml.UIElement)) {
                    Bridge.cast(this, Windows.UI.Xaml.UIElement).XamlSourcePath = "CSHTML5\\DefaultStyles\\INTERNAL_DefaultScrollBarStyle.xaml";
                }



                var Style_8438f3596a8c4d99850b8e4f85ac8ce8 = new Bridge.global.Windows.UI.Xaml.Style.ctor();
                Style_8438f3596a8c4d99850b8e4f85ac8ce8.TargetType = Bridge.global.Windows.UI.Xaml.Controls.Primitives.ScrollBar;
                var Setter_8461cf4152c445b9bfe8fdb79e76650a = new Bridge.global.Windows.UI.Xaml.Setter.ctor();
                Setter_8461cf4152c445b9bfe8fdb79e76650a.Property = Bridge.global.Windows.UI.Xaml.Controls.Control.BackgroundProperty;
                Setter_8461cf4152c445b9bfe8fdb79e76650a.Value = new Bridge.global.Windows.UI.Xaml.Media.SolidColorBrush.$ctor1(($t = new Bridge.global.Windows.UI.Color(), $t.A = 255, $t.R = 241, $t.G = 241, $t.B = 241, $t));

                var Setter_8f2ce6d1f2964887a0758601b2ea2ee9 = new Bridge.global.Windows.UI.Xaml.Setter.ctor();
                Setter_8f2ce6d1f2964887a0758601b2ea2ee9.Property = Bridge.global.Windows.UI.Xaml.Controls.Control.ForegroundProperty;
                Setter_8f2ce6d1f2964887a0758601b2ea2ee9.Value = new Bridge.global.Windows.UI.Xaml.Media.SolidColorBrush.$ctor1(($t = new Bridge.global.Windows.UI.Color(), $t.A = 255, $t.R = 51, $t.G = 51, $t.B = 51, $t));

                var Setter_22e7fbbc5a4b4bcca95ae06c77cd0577 = new Bridge.global.Windows.UI.Xaml.Setter.ctor();
                Setter_22e7fbbc5a4b4bcca95ae06c77cd0577.Property = Bridge.global.Windows.UI.Xaml.Controls.Control.TemplateProperty;
                var ControlTemplate_7361eebb3a3f4bf18f05f5d2ba4e9c44 = new Bridge.global.Windows.UI.Xaml.Controls.ControlTemplate();
                ControlTemplate_7361eebb3a3f4bf18f05f5d2ba4e9c44.TargetType = Bridge.global.Windows.UI.Xaml.Controls.Primitives.ScrollBar;
                ControlTemplate_7361eebb3a3f4bf18f05f5d2ba4e9c44.SetMethodToInstantiateFrameworkTemplate(function (templateOwner_ControlTemplate_7361eebb3a3f4bf18f05f5d2ba4e9c44) {
                    var $t1;
                    var templateInstance_27f7b90524d9426b9e8e5e857f519852 = new Bridge.global.Windows.UI.Xaml.TemplateInstance();
                    templateInstance_27f7b90524d9426b9e8e5e857f519852.TemplateOwner = templateOwner_ControlTemplate_7361eebb3a3f4bf18f05f5d2ba4e9c44;
                    var VisualState_f637c388af7f47d39f159a47fd8ea838 = new Bridge.global.Windows.UI.Xaml.VisualState();
                    var VisualState_1f26c211243d49d6bce67fbdd0f2db9f = new Bridge.global.Windows.UI.Xaml.VisualState();
                    var VisualState_c6685ef95a3f4d32b366dbef701a36d1 = new Bridge.global.Windows.UI.Xaml.VisualState();
                    var VisualStateGroup_fb6629f1e8df462bbe9b71f738b276e9 = new Bridge.global.Windows.UI.Xaml.VisualStateGroup();
                    var Path_25ae2db4019d4519818f4dcf37a1cd13 = new Bridge.global.Windows.UI.Xaml.Shapes.Path();
                    var Button_58f9675fe9e6422e9e95278dcb3e91a9 = new Bridge.global.Windows.UI.Xaml.Controls.Button();
                    var Path_9588caf77507488abef649703c8c7919 = new Bridge.global.Windows.UI.Xaml.Shapes.Path();
                    var Button_c36bf13b9c87414896904030a9268c6c = new Bridge.global.Windows.UI.Xaml.Controls.Button();
                    var Button_953f2c31edca4562afd7dfbf67669950 = new Bridge.global.Windows.UI.Xaml.Controls.Button();
                    var Button_1d0f6d11c28d4ae598695fec104256d7 = new Bridge.global.Windows.UI.Xaml.Controls.Button();
                    var Thumb_5f977c45fcd44766b452bb93d2a6fef5 = new Bridge.global.Windows.UI.Xaml.Controls.Primitives.Thumb();
                    var Canvas_1e5fb39faa9245d089e403c5915e96a9 = new Bridge.global.Windows.UI.Xaml.Controls.Canvas();
                    var Path_409ec60dcb1145bdacca2a4c4d922c15 = new Bridge.global.Windows.UI.Xaml.Shapes.Path();
                    var Button_3f742f73e8484bb881475ecbb477fef7 = new Bridge.global.Windows.UI.Xaml.Controls.Button();
                    var Path_d94465ff8099460d93050eca6707620f = new Bridge.global.Windows.UI.Xaml.Shapes.Path();
                    var Button_a8b593478ed443ed9c8c131120ef1a7d = new Bridge.global.Windows.UI.Xaml.Controls.Button();
                    var Button_64dbacbd6aaf40a79503da8377b5239e = new Bridge.global.Windows.UI.Xaml.Controls.Button();
                    var Button_4f3a80492b2b4ec68e9b725059f34ec3 = new Bridge.global.Windows.UI.Xaml.Controls.Button();
                    var Thumb_fbdc9804f0b54a93aa82d97009da672e = new Bridge.global.Windows.UI.Xaml.Controls.Primitives.Thumb();
                    var Canvas_8657a791d83149d399287616e63b3e0f = new Bridge.global.Windows.UI.Xaml.Controls.Canvas();
                    var Grid_dc16e46733ba4ce8a50cf72d3d37de46 = new Bridge.global.Windows.UI.Xaml.Controls.Grid();
                    templateOwner_ControlTemplate_7361eebb3a3f4bf18f05f5d2ba4e9c44.RegisterName("Root", Grid_dc16e46733ba4ce8a50cf72d3d37de46);
                    Grid_dc16e46733ba4ce8a50cf72d3d37de46.Name = "Root";
                    templateOwner_ControlTemplate_7361eebb3a3f4bf18f05f5d2ba4e9c44.RegisterName("CommonStates", VisualStateGroup_fb6629f1e8df462bbe9b71f738b276e9);
                    VisualStateGroup_fb6629f1e8df462bbe9b71f738b276e9.Name = "CommonStates";
                    templateOwner_ControlTemplate_7361eebb3a3f4bf18f05f5d2ba4e9c44.RegisterName("Normal", VisualState_f637c388af7f47d39f159a47fd8ea838);
                    VisualState_f637c388af7f47d39f159a47fd8ea838.Name = "Normal";

                    templateOwner_ControlTemplate_7361eebb3a3f4bf18f05f5d2ba4e9c44.RegisterName("PointerOver", VisualState_1f26c211243d49d6bce67fbdd0f2db9f);
                    VisualState_1f26c211243d49d6bce67fbdd0f2db9f.Name = "PointerOver";

                    templateOwner_ControlTemplate_7361eebb3a3f4bf18f05f5d2ba4e9c44.RegisterName("Disabled", VisualState_c6685ef95a3f4d32b366dbef701a36d1);
                    VisualState_c6685ef95a3f4d32b366dbef701a36d1.Name = "Disabled";

                    System.Array.add(VisualStateGroup_fb6629f1e8df462bbe9b71f738b276e9.States, VisualState_f637c388af7f47d39f159a47fd8ea838, Object);
                    System.Array.add(VisualStateGroup_fb6629f1e8df462bbe9b71f738b276e9.States, VisualState_1f26c211243d49d6bce67fbdd0f2db9f, Object);
                    System.Array.add(VisualStateGroup_fb6629f1e8df462bbe9b71f738b276e9.States, VisualState_c6685ef95a3f4d32b366dbef701a36d1, Object);


                    templateOwner_ControlTemplate_7361eebb3a3f4bf18f05f5d2ba4e9c44.INTERNAL_GetVisualStateGroups().add(VisualStateGroup_fb6629f1e8df462bbe9b71f738b276e9);

                    templateOwner_ControlTemplate_7361eebb3a3f4bf18f05f5d2ba4e9c44.RegisterName("HorizontalRoot", Canvas_1e5fb39faa9245d089e403c5915e96a9);
                    Canvas_1e5fb39faa9245d089e403c5915e96a9.Name = "HorizontalRoot";
                    Canvas_1e5fb39faa9245d089e403c5915e96a9.Visibility = Bridge.global.Windows.UI.Xaml.Visibility.Collapsed;
                    templateOwner_ControlTemplate_7361eebb3a3f4bf18f05f5d2ba4e9c44.RegisterName("HorizontalSmallDecrease", Button_58f9675fe9e6422e9e95278dcb3e91a9);
                    Button_58f9675fe9e6422e9e95278dcb3e91a9.Name = "HorizontalSmallDecrease";
                    Button_58f9675fe9e6422e9e95278dcb3e91a9.Padding = new Bridge.global.Windows.UI.Xaml.Thickness.$ctor1(0);
                    Button_58f9675fe9e6422e9e95278dcb3e91a9.Cursor = Bridge.global.System.Windows.Input.Cursors.Arrow;
                    templateOwner_ControlTemplate_7361eebb3a3f4bf18f05f5d2ba4e9c44.RegisterName("ArrowLeft", Path_25ae2db4019d4519818f4dcf37a1cd13);
                    Path_25ae2db4019d4519818f4dcf37a1cd13.Name = "ArrowLeft";
                    Path_25ae2db4019d4519818f4dcf37a1cd13.Fill = new Bridge.global.Windows.UI.Xaml.Media.SolidColorBrush.$ctor1(($t1 = new Bridge.global.Windows.UI.Color(), $t1.A = 255, $t1.R = 85, $t1.G = 85, $t1.B = 85, $t1));
                    Path_25ae2db4019d4519818f4dcf37a1cd13.Width = 9.0;
                    Path_25ae2db4019d4519818f4dcf37a1cd13.Height = 9.0;
                    Path_25ae2db4019d4519818f4dcf37a1cd13.Margin = new Bridge.global.Windows.UI.Xaml.Thickness.$ctor2(0, 0, 3, 0);
                    Path_25ae2db4019d4519818f4dcf37a1cd13.StrokeThickness = 3.0;
                    Path_25ae2db4019d4519818f4dcf37a1cd13.HorizontalAlignment = Bridge.global.Windows.UI.Xaml.HorizontalAlignment.Center;
                    Path_25ae2db4019d4519818f4dcf37a1cd13.VerticalAlignment = Bridge.global.Windows.UI.Xaml.VerticalAlignment.Center;
                    Path_25ae2db4019d4519818f4dcf37a1cd13.Stretch = Bridge.global.Windows.UI.Xaml.Media.Stretch.Fill;
                    Path_25ae2db4019d4519818f4dcf37a1cd13.Data = Bridge.cast(Bridge.global.DotNetForHtml5.Core.TypeFromStringConverters.ConvertFromInvariantString(Bridge.global.Windows.UI.Xaml.Media.Geometry, "M 2,4.5 L 5.5,1 L 5.5,8"), Windows.UI.Xaml.Media.Geometry);

                    Button_58f9675fe9e6422e9e95278dcb3e91a9.Content = Path_25ae2db4019d4519818f4dcf37a1cd13;


                    templateOwner_ControlTemplate_7361eebb3a3f4bf18f05f5d2ba4e9c44.RegisterName("HorizontalSmallIncrease", Button_c36bf13b9c87414896904030a9268c6c);
                    Button_c36bf13b9c87414896904030a9268c6c.Name = "HorizontalSmallIncrease";
                    Button_c36bf13b9c87414896904030a9268c6c.Padding = new Bridge.global.Windows.UI.Xaml.Thickness.$ctor1(0);
                    Button_c36bf13b9c87414896904030a9268c6c.Cursor = Bridge.global.System.Windows.Input.Cursors.Arrow;
                    templateOwner_ControlTemplate_7361eebb3a3f4bf18f05f5d2ba4e9c44.RegisterName("ArrowRight", Path_9588caf77507488abef649703c8c7919);
                    Path_9588caf77507488abef649703c8c7919.Name = "ArrowRight";
                    Path_9588caf77507488abef649703c8c7919.Fill = new Bridge.global.Windows.UI.Xaml.Media.SolidColorBrush.$ctor1(($t1 = new Bridge.global.Windows.UI.Color(), $t1.A = 255, $t1.R = 85, $t1.G = 85, $t1.B = 85, $t1));
                    Path_9588caf77507488abef649703c8c7919.Width = 9.0;
                    Path_9588caf77507488abef649703c8c7919.Height = 9.0;
                    Path_9588caf77507488abef649703c8c7919.Margin = new Bridge.global.Windows.UI.Xaml.Thickness.$ctor2(0, 0, 3, 0);
                    Path_9588caf77507488abef649703c8c7919.StrokeThickness = 3.0;
                    Path_9588caf77507488abef649703c8c7919.HorizontalAlignment = Bridge.global.Windows.UI.Xaml.HorizontalAlignment.Center;
                    Path_9588caf77507488abef649703c8c7919.VerticalAlignment = Bridge.global.Windows.UI.Xaml.VerticalAlignment.Center;
                    Path_9588caf77507488abef649703c8c7919.Stretch = Bridge.global.Windows.UI.Xaml.Media.Stretch.Fill;
                    Path_9588caf77507488abef649703c8c7919.Data = Bridge.cast(Bridge.global.DotNetForHtml5.Core.TypeFromStringConverters.ConvertFromInvariantString(Bridge.global.Windows.UI.Xaml.Media.Geometry, "M 2,1 L 5.5,4.5 L 2,8"), Windows.UI.Xaml.Media.Geometry);

                    Button_c36bf13b9c87414896904030a9268c6c.Content = Path_9588caf77507488abef649703c8c7919;


                    templateOwner_ControlTemplate_7361eebb3a3f4bf18f05f5d2ba4e9c44.RegisterName("HorizontalLargeDecrease", Button_953f2c31edca4562afd7dfbf67669950);
                    Button_953f2c31edca4562afd7dfbf67669950.Name = "HorizontalLargeDecrease";
                    Button_953f2c31edca4562afd7dfbf67669950.Padding = new Bridge.global.Windows.UI.Xaml.Thickness.$ctor1(0);
                    Button_953f2c31edca4562afd7dfbf67669950.Cursor = Bridge.global.System.Windows.Input.Cursors.Arrow;

                    templateOwner_ControlTemplate_7361eebb3a3f4bf18f05f5d2ba4e9c44.RegisterName("HorizontalLargeIncrease", Button_1d0f6d11c28d4ae598695fec104256d7);
                    Button_1d0f6d11c28d4ae598695fec104256d7.Name = "HorizontalLargeIncrease";
                    Button_1d0f6d11c28d4ae598695fec104256d7.Padding = new Bridge.global.Windows.UI.Xaml.Thickness.$ctor1(0);
                    Button_1d0f6d11c28d4ae598695fec104256d7.Cursor = Bridge.global.System.Windows.Input.Cursors.Arrow;

                    templateOwner_ControlTemplate_7361eebb3a3f4bf18f05f5d2ba4e9c44.RegisterName("HorizontalThumb", Thumb_5f977c45fcd44766b452bb93d2a6fef5);
                    Thumb_5f977c45fcd44766b452bb93d2a6fef5.Name = "HorizontalThumb";
                    Thumb_5f977c45fcd44766b452bb93d2a6fef5.Cursor = Bridge.global.System.Windows.Input.Cursors.Arrow;
                    var ControlTemplate_89f885be3a524ce280cea73fcb831cdc = new Bridge.global.Windows.UI.Xaml.Controls.ControlTemplate();
                    ControlTemplate_89f885be3a524ce280cea73fcb831cdc.SetMethodToInstantiateFrameworkTemplate(function (templateOwner_ControlTemplate_89f885be3a524ce280cea73fcb831cdc) {
                        var $t2;
                        var templateInstance_dad76939ab5e427095b20813a6d52b0a = new Bridge.global.Windows.UI.Xaml.TemplateInstance();
                        templateInstance_dad76939ab5e427095b20813a6d52b0a.TemplateOwner = templateOwner_ControlTemplate_89f885be3a524ce280cea73fcb831cdc;

                        var Border_2036eb2d9275450abc770664615e836e = new Bridge.global.Windows.UI.Xaml.Controls.Border();
                        Border_2036eb2d9275450abc770664615e836e.Background = new Bridge.global.Windows.UI.Xaml.Media.SolidColorBrush.$ctor1(($t2 = new Bridge.global.Windows.UI.Color(), $t2.A = 255, $t2.R = 187, $t2.G = 187, $t2.B = 187, $t2));



                        templateInstance_dad76939ab5e427095b20813a6d52b0a.TemplateContent = Border_2036eb2d9275450abc770664615e836e;
                        return templateInstance_dad76939ab5e427095b20813a6d52b0a;
                    });

                    Thumb_5f977c45fcd44766b452bb93d2a6fef5.Template = ControlTemplate_89f885be3a524ce280cea73fcb831cdc;


                    Canvas_1e5fb39faa9245d089e403c5915e96a9.Children.add(Button_58f9675fe9e6422e9e95278dcb3e91a9);
                    Canvas_1e5fb39faa9245d089e403c5915e96a9.Children.add(Button_c36bf13b9c87414896904030a9268c6c);
                    Canvas_1e5fb39faa9245d089e403c5915e96a9.Children.add(Button_953f2c31edca4562afd7dfbf67669950);
                    Canvas_1e5fb39faa9245d089e403c5915e96a9.Children.add(Button_1d0f6d11c28d4ae598695fec104256d7);
                    Canvas_1e5fb39faa9245d089e403c5915e96a9.Children.add(Thumb_5f977c45fcd44766b452bb93d2a6fef5);

                    var Binding_6677240f2c554d9eb754f6f282f7f891 = new Bridge.global.Windows.UI.Xaml.Data.Binding.ctor();
                    Binding_6677240f2c554d9eb754f6f282f7f891.Path = new Bridge.global.Windows.UI.Xaml.PropertyPath.ctor("Background");
                    var RelativeSource_ca9464af394c48748c9590f0acce4786 = new Bridge.global.Windows.UI.Xaml.Data.RelativeSource.ctor();
                    RelativeSource_ca9464af394c48748c9590f0acce4786.Mode = Bridge.global.Windows.UI.Xaml.Data.RelativeSourceMode.TemplatedParent;

                    Binding_6677240f2c554d9eb754f6f282f7f891.RelativeSource = RelativeSource_ca9464af394c48748c9590f0acce4786;


                    Binding_6677240f2c554d9eb754f6f282f7f891.TemplateOwner = templateInstance_27f7b90524d9426b9e8e5e857f519852;


                    templateOwner_ControlTemplate_7361eebb3a3f4bf18f05f5d2ba4e9c44.RegisterName("VerticalRoot", Canvas_8657a791d83149d399287616e63b3e0f);
                    Canvas_8657a791d83149d399287616e63b3e0f.Name = "VerticalRoot";
                    Canvas_8657a791d83149d399287616e63b3e0f.Visibility = Bridge.global.Windows.UI.Xaml.Visibility.Collapsed;
                    templateOwner_ControlTemplate_7361eebb3a3f4bf18f05f5d2ba4e9c44.RegisterName("VerticalSmallDecrease", Button_3f742f73e8484bb881475ecbb477fef7);
                    Button_3f742f73e8484bb881475ecbb477fef7.Name = "VerticalSmallDecrease";
                    Button_3f742f73e8484bb881475ecbb477fef7.Padding = new Bridge.global.Windows.UI.Xaml.Thickness.$ctor1(0);
                    Button_3f742f73e8484bb881475ecbb477fef7.Cursor = Bridge.global.System.Windows.Input.Cursors.Arrow;
                    templateOwner_ControlTemplate_7361eebb3a3f4bf18f05f5d2ba4e9c44.RegisterName("ArrowTop", Path_409ec60dcb1145bdacca2a4c4d922c15);
                    Path_409ec60dcb1145bdacca2a4c4d922c15.Name = "ArrowTop";
                    Path_409ec60dcb1145bdacca2a4c4d922c15.Fill = new Bridge.global.Windows.UI.Xaml.Media.SolidColorBrush.$ctor1(($t1 = new Bridge.global.Windows.UI.Color(), $t1.A = 255, $t1.R = 85, $t1.G = 85, $t1.B = 85, $t1));
                    Path_409ec60dcb1145bdacca2a4c4d922c15.Width = 9.0;
                    Path_409ec60dcb1145bdacca2a4c4d922c15.Height = 9.0;
                    Path_409ec60dcb1145bdacca2a4c4d922c15.Margin = new Bridge.global.Windows.UI.Xaml.Thickness.$ctor2(0, 0, 3, 0);
                    Path_409ec60dcb1145bdacca2a4c4d922c15.StrokeThickness = 3.0;
                    Path_409ec60dcb1145bdacca2a4c4d922c15.HorizontalAlignment = Bridge.global.Windows.UI.Xaml.HorizontalAlignment.Center;
                    Path_409ec60dcb1145bdacca2a4c4d922c15.VerticalAlignment = Bridge.global.Windows.UI.Xaml.VerticalAlignment.Center;
                    Path_409ec60dcb1145bdacca2a4c4d922c15.Stretch = Bridge.global.Windows.UI.Xaml.Media.Stretch.Fill;
                    Path_409ec60dcb1145bdacca2a4c4d922c15.Data = Bridge.cast(Bridge.global.DotNetForHtml5.Core.TypeFromStringConverters.ConvertFromInvariantString(Bridge.global.Windows.UI.Xaml.Media.Geometry, "M 4.5,2 L 1,5.5 L 8,5.5"), Windows.UI.Xaml.Media.Geometry);

                    Button_3f742f73e8484bb881475ecbb477fef7.Content = Path_409ec60dcb1145bdacca2a4c4d922c15;


                    templateOwner_ControlTemplate_7361eebb3a3f4bf18f05f5d2ba4e9c44.RegisterName("VerticalSmallIncrease", Button_a8b593478ed443ed9c8c131120ef1a7d);
                    Button_a8b593478ed443ed9c8c131120ef1a7d.Name = "VerticalSmallIncrease";
                    Button_a8b593478ed443ed9c8c131120ef1a7d.Padding = new Bridge.global.Windows.UI.Xaml.Thickness.$ctor1(0);
                    Button_a8b593478ed443ed9c8c131120ef1a7d.Cursor = Bridge.global.System.Windows.Input.Cursors.Arrow;
                    templateOwner_ControlTemplate_7361eebb3a3f4bf18f05f5d2ba4e9c44.RegisterName("ArrowBottom", Path_d94465ff8099460d93050eca6707620f);
                    Path_d94465ff8099460d93050eca6707620f.Name = "ArrowBottom";
                    Path_d94465ff8099460d93050eca6707620f.Fill = new Bridge.global.Windows.UI.Xaml.Media.SolidColorBrush.$ctor1(($t1 = new Bridge.global.Windows.UI.Color(), $t1.A = 255, $t1.R = 85, $t1.G = 85, $t1.B = 85, $t1));
                    Path_d94465ff8099460d93050eca6707620f.Width = 9.0;
                    Path_d94465ff8099460d93050eca6707620f.Height = 9.0;
                    Path_d94465ff8099460d93050eca6707620f.Margin = new Bridge.global.Windows.UI.Xaml.Thickness.$ctor2(0, 0, 3, 0);
                    Path_d94465ff8099460d93050eca6707620f.StrokeThickness = 3.0;
                    Path_d94465ff8099460d93050eca6707620f.HorizontalAlignment = Bridge.global.Windows.UI.Xaml.HorizontalAlignment.Center;
                    Path_d94465ff8099460d93050eca6707620f.VerticalAlignment = Bridge.global.Windows.UI.Xaml.VerticalAlignment.Center;
                    Path_d94465ff8099460d93050eca6707620f.Stretch = Bridge.global.Windows.UI.Xaml.Media.Stretch.Fill;
                    Path_d94465ff8099460d93050eca6707620f.Data = Bridge.cast(Bridge.global.DotNetForHtml5.Core.TypeFromStringConverters.ConvertFromInvariantString(Bridge.global.Windows.UI.Xaml.Media.Geometry, "M 1,2 L 4.5,5.5 L 8,2"), Windows.UI.Xaml.Media.Geometry);

                    Button_a8b593478ed443ed9c8c131120ef1a7d.Content = Path_d94465ff8099460d93050eca6707620f;


                    templateOwner_ControlTemplate_7361eebb3a3f4bf18f05f5d2ba4e9c44.RegisterName("VerticalLargeDecrease", Button_64dbacbd6aaf40a79503da8377b5239e);
                    Button_64dbacbd6aaf40a79503da8377b5239e.Name = "VerticalLargeDecrease";
                    Button_64dbacbd6aaf40a79503da8377b5239e.Padding = new Bridge.global.Windows.UI.Xaml.Thickness.$ctor1(0);
                    Button_64dbacbd6aaf40a79503da8377b5239e.Cursor = Bridge.global.System.Windows.Input.Cursors.Arrow;

                    templateOwner_ControlTemplate_7361eebb3a3f4bf18f05f5d2ba4e9c44.RegisterName("VerticalLargeIncrease", Button_4f3a80492b2b4ec68e9b725059f34ec3);
                    Button_4f3a80492b2b4ec68e9b725059f34ec3.Name = "VerticalLargeIncrease";
                    Button_4f3a80492b2b4ec68e9b725059f34ec3.Padding = new Bridge.global.Windows.UI.Xaml.Thickness.$ctor1(0);
                    Button_4f3a80492b2b4ec68e9b725059f34ec3.Cursor = Bridge.global.System.Windows.Input.Cursors.Arrow;

                    templateOwner_ControlTemplate_7361eebb3a3f4bf18f05f5d2ba4e9c44.RegisterName("VerticalThumb", Thumb_fbdc9804f0b54a93aa82d97009da672e);
                    Thumb_fbdc9804f0b54a93aa82d97009da672e.Name = "VerticalThumb";
                    Thumb_fbdc9804f0b54a93aa82d97009da672e.Background = new Bridge.global.Windows.UI.Xaml.Media.SolidColorBrush.$ctor1(($t1 = new Bridge.global.Windows.UI.Color(), $t1.A = 255, $t1.R = 187, $t1.G = 187, $t1.B = 187, $t1));
                    Thumb_fbdc9804f0b54a93aa82d97009da672e.Cursor = Bridge.global.System.Windows.Input.Cursors.Arrow;
                    var ControlTemplate_e0c041a4bd62455d9e770e77979dec61 = new Bridge.global.Windows.UI.Xaml.Controls.ControlTemplate();
                    ControlTemplate_e0c041a4bd62455d9e770e77979dec61.SetMethodToInstantiateFrameworkTemplate(function (templateOwner_ControlTemplate_e0c041a4bd62455d9e770e77979dec61) {
                        var $t2;
                        var templateInstance_10016036eeb84fda97d06ff48e550fb6 = new Bridge.global.Windows.UI.Xaml.TemplateInstance();
                        templateInstance_10016036eeb84fda97d06ff48e550fb6.TemplateOwner = templateOwner_ControlTemplate_e0c041a4bd62455d9e770e77979dec61;

                        var Border_67c199d075884e89b9f34c5a8a50f93f = new Bridge.global.Windows.UI.Xaml.Controls.Border();
                        Border_67c199d075884e89b9f34c5a8a50f93f.Background = new Bridge.global.Windows.UI.Xaml.Media.SolidColorBrush.$ctor1(($t2 = new Bridge.global.Windows.UI.Color(), $t2.A = 255, $t2.R = 187, $t2.G = 187, $t2.B = 187, $t2));



                        templateInstance_10016036eeb84fda97d06ff48e550fb6.TemplateContent = Border_67c199d075884e89b9f34c5a8a50f93f;
                        return templateInstance_10016036eeb84fda97d06ff48e550fb6;
                    });

                    Thumb_fbdc9804f0b54a93aa82d97009da672e.Template = ControlTemplate_e0c041a4bd62455d9e770e77979dec61;


                    Canvas_8657a791d83149d399287616e63b3e0f.Children.add(Button_3f742f73e8484bb881475ecbb477fef7);
                    Canvas_8657a791d83149d399287616e63b3e0f.Children.add(Button_a8b593478ed443ed9c8c131120ef1a7d);
                    Canvas_8657a791d83149d399287616e63b3e0f.Children.add(Button_64dbacbd6aaf40a79503da8377b5239e);
                    Canvas_8657a791d83149d399287616e63b3e0f.Children.add(Button_4f3a80492b2b4ec68e9b725059f34ec3);
                    Canvas_8657a791d83149d399287616e63b3e0f.Children.add(Thumb_fbdc9804f0b54a93aa82d97009da672e);

                    var Binding_1d5572a142e340c0aab2592f0f90b0ed = new Bridge.global.Windows.UI.Xaml.Data.Binding.ctor();
                    Binding_1d5572a142e340c0aab2592f0f90b0ed.Path = new Bridge.global.Windows.UI.Xaml.PropertyPath.ctor("Background");
                    var RelativeSource_513fad577eae485db8717e61384245de = new Bridge.global.Windows.UI.Xaml.Data.RelativeSource.ctor();
                    RelativeSource_513fad577eae485db8717e61384245de.Mode = Bridge.global.Windows.UI.Xaml.Data.RelativeSourceMode.TemplatedParent;

                    Binding_1d5572a142e340c0aab2592f0f90b0ed.RelativeSource = RelativeSource_513fad577eae485db8717e61384245de;


                    Binding_1d5572a142e340c0aab2592f0f90b0ed.TemplateOwner = templateInstance_27f7b90524d9426b9e8e5e857f519852;


                    Grid_dc16e46733ba4ce8a50cf72d3d37de46.Children.add(Canvas_1e5fb39faa9245d089e403c5915e96a9);
                    Grid_dc16e46733ba4ce8a50cf72d3d37de46.Children.add(Canvas_8657a791d83149d399287616e63b3e0f);


                    Bridge.global.Windows.UI.Xaml.Data.BindingOperations.SetBinding(Canvas_1e5fb39faa9245d089e403c5915e96a9, Bridge.global.Windows.UI.Xaml.Controls.Panel.BackgroundProperty, Binding_6677240f2c554d9eb754f6f282f7f891);
                    Bridge.global.Windows.UI.Xaml.Data.BindingOperations.SetBinding(Canvas_8657a791d83149d399287616e63b3e0f, Bridge.global.Windows.UI.Xaml.Controls.Panel.BackgroundProperty, Binding_1d5572a142e340c0aab2592f0f90b0ed);

                    templateInstance_27f7b90524d9426b9e8e5e857f519852.TemplateContent = Grid_dc16e46733ba4ce8a50cf72d3d37de46;
                    return templateInstance_27f7b90524d9426b9e8e5e857f519852;
                });

                Setter_22e7fbbc5a4b4bcca95ae06c77cd0577.Value = ControlTemplate_7361eebb3a3f4bf18f05f5d2ba4e9c44;


                Style_8438f3596a8c4d99850b8e4f85ac8ce8.Setters.add(Setter_8461cf4152c445b9bfe8fdb79e76650a);
                Style_8438f3596a8c4d99850b8e4f85ac8ce8.Setters.add(Setter_8f2ce6d1f2964887a0758601b2ea2ee9);
                Style_8438f3596a8c4d99850b8e4f85ac8ce8.Setters.add(Setter_22e7fbbc5a4b4bcca95ae06c77cd0577);


                this.setItem(Bridge.global.Windows.UI.Xaml.Controls.Primitives.ScrollBar, Style_8438f3596a8c4d99850b8e4f85ac8ce8);






            }
        }
    });

    Bridge.define("___CSHTML5__Component__Defaultstyles__INTERNAL_Defaultsliderstyle__Xaml", {
        inherits: [Bridge.global.Windows.UI.Xaml.ResourceDictionary],
        fields: {
            _contentLoaded: false
        },
        ctors: {
            ctor: function () {
                this.$initialize();
                Bridge.global.Windows.UI.Xaml.ResourceDictionary.ctor.call(this);
                this.InitializeComponent();

                if (Bridge.is(this, Windows.UI.Xaml.UIElement)) {
                    Bridge.cast(this, Windows.UI.Xaml.UIElement).XamlSourcePath = "CSHTML5\\DefaultStyles\\INTERNAL_DefaultSliderStyle.xaml";
                }
            }
        },
        methods: {
            InitializeComponent: function () {
                var $t;
                if (this._contentLoaded) {
                    return;
                }
                this._contentLoaded = true;


                if (Bridge.is(this, Windows.UI.Xaml.UIElement)) {
                    Bridge.cast(this, Windows.UI.Xaml.UIElement).XamlSourcePath = "CSHTML5\\DefaultStyles\\INTERNAL_DefaultSliderStyle.xaml";
                }



                var Style_e33458873f3d4d6ba1b912399da7a442 = new Bridge.global.Windows.UI.Xaml.Style.ctor();
                Style_e33458873f3d4d6ba1b912399da7a442.TargetType = Bridge.global.Windows.UI.Xaml.Controls.Slider;
                var Setter_7c4056087ca54d0197229a573fd40c57 = new Bridge.global.Windows.UI.Xaml.Setter.ctor();
                Setter_7c4056087ca54d0197229a573fd40c57.Property = Bridge.global.Windows.UI.Xaml.Controls.Control.BackgroundProperty;
                Setter_7c4056087ca54d0197229a573fd40c57.Value = new Bridge.global.Windows.UI.Xaml.Media.SolidColorBrush.$ctor1(($t = new Bridge.global.Windows.UI.Color(), $t.A = 255, $t.R = 255, $t.G = 255, $t.B = 255, $t));

                var Setter_608c31e640484162aea98461b9b8a5e2 = new Bridge.global.Windows.UI.Xaml.Setter.ctor();
                Setter_608c31e640484162aea98461b9b8a5e2.Property = Bridge.global.Windows.UI.Xaml.Controls.Control.ForegroundProperty;
                Setter_608c31e640484162aea98461b9b8a5e2.Value = new Bridge.global.Windows.UI.Xaml.Media.SolidColorBrush.$ctor1(($t = new Bridge.global.Windows.UI.Color(), $t.A = 255, $t.R = 51, $t.G = 51, $t.B = 51, $t));

                var Setter_c7fb07eee96d413193f35e6c0b334a35 = new Bridge.global.Windows.UI.Xaml.Setter.ctor();
                Setter_c7fb07eee96d413193f35e6c0b334a35.Property = Bridge.global.Windows.UI.Xaml.Controls.Control.TemplateProperty;
                var ControlTemplate_b4e2a1f389d5408db9c64208c1918750 = new Bridge.global.Windows.UI.Xaml.Controls.ControlTemplate();
                ControlTemplate_b4e2a1f389d5408db9c64208c1918750.TargetType = Bridge.global.Windows.UI.Xaml.Controls.Slider;
                ControlTemplate_b4e2a1f389d5408db9c64208c1918750.SetMethodToInstantiateFrameworkTemplate(function (templateOwner_ControlTemplate_b4e2a1f389d5408db9c64208c1918750) {
                    var $t1;
                    var templateInstance_577ed531105a40f19ba3da09da66ef9d = new Bridge.global.Windows.UI.Xaml.TemplateInstance();
                    templateInstance_577ed531105a40f19ba3da09da66ef9d.TemplateOwner = templateOwner_ControlTemplate_b4e2a1f389d5408db9c64208c1918750;
                    var VisualState_b778741baa1946218ba9c4ba68d9bb26 = new Bridge.global.Windows.UI.Xaml.VisualState();
                    var VisualState_db0953a69bd7481d99efaa1ed8d4f9a9 = new Bridge.global.Windows.UI.Xaml.VisualState();
                    var VisualState_abf0862546bc417db5d4500799ba0024 = new Bridge.global.Windows.UI.Xaml.VisualState();
                    var VisualStateGroup_5a1fb19c9fb04eec82bf100e8a28b360 = new Bridge.global.Windows.UI.Xaml.VisualStateGroup();
                    var Rectangle_702860e79ef94c82bcdd9f59fe0e3cd4 = new Bridge.global.Windows.UI.Xaml.Shapes.Rectangle();
                    var RepeatButton_e63d361a5474452aba8c009d751b4c39 = new Bridge.global.Windows.UI.Xaml.Controls.Primitives.RepeatButton();
                    var RepeatButton_d40cd9b5aad341ecb4f5cc7c08d52564 = new Bridge.global.Windows.UI.Xaml.Controls.Primitives.RepeatButton();
                    var Thumb_b9f87f3675354688a5db62c1c47791b8 = new Bridge.global.Windows.UI.Xaml.Controls.Primitives.Thumb();
                    var Canvas_bf2825a88a88413f9416f533bc13ef30 = new Bridge.global.Windows.UI.Xaml.Controls.Canvas();
                    var Rectangle_881a4b16cf7b4ec8a19ce5496fe8aba8 = new Bridge.global.Windows.UI.Xaml.Shapes.Rectangle();
                    var RepeatButton_cceffe2be23f4d32a47d0723d42021c5 = new Bridge.global.Windows.UI.Xaml.Controls.Primitives.RepeatButton();
                    var RepeatButton_959adb6e712a4cc9b4006fbd3aeb68db = new Bridge.global.Windows.UI.Xaml.Controls.Primitives.RepeatButton();
                    var Thumb_b52bbd63e4314ff49416a7bf0aad83af = new Bridge.global.Windows.UI.Xaml.Controls.Primitives.Thumb();
                    var Canvas_3dfb687573d54564aa002affbe2e17a9 = new Bridge.global.Windows.UI.Xaml.Controls.Canvas();
                    var Grid_bc672a5ee71749b0aacbb98cd999ec73 = new Bridge.global.Windows.UI.Xaml.Controls.Grid();
                    templateOwner_ControlTemplate_b4e2a1f389d5408db9c64208c1918750.RegisterName("Root", Grid_bc672a5ee71749b0aacbb98cd999ec73);
                    Grid_bc672a5ee71749b0aacbb98cd999ec73.Name = "Root";
                    templateOwner_ControlTemplate_b4e2a1f389d5408db9c64208c1918750.RegisterName("CommonStates", VisualStateGroup_5a1fb19c9fb04eec82bf100e8a28b360);
                    VisualStateGroup_5a1fb19c9fb04eec82bf100e8a28b360.Name = "CommonStates";
                    templateOwner_ControlTemplate_b4e2a1f389d5408db9c64208c1918750.RegisterName("Normal", VisualState_b778741baa1946218ba9c4ba68d9bb26);
                    VisualState_b778741baa1946218ba9c4ba68d9bb26.Name = "Normal";

                    templateOwner_ControlTemplate_b4e2a1f389d5408db9c64208c1918750.RegisterName("PointerOver", VisualState_db0953a69bd7481d99efaa1ed8d4f9a9);
                    VisualState_db0953a69bd7481d99efaa1ed8d4f9a9.Name = "PointerOver";

                    templateOwner_ControlTemplate_b4e2a1f389d5408db9c64208c1918750.RegisterName("Disabled", VisualState_abf0862546bc417db5d4500799ba0024);
                    VisualState_abf0862546bc417db5d4500799ba0024.Name = "Disabled";

                    System.Array.add(VisualStateGroup_5a1fb19c9fb04eec82bf100e8a28b360.States, VisualState_b778741baa1946218ba9c4ba68d9bb26, Object);
                    System.Array.add(VisualStateGroup_5a1fb19c9fb04eec82bf100e8a28b360.States, VisualState_db0953a69bd7481d99efaa1ed8d4f9a9, Object);
                    System.Array.add(VisualStateGroup_5a1fb19c9fb04eec82bf100e8a28b360.States, VisualState_abf0862546bc417db5d4500799ba0024, Object);


                    templateOwner_ControlTemplate_b4e2a1f389d5408db9c64208c1918750.INTERNAL_GetVisualStateGroups().add(VisualStateGroup_5a1fb19c9fb04eec82bf100e8a28b360);

                    templateOwner_ControlTemplate_b4e2a1f389d5408db9c64208c1918750.RegisterName("HorizontalRoot", Canvas_bf2825a88a88413f9416f533bc13ef30);
                    Canvas_bf2825a88a88413f9416f533bc13ef30.Name = "HorizontalRoot";
                    Canvas_bf2825a88a88413f9416f533bc13ef30.Visibility = Bridge.global.Windows.UI.Xaml.Visibility.Collapsed;
                    templateOwner_ControlTemplate_b4e2a1f389d5408db9c64208c1918750.RegisterName("HorizontalRail", Rectangle_702860e79ef94c82bcdd9f59fe0e3cd4);
                    Rectangle_702860e79ef94c82bcdd9f59fe0e3cd4.Name = "HorizontalRail";
                    Rectangle_702860e79ef94c82bcdd9f59fe0e3cd4.Fill = new Bridge.global.Windows.UI.Xaml.Media.SolidColorBrush.$ctor1(($t1 = new Bridge.global.Windows.UI.Color(), $t1.A = 255, $t1.R = 241, $t1.G = 241, $t1.B = 241, $t1));
                    Rectangle_702860e79ef94c82bcdd9f59fe0e3cd4.Stroke = new Bridge.global.Windows.UI.Xaml.Media.SolidColorBrush.$ctor1(($t1 = new Bridge.global.Windows.UI.Color(), $t1.A = 255, $t1.R = 128, $t1.G = 128, $t1.B = 128, $t1));
                    Rectangle_702860e79ef94c82bcdd9f59fe0e3cd4.StrokeThickness = 1.0;
                    Rectangle_702860e79ef94c82bcdd9f59fe0e3cd4.Height = 3.0;
                    var Binding_a23d06da7fce4ff19d4581cdf4fa373a = new Bridge.global.Windows.UI.Xaml.Data.Binding.ctor();
                    Binding_a23d06da7fce4ff19d4581cdf4fa373a.Path = new Bridge.global.Windows.UI.Xaml.PropertyPath.ctor("Width");
                    var RelativeSource_87753f218b924124a04eb64f26386d72 = new Bridge.global.Windows.UI.Xaml.Data.RelativeSource.ctor();
                    RelativeSource_87753f218b924124a04eb64f26386d72.Mode = Bridge.global.Windows.UI.Xaml.Data.RelativeSourceMode.TemplatedParent;

                    Binding_a23d06da7fce4ff19d4581cdf4fa373a.RelativeSource = RelativeSource_87753f218b924124a04eb64f26386d72;


                    Binding_a23d06da7fce4ff19d4581cdf4fa373a.TemplateOwner = templateInstance_577ed531105a40f19ba3da09da66ef9d;


                    templateOwner_ControlTemplate_b4e2a1f389d5408db9c64208c1918750.RegisterName("HorizontalLargeDecrease", RepeatButton_e63d361a5474452aba8c009d751b4c39);
                    RepeatButton_e63d361a5474452aba8c009d751b4c39.Name = "HorizontalLargeDecrease";
                    RepeatButton_e63d361a5474452aba8c009d751b4c39.Padding = new Bridge.global.Windows.UI.Xaml.Thickness.$ctor1(0);
                    RepeatButton_e63d361a5474452aba8c009d751b4c39.Cursor = Bridge.global.System.Windows.Input.Cursors.Arrow;
                    RepeatButton_e63d361a5474452aba8c009d751b4c39.Background = new Bridge.global.Windows.UI.Xaml.Media.SolidColorBrush.$ctor1(($t1 = new Bridge.global.Windows.UI.Color(), $t1.A = 0, $t1.R = 255, $t1.G = 255, $t1.B = 255, $t1));

                    templateOwner_ControlTemplate_b4e2a1f389d5408db9c64208c1918750.RegisterName("HorizontalLargeIncrease", RepeatButton_d40cd9b5aad341ecb4f5cc7c08d52564);
                    RepeatButton_d40cd9b5aad341ecb4f5cc7c08d52564.Name = "HorizontalLargeIncrease";
                    RepeatButton_d40cd9b5aad341ecb4f5cc7c08d52564.Padding = new Bridge.global.Windows.UI.Xaml.Thickness.$ctor1(0);
                    RepeatButton_d40cd9b5aad341ecb4f5cc7c08d52564.Cursor = Bridge.global.System.Windows.Input.Cursors.Arrow;
                    RepeatButton_d40cd9b5aad341ecb4f5cc7c08d52564.Background = new Bridge.global.Windows.UI.Xaml.Media.SolidColorBrush.$ctor1(($t1 = new Bridge.global.Windows.UI.Color(), $t1.A = 0, $t1.R = 255, $t1.G = 255, $t1.B = 255, $t1));

                    templateOwner_ControlTemplate_b4e2a1f389d5408db9c64208c1918750.RegisterName("HorizontalThumb", Thumb_b9f87f3675354688a5db62c1c47791b8);
                    Thumb_b9f87f3675354688a5db62c1c47791b8.Name = "HorizontalThumb";
                    Thumb_b9f87f3675354688a5db62c1c47791b8.Cursor = Bridge.global.System.Windows.Input.Cursors.Arrow;
                    var ControlTemplate_346067e70f7b411e9ad0a62685fb3207 = new Bridge.global.Windows.UI.Xaml.Controls.ControlTemplate();
                    ControlTemplate_346067e70f7b411e9ad0a62685fb3207.SetMethodToInstantiateFrameworkTemplate(function (templateOwner_ControlTemplate_346067e70f7b411e9ad0a62685fb3207) {
                        var $t2;
                        var templateInstance_8b166ccca57f4ff3af6dcdd6b1e8e3ec = new Bridge.global.Windows.UI.Xaml.TemplateInstance();
                        templateInstance_8b166ccca57f4ff3af6dcdd6b1e8e3ec.TemplateOwner = templateOwner_ControlTemplate_346067e70f7b411e9ad0a62685fb3207;

                        var Border_5f4856d496504b8caa1065f50a644236 = new Bridge.global.Windows.UI.Xaml.Controls.Border();
                        Border_5f4856d496504b8caa1065f50a644236.BorderBrush = new Bridge.global.Windows.UI.Xaml.Media.SolidColorBrush.$ctor1(($t2 = new Bridge.global.Windows.UI.Color(), $t2.A = 255, $t2.R = 0, $t2.G = 0, $t2.B = 0, $t2));
                        Border_5f4856d496504b8caa1065f50a644236.BorderThickness = new Bridge.global.Windows.UI.Xaml.Thickness.$ctor1(1);
                        Border_5f4856d496504b8caa1065f50a644236.Width = 9.0;
                        Border_5f4856d496504b8caa1065f50a644236.Height = 18.0;
                        Border_5f4856d496504b8caa1065f50a644236.CornerRadius = new Bridge.global.Windows.UI.Xaml.CornerRadius.$ctor1(2);
                        var Border_d08021a39eae404aa43f9f607d3557e4 = new Bridge.global.Windows.UI.Xaml.Controls.Border();
                        Border_d08021a39eae404aa43f9f607d3557e4.BorderBrush = new Bridge.global.Windows.UI.Xaml.Media.SolidColorBrush.$ctor1(($t2 = new Bridge.global.Windows.UI.Color(), $t2.A = 255, $t2.R = 255, $t2.G = 255, $t2.B = 255, $t2));
                        Border_d08021a39eae404aa43f9f607d3557e4.BorderThickness = new Bridge.global.Windows.UI.Xaml.Thickness.$ctor1(1);
                        var LinearGradientBrush_02232847dd1c4093a47b666b44905cc0 = new Bridge.global.Windows.UI.Xaml.Media.LinearGradientBrush.ctor();
                        LinearGradientBrush_02232847dd1c4093a47b666b44905cc0.StartPoint = new Bridge.global.Windows.Foundation.Point.$ctor1(0.5, 0);
                        LinearGradientBrush_02232847dd1c4093a47b666b44905cc0.EndPoint = new Bridge.global.Windows.Foundation.Point.$ctor1(0.5, 1);
                        var GradientStopCollection_01bd49108e6b4449a0bef684713f72ba = new Bridge.global.Windows.UI.Xaml.Media.GradientStopCollection();
                        var GradientStop_dc1087a359d0493fb05d283442694ae6 = new Bridge.global.Windows.UI.Xaml.Media.GradientStop();
                        GradientStop_dc1087a359d0493fb05d283442694ae6.Color = ($t2 = new Bridge.global.Windows.UI.Color(), $t2.A = 255, $t2.R = 255, $t2.G = 255, $t2.B = 255, $t2);
                        GradientStop_dc1087a359d0493fb05d283442694ae6.Offset = 0.0;

                        var GradientStop_1126141cba4d4f43b5f607688dbfa37f = new Bridge.global.Windows.UI.Xaml.Media.GradientStop();
                        GradientStop_1126141cba4d4f43b5f607688dbfa37f.Color = ($t2 = new Bridge.global.Windows.UI.Color(), $t2.A = 255, $t2.R = 211, $t2.G = 211, $t2.B = 211, $t2);
                        GradientStop_1126141cba4d4f43b5f607688dbfa37f.Offset = 1.0;

                        GradientStopCollection_01bd49108e6b4449a0bef684713f72ba.add(GradientStop_dc1087a359d0493fb05d283442694ae6);
                        GradientStopCollection_01bd49108e6b4449a0bef684713f72ba.add(GradientStop_1126141cba4d4f43b5f607688dbfa37f);

                        LinearGradientBrush_02232847dd1c4093a47b666b44905cc0.GradientStops = GradientStopCollection_01bd49108e6b4449a0bef684713f72ba;


                        Border_d08021a39eae404aa43f9f607d3557e4.Background = LinearGradientBrush_02232847dd1c4093a47b666b44905cc0;


                        Border_5f4856d496504b8caa1065f50a644236.Child = Border_d08021a39eae404aa43f9f607d3557e4;




                        templateInstance_8b166ccca57f4ff3af6dcdd6b1e8e3ec.TemplateContent = Border_5f4856d496504b8caa1065f50a644236;
                        return templateInstance_8b166ccca57f4ff3af6dcdd6b1e8e3ec;
                    });

                    Thumb_b9f87f3675354688a5db62c1c47791b8.Template = ControlTemplate_346067e70f7b411e9ad0a62685fb3207;


                    Canvas_bf2825a88a88413f9416f533bc13ef30.Children.add(Rectangle_702860e79ef94c82bcdd9f59fe0e3cd4);
                    Canvas_bf2825a88a88413f9416f533bc13ef30.Children.add(RepeatButton_e63d361a5474452aba8c009d751b4c39);
                    Canvas_bf2825a88a88413f9416f533bc13ef30.Children.add(RepeatButton_d40cd9b5aad341ecb4f5cc7c08d52564);
                    Canvas_bf2825a88a88413f9416f533bc13ef30.Children.add(Thumb_b9f87f3675354688a5db62c1c47791b8);

                    var Binding_89115005345f4384b6c479ae1266e45c = new Bridge.global.Windows.UI.Xaml.Data.Binding.ctor();
                    Binding_89115005345f4384b6c479ae1266e45c.Path = new Bridge.global.Windows.UI.Xaml.PropertyPath.ctor("Background");
                    var RelativeSource_94e1eae3f0c14f7ca475b9cd3b56294f = new Bridge.global.Windows.UI.Xaml.Data.RelativeSource.ctor();
                    RelativeSource_94e1eae3f0c14f7ca475b9cd3b56294f.Mode = Bridge.global.Windows.UI.Xaml.Data.RelativeSourceMode.TemplatedParent;

                    Binding_89115005345f4384b6c479ae1266e45c.RelativeSource = RelativeSource_94e1eae3f0c14f7ca475b9cd3b56294f;


                    Binding_89115005345f4384b6c479ae1266e45c.TemplateOwner = templateInstance_577ed531105a40f19ba3da09da66ef9d;

                    var Binding_e0346b2762d148469b291ebb20878ab4 = new Bridge.global.Windows.UI.Xaml.Data.Binding.ctor();
                    Binding_e0346b2762d148469b291ebb20878ab4.Path = new Bridge.global.Windows.UI.Xaml.PropertyPath.ctor("Width");
                    var RelativeSource_e3c9e41ae9c8479c92bdcfd43590316c = new Bridge.global.Windows.UI.Xaml.Data.RelativeSource.ctor();
                    RelativeSource_e3c9e41ae9c8479c92bdcfd43590316c.Mode = Bridge.global.Windows.UI.Xaml.Data.RelativeSourceMode.TemplatedParent;

                    Binding_e0346b2762d148469b291ebb20878ab4.RelativeSource = RelativeSource_e3c9e41ae9c8479c92bdcfd43590316c;


                    Binding_e0346b2762d148469b291ebb20878ab4.TemplateOwner = templateInstance_577ed531105a40f19ba3da09da66ef9d;

                    var Binding_222e97f586ed4c94bb818379229fc3fa = new Bridge.global.Windows.UI.Xaml.Data.Binding.ctor();
                    Binding_222e97f586ed4c94bb818379229fc3fa.Path = new Bridge.global.Windows.UI.Xaml.PropertyPath.ctor("Height");
                    var RelativeSource_4727c6af558a4a27814df8c899b7b2f9 = new Bridge.global.Windows.UI.Xaml.Data.RelativeSource.ctor();
                    RelativeSource_4727c6af558a4a27814df8c899b7b2f9.Mode = Bridge.global.Windows.UI.Xaml.Data.RelativeSourceMode.TemplatedParent;

                    Binding_222e97f586ed4c94bb818379229fc3fa.RelativeSource = RelativeSource_4727c6af558a4a27814df8c899b7b2f9;


                    Binding_222e97f586ed4c94bb818379229fc3fa.TemplateOwner = templateInstance_577ed531105a40f19ba3da09da66ef9d;


                    templateOwner_ControlTemplate_b4e2a1f389d5408db9c64208c1918750.RegisterName("VerticalRoot", Canvas_3dfb687573d54564aa002affbe2e17a9);
                    Canvas_3dfb687573d54564aa002affbe2e17a9.Name = "VerticalRoot";
                    Canvas_3dfb687573d54564aa002affbe2e17a9.Visibility = Bridge.global.Windows.UI.Xaml.Visibility.Collapsed;
                    templateOwner_ControlTemplate_b4e2a1f389d5408db9c64208c1918750.RegisterName("VerticalRail", Rectangle_881a4b16cf7b4ec8a19ce5496fe8aba8);
                    Rectangle_881a4b16cf7b4ec8a19ce5496fe8aba8.Name = "VerticalRail";
                    Rectangle_881a4b16cf7b4ec8a19ce5496fe8aba8.Fill = new Bridge.global.Windows.UI.Xaml.Media.SolidColorBrush.$ctor1(($t1 = new Bridge.global.Windows.UI.Color(), $t1.A = 255, $t1.R = 241, $t1.G = 241, $t1.B = 241, $t1));
                    Rectangle_881a4b16cf7b4ec8a19ce5496fe8aba8.Stroke = new Bridge.global.Windows.UI.Xaml.Media.SolidColorBrush.$ctor1(($t1 = new Bridge.global.Windows.UI.Color(), $t1.A = 255, $t1.R = 128, $t1.G = 128, $t1.B = 128, $t1));
                    Rectangle_881a4b16cf7b4ec8a19ce5496fe8aba8.StrokeThickness = 1.0;
                    Rectangle_881a4b16cf7b4ec8a19ce5496fe8aba8.Width = 3.0;
                    var Binding_a8ccf5860a574d5999ad7b41ddccf8f1 = new Bridge.global.Windows.UI.Xaml.Data.Binding.ctor();
                    Binding_a8ccf5860a574d5999ad7b41ddccf8f1.Path = new Bridge.global.Windows.UI.Xaml.PropertyPath.ctor("Height");
                    var RelativeSource_20391c311f2340588a089947323ecb26 = new Bridge.global.Windows.UI.Xaml.Data.RelativeSource.ctor();
                    RelativeSource_20391c311f2340588a089947323ecb26.Mode = Bridge.global.Windows.UI.Xaml.Data.RelativeSourceMode.TemplatedParent;

                    Binding_a8ccf5860a574d5999ad7b41ddccf8f1.RelativeSource = RelativeSource_20391c311f2340588a089947323ecb26;


                    Binding_a8ccf5860a574d5999ad7b41ddccf8f1.TemplateOwner = templateInstance_577ed531105a40f19ba3da09da66ef9d;


                    templateOwner_ControlTemplate_b4e2a1f389d5408db9c64208c1918750.RegisterName("VerticalLargeDecrease", RepeatButton_cceffe2be23f4d32a47d0723d42021c5);
                    RepeatButton_cceffe2be23f4d32a47d0723d42021c5.Name = "VerticalLargeDecrease";
                    RepeatButton_cceffe2be23f4d32a47d0723d42021c5.Padding = new Bridge.global.Windows.UI.Xaml.Thickness.$ctor1(0);
                    RepeatButton_cceffe2be23f4d32a47d0723d42021c5.Cursor = Bridge.global.System.Windows.Input.Cursors.Arrow;
                    RepeatButton_cceffe2be23f4d32a47d0723d42021c5.Background = new Bridge.global.Windows.UI.Xaml.Media.SolidColorBrush.$ctor1(($t1 = new Bridge.global.Windows.UI.Color(), $t1.A = 0, $t1.R = 255, $t1.G = 255, $t1.B = 255, $t1));

                    templateOwner_ControlTemplate_b4e2a1f389d5408db9c64208c1918750.RegisterName("VerticalLargeIncrease", RepeatButton_959adb6e712a4cc9b4006fbd3aeb68db);
                    RepeatButton_959adb6e712a4cc9b4006fbd3aeb68db.Name = "VerticalLargeIncrease";
                    RepeatButton_959adb6e712a4cc9b4006fbd3aeb68db.Padding = new Bridge.global.Windows.UI.Xaml.Thickness.$ctor1(0);
                    RepeatButton_959adb6e712a4cc9b4006fbd3aeb68db.Cursor = Bridge.global.System.Windows.Input.Cursors.Arrow;
                    RepeatButton_959adb6e712a4cc9b4006fbd3aeb68db.Background = new Bridge.global.Windows.UI.Xaml.Media.SolidColorBrush.$ctor1(($t1 = new Bridge.global.Windows.UI.Color(), $t1.A = 0, $t1.R = 255, $t1.G = 255, $t1.B = 255, $t1));

                    templateOwner_ControlTemplate_b4e2a1f389d5408db9c64208c1918750.RegisterName("VerticalThumb", Thumb_b52bbd63e4314ff49416a7bf0aad83af);
                    Thumb_b52bbd63e4314ff49416a7bf0aad83af.Name = "VerticalThumb";
                    Thumb_b52bbd63e4314ff49416a7bf0aad83af.Background = new Bridge.global.Windows.UI.Xaml.Media.SolidColorBrush.$ctor1(($t1 = new Bridge.global.Windows.UI.Color(), $t1.A = 255, $t1.R = 187, $t1.G = 187, $t1.B = 187, $t1));
                    Thumb_b52bbd63e4314ff49416a7bf0aad83af.Cursor = Bridge.global.System.Windows.Input.Cursors.Arrow;
                    var ControlTemplate_7888b68a349b4beab151d0115fc43ea7 = new Bridge.global.Windows.UI.Xaml.Controls.ControlTemplate();
                    ControlTemplate_7888b68a349b4beab151d0115fc43ea7.SetMethodToInstantiateFrameworkTemplate(function (templateOwner_ControlTemplate_7888b68a349b4beab151d0115fc43ea7) {
                        var $t2;
                        var templateInstance_2312a89cc8d746aeb0f5c5614099a5ef = new Bridge.global.Windows.UI.Xaml.TemplateInstance();
                        templateInstance_2312a89cc8d746aeb0f5c5614099a5ef.TemplateOwner = templateOwner_ControlTemplate_7888b68a349b4beab151d0115fc43ea7;

                        var Border_05c33d1c59ed455d837ee9b273cadda7 = new Bridge.global.Windows.UI.Xaml.Controls.Border();
                        Border_05c33d1c59ed455d837ee9b273cadda7.BorderBrush = new Bridge.global.Windows.UI.Xaml.Media.SolidColorBrush.$ctor1(($t2 = new Bridge.global.Windows.UI.Color(), $t2.A = 255, $t2.R = 0, $t2.G = 0, $t2.B = 0, $t2));
                        Border_05c33d1c59ed455d837ee9b273cadda7.BorderThickness = new Bridge.global.Windows.UI.Xaml.Thickness.$ctor1(1);
                        Border_05c33d1c59ed455d837ee9b273cadda7.Width = 18.0;
                        Border_05c33d1c59ed455d837ee9b273cadda7.Height = 9.0;
                        Border_05c33d1c59ed455d837ee9b273cadda7.CornerRadius = new Bridge.global.Windows.UI.Xaml.CornerRadius.$ctor1(2);
                        var Border_42c9e4decc3d46e788bc960b3a4b3ce4 = new Bridge.global.Windows.UI.Xaml.Controls.Border();
                        Border_42c9e4decc3d46e788bc960b3a4b3ce4.BorderBrush = new Bridge.global.Windows.UI.Xaml.Media.SolidColorBrush.$ctor1(($t2 = new Bridge.global.Windows.UI.Color(), $t2.A = 255, $t2.R = 255, $t2.G = 255, $t2.B = 255, $t2));
                        Border_42c9e4decc3d46e788bc960b3a4b3ce4.BorderThickness = new Bridge.global.Windows.UI.Xaml.Thickness.$ctor1(1);
                        var LinearGradientBrush_53c8c62bab4749888d6584e21fa51009 = new Bridge.global.Windows.UI.Xaml.Media.LinearGradientBrush.ctor();
                        LinearGradientBrush_53c8c62bab4749888d6584e21fa51009.StartPoint = new Bridge.global.Windows.Foundation.Point.$ctor1(0.5, 0);
                        LinearGradientBrush_53c8c62bab4749888d6584e21fa51009.EndPoint = new Bridge.global.Windows.Foundation.Point.$ctor1(0.5, 1);
                        var GradientStopCollection_c7c8b3a102bf4e058dbfcd5cfbe043f2 = new Bridge.global.Windows.UI.Xaml.Media.GradientStopCollection();
                        var GradientStop_21235d37cb5b429cb0a6b6685dd90a12 = new Bridge.global.Windows.UI.Xaml.Media.GradientStop();
                        GradientStop_21235d37cb5b429cb0a6b6685dd90a12.Color = ($t2 = new Bridge.global.Windows.UI.Color(), $t2.A = 255, $t2.R = 255, $t2.G = 255, $t2.B = 255, $t2);
                        GradientStop_21235d37cb5b429cb0a6b6685dd90a12.Offset = 0.0;

                        var GradientStop_0ba935813bb943cea99abc6523626bb9 = new Bridge.global.Windows.UI.Xaml.Media.GradientStop();
                        GradientStop_0ba935813bb943cea99abc6523626bb9.Color = ($t2 = new Bridge.global.Windows.UI.Color(), $t2.A = 255, $t2.R = 211, $t2.G = 211, $t2.B = 211, $t2);
                        GradientStop_0ba935813bb943cea99abc6523626bb9.Offset = 1.0;

                        GradientStopCollection_c7c8b3a102bf4e058dbfcd5cfbe043f2.add(GradientStop_21235d37cb5b429cb0a6b6685dd90a12);
                        GradientStopCollection_c7c8b3a102bf4e058dbfcd5cfbe043f2.add(GradientStop_0ba935813bb943cea99abc6523626bb9);

                        LinearGradientBrush_53c8c62bab4749888d6584e21fa51009.GradientStops = GradientStopCollection_c7c8b3a102bf4e058dbfcd5cfbe043f2;


                        Border_42c9e4decc3d46e788bc960b3a4b3ce4.Background = LinearGradientBrush_53c8c62bab4749888d6584e21fa51009;


                        Border_05c33d1c59ed455d837ee9b273cadda7.Child = Border_42c9e4decc3d46e788bc960b3a4b3ce4;




                        templateInstance_2312a89cc8d746aeb0f5c5614099a5ef.TemplateContent = Border_05c33d1c59ed455d837ee9b273cadda7;
                        return templateInstance_2312a89cc8d746aeb0f5c5614099a5ef;
                    });

                    Thumb_b52bbd63e4314ff49416a7bf0aad83af.Template = ControlTemplate_7888b68a349b4beab151d0115fc43ea7;


                    Canvas_3dfb687573d54564aa002affbe2e17a9.Children.add(Rectangle_881a4b16cf7b4ec8a19ce5496fe8aba8);
                    Canvas_3dfb687573d54564aa002affbe2e17a9.Children.add(RepeatButton_cceffe2be23f4d32a47d0723d42021c5);
                    Canvas_3dfb687573d54564aa002affbe2e17a9.Children.add(RepeatButton_959adb6e712a4cc9b4006fbd3aeb68db);
                    Canvas_3dfb687573d54564aa002affbe2e17a9.Children.add(Thumb_b52bbd63e4314ff49416a7bf0aad83af);

                    var Binding_b122698b392848dd8f5200dd5a2652f9 = new Bridge.global.Windows.UI.Xaml.Data.Binding.ctor();
                    Binding_b122698b392848dd8f5200dd5a2652f9.Path = new Bridge.global.Windows.UI.Xaml.PropertyPath.ctor("Background");
                    var RelativeSource_be20dd22d43644ca9f2c76308496b9ea = new Bridge.global.Windows.UI.Xaml.Data.RelativeSource.ctor();
                    RelativeSource_be20dd22d43644ca9f2c76308496b9ea.Mode = Bridge.global.Windows.UI.Xaml.Data.RelativeSourceMode.TemplatedParent;

                    Binding_b122698b392848dd8f5200dd5a2652f9.RelativeSource = RelativeSource_be20dd22d43644ca9f2c76308496b9ea;


                    Binding_b122698b392848dd8f5200dd5a2652f9.TemplateOwner = templateInstance_577ed531105a40f19ba3da09da66ef9d;


                    Grid_bc672a5ee71749b0aacbb98cd999ec73.Children.add(Canvas_bf2825a88a88413f9416f533bc13ef30);
                    Grid_bc672a5ee71749b0aacbb98cd999ec73.Children.add(Canvas_3dfb687573d54564aa002affbe2e17a9);


                    Bridge.global.Windows.UI.Xaml.Data.BindingOperations.SetBinding(Rectangle_702860e79ef94c82bcdd9f59fe0e3cd4, Bridge.global.Windows.UI.Xaml.FrameworkElement.WidthProperty, Binding_a23d06da7fce4ff19d4581cdf4fa373a);
                    Bridge.global.Windows.UI.Xaml.Data.BindingOperations.SetBinding(Canvas_bf2825a88a88413f9416f533bc13ef30, Bridge.global.Windows.UI.Xaml.Controls.Panel.BackgroundProperty, Binding_89115005345f4384b6c479ae1266e45c);
                    Bridge.global.Windows.UI.Xaml.Data.BindingOperations.SetBinding(Canvas_bf2825a88a88413f9416f533bc13ef30, Bridge.global.Windows.UI.Xaml.FrameworkElement.WidthProperty, Binding_e0346b2762d148469b291ebb20878ab4);
                    Bridge.global.Windows.UI.Xaml.Data.BindingOperations.SetBinding(Canvas_bf2825a88a88413f9416f533bc13ef30, Bridge.global.Windows.UI.Xaml.FrameworkElement.HeightProperty, Binding_222e97f586ed4c94bb818379229fc3fa);
                    Bridge.global.Windows.UI.Xaml.Data.BindingOperations.SetBinding(Rectangle_881a4b16cf7b4ec8a19ce5496fe8aba8, Bridge.global.Windows.UI.Xaml.FrameworkElement.HeightProperty, Binding_a8ccf5860a574d5999ad7b41ddccf8f1);
                    Bridge.global.Windows.UI.Xaml.Data.BindingOperations.SetBinding(Canvas_3dfb687573d54564aa002affbe2e17a9, Bridge.global.Windows.UI.Xaml.Controls.Panel.BackgroundProperty, Binding_b122698b392848dd8f5200dd5a2652f9);

                    templateInstance_577ed531105a40f19ba3da09da66ef9d.TemplateContent = Grid_bc672a5ee71749b0aacbb98cd999ec73;
                    return templateInstance_577ed531105a40f19ba3da09da66ef9d;
                });

                Setter_c7fb07eee96d413193f35e6c0b334a35.Value = ControlTemplate_b4e2a1f389d5408db9c64208c1918750;


                Style_e33458873f3d4d6ba1b912399da7a442.Setters.add(Setter_7c4056087ca54d0197229a573fd40c57);
                Style_e33458873f3d4d6ba1b912399da7a442.Setters.add(Setter_608c31e640484162aea98461b9b8a5e2);
                Style_e33458873f3d4d6ba1b912399da7a442.Setters.add(Setter_c7fb07eee96d413193f35e6c0b334a35);


                this.setItem(Bridge.global.Windows.UI.Xaml.Controls.Slider, Style_e33458873f3d4d6ba1b912399da7a442);






            }
        }
    });

    Bridge.define("___CSHTML5__Component__Defaultstyles__INTERNAL_Defaulttabcontrolstyle__Xaml", {
        inherits: [Bridge.global.Windows.UI.Xaml.ResourceDictionary],
        fields: {
            _contentLoaded: false
        },
        ctors: {
            ctor: function () {
                this.$initialize();
                Bridge.global.Windows.UI.Xaml.ResourceDictionary.ctor.call(this);
                this.InitializeComponent();

                if (Bridge.is(this, Windows.UI.Xaml.UIElement)) {
                    Bridge.cast(this, Windows.UI.Xaml.UIElement).XamlSourcePath = "CSHTML5\\DefaultStyles\\INTERNAL_DefaultTabControlStyle.xaml";
                }
            }
        },
        methods: {
            InitializeComponent: function () {
                var $t;
                if (this._contentLoaded) {
                    return;
                }
                this._contentLoaded = true;


                if (Bridge.is(this, Windows.UI.Xaml.UIElement)) {
                    Bridge.cast(this, Windows.UI.Xaml.UIElement).XamlSourcePath = "CSHTML5\\DefaultStyles\\INTERNAL_DefaultTabControlStyle.xaml";
                }



                var Style_42eb0a345e4b4160a95287ffe4f2b862 = new Bridge.global.Windows.UI.Xaml.Style.ctor();
                Style_42eb0a345e4b4160a95287ffe4f2b862.TargetType = Bridge.global.Windows.UI.Xaml.Controls.TabControl;
                var Setter_d2313654941b425cb262d8730aee0ea5 = new Bridge.global.Windows.UI.Xaml.Setter.ctor();
                Setter_d2313654941b425cb262d8730aee0ea5.Property = Bridge.global.Windows.UI.Xaml.Controls.Control.BackgroundProperty;
                Setter_d2313654941b425cb262d8730aee0ea5.Value = new Bridge.global.Windows.UI.Xaml.Media.SolidColorBrush.$ctor1(($t = new Bridge.global.Windows.UI.Color(), $t.A = 255, $t.R = 255, $t.G = 255, $t.B = 255, $t));

                var Setter_51f4c81f8a9049a8b3c47f576a0a140f = new Bridge.global.Windows.UI.Xaml.Setter.ctor();
                Setter_51f4c81f8a9049a8b3c47f576a0a140f.Property = Bridge.global.Windows.UI.Xaml.Controls.Control.BorderBrushProperty;
                Setter_51f4c81f8a9049a8b3c47f576a0a140f.Value = new Bridge.global.Windows.UI.Xaml.Media.SolidColorBrush.$ctor1(($t = new Bridge.global.Windows.UI.Color(), $t.A = 255, $t.R = 221, $t.G = 221, $t.B = 221, $t));

                var Setter_c9bd6a13be1f4dfea91b88f5a7eb1d28 = new Bridge.global.Windows.UI.Xaml.Setter.ctor();
                Setter_c9bd6a13be1f4dfea91b88f5a7eb1d28.Property = Bridge.global.Windows.UI.Xaml.Controls.Control.BorderThicknessProperty;
                Setter_c9bd6a13be1f4dfea91b88f5a7eb1d28.Value = new Bridge.global.Windows.UI.Xaml.Thickness.$ctor2(1, 1, 1, 1).$clone();

                var Setter_fd27dbdf489d4b69aba1c6d546bceb74 = new Bridge.global.Windows.UI.Xaml.Setter.ctor();
                Setter_fd27dbdf489d4b69aba1c6d546bceb74.Property = Bridge.global.Windows.UI.Xaml.Controls.Control.PaddingProperty;
                Setter_fd27dbdf489d4b69aba1c6d546bceb74.Value = new Bridge.global.Windows.UI.Xaml.Thickness.$ctor1(5).$clone();

                var Setter_59d5ccb504d6489293ee0fb4bca6c57b = new Bridge.global.Windows.UI.Xaml.Setter.ctor();
                Setter_59d5ccb504d6489293ee0fb4bca6c57b.Property = Bridge.global.Windows.UI.Xaml.Controls.Control.HorizontalContentAlignmentProperty;
                Setter_59d5ccb504d6489293ee0fb4bca6c57b.Value = Bridge.box(Bridge.global.Windows.UI.Xaml.HorizontalAlignment.Stretch, Windows.UI.Xaml.HorizontalAlignment, System.Enum.toStringFn(Windows.UI.Xaml.HorizontalAlignment));

                var Setter_393321a379264cc4b6c9d90c432f34e3 = new Bridge.global.Windows.UI.Xaml.Setter.ctor();
                Setter_393321a379264cc4b6c9d90c432f34e3.Property = Bridge.global.Windows.UI.Xaml.Controls.Control.VerticalContentAlignmentProperty;
                Setter_393321a379264cc4b6c9d90c432f34e3.Value = Bridge.box(Bridge.global.Windows.UI.Xaml.VerticalAlignment.Stretch, Windows.UI.Xaml.VerticalAlignment, System.Enum.toStringFn(Windows.UI.Xaml.VerticalAlignment));

                var Setter_cb81f967761946bc954b4dca2c8ec095 = new Bridge.global.Windows.UI.Xaml.Setter.ctor();
                Setter_cb81f967761946bc954b4dca2c8ec095.Property = Bridge.global.Windows.UI.Xaml.Controls.Control.TemplateProperty;
                var ControlTemplate_661c05b426b848df9f15bade368f37da = new Bridge.global.Windows.UI.Xaml.Controls.ControlTemplate();
                ControlTemplate_661c05b426b848df9f15bade368f37da.TargetType = Bridge.global.Windows.UI.Xaml.Controls.TabControl;
                ControlTemplate_661c05b426b848df9f15bade368f37da.SetMethodToInstantiateFrameworkTemplate(function (templateOwner_ControlTemplate_661c05b426b848df9f15bade368f37da) {
                    var templateInstance_58e85f93b10d435ca20291d55ec06dfe = new Bridge.global.Windows.UI.Xaml.TemplateInstance();
                    templateInstance_58e85f93b10d435ca20291d55ec06dfe.TemplateOwner = templateOwner_ControlTemplate_661c05b426b848df9f15bade368f37da;
                    var VisualState_599e095a13454186bcaece851f221944 = new Bridge.global.Windows.UI.Xaml.VisualState();
                    var VisualState_9d7b032dce364c9ea1a4e97b4be75201 = new Bridge.global.Windows.UI.Xaml.VisualState();
                    var VisualStateGroup_c746c1a1ab4543b69a1705789cfbe68f = new Bridge.global.Windows.UI.Xaml.VisualStateGroup();
                    var TabPanel_1aaaf84b99d149fbac9ceb6f0ce9b07b = new Bridge.global.Windows.UI.Xaml.Controls.TabPanel();
                    var ContentPresenter_d82ceb1c5f364335ac106082c6a2177a = new Bridge.global.Windows.UI.Xaml.Controls.ContentPresenter();
                    var Grid_244f203a38004a8280244bb5e57e69b0 = new Bridge.global.Windows.UI.Xaml.Controls.Grid();
                    var Border_f9b213b546324f218fe2dbd9d7f022fd = new Bridge.global.Windows.UI.Xaml.Controls.Border();
                    templateOwner_ControlTemplate_661c05b426b848df9f15bade368f37da.RegisterName("CommonStates", VisualStateGroup_c746c1a1ab4543b69a1705789cfbe68f);
                    VisualStateGroup_c746c1a1ab4543b69a1705789cfbe68f.Name = "CommonStates";
                    templateOwner_ControlTemplate_661c05b426b848df9f15bade368f37da.RegisterName("Normal", VisualState_599e095a13454186bcaece851f221944);
                    VisualState_599e095a13454186bcaece851f221944.Name = "Normal";

                    templateOwner_ControlTemplate_661c05b426b848df9f15bade368f37da.RegisterName("Disabled", VisualState_9d7b032dce364c9ea1a4e97b4be75201);
                    VisualState_9d7b032dce364c9ea1a4e97b4be75201.Name = "Disabled";

                    System.Array.add(VisualStateGroup_c746c1a1ab4543b69a1705789cfbe68f.States, VisualState_599e095a13454186bcaece851f221944, Object);
                    System.Array.add(VisualStateGroup_c746c1a1ab4543b69a1705789cfbe68f.States, VisualState_9d7b032dce364c9ea1a4e97b4be75201, Object);


                    templateOwner_ControlTemplate_661c05b426b848df9f15bade368f37da.INTERNAL_GetVisualStateGroups().add(VisualStateGroup_c746c1a1ab4543b69a1705789cfbe68f);

                    templateOwner_ControlTemplate_661c05b426b848df9f15bade368f37da.RegisterName("TemplateTop", Grid_244f203a38004a8280244bb5e57e69b0);
                    Grid_244f203a38004a8280244bb5e57e69b0.Name = "TemplateTop";
                    Grid_244f203a38004a8280244bb5e57e69b0.Visibility = Bridge.global.Windows.UI.Xaml.Visibility.Collapsed;
                    var RowDefinition_756311ac0a7c402685f77d2dab065e3f = new Bridge.global.Windows.UI.Xaml.Controls.RowDefinition();
                    RowDefinition_756311ac0a7c402685f77d2dab065e3f.Height = new Bridge.global.Windows.UI.Xaml.GridLength.$ctor2(1.0, Bridge.global.Windows.UI.Xaml.GridUnitType.Auto);

                    var RowDefinition_312009e759a849f6b0a6ca2e3d0b06f5 = new Bridge.global.Windows.UI.Xaml.Controls.RowDefinition();
                    RowDefinition_312009e759a849f6b0a6ca2e3d0b06f5.Height = new Bridge.global.Windows.UI.Xaml.GridLength.$ctor2(1.0, Bridge.global.Windows.UI.Xaml.GridUnitType.Star);

                    Grid_244f203a38004a8280244bb5e57e69b0.RowDefinitions.add(RowDefinition_756311ac0a7c402685f77d2dab065e3f);
                    Grid_244f203a38004a8280244bb5e57e69b0.RowDefinitions.add(RowDefinition_312009e759a849f6b0a6ca2e3d0b06f5);

                    templateOwner_ControlTemplate_661c05b426b848df9f15bade368f37da.RegisterName("TabPanelTop", TabPanel_1aaaf84b99d149fbac9ceb6f0ce9b07b);
                    TabPanel_1aaaf84b99d149fbac9ceb6f0ce9b07b.Name = "TabPanelTop";

                    var Border_935ce328cc644056b00abdf139db85b8 = new Bridge.global.Windows.UI.Xaml.Controls.Border();
                    Bridge.global.Windows.UI.Xaml.Controls.Grid.SetRow(Border_935ce328cc644056b00abdf139db85b8, 1);
                    Border_935ce328cc644056b00abdf139db85b8.CornerRadius = new Bridge.global.Windows.UI.Xaml.CornerRadius.$ctor2(0, 0, 3, 3);
                    templateOwner_ControlTemplate_661c05b426b848df9f15bade368f37da.RegisterName("ContentTop", ContentPresenter_d82ceb1c5f364335ac106082c6a2177a);
                    ContentPresenter_d82ceb1c5f364335ac106082c6a2177a.Name = "ContentTop";
                    var Binding_b9dcd88265864bcfbc03ed83443089f9 = new Bridge.global.Windows.UI.Xaml.Data.Binding.ctor();
                    Binding_b9dcd88265864bcfbc03ed83443089f9.Path = new Bridge.global.Windows.UI.Xaml.PropertyPath.ctor("HorizontalContentAlignment");
                    var RelativeSource_f9a3273a2c2b4fcdaaeaea476b64afd6 = new Bridge.global.Windows.UI.Xaml.Data.RelativeSource.ctor();
                    RelativeSource_f9a3273a2c2b4fcdaaeaea476b64afd6.Mode = Bridge.global.Windows.UI.Xaml.Data.RelativeSourceMode.TemplatedParent;

                    Binding_b9dcd88265864bcfbc03ed83443089f9.RelativeSource = RelativeSource_f9a3273a2c2b4fcdaaeaea476b64afd6;


                    Binding_b9dcd88265864bcfbc03ed83443089f9.TemplateOwner = templateInstance_58e85f93b10d435ca20291d55ec06dfe;

                    var Binding_40e3ca9e85c64b02a72eace62503834b = new Bridge.global.Windows.UI.Xaml.Data.Binding.ctor();
                    Binding_40e3ca9e85c64b02a72eace62503834b.Path = new Bridge.global.Windows.UI.Xaml.PropertyPath.ctor("VerticalContentAlignment");
                    var RelativeSource_d5c97e9725fa4e0d9d98d7d2d7dee2b1 = new Bridge.global.Windows.UI.Xaml.Data.RelativeSource.ctor();
                    RelativeSource_d5c97e9725fa4e0d9d98d7d2d7dee2b1.Mode = Bridge.global.Windows.UI.Xaml.Data.RelativeSourceMode.TemplatedParent;

                    Binding_40e3ca9e85c64b02a72eace62503834b.RelativeSource = RelativeSource_d5c97e9725fa4e0d9d98d7d2d7dee2b1;


                    Binding_40e3ca9e85c64b02a72eace62503834b.TemplateOwner = templateInstance_58e85f93b10d435ca20291d55ec06dfe;

                    var Binding_cbac0aeee0994ed48a390dfa5a9a70e5 = new Bridge.global.Windows.UI.Xaml.Data.Binding.ctor();
                    Binding_cbac0aeee0994ed48a390dfa5a9a70e5.Path = new Bridge.global.Windows.UI.Xaml.PropertyPath.ctor("Content");
                    var RelativeSource_fea37b455382465291f0c37326219c15 = new Bridge.global.Windows.UI.Xaml.Data.RelativeSource.ctor();
                    RelativeSource_fea37b455382465291f0c37326219c15.Mode = Bridge.global.Windows.UI.Xaml.Data.RelativeSourceMode.TemplatedParent;

                    Binding_cbac0aeee0994ed48a390dfa5a9a70e5.RelativeSource = RelativeSource_fea37b455382465291f0c37326219c15;


                    Binding_cbac0aeee0994ed48a390dfa5a9a70e5.TemplateOwner = templateInstance_58e85f93b10d435ca20291d55ec06dfe;

                    var Binding_e9422faa3b31418d89cbf6c1c5604751 = new Bridge.global.Windows.UI.Xaml.Data.Binding.ctor();
                    Binding_e9422faa3b31418d89cbf6c1c5604751.Path = new Bridge.global.Windows.UI.Xaml.PropertyPath.ctor("ContentTemplate");
                    var RelativeSource_f98580e7b88c4c34a53d2423eefa75c5 = new Bridge.global.Windows.UI.Xaml.Data.RelativeSource.ctor();
                    RelativeSource_f98580e7b88c4c34a53d2423eefa75c5.Mode = Bridge.global.Windows.UI.Xaml.Data.RelativeSourceMode.TemplatedParent;

                    Binding_e9422faa3b31418d89cbf6c1c5604751.RelativeSource = RelativeSource_f98580e7b88c4c34a53d2423eefa75c5;


                    Binding_e9422faa3b31418d89cbf6c1c5604751.TemplateOwner = templateInstance_58e85f93b10d435ca20291d55ec06dfe;


                    Border_935ce328cc644056b00abdf139db85b8.Child = ContentPresenter_d82ceb1c5f364335ac106082c6a2177a;

                    var Binding_0fafdf371dcc4b03a085e2873f2de4ef = new Bridge.global.Windows.UI.Xaml.Data.Binding.ctor();
                    Binding_0fafdf371dcc4b03a085e2873f2de4ef.Path = new Bridge.global.Windows.UI.Xaml.PropertyPath.ctor("BorderBrush");
                    var RelativeSource_e6fb9c856dcd4213acf204a216cfaacb = new Bridge.global.Windows.UI.Xaml.Data.RelativeSource.ctor();
                    RelativeSource_e6fb9c856dcd4213acf204a216cfaacb.Mode = Bridge.global.Windows.UI.Xaml.Data.RelativeSourceMode.TemplatedParent;

                    Binding_0fafdf371dcc4b03a085e2873f2de4ef.RelativeSource = RelativeSource_e6fb9c856dcd4213acf204a216cfaacb;


                    Binding_0fafdf371dcc4b03a085e2873f2de4ef.TemplateOwner = templateInstance_58e85f93b10d435ca20291d55ec06dfe;

                    var Binding_6e3c4254267c48a9bbf8bbc7956e7028 = new Bridge.global.Windows.UI.Xaml.Data.Binding.ctor();
                    Binding_6e3c4254267c48a9bbf8bbc7956e7028.Path = new Bridge.global.Windows.UI.Xaml.PropertyPath.ctor("BorderThickness");
                    var RelativeSource_1f05263c7bf44b5f8fac2fe24366b41b = new Bridge.global.Windows.UI.Xaml.Data.RelativeSource.ctor();
                    RelativeSource_1f05263c7bf44b5f8fac2fe24366b41b.Mode = Bridge.global.Windows.UI.Xaml.Data.RelativeSourceMode.TemplatedParent;

                    Binding_6e3c4254267c48a9bbf8bbc7956e7028.RelativeSource = RelativeSource_1f05263c7bf44b5f8fac2fe24366b41b;


                    Binding_6e3c4254267c48a9bbf8bbc7956e7028.TemplateOwner = templateInstance_58e85f93b10d435ca20291d55ec06dfe;

                    var Binding_53fb972e89694431bf059d481356d236 = new Bridge.global.Windows.UI.Xaml.Data.Binding.ctor();
                    Binding_53fb972e89694431bf059d481356d236.Path = new Bridge.global.Windows.UI.Xaml.PropertyPath.ctor("Background");
                    var RelativeSource_0b37909761d34ce0bea874c5aeb02197 = new Bridge.global.Windows.UI.Xaml.Data.RelativeSource.ctor();
                    RelativeSource_0b37909761d34ce0bea874c5aeb02197.Mode = Bridge.global.Windows.UI.Xaml.Data.RelativeSourceMode.TemplatedParent;

                    Binding_53fb972e89694431bf059d481356d236.RelativeSource = RelativeSource_0b37909761d34ce0bea874c5aeb02197;


                    Binding_53fb972e89694431bf059d481356d236.TemplateOwner = templateInstance_58e85f93b10d435ca20291d55ec06dfe;


                    Grid_244f203a38004a8280244bb5e57e69b0.Children.add(TabPanel_1aaaf84b99d149fbac9ceb6f0ce9b07b);
                    Grid_244f203a38004a8280244bb5e57e69b0.Children.add(Border_935ce328cc644056b00abdf139db85b8);


                    Border_f9b213b546324f218fe2dbd9d7f022fd.Child = Grid_244f203a38004a8280244bb5e57e69b0;


                    Bridge.global.Windows.UI.Xaml.Data.BindingOperations.SetBinding(ContentPresenter_d82ceb1c5f364335ac106082c6a2177a, Bridge.global.Windows.UI.Xaml.FrameworkElement.HorizontalAlignmentProperty, Binding_b9dcd88265864bcfbc03ed83443089f9);
                    Bridge.global.Windows.UI.Xaml.Data.BindingOperations.SetBinding(ContentPresenter_d82ceb1c5f364335ac106082c6a2177a, Bridge.global.Windows.UI.Xaml.FrameworkElement.VerticalAlignmentProperty, Binding_40e3ca9e85c64b02a72eace62503834b);
                    Bridge.global.Windows.UI.Xaml.Data.BindingOperations.SetBinding(ContentPresenter_d82ceb1c5f364335ac106082c6a2177a, Bridge.global.Windows.UI.Xaml.Controls.ContentControl.ContentProperty, Binding_cbac0aeee0994ed48a390dfa5a9a70e5);
                    Bridge.global.Windows.UI.Xaml.Data.BindingOperations.SetBinding(ContentPresenter_d82ceb1c5f364335ac106082c6a2177a, Bridge.global.Windows.UI.Xaml.Controls.ContentControl.ContentTemplateProperty, Binding_e9422faa3b31418d89cbf6c1c5604751);
                    Bridge.global.Windows.UI.Xaml.Data.BindingOperations.SetBinding(Border_935ce328cc644056b00abdf139db85b8, Bridge.global.Windows.UI.Xaml.Controls.Border.BorderBrushProperty, Binding_0fafdf371dcc4b03a085e2873f2de4ef);
                    Bridge.global.Windows.UI.Xaml.Data.BindingOperations.SetBinding(Border_935ce328cc644056b00abdf139db85b8, Bridge.global.Windows.UI.Xaml.Controls.Border.BorderThicknessProperty, Binding_6e3c4254267c48a9bbf8bbc7956e7028);
                    Bridge.global.Windows.UI.Xaml.Data.BindingOperations.SetBinding(Border_935ce328cc644056b00abdf139db85b8, Bridge.global.Windows.UI.Xaml.Controls.Border.BackgroundProperty, Binding_53fb972e89694431bf059d481356d236);

                    templateInstance_58e85f93b10d435ca20291d55ec06dfe.TemplateContent = Border_f9b213b546324f218fe2dbd9d7f022fd;
                    return templateInstance_58e85f93b10d435ca20291d55ec06dfe;
                });

                Setter_cb81f967761946bc954b4dca2c8ec095.Value = ControlTemplate_661c05b426b848df9f15bade368f37da;


                Style_42eb0a345e4b4160a95287ffe4f2b862.Setters.add(Setter_d2313654941b425cb262d8730aee0ea5);
                Style_42eb0a345e4b4160a95287ffe4f2b862.Setters.add(Setter_51f4c81f8a9049a8b3c47f576a0a140f);
                Style_42eb0a345e4b4160a95287ffe4f2b862.Setters.add(Setter_c9bd6a13be1f4dfea91b88f5a7eb1d28);
                Style_42eb0a345e4b4160a95287ffe4f2b862.Setters.add(Setter_fd27dbdf489d4b69aba1c6d546bceb74);
                Style_42eb0a345e4b4160a95287ffe4f2b862.Setters.add(Setter_59d5ccb504d6489293ee0fb4bca6c57b);
                Style_42eb0a345e4b4160a95287ffe4f2b862.Setters.add(Setter_393321a379264cc4b6c9d90c432f34e3);
                Style_42eb0a345e4b4160a95287ffe4f2b862.Setters.add(Setter_cb81f967761946bc954b4dca2c8ec095);


                this.setItem(Bridge.global.Windows.UI.Xaml.Controls.TabControl, Style_42eb0a345e4b4160a95287ffe4f2b862);






            }
        }
    });

    Bridge.define("___CSHTML5__Component__Defaultstyles__INTERNAL_Defaulttabitemstyle__Xaml", {
        inherits: [Bridge.global.Windows.UI.Xaml.ResourceDictionary],
        fields: {
            _contentLoaded: false
        },
        ctors: {
            ctor: function () {
                this.$initialize();
                Bridge.global.Windows.UI.Xaml.ResourceDictionary.ctor.call(this);
                this.InitializeComponent();

                if (Bridge.is(this, Windows.UI.Xaml.UIElement)) {
                    Bridge.cast(this, Windows.UI.Xaml.UIElement).XamlSourcePath = "CSHTML5\\DefaultStyles\\INTERNAL_DefaultTabItemStyle.xaml";
                }
            }
        },
        methods: {
            InitializeComponent: function () {
                var $t;
                if (this._contentLoaded) {
                    return;
                }
                this._contentLoaded = true;


                if (Bridge.is(this, Windows.UI.Xaml.UIElement)) {
                    Bridge.cast(this, Windows.UI.Xaml.UIElement).XamlSourcePath = "CSHTML5\\DefaultStyles\\INTERNAL_DefaultTabItemStyle.xaml";
                }



                var Style_5a9b7fc40e014f3aa1b7c3beaa3dc441 = new Bridge.global.Windows.UI.Xaml.Style.ctor();
                Style_5a9b7fc40e014f3aa1b7c3beaa3dc441.TargetType = Bridge.global.Windows.UI.Xaml.Controls.TabItem;
                var Setter_65c953fbf6f64923a4da0ad96cb4f6f7 = new Bridge.global.Windows.UI.Xaml.Setter.ctor();
                Setter_65c953fbf6f64923a4da0ad96cb4f6f7.Property = Bridge.global.Windows.UI.Xaml.Controls.Control.BackgroundProperty;
                Setter_65c953fbf6f64923a4da0ad96cb4f6f7.Value = new Bridge.global.Windows.UI.Xaml.Media.SolidColorBrush.$ctor1(($t = new Bridge.global.Windows.UI.Color(), $t.A = 255, $t.R = 255, $t.G = 255, $t.B = 255, $t));

                var Setter_25ac305d5ccb4bc484f7477c727c8d13 = new Bridge.global.Windows.UI.Xaml.Setter.ctor();
                Setter_25ac305d5ccb4bc484f7477c727c8d13.Property = Bridge.global.Windows.UI.Xaml.Controls.Control.BorderBrushProperty;
                Setter_25ac305d5ccb4bc484f7477c727c8d13.Value = new Bridge.global.Windows.UI.Xaml.Media.SolidColorBrush.$ctor1(($t = new Bridge.global.Windows.UI.Color(), $t.A = 255, $t.R = 221, $t.G = 221, $t.B = 221, $t));

                var Setter_9a155545fa6943029088fd0d2da83c9a = new Bridge.global.Windows.UI.Xaml.Setter.ctor();
                Setter_9a155545fa6943029088fd0d2da83c9a.Property = Bridge.global.Windows.UI.Xaml.Controls.Control.BorderThicknessProperty;
                Setter_9a155545fa6943029088fd0d2da83c9a.Value = new Bridge.global.Windows.UI.Xaml.Thickness.$ctor1(2).$clone();

                var Setter_dd6f6f8600a04a7c954b05658651b03c = new Bridge.global.Windows.UI.Xaml.Setter.ctor();
                Setter_dd6f6f8600a04a7c954b05658651b03c.Property = Bridge.global.Windows.UI.Xaml.FrameworkElement.CursorProperty;
                Setter_dd6f6f8600a04a7c954b05658651b03c.Value = Bridge.global.System.Windows.Input.Cursors.Hand;

                var Setter_02c3f7d25a524f4c882c2a72709345c3 = new Bridge.global.Windows.UI.Xaml.Setter.ctor();
                Setter_02c3f7d25a524f4c882c2a72709345c3.Property = Bridge.global.Windows.UI.Xaml.Controls.Control.ForegroundProperty;
                Setter_02c3f7d25a524f4c882c2a72709345c3.Value = new Bridge.global.Windows.UI.Xaml.Media.SolidColorBrush.$ctor1(($t = new Bridge.global.Windows.UI.Color(), $t.A = 255, $t.R = 0, $t.G = 0, $t.B = 0, $t));

                var Setter_9f1a153c37a04bbd9d0e8328d0c8302e = new Bridge.global.Windows.UI.Xaml.Setter.ctor();
                Setter_9f1a153c37a04bbd9d0e8328d0c8302e.Property = Bridge.global.Windows.UI.Xaml.Controls.Control.HorizontalContentAlignmentProperty;
                Setter_9f1a153c37a04bbd9d0e8328d0c8302e.Value = Bridge.box(Bridge.global.Windows.UI.Xaml.HorizontalAlignment.Stretch, Windows.UI.Xaml.HorizontalAlignment, System.Enum.toStringFn(Windows.UI.Xaml.HorizontalAlignment));

                var Setter_d81eda0ca8ae443d93e29a63c86da128 = new Bridge.global.Windows.UI.Xaml.Setter.ctor();
                Setter_d81eda0ca8ae443d93e29a63c86da128.Property = Bridge.global.Windows.UI.Xaml.Controls.Control.VerticalContentAlignmentProperty;
                Setter_d81eda0ca8ae443d93e29a63c86da128.Value = Bridge.box(Bridge.global.Windows.UI.Xaml.VerticalAlignment.Stretch, Windows.UI.Xaml.VerticalAlignment, System.Enum.toStringFn(Windows.UI.Xaml.VerticalAlignment));

                var Setter_9da1914b0a8040c9add8e541f83e68d9 = new Bridge.global.Windows.UI.Xaml.Setter.ctor();
                Setter_9da1914b0a8040c9add8e541f83e68d9.Property = Bridge.global.Windows.UI.Xaml.FrameworkElement.MarginProperty;
                Setter_9da1914b0a8040c9add8e541f83e68d9.Value = new Bridge.global.Windows.UI.Xaml.Thickness.$ctor2(0, 0, 5, 0).$clone();

                var Setter_cfc33ccfc5fd4b9689d5dab737044844 = new Bridge.global.Windows.UI.Xaml.Setter.ctor();
                Setter_cfc33ccfc5fd4b9689d5dab737044844.Property = Bridge.global.Windows.UI.Xaml.Controls.Control.PaddingProperty;
                Setter_cfc33ccfc5fd4b9689d5dab737044844.Value = new Bridge.global.Windows.UI.Xaml.Thickness.$ctor2(6, 0, 6, 3).$clone();

                var Setter_f06c766527ac44e3aef50365529772a8 = new Bridge.global.Windows.UI.Xaml.Setter.ctor();
                Setter_f06c766527ac44e3aef50365529772a8.Property = Bridge.global.Windows.UI.Xaml.Controls.TabItem.SelectedBackgroundProperty;
                Setter_f06c766527ac44e3aef50365529772a8.Value = new Bridge.global.Windows.UI.Xaml.Media.SolidColorBrush.$ctor1(($t = new Bridge.global.Windows.UI.Color(), $t.A = 255, $t.R = 255, $t.G = 255, $t.B = 255, $t));

                var Setter_9dca73dd218148c0b4f90b551c351427 = new Bridge.global.Windows.UI.Xaml.Setter.ctor();
                Setter_9dca73dd218148c0b4f90b551c351427.Property = Bridge.global.Windows.UI.Xaml.Controls.TabItem.SelectedForegroundProperty;
                Setter_9dca73dd218148c0b4f90b551c351427.Value = new Bridge.global.Windows.UI.Xaml.Media.SolidColorBrush.$ctor1(($t = new Bridge.global.Windows.UI.Color(), $t.A = 255, $t.R = 0, $t.G = 0, $t.B = 0, $t));

                var Setter_4c0d245e6cdd45c581f8f210a4304698 = new Bridge.global.Windows.UI.Xaml.Setter.ctor();
                Setter_4c0d245e6cdd45c581f8f210a4304698.Property = Bridge.global.Windows.UI.Xaml.Controls.TabItem.SelectedAccentProperty;
                Setter_4c0d245e6cdd45c581f8f210a4304698.Value = new Bridge.global.Windows.UI.Xaml.Media.SolidColorBrush.$ctor1(($t = new Bridge.global.Windows.UI.Color(), $t.A = 255, $t.R = 0, $t.G = 0, $t.B = 255, $t));

                var Setter_301d982b49cf4fe0819a280919a7e134 = new Bridge.global.Windows.UI.Xaml.Setter.ctor();
                Setter_301d982b49cf4fe0819a280919a7e134.Property = Bridge.global.Windows.UI.Xaml.Controls.Control.TemplateProperty;
                var ControlTemplate_ad94df07d0614735a1d7fc6be3595dc0 = new Bridge.global.Windows.UI.Xaml.Controls.ControlTemplate();
                ControlTemplate_ad94df07d0614735a1d7fc6be3595dc0.TargetType = Bridge.global.Windows.UI.Xaml.Controls.TabItem;
                ControlTemplate_ad94df07d0614735a1d7fc6be3595dc0.SetMethodToInstantiateFrameworkTemplate(Bridge.fn.bind(this, function (templateOwner_ControlTemplate_ad94df07d0614735a1d7fc6be3595dc0) {
                    var $t1;
                    var templateInstance_1f29572497fd442fa5ffc429f3165496 = new Bridge.global.Windows.UI.Xaml.TemplateInstance();
                    templateInstance_1f29572497fd442fa5ffc429f3165496.TemplateOwner = templateOwner_ControlTemplate_ad94df07d0614735a1d7fc6be3595dc0;
                    var VisualState_187633b6811740f3ac4c41b32e944284 = new Bridge.global.Windows.UI.Xaml.VisualState();
                    var VisualState_d0c0e450c1f442e58900c41e6b9cd2f3 = new Bridge.global.Windows.UI.Xaml.VisualState();
                    var VisualState_cddbedfdc47c460d8bc1a871cc4b9b2b = new Bridge.global.Windows.UI.Xaml.VisualState();
                    var VisualStateGroup_c21c74cd43844cf59bb6b483dff34e79 = new Bridge.global.Windows.UI.Xaml.VisualStateGroup();
                    var VisualState_0811cc16bdb94003af3813f75dfe2dc8 = new Bridge.global.Windows.UI.Xaml.VisualState();
                    var VisualState_6ee19dd8e4f840079d7aeb39af87d17e = new Bridge.global.Windows.UI.Xaml.VisualState();
                    var VisualStateGroup_70cb7be1f98f4122ad0d40a40bf6091c = new Bridge.global.Windows.UI.Xaml.VisualStateGroup();
                    var ContentControl_73de0d026d21410faf8af555951519e1 = new Bridge.global.Windows.UI.Xaml.Controls.ContentControl();
                    var Border_c768aafcd7764da4aae7ffd32c0f617b = new Bridge.global.Windows.UI.Xaml.Controls.Border();
                    var Border_4d665ce372c34cfe804f94461143c6b7 = new Bridge.global.Windows.UI.Xaml.Controls.Border();
                    var ContentControl_91d9059923c9438093997117dcadcfc2 = new Bridge.global.Windows.UI.Xaml.Controls.ContentControl();
                    var Border_07bcb50098de48aa95184f12a05486a9 = new Bridge.global.Windows.UI.Xaml.Controls.Border();
                    var StackPanel_f24ca7f2d3c34559b131ba61f1fc2312 = new Bridge.global.Windows.UI.Xaml.Controls.StackPanel();
                    templateOwner_ControlTemplate_ad94df07d0614735a1d7fc6be3595dc0.RegisterName("Root", StackPanel_f24ca7f2d3c34559b131ba61f1fc2312);
                    StackPanel_f24ca7f2d3c34559b131ba61f1fc2312.Name = "Root";
                    templateOwner_ControlTemplate_ad94df07d0614735a1d7fc6be3595dc0.RegisterName("CommonStates", VisualStateGroup_c21c74cd43844cf59bb6b483dff34e79);
                    VisualStateGroup_c21c74cd43844cf59bb6b483dff34e79.Name = "CommonStates";
                    templateOwner_ControlTemplate_ad94df07d0614735a1d7fc6be3595dc0.RegisterName("Normal", VisualState_187633b6811740f3ac4c41b32e944284);
                    VisualState_187633b6811740f3ac4c41b32e944284.Name = "Normal";

                    templateOwner_ControlTemplate_ad94df07d0614735a1d7fc6be3595dc0.RegisterName("MouseOver", VisualState_d0c0e450c1f442e58900c41e6b9cd2f3);
                    VisualState_d0c0e450c1f442e58900c41e6b9cd2f3.Name = "MouseOver";
                    var Storyboard_f0ed34667add4803835e1b3448cf3853 = new Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard();
                    var ObjectAnimationUsingKeyFrames_529055ebc13f4e028bed2a672f17928b = new Bridge.global.Windows.UI.Xaml.Media.Animation.ObjectAnimationUsingKeyFrames();
                    ObjectAnimationUsingKeyFrames_529055ebc13f4e028bed2a672f17928b.Duration = new Bridge.global.Windows.UI.Xaml.Duration.$ctor1(new Bridge.global.System.TimeSpan(System.Int64(0)));
                    Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard.SetTargetName(ObjectAnimationUsingKeyFrames_529055ebc13f4e028bed2a672f17928b, "PointerOverBorder");
                    var DiscreteObjectKeyFrame_dbc6d853f1844772ae83daf4209d8bfb = new Bridge.global.Windows.UI.Xaml.Media.Animation.DiscreteObjectKeyFrame();
                    DiscreteObjectKeyFrame_dbc6d853f1844772ae83daf4209d8bfb.KeyTime = Bridge.global.Windows.UI.Xaml.Media.Animation.KeyTime.FromTimeSpan(new Bridge.global.System.TimeSpan(System.Int64(0)));
                    DiscreteObjectKeyFrame_dbc6d853f1844772ae83daf4209d8bfb.Value = "#FFCFCFCF";

                    ObjectAnimationUsingKeyFrames_529055ebc13f4e028bed2a672f17928b.KeyFrames.add(DiscreteObjectKeyFrame_dbc6d853f1844772ae83daf4209d8bfb);


                    Storyboard_f0ed34667add4803835e1b3448cf3853.Children.add(ObjectAnimationUsingKeyFrames_529055ebc13f4e028bed2a672f17928b);


                    VisualState_d0c0e450c1f442e58900c41e6b9cd2f3.Storyboard = Storyboard_f0ed34667add4803835e1b3448cf3853;


                    templateOwner_ControlTemplate_ad94df07d0614735a1d7fc6be3595dc0.RegisterName("Disabled", VisualState_cddbedfdc47c460d8bc1a871cc4b9b2b);
                    VisualState_cddbedfdc47c460d8bc1a871cc4b9b2b.Name = "Disabled";

                    System.Array.add(VisualStateGroup_c21c74cd43844cf59bb6b483dff34e79.States, VisualState_187633b6811740f3ac4c41b32e944284, Object);
                    System.Array.add(VisualStateGroup_c21c74cd43844cf59bb6b483dff34e79.States, VisualState_d0c0e450c1f442e58900c41e6b9cd2f3, Object);
                    System.Array.add(VisualStateGroup_c21c74cd43844cf59bb6b483dff34e79.States, VisualState_cddbedfdc47c460d8bc1a871cc4b9b2b, Object);


                    templateOwner_ControlTemplate_ad94df07d0614735a1d7fc6be3595dc0.RegisterName("SelectionStates", VisualStateGroup_70cb7be1f98f4122ad0d40a40bf6091c);
                    VisualStateGroup_70cb7be1f98f4122ad0d40a40bf6091c.Name = "SelectionStates";
                    templateOwner_ControlTemplate_ad94df07d0614735a1d7fc6be3595dc0.RegisterName("Unselected", VisualState_0811cc16bdb94003af3813f75dfe2dc8);
                    VisualState_0811cc16bdb94003af3813f75dfe2dc8.Name = "Unselected";

                    templateOwner_ControlTemplate_ad94df07d0614735a1d7fc6be3595dc0.RegisterName("Selected", VisualState_6ee19dd8e4f840079d7aeb39af87d17e);
                    VisualState_6ee19dd8e4f840079d7aeb39af87d17e.Name = "Selected";

                    System.Array.add(VisualStateGroup_70cb7be1f98f4122ad0d40a40bf6091c.States, VisualState_0811cc16bdb94003af3813f75dfe2dc8, Object);
                    System.Array.add(VisualStateGroup_70cb7be1f98f4122ad0d40a40bf6091c.States, VisualState_6ee19dd8e4f840079d7aeb39af87d17e, Object);


                    templateOwner_ControlTemplate_ad94df07d0614735a1d7fc6be3595dc0.INTERNAL_GetVisualStateGroups().add(VisualStateGroup_c21c74cd43844cf59bb6b483dff34e79);
                    templateOwner_ControlTemplate_ad94df07d0614735a1d7fc6be3595dc0.INTERNAL_GetVisualStateGroups().add(VisualStateGroup_70cb7be1f98f4122ad0d40a40bf6091c);

                    templateOwner_ControlTemplate_ad94df07d0614735a1d7fc6be3595dc0.RegisterName("TemplateTopSelected", Border_c768aafcd7764da4aae7ffd32c0f617b);
                    Border_c768aafcd7764da4aae7ffd32c0f617b.Name = "TemplateTopSelected";
                    Border_c768aafcd7764da4aae7ffd32c0f617b.Visibility = Bridge.global.Windows.UI.Xaml.Visibility.Collapsed;
                    Border_c768aafcd7764da4aae7ffd32c0f617b.BorderThickness = new Bridge.global.Windows.UI.Xaml.Thickness.$ctor2(1, 0, 1, 0);
                    Border_c768aafcd7764da4aae7ffd32c0f617b.CornerRadius = new Bridge.global.Windows.UI.Xaml.CornerRadius.$ctor2(3, 3, 0, 0);
                    Border_c768aafcd7764da4aae7ffd32c0f617b.Cursor = Bridge.global.System.Windows.Input.Cursors.Arrow;
                    var StackPanel_e8053d54a082429ca2518dedd9371a1e = new Bridge.global.Windows.UI.Xaml.Controls.StackPanel();
                    var Border_f699bee9a0414f3595bfc9091c843250 = new Bridge.global.Windows.UI.Xaml.Controls.Border();
                    Border_f699bee9a0414f3595bfc9091c843250.Height = 3.0;
                    Border_f699bee9a0414f3595bfc9091c843250.CornerRadius = new Bridge.global.Windows.UI.Xaml.CornerRadius.$ctor2(3, 3, 0, 0);
                    var Binding_fb75aa096d294fdd8c71ba28e5e97727 = new Bridge.global.Windows.UI.Xaml.Data.Binding.ctor();
                    Binding_fb75aa096d294fdd8c71ba28e5e97727.Path = new Bridge.global.Windows.UI.Xaml.PropertyPath.ctor("SelectedAccent");
                    var RelativeSource_3b3cd13a79384430bd75f179b9fc5105 = new Bridge.global.Windows.UI.Xaml.Data.RelativeSource.ctor();
                    RelativeSource_3b3cd13a79384430bd75f179b9fc5105.Mode = Bridge.global.Windows.UI.Xaml.Data.RelativeSourceMode.TemplatedParent;

                    Binding_fb75aa096d294fdd8c71ba28e5e97727.RelativeSource = RelativeSource_3b3cd13a79384430bd75f179b9fc5105;


                    Binding_fb75aa096d294fdd8c71ba28e5e97727.TemplateOwner = templateInstance_1f29572497fd442fa5ffc429f3165496;


                    templateOwner_ControlTemplate_ad94df07d0614735a1d7fc6be3595dc0.RegisterName("HeaderTopSelected", ContentControl_73de0d026d21410faf8af555951519e1);
                    ContentControl_73de0d026d21410faf8af555951519e1.Name = "HeaderTopSelected";
                    var Binding_bdf3aa412dc143c29ecc69a43df9ab1d = new Bridge.global.Windows.UI.Xaml.Data.Binding.ctor();
                    Binding_bdf3aa412dc143c29ecc69a43df9ab1d.Path = new Bridge.global.Windows.UI.Xaml.PropertyPath.ctor("SelectedForeground");
                    var RelativeSource_2cbbb1bbc6cf4bfbb87b059bc6cc1899 = new Bridge.global.Windows.UI.Xaml.Data.RelativeSource.ctor();
                    RelativeSource_2cbbb1bbc6cf4bfbb87b059bc6cc1899.Mode = Bridge.global.Windows.UI.Xaml.Data.RelativeSourceMode.TemplatedParent;

                    Binding_bdf3aa412dc143c29ecc69a43df9ab1d.RelativeSource = RelativeSource_2cbbb1bbc6cf4bfbb87b059bc6cc1899;


                    Binding_bdf3aa412dc143c29ecc69a43df9ab1d.TemplateOwner = templateInstance_1f29572497fd442fa5ffc429f3165496;

                    var Binding_cc9abba5da604c80a02e74bdca1ed297 = new Bridge.global.Windows.UI.Xaml.Data.Binding.ctor();
                    Binding_cc9abba5da604c80a02e74bdca1ed297.Path = new Bridge.global.Windows.UI.Xaml.PropertyPath.ctor("Padding");
                    var RelativeSource_6a3950495e544e1f9b3f3896f6f09e1d = new Bridge.global.Windows.UI.Xaml.Data.RelativeSource.ctor();
                    RelativeSource_6a3950495e544e1f9b3f3896f6f09e1d.Mode = Bridge.global.Windows.UI.Xaml.Data.RelativeSourceMode.TemplatedParent;

                    Binding_cc9abba5da604c80a02e74bdca1ed297.RelativeSource = RelativeSource_6a3950495e544e1f9b3f3896f6f09e1d;


                    Binding_cc9abba5da604c80a02e74bdca1ed297.TemplateOwner = templateInstance_1f29572497fd442fa5ffc429f3165496;


                    StackPanel_e8053d54a082429ca2518dedd9371a1e.Children.add(Border_f699bee9a0414f3595bfc9091c843250);
                    StackPanel_e8053d54a082429ca2518dedd9371a1e.Children.add(ContentControl_73de0d026d21410faf8af555951519e1);


                    Border_c768aafcd7764da4aae7ffd32c0f617b.Child = StackPanel_e8053d54a082429ca2518dedd9371a1e;

                    var Binding_062dc9457b934c829d4e1970049f1f6c = new Bridge.global.Windows.UI.Xaml.Data.Binding.ctor();
                    Binding_062dc9457b934c829d4e1970049f1f6c.Path = new Bridge.global.Windows.UI.Xaml.PropertyPath.ctor("BorderBrush");
                    var RelativeSource_1708173f438042c4bfc30b534b5b49a1 = new Bridge.global.Windows.UI.Xaml.Data.RelativeSource.ctor();
                    RelativeSource_1708173f438042c4bfc30b534b5b49a1.Mode = Bridge.global.Windows.UI.Xaml.Data.RelativeSourceMode.TemplatedParent;

                    Binding_062dc9457b934c829d4e1970049f1f6c.RelativeSource = RelativeSource_1708173f438042c4bfc30b534b5b49a1;


                    Binding_062dc9457b934c829d4e1970049f1f6c.TemplateOwner = templateInstance_1f29572497fd442fa5ffc429f3165496;

                    var Binding_afeafd6ae6ac432985c81d729af094d9 = new Bridge.global.Windows.UI.Xaml.Data.Binding.ctor();
                    Binding_afeafd6ae6ac432985c81d729af094d9.Path = new Bridge.global.Windows.UI.Xaml.PropertyPath.ctor("SelectedBackground");
                    var RelativeSource_62b1d909323945f89f48815f3a159a72 = new Bridge.global.Windows.UI.Xaml.Data.RelativeSource.ctor();
                    RelativeSource_62b1d909323945f89f48815f3a159a72.Mode = Bridge.global.Windows.UI.Xaml.Data.RelativeSourceMode.TemplatedParent;

                    Binding_afeafd6ae6ac432985c81d729af094d9.RelativeSource = RelativeSource_62b1d909323945f89f48815f3a159a72;


                    Binding_afeafd6ae6ac432985c81d729af094d9.TemplateOwner = templateInstance_1f29572497fd442fa5ffc429f3165496;


                    templateOwner_ControlTemplate_ad94df07d0614735a1d7fc6be3595dc0.RegisterName("TemplateTopUnselected", Border_07bcb50098de48aa95184f12a05486a9);
                    Border_07bcb50098de48aa95184f12a05486a9.Name = "TemplateTopUnselected";
                    Border_07bcb50098de48aa95184f12a05486a9.Visibility = Bridge.global.Windows.UI.Xaml.Visibility.Collapsed;
                    Border_07bcb50098de48aa95184f12a05486a9.BorderThickness = new Bridge.global.Windows.UI.Xaml.Thickness.$ctor2(1, 1, 1, 0);
                    Border_07bcb50098de48aa95184f12a05486a9.CornerRadius = new Bridge.global.Windows.UI.Xaml.CornerRadius.$ctor2(3, 3, 0, 0);
                    var StackPanel_5bf7a116cf234e8e850a14c4fbad23b7 = new Bridge.global.Windows.UI.Xaml.Controls.StackPanel();
                    templateOwner_ControlTemplate_ad94df07d0614735a1d7fc6be3595dc0.RegisterName("PointerOverBorder", Border_4d665ce372c34cfe804f94461143c6b7);
                    Border_4d665ce372c34cfe804f94461143c6b7.Name = "PointerOverBorder";
                    Border_4d665ce372c34cfe804f94461143c6b7.Height = 2.0;
                    Border_4d665ce372c34cfe804f94461143c6b7.CornerRadius = new Bridge.global.Windows.UI.Xaml.CornerRadius.$ctor2(3, 3, 0, 0);
                    Border_4d665ce372c34cfe804f94461143c6b7.Background = new Bridge.global.Windows.UI.Xaml.Media.SolidColorBrush.$ctor1(($t1 = new Bridge.global.Windows.UI.Color(), $t1.A = 0, $t1.R = 255, $t1.G = 255, $t1.B = 255, $t1));

                    templateOwner_ControlTemplate_ad94df07d0614735a1d7fc6be3595dc0.RegisterName("HeaderTopUnselected", ContentControl_91d9059923c9438093997117dcadcfc2);
                    ContentControl_91d9059923c9438093997117dcadcfc2.Name = "HeaderTopUnselected";
                    var Binding_136dcb4dc31c453e90a7477950ee71db = new Bridge.global.Windows.UI.Xaml.Data.Binding.ctor();
                    Binding_136dcb4dc31c453e90a7477950ee71db.Path = new Bridge.global.Windows.UI.Xaml.PropertyPath.ctor("Foreground");
                    var RelativeSource_953dbafbc0c84e6e84dae2e99cd3ad22 = new Bridge.global.Windows.UI.Xaml.Data.RelativeSource.ctor();
                    RelativeSource_953dbafbc0c84e6e84dae2e99cd3ad22.Mode = Bridge.global.Windows.UI.Xaml.Data.RelativeSourceMode.TemplatedParent;

                    Binding_136dcb4dc31c453e90a7477950ee71db.RelativeSource = RelativeSource_953dbafbc0c84e6e84dae2e99cd3ad22;


                    Binding_136dcb4dc31c453e90a7477950ee71db.TemplateOwner = templateInstance_1f29572497fd442fa5ffc429f3165496;

                    var Binding_4d7704e42a8c46318fc20a81337d05af = new Bridge.global.Windows.UI.Xaml.Data.Binding.ctor();
                    Binding_4d7704e42a8c46318fc20a81337d05af.Path = new Bridge.global.Windows.UI.Xaml.PropertyPath.ctor("Padding");
                    var RelativeSource_1d7503a370b040a28d6aca8e7af07ffe = new Bridge.global.Windows.UI.Xaml.Data.RelativeSource.ctor();
                    RelativeSource_1d7503a370b040a28d6aca8e7af07ffe.Mode = Bridge.global.Windows.UI.Xaml.Data.RelativeSourceMode.TemplatedParent;

                    Binding_4d7704e42a8c46318fc20a81337d05af.RelativeSource = RelativeSource_1d7503a370b040a28d6aca8e7af07ffe;


                    Binding_4d7704e42a8c46318fc20a81337d05af.TemplateOwner = templateInstance_1f29572497fd442fa5ffc429f3165496;


                    StackPanel_5bf7a116cf234e8e850a14c4fbad23b7.Children.add(Border_4d665ce372c34cfe804f94461143c6b7);
                    StackPanel_5bf7a116cf234e8e850a14c4fbad23b7.Children.add(ContentControl_91d9059923c9438093997117dcadcfc2);


                    Border_07bcb50098de48aa95184f12a05486a9.Child = StackPanel_5bf7a116cf234e8e850a14c4fbad23b7;

                    var Binding_8127d0feeadc4bfca07d7be867de2fc1 = new Bridge.global.Windows.UI.Xaml.Data.Binding.ctor();
                    Binding_8127d0feeadc4bfca07d7be867de2fc1.Path = new Bridge.global.Windows.UI.Xaml.PropertyPath.ctor("BorderBrush");
                    var RelativeSource_fb117dddbb06408786bdee224ba8f238 = new Bridge.global.Windows.UI.Xaml.Data.RelativeSource.ctor();
                    RelativeSource_fb117dddbb06408786bdee224ba8f238.Mode = Bridge.global.Windows.UI.Xaml.Data.RelativeSourceMode.TemplatedParent;

                    Binding_8127d0feeadc4bfca07d7be867de2fc1.RelativeSource = RelativeSource_fb117dddbb06408786bdee224ba8f238;


                    Binding_8127d0feeadc4bfca07d7be867de2fc1.TemplateOwner = templateInstance_1f29572497fd442fa5ffc429f3165496;

                    var Binding_8fcc8395dfa043b1bdb75a240ddd94ad = new Bridge.global.Windows.UI.Xaml.Data.Binding.ctor();
                    Binding_8fcc8395dfa043b1bdb75a240ddd94ad.Path = new Bridge.global.Windows.UI.Xaml.PropertyPath.ctor("Background");
                    var RelativeSource_2d139ffda998499d833a184627b8b2f8 = new Bridge.global.Windows.UI.Xaml.Data.RelativeSource.ctor();
                    RelativeSource_2d139ffda998499d833a184627b8b2f8.Mode = Bridge.global.Windows.UI.Xaml.Data.RelativeSourceMode.TemplatedParent;

                    Binding_8fcc8395dfa043b1bdb75a240ddd94ad.RelativeSource = RelativeSource_2d139ffda998499d833a184627b8b2f8;


                    Binding_8fcc8395dfa043b1bdb75a240ddd94ad.TemplateOwner = templateInstance_1f29572497fd442fa5ffc429f3165496;

                    var Binding_b103a88947aa48859e7a385aa7c8c6bd = new Bridge.global.Windows.UI.Xaml.Data.Binding.ctor();
                    Binding_b103a88947aa48859e7a385aa7c8c6bd.Path = new Bridge.global.Windows.UI.Xaml.PropertyPath.ctor("Cursor");
                    var RelativeSource_1457be36dc3b4209a4147df450823e49 = new Bridge.global.Windows.UI.Xaml.Data.RelativeSource.ctor();
                    RelativeSource_1457be36dc3b4209a4147df450823e49.Mode = Bridge.global.Windows.UI.Xaml.Data.RelativeSourceMode.TemplatedParent;

                    Binding_b103a88947aa48859e7a385aa7c8c6bd.RelativeSource = RelativeSource_1457be36dc3b4209a4147df450823e49;


                    Binding_b103a88947aa48859e7a385aa7c8c6bd.TemplateOwner = templateInstance_1f29572497fd442fa5ffc429f3165496;


                    StackPanel_f24ca7f2d3c34559b131ba61f1fc2312.Children.add(Border_c768aafcd7764da4aae7ffd32c0f617b);
                    StackPanel_f24ca7f2d3c34559b131ba61f1fc2312.Children.add(Border_07bcb50098de48aa95184f12a05486a9);


                    Bridge.global.Windows.UI.Xaml.Data.BindingOperations.SetBinding(Border_f699bee9a0414f3595bfc9091c843250, Bridge.global.Windows.UI.Xaml.Controls.Border.BackgroundProperty, Binding_fb75aa096d294fdd8c71ba28e5e97727);
                    Bridge.global.Windows.UI.Xaml.Data.BindingOperations.SetBinding(ContentControl_73de0d026d21410faf8af555951519e1, Bridge.global.Windows.UI.Xaml.Controls.Control.ForegroundProperty, Binding_bdf3aa412dc143c29ecc69a43df9ab1d);
                    Bridge.global.Windows.UI.Xaml.Data.BindingOperations.SetBinding(ContentControl_73de0d026d21410faf8af555951519e1, Bridge.global.Windows.UI.Xaml.FrameworkElement.MarginProperty, Binding_cc9abba5da604c80a02e74bdca1ed297);
                    Bridge.global.Windows.UI.Xaml.Data.BindingOperations.SetBinding(Border_c768aafcd7764da4aae7ffd32c0f617b, Bridge.global.Windows.UI.Xaml.Controls.Border.BorderBrushProperty, Binding_062dc9457b934c829d4e1970049f1f6c);
                    Bridge.global.Windows.UI.Xaml.Data.BindingOperations.SetBinding(Border_c768aafcd7764da4aae7ffd32c0f617b, Bridge.global.Windows.UI.Xaml.Controls.Border.BackgroundProperty, Binding_afeafd6ae6ac432985c81d729af094d9);
                    Bridge.global.Windows.UI.Xaml.Data.BindingOperations.SetBinding(ContentControl_91d9059923c9438093997117dcadcfc2, Bridge.global.Windows.UI.Xaml.Controls.Control.ForegroundProperty, Binding_136dcb4dc31c453e90a7477950ee71db);
                    Bridge.global.Windows.UI.Xaml.Data.BindingOperations.SetBinding(ContentControl_91d9059923c9438093997117dcadcfc2, Bridge.global.Windows.UI.Xaml.FrameworkElement.MarginProperty, Binding_4d7704e42a8c46318fc20a81337d05af);
                    Bridge.global.Windows.UI.Xaml.Data.BindingOperations.SetBinding(Border_07bcb50098de48aa95184f12a05486a9, Bridge.global.Windows.UI.Xaml.Controls.Border.BorderBrushProperty, Binding_8127d0feeadc4bfca07d7be867de2fc1);
                    Bridge.global.Windows.UI.Xaml.Data.BindingOperations.SetBinding(Border_07bcb50098de48aa95184f12a05486a9, Bridge.global.Windows.UI.Xaml.Controls.Border.BackgroundProperty, Binding_8fcc8395dfa043b1bdb75a240ddd94ad);
                    Bridge.global.Windows.UI.Xaml.Data.BindingOperations.SetBinding(Border_07bcb50098de48aa95184f12a05486a9, Bridge.global.Windows.UI.Xaml.FrameworkElement.CursorProperty, Binding_b103a88947aa48859e7a385aa7c8c6bd);

                    Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard.SetTargetProperty(ObjectAnimationUsingKeyFrames_529055ebc13f4e028bed2a672f17928b, new Bridge.global.Windows.UI.Xaml.PropertyPath.$ctor1("Background", "Background", Bridge.fn.cacheBind(this, this.accessVisualStateProperty_9b505070a398464880084850a3c62436), Bridge.fn.cacheBind(this, this.setVisualStateProperty_9b505070a398464880084850a3c62436), Bridge.fn.cacheBind(this, this.setAnimationVisualStateProperty_9b505070a398464880084850a3c62436), Bridge.fn.cacheBind(this, this.setLocalVisualStateProperty_9b505070a398464880084850a3c62436), Bridge.fn.cacheBind(this, this.getVisualStateProperty_9b505070a398464880084850a3c62436)));
                    Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard.SetTarget(ObjectAnimationUsingKeyFrames_529055ebc13f4e028bed2a672f17928b, Border_4d665ce372c34cfe804f94461143c6b7);

                    templateInstance_1f29572497fd442fa5ffc429f3165496.TemplateContent = StackPanel_f24ca7f2d3c34559b131ba61f1fc2312;
                    return templateInstance_1f29572497fd442fa5ffc429f3165496;
                }));

                Setter_301d982b49cf4fe0819a280919a7e134.Value = ControlTemplate_ad94df07d0614735a1d7fc6be3595dc0;


                Style_5a9b7fc40e014f3aa1b7c3beaa3dc441.Setters.add(Setter_65c953fbf6f64923a4da0ad96cb4f6f7);
                Style_5a9b7fc40e014f3aa1b7c3beaa3dc441.Setters.add(Setter_25ac305d5ccb4bc484f7477c727c8d13);
                Style_5a9b7fc40e014f3aa1b7c3beaa3dc441.Setters.add(Setter_9a155545fa6943029088fd0d2da83c9a);
                Style_5a9b7fc40e014f3aa1b7c3beaa3dc441.Setters.add(Setter_dd6f6f8600a04a7c954b05658651b03c);
                Style_5a9b7fc40e014f3aa1b7c3beaa3dc441.Setters.add(Setter_02c3f7d25a524f4c882c2a72709345c3);
                Style_5a9b7fc40e014f3aa1b7c3beaa3dc441.Setters.add(Setter_9f1a153c37a04bbd9d0e8328d0c8302e);
                Style_5a9b7fc40e014f3aa1b7c3beaa3dc441.Setters.add(Setter_d81eda0ca8ae443d93e29a63c86da128);
                Style_5a9b7fc40e014f3aa1b7c3beaa3dc441.Setters.add(Setter_9da1914b0a8040c9add8e541f83e68d9);
                Style_5a9b7fc40e014f3aa1b7c3beaa3dc441.Setters.add(Setter_cfc33ccfc5fd4b9689d5dab737044844);
                Style_5a9b7fc40e014f3aa1b7c3beaa3dc441.Setters.add(Setter_f06c766527ac44e3aef50365529772a8);
                Style_5a9b7fc40e014f3aa1b7c3beaa3dc441.Setters.add(Setter_9dca73dd218148c0b4f90b551c351427);
                Style_5a9b7fc40e014f3aa1b7c3beaa3dc441.Setters.add(Setter_4c0d245e6cdd45c581f8f210a4304698);
                Style_5a9b7fc40e014f3aa1b7c3beaa3dc441.Setters.add(Setter_301d982b49cf4fe0819a280919a7e134);


                this.setItem(Bridge.global.Windows.UI.Xaml.Controls.TabItem, Style_5a9b7fc40e014f3aa1b7c3beaa3dc441);






            },
            accessVisualStateProperty_9b505070a398464880084850a3c62436: function (rootTargetObjectInstance) {
                return new (Bridge.GeneratorEnumerable$1(System.Tuple$3(Windows.UI.Xaml.DependencyObject,Windows.UI.Xaml.DependencyProperty,System.Nullable$1(System.Int32))))(Bridge.fn.bind(this, function (rootTargetObjectInstance) {
                    var $step = 0,
                        $jumpFromFinally,
                        $returnValue,
                        $async_e;

                    var $enumerator = new (Bridge.GeneratorEnumerator$1(System.Tuple$3(Windows.UI.Xaml.DependencyObject,Windows.UI.Xaml.DependencyProperty,System.Nullable$1(System.Int32))))(Bridge.fn.bind(this, function () {
                        try {
                            for (;;) {
                                switch ($step) {
                                    case 0: {
                                        return false;
                                    }
                                    default: {
                                        return false;
                                    }
                                }
                            }
                        } catch($async_e1) {
                            $async_e = System.Exception.create($async_e1);
                            throw $async_e;
                        }
                    }));
                    return $enumerator;
                }, arguments));
            },
            setVisualStateProperty_9b505070a398464880084850a3c62436: function (finalTargetInstance, value) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Background").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "BackgroundProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                (finalTargetInstance).SetVisualStateValue(property, value);
            },
            setAnimationVisualStateProperty_9b505070a398464880084850a3c62436: function (finalTargetInstance, value) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Background").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "BackgroundProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                (finalTargetInstance).SetAnimationValue(property, value);
            },
            setLocalVisualStateProperty_9b505070a398464880084850a3c62436: function (finalTargetInstance, value) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Background").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "BackgroundProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                (finalTargetInstance).SetLocalValue(property, value);
            },
            getVisualStateProperty_9b505070a398464880084850a3c62436: function (finalTargetInstance) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Background").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "BackgroundProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                return finalTargetInstance.GetVisualStateValue(property);
            }
        }
    });

    Bridge.define("___CSHTML5__Component__Defaultstyles__INTERNAL_Defaultthumbstyle__Xaml", {
        inherits: [Bridge.global.Windows.UI.Xaml.ResourceDictionary],
        fields: {
            _contentLoaded: false
        },
        ctors: {
            ctor: function () {
                this.$initialize();
                Bridge.global.Windows.UI.Xaml.ResourceDictionary.ctor.call(this);
                this.InitializeComponent();

                if (Bridge.is(this, Windows.UI.Xaml.UIElement)) {
                    Bridge.cast(this, Windows.UI.Xaml.UIElement).XamlSourcePath = "CSHTML5\\DefaultStyles\\INTERNAL_DefaultThumbStyle.xaml";
                }
            }
        },
        methods: {
            InitializeComponent: function () {
                var $t;
                if (this._contentLoaded) {
                    return;
                }
                this._contentLoaded = true;


                if (Bridge.is(this, Windows.UI.Xaml.UIElement)) {
                    Bridge.cast(this, Windows.UI.Xaml.UIElement).XamlSourcePath = "CSHTML5\\DefaultStyles\\INTERNAL_DefaultThumbStyle.xaml";
                }



                var Style_547d060bf12e40168c05af039e4c3829 = new Bridge.global.Windows.UI.Xaml.Style.ctor();
                Style_547d060bf12e40168c05af039e4c3829.TargetType = Bridge.global.Windows.UI.Xaml.Controls.Primitives.Thumb;
                var Setter_b9a248b44efa4daa9d0c861801f1477e = new Bridge.global.Windows.UI.Xaml.Setter.ctor();
                Setter_b9a248b44efa4daa9d0c861801f1477e.Property = Bridge.global.Windows.UI.Xaml.Controls.Control.BackgroundProperty;
                Setter_b9a248b44efa4daa9d0c861801f1477e.Value = new Bridge.global.Windows.UI.Xaml.Media.SolidColorBrush.$ctor1(($t = new Bridge.global.Windows.UI.Color(), $t.A = 255, $t.R = 226, $t.G = 226, $t.B = 226, $t));

                var Setter_a38ba0fe8e0548b6b5b71a95fa48feb8 = new Bridge.global.Windows.UI.Xaml.Setter.ctor();
                Setter_a38ba0fe8e0548b6b5b71a95fa48feb8.Property = Bridge.global.Windows.UI.Xaml.Controls.Control.BorderThicknessProperty;
                Setter_a38ba0fe8e0548b6b5b71a95fa48feb8.Value = new Bridge.global.Windows.UI.Xaml.Thickness.$ctor1(0).$clone();

                var Setter_0bca64b079774b518d6efcc3096003f9 = new Bridge.global.Windows.UI.Xaml.Setter.ctor();
                Setter_0bca64b079774b518d6efcc3096003f9.Property = Bridge.global.Windows.UI.Xaml.Controls.Control.TemplateProperty;
                var ControlTemplate_fdc984696fe445eba0c7530491f12992 = new Bridge.global.Windows.UI.Xaml.Controls.ControlTemplate();
                ControlTemplate_fdc984696fe445eba0c7530491f12992.TargetType = Bridge.global.Windows.UI.Xaml.Controls.Primitives.Thumb;
                ControlTemplate_fdc984696fe445eba0c7530491f12992.SetMethodToInstantiateFrameworkTemplate(Bridge.fn.bind(this, function (templateOwner_ControlTemplate_fdc984696fe445eba0c7530491f12992) {
                    var $t1;
                    var templateInstance_684e62ee889e40619092425d2e66a1b6 = new Bridge.global.Windows.UI.Xaml.TemplateInstance();
                    templateInstance_684e62ee889e40619092425d2e66a1b6.TemplateOwner = templateOwner_ControlTemplate_fdc984696fe445eba0c7530491f12992;
                    var VisualState_de44a076d40d40a5a9a0655b52123c77 = new Bridge.global.Windows.UI.Xaml.VisualState();
                    var VisualState_a4804a04a0dd4ec99a74199da4a86555 = new Bridge.global.Windows.UI.Xaml.VisualState();
                    var VisualState_a375618c166042a0a91b859a3a575f9d = new Bridge.global.Windows.UI.Xaml.VisualState();
                    var VisualState_4d444a4da3264388821e193de18c4c3c = new Bridge.global.Windows.UI.Xaml.VisualState();
                    var VisualStateGroup_c5c557afa7b3468b80a20a349ba1b26f = new Bridge.global.Windows.UI.Xaml.VisualStateGroup();
                    var Border_58d9622386ff4b75a2d15aebed5ad9ad = new Bridge.global.Windows.UI.Xaml.Controls.Border();
                    var Border_6461f3cb0be4479681960b093f6b398f = new Bridge.global.Windows.UI.Xaml.Controls.Border();
                    var Border_5e4a7caacfde480e89b5b61c6778b5bf = new Bridge.global.Windows.UI.Xaml.Controls.Border();
                    var Border_bbca27caeec44de6bf64e3077293e38e = new Bridge.global.Windows.UI.Xaml.Controls.Border();
                    var Grid_17f39a3d379c4e66bb334a8c888a0751 = new Bridge.global.Windows.UI.Xaml.Controls.Grid();
                    templateOwner_ControlTemplate_fdc984696fe445eba0c7530491f12992.RegisterName("CommonStates", VisualStateGroup_c5c557afa7b3468b80a20a349ba1b26f);
                    VisualStateGroup_c5c557afa7b3468b80a20a349ba1b26f.Name = "CommonStates";
                    templateOwner_ControlTemplate_fdc984696fe445eba0c7530491f12992.RegisterName("Normal", VisualState_de44a076d40d40a5a9a0655b52123c77);
                    VisualState_de44a076d40d40a5a9a0655b52123c77.Name = "Normal";

                    templateOwner_ControlTemplate_fdc984696fe445eba0c7530491f12992.RegisterName("PointerOver", VisualState_a4804a04a0dd4ec99a74199da4a86555);
                    VisualState_a4804a04a0dd4ec99a74199da4a86555.Name = "PointerOver";
                    var Storyboard_66862cb06ce344baa1e32edf6370d02c = new Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard();
                    var DoubleAnimation_9c40079cf5ad4672932f4b19de205919 = new Bridge.global.Windows.UI.Xaml.Media.Animation.DoubleAnimation();
                    DoubleAnimation_9c40079cf5ad4672932f4b19de205919.Duration = new Bridge.global.Windows.UI.Xaml.Duration.$ctor1(new Bridge.global.System.TimeSpan(System.Int64(0)));
                    DoubleAnimation_9c40079cf5ad4672932f4b19de205919.To = 1.0;
                    Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard.SetTargetName(DoubleAnimation_9c40079cf5ad4672932f4b19de205919, "BackgroundPointerOver");

                    var DoubleAnimation_63ec88adadd848bc8067e3ea4cd670a6 = new Bridge.global.Windows.UI.Xaml.Media.Animation.DoubleAnimation();
                    DoubleAnimation_63ec88adadd848bc8067e3ea4cd670a6.Duration = new Bridge.global.Windows.UI.Xaml.Duration.$ctor1(new Bridge.global.System.TimeSpan(System.Int64(0)));
                    DoubleAnimation_63ec88adadd848bc8067e3ea4cd670a6.To = 0.0;
                    Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard.SetTargetName(DoubleAnimation_63ec88adadd848bc8067e3ea4cd670a6, "Background");

                    Storyboard_66862cb06ce344baa1e32edf6370d02c.Children.add(DoubleAnimation_9c40079cf5ad4672932f4b19de205919);
                    Storyboard_66862cb06ce344baa1e32edf6370d02c.Children.add(DoubleAnimation_63ec88adadd848bc8067e3ea4cd670a6);


                    VisualState_a4804a04a0dd4ec99a74199da4a86555.Storyboard = Storyboard_66862cb06ce344baa1e32edf6370d02c;


                    templateOwner_ControlTemplate_fdc984696fe445eba0c7530491f12992.RegisterName("Pressed", VisualState_a375618c166042a0a91b859a3a575f9d);
                    VisualState_a375618c166042a0a91b859a3a575f9d.Name = "Pressed";
                    var Storyboard_5335d574b60a4df09c02253f83cded2d = new Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard();
                    var DoubleAnimation_2d55cf9f7822462e83a7c41c8529e841 = new Bridge.global.Windows.UI.Xaml.Media.Animation.DoubleAnimation();
                    DoubleAnimation_2d55cf9f7822462e83a7c41c8529e841.Duration = new Bridge.global.Windows.UI.Xaml.Duration.$ctor1(new Bridge.global.System.TimeSpan(System.Int64(0)));
                    DoubleAnimation_2d55cf9f7822462e83a7c41c8529e841.To = 1.0;
                    Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard.SetTargetName(DoubleAnimation_2d55cf9f7822462e83a7c41c8529e841, "BackgroundPressed");

                    var DoubleAnimation_cbf8a095e5b14e1caeb9ca4ef4a38e41 = new Bridge.global.Windows.UI.Xaml.Media.Animation.DoubleAnimation();
                    DoubleAnimation_cbf8a095e5b14e1caeb9ca4ef4a38e41.Duration = new Bridge.global.Windows.UI.Xaml.Duration.$ctor1(new Bridge.global.System.TimeSpan(System.Int64(0)));
                    DoubleAnimation_cbf8a095e5b14e1caeb9ca4ef4a38e41.To = 0.0;
                    Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard.SetTargetName(DoubleAnimation_cbf8a095e5b14e1caeb9ca4ef4a38e41, "Background");

                    Storyboard_5335d574b60a4df09c02253f83cded2d.Children.add(DoubleAnimation_2d55cf9f7822462e83a7c41c8529e841);
                    Storyboard_5335d574b60a4df09c02253f83cded2d.Children.add(DoubleAnimation_cbf8a095e5b14e1caeb9ca4ef4a38e41);


                    VisualState_a375618c166042a0a91b859a3a575f9d.Storyboard = Storyboard_5335d574b60a4df09c02253f83cded2d;


                    templateOwner_ControlTemplate_fdc984696fe445eba0c7530491f12992.RegisterName("Disabled", VisualState_4d444a4da3264388821e193de18c4c3c);
                    VisualState_4d444a4da3264388821e193de18c4c3c.Name = "Disabled";
                    var Storyboard_34aef763a94e44b2873d6e181f88ed2a = new Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard();
                    var DoubleAnimation_300cae50b5704817929b4c4fecadce1b = new Bridge.global.Windows.UI.Xaml.Media.Animation.DoubleAnimation();
                    DoubleAnimation_300cae50b5704817929b4c4fecadce1b.Duration = new Bridge.global.Windows.UI.Xaml.Duration.$ctor1(new Bridge.global.System.TimeSpan(System.Int64(0)));
                    DoubleAnimation_300cae50b5704817929b4c4fecadce1b.To = 1.0;
                    Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard.SetTargetName(DoubleAnimation_300cae50b5704817929b4c4fecadce1b, "BackgroundDisabled");

                    var DoubleAnimation_ac0aac1f89084e20b64593926003885a = new Bridge.global.Windows.UI.Xaml.Media.Animation.DoubleAnimation();
                    DoubleAnimation_ac0aac1f89084e20b64593926003885a.Duration = new Bridge.global.Windows.UI.Xaml.Duration.$ctor1(new Bridge.global.System.TimeSpan(System.Int64(0)));
                    DoubleAnimation_ac0aac1f89084e20b64593926003885a.To = 0.0;
                    Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard.SetTargetName(DoubleAnimation_ac0aac1f89084e20b64593926003885a, "Background");

                    Storyboard_34aef763a94e44b2873d6e181f88ed2a.Children.add(DoubleAnimation_300cae50b5704817929b4c4fecadce1b);
                    Storyboard_34aef763a94e44b2873d6e181f88ed2a.Children.add(DoubleAnimation_ac0aac1f89084e20b64593926003885a);


                    VisualState_4d444a4da3264388821e193de18c4c3c.Storyboard = Storyboard_34aef763a94e44b2873d6e181f88ed2a;


                    System.Array.add(VisualStateGroup_c5c557afa7b3468b80a20a349ba1b26f.States, VisualState_de44a076d40d40a5a9a0655b52123c77, Object);
                    System.Array.add(VisualStateGroup_c5c557afa7b3468b80a20a349ba1b26f.States, VisualState_a4804a04a0dd4ec99a74199da4a86555, Object);
                    System.Array.add(VisualStateGroup_c5c557afa7b3468b80a20a349ba1b26f.States, VisualState_a375618c166042a0a91b859a3a575f9d, Object);
                    System.Array.add(VisualStateGroup_c5c557afa7b3468b80a20a349ba1b26f.States, VisualState_4d444a4da3264388821e193de18c4c3c, Object);


                    templateOwner_ControlTemplate_fdc984696fe445eba0c7530491f12992.INTERNAL_GetVisualStateGroups().add(VisualStateGroup_c5c557afa7b3468b80a20a349ba1b26f);

                    templateOwner_ControlTemplate_fdc984696fe445eba0c7530491f12992.RegisterName("Background", Border_58d9622386ff4b75a2d15aebed5ad9ad);
                    Border_58d9622386ff4b75a2d15aebed5ad9ad.Name = "Background";
                    var Binding_8cde840a08eb4c739db304f23797e2fd = new Bridge.global.Windows.UI.Xaml.Data.Binding.ctor();
                    Binding_8cde840a08eb4c739db304f23797e2fd.Path = new Bridge.global.Windows.UI.Xaml.PropertyPath.ctor("Background");
                    var RelativeSource_3692844a4cce4e2692be783d44ebafd5 = new Bridge.global.Windows.UI.Xaml.Data.RelativeSource.ctor();
                    RelativeSource_3692844a4cce4e2692be783d44ebafd5.Mode = Bridge.global.Windows.UI.Xaml.Data.RelativeSourceMode.TemplatedParent;

                    Binding_8cde840a08eb4c739db304f23797e2fd.RelativeSource = RelativeSource_3692844a4cce4e2692be783d44ebafd5;


                    Binding_8cde840a08eb4c739db304f23797e2fd.TemplateOwner = templateInstance_684e62ee889e40619092425d2e66a1b6;

                    var Binding_ba9a493800524dcc9c23993841321765 = new Bridge.global.Windows.UI.Xaml.Data.Binding.ctor();
                    Binding_ba9a493800524dcc9c23993841321765.Path = new Bridge.global.Windows.UI.Xaml.PropertyPath.ctor("BorderBrush");
                    var RelativeSource_5bdd109830c747e4958e34adbb226249 = new Bridge.global.Windows.UI.Xaml.Data.RelativeSource.ctor();
                    RelativeSource_5bdd109830c747e4958e34adbb226249.Mode = Bridge.global.Windows.UI.Xaml.Data.RelativeSourceMode.TemplatedParent;

                    Binding_ba9a493800524dcc9c23993841321765.RelativeSource = RelativeSource_5bdd109830c747e4958e34adbb226249;


                    Binding_ba9a493800524dcc9c23993841321765.TemplateOwner = templateInstance_684e62ee889e40619092425d2e66a1b6;

                    var Binding_8cc6f18be0e94ac59cc2384b0e887047 = new Bridge.global.Windows.UI.Xaml.Data.Binding.ctor();
                    Binding_8cc6f18be0e94ac59cc2384b0e887047.Path = new Bridge.global.Windows.UI.Xaml.PropertyPath.ctor("BorderThickness");
                    var RelativeSource_d2c142924d7e4f3cb62bca6b83b035e7 = new Bridge.global.Windows.UI.Xaml.Data.RelativeSource.ctor();
                    RelativeSource_d2c142924d7e4f3cb62bca6b83b035e7.Mode = Bridge.global.Windows.UI.Xaml.Data.RelativeSourceMode.TemplatedParent;

                    Binding_8cc6f18be0e94ac59cc2384b0e887047.RelativeSource = RelativeSource_d2c142924d7e4f3cb62bca6b83b035e7;


                    Binding_8cc6f18be0e94ac59cc2384b0e887047.TemplateOwner = templateInstance_684e62ee889e40619092425d2e66a1b6;


                    templateOwner_ControlTemplate_fdc984696fe445eba0c7530491f12992.RegisterName("BackgroundPointerOver", Border_6461f3cb0be4479681960b093f6b398f);
                    Border_6461f3cb0be4479681960b093f6b398f.Name = "BackgroundPointerOver";
                    Border_6461f3cb0be4479681960b093f6b398f.Background = new Bridge.global.Windows.UI.Xaml.Media.SolidColorBrush.$ctor1(($t1 = new Bridge.global.Windows.UI.Color(), $t1.A = 17, $t1.R = 0, $t1.G = 0, $t1.B = 0, $t1));
                    Border_6461f3cb0be4479681960b093f6b398f.Opacity = 0.0;
                    var Binding_6e23fb1daac448f88013f86f81d38891 = new Bridge.global.Windows.UI.Xaml.Data.Binding.ctor();
                    Binding_6e23fb1daac448f88013f86f81d38891.Path = new Bridge.global.Windows.UI.Xaml.PropertyPath.ctor("BorderBrush");
                    var RelativeSource_171e39d864c4437da4b785b4f40c195e = new Bridge.global.Windows.UI.Xaml.Data.RelativeSource.ctor();
                    RelativeSource_171e39d864c4437da4b785b4f40c195e.Mode = Bridge.global.Windows.UI.Xaml.Data.RelativeSourceMode.TemplatedParent;

                    Binding_6e23fb1daac448f88013f86f81d38891.RelativeSource = RelativeSource_171e39d864c4437da4b785b4f40c195e;


                    Binding_6e23fb1daac448f88013f86f81d38891.TemplateOwner = templateInstance_684e62ee889e40619092425d2e66a1b6;

                    var Binding_0c3732976e084832b83bcb7d8376ffe3 = new Bridge.global.Windows.UI.Xaml.Data.Binding.ctor();
                    Binding_0c3732976e084832b83bcb7d8376ffe3.Path = new Bridge.global.Windows.UI.Xaml.PropertyPath.ctor("BorderThickness");
                    var RelativeSource_8135f33171e747b6a327e0e215a57e55 = new Bridge.global.Windows.UI.Xaml.Data.RelativeSource.ctor();
                    RelativeSource_8135f33171e747b6a327e0e215a57e55.Mode = Bridge.global.Windows.UI.Xaml.Data.RelativeSourceMode.TemplatedParent;

                    Binding_0c3732976e084832b83bcb7d8376ffe3.RelativeSource = RelativeSource_8135f33171e747b6a327e0e215a57e55;


                    Binding_0c3732976e084832b83bcb7d8376ffe3.TemplateOwner = templateInstance_684e62ee889e40619092425d2e66a1b6;


                    templateOwner_ControlTemplate_fdc984696fe445eba0c7530491f12992.RegisterName("BackgroundPressed", Border_5e4a7caacfde480e89b5b61c6778b5bf);
                    Border_5e4a7caacfde480e89b5b61c6778b5bf.Name = "BackgroundPressed";
                    Border_5e4a7caacfde480e89b5b61c6778b5bf.Background = new Bridge.global.Windows.UI.Xaml.Media.SolidColorBrush.$ctor1(($t1 = new Bridge.global.Windows.UI.Color(), $t1.A = 34, $t1.R = 0, $t1.G = 0, $t1.B = 0, $t1));
                    Border_5e4a7caacfde480e89b5b61c6778b5bf.Opacity = 0.0;
                    var Binding_e2319d8272df4b5db0f288cae97983ae = new Bridge.global.Windows.UI.Xaml.Data.Binding.ctor();
                    Binding_e2319d8272df4b5db0f288cae97983ae.Path = new Bridge.global.Windows.UI.Xaml.PropertyPath.ctor("BorderBrush");
                    var RelativeSource_7196091d50e44fe5860827c70bc611d7 = new Bridge.global.Windows.UI.Xaml.Data.RelativeSource.ctor();
                    RelativeSource_7196091d50e44fe5860827c70bc611d7.Mode = Bridge.global.Windows.UI.Xaml.Data.RelativeSourceMode.TemplatedParent;

                    Binding_e2319d8272df4b5db0f288cae97983ae.RelativeSource = RelativeSource_7196091d50e44fe5860827c70bc611d7;


                    Binding_e2319d8272df4b5db0f288cae97983ae.TemplateOwner = templateInstance_684e62ee889e40619092425d2e66a1b6;

                    var Binding_d8b714549a6d4a37a1164c80ad6668d6 = new Bridge.global.Windows.UI.Xaml.Data.Binding.ctor();
                    Binding_d8b714549a6d4a37a1164c80ad6668d6.Path = new Bridge.global.Windows.UI.Xaml.PropertyPath.ctor("BorderThickness");
                    var RelativeSource_21d75843fe694ff8b9f2e9df01b31350 = new Bridge.global.Windows.UI.Xaml.Data.RelativeSource.ctor();
                    RelativeSource_21d75843fe694ff8b9f2e9df01b31350.Mode = Bridge.global.Windows.UI.Xaml.Data.RelativeSourceMode.TemplatedParent;

                    Binding_d8b714549a6d4a37a1164c80ad6668d6.RelativeSource = RelativeSource_21d75843fe694ff8b9f2e9df01b31350;


                    Binding_d8b714549a6d4a37a1164c80ad6668d6.TemplateOwner = templateInstance_684e62ee889e40619092425d2e66a1b6;


                    templateOwner_ControlTemplate_fdc984696fe445eba0c7530491f12992.RegisterName("BackgroundDisabled", Border_bbca27caeec44de6bf64e3077293e38e);
                    Border_bbca27caeec44de6bf64e3077293e38e.Name = "BackgroundDisabled";
                    Border_bbca27caeec44de6bf64e3077293e38e.Background = new Bridge.global.Windows.UI.Xaml.Media.SolidColorBrush.$ctor1(($t1 = new Bridge.global.Windows.UI.Color(), $t1.A = 51, $t1.R = 255, $t1.G = 255, $t1.B = 255, $t1));
                    Border_bbca27caeec44de6bf64e3077293e38e.Opacity = 0.0;
                    var Binding_38de2d73aa2e4b208980313103195899 = new Bridge.global.Windows.UI.Xaml.Data.Binding.ctor();
                    Binding_38de2d73aa2e4b208980313103195899.Path = new Bridge.global.Windows.UI.Xaml.PropertyPath.ctor("BorderBrush");
                    var RelativeSource_98ddfa9fe5a040ea9843243b912c36b4 = new Bridge.global.Windows.UI.Xaml.Data.RelativeSource.ctor();
                    RelativeSource_98ddfa9fe5a040ea9843243b912c36b4.Mode = Bridge.global.Windows.UI.Xaml.Data.RelativeSourceMode.TemplatedParent;

                    Binding_38de2d73aa2e4b208980313103195899.RelativeSource = RelativeSource_98ddfa9fe5a040ea9843243b912c36b4;


                    Binding_38de2d73aa2e4b208980313103195899.TemplateOwner = templateInstance_684e62ee889e40619092425d2e66a1b6;

                    var Binding_d4e5d909e0ee4b7ebab6d0b2251b54eb = new Bridge.global.Windows.UI.Xaml.Data.Binding.ctor();
                    Binding_d4e5d909e0ee4b7ebab6d0b2251b54eb.Path = new Bridge.global.Windows.UI.Xaml.PropertyPath.ctor("BorderThickness");
                    var RelativeSource_1d48e0cf2cf742e786862bfac1f37483 = new Bridge.global.Windows.UI.Xaml.Data.RelativeSource.ctor();
                    RelativeSource_1d48e0cf2cf742e786862bfac1f37483.Mode = Bridge.global.Windows.UI.Xaml.Data.RelativeSourceMode.TemplatedParent;

                    Binding_d4e5d909e0ee4b7ebab6d0b2251b54eb.RelativeSource = RelativeSource_1d48e0cf2cf742e786862bfac1f37483;


                    Binding_d4e5d909e0ee4b7ebab6d0b2251b54eb.TemplateOwner = templateInstance_684e62ee889e40619092425d2e66a1b6;


                    Grid_17f39a3d379c4e66bb334a8c888a0751.Children.add(Border_58d9622386ff4b75a2d15aebed5ad9ad);
                    Grid_17f39a3d379c4e66bb334a8c888a0751.Children.add(Border_6461f3cb0be4479681960b093f6b398f);
                    Grid_17f39a3d379c4e66bb334a8c888a0751.Children.add(Border_5e4a7caacfde480e89b5b61c6778b5bf);
                    Grid_17f39a3d379c4e66bb334a8c888a0751.Children.add(Border_bbca27caeec44de6bf64e3077293e38e);


                    Bridge.global.Windows.UI.Xaml.Data.BindingOperations.SetBinding(Border_58d9622386ff4b75a2d15aebed5ad9ad, Bridge.global.Windows.UI.Xaml.Controls.Border.BackgroundProperty, Binding_8cde840a08eb4c739db304f23797e2fd);
                    Bridge.global.Windows.UI.Xaml.Data.BindingOperations.SetBinding(Border_58d9622386ff4b75a2d15aebed5ad9ad, Bridge.global.Windows.UI.Xaml.Controls.Border.BorderBrushProperty, Binding_ba9a493800524dcc9c23993841321765);
                    Bridge.global.Windows.UI.Xaml.Data.BindingOperations.SetBinding(Border_58d9622386ff4b75a2d15aebed5ad9ad, Bridge.global.Windows.UI.Xaml.Controls.Border.BorderThicknessProperty, Binding_8cc6f18be0e94ac59cc2384b0e887047);
                    Bridge.global.Windows.UI.Xaml.Data.BindingOperations.SetBinding(Border_6461f3cb0be4479681960b093f6b398f, Bridge.global.Windows.UI.Xaml.Controls.Border.BorderBrushProperty, Binding_6e23fb1daac448f88013f86f81d38891);
                    Bridge.global.Windows.UI.Xaml.Data.BindingOperations.SetBinding(Border_6461f3cb0be4479681960b093f6b398f, Bridge.global.Windows.UI.Xaml.Controls.Border.BorderThicknessProperty, Binding_0c3732976e084832b83bcb7d8376ffe3);
                    Bridge.global.Windows.UI.Xaml.Data.BindingOperations.SetBinding(Border_5e4a7caacfde480e89b5b61c6778b5bf, Bridge.global.Windows.UI.Xaml.Controls.Border.BorderBrushProperty, Binding_e2319d8272df4b5db0f288cae97983ae);
                    Bridge.global.Windows.UI.Xaml.Data.BindingOperations.SetBinding(Border_5e4a7caacfde480e89b5b61c6778b5bf, Bridge.global.Windows.UI.Xaml.Controls.Border.BorderThicknessProperty, Binding_d8b714549a6d4a37a1164c80ad6668d6);
                    Bridge.global.Windows.UI.Xaml.Data.BindingOperations.SetBinding(Border_bbca27caeec44de6bf64e3077293e38e, Bridge.global.Windows.UI.Xaml.Controls.Border.BorderBrushProperty, Binding_38de2d73aa2e4b208980313103195899);
                    Bridge.global.Windows.UI.Xaml.Data.BindingOperations.SetBinding(Border_bbca27caeec44de6bf64e3077293e38e, Bridge.global.Windows.UI.Xaml.Controls.Border.BorderThicknessProperty, Binding_d4e5d909e0ee4b7ebab6d0b2251b54eb);

                    Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard.SetTargetProperty(DoubleAnimation_9c40079cf5ad4672932f4b19de205919, new Bridge.global.Windows.UI.Xaml.PropertyPath.$ctor1("Opacity", "Opacity", Bridge.fn.cacheBind(this, this.accessVisualStateProperty_6b2bc3610c544c818a21d04e97b1c71d), Bridge.fn.cacheBind(this, this.setVisualStateProperty_6b2bc3610c544c818a21d04e97b1c71d), Bridge.fn.cacheBind(this, this.setAnimationVisualStateProperty_6b2bc3610c544c818a21d04e97b1c71d), Bridge.fn.cacheBind(this, this.setLocalVisualStateProperty_6b2bc3610c544c818a21d04e97b1c71d), Bridge.fn.cacheBind(this, this.getVisualStateProperty_6b2bc3610c544c818a21d04e97b1c71d)));
                    Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard.SetTarget(DoubleAnimation_9c40079cf5ad4672932f4b19de205919, Border_6461f3cb0be4479681960b093f6b398f);


                    Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard.SetTargetProperty(DoubleAnimation_63ec88adadd848bc8067e3ea4cd670a6, new Bridge.global.Windows.UI.Xaml.PropertyPath.$ctor1("Opacity", "Opacity", Bridge.fn.cacheBind(this, this.accessVisualStateProperty_7bd8a098a38c4f9fbf6a92f9ed949edf), Bridge.fn.cacheBind(this, this.setVisualStateProperty_7bd8a098a38c4f9fbf6a92f9ed949edf), Bridge.fn.cacheBind(this, this.setAnimationVisualStateProperty_7bd8a098a38c4f9fbf6a92f9ed949edf), Bridge.fn.cacheBind(this, this.setLocalVisualStateProperty_7bd8a098a38c4f9fbf6a92f9ed949edf), Bridge.fn.cacheBind(this, this.getVisualStateProperty_7bd8a098a38c4f9fbf6a92f9ed949edf)));
                    Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard.SetTarget(DoubleAnimation_63ec88adadd848bc8067e3ea4cd670a6, Border_58d9622386ff4b75a2d15aebed5ad9ad);


                    Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard.SetTargetProperty(DoubleAnimation_2d55cf9f7822462e83a7c41c8529e841, new Bridge.global.Windows.UI.Xaml.PropertyPath.$ctor1("Opacity", "Opacity", Bridge.fn.cacheBind(this, this.accessVisualStateProperty_c7aabeded409438eb5068c8366bb5e7a), Bridge.fn.cacheBind(this, this.setVisualStateProperty_c7aabeded409438eb5068c8366bb5e7a), Bridge.fn.cacheBind(this, this.setAnimationVisualStateProperty_c7aabeded409438eb5068c8366bb5e7a), Bridge.fn.cacheBind(this, this.setLocalVisualStateProperty_c7aabeded409438eb5068c8366bb5e7a), Bridge.fn.cacheBind(this, this.getVisualStateProperty_c7aabeded409438eb5068c8366bb5e7a)));
                    Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard.SetTarget(DoubleAnimation_2d55cf9f7822462e83a7c41c8529e841, Border_5e4a7caacfde480e89b5b61c6778b5bf);


                    Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard.SetTargetProperty(DoubleAnimation_cbf8a095e5b14e1caeb9ca4ef4a38e41, new Bridge.global.Windows.UI.Xaml.PropertyPath.$ctor1("Opacity", "Opacity", Bridge.fn.cacheBind(this, this.accessVisualStateProperty_7760f6ed5e6843e9a6ee1c1d00bb88f0), Bridge.fn.cacheBind(this, this.setVisualStateProperty_7760f6ed5e6843e9a6ee1c1d00bb88f0), Bridge.fn.cacheBind(this, this.setAnimationVisualStateProperty_7760f6ed5e6843e9a6ee1c1d00bb88f0), Bridge.fn.cacheBind(this, this.setLocalVisualStateProperty_7760f6ed5e6843e9a6ee1c1d00bb88f0), Bridge.fn.cacheBind(this, this.getVisualStateProperty_7760f6ed5e6843e9a6ee1c1d00bb88f0)));
                    Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard.SetTarget(DoubleAnimation_cbf8a095e5b14e1caeb9ca4ef4a38e41, Border_58d9622386ff4b75a2d15aebed5ad9ad);


                    Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard.SetTargetProperty(DoubleAnimation_300cae50b5704817929b4c4fecadce1b, new Bridge.global.Windows.UI.Xaml.PropertyPath.$ctor1("Opacity", "Opacity", Bridge.fn.cacheBind(this, this.accessVisualStateProperty_5883215d23c54dd0ba3738ce433af91f), Bridge.fn.cacheBind(this, this.setVisualStateProperty_5883215d23c54dd0ba3738ce433af91f), Bridge.fn.cacheBind(this, this.setAnimationVisualStateProperty_5883215d23c54dd0ba3738ce433af91f), Bridge.fn.cacheBind(this, this.setLocalVisualStateProperty_5883215d23c54dd0ba3738ce433af91f), Bridge.fn.cacheBind(this, this.getVisualStateProperty_5883215d23c54dd0ba3738ce433af91f)));
                    Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard.SetTarget(DoubleAnimation_300cae50b5704817929b4c4fecadce1b, Border_bbca27caeec44de6bf64e3077293e38e);


                    Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard.SetTargetProperty(DoubleAnimation_ac0aac1f89084e20b64593926003885a, new Bridge.global.Windows.UI.Xaml.PropertyPath.$ctor1("Opacity", "Opacity", Bridge.fn.cacheBind(this, this.accessVisualStateProperty_93879d04fd284634a2cc67f14f07094e), Bridge.fn.cacheBind(this, this.setVisualStateProperty_93879d04fd284634a2cc67f14f07094e), Bridge.fn.cacheBind(this, this.setAnimationVisualStateProperty_93879d04fd284634a2cc67f14f07094e), Bridge.fn.cacheBind(this, this.setLocalVisualStateProperty_93879d04fd284634a2cc67f14f07094e), Bridge.fn.cacheBind(this, this.getVisualStateProperty_93879d04fd284634a2cc67f14f07094e)));
                    Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard.SetTarget(DoubleAnimation_ac0aac1f89084e20b64593926003885a, Border_58d9622386ff4b75a2d15aebed5ad9ad);

                    templateInstance_684e62ee889e40619092425d2e66a1b6.TemplateContent = Grid_17f39a3d379c4e66bb334a8c888a0751;
                    return templateInstance_684e62ee889e40619092425d2e66a1b6;
                }));

                Setter_0bca64b079774b518d6efcc3096003f9.Value = ControlTemplate_fdc984696fe445eba0c7530491f12992;


                Style_547d060bf12e40168c05af039e4c3829.Setters.add(Setter_b9a248b44efa4daa9d0c861801f1477e);
                Style_547d060bf12e40168c05af039e4c3829.Setters.add(Setter_a38ba0fe8e0548b6b5b71a95fa48feb8);
                Style_547d060bf12e40168c05af039e4c3829.Setters.add(Setter_0bca64b079774b518d6efcc3096003f9);


                this.setItem(Bridge.global.Windows.UI.Xaml.Controls.Primitives.Thumb, Style_547d060bf12e40168c05af039e4c3829);






            },
            accessVisualStateProperty_6b2bc3610c544c818a21d04e97b1c71d: function (rootTargetObjectInstance) {
                return new (Bridge.GeneratorEnumerable$1(System.Tuple$3(Windows.UI.Xaml.DependencyObject,Windows.UI.Xaml.DependencyProperty,System.Nullable$1(System.Int32))))(Bridge.fn.bind(this, function (rootTargetObjectInstance) {
                    var $step = 0,
                        $jumpFromFinally,
                        $returnValue,
                        $async_e;

                    var $enumerator = new (Bridge.GeneratorEnumerator$1(System.Tuple$3(Windows.UI.Xaml.DependencyObject,Windows.UI.Xaml.DependencyProperty,System.Nullable$1(System.Int32))))(Bridge.fn.bind(this, function () {
                        try {
                            for (;;) {
                                switch ($step) {
                                    case 0: {
                                        return false;
                                    }
                                    default: {
                                        return false;
                                    }
                                }
                            }
                        } catch($async_e1) {
                            $async_e = System.Exception.create($async_e1);
                            throw $async_e;
                        }
                    }));
                    return $enumerator;
                }, arguments));
            },
            setVisualStateProperty_6b2bc3610c544c818a21d04e97b1c71d: function (finalTargetInstance, value) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Opacity").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "OpacityProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                (finalTargetInstance).SetVisualStateValue(property, value);
            },
            setAnimationVisualStateProperty_6b2bc3610c544c818a21d04e97b1c71d: function (finalTargetInstance, value) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Opacity").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "OpacityProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                (finalTargetInstance).SetAnimationValue(property, value);
            },
            setLocalVisualStateProperty_6b2bc3610c544c818a21d04e97b1c71d: function (finalTargetInstance, value) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Opacity").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "OpacityProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                (finalTargetInstance).SetLocalValue(property, value);
            },
            getVisualStateProperty_6b2bc3610c544c818a21d04e97b1c71d: function (finalTargetInstance) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Opacity").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "OpacityProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                return finalTargetInstance.GetVisualStateValue(property);
            },
            accessVisualStateProperty_7bd8a098a38c4f9fbf6a92f9ed949edf: function (rootTargetObjectInstance) {
                return new (Bridge.GeneratorEnumerable$1(System.Tuple$3(Windows.UI.Xaml.DependencyObject,Windows.UI.Xaml.DependencyProperty,System.Nullable$1(System.Int32))))(Bridge.fn.bind(this, function (rootTargetObjectInstance) {
                    var $step = 0,
                        $jumpFromFinally,
                        $returnValue,
                        $async_e;

                    var $enumerator = new (Bridge.GeneratorEnumerator$1(System.Tuple$3(Windows.UI.Xaml.DependencyObject,Windows.UI.Xaml.DependencyProperty,System.Nullable$1(System.Int32))))(Bridge.fn.bind(this, function () {
                        try {
                            for (;;) {
                                switch ($step) {
                                    case 0: {
                                        return false;
                                    }
                                    default: {
                                        return false;
                                    }
                                }
                            }
                        } catch($async_e1) {
                            $async_e = System.Exception.create($async_e1);
                            throw $async_e;
                        }
                    }));
                    return $enumerator;
                }, arguments));
            },
            setVisualStateProperty_7bd8a098a38c4f9fbf6a92f9ed949edf: function (finalTargetInstance, value) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Opacity").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "OpacityProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                (finalTargetInstance).SetVisualStateValue(property, value);
            },
            setAnimationVisualStateProperty_7bd8a098a38c4f9fbf6a92f9ed949edf: function (finalTargetInstance, value) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Opacity").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "OpacityProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                (finalTargetInstance).SetAnimationValue(property, value);
            },
            setLocalVisualStateProperty_7bd8a098a38c4f9fbf6a92f9ed949edf: function (finalTargetInstance, value) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Opacity").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "OpacityProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                (finalTargetInstance).SetLocalValue(property, value);
            },
            getVisualStateProperty_7bd8a098a38c4f9fbf6a92f9ed949edf: function (finalTargetInstance) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Opacity").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "OpacityProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                return finalTargetInstance.GetVisualStateValue(property);
            },
            accessVisualStateProperty_c7aabeded409438eb5068c8366bb5e7a: function (rootTargetObjectInstance) {
                return new (Bridge.GeneratorEnumerable$1(System.Tuple$3(Windows.UI.Xaml.DependencyObject,Windows.UI.Xaml.DependencyProperty,System.Nullable$1(System.Int32))))(Bridge.fn.bind(this, function (rootTargetObjectInstance) {
                    var $step = 0,
                        $jumpFromFinally,
                        $returnValue,
                        $async_e;

                    var $enumerator = new (Bridge.GeneratorEnumerator$1(System.Tuple$3(Windows.UI.Xaml.DependencyObject,Windows.UI.Xaml.DependencyProperty,System.Nullable$1(System.Int32))))(Bridge.fn.bind(this, function () {
                        try {
                            for (;;) {
                                switch ($step) {
                                    case 0: {
                                        return false;
                                    }
                                    default: {
                                        return false;
                                    }
                                }
                            }
                        } catch($async_e1) {
                            $async_e = System.Exception.create($async_e1);
                            throw $async_e;
                        }
                    }));
                    return $enumerator;
                }, arguments));
            },
            setVisualStateProperty_c7aabeded409438eb5068c8366bb5e7a: function (finalTargetInstance, value) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Opacity").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "OpacityProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                (finalTargetInstance).SetVisualStateValue(property, value);
            },
            setAnimationVisualStateProperty_c7aabeded409438eb5068c8366bb5e7a: function (finalTargetInstance, value) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Opacity").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "OpacityProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                (finalTargetInstance).SetAnimationValue(property, value);
            },
            setLocalVisualStateProperty_c7aabeded409438eb5068c8366bb5e7a: function (finalTargetInstance, value) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Opacity").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "OpacityProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                (finalTargetInstance).SetLocalValue(property, value);
            },
            getVisualStateProperty_c7aabeded409438eb5068c8366bb5e7a: function (finalTargetInstance) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Opacity").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "OpacityProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                return finalTargetInstance.GetVisualStateValue(property);
            },
            accessVisualStateProperty_7760f6ed5e6843e9a6ee1c1d00bb88f0: function (rootTargetObjectInstance) {
                return new (Bridge.GeneratorEnumerable$1(System.Tuple$3(Windows.UI.Xaml.DependencyObject,Windows.UI.Xaml.DependencyProperty,System.Nullable$1(System.Int32))))(Bridge.fn.bind(this, function (rootTargetObjectInstance) {
                    var $step = 0,
                        $jumpFromFinally,
                        $returnValue,
                        $async_e;

                    var $enumerator = new (Bridge.GeneratorEnumerator$1(System.Tuple$3(Windows.UI.Xaml.DependencyObject,Windows.UI.Xaml.DependencyProperty,System.Nullable$1(System.Int32))))(Bridge.fn.bind(this, function () {
                        try {
                            for (;;) {
                                switch ($step) {
                                    case 0: {
                                        return false;
                                    }
                                    default: {
                                        return false;
                                    }
                                }
                            }
                        } catch($async_e1) {
                            $async_e = System.Exception.create($async_e1);
                            throw $async_e;
                        }
                    }));
                    return $enumerator;
                }, arguments));
            },
            setVisualStateProperty_7760f6ed5e6843e9a6ee1c1d00bb88f0: function (finalTargetInstance, value) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Opacity").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "OpacityProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                (finalTargetInstance).SetVisualStateValue(property, value);
            },
            setAnimationVisualStateProperty_7760f6ed5e6843e9a6ee1c1d00bb88f0: function (finalTargetInstance, value) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Opacity").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "OpacityProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                (finalTargetInstance).SetAnimationValue(property, value);
            },
            setLocalVisualStateProperty_7760f6ed5e6843e9a6ee1c1d00bb88f0: function (finalTargetInstance, value) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Opacity").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "OpacityProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                (finalTargetInstance).SetLocalValue(property, value);
            },
            getVisualStateProperty_7760f6ed5e6843e9a6ee1c1d00bb88f0: function (finalTargetInstance) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Opacity").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "OpacityProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                return finalTargetInstance.GetVisualStateValue(property);
            },
            accessVisualStateProperty_5883215d23c54dd0ba3738ce433af91f: function (rootTargetObjectInstance) {
                return new (Bridge.GeneratorEnumerable$1(System.Tuple$3(Windows.UI.Xaml.DependencyObject,Windows.UI.Xaml.DependencyProperty,System.Nullable$1(System.Int32))))(Bridge.fn.bind(this, function (rootTargetObjectInstance) {
                    var $step = 0,
                        $jumpFromFinally,
                        $returnValue,
                        $async_e;

                    var $enumerator = new (Bridge.GeneratorEnumerator$1(System.Tuple$3(Windows.UI.Xaml.DependencyObject,Windows.UI.Xaml.DependencyProperty,System.Nullable$1(System.Int32))))(Bridge.fn.bind(this, function () {
                        try {
                            for (;;) {
                                switch ($step) {
                                    case 0: {
                                        return false;
                                    }
                                    default: {
                                        return false;
                                    }
                                }
                            }
                        } catch($async_e1) {
                            $async_e = System.Exception.create($async_e1);
                            throw $async_e;
                        }
                    }));
                    return $enumerator;
                }, arguments));
            },
            setVisualStateProperty_5883215d23c54dd0ba3738ce433af91f: function (finalTargetInstance, value) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Opacity").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "OpacityProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                (finalTargetInstance).SetVisualStateValue(property, value);
            },
            setAnimationVisualStateProperty_5883215d23c54dd0ba3738ce433af91f: function (finalTargetInstance, value) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Opacity").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "OpacityProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                (finalTargetInstance).SetAnimationValue(property, value);
            },
            setLocalVisualStateProperty_5883215d23c54dd0ba3738ce433af91f: function (finalTargetInstance, value) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Opacity").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "OpacityProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                (finalTargetInstance).SetLocalValue(property, value);
            },
            getVisualStateProperty_5883215d23c54dd0ba3738ce433af91f: function (finalTargetInstance) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Opacity").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "OpacityProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                return finalTargetInstance.GetVisualStateValue(property);
            },
            accessVisualStateProperty_93879d04fd284634a2cc67f14f07094e: function (rootTargetObjectInstance) {
                return new (Bridge.GeneratorEnumerable$1(System.Tuple$3(Windows.UI.Xaml.DependencyObject,Windows.UI.Xaml.DependencyProperty,System.Nullable$1(System.Int32))))(Bridge.fn.bind(this, function (rootTargetObjectInstance) {
                    var $step = 0,
                        $jumpFromFinally,
                        $returnValue,
                        $async_e;

                    var $enumerator = new (Bridge.GeneratorEnumerator$1(System.Tuple$3(Windows.UI.Xaml.DependencyObject,Windows.UI.Xaml.DependencyProperty,System.Nullable$1(System.Int32))))(Bridge.fn.bind(this, function () {
                        try {
                            for (;;) {
                                switch ($step) {
                                    case 0: {
                                        return false;
                                    }
                                    default: {
                                        return false;
                                    }
                                }
                            }
                        } catch($async_e1) {
                            $async_e = System.Exception.create($async_e1);
                            throw $async_e;
                        }
                    }));
                    return $enumerator;
                }, arguments));
            },
            setVisualStateProperty_93879d04fd284634a2cc67f14f07094e: function (finalTargetInstance, value) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Opacity").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "OpacityProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                (finalTargetInstance).SetVisualStateValue(property, value);
            },
            setAnimationVisualStateProperty_93879d04fd284634a2cc67f14f07094e: function (finalTargetInstance, value) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Opacity").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "OpacityProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                (finalTargetInstance).SetAnimationValue(property, value);
            },
            setLocalVisualStateProperty_93879d04fd284634a2cc67f14f07094e: function (finalTargetInstance, value) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Opacity").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "OpacityProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                (finalTargetInstance).SetLocalValue(property, value);
            },
            getVisualStateProperty_93879d04fd284634a2cc67f14f07094e: function (finalTargetInstance) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Opacity").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "OpacityProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                return finalTargetInstance.GetVisualStateValue(property);
            }
        }
    });

    Bridge.define("___CSHTML5__Component__Defaultstyles__INTERNAL_Defaulttileviewitemstyle__Xaml", {
        inherits: [Bridge.global.Windows.UI.Xaml.ResourceDictionary],
        fields: {
            _contentLoaded: false
        },
        ctors: {
            ctor: function () {
                this.$initialize();
                Bridge.global.Windows.UI.Xaml.ResourceDictionary.ctor.call(this);
                this.InitializeComponent();

                if (Bridge.is(this, Windows.UI.Xaml.UIElement)) {
                    Bridge.cast(this, Windows.UI.Xaml.UIElement).XamlSourcePath = "CSHTML5\\DefaultStyles\\INTERNAL_DefaultTileViewItemStyle.xaml";
                }
            }
        },
        methods: {
            InitializeComponent: function () {
                if (this._contentLoaded) {
                    return;
                }
                this._contentLoaded = true;


                if (Bridge.is(this, Windows.UI.Xaml.UIElement)) {
                    Bridge.cast(this, Windows.UI.Xaml.UIElement).XamlSourcePath = "CSHTML5\\DefaultStyles\\INTERNAL_DefaultTileViewItemStyle.xaml";
                }



                var Style_2bbf2d757d60443dbaffbbe0dae6a448 = new Bridge.global.Windows.UI.Xaml.Style.ctor();
                Style_2bbf2d757d60443dbaffbbe0dae6a448.TargetType = Bridge.global.Windows.UI.Xaml.Controls.TileViewItem;
                var Setter_ffcb0d13773b4ebf883fbdb19f92d785 = new Bridge.global.Windows.UI.Xaml.Setter.ctor();
                Setter_ffcb0d13773b4ebf883fbdb19f92d785.Property = Bridge.global.Windows.UI.Xaml.Controls.Control.TemplateProperty;
                var ControlTemplate_f92ce249039347c79b426a22cb836673 = new Bridge.global.Windows.UI.Xaml.Controls.ControlTemplate();
                ControlTemplate_f92ce249039347c79b426a22cb836673.SetMethodToInstantiateFrameworkTemplate(Bridge.fn.bind(this, function (templateOwner_ControlTemplate_f92ce249039347c79b426a22cb836673) {
                    var $t;
                    var templateInstance_63054cafb952411c96cdc5ead0daf6f4 = new Bridge.global.Windows.UI.Xaml.TemplateInstance();
                    templateInstance_63054cafb952411c96cdc5ead0daf6f4.TemplateOwner = templateOwner_ControlTemplate_f92ce249039347c79b426a22cb836673;
                    var VisualState_2bee5949f56e420387cadf18474065fb = new Bridge.global.Windows.UI.Xaml.VisualState();
                    var VisualState_bc8ad687349440b1905d2f0b67f42d80 = new Bridge.global.Windows.UI.Xaml.VisualState();
                    var VisualStateGroup_35653e54b51246b09d48a187a1b6b960 = new Bridge.global.Windows.UI.Xaml.VisualStateGroup();
                    var Button_93248fb1f54c48298355a35f5638f613 = new Bridge.global.Windows.UI.Xaml.Controls.Button();
                    var Border_4d93900e94224740bdcc71e5956a5c88 = new Bridge.global.Windows.UI.Xaml.Controls.Border();
                    Border_4d93900e94224740bdcc71e5956a5c88.BorderBrush = new Bridge.global.Windows.UI.Xaml.Media.SolidColorBrush.$ctor1(($t = new Bridge.global.Windows.UI.Color(), $t.A = 255, $t.R = 109, $t.G = 149, $t.B = 161, $t));
                    Border_4d93900e94224740bdcc71e5956a5c88.BorderThickness = new Bridge.global.Windows.UI.Xaml.Thickness.$ctor1(1);
                    Border_4d93900e94224740bdcc71e5956a5c88.Margin = new Bridge.global.Windows.UI.Xaml.Thickness.$ctor1(5);
                    templateOwner_ControlTemplate_f92ce249039347c79b426a22cb836673.RegisterName("SizeStates", VisualStateGroup_35653e54b51246b09d48a187a1b6b960);
                    VisualStateGroup_35653e54b51246b09d48a187a1b6b960.Name = "SizeStates";
                    templateOwner_ControlTemplate_f92ce249039347c79b426a22cb836673.RegisterName("Minimized", VisualState_2bee5949f56e420387cadf18474065fb);
                    VisualState_2bee5949f56e420387cadf18474065fb.Name = "Minimized";

                    templateOwner_ControlTemplate_f92ce249039347c79b426a22cb836673.RegisterName("Maximized", VisualState_bc8ad687349440b1905d2f0b67f42d80);
                    VisualState_bc8ad687349440b1905d2f0b67f42d80.Name = "Maximized";
                    var Storyboard_ec5f76990b5c4bafb8f4b7f88d851f59 = new Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard();
                    var ObjectAnimationUsingKeyFrames_a7e6e215c98646deb4c0bacec05015e1 = new Bridge.global.Windows.UI.Xaml.Media.Animation.ObjectAnimationUsingKeyFrames();
                    ObjectAnimationUsingKeyFrames_a7e6e215c98646deb4c0bacec05015e1.Duration = new Bridge.global.Windows.UI.Xaml.Duration.$ctor1(new Bridge.global.System.TimeSpan(System.Int64(0)));
                    Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard.SetTargetName(ObjectAnimationUsingKeyFrames_a7e6e215c98646deb4c0bacec05015e1, "PART_MaximizeButton");
                    var DiscreteObjectKeyFrame_a20c34b8f6e24a6c96d2c8ded36a9c4e = new Bridge.global.Windows.UI.Xaml.Media.Animation.DiscreteObjectKeyFrame();
                    DiscreteObjectKeyFrame_a20c34b8f6e24a6c96d2c8ded36a9c4e.KeyTime = Bridge.global.Windows.UI.Xaml.Media.Animation.KeyTime.FromTimeSpan(new Bridge.global.System.TimeSpan(System.Int64(0)));
                    DiscreteObjectKeyFrame_a20c34b8f6e24a6c96d2c8ded36a9c4e.Value = "Collapsed";

                    ObjectAnimationUsingKeyFrames_a7e6e215c98646deb4c0bacec05015e1.KeyFrames.add(DiscreteObjectKeyFrame_a20c34b8f6e24a6c96d2c8ded36a9c4e);


                    Storyboard_ec5f76990b5c4bafb8f4b7f88d851f59.Children.add(ObjectAnimationUsingKeyFrames_a7e6e215c98646deb4c0bacec05015e1);


                    VisualState_bc8ad687349440b1905d2f0b67f42d80.Storyboard = Storyboard_ec5f76990b5c4bafb8f4b7f88d851f59;


                    System.Array.add(VisualStateGroup_35653e54b51246b09d48a187a1b6b960.States, VisualState_2bee5949f56e420387cadf18474065fb, Object);
                    System.Array.add(VisualStateGroup_35653e54b51246b09d48a187a1b6b960.States, VisualState_bc8ad687349440b1905d2f0b67f42d80, Object);


                    templateOwner_ControlTemplate_f92ce249039347c79b426a22cb836673.INTERNAL_GetVisualStateGroups().add(VisualStateGroup_35653e54b51246b09d48a187a1b6b960);

                    var Grid_e4d004867e9444c7aef1e4a06366aead = new Bridge.global.Windows.UI.Xaml.Controls.Grid();
                    var RowDefinition_f898c0f9cbb94364a964edb796e09913 = new Bridge.global.Windows.UI.Xaml.Controls.RowDefinition();
                    RowDefinition_f898c0f9cbb94364a964edb796e09913.Height = new Bridge.global.Windows.UI.Xaml.GridLength.$ctor2(1.0, Bridge.global.Windows.UI.Xaml.GridUnitType.Auto);

                    var RowDefinition_190740f051364688af1c4634fd8bfb88 = new Bridge.global.Windows.UI.Xaml.Controls.RowDefinition();
                    RowDefinition_190740f051364688af1c4634fd8bfb88.Height = new Bridge.global.Windows.UI.Xaml.GridLength.$ctor2(1.0, Bridge.global.Windows.UI.Xaml.GridUnitType.Star);

                    Grid_e4d004867e9444c7aef1e4a06366aead.RowDefinitions.add(RowDefinition_f898c0f9cbb94364a964edb796e09913);
                    Grid_e4d004867e9444c7aef1e4a06366aead.RowDefinitions.add(RowDefinition_190740f051364688af1c4634fd8bfb88);

                    var Border_91c9b211c8824e53bf5e264424057d43 = new Bridge.global.Windows.UI.Xaml.Controls.Border();
                    Border_91c9b211c8824e53bf5e264424057d43.Background = new Bridge.global.Windows.UI.Xaml.Media.SolidColorBrush.$ctor1(($t = new Bridge.global.Windows.UI.Color(), $t.A = 255, $t.R = 240, $t.G = 240, $t.B = 240, $t));
                    Border_91c9b211c8824e53bf5e264424057d43.BorderBrush = new Bridge.global.Windows.UI.Xaml.Media.SolidColorBrush.$ctor1(($t = new Bridge.global.Windows.UI.Color(), $t.A = 255, $t.R = 229, $t.G = 229, $t.B = 229, $t));
                    Border_91c9b211c8824e53bf5e264424057d43.BorderThickness = new Bridge.global.Windows.UI.Xaml.Thickness.$ctor2(0, 0, 0, 1);
                    Border_91c9b211c8824e53bf5e264424057d43.Padding = new Bridge.global.Windows.UI.Xaml.Thickness.$ctor2(10, 3, 10, 3);
                    var Grid_acef88410e0c4b95a559731a83e9d37e = new Bridge.global.Windows.UI.Xaml.Controls.Grid();
                    var ColumnDefinition_2fb7b6ab4e174a788bc0c2e189616d84 = new Bridge.global.Windows.UI.Xaml.Controls.ColumnDefinition();
                    ColumnDefinition_2fb7b6ab4e174a788bc0c2e189616d84.Width = new Bridge.global.Windows.UI.Xaml.GridLength.$ctor2(1.0, Bridge.global.Windows.UI.Xaml.GridUnitType.Star);

                    var ColumnDefinition_131b576e18a14785ad48dbe7df8a9b5e = new Bridge.global.Windows.UI.Xaml.Controls.ColumnDefinition();
                    ColumnDefinition_131b576e18a14785ad48dbe7df8a9b5e.Width = new Bridge.global.Windows.UI.Xaml.GridLength.$ctor2(1.0, Bridge.global.Windows.UI.Xaml.GridUnitType.Auto);

                    Grid_acef88410e0c4b95a559731a83e9d37e.ColumnDefinitions.add(ColumnDefinition_2fb7b6ab4e174a788bc0c2e189616d84);
                    Grid_acef88410e0c4b95a559731a83e9d37e.ColumnDefinitions.add(ColumnDefinition_131b576e18a14785ad48dbe7df8a9b5e);

                    var ContentPresenter_b704aa759c964cc1a79e1a6aca366c72 = new Bridge.global.Windows.UI.Xaml.Controls.ContentPresenter();
                    var Binding_a868fa49fa004b028c163ce903ae9d3d = new Bridge.global.Windows.UI.Xaml.Data.Binding.ctor();
                    Binding_a868fa49fa004b028c163ce903ae9d3d.Path = new Bridge.global.Windows.UI.Xaml.PropertyPath.ctor("Header");
                    var RelativeSource_bd742ed7a3054a529c9694b8ea5819e3 = new Bridge.global.Windows.UI.Xaml.Data.RelativeSource.ctor();
                    RelativeSource_bd742ed7a3054a529c9694b8ea5819e3.Mode = Bridge.global.Windows.UI.Xaml.Data.RelativeSourceMode.TemplatedParent;

                    Binding_a868fa49fa004b028c163ce903ae9d3d.RelativeSource = RelativeSource_bd742ed7a3054a529c9694b8ea5819e3;


                    Binding_a868fa49fa004b028c163ce903ae9d3d.TemplateOwner = templateInstance_63054cafb952411c96cdc5ead0daf6f4;

                    var Binding_21a041209f834c309cc3f0a03e8e1fa4 = new Bridge.global.Windows.UI.Xaml.Data.Binding.ctor();
                    Binding_21a041209f834c309cc3f0a03e8e1fa4.Path = new Bridge.global.Windows.UI.Xaml.PropertyPath.ctor("ContentTemplate");
                    var RelativeSource_4b0aca198bcd4760a8442de219d7a82d = new Bridge.global.Windows.UI.Xaml.Data.RelativeSource.ctor();
                    RelativeSource_4b0aca198bcd4760a8442de219d7a82d.Mode = Bridge.global.Windows.UI.Xaml.Data.RelativeSourceMode.TemplatedParent;

                    Binding_21a041209f834c309cc3f0a03e8e1fa4.RelativeSource = RelativeSource_4b0aca198bcd4760a8442de219d7a82d;


                    Binding_21a041209f834c309cc3f0a03e8e1fa4.TemplateOwner = templateInstance_63054cafb952411c96cdc5ead0daf6f4;


                    templateOwner_ControlTemplate_f92ce249039347c79b426a22cb836673.RegisterName("PART_MaximizeButton", Button_93248fb1f54c48298355a35f5638f613);
                    Button_93248fb1f54c48298355a35f5638f613.Name = "PART_MaximizeButton";
                    Bridge.global.Windows.UI.Xaml.Controls.Grid.SetColumn(Button_93248fb1f54c48298355a35f5638f613, 1);
                    Button_93248fb1f54c48298355a35f5638f613.Margin = new Bridge.global.Windows.UI.Xaml.Thickness.$ctor1(5);
                    Button_93248fb1f54c48298355a35f5638f613.Background = new Bridge.global.Windows.UI.Xaml.Media.SolidColorBrush.$ctor1(($t = new Bridge.global.Windows.UI.Color(), $t.A = 0, $t.R = 255, $t.G = 255, $t.B = 255, $t));
                    var Border_dc73b6b5b7b047839d4e8bb4d9874bd0 = new Bridge.global.Windows.UI.Xaml.Controls.Border();
                    Border_dc73b6b5b7b047839d4e8bb4d9874bd0.Background = new Bridge.global.Windows.UI.Xaml.Media.SolidColorBrush.$ctor1(($t = new Bridge.global.Windows.UI.Color(), $t.A = 0, $t.R = 255, $t.G = 255, $t.B = 255, $t));
                    Border_dc73b6b5b7b047839d4e8bb4d9874bd0.BorderBrush = new Bridge.global.Windows.UI.Xaml.Media.SolidColorBrush.$ctor1(($t = new Bridge.global.Windows.UI.Color(), $t.A = 255, $t.R = 0, $t.G = 0, $t.B = 0, $t));
                    Border_dc73b6b5b7b047839d4e8bb4d9874bd0.BorderThickness = new Bridge.global.Windows.UI.Xaml.Thickness.$ctor2(1, 1, 1, 2);
                    Border_dc73b6b5b7b047839d4e8bb4d9874bd0.Width = 10.0;
                    Border_dc73b6b5b7b047839d4e8bb4d9874bd0.Height = 10.0;

                    Button_93248fb1f54c48298355a35f5638f613.Content = Border_dc73b6b5b7b047839d4e8bb4d9874bd0;


                    Grid_acef88410e0c4b95a559731a83e9d37e.Children.add(ContentPresenter_b704aa759c964cc1a79e1a6aca366c72);
                    Grid_acef88410e0c4b95a559731a83e9d37e.Children.add(Button_93248fb1f54c48298355a35f5638f613);


                    Border_91c9b211c8824e53bf5e264424057d43.Child = Grid_acef88410e0c4b95a559731a83e9d37e;


                    var ContentPresenter_3cd81832a828488cbe296de348112268 = new Bridge.global.Windows.UI.Xaml.Controls.ContentPresenter();
                    Bridge.global.Windows.UI.Xaml.Controls.Grid.SetRow(ContentPresenter_3cd81832a828488cbe296de348112268, 1);
                    var Binding_6cd682f4c6cb47eea7d609ed929cb13c = new Bridge.global.Windows.UI.Xaml.Data.Binding.ctor();
                    Binding_6cd682f4c6cb47eea7d609ed929cb13c.Path = new Bridge.global.Windows.UI.Xaml.PropertyPath.ctor("Content");
                    var RelativeSource_c38e71c247d34753ae738b3f9d918936 = new Bridge.global.Windows.UI.Xaml.Data.RelativeSource.ctor();
                    RelativeSource_c38e71c247d34753ae738b3f9d918936.Mode = Bridge.global.Windows.UI.Xaml.Data.RelativeSourceMode.TemplatedParent;

                    Binding_6cd682f4c6cb47eea7d609ed929cb13c.RelativeSource = RelativeSource_c38e71c247d34753ae738b3f9d918936;


                    Binding_6cd682f4c6cb47eea7d609ed929cb13c.TemplateOwner = templateInstance_63054cafb952411c96cdc5ead0daf6f4;

                    var Binding_52cc640ed2fd44f184670e73e5c6faf8 = new Bridge.global.Windows.UI.Xaml.Data.Binding.ctor();
                    Binding_52cc640ed2fd44f184670e73e5c6faf8.Path = new Bridge.global.Windows.UI.Xaml.PropertyPath.ctor("ContentTemplate");
                    var RelativeSource_97008fd1794f416981b66a589cb32363 = new Bridge.global.Windows.UI.Xaml.Data.RelativeSource.ctor();
                    RelativeSource_97008fd1794f416981b66a589cb32363.Mode = Bridge.global.Windows.UI.Xaml.Data.RelativeSourceMode.TemplatedParent;

                    Binding_52cc640ed2fd44f184670e73e5c6faf8.RelativeSource = RelativeSource_97008fd1794f416981b66a589cb32363;


                    Binding_52cc640ed2fd44f184670e73e5c6faf8.TemplateOwner = templateInstance_63054cafb952411c96cdc5ead0daf6f4;


                    Grid_e4d004867e9444c7aef1e4a06366aead.Children.add(Border_91c9b211c8824e53bf5e264424057d43);
                    Grid_e4d004867e9444c7aef1e4a06366aead.Children.add(ContentPresenter_3cd81832a828488cbe296de348112268);


                    Border_4d93900e94224740bdcc71e5956a5c88.Child = Grid_e4d004867e9444c7aef1e4a06366aead;


                    Bridge.global.Windows.UI.Xaml.Data.BindingOperations.SetBinding(ContentPresenter_b704aa759c964cc1a79e1a6aca366c72, Bridge.global.Windows.UI.Xaml.Controls.ContentControl.ContentProperty, Binding_a868fa49fa004b028c163ce903ae9d3d);
                    Bridge.global.Windows.UI.Xaml.Data.BindingOperations.SetBinding(ContentPresenter_b704aa759c964cc1a79e1a6aca366c72, Bridge.global.Windows.UI.Xaml.Controls.ContentControl.ContentTemplateProperty, Binding_21a041209f834c309cc3f0a03e8e1fa4);
                    Bridge.global.Windows.UI.Xaml.Data.BindingOperations.SetBinding(ContentPresenter_3cd81832a828488cbe296de348112268, Bridge.global.Windows.UI.Xaml.Controls.ContentControl.ContentProperty, Binding_6cd682f4c6cb47eea7d609ed929cb13c);
                    Bridge.global.Windows.UI.Xaml.Data.BindingOperations.SetBinding(ContentPresenter_3cd81832a828488cbe296de348112268, Bridge.global.Windows.UI.Xaml.Controls.ContentControl.ContentTemplateProperty, Binding_52cc640ed2fd44f184670e73e5c6faf8);

                    Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard.SetTargetProperty(ObjectAnimationUsingKeyFrames_a7e6e215c98646deb4c0bacec05015e1, new Bridge.global.Windows.UI.Xaml.PropertyPath.$ctor1("Visibility", "Visibility", Bridge.fn.cacheBind(this, this.accessVisualStateProperty_2426dc2ba75247c5a52056c350a68b3c), Bridge.fn.cacheBind(this, this.setVisualStateProperty_2426dc2ba75247c5a52056c350a68b3c), Bridge.fn.cacheBind(this, this.setAnimationVisualStateProperty_2426dc2ba75247c5a52056c350a68b3c), Bridge.fn.cacheBind(this, this.setLocalVisualStateProperty_2426dc2ba75247c5a52056c350a68b3c), Bridge.fn.cacheBind(this, this.getVisualStateProperty_2426dc2ba75247c5a52056c350a68b3c)));
                    Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard.SetTarget(ObjectAnimationUsingKeyFrames_a7e6e215c98646deb4c0bacec05015e1, Button_93248fb1f54c48298355a35f5638f613);

                    templateInstance_63054cafb952411c96cdc5ead0daf6f4.TemplateContent = Border_4d93900e94224740bdcc71e5956a5c88;
                    return templateInstance_63054cafb952411c96cdc5ead0daf6f4;
                }));

                Setter_ffcb0d13773b4ebf883fbdb19f92d785.Value = ControlTemplate_f92ce249039347c79b426a22cb836673;


                Style_2bbf2d757d60443dbaffbbe0dae6a448.Setters.add(Setter_ffcb0d13773b4ebf883fbdb19f92d785);


                this.setItem(Bridge.global.Windows.UI.Xaml.Controls.TileViewItem, Style_2bbf2d757d60443dbaffbbe0dae6a448);






            },
            accessVisualStateProperty_2426dc2ba75247c5a52056c350a68b3c: function (rootTargetObjectInstance) {
                return new (Bridge.GeneratorEnumerable$1(System.Tuple$3(Windows.UI.Xaml.DependencyObject,Windows.UI.Xaml.DependencyProperty,System.Nullable$1(System.Int32))))(Bridge.fn.bind(this, function (rootTargetObjectInstance) {
                    var $step = 0,
                        $jumpFromFinally,
                        $returnValue,
                        $async_e;

                    var $enumerator = new (Bridge.GeneratorEnumerator$1(System.Tuple$3(Windows.UI.Xaml.DependencyObject,Windows.UI.Xaml.DependencyProperty,System.Nullable$1(System.Int32))))(Bridge.fn.bind(this, function () {
                        try {
                            for (;;) {
                                switch ($step) {
                                    case 0: {
                                        return false;
                                    }
                                    default: {
                                        return false;
                                    }
                                }
                            }
                        } catch($async_e1) {
                            $async_e = System.Exception.create($async_e1);
                            throw $async_e;
                        }
                    }));
                    return $enumerator;
                }, arguments));
            },
            setVisualStateProperty_2426dc2ba75247c5a52056c350a68b3c: function (finalTargetInstance, value) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Visibility").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "VisibilityProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                (finalTargetInstance).SetVisualStateValue(property, value);
            },
            setAnimationVisualStateProperty_2426dc2ba75247c5a52056c350a68b3c: function (finalTargetInstance, value) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Visibility").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "VisibilityProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                (finalTargetInstance).SetAnimationValue(property, value);
            },
            setLocalVisualStateProperty_2426dc2ba75247c5a52056c350a68b3c: function (finalTargetInstance, value) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Visibility").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "VisibilityProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                (finalTargetInstance).SetLocalValue(property, value);
            },
            getVisualStateProperty_2426dc2ba75247c5a52056c350a68b3c: function (finalTargetInstance) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Visibility").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "VisibilityProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                return finalTargetInstance.GetVisualStateValue(property);
            }
        }
    });

    Bridge.define("___CSHTML5__Component__Defaultstyles__INTERNAL_Defaulttileviewstyle__Xaml", {
        inherits: [Bridge.global.Windows.UI.Xaml.ResourceDictionary],
        fields: {
            _contentLoaded: false
        },
        ctors: {
            ctor: function () {
                this.$initialize();
                Bridge.global.Windows.UI.Xaml.ResourceDictionary.ctor.call(this);
                this.InitializeComponent();

                if (Bridge.is(this, Windows.UI.Xaml.UIElement)) {
                    Bridge.cast(this, Windows.UI.Xaml.UIElement).XamlSourcePath = "CSHTML5\\DefaultStyles\\INTERNAL_DefaultTileViewStyle.xaml";
                }
            }
        },
        methods: {
            InitializeComponent: function () {
                if (this._contentLoaded) {
                    return;
                }
                this._contentLoaded = true;


                if (Bridge.is(this, Windows.UI.Xaml.UIElement)) {
                    Bridge.cast(this, Windows.UI.Xaml.UIElement).XamlSourcePath = "CSHTML5\\DefaultStyles\\INTERNAL_DefaultTileViewStyle.xaml";
                }



                var Style_699f320870634585b852f964555d2b2a = new Bridge.global.Windows.UI.Xaml.Style.ctor();
                Style_699f320870634585b852f964555d2b2a.TargetType = Bridge.global.Windows.UI.Xaml.Controls.TileView;
                var Setter_d539b9d32dfb4a27b26e46261aed654f = new Bridge.global.Windows.UI.Xaml.Setter.ctor();
                Setter_d539b9d32dfb4a27b26e46261aed654f.Property = Bridge.global.Windows.UI.Xaml.Controls.Control.TemplateProperty;
                var ControlTemplate_ddd5e6e64ed346e883919989588b43b4 = new Bridge.global.Windows.UI.Xaml.Controls.ControlTemplate();
                ControlTemplate_ddd5e6e64ed346e883919989588b43b4.SetMethodToInstantiateFrameworkTemplate(function (templateOwner_ControlTemplate_ddd5e6e64ed346e883919989588b43b4) {
                    var templateInstance_8af7949f046442568ff0b24f0d207357 = new Bridge.global.Windows.UI.Xaml.TemplateInstance();
                    templateInstance_8af7949f046442568ff0b24f0d207357.TemplateOwner = templateOwner_ControlTemplate_ddd5e6e64ed346e883919989588b43b4;
                    var Grid_9a4c2517cad343928ac5c7fa2c48ac7c = new Bridge.global.Windows.UI.Xaml.Controls.Grid();
                    templateOwner_ControlTemplate_ddd5e6e64ed346e883919989588b43b4.RegisterName("PART_ItemsDisplayGrid", Grid_9a4c2517cad343928ac5c7fa2c48ac7c);
                    Grid_9a4c2517cad343928ac5c7fa2c48ac7c.Name = "PART_ItemsDisplayGrid";



                    templateInstance_8af7949f046442568ff0b24f0d207357.TemplateContent = Grid_9a4c2517cad343928ac5c7fa2c48ac7c;
                    return templateInstance_8af7949f046442568ff0b24f0d207357;
                });

                Setter_d539b9d32dfb4a27b26e46261aed654f.Value = ControlTemplate_ddd5e6e64ed346e883919989588b43b4;


                Style_699f320870634585b852f964555d2b2a.Setters.add(Setter_d539b9d32dfb4a27b26e46261aed654f);


                this.setItem(Bridge.global.Windows.UI.Xaml.Controls.TileView, Style_699f320870634585b852f964555d2b2a);






            }
        }
    });

    Bridge.define("___CSHTML5__Component__Defaultstyles__INTERNAL_Defaulttimepickerstyle__Xaml", {
        inherits: [Bridge.global.Windows.UI.Xaml.ResourceDictionary],
        fields: {
            _contentLoaded: false
        },
        ctors: {
            ctor: function () {
                this.$initialize();
                Bridge.global.Windows.UI.Xaml.ResourceDictionary.ctor.call(this);
                this.InitializeComponent();

                if (Bridge.is(this, Windows.UI.Xaml.UIElement)) {
                    Bridge.cast(this, Windows.UI.Xaml.UIElement).XamlSourcePath = "CSHTML5\\DefaultStyles\\INTERNAL_DefaultTimePickerStyle.xaml";
                }
            }
        },
        methods: {
            InitializeComponent: function () {
                var $t;
                if (this._contentLoaded) {
                    return;
                }
                this._contentLoaded = true;


                if (Bridge.is(this, Windows.UI.Xaml.UIElement)) {
                    Bridge.cast(this, Windows.UI.Xaml.UIElement).XamlSourcePath = "CSHTML5\\DefaultStyles\\INTERNAL_DefaultTimePickerStyle.xaml";
                }



                var Style_c797be74946a4879951f1524b2d02ea7 = new Bridge.global.Windows.UI.Xaml.Style.ctor();
                Style_c797be74946a4879951f1524b2d02ea7.TargetType = Bridge.global.Windows.UI.Xaml.Controls.TimePicker;
                var Setter_07a73e0030af4d22ae1350b0c1758401 = new Bridge.global.Windows.UI.Xaml.Setter.ctor();
                Setter_07a73e0030af4d22ae1350b0c1758401.Property = Bridge.global.Windows.UI.Xaml.Controls.Control.BorderBrushProperty;
                Setter_07a73e0030af4d22ae1350b0c1758401.Value = new Bridge.global.Windows.UI.Xaml.Media.SolidColorBrush.$ctor1(($t = new Bridge.global.Windows.UI.Color(), $t.A = 255, $t.R = 128, $t.G = 128, $t.B = 128, $t));

                var Setter_f090d44689af4250b3848c1118c3d7e7 = new Bridge.global.Windows.UI.Xaml.Setter.ctor();
                Setter_f090d44689af4250b3848c1118c3d7e7.Property = Bridge.global.Windows.UI.Xaml.Controls.Control.BorderThicknessProperty;
                Setter_f090d44689af4250b3848c1118c3d7e7.Value = new Bridge.global.Windows.UI.Xaml.Thickness.$ctor1(1).$clone();

                var Setter_4363fb1d01174ed9a1ef3ed64fdcab7e = new Bridge.global.Windows.UI.Xaml.Setter.ctor();
                Setter_4363fb1d01174ed9a1ef3ed64fdcab7e.Property = Bridge.global.Windows.UI.Xaml.Controls.Control.TemplateProperty;
                var ControlTemplate_da8f2d16e5934ce5aa98c1a8b35aa68f = new Bridge.global.Windows.UI.Xaml.Controls.ControlTemplate();
                ControlTemplate_da8f2d16e5934ce5aa98c1a8b35aa68f.TargetType = Bridge.global.Windows.UI.Xaml.Controls.TimePicker;
                ControlTemplate_da8f2d16e5934ce5aa98c1a8b35aa68f.SetMethodToInstantiateFrameworkTemplate(function (templateOwner_ControlTemplate_da8f2d16e5934ce5aa98c1a8b35aa68f) {
                    var templateInstance_431fdfec98cd4c1bb9e08b4a189e9b7a = new Bridge.global.Windows.UI.Xaml.TemplateInstance();
                    templateInstance_431fdfec98cd4c1bb9e08b4a189e9b7a.TemplateOwner = templateOwner_ControlTemplate_da8f2d16e5934ce5aa98c1a8b35aa68f;
                    var TextBox_98cb647ff72a49cb8a30a84e47f85401 = new Bridge.global.Windows.UI.Xaml.Controls.TextBox();
                    var Button_cf908b54b57f4326a4449f1b0072f9b0 = new Bridge.global.Windows.UI.Xaml.Controls.Button();
                    var Popup_827b204e14e24aa7b4308326ce88e829 = new Bridge.global.Windows.UI.Xaml.Controls.Primitives.Popup();
                    var Grid_d4d342c6ceca46f9bea7e3969090e21c = new Bridge.global.Windows.UI.Xaml.Controls.Grid();
                    templateOwner_ControlTemplate_da8f2d16e5934ce5aa98c1a8b35aa68f.RegisterName("Root", Grid_d4d342c6ceca46f9bea7e3969090e21c);
                    Grid_d4d342c6ceca46f9bea7e3969090e21c.Name = "Root";
                    var ColumnDefinition_34ace8be9cc94ab69575942d5e0c84b2 = new Bridge.global.Windows.UI.Xaml.Controls.ColumnDefinition();
                    ColumnDefinition_34ace8be9cc94ab69575942d5e0c84b2.Width = new Bridge.global.Windows.UI.Xaml.GridLength.$ctor2(1.0, Bridge.global.Windows.UI.Xaml.GridUnitType.Star);

                    var ColumnDefinition_08973babee83474ca3fe50ba1cf336ab = new Bridge.global.Windows.UI.Xaml.Controls.ColumnDefinition();
                    ColumnDefinition_08973babee83474ca3fe50ba1cf336ab.Width = new Bridge.global.Windows.UI.Xaml.GridLength.$ctor2(1.0, Bridge.global.Windows.UI.Xaml.GridUnitType.Auto);

                    Grid_d4d342c6ceca46f9bea7e3969090e21c.ColumnDefinitions.add(ColumnDefinition_34ace8be9cc94ab69575942d5e0c84b2);
                    Grid_d4d342c6ceca46f9bea7e3969090e21c.ColumnDefinitions.add(ColumnDefinition_08973babee83474ca3fe50ba1cf336ab);

                    templateOwner_ControlTemplate_da8f2d16e5934ce5aa98c1a8b35aa68f.RegisterName("TextBox", TextBox_98cb647ff72a49cb8a30a84e47f85401);
                    TextBox_98cb647ff72a49cb8a30a84e47f85401.Name = "TextBox";
                    Bridge.global.Windows.UI.Xaml.Controls.Grid.SetColumn(TextBox_98cb647ff72a49cb8a30a84e47f85401, 0);
                    var Binding_745f6a9998ec448f93c86d7c1f2b3371 = new Bridge.global.Windows.UI.Xaml.Data.Binding.ctor();
                    Binding_745f6a9998ec448f93c86d7c1f2b3371.Path = new Bridge.global.Windows.UI.Xaml.PropertyPath.ctor("Background");
                    var RelativeSource_47448e946ade41faa1e0da11c3ac0078 = new Bridge.global.Windows.UI.Xaml.Data.RelativeSource.ctor();
                    RelativeSource_47448e946ade41faa1e0da11c3ac0078.Mode = Bridge.global.Windows.UI.Xaml.Data.RelativeSourceMode.TemplatedParent;

                    Binding_745f6a9998ec448f93c86d7c1f2b3371.RelativeSource = RelativeSource_47448e946ade41faa1e0da11c3ac0078;


                    Binding_745f6a9998ec448f93c86d7c1f2b3371.TemplateOwner = templateInstance_431fdfec98cd4c1bb9e08b4a189e9b7a;

                    var Binding_468249ea34124a1c925f4aef70fa5d6b = new Bridge.global.Windows.UI.Xaml.Data.Binding.ctor();
                    Binding_468249ea34124a1c925f4aef70fa5d6b.Path = new Bridge.global.Windows.UI.Xaml.PropertyPath.ctor("BorderBrush");
                    var RelativeSource_0961c0f642af49bdb8d0beba5f2c9717 = new Bridge.global.Windows.UI.Xaml.Data.RelativeSource.ctor();
                    RelativeSource_0961c0f642af49bdb8d0beba5f2c9717.Mode = Bridge.global.Windows.UI.Xaml.Data.RelativeSourceMode.TemplatedParent;

                    Binding_468249ea34124a1c925f4aef70fa5d6b.RelativeSource = RelativeSource_0961c0f642af49bdb8d0beba5f2c9717;


                    Binding_468249ea34124a1c925f4aef70fa5d6b.TemplateOwner = templateInstance_431fdfec98cd4c1bb9e08b4a189e9b7a;

                    var Binding_81a8309bdaaf49eb9773a1c068cf79eb = new Bridge.global.Windows.UI.Xaml.Data.Binding.ctor();
                    Binding_81a8309bdaaf49eb9773a1c068cf79eb.Path = new Bridge.global.Windows.UI.Xaml.PropertyPath.ctor("BorderThickness");
                    var RelativeSource_298af24a54314b21b435a4ac944b085c = new Bridge.global.Windows.UI.Xaml.Data.RelativeSource.ctor();
                    RelativeSource_298af24a54314b21b435a4ac944b085c.Mode = Bridge.global.Windows.UI.Xaml.Data.RelativeSourceMode.TemplatedParent;

                    Binding_81a8309bdaaf49eb9773a1c068cf79eb.RelativeSource = RelativeSource_298af24a54314b21b435a4ac944b085c;


                    Binding_81a8309bdaaf49eb9773a1c068cf79eb.TemplateOwner = templateInstance_431fdfec98cd4c1bb9e08b4a189e9b7a;

                    var Binding_f2ebd7ad71e549cb833e6af18a7e7547 = new Bridge.global.Windows.UI.Xaml.Data.Binding.ctor();
                    Binding_f2ebd7ad71e549cb833e6af18a7e7547.Path = new Bridge.global.Windows.UI.Xaml.PropertyPath.ctor("Padding");
                    var RelativeSource_9ff0391a9a4a493dbb8a032fa50aa6c3 = new Bridge.global.Windows.UI.Xaml.Data.RelativeSource.ctor();
                    RelativeSource_9ff0391a9a4a493dbb8a032fa50aa6c3.Mode = Bridge.global.Windows.UI.Xaml.Data.RelativeSourceMode.TemplatedParent;

                    Binding_f2ebd7ad71e549cb833e6af18a7e7547.RelativeSource = RelativeSource_9ff0391a9a4a493dbb8a032fa50aa6c3;


                    Binding_f2ebd7ad71e549cb833e6af18a7e7547.TemplateOwner = templateInstance_431fdfec98cd4c1bb9e08b4a189e9b7a;


                    templateOwner_ControlTemplate_da8f2d16e5934ce5aa98c1a8b35aa68f.RegisterName("Button", Button_cf908b54b57f4326a4449f1b0072f9b0);
                    Button_cf908b54b57f4326a4449f1b0072f9b0.Name = "Button";
                    Bridge.global.Windows.UI.Xaml.Controls.Grid.SetColumn(Button_cf908b54b57f4326a4449f1b0072f9b0, 1);
                    Button_cf908b54b57f4326a4449f1b0072f9b0.Content = "\u25bc";
                    Button_cf908b54b57f4326a4449f1b0072f9b0.Width = 30.0;
                    Button_cf908b54b57f4326a4449f1b0072f9b0.Margin = new Bridge.global.Windows.UI.Xaml.Thickness.$ctor2(2, 0, 2, 0);
                    var Binding_401f3593af004a27bf019114d452b0f2 = new Bridge.global.Windows.UI.Xaml.Data.Binding.ctor();
                    Binding_401f3593af004a27bf019114d452b0f2.Path = new Bridge.global.Windows.UI.Xaml.PropertyPath.ctor("Foreground");
                    var RelativeSource_669961f760c34103a25718557484898b = new Bridge.global.Windows.UI.Xaml.Data.RelativeSource.ctor();
                    RelativeSource_669961f760c34103a25718557484898b.Mode = Bridge.global.Windows.UI.Xaml.Data.RelativeSourceMode.TemplatedParent;

                    Binding_401f3593af004a27bf019114d452b0f2.RelativeSource = RelativeSource_669961f760c34103a25718557484898b;


                    Binding_401f3593af004a27bf019114d452b0f2.TemplateOwner = templateInstance_431fdfec98cd4c1bb9e08b4a189e9b7a;

                    var Binding_c1b314f139904a338f8c688dfb93b6a2 = new Bridge.global.Windows.UI.Xaml.Data.Binding.ctor();
                    Binding_c1b314f139904a338f8c688dfb93b6a2.Path = new Bridge.global.Windows.UI.Xaml.PropertyPath.ctor("BorderBrush");
                    var RelativeSource_0c6f6cb4ec72498588699c839075a697 = new Bridge.global.Windows.UI.Xaml.Data.RelativeSource.ctor();
                    RelativeSource_0c6f6cb4ec72498588699c839075a697.Mode = Bridge.global.Windows.UI.Xaml.Data.RelativeSourceMode.TemplatedParent;

                    Binding_c1b314f139904a338f8c688dfb93b6a2.RelativeSource = RelativeSource_0c6f6cb4ec72498588699c839075a697;


                    Binding_c1b314f139904a338f8c688dfb93b6a2.TemplateOwner = templateInstance_431fdfec98cd4c1bb9e08b4a189e9b7a;

                    var Binding_1f39f89c56cc4790a46aa2c0a06032b7 = new Bridge.global.Windows.UI.Xaml.Data.Binding.ctor();
                    Binding_1f39f89c56cc4790a46aa2c0a06032b7.Path = new Bridge.global.Windows.UI.Xaml.PropertyPath.ctor("BorderThickness");
                    var RelativeSource_01dd0c9bf3cb41309cdc6cb92cb31477 = new Bridge.global.Windows.UI.Xaml.Data.RelativeSource.ctor();
                    RelativeSource_01dd0c9bf3cb41309cdc6cb92cb31477.Mode = Bridge.global.Windows.UI.Xaml.Data.RelativeSourceMode.TemplatedParent;

                    Binding_1f39f89c56cc4790a46aa2c0a06032b7.RelativeSource = RelativeSource_01dd0c9bf3cb41309cdc6cb92cb31477;


                    Binding_1f39f89c56cc4790a46aa2c0a06032b7.TemplateOwner = templateInstance_431fdfec98cd4c1bb9e08b4a189e9b7a;


                    templateOwner_ControlTemplate_da8f2d16e5934ce5aa98c1a8b35aa68f.RegisterName("Popup", Popup_827b204e14e24aa7b4308326ce88e829);
                    Popup_827b204e14e24aa7b4308326ce88e829.Name = "Popup";
                    Popup_827b204e14e24aa7b4308326ce88e829.VerticalAlignment = Bridge.global.Windows.UI.Xaml.VerticalAlignment.Bottom;

                    Grid_d4d342c6ceca46f9bea7e3969090e21c.Children.add(TextBox_98cb647ff72a49cb8a30a84e47f85401);
                    Grid_d4d342c6ceca46f9bea7e3969090e21c.Children.add(Button_cf908b54b57f4326a4449f1b0072f9b0);
                    Grid_d4d342c6ceca46f9bea7e3969090e21c.Children.add(Popup_827b204e14e24aa7b4308326ce88e829);


                    Bridge.global.Windows.UI.Xaml.Data.BindingOperations.SetBinding(TextBox_98cb647ff72a49cb8a30a84e47f85401, Bridge.global.Windows.UI.Xaml.Controls.Control.BackgroundProperty, Binding_745f6a9998ec448f93c86d7c1f2b3371);
                    Bridge.global.Windows.UI.Xaml.Data.BindingOperations.SetBinding(TextBox_98cb647ff72a49cb8a30a84e47f85401, Bridge.global.Windows.UI.Xaml.Controls.Control.BorderBrushProperty, Binding_468249ea34124a1c925f4aef70fa5d6b);
                    Bridge.global.Windows.UI.Xaml.Data.BindingOperations.SetBinding(TextBox_98cb647ff72a49cb8a30a84e47f85401, Bridge.global.Windows.UI.Xaml.Controls.Control.BorderThicknessProperty, Binding_81a8309bdaaf49eb9773a1c068cf79eb);
                    Bridge.global.Windows.UI.Xaml.Data.BindingOperations.SetBinding(TextBox_98cb647ff72a49cb8a30a84e47f85401, Bridge.global.Windows.UI.Xaml.Controls.Control.PaddingProperty, Binding_f2ebd7ad71e549cb833e6af18a7e7547);
                    Bridge.global.Windows.UI.Xaml.Data.BindingOperations.SetBinding(Button_cf908b54b57f4326a4449f1b0072f9b0, Bridge.global.Windows.UI.Xaml.Controls.Control.ForegroundProperty, Binding_401f3593af004a27bf019114d452b0f2);
                    Bridge.global.Windows.UI.Xaml.Data.BindingOperations.SetBinding(Button_cf908b54b57f4326a4449f1b0072f9b0, Bridge.global.Windows.UI.Xaml.Controls.Control.BorderBrushProperty, Binding_c1b314f139904a338f8c688dfb93b6a2);
                    Bridge.global.Windows.UI.Xaml.Data.BindingOperations.SetBinding(Button_cf908b54b57f4326a4449f1b0072f9b0, Bridge.global.Windows.UI.Xaml.Controls.Control.BorderThicknessProperty, Binding_1f39f89c56cc4790a46aa2c0a06032b7);

                    templateInstance_431fdfec98cd4c1bb9e08b4a189e9b7a.TemplateContent = Grid_d4d342c6ceca46f9bea7e3969090e21c;
                    return templateInstance_431fdfec98cd4c1bb9e08b4a189e9b7a;
                });

                Setter_4363fb1d01174ed9a1ef3ed64fdcab7e.Value = ControlTemplate_da8f2d16e5934ce5aa98c1a8b35aa68f;


                Style_c797be74946a4879951f1524b2d02ea7.Setters.add(Setter_07a73e0030af4d22ae1350b0c1758401);
                Style_c797be74946a4879951f1524b2d02ea7.Setters.add(Setter_f090d44689af4250b3848c1118c3d7e7);
                Style_c797be74946a4879951f1524b2d02ea7.Setters.add(Setter_4363fb1d01174ed9a1ef3ed64fdcab7e);


                this.setItem(Bridge.global.Windows.UI.Xaml.Controls.TimePicker, Style_c797be74946a4879951f1524b2d02ea7);






            }
        }
    });

    Bridge.define("___CSHTML5__Component__Defaultstyles__INTERNAL_Defaulttogglebuttonstyle__Xaml", {
        inherits: [Bridge.global.Windows.UI.Xaml.ResourceDictionary],
        fields: {
            _contentLoaded: false
        },
        ctors: {
            ctor: function () {
                this.$initialize();
                Bridge.global.Windows.UI.Xaml.ResourceDictionary.ctor.call(this);
                this.InitializeComponent();

                if (Bridge.is(this, Windows.UI.Xaml.UIElement)) {
                    Bridge.cast(this, Windows.UI.Xaml.UIElement).XamlSourcePath = "CSHTML5\\DefaultStyles\\INTERNAL_DefaultToggleButtonStyle.xaml";
                }
            }
        },
        methods: {
            InitializeComponent: function () {
                var $t;
                if (this._contentLoaded) {
                    return;
                }
                this._contentLoaded = true;


                if (Bridge.is(this, Windows.UI.Xaml.UIElement)) {
                    Bridge.cast(this, Windows.UI.Xaml.UIElement).XamlSourcePath = "CSHTML5\\DefaultStyles\\INTERNAL_DefaultToggleButtonStyle.xaml";
                }



                var Style_65fe9771e9b04e3f8983b6b90e7788b8 = new Bridge.global.Windows.UI.Xaml.Style.ctor();
                Style_65fe9771e9b04e3f8983b6b90e7788b8.TargetType = Bridge.global.Windows.UI.Xaml.Controls.Primitives.ToggleButton;
                var Setter_eaacf59bc589419a84b6bea2923f8692 = new Bridge.global.Windows.UI.Xaml.Setter.ctor();
                Setter_eaacf59bc589419a84b6bea2923f8692.Property = Bridge.global.Windows.UI.Xaml.Controls.Control.BackgroundProperty;
                Setter_eaacf59bc589419a84b6bea2923f8692.Value = new Bridge.global.Windows.UI.Xaml.Media.SolidColorBrush.$ctor1(($t = new Bridge.global.Windows.UI.Color(), $t.A = 255, $t.R = 226, $t.G = 226, $t.B = 226, $t));

                var Setter_1848fca41ff54b98b7fa87c3ffd0a411 = new Bridge.global.Windows.UI.Xaml.Setter.ctor();
                Setter_1848fca41ff54b98b7fa87c3ffd0a411.Property = Bridge.global.Windows.UI.Xaml.Controls.Control.ForegroundProperty;
                Setter_1848fca41ff54b98b7fa87c3ffd0a411.Value = new Bridge.global.Windows.UI.Xaml.Media.SolidColorBrush.$ctor1(($t = new Bridge.global.Windows.UI.Color(), $t.A = 255, $t.R = 0, $t.G = 0, $t.B = 0, $t));

                var Setter_adadeee8d19444f49cdf2882e5159268 = new Bridge.global.Windows.UI.Xaml.Setter.ctor();
                Setter_adadeee8d19444f49cdf2882e5159268.Property = Bridge.global.Windows.UI.Xaml.Controls.Control.BorderThicknessProperty;
                Setter_adadeee8d19444f49cdf2882e5159268.Value = new Bridge.global.Windows.UI.Xaml.Thickness.$ctor1(0).$clone();

                var Setter_e9c162bcaaed4bc2a6448f9f931387e6 = new Bridge.global.Windows.UI.Xaml.Setter.ctor();
                Setter_e9c162bcaaed4bc2a6448f9f931387e6.Property = Bridge.global.Windows.UI.Xaml.Controls.Control.PaddingProperty;
                Setter_e9c162bcaaed4bc2a6448f9f931387e6.Value = new Bridge.global.Windows.UI.Xaml.Thickness.$ctor2(12, 4, 12, 4).$clone();

                var Setter_ee7c80563b9e4379938dee09e38a2b68 = new Bridge.global.Windows.UI.Xaml.Setter.ctor();
                Setter_ee7c80563b9e4379938dee09e38a2b68.Property = Bridge.global.Windows.UI.Xaml.FrameworkElement.CursorProperty;
                Setter_ee7c80563b9e4379938dee09e38a2b68.Value = Bridge.global.System.Windows.Input.Cursors.Hand;

                var Setter_f11c0b2969fa4d4986e59571be84f9cd = new Bridge.global.Windows.UI.Xaml.Setter.ctor();
                Setter_f11c0b2969fa4d4986e59571be84f9cd.Property = Bridge.global.Windows.UI.Xaml.Controls.Control.HorizontalContentAlignmentProperty;
                Setter_f11c0b2969fa4d4986e59571be84f9cd.Value = Bridge.box(Bridge.global.Windows.UI.Xaml.HorizontalAlignment.Center, Windows.UI.Xaml.HorizontalAlignment, System.Enum.toStringFn(Windows.UI.Xaml.HorizontalAlignment));

                var Setter_905f305ca80d4654b4dc443e2dbb05e0 = new Bridge.global.Windows.UI.Xaml.Setter.ctor();
                Setter_905f305ca80d4654b4dc443e2dbb05e0.Property = Bridge.global.Windows.UI.Xaml.Controls.Control.VerticalContentAlignmentProperty;
                Setter_905f305ca80d4654b4dc443e2dbb05e0.Value = Bridge.box(Bridge.global.Windows.UI.Xaml.VerticalAlignment.Center, Windows.UI.Xaml.VerticalAlignment, System.Enum.toStringFn(Windows.UI.Xaml.VerticalAlignment));

                var Setter_975f3cb516bb4c23abd2408ec729cadd = new Bridge.global.Windows.UI.Xaml.Setter.ctor();
                Setter_975f3cb516bb4c23abd2408ec729cadd.Property = Bridge.global.Windows.UI.Xaml.Controls.Control.TemplateProperty;
                var ControlTemplate_05bc8a33568c44a19d2437e36e9633ef = new Bridge.global.Windows.UI.Xaml.Controls.ControlTemplate();
                ControlTemplate_05bc8a33568c44a19d2437e36e9633ef.TargetType = Bridge.global.Windows.UI.Xaml.Controls.Primitives.ToggleButton;
                ControlTemplate_05bc8a33568c44a19d2437e36e9633ef.SetMethodToInstantiateFrameworkTemplate(Bridge.fn.bind(this, function (templateOwner_ControlTemplate_05bc8a33568c44a19d2437e36e9633ef) {
                    var templateInstance_735791bc23f24113b707ed2d84c8ccfe = new Bridge.global.Windows.UI.Xaml.TemplateInstance();
                    templateInstance_735791bc23f24113b707ed2d84c8ccfe.TemplateOwner = templateOwner_ControlTemplate_05bc8a33568c44a19d2437e36e9633ef;
                    var VisualState_0e4baa06660e4a0bb1334cebfaa62c64 = new Bridge.global.Windows.UI.Xaml.VisualState();
                    var VisualState_fa2954ea0f3d44e09d9db11d17eca266 = new Bridge.global.Windows.UI.Xaml.VisualState();
                    var VisualState_e16936e418fd4426a5c24e480a13b49b = new Bridge.global.Windows.UI.Xaml.VisualState();
                    var VisualState_723e52fb5e5046fab9a99f19e5382937 = new Bridge.global.Windows.UI.Xaml.VisualState();
                    var VisualState_25793c265e704b0ca514b2d73fff9154 = new Bridge.global.Windows.UI.Xaml.VisualState();
                    var VisualState_263935d3ab2d4a18bb7d5ce619a84210 = new Bridge.global.Windows.UI.Xaml.VisualState();
                    var VisualState_523e42bdfd5a493face382a2149b47e4 = new Bridge.global.Windows.UI.Xaml.VisualState();
                    var VisualState_11ed485102f24d64ad36b98735b7872c = new Bridge.global.Windows.UI.Xaml.VisualState();
                    var VisualState_4e97d7b97ee74320b62ae88219eb1c19 = new Bridge.global.Windows.UI.Xaml.VisualState();
                    var VisualState_b43d8f99a8184edfa0133feaa793d0c1 = new Bridge.global.Windows.UI.Xaml.VisualState();
                    var VisualState_e655af89cd35463cb1285c77c416c73d = new Bridge.global.Windows.UI.Xaml.VisualState();
                    var VisualState_0f4cd0a380034d40af9a5b4613b91067 = new Bridge.global.Windows.UI.Xaml.VisualState();
                    var VisualStateGroup_e8505ebc8920481ca38eacb84c3f02d1 = new Bridge.global.Windows.UI.Xaml.VisualStateGroup();
                    var ContentPresenter_22b1eed9162d4eb5beb7cc3182a21735 = new Bridge.global.Windows.UI.Xaml.Controls.ContentPresenter();
                    var Border_00f6894029e14a8f8c23a2be0d5b6078 = new Bridge.global.Windows.UI.Xaml.Controls.Border();
                    var Border_08ce5701cc024b6693a7487d9b07b912 = new Bridge.global.Windows.UI.Xaml.Controls.Border();
                    templateOwner_ControlTemplate_05bc8a33568c44a19d2437e36e9633ef.RegisterName("OuterBorder", Border_08ce5701cc024b6693a7487d9b07b912);
                    Border_08ce5701cc024b6693a7487d9b07b912.Name = "OuterBorder";
                    templateOwner_ControlTemplate_05bc8a33568c44a19d2437e36e9633ef.RegisterName("CommonStates", VisualStateGroup_e8505ebc8920481ca38eacb84c3f02d1);
                    VisualStateGroup_e8505ebc8920481ca38eacb84c3f02d1.Name = "CommonStates";
                    templateOwner_ControlTemplate_05bc8a33568c44a19d2437e36e9633ef.RegisterName("Normal", VisualState_0e4baa06660e4a0bb1334cebfaa62c64);
                    VisualState_0e4baa06660e4a0bb1334cebfaa62c64.Name = "Normal";

                    templateOwner_ControlTemplate_05bc8a33568c44a19d2437e36e9633ef.RegisterName("PointerOver", VisualState_fa2954ea0f3d44e09d9db11d17eca266);
                    VisualState_fa2954ea0f3d44e09d9db11d17eca266.Name = "PointerOver";
                    var Storyboard_5e1024463fc6472a8ee286a95c25015c = new Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard();
                    var ObjectAnimationUsingKeyFrames_d06ee0a253864a939c9e3b9f173d91c2 = new Bridge.global.Windows.UI.Xaml.Media.Animation.ObjectAnimationUsingKeyFrames();
                    Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard.SetTargetName(ObjectAnimationUsingKeyFrames_d06ee0a253864a939c9e3b9f173d91c2, "InnerBorder");
                    var DiscreteObjectKeyFrame_4a697fd1fa0f4094ad46627ef092e0b9 = new Bridge.global.Windows.UI.Xaml.Media.Animation.DiscreteObjectKeyFrame();
                    DiscreteObjectKeyFrame_4a697fd1fa0f4094ad46627ef092e0b9.KeyTime = Bridge.global.Windows.UI.Xaml.Media.Animation.KeyTime.FromTimeSpan(new Bridge.global.System.TimeSpan(System.Int64(0)));
                    DiscreteObjectKeyFrame_4a697fd1fa0f4094ad46627ef092e0b9.Value = "#11000000";

                    ObjectAnimationUsingKeyFrames_d06ee0a253864a939c9e3b9f173d91c2.KeyFrames.add(DiscreteObjectKeyFrame_4a697fd1fa0f4094ad46627ef092e0b9);


                    Storyboard_5e1024463fc6472a8ee286a95c25015c.Children.add(ObjectAnimationUsingKeyFrames_d06ee0a253864a939c9e3b9f173d91c2);


                    VisualState_fa2954ea0f3d44e09d9db11d17eca266.Storyboard = Storyboard_5e1024463fc6472a8ee286a95c25015c;


                    templateOwner_ControlTemplate_05bc8a33568c44a19d2437e36e9633ef.RegisterName("Pressed", VisualState_e16936e418fd4426a5c24e480a13b49b);
                    VisualState_e16936e418fd4426a5c24e480a13b49b.Name = "Pressed";
                    var Storyboard_99bc60bad62f4d83a91d9803d0691999 = new Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard();
                    var ObjectAnimationUsingKeyFrames_428bde39f185476597a402502b22eb7e = new Bridge.global.Windows.UI.Xaml.Media.Animation.ObjectAnimationUsingKeyFrames();
                    Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard.SetTargetName(ObjectAnimationUsingKeyFrames_428bde39f185476597a402502b22eb7e, "InnerBorder");
                    var DiscreteObjectKeyFrame_5e9231506e544f468772a9bb21535be9 = new Bridge.global.Windows.UI.Xaml.Media.Animation.DiscreteObjectKeyFrame();
                    DiscreteObjectKeyFrame_5e9231506e544f468772a9bb21535be9.KeyTime = Bridge.global.Windows.UI.Xaml.Media.Animation.KeyTime.FromTimeSpan(new Bridge.global.System.TimeSpan(System.Int64(0)));
                    DiscreteObjectKeyFrame_5e9231506e544f468772a9bb21535be9.Value = "#44000000";

                    ObjectAnimationUsingKeyFrames_428bde39f185476597a402502b22eb7e.KeyFrames.add(DiscreteObjectKeyFrame_5e9231506e544f468772a9bb21535be9);


                    Storyboard_99bc60bad62f4d83a91d9803d0691999.Children.add(ObjectAnimationUsingKeyFrames_428bde39f185476597a402502b22eb7e);


                    VisualState_e16936e418fd4426a5c24e480a13b49b.Storyboard = Storyboard_99bc60bad62f4d83a91d9803d0691999;


                    templateOwner_ControlTemplate_05bc8a33568c44a19d2437e36e9633ef.RegisterName("Disabled", VisualState_723e52fb5e5046fab9a99f19e5382937);
                    VisualState_723e52fb5e5046fab9a99f19e5382937.Name = "Disabled";
                    var Storyboard_38a6843df1144acb87b6235323198849 = new Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard();
                    var ObjectAnimationUsingKeyFrames_abea7c0e35344c3181df194a2108dfb7 = new Bridge.global.Windows.UI.Xaml.Media.Animation.ObjectAnimationUsingKeyFrames();
                    Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard.SetTargetName(ObjectAnimationUsingKeyFrames_abea7c0e35344c3181df194a2108dfb7, "InnerBorder");
                    var DiscreteObjectKeyFrame_7c9818b75e494b5ba7f2ac64c2e54075 = new Bridge.global.Windows.UI.Xaml.Media.Animation.DiscreteObjectKeyFrame();
                    DiscreteObjectKeyFrame_7c9818b75e494b5ba7f2ac64c2e54075.KeyTime = Bridge.global.Windows.UI.Xaml.Media.Animation.KeyTime.FromTimeSpan(new Bridge.global.System.TimeSpan(System.Int64(0)));
                    DiscreteObjectKeyFrame_7c9818b75e494b5ba7f2ac64c2e54075.Value = "Transparent";

                    ObjectAnimationUsingKeyFrames_abea7c0e35344c3181df194a2108dfb7.KeyFrames.add(DiscreteObjectKeyFrame_7c9818b75e494b5ba7f2ac64c2e54075);


                    Storyboard_38a6843df1144acb87b6235323198849.Children.add(ObjectAnimationUsingKeyFrames_abea7c0e35344c3181df194a2108dfb7);


                    VisualState_723e52fb5e5046fab9a99f19e5382937.Storyboard = Storyboard_38a6843df1144acb87b6235323198849;


                    templateOwner_ControlTemplate_05bc8a33568c44a19d2437e36e9633ef.RegisterName("Checked", VisualState_25793c265e704b0ca514b2d73fff9154);
                    VisualState_25793c265e704b0ca514b2d73fff9154.Name = "Checked";
                    var Storyboard_d4084fdcf3d7432ca141333e6fdd0c61 = new Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard();
                    var ObjectAnimationUsingKeyFrames_7efef32eb7f44446b41b12287209db39 = new Bridge.global.Windows.UI.Xaml.Media.Animation.ObjectAnimationUsingKeyFrames();
                    Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard.SetTargetName(ObjectAnimationUsingKeyFrames_7efef32eb7f44446b41b12287209db39, "InnerBorder");
                    var DiscreteObjectKeyFrame_8f893a4256304d3baa943c1639c0d505 = new Bridge.global.Windows.UI.Xaml.Media.Animation.DiscreteObjectKeyFrame();
                    DiscreteObjectKeyFrame_8f893a4256304d3baa943c1639c0d505.KeyTime = Bridge.global.Windows.UI.Xaml.Media.Animation.KeyTime.FromTimeSpan(new Bridge.global.System.TimeSpan(System.Int64(0)));
                    DiscreteObjectKeyFrame_8f893a4256304d3baa943c1639c0d505.Value = "#33000000";

                    ObjectAnimationUsingKeyFrames_7efef32eb7f44446b41b12287209db39.KeyFrames.add(DiscreteObjectKeyFrame_8f893a4256304d3baa943c1639c0d505);


                    Storyboard_d4084fdcf3d7432ca141333e6fdd0c61.Children.add(ObjectAnimationUsingKeyFrames_7efef32eb7f44446b41b12287209db39);


                    VisualState_25793c265e704b0ca514b2d73fff9154.Storyboard = Storyboard_d4084fdcf3d7432ca141333e6fdd0c61;


                    templateOwner_ControlTemplate_05bc8a33568c44a19d2437e36e9633ef.RegisterName("CheckedPointerOver", VisualState_263935d3ab2d4a18bb7d5ce619a84210);
                    VisualState_263935d3ab2d4a18bb7d5ce619a84210.Name = "CheckedPointerOver";
                    var Storyboard_026ff769ce194a798b7aee9baf9a6ace = new Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard();
                    var ObjectAnimationUsingKeyFrames_328a05cdda1a4b50bedf8a76bce3b94f = new Bridge.global.Windows.UI.Xaml.Media.Animation.ObjectAnimationUsingKeyFrames();
                    Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard.SetTargetName(ObjectAnimationUsingKeyFrames_328a05cdda1a4b50bedf8a76bce3b94f, "InnerBorder");
                    var DiscreteObjectKeyFrame_2f98d879c8cf44e4bfc97a3d9d7edfbf = new Bridge.global.Windows.UI.Xaml.Media.Animation.DiscreteObjectKeyFrame();
                    DiscreteObjectKeyFrame_2f98d879c8cf44e4bfc97a3d9d7edfbf.KeyTime = Bridge.global.Windows.UI.Xaml.Media.Animation.KeyTime.FromTimeSpan(new Bridge.global.System.TimeSpan(System.Int64(0)));
                    DiscreteObjectKeyFrame_2f98d879c8cf44e4bfc97a3d9d7edfbf.Value = "#22000000";

                    ObjectAnimationUsingKeyFrames_328a05cdda1a4b50bedf8a76bce3b94f.KeyFrames.add(DiscreteObjectKeyFrame_2f98d879c8cf44e4bfc97a3d9d7edfbf);


                    Storyboard_026ff769ce194a798b7aee9baf9a6ace.Children.add(ObjectAnimationUsingKeyFrames_328a05cdda1a4b50bedf8a76bce3b94f);


                    VisualState_263935d3ab2d4a18bb7d5ce619a84210.Storyboard = Storyboard_026ff769ce194a798b7aee9baf9a6ace;


                    templateOwner_ControlTemplate_05bc8a33568c44a19d2437e36e9633ef.RegisterName("CheckedPressed", VisualState_523e42bdfd5a493face382a2149b47e4);
                    VisualState_523e42bdfd5a493face382a2149b47e4.Name = "CheckedPressed";
                    var Storyboard_c0f2fc922bfe4c25801cf0b47f46e404 = new Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard();
                    var ObjectAnimationUsingKeyFrames_59a6f936c6be45b0b13127a8f7c3836b = new Bridge.global.Windows.UI.Xaml.Media.Animation.ObjectAnimationUsingKeyFrames();
                    Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard.SetTargetName(ObjectAnimationUsingKeyFrames_59a6f936c6be45b0b13127a8f7c3836b, "InnerBorder");
                    var DiscreteObjectKeyFrame_7754403a520d44d9b0e6bab822178bc9 = new Bridge.global.Windows.UI.Xaml.Media.Animation.DiscreteObjectKeyFrame();
                    DiscreteObjectKeyFrame_7754403a520d44d9b0e6bab822178bc9.KeyTime = Bridge.global.Windows.UI.Xaml.Media.Animation.KeyTime.FromTimeSpan(new Bridge.global.System.TimeSpan(System.Int64(0)));
                    DiscreteObjectKeyFrame_7754403a520d44d9b0e6bab822178bc9.Value = "#44000000";

                    ObjectAnimationUsingKeyFrames_59a6f936c6be45b0b13127a8f7c3836b.KeyFrames.add(DiscreteObjectKeyFrame_7754403a520d44d9b0e6bab822178bc9);


                    Storyboard_c0f2fc922bfe4c25801cf0b47f46e404.Children.add(ObjectAnimationUsingKeyFrames_59a6f936c6be45b0b13127a8f7c3836b);


                    VisualState_523e42bdfd5a493face382a2149b47e4.Storyboard = Storyboard_c0f2fc922bfe4c25801cf0b47f46e404;


                    templateOwner_ControlTemplate_05bc8a33568c44a19d2437e36e9633ef.RegisterName("CheckedDisabled", VisualState_11ed485102f24d64ad36b98735b7872c);
                    VisualState_11ed485102f24d64ad36b98735b7872c.Name = "CheckedDisabled";
                    var Storyboard_8a0a13ee8b6d40fe9fcb4ae6a7a5302f = new Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard();
                    var ObjectAnimationUsingKeyFrames_5a052db5055e4fc9bfbfa14bfc0f0dad = new Bridge.global.Windows.UI.Xaml.Media.Animation.ObjectAnimationUsingKeyFrames();
                    Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard.SetTargetName(ObjectAnimationUsingKeyFrames_5a052db5055e4fc9bfbfa14bfc0f0dad, "InnerBorder");
                    var DiscreteObjectKeyFrame_c24dfb2abb8f4888830aebf38976aa32 = new Bridge.global.Windows.UI.Xaml.Media.Animation.DiscreteObjectKeyFrame();
                    DiscreteObjectKeyFrame_c24dfb2abb8f4888830aebf38976aa32.KeyTime = Bridge.global.Windows.UI.Xaml.Media.Animation.KeyTime.FromTimeSpan(new Bridge.global.System.TimeSpan(System.Int64(0)));
                    DiscreteObjectKeyFrame_c24dfb2abb8f4888830aebf38976aa32.Value = "#33000000";

                    ObjectAnimationUsingKeyFrames_5a052db5055e4fc9bfbfa14bfc0f0dad.KeyFrames.add(DiscreteObjectKeyFrame_c24dfb2abb8f4888830aebf38976aa32);


                    Storyboard_8a0a13ee8b6d40fe9fcb4ae6a7a5302f.Children.add(ObjectAnimationUsingKeyFrames_5a052db5055e4fc9bfbfa14bfc0f0dad);


                    VisualState_11ed485102f24d64ad36b98735b7872c.Storyboard = Storyboard_8a0a13ee8b6d40fe9fcb4ae6a7a5302f;


                    templateOwner_ControlTemplate_05bc8a33568c44a19d2437e36e9633ef.RegisterName("Indeterminate", VisualState_4e97d7b97ee74320b62ae88219eb1c19);
                    VisualState_4e97d7b97ee74320b62ae88219eb1c19.Name = "Indeterminate";

                    templateOwner_ControlTemplate_05bc8a33568c44a19d2437e36e9633ef.RegisterName("IndeterminatePointerOver", VisualState_b43d8f99a8184edfa0133feaa793d0c1);
                    VisualState_b43d8f99a8184edfa0133feaa793d0c1.Name = "IndeterminatePointerOver";
                    var Storyboard_91cdbcc2889c4a38bea879f3c717f95d = new Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard();
                    var ObjectAnimationUsingKeyFrames_447f6428d19447ab8ce1ecd1d52ae9ae = new Bridge.global.Windows.UI.Xaml.Media.Animation.ObjectAnimationUsingKeyFrames();
                    Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard.SetTargetName(ObjectAnimationUsingKeyFrames_447f6428d19447ab8ce1ecd1d52ae9ae, "InnerBorder");
                    var DiscreteObjectKeyFrame_9bbdb27a00cd486c92128c281b5b3a26 = new Bridge.global.Windows.UI.Xaml.Media.Animation.DiscreteObjectKeyFrame();
                    DiscreteObjectKeyFrame_9bbdb27a00cd486c92128c281b5b3a26.KeyTime = Bridge.global.Windows.UI.Xaml.Media.Animation.KeyTime.FromTimeSpan(new Bridge.global.System.TimeSpan(System.Int64(0)));
                    DiscreteObjectKeyFrame_9bbdb27a00cd486c92128c281b5b3a26.Value = "#11000000";

                    ObjectAnimationUsingKeyFrames_447f6428d19447ab8ce1ecd1d52ae9ae.KeyFrames.add(DiscreteObjectKeyFrame_9bbdb27a00cd486c92128c281b5b3a26);


                    Storyboard_91cdbcc2889c4a38bea879f3c717f95d.Children.add(ObjectAnimationUsingKeyFrames_447f6428d19447ab8ce1ecd1d52ae9ae);


                    VisualState_b43d8f99a8184edfa0133feaa793d0c1.Storyboard = Storyboard_91cdbcc2889c4a38bea879f3c717f95d;


                    templateOwner_ControlTemplate_05bc8a33568c44a19d2437e36e9633ef.RegisterName("IndeterminatePressed", VisualState_e655af89cd35463cb1285c77c416c73d);
                    VisualState_e655af89cd35463cb1285c77c416c73d.Name = "IndeterminatePressed";
                    var Storyboard_61b34783583e4913999e21a4bb4157a4 = new Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard();
                    var ObjectAnimationUsingKeyFrames_1121f1192e6e4481b40be94667f812da = new Bridge.global.Windows.UI.Xaml.Media.Animation.ObjectAnimationUsingKeyFrames();
                    Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard.SetTargetName(ObjectAnimationUsingKeyFrames_1121f1192e6e4481b40be94667f812da, "InnerBorder");
                    var DiscreteObjectKeyFrame_14535ddd646547ec851de9b7c6f9cfba = new Bridge.global.Windows.UI.Xaml.Media.Animation.DiscreteObjectKeyFrame();
                    DiscreteObjectKeyFrame_14535ddd646547ec851de9b7c6f9cfba.KeyTime = Bridge.global.Windows.UI.Xaml.Media.Animation.KeyTime.FromTimeSpan(new Bridge.global.System.TimeSpan(System.Int64(0)));
                    DiscreteObjectKeyFrame_14535ddd646547ec851de9b7c6f9cfba.Value = "#22000000";

                    ObjectAnimationUsingKeyFrames_1121f1192e6e4481b40be94667f812da.KeyFrames.add(DiscreteObjectKeyFrame_14535ddd646547ec851de9b7c6f9cfba);


                    Storyboard_61b34783583e4913999e21a4bb4157a4.Children.add(ObjectAnimationUsingKeyFrames_1121f1192e6e4481b40be94667f812da);


                    VisualState_e655af89cd35463cb1285c77c416c73d.Storyboard = Storyboard_61b34783583e4913999e21a4bb4157a4;


                    templateOwner_ControlTemplate_05bc8a33568c44a19d2437e36e9633ef.RegisterName("IndeterminateDisabled", VisualState_0f4cd0a380034d40af9a5b4613b91067);
                    VisualState_0f4cd0a380034d40af9a5b4613b91067.Name = "IndeterminateDisabled";
                    var Storyboard_b84fae2de6144151a7031ccf205ec4fd = new Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard();
                    var ObjectAnimationUsingKeyFrames_fb778b8ba66545b1bf08313241ae18a5 = new Bridge.global.Windows.UI.Xaml.Media.Animation.ObjectAnimationUsingKeyFrames();
                    Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard.SetTargetName(ObjectAnimationUsingKeyFrames_fb778b8ba66545b1bf08313241ae18a5, "InnerBorder");
                    var DiscreteObjectKeyFrame_d1f387331fdf47669381451046123423 = new Bridge.global.Windows.UI.Xaml.Media.Animation.DiscreteObjectKeyFrame();
                    DiscreteObjectKeyFrame_d1f387331fdf47669381451046123423.KeyTime = Bridge.global.Windows.UI.Xaml.Media.Animation.KeyTime.FromTimeSpan(new Bridge.global.System.TimeSpan(System.Int64(0)));
                    DiscreteObjectKeyFrame_d1f387331fdf47669381451046123423.Value = "Transparent";

                    ObjectAnimationUsingKeyFrames_fb778b8ba66545b1bf08313241ae18a5.KeyFrames.add(DiscreteObjectKeyFrame_d1f387331fdf47669381451046123423);


                    Storyboard_b84fae2de6144151a7031ccf205ec4fd.Children.add(ObjectAnimationUsingKeyFrames_fb778b8ba66545b1bf08313241ae18a5);


                    VisualState_0f4cd0a380034d40af9a5b4613b91067.Storyboard = Storyboard_b84fae2de6144151a7031ccf205ec4fd;


                    System.Array.add(VisualStateGroup_e8505ebc8920481ca38eacb84c3f02d1.States, VisualState_0e4baa06660e4a0bb1334cebfaa62c64, Object);
                    System.Array.add(VisualStateGroup_e8505ebc8920481ca38eacb84c3f02d1.States, VisualState_fa2954ea0f3d44e09d9db11d17eca266, Object);
                    System.Array.add(VisualStateGroup_e8505ebc8920481ca38eacb84c3f02d1.States, VisualState_e16936e418fd4426a5c24e480a13b49b, Object);
                    System.Array.add(VisualStateGroup_e8505ebc8920481ca38eacb84c3f02d1.States, VisualState_723e52fb5e5046fab9a99f19e5382937, Object);
                    System.Array.add(VisualStateGroup_e8505ebc8920481ca38eacb84c3f02d1.States, VisualState_25793c265e704b0ca514b2d73fff9154, Object);
                    System.Array.add(VisualStateGroup_e8505ebc8920481ca38eacb84c3f02d1.States, VisualState_263935d3ab2d4a18bb7d5ce619a84210, Object);
                    System.Array.add(VisualStateGroup_e8505ebc8920481ca38eacb84c3f02d1.States, VisualState_523e42bdfd5a493face382a2149b47e4, Object);
                    System.Array.add(VisualStateGroup_e8505ebc8920481ca38eacb84c3f02d1.States, VisualState_11ed485102f24d64ad36b98735b7872c, Object);
                    System.Array.add(VisualStateGroup_e8505ebc8920481ca38eacb84c3f02d1.States, VisualState_4e97d7b97ee74320b62ae88219eb1c19, Object);
                    System.Array.add(VisualStateGroup_e8505ebc8920481ca38eacb84c3f02d1.States, VisualState_b43d8f99a8184edfa0133feaa793d0c1, Object);
                    System.Array.add(VisualStateGroup_e8505ebc8920481ca38eacb84c3f02d1.States, VisualState_e655af89cd35463cb1285c77c416c73d, Object);
                    System.Array.add(VisualStateGroup_e8505ebc8920481ca38eacb84c3f02d1.States, VisualState_0f4cd0a380034d40af9a5b4613b91067, Object);


                    templateOwner_ControlTemplate_05bc8a33568c44a19d2437e36e9633ef.INTERNAL_GetVisualStateGroups().add(VisualStateGroup_e8505ebc8920481ca38eacb84c3f02d1);

                    templateOwner_ControlTemplate_05bc8a33568c44a19d2437e36e9633ef.RegisterName("InnerBorder", Border_00f6894029e14a8f8c23a2be0d5b6078);
                    Border_00f6894029e14a8f8c23a2be0d5b6078.Name = "InnerBorder";
                    templateOwner_ControlTemplate_05bc8a33568c44a19d2437e36e9633ef.RegisterName("ContentPresenter", ContentPresenter_22b1eed9162d4eb5beb7cc3182a21735);
                    ContentPresenter_22b1eed9162d4eb5beb7cc3182a21735.Name = "ContentPresenter";
                    var Binding_b3af216428c04d8ea8095fce04177597 = new Bridge.global.Windows.UI.Xaml.Data.Binding.ctor();
                    Binding_b3af216428c04d8ea8095fce04177597.Path = new Bridge.global.Windows.UI.Xaml.PropertyPath.ctor("ContentTemplate");
                    var RelativeSource_7b440705c995489185038e7910326fd0 = new Bridge.global.Windows.UI.Xaml.Data.RelativeSource.ctor();
                    RelativeSource_7b440705c995489185038e7910326fd0.Mode = Bridge.global.Windows.UI.Xaml.Data.RelativeSourceMode.TemplatedParent;

                    Binding_b3af216428c04d8ea8095fce04177597.RelativeSource = RelativeSource_7b440705c995489185038e7910326fd0;


                    Binding_b3af216428c04d8ea8095fce04177597.TemplateOwner = templateInstance_735791bc23f24113b707ed2d84c8ccfe;

                    var Binding_5edda34a713946b78c2da802c7ea4300 = new Bridge.global.Windows.UI.Xaml.Data.Binding.ctor();
                    Binding_5edda34a713946b78c2da802c7ea4300.Path = new Bridge.global.Windows.UI.Xaml.PropertyPath.ctor("Content");
                    var RelativeSource_7a347d3b46fe4fefb387cab017726bf3 = new Bridge.global.Windows.UI.Xaml.Data.RelativeSource.ctor();
                    RelativeSource_7a347d3b46fe4fefb387cab017726bf3.Mode = Bridge.global.Windows.UI.Xaml.Data.RelativeSourceMode.TemplatedParent;

                    Binding_5edda34a713946b78c2da802c7ea4300.RelativeSource = RelativeSource_7a347d3b46fe4fefb387cab017726bf3;


                    Binding_5edda34a713946b78c2da802c7ea4300.TemplateOwner = templateInstance_735791bc23f24113b707ed2d84c8ccfe;

                    var Binding_5084646a4dc843abbcf90816748094aa = new Bridge.global.Windows.UI.Xaml.Data.Binding.ctor();
                    Binding_5084646a4dc843abbcf90816748094aa.Path = new Bridge.global.Windows.UI.Xaml.PropertyPath.ctor("Padding");
                    var RelativeSource_c2a7a5a65b174c0eb32fdd06ca08ab4e = new Bridge.global.Windows.UI.Xaml.Data.RelativeSource.ctor();
                    RelativeSource_c2a7a5a65b174c0eb32fdd06ca08ab4e.Mode = Bridge.global.Windows.UI.Xaml.Data.RelativeSourceMode.TemplatedParent;

                    Binding_5084646a4dc843abbcf90816748094aa.RelativeSource = RelativeSource_c2a7a5a65b174c0eb32fdd06ca08ab4e;


                    Binding_5084646a4dc843abbcf90816748094aa.TemplateOwner = templateInstance_735791bc23f24113b707ed2d84c8ccfe;

                    var Binding_553b1e8383764b738b47ba180e0de129 = new Bridge.global.Windows.UI.Xaml.Data.Binding.ctor();
                    Binding_553b1e8383764b738b47ba180e0de129.Path = new Bridge.global.Windows.UI.Xaml.PropertyPath.ctor("HorizontalContentAlignment");
                    var RelativeSource_6973889ff1174d37aaad135a13e434df = new Bridge.global.Windows.UI.Xaml.Data.RelativeSource.ctor();
                    RelativeSource_6973889ff1174d37aaad135a13e434df.Mode = Bridge.global.Windows.UI.Xaml.Data.RelativeSourceMode.TemplatedParent;

                    Binding_553b1e8383764b738b47ba180e0de129.RelativeSource = RelativeSource_6973889ff1174d37aaad135a13e434df;


                    Binding_553b1e8383764b738b47ba180e0de129.TemplateOwner = templateInstance_735791bc23f24113b707ed2d84c8ccfe;

                    var Binding_273549f6910348f584b059794ef1874f = new Bridge.global.Windows.UI.Xaml.Data.Binding.ctor();
                    Binding_273549f6910348f584b059794ef1874f.Path = new Bridge.global.Windows.UI.Xaml.PropertyPath.ctor("VerticalContentAlignment");
                    var RelativeSource_99d5ebaa0fe74748bba35c5def040e9e = new Bridge.global.Windows.UI.Xaml.Data.RelativeSource.ctor();
                    RelativeSource_99d5ebaa0fe74748bba35c5def040e9e.Mode = Bridge.global.Windows.UI.Xaml.Data.RelativeSourceMode.TemplatedParent;

                    Binding_273549f6910348f584b059794ef1874f.RelativeSource = RelativeSource_99d5ebaa0fe74748bba35c5def040e9e;


                    Binding_273549f6910348f584b059794ef1874f.TemplateOwner = templateInstance_735791bc23f24113b707ed2d84c8ccfe;


                    Border_00f6894029e14a8f8c23a2be0d5b6078.Child = ContentPresenter_22b1eed9162d4eb5beb7cc3182a21735;

                    var Binding_d7037d3798e4443ba6d84fa05ab24565 = new Bridge.global.Windows.UI.Xaml.Data.Binding.ctor();
                    Binding_d7037d3798e4443ba6d84fa05ab24565.Path = new Bridge.global.Windows.UI.Xaml.PropertyPath.ctor("Background");
                    var RelativeSource_32a7c304d27c4e148ab1649f5b593a99 = new Bridge.global.Windows.UI.Xaml.Data.RelativeSource.ctor();
                    RelativeSource_32a7c304d27c4e148ab1649f5b593a99.Mode = Bridge.global.Windows.UI.Xaml.Data.RelativeSourceMode.TemplatedParent;

                    Binding_d7037d3798e4443ba6d84fa05ab24565.RelativeSource = RelativeSource_32a7c304d27c4e148ab1649f5b593a99;


                    Binding_d7037d3798e4443ba6d84fa05ab24565.TemplateOwner = templateInstance_735791bc23f24113b707ed2d84c8ccfe;


                    Border_08ce5701cc024b6693a7487d9b07b912.Child = Border_00f6894029e14a8f8c23a2be0d5b6078;

                    var Binding_02a582bdcf834c24a07ae31ca5331a62 = new Bridge.global.Windows.UI.Xaml.Data.Binding.ctor();
                    Binding_02a582bdcf834c24a07ae31ca5331a62.Path = new Bridge.global.Windows.UI.Xaml.PropertyPath.ctor("Background");
                    var RelativeSource_dda22a8d04834df1bed7b17816fd6cfd = new Bridge.global.Windows.UI.Xaml.Data.RelativeSource.ctor();
                    RelativeSource_dda22a8d04834df1bed7b17816fd6cfd.Mode = Bridge.global.Windows.UI.Xaml.Data.RelativeSourceMode.TemplatedParent;

                    Binding_02a582bdcf834c24a07ae31ca5331a62.RelativeSource = RelativeSource_dda22a8d04834df1bed7b17816fd6cfd;


                    Binding_02a582bdcf834c24a07ae31ca5331a62.TemplateOwner = templateInstance_735791bc23f24113b707ed2d84c8ccfe;

                    var Binding_223490ca52544fc89eda1dde87d607e5 = new Bridge.global.Windows.UI.Xaml.Data.Binding.ctor();
                    Binding_223490ca52544fc89eda1dde87d607e5.Path = new Bridge.global.Windows.UI.Xaml.PropertyPath.ctor("BorderBrush");
                    var RelativeSource_8e92d242498f4575adf4194d62370b65 = new Bridge.global.Windows.UI.Xaml.Data.RelativeSource.ctor();
                    RelativeSource_8e92d242498f4575adf4194d62370b65.Mode = Bridge.global.Windows.UI.Xaml.Data.RelativeSourceMode.TemplatedParent;

                    Binding_223490ca52544fc89eda1dde87d607e5.RelativeSource = RelativeSource_8e92d242498f4575adf4194d62370b65;


                    Binding_223490ca52544fc89eda1dde87d607e5.TemplateOwner = templateInstance_735791bc23f24113b707ed2d84c8ccfe;

                    var Binding_181d6f01fafb4d93b07cf68428ddc3f5 = new Bridge.global.Windows.UI.Xaml.Data.Binding.ctor();
                    Binding_181d6f01fafb4d93b07cf68428ddc3f5.Path = new Bridge.global.Windows.UI.Xaml.PropertyPath.ctor("BorderThickness");
                    var RelativeSource_bc635aa023ed4a2eb748a280c7409ee0 = new Bridge.global.Windows.UI.Xaml.Data.RelativeSource.ctor();
                    RelativeSource_bc635aa023ed4a2eb748a280c7409ee0.Mode = Bridge.global.Windows.UI.Xaml.Data.RelativeSourceMode.TemplatedParent;

                    Binding_181d6f01fafb4d93b07cf68428ddc3f5.RelativeSource = RelativeSource_bc635aa023ed4a2eb748a280c7409ee0;


                    Binding_181d6f01fafb4d93b07cf68428ddc3f5.TemplateOwner = templateInstance_735791bc23f24113b707ed2d84c8ccfe;


                    Bridge.global.Windows.UI.Xaml.Data.BindingOperations.SetBinding(ContentPresenter_22b1eed9162d4eb5beb7cc3182a21735, Bridge.global.Windows.UI.Xaml.Controls.ContentControl.ContentTemplateProperty, Binding_b3af216428c04d8ea8095fce04177597);
                    Bridge.global.Windows.UI.Xaml.Data.BindingOperations.SetBinding(ContentPresenter_22b1eed9162d4eb5beb7cc3182a21735, Bridge.global.Windows.UI.Xaml.Controls.ContentControl.ContentProperty, Binding_5edda34a713946b78c2da802c7ea4300);
                    Bridge.global.Windows.UI.Xaml.Data.BindingOperations.SetBinding(ContentPresenter_22b1eed9162d4eb5beb7cc3182a21735, Bridge.global.Windows.UI.Xaml.FrameworkElement.MarginProperty, Binding_5084646a4dc843abbcf90816748094aa);
                    Bridge.global.Windows.UI.Xaml.Data.BindingOperations.SetBinding(ContentPresenter_22b1eed9162d4eb5beb7cc3182a21735, Bridge.global.Windows.UI.Xaml.FrameworkElement.HorizontalAlignmentProperty, Binding_553b1e8383764b738b47ba180e0de129);
                    Bridge.global.Windows.UI.Xaml.Data.BindingOperations.SetBinding(ContentPresenter_22b1eed9162d4eb5beb7cc3182a21735, Bridge.global.Windows.UI.Xaml.FrameworkElement.VerticalAlignmentProperty, Binding_273549f6910348f584b059794ef1874f);
                    Bridge.global.Windows.UI.Xaml.Data.BindingOperations.SetBinding(Border_00f6894029e14a8f8c23a2be0d5b6078, Bridge.global.Windows.UI.Xaml.Controls.Border.BackgroundProperty, Binding_d7037d3798e4443ba6d84fa05ab24565);
                    Bridge.global.Windows.UI.Xaml.Data.BindingOperations.SetBinding(Border_08ce5701cc024b6693a7487d9b07b912, Bridge.global.Windows.UI.Xaml.Controls.Border.BackgroundProperty, Binding_02a582bdcf834c24a07ae31ca5331a62);
                    Bridge.global.Windows.UI.Xaml.Data.BindingOperations.SetBinding(Border_08ce5701cc024b6693a7487d9b07b912, Bridge.global.Windows.UI.Xaml.Controls.Border.BorderBrushProperty, Binding_223490ca52544fc89eda1dde87d607e5);
                    Bridge.global.Windows.UI.Xaml.Data.BindingOperations.SetBinding(Border_08ce5701cc024b6693a7487d9b07b912, Bridge.global.Windows.UI.Xaml.Controls.Border.BorderThicknessProperty, Binding_181d6f01fafb4d93b07cf68428ddc3f5);

                    Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard.SetTargetProperty(ObjectAnimationUsingKeyFrames_d06ee0a253864a939c9e3b9f173d91c2, new Bridge.global.Windows.UI.Xaml.PropertyPath.$ctor1("Background", "Background", Bridge.fn.cacheBind(this, this.accessVisualStateProperty_470b309e8eac48f199e4c9a667787437), Bridge.fn.cacheBind(this, this.setVisualStateProperty_470b309e8eac48f199e4c9a667787437), Bridge.fn.cacheBind(this, this.setAnimationVisualStateProperty_470b309e8eac48f199e4c9a667787437), Bridge.fn.cacheBind(this, this.setLocalVisualStateProperty_470b309e8eac48f199e4c9a667787437), Bridge.fn.cacheBind(this, this.getVisualStateProperty_470b309e8eac48f199e4c9a667787437)));
                    Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard.SetTarget(ObjectAnimationUsingKeyFrames_d06ee0a253864a939c9e3b9f173d91c2, Border_00f6894029e14a8f8c23a2be0d5b6078);


                    Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard.SetTargetProperty(ObjectAnimationUsingKeyFrames_428bde39f185476597a402502b22eb7e, new Bridge.global.Windows.UI.Xaml.PropertyPath.$ctor1("Background", "Background", Bridge.fn.cacheBind(this, this.accessVisualStateProperty_b02e07de44a94f42bb0da21689e6877e), Bridge.fn.cacheBind(this, this.setVisualStateProperty_b02e07de44a94f42bb0da21689e6877e), Bridge.fn.cacheBind(this, this.setAnimationVisualStateProperty_b02e07de44a94f42bb0da21689e6877e), Bridge.fn.cacheBind(this, this.setLocalVisualStateProperty_b02e07de44a94f42bb0da21689e6877e), Bridge.fn.cacheBind(this, this.getVisualStateProperty_b02e07de44a94f42bb0da21689e6877e)));
                    Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard.SetTarget(ObjectAnimationUsingKeyFrames_428bde39f185476597a402502b22eb7e, Border_00f6894029e14a8f8c23a2be0d5b6078);


                    Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard.SetTargetProperty(ObjectAnimationUsingKeyFrames_abea7c0e35344c3181df194a2108dfb7, new Bridge.global.Windows.UI.Xaml.PropertyPath.$ctor1("Background", "Background", Bridge.fn.cacheBind(this, this.accessVisualStateProperty_dbb0e12eeff1468f92a2e33544198d8a), Bridge.fn.cacheBind(this, this.setVisualStateProperty_dbb0e12eeff1468f92a2e33544198d8a), Bridge.fn.cacheBind(this, this.setAnimationVisualStateProperty_dbb0e12eeff1468f92a2e33544198d8a), Bridge.fn.cacheBind(this, this.setLocalVisualStateProperty_dbb0e12eeff1468f92a2e33544198d8a), Bridge.fn.cacheBind(this, this.getVisualStateProperty_dbb0e12eeff1468f92a2e33544198d8a)));
                    Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard.SetTarget(ObjectAnimationUsingKeyFrames_abea7c0e35344c3181df194a2108dfb7, Border_00f6894029e14a8f8c23a2be0d5b6078);


                    Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard.SetTargetProperty(ObjectAnimationUsingKeyFrames_7efef32eb7f44446b41b12287209db39, new Bridge.global.Windows.UI.Xaml.PropertyPath.$ctor1("Background", "Background", Bridge.fn.cacheBind(this, this.accessVisualStateProperty_fc7934b10b5d48e8bfba30c2970d1211), Bridge.fn.cacheBind(this, this.setVisualStateProperty_fc7934b10b5d48e8bfba30c2970d1211), Bridge.fn.cacheBind(this, this.setAnimationVisualStateProperty_fc7934b10b5d48e8bfba30c2970d1211), Bridge.fn.cacheBind(this, this.setLocalVisualStateProperty_fc7934b10b5d48e8bfba30c2970d1211), Bridge.fn.cacheBind(this, this.getVisualStateProperty_fc7934b10b5d48e8bfba30c2970d1211)));
                    Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard.SetTarget(ObjectAnimationUsingKeyFrames_7efef32eb7f44446b41b12287209db39, Border_00f6894029e14a8f8c23a2be0d5b6078);


                    Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard.SetTargetProperty(ObjectAnimationUsingKeyFrames_328a05cdda1a4b50bedf8a76bce3b94f, new Bridge.global.Windows.UI.Xaml.PropertyPath.$ctor1("Background", "Background", Bridge.fn.cacheBind(this, this.accessVisualStateProperty_b8a143d4613e4a75b8d7c08bfe836ecc), Bridge.fn.cacheBind(this, this.setVisualStateProperty_b8a143d4613e4a75b8d7c08bfe836ecc), Bridge.fn.cacheBind(this, this.setAnimationVisualStateProperty_b8a143d4613e4a75b8d7c08bfe836ecc), Bridge.fn.cacheBind(this, this.setLocalVisualStateProperty_b8a143d4613e4a75b8d7c08bfe836ecc), Bridge.fn.cacheBind(this, this.getVisualStateProperty_b8a143d4613e4a75b8d7c08bfe836ecc)));
                    Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard.SetTarget(ObjectAnimationUsingKeyFrames_328a05cdda1a4b50bedf8a76bce3b94f, Border_00f6894029e14a8f8c23a2be0d5b6078);


                    Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard.SetTargetProperty(ObjectAnimationUsingKeyFrames_59a6f936c6be45b0b13127a8f7c3836b, new Bridge.global.Windows.UI.Xaml.PropertyPath.$ctor1("Background", "Background", Bridge.fn.cacheBind(this, this.accessVisualStateProperty_d930e5a5734b4e4a9aee4bbaea2998b6), Bridge.fn.cacheBind(this, this.setVisualStateProperty_d930e5a5734b4e4a9aee4bbaea2998b6), Bridge.fn.cacheBind(this, this.setAnimationVisualStateProperty_d930e5a5734b4e4a9aee4bbaea2998b6), Bridge.fn.cacheBind(this, this.setLocalVisualStateProperty_d930e5a5734b4e4a9aee4bbaea2998b6), Bridge.fn.cacheBind(this, this.getVisualStateProperty_d930e5a5734b4e4a9aee4bbaea2998b6)));
                    Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard.SetTarget(ObjectAnimationUsingKeyFrames_59a6f936c6be45b0b13127a8f7c3836b, Border_00f6894029e14a8f8c23a2be0d5b6078);


                    Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard.SetTargetProperty(ObjectAnimationUsingKeyFrames_5a052db5055e4fc9bfbfa14bfc0f0dad, new Bridge.global.Windows.UI.Xaml.PropertyPath.$ctor1("Background", "Background", Bridge.fn.cacheBind(this, this.accessVisualStateProperty_1cd2be2b60494cb2ab753dbda1e770dd), Bridge.fn.cacheBind(this, this.setVisualStateProperty_1cd2be2b60494cb2ab753dbda1e770dd), Bridge.fn.cacheBind(this, this.setAnimationVisualStateProperty_1cd2be2b60494cb2ab753dbda1e770dd), Bridge.fn.cacheBind(this, this.setLocalVisualStateProperty_1cd2be2b60494cb2ab753dbda1e770dd), Bridge.fn.cacheBind(this, this.getVisualStateProperty_1cd2be2b60494cb2ab753dbda1e770dd)));
                    Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard.SetTarget(ObjectAnimationUsingKeyFrames_5a052db5055e4fc9bfbfa14bfc0f0dad, Border_00f6894029e14a8f8c23a2be0d5b6078);


                    Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard.SetTargetProperty(ObjectAnimationUsingKeyFrames_447f6428d19447ab8ce1ecd1d52ae9ae, new Bridge.global.Windows.UI.Xaml.PropertyPath.$ctor1("Background", "Background", Bridge.fn.cacheBind(this, this.accessVisualStateProperty_932b42a36ef2485e8f5ba05fdc97314c), Bridge.fn.cacheBind(this, this.setVisualStateProperty_932b42a36ef2485e8f5ba05fdc97314c), Bridge.fn.cacheBind(this, this.setAnimationVisualStateProperty_932b42a36ef2485e8f5ba05fdc97314c), Bridge.fn.cacheBind(this, this.setLocalVisualStateProperty_932b42a36ef2485e8f5ba05fdc97314c), Bridge.fn.cacheBind(this, this.getVisualStateProperty_932b42a36ef2485e8f5ba05fdc97314c)));
                    Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard.SetTarget(ObjectAnimationUsingKeyFrames_447f6428d19447ab8ce1ecd1d52ae9ae, Border_00f6894029e14a8f8c23a2be0d5b6078);


                    Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard.SetTargetProperty(ObjectAnimationUsingKeyFrames_1121f1192e6e4481b40be94667f812da, new Bridge.global.Windows.UI.Xaml.PropertyPath.$ctor1("Background", "Background", Bridge.fn.cacheBind(this, this.accessVisualStateProperty_bb5c51247a7441ff82d13b07a6eaa990), Bridge.fn.cacheBind(this, this.setVisualStateProperty_bb5c51247a7441ff82d13b07a6eaa990), Bridge.fn.cacheBind(this, this.setAnimationVisualStateProperty_bb5c51247a7441ff82d13b07a6eaa990), Bridge.fn.cacheBind(this, this.setLocalVisualStateProperty_bb5c51247a7441ff82d13b07a6eaa990), Bridge.fn.cacheBind(this, this.getVisualStateProperty_bb5c51247a7441ff82d13b07a6eaa990)));
                    Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard.SetTarget(ObjectAnimationUsingKeyFrames_1121f1192e6e4481b40be94667f812da, Border_00f6894029e14a8f8c23a2be0d5b6078);


                    Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard.SetTargetProperty(ObjectAnimationUsingKeyFrames_fb778b8ba66545b1bf08313241ae18a5, new Bridge.global.Windows.UI.Xaml.PropertyPath.$ctor1("Background", "Background", Bridge.fn.cacheBind(this, this.accessVisualStateProperty_13b3a4266b3a46ce93d50acf4019f209), Bridge.fn.cacheBind(this, this.setVisualStateProperty_13b3a4266b3a46ce93d50acf4019f209), Bridge.fn.cacheBind(this, this.setAnimationVisualStateProperty_13b3a4266b3a46ce93d50acf4019f209), Bridge.fn.cacheBind(this, this.setLocalVisualStateProperty_13b3a4266b3a46ce93d50acf4019f209), Bridge.fn.cacheBind(this, this.getVisualStateProperty_13b3a4266b3a46ce93d50acf4019f209)));
                    Bridge.global.Windows.UI.Xaml.Media.Animation.Storyboard.SetTarget(ObjectAnimationUsingKeyFrames_fb778b8ba66545b1bf08313241ae18a5, Border_00f6894029e14a8f8c23a2be0d5b6078);

                    templateInstance_735791bc23f24113b707ed2d84c8ccfe.TemplateContent = Border_08ce5701cc024b6693a7487d9b07b912;
                    return templateInstance_735791bc23f24113b707ed2d84c8ccfe;
                }));

                Setter_975f3cb516bb4c23abd2408ec729cadd.Value = ControlTemplate_05bc8a33568c44a19d2437e36e9633ef;


                Style_65fe9771e9b04e3f8983b6b90e7788b8.Setters.add(Setter_eaacf59bc589419a84b6bea2923f8692);
                Style_65fe9771e9b04e3f8983b6b90e7788b8.Setters.add(Setter_1848fca41ff54b98b7fa87c3ffd0a411);
                Style_65fe9771e9b04e3f8983b6b90e7788b8.Setters.add(Setter_adadeee8d19444f49cdf2882e5159268);
                Style_65fe9771e9b04e3f8983b6b90e7788b8.Setters.add(Setter_e9c162bcaaed4bc2a6448f9f931387e6);
                Style_65fe9771e9b04e3f8983b6b90e7788b8.Setters.add(Setter_ee7c80563b9e4379938dee09e38a2b68);
                Style_65fe9771e9b04e3f8983b6b90e7788b8.Setters.add(Setter_f11c0b2969fa4d4986e59571be84f9cd);
                Style_65fe9771e9b04e3f8983b6b90e7788b8.Setters.add(Setter_905f305ca80d4654b4dc443e2dbb05e0);
                Style_65fe9771e9b04e3f8983b6b90e7788b8.Setters.add(Setter_975f3cb516bb4c23abd2408ec729cadd);


                this.setItem(Bridge.global.Windows.UI.Xaml.Controls.Primitives.ToggleButton, Style_65fe9771e9b04e3f8983b6b90e7788b8);






            },
            accessVisualStateProperty_470b309e8eac48f199e4c9a667787437: function (rootTargetObjectInstance) {
                return new (Bridge.GeneratorEnumerable$1(System.Tuple$3(Windows.UI.Xaml.DependencyObject,Windows.UI.Xaml.DependencyProperty,System.Nullable$1(System.Int32))))(Bridge.fn.bind(this, function (rootTargetObjectInstance) {
                    var $step = 0,
                        $jumpFromFinally,
                        $returnValue,
                        $async_e;

                    var $enumerator = new (Bridge.GeneratorEnumerator$1(System.Tuple$3(Windows.UI.Xaml.DependencyObject,Windows.UI.Xaml.DependencyProperty,System.Nullable$1(System.Int32))))(Bridge.fn.bind(this, function () {
                        try {
                            for (;;) {
                                switch ($step) {
                                    case 0: {
                                        return false;
                                    }
                                    default: {
                                        return false;
                                    }
                                }
                            }
                        } catch($async_e1) {
                            $async_e = System.Exception.create($async_e1);
                            throw $async_e;
                        }
                    }));
                    return $enumerator;
                }, arguments));
            },
            setVisualStateProperty_470b309e8eac48f199e4c9a667787437: function (finalTargetInstance, value) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Background").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "BackgroundProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                (finalTargetInstance).SetVisualStateValue(property, value);
            },
            setAnimationVisualStateProperty_470b309e8eac48f199e4c9a667787437: function (finalTargetInstance, value) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Background").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "BackgroundProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                (finalTargetInstance).SetAnimationValue(property, value);
            },
            setLocalVisualStateProperty_470b309e8eac48f199e4c9a667787437: function (finalTargetInstance, value) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Background").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "BackgroundProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                (finalTargetInstance).SetLocalValue(property, value);
            },
            getVisualStateProperty_470b309e8eac48f199e4c9a667787437: function (finalTargetInstance) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Background").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "BackgroundProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                return finalTargetInstance.GetVisualStateValue(property);
            },
            accessVisualStateProperty_b02e07de44a94f42bb0da21689e6877e: function (rootTargetObjectInstance) {
                return new (Bridge.GeneratorEnumerable$1(System.Tuple$3(Windows.UI.Xaml.DependencyObject,Windows.UI.Xaml.DependencyProperty,System.Nullable$1(System.Int32))))(Bridge.fn.bind(this, function (rootTargetObjectInstance) {
                    var $step = 0,
                        $jumpFromFinally,
                        $returnValue,
                        $async_e;

                    var $enumerator = new (Bridge.GeneratorEnumerator$1(System.Tuple$3(Windows.UI.Xaml.DependencyObject,Windows.UI.Xaml.DependencyProperty,System.Nullable$1(System.Int32))))(Bridge.fn.bind(this, function () {
                        try {
                            for (;;) {
                                switch ($step) {
                                    case 0: {
                                        return false;
                                    }
                                    default: {
                                        return false;
                                    }
                                }
                            }
                        } catch($async_e1) {
                            $async_e = System.Exception.create($async_e1);
                            throw $async_e;
                        }
                    }));
                    return $enumerator;
                }, arguments));
            },
            setVisualStateProperty_b02e07de44a94f42bb0da21689e6877e: function (finalTargetInstance, value) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Background").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "BackgroundProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                (finalTargetInstance).SetVisualStateValue(property, value);
            },
            setAnimationVisualStateProperty_b02e07de44a94f42bb0da21689e6877e: function (finalTargetInstance, value) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Background").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "BackgroundProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                (finalTargetInstance).SetAnimationValue(property, value);
            },
            setLocalVisualStateProperty_b02e07de44a94f42bb0da21689e6877e: function (finalTargetInstance, value) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Background").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "BackgroundProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                (finalTargetInstance).SetLocalValue(property, value);
            },
            getVisualStateProperty_b02e07de44a94f42bb0da21689e6877e: function (finalTargetInstance) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Background").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "BackgroundProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                return finalTargetInstance.GetVisualStateValue(property);
            },
            accessVisualStateProperty_dbb0e12eeff1468f92a2e33544198d8a: function (rootTargetObjectInstance) {
                return new (Bridge.GeneratorEnumerable$1(System.Tuple$3(Windows.UI.Xaml.DependencyObject,Windows.UI.Xaml.DependencyProperty,System.Nullable$1(System.Int32))))(Bridge.fn.bind(this, function (rootTargetObjectInstance) {
                    var $step = 0,
                        $jumpFromFinally,
                        $returnValue,
                        $async_e;

                    var $enumerator = new (Bridge.GeneratorEnumerator$1(System.Tuple$3(Windows.UI.Xaml.DependencyObject,Windows.UI.Xaml.DependencyProperty,System.Nullable$1(System.Int32))))(Bridge.fn.bind(this, function () {
                        try {
                            for (;;) {
                                switch ($step) {
                                    case 0: {
                                        return false;
                                    }
                                    default: {
                                        return false;
                                    }
                                }
                            }
                        } catch($async_e1) {
                            $async_e = System.Exception.create($async_e1);
                            throw $async_e;
                        }
                    }));
                    return $enumerator;
                }, arguments));
            },
            setVisualStateProperty_dbb0e12eeff1468f92a2e33544198d8a: function (finalTargetInstance, value) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Background").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "BackgroundProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                (finalTargetInstance).SetVisualStateValue(property, value);
            },
            setAnimationVisualStateProperty_dbb0e12eeff1468f92a2e33544198d8a: function (finalTargetInstance, value) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Background").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "BackgroundProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                (finalTargetInstance).SetAnimationValue(property, value);
            },
            setLocalVisualStateProperty_dbb0e12eeff1468f92a2e33544198d8a: function (finalTargetInstance, value) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Background").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "BackgroundProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                (finalTargetInstance).SetLocalValue(property, value);
            },
            getVisualStateProperty_dbb0e12eeff1468f92a2e33544198d8a: function (finalTargetInstance) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Background").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "BackgroundProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                return finalTargetInstance.GetVisualStateValue(property);
            },
            accessVisualStateProperty_fc7934b10b5d48e8bfba30c2970d1211: function (rootTargetObjectInstance) {
                return new (Bridge.GeneratorEnumerable$1(System.Tuple$3(Windows.UI.Xaml.DependencyObject,Windows.UI.Xaml.DependencyProperty,System.Nullable$1(System.Int32))))(Bridge.fn.bind(this, function (rootTargetObjectInstance) {
                    var $step = 0,
                        $jumpFromFinally,
                        $returnValue,
                        $async_e;

                    var $enumerator = new (Bridge.GeneratorEnumerator$1(System.Tuple$3(Windows.UI.Xaml.DependencyObject,Windows.UI.Xaml.DependencyProperty,System.Nullable$1(System.Int32))))(Bridge.fn.bind(this, function () {
                        try {
                            for (;;) {
                                switch ($step) {
                                    case 0: {
                                        return false;
                                    }
                                    default: {
                                        return false;
                                    }
                                }
                            }
                        } catch($async_e1) {
                            $async_e = System.Exception.create($async_e1);
                            throw $async_e;
                        }
                    }));
                    return $enumerator;
                }, arguments));
            },
            setVisualStateProperty_fc7934b10b5d48e8bfba30c2970d1211: function (finalTargetInstance, value) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Background").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "BackgroundProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                (finalTargetInstance).SetVisualStateValue(property, value);
            },
            setAnimationVisualStateProperty_fc7934b10b5d48e8bfba30c2970d1211: function (finalTargetInstance, value) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Background").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "BackgroundProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                (finalTargetInstance).SetAnimationValue(property, value);
            },
            setLocalVisualStateProperty_fc7934b10b5d48e8bfba30c2970d1211: function (finalTargetInstance, value) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Background").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "BackgroundProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                (finalTargetInstance).SetLocalValue(property, value);
            },
            getVisualStateProperty_fc7934b10b5d48e8bfba30c2970d1211: function (finalTargetInstance) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Background").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "BackgroundProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                return finalTargetInstance.GetVisualStateValue(property);
            },
            accessVisualStateProperty_b8a143d4613e4a75b8d7c08bfe836ecc: function (rootTargetObjectInstance) {
                return new (Bridge.GeneratorEnumerable$1(System.Tuple$3(Windows.UI.Xaml.DependencyObject,Windows.UI.Xaml.DependencyProperty,System.Nullable$1(System.Int32))))(Bridge.fn.bind(this, function (rootTargetObjectInstance) {
                    var $step = 0,
                        $jumpFromFinally,
                        $returnValue,
                        $async_e;

                    var $enumerator = new (Bridge.GeneratorEnumerator$1(System.Tuple$3(Windows.UI.Xaml.DependencyObject,Windows.UI.Xaml.DependencyProperty,System.Nullable$1(System.Int32))))(Bridge.fn.bind(this, function () {
                        try {
                            for (;;) {
                                switch ($step) {
                                    case 0: {
                                        return false;
                                    }
                                    default: {
                                        return false;
                                    }
                                }
                            }
                        } catch($async_e1) {
                            $async_e = System.Exception.create($async_e1);
                            throw $async_e;
                        }
                    }));
                    return $enumerator;
                }, arguments));
            },
            setVisualStateProperty_b8a143d4613e4a75b8d7c08bfe836ecc: function (finalTargetInstance, value) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Background").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "BackgroundProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                (finalTargetInstance).SetVisualStateValue(property, value);
            },
            setAnimationVisualStateProperty_b8a143d4613e4a75b8d7c08bfe836ecc: function (finalTargetInstance, value) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Background").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "BackgroundProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                (finalTargetInstance).SetAnimationValue(property, value);
            },
            setLocalVisualStateProperty_b8a143d4613e4a75b8d7c08bfe836ecc: function (finalTargetInstance, value) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Background").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "BackgroundProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                (finalTargetInstance).SetLocalValue(property, value);
            },
            getVisualStateProperty_b8a143d4613e4a75b8d7c08bfe836ecc: function (finalTargetInstance) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Background").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "BackgroundProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                return finalTargetInstance.GetVisualStateValue(property);
            },
            accessVisualStateProperty_d930e5a5734b4e4a9aee4bbaea2998b6: function (rootTargetObjectInstance) {
                return new (Bridge.GeneratorEnumerable$1(System.Tuple$3(Windows.UI.Xaml.DependencyObject,Windows.UI.Xaml.DependencyProperty,System.Nullable$1(System.Int32))))(Bridge.fn.bind(this, function (rootTargetObjectInstance) {
                    var $step = 0,
                        $jumpFromFinally,
                        $returnValue,
                        $async_e;

                    var $enumerator = new (Bridge.GeneratorEnumerator$1(System.Tuple$3(Windows.UI.Xaml.DependencyObject,Windows.UI.Xaml.DependencyProperty,System.Nullable$1(System.Int32))))(Bridge.fn.bind(this, function () {
                        try {
                            for (;;) {
                                switch ($step) {
                                    case 0: {
                                        return false;
                                    }
                                    default: {
                                        return false;
                                    }
                                }
                            }
                        } catch($async_e1) {
                            $async_e = System.Exception.create($async_e1);
                            throw $async_e;
                        }
                    }));
                    return $enumerator;
                }, arguments));
            },
            setVisualStateProperty_d930e5a5734b4e4a9aee4bbaea2998b6: function (finalTargetInstance, value) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Background").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "BackgroundProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                (finalTargetInstance).SetVisualStateValue(property, value);
            },
            setAnimationVisualStateProperty_d930e5a5734b4e4a9aee4bbaea2998b6: function (finalTargetInstance, value) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Background").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "BackgroundProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                (finalTargetInstance).SetAnimationValue(property, value);
            },
            setLocalVisualStateProperty_d930e5a5734b4e4a9aee4bbaea2998b6: function (finalTargetInstance, value) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Background").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "BackgroundProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                (finalTargetInstance).SetLocalValue(property, value);
            },
            getVisualStateProperty_d930e5a5734b4e4a9aee4bbaea2998b6: function (finalTargetInstance) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Background").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "BackgroundProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                return finalTargetInstance.GetVisualStateValue(property);
            },
            accessVisualStateProperty_1cd2be2b60494cb2ab753dbda1e770dd: function (rootTargetObjectInstance) {
                return new (Bridge.GeneratorEnumerable$1(System.Tuple$3(Windows.UI.Xaml.DependencyObject,Windows.UI.Xaml.DependencyProperty,System.Nullable$1(System.Int32))))(Bridge.fn.bind(this, function (rootTargetObjectInstance) {
                    var $step = 0,
                        $jumpFromFinally,
                        $returnValue,
                        $async_e;

                    var $enumerator = new (Bridge.GeneratorEnumerator$1(System.Tuple$3(Windows.UI.Xaml.DependencyObject,Windows.UI.Xaml.DependencyProperty,System.Nullable$1(System.Int32))))(Bridge.fn.bind(this, function () {
                        try {
                            for (;;) {
                                switch ($step) {
                                    case 0: {
                                        return false;
                                    }
                                    default: {
                                        return false;
                                    }
                                }
                            }
                        } catch($async_e1) {
                            $async_e = System.Exception.create($async_e1);
                            throw $async_e;
                        }
                    }));
                    return $enumerator;
                }, arguments));
            },
            setVisualStateProperty_1cd2be2b60494cb2ab753dbda1e770dd: function (finalTargetInstance, value) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Background").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "BackgroundProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                (finalTargetInstance).SetVisualStateValue(property, value);
            },
            setAnimationVisualStateProperty_1cd2be2b60494cb2ab753dbda1e770dd: function (finalTargetInstance, value) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Background").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "BackgroundProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                (finalTargetInstance).SetAnimationValue(property, value);
            },
            setLocalVisualStateProperty_1cd2be2b60494cb2ab753dbda1e770dd: function (finalTargetInstance, value) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Background").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "BackgroundProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                (finalTargetInstance).SetLocalValue(property, value);
            },
            getVisualStateProperty_1cd2be2b60494cb2ab753dbda1e770dd: function (finalTargetInstance) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Background").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "BackgroundProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                return finalTargetInstance.GetVisualStateValue(property);
            },
            accessVisualStateProperty_932b42a36ef2485e8f5ba05fdc97314c: function (rootTargetObjectInstance) {
                return new (Bridge.GeneratorEnumerable$1(System.Tuple$3(Windows.UI.Xaml.DependencyObject,Windows.UI.Xaml.DependencyProperty,System.Nullable$1(System.Int32))))(Bridge.fn.bind(this, function (rootTargetObjectInstance) {
                    var $step = 0,
                        $jumpFromFinally,
                        $returnValue,
                        $async_e;

                    var $enumerator = new (Bridge.GeneratorEnumerator$1(System.Tuple$3(Windows.UI.Xaml.DependencyObject,Windows.UI.Xaml.DependencyProperty,System.Nullable$1(System.Int32))))(Bridge.fn.bind(this, function () {
                        try {
                            for (;;) {
                                switch ($step) {
                                    case 0: {
                                        return false;
                                    }
                                    default: {
                                        return false;
                                    }
                                }
                            }
                        } catch($async_e1) {
                            $async_e = System.Exception.create($async_e1);
                            throw $async_e;
                        }
                    }));
                    return $enumerator;
                }, arguments));
            },
            setVisualStateProperty_932b42a36ef2485e8f5ba05fdc97314c: function (finalTargetInstance, value) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Background").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "BackgroundProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                (finalTargetInstance).SetVisualStateValue(property, value);
            },
            setAnimationVisualStateProperty_932b42a36ef2485e8f5ba05fdc97314c: function (finalTargetInstance, value) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Background").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "BackgroundProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                (finalTargetInstance).SetAnimationValue(property, value);
            },
            setLocalVisualStateProperty_932b42a36ef2485e8f5ba05fdc97314c: function (finalTargetInstance, value) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Background").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "BackgroundProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                (finalTargetInstance).SetLocalValue(property, value);
            },
            getVisualStateProperty_932b42a36ef2485e8f5ba05fdc97314c: function (finalTargetInstance) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Background").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "BackgroundProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                return finalTargetInstance.GetVisualStateValue(property);
            },
            accessVisualStateProperty_bb5c51247a7441ff82d13b07a6eaa990: function (rootTargetObjectInstance) {
                return new (Bridge.GeneratorEnumerable$1(System.Tuple$3(Windows.UI.Xaml.DependencyObject,Windows.UI.Xaml.DependencyProperty,System.Nullable$1(System.Int32))))(Bridge.fn.bind(this, function (rootTargetObjectInstance) {
                    var $step = 0,
                        $jumpFromFinally,
                        $returnValue,
                        $async_e;

                    var $enumerator = new (Bridge.GeneratorEnumerator$1(System.Tuple$3(Windows.UI.Xaml.DependencyObject,Windows.UI.Xaml.DependencyProperty,System.Nullable$1(System.Int32))))(Bridge.fn.bind(this, function () {
                        try {
                            for (;;) {
                                switch ($step) {
                                    case 0: {
                                        return false;
                                    }
                                    default: {
                                        return false;
                                    }
                                }
                            }
                        } catch($async_e1) {
                            $async_e = System.Exception.create($async_e1);
                            throw $async_e;
                        }
                    }));
                    return $enumerator;
                }, arguments));
            },
            setVisualStateProperty_bb5c51247a7441ff82d13b07a6eaa990: function (finalTargetInstance, value) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Background").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "BackgroundProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                (finalTargetInstance).SetVisualStateValue(property, value);
            },
            setAnimationVisualStateProperty_bb5c51247a7441ff82d13b07a6eaa990: function (finalTargetInstance, value) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Background").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "BackgroundProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                (finalTargetInstance).SetAnimationValue(property, value);
            },
            setLocalVisualStateProperty_bb5c51247a7441ff82d13b07a6eaa990: function (finalTargetInstance, value) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Background").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "BackgroundProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                (finalTargetInstance).SetLocalValue(property, value);
            },
            getVisualStateProperty_bb5c51247a7441ff82d13b07a6eaa990: function (finalTargetInstance) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Background").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "BackgroundProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                return finalTargetInstance.GetVisualStateValue(property);
            },
            accessVisualStateProperty_13b3a4266b3a46ce93d50acf4019f209: function (rootTargetObjectInstance) {
                return new (Bridge.GeneratorEnumerable$1(System.Tuple$3(Windows.UI.Xaml.DependencyObject,Windows.UI.Xaml.DependencyProperty,System.Nullable$1(System.Int32))))(Bridge.fn.bind(this, function (rootTargetObjectInstance) {
                    var $step = 0,
                        $jumpFromFinally,
                        $returnValue,
                        $async_e;

                    var $enumerator = new (Bridge.GeneratorEnumerator$1(System.Tuple$3(Windows.UI.Xaml.DependencyObject,Windows.UI.Xaml.DependencyProperty,System.Nullable$1(System.Int32))))(Bridge.fn.bind(this, function () {
                        try {
                            for (;;) {
                                switch ($step) {
                                    case 0: {
                                        return false;
                                    }
                                    default: {
                                        return false;
                                    }
                                }
                            }
                        } catch($async_e1) {
                            $async_e = System.Exception.create($async_e1);
                            throw $async_e;
                        }
                    }));
                    return $enumerator;
                }, arguments));
            },
            setVisualStateProperty_13b3a4266b3a46ce93d50acf4019f209: function (finalTargetInstance, value) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Background").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "BackgroundProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                (finalTargetInstance).SetVisualStateValue(property, value);
            },
            setAnimationVisualStateProperty_13b3a4266b3a46ce93d50acf4019f209: function (finalTargetInstance, value) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Background").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "BackgroundProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                (finalTargetInstance).SetAnimationValue(property, value);
            },
            setLocalVisualStateProperty_13b3a4266b3a46ce93d50acf4019f209: function (finalTargetInstance, value) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Background").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "BackgroundProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                (finalTargetInstance).SetLocalValue(property, value);
            },
            getVisualStateProperty_13b3a4266b3a46ce93d50acf4019f209: function (finalTargetInstance) {

                var finalTargetInstanceType = Bridge.getType(finalTargetInstance);
                var propertyDeclaringType = Bridge.Reflection.getMembers(finalTargetInstanceType, 16, 284, "Background").td;
                var propertyField = Bridge.Reflection.getMembers(propertyDeclaringType, 4, 284, "BackgroundProperty");
                var property = Bridge.cast(Bridge.Reflection.fieldAccess(propertyField, null), Windows.UI.Xaml.DependencyProperty);

                return finalTargetInstance.GetVisualStateValue(property);
            }
        }
    });

    Bridge.define("___CSHTML5__Component__Defaultstyles__INTERNAL_Defaulttooltipstyle__Xaml", {
        inherits: [Bridge.global.Windows.UI.Xaml.ResourceDictionary],
        fields: {
            _contentLoaded: false
        },
        ctors: {
            ctor: function () {
                this.$initialize();
                Bridge.global.Windows.UI.Xaml.ResourceDictionary.ctor.call(this);
                this.InitializeComponent();

                if (Bridge.is(this, Windows.UI.Xaml.UIElement)) {
                    Bridge.cast(this, Windows.UI.Xaml.UIElement).XamlSourcePath = "CSHTML5\\DefaultStyles\\INTERNAL_DefaultToolTipStyle.xaml";
                }
            }
        },
        methods: {
            InitializeComponent: function () {
                var $t;
                if (this._contentLoaded) {
                    return;
                }
                this._contentLoaded = true;


                if (Bridge.is(this, Windows.UI.Xaml.UIElement)) {
                    Bridge.cast(this, Windows.UI.Xaml.UIElement).XamlSourcePath = "CSHTML5\\DefaultStyles\\INTERNAL_DefaultToolTipStyle.xaml";
                }



                var Style_7601e1b0078240918a516230ccae3f64 = new Bridge.global.Windows.UI.Xaml.Style.ctor();
                Style_7601e1b0078240918a516230ccae3f64.TargetType = Bridge.global.Windows.UI.Xaml.Controls.ToolTip;
                var Setter_69fbb64214394265a2214335b9e5d1ab = new Bridge.global.Windows.UI.Xaml.Setter.ctor();
                Setter_69fbb64214394265a2214335b9e5d1ab.Property = Bridge.global.Windows.UI.Xaml.Controls.Control.ForegroundProperty;
                Setter_69fbb64214394265a2214335b9e5d1ab.Value = new Bridge.global.Windows.UI.Xaml.Media.SolidColorBrush.$ctor1(($t = new Bridge.global.Windows.UI.Color(), $t.A = 255, $t.R = 102, $t.G = 102, $t.B = 102, $t));

                var Setter_4c22f73af63544fc8770f290f1d86713 = new Bridge.global.Windows.UI.Xaml.Setter.ctor();
                Setter_4c22f73af63544fc8770f290f1d86713.Property = Bridge.global.Windows.UI.Xaml.Controls.Control.BackgroundProperty;
                Setter_4c22f73af63544fc8770f290f1d86713.Value = new Bridge.global.Windows.UI.Xaml.Media.SolidColorBrush.$ctor1(($t = new Bridge.global.Windows.UI.Color(), $t.A = 255, $t.R = 255, $t.G = 255, $t.B = 255, $t));

                var Setter_a64021ba97164b34bb1739e94ed37556 = new Bridge.global.Windows.UI.Xaml.Setter.ctor();
                Setter_a64021ba97164b34bb1739e94ed37556.Property = Bridge.global.Windows.UI.Xaml.Controls.Control.BorderBrushProperty;
                Setter_a64021ba97164b34bb1739e94ed37556.Value = new Bridge.global.Windows.UI.Xaml.Media.SolidColorBrush.$ctor1(($t = new Bridge.global.Windows.UI.Color(), $t.A = 255, $t.R = 128, $t.G = 128, $t.B = 128, $t));

                var Setter_e26bb8c6c4a54cfb9cebfca77f4c5105 = new Bridge.global.Windows.UI.Xaml.Setter.ctor();
                Setter_e26bb8c6c4a54cfb9cebfca77f4c5105.Property = Bridge.global.Windows.UI.Xaml.Controls.Control.BorderThicknessProperty;
                Setter_e26bb8c6c4a54cfb9cebfca77f4c5105.Value = new Bridge.global.Windows.UI.Xaml.Thickness.$ctor1(2).$clone();

                var Setter_5c155cae315844a2a3cf737ea1f36a68 = new Bridge.global.Windows.UI.Xaml.Setter.ctor();
                Setter_5c155cae315844a2a3cf737ea1f36a68.Property = Bridge.global.Windows.UI.Xaml.Controls.Control.FontSizeProperty;
                Setter_5c155cae315844a2a3cf737ea1f36a68.Value = Bridge.box(12.0, System.Double, System.Double.format, System.Double.getHashCode);

                var Setter_d337f0341a1e41dc952eb04aa2e4df5a = new Bridge.global.Windows.UI.Xaml.Setter.ctor();
                Setter_d337f0341a1e41dc952eb04aa2e4df5a.Property = Bridge.global.Windows.UI.Xaml.Controls.Control.PaddingProperty;
                Setter_d337f0341a1e41dc952eb04aa2e4df5a.Value = new Bridge.global.Windows.UI.Xaml.Thickness.$ctor2(10, 6, 10, 7).$clone();

                var Setter_04aed9c0b19d4ee3b4b62e9695590bad = new Bridge.global.Windows.UI.Xaml.Setter.ctor();
                Setter_04aed9c0b19d4ee3b4b62e9695590bad.Property = Bridge.global.Windows.UI.Xaml.Controls.Control.TemplateProperty;
                var ControlTemplate_660ae2a0fd83483f8461e6e4590b6c92 = new Bridge.global.Windows.UI.Xaml.Controls.ControlTemplate();
                ControlTemplate_660ae2a0fd83483f8461e6e4590b6c92.TargetType = Bridge.global.Windows.UI.Xaml.Controls.ToolTip;
                ControlTemplate_660ae2a0fd83483f8461e6e4590b6c92.SetMethodToInstantiateFrameworkTemplate(function (templateOwner_ControlTemplate_660ae2a0fd83483f8461e6e4590b6c92) {
                    var templateInstance_61e489204d254f299ca9894a40affa63 = new Bridge.global.Windows.UI.Xaml.TemplateInstance();
                    templateInstance_61e489204d254f299ca9894a40affa63.TemplateOwner = templateOwner_ControlTemplate_660ae2a0fd83483f8461e6e4590b6c92;
                    var Border_301bb58a627346faaebf118177b51ae5 = new Bridge.global.Windows.UI.Xaml.Controls.Border();
                    templateOwner_ControlTemplate_660ae2a0fd83483f8461e6e4590b6c92.RegisterName("LayoutRoot", Border_301bb58a627346faaebf118177b51ae5);
                    Border_301bb58a627346faaebf118177b51ae5.Name = "LayoutRoot";
                    var ContentPresenter_cdffc3f02e484b218d294392a47602ee = new Bridge.global.Windows.UI.Xaml.Controls.ContentPresenter();
                    var Binding_4263ee9712e6462a9cac909ee6af62bb = new Bridge.global.Windows.UI.Xaml.Data.Binding.ctor();
                    Binding_4263ee9712e6462a9cac909ee6af62bb.Path = new Bridge.global.Windows.UI.Xaml.PropertyPath.ctor("ContentTemplate");
                    var RelativeSource_119d3d4fee6b422ea148f018eefe6db5 = new Bridge.global.Windows.UI.Xaml.Data.RelativeSource.ctor();
                    RelativeSource_119d3d4fee6b422ea148f018eefe6db5.Mode = Bridge.global.Windows.UI.Xaml.Data.RelativeSourceMode.TemplatedParent;

                    Binding_4263ee9712e6462a9cac909ee6af62bb.RelativeSource = RelativeSource_119d3d4fee6b422ea148f018eefe6db5;


                    Binding_4263ee9712e6462a9cac909ee6af62bb.TemplateOwner = templateInstance_61e489204d254f299ca9894a40affa63;

                    var Binding_e5054ef604534424aaa3fddfb055b946 = new Bridge.global.Windows.UI.Xaml.Data.Binding.ctor();
                    Binding_e5054ef604534424aaa3fddfb055b946.Path = new Bridge.global.Windows.UI.Xaml.PropertyPath.ctor("Content");
                    var RelativeSource_b905f72106544de4a9a2c9d389ad66b8 = new Bridge.global.Windows.UI.Xaml.Data.RelativeSource.ctor();
                    RelativeSource_b905f72106544de4a9a2c9d389ad66b8.Mode = Bridge.global.Windows.UI.Xaml.Data.RelativeSourceMode.TemplatedParent;

                    Binding_e5054ef604534424aaa3fddfb055b946.RelativeSource = RelativeSource_b905f72106544de4a9a2c9d389ad66b8;


                    Binding_e5054ef604534424aaa3fddfb055b946.TemplateOwner = templateInstance_61e489204d254f299ca9894a40affa63;

                    var Binding_d9f4b283ea74489a93578bff4bbeaec0 = new Bridge.global.Windows.UI.Xaml.Data.Binding.ctor();
                    Binding_d9f4b283ea74489a93578bff4bbeaec0.Path = new Bridge.global.Windows.UI.Xaml.PropertyPath.ctor("Padding");
                    var RelativeSource_1a81a238e876403e83720e315d76ac4b = new Bridge.global.Windows.UI.Xaml.Data.RelativeSource.ctor();
                    RelativeSource_1a81a238e876403e83720e315d76ac4b.Mode = Bridge.global.Windows.UI.Xaml.Data.RelativeSourceMode.TemplatedParent;

                    Binding_d9f4b283ea74489a93578bff4bbeaec0.RelativeSource = RelativeSource_1a81a238e876403e83720e315d76ac4b;


                    Binding_d9f4b283ea74489a93578bff4bbeaec0.TemplateOwner = templateInstance_61e489204d254f299ca9894a40affa63;


                    Border_301bb58a627346faaebf118177b51ae5.Child = ContentPresenter_cdffc3f02e484b218d294392a47602ee;

                    var Binding_9aff7f3e56da41c4b0377385e792de12 = new Bridge.global.Windows.UI.Xaml.Data.Binding.ctor();
                    Binding_9aff7f3e56da41c4b0377385e792de12.Path = new Bridge.global.Windows.UI.Xaml.PropertyPath.ctor("BorderBrush");
                    var RelativeSource_df147367850c4d808e9b906b6aa88d9d = new Bridge.global.Windows.UI.Xaml.Data.RelativeSource.ctor();
                    RelativeSource_df147367850c4d808e9b906b6aa88d9d.Mode = Bridge.global.Windows.UI.Xaml.Data.RelativeSourceMode.TemplatedParent;

                    Binding_9aff7f3e56da41c4b0377385e792de12.RelativeSource = RelativeSource_df147367850c4d808e9b906b6aa88d9d;


                    Binding_9aff7f3e56da41c4b0377385e792de12.TemplateOwner = templateInstance_61e489204d254f299ca9894a40affa63;

                    var Binding_73cc119c96d34329a812ed60d14497b8 = new Bridge.global.Windows.UI.Xaml.Data.Binding.ctor();
                    Binding_73cc119c96d34329a812ed60d14497b8.Path = new Bridge.global.Windows.UI.Xaml.PropertyPath.ctor("BorderThickness");
                    var RelativeSource_a6a05d1e871747859a7a140337912702 = new Bridge.global.Windows.UI.Xaml.Data.RelativeSource.ctor();
                    RelativeSource_a6a05d1e871747859a7a140337912702.Mode = Bridge.global.Windows.UI.Xaml.Data.RelativeSourceMode.TemplatedParent;

                    Binding_73cc119c96d34329a812ed60d14497b8.RelativeSource = RelativeSource_a6a05d1e871747859a7a140337912702;


                    Binding_73cc119c96d34329a812ed60d14497b8.TemplateOwner = templateInstance_61e489204d254f299ca9894a40affa63;

                    var Binding_782d012ead2b43bcbbb49f2786cad508 = new Bridge.global.Windows.UI.Xaml.Data.Binding.ctor();
                    Binding_782d012ead2b43bcbbb49f2786cad508.Path = new Bridge.global.Windows.UI.Xaml.PropertyPath.ctor("Background");
                    var RelativeSource_98601b1abf754e0da08caa4a41c39e01 = new Bridge.global.Windows.UI.Xaml.Data.RelativeSource.ctor();
                    RelativeSource_98601b1abf754e0da08caa4a41c39e01.Mode = Bridge.global.Windows.UI.Xaml.Data.RelativeSourceMode.TemplatedParent;

                    Binding_782d012ead2b43bcbbb49f2786cad508.RelativeSource = RelativeSource_98601b1abf754e0da08caa4a41c39e01;


                    Binding_782d012ead2b43bcbbb49f2786cad508.TemplateOwner = templateInstance_61e489204d254f299ca9894a40affa63;


                    Bridge.global.Windows.UI.Xaml.Data.BindingOperations.SetBinding(ContentPresenter_cdffc3f02e484b218d294392a47602ee, Bridge.global.Windows.UI.Xaml.Controls.ContentControl.ContentTemplateProperty, Binding_4263ee9712e6462a9cac909ee6af62bb);
                    Bridge.global.Windows.UI.Xaml.Data.BindingOperations.SetBinding(ContentPresenter_cdffc3f02e484b218d294392a47602ee, Bridge.global.Windows.UI.Xaml.Controls.ContentControl.ContentProperty, Binding_e5054ef604534424aaa3fddfb055b946);
                    Bridge.global.Windows.UI.Xaml.Data.BindingOperations.SetBinding(ContentPresenter_cdffc3f02e484b218d294392a47602ee, Bridge.global.Windows.UI.Xaml.FrameworkElement.MarginProperty, Binding_d9f4b283ea74489a93578bff4bbeaec0);
                    Bridge.global.Windows.UI.Xaml.Data.BindingOperations.SetBinding(Border_301bb58a627346faaebf118177b51ae5, Bridge.global.Windows.UI.Xaml.Controls.Border.BorderBrushProperty, Binding_9aff7f3e56da41c4b0377385e792de12);
                    Bridge.global.Windows.UI.Xaml.Data.BindingOperations.SetBinding(Border_301bb58a627346faaebf118177b51ae5, Bridge.global.Windows.UI.Xaml.Controls.Border.BorderThicknessProperty, Binding_73cc119c96d34329a812ed60d14497b8);
                    Bridge.global.Windows.UI.Xaml.Data.BindingOperations.SetBinding(Border_301bb58a627346faaebf118177b51ae5, Bridge.global.Windows.UI.Xaml.Controls.Border.BackgroundProperty, Binding_782d012ead2b43bcbbb49f2786cad508);

                    templateInstance_61e489204d254f299ca9894a40affa63.TemplateContent = Border_301bb58a627346faaebf118177b51ae5;
                    return templateInstance_61e489204d254f299ca9894a40affa63;
                });

                Setter_04aed9c0b19d4ee3b4b62e9695590bad.Value = ControlTemplate_660ae2a0fd83483f8461e6e4590b6c92;


                Style_7601e1b0078240918a516230ccae3f64.Setters.add(Setter_69fbb64214394265a2214335b9e5d1ab);
                Style_7601e1b0078240918a516230ccae3f64.Setters.add(Setter_4c22f73af63544fc8770f290f1d86713);
                Style_7601e1b0078240918a516230ccae3f64.Setters.add(Setter_a64021ba97164b34bb1739e94ed37556);
                Style_7601e1b0078240918a516230ccae3f64.Setters.add(Setter_e26bb8c6c4a54cfb9cebfca77f4c5105);
                Style_7601e1b0078240918a516230ccae3f64.Setters.add(Setter_5c155cae315844a2a3cf737ea1f36a68);
                Style_7601e1b0078240918a516230ccae3f64.Setters.add(Setter_d337f0341a1e41dc952eb04aa2e4df5a);
                Style_7601e1b0078240918a516230ccae3f64.Setters.add(Setter_04aed9c0b19d4ee3b4b62e9695590bad);


                this.setItem(Bridge.global.Windows.UI.Xaml.Controls.ToolTip, Style_7601e1b0078240918a516230ccae3f64);






            }
        }
    });

    Bridge.define("___CSHTML5__Component__Defaultstyles__INTERNAL_Defaultviewboxstyle__Xaml", {
        inherits: [Bridge.global.Windows.UI.Xaml.ResourceDictionary],
        fields: {
            _contentLoaded: false
        },
        ctors: {
            ctor: function () {
                this.$initialize();
                Bridge.global.Windows.UI.Xaml.ResourceDictionary.ctor.call(this);
                this.InitializeComponent();

                if (Bridge.is(this, Windows.UI.Xaml.UIElement)) {
                    Bridge.cast(this, Windows.UI.Xaml.UIElement).XamlSourcePath = "CSHTML5\\DefaultStyles\\INTERNAL_DefaultViewboxStyle.xaml";
                }
            }
        },
        methods: {
            InitializeComponent: function () {
                if (this._contentLoaded) {
                    return;
                }
                this._contentLoaded = true;


                if (Bridge.is(this, Windows.UI.Xaml.UIElement)) {
                    Bridge.cast(this, Windows.UI.Xaml.UIElement).XamlSourcePath = "CSHTML5\\DefaultStyles\\INTERNAL_DefaultViewboxStyle.xaml";
                }



                var Style_1caa9ada5c9b4a9ab408585530bf032d = new Bridge.global.Windows.UI.Xaml.Style.ctor();
                Style_1caa9ada5c9b4a9ab408585530bf032d.TargetType = Bridge.global.Windows.UI.Xaml.Controls.Viewbox;
                var Setter_b0485942b3b64009bddc835a29d50ed9 = new Bridge.global.Windows.UI.Xaml.Setter.ctor();
                Setter_b0485942b3b64009bddc835a29d50ed9.Property = Bridge.global.Windows.UI.Xaml.UIElement.ClipToBoundsProperty;
                Setter_b0485942b3b64009bddc835a29d50ed9.Value = Bridge.box(true, System.Boolean, System.Boolean.toString);

                var Setter_12dced2da5d04eb5a68484dfbd767651 = new Bridge.global.Windows.UI.Xaml.Setter.ctor();
                Setter_12dced2da5d04eb5a68484dfbd767651.Property = Bridge.global.Windows.UI.Xaml.Controls.Control.TemplateProperty;
                var ControlTemplate_1836e40b75c0411691071eb12d3cb103 = new Bridge.global.Windows.UI.Xaml.Controls.ControlTemplate();
                ControlTemplate_1836e40b75c0411691071eb12d3cb103.TargetType = Bridge.global.Windows.UI.Xaml.Controls.Viewbox;
                ControlTemplate_1836e40b75c0411691071eb12d3cb103.SetMethodToInstantiateFrameworkTemplate(function (templateOwner_ControlTemplate_1836e40b75c0411691071eb12d3cb103) {
                    var templateInstance_84f9a2e39a6c4582aefccdfc9acaa6dd = new Bridge.global.Windows.UI.Xaml.TemplateInstance();
                    templateInstance_84f9a2e39a6c4582aefccdfc9acaa6dd.TemplateOwner = templateOwner_ControlTemplate_1836e40b75c0411691071eb12d3cb103;
                    var ContentPresenter_4685bf70d5c84d5e93d3998ee3c78811 = new Bridge.global.Windows.UI.Xaml.Controls.ContentPresenter();
                    var Canvas_ff58a5844f5c4d95af6b246ce9be9bc1 = new Bridge.global.Windows.UI.Xaml.Controls.Canvas();
                    templateOwner_ControlTemplate_1836e40b75c0411691071eb12d3cb103.RegisterName("RootCanvas", Canvas_ff58a5844f5c4d95af6b246ce9be9bc1);
                    Canvas_ff58a5844f5c4d95af6b246ce9be9bc1.Name = "RootCanvas";
                    templateOwner_ControlTemplate_1836e40b75c0411691071eb12d3cb103.RegisterName("Child", ContentPresenter_4685bf70d5c84d5e93d3998ee3c78811);
                    ContentPresenter_4685bf70d5c84d5e93d3998ee3c78811.Name = "Child";
                    var Binding_fd1a39fa81ab4cf2884997685671aad5 = new Bridge.global.Windows.UI.Xaml.Data.Binding.ctor();
                    Binding_fd1a39fa81ab4cf2884997685671aad5.Path = new Bridge.global.Windows.UI.Xaml.PropertyPath.ctor("HorizontalAlignment");
                    var RelativeSource_da3e090e02074f81936b06eef8b6ea62 = new Bridge.global.Windows.UI.Xaml.Data.RelativeSource.ctor();
                    RelativeSource_da3e090e02074f81936b06eef8b6ea62.Mode = Bridge.global.Windows.UI.Xaml.Data.RelativeSourceMode.TemplatedParent;

                    Binding_fd1a39fa81ab4cf2884997685671aad5.RelativeSource = RelativeSource_da3e090e02074f81936b06eef8b6ea62;


                    Binding_fd1a39fa81ab4cf2884997685671aad5.TemplateOwner = templateInstance_84f9a2e39a6c4582aefccdfc9acaa6dd;

                    var Binding_c63fabab7dad40c6a66348352ea889d8 = new Bridge.global.Windows.UI.Xaml.Data.Binding.ctor();
                    Binding_c63fabab7dad40c6a66348352ea889d8.Path = new Bridge.global.Windows.UI.Xaml.PropertyPath.ctor("VerticalAlignment");
                    var RelativeSource_af83c17fc4974a91904c25c1a808b5f2 = new Bridge.global.Windows.UI.Xaml.Data.RelativeSource.ctor();
                    RelativeSource_af83c17fc4974a91904c25c1a808b5f2.Mode = Bridge.global.Windows.UI.Xaml.Data.RelativeSourceMode.TemplatedParent;

                    Binding_c63fabab7dad40c6a66348352ea889d8.RelativeSource = RelativeSource_af83c17fc4974a91904c25c1a808b5f2;


                    Binding_c63fabab7dad40c6a66348352ea889d8.TemplateOwner = templateInstance_84f9a2e39a6c4582aefccdfc9acaa6dd;

                    var Binding_c48891a341794f3fae177181a4ec2389 = new Bridge.global.Windows.UI.Xaml.Data.Binding.ctor();
                    Binding_c48891a341794f3fae177181a4ec2389.Path = new Bridge.global.Windows.UI.Xaml.PropertyPath.ctor("Content");
                    var RelativeSource_1697824bfe5844a1b52fb91cd448a4c3 = new Bridge.global.Windows.UI.Xaml.Data.RelativeSource.ctor();
                    RelativeSource_1697824bfe5844a1b52fb91cd448a4c3.Mode = Bridge.global.Windows.UI.Xaml.Data.RelativeSourceMode.TemplatedParent;

                    Binding_c48891a341794f3fae177181a4ec2389.RelativeSource = RelativeSource_1697824bfe5844a1b52fb91cd448a4c3;


                    Binding_c48891a341794f3fae177181a4ec2389.TemplateOwner = templateInstance_84f9a2e39a6c4582aefccdfc9acaa6dd;

                    var Binding_0f3faf8489be451c826204a5a6b2a79d = new Bridge.global.Windows.UI.Xaml.Data.Binding.ctor();
                    Binding_0f3faf8489be451c826204a5a6b2a79d.Path = new Bridge.global.Windows.UI.Xaml.PropertyPath.ctor("ContentTemplate");
                    var RelativeSource_6c4fad2b1f904ad692e649def3bf4407 = new Bridge.global.Windows.UI.Xaml.Data.RelativeSource.ctor();
                    RelativeSource_6c4fad2b1f904ad692e649def3bf4407.Mode = Bridge.global.Windows.UI.Xaml.Data.RelativeSourceMode.TemplatedParent;

                    Binding_0f3faf8489be451c826204a5a6b2a79d.RelativeSource = RelativeSource_6c4fad2b1f904ad692e649def3bf4407;


                    Binding_0f3faf8489be451c826204a5a6b2a79d.TemplateOwner = templateInstance_84f9a2e39a6c4582aefccdfc9acaa6dd;


                    Canvas_ff58a5844f5c4d95af6b246ce9be9bc1.Children.add(ContentPresenter_4685bf70d5c84d5e93d3998ee3c78811);


                    Bridge.global.Windows.UI.Xaml.Data.BindingOperations.SetBinding(ContentPresenter_4685bf70d5c84d5e93d3998ee3c78811, Bridge.global.Windows.UI.Xaml.FrameworkElement.HorizontalAlignmentProperty, Binding_fd1a39fa81ab4cf2884997685671aad5);
                    Bridge.global.Windows.UI.Xaml.Data.BindingOperations.SetBinding(ContentPresenter_4685bf70d5c84d5e93d3998ee3c78811, Bridge.global.Windows.UI.Xaml.FrameworkElement.VerticalAlignmentProperty, Binding_c63fabab7dad40c6a66348352ea889d8);
                    Bridge.global.Windows.UI.Xaml.Data.BindingOperations.SetBinding(ContentPresenter_4685bf70d5c84d5e93d3998ee3c78811, Bridge.global.Windows.UI.Xaml.Controls.ContentControl.ContentProperty, Binding_c48891a341794f3fae177181a4ec2389);
                    Bridge.global.Windows.UI.Xaml.Data.BindingOperations.SetBinding(ContentPresenter_4685bf70d5c84d5e93d3998ee3c78811, Bridge.global.Windows.UI.Xaml.Controls.ContentControl.ContentTemplateProperty, Binding_0f3faf8489be451c826204a5a6b2a79d);

                    templateInstance_84f9a2e39a6c4582aefccdfc9acaa6dd.TemplateContent = Canvas_ff58a5844f5c4d95af6b246ce9be9bc1;
                    return templateInstance_84f9a2e39a6c4582aefccdfc9acaa6dd;
                });

                Setter_12dced2da5d04eb5a68484dfbd767651.Value = ControlTemplate_1836e40b75c0411691071eb12d3cb103;


                Style_1caa9ada5c9b4a9ab408585530bf032d.Setters.add(Setter_b0485942b3b64009bddc835a29d50ed9);
                Style_1caa9ada5c9b4a9ab408585530bf032d.Setters.add(Setter_12dced2da5d04eb5a68484dfbd767651);


                this.setItem(Bridge.global.Windows.UI.Xaml.Controls.Viewbox, Style_1caa9ada5c9b4a9ab408585530bf032d);






            }
        }
    });

    Bridge.define("___CSHTML5__Component__Themes__Generic__Xaml", {
        inherits: [Bridge.global.Windows.UI.Xaml.ResourceDictionary],
        fields: {
            _contentLoaded: false
        },
        ctors: {
            ctor: function () {
                this.$initialize();
                Bridge.global.Windows.UI.Xaml.ResourceDictionary.ctor.call(this);
                this.InitializeComponent();

                if (Bridge.is(this, Windows.UI.Xaml.UIElement)) {
                    Bridge.cast(this, Windows.UI.Xaml.UIElement).XamlSourcePath = "CSHTML5\\Themes\\generic.xaml";
                }
            }
        },
        methods: {
            InitializeComponent: function () {
                if (this._contentLoaded) {
                    return;
                }
                this._contentLoaded = true;


                if (Bridge.is(this, Windows.UI.Xaml.UIElement)) {
                    Bridge.cast(this, Windows.UI.Xaml.UIElement).XamlSourcePath = "CSHTML5\\Themes\\generic.xaml";
                }



                var ResourceDictionary_2fdecdf942c341deb3ad31bdf2fd5f3f = new Bridge.global.___CSHTML5__Component__Defaultstyles__INTERNAL_Defaultautocompleteboxstyle__Xaml();
                ResourceDictionary_2fdecdf942c341deb3ad31bdf2fd5f3f.Source = Bridge.cast(Bridge.global.DotNetForHtml5.Core.TypeFromStringConverters.ConvertFromInvariantString(Bridge.global.System.Uri, "../DefaultStyles/INTERNAL_DefaultAutoCompleteBoxStyle.xaml"), System.Uri);

                this.MergedDictionaries.add(ResourceDictionary_2fdecdf942c341deb3ad31bdf2fd5f3f);
                var ResourceDictionary_07015c5b00c74cc6933f379314ed293f = new Bridge.global.___CSHTML5__Component__Defaultstyles__INTERNAL_Defaultbuttonstyle__Xaml();
                ResourceDictionary_07015c5b00c74cc6933f379314ed293f.Source = Bridge.cast(Bridge.global.DotNetForHtml5.Core.TypeFromStringConverters.ConvertFromInvariantString(Bridge.global.System.Uri, "../DefaultStyles/INTERNAL_DefaultButtonStyle.xaml"), System.Uri);

                this.MergedDictionaries.add(ResourceDictionary_07015c5b00c74cc6933f379314ed293f);
                var ResourceDictionary_d90c138e3b024db4883cf05feca52cc7 = new Bridge.global.___CSHTML5__Component__Defaultstyles__INTERNAL_Defaultchildwindowstyle__Xaml();
                ResourceDictionary_d90c138e3b024db4883cf05feca52cc7.Source = Bridge.cast(Bridge.global.DotNetForHtml5.Core.TypeFromStringConverters.ConvertFromInvariantString(Bridge.global.System.Uri, "../DefaultStyles/INTERNAL_DefaultChildWindowStyle.xaml"), System.Uri);

                this.MergedDictionaries.add(ResourceDictionary_d90c138e3b024db4883cf05feca52cc7);
                var ResourceDictionary_9afbc206a23b4b8c8b6ce36dca31a1fb = new Bridge.global.___CSHTML5__Component__Defaultstyles__INTERNAL_Defaultcomboboxstyle__Xaml();
                ResourceDictionary_9afbc206a23b4b8c8b6ce36dca31a1fb.Source = Bridge.cast(Bridge.global.DotNetForHtml5.Core.TypeFromStringConverters.ConvertFromInvariantString(Bridge.global.System.Uri, "../DefaultStyles/INTERNAL_DefaultComboBoxStyle.xaml"), System.Uri);

                this.MergedDictionaries.add(ResourceDictionary_9afbc206a23b4b8c8b6ce36dca31a1fb);
                var ResourceDictionary_a956baa2b6f449a4bdfca3d1d0efd5c7 = new Bridge.global.___CSHTML5__Component__Defaultstyles__INTERNAL_Defaultdatapagerstyle__Xaml();
                ResourceDictionary_a956baa2b6f449a4bdfca3d1d0efd5c7.Source = Bridge.cast(Bridge.global.DotNetForHtml5.Core.TypeFromStringConverters.ConvertFromInvariantString(Bridge.global.System.Uri, "../DefaultStyles/INTERNAL_DefaultDataPagerStyle.xaml"), System.Uri);

                this.MergedDictionaries.add(ResourceDictionary_a956baa2b6f449a4bdfca3d1d0efd5c7);
                var ResourceDictionary_557ec46a58a049a9b602b17aa186f790 = new Bridge.global.___CSHTML5__Component__Defaultstyles__INTERNAL_Defaultdatepickerstyle__Xaml();
                ResourceDictionary_557ec46a58a049a9b602b17aa186f790.Source = Bridge.cast(Bridge.global.DotNetForHtml5.Core.TypeFromStringConverters.ConvertFromInvariantString(Bridge.global.System.Uri, "../DefaultStyles/INTERNAL_DefaultDatePickerStyle.xaml"), System.Uri);

                this.MergedDictionaries.add(ResourceDictionary_557ec46a58a049a9b602b17aa186f790);
                var ResourceDictionary_939f2fbdac844d4c91ced18c819b32e4 = new Bridge.global.___CSHTML5__Component__Defaultstyles__INTERNAL_Defaultexpanderstyle__Xaml();
                ResourceDictionary_939f2fbdac844d4c91ced18c819b32e4.Source = Bridge.cast(Bridge.global.DotNetForHtml5.Core.TypeFromStringConverters.ConvertFromInvariantString(Bridge.global.System.Uri, "../DefaultStyles/INTERNAL_DefaultExpanderStyle.xaml"), System.Uri);

                this.MergedDictionaries.add(ResourceDictionary_939f2fbdac844d4c91ced18c819b32e4);
                var ResourceDictionary_c6eed25f28074014998aeda1df3faa2f = new Bridge.global.___CSHTML5__Component__Defaultstyles__INTERNAL_Defaultgridsplitterstyle__Xaml();
                ResourceDictionary_c6eed25f28074014998aeda1df3faa2f.Source = Bridge.cast(Bridge.global.DotNetForHtml5.Core.TypeFromStringConverters.ConvertFromInvariantString(Bridge.global.System.Uri, "../DefaultStyles/INTERNAL_DefaultGridSplitterStyle.xaml"), System.Uri);

                this.MergedDictionaries.add(ResourceDictionary_c6eed25f28074014998aeda1df3faa2f);
                var ResourceDictionary_19951da7f6e84376b6d7a606668e8f9d = new Bridge.global.___CSHTML5__Component__Defaultstyles__INTERNAL_Defaulthyperlinkbuttonstyle__Xaml();
                ResourceDictionary_19951da7f6e84376b6d7a606668e8f9d.Source = Bridge.cast(Bridge.global.DotNetForHtml5.Core.TypeFromStringConverters.ConvertFromInvariantString(Bridge.global.System.Uri, "../DefaultStyles/INTERNAL_DefaultHyperlinkButtonStyle.xaml"), System.Uri);

                this.MergedDictionaries.add(ResourceDictionary_19951da7f6e84376b6d7a606668e8f9d);
                var ResourceDictionary_56e857d7048b463e8b111f455a573095 = new Bridge.global.___CSHTML5__Component__Defaultstyles__INTERNAL_Defaultlabelstyle__Xaml();
                ResourceDictionary_56e857d7048b463e8b111f455a573095.Source = Bridge.cast(Bridge.global.DotNetForHtml5.Core.TypeFromStringConverters.ConvertFromInvariantString(Bridge.global.System.Uri, "../DefaultStyles/INTERNAL_DefaultLabelStyle.xaml"), System.Uri);

                this.MergedDictionaries.add(ResourceDictionary_56e857d7048b463e8b111f455a573095);
                var ResourceDictionary_727f4ee787cd4006894d61df7c587bcc = new Bridge.global.___CSHTML5__Component__Defaultstyles__INTERNAL_Defaultmenuitemstyle__Xaml();
                ResourceDictionary_727f4ee787cd4006894d61df7c587bcc.Source = Bridge.cast(Bridge.global.DotNetForHtml5.Core.TypeFromStringConverters.ConvertFromInvariantString(Bridge.global.System.Uri, "../DefaultStyles/INTERNAL_DefaultMenuItemStyle.xaml"), System.Uri);

                this.MergedDictionaries.add(ResourceDictionary_727f4ee787cd4006894d61df7c587bcc);
                var ResourceDictionary_41056cc1d18843fb92a8005dd52764ce = new Bridge.global.___CSHTML5__Component__Defaultstyles__INTERNAL_Defaultnumericupdownstyle__Xaml();
                ResourceDictionary_41056cc1d18843fb92a8005dd52764ce.Source = Bridge.cast(Bridge.global.DotNetForHtml5.Core.TypeFromStringConverters.ConvertFromInvariantString(Bridge.global.System.Uri, "../DefaultStyles/INTERNAL_DefaultNumericUpDownStyle.xaml"), System.Uri);

                this.MergedDictionaries.add(ResourceDictionary_41056cc1d18843fb92a8005dd52764ce);
                var ResourceDictionary_2141870887e04ca7bea0af9d58530bb0 = new Bridge.global.___CSHTML5__Component__Defaultstyles__INTERNAL_Defaultsliderstyle__Xaml();
                ResourceDictionary_2141870887e04ca7bea0af9d58530bb0.Source = Bridge.cast(Bridge.global.DotNetForHtml5.Core.TypeFromStringConverters.ConvertFromInvariantString(Bridge.global.System.Uri, "../DefaultStyles/INTERNAL_DefaultSliderStyle.xaml"), System.Uri);

                this.MergedDictionaries.add(ResourceDictionary_2141870887e04ca7bea0af9d58530bb0);
                var ResourceDictionary_57d156d2673b418d8a486668e9193609 = new Bridge.global.___CSHTML5__Component__Defaultstyles__INTERNAL_Defaultscrollbarstyle__Xaml();
                ResourceDictionary_57d156d2673b418d8a486668e9193609.Source = Bridge.cast(Bridge.global.DotNetForHtml5.Core.TypeFromStringConverters.ConvertFromInvariantString(Bridge.global.System.Uri, "../DefaultStyles/INTERNAL_DefaultScrollBarStyle.xaml"), System.Uri);

                this.MergedDictionaries.add(ResourceDictionary_57d156d2673b418d8a486668e9193609);
                var ResourceDictionary_070cbeddf3ae410f98ab900f3a331605 = new Bridge.global.___CSHTML5__Component__Defaultstyles__INTERNAL_Defaulttabcontrolstyle__Xaml();
                ResourceDictionary_070cbeddf3ae410f98ab900f3a331605.Source = Bridge.cast(Bridge.global.DotNetForHtml5.Core.TypeFromStringConverters.ConvertFromInvariantString(Bridge.global.System.Uri, "../DefaultStyles/INTERNAL_DefaultTabControlStyle.xaml"), System.Uri);

                this.MergedDictionaries.add(ResourceDictionary_070cbeddf3ae410f98ab900f3a331605);
                var ResourceDictionary_96478d2ce5b54d48a75cd66043754a29 = new Bridge.global.___CSHTML5__Component__Defaultstyles__INTERNAL_Defaulttabitemstyle__Xaml();
                ResourceDictionary_96478d2ce5b54d48a75cd66043754a29.Source = Bridge.cast(Bridge.global.DotNetForHtml5.Core.TypeFromStringConverters.ConvertFromInvariantString(Bridge.global.System.Uri, "../DefaultStyles/INTERNAL_DefaultTabItemStyle.xaml"), System.Uri);

                this.MergedDictionaries.add(ResourceDictionary_96478d2ce5b54d48a75cd66043754a29);
                var ResourceDictionary_e072793fe2da41ffab4c2fade00fe1a1 = new Bridge.global.___CSHTML5__Component__Defaultstyles__INTERNAL_Defaultthumbstyle__Xaml();
                ResourceDictionary_e072793fe2da41ffab4c2fade00fe1a1.Source = Bridge.cast(Bridge.global.DotNetForHtml5.Core.TypeFromStringConverters.ConvertFromInvariantString(Bridge.global.System.Uri, "../DefaultStyles/INTERNAL_DefaultThumbStyle.xaml"), System.Uri);

                this.MergedDictionaries.add(ResourceDictionary_e072793fe2da41ffab4c2fade00fe1a1);
                var ResourceDictionary_70ab9a6bfb324564b77dbfb808f51d67 = new Bridge.global.___CSHTML5__Component__Defaultstyles__INTERNAL_Defaulttileviewitemstyle__Xaml();
                ResourceDictionary_70ab9a6bfb324564b77dbfb808f51d67.Source = Bridge.cast(Bridge.global.DotNetForHtml5.Core.TypeFromStringConverters.ConvertFromInvariantString(Bridge.global.System.Uri, "../DefaultStyles/INTERNAL_DefaultTileViewItemStyle.xaml"), System.Uri);

                this.MergedDictionaries.add(ResourceDictionary_70ab9a6bfb324564b77dbfb808f51d67);
                var ResourceDictionary_e2a7b0a08f4447d5898a9d27edf303b7 = new Bridge.global.___CSHTML5__Component__Defaultstyles__INTERNAL_Defaulttileviewstyle__Xaml();
                ResourceDictionary_e2a7b0a08f4447d5898a9d27edf303b7.Source = Bridge.cast(Bridge.global.DotNetForHtml5.Core.TypeFromStringConverters.ConvertFromInvariantString(Bridge.global.System.Uri, "../DefaultStyles/INTERNAL_DefaultTileViewStyle.xaml"), System.Uri);

                this.MergedDictionaries.add(ResourceDictionary_e2a7b0a08f4447d5898a9d27edf303b7);
                var ResourceDictionary_38f7b9066e8c4f47865caf19cccde1ed = new Bridge.global.___CSHTML5__Component__Defaultstyles__INTERNAL_Defaulttimepickerstyle__Xaml();
                ResourceDictionary_38f7b9066e8c4f47865caf19cccde1ed.Source = Bridge.cast(Bridge.global.DotNetForHtml5.Core.TypeFromStringConverters.ConvertFromInvariantString(Bridge.global.System.Uri, "../DefaultStyles/INTERNAL_DefaultTimePickerStyle.xaml"), System.Uri);

                this.MergedDictionaries.add(ResourceDictionary_38f7b9066e8c4f47865caf19cccde1ed);
                var ResourceDictionary_8ebc543dfc6a4e6bacaf54474f9ebfb8 = new Bridge.global.___CSHTML5__Component__Defaultstyles__INTERNAL_Defaulttogglebuttonstyle__Xaml();
                ResourceDictionary_8ebc543dfc6a4e6bacaf54474f9ebfb8.Source = Bridge.cast(Bridge.global.DotNetForHtml5.Core.TypeFromStringConverters.ConvertFromInvariantString(Bridge.global.System.Uri, "../DefaultStyles/INTERNAL_DefaultToggleButtonStyle.xaml"), System.Uri);

                this.MergedDictionaries.add(ResourceDictionary_8ebc543dfc6a4e6bacaf54474f9ebfb8);
                var ResourceDictionary_007574d4b35f42a1867f25f071930025 = new Bridge.global.___CSHTML5__Component__Defaultstyles__INTERNAL_Defaulttooltipstyle__Xaml();
                ResourceDictionary_007574d4b35f42a1867f25f071930025.Source = Bridge.cast(Bridge.global.DotNetForHtml5.Core.TypeFromStringConverters.ConvertFromInvariantString(Bridge.global.System.Uri, "../DefaultStyles/INTERNAL_DefaultToolTipStyle.xaml"), System.Uri);

                this.MergedDictionaries.add(ResourceDictionary_007574d4b35f42a1867f25f071930025);
                var ResourceDictionary_fb202f76a6584d2690567ae17bc09ca8 = new Bridge.global.___CSHTML5__Component__Defaultstyles__INTERNAL_Defaultviewboxstyle__Xaml();
                ResourceDictionary_fb202f76a6584d2690567ae17bc09ca8.Source = Bridge.cast(Bridge.global.DotNetForHtml5.Core.TypeFromStringConverters.ConvertFromInvariantString(Bridge.global.System.Uri, "../DefaultStyles/INTERNAL_DefaultViewboxStyle.xaml"), System.Uri);

                this.MergedDictionaries.add(ResourceDictionary_fb202f76a6584d2690567ae17bc09ca8);
                var ResourceDictionary_d3a3894d7ba54686a853d12c35f06e19 = new Bridge.global.___CSHTML5__Component__Defaultstyles__INTERNAL_Defaultrepeatbuttonstyle__Xaml();
                ResourceDictionary_d3a3894d7ba54686a853d12c35f06e19.Source = Bridge.cast(Bridge.global.DotNetForHtml5.Core.TypeFromStringConverters.ConvertFromInvariantString(Bridge.global.System.Uri, "../DefaultStyles/INTERNAL_DefaultRepeatButtonStyle.xaml"), System.Uri);

                this.MergedDictionaries.add(ResourceDictionary_d3a3894d7ba54686a853d12c35f06e19);
                var ResourceDictionary_3d095857de1c4559a9f4b3502707f27f = new Bridge.global.___CSHTML5__Component__Defaultstyles__INTERNAL_Defaultitemscontrolstyle__Xaml();
                ResourceDictionary_3d095857de1c4559a9f4b3502707f27f.Source = Bridge.cast(Bridge.global.DotNetForHtml5.Core.TypeFromStringConverters.ConvertFromInvariantString(Bridge.global.System.Uri, "../DefaultStyles/INTERNAL_DefaultItemsControlStyle.xaml"), System.Uri);

                this.MergedDictionaries.add(ResourceDictionary_3d095857de1c4559a9f4b3502707f27f);







            }
        }
    });

    /**
     * @memberof Windows.UI.Xaml
     * @callback Windows.UI.Xaml.DependencyPropertyChangedEventHandler
     * @param   {System.Object}                                         sender    
     * @param   {Windows.UI.Xaml.DependencyPropertyChangedEventArgs}    e
     * @return  {void}
     */

    /**
     * Provides a framework of common APIs for objects that participate in UI and
     programmatic layout. FrameworkElement also defines APIs related to data binding,
     object tree, and object lifetime feature areas.
     *
     * @abstract
     * @public
     * @class Windows.UI.Xaml.FrameworkElement
     * @augments Windows.UI.Xaml.UIElement
     */
    Bridge.define("Windows.UI.Xaml.FrameworkElement", {
        inherits: [Windows.UI.Xaml.UIElement],
        statics: {
            fields: {
                CursorProperty: null,
                /**
                 * Identifies the IsEnabled dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.FrameworkElement
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                IsEnabledProperty: null,
                /**
                 * Identifies the Name dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.FrameworkElement
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                NameProperty: null,
                /**
                 * Identifies the DataContext dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.FrameworkElement
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                DataContextProperty: null,
                /**
                 * Identifies the Tag dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.FrameworkElement
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                TagProperty: null,
                /**
                 * Identifies the Style dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.FrameworkElement
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                StyleProperty: null,
                /**
                 * Identifies the {@link } dependency
                 property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.FrameworkElement
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                DefaultStyleKeyProperty: null,
                /**
                 * Identifies the Height dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.FrameworkElement
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                HeightProperty: null,
                /**
                 * Identifies the Width dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.FrameworkElement
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                WidthProperty: null,
                /**
                 * Identifies the HorizontalAlignment dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.FrameworkElement
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                HorizontalAlignmentProperty: null,
                /**
                 * Identifies the VerticalAlignment dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.FrameworkElement
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                VerticalAlignmentProperty: null,
                /**
                 * Identifies the Margin dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.FrameworkElement
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                MarginProperty: null,
                /**
                 * Identifies the MinHeight dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.FrameworkElement
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                MinHeightProperty: null,
                /**
                 * Identifies the MinWidth dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.FrameworkElement
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                MinWidthProperty: null,
                /**
                 * Identifies the MaxHeight dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.FrameworkElement
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                MaxHeightProperty: null,
                /**
                 * Identifies the MaxWidth dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.FrameworkElement
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                MaxWidthProperty: null,
                /**
                 * Identifies the ContextMenu dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.FrameworkElement
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                ContextMenuProperty: null
            },
            ctors: {
                init: function () {
                    var $t;
                    this.CursorProperty = Windows.UI.Xaml.DependencyProperty.Register("Cursor", System.Windows.Input.Cursor, Windows.UI.Xaml.FrameworkElement, ($t = new Windows.UI.Xaml.PropertyMetadata.$ctor1(null), $t.MethodToUpdateDom = Windows.UI.Xaml.FrameworkElement.Cursor_MethodToUpdateDom, $t));
                    this.IsEnabledProperty = Windows.UI.Xaml.DependencyProperty.Register("IsEnabled", System.Boolean, Windows.UI.Xaml.FrameworkElement, ($t = new Windows.UI.Xaml.PropertyMetadata.$ctor3(Bridge.box(true, System.Boolean, System.Boolean.toString), Windows.UI.Xaml.FrameworkElement.IsEnabled_Changed, Windows.UI.Xaml.FrameworkElement.CoerceIsEnabledProperty), $t.MethodToUpdateDom = Windows.UI.Xaml.FrameworkElement.IsEnabled_MethodToUpdateDom, $t));
                    this.NameProperty = Windows.UI.Xaml.DependencyProperty.Register("Name", System.String, Windows.UI.Xaml.FrameworkElement, ($t = new Windows.UI.Xaml.PropertyMetadata.$ctor1(""), $t.MethodToUpdateDom = Windows.UI.Xaml.FrameworkElement.OnNameChanged_MethodToUpdateDom, $t));
                    this.DataContextProperty = Windows.UI.Xaml.DependencyProperty.Register("DataContext", System.Object, Windows.UI.Xaml.FrameworkElement, ($t = new Windows.UI.Xaml.PropertyMetadata.$ctor2(null, Windows.UI.Xaml.FrameworkElement.OnDataContextPropertyChanged), $t.Inherits = true, $t));
                    this.TagProperty = Windows.UI.Xaml.DependencyProperty.Register("Tag", System.Object, Windows.UI.Xaml.FrameworkElement, new Windows.UI.Xaml.PropertyMetadata.$ctor1(null));
                    this.StyleProperty = Windows.UI.Xaml.DependencyProperty.Register("Style", Windows.UI.Xaml.Style, Windows.UI.Xaml.FrameworkElement, new Windows.UI.Xaml.PropertyMetadata.$ctor2(null, Windows.UI.Xaml.FrameworkElement.OnStyleChanged));
                    this.DefaultStyleKeyProperty = Windows.UI.Xaml.DependencyProperty.Register("DefaultStyleKey", System.Object, Windows.UI.Xaml.FrameworkElement, new Windows.UI.Xaml.PropertyMetadata.$ctor2(null, Windows.UI.Xaml.FrameworkElement.OnThemeStyleKeyChanged));
                    this.HeightProperty = Windows.UI.Xaml.DependencyProperty.Register("Height", System.Double, Windows.UI.Xaml.FrameworkElement, ($t = new Windows.UI.Xaml.PropertyMetadata.$ctor1(Bridge.box(Number.NaN, System.Double, System.Double.format, System.Double.getHashCode)), $t.GetCSSEquivalent = function (instance) {
                        var $t1;
                        return ($t1 = new Windows.UI.Xaml.CSSEquivalent(), $t1.Value = function (inst, value) {
                            if (Bridge.is(value, System.Double)) {
                                return !isNaN(System.Nullable.getValue(Bridge.cast(Bridge.unbox(value, System.Double), System.Double))) ? (Bridge.toString(value) || "") + "px" : "auto";
                            }
                            throw new System.InvalidOperationException.$ctor1("Error when trying to set FrameworkElement.Height: expected double, got " + (Bridge.Reflection.getTypeFullName(Bridge.getType(value)) || ""));
                        }, $t1.CallbackMethod = Windows.UI.Xaml.FrameworkElement.Height_Changed, $t1.UIElement = Bridge.cast(instance, Windows.UI.Xaml.UIElement), $t1.Name = function (_o1) {
                                _o1.add("height");
                                return _o1;
                            }(new (System.Collections.Generic.List$1(System.String)).ctor()), $t1.ApplyAlsoWhenThereIsAControlTemplate = true, $t1);
                    }, $t));
                    this.WidthProperty = Windows.UI.Xaml.DependencyProperty.Register("Width", System.Double, Windows.UI.Xaml.FrameworkElement, ($t = new Windows.UI.Xaml.PropertyMetadata.$ctor1(Bridge.box(Number.NaN, System.Double, System.Double.format, System.Double.getHashCode)), $t.GetCSSEquivalent = function (instance) {
                        var $t1;
                        return ($t1 = new Windows.UI.Xaml.CSSEquivalent(), $t1.Value = function (inst, value) {
                            if (Bridge.is(value, System.Double)) {
                                return !isNaN(System.Nullable.getValue(Bridge.cast(Bridge.unbox(value, System.Double), System.Double))) ? (Bridge.toString(value) || "") + "px" : "auto";
                            }
                            throw new System.InvalidOperationException.$ctor1("Error when trying to set FrameworkElement.Width: expected double, got " + (Bridge.Reflection.getTypeFullName(Bridge.getType(value)) || ""));
                        }, $t1.CallbackMethod = Windows.UI.Xaml.FrameworkElement.Width_Changed, $t1.UIElement = Bridge.cast(instance, Windows.UI.Xaml.UIElement), $t1.Name = function (_o2) {
                                _o2.add("width");
                                return _o2;
                            }(new (System.Collections.Generic.List$1(System.String)).ctor()), $t1.ApplyAlsoWhenThereIsAControlTemplate = true, $t1);
                    }, $t));
                    this.HorizontalAlignmentProperty = Windows.UI.Xaml.DependencyProperty.Register("HorizontalAlignment", Windows.UI.Xaml.HorizontalAlignment, Windows.UI.Xaml.FrameworkElement, ($t = new Windows.UI.Xaml.PropertyMetadata.$ctor2(Bridge.box(Windows.UI.Xaml.HorizontalAlignment.Stretch, Windows.UI.Xaml.HorizontalAlignment, System.Enum.toStringFn(Windows.UI.Xaml.HorizontalAlignment)), Windows.UI.Xaml.FrameworkElement.HorizontalAlignment_Changed), $t.CallPropertyChangedWhenLoadedIntoVisualTree = Windows.UI.Xaml.WhenToCallPropertyChangedEnum.IfPropertyIsSet, $t));
                    this.VerticalAlignmentProperty = Windows.UI.Xaml.DependencyProperty.Register("VerticalAlignment", Windows.UI.Xaml.VerticalAlignment, Windows.UI.Xaml.FrameworkElement, ($t = new Windows.UI.Xaml.PropertyMetadata.$ctor2(Bridge.box(Windows.UI.Xaml.VerticalAlignment.Stretch, Windows.UI.Xaml.VerticalAlignment, System.Enum.toStringFn(Windows.UI.Xaml.VerticalAlignment)), Windows.UI.Xaml.FrameworkElement.VerticalAlignment_Changed), $t.CallPropertyChangedWhenLoadedIntoVisualTree = Windows.UI.Xaml.WhenToCallPropertyChangedEnum.IfPropertyIsSet, $t));
                    this.MarginProperty = Windows.UI.Xaml.DependencyProperty.Register("Margin", Windows.UI.Xaml.Thickness, Windows.UI.Xaml.FrameworkElement, ($t = new Windows.UI.Xaml.PropertyMetadata.$ctor1(new Windows.UI.Xaml.Thickness.ctor().$clone()), $t.MethodToUpdateDom = Windows.UI.Xaml.FrameworkElement.Margin_MethodToUpdateDom, $t));
                    this.MinHeightProperty = Windows.UI.Xaml.DependencyProperty.Register("MinHeight", System.Double, Windows.UI.Xaml.FrameworkElement, ($t = new Windows.UI.Xaml.PropertyMetadata.$ctor2(Bridge.box(0.0, System.Double, System.Double.format, System.Double.getHashCode), Windows.UI.Xaml.FrameworkElement.MinHeight_Changed), $t.CallPropertyChangedWhenLoadedIntoVisualTree = Windows.UI.Xaml.WhenToCallPropertyChangedEnum.IfPropertyIsSet, $t));
                    this.MinWidthProperty = Windows.UI.Xaml.DependencyProperty.Register("MinWidth", System.Double, Windows.UI.Xaml.FrameworkElement, ($t = new Windows.UI.Xaml.PropertyMetadata.$ctor2(Bridge.box(0.0, System.Double, System.Double.format, System.Double.getHashCode), Windows.UI.Xaml.FrameworkElement.MinWidth_Changed), $t.CallPropertyChangedWhenLoadedIntoVisualTree = Windows.UI.Xaml.WhenToCallPropertyChangedEnum.IfPropertyIsSet, $t));
                    this.MaxHeightProperty = Windows.UI.Xaml.DependencyProperty.Register("MaxHeight", System.Double, Windows.UI.Xaml.FrameworkElement, ($t = new Windows.UI.Xaml.PropertyMetadata.$ctor2(Bridge.box(Number.POSITIVE_INFINITY, System.Double, System.Double.format, System.Double.getHashCode), Windows.UI.Xaml.FrameworkElement.MaxHeight_Changed), $t.CallPropertyChangedWhenLoadedIntoVisualTree = Windows.UI.Xaml.WhenToCallPropertyChangedEnum.IfPropertyIsSet, $t));
                    this.MaxWidthProperty = Windows.UI.Xaml.DependencyProperty.Register("MaxWidth", System.Double, Windows.UI.Xaml.FrameworkElement, ($t = new Windows.UI.Xaml.PropertyMetadata.$ctor2(Bridge.box(Number.POSITIVE_INFINITY, System.Double, System.Double.format, System.Double.getHashCode), Windows.UI.Xaml.FrameworkElement.MaxWidth_Changed), $t.CallPropertyChangedWhenLoadedIntoVisualTree = Windows.UI.Xaml.WhenToCallPropertyChangedEnum.IfPropertyIsSet, $t));
                    this.ContextMenuProperty = Windows.UI.Xaml.DependencyProperty.Register("ContextMenu", Windows.UI.Xaml.Controls.ContextMenu, Windows.UI.Xaml.FrameworkElement, ($t = new Windows.UI.Xaml.PropertyMetadata.$ctor2(null, Windows.UI.Xaml.FrameworkElement.ContextMenu_Changed), $t.CallPropertyChangedWhenLoadedIntoVisualTree = Windows.UI.Xaml.WhenToCallPropertyChangedEnum.IfPropertyIsSet, $t));
                }
            },
            methods: {
                Cursor_MethodToUpdateDom: function (d, newValue) {
                    var frameworkElement = Bridge.cast(d, Windows.UI.Xaml.FrameworkElement);
                    var newCursor = Bridge.cast(newValue, System.Windows.Input.Cursor);
                    var outerDomElement = frameworkElement.INTERNAL_OuterDomElement;
                    var styleOfOuterDomElement = CSHTML5.Internal.INTERNAL_HtmlDomManager.GetDomElementStyleForModification(outerDomElement);
                    if (newCursor == null) {
                        styleOfOuterDomElement.cursor = "inherit";
                    } else {
                        styleOfOuterDomElement.cursor = newCursor._cursorHtmlString;
                    }
                },
                IsEnabled_Changed: function (d, e) {
                    var frameworkElement = Bridge.cast(d, Windows.UI.Xaml.FrameworkElement);
                    if (!Bridge.staticEquals(frameworkElement.IsEnabledChanged, null)) {
                        frameworkElement.IsEnabledChanged(frameworkElement, e);
                    }
                    Windows.UI.Xaml.UIElement.InvalidateForceInheritPropertyOnChildren(frameworkElement, e.Property);
                },
                CoerceIsEnabledProperty: function (d, baseValue) {
                    var $this = Bridge.cast(d, Windows.UI.Xaml.FrameworkElement);

                    if (!(Bridge.is(baseValue, System.Boolean))) {
                        return Bridge.box(true, System.Boolean, System.Boolean.toString);
                    }

                    if (System.Nullable.getValue(Bridge.cast(Bridge.unbox(baseValue, System.Boolean), System.Boolean))) {
                        var parent = $this.Parent;
                        if (parent == null || System.Nullable.getValue(Bridge.cast(Bridge.unbox(parent.GetValue(Windows.UI.Xaml.FrameworkElement.IsEnabledProperty), System.Boolean), System.Boolean))) {
                            return Bridge.box(true, System.Boolean, System.Boolean.toString);
                        } else {
                            return Bridge.box(false, System.Boolean, System.Boolean.toString);
                        }
                    } else {
                        return Bridge.box(false, System.Boolean, System.Boolean.toString);
                    }
                },
                IsEnabled_MethodToUpdateDom: function (d, newValue) {
                    var element = Bridge.cast(d, Windows.UI.Xaml.FrameworkElement);
                    Windows.UI.Xaml.UIElement.INTERNAL_UpdateCssPointerEventsPropertyBasedOnIsHitTestVisibleAndIsEnabled(element, element.IsHitTestVisible, System.Nullable.getValue(Bridge.cast(Bridge.unbox(newValue, System.Boolean), System.Boolean)));
                    element.ManageIsEnabled(newValue != null ? System.Nullable.getValue(Bridge.cast(Bridge.unbox(newValue, System.Boolean), System.Boolean)) : true);
                },
                OnNameChanged_MethodToUpdateDom: function (d, value) {
                    var $t;
                    var $this = Bridge.cast(d, Windows.UI.Xaml.FrameworkElement);
                    CSHTML5.Internal.INTERNAL_HtmlDomManager.SetDomElementAttribute($this.INTERNAL_OuterDomElement, "dataId", Bridge.toString((($t = value, $t != null ? $t : ""))));
                },
                OnDataContextPropertyChanged: function (d, e) {
                    Bridge.cast(d, Windows.UI.Xaml.FrameworkElement).RaiseDataContextChangedEvent(e);
                },
                OnStyleChanged: function (d, e) {
                    var fe = Bridge.cast(d, Windows.UI.Xaml.FrameworkElement);
                    fe.HasLocalStyle = !Bridge.referenceEquals(fe.ReadLocalValueInternal(Windows.UI.Xaml.FrameworkElement.StyleProperty), Windows.UI.Xaml.DependencyProperty.UnsetValue);
                    Windows.UI.Xaml.FrameworkElement.UpdateStyleCache(fe, Bridge.cast(e.OldValue, Windows.UI.Xaml.Style), Bridge.cast(e.NewValue, Windows.UI.Xaml.Style), Bridge.ref(fe, "_styleCache"));
                },
                UpdateStyleCache: function (fe, oldStyle, newStyle, styleCache) {
                    var $t, $t1;
                    var newStylePropertyValues = null;
                    if (newStyle != null) {

                        newStyle.CheckTargetType(fe);
                        newStyle.Seal();
                        newStylePropertyValues = newStyle.EffectiveValues;
                    }

                    styleCache.v = newStyle;

                    if (oldStyle != null) {
                        var oldStylePropertyValues = oldStyle.EffectiveValues;
                        var removedProperties = System.Linq.Enumerable.from(System.Linq.Enumerable.from(oldStylePropertyValues).where(function (kp) {
                                    return newStylePropertyValues == null || !newStylePropertyValues.containsKey(kp.key);
                                })).select(function (kp) {
                                return kp.key;
                            });
                        $t = Bridge.getEnumerator(removedProperties, Windows.UI.Xaml.DependencyProperty);
                        try {
                            while ($t.moveNext()) {
                                var property = $t.Current;
                                fe.SetLocalStyleValue(property, Windows.UI.Xaml.DependencyProperty.UnsetValue);
                            }
                        } finally {
                            if (Bridge.is($t, System.IDisposable)) {
                                $t.System$IDisposable$Dispose();
                            }
                        }
                    }

                    if (newStyle != null) {
                        $t1 = Bridge.getEnumerator(newStylePropertyValues);
                        try {
                            while ($t1.moveNext()) {
                                var propertyValue = $t1.Current;
                                var value;
                                var expr = Bridge.as(propertyValue.value, Windows.UI.Xaml.Data.BindingExpression);
                                if (expr != null) {
                                    value = new Windows.UI.Xaml.Data.BindingExpression.$ctor1(expr.ParentBinding.Clone(), propertyValue.key);
                                } else {
                                    value = propertyValue.value;
                                }
                                fe.SetLocalStyleValue(propertyValue.key, value);
                            }
                        } finally {
                            if (Bridge.is($t1, System.IDisposable)) {
                                $t1.System$IDisposable$Dispose();
                            }
                        }
                    }
                },
                FindImplicitStyleResource: function (fe, resourceKey) {
                    if (fe.ShouldLookupImplicitStyles) {
                        var implicitStyle;
                        for (var f = fe; f != null; f = Bridge.cast(f.Parent, Windows.UI.Xaml.FrameworkElement)) {
                            if (f.HasResources && f.Resources.HasImplicitStyles) {
                                implicitStyle = f.Resources.getItem(resourceKey);
                                if (implicitStyle != null) {
                                    return implicitStyle;
                                }
                            }
                        }
                        var app = Windows.UI.Xaml.Application.Current;
                        if (app != null) {
                            implicitStyle = app.Resources.getItem(resourceKey);
                            if (implicitStyle != null) {
                                return implicitStyle;
                            }
                        }
                    }
                    return Windows.UI.Xaml.DependencyProperty.UnsetValue;
                },
                OnThemeStyleKeyChanged: function (d, e) {
                    Bridge.cast(d, Windows.UI.Xaml.FrameworkElement).UpdateThemeStyleProperty();
                },
                GetThemeStyle: function (fe) {
                    fe.HasThemeStyleEverBeenFetched = true;

                    var themeStyleKey = fe.DefaultStyleKey;
                    var oldThemeStyle = fe.ThemeStyle;
                    var newThemeStyle = null;

                    if (themeStyleKey != null) {
                        var styleLookup;

                        var typeKey = Bridge.as(themeStyleKey, System.Type);
                        if (typeKey != null) {
                            styleLookup = Windows.UI.Xaml.Application.Current.XamlResourcesHandler.TryFindResourceInGenericXaml(Bridge.Reflection.getTypeAssembly(typeKey), themeStyleKey);
                        } else {
                            styleLookup = null;
                        }

                        if (styleLookup != null) {
                            if (Bridge.is(styleLookup, Windows.UI.Xaml.Style)) {
                                newThemeStyle = Bridge.cast(styleLookup, Windows.UI.Xaml.Style);
                            } else {
                                throw new System.InvalidOperationException.$ctor1(System.String.format("System resource for type '{0}' is not a Style object.", [themeStyleKey]));
                            }
                        }

                        if (newThemeStyle == null) {
                            var themeStyleTypeKey = Bridge.as(themeStyleKey, System.Type);
                            if (themeStyleTypeKey != null) {
                                var styleMetadata = Windows.UI.Xaml.FrameworkElement.StyleProperty.GetMetadata(themeStyleTypeKey);

                                if (styleMetadata != null) {
                                    newThemeStyle = Bridge.as(styleMetadata.DefaultValue, Windows.UI.Xaml.Style);
                                }
                            }
                        }
                    }

                    if (!Bridge.referenceEquals(oldThemeStyle, newThemeStyle)) {
                        Windows.UI.Xaml.FrameworkElement.OnThemeStyleChanged(fe, oldThemeStyle, newThemeStyle);
                    }

                    return newThemeStyle;
                },
                OnThemeStyleChanged: function (d, oldValue, newValue) {
                    var fe = Bridge.cast(d, Windows.UI.Xaml.FrameworkElement);
                    Windows.UI.Xaml.FrameworkElement.UpdateThemeStyleCache(fe, Bridge.cast(oldValue, Windows.UI.Xaml.Style), Bridge.cast(newValue, Windows.UI.Xaml.Style), Bridge.ref(fe, "_themeStyleCache"));
                },
                UpdateThemeStyleCache: function (fe, oldThemeStyle, newThemeStyle, themeStyleCache) {
                    var $t, $t1;
                    var newStylePropertyValues = null;
                    if (newThemeStyle != null) {

                        newThemeStyle.CheckTargetType(fe);
                        newThemeStyle.Seal();
                        newStylePropertyValues = newThemeStyle.EffectiveValues;
                    }

                    themeStyleCache.v = newThemeStyle;

                    if (oldThemeStyle != null) {
                        var oldStylePropertyValues = oldThemeStyle.EffectiveValues;
                        var removedProperties = System.Linq.Enumerable.from(System.Linq.Enumerable.from(oldStylePropertyValues).where(function (kp) {
                                    return newStylePropertyValues == null || !newStylePropertyValues.containsKey(kp.key);
                                })).select(function (kp) {
                                return kp.key;
                            });
                        $t = Bridge.getEnumerator(removedProperties, Windows.UI.Xaml.DependencyProperty);
                        try {
                            while ($t.moveNext()) {
                                var property = $t.Current;
                                fe.SetThemeStyleValue(property, Windows.UI.Xaml.DependencyProperty.UnsetValue);
                            }
                        } finally {
                            if (Bridge.is($t, System.IDisposable)) {
                                $t.System$IDisposable$Dispose();
                            }
                        }
                    }

                    if (newThemeStyle != null) {
                        $t1 = Bridge.getEnumerator(newStylePropertyValues);
                        try {
                            while ($t1.moveNext()) {
                                var propertyValue = $t1.Current;
                                var value;
                                var expr = Bridge.as(propertyValue.value, Windows.UI.Xaml.Data.BindingExpression);
                                if (expr != null) {
                                    value = new Windows.UI.Xaml.Data.BindingExpression.$ctor1(expr.ParentBinding.Clone(), propertyValue.key);
                                } else {
                                    value = propertyValue.value;
                                }
                                fe.SetThemeStyleValue(propertyValue.key, value);
                            }
                        } finally {
                            if (Bridge.is($t1, System.IDisposable)) {
                                $t1.System$IDisposable$Dispose();
                            }
                        }
                    }
                },
                INTERNAL_InitializeOuterDomElementWidthAndHeight: function (element, outerDomElement) {

                    var style = CSHTML5.Internal.INTERNAL_HtmlDomManager.GetDomElementStyleForModification(outerDomElement);

                    if (!isNaN(element.Height)) {
                        style.height = System.Double.format(element.Height) + "px";
                    } else {
                        if (element.VerticalAlignment === Windows.UI.Xaml.VerticalAlignment.Stretch && !(Bridge.is(element.INTERNAL_VisualParent, Windows.UI.Xaml.Controls.Canvas)) && !(Bridge.is(element, Windows.UI.Xaml.Controls.CheckBox))) {
                            style.height = "100%";
                        } else {
                            style.height = "auto";
                        }
                    }

                    if (!isNaN(element.Width)) {
                        style.width = System.Double.format(element.Width) + "px";
                    } else {
                        if (element.HorizontalAlignment === Windows.UI.Xaml.HorizontalAlignment.Stretch && !(Bridge.is(element.INTERNAL_VisualParent, Windows.UI.Xaml.Controls.Canvas)) && !(Bridge.is(element, Windows.UI.Xaml.Controls.CheckBox))) {
                            style.width = "100%";
                        } else {
                            style.width = "auto";
                        }
                    }

                },
                Height_Changed: function (d, e) {
                    var frameworkElement = Bridge.cast(d, Windows.UI.Xaml.FrameworkElement);

                    Windows.UI.Xaml.FrameworkElement.INTERNAL_ApplyVerticalAlignmentAndHeight(frameworkElement, frameworkElement.VerticalAlignment);
                },
                Width_Changed: function (d, e) {
                    var frameworkElement = Bridge.cast(d, Windows.UI.Xaml.FrameworkElement);

                    Windows.UI.Xaml.FrameworkElement.INTERNAL_ApplyHorizontalAlignmentAndWidth(frameworkElement, frameworkElement.HorizontalAlignment);
                },
                HorizontalAlignment_Changed: function (d, e) {
                    var frameworkElement = Bridge.cast(d, Windows.UI.Xaml.FrameworkElement);
                    var newHorizontalAlignment = System.Nullable.getValue(Bridge.cast(Bridge.unbox(e.NewValue, Windows.UI.Xaml.HorizontalAlignment), System.Int32));
                    Windows.UI.Xaml.FrameworkElement.INTERNAL_ApplyHorizontalAlignmentAndWidth(frameworkElement, newHorizontalAlignment);
                },
                INTERNAL_ApplyHorizontalAlignmentAndWidth: function (frameworkElement, newHorizontalAlignment) {
                    var $t;

                    if (CSHTML5.Internal.INTERNAL_VisualTreeManager.IsElementInVisualTree(frameworkElement) && frameworkElement.Visibility !== Windows.UI.Xaml.Visibility.Collapsed) {
                        var styleOfOuterDomElement = CSHTML5.Internal.INTERNAL_HtmlDomManager.GetFrameworkElementOuterStyleForModification(frameworkElement);

                        if (Bridge.is(frameworkElement.INTERNAL_VisualParent, Windows.UI.Xaml.Controls.Canvas)) {
                            styleOfOuterDomElement.width = !isNaN(frameworkElement.Width) ? (System.Double.format(frameworkElement.Width) || "") + "px" : "auto";
                        } else {
                            var isParentAHorizontalStackPanel = Bridge.is(frameworkElement.INTERNAL_VisualParent, Windows.UI.Xaml.Controls.StackPanel) && Bridge.cast(frameworkElement.INTERNAL_VisualParent, Windows.UI.Xaml.Controls.StackPanel).Orientation === Windows.UI.Xaml.Controls.Orientation.Horizontal;
                            var isParentAWrapPanel = Bridge.is(frameworkElement.INTERNAL_VisualParent, Windows.UI.Xaml.Controls.WrapPanel);
                            var isParentAWrapPanelOrAHorizontalStackPanel = isParentAHorizontalStackPanel || isParentAWrapPanel;
                            var margin = frameworkElement.Margin.$clone();
                            var containsNegativeMargins = (margin.Left < 0.0 || margin.Top < 0.0 || margin.Right < 0.0 || margin.Bottom < 0.0);



                            var childOfOuterDomElement = frameworkElement.INTERNAL_OuterDomElement.firstChild;
                            var styleOfChildOfOuterDomElement = CSHTML5.Internal.INTERNAL_HtmlDomManager.IsNotUndefinedOrNull(childOfOuterDomElement) ? CSHTML5.Internal.INTERNAL_HtmlDomManager.GetDomElementStyleForModification(childOfOuterDomElement) : null;
                            var wrapperElement = ($t = frameworkElement.INTERNAL_InnerDivOfTheChildWrapperOfTheParentIfAny, $t != null ? $t : frameworkElement.INTERNAL_AdditionalOutsideDivForMargins);
                            var styleOfWrapperElement = CSHTML5.Internal.INTERNAL_HtmlDomManager.GetDomElementStyleForModification(wrapperElement);


                            if (Bridge.is(frameworkElement, Windows.UI.Xaml.Controls.ChildWindow)) {
                                styleOfWrapperElement.width = "100%";
                                styleOfOuterDomElement.marginLeft = "0px";
                                styleOfOuterDomElement.marginRight = "0px";
                                styleOfOuterDomElement.display = "block";
                                styleOfOuterDomElement.width = "100%";
                                return;
                            }

                            if (newHorizontalAlignment === Windows.UI.Xaml.HorizontalAlignment.Stretch && !isNaN(frameworkElement.Width)) {
                                newHorizontalAlignment = Windows.UI.Xaml.HorizontalAlignment.Center;
                            }

                            if (Bridge.is(frameworkElement, Windows.UI.Xaml.Controls.Image) && isNaN(frameworkElement.Width) && isNaN(frameworkElement.Height) && Bridge.cast(frameworkElement, Windows.UI.Xaml.Controls.Image).Stretch !== Windows.UI.Xaml.Media.Stretch.None) {
                                newHorizontalAlignment = Windows.UI.Xaml.HorizontalAlignment.Stretch;
                            }


                            var isCSSGrid = document.isGridSupported;

                            switch (newHorizontalAlignment) {
                                case Windows.UI.Xaml.HorizontalAlignment.Left: 
                                    if (!isParentAWrapPanelOrAHorizontalStackPanel) {
                                        if (!containsNegativeMargins) {
                                            styleOfWrapperElement.width = "100%";
                                        }
                                        styleOfOuterDomElement.marginLeft = "0px";
                                        styleOfOuterDomElement.marginRight = "auto";
                                        if (!(Bridge.is(frameworkElement, Windows.UI.Xaml.Controls.ScrollViewer)) && !(Bridge.is(frameworkElement, Windows.UI.Xaml.Controls.WrapPanel))) {

                                            if (!isCSSGrid || !(Bridge.is(frameworkElement, Windows.UI.Xaml.Controls.Grid))) {
                                                styleOfOuterDomElement.display = "table";
                                                if (CSHTML5.Internal.INTERNAL_HtmlDomManager.IsNotUndefinedOrNull(styleOfChildOfOuterDomElement)) {
                                                    if (!Bridge.referenceEquals(styleOfChildOfOuterDomElement.display, "table")) {

if (                                                        childOfOuterDomElement.tagName.toLowerCase() != 'span')
{
    childOfOuterDomElement.style.display = 'table-cell';
}
;
                                                    }
                                                }
                                            }
                                        }
                                    }
                                    styleOfOuterDomElement.width = !isNaN(frameworkElement.Width) ? (System.Double.format(frameworkElement.Width) || "") + "px" : "auto";
                                    break;
                                case Windows.UI.Xaml.HorizontalAlignment.Center: 
                                    if (!isParentAWrapPanelOrAHorizontalStackPanel) {
                                        styleOfWrapperElement.width = "auto";
                                        styleOfOuterDomElement.marginLeft = "auto";
                                        styleOfOuterDomElement.marginRight = "auto";
                                        if (!(Bridge.is(frameworkElement, Windows.UI.Xaml.Controls.ScrollViewer)) && !(Bridge.is(frameworkElement, Windows.UI.Xaml.Controls.WrapPanel))) {
                                            if (!isCSSGrid || !(Bridge.is(frameworkElement, Windows.UI.Xaml.Controls.Grid))) {
                                                styleOfOuterDomElement.display = "table";
                                                if (CSHTML5.Internal.INTERNAL_HtmlDomManager.IsNotUndefinedOrNull(styleOfChildOfOuterDomElement)) {
                                                    if (!Bridge.referenceEquals(styleOfChildOfOuterDomElement.display, "table")) {

if (                                                        childOfOuterDomElement.tagName.toLowerCase() != 'span')
{
    childOfOuterDomElement.style.display = 'table-cell';
}
;
                                                    }
                                                }
                                            }
                                        }
                                    }
                                    styleOfOuterDomElement.width = !isNaN(frameworkElement.Width) ? (System.Double.format(frameworkElement.Width) || "") + "px" : "auto";
                                    break;
                                case Windows.UI.Xaml.HorizontalAlignment.Right: 
                                    if (!isParentAWrapPanelOrAHorizontalStackPanel) {
                                        styleOfWrapperElement.width = "auto";
                                        styleOfOuterDomElement.marginLeft = "auto";
                                        styleOfOuterDomElement.marginRight = "0px";
                                        if (!(Bridge.is(frameworkElement, Windows.UI.Xaml.Controls.ScrollViewer)) && !(Bridge.is(frameworkElement, Windows.UI.Xaml.Controls.WrapPanel))) {
                                            if (!isCSSGrid || !(Bridge.is(frameworkElement, Windows.UI.Xaml.Controls.Grid))) {
                                                styleOfOuterDomElement.display = "table";
                                                if (CSHTML5.Internal.INTERNAL_HtmlDomManager.IsNotUndefinedOrNull(styleOfChildOfOuterDomElement)) {
                                                    if (!Bridge.referenceEquals(styleOfChildOfOuterDomElement.display, "table")) {

if (                                                        childOfOuterDomElement.tagName.toLowerCase() != 'span')
{
    childOfOuterDomElement.style.display = 'table-cell';
}
;
                                                    }
                                                }
                                            }
                                        }
                                    }
                                    styleOfOuterDomElement.width = !isNaN(frameworkElement.Width) ? (System.Double.format(frameworkElement.Width) || "") + "px" : "auto";
                                    break;
                                case Windows.UI.Xaml.HorizontalAlignment.Stretch: 
                                    if (!isParentAWrapPanelOrAHorizontalStackPanel) {
                                        if (!containsNegativeMargins) {
                                            styleOfWrapperElement.width = "100%";
                                        }
                                        styleOfOuterDomElement.marginLeft = "0px";
                                        styleOfOuterDomElement.marginRight = "0px";
                                        if (Bridge.is(frameworkElement, Windows.UI.Xaml.Controls.StackPanel) && Bridge.cast(frameworkElement, Windows.UI.Xaml.Controls.StackPanel).Orientation === Windows.UI.Xaml.Controls.Orientation.Horizontal) {
                                            styleOfOuterDomElement.display = "grid";
                                        } else {
                                            styleOfOuterDomElement.display = "block";
                                        }
                                        styleOfOuterDomElement.width = "100%";
                                    }
                                    break;
                                default: 
                                    break;
                            }
                        }


                        /* 

                        // COMMENTED ON 2016.09.02 because it prevents properly displaying child elements with NEGATIVE MARGINS. To reproduce: put a border with negative margins inside another border.

                        if (!(frameworkElement is ScrollViewer)) //Note: The ScrollViewer handles the "overflow" property by itself.
                        {
                           // We always display the portions of the child exceeding the edges, unless the element has a fixed size in pixels AND it is not a canvas:
                           if (!double.IsNaN(frameworkElement.Width) && !(frameworkElement is Canvas))
                               styleOfOuterDomElement.overflowX = "hidden"; //Note: This value means to crop the portion of the child exceeding the edges.
                           else
                               styleOfOuterDomElement.overflowX = ""; //Note: the default value is "visible"
                        }
                        */


                        frameworkElement.OnAfterApplyHorizontalAlignmentAndWidth();

                    }
                },
                VerticalAlignment_Changed: function (d, e) {
                    var frameworkElement = Bridge.cast(d, Windows.UI.Xaml.FrameworkElement);
                    var newVerticalAlignment = System.Nullable.getValue(Bridge.cast(Bridge.unbox(e.NewValue, Windows.UI.Xaml.VerticalAlignment), System.Int32));
                    Windows.UI.Xaml.FrameworkElement.INTERNAL_ApplyVerticalAlignmentAndHeight(frameworkElement, newVerticalAlignment);
                },
                INTERNAL_ApplyVerticalAlignmentAndHeight: function (frameworkElement, newVerticalAlignment) {
                    var $t;

                    if (CSHTML5.Internal.INTERNAL_VisualTreeManager.IsElementInVisualTree(frameworkElement) && frameworkElement.Visibility !== Windows.UI.Xaml.Visibility.Collapsed) {
                        var styleOfOuterDomElement = CSHTML5.Internal.INTERNAL_HtmlDomManager.GetFrameworkElementOuterStyleForModification(frameworkElement);

                        if (Bridge.is(frameworkElement.INTERNAL_VisualParent, Windows.UI.Xaml.Controls.Canvas)) {
                            styleOfOuterDomElement.height = !isNaN(frameworkElement.Height) ? (System.Double.format(frameworkElement.Height) || "") + "px" : "auto";
                        } else {
                            var isParentAWrapPanelOrAVerticalStackPanel = Bridge.is(frameworkElement.INTERNAL_VisualParent, Windows.UI.Xaml.Controls.WrapPanel) || (Bridge.is(frameworkElement.INTERNAL_VisualParent, Windows.UI.Xaml.Controls.StackPanel) && Bridge.cast(frameworkElement.INTERNAL_VisualParent, Windows.UI.Xaml.Controls.StackPanel).Orientation === Windows.UI.Xaml.Controls.Orientation.Vertical);



                            var wrapperElement = ($t = frameworkElement.INTERNAL_InnerDivOfTheChildWrapperOfTheParentIfAny, $t != null ? $t : frameworkElement.INTERNAL_AdditionalOutsideDivForMargins);
                            var styleOfWrapperElement = CSHTML5.Internal.INTERNAL_HtmlDomManager.GetDomElementStyleForModification(wrapperElement);
                            var parentOfTheWrapperElement = wrapperElement.parentNode;
                            var styleOfParentOfTheWrapperElement = CSHTML5.Internal.INTERNAL_HtmlDomManager.GetDomElementStyleForModification(parentOfTheWrapperElement);


                            if (Bridge.is(frameworkElement, Windows.UI.Xaml.Controls.ChildWindow)) {
                                styleOfWrapperElement.verticalAlign = "middle";
                                styleOfOuterDomElement.height = "100%";
                                return;
                            }


                            if (newVerticalAlignment === Windows.UI.Xaml.VerticalAlignment.Stretch && !isNaN(frameworkElement.Height)) {
                                newVerticalAlignment = Windows.UI.Xaml.VerticalAlignment.Center;
                            }

                            if (Bridge.is(frameworkElement, Windows.UI.Xaml.Controls.Image) && isNaN(frameworkElement.Width) && isNaN(frameworkElement.Height) && Bridge.cast(frameworkElement, Windows.UI.Xaml.Controls.Image).Stretch !== Windows.UI.Xaml.Media.Stretch.None) {
                                newVerticalAlignment = Windows.UI.Xaml.VerticalAlignment.Stretch;
                            }


                            var isCSSGrid = document.isGridSupported;
                            var isMsGrid = document.isMSGrid;
                            switch (newVerticalAlignment) {
                                case Windows.UI.Xaml.VerticalAlignment.Top: 
                                    if (!isParentAWrapPanelOrAVerticalStackPanel) {
                                        if (isCSSGrid) {
                                            if ((Bridge.is(frameworkElement.Parent, Windows.UI.Xaml.Controls.Grid))) {
                                                var boxSizingStyle = CSHTML5.Internal.INTERNAL_HtmlDomManager.GetFrameworkElementBoxSizingStyleForModification(frameworkElement);
                                                if (!isMsGrid) {
                                                    boxSizingStyle.marginTop = "0px";
                                                    boxSizingStyle.marginBottom = "auto";
                                                } else {
                                                    boxSizingStyle.msGridRowAlign = "start";
                                                }
                                            }
                                        }
                                        styleOfWrapperElement.verticalAlign = "top";
                                    }
                                    styleOfOuterDomElement.height = !isNaN(frameworkElement.Height) ? (System.Double.format(frameworkElement.Height) || "") + "px" : "auto";
                                    break;
                                case Windows.UI.Xaml.VerticalAlignment.Center: 
                                    if (!isParentAWrapPanelOrAVerticalStackPanel) {
                                        if (isCSSGrid && (Bridge.is(frameworkElement.Parent, Windows.UI.Xaml.Controls.Grid))) {
                                            var boxSizingStyle1 = CSHTML5.Internal.INTERNAL_HtmlDomManager.GetFrameworkElementBoxSizingStyleForModification(frameworkElement);
                                            if (!isMsGrid) {
                                                boxSizingStyle1.marginTop = "auto";
                                                boxSizingStyle1.marginBottom = "auto";

                                            } else {
                                                boxSizingStyle1.msGridRowAlign = "center";
                                            }
                                        } else {
                                            styleOfParentOfTheWrapperElement.display = "table";
                                            styleOfWrapperElement.display = "table-cell";
                                        }
                                        styleOfWrapperElement.verticalAlign = "middle";
                                    }
                                    styleOfOuterDomElement.height = !isNaN(frameworkElement.Height) ? (System.Double.format(frameworkElement.Height) || "") + "px" : "auto";
                                    break;
                                case Windows.UI.Xaml.VerticalAlignment.Bottom: 
                                    if (!isParentAWrapPanelOrAVerticalStackPanel) {
                                        if (isCSSGrid && (Bridge.is(frameworkElement.Parent, Windows.UI.Xaml.Controls.Grid))) {
                                            var boxSizingStyle2 = CSHTML5.Internal.INTERNAL_HtmlDomManager.GetFrameworkElementBoxSizingStyleForModification(frameworkElement);
                                            if (!isMsGrid) {
                                                boxSizingStyle2.marginTop = "auto";
                                                boxSizingStyle2.marginBottom = "0px";

                                            } else {
                                                boxSizingStyle2.msGridRowAlign = "end";
                                            }
                                        } else {
                                            styleOfParentOfTheWrapperElement.display = "table";
                                            styleOfWrapperElement.display = "table-cell";
                                            styleOfWrapperElement.verticalAlign = "bottom";
                                        }
                                    }
                                    styleOfOuterDomElement.height = !isNaN(frameworkElement.Height) ? (System.Double.format(frameworkElement.Height) || "") + "px" : "auto";
                                    break;
                                case Windows.UI.Xaml.VerticalAlignment.Stretch: 
                                    if (!isParentAWrapPanelOrAVerticalStackPanel) {
                                        if (!(isCSSGrid && (Bridge.is(frameworkElement.Parent, Windows.UI.Xaml.Controls.Grid)))) {
                                            styleOfWrapperElement.verticalAlign = "middle";
                                            styleOfOuterDomElement.height = "100%";
                                        } else {
                                            var boxSizingStyle3 = CSHTML5.Internal.INTERNAL_HtmlDomManager.GetFrameworkElementBoxSizingStyleForModification(frameworkElement);
                                            boxSizingStyle3.msGridRowAlign = "stretch";
                                        }
                                    }
                                    break;
                                default: 
                                    throw new System.NotSupportedException.ctor();
                            }
                        }


                        /* 

                        // COMMENTED ON 2016.09.02 because it prevents properly displaying child elements with NEGATIVE MARGINS. To reproduce: put a border with negative margins inside another border.

                        if (!(frameworkElement is ScrollViewer) && !(frameworkElement is TextBox)) //Note: The ScrollViewer and the TextBox handle the "overflow" property by itself.
                        {
                           // We always display the portions of the child exceeding the edges, unless the element has a fixed size in pixels AND it is not a canvas, or if the element is a TextBox:
                           if (!double.IsNaN(frameworkElement.Height) && !(frameworkElement is Canvas))
                               styleOfOuterDomElement.overflowY = "hidden"; //Note: This value means to crop the portion of the child exceeding the edges.
                           else
                               styleOfOuterDomElement.overflowY = ""; //Note: the default value is "visible"
                        }
                        */


                        frameworkElement.OnAfterApplyVerticalAlignmentAndWidth();

                    }
                },
                Margin_MethodToUpdateDom: function (d, newValue) {

                    var frameworkElement = Bridge.cast(d, Windows.UI.Xaml.FrameworkElement);

                    var newMargin = System.Nullable.getValue(Bridge.cast(Bridge.unbox(newValue, Windows.UI.Xaml.Thickness), Windows.UI.Xaml.Thickness));
                    if (CSHTML5.Internal.INTERNAL_VisualTreeManager.IsElementInVisualTree(frameworkElement)) {
                        /* 
                        // Display an error if the user is setting the Margin property AFTER adding the element to the Visual Tree.
                        // In fact, due to the optimization made in October 2016, when attaching to the Visual Tree, we create the DIV
                        // for margins ONLY if some margins have been set. So it is too late to add the DIV after attached to the visual
                        // tree.
                        if (oldMargin.Left == 0d && oldMargin.Top == 0d && oldMargin.Bottom == 0d && oldMargin.Right == 0d
                           && (newMargin.Left != 0d || newMargin.Top != 0d || newMargin.Right != 0d || newMargin.Bottom != 0d))
                        {
                           if (!_theWarningAboutMarginsHasAlreadyBeenDisplayed && CSHTML5.Interop.IsRunningInTheSimulator)
                           {
                               MessageBox.Show("TIP: For the most accurate result, please set the 'Margin' property BEFORE adding the element to the visual tree.");
                               _theWarningAboutMarginsHasAlreadyBeenDisplayed = true;
                           }
                        }
                        */
                        var isCSSGrid = document.isGridSupported;
                        var isMsGrid = document.isMSGrid;
                        var isInsideACSSBasedGrid = isCSSGrid && !isMsGrid && Bridge.is(frameworkElement.INTERNAL_VisualParent, Windows.UI.Xaml.Controls.Grid);

                        var boxSizingElement = frameworkElement.INTERNAL_AdditionalOutsideDivForMargins;
                        var styleOfBoxSizingElement = CSHTML5.Internal.INTERNAL_HtmlDomManager.GetDomElementStyleForModification(boxSizingElement);
                        var styleOfOuterDomElement = CSHTML5.Internal.INTERNAL_HtmlDomManager.GetFrameworkElementOuterStyleForModification(frameworkElement);
                        if (System.Nullable.lifteq(Windows.UI.Xaml.Thickness.op_Equality, newMargin.$clone(), null)) {
                            newMargin = new Windows.UI.Xaml.Thickness.ctor();
                        }

                        if (newMargin.Left >= 0) {
                            styleOfBoxSizingElement.paddingLeft = (System.Double.format(newMargin.Left) || "") + "px";
                            styleOfBoxSizingElement.marginLeft = "";
                        } else {
                            styleOfBoxSizingElement.paddingLeft = "";
                            styleOfBoxSizingElement.marginLeft = (System.Double.format(newMargin.Left) || "") + "px";
                        }

                        if (newMargin.Top >= 0) {
                            styleOfBoxSizingElement.paddingTop = (System.Double.format(newMargin.Top) || "") + "px";
                            styleOfOuterDomElement.marginTop = "";
                        } else {
                            styleOfBoxSizingElement.paddingTop = "";
                            if (!isInsideACSSBasedGrid || frameworkElement.VerticalAlignment === Windows.UI.Xaml.VerticalAlignment.Top || frameworkElement.VerticalAlignment === Windows.UI.Xaml.VerticalAlignment.Stretch) {
                                styleOfOuterDomElement.marginTop = (System.Double.format(newMargin.Top) || "") + "px";
                            }
                        }
                        if (newMargin.Right >= 0) {
                            styleOfBoxSizingElement.paddingRight = (System.Double.format(newMargin.Right) || "") + "px";
                            styleOfBoxSizingElement.marginRight = "";
                        } else {
                            styleOfBoxSizingElement.paddingRight = "";
                            styleOfBoxSizingElement.marginRight = (System.Double.format(newMargin.Right) || "") + "px";
                        }
                        if (newMargin.Bottom >= 0) {
                            styleOfBoxSizingElement.paddingBottom = (System.Double.format(newMargin.Bottom) || "") + "px";
                            styleOfOuterDomElement.marginBottom = "";
                        } else {
                            styleOfBoxSizingElement.paddingBottom = "";
                            styleOfOuterDomElement.marginBottom = (System.Double.format(newMargin.Bottom) || "") + "px";
                        }
                    }

                },
                MinHeight_Changed: function (d, e) {
                    var $t;
                    var frameworkElement = Bridge.cast(d, Windows.UI.Xaml.FrameworkElement);
                    if (CSHTML5.Internal.INTERNAL_VisualTreeManager.IsElementInVisualTree(frameworkElement) && Bridge.is(e.NewValue, System.Double)) {
                        var newValue = System.Nullable.getValue(Bridge.cast(Bridge.unbox(e.NewValue, System.Double), System.Double));
                        var domElementConcernedByTheCssProperty = ($t = frameworkElement.INTERNAL_OptionalSpecifyDomElementConcernedByMinMaxHeightAndWidth, $t != null ? $t : frameworkElement.INTERNAL_OuterDomElement);
                        var style = CSHTML5.Internal.INTERNAL_HtmlDomManager.GetDomElementStyleForModification(domElementConcernedByTheCssProperty);
                        if (isNaN(frameworkElement.Height) && Bridge.referenceEquals(style.display, "table")) {
                            style.height = !isNaN(newValue) && newValue > 0 ? (System.Double.format(newValue) || "") + "px" : "initial";
                        } else {
                            style.minHeight = !isNaN(newValue) && newValue > 0 ? (System.Double.format(newValue) || "") + "px" : "initial";
                        }
                    }
                },
                MinWidth_Changed: function (d, e) {
                    var $t;
                    var frameworkElement = Bridge.cast(d, Windows.UI.Xaml.FrameworkElement);
                    if (CSHTML5.Internal.INTERNAL_VisualTreeManager.IsElementInVisualTree(frameworkElement) && Bridge.is(e.NewValue, System.Double)) {
                        var newValue = System.Nullable.getValue(Bridge.cast(Bridge.unbox(e.NewValue, System.Double), System.Double));
                        var domElementConcernedByTheCssProperty = ($t = frameworkElement.INTERNAL_OptionalSpecifyDomElementConcernedByMinMaxHeightAndWidth, $t != null ? $t : frameworkElement.INTERNAL_OuterDomElement);
                        var style = CSHTML5.Internal.INTERNAL_HtmlDomManager.GetDomElementStyleForModification(domElementConcernedByTheCssProperty);
                        style.minWidth = !isNaN(newValue) && newValue > 0 ? (System.Double.format(newValue) || "") + "px" : "initial";
                    }
                },
                MaxHeight_Changed: function (d, e) {
                    var $t;
                    var frameworkElement = Bridge.cast(d, Windows.UI.Xaml.FrameworkElement);
                    if (CSHTML5.Internal.INTERNAL_VisualTreeManager.IsElementInVisualTree(frameworkElement) && Bridge.is(e.NewValue, System.Double)) {
                        var newValue = System.Nullable.getValue(Bridge.cast(Bridge.unbox(e.NewValue, System.Double), System.Double));
                        var domElementConcernedByTheCssProperty = ($t = frameworkElement.INTERNAL_OptionalSpecifyDomElementConcernedByMinMaxHeightAndWidth, $t != null ? $t : frameworkElement.INTERNAL_OuterDomElement);
                        var style = CSHTML5.Internal.INTERNAL_HtmlDomManager.GetDomElementStyleForModification(domElementConcernedByTheCssProperty);

                        style.maxHeight = (newValue !== System.Double.max) ? (System.Double.format(newValue) || "") + "px" : "initial";
                    }
                },
                MaxWidth_Changed: function (d, e) {
                    var $t;
                    var frameworkElement = Bridge.cast(d, Windows.UI.Xaml.FrameworkElement);
                    if (CSHTML5.Internal.INTERNAL_VisualTreeManager.IsElementInVisualTree(frameworkElement) && Bridge.is(e.NewValue, System.Double)) {
                        var newValue = System.Nullable.getValue(Bridge.cast(Bridge.unbox(e.NewValue, System.Double), System.Double));
                        var domElementConcernedByTheCssProperty = ($t = frameworkElement.INTERNAL_OptionalSpecifyDomElementConcernedByMinMaxHeightAndWidth, $t != null ? $t : frameworkElement.INTERNAL_OuterDomElement);
                        var style = CSHTML5.Internal.INTERNAL_HtmlDomManager.GetDomElementStyleForModification(domElementConcernedByTheCssProperty);

                        style.maxWidth = (newValue !== System.Double.max) ? (System.Double.format(newValue) || "") + "px" : "initial";
                    }
                },
                ContextMenu_Changed: function (d, e) {
                    var frameworkElement = Bridge.cast(d, Windows.UI.Xaml.FrameworkElement);
                    var contextMenu = Bridge.cast(e.NewValue, Windows.UI.Xaml.Controls.ContextMenu);

                    Windows.UI.Xaml.Controls.INTERNAL_ContextMenuHelpers.RegisterContextMenu$1(frameworkElement, contextMenu);
                }
            }
        },
        fields: {
            INTERNAL_ForceEnableAllPointerEvents: false,
            INTERNAL_defaultStyle: null,
            _resources: null,
            /**
             * Derived classes can set this flag in their constructor to prevent the "Style" property from being applied.
             *
             * @instance
             * @protected
             * @memberof Windows.UI.Xaml.FrameworkElement
             * @default false
             * @type boolean
             */
            INTERNAL_DoNotApplyStyle: false,
            _styleCache: null,
            _themeStyleCache: null,
            INTERNAL_AreThereAnyBindingValidationErrorHandlers: false,
            _bindingValidationErrorHandlers: null,
            _flags: 0,
            _valueOfLastSizeChanged: null,
            _sizeChangedEventHandlers: null,
            _resizeSensor: null
        },
        events: {
            /**
             * Occurs when the IsEnabled property changes.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.FrameworkElement
             * @memberof Windows.UI.Xaml.FrameworkElement
             * @function addIsEnabledChanged
             * @param   {Windows.UI.Xaml.DependencyPropertyChangedEventHandler}    value
             * @return  {void}
             */
            /**
             * Occurs when the IsEnabled property changes.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.FrameworkElement
             * @memberof Windows.UI.Xaml.FrameworkElement
             * @function removeIsEnabledChanged
             * @param   {Windows.UI.Xaml.DependencyPropertyChangedEventHandler}    value
             * @return  {void}
             */
            IsEnabledChanged: null,
            /**
             * Occurs when the data context for this element changes.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.FrameworkElement
             * @memberof Windows.UI.Xaml.FrameworkElement
             * @function addDataContextChanged
             * @param   {Windows.UI.Xaml.DependencyPropertyChangedEventHandler}    value
             * @return  {void}
             */
            /**
             * Occurs when the data context for this element changes.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.FrameworkElement
             * @memberof Windows.UI.Xaml.FrameworkElement
             * @function removeDataContextChanged
             * @param   {Windows.UI.Xaml.DependencyPropertyChangedEventHandler}    value
             * @return  {void}
             */
            DataContextChanged: null,
            /**
             * Occurs when a FrameworkElement has been constructed and added to the object tree.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.FrameworkElement
             * @memberof Windows.UI.Xaml.FrameworkElement
             * @function addLoaded
             * @param   {Windows.UI.Xaml.RoutedEventHandler}    value
             * @return  {void}
             */
            /**
             * Occurs when a FrameworkElement has been constructed and added to the object tree.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.FrameworkElement
             * @memberof Windows.UI.Xaml.FrameworkElement
             * @function removeLoaded
             * @param   {Windows.UI.Xaml.RoutedEventHandler}    value
             * @return  {void}
             */
            Loaded: null,
            /**
             * Occurs when this object is no longer connected to the main object tree.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.FrameworkElement
             * @memberof Windows.UI.Xaml.FrameworkElement
             * @function addUnloaded
             * @param   {Windows.UI.Xaml.RoutedEventHandler}    value
             * @return  {void}
             */
            /**
             * Occurs when this object is no longer connected to the main object tree.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.FrameworkElement
             * @memberof Windows.UI.Xaml.FrameworkElement
             * @function removeUnloaded
             * @param   {Windows.UI.Xaml.RoutedEventHandler}    value
             * @return  {void}
             */
            Unloaded: null,
            /**
             * Occurs when any context menu on the element is opened.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.FrameworkElement
             * @memberof Windows.UI.Xaml.FrameworkElement
             * @function addContextMenuOpening
             * @param   {Windows.UI.Xaml.Controls.ContextMenuEventHandler}    value
             * @return  {void}
             */
            /**
             * Occurs when any context menu on the element is opened.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.FrameworkElement
             * @memberof Windows.UI.Xaml.FrameworkElement
             * @function removeContextMenuOpening
             * @param   {Windows.UI.Xaml.Controls.ContextMenuEventHandler}    value
             * @return  {void}
             */
            ContextMenuOpening: null
        },
        props: {
            /**
             * Check if resource is not empty.
                 Call HasResources before accessing resources every time you need
                 to query for a resource.
             *
             * @instance
             * @readonly
             * @memberof Windows.UI.Xaml.FrameworkElement
             * @function HasResources
             * @type boolean
             */
            HasResources: {
                get: function () {
                    var resources = this._resources;
                    return (resources != null && ((resources.Count > 0) || (resources.MergedDictionaries.Count > 0)));
                }
            },
            /**
             * Gets the locally defined resource dictionary. In XAML, you can establish
             resource items as child object elements of a frameworkElement.Resources property
             element, through XAML implicit collection syntax.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.FrameworkElement
             * @function Resources
             * @type Windows.UI.Xaml.ResourceDictionary
             */
            Resources: {
                get: function () {
                    if (this._resources == null) {
                        var resource = new Windows.UI.Xaml.ResourceDictionary();
                        resource.AddOwner(this);
                        this._resources = resource;
                    }
                    return this._resources;
                },
                set: function (value) {
                    var oldValue = this._resources;
                    this._resources = value;

                    if (oldValue != null) {
                        oldValue.RemoveOwner(this);
                    }

                    if (value != null) {
                        if (!value.ContainsOwner(this)) {
                            value.AddOwner(this);
                        }
                    }


                    {
                        this.HasStyleInvalidated = false;

                        if (this.HasImplicitStyleFromResources === true && (oldValue.contains(Bridge.getType(this)) || Bridge.referenceEquals(this.Style, Windows.UI.Xaml.FrameworkElement.StyleProperty.GetMetadata(Bridge.getType(this)).DefaultValue))) {
                            this.UpdateStyleProperty();
                        }
                    }
                }
            },
            /**
             * Gets the parent object of this FrameworkElement in the object tree.
             *
             * @instance
             * @public
             * @readonly
             * @memberof Windows.UI.Xaml.FrameworkElement
             * @function Parent
             * @type Windows.UI.Xaml.DependencyObject
             */
            Parent: {
                get: function () {
                    return this.INTERNAL_VisualParent;
                }
            },
            /**
             * Gets a value that indicates whether this element is in the Visual Tree, that is, if it has been loaded for presentation.
             *
             * @instance
             * @public
             * @readonly
             * @memberof Windows.UI.Xaml.FrameworkElement
             * @function IsLoaded
             * @type boolean
             */
            IsLoaded: {
                get: function () {
                    return this._isLoaded;
                }
            },
            /**
             * Gets or sets the cursor that displays when the mouse pointer is over this
             element.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.FrameworkElement
             * @function Cursor
             * @type System.Windows.Input.Cursor
             */
            Cursor: {
                get: function () {
                    return Bridge.cast(this.GetValue(Windows.UI.Xaml.FrameworkElement.CursorProperty), System.Windows.Input.Cursor);
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.FrameworkElement.CursorProperty, value);
                }
            },
            /**
             * Gets or sets a value indicating whether the user can interact with the control.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.FrameworkElement
             * @function IsEnabled
             * @type boolean
             */
            IsEnabled: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.FrameworkElement.IsEnabledProperty), System.Boolean), System.Boolean));
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.FrameworkElement.IsEnabledProperty, Bridge.box(value, System.Boolean, System.Boolean.toString));
                }
            },
            /**
             * Gets or sets the identifying name of the object. When a XAML processor creates
             the object tree from XAML markup, run-time code can refer to the XAML-declared
             object by this name.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.FrameworkElement
             * @function Name
             * @type string
             */
            Name: {
                get: function () {
                    return Bridge.cast(this.GetValue(Windows.UI.Xaml.FrameworkElement.NameProperty), System.String);
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.FrameworkElement.NameProperty, value);
                }
            },
            /**
             * Gets or sets the data context for a FrameworkElement when it participates
             in data binding.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.FrameworkElement
             * @function DataContext
             * @type System.Object
             */
            DataContext: {
                get: function () {
                    return this.GetValue(Windows.UI.Xaml.FrameworkElement.DataContextProperty);
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.FrameworkElement.DataContextProperty, value);
                }
            },
            /**
             * Gets or sets an arbitrary object value that can be used to store custom information
             about this object.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.FrameworkElement
             * @function Tag
             * @type System.Object
             */
            Tag: {
                get: function () {
                    return this.GetValue(Windows.UI.Xaml.FrameworkElement.TagProperty);
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.FrameworkElement.TagProperty, value);
                }
            },
            /**
             * Gets or sets an instance Style that is applied for this object during rendering.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.FrameworkElement
             * @function Style
             * @type Windows.UI.Xaml.Style
             */
            Style: {
                get: function () {
                    return this._styleCache;
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.FrameworkElement.StyleProperty, value);
                }
            },
            HasStyleInvalidated: {
                get: function () {
                    return this.ReadInternalFlag(Windows.UI.Xaml.InternalFlags.HasStyleInvalidated);
                },
                set: function (value) {
                    this.WriteInternalFlag(Windows.UI.Xaml.InternalFlags.HasStyleInvalidated, value);
                }
            },
            IsStyleUpdateInProgress: {
                get: function () {
                    return this.ReadInternalFlag(Windows.UI.Xaml.InternalFlags.IsStyleUpdateInProgress);
                },
                set: function (value) {
                    this.WriteInternalFlag(Windows.UI.Xaml.InternalFlags.IsStyleUpdateInProgress, value);
                }
            },
            ShouldLookupImplicitStyles: {
                get: function () {
                    return this.ReadInternalFlag(Windows.UI.Xaml.InternalFlags.ShouldLookupImplicitStyles);
                },
                set: function (value) {
                    this.WriteInternalFlag(Windows.UI.Xaml.InternalFlags.ShouldLookupImplicitStyles, value);
                }
            },
            HasImplicitStyleFromResources: {
                get: function () {
                    return this.ReadInternalFlag(Windows.UI.Xaml.InternalFlags.HasImplicitStyleFromResources);
                },
                set: function (value) {
                    this.WriteInternalFlag(Windows.UI.Xaml.InternalFlags.HasImplicitStyleFromResources, value);
                }
            },
            IsThemeStyleUpdateInProgress: {
                get: function () {
                    return this.ReadInternalFlag(Windows.UI.Xaml.InternalFlags.IsThemeStyleUpdateInProgress);
                },
                set: function (value) {
                    this.WriteInternalFlag(Windows.UI.Xaml.InternalFlags.IsThemeStyleUpdateInProgress, value);
                }
            },
            HasThemeStyleEverBeenFetched: {
                get: function () {
                    return this.ReadInternalFlag(Windows.UI.Xaml.InternalFlags.HasThemeStyleEverBeenFetched);
                },
                set: function (value) {
                    this.WriteInternalFlag(Windows.UI.Xaml.InternalFlags.HasThemeStyleEverBeenFetched, value);
                }
            },
            HasLocalStyle: {
                get: function () {
                    return this.ReadInternalFlag(Windows.UI.Xaml.InternalFlags.HasLocalStyle);
                },
                set: function (value) {
                    this.WriteInternalFlag(Windows.UI.Xaml.InternalFlags.HasLocalStyle, value);
                }
            },
            /**
             * Gets or sets the key that references the default style for the control.
             *
             * @instance
             * @protected
             * @memberof Windows.UI.Xaml.FrameworkElement
             * @function DefaultStyleKey
             * @type System.Object
             */
            DefaultStyleKey: {
                get: function () {
                    return this.GetValue(Windows.UI.Xaml.FrameworkElement.DefaultStyleKeyProperty);
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.FrameworkElement.DefaultStyleKeyProperty, value);
                }
            },
            ThemeStyle: {
                get: function () {
                    return this._themeStyleCache;
                }
            },
            /**
             * Gets or sets the suggested height of a FrameworkElement.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.FrameworkElement
             * @function Height
             * @type number
             */
            Height: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.FrameworkElement.HeightProperty), System.Double), System.Double));
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.FrameworkElement.HeightProperty, Bridge.box(value, System.Double, System.Double.format, System.Double.getHashCode));
                }
            },
            /**
             * Gets or sets the width of a FrameworkElement.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.FrameworkElement
             * @function Width
             * @type number
             */
            Width: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.FrameworkElement.WidthProperty), System.Double), System.Double));
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.FrameworkElement.WidthProperty, Bridge.box(value, System.Double, System.Double.format, System.Double.getHashCode));
                }
            },
            /**
             * Gets or sets the horizontal alignment characteristics that are applied to
             a FrameworkElement when it is composed in a layout parent, such as a panel
             or items control.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.FrameworkElement
             * @function HorizontalAlignment
             * @type Windows.UI.Xaml.HorizontalAlignment
             */
            HorizontalAlignment: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.FrameworkElement.HorizontalAlignmentProperty), Windows.UI.Xaml.HorizontalAlignment), System.Int32));
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.FrameworkElement.HorizontalAlignmentProperty, Bridge.box(value, Windows.UI.Xaml.HorizontalAlignment, System.Enum.toStringFn(Windows.UI.Xaml.HorizontalAlignment)));
                }
            },
            /**
             * Gets or sets the vertical alignment characteristics that are applied to a
             FrameworkElement when it is composed in a parent object such as a panel or
             items control.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.FrameworkElement
             * @function VerticalAlignment
             * @type Windows.UI.Xaml.VerticalAlignment
             */
            VerticalAlignment: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.FrameworkElement.VerticalAlignmentProperty), Windows.UI.Xaml.VerticalAlignment), System.Int32));
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.FrameworkElement.VerticalAlignmentProperty, Bridge.box(value, Windows.UI.Xaml.VerticalAlignment, System.Enum.toStringFn(Windows.UI.Xaml.VerticalAlignment)));
                }
            },
            /**
             * Gets or sets the outer margin of a FrameworkElement.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.FrameworkElement
             * @function Margin
             * @type Windows.UI.Xaml.Thickness
             */
            Margin: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.FrameworkElement.MarginProperty), Windows.UI.Xaml.Thickness), Windows.UI.Xaml.Thickness));
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.FrameworkElement.MarginProperty, value.$clone());
                }
            },
            /**
             * Gets or sets the minimum height constraint of a FrameworkElement.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.FrameworkElement
             * @function MinHeight
             * @type number
             */
            MinHeight: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.FrameworkElement.MinHeightProperty), System.Double), System.Double));
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.FrameworkElement.MinHeightProperty, Bridge.box(value, System.Double, System.Double.format, System.Double.getHashCode));
                }
            },
            /**
             * Gets or sets the minimum width constraint of a FrameworkElement.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.FrameworkElement
             * @function MinWidth
             * @type number
             */
            MinWidth: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.FrameworkElement.MinWidthProperty), System.Double), System.Double));
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.FrameworkElement.MinWidthProperty, Bridge.box(value, System.Double, System.Double.format, System.Double.getHashCode));
                }
            },
            /**
             * Gets or sets the maximum height constraint of a FrameworkElement.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.FrameworkElement
             * @function MaxHeight
             * @type number
             */
            MaxHeight: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.FrameworkElement.MaxHeightProperty), System.Double), System.Double));
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.FrameworkElement.MaxHeightProperty, Bridge.box(value, System.Double, System.Double.format, System.Double.getHashCode));
                }
            },
            /**
             * Gets or sets the maximum width constraint of a FrameworkElement.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.FrameworkElement
             * @function MaxWidth
             * @type number
             */
            MaxWidth: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.FrameworkElement.MaxWidthProperty), System.Double), System.Double));
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.FrameworkElement.MaxWidthProperty, Bridge.box(value, System.Double, System.Double.format, System.Double.getHashCode));
                }
            },
            /**
             * Gets the rendered width of a FrameworkElement. The FrameworkElement must be in the visual tree,
             otherwise this property will return 0.
             *
             * @instance
             * @public
             * @readonly
             * @memberof Windows.UI.Xaml.FrameworkElement
             * @function ActualWidth
             * @type number
             */
            ActualWidth: {
                get: function () {
                    if (CSHTML5.Internal.INTERNAL_VisualTreeManager.IsElementInVisualTree(this) && this.INTERNAL_OuterDomElement != null) {
                        if (true) {
                            return this.INTERNAL_OuterDomElement.offsetWidth;
                        } else {
                            try {
                                return System.Convert.toDouble(CSHTML5.Internal.INTERNAL_HtmlDomManager.GetDomElementAttribute(this.INTERNAL_OuterDomElement, "offsetWidth"));
                            } catch ($e1) {
                                $e1 = System.Exception.create($e1);
                                return 0.0;
                            }
                        }
                    } else {
                        return 0.0;
                    }
                }
            },
            /**
             * Gets the rendered height of a FrameworkElement. The FrameworkElement must be in the visual tree,
             otherwise this property will return 0.
             *
             * @instance
             * @public
             * @readonly
             * @memberof Windows.UI.Xaml.FrameworkElement
             * @function ActualHeight
             * @type number
             */
            ActualHeight: {
                get: function () {
                    if (CSHTML5.Internal.INTERNAL_VisualTreeManager.IsElementInVisualTree(this) && this.INTERNAL_OuterDomElement != null) {
                        if (true) {
                            return this.INTERNAL_OuterDomElement.offsetHeight;
                        } else {
                            try {
                                return System.Convert.toDouble(CSHTML5.Internal.INTERNAL_HtmlDomManager.GetDomElementAttribute(this.INTERNAL_OuterDomElement, "offsetHeight"));
                            } catch ($e1) {
                                $e1 = System.Exception.create($e1);
                                return 0.0;
                            }
                        }
                    } else {
                        return 0.0;
                    }
                }
            },
            /**
             * Gets or sets the context menu element that should appear whenever the context menu is requested through user interface (UI) from within this element.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.FrameworkElement
             * @function ContextMenu
             * @type Windows.UI.Xaml.Controls.ContextMenu
             */
            ContextMenu: {
                get: function () {
                    return Bridge.cast(this.GetValue(Windows.UI.Xaml.FrameworkElement.ContextMenuProperty), Windows.UI.Xaml.Controls.ContextMenu);
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.FrameworkElement.ContextMenuProperty, value);
                }
            }
        },
        ctors: {
            init: function () {
                this._valueOfLastSizeChanged = new Windows.Foundation.Size();
                this.INTERNAL_ForceEnableAllPointerEvents = false;
                this.INTERNAL_DoNotApplyStyle = false;
                this.INTERNAL_AreThereAnyBindingValidationErrorHandlers = false;
                this._flags = 0;
                this._valueOfLastSizeChanged = new Windows.Foundation.Size.$ctor1(0.0, 0.0);
            },
            /**
             * Provides base class initialization behavior for FrameworkElement-derived
             classes.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.FrameworkElement
             * @memberof Windows.UI.Xaml.FrameworkElement
             * @return  {void}
             */
            ctor: function () {
                this.$initialize();
                Windows.UI.Xaml.UIElement.ctor.call(this);
                var metadata = Windows.UI.Xaml.FrameworkElement.StyleProperty.GetMetadata(Bridge.getType(this));
                var defaultValue = Bridge.cast(metadata.DefaultValue, Windows.UI.Xaml.Style);
                if (defaultValue != null) {
                    Windows.UI.Xaml.FrameworkElement.OnStyleChanged(this, new Windows.UI.Xaml.DependencyPropertyChangedEventArgs.$ctor1(null, defaultValue, Windows.UI.Xaml.FrameworkElement.StyleProperty));
                }

                var app = Windows.UI.Xaml.Application.Current;
                if (app != null && app.HasImplicitStylesInResources) {
                    this.ShouldLookupImplicitStyles = true;
                }
            }
        },
        methods: {
            /**
             * Occurs when a data validation error is reported by a binding source.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.FrameworkElement
             * @event Windows.UI.Xaml.FrameworkElement#BindingValidationError
             * @return  {System.EventHandler}
             */
            addBindingValidationError: function (value) {
                if (this._bindingValidationErrorHandlers == null) {
                    this._bindingValidationErrorHandlers = new (System.Collections.Generic.List$1(Function)).ctor();
                }
                this._bindingValidationErrorHandlers.add(value);

                this.INTERNAL_AreThereAnyBindingValidationErrorHandlers = true;
            },
            /**
             * Occurs when a data validation error is reported by a binding source.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.FrameworkElement
             * @event Windows.UI.Xaml.FrameworkElement#BindingValidationError
             * @return  {System.EventHandler}
             */
            removeBindingValidationError: function (value) {
                if (this._bindingValidationErrorHandlers != null) {
                    this._bindingValidationErrorHandlers.remove(value);

                    if (this._bindingValidationErrorHandlers.Count === 0) {
                        this.INTERNAL_AreThereAnyBindingValidationErrorHandlers = false;
                    }
                } else {
                    this.INTERNAL_AreThereAnyBindingValidationErrorHandlers = false;
                }
            },
            addSizeChanged: function (value) {
                if (this._sizeChangedEventHandlers == null) {
                    this._sizeChangedEventHandlers = new (System.Collections.Generic.List$1(Function)).ctor();
                }
                if (this._sizeChangedEventHandlers.Count === 0 && this.INTERNAL_OuterDomElement != null) {
                    this._resizeSensor = new ResizeSensor(this.INTERNAL_OuterDomElement, Bridge.fn.cacheBind(this, this.HandleSizeChanged));
                }
                this._sizeChangedEventHandlers.add(value);
            },
            removeSizeChanged: function (value) {
                if (this._sizeChangedEventHandlers == null) {
                    return;
                }

                if (this._sizeChangedEventHandlers.remove(value)) {
                    if (this._sizeChangedEventHandlers.Count === 0 && this._resizeSensor != null) {
                        this._resizeSensor.detach(this.INTERNAL_OuterDomElement);
                    }
                }
            },
            /**
             * Provides a base implementation for creating the dom elements designed to represent an instance of a FrameworkElement and defines the place where its child(ren) will be added.
             *
             * @instance
             * @public
             * @override
             * @this Windows.UI.Xaml.FrameworkElement
             * @memberof Windows.UI.Xaml.FrameworkElement
             * @param   {System.Object}    parentRef                         The parent of the FrameworkElement
             * @param   {System.Object}    domElementWhereToPlaceChildren    The dom element where the FrameworkElement's children will be added.
             * @return  {System.Object}                                      The "root" dom element of the FrameworkElement.
             */
            CreateDomElement: function (parentRef, domElementWhereToPlaceChildren) {

                var div1 = { };
                var div1style = CSHTML5.Internal.INTERNAL_HtmlDomManager.CreateDomElementAppendItAndGetStyle("div", parentRef, this, div1);
                var div2 = { };
                var div2style = CSHTML5.Internal.INTERNAL_HtmlDomManager.CreateDomElementAppendItAndGetStyle("div", div1.v, this, div2);
                div2style.width = "100%";
                div2style.height = "100%";
                if (this.INTERNAL_ForceEnableAllPointerEvents) {
                    div2style.pointerEvents = "all";
                }
                domElementWhereToPlaceChildren.v = div2.v;
                return div1.v;
            },
            CreateDomElement_WorkaroundBridgeInheritanceBug: function (parentRef, domElementWhereToPlaceChildren) {

                var div1 = { };
                var div1style = CSHTML5.Internal.INTERNAL_HtmlDomManager.CreateDomElementAppendItAndGetStyle("div", parentRef, this, div1);
                var div2 = { };
                var div2style = CSHTML5.Internal.INTERNAL_HtmlDomManager.CreateDomElementAppendItAndGetStyle("div", div1.v, this, div2);
                div2style.width = "100%";
                div2style.height = "100%";
                if (this.INTERNAL_ForceEnableAllPointerEvents) {
                    div2style.pointerEvents = "all";
                }
                domElementWhereToPlaceChildren.v = div2.v;
                return div1.v;
            },
            /**
             * Invoked whenever application code or internal processes (such as a rebuilding
             layout pass) call ApplyTemplate. In simplest terms, this means the method
             is called just before a UI element displays in your app. Override this method
             to influence the default post-template logic of a class.
             *
             * @instance
             * @protected
             * @this Windows.UI.Xaml.FrameworkElement
             * @memberof Windows.UI.Xaml.FrameworkElement
             * @return  {void}
             */
            OnApplyTemplate: function () {

            },
            /**
             * Attaches a binding to a FrameworkElement, using the provided binding object.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.FrameworkElement
             * @memberof Windows.UI.Xaml.FrameworkElement
             * @param   {Windows.UI.Xaml.DependencyProperty}        dependencyProperty    The dependency property identifier of the property that is data bound.
             * @param   {Windows.UI.Xaml.Data.Binding}              binding               The binding to use for the property.
             * @return  {Windows.UI.Xaml.Data.BindingExpression}                          The BindingExpression created.
             */
            SetBinding: function (dependencyProperty, binding) {
                return Windows.UI.Xaml.Data.BindingOperations.SetBinding(this, dependencyProperty, binding);
            },
            GetBindingExpression: function (dp) {
                return Windows.UI.Xaml.Data.BindingOperations.GetBindingExpression(this, dp);
            },
            ManageIsEnabled: function (isEnabled) {
                if (CSHTML5.Internal.INTERNAL_VisualTreeManager.IsElementInVisualTree(this)) {
                    var domElementToEnableOrDisable = (this.INTERNAL_OptionalSpecifyDomElementConcernedByIsEnabled != null ? this.INTERNAL_OptionalSpecifyDomElementConcernedByIsEnabled : this.INTERNAL_OuterDomElement);
                    if (isEnabled) {
                        CSHTML5.Internal.INTERNAL_HtmlDomManager.RemoveDomElementAttribute(domElementToEnableOrDisable, "disabled", true);
                    } else {
                        CSHTML5.Internal.INTERNAL_HtmlDomManager.SetDomElementAttribute(domElementToEnableOrDisable, "disabled", Bridge.box(true, System.Boolean, System.Boolean.toString), true);
                    }
                }
            },
            /**
             * Retrieves an object that has the specified identifier name.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.FrameworkElement
             * @memberof Windows.UI.Xaml.FrameworkElement
             * @param   {string}           name    The name of the requested object.
             * @return  {System.Object}            The requested object. This can be null if no matching object was found in
             the current XAML namescope.
             */
            FindName: function (name) {
                if (name == null) {
                    throw new System.ArgumentNullException.$ctor1("name");
                }

                if (Bridge.is(this, Windows.UI.Xaml.Markup.INameScope)) {
                    return Bridge.cast(this, Windows.UI.Xaml.Markup.INameScope).Windows$UI$Xaml$Markup$INameScope$FindName(name);
                } else {
                    if (this.INTERNAL_VisualParent != null) {
                        if (Bridge.is(this.INTERNAL_VisualParent, Windows.UI.Xaml.FrameworkElement)) {
                            return Bridge.cast(this.INTERNAL_VisualParent, Windows.UI.Xaml.FrameworkElement).FindName(name);
                        }
                    }
                }

                return null;
            },
            RaiseDataContextChangedEvent: function (e) {
                if (!Bridge.staticEquals(this.DataContextChanged, null)) {
                    this.DataContextChanged(this, e);
                }
            },
            INTERNAL_SetDefaultStyle: function (defaultStyle) {
                this.INTERNAL_defaultStyle = defaultStyle;
            },
            InvalidateStyleProperty: function () {
                var implicitStyle = Windows.UI.Xaml.FrameworkElement.FindImplicitStyleResource(this, Bridge.getType(this));

                this.HasImplicitStyleFromResources = !Bridge.referenceEquals(implicitStyle, Windows.UI.Xaml.DependencyProperty.UnsetValue);
                this.SetImplicitReferenceValue(Windows.UI.Xaml.FrameworkElement.StyleProperty, implicitStyle);
            },
            UpdateStyleProperty: function () {
                if (!this.HasStyleInvalidated) {
                    if (this.IsStyleUpdateInProgress === false) {
                        this.IsStyleUpdateInProgress = true;
                        try {
                            this.InvalidateStyleProperty();
                            this.HasStyleInvalidated = true;
                        } finally {
                            this.IsStyleUpdateInProgress = false;
                        }
                    } else {
                        throw new System.InvalidOperationException.$ctor1(System.String.format("Cyclic reference found while evaluating the Style property on element '{0}'.", [this]));
                    }
                }
            },
            /**
             * This method causes the ThemeStyleProperty to be re-evaluated
             *
             * @instance
             * @this Windows.UI.Xaml.FrameworkElement
             * @memberof Windows.UI.Xaml.FrameworkElement
             * @return  {void}
             */
            UpdateThemeStyleProperty: function () {
                if (this.IsThemeStyleUpdateInProgress === false) {
                    this.IsThemeStyleUpdateInProgress = true;
                    try {
                        Windows.UI.Xaml.FrameworkElement.GetThemeStyle(this);
                    } finally {
                        this.IsThemeStyleUpdateInProgress = false;
                    }
                } else {
                    throw new System.InvalidOperationException.$ctor1(System.String.format("Cyclic reference found while evaluating the ThemeStyle property on element '{0}'.", [this]));
                }
            },
            INTERNAL_RaiseLoadedEvent: function () {
                if (!Bridge.staticEquals(this.Loaded, null)) {
                    this.Loaded(this, new Windows.UI.Xaml.RoutedEventArgs());
                }
            },
            INTERNAL_RaiseUnloadedEvent: function () {
                if (!Bridge.staticEquals(this.Unloaded, null)) {
                    this.Unloaded(this, new Windows.UI.Xaml.RoutedEventArgs());
                }
            },
            INTERNAL_RaiseBindingValidationErrorEvent: function (eventArgs) {
                var $t;
                if (this._bindingValidationErrorHandlers != null) {
                    $t = Bridge.getEnumerator(this._bindingValidationErrorHandlers);
                    try {
                        while ($t.moveNext()) {
                            var eventHandler = $t.Current;
                            if (!Bridge.staticEquals(eventHandler, null)) {
                                eventHandler(this, eventArgs);
                            }
                        }
                    } finally {
                        if (Bridge.is($t, System.IDisposable)) {
                            $t.System$IDisposable$Dispose();
                        }
                    }
                }
            },
            INTERNAL_OnDetachedFromVisualTree: function () {
                Windows.UI.Xaml.UIElement.prototype.INTERNAL_OnDetachedFromVisualTree.call(this);
                if (this.HasImplicitStyleFromResources && (!this.HasResources || !this.Resources.contains(Bridge.getType(this)))) {
                    this.HasStyleInvalidated = false;
                    this.UpdateStyleProperty();
                }
            },
            INTERNAL_OnAttachedToVisualTree: function () {
                Windows.UI.Xaml.UIElement.prototype.INTERNAL_OnAttachedToVisualTree.call(this);

                if (!this.HasImplicitStyleFromResources || (!this.HasResources || !this.Resources.contains(Bridge.getType(this)))) {
                    this.HasStyleInvalidated = false;
                    this.UpdateStyleProperty();
                }

                if (!this.HasThemeStyleEverBeenFetched) {
                    this.UpdateThemeStyleProperty();
                }
            },
            ReadInternalFlag: function (reqFlag) {
                return (((this._flags & reqFlag) >>> 0)) !== 0;
            },
            WriteInternalFlag: function (reqFlag, set) {
                if (set) {
                    this._flags |= reqFlag >>> 0;
                } else {
                    this._flags &= ((~reqFlag)) >>> 0;
                }
            },
            OnAfterApplyHorizontalAlignmentAndWidth: function () { },
            OnAfterApplyVerticalAlignmentAndWidth: function () { },
            /**
             * Use this method for better performance in the Simulator compared to requesting the ActualWidth and ActualHeight separately.
             *
             * @instance
             * @this Windows.UI.Xaml.FrameworkElement
             * @memberof Windows.UI.Xaml.FrameworkElement
             * @return  {Windows.Foundation.Size}        The actual size of the element.
             */
            INTERNAL_GetActualWidthAndHeight: function () {
                if (CSHTML5.Internal.INTERNAL_VisualTreeManager.IsElementInVisualTree(this) && this.INTERNAL_OuterDomElement != null) {
                    if (true) {
                        var actualWidth = this.INTERNAL_OuterDomElement.offsetWidth;
                        var actualHeight = this.INTERNAL_OuterDomElement.offsetHeight;
                        return new Windows.Foundation.Size.$ctor1(actualWidth, actualHeight);
                    } else {
                        try {
                            var concatenated = this.INTERNAL_OuterDomElement['offsetWidth'].toFixed(3) + '|' + this.INTERNAL_OuterDomElement['offsetHeight'].toFixed(3).ToString();
                            var sepIndex = concatenated != null ? System.String.indexOf(concatenated, String.fromCharCode(124)) : -1;
                            if (sepIndex > -1) {
                                var actualWidthAsString = concatenated.substr(0, sepIndex);
                                var actualHeightAsString = concatenated.substr(((sepIndex + 1) | 0));
                                var actualWidth1 = Bridge.Int.parseFloat(actualWidthAsString, Bridge.global.System.Globalization.CultureInfo.invariantCulture);
                                var actualHeight1 = Bridge.Int.parseFloat(actualHeightAsString, Bridge.global.System.Globalization.CultureInfo.invariantCulture);
                                return new Windows.Foundation.Size.$ctor1(actualWidth1, actualHeight1);
                            } else {
                                return new Windows.Foundation.Size.$ctor1(0.0, 0.0);
                            }
                        } catch ($e1) {
                            $e1 = System.Exception.create($e1);
                            return new Windows.Foundation.Size.$ctor1(0.0, 0.0);
                        }
                    }
                } else {
                    return new Windows.Foundation.Size.$ctor1(0.0, 0.0);
                }
            },
            HandleSizeChanged: function () {
                var $t;
                if (this._sizeChangedEventHandlers != null && this._sizeChangedEventHandlers.Count > 0 && CSHTML5.Internal.INTERNAL_VisualTreeManager.IsElementInVisualTree(this) && this._isLoaded) {

                    var currentSize = this.INTERNAL_GetActualWidthAndHeight();

                    if (!Windows.Foundation.Size.Equals(this._valueOfLastSizeChanged.$clone(), currentSize.$clone())) {
                        this._valueOfLastSizeChanged = currentSize.$clone();

                        $t = Bridge.getEnumerator(this._sizeChangedEventHandlers);
                        try {
                            while ($t.moveNext()) {
                                var sizeChangedEventHandler = $t.Current;
                                sizeChangedEventHandler(this, new Windows.UI.Xaml.SizeChangedEventArgs(currentSize.$clone()));
                            }
                        } finally {
                            if (Bridge.is($t, System.IDisposable)) {
                                $t.System$IDisposable$Dispose();
                            }
                        }
                    }
                }
            },
            INTERNAL_SizeChangedWhenAttachedToVisualTree: function () {
                this._valueOfLastSizeChanged = Windows.Foundation.Size.Empty.$clone();
                this.HandleSizeChanged();
            },
            INTERNAL_RaiseContextMenuOpeningEvent: function (pointerLeft, pointerTop) {
                if (!Bridge.staticEquals(this.ContextMenuOpening, null)) {
                    this.ContextMenuOpening(this, new Windows.UI.Xaml.Controls.ContextMenuEventArgs(pointerLeft, pointerTop));
                }
            }
        }
    });

    Bridge.define("CSHTML5.Native.Html.Input.HtmlCanvasPointerRoutedEventArgs", {
        inherits: [Windows.UI.Xaml.Input.PointerRoutedEventArgs],
        fields: {
            HtmlCanvas: null
        },
        ctors: {
            ctor: function (e, htmlCanvas) {
                this.$initialize();
                Windows.UI.Xaml.Input.PointerRoutedEventArgs.ctor.call(this);
                this.OriginalSource = e.OriginalSource;
                this.Handled = e.Handled;
                this.KeyModifiers = e.KeyModifiers;
                this.Pointer = e.Pointer;
                this._pointerAbsoluteX = e._pointerAbsoluteX;
                this._pointerAbsoluteY = e._pointerAbsoluteY;

                this.HtmlCanvas = htmlCanvas;
            },
            $ctor1: function (e, htmlCanvas) {
                this.$initialize();
                Windows.UI.Xaml.Input.PointerRoutedEventArgs.ctor.call(this);
                this.OriginalSource = e.OriginalSource;
                this.Handled = e.Handled;
                this._pointerAbsoluteX = e._pointerAbsoluteX;
                this._pointerAbsoluteY = e._pointerAbsoluteY;

                this.HtmlCanvas = htmlCanvas;
            }
        },
        methods: {
            GetCurrentPoint$1: function (relativeTo) {
                if (relativeTo == null) {
                    return this.GetCurrentPoint(null);
                } else {
                    var pointerPoint = this.GetCurrentPoint(this.HtmlCanvas);
                    var elem = CSHTML5.Native.Html.Controls.HtmlCanvas.SearchElement(this.HtmlCanvas, relativeTo);

                    var x = pointerPoint.Position.X;
                    var y = pointerPoint.Position.Y;
                    if (elem == null) {
                        return pointerPoint;
                    }

                    while (elem.Count > 0) {
                        var e = elem.Pop();
                        x -= e.X;
                        y -= e.Y;
                    }


                    pointerPoint.Position = new Windows.Foundation.Point.$ctor1(x, y);
                    return pointerPoint;
                }
            }
        }
    });

    /**
     * Represents a binding that a Windows Communication Foundation (WCF) service
     can use to configure and expose endpoints that are able to communicate with
     ASMX-based Web services and clients and other services that conform to the
     WS-I Basic Profile 1.1.
     *
     * @public
     * @class System.ServiceModel.BasicHttpBinding
     * @augments System.ServiceModel.HttpBindingBase
     */
    Bridge.define("System.ServiceModel.BasicHttpBinding", {
        inherits: [System.ServiceModel.HttpBindingBase],
        fields: {
            /**
             * Gets the type of security used with this binding.
             *
             * @instance
             * @public
             * @memberof System.ServiceModel.BasicHttpBinding
             * @function Security
             * @type System.ServiceModel.BasicHttpSecurity
             */
            Security: null
        },
        ctors: {
            /**
             * Initializes a new instance of the System.ServiceModel.BasicHttpBinding class.
             *
             * @instance
             * @public
             * @this System.ServiceModel.BasicHttpBinding
             * @memberof System.ServiceModel.BasicHttpBinding
             * @return  {void}
             */
            ctor: function () {
                var $t;
                this.$initialize();
                System.ServiceModel.HttpBindingBase.ctor.call(this);
                this.Security = ($t = new System.ServiceModel.BasicHttpSecurity(), $t.Mode = System.ServiceModel.BasicHttpSecurityMode.None, $t);
            },
            /**
             * Initializes a new instance of the System.ServiceModel.BasicHttpBinding class
             with a specified type of security used by the binding.
             *
             * @instance
             * @public
             * @this System.ServiceModel.BasicHttpBinding
             * @memberof System.ServiceModel.BasicHttpBinding
             * @param   {System.ServiceModel.BasicHttpSecurityMode}    securityMode    The value of System.ServiceModel.BasicHttpSecurityMode that specifies the
             type of security that is used with the SOAP message and for the client.
             * @return  {void}
             */
            $ctor1: function (securityMode) {
                var $t;
                this.$initialize();
                System.ServiceModel.HttpBindingBase.ctor.call(this);
                this.Security = ($t = new System.ServiceModel.BasicHttpSecurity(), $t.Mode = securityMode, $t);
            }
        }
    });

    /**
     * Represents a collection of IAttachedObject with a shared AssociatedObject
     and provides change notifications to its contents when that AssociatedObject
     changes.
     *
     * @abstract
     * @public
     * @class System.Windows.Interactivity.AttachableCollection$1
     * @augments Windows.UI.Xaml.DependencyObjectCollection$1
     * @implements  System.Windows.Interactivity.IAttachedObject
     */
    Bridge.define("System.Windows.Interactivity.AttachableCollection$1", function (T) { return {
        inherits: [Windows.UI.Xaml.DependencyObjectCollection$1(T),System.Windows.Interactivity.IAttachedObject],
        fields: {
            _associatedObject: null,
            _unorderedCopy: null
        },
        props: {
            /**
             * The object on which the collection is hosted.
             *
             * @instance
             * @public
             * @readonly
             * @memberof System.Windows.Interactivity.AttachableCollection$1
             * @function AssociatedObject
             * @type Windows.UI.Xaml.DependencyObject
             */
            AssociatedObject: {
                get: function () {
                    return this._associatedObject;
                }
            },
            System$Windows$Interactivity$IAttachedObject$AssociatedObject: {
                get: function () {
                    return this.AssociatedObject;
                }
            }
        },
        alias: [
            "Attach", "System$Windows$Interactivity$IAttachedObject$Attach",
            "Detach", "System$Windows$Interactivity$IAttachedObject$Detach"
        ],
        ctors: {
            ctor: function () {
                this.$initialize();
                Windows.UI.Xaml.DependencyObjectCollection$1(T).ctor.call(this);
                this._unorderedCopy = new (System.Collections.Generic.List$1(T)).ctor();
                this.addCollectionChanged(Bridge.fn.cacheBind(this, this.OnCollectionChanged$1));
            }
        },
        methods: {
            OnCollectionChanged$1: function (sender, e) {
                var $t, $t1, $t2, $t3;
                if (e.Action === System.Collections.Specialized.NotifyCollectionChangedAction.Remove || e.Action === System.Collections.Specialized.NotifyCollectionChangedAction.Replace) {
                    $t = Bridge.getEnumerator(e.OldItems);
                    try {
                        while ($t.moveNext()) {
                            var item = Bridge.cast($t.Current, T);
                            this._unorderedCopy.remove(item);
                            item.System$Windows$Interactivity$IAttachedObject$Detach();
                        }
                    } finally {
                        if (Bridge.is($t, System.IDisposable)) {
                            $t.System$IDisposable$Dispose();
                        }
                    }
                }

                if (e.Action === System.Collections.Specialized.NotifyCollectionChangedAction.Add || e.Action === System.Collections.Specialized.NotifyCollectionChangedAction.Replace) {
                    $t1 = Bridge.getEnumerator(e.NewItems);
                    try {
                        while ($t1.moveNext()) {
                            var item1 = Bridge.cast($t1.Current, T);
                            this._unorderedCopy.add(item1);
                            if (this._associatedObject != null) {
                                item1.System$Windows$Interactivity$IAttachedObject$Attach(this._associatedObject);
                            }
                        }
                    } finally {
                        if (Bridge.is($t1, System.IDisposable)) {
                            $t1.System$IDisposable$Dispose();
                        }
                    }
                }

                if (e.Action === System.Collections.Specialized.NotifyCollectionChangedAction.Reset) {
                    $t2 = Bridge.getEnumerator(this._unorderedCopy);
                    try {
                        while ($t2.moveNext()) {
                            var item2 = $t2.Current;
                            item2.System$Windows$Interactivity$IAttachedObject$Detach();
                        }
                    } finally {
                        if (Bridge.is($t2, System.IDisposable)) {
                            $t2.System$IDisposable$Dispose();
                        }
                    }
                    this._unorderedCopy.clear();
                    $t3 = Bridge.getEnumerator(this);
                    try {
                        while ($t3.moveNext()) {
                            var item3 = $t3.Current;
                            this._unorderedCopy.add(item3);
                            if (this._associatedObject != null) {
                                item3.System$Windows$Interactivity$IAttachedObject$Attach(this._associatedObject);
                            }
                        }
                    } finally {
                        if (Bridge.is($t3, System.IDisposable)) {
                            $t3.System$IDisposable$Dispose();
                        }
                    }
                }
            },
            /**
             * Attaches to the specified object.
             *
             * @instance
             * @public
             * @this System.Windows.Interactivity.AttachableCollection$1
             * @memberof System.Windows.Interactivity.AttachableCollection$1
             * @throws The IAttachedObject is already attached to a different object.
             * @param   {Windows.UI.Xaml.DependencyObject}    dependencyObject    The object to attach to.
             * @return  {void}
             */
            Attach: function (dependencyObject) {
                var $t;
                if (this._associatedObject != null) {
                    throw new System.InvalidOperationException.$ctor1("The AttachableCollection is already attached to a different object.");
                }
                this._associatedObject = dependencyObject;
                $t = Bridge.getEnumerator(this);
                try {
                    while ($t.moveNext()) {
                        var element = $t.Current;
                        element.System$Windows$Interactivity$IAttachedObject$Attach(dependencyObject);
                    }
                } finally {
                    if (Bridge.is($t, System.IDisposable)) {
                        $t.System$IDisposable$Dispose();
                    }
                }
                this.OnAttached();
            },
            /**
             * Detaches this instance from its associated object.
             *
             * @instance
             * @public
             * @this System.Windows.Interactivity.AttachableCollection$1
             * @memberof System.Windows.Interactivity.AttachableCollection$1
             * @return  {void}
             */
            Detach: function () {
                var $t;
                this.OnDetaching();
                $t = Bridge.getEnumerator(this);
                try {
                    while ($t.moveNext()) {
                        var element = $t.Current;
                        element.System$Windows$Interactivity$IAttachedObject$Detach();
                    }
                } finally {
                    if (Bridge.is($t, System.IDisposable)) {
                        $t.System$IDisposable$Dispose();
                    }
                }
                this._associatedObject = null;
            }
        }
    }; });

    /**
     * Encapsulates state information and zero or more ICommands into an attachable
     object.
     *
     * @abstract
     * @public
     * @class System.Windows.Interactivity.Behavior$1
     * @augments System.Windows.Interactivity.Behavior
     * @param   {Function}    [name]    The type the System.Windows.Interactivity.Behavior`1 can be attached to.
     */
    Bridge.define("System.Windows.Interactivity.Behavior$1", function (T) { return {
        inherits: [System.Windows.Interactivity.Behavior],
        props: {
            /**
             * Gets the object to which this System.Windows.Interactivity.Behavior`1 is
             attached.
             *
             * @instance
             * @public
             * @readonly
             * @memberof System.Windows.Interactivity.Behavior$1
             * @function AssociatedObject$1
             * @type T
             */
            AssociatedObject$1: {
                get: function () {
                    return Bridge.cast(this._associatedObject, T);
                }
            }
        },
        ctors: {
            /**
             * Initializes a new instance of the System.Windows.Interactivity.Behavior`1
             class.
             *
             * @instance
             * @protected
             * @this System.Windows.Interactivity.Behavior$1
             * @memberof System.Windows.Interactivity.Behavior$1
             * @return  {void}
             */
            ctor: function () {
                this.$initialize();
                System.Windows.Interactivity.Behavior.ctor.call(this);
                this._associatedType = T;
            }
        }
    }; });

    /**
     * A trigger that listens for a specified event on its source and fires when that event is fired.
     *
     * @public
     * @class System.Windows.Interactivity.EventTrigger
     * @augments System.Windows.Interactivity.TriggerBase
     */
    Bridge.define("System.Windows.Interactivity.EventTrigger", {
        inherits: [System.Windows.Interactivity.TriggerBase],
        statics: {
            fields: {
                EventNameProperty: null
            },
            ctors: {
                init: function () {
                    var $t;
                    this.EventNameProperty = Windows.UI.Xaml.DependencyProperty.Register("EventName", System.String, System.Windows.Interactivity.EventTrigger, ($t = new Windows.UI.Xaml.PropertyMetadata.$ctor2("Loaded", System.Windows.Interactivity.EventTrigger.OnEventNameChanged), $t.CallPropertyChangedWhenLoadedIntoVisualTree = Windows.UI.Xaml.WhenToCallPropertyChangedEnum.IfPropertyIsSet, $t));
                },
                ctor: function () {
                }
            },
            methods: {
                OnEventNameChanged: function (sender, args) {
                    var eventTrigger = Bridge.cast(sender, System.Windows.Interactivity.EventTrigger);
                    if (eventTrigger.AssociatedObject != null) {
                        var oldEventName = Bridge.cast(args.OldValue, System.String);
                        var newEventName = Bridge.cast(args.NewValue, System.String);
                        if (!System.String.isNullOrWhiteSpace(oldEventName)) {
                            eventTrigger.UnregisterEvent(eventTrigger.AssociatedObject, oldEventName);
                        }
                        if (!System.String.isNullOrWhiteSpace(newEventName)) {
                            eventTrigger.RegisterEvent(eventTrigger.AssociatedObject, newEventName);
                        }
                    }
                }
            }
        },
        props: {
            /**
             * Gets or sets the name of the event to listen for. This is a dependency property.
             *
             * @instance
             * @public
             * @memberof System.Windows.Interactivity.EventTrigger
             * @function EventName
             * @type string
             */
            EventName: {
                get: function () {
                    return Bridge.cast(this.GetValue(System.Windows.Interactivity.EventTrigger.EventNameProperty), System.String);
                },
                set: function (value) {
                    this.SetValue(System.Windows.Interactivity.EventTrigger.EventNameProperty, value);
                }
            }
        },
        ctors: {
            /**
             * Initializes a new instance of the {@link } class.
             *
             * @instance
             * @public
             * @this System.Windows.Interactivity.EventTrigger
             * @memberof System.Windows.Interactivity.EventTrigger
             * @return  {void}
             */
            ctor: function () {
                this.$initialize();
                System.Windows.Interactivity.TriggerBase.ctor.call(this);
            },
            /**
             * Initializes a new instance of the {@link } class.
             *
             * @instance
             * @public
             * @this System.Windows.Interactivity.EventTrigger
             * @memberof System.Windows.Interactivity.EventTrigger
             * @param   {string}    eventName    Name of the event.
             * @return  {void}
             */
            $ctor1: function (eventName) {
                this.$initialize();
                System.Windows.Interactivity.TriggerBase.ctor.call(this);
                this.EventName = eventName;
            }
        },
        methods: {
            GetEventName: function () {
                return this.EventName;
            },
            UnregisterEvent: function (associatedObject, eventName) {
                var type = Bridge.getType(associatedObject);
                var eventInfo = Bridge.Reflection.getMembers(type, 2, 284, eventName);
                if (eventInfo != null) {
                    var eventHandler = Bridge.fn.cacheBind(this, this.OnEventImpl);
                    Bridge.Reflection.midel(eventInfo.r, associatedObject)(eventHandler);
                }
            },
            RegisterEvent: function (associatedObject, eventName) {
                if (CSHTML5.Interop.IsRunningInTheSimulator) {
                    this.RegisterEvent_SimulatorOnly(associatedObject, eventName);
                } else {
                    var type = Bridge.getType(associatedObject);
                    var eventInfo = Bridge.Reflection.getMembers(type, 2, 284, eventName);
                    if (eventInfo != null) {
                        var eventHandler = Bridge.fn.cacheBind(this, this.OnEventImpl);
                        Bridge.Reflection.midel(eventInfo.r, associatedObject)(eventHandler);
                        Bridge.Reflection.midel(eventInfo.ad, associatedObject)(eventHandler);
                    }
                }
            },
            RegisterEvent_SimulatorOnly: function (associatedObject, eventName) {
                var type = Bridge.getType(associatedObject);
                var eventInfo = Bridge.Reflection.getMembers(type, 2, 284, eventName);
                if (eventInfo != null) {
                    var eventInfoType = Bridge.getType(eventInfo);
                    var eventhandlerTypeProperty = Bridge.Reflection.getMembers(eventInfoType, 16, 20 | 256, "EventHandlerType");
                    var eventHandlerType = Bridge.cast(Bridge.Reflection.midel(eventhandlerTypeProperty.g, eventInfo)(), System.Type);

                    if (Bridge.referenceEquals(eventHandlerType, Function)) {
                        var eventHandler = Bridge.fn.cacheBind(this, this.OnEventImpl);
                        Bridge.Reflection.midel(eventInfo.r, associatedObject)(eventHandler);
                        Bridge.Reflection.midel(eventInfo.ad, associatedObject)(eventHandler);
                    } else if (Bridge.referenceEquals(eventHandlerType, Function)) {
                        var eventHandler1 = Bridge.fn.cacheBind(this, this.OnEventImpl);
                        Bridge.Reflection.midel(eventInfo.r, associatedObject)(eventHandler1);
                        Bridge.Reflection.midel(eventInfo.ad, associatedObject)(eventHandler1);
                    } else if (Bridge.referenceEquals(eventHandlerType, Function)) {
                        var eventHandler2 = Bridge.fn.cacheBind(this, this.OnEventImpl);
                        Bridge.Reflection.midel(eventInfo.r, associatedObject)(eventHandler2);
                        Bridge.Reflection.midel(eventInfo.ad, associatedObject)(eventHandler2);
                    } else if (Bridge.referenceEquals(eventHandlerType, Function)) {
                        var eventHandler3 = Bridge.fn.cacheBind(this, this.OnEventImpl);
                        Bridge.Reflection.midel(eventInfo.r, associatedObject)(eventHandler3);
                        Bridge.Reflection.midel(eventInfo.ad, associatedObject)(eventHandler3);
                    }

                }
            },
            OnEventImpl: function (sender, eventArgs) {
                this.OnEvent(eventArgs);
            },
            OnEvent: function (eventArgs) {
                this.InvokeActions(eventArgs);
            },
            OnAttached: function () {
                System.Windows.Interactivity.TriggerBase.prototype.OnAttached.call(this);
                if (this.AssociatedObject != null) {
                    var eventName = this.EventName;
                    if (!System.String.isNullOrWhiteSpace(eventName)) {
                        this.RegisterEvent(this.AssociatedObject, eventName);
                    }
                }
            }
        }
    });

    /**
     * Executes a specified ICommand when invoked.
     *
     * @public
     * @class System.Windows.Interactivity.InvokeCommandAction
     * @augments System.Windows.Interactivity.TriggerAction
     */
    Bridge.define("System.Windows.Interactivity.InvokeCommandAction", {
        inherits: [System.Windows.Interactivity.TriggerAction],
        statics: {
            fields: {
                CommandProperty: null,
                CommandParameterProperty: null
            },
            ctors: {
                init: function () {
                    this.CommandProperty = Windows.UI.Xaml.DependencyProperty.Register("Command", System.Windows.Input.ICommand, System.Windows.Interactivity.InvokeCommandAction, null);
                    this.CommandParameterProperty = Windows.UI.Xaml.DependencyProperty.Register("CommandParameter", System.Object, System.Windows.Interactivity.InvokeCommandAction, null);
                },
                ctor: function () {
                }
            }
        },
        fields: {
            commandName: null
        },
        props: {
            /**
             * Gets or sets the name of the command this action should invoke.
             *
             * @instance
             * @public
             * @memberof System.Windows.Interactivity.InvokeCommandAction
             * @function CommandName
             * @type string
             */
            CommandName: {
                get: function () {
                    return this.commandName;
                },
                set: function (value) {
                    if (!(!Bridge.referenceEquals(this.CommandName, value))) {
                        return;
                    }
                    this.commandName = value;
                }
            },
            /**
             * Gets or sets the command this action should invoke. This is a dependency property.
             *
             * @instance
             * @public
             * @memberof System.Windows.Interactivity.InvokeCommandAction
             * @function Command
             * @type System.Windows.Input.ICommand
             */
            Command: {
                get: function () {
                    return Bridge.cast(this.GetValue(System.Windows.Interactivity.InvokeCommandAction.CommandProperty), System.Windows.Input.ICommand);
                },
                set: function (value) {
                    this.SetValue(System.Windows.Interactivity.InvokeCommandAction.CommandProperty, value);
                }
            },
            /**
             * Gets or sets the command parameter. This is a dependency property.
             *
             * @instance
             * @public
             * @memberof System.Windows.Interactivity.InvokeCommandAction
             * @function CommandParameter
             * @type System.Object
             */
            CommandParameter: {
                get: function () {
                    return this.GetValue(System.Windows.Interactivity.InvokeCommandAction.CommandParameterProperty);
                },
                set: function (value) {
                    this.SetValue(System.Windows.Interactivity.InvokeCommandAction.CommandParameterProperty, value);
                }
            }
        },
        methods: {
            /**
             * Invokes the action.
             *
             * @instance
             * @protected
             * @override
             * @this System.Windows.Interactivity.InvokeCommandAction
             * @memberof System.Windows.Interactivity.InvokeCommandAction
             * @param   {System.Object}    parameter    The parameter to the action. If the action does not require a parameter, the parameter may be set to a null reference.
             * @return  {void}
             */
            Invoke: function (parameter) {
                var $t;
                if (this.AssociatedObject == null) {
                    return;
                }
                var command = this.ResolveCommand();
                var param = ($t = this.CommandParameter, $t != null ? $t : parameter);
                if (command != null && command.System$Windows$Input$ICommand$CanExecute(param)) {
                    command.System$Windows$Input$ICommand$Execute(param);
                }
            },
            ResolveCommand: function () {
                var $t;
                var command = null;

                if (this.Command != null) {
                    command = this.Command;
                } else if (this.AssociatedObject != null) {
                    $t = Bridge.getEnumerator(Bridge.Reflection.getMembers(Bridge.getType(this.AssociatedObject), 16, 28));
                    try {
                        while ($t.moveNext()) {
                            var propertyInfo = $t.Current;
                            if (Bridge.Reflection.isAssignableFrom(Bridge.getType(System.Windows.Input.ICommand, System.Type), Bridge.getType(propertyInfo.rt, System.Type)) && System.String.equals(propertyInfo.n, this.CommandName, 4)) {
                                command = Bridge.cast(Bridge.Reflection.midel(propertyInfo.g, Bridge.unbox(this.AssociatedObject)).apply(null, Bridge.unbox(null)), System.Windows.Input.ICommand);
                            }
                        }
                    } finally {
                        if (Bridge.is($t, System.IDisposable)) {
                            $t.System$IDisposable$Dispose();
                        }
                    }
                }
                return command;
            }
        }
    });

    /**
     * A bitmap effect that paints a drop shadow around the target texture.
     * // Creates a modifiable clone of this System.Windows.Media.Effects.Effect object,
    // making deep copies of this object's values. When copying this object's dependency
    // properties, this method copies resource references and data bindings (which
    // may no longer resolve), but not animations or their current values.
    //
     * // Creates a modifiable clone of this System.Windows.Media.Effects.Effect object,
    // making deep copies of this object's current values. Resource references,
    // data bindings, and animations are not copied, but their current values are
    // copied.
    //
     *
     * @public
     * @class System.Windows.Media.Effects.DropShadowEffect
     * @augments System.Windows.Media.Effects.Effect
     * @return  {[type]}        // A modifiable clone of the current object. The cloned object's System.Windows.Freezable.IsFrozen
    // property will be false even if the source's System.Windows.Freezable.IsFrozen
    // property was true.
    //
     */
    Bridge.define("System.Windows.Media.Effects.DropShadowEffect", {
        inherits: [System.Windows.Media.Effects.Effect],
        statics: {
            fields: {
                /**
                 * Identifies the System.Windows.Media.Effects.DropShadowEffect.BlurRadius dependency
                 property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof System.Windows.Media.Effects.DropShadowEffect
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                BlurRadiusProperty: null,
                /**
                 * Identifies the System.Windows.Media.Effects.DropShadowEffect.Color dependency
                 property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof System.Windows.Media.Effects.DropShadowEffect
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                ColorProperty: null,
                /**
                 * Identifies the System.Windows.Media.Effects.DropShadowEffect.Direction dependency
                 property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof System.Windows.Media.Effects.DropShadowEffect
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                DirectionProperty: null,
                /**
                 * Identifies the System.Windows.Media.Effects.DropShadowEffect.Opacity dependency
                 property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof System.Windows.Media.Effects.DropShadowEffect
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                OpacityProperty: null,
                /**
                 * Identifies the System.Windows.Media.Effects.DropShadowEffect.ShadowDepth
                 dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof System.Windows.Media.Effects.DropShadowEffect
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                ShadowDepthProperty: null
            },
            ctors: {
                init: function () {
                    this.BlurRadiusProperty = Windows.UI.Xaml.DependencyProperty.Register("BlurRadius", System.Double, System.Windows.Media.Effects.DropShadowEffect, new Windows.UI.Xaml.PropertyMetadata.$ctor2(Bridge.box(5.0, System.Double, System.Double.format, System.Double.getHashCode), System.Windows.Media.Effects.DropShadowEffect.ManageChange));
                    this.ColorProperty = Windows.UI.Xaml.DependencyProperty.Register("Color", Windows.UI.Color, System.Windows.Media.Effects.DropShadowEffect, new Windows.UI.Xaml.PropertyMetadata.$ctor2(Windows.UI.Colors.Black.$clone(), System.Windows.Media.Effects.DropShadowEffect.ManageChange));
                    this.DirectionProperty = Windows.UI.Xaml.DependencyProperty.Register("Direction", System.Double, System.Windows.Media.Effects.DropShadowEffect, new Windows.UI.Xaml.PropertyMetadata.$ctor2(Bridge.box(315.0, System.Double, System.Double.format, System.Double.getHashCode), System.Windows.Media.Effects.DropShadowEffect.ManageChange));
                    this.OpacityProperty = Windows.UI.Xaml.DependencyProperty.Register("Opacity", System.Double, System.Windows.Media.Effects.DropShadowEffect, new Windows.UI.Xaml.PropertyMetadata.$ctor2(Bridge.box(1.0, System.Double, System.Double.format, System.Double.getHashCode), System.Windows.Media.Effects.DropShadowEffect.ManageChange));
                    this.ShadowDepthProperty = Windows.UI.Xaml.DependencyProperty.Register("ShadowDepth", System.Double, System.Windows.Media.Effects.DropShadowEffect, new Windows.UI.Xaml.PropertyMetadata.$ctor2(Bridge.box(5.0, System.Double, System.Double.format, System.Double.getHashCode), System.Windows.Media.Effects.DropShadowEffect.ManageChange));
                }
            },
            methods: {
                ManageChange: function (d, e) {
                    Bridge.cast(d, System.Windows.Media.Effects.DropShadowEffect).RefreshEffect();
                }
            }
        },
        props: {
            /**
             * Gets or sets a value that indicates the radius of the shadow's blur effect.
             *
             * @instance
             * @public
             * @memberof System.Windows.Media.Effects.DropShadowEffect
             * @function BlurRadius
             * @type number
             */
            BlurRadius: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(System.Windows.Media.Effects.DropShadowEffect.BlurRadiusProperty), System.Double), System.Double));
                },
                set: function (value) {
                    this.SetValue(System.Windows.Media.Effects.DropShadowEffect.BlurRadiusProperty, Bridge.box(value, System.Double, System.Double.format, System.Double.getHashCode));
                }
            },
            /**
             * Gets or sets the color of the drop shadow.
             *
             * @instance
             * @public
             * @memberof System.Windows.Media.Effects.DropShadowEffect
             * @function Color
             * @type Windows.UI.Color
             */
            Color: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(System.Windows.Media.Effects.DropShadowEffect.ColorProperty), Windows.UI.Color), Windows.UI.Color));
                },
                set: function (value) {
                    this.SetValue(System.Windows.Media.Effects.DropShadowEffect.ColorProperty, value.$clone());
                }
            },
            /**
             * Gets or sets the direction of the drop shadow.
             *
             * @instance
             * @public
             * @memberof System.Windows.Media.Effects.DropShadowEffect
             * @function Direction
             * @type number
             */
            Direction: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(System.Windows.Media.Effects.DropShadowEffect.DirectionProperty), System.Double), System.Double));
                },
                set: function (value) {
                    this.SetValue(System.Windows.Media.Effects.DropShadowEffect.DirectionProperty, Bridge.box(value, System.Double, System.Double.format, System.Double.getHashCode));
                }
            },
            /**
             * Gets or sets the opacity of the drop shadow.
             *
             * @instance
             * @public
             * @memberof System.Windows.Media.Effects.DropShadowEffect
             * @function Opacity
             * @type number
             */
            Opacity: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(System.Windows.Media.Effects.DropShadowEffect.OpacityProperty), System.Double), System.Double));
                },
                set: function (value) {
                    this.SetValue(System.Windows.Media.Effects.DropShadowEffect.OpacityProperty, Bridge.box(value, System.Double, System.Double.format, System.Double.getHashCode));
                }
            },
            /**
             * // Gets or sets a value that indicates whether the system renders the drop shadow
            // with emphasis on speed or quality.
            //
             * // Identifies the System.Windows.Media.Effects.DropShadowEffect.RenderingBias
            // dependency property.
            //
             * Gets or sets the distance of the drop shadow below the texture.
             *
             * @instance
             * @public
             * @memberof System.Windows.Media.Effects.DropShadowEffect
             * @function ShadowDepth
             * @type number
             */
            ShadowDepth: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(System.Windows.Media.Effects.DropShadowEffect.ShadowDepthProperty), System.Double), System.Double));
                },
                set: function (value) {
                    this.SetValue(System.Windows.Media.Effects.DropShadowEffect.ShadowDepthProperty, Bridge.box(value, System.Double, System.Double.format, System.Double.getHashCode));
                }
            }
        },
        ctors: {
            /**
             * Initializes a new instance of the System.Windows.Media.Effects.DropShadowEffect
             class.
             *
             * @instance
             * @public
             * @this System.Windows.Media.Effects.DropShadowEffect
             * @memberof System.Windows.Media.Effects.DropShadowEffect
             * @return  {void}
             */
            ctor: function () {
                this.$initialize();
                System.Windows.Media.Effects.Effect.ctor.call(this);

            }
        },
        methods: {
            SetParentUIElement: function (newParent) {
                System.Windows.Media.Effects.Effect.prototype.SetParentUIElement.call(this, newParent);
                this.RefreshEffect();
            },
            RefreshEffect: function () {
                if (this._parentUIElement != null && CSHTML5.Internal.INTERNAL_VisualTreeManager.IsElementInVisualTree(this._parentUIElement)) {
                    var domStyle = CSHTML5.Internal.INTERNAL_HtmlDomManager.GetFrameworkElementOuterStyleForModification(this._parentUIElement);
                    var x = { }, y = { };
                    this.GetXYPositionFromDirectionAndDepth(x, y);
                    var opacity = this.Opacity;
                    if (opacity > 1.0) {
                        opacity = 1.0;
                    } else {
                        if (opacity < 0.0) {
                            opacity = 0.0;
                        }
                    }
                    if (Bridge.is(this._parentUIElement, Windows.UI.Xaml.Controls.TextBlock)) {
                        domStyle.textShadow = (System.Double.format(x.v) || "") + "px " + (System.Double.format(y.v) || "") + "px " + (System.Double.format(this.BlurRadius) || "") + "px " + (Windows.UI.Color.FromArgb(System.Convert.toByte(Bridge.box(opacity * 255.0, System.Double, System.Double.format, System.Double.getHashCode)), this.Color.R, this.Color.G, this.Color.B).INTERNAL_ToHtmlString(1.0) || "");

                    } else {
                        domStyle.boxShadow = (System.Double.format(x.v) || "") + "px " + (System.Double.format(y.v) || "") + "px " + (System.Double.format(this.BlurRadius) || "") + "px " + (Windows.UI.Color.FromArgb(System.Convert.toByte(Bridge.box(opacity * 255.0, System.Double, System.Double.format, System.Double.getHashCode)), this.Color.R, this.Color.G, this.Color.B).INTERNAL_ToHtmlString(1.0) || "");

                        domStyle.borderCollapse = "separate";
                        domStyle.borderSpacing = "0px";
                    }
                }
            },
            GetXYPositionFromDirectionAndDepth: function (x, y) {
                x.v = Math.cos(this.Direction * Math.PI / 180.0) * this.ShadowDepth;
                y.v = -(Math.sin(this.Direction * Math.PI / 180.0) * this.ShadowDepth);
            }
        }
    });

    
    Bridge.define("System.Xml.Linq.XContainer", {
        inherits: [System.Xml.Linq.XNode],
        props: {
            /**
             * Get the first child node of this node.
             *
             * @instance
             * @public
             * @readonly
             * @memberof System.Xml.Linq.XContainer
             * @function FirstNode
             * @type System.Xml.Linq.XNode
             */
            FirstNode: {
                get: function () {
                    var jsNodeForElement = this.INTERNAL_jsnode.firstChild;
                    if (System.Xml.Linq.XDocument.IsNullOrUndefined(jsNodeForElement)) {
                        return null;
                    }
                    return System.Xml.Linq.XDocument.GetXNodeFromJSNode(jsNodeForElement);
                }
            }
        },
        methods: {
            /**
             * Returns a collection of the child nodes of this element or document, in document
             order.
             *
             * @instance
             * @public
             * @this System.Xml.Linq.XContainer
             * @memberof System.Xml.Linq.XContainer
             * @return  {System.Collections.Generic.IEnumerable$1}        An System.Collections.Generic.IEnumerable&lt;T&gt; of System.Xml.Linq.XNode containing
             the contents of this System.Xml.Linq.XContainer, in document order.
             */
            Nodes: function () {
                return new (Bridge.GeneratorEnumerable$1(System.Xml.Linq.XNode))(Bridge.fn.bind(this, function ()  {
                    var $step = 0,
                        $jumpFromFinally,
                        $returnValue,
                        i,
                        nodeAtI,
                        $async_e;

                    var $enumerator = new (Bridge.GeneratorEnumerator$1(System.Xml.Linq.XNode))(Bridge.fn.bind(this, function () {
                        try {
                            for (;;) {
                                switch ($step) {
                                    case 0: {
                                        i = 0;
                                        $step = 1;
                                        continue;
                                    }
                                    case 1: {
                                        if ( true ) {
                                                $step = 2;
                                                continue;
                                            } 
                                            $step = 4;
                                            continue;
                                    }
                                    case 2: {
                                        nodeAtI = this.INTERNAL_jsnode.childNodes[Bridge.box(i, System.Int32)];
                                            if ((typeof(nodeAtI) === 'undefined')) {
                                                $step = 4;
                                                continue;
                                            }

                                            $enumerator.current = System.Xml.Linq.XDocument.GetXNodeFromJSNode(nodeAtI);
                                            $step = 3;
                                            return true;
                                    }
                                    case 3: {
                                        i = (i + 1) | 0;

                                            $step = 1;
                                            continue;
                                    }
                                    case 4: {
                                        return false;
                                    }
                                    default: {
                                        return false;
                                    }
                                }
                            }
                        } catch($async_e1) {
                            $async_e = System.Exception.create($async_e1);
                            throw $async_e;
                        }
                    }));
                    return $enumerator;
                }));
            },
            /**
             * Adds the specified content as children of this System.Xml.Linq.XContainer.
             *
             * @instance
             * @public
             * @this System.Xml.Linq.XContainer
             * @memberof System.Xml.Linq.XContainer
             * @param   {System.Object}    content    A content object containing simple content or a collection of content objects
             to be added.
             * @return  {void}
             */
            Add: function (content) {
                if (Bridge.is(content, System.Xml.Linq.XNode)) {
                    var contentAsXNode = Bridge.cast(content, System.Xml.Linq.XNode);
                    if (contentAsXNode.HasParent()) {
                        contentAsXNode = contentAsXNode.Clone();
                    }
                    this.INTERNAL_jsnode.appendChild(contentAsXNode.INTERNAL_jsnode);
                } else if (Bridge.is(content, System.Xml.Linq.XAttribute)) {
                    var contentAsXAttribute = Bridge.cast(content, System.Xml.Linq.XAttribute);
                    if (contentAsXAttribute.INTERNAL_containerJSNode == null) {
                        contentAsXAttribute.INTERNAL_containerJSNode = this.INTERNAL_jsnode;
                    } else {
                        var attributeCopy = new System.Xml.Linq.XAttribute.ctor(contentAsXAttribute);
                        attributeCopy.INTERNAL_containerJSNode = this.INTERNAL_jsnode;
                    }


                    if (System.String.isNullOrWhiteSpace(contentAsXAttribute.Name.NamespaceName)) {
                        this.INTERNAL_jsnode.setAttribute(contentAsXAttribute._nameWithPrefix.LocalName, contentAsXAttribute.Value);
                    } else {
                        var attributeName = contentAsXAttribute._nameWithPrefix;
                        var attributeLocalName = attributeName.LocalName;
                        if (Bridge.referenceEquals(attributeName.NamespaceName, System.Runtime.Serialization.DataContractSerializer_Helpers.XMLNS_NAMESPACE) && !System.String.startsWith(attributeName.LocalName, "xmlns:")) {
                            attributeLocalName = "xmlns:" + (attributeLocalName || "");
                        }
                        this.INTERNAL_jsnode.setAttributeNS(attributeName.NamespaceName, attributeLocalName, contentAsXAttribute.Value);
                    }
                }
            },
            /**
             * Gets the first (in document order) child element with the specified System.Xml.Linq.XName.
             *
             * @instance
             * @public
             * @this System.Xml.Linq.XContainer
             * @memberof System.Xml.Linq.XContainer
             * @param   {System.Xml.Linq.XName}       name    The System.Xml.Linq.XName to match.
             * @return  {System.Xml.Linq.XElement}            A System.Xml.Linq.XElement that matches the specified System.Xml.Linq.XName,
             or null.
             */
            Element: function (name) {


                var nameAsString = name.LocalName;
                var jsNodeForElement = Array.from(this.INTERNAL_jsnode.childNodes).filter(function(node) { return node.tagName == nameAsString})[0];

                if (System.Xml.Linq.XDocument.IsNullOrUndefined(jsNodeForElement)) {
                    return null;
                }

                var node = System.Xml.Linq.XDocument.GetXNodeFromJSNode(jsNodeForElement);
                if (Bridge.is(node, System.Xml.Linq.XElement)) {
                    return Bridge.cast(node, System.Xml.Linq.XElement);
                }
                return null;
            },
            /**
             * Returns a collection of the child elements of this element or document, in
             document order.
             *
             * @instance
             * @public
             * @this System.Xml.Linq.XContainer
             * @memberof System.Xml.Linq.XContainer
             * @return  {System.Collections.Generic.IEnumerable$1}        An System.Collections.Generic.IEnumerable&lt;XElement&gt; of System.Xml.Linq.XElement
             containing the child elements of this System.Xml.Linq.XContainer, in document
             order.
             */
            Elements: function () {
                return new (Bridge.GeneratorEnumerable$1(System.Xml.Linq.XElement))(Bridge.fn.bind(this, function ()  {
                    var $step = 0,
                        $jumpFromFinally,
                        $returnValue,
                        jsNodesForElement,
                        i,
                        nodeAtI,
                        $async_e;

                    var $enumerator = new (Bridge.GeneratorEnumerator$1(System.Xml.Linq.XElement))(Bridge.fn.bind(this, function () {
                        try {
                            for (;;) {
                                switch ($step) {
                                    case 0: {
                                        jsNodesForElement = Array.from(this.INTERNAL_jsnode.childNodes).filter(function(node) { return node.tagName !== undefined});

                                            if (System.Xml.Linq.XDocument.IsNullOrUndefined(jsNodesForElement)) {
                                                $step = 1;
                                                continue;
                                            } 
                                            $step = 2;
                                            continue;
                                    }
                                    case 1: {
                                        return false;
                                    }
                                    case 2: {
                                        i = 0;
                                        $step = 3;
                                        continue;
                                    }
                                    case 3: {
                                        if ( true ) {
                                                $step = 4;
                                                continue;
                                            } 
                                            $step = 6;
                                            continue;
                                    }
                                    case 4: {
                                        nodeAtI = jsNodesForElement[Bridge.box(i, System.Int32)];
                                            if ((typeof(nodeAtI) === 'undefined')) {
                                                $step = 6;
                                                continue;
                                            }

                                            $enumerator.current = Bridge.cast(System.Xml.Linq.XDocument.GetXNodeFromJSNode(nodeAtI), System.Xml.Linq.XElement);
                                            $step = 5;
                                            return true;
                                    }
                                    case 5: {
                                        i = (i + 1) | 0;

                                            $step = 3;
                                            continue;
                                    }
                                    case 6: {
                                        return false;
                                    }
                                    default: {
                                        return false;
                                    }
                                }
                            }
                        } catch($async_e1) {
                            $async_e = System.Exception.create($async_e1);
                            throw $async_e;
                        }
                    }));
                    return $enumerator;
                }));
            },
            /**
             * Returns a filtered collection of the child elements of this element or document,
             in document order. Only elements that have a matching System.Xml.Linq.XName
             are included in the collection.
             *
             * @instance
             * @public
             * @this System.Xml.Linq.XContainer
             * @memberof System.Xml.Linq.XContainer
             * @param   {System.Xml.Linq.XName}                       name    The System.Xml.Linq.XName to match.
             * @return  {System.Collections.Generic.IEnumerable$1}            An System.Collections.Generic.IEnumerable`1 of System.Xml.Linq.XElement
             containing the children of the System.Xml.Linq.XContainer that have a matching
             System.Xml.Linq.XName, in document order.
             */
            Elements$1: function (name) {
                return new (Bridge.GeneratorEnumerable$1(System.Xml.Linq.XElement))(Bridge.fn.bind(this, function (name) {
                    var $step = 0,
                        $jumpFromFinally,
                        $returnValue,
                        jsNodesForElement,
                        i,
                        nodeAtI,
                        $async_e;

                    var $enumerator = new (Bridge.GeneratorEnumerator$1(System.Xml.Linq.XElement))(Bridge.fn.bind(this, function () {
                        try {
                            for (;;) {
                                switch ($step) {
                                    case 0: {
                                        jsNodesForElement = Array.from(this.INTERNAL_jsnode.childNodes).filter(document.functionToCompareWordForFilter(name.toString()));

                                            if (System.Xml.Linq.XDocument.IsNullOrUndefined(jsNodesForElement)) {
                                                $step = 1;
                                                continue;
                                            } 
                                            $step = 2;
                                            continue;
                                    }
                                    case 1: {
                                        return false;
                                    }
                                    case 2: {
                                        i = 0;
                                        $step = 3;
                                        continue;
                                    }
                                    case 3: {
                                        if ( true ) {
                                                $step = 4;
                                                continue;
                                            } 
                                            $step = 6;
                                            continue;
                                    }
                                    case 4: {
                                        nodeAtI = jsNodesForElement[Bridge.box(i, System.Int32)];
                                            if ((typeof(nodeAtI) === 'undefined')) {
                                                $step = 6;
                                                continue;
                                            }

                                            $enumerator.current = Bridge.cast(System.Xml.Linq.XDocument.GetXNodeFromJSNode(nodeAtI), System.Xml.Linq.XElement);
                                            $step = 5;
                                            return true;
                                    }
                                    case 5: {
                                        i = (i + 1) | 0;

                                            $step = 3;
                                            continue;
                                    }
                                    case 6: {
                                        return false;
                                    }
                                    default: {
                                        return false;
                                    }
                                }
                            }
                        } catch($async_e1) {
                            $async_e = System.Exception.create($async_e1);
                            throw $async_e;
                        }
                    }));
                    return $enumerator;
                }, arguments));
            },
            /**
             * Returns a collection of the descendant elements for this document or element,
             in document order.
             *
             * @instance
             * @public
             * @this System.Xml.Linq.XContainer
             * @memberof System.Xml.Linq.XContainer
             * @return  {System.Collections.Generic.IEnumerable$1}        An System.Collections.Generic.IEnumerable`1 of System.Xml.Linq.XElement
             containing the descendant elements of the System.Xml.Linq.XContainer.
             */
            Descendants: function () {
                return new (Bridge.GeneratorEnumerable$1(System.Xml.Linq.XElement))(Bridge.fn.bind(this, function ()  {
                    var $step = 0,
                        $jumpFromFinally,
                        $returnValue,
                        $t,
                        node,
                        nodeAsXElement,
                        $t1,
                        descendantElement,
                        $async_e;

                    var $enumerator = new (Bridge.GeneratorEnumerator$1(System.Xml.Linq.XElement))(Bridge.fn.bind(this, function () {
                        try {
                            for (;;) {
                                switch ($step) {
                                    case 0: {
                                        $t = Bridge.getEnumerator(this.Nodes(), System.Xml.Linq.XNode);
                                            $step = 1;
                                            continue;
                                    }
                                    case 1: {
                                        if ($t.moveNext()) {
                                                node = $t.Current;
                                                $step = 2;
                                                continue;
                                            }
                                        $step = 10;
                                        continue;
                                    }
                                    case 2: {
                                        if (Bridge.is(node, System.Xml.Linq.XElement)) {
                                                $step = 3;
                                                continue;
                                            } 
                                            $step = 9;
                                            continue;
                                    }
                                    case 3: {
                                        nodeAsXElement = Bridge.cast(node, System.Xml.Linq.XElement);
                                            $enumerator.current = nodeAsXElement;
                                            $step = 4;
                                            return true;
                                    }
                                    case 4: {
                                        $t1 = Bridge.getEnumerator(nodeAsXElement.Descendants(), System.Xml.Linq.XElement);
                                            $step = 5;
                                            continue;
                                    }
                                    case 5: {
                                        if ($t1.moveNext()) {
                                                descendantElement = $t1.Current;
                                                $step = 6;
                                                continue;
                                            }
                                        $step = 8;
                                        continue;
                                    }
                                    case 6: {
                                        $enumerator.current = descendantElement;
                                            $step = 7;
                                            return true;
                                    }
                                    case 7: {
                                        $step = 5;
                                        continue;
                                    }
                                    case 8: {
                                        $step = 9;
                                        continue;
                                    }
                                    case 9: {
                                        $step = 1;
                                        continue;
                                    }
                                    case 10: {
                                        return false;
                                    }
                                    default: {
                                        return false;
                                    }
                                }
                            }
                        } catch($async_e1) {
                            $async_e = System.Exception.create($async_e1);
                            throw $async_e;
                        }
                    }));
                    return $enumerator;
                }));
            },
            /**
             * Returns a filtered collection of the descendant elements for this document
             or element, in document order. Only elements that have a matching System.Xml.Linq.XName
             are included in the collection.
             *
             * @instance
             * @public
             * @this System.Xml.Linq.XContainer
             * @memberof System.Xml.Linq.XContainer
             * @param   {System.Xml.Linq.XName}                       name    The System.Xml.Linq.XName to match.
             * @return  {System.Collections.Generic.IEnumerable$1}            An System.Collections.Generic.IEnumerable`1 of System.Xml.Linq.XElement
             containing the descendant elements of the System.Xml.Linq.XContainer that
             match the specified System.Xml.Linq.XName.
             */
            Descendants$1: function (name) {
                return new (Bridge.GeneratorEnumerable$1(System.Xml.Linq.XElement))(Bridge.fn.bind(this, function (name) {
                    var $step = 0,
                        $jumpFromFinally,
                        $returnValue,
                        jsNodesForElement,
                        i,
                        nodeAtI,
                        $async_e;

                    var $enumerator = new (Bridge.GeneratorEnumerator$1(System.Xml.Linq.XElement))(Bridge.fn.bind(this, function () {
                        try {
                            for (;;) {
                                switch ($step) {
                                    case 0: {
                                        jsNodesForElement = this.INTERNAL_jsnode.getElementsByTagName(name.toString());

                                            if (System.Xml.Linq.XDocument.IsNullOrUndefined(jsNodesForElement)) {
                                                $step = 1;
                                                continue;
                                            } 
                                            $step = 2;
                                            continue;
                                    }
                                    case 1: {
                                        return false;
                                    }
                                    case 2: {
                                        i = 0;
                                        $step = 3;
                                        continue;
                                    }
                                    case 3: {
                                        if ( true ) {
                                                $step = 4;
                                                continue;
                                            } 
                                            $step = 6;
                                            continue;
                                    }
                                    case 4: {
                                        nodeAtI = jsNodesForElement[Bridge.box(i, System.Int32)];
                                            if ((typeof(nodeAtI) === 'undefined')) {
                                                $step = 6;
                                                continue;
                                            }

                                            $enumerator.current = Bridge.cast(System.Xml.Linq.XDocument.GetXNodeFromJSNode(nodeAtI), System.Xml.Linq.XElement);
                                            $step = 5;
                                            return true;
                                    }
                                    case 5: {
                                        i = (i + 1) | 0;

                                            $step = 3;
                                            continue;
                                    }
                                    case 6: {
                                        return false;
                                    }
                                    default: {
                                        return false;
                                    }
                                }
                            }
                        } catch($async_e1) {
                            $async_e = System.Exception.create($async_e1);
                            throw $async_e;
                        }
                    }));
                    return $enumerator;
                }, arguments));
            }
        }
    });

    /**
     * Represents a text node.
     * // Initializes a new instance of the System.Xml.Linq.XText class from another
    // System.Xml.Linq.XText object.
    //
     * // Gets the node type for this node.
    //
     * // Writes this node to an System.Xml.XmlWriter.
    //
     *
     * @public
     * @class System.Xml.Linq.XText
     * @augments System.Xml.Linq.XNode
     */
    Bridge.define("System.Xml.Linq.XText", {
        inherits: [System.Xml.Linq.XNode],
        fields: {
            _value: null
        },
        props: {
            /**
             * Gets or sets the value of this node.
             *
             * @instance
             * @public
             * @memberof System.Xml.Linq.XText
             * @function Value
             * @type string
             */
            Value: {
                get: function () {
                    return this._value;
                },
                set: function (value) {
                    this._value = value;
                    this.INTERNAL_jsnode.nodeValue = value;
                }
            }
        },
        ctors: {
            ctor: function (jsNode) {
                this.$initialize();
                System.Xml.Linq.XNode.ctor.call(this);
                this.INTERNAL_jsnode = jsNode;
                this._value = System.Convert.toString(jsNode.wholeText);
            },
            /**
             * Initializes a new instance of the System.Xml.Linq.XText class.
             *
             * @instance
             * @public
             * @this System.Xml.Linq.XText
             * @memberof System.Xml.Linq.XText
             * @param   {string}    value
             * @return  {void}
             */
            $ctor1: function (value) {
                this.$initialize();
                System.Xml.Linq.XNode.ctor.call(this);
                this.INTERNAL_jsnode = document.createTextNode(value);
                this._value = value;
            }
        },
        methods: {
            Clone: function () {
                return new System.Xml.Linq.XText.$ctor1(this.Value);
            }
        }
    });

    Bridge.define("TypeScriptDefinitionsSupport.JSArray$1", function (T) { return {
        inherits: [TypeScriptDefinitionsSupport.JSObject,System.Collections.Generic.IList$1(T)],
        statics: {
            methods: {
                FromJavaScriptInstance: function (jsObj) {
                    return new (TypeScriptDefinitionsSupport.JSArray$1(T)).$ctor1(jsObj);
                }
            }
        },
        props: {
            Count: {
                get: function () {
                    return System.Convert.toInt32(this.UnderlyingJSInstance.length);
                }
            },
            IsReadOnly: {
                get: function () {
                    return false;
                }
            }
        },
        alias: [
            "Count", "System$Collections$Generic$ICollection$1$" + Bridge.getTypeAlias(T) + "$Count",
            "IsReadOnly", "System$Collections$Generic$ICollection$1$" + Bridge.getTypeAlias(T) + "$IsReadOnly",
            "getItem", "System$Collections$Generic$IList$1$" + Bridge.getTypeAlias(T) + "$getItem",
            "setItem", "System$Collections$Generic$IList$1$" + Bridge.getTypeAlias(T) + "$setItem",
            "add", "System$Collections$Generic$ICollection$1$" + Bridge.getTypeAlias(T) + "$add",
            "clear", "System$Collections$Generic$ICollection$1$" + Bridge.getTypeAlias(T) + "$clear",
            "contains", "System$Collections$Generic$ICollection$1$" + Bridge.getTypeAlias(T) + "$contains",
            "copyTo", "System$Collections$Generic$ICollection$1$" + Bridge.getTypeAlias(T) + "$copyTo",
            "GetEnumerator", ["System$Collections$Generic$IEnumerable$1$" + Bridge.getTypeAlias(T) + "$GetEnumerator", "System$Collections$Generic$IEnumerable$1$GetEnumerator"],
            "indexOf", "System$Collections$Generic$IList$1$" + Bridge.getTypeAlias(T) + "$indexOf",
            "insert", "System$Collections$Generic$IList$1$" + Bridge.getTypeAlias(T) + "$insert",
            "remove", "System$Collections$Generic$ICollection$1$" + Bridge.getTypeAlias(T) + "$remove",
            "removeAt", "System$Collections$Generic$IList$1$" + Bridge.getTypeAlias(T) + "$removeAt"
        ],
        ctors: {
            ctor: function () {
                this.$initialize();
                TypeScriptDefinitionsSupport.JSObject.ctor.call(this);
                this.UnderlyingJSInstance = [];
            },
            $ctor1: function (jsObj) {
                this.$initialize();
                TypeScriptDefinitionsSupport.JSObject.ctor.call(this);
                this.UnderlyingJSInstance = jsObj;
            }
        },
        methods: {
            getItem: function (index) {
                if (index < 0 || index >= this.Count) {
                    throw new System.ArgumentOutOfRangeException.ctor();
                }

                var value = this.UnderlyingJSInstance[Bridge.box(index, System.Int32)];
                if (Bridge.Reflection.isAssignableFrom(TypeScriptDefinitionsSupport.IJSObject, T)) {
                    var o = Bridge.cast(Bridge.createInstance(T), TypeScriptDefinitionsSupport.IJSObject);
                    o.TypeScriptDefinitionsSupport$IJSObject$UnderlyingJSInstance = value;
                    return Bridge.cast(Bridge.unbox(o, T), T);
                } else {
                    return Bridge.cast(Bridge.unbox(value, T), T);
                }
            },
            setItem: function (index, value) {
                if (index < 0 || index >= this.Count) {
                    throw new System.ArgumentOutOfRangeException.ctor();
                }

                var _value;

                if (Bridge.Reflection.isAssignableFrom(TypeScriptDefinitionsSupport.IJSObject, T)) {
                    _value = Bridge.cast(value, TypeScriptDefinitionsSupport.IJSObject).TypeScriptDefinitionsSupport$IJSObject$UnderlyingJSInstance;
                } else {
                    _value = value;
                }
                this.UnderlyingJSInstance[Bridge.box(index, System.Int32)] = _value;
            },
            add: function (item) {
                var value;

                if (item == null) {
                    value = null;
                } else {
                    if (Bridge.Reflection.isAssignableFrom(TypeScriptDefinitionsSupport.IJSObject, T)) {
                        value = Bridge.cast(item, TypeScriptDefinitionsSupport.IJSObject).TypeScriptDefinitionsSupport$IJSObject$UnderlyingJSInstance;
                    } else {
                        value = item;
                    }
                }

                this.UnderlyingJSInstance.push(value);
            },
            clear: function () {
                this.UnderlyingJSInstance.splice(0, this.UnderlyingJSInstance.length);
            },
            contains: function (item) {
                var value;

                if (item == null) {
                    value = null;
                } else {
                    if (Bridge.Reflection.isAssignableFrom(TypeScriptDefinitionsSupport.IJSObject, T)) {
                        value = Bridge.cast(item, TypeScriptDefinitionsSupport.IJSObject).TypeScriptDefinitionsSupport$IJSObject$UnderlyingJSInstance;
                    } else {
                        value = item;
                    }
                }

                var index = System.Convert.toInt32(this.UnderlyingJSInstance.indexOf(value));

                return index !== -1;
            },
            copyTo: function (array, arrayIndex) {
                if (array == null) {
                    throw new System.ArgumentNullException.ctor();
                }
                if (arrayIndex < 0) {
                    throw new System.ArgumentOutOfRangeException.ctor();
                }
                if (this.Count > ((array.length - arrayIndex) | 0)) {
                    throw new System.ArgumentException.ctor();
                }

                for (var i = 0; i < this.Count; i = (i + 1) | 0) {
                    array[System.Array.index(((arrayIndex + i) | 0), array)] = this.getItem(i);
                }
            },
            GetEnumerator: function () {
                var $step = 0,
                    $jumpFromFinally,
                    $returnValue,
                    i,
                    value,
                    o,
                    i1,
                    value1,
                    $async_e;

                var $enumerator = new (Bridge.GeneratorEnumerator$1(T))(Bridge.fn.bind(this, function () {
                    try {
                        for (;;) {
                            switch ($step) {
                                case 0: {
                                    if (Bridge.Reflection.isAssignableFrom(TypeScriptDefinitionsSupport.IJSObject, T)) {
                                            $step = 1;
                                            continue;
                                        } else  {
                                            $step = 7;
                                            continue;
                                        }
                                }
                                case 1: {
                                    i = 0;
                                        $step = 2;
                                        continue;
                                }
                                case 2: {
                                    if ( i < this.Count ) {
                                            $step = 3;
                                            continue;
                                        }
                                    $step = 6;
                                    continue;
                                }
                                case 3: {
                                    value = this.UnderlyingJSInstance[Bridge.box(i, System.Int32)];
                                        o = Bridge.cast(Bridge.createInstance(T), TypeScriptDefinitionsSupport.IJSObject);
                                        o.TypeScriptDefinitionsSupport$IJSObject$UnderlyingJSInstance = value;
                                        $enumerator.current = Bridge.cast(Bridge.unbox(o, T), T);
                                        $step = 4;
                                        return true;
                                }
                                case 4: {
                                    $step = 5;
                                    continue;
                                }
                                case 5: {
                                    i = (i + 1) | 0;
                                    $step = 2;
                                    continue;
                                }
                                case 6: {
                                    $step = 13;
                                    continue;
                                }
                                case 7: {
                                    i1 = 0;
                                        $step = 8;
                                        continue;
                                }
                                case 8: {
                                    if ( i1 < this.Count ) {
                                            $step = 9;
                                            continue;
                                        }
                                    $step = 12;
                                    continue;
                                }
                                case 9: {
                                    value1 = this.UnderlyingJSInstance[Bridge.box(i1, System.Int32)];
                                        $enumerator.current = TypeScriptDefinitionsSupport.JSObject.Helper_ConvertTo(T, value1);
                                        $step = 10;
                                        return true;
                                }
                                case 10: {
                                    $step = 11;
                                    continue;
                                }
                                case 11: {
                                    i1 = (i1 + 1) | 0;
                                    $step = 8;
                                    continue;
                                }
                                case 12: {
                                    $step = 13;
                                    continue;
                                }
                                case 13: {

                                }
                                default: {
                                    return false;
                                }
                            }
                        }
                    } catch($async_e1) {
                        $async_e = System.Exception.create($async_e1);
                        throw $async_e;
                    }
                }));
                return $enumerator;
            },
            System$Collections$IEnumerable$GetEnumerator: function () {
                return this.GetEnumerator();
            },
            indexOf: function (item) {
                var value;

                if (item == null) {
                    value = null;
                } else {
                    if (Bridge.Reflection.isAssignableFrom(TypeScriptDefinitionsSupport.IJSObject, T)) {
                        value = Bridge.cast(item, TypeScriptDefinitionsSupport.IJSObject).TypeScriptDefinitionsSupport$IJSObject$UnderlyingJSInstance;
                    } else {
                        value = item;
                    }
                }

                return System.Convert.toInt32(this.UnderlyingJSInstance.indexOf(value));
            },
            insert: function (index, item) {
                if (index < 0 || index >= this.Count) {
                    throw new System.ArgumentOutOfRangeException.ctor();
                }

                var value;

                if (item == null) {
                    value = null;
                } else {
                    if (Bridge.Reflection.isAssignableFrom(TypeScriptDefinitionsSupport.IJSObject, T)) {
                        value = Bridge.cast(item, TypeScriptDefinitionsSupport.IJSObject).TypeScriptDefinitionsSupport$IJSObject$UnderlyingJSInstance;
                    } else {
                        value = item;
                    }
                }

                this.UnderlyingJSInstance.splice(Bridge.box(index, System.Int32), 0, value);
            },
            remove: function (item) {
                var value;

                if (item == null) {
                    value = null;
                } else {
                    if (Bridge.Reflection.isAssignableFrom(TypeScriptDefinitionsSupport.IJSObject, T)) {
                        value = Bridge.cast(item, TypeScriptDefinitionsSupport.IJSObject).TypeScriptDefinitionsSupport$IJSObject$UnderlyingJSInstance;
                    } else {
                        value = item;
                    }
                }

                var index = System.Convert.toInt32(this.UnderlyingJSInstance.indexOf(value));

                if (index !== -1) {
                    this.UnderlyingJSInstance.splice(Bridge.box(index, System.Int32), 1);
                }

                return index !== -1;
            },
            removeAt: function (index) {
                if (index < 0 || index >= this.Count) {
                    throw new System.ArgumentOutOfRangeException.ctor();
                }

                this.UnderlyingJSInstance.splice(Bridge.box(index, System.Int32), 1);
            }
        }
    }; });

    Bridge.define("TypeScriptDefinitionsSupport.Promise", {
        inherits: [TypeScriptDefinitionsSupport.JSObject],
        methods: {
            then: function (onFulfilled, onRejected) {
                this.UnderlyingJSInstance.then(onFulfilled, onRejected);
            }
        }
    });

    Bridge.define("TypeScriptDefinitionsSupport.Promise$1", function (T) { return {
        inherits: [TypeScriptDefinitionsSupport.JSObject],
        methods: {
            then: function (onFulfilled, onRejected) {
                this.UnderlyingJSInstance.then(onFulfilled, onRejected);
            }
        }
    }; });

    Bridge.define("TypeScriptDefinitionsSupport.UnionType", {
        inherits: [TypeScriptDefinitionsSupport.JSObject],
        fields: {
            _Type: null
        },
        methods: {
            instanceof: function (type) {
                return System.Convert.toBoolean(this.UnderlyingJSInstance instanceof type);
            },
            CreateInstance: function (T) {
                if (Bridge.referenceEquals(T, System.Double)) {
                    return Bridge.cast(Bridge.unbox(Bridge.box(System.Convert.toDouble(this.UnderlyingJSInstance), System.Double, System.Double.format, System.Double.getHashCode), T), T);
                } else {
                    if (Bridge.referenceEquals(T, System.String)) {
                        return Bridge.cast(Bridge.unbox(System.Convert.toString(this.UnderlyingJSInstance), T), T);
                    } else {
                        if (Bridge.referenceEquals(T, System.Boolean)) {
                            return Bridge.cast(Bridge.unbox(Bridge.box(System.Convert.toBoolean(this.UnderlyingJSInstance), System.Boolean, System.Boolean.toString), T), T);
                        } else {
                            if (Bridge.referenceEquals(T, System.Object)) {
                                return Bridge.cast(Bridge.unbox(this.UnderlyingJSInstance, T), T);
                            } else {
                                return Bridge.cast(Bridge.unbox(Bridge.createInstance(T, [this.UnderlyingJSInstance]), T), T);
                            }
                        }
                    }
                }
            }
        }
    });

    /**
     * @public
     * @class Windows.UI.Xaml.Controls.ColumnDefinitionCollection
     * @augments System.Collections.ObjectModel.ObservableCollection$1
     */
    Bridge.define("Windows.UI.Xaml.Controls.ColumnDefinitionCollection", {
        inherits: [System.Collections.ObjectModel.ObservableCollection$1(Windows.UI.Xaml.Controls.ColumnDefinition)]
    });

    /**
     * Defines the element tree that is applied as the control template for a control.
     *
     * @public
     * @class Windows.UI.Xaml.Controls.ControlTemplate
     * @augments Windows.UI.Xaml.FrameworkTemplate
     */
    Bridge.define("Windows.UI.Xaml.Controls.ControlTemplate", {
        inherits: [Windows.UI.Xaml.FrameworkTemplate],
        fields: {
            /**
             * Gets or sets the type to which the ControlTemplate is applied.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.ControlTemplate
             * @function TargetType
             * @type System.Type
             */
            TargetType: null
        },
        ctors: {
            /**
             * Initializes a new instance of the ControlTemplate class.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Controls.ControlTemplate
             * @memberof Windows.UI.Xaml.Controls.ControlTemplate
             * @return  {void}
             */
            ctor: function () {
                this.$initialize();
                Windows.UI.Xaml.FrameworkTemplate.ctor.call(this);
            }
        }
    });

    /**
     * Serves as the base class for columns that can bind to a property in the data
     source of a System.Windows.Controls.DataGrid.
     *
     * @abstract
     * @public
     * @class Windows.UI.Xaml.Controls.DataGridBoundColumn
     * @augments Windows.UI.Xaml.Controls.DataGridColumn
     */
    Bridge.define("Windows.UI.Xaml.Controls.DataGridBoundColumn", {
        inherits: [Windows.UI.Xaml.Controls.DataGridColumn],
        fields: {
            _binding: null
        },
        props: {
            Binding: {
                get: function () {
                    return this._binding;
                },
                set: function (value) {
                    if (!Bridge.referenceEquals(this._binding, value)) {
                        var oldBinding = this._binding;
                        this._binding = value;
                        this.OnBindingChanged(oldBinding, this._binding);
                    }
                }
            }
        },
        methods: {
            OnBindingChanged: function (oldBinding, newBinding) { }
        }
    });

    Bridge.define("Windows.UI.Xaml.Controls.ItemCollection", {
        inherits: [Windows.UI.Xaml.PresentationFrameworkCollection$1(System.Object),System.Collections.Specialized.INotifyCollectionChanged],
        fields: {
            _monitor: null,
            _isUsingItemsSource: false,
            _itemsSource: null,
            _isUsingListWrapper: false,
            _listWrapper: null
        },
        events: {
            CollectionChanged: null
        },
        props: {
            IsFixedSizeImpl: {
                get: function () {
                    return this.IsUsingItemsSource;
                }
            },
            IsReadOnlyImpl: {
                get: function () {
                    return this.IsUsingItemsSource;
                }
            },
            IsUsingItemsSource: {
                get: function () {
                    return this._isUsingItemsSource;
                }
            },
            SourceCollection: {
                get: function () {
                    return this._itemsSource;
                }
            },
            SourceList: {
                get: function () {
                    if (this._isUsingListWrapper) {
                        return this._listWrapper;
                    }
                    return Bridge.cast(this._itemsSource, System.Collections.IList);
                }
            },
            CountInternal$1: {
                get: function () {
                    if (this.IsUsingItemsSource) {
                        return System.Array.getCount(this.SourceList);
                    } else {
                        return this.CountInternal;
                    }
                }
            }
        },
        alias: ["addCollectionChanged", "System$Collections$Specialized$INotifyCollectionChanged$addCollectionChanged",
        "removeCollectionChanged", "System$Collections$Specialized$INotifyCollectionChanged$removeCollectionChanged"],
        ctors: {
            init: function () {
                this._monitor = new Windows.UI.Xaml.Controls.ItemCollection.SimpleMonitor();
            },
            ctor: function () {
                this.$initialize();
                Windows.UI.Xaml.PresentationFrameworkCollection$1(System.Object).ctor.call(this);

            }
        },
        methods: {
            AddOverride: function (value) {
                this.CheckReentrancy();
                if (this.IsUsingItemsSource) {
                    throw new System.InvalidOperationException.$ctor1("Operation is not valid while ItemsSource is in use. Access and modify elements with ItemsControl.ItemsSource instead.");
                }
                this.AddInternal(value);
                this.OnCollectionChanged(new System.Collections.Specialized.NotifyCollectionChangedEventArgs.$ctor7(System.Collections.Specialized.NotifyCollectionChangedAction.Add, value, ((this.CountInternal$1 - 1) | 0)));
            },
            ClearOverride: function () {
                this.CheckReentrancy();
                if (this.IsUsingItemsSource) {
                    throw new System.InvalidOperationException.$ctor1("Operation is not valid while ItemsSource is in use. Access and modify elements with ItemsControl.ItemsSource instead.");
                }
                this.ClearInternal();
                this.OnCollectionChanged(new System.Collections.Specialized.NotifyCollectionChangedEventArgs.ctor(System.Collections.Specialized.NotifyCollectionChangedAction.Reset));
            },
            InsertOverride: function (index, value) {
                this.CheckReentrancy();
                if (this.IsUsingItemsSource) {
                    throw new System.InvalidOperationException.$ctor1("Operation is not valid while ItemsSource is in use. Access and modify elements with ItemsControl.ItemsSource instead.");
                }
                this.InsertInternal(index, value);
                this.OnCollectionChanged(new System.Collections.Specialized.NotifyCollectionChangedEventArgs.$ctor7(System.Collections.Specialized.NotifyCollectionChangedAction.Add, value, index));
            },
            RemoveAtOverride: function (index) {
                this.CheckReentrancy();
                if (this.IsUsingItemsSource) {
                    throw new System.InvalidOperationException.$ctor1("Operation is not valid while ItemsSource is in use. Access and modify elements with ItemsControl.ItemsSource instead.");
                }
                var removedItem = this.GetItemInternal(index);
                this.RemoveAtInternal(index);
                this.OnCollectionChanged(new System.Collections.Specialized.NotifyCollectionChangedEventArgs.$ctor7(System.Collections.Specialized.NotifyCollectionChangedAction.Remove, removedItem, index));
            },
            RemoveOverride: function (value) {
                this.CheckReentrancy();
                if (this.IsUsingItemsSource) {
                    throw new System.InvalidOperationException.$ctor1("Operation is not valid while ItemsSource is in use. Access and modify elements with ItemsControl.ItemsSource instead.");
                }
                var index = this.indexOf(value);
                if (index > -1) {
                    var oldItem = this.GetItemInternal(index);
                    this.RemoveInternal(value);
                    this.OnCollectionChanged(new System.Collections.Specialized.NotifyCollectionChangedEventArgs.$ctor7(System.Collections.Specialized.NotifyCollectionChangedAction.Remove, oldItem, index));
                    return true;
                }
                return false;
            },
            GetItemOverride: function (index) {
                if (this.IsUsingItemsSource) {
                    return System.Array.getItem(this.SourceList, index);
                }
                return this.GetItemInternal(index);
            },
            SetItemOverride: function (index, value) {
                this.CheckReentrancy();
                if (this.IsUsingItemsSource) {
                    throw new System.InvalidOperationException.$ctor1("Operation is not valid while ItemsSource is in use. Access and modify elements with ItemsControl.ItemsSource instead.");
                }
                var originalItem = this.GetItemInternal(index);
                this.SetItemInternal(index, value);
                this.OnCollectionChanged(new System.Collections.Specialized.NotifyCollectionChangedEventArgs.$ctor10(System.Collections.Specialized.NotifyCollectionChangedAction.Replace, originalItem, value, index));
            },
            ContainsImpl: function (value) {
                if (this.IsUsingItemsSource) {
                    return System.Array.contains(this.SourceList, value);
                }
                return Windows.UI.Xaml.PresentationFrameworkCollection$1(System.Object).prototype.ContainsImpl.call(this, value);
            },
            IndexOfImpl: function (value) {
                if (this.IsUsingItemsSource) {
                    return System.Array.indexOf(this.SourceList, value, 0, null);
                }
                return Windows.UI.Xaml.PresentationFrameworkCollection$1(System.Object).prototype.IndexOfImpl.call(this, value);
            },
            GetEnumeratorImpl: function () {
                if (this.IsUsingItemsSource) {
                    return this.GetEnumeratorPrivateItemsSourceOnly();
                }
                return Windows.UI.Xaml.PresentationFrameworkCollection$1(System.Object).prototype.GetEnumeratorImpl.call(this);
            },
            GetEnumeratorPrivateItemsSourceOnly: function () {
                var $step = 0,
                    $jumpFromFinally,
                    $returnValue,
                    enumerator,
                    $async_e;

                var $enumerator = new (Bridge.GeneratorEnumerator$1(System.Object))(Bridge.fn.bind(this, function () {
                    try {
                        for (;;) {
                            switch ($step) {
                                case 0: {
                                    enumerator = Bridge.getEnumerator(this.SourceList);
                                    $step = 1;
                                    continue;
                                }
                                case 1: {
                                    if ( enumerator.System$Collections$IEnumerator$moveNext() ) {
                                            $step = 2;
                                            continue;
                                        } 
                                        $step = 4;
                                        continue;
                                }
                                case 2: {
                                    $enumerator.current = enumerator.System$Collections$IEnumerator$Current;
                                        $step = 3;
                                        return true;
                                }
                                case 3: {
                                    
                                        $step = 1;
                                        continue;
                                }
                                case 4: {

                                }
                                default: {
                                    return false;
                                }
                            }
                        }
                    } catch($async_e1) {
                        $async_e = System.Exception.create($async_e1);
                        throw $async_e;
                    }
                }));
                return $enumerator;
            },
            OnCollectionChanged: function (e) {
                var $t;
                if (!Bridge.staticEquals(this.CollectionChanged, null)) {
                    $t = this.BlockReentrancy();
                    try {
                        this.CollectionChanged(this, e);
                    }
                    finally {
                        if (Bridge.hasValue($t)) {
                            $t.System$IDisposable$Dispose();
                        }
                    }
                }
            },
            /**
             * Disallow reentrant attempts to change this collection. E.g. a event handler
             of the CollectionChanged event is not allowed to make changes to this collection.
             *
             * @instance
             * @private
             * @this Windows.UI.Xaml.Controls.ItemCollection
             * @memberof Windows.UI.Xaml.Controls.ItemCollection
             * @return  {System.IDisposable}
             */
            BlockReentrancy: function () {
                this._monitor.Enter();
                return this._monitor;
            },
            /**
             * Check and assert for reentrant attempts to change this collection.
             *
             * @instance
             * @private
             * @this Windows.UI.Xaml.Controls.ItemCollection
             * @memberof Windows.UI.Xaml.Controls.ItemCollection
             * @throws raised when changing the collection
             while another collection change is still being notified to other listeners
             * @return  {void}
             */
            CheckReentrancy: function () {
                if (this._monitor.Busy) {
                    if ((!Bridge.staticEquals(this.CollectionChanged, null)) && (Bridge.fn.getInvocationList(this.CollectionChanged).length > 1)) {
                        throw new System.InvalidOperationException.$ctor1("RowDefinitionCollection Reentrancy not allowed");
                    }
                }
            },
            SetItemsSource: function (value) {
                if (!this.IsUsingItemsSource && this.CountInternal$1 !== 0) {
                    throw new System.InvalidOperationException.$ctor1("Items collection must be empty before using ItemsSource.");
                }

                this.TryUnsubscribeFromCollectionChangedEvent(this._itemsSource);

                this._itemsSource = value;
                this._isUsingItemsSource = true;

                this.TrySubscribeToCollectionChangedEvent(value);

                this.InitializeSourceList(value);

                this.UpdateCountProperty(this.CountInternal$1);

                this.OnCollectionChanged(new System.Collections.Specialized.NotifyCollectionChangedEventArgs.ctor(System.Collections.Specialized.NotifyCollectionChangedAction.Reset));
            },
            ClearItemsSource: function () {
                if (this.IsUsingItemsSource) {
                    this.TryUnsubscribeFromCollectionChangedEvent(this._itemsSource);

                    this._itemsSource = null;
                    this._listWrapper = null;
                    this._isUsingItemsSource = false;
                    this._isUsingListWrapper = false;

                    this.UpdateCountProperty(this.CountInternal$1);

                    this.OnCollectionChanged(new System.Collections.Specialized.NotifyCollectionChangedEventArgs.ctor(System.Collections.Specialized.NotifyCollectionChangedAction.Reset));
                }
            },
            InitializeSourceList: function (sourceCollection) {
                var sourceAsList = Bridge.as(sourceCollection, System.Collections.IList);
                if (sourceAsList == null) {
                    this._listWrapper = new Windows.UI.Xaml.Controls.ItemCollection.EnumerableWrapper(sourceCollection, this);
                    this._isUsingListWrapper = true;
                } else {
                    this._listWrapper = null;
                    this._isUsingListWrapper = false;
                }
            },
            ValidateCollectionChangedEventArgs: function (e) {
                switch (e.Action) {
                    case System.Collections.Specialized.NotifyCollectionChangedAction.Add: 
                        if (System.Array.getCount(e.NewItems) !== 1) {
                            throw new System.NotSupportedException.$ctor1("Range actions are not supported.");
                        }
                        break;
                    case System.Collections.Specialized.NotifyCollectionChangedAction.Remove: 
                        if (System.Array.getCount(e.OldItems) !== 1) {
                            throw new System.NotSupportedException.$ctor1("Range actions are not supported.");
                        }
                        break;
                    case System.Collections.Specialized.NotifyCollectionChangedAction.Replace: 
                        if (System.Array.getCount(e.NewItems) !== 1 || System.Array.getCount(e.OldItems) !== 1) {
                            throw new System.NotSupportedException.$ctor1("Range actions are not supported.");
                        }
                        break;
                    case System.Collections.Specialized.NotifyCollectionChangedAction.Reset: 
                        break;
                    default: 
                        throw new System.NotSupportedException.$ctor1(System.String.format("Unexpected collection change action '{0}'.", [Bridge.box(e.Action, System.Collections.Specialized.NotifyCollectionChangedAction, System.Enum.toStringFn(System.Collections.Specialized.NotifyCollectionChangedAction))]));
                }
            },
            OnSourceCollectionChanged: function (sender, e) {
                this.ValidateCollectionChangedEventArgs(e);

                if (this._isUsingListWrapper) {
                    switch (e.Action) {
                        case System.Collections.Specialized.NotifyCollectionChangedAction.Add: 
                            this._listWrapper.insert(e.NewStartingIndex, System.Array.getItem(e.NewItems, 0));
                            break;
                        case System.Collections.Specialized.NotifyCollectionChangedAction.Remove: 
                            this._listWrapper.removeAt(e.OldStartingIndex);
                            break;
                        case System.Collections.Specialized.NotifyCollectionChangedAction.Replace: 
                            this._listWrapper.setItem(e.OldStartingIndex, System.Array.getItem(e.NewItems, 0));
                            break;
                        case System.Collections.Specialized.NotifyCollectionChangedAction.Reset: 
                            this._listWrapper.Refresh();
                            break;
                    }
                }

                this.UpdateCountProperty(this.CountInternal$1);

                this.OnCollectionChanged(e);
            },
            TrySubscribeToCollectionChangedEvent: function (collection) {
                var incc = Bridge.as(collection, System.Collections.Specialized.INotifyCollectionChanged);
                if (incc != null) {
                    incc.System$Collections$Specialized$INotifyCollectionChanged$addCollectionChanged(Bridge.fn.cacheBind(this, this.OnSourceCollectionChanged));
                }
            },
            TryUnsubscribeFromCollectionChangedEvent: function (collection) {
                var incc = Bridge.as(collection, System.Collections.Specialized.INotifyCollectionChanged);
                if (incc != null) {
                    incc.System$Collections$Specialized$INotifyCollectionChanged$removeCollectionChanged(Bridge.fn.cacheBind(this, this.OnSourceCollectionChanged));
                }
            }
        }
    });

    /**
     * Specifies the panel that the ItemsPresenter creates for the layout of the
     items of an ItemsControl.
     *
     * @public
     * @class Windows.UI.Xaml.Controls.ItemsPanelTemplate
     * @augments Windows.UI.Xaml.FrameworkTemplate
     */
    Bridge.define("Windows.UI.Xaml.Controls.ItemsPanelTemplate", {
        inherits: [Windows.UI.Xaml.FrameworkTemplate],
        ctors: {
            /**
             * Initializes a new instance of the ItemsPanelTemplate class.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Controls.ItemsPanelTemplate
             * @memberof Windows.UI.Xaml.Controls.ItemsPanelTemplate
             * @return  {void}
             */
            ctor: function () {
                this.$initialize();
                Windows.UI.Xaml.FrameworkTemplate.ctor.call(this);
            }
        }
    });

    /**
     * @public
     * @class Windows.UI.Xaml.Controls.RowDefinitionCollection
     * @augments System.Collections.ObjectModel.ObservableCollection$1
     */
    Bridge.define("Windows.UI.Xaml.Controls.RowDefinitionCollection", {
        inherits: [System.Collections.ObjectModel.ObservableCollection$1(Windows.UI.Xaml.Controls.RowDefinition)]
    });

    Bridge.define("Windows.UI.Xaml.Controls.SelectedDatesCollection", {
        inherits: [System.Collections.ObjectModel.ObservableCollection$1(System.DateTime)],
        fields: {
            _addedItems: null,
            _removedItems: null,
            _owner: null
        },
        ctors: {
            /**
             * Initializes a new instance of the CalendarSelectedDatesCollection class.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Controls.SelectedDatesCollection
             * @memberof Windows.UI.Xaml.Controls.SelectedDatesCollection
             * @param   {Windows.UI.Xaml.Controls.INTERNAL_CalendarOrClockBase}    owner
             * @return  {void}
             */
            ctor: function (owner) {
                this.$initialize();
                System.Collections.ObjectModel.ObservableCollection$1(System.DateTime).ctor.call(this);
                this._owner = owner;
                this._addedItems = new (System.Collections.ObjectModel.Collection$1(System.Object)).ctor();
                this._removedItems = new (System.Collections.ObjectModel.Collection$1(System.Object)).ctor();
            }
        },
        methods: {
            RaiseSelectionChanged: function (removedItems, addedItems) {
                this._owner.OnSelectedDatesCollectionChanged(new Windows.UI.Xaml.Controls.SelectionChangedEventArgs(removedItems, addedItems));
            },
            /**
             * Special method for using only one date
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Controls.SelectedDatesCollection
             * @memberof Windows.UI.Xaml.Controls.SelectedDatesCollection
             * @param   {System.DateTime}    item
             * @return  {void}
             */
            SetDate: function (item) {
                this._addedItems.clear();
                this._removedItems.clear();

                this._addedItems.add(Bridge.box(item, System.DateTime, System.DateTime.format));

                this.RaiseSelectionChanged(this._removedItems, this._addedItems);
            }
        }
    });

    /**
     * Defines a binding that connects the properties of binding targets and data sources.
     *
     * @public
     * @class Windows.UI.Xaml.Data.Binding
     * @augments Windows.UI.Xaml.Data.BindingBase
     * @example You can add a Binding using XAML as follows:
 
     Note: you can create the Binding directly using C#:
 
     Here is another example using the TwoWay mode, and also using a Converter:
 
     Using C#:
     *
     * 
     * 
     *
     * MyTextBlock.DataContext = myPlanet;
     * 
     *
     * Binding myBinding = new Binding("Size");
     * MyTextBlock.SetBinding(TextBlock.TextProperty, myBinding);
     * MyTextBlock.DataContext = myPlanet;
     * 
     *
     * MyTextBox.DataContext = myPlanet;
     * 
     *
     * 
     * 
     *     
     *         
     *     
     *     
     * 
     * 
     *
     * Binding myBinding = new Binding("Size");
     * //we set the binding in TwoWay mode:
     * myBinding.Mode = BindingMode.TwoWay;
     * //We create the converter:
     * myBinding.Converter = new KilometersToMilesConverter();
     * MyTextBox.SetBinding(TextBox.TextProperty, myBinding);
     * MyTextBox.DataContext = myPlanet;
     * 
     *
     *
     */
    Bridge.define("Windows.UI.Xaml.Data.Binding", {
        inherits: [Windows.UI.Xaml.Data.BindingBase],
        fields: {
            _isInStyle: false,
            /**
             * Gets or sets the converter object that is called by the binding engine to
             modify the data as it is passed between the source and target, or vice versa.
             Returns the IValueConverter object that modifies the data.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Data.Binding
             * @function Converter
             * @type Windows.UI.Xaml.Data.IValueConverter
             */
            Converter: null,
            /**
             * Gets or sets a value that names the language to pass to any converter specified
             by the Converter property.
             Returns a string that names a language. Interpretation of this value is ultimately up to the converter logic.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Data.Binding
             * @function ConverterLanguage
             * @type string
             */
            ConverterLanguage: null,
            /**
             * Gets or sets a parameter that can be used in the Converter logic.
             Returns a parameter to be passed to the Converter. This can be used in the conversion logic. The default is null.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Data.Binding
             * @function ConverterParameter
             * @type System.Object
             */
            ConverterParameter: null,
            /**
             * Gets or sets the name of the element to use as the binding source for the Binding.
             Returns the value of the Name property or x:Name attribute for the element to bind to. The default is null.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Data.Binding
             * @function ElementName
             * @type string
             */
            ElementName: null,
            _mode: 0,
            _wasModeSetByUserRatherThanDefaultValue: false,
            _path: null,
            /**
             * Gets or sets the binding source by specifying its location relative to the position of the binding target.
             Returns the relative location of the binding source to use. The default is null.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Data.Binding
             * @function RelativeSource
             * @type Windows.UI.Xaml.Data.RelativeSource
             */
            RelativeSource: null,
            _source: null,
            /**
             * Update type
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Data.Binding
             * @function UpdateSourceTrigger
             * @type Windows.UI.Xaml.Data.UpdateSourceTrigger
             */
            UpdateSourceTrigger: 0,
            /**
             * Do not use this property.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Data.Binding
             * @function TemplateOwner
             * @type Windows.UI.Xaml.TemplateInstance
             */
            TemplateOwner: null,
            /**
             * Gets or sets a value that indicates whether the binding engine will report
             exception validation errors.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Data.Binding
             * @function ValidatesOnExceptions
             * @type boolean
             */
            ValidatesOnExceptions: false,
            /**
             * Gets or sets a value that indicates whether the System.Windows.FrameworkElement.BindingValidationError
             event is raised on validation errors.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Data.Binding
             * @function NotifyOnValidationError
             * @type boolean
             */
            NotifyOnValidationError: false,
            /**
             * True to force the property to go through the Validation process when the Binding is set or when the Target is added in the Visual tree.
             This way, if the source property has an Invalid value when setting the Binding, it will immediately be marked as Invalid instead of waiting
             for a value change that keeps/makes it Invalid (which is what happens on Silverlight).
             Defaults to False since it is the behaviour of Silverlight and WPF.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Data.Binding
             * @function ValidatesOnLoad
             * @type boolean
             */
            ValidatesOnLoad: false
        },
        props: {
            /**
             * Gets or sets a value that indicates the direction of the data flow in the binding.
             Returns one of the BindingMode values.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Data.Binding
             * @function Mode
             * @type Windows.UI.Xaml.Data.BindingMode
             */
            Mode: {
                get: function () {
                    return this._mode;
                },
                set: function (value) {
                    this._wasModeSetByUserRatherThanDefaultValue = true;
                    this._mode = value;
                }
            },
            /**
             * Gets or sets the path to the binding source property.
             Returns the property path for the source of the binding.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Data.Binding
             * @function Path
             * @type Windows.UI.Xaml.PropertyPath
             */
            Path: {
                get: function () {
                    if (this._path == null) {
                        this._path = new Windows.UI.Xaml.PropertyPath.ctor("");
                    }
                    return this._path;
                },
                set: function (value) {
                    this._path = value;
                }
            },
            /**
             * Gets or sets the data source for the binding.
             Returns the source object that contains the data for the binding.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Data.Binding
             * @function Source
             * @type System.Object
             */
            Source: {
                get: function () {
                    return this._source;
                },
                set: function (value) {
                    this._source = value;
                }
            }
        },
        ctors: {
            init: function () {
                this._mode = Windows.UI.Xaml.Data.BindingMode.OneWay;
                this._wasModeSetByUserRatherThanDefaultValue = false;
            },
            /**
             * Initializes a new instance of the Binding class.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Data.Binding
             * @memberof Windows.UI.Xaml.Data.Binding
             * @return  {void}
             */
            ctor: function () {
                this.$initialize();
                Windows.UI.Xaml.Data.BindingBase.ctor.call(this);
            },
            /**
             * Initializes a new instance of the Binding class with the given path.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Data.Binding
             * @memberof Windows.UI.Xaml.Data.Binding
             * @param   {string}    path    The path to the source of the binding.
             * @return  {void}
             */
            $ctor1: function (path) {
                this.$initialize();
                Windows.UI.Xaml.Data.BindingBase.ctor.call(this);
                if (!System.String.isNullOrWhiteSpace(path)) {
                    this.Path = new Windows.UI.Xaml.PropertyPath.ctor(path);
                }
            }
        },
        methods: {
            INTERNAL_WasModeSetByUserRatherThanDefaultValue: function () {
                return this._wasModeSetByUserRatherThanDefaultValue;
            },
            Clone: function () {
                var b = new Windows.UI.Xaml.Data.Binding.$ctor1(this.Path.Path);
                b.Converter = this.Converter;
                b.ConverterLanguage = this.ConverterLanguage;
                b.ConverterParameter = this.ConverterParameter;
                b.ElementName = this.ElementName;
                b._mode = this._mode;
                b._wasModeSetByUserRatherThanDefaultValue = this._wasModeSetByUserRatherThanDefaultValue;
                b.RelativeSource = this.RelativeSource;
                b.Source = this.Source;
                b.UpdateSourceTrigger = this.UpdateSourceTrigger;
                b.StringFormat = this.StringFormat;
                return b;
            },
            CopyTo: function (target) {
                target.Path = new Windows.UI.Xaml.PropertyPath.ctor(this.Path == null ? "" : this.Path.Path);
                target.Converter = this.Converter;
                target.ConverterLanguage = this.ConverterLanguage;
                target.ConverterParameter = this.ConverterParameter;
                target.ElementName = this.ElementName;
                target._mode = this._mode;
                target._wasModeSetByUserRatherThanDefaultValue = this._wasModeSetByUserRatherThanDefaultValue;
                target.RelativeSource = this.RelativeSource;
                target.Source = this.Source;
                target.UpdateSourceTrigger = this.UpdateSourceTrigger;
                target.StringFormat = this.StringFormat;
                target._isInStyle = this._isInStyle;
            }
        }
    });

    Bridge.define("Windows.UI.Xaml.Data.IndexedPropertyPathNode", {
        inherits: [Windows.UI.Xaml.Data.PropertyPathNode],
        fields: {
            _index: null,
            _parsedIndex: null,
            _setMethod: null,
            _getMethod: null
        },
        ctors: {
            ctor: function (index) {
                this.$initialize();
                Windows.UI.Xaml.Data.PropertyPathNode.ctor.call(this);
                this._index = index;
            }
        },
        methods: {
            SetValue: function (value) {
                if (this._setMethod != null) {
                    this.TryInvokeSetMethod(this._parsedIndex, value);
                }
            },
            TryInvokeSetMethod: function (index, value) {
                try {
                    Bridge.Reflection.midel(this._setMethod, Bridge.unbox(this.Source)).apply(null, Bridge.unbox(System.Array.init([index, value], System.Object)));
                    return true;
                } catch ($e1) {
                    $e1 = System.Exception.create($e1);
                    return false;
                }
            },
            UpdateValue: function () {
                if (this._getMethod != null) {
                    var result = { };
                    if (this.TryInvokeGetMethod(this._parsedIndex, result)) {
                        this.UpdateValueAndIsBroken(result.v, this.CheckIsBroken(true));
                    }
                }
            },
            TryInvokeGetMethod: function (index, result) {
                try {
                    result.v = Bridge.Reflection.midel(this._getMethod, Bridge.unbox(this.Source)).apply(null, Bridge.unbox(System.Array.init([index], System.Object)));
                    return true;
                } catch ($e1) {
                    $e1 = System.Exception.create($e1);
                    result.v = null;
                    return false;
                }
            },
            OnSourceChanged: function (oldvalue, newValue) {
                if (this.Source != null) {
                    if (newValue != null) {
                        this.GetIndexerInfo(Bridge.getType(newValue));
                    }
                }
            },
            GetIndexer: function (type) {
                var singleParameterIndexer = System.Linq.Enumerable.from(Bridge.Reflection.getMembers(type, 16, 20)).firstOrDefault(function (property) {
                        return property.g != null && (property.ipi || []).length === 1;
                    }, null);
                if (singleParameterIndexer == null) {
                    throw new System.NotSupportedException.$ctor1("Indexers with multiple parameters are not supported yet for Binding.");
                }
                return singleParameterIndexer;
            },
            GetIndexerInfo: function (sourceType) {
                var indexer = this.GetIndexer(sourceType);
                this.PrepareIndexForIndexer(indexer);
                if (indexer != null) {
                    this._getMethod = indexer.g;
                    this._setMethod = indexer.s;
                } else {
                    this._getMethod = null;
                    this._setMethod = null;
                }
            },
            PrepareIndexForIndexer: function (indexer) {
                var indexType = System.Linq.Enumerable.from((indexer.ipi || [])).first().pt;
                try {
                    this._parsedIndex = System.Convert.changeType(this._index, indexType);
                } catch ($e1) {
                    $e1 = System.Exception.create($e1);
                    if (DotNetForHtml5.Core.TypeFromStringConverters.CanTypeBeConverted(indexType)) {
                        this._parsedIndex = DotNetForHtml5.Core.TypeFromStringConverters.ConvertFromInvariantString(indexType, this._index);
                    } else {
                        this._parsedIndex = this._index;
                    }
                }
            }
        }
    });

    Bridge.define("Windows.UI.Xaml.Data.StandardPropertyPathNode", {
        inherits: [Windows.UI.Xaml.Data.PropertyPathNode],
        fields: {
            _propertyName: null,
            BindsDirectlyToSource: false,
            _dependencyPropertyListener: null
        },
        ctors: {
            init: function () {
                this.BindsDirectlyToSource = false;
            },
            $ctor1: function (typeName, propertyName) {
                this.$initialize();
                Windows.UI.Xaml.Data.PropertyPathNode.ctor.call(this);
                this._propertyName = propertyName;
            },
            /**
             * This constructor is called only when there is no path, which means that the binding's source is directly the value we are looking for.
             *
             * @instance
             * @this Windows.UI.Xaml.Data.StandardPropertyPathNode
             * @memberof Windows.UI.Xaml.Data.StandardPropertyPathNode
             * @return  {void}
             */
            ctor: function () {
                this.$initialize();
                Windows.UI.Xaml.Data.PropertyPathNode.ctor.call(this);
                this.BindsDirectlyToSource = true;
            }
        },
        methods: {
            SetValue: function (value) {
                if (this.DependencyProperty != null) {
                    Bridge.cast(this.Source, Windows.UI.Xaml.DependencyObject).SetValue(this.DependencyProperty, value);
                } else {
                    if (this.PropertyInfo != null) {
                        Bridge.Reflection.midel(this.PropertyInfo.s, this.Source)(value);
                    } else {
                        if (this.FieldInfo != null) {
                            Bridge.Reflection.fieldAccess(this.FieldInfo, Bridge.unbox(this.Source), Bridge.unbox(value));
                        }
                    }
                }
            },
            UpdateValue: function () {
                if (this.DependencyProperty != null) {

                    this.UpdateValueAndIsBroken(Bridge.cast(this.Source, Windows.UI.Xaml.DependencyObject).GetValue(this.DependencyProperty), this.CheckIsBroken());
                } else if (this.PropertyInfo != null) {
                    try {
                        var propertyValue = null;
                        var methodInfo = this.PropertyInfo.g;
                        if (INTERNAL_BridgeWorkarounds.MethodInfoIsStatic_SimulatorCompatible(methodInfo)) {
                            propertyValue = Bridge.Reflection.midel(this.PropertyInfo.g, null)();
                        } else {
                            propertyValue = Bridge.Reflection.midel(this.PropertyInfo.g, Bridge.unbox(this.Source))();
                        }
                        this.UpdateValueAndIsBroken(propertyValue, this.CheckIsBroken());
                    } catch ($e1) {
                        $e1 = System.Exception.create($e1);
                        this.UpdateValueAndIsBroken(null, this.CheckIsBroken());
                    }
                } else if (this.FieldInfo != null) {
                    try {
                        this.UpdateValueAndIsBroken(Bridge.Reflection.fieldAccess(this.FieldInfo, Bridge.unbox(this.Source)), this.CheckIsBroken());
                    } catch ($e2) {
                        $e2 = System.Exception.create($e2);
                        this.UpdateValueAndIsBroken(null, this.CheckIsBroken());
                    }
                } else if (this.BindsDirectlyToSource) {
                    this.UpdateValueAndIsBroken(this.Source, this.CheckIsBroken(this.BindsDirectlyToSource));
                } else {
                    this.UpdateValueAndIsBroken(null, this.CheckIsBroken());
                }
            },
            OnSourceChanged: function (oldvalue, newValue) {
                var oldSource = Bridge.as(oldvalue, Windows.UI.Xaml.DependencyObject);
                var newSource = Bridge.as(newValue, Windows.UI.Xaml.DependencyObject);

                var listener = this._dependencyPropertyListener;
                if (listener != null) {
                    listener.Windows$UI$Xaml$Data$IPropertyChangedListener$Detach();
                    this._dependencyPropertyListener = (listener = null);
                }

                this.DependencyProperty = null;
                this.PropertyInfo = null;
                this.FieldInfo = null;
                if (this.Source == null) {
                    return;
                }

                if (newSource != null) {
                    if (!this.BindsDirectlyToSource) {
                        var dependencyProperty;
                        if (Bridge.referenceEquals(this._propertyName, "DataContext")) {
                            dependencyProperty = Windows.UI.Xaml.FrameworkElement.DataContextProperty;
                        } else {
                            var type = Bridge.getType(this.Source);
                            dependencyProperty = CSHTML5.Internal.INTERNAL_TypeToStringsToDependencyProperties.GetPropertyInTypeOrItsBaseTypes(type, this._propertyName);
                        }

                        if (dependencyProperty != null) {
                            this.DependencyProperty = dependencyProperty;
                            this._dependencyPropertyListener = (listener = CSHTML5.Internal.INTERNAL_PropertyStore.ListenToChanged(newSource, dependencyProperty, Bridge.fn.cacheBind(this, this.OnPropertyChanged)));
                        }
                    }
                }

                if (this.DependencyProperty == null) {
                    if (!this.BindsDirectlyToSource) {
                        var sourceType = Bridge.getType(this.Source);
                        this.PropertyInfo = Bridge.Reflection.getMembers(sourceType, 16, 284, this._propertyName);
                        if (this.PropertyInfo == null) {
                            this.FieldInfo = Bridge.Reflection.getMembers(sourceType, 4, 284, this._propertyName);
                        }
                    }
                }
            },
            OnPropertyChanged: function (sender, args) {
                try {
                    this.UpdateValue();
                    if (this.Next != null) {
                        this.Next.Windows$UI$Xaml$Data$IPropertyPathNode$SetSource(this.Value);
                    }
                } catch ($e1) {
                    $e1 = System.Exception.create($e1);
                    var ex;
                    if (Bridge.is($e1, System.Windows.Markup.XamlParseException)) {
                        ex = $e1;
                        if (CSharpXamlForHtml5.Environment.IsRunningInJavaScript) {
                            throw ex;
                        } else {
                            Windows.UI.Xaml.MessageBox.Show(Bridge.toString(ex));
                        }
                    } else {
                        ex = $e1;
                        System.Diagnostics.Debug.WriteLine$2("Binding exception: " + (Bridge.toString(ex) || ""));
                    }
                }
            },
            OnSourcePropertyChanged: function (sender, e) {
                if ((Bridge.referenceEquals(e.PropertyName, this._propertyName) || System.String.isNullOrEmpty(e.PropertyName)) && (this.PropertyInfo != null || this.FieldInfo != null)) {
                    this.UpdateValue();
                    var next = this.Next;
                    if (next != null) {
                        next.Windows$UI$Xaml$Data$IPropertyPathNode$SetSource(this.Value);
                    }
                }
            }
        }
    });

    Bridge.define("Windows.UI.Xaml.Data.TemplateBinding", {
        inherits: [Windows.UI.Xaml.Data.BindingBase],
        fields: {
            _property: null,
            _converter: null,
            _converterLanguage: null,
            _converterParameter: null
        },
        props: {
            Property: {
                get: function () {
                    return this._property;
                },
                set: function (value) {
                    this._property = value;
                }
            },
            Converter: {
                get: function () {
                    return this._converter;
                },
                set: function (value) {
                    this._converter = value;
                }
            },
            ConverterLanguage: {
                get: function () {
                    return this._converterLanguage;
                },
                set: function (value) {
                    this._converterLanguage = value;
                }
            },
            ConverterParameter: {
                get: function () {
                    return this._converterParameter;
                },
                set: function (value) {
                    this._converterParameter = value;
                }
            }
        },
        ctors: {
            ctor: function () {
                this.$initialize();
                Windows.UI.Xaml.Data.BindingBase.ctor.call(this);
            },
            $ctor1: function (property) {
                this.$initialize();
                Windows.UI.Xaml.Data.BindingBase.ctor.call(this);
            }
        }
    });

    /**
     * Describes the visual structure of a data object.
     *
     * @public
     * @class Windows.UI.Xaml.DataTemplate
     * @augments Windows.UI.Xaml.FrameworkTemplate
     */
    Bridge.define("Windows.UI.Xaml.DataTemplate", {
        inherits: [Windows.UI.Xaml.FrameworkTemplate],
        ctors: {
            /**
             * Initializes a new instance of the DataTemplate class.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.DataTemplate
             * @memberof Windows.UI.Xaml.DataTemplate
             * @return  {void}
             */
            ctor: function () {
                this.$initialize();
                Windows.UI.Xaml.FrameworkTemplate.ctor.call(this);
            }
        },
        methods: {
            /**
             * Creates the System.Windows.UIElement objects in the System.Windows.DataTemplate.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.DataTemplate
             * @memberof Windows.UI.Xaml.DataTemplate
             * @return  {Windows.UI.Xaml.DependencyObject}        The root System.Windows.UIElement of the System.Windows.DataTemplate.
             */
            LoadContent: function () {
                return this.INTERNAL_InstantiateFrameworkTemplate();
            }
        }
    });

    Bridge.define("Windows.UI.Xaml.DependencyObjectCollectionInternal", {
        inherits: [Windows.UI.Xaml.PresentationFrameworkCollection$1(Windows.UI.Xaml.DependencyObject)],
        ctors: {
            ctor: function (owner) {
                this.$initialize();
                Windows.UI.Xaml.PresentationFrameworkCollection$1(Windows.UI.Xaml.DependencyObject).ctor.call(this);
                owner.ProvideSelfAsInheritanceContext$1(this, null);
                this.IsInheritanceContextSealed = true;
            }
        },
        methods: {
            AddOverride: function (value) {
                this.AddDependencyObjectInternal(value);
            },
            ClearOverride: function () {
                this.ClearDependencyObjectInternal();
            },
            InsertOverride: function (index, value) {
                this.InsertDependencyObjectInternal(index, value);
            },
            RemoveAtOverride: function (index) {
                this.RemoveAtDependencyObjectInternal(index);
            },
            RemoveOverride: function (value) {
                return this.RemoveDependencyObjectInternal(value);
            },
            GetItemOverride: function (index) {
                return this.GetItemInternal(index);
            },
            SetItemOverride: function (index, value) {
                this.SetItemDependencyObjectInternal(index, value);
            }
        }
    });

    Bridge.define("Windows.UI.Xaml.Documents.INTERNAL_TextContainerSpan", {
        inherits: [Windows.UI.Xaml.Documents.INTERNAL_TextContainer],
        props: {
            Span: {
                get: function () {
                    return Bridge.cast(this.Parent, Windows.UI.Xaml.Documents.Span);
                }
            },
            Text: {
                get: function () {
                    var $t;
                    var text = "";
                    $t = Bridge.getEnumerator(this.Span.Inlines);
                    try {
                        while ($t.moveNext()) {
                            var inline = $t.Current;
                            if (Bridge.is(inline, Windows.UI.Xaml.Documents.Run)) {
                                text = (text || "") + (Bridge.cast(inline, Windows.UI.Xaml.Documents.Run).Text || "");
                            } else if (Bridge.is(inline, Windows.UI.Xaml.Documents.Span)) {
                                text = (text || "") + (new Windows.UI.Xaml.Documents.INTERNAL_TextContainerSpan(Bridge.cast(inline, Windows.UI.Xaml.Documents.Span)).Text || "");
                            } else {
                            }
                        }
                    } finally {
                        if (Bridge.is($t, System.IDisposable)) {
                            $t.System$IDisposable$Dispose();
                        }
                    }
                    return text;
                }
            }
        },
        alias: [
            "Text", "Windows$UI$Xaml$Documents$INTERNAL_ITextContainer$Text",
            "EndChange", "Windows$UI$Xaml$Documents$INTERNAL_ITextContainer$EndChange"
        ],
        ctors: {
            ctor: function (span) {
                this.$initialize();
                Windows.UI.Xaml.Documents.INTERNAL_TextContainer.ctor.call(this, span);

            }
        },
        methods: {
            EndChange: function () {
                if (CSHTML5.Internal.INTERNAL_VisualTreeManager.IsElementInVisualTree(this.Span)) {
                    Windows.UI.Xaml.Documents.INTERNAL_TextContainerHelper.FromOwner(this.Span.Parent).Windows$UI$Xaml$Documents$INTERNAL_ITextContainer$EndChange();
                }
            },
            OnTextAddedOverride: function (textElement) {
                if (CSHTML5.Internal.INTERNAL_VisualTreeManager.IsElementInVisualTree(this.Span)) {
                    CSHTML5.Internal.INTERNAL_VisualTreeManager.AttachVisualChildIfNotAlreadyAttached(textElement, this.Span);
                }
            },
            OnTextRemovedOverride: function (textElement) {
                if (CSHTML5.Internal.INTERNAL_VisualTreeManager.IsElementInVisualTree(this.Span)) {
                    CSHTML5.Internal.INTERNAL_VisualTreeManager.DetachVisualChildIfNotNull(textElement, this.Span);
                }
            }
        }
    });

    Bridge.define("Windows.UI.Xaml.Documents.INTERNAL_TextContainerTextBlock", {
        inherits: [Windows.UI.Xaml.Documents.INTERNAL_TextContainer],
        props: {
            TextBlock: {
                get: function () {
                    return Bridge.cast(this.Parent, Windows.UI.Xaml.Controls.TextBlock);
                }
            },
            Text: {
                get: function () {
                    var $t;
                    var text = "";
                    $t = Bridge.getEnumerator(this.TextBlock.Inlines);
                    try {
                        while ($t.moveNext()) {
                            var inline = $t.Current;
                            if (Bridge.is(inline, Windows.UI.Xaml.Documents.Run)) {
                                text = (text || "") + (Bridge.cast(inline, Windows.UI.Xaml.Documents.Run).Text || "");
                            } else if (Bridge.is(inline, Windows.UI.Xaml.Documents.Span)) {
                                text = (text || "") + (new Windows.UI.Xaml.Documents.INTERNAL_TextContainerSpan(Bridge.cast(inline, Windows.UI.Xaml.Documents.Span)).Text || "");
                            } else {
                            }
                        }
                    } finally {
                        if (Bridge.is($t, System.IDisposable)) {
                            $t.System$IDisposable$Dispose();
                        }
                    }
                    return text;
                }
            }
        },
        alias: [
            "Text", "Windows$UI$Xaml$Documents$INTERNAL_ITextContainer$Text",
            "EndChange", "Windows$UI$Xaml$Documents$INTERNAL_ITextContainer$EndChange"
        ],
        ctors: {
            ctor: function (parent) {
                this.$initialize();
                Windows.UI.Xaml.Documents.INTERNAL_TextContainer.ctor.call(this, parent);

            }
        },
        methods: {
            EndChange: function () {
                this.TextBlock.SetTextPropertyNoCallBack(this.Text);
            },
            OnTextAddedOverride: function (textElement) {
                if (CSHTML5.Internal.INTERNAL_VisualTreeManager.IsElementInVisualTree(this.TextBlock)) {
                    CSHTML5.Internal.INTERNAL_VisualTreeManager.AttachVisualChildIfNotAlreadyAttached(textElement, this.TextBlock);
                }
            },
            OnTextRemovedOverride: function (textElement) {
                if (CSHTML5.Internal.INTERNAL_VisualTreeManager.IsElementInVisualTree(this.TextBlock)) {
                    CSHTML5.Internal.INTERNAL_VisualTreeManager.DetachVisualChildIfNotNull(textElement, this.TextBlock);
                }
            }
        }
    });

    /**
     * Provides event data for the RightTapped event.
     * // Initializes a new instance of the RightTappedRoutedEventArgs class.
    //
     *
     * @public
     * @class Windows.UI.Xaml.Input.RightTappedRoutedEventArgs
     * @augments Windows.UI.Xaml.Input.PointerRoutedEventArgs
     */
    Bridge.define("Windows.UI.Xaml.Input.RightTappedRoutedEventArgs", {
        inherits: [Windows.UI.Xaml.Input.PointerRoutedEventArgs]
    });

    /**
     * Provides event data for the Tapped event.
     * // Initializes a new instance of the TappedRoutedEventArgs class.
    //
     *
     * @public
     * @class Windows.UI.Xaml.Input.TappedRoutedEventArgs
     * @augments Windows.UI.Xaml.Input.PointerRoutedEventArgs
     */
    Bridge.define("Windows.UI.Xaml.Input.TappedRoutedEventArgs", {
        inherits: [Windows.UI.Xaml.Input.PointerRoutedEventArgs]
    });

    /**
     * Do not use this class
     *
     * @public
     * @class Windows.UI.Xaml.INTERNAL_VisualStateGroupCollection
     * @implements  System.Collections.Generic.IList$1
     */
    Bridge.define("Windows.UI.Xaml.INTERNAL_VisualStateGroupCollection", {
        inherits: [System.Collections.Generic.IList$1(Windows.UI.Xaml.VisualStateGroup)],
        fields: {
            _visualStateGroups: null,
            _visualStates: null
        },
        props: {
            Count: {
                get: function () {
                    return this._visualStateGroups.Count;
                }
            },
            IsReadOnly: {
                get: function () {
                    throw new System.NotImplementedException.ctor();
                }
            }
        },
        alias: [
            "indexOf", "System$Collections$Generic$IList$1$Windows$UI$Xaml$VisualStateGroup$indexOf",
            "insert", "System$Collections$Generic$IList$1$Windows$UI$Xaml$VisualStateGroup$insert",
            "removeAt", "System$Collections$Generic$IList$1$Windows$UI$Xaml$VisualStateGroup$removeAt",
            "getItem", "System$Collections$Generic$IList$1$Windows$UI$Xaml$VisualStateGroup$getItem",
            "setItem", "System$Collections$Generic$IList$1$Windows$UI$Xaml$VisualStateGroup$setItem",
            "add", "System$Collections$Generic$ICollection$1$Windows$UI$Xaml$VisualStateGroup$add",
            "clear", "System$Collections$Generic$ICollection$1$Windows$UI$Xaml$VisualStateGroup$clear",
            "contains", "System$Collections$Generic$ICollection$1$Windows$UI$Xaml$VisualStateGroup$contains",
            "copyTo", "System$Collections$Generic$ICollection$1$Windows$UI$Xaml$VisualStateGroup$copyTo",
            "Count", "System$Collections$Generic$ICollection$1$Windows$UI$Xaml$VisualStateGroup$Count",
            "IsReadOnly", "System$Collections$Generic$ICollection$1$Windows$UI$Xaml$VisualStateGroup$IsReadOnly",
            "remove", "System$Collections$Generic$ICollection$1$Windows$UI$Xaml$VisualStateGroup$remove",
            "GetEnumerator", ["System$Collections$Generic$IEnumerable$1$Windows$UI$Xaml$VisualStateGroup$GetEnumerator", "System$Collections$Generic$IEnumerable$1$GetEnumerator"]
        ],
        ctors: {
            init: function () {
                this._visualStateGroups = new (System.Collections.Generic.Dictionary$2(System.String,Windows.UI.Xaml.VisualStateGroup)).ctor();
                this._visualStates = new (System.Collections.Generic.Dictionary$2(System.String,Windows.UI.Xaml.VisualState)).ctor();
            }
        },
        methods: {
            getItem: function (index) {
                return System.Linq.Enumerable.from(INTERNAL_BridgeWorkarounds.GetDictionaryValues_SimulatorCompatible(System.String, Windows.UI.Xaml.VisualStateGroup, this._visualStateGroups)).elementAt(index);
            },
            setItem: function (index, value) {
                throw new System.NotImplementedException.ctor();
            },
            GetVisualState: function (visualStateName) {
                if (visualStateName != null) {
                    if (this._visualStates != null && this._visualStates.containsKey(visualStateName)) {
                        return this._visualStates.getItem(visualStateName);
                    } else {
                        return null;

                    }
                }
                throw new System.ArgumentNullException.$ctor1("visualStateName");
            },
            indexOf: function (item) {
                var i = 0;
                while (i < this._visualStateGroups.Count) {
                    if (Bridge.referenceEquals(System.Linq.Enumerable.from(INTERNAL_BridgeWorkarounds.GetDictionaryValues_SimulatorCompatible(System.String, Windows.UI.Xaml.VisualStateGroup, this._visualStateGroups)).elementAt(i), item)) {
                        return i;
                    }
                    i = (i + 1) | 0;
                }
                return -1;
            },
            insert: function (index, item) {
                throw new System.NotImplementedException.ctor();
            },
            removeAt: function (index) {
                throw new System.NotImplementedException.ctor();
            },
            add: function (item) {
                var $t;
                if (this._visualStateGroups.containsKey(item.Name)) {
                    throw new System.InvalidOperationException.$ctor1(System.String.format("Multiple VisualStatesGroups with the Name:\"{0}\" appeared in the same scope.", [item.Name]));
                }
                this._visualStateGroups.add(item.Name, item);
                $t = Bridge.getEnumerator(item.States);
                try {
                    while ($t.moveNext()) {
                        var visualState = Bridge.cast($t.Current, Windows.UI.Xaml.VisualState);
                        if (this._visualStates.containsKey(visualState.Name)) {
                            throw new System.InvalidOperationException.$ctor1(System.String.format("Multiple VisualStates with the Name:\"{0}\" appeared in the same scope.", [visualState.Name]));
                        }
                        visualState.INTERNAL_Group = item;
                        this._visualStates.add(visualState.Name, visualState);
                    }
                } finally {
                    if (Bridge.is($t, System.IDisposable)) {
                        $t.System$IDisposable$Dispose();
                    }
                }

            },
            clear: function () {
                this._visualStateGroups.clear();
                this._visualStates.clear();
            },
            contains: function (item) {
                return this._visualStateGroups.ContainsValue(item);
            },
            copyTo: function (array, arrayIndex) {
                throw new System.NotImplementedException.ctor();
            },
            remove: function (item) {
                var $t;
                this._visualStateGroups.remove(item.Name);
                $t = Bridge.getEnumerator(item.States);
                try {
                    while ($t.moveNext()) {
                        var state = Bridge.cast($t.Current, Windows.UI.Xaml.VisualState);
                        if (this._visualStates.containsKey(state.Name)) {
                            this._visualStates.remove(state.Name);
                        }
                    }
                } finally {
                    if (Bridge.is($t, System.IDisposable)) {
                        $t.System$IDisposable$Dispose();
                    }
                }
                return true;
            },
            GetEnumerator: function () {
                return Bridge.getEnumerator(INTERNAL_BridgeWorkarounds.GetDictionaryValues_SimulatorCompatible(System.String, Windows.UI.Xaml.VisualStateGroup, this._visualStateGroups), Windows.UI.Xaml.VisualStateGroup);
            },
            System$Collections$IEnumerable$GetEnumerator: function () {
                return Bridge.getEnumerator(INTERNAL_BridgeWorkarounds.GetDictionaryValues_SimulatorCompatible(System.String, Windows.UI.Xaml.VisualStateGroup, this._visualStateGroups), Windows.UI.Xaml.VisualStateGroup);
            },
            ContainsVisualState: function (visualStateName) {
                return this._visualStates.containsKey(visualStateName);
            }
        }
    });

    Bridge.define("Windows.UI.Xaml.Media.Animation.AnimationTimeline", {
        inherits: [Windows.UI.Xaml.Media.Animation.Timeline],
        fields: {
            _targetName: null,
            _targetProperty: null,
            _propertyContainer: null,
            _target: null,
            _parameters$1: null,
            _propDp: null,
            _isInitialized: false,
            _cancelledAnimation: false
        },
        ctors: {
            init: function () {
                this._isInitialized = false;
                this._cancelledAnimation = false;
            }
        },
        methods: {
            GetNaturalDurationCore: function () {
                return Windows.UI.Xaml.Duration.op_Implicit(new System.TimeSpan(0, 0, 1));
            },
            IterateOnce: function (parameters, isLastLoop) {
                if (!this._isInitialized) {
                    this.Initialize(parameters);
                } else {
                    this.RestoreDefault();
                }
                this.SetInitialAnimationValue(parameters.IsVisualStateChange);
                Windows.UI.Xaml.Media.Animation.Timeline.prototype.IterateOnce.call(this, parameters, isLastLoop);

                this.Apply(parameters, isLastLoop);
            },
            SetInitialAnimationValue: function (isVisualStateChange) {
                var initialAnimationValue = this._propertyContainer.GetValue(this._propDp);
                Windows.UI.Xaml.Media.Animation.AnimationHelpers.ApplyValue(this._propertyContainer, this._targetProperty, initialAnimationValue, isVisualStateChange);
            },
            Apply: function (parameters, isLastLoop) { },
            RestoreDefault: function () {
                this._cancelledAnimation = false;
                this.RestoreDefaultCore();
            },
            RestoreDefaultCore: function () {

            },
            Initialize: function (parameters) {
                this.GetTargetInformation(parameters);
                this.InitializeCore();
                this.ComputeDuration();
                this._isInitialized = true;
            },
            GetTargetInformation: function (parameters) {

            },
            InitializeCore: function () {

            },
            Stop$1: function (frameworkElement, groupName, revertToFormerValue) {
                if (revertToFormerValue === void 0) { revertToFormerValue = false; }
                if (this._isInitialized) {
                    this._cancelledAnimation = revertToFormerValue;
                    Windows.UI.Xaml.Media.Animation.Timeline.prototype.Stop$1.call(this, frameworkElement, groupName, revertToFormerValue);
                    this.StopAnimation(groupName);
                    if (revertToFormerValue) {
                        this.UnApply$1(this._parameters$1.IsVisualStateChange);
                    }
                }
            },
            StopAnimation: function (groupName) {

            },
            UnApply$1: function (isVisualStateChange) {
                Windows.UI.Xaml.Media.Animation.AnimationHelpers.ApplyValue(this._propertyContainer, this._targetProperty, Windows.UI.Xaml.DependencyProperty.UnsetValue, isVisualStateChange);
            }
        }
    });

    /**
     * Represents an easing function that creates an animation that accelerates
     and/or decelerates using a circular function.
     *
     * @public
     * @class Windows.UI.Xaml.Media.Animation.CircleEase
     * @augments Windows.UI.Xaml.Media.Animation.EasingFunctionBase
     */
    Bridge.define("Windows.UI.Xaml.Media.Animation.CircleEase", {
        inherits: [Windows.UI.Xaml.Media.Animation.EasingFunctionBase],
        statics: {
            fields: {
                FUNCTION_TYPE_STRING: null
            },
            ctors: {
                init: function () {
                    this.FUNCTION_TYPE_STRING = "Circ";
                }
            }
        },
        methods: {
            /**
             * @instance
             * @override
             * @this Windows.UI.Xaml.Media.Animation.CircleEase
             * @memberof Windows.UI.Xaml.Media.Animation.CircleEase
             * @return  {string}
             */
            GetFunctionAsString: function () {
                return (this.GetEasingModeAsString() || "") + (Windows.UI.Xaml.Media.Animation.CircleEase.FUNCTION_TYPE_STRING || "");
            }
        }
    });

    /**
     * Represents an easing function that creates an animation that accelerates
     and/or decelerates using the formula f(t) = t3.
     *
     * @public
     * @class Windows.UI.Xaml.Media.Animation.CubicEase
     * @augments Windows.UI.Xaml.Media.Animation.EasingFunctionBase
     */
    Bridge.define("Windows.UI.Xaml.Media.Animation.CubicEase", {
        inherits: [Windows.UI.Xaml.Media.Animation.EasingFunctionBase],
        statics: {
            fields: {
                FUNCTION_TYPE_STRING: null
            },
            ctors: {
                init: function () {
                    this.FUNCTION_TYPE_STRING = "Cubic";
                }
            }
        },
        methods: {
            /**
             * @instance
             * @override
             * @this Windows.UI.Xaml.Media.Animation.CubicEase
             * @memberof Windows.UI.Xaml.Media.Animation.CubicEase
             * @return  {string}
             */
            GetFunctionAsString: function () {
                return (this.GetEasingModeAsString() || "") + (Windows.UI.Xaml.Media.Animation.CubicEase.FUNCTION_TYPE_STRING || "");
            }
        }
    });

    /**
     * Animates from the Object value of the previous key frame to its own Value
     using discrete values.
     * // Initializes a new instance of the DiscreteObjectKeyFrame class.
    //
     *
     * @public
     * @class Windows.UI.Xaml.Media.Animation.DiscreteObjectKeyFrame
     * @augments Windows.UI.Xaml.Media.Animation.ObjectKeyFrame
     */
    Bridge.define("Windows.UI.Xaml.Media.Animation.DiscreteObjectKeyFrame", {
        inherits: [Windows.UI.Xaml.Media.Animation.ObjectKeyFrame]
    });

    Bridge.define("Windows.UI.Xaml.Media.Animation.DoubleKeyFrameCollection", {
        inherits: [System.Collections.Generic.List$1(Windows.UI.Xaml.Media.Animation.DoubleKeyFrame)]
    });

    Bridge.define("Windows.UI.Xaml.Media.Animation.EasingDoubleKeyFrame", {
        inherits: [Windows.UI.Xaml.Media.Animation.DoubleKeyFrame],
        statics: {
            fields: {
                EasingFunctionProperty: null
            },
            ctors: {
                init: function () {
                    this.EasingFunctionProperty = Windows.UI.Xaml.DependencyProperty.Register("EasingFunction", Windows.UI.Xaml.Media.Animation.EasingFunctionBase, Windows.UI.Xaml.Media.Animation.EasingDoubleKeyFrame, new Windows.UI.Xaml.PropertyMetadata.$ctor4(null));
                }
            }
        },
        props: {
            /**
             * EasingFunction
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Media.Animation.EasingDoubleKeyFrame
             * @function EasingFunction
             * @type Windows.UI.Xaml.Media.Animation.EasingFunctionBase
             */
            EasingFunction: {
                get: function () {
                    return Bridge.cast(this.GetValue(Windows.UI.Xaml.Media.Animation.EasingDoubleKeyFrame.EasingFunctionProperty), Windows.UI.Xaml.Media.Animation.EasingFunctionBase);
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Media.Animation.EasingDoubleKeyFrame.EasingFunctionProperty, value);
                }
            }
        },
        methods: {
            INTERNAL_GetEasingFunction: function () {
                return this.EasingFunction;
            }
        }
    });

    /**
     * Represents an easing function that creates an animation that accelerates
     and/or decelerates using an exponential formula.
     *
     * @public
     * @class Windows.UI.Xaml.Media.Animation.ExponentialEase
     * @augments Windows.UI.Xaml.Media.Animation.EasingFunctionBase
     */
    Bridge.define("Windows.UI.Xaml.Media.Animation.ExponentialEase", {
        inherits: [Windows.UI.Xaml.Media.Animation.EasingFunctionBase],
        statics: {
            fields: {
                FUNCTION_TYPE_STRING: null,
                velocityInitializationMade: false,
                /**
                 * Identifies the Exponent dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Media.Animation.ExponentialEase
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                ExponentProperty: null
            },
            ctors: {
                init: function () {
                    this.FUNCTION_TYPE_STRING = "ExponentialCustomEasing";
                    this.velocityInitializationMade = false;
                    this.ExponentProperty = Windows.UI.Xaml.DependencyProperty.Register("Exponent", System.Double, Windows.UI.Xaml.Media.Animation.ExponentialEase, new Windows.UI.Xaml.PropertyMetadata.$ctor1(Bridge.box(2.0, System.Double, System.Double.format, System.Double.getHashCode)));
                }
            }
        },
        props: {
            /**
             * Gets or sets the exponent used to determine the interpolation of the animation.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Media.Animation.ExponentialEase
             * @function Exponent
             * @type number
             */
            Exponent: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.Media.Animation.ExponentialEase.ExponentProperty), System.Double), System.Double));
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Media.Animation.ExponentialEase.ExponentProperty, Bridge.box(value, System.Double, System.Double.format, System.Double.getHashCode));
                }
            }
        },
        methods: {
            GetAdditionalOptionsForVelocityCall: function () {
                var additionalOptions = new (System.Collections.Generic.Dictionary$2(System.String,System.Object)).ctor();
                additionalOptions.add("exponent", Bridge.box(this.Exponent, System.Double, System.Double.format, System.Double.getHashCode));
                return additionalOptions;
            },
            GetFunctionAsString: function () {
                if (!Windows.UI.Xaml.Media.Animation.ExponentialEase.velocityInitializationMade) {

                    
        Velocity.Easings.easeInExponentialCustomEasing = function (p, opts, tweenDelta) {
    var exponent = 2;
    if(opts != undefined) {
        if(opts.exponent != undefined) {
            exponent = opts.exponent;
        }
    }
    return (Math.exp(opts.exponent * p)- 1) / (Math.exp(opts.exponent) - 1);
};


                    
        Velocity.Easings.easeOutExponentialCustomEasing = function (p, opts, tweenDelta) {
    var exponent = 2;
    if(opts != undefined) {
        if(opts.exponent != undefined) {
            exponent = opts.exponent;
        }
    }
    return 1 - ((Math.exp(opts.exponent * (1-p))- 1) / (Math.exp(opts.exponent) - 1));
};


                    
        Velocity.Easings.easeInOutExponentialCustomEasing = function (p, opts, tweenDelta) {
    var exponent = 2;
    if(opts != undefined) {
        if(opts.exponent != undefined) {
            exponent = opts.exponent;
        }
    }
    if(p < 0.5) {
        return (Math.exp(opts.exponent * 2 * p)- 1) / (2 * (Math.exp(opts.exponent) - 1));
    }
    else {
        return 1 - ((Math.exp(2 * opts.exponent * (1 - p)) - 1) / (2 * (Math.exp(opts.exponent) - 1)));
    }
};

                    Windows.UI.Xaml.Media.Animation.ExponentialEase.velocityInitializationMade = true;
                }
                return (this.GetEasingModeAsString() || "") + (Windows.UI.Xaml.Media.Animation.ExponentialEase.FUNCTION_TYPE_STRING || "");
            }
        }
    });

    /**
     * Represents a collection of ObjectKeyFrame objects that can be individually
     accessed by index.
     *
     * @public
     * @class Windows.UI.Xaml.Media.Animation.ObjectKeyFrameCollection
     * @augments System.Collections.Generic.List$1
     */
    Bridge.define("Windows.UI.Xaml.Media.Animation.ObjectKeyFrameCollection", {
        inherits: [System.Collections.Generic.List$1(Windows.UI.Xaml.Media.Animation.ObjectKeyFrame)]
    });

    /**
     * Represents an easing function that creates an animation that accelerates
     and/or decelerates using the formula f(t) = t^2
     *
     * @public
     * @class Windows.UI.Xaml.Media.Animation.QuadraticEase
     * @augments Windows.UI.Xaml.Media.Animation.EasingFunctionBase
     */
    Bridge.define("Windows.UI.Xaml.Media.Animation.QuadraticEase", {
        inherits: [Windows.UI.Xaml.Media.Animation.EasingFunctionBase],
        statics: {
            fields: {
                FUNCTION_TYPE_STRING: null
            },
            ctors: {
                init: function () {
                    this.FUNCTION_TYPE_STRING = "Quad";
                }
            }
        },
        methods: {
            /**
             * @instance
             * @override
             * @this Windows.UI.Xaml.Media.Animation.QuadraticEase
             * @memberof Windows.UI.Xaml.Media.Animation.QuadraticEase
             * @return  {string}
             */
            GetFunctionAsString: function () {
                return (this.GetEasingModeAsString() || "") + (Windows.UI.Xaml.Media.Animation.QuadraticEase.FUNCTION_TYPE_STRING || "");
            }
        }
    });

    /**
     * Represents an easing function that creates an animation that accelerates
     and/or decelerates using the formula f(t) = t4.
     *
     * @public
     * @class Windows.UI.Xaml.Media.Animation.QuarticEase
     * @augments Windows.UI.Xaml.Media.Animation.EasingFunctionBase
     */
    Bridge.define("Windows.UI.Xaml.Media.Animation.QuarticEase", {
        inherits: [Windows.UI.Xaml.Media.Animation.EasingFunctionBase],
        statics: {
            fields: {
                FUNCTION_TYPE_STRING: null
            },
            ctors: {
                init: function () {
                    this.FUNCTION_TYPE_STRING = "Quart";
                }
            }
        },
        methods: {
            /**
             * @instance
             * @override
             * @this Windows.UI.Xaml.Media.Animation.QuarticEase
             * @memberof Windows.UI.Xaml.Media.Animation.QuarticEase
             * @return  {string}
             */
            GetFunctionAsString: function () {
                return (this.GetEasingModeAsString() || "") + (Windows.UI.Xaml.Media.Animation.QuarticEase.FUNCTION_TYPE_STRING || "");
            }
        }
    });

    /**
     * Represents an easing function that creates an animation that accelerates
     and/or decelerates using the formula f(t) = t5.
     *
     * @public
     * @class Windows.UI.Xaml.Media.Animation.QuinticEase
     * @augments Windows.UI.Xaml.Media.Animation.EasingFunctionBase
     */
    Bridge.define("Windows.UI.Xaml.Media.Animation.QuinticEase", {
        inherits: [Windows.UI.Xaml.Media.Animation.EasingFunctionBase],
        statics: {
            fields: {
                FUNCTION_TYPE_STRING: null
            },
            ctors: {
                init: function () {
                    this.FUNCTION_TYPE_STRING = "Quint";
                }
            }
        },
        methods: {
            /**
             * @instance
             * @override
             * @this Windows.UI.Xaml.Media.Animation.QuinticEase
             * @memberof Windows.UI.Xaml.Media.Animation.QuinticEase
             * @return  {string}
             */
            GetFunctionAsString: function () {
                return (this.GetEasingModeAsString() || "") + (Windows.UI.Xaml.Media.Animation.QuinticEase.FUNCTION_TYPE_STRING || "");
            }
        }
    });

    /**
     * Represents an easing function that creates an animation that accelerates
     and/or decelerates using a sine formula.
     *
     * @public
     * @class Windows.UI.Xaml.Media.Animation.SineEase
     * @augments Windows.UI.Xaml.Media.Animation.EasingFunctionBase
     */
    Bridge.define("Windows.UI.Xaml.Media.Animation.SineEase", {
        inherits: [Windows.UI.Xaml.Media.Animation.EasingFunctionBase],
        statics: {
            fields: {
                FUNCTION_TYPE_STRING: null
            },
            ctors: {
                init: function () {
                    this.FUNCTION_TYPE_STRING = "Sine";
                }
            }
        },
        methods: {
            /**
             * @instance
             * @override
             * @this Windows.UI.Xaml.Media.Animation.SineEase
             * @memberof Windows.UI.Xaml.Media.Animation.SineEase
             * @return  {string}
             */
            GetFunctionAsString: function () {
                return (this.GetEasingModeAsString() || "") + (Windows.UI.Xaml.Media.Animation.SineEase.FUNCTION_TYPE_STRING || "");
            }
        }
    });

    /**
     * Controls animations with a timeline, and provides object and property targeting
     information for its child animations.
     *
     * @public
     * @class Windows.UI.Xaml.Media.Animation.Storyboard
     * @augments Windows.UI.Xaml.Media.Animation.Timeline
     */
    Bridge.define("Windows.UI.Xaml.Media.Animation.Storyboard", {
        inherits: [Windows.UI.Xaml.Media.Animation.Timeline],
        statics: {
            fields: {
                /**
                 * Identifies the Storyboard.TargetName XAML attached property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Media.Animation.Storyboard
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                TargetNameProperty: null,
                /**
                 * Identifies the Storyboard.TargetProperty XAML attached property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Media.Animation.Storyboard
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                TargetPropertyProperty: null,
                /**
                 * Identifies the Storyboard.TargetName XAML attached property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Media.Animation.Storyboard
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                TargetProperty: null
            },
            ctors: {
                init: function () {
                    this.TargetNameProperty = Windows.UI.Xaml.DependencyProperty.RegisterAttached("TargetName", System.String, Windows.UI.Xaml.Media.Animation.Storyboard, new Windows.UI.Xaml.PropertyMetadata.$ctor4(null));
                    this.TargetPropertyProperty = Windows.UI.Xaml.DependencyProperty.RegisterAttached("TargetProperty", Windows.UI.Xaml.PropertyPath, Windows.UI.Xaml.Media.Animation.Storyboard, new Windows.UI.Xaml.PropertyMetadata.$ctor4(null));
                    this.TargetProperty = Windows.UI.Xaml.DependencyProperty.RegisterAttached("Target", Windows.UI.Xaml.DependencyObject, Windows.UI.Xaml.Media.Animation.Storyboard, new Windows.UI.Xaml.PropertyMetadata.$ctor4(null));
                }
            },
            methods: {
                /**
                 * Gets the value of the Storyboard.TargetName XAML attached property from a
                 target element.
                 *
                 * @static
                 * @public
                 * @this Windows.UI.Xaml.Media.Animation.Storyboard
                 * @memberof Windows.UI.Xaml.Media.Animation.Storyboard
                 * @param   {Windows.UI.Xaml.Media.Animation.Timeline}    element    The target element from which to get the value.
                 * @return  {string}                                                 The Storyboard.TargetName value of the target element.
                 */
                GetTargetName: function (element) {
                    return Bridge.cast(element.GetValue(Windows.UI.Xaml.Media.Animation.Storyboard.TargetNameProperty), System.String);
                },
                /**
                 * Sets the value of the Storyboard.TargetName XAML attached property for a
                 target element.
                 *
                 * @static
                 * @public
                 * @this Windows.UI.Xaml.Media.Animation.Storyboard
                 * @memberof Windows.UI.Xaml.Media.Animation.Storyboard
                 * @param   {Windows.UI.Xaml.Media.Animation.Timeline}    element    The target element to set the value for.
                 * @param   {string}                                      name       The Storyboard.TargetName value of the target element to set. This should
                 correspond to an existing Name or x:Name value on the element that the animation
                 targets.
                 * @return  {void}
                 */
                SetTargetName: function (element, name) {
                    element.SetValue(Windows.UI.Xaml.Media.Animation.Storyboard.TargetNameProperty, name);
                },
                /**
                 * Gets the value of the Storyboard.TargetProperty XAML attached property from
                 a target element.
                 *
                 * @static
                 * @public
                 * @this Windows.UI.Xaml.Media.Animation.Storyboard
                 * @memberof Windows.UI.Xaml.Media.Animation.Storyboard
                 * @param   {Windows.UI.Xaml.Media.Animation.Timeline}    element    The target element from which to get the value.
                 * @return  {Windows.UI.Xaml.PropertyPath}                           The Storyboard.TargetProperty value of the target element.
                 */
                GetTargetProperty: function (element) {
                    return Bridge.cast(element.GetValue(Windows.UI.Xaml.Media.Animation.Storyboard.TargetPropertyProperty), Windows.UI.Xaml.PropertyPath);
                },
                /**
                 * // Sets the value of the Storyboard.TargetProperty XAML attached property for
                // a target element.
                //
                 * Sets the value of the Storyboard.TargetProperty XAML attached property for
                 a target element.
                 *
                 * @static
                 * @public
                 * @this Windows.UI.Xaml.Media.Animation.Storyboard
                 * @memberof Windows.UI.Xaml.Media.Animation.Storyboard
                 * @param   {Windows.UI.Xaml.Media.Animation.Timeline}    element    The target element for which to set the value.
                 * @param   {Windows.UI.Xaml.PropertyPath}                value
                 * @return  {void}
                 */
                SetTargetProperty: function (element, value) {
                    element.SetValue(Windows.UI.Xaml.Media.Animation.Storyboard.TargetPropertyProperty, value);
                },
                /**
                 * // Gets the clock state of the Storyboard.
                //
                 * // Gets the current animation clock time of the Storyboard.
                //
                 * // Pauses the animation clock associated with the storyboard.
                //
                 * // Resumes the animation clock, or run-time state, associated with the storyboard.
                //
                 * Gets the value of the Storyboard.Target XAML attached property from a
                 target element.
                 *
                 * @static
                 * @public
                 * @this Windows.UI.Xaml.Media.Animation.Storyboard
                 * @memberof Windows.UI.Xaml.Media.Animation.Storyboard
                 * @param   {Windows.UI.Xaml.Media.Animation.Timeline}    element    The target element from which to get the value.
                 * @return  {Windows.UI.Xaml.DependencyObject}                       The Storyboard.Target value of the target element.
                 */
                GetTarget: function (element) {
                    return Bridge.cast(element.GetValue(Windows.UI.Xaml.Media.Animation.Storyboard.TargetProperty), Windows.UI.Xaml.DependencyObject);
                },
                /**
                 * Causes the specified Timeline to target the specified object.
                 *
                 * @static
                 * @public
                 * @this Windows.UI.Xaml.Media.Animation.Storyboard
                 * @memberof Windows.UI.Xaml.Media.Animation.Storyboard
                 * @param   {Windows.UI.Xaml.Media.Animation.Timeline}    element    The timeline that targets the specified dependency object.
                 * @param   {Windows.UI.Xaml.DependencyObject}            target     The actual instance of the object to target.
                 * @return  {void}
                 */
                SetTarget: function (element, target) {
                    element.SetValue(Windows.UI.Xaml.Media.Animation.Storyboard.TargetProperty, target);
                }
            }
        },
        fields: {
            INTERNAL_propertiesChanged: null,
            isUnApplied: false,
            _children: null,
            _expectedAmountOfTimelineEnds: 0,
            _expectedAmountOfTimelineEndsDict: null,
            thisLock: null,
            /**
             * @instance
             * @private
             * @memberof Windows.UI.Xaml.Media.Animation.Storyboard
             * @type System.Collections.Generic.Dictionary$2
             */
            _guidToIterationParametersDict: null
        },
        props: {
            /**
             * Gets the collection of child Timeline objects.
             *
             * @instance
             * @public
             * @readonly
             * @memberof Windows.UI.Xaml.Media.Animation.Storyboard
             * @function Children
             * @type Windows.UI.Xaml.Media.Animation.TimelineCollection
             */
            Children: {
                get: function () {
                    return this._children;
                }
            }
        },
        ctors: {
            init: function () {
                this.isUnApplied = false;
                this._children = new Windows.UI.Xaml.Media.Animation.TimelineCollection();
                this._expectedAmountOfTimelineEnds = 0;
                this._expectedAmountOfTimelineEndsDict = new (System.Collections.Generic.Dictionary$2(System.Guid,System.Int32)).ctor();
                this.thisLock = { };
                this._guidToIterationParametersDict = new (System.Collections.Generic.Dictionary$2(System.Guid,Windows.UI.Xaml.Media.Animation.IterationParameters)).ctor();
            }
        },
        methods: {
            GetPropertiesChanged: function () {
                var $t, $t1;
                if (this.INTERNAL_propertiesChanged == null) {
                    this.INTERNAL_propertiesChanged = new (System.Collections.Generic.Dictionary$2(System.Tuple$2(System.String,System.String),Windows.UI.Xaml.Media.Animation.Timeline)).ctor();
                    $t = Bridge.getEnumerator(this._children);
                    try {
                        while ($t.moveNext()) {
                            var timeLine = $t.Current;
                            if (Bridge.is(timeLine, Windows.UI.Xaml.Media.Animation.Storyboard)) {
                                var timelineAsStoryboard = Bridge.as(timeLine, Windows.UI.Xaml.Media.Animation.Storyboard);
                                var innerStoryboardPropertiesChanged = timelineAsStoryboard.GetPropertiesChanged();
                                $t1 = Bridge.getEnumerator(innerStoryboardPropertiesChanged.Keys);
                                try {
                                    while ($t1.moveNext()) {
                                        var key = $t1.Current;
                                        if (!this.INTERNAL_propertiesChanged.containsKey(key)) {
                                            this.INTERNAL_propertiesChanged.add(key, innerStoryboardPropertiesChanged.getItem(key));
                                        }
                                    }
                                } finally {
                                    if (Bridge.is($t1, System.IDisposable)) {
                                        $t1.System$IDisposable$Dispose();
                                    }
                                }
                            } else {
                                var key1 = { Item1: Windows.UI.Xaml.Media.Animation.Storyboard.GetTargetName(timeLine), Item2: Windows.UI.Xaml.Media.Animation.Storyboard.GetTargetProperty(timeLine).Path };
                                if (!this.INTERNAL_propertiesChanged.containsKey(key1)) {
                                    this.INTERNAL_propertiesChanged.add(key1, timeLine);
                                }
                            }
                        }
                    } finally {
                        if (Bridge.is($t, System.IDisposable)) {
                            $t.System$IDisposable$Dispose();
                        }
                    }
                }
                return this.INTERNAL_propertiesChanged;
            },
            /**
             * Initiates the set of animations associated with the storyboard.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Media.Animation.Storyboard
             * @memberof Windows.UI.Xaml.Media.Animation.Storyboard
             * @return  {void}
             */
            Begin: function () {
                var target = Bridge.cast(Windows.UI.Xaml.Media.Animation.Storyboard.GetTarget(this), Windows.UI.Xaml.FrameworkElement);
                this.Begin$1(target, true, "DirectlyStoryboard", false);
            },
            Begin$1: function (target, useTransitions, visualStateGroupName, isVisualStateChange) {
                this.isUnApplied = false;
                Windows.UI.Core.CoreDispatcher.INTERNAL_GetCurrentDispatcher().BeginInvoke(Bridge.fn.bind(this, function () {
                    var $t;
                    if (!this.isUnApplied) {
                        var guid = System.Guid.NewGuid();
                        var parameters = ($t = new Windows.UI.Xaml.Media.Animation.IterationParameters(), $t.Target = target, $t.Guid = guid, $t.UseTransitions = useTransitions, $t.VisualStateGroupName = visualStateGroupName, $t.IsVisualStateChange = isVisualStateChange, $t);

                        this.InitializeIteration();

                        var isThisSingleLoop = this.RepeatBehavior.Type === Windows.UI.Xaml.Media.Animation.RepeatBehaviorType.Count && this.RepeatBehavior.Count === 1;

                        this.StartFirstIteration(parameters, isThisSingleLoop, new System.TimeSpan());
                    }
                }));
            },
            timeLine_Completed: function (sender, e) {
                this.NotifyStoryboardOfTimelineEnd(Bridge.cast(sender, Windows.UI.Xaml.Media.Animation.Timeline));
            },
            NotifyStoryboardOfTimelineEnd: function (timeline) {
                var $t, $t1;
                var completedguids = timeline.CompletedGuids;
                var guidsDealtWith = new (System.Collections.Generic.List$1(System.Guid)).ctor();
                $t = Bridge.getEnumerator(completedguids);
                try {
                    while ($t.moveNext()) {
                        var guid = $t.Current;
                        var raiseEvent = false;
                        this.thisLock;
                        {
                            if (this._expectedAmountOfTimelineEndsDict.containsKey(guid)) {
                                var expectedAmount = this._expectedAmountOfTimelineEndsDict.getItem(guid);
                                this._expectedAmountOfTimelineEnds = (this._expectedAmountOfTimelineEnds - 1) | 0;
                                expectedAmount = (expectedAmount - 1) | 0;
                                this._expectedAmountOfTimelineEndsDict.setItem(guid, expectedAmount);
                                guidsDealtWith.add(guid);
                                if (expectedAmount <= 0) {
                                    raiseEvent = true;
                                }
                            }
                        }
                        if (raiseEvent) {
                            this.OnIterationCompleted(this._guidToIterationParametersDict.getItem(guid));
                            this._guidToIterationParametersDict.remove(guid);
                        }
                    }
                } finally {
                    if (Bridge.is($t, System.IDisposable)) {
                        $t.System$IDisposable$Dispose();
                    }
                }
                $t1 = Bridge.getEnumerator(guidsDealtWith);
                try {
                    while ($t1.moveNext()) {
                        var guid1 = $t1.Current;
                        timeline.CompletedGuids.remove(guid1);
                    }
                } finally {
                    if (Bridge.is($t1, System.IDisposable)) {
                        $t1.System$IDisposable$Dispose();
                    }
                }
            },
            /**
             * // Causes the specified Timeline to target the specified object.
            //
             * // Advances the current time of the storyboard's clock to the end of its active
            // period.
            //
             * Stops the storyboard.
             *
             * @instance
             * @this Windows.UI.Xaml.Media.Animation.Storyboard
             * @memberof Windows.UI.Xaml.Media.Animation.Storyboard
             * @param   {Windows.UI.Xaml.FrameworkElement}    frameworkElement    
             * @param   {string}                              groupName
             * @return  {void}
             */
            Stop$4: function (frameworkElement, groupName) {
                var $t;
                $t = Bridge.getEnumerator(this._children);
                try {
                    while ($t.moveNext()) {
                        var timeLine = $t.Current;
                        timeLine.Stop$1(frameworkElement, groupName);
                    }
                } finally {
                    if (Bridge.is($t, System.IDisposable)) {
                        $t.System$IDisposable$Dispose();
                    }
                }
            },
            /**
             * Stops the storyboard.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Media.Animation.Storyboard
             * @memberof Windows.UI.Xaml.Media.Animation.Storyboard
             * @param   {Windows.UI.Xaml.FrameworkElement}    frameworkElement
             * @return  {void}
             */
            Stop$3: function (frameworkElement) {
                var $t;
                Bridge.cast(this, Windows.UI.Xaml.Media.Animation.Timeline).Stop(frameworkElement, true);
                $t = Bridge.getEnumerator(this._children);
                try {
                    while ($t.moveNext()) {
                        var timeLine = $t.Current;
                        timeLine.Stop(frameworkElement, true);
                    }
                } finally {
                    if (Bridge.is($t, System.IDisposable)) {
                        $t.System$IDisposable$Dispose();
                    }
                }
            },
            Stop$2: function () {
                this.Stop$3(null);
            },
            IterateOnce: function (parameters, isLastLoop) {
                var $t, $t1, $t2;
                Windows.UI.Xaml.Media.Animation.Timeline.prototype.IterateOnce.call(this, parameters, isLastLoop);
                $t = Bridge.getEnumerator(this.Children);
                try {
                    while ($t.moveNext()) {
                        var timeline = $t.Current;
                        timeline.InitializeIteration();
                    }
                } finally {
                    if (Bridge.is($t, System.IDisposable)) {
                        $t.System$IDisposable$Dispose();
                    }
                }

                this.GetPropertiesChanged();

                this._expectedAmountOfTimelineEnds = this._children.Count;
                if (!this._expectedAmountOfTimelineEndsDict.containsKey(parameters.Guid)) {
                    this._expectedAmountOfTimelineEndsDict.add(parameters.Guid, this._children.Count);
                    this._guidToIterationParametersDict.add(parameters.Guid, parameters);
                } else {
                    this._expectedAmountOfTimelineEndsDict.setItem(parameters.Guid, this._children.Count);
                    this._guidToIterationParametersDict.setItem(parameters.Guid, parameters);
                }
                if (parameters.Target != null) {
                    $t1 = Bridge.getEnumerator(this._children);
                    try {
                        while ($t1.moveNext()) {
                            var timeLine = $t1.Current;
                            var currentParameters = parameters.Clone();
                            timeLine.removeCompleted(Bridge.fn.cacheBind(this, this.timeLine_Completed));
                            timeLine.addCompleted(Bridge.fn.cacheBind(this, this.timeLine_Completed));
                            currentParameters.IsTargetParentTheTarget = true;
                            if (currentParameters.IsVisualStateChange && Windows.UI.Xaml.Media.Animation.Storyboard.GetTargetName(timeLine) != null) {
                                currentParameters.IsTargetParentTheTarget = false;
                            }
                            var isTimelineSingleLoop = timeLine.RepeatBehavior.Type === Windows.UI.Xaml.Media.Animation.RepeatBehaviorType.Count && timeLine.RepeatBehavior.Count === 1;
                            timeLine.StartFirstIteration(currentParameters, isTimelineSingleLoop, this.BeginTime);
                        }
                    } finally {
                        if (Bridge.is($t1, System.IDisposable)) {
                            $t1.System$IDisposable$Dispose();
                        }
                    }
                } else {
                    $t2 = Bridge.getEnumerator(this._children);
                    try {
                        while ($t2.moveNext()) {
                            var timeLine1 = $t2.Current;
                            var target = Windows.UI.Xaml.Media.Animation.Storyboard.GetTarget(timeLine1);
                            if (Bridge.is(target, Windows.UI.Xaml.FrameworkElement)) {
                                parameters.Target = Bridge.cast(target, Windows.UI.Xaml.FrameworkElement);
                                timeLine1.removeCompleted(Bridge.fn.cacheBind(this, this.timeLine_Completed));
                                timeLine1.addCompleted(Bridge.fn.cacheBind(this, this.timeLine_Completed));
                                parameters.VisualStateGroupName = "visualStateGroupName";
                                parameters.IsTargetParentTheTarget = false;
                                var isTimelineSingleLoop1 = timeLine1.RepeatBehavior.Type === Windows.UI.Xaml.Media.Animation.RepeatBehaviorType.Count && timeLine1.RepeatBehavior.Count === 1;
                                timeLine1.StartFirstIteration(parameters, isTimelineSingleLoop1, this.BeginTime);
                            }
                        }
                    } finally {
                        if (Bridge.is($t2, System.IDisposable)) {
                            $t2.System$IDisposable$Dispose();
                        }
                    }
                }
            }
        }
    });

    /**
     * Represents a collection of Timeline objects.
     *
     * @public
     * @class Windows.UI.Xaml.Media.Animation.TimelineCollection
     * @augments System.Collections.Generic.List$1
     */
    Bridge.define("Windows.UI.Xaml.Media.Animation.TimelineCollection", {
        inherits: [System.Collections.Generic.List$1(Windows.UI.Xaml.Media.Animation.Timeline)]
    });

    /**
     * Represents a collection of Transition objects.
     * // Initializes a new instance of the TransitionCollection class.
    //
     *
     * @public
     * @class Windows.UI.Xaml.Media.Animation.TransitionCollection
     * @augments System.Collections.Generic.List$1
     */
    Bridge.define("Windows.UI.Xaml.Media.Animation.TransitionCollection", {
        inherits: [System.Collections.Generic.List$1(Windows.UI.Xaml.Media.Animation.Transition)]
    });

    /**
     * Represents an elliptical arc between two points.
     *
     * @public
     * @class Windows.UI.Xaml.Media.ArcSegment
     * @augments Windows.UI.Xaml.Media.PathSegment
     */
    Bridge.define("Windows.UI.Xaml.Media.ArcSegment", {
        inherits: [Windows.UI.Xaml.Media.PathSegment],
        statics: {
            fields: {
                /**
                 * Identifies the IsLargeArc dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Media.ArcSegment
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                IsLargeArcProperty: null,
                /**
                 * Identifies the Point dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Media.ArcSegment
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                PointProperty: null,
                /**
                 * Identifies the RotationAngle dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Media.ArcSegment
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                RotationAngleProperty: null,
                /**
                 * Identifies the Size dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Media.ArcSegment
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                SizeProperty: null,
                /**
                 * Identifies the SweepDirection dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Media.ArcSegment
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                SweepDirectionProperty: null
            },
            ctors: {
                init: function () {
                    this.IsLargeArcProperty = Windows.UI.Xaml.DependencyProperty.Register("IsLargeArc", System.Boolean, Windows.UI.Xaml.Media.ArcSegment, new Windows.UI.Xaml.PropertyMetadata.$ctor2(Bridge.box(false, System.Boolean, System.Boolean.toString), Windows.UI.Xaml.Media.ArcSegment.IsLargeArc_Changed));
                    this.PointProperty = Windows.UI.Xaml.DependencyProperty.Register("Point", Windows.Foundation.Point, Windows.UI.Xaml.Media.ArcSegment, new Windows.UI.Xaml.PropertyMetadata.$ctor2(new Windows.Foundation.Point.ctor().$clone(), Windows.UI.Xaml.Media.ArcSegment.Point_Changed));
                    this.RotationAngleProperty = Windows.UI.Xaml.DependencyProperty.Register("RotationAngle", System.Double, Windows.UI.Xaml.Media.ArcSegment, new Windows.UI.Xaml.PropertyMetadata.$ctor2(Bridge.box(0.0, System.Double, System.Double.format, System.Double.getHashCode), Windows.UI.Xaml.Media.ArcSegment.RotationAngle_Changed));
                    this.SizeProperty = Windows.UI.Xaml.DependencyProperty.Register("Size", Windows.Foundation.Size, Windows.UI.Xaml.Media.ArcSegment, new Windows.UI.Xaml.PropertyMetadata.$ctor2(new Windows.Foundation.Size.ctor().$clone(), Windows.UI.Xaml.Media.ArcSegment.Size_Changed));
                    this.SweepDirectionProperty = Windows.UI.Xaml.DependencyProperty.Register("SweepDirection", Windows.UI.Xaml.Media.SweepDirection, Windows.UI.Xaml.Media.ArcSegment, new Windows.UI.Xaml.PropertyMetadata.$ctor2(Bridge.box(Windows.UI.Xaml.Media.SweepDirection.Counterclockwise, Windows.UI.Xaml.Media.SweepDirection, System.Enum.toStringFn(Windows.UI.Xaml.Media.SweepDirection)), Windows.UI.Xaml.Media.ArcSegment.SweepDirection_Changed));
                }
            },
            methods: {
                IsLargeArc_Changed: function (d, e) {
                    var arc = Bridge.cast(d, Windows.UI.Xaml.Media.ArcSegment);
                    arc._isUpToDate = false;

                    if (arc.ParentPath != null) {
                        arc.ParentPath.ScheduleRedraw();
                    }
                },
                Point_Changed: function (d, e) {
                    var arc = Bridge.cast(d, Windows.UI.Xaml.Media.ArcSegment);
                    arc._isUpToDate = false;

                    if (arc.ParentPath != null) {
                        arc.ParentPath.ScheduleRedraw();
                    }
                },
                RotationAngle_Changed: function (d, e) {
                    var arc = Bridge.cast(d, Windows.UI.Xaml.Media.ArcSegment);
                    arc._isUpToDate = false;

                    if (arc.ParentPath != null) {
                        arc.ParentPath.ScheduleRedraw();
                    }
                },
                Size_Changed: function (d, e) {
                    var arc = Bridge.cast(d, Windows.UI.Xaml.Media.ArcSegment);
                    arc._isUpToDate = false;

                    if (arc.ParentPath != null) {
                        arc.ParentPath.ScheduleRedraw();
                    }
                },
                SweepDirection_Changed: function (d, e) {
                    var arc = Bridge.cast(d, Windows.UI.Xaml.Media.ArcSegment);
                    arc._isUpToDate = false;

                    if (arc.ParentPath != null) {
                        arc.ParentPath.ScheduleRedraw();
                    }
                }
            }
        },
        fields: {
            _ellipseCenterInCircleCoordinates: null,
            _angle1: 0,
            _angle2: 0,
            _minX: 0,
            _minY: 0,
            _maxX: 0,
            _maxY: 0,
            _startingPoint: null,
            _additionalScalingForShapetoSmallToReachEndPoint: 0,
            _strokeThickness: 0,
            _isUpToDate: false
        },
        props: {
            /**
             * Gets or sets a value that indicates whether the arc should be greater than
             180 degrees.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Media.ArcSegment
             * @function IsLargeArc
             * @type boolean
             */
            IsLargeArc: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.Media.ArcSegment.IsLargeArcProperty), System.Boolean), System.Boolean));
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Media.ArcSegment.IsLargeArcProperty, Bridge.box(value, System.Boolean, System.Boolean.toString));
                }
            },
            /**
             * Gets or sets the endpoint of the elliptical arc.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Media.ArcSegment
             * @function Point
             * @type Windows.Foundation.Point
             * @return  {[type]}        The point to which the arc is drawn. The default is a Point with value 0,0.
             */
            Point: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.Media.ArcSegment.PointProperty), Windows.Foundation.Point), Windows.Foundation.Point));
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Media.ArcSegment.PointProperty, value.$clone());
                }
            },
            /**
             * Gets or sets the amount (in degrees) by which the ellipse is rotated about
             the x-axis.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Media.ArcSegment
             * @function RotationAngle
             * @type number
             */
            RotationAngle: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.Media.ArcSegment.RotationAngleProperty), System.Double), System.Double));
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Media.ArcSegment.RotationAngleProperty, Bridge.box(value, System.Double, System.Double.format, System.Double.getHashCode));
                }
            },
            /**
             * Gets or sets the x-radius and y-radius of the arc as a Size structure.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Media.ArcSegment
             * @function Size
             * @type Windows.Foundation.Size
             * @return  {[type]}        A Size structure that describes the x-radius and y-radius of the elliptical
             arc. The Size structure's Width value specifies the arc's x-radius; its Height
             value specifies the arc's y-radius. The default is a Size with value 0,0.
             */
            Size: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.Media.ArcSegment.SizeProperty), Windows.Foundation.Size), Windows.Foundation.Size));
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Media.ArcSegment.SizeProperty, value.$clone());
                }
            },
            /**
             * Gets or sets a value that specifies whether the arc is drawn in the Clockwise
             or Counterclockwise direction.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Media.ArcSegment
             * @function SweepDirection
             * @type Windows.UI.Xaml.Media.SweepDirection
             */
            SweepDirection: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.Media.ArcSegment.SweepDirectionProperty), Windows.UI.Xaml.Media.SweepDirection), System.Int32));
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Media.ArcSegment.SweepDirectionProperty, Bridge.box(value, Windows.UI.Xaml.Media.SweepDirection, System.Enum.toStringFn(Windows.UI.Xaml.Media.SweepDirection)));
                }
            }
        },
        ctors: {
            init: function () {
                this._ellipseCenterInCircleCoordinates = new Windows.Foundation.Point();
                this._startingPoint = new Windows.Foundation.Point();
                this._additionalScalingForShapetoSmallToReachEndPoint = 1;
                this._isUpToDate = false;
            },
            /**
             * Initializes a new instance of the ArcSegment class.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Media.ArcSegment
             * @memberof Windows.UI.Xaml.Media.ArcSegment
             * @return  {void}
             */
            ctor: function () {
                this.$initialize();
                Windows.UI.Xaml.Media.PathSegment.ctor.call(this);

            }
        },
        methods: {
            DefineInCanvas: function (xOffsetToApplyBeforeMultiplication, yOffsetToApplyBeforeMultiplication, xOffsetToApplyAfterMultiplication, yOffsetToApplyAfterMultiplication, horizontalMultiplicator, verticalMultiplicator, canvasDomElement, previousLastPoint) {



                this.UpdateStartPosition(previousLastPoint.$clone());
                this.UpdateStrokeThickness(this.ParentPath.StrokeThickness);

                this.UpdateArcData();


                var context = CSHTML5.Internal.INTERNAL_HtmlDomManager.Get2dCanvasContext(canvasDomElement);
                context.save();
                context.translate(xOffsetToApplyBeforeMultiplication * horizontalMultiplicator + xOffsetToApplyAfterMultiplication, yOffsetToApplyBeforeMultiplication * verticalMultiplicator + yOffsetToApplyAfterMultiplication);
                context.scale(horizontalMultiplicator, verticalMultiplicator);
                context.rotate(this.RotationAngle * Math.PI / 180);
                var horizontalScaling = (this.Size.Width) / (this.Size.Height);
                context.scale(horizontalScaling, 1);

                var centerX = this._ellipseCenterInCircleCoordinates.X;
                var centerY = this._ellipseCenterInCircleCoordinates.Y;
                context.arc(centerX, centerY, this.Size.Height * this._additionalScalingForShapetoSmallToReachEndPoint, this._angle1, this._angle2, this.SweepDirection === Windows.UI.Xaml.Media.SweepDirection.Counterclockwise);

                context.restore();
                context.stroke();

                return this.Point.$clone();
            },
            GetMinMaxXY: function (minX, maxX, minY, maxY, startingPoint) {
                this.UpdateStartPosition(startingPoint.$clone());
                this.UpdateStrokeThickness(this.ParentPath.StrokeThickness);

                this.UpdateArcData();

                minX.v = Math.min(minX.v, this._minX);
                maxX.v = Math.max(maxX.v, this._maxX);
                minY.v = Math.min(minY.v, this._minY);
                maxY.v = Math.max(maxY.v, this._maxY);

                return this.Point.$clone();
            },
            GetMaxXY: function () {
                this.UpdateArcData();
                return new Windows.Foundation.Point.$ctor1(this._maxX, this._maxY);
            },
            FlattenArc: function (originalPt1, originalPt2, radiusX, radiusY, angleRotation, isLargeArc, isCounterclockwise, saveChanges, center, angle1, angle2) {
                var $t, $t1, $t2;
                this._startingPoint = originalPt1.$clone();
                this._additionalScalingForShapetoSmallToReachEndPoint = 1;








                var scalingOnX = radiusY / radiusX;
                var cosValue = Math.cos(-(angleRotation * Math.PI / 180));
                var sinValue = Math.sin(-(angleRotation * Math.PI / 180));
                var pt1 = new Windows.Foundation.Point.$ctor1(originalPt1.X * scalingOnX * cosValue - originalPt1.Y * scalingOnX * sinValue, originalPt1.X * sinValue + originalPt1.Y * cosValue);
                var pt2 = new Windows.Foundation.Point.$ctor1(originalPt2.X * scalingOnX * cosValue - originalPt2.Y * scalingOnX * sinValue, originalPt2.X * sinValue + originalPt2.Y * cosValue);

                var midPoint = new Windows.Foundation.Point.$ctor1((pt1.X + pt2.X) / 2, (pt1.Y + pt2.Y) / 2);
                var vect = new Windows.Foundation.Point.$ctor1(pt2.X - pt1.X, pt2.Y - pt1.Y);
                var vectorLength = Math.sqrt((Math.pow(vect.X, 2) + (Math.pow(vect.Y, 2))));
                var halfChord = vectorLength / 2;


                if (halfChord > radiusY) {

                    this._additionalScalingForShapetoSmallToReachEndPoint = halfChord / radiusY;

                }



                var vectRotated = new Windows.Foundation.Point();

                if (isLargeArc === isCounterclockwise) {
                    vectRotated = new Windows.Foundation.Point.$ctor1(-vect.Y, vect.X);
                } else {
                    vectRotated = new Windows.Foundation.Point.$ctor1(vect.Y, -vect.X);
                }

                vectRotated.X = vectRotated.X / vectorLength;
                vectRotated.Y = vectRotated.Y / vectorLength;

                var squaredRadiusAfterScaling = this._additionalScalingForShapetoSmallToReachEndPoint * this._additionalScalingForShapetoSmallToReachEndPoint * radiusY * radiusY;

                var centerDistance = Math.sqrt(Math.abs(squaredRadiusAfterScaling - halfChord * halfChord));

                center.v = new Windows.Foundation.Point.$ctor1(midPoint.X + centerDistance * vectRotated.X, midPoint.Y + centerDistance * vectRotated.Y);
                if (saveChanges) {
                    this._ellipseCenterInCircleCoordinates = center.v.$clone();


                    var aSquared = Math.pow(this.Size.Width * this._additionalScalingForShapetoSmallToReachEndPoint / 2, 2);
                    var bSquared = Math.pow(this.Size.Height * this._additionalScalingForShapetoSmallToReachEndPoint / 2, 2);
                    var cos = Math.cos(this.RotationAngle * Math.PI / 180);
                    var sin = Math.sin(this.RotationAngle * Math.PI / 180);
                    var cosSquared = Math.pow(cos, 2);
                    var sinSquared = Math.pow(sin, 2);
                    var xLimitsOffset = 2 * Math.sqrt(aSquared * cosSquared + bSquared * sinSquared);
                    var yLimitsOffset = 2 * Math.sqrt(aSquared * sinSquared + bSquared * cosSquared);


                    var finalCenterX = (1 / scalingOnX) * (this._ellipseCenterInCircleCoordinates.X * cosValue + scalingOnX * this._ellipseCenterInCircleCoordinates.Y * sinValue);
                    var finalCenterY = (1 / scalingOnX) * (scalingOnX * this._ellipseCenterInCircleCoordinates.Y * cosValue - this._ellipseCenterInCircleCoordinates.X * sinValue);

                    this._minX = finalCenterX - xLimitsOffset - (this._strokeThickness / 2);
                    this._maxX = finalCenterX + xLimitsOffset + (this._strokeThickness / 2);
                    this._minY = finalCenterY - yLimitsOffset - (this._strokeThickness / 2);
                    this._maxY = finalCenterY + yLimitsOffset + (this._strokeThickness / 2);

                    var equationA = System.Double.max;
                    var equationB;
                    var groupAbove = new (System.Collections.Generic.List$1(Windows.Foundation.Point)).ctor();
                    var groupBelow = new (System.Collections.Generic.List$1(Windows.Foundation.Point)).ctor();
                    if (originalPt1.X !== originalPt2.X) {
                        equationA = (originalPt2.Y - originalPt1.Y) / (originalPt2.X - originalPt1.X);
                        equationB = originalPt1.Y - (equationA * originalPt1.X);
                    } else {
                        equationB = originalPt1.X;
                    }

                    this.PutPointInCorrectGroup(equationA, equationB, new Windows.Foundation.Point.$ctor1(this._minX, this._minY), groupAbove, groupBelow);
                    this.PutPointInCorrectGroup(equationA, equationB, new Windows.Foundation.Point.$ctor1(this._minX, this._maxY), groupAbove, groupBelow);
                    this.PutPointInCorrectGroup(equationA, equationB, new Windows.Foundation.Point.$ctor1(this._maxX, this._minY), groupAbove, groupBelow);
                    this.PutPointInCorrectGroup(equationA, equationB, new Windows.Foundation.Point.$ctor1(this._maxX, this._maxY), groupAbove, groupBelow);

                    var aboveDistance = -1;
                    $t = Bridge.getEnumerator(groupAbove);
                    try {
                        while ($t.moveNext()) {
                            var point = $t.Current.$clone();
                            var currentDistance = Math.abs(equationA * point.X - point.Y + equationB);
                            if (currentDistance > aboveDistance) {
                                aboveDistance = currentDistance;
                            }
                        }
                    } finally {
                        if (Bridge.is($t, System.IDisposable)) {
                            $t.System$IDisposable$Dispose();
                        }
                    }
                    var belowDistance = -1;
                    $t1 = Bridge.getEnumerator(groupBelow);
                    try {
                        while ($t1.moveNext()) {
                            var point1 = $t1.Current.$clone();
                            var currentDistance1 = Math.abs(equationA * point1.X - point1.Y + equationB);
                            if (currentDistance1 > belowDistance) {
                                belowDistance = currentDistance1;
                            }
                        }
                    } finally {
                        if (Bridge.is($t1, System.IDisposable)) {
                            $t1.System$IDisposable$Dispose();
                        }
                    }

                    var relevantPoints;
                    if (aboveDistance === belowDistance) {
                        var isUpsideDown = originalPt2.X < originalPt1.X;
                        if (originalPt1.X === originalPt2.X) {
                            isUpsideDown = originalPt2.Y < originalPt1.Y;
                        }
                        if (isCounterclockwise != isUpsideDown) {
                            relevantPoints = groupBelow;
                        } else {
                            relevantPoints = groupAbove;
                        }
                    } else if (aboveDistance > belowDistance === isLargeArc) {
                        relevantPoints = groupAbove;
                    } else {
                        relevantPoints = groupBelow;
                    }

                    this._minX = originalPt1.X;
                    this._maxX = originalPt1.X;
                    this._minY = originalPt1.Y;
                    this._maxY = originalPt1.Y;
                    if (originalPt2.X < this._minX) {
                        this._minX = originalPt2.X;
                    }
                    if (originalPt2.X > this._maxX) {
                        this._maxX = originalPt2.X;
                    }
                    if (originalPt2.Y < this._minY) {
                        this._minY = originalPt2.Y;
                    }
                    if (originalPt2.Y > this._maxY) {
                        this._maxY = originalPt2.Y;
                    }
                    $t2 = Bridge.getEnumerator(relevantPoints);
                    try {
                        while ($t2.moveNext()) {
                            var point2 = $t2.Current.$clone();
                            if (point2.X < this._minX) {
                                this._minX = point2.X;
                            }
                            if (point2.X > this._maxX) {
                                this._maxX = point2.X;
                            }
                            if (point2.Y < this._minY) {
                                this._minY = point2.Y;
                            }
                            if (point2.Y > this._maxY) {
                                this._maxY = point2.Y;
                            }
                        }
                    } finally {
                        if (Bridge.is($t2, System.IDisposable)) {
                            $t2.System$IDisposable$Dispose();
                        }
                    }
                }
                angle1.v = Math.atan2(pt1.Y - center.v.Y, pt1.X - center.v.X);
                angle2.v = Math.atan2(pt2.Y - center.v.Y, pt2.X - center.v.X);

                if (isLargeArc === (Math.abs(angle2.v - angle1.v) < Math.PI)) {
                    if (angle1.v < angle2.v) {
                        angle1.v += 6.2831853071795862;
                    } else {
                        angle2.v += 6.2831853071795862;
                    }
                }
                if (saveChanges) {
                    this._angle1 = angle1.v;
                    this._angle2 = angle2.v;
                    this._isUpToDate = true;
                }
            },
            PutPointInCorrectGroup: function (equationA, equationB, point, groupAbove, groupBelow) {
                if (!(equationA === Number.POSITIVE_INFINITY)) {
                    var OrdinateOnXOnReferenceLine = equationA * this._minX + equationB;
                    if (point.Y > OrdinateOnXOnReferenceLine) {
                        groupAbove.add(point.$clone());
                    } else {
                        groupBelow.add(point.$clone());
                    }
                } else {
                    if (point.X > equationB) {
                        groupAbove.add(point.$clone());
                    } else {
                        groupBelow.add(point.$clone());
                    }
                }
            },
            UpdateArcData: function () {
                if (!this._isUpToDate) {
                    var angle1 = { }, angle2 = { };
                    var point = { v : new Windows.Foundation.Point() };
                    this.FlattenArc(this._startingPoint.$clone(), this.Point.$clone(), this.Size.Width, this.Size.Height, this.RotationAngle, this.IsLargeArc, this.SweepDirection === Windows.UI.Xaml.Media.SweepDirection.Counterclockwise, true, point, angle1, angle2);
                }
            },
            UpdateStartPosition: function (newStartingPoint) {
                if (Windows.Foundation.Point.op_Inequality(this._startingPoint.$clone(), newStartingPoint.$clone())) {
                    this._isUpToDate = false;
                    this._startingPoint = newStartingPoint.$clone();
                }
            },
            UpdateStrokeThickness: function (newStrokeThickness) {
                if (newStrokeThickness !== this._strokeThickness) {
                    this._strokeThickness = newStrokeThickness;
                    this._isUpToDate = false;
                }
            }
        }
    });

    /**
     * Represents a cubic Bezier curve drawn between two points.
     *
     * @public
     * @class Windows.UI.Xaml.Media.BezierSegment
     * @augments Windows.UI.Xaml.Media.PathSegment
     */
    Bridge.define("Windows.UI.Xaml.Media.BezierSegment", {
        inherits: [Windows.UI.Xaml.Media.PathSegment],
        statics: {
            fields: {
                /**
                 * Identifies the Point1 dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Media.BezierSegment
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                Point1Property: null,
                /**
                 * Identifies the Point2 dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Media.BezierSegment
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                Point2Property: null,
                /**
                 * Identifies the Point3 dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Media.BezierSegment
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                Point3Property: null
            },
            ctors: {
                init: function () {
                    this.Point1Property = Windows.UI.Xaml.DependencyProperty.Register("Point1", Windows.Foundation.Point, Windows.UI.Xaml.Media.BezierSegment, new Windows.UI.Xaml.PropertyMetadata.$ctor2(new Windows.Foundation.Point.ctor().$clone(), Windows.UI.Xaml.Media.BezierSegment.Point1_Changed));
                    this.Point2Property = Windows.UI.Xaml.DependencyProperty.Register("Point2", Windows.Foundation.Point, Windows.UI.Xaml.Media.BezierSegment, new Windows.UI.Xaml.PropertyMetadata.$ctor2(new Windows.Foundation.Point.ctor().$clone(), Windows.UI.Xaml.Media.BezierSegment.Point2_Changed));
                    this.Point3Property = Windows.UI.Xaml.DependencyProperty.Register("Point3", Windows.Foundation.Point, Windows.UI.Xaml.Media.BezierSegment, new Windows.UI.Xaml.PropertyMetadata.$ctor2(new Windows.Foundation.Point.ctor().$clone(), Windows.UI.Xaml.Media.BezierSegment.Point3_Changed));
                }
            },
            methods: {
                Point1_Changed: function (d, e) {
                    var segment = Bridge.cast(d, Windows.UI.Xaml.Media.BezierSegment);
                    if (segment.ParentPath != null) {
                        segment.ParentPath.ScheduleRedraw();
                    }
                },
                Point2_Changed: function (d, e) {
                    var segment = Bridge.cast(d, Windows.UI.Xaml.Media.BezierSegment);
                    if (segment.ParentPath != null) {
                        segment.ParentPath.ScheduleRedraw();
                    }
                },
                Point3_Changed: function (d, e) {
                    var segment = Bridge.cast(d, Windows.UI.Xaml.Media.BezierSegment);
                    if (segment.ParentPath != null) {
                        segment.ParentPath.ScheduleRedraw();
                    }
                }
            }
        },
        props: {
            /**
             * // Initializes a new instance of the BezierSegment class.
            //
             * Gets or sets the first control point of the curve.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Media.BezierSegment
             * @function Point1
             * @type Windows.Foundation.Point
             */
            Point1: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.Media.BezierSegment.Point1Property), Windows.Foundation.Point), Windows.Foundation.Point));
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Media.BezierSegment.Point1Property, value.$clone());
                }
            },
            /**
             * Gets or sets the second control point of the curve.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Media.BezierSegment
             * @function Point2
             * @type Windows.Foundation.Point
             */
            Point2: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.Media.BezierSegment.Point2Property), Windows.Foundation.Point), Windows.Foundation.Point));
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Media.BezierSegment.Point2Property, value.$clone());
                }
            },
            /**
             * Gets or sets the end point of the curve.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Media.BezierSegment
             * @function Point3
             * @type Windows.Foundation.Point
             */
            Point3: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.Media.BezierSegment.Point3Property), Windows.Foundation.Point), Windows.Foundation.Point));
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Media.BezierSegment.Point3Property, value.$clone());
                }
            }
        },
        methods: {
            DefineInCanvas: function (xOffsetToApplyBeforeMultiplication, yOffsetToApplyBeforeMultiplication, xOffsetToApplyAfterMultiplication, yOffsetToApplyAfterMultiplication, horizontalMultiplicator, verticalMultiplicator, canvasDomElement, previousLastPoint) {
                var context = CSHTML5.Internal.INTERNAL_HtmlDomManager.Get2dCanvasContext(canvasDomElement);

                context.bezierCurveTo((this.Point1.X + xOffsetToApplyBeforeMultiplication) * horizontalMultiplicator + xOffsetToApplyAfterMultiplication, (this.Point1.Y + yOffsetToApplyBeforeMultiplication) * verticalMultiplicator + yOffsetToApplyAfterMultiplication, (this.Point2.X + xOffsetToApplyBeforeMultiplication) * horizontalMultiplicator + xOffsetToApplyAfterMultiplication, (this.Point2.Y + yOffsetToApplyBeforeMultiplication) * verticalMultiplicator + yOffsetToApplyAfterMultiplication, (this.Point3.X + xOffsetToApplyBeforeMultiplication) * horizontalMultiplicator + xOffsetToApplyAfterMultiplication, (this.Point3.Y + yOffsetToApplyBeforeMultiplication) * verticalMultiplicator + yOffsetToApplyAfterMultiplication);
                return this.Point3.$clone();
            },
            GetMaxXY: function () {
                return new Windows.Foundation.Point.$ctor1(Math.max(this.Point1.X, Math.max(this.Point2.X, this.Point3.X)), Math.max(this.Point1.Y, Math.max(this.Point2.Y, this.Point3.Y)));
            },
            GetMinMaxXY: function (minX, maxX, minY, maxY, startingPoint) {
                minX.v = Math.min(minX.v, Math.min(this.Point1.X, Math.min(this.Point2.X, this.Point3.X)));
                maxX.v = Math.max(maxX.v, Math.max(this.Point1.X, Math.max(this.Point2.X, this.Point3.X)));
                minY.v = Math.min(minY.v, Math.min(this.Point1.Y, Math.min(this.Point2.Y, this.Point3.Y)));
                maxY.v = Math.max(maxY.v, Math.max(this.Point1.Y, Math.max(this.Point2.Y, this.Point3.Y)));

                return this.Point3.$clone();
            }
        }
    });

    /**
     * Defines functionality that enables transformations in a two-dimensional plane.
     *
     * @abstract
     * @public
     * @class Windows.UI.Xaml.Media.Transform
     * @augments Windows.UI.Xaml.Media.GeneralTransform
     */
    Bridge.define("Windows.UI.Xaml.Media.Transform", {
        inherits: [Windows.UI.Xaml.Media.GeneralTransform]
    });

    /**
     * Represents an ordered collection of Double values.
     *
     * @public
     * @class Windows.UI.Xaml.Media.DoubleCollection
     * @augments Windows.UI.Xaml.PresentationFrameworkCollection$1
     */
    Bridge.define("Windows.UI.Xaml.Media.DoubleCollection", {
        inherits: [Windows.UI.Xaml.PresentationFrameworkCollection$1(System.Double)],
        statics: {
            ctors: {
                ctor: function () {
                    DotNetForHtml5.Core.TypeFromStringConverters.RegisterConverter(Windows.UI.Xaml.Media.DoubleCollection, Windows.UI.Xaml.Media.DoubleCollection.INTERNAL_ConvertFromString);
                }
            },
            methods: {
                INTERNAL_ConvertFromString: function (doubleCollectionAsString) {
                    var $t;
                    var separator = 32;
                    if (System.String.contains(doubleCollectionAsString.trim(),",")) {
                        separator = 44;
                    }
                    var split = System.String.split(doubleCollectionAsString, [separator].map(function (i) {{ return String.fromCharCode(i); }}));
                    var doubleCollection = new Windows.UI.Xaml.Media.DoubleCollection();
                    $t = Bridge.getEnumerator(split);
                    try {
                        while ($t.moveNext()) {
                            var element = $t.Current;
                            if (!System.String.isNullOrWhiteSpace(element)) {
                                doubleCollection.add(System.Double.parse(element));
                            }
                        }
                    } finally {
                        if (Bridge.is($t, System.IDisposable)) {
                            $t.System$IDisposable$Dispose();
                        }
                    }
                    return doubleCollection;
                }
            }
        },
        ctors: {
            ctor: function () {
                this.$initialize();
                Windows.UI.Xaml.PresentationFrameworkCollection$1(System.Double).ctor.call(this);

            }
        },
        methods: {
            AddOverride: function (value) {
                this.AddInternal(value);
            },
            ClearOverride: function () {
                this.ClearInternal();
            },
            InsertOverride: function (index, value) {
                this.InsertInternal(index, value);
            },
            RemoveAtOverride: function (index) {
                this.RemoveAtInternal(index);
            },
            RemoveOverride: function (value) {
                return this.RemoveInternal(value);
            },
            GetItemOverride: function (index) {
                return this.GetItemInternal(index);
            },
            SetItemOverride: function (index, value) {
                this.SetItemInternal(index, value);
            }
        }
    });

    /**
     * Represents the geometry of a circle or ellipse.
     *
     * @public
     * @class Windows.UI.Xaml.Media.EllipseGeometry
     * @augments Windows.UI.Xaml.Media.Geometry
     */
    Bridge.define("Windows.UI.Xaml.Media.EllipseGeometry", {
        inherits: [Windows.UI.Xaml.Media.Geometry],
        statics: {
            fields: {
                /**
                 * Identifies the Center dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Media.EllipseGeometry
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                CenterProperty: null,
                /**
                 * Identifies the RadiusX dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Media.EllipseGeometry
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                RadiusXProperty: null,
                /**
                 * Identifies the RadiusY dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Media.EllipseGeometry
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                RadiusYProperty: null
            },
            ctors: {
                init: function () {
                    this.CenterProperty = Windows.UI.Xaml.DependencyProperty.Register("Center", Windows.Foundation.Point, Windows.UI.Xaml.Media.EllipseGeometry, new Windows.UI.Xaml.PropertyMetadata.$ctor2(new Windows.Foundation.Point.ctor().$clone(), Windows.UI.Xaml.Media.EllipseGeometry.Point_Changed));
                    this.RadiusXProperty = Windows.UI.Xaml.DependencyProperty.Register("RadiusX", System.Double, Windows.UI.Xaml.Media.EllipseGeometry, new Windows.UI.Xaml.PropertyMetadata.$ctor2(Bridge.box(0.0, System.Double, System.Double.format, System.Double.getHashCode), Windows.UI.Xaml.Media.EllipseGeometry.RadiusX_Changed));
                    this.RadiusYProperty = Windows.UI.Xaml.DependencyProperty.Register("RadiusY", System.Double, Windows.UI.Xaml.Media.EllipseGeometry, new Windows.UI.Xaml.PropertyMetadata.$ctor2(Bridge.box(0.0, System.Double, System.Double.format, System.Double.getHashCode), Windows.UI.Xaml.Media.EllipseGeometry.RadiusY_Changed));
                }
            },
            methods: {
                Point_Changed: function (d, e) {
                    var geometry = Bridge.cast(d, Windows.UI.Xaml.Media.EllipseGeometry);
                    if (geometry.ParentPath != null && geometry.ParentPath._isLoaded) {
                        geometry.ParentPath.ScheduleRedraw();
                    }
                },
                RadiusX_Changed: function (d, e) {
                    var geometry = Bridge.cast(d, Windows.UI.Xaml.Media.EllipseGeometry);
                    if (geometry.ParentPath != null && geometry.ParentPath._isLoaded) {
                        geometry.ParentPath.ScheduleRedraw();
                    }
                },
                RadiusY_Changed: function (d, e) {
                    var geometry = Bridge.cast(d, Windows.UI.Xaml.Media.EllipseGeometry);
                    if (geometry.ParentPath != null && geometry.ParentPath._isLoaded) {
                        geometry.ParentPath.ScheduleRedraw();
                    }
                }
            }
        },
        props: {
            /**
             * Gets or sets the center point of the EllipseGeometry.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Media.EllipseGeometry
             * @function Center
             * @type Windows.Foundation.Point
             */
            Center: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.Media.EllipseGeometry.CenterProperty), Windows.Foundation.Point), Windows.Foundation.Point));
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Media.EllipseGeometry.CenterProperty, value.$clone());
                }
            },
            /**
             * Gets or sets the x-radius value of the EllipseGeometry.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Media.EllipseGeometry
             * @function RadiusX
             * @type number
             */
            RadiusX: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.Media.EllipseGeometry.RadiusXProperty), System.Double), System.Double));
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Media.EllipseGeometry.RadiusXProperty, Bridge.box(value, System.Double, System.Double.format, System.Double.getHashCode));
                }
            },
            /**
             * Gets or sets the y-radius value of the EllipseGeometry.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Media.EllipseGeometry
             * @function RadiusY
             * @type number
             */
            RadiusY: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.Media.EllipseGeometry.RadiusYProperty), System.Double), System.Double));
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Media.EllipseGeometry.RadiusYProperty, Bridge.box(value, System.Double, System.Double.format, System.Double.getHashCode));
                }
            }
        },
        ctors: {
            /**
             * Initializes a new instance of the EllipseGeometry class.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Media.EllipseGeometry
             * @memberof Windows.UI.Xaml.Media.EllipseGeometry
             * @return  {void}
             */
            ctor: function () {
                this.$initialize();
                Windows.UI.Xaml.Media.Geometry.ctor.call(this);

            }
        },
        methods: {
            GetMinMaxXY: function (minX, maxX, minY, maxY) {
                var maxAbs = this.Center.X + this.RadiusX;
                var minAbs = this.Center.X - this.RadiusX;
                var minOrd = this.Center.Y - this.RadiusY;
                var maxOrd = this.Center.Y + this.RadiusY;
                minX.v = Math.min(minX.v, minAbs);
                maxX.v = Math.max(maxX.v, maxAbs);
                minY.v = Math.min(minY.v, minOrd);
                maxY.v = Math.max(maxY.v, maxOrd);
            },
            DefineInCanvas: function (path, canvasDomElement, horizontalMultiplicator, verticalMultiplicator, xOffsetToApplyBeforeMultiplication, yOffsetToApplyBeforeMultiplication, xOffsetToApplyAfterMultiplication, yOffsetToApplyAfterMultiplication, shapeActualSize) {
                var strokeAsString = "";

                if (path.Stroke == null || Bridge.is(path.Stroke, Windows.UI.Xaml.Media.SolidColorBrush)) {
                    if (path.Stroke != null) {
                        strokeAsString = Bridge.cast(path.Stroke, Windows.UI.Xaml.Media.SolidColorBrush).INTERNAL_ToHtmlString();
                    }
                } else {
                    throw new System.NotSupportedException.$ctor1("The specified brush is not supported.");
                }
                var context = CSHTML5.Internal.INTERNAL_HtmlDomManager.Get2dCanvasContext(canvasDomElement);
                var actualWidth = this.RadiusX * 2;
                var actualHeight = this.RadiusY * 2;

                CSHTML5.Internal.INTERNAL_ShapesDrawHelpers.PrepareEllipse(canvasDomElement, actualWidth, actualHeight, this.Center.X + xOffsetToApplyBeforeMultiplication + xOffsetToApplyAfterMultiplication, this.Center.Y + yOffsetToApplyBeforeMultiplication + yOffsetToApplyAfterMultiplication);
                context.strokeStyle = strokeAsString;
            }
        }
    });

    /**
     * An abstract class that describes a gradient, composed of gradient stops.
     Classes that derive from GradientBrush describe different ways of interpreting
     gradient stops.
     *
     * @public
     * @class Windows.UI.Xaml.Media.GradientBrush
     * @augments Windows.UI.Xaml.Media.Brush
     */
    Bridge.define("Windows.UI.Xaml.Media.GradientBrush", {
        inherits: [Windows.UI.Xaml.Media.Brush],
        statics: {
            fields: {
                /**
                 * Identifies the GradientStops dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Media.GradientBrush
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                GradientStopsProperty: null,
                /**
                 * Identifies the MappingMode dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Media.GradientBrush
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                MappingModeProperty: null,
                /**
                 * Identifies the SpreadMethod dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Media.GradientBrush
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                SpreadMethodProperty: null
            },
            ctors: {
                init: function () {
                    var $t;
                    this.GradientStopsProperty = Windows.UI.Xaml.DependencyProperty.Register("GradientStops", Windows.UI.Xaml.Media.GradientStopCollection, Windows.UI.Xaml.Media.GradientBrush, ($t = new Windows.UI.Xaml.PropertyMetadata.$ctor2(null, Windows.UI.Xaml.Media.GradientBrush.OnGradientStopsChanged), $t.CallPropertyChangedWhenLoadedIntoVisualTree = Windows.UI.Xaml.WhenToCallPropertyChangedEnum.IfPropertyIsSet, $t));
                    this.MappingModeProperty = Windows.UI.Xaml.DependencyProperty.Register("MappingMode", Windows.UI.Xaml.Media.BrushMappingMode, Windows.UI.Xaml.Media.GradientBrush, new Windows.UI.Xaml.PropertyMetadata.$ctor1(Bridge.box(Windows.UI.Xaml.Media.BrushMappingMode.RelativeToBoundingBox, Windows.UI.Xaml.Media.BrushMappingMode, System.Enum.toStringFn(Windows.UI.Xaml.Media.BrushMappingMode))));
                    this.SpreadMethodProperty = Windows.UI.Xaml.DependencyProperty.Register("SpreadMethod", Windows.UI.Xaml.Media.GradientSpreadMethod, Windows.UI.Xaml.Media.GradientBrush, new Windows.UI.Xaml.PropertyMetadata.$ctor1(Bridge.box(Windows.UI.Xaml.Media.GradientSpreadMethod.Pad, Windows.UI.Xaml.Media.GradientSpreadMethod, System.Enum.toStringFn(Windows.UI.Xaml.Media.GradientSpreadMethod))));
                }
            },
            methods: {
                OnGradientStopsChanged: function (d, e) { }
            }
        },
        props: {
            /**
             * // Gets or sets a ColorInterpolationMode enumeration value that specifies how
            // the gradient's colors are interpolated.
            //
             * // Identifies the ColorInterpolationMode dependency property.
            //
             * Gets or sets the brush's gradient stops, which is a collection of the GradientStop
             objects associated with the brush, each of which specifies a color and an offset
             along the brush's gradient axis. The default is an empty GradientStopCollection.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Media.GradientBrush
             * @function GradientStops
             * @type Windows.UI.Xaml.Media.GradientStopCollection
             */
            GradientStops: {
                get: function () {
                    var collection = Bridge.cast(this.GetValue(Windows.UI.Xaml.Media.GradientBrush.GradientStopsProperty), Windows.UI.Xaml.Media.GradientStopCollection);
                    if (collection == null) {
                        collection = new Windows.UI.Xaml.Media.GradientStopCollection();
                        this.SetValue(Windows.UI.Xaml.Media.GradientBrush.GradientStopsProperty, collection);
                    }
                    return collection;
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Media.GradientBrush.GradientStopsProperty, value);
                }
            },
            /**
             * Gets or sets a BrushMappingMode enumeration value that specifies whether
             the positioning coordinates of the gradient brush are absolute or relative
             to the output area.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Media.GradientBrush
             * @function MappingMode
             * @type Windows.UI.Xaml.Media.BrushMappingMode
             */
            MappingMode: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.Media.GradientBrush.MappingModeProperty), Windows.UI.Xaml.Media.BrushMappingMode), System.Int32));
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Media.GradientBrush.MappingModeProperty, Bridge.box(value, Windows.UI.Xaml.Media.BrushMappingMode, System.Enum.toStringFn(Windows.UI.Xaml.Media.BrushMappingMode)));
                }
            },
            /**
             * Gets or sets the type of spread method that specifies how to draw a gradient
             that starts or ends inside the bounds of the object to be painted.
             The default is Pad.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Media.GradientBrush
             * @function SpreadMethod
             * @type Windows.UI.Xaml.Media.GradientSpreadMethod
             */
            SpreadMethod: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.Media.GradientBrush.SpreadMethodProperty), Windows.UI.Xaml.Media.GradientSpreadMethod), System.Int32));
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Media.GradientBrush.SpreadMethodProperty, Bridge.box(value, Windows.UI.Xaml.Media.GradientSpreadMethod, System.Enum.toStringFn(Windows.UI.Xaml.Media.GradientSpreadMethod)));
                }
            }
        }
    });

    /**
     * Represents a collection of GradientStop objects that can be individually
     accessed by index.
     *
     * @public
     * @class Windows.UI.Xaml.Media.GradientStopCollection
     * @augments System.Collections.Generic.List$1
     */
    Bridge.define("Windows.UI.Xaml.Media.GradientStopCollection", {
        inherits: [System.Collections.Generic.List$1(Windows.UI.Xaml.Media.GradientStop)]
    });

    /**
     * Provides a source object for properties that use a bitmap.
     *
     * @public
     * @class Windows.UI.Xaml.Media.Imaging.BitmapSource
     * @augments Windows.UI.Xaml.Media.ImageSource
     */
    Bridge.define("Windows.UI.Xaml.Media.Imaging.BitmapSource", {
        inherits: [Windows.UI.Xaml.Media.ImageSource],
        fields: {
            _isStreamAsBase64StringValid: false,
            _streamAsBase64String: null,
            _streamSource: null,
            _dataUrl: null
        },
        props: {
            INTERNAL_StreamAsBase64String: {
                get: function () {
                    if (!this._isStreamAsBase64StringValid) {
                        var bytes = System.Array.init(this.INTERNAL_StreamSource.Length.add(System.Int64(10)), 0, System.Byte);
                        if (this.INTERNAL_StreamSource.Length.gt(System.Int64(2147483647))) {
                            throw new System.InvalidOperationException.$ctor1("The Stream set as the BitmapSource's Source is too big (more than int.MaxValue (2,147,483,647) bytes).");
                        }

                        var n = this.INTERNAL_StreamSource.Read(bytes, 0, System.Int64.clip32(this.INTERNAL_StreamSource.Length));

                        this.INTERNAL_StreamSource.Close();
                        this._streamAsBase64String = System.Convert.toBase64String(bytes, null, null, null);
                        this._isStreamAsBase64StringValid = true;
                    }
                    return this._streamAsBase64String;
                }
            },
            INTERNAL_StreamSource: {
                get: function () {
                    return this._streamSource;
                },
                set: function (value) {
                    this._streamSource = value;
                }
            },
            INTERNAL_DataURL: {
                get: function () {
                    return this._dataUrl;
                },
                set: function (value) {
                    this._dataUrl = value;
                }
            }
        },
        ctors: {
            init: function () {
                this._isStreamAsBase64StringValid = false;
            },
            /**
             * Provides base class initialization behavior for BitmapSource-derived classes.
             *
             * @instance
             * @protected
             * @this Windows.UI.Xaml.Media.Imaging.BitmapSource
             * @memberof Windows.UI.Xaml.Media.Imaging.BitmapSource
             * @return  {void}
             */
            ctor: function () {
                this.$initialize();
                Windows.UI.Xaml.Media.ImageSource.ctor.call(this);
            }
        },
        methods: {
            /**
             * Sets the source image for a BitmapSource by accessing a stream.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Media.Imaging.BitmapSource
             * @memberof Windows.UI.Xaml.Media.Imaging.BitmapSource
             * @param   {System.IO.Stream}    streamSource    The stream source that sets the image source value.
             * @return  {void}
             */
            SetSource: function (streamSource) {
                this.INTERNAL_StreamSource = streamSource;
                this._isStreamAsBase64StringValid = false;
            },
            /**
             * Sets the source image for a BitmapSource by passing a "data URL".
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Media.Imaging.BitmapSource
             * @memberof Windows.UI.Xaml.Media.Imaging.BitmapSource
             * @param   {string}    dataUrl    The image encoded in "data URL" format.
             * @return  {void}
             */
            SetSource$1: function (dataUrl) {
                this.INTERNAL_DataURL = dataUrl;
            }
        }
    });

    /**
     * Represents the geometry of a line.
     *
     * @public
     * @class Windows.UI.Xaml.Media.LineGeometry
     * @augments Windows.UI.Xaml.Media.Geometry
     */
    Bridge.define("Windows.UI.Xaml.Media.LineGeometry", {
        inherits: [Windows.UI.Xaml.Media.Geometry],
        statics: {
            fields: {
                /**
                 * Identifies the EndPoint dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Media.LineGeometry
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                EndPointProperty: null,
                /**
                 * Identifies the StartPoint dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Media.LineGeometry
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                StartPointProperty: null
            },
            ctors: {
                init: function () {
                    this.EndPointProperty = Windows.UI.Xaml.DependencyProperty.Register("EndPoint", Windows.Foundation.Point, Windows.UI.Xaml.Media.LineGeometry, new Windows.UI.Xaml.PropertyMetadata.$ctor2(new Windows.Foundation.Point.ctor().$clone(), Windows.UI.Xaml.Media.LineGeometry.EndPoint_Changed));
                    this.StartPointProperty = Windows.UI.Xaml.DependencyProperty.Register("StartPoint", Windows.Foundation.Point, Windows.UI.Xaml.Media.LineGeometry, new Windows.UI.Xaml.PropertyMetadata.$ctor2(new Windows.Foundation.Point.ctor().$clone(), Windows.UI.Xaml.Media.LineGeometry.StartPoint_Changed));
                }
            },
            methods: {
                EndPoint_Changed: function (d, e) {
                    var geometry = Bridge.cast(d, Windows.UI.Xaml.Media.LineGeometry);
                    if (geometry.ParentPath != null) {
                        geometry.ParentPath.ScheduleRedraw();
                    }
                },
                StartPoint_Changed: function (d, e) {
                    var geometry = Bridge.cast(d, Windows.UI.Xaml.Media.LineGeometry);
                    if (geometry.ParentPath != null) {
                        geometry.ParentPath.ScheduleRedraw();
                    }
                }
            }
        },
        props: {
            /**
             * Gets or sets the end point of a line.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Media.LineGeometry
             * @function EndPoint
             * @type Windows.Foundation.Point
             */
            EndPoint: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.Media.LineGeometry.EndPointProperty), Windows.Foundation.Point), Windows.Foundation.Point));
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Media.LineGeometry.EndPointProperty, value.$clone());
                }
            },
            /**
             * Gets or sets the start point of the line.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Media.LineGeometry
             * @function StartPoint
             * @type Windows.Foundation.Point
             */
            StartPoint: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.Media.LineGeometry.StartPointProperty), Windows.Foundation.Point), Windows.Foundation.Point));
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Media.LineGeometry.StartPointProperty, value.$clone());
                }
            }
        },
        ctors: {
            ctor: function () {
                this.$initialize();
                Windows.UI.Xaml.Media.Geometry.ctor.call(this);

            },
            /**
             * @instance
             * @public
             * @this Windows.UI.Xaml.Media.LineGeometry
             * @memberof Windows.UI.Xaml.Media.LineGeometry
             * @param   {Windows.Foundation.Point}    startPoint    
             * @param   {Windows.Foundation.Point}    endPoint
             * @return  {void}
             */
            $ctor1: function (startPoint, endPoint) {
                this.$initialize();
                Windows.UI.Xaml.Media.Geometry.ctor.call(this);
                this.StartPoint = startPoint.$clone();
                this.EndPoint = endPoint.$clone();
            }
        },
        methods: {
            GetMinMaxXY: function (minX, maxX, minY, maxY) {
                var maxAbs = this.StartPoint.X > this.EndPoint.X ? this.StartPoint.X : this.EndPoint.X;
                var minAbs = this.StartPoint.X < this.EndPoint.X ? this.StartPoint.X : this.EndPoint.X;
                var minOrd = this.StartPoint.Y < this.EndPoint.Y ? this.StartPoint.Y : this.EndPoint.Y;
                var maxOrd = this.StartPoint.Y > this.EndPoint.Y ? this.StartPoint.Y : this.EndPoint.Y;
                if (maxX.v < maxAbs) {
                    maxX.v = maxAbs;
                }
                if (maxY.v < maxOrd) {
                    maxY.v = maxOrd;
                }
                if (minX.v > minAbs) {
                    minX.v = minAbs;
                }
                if (minY.v > minOrd) {
                    minY.v = minOrd;
                }
            },
            DefineInCanvas: function (path, canvasDomElement, horizontalMultiplicator, verticalMultiplicator, xOffsetToApplyBeforeMultiplication, yOffsetToApplyBeforeMultiplication, xOffsetToApplyAfterMultiplication, yOffsetToApplyAfterMultiplication, shapeActualSize) {
                var strokeAsString = "";

                CSHTML5.Internal.INTERNAL_ShapesDrawHelpers.PrepareLine(path._canvasDomElement, this.StartPoint.$clone(), this.EndPoint.$clone());

                var context = CSHTML5.Internal.INTERNAL_HtmlDomManager.Get2dCanvasContext(canvasDomElement);
                context.strokeStyle = strokeAsString;
                context.lineWidth = System.Double.format(path.StrokeThickness) + "px";
            }
        }
    });

    /**
     * Represents a line drawn between two points, which can be part of a PathFigure
     within Path data.
     *
     * @public
     * @class Windows.UI.Xaml.Media.LineSegment
     * @augments Windows.UI.Xaml.Media.PathSegment
     */
    Bridge.define("Windows.UI.Xaml.Media.LineSegment", {
        inherits: [Windows.UI.Xaml.Media.PathSegment],
        statics: {
            fields: {
                /**
                 * Identifies the Point dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Media.LineSegment
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                PointProperty: null
            },
            ctors: {
                init: function () {
                    this.PointProperty = Windows.UI.Xaml.DependencyProperty.Register("Point", Windows.Foundation.Point, Windows.UI.Xaml.Media.LineSegment, new Windows.UI.Xaml.PropertyMetadata.$ctor2(new Windows.Foundation.Point.ctor().$clone(), Windows.UI.Xaml.Media.LineSegment.Point_Changed));
                }
            },
            methods: {
                Point_Changed: function (d, e) {
                    var segment = Bridge.cast(d, Windows.UI.Xaml.Media.LineSegment);
                    if (segment.ParentPath != null) {
                        segment.ParentPath.ScheduleRedraw();
                    }
                }
            }
        },
        props: {
            /**
             * Gets or sets the end point of the line segment.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Media.LineSegment
             * @function Point
             * @type Windows.Foundation.Point
             */
            Point: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.Media.LineSegment.PointProperty), Windows.Foundation.Point), Windows.Foundation.Point));
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Media.LineSegment.PointProperty, value.$clone());
                }
            }
        },
        ctors: {
            /**
             * Initializes a new instance of the LineSegment class.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Media.LineSegment
             * @memberof Windows.UI.Xaml.Media.LineSegment
             * @return  {void}
             */
            ctor: function () {
                this.$initialize();
                Windows.UI.Xaml.Media.PathSegment.ctor.call(this);

            }
        },
        methods: {
            DefineInCanvas: function (xOffsetToApplyBeforeMultiplication, yOffsetToApplyBeforeMultiplication, xOffsetToApplyAfterMultiplication, yOffsetToApplyAfterMultiplication, horizontalMultiplicator, verticalMultiplicator, canvasDomElement, previousLastPoint) {
                var context = CSHTML5.Internal.INTERNAL_HtmlDomManager.Get2dCanvasContext(canvasDomElement);

                context.lineTo(this.Point.X * horizontalMultiplicator + xOffsetToApplyBeforeMultiplication * horizontalMultiplicator + xOffsetToApplyAfterMultiplication, this.Point.Y * verticalMultiplicator + yOffsetToApplyBeforeMultiplication * verticalMultiplicator + yOffsetToApplyAfterMultiplication);

                return this.Point.$clone();
            },
            GetMaxXY: function () {
                return new Windows.Foundation.Point.$ctor1(this.Point.X, this.Point.X);
            },
            GetMinMaxXY: function (minX, maxX, minY, maxY, startingPoint) {
                minX.v = Math.min(minX.v, this.Point.X);
                maxX.v = Math.max(maxX.v, this.Point.X);
                minY.v = Math.min(minY.v, this.Point.Y);
                maxY.v = Math.max(maxY.v, this.Point.Y);
                return this.Point.$clone();
            }
        }
    });

    /**
     * Represents a collection of PathFigure objects that collectively make up the
     geometry of a PathGeometry.
     *
     * @public
     * @class Windows.UI.Xaml.Media.PathFigureCollection
     * @augments Windows.UI.Xaml.PresentationFrameworkCollection$1
     */
    Bridge.define("Windows.UI.Xaml.Media.PathFigureCollection", {
        inherits: [Windows.UI.Xaml.PresentationFrameworkCollection$1(Windows.UI.Xaml.Media.PathFigure)],
        fields: {
            _parentPath: null
        },
        ctors: {
            /**
             * Initializes a new instance that is empty.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Media.PathFigureCollection
             * @memberof Windows.UI.Xaml.Media.PathFigureCollection
             * @return  {void}
             */
            ctor: function () {
                this.$initialize();
                Windows.UI.Xaml.PresentationFrameworkCollection$1(Windows.UI.Xaml.Media.PathFigure).ctor.call(this);

            },
            /**
             * Initializes a new instance that is empty and has the specified initial capacity.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Media.PathFigureCollection
             * @memberof Windows.UI.Xaml.Media.PathFigureCollection
             * @param   {number}    capacity    int - The number of elements that the new list is initially capable of storing.
             * @return  {void}
             */
            $ctor2: function (capacity) {
                this.$initialize();
                Windows.UI.Xaml.PresentationFrameworkCollection$1(Windows.UI.Xaml.Media.PathFigure).$ctor2.call(this, capacity);

            },
            /**
             * Creates a PathFigureCollection with all of the same elements as collection
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Media.PathFigureCollection
             * @memberof Windows.UI.Xaml.Media.PathFigureCollection
             * @param   {System.Collections.Generic.IEnumerable$1}    figures
             * @return  {void}
             */
            $ctor1: function (figures) {
                this.$initialize();
                Windows.UI.Xaml.PresentationFrameworkCollection$1(Windows.UI.Xaml.Media.PathFigure).$ctor1.call(this, figures);

            }
        },
        methods: {
            AddOverride: function (figure) {
                if (figure == null) {
                    throw new System.ArgumentNullException.$ctor1("figure");
                }
                figure.SetParentPath(this._parentPath);
                this.AddDependencyObjectInternal(figure);
                this.NotifyCollectionChanged();
            },
            RemoveOverride: function (figure) {
                if (this.RemoveDependencyObjectInternal(figure)) {
                    figure.SetParentPath(null);
                    this.NotifyCollectionChanged();
                    return true;
                }
                return false;
            },
            RemoveAtOverride: function (index) {
                if (index < 0 || index >= this.CountInternal) {
                    throw new System.ArgumentOutOfRangeException.$ctor1("index");
                }
                this.GetItemInternal(index).SetParentPath(null);
                this.RemoveAtDependencyObjectInternal(index);
                this.NotifyCollectionChanged();
            },
            InsertOverride: function (index, figure) {
                if (figure == null) {
                    throw new System.ArgumentNullException.$ctor1("figure");
                }
                figure.SetParentPath(this._parentPath);
                this.InsertDependencyObjectInternal(index, figure);
                this.NotifyCollectionChanged();
            },
            ClearOverride: function () {
                var $t;
                $t = Bridge.getEnumerator(this);
                try {
                    while ($t.moveNext()) {
                        var figure = $t.Current;
                        figure.SetParentPath(null);
                    }
                } finally {
                    if (Bridge.is($t, System.IDisposable)) {
                        $t.System$IDisposable$Dispose();
                    }
                }
                this.ClearDependencyObjectInternal();
                this.NotifyCollectionChanged();
            },
            GetItemOverride: function (index) {
                return this.GetItemInternal(index);
            },
            SetItemOverride: function (index, figure) {
                var oldItem = this.GetItemInternal(index);
                oldItem.SetParentPath(null);
                figure.SetParentPath(this._parentPath);
                this.SetItemDependencyObjectInternal(index, figure);
                this.NotifyCollectionChanged();
            },
            SetParentPath: function (path) {
                var $t;
                if (!Bridge.referenceEquals(this._parentPath, path)) {
                    this._parentPath = path;
                    $t = Bridge.getEnumerator(this);
                    try {
                        while ($t.moveNext()) {
                            var figure = $t.Current;
                            figure.SetParentPath(path);
                        }
                    } finally {
                        if (Bridge.is($t, System.IDisposable)) {
                            $t.System$IDisposable$Dispose();
                        }
                    }
                }
            },
            NotifyCollectionChanged: function () {
                if (this._parentPath != null) {
                    this._parentPath.ScheduleRedraw();
                }
            }
        }
    });

    /**
     * Represents a complex shape that may be composed of arcs, curves, ellipses,
     lines, and rectangles.
     *
     * @public
     * @class Windows.UI.Xaml.Media.PathGeometry
     * @augments Windows.UI.Xaml.Media.Geometry
     */
    Bridge.define("Windows.UI.Xaml.Media.PathGeometry", {
        inherits: [Windows.UI.Xaml.Media.Geometry],
        statics: {
            fields: {
                /**
                 * Identifies the Figures dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Media.PathGeometry
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                FiguresProperty: null,
                /**
                 * Identifies the FillRule dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Media.PathGeometry
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                FillRuleProperty: null
            },
            ctors: {
                init: function () {
                    this.FiguresProperty = Windows.UI.Xaml.DependencyProperty.Register("Figures", Windows.UI.Xaml.Media.PathFigureCollection, Windows.UI.Xaml.Media.PathGeometry, new Windows.UI.Xaml.PropertyMetadata.$ctor2(null, Windows.UI.Xaml.Media.PathGeometry.Figures_Changed));
                    this.FillRuleProperty = Windows.UI.Xaml.DependencyProperty.Register("FillRule", Windows.UI.Xaml.Media.FillRule, Windows.UI.Xaml.Media.PathGeometry, new Windows.UI.Xaml.PropertyMetadata.$ctor2(Bridge.box(Windows.UI.Xaml.Media.FillRule.EvenOdd, Windows.UI.Xaml.Media.FillRule, System.Enum.toStringFn(Windows.UI.Xaml.Media.FillRule)), Windows.UI.Xaml.Media.PathGeometry.FillRule_Changed));
                }
            },
            methods: {
                Figures_Changed: function (d, e) {
                    var geometry = Bridge.cast(d, Windows.UI.Xaml.Media.PathGeometry);
                    if (null != e.OldValue) {
                        Bridge.cast(e.OldValue, Windows.UI.Xaml.Media.PathFigureCollection).SetParentPath(null);
                    }
                    if (geometry.ParentPath != null) {
                        if (null != e.NewValue) {
                            Bridge.cast(e.NewValue, Windows.UI.Xaml.Media.PathFigureCollection).SetParentPath(geometry.ParentPath);
                        }

                        geometry.ParentPath.ScheduleRedraw();
                    }
                },
                FillRule_Changed: function (d, e) {
                    var geometry = Bridge.cast(d, Windows.UI.Xaml.Media.PathGeometry);
                    if (geometry.ParentPath != null && geometry.ParentPath._isLoaded) {
                        geometry.ParentPath.ScheduleRedraw();
                    }
                },
                INTERNAL_ConvertFromString: function (pathAsString) {
                    return Windows.UI.Xaml.Media.GeometryParser.ParseGeometry(pathAsString);
                }
            }
        },
        props: {
            /**
             * Gets or sets the collection of PathFigure objects that describe the contents
             of a path.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Media.PathGeometry
             * @function Figures
             * @type Windows.UI.Xaml.Media.PathFigureCollection
             */
            Figures: {
                get: function () {
                    var collection = Bridge.cast(this.GetValue(Windows.UI.Xaml.Media.PathGeometry.FiguresProperty), Windows.UI.Xaml.Media.PathFigureCollection);
                    if (collection == null) {
                        collection = new Windows.UI.Xaml.Media.PathFigureCollection.ctor();
                        this.SetValue(Windows.UI.Xaml.Media.PathGeometry.FiguresProperty, collection);
                    }
                    return collection;
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Media.PathGeometry.FiguresProperty, value);
                }
            },
            /**
             * Gets or sets a value that determines how the intersecting areas contained
             in the PathGeometry are combined.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Media.PathGeometry
             * @function FillRule
             * @type Windows.UI.Xaml.Media.FillRule
             */
            FillRule: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.Media.PathGeometry.FillRuleProperty), Windows.UI.Xaml.Media.FillRule), System.Int32));
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Media.PathGeometry.FillRuleProperty, Bridge.box(value, Windows.UI.Xaml.Media.FillRule, System.Enum.toStringFn(Windows.UI.Xaml.Media.FillRule)));
                }
            }
        },
        ctors: {
            /**
             * Initializes a new instance of the System.Windows.Media.PathGeometry class.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Media.PathGeometry
             * @memberof Windows.UI.Xaml.Media.PathGeometry
             * @return  {void}
             */
            ctor: function () {
                this.$initialize();
                Windows.UI.Xaml.Media.Geometry.ctor.call(this);

            },
            /**
             * Initializes a new instance of the System.Windows.Media.PathGeometry class.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Media.PathGeometry
             * @memberof Windows.UI.Xaml.Media.PathGeometry
             * @param   {System.Collections.Generic.IEnumerable$1}    figures    A collection of figures
             * @return  {void}
             */
            $ctor1: function (figures) {
                var $t;
                this.$initialize();
                Windows.UI.Xaml.Media.Geometry.ctor.call(this);
                if (figures != null) {
                    $t = Bridge.getEnumerator(figures, Windows.UI.Xaml.Media.PathFigure);
                    try {
                        while ($t.moveNext()) {
                            var item = $t.Current;
                            this.Figures.add(item);
                        }
                    } finally {
                        if (Bridge.is($t, System.IDisposable)) {
                            $t.System$IDisposable$Dispose();
                        }
                    }
                } else {
                    throw new System.ArgumentNullException.$ctor1("figures");
                }
            }
        },
        methods: {
            SetParentPath: function (path) {
                Windows.UI.Xaml.Media.Geometry.prototype.SetParentPath.call(this, path);
                this.Figures.SetParentPath(path);
            },
            GetFillRuleAsString: function () {
                return (this.FillRule === Windows.UI.Xaml.Media.FillRule.Nonzero) ? "nonzero" : "evenodd";
            },
            GetMinMaxXY: function (minX, maxX, minY, maxY) {
                var $t;
                $t = Bridge.getEnumerator(this.Figures);
                try {
                    while ($t.moveNext()) {
                        var figure = $t.Current;
                        figure.GetMinMaxXY(minX, maxX, minY, maxY);
                    }
                } finally {
                    if (Bridge.is($t, System.IDisposable)) {
                        $t.System$IDisposable$Dispose();
                    }
                }
            },
            /**
             * Applies FillStyle, StrokeStyle + Adds the figures to the canvas' context, then calls the Fill method.
             *
             * @instance
             * @override
             * @this Windows.UI.Xaml.Media.PathGeometry
             * @memberof Windows.UI.Xaml.Media.PathGeometry
             * @param   {Windows.UI.Xaml.Shapes.Path}    path                                  
             * @param   {System.Object}                  canvasDomElement                      
             * @param   {number}                         horizontalMultiplicator               
             * @param   {number}                         verticalMultiplicator                 
             * @param   {number}                         xOffsetToApplyBeforeMultiplication    
             * @param   {number}                         yOffsetToApplyBeforeMultiplication    
             * @param   {number}                         xOffsetToApplyAfterMultiplication     
             * @param   {number}                         yOffsetToApplyAfterMultiplication     
             * @param   {Windows.Foundation.Size}        shapeActualSize
             * @return  {void}
             */
            DefineInCanvas: function (path, canvasDomElement, horizontalMultiplicator, verticalMultiplicator, xOffsetToApplyBeforeMultiplication, yOffsetToApplyBeforeMultiplication, xOffsetToApplyAfterMultiplication, yOffsetToApplyAfterMultiplication, shapeActualSize) {
                var $t;

                if (shapeActualSize.Width > 0 && shapeActualSize.Height > 0) {

                    $t = Bridge.getEnumerator(this.Figures);
                    try {
                        while ($t.moveNext()) {
                            var figure = $t.Current;
                            figure.DefineInCanvas(xOffsetToApplyBeforeMultiplication, yOffsetToApplyBeforeMultiplication, xOffsetToApplyAfterMultiplication, yOffsetToApplyAfterMultiplication, horizontalMultiplicator, verticalMultiplicator, canvasDomElement, this.ParentPath.StrokeThickness, shapeActualSize.$clone());
                        }
                    } finally {
                        if (Bridge.is($t, System.IDisposable)) {
                            $t.System$IDisposable$Dispose();
                        }
                    }
                }
            }
        }
    });

    /**
     * Represents a collection of PathSegment objects that can be individually accessed
     by index.
     *
     * @public
     * @class Windows.UI.Xaml.Media.PathSegmentCollection
     * @augments Windows.UI.Xaml.PresentationFrameworkCollection$1
     */
    Bridge.define("Windows.UI.Xaml.Media.PathSegmentCollection", {
        inherits: [Windows.UI.Xaml.PresentationFrameworkCollection$1(Windows.UI.Xaml.Media.PathSegment)],
        fields: {
            _parentPath: null
        },
        ctors: {
            /**
             * Initializes a new instance that is empty.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Media.PathSegmentCollection
             * @memberof Windows.UI.Xaml.Media.PathSegmentCollection
             * @return  {void}
             */
            ctor: function () {
                this.$initialize();
                Windows.UI.Xaml.PresentationFrameworkCollection$1(Windows.UI.Xaml.Media.PathSegment).ctor.call(this);

            },
            /**
             * Initializes a new instance that is empty and has the specified initial capacity.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Media.PathSegmentCollection
             * @memberof Windows.UI.Xaml.Media.PathSegmentCollection
             * @param   {number}    capacity    int - The number of elements that the new list is initially capable of storing.
             * @return  {void}
             */
            $ctor2: function (capacity) {
                this.$initialize();
                Windows.UI.Xaml.PresentationFrameworkCollection$1(Windows.UI.Xaml.Media.PathSegment).$ctor2.call(this, capacity);

            },
            /**
             * Creates a PathSegmentCollection with all of the same elements as collection
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Media.PathSegmentCollection
             * @memberof Windows.UI.Xaml.Media.PathSegmentCollection
             * @param   {System.Collections.Generic.IEnumerable$1}    segments
             * @return  {void}
             */
            $ctor1: function (segments) {
                this.$initialize();
                Windows.UI.Xaml.PresentationFrameworkCollection$1(Windows.UI.Xaml.Media.PathSegment).$ctor1.call(this, segments);

            }
        },
        methods: {
            AddOverride: function (segment) {
                if (segment == null) {
                    throw new System.ArgumentNullException.$ctor1("value");
                }
                if (this._parentPath != null) {
                    segment.SetParentPath(this._parentPath);
                }
                this.AddDependencyObjectInternal(segment);
                this.NotifyCollectionChanged();
            },
            RemoveOverride: function (segment) {
                if (this.RemoveDependencyObjectInternal(segment)) {
                    segment.SetParentPath(null);
                    this.NotifyCollectionChanged();
                    return true;
                }
                return false;
            },
            RemoveAtOverride: function (index) {
                if (index < 0 || index >= this.Count) {
                    throw new System.ArgumentOutOfRangeException.$ctor1("index");
                }
                this.GetItemInternal(index).SetParentPath(null);
                this.RemoveAtDependencyObjectInternal(index);
                this.NotifyCollectionChanged();
            },
            InsertOverride: function (index, segment) {
                if (segment == null) {
                    throw new System.ArgumentNullException.$ctor1("value");
                }
                if (this._parentPath != null) {
                    segment.SetParentPath(this._parentPath);
                }
                this.InsertDependencyObjectInternal(index, segment);
                this.NotifyCollectionChanged();
            },
            ClearOverride: function () {
                var $t;
                $t = Bridge.getEnumerator(this);
                try {
                    while ($t.moveNext()) {
                        var segment = $t.Current;
                        segment.SetParentPath(null);
                    }
                } finally {
                    if (Bridge.is($t, System.IDisposable)) {
                        $t.System$IDisposable$Dispose();
                    }
                }
                this.ClearDependencyObjectInternal();
                this.NotifyCollectionChanged();
            },
            GetItemOverride: function (index) {
                return this.GetItemInternal(index);
            },
            SetItemOverride: function (index, segment) {
                if (this._parentPath != null) {
                    var oldItem = this.getItem(index);
                    oldItem.SetParentPath(null);
                    segment.SetParentPath(this._parentPath);
                }
                this.SetItemDependencyObjectInternal(index, segment);
                this.NotifyCollectionChanged();
            },
            SetParentPath: function (path) {
                var $t;
                if (!Bridge.referenceEquals(this._parentPath, path)) {
                    this._parentPath = path;
                    $t = Bridge.getEnumerator(this);
                    try {
                        while ($t.moveNext()) {
                            var segment = $t.Current;
                            segment.SetParentPath(path);
                        }
                    } finally {
                        if (Bridge.is($t, System.IDisposable)) {
                            $t.System$IDisposable$Dispose();
                        }
                    }
                }
            },
            NotifyCollectionChanged: function () {
                if (this._parentPath != null) {
                    this._parentPath.ScheduleRedraw();
                }
            }
        }
    });

    Bridge.define("Windows.UI.Xaml.Media.PointCollection", {
        inherits: [Windows.UI.Xaml.PresentationFrameworkCollection$1(Windows.Foundation.Point)],
        fields: {
            _parentPath: null
        },
        ctors: {
            /**
             * Initializes a new instance that is empty.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Media.PointCollection
             * @memberof Windows.UI.Xaml.Media.PointCollection
             * @return  {void}
             */
            ctor: function () {
                this.$initialize();
                Windows.UI.Xaml.PresentationFrameworkCollection$1(Windows.Foundation.Point).ctor.call(this);

            },
            /**
             * Initializes a new instance that is empty and has the specified initial capacity.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Media.PointCollection
             * @memberof Windows.UI.Xaml.Media.PointCollection
             * @param   {number}    capacity    int - The number of elements that the new list is initially capable of storing.
             * @return  {void}
             */
            $ctor2: function (capacity) {
                this.$initialize();
                Windows.UI.Xaml.PresentationFrameworkCollection$1(Windows.Foundation.Point).$ctor2.call(this, capacity);

            },
            /**
             * Creates a PointCollection with all of the same elements as collection
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Media.PointCollection
             * @memberof Windows.UI.Xaml.Media.PointCollection
             * @param   {System.Collections.Generic.IEnumerable$1}    points
             * @return  {void}
             */
            $ctor1: function (points) {
                this.$initialize();
                Windows.UI.Xaml.PresentationFrameworkCollection$1(Windows.Foundation.Point).$ctor1.call(this, points);

            }
        },
        methods: {
            AddOverride: function (point) {
                this.AddInternal(point.$clone());
                this.NotifyCollectionChanged();
            },
            ClearOverride: function () {
                this.ClearInternal();
                this.NotifyCollectionChanged();
            },
            RemoveAtOverride: function (index) {
                this.RemoveAtInternal(index);
                this.NotifyCollectionChanged();
            },
            RemoveOverride: function (point) {
                if (this.RemoveInternal(point.$clone())) {
                    this.NotifyCollectionChanged();
                    return true;
                }
                return false;
            },
            InsertOverride: function (index, point) {
                this.InsertInternal(index, point.$clone());
                this.NotifyCollectionChanged();
            },
            GetItemOverride: function (index) {
                return this.GetItemInternal(index);
            },
            SetItemOverride: function (index, point) {
                this.SetItemInternal(index, point.$clone());
                this.NotifyCollectionChanged();
            },
            SetParentPath: function (path) {
                this._parentPath = path;
            },
            NotifyCollectionChanged: function () {
                if (this._parentPath != null) {
                    this._parentPath.ScheduleRedraw();
                }
            }
        }
    });

    /**
     * Represents one or more cubic Bezier curves.
     *
     * @public
     * @class Windows.UI.Xaml.Media.PolyBezierSegment
     * @augments Windows.UI.Xaml.Media.PathSegment
     */
    Bridge.define("Windows.UI.Xaml.Media.PolyBezierSegment", {
        inherits: [Windows.UI.Xaml.Media.PathSegment],
        statics: {
            fields: {
                /**
                 * Identifies the Points dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Media.PolyBezierSegment
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                PointsProperty: null
            },
            ctors: {
                init: function () {
                    this.PointsProperty = Windows.UI.Xaml.DependencyProperty.Register("Points", Windows.UI.Xaml.Media.PointCollection, Windows.UI.Xaml.Media.PolyBezierSegment, new Windows.UI.Xaml.PropertyMetadata.$ctor2(null, Windows.UI.Xaml.Media.PolyBezierSegment.Points_Changed));
                }
            },
            methods: {
                Points_Changed: function (d, e) {
                    var segment = Bridge.cast(d, Windows.UI.Xaml.Media.PolyBezierSegment);
                    if (segment.ParentPath != null) {
                        segment.ParentPath.ScheduleRedraw();
                    }
                }
            }
        },
        props: {
            /**
             * Gets or sets the Point collection that defines this PolyBezierSegment object.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Media.PolyBezierSegment
             * @function Points
             * @type Windows.UI.Xaml.Media.PointCollection
             */
            Points: {
                get: function () {
                    var points = Bridge.cast(this.GetValue(Windows.UI.Xaml.Media.PolyBezierSegment.PointsProperty), Windows.UI.Xaml.Media.PointCollection);
                    if (points == null) {
                        points = new Windows.UI.Xaml.Media.PointCollection.ctor();
                        this.SetValue(Windows.UI.Xaml.Media.PolyBezierSegment.PointsProperty, points);
                    }
                    return points;
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Media.PolyBezierSegment.PointsProperty, value);
                }
            }
        },
        ctors: {
            /**
             * Initializes a new instance of the PolyBezierSegment class.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Media.PolyBezierSegment
             * @memberof Windows.UI.Xaml.Media.PolyBezierSegment
             * @return  {void}
             */
            ctor: function () {
                this.$initialize();
                Windows.UI.Xaml.Media.PathSegment.ctor.call(this);

            }
        },
        methods: {
            SetParentPath: function (path) {
                Windows.UI.Xaml.Media.PathSegment.prototype.SetParentPath.call(this, path);
                this.Points.SetParentPath(path);
            },
            DefineInCanvas: function (xOffsetToApplyBeforeMultiplication, yOffsetToApplyBeforeMultiplication, xOffsetToApplyAfterMultiplication, yOffsetToApplyAfterMultiplication, horizontalMultiplicator, verticalMultiplicator, canvasDomElement, previousLastPoint) {
                var context = CSHTML5.Internal.INTERNAL_HtmlDomManager.Get2dCanvasContext(canvasDomElement);
                var i = 0;
                var lastPoint = previousLastPoint.$clone();
                while (i < ((this.Points.Count - 2) | 0)) {
                    var controlPoint1X = this.Points.getItem(i).$clone().X;
                    var controlPoint1Y = this.Points.getItem(i).$clone().Y;
                    i = (i + 1) | 0;
                    var controlPoint2X = this.Points.getItem(i).$clone().X;
                    var controlPoint2Y = this.Points.getItem(i).$clone().Y;
                    i = (i + 1) | 0;
                    lastPoint = this.Points.getItem(i).$clone();
                    var endPointX = lastPoint.X;
                    var endPointY = lastPoint.Y;
                    i = (i + 1) | 0;

                    context.bezierCurveTo((controlPoint1X + xOffsetToApplyBeforeMultiplication) * horizontalMultiplicator + xOffsetToApplyAfterMultiplication, (controlPoint1Y + yOffsetToApplyBeforeMultiplication) * verticalMultiplicator + yOffsetToApplyAfterMultiplication, (controlPoint2X + xOffsetToApplyBeforeMultiplication) * horizontalMultiplicator + xOffsetToApplyAfterMultiplication, (controlPoint2Y + yOffsetToApplyBeforeMultiplication) * verticalMultiplicator + yOffsetToApplyAfterMultiplication, (endPointX + xOffsetToApplyBeforeMultiplication) * horizontalMultiplicator + xOffsetToApplyAfterMultiplication, (endPointY + yOffsetToApplyBeforeMultiplication) * verticalMultiplicator + yOffsetToApplyAfterMultiplication);
                }
                return lastPoint.$clone();
            },
            GetMaxXY: function () {
                var $t;
                var currentMax = new Windows.Foundation.Point.$ctor1(Number.NEGATIVE_INFINITY, Number.NEGATIVE_INFINITY);
                $t = Bridge.getEnumerator(this.Points);
                try {
                    while ($t.moveNext()) {
                        var point = $t.Current.$clone();
                        if (point.X > currentMax.X) {
                            currentMax.X = point.X;
                        }
                        if (point.Y > currentMax.Y) {
                            currentMax.Y = point.Y;
                        }
                    }
                } finally {
                    if (Bridge.is($t, System.IDisposable)) {
                        $t.System$IDisposable$Dispose();
                    }
                }
                return currentMax.$clone();
            },
            GetMinMaxXY: function (minX, maxX, minY, maxY, startingPoint) {
                var $t;
                $t = Bridge.getEnumerator(this.Points);
                try {
                    while ($t.moveNext()) {
                        var point = $t.Current.$clone();
                        minX.v = Math.min(minX.v, point.X);
                        maxX.v = Math.max(maxX.v, point.X);
                        minY.v = Math.min(minY.v, point.Y);
                        maxY.v = Math.max(maxY.v, point.Y);
                    }
                } finally {
                    if (Bridge.is($t, System.IDisposable)) {
                        $t.System$IDisposable$Dispose();
                    }
                }
                return this.Points.Count === 0 ? startingPoint : this.Points.getItem(((this.Points.Count - 1) | 0)).$clone();
            }
        }
    });

    /**
     * Represents a set of line segments defined by a Point collection with each
     Point specifying the end point of a line segment.
     *
     * @public
     * @class Windows.UI.Xaml.Media.PolyLineSegment
     * @augments Windows.UI.Xaml.Media.PathSegment
     */
    Bridge.define("Windows.UI.Xaml.Media.PolyLineSegment", {
        inherits: [Windows.UI.Xaml.Media.PathSegment],
        statics: {
            fields: {
                /**
                 * Identifies the Points dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Media.PolyLineSegment
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                PointsProperty: null
            },
            ctors: {
                init: function () {
                    this.PointsProperty = Windows.UI.Xaml.DependencyProperty.Register("Points", Windows.UI.Xaml.Media.PointCollection, Windows.UI.Xaml.Media.PolyLineSegment, new Windows.UI.Xaml.PropertyMetadata.$ctor2(null, Windows.UI.Xaml.Media.PolyLineSegment.Points_Changed));
                }
            },
            methods: {
                Points_Changed: function (d, e) {
                    var segment = Bridge.cast(d, Windows.UI.Xaml.Media.PolyLineSegment);
                    if (segment.ParentPath != null) {
                        segment.ParentPath.ScheduleRedraw();
                    }
                }
            }
        },
        props: {
            /**
             * Gets or sets the collection of Point values that defines this PolyLineSegment
             object.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Media.PolyLineSegment
             * @function Points
             * @type Windows.UI.Xaml.Media.PointCollection
             */
            Points: {
                get: function () {
                    return Bridge.cast(this.GetValue(Windows.UI.Xaml.Media.PolyLineSegment.PointsProperty), Windows.UI.Xaml.Media.PointCollection);
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Media.PolyLineSegment.PointsProperty, value);
                }
            }
        },
        ctors: {
            /**
             * Initializes a new instance of the PolyLineSegment class.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Media.PolyLineSegment
             * @memberof Windows.UI.Xaml.Media.PolyLineSegment
             * @return  {void}
             */
            ctor: function () {
                this.$initialize();
                Windows.UI.Xaml.Media.PathSegment.ctor.call(this);

            }
        },
        methods: {
            SetParentPath: function (path) {
                Windows.UI.Xaml.Media.PathSegment.prototype.SetParentPath.call(this, path);
                this.Points.SetParentPath(path);
            },
            DefineInCanvas: function (xOffsetToApplyBeforeMultiplication, yOffsetToApplyBeforeMultiplication, xOffsetToApplyAfterMultiplication, yOffsetToApplyAfterMultiplication, horizontalMultiplicator, verticalMultiplicator, canvasDomElement, previousLastPoint) {
                var $t;
                var context = CSHTML5.Internal.INTERNAL_HtmlDomManager.Get2dCanvasContext(canvasDomElement);
                $t = Bridge.getEnumerator(this.Points);
                try {
                    while ($t.moveNext()) {
                        var point = $t.Current.$clone();
                        context.lineTo((point.X + xOffsetToApplyBeforeMultiplication) * horizontalMultiplicator + xOffsetToApplyAfterMultiplication, (point.Y + yOffsetToApplyBeforeMultiplication) * verticalMultiplicator + yOffsetToApplyAfterMultiplication);
                    }
                } finally {
                    if (Bridge.is($t, System.IDisposable)) {
                        $t.System$IDisposable$Dispose();
                    }
                }
                return this.Points.Count === 0 ? previousLastPoint : this.Points.getItem(((this.Points.Count - 1) | 0)).$clone();
            },
            GetMaxXY: function () {
                var $t;
                var currentMaxXY = new Windows.Foundation.Point.$ctor1(Number.NEGATIVE_INFINITY, Number.NEGATIVE_INFINITY);
                $t = Bridge.getEnumerator(this.Points);
                try {
                    while ($t.moveNext()) {
                        var point = $t.Current.$clone();
                        if (point.X > currentMaxXY.X) {
                            currentMaxXY.X = point.X;
                        }
                        if (point.Y > currentMaxXY.Y) {
                            currentMaxXY.Y = point.Y;
                        }
                    }
                } finally {
                    if (Bridge.is($t, System.IDisposable)) {
                        $t.System$IDisposable$Dispose();
                    }
                }
                return currentMaxXY.$clone();
            },
            GetMinMaxXY: function (minX, maxX, minY, maxY, startingPoint) {
                var $t;
                $t = Bridge.getEnumerator(this.Points);
                try {
                    while ($t.moveNext()) {
                        var point = $t.Current.$clone();
                        minX.v = Math.min(minX.v, point.X);
                        maxX.v = Math.max(maxX.v, point.X);
                        minY.v = Math.min(minY.v, point.Y);
                        maxY.v = Math.max(maxY.v, point.Y);
                    }
                } finally {
                    if (Bridge.is($t, System.IDisposable)) {
                        $t.System$IDisposable$Dispose();
                    }
                }
                return this.Points.Count === 0 ? startingPoint : this.Points.getItem(((this.Points.Count - 1) | 0)).$clone();
            }
        }
    });

    /**
     * Represents a set of quadratic Bezier segments.
     *
     * @public
     * @class Windows.UI.Xaml.Media.PolyQuadraticBezierSegment
     * @augments Windows.UI.Xaml.Media.PathSegment
     */
    Bridge.define("Windows.UI.Xaml.Media.PolyQuadraticBezierSegment", {
        inherits: [Windows.UI.Xaml.Media.PathSegment],
        statics: {
            fields: {
                /**
                 * Identifies the Points dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Media.PolyQuadraticBezierSegment
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                PointsProperty: null
            },
            ctors: {
                init: function () {
                    this.PointsProperty = Windows.UI.Xaml.DependencyProperty.Register("Points", Windows.UI.Xaml.Media.PointCollection, Windows.UI.Xaml.Media.PolyQuadraticBezierSegment, new Windows.UI.Xaml.PropertyMetadata.$ctor2(null, Windows.UI.Xaml.Media.PolyQuadraticBezierSegment.Points_Changed));
                }
            },
            methods: {
                Points_Changed: function (d, e) {
                    var segment = Bridge.cast(d, Windows.UI.Xaml.Media.PolyQuadraticBezierSegment);
                    if (segment.ParentPath != null) {
                        segment.ParentPath.ScheduleRedraw();
                    }
                }
            }
        },
        props: {
            /**
             * Gets or sets the Point collection that defines this PolyQuadraticBezierSegment
             object.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Media.PolyQuadraticBezierSegment
             * @function Points
             * @type Windows.UI.Xaml.Media.PointCollection
             */
            Points: {
                get: function () {
                    return Bridge.cast(this.GetValue(Windows.UI.Xaml.Media.PolyQuadraticBezierSegment.PointsProperty), Windows.UI.Xaml.Media.PointCollection);
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Media.PolyQuadraticBezierSegment.PointsProperty, value);
                }
            }
        },
        ctors: {
            /**
             * Initializes a new instance of the PolyQuadraticBezierSegment class.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Media.PolyQuadraticBezierSegment
             * @memberof Windows.UI.Xaml.Media.PolyQuadraticBezierSegment
             * @return  {void}
             */
            ctor: function () {
                this.$initialize();
                Windows.UI.Xaml.Media.PathSegment.ctor.call(this);

            }
        },
        methods: {
            SetParentPath: function (path) {
                Windows.UI.Xaml.Media.PathSegment.prototype.SetParentPath.call(this, path);
                this.Points.SetParentPath(path);
            },
            DefineInCanvas: function (xOffsetToApplyBeforeMultiplication, yOffsetToApplyBeforeMultiplication, xOffsetToApplyAfterMultiplication, yOffsetToApplyAfterMultiplication, horizontalMultiplicator, verticalMultiplicator, canvasDomElement, previousLastPoint) {
                var context = CSHTML5.Internal.INTERNAL_HtmlDomManager.Get2dCanvasContext(canvasDomElement);
                var i = 0;
                var lastPoint = previousLastPoint.$clone();
                while (i < ((this.Points.Count - 1) | 0)) {
                    var controlPoint1X = this.Points.getItem(i).$clone().X;
                    var controlPoint1Y = this.Points.getItem(i).$clone().Y;
                    i = (i + 1) | 0;
                    lastPoint = this.Points.getItem(i).$clone();
                    var endPointX = lastPoint.X;
                    var endPointY = lastPoint.Y;
                    i = (i + 1) | 0;

                    context.quadraticCurveTo(controlPoint1X * horizontalMultiplicator + xOffsetToApplyBeforeMultiplication * horizontalMultiplicator + xOffsetToApplyAfterMultiplication, controlPoint1Y * verticalMultiplicator + yOffsetToApplyBeforeMultiplication * verticalMultiplicator + yOffsetToApplyAfterMultiplication, endPointX * horizontalMultiplicator + xOffsetToApplyBeforeMultiplication * horizontalMultiplicator + xOffsetToApplyAfterMultiplication, endPointY * verticalMultiplicator + yOffsetToApplyBeforeMultiplication * verticalMultiplicator + yOffsetToApplyAfterMultiplication);
                }
                return lastPoint.$clone();
            },
            GetMaxXY: function () {
                var $t;
                var currentMax = new Windows.Foundation.Point.$ctor1(Number.NEGATIVE_INFINITY, Number.NEGATIVE_INFINITY);
                $t = Bridge.getEnumerator(this.Points);
                try {
                    while ($t.moveNext()) {
                        var point = $t.Current.$clone();
                        if (point.X > currentMax.X) {
                            currentMax.X = point.X;
                        }
                        if (point.Y > currentMax.Y) {
                            currentMax.Y = point.Y;
                        }
                    }
                } finally {
                    if (Bridge.is($t, System.IDisposable)) {
                        $t.System$IDisposable$Dispose();
                    }
                }
                return currentMax.$clone();
            },
            GetMinMaxXY: function (minX, maxX, minY, maxY, startingPoint) {
                var $t;
                $t = Bridge.getEnumerator(this.Points);
                try {
                    while ($t.moveNext()) {
                        var point = $t.Current.$clone();
                        minX.v = Math.min(minX.v, point.X);
                        maxX.v = Math.max(maxX.v, point.X);
                        minY.v = Math.min(minY.v, point.Y);
                        maxY.v = Math.max(maxY.v, point.Y);
                    }
                } finally {
                    if (Bridge.is($t, System.IDisposable)) {
                        $t.System$IDisposable$Dispose();
                    }
                }
                return this.Points.Count === 0 ? startingPoint : this.Points.getItem(((this.Points.Count - 1) | 0)).$clone();
            }
        }
    });

    /**
     * Creates a quadratic Bezier curve between two points in a PathFigure.
     *
     * @public
     * @class Windows.UI.Xaml.Media.QuadraticBezierSegment
     * @augments Windows.UI.Xaml.Media.PathSegment
     */
    Bridge.define("Windows.UI.Xaml.Media.QuadraticBezierSegment", {
        inherits: [Windows.UI.Xaml.Media.PathSegment],
        statics: {
            fields: {
                /**
                 * Identifies the Point1 dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Media.QuadraticBezierSegment
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                Point1Property: null,
                /**
                 * Identifies the Point2 dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Media.QuadraticBezierSegment
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                Point2Property: null
            },
            ctors: {
                init: function () {
                    this.Point1Property = Windows.UI.Xaml.DependencyProperty.Register("Point1", Windows.Foundation.Point, Windows.UI.Xaml.Media.QuadraticBezierSegment, new Windows.UI.Xaml.PropertyMetadata.$ctor2(new Windows.Foundation.Point.ctor().$clone(), Windows.UI.Xaml.Media.QuadraticBezierSegment.Point1_Changed));
                    this.Point2Property = Windows.UI.Xaml.DependencyProperty.Register("Point2", Windows.Foundation.Point, Windows.UI.Xaml.Media.QuadraticBezierSegment, new Windows.UI.Xaml.PropertyMetadata.$ctor2(new Windows.Foundation.Point.ctor().$clone(), Windows.UI.Xaml.Media.QuadraticBezierSegment.Point2_Changed));
                }
            },
            methods: {
                Point1_Changed: function (d, e) {
                    var segment = Bridge.cast(d, Windows.UI.Xaml.Media.QuadraticBezierSegment);
                    if (segment.ParentPath != null) {
                        segment.ParentPath.ScheduleRedraw();
                    }
                },
                Point2_Changed: function (d, e) {
                    var segment = Bridge.cast(d, Windows.UI.Xaml.Media.QuadraticBezierSegment);
                    if (segment.ParentPath != null) {
                        segment.ParentPath.ScheduleRedraw();
                    }
                }
            }
        },
        props: {
            /**
             * Gets or sets the control point of the curve.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Media.QuadraticBezierSegment
             * @function Point1
             * @type Windows.Foundation.Point
             */
            Point1: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.Media.QuadraticBezierSegment.Point1Property), Windows.Foundation.Point), Windows.Foundation.Point));
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Media.QuadraticBezierSegment.Point1Property, value.$clone());
                }
            },
            /**
             * Gets or sets the end Point of this QuadraticBezierSegment.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Media.QuadraticBezierSegment
             * @function Point2
             * @type Windows.Foundation.Point
             */
            Point2: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.Media.QuadraticBezierSegment.Point2Property), Windows.Foundation.Point), Windows.Foundation.Point));
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Media.QuadraticBezierSegment.Point2Property, value.$clone());
                }
            }
        },
        ctors: {
            /**
             * Initializes a new instance of the QuadraticBezierSegment class.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Media.QuadraticBezierSegment
             * @memberof Windows.UI.Xaml.Media.QuadraticBezierSegment
             * @return  {void}
             */
            ctor: function () {
                this.$initialize();
                Windows.UI.Xaml.Media.PathSegment.ctor.call(this);

            }
        },
        methods: {
            DefineInCanvas: function (xOffsetToApplyBeforeMultiplication, yOffsetToApplyBeforeMultiplication, xOffsetToApplyAfterMultiplication, yOffsetToApplyAfterMultiplication, horizontalMultiplicator, verticalMultiplicator, canvasDomElement, previousLastPoint) {
                var context = CSHTML5.Internal.INTERNAL_HtmlDomManager.Get2dCanvasContext(canvasDomElement);

                context.quadraticCurveTo(this.Point1.X * horizontalMultiplicator + xOffsetToApplyBeforeMultiplication * horizontalMultiplicator + xOffsetToApplyAfterMultiplication, this.Point1.Y * verticalMultiplicator + yOffsetToApplyBeforeMultiplication * verticalMultiplicator + yOffsetToApplyAfterMultiplication, this.Point2.X * horizontalMultiplicator + xOffsetToApplyBeforeMultiplication * horizontalMultiplicator + xOffsetToApplyAfterMultiplication, this.Point2.Y * verticalMultiplicator + yOffsetToApplyBeforeMultiplication * verticalMultiplicator + yOffsetToApplyAfterMultiplication);

                return this.Point2.$clone();
            },
            GetMaxXY: function () {
                var maxX = this.Point1.X;
                var maxY = this.Point1.Y;
                if (this.Point2.X > maxX) {
                    maxX = this.Point2.X;
                }
                if (this.Point2.Y > maxY) {
                    maxY = this.Point2.Y;
                }
                return new Windows.Foundation.Point.$ctor1(maxX, maxY);
            },
            GetMinMaxXY: function (minX, maxX, minY, maxY, startingPoint) {
                minX.v = Math.min(minX.v, Math.min(this.Point1.X, this.Point2.X));
                maxX.v = Math.max(maxX.v, Math.max(this.Point1.X, this.Point2.X));
                minY.v = Math.min(minY.v, Math.min(this.Point1.Y, this.Point2.Y));
                maxY.v = Math.max(maxY.v, Math.max(this.Point1.Y, this.Point2.Y));
                return this.Point2.$clone();
            }
        }
    });

    /**
     * Paints an area with a solid color.
     *
     * @public
     * @class Windows.UI.Xaml.Media.SolidColorBrush
     * @augments Windows.UI.Xaml.Media.Brush
     * @implements  Windows.UI.Xaml.ICanConvertToCSSValue
     * @implements  Windows.UI.Xaml.ICloneOnAnimation
     */
    Bridge.define("Windows.UI.Xaml.Media.SolidColorBrush", {
        inherits: [Windows.UI.Xaml.Media.Brush,Windows.UI.Xaml.ICanConvertToCSSValue,Windows.UI.Xaml.ICloneOnAnimation],
        statics: {
            fields: {
                /**
                 * Identifies the Color dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Media.SolidColorBrush
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                ColorProperty: null
            },
            ctors: {
                init: function () {
                    var $t;
                    this.ColorProperty = Windows.UI.Xaml.DependencyProperty.Register("Color", Windows.UI.Color, Windows.UI.Xaml.Media.SolidColorBrush, ($t = new Windows.UI.Xaml.PropertyMetadata.$ctor1(Windows.UI.Color.FromArgb(0, 0, 0, 0).$clone()), $t.GetCSSEquivalents = function (instance) {
                        var brush = Bridge.as(instance, Windows.UI.Xaml.Media.Brush);
                        if (brush != null) {
                            var parentPropertyToValueToHtmlConverter = function (parentPropertyCSSEquivalent) {
                                return (function (inst, value) {
                                    var $t1;
                                    var valuesDict = new (System.Collections.Generic.Dictionary$2(System.String,System.Object)).ctor();
                                    $t1 = Bridge.getEnumerator(parentPropertyCSSEquivalent.Name);
                                    try {
                                        while ($t1.moveNext()) {
                                            var name = $t1.Current;
                                            if (!System.String.endsWith(name, "Alpha")) {
                                                valuesDict.add(name, System.Nullable.getValue(Bridge.cast(Bridge.unbox(value, Windows.UI.Color), Windows.UI.Color)).INTERNAL_ToHtmlStringForVelocity());
                                            } else {
                                                valuesDict.add(name, Bridge.box(System.Nullable.getValue(Bridge.cast(Bridge.unbox(value, Windows.UI.Color), Windows.UI.Color)).A / 255, System.Double, System.Double.format, System.Double.getHashCode));
                                            }
                                        }
                                    } finally {
                                        if (Bridge.is($t1, System.IDisposable)) {
                                            $t1.System$IDisposable$Dispose();
                                        }
                                    }
                                    return valuesDict;
                                });
                            };

                            return Windows.UI.Xaml.Media.Brush.MergeCSSEquivalentsOfTheParentsProperties(brush, parentPropertyToValueToHtmlConverter);
                        } else {
                            throw new System.ArgumentException.ctor();
                        }
                    }, $t));
                }
            }
        },
        fields: {
            _isAlreadyAClone: false
        },
        props: {
            /**
             * Gets or sets the color of this SolidColorBrush.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Media.SolidColorBrush
             * @function Color
             * @type Windows.UI.Color
             */
            Color: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.Media.SolidColorBrush.ColorProperty), Windows.UI.Color), Windows.UI.Color));
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Media.SolidColorBrush.ColorProperty, value.$clone());
                }
            }
        },
        alias: [
            "ConvertToCSSValue", "Windows$UI$Xaml$ICanConvertToCSSValue$ConvertToCSSValue",
            "Clone", "Windows$UI$Xaml$ICloneOnAnimation$Clone",
            "IsAlreadyAClone", "Windows$UI$Xaml$ICloneOnAnimation$IsAlreadyAClone"
        ],
        ctors: {
            init: function () {
                this._isAlreadyAClone = false;
            },
            /**
             * Initializes a new instance of the SolidColorBrush class with no color.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Media.SolidColorBrush
             * @memberof Windows.UI.Xaml.Media.SolidColorBrush
             * @return  {void}
             */
            ctor: function () {
                this.$initialize();
                Windows.UI.Xaml.Media.Brush.ctor.call(this);
            },
            /**
             * Initializes a new instance of the SolidColorBrush class with the specified
             Color.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Media.SolidColorBrush
             * @memberof Windows.UI.Xaml.Media.SolidColorBrush
             * @param   {Windows.UI.Color}    color    The color to apply to the brush.
             * @return  {void}
             */
            $ctor1: function (color) {
                this.$initialize();
                Windows.UI.Xaml.Media.Brush.ctor.call(this);
                this.Color = color.$clone();
            }
        },
        methods: {
            INTERNAL_ToHtmlString: function () {
                return this.Color.INTERNAL_ToHtmlString(this.Opacity);
            },
            /**
             * // Intended to be called by objects that use the brush in order to apply the brush.
            //
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Media.SolidColorBrush
             * @memberof Windows.UI.Xaml.Media.SolidColorBrush
             * @return  {System.Object}
             */
            ConvertToCSSValue: function () {
                return this.Color.INTERNAL_ToHtmlString(this.Opacity);
            },
            Clone: function () {
                var $t;
                return ($t = new Windows.UI.Xaml.Media.SolidColorBrush.$ctor1(this.Color.$clone()), $t._isAlreadyAClone = true, $t);
            },
            IsAlreadyAClone: function () {
                return this._isAlreadyAClone;
            },
            toString: function () {
                return this.Color.toString();
            }
        }
    });

    Bridge.define("Windows.UI.Xaml.ResourceDictionaryCollection", {
        inherits: [Windows.UI.Xaml.PresentationFrameworkCollection$1(Windows.UI.Xaml.ResourceDictionary)],
        fields: {
            _owner: null,
            _monitor: null
        },
        events: {
            CollectionChanged: null
        },
        ctors: {
            init: function () {
                this._monitor = new Windows.UI.Xaml.ResourceDictionaryCollection.SimpleMonitor();
            },
            ctor: function (owner) {
                this.$initialize();
                Windows.UI.Xaml.PresentationFrameworkCollection$1(Windows.UI.Xaml.ResourceDictionary).ctor.call(this);
                System.Diagnostics.Debug.Assert$1(owner != null, "ResourceDictionaryCollection's owner cannot be null");
                this._owner = owner;
            }
        },
        methods: {
            AddOverride: function (value) {
                this.CheckReentrancy();
                this.CheckValue(value);
                this.AddInternal(value);
                value._parentDictionary = this._owner;
                this.OnCollectionChanged(new System.Collections.Specialized.NotifyCollectionChangedEventArgs.$ctor7(System.Collections.Specialized.NotifyCollectionChangedAction.Add, value, this.CountInternal));
            },
            ClearOverride: function () {
                var $t;
                this.CheckReentrancy();
                $t = Bridge.getEnumerator(this);
                try {
                    while ($t.moveNext()) {
                        var dictionary = $t.Current;
                        dictionary._parentDictionary = null;
                        this._owner.RemoveParentOwners(dictionary);
                    }
                } finally {
                    if (Bridge.is($t, System.IDisposable)) {
                        $t.System$IDisposable$Dispose();
                    }
                }
                this.ClearInternal();
                this.OnCollectionChanged(new System.Collections.Specialized.NotifyCollectionChangedEventArgs.ctor(System.Collections.Specialized.NotifyCollectionChangedAction.Reset));
            },
            InsertOverride: function (index, value) {
                this.CheckReentrancy();
                this.CheckValue(value);
                this.InsertInternal(index, value);
                value._parentDictionary = this._owner;
                this.OnCollectionChanged(new System.Collections.Specialized.NotifyCollectionChangedEventArgs.$ctor7(System.Collections.Specialized.NotifyCollectionChangedAction.Add, value, index));
            },
            RemoveAtOverride: function (index) {
                this.CheckReentrancy();
                var removedItem = this.GetItemInternal(index);
                this.RemoveAtInternal(index);
                removedItem._parentDictionary = null;
                this.OnCollectionChanged(new System.Collections.Specialized.NotifyCollectionChangedEventArgs.$ctor7(System.Collections.Specialized.NotifyCollectionChangedAction.Remove, removedItem, index));
            },
            RemoveOverride: function (value) {
                this.CheckReentrancy();
                var index = this.indexOf(value);
                if (index > -1) {
                    var removedItem = this.GetItemInternal(index);
                    this.RemoveAtInternal(index);
                    removedItem._parentDictionary = null;
                    this.OnCollectionChanged(new System.Collections.Specialized.NotifyCollectionChangedEventArgs.$ctor7(System.Collections.Specialized.NotifyCollectionChangedAction.Remove, removedItem, index));
                    return true;
                }
                return false;
            },
            GetItemOverride: function (index) {
                return this.GetItemInternal(index);
            },
            SetItemOverride: function (index, value) {
                this.CheckReentrancy();
                this.CheckValue(value);
                var originalItem = this.GetItemInternal(index);
                this.SetItemInternal(index, value);
                originalItem._parentDictionary = null;
                value._parentDictionary = this._owner;
                this.OnCollectionChanged(new System.Collections.Specialized.NotifyCollectionChangedEventArgs.$ctor10(System.Collections.Specialized.NotifyCollectionChangedAction.Replace, value, originalItem, index));
            },
            CheckValue: function (value) {
                if (value == null) {
                    throw new System.ArgumentNullException.$ctor1("value");
                }
                if (value._parentDictionary != null) {
                    throw new System.InvalidOperationException.$ctor1("Element is already the child of another element.");
                }
            },
            OnCollectionChanged: function (e) {
                var $t;
                if (!Bridge.staticEquals(this.CollectionChanged, null)) {
                    $t = this.BlockReentrancy();
                    try {
                        this.CollectionChanged(this, e);
                    }
                    finally {
                        if (Bridge.hasValue($t)) {
                            $t.System$IDisposable$Dispose();
                        }
                    }
                }
            },
            /**
             * Disallow reentrant attempts to change this collection. E.g. a event handler
             of the CollectionChanged event is not allowed to make changes to this collection.
             *
             * @instance
             * @private
             * @this Windows.UI.Xaml.ResourceDictionaryCollection
             * @memberof Windows.UI.Xaml.ResourceDictionaryCollection
             * @return  {System.IDisposable}
             */
            BlockReentrancy: function () {
                this._monitor.Enter();
                return this._monitor;
            },
            /**
             * Check and assert for reentrant attempts to change this collection.
             *
             * @instance
             * @private
             * @this Windows.UI.Xaml.ResourceDictionaryCollection
             * @memberof Windows.UI.Xaml.ResourceDictionaryCollection
             * @throws raised when changing the collection
             while another collection change is still being notified to other listeners
             * @return  {void}
             */
            CheckReentrancy: function () {
                if (this._monitor.Busy) {
                    if ((!Bridge.staticEquals(this.CollectionChanged, null)) && (Bridge.fn.getInvocationList(this.CollectionChanged).length > 1)) {
                        throw new System.InvalidOperationException.$ctor1("RowDefinitionCollection Reentrancy not allowed");
                    }
                }
            }
        }
    });

    /**
     * Applies a value to a property in a Style.
     *
     * @public
     * @class Windows.UI.Xaml.Setter
     * @augments Windows.UI.Xaml.SetterBase
     */
    Bridge.define("Windows.UI.Xaml.Setter", {
        inherits: [Windows.UI.Xaml.SetterBase],
        statics: {
            fields: {
                ValueProperty: null
            },
            ctors: {
                init: function () {
                    this.ValueProperty = Windows.UI.Xaml.DependencyProperty.Register("Value", System.Object, Windows.UI.Xaml.Setter, new Windows.UI.Xaml.PropertyMetadata.$ctor2(null, Windows.UI.Xaml.Setter.OnValueChanged));
                }
            },
            methods: {
                OnValueChanged: function (d, e) {
                    var setter = Bridge.cast(d, Windows.UI.Xaml.Setter);
                    if (setter._throwOnNextValueChange) {
                        setter._throwOnNextValueChange = false;
                        setter.CheckSealed();
                        throw new System.InvalidOperationException.$ctor1("Cannot unset a Setter value.");
                    }
                    if (setter.IsSealed || Bridge.referenceEquals(e.NewValue, Windows.UI.Xaml.DependencyProperty.UnsetValue)) {
                        setter._throwOnNextValueChange = true;
                        setter.Value = e.OldValue;
                    }
                }
            }
        },
        fields: {
            _property: null,
            _throwOnNextValueChange: false
        },
        props: {
            /**
             * Gets or sets the property to apply the Value to. The default is null.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Setter
             * @function Property
             * @type Windows.UI.Xaml.DependencyProperty
             */
            Property: {
                get: function () {
                    return this._property;
                },
                set: function (value) {
                    this.CheckValidProperty(value);
                    this.CheckSealed();
                    this._property = value;
                }
            },
            /**
             * Gets or sets the value to apply to the property that is specified by the
             Setter.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Setter
             * @function Value
             * @type System.Object
             */
            Value: {
                get: function () {
                    return this.GetValue(Windows.UI.Xaml.Setter.ValueProperty);
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Setter.ValueProperty, value);
                }
            }
        },
        ctors: {
            /**
             * Initializes a new instance of the Setter class with no initial Property or
             Value.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Setter
             * @memberof Windows.UI.Xaml.Setter
             * @return  {void}
             */
            ctor: function () {
                this.$initialize();
                Windows.UI.Xaml.SetterBase.ctor.call(this);
            },
            /**
             * Initializes a new instance of the Setter class with initial Property and
             Value information.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Setter
             * @memberof Windows.UI.Xaml.Setter
             * @param   {Windows.UI.Xaml.DependencyProperty}    property    The dependency property identifier for the property that is being styled.
             * @param   {System.Object}                         value       The value to assign to the value when the Setter applies.
             * @return  {void}
             */
            $ctor1: function (property, value) {
                this.$initialize();
                Windows.UI.Xaml.SetterBase.ctor.call(this);
                if (Bridge.referenceEquals(value, Windows.UI.Xaml.DependencyProperty.UnsetValue)) {
                    throw new System.ArgumentException.$ctor1("Cannot unset a Setter value.");
                }
                this.CheckValidProperty(property);

                this.Property = property;
                this.Value = value;
            }
        },
        methods: {
            /**
             * Seals this setter
             *
             * @instance
             * @override
             * @this Windows.UI.Xaml.Setter
             * @memberof Windows.UI.Xaml.Setter
             * @return  {void}
             */
            Seal: function () {

                var dp = this.Property;
                var value = this.Value;

                if (dp == null) {
                    throw new System.ArgumentException.$ctor1(System.String.format("Must have non-null value for '{0}'.", ["Setter.Property"]));
                }


                Windows.UI.Xaml.SetterBase.prototype.Seal.call(this);
            },
            CheckValidProperty: function (property) {
                if (property == null) {
                    throw new System.ArgumentNullException.$ctor1("property");
                }

                if (Bridge.referenceEquals(property, Windows.UI.Xaml.FrameworkElement.NameProperty)) {
                    throw new System.InvalidOperationException.$ctor1(System.String.format("'{0}' property cannot be set in the current element's Style.", [Windows.UI.Xaml.FrameworkElement.NameProperty.Name]));
                }
            }
        }
    });

    /**
     * Represents a collection of objects that inherit from System.Windows.SetterBase.
     *
     * @public
     * @class Windows.UI.Xaml.SetterBaseCollection
     * @augments Windows.UI.Xaml.PresentationFrameworkCollection$1
     */
    Bridge.define("Windows.UI.Xaml.SetterBaseCollection", {
        inherits: [Windows.UI.Xaml.PresentationFrameworkCollection$1(Windows.UI.Xaml.SetterBase)],
        fields: {
            _sealed: false
        },
        props: {
            /**
             * Returns the sealed state of this object.  If true, any attempt
                 at modifying the state of this object will trigger an exception.
             *
             * @instance
             * @public
             * @readonly
             * @memberof Windows.UI.Xaml.SetterBaseCollection
             * @function IsSealed
             * @type boolean
             */
            IsSealed: {
                get: function () {
                    return this._sealed;
                }
            }
        },
        ctors: {
            /**
             * Initializes a new instance of the System.Windows.SetterBaseCollection class.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.SetterBaseCollection
             * @memberof Windows.UI.Xaml.SetterBaseCollection
             * @return  {void}
             */
            ctor: function () {
                this.$initialize();
                Windows.UI.Xaml.PresentationFrameworkCollection$1(Windows.UI.Xaml.SetterBase).ctor.call(this);

            }
        },
        methods: {
            AddOverride: function (value) {
                this.CheckSealed();
                this.SetterBaseValidation(value);
                this.AddInternal(value);
            },
            ClearOverride: function () {
                this.CheckSealed();
                this.ClearInternal();
            },
            InsertOverride: function (index, value) {
                this.CheckSealed();
                this.SetterBaseValidation(value);
                this.InsertInternal(index, value);
            },
            RemoveAtOverride: function (index) {
                this.CheckSealed();
                this.RemoveAtInternal(index);
            },
            RemoveOverride: function (value) {
                this.CheckSealed();
                this.SetterBaseValidation(value);
                return this.RemoveInternal(value);
            },
            GetItemOverride: function (index) {
                return this.GetItemInternal(index);
            },
            SetItemOverride: function (index, value) {
                this.CheckSealed();
                this.SetterBaseValidation(value);
                this.SetItemInternal(index, value);
            },
            Seal: function () {
                this._sealed = true;

                for (var i = 0; i < this.Count; i = (i + 1) | 0) {
                    this.getItem(i).Seal();
                }
            },
            CheckSealed: function () {
                if (this._sealed) {
                    throw new System.InvalidOperationException.$ctor1(System.String.format("Cannot modify a '{0}' after it is sealed.", ["SetterBaseCollection"]));
                }
            },
            SetterBaseValidation: function (setterBase) {
                if (setterBase == null) {
                    throw new System.ArgumentNullException.$ctor1("setterBase");
                }
            }
        }
    });

    
    Bridge.define("CSHTML5.Native.Html.Controls.HtmlCanvas", {
        inherits: [Windows.UI.Xaml.FrameworkElement],
        statics: {
            methods: {
                /**
                 * Return the path to the topmost element that is under the pointer
                 *
                 * @static
                 * @this CSHTML5.Native.Html.Controls.HtmlCanvas
                 * @memberof CSHTML5.Native.Html.Controls.HtmlCanvas
                 * @param   {CSHTML5.Native.Html.Controls.HtmlCanvas}    canvas    The container of the elements
                 * @param   {number}                                     x         
                 * @param   {number}                                     y
                 * @return  {System.Collections.Generic.Stack$1}                   The path to the topmost element that is under the pointer
                 */
                GetPointedElements: function (canvas, x, y) {
                    var finalPath = new (System.Collections.Generic.Stack$1(CSHTML5.Native.Html.Controls.HtmlCanvasElement)).ctor();

                    var currentPath = new (System.Collections.Generic.Stack$1(CSHTML5.Native.Html.Controls.HtmlCanvasElement)).ctor();

                    var currentPathIndexes = new (System.Collections.Generic.Stack$1(System.Int32)).ctor();

                    for (var i = 0; i < canvas.Children.Count; i = (i + 1) | 0) {

                        currentPath.Push(canvas.Children.getItem(i));
                        currentPathIndexes.Push(0);

                        while (currentPath.Count > 0) {
                            var currentElement = currentPath.Peek();
                            var index = currentPathIndexes.Pop();

                            if (currentElement.IsHitTestVisible && currentElement.Visibility === Windows.UI.Xaml.Visibility.Visible && currentElement.IsPointed(x, y) && System.Array.indexOfT(finalPath.ToArray(), currentElement) === -1) {
                                finalPath.Clear();
                                var path = currentPath.ToArray();
                                for (var j = (path.length - 1) | 0; j >= 0; j = (j - 1) | 0) {
                                    finalPath.Push(path[System.Array.index(j, path)]);
                                }
                            }

                            if (currentElement.IsHitTestVisible && currentElement.Visibility === Windows.UI.Xaml.Visibility.Visible && Bridge.is(currentElement, CSHTML5.Native.Html.Controls.ContainerElement) && index < Bridge.cast(currentElement, CSHTML5.Native.Html.Controls.ContainerElement).Children.Count) {
                                currentPathIndexes.Push(((index + 1) | 0));

                                x -= currentElement.X;
                                y -= currentElement.Y;
                                currentPath.Push(Bridge.cast(currentElement, CSHTML5.Native.Html.Controls.ContainerElement).Children.getItem(index));
                                currentPathIndexes.Push(0);
                            } else {
                                currentPath.Pop();
                                if (currentPath.Count > 0) {
                                    x += currentPath.Peek().X;
                                    y += currentPath.Peek().Y;
                                }
                            }
                        }
                    }
                    return finalPath;
                },
                SearchElement: function (canvas, elem) {
                    var path = new (System.Collections.Generic.Stack$1(CSHTML5.Native.Html.Controls.HtmlCanvasElement)).ctor();

                    var currentPathIndexes = new (System.Collections.Generic.Stack$1(System.Int32)).ctor();

                    for (var i = 0; i < canvas.Children.Count; i = (i + 1) | 0) {
                        path.Push(canvas.Children.getItem(i));
                        currentPathIndexes.Push(0);

                        while (path.Count > 0) {
                            var currentElement = path.Peek();
                            var index = currentPathIndexes.Pop();

                            if (Bridge.referenceEquals(currentElement, elem)) {
                                return path;
                            }

                            if (currentElement.IsHitTestVisible && Bridge.is(currentElement, CSHTML5.Native.Html.Controls.ContainerElement) && index < Bridge.cast(currentElement, CSHTML5.Native.Html.Controls.ContainerElement).Children.Count) {
                                currentPathIndexes.Push(((index + 1) | 0));

                                path.Push(Bridge.cast(currentElement, CSHTML5.Native.Html.Controls.ContainerElement).Children.getItem(index));
                                currentPathIndexes.Push(0);
                            } else {
                                path.Pop();
                            }
                        }
                    }
                    return null;
                }
            }
        },
        fields: {
            _jsCanvas: null,
            _jsContext2d: null,
            _currentDrawingStyle: null,
            /**
             * Gets the collection of child elements of the canvas.
             *
             * @instance
             * @public
             * @memberof CSHTML5.Native.Html.Controls.HtmlCanvas
             * @type System.Collections.Generic.List$1
             */
            Children: null,
            _LastPointerMove: null
        },
        ctors: {
            /**
             * Create an html5 native canvas
             *
             * @instance
             * @public
             * @this CSHTML5.Native.Html.Controls.HtmlCanvas
             * @memberof CSHTML5.Native.Html.Controls.HtmlCanvas
             * @return  {void}
             */
            ctor: function () {
                this.$initialize();
                Windows.UI.Xaml.FrameworkElement.ctor.call(this);
                this.Children = new (System.Collections.Generic.List$1(CSHTML5.Native.Html.Controls.HtmlCanvasElement)).ctor();

                this.addPointerMoved(Bridge.fn.cacheBind(this, this.HtmlCanvas_PointerMoved));
                this.addPointerPressed(Bridge.fn.cacheBind(this, this.HtmlCanvas_PointerPressed));
                this.addPointerReleased(Bridge.fn.cacheBind(this, this.HtmlCanvas_PointerReleased));
                this.addRightTapped(Bridge.fn.cacheBind(this, this.HtmlCanvas_RightTapped));
            },
            /**
             * Create an html5 native canvas
             *
             * @instance
             * @public
             * @this CSHTML5.Native.Html.Controls.HtmlCanvas
             * @memberof CSHTML5.Native.Html.Controls.HtmlCanvas
             * @param   {number}    Width     The Width of the canvas.
             * @param   {number}    Height    The Height of the canvas.
             * @return  {void}
             */
            $ctor1: function (Width, Height) {
                this.$initialize();
                Windows.UI.Xaml.FrameworkElement.ctor.call(this);
                this.Width = Width;
                this.Height = Height;
            }
        },
        methods: {
            HtmlCanvas_PointerMoved: function (sender, e) {
                var pos = e.GetCurrentPoint(this).Position.$clone();

                var elements = CSHTML5.Native.Html.Controls.HtmlCanvas.GetPointedElements(this, pos.X, pos.Y).ToArray();

                var e2 = new CSHTML5.Native.Html.Input.HtmlCanvasPointerRoutedEventArgs.ctor(e, this);

                for (var i = 0; i < elements.length && !e.Handled; i = (i + 1) | 0) {
                    elements[System.Array.index(i, elements)].OnPointerMoved(e2);
                }

                if (this._LastPointerMove != null) {
                    e2 = new CSHTML5.Native.Html.Input.HtmlCanvasPointerRoutedEventArgs.ctor(e, this);
                    for (var i1 = 0; i1 < elements.length && !e.Handled; i1 = (i1 + 1) | 0) {
                        if (System.Array.indexOfT(this._LastPointerMove, elements[System.Array.index(i1, elements)]) === -1) {
                            elements[System.Array.index(i1, elements)].OnPointerEntered(e2);
                        }
                    }

                    e2 = new CSHTML5.Native.Html.Input.HtmlCanvasPointerRoutedEventArgs.ctor(e, this);
                    for (var i2 = 0; i2 < this._LastPointerMove.length && !e.Handled; i2 = (i2 + 1) | 0) {
                        if (System.Array.indexOfT(elements, this._LastPointerMove[System.Array.index(i2, this._LastPointerMove)]) === -1) {
                            this._LastPointerMove[System.Array.index(i2, this._LastPointerMove)].OnPointerExited(e2);
                        }
                    }
                }

                this._LastPointerMove = elements;
            },
            HtmlCanvas_PointerPressed: function (sender, e) {
                var pos = e.GetCurrentPoint(this).Position.$clone();

                var elements = CSHTML5.Native.Html.Controls.HtmlCanvas.GetPointedElements(this, pos.X, pos.Y);

                var e2 = new CSHTML5.Native.Html.Input.HtmlCanvasPointerRoutedEventArgs.ctor(e, this);

                while (!e.Handled && elements.Count > 0) {
                    var el = elements.Pop();
                    el.OnPointerPressed(e2);
                }
            },
            HtmlCanvas_RightTapped: function (sender, e) {
                var pos = e.GetCurrentPoint(this).Position.$clone();

                var elements = CSHTML5.Native.Html.Controls.HtmlCanvas.GetPointedElements(this, pos.X, pos.Y);

                var e2 = new CSHTML5.Native.Html.Input.HtmlCanvasPointerRoutedEventArgs.$ctor1(e, this);

                while (!e.Handled && elements.Count > 0) {
                    var el = elements.Pop();
                    el.OnRightTapped(e2);
                }
            },
            HtmlCanvas_PointerReleased: function (sender, e) {
                var pos = e.GetCurrentPoint(this).Position.$clone();

                var elements = CSHTML5.Native.Html.Controls.HtmlCanvas.GetPointedElements(this, pos.X, pos.Y);

                var e2 = new CSHTML5.Native.Html.Input.HtmlCanvasPointerRoutedEventArgs.ctor(e, this);

                while (!e.Handled && elements.Count > 0) {
                    elements.Pop().OnPointerReleased(e2);
                }
            },
            CreateDomElement: function (parentRef, domElementWhereToPlaceChildren) {

                var div1 = { };
                var div1style = CSHTML5.Internal.INTERNAL_HtmlDomManager.CreateDomElementAppendItAndGetStyle("canvas", parentRef, this, div1);
                domElementWhereToPlaceChildren.v = div1.v;

                this._jsCanvas = div1.v;
                this._jsContext2d = this._jsCanvas.getContext('2d');

                this._jsCanvas.onselectstart = function() { return false; };

                return div1.v;
            },
            /**
             * Render the canvas content
             *
             * @instance
             * @public
             * @this CSHTML5.Native.Html.Controls.HtmlCanvas
             * @memberof CSHTML5.Native.Html.Controls.HtmlCanvas
             * @return  {void}
             */
            Draw: function () {
                var $t;
                if (this.IsLoaded) {
                    this._jsCanvas.width = this._jsCanvas.scrollWidth;
                    this._jsCanvas.height = this._jsCanvas.scrollHeight;
                    $t = Bridge.getEnumerator(this.Children);
                    try {
                        while ($t.moveNext()) {
                            var elem = $t.Current;
                            this._currentDrawingStyle = elem.Draw(this._currentDrawingStyle, this._jsContext2d);
                        }
                    } finally {
                        if (Bridge.is($t, System.IDisposable)) {
                            $t.System$IDisposable$Dispose();
                        }
                    }
                } else {
                    throw new System.Exception("The 'Draw' method of the HtmlCanvas control can only be called after the control has been added to the Visual Tree. For example, you can register to the Loaded event of the HtmlCanvas and call the Draw method at that moment. To know whether the HtmlCanvas is in the Visual Tree or not, you can read the IsLoaded property of the HtmlCanvas.");
                }
            }
        }
    });

    Bridge.define("CSHTML5.Native.Html.Controls.HtmlPresenter", {
        inherits: [Windows.UI.Xaml.FrameworkElement],
        statics: {
            methods: {
                IsNullOrUndefined: function (jsObject) {
                    if (CSHTML5.Interop.IsRunningInTheSimulator) {
                        if (jsObject == null) {
                            return true;
                        }
                        if (!(Bridge.is(jsObject, DotNetBrowser.JSValue))) {
                            return false;
                        }
                        var value = Bridge.cast(jsObject, DotNetBrowser.JSValue);
                        return value.IsNull() || value.IsUndefined();
                    } else {
                        return System.Convert.toBoolean((typeof jsObject === 'undefined' || jsObject === null));
                    }
                }
            }
        },
        fields: {
            _jsDiv: null,
            _htmlContent: null
        },
        props: {
            Html: {
                get: function () {
                    return this._htmlContent;
                },
                set: function (value) {
                    this._htmlContent = value;

                    if (this.IsLoaded) {
                        this.ApplyHtmlContent(this._htmlContent);
                    }
                }
            },
            DomElement: {
                get: function () {
                    if (this.IsLoaded) {
                        if (!CSHTML5.Native.Html.Controls.HtmlPresenter.IsNullOrUndefined(this._jsDiv)) {
                            if (System.Convert.toBoolean(this._jsDiv.hasChildNodes())) {
                                return this._jsDiv.firstChild;
                            }
                        }
                    }
                    return null;
                }
            }
        },
        methods: {
            CreateDomElement: function (parentRef, domElementWhereToPlaceChildren) {
                this._jsDiv = CSHTML5.Internal.INTERNAL_HtmlDomManager.CreateDomElementAndAppendIt("div", parentRef, this);
                domElementWhereToPlaceChildren.v = this._jsDiv;
                return this._jsDiv;
            },
            INTERNAL_OnAttachedToVisualTree: function () {
                Windows.UI.Xaml.FrameworkElement.prototype.INTERNAL_OnAttachedToVisualTree.call(this);

                if (this._htmlContent != null) {
                    this.ApplyHtmlContent(this._htmlContent);
                }
            },
            ApplyHtmlContent: function (htmlContent) {
                this._jsDiv.innerHTML = this._htmlContent;
            }
        }
    });

    /**
     * @memberof Windows.UI.Xaml.Controls
     * @callback Windows.UI.Xaml.Controls.SelectionChangedEventHandler
     * @param   {System.Object}                                         sender    
     * @param   {Windows.UI.Xaml.Controls.SelectionChangedEventArgs}    e
     * @return  {void}
     */

    Bridge.define("CSHTML5.Native.Html.Controls.NativeComboBox", {
        inherits: [Windows.UI.Xaml.FrameworkElement],
        statics: {
            fields: {
                /**
                 * Identifies the ItemsSource dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof CSHTML5.Native.Html.Controls.NativeComboBox
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                ItemsSourceProperty: null,
                /**
                 * Identifies the DisplayMemberPath dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof CSHTML5.Native.Html.Controls.NativeComboBox
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                DisplayMemberPathProperty: null,
                /**
                 * Identifies the SelectedIndex dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof CSHTML5.Native.Html.Controls.NativeComboBox
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                SelectedIndexProperty: null,
                /**
                 * Identifies the SelectedItem dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof CSHTML5.Native.Html.Controls.NativeComboBox
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                SelectedItemProperty: null,
                /**
                 * Identifies the SelectedValue dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof CSHTML5.Native.Html.Controls.NativeComboBox
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                SelectedValueProperty: null,
                /**
                 * Identifies the SelectedValuePath dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof CSHTML5.Native.Html.Controls.NativeComboBox
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                SelectedValuePathProperty: null
            },
            ctors: {
                init: function () {
                    this.ItemsSourceProperty = Windows.UI.Xaml.DependencyProperty.Register("ItemsSource", System.Collections.IEnumerable, CSHTML5.Native.Html.Controls.NativeComboBox, new Windows.UI.Xaml.PropertyMetadata.$ctor2(null, CSHTML5.Native.Html.Controls.NativeComboBox.OnItemsSourceChanged));
                    this.DisplayMemberPathProperty = Windows.UI.Xaml.DependencyProperty.Register("DisplayMemberPath", System.String, CSHTML5.Native.Html.Controls.NativeComboBox, new Windows.UI.Xaml.PropertyMetadata.$ctor2("", CSHTML5.Native.Html.Controls.NativeComboBox.OnDisplayMemberPathChanged));
                    this.SelectedIndexProperty = Windows.UI.Xaml.DependencyProperty.Register("SelectedIndex", System.Int32, CSHTML5.Native.Html.Controls.NativeComboBox, new Windows.UI.Xaml.PropertyMetadata.$ctor3(Bridge.box(-1, System.Int32), CSHTML5.Native.Html.Controls.NativeComboBox.OnSelectedIndexChanged, CSHTML5.Native.Html.Controls.NativeComboBox.CoerceSelectedIndex));
                    this.SelectedItemProperty = Windows.UI.Xaml.DependencyProperty.Register("SelectedItem", System.Object, CSHTML5.Native.Html.Controls.NativeComboBox, new Windows.UI.Xaml.PropertyMetadata.$ctor3(null, CSHTML5.Native.Html.Controls.NativeComboBox.OnSelectedItemChanged, CSHTML5.Native.Html.Controls.NativeComboBox.CoerceSelectedItem));
                    this.SelectedValueProperty = Windows.UI.Xaml.DependencyProperty.Register("SelectedValue", System.Object, CSHTML5.Native.Html.Controls.NativeComboBox, new Windows.UI.Xaml.PropertyMetadata.$ctor3(null, CSHTML5.Native.Html.Controls.NativeComboBox.OnSelectedValueChanged, CSHTML5.Native.Html.Controls.NativeComboBox.CoerceSelectedValue));
                    this.SelectedValuePathProperty = Windows.UI.Xaml.DependencyProperty.Register("SelectedValuePath", System.String, CSHTML5.Native.Html.Controls.NativeComboBox, new Windows.UI.Xaml.PropertyMetadata.$ctor2("", CSHTML5.Native.Html.Controls.NativeComboBox.OnSelectedValuePathChanged));
                }
            },
            methods: {
                OnItemsSourceChanged: function (d, e) {
                    var cb = Bridge.cast(d, CSHTML5.Native.Html.Controls.NativeComboBox);
                    var newValue = Bridge.cast(e.NewValue, System.Collections.IEnumerable);

                    if (e.NewValue != null) {
                        cb.Items.SetItemsSource(newValue);
                    } else {
                        cb.Items.ClearItemsSource();
                    }

                    cb.SetCurrentValue(CSHTML5.Native.Html.Controls.NativeComboBox.SelectedIndexProperty, Bridge.box(-1, System.Int32));
                },
                OnDisplayMemberPathChanged: function (d, e) {
                    var cb = Bridge.cast(d, CSHTML5.Native.Html.Controls.NativeComboBox);

                    cb.Refresh();
                },
                OnSelectedIndexChanged: function (d, e) {
                    var cb = Bridge.cast(d, CSHTML5.Native.Html.Controls.NativeComboBox);

                    var newValue = System.Nullable.getValue(Bridge.cast(Bridge.unbox(e.NewValue, System.Int32), System.Int32));

                    if (!cb._changingSelectionInHtml) {
                        cb._changingSelectionProgrammatically = true;

                        try {
                            cb.SetSelectedIndexInNativeHtmlDom(newValue);
                        } finally {
                            cb._changingSelectionProgrammatically = false;
                        }
                    }

                    if (!cb._selectionChangeIsOnValue && !cb._selectionChangeIsOnItem) {
                        cb._selectionChangeIsOnIndex = true;

                        try {
                            if (newValue === -1) {
                                cb.SetCurrentValue(CSHTML5.Native.Html.Controls.NativeComboBox.SelectedValueProperty, null);

                                cb.SkipCoerceSelectedItemCheck = true;

                                try {
                                    cb.SetCurrentValue(CSHTML5.Native.Html.Controls.NativeComboBox.SelectedItemProperty, null);
                                } finally {
                                    cb.SkipCoerceSelectedItemCheck = false;
                                }
                            } else {
                                var item = cb.Items.getItem(newValue);

                                cb.SetCurrentValue(CSHTML5.Native.Html.Controls.NativeComboBox.SelectedValueProperty, Windows.UI.Xaml.PropertyPathHelper.AccessValueByApplyingPropertyPathIfAny(item, cb.SelectedValuePath));

                                cb.SkipCoerceSelectedItemCheck = true;

                                try {
                                    cb.SetCurrentValue(CSHTML5.Native.Html.Controls.NativeComboBox.SelectedItemProperty, item);
                                } finally {
                                    cb.SkipCoerceSelectedItemCheck = false;
                                }
                            }
                        } finally {
                            cb._selectionChangeIsOnIndex = false;
                        }
                    }
                },
                CoerceSelectedIndex: function (d, baseValue) {
                    var cb = Bridge.cast(d, CSHTML5.Native.Html.Controls.NativeComboBox);

                    var index = System.Nullable.getValue(Bridge.cast(Bridge.unbox(baseValue, System.Int32), System.Int32));
                    if (index < 0) {
                        return Bridge.box(-1, System.Int32);
                    } else if (index >= cb.Items.Count) {
                        return Bridge.box(((cb.Items.Count - 1) | 0), System.Int32);
                    } else {
                        return Bridge.box(index, System.Int32);
                    }
                },
                OnSelectedItemChanged: function (d, e) {
                    var cb = Bridge.cast(d, CSHTML5.Native.Html.Controls.NativeComboBox);
                    var newValue = e.NewValue;

                    if (!cb._selectionChangeIsOnValue && !cb._selectionChangeIsOnIndex) {
                        cb._selectionChangeIsOnItem = true;

                        try {
                            if (newValue == null) {
                                cb.SetCurrentValue(CSHTML5.Native.Html.Controls.NativeComboBox.SelectedValueProperty, null);
                                cb.SetCurrentValue(CSHTML5.Native.Html.Controls.NativeComboBox.SelectedIndexProperty, Bridge.box(-1, System.Int32));
                            } else {
                                cb.SetCurrentValue(CSHTML5.Native.Html.Controls.NativeComboBox.SelectedValueProperty, Windows.UI.Xaml.PropertyPathHelper.AccessValueByApplyingPropertyPathIfAny(newValue, cb.SelectedValuePath));
                                cb.SetCurrentValue(CSHTML5.Native.Html.Controls.NativeComboBox.SelectedIndexProperty, Bridge.box(cb.Items.indexOf(newValue), System.Int32));
                            }
                        } finally {
                            cb._selectionChangeIsOnItem = false;
                        }
                    }

                    cb.OnSelectionChanged(e.OldValue, e.NewValue);
                },
                CoerceSelectedItem: function (d, baseValue) {
                    var cb = Bridge.cast(d, CSHTML5.Native.Html.Controls.NativeComboBox);

                    if (baseValue == null || cb.SkipCoerceSelectedItemCheck) {
                        return baseValue;
                    }

                    var selectedIndex = cb.SelectedIndex;

                    if ((selectedIndex > -1 && selectedIndex < cb.Items.Count && Bridge.referenceEquals(cb.Items.getItem(selectedIndex), baseValue)) || cb.Items.contains(baseValue)) {
                        return baseValue;
                    }

                    return Windows.UI.Xaml.DependencyProperty.UnsetValue;
                },
                OnSelectedValueChanged: function (d, e) {
                    var cb = Bridge.cast(d, CSHTML5.Native.Html.Controls.NativeComboBox);

                    if (!cb._selectionChangeIsOnItem && !cb._selectionChangeIsOnIndex) {
                        cb._selectionChangeIsOnValue = true;

                        try {
                            cb.SetCurrentValue(CSHTML5.Native.Html.Controls.NativeComboBox.SelectedIndexProperty, Bridge.box(cb._selectionInfo.Index, System.Int32));

                            cb.SetCurrentValue(CSHTML5.Native.Html.Controls.NativeComboBox.SelectedItemProperty, cb._selectionInfo.Item);
                        } finally {
                            cb._selectionChangeIsOnValue = false;
                            cb._selectionInfo = null;
                        }
                    }
                },
                CoerceSelectedValue: function (d, baseValue) {
                    var cb = Bridge.cast(d, CSHTML5.Native.Html.Controls.NativeComboBox);

                    if (cb._selectionChangeIsOnIndex || cb._selectionChangeIsOnItem) {
                        return baseValue;
                    } else {
                        var index = { };
                        var item = cb.FindItemWithValue(baseValue, index);

                        if (Bridge.referenceEquals(item, Windows.UI.Xaml.DependencyProperty.UnsetValue) && cb.Items.Count > 0) {
                            baseValue = null;
                        }

                        cb._selectionInfo = new CSHTML5.Native.Html.Controls.NativeComboBox.SelectionInfo(Bridge.referenceEquals(item, Windows.UI.Xaml.DependencyProperty.UnsetValue) ? null : item, index.v);
                    }

                    return baseValue;
                },
                OnSelectedValuePathChanged: function (d, e) {
                    var cb = Bridge.cast(d, CSHTML5.Native.Html.Controls.NativeComboBox);

                    cb._selectionChangeIsOnItem = true;
                    cb._selectionChangeIsOnIndex = true;

                    try {
                        var item = null;
                        if (cb.SelectedItem != null) {
                            item = Windows.UI.Xaml.PropertyPathHelper.AccessValueByApplyingPropertyPathIfAny(cb.SelectedItem, Bridge.cast(e.NewValue, System.String));
                        }
                        cb.SetCurrentValue(CSHTML5.Native.Html.Controls.NativeComboBox.SelectedValueProperty, item);
                    } finally {
                        cb._selectionChangeIsOnItem = false;
                        cb._selectionChangeIsOnIndex = false;
                    }
                }
            }
        },
        fields: {
            _nativeComboBoxDomElement: null,
            _changeEventProxy: null,
            _changingSelectionProgrammatically: false,
            _changingSelectionInHtml: false,
            _selectionChangeIsOnIndex: false,
            _selectionChangeIsOnItem: false,
            _selectionChangeIsOnValue: false,
            SkipCoerceSelectedItemCheck: false,
            _selectionInfo: null,
            _items: null
        },
        events: {
            /**
             * Occurs when the selection is changed.
             *
             * @instance
             * @public
             * @this CSHTML5.Native.Html.Controls.NativeComboBox
             * @memberof CSHTML5.Native.Html.Controls.NativeComboBox
             * @function addSelectionChanged
             * @param   {Windows.UI.Xaml.Controls.SelectionChangedEventHandler}    value
             * @return  {void}
             */
            /**
             * Occurs when the selection is changed.
             *
             * @instance
             * @public
             * @this CSHTML5.Native.Html.Controls.NativeComboBox
             * @memberof CSHTML5.Native.Html.Controls.NativeComboBox
             * @function removeSelectionChanged
             * @param   {Windows.UI.Xaml.Controls.SelectionChangedEventHandler}    value
             * @return  {void}
             */
            SelectionChanged: null
        },
        props: {
            /**
             * Items is the collection of data that is used to generate the content
             of this control.
             *
             * @instance
             * @public
             * @readonly
             * @memberof CSHTML5.Native.Html.Controls.NativeComboBox
             * @function Items
             * @type Windows.UI.Xaml.Controls.ItemCollection
             */
            Items: {
                get: function () {
                    if (this._items == null) {
                        this._items = new Windows.UI.Xaml.Controls.ItemCollection();
                        this._items.addCollectionChanged(Bridge.fn.cacheBind(this, this.OnItemCollectionChanged));
                    }
                    return this._items;
                }
            },
            /**
             * Gets or sets an object source used to generate the content of the NativeComboBox.
             *
             * @instance
             * @public
             * @memberof CSHTML5.Native.Html.Controls.NativeComboBox
             * @function ItemsSource
             * @type System.Collections.IEnumerable
             */
            ItemsSource: {
                get: function () {
                    return Bridge.cast(this.GetValue(CSHTML5.Native.Html.Controls.NativeComboBox.ItemsSourceProperty), System.Collections.IEnumerable);
                },
                set: function (value) {
                    this.SetValue(CSHTML5.Native.Html.Controls.NativeComboBox.ItemsSourceProperty, value);
                }
            },
            /**
             * Gets or sets a path to a value on the source object to serve as the visual
             representation of the object.
             *
             * @instance
             * @public
             * @memberof CSHTML5.Native.Html.Controls.NativeComboBox
             * @function DisplayMemberPath
             * @type string
             */
            DisplayMemberPath: {
                get: function () {
                    return Bridge.cast(this.GetValue(CSHTML5.Native.Html.Controls.NativeComboBox.DisplayMemberPathProperty), System.String);
                },
                set: function (value) {
                    this.SetValue(CSHTML5.Native.Html.Controls.NativeComboBox.DisplayMemberPathProperty, value);
                }
            },
            /**
             * The index of the first item in the current selection or -1 if the selection is empty.
             *
             * @instance
             * @public
             * @memberof CSHTML5.Native.Html.Controls.NativeComboBox
             * @function SelectedIndex
             * @type number
             */
            SelectedIndex: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(CSHTML5.Native.Html.Controls.NativeComboBox.SelectedIndexProperty), System.Int32), System.Int32));
                },
                set: function (value) {
                    this.SetValue(CSHTML5.Native.Html.Controls.NativeComboBox.SelectedIndexProperty, Bridge.box(value, System.Int32));
                }
            },
            /**
             * Gets or sets the selected item.
             *
             * @instance
             * @public
             * @memberof CSHTML5.Native.Html.Controls.NativeComboBox
             * @function SelectedItem
             * @type System.Object
             */
            SelectedItem: {
                get: function () {
                    return this.GetValue(CSHTML5.Native.Html.Controls.NativeComboBox.SelectedItemProperty);
                },
                set: function (value) {
                    this.SetValue(CSHTML5.Native.Html.Controls.NativeComboBox.SelectedItemProperty, value);
                }
            },
            /**
             * Gets or sets the value of the selected item, obtained by using the SelectedValuePath.
             *
             * @instance
             * @public
             * @memberof CSHTML5.Native.Html.Controls.NativeComboBox
             * @function SelectedValue
             * @type System.Object
             */
            SelectedValue: {
                get: function () {
                    return this.GetValue(CSHTML5.Native.Html.Controls.NativeComboBox.SelectedValueProperty);
                },
                set: function (value) {
                    this.SetValue(CSHTML5.Native.Html.Controls.NativeComboBox.SelectedValueProperty, value);
                }
            },
            /**
             * Gets or sets the property path that is used to get the SelectedValue property
             of the SelectedItem property.
             *
             * @instance
             * @public
             * @memberof CSHTML5.Native.Html.Controls.NativeComboBox
             * @function SelectedValuePath
             * @type string
             */
            SelectedValuePath: {
                get: function () {
                    return Bridge.cast(this.GetValue(CSHTML5.Native.Html.Controls.NativeComboBox.SelectedValuePathProperty), System.String);
                },
                set: function (value) {
                    this.SetValue(CSHTML5.Native.Html.Controls.NativeComboBox.SelectedValuePathProperty, value);
                }
            }
        },
        ctors: {
            init: function () {
                this._selectionChangeIsOnIndex = false;
                this._selectionChangeIsOnItem = false;
                this._selectionChangeIsOnValue = false;
            },
            /**
             * Instantiate a new Native ComboBox.
             *
             * @instance
             * @public
             * @this CSHTML5.Native.Html.Controls.NativeComboBox
             * @memberof CSHTML5.Native.Html.Controls.NativeComboBox
             * @return  {void}
             */
            ctor: function () {
                this.$initialize();
                Windows.UI.Xaml.FrameworkElement.ctor.call(this);
                this.INTERNAL_DoNotApplyStyle = true;
            }
        },
        methods: {
            CreateDomElement: function (parentRef, domElementWhereToPlaceChildren) {
                var select = CSHTML5.Internal.INTERNAL_HtmlDomManager.CreateDomElementAndAppendIt("select", parentRef, this);
                domElementWhereToPlaceChildren.v = select;
                this._nativeComboBoxDomElement = select;

                CSHTML5.Internal.INTERNAL_HtmlDomManager.SetDomElementStyleProperty(select, function (_o1) {
                        _o1.add("fontSize");
                        return _o1;
                    }(new (System.Collections.Generic.List$1(System.String)).ctor()), "inherit");

                CSHTML5.Internal.INTERNAL_HtmlDomManager.SetDomElementProperty(select, "data-absorb-events", Bridge.box(true, System.Boolean, System.Boolean.toString));

                this.Refresh();

                this.SubscribeToHtmlChangeEvent();

                return select;
            },
            SubscribeToHtmlChangeEvent: function () {
                this._changeEventProxy = CSHTML5.Internal.INTERNAL_EventsHelper.AttachToDomEvents("change", this._nativeComboBoxDomElement, Bridge.fn.cacheBind(this, this.DomSelectionChanged));
            },
            UnsubscribeFromHtmlChangeEvent: function () {
                if (this._changeEventProxy == null) {
                    return;
                }

                this._nativeComboBoxDomElement.removeEventListener("change",Bridge.fn.cacheBind(this, this.DomSelectionChanged));
            },
            FindItemWithValue: function (value, index) {
                var $t;
                index.v = -1;

                if (this.Items.Count === 0) {
                    return Windows.UI.Xaml.DependencyProperty.UnsetValue;
                }

                var selectedValuePath = this.SelectedValuePath;

                if (System.String.isNullOrEmpty(selectedValuePath)) {
                    index.v = this.Items.indexOf(value);
                    if (index.v >= 0) {
                        return value;
                    } else {
                        return Windows.UI.Xaml.DependencyProperty.UnsetValue;
                    }
                }

                index.v = 0;
                $t = Bridge.getEnumerator(this.Items);
                try {
                    while ($t.moveNext()) {
                        var item = $t.Current;
                        var displayedItem = Windows.UI.Xaml.PropertyPathHelper.AccessValueByApplyingPropertyPathIfAny(item, selectedValuePath);
                        if (Windows.UI.Xaml.Controls.ItemsControl.EqualsEx(displayedItem, value)) {
                            return item;
                        }
                        index.v = (index.v + 1) | 0;
                    }
                } finally {
                    if (Bridge.is($t, System.IDisposable)) {
                        $t.System$IDisposable$Dispose();
                    }
                }

                index.v = -1;
                return Windows.UI.Xaml.DependencyProperty.UnsetValue;
            },
            Refresh: function () {
                this.ResetOptions();

                this.OnItemsChanged(new System.Collections.Specialized.NotifyCollectionChangedEventArgs.ctor(System.Collections.Specialized.NotifyCollectionChangedAction.Reset));
            },
            OnItemCollectionChanged: function (sender, e) {
                this.ManageCollectionChanged(e);

                this.OnItemsChanged(e);
            },
            ManageCollectionChanged: function (e) {

                if (e.Action === System.Collections.Specialized.NotifyCollectionChangedAction.Reset) {
                    this.ResetOptions();
                } else if (e.Action === System.Collections.Specialized.NotifyCollectionChangedAction.Add) {
                    this.AddOption(System.Array.getItem(e.NewItems, 0), ((e.NewStartingIndex + 1) | 0));
                } else if (e.Action === System.Collections.Specialized.NotifyCollectionChangedAction.Remove) {
                    this.RemoveOption(System.Array.getItem(e.OldItems, 0), ((e.OldStartingIndex + 1) | 0));
                } else if (e.Action === System.Collections.Specialized.NotifyCollectionChangedAction.Replace) {
                    this.RemoveOption(System.Array.getItem(e.OldItems, 0), ((e.OldStartingIndex + 1) | 0));
                    this.AddOption(System.Array.getItem(e.NewItems, 0), ((e.OldStartingIndex + 1) | 0));
                } else {
                    throw new System.NotSupportedException.$ctor1(System.String.format("Unexpected collection change action '{0}'.", [Bridge.box(e.Action, System.Collections.Specialized.NotifyCollectionChangedAction, System.Enum.toStringFn(System.Collections.Specialized.NotifyCollectionChangedAction))]));
                }
            },
            ResetOptions: function () {
                if (this._nativeComboBoxDomElement == null) {
                    return;
                }

                for (var i = 0; i < ((this.Items.Count + 1) | 0); i = (i + 1) | 0) {
                    CSHTML5.Internal.INTERNAL_HtmlDomManager.RemoveOptionFromNativeComboBox(this._nativeComboBoxDomElement, 0);
                }

                this.AddEmptyOption();

                for (var i1 = 0; i1 < this.Items.Count; i1 = (i1 + 1) | 0) {
                    this.AddOption(this.Items.getItem(i1), ((i1 + 1) | 0));
                }

                this.SetSelectedIndexInNativeHtmlDom(this.SelectedIndex);
            },
            AddOption: function (option, index) {
                if (this._nativeComboBoxDomElement == null) {
                    return;
                }

                var value = Windows.UI.Xaml.PropertyPathHelper.AccessValueByApplyingPropertyPathIfAny(option, this.DisplayMemberPath);
                if (value != null) {
                    CSHTML5.Internal.INTERNAL_HtmlDomManager.AddOptionToNativeComboBox$1(this._nativeComboBoxDomElement, Bridge.toString(value), index);
                }
            },
            RemoveOption: function (option, index) {
                if (this._nativeComboBoxDomElement == null) {
                    return;
                }

                CSHTML5.Internal.INTERNAL_HtmlDomManager.RemoveOptionFromNativeComboBox(this._nativeComboBoxDomElement, index);
            },
            AddEmptyOption: function () {
                var emptyOption = CSHTML5.Internal.INTERNAL_HtmlDomManager.AddOptionToNativeComboBox$1(this._nativeComboBoxDomElement, "", 0);
                emptyOption.disabled = true;
                emptyOption.selected = true;
                emptyOption.style.display = 'hidden';
            },
            OnItemsChanged: function (e) {
                var selectedIndex = this.SelectedIndex;

                switch (e.Action) {
                    case System.Collections.Specialized.NotifyCollectionChangedAction.Reset: 
                        if (!this.Items.contains(this.SelectedItem)) {
                            selectedIndex = -1;
                        }
                        break;
                    case System.Collections.Specialized.NotifyCollectionChangedAction.Add: 
                        if (e.NewStartingIndex <= this.SelectedIndex) {
                            selectedIndex = (selectedIndex + 1) | 0;
                        }
                        break;
                    case System.Collections.Specialized.NotifyCollectionChangedAction.Remove: 
                        if (Bridge.referenceEquals(System.Array.getItem(e.OldItems, 0), this.SelectedItem)) {
                            selectedIndex = -1;
                        } else if (e.OldStartingIndex <= this.SelectedIndex) {
                            selectedIndex = (selectedIndex - 1) | 0;
                        }
                        break;
                    case System.Collections.Specialized.NotifyCollectionChangedAction.Replace: 
                        if (Bridge.referenceEquals(System.Array.getItem(e.OldItems, 0), this.SelectedItem)) {
                            selectedIndex = -1;
                        }
                        break;
                    default: 
                        throw new System.NotSupportedException.$ctor1(System.String.format("Unexpected collection change action '{0}'.", [Bridge.box(e.Action, System.Collections.Specialized.NotifyCollectionChangedAction, System.Enum.toStringFn(System.Collections.Specialized.NotifyCollectionChangedAction))]));
                }

                if (selectedIndex !== this.SelectedIndex) {
                    this.SetCurrentValue(CSHTML5.Native.Html.Controls.NativeComboBox.SelectedIndexProperty, Bridge.box(selectedIndex, System.Int32));
                }
            },
            OnSelectionChanged: function (oldItem, newItem) {
                if (!Bridge.staticEquals(this.SelectionChanged, null)) {
                    var oldItems = new (System.Collections.Generic.List$1(System.Object)).ctor();
                    oldItems.add(oldItem);
                    var newItems = new (System.Collections.Generic.List$1(System.Object)).ctor();
                    newItems.add(newItem);

                    var args = new Windows.UI.Xaml.Controls.SelectionChangedEventArgs(oldItems, newItems);
                    args.OriginalSource = this;

                    this.SelectionChanged(this, args);
                }
            },
            DomSelectionChanged: function (element) {
                if (this._changingSelectionProgrammatically) {
                    return;
                }

                var selectedIndex = this.GetSelectedIndexInNativeHtmlDom();

                this._changingSelectionInHtml = true;

                try {
                    this.SetCurrentValue(CSHTML5.Native.Html.Controls.NativeComboBox.SelectedIndexProperty, Bridge.box(selectedIndex, System.Int32));
                } finally {
                    this._changingSelectionInHtml = false;
                }
            },
            SetSelectedIndexInNativeHtmlDom: function (value) {
                if (this._nativeComboBoxDomElement != null) {
                    CSHTML5.Internal.INTERNAL_HtmlDomManager.SetDomElementProperty(this._nativeComboBoxDomElement, "selectedIndex", Bridge.box(value >= 0 ? ((value + 1) | 0) : value, System.Int32), true);
                }
            },
            GetSelectedIndexInNativeHtmlDom: function () {
                if (this._nativeComboBoxDomElement != null) {
                    var selectedIndexInHtmlDom = System.Convert.toInt32(CSHTML5.Internal.INTERNAL_HtmlDomManager.GetDomElementAttribute(this._nativeComboBoxDomElement, "selectedIndex"));

                    if (selectedIndexInHtmlDom >= 0) {
                        selectedIndexInHtmlDom = (selectedIndexInHtmlDom - 1) | 0;
                    }

                    return selectedIndexInHtmlDom;
                }

                return -1;
            }
        }
    });

    /**
     * Represents a collection of behaviors with a shared AssociatedObject and provides
     change notifications to its contents when that AssociatedObject changes.
     *
     * @public
     * @class System.Windows.Interactivity.BehaviorCollection
     * @augments System.Windows.Interactivity.AttachableCollection$1
     */
    Bridge.define("System.Windows.Interactivity.BehaviorCollection", {
        inherits: [System.Windows.Interactivity.AttachableCollection$1(System.Windows.Interactivity.Behavior)],
        methods: {
            /**
             * Called immediately after the collection is attached to an AssociatedObject.
             *
             * @instance
             * @protected
             * @override
             * @this System.Windows.Interactivity.BehaviorCollection
             * @memberof System.Windows.Interactivity.BehaviorCollection
             * @return  {void}
             */
            OnAttached: function () { },
            /**
             * Called when the collection is being detached from its AssociatedObject, but
             before it has actually occurred.
             *
             * @instance
             * @protected
             * @override
             * @this System.Windows.Interactivity.BehaviorCollection
             * @memberof System.Windows.Interactivity.BehaviorCollection
             * @return  {void}
             */
            OnDetaching: function () { }
        }
    });

    /**
     * Represents a collection of actions with a shared AssociatedObject and provides change notifications to its contents when that AssociatedObject changes.
     *
     * @public
     * @class System.Windows.Interactivity.TriggerActionCollection
     * @augments System.Windows.Interactivity.AttachableCollection$1
     */
    Bridge.define("System.Windows.Interactivity.TriggerActionCollection", {
        inherits: [System.Windows.Interactivity.AttachableCollection$1(System.Windows.Interactivity.TriggerAction)],
        ctors: {
            /**
             * Initializes a new instance of the {@link } class.
             *
             * @instance
             * @this System.Windows.Interactivity.TriggerActionCollection
             * @memberof System.Windows.Interactivity.TriggerActionCollection
             * @return  {void}
             */
            ctor: function () {
                this.$initialize();
                System.Windows.Interactivity.AttachableCollection$1(System.Windows.Interactivity.TriggerAction).ctor.call(this);
            }
        },
        methods: {
            /**
             * Called immediately after the collection is attached to an AssociatedObject.
             *
             * @instance
             * @protected
             * @override
             * @this System.Windows.Interactivity.TriggerActionCollection
             * @memberof System.Windows.Interactivity.TriggerActionCollection
             * @return  {void}
             */
            OnAttached: function () {
                var $t;
                $t = Bridge.getEnumerator(this);
                try {
                    while ($t.moveNext()) {
                        var triggerAction = $t.Current;
                        triggerAction.Attach$1(Bridge.cast(this.AssociatedObject, Windows.UI.Xaml.FrameworkElement));
                    }
                } finally {
                    if (Bridge.is($t, System.IDisposable)) {
                        $t.System$IDisposable$Dispose();
                    }
                }
            },
            /**
             * Called when the collection is being detached from its AssociatedObject, but before it has actually occurred.
             *
             * @instance
             * @protected
             * @override
             * @this System.Windows.Interactivity.TriggerActionCollection
             * @memberof System.Windows.Interactivity.TriggerActionCollection
             * @return  {void}
             */
            OnDetaching: function () {
                var $t;
                $t = Bridge.getEnumerator(this);
                try {
                    while ($t.moveNext()) {
                        var triggerAction = $t.Current;
                        triggerAction.Detach();
                    }
                } finally {
                    if (Bridge.is($t, System.IDisposable)) {
                        $t.System$IDisposable$Dispose();
                    }
                }
            },
            /**
             * Called when a new item is added to the collection.
             *
             * @instance
             * @this System.Windows.Interactivity.TriggerActionCollection
             * @memberof System.Windows.Interactivity.TriggerActionCollection
             * @param   {System.Windows.Interactivity.TriggerAction}    item    The new item.
             * @return  {void}
             */
            ItemAdded: function (item) {
                if (item.IsHosted) {
                    throw new System.InvalidOperationException.$ctor1("Cannot Host TriggerAction Multiple Times");
                }
                if (this.AssociatedObject != null) {
                    item.Attach$1(Bridge.cast(this.AssociatedObject, Windows.UI.Xaml.FrameworkElement));
                }
                item.IsHosted = true;
            },
            /**
             * Called when an item is removed from the collection.
             *
             * @instance
             * @this System.Windows.Interactivity.TriggerActionCollection
             * @memberof System.Windows.Interactivity.TriggerActionCollection
             * @param   {System.Windows.Interactivity.TriggerAction}    item    The removed item.
             * @return  {void}
             */
            ItemRemoved: function (item) {
                if (Bridge.cast(item, System.Windows.Interactivity.IAttachedObject).System$Windows$Interactivity$IAttachedObject$AssociatedObject != null) {
                    item.Detach();
                }
                item.IsHosted = false;
            }
        }
    });

    /**
     * Represents a collection of triggers with a shared AssociatedObject and provides change notifications to its contents when that AssociatedObject changes.
     *
     * @public
     * @class System.Windows.Interactivity.TriggerCollection
     * @augments System.Windows.Interactivity.AttachableCollection$1
     */
    Bridge.define("System.Windows.Interactivity.TriggerCollection", {
        inherits: [System.Windows.Interactivity.AttachableCollection$1(System.Windows.Interactivity.TriggerBase)],
        ctors: {
            /**
             * Initializes a new instance of the {@link } class.
             *
             * @instance
             * @this System.Windows.Interactivity.TriggerCollection
             * @memberof System.Windows.Interactivity.TriggerCollection
             * @return  {void}
             */
            ctor: function () {
                this.$initialize();
                System.Windows.Interactivity.AttachableCollection$1(System.Windows.Interactivity.TriggerBase).ctor.call(this);
            }
        },
        methods: {
            /**
             * Called immediately after the collection is attached to an AssociatedObject.
             *
             * @instance
             * @protected
             * @override
             * @this System.Windows.Interactivity.TriggerCollection
             * @memberof System.Windows.Interactivity.TriggerCollection
             * @return  {void}
             */
            OnAttached: function () { },
            /**
             * Called when the collection is being detached from its AssociatedObject, but before it has actually occurred.
             *
             * @instance
             * @protected
             * @override
             * @this System.Windows.Interactivity.TriggerCollection
             * @memberof System.Windows.Interactivity.TriggerCollection
             * @return  {void}
             */
            OnDetaching: function () { }
        }
    });

    /**
     * Represents an XML document.
     * // Initializes a new instance of the System.Xml.Linq.XDocument class with the
    // specified content.
    //
     * // Initializes a new instance of the System.Xml.Linq.XDocument class from an
    // existing System.Xml.Linq.XDocument object.
    //
     * // Initializes a new instance of the System.Xml.Linq.XDocument class with the
    // specified System.Xml.Linq.XDeclaration and content.
    //
     * // Gets or sets the XML declaration for this document.
    //
     * // Gets the Document Type Definition (DTD) for this document.
    //
     * // Gets the node type for this node.
    //
     * // Creates a new System.Xml.Linq.XDocument instance using the specified stream.
    //
     * // Creates a new System.Xml.Linq.XDocument from a file located in the application's
    // XAP package.
    //
     * // Creates a new System.Xml.Linq.XDocument from a System.IO.TextReader.
    //
     * // Creates a new System.Xml.Linq.XDocument from an System.Xml.XmlReader.
    //
     * // Creates a new System.Xml.Linq.XDocument instance using the specified stream,
    // optionally preserving white space, setting the base URI, and retaining line
    // information.
    //
     * // Creates a new System.Xml.Linq.XDocument from a file located in the application's
    // XAP package, optionally preserving white space, setting the base URI, and
    // retaining line information.
    //
     * // Creates a new System.Xml.Linq.XDocument from a System.IO.TextReader, optionally
    // preserving white space, setting the base URI, and retaining line information.
    //
     * // Creates a new System.Xml.Linq.XDocument from an System.Xml.XmlReader, optionally
    // setting the base URI, and retaining line information.
    //
     * // Creates a new System.Xml.Linq.XDocument from a string, optionally preserving
    // white space, setting the base URI, and retaining line information.
    //
     * // Outputs this System.Xml.Linq.XDocument to the specified System.IO.Stream.
    //
     * // Serialize this System.Xml.Linq.XDocument to a System.IO.TextWriter.
    //
     * // Serialize this System.Xml.Linq.XDocument to an System.Xml.XmlWriter.
    //
     * // Outputs this System.Xml.Linq.XDocument to the specified System.IO.Stream,
    // optionally specifying formatting behavior.
    //
     * // Serialize this System.Xml.Linq.XDocument to a System.IO.TextWriter, optionally
    // disabling formatting.
    //
     * // Write this document to an System.Xml.XmlWriter.
    //
     *
     * @public
     * @class System.Xml.Linq.XDocument
     * @augments System.Xml.Linq.XContainer
     * @return  {[type]}        An System.Xml.Linq.XDocument populated from the string that contains XML.
     */
    Bridge.define("System.Xml.Linq.XDocument", {
        inherits: [System.Xml.Linq.XContainer],
        statics: {
            fields: {
                INTERNAL_idsToXNodes: null
            },
            ctors: {
                ctor: function () {
                    if (CSHTML5.Interop.IsRunningInTheSimulator) {
                        System.Xml.Linq.XDocument.INTERNAL_idsToXNodes = new (System.Collections.Generic.Dictionary$2(System.String,System.Xml.Linq.XNode)).ctor();
                    }
                }
            },
            methods: {
                /**
                 * Creates a new System.Xml.Linq.XDocument from a string.
                 *
                 * @static
                 * @public
                 * @this System.Xml.Linq.XDocument
                 * @memberof System.Xml.Linq.XDocument
                 * @param   {string}                       text    A string that contains XML.
                 * @return  {System.Xml.Linq.XDocument}            An System.Xml.Linq.XDocument populated from the string that contains XML.
                 */
                Parse: function (text) {
                    var newDocument = new System.Xml.Linq.XDocument();
                    newDocument._text = text;
                    var parser = new DOMParser();
                    newDocument.INTERNAL_jsnode = 
parser.parseFromString(text, "application/xml")
;

                    return newDocument;
                },
                GetXNodeFromJSNode: function (node) {
                    var result = null;

                    if (CSHTML5.Interop.IsRunningInTheSimulator) {


                        var newId = System.Guid.NewGuid().toString();
                        var nodeType = System.Convert.toInt32(node.nodeType);


                        switch (nodeType) {
                            case 1: 
                                result = new System.Xml.Linq.XElement.ctor(node);
                                break;
                            case 3: 
                                result = new System.Xml.Linq.XText.ctor(node);
                                result.INTERNAL_jsnode = node;
                                break;
                            case 8: 
                                break;
                            case 9: 
                                break;
                            default: 
                                break;
                        }
                    } else {

                        var nodeType1 = System.Convert.toInt32(node.nodeType);


                        switch (nodeType1) {
                            case 1: 
                                result = new System.Xml.Linq.XElement.ctor(node);
                                break;
                            case 3: 
                                result = new System.Xml.Linq.XText.ctor(node);
                                result.INTERNAL_jsnode = node;
                                break;
                            case 8: 
                                break;
                            case 9: 
                                break;
                            default: 
                                break;
                        }
                    }
                    return result;
                },
                IsNullOrUndefined: function (jsObject) {
                    if (CSHTML5.Interop.IsRunningInTheSimulator) {
                        return (typeof(jsObject) === 'undefined') || (jsObject === null);
                    } else {
                        return System.Convert.toBoolean((typeof jsObject === 'undefined' || jsObject === null));
                    }
                }
            }
        },
        fields: {
            _text: null,
            INTERNAL_jsDocument: null
        },
        props: {
            /**
             * Gets the root element of the XML Tree for this document.
             *
             * @instance
             * @public
             * @memberof System.Xml.Linq.XDocument
             * @function Root
             * @type System.Xml.Linq.XElement
             */
            Root: {
                get: function () {
                    var $t;
                    $t = Bridge.getEnumerator(this.Nodes(), System.Xml.Linq.XNode);
                    try {
                        while ($t.moveNext()) {
                            var xNode = $t.Current;
                            if (Bridge.is(xNode, System.Xml.Linq.XElement)) {
                                return Bridge.cast(xNode, System.Xml.Linq.XElement);
                            }
                        }
                    } finally {
                        if (Bridge.is($t, System.IDisposable)) {
                            $t.System$IDisposable$Dispose();
                        }
                    }
                    throw new System.Exception("Root not found");
                },
                set: function (value) {
                    this.INTERNAL_jsDocument.documentElement.appendChild(value.INTERNAL_jsnode);
                }
            }
        },
        ctors: {
            /**
             * Initializes a new instance of the System.Xml.Linq.XDocument class.
             *
             * @instance
             * @public
             * @this System.Xml.Linq.XDocument
             * @memberof System.Xml.Linq.XDocument
             * @return  {void}
             */
            ctor: function () {
                this.$initialize();
                System.Xml.Linq.XContainer.ctor.call(this);
                this.INTERNAL_jsDocument = document.implementation.createDocument(null, "root", null);
                this.INTERNAL_jsnode = this.INTERNAL_jsDocument.documentElement;

            }
        },
        methods: {
            toString: function () {
                return this.Root.toString();
            },
            ToString: function (indentXml) {
                return this.Root.ToString(indentXml);
            },
            /**
             * Creates an {@link } used to add either nodes 
             or attributes to the {@link }. The later option
             applies only for {@link }.
             *
             * @instance
             * @public
             * @this System.Xml.Linq.XDocument
             * @memberof System.Xml.Linq.XDocument
             * @return  {System.Xml.XmlWriter}        An {@link }
             */
            CreateWriter: function () {
                return new System.Xml.Cshtml5_XmlWriter.$ctor1(this);
            },
            CreateReader: function () {
                return new System.Xml.Cshtml5_XmlReader(this, this._text);
            }
        }
    });

    
    Bridge.define("System.Xml.Linq.XElement", {
        inherits: [System.Xml.Linq.XContainer],
        statics: {
            methods: {
                GetAttributeFromJSAttribute: function (jsAttribute) {
                    var ns;
                    var nameWithPrefix;
                    var name;
                    var value;
                    var concatenated = System.Convert.toString(jsAttribute.namespaceURI + '|' + jsAttribute.name + '|' + jsAttribute.localName + '|' + jsAttribute.value);

                    var attributeDatas = System.String.split(concatenated, [124].map(function (i) {{ return String.fromCharCode(i); }}));
                    if (attributeDatas.length === 4) {
                        ns = System.Xml.Linq.XNamespace.Get(attributeDatas[System.Array.index(0, attributeDatas)]);
                        nameWithPrefix = ns.GetName(attributeDatas[System.Array.index(1, attributeDatas)]);
                        name = ns.GetName(attributeDatas[System.Array.index(2, attributeDatas)]);
                        value = attributeDatas[System.Array.index(3, attributeDatas)];
                    } else {
                        ns = System.Xml.Linq.XNamespace.Get(System.Convert.toString(jsAttribute.namespaceURI));
                        nameWithPrefix = ns.GetName(System.Convert.toString(jsAttribute.name));
                        name = ns.GetName(System.Convert.toString(jsAttribute.localName));
                        value = System.Convert.toString(jsAttribute.value);
                    }

                    return new System.Xml.Linq.XAttribute.$ctor2(name, value, nameWithPrefix);
                }/**
                 * Cast the value of this System.Xml.Linq.XElement to a System.Guid.
                 *
                 * @static
                 * @public
                 * @this System.Xml.Linq.XElement
                 * @memberof System.Xml.Linq.XElement
                 * @param   {System.Xml.Linq.XElement}    element    The System.Xml.Linq.XElement to cast to System.Guid.
                 * @return  {System.Guid}                            A System.Guid that contains the content of this System.Xml.Linq.XElement.
                 */
                ,
                op_Explicit$3: function (element) {
                    if (element == null) {
                        throw new System.ArgumentNullException.$ctor1("The parameter cannot be null");
                    }
                    var content = System.Linq.Enumerable.from(element.Nodes()).first();
                    if (Bridge.is(content, System.Xml.Linq.XText)) {
                        var guidAsString = Bridge.cast(content, System.Xml.Linq.XText).Value;
                        return System.Guid.Parse(guidAsString);
                    }
                    throw new System.FormatException.$ctor1("Could not convert element to Guid.");
                }/**
                 * Cast the value of this System.Xml.Linq.XElement to a System.Double.
                 *
                 * @static
                 * @public
                 * @this System.Xml.Linq.XElement
                 * @memberof System.Xml.Linq.XElement
                 * @param   {System.Xml.Linq.XElement}    element    The System.Xml.Linq.XElement to cast to System.Double.
                 * @return  {number}                                 A System.Double that contains the content of this System.Xml.Linq.XElement.
                 */
                ,
                op_Explicit$2: function (element) {
                    if (element == null) {
                        throw new System.ArgumentNullException.$ctor1("The parameter cannot be null");
                    }
                    var content = System.Linq.Enumerable.from(element.Nodes()).first();
                    if (Bridge.is(content, System.Xml.Linq.XText)) {
                        var doubleAsString = Bridge.cast(content, System.Xml.Linq.XText).Value;
                        return System.Double.parse(doubleAsString);
                    }
                    throw new System.FormatException.$ctor1("Could not convert element to Double.");
                }/**
                 * Cast the value of this System.Xml.Linq.XElement to a System.DateTime.
                 *
                 * @static
                 * @public
                 * @this System.Xml.Linq.XElement
                 * @memberof System.Xml.Linq.XElement
                 * @param   {System.Xml.Linq.XElement}    element    The System.Xml.Linq.XElement to cast to System.DateTime.
                 * @return  {System.DateTime}                        A System.DateTime that contains the content of this System.Xml.Linq.XElement.
                 */
                ,
                op_Explicit$1: function (element) {
                    if (element == null) {
                        throw new System.ArgumentNullException.$ctor1("The parameter cannot be null");
                    }
                    var content = System.Linq.Enumerable.from(element.Nodes()).first();
                    if (Bridge.is(content, System.Xml.Linq.XText)) {
                        var dateTimeAsString = Bridge.cast(content, System.Xml.Linq.XText).Value;
                        return System.DateTime.parse(dateTimeAsString);
                    }
                    throw new System.FormatException.$ctor1("Could not convert element to DateTime.");
                }/**
                 * // Cast the value of this System.Xml.Linq.XElement to a System.Decimal.
                //
                 * // Cast the value of this System.Xml.Linq.XAttribute to a System.DateTimeOffset.
                //
                 * Cast the value of this System.Xml.Linq.XElement to an System.Int64.
                 *
                 * @static
                 * @public
                 * @this System.Xml.Linq.XElement
                 * @memberof System.Xml.Linq.XElement
                 * @param   {System.Xml.Linq.XElement}    element    The System.Xml.Linq.XElement to cast to System.Int64.
                 * @return  {System.Int64}                           A System.Int64 that contains the content of this System.Xml.Linq.XElement.
                 */
                ,
                op_Explicit$5: function (element) {
                    if (element == null) {
                        throw new System.ArgumentNullException.$ctor1("The parameter cannot be null");
                    }
                    var content = System.Linq.Enumerable.from(element.Nodes()).first();
                    if (Bridge.is(content, System.Xml.Linq.XText)) {
                        var longAsString = Bridge.cast(content, System.Xml.Linq.XText).Value;
                        return System.Int64.parse(longAsString);
                    }
                    throw new System.FormatException.$ctor1("Could not convert element to long.");
                }/**
                 * Cast the value of this System.Xml.Linq.XElement to an System.Int32.
                 *
                 * @static
                 * @public
                 * @this System.Xml.Linq.XElement
                 * @memberof System.Xml.Linq.XElement
                 * @param   {System.Xml.Linq.XElement}    element    The System.Xml.Linq.XElement to cast to System.Int32.
                 * @return  {number}                                 A System.Int32 that contains the content of this System.Xml.Linq.XElement.
                 */
                ,
                op_Explicit$4: function (element) {
                    if (element == null) {
                        throw new System.ArgumentNullException.$ctor1("The parameter cannot be null");
                    }
                    var content = System.Linq.Enumerable.from(element.Nodes()).first();
                    if (Bridge.is(content, System.Xml.Linq.XText)) {
                        var intAsString = Bridge.cast(content, System.Xml.Linq.XText).Value;
                        return System.Int32.parse(intAsString);
                    }
                    throw new System.FormatException.$ctor1("Could not convert element to int.");
                }/**
                 * Cast the value of this System.Xml.Linq.XElement to a System.Boolean.
                 *
                 * @static
                 * @public
                 * @this System.Xml.Linq.XElement
                 * @memberof System.Xml.Linq.XElement
                 * @param   {System.Xml.Linq.XElement}    element    The System.Xml.Linq.XElement to cast to System.Boolean.
                 * @return  {boolean}                                A System.Boolean that contains the content of this System.Xml.Linq.XElement.
                 */
                ,
                op_Explicit: function (element) {
                    if (element == null) {
                        throw new System.ArgumentNullException.$ctor1("The parameter cannot be null");
                    }
                    var content = System.Linq.Enumerable.from(element.Nodes()).first();
                    if (Bridge.is(content, System.Xml.Linq.XText)) {
                        var boolAsString = Bridge.cast(content, System.Xml.Linq.XText).Value;
                        return System.Boolean.parse(boolAsString);
                    }
                    throw new System.FormatException.$ctor1("Could not convert element to bool.");
                }/**
                 * Cast the value of this System.Xml.Linq.XElement to a System.UInt64.
                 *
                 * @static
                 * @public
                 * @this System.Xml.Linq.XElement
                 * @memberof System.Xml.Linq.XElement
                 * @param   {System.Xml.Linq.XElement}    element    The System.Xml.Linq.XElement to cast to System.UInt64.
                 * @return  {System.UInt64}                          A System.UInt64 that contains the content of this System.Xml.Linq.XElement.
                 */
                ,
                op_Explicit$20: function (element) {
                    if (element == null) {
                        throw new System.ArgumentNullException.$ctor1("The parameter cannot be null");
                    }
                    var content = System.Linq.Enumerable.from(element.Nodes()).first();
                    if (Bridge.is(content, System.Xml.Linq.XText)) {
                        var ulongAsString = Bridge.cast(content, System.Xml.Linq.XText).Value;
                        return System.UInt64.parse(ulongAsString);
                    }
                    throw new System.FormatException.$ctor1("Could not convert element to UInt64.");
                }/**
                 * Cast the value of this System.Xml.Linq.XElement to a System.UInt32.
                 *
                 * @static
                 * @public
                 * @this System.Xml.Linq.XElement
                 * @memberof System.Xml.Linq.XElement
                 * @param   {System.Xml.Linq.XElement}    element    The System.Xml.Linq.XElement to cast to System.UInt32.
                 * @return  {number}                                 A System.UInt32 that contains the content of this System.Xml.Linq.XElement.
                 */
                ,
                op_Explicit$19: function (element) {
                    if (element == null) {
                        throw new System.ArgumentNullException.$ctor1("The parameter cannot be null");
                    }
                    var content = System.Linq.Enumerable.from(element.Nodes()).first();
                    if (Bridge.is(content, System.Xml.Linq.XText)) {
                        var uintAsString = Bridge.cast(content, System.Xml.Linq.XText).Value;
                        return System.UInt32.parse(uintAsString);
                    }
                    throw new System.FormatException.$ctor1("Could not convert element to UInt32.");
                }/**
                 * Cast the value of this System.Xml.Linq.XElement to a System.TimeSpan.
                 *
                 * @static
                 * @public
                 * @this System.Xml.Linq.XElement
                 * @memberof System.Xml.Linq.XElement
                 * @param   {System.Xml.Linq.XElement}    element    The System.Xml.Linq.XElement to cast to System.TimeSpan.
                 * @return  {System.TimeSpan}                        A System.TimeSpan that contains the content of this System.Xml.Linq.XElement.
                 */
                ,
                op_Explicit$18: function (element) {
                    if (element == null) {
                        throw new System.ArgumentNullException.$ctor1("The parameter cannot be null");
                    }
                    var content = System.Linq.Enumerable.from(element.Nodes()).first();
                    if (Bridge.is(content, System.Xml.Linq.XText)) {
                        var timeSpanAsString = Bridge.cast(content, System.Xml.Linq.XText).Value;
                        return INTERNAL_BridgeWorkarounds.TimeSpanParse(timeSpanAsString);
                    }
                    throw new System.FormatException.$ctor1("Could not convert element to TimeSpan.");
                }/**
                 * Cast the value of this System.Xml.Linq.XElement to a System.String.
                 *
                 * @static
                 * @public
                 * @this System.Xml.Linq.XElement
                 * @memberof System.Xml.Linq.XElement
                 * @param   {System.Xml.Linq.XElement}    element    The System.Xml.Linq.XElement to cast to System.String.
                 * @return  {string}                                 A System.String that contains the content of this System.Xml.Linq.XElement.
                 */
                ,
                op_Explicit$17: function (element) {
                    if (element == null) {
                        throw new System.ArgumentNullException.$ctor1("The parameter cannot be null");
                    }
                    var content = System.Linq.Enumerable.from(element.Nodes()).first();
                    if (Bridge.is(content, System.Xml.Linq.XText)) {
                        return Bridge.cast(content, System.Xml.Linq.XText).Value;
                    }
                    throw new System.FormatException.$ctor1("Could not convert element to UInt32.");
                }/**
                 * Cast the value of this System.Xml.Linq.XElement to a System.Single.
                 *
                 * @static
                 * @public
                 * @this System.Xml.Linq.XElement
                 * @memberof System.Xml.Linq.XElement
                 * @param   {System.Xml.Linq.XElement}    element    The System.Xml.Linq.XElement to cast to System.Single.
                 * @return  {number}                                 A System.Single that contains the content of this System.Xml.Linq.XElement.
                 */
                ,
                op_Explicit$16: function (element) {
                    if (element == null) {
                        throw new System.ArgumentNullException.$ctor1("The parameter cannot be null");
                    }
                    var content = System.Linq.Enumerable.from(element.Nodes()).first();
                    if (Bridge.is(content, System.Xml.Linq.XText)) {
                        var floatAsString = Bridge.cast(content, System.Xml.Linq.XText).Value;
                        return System.Single.parse(floatAsString);
                    }
                    throw new System.FormatException.$ctor1("Could not convert element to float.");
                },
                op_Explicit$6: function (element) {
                    var $t;
                    if (element == null) {
                        throw new System.ArgumentNullException.$ctor1("The parameter cannot be null");
                    }
                    var content = null;
                    $t = Bridge.getEnumerator(element.Nodes(), System.Xml.Linq.XNode);
                    try {
                        while ($t.moveNext()) {
                            var node = $t.Current;
                            content = node;
                            break;
                        }
                    } finally {
                        if (Bridge.is($t, System.IDisposable)) {
                            $t.System$IDisposable$Dispose();
                        }
                    }
                    if (content == null) {
                        return null;
                    }
                    if (Bridge.is(content, System.Xml.Linq.XText)) {
                        var boolAsString = Bridge.cast(content, System.Xml.Linq.XText).Value;
                        if (System.String.isNullOrWhiteSpace(boolAsString)) {
                            return null;
                        }
                        return System.Boolean.parse(boolAsString);
                    }
                    throw new System.FormatException.$ctor1("Could not convert element to bool.");
                },
                op_Explicit$7: function (element) {
                    var $t;
                    if (element == null) {
                        throw new System.ArgumentNullException.$ctor1("The parameter cannot be null");
                    }
                    var content = null;
                    $t = Bridge.getEnumerator(element.Nodes(), System.Xml.Linq.XNode);
                    try {
                        while ($t.moveNext()) {
                            var node = $t.Current;
                            content = node;
                            break;
                        }
                    } finally {
                        if (Bridge.is($t, System.IDisposable)) {
                            $t.System$IDisposable$Dispose();
                        }
                    }
                    if (content == null) {
                        return null;
                    }
                    if (Bridge.is(content, System.Xml.Linq.XText)) {
                        var dateTimeAsString = Bridge.cast(content, System.Xml.Linq.XText).Value;
                        if (System.String.isNullOrWhiteSpace(dateTimeAsString)) {
                            return null;
                        }
                        return System.DateTime.parse(dateTimeAsString);
                    }
                    throw new System.FormatException.$ctor1("Could not convert element to Datetime.");
                },
                op_Explicit$8: function (element) {
                    var $t;
                    if (element == null) {
                        throw new System.ArgumentNullException.$ctor1("The parameter cannot be null");
                    }
                    var content = null;
                    $t = Bridge.getEnumerator(element.Nodes(), System.Xml.Linq.XNode);
                    try {
                        while ($t.moveNext()) {
                            var node = $t.Current;
                            content = node;
                            break;
                        }
                    } finally {
                        if (Bridge.is($t, System.IDisposable)) {
                            $t.System$IDisposable$Dispose();
                        }
                    }
                    if (content == null) {
                        return null;
                    }
                    if (Bridge.is(content, System.Xml.Linq.XText)) {
                        var doubleAsString = Bridge.cast(content, System.Xml.Linq.XText).Value;
                        if (System.String.isNullOrWhiteSpace(doubleAsString)) {
                            return null;
                        }
                        return System.Double.parse(doubleAsString);
                    }
                    throw new System.FormatException.$ctor1("Could not convert element to double.");
                },
                op_Explicit$9: function (element) {
                    var $t;
                    if (element == null) {
                        throw new System.ArgumentNullException.$ctor1("The parameter cannot be null");
                    }
                    var content = null;
                    $t = Bridge.getEnumerator(element.Nodes(), System.Xml.Linq.XNode);
                    try {
                        while ($t.moveNext()) {
                            var node = $t.Current;
                            content = node;
                            break;
                        }
                    } finally {
                        if (Bridge.is($t, System.IDisposable)) {
                            $t.System$IDisposable$Dispose();
                        }
                    }
                    if (content == null) {
                        return null;
                    }
                    if (Bridge.is(content, System.Xml.Linq.XText)) {
                        var guidAsString = Bridge.cast(content, System.Xml.Linq.XText).Value;
                        if (System.String.isNullOrWhiteSpace(guidAsString)) {
                            return null;
                        }
                        return System.Guid.Parse(guidAsString);
                    }
                    throw new System.FormatException.$ctor1("Could not convert element to Guid.");
                },
                op_Explicit$10: function (element) {
                    var $t;
                    if (element == null) {
                        throw new System.ArgumentNullException.$ctor1("The parameter cannot be null");
                    }
                    var content = null;
                    $t = Bridge.getEnumerator(element.Nodes(), System.Xml.Linq.XNode);
                    try {
                        while ($t.moveNext()) {
                            var node = $t.Current;
                            content = node;
                            break;
                        }
                    } finally {
                        if (Bridge.is($t, System.IDisposable)) {
                            $t.System$IDisposable$Dispose();
                        }
                    }
                    if (content == null) {
                        return null;
                    }
                    if (Bridge.is(content, System.Xml.Linq.XText)) {
                        var intAsString = Bridge.cast(content, System.Xml.Linq.XText).Value;
                        if (System.String.isNullOrWhiteSpace(intAsString)) {
                            return null;
                        }
                        return System.Int32.parse(intAsString);
                    }
                    throw new System.FormatException.$ctor1("Could not convert element to int.");
                },
                op_Explicit$11: function (element) {
                    var $t;
                    if (element == null) {
                        throw new System.ArgumentNullException.$ctor1("The parameter cannot be null");
                    }
                    var content = null;
                    $t = Bridge.getEnumerator(element.Nodes(), System.Xml.Linq.XNode);
                    try {
                        while ($t.moveNext()) {
                            var node = $t.Current;
                            content = node;
                            break;
                        }
                    } finally {
                        if (Bridge.is($t, System.IDisposable)) {
                            $t.System$IDisposable$Dispose();
                        }
                    }
                    if (content == null) {
                        return System.Int64.lift(null);
                    }
                    if (Bridge.is(content, System.Xml.Linq.XText)) {
                        var longAsString = Bridge.cast(content, System.Xml.Linq.XText).Value;
                        if (System.String.isNullOrWhiteSpace(longAsString)) {
                            return System.Int64.lift(null);
                        }
                        return System.Int64.parse(longAsString);
                    }
                    throw new System.FormatException.$ctor1("Could not convert element to long.");
                },
                op_Explicit$12: function (element) {
                    var $t;
                    if (element == null) {
                        throw new System.ArgumentNullException.$ctor1("The parameter cannot be null");
                    }
                    var content = null;
                    $t = Bridge.getEnumerator(element.Nodes(), System.Xml.Linq.XNode);
                    try {
                        while ($t.moveNext()) {
                            var node = $t.Current;
                            content = node;
                            break;
                        }
                    } finally {
                        if (Bridge.is($t, System.IDisposable)) {
                            $t.System$IDisposable$Dispose();
                        }
                    }
                    if (content == null) {
                        return null;
                    }
                    if (Bridge.is(content, System.Xml.Linq.XText)) {
                        var floatAsString = Bridge.cast(content, System.Xml.Linq.XText).Value;
                        if (System.String.isNullOrWhiteSpace(floatAsString)) {
                            return null;
                        }
                        return System.Single.parse(floatAsString);
                    }
                    throw new System.FormatException.$ctor1("Could not convert element to float.");
                },
                op_Explicit$13: function (element) {
                    var $t;
                    if (element == null) {
                        throw new System.ArgumentNullException.$ctor1("The parameter cannot be null");
                    }
                    var content = null;
                    $t = Bridge.getEnumerator(element.Nodes(), System.Xml.Linq.XNode);
                    try {
                        while ($t.moveNext()) {
                            var node = $t.Current;
                            content = node;
                            break;
                        }
                    } finally {
                        if (Bridge.is($t, System.IDisposable)) {
                            $t.System$IDisposable$Dispose();
                        }
                    }
                    if (content == null) {
                        return null;
                    }
                    if (Bridge.is(content, System.Xml.Linq.XText)) {
                        var timeSpanAsString = Bridge.cast(content, System.Xml.Linq.XText).Value;
                        if (System.String.isNullOrWhiteSpace(timeSpanAsString)) {
                            return null;
                        }
                        return INTERNAL_BridgeWorkarounds.TimeSpanParse(timeSpanAsString);

                    }
                    throw new System.FormatException.$ctor1("Could not convert element to TimeSpan.");
                },
                op_Explicit$14: function (element) {
                    var $t;
                    if (element == null) {
                        throw new System.ArgumentNullException.$ctor1("The parameter cannot be null");
                    }
                    var content = null;
                    $t = Bridge.getEnumerator(element.Nodes(), System.Xml.Linq.XNode);
                    try {
                        while ($t.moveNext()) {
                            var node = $t.Current;
                            content = node;
                            break;
                        }
                    } finally {
                        if (Bridge.is($t, System.IDisposable)) {
                            $t.System$IDisposable$Dispose();
                        }
                    }
                    if (content == null) {
                        return null;
                    }
                    if (Bridge.is(content, System.Xml.Linq.XText)) {
                        var uintAsString = Bridge.cast(content, System.Xml.Linq.XText).Value;
                        if (System.String.isNullOrWhiteSpace(uintAsString)) {
                            return null;
                        }
                        return System.UInt32.parse(uintAsString);
                    }
                    throw new System.FormatException.$ctor1("Could not convert element to uint.");
                },
                op_Explicit$15: function (element) {
                    var $t;
                    if (element == null) {
                        throw new System.ArgumentNullException.$ctor1("The parameter cannot be null");
                    }
                    var content = null;
                    $t = Bridge.getEnumerator(element.Nodes(), System.Xml.Linq.XNode);
                    try {
                        while ($t.moveNext()) {
                            var node = $t.Current;
                            content = node;
                            break;
                        }
                    } finally {
                        if (Bridge.is($t, System.IDisposable)) {
                            $t.System$IDisposable$Dispose();
                        }
                    }
                    if (content == null) {
                        return System.UInt64.lift(null);
                    }
                    if (Bridge.is(content, System.Xml.Linq.XText)) {
                        var ulongAsString = Bridge.cast(content, System.Xml.Linq.XText).Value;
                        if (System.String.isNullOrWhiteSpace(ulongAsString)) {
                            return System.UInt64.lift(null);
                        }
                        return System.UInt64.parse(ulongAsString);
                    }
                    throw new System.FormatException.$ctor1("Could not convert element to ulong.");
                }
            }
        },
        fields: {
            _name: null
        },
        props: {
            /**
             * Gets or sets the name of this element.
             *
             * @instance
             * @public
             * @memberof System.Xml.Linq.XElement
             * @function Name
             * @type System.Xml.Linq.XName
             */
            Name: {
                get: function () {
                    return this._name;
                },
                set: function (value) {
                    var $t, $t1;
                    this._name = value;

                    var newNode = this.CreateJsNodeForName(value);

                    $t = Bridge.getEnumerator(this.Attributes(), System.Xml.Linq.XAttribute);
                    try {
                        while ($t.moveNext()) {
                            var attribute = $t.Current;
                            this.MoveNodeElement(attribute, newNode);
                        }
                    } finally {
                        if (Bridge.is($t, System.IDisposable)) {
                            $t.System$IDisposable$Dispose();
                        }
                    }

                    $t1 = Bridge.getEnumerator(this.Nodes(), System.Xml.Linq.XNode);
                    try {
                        while ($t1.moveNext()) {
                            var child = $t1.Current;
                            this.MoveNodeElement(child, newNode);
                        }
                    } finally {
                        if (Bridge.is($t1, System.IDisposable)) {
                            $t1.System$IDisposable$Dispose();
                        }
                    }

                    this.INTERNAL_jsnode.insertAdjacentElement("beforeBegin", newNode);


                    this.INTERNAL_jsnode.remove;

                    this.INTERNAL_jsnode = newNode;

                }
            },
            /**
             * Gets or sets the concatenated text contents of this element.
             *
             * @instance
             * @public
             * @memberof System.Xml.Linq.XElement
             * @function Value
             * @type string
             */
            Value: {
                get: function () {
                    return System.Convert.toString(this.INTERNAL_jsnode.textContent);
                },
                set: function (value) {
                    this.INTERNAL_jsnode.textContent = value;
                }
            }
        },
        ctors: {
            ctor: function (jsNode) {
                this.$initialize();
                System.Xml.Linq.XContainer.ctor.call(this);
                this.INTERNAL_jsnode = jsNode;
                var xName = new System.Xml.Linq.XName();
                if (CSHTML5.Interop.IsRunningInTheSimulator) {
                    var concatenated = System.Convert.toString(this.INTERNAL_jsnode.localName + '|' + this.INTERNAL_jsnode.namespaceURI);
                    var sepIndex = System.String.indexOf(concatenated, String.fromCharCode(124));
                    xName.LocalName = concatenated.substr(0, sepIndex);
                    xName.Namespace = System.Xml.Linq.XNamespace.op_Implicit(concatenated.substr(((sepIndex + 1) | 0)));
                } else {
                    xName.LocalName = System.Convert.toString(this.INTERNAL_jsnode.localName);
                    xName.Namespace = System.Xml.Linq.XNamespace.op_Implicit(System.Convert.toString(this.INTERNAL_jsnode.namespaceURI));
                }
                this._name = xName;
            },
            /**
             * Initializes a new instance of the System.Xml.Linq.XElement class with the
             specified name.
             *
             * @instance
             * @public
             * @this System.Xml.Linq.XElement
             * @memberof System.Xml.Linq.XElement
             * @param   {System.Xml.Linq.XName}    name    An System.Xml.Linq.XName that contains the name of the element.
             * @return  {void}
             */
            $ctor2: function (name) {
                this.$initialize();
                System.Xml.Linq.XContainer.ctor.call(this);
                this._name = name;

                this.INTERNAL_jsnode = this.CreateJsNodeForName(name);
            },
            /**
             * Initializes a new instance of the System.Xml.Linq.XElement class from another
             System.Xml.Linq.XElement object.
             *
             * @instance
             * @public
             * @this System.Xml.Linq.XElement
             * @memberof System.Xml.Linq.XElement
             * @param   {System.Xml.Linq.XElement}    other    An System.Xml.Linq.XElement object to copy from.
             * @return  {void}
             */
            $ctor1: function (other) {
                System.Xml.Linq.XElement.$ctor2.call(this, other._name);
            },
            /**
             * Initializes a new instance of the System.Xml.Linq.XElement class with the
             specified name and content.
             *
             * @instance
             * @public
             * @this System.Xml.Linq.XElement
             * @memberof System.Xml.Linq.XElement
             * @param   {System.Xml.Linq.XName}    name       A System.Xml.Linq.XName that contains the element name.
             * @param   {System.Object}            content    The contents of the element.
             * @return  {void}
             */
            $ctor3: function (name, content) {
                var $t;
                System.Xml.Linq.XElement.$ctor2.call(this, name);
                if (Bridge.is(content, System.Collections.IEnumerable)) {
                    var contentAsIEnumerable = Bridge.cast(content, System.Collections.IEnumerable);
                    $t = Bridge.getEnumerator(contentAsIEnumerable);
                    try {
                        while ($t.moveNext()) {
                            var current = $t.Current;
                            this.Add(current);
                        }
                    } finally {
                        if (Bridge.is($t, System.IDisposable)) {
                            $t.System$IDisposable$Dispose();
                        }
                    }
                } else {
                    this.Add(content);
                }
            },
            /**
             * Initializes a new instance of the System.Xml.Linq.XElement class with the
             specified name and content.
             *
             * @instance
             * @public
             * @this System.Xml.Linq.XElement
             * @memberof System.Xml.Linq.XElement
             * @param   {System.Xml.Linq.XName}    name       A System.Xml.Linq.XName that contains the element name.
             * @param   {Array.<System.Object>}    content    The initial content of the element.
             * @return  {void}
             */
            $ctor4: function (name, content) {
                if (content === void 0) { content = []; }
                var $t;

                System.Xml.Linq.XElement.$ctor2.call(this, name);
                $t = Bridge.getEnumerator(content);
                try {
                    while ($t.moveNext()) {
                        var contentElement = $t.Current;
                        this.Add(contentElement);
                    }
                } finally {
                    if (Bridge.is($t, System.IDisposable)) {
                        $t.System$IDisposable$Dispose();
                    }
                }
            }
        },
        methods: {
            CreateJsNodeForName: function (name) {
                var tempDoc = document.implementation.createDocument(null, "temp", null);
                var jsNode;
                if (System.Xml.Linq.XNamespace.op_Equality(name.Namespace, null)) {
                    jsNode = tempDoc.createElement(name.LocalName);
                } else {
                    jsNode = tempDoc.createElementNS(name.NamespaceName, name.LocalName);
                }
                return jsNode;
            },
            MoveNodeElement: function (nodeElement, destinationNode) {
                if (Bridge.is(nodeElement, System.Xml.Linq.XNode)) {
                    var contentAsXNode = Bridge.cast(nodeElement, System.Xml.Linq.XNode);
                    destinationNode.appendChild(contentAsXNode.INTERNAL_jsnode);
                } else if (Bridge.is(nodeElement, System.Xml.Linq.XAttribute)) {
                    var contentAsXAttribute = Bridge.cast(nodeElement, System.Xml.Linq.XAttribute);
                    contentAsXAttribute.INTERNAL_containerJSNode = destinationNode;

                    if (System.String.isNullOrWhiteSpace(contentAsXAttribute.Name.NamespaceName)) {
                        destinationNode.setAttribute(contentAsXAttribute.Name.LocalName, contentAsXAttribute.Value);
                    } else {
                        destinationNode.setAttributeNS(contentAsXAttribute.Name.NamespaceName, contentAsXAttribute.Name.LocalName, contentAsXAttribute.Value);
                    }
                }
            },
            /**
             * Returns a collection of attributes of this element.
             *
             * @instance
             * @public
             * @this System.Xml.Linq.XElement
             * @memberof System.Xml.Linq.XElement
             * @return  {System.Collections.Generic.IEnumerable$1}        An System.Collections.Generic.IEnumerable`1 of System.Xml.Linq.XAttribute
             of attributes of this element.
             */
            Attributes: function () {
                return new (Bridge.GeneratorEnumerable$1(System.Xml.Linq.XAttribute))(Bridge.fn.bind(this, function ()  {
                    var $step = 0,
                        $jumpFromFinally,
                        $returnValue,
                        jsAttributes,
                        i,
                        currentJsAttribute,
                        $async_e;

                    var $enumerator = new (Bridge.GeneratorEnumerator$1(System.Xml.Linq.XAttribute))(Bridge.fn.bind(this, function () {
                        try {
                            for (;;) {
                                switch ($step) {
                                    case 0: {
                                        jsAttributes = this.INTERNAL_jsnode.attributes;
                                            i = 0;
                                        $step = 1;
                                        continue;
                                    }
                                    case 1: {
                                        if ( true ) {
                                                $step = 2;
                                                continue;
                                            } 
                                            $step = 7;
                                            continue;
                                    }
                                    case 2: {
                                        currentJsAttribute = jsAttributes[Bridge.box(i, System.Int32)];
                                            if ((CSHTML5.Interop.IsRunningInTheSimulator && ((typeof(currentJsAttribute) === 'undefined') || (currentJsAttribute === null))) || System.Convert.toBoolean(currentJsAttribute == undefined || currentJsAttribute == null)) {
                                                $step = 3;
                                                continue;
                                            } else  {
                                                $step = 4;
                                                continue;
                                            }
                                    }
                                    case 3: {
                                        return false;
                                    }
                                    case 4: {
                                        $enumerator.current = System.Xml.Linq.XElement.GetAttributeFromJSAttribute(currentJsAttribute);
                                            $step = 5;
                                            return true;
                                    }
                                    case 5: {
                                        $step = 6;
                                        continue;
                                    }
                                    case 6: {
                                        i = (i + 1) | 0;

                                            $step = 1;
                                            continue;
                                    }
                                    case 7: {

                                    }
                                    default: {
                                        return false;
                                    }
                                }
                            }
                        } catch($async_e1) {
                            $async_e = System.Exception.create($async_e1);
                            throw $async_e;
                        }
                    }));
                    return $enumerator;
                }));
            },
            /**
             * Returns a filtered collection of attributes of this element. Only elements
             that have a matching System.Xml.Linq.XName are included in the collection.
             *
             * @instance
             * @public
             * @this System.Xml.Linq.XElement
             * @memberof System.Xml.Linq.XElement
             * @param   {System.Xml.Linq.XName}                       name    The System.Xml.Linq.XName to match.
             * @return  {System.Collections.Generic.IEnumerable$1}            An System.Collections.Generic.IEnumerable`1 of System.Xml.Linq.XAttribute
             that contains the attributes of this element. Only elements that have a matching
             System.Xml.Linq.XName are included in the collection.
             */
            Attributes$1: function (name) {
                return new (Bridge.GeneratorEnumerable$1(System.Xml.Linq.XAttribute))(Bridge.fn.bind(this, function (name) {
                    var $step = 0,
                        $jumpFromFinally,
                        $returnValue,
                        jsAttribute,
                        $async_e;

                    var $enumerator = new (Bridge.GeneratorEnumerator$1(System.Xml.Linq.XAttribute))(Bridge.fn.bind(this, function () {
                        try {
                            for (;;) {
                                switch ($step) {
                                    case 0: {
                                        if (!System.String.isNullOrWhiteSpace(name.NamespaceName)) {
                                                jsAttribute = this.INTERNAL_jsnode.attributes.getNamedItemNS(name.NamespaceName,name.LocalName);
                                            } else {
                                                jsAttribute = this.INTERNAL_jsnode.attributes.getNamedItem(name.LocalName);
                                            }
                                            if (System.Convert.toBoolean(jsAttribute != null && jsAttribute != undefined)) {
                                                $step = 1;
                                                continue;
                                            } 
                                            $step = 3;
                                            continue;
                                    }
                                    case 1: {
                                        $enumerator.current = System.Xml.Linq.XElement.GetAttributeFromJSAttribute(jsAttribute);
                                            $step = 2;
                                            return true;
                                    }
                                    case 2: {
                                        $step = 3;
                                        continue;
                                    }
                                    case 3: {
                                        return false;
                                    }
                                    default: {
                                        return false;
                                    }
                                }
                            }
                        } catch($async_e1) {
                            $async_e = System.Exception.create($async_e1);
                            throw $async_e;
                        }
                    }));
                    return $enumerator;
                }, arguments));
            },
            /**
             * Gets the default System.Xml.Linq.XNamespace of this System.Xml.Linq.XElement.
             *
             * @instance
             * @public
             * @this System.Xml.Linq.XElement
             * @memberof System.Xml.Linq.XElement
             * @return  {System.Xml.Linq.XNamespace}        A System.Xml.Linq.XNamespace that contains the default namespace of this
             System.Xml.Linq.XElement.
             */
            GetDefaultNamespace: function () {
                var str = System.Convert.toString(this.INTERNAL_jsnode.namespaceURI);
                return System.Xml.Linq.XNamespace.Get(str);
            },
            /**
             * Gets the namespace associated with a particular prefix for this System.Xml.Linq.XElement.
             *
             * @instance
             * @public
             * @this System.Xml.Linq.XElement
             * @memberof System.Xml.Linq.XElement
             * @param   {string}                        prefix    A string that contains the namespace prefix to look up.
             * @return  {System.Xml.Linq.XNamespace}              An System.Xml.Linq.XNamespace for the namespace associated with the prefix
             for this System.Xml.Linq.XElement.
             */
            GetNamespaceOfPrefix: function (prefix) {
                var str = System.Convert.toString(this.INTERNAL_jsnode.lookupNamespaceURI(prefix));
                return System.Xml.Linq.XNamespace.Get(str);
            },
            /**
             * Gets the prefix associated with a namespace for this System.Xml.Linq.XElement.
             *
             * @instance
             * @public
             * @this System.Xml.Linq.XElement
             * @memberof System.Xml.Linq.XElement
             * @param   {System.Xml.Linq.XNamespace}    ns    An System.Xml.Linq.XNamespace to look up.
             * @return  {string}                              A System.String that contains the namespace prefix.
             */
            GetPrefixOfNamespace: function (ns) {
                var str = System.Convert.toString(this.INTERNAL_jsnode.lookupPrefix(ns.NamespaceName));
                return str;
            },
            /**
             * Sets the value of an attribute, adds an attribute, or removes an attribute.
             *
             * @instance
             * @public
             * @this System.Xml.Linq.XElement
             * @memberof System.Xml.Linq.XElement
             * @param   {System.Xml.Linq.XName}    name     An System.Xml.Linq.XName that contains the name of the attribute to change.
             * @param   {System.Object}            value    The value to assign to the attribute. The attribute is removed if the value
             is null. Otherwise, the value is converted to its string representation and
             assigned to the System.Xml.Linq.XAttribute.Value property of the attribute.
             * @return  {void}
             */
            SetAttributeValue: function (name, value) {
                if (value != null) {
                    this.Add(new System.Xml.Linq.XAttribute.$ctor1(name, value));
                } else {
                    if (System.String.isNullOrWhiteSpace(name.NamespaceName)) {
                        this.INTERNAL_jsnode.removeAttributeNS(name.LocalName);
                    } else {
                        this.INTERNAL_jsnode.removeAttributeNS(name.NamespaceName,name.LocalName);
                    }
                }
            },
            Clone: function () {
                var $t;
                var clone = new System.Xml.Linq.XElement.$ctor3(this.Name, this.Nodes());
                $t = Bridge.getEnumerator(this.Attributes(), System.Xml.Linq.XAttribute);
                try {
                    while ($t.moveNext()) {
                        var attribute = $t.Current;
                        clone.Add(attribute);
                    }
                } finally {
                    if (Bridge.is($t, System.IDisposable)) {
                        $t.System$IDisposable$Dispose();
                    }
                }
                return clone;
            }
        }
    });

    Bridge.define("TypeScriptDefinitionsSupport.UnionType$2", function (T0, T1) { return {
        inherits: [TypeScriptDefinitionsSupport.UnionType],
        statics: {
            methods: {
                FromJavaScriptInstance: function (jsObj) {
                    return new (TypeScriptDefinitionsSupport.UnionType$2(T0,T1)).$ctor2(jsObj);
                },
                op_Implicit$2: function (t) {
                    return new (TypeScriptDefinitionsSupport.UnionType$2(T0,T1)).ctor(t);
                },
                op_Implicit: function (value) {
                    if (value._Type == null) {
                        value._t0 = value.CreateInstance(T0);
                        value._Type = T0;
                        return value._t0;
                    }
                    if (Bridge.referenceEquals(value._Type, T0)) {
                        return value._t0;
                    } else {
                        throw new System.Exception("Unable to cast this UnionType to " + (Bridge.Reflection.getTypeName(T0) || "") + " because it contains a value that is of another type.");
                    }
                },
                op_Implicit$3: function (t) {
                    return new (TypeScriptDefinitionsSupport.UnionType$2(T0,T1)).$ctor1(t);
                },
                op_Implicit$1: function (value) {
                    if (value._Type == null) {
                        value._t1 = value.CreateInstance(T1);
                        value._Type = T1;
                        return value._t1;
                    }
                    if (Bridge.referenceEquals(value._Type, T1)) {
                        return value._t1;
                    } else {
                        throw new System.Exception("Unable to cast this UnionType to " + (Bridge.Reflection.getTypeName(T1) || "") + " because it contains a value that is of another type.");
                    }
                }
            }
        },
        fields: {
            _t0: Bridge.getDefaultValue(T0),
            _t1: Bridge.getDefaultValue(T1)
        },
        ctors: {
            $ctor2: function (jsObj) {
                this.$initialize();
                TypeScriptDefinitionsSupport.UnionType.ctor.call(this);
                this.UnderlyingJSInstance = jsObj;
            },
            ctor: function (t) {
                this.$initialize();
                TypeScriptDefinitionsSupport.UnionType.ctor.call(this);
                this._t0 = t;
                this._Type = T0;
                var o = TypeScriptDefinitionsSupport.JSObject.CreateFrom(t);
                ;
                this.UnderlyingJSInstance = o.UnderlyingJSInstance;
            },
            $ctor1: function (t) {
                this.$initialize();
                TypeScriptDefinitionsSupport.UnionType.ctor.call(this);
                this._t1 = t;
                this._Type = T1;
                var o = TypeScriptDefinitionsSupport.JSObject.CreateFrom(t);
                ;
                this.UnderlyingJSInstance = o.UnderlyingJSInstance;
            }
        }
    }; });

    Bridge.define("TypeScriptDefinitionsSupport.UnionType$3", function (T0, T1, T2) { return {
        inherits: [TypeScriptDefinitionsSupport.UnionType],
        statics: {
            methods: {
                FromJavaScriptInstance: function (jsObj) {
                    return new (TypeScriptDefinitionsSupport.UnionType$3(T0,T1,T2)).$ctor3(jsObj);
                },
                op_Implicit$3: function (t) {
                    return new (TypeScriptDefinitionsSupport.UnionType$3(T0,T1,T2)).ctor(t);
                },
                op_Implicit: function (value) {
                    if (value._Type == null) {
                        value._t0 = value.CreateInstance(T0);
                        value._Type = T0;
                        return value._t0;
                    }
                    if (Bridge.referenceEquals(value._Type, T0)) {
                        return value._t0;
                    } else {
                        throw new System.Exception("Unable to cast this UnionType to " + (Bridge.Reflection.getTypeName(T0) || "") + " because it contains a value that is of another type.");
                    }
                },
                op_Implicit$4: function (t) {
                    return new (TypeScriptDefinitionsSupport.UnionType$3(T0,T1,T2)).$ctor1(t);
                },
                op_Implicit$1: function (value) {
                    if (value._Type == null) {
                        value._t1 = value.CreateInstance(T1);
                        value._Type = T1;
                        return value._t1;
                    }
                    if (Bridge.referenceEquals(value._Type, T1)) {
                        return value._t1;
                    } else {
                        throw new System.Exception("Unable to cast this UnionType to " + (Bridge.Reflection.getTypeName(T1) || "") + " because it contains a value that is of another type.");
                    }
                },
                op_Implicit$5: function (t) {
                    return new (TypeScriptDefinitionsSupport.UnionType$3(T0,T1,T2)).$ctor2(t);
                },
                op_Implicit$2: function (value) {
                    if (value._Type == null) {
                        value._t2 = value.CreateInstance(T2);
                        value._Type = T2;
                        return value._t2;
                    }
                    if (Bridge.referenceEquals(value._Type, T2)) {
                        return value._t2;
                    } else {
                        throw new System.Exception("Unable to cast this UnionType to " + (Bridge.Reflection.getTypeName(T2) || "") + " because it contains a value that is of another type.");
                    }
                }
            }
        },
        fields: {
            _t0: Bridge.getDefaultValue(T0),
            _t1: Bridge.getDefaultValue(T1),
            _t2: Bridge.getDefaultValue(T2)
        },
        ctors: {
            $ctor3: function (jsObj) {
                this.$initialize();
                TypeScriptDefinitionsSupport.UnionType.ctor.call(this);
                this.UnderlyingJSInstance = jsObj;
            },
            ctor: function (t) {
                this.$initialize();
                TypeScriptDefinitionsSupport.UnionType.ctor.call(this);
                this._t0 = t;
                this._Type = T0;
                var o = TypeScriptDefinitionsSupport.JSObject.CreateFrom(t);
                ;
                this.UnderlyingJSInstance = o.UnderlyingJSInstance;
            },
            $ctor1: function (t) {
                this.$initialize();
                TypeScriptDefinitionsSupport.UnionType.ctor.call(this);
                this._t1 = t;
                this._Type = T1;
                var o = TypeScriptDefinitionsSupport.JSObject.CreateFrom(t);
                ;
                this.UnderlyingJSInstance = o.UnderlyingJSInstance;
            },
            $ctor2: function (t) {
                this.$initialize();
                TypeScriptDefinitionsSupport.UnionType.ctor.call(this);
                this._t2 = t;
                this._Type = T2;
                var o = TypeScriptDefinitionsSupport.JSObject.CreateFrom(t);
                ;
                this.UnderlyingJSInstance = o.UnderlyingJSInstance;
            }
        }
    }; });

    Bridge.define("TypeScriptDefinitionsSupport.UnionType$4", function (T0, T1, T2, T3) { return {
        inherits: [TypeScriptDefinitionsSupport.UnionType],
        statics: {
            methods: {
                FromJavaScriptInstance: function (jsObj) {
                    return new (TypeScriptDefinitionsSupport.UnionType$4(T0,T1,T2,T3)).$ctor4(jsObj);
                },
                op_Implicit$4: function (t) {
                    return new (TypeScriptDefinitionsSupport.UnionType$4(T0,T1,T2,T3)).ctor(t);
                },
                op_Implicit: function (value) {
                    if (value._Type == null) {
                        value._t0 = value.CreateInstance(T0);
                        value._Type = T0;
                        return value._t0;
                    }
                    if (Bridge.referenceEquals(value._Type, T0)) {
                        return value._t0;
                    } else {
                        throw new System.Exception("Unable to cast this UnionType to " + (Bridge.Reflection.getTypeName(T0) || "") + " because it contains a value that is of another type.");
                    }
                },
                op_Implicit$5: function (t) {
                    return new (TypeScriptDefinitionsSupport.UnionType$4(T0,T1,T2,T3)).$ctor1(t);
                },
                op_Implicit$1: function (value) {
                    if (value._Type == null) {
                        value._t1 = value.CreateInstance(T1);
                        value._Type = T1;
                        return value._t1;
                    }
                    if (Bridge.referenceEquals(value._Type, T1)) {
                        return value._t1;
                    } else {
                        throw new System.Exception("Unable to cast this UnionType to " + (Bridge.Reflection.getTypeName(T1) || "") + " because it contains a value that is of another type.");
                    }
                },
                op_Implicit$6: function (t) {
                    return new (TypeScriptDefinitionsSupport.UnionType$4(T0,T1,T2,T3)).$ctor2(t);
                },
                op_Implicit$2: function (value) {
                    if (value._Type == null) {
                        value._t2 = value.CreateInstance(T2);
                        value._Type = T2;
                        return value._t2;
                    }
                    if (Bridge.referenceEquals(value._Type, T2)) {
                        return value._t2;
                    } else {
                        throw new System.Exception("Unable to cast this UnionType to " + (Bridge.Reflection.getTypeName(T2) || "") + " because it contains a value that is of another type.");
                    }
                },
                op_Implicit$7: function (t) {
                    return new (TypeScriptDefinitionsSupport.UnionType$4(T0,T1,T2,T3)).$ctor3(t);
                },
                op_Implicit$3: function (value) {
                    if (value._Type == null) {
                        value._t3 = value.CreateInstance(T3);
                        value._Type = T3;
                        return value._t3;
                    }
                    if (Bridge.referenceEquals(value._Type, T3)) {
                        return value._t3;
                    } else {
                        throw new System.Exception("Unable to cast this UnionType to " + (Bridge.Reflection.getTypeName(T3) || "") + " because it contains a value that is of another type.");
                    }
                }
            }
        },
        fields: {
            _t0: Bridge.getDefaultValue(T0),
            _t1: Bridge.getDefaultValue(T1),
            _t2: Bridge.getDefaultValue(T2),
            _t3: Bridge.getDefaultValue(T3)
        },
        ctors: {
            $ctor4: function (jsObj) {
                this.$initialize();
                TypeScriptDefinitionsSupport.UnionType.ctor.call(this);
                this.UnderlyingJSInstance = jsObj;
            },
            ctor: function (t) {
                this.$initialize();
                TypeScriptDefinitionsSupport.UnionType.ctor.call(this);
                this._t0 = t;
                this._Type = T0;
                var o = TypeScriptDefinitionsSupport.JSObject.CreateFrom(t);
                ;
                this.UnderlyingJSInstance = o.UnderlyingJSInstance;
            },
            $ctor1: function (t) {
                this.$initialize();
                TypeScriptDefinitionsSupport.UnionType.ctor.call(this);
                this._t1 = t;
                this._Type = T1;
                var o = TypeScriptDefinitionsSupport.JSObject.CreateFrom(t);
                ;
                this.UnderlyingJSInstance = o.UnderlyingJSInstance;
            },
            $ctor2: function (t) {
                this.$initialize();
                TypeScriptDefinitionsSupport.UnionType.ctor.call(this);
                this._t2 = t;
                this._Type = T2;
                var o = TypeScriptDefinitionsSupport.JSObject.CreateFrom(t);
                ;
                this.UnderlyingJSInstance = o.UnderlyingJSInstance;
            },
            $ctor3: function (t) {
                this.$initialize();
                TypeScriptDefinitionsSupport.UnionType.ctor.call(this);
                this._t3 = t;
                this._Type = T3;
                var o = TypeScriptDefinitionsSupport.JSObject.CreateFrom(t);
                ;
                this.UnderlyingJSInstance = o.UnderlyingJSInstance;
            }
        }
    }; });

    Bridge.define("TypeScriptDefinitionsSupport.UnionType$5", function (T0, T1, T2, T3, T4) { return {
        inherits: [TypeScriptDefinitionsSupport.UnionType],
        statics: {
            methods: {
                FromJavaScriptInstance: function (jsObj) {
                    return new (TypeScriptDefinitionsSupport.UnionType$5(T0,T1,T2,T3,T4)).$ctor5(jsObj);
                },
                op_Implicit$5: function (t) {
                    return new (TypeScriptDefinitionsSupport.UnionType$5(T0,T1,T2,T3,T4)).ctor(t);
                },
                op_Implicit: function (value) {
                    if (value._Type == null) {
                        value._t0 = value.CreateInstance(T0);
                        value._Type = T0;
                        return value._t0;
                    }
                    if (Bridge.referenceEquals(value._Type, T0)) {
                        return value._t0;
                    } else {
                        throw new System.Exception("Unable to cast this UnionType to " + (Bridge.Reflection.getTypeName(T0) || "") + " because it contains a value that is of another type.");
                    }
                },
                op_Implicit$6: function (t) {
                    return new (TypeScriptDefinitionsSupport.UnionType$5(T0,T1,T2,T3,T4)).$ctor1(t);
                },
                op_Implicit$1: function (value) {
                    if (value._Type == null) {
                        value._t1 = value.CreateInstance(T1);
                        value._Type = T1;
                        return value._t1;
                    }
                    if (Bridge.referenceEquals(value._Type, T1)) {
                        return value._t1;
                    } else {
                        throw new System.Exception("Unable to cast this UnionType to " + (Bridge.Reflection.getTypeName(T1) || "") + " because it contains a value that is of another type.");
                    }
                },
                op_Implicit$7: function (t) {
                    return new (TypeScriptDefinitionsSupport.UnionType$5(T0,T1,T2,T3,T4)).$ctor2(t);
                },
                op_Implicit$2: function (value) {
                    if (value._Type == null) {
                        value._t2 = value.CreateInstance(T2);
                        value._Type = T2;
                        return value._t2;
                    }
                    if (Bridge.referenceEquals(value._Type, T2)) {
                        return value._t2;
                    } else {
                        throw new System.Exception("Unable to cast this UnionType to " + (Bridge.Reflection.getTypeName(T2) || "") + " because it contains a value that is of another type.");
                    }
                },
                op_Implicit$8: function (t) {
                    return new (TypeScriptDefinitionsSupport.UnionType$5(T0,T1,T2,T3,T4)).$ctor3(t);
                },
                op_Implicit$3: function (value) {
                    if (value._Type == null) {
                        value._t3 = value.CreateInstance(T3);
                        value._Type = T3;
                        return value._t3;
                    }
                    if (Bridge.referenceEquals(value._Type, T3)) {
                        return value._t3;
                    } else {
                        throw new System.Exception("Unable to cast this UnionType to " + (Bridge.Reflection.getTypeName(T3) || "") + " because it contains a value that is of another type.");
                    }
                },
                op_Implicit$9: function (t) {
                    return new (TypeScriptDefinitionsSupport.UnionType$5(T0,T1,T2,T3,T4)).$ctor4(t);
                },
                op_Implicit$4: function (value) {
                    if (value._Type == null) {
                        value._t4 = value.CreateInstance(T4);
                        value._Type = T4;
                        return value._t4;
                    }
                    if (Bridge.referenceEquals(value._Type, T4)) {
                        return value._t4;
                    } else {
                        throw new System.Exception("Unable to cast this UnionType to " + (Bridge.Reflection.getTypeName(T4) || "") + " because it contains a value that is of another type.");
                    }
                }
            }
        },
        fields: {
            _t0: Bridge.getDefaultValue(T0),
            _t1: Bridge.getDefaultValue(T1),
            _t2: Bridge.getDefaultValue(T2),
            _t3: Bridge.getDefaultValue(T3),
            _t4: Bridge.getDefaultValue(T4)
        },
        ctors: {
            $ctor5: function (jsObj) {
                this.$initialize();
                TypeScriptDefinitionsSupport.UnionType.ctor.call(this);
                this.UnderlyingJSInstance = jsObj;
            },
            ctor: function (t) {
                this.$initialize();
                TypeScriptDefinitionsSupport.UnionType.ctor.call(this);
                this._t0 = t;
                this._Type = T0;
                var o = TypeScriptDefinitionsSupport.JSObject.CreateFrom(t);
                ;
                this.UnderlyingJSInstance = o.UnderlyingJSInstance;
            },
            $ctor1: function (t) {
                this.$initialize();
                TypeScriptDefinitionsSupport.UnionType.ctor.call(this);
                this._t1 = t;
                this._Type = T1;
                var o = TypeScriptDefinitionsSupport.JSObject.CreateFrom(t);
                ;
                this.UnderlyingJSInstance = o.UnderlyingJSInstance;
            },
            $ctor2: function (t) {
                this.$initialize();
                TypeScriptDefinitionsSupport.UnionType.ctor.call(this);
                this._t2 = t;
                this._Type = T2;
                var o = TypeScriptDefinitionsSupport.JSObject.CreateFrom(t);
                ;
                this.UnderlyingJSInstance = o.UnderlyingJSInstance;
            },
            $ctor3: function (t) {
                this.$initialize();
                TypeScriptDefinitionsSupport.UnionType.ctor.call(this);
                this._t3 = t;
                this._Type = T3;
                var o = TypeScriptDefinitionsSupport.JSObject.CreateFrom(t);
                ;
                this.UnderlyingJSInstance = o.UnderlyingJSInstance;
            },
            $ctor4: function (t) {
                this.$initialize();
                TypeScriptDefinitionsSupport.UnionType.ctor.call(this);
                this._t4 = t;
                this._Type = T4;
                var o = TypeScriptDefinitionsSupport.JSObject.CreateFrom(t);
                ;
                this.UnderlyingJSInstance = o.UnderlyingJSInstance;
            }
        }
    }; });

    /**
     * Represents the base class for UI elements that use a ControlTemplate to define
     their appearance.
     *
     * @public
     * @class Windows.UI.Xaml.Controls.Control
     * @augments Windows.UI.Xaml.FrameworkElement
     */
    Bridge.define("Windows.UI.Xaml.Controls.Control", {
        inherits: [Windows.UI.Xaml.FrameworkElement],
        statics: {
            fields: {
                TABINDEX_BROWSER_MAX_VALUE: 0,
                /**
                 * Identifies the Background dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Controls.Control
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                BackgroundProperty: null,
                /**
                 * Identifies the BorderBrush dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Controls.Control
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                BorderBrushProperty: null,
                /**
                 * Identifies the BorderThickness dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Controls.Control
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                BorderThicknessProperty: null,
                /**
                 * Identifies the FontWeight dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Controls.Control
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                FontWeightProperty: null,
                /**
                 * Identifies the FontStyle dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Controls.Control
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                FontStyleProperty: null,
                /**
                 * Identifies the Foreground dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Controls.Control
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                ForegroundProperty: null,
                /**
                 * Identifies the FontFamily dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Controls.Control
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                FontFamilyProperty: null,
                /**
                 * Identifies the FontSize dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Controls.Control
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                FontSizeProperty: null,
                /**
                 * Identifies the TextDecorations dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Controls.Control
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                TextDecorationsProperty: null,
                /**
                 * Identifies the Padding dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Controls.Control
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                PaddingProperty: null,
                /**
                 * Identifies the HorizontalContentAlignment dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Controls.Control
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                HorizontalContentAlignmentProperty: null,
                /**
                 * Identifies the VerticalContentAlignment dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Controls.Control
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                VerticalContentAlignmentProperty: null,
                TabIndexProperty: null,
                /**
                 * Identifies the Control.IsTabStop dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Controls.Control
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                IsTabStopProperty: null,
                /**
                 * Identifies the Template dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Controls.Control
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                TemplateProperty: null
            },
            ctors: {
                init: function () {
                    var $t;
                    this.TABINDEX_BROWSER_MAX_VALUE = 32767;
                    this.BackgroundProperty = Windows.UI.Xaml.DependencyProperty.Register("Background", Windows.UI.Xaml.Media.Brush, Windows.UI.Xaml.Controls.Control, ($t = new Windows.UI.Xaml.PropertyMetadata.$ctor4(null), $t.GetCSSEquivalent = function (instance) {
                        var $t1;
                        return ($t1 = new Windows.UI.Xaml.CSSEquivalent(), $t1.Name = function (_o1) {
                                _o1.add("background");
                                _o1.add("backgroundColor");
                                _o1.add("backgroundColorAlpha");
                                return _o1;
                            }(new (System.Collections.Generic.List$1(System.String)).ctor()), $t1);
                    }, $t));
                    this.BorderBrushProperty = Windows.UI.Xaml.DependencyProperty.Register("BorderBrush", Windows.UI.Xaml.Media.Brush, Windows.UI.Xaml.Controls.Control, ($t = new Windows.UI.Xaml.PropertyMetadata.$ctor4(null), $t.GetCSSEquivalent = function (instance) {
                        var $t1;
                        return ($t1 = new Windows.UI.Xaml.CSSEquivalent(), $t1.Name = function (_o2) {
                                _o2.add("borderColor");
                                return _o2;
                            }(new (System.Collections.Generic.List$1(System.String)).ctor()), $t1);
                    }, $t));
                    this.BorderThicknessProperty = Windows.UI.Xaml.DependencyProperty.Register("BorderThickness", Windows.UI.Xaml.Thickness, Windows.UI.Xaml.Controls.Control, ($t = new Windows.UI.Xaml.PropertyMetadata.$ctor1(new Windows.UI.Xaml.Thickness.ctor().$clone()), $t.MethodToUpdateDom = Windows.UI.Xaml.Controls.Control.BorderThickness_MethodToUpdateDom, $t));
                    this.FontWeightProperty = Windows.UI.Xaml.DependencyProperty.Register("FontWeight", Windows.UI.Text.FontWeight, Windows.UI.Xaml.Controls.Control, ($t = new Windows.UI.Xaml.PropertyMetadata.$ctor1(Windows.UI.Text.FontWeights.Normal.$clone()), $t.GetCSSEquivalent = function (instance) {
                        var $t1;
                        return ($t1 = new Windows.UI.Xaml.CSSEquivalent(), $t1.Value = function (inst, value) {
                            if (value != null) {
                                return Bridge.toString(System.Nullable.getValue(Bridge.cast(Bridge.unbox(value, Windows.UI.Text.FontWeight), Windows.UI.Text.FontWeight)).Weight);
                            } else {
                                return "";
                            }
                        }, $t1.Name = function (_o3) {
                                _o3.add("fontWeight");
                                return _o3;
                            }(new (System.Collections.Generic.List$1(System.String)).ctor()), $t1.ApplyAlsoWhenThereIsAControlTemplate = true, $t1);
                    }, $t));
                    this.FontStyleProperty = Windows.UI.Xaml.DependencyProperty.Register("FontStyle", Windows.UI.Text.FontStyle, Windows.UI.Xaml.Controls.Control, ($t = new Windows.UI.Xaml.PropertyMetadata.$ctor1(Bridge.box(Windows.UI.Text.FontStyle.Normal, Windows.UI.Text.FontStyle, System.Enum.toStringFn(Windows.UI.Text.FontStyle))), $t.GetCSSEquivalent = function (instance) {
                        var $t1;
                        return ($t1 = new Windows.UI.Xaml.CSSEquivalent(), $t1.Value = function (inst, value) {
                            if (value != null) {
                                return System.Enum.toString(Windows.UI.Text.FontStyle, System.Nullable.getValue(Bridge.cast(Bridge.unbox(value, Windows.UI.Text.FontStyle), System.Int32))).toLowerCase();
                            } else {
                                return "";
                            }
                        }, $t1.Name = function (_o4) {
                                _o4.add("fontStyle");
                                return _o4;
                            }(new (System.Collections.Generic.List$1(System.String)).ctor()), $t1.ApplyAlsoWhenThereIsAControlTemplate = true, $t1);
                    }, $t));
                    this.ForegroundProperty = Windows.UI.Xaml.DependencyProperty.Register("Foreground", Windows.UI.Xaml.Media.Brush, Windows.UI.Xaml.Controls.Control, ($t = new Windows.UI.Xaml.PropertyMetadata.$ctor1(new Windows.UI.Xaml.Media.SolidColorBrush.$ctor1(Windows.UI.Colors.Black.$clone())), $t.GetCSSEquivalent = function (instance) {
                        var $t1;
                        return ($t1 = new Windows.UI.Xaml.CSSEquivalent(), $t1.Name = function (_o5) {
                                _o5.add("color");
                                _o5.add("colorAlpha");
                                return _o5;
                            }(new (System.Collections.Generic.List$1(System.String)).ctor()), $t1.ApplyAlsoWhenThereIsAControlTemplate = true, $t1);
                    }, $t));
                    this.FontFamilyProperty = Windows.UI.Xaml.DependencyProperty.Register("FontFamily", Windows.UI.Xaml.Media.FontFamily, Windows.UI.Xaml.Controls.Control, ($t = new Windows.UI.Xaml.PropertyMetadata.$ctor4(null), $t.GetCSSEquivalent = function (instance) {
                        var $t1;
                        return ($t1 = new Windows.UI.Xaml.CSSEquivalent(), $t1.Value = function (inst, value) {
                            if (value != null) {
                                if (Bridge.is(value, Windows.UI.Xaml.Media.FontFamily)) {
                                    return CSHTML5.Internal.INTERNAL_FontsHelper.LoadFont(Bridge.cast(value, Windows.UI.Xaml.Media.FontFamily).Source, Bridge.cast(instance, Windows.UI.Xaml.UIElement));
                                } else {
                                    try {
                                        return CSHTML5.Internal.INTERNAL_FontsHelper.LoadFont(Bridge.toString(value), Bridge.cast(instance, Windows.UI.Xaml.UIElement));
                                    } catch ($e1) {
                                        $e1 = System.Exception.create($e1);
                                        return "";
                                    }
                                }
                            } else {
                                return "";
                            }
                        }, $t1.Name = function (_o6) {
                                _o6.add("fontFamily");
                                return _o6;
                            }(new (System.Collections.Generic.List$1(System.String)).ctor()), $t1.ApplyAlsoWhenThereIsAControlTemplate = true, $t1);
                    }, $t));
                    this.FontSizeProperty = Windows.UI.Xaml.DependencyProperty.Register("FontSize", System.Double, Windows.UI.Xaml.Controls.Control, ($t = new Windows.UI.Xaml.PropertyMetadata.$ctor1(Bridge.box(11.0, System.Double, System.Double.format, System.Double.getHashCode)), $t.MethodToUpdateDom = function (instance, newValue) {
                        var domStyle = CSHTML5.Internal.INTERNAL_HtmlDomManager.GetFrameworkElementOuterStyleForModification(Bridge.cast(instance, Windows.UI.Xaml.UIElement));
                        domStyle.lineHeight = "125%";
                    }, $t.GetCSSEquivalent = function (instance) {
                        var $t1;
                        return ($t1 = new Windows.UI.Xaml.CSSEquivalent(), $t1.Value = function (inst, value) {
                            return (System.Double.format((Math.floor(System.Nullable.getValue(Bridge.cast(Bridge.unbox(value, System.Double), System.Double)) * 1000) / 1000)) || "") + "px";
                        }, $t1.Name = function (_o7) {
                                _o7.add("fontSize");
                                return _o7;
                            }(new (System.Collections.Generic.List$1(System.String)).ctor()), $t1.ApplyAlsoWhenThereIsAControlTemplate = true, $t1);
                    }, $t));
                    this.TextDecorationsProperty = Windows.UI.Xaml.DependencyProperty.Register("TextDecorations", System.Nullable$1(Windows.UI.Text.TextDecorations), Windows.UI.Xaml.Controls.Control, ($t = new Windows.UI.Xaml.PropertyMetadata.$ctor4(null), $t.GetCSSEquivalent = Windows.UI.Xaml.Controls.Control.INTERNAL_GetCSSEquivalentForTextDecorations, $t));
                    this.PaddingProperty = Windows.UI.Xaml.DependencyProperty.Register("Padding", Windows.UI.Xaml.Thickness, Windows.UI.Xaml.Controls.Control, ($t = new Windows.UI.Xaml.PropertyMetadata.$ctor1(new Windows.UI.Xaml.Thickness.ctor().$clone()), $t.MethodToUpdateDom = Windows.UI.Xaml.Controls.Control.Padding_MethodToUpdateDom, $t));
                    this.HorizontalContentAlignmentProperty = Windows.UI.Xaml.DependencyProperty.Register("HorizontalContentAlignment", Windows.UI.Xaml.HorizontalAlignment, Windows.UI.Xaml.Controls.Control, new Windows.UI.Xaml.PropertyMetadata.$ctor1(Bridge.box(Windows.UI.Xaml.HorizontalAlignment.Center, Windows.UI.Xaml.HorizontalAlignment, System.Enum.toStringFn(Windows.UI.Xaml.HorizontalAlignment))));
                    this.VerticalContentAlignmentProperty = Windows.UI.Xaml.DependencyProperty.Register("VerticalContentAlignment", Windows.UI.Xaml.VerticalAlignment, Windows.UI.Xaml.Controls.Control, new Windows.UI.Xaml.PropertyMetadata.$ctor1(Bridge.box(Windows.UI.Xaml.VerticalAlignment.Center, Windows.UI.Xaml.VerticalAlignment, System.Enum.toStringFn(Windows.UI.Xaml.VerticalAlignment))));
                    this.TabIndexProperty = Windows.UI.Xaml.DependencyProperty.Register("TabIndex", System.Int32, Windows.UI.Xaml.Controls.Control, ($t = new Windows.UI.Xaml.PropertyMetadata.$ctor1(Bridge.box(2147483647, System.Int32)), $t.MethodToUpdateDom = Windows.UI.Xaml.Controls.Control.TabIndexProperty_MethodToUpdateDom, $t));
                    this.IsTabStopProperty = Windows.UI.Xaml.DependencyProperty.Register("IsTabStop", System.Boolean, Windows.UI.Xaml.Controls.Control, ($t = new Windows.UI.Xaml.PropertyMetadata.$ctor1(Bridge.box(true, System.Boolean, System.Boolean.toString)), $t.MethodToUpdateDom = Windows.UI.Xaml.Controls.Control.TabStopProperty_MethodToUpdateDom, $t));
                    this.TemplateProperty = Windows.UI.Xaml.DependencyProperty.Register("Template", Windows.UI.Xaml.Controls.ControlTemplate, Windows.UI.Xaml.Controls.Control, new Windows.UI.Xaml.PropertyMetadata.$ctor2(null, Windows.UI.Xaml.Controls.Control.OnTemplateChanged));
                }
            },
            methods: {
                BorderThickness_MethodToUpdateDom: function (d, newValue) {
                    var control = Bridge.cast(d, Windows.UI.Xaml.Controls.Control);
                    if (!control.HasTemplate && newValue != null) {
                        var newThickness = System.Nullable.getValue(Bridge.cast(Bridge.unbox(newValue, Windows.UI.Xaml.Thickness), Windows.UI.Xaml.Thickness));
                        var domElement = CSHTML5.Internal.INTERNAL_HtmlDomManager.GetFrameworkElementOuterStyleForModification(control);
                        domElement.borderStyle = "solid";
                        domElement.borderWidth = System.Double.format(newThickness.Top) + "px " + System.Double.format(newThickness.Right) + "px " + System.Double.format(newThickness.Bottom) + "px " + System.Double.format(newThickness.Left) + "px ";
                    }
                },
                INTERNAL_GetCSSEquivalentForTextDecorations: function (instance) {
                    var $t;
                    return ($t = new Windows.UI.Xaml.CSSEquivalent(), $t.Value = function (inst, value) {
                        if (value != null) {
                            var newTextDecoration = System.Nullable.getValue(Bridge.cast(Bridge.unbox(value, Windows.UI.Text.TextDecorations), System.Int32, true));
                            if (System.Nullable.hasValue(newTextDecoration)) {
                                switch (newTextDecoration) {
                                    case Windows.UI.Text.TextDecorations.OverLine: 
                                        return "overline";
                                    case Windows.UI.Text.TextDecorations.Strikethrough: 
                                        return "line-through";
                                    case Windows.UI.Text.TextDecorations.Underline: 
                                        return "underline";
                                    case Windows.UI.Text.TextDecorations.None: 
                                    default: 
                                        return "";
                                }
                            } else {
                                return "";
                            }
                        } else {
                            return "";
                        }
                    }, $t.Name = function (_o1) {
                            _o1.add("textDecoration");
                            return _o1;
                        }(new (System.Collections.Generic.List$1(System.String)).ctor()), $t);
                },
                Padding_MethodToUpdateDom: function (d, newValue) {
                    var control = Bridge.cast(d, Windows.UI.Xaml.Controls.Control);
                    if (!(Bridge.is(control.INTERNAL_VisualParent, Windows.UI.Xaml.Controls.Canvas)) && !control.HasTemplate) {
                        var innerDomElement = control.INTERNAL_InnerDomElement;
                        if (innerDomElement != null) {
                            var styleOfInnerDomElement = CSHTML5.Internal.INTERNAL_HtmlDomManager.GetDomElementStyleForModification(innerDomElement);
                            var newPadding = new Windows.UI.Xaml.Thickness();
                            if (newValue != null) {
                                newPadding = System.Nullable.getValue(Bridge.cast(Bridge.unbox(newValue, Windows.UI.Xaml.Thickness), Windows.UI.Xaml.Thickness));
                            } else {
                                newPadding = new Windows.UI.Xaml.Thickness.ctor();
                            }
                            if (System.Nullable.lifteq(Windows.UI.Xaml.Thickness.op_Equality, newPadding.$clone(), null)) {
                                newPadding = new Windows.UI.Xaml.Thickness.ctor();
                            }
                            styleOfInnerDomElement.boxSizing = "border-box";
                            styleOfInnerDomElement.paddingLeft = System.Double.format(newPadding.Left) + "px";
                            styleOfInnerDomElement.paddingTop = System.Double.format(newPadding.Top) + "px";
                            styleOfInnerDomElement.paddingRight = System.Double.format(newPadding.Right) + "px";
                            styleOfInnerDomElement.paddingBottom = System.Double.format(newPadding.Bottom) + "px";
                        }
                    }
                },
                TabIndexProperty_MethodToUpdateDom: function (d, newValue) {
                    var control = Bridge.cast(d, Windows.UI.Xaml.Controls.Control);
                    control.UpdateTabIndex(control.IsTabStop, System.Nullable.getValue(Bridge.cast(Bridge.unbox(newValue, System.Int32), System.Int32)));
                },
                TabStopProperty_MethodToUpdateDom: function (d, newValue) {
                    var control = Bridge.cast(d, Windows.UI.Xaml.Controls.Control);
                    control.UpdateTabIndex(System.Nullable.getValue(Bridge.cast(Bridge.unbox(newValue, System.Boolean), System.Boolean)), control.TabIndex);
                },
                OnTemplateChanged: function (d, e) {
                    var control = Bridge.cast(d, Windows.UI.Xaml.Controls.Control);

                    control.INTERNAL_IsTemplated = e.NewValue != null;

                    if (control._renderedControlTemplate != null) {
                        CSHTML5.Internal.INTERNAL_VisualTreeManager.DetachVisualChildIfNotNull(control._renderedControlTemplate, control);
                        control._renderedControlTemplate = null;
                        control.ClearRegisteredNames();
                        control.INTERNAL_GetVisualStateGroups().clear();
                    }

                    control.ApplyTemplate();
                }
            }
        },
        fields: {
            _renderedControlTemplate: null,
            _isDisabled: false,
            /**
             * Derived classes can set this flag in their constructor to prevent the "Template" property from being applied.
             *
             * @instance
             * @protected
             * @memberof Windows.UI.Xaml.Controls.Control
             * @default false
             * @type boolean
             */
            INTERNAL_DoNotApplyControlTemplate: false,
            /**
             * Derived classes can set this flag to True in their constructor in order to disable the "GoToState" calls of this class related to PointerOver/Pressed/Disabled, and handle them by themselves. An example is the ToggleButton control, which contains states such as "CheckedPressed", "CheckedPointerOver", etc.
             *
             * @instance
             * @protected
             * @memberof Windows.UI.Xaml.Controls.Control
             * @default false
             * @type boolean
             */
            DisableBaseControlHandlingOfVisualStates: false,
            INTERNAL_IsTemplated: false,
            _nameScopeDictionary: null,
            _useSystemFocusVisuals: false,
            _visualStateGroups: null,
            _isFocused: false,
            _isPointerOver: false,
            _isPressed: false
        },
        props: {
            /**
             * Gets or sets a brush that provides the background of the control.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.Control
             * @function Background
             * @type Windows.UI.Xaml.Media.Brush
             */
            Background: {
                get: function () {
                    return Bridge.cast(this.GetValue(Windows.UI.Xaml.Controls.Control.BackgroundProperty), Windows.UI.Xaml.Media.Brush);
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Controls.Control.BackgroundProperty, value);
                }
            },
            INTERNAL_IsLegacyVisualStates: {
                get: function () {
                    return this.INTERNAL_GetVisualStateGroups().ContainsVisualState(Windows.UI.Xaml.Controls.VisualStates.StateMouseOver);
                }
            },
            /**
             * Gets or sets a brush that describes the border background of a control.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.Control
             * @function BorderBrush
             * @type Windows.UI.Xaml.Media.Brush
             */
            BorderBrush: {
                get: function () {
                    return Bridge.cast(this.GetValue(Windows.UI.Xaml.Controls.Control.BorderBrushProperty), Windows.UI.Xaml.Media.Brush);
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Controls.Control.BorderBrushProperty, value);
                }
            },
            /**
             * Gets or sets the thickness of the border.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.Control
             * @function BorderThickness
             * @type Windows.UI.Xaml.Thickness
             */
            BorderThickness: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.Controls.Control.BorderThicknessProperty), Windows.UI.Xaml.Thickness), Windows.UI.Xaml.Thickness));
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Controls.Control.BorderThicknessProperty, value.$clone());
                }
            },
            /**
             * Gets or sets the thickness of the specified font.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.Control
             * @function FontWeight
             * @type Windows.UI.Text.FontWeight
             */
            FontWeight: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.Controls.Control.FontWeightProperty), Windows.UI.Text.FontWeight), Windows.UI.Text.FontWeight));
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Controls.Control.FontWeightProperty, value.$clone());
                }
            },
            /**
             * Gets or sets the style in which the text is rendered.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.Control
             * @function FontStyle
             * @type Windows.UI.Text.FontStyle
             */
            FontStyle: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.Controls.Control.FontStyleProperty), Windows.UI.Text.FontStyle), System.Int32));
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Controls.Control.FontStyleProperty, Bridge.box(value, Windows.UI.Text.FontStyle, System.Enum.toStringFn(Windows.UI.Text.FontStyle)));
                }
            },
            /**
             * Gets or sets a brush that describes the foreground color.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.Control
             * @function Foreground
             * @type Windows.UI.Xaml.Media.Brush
             */
            Foreground: {
                get: function () {
                    return Bridge.cast(this.GetValue(Windows.UI.Xaml.Controls.Control.ForegroundProperty), Windows.UI.Xaml.Media.Brush);
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Controls.Control.ForegroundProperty, value);
                }
            },
            /**
             * Gets or sets the font used to display text in the control.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.Control
             * @function FontFamily
             * @type Windows.UI.Xaml.Media.FontFamily
             */
            FontFamily: {
                get: function () {
                    return Bridge.cast(this.GetValue(Windows.UI.Xaml.Controls.Control.FontFamilyProperty), Windows.UI.Xaml.Media.FontFamily);
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Controls.Control.FontFamilyProperty, value);
                }
            },
            /**
             * Gets or sets the size of the text in this control.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.Control
             * @function FontSize
             * @type number
             */
            FontSize: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.Controls.Control.FontSizeProperty), System.Double), System.Double));
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Controls.Control.FontSizeProperty, Bridge.box(value, System.Double, System.Double.format, System.Double.getHashCode));
                }
            },
            /**
             * Gets or sets the text decorations (underline, strikethrough...).
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.Control
             * @function TextDecorations
             * @type ?Windows.UI.Text.TextDecorations
             */
            TextDecorations: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.Controls.Control.TextDecorationsProperty), Windows.UI.Text.TextDecorations), System.Int32, true));
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Controls.Control.TextDecorationsProperty, Bridge.box(value, Windows.UI.Text.TextDecorations, System.Nullable.toStringFn(System.Enum.toStringFn(Windows.UI.Text.TextDecorations)), System.Nullable.getHashCode));
                }
            },
            /**
             * Gets or sets the distance between the border and its child object.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.Control
             * @function Padding
             * @type Windows.UI.Xaml.Thickness
             */
            Padding: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.Controls.Control.PaddingProperty), Windows.UI.Xaml.Thickness), Windows.UI.Xaml.Thickness));
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Controls.Control.PaddingProperty, value.$clone());
                }
            },
            /**
             * Gets or sets the horizontal alignment of the control's content.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.Control
             * @function HorizontalContentAlignment
             * @type Windows.UI.Xaml.HorizontalAlignment
             */
            HorizontalContentAlignment: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.Controls.Control.HorizontalContentAlignmentProperty), Windows.UI.Xaml.HorizontalAlignment), System.Int32));
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Controls.Control.HorizontalContentAlignmentProperty, Bridge.box(value, Windows.UI.Xaml.HorizontalAlignment, System.Enum.toStringFn(Windows.UI.Xaml.HorizontalAlignment)));
                }
            },
            /**
             * Gets or sets the vertical alignment of the control's content.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.Control
             * @function VerticalContentAlignment
             * @type Windows.UI.Xaml.VerticalAlignment
             */
            VerticalContentAlignment: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.Controls.Control.VerticalContentAlignmentProperty), Windows.UI.Xaml.VerticalAlignment), System.Int32));
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Controls.Control.VerticalContentAlignmentProperty, Bridge.box(value, Windows.UI.Xaml.VerticalAlignment, System.Enum.toStringFn(Windows.UI.Xaml.VerticalAlignment)));
                }
            },
            /**
             * Gets or sets a value that determines the order in which elements receive
             focus when the user navigates through controls by pressing the Tab key.
             The default value is MaxValue
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.Control
             * @function TabIndex
             * @type number
             */
            TabIndex: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.Controls.Control.TabIndexProperty), System.Int32), System.Int32));
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Controls.Control.TabIndexProperty, Bridge.box(value, System.Int32));
                }
            },
            INTERNAL_GetFocusInBrowser: {
                get: function () {
                    return false;
                }
            },
            /**
             * Gets or sets a value that indicates whether a control is included in tab
             navigation.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.Control
             * @function IsTabStop
             * @type boolean
             */
            IsTabStop: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.Controls.Control.IsTabStopProperty), System.Boolean), System.Boolean));
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Controls.Control.IsTabStopProperty, Bridge.box(value, System.Boolean, System.Boolean.toString));
                }
            },
            /**
             * Gets or sets a control template.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.Control
             * @function Template
             * @type Windows.UI.Xaml.Controls.ControlTemplate
             */
            Template: {
                get: function () {
                    return Bridge.cast(this.GetValue(Windows.UI.Xaml.Controls.Control.TemplateProperty), Windows.UI.Xaml.Controls.ControlTemplate);
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Controls.Control.TemplateProperty, value);
                }
            },
            /**
             * Determines whether the control displays the browser's default outline when Focused.
             This property is ignored for Controls with a Template that defines the "Focused" VisualState.
             The default value is False.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.Control
             * @function UseSystemFocusVisuals
             * @type boolean
             */
            UseSystemFocusVisuals: {
                get: function () {
                    return this._useSystemFocusVisuals;
                },
                set: function (value) {
                    this._useSystemFocusVisuals = value;
                }
            },
            /**
             * Returns a value that indicates whether the control is to be rendered with a ControlTemplate.
             *
             * @instance
             * @readonly
             * @memberof Windows.UI.Xaml.Controls.Control
             * @function HasTemplate
             * @type boolean
             */
            HasTemplate: {
                get: function () {
                    return this.Template != null && this.INTERNAL_DoNotApplyControlTemplate === false;
                }
            }
        },
        ctors: {
            init: function () {
                this._isDisabled = false;
                this.INTERNAL_DoNotApplyControlTemplate = false;
                this.DisableBaseControlHandlingOfVisualStates = false;
                this.INTERNAL_IsTemplated = false;
                this._nameScopeDictionary = new (System.Collections.Generic.Dictionary$2(System.String,System.Object)).ctor();
                this._useSystemFocusVisuals = false;
                this._isFocused = false;
                this._isPointerOver = false;
                this._isPressed = false;
            }
        },
        methods: {
            ManageIsEnabled: function (isEnabled) {
                Windows.UI.Xaml.FrameworkElement.prototype.ManageIsEnabled.call(this, isEnabled);

                this.UpdateTabIndex(this.IsTabStop, this.TabIndex);

                this._isDisabled = !isEnabled;
                this.UpdateVisualStates();
            },
            UpdateTabIndex: function (isTabStop, tabIndex) {
                var domElementConcernedByFocus = this.INTERNAL_OptionalSpecifyDomElementConcernedByFocus || this.INTERNAL_OuterDomElement;
                if (!isTabStop || !this.IsEnabled) {
                    this.PreventFocusEvents();
                    CSHTML5.Internal.INTERNAL_HtmlDomManager.SetDomElementAttribute(domElementConcernedByFocus, "tabIndex", this.INTERNAL_GetFocusInBrowser ? "-1" : "");
                } else {

                    this.AllowFocusEvents();

                    var index;
                    if (tabIndex < (32667)) {
                        index = Math.max(((tabIndex + 100) | 0), 0);
                    } else {
                        index = Windows.UI.Xaml.Controls.Control.TABINDEX_BROWSER_MAX_VALUE;
                    }

                    CSHTML5.Internal.INTERNAL_HtmlDomManager.SetDomElementAttribute(domElementConcernedByFocus, "tabIndex", Bridge.toString(index));

                    if (!this.UseSystemFocusVisuals || this.INTERNAL_GetVisualStateGroups().ContainsVisualState("Focused")) {
                        CSHTML5.Internal.INTERNAL_HtmlDomManager.SetDomElementStyleProperty(domElementConcernedByFocus, function (_o1) {
                                _o1.add("outline");
                                return _o1;
                            }(new (System.Collections.Generic.List$1(System.String)).ctor()), "none");
                    }
                }
            },
            ApplyTemplate: function () {
                var visualsCreated = false;
                var visualChild = null;

                if (this.INTERNAL_IsTemplated && !this.INTERNAL_DoNotApplyControlTemplate && CSHTML5.Internal.INTERNAL_VisualTreeManager.IsElementInVisualTree(this)) {
                    var template = this.Template;

                    if (this._renderedControlTemplate == null) {
                        visualChild = template.INTERNAL_InstantiateFrameworkTemplate$1(this);
                        if (visualChild != null) {
                            visualsCreated = true;
                        }
                    } else {
                        visualChild = this._renderedControlTemplate;
                    }
                }

                CSHTML5.Internal.INTERNAL_VisualTreeManager.AttachVisualChildIfNotAlreadyAttached(visualChild, this);

                if (visualsCreated) {
                    this.OnApplyTemplate();

                    this._renderedControlTemplate = visualChild;
                }

                return visualsCreated;
            },
            INTERNAL_OnAttachedToVisualTree: function () {
                Windows.UI.Xaml.FrameworkElement.prototype.INTERNAL_OnAttachedToVisualTree.call(this);

                this.ApplyTemplate();
            },
            /**
             * Retrieves the named element in the instantiated ControlTemplate visual tree.
             *
             * @instance
             * @this Windows.UI.Xaml.Controls.Control
             * @memberof Windows.UI.Xaml.Controls.Control
             * @param   {string}                              childName    The name of the element to find.
             * @return  {Windows.UI.Xaml.DependencyObject}                 The named element from the template, if the element is found. Can return
             null if no element with name childName was found in the template.
             */
            GetTemplateChild: function (childName) {
                return Bridge.cast(this.TryFindTemplateChildFromName(childName), Windows.UI.Xaml.DependencyObject);
            },
            RaiseOnApplyTemplate: function () {
                this.OnApplyTemplate();
            },
            /**
             * Finds the UIElement with the specified name. Returns null if not found.
             *
             * @instance
             * @private
             * @this Windows.UI.Xaml.Controls.Control
             * @memberof Windows.UI.Xaml.Controls.Control
             * @param   {string}           name    The name to look for.
             * @return  {System.Object}            The object with the specified name if any; otherwise null.
             */
            TryFindTemplateChildFromName: function (name) {
                if (this._nameScopeDictionary.containsKey(name)) {
                    return this._nameScopeDictionary.getItem(name);
                } else {
                    return null;
                }
            },
            RegisterName: function (name, scopedElement) {
                if (this._nameScopeDictionary.containsKey(name) && !Bridge.referenceEquals(this._nameScopeDictionary.getItem(name), scopedElement)) {
                    throw new System.ArgumentException.$ctor1(System.String.format("Cannot register duplicate name '{0}' in this scope.", [name]));
                }

                this._nameScopeDictionary.setItem(name, scopedElement);
            },
            registerName: function (name, scopedElement) {
                this.RegisterName(name, scopedElement);
            },
            UnregisterName: function (name) {
                if (!this._nameScopeDictionary.containsKey(name)) {
                    throw new System.ArgumentException.$ctor1(System.String.format("Name '{0}' was not found.", [name]));
                }

                this._nameScopeDictionary.remove(name);
            },
            ClearRegisteredNames: function () {
                this._nameScopeDictionary.clear();
            },
            /**
             * Attempts to set the focus on the control.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Controls.Control
             * @memberof Windows.UI.Xaml.Controls.Control
             * @return  {boolean}        true if focus was set to the control, or focus was already on the control.
             false if the control is not focusable.
             */
            Focus: function () {
                if (this.IsTabStop) {
                    CSHTML5.Internal.INTERNAL_HtmlDomManager.SetFocus(this);
                    return true;
                }
                return false;
            },
            INTERNAL_GetVisualStateGroups: function () {
                if (this._visualStateGroups == null) {
                    this._visualStateGroups = new Windows.UI.Xaml.INTERNAL_VisualStateGroupCollection();
                }
                return this._visualStateGroups;
            },
            iNTERNAL_GetVisualStateGroups: function () {
                return this.INTERNAL_GetVisualStateGroups();
            },
            OnApplyTemplate: function () {
                Windows.UI.Xaml.FrameworkElement.prototype.OnApplyTemplate.call(this);

                if (!this.DisableBaseControlHandlingOfVisualStates) {
                    this.UpdateVisualStates();

                    if (this._visualStateGroups != null && this._visualStateGroups.ContainsVisualState("PointerOver")) {
                        this.removePointerEntered(Bridge.fn.cacheBind(this, this.Control_PointerEntered));
                        this.addPointerEntered(Bridge.fn.cacheBind(this, this.Control_PointerEntered));
                        this.removePointerExited(Bridge.fn.cacheBind(this, this.Control_PointerExited));
                        this.addPointerExited(Bridge.fn.cacheBind(this, this.Control_PointerExited));
                    }

                    if (this._visualStateGroups != null && this._visualStateGroups.ContainsVisualState("Pressed")) {
                        this.removePointerPressed(Bridge.fn.cacheBind(this, this.Control_PointerPressed));
                        this.addPointerPressed(Bridge.fn.cacheBind(this, this.Control_PointerPressed));
                        this.removePointerReleased(Bridge.fn.cacheBind(this, this.Control_PointerReleased));
                        this.addPointerReleased(Bridge.fn.cacheBind(this, this.Control_PointerReleased));
                    }

                    if (this._visualStateGroups != null && this._visualStateGroups.ContainsVisualState("Focused")) {
                        this.removeGotFocus(Bridge.fn.cacheBind(this, this.Control_GotFocus));
                        this.addGotFocus(Bridge.fn.cacheBind(this, this.Control_GotFocus));
                        this.removeLostFocus(Bridge.fn.cacheBind(this, this.Control_LostFocus));
                        this.addLostFocus(Bridge.fn.cacheBind(this, this.Control_LostFocus));
                    }
                }
            },
            Control_LostFocus: function (sender, e) {
                this._isFocused = false;
                this.UpdateVisualStatesForFocus();
            },
            Control_GotFocus: function (sender, e) {
                this._isFocused = true;
                this.UpdateVisualStatesForFocus();
            },
            Control_PointerEntered: function (sender, e) {
                this._isPointerOver = true;
                this.UpdateVisualStates();
            },
            Control_PointerExited: function (sender, e) {
                this._isPointerOver = false;
                this.UpdateVisualStates();
            },
            Control_PointerPressed: function (sender, e) {
                this._isPressed = true;
                this.UpdateVisualStates();
            },
            Control_PointerReleased: function (sender, e) {
                this._isPressed = false;
                this.UpdateVisualStates();
            },
            UpdateVisualStates: function () {
                if (!this.DisableBaseControlHandlingOfVisualStates) {
                    if (this._isDisabled) {
                        Windows.UI.Xaml.VisualStateManager.GoToState(this, "Disabled", true);
                    } else {
                        if (this._isPressed) {
                            Windows.UI.Xaml.VisualStateManager.GoToState(this, "Pressed", true);
                        } else {
                            if (this._isPointerOver) {
                                Windows.UI.Xaml.VisualStateManager.GoToState(this, "PointerOver", true);
                            } else {
                                Windows.UI.Xaml.VisualStateManager.GoToState(this, "Normal", true);
                            }
                        }
                    }


                }
            },
            UpdateVisualStatesForFocus: function () {
                if (!this.DisableBaseControlHandlingOfVisualStates) {
                    if (this._isFocused) {
                        Windows.UI.Xaml.VisualStateManager.GoToState(this, "Focused", true);
                    } else {
                        Windows.UI.Xaml.VisualStateManager.GoToState(this, "Unfocused", true);
                    }
                }
            },
            CreateDomElement: function (parentRef, domElementWhereToPlaceChildren) {
                return this.CreateDomElement_WorkaroundBridgeInheritanceBug(parentRef, domElementWhereToPlaceChildren);
            },
            /**
             * This method is here to avoid creating the dom for a control which has a Template.
             It creates the basic dom elements in which we will be able to add the template.
             *
             * @instance
             * @this Windows.UI.Xaml.Controls.Control
             * @memberof Windows.UI.Xaml.Controls.Control
             * @param   {System.Object}    parentRef                         The parent of the FrameworkElement
             * @param   {System.Object}    domElementWhereToPlaceChildren    The dom element where the FrameworkElement's template constructed children will be added.
             * @return  {System.Object}                                      The "root" dom element of the FrameworkElement.
             */
            CreateDomElementForControlTemplate: function (parentRef, domElementWhereToPlaceChildren) {
                return this.CreateDomElement_WorkaroundBridgeInheritanceBug(parentRef, domElementWhereToPlaceChildren);
            }
        }
    });

    /**
     * Draws a border, background, or both, around another object.
     *
     * @public
     * @class Windows.UI.Xaml.Controls.Border
     * @augments Windows.UI.Xaml.FrameworkElement
     * @example You can add a Border to the XAML as follows:
 
     Or in C# (assuming we have a StackPanel Named MyStackPanel):
     *
     * 
     *     
     * 
     * 
     *
     * Border myBorder = new Border();
     * myBorder.Width = 60;
     * myBorder.Height = 30;
     * myBorder.CornerRadius = new CornerRadius(15);
     * myBorder.Padding = new Thickness(20);
     * myBorder.Background = new SolidColorBrush(Windows.UI.Colors.Blue);
     * myBorder.HorizontalAlignment=HorizontalAlignment.Left;
     * MyStackPanel.Children.Add(myBorder);
     * 
     *
     *
     */
    Bridge.define("Windows.UI.Xaml.Controls.Border", {
        inherits: [Windows.UI.Xaml.FrameworkElement],
        statics: {
            fields: {
                /**
                 * Identifies the Background dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Controls.Border
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                BackgroundProperty: null,
                /**
                 * Identifies the BorderBrush dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Controls.Border
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                BorderBrushProperty: null,
                /**
                 * Identifies the BorderThickness dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Controls.Border
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                BorderThicknessProperty: null,
                /**
                 * Identifies the CornerRadius dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Controls.Border
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                CornerRadiusProperty: null,
                /**
                 * Identifies the Padding dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Controls.Border
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                PaddingProperty: null
            },
            ctors: {
                init: function () {
                    var $t;
                    this.BackgroundProperty = Windows.UI.Xaml.DependencyProperty.Register("Background", Windows.UI.Xaml.Media.Brush, Windows.UI.Xaml.Controls.Border, ($t = new Windows.UI.Xaml.PropertyMetadata.$ctor4(null), $t.GetCSSEquivalent = function (instance) {
                        var $t1;
                        return ($t1 = new Windows.UI.Xaml.CSSEquivalent(), $t1.Name = function (_o1) {
                                _o1.add("background");
                                _o1.add("backgroundColor");
                                _o1.add("backgroundColorAlpha");
                                return _o1;
                            }(new (System.Collections.Generic.List$1(System.String)).ctor()), $t1);
                    }, $t));
                    this.BorderBrushProperty = Windows.UI.Xaml.DependencyProperty.Register("BorderBrush", Windows.UI.Xaml.Media.Brush, Windows.UI.Xaml.Controls.Border, ($t = new Windows.UI.Xaml.PropertyMetadata.$ctor4(null), $t.GetCSSEquivalent = function (instance) {
                        var $t1;
                        return ($t1 = new Windows.UI.Xaml.CSSEquivalent(), $t1.Value = function (inst, value) {
                            if (value == null) {
                                return "transparent";
                            } else {
                                return value;
                            }
                        }, $t1.Name = function (_o2) {
                                _o2.add("borderColor");
                                return _o2;
                            }(new (System.Collections.Generic.List$1(System.String)).ctor()), $t1);
                    }, $t));
                    this.BorderThicknessProperty = Windows.UI.Xaml.DependencyProperty.Register("BorderThickness", Windows.UI.Xaml.Thickness, Windows.UI.Xaml.Controls.Border, ($t = new Windows.UI.Xaml.PropertyMetadata.$ctor1(new Windows.UI.Xaml.Thickness.ctor().$clone()), $t.MethodToUpdateDom = Windows.UI.Xaml.Controls.Border.BorderThickness_MethodToUpdateDom, $t));
                    this.CornerRadiusProperty = Windows.UI.Xaml.DependencyProperty.Register("CornerRadius", Windows.UI.Xaml.CornerRadius, Windows.UI.Xaml.Controls.Border, ($t = new Windows.UI.Xaml.PropertyMetadata.$ctor1(new Windows.UI.Xaml.CornerRadius.ctor().$clone()), $t.MethodToUpdateDom = Windows.UI.Xaml.Controls.Border.CornerRadius_MethodToUpdateDom, $t));
                    this.PaddingProperty = Windows.UI.Xaml.DependencyProperty.Register("Padding", Windows.UI.Xaml.Thickness, Windows.UI.Xaml.Controls.Border, ($t = new Windows.UI.Xaml.PropertyMetadata.$ctor1(new Windows.UI.Xaml.Thickness.ctor().$clone()), $t.MethodToUpdateDom = Windows.UI.Xaml.Controls.Border.Padding_MethodToUpdateDom, $t));
                }
            },
            methods: {
                BorderThickness_MethodToUpdateDom: function (d, newValue) {
                    if (newValue != null) {
                        var border = Bridge.cast(d, Windows.UI.Xaml.Controls.Border);
                        var domElement = CSHTML5.Internal.INTERNAL_HtmlDomManager.GetFrameworkElementOuterStyleForModification(border);
                        var thickness = System.Nullable.getValue(Bridge.cast(Bridge.unbox(newValue, Windows.UI.Xaml.Thickness), Windows.UI.Xaml.Thickness));
                        domElement.borderStyle = "solid";
                        domElement.borderWidth = System.Double.format(thickness.Top) + "px " + System.Double.format(thickness.Right) + "px " + System.Double.format(thickness.Bottom) + "px " + System.Double.format(thickness.Left) + "px ";
                        domElement.boxSizing = "border-box";
                    }
                },
                CornerRadius_MethodToUpdateDom: function (d, newValue) {
                    var border = Bridge.cast(d, Windows.UI.Xaml.Controls.Border);
                    var cornerRadius = System.Nullable.getValue(Bridge.cast(Bridge.unbox(newValue, Windows.UI.Xaml.CornerRadius), Windows.UI.Xaml.CornerRadius));
                    var domStyle = CSHTML5.Internal.INTERNAL_HtmlDomManager.GetFrameworkElementOuterStyleForModification(border);
                    domStyle.borderTopLeftRadius = System.Double.format(cornerRadius.TopLeft) + "px";
                    domStyle.borderTopRightRadius = System.Double.format(cornerRadius.TopRight) + "px";
                    domStyle.borderBottomRightRadius = System.Double.format(cornerRadius.BottomRight) + "px";
                    domStyle.borderBottomLeftRadius = System.Double.format(cornerRadius.BottomLeft) + "px";
                },
                Padding_MethodToUpdateDom: function (d, newValue) {
                    var border = Bridge.cast(d, Windows.UI.Xaml.Controls.Border);
                    var newPadding = System.Nullable.getValue(Bridge.cast(Bridge.unbox(newValue, Windows.UI.Xaml.Thickness), Windows.UI.Xaml.Thickness));
                    var innerDomElement = border.INTERNAL_InnerDomElement;
                    var styleOfInnerDomElement = CSHTML5.Internal.INTERNAL_HtmlDomManager.GetDomElementStyleForModification(innerDomElement);
                    if (System.Nullable.lifteq(Windows.UI.Xaml.Thickness.op_Equality, newPadding.$clone(), null)) {
                        newPadding = new Windows.UI.Xaml.Thickness.ctor();
                    }
                    styleOfInnerDomElement.boxSizing = "border-box";
                    styleOfInnerDomElement.paddingLeft = System.Double.format(newPadding.Left) + "px";
                    styleOfInnerDomElement.paddingTop = System.Double.format(newPadding.Top) + "px";
                    styleOfInnerDomElement.paddingRight = System.Double.format(newPadding.Right) + "px";
                    styleOfInnerDomElement.paddingBottom = System.Double.format(newPadding.Bottom) + "px";
                }
            }
        },
        fields: {
            _child: null
        },
        props: {
            /**
             * Gets or sets the child element to draw the border around.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.Border
             * @function Child
             * @type Windows.UI.Xaml.UIElement
             */
            Child: {
                get: function () {
                    return this._child;
                },
                set: function (value) {
                    if (this._isLoaded) {
                        CSHTML5.Internal.INTERNAL_VisualTreeManager.DetachVisualChildIfNotNull(this._child, this);
                        CSHTML5.Internal.INTERNAL_VisualTreeManager.AttachVisualChildIfNotAlreadyAttached(value, this);
                    }
                    this._child = value;
                }
            },
            /**
             * Gets or sets the Brush that fills the background of the border.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.Border
             * @function Background
             * @type Windows.UI.Xaml.Media.Brush
             */
            Background: {
                get: function () {
                    return Bridge.cast(this.GetValue(Windows.UI.Xaml.Controls.Border.BackgroundProperty), Windows.UI.Xaml.Media.Brush);
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Controls.Border.BackgroundProperty, value);
                }
            },
            /**
             * Gets or sets a brush that describes the border background of a control.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.Border
             * @function BorderBrush
             * @type Windows.UI.Xaml.Media.Brush
             */
            BorderBrush: {
                get: function () {
                    return Bridge.cast(this.GetValue(Windows.UI.Xaml.Controls.Border.BorderBrushProperty), Windows.UI.Xaml.Media.Brush);
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Controls.Border.BorderBrushProperty, value);
                }
            },
            /**
             * Gets or sets the thickness of the border.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.Border
             * @function BorderThickness
             * @type Windows.UI.Xaml.Thickness
             */
            BorderThickness: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.Controls.Border.BorderThicknessProperty), Windows.UI.Xaml.Thickness), Windows.UI.Xaml.Thickness));
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Controls.Border.BorderThicknessProperty, value.$clone());
                }
            },
            /**
             * Gets or sets the radius for the corners of the border.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.Border
             * @function CornerRadius
             * @type Windows.UI.Xaml.CornerRadius
             */
            CornerRadius: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.Controls.Border.CornerRadiusProperty), Windows.UI.Xaml.CornerRadius), Windows.UI.Xaml.CornerRadius));
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Controls.Border.CornerRadiusProperty, value.$clone());
                }
            },
            /**
             * Gets or sets the distance between the border and its child object.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.Border
             * @function Padding
             * @type Windows.UI.Xaml.Thickness
             */
            Padding: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.Controls.Border.PaddingProperty), Windows.UI.Xaml.Thickness), Windows.UI.Xaml.Thickness));
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Controls.Border.PaddingProperty, value.$clone());
                }
            }
        },
        methods: {
            INTERNAL_OnAttachedToVisualTree: function () {
                Windows.UI.Xaml.FrameworkElement.prototype.INTERNAL_OnAttachedToVisualTree.call(this);

                if (this.BorderBrush == null) {
                    CSHTML5.Internal.INTERNAL_PropertyStore.ApplyCssChanges(null, null, Windows.UI.Xaml.Controls.Border.BorderBrushProperty.GetMetadata(Windows.UI.Xaml.Controls.Border), this);
                }

                if (!this.INTERNAL_EnableProgressiveLoading) {
                    CSHTML5.Internal.INTERNAL_VisualTreeManager.AttachVisualChildIfNotAlreadyAttached(this._child, this);
                } else {
                    this.ProgressivelyAttachChild();
                }
            },
            ProgressivelyAttachChild: function () {
                var $step = 0,
                    $task1, 
                    $jumpFromFinally, 
                    $asyncBody = Bridge.fn.bind(this, function () {
                        for (;;) {
                            $step = System.Array.min([0,1], $step);
                            switch ($step) {
                                case 0: {
                                    $task1 = System.Threading.Tasks.Task.delay(1);
                                    $step = 1;
                                    if ($task1.isCompleted()) {
                                        continue;
                                    }
                                    $task1.continue($asyncBody);
                                    return;
                                }
                                case 1: {
                                    $task1.getAwaitedResult();
                                    if (!CSHTML5.Internal.INTERNAL_VisualTreeManager.IsElementInVisualTree(this)) {
                                        return;
                                    }
                                    CSHTML5.Internal.INTERNAL_VisualTreeManager.AttachVisualChildIfNotAlreadyAttached(this._child, this);
                                    return;
                                }
                                default: {
                                    return;
                                }
                            }
                        }
                    }, arguments);

                $asyncBody();
            }
        }
    });

    Bridge.define("Windows.UI.Xaml.Controls.INTERNAL_CalendarOrClockBase", {
        inherits: [Windows.UI.Xaml.FrameworkElement],
        statics: {
            fields: {
                /**
                 * Identifies the dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Controls.INTERNAL_CalendarOrClockBase
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                SelectedValueProperty: null
            },
            ctors: {
                init: function () {
                    var $t;
                    this.SelectedValueProperty = Windows.UI.Xaml.DependencyProperty.Register("SelectedValue", System.Nullable$1(System.DateTime), Windows.UI.Xaml.Controls.INTERNAL_CalendarOrClockBase, ($t = new Windows.UI.Xaml.PropertyMetadata.$ctor4(Windows.UI.Xaml.Controls.INTERNAL_CalendarOrClockBase.OnSelectedValueChanged), $t.CallPropertyChangedWhenLoadedIntoVisualTree = Windows.UI.Xaml.WhenToCallPropertyChangedEnum.IfPropertyIsSet, $t));
                }
            },
            methods: {
                /**
                 * SelectedValueProperty property changed handler.
                 *
                 * @static
                 * @private
                 * @this Windows.UI.Xaml.Controls.INTERNAL_CalendarOrClockBase
                 * @memberof Windows.UI.Xaml.Controls.INTERNAL_CalendarOrClockBase
                 * @param   {Windows.UI.Xaml.DependencyObject}                      d    the calender or clock that changed its SelectedDate.
                 * @param   {Windows.UI.Xaml.DependencyPropertyChangedEventArgs}    e    The DependencyPropertyChangedEventArgs.
                 * @return  {void}
                 */
                OnSelectedValueChanged: function (d, e) {
                    if (!Bridge.referenceEquals(e.NewValue, e.OldValue)) {
                        var calendarOrClock = Bridge.cast(d, Windows.UI.Xaml.Controls.INTERNAL_CalendarOrClockBase);
                        if (e.NewValue != null) {
                            var newDate = System.Nullable.getValue(Bridge.cast(Bridge.unbox(e.NewValue, System.DateTime), System.DateTime));

                            var newSelectedDate = new Windows.UI.Xaml.Controls.SelectedDatesCollection(calendarOrClock);
                            newSelectedDate.SetDate(newDate);

                            if (calendarOrClock._flatpickrInstance != null) {
                                var newDateJS = new Date(Bridge.box(System.DateTime.getYear(newDate), System.Int32), Bridge.box(((System.DateTime.getMonth(newDate) - 1) | 0), System.Int32), Bridge.box(System.DateTime.getDay(newDate), System.Int32), Bridge.box(System.DateTime.getHour(newDate), System.Int32), Bridge.box(System.DateTime.getMinute(newDate), System.Int32));

                                calendarOrClock._flatpickrInstance.setDate(newDateJS);
                            }
                        } else {
                            if (calendarOrClock._flatpickrInstance != null) {
                                calendarOrClock._flatpickrInstance.setDate(undefined);
                            }
                        }
                    }
                }
            }
        },
        fields: {
            _flatpickrInstance: null
        },
        events: {
            SelectionChanged: null,
            /**
             * Occurs when the collection returned by the property has changed.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Controls.INTERNAL_CalendarOrClockBase
             * @memberof Windows.UI.Xaml.Controls.INTERNAL_CalendarOrClockBase
             * @function addSelectedDatesChanged
             * @param   {System.EventHandler}    value
             * @return  {void}
             */
            /**
             * Occurs when the collection returned by the property has changed.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Controls.INTERNAL_CalendarOrClockBase
             * @memberof Windows.UI.Xaml.Controls.INTERNAL_CalendarOrClockBase
             * @function removeSelectedDatesChanged
             * @param   {System.EventHandler}    value
             * @return  {void}
             */
            SelectedDatesChanged: null
        },
        props: {
            /**
             * Gets or sets the currently selected date or time.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.INTERNAL_CalendarOrClockBase
             * @function SelectedValue
             * @type ?System.DateTime
             */
            SelectedValue: {
                get: function () {
                    return Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.Controls.INTERNAL_CalendarOrClockBase.SelectedValueProperty), System.DateTime), System.DateTime, true);
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Controls.INTERNAL_CalendarOrClockBase.SelectedValueProperty, Bridge.box(value, System.DateTime, System.Nullable.toStringFn(System.DateTime.format), System.Nullable.getHashCode));
                }
            }
        },
        ctors: {
            ctor: function () {
                this.$initialize();
                Windows.UI.Xaml.FrameworkElement.ctor.call(this);
                this.SelectedValue = null;
            }
        },
        methods: {
            OnSelectedDatesCollectionChanged: function (e) {
                if (!Bridge.staticEquals(this.SelectionChanged, null)) {
                    this.SelectionChanged(this, e);
                }
                if (!Bridge.staticEquals(this.SelectedDatesChanged, null)) {
                    this.SelectedDatesChanged(this, e);
                }
            }
        }
    });

    /**
     * Provides a base class for all Panel elements. Use Panel elements to position
     and arrange child objects in a UI page.
     *
     * @abstract
     * @public
     * @class Windows.UI.Xaml.Controls.Panel
     * @augments Windows.UI.Xaml.FrameworkElement
     */
    Bridge.define("Windows.UI.Xaml.Controls.Panel", {
        inherits: [Windows.UI.Xaml.FrameworkElement],
        statics: {
            fields: {
                INTERNAL_ApplicationWideEnableProgressiveRendering: false,
                /**
                 * Identifies the Background dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Controls.Panel
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                BackgroundProperty: null,
                IsItemsHostProperty: null
            },
            ctors: {
                init: function () {
                    var $t;
                    this.BackgroundProperty = Windows.UI.Xaml.DependencyProperty.Register("Background", Windows.UI.Xaml.Media.Brush, Windows.UI.Xaml.Controls.Panel, ($t = new Windows.UI.Xaml.PropertyMetadata.$ctor4(null), $t.GetCSSEquivalent = function (instance) {
                        var $t1;
                        return ($t1 = new Windows.UI.Xaml.CSSEquivalent(), $t1.Name = function (_o1) {
                                _o1.add("background");
                                _o1.add("backgroundColor");
                                _o1.add("backgroundColorAlpha");
                                return _o1;
                            }(new (System.Collections.Generic.List$1(System.String)).ctor()), $t1);
                    }, $t.CallPropertyChangedWhenLoadedIntoVisualTree = Windows.UI.Xaml.WhenToCallPropertyChangedEnum.IfPropertyIsSet, $t));
                    this.IsItemsHostProperty = Windows.UI.Xaml.DependencyProperty.Register("IsItemsHost", System.Boolean, Windows.UI.Xaml.Controls.Panel, new Windows.UI.Xaml.PropertyMetadata.$ctor1(Bridge.box(false, System.Boolean, System.Boolean.toString)));
                }
            }
        },
        fields: {
            _children: null,
            _enableProgressiveRendering: false,
            _nameScopeDictionary: null
        },
        props: {
            EnableProgressiveRendering: {
                get: function () {
                    return this._enableProgressiveRendering || Windows.UI.Xaml.Controls.Panel.INTERNAL_ApplicationWideEnableProgressiveRendering;
                },
                set: function (value) {
                    this._enableProgressiveRendering = value;
                }
            },
            /**
             * Gets or sets a Brush that is used to fill the panel.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.Panel
             * @function Background
             * @type Windows.UI.Xaml.Media.Brush
             */
            Background: {
                get: function () {
                    return Bridge.cast(this.GetValue(Windows.UI.Xaml.Controls.Panel.BackgroundProperty), Windows.UI.Xaml.Media.Brush);
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Controls.Panel.BackgroundProperty, value);
                }
            },
            /**
             * Gets the collection of child elements of the panel.
             *
             * @instance
             * @public
             * @readonly
             * @memberof Windows.UI.Xaml.Controls.Panel
             * @function Children
             * @type Windows.UI.Xaml.Controls.UIElementCollection
             */
            Children: {
                get: function () {
                    if (this._children == null) {
                        this._children = new Windows.UI.Xaml.Controls.UIElementCollection();

                        if (this._isLoaded) {
                            this._children.addCollectionChanged(Bridge.fn.cacheBind(this, this.OnChildrenCollectionChanged));
                        }
                    }

                    return this._children;
                }
            },
            IsItemsHost: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.Controls.Panel.IsItemsHostProperty), System.Boolean), System.Boolean));
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Controls.Panel.IsItemsHostProperty, Bridge.box(value, System.Boolean, System.Boolean.toString));
                }
            }
        },
        ctors: {
            init: function () {
                this._nameScopeDictionary = new (System.Collections.Generic.Dictionary$2(System.String,System.Object)).ctor();
            }
        },
        methods: {
            OnChildrenCollectionChanged: function (sender, e) {
                switch (e.Action) {
                    case System.Collections.Specialized.NotifyCollectionChangedAction.Reset: 
                        this.OnChildrenReset();
                        break;
                    case System.Collections.Specialized.NotifyCollectionChangedAction.Add: 
                        System.Diagnostics.Debug.Assert(System.Array.getCount(e.NewItems) === 1);
                        this.OnChildrenAdded(Bridge.cast(System.Array.getItem(e.NewItems, 0), Windows.UI.Xaml.UIElement), e.NewStartingIndex);
                        break;
                    case System.Collections.Specialized.NotifyCollectionChangedAction.Remove: 
                        System.Diagnostics.Debug.Assert(System.Array.getCount(e.OldItems) === 1);
                        this.OnChildrenRemoved(Bridge.cast(System.Array.getItem(e.OldItems, 0), Windows.UI.Xaml.UIElement), e.OldStartingIndex);
                        break;
                    case System.Collections.Specialized.NotifyCollectionChangedAction.Replace: 
                        System.Diagnostics.Debug.Assert(System.Array.getCount(e.OldItems) === 1 && System.Array.getCount(e.NewItems) === 1);
                        this.OnChildrenReplaced(Bridge.cast(System.Array.getItem(e.OldItems, 0), Windows.UI.Xaml.UIElement), Bridge.cast(System.Array.getItem(e.NewItems, 0), Windows.UI.Xaml.UIElement), e.OldStartingIndex);
                        break;
                    case System.Collections.Specialized.NotifyCollectionChangedAction.Move: 
                        System.Diagnostics.Debug.Assert(System.Array.getCount(e.OldItems) === 1);
                        this.OnChildrenMoved(Bridge.cast(System.Array.getItem(e.OldItems, 0), Windows.UI.Xaml.UIElement), e.NewStartingIndex, e.OldStartingIndex);
                        break;
                }
            },
            OnChildrenReset: function () {
                var $t;
                if (this.INTERNAL_VisualChildrenInformation != null) {
                    $t = Bridge.getEnumerator(System.Linq.Enumerable.from(System.Linq.Enumerable.from(this.INTERNAL_VisualChildrenInformation).select(function (kp) {
                                return kp.value;
                            })).ToArray());
                    try {
                        while ($t.moveNext()) {
                            var childInfo = $t.Current;
                            CSHTML5.Internal.INTERNAL_VisualTreeManager.DetachVisualChildIfNotNull(childInfo.INTERNAL_UIElement, this);
                        }
                    } finally {
                        if (Bridge.is($t, System.IDisposable)) {
                            $t.System$IDisposable$Dispose();
                        }
                    }
                }

                if (this._enableProgressiveRendering || this.INTERNAL_EnableProgressiveLoading) {
                    this.ProgressivelyAttachChildren(this.Children);
                } else {
                    for (var i = 0; i < this.Children.Count; i = (i + 1) | 0) {
                        CSHTML5.Internal.INTERNAL_VisualTreeManager.AttachVisualChildIfNotAlreadyAttached(this.Children.getItem(i), this, i);
                    }
                }
            },
            OnChildrenAdded: function (newChild, index) {
                CSHTML5.Internal.INTERNAL_VisualTreeManager.AttachVisualChildIfNotAlreadyAttached(newChild, this, index);
            },
            OnChildrenRemoved: function (oldChild, index) {
                CSHTML5.Internal.INTERNAL_VisualTreeManager.DetachVisualChildIfNotNull(oldChild, this);
            },
            OnChildrenReplaced: function (oldChild, newChild, index) {
                if (Bridge.referenceEquals(oldChild, newChild)) {
                    return;
                }

                CSHTML5.Internal.INTERNAL_VisualTreeManager.DetachVisualChildIfNotNull(oldChild, this);

                CSHTML5.Internal.INTERNAL_VisualTreeManager.AttachVisualChildIfNotAlreadyAttached(newChild, this, index);
            },
            OnChildrenMoved: function (oldChild, newIndex, oldIndex) {
                if (newIndex === oldIndex) {
                    return;
                }

                CSHTML5.Internal.INTERNAL_VisualTreeManager.MoveVisualChildInSameParent$1(oldChild, this, newIndex, oldIndex);
            },
            INTERNAL_OnAttachedToVisualTree: function () {
                Windows.UI.Xaml.FrameworkElement.prototype.INTERNAL_OnAttachedToVisualTree.call(this);

                if (this._children != null) {
                    this._children.removeCollectionChanged(Bridge.fn.cacheBind(this, this.OnChildrenCollectionChanged));
                    this._children.addCollectionChanged(Bridge.fn.cacheBind(this, this.OnChildrenCollectionChanged));
                }

                this.OnChildrenReset();
            },
            INTERNAL_OnDetachedFromVisualTree: function () {
                Windows.UI.Xaml.FrameworkElement.prototype.INTERNAL_OnDetachedFromVisualTree.call(this);

                if (this._children != null) {
                    this._children.removeCollectionChanged(Bridge.fn.cacheBind(this, this.OnChildrenCollectionChanged));
                }
            },
            ProgressivelyAttachChildren: function (newChildren) {
                var $step = 0,
                    $task1, 
                    $jumpFromFinally, 
                    i, 
                    $asyncBody = Bridge.fn.bind(this, function () {
                        for (;;) {
                            $step = System.Array.min([0,1,2,3,4,5], $step);
                            switch ($step) {
                                case 0: {
                                    i = 0;
                                    $step = 1;
                                    continue;
                                }
                                case 1: {
                                    if ( i < System.Array.getCount(newChildren, Windows.UI.Xaml.UIElement) ) {
                                        $step = 2;
                                        continue;
                                    }
                                    $step = 5;
                                    continue;
                                }
                                case 2: {
                                    $task1 = System.Threading.Tasks.Task.delay(1);
                                    $step = 3;
                                    if ($task1.isCompleted()) {
                                        continue;
                                    }
                                    $task1.continue($asyncBody);
                                    return;
                                }
                                case 3: {
                                    $task1.getAwaitedResult();
                                    if (!CSHTML5.Internal.INTERNAL_VisualTreeManager.IsElementInVisualTree(this)) {
                                        $step = 5;
                                        continue;
                                    }
                                    CSHTML5.Internal.INTERNAL_VisualTreeManager.AttachVisualChildIfNotAlreadyAttached(System.Array.getItem(newChildren, i, Windows.UI.Xaml.UIElement), this);
                                    this.INTERNAL_OnChildProgressivelyLoaded();
                                    $step = 4;
                                    continue;
                                }
                                case 4: {
                                    i = (i + 1) | 0;
                                    $step = 1;
                                    continue;
                                }
                                case 5: {
                                    return;
                                }
                                default: {
                                    return;
                                }
                            }
                        }
                    }, arguments);

                $asyncBody();
            },
            INTERNAL_OnChildProgressivelyLoaded: function () { },
            /**
             * Retrieves the named element in the instantiated ControlTemplate visual tree.
             *
             * @instance
             * @this Windows.UI.Xaml.Controls.Panel
             * @memberof Windows.UI.Xaml.Controls.Panel
             * @param   {string}                              childName    The name of the element to find.
             * @return  {Windows.UI.Xaml.DependencyObject}                 The named element from the template, if the element is found. Can return
             null if no element with name childName was found in the template.
             */
            GetTemplateChild: function (childName) {
                return Bridge.cast(this.TryFindTemplateChildFromName(childName), Windows.UI.Xaml.DependencyObject);
            },
            /**
             * Finds the UIElement with the specified name. Returns null if not found.
             *
             * @instance
             * @private
             * @this Windows.UI.Xaml.Controls.Panel
             * @memberof Windows.UI.Xaml.Controls.Panel
             * @param   {string}           name    The name to look for.
             * @return  {System.Object}            The object with the specified name if any; otherwise null.
             */
            TryFindTemplateChildFromName: function (name) {
                if (this._nameScopeDictionary.containsKey(name)) {
                    return this._nameScopeDictionary.getItem(name);
                } else {
                    return null;
                }
            },
            RegisterName: function (name, scopedElement) {
                if (this._nameScopeDictionary.containsKey(name) && !Bridge.referenceEquals(this._nameScopeDictionary.getItem(name), scopedElement)) {
                    throw new System.ArgumentException.$ctor1(System.String.format("Cannot register duplicate name '{0}' in this scope.", [name]));
                }

                this._nameScopeDictionary.setItem(name, scopedElement);
            },
            UnregisterName: function (name) {
                if (!this._nameScopeDictionary.containsKey(name)) {
                    throw new System.ArgumentException.$ctor1(System.String.format("Name '{0}' was not found.", [name]));
                }

                this._nameScopeDictionary.remove(name);
            },
            ClearRegisteredNames: function () {
                this._nameScopeDictionary.clear();
            }
        }
    });

    /**
     * Represents a System.Windows.Controls.DataGrid column that hosts System.Windows.Controls.CheckBox
     controls in its cells.
     *
     * @public
     * @class Windows.UI.Xaml.Controls.DataGridCheckBoxColumn
     * @augments Windows.UI.Xaml.Controls.DataGridBoundColumn
     */
    Bridge.define("Windows.UI.Xaml.Controls.DataGridCheckBoxColumn", {
        inherits: [Windows.UI.Xaml.Controls.DataGridBoundColumn],
        methods: {
            GenerateEditingElement: function (childData) {
                return null;
            },
            GenerateElement: function (childData) {
                return this.GenerateElement$1(childData, false);
            },
            GenerateElement$1: function (childData, enable) {
                var checkBox = new Windows.UI.Xaml.Controls.CheckBox();
                checkBox.DataContext = childData;
                checkBox.IsEnabled = enable;
                var b = Bridge.as(this.Binding, Windows.UI.Xaml.Data.Binding);

                if (b != null) {
                    if (b.Mode === Windows.UI.Xaml.Data.BindingMode.OneWay) {
                        if (!b.INTERNAL_WasModeSetByUserRatherThanDefaultValue()) {
                            b.Mode = Windows.UI.Xaml.Data.BindingMode.TwoWay;
                        }
                    }
                    checkBox.SetBinding(Windows.UI.Xaml.Controls.Primitives.ToggleButton.IsCheckedProperty, b);
                } else if (Bridge.is(childData, System.Boolean)) {
                    checkBox.IsChecked = System.Nullable.getValue(Bridge.cast(Bridge.unbox(childData, System.Boolean), System.Boolean));
                }
                return checkBox;
            },
            EnterEditionMode: function (dataGridCell) {
                Bridge.cast(dataGridCell.Content, Windows.UI.Xaml.FrameworkElement).IsEnabled = true;
            },
            LeaveEditionMode: function (dataGridCell) {
                Bridge.cast(dataGridCell.Content, Windows.UI.Xaml.FrameworkElement).IsEnabled = false;
            }
        }
    });

    /**
     * Represents a System.Windows.Controls.DataGrid column that hosts System.Windows.Controls.ComboBox
     controls in its cells.
     *
     * @class Windows.UI.Xaml.Controls.DataGridComboBoxColumn
     * @augments Windows.UI.Xaml.Controls.DataGridBoundColumn
     */
    Bridge.define("Windows.UI.Xaml.Controls.DataGridComboBoxColumn", {
        inherits: [Windows.UI.Xaml.Controls.DataGridBoundColumn],
        methods: {
            GenerateEditingElement: function (childData) {
                var $t;
                var b = Bridge.as(this.Binding, Windows.UI.Xaml.Data.Binding);
                if (b.Mode === Windows.UI.Xaml.Data.BindingMode.OneWay) {
                    if (!b.INTERNAL_WasModeSetByUserRatherThanDefaultValue()) {
                        b.Mode = Windows.UI.Xaml.Data.BindingMode.TwoWay;
                    }
                }
                var value = Windows.UI.Xaml.PropertyPathHelper.AccessValueByApplyingPropertyPathIfAny(childData, b.Path.Path);
                if (!(Bridge.is(value, System.Enum))) {
                    throw new System.NotImplementedException.$ctor1("DataGridComboBoxColumns currently only works with Enum types");
                }
                var enumType = Bridge.getType(value);
                var enumValues = System.Enum.getValues(enumType);
                var cb = new CSHTML5.Native.Html.Controls.NativeComboBox();
                cb.DataContext = childData;
                if (enumValues != null && enumValues.length > 0) {
                    var enumValuesAsStrings = new (System.Collections.Generic.List$1(System.String)).ctor();
                    var valueType = Bridge.getType(value);
                    $t = Bridge.getEnumerator(enumValues);
                    try {
                        while ($t.moveNext()) {
                            var val = $t.Current;
                            enumValuesAsStrings.add(System.Enum.getName(valueType, val));
                        }
                    } finally {
                        if (Bridge.is($t, System.IDisposable)) {
                            $t.System$IDisposable$Dispose();
                        }
                    }
                    cb.ItemsSource = enumValuesAsStrings;

                    b.Converter = new Windows.UI.Xaml.Controls.DataGridComboBoxColumn.MyConverter();
                    b.ConverterParameter = valueType;

                    cb.SetBinding(CSHTML5.Native.Html.Controls.NativeComboBox.SelectedItemProperty, b);
                }
                return cb;
            },
            GenerateElement: function (childData) {
                var textBlock = new Windows.UI.Xaml.Controls.TextBlock();
                var b = Bridge.as(this.Binding, Windows.UI.Xaml.Data.Binding);
                if (b != null) {
                    if (b.Mode === Windows.UI.Xaml.Data.BindingMode.OneWay) {
                        if (!b.INTERNAL_WasModeSetByUserRatherThanDefaultValue()) {
                            b.Mode = Windows.UI.Xaml.Data.BindingMode.TwoWay;
                        }
                    }
                    textBlock.SetBinding(Windows.UI.Xaml.Controls.TextBlock.TextProperty, b);
                } else if (Bridge.is(childData, System.String)) {
                    textBlock.Text = Bridge.cast(childData, System.String);
                }
                return textBlock;
            }
        }
    });

    /**
     * Represents a System.Windows.Controls.DataGrid column that hosts System.Uri
     elements in its cells.
     *
     * @public
     * @class Windows.UI.Xaml.Controls.DataGridHyperlinkColumn
     * @augments Windows.UI.Xaml.Controls.DataGridBoundColumn
     */
    Bridge.define("Windows.UI.Xaml.Controls.DataGridHyperlinkColumn", {
        inherits: [Windows.UI.Xaml.Controls.DataGridBoundColumn],
        statics: {
            fields: {
                /**
                 * Identifies the System.Windows.Controls.DataGridBoundColumn.Binding dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Controls.DataGridHyperlinkColumn
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                ContentBindingProperty: null
            },
            ctors: {
                init: function () {
                    var $t;
                    this.ContentBindingProperty = Windows.UI.Xaml.DependencyProperty.Register("ContentBinding", Windows.UI.Xaml.Data.BindingBase, Windows.UI.Xaml.Controls.DataGridBoundColumn, ($t = new Windows.UI.Xaml.PropertyMetadata.$ctor2(null, Windows.UI.Xaml.Controls.DataGridHyperlinkColumn.ContentBinding_Changed), $t.CallPropertyChangedWhenLoadedIntoVisualTree = Windows.UI.Xaml.WhenToCallPropertyChangedEnum.IfPropertyIsSet, $t));
                }
            },
            methods: {
                ContentBinding_Changed: function (d, e) { }
            }
        },
        props: {
            /**
             * Gets or sets the binding to the text of the hyperlink.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.DataGridHyperlinkColumn
             * @function ContentBinding
             * @type Windows.UI.Xaml.Data.BindingBase
             */
            ContentBinding: {
                get: function () {
                    return Bridge.cast(this.GetValue(Windows.UI.Xaml.Controls.DataGridHyperlinkColumn.ContentBindingProperty), Windows.UI.Xaml.Data.BindingBase);
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Controls.DataGridHyperlinkColumn.ContentBindingProperty, value);
                }
            }
        },
        methods: {
            GenerateEditingElement: function (childData) {

                var textBox = new Windows.UI.Xaml.Controls.TextBox();
                textBox.DataContext = childData;
                var b = Bridge.as(this.Binding, Windows.UI.Xaml.Data.Binding);
                if (b != null) {
                    if (b.Mode === Windows.UI.Xaml.Data.BindingMode.OneWay) {
                        if (!b.INTERNAL_WasModeSetByUserRatherThanDefaultValue()) {
                            b.Mode = Windows.UI.Xaml.Data.BindingMode.TwoWay;
                        }
                    }
                    textBox.SetBinding(Windows.UI.Xaml.Controls.TextBox.TextProperty, b);
                } else if (Bridge.is(childData, System.String)) {
                    textBox.Text = Bridge.cast(childData, System.String);
                }
                return textBox;
            },
            GenerateElement: function (childData) {
                var hyperlink = new Windows.UI.Xaml.Controls.HyperlinkButton();
                hyperlink.DataContext = childData;
                var contentBinding = Bridge.as(this.ContentBinding, Windows.UI.Xaml.Data.Binding);
                var b = Bridge.as(this.Binding, Windows.UI.Xaml.Data.Binding);
                if (b != null) {
                    if (b.Mode === Windows.UI.Xaml.Data.BindingMode.OneWay) {
                        if (!b.INTERNAL_WasModeSetByUserRatherThanDefaultValue()) {
                            b.Mode = Windows.UI.Xaml.Data.BindingMode.TwoWay;
                        }
                    }
                    hyperlink.SetBinding(Windows.UI.Xaml.Controls.HyperlinkButton.NavigateUriProperty, b);
                }
                if (contentBinding != null) {
                    if (contentBinding.Mode === Windows.UI.Xaml.Data.BindingMode.OneWay) {
                        if (!contentBinding.INTERNAL_WasModeSetByUserRatherThanDefaultValue()) {
                            contentBinding.Mode = Windows.UI.Xaml.Data.BindingMode.TwoWay;
                        }
                    }
                    hyperlink.SetBinding(Windows.UI.Xaml.Controls.ContentControl.ContentProperty, contentBinding);
                } else if (b != null) {
                    hyperlink.SetBinding(Windows.UI.Xaml.Controls.ContentControl.ContentProperty, b);
                } else if (Bridge.is(childData, System.String)) {
                    hyperlink.NavigateUri = new System.Uri(Bridge.cast(childData, System.String));
                    hyperlink.Content = Bridge.cast(childData, System.String);
                }
                return hyperlink;
            }
        }
    });

    /**
     * Represents a System.Windows.Controls.DataGrid column that hosts template-specified
     content in its cells.
     *
     * @public
     * @class Windows.UI.Xaml.Controls.DataGridTemplateColumn
     * @augments Windows.UI.Xaml.Controls.DataGridBoundColumn
     */
    Bridge.define("Windows.UI.Xaml.Controls.DataGridTemplateColumn", {
        inherits: [Windows.UI.Xaml.Controls.DataGridBoundColumn],
        statics: {
            fields: {
                /**
                 * Identifies the System.Windows.Controls.DataGridTemplateColumn.CellTemplate
                 dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Controls.DataGridTemplateColumn
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                CellTemplateProperty: null,
                /**
                 * Identifies the System.Windows.Controls.DataGridTemplateColumn.CellEditingTemplate
                 dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Controls.DataGridTemplateColumn
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                CellEditingTemplateProperty: null
            },
            ctors: {
                init: function () {
                    var $t;
                    this.CellTemplateProperty = Windows.UI.Xaml.DependencyProperty.Register("CellTemplate", Windows.UI.Xaml.DataTemplate, Windows.UI.Xaml.Controls.DataGridTemplateColumn, ($t = new Windows.UI.Xaml.PropertyMetadata.$ctor2(null, Windows.UI.Xaml.Controls.DataGridTemplateColumn.CellTemplateProperty_Changed), $t.CallPropertyChangedWhenLoadedIntoVisualTree = Windows.UI.Xaml.WhenToCallPropertyChangedEnum.IfPropertyIsSet, $t));
                    this.CellEditingTemplateProperty = Windows.UI.Xaml.DependencyProperty.Register("CellEditingTemplate", Windows.UI.Xaml.DataTemplate, Windows.UI.Xaml.Controls.DataGridTemplateColumn, ($t = new Windows.UI.Xaml.PropertyMetadata.$ctor2(null, Windows.UI.Xaml.Controls.DataGridTemplateColumn.CellEditingTemplateProperty_Changed), $t.CallPropertyChangedWhenLoadedIntoVisualTree = Windows.UI.Xaml.WhenToCallPropertyChangedEnum.IfPropertyIsSet, $t));
                }
            },
            methods: {
                CellTemplateProperty_Changed: function (d, e) { },
                CellEditingTemplateProperty_Changed: function (d, e) { }
            }
        },
        props: {
            /**
             * Gets or sets the template to use to display the contents of a cell that is
             not in editing mode.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.DataGridTemplateColumn
             * @function CellTemplate
             * @type Windows.UI.Xaml.DataTemplate
             */
            CellTemplate: {
                get: function () {
                    return Bridge.cast(this.GetValue(Windows.UI.Xaml.Controls.DataGridTemplateColumn.CellTemplateProperty), Windows.UI.Xaml.DataTemplate);
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Controls.DataGridTemplateColumn.CellTemplateProperty, value);
                }
            },
            /**
             * Gets or sets the template to use to display the contents of a cell that is
             in editing mode.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.DataGridTemplateColumn
             * @function CellEditingTemplate
             * @type Windows.UI.Xaml.DataTemplate
             */
            CellEditingTemplate: {
                get: function () {
                    return Bridge.cast(this.GetValue(Windows.UI.Xaml.Controls.DataGridTemplateColumn.CellEditingTemplateProperty), Windows.UI.Xaml.DataTemplate);
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Controls.DataGridTemplateColumn.CellEditingTemplateProperty, value);
                }
            }
        },
        methods: {
            GenerateElement: function (childData) {
                if (this.CellTemplate != null) {
                    var element = this.CellTemplate.INTERNAL_InstantiateFrameworkTemplate();
                    element.DataContext = childData;
                    return element;
                } else {
                    var textBlock = new Windows.UI.Xaml.Controls.TextBlock();
                    textBlock.Text = Bridge.toString(childData);
                    return textBlock;
                }
            },
            GenerateEditingElement: function (childData) {
                if (this.CellEditingTemplate != null) {
                    var element = this.CellEditingTemplate.INTERNAL_InstantiateFrameworkTemplate();
                    element.DataContext = childData;
                    return element;
                } else {
                    return null;
                }
            }
        }
    });

    /**
     * Represents a System.Windows.Controls.DataGrid column that hosts textual content
     in its cells.
     *
     * @public
     * @class Windows.UI.Xaml.Controls.DataGridTextColumn
     * @augments Windows.UI.Xaml.Controls.DataGridBoundColumn
     */
    Bridge.define("Windows.UI.Xaml.Controls.DataGridTextColumn", {
        inherits: [Windows.UI.Xaml.Controls.DataGridBoundColumn],
        methods: {
            GenerateEditingElement: function (childData) {
                var textBox = new Windows.UI.Xaml.Controls.TextBox();
                textBox.DataContext = childData;
                var b = Bridge.as(this.Binding, Windows.UI.Xaml.Data.Binding);
                if (b != null) {
                    if (b.Mode === Windows.UI.Xaml.Data.BindingMode.OneWay) {
                        if (!b.INTERNAL_WasModeSetByUserRatherThanDefaultValue()) {
                            b.Mode = Windows.UI.Xaml.Data.BindingMode.TwoWay;
                        }
                    }
                    textBox.SetBinding(Windows.UI.Xaml.Controls.TextBox.TextProperty, b);
                } else if (Bridge.is(childData, System.String)) {
                    textBox.Text = Bridge.cast(childData, System.String);
                }

                textBox.addLoaded(function (s, e) {
                    textBox.Focus();
                    textBox.SelectAll();
                });

                return textBox;
            },
            GenerateElement: function (childData) {
                var textBlock = new Windows.UI.Xaml.Controls.TextBlock();
                textBlock.VerticalAlignment = Windows.UI.Xaml.VerticalAlignment.Center;
                var b = Bridge.as(this.Binding, Windows.UI.Xaml.Data.Binding);
                if (b != null) {
                    textBlock.SetBinding(Windows.UI.Xaml.Controls.TextBlock.TextProperty, b);
                } else {
                    if (Bridge.is(childData, System.String)) {
                        textBlock.Text = Bridge.cast(childData, System.String);
                    }
                }
                return textBlock;
            }
        }
    });

    /**
     * @memberof Windows.UI.Xaml
     * @callback Windows.UI.Xaml.ExceptionRoutedEventHandler
     * @param   {System.Object}                               sender    
     * @param   {Windows.UI.Xaml.ExceptionRoutedEventArgs}    e
     * @return  {void}
     */

    /**
     * Represents a control that displays an image. The image is specified as an image file in several possible formats, see Remarks.
     *
     * @public
     * @class Windows.UI.Xaml.Controls.Image
     * @augments Windows.UI.Xaml.FrameworkElement
     * @example
     *
     *
     * Image image = new Image();
     * image.Source = new Windows.UI.Xaml.Media.Imaging.BitmapImage(new Uri("ms-appx:/MyImage.png"));
     * MyStackPanel.Children.Add(image);
     * 
     *
     *
     */
    Bridge.define("Windows.UI.Xaml.Controls.Image", {
        inherits: [Windows.UI.Xaml.FrameworkElement],
        statics: {
            fields: {
                /**
                 * Identifies the Source dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Controls.Image
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                SourceProperty: null,
                /**
                 * Identifies the Stretch dependency property
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Controls.Image
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                StretchProperty: null
            },
            ctors: {
                init: function () {
                    var $t;
                    this.SourceProperty = Windows.UI.Xaml.DependencyProperty.Register("Source", Windows.UI.Xaml.Media.ImageSource, Windows.UI.Xaml.Controls.Image, ($t = new Windows.UI.Xaml.PropertyMetadata.$ctor2(null, Windows.UI.Xaml.Controls.Image.Source_Changed), $t.CallPropertyChangedWhenLoadedIntoVisualTree = Windows.UI.Xaml.WhenToCallPropertyChangedEnum.IfPropertyIsSet, $t));
                    this.StretchProperty = Windows.UI.Xaml.DependencyProperty.Register("Stretch", Windows.UI.Xaml.Media.Stretch, Windows.UI.Xaml.Controls.Image, ($t = new Windows.UI.Xaml.PropertyMetadata.$ctor2(Bridge.box(Windows.UI.Xaml.Media.Stretch.Uniform, Windows.UI.Xaml.Media.Stretch, System.Enum.toStringFn(Windows.UI.Xaml.Media.Stretch)), Windows.UI.Xaml.Controls.Image.Stretch_Changed), $t.CallPropertyChangedWhenLoadedIntoVisualTree = Windows.UI.Xaml.WhenToCallPropertyChangedEnum.IfPropertyIsSet, $t));
                }
            },
            methods: {
                Source_Changed: function (i, e) {
                    var image = Bridge.cast(i, Windows.UI.Xaml.Controls.Image);
                    var newValue = Bridge.cast(e.NewValue, Windows.UI.Xaml.Media.ImageSource);
                    if (CSHTML5.Internal.INTERNAL_VisualTreeManager.IsElementInVisualTree(image)) {
                        if (Bridge.is(newValue, Windows.UI.Xaml.Media.Imaging.BitmapImage)) {
                            var bmpImage = Bridge.cast(newValue, Windows.UI.Xaml.Media.Imaging.BitmapImage);
                            bmpImage.removeImageFailed(Bridge.fn.cacheBind(image, image.bmpImage_ImageFailed));
                            bmpImage.addImageFailed(Bridge.fn.cacheBind(image, image.bmpImage_ImageFailed));
                            bmpImage.removeImageOpened(Bridge.fn.cacheBind(image, image.bmpImage_ImageOpened));
                            bmpImage.addImageOpened(Bridge.fn.cacheBind(image, image.bmpImage_ImageOpened));
                            bmpImage.removeUriSourceChanged(Bridge.fn.cacheBind(image, image.bmpImage_UriSourceChanged));
                            bmpImage.addUriSourceChanged(Bridge.fn.cacheBind(image, image.bmpImage_UriSourceChanged));
                        }
                        image.RefreshSource();
                    }
                },
                Stretch_Changed: function (i, e) {
                    var image = Bridge.cast(i, Windows.UI.Xaml.Controls.Image);
                    var newValue = System.Nullable.getValue(Bridge.cast(Bridge.unbox(e.NewValue, Windows.UI.Xaml.Media.Stretch), System.Int32));
                    if (CSHTML5.Internal.INTERNAL_VisualTreeManager.IsElementInVisualTree(image)) {
                        var objectFitvalue = "";
                        var objectPosition = "center top";
                        switch (newValue) {
                            case Windows.UI.Xaml.Media.Stretch.None: 
                                objectFitvalue = "none";
                                objectPosition = "left top";
                                break;
                            case Windows.UI.Xaml.Media.Stretch.Fill: 
                                objectFitvalue = "fill";
                                objectPosition = "center center";
                                break;
                            case Windows.UI.Xaml.Media.Stretch.Uniform: 
                                objectFitvalue = "contain";
                                objectPosition = "center center";
                                break;
                            case Windows.UI.Xaml.Media.Stretch.UniformToFill: 
                                objectFitvalue = "cover";
                                objectPosition = "left top";
                                break;
                            default: 
                                break;
                        }
                        image._imageDiv.style.objectFit = objectFitvalue;
image._imageDiv.style.objectPosition = objectPosition;

                    }
                }
            }
        },
        fields: {
            _imageDiv: null,
            imgWidth: 0,
            imgHeight: 0,
            _imageFailedEventManager: null,
            _imageOpenedEventManager: null
        },
        props: {
            /**
             * Gets or sets the source for the image.
             Returns an object that represents the image source file for the drawn image. Typically
             you set this with a BitmapSource object, constructed with the that describes
             the path to a valid image source file.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.Image
             * @function Source
             * @type Windows.UI.Xaml.Media.ImageSource
             */
            Source: {
                get: function () {
                    return Bridge.cast(this.GetValue(Windows.UI.Xaml.Controls.Image.SourceProperty), Windows.UI.Xaml.Media.ImageSource);
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Controls.Image.SourceProperty, value);
                }
            },
            /**
             * Gets or sets a value that describes how an Image should be stretched to fill the destination rectangle.
             Returns a value of the Stretch enumeration that specifies how the source image is applied if the Height and Width of the Image are specified and are different
             than the source image's height and width. The default value is Uniform.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.Image
             * @function Stretch
             * @type Windows.UI.Xaml.Media.Stretch
             */
            Stretch: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.Controls.Image.StretchProperty), Windows.UI.Xaml.Media.Stretch), System.Int32));
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Controls.Image.StretchProperty, Bridge.box(value, Windows.UI.Xaml.Media.Stretch, System.Enum.toStringFn(Windows.UI.Xaml.Media.Stretch)));
                }
            },
            INTERNAL_DomImageElement: {
                get: function () {
                    return this._imageDiv;
                }
            }
        },
        ctors: {
            init: function () {
                this.imgWidth = 0;
                this.imgHeight = 0;
            }
        },
        methods: {
            /**
             * Occurs when there is an error associated with image retrieval or format.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.Image
             * @event Windows.UI.Xaml.Controls.Image#ImageFailed
             * @return  {Windows.UI.Xaml.ExceptionRoutedEventHandler}
             */
            addImageFailed: function (value) {
                if (this._imageFailedEventManager == null) {
                    this._imageFailedEventManager = new (CSHTML5.Internal.INTERNAL_EventManager$2(Function,Windows.UI.Xaml.ExceptionRoutedEventArgs)).ctor(Bridge.fn.bind(this, function () {
                        return this._imageDiv;
                    }), "error", Bridge.fn.cacheBind(this, this.ProcessOnImageFailed));
                    this._imageFailedEventManager.Add(value);
                } else {
                    this._imageFailedEventManager.Add(value);
                }
            },
            /**
             * Occurs when there is an error associated with image retrieval or format.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.Image
             * @event Windows.UI.Xaml.Controls.Image#ImageFailed
             * @return  {Windows.UI.Xaml.ExceptionRoutedEventHandler}
             */
            removeImageFailed: function (value) {
                if (this._imageFailedEventManager != null) {
                    this._imageFailedEventManager.Remove(value);
                }

            },
            /**
             * Occurs when the image source is downloaded and decoded with no failure.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.Image
             * @event Windows.UI.Xaml.Controls.Image#ImageOpened
             * @return  {Windows.UI.Xaml.RoutedEventHandler}
             */
            addImageOpened: function (value) {
                if (this._imageOpenedEventManager == null) {
                    this._imageOpenedEventManager = new (CSHTML5.Internal.INTERNAL_EventManager$2(Function,Windows.UI.Xaml.RoutedEventArgs)).ctor(Bridge.fn.bind(this, function () {
                        return this._imageDiv;
                    }), "load", Bridge.fn.cacheBind(this, this.ProcessOnImageOpened));
                    this._imageOpenedEventManager.Add(value);
                } else {
                    this._imageOpenedEventManager.Add(value);
                }
            },
            /**
             * Occurs when the image source is downloaded and decoded with no failure.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.Image
             * @event Windows.UI.Xaml.Controls.Image#ImageOpened
             * @return  {Windows.UI.Xaml.RoutedEventHandler}
             */
            removeImageOpened: function (value) {
                if (this._imageOpenedEventManager != null) {
                    this._imageOpenedEventManager.Remove(value);
                }
            },
            bmpImage_UriSourceChanged: function (sender, e) {
                this.RefreshSource();
            },
            RefreshSource: function () {
                if (this.Source != null) {
                    this.addLoaded(Bridge.fn.cacheBind(this, this.Image_Loaded));
                    if (Bridge.is(this.Source, Windows.UI.Xaml.Media.Imaging.BitmapImage)) {
                        var sourceAsBitmapImage = Bridge.cast(this.Source, Windows.UI.Xaml.Media.Imaging.BitmapImage);
                        if (System.Uri.notEquals(sourceAsBitmapImage.UriSource, null)) {
                            var sourceUri = null;
                            sourceUri = Bridge.cast(this.Source, Windows.UI.Xaml.Media.Imaging.BitmapImage).UriSource;

                            var html5Path = CSHTML5.Internal.INTERNAL_UriHelper.ConvertToHtml5Path(sourceUri.getOriginalString(), this);

                            CSHTML5.Internal.INTERNAL_HtmlDomManager.SetDomElementAttribute(this._imageDiv, "src", html5Path);
                        } else if (sourceAsBitmapImage.INTERNAL_StreamSource != null) {
                            var dataUrl = "data:image/png;base64," + (sourceAsBitmapImage.INTERNAL_StreamAsBase64String || "");
                            CSHTML5.Internal.INTERNAL_HtmlDomManager.SetDomElementAttribute(this._imageDiv, "src", dataUrl);
                        } else if (!System.String.isNullOrEmpty(sourceAsBitmapImage.INTERNAL_DataURL)) {
                            var dataUrl1 = sourceAsBitmapImage.INTERNAL_DataURL;
                            CSHTML5.Internal.INTERNAL_HtmlDomManager.SetDomElementAttribute(this._imageDiv, "src", dataUrl1);
                        }
                    }
                } else {
                    CSHTML5.Internal.INTERNAL_HtmlDomManager.SetDomElementAttribute(this._imageDiv, "src", "");
                }
                CSHTML5.Internal.INTERNAL_HtmlDomManager.SetDomElementAttribute(this._imageDiv, "alt", " ");
            },
            Image_Loaded: function (sender, e) {
                this.removeLoaded(Bridge.fn.cacheBind(this, this.Image_Loaded));


                var parentWidth = Bridge.cast(this.Parent, Windows.UI.Xaml.FrameworkElement).ActualWidth;
                var parentHeight = Bridge.cast(this.Parent, Windows.UI.Xaml.FrameworkElement).ActualHeight;

                if (CSHTML5.Interop.IsRunningInTheSimulator) {
                    var concatenated = System.Convert.toString(this._imageDiv.naturalWidth + '|' + this._imageDiv.naturalHeight);
                    var sepIndex = System.String.indexOf(concatenated, String.fromCharCode(124));
                    var imgWidthAsString = concatenated.substr(0, sepIndex);
                    var imgHeightAsString = concatenated.substr(((sepIndex + 1) | 0));
                    this.imgWidth = Bridge.Int.parseFloat(imgWidthAsString, Bridge.global.System.Globalization.CultureInfo.invariantCulture);
                    this.imgHeight = Bridge.Int.parseFloat(imgHeightAsString, Bridge.global.System.Globalization.CultureInfo.invariantCulture);
                } else {
                    this.imgHeight = System.Convert.toDouble(this._imageDiv.naturalHeight);
                    this.imgWidth = System.Convert.toDouble(this._imageDiv.naturalWidth);
                }

                var currentWidth = this.ActualWidth;
                var currentHeight = this.ActualHeight;


                var isParentLimitingHorizontalSize = !((Bridge.is(this.Parent, Windows.UI.Xaml.Controls.StackPanel) && Bridge.cast(this.Parent, Windows.UI.Xaml.Controls.StackPanel).Orientation === Windows.UI.Xaml.Controls.Orientation.Horizontal) || (Bridge.is(this.Parent, Windows.UI.Xaml.Controls.WrapPanel) && Bridge.cast(this.Parent, Windows.UI.Xaml.Controls.WrapPanel).Orientation === Windows.UI.Xaml.Controls.Orientation.Horizontal) || Bridge.is(this.Parent, Windows.UI.Xaml.Controls.Canvas));
                var limitSize = isParentLimitingHorizontalSize && (this.HorizontalAlignment === Windows.UI.Xaml.HorizontalAlignment.Stretch && (isNaN(this.Width) && currentWidth !== parentWidth)) || ((isNaN(this.Width) && currentWidth > parentWidth));

                if (isParentLimitingHorizontalSize) {
                    if (isNaN(this.Width) && currentWidth !== parentWidth) {
                        this.imgWidth = System.Convert.toDouble(this._imageDiv.style.width = Bridge.box(parentWidth, System.Double, System.Double.format, System.Double.getHashCode));
                    }
                }

                var isParentLimitingVerticalSize = !((Bridge.is(this.Parent, Windows.UI.Xaml.Controls.StackPanel) && Bridge.cast(this.Parent, Windows.UI.Xaml.Controls.StackPanel).Orientation === Windows.UI.Xaml.Controls.Orientation.Vertical) || (Bridge.is(this.Parent, Windows.UI.Xaml.Controls.WrapPanel) && Bridge.cast(this.Parent, Windows.UI.Xaml.Controls.WrapPanel).Orientation === Windows.UI.Xaml.Controls.Orientation.Vertical) || Bridge.is(this.Parent, Windows.UI.Xaml.Controls.Canvas));
                if (isParentLimitingVerticalSize) {
                    if (isNaN(this.Height) && currentHeight !== parentHeight) {
                        this.imgWidth = System.Convert.toDouble(this._imageDiv.style.height = Bridge.box(parentHeight, System.Double, System.Double.format, System.Double.getHashCode));
                    }
                }
            },
            bmpImage_ImageOpened: function (sender, e) {
                this.OnImageOpened(e);
            },
            bmpImage_ImageFailed: function (sender, e) {
                this.OnImageFailed(e);
            },
            /**
             * Raises the ImageFailed event
             *
             * @instance
             * @private
             * @this Windows.UI.Xaml.Controls.Image
             * @memberof Windows.UI.Xaml.Controls.Image
             * @param   {System.Object}    jsEventArg
             * @return  {void}
             */
            ProcessOnImageFailed: function (jsEventArg) {
                var $t;
                var eventArgs = ($t = new Windows.UI.Xaml.ExceptionRoutedEventArgs.ctor(), $t.OriginalSource = this, $t);
                this.OnImageFailed(eventArgs);
            },
            /**
             * Raises the ImageFailed event
             *
             * @instance
             * @private
             * @this Windows.UI.Xaml.Controls.Image
             * @memberof Windows.UI.Xaml.Controls.Image
             * @param   {Windows.UI.Xaml.ExceptionRoutedEventArgs}    eventArgs
             * @return  {void}
             */
            OnImageFailed: function (eventArgs) {
                var $t;
                $t = Bridge.getEnumerator(System.Linq.Enumerable.from(this._imageFailedEventManager.Handlers).toList(Function));
                try {
                    while ($t.moveNext()) {
                        var handler = $t.Current;
                        handler(this, eventArgs);
                    }
                } finally {
                    if (Bridge.is($t, System.IDisposable)) {
                        $t.System$IDisposable$Dispose();
                    }
                }
            },
            /**
             * Raises the ImageOpened event
             *
             * @instance
             * @private
             * @this Windows.UI.Xaml.Controls.Image
             * @memberof Windows.UI.Xaml.Controls.Image
             * @param   {System.Object}    jsEventArg
             * @return  {void}
             */
            ProcessOnImageOpened: function (jsEventArg) {
                var $t;
                var eventArgs = ($t = new Windows.UI.Xaml.RoutedEventArgs(), $t.OriginalSource = this, $t);
                this.OnImageOpened(eventArgs);
            },
            /**
             * Raises the ImageOpened event
             *
             * @instance
             * @private
             * @this Windows.UI.Xaml.Controls.Image
             * @memberof Windows.UI.Xaml.Controls.Image
             * @param   {Windows.UI.Xaml.RoutedEventArgs}    eventArgs
             * @return  {void}
             */
            OnImageOpened: function (eventArgs) {
                var $t;
                $t = Bridge.getEnumerator(System.Linq.Enumerable.from(this._imageOpenedEventManager.Handlers).toList(Function));
                try {
                    while ($t.moveNext()) {
                        var handler = $t.Current;
                        handler(this, eventArgs);
                    }
                } finally {
                    if (Bridge.is($t, System.IDisposable)) {
                        $t.System$IDisposable$Dispose();
                    }
                }
            },
            INTERNAL_AttachToDomEvents: function () {
                Windows.UI.Xaml.FrameworkElement.prototype.INTERNAL_AttachToDomEvents.call(this);
                if (this._imageOpenedEventManager != null) {
                    this._imageOpenedEventManager.AttachToDomEvents();
                }
                if (this._imageFailedEventManager != null) {
                    this._imageFailedEventManager.AttachToDomEvents();
                }
            },
            INTERNAL_DetachFromDomEvents: function () {
                Windows.UI.Xaml.FrameworkElement.prototype.INTERNAL_DetachFromDomEvents.call(this);
                if (this._imageOpenedEventManager != null) {
                    this._imageOpenedEventManager.DetachFromDomEvents();
                }
                if (this._imageFailedEventManager != null) {
                    this._imageFailedEventManager.DetachFromDomEvents();
                }
            },
            CreateDomElement: function (parentRef, domElementWhereToPlaceChildren) {
                var div = CSHTML5.Internal.INTERNAL_HtmlDomManager.CreateDomElementAndAppendIt("div", parentRef, this);
                var intermediaryDomStyle = CSHTML5.Internal.INTERNAL_HtmlDomManager.GetDomElementStyleForModification(div);
                intermediaryDomStyle.lineHeight = "0px";

                var img = CSHTML5.Internal.INTERNAL_HtmlDomManager.CreateDomElementAndAppendIt("img", div, this);
                CSHTML5.Internal.INTERNAL_HtmlDomManager.SetDomElementAttribute(img, "alt", " ");

                var style = CSHTML5.Internal.INTERNAL_HtmlDomManager.GetDomElementStyleForModification(img);
                style.display = "block";
                style.width = "inherit";
                style.height = "inherit";
                style.objectPosition = "center top";


                img.addEventListener('mousedown', function(e) {
e.preventDefault();
}, false);

                this._imageDiv = img;
                domElementWhereToPlaceChildren.v = null;
                return div;
            }
        }
    });

    /**
     * Displays the content of a ItemsPresenter.
     *
     * @public
     * @class Windows.UI.Xaml.Controls.ItemsPresenter
     * @augments Windows.UI.Xaml.FrameworkElement
     */
    Bridge.define("Windows.UI.Xaml.Controls.ItemsPresenter", {
        inherits: [Windows.UI.Xaml.FrameworkElement],
        statics: {
            fields: {
                /**
                 * TemplateProperty
                 *
                 * @static
                 * @readonly
                 * @memberof Windows.UI.Xaml.Controls.ItemsPresenter
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                TemplateProperty: null
            },
            ctors: {
                init: function () {
                    this.TemplateProperty = Windows.UI.Xaml.DependencyProperty.Register("Template", Windows.UI.Xaml.Controls.ItemsPanelTemplate, Windows.UI.Xaml.Controls.ItemsPresenter, new Windows.UI.Xaml.PropertyMetadata.$ctor2(null, Windows.UI.Xaml.Controls.ItemsPresenter.OnTemplateChanged));
                }
            },
            methods: {
                OnTemplateChanged: function (d, e) {
                    var ip = Bridge.cast(d, Windows.UI.Xaml.Controls.ItemsPresenter);

                    ip._templateCache = Bridge.cast(e.NewValue, Windows.UI.Xaml.Controls.ItemsPanelTemplate);

                    ip.OnPanelChangedInternal();
                },
                FromPanel: function (panel) {
                    if (panel == null) {
                        return null;
                    }

                    return Bridge.as(panel.Parent, Windows.UI.Xaml.Controls.ItemsPresenter);
                }
            }
        },
        fields: {
            _templateCache: null,
            _owner: null,
            _itemsHost: null
        },
        props: {
            Owner: {
                get: function () {
                    return this._owner;
                }
            },
            ItemsHost: {
                get: function () {
                    return this._itemsHost;
                }
            },
            /**
             * Template Property
             *
             * @instance
             * @memberof Windows.UI.Xaml.Controls.ItemsPresenter
             * @function Template
             * @type Windows.UI.Xaml.Controls.ItemsPanelTemplate
             */
            Template: {
                get: function () {
                    return this._templateCache;
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Controls.ItemsPresenter.TemplateProperty, value);
                }
            }
        },
        methods: {
            OnPanelChangedInternal: function () {
                if (this.ItemsHost != null) {

                    CSHTML5.Internal.INTERNAL_VisualTreeManager.DetachVisualChildIfNotNull(this.ItemsHost, this);
                }

                if (this._templateCache != null) {
                    var visualTree = this._templateCache.INTERNAL_InstantiateFrameworkTemplate$1(null);
                    var panel = Bridge.as(visualTree, Windows.UI.Xaml.Controls.Panel);
                    if (panel != null) {
                        if (panel.Children.Count > 0) {
                            throw new System.InvalidOperationException.$ctor1("VisualTree of ItemsPanelTemplate must be a single element.");
                        }

                        this._itemsHost = panel;

                    } else {
                        throw new System.InvalidOperationException.$ctor1(System.String.format("VisualTree of ItemsPanelTemplate must contain a Panel. '{0}' is not a Panel.", [Bridge.getType(visualTree)]));
                    }

                    CSHTML5.Internal.INTERNAL_VisualTreeManager.AttachVisualChildIfNotAlreadyAttached(panel, this);

                    if (this.Owner != null) {
                        this.Owner.Refresh();
                    }
                } else {
                    this._itemsHost = null;
                    this.Owner.ItemContainerGenerator.INTERNAL_Clear();
                }
            },
            AttachToOwner: function (owner) {
                this._owner = owner;

                var template = (this._owner != null) ? this._owner.ItemsPanel : null;

                this.Template = template;
            },
            INTERNAL_OnAttachedToVisualTree: function () {
                Windows.UI.Xaml.FrameworkElement.prototype.INTERNAL_OnAttachedToVisualTree.call(this);

                if (this.ItemsHost != null) {
                    CSHTML5.Internal.INTERNAL_VisualTreeManager.AttachVisualChildIfNotAlreadyAttached(this.ItemsHost, this);
                }
            }
        }
    });

    /**
     * Represents an object that contains audio, video, or both.
     * // Gets the height portion of the native aspect ratio of the media.
    //
     * // Identifies the AspectRatioHeight dependency property.
    //
     * // Gets the width portion of the native aspect ratio of the media.
    //
     * // Identifies the AspectRatioWidth dependency property.
    //
     * // Gets or sets a value that describes the purpose of the audio information
    // in an audio stream.
    //
     * // Identifies the AudioCategory dependency property.
    //
     * // Gets or sets a value that describes the primary usage of the device that
    // is being used to play back audio.
    //
     * // Identifies the AudioDeviceType dependency property.
    //
     * // Gets the number of audio streams that exist in the current media file.
    //
     * // Identifies the AudioStreamCount dependency property.
    //
     * // Gets or sets the index of the audio stream that plays along with the video
    // component. The collection of audio streams is composed at run time and represents
    // all audio streams that are available in the media file.
    //
     * // Identifies the AudioStreamIndex dependency property.
    //
     * // Gets or sets a ratio of volume across stereo speakers.
    //
     * // Identifies the Balance dependency property.
    //
     * // Gets a value that indicates the current buffering progress.
    //
     * // Identifies the BufferingProgress dependency property.
    //
     * // Gets a value that indicates whether media can be repositioned by setting
    // the value of the Position property.
    //
     * // Identifies the CanSeek dependency property.
    //
     * // Gets or sets the default playback rate for the media engine. The playback
    // rate applies when the user isn't using fast forward or reverse.
    //
     * // Identifies the DefaultPlaybackRate dependency property.
    //
     * // Gets a value that indicates the amount of download completed for content
    // located on a remote server.
    //
     * // Identifies the DownloadProgress dependency property.
    //
     * // Gets the offset of download progress, which is relevant in seek-ahead scenarios.
    //
     * // Identifies the DownloadProgressOffset dependency property.
    //
     * // Gets a value that reports whether the current source media is a stereo 3-D
    // video media file.
    //
     * // Identifies the IsStereo3DVideo dependency property.
    //
     * // Gets the collection of timeline markers associated with the currently loaded
    // media file.
    //
     * // Gets the duration of the media file currently opened.
    //
     * // Identifies the NaturalDuration dependency property.
    //
     * // Gets the height of the video associated with the media.
    //
     * // Identifies the NaturalVideoHeight dependency property.
    //
     * // Gets the width of the video associated with the media.
    //
     * // Identifies the NaturalVideoWidth dependency property.
    //
     * // Gets or sets the playback rate ratio for the media engine.
    //
     * // Identifies the PlaybackRate dependency property.
    //
     * // Gets the information that is transmitted if the MediaElement is used for
    // a "PlayTo" scenario.
    //
     * // Identifies the PlayToSource dependency property.
    //
     * // Gets or sets the image source that is used for a placeholder image during
    // MediaElement loading transition states.
    //
     * // Identifies the PosterSource dependency property.
    //
     * // Gets or sets the dedicated object for media content protection that is associated
    // with this MediaElement.
    //
     * // Identifies the ProtectionManager dependency property.
    //
     * // Gets or sets a value that configures the MediaElement for real-time communications
    // scenarios.
    //
     * // Identifies the RealTimePlayback dependency property.
    //
     * // Gets or sets an enumeration value that determines the stereo 3-D video frame-packing
    // mode for the current media source.
    //
     * // Identifies the Stereo3DVideoPackingMode dependency property.
    //
     * // Gets or sets an enumeration value that determines the stereo 3-D video render
    // mode for the current media source.
    //
     * // Identifies the Stereo3DVideoRenderMode dependency property.
    //
     * // Occurs when the BufferingProgress property changes.
    //
     * // Occurs when the DownloadProgress property has changed.
    //
     * // Occurs when a timeline marker is encountered during media playback.
    //
     * // Occurs when PlaybackRate changes value.
    //
     * // Occurs when the seek point of a requested seek operation is ready for playback.
    //
     * // Applies an audio effect to playback. Takes effect for the next source that
    // is set on this MediaElement.
    //
     * // Applies a video effect to playback. Takes effect for the next source that
    // is set on this MediaElement.
    //
     * // Removes all effects for the next source set for this MediaElement.
    //
     *
     * @public
     * @class Windows.UI.Xaml.Controls.MediaElement
     * @augments Windows.UI.Xaml.FrameworkElement
     */
    Bridge.define("Windows.UI.Xaml.Controls.MediaElement", {
        inherits: [Windows.UI.Xaml.FrameworkElement],
        statics: {
            fields: {
                HTML_SHOWCONTROLS_PROPERTY_NAME: null,
                HTML_AUTOPLAY_PROPERTY_NAME: null,
                HTML_ISLOOPING_PROPERTY_NAME: null,
                HTML_ISMUTED_PROPERTY_NAME: null,
                SupportedVideoTypes: null,
                SupportedAudioTypes: null,
                /**
                 * Identifies the AutoPlay dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Controls.MediaElement
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                AutoPlayProperty: null,
                /**
                 * Identifies the IsAudioOnly dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Controls.MediaElement
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                IsAudioOnlyProperty: null,
                /**
                 * Identifies the IsLooping dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Controls.MediaElement
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                IsLoopingProperty: null,
                /**
                 * Identifies the IsMuted dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Controls.MediaElement
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                IsMutedProperty: null,
                /**
                 * Identifies the Source dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Controls.MediaElement
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                SourceProperty: null,
                /**
                 * Identifies the Volume dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Controls.MediaElement
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                VolumeProperty: null,
                ShowControlsProperty: null
            },
            ctors: {
                init: function () {
                    var $t;
                    this.HTML_SHOWCONTROLS_PROPERTY_NAME = "controls";
                    this.HTML_AUTOPLAY_PROPERTY_NAME = "autoplay";
                    this.HTML_ISLOOPING_PROPERTY_NAME = "loop";
                    this.HTML_ISMUTED_PROPERTY_NAME = "muted";
                    this.SupportedVideoTypes = function (_o1) {
                            _o1.add("mp4");
                            _o1.add("ogv");
                            _o1.add("webm");
                            _o1.add("3gp");
                            return _o1;
                        }(new (System.Collections.Generic.List$1(System.String)).ctor());
                    this.SupportedAudioTypes = function (_o2) {
                            _o2.add("mp3");
                            _o2.add("ogg");
                            return _o2;
                        }(new (System.Collections.Generic.List$1(System.String)).ctor());
                    this.AutoPlayProperty = Windows.UI.Xaml.DependencyProperty.Register("AutoPlay", System.Boolean, Windows.UI.Xaml.Controls.MediaElement, ($t = new Windows.UI.Xaml.PropertyMetadata.$ctor2(Bridge.box(true, System.Boolean, System.Boolean.toString), Windows.UI.Xaml.Controls.MediaElement.AutoPlay_Changed), $t.CallPropertyChangedWhenLoadedIntoVisualTree = Windows.UI.Xaml.WhenToCallPropertyChangedEnum.IfPropertyIsSet, $t));
                    this.IsAudioOnlyProperty = Windows.UI.Xaml.DependencyProperty.Register("IsAudioOnly", System.Boolean, Windows.UI.Xaml.Controls.MediaElement, new Windows.UI.Xaml.PropertyMetadata.$ctor1(Bridge.box(false, System.Boolean, System.Boolean.toString)));
                    this.IsLoopingProperty = Windows.UI.Xaml.DependencyProperty.Register("IsLooping", System.Boolean, Windows.UI.Xaml.Controls.MediaElement, ($t = new Windows.UI.Xaml.PropertyMetadata.$ctor2(Bridge.box(false, System.Boolean, System.Boolean.toString), Windows.UI.Xaml.Controls.MediaElement.IsLooping_Changed), $t.CallPropertyChangedWhenLoadedIntoVisualTree = Windows.UI.Xaml.WhenToCallPropertyChangedEnum.IfPropertyIsSet, $t));
                    this.IsMutedProperty = Windows.UI.Xaml.DependencyProperty.Register("IsMuted", System.Boolean, Windows.UI.Xaml.Controls.MediaElement, ($t = new Windows.UI.Xaml.PropertyMetadata.$ctor2(Bridge.box(false, System.Boolean, System.Boolean.toString), Windows.UI.Xaml.Controls.MediaElement.IsMuted_Changed), $t.CallPropertyChangedWhenLoadedIntoVisualTree = Windows.UI.Xaml.WhenToCallPropertyChangedEnum.IfPropertyIsSet, $t));
                    this.SourceProperty = Windows.UI.Xaml.DependencyProperty.Register("Source", System.Uri, Windows.UI.Xaml.Controls.MediaElement, ($t = new Windows.UI.Xaml.PropertyMetadata.$ctor2(null, Windows.UI.Xaml.Controls.MediaElement.Source_Changed), $t.CallPropertyChangedWhenLoadedIntoVisualTree = Windows.UI.Xaml.WhenToCallPropertyChangedEnum.IfPropertyIsSet, $t));
                    this.VolumeProperty = Windows.UI.Xaml.DependencyProperty.Register("Volume", System.Double, Windows.UI.Xaml.Controls.MediaElement, ($t = new Windows.UI.Xaml.PropertyMetadata.$ctor2(Bridge.box(0.5, System.Double, System.Double.format, System.Double.getHashCode), Windows.UI.Xaml.Controls.MediaElement.Volume_Changed), $t.CallPropertyChangedWhenLoadedIntoVisualTree = Windows.UI.Xaml.WhenToCallPropertyChangedEnum.IfPropertyIsSet, $t));
                    this.ShowControlsProperty = Windows.UI.Xaml.DependencyProperty.Register("ShowControls", System.Boolean, Windows.UI.Xaml.Controls.MediaElement, ($t = new Windows.UI.Xaml.PropertyMetadata.$ctor2(Bridge.box(false, System.Boolean, System.Boolean.toString), Windows.UI.Xaml.Controls.MediaElement.ShowControls_Changed), $t.CallPropertyChangedWhenLoadedIntoVisualTree = Windows.UI.Xaml.WhenToCallPropertyChangedEnum.IfPropertyIsSet, $t));
                }
            },
            methods: {
                AutoPlay_Changed: function (d, e) {
                    var control = Bridge.cast(d, Windows.UI.Xaml.Controls.MediaElement);
                    if (control._mediaElement_ForAudioOnly_ForSimulatorOnly == null) {
                        control.ManageDomBoolProperty_Changed(Windows.UI.Xaml.Controls.MediaElement.HTML_AUTOPLAY_PROPERTY_NAME, System.Nullable.getValue(Bridge.cast(Bridge.unbox(e.NewValue, System.Boolean), System.Boolean)));
                    } else {
                    }
                },
                IsLooping_Changed: function (d, e) {
                    var control = Bridge.cast(d, Windows.UI.Xaml.Controls.MediaElement);
                    if (control._mediaElement_ForAudioOnly_ForSimulatorOnly == null) {
                        control.ManageDomBoolProperty_Changed(Windows.UI.Xaml.Controls.MediaElement.HTML_ISLOOPING_PROPERTY_NAME, System.Nullable.getValue(Bridge.cast(Bridge.unbox(e.NewValue, System.Boolean), System.Boolean)));
                    } else {
                        control._mediaElement_ForAudioOnly_ForSimulatorOnly.IsLooping = control.IsLooping;
                    }
                },
                IsMuted_Changed: function (d, e) {
                    var control = Bridge.cast(d, Windows.UI.Xaml.Controls.MediaElement);
                    if (control._mediaElement_ForAudioOnly_ForSimulatorOnly == null) {
                        control.ManageDomBoolProperty_Changed(Windows.UI.Xaml.Controls.MediaElement.HTML_ISMUTED_PROPERTY_NAME, System.Nullable.getValue(Bridge.cast(Bridge.unbox(e.NewValue, System.Boolean), System.Boolean)));
                    } else {
                        control._mediaElement_ForAudioOnly_ForSimulatorOnly.IsMuted = control.IsMuted;
                    }
                },
                Source_Changed: function (d, e) {
                    var control = Bridge.cast(d, Windows.UI.Xaml.Controls.MediaElement);
                    var newValue = Bridge.cast(e.NewValue, System.Uri);

                    if (CSHTML5.Internal.INTERNAL_VisualTreeManager.IsElementInVisualTree(control)) {
                        var newUri = Bridge.toString(newValue);

                        if (!System.String.isNullOrWhiteSpace(newUri)) {
                            var tagString = "none";

                            var valueForHtml5SourceProperty = CSHTML5.Internal.INTERNAL_UriHelper.ConvertToHtml5Path(Bridge.toString(newValue), control);

                            var newExtensionLowercase = Windows.UI.Xaml.Controls.MediaElement.GetExtension(newUri).toLowerCase();

                            if (Windows.UI.Xaml.Controls.MediaElement.SupportedVideoTypes.contains(newExtensionLowercase)) {
                                if (control.IsAudioOnly || control._mediaElement == null) {
                                    tagString = "video";
                                    control.IsAudioOnly = false;
                                }
                            } else if (Windows.UI.Xaml.Controls.MediaElement.SupportedAudioTypes.contains(newExtensionLowercase)) {
                                if (!control.IsAudioOnly || control._mediaElement == null) {
                                    tagString = "audio";
                                    control.IsAudioOnly = true;
                                }
                            } else {
                                throw new System.NotSupportedException.$ctor1("ERROR: The MediaElement control only supports files of the following types: VIDEO: mp4, ogv, webm, 3gp - AUDIO: mp3, ogg - Note: for best browser compatibility, it is recommended to use only MP3 and MP4 files.");
                            }
                            if (!Bridge.referenceEquals(tagString, "none")) {
                                if (control._mediaElement != null) {
                                    CSHTML5.Internal.INTERNAL_HtmlDomManager.RemoveFromDom(control._mediaElement);
                                }
                                if (CSharpXamlForHtml5.Environment.IsRunningInJavaScript || Bridge.referenceEquals(tagString, "video")) {
                                    var element = { v : null };
                                    var outerDiv = control.INTERNAL_OuterDomElement;
                                    var elementStyle = CSHTML5.Internal.INTERNAL_HtmlDomManager.CreateDomElementAppendItAndGetStyle(tagString, outerDiv, control, element);

                                    control._mediaElement_ForAudioOnly_ForSimulatorOnly = null;

                                    control._mediaElement = element.v;

                                    if (Bridge.referenceEquals(tagString, "video")) {
                                        elementStyle.width = "100%";
                                        elementStyle.height = "100%";
                                    }

                                    control.Refresh();
                                } else {
                                    if (control._mediaElement_ForAudioOnly_ForSimulatorOnly == null) {
                                        control._mediaElement_ForAudioOnly_ForSimulatorOnly = DotNetForHtml5.Core.INTERNAL_Simulator.WpfMediaElementFactory.Create(Bridge.fn.cacheBind(control, control.SimulatorMediaElement_Loaded), Bridge.fn.cacheBind(control, control.SimulatorMediaElement_MediaEnded));
                                    }
                                    control._mediaElement_ForAudioOnly_ForSimulatorOnly.Source = new System.Uri(valueForHtml5SourceProperty);
                                    control.Refresh_SimulatorOnly();
                                    return;
                                }
                            }

                            CSHTML5.Internal.INTERNAL_HtmlDomManager.SetDomElementAttribute(control._mediaElement, "src", valueForHtml5SourceProperty, true);
                        } else {
                            if (control._mediaElement != null) {
                                CSHTML5.Internal.INTERNAL_HtmlDomManager.SetDomElementAttribute(control._mediaElement, "src", "", true);
                            }
                        }
                    }
                },
                Volume_Changed: function (d, e) {
                    var control = Bridge.cast(d, Windows.UI.Xaml.Controls.MediaElement);
                    control.ManageVolume_Changed();
                },
                GetExtension: function (uriString) {
                    if (uriString != null) {
                        var lastDotIndex = uriString.lastIndexOf(String.fromCharCode(46));
                        if (lastDotIndex > -1) {
                            return uriString.substr(((lastDotIndex + 1) | 0));
                        }
                    }
                    return "";
                },
                ShowControls_Changed: function (sender, e) {
                    var control = Bridge.cast(sender, Windows.UI.Xaml.Controls.MediaElement);
                    control.ManageDomBoolProperty_Changed(Windows.UI.Xaml.Controls.MediaElement.HTML_SHOWCONTROLS_PROPERTY_NAME, System.Nullable.getValue(Bridge.cast(Bridge.unbox(e.NewValue, System.Boolean), System.Boolean)));
                }
            }
        },
        fields: {
            _mediaElement: null,
            _mediaElement_ForAudioOnly_ForSimulatorOnly: null,
            _nameOfAssemblyThatSetTheSourceUri: null
        },
        props: {
            /**
             * Gets or sets a value that indicates whether media will begin playback automatically
             when the Source property is set.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.MediaElement
             * @function AutoPlay
             * @type boolean
             */
            AutoPlay: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.Controls.MediaElement.AutoPlayProperty), System.Boolean), System.Boolean));
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Controls.MediaElement.AutoPlayProperty, Bridge.box(value, System.Boolean, System.Boolean.toString));
                }
            },
            /**
             * // Gets a value that indicates whether media can be paused if the Pause method
            // is called.
            //
             * // Identifies the CanPause dependency property.
            //
             * // Gets the status of this MediaElement.
            //
             * // Identifies the CurrentState dependency property.
            //
             * Gets a value that reports whether the current source media is an audio-only
             media file.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.MediaElement
             * @function IsAudioOnly
             * @type boolean
             */
            IsAudioOnly: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.Controls.MediaElement.IsAudioOnlyProperty), System.Boolean), System.Boolean));
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Controls.MediaElement.IsAudioOnlyProperty, Bridge.box(value, System.Boolean, System.Boolean.toString));
                }
            },
            /**
             * Gets or sets a value that describes whether the media source currently loaded
             in the media engine should automatically set the position to the media start
             after reaching its end.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.MediaElement
             * @function IsLooping
             * @type boolean
             */
            IsLooping: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.Controls.MediaElement.IsLoopingProperty), System.Boolean), System.Boolean));
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Controls.MediaElement.IsLoopingProperty, Bridge.box(value, System.Boolean, System.Boolean.toString));
                }
            },
            /**
             * Gets or sets a value indicating whether the audio is muted.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.MediaElement
             * @function IsMuted
             * @type boolean
             */
            IsMuted: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.Controls.MediaElement.IsMutedProperty), System.Boolean), System.Boolean));
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Controls.MediaElement.IsMutedProperty, Bridge.box(value, System.Boolean, System.Boolean.toString));
                }
            },
            /**
             * // Gets or sets the current position of progress through the media's playback
            // time.
            //
             * // Identifies the Position dependency property.
            //
             * Gets or sets a media source on the MediaElement.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.MediaElement
             * @function Source
             * @type System.Uri
             */
            Source: {
                get: function () {
                    return Bridge.cast(this.GetValue(Windows.UI.Xaml.Controls.MediaElement.SourceProperty), System.Uri);
                },
                set: function (value) {

                    var callerAssemblyName = CSHTML5.Internal.INTERNAL_UriHelper.GetJavaScriptCallingAssembly();
                    this._nameOfAssemblyThatSetTheSourceUri = callerAssemblyName;
                    this.SetValue(Windows.UI.Xaml.Controls.MediaElement.SourceProperty, value);
                }
            },
            /**
             * Gets or sets the media's volume.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.MediaElement
             * @function Volume
             * @type number
             */
            Volume: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.Controls.MediaElement.VolumeProperty), System.Double), System.Double));
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Controls.MediaElement.VolumeProperty, Bridge.box(value, System.Double, System.Double.format, System.Double.getHashCode));
                }
            },
            ShowControls: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.Controls.MediaElement.ShowControlsProperty), System.Boolean), System.Boolean));
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Controls.MediaElement.ShowControlsProperty, Bridge.box(value, System.Boolean, System.Boolean.toString));
                }
            }
        },
        methods: {
            ManageVolume_Changed: function () {
                if (CSHTML5.Internal.INTERNAL_VisualTreeManager.IsElementInVisualTree(this)) {
                    if (this._mediaElement != null) {
                        CSHTML5.Internal.INTERNAL_HtmlDomManager.SetDomElementProperty(this._mediaElement, "volume", Bridge.box(this.Volume, System.Double, System.Double.format, System.Double.getHashCode));
                    } else if (this._mediaElement_ForAudioOnly_ForSimulatorOnly != null) {
                        this._mediaElement_ForAudioOnly_ForSimulatorOnly.Pause();
                    }
                }
            },
            /**
             * // Occurs when the value of the CurrentState property changes.
            //
             * // Occurs when the MediaElement is no longer playing audio or video.
            //
             * // Occurs when there is an error associated with the media Source.
            //
             * // Occurs when the media stream has been validated and opened, and the file
            // headers have been read.
            //
             * // Occurs when the value of the Volume property changes.
            //
             * Returns an enumeration value that describes the likelihood that the current
             MediaElement and its client configuration can play that media source.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Controls.MediaElement
             * @memberof Windows.UI.Xaml.Controls.MediaElement
             * @param   {string}                                        type    A string that describes the desired type as a string.
             * @return  {Windows.UI.Xaml.Media.MediaCanPlayResponse}            A value of the enumeration that describes the likelihood that the current
             media engine can play the source.
             */
            CanPlayType: function (type) {
                var canplay = CSHTML5.Internal.INTERNAL_HtmlDomManager.CallDomMethod(this._mediaElement, "canPlayType", [type]).ToString();
                return this.ConvertHtmlCanPlayTypeResult(canplay);
            },
            ConvertHtmlCanPlayTypeResult: function (htmlResult) {
                switch (htmlResult) {
                    case "maybe": 
                        return Windows.UI.Xaml.Media.MediaCanPlayResponse.Maybe;
                    case "probably": 
                        return Windows.UI.Xaml.Media.MediaCanPlayResponse.Probably;
                    default: 
                        return Windows.UI.Xaml.Media.MediaCanPlayResponse.NotSupported;
                }
            },
            /**
             * Pauses media at the current position.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Controls.MediaElement
             * @memberof Windows.UI.Xaml.Controls.MediaElement
             * @return  {void}
             */
            Pause: function () {
                if (CSHTML5.Internal.INTERNAL_VisualTreeManager.IsElementInVisualTree(this)) {

                    if (this._mediaElement != null) {
                        CSHTML5.Internal.INTERNAL_HtmlDomManager.CallDomMethod(this._mediaElement, "pause");
                    } else if (this._mediaElement_ForAudioOnly_ForSimulatorOnly != null) {
                        this._mediaElement_ForAudioOnly_ForSimulatorOnly.Pause();
                    }
                }
            },
            /**
             * Plays media from the current position.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Controls.MediaElement
             * @memberof Windows.UI.Xaml.Controls.MediaElement
             * @return  {void}
             */
            Play: function () {
                if (CSHTML5.Internal.INTERNAL_VisualTreeManager.IsElementInVisualTree(this)) {
                    if (this._mediaElement != null) {
                        CSHTML5.Internal.INTERNAL_HtmlDomManager.CallDomMethod(this._mediaElement, "play");
                    } else if (this._mediaElement_ForAudioOnly_ForSimulatorOnly != null) {
                        this._mediaElement_ForAudioOnly_ForSimulatorOnly.Play();
                    }
                }
            },
            /**
             * // Sets the Source property using the supplied stream.
            //
             * Stops and resets media to be played from the beginning.
             *
             * @instance
             * @public
             * @override
             * @this Windows.UI.Xaml.Controls.MediaElement
             * @memberof Windows.UI.Xaml.Controls.MediaElement
             * @param   {System.Object}    parentRef                         
             * @param   {System.Object}    domElementWhereToPlaceChildren
             * @return  {System.Object}
             */
            CreateDomElement: function (parentRef, domElementWhereToPlaceChildren) {
                domElementWhereToPlaceChildren.v = null;
                var outerDiv = { };
                var outerDivStyle = CSHTML5.Internal.INTERNAL_HtmlDomManager.CreateDomElementAppendItAndGetStyle("div", parentRef, this, outerDiv);
                return outerDiv.v;
            },
            ManageDomBoolProperty_Changed: function (htmlPropertyName, newValue) {
                if (CSHTML5.Internal.INTERNAL_VisualTreeManager.IsElementInVisualTree(this) && this._mediaElement != null) {
                    if (newValue) {
                        CSHTML5.Internal.INTERNAL_HtmlDomManager.SetDomElementAttribute(this._mediaElement, htmlPropertyName, "true");
                    } else {
                        CSHTML5.Internal.INTERNAL_HtmlDomManager.RemoveDomElementAttribute(this._mediaElement, htmlPropertyName, true);
                    }
                }
            },
            Refresh_SimulatorOnly: function () {
                this._mediaElement_ForAudioOnly_ForSimulatorOnly.IsMuted = this.IsMuted;
                this._mediaElement_ForAudioOnly_ForSimulatorOnly.Volume = this.Volume;
            },
            SimulatorMediaElement_Loaded: function () {
                if (this.AutoPlay) {
                    this._mediaElement_ForAudioOnly_ForSimulatorOnly.Play();
                }
            },
            SimulatorMediaElement_MediaEnded: function () {
                if (this.IsLooping) {
                    this._mediaElement_ForAudioOnly_ForSimulatorOnly.Stop();
                    this._mediaElement_ForAudioOnly_ForSimulatorOnly.Play();
                }
            },
            Refresh: function () {
                this.ManageDomBoolProperty_Changed(Windows.UI.Xaml.Controls.MediaElement.HTML_SHOWCONTROLS_PROPERTY_NAME, this.ShowControls);
                this.ManageDomBoolProperty_Changed(Windows.UI.Xaml.Controls.MediaElement.HTML_AUTOPLAY_PROPERTY_NAME, this.AutoPlay);
                this.ManageDomBoolProperty_Changed(Windows.UI.Xaml.Controls.MediaElement.HTML_ISLOOPING_PROPERTY_NAME, this.IsLooping);
                this.ManageDomBoolProperty_Changed(Windows.UI.Xaml.Controls.MediaElement.HTML_ISMUTED_PROPERTY_NAME, this.IsMuted);
                this.ManageVolume_Changed();
            }
        }
    });

    /**
     * Displays content on top of existing content, within the bounds of the application window.
     *
     * @public
     * @class Windows.UI.Xaml.Controls.Primitives.Popup
     * @augments Windows.UI.Xaml.FrameworkElement
     */
    Bridge.define("Windows.UI.Xaml.Controls.Primitives.Popup", {
        inherits: [Windows.UI.Xaml.FrameworkElement],
        statics: {
            fields: {
                /**
                 * Gets the identifier for the Child dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Controls.Primitives.Popup
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                ChildProperty: null,
                /**
                 * Gets the identifier for the IsOpen dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Controls.Primitives.Popup
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                IsOpenProperty: null,
                /**
                 * Gets the identifier for the HorizontalOffset dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Controls.Primitives.Popup
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                HorizontalOffsetProperty: null,
                /**
                 * Gets the identifier for the VerticalOffset dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Controls.Primitives.Popup
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                VerticalOffsetProperty: null,
                /**
                 * Identifies the HorizontalContentAlignment dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Controls.Primitives.Popup
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                HorizontalContentAlignmentProperty: null,
                /**
                 * Identifies the VerticalContentAlignment dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Controls.Primitives.Popup
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                VerticalContentAlignmentProperty: null
            },
            ctors: {
                init: function () {
                    var $t;
                    this.ChildProperty = Windows.UI.Xaml.DependencyProperty.Register("Child", Windows.UI.Xaml.UIElement, Windows.UI.Xaml.Controls.Primitives.Popup, ($t = new Windows.UI.Xaml.PropertyMetadata.$ctor2(null, Windows.UI.Xaml.Controls.Primitives.Popup.Child_Changed), $t.CallPropertyChangedWhenLoadedIntoVisualTree = Windows.UI.Xaml.WhenToCallPropertyChangedEnum.IfPropertyIsSet, $t));
                    this.IsOpenProperty = Windows.UI.Xaml.DependencyProperty.Register("IsOpen", System.Boolean, Windows.UI.Xaml.Controls.Primitives.Popup, ($t = new Windows.UI.Xaml.PropertyMetadata.$ctor2(Bridge.box(false, System.Boolean, System.Boolean.toString), Windows.UI.Xaml.Controls.Primitives.Popup.IsOpen_Changed), $t.CallPropertyChangedWhenLoadedIntoVisualTree = Windows.UI.Xaml.WhenToCallPropertyChangedEnum.IfPropertyIsSet, $t));
                    this.HorizontalOffsetProperty = Windows.UI.Xaml.DependencyProperty.Register("HorizontalOffset", System.Double, Windows.UI.Xaml.Controls.Primitives.Popup, ($t = new Windows.UI.Xaml.PropertyMetadata.$ctor2(Bridge.box(0.0, System.Double, System.Double.format, System.Double.getHashCode), Windows.UI.Xaml.Controls.Primitives.Popup.HorizontalOffset_Changed), $t.CallPropertyChangedWhenLoadedIntoVisualTree = Windows.UI.Xaml.WhenToCallPropertyChangedEnum.IfPropertyIsSet, $t));
                    this.VerticalOffsetProperty = Windows.UI.Xaml.DependencyProperty.Register("VerticalOffset", System.Double, Windows.UI.Xaml.Controls.Primitives.Popup, ($t = new Windows.UI.Xaml.PropertyMetadata.$ctor2(Bridge.box(0.0, System.Double, System.Double.format, System.Double.getHashCode), Windows.UI.Xaml.Controls.Primitives.Popup.VerticalOffset_Changed), $t.CallPropertyChangedWhenLoadedIntoVisualTree = Windows.UI.Xaml.WhenToCallPropertyChangedEnum.IfPropertyIsSet, $t));
                    this.HorizontalContentAlignmentProperty = Windows.UI.Xaml.DependencyProperty.Register("HorizontalContentAlignment", Windows.UI.Xaml.HorizontalAlignment, Windows.UI.Xaml.Controls.Primitives.Popup, ($t = new Windows.UI.Xaml.PropertyMetadata.$ctor2(Bridge.box(Windows.UI.Xaml.HorizontalAlignment.Left, Windows.UI.Xaml.HorizontalAlignment, System.Enum.toStringFn(Windows.UI.Xaml.HorizontalAlignment)), Windows.UI.Xaml.Controls.Primitives.Popup.HorizontalContentAlignment_Changed), $t.CallPropertyChangedWhenLoadedIntoVisualTree = Windows.UI.Xaml.WhenToCallPropertyChangedEnum.IfPropertyIsSet, $t));
                    this.VerticalContentAlignmentProperty = Windows.UI.Xaml.DependencyProperty.Register("VerticalContentAlignment", Windows.UI.Xaml.VerticalAlignment, Windows.UI.Xaml.Controls.Primitives.Popup, ($t = new Windows.UI.Xaml.PropertyMetadata.$ctor2(Bridge.box(Windows.UI.Xaml.VerticalAlignment.Top, Windows.UI.Xaml.VerticalAlignment, System.Enum.toStringFn(Windows.UI.Xaml.VerticalAlignment)), Windows.UI.Xaml.Controls.Primitives.Popup.VerticalContentAlignment_Changed), $t.CallPropertyChangedWhenLoadedIntoVisualTree = Windows.UI.Xaml.WhenToCallPropertyChangedEnum.IfPropertyIsSet, $t));
                }
            },
            methods: {
                Child_Changed: function (d, e) {
                    var popup = Bridge.cast(d, Windows.UI.Xaml.Controls.Primitives.Popup);
                    if (popup._isVisible) {
                        var newContent = Bridge.cast(e.NewValue, Windows.UI.Xaml.UIElement);
                        popup._outerBorder.Child = newContent;
                    } else {
                    }
                },
                IsOpen_Changed: function (d, e) {
                    var popup = Bridge.cast(d, Windows.UI.Xaml.Controls.Primitives.Popup);
                    var isOpen = System.Nullable.getValue(Bridge.cast(Bridge.unbox(e.NewValue, System.Boolean), System.Boolean));
                    if (popup.PlacementTarget == null) {

                        if (isOpen) {
                            popup.ShowPopupRootIfNotAlreadyVisible();
                        } else {
                            popup.HidePopupRootIfVisible();
                        }
                    } else {

                        var targetElement = popup.PlacementTarget;
                        if (CSHTML5.Internal.INTERNAL_VisualTreeManager.IsElementInVisualTree(targetElement)) {
                            if (isOpen) {
                                Windows.UI.Xaml.Window.Current.INTERNAL_PositionsWatcher.AddControlToWatch(targetElement, Bridge.fn.cacheBind(popup, popup.RefreshPopupPosition));
                                popup.ShowPopupRootIfNotAlreadyVisible();


                                var placementTargetPosition = DotNetForHtml5.Core.INTERNAL_PopupsManager.GetUIElementAbsolutePosition(targetElement);

                                var elementCurrentSize = new Windows.Foundation.Size();
                                if (Bridge.is(targetElement, Windows.UI.Xaml.FrameworkElement)) {
                                    elementCurrentSize = Bridge.cast(targetElement, Windows.UI.Xaml.FrameworkElement).INTERNAL_GetActualWidthAndHeight();
                                } else {
                                    elementCurrentSize = new Windows.Foundation.Size.ctor();
                                }

                                popup.RefreshPopupPosition(placementTargetPosition.$clone(), elementCurrentSize.$clone());
                            } else {
                                Windows.UI.Xaml.Window.Current.INTERNAL_PositionsWatcher.RemoveControlToWatch(popup._controlToWatch);
                                popup.HidePopupRootIfVisible();
                            }
                        }
                    }
                },
                HorizontalOffset_Changed: function (d, e) {
                    var popup = Bridge.cast(d, Windows.UI.Xaml.Controls.Primitives.Popup);
                    popup.RepositionPopup(System.Nullable.getValue(Bridge.cast(Bridge.unbox(e.NewValue, System.Double), System.Double)), popup.VerticalOffset);
                },
                VerticalOffset_Changed: function (d, e) {
                    var popup = Bridge.cast(d, Windows.UI.Xaml.Controls.Primitives.Popup);
                    popup.RepositionPopup(popup.HorizontalOffset, System.Nullable.getValue(Bridge.cast(Bridge.unbox(e.NewValue, System.Double), System.Double)));
                },
                HorizontalContentAlignment_Changed: function (d, e) {
                    var popup = Bridge.cast(d, Windows.UI.Xaml.Controls.Primitives.Popup);
                    if (popup._outerBorder != null) {
                        popup._outerBorder.HorizontalAlignment = System.Nullable.getValue(Bridge.cast(Bridge.unbox(e.NewValue, Windows.UI.Xaml.HorizontalAlignment), System.Int32));
                    }
                },
                VerticalContentAlignment_Changed: function (d, e) {
                    var popup = Bridge.cast(d, Windows.UI.Xaml.Controls.Primitives.Popup);
                    if (popup._outerBorder != null) {
                        popup._outerBorder.VerticalAlignment = System.Nullable.getValue(Bridge.cast(Bridge.unbox(e.NewValue, Windows.UI.Xaml.VerticalAlignment), System.Int32));
                    }
                }
            }
        },
        fields: {
            _popupRoot: null,
            _outerBorder: null,
            _isVisible: false,
            _referencePosition: null,
            _controlToWatch: null,
            ParentPopup: null,
            _placementTarget: null,
            _placement: 0,
            _stayOpen: false
        },
        events: {
            INTERNAL_PopupMoved: null,
            ClosedDueToOutsideClick: null
        },
        props: {
            PopupRoot: {
                get: function () {
                    return this._popupRoot;
                }
            },
            /**
             * Gets or Sets the UIElement that the Popup will stick to. A null value will make the Popup stay at its originally defined position.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.Primitives.Popup
             * @function PlacementTarget
             * @type Windows.UI.Xaml.UIElement
             */
            PlacementTarget: {
                get: function () {
                    return this._placementTarget;
                },
                set: function (value) {
                    this._placementTarget = value;
                }
            },
            /**
             * Gets or sets the position of the Popup relative to the UIElement it is attached to. NOTE: The only currently supported positions are Right and Bottom.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.Primitives.Popup
             * @function Placement
             * @type Windows.UI.Xaml.Controls.Primitives.PlacementMode
             */
            Placement: {
                get: function () {
                    return this._placement;
                },
                set: function (value) {
                    this._placement = value;
                }
            },
            /**
             * Gets or sets the content to be hosted in the popup.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.Primitives.Popup
             * @function Child
             * @type Windows.UI.Xaml.UIElement
             */
            Child: {
                get: function () {
                    return Bridge.cast(this.GetValue(Windows.UI.Xaml.Controls.Primitives.Popup.ChildProperty), Windows.UI.Xaml.UIElement);
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Controls.Primitives.Popup.ChildProperty, value);
                }
            },
            /**
             * Gets or sets whether the popup is currently displayed on the screen.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.Primitives.Popup
             * @function IsOpen
             * @type boolean
             */
            IsOpen: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.Controls.Primitives.Popup.IsOpenProperty), System.Boolean), System.Boolean));
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Controls.Primitives.Popup.IsOpenProperty, Bridge.box(value, System.Boolean, System.Boolean.toString));
                }
            },
            /**
             * Gets or sets the distance between the left side of the application window and the left side of the popup.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.Primitives.Popup
             * @function HorizontalOffset
             * @type number
             */
            HorizontalOffset: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.Controls.Primitives.Popup.HorizontalOffsetProperty), System.Double), System.Double));
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Controls.Primitives.Popup.HorizontalOffsetProperty, Bridge.box(value, System.Double, System.Double.format, System.Double.getHashCode));
                }
            },
            /**
             * Gets or sets the distance between the top of the application window and the top of the popup.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.Primitives.Popup
             * @function VerticalOffset
             * @type number
             */
            VerticalOffset: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.Controls.Primitives.Popup.VerticalOffsetProperty), System.Double), System.Double));
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Controls.Primitives.Popup.VerticalOffsetProperty, Bridge.box(value, System.Double, System.Double.format, System.Double.getHashCode));
                }
            },
            /**
             * Gets or sets the horizontal alignment of the control's content.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.Primitives.Popup
             * @function HorizontalContentAlignment
             * @type Windows.UI.Xaml.HorizontalAlignment
             */
            HorizontalContentAlignment: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.Controls.Primitives.Popup.HorizontalContentAlignmentProperty), Windows.UI.Xaml.HorizontalAlignment), System.Int32));
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Controls.Primitives.Popup.HorizontalContentAlignmentProperty, Bridge.box(value, Windows.UI.Xaml.HorizontalAlignment, System.Enum.toStringFn(Windows.UI.Xaml.HorizontalAlignment)));
                }
            },
            /**
             * Gets or sets the vertical alignment of the control's content.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.Primitives.Popup
             * @function VerticalContentAlignment
             * @type Windows.UI.Xaml.VerticalAlignment
             */
            VerticalContentAlignment: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.Controls.Primitives.Popup.VerticalContentAlignmentProperty), Windows.UI.Xaml.VerticalAlignment), System.Int32));
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Controls.Primitives.Popup.VerticalContentAlignmentProperty, Bridge.box(value, Windows.UI.Xaml.VerticalAlignment, System.Enum.toStringFn(Windows.UI.Xaml.VerticalAlignment)));
                }
            },
            StayOpen: {
                get: function () {
                    return this._stayOpen;
                },
                set: function (value) {
                    this._stayOpen = value;
                }
            }
        },
        ctors: {
            init: function () {
                this._referencePosition = new Windows.Foundation.Point();
                this._stayOpen = true;
            }
        },
        methods: {
            INTERNAL_OnDetachedFromVisualTree: function () {
                Windows.UI.Xaml.FrameworkElement.prototype.INTERNAL_OnDetachedFromVisualTree.call(this);


                this.HidePopupRootIfVisible();
            },
            RefreshPopupPosition: function (placementTargetPosition, placementTargetSize) {
                if (this.PlacementTarget != null && !CSHTML5.Internal.INTERNAL_VisualTreeManager.IsElementInVisualTree(this.PlacementTarget)) {
                    Windows.UI.Xaml.Window.Current.INTERNAL_PositionsWatcher.RemoveControlToWatch(this._controlToWatch);
                    this.HidePopupRootIfVisible();
                } else if (this.PlacementTarget != null) {
                    switch (this.Placement) {
                        case Windows.UI.Xaml.Controls.Primitives.PlacementMode.Bottom: 
                            if (!isNaN(placementTargetSize.Height)) {
                                placementTargetPosition.Y += placementTargetSize.Height;
                            }
                            break;
                        default: 
                            if (!isNaN(placementTargetSize.Width)) {
                                placementTargetPosition.X += placementTargetSize.Width;
                            }
                            break;
                    }

                    this._referencePosition = placementTargetPosition.$clone();
                    this.RepositionPopup(this.HorizontalOffset, this.VerticalOffset);
                }

                if (!Bridge.staticEquals(this.INTERNAL_PopupMoved, null)) {
                    this.INTERNAL_PopupMoved(this, { });
                }
            },
            RepositionPopup: function (horizontalOffset, verticalOffset) {
                if (this._outerBorder != null) {
                    this._outerBorder.Margin = new Windows.UI.Xaml.Thickness.$ctor2(this._referencePosition.X + horizontalOffset, this._referencePosition.Y + verticalOffset, 0.0, 0.0);
                }
            },
            ShowPopupRootIfNotAlreadyVisible: function () {
                var $t, $t1;
                if (!this._isVisible) {

                    var child = this.Child;

                    var parentWindow = this.GetParentWindowOfPopup();

                    var popupRoot = DotNetForHtml5.Core.INTERNAL_PopupsManager.CreateAndAppendNewPopupRoot(parentWindow);
                    this._popupRoot = popupRoot;
                    this._popupRoot.INTERNAL_LinkedPopup = this;
                    this.UpdatePopupParent();

                    if (this._outerBorder != null) {
                        this._outerBorder.Child = null;
                    }

                    this._referencePosition = ($t = this.CalculateReferencePosition(parentWindow), $t != null ? $t : new Windows.Foundation.Point.ctor());

                    var transparentToClicks = (!this.IsHitTestVisible) || (Bridge.is(child, Windows.UI.Xaml.FrameworkElement) && !Bridge.cast(child, Windows.UI.Xaml.FrameworkElement).IsHitTestVisible);

                    this._outerBorder = ($t1 = new Windows.UI.Xaml.Controls.Border(), $t1.Margin = new Windows.UI.Xaml.Thickness.$ctor2(this._referencePosition.X + this.HorizontalOffset, this._referencePosition.Y + this.VerticalOffset, 0.0, 0.0), $t1.Child = child, $t1.HorizontalAlignment = this.HorizontalContentAlignment, $t1.VerticalAlignment = this.VerticalContentAlignment, $t1.INTERNAL_ForceEnableAllPointerEvents = !transparentToClicks, $t1);

                    this._outerBorder.addLoaded(function (s, e) {
                        popupRoot.INTERNAL_RaiseLoadedEvent();
                    });

                    popupRoot.Content = this._outerBorder;
                    this._isVisible = true;
                } else {
                }
            },
            HidePopupRootIfVisible: function () {
                if (this._isVisible) {
                    var popupRoot = this._popupRoot;
                    popupRoot.INTERNAL_LinkedPopup = null;
                    popupRoot.Content = null;
                    DotNetForHtml5.Core.INTERNAL_PopupsManager.RemovePopupRoot(popupRoot);
                    this._popupRoot = null;
                    this._isVisible = false;
                } else {
                }
            },
            GetParentWindowOfPopup: function () {
                var parentWindow = (this._placementTarget != null ? this._placementTarget.INTERNAL_ParentWindow : null) || this.INTERNAL_ParentWindow || Windows.UI.Xaml.Application.Current.MainWindow;

                return parentWindow;
            },
            CalculateReferencePosition: function (parentWindow) {
                var placementTarget = this.PlacementTarget;
                if (placementTarget != null && CSHTML5.Internal.INTERNAL_VisualTreeManager.IsElementInVisualTree(placementTarget)) {
                    var gt = placementTarget.TransformToVisual(parentWindow);
                    var p = gt.Transform(new Windows.Foundation.Point.$ctor1(0.0, 0.0));
                } else if (CSHTML5.Internal.INTERNAL_VisualTreeManager.IsElementInVisualTree(this)) {
                    var gt1 = this.TransformToVisual(parentWindow);
                    var p1 = gt1.Transform(new Windows.Foundation.Point.$ctor1(0.0, 0.0));
                    return p1.$clone();
                }
                return new Windows.Foundation.Point.ctor();
            },
            CloseFromAnOutsideClick: function () {
                if (!Bridge.staticEquals(this.ClosedDueToOutsideClick, null)) {
                    this.ClosedDueToOutsideClick(this, { });
                }

                if (this.IsOpen) {
                    this.IsOpen = false;
                }
            },
            UpdatePopupParent: function () {

                var element = this.PlacementTarget || Bridge.cast(this.INTERNAL_VisualParent, Windows.UI.Xaml.UIElement);

                if (element == null) {
                    this.ParentPopup = null;
                    return;
                }
                while (true) {
                    if (Bridge.is(element, Windows.UI.Xaml.Controls.Primitives.PopupRoot)) {
                        this.ParentPopup = Bridge.cast(element, Windows.UI.Xaml.Controls.Primitives.PopupRoot).INTERNAL_LinkedPopup;
                        return;
                    }

                    var obj = Windows.UI.Xaml.Media.VisualTreeHelper.GetParent(element);
                    if (Bridge.is(obj, Windows.UI.Xaml.UIElement)) {
                        element = Bridge.cast(obj, Windows.UI.Xaml.UIElement);
                    } else {
                        break;
                    }
                }

            }
        }
    });

    Bridge.define("Windows.UI.Xaml.Controls.Primitives.PopupRoot", {
        inherits: [Windows.UI.Xaml.FrameworkElement],
        statics: {
            fields: {
                /**
                 * Identifies the PopupRoot.Content dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Controls.Primitives.PopupRoot
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                ContentProperty: null
            },
            ctors: {
                init: function () {
                    var $t;
                    this.ContentProperty = Windows.UI.Xaml.DependencyProperty.Register("Content", Windows.UI.Xaml.UIElement, Windows.UI.Xaml.Controls.Primitives.PopupRoot, ($t = new Windows.UI.Xaml.PropertyMetadata.$ctor2(null, Windows.UI.Xaml.Controls.Primitives.PopupRoot.Content_Changed), $t.CallPropertyChangedWhenLoadedIntoVisualTree = Windows.UI.Xaml.WhenToCallPropertyChangedEnum.IfPropertyIsSet, $t));
                }
            },
            methods: {
                Content_Changed: function (d, e) {
                    var parent = Bridge.cast(d, Windows.UI.Xaml.UIElement);
                    var oldChild = Bridge.cast(e.OldValue, Windows.UI.Xaml.UIElement);
                    var newChild = Bridge.cast(e.NewValue, Windows.UI.Xaml.UIElement);

                    CSHTML5.Internal.INTERNAL_VisualTreeManager.DetachVisualChildIfNotNull(oldChild, parent);
                    CSHTML5.Internal.INTERNAL_VisualTreeManager.AttachVisualChildIfNotAlreadyAttached(newChild, parent);
                }
            }
        },
        fields: {
            INTERNAL_UniqueIndentifier: null,
            INTERNAL_LinkedPopup: null
        },
        props: {
            /**
             * Gets or sets the visual root of a popup
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.Primitives.PopupRoot
             * @function Content
             * @type Windows.UI.Xaml.UIElement
             */
            Content: {
                get: function () {
                    return Bridge.cast(this.GetValue(Windows.UI.Xaml.Controls.Primitives.PopupRoot.ContentProperty), Windows.UI.Xaml.UIElement);
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Controls.Primitives.PopupRoot.ContentProperty, value);
                }
            }
        },
        ctors: {
            ctor: function (uniqueIdentifier, parentWindow) {
                this.$initialize();
                Windows.UI.Xaml.FrameworkElement.ctor.call(this);
                this.INTERNAL_UniqueIndentifier = uniqueIdentifier;
                this.INTERNAL_ParentWindow = parentWindow;

                this.addLoaded(Bridge.fn.cacheBind(this, this.PopupRoot_Loaded));
            }
        },
        methods: {
            PopupRoot_Loaded: function (sender, e) {
                if (this.INTERNAL_LinkedPopup != null && this.INTERNAL_LinkedPopup.INTERNAL_VisualParent == null) {
                    this.INTERNAL_LinkedPopup.INTERNAL_RaiseLoadedEvent();
                }
            }
        }
    });

    /**
     * Provides a control that hosts an HTML page in an app using an iframe.
     Note: some websites explicitly forbid being embedded in an iframe.
     Note: to embed a piece of HTML code without using an iframe, use the HtmlPresenter control instead.
     *
     * @public
     * @class Windows.UI.Xaml.Controls.WebView
     * @augments Windows.UI.Xaml.FrameworkElement
     */
    Bridge.define("Windows.UI.Xaml.Controls.WebView", {
        inherits: [Windows.UI.Xaml.FrameworkElement],
        statics: {
            fields: {
                /**
                 * Gets or sets the Uniform Resource Identifier (URI) source of the HTML content to display in the control inside an iframe.
                 Note: some websites explicitly forbid being embedded in an iframe.
                 Note: to embed a piece of HTML code without using an iframe, use the HtmlPresenter control instead.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Controls.WebView
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                SourceProperty: null
            },
            ctors: {
                init: function () {
                    var $t;
                    this.SourceProperty = Windows.UI.Xaml.DependencyProperty.Register("Source", System.Uri, Windows.UI.Xaml.Controls.WebView, ($t = new Windows.UI.Xaml.PropertyMetadata.$ctor2(null, Windows.UI.Xaml.Controls.WebView.Source_Changed), $t.CallPropertyChangedWhenLoadedIntoVisualTree = Windows.UI.Xaml.WhenToCallPropertyChangedEnum.IfPropertyIsSet, $t));
                }
            },
            methods: {
                Source_Changed: function (d, e) {
                    var webView = Bridge.cast(d, Windows.UI.Xaml.Controls.WebView);

                    if (webView._isLoaded) {
                        var source = webView.Source;
                        if (System.Uri.notEquals(source, null) && !System.String.isNullOrEmpty(source.getOriginalString())) {
                            var uri = CSHTML5.Internal.INTERNAL_UriHelper.ConvertToHtml5Path(source.getOriginalString(), null);
                            webView._iFrame.src = uri;
                        } else {
                            webView._iFrame.src = 'about:blank';
                        }
                    }
                }
            }
        },
        fields: {
            _iFrame: null,
            _htmlString: null
        },
        props: {
            /**
             * Gets or sets the Uniform Resource Identifier (URI) source of the HTML content to display in the control inside an iframe.
             Note: some websites explicitly forbid being embedded in an iframe.
             Note: to embed a piece of HTML code without using an iframe, use the HtmlPresenter control instead.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.WebView
             * @function Source
             * @type System.Uri
             */
            Source: {
                get: function () {
                    return Bridge.cast(this.GetValue(Windows.UI.Xaml.Controls.WebView.SourceProperty), System.Uri);
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Controls.WebView.SourceProperty, value);
                }
            }
        },
        methods: {
            CreateDomElement: function (parentRef, domElementWhereToPlaceChildren) {
                var outerDiv = { };
                var outerDivStyle = CSHTML5.Internal.INTERNAL_HtmlDomManager.CreateDomElementAppendItAndGetStyle("div", parentRef, this, outerDiv);
                outerDivStyle.width = "100%";
                outerDivStyle.height = "100%";

                var iFrameStyle = CSHTML5.Internal.INTERNAL_HtmlDomManager.CreateDomElementAppendItAndGetStyle("iframe", outerDiv.v, this, Bridge.ref(this, "_iFrame"));
                iFrameStyle.width = "100%";
                iFrameStyle.height = "100%";
                iFrameStyle.border = "none";

                var source = this.Source;
                if (System.Uri.notEquals(source, null) && !System.String.isNullOrEmpty(source.getOriginalString())) {
                    this._iFrame.src = source.getOriginalString();
                } else if (this._htmlString != null) {
srcDoc.set(                    this._iFrame, this._htmlString);
                } else {
                    this._iFrame.src = 'about:blank';
                }

                domElementWhereToPlaceChildren.v = this._iFrame;
                return outerDiv.v;
            },
            /**
             * Loads the HTML content at the specified Uniform Resource Identifier (URI) inside an iframe.
             Note: some websites explicitly forbid being embedded in an iframe.
             Note: to embed a piece of HTML code without using an iframe, use the HtmlPresenter control instead.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Controls.WebView
             * @memberof Windows.UI.Xaml.Controls.WebView
             * @param   {System.Uri}    source    The Uniform Resource Identifier (URI) to load.
             * @return  {void}
             */
            Navigate: function (source) {
                this.Source = source;
            },
            /**
             * Displays the specified HTML content inside an iframe.
             Note: to embed a piece of HTML code without using an iframe, use the HtmlPresenter control instead.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Controls.WebView
             * @memberof Windows.UI.Xaml.Controls.WebView
             * @param   {string}    text    The HTML content to display in the control.
             * @return  {void}
             */
            NavigateToString: function (text) {
                this._htmlString = text;

                if (this._isLoaded) {
                    if (this._htmlString != null) {
srcDoc.set(                        this._iFrame, this._htmlString);
                    } else {
                        this._iFrame.src = 'about:blank';
                    }
                }
            }
        }
    });

    /**
     * Animates the value of a Color property between two target values using linear
     interpolation over a specified Duration.
     *
     * @public
     * @class Windows.UI.Xaml.Media.Animation.ColorAnimation
     * @augments Windows.UI.Xaml.Media.Animation.AnimationTimeline
     */
    Bridge.define("Windows.UI.Xaml.Media.Animation.ColorAnimation", {
        inherits: [Windows.UI.Xaml.Media.Animation.AnimationTimeline],
        statics: {
            fields: {
                /**
                 * Identifies the EasingFunction dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Media.Animation.ColorAnimation
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                EasingFunctionProperty: null,
                /**
                 * Identifies the From dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Media.Animation.ColorAnimation
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                FromProperty: null,
                /**
                 * Identifies the To dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Media.Animation.ColorAnimation
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                ToProperty: null
            },
            ctors: {
                init: function () {
                    this.EasingFunctionProperty = Windows.UI.Xaml.DependencyProperty.Register("EasingFunction", Windows.UI.Xaml.Media.Animation.EasingFunctionBase, Windows.UI.Xaml.Media.Animation.ColorAnimation, new Windows.UI.Xaml.PropertyMetadata.$ctor4(null));
                    this.FromProperty = Windows.UI.Xaml.DependencyProperty.Register("From", System.Nullable$1(Windows.UI.Color), Windows.UI.Xaml.Media.Animation.ColorAnimation, new Windows.UI.Xaml.PropertyMetadata.$ctor4(null));
                    this.ToProperty = Windows.UI.Xaml.DependencyProperty.Register("To", System.Nullable$1(Windows.UI.Color), Windows.UI.Xaml.Media.Animation.ColorAnimation, new Windows.UI.Xaml.PropertyMetadata.$ctor4(null));
                }
            },
            methods: {
                TryStartAnimation: function (target, cssEquivalent, from, to, Duration, easingFunction, visualStateGroupName, callbackForWhenfinished) {
                    var $t, $t1, $t2;
                    if (callbackForWhenfinished === void 0) { callbackForWhenfinished = null; }
                    if (cssEquivalent.Name != null && cssEquivalent.Name.Count !== 0) {
                        var uiElement = cssEquivalent.UIElement || (Bridge.as(target, Windows.UI.Xaml.UIElement));

                        var hasTemplate = (Bridge.is(uiElement, Windows.UI.Xaml.Controls.Control)) && Bridge.cast(uiElement, Windows.UI.Xaml.Controls.Control).HasTemplate;

                        if (!hasTemplate || cssEquivalent.ApplyAlsoWhenThereIsAControlTemplate) {
                            if (cssEquivalent.DomElement == null && uiElement != null) {
                                cssEquivalent.DomElement = uiElement.INTERNAL_OuterDomElement;
                            }
                            if (cssEquivalent.DomElement != null) {
                                if (Bridge.staticEquals(cssEquivalent.Value, null)) {
                                    cssEquivalent.Value = function (finalInstance, value) {
                                        var $t;
                                        if (value == null) {
                                            return "";
                                        } else {
                                            var valuesDict = new (System.Collections.Generic.Dictionary$2(System.String,System.Object)).ctor();
                                            $t = Bridge.getEnumerator(cssEquivalent.Name);
                                            try {
                                                while ($t.moveNext()) {
                                                    var name = $t.Current;
                                                    if (!System.String.endsWith(name, "Alpha")) {
                                                        valuesDict.add(name, System.Nullable.getValue(Bridge.cast(Bridge.unbox(value, Windows.UI.Color), Windows.UI.Color)).INTERNAL_ToHtmlStringForVelocity());
                                                    } else {
                                                        valuesDict.add(name, Bridge.box(System.Nullable.getValue(Bridge.cast(Bridge.unbox(value, Windows.UI.Color), Windows.UI.Color)).A / 255, System.Double, System.Double.format, System.Double.getHashCode));
                                                    }
                                                }
                                            } finally {
                                                if (Bridge.is($t, System.IDisposable)) {
                                                    $t.System$IDisposable$Dispose();
                                                }
                                            }
                                            return valuesDict;
                                        }
                                    };
                                }
                                var cssValue = cssEquivalent.Value(target, to);



                                var newObj = new Object();

                                if (Windows.UI.Xaml.Media.Animation.AnimationHelpers.IsValueNull(System.Nullable.lift1("$clone", from))) {
                                    if (!(Bridge.is(cssValue, System.Collections.Generic.Dictionary$2(System.String,System.Object)))) {
                                        $t = Bridge.getEnumerator(cssEquivalent.Name);
                                        try {
                                            while ($t.moveNext()) {
                                                var csspropertyName = $t.Current;
                                                if (!Bridge.referenceEquals(csspropertyName, "background")) {
                                                    newObj[csspropertyName] = cssValue;
                                                }
                                            }
                                        } finally {
                                            if (Bridge.is($t, System.IDisposable)) {
                                                $t.System$IDisposable$Dispose();
                                            }
                                        }
                                    } else {
                                        var cssValueAsDictionary = Bridge.cast(cssValue, System.Collections.Generic.Dictionary$2(System.String,System.Object));
                                        $t1 = Bridge.getEnumerator(cssEquivalent.Name);
                                        try {
                                            while ($t1.moveNext()) {
                                                var csspropertyName1 = $t1.Current;
                                                if (!Bridge.referenceEquals(csspropertyName1, "background")) {
                                                    newObj[csspropertyName1] = cssValueAsDictionary.getItem(csspropertyName1);
                                                }
                                            }
                                        } finally {
                                            if (Bridge.is($t1, System.IDisposable)) {
                                                $t1.System$IDisposable$Dispose();
                                            }
                                        }
                                    }
                                } else {
                                    var fromCssValue = cssEquivalent.Value(target, System.Nullable.lift1("$clone", from));
                                    $t2 = Bridge.getEnumerator(cssEquivalent.Name);
                                    try {
                                        while ($t2.moveNext()) {
                                            var csspropertyName2 = $t2.Current;
                                            if (!Bridge.referenceEquals(csspropertyName2, "background")) {
                                                var currentCssValue = cssValue;
                                                if (Bridge.is(cssValue, System.Collections.Generic.Dictionary$2(System.String,System.Object))) {
                                                    currentCssValue = Bridge.cast(cssValue, System.Collections.Generic.Dictionary$2(System.String,System.Object)).getItem(csspropertyName2);
                                                }
                                                var currentFromCssValue = fromCssValue;
                                                if (Bridge.is(fromCssValue, System.Collections.Generic.Dictionary$2(System.String,System.Object))) {
                                                    currentFromCssValue = Bridge.cast(fromCssValue, System.Collections.Generic.Dictionary$2(System.String,System.Object)).getItem(csspropertyName2);
                                                }
                                                newObj[csspropertyName2] = [currentCssValue, currentFromCssValue];
                                            }
                                        }
                                    } finally {
                                        if (Bridge.is($t2, System.IDisposable)) {
                                            $t2.System$IDisposable$Dispose();
                                        }
                                    }
                                }

                                Windows.UI.Xaml.Media.Animation.AnimationHelpers.CallVelocity(cssEquivalent.DomElement, Duration.$clone(), easingFunction, visualStateGroupName, callbackForWhenfinished, newObj);

                                return true;
                            }
                        }
                        return false;
                    } else {
                        throw new System.InvalidOperationException.$ctor1("Please set the Name property of the CSSEquivalent class.");
                    }
                }
            }
        },
        fields: {
            _animationID: null
        },
        props: {
            /**
             * Gets or sets the easing function applied to this animation.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Media.Animation.ColorAnimation
             * @function EasingFunction
             * @type Windows.UI.Xaml.Media.Animation.EasingFunctionBase
             */
            EasingFunction: {
                get: function () {
                    return Bridge.cast(this.GetValue(Windows.UI.Xaml.Media.Animation.ColorAnimation.EasingFunctionProperty), Windows.UI.Xaml.Media.Animation.EasingFunctionBase);
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Media.Animation.ColorAnimation.EasingFunctionProperty, value);
                }
            },
            /**
             * Gets or sets the animation's starting value.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Media.Animation.ColorAnimation
             * @function From
             * @type ?Windows.UI.Color
             */
            From: {
                get: function () {
                    return Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.Media.Animation.ColorAnimation.FromProperty), Windows.UI.Color), Windows.UI.Color, true);
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Media.Animation.ColorAnimation.FromProperty, System.Nullable.lift1("$clone", value));
                }
            },
            /**
             * Gets or sets the animation's ending value.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Media.Animation.ColorAnimation
             * @function To
             * @type ?Windows.UI.Color
             */
            To: {
                get: function () {
                    return Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.Media.Animation.ColorAnimation.ToProperty), Windows.UI.Color), Windows.UI.Color, true);
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Media.Animation.ColorAnimation.ToProperty, System.Nullable.lift1("$clone", value));
                }
            }
        },
        ctors: {
            init: function () {
                this._animationID = new System.Guid();
            }
        },
        methods: {
            GetTargetInformation: function (parameters) {
                var $t;
                this._parameters$1 = parameters;
                var target = { };
                var propertyPath = { };
                var targetBeforePath = { };
                this.GetPropertyPathAndTargetBeforePath(parameters.Target, targetBeforePath, propertyPath, parameters.IsTargetParentTheTarget);
                var parentElement = targetBeforePath.v;
                $t = Bridge.getEnumerator(this.GoThroughElementsToAccessProperty(propertyPath.v, targetBeforePath.v), System.Tuple$3(Windows.UI.Xaml.DependencyObject,Windows.UI.Xaml.DependencyProperty,System.Nullable$1(System.Int32)));
                try {
                    while ($t.moveNext()) {
                        var element = $t.Current;
                        var depObject = element.Item1;
                        var depProp = element.Item2;
                        var index = element.Item3;
                        if (Bridge.is(depObject, Windows.UI.Xaml.ICloneOnAnimation)) {
                            if (!Bridge.cast(depObject, Windows.UI.Xaml.ICloneOnAnimation).Windows$UI$Xaml$ICloneOnAnimation$IsAlreadyAClone()) {
                                var clone = Bridge.cast(depObject, Windows.UI.Xaml.ICloneOnAnimation).Windows$UI$Xaml$ICloneOnAnimation$Clone();
                                if (index != null) {
                                    Bridge.Reflection.midel(Bridge.Reflection.getMembers(Bridge.getType(parentElement), 16, 284, "Item").s, parentElement).apply(null, (System.Array.init([index], System.Object) || []).concat(clone));
                                } else {
                                    parentElement.SetValue(depProp, clone);
                                }
                            }
                            break;
                        } else {
                            parentElement = depObject;
                        }
                    }
                } finally {
                    if (Bridge.is($t, System.IDisposable)) {
                        $t.System$IDisposable$Dispose();
                    }
                }

                this.GetTargetElementAndPropertyInfo(parameters.Target, target, propertyPath, parameters.IsTargetParentTheTarget);

                this._propertyContainer = target.v;
                this._targetProperty = propertyPath.v;
                this._propDp = this.GetProperty(this._propertyContainer, this._targetProperty);
                this._target = Windows.UI.Xaml.Media.Animation.Storyboard.GetTarget(this);
                this._targetName = Windows.UI.Xaml.Media.Animation.Storyboard.GetTargetName(this);
            },
            Apply: function (parameters, isLastLoop) {
                var $t;
                if (System.Nullable.liftne(Windows.UI.Color.op_Inequality, System.Nullable.lift1("$clone", this.To), null)) {
                    var castedValue = Windows.UI.Xaml.Media.Animation.Timeline.DynamicCast(System.Nullable.lift1("$clone", this.To), this._propDp.PropertyType);

                    var propertyMetadata = this._propDp.GetTypeMetaData(Bridge.getType(this._propertyContainer));

                    var specificGroupName = (parameters.VisualStateGroupName || "") + (this.animationInstanceSpecificName.toString() || "");

                    var cssEquivalentExists = false;
                    if (!Bridge.staticEquals(propertyMetadata.GetCSSEquivalent, null)) {
                        var cssEquivalent = propertyMetadata.GetCSSEquivalent(this._propertyContainer);
                        if (cssEquivalent != null) {
                            cssEquivalentExists = true;
                            Windows.UI.Xaml.Media.Animation.ColorAnimation.TryStartAnimation(this._propertyContainer, cssEquivalent, System.Nullable.lift1("$clone", this.From), System.Nullable.lift1("$clone", this.To), this.Duration.$clone(), this.EasingFunction, specificGroupName, this.OnAnimationCompleted(parameters, isLastLoop, castedValue, this._propertyContainer, this._targetProperty, this._animationID));
                        }
                    }
                    if (!Bridge.staticEquals(propertyMetadata.GetCSSEquivalents, null)) {
                        var cssEquivalents = propertyMetadata.GetCSSEquivalents(this._propertyContainer);
                        var isFirst = true;
                        $t = Bridge.getEnumerator(cssEquivalents);
                        try {
                            while ($t.moveNext()) {
                                var equivalent = $t.Current;
                                cssEquivalentExists = true;
                                if (Bridge.staticEquals(equivalent.CallbackMethod, null)) {
                                    if (isFirst) {
                                        var updateIsFirst = Windows.UI.Xaml.Media.Animation.ColorAnimation.TryStartAnimation(this._propertyContainer, equivalent, System.Nullable.lift1("$clone", this.From), System.Nullable.lift1("$clone", this.To), this.Duration.$clone(), this.EasingFunction, specificGroupName, this.OnAnimationCompleted(parameters, isLastLoop, castedValue, this._propertyContainer, this._targetProperty, this._animationID));
                                        if (updateIsFirst) {
                                            isFirst = false;
                                        }
                                    } else {
                                        Windows.UI.Xaml.Media.Animation.ColorAnimation.TryStartAnimation(this._propertyContainer, equivalent, System.Nullable.lift1("$clone", this.From), System.Nullable.lift1("$clone", this.To), this.Duration.$clone(), this.EasingFunction, specificGroupName, this.OnAnimationCompleted(parameters, isLastLoop, castedValue, this._propertyContainer, this._targetProperty, this._animationID));
                                    }
                                } else {
                                    this.OnAnimationCompleted(parameters, isLastLoop, castedValue, this._propertyContainer, this._targetProperty, this._animationID)();
                                }
                            }
                        } finally {
                            if (Bridge.is($t, System.IDisposable)) {
                                $t.System$IDisposable$Dispose();
                            }
                        }
                    }
                    if (!cssEquivalentExists) {
                        this.OnAnimationCompleted(parameters, isLastLoop, castedValue, this._propertyContainer, this._targetProperty, this._animationID)();
                    }
                } else {
                    this.OnAnimationCompleted(parameters, isLastLoop, System.Nullable.lift1("$clone", this.To), this._propertyContainer, this._targetProperty, this._animationID)();
                }
            },
            OnAnimationCompleted: function (parameters, isLastLoop, value, target, propertyPath, callBackGuid) {
                return Bridge.fn.bind(this, function () {
                    if (isLastLoop && System.Guid.op_Equality(this._animationID, callBackGuid)) {
                        Windows.UI.Xaml.Media.Animation.AnimationHelpers.ApplyValue(target, propertyPath, value, parameters.IsVisualStateChange);
                    }
                    this.OnIterationCompleted(parameters);
                });
            },
            StopAnimation: function (groupName) {
                var $t;
                if (this._isInitialized) {
                    var lastElementType = Bridge.getType(this._propertyContainer);
                    var propertyInfo = Bridge.Reflection.getMembers(lastElementType, 16, 284, this._targetProperty.INTERNAL_DependencyPropertyName);

                    if (System.Nullable.liftne(Windows.UI.Color.op_Inequality, System.Nullable.lift1("$clone", this.To), null)) {
                        var dependencyPropertyContainerType = propertyInfo.td;
                        var dependencyPropertyField = Bridge.Reflection.getMembers(dependencyPropertyContainerType, 4, 284, (this._targetProperty.INTERNAL_DependencyPropertyName || "") + "Property");
                        var dp = Bridge.cast(Bridge.Reflection.fieldAccess(dependencyPropertyField, null), Windows.UI.Xaml.DependencyProperty);
                        var propertyMetadata = dp.GetTypeMetaData(Bridge.getType(this._propertyContainer));

                        var specificGroupName = (groupName || "") + (this.animationInstanceSpecificName.toString() || "");

                        var cssEquivalentExists = false;
                        if (!Bridge.staticEquals(propertyMetadata.GetCSSEquivalent, null)) {
                            var cssEquivalent = propertyMetadata.GetCSSEquivalent(this._propertyContainer);
                            if (cssEquivalent != null) {
                                var uiElement = cssEquivalent.UIElement || (Bridge.as(this._propertyContainer, Windows.UI.Xaml.UIElement));

                                var hasTemplate = (Bridge.is(uiElement, Windows.UI.Xaml.Controls.Control)) && Bridge.cast(uiElement, Windows.UI.Xaml.Controls.Control).HasTemplate;

                                if (!hasTemplate || cssEquivalent.ApplyAlsoWhenThereIsAControlTemplate) {
                                    if (cssEquivalent.DomElement == null && uiElement != null) {
                                        cssEquivalent.DomElement = uiElement.INTERNAL_OuterDomElement;
                                    }
                                    if (cssEquivalent.DomElement != null) {
                                        cssEquivalentExists = true;
Velocity(                                        cssEquivalent.DomElement, "stop", specificGroupName);
                                    }
                                }
                            }
                        }
                        if (!Bridge.staticEquals(propertyMetadata.GetCSSEquivalents, null)) {
                            var cssEquivalents = propertyMetadata.GetCSSEquivalents(this._propertyContainer);
                            $t = Bridge.getEnumerator(cssEquivalents);
                            try {
                                while ($t.moveNext()) {
                                    var equivalent = $t.Current;
                                    if (equivalent.DomElement != null && Bridge.staticEquals(equivalent.CallbackMethod, null)) {
                                        cssEquivalentExists = true;
Velocity(                                        equivalent.DomElement, "stop", specificGroupName);
                                    }
                                }
                            } finally {
                                if (Bridge.is($t, System.IDisposable)) {
                                    $t.System$IDisposable$Dispose();
                                }
                            }
                        }
                    }
                }
            }
        }
    });

    /**
     * Animates the value of a Double property between two target values using linear
     interpolation over a specified Duration.
     *
     * @public
     * @class Windows.UI.Xaml.Media.Animation.DoubleAnimation
     * @augments Windows.UI.Xaml.Media.Animation.AnimationTimeline
     */
    Bridge.define("Windows.UI.Xaml.Media.Animation.DoubleAnimation", {
        inherits: [Windows.UI.Xaml.Media.Animation.AnimationTimeline],
        statics: {
            fields: {
                /**
                 * Identifies the EasingFunction dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Media.Animation.DoubleAnimation
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                EasingFunctionProperty: null,
                /**
                 * Identifies the From dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Media.Animation.DoubleAnimation
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                FromProperty: null,
                /**
                 * Identifies the To dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Media.Animation.DoubleAnimation
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                ToProperty: null
            },
            ctors: {
                init: function () {
                    this.EasingFunctionProperty = Windows.UI.Xaml.DependencyProperty.Register("EasingFunction", Windows.UI.Xaml.Media.Animation.EasingFunctionBase, Windows.UI.Xaml.Media.Animation.DoubleAnimation, new Windows.UI.Xaml.PropertyMetadata.$ctor4(null));
                    this.FromProperty = Windows.UI.Xaml.DependencyProperty.Register("From", System.Nullable$1(System.Double), Windows.UI.Xaml.Media.Animation.DoubleAnimation, new Windows.UI.Xaml.PropertyMetadata.$ctor4(null));
                    this.ToProperty = Windows.UI.Xaml.DependencyProperty.Register("To", System.Nullable$1(System.Double), Windows.UI.Xaml.Media.Animation.DoubleAnimation, new Windows.UI.Xaml.PropertyMetadata.$ctor4(null));
                }
            },
            methods: {
                StartAnimation: function (target, cssEquivalent, from, to, Duration, easingFunction, visualStateGroupName, callbackForWhenfinished) {
                    var $t, $t1;
                    if (callbackForWhenfinished === void 0) { callbackForWhenfinished = null; }
                    if (cssEquivalent.Name != null && cssEquivalent.Name.Count !== 0) {
                        var uiElement = cssEquivalent.UIElement || (Bridge.as(target, Windows.UI.Xaml.UIElement));

                        var hasTemplate = (Bridge.is(uiElement, Windows.UI.Xaml.Controls.Control)) && Bridge.cast(uiElement, Windows.UI.Xaml.Controls.Control).HasTemplate;

                        if (!hasTemplate || cssEquivalent.ApplyAlsoWhenThereIsAControlTemplate) {
                            if (cssEquivalent.DomElement == null && uiElement != null) {
                                cssEquivalent.DomElement = uiElement.INTERNAL_OuterDomElement;
                            }
                            if (cssEquivalent.DomElement != null) {
                                if (Bridge.staticEquals(cssEquivalent.Value, null)) {
                                    cssEquivalent.Value = function (finalInstance, value) {
                                        var $t;
                                        return ($t = value, $t != null ? $t : "");
                                    };
                                }
                                var cssValue = cssEquivalent.Value(target, to);

                                var newObj = new Object();

                                if (Windows.UI.Xaml.Media.Animation.AnimationHelpers.IsValueNull(Bridge.box(from, System.Double, System.Nullable.toStringFn(System.Double.format), System.Nullable.getHashCodeFn(System.Double.getHashCode)))) {
                                    $t = Bridge.getEnumerator(cssEquivalent.Name);
                                    try {
                                        while ($t.moveNext()) {
                                            var csspropertyName = $t.Current;
                                            newObj[csspropertyName] = cssValue;
                                        }
                                    } finally {
                                        if (Bridge.is($t, System.IDisposable)) {
                                            $t.System$IDisposable$Dispose();
                                        }
                                    }
                                } else {
                                    $t1 = Bridge.getEnumerator(cssEquivalent.Name);
                                    try {
                                        while ($t1.moveNext()) {
                                            var csspropertyName1 = $t1.Current;
                                            newObj[csspropertyName1] = [cssValue, Bridge.box(from, System.Double, System.Nullable.toStringFn(System.Double.format), System.Nullable.getHashCodeFn(System.Double.getHashCode))];
                                        }
                                    } finally {
                                        if (Bridge.is($t1, System.IDisposable)) {
                                            $t1.System$IDisposable$Dispose();
                                        }
                                    }
                                }
                                Windows.UI.Xaml.Media.Animation.AnimationHelpers.CallVelocity(cssEquivalent.DomElement, Duration.$clone(), easingFunction, visualStateGroupName, callbackForWhenfinished, newObj);
                            }
                        }
                    } else {
                        throw new System.InvalidOperationException.$ctor1("Please set the Name property of the CSSEquivalent class.");
                    }
                }
            }
        },
        fields: {
            _animationID: null
        },
        props: {
            /**
             * Gets or sets the easing function applied to this animation.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Media.Animation.DoubleAnimation
             * @function EasingFunction
             * @type Windows.UI.Xaml.Media.Animation.EasingFunctionBase
             */
            EasingFunction: {
                get: function () {
                    return Bridge.cast(this.GetValue(Windows.UI.Xaml.Media.Animation.DoubleAnimation.EasingFunctionProperty), Windows.UI.Xaml.Media.Animation.EasingFunctionBase);
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Media.Animation.DoubleAnimation.EasingFunctionProperty, value);
                }
            },
            /**
             * Gets or sets the animation's starting value.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Media.Animation.DoubleAnimation
             * @function From
             * @type ?number
             */
            From: {
                get: function () {
                    return Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.Media.Animation.DoubleAnimation.FromProperty), System.Double), System.Double, true);
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Media.Animation.DoubleAnimation.FromProperty, Bridge.box(value, System.Double, System.Nullable.toStringFn(System.Double.format), System.Nullable.getHashCodeFn(System.Double.getHashCode)));
                }
            },
            /**
             * Gets or sets the animation's ending value.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Media.Animation.DoubleAnimation
             * @function To
             * @type ?number
             */
            To: {
                get: function () {
                    return Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.Media.Animation.DoubleAnimation.ToProperty), System.Double), System.Double, true);
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Media.Animation.DoubleAnimation.ToProperty, Bridge.box(value, System.Double, System.Nullable.toStringFn(System.Double.format), System.Nullable.getHashCodeFn(System.Double.getHashCode)));
                }
            }
        },
        ctors: {
            init: function () {
                this._animationID = new System.Guid();
            }
        },
        methods: {
            GetTargetInformation: function (parameters) {
                this._parameters$1 = parameters;
                var target = { };
                var propertyPath = { };

                this.GetTargetElementAndPropertyInfo(parameters.Target, target, propertyPath, parameters.IsTargetParentTheTarget);

                this._propertyContainer = target.v;
                this._targetProperty = propertyPath.v;
                this._propDp = this.GetProperty(this._propertyContainer, this._targetProperty);
                this._target = Windows.UI.Xaml.Media.Animation.Storyboard.GetTarget(this);
                this._targetName = Windows.UI.Xaml.Media.Animation.Storyboard.GetTargetName(this);
            },
            Apply: function (parameters, isLastLoop) {
                var $t;
                if (this.To != null) {
                    var propertyMetadata = this._propDp.GetTypeMetaData(Bridge.getType(this._propertyContainer));

                    var specificGroupName = (parameters.VisualStateGroupName || "") + (this.animationInstanceSpecificName.toString() || "");

                    this._animationID = System.Guid.NewGuid();

                    var cssEquivalentExists = false;
                    if (!Bridge.staticEquals(propertyMetadata.GetCSSEquivalent, null)) {
                        var cssEquivalent = propertyMetadata.GetCSSEquivalent(this._propertyContainer);
                        if (cssEquivalent != null) {
                            cssEquivalentExists = true;
                            Windows.UI.Xaml.Media.Animation.DoubleAnimation.StartAnimation(this._propertyContainer, cssEquivalent, this.From, Bridge.box(this.To, System.Double, System.Nullable.toStringFn(System.Double.format), System.Nullable.getHashCodeFn(System.Double.getHashCode)), this.Duration.$clone(), this.EasingFunction, specificGroupName, this.OnAnimationCompleted(parameters, isLastLoop, Bridge.box(System.Nullable.getValue(this.To), System.Double, System.Double.format, System.Double.getHashCode), this._propertyContainer, this._targetProperty, this._animationID));

                        }
                    }
                    if (!Bridge.staticEquals(propertyMetadata.GetCSSEquivalents, null)) {
                        var cssEquivalents = propertyMetadata.GetCSSEquivalents(this._propertyContainer);
                        $t = Bridge.getEnumerator(cssEquivalents);
                        try {
                            while ($t.moveNext()) {
                                var equivalent = $t.Current;
                                cssEquivalentExists = true;
                                Windows.UI.Xaml.Media.Animation.DoubleAnimation.StartAnimation(this._propertyContainer, equivalent, this.From, Bridge.box(this.To, System.Double, System.Nullable.toStringFn(System.Double.format), System.Nullable.getHashCodeFn(System.Double.getHashCode)), this.Duration.$clone(), this.EasingFunction, specificGroupName, this.OnAnimationCompleted(parameters, isLastLoop, Bridge.box(System.Nullable.getValue(this.To), System.Double, System.Double.format, System.Double.getHashCode), this._propertyContainer, this._targetProperty, this._animationID));
                            }
                        } finally {
                            if (Bridge.is($t, System.IDisposable)) {
                                $t.System$IDisposable$Dispose();
                            }
                        }
                    }

                    if (!cssEquivalentExists) {
                        this.OnAnimationCompleted(parameters, isLastLoop, Bridge.box(System.Nullable.getValue(this.To), System.Double, System.Double.format, System.Double.getHashCode), this._propertyContainer, this._targetProperty, this._animationID)();
                    }
                }
            },
            OnAnimationCompleted: function (parameters, isLastLoop, value, target, propertyPath, callBackGuid) {
                return Bridge.fn.bind(this, function () {
                    if (isLastLoop && System.Guid.op_Equality(this._animationID, callBackGuid)) {
                        Windows.UI.Xaml.Media.Animation.AnimationHelpers.ApplyValue(target, propertyPath, value, parameters.IsVisualStateChange);
                    }
                    this.OnIterationCompleted(parameters);
                });
            },
            StopAnimation: function (groupName) {
                var $t;
                if (this._isInitialized) {
                    if (this.To != null) {
                        var propertyMetadata = this._propDp.GetTypeMetaData(Bridge.getType(this._propertyContainer));

                        var specificGroupName = (groupName || "") + (this.animationInstanceSpecificName.toString() || "");

                        if (!Bridge.staticEquals(propertyMetadata.GetCSSEquivalent, null)) {
                            var cssEquivalent = propertyMetadata.GetCSSEquivalent(this._propertyContainer);
                            if (cssEquivalent != null) {
                                var uiElement = cssEquivalent.UIElement || (Bridge.as(this._propertyContainer, Windows.UI.Xaml.UIElement));

                                var hasTemplate = (Bridge.is(uiElement, Windows.UI.Xaml.Controls.Control)) && Bridge.cast(uiElement, Windows.UI.Xaml.Controls.Control).HasTemplate;

                                if (!hasTemplate || cssEquivalent.ApplyAlsoWhenThereIsAControlTemplate) {
                                    if (cssEquivalent.DomElement == null && uiElement != null) {
                                        cssEquivalent.DomElement = uiElement.INTERNAL_OuterDomElement;
                                    }
                                    if (cssEquivalent.DomElement != null) {
Velocity(                                        cssEquivalent.DomElement, "stop", specificGroupName);
                                    }
                                }
                            }
                        }
                        if (!Bridge.staticEquals(propertyMetadata.GetCSSEquivalents, null)) {
                            var cssEquivalents = propertyMetadata.GetCSSEquivalents(this._propertyContainer);
                            $t = Bridge.getEnumerator(cssEquivalents);
                            try {
                                while ($t.moveNext()) {
                                    var equivalent = $t.Current;
                                    if (equivalent.DomElement != null) {
Velocity(                                        equivalent.DomElement, "stop", specificGroupName);
                                    }
                                }
                            } finally {
                                if (Bridge.is($t, System.IDisposable)) {
                                    $t.System$IDisposable$Dispose();
                                }
                            }
                        }
                    }
                }
            }
        }
    });

    /**
     * Animates the value of a Double property along a set of key frames.
     *
     * @public
     * @class Windows.UI.Xaml.Media.Animation.DoubleAnimationUsingKeyFrames
     * @augments Windows.UI.Xaml.Media.Animation.AnimationTimeline
     */
    Bridge.define("Windows.UI.Xaml.Media.Animation.DoubleAnimationUsingKeyFrames", {
        inherits: [Windows.UI.Xaml.Media.Animation.AnimationTimeline],
        statics: {
            methods: {
                StartAnimation: function (target, cssEquivalent, from, to, Duration, easingFunction, visualStateGroupName, callbackForWhenfinished) {
                    var $t, $t1;
                    if (callbackForWhenfinished === void 0) { callbackForWhenfinished = null; }
                    if (cssEquivalent.Name != null && cssEquivalent.Name.Count !== 0) {
                        var uiElement = cssEquivalent.UIElement || (Bridge.as(target, Windows.UI.Xaml.UIElement));

                        var hasTemplate = (Bridge.is(uiElement, Windows.UI.Xaml.Controls.Control)) && Bridge.cast(uiElement, Windows.UI.Xaml.Controls.Control).HasTemplate;

                        if (!hasTemplate || cssEquivalent.ApplyAlsoWhenThereIsAControlTemplate) {
                            if (cssEquivalent.DomElement == null && uiElement != null) {
                                cssEquivalent.DomElement = uiElement.INTERNAL_OuterDomElement;
                            }
                            if (cssEquivalent.DomElement != null) {
                                if (Bridge.staticEquals(cssEquivalent.Value, null)) {
                                    cssEquivalent.Value = function (finalInstance, value) {
                                        var $t;
                                        return ($t = value, $t != null ? $t : "");
                                    };
                                }
                                var cssValue = cssEquivalent.Value(target, to);

                                var newObj = new Object();

                                if (Windows.UI.Xaml.Media.Animation.AnimationHelpers.IsValueNull(Bridge.box(from, System.Double, System.Nullable.toStringFn(System.Double.format), System.Nullable.getHashCodeFn(System.Double.getHashCode)))) {
                                    $t = Bridge.getEnumerator(cssEquivalent.Name);
                                    try {
                                        while ($t.moveNext()) {
                                            var csspropertyName = $t.Current;
                                            newObj[csspropertyName] = cssValue;
                                        }
                                    } finally {
                                        if (Bridge.is($t, System.IDisposable)) {
                                            $t.System$IDisposable$Dispose();
                                        }
                                    }
                                } else {
                                    $t1 = Bridge.getEnumerator(cssEquivalent.Name);
                                    try {
                                        while ($t1.moveNext()) {
                                            var csspropertyName1 = $t1.Current;
                                            newObj[csspropertyName1] = [cssValue, Bridge.box(from, System.Double, System.Nullable.toStringFn(System.Double.format), System.Nullable.getHashCodeFn(System.Double.getHashCode))];
                                        }
                                    } finally {
                                        if (Bridge.is($t1, System.IDisposable)) {
                                            $t1.System$IDisposable$Dispose();
                                        }
                                    }
                                }
                                Windows.UI.Xaml.Media.Animation.AnimationHelpers.CallVelocity(cssEquivalent.DomElement, Duration.$clone(), easingFunction, visualStateGroupName, callbackForWhenfinished, newObj);
                            }
                        }
                    } else {
                        throw new System.InvalidOperationException.$ctor1("Please set the Name property of the CSSEquivalent class.");
                    }
                }
            }
        },
        fields: {
            _keyFrames: null,
            _appliedKeyFramesCount: 0,
            _resolvedKeyFrames: null,
            _keyFrameToDurationMap: null,
            _propertyMetadata: null,
            _animationID: null,
            thisLock: null
        },
        props: {
            /**
             * Gets the collection of DoubleKeyFrame objects that define the animation.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Media.Animation.DoubleAnimationUsingKeyFrames
             * @function KeyFrames
             * @type Windows.UI.Xaml.Media.Animation.DoubleKeyFrameCollection
             */
            KeyFrames: {
                get: function () {
                    if (this._keyFrames == null) {
                        this._keyFrames = new Windows.UI.Xaml.Media.Animation.DoubleKeyFrameCollection();
                    }
                    return this._keyFrames;
                },
                set: function (value) {
                    this._keyFrames = value;
                }
            },
            /**
             * Returns the largest time span specified key time from all of the key frames.
             If there are not time span key times a time span of one second is returned
             to match the default natural duration of the From/To/By animations.
             *
             * @instance
             * @private
             * @readonly
             * @memberof Windows.UI.Xaml.Media.Animation.DoubleAnimationUsingKeyFrames
             * @function LargestTimeSpanKeyTime
             * @type System.TimeSpan
             */
            LargestTimeSpanKeyTime: {
                get: function () {
                    if (this._keyFrames == null || this._keyFrames.Count === 0) {
                        return System.TimeSpan.fromTicks(System.Int64(0));
                    }
                    if (this._resolvedKeyFrames != null) {
                        return this._keyFrames.getItem(this._resolvedKeyFrames.GetNextKeyFrameIndex(((this._keyFrames.Count - 1) | 0))).KeyTime.TimeSpan;
                    } else {
                        throw new System.Exception("DoubleAnimationUsingKeyFrames has not been setup yet.");
                    }
                }
            }
        },
        ctors: {
            init: function () {
                this._animationID = new System.Guid();
                this._appliedKeyFramesCount = -1;
                this.thisLock = { };
            }
        },
        methods: {
            InitializeKeyFramesSet: function () {
                this._resolvedKeyFrames = new (Windows.UI.Xaml.Media.Animation.INTERNAL_ResolvedKeyFramesEntries$1(Windows.UI.Xaml.Media.Animation.DoubleKeyFrame)).$ctor1(this._keyFrames);
                this._keyFrameToDurationMap = new (System.Collections.Generic.Dictionary$2(Windows.UI.Xaml.Media.Animation.DoubleKeyFrame,Windows.UI.Xaml.Duration)).ctor();
                var keyFrame;
                for (var i = 0; i < this.KeyFrames.Count; i = (i + 1) | 0) {
                    keyFrame = this.KeyFrames.getItem(this._resolvedKeyFrames.GetNextKeyFrameIndex(i));
                    this._keyFrameToDurationMap.add(keyFrame, Windows.UI.Xaml.Duration.op_Implicit(System.TimeSpan.sub(keyFrame.KeyTime.TimeSpan, (i > 0 ? this.KeyFrames.getItem(this._resolvedKeyFrames.GetNextKeyFrameIndex(((i - 1) | 0))).KeyTime.TimeSpan : System.TimeSpan.zero))));
                }
                this._appliedKeyFramesCount = 0;
            },
            GetTargetInformation: function (parameters) {
                var $t;
                this._parameters$1 = parameters;
                var target = { };
                var propertyPath = { };
                var targetBeforePath = { };
                this.GetPropertyPathAndTargetBeforePath(parameters.Target, targetBeforePath, propertyPath, parameters.IsTargetParentTheTarget);
                var parentElement = targetBeforePath.v;
                $t = Bridge.getEnumerator(this.GoThroughElementsToAccessProperty(propertyPath.v, targetBeforePath.v), System.Tuple$3(Windows.UI.Xaml.DependencyObject,Windows.UI.Xaml.DependencyProperty,System.Nullable$1(System.Int32)));
                try {
                    while ($t.moveNext()) {
                        var element = $t.Current;
                        var depObject = element.Item1;
                        var depProp = element.Item2;
                        var index = element.Item3;
                        if (Bridge.is(depObject, Windows.UI.Xaml.ICloneOnAnimation)) {
                            if (!Bridge.cast(depObject, Windows.UI.Xaml.ICloneOnAnimation).Windows$UI$Xaml$ICloneOnAnimation$IsAlreadyAClone()) {
                                var clone = Bridge.cast(depObject, Windows.UI.Xaml.ICloneOnAnimation).Windows$UI$Xaml$ICloneOnAnimation$Clone();
                                if (index != null) {
                                    Bridge.Reflection.midel(Bridge.Reflection.getMembers(Bridge.getType(parentElement), 16, 284, "Item").s, parentElement).apply(null, (System.Array.init([index], System.Object) || []).concat(clone));
                                } else {
                                    parentElement.SetValue(depProp, clone);
                                }
                            }
                            break;
                        } else {
                            parentElement = depObject;
                        }
                    }
                } finally {
                    if (Bridge.is($t, System.IDisposable)) {
                        $t.System$IDisposable$Dispose();
                    }
                }

                this.GetTargetElementAndPropertyInfo(parameters.Target, target, propertyPath, parameters.IsTargetParentTheTarget);

                this._propertyContainer = target.v;
                this._targetProperty = propertyPath.v;
                this._propDp = this.GetProperty(this._propertyContainer, this._targetProperty);
                this._target = Windows.UI.Xaml.Media.Animation.Storyboard.GetTarget(this);
                this._targetName = Windows.UI.Xaml.Media.Animation.Storyboard.GetTargetName(this);
            },
            Apply: function (parameters, isLastLoop) {
                this._animationID = System.Guid.NewGuid();
                this.ApplyKeyFrame(this.GetNextKeyFrame(), isLastLoop);
            },
            ApplyKeyFrame: function (keyFrame, isLastLoop) {
                var $t;
                if (keyFrame != null) {
                    var specificGroupName = (this._parameters$1.VisualStateGroupName || "") + (this.animationInstanceSpecificName.toString() || "");

                    var cssEquivalentExists = false;
                    if (!Bridge.staticEquals(this._propertyMetadata.GetCSSEquivalent, null)) {
                        var cssEquivalent = this._propertyMetadata.GetCSSEquivalent(this._propertyContainer);
                        if (cssEquivalent != null) {
                            cssEquivalentExists = true;
                            Windows.UI.Xaml.Media.Animation.DoubleAnimationUsingKeyFrames.StartAnimation(this._propertyContainer, cssEquivalent, null, Bridge.box(keyFrame.Value, System.Double, System.Double.format, System.Double.getHashCode), this.GetKeyFrameDuration(keyFrame), keyFrame.INTERNAL_GetEasingFunction(), specificGroupName, this.OnKeyFrameCompleted(this._parameters$1, isLastLoop, Bridge.box(keyFrame.Value, System.Double, System.Double.format, System.Double.getHashCode), this._propertyContainer, this._targetProperty, this._animationID));
                        }
                    }
                    if (!Bridge.staticEquals(this._propertyMetadata.GetCSSEquivalents, null)) {
                        var cssEquivalents = this._propertyMetadata.GetCSSEquivalents(this._propertyContainer);
                        $t = Bridge.getEnumerator(cssEquivalents);
                        try {
                            while ($t.moveNext()) {
                                var equivalent = $t.Current;
                                cssEquivalentExists = true;
                                Windows.UI.Xaml.Media.Animation.DoubleAnimationUsingKeyFrames.StartAnimation(this._propertyContainer, equivalent, null, Bridge.box(keyFrame.Value, System.Double, System.Double.format, System.Double.getHashCode), this.Duration.$clone(), keyFrame.INTERNAL_GetEasingFunction(), specificGroupName, this.OnKeyFrameCompleted(this._parameters$1, isLastLoop, Bridge.box(keyFrame.Value, System.Double, System.Double.format, System.Double.getHashCode), this._propertyContainer, this._targetProperty, this._animationID));
                            }
                        } finally {
                            if (Bridge.is($t, System.IDisposable)) {
                                $t.System$IDisposable$Dispose();
                            }
                        }
                    }

                    if (!cssEquivalentExists) {
                        this.OnKeyFrameCompleted(this._parameters$1, isLastLoop, Bridge.box(keyFrame.Value, System.Double, System.Double.format, System.Double.getHashCode), this._propertyContainer, this._targetProperty, this._animationID)();
                    }
                }
            },
            GetKeyFrameDuration: function (keyFrame) {
                return this._keyFrameToDurationMap.getItem(keyFrame).$clone();
            },
            OnKeyFrameCompleted: function (parameters, isLastLoop, value, target, propertyPath, callBackGuid) {
                return Bridge.fn.bind(this, function () {
                    if (System.Guid.op_Equality(this._animationID, callBackGuid)) {
                        Windows.UI.Xaml.Media.Animation.AnimationHelpers.ApplyValue(target, propertyPath, value, parameters.IsVisualStateChange);
                        this._appliedKeyFramesCount = (this._appliedKeyFramesCount + 1) | 0;
                        if (!this.CheckTimeLineEndAndRaiseCompletedEvent(this._parameters$1)) {
                            this.ApplyKeyFrame(this.GetNextKeyFrame(), isLastLoop);
                        }
                    }
                });
            },
            GetNextKeyFrame: function () {
                var nextKeyFrameIndex = this._resolvedKeyFrames.GetNextKeyFrameIndex(this._appliedKeyFramesCount);
                if (nextKeyFrameIndex === -1) {
                    return null;
                } else {
                    return this._keyFrames.getItem(nextKeyFrameIndex);
                }
            },
            ApplyLastKeyFrame: function (sender, e) {
                if (!this._cancelledAnimation) {
                    var lastKeyFrame = this._keyFrames.getItem(this._resolvedKeyFrames.GetNextKeyFrameIndex(((this._keyFrames.Count - 1) | 0)));
                    Windows.UI.Xaml.Media.Animation.AnimationHelpers.ApplyValue(this._propertyContainer, this._targetProperty, Bridge.box(lastKeyFrame.Value, System.Double, System.Double.format, System.Double.getHashCode), this._parameters$1.IsVisualStateChange);
                }
            },
            StopAnimation: function (groupName) {
                var $t;
                if (this._isInitialized) {
                    var specificGroupName = (groupName || "") + (this.animationInstanceSpecificName.toString() || "");

                    if (!Bridge.staticEquals(this._propertyMetadata.GetCSSEquivalent, null)) {
                        var cssEquivalent = this._propertyMetadata.GetCSSEquivalent(this._propertyContainer);
                        if (cssEquivalent != null) {
                            var uiElement = cssEquivalent.UIElement || (Bridge.as(this._propertyContainer, Windows.UI.Xaml.UIElement));

                            var hasTemplate = (Bridge.is(uiElement, Windows.UI.Xaml.Controls.Control)) && Bridge.cast(uiElement, Windows.UI.Xaml.Controls.Control).HasTemplate;

                            if (!hasTemplate || cssEquivalent.ApplyAlsoWhenThereIsAControlTemplate) {
                                if (cssEquivalent.DomElement == null && uiElement != null) {
                                    cssEquivalent.DomElement = uiElement.INTERNAL_OuterDomElement;
                                }
                                if (cssEquivalent.DomElement != null) {
Velocity(                                    cssEquivalent.DomElement, "stop", specificGroupName);
                                }
                            }
                        }
                    }
                    if (!Bridge.staticEquals(this._propertyMetadata.GetCSSEquivalents, null)) {
                        var cssEquivalents = this._propertyMetadata.GetCSSEquivalents(this._propertyContainer);
                        $t = Bridge.getEnumerator(cssEquivalents);
                        try {
                            while ($t.moveNext()) {
                                var equivalent = $t.Current;
                                if (equivalent.DomElement != null) {
Velocity(                                    equivalent.DomElement, "stop", specificGroupName);
                                }
                            }
                        } finally {
                            if (Bridge.is($t, System.IDisposable)) {
                                $t.System$IDisposable$Dispose();
                            }
                        }
                    }
                }
            },
            CheckTimeLineEndAndRaiseCompletedEvent: function (parameters) {
                var raiseEvent = false;
                this.thisLock;
                {
                    if (this._appliedKeyFramesCount >= this.KeyFrames.Count) {
                        raiseEvent = true;
                    }
                }
                if (raiseEvent || this._cancelledAnimation) {
                    this.OnIterationCompleted(parameters);
                }
                return raiseEvent || this._cancelledAnimation;
            },
            InitializeCore: function () {
                this.removeCompleted(Bridge.fn.cacheBind(this, this.ApplyLastKeyFrame));
                this.addCompleted(Bridge.fn.cacheBind(this, this.ApplyLastKeyFrame));
                this.InitializeKeyFramesSet();
                this._propertyMetadata = this._propDp.GetTypeMetaData(Bridge.getType(this._propertyContainer));
            },
            RestoreDefaultCore: function () {
                this._appliedKeyFramesCount = 0;
            },
            GetNaturalDurationCore: function () {
                return new Windows.UI.Xaml.Duration.$ctor1(this.LargestTimeSpanKeyTime);
            }
        }
    });

    /**
     * Animates the value of an Object property along a set of KeyFrames over a
     specified Duration.
     *
     * @public
     * @class Windows.UI.Xaml.Media.Animation.ObjectAnimationUsingKeyFrames
     * @augments Windows.UI.Xaml.Media.Animation.AnimationTimeline
     */
    Bridge.define("Windows.UI.Xaml.Media.Animation.ObjectAnimationUsingKeyFrames", {
        inherits: [Windows.UI.Xaml.Media.Animation.AnimationTimeline],
        fields: {
            _keyFrames: null,
            _appliedKeyFramesCount: 0,
            _resolvedKeyFrames: null,
            _keyFramesToObjectTimers: null,
            thisLock: null
        },
        props: {
            /**
             * Gets the collection of ObjectKeyFrame objects that define the animation.
             *
             * @instance
             * @public
             * @readonly
             * @memberof Windows.UI.Xaml.Media.Animation.ObjectAnimationUsingKeyFrames
             * @function KeyFrames
             * @type Windows.UI.Xaml.Media.Animation.ObjectKeyFrameCollection
             */
            KeyFrames: {
                get: function () {
                    if (this._keyFrames == null) {
                        this._keyFrames = new Windows.UI.Xaml.Media.Animation.ObjectKeyFrameCollection();
                    }
                    return this._keyFrames;
                }
            },
            /**
             * Returns the largest time span specified key time from all of the key frames.
             If there are not time span key times a time span of one second is returned
             to match the default natural duration of the From/To/By animations.
             *
             * @instance
             * @private
             * @readonly
             * @memberof Windows.UI.Xaml.Media.Animation.ObjectAnimationUsingKeyFrames
             * @function LargestTimeSpanKeyTime
             * @type System.TimeSpan
             */
            LargestTimeSpanKeyTime: {
                get: function () {
                    if (this._keyFrames == null || this._keyFrames.Count === 0) {
                        return System.TimeSpan.fromTicks(System.Int64(0));
                    }
                    if (this._resolvedKeyFrames != null) {
                        return this._keyFrames.getItem(this._resolvedKeyFrames.GetNextKeyFrameIndex(((this._keyFrames.Count - 1) | 0))).KeyTime.TimeSpan;
                    } else {
                        throw new System.Exception("DoubleAnimationUsingKeyFrames has not been setup yet.");
                    }
                }
            }
        },
        ctors: {
            init: function () {
                this.thisLock = { };
            }
        },
        methods: {
            InitializeKeyFramesSet: function () {
                this._resolvedKeyFrames = new (Windows.UI.Xaml.Media.Animation.INTERNAL_ResolvedKeyFramesEntries$1(Windows.UI.Xaml.Media.Animation.ObjectKeyFrame)).$ctor1(this._keyFrames);
                this._keyFramesToObjectTimers = new (System.Collections.Generic.Dictionary$2(Windows.UI.Xaml.Media.Animation.ObjectKeyFrame,Windows.UI.Xaml.Media.Animation.ObjectAnimationUsingKeyFrames.NullableTimer)).ctor();
                for (var i = 0; i < this.KeyFrames.Count; i = (i + 1) | 0) {
                    var keyFrameIndex = this._resolvedKeyFrames.GetNextKeyFrameIndex(i);
                    var keyFrame = this.KeyFrames.getItem(keyFrameIndex);
                    var timer = new Windows.UI.Xaml.Media.Animation.ObjectAnimationUsingKeyFrames.NullableTimer(System.TimeSpan.sub(keyFrame.KeyTime.TimeSpan, (i > 0 ? this.KeyFrames.getItem(this._resolvedKeyFrames.GetNextKeyFrameIndex(((i - 1) | 0))).KeyTime.TimeSpan : System.TimeSpan.zero)));
                    timer.addCompleted(Bridge.fn.cacheBind(this, this.ApplyNextKeyFrame));
                    this._keyFramesToObjectTimers.add(keyFrame, timer);
                }
                this._appliedKeyFramesCount = 0;
            },
            ApplyNextKeyFrame: function (sender, e) {
                this.ApplyKeyFrame(this.GetNextKeyFrame());

                this._appliedKeyFramesCount = (this._appliedKeyFramesCount + 1) | 0;
                if (!this.CheckTimeLineEndAndRaiseCompletedEvent(this._parameters$1)) {
                    this.StartKeyFrame(this.GetNextKeyFrame());
                }
            },
            GetTargetInformation: function (parameters) {
                var $t;
                this._parameters$1 = parameters;
                var target = { };
                var propertyPath = { };
                var targetBeforePath = { };
                this.GetPropertyPathAndTargetBeforePath(parameters.Target, targetBeforePath, propertyPath);
                var parentElement = targetBeforePath.v;
                $t = Bridge.getEnumerator(this.GoThroughElementsToAccessProperty(propertyPath.v, targetBeforePath.v), System.Tuple$3(Windows.UI.Xaml.DependencyObject,Windows.UI.Xaml.DependencyProperty,System.Nullable$1(System.Int32)));
                try {
                    while ($t.moveNext()) {
                        var element = $t.Current;
                        var depObject = element.Item1;
                        var depProp = element.Item2;
                        var index = element.Item3;
                        if (Bridge.is(depObject, Windows.UI.Xaml.ICloneOnAnimation)) {
                            if (!Bridge.cast(depObject, Windows.UI.Xaml.ICloneOnAnimation).Windows$UI$Xaml$ICloneOnAnimation$IsAlreadyAClone()) {
                                var clone = Bridge.cast(depObject, Windows.UI.Xaml.ICloneOnAnimation).Windows$UI$Xaml$ICloneOnAnimation$Clone();
                                if (index != null) {
                                    Bridge.Reflection.midel(Bridge.Reflection.getMembers(Bridge.getType(parentElement), 16, 284, "Item").s, parentElement).apply(null, (System.Array.init([index], System.Object) || []).concat(clone));
                                } else {
                                    parentElement.SetValue(depProp, clone);
                                }
                            }
                            break;
                        } else {
                            parentElement = depObject;
                        }
                    }
                } finally {
                    if (Bridge.is($t, System.IDisposable)) {
                        $t.System$IDisposable$Dispose();
                    }
                }

                this.GetTargetElementAndPropertyInfo(parameters.Target, target, propertyPath);

                this._propertyContainer = target.v;
                this._targetProperty = propertyPath.v;
                this._propDp = this.GetProperty(this._propertyContainer, this._targetProperty);
            },
            Apply: function (parameters, isLastLoop) {
                this.StartKeyFrame(this.GetNextKeyFrame());
            },
            StartKeyFrame: function (keyFrame) {
                if (keyFrame != null) {
                    this._keyFramesToObjectTimers.getItem(keyFrame).Start();
                }
            },
            ApplyKeyFrame: function (keyFrame) {
                var value = keyFrame.Value;
                if (Bridge.is(value, System.String) && !Bridge.referenceEquals(this._propDp.PropertyType, System.String)) {
                    if (Bridge.Reflection.isEnum(this._propDp.PropertyType)) {
                        value = System.Enum.parse(this._propDp.PropertyType, Bridge.cast(value, System.String));
                    } else {
                        value = DotNetForHtml5.Core.TypeFromStringConverters.ConvertFromInvariantString(this._propDp.PropertyType, Bridge.cast(value, System.String));
                    }
                }

                var castedValue = Windows.UI.Xaml.Media.Animation.Timeline.DynamicCast(value, this._propDp.PropertyType);

                Windows.UI.Xaml.Media.Animation.AnimationHelpers.ApplyValue(this._propertyContainer, this._targetProperty, castedValue, this._parameters$1.IsVisualStateChange);
            },
            GetNextKeyFrame: function () {
                var nextKeyFrameIndex = this._resolvedKeyFrames.GetNextKeyFrameIndex(this._appliedKeyFramesCount);
                if (nextKeyFrameIndex === -1) {
                    return null;
                } else {
                    return this._keyFrames.getItem(nextKeyFrameIndex);
                }
            },
            ApplyLastKeyFrame: function (sender, e) {
                if (!this._cancelledAnimation) {
                    var lastKeyFrame = this._keyFrames.getItem(this._resolvedKeyFrames.GetNextKeyFrameIndex(((this._keyFrames.Count - 1) | 0)));
                    this.ApplyKeyFrame(lastKeyFrame);
                }
            },
            StopAnimation: function (groupName) {
                if (this._isInitialized) {
                    this.StopAllTimers();
                }
            },
            StopAllTimers: function () {
                var $t;
                if (this._keyFramesToObjectTimers != null) {
                    if (this._keyFramesToObjectTimers.Values != null) {
                        $t = Bridge.getEnumerator(this._keyFramesToObjectTimers.Values);
                        try {
                            while ($t.moveNext()) {
                                var frameTimers = $t.Current;
                                frameTimers.Stop();
                            }
                        } finally {
                            if (Bridge.is($t, System.IDisposable)) {
                                $t.System$IDisposable$Dispose();
                            }
                        }
                    }
                }
            },
            CheckTimeLineEndAndRaiseCompletedEvent: function (parameters) {
                var raiseEvent = false;
                this.thisLock;
                {
                    if (this._appliedKeyFramesCount >= this.KeyFrames.Count) {
                        raiseEvent = true;
                    }
                }
                if (raiseEvent || this._cancelledAnimation) {
                    this.OnIterationCompleted(parameters);
                }
                return raiseEvent || this._cancelledAnimation;
            },
            InitializeCore: function () {
                this.removeCompleted(Bridge.fn.cacheBind(this, this.ApplyLastKeyFrame));
                this.addCompleted(Bridge.fn.cacheBind(this, this.ApplyLastKeyFrame));
                this.InitializeKeyFramesSet();
            },
            RestoreDefaultCore: function () {
                this.ResetAllTimers();
                this._appliedKeyFramesCount = 0;
            },
            ResetAllTimers: function () {
                var $t;
                if (this._keyFramesToObjectTimers != null) {
                    if (this._keyFramesToObjectTimers.Values != null) {
                        $t = Bridge.getEnumerator(this._keyFramesToObjectTimers.Values);
                        try {
                            while ($t.moveNext()) {
                                var frameTimers = $t.Current;
                                frameTimers.Reset();
                            }
                        } finally {
                            if (Bridge.is($t, System.IDisposable)) {
                                $t.System$IDisposable$Dispose();
                            }
                        }
                    }
                }
            },
            GetNaturalDurationCore: function () {
                return new Windows.UI.Xaml.Duration.$ctor1(this.LargestTimeSpanKeyTime);
            }
        }
    });

    /**
     * Applies multiple transform operations to an object.
     *
     * @public
     * @class Windows.UI.Xaml.Media.CompositeTransform
     * @augments Windows.UI.Xaml.Media.Transform
     */
    Bridge.define("Windows.UI.Xaml.Media.CompositeTransform", {
        inherits: [Windows.UI.Xaml.Media.Transform],
        statics: {
            fields: {
                /**
                 * Identifies the ScaleX dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Media.CompositeTransform
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                ScaleXProperty: null,
                /**
                 * Identifies the ScaleY dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Media.CompositeTransform
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                ScaleYProperty: null,
                /**
                 * Identifies the SkewX dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Media.CompositeTransform
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                SkewXProperty: null,
                /**
                 * Identifies the SkewY dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Media.CompositeTransform
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                SkewYProperty: null,
                /**
                 * Identifies the Rotation dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Media.CompositeTransform
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                RotationProperty: null,
                /**
                 * Identifies the TranslateX dependency property. The default is 0.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Media.CompositeTransform
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                TranslateXProperty: null,
                /**
                 * Identifies the TranslateY dependency property
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Media.CompositeTransform
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                TranslateYProperty: null
            },
            ctors: {
                init: function () {
                    var $t;
                    this.ScaleXProperty = Windows.UI.Xaml.DependencyProperty.Register("ScaleX", System.Double, Windows.UI.Xaml.Media.CompositeTransform, ($t = new Windows.UI.Xaml.PropertyMetadata.$ctor1(Bridge.box(1.0, System.Double, System.Double.format, System.Double.getHashCode)), $t.GetCSSEquivalent = function (instance) {
                        var $t1;
                        if (Bridge.cast(instance, Windows.UI.Xaml.Media.CompositeTransform).INTERNAL_parent != null) {
                            return ($t1 = new Windows.UI.Xaml.CSSEquivalent(), $t1.DomElement = Bridge.cast(instance, Windows.UI.Xaml.Media.CompositeTransform).INTERNAL_parent.INTERNAL_OuterDomElement, $t1.Value = function (inst, value) {
                                return value;
                            }, $t1.Name = function (_o1) {
                                    _o1.add("scaleX");
                                    return _o1;
                                }(new (System.Collections.Generic.List$1(System.String)).ctor()), $t1.UIElement = Bridge.cast(instance, Windows.UI.Xaml.Media.CompositeTransform).INTERNAL_parent, $t1.ApplyAlsoWhenThereIsAControlTemplate = true, $t1.OnlyUseVelocity = true, $t1);
                        }
                        return null;
                    }, $t));
                    this.ScaleYProperty = Windows.UI.Xaml.DependencyProperty.Register("ScaleY", System.Double, Windows.UI.Xaml.Media.CompositeTransform, ($t = new Windows.UI.Xaml.PropertyMetadata.$ctor1(Bridge.box(1.0, System.Double, System.Double.format, System.Double.getHashCode)), $t.GetCSSEquivalent = function (instance) {
                        var $t1;
                        if (Bridge.cast(instance, Windows.UI.Xaml.Media.CompositeTransform).INTERNAL_parent != null) {
                            return ($t1 = new Windows.UI.Xaml.CSSEquivalent(), $t1.DomElement = Bridge.cast(instance, Windows.UI.Xaml.Media.CompositeTransform).INTERNAL_parent.INTERNAL_OuterDomElement, $t1.Value = function (inst, value) {
                                return value;
                            }, $t1.Name = function (_o2) {
                                    _o2.add("scaleY");
                                    return _o2;
                                }(new (System.Collections.Generic.List$1(System.String)).ctor()), $t1.UIElement = Bridge.cast(instance, Windows.UI.Xaml.Media.CompositeTransform).INTERNAL_parent, $t1.ApplyAlsoWhenThereIsAControlTemplate = true, $t1.OnlyUseVelocity = true, $t1);
                        }
                        return null;
                    }, $t));
                    this.SkewXProperty = Windows.UI.Xaml.DependencyProperty.Register("SkewX", System.Double, Windows.UI.Xaml.Media.CompositeTransform, ($t = new Windows.UI.Xaml.PropertyMetadata.$ctor1(Bridge.box(0.0, System.Double, System.Double.format, System.Double.getHashCode)), $t.GetCSSEquivalent = function (instance) {
                        var $t1;
                        if (Bridge.cast(instance, Windows.UI.Xaml.Media.CompositeTransform).INTERNAL_parent != null) {
                            return ($t1 = new Windows.UI.Xaml.CSSEquivalent(), $t1.DomElement = Bridge.cast(instance, Windows.UI.Xaml.Media.CompositeTransform).INTERNAL_parent.INTERNAL_OuterDomElement, $t1.Value = function (inst, value) {
                                return System.String.concat(value, "deg");
                            }, $t1.Name = function (_o3) {
                                    _o3.add("skewX");
                                    return _o3;
                                }(new (System.Collections.Generic.List$1(System.String)).ctor()), $t1.UIElement = Bridge.cast(instance, Windows.UI.Xaml.Media.CompositeTransform).INTERNAL_parent, $t1.ApplyAlsoWhenThereIsAControlTemplate = true, $t1.OnlyUseVelocity = true, $t1);
                        }
                        return null;
                    }, $t));
                    this.SkewYProperty = Windows.UI.Xaml.DependencyProperty.Register("SkewY", System.Double, Windows.UI.Xaml.Media.CompositeTransform, ($t = new Windows.UI.Xaml.PropertyMetadata.$ctor1(Bridge.box(0.0, System.Double, System.Double.format, System.Double.getHashCode)), $t.GetCSSEquivalent = function (instance) {
                        var $t1;
                        if (Bridge.cast(instance, Windows.UI.Xaml.Media.CompositeTransform).INTERNAL_parent != null) {
                            return ($t1 = new Windows.UI.Xaml.CSSEquivalent(), $t1.DomElement = Bridge.cast(instance, Windows.UI.Xaml.Media.CompositeTransform).INTERNAL_parent.INTERNAL_OuterDomElement, $t1.Value = function (inst, value) {
                                return System.String.concat(value, "deg");
                            }, $t1.Name = function (_o4) {
                                    _o4.add("skewY");
                                    return _o4;
                                }(new (System.Collections.Generic.List$1(System.String)).ctor()), $t1.UIElement = Bridge.cast(instance, Windows.UI.Xaml.Media.CompositeTransform).INTERNAL_parent, $t1.ApplyAlsoWhenThereIsAControlTemplate = true, $t1.OnlyUseVelocity = true, $t1);
                        }
                        return null;
                    }, $t));
                    this.RotationProperty = Windows.UI.Xaml.DependencyProperty.Register("Rotation", System.Double, Windows.UI.Xaml.Media.CompositeTransform, ($t = new Windows.UI.Xaml.PropertyMetadata.$ctor1(Bridge.box(0.0, System.Double, System.Double.format, System.Double.getHashCode)), $t.GetCSSEquivalent = function (instance) {
                        var $t1;
                        if (Bridge.cast(instance, Windows.UI.Xaml.Media.CompositeTransform).INTERNAL_parent != null) {
                            return ($t1 = new Windows.UI.Xaml.CSSEquivalent(), $t1.DomElement = Bridge.cast(instance, Windows.UI.Xaml.Media.CompositeTransform).INTERNAL_parent.INTERNAL_OuterDomElement, $t1.Value = function (inst, value) {
                                return System.String.concat(value, "deg");
                            }, $t1.Name = function (_o5) {
                                    _o5.add("rotateZ");
                                    return _o5;
                                }(new (System.Collections.Generic.List$1(System.String)).ctor()), $t1.UIElement = Bridge.cast(instance, Windows.UI.Xaml.Media.CompositeTransform).INTERNAL_parent, $t1.ApplyAlsoWhenThereIsAControlTemplate = true, $t1.OnlyUseVelocity = true, $t1);
                        }
                        return null;
                    }, $t));
                    this.TranslateXProperty = Windows.UI.Xaml.DependencyProperty.Register("TranslateX", System.Double, Windows.UI.Xaml.Media.CompositeTransform, ($t = new Windows.UI.Xaml.PropertyMetadata.$ctor1(Bridge.box(0.0, System.Double, System.Double.format, System.Double.getHashCode)), $t.GetCSSEquivalent = function (instance) {
                        var $t1;
                        if (Bridge.cast(instance, Windows.UI.Xaml.Media.CompositeTransform).INTERNAL_parent != null) {
                            return ($t1 = new Windows.UI.Xaml.CSSEquivalent(), $t1.DomElement = Bridge.cast(instance, Windows.UI.Xaml.Media.CompositeTransform).INTERNAL_parent.INTERNAL_OuterDomElement, $t1.Value = function (inst, value) {
                                return System.String.concat(value, "px");
                            }, $t1.Name = function (_o6) {
                                    _o6.add("translateX");
                                    return _o6;
                                }(new (System.Collections.Generic.List$1(System.String)).ctor()), $t1.UIElement = Bridge.cast(instance, Windows.UI.Xaml.Media.CompositeTransform).INTERNAL_parent, $t1.ApplyAlsoWhenThereIsAControlTemplate = true, $t1.OnlyUseVelocity = true, $t1);
                        }
                        return null;
                    }, $t));
                    this.TranslateYProperty = Windows.UI.Xaml.DependencyProperty.Register("TranslateY", System.Double, Windows.UI.Xaml.Media.CompositeTransform, ($t = new Windows.UI.Xaml.PropertyMetadata.$ctor1(Bridge.box(0.0, System.Double, System.Double.format, System.Double.getHashCode)), $t.GetCSSEquivalent = function (instance) {
                        var $t1;
                        if (Bridge.cast(instance, Windows.UI.Xaml.Media.CompositeTransform).INTERNAL_parent != null) {
                            return ($t1 = new Windows.UI.Xaml.CSSEquivalent(), $t1.DomElement = Bridge.cast(instance, Windows.UI.Xaml.Media.CompositeTransform).INTERNAL_parent.INTERNAL_OuterDomElement, $t1.Value = function (inst, value) {
                                return System.String.concat(value, "px");
                            }, $t1.Name = function (_o7) {
                                    _o7.add("translateY");
                                    return _o7;
                                }(new (System.Collections.Generic.List$1(System.String)).ctor()), $t1.UIElement = Bridge.cast(instance, Windows.UI.Xaml.Media.CompositeTransform).INTERNAL_parent, $t1.ApplyAlsoWhenThereIsAControlTemplate = true, $t1.OnlyUseVelocity = true, $t1);
                        }
                        return null;
                    }, $t));
                }
            },
            methods: {
                ScaleX_Changed: function (d, e) {
                    var compositeTransform = Bridge.cast(d, Windows.UI.Xaml.Media.CompositeTransform);
                    compositeTransform.INTERNAL_ApplyTransform();
                },
                ScaleY_Changed: function (d, e) {
                    var compositeTransform = Bridge.cast(d, Windows.UI.Xaml.Media.CompositeTransform);
                    compositeTransform.INTERNAL_ApplyTransform();
                },
                SkewX_Changed: function (d, e) {
                    var compositeTransform = Bridge.cast(d, Windows.UI.Xaml.Media.CompositeTransform);
                    compositeTransform.INTERNAL_ApplyTransform();
                },
                SkewY_Changed: function (d, e) {
                    var compositeTransform = Bridge.cast(d, Windows.UI.Xaml.Media.CompositeTransform);
                    compositeTransform.INTERNAL_ApplyTransform();
                },
                Rotation_Changed: function (d, e) {
                    var compositeTransform = Bridge.cast(d, Windows.UI.Xaml.Media.CompositeTransform);
                    compositeTransform.INTERNAL_ApplyTransform();
                },
                TranslateX_Changed: function (d, e) {
                    var compositeTransform = Bridge.cast(d, Windows.UI.Xaml.Media.CompositeTransform);
                    compositeTransform.INTERNAL_ApplyTransform();
                },
                TranslateY_Changed: function (d, e) {
                    var compositeTransform = Bridge.cast(d, Windows.UI.Xaml.Media.CompositeTransform);
                    compositeTransform.INTERNAL_ApplyTransform();
                }
            }
        },
        props: {
            /**
             * Gets or sets the x-axis scale factor. You can use this property to stretch
             or shrink an object horizontally. The default is 1.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Media.CompositeTransform
             * @function ScaleX
             * @type number
             */
            ScaleX: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.Media.CompositeTransform.ScaleXProperty), System.Double), System.Double));
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Media.CompositeTransform.ScaleXProperty, Bridge.box(value, System.Double, System.Double.format, System.Double.getHashCode));
                }
            },
            /**
             * Gets or sets the y-axis scale factor. You can use this property to stretch
             or shrink an object vertically. The default is 1.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Media.CompositeTransform
             * @function ScaleY
             * @type number
             */
            ScaleY: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.Media.CompositeTransform.ScaleYProperty), System.Double), System.Double));
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Media.CompositeTransform.ScaleYProperty, Bridge.box(value, System.Double, System.Double.format, System.Double.getHashCode));
                }
            },
            /**
             * Gets or sets the x-axis skew angle, which is measured in degrees counterclockwise
             from the y-axis. A skew transform can be useful for creating the illusion
             of three-dimensional depth in a two-dimensional object. The default is 0.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Media.CompositeTransform
             * @function SkewX
             * @type number
             */
            SkewX: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.Media.CompositeTransform.SkewXProperty), System.Double), System.Double));
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Media.CompositeTransform.SkewXProperty, Bridge.box(value, System.Double, System.Double.format, System.Double.getHashCode));
                }
            },
            /**
             * Gets or sets the y-axis skew angle, which is measured in degrees counterclockwise
             from the x-axis. A skew transform can be useful for creating the illusion
             of three-dimensional depth in a two-dimensional object. The default is 0.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Media.CompositeTransform
             * @function SkewY
             * @type number
             */
            SkewY: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.Media.CompositeTransform.SkewYProperty), System.Double), System.Double));
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Media.CompositeTransform.SkewYProperty, Bridge.box(value, System.Double, System.Double.format, System.Double.getHashCode));
                }
            },
            /**
             * Gets or sets the angle, in degrees, of clockwise rotation. The default is 0.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Media.CompositeTransform
             * @function Rotation
             * @type number
             */
            Rotation: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.Media.CompositeTransform.RotationProperty), System.Double), System.Double));
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Media.CompositeTransform.RotationProperty, Bridge.box(value, System.Double, System.Double.format, System.Double.getHashCode));
                }
            },
            /**
             * Gets or sets the distance to translate along the x-axis.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Media.CompositeTransform
             * @function TranslateX
             * @type number
             */
            TranslateX: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.Media.CompositeTransform.TranslateXProperty), System.Double), System.Double));
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Media.CompositeTransform.TranslateXProperty, Bridge.box(value, System.Double, System.Double.format, System.Double.getHashCode));
                }
            },
            /**
             * Gets or sets the distance to translate (move) an object along the y-axis. The default is 0.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Media.CompositeTransform
             * @function TranslateY
             * @type number
             */
            TranslateY: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.Media.CompositeTransform.TranslateYProperty), System.Double), System.Double));
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Media.CompositeTransform.TranslateYProperty, Bridge.box(value, System.Double, System.Double.format, System.Double.getHashCode));
                }
            }
        },
        methods: {
            ApplyCSSChanges: function (compositeTransform, scaleX, scaleY, skewX, skewY, rotation, translateX, translateY) {

                var translateXcssEquivalent = Windows.UI.Xaml.Media.CompositeTransform.TranslateXProperty.GetTypeMetaData(Windows.UI.Xaml.Media.CompositeTransform).GetCSSEquivalent(compositeTransform);
                if (translateXcssEquivalent != null) {
                    CSHTML5.Internal.INTERNAL_HtmlDomManager.SetDomElementStylePropertyUsingVelocity(translateXcssEquivalent.DomElement, translateXcssEquivalent.Name, translateXcssEquivalent.Value(compositeTransform, Bridge.box(translateX, System.Double, System.Double.format, System.Double.getHashCode)));
                }

                var translateYcssEquivalent = Windows.UI.Xaml.Media.CompositeTransform.TranslateYProperty.GetTypeMetaData(Windows.UI.Xaml.Media.CompositeTransform).GetCSSEquivalent(compositeTransform);
                if (translateYcssEquivalent != null) {
                    CSHTML5.Internal.INTERNAL_HtmlDomManager.SetDomElementStylePropertyUsingVelocity(translateYcssEquivalent.DomElement, translateYcssEquivalent.Name, translateYcssEquivalent.Value(compositeTransform, Bridge.box(translateY, System.Double, System.Double.format, System.Double.getHashCode)));
                }

                var rotationcssEquivalent = Windows.UI.Xaml.Media.CompositeTransform.RotationProperty.GetTypeMetaData(Windows.UI.Xaml.Media.CompositeTransform).GetCSSEquivalent(compositeTransform);
                if (rotationcssEquivalent != null) {
                    CSHTML5.Internal.INTERNAL_HtmlDomManager.SetDomElementStylePropertyUsingVelocity(rotationcssEquivalent.DomElement, rotationcssEquivalent.Name, rotationcssEquivalent.Value(compositeTransform, Bridge.box(rotation, System.Double, System.Double.format, System.Double.getHashCode)));
                }

                var skewXcssEquivalent = Windows.UI.Xaml.Media.CompositeTransform.SkewXProperty.GetTypeMetaData(Windows.UI.Xaml.Media.CompositeTransform).GetCSSEquivalent(compositeTransform);
                if (skewXcssEquivalent != null) {
                    CSHTML5.Internal.INTERNAL_HtmlDomManager.SetDomElementStylePropertyUsingVelocity(skewXcssEquivalent.DomElement, skewXcssEquivalent.Name, skewXcssEquivalent.Value(compositeTransform, Bridge.box(skewX, System.Double, System.Double.format, System.Double.getHashCode)));
                }

                var skewYcssEquivalent = Windows.UI.Xaml.Media.CompositeTransform.SkewYProperty.GetTypeMetaData(Windows.UI.Xaml.Media.CompositeTransform).GetCSSEquivalent(compositeTransform);
                if (skewYcssEquivalent != null) {
                    CSHTML5.Internal.INTERNAL_HtmlDomManager.SetDomElementStylePropertyUsingVelocity(skewYcssEquivalent.DomElement, skewYcssEquivalent.Name, skewYcssEquivalent.Value(compositeTransform, Bridge.box(skewY, System.Double, System.Double.format, System.Double.getHashCode)));
                }

                var scaleXcssEquivalent = Windows.UI.Xaml.Media.CompositeTransform.ScaleXProperty.GetTypeMetaData(Windows.UI.Xaml.Media.CompositeTransform).GetCSSEquivalent(compositeTransform);
                if (scaleXcssEquivalent != null) {
                    CSHTML5.Internal.INTERNAL_HtmlDomManager.SetDomElementStylePropertyUsingVelocity(scaleXcssEquivalent.DomElement, scaleXcssEquivalent.Name, scaleXcssEquivalent.Value(compositeTransform, Bridge.box(scaleX, System.Double, System.Double.format, System.Double.getHashCode)));
                }

                var scaleYcssEquivalent = Windows.UI.Xaml.Media.CompositeTransform.ScaleYProperty.GetTypeMetaData(Windows.UI.Xaml.Media.CompositeTransform).GetCSSEquivalent(compositeTransform);
                if (scaleYcssEquivalent != null) {
                    CSHTML5.Internal.INTERNAL_HtmlDomManager.SetDomElementStylePropertyUsingVelocity(scaleYcssEquivalent.DomElement, scaleYcssEquivalent.Name, scaleYcssEquivalent.Value(compositeTransform, Bridge.box(scaleY, System.Double, System.Double.format, System.Double.getHashCode)));
                }

            },
            INTERNAL_ApplyCSSChanges: function () {
                this.ApplyCSSChanges(this, this.ScaleX, this.ScaleY, this.SkewX, this.SkewY, this.Rotation, this.TranslateX, this.TranslateY);
            },
            INTERNAL_UnapplyCSSChanges: function () {
                this.ApplyCSSChanges(this, 1, 1, 0, 0, 0, 0, 0);
            },
            ApplyCompositeTransform: function (scaleX, scaleY, skewX, skewY, rotation, translateX, translateY) {
                if (this.INTERNAL_parent != null && CSHTML5.Internal.INTERNAL_VisualTreeManager.IsElementInVisualTree(this.INTERNAL_parent)) {
                    this.ApplyCSSChanges(this, scaleX, scaleY, skewX, skewY, rotation, translateX, translateY);
                }
            },
            INTERNAL_ApplyTransform: function () {
                this.ApplyCompositeTransform(this.ScaleX, this.ScaleY, this.SkewX, this.SkewY, this.Rotation, this.TranslateX, this.TranslateY);
            },
            INTERNAL_UnapplyTransform: function () {
                if (this.INTERNAL_parent != null && CSHTML5.Internal.INTERNAL_VisualTreeManager.IsElementInVisualTree(this.INTERNAL_parent)) {
                    this.INTERNAL_UnapplyCSSChanges();
                }
            },
            INTERNAL_TransformPoint: function (point) {
                throw new System.NotImplementedException.$ctor1("Please contact support@cshtml5.com");
            }
        }
    });

    /**
     * Provides the practical object source type for the Source and ImageSource properties.
     *
     * @public
     * @class Windows.UI.Xaml.Media.Imaging.BitmapImage
     * @augments Windows.UI.Xaml.Media.Imaging.BitmapSource
     */
    Bridge.define("Windows.UI.Xaml.Media.Imaging.BitmapImage", {
        inherits: [Windows.UI.Xaml.Media.Imaging.BitmapSource],
        statics: {
            fields: {
                /**
                 * Identifies the UriSource dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Media.Imaging.BitmapImage
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                UriSourceProperty: null
            },
            ctors: {
                init: function () {
                    var $t;
                    this.UriSourceProperty = Windows.UI.Xaml.DependencyProperty.Register("UriSource", System.Uri, Windows.UI.Xaml.Media.Imaging.BitmapImage, ($t = new Windows.UI.Xaml.PropertyMetadata.$ctor2(null, Windows.UI.Xaml.Media.Imaging.BitmapImage.UriSource_Changed), $t.CallPropertyChangedWhenLoadedIntoVisualTree = Windows.UI.Xaml.WhenToCallPropertyChangedEnum.IfPropertyIsSet, $t));
                }
            },
            methods: {
                UriSource_Changed: function (i, e) {
                    var bitmapImage = Bridge.cast(i, Windows.UI.Xaml.Media.Imaging.BitmapImage);
                    var newValue = Bridge.cast(e.NewValue, System.Uri);
                    bitmapImage.OnUriSourceChanged();


                }
            }
        },
        fields: {
            INTERNAL_NameOfAssemblyThatSetTheSourceUri: null
        },
        events: {
            /**
             * Occurs when there is an error associated with image retrieval or format.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Media.Imaging.BitmapImage
             * @memberof Windows.UI.Xaml.Media.Imaging.BitmapImage
             * @function addImageFailed
             * @param   {Windows.UI.Xaml.ExceptionRoutedEventHandler}    value
             * @return  {void}
             */
            /**
             * Occurs when there is an error associated with image retrieval or format.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Media.Imaging.BitmapImage
             * @memberof Windows.UI.Xaml.Media.Imaging.BitmapImage
             * @function removeImageFailed
             * @param   {Windows.UI.Xaml.ExceptionRoutedEventHandler}    value
             * @return  {void}
             */
            ImageFailed: null,
            /**
             * Occurs when the image source is downloaded and decoded with no failure. You can use this event to determine the size of an image before rendering it.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Media.Imaging.BitmapImage
             * @memberof Windows.UI.Xaml.Media.Imaging.BitmapImage
             * @function addImageOpened
             * @param   {Windows.UI.Xaml.RoutedEventHandler}    value
             * @return  {void}
             */
            /**
             * Occurs when the image source is downloaded and decoded with no failure. You can use this event to determine the size of an image before rendering it.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Media.Imaging.BitmapImage
             * @memberof Windows.UI.Xaml.Media.Imaging.BitmapImage
             * @function removeImageOpened
             * @param   {Windows.UI.Xaml.RoutedEventHandler}    value
             * @return  {void}
             */
            ImageOpened: null,
            /**
             * Occurs when the UriSource is changed
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Media.Imaging.BitmapImage
             * @memberof Windows.UI.Xaml.Media.Imaging.BitmapImage
             * @function addUriSourceChanged
             * @param   {System.EventHandler}    value
             * @return  {void}
             */
            /**
             * Occurs when the UriSource is changed
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Media.Imaging.BitmapImage
             * @memberof Windows.UI.Xaml.Media.Imaging.BitmapImage
             * @function removeUriSourceChanged
             * @param   {System.EventHandler}    value
             * @return  {void}
             */
            UriSourceChanged: null
        },
        props: {
            /**
             * / Gets or sets the height to use for image decoding operations.
            /
             * Gets or sets the Uniform Resource Identifier (URI) of the graphics source
             file that generated this BitmapImage.
             Returns an object describing the of the graphics source file that generated this BitmapImage.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Media.Imaging.BitmapImage
             * @function UriSource
             * @type System.Uri
             */
            UriSource: {
                get: function () {
                    return Bridge.cast(this.GetValue(Windows.UI.Xaml.Media.Imaging.BitmapImage.UriSourceProperty), System.Uri);
                },
                set: function (value) {
                    var callerAssemblyName = CSHTML5.Internal.INTERNAL_UriHelper.GetJavaScriptCallingAssembly();

                    this.INTERNAL_NameOfAssemblyThatSetTheSourceUri = callerAssemblyName;
                    this.SetValue(Windows.UI.Xaml.Media.Imaging.BitmapImage.UriSourceProperty, value);
                }
            }
        },
        ctors: {
            /**
             * Initializes a new instance of the BitmapImage class.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Media.Imaging.BitmapImage
             * @memberof Windows.UI.Xaml.Media.Imaging.BitmapImage
             * @return  {void}
             */
            ctor: function () {
                this.$initialize();
                Windows.UI.Xaml.Media.Imaging.BitmapSource.ctor.call(this);
            },
            /**
             * Initializes a new instance of the BitmapImage class with the specified path to the source.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Media.Imaging.BitmapImage
             * @memberof Windows.UI.Xaml.Media.Imaging.BitmapImage
             * @param   {System.Uri}    uriSource    The path to the source of the image.
             * @return  {void}
             */
            $ctor1: function (uriSource) {
                this.$initialize();
                Windows.UI.Xaml.Media.Imaging.BitmapSource.ctor.call(this);
                this.UriSource = uriSource;
            }
        },
        methods: {
            OnImageFailed: function () {
                var $t;
                if (!Bridge.staticEquals(this.ImageFailed, null)) {
                    this.ImageFailed(this, ($t = new Windows.UI.Xaml.ExceptionRoutedEventArgs.ctor(), $t.OriginalSource = this, $t));
                }
            },
            OnImageOpened: function () {
                var $t;
                if (!Bridge.staticEquals(this.ImageOpened, null)) {
                    this.ImageOpened(this, ($t = new Windows.UI.Xaml.RoutedEventArgs(), $t.OriginalSource = this, $t));
                }
            },
            OnUriSourceChanged: function () {
                if (!Bridge.staticEquals(this.UriSourceChanged, null)) {
                    this.UriSourceChanged(this, { });
                }
            }
        }
    });

    /**
     * Paints an area with a linear gradient.
     *
     * @public
     * @class Windows.UI.Xaml.Media.LinearGradientBrush
     * @augments Windows.UI.Xaml.Media.GradientBrush
     * @implements  Windows.UI.Xaml.ICanConvertToCSSValues
     */
    Bridge.define("Windows.UI.Xaml.Media.LinearGradientBrush", {
        inherits: [Windows.UI.Xaml.Media.GradientBrush,Windows.UI.Xaml.ICanConvertToCSSValues],
        statics: {
            fields: {
                /**
                 * Identifies the EndPoint dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Media.LinearGradientBrush
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                EndPointProperty: null,
                /**
                 * Identifies the StartPoint dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Media.LinearGradientBrush
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                StartPointProperty: null
            },
            ctors: {
                init: function () {
                    var $t;
                    this.EndPointProperty = Windows.UI.Xaml.DependencyProperty.Register("EndPoint", Windows.Foundation.Point, Windows.UI.Xaml.Media.LinearGradientBrush, ($t = new Windows.UI.Xaml.PropertyMetadata.$ctor2(new Windows.Foundation.Point.$ctor1(1, 1).$clone(), Windows.UI.Xaml.Media.LinearGradientBrush.EndPoint_Changed), $t.CallPropertyChangedWhenLoadedIntoVisualTree = Windows.UI.Xaml.WhenToCallPropertyChangedEnum.IfPropertyIsSet, $t));
                    this.StartPointProperty = Windows.UI.Xaml.DependencyProperty.Register("StartPoint", Windows.Foundation.Point, Windows.UI.Xaml.Media.LinearGradientBrush, ($t = new Windows.UI.Xaml.PropertyMetadata.$ctor2(new Windows.Foundation.Point.ctor().$clone(), Windows.UI.Xaml.Media.LinearGradientBrush.StartPoint_Changed), $t.CallPropertyChangedWhenLoadedIntoVisualTree = Windows.UI.Xaml.WhenToCallPropertyChangedEnum.IfPropertyIsSet, $t));
                }
            },
            methods: {
                EndPoint_Changed: function (d, e) {
                    Bridge.cast(d, Windows.UI.Xaml.Media.LinearGradientBrush).RecalculateAngle();
                },
                StartPoint_Changed: function (d, e) {
                    Bridge.cast(d, Windows.UI.Xaml.Media.LinearGradientBrush).RecalculateAngle();
                }
            }
        },
        fields: {
            _angle: 0
        },
        props: {
            /**
             * Gets or sets the ending two-dimensional coordinates of the linear gradient.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Media.LinearGradientBrush
             * @function EndPoint
             * @type Windows.Foundation.Point
             */
            EndPoint: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.Media.LinearGradientBrush.EndPointProperty), Windows.Foundation.Point), Windows.Foundation.Point));
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Media.LinearGradientBrush.EndPointProperty, value.$clone());
                }
            },
            /**
             * Gets or sets the starting two-dimensional coordinates of the linear gradient.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Media.LinearGradientBrush
             * @function StartPoint
             * @type Windows.Foundation.Point
             */
            StartPoint: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.Media.LinearGradientBrush.StartPointProperty), Windows.Foundation.Point), Windows.Foundation.Point));
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Media.LinearGradientBrush.StartPointProperty, value.$clone());
                }
            }
        },
        alias: ["ConvertToCSSValues", "Windows$UI$Xaml$ICanConvertToCSSValues$ConvertToCSSValues"],
        ctors: {
            ctor: function () {
                this.$initialize();
                Windows.UI.Xaml.Media.GradientBrush.ctor.call(this);
            },
            /**
             * Initializes a new instance of the LinearGradientBrush class that has the
             specified GradientStopCollection and angle.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Media.LinearGradientBrush
             * @memberof Windows.UI.Xaml.Media.LinearGradientBrush
             * @param   {Windows.UI.Xaml.Media.GradientStopCollection}    gradientStopCollection    The GradientStops to set on this brush.
             * @param   {number}                                          angle                     A System.Double that represents the angle, in degrees, of the gradient. A
             value of 0 creates a horizontal gradient, and a value of 90 creates a vertical
             gradient. Negative values are permitted, as are values over 360 (which are
             treated as mod 360).
             * @return  {void}
             */
            $ctor1: function (gradientStopCollection, angle) {
                this.$initialize();
                Windows.UI.Xaml.Media.GradientBrush.ctor.call(this);
                this.GradientStops = gradientStopCollection;

                this._angle = -angle;
            }
        },
        methods: {
            RecalculateAngle: function () {
                this._angle = Math.atan2(this.EndPoint.X - this.StartPoint.X, this.EndPoint.Y - this.StartPoint.Y) * 180.0 / Math.PI - 90.0;
            },
            ToHtmlStringForAbsoluteMappingMode: function (parent, gradientStopsString, alpha) {

                var startDistance;
                var endDistance;

                if (this.EndPoint.X === this.StartPoint.X) {
                    startDistance = this.StartPoint.Y;
                    endDistance = this.EndPoint.Y;
                    alpha.v = -1.5707963267948966;
                } else if (this.EndPoint.Y === this.StartPoint.Y) {

                    startDistance = this.StartPoint.X;
                    endDistance = this.EndPoint.X;
                    alpha.v = 0;
                } else {
                    var slope = (this.EndPoint.Y - this.StartPoint.Y) / (this.EndPoint.X - this.StartPoint.X);
                    var perpSlope = -1 / slope;

                    var kStart = this.StartPoint.Y + 1 / slope * this.StartPoint.X;
                    var kEnd = this.EndPoint.Y + 1 / slope * this.EndPoint.X;


                    if (slope > 0) {

                        var XIStart = kStart / (slope - perpSlope);
                        var YIStart = slope * XIStart;

                        var XIEnd = kEnd / (slope - perpSlope);
                        var YIEnd = slope * XIEnd;

                        startDistance = Math.sqrt(Math.pow(XIStart, 2) + Math.pow(YIStart, 2));
                        endDistance = Math.sqrt(Math.pow(XIEnd, 2) + Math.pow(YIEnd, 2));
                    } else {

                        var height = 1;
                        if (!(parent == null) && Bridge.is(parent, Windows.UI.Xaml.FrameworkElement)) {
                            var parentAsFrameworkElement = Bridge.cast(parent, Windows.UI.Xaml.FrameworkElement);
                            height = parentAsFrameworkElement.ActualHeight;
                        }

                        var XIStart1 = (height - kStart) / (1 - 1 / slope);
                        var YIStart1 = height - slope * XIStart1;

                        var XIEnd1 = (height - kEnd) / (1 - 1 / slope);
                        var YIEnd1 = height - slope * XIEnd1;


                        startDistance = Math.sqrt(Math.pow(XIStart1, 2) + Math.pow(height - YIStart1, 2));
                        endDistance = Math.sqrt(Math.pow(XIEnd1, 2) + Math.pow(height - YIEnd1, 2));
                    }

                    var XVariation = (this.EndPoint.X - this.StartPoint.X);
                    var YVariation = (this.EndPoint.Y - this.StartPoint.Y);
                    if (XVariation < 0) {
                        XVariation = -XVariation;
                        YVariation = -YVariation;
                    }
                    alpha.v = Math.atan2(XVariation, YVariation) - 1.5707963267948966;
                }
                var startToEndDistance = endDistance - startDistance;
                gradientStopsString.v = this.GetOffsetsString(startDistance, endDistance, startToEndDistance, "px");
            },
            ToHtmlStringForRelativeMappingMode: function (parent, gradientStopsString, alpha) {
                var height = 1;
                var width = 1;
                if (!(parent == null) && Bridge.is(parent, Windows.UI.Xaml.FrameworkElement)) {
                    var parentAsFrameworkElement = Bridge.cast(parent, Windows.UI.Xaml.FrameworkElement);
                    var actualSize = parentAsFrameworkElement.INTERNAL_GetActualWidthAndHeight();
                    width = actualSize.Width;
                    height = actualSize.Height;
                }

                var startX = this.StartPoint.X;
                var startY = this.StartPoint.Y;
                var endX = this.EndPoint.X;
                var endY = this.EndPoint.Y;









                var XVariation = (endX - startX) * width;
                var YVariation = (endY - startY) * height;
                if (XVariation < 0) {
                    XVariation = -XVariation;
                    YVariation = -YVariation;
                }

                alpha.v = Math.atan2(XVariation, YVariation) - 1.5707963267948966;

                var D;


                var m = (endY - startY) / (endX - startX);
                var startPointPercentage;
                var endPointPercentage;
                if (m !== 0) {
                    var kStart = startY + 1 / m * startX;
                    var kEnd = endY + 1 / m * endX;

                    D = Math.sqrt(2);

                    if (m > 0) {

                        var XIStart = kStart / (1 + 1 / m);
                        var XIEnd = kEnd / (1 + 1 / m);


                        startPointPercentage = Math.sqrt(2 * XIStart * XIStart) / D * 100;
                        endPointPercentage = Math.sqrt(2 * XIEnd * XIEnd) / D * 100;
                    } else {

                        var XIStart1 = (1 - kStart) / (1 - 1 / m);

                        var XIEnd1 = (1 - kEnd) / (1 - 1 / m);

                        var startDistance = Math.sqrt(2 * XIStart1 * XIStart1);
                        startPointPercentage = startDistance / D * 100;

                        var endDistance = Math.sqrt(2 * XIEnd1 * XIEnd1);
                        endPointPercentage = endDistance / D * 100;
                    }
                } else {
                    startPointPercentage = startX * 100;
                    endPointPercentage = endX * 100;
                }





                var startToEndPercentage = endPointPercentage - startPointPercentage;


                gradientStopsString.v = this.GetOffsetsString(startPointPercentage, endPointPercentage, startToEndPercentage, "%");
            },
            INTERNAL_ToHtmlString: function (parent) {
                var $t;

                var alpha = { };
                var gradientStopsString = { };
                if (Windows.Foundation.Point.op_Equality(this.StartPoint.$clone(), this.EndPoint.$clone())) {
                    var color = null;
                    var biggestOffset = 0;
                    $t = Bridge.getEnumerator(this.GradientStops);
                    try {
                        while ($t.moveNext()) {
                            var gradientStop = $t.Current;
                            if (gradientStop.Offset > biggestOffset) {
                                biggestOffset = gradientStop.Offset;
                                color = gradientStop.Color.INTERNAL_ToHtmlString(this.Opacity);
                            }
                        }
                    } finally {
                        if (Bridge.is($t, System.IDisposable)) {
                            $t.System$IDisposable$Dispose();
                        }
                    }
                    gradientStopsString.v = (color || "") + " 0%, " + (color || "") + " 100%";
                    alpha.v = 0;
                } else {
                    if (this.MappingMode === Windows.UI.Xaml.Media.BrushMappingMode.RelativeToBoundingBox) {
                        this.ToHtmlStringForRelativeMappingMode(parent, gradientStopsString, alpha);
                    } else {
                        this.ToHtmlStringForAbsoluteMappingMode(parent, gradientStopsString, alpha);
                    }
                }

                var angle = alpha.v * 180 / Math.PI;


                if (!CSHTML5.Interop.IsRunningInTheSimulator) {
                    angle = 360 - angle + 90;
                }


                var gradientType = this.SpreadMethod === Windows.UI.Xaml.Media.GradientSpreadMethod.Repeat ? "repeating-linear-gradient" : "linear-gradient";
                var baseString = (gradientType || "") + "(" + (System.Double.format(angle) || "") + "deg, " + (gradientStopsString.v || "") + ")";
                var returnValues = new (System.Collections.Generic.List$1(System.Object)).ctor();
                returnValues.add("-webkit-" + (baseString || ""));
                returnValues.add("-o-" + (baseString || ""));
                returnValues.add("-moz-" + (baseString || ""));
                returnValues.add(baseString);
                return returnValues;
            },
            GetOffsetsString: function (startPointPercentage, endPointPercentage, startToEndPercentage, percentageSymbol) {
                var $t;
                var gradientStopsString = "";
                var isFirst = true;
                var tempList = new (System.Collections.Generic.List$1(System.Tuple$2(System.Double,System.String))).ctor();

                var smallestOffset = System.Double.max;
                var smallestColor = "";
                var biggestOffset = System.Double.min;
                var biggestColor = "";

                $t = Bridge.getEnumerator(this.GradientStops);
                try {
                    while ($t.moveNext()) {
                        var gradientStop = $t.Current;
                        var currentColor = gradientStop.Color.INTERNAL_ToHtmlString(this.Opacity);
                        var str = (currentColor || "") + " " + (System.Double.format((startPointPercentage + gradientStop.Offset * startToEndPercentage)) || "") + (percentageSymbol || "");
                        tempList.add({ Item1: gradientStop.Offset, Item2: str });
                        if (smallestOffset > gradientStop.Offset) {
                            smallestOffset = gradientStop.Offset;
                            smallestColor = currentColor;
                        }
                        if (biggestOffset <= gradientStop.Offset) {
                            biggestOffset = gradientStop.Offset;
                            biggestColor = currentColor;
                        }
                    }
                } finally {
                    if (Bridge.is($t, System.IDisposable)) {
                        $t.System$IDisposable$Dispose();
                    }
                }

                if (startToEndPercentage > 0) {
                    if (smallestOffset !== 0) {
                        gradientStopsString = (smallestColor || "") + " " + (System.Double.format(startPointPercentage) || "") + (percentageSymbol || "");
                        isFirst = false;
                    }
                } else {
                    if (biggestOffset !== 1) {
                        gradientStopsString = (biggestColor || "") + " " + (System.Double.format(endPointPercentage) || "") + (percentageSymbol || "");
                        isFirst = false;
                    }
                }

                while (tempList.Count > 0) {
                    if (!isFirst) {
                        gradientStopsString = (gradientStopsString || "") + ", ";
                    }

                    var indexOfMin = 0;
                    var currentMin = System.Linq.Enumerable.from(tempList).elementAt(0).Item1;

                    for (var i = 1; i < tempList.Count; i = (i + 1) | 0) {
                        if (startToEndPercentage > 0) {
                            if (System.Linq.Enumerable.from(tempList).elementAt(i).Item1 < currentMin) {
                                indexOfMin = i;
                                currentMin = System.Linq.Enumerable.from(tempList).elementAt(i).Item1;
                            }
                        } else {
                            if (System.Linq.Enumerable.from(tempList).elementAt(i).Item1 > currentMin) {
                                indexOfMin = i;
                                currentMin = System.Linq.Enumerable.from(tempList).elementAt(i).Item1;
                            }
                        }
                    }

                    gradientStopsString = (gradientStopsString || "") + (System.Linq.Enumerable.from(tempList).elementAt(indexOfMin).Item2 || "");
                    tempList.removeAt(indexOfMin);
                    isFirst = false;
                }

                if (startToEndPercentage > 0) {
                    if (biggestOffset !== 1) {
                        gradientStopsString = (gradientStopsString || "") + ((", " + (biggestColor || "") + " " + (System.Double.format(endPointPercentage) || "") + (percentageSymbol || "")) || "");
                    }
                } else {
                    if (smallestOffset !== 0) {
                        gradientStopsString = (gradientStopsString || "") + ((", " + (smallestColor || "") + " " + (System.Double.format(startToEndPercentage) || "") + (percentageSymbol || "")) || "");
                    }
                }
                return gradientStopsString;
            },
            ConvertToCSSValues: function (parent) {
                return this.INTERNAL_ToHtmlString(parent);
            }
        }
    });

    /**
     * Creates an arbitrary affine matrix transformation that is used to manipulate objects or coordinate systems in a two-dimensional plane.
     *
     * @public
     * @class Windows.UI.Xaml.Media.MatrixTransform
     * @augments Windows.UI.Xaml.Media.Transform
     */
    Bridge.define("Windows.UI.Xaml.Media.MatrixTransform", {
        inherits: [Windows.UI.Xaml.Media.Transform],
        statics: {
            fields: {
                /**
                 * Identifies the Matrix dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Media.MatrixTransform
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                MatrixProperty: null
            },
            ctors: {
                init: function () {
                    this.MatrixProperty = Windows.UI.Xaml.DependencyProperty.Register("Matrix", Windows.UI.Xaml.Media.Matrix, Windows.UI.Xaml.Media.MatrixTransform, new Windows.UI.Xaml.PropertyMetadata.$ctor1(Windows.UI.Xaml.Media.Matrix.Identity.$clone()));
                }
            }
        },
        props: {
            /**
             * Gets or sets the Matrix that defines this transformation. The default is an
             identity Matrix. An identity matrix has a value of 1 in coefficients [1,1],
             [2,2], and [3,3]; and a value of 0 in the rest of the coefficients.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Media.MatrixTransform
             * @function Matrix
             * @type Windows.UI.Xaml.Media.Matrix
             */
            Matrix: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.Media.MatrixTransform.MatrixProperty), Windows.UI.Xaml.Media.Matrix), Windows.UI.Xaml.Media.Matrix));
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Media.MatrixTransform.MatrixProperty, value.$clone());
                }
            }
        },
        methods: {
            ApplyCSSChanges: function (matrixTransform) {
                throw new System.NotImplementedException.ctor();
            },
            INTERNAL_ApplyCSSChanges: function () {
                this.ApplyCSSChanges(this);
            },
            INTERNAL_UnapplyCSSChanges: function () {
                this.ApplyCSSChanges(this);
            },
            INTERNAL_ApplyTransform: function () {
                if (this.INTERNAL_parent != null && CSHTML5.Internal.INTERNAL_VisualTreeManager.IsElementInVisualTree(this.INTERNAL_parent)) {
                    var domStyle = CSHTML5.Internal.INTERNAL_HtmlDomManager.GetFrameworkElementOuterStyleForModification(this.INTERNAL_parent);

                    var matrix = this.Matrix.$clone();

                    var value = "matrix(" + (System.Double.format(matrix.M11, "G", System.Globalization.CultureInfo.invariantCulture) || "") + "," + (System.Double.format(matrix.M12, "G", System.Globalization.CultureInfo.invariantCulture) || "") + "," + (System.Double.format(matrix.M21, "G", System.Globalization.CultureInfo.invariantCulture) || "") + "," + (System.Double.format(matrix.M22, "G", System.Globalization.CultureInfo.invariantCulture) || "") + "," + (System.Double.format(matrix.OffsetX, "G", System.Globalization.CultureInfo.invariantCulture) || "") + "," + (System.Double.format(matrix.OffsetY, "G", System.Globalization.CultureInfo.invariantCulture) || "") + ")";

                    try {
                        domStyle.transform = value;
                    } catch ($e1) {
                        $e1 = System.Exception.create($e1);
                    }
                    try {
                        domStyle.msTransform = value;
                    } catch ($e2) {
                        $e2 = System.Exception.create($e2);
                    }
                    try {
                        domStyle.WebkitTransform = value;
                    } catch ($e3) {
                        $e3 = System.Exception.create($e3);
                    }

                }
            },
            INTERNAL_UnapplyTransform: function () {
                if (this.INTERNAL_parent != null && CSHTML5.Internal.INTERNAL_VisualTreeManager.IsElementInVisualTree(this.INTERNAL_parent)) {
                    var domStyle = CSHTML5.Internal.INTERNAL_HtmlDomManager.GetFrameworkElementOuterStyleForModification(this.INTERNAL_parent);

                    var value = "";

                    try {
                        domStyle.transform = value;
                    } catch ($e1) {
                        $e1 = System.Exception.create($e1);
                    }
                    try {
                        domStyle.msTransform = value;
                    } catch ($e2) {
                        $e2 = System.Exception.create($e2);
                    }
                    try {
                        domStyle.WebkitTransform = value;
                    } catch ($e3) {
                        $e3 = System.Exception.create($e3);
                    }
                }
            },
            INTERNAL_TransformPoint: function (point) {
                throw new System.NotImplementedException.$ctor1("Please contact support@cshtml5.com");
            }
        }
    });

    /**
     * Paints an area with a radial gradient. A focal point defines the beginning
     of the gradient, and a circle defines the end point of the gradient.
     * // Creates a modifiable clone of this System.Windows.Media.RadialGradientBrush,
    // making deep copies of this object's values. When copying dependency properties,
    // this method copies resource references and data bindings (but they might
    // no longer resolve) but not animations or their current values.
    //
     * // Creates a modifiable clone of this System.Windows.Media.RadialGradientBrush
    // object, making deep copies of this object's current values. Resource references,
    // data bindings, and animations are not copied, but their current values are.
    //
     *
     * @public
     * @class Windows.UI.Xaml.Media.RadialGradientBrush
     * @augments Windows.UI.Xaml.Media.GradientBrush
     * @implements  Windows.UI.Xaml.ICanConvertToCSSValues
     * @return  {[type]}
     */
    Bridge.define("Windows.UI.Xaml.Media.RadialGradientBrush", {
        inherits: [Windows.UI.Xaml.Media.GradientBrush,Windows.UI.Xaml.ICanConvertToCSSValues],
        statics: {
            fields: {
                /**
                 * Identifies the System.Windows.Media.RadialGradientBrush.Center dependency
                 property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Media.RadialGradientBrush
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                CenterProperty: null,
                /**
                 * Identifies the System.Windows.Media.RadialGradientBrush.GradientOrigin dependency
                 property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Media.RadialGradientBrush
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                GradientOriginProperty: null,
                /**
                 * Identifies the System.Windows.Media.RadialGradientBrush.RadiusX dependency
                 property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Media.RadialGradientBrush
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                RadiusXProperty: null,
                /**
                 * Identifies the System.Windows.Media.RadialGradientBrush.RadiusY dependency
                 property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Media.RadialGradientBrush
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                RadiusYProperty: null
            },
            ctors: {
                init: function () {
                    this.CenterProperty = Windows.UI.Xaml.DependencyProperty.Register("Center", Windows.Foundation.Point, Windows.UI.Xaml.Media.RadialGradientBrush, new Windows.UI.Xaml.PropertyMetadata.$ctor1(new Windows.Foundation.Point.$ctor1(0.5, 0.5).$clone()));
                    this.GradientOriginProperty = Windows.UI.Xaml.DependencyProperty.Register("GradientOrigin", Windows.Foundation.Point, Windows.UI.Xaml.Media.RadialGradientBrush, new Windows.UI.Xaml.PropertyMetadata.$ctor1(new Windows.Foundation.Point.$ctor1(0.5, 0.5).$clone()));
                    this.RadiusXProperty = Windows.UI.Xaml.DependencyProperty.Register("RadiusX", System.Double, Windows.UI.Xaml.Media.RadialGradientBrush, new Windows.UI.Xaml.PropertyMetadata.$ctor1(Bridge.box(0.5, System.Double, System.Double.format, System.Double.getHashCode)));
                    this.RadiusYProperty = Windows.UI.Xaml.DependencyProperty.Register("RadiusY", System.Double, Windows.UI.Xaml.Media.RadialGradientBrush, new Windows.UI.Xaml.PropertyMetadata.$ctor1(Bridge.box(0.5, System.Double, System.Double.format, System.Double.getHashCode)));
                }
            }
        },
        props: {
            /**
             * Gets or sets the center of the outermost circle of the radial gradient.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Media.RadialGradientBrush
             * @function Center
             * @type Windows.Foundation.Point
             */
            Center: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.Media.RadialGradientBrush.CenterProperty), Windows.Foundation.Point), Windows.Foundation.Point));
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Media.RadialGradientBrush.CenterProperty, value.$clone());
                }
            },
            /**
             * Gets or sets the location of the two-dimensional focal point that defines
             the beginning of the gradient.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Media.RadialGradientBrush
             * @function GradientOrigin
             * @type Windows.Foundation.Point
             */
            GradientOrigin: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.Media.RadialGradientBrush.GradientOriginProperty), Windows.Foundation.Point), Windows.Foundation.Point));
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Media.RadialGradientBrush.GradientOriginProperty, value.$clone());
                }
            },
            /**
             * Gets or sets the horizontal radius of the outermost circle of the radial
             gradient.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Media.RadialGradientBrush
             * @function RadiusX
             * @type number
             */
            RadiusX: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.Media.RadialGradientBrush.RadiusXProperty), System.Double), System.Double));
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Media.RadialGradientBrush.RadiusXProperty, Bridge.box(value, System.Double, System.Double.format, System.Double.getHashCode));
                }
            },
            /**
             * Gets or sets the vertical radius of the outermost circle of a radial gradient.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Media.RadialGradientBrush
             * @function RadiusY
             * @type number
             */
            RadiusY: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.Media.RadialGradientBrush.RadiusYProperty), System.Double), System.Double));
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Media.RadialGradientBrush.RadiusYProperty, Bridge.box(value, System.Double, System.Double.format, System.Double.getHashCode));
                }
            }
        },
        alias: ["ConvertToCSSValues", "Windows$UI$Xaml$ICanConvertToCSSValues$ConvertToCSSValues"],
        ctors: {
            /**
             * Initializes a new instance of the System.Windows.Media.RadialGradientBrush
             class.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Media.RadialGradientBrush
             * @memberof Windows.UI.Xaml.Media.RadialGradientBrush
             * @return  {void}
             */
            ctor: function () {
                this.$initialize();
                Windows.UI.Xaml.Media.GradientBrush.ctor.call(this);
            },
            /**
             * Initializes a new instance of the System.Windows.Media.RadialGradientBrush
             class that has the specified gradient stops.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Media.RadialGradientBrush
             * @memberof Windows.UI.Xaml.Media.RadialGradientBrush
             * @param   {Windows.UI.Xaml.Media.GradientStopCollection}    gradientStopCollection    The gradient stops to set on this brush.
             * @return  {void}
             */
            $ctor2: function (gradientStopCollection) {
                this.$initialize();
                Windows.UI.Xaml.Media.GradientBrush.ctor.call(this);
                this.GradientStops = gradientStopCollection;
            },
            /**
             * Initializes a new instance of the System.Windows.Media.RadialGradientBrush
             class with the specified start and stop colors.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Media.RadialGradientBrush
             * @memberof Windows.UI.Xaml.Media.RadialGradientBrush
             * @param   {Windows.UI.Color}    startColor    Color value at the focus (System.Windows.Media.RadialGradientBrush.GradientOrigin)
             of the radial gradient.
             * @param   {Windows.UI.Color}    endColor      Color value at the outer edge of the radial gradient.
             * @return  {void}
             */
            $ctor1: function (startColor, endColor) {
                var $t;
                this.$initialize();
                Windows.UI.Xaml.Media.GradientBrush.ctor.call(this);
                var gradientStops = new Windows.UI.Xaml.Media.GradientStopCollection();
                gradientStops.add(($t = new Windows.UI.Xaml.Media.GradientStop(), $t.Color = startColor.$clone(), $t.Offset = 0, $t));
                gradientStops.add(($t = new Windows.UI.Xaml.Media.GradientStop(), $t.Color = endColor.$clone(), $t.Offset = 1, $t));
                this.GradientStops = gradientStops;
            }
        },
        methods: {
            GetGradientStopsString: function () {
                var $t;
                var gradientStopsString = "";
                var orderedGradientStops = System.Linq.Enumerable.from(this.GradientStops).orderBy(function (element) {
                        return element.Offset;
                    });
                var isFirst = true;

                $t = Bridge.getEnumerator(orderedGradientStops, Windows.UI.Xaml.Media.GradientStop);
                try {
                    while ($t.moveNext()) {
                        var gradientStop = $t.Current;
                        if (!isFirst) {
                            gradientStopsString = (gradientStopsString || "") + ", ";
                        }
                        gradientStopsString = (gradientStopsString || "") + (((gradientStop.Color.INTERNAL_ToHtmlString(this.Opacity) || "") + System.Double.format(gradientStop.Offset * 100) + "%") || "");
                        isFirst = false;
                    }
                } finally {
                    if (Bridge.is($t, System.IDisposable)) {
                        $t.System$IDisposable$Dispose();
                    }
                }

                return gradientStopsString;
            },
            INTERNAL_ToHtmlString: function (parent) {
                var gradientStopsString = this.GetGradientStopsString();

                var percentageSymbol = "%";
                var multiplicatorForPercentage = 100;
                if (this.MappingMode === Windows.UI.Xaml.Media.BrushMappingMode.Absolute) {
                    percentageSymbol = "px";
                    multiplicatorForPercentage = 1;
                }

                var gradientString = System.String.format("radial-gradient({0}{5} {1}{5} at {2}{5} {3}{5}, {4})", Bridge.box(Bridge.Int.clip32(this.RadiusX * multiplicatorForPercentage), System.Int32), Bridge.box(Bridge.Int.clip32(this.RadiusY * multiplicatorForPercentage), System.Int32), Bridge.box(Bridge.Int.clip32(this.Center.X * multiplicatorForPercentage), System.Int32), Bridge.box(Bridge.Int.clip32(this.Center.Y * multiplicatorForPercentage), System.Int32), gradientStopsString, percentageSymbol);

                if (this.SpreadMethod === Windows.UI.Xaml.Media.GradientSpreadMethod.Repeat) {
                    gradientString = "repeating-" + (gradientString || "");
                }
                var returnValues = new (System.Collections.Generic.List$1(System.Object)).ctor();
                returnValues.add("-webkit-" + (gradientString || ""));
                returnValues.add("-o-" + (gradientString || ""));
                returnValues.add("-moz-" + (gradientString || ""));
                returnValues.add(gradientString);
                return returnValues;
            },
            ConvertToCSSValues: function (parent) {
                return this.INTERNAL_ToHtmlString(parent);
            }
        }
    });

    /**
     * Rotates an object clockwise about a specified point in a two-dimensional
     x-y coordinate system.
     *
     * @public
     * @class Windows.UI.Xaml.Media.RotateTransform
     * @augments Windows.UI.Xaml.Media.Transform
     */
    Bridge.define("Windows.UI.Xaml.Media.RotateTransform", {
        inherits: [Windows.UI.Xaml.Media.Transform],
        statics: {
            fields: {
                /**
                 * Identifies the Angle dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Media.RotateTransform
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                AngleProperty: null
            },
            ctors: {
                init: function () {
                    var $t;
                    this.AngleProperty = Windows.UI.Xaml.DependencyProperty.Register("Angle", System.Double, Windows.UI.Xaml.Media.RotateTransform, ($t = new Windows.UI.Xaml.PropertyMetadata.$ctor1(Bridge.box(0.0, System.Double, System.Double.format, System.Double.getHashCode)), $t.GetCSSEquivalent = function (instance) {
                        var $t1;
                        if (Bridge.cast(instance, Windows.UI.Xaml.Media.RotateTransform).INTERNAL_parent != null) {
                            return ($t1 = new Windows.UI.Xaml.CSSEquivalent(), $t1.DomElement = Bridge.cast(instance, Windows.UI.Xaml.Media.RotateTransform).INTERNAL_parent.INTERNAL_OuterDomElement, $t1.Value = function (inst, value) {
                                return System.String.concat(value, "deg");
                            }, $t1.Name = function (_o1) {
                                    _o1.add("rotateZ");
                                    return _o1;
                                }(new (System.Collections.Generic.List$1(System.String)).ctor()), $t1.UIElement = Bridge.cast(instance, Windows.UI.Xaml.Media.RotateTransform).INTERNAL_parent, $t1.ApplyAlsoWhenThereIsAControlTemplate = true, $t1.OnlyUseVelocity = true, $t1);
                        }
                        return null;
                    }, $t));
                }
            }
        },
        fields: {
            _appliedCssAngle: 0,
            _domElementToWhichTheCssWasApplied: null
        },
        props: {
            /**
             * Gets or sets the angle, in degrees, of clockwise rotation.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Media.RotateTransform
             * @function Angle
             * @type number
             */
            Angle: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.Media.RotateTransform.AngleProperty), System.Double), System.Double));
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Media.RotateTransform.AngleProperty, Bridge.box(value, System.Double, System.Double.format, System.Double.getHashCode));
                }
            }
        },
        methods: {
            ApplyCSSChanges: function (rotateTransform, angle) {
                var anglecssEquivalent = Windows.UI.Xaml.Media.RotateTransform.AngleProperty.GetTypeMetaData(Windows.UI.Xaml.Media.RotateTransform).GetCSSEquivalent(rotateTransform);
                if (anglecssEquivalent != null) {
                    var domElement = anglecssEquivalent.DomElement;
                    if (angle !== this._appliedCssAngle || (this._domElementToWhichTheCssWasApplied != null && !Bridge.referenceEquals(domElement, this._domElementToWhichTheCssWasApplied))) {
                        CSHTML5.Internal.INTERNAL_HtmlDomManager.SetDomElementStylePropertyUsingVelocity(anglecssEquivalent.DomElement, anglecssEquivalent.Name, anglecssEquivalent.Value(rotateTransform, Bridge.box(angle, System.Double, System.Double.format, System.Double.getHashCode)));
                        this._appliedCssAngle = angle;
                        this._domElementToWhichTheCssWasApplied = domElement;
                    }
                }
            },
            INTERNAL_ApplyCSSChanges: function () {
                this.ApplyCSSChanges(this, this.Angle);
            },
            INTERNAL_UnapplyCSSChanges: function () {
                this.ApplyCSSChanges(this, 0);
            },
            ApplyRotateTransform: function (angle) {
                if (this.INTERNAL_parent != null && CSHTML5.Internal.INTERNAL_VisualTreeManager.IsElementInVisualTree(this.INTERNAL_parent)) {
                    this.ApplyCSSChanges(this, angle);



                }
            },
            INTERNAL_ApplyTransform: function () {
                this.ApplyRotateTransform(this.Angle);
            },
            INTERNAL_UnapplyTransform: function () {
                if (this.INTERNAL_parent != null && CSHTML5.Internal.INTERNAL_VisualTreeManager.IsElementInVisualTree(this.INTERNAL_parent)) {
                    this.INTERNAL_UnapplyCSSChanges();
                }
            },
            INTERNAL_TransformPoint: function (point) {
                throw new System.NotImplementedException.$ctor1("Please contact support@cshtml5.com");
            }
        }
    });

    /**
     * Scales an object in the two-dimensional x-y coordinate system.
     *
     * @public
     * @class Windows.UI.Xaml.Media.ScaleTransform
     * @augments Windows.UI.Xaml.Media.Transform
     */
    Bridge.define("Windows.UI.Xaml.Media.ScaleTransform", {
        inherits: [Windows.UI.Xaml.Media.Transform],
        statics: {
            fields: {
                /**
                 * Identifies the ScaleX dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Media.ScaleTransform
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                ScaleXProperty: null,
                /**
                 * Identifies the ScaleY dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Media.ScaleTransform
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                ScaleYProperty: null
            },
            ctors: {
                init: function () {
                    var $t;
                    this.ScaleXProperty = Windows.UI.Xaml.DependencyProperty.Register("ScaleX", System.Double, Windows.UI.Xaml.Media.ScaleTransform, ($t = new Windows.UI.Xaml.PropertyMetadata.$ctor1(Bridge.box(1.0, System.Double, System.Double.format, System.Double.getHashCode)), $t.GetCSSEquivalent = function (instance) {
                        var $t1;
                        if (Bridge.cast(instance, Windows.UI.Xaml.Media.ScaleTransform).INTERNAL_parent != null) {
                            return ($t1 = new Windows.UI.Xaml.CSSEquivalent(), $t1.DomElement = Bridge.cast(instance, Windows.UI.Xaml.Media.ScaleTransform).INTERNAL_parent.INTERNAL_OuterDomElement, $t1.Value = function (inst, value) {
                                return value;
                            }, $t1.Name = function (_o1) {
                                    _o1.add("scaleX");
                                    return _o1;
                                }(new (System.Collections.Generic.List$1(System.String)).ctor()), $t1.UIElement = Bridge.cast(instance, Windows.UI.Xaml.Media.ScaleTransform).INTERNAL_parent, $t1.ApplyAlsoWhenThereIsAControlTemplate = true, $t1.OnlyUseVelocity = true, $t1);
                        }
                        return null;
                    }, $t));
                    this.ScaleYProperty = Windows.UI.Xaml.DependencyProperty.Register("ScaleY", System.Double, Windows.UI.Xaml.Media.ScaleTransform, ($t = new Windows.UI.Xaml.PropertyMetadata.$ctor1(Bridge.box(1.0, System.Double, System.Double.format, System.Double.getHashCode)), $t.GetCSSEquivalent = function (instance) {
                        var $t1;
                        if (Bridge.cast(instance, Windows.UI.Xaml.Media.ScaleTransform).INTERNAL_parent != null) {
                            return ($t1 = new Windows.UI.Xaml.CSSEquivalent(), $t1.DomElement = Bridge.cast(instance, Windows.UI.Xaml.Media.ScaleTransform).INTERNAL_parent.INTERNAL_OuterDomElement, $t1.Value = function (inst, value) {
                                return value;
                            }, $t1.Name = function (_o2) {
                                    _o2.add("scaleY");
                                    return _o2;
                                }(new (System.Collections.Generic.List$1(System.String)).ctor()), $t1.UIElement = Bridge.cast(instance, Windows.UI.Xaml.Media.ScaleTransform).INTERNAL_parent, $t1.ApplyAlsoWhenThereIsAControlTemplate = true, $t1.OnlyUseVelocity = true, $t1);
                        }
                        return null;
                    }, $t));
                }
            }
        },
        fields: {
            _appliedCssScaleX: 0,
            _appliedCssScaleY: 0,
            _domElementToWhichTheCssScaleXWasApplied: null,
            _domElementToWhichTheCssScaleYWasApplied: null
        },
        props: {
            /**
             * Gets or sets the x-axis scale factor.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Media.ScaleTransform
             * @function ScaleX
             * @type number
             */
            ScaleX: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.Media.ScaleTransform.ScaleXProperty), System.Double), System.Double));
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Media.ScaleTransform.ScaleXProperty, Bridge.box(value, System.Double, System.Double.format, System.Double.getHashCode));
                }
            },
            /**
             * Gets or sets the y-axis scale factor.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Media.ScaleTransform
             * @function ScaleY
             * @type number
             */
            ScaleY: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.Media.ScaleTransform.ScaleYProperty), System.Double), System.Double));
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Media.ScaleTransform.ScaleYProperty, Bridge.box(value, System.Double, System.Double.format, System.Double.getHashCode));
                }
            }
        },
        ctors: {
            init: function () {
                this._appliedCssScaleX = 1.0;
                this._appliedCssScaleY = 1.0;
            }
        },
        methods: {
            ApplyCSSChanges: function (scaleTransform, scaleX, scaleY) {
                var scaleXcssEquivalent = Windows.UI.Xaml.Media.ScaleTransform.ScaleXProperty.GetTypeMetaData(Windows.UI.Xaml.Media.ScaleTransform).GetCSSEquivalent(scaleTransform);
                if (scaleXcssEquivalent != null) {
                    var domElementX = scaleXcssEquivalent.DomElement;
                    if (scaleX !== this._appliedCssScaleX || (this._domElementToWhichTheCssScaleXWasApplied != null && !Bridge.referenceEquals(domElementX, this._domElementToWhichTheCssScaleXWasApplied))) {
                        CSHTML5.Internal.INTERNAL_HtmlDomManager.SetDomElementStylePropertyUsingVelocity(scaleXcssEquivalent.DomElement, scaleXcssEquivalent.Name, scaleXcssEquivalent.Value(scaleTransform, Bridge.box(scaleX, System.Double, System.Double.format, System.Double.getHashCode)));
                        this._appliedCssScaleX = scaleX;
                        this._domElementToWhichTheCssScaleXWasApplied = domElementX;
                    }
                }

                var scaleYcssEquivalent = Windows.UI.Xaml.Media.ScaleTransform.ScaleYProperty.GetTypeMetaData(Windows.UI.Xaml.Media.ScaleTransform).GetCSSEquivalent(scaleTransform);
                if (scaleYcssEquivalent != null) {
                    var domElementY = scaleYcssEquivalent.DomElement;
                    if (scaleY !== this._appliedCssScaleY || (this._domElementToWhichTheCssScaleYWasApplied != null && !Bridge.referenceEquals(domElementY, this._domElementToWhichTheCssScaleYWasApplied))) {
                        CSHTML5.Internal.INTERNAL_HtmlDomManager.SetDomElementStylePropertyUsingVelocity(scaleYcssEquivalent.DomElement, scaleYcssEquivalent.Name, scaleYcssEquivalent.Value(scaleTransform, Bridge.box(scaleY, System.Double, System.Double.format, System.Double.getHashCode)));
                        this._appliedCssScaleY = scaleY;
                        this._domElementToWhichTheCssScaleYWasApplied = domElementY;
                    }
                }
            },
            INTERNAL_ApplyCSSChanges: function () {
                this.ApplyCSSChanges(this, this.ScaleX, this.ScaleY);
            },
            INTERNAL_UnapplyCSSChanges: function () {
                this.ApplyCSSChanges(this, 1, 1);
            },
            ApplyScaleTransform: function (scaleX, scaleY) {

                if (this.INTERNAL_parent != null && CSHTML5.Internal.INTERNAL_VisualTreeManager.IsElementInVisualTree(this.INTERNAL_parent)) {
                    this.ApplyCSSChanges(this, scaleX, scaleY);


                }
            },
            INTERNAL_ApplyTransform: function () {
                this.ApplyScaleTransform(this.ScaleX, this.ScaleY);
            },
            INTERNAL_UnapplyTransform: function () {
                if (this.INTERNAL_parent != null && CSHTML5.Internal.INTERNAL_VisualTreeManager.IsElementInVisualTree(this.INTERNAL_parent)) {
                    this.INTERNAL_UnapplyCSSChanges();
                }
            },
            INTERNAL_TransformPoint: function (point) {
                throw new System.NotImplementedException.$ctor1("Please contact support@cshtml5.com");
            }
        }
    });

    /**
     * Represents a two-dimensional skew.
     *
     * @public
     * @class Windows.UI.Xaml.Media.SkewTransform
     * @augments Windows.UI.Xaml.Media.Transform
     */
    Bridge.define("Windows.UI.Xaml.Media.SkewTransform", {
        inherits: [Windows.UI.Xaml.Media.Transform],
        statics: {
            fields: {
                /**
                 * Identifies the AngleX dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Media.SkewTransform
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                AngleXProperty: null,
                /**
                 * Identifies the AngleY dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Media.SkewTransform
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                AngleYProperty: null
            },
            ctors: {
                init: function () {
                    var $t;
                    this.AngleXProperty = Windows.UI.Xaml.DependencyProperty.Register("AngleX", System.Double, Windows.UI.Xaml.Media.SkewTransform, ($t = new Windows.UI.Xaml.PropertyMetadata.$ctor1(Bridge.box(0.0, System.Double, System.Double.format, System.Double.getHashCode)), $t.GetCSSEquivalent = function (instance) {
                        var $t1;
                        if (Bridge.cast(instance, Windows.UI.Xaml.Media.SkewTransform).INTERNAL_parent != null) {
                            return ($t1 = new Windows.UI.Xaml.CSSEquivalent(), $t1.DomElement = Bridge.cast(instance, Windows.UI.Xaml.Media.SkewTransform).INTERNAL_parent.INTERNAL_OuterDomElement, $t1.Value = function (inst, value) {
                                return System.String.concat(value, "deg");
                            }, $t1.Name = function (_o1) {
                                    _o1.add("skewX");
                                    return _o1;
                                }(new (System.Collections.Generic.List$1(System.String)).ctor()), $t1.UIElement = Bridge.cast(instance, Windows.UI.Xaml.Media.SkewTransform).INTERNAL_parent, $t1.ApplyAlsoWhenThereIsAControlTemplate = true, $t1.OnlyUseVelocity = true, $t1);
                        }
                        return null;
                    }, $t));
                    this.AngleYProperty = Windows.UI.Xaml.DependencyProperty.Register("AngleY", System.Double, Windows.UI.Xaml.Media.SkewTransform, ($t = new Windows.UI.Xaml.PropertyMetadata.$ctor1(Bridge.box(0.0, System.Double, System.Double.format, System.Double.getHashCode)), $t.GetCSSEquivalent = function (instance) {
                        var $t1;
                        if (Bridge.cast(instance, Windows.UI.Xaml.Media.SkewTransform).INTERNAL_parent != null) {
                            return ($t1 = new Windows.UI.Xaml.CSSEquivalent(), $t1.DomElement = Bridge.cast(instance, Windows.UI.Xaml.Media.SkewTransform).INTERNAL_parent.INTERNAL_OuterDomElement, $t1.Value = function (inst, value) {
                                return System.String.concat(value, "deg");
                            }, $t1.Name = function (_o2) {
                                    _o2.add("skewY");
                                    return _o2;
                                }(new (System.Collections.Generic.List$1(System.String)).ctor()), $t1.UIElement = Bridge.cast(instance, Windows.UI.Xaml.Media.SkewTransform).INTERNAL_parent, $t1.ApplyAlsoWhenThereIsAControlTemplate = true, $t1.OnlyUseVelocity = true, $t1);
                        }
                        return null;
                    }, $t));
                }
            }
        },
        fields: {
            _appliedCssAngleX: 0,
            _appliedCssAngleY: 0,
            _domElementToWhichTheCssAngleXWasApplied: null,
            _domElementToWhichTheCssAngleYWasApplied: null
        },
        props: {
            /**
             * Gets or sets the x-axis skew angle, which is measured in degrees counterclockwise
             from the y-axis.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Media.SkewTransform
             * @function AngleX
             * @type number
             */
            AngleX: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.Media.SkewTransform.AngleXProperty), System.Double), System.Double));
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Media.SkewTransform.AngleXProperty, Bridge.box(value, System.Double, System.Double.format, System.Double.getHashCode));
                }
            },
            /**
             * Gets or sets the y-axis skew angle, which is measured in degrees counterclockwise
             from the x-axis.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Media.SkewTransform
             * @function AngleY
             * @type number
             */
            AngleY: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.Media.SkewTransform.AngleYProperty), System.Double), System.Double));
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Media.SkewTransform.AngleYProperty, Bridge.box(value, System.Double, System.Double.format, System.Double.getHashCode));
                }
            }
        },
        methods: {
            ApplyCSSChanges: function (skewTransform, angleX, angleY) {
                var angleXcssEquivalent = Windows.UI.Xaml.Media.SkewTransform.AngleXProperty.GetTypeMetaData(Windows.UI.Xaml.Media.SkewTransform).GetCSSEquivalent(skewTransform);
                if (angleXcssEquivalent != null) {
                    var domElementX = angleXcssEquivalent.DomElement;
                    if (angleX !== this._appliedCssAngleX || (this._domElementToWhichTheCssAngleXWasApplied != null && !Bridge.referenceEquals(domElementX, this._domElementToWhichTheCssAngleXWasApplied))) {
                        CSHTML5.Internal.INTERNAL_HtmlDomManager.SetDomElementStylePropertyUsingVelocity(angleXcssEquivalent.DomElement, angleXcssEquivalent.Name, angleXcssEquivalent.Value(skewTransform, Bridge.box(angleX, System.Double, System.Double.format, System.Double.getHashCode)));
                        this._appliedCssAngleX = angleX;
                        this._domElementToWhichTheCssAngleXWasApplied = domElementX;
                    }
                }

                var angleYcssEquivalent = Windows.UI.Xaml.Media.SkewTransform.AngleYProperty.GetTypeMetaData(Windows.UI.Xaml.Media.SkewTransform).GetCSSEquivalent(skewTransform);
                if (angleYcssEquivalent != null) {
                    var domElementY = angleYcssEquivalent.DomElement;
                    if (angleY !== this._appliedCssAngleY || (this._domElementToWhichTheCssAngleYWasApplied != null && !Bridge.referenceEquals(domElementY, this._domElementToWhichTheCssAngleYWasApplied))) {
                        CSHTML5.Internal.INTERNAL_HtmlDomManager.SetDomElementStylePropertyUsingVelocity(angleYcssEquivalent.DomElement, angleYcssEquivalent.Name, angleYcssEquivalent.Value(skewTransform, Bridge.box(angleY, System.Double, System.Double.format, System.Double.getHashCode)));
                        this._appliedCssAngleY = angleY;
                        this._domElementToWhichTheCssAngleYWasApplied = domElementY;
                    }
                }
            },
            INTERNAL_ApplyCSSChanges: function () {
                this.ApplyCSSChanges(this, this.AngleX, this.AngleY);
            },
            INTERNAL_UnapplyCSSChanges: function () {
                this.ApplyCSSChanges(this, 0, 0);
            },
            ApplySkewTransform: function (angleX, angleY) {
                if (this.INTERNAL_parent != null && CSHTML5.Internal.INTERNAL_VisualTreeManager.IsElementInVisualTree(this.INTERNAL_parent)) {
                    this.ApplyCSSChanges(this, angleX, angleY);


                }
            },
            INTERNAL_ApplyTransform: function () {
                this.ApplySkewTransform(this.AngleX, this.AngleY);
            },
            INTERNAL_UnapplyTransform: function () {
                if (this.INTERNAL_parent != null && CSHTML5.Internal.INTERNAL_VisualTreeManager.IsElementInVisualTree(this.INTERNAL_parent)) {
                    this.INTERNAL_UnapplyCSSChanges();
                }
            },
            INTERNAL_TransformPoint: function (point) {
                throw new System.NotImplementedException.$ctor1("Please contact support@cshtml5.com");
            }
        }
    });

    Bridge.define("Windows.UI.Xaml.Media.TransformCollection", {
        inherits: [System.Collections.Generic.List$1(Windows.UI.Xaml.Media.Transform)]
    });

    Bridge.define("Windows.UI.Xaml.Media.TransformGroup", {
        inherits: [Windows.UI.Xaml.Media.Transform],
        statics: {
            fields: {
                ChildrenProperty: null
            },
            ctors: {
                init: function () {
                    this.ChildrenProperty = Windows.UI.Xaml.DependencyProperty.Register("Children", Windows.UI.Xaml.Media.TransformCollection, Windows.UI.Xaml.Media.TransformGroup, new Windows.UI.Xaml.PropertyMetadata.$ctor4(null));
                }
            }
        },
        props: {
            Children: {
                get: function () {
                    var collection = Bridge.cast(this.GetValue(Windows.UI.Xaml.Media.TransformGroup.ChildrenProperty), Windows.UI.Xaml.Media.TransformCollection);
                    if (collection == null) {
                        collection = new Windows.UI.Xaml.Media.TransformCollection();
                        this.SetValue(Windows.UI.Xaml.Media.TransformGroup.ChildrenProperty, collection);
                    }
                    return collection;
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Media.TransformGroup.ChildrenProperty, value);
                }
            }
        },
        ctors: {
            ctor: function () {
                this.$initialize();
                Windows.UI.Xaml.Media.Transform.ctor.call(this);

            }
        },
        methods: {
            INTERNAL_TransformPoint: function (point) {
                return new Windows.Foundation.Point.ctor();
            },
            ApplyCSSChanges: function (transformGroup, children) {
                var childrenCount = (children != null) ? children.Count : 0;
                var parent = transformGroup.INTERNAL_parent;
                var child;
                for (var i = (childrenCount - 1) | 0; i > -1; i = (i - 1) | 0) {
                    child = children.getItem(i);
                    if (!Bridge.referenceEquals(child.INTERNAL_parent, parent)) {
                        child.INTERNAL_parent = parent;
                    }
                    child.INTERNAL_ApplyCSSChanges();
                }
            },
            UnapplyCSSChanges: function (transformGroup, children) {
                var childrenCount = (children != null) ? children.Count : 0;
                var parent = transformGroup.INTERNAL_parent;
                var child;
                for (var i = (childrenCount - 1) | 0; i > -1; i = (i - 1) | 0) {
                    child = children.getItem(i);
                    if (!Bridge.referenceEquals(child.INTERNAL_parent, parent)) {
                        child.INTERNAL_parent = parent;
                    }
                    child.INTERNAL_UnapplyCSSChanges();
                }
            },
            INTERNAL_ApplyCSSChanges: function () {
                this.ApplyCSSChanges(this, this.Children);
            },
            INTERNAL_UnapplyCSSChanges: function () {
                this.UnapplyCSSChanges(this, this.Children);
            },
            ApplyTransformGroup: function (children) {
                if (this.INTERNAL_parent != null && CSHTML5.Internal.INTERNAL_VisualTreeManager.IsElementInVisualTree(this.INTERNAL_parent)) {
                    this.ApplyCSSChanges(this, children);
                }
            },
            UnapplyTransformGroup: function (children) {
                if (this.INTERNAL_parent != null && CSHTML5.Internal.INTERNAL_VisualTreeManager.IsElementInVisualTree(this.INTERNAL_parent)) {
                    this.UnapplyCSSChanges(this, children);
                }
            },
            INTERNAL_ApplyTransform: function () {
                this.ApplyTransformGroup(this.Children);
            },
            INTERNAL_UnapplyTransform: function () {
                this.UnapplyTransformGroup(this.Children);
            }
        }
    });

    /**
     * Translates (moves) an object in the two-dimensional x-y coordinate system.
     *
     * @public
     * @class Windows.UI.Xaml.Media.TranslateTransform
     * @augments Windows.UI.Xaml.Media.Transform
     */
    Bridge.define("Windows.UI.Xaml.Media.TranslateTransform", {
        inherits: [Windows.UI.Xaml.Media.Transform],
        statics: {
            fields: {
                /**
                 * Identifies the X dependency property
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Media.TranslateTransform
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                XProperty: null,
                /**
                 * Identifies the Y dependency property
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Media.TranslateTransform
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                YProperty: null
            },
            ctors: {
                init: function () {
                    var $t;
                    this.XProperty = Windows.UI.Xaml.DependencyProperty.Register("X", System.Double, Windows.UI.Xaml.Media.TranslateTransform, ($t = new Windows.UI.Xaml.PropertyMetadata.$ctor1(Bridge.box(0.0, System.Double, System.Double.format, System.Double.getHashCode)), $t.GetCSSEquivalent = function (instance) {
                        var $t1;
                        if (Bridge.cast(instance, Windows.UI.Xaml.Media.TranslateTransform).INTERNAL_parent != null) {
                            return ($t1 = new Windows.UI.Xaml.CSSEquivalent(), $t1.DomElement = Bridge.cast(instance, Windows.UI.Xaml.Media.TranslateTransform).INTERNAL_parent.INTERNAL_OuterDomElement, $t1.Value = function (inst, value) {
                                return System.String.concat(value, "px");
                            }, $t1.Name = function (_o1) {
                                    _o1.add("translateX");
                                    return _o1;
                                }(new (System.Collections.Generic.List$1(System.String)).ctor()), $t1.UIElement = Bridge.cast(instance, Windows.UI.Xaml.Media.TranslateTransform).INTERNAL_parent, $t1.ApplyAlsoWhenThereIsAControlTemplate = true, $t1.OnlyUseVelocity = true, $t1);
                        }
                        return null;
                    }, $t));
                    this.YProperty = Windows.UI.Xaml.DependencyProperty.Register("Y", System.Double, Windows.UI.Xaml.Media.TranslateTransform, ($t = new Windows.UI.Xaml.PropertyMetadata.$ctor1(Bridge.box(0.0, System.Double, System.Double.format, System.Double.getHashCode)), $t.GetCSSEquivalent = function (instance) {
                        var $t1;
                        if (Bridge.cast(instance, Windows.UI.Xaml.Media.TranslateTransform).INTERNAL_parent != null) {
                            return ($t1 = new Windows.UI.Xaml.CSSEquivalent(), $t1.DomElement = Bridge.cast(instance, Windows.UI.Xaml.Media.TranslateTransform).INTERNAL_parent.INTERNAL_OuterDomElement, $t1.Value = function (inst, value) {
                                return System.String.concat(value, "px");
                            }, $t1.Name = function (_o2) {
                                    _o2.add("translateY");
                                    return _o2;
                                }(new (System.Collections.Generic.List$1(System.String)).ctor()), $t1.UIElement = Bridge.cast(instance, Windows.UI.Xaml.Media.TranslateTransform).INTERNAL_parent, $t1.ApplyAlsoWhenThereIsAControlTemplate = true, $t1.OnlyUseVelocity = true, $t1);
                        }
                        return null;
                    }, $t));
                }
            },
            methods: {
                X_Changed: function (d, e) {
                    var translateTransform = Bridge.cast(d, Windows.UI.Xaml.Media.TranslateTransform);
                    var newX = System.Nullable.getValue(Bridge.cast(Bridge.unbox(e.NewValue, System.Double), System.Double));
                    translateTransform.ApplyTranslateTransform(newX, translateTransform.Y);
                }
            }
        },
        fields: {
            _appliedCssX: 0,
            _appliedCssY: 0,
            _domElementToWhichTheCssXWasApplied: null,
            _domElementToWhichTheCssYWasApplied: null
        },
        props: {
            /**
             * Gets or sets the distance to translate along the x-axis.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Media.TranslateTransform
             * @function X
             * @type number
             */
            X: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.Media.TranslateTransform.XProperty), System.Double), System.Double));
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Media.TranslateTransform.XProperty, Bridge.box(value, System.Double, System.Double.format, System.Double.getHashCode));
                }
            },
            /**
             * Gets or sets the distance to translate (move) an object along the y-axis.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Media.TranslateTransform
             * @function Y
             * @type number
             */
            Y: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.Media.TranslateTransform.YProperty), System.Double), System.Double));
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Media.TranslateTransform.YProperty, Bridge.box(value, System.Double, System.Double.format, System.Double.getHashCode));
                }
            }
        },
        methods: {
            ApplyCSSChanges: function (translateTransform, x, y) {
                var translateXcssEquivalent = Windows.UI.Xaml.Media.TranslateTransform.XProperty.GetTypeMetaData(Windows.UI.Xaml.Media.TranslateTransform).GetCSSEquivalent(translateTransform);
                if (translateXcssEquivalent != null) {
                    var domElementX = translateXcssEquivalent.DomElement;
                    if (x !== this._appliedCssX || (this._domElementToWhichTheCssXWasApplied != null && !Bridge.referenceEquals(domElementX, this._domElementToWhichTheCssXWasApplied))) {
                        CSHTML5.Internal.INTERNAL_HtmlDomManager.SetDomElementStylePropertyUsingVelocity(domElementX, translateXcssEquivalent.Name, translateXcssEquivalent.Value(translateTransform, Bridge.box(x, System.Double, System.Double.format, System.Double.getHashCode)));
                        this._appliedCssX = x;
                        this._domElementToWhichTheCssXWasApplied = domElementX;
                    }
                }

                var translateYcssEquivalent = Windows.UI.Xaml.Media.TranslateTransform.YProperty.GetTypeMetaData(Windows.UI.Xaml.Media.TranslateTransform).GetCSSEquivalent(translateTransform);
                if (translateYcssEquivalent != null) {
                    var domElementY = translateYcssEquivalent.DomElement;
                    if (y !== this._appliedCssY || (this._domElementToWhichTheCssYWasApplied != null && !Bridge.referenceEquals(domElementY, this._domElementToWhichTheCssYWasApplied))) {
                        CSHTML5.Internal.INTERNAL_HtmlDomManager.SetDomElementStylePropertyUsingVelocity(translateYcssEquivalent.DomElement, translateYcssEquivalent.Name, translateYcssEquivalent.Value(translateTransform, Bridge.box(y, System.Double, System.Double.format, System.Double.getHashCode)));
                        this._appliedCssY = x;
                        this._domElementToWhichTheCssYWasApplied = domElementY;
                    }
                }
            },
            INTERNAL_ApplyCSSChanges: function () {
                this.ApplyCSSChanges(this, this.X, this.Y);
            },
            INTERNAL_UnapplyCSSChanges: function () {
                this.ApplyCSSChanges(this, 0, 0);
            },
            ApplyTranslateTransform: function (x, y) {
                if (this.INTERNAL_parent != null && CSHTML5.Internal.INTERNAL_VisualTreeManager.IsElementInVisualTree(this.INTERNAL_parent)) {
                    this.ApplyCSSChanges(this, x, y);



                }
            },
            INTERNAL_ApplyTransform: function () {
                this.ApplyTranslateTransform(this.X, this.Y);
            },
            INTERNAL_UnapplyTransform: function () {
                if (this.INTERNAL_parent != null && CSHTML5.Internal.INTERNAL_VisualTreeManager.IsElementInVisualTree(this.INTERNAL_parent)) {
                    this.INTERNAL_UnapplyCSSChanges();
                }
            },
            INTERNAL_TransformPoint: function (point) {
                return new Windows.Foundation.Point.$ctor1(point.X + this.X, point.Y + this.Y);
            }
        }
    });

    /**
     * Provides a base class for shape elements, such as Ellipse, Polygon, and Rectangle.
     *
     * @abstract
     * @public
     * @class Windows.UI.Xaml.Shapes.Shape
     * @augments Windows.UI.Xaml.FrameworkElement
     */
    Bridge.define("Windows.UI.Xaml.Shapes.Shape", {
        inherits: [Windows.UI.Xaml.FrameworkElement],
        statics: {
            fields: {
                /**
                 * Identifies the Fill dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Shapes.Shape
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                FillProperty: null,
                /**
                 * Identifies the Stretch dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Shapes.Shape
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                StretchProperty: null,
                /**
                 * Identifies the Stroke dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Shapes.Shape
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                StrokeProperty: null,
                /**
                 * Identifies the StrokeThickness dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Shapes.Shape
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                StrokeThicknessProperty: null,
                /**
                 * Identifies the StrokeStartLineCap dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Shapes.Shape
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                StrokeStartLineCapProperty: null,
                /**
                 * Identifies the StrokeEndLineCap dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Shapes.Shape
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                StrokeEndLineCapProperty: null,
                /**
                 * Identifies the StrokeLineJoin dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Shapes.Shape
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                StrokeLineJoinProperty: null,
                /**
                 * Identifies the StrokeMiterLimit dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Shapes.Shape
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                StrokeMiterLimitProperty: null,
                /**
                 * Identifies the StrokeDashArray dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Shapes.Shape
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                StrokeDashArrayProperty: null,
                /**
                 * Identifies the StrokeDashOffset dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Shapes.Shape
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                StrokeDashOffsetProperty: null
            },
            ctors: {
                init: function () {
                    this.FillProperty = Windows.UI.Xaml.DependencyProperty.Register("Fill", Windows.UI.Xaml.Media.Brush, Windows.UI.Xaml.Shapes.Shape, new Windows.UI.Xaml.PropertyMetadata.$ctor2(null, Windows.UI.Xaml.Shapes.Shape.Fill_Changed));
                    this.StretchProperty = Windows.UI.Xaml.DependencyProperty.Register("Stretch", Windows.UI.Xaml.Media.Stretch, Windows.UI.Xaml.Shapes.Shape, new Windows.UI.Xaml.PropertyMetadata.$ctor2(Bridge.box(Windows.UI.Xaml.Media.Stretch.None, Windows.UI.Xaml.Media.Stretch, System.Enum.toStringFn(Windows.UI.Xaml.Media.Stretch)), Windows.UI.Xaml.Shapes.Shape.Stretch_Changed));
                    this.StrokeProperty = Windows.UI.Xaml.DependencyProperty.Register("Stroke", Windows.UI.Xaml.Media.Brush, Windows.UI.Xaml.Shapes.Shape, new Windows.UI.Xaml.PropertyMetadata.$ctor2(null, Windows.UI.Xaml.Shapes.Shape.Stroke_Changed));
                    this.StrokeThicknessProperty = Windows.UI.Xaml.DependencyProperty.Register("StrokeThickness", System.Double, Windows.UI.Xaml.Shapes.Shape, new Windows.UI.Xaml.PropertyMetadata.$ctor2(Bridge.box(1.0, System.Double, System.Double.format, System.Double.getHashCode), Windows.UI.Xaml.Shapes.Shape.StrokeThickness_Changed));
                    this.StrokeStartLineCapProperty = Windows.UI.Xaml.DependencyProperty.Register("StrokeStartLineCap", Windows.UI.Xaml.Media.PenLineCap, Windows.UI.Xaml.Shapes.Shape, new Windows.UI.Xaml.PropertyMetadata.$ctor1(Bridge.box(Windows.UI.Xaml.Media.PenLineCap.Flat, Windows.UI.Xaml.Media.PenLineCap, System.Enum.toStringFn(Windows.UI.Xaml.Media.PenLineCap))));
                    this.StrokeEndLineCapProperty = Windows.UI.Xaml.DependencyProperty.Register("StrokeEndLineCap", Windows.UI.Xaml.Media.PenLineCap, Windows.UI.Xaml.Shapes.Shape, new Windows.UI.Xaml.PropertyMetadata.$ctor1(Bridge.box(Windows.UI.Xaml.Media.PenLineCap.Flat, Windows.UI.Xaml.Media.PenLineCap, System.Enum.toStringFn(Windows.UI.Xaml.Media.PenLineCap))));
                    this.StrokeLineJoinProperty = Windows.UI.Xaml.DependencyProperty.Register("StrokeLineJoin", Windows.UI.Xaml.Media.PenLineJoin, Windows.UI.Xaml.Shapes.Shape, new Windows.UI.Xaml.PropertyMetadata.$ctor1(Bridge.box(Windows.UI.Xaml.Media.PenLineJoin.Miter, Windows.UI.Xaml.Media.PenLineJoin, System.Enum.toStringFn(Windows.UI.Xaml.Media.PenLineJoin))));
                    this.StrokeMiterLimitProperty = Windows.UI.Xaml.DependencyProperty.Register("StrokeMiterLimit", System.Double, Windows.UI.Xaml.Shapes.Shape, new Windows.UI.Xaml.PropertyMetadata.$ctor1(Bridge.box(0.0, System.Double, System.Double.format, System.Double.getHashCode)));
                    this.StrokeDashArrayProperty = Windows.UI.Xaml.DependencyProperty.Register("StrokeDashArray", Windows.UI.Xaml.Media.DoubleCollection, Windows.UI.Xaml.Shapes.Shape, new Windows.UI.Xaml.PropertyMetadata.$ctor2(null, Windows.UI.Xaml.Shapes.Shape.StrokeDashArray_Changed));
                    this.StrokeDashOffsetProperty = Windows.UI.Xaml.DependencyProperty.Register("StrokeDashOffset", System.Double, Windows.UI.Xaml.Shapes.Shape, new Windows.UI.Xaml.PropertyMetadata.$ctor2(Bridge.box(0.0, System.Double, System.Double.format, System.Double.getHashCode), Windows.UI.Xaml.Shapes.Shape.StrokeDashOffset_Changed));
                }
            },
            methods: {
                Fill_Changed: function (d, e) {
                    var shape = Bridge.cast(d, Windows.UI.Xaml.Shapes.Shape);
                    shape.ScheduleRedraw();
                },
                Stretch_Changed: function (i, e) {

                    var shape = Bridge.cast(i, Windows.UI.Xaml.Shapes.Shape);
                    if (CSHTML5.Internal.INTERNAL_VisualTreeManager.IsElementInVisualTree(shape)) {
                        var shapeDom = CSHTML5.Internal.INTERNAL_HtmlDomManager.GetFrameworkElementOuterStyleForModification(shape);

                        var newValue = System.Nullable.getValue(Bridge.cast(Bridge.unbox(e.NewValue, Windows.UI.Xaml.Media.Stretch), System.Int32));
                        if (isNaN(shape.Width)) {
                            switch (newValue) {
                                case Windows.UI.Xaml.Media.Stretch.None: 
                                    shapeDom.width = "auto";
                                    break;
                                case Windows.UI.Xaml.Media.Stretch.Fill: 
                                    shapeDom.width = "100%";
                                    break;
                                case Windows.UI.Xaml.Media.Stretch.Uniform: 
                                    shapeDom.maxWidth = "100%";
                                    break;
                                case Windows.UI.Xaml.Media.Stretch.UniformToFill: 
                                    shapeDom.minWidth = "100%";
                                    break;
                                default: 
                                    break;
                            }
                        }
                        if (isNaN(shape.Height)) {
                            switch (newValue) {
                                case Windows.UI.Xaml.Media.Stretch.None: 
                                    shapeDom.height = "auto";
                                    break;
                                case Windows.UI.Xaml.Media.Stretch.Fill: 
                                    shapeDom.height = "100%";
                                    break;
                                case Windows.UI.Xaml.Media.Stretch.Uniform: 
                                    shapeDom.maxHeight = "100%";
                                    break;
                                case Windows.UI.Xaml.Media.Stretch.UniformToFill: 
                                    shapeDom.minHeight = "100%";
                                    break;
                                default: 
                                    break;
                            }
                        }

                        shape.ScheduleRedraw();
                    }
                },
                Stroke_Changed: function (d, e) {
                    Bridge.cast(d, Windows.UI.Xaml.Shapes.Shape).ManageStrokeChanged();
                },
                StrokeThickness_Changed: function (d, e) {
                    Bridge.cast(d, Windows.UI.Xaml.Shapes.Shape).ManageStrokeThicknessChanged();
                },
                StrokeDashArray_Changed: function (d, e) {
                    var shape = Bridge.cast(d, Windows.UI.Xaml.Shapes.Shape);
                    if (CSHTML5.Internal.INTERNAL_VisualTreeManager.IsElementInVisualTree(shape)) {
                        shape.ScheduleRedraw();
                    }
                },
                StrokeDashOffset_Changed: function (d, e) {
                    var shape = Bridge.cast(d, Windows.UI.Xaml.Shapes.Shape);
                    if (CSHTML5.Internal.INTERNAL_VisualTreeManager.IsElementInVisualTree(shape)) {
                        if (shape._canvasDomElement != null) {
                            var context = CSHTML5.Internal.INTERNAL_HtmlDomManager.Get2dCanvasContext(shape._canvasDomElement);
                            context.lineDashOffset = System.Double.format(shape.StrokeDashOffset);
                        }
                    }
                },
                GetShapeInfos: function (shape, xOffsetToApplyBeforeMultiplication, yOffsetToApplyBeforeMultiplication, xOffsetToApplyAfterMultiplication, yOffsetToApplyAfterMultiplication, sizeX, sizeY, horizontalMultiplicator, verticalMultiplicator, shapeActualSize) {
                    var width = shape.Width;
                    var height = shape.Height;
                    var strokeThickness = shape.StrokeThickness;
                    var strech = shape.Stretch;

                    var minX = 0;
                    var maxX = isNaN(width) ? 0 : width;
                    var minY = 0;
                    var maxY = isNaN(height) ? 0 : height;

                    CSHTML5.Internal.INTERNAL_ShapesDrawHelpers.PrepareStretch(shape, shape._canvasDomElement, minX, maxX, minY, maxY, strech, shapeActualSize);

                    if (shape.Stroke == null) {
                        strokeThickness = 0;
                    }

                    CSHTML5.Internal.INTERNAL_ShapesDrawHelpers.GetMultiplicatorsAndOffsetForStretch(shape, strokeThickness, minX, maxX, minY, maxY, strech, shapeActualSize.v.$clone(), horizontalMultiplicator, verticalMultiplicator, xOffsetToApplyBeforeMultiplication, yOffsetToApplyBeforeMultiplication, xOffsetToApplyAfterMultiplication, yOffsetToApplyAfterMultiplication, Bridge.ref(shape, "_marginOffsets"));

                    if (strech === Windows.UI.Xaml.Media.Stretch.None) {
                        var margin = shape.Margin.$clone();
                        Windows.UI.Xaml.FrameworkElement.Margin_MethodToUpdateDom(shape, new Windows.UI.Xaml.Thickness.$ctor2(margin.Left + shape._marginOffsets.X, margin.Top + shape._marginOffsets.Y, margin.Right, margin.Bottom).$clone());
                    }
                    sizeX.v = Math.max(1, maxX - minX) * horizontalMultiplicator.v;
                    sizeY.v = Math.max(1, maxY - minY) * verticalMultiplicator.v;
                },
                /**
                 * This method calls the fill and stroke methods while isolating the possible transformations that the GetHtmlBrush does.
                 *
                 * @static
                 * @this Windows.UI.Xaml.Shapes.Shape
                 * @memberof Windows.UI.Xaml.Shapes.Shape
                 * @param   {Windows.UI.Xaml.Shapes.Shape}    shape                                 
                 * @param   {string}                          fillRule                              
                 * @param   {number}                          minX                                  
                 * @param   {number}                          minY                                  
                 * @param   {number}                          maxX                                  
                 * @param   {number}                          maxY                                  
                 * @param   {number}                          horizontalMultiplicator               
                 * @param   {number}                          verticalMultiplicator                 
                 * @param   {number}                          xOffsetToApplyBeforeMultiplication    
                 * @param   {number}                          yOffsetToApplyBeforeMultiplication    
                 * @param   {Windows.Foundation.Size}         shapeActualSize
                 * @return  {void}
                 */
                DrawFillAndStroke: function (shape, fillRule, minX, minY, maxX, maxY, horizontalMultiplicator, verticalMultiplicator, xOffsetToApplyBeforeMultiplication, yOffsetToApplyBeforeMultiplication, shapeActualSize) {
                    var context = shape._canvasDomElement.getContext('2d');

                    context.clearRect(0, 0, Bridge.box(shapeActualSize.Width, System.Double, System.Double.format, System.Double.getHashCode), Bridge.box(shapeActualSize.Height, System.Double, System.Double.format, System.Double.getHashCode));



                    context.save();

                    var opacity = shape.Fill == null ? 1 : shape.Fill.Opacity;
                    var fillValue = Windows.UI.Xaml.Shapes.Shape.GetHtmlBrush(shape, shape.Fill, opacity, minX, minY, maxX, maxY, horizontalMultiplicator, verticalMultiplicator, xOffsetToApplyBeforeMultiplication, yOffsetToApplyBeforeMultiplication, shapeActualSize.$clone());

                    if (fillValue != null) {
                        context.fillStyle = fillValue;
                    } else {
                        context.fillStyle = '';
                    }

                    if (shape.Fill != null) {
                        context.fill(fillRule);
                    }

                    context.restore(); context.save();

                    var strokeValue = null;
                    opacity = shape.Stroke == null ? 1 : shape.Stroke.Opacity;
                    strokeValue = Windows.UI.Xaml.Shapes.Shape.GetHtmlBrush(shape, shape.Stroke, opacity, minX, minY, maxX, maxY, horizontalMultiplicator, verticalMultiplicator, xOffsetToApplyBeforeMultiplication, yOffsetToApplyBeforeMultiplication, shapeActualSize.$clone());

                    var strokeThickness = shape.StrokeThickness;
                    if (shape.Stroke == null) {
                        strokeThickness = 0;
                    }

                    if (strokeValue != null && strokeThickness > 0) {
                        var thickness = shape.StrokeThickness;
                        context.strokeStyle = strokeValue;
                        context.lineWidth = Bridge.box(strokeThickness, System.Double, System.Double.format, System.Double.getHashCode);
                        var strokeDashArray = shape.StrokeDashArray;
                        if (strokeDashArray != null) {
                            if (CSHTML5.Interop.IsRunningInTheSimulator) {
                            } else {
                                var options = new Array();
                                for (var i = 0; i < strokeDashArray.Count; i = (i + 1) | 0) {
                                    options[Bridge.box(i, System.Int32)] = Bridge.box(strokeDashArray.getItem(i) * thickness, System.Double, System.Double.format, System.Double.getHashCode);
                                }

if (                                context.setLineDash) context.setLineDash(options);
                            }
                        }

                        if (shape.Stroke != null && shape.StrokeThickness > 0) {
                            context.stroke();
                        }

                        context.restore();
                    }

                    context.lineWidth = System.Double.format(shape.StrokeThickness) + "";
                },
                GetHtmlBrush: function (shape, brush, opacity, minX, minY, maxX, maxY, horizontalMultiplicator, verticalMultiplicator, xOffsetToApplyBeforeMultiplication, yOffsetToApplyBeforeMultiplication, shapeActualSize) {
                    var $t, $t1, $t2, $t3;
                    var context = shape._canvasDomElement.getContext('2d');

                    var returnValue = null;
                    if (brush == null || Bridge.is(brush, Windows.UI.Xaml.Media.SolidColorBrush)) {
                        if (brush != null) {
                            returnValue = Bridge.cast(brush, Windows.UI.Xaml.Media.SolidColorBrush).INTERNAL_ToHtmlString();
                        }
                    } else if (Bridge.is(brush, Windows.UI.Xaml.Media.LinearGradientBrush)) {
                        var fillAsLinearGradientBrush = Bridge.cast(brush, Windows.UI.Xaml.Media.LinearGradientBrush);

                        var x0 = fillAsLinearGradientBrush.StartPoint.X;
                        var x1 = fillAsLinearGradientBrush.EndPoint.X;
                        var y0 = fillAsLinearGradientBrush.StartPoint.Y;
                        var y1 = fillAsLinearGradientBrush.EndPoint.Y;

                        if (fillAsLinearGradientBrush.MappingMode === Windows.UI.Xaml.Media.BrushMappingMode.RelativeToBoundingBox) {
                            x0 = fillAsLinearGradientBrush.StartPoint.X * (xOffsetToApplyBeforeMultiplication + maxX - minX) * horizontalMultiplicator + minX;
                            x1 = fillAsLinearGradientBrush.EndPoint.X * (xOffsetToApplyBeforeMultiplication + maxX - minX) * horizontalMultiplicator + minX;
                            y0 = fillAsLinearGradientBrush.StartPoint.Y * (yOffsetToApplyBeforeMultiplication + maxY - minY) * verticalMultiplicator + minY;
                            y1 = fillAsLinearGradientBrush.EndPoint.Y * (yOffsetToApplyBeforeMultiplication + maxY - minY) * verticalMultiplicator + minY;
                        }

                        if (fillAsLinearGradientBrush.SpreadMethod === Windows.UI.Xaml.Media.GradientSpreadMethod.Pad) {
                            returnValue = context.createLinearGradient(Bridge.box(x0, System.Double, System.Double.format, System.Double.getHashCode),Bridge.box(y0, System.Double, System.Double.format, System.Double.getHashCode),Bridge.box(x1, System.Double, System.Double.format, System.Double.getHashCode),Bridge.box(y1, System.Double, System.Double.format, System.Double.getHashCode));

                            $t = Bridge.getEnumerator(fillAsLinearGradientBrush.GradientStops);
                            try {
                                while ($t.moveNext()) {
                                    var gradientStop = $t.Current;
                                    returnValue.addColorStop(Bridge.box(gradientStop.Offset, System.Double, System.Double.format, System.Double.getHashCode),gradientStop.Color.INTERNAL_ToHtmlString(opacity));
                                }
                            } finally {
                                if (Bridge.is($t, System.IDisposable)) {
                                    $t.System$IDisposable$Dispose();
                                }
                            }
                        } else {






                            var angle;
                            if (fillAsLinearGradientBrush.MappingMode === Windows.UI.Xaml.Media.BrushMappingMode.Absolute) {
                                angle = Math.atan2(y1 - y0, x1 - x0);
                            } else {
                                angle = Math.atan2((fillAsLinearGradientBrush.EndPoint.Y - fillAsLinearGradientBrush.StartPoint.Y) * (maxY - minY), (fillAsLinearGradientBrush.EndPoint.X - fillAsLinearGradientBrush.StartPoint.X) * (maxX - minX));
                            }

                            var distance = Math.sqrt(Math.pow((x1 - x0), 2) + Math.pow((y1 - y0), 2));


                            var m = (y1 - y0) / (x1 - x0);
                            var perpM = -1 / m;

                            var k = y0 - x0 * perpM;


                            var xInter = k / (m - perpM);
                            var offset = xInter * Math.sqrt(1 + m * m);

                            var shapeHeight = shapeActualSize.Height;
                            var shapeWidth = shapeActualSize.Width;
                            var tempCanvasHeight = shapeHeight > shapeWidth ? shapeHeight : shapeWidth;


                            var canvas = document.createElement('canvas');

                            canvas.setAttribute('width', System.Double.format(distance) + "px");
canvas.setAttribute('height', System.Double.format(tempCanvasHeight) + "px");
canvas.style.width = System.Double.format(distance) + "px";
canvas.style.height = System.Double.format(tempCanvasHeight) + "px";

                            var ctx = canvas.getContext('2d');
                            var gradient = ctx.createLinearGradient(0, 0, Bridge.box(distance, System.Double, System.Double.format, System.Double.getHashCode), 0);

                            $t1 = Bridge.getEnumerator(fillAsLinearGradientBrush.GradientStops);
                            try {
                                while ($t1.moveNext()) {
                                    var gradientStop1 = $t1.Current;
                                    gradient.addColorStop(Bridge.box(gradientStop1.Offset, System.Double, System.Double.format, System.Double.getHashCode),gradientStop1.Color.INTERNAL_ToHtmlString(opacity));
                                }
                            } finally {
                                if (Bridge.is($t1, System.IDisposable)) {
                                    $t1.System$IDisposable$Dispose();
                                }
                            }


                            ctx.fillStyle = gradient;
ctx.fillRect(0, 0, Bridge.box(distance, System.Double, System.Double.format, System.Double.getHashCode), Bridge.box(tempCanvasHeight, System.Double, System.Double.format, System.Double.getHashCode));

if(shape._canvasDomElement != undefined && shape._canvasDomElement != null) {
var context = shape._canvasDomElement.getContext('2d');
//context.save();
context.rotate(Bridge.box(angle, System.Double, System.Double.format, System.Double.getHashCode));
context.translate(Bridge.box(offset, System.Double, System.Double.format, System.Double.getHashCode), 0);
//var pat = context.createPattern(canvas, 'repeat');

//context.fillStyle = pat;
//context.fill();
//context.restore();
}
;


                            returnValue = 
context.createPattern(canvas, 'repeat');
                        }
                    } else if (Bridge.is(brush, Windows.UI.Xaml.Media.RadialGradientBrush)) {


                        var fillAsRadialGradientBrush = Bridge.cast(brush, Windows.UI.Xaml.Media.RadialGradientBrush);

                        var radiusScaling = fillAsRadialGradientBrush.RadiusX / fillAsRadialGradientBrush.RadiusY;
                        var centerX = fillAsRadialGradientBrush.Center.X;
                        var gradientOriginX = fillAsRadialGradientBrush.GradientOrigin.X;
                        var centerY = fillAsRadialGradientBrush.Center.Y;
                        var gradientOriginY = fillAsRadialGradientBrush.GradientOrigin.Y;
                        var r = fillAsRadialGradientBrush.RadiusY;

                        if (fillAsRadialGradientBrush.MappingMode === Windows.UI.Xaml.Media.BrushMappingMode.RelativeToBoundingBox) {
                            centerX = fillAsRadialGradientBrush.Center.X * (xOffsetToApplyBeforeMultiplication + maxX - minX) * horizontalMultiplicator + minX;
                            gradientOriginX = fillAsRadialGradientBrush.GradientOrigin.X * (xOffsetToApplyBeforeMultiplication + maxX - minX) * horizontalMultiplicator + minX;
                            centerY = fillAsRadialGradientBrush.Center.Y * (yOffsetToApplyBeforeMultiplication + maxY - minY) * verticalMultiplicator + minY;
                            gradientOriginY = fillAsRadialGradientBrush.GradientOrigin.Y * (yOffsetToApplyBeforeMultiplication + maxY - minY) * verticalMultiplicator + minY;
                            r = r * (yOffsetToApplyBeforeMultiplication + maxY - minY) * verticalMultiplicator + minY;
                        }

                        context.scale(Bridge.box(radiusScaling, System.Double, System.Double.format, System.Double.getHashCode),1);
                        if (fillAsRadialGradientBrush.SpreadMethod === Windows.UI.Xaml.Media.GradientSpreadMethod.Pad) {
                            returnValue = context.createRadialGradient(Bridge.box(gradientOriginX, System.Double, System.Double.format, System.Double.getHashCode),Bridge.box(gradientOriginY, System.Double, System.Double.format, System.Double.getHashCode),0,Bridge.box(centerX, System.Double, System.Double.format, System.Double.getHashCode),Bridge.box(centerY, System.Double, System.Double.format, System.Double.getHashCode),Bridge.box(r, System.Double, System.Double.format, System.Double.getHashCode));

                            $t2 = Bridge.getEnumerator(System.Linq.Enumerable.from(fillAsRadialGradientBrush.GradientStops).orderBy(function (element) {
                                    return element.Offset;
                                }), Windows.UI.Xaml.Media.GradientStop);
                            try {
                                while ($t2.moveNext()) {
                                    var gradientStop2 = $t2.Current;
                                    returnValue.addColorStop(Bridge.box(gradientStop2.Offset, System.Double, System.Double.format, System.Double.getHashCode),gradientStop2.Color.INTERNAL_ToHtmlString(opacity));
                                }
                            } finally {
                                if (Bridge.is($t2, System.IDisposable)) {
                                    $t2.System$IDisposable$Dispose();
                                }
                            }
                        } else {
                            var repeatingTimes = 5;

                            var xVariation = Math.abs(fillAsRadialGradientBrush.Center.X - fillAsRadialGradientBrush.GradientOrigin.X);
                            var yVariation = Math.abs(fillAsRadialGradientBrush.Center.Y - fillAsRadialGradientBrush.GradientOrigin.Y);
                            var xCorrection = 0;
                            var yCorrection = 0;
                            if (xVariation !== 0 || yVariation !== 0) {
                                var vectorSize = Math.sqrt(Math.pow(xVariation, 2) + Math.pow(yVariation, 2));
                                xCorrection = xVariation / vectorSize;
                                yCorrection = yVariation / vectorSize;
                            }
                            var additionalRepetitions = (repeatingTimes - 1) | 0;

                            returnValue = context.createRadialGradient(Bridge.box(gradientOriginX, System.Double, System.Double.format, System.Double.getHashCode),Bridge.box(gradientOriginY, System.Double, System.Double.format, System.Double.getHashCode),0,Bridge.box(centerX - xCorrection * additionalRepetitions * r, System.Double, System.Double.format, System.Double.getHashCode),Bridge.box(centerY - yCorrection * additionalRepetitions * r, System.Double, System.Double.format, System.Double.getHashCode),Bridge.box(r + additionalRepetitions * r, System.Double, System.Double.format, System.Double.getHashCode));

                            var orderedGradients = System.Linq.Enumerable.from(fillAsRadialGradientBrush.GradientStops).orderBy(function (element) {
                                    return element.Offset;
                                });
                            var repetitionOffset = 1.0 / repeatingTimes;

                            for (var i = 0; i < repeatingTimes; i = (i + 1) | 0) {
                                $t3 = Bridge.getEnumerator(orderedGradients, Windows.UI.Xaml.Media.GradientStop);
                                try {
                                    while ($t3.moveNext()) {
                                        var gradientStop3 = $t3.Current;
                                        returnValue.addColorStop(Bridge.box(gradientStop3.Offset / repeatingTimes + i * repetitionOffset, System.Double, System.Double.format, System.Double.getHashCode),gradientStop3.Color.INTERNAL_ToHtmlString(opacity));
                                    }
                                } finally {
                                    if (Bridge.is($t3, System.IDisposable)) {
                                        $t3.System$IDisposable$Dispose();
                                    }
                                }
                            }
                        }
                    } else {
                        throw new System.NotSupportedException.$ctor1("The specified brush is not supported.");
                    }
                    return returnValue;
                }
            }
        },
        fields: {
            _canvasDomElement: null,
            _marginOffsets: null,
            _isListeningToAncestorsVisibilityChanged: false,
            _redrawPending: false,
            _redrawWhenBecomeVisible: false
        },
        props: {
            /**
             * Gets or sets the Brush that specifies how to paint the interior of the shape.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Shapes.Shape
             * @function Fill
             * @type Windows.UI.Xaml.Media.Brush
             */
            Fill: {
                get: function () {
                    return Bridge.cast(this.GetValue(Windows.UI.Xaml.Shapes.Shape.FillProperty), Windows.UI.Xaml.Media.Brush);
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Shapes.Shape.FillProperty, value);
                }
            },
            /**
             * Gets or sets a Stretch enumeration value that describes how the shape fills
             its allocated space.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Shapes.Shape
             * @function Stretch
             * @type Windows.UI.Xaml.Media.Stretch
             */
            Stretch: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.Shapes.Shape.StretchProperty), Windows.UI.Xaml.Media.Stretch), System.Int32));
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Shapes.Shape.StretchProperty, Bridge.box(value, Windows.UI.Xaml.Media.Stretch, System.Enum.toStringFn(Windows.UI.Xaml.Media.Stretch)));
                }
            },
            /**
             * Gets or sets the Brush that specifies how the Shape outline is painted.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Shapes.Shape
             * @function Stroke
             * @type Windows.UI.Xaml.Media.Brush
             */
            Stroke: {
                get: function () {
                    return Bridge.cast(this.GetValue(Windows.UI.Xaml.Shapes.Shape.StrokeProperty), Windows.UI.Xaml.Media.Brush);
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Shapes.Shape.StrokeProperty, value);
                }
            },
            /**
             * Gets or sets the width of the Shape stroke outline.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Shapes.Shape
             * @function StrokeThickness
             * @type number
             */
            StrokeThickness: {
                get: function () {
                    return System.Convert.toDouble(this.GetValue(Windows.UI.Xaml.Shapes.Shape.StrokeThicknessProperty));
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Shapes.Shape.StrokeThicknessProperty, Bridge.box(value, System.Double, System.Double.format, System.Double.getHashCode));
                }
            },
            /**
             * Gets or sets a PenLineCap enumeration value that describes the Shape at the
             start of a Stroke. The default is Flat.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Shapes.Shape
             * @function StrokeStartLineCap
             * @type Windows.UI.Xaml.Media.PenLineCap
             */
            StrokeStartLineCap: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.Shapes.Shape.StrokeStartLineCapProperty), Windows.UI.Xaml.Media.PenLineCap), System.Int32));
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Shapes.Shape.StrokeStartLineCapProperty, Bridge.box(value, Windows.UI.Xaml.Media.PenLineCap, System.Enum.toStringFn(Windows.UI.Xaml.Media.PenLineCap)));
                }
            },
            /**
             * Gets or sets a PenLineCap enumeration value that describes the Shape at the
             end of a line. The default is Flat.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Shapes.Shape
             * @function StrokeEndLineCap
             * @type Windows.UI.Xaml.Media.PenLineCap
             */
            StrokeEndLineCap: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.Shapes.Shape.StrokeEndLineCapProperty), Windows.UI.Xaml.Media.PenLineCap), System.Int32));
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Shapes.Shape.StrokeEndLineCapProperty, Bridge.box(value, Windows.UI.Xaml.Media.PenLineCap, System.Enum.toStringFn(Windows.UI.Xaml.Media.PenLineCap)));
                }
            },
            /**
             * Gets or sets a PenLineJoin enumeration value that specifies the type of join
             that is used at the vertices of a Shape. The default value is Miter.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Shapes.Shape
             * @function StrokeLineJoin
             * @type Windows.UI.Xaml.Media.PenLineJoin
             */
            StrokeLineJoin: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.Shapes.Shape.StrokeLineJoinProperty), Windows.UI.Xaml.Media.PenLineJoin), System.Int32));
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Shapes.Shape.StrokeLineJoinProperty, Bridge.box(value, Windows.UI.Xaml.Media.PenLineJoin, System.Enum.toStringFn(Windows.UI.Xaml.Media.PenLineJoin)));
                }
            },
            /**
             * Gets or sets a limit on the ratio of the miter length to half the StrokeThickness
             of a Shape element. This value is always a positive number that is greater than
             or equal to 1.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Shapes.Shape
             * @function StrokeMiterLimit
             * @type number
             */
            StrokeMiterLimit: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.Shapes.Shape.StrokeMiterLimitProperty), System.Double), System.Double));
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Shapes.Shape.StrokeMiterLimitProperty, Bridge.box(value, System.Double, System.Double.format, System.Double.getHashCode));
                }
            },
            /**
             * Gets or sets a collection of Double values that indicates the pattern of
             dashes and gaps that is used to outline shapes.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Shapes.Shape
             * @function StrokeDashArray
             * @type Windows.UI.Xaml.Media.DoubleCollection
             */
            StrokeDashArray: {
                get: function () {
                    var collection = Bridge.cast(this.GetValue(Windows.UI.Xaml.Shapes.Shape.StrokeDashArrayProperty), Windows.UI.Xaml.Media.DoubleCollection);
                    if (collection == null) {
                        collection = new Windows.UI.Xaml.Media.DoubleCollection();
                        this.SetValue(Windows.UI.Xaml.Shapes.Shape.StrokeDashArrayProperty, collection);
                    }
                    return collection;
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Shapes.Shape.StrokeDashArrayProperty, value);
                }
            },
            /**
             * Gets or sets a value that specifies the distance within the dash pattern
             where a dash begins.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Shapes.Shape
             * @function StrokeDashOffset
             * @type number
             */
            StrokeDashOffset: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.Shapes.Shape.StrokeDashOffsetProperty), System.Double), System.Double));
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Shapes.Shape.StrokeDashOffsetProperty, Bridge.box(value, System.Double, System.Double.format, System.Double.getHashCode));
                }
            }
        },
        ctors: {
            init: function () {
                this._marginOffsets = new Windows.Foundation.Point();
                this._redrawPending = false;
                this._redrawWhenBecomeVisible = true;
            },
            /**
             * Provides base class initialization behavior for Shape derived classes.
             *
             * @instance
             * @protected
             * @this Windows.UI.Xaml.Shapes.Shape
             * @memberof Windows.UI.Xaml.Shapes.Shape
             * @return  {void}
             */
            ctor: function () {
                this.$initialize();
                Windows.UI.Xaml.FrameworkElement.ctor.call(this);
                this.addSizeChanged(Bridge.fn.cacheBind(this, this.Shape_SizeChanged));

                if (!document.isGridSupported) {
                    Windows.UI.Xaml.Window.Current.addSizeChanged$1(Bridge.fn.cacheBind(this, this.Current_SizeChanged));
                }

                this.addIsVisibleChanged(Bridge.fn.bind(this, function (d, e) {
                    var newValue = System.Nullable.getValue(Bridge.cast(Bridge.unbox(e.NewValue, System.Boolean), System.Boolean));
                    if (newValue && this._redrawWhenBecomeVisible) {
                        this.ScheduleRedraw();
                    }
                }));
            }
        },
        methods: {
            /**
             * method that end-users are supposed to call if they change the geometry and want to redraw the Path.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Shapes.Shape
             * @memberof Windows.UI.Xaml.Shapes.Shape
             * @return  {void}
             */
            Refresh: function () {
                this.RefreshOverride();
            },
            Redraw: function () {
                this._redrawWhenBecomeVisible = false;
            },
            ManageStrokeChanged: function () {
                this.ScheduleRedraw();
            },
            ManageStrokeThicknessChanged: function () {
                if (CSHTML5.Internal.INTERNAL_VisualTreeManager.IsElementInVisualTree(this)) {
                    var context = CSHTML5.Internal.INTERNAL_HtmlDomManager.Get2dCanvasContext(this._canvasDomElement);
                    context.lineWidth = System.Double.format(this.StrokeThickness) + "";
                    this.ScheduleRedraw();
                }
            },
            INTERNAL_OnDetachedFromVisualTree: function () {
                if (this._isListeningToAncestorsVisibilityChanged) {
                    CSHTML5.Internal.INTERNAL_VisibilityChangedNotifier.StopListeningToAncestorsVisibilityChanged(this);
                }
                this._isListeningToAncestorsVisibilityChanged = false;

                Windows.UI.Xaml.FrameworkElement.prototype.INTERNAL_OnDetachedFromVisualTree.call(this);
            },
            RefreshOverride: function () {
                this.ScheduleRedraw();
            },
            /**
             * This is used to redraw only once instead of on every property change.
             *
             * @instance
             * @this Windows.UI.Xaml.Shapes.Shape
             * @memberof Windows.UI.Xaml.Shapes.Shape
             * @return  {void}
             */
            ScheduleRedraw: function () {
                if (!this._redrawPending) {
                    if (this._isLoaded) {
                        this._redrawPending = true;
                        CSHTML5.Internal.INTERNAL_DispatcherHelpers.QueueAction(Bridge.fn.bind(this, function () {
                            this._redrawPending = false;

                            if ((!(typeof this.INTERNAL_OuterDomElement === 'undefined' || this.INTERNAL_OuterDomElement === null) && !document.doesElementInheritDisplayNone(this.INTERNAL_OuterDomElement))) {
                                this.Redraw();
                            } else {
                                this._redrawWhenBecomeVisible = true;
                            }
                        }));
                    }
                }
            },
            Current_SizeChanged: function (sender, e) {
                if ((this.HorizontalAlignment === Windows.UI.Xaml.HorizontalAlignment.Stretch && isNaN(this.Width)) || (this.VerticalAlignment === Windows.UI.Xaml.VerticalAlignment.Stretch && isNaN(this.Height))) {
                    this.ScheduleRedraw();
                }
            },
            Shape_SizeChanged: function (sender, e) {
                this.ScheduleRedraw();
            },
            ApplyMarginToFixNegativeCoordinates: function (newFixingMargin) {
                if (CSHTML5.Internal.INTERNAL_VisualTreeManager.IsElementInVisualTree(this)) {
                    var styleOfcanvasElement = CSHTML5.Internal.INTERNAL_HtmlDomManager.GetDomElementStyleForModification(this._canvasDomElement);
                    if (System.Nullable.lifteq(Windows.Foundation.Point.op_Equality, newFixingMargin.$clone(), null)) {
                        newFixingMargin = new Windows.Foundation.Point.ctor();
                    }
                    try {
                        styleOfcanvasElement.transform = "translate(" + (System.Double.format(newFixingMargin.X) || "") + "px, " + (System.Double.format(newFixingMargin.Y) || "") + "px)";
                    } catch ($e1) {
                        $e1 = System.Exception.create($e1);
                    }
                    try {
                        styleOfcanvasElement.msTransform = "translate(" + (System.Double.format(newFixingMargin.X) || "") + "px, " + (System.Double.format(newFixingMargin.Y) || "") + "px)";
                    } catch ($e2) {
                        $e2 = System.Exception.create($e2);
                    }
                    try {
                        styleOfcanvasElement.WebkitTransform = "translate(" + (System.Double.format(newFixingMargin.X) || "") + "px, " + (System.Double.format(newFixingMargin.Y) || "") + "px)";
                    } catch ($e3) {
                        $e3 = System.Exception.create($e3);
                    }
                }
            }
        }
    });

    /**
     * @memberof Windows.UI.Xaml
     * @callback Windows.UI.Xaml.WindowSizeChangedEventHandler
     * @param   {System.Object}                                 sender    
     * @param   {Windows.UI.Core.WindowSizeChangedEventArgs}    e
     * @return  {void}
     */

    /**
     * Represents an application window.
     *
     * @public
     * @class Windows.UI.Xaml.Window
     * @augments Windows.UI.Xaml.FrameworkElement
     * @implements  Windows.UI.Xaml.Markup.INameScope
     */
    Bridge.define("Windows.UI.Xaml.Window", {
        inherits: [Windows.UI.Xaml.FrameworkElement,Windows.UI.Xaml.Markup.INameScope],
        statics: {
            fields: {
                /**
                 * Gets the currently activated window for an application.
                 *
                 * @static
                 * @public
                 * @memberof Windows.UI.Xaml.Window
                 * @function Current
                 * @type Windows.UI.Xaml.Window
                 */
                Current: null,
                /**
                 * Identifies the Content dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Window
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                ContentProperty: null
            },
            ctors: {
                init: function () {
                    var $t;
                    this.ContentProperty = Windows.UI.Xaml.DependencyProperty.Register("Content", System.Object, Windows.UI.Xaml.Window, ($t = new Windows.UI.Xaml.PropertyMetadata.$ctor2(null, Windows.UI.Xaml.Window.Content_Changed), $t.CallPropertyChangedWhenLoadedIntoVisualTree = Windows.UI.Xaml.WhenToCallPropertyChangedEnum.IfPropertyIsSet, $t));
                }
            },
            methods: {
                Content_Changed: function (d, e) {
                    var $window = Bridge.cast(d, Windows.UI.Xaml.Window);
                    $window.OnContentChanged(e.OldValue, e.NewValue);
                }
            }
        },
        fields: {
            INTERNAL_PositionsWatcher: null,
            INTERNAL_RootDomElement: null,
            _windowSizeChangedEventManager: null,
            _nameScopeDictionary: null,
            _closingEventManager: null
        },
        props: {
            /**
             * Gets the height and width of the application window, as a Rect value.
             *
             * @instance
             * @public
             * @readonly
             * @memberof Windows.UI.Xaml.Window
             * @function Bounds
             * @type Windows.Foundation.Rect
             */
            Bounds: {
                get: function () {
                    var actualSize = this.INTERNAL_GetActualWidthAndHeight();
                    var width = actualSize.Width;
                    var height = actualSize.Height;
                    if (!isNaN(width) && !isNaN(height)) {
                        return new Windows.Foundation.Rect.$ctor1(0, 0, width, height);
                    } else {
                        return new Windows.Foundation.Rect.$ctor1(0, 0, 0, 0);
                    }
                }
            },
            /**
             * Gets or sets the content of the Window.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Window
             * @function Content
             * @type System.Object
             */
            Content: {
                get: function () {
                    return this.GetValue(Windows.UI.Xaml.Window.ContentProperty);
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Window.ContentProperty, value);
                }
            },
            ClosingEventManager: {
                get: function () {
                    if (this._closingEventManager == null) {
                        this._closingEventManager = new (CSHTML5.Internal.INTERNAL_EventManager$2(Function,System.ComponentModel.ClosingEventArgs)).ctor(Bridge.fn.cacheBind(Windows.UI.Xaml.Application.Current, Windows.UI.Xaml.Application.Current.GetWindow), "beforeunload", Bridge.fn.cacheBind(this, this.ProcessOnClosing));
                    }
                    return this._closingEventManager;
                }
            }
        },
        alias: [
            "FindName$1", "Windows$UI$Xaml$Markup$INameScope$FindName",
            "RegisterName", "Windows$UI$Xaml$Markup$INameScope$RegisterName",
            "UnregisterName", "Windows$UI$Xaml$Markup$INameScope$UnregisterName"
        ],
        ctors: {
            init: function () {
                this.INTERNAL_PositionsWatcher = new Windows.UI.Xaml.PositionsWatcher();
                this._nameScopeDictionary = new (System.Collections.Generic.Dictionary$2(System.String,System.Object)).ctor();
            }
        },
        methods: {
            /**
             * Occurs when the window has rendered or changed its rendering size.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Window
             * @event Windows.UI.Xaml.Window#SizeChanged
             * @return  {Windows.UI.Xaml.WindowSizeChangedEventHandler}
             */
            addSizeChanged$1: function (value) {
                if (this._windowSizeChangedEventManager == null) {
                    this._windowSizeChangedEventManager = new (CSHTML5.Internal.INTERNAL_EventManager$2(Function,Windows.UI.Core.WindowSizeChangedEventArgs)).ctor(function () {
                        return window;
                    }, "resize", Bridge.fn.cacheBind(this, this.ProcessOnWindowSizeChanged));
                }
                this._windowSizeChangedEventManager.Add(value);
            },
            /**
             * Occurs when the window has rendered or changed its rendering size.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Window
             * @event Windows.UI.Xaml.Window#SizeChanged
             * @return  {Windows.UI.Xaml.WindowSizeChangedEventHandler}
             */
            removeSizeChanged$1: function (value) {
                if (this._windowSizeChangedEventManager != null) {
                    this._windowSizeChangedEventManager.Remove(value);
                }
            },
            /**
             * Occurs when the window is about to close.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Window
             * @event Windows.UI.Xaml.Window#Closing
             * @return  {System.EventHandler}
             */
            addClosing: function (value) {
                this.ClosingEventManager.Add(value);
            },
            /**
             * Occurs when the window is about to close.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Window
             * @event Windows.UI.Xaml.Window#Closing
             * @return  {System.EventHandler}
             */
            removeClosing: function (value) {
                this.ClosingEventManager.Remove(value);
            },
            /**
             * Set the DOM element that will host the window. This can be set only to new windows. The MainWindow looks for a DIV that has the ID "cshtml5-root".
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Window
             * @memberof Windows.UI.Xaml.Window
             * @param   {System.Object}    rootDomElement    The DOM element that will host the window
             * @return  {void}
             */
            AttachToDomElement: function (rootDomElement) {
                if (this.INTERNAL_OuterDomElement != null || this.INTERNAL_RootDomElement != null) {
                    throw new System.InvalidOperationException.$ctor1("The method 'Window.AttachToDomElement' can be called only once.");
                }

                if (rootDomElement == null) {
                    throw new System.ArgumentNullException.$ctor1("rootDomElement");
                }

                this.INTERNAL_RootDomElement = rootDomElement;

                rootDomElement.innerHTML = '';

                if (document.isGridSupported) {
                    rootDomElement.style.display = 'grid';
                }

                var windowRootDiv = { };
                var windowRootDivStyle = CSHTML5.Internal.INTERNAL_HtmlDomManager.CreateDomElementAppendItAndGetStyle("div", rootDomElement, this, windowRootDiv);

                windowRootDivStyle.position = "absolute";
                windowRootDivStyle.width = "100%";
                windowRootDivStyle.height = "100%";
                windowRootDivStyle.overflowX = "hidden";
                windowRootDivStyle.overflowY = "hidden";

                this.INTERNAL_OuterDomElement = windowRootDiv.v;
                this.INTERNAL_InnerDomElement = windowRootDiv.v;

                this._isLoaded = true;

                var content = this.Content;
                if (content != null) {
                    this.OnContentChanged(null, content);
                }

                this.INTERNAL_RaiseLoadedEvent();
            },
            ProcessOnWindowSizeChanged: function (jsEventArg) {
                var $t;
                var width;
                var height;
                if (CSHTML5.Interop.IsRunningInTheSimulator) {
                    var concatenated = System.Convert.toString(this.INTERNAL_OuterDomElement.offsetWidth + '|' + this.INTERNAL_OuterDomElement.offsetHeight);
                    var sepIndex = System.String.indexOf(concatenated, String.fromCharCode(124));
                    var widthAsString = concatenated.substr(0, sepIndex);
                    var heightAsString = concatenated.substr(((sepIndex + 1) | 0));
                    width = Bridge.Int.parseFloat(widthAsString, System.Globalization.CultureInfo.invariantCulture);
                    height = Bridge.Int.parseFloat(heightAsString, System.Globalization.CultureInfo.invariantCulture);
                } else {
                    width = System.Convert.toDouble(this.INTERNAL_OuterDomElement.offsetWidth);
                    height = System.Convert.toDouble(this.INTERNAL_OuterDomElement.offsetHeight);
                }

                var eventArgs = ($t = new Windows.UI.Core.WindowSizeChangedEventArgs(), $t.Size = new Windows.Foundation.Size.$ctor1(width, height), $t);
                this.OnWindowSizeChanged(eventArgs);
            },
            OnWindowSizeChanged: function (eventArgs) {
                var $t;
                $t = Bridge.getEnumerator(System.Linq.Enumerable.from(this._windowSizeChangedEventManager.Handlers).toList(Function));
                try {
                    while ($t.moveNext()) {
                        var handler = $t.Current;
                        handler(this, eventArgs);
                    }
                } finally {
                    if (Bridge.is($t, System.IDisposable)) {
                        $t.System$IDisposable$Dispose();
                    }
                }
            },
            OnContentChanged: function (oldContent, newContent) {
                if (this._isLoaded) {
                    this.INTERNAL_ParentWindow = this;

                    CSHTML5.Internal.INTERNAL_VisualTreeManager.DetachVisualChildIfNotNull(Bridge.as(oldContent, Windows.UI.Xaml.UIElement), this);
                    CSHTML5.Internal.INTERNAL_VisualTreeManager.AttachVisualChildIfNotAlreadyAttached(Bridge.as(newContent, Windows.UI.Xaml.UIElement), this);
                    this.INTERNAL_ParentWindow = null;


                    if (!CSHTML5.Interop.IsRunningInTheSimulator) {
                        if (CSHTML5.Native.Html.Printing.PrintManager.IsDefaultPrintArea) {
                            CSHTML5.Native.Html.Printing.PrintManager.ResetPrintArea();
                        }
                    }
                }
            },
            /**
             * Finds the UIElement with the specified name.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Window
             * @memberof Windows.UI.Xaml.Window
             * @param   {string}           name    The name to look for.
             * @return  {System.Object}            The object with the specified name if any; otherwise null.
             */
            FindName$1: function (name) {
                if (this._nameScopeDictionary.containsKey(name)) {
                    return this._nameScopeDictionary.getItem(name);
                } else {
                    return null;
                }
            },
            RegisterName: function (name, scopedElement) {
                if (this._nameScopeDictionary.containsKey(name) && !Bridge.referenceEquals(this._nameScopeDictionary.getItem(name), scopedElement)) {
                    throw new System.ArgumentException.$ctor1(System.String.format("Cannot register duplicate name '{0}' in this scope.", [name]));
                }

                this._nameScopeDictionary.setItem(name, scopedElement);
            },
            UnregisterName: function (name) {
                if (!this._nameScopeDictionary.containsKey(name)) {
                    throw new System.ArgumentException.$ctor1(System.String.format("Name '{0}' was not found.", [name]));
                }

                this._nameScopeDictionary.remove(name);
            },
            /**
             * Attempts to activate the application window by bringing it to the foreground
             and setting the input focus to it.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Window
             * @memberof Windows.UI.Xaml.Window
             * @return  {void}
             */
            Activate: function () { },
            CreateDomElement: function (parentRef, domElementWhereToPlaceChildren) {
                throw new System.InvalidOperationException.$ctor1("\"CreateDomElement\" should not be called for the Window object.");
            },
            /**
             * Raises the Closing event
             *
             * @instance
             * @private
             * @this Windows.UI.Xaml.Window
             * @memberof Windows.UI.Xaml.Window
             * @param   {System.Object}    jsEventArg
             * @return  {void}
             */
            ProcessOnClosing: function (jsEventArg) {

                var eventArgs = new System.ComponentModel.ClosingEventArgs(true);
                eventArgs.INTERNAL_JSArgs = jsEventArg;
                this.OnClosing(eventArgs);
            },
            /**
             * Raises the Closing event
             *
             * @instance
             * @protected
             * @this Windows.UI.Xaml.Window
             * @memberof Windows.UI.Xaml.Window
             * @param   {System.ComponentModel.ClosingEventArgs}    eventArgs    The arguments for the event.
             * @return  {void}
             */
            OnClosing: function (eventArgs) {
                var $t;
                $t = Bridge.getEnumerator(this._closingEventManager.Handlers);
                try {
                    while ($t.moveNext()) {
                        var handler = $t.Current;
                        handler(this, eventArgs);
                    }
                } finally {
                    if (Bridge.is($t, System.IDisposable)) {
                        $t.System$IDisposable$Dispose();
                    }
                }
            }
        }
    });

    /**
     * Represents a control that can be used to present a collection of items.
     *
     * @public
     * @class Windows.UI.Xaml.Controls.ItemsControl
     * @augments Windows.UI.Xaml.Controls.Control
     * @implements  CSHTML5.Internals.Controls.IGeneratorHost
     */
    Bridge.define("Windows.UI.Xaml.Controls.ItemsControl", {
        inherits: [Windows.UI.Xaml.Controls.Control,CSHTML5.Internals.Controls.IGeneratorHost],
        statics: {
            fields: {
                /**
                 * Identifies the ItemsPanel dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Controls.ItemsControl
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                ItemsPanelProperty: null,
                /**
                 * Identifies the ItemsSource dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Controls.ItemsControl
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                ItemsSourceProperty: null,
                /**
                 * Identifies the ItemTemplate dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Controls.ItemsControl
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                ItemTemplateProperty: null,
                /**
                 * Identifies the DisplayMemberPath dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Controls.ItemsControl
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                DisplayMemberPathProperty: null,
                /**
                 * Identifies the ItemContainerStyle dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Controls.ItemsControl
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                ItemContainerStyleProperty: null
            },
            ctors: {
                init: function () {
                    this.ItemsPanelProperty = Windows.UI.Xaml.DependencyProperty.Register("ItemsPanel", Windows.UI.Xaml.Controls.ItemsPanelTemplate, Windows.UI.Xaml.Controls.ItemsControl, new Windows.UI.Xaml.PropertyMetadata.$ctor2(Windows.UI.Xaml.Controls.ItemsControl.GetDefaultItemsPanel(), Windows.UI.Xaml.Controls.ItemsControl.OnItemsPanelChanged));
                    this.ItemsSourceProperty = Windows.UI.Xaml.DependencyProperty.Register("ItemsSource", System.Collections.IEnumerable, Windows.UI.Xaml.Controls.ItemsControl, new Windows.UI.Xaml.PropertyMetadata.$ctor2(null, Windows.UI.Xaml.Controls.ItemsControl.OnItemsSourceChanged));
                    this.ItemTemplateProperty = Windows.UI.Xaml.DependencyProperty.Register("ItemTemplate", Windows.UI.Xaml.DataTemplate, Windows.UI.Xaml.Controls.ItemsControl, new Windows.UI.Xaml.PropertyMetadata.$ctor2(null, Windows.UI.Xaml.Controls.ItemsControl.OnItemTemplateChanged));
                    this.DisplayMemberPathProperty = Windows.UI.Xaml.DependencyProperty.Register("DisplayMemberPath", System.String, Windows.UI.Xaml.Controls.ItemsControl, new Windows.UI.Xaml.PropertyMetadata.$ctor2("", Windows.UI.Xaml.Controls.ItemsControl.OnDisplayMemberPathChanged));
                    this.ItemContainerStyleProperty = Windows.UI.Xaml.DependencyProperty.Register("ItemContainerStyle", Windows.UI.Xaml.Style, Windows.UI.Xaml.Controls.ItemsControl, new Windows.UI.Xaml.PropertyMetadata.$ctor2(null, Windows.UI.Xaml.Controls.ItemsControl.OnItemContainerStyleChanged));
                }
            },
            methods: {
                GetDefaultItemsPanel: function () {
                    var $t;
                    var template = ($t = new Windows.UI.Xaml.Controls.ItemsPanelTemplate(), $t._methodToInstantiateFrameworkTemplate = function (templateOwner) {
                        var $t1;
                        return ($t1 = new Windows.UI.Xaml.TemplateInstance(), $t1.TemplateContent = new Windows.UI.Xaml.Controls.StackPanel(), $t1);
                    }, $t);
                    template.Seal();

                    return template;
                },
                OnItemsPanelChanged: function (d, e) {
                    Bridge.cast(d, Windows.UI.Xaml.Controls.ItemsControl).UpdateItemsPanel(Bridge.cast(e.NewValue, Windows.UI.Xaml.Controls.ItemsPanelTemplate));
                },
                OnItemsSourceChanged: function (d, e) {
                    var ic = Bridge.cast(d, Windows.UI.Xaml.Controls.ItemsControl);
                    var oldValue = Bridge.cast(e.OldValue, System.Collections.IEnumerable);
                    var newValue = Bridge.cast(e.NewValue, System.Collections.IEnumerable);

                    if (e.NewValue != null) {
                        ic.Items.SetItemsSource(newValue);
                    } else {
                        ic.Items.ClearItemsSource();
                    }

                    ic.OnItemsSourceChanged(oldValue, newValue);
                },
                /**
                 * Invoked when the value of the ItemTemplate property changes.
                 *
                 * @static
                 * @private
                 * @this Windows.UI.Xaml.Controls.ItemsControl
                 * @memberof Windows.UI.Xaml.Controls.ItemsControl
                 * @param   {Windows.UI.Xaml.DependencyObject}                      d    
                 * @param   {Windows.UI.Xaml.DependencyPropertyChangedEventArgs}    e
                 * @return  {void}
                 */
                OnItemTemplateChanged: function (d, e) {
                    var itemsControl = Bridge.cast(d, Windows.UI.Xaml.Controls.ItemsControl);
                    if (itemsControl.ItemsHost != null) {
                        itemsControl.Refresh();
                    }
                },
                OnDisplayMemberPathChanged: function (d, e) {
                    var itemsControl = Bridge.cast(d, Windows.UI.Xaml.Controls.ItemsControl);
                    if (itemsControl.ItemsHost != null) {
                        itemsControl.Refresh();
                    }
                },
                OnItemContainerStyleChanged: function (d, e) {
                    var itemsControl = Bridge.cast(d, Windows.UI.Xaml.Controls.ItemsControl);
                    if (itemsControl.ItemsHost != null) {
                        itemsControl.Refresh();
                    }
                },
                EqualsEx: function (o1, o2) {
                    try {
                        return Bridge.equals(o1, o2);
                    } catch ($e1) {
                        $e1 = System.Exception.create($e1);
                        if (Bridge.is($e1, System.InvalidCastException)) {
                            return false;
                        } else {
                            throw $e1;
                        }
                    }
                },
                GetDataTemplateForDisplayMemberPath: function (displayMemberPath) {
                    var template = new Windows.UI.Xaml.DataTemplate();

                    template._methodToInstantiateFrameworkTemplate = function (control) {
                        var $t;
                        var templateInstance = new Windows.UI.Xaml.TemplateInstance();

                        var textBlock = new Windows.UI.Xaml.Controls.TextBlock();
                        textBlock.SetBinding(Windows.UI.Xaml.Controls.TextBlock.TextProperty, new Windows.UI.Xaml.Data.Binding.$ctor1(($t = displayMemberPath, $t != null ? $t : "")));

                        templateInstance.TemplateContent = textBlock;

                        return templateInstance;
                    };

                    return template;
                },
                PrepareContentControl: function (cc, item, template) {
                    if (!Bridge.referenceEquals(item, cc)) {
                        cc.ContentTemplate = template;
                        cc.Content = item;
                    }
                },
                PrepareContentPresenter: function (cp, item, template) {
                    if (!Bridge.referenceEquals(item, cp)) {
                        cp.ContentTemplate = template;
                        cp.Content = item;
                    }
                },
                ClearContentControl: function (cc, item) {
                    if (!Bridge.referenceEquals(cc, item)) {
                        cc.ClearValue(Windows.UI.Xaml.Controls.ContentControl.ContentProperty);
                    }
                },
                ClearContentPresenter: function (cp, item) {
                    if (!Bridge.referenceEquals(cp, item)) {
                        cp.ClearValue(Windows.UI.Xaml.Controls.ContentControl.ContentProperty);
                    }
                },
                GetItemsOwner: function (element) {
                    var container = null;
                    var panel = Bridge.as(element, Windows.UI.Xaml.Controls.Panel);

                    if (panel != null && panel.IsItemsHost) {
                        var ip = Windows.UI.Xaml.Controls.ItemsPresenter.FromPanel(panel);

                        if (ip != null) {
                            container = ip.Owner;
                        }
                    }

                    return container;
                },
                ItemsControlFromItemContainer: function (container) {
                    var ui = Bridge.as(container, Windows.UI.Xaml.UIElement);
                    if (ui == null) {
                        return null;
                    }

                    var ic = Bridge.as(ui.INTERNAL_VisualParent, Windows.UI.Xaml.Controls.ItemsControl);
                    if (ic != null) {
                        var host = Bridge.as(ic, CSHTML5.Internals.Controls.IGeneratorHost);
                        if (host.CSHTML5$Internals$Controls$IGeneratorHost$IsItemItsOwnContainer(ui)) {
                            return ic;
                        } else {
                            return null;
                        }
                    }

                    ui = Bridge.as(Windows.UI.Xaml.Media.VisualTreeHelper.GetParent(ui), Windows.UI.Xaml.UIElement);

                    return Windows.UI.Xaml.Controls.ItemsControl.GetItemsOwner(ui);
                }
            }
        },
        fields: {
            _itemContainerGenerator: null,
            _itemsPresenter: null,
            _items: null
        },
        props: {
            /**
             * Items is the collection of data that is used to generate the content
             of this control.
             *
             * @instance
             * @public
             * @readonly
             * @memberof Windows.UI.Xaml.Controls.ItemsControl
             * @function Items
             * @type Windows.UI.Xaml.Controls.ItemCollection
             */
            Items: {
                get: function () {
                    if (this._items == null) {
                        this.CreateItemCollectionAndGenerator();
                    }
                    return this._items;
                }
            },
            /**
             * Gets the ItemContainerGenerator associated with this ItemsControl.
             *
             * @instance
             * @public
             * @readonly
             * @memberof Windows.UI.Xaml.Controls.ItemsControl
             * @function ItemContainerGenerator
             * @type Windows.UI.Xaml.Controls.ItemContainerGenerator
             */
            ItemContainerGenerator: {
                get: function () {
                    if (this._itemContainerGenerator == null) {
                        this.CreateItemCollectionAndGenerator();
                    }
                    return this._itemContainerGenerator;
                }
            },
            /**
             * Gets or sets the template that defines the panel that controls the layout
             of items.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.ItemsControl
             * @function ItemsPanel
             * @type Windows.UI.Xaml.Controls.ItemsPanelTemplate
             */
            ItemsPanel: {
                get: function () {
                    return Bridge.cast(this.GetValue(Windows.UI.Xaml.Controls.ItemsControl.ItemsPanelProperty), Windows.UI.Xaml.Controls.ItemsPanelTemplate);
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Controls.ItemsControl.ItemsPanelProperty, value);
                }
            },
            /**
             * Gets or sets an object source used to generate the content of the ItemsControl.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.ItemsControl
             * @function ItemsSource
             * @type System.Collections.IEnumerable
             */
            ItemsSource: {
                get: function () {
                    return Bridge.cast(this.GetValue(Windows.UI.Xaml.Controls.ItemsControl.ItemsSourceProperty), System.Collections.IEnumerable);
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Controls.ItemsControl.ItemsSourceProperty, value);
                }
            },
            /**
             * Gets or sets the DataTemplate used to display each item.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.ItemsControl
             * @function ItemTemplate
             * @type Windows.UI.Xaml.DataTemplate
             */
            ItemTemplate: {
                get: function () {
                    return Bridge.cast(this.GetValue(Windows.UI.Xaml.Controls.ItemsControl.ItemTemplateProperty), Windows.UI.Xaml.DataTemplate);
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Controls.ItemsControl.ItemTemplateProperty, value);
                }
            },
            /**
             * Gets or sets a path to a value on the source object to serve as the visual
             representation of the object.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.ItemsControl
             * @function DisplayMemberPath
             * @type string
             */
            DisplayMemberPath: {
                get: function () {
                    return Bridge.cast(this.GetValue(Windows.UI.Xaml.Controls.ItemsControl.DisplayMemberPathProperty), System.String);
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Controls.ItemsControl.DisplayMemberPathProperty, value);
                }
            },
            /**
             * Gets or sets the style that is used when rendering the item containers.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.ItemsControl
             * @function ItemContainerStyle
             * @type Windows.UI.Xaml.Style
             */
            ItemContainerStyle: {
                get: function () {
                    return Bridge.cast(this.GetValue(Windows.UI.Xaml.Controls.ItemsControl.ItemContainerStyleProperty), Windows.UI.Xaml.Style);
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Controls.ItemsControl.ItemContainerStyleProperty, value);
                }
            },
            /**
             * The view of the data
             *
             * @instance
             * @readonly
             * @memberof Windows.UI.Xaml.Controls.ItemsControl
             * @function CSHTML5$Internals$Controls$IGeneratorHost$View
             * @type System.Collections.IList
             */
            CSHTML5$Internals$Controls$IGeneratorHost$View: {
                get: function () {
                    return this.Items;
                }
            },
            ItemsPresenter: {
                get: function () {
                    return this._itemsPresenter;
                }
            },
            ItemsHost: {
                get: function () {
                    if (this.ItemsPresenter != null) {
                        return this.ItemsPresenter.ItemsHost;
                    }
                    return null;
                }
            },
            HasItems: {
                get: function () {
                    return this.Items.Count > 0;
                }
            }
        },
        ctors: {
            /**
             * Initializes a new instance of the ItemsControl class.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Controls.ItemsControl
             * @memberof Windows.UI.Xaml.Controls.ItemsControl
             * @return  {void}
             */
            ctor: function () {
                this.$initialize();
                Windows.UI.Xaml.Controls.Control.ctor.call(this);
                this.DefaultStyleKey = Windows.UI.Xaml.Controls.ItemsControl;
            }
        },
        methods: {
            CreateItemCollectionAndGenerator: function () {
                this._items = new Windows.UI.Xaml.Controls.ItemCollection();

                this._itemContainerGenerator = new Windows.UI.Xaml.Controls.ItemContainerGenerator();

                this._items.addCollectionChanged(Bridge.fn.cacheBind(this, this.OnItemCollectionChanged));
            },
            Refresh: function () {
                var $t;
                var host = Bridge.cast(this, CSHTML5.Internals.Controls.IGeneratorHost);

                this.ItemsHost.Children.clear();
                this.ItemContainerGenerator.INTERNAL_Clear();
                $t = Bridge.getEnumerator(this.Items);
                try {
                    while ($t.moveNext()) {
                        var item = $t.Current;
                        var container = host.CSHTML5$Internals$Controls$IGeneratorHost$GetContainerForItem(item, null);
                        if (!Bridge.referenceEquals(container, item)) {
                            container.SetValue(Windows.UI.Xaml.FrameworkElement.DataContextProperty, item);
                        }
                        host.CSHTML5$Internals$Controls$IGeneratorHost$PrepareItemContainer(container, item);

                        this.ItemContainerGenerator.INTERNAL_RegisterContainer(container, item);

                        this.ItemsHost.Children.add(Bridge.cast(container, Windows.UI.Xaml.UIElement));
                    }
                } finally {
                    if (Bridge.is($t, System.IDisposable)) {
                        $t.System$IDisposable$Dispose();
                    }
                }
            },
            CSHTML5$Internals$Controls$IGeneratorHost$ClearContainerForItem: function (container, item) {
                this.ClearContainerForItemOverride(container, item);
            },
            CSHTML5$Internals$Controls$IGeneratorHost$GetContainerForItem: function (item, recycledContainer) {

                var container;

                if (this.IsItemItsOwnContainerOverride(item)) {
                    if (this.Items.IsUsingItemsSource && this.ItemTemplate != null) {
                        throw new System.NotSupportedException.$ctor1("ItemsControl.Items must not be a UIElement type when an ItemTemplate is set.");
                    }
                    container = Bridge.as(item, Windows.UI.Xaml.DependencyObject);
                } else {
                    container = this.GetContainerForItemOverride();
                }


                var visual = Bridge.as(container, Windows.UI.Xaml.UIElement);
                if (visual != null) {
                    var parent = Bridge.as(Windows.UI.Xaml.Media.VisualTreeHelper.GetParent(visual), Windows.UI.Xaml.UIElement);
                    if (parent != null) {
                        System.Diagnostics.Debug.Assert$1(Bridge.is(parent, Windows.UI.Xaml.FrameworkElement), "The Parent of the container should always be a FrameworkElement");
                        var p = Bridge.as(parent, Windows.UI.Xaml.Controls.Panel);
                        if (p != null) {
                            p.Children.remove(visual);
                        }
                    }
                }

                return container;
            },
            CSHTML5$Internals$Controls$IGeneratorHost$IsHostForItemContainer: function (container) {
                var ic = Windows.UI.Xaml.Controls.ItemsControl.ItemsControlFromItemContainer(container);
                if (ic != null) {
                    return (Bridge.referenceEquals(ic, this));
                }


                var parent = Windows.UI.Xaml.Media.VisualTreeHelper.GetParent(container);

                if (parent == null) {
                    return this.IsItemItsOwnContainerOverride(container) && this.HasItems && this.Items.contains(container);
                }

                return false;
            },
            /**
             * Return true if the item is (or is eligible to be) its own ItemContainer
             *
             * @instance
             * @this Windows.UI.Xaml.Controls.ItemsControl
             * @memberof Windows.UI.Xaml.Controls.ItemsControl
             * @param   {System.Object}    item
             * @return  {boolean}
             */
            CSHTML5$Internals$Controls$IGeneratorHost$IsItemItsOwnContainer: function (item) {
                return this.IsItemItsOwnContainer(item);
            },
            /**
             * Return true if the item is (or should be) its own item container
             *
             * @instance
             * @this Windows.UI.Xaml.Controls.ItemsControl
             * @memberof Windows.UI.Xaml.Controls.ItemsControl
             * @param   {System.Object}    item
             * @return  {boolean}
             */
            IsItemItsOwnContainer: function (item) {
                return this.IsItemItsOwnContainerOverride(item);
            },
            CSHTML5$Internals$Controls$IGeneratorHost$PrepareItemContainer: function (container, item) {
                if (this.ShouldApplyItemContainerStyle(container, item)) {
                    this.ApplyItemContainerStyle(container, item);
                }

                this.PrepareContainerForItemOverride(container, item);
            },
            OnApplyTemplate: function () {
                Windows.UI.Xaml.Controls.Control.prototype.OnApplyTemplate.call(this);

                var itemsPresenter = Bridge.as(this.GetTemplateChild("ItemsHost"), Windows.UI.Xaml.Controls.ItemsPresenter);
                if (itemsPresenter == null) {
                    itemsPresenter = CSHTML5.Internal.INTERNAL_VisualTreeManager.GetChildOfType(Windows.UI.Xaml.Controls.ItemsPresenter, this);
                }

                if (itemsPresenter != null) {
                    this._itemsPresenter = itemsPresenter;
                    itemsPresenter.AttachToOwner(this);
                }
            },
            ManageCollectionChanged: function (e) {
                this.HandleItemsChanged(e);
            },
            UpdateItemsPanel: function (newTemplate) {
                if (this.ItemsPresenter != null) {
                    this.ItemsPresenter.Template = newTemplate;
                }
            },
            OnItemsSourceChanged: function (oldValue, newValue) {
                this.OnItemsSourceChanged_BeforeVisualUpdate(oldValue, newValue);
            },
            OnItemsSourceChanged_BeforeVisualUpdate: function (oldValue, newValue) { },
            /**
             * Called when the value of the {@link } property changes.
             *
             * @instance
             * @protected
             * @this Windows.UI.Xaml.Controls.ItemsControl
             * @memberof Windows.UI.Xaml.Controls.ItemsControl
             * @param   {System.Collections.Specialized.NotifyCollectionChangedEventArgs}    e    A {@link } that contains the event data
             * @return  {void}
             */
            OnItemsChanged: function (e) { },
            INTERNAL_OnAttachedToVisualTree: function () {
                Windows.UI.Xaml.Controls.Control.prototype.INTERNAL_OnAttachedToVisualTree.call(this);

                if (!this.HasTemplate) {
                    this._itemsPresenter = new Windows.UI.Xaml.Controls.ItemsPresenter();
                    this._itemsPresenter.AttachToOwner(this);

                    CSHTML5.Internal.INTERNAL_VisualTreeManager.AttachVisualChildIfNotAlreadyAttached(this._itemsPresenter, this);

                    this._renderedControlTemplate = this._itemsPresenter;
                }
            },
            /**
             * Determine whether the ItemContainerStyle/StyleSelector should apply to the container
             *
             * @instance
             * @this Windows.UI.Xaml.Controls.ItemsControl
             * @memberof Windows.UI.Xaml.Controls.ItemsControl
             * @param   {Windows.UI.Xaml.DependencyObject}    container    
             * @param   {System.Object}                       item
             * @return  {boolean}                                          true if the ItemContainerStyle should apply to the item
             */
            ShouldApplyItemContainerStyle: function (container, item) {
                return true;
            },
            ApplyItemContainerStyle: function (container, item) {
                var feContainer = Bridge.as(container, Windows.UI.Xaml.FrameworkElement);

                var style = this.ItemContainerStyle;

                if (style != null) {
                    if (!Bridge.Reflection.isInstanceOfType(container, style.TargetType)) {
                        throw new System.InvalidOperationException.$ctor1(System.String.format("A style intended for type '{0}' cannot be applied to type '{1}'.", Bridge.Reflection.getTypeName(style.TargetType), Bridge.Reflection.getTypeName(Bridge.getType(container))));
                    }

                    feContainer.Style = style;
                }
            },
            HandleItemsChanged: function (e) {
                if (this.ItemsHost == null) {
                    return;
                }

                var host = Bridge.cast(this, CSHTML5.Internals.Controls.IGeneratorHost);

                if (e.Action === System.Collections.Specialized.NotifyCollectionChangedAction.Reset) {
                    this.Refresh();
                } else if (e.Action === System.Collections.Specialized.NotifyCollectionChangedAction.Add) {
                    var item = System.Array.getItem(e.NewItems, 0);
                    var container = host.CSHTML5$Internals$Controls$IGeneratorHost$GetContainerForItem(item, null);
                    if (!Bridge.referenceEquals(container, item)) {
                        container.SetValue(Windows.UI.Xaml.FrameworkElement.DataContextProperty, item);
                    }
                    host.CSHTML5$Internals$Controls$IGeneratorHost$PrepareItemContainer(container, item);

                    this.ItemContainerGenerator.INTERNAL_RegisterContainer(container, item);

                    this.ItemsHost.Children.insert(e.NewStartingIndex, Bridge.cast(container, Windows.UI.Xaml.UIElement));

                } else if (e.Action === System.Collections.Specialized.NotifyCollectionChangedAction.Remove) {
                    this.ItemContainerGenerator.INTERNAL_TryUnregisterContainer(this.ItemsHost.Children.getItem(e.OldStartingIndex), System.Array.getItem(e.OldItems, 0));

                    this.ItemsHost.Children.removeAt(e.OldStartingIndex);
                } else if (e.Action === System.Collections.Specialized.NotifyCollectionChangedAction.Replace) {
                    this.ItemContainerGenerator.INTERNAL_TryUnregisterContainer(this.ItemsHost.Children.getItem(e.OldStartingIndex), System.Array.getItem(e.OldItems, 0));

                    var item1 = System.Array.getItem(e.NewItems, 0);
                    var container1 = host.CSHTML5$Internals$Controls$IGeneratorHost$GetContainerForItem(item1, null);
                    if (!Bridge.referenceEquals(container1, item1)) {
                        container1.SetValue(Windows.UI.Xaml.FrameworkElement.DataContextProperty, item1);
                    }
                    host.CSHTML5$Internals$Controls$IGeneratorHost$PrepareItemContainer(container1, item1);

                    this.ItemContainerGenerator.INTERNAL_RegisterContainer(container1, item1);

                    this.ItemsHost.Children.setItem(e.OldStartingIndex, Bridge.cast(container1, Windows.UI.Xaml.UIElement));
                } else {
                    throw new System.NotSupportedException.$ctor1(System.String.format("Unexpected collection change action '{0}'.", [Bridge.box(e.Action, System.Collections.Specialized.NotifyCollectionChangedAction, System.Enum.toStringFn(System.Collections.Specialized.NotifyCollectionChangedAction))]));
                }
            },
            OnItemCollectionChanged: function (sender, e) {
                this.ManageCollectionChanged(e);

                this.OnItemsChanged(e);
            },
            /**
             * Undoes the effects of the PrepareContainerForItemOverride method.
             *
             * @instance
             * @protected
             * @this Windows.UI.Xaml.Controls.ItemsControl
             * @memberof Windows.UI.Xaml.Controls.ItemsControl
             * @param   {Windows.UI.Xaml.DependencyObject}    element    The container element.
             * @param   {System.Object}                       item       The item.
             * @return  {void}
             */
            ClearContainerForItemOverride: function (element, item) {
                var cc;
                var cp;

                if (((cc = Bridge.as(element, Windows.UI.Xaml.Controls.ContentControl))) != null) {
                    Windows.UI.Xaml.Controls.ItemsControl.ClearContentControl(cc, item);
                } else if (((cp = Bridge.as(element, Windows.UI.Xaml.Controls.ContentPresenter))) != null) {
                    Windows.UI.Xaml.Controls.ItemsControl.ClearContentPresenter(cp, item);
                }
            },
            /**
             * Creates or identifies the element that is used to display the given item.
             *
             * @instance
             * @protected
             * @this Windows.UI.Xaml.Controls.ItemsControl
             * @memberof Windows.UI.Xaml.Controls.ItemsControl
             * @return  {Windows.UI.Xaml.DependencyObject}        The element that is used to display the given item.
             */
            GetContainerForItemOverride: function () {
                return new Windows.UI.Xaml.Controls.ContentPresenter();
            },
            /**
             * Determines whether the specified item is (or is eligible to be) its 
             own container.
             *
             * @instance
             * @protected
             * @this Windows.UI.Xaml.Controls.ItemsControl
             * @memberof Windows.UI.Xaml.Controls.ItemsControl
             * @param   {System.Object}    item    The item to check.
             * @return  {boolean}                  True if the item is (or is eligible to be) its own container; otherwise,
             false.
             */
            IsItemItsOwnContainerOverride: function (item) {
                return (Bridge.is(item, Windows.UI.Xaml.UIElement));
            },
            /**
             * Prepares the specified element to display the specified item.
             *
             * @instance
             * @protected
             * @this Windows.UI.Xaml.Controls.ItemsControl
             * @memberof Windows.UI.Xaml.Controls.ItemsControl
             * @param   {Windows.UI.Xaml.DependencyObject}    element    The element that's used to display the specified item.
             * @param   {System.Object}                       item       The item to display.
             * @return  {void}
             */
            PrepareContainerForItemOverride: function (element, item) {
                var cc;
                var cp;

                if (this.ItemTemplate != null && !System.String.isNullOrWhiteSpace(this.DisplayMemberPath)) {
                    throw new System.InvalidOperationException.$ctor1("Cannot set both DisplayMemberPath and ItemTemplate.");
                }

                var template = null;
                if (!(Bridge.is(item, Windows.UI.Xaml.UIElement))) {
                    template = this.ItemTemplate;
                    if (template == null) {
                        template = Windows.UI.Xaml.Controls.ItemsControl.GetDataTemplateForDisplayMemberPath(this.DisplayMemberPath);
                    }
                }

                if (((cc = Bridge.as(element, Windows.UI.Xaml.Controls.ContentControl))) != null) {
                    Windows.UI.Xaml.Controls.ItemsControl.PrepareContentControl(cc, item, template);
                } else if (((cp = Bridge.as(element, Windows.UI.Xaml.Controls.ContentPresenter))) != null) {
                    Windows.UI.Xaml.Controls.ItemsControl.PrepareContentPresenter(cp, item, template);
                }
            },
            /**
             * Derived classes can call this methed in their constructor if they 
             want to disable the default rendering of the ItemsControl. It can 
             be useful for example to replace the rendering with a custom 
             HTML-based one.
             *
             * @instance
             * @protected
             * @this Windows.UI.Xaml.Controls.ItemsControl
             * @memberof Windows.UI.Xaml.Controls.ItemsControl
             * @return  {void}
             */
            DisableDefaultRendering: function () { },
            /**
             * Returns the item itself if the item is already a container of the 
             correct type, otherwise it returns null if no container is to be 
             created, or it returns the new container otherwise.
             *
             * @instance
             * @protected
             * @this Windows.UI.Xaml.Controls.ItemsControl
             * @memberof Windows.UI.Xaml.Controls.ItemsControl
             * @param   {System.Object}                                       item    The item to generate the container with.
             * @return  {Windows.UI.Xaml.Controls.Primitives.SelectorItem}            Returns the item itself if the item is already a container of the 
             correct type, otherwise it returns null if no container is to be 
             created, or it returns the new container otherwise.
             */
            INTERNAL_GenerateContainer: function (item) {
                return Bridge.cast(this.GetContainerFromItem(item), Windows.UI.Xaml.Controls.Primitives.SelectorItem);
            },
            OnChildItemRemoved: function (item) { },
            GenerateFrameworkElementToRenderTheItem: function (item) {
                var result = Bridge.as(item, Windows.UI.Xaml.FrameworkElement);
                if (result == null) {
                    var displayElement = Windows.UI.Xaml.PropertyPathHelper.AccessValueByApplyingPropertyPathIfAny(item, this.DisplayMemberPath);
                    if (this.ItemTemplate != null) {

                        result = this.ItemTemplate.INTERNAL_InstantiateFrameworkTemplate();
                        result.DataContext = displayElement;
                    } else {

                        var container = new Windows.UI.Xaml.Controls.ContentPresenter();
                        var b = new Windows.UI.Xaml.Data.Binding.$ctor1(this.DisplayMemberPath);
                        container.SetBinding(Windows.UI.Xaml.Controls.ContentControl.ContentProperty, b);
                        container.DataContext = item;
                        result = container;
                    }
                }
                return result;
            },
            /**
             * Create or identify the element used to display the given item.
             *
             * @instance
             * @protected
             * @this Windows.UI.Xaml.Controls.ItemsControl
             * @memberof Windows.UI.Xaml.Controls.ItemsControl
             * @param   {System.Object}                       item
             * @return  {Windows.UI.Xaml.DependencyObject}            The element that is used to display the given item.
             */
            GetContainerFromItem: function (item) {
                return null;
            }
        }
    });

    /**
     * Represents a control with a single piece of content. Controls such as Button,
     CheckBox, and ScrollViewer directly or indirectly inherit from this class.
     *
     * @public
     * @class Windows.UI.Xaml.Controls.ContentControl
     * @augments Windows.UI.Xaml.Controls.Control
     */
    Bridge.define("Windows.UI.Xaml.Controls.ContentControl", {
        inherits: [Windows.UI.Xaml.Controls.Control],
        statics: {
            fields: {
                /**
                 * Identifies the Content dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Controls.ContentControl
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                ContentProperty: null,
                /**
                 * Identifies the ContentTemplate dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Controls.ContentControl
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                ContentTemplateProperty: null
            },
            ctors: {
                init: function () {
                    this.ContentProperty = Windows.UI.Xaml.DependencyProperty.Register("Content", System.Object, Windows.UI.Xaml.Controls.ContentControl, new Windows.UI.Xaml.PropertyMetadata.$ctor2(null, Windows.UI.Xaml.Controls.ContentControl.OnContentPropertyChanged));
                    this.ContentTemplateProperty = Windows.UI.Xaml.DependencyProperty.Register("ContentTemplate", Windows.UI.Xaml.DataTemplate, Windows.UI.Xaml.Controls.ContentControl, new Windows.UI.Xaml.PropertyMetadata.$ctor2(null, Windows.UI.Xaml.Controls.ContentControl.OnContentTemplatePropertyChanged));
                }
            },
            methods: {
                OnContentPropertyChanged: function (d, e) {
                    Bridge.cast(d, Windows.UI.Xaml.Controls.ContentControl).OnContentChanged(e.OldValue, e.NewValue);
                },
                OnContentTemplatePropertyChanged: function (d, e) {
                    var contentControl = Bridge.cast(d, Windows.UI.Xaml.Controls.ContentControl);
                    if (!contentControl.IsLoaded) {
                        return;
                    }
                    if (contentControl.HasTemplate) {
                        return;
                    }
                    contentControl.ApplyContentTemplate(Bridge.cast(e.NewValue, Windows.UI.Xaml.DataTemplate));
                }
            }
        },
        fields: {
            Child: null
        },
        props: {
            /**
             * Gets or sets the content of a ContentControl.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.ContentControl
             * @function Content
             * @type System.Object
             */
            Content: {
                get: function () {
                    return this.GetValue(Windows.UI.Xaml.Controls.ContentControl.ContentProperty);
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Controls.ContentControl.ContentProperty, value);
                }
            },
            /**
             * Gets or sets the data template that is used to display the content of the
             ContentControl.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.ContentControl
             * @function ContentTemplate
             * @type Windows.UI.Xaml.DataTemplate
             */
            ContentTemplate: {
                get: function () {
                    return Bridge.cast(this.GetValue(Windows.UI.Xaml.Controls.ContentControl.ContentTemplateProperty), Windows.UI.Xaml.DataTemplate);
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Controls.ContentControl.ContentTemplateProperty, value);
                }
            }
        },
        ctors: {
            ctor: function () {
                this.$initialize();
                Windows.UI.Xaml.Controls.Control.ctor.call(this);

            }
        },
        methods: {
            OnContentChanged: function (oldContent, newContent) {
                if (!this.IsLoaded) {
                    return;
                }
                if (this.HasTemplate) {
                    return;
                }
                if (this.ContentTemplate != null) {
                    if (this.Child != null) {
                        Bridge.cast(this.Child, Windows.UI.Xaml.FrameworkElement).DataContext = newContent;
                    }
                } else {
                    this.ChangeChild(newContent);
                }
            },
            INTERNAL_OnAttachedToVisualTree: function () {
                Windows.UI.Xaml.Controls.Control.prototype.INTERNAL_OnAttachedToVisualTree.call(this);

                if (this.HasTemplate) {
                    return;
                }
                this.ApplyContentTemplate(this.ContentTemplate);
            },
            ApplyContentTemplate: function (template) {
                var newChild;
                if (template == null) {
                    newChild = this.Content;
                } else {
                    var generatedContent = template.INTERNAL_InstantiateFrameworkTemplate();
                    if (generatedContent != null) {
                        generatedContent.DataContext = this.Content;
                    }
                    newChild = generatedContent;
                }
                this.ChangeChild(newChild);
            },
            ChangeChild: function (newChild) {
                this.DetachChild();

                var newChildAsUIElement = Bridge.as(newChild, Windows.UI.Xaml.UIElement);
                if (newChildAsUIElement != null) {
                    CSHTML5.Internal.INTERNAL_VisualTreeManager.AttachVisualChildIfNotAlreadyAttached(newChildAsUIElement, this);
                    this.Child = newChildAsUIElement;
                } else {
                    var contentAsString = newChild == null ? "" : Bridge.toString(newChild);
                    CSHTML5.Internal.INTERNAL_HtmlDomManager.SetContentString(this, contentAsString, true);
                    this.Child = this;
                }
            },
            DetachChild: function () {
                if (Bridge.referenceEquals(this, this.Child)) {
                    CSHTML5.Internal.INTERNAL_HtmlDomManager.SetContentString(this, "", true);
                } else {
                    CSHTML5.Internal.INTERNAL_VisualTreeManager.DetachVisualChildIfNotNull(this.Child, this);
                }
                this.Child = null;
            }
        }
    });

    Bridge.define("Windows.UI.Xaml.Controls.Calendar", {
        inherits: [Windows.UI.Xaml.Controls.INTERNAL_CalendarOrClockBase],
        statics: {
            methods: {
                WorkaroundJSILBug: function (calendarInstance, year, month, day) {
                    Bridge.cast(calendarInstance, Windows.UI.Xaml.Controls.Calendar).OnJavaScriptEvent_Change(year, month, day);
                },
                WorkaroundJSILBugForOnMonthChange: function (calendarInstance) {
                    Bridge.cast(calendarInstance, Windows.UI.Xaml.Controls.Calendar).OnMonthChange();
                }
            }
        },
        fields: {
            isLoaded: false,
            _isTodayHighlighted: false,
            _displayDateStart: null,
            _displayDateEnd: null,
            _displayDate: null
        },
        props: {
            /**
             * Gets or sets a value indicating whether the current date is highlighted.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.Calendar
             * @function IsTodayHighlighted
             * @type boolean
             */
            IsTodayHighlighted: {
                get: function () {
                    return this._isTodayHighlighted;
                },
                set: function (value) {
                    this.RefreshTodayHighlight(value);
                    this._isTodayHighlighted = value;
                }
            },
            /**
             * Gets or sets the first date to be displayed (enabled)
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.Calendar
             * @function DisplayDateStart
             * @type ?System.DateTime
             */
            DisplayDateStart: {
                get: function () {
                    return this._displayDateStart;
                },
                set: function (value) {
                    this.SetDisplayDateStart(value);
                    this._displayDateStart = value;
                }
            },
            /**
             * Gets or sets the last date to be displayed (enabled)
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.Calendar
             * @function DisplayDateEnd
             * @type ?System.DateTime
             */
            DisplayDateEnd: {
                get: function () {
                    return this._displayDateEnd;
                },
                set: function (value) {
                    this.SetDisplayDateEnd(value);
                    this._displayDateEnd = value;
                }
            },
            /**
             * Gets or sets the date to display
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.Calendar
             * @function DisplayDate
             * @type ?System.DateTime
             */
            DisplayDate: {
                get: function () {
                    return this._displayDate;
                },
                set: function (value) {
                    this.SetDisplayDate(value);
                    this._displayDate = value;
                }
            }
        },
        ctors: {
            init: function () {
                this.isLoaded = false;
                this._isTodayHighlighted = true;
            },
            ctor: function () {
                this.$initialize();
                Windows.UI.Xaml.Controls.INTERNAL_CalendarOrClockBase.ctor.call(this);
                this.addLoaded(Bridge.fn.cacheBind(this, this.DatePicker_Loaded));
            }
        },
        methods: {
            DatePicker_Loaded: function (sender, e) {
                var defaultDate = Bridge.equals(this.SelectedValue, null) ? System.DateTime.getToday() : System.Nullable.getValue(this.SelectedValue);

                this.isLoaded = true;

                var div = CSHTML5.Interop.GetDiv(this);

                this._flatpickrInstance = flatpickr(div, {
                inline: true, 
                dateFormat: "YYYY-MM-DD HH:MM",
                defaultDate: new Date(System.DateTime.getYear(defaultDate), ((System.DateTime.getMonth(defaultDate) - 1) | 0), System.DateTime.getDay(defaultDate))
                });

                if (CSHTML5.Interop.IsRunningInTheSimulator) {
                    this._flatpickrInstance.config.onChange.push(function(args) {
                    var date = args[0];
                    if(date !== undefined)
                    {
                        var month = date.getMonth() + 1;
                        var day = date.getDate();
                        var year = date.getFullYear();
                        Bridge.fn.cacheBind(this, this.OnJavaScriptEvent_Change)(year, month, day);
                    }
                });

                    this._flatpickrInstance.config.onMonthChange.push(function(args) {
                    Bridge.fn.cacheBind(this, this.OnMonthChange)();
                });


                } else {

                    var calendar = this;

                    this._flatpickrInstance.config.onChange.push(function(args) {
                    var date = args[0];
                    if(date !== undefined)
                    {
                        var month = date.getMonth() + 1;
                        var day = date.getDate();
                        var year = date.getFullYear();
                        Windows.UI.Xaml.Controls.Calendar.WorkaroundJSILBug(calendar, year, month, day);
                    }
                });

                    this._flatpickrInstance.config.onMonthChange.push(function(args) {
                    Windows.UI.Xaml.Controls.Calendar.WorkaroundJSILBugForOnMonthChange(calendar);
                });
                }

                this.RefreshTodayHighlight(this.IsTodayHighlighted);

                div.style.display = 'none';
            },
            OnJavaScriptEvent_Change: function (year, month, day) {

                var intYear = System.Int32.parse(Bridge.toString(year));
                var intMonth = System.Int32.parse(Bridge.toString(month));
                var intDay = System.Int32.parse(Bridge.toString(day));

                var dateTime = System.DateTime.create(intYear, intMonth, intDay);

                this.SelectedValue = Bridge.cast(dateTime, System.DateTime, true);
                this.RefreshTodayHighlight(this.IsTodayHighlighted);
            },
            OnMonthChange: function () {
                this.RefreshTodayHighlight(this.IsTodayHighlighted);
            },
            /**
             * This method hides or displays the highlighting around the date of today.
             *
             * @instance
             * @private
             * @this Windows.UI.Xaml.Controls.Calendar
             * @memberof Windows.UI.Xaml.Controls.Calendar
             * @param   {boolean}    isTodayHighlighted    True if there should be a circle highlighting the date of today in the calendar, False otherwise
             * @return  {void}
             */
            RefreshTodayHighlight: function (isTodayHighlighted) {
                if (this.isLoaded) {
                    var c = "transparent";
                    if (isTodayHighlighted) {
                        c = "";
                    }
                    var todaySpan = this._flatpickrInstance.calendarContainer.querySelector('span.today');
                    if (todaySpan != null) {
                        todaySpan.style.borderColor = c;
                    }
                }
            },
            SetDisplayDateStart: function (dateStart) {
                if (Bridge.equals(dateStart, null)) {
                    this._flatpickrInstance.config.minDate = undefined;
                } else {
                    var nonNullDateStart = System.Nullable.getValue(dateStart);
                    this._flatpickrInstance.config.minDate = new Date(System.DateTime.getYear(nonNullDateStart), ((System.DateTime.getMonth(nonNullDateStart) - 1) | 0), System.DateTime.getDay(nonNullDateStart));
                }
            },
            SetDisplayDateEnd: function (dateEnd) {
                if (Bridge.equals(dateEnd, null)) {
                    this._flatpickrInstance.config.maxDate = undefined;
                } else {
                    var nonNullDateEnd = System.Nullable.getValue(dateEnd);
                    this._flatpickrInstance.config.maxDate = new Date(System.DateTime.getYear(nonNullDateEnd), ((System.DateTime.getMonth(nonNullDateEnd) - 1) | 0), System.DateTime.getDay(nonNullDateEnd));
                }
            },
            SetDisplayDate: function (dateTime) {
                if (Bridge.equals(dateTime, null)) {
                    this._flatpickrInstance.config.maxDate = undefined;
                } else {
                    var nonNullDate = System.Nullable.getValue(dateTime);
                    if ((!Bridge.equals(this.DisplayDateStart, null) && System.DateTime.lt(dateTime, this.DisplayDateStart)) || (!Bridge.equals(this.DisplayDateEnd, null) && System.DateTime.gt(dateTime, this.DisplayDateEnd))) {
                        throw new System.ArgumentOutOfRangeException.$ctor1("The given date is not in the range specified by System.Windows.Controls.Calendar.DisplayDateStart and System.Windows.Controls.Calendar.DisplayDateEnd");
                    }
                    this._flatpickrInstance.jumpToDate(new Date(System.DateTime.getYear(nonNullDate), ((System.DateTime.getMonth(nonNullDate) - 1) | 0), System.DateTime.getDay(nonNullDate)));
                }
            }
        }
    });

    /**
     * Defines an area within which you can explicitly position child objects, using
     coordinates that are relative to the Canvas area.
     *
     * @public
     * @class Windows.UI.Xaml.Controls.Canvas
     * @augments Windows.UI.Xaml.Controls.Panel
     * @example You can add a Canvas to the XAML as follows:
 
     Or in C#:
     *
     * 
     *         
     * 
     * 
     *
     * Canvas myCanvas = new Canvas();
     * myCanvas.Width = 100;
     * myCanvas.Height = 100;
     * myCanvas.Background = new SolidColorBrush(Windows.UI.Colors.Blue);
     * myCanvas.HorizontalAlignment=HorizontalAlignment.Left;
     * 
     *
     *
     */
    Bridge.define("Windows.UI.Xaml.Controls.Canvas", {
        inherits: [Windows.UI.Xaml.Controls.Panel],
        statics: {
            fields: {
                /**
                 * Identifies the Canvas.Left XAML attached property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Controls.Canvas
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                LeftProperty: null,
                /**
                 * Identifies the Canvas.Top XAML attached property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Controls.Canvas
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                TopProperty: null,
                /**
                 * Identifies the Canvas.ZIndex XAML attached property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Controls.Canvas
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                ZIndexProperty: null
            },
            ctors: {
                init: function () {
                    var $t;
                    this.LeftProperty = Windows.UI.Xaml.DependencyProperty.RegisterAttached("Left", System.Double, Windows.UI.Xaml.UIElement, ($t = new Windows.UI.Xaml.PropertyMetadata.$ctor1(Bridge.box(0.0, System.Double, System.Double.format, System.Double.getHashCode)), $t.GetCSSEquivalent = function (instance) {
                        var $t1;
                        var uielement = Bridge.as(instance, Windows.UI.Xaml.UIElement);
                        if (uielement != null && Bridge.is(uielement.INTERNAL_VisualParent, Windows.UI.Xaml.Controls.Canvas)) {
                            return ($t1 = new Windows.UI.Xaml.CSSEquivalent(), $t1.Value = function (inst, value) {
                                return (Bridge.toString(value) || "") + "px";
                            }, $t1.Name = function (_o1) {
                                    _o1.add("left");
                                    return _o1;
                                }(new (System.Collections.Generic.List$1(System.String)).ctor()), $t1.DomElement = uielement.INTERNAL_AdditionalOutsideDivForMargins, $t1.ApplyAlsoWhenThereIsAControlTemplate = true, $t1);
                        } else {
                            return null;
                        }
                    }, $t));
                    this.TopProperty = Windows.UI.Xaml.DependencyProperty.RegisterAttached("Top", System.Double, Windows.UI.Xaml.UIElement, ($t = new Windows.UI.Xaml.PropertyMetadata.$ctor1(Bridge.box(0.0, System.Double, System.Double.format, System.Double.getHashCode)), $t.GetCSSEquivalent = function (instance) {
                        var $t1;
                        var uielement = Bridge.as(instance, Windows.UI.Xaml.UIElement);
                        if (uielement != null && Bridge.is(uielement.INTERNAL_VisualParent, Windows.UI.Xaml.Controls.Canvas)) {
                            return ($t1 = new Windows.UI.Xaml.CSSEquivalent(), $t1.Value = function (inst, value) {
                                return (Bridge.toString(value) || "") + "px";
                            }, $t1.Name = function (_o2) {
                                    _o2.add("top");
                                    return _o2;
                                }(new (System.Collections.Generic.List$1(System.String)).ctor()), $t1.DomElement = uielement.INTERNAL_AdditionalOutsideDivForMargins, $t1.ApplyAlsoWhenThereIsAControlTemplate = true, $t1);
                        } else {
                            return null;
                        }
                    }, $t));
                    this.ZIndexProperty = Windows.UI.Xaml.DependencyProperty.RegisterAttached("ZIndex", System.Int32, Windows.UI.Xaml.UIElement, ($t = new Windows.UI.Xaml.PropertyMetadata.$ctor1(Bridge.box(0, System.Int32)), $t.GetCSSEquivalent = function (instance) {
                        var $t1;
                        return ($t1 = new Windows.UI.Xaml.CSSEquivalent(), $t1.Value = function (inst, value) {
                            return Bridge.toString(value);
                        }, $t1.Name = function (_o3) {
                                _o3.add("zIndex");
                                return _o3;
                            }(new (System.Collections.Generic.List$1(System.String)).ctor()), $t1.DomElement = Bridge.cast(instance, Windows.UI.Xaml.UIElement).INTERNAL_AdditionalOutsideDivForMargins, $t1.ApplyAlsoWhenThereIsAControlTemplate = true, $t1);
                    }, $t));
                }
            },
            methods: {
                /**
                 * Sets the value of the Canvas.Left XAML attached property for a target element.
                 *
                 * @static
                 * @public
                 * @this Windows.UI.Xaml.Controls.Canvas
                 * @memberof Windows.UI.Xaml.Controls.Canvas
                 * @param   {Windows.UI.Xaml.UIElement}    element    The object to which the property value is written.
                 * @param   {number}                       value      The value to set.
                 * @return  {void}
                 */
                SetLeft: function (element, value) {
                    element.SetValue(Windows.UI.Xaml.Controls.Canvas.LeftProperty, Bridge.box(value, System.Double, System.Double.format, System.Double.getHashCode));
                },
                /**
                 * Gets the value of the Canvas.Left XAML attached property for the target element.
                 *
                 * @static
                 * @public
                 * @this Windows.UI.Xaml.Controls.Canvas
                 * @memberof Windows.UI.Xaml.Controls.Canvas
                 * @param   {Windows.UI.Xaml.UIElement}    element    The object from which the property value is read.
                 * @return  {number}                                  The Canvas.Left XAML attached property value of the specified object.
                 */
                GetLeft: function (element) {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(element.GetValue(Windows.UI.Xaml.Controls.Canvas.LeftProperty), System.Double), System.Double));
                },
                /**
                 * Sets the value of the Canvas.Top XAML attached property for a target element.
                 *
                 * @static
                 * @public
                 * @this Windows.UI.Xaml.Controls.Canvas
                 * @memberof Windows.UI.Xaml.Controls.Canvas
                 * @param   {Windows.UI.Xaml.UIElement}    element    The object to which the property value is written.
                 * @param   {number}                       value      The value to set.
                 * @return  {void}
                 */
                SetTop: function (element, value) {
                    element.SetValue(Windows.UI.Xaml.Controls.Canvas.TopProperty, Bridge.box(value, System.Double, System.Double.format, System.Double.getHashCode));
                },
                /**
                 * Gets the value of the Canvas.Top XAML attached property for the target element.
                 *
                 * @static
                 * @public
                 * @this Windows.UI.Xaml.Controls.Canvas
                 * @memberof Windows.UI.Xaml.Controls.Canvas
                 * @param   {Windows.UI.Xaml.UIElement}    element    The object from which the property value is read.
                 * @return  {number}                                  The Canvas.Top XAML attached property value of the specified object.
                 */
                GetTop: function (element) {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(element.GetValue(Windows.UI.Xaml.Controls.Canvas.TopProperty), System.Double), System.Double));
                },
                /**
                 * Sets the value of the Canvas.ZIndex XAML attached property for a target element.
                 *
                 * @static
                 * @public
                 * @this Windows.UI.Xaml.Controls.Canvas
                 * @memberof Windows.UI.Xaml.Controls.Canvas
                 * @param   {Windows.UI.Xaml.UIElement}    element    The object to which the property value is written.
                 * @param   {number}                       value      The value to set.
                 * @return  {void}
                 */
                SetZIndex: function (element, value) {
                    element.SetValue(Windows.UI.Xaml.Controls.Canvas.ZIndexProperty, Bridge.box(value, System.Int32));
                },
                /**
                 * Gets the value of the Canvas.ZIndex XAML attached property for the target element.
                 *
                 * @static
                 * @public
                 * @this Windows.UI.Xaml.Controls.Canvas
                 * @memberof Windows.UI.Xaml.Controls.Canvas
                 * @param   {Windows.UI.Xaml.UIElement}    element    The object from which the property value is read.
                 * @return  {number}                                  The Canvas.ZIndex XAML attached property value of the specified object.
                 */
                GetZIndex: function (element) {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(element.GetValue(Windows.UI.Xaml.Controls.Canvas.ZIndexProperty), System.Int32), System.Int32));
                }
            }
        },
        methods: {
            CreateDomElement: function (parentRef, domElementWhereToPlaceChildren) {
                var div = CSHTML5.Internal.INTERNAL_HtmlDomManager.CreateDomElementAndAppendIt("div", parentRef, this);
                var style = CSHTML5.Internal.INTERNAL_HtmlDomManager.GetDomElementStyleForModification(div);
                style.overflow = "display";
                style.position = "relative";

                domElementWhereToPlaceChildren.v = div;
                return div;




                /* -------------------------------
                  A canvas should look like this:
                  -------------------------------
                  <div style="width: 50px; height: 50px;">
                      <div style="position:relative"> width & height are the size of the canvas itself
                          ... children (with position: absolute), below are two example of children
                          <div style="background-color: rgb(200,0,200);width:20px;height:20px;  position: absolute"></div>
                          <div style="background-color: rgb(100,0,200);width:20px;height:20px;margin-left:20px;margin-right:auto;  position: absolute"></div>
                      </div>
                  </div>
                */

            }
        }
    });

    Bridge.define("Windows.UI.Xaml.Controls.Clock", {
        inherits: [Windows.UI.Xaml.Controls.INTERNAL_CalendarOrClockBase],
        statics: {
            methods: {
                WorkaroundJSILBug: function (clockInstance, hours, minutes) {
                    Bridge.cast(clockInstance, Windows.UI.Xaml.Controls.Clock).OnJavaScriptEvent_Change(hours, minutes);
                }
            }
        },
        ctors: {
            ctor: function () {
                this.$initialize();
                Windows.UI.Xaml.Controls.INTERNAL_CalendarOrClockBase.ctor.call(this);
                this.addLoaded(Bridge.fn.cacheBind(this, this.TimePicker_Loaded));
            }
        },
        methods: {
            TimePicker_Loaded: function (sender, e) {
                var defaultDate = Bridge.equals(this.SelectedValue, null) ? System.DateTime.getNow() : System.Nullable.getValue(this.SelectedValue);

                var div = CSHTML5.Interop.GetDiv(this);

                this._flatpickrInstance = flatpickr(div, {
                    inline: true,
                    enableTime: true,
                    noCalendar: true,
                    time_24hr: true,
                    dateFormat: "YYYY-MM-DD HH:MM",
                    defaultDate: new Date(1, 1, 1, Bridge.box(System.DateTime.getHour(defaultDate), System.Int32), Bridge.box(((System.DateTime.getMinute(defaultDate) - (System.DateTime.getMinute(defaultDate) % 5)) | 0), System.Int32))
                    });


                if (CSHTML5.Interop.IsRunningInTheSimulator) {
                    this._flatpickrInstance.config.onChange.push(function(args) {
                var date = args[0];
                var hours = date.getHours();
                var minutes = date.getMinutes();
                Bridge.fn.cacheBind(this, this.OnJavaScriptEvent_Change)(hours, minutes);
                });
                } else {

                    var clock = this;

                    this._flatpickrInstance.config.onChange.push(function(args) {
                var date = args[0];
                var hours = date.getHours();
                var minutes = date.getMinutes();
                Windows.UI.Xaml.Controls.Clock.WorkaroundJSILBug(clock, hours, minutes);
                });
                }

                div.style.display = 'none';
            },
            OnJavaScriptEvent_Change: function (hours, minutes) {

                var newDate = System.DateTime.create(2016, 10, 20, System.Int32.parse(Bridge.toString(hours)), System.Int32.parse(Bridge.toString(minutes)), 20);
                this.SelectedValue = Bridge.cast(newDate, System.DateTime, true);
            }
        }
    });

    Bridge.define("Windows.UI.Xaml.Controls.DataPager", {
        inherits: [Windows.UI.Xaml.Controls.Control],
        statics: {
            fields: {
                /**
                 * Identifies the dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Controls.DataPager
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                DisplayModeProperty: null,
                /**
                 * Identifies the dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Controls.DataPager
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                NumericButtonCountProperty: null,
                /**
                 * Identifies the dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Controls.DataPager
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                SourceProperty: null,
                /**
                 * Identifies the dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Controls.DataPager
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                PageSizeProperty: null,
                /**
                 * Identifies the dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Controls.DataPager
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                CurrentPageProperty: null,
                /**
                 * Identifies the Style dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Controls.DataPager
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                NumericButtonStyleProperty: null
            },
            ctors: {
                init: function () {
                    var $t;
                    this.DisplayModeProperty = Windows.UI.Xaml.DependencyProperty.Register("DisplayMode", Windows.UI.Xaml.Controls.DataPager_DisplayMode, Windows.UI.Xaml.Controls.DataPager, ($t = new Windows.UI.Xaml.PropertyMetadata.$ctor2(Bridge.box(Windows.UI.Xaml.Controls.DataPager_DisplayMode.Numeric, Windows.UI.Xaml.Controls.DataPager_DisplayMode, System.Enum.toStringFn(Windows.UI.Xaml.Controls.DataPager_DisplayMode)), Windows.UI.Xaml.Controls.DataPager.OnDisplayModeChanged), $t.CallPropertyChangedWhenLoadedIntoVisualTree = Windows.UI.Xaml.WhenToCallPropertyChangedEnum.IfPropertyIsSet, $t));
                    this.NumericButtonCountProperty = Windows.UI.Xaml.DependencyProperty.Register("NumericButtonCount", System.Int32, Windows.UI.Xaml.Controls.DataPager, ($t = new Windows.UI.Xaml.PropertyMetadata.$ctor2(Bridge.box(0, System.Int32), Windows.UI.Xaml.Controls.DataPager.OnNumericButtonCountChanged), $t.CallPropertyChangedWhenLoadedIntoVisualTree = Windows.UI.Xaml.WhenToCallPropertyChangedEnum.IfPropertyIsSet, $t));
                    this.SourceProperty = Windows.UI.Xaml.DependencyProperty.Register("Source", Windows.UI.Xaml.Data.INTERNAL_PagedCollectionView, Windows.UI.Xaml.Controls.DataPager, new Windows.UI.Xaml.PropertyMetadata.$ctor4(Windows.UI.Xaml.Controls.DataPager.OnSourceChanged));
                    this.PageSizeProperty = Windows.UI.Xaml.DependencyProperty.Register("PageSize", System.Int32, Windows.UI.Xaml.Controls.DataPager, ($t = new Windows.UI.Xaml.PropertyMetadata.$ctor2(Bridge.box(-1, System.Int32), Windows.UI.Xaml.Controls.DataPager.OnPageSizeChanged), $t.CallPropertyChangedWhenLoadedIntoVisualTree = Windows.UI.Xaml.WhenToCallPropertyChangedEnum.IfPropertyIsSet, $t));
                    this.CurrentPageProperty = Windows.UI.Xaml.DependencyProperty.Register("CurrentPage", System.Int32, Windows.UI.Xaml.Controls.DataPager, ($t = new Windows.UI.Xaml.PropertyMetadata.$ctor2(Bridge.box(0, System.Int32), Windows.UI.Xaml.Controls.DataPager.OnCurrentPageChanged), $t.CallPropertyChangedWhenLoadedIntoVisualTree = Windows.UI.Xaml.WhenToCallPropertyChangedEnum.IfPropertyIsSet, $t));
                    this.NumericButtonStyleProperty = Windows.UI.Xaml.DependencyProperty.Register("NumericButtonStyle", Windows.UI.Xaml.Style, Windows.UI.Xaml.Controls.DataPager, new Windows.UI.Xaml.PropertyMetadata.$ctor4(null));
                }
            },
            methods: {
                OnDisplayModeChanged: function (d, e) {
                    var datapager = Bridge.cast(d, Windows.UI.Xaml.Controls.DataPager);

                    if (datapager.Source != null && System.Nullable.getValue(Bridge.cast(Bridge.unbox(e.NewValue, Windows.UI.Xaml.Controls.DataPager_DisplayMode), System.Int32)) !== System.Nullable.getValue(Bridge.cast(Bridge.unbox(e.OldValue, Windows.UI.Xaml.Controls.DataPager_DisplayMode), System.Int32))) {
                        datapager.GenerateControls();
                    }
                },
                OnNumericButtonCountChanged: function (d, e) {
                    var datapager = Bridge.cast(d, Windows.UI.Xaml.Controls.DataPager);

                    if (datapager.Source != null && System.Nullable.getValue(Bridge.cast(Bridge.unbox(e.NewValue, System.Int32), System.Int32)) !== System.Nullable.getValue(Bridge.cast(Bridge.unbox(e.OldValue, System.Int32), System.Int32))) {
                        datapager.GenerateControls();
                    }
                },
                OnSourceChanged: function (d, e) {
                    var datapager = Bridge.cast(d, Windows.UI.Xaml.Controls.DataPager);

                    if (e.OldValue != null) {
                        Bridge.cast(e.OldValue, Windows.UI.Xaml.Data.INTERNAL_PagedCollectionView).removeCollectionChanged(Bridge.fn.cacheBind(datapager, datapager.Source_CollectionChanged));
                        Bridge.cast(e.OldValue, Windows.UI.Xaml.Data.INTERNAL_PagedCollectionView).removePageChanged(Bridge.fn.cacheBind(datapager, datapager.Source_PageChanged));
                    }

                    if (e.NewValue != null) {
                        datapager.Source.PageSize = datapager.PageSize;

                        datapager.NumberOfPages = datapager.CountNumberPage();

                        datapager.CurrentPage = datapager.CurrentPage;

                        datapager.Source.addCollectionChanged(Bridge.fn.cacheBind(datapager, datapager.Source_CollectionChanged));

                        datapager.Source.addPageChanged(Bridge.fn.cacheBind(datapager, datapager.Source_PageChanged));

                        datapager.GenerateControls();
                    } else {
                        datapager.NumberOfPages = 0;
                    }
                },
                /**
                 * the data PageSize
                 *
                 * @static
                 * @private
                 * @this Windows.UI.Xaml.Controls.DataPager
                 * @memberof Windows.UI.Xaml.Controls.DataPager
                 * @param   {Windows.UI.Xaml.DependencyObject}                      d    TempCanvas that changed its SelectedDate.
                 * @param   {Windows.UI.Xaml.DependencyPropertyChangedEventArgs}    e    The DependencyPropertyChangedEventArgs.
                 * @return  {void}
                 */
                OnPageSizeChanged: function (d, e) {
                    var datapager = Bridge.cast(d, Windows.UI.Xaml.Controls.DataPager);

                    if (e.NewValue != null && datapager.Source != null && System.Nullable.getValue(Bridge.cast(Bridge.unbox(e.NewValue, System.Int32), System.Int32)) !== System.Nullable.getValue(Bridge.cast(Bridge.unbox(e.OldValue, System.Int32), System.Int32))) {
                        datapager.Source.PageSize = System.Nullable.getValue(Bridge.cast(Bridge.unbox(e.NewValue, System.Int32), System.Int32));

                        datapager.NumberOfPages = datapager.CountNumberPage();
                    }
                },
                OnCurrentPageChanged: function (d, e) {
                    var datapager = Bridge.cast(d, Windows.UI.Xaml.Controls.DataPager);

                    if (e.NewValue != null && datapager.Source != null && System.Nullable.getValue(Bridge.cast(Bridge.unbox(e.NewValue, System.Int32), System.Int32)) !== System.Nullable.getValue(Bridge.cast(Bridge.unbox(e.OldValue, System.Int32), System.Int32))) {
                        var newIndex = datapager.GetNewCurrentPageIndex(System.Nullable.getValue(Bridge.cast(Bridge.unbox(e.NewValue, System.Int32), System.Int32)));

                        if (datapager.CurrentPage === newIndex) {
                            datapager.Source.MoveToPage(((System.Nullable.getValue(Bridge.cast(Bridge.unbox(e.NewValue, System.Int32), System.Int32)) - 1) | 0));
                        }

                        if (datapager._buttonPanel != null) {
                            datapager.RefreshControl();
                        }
                    }
                }
            }
        },
        fields: {
            _textBox: null,
            _buttonPanel: null,
            NumberOfPages: 0
        },
        props: {
            /**
             * Gets or sets the currently selected mode.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.DataPager
             * @function DisplayMode
             * @type Windows.UI.Xaml.Controls.DataPager_DisplayMode
             */
            DisplayMode: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.Controls.DataPager.DisplayModeProperty), Windows.UI.Xaml.Controls.DataPager_DisplayMode), System.Int32));
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Controls.DataPager.DisplayModeProperty, Bridge.box(value, Windows.UI.Xaml.Controls.DataPager_DisplayMode, System.Enum.toStringFn(Windows.UI.Xaml.Controls.DataPager_DisplayMode)));
                }
            },
            /**
             * The max number of numeric button that can be displayed
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.DataPager
             * @function NumericButtonCount
             * @type number
             */
            NumericButtonCount: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.Controls.DataPager.NumericButtonCountProperty), System.Int32), System.Int32));
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Controls.DataPager.NumericButtonCountProperty, Bridge.box(value, System.Int32));
                }
            },
            /**
             * The max number of numeric buttons that can be displayed
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.DataPager
             * @function Source
             * @type Windows.UI.Xaml.Data.INTERNAL_PagedCollectionView
             */
            Source: {
                get: function () {
                    return Bridge.cast(this.GetValue(Windows.UI.Xaml.Controls.DataPager.SourceProperty), Windows.UI.Xaml.Data.INTERNAL_PagedCollectionView);
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Controls.DataPager.SourceProperty, value);
                }
            },
            /**
             * Gets or sets the maximum number of items to display on a page.
             -1 means infinity. Default is -1.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.DataPager
             * @function PageSize
             * @type number
             */
            PageSize: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.Controls.DataPager.PageSizeProperty), System.Int32), System.Int32));
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Controls.DataPager.PageSizeProperty, Bridge.box(value, System.Int32));
                }
            },
            /**
             * // the current page that is shown
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.DataPager
             * @function CurrentPage
             * @type number
             */
            CurrentPage: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.Controls.DataPager.CurrentPageProperty), System.Int32), System.Int32));
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Controls.DataPager.CurrentPageProperty, Bridge.box(this.GetNewCurrentPageIndex(value), System.Int32));
                }
            },
            /**
             * Gets or sets an instance Style that is applied for this object during rendering.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.DataPager
             * @function NumericButtonStyle
             * @type Windows.UI.Xaml.Style
             */
            NumericButtonStyle: {
                get: function () {
                    return Bridge.cast(this.GetValue(Windows.UI.Xaml.Controls.DataPager.NumericButtonStyleProperty), Windows.UI.Xaml.Style);
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Controls.DataPager.NumericButtonStyleProperty, value);
                }
            },
            SourceCollection: {
                get: function () {
                    return this.Source.SourceCollection;
                }
            }
        },
        ctors: {
            ctor: function () {
                this.$initialize();
                Windows.UI.Xaml.Controls.Control.ctor.call(this);
                this.DefaultStyleKey = Windows.UI.Xaml.Controls.DataPager;
            }
        },
        methods: {
            CountNumberPage: function () {
                var nbElements = this.Source.TotalItemCount;

                if (nbElements === 0) {
                    return 0;
                }

                if (this.PageSize === -1) {
                    return 1;
                }

                var nbPage = (Bridge.Int.div(nbElements, this.PageSize)) | 0;

                return ((nbPage + (nbElements % this.PageSize === 0 ? 0 : 1)) | 0);
            },
            GetNewCurrentPageIndex: function (requestedIndex) {
                var newIndex;

                if (this.NumberOfPages === 0) {
                    newIndex = 0;
                } else {
                    if (requestedIndex < 1) {
                        newIndex = 1;
                    } else {
                        if (requestedIndex > this.NumberOfPages) {
                            newIndex = this.NumberOfPages;
                        } else {
                            newIndex = requestedIndex;
                        }
                    }
                }

                return newIndex;
            },
            OnApplyTemplate: function () {
                Windows.UI.Xaml.Controls.Control.prototype.OnApplyTemplate.call(this);

                this._buttonPanel = Bridge.as(this.GetTemplateChild("NumericButtonPanel"), Windows.UI.Xaml.Controls.StackPanel);

                if (this._buttonPanel != null) {
                    this.EventRegister("FirstPageButton");
                    this.EventRegister("LastPageButton");
                    this.EventRegister("PreviousPageButton");
                    this.EventRegister("NextPageButton");
                    this.EventRegister("CurrentPageTextBox");

                    if (this.Source != null) {
                        this.GenerateControls();
                    }
                }
            },
            EventRegister: function (childName) {
                var child = this.GetTemplateChild(childName);
                if (Bridge.is(child, Windows.UI.Xaml.Controls.Button)) {
                    var button = Bridge.cast(child, Windows.UI.Xaml.Controls.Button);
                    button.removeClick(Bridge.fn.cacheBind(this, this.OnPageButtonClick));
                    button.addClick(Bridge.fn.cacheBind(this, this.OnPageButtonClick));
                }
                if (Bridge.is(child, Windows.UI.Xaml.Controls.TextBox)) {
                    var text = Bridge.cast(child, Windows.UI.Xaml.Controls.TextBox);
                    this._textBox = text;
                    text.removeTextChanged(Bridge.fn.cacheBind(this, this.OnPageSelectionTextChanged));
                    text.addTextChanged(Bridge.fn.cacheBind(this, this.OnPageSelectionTextChanged));
                }
            },
            AddButton: function () {
                var nameAndContent = (Bridge.toString(this._buttonPanel.Children.Count) || "") + 1;

                var newButton = new Windows.UI.Xaml.Controls.Primitives.ToggleButton();
                newButton.Style = this.NumericButtonStyle;

                newButton.Cursor = System.Windows.Input.Cursors.Hand;

                newButton.Content = nameAndContent;
                newButton.Name = nameAndContent;
                newButton.addClick(Bridge.fn.cacheBind(this, this.OnPageButtonClick));

                this._buttonPanel.Children.add(newButton);
            },
            GetStartButtonIndex: function () {
                if (this.CurrentPage === 0) {
                    return 0;
                }

                var nbButtonFromMiddle = (Bridge.Int.div(this.NumericButtonCount, 2)) | 0;

                var nbPageIncreaseIfNecessary = Math.max(this.NumericButtonCount, this.NumberOfPages);

                var min = (this.CurrentPage - nbButtonFromMiddle) | 0;
                var max = (((this.CurrentPage + nbButtonFromMiddle) | 0) - (this.NumericButtonCount % 2 === 0 ? 1 : 0)) | 0;

                if (min < 1) {
                    min = (min + (((((-min) | 0) + 1) | 0))) | 0;
                    max = (max + (((((-min) | 0) + 1) | 0))) | 0;
                }
                if (max > nbPageIncreaseIfNecessary) {
                    min = (min - (((max - nbPageIncreaseIfNecessary) | 0))) | 0;
                    max = (max - (((max - nbPageIncreaseIfNecessary) | 0))) | 0;
                }
                return min;
            },
            RefreshControl: function () {
                var $t;
                var newButtonNb = this.GetStartButtonIndex();

                $t = Bridge.getEnumerator(this._buttonPanel.Children);
                try {
                    while ($t.moveNext()) {
                        var element = $t.Current;
                        if (Bridge.is(element, Windows.UI.Xaml.Controls.Button)) {
                            var button = Bridge.cast(element, Windows.UI.Xaml.Controls.Button);

                            button.Name = Bridge.toString(newButtonNb);
                            button.Content = Bridge.toString(newButtonNb);

                            if (newButtonNb === this.CurrentPage) {
                                Windows.UI.Xaml.VisualStateManager.GoToState(button, "Checked", false);
                            } else {
                                Windows.UI.Xaml.VisualStateManager.GoToState(button, "Unchecked", false);
                            }

                            newButtonNb = (newButtonNb + 1) | 0;
                        }
                    }
                } finally {
                    if (Bridge.is($t, System.IDisposable)) {
                        $t.System$IDisposable$Dispose();
                    }
                }

                if (this._textBox != null) {
                    this._textBox.Text = Bridge.toString(this.CurrentPage);
                }
            },
            RefereshNumberOfButtonsAndPages: function () {
                var oldNumberOfPages = this.NumberOfPages;
                this.NumberOfPages = this.CountNumberPage();

                var nbButtonMax = Math.min(this.NumericButtonCount, this.NumberOfPages);

                if (this._buttonPanel != null) {
                    var add = Math.max(((nbButtonMax - this._buttonPanel.Children.Count) | 0), 0);
                    var remove = Math.max(((this._buttonPanel.Children.Count - nbButtonMax) | 0), 0);

                    for (var i = 0; i < add; i = (i + 1) | 0) {
                        this.AddButton();
                    }

                    for (var i1 = 0; i1 < remove; i1 = (i1 + 1) | 0) {
                        this._buttonPanel.Children.removeAt(0);
                    }

                    if (this._buttonPanel.Children.Count === 0) {
                        add = 1;
                        this.AddButton();
                    }

                    if (this.NumberOfPages <= 1) {
                        this.Visibility = Windows.UI.Xaml.Visibility.Collapsed;
                    } else {
                        this.Visibility = Windows.UI.Xaml.Visibility.Visible;
                    }

                    this.CurrentPage = this.GetNewCurrentPageIndex(this.CurrentPage);

                    if (add !== 0 || remove !== 0 || oldNumberOfPages !== this.NumberOfPages) {
                        this.RefreshControl();
                    }
                }
            },
            GenerateControls: function () {
                var mode = System.Enum.toString(Windows.UI.Xaml.Controls.DataPager_DisplayMode, this.DisplayMode);

                Windows.UI.Xaml.VisualStateManager.GoToState(this, mode, false);

                if (this._buttonPanel != null) {
                    this.RefereshNumberOfButtonsAndPages();
                }
            },
            OnPageButtonClick: function (sender, e) {
                var buttonName = Bridge.is(sender, Windows.UI.Xaml.Controls.Button) ? Bridge.cast(sender, Windows.UI.Xaml.Controls.Button).Name : Bridge.cast(sender, Windows.UI.Xaml.Controls.Button).Name;

                switch (buttonName) {
                    case "FirstPageButton": 
                        this.CurrentPage = 1;
                        break;
                    case "LastPageButton": 
                        this.CurrentPage = this.NumberOfPages;
                        break;
                    case "PreviousPageButton": 
                        this.CurrentPage = (this.CurrentPage - 1) | 0;
                        break;
                    case "NextPageButton": 
                        this.CurrentPage = (this.CurrentPage + 1) | 0;
                        break;
                    default: 
                        var buttonNumber = System.Int32.parse(buttonName);
                        this.CurrentPage = buttonNumber;
                        break;
                }
            },
            OnPageSelectionTextChanged: function (sender, e) {
                var newPageIndex = { };

                var textBox = Bridge.cast(e.OriginalSource, Windows.UI.Xaml.Controls.TextBox);

                if (!Bridge.referenceEquals(textBox.Text, "")) {
                    if (System.Int32.tryParse(textBox.Text, newPageIndex)) {
                        if (newPageIndex.v !== this.CurrentPage) {
                            this.CurrentPage = newPageIndex.v;
                        }
                    } else {
                        textBox.Text = Bridge.toString(this.CurrentPage);
                    }
                }
            },
            Source_PageChanged: function (sender, e) {
                this.CurrentPage = (this.Source.PageIndex + 1) | 0;
            },
            Source_CollectionChanged: function (sender, e) {
                this.RefereshNumberOfButtonsAndPages();
            }
        }
    });

    Bridge.define("Windows.UI.Xaml.Controls.INTERNAL_DateTimePickerBase", {
        inherits: [Windows.UI.Xaml.Controls.Control],
        statics: {
            fields: {
                ElementRoot: null,
                ElementTextBox: null,
                ElementButton: null,
                ElementPopup: null,
                /**
                 * Identifies the dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Controls.INTERNAL_DateTimePickerBase
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                IsDropDownOpenProperty: null,
                /**
                 * Identifies the dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Controls.INTERNAL_DateTimePickerBase
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                TextProperty: null
            },
            ctors: {
                init: function () {
                    var $t;
                    this.ElementRoot = "Root";
                    this.ElementTextBox = "TextBox";
                    this.ElementButton = "Button";
                    this.ElementPopup = "Popup";
                    this.IsDropDownOpenProperty = Windows.UI.Xaml.DependencyProperty.Register("IsDropDownOpen", System.Boolean, Windows.UI.Xaml.Controls.INTERNAL_DateTimePickerBase, ($t = new Windows.UI.Xaml.PropertyMetadata.$ctor2(Bridge.box(false, System.Boolean, System.Boolean.toString), Windows.UI.Xaml.Controls.INTERNAL_DateTimePickerBase.OnIsDropDownOpenChanged), $t.CallPropertyChangedWhenLoadedIntoVisualTree = Windows.UI.Xaml.WhenToCallPropertyChangedEnum.IfPropertyIsSet, $t));
                    this.TextProperty = Windows.UI.Xaml.DependencyProperty.Register("Text", System.String, Windows.UI.Xaml.Controls.INTERNAL_DateTimePickerBase, ($t = new Windows.UI.Xaml.PropertyMetadata.$ctor2("", Windows.UI.Xaml.Controls.INTERNAL_DateTimePickerBase.OnTextChanged), $t.CallPropertyChangedWhenLoadedIntoVisualTree = Windows.UI.Xaml.WhenToCallPropertyChangedEnum.IfPropertyIsSet, $t));
                }
            },
            methods: {
                /**
                 * IsDropDownOpenProperty property changed handler.
                 *
                 * @static
                 * @protected
                 * @this Windows.UI.Xaml.Controls.INTERNAL_DateTimePickerBase
                 * @memberof Windows.UI.Xaml.Controls.INTERNAL_DateTimePickerBase
                 * @param   {Windows.UI.Xaml.DependencyObject}                      d    the control that changed its IsDropDownOpen.
                 * @param   {Windows.UI.Xaml.DependencyPropertyChangedEventArgs}    e    The DependencyPropertyChangedEventArgs.
                 * @return  {void}
                 */
                OnIsDropDownOpenChanged: function (d, e) {
                    var dp = Bridge.as(d, Windows.UI.Xaml.Controls.INTERNAL_DateTimePickerBase);
                    var newValue = System.Nullable.getValue(Bridge.cast(Bridge.unbox(e.NewValue, System.Boolean), System.Boolean));

                    if (dp._popup != null && dp._popup.Child != null) {
                        if (newValue) {
                            dp.OpenDropDown();
                        } else {
                            dp.CloseDropDown();
                        }
                    }
                },
                /**
                 * TextProperty property changed handler.
                 *
                 * @static
                 * @private
                 * @this Windows.UI.Xaml.Controls.INTERNAL_DateTimePickerBase
                 * @memberof Windows.UI.Xaml.Controls.INTERNAL_DateTimePickerBase
                 * @param   {Windows.UI.Xaml.DependencyObject}                      d    Date picker or time picker that changed its Text.
                 * @param   {Windows.UI.Xaml.DependencyPropertyChangedEventArgs}    e    The DependencyPropertyChangedEventArgs.
                 * @return  {void}
                 */
                OnTextChanged: function (d, e) {
                    var dp = Bridge.cast(d, Windows.UI.Xaml.Controls.INTERNAL_DateTimePickerBase);

                    var newText = Bridge.cast(e.NewValue, System.String);

                    if (dp._textBox != null) {

                        dp._textBox.Text = newText;
                    }
                },
                ToDateTime: function (dateTimeAsString) {
                    var year, month, day, hours, minutes, seconds;
                    var split = System.String.split(dateTimeAsString, [32].map(function (i) {{ return String.fromCharCode(i); }}));
                    var datePart = split[System.Array.index(0, split)];
                    var splittedDate = System.String.split(datePart, [47].map(function (i) {{ return String.fromCharCode(i); }}));
                    month = System.Int32.parse(splittedDate[System.Array.index(0, splittedDate)]);
                    day = System.Int32.parse(splittedDate[System.Array.index(1, splittedDate)]);
                    year = System.Int32.parse(splittedDate[System.Array.index(2, splittedDate)]);

                    var TimePart = split[System.Array.index(1, split)];
                    var splittedTime = System.String.split(TimePart, [58].map(function (i) {{ return String.fromCharCode(i); }}));
                    hours = System.Int32.parse(splittedTime[System.Array.index(0, splittedTime)]);
                    minutes = System.Int32.parse(splittedTime[System.Array.index(1, splittedTime)]);
                    seconds = System.Int32.parse(splittedTime[System.Array.index(2, splittedTime)]);

                    return System.DateTime.create(year, month, day, hours, minutes, seconds);
                }
            }
        },
        fields: {
            _root: null,
            _textBox: null,
            _dropDownButton: null,
            _popup: null,
            _defaultText: null,
            _calendarOrClock: null
        },
        events: {
            SelectedDateChanged: null
        },
        props: {
            /**
             * Gets or sets a value indicating whether the drop-down is open or closed.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.INTERNAL_DateTimePickerBase
             * @function IsDropDownOpen
             * @type boolean
             */
            IsDropDownOpen: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.Controls.INTERNAL_DateTimePickerBase.IsDropDownOpenProperty), System.Boolean), System.Boolean));
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Controls.INTERNAL_DateTimePickerBase.IsDropDownOpenProperty, Bridge.box(value, System.Boolean, System.Boolean.toString));
                }
            },
            INTERNAL_SelectedDate: {
                get: function () {
                    return this._calendarOrClock.SelectedValue;
                },
                set: function (value) {
                    this._calendarOrClock.SelectedValue = value;
                }
            },
            /**
             * Gets or sets the text that is displayed by the textbox
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.INTERNAL_DateTimePickerBase
             * @function Text
             * @type string
             */
            Text: {
                get: function () {
                    return Bridge.cast(this.GetValue(Windows.UI.Xaml.Controls.INTERNAL_DateTimePickerBase.TextProperty), System.String);
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Controls.INTERNAL_DateTimePickerBase.TextProperty, value);
                }
            }
        },
        ctors: {
            init: function () {
                this._defaultText = "Select Date...";
            },
            ctor: function () {
                this.$initialize();
                Windows.UI.Xaml.Controls.Control.ctor.call(this);
                this._calendarOrClock = this.GenerateCalendarOrClock();
                this._calendarOrClock.addSelectedDatesChanged(Bridge.fn.cacheBind(this, this.CalendarOrClock_SelectionChanged));
                this.INTERNAL_SelectedDate = null;
            }
        },
        methods: {
            OnApplyTemplate: function () {
                Windows.UI.Xaml.Controls.Control.prototype.OnApplyTemplate.call(this);

                if (this._popup != null) {
                    this._popup.removeClosedDueToOutsideClick(Bridge.fn.cacheBind(this, this.Popup_ClosedDueToOutsideClick));
                }

                this._root = Bridge.as(this.GetTemplateChild(Windows.UI.Xaml.Controls.INTERNAL_DateTimePickerBase.ElementRoot), Windows.UI.Xaml.FrameworkElement);
                this._textBox = Bridge.as(this.GetTemplateChild(Windows.UI.Xaml.Controls.INTERNAL_DateTimePickerBase.ElementTextBox), Windows.UI.Xaml.Controls.TextBox);
                this._dropDownButton = Bridge.as(this.GetTemplateChild(Windows.UI.Xaml.Controls.INTERNAL_DateTimePickerBase.ElementButton), Windows.UI.Xaml.Controls.Button);
                this._popup = Bridge.as(this.GetTemplateChild(Windows.UI.Xaml.Controls.INTERNAL_DateTimePickerBase.ElementPopup), Windows.UI.Xaml.Controls.Primitives.Popup);

                if (this._dropDownButton != null) {
                    this._dropDownButton.addClick(Bridge.fn.cacheBind(this, this.DropDownButton_Click));
                }

                if (this._popup != null) {
                    this._popup.StayOpen = false;
                    this._popup.Child = this._calendarOrClock;
                    this._popup.IsOpen = false;
                    this._popup.addClosedDueToOutsideClick(Bridge.fn.cacheBind(this, this.Popup_ClosedDueToOutsideClick));
                }

                if (this._textBox != null) {
                    this._textBox.IsReadOnly = true;
                }

                this.RefreshTextBox();
            },
            Popup_ClosedDueToOutsideClick: function (sender, e) {
                if (this._dropDownButton != null) {
                    if (this._dropDownButton.IsPointerCaptured) {
                        this._dropDownButton.ReleasePointerCapture();
                    }
                    this.IsDropDownOpen = false;
                }
            },
            DropDownButton_Click: function (sender, e) {
                this.IsDropDownOpen = !this.IsDropDownOpen;
            },
            OpenDropDown: function () {
                var $step = 0,
                    $task1, 
                    $jumpFromFinally, 
                    $asyncBody = Bridge.fn.bind(this, function () {
                        for (;;) {
                            $step = System.Array.min([0,1], $step);
                            switch ($step) {
                                case 0: {
                                    this._popup.IsOpen = true;
                                    $task1 = System.Threading.Tasks.Task.delay(System.TimeSpan.fromSeconds(0.5));
                                    $step = 1;
                                    if ($task1.isCompleted()) {
                                        continue;
                                    }
                                    $task1.continue($asyncBody);
                                    return;
                                }
                                case 1: {
                                    $task1.getAwaitedResult();
                                    DotNetForHtml5.Core.INTERNAL_PopupsManager.EnsurePopupStaysWithinScreenBounds(this._popup);
                                    return;
                                }
                                default: {
                                    return;
                                }
                            }
                        }
                    }, arguments);

                $asyncBody();
            },
            CloseDropDown: function () {
                this._popup.IsOpen = false;
            },
            CalendarOrClock_SelectionChanged: function (sender, e) {

                this.OnSelectionChanged(Bridge.cast(Bridge.unbox(System.Array.getItem(e.AddedItems, 0), System.DateTime), System.DateTime, true));

                this.RefreshTextBox();

                if (!Bridge.staticEquals(this.SelectedDateChanged, null)) {
                    this.SelectedDateChanged(this, e);
                }
            },
            RefreshTextBox: function () {
                if (this._textBox != null) {
                    var selectedDate = this.INTERNAL_SelectedDate;
                    if (!Bridge.equals(selectedDate, null)) {
                        this.Text = this.SetTextFromDate(System.Nullable.toString(selectedDate, function ($t) { return System.DateTime.format($t); }));
                    } else {
                        this.SetWaterMarkText();
                    }
                }
            },
            SetWaterMarkText: function () {
                if (this._textBox != null) {
                    this.Text = this._defaultText;
                }
            },
            /**
             * Input text is parsed in the correct format and changed into a DateTime object.
             If the text can not be parsed TextParseError Event is thrown.
             *
             * @instance
             * @private
             * @this Windows.UI.Xaml.Controls.INTERNAL_DateTimePickerBase
             * @memberof Windows.UI.Xaml.Controls.INTERNAL_DateTimePickerBase
             * @param   {string}              text
             * @return  {?System.DateTime}
             */
            ParseText: function (text) {
                var newSelectedDate;

                newSelectedDate = Windows.UI.Xaml.Controls.INTERNAL_DateTimePickerBase.ToDateTime(text);

                return newSelectedDate;
            }
        }
    });

    /**
     * Defines an area where you can arrange child elements either horizontally
     or vertically, relative to each other.
     *
     * @public
     * @class Windows.UI.Xaml.Controls.DockPanel
     * @augments Windows.UI.Xaml.Controls.Panel
     */
    Bridge.define("Windows.UI.Xaml.Controls.DockPanel", {
        inherits: [Windows.UI.Xaml.Controls.Panel],
        statics: {
            fields: {
                /**
                 * Identifies the DockPanel.LastChildFill dependency
                 property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Controls.DockPanel
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                LastChildFillProperty: null,
                /**
                 * Identifies the DockPanel.Dock attached property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Controls.DockPanel
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                DockProperty: null
            },
            ctors: {
                init: function () {
                    this.LastChildFillProperty = Windows.UI.Xaml.DependencyProperty.Register("LastChildFill", System.Boolean, Windows.UI.Xaml.Controls.DockPanel, new Windows.UI.Xaml.PropertyMetadata.$ctor1(Bridge.box(true, System.Boolean, System.Boolean.toString)));
                    this.DockProperty = Windows.UI.Xaml.DependencyProperty.RegisterAttached("Dock", Windows.UI.Xaml.Controls.Dock, Windows.UI.Xaml.Controls.DockPanel, new Windows.UI.Xaml.PropertyMetadata.$ctor1(Bridge.box(Windows.UI.Xaml.Controls.Dock.Left, Windows.UI.Xaml.Controls.Dock, System.Enum.toStringFn(Windows.UI.Xaml.Controls.Dock))));
                }
            },
            methods: {
                /**
                 * Gets the value of the DockPanel.Dock attached property
                 for a specified System.Windows.UIElement.
                 *
                 * @static
                 * @public
                 * @this Windows.UI.Xaml.Controls.DockPanel
                 * @memberof Windows.UI.Xaml.Controls.DockPanel
                 * @param   {Windows.UI.Xaml.DependencyObject}    element    The element from which the property value is read.
                 * @return  {Windows.UI.Xaml.Controls.Dock}                  The DockPanel.Dock property value for the element.
                 */
                GetDock: function (element) {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(element.GetValue(Windows.UI.Xaml.Controls.DockPanel.DockProperty), Windows.UI.Xaml.Controls.Dock), System.Int32));
                },
                /**
                 * Sets the value of the DockPanel.Dock attached property
                 to a specified element.
                 *
                 * @static
                 * @public
                 * @this Windows.UI.Xaml.Controls.DockPanel
                 * @memberof Windows.UI.Xaml.Controls.DockPanel
                 * @param   {Windows.UI.Xaml.DependencyObject}    element    The element to which the attached property is written.
                 * @param   {Windows.UI.Xaml.Controls.Dock}       dock       The needed Dock value.
                 * @return  {void}
                 */
                SetDock: function (element, dock) {
                    element.SetValue(Windows.UI.Xaml.Controls.DockPanel.DockProperty, Bridge.box(dock, Windows.UI.Xaml.Controls.Dock, System.Enum.toStringFn(Windows.UI.Xaml.Controls.Dock)));
                }
            }
        },
        fields: {
            _grid: null
        },
        props: {
            /**
             * // Initializes a new instance of the DockPanel class.
            //
             * Gets or sets a value that indicates whether the last child element within
             a DockPanel stretches to fill the remaining available
             space.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.DockPanel
             * @function LastChildFill
             * @type boolean
             */
            LastChildFill: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.Controls.DockPanel.LastChildFillProperty), System.Boolean), System.Boolean));
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Controls.DockPanel.LastChildFillProperty, Bridge.box(value, System.Boolean, System.Boolean.toString));
                }
            }
        },
        methods: {
            INTERNAL_OnAttachedToVisualTree: function () {
                if (this._grid == null) {
                    this._grid = new Windows.UI.Xaml.Controls.Grid();
                    this.MakeUIStructure();
                }

                Windows.UI.Xaml.Controls.Panel.prototype.INTERNAL_OnAttachedToVisualTree.call(this);
            },
            MakeUIStructure: function () {
                var $t, $t1, $t2;
                if (this.Children != null && this.Children.Count > 0) {
                    var amountOfRows = 1;
                    var amountOfColumns = 1;
                    var lastChildDock = Windows.UI.Xaml.Controls.Dock.Left;

                    $t = Bridge.getEnumerator(this.Children);
                    try {
                        while ($t.moveNext()) {
                            var child = $t.Current;
                            var dock = Windows.UI.Xaml.Controls.DockPanel.GetDock(child);

                            if (dock === Windows.UI.Xaml.Controls.Dock.Left || dock === Windows.UI.Xaml.Controls.Dock.Right) {
                                amountOfColumns = (amountOfColumns + 1) | 0;
                            } else {
                                amountOfRows = (amountOfRows + 1) | 0;
                            }
                        }
                    } finally {
                        if (Bridge.is($t, System.IDisposable)) {
                            $t.System$IDisposable$Dispose();
                        }
                    }
                    if (this.LastChildFill) {
                        lastChildDock = Windows.UI.Xaml.Controls.DockPanel.GetDock(this.Children.getItem(((this.Children.Count - 1) | 0)));
                        if (lastChildDock === Windows.UI.Xaml.Controls.Dock.Right || lastChildDock === Windows.UI.Xaml.Controls.Dock.Left) {
                            amountOfColumns = (amountOfColumns - 1) | 0;
                        } else {
                            amountOfRows = (amountOfRows - 1) | 0;
                        }
                    }

                    var amountOfRightPlaced = 0;
                    var amountOfLeftPlaced = 0;
                    var amountOfTopPlaced = 0;
                    var amountOfBottomPlaced = 0;

                    $t1 = Bridge.getEnumerator(this.Children);
                    try {
                        while ($t1.moveNext()) {
                            var child1 = $t1.Current;
                            var dock1 = Windows.UI.Xaml.Controls.DockPanel.GetDock(child1);

                            switch (dock1) {
                                case Windows.UI.Xaml.Controls.Dock.Left: 
                                    Windows.UI.Xaml.Controls.Grid.SetRow(child1, amountOfTopPlaced);
                                    Windows.UI.Xaml.Controls.Grid.SetColumn(child1, amountOfLeftPlaced);
                                    Windows.UI.Xaml.Controls.Grid.SetRowSpan(child1, ((((amountOfRows - amountOfTopPlaced) | 0) - amountOfBottomPlaced) | 0));
                                    Windows.UI.Xaml.Controls.Grid.SetColumnSpan(child1, 1);
                                    amountOfLeftPlaced = (amountOfLeftPlaced + 1) | 0;
                                    break;
                                case Windows.UI.Xaml.Controls.Dock.Top: 
                                    Windows.UI.Xaml.Controls.Grid.SetRow(child1, amountOfTopPlaced);
                                    Windows.UI.Xaml.Controls.Grid.SetColumn(child1, amountOfLeftPlaced);
                                    Windows.UI.Xaml.Controls.Grid.SetRowSpan(child1, 1);
                                    Windows.UI.Xaml.Controls.Grid.SetColumnSpan(child1, ((((amountOfColumns - amountOfLeftPlaced) | 0) - amountOfRightPlaced) | 0));
                                    amountOfTopPlaced = (amountOfTopPlaced + 1) | 0;
                                    break;
                                case Windows.UI.Xaml.Controls.Dock.Right: 
                                    Windows.UI.Xaml.Controls.Grid.SetRow(child1, amountOfTopPlaced);
                                    Windows.UI.Xaml.Controls.Grid.SetColumn(child1, ((((amountOfColumns - amountOfRightPlaced) | 0) - 1) | 0));
                                    Windows.UI.Xaml.Controls.Grid.SetRowSpan(child1, ((((amountOfRows - amountOfTopPlaced) | 0) - amountOfBottomPlaced) | 0));
                                    Windows.UI.Xaml.Controls.Grid.SetColumnSpan(child1, 1);
                                    amountOfRightPlaced = (amountOfRightPlaced + 1) | 0;
                                    break;
                                case Windows.UI.Xaml.Controls.Dock.Bottom: 
                                    Windows.UI.Xaml.Controls.Grid.SetRow(child1, ((((amountOfRows - amountOfBottomPlaced) | 0) - 1) | 0));
                                    Windows.UI.Xaml.Controls.Grid.SetColumn(child1, amountOfLeftPlaced);
                                    Windows.UI.Xaml.Controls.Grid.SetRowSpan(child1, 1);
                                    Windows.UI.Xaml.Controls.Grid.SetColumnSpan(child1, ((((amountOfColumns - amountOfLeftPlaced) | 0) - amountOfRightPlaced) | 0));
                                    amountOfBottomPlaced = (amountOfBottomPlaced + 1) | 0;
                                    break;
                                default: 
                                    break;
                            }
                        }
                    } finally {
                        if (Bridge.is($t1, System.IDisposable)) {
                            $t1.System$IDisposable$Dispose();
                        }
                    }

                    CSHTML5.Internal.INTERNAL_VisualTreeManager.DetachVisualChildIfNotNull(this._grid, this);

                    var columnsDefinitions = this._grid.ColumnDefinitions;
                    columnsDefinitions.clear();
                    for (var i = 0; i < amountOfColumns; i = (i + 1) | 0) {
                        columnsDefinitions.add(($t2 = new Windows.UI.Xaml.Controls.ColumnDefinition(), $t2.Width = Windows.UI.Xaml.GridLength.Auto.$clone(), $t2));
                    }
                    var rowsDefinitions = this._grid.RowDefinitions;
                    rowsDefinitions.clear();
                    for (var i1 = 0; i1 < amountOfRows; i1 = (i1 + 1) | 0) {
                        rowsDefinitions.add(($t2 = new Windows.UI.Xaml.Controls.RowDefinition(), $t2.Height = Windows.UI.Xaml.GridLength.Auto.$clone(), $t2));
                    }

                    if (!this.LastChildFill) {
                        System.Linq.Enumerable.from(columnsDefinitions).elementAt(amountOfLeftPlaced).Width = new Windows.UI.Xaml.GridLength.$ctor2(1, Windows.UI.Xaml.GridUnitType.Star);
                        System.Linq.Enumerable.from(rowsDefinitions).elementAt(amountOfTopPlaced).Height = new Windows.UI.Xaml.GridLength.$ctor2(1, Windows.UI.Xaml.GridUnitType.Star);
                    } else {
                        if (lastChildDock === Windows.UI.Xaml.Controls.Dock.Left) {
                            System.Linq.Enumerable.from(columnsDefinitions).elementAt(((amountOfLeftPlaced - 1) | 0)).Width = new Windows.UI.Xaml.GridLength.$ctor2(1, Windows.UI.Xaml.GridUnitType.Star);
                        } else {
                            System.Linq.Enumerable.from(columnsDefinitions).elementAt(amountOfLeftPlaced).Width = new Windows.UI.Xaml.GridLength.$ctor2(1, Windows.UI.Xaml.GridUnitType.Star);
                        }

                        if (lastChildDock === Windows.UI.Xaml.Controls.Dock.Top) {
                            System.Linq.Enumerable.from(rowsDefinitions).elementAt(((amountOfTopPlaced - 1) | 0)).Height = new Windows.UI.Xaml.GridLength.$ctor2(1, Windows.UI.Xaml.GridUnitType.Star);
                        } else {
                            System.Linq.Enumerable.from(rowsDefinitions).elementAt(amountOfTopPlaced).Height = new Windows.UI.Xaml.GridLength.$ctor2(1, Windows.UI.Xaml.GridUnitType.Star);
                        }
                    }
                    CSHTML5.Internal.INTERNAL_VisualTreeManager.AttachVisualChildIfNotAlreadyAttached(this._grid, this);
                }
            },
            OnChildrenAdded: function (newChild, index) {
                this.MakeUIStructure();
                this._grid.Children.insert(index, newChild);
            },
            OnChildrenRemoved: function (oldChild, index) {
                this._grid.Children.removeAt(index);
            },
            OnChildrenReplaced: function (oldChild, newChild, index) {
                if (Bridge.referenceEquals(oldChild, newChild)) {
                    return;
                }

                System.Diagnostics.Debug.Assert(Bridge.referenceEquals(oldChild, this._grid.Children.getItem(index)));

                this.MakeUIStructure();
                this._grid.Children.setItem(index, newChild);
            },
            OnChildrenMoved: function (oldChild, newIndex, oldIndex) {
                if (newIndex === oldIndex) {
                    return;
                }

                this.MakeUIStructure();
                this._grid.Children.Move(oldIndex, newIndex);
            },
            OnChildrenReset: function () {
                var $t;
                this._grid.Children.clear();

                this.MakeUIStructure();

                $t = Bridge.getEnumerator(this.Children);
                try {
                    while ($t.moveNext()) {
                        var child = $t.Current;
                        this._grid.Children.add(child);
                    }
                } finally {
                    if (Bridge.is($t, System.IDisposable)) {
                        $t.System$IDisposable$Dispose();
                    }
                }
            }
        }
    });

    /**
     * Defines a flexible grid area that consists of columns and rows. Child elements
     of the Grid are measured and arranged according to their row/column assignments
     and internal partial class logic.
     *
     * @public
     * @class Windows.UI.Xaml.Controls.Grid
     * @augments Windows.UI.Xaml.Controls.Panel
     * @example You can add a Grid with two rows and columns to the XAML as follows:
 
     Or in C#:
     *
     * 
     *     
     *         
     *         
     *     
     *     
     *         
     *         
     *     
     *         
     *     
     * 
     *
     * Grid myGrid = new Grid();
     * myGrid.Width = 300;
     * myGrid.Height = 200;
     * myGrid.Background = new SolidColorBrush(Windows.UI.Colors.Blue);
     * myGrid.HorizontalAlignment = HorizontalAlignment.Left;
     * //We create and add the rows and columns:
     * //First column:
     * ColumnDefinition columnDefinition = new ColumnDefinition();
     * columnDefinition.Width = new GridLength(70, GridUnitType.Pixel);
     * myGrid.ColumnDefinitions.Add(columnDefinition);
     * //Second column:
     * ColumnDefinition columnDefinition2 = new ColumnDefinition();
     * columnDefinition2.Width = new GridLength(1, GridUnitType.Star);
     * myGrid.ColumnDefinitions.Add(columnDefinition2);
     * //First row:
     * RowDefinition rowDefinition = new RowDefinition();
     * rowDefinition.Height = new GridLength(40, GridUnitType.Pixel);
     * myGrid.RowDefinitions.Add(rowDefinition);
     * //Second Row:
     * RowDefinition rowDefinition2 = new RowDefinition();
     * rowDefinition2.Height = new GridLength(1, GridUnitType.Star);
     * myGrid.RowDefinitions.Add(rowDefinition2);
     * //Do not forget to add the Grid to the visual tree.
     * 
     *
     *
     */
    Bridge.define("Windows.UI.Xaml.Controls.Grid", {
        inherits: [Windows.UI.Xaml.Controls.Panel],
        statics: {
            fields: {
                /**
                 * Identifies the Grid.Row XAML attached property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Controls.Grid
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                RowProperty: null,
                RowSpanProperty: null,
                /**
                 * Identifies the Grid.Column XAML attached property
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Controls.Grid
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                ColumnProperty: null,
                ColumnSpanProperty: null
            },
            ctors: {
                init: function () {
                    var $t;
                    this.RowProperty = Windows.UI.Xaml.DependencyProperty.RegisterAttached("Row", System.Int32, Windows.UI.Xaml.UIElement, ($t = new Windows.UI.Xaml.PropertyMetadata.$ctor2(Bridge.box(0, System.Int32), Windows.UI.Xaml.Controls.Grid.Row_Changed), $t.CallPropertyChangedWhenLoadedIntoVisualTree = Windows.UI.Xaml.WhenToCallPropertyChangedEnum.IfPropertyIsSet, $t));
                    this.RowSpanProperty = Windows.UI.Xaml.DependencyProperty.RegisterAttached("RowSpan", System.Int32, Windows.UI.Xaml.UIElement, ($t = new Windows.UI.Xaml.PropertyMetadata.$ctor2(Bridge.box(1, System.Int32), Windows.UI.Xaml.Controls.Grid.RowSpan_Changed), $t.CallPropertyChangedWhenLoadedIntoVisualTree = Windows.UI.Xaml.WhenToCallPropertyChangedEnum.IfPropertyIsSet, $t));
                    this.ColumnProperty = Windows.UI.Xaml.DependencyProperty.RegisterAttached("Column", System.Int32, Windows.UI.Xaml.UIElement, ($t = new Windows.UI.Xaml.PropertyMetadata.$ctor2(Bridge.box(0, System.Int32), Windows.UI.Xaml.Controls.Grid.Column_Changed), $t.CallPropertyChangedWhenLoadedIntoVisualTree = Windows.UI.Xaml.WhenToCallPropertyChangedEnum.IfPropertyIsSet, $t));
                    this.ColumnSpanProperty = Windows.UI.Xaml.DependencyProperty.RegisterAttached("ColumnSpan", System.Int32, Windows.UI.Xaml.UIElement, ($t = new Windows.UI.Xaml.PropertyMetadata.$ctor2(Bridge.box(1, System.Int32), Windows.UI.Xaml.Controls.Grid.ColumnSpan_Changed), $t.CallPropertyChangedWhenLoadedIntoVisualTree = Windows.UI.Xaml.WhenToCallPropertyChangedEnum.IfPropertyIsSet, $t));
                }
            },
            methods: {
                /**
                 * Sets the value of the Grid.Row XAML attached property on the specified FrameworkElement.
                 *
                 * @static
                 * @public
                 * @this Windows.UI.Xaml.Controls.Grid
                 * @memberof Windows.UI.Xaml.Controls.Grid
                 * @param   {Windows.UI.Xaml.UIElement}    element    The target element on which to set the Grid.Row XAML attached property.
                 * @param   {number}                       value      The property value to set.
                 * @return  {void}
                 */
                SetRow: function (element, value) {
                    element.SetValue(Windows.UI.Xaml.Controls.Grid.RowProperty, Bridge.box(value, System.Int32));
                },
                /**
                 * Gets the value of the Grid.Row XAML attached property from the specified
                 FrameworkElement.
                 *
                 * @static
                 * @public
                 * @this Windows.UI.Xaml.Controls.Grid
                 * @memberof Windows.UI.Xaml.Controls.Grid
                 * @param   {Windows.UI.Xaml.UIElement}    element    The element from which to read the property value.
                 * @return  {number}                                  The value of the Grid.Row XAML attached property on the target element.
                 */
                GetRow: function (element) {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(element.GetValue(Windows.UI.Xaml.Controls.Grid.RowProperty), System.Int32), System.Int32));
                },
                Row_Changed: function (d, e) {
                    var isCSSGrid = document.isGridSupported;
                    if (isCSSGrid) {
                        Windows.UI.Xaml.Controls.Grid.Row_Changed_CSSVersion(d, e);
                    } else {




                    }
                },
                SetRowSpan: function (element, value) {
                    element.SetValue(Windows.UI.Xaml.Controls.Grid.RowSpanProperty, Bridge.box(value, System.Int32));
                },
                GetRowSpan: function (element) {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(element.GetValue(Windows.UI.Xaml.Controls.Grid.RowSpanProperty), System.Int32), System.Int32));
                },
                RowSpan_Changed: function (d, e) {
                    var isCSSGrid = document.isGridSupported;
                    if (isCSSGrid) {
                        Windows.UI.Xaml.Controls.Grid.RowSpan_Changed_CSSVersion(d, e);
                    } else {




                    }
                },
                /**
                 * Sets the value of the Grid.Column XAML attached property on the specified FrameworkElement.
                 *
                 * @static
                 * @public
                 * @this Windows.UI.Xaml.Controls.Grid
                 * @memberof Windows.UI.Xaml.Controls.Grid
                 * @param   {Windows.UI.Xaml.UIElement}    element    The target element on which to set the Grid.Row XAML attached property.
                 * @param   {number}                       value      The property value to set.
                 * @return  {void}
                 */
                SetColumn: function (element, value) {
                    element.SetValue(Windows.UI.Xaml.Controls.Grid.ColumnProperty, Bridge.box(value, System.Int32));
                },
                /**
                 * Gets the value of the Grid.Column XAML attached property from the specified
                 FrameworkElement.
                 *
                 * @static
                 * @public
                 * @this Windows.UI.Xaml.Controls.Grid
                 * @memberof Windows.UI.Xaml.Controls.Grid
                 * @param   {Windows.UI.Xaml.UIElement}    element    The element from which to read the property value.
                 * @return  {number}                                  The value of the Grid.Column XAML attached property on the target element.
                 */
                GetColumn: function (element) {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(element.GetValue(Windows.UI.Xaml.Controls.Grid.ColumnProperty), System.Int32), System.Int32));
                },
                Column_Changed: function (d, e) {
                    var isCSSGrid = document.isGridSupported;
                    if (isCSSGrid) {
                        Windows.UI.Xaml.Controls.Grid.Column_Changed_CSSVersion(d, e);
                    } else {



                    }
                },
                SetColumnSpan: function (element, value) {
                    element.SetValue(Windows.UI.Xaml.Controls.Grid.ColumnSpanProperty, Bridge.box(value, System.Int32));
                },
                GetColumnSpan: function (element) {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(element.GetValue(Windows.UI.Xaml.Controls.Grid.ColumnSpanProperty), System.Int32), System.Int32));
                },
                ColumnSpan_Changed: function (d, e) {
                    var isCSSGrid = document.isGridSupported;
                    if (isCSSGrid) {
                        Windows.UI.Xaml.Controls.Grid.ColumnSpan_Changed_CSSVersion(d, e);
                    } else {




                    }
                },
                /**
                 * Makes sure that the given index of Grid.Row or Grid.Column is between 0 and the maximum Row/Column index
                 *
                 * @static
                 * @private
                 * @this Windows.UI.Xaml.Controls.Grid
                 * @memberof Windows.UI.Xaml.Controls.Grid
                 * @param   {System.Int32}    gridPosition    the row/column index to "coerce"
                 * @param   {number}          maxValue        the maximum value for a row/column index
                 * @return  {void}
                 */
                MakeGridPositionCorrect: function (gridPosition, maxValue) {
                    if (gridPosition.v < 0) {
                        gridPosition.v = 0;
                    }
                    if (gridPosition.v > maxValue) {
                        gridPosition.v = maxValue;
                    }
                },
                Row_Changed_CSSVersion: function (d, e) {
                    var element = Bridge.cast(d, Windows.UI.Xaml.UIElement);
                    Windows.UI.Xaml.Controls.Grid.ApplyRowPosition(element);
                },
                ApplyRowPosition: function (element) {
                    var maxRow = 0;
                    if (element.INTERNAL_VisualParent != null && Bridge.is(element.INTERNAL_VisualParent, Windows.UI.Xaml.Controls.Grid)) {
                        var parent = Bridge.cast(element.INTERNAL_VisualParent, Windows.UI.Xaml.Controls.Grid);
                        if (parent._rowDefinitionsOrNull != null && parent._rowDefinitionsOrNull.Count > 0) {
                            maxRow = (parent._rowDefinitionsOrNull.Count - 1) | 0;
                        }


                        var elementRow = { v : Windows.UI.Xaml.Controls.Grid.GetRow(element) };
                        Windows.UI.Xaml.Controls.Grid.MakeGridPositionCorrect(elementRow, maxRow);


                        var rowSpan = Windows.UI.Xaml.Controls.Grid.GetRowSpan(element);
                        if (rowSpan <= 1) {
                            rowSpan = 1;
                        }


                        var style = CSHTML5.Internal.INTERNAL_HtmlDomManager.GetFrameworkElementBoxSizingStyleForModification(element);
                        var isMsGrid = document.isMSGrid;
                        if (!isMsGrid) {
                            var lastRow = { v : (((elementRow.v + rowSpan) | 0) - 1) | 0 };
                            Windows.UI.Xaml.Controls.Grid.MakeGridPositionCorrect(lastRow, maxRow);


                            style.gridRowStart = Bridge.toString((((elementRow.v + 1) | 0)));
                            style.gridRowEnd = Bridge.toString((((lastRow.v + 2) | 0)));
                        } else {
                            style.msGridRow = Bridge.toString((((elementRow.v + 1) | 0)));
                            style.msGridRowSpan = Bridge.toString((rowSpan));
                        }


                    }
                },
                ApplyColumnPosition: function (element) {
                    var maxColumn = 0;
                    if (element.INTERNAL_VisualParent != null && Bridge.is(element.INTERNAL_VisualParent, Windows.UI.Xaml.Controls.Grid)) {
                        var parent = Bridge.cast(element.INTERNAL_VisualParent, Windows.UI.Xaml.Controls.Grid);
                        if (parent._columnDefinitionsOrNull != null && parent._columnDefinitionsOrNull.Count > 0) {
                            maxColumn = (parent._columnDefinitionsOrNull.Count - 1) | 0;


                            var elementColumn = { v : Windows.UI.Xaml.Controls.Grid.GetColumn(element) };
                            Windows.UI.Xaml.Controls.Grid.MakeGridPositionCorrect(elementColumn, maxColumn);


                            var style = CSHTML5.Internal.INTERNAL_HtmlDomManager.GetFrameworkElementBoxSizingStyleForModification(element);
                            var columnSpan = Windows.UI.Xaml.Controls.Grid.GetColumnSpan(element);
                            if (columnSpan <= 1) {
                                columnSpan = 1;
                            }
                            var lastColumn = { v : (((elementColumn.v + columnSpan) | 0) - 1) | 0 };
                            Windows.UI.Xaml.Controls.Grid.MakeGridPositionCorrect(lastColumn, maxColumn);

                            var isMsGrid = document.isMSGrid;
                            if (!isMsGrid) {
                                style.gridColumnStart = Bridge.toString((((elementColumn.v + 1) | 0)));
                                style.gridColumnEnd = Bridge.toString((((lastColumn.v + 2) | 0)));
                            } else {
                                style.msGridColumn = Bridge.toString((((elementColumn.v + 1) | 0)));
                                style.msGridColumnSpan = Bridge.toString((columnSpan));
                            }

                        }
                    }
                },
                RowSpan_Changed_CSSVersion: function (d, e) {
                    var element = Bridge.cast(d, Windows.UI.Xaml.UIElement);
                    Windows.UI.Xaml.Controls.Grid.ApplyRowPosition(element);
                },
                Column_Changed_CSSVersion: function (d, e) {
                    var element = Bridge.cast(d, Windows.UI.Xaml.UIElement);
                    Windows.UI.Xaml.Controls.Grid.ApplyColumnPosition(element);
                },
                ColumnSpan_Changed_CSSVersion: function (d, e) {
                    var element = Bridge.cast(d, Windows.UI.Xaml.UIElement);
                    Windows.UI.Xaml.Controls.Grid.ApplyColumnPosition(element);
                }
            }
        },
        fields: {
            INTERNAL_StringToSetVerticalGridLinesInCss: null,
            INTERNAL_StringToSetHorizontalGridLinesInCss: null,
            _currentCellsStructure: null,
            _innerDiv: null,
            _currentDomTable: null,
            _columnDefinitionsOrNull: null,
            _rowDefinitionsOrNull: null
        },
        props: {
            /**
             * Gets a list of ColumnDefinition objects defined on this instance of Grid.
             *
             * @instance
             * @public
             * @readonly
             * @memberof Windows.UI.Xaml.Controls.Grid
             * @function ColumnDefinitions
             * @type Windows.UI.Xaml.Controls.ColumnDefinitionCollection
             */
            ColumnDefinitions: {
                get: function () {
                    if (this._columnDefinitionsOrNull == null) {
                        this._columnDefinitionsOrNull = new Windows.UI.Xaml.Controls.ColumnDefinitionCollection();
                        this._columnDefinitionsOrNull.addCollectionChanged(Bridge.fn.cacheBind(this, this.ColumnDefinitions_CollectionChanged));
                    }
                    return this._columnDefinitionsOrNull;
                }
            },
            /**
             * Gets a list of RowDefinition objects defined on this instance of Grid.
             *
             * @instance
             * @public
             * @readonly
             * @memberof Windows.UI.Xaml.Controls.Grid
             * @function RowDefinitions
             * @type Windows.UI.Xaml.Controls.RowDefinitionCollection
             */
            RowDefinitions: {
                get: function () {
                    if (this._rowDefinitionsOrNull == null) {
                        this._rowDefinitionsOrNull = new Windows.UI.Xaml.Controls.RowDefinitionCollection();
                        this._rowDefinitionsOrNull.addCollectionChanged(Bridge.fn.cacheBind(this, this.RowDefinitions_CollectionChanged));
                    }
                    return this._rowDefinitionsOrNull;
                }
            }
        },
        ctors: {
            /**
             * Initializes a new instance of the Grid class.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Controls.Grid
             * @memberof Windows.UI.Xaml.Controls.Grid
             * @return  {void}
             */
            ctor: function () {
                this.$initialize();
                Windows.UI.Xaml.Controls.Panel.ctor.call(this);
            }
        },
        methods: {
            ColumnDefinitions_CollectionChanged: function (sender, e) {
                var $t;
                if (e.NewItems != null) {
                    $t = Bridge.getEnumerator(e.NewItems);
                    try {
                        while ($t.moveNext()) {
                            var columnDefinition = Bridge.cast($t.Current, Windows.UI.Xaml.Controls.ColumnDefinition);
                            columnDefinition.Parent = this;
                        }
                    } finally {
                        if (Bridge.is($t, System.IDisposable)) {
                            $t.System$IDisposable$Dispose();
                        }
                    }
                }

                var isCSSGrid = document.isGridSupported;
                if (isCSSGrid) {
                    this.ColumnDefinitions_CollectionChanged_CSSVersion(sender, e);
                } else {
                    this.ColumnDefinitions_CollectionChanged_NonCSSVersion(sender, e);
                }
            },
            ColumnDefinitions_CollectionChanged_NonCSSVersion: function (sender, e) {
                if (this._isLoaded) {
                    this.RebuildDomStructure_NonCSSVersion(true);
                    this.LocallyManageChildrenChanged();
                }
            },
            RowDefinitions_CollectionChanged: function (sender, e) {
                var $t;
                if (e.NewItems != null) {
                    $t = Bridge.getEnumerator(e.NewItems);
                    try {
                        while ($t.moveNext()) {
                            var rowDefinition = Bridge.cast($t.Current, Windows.UI.Xaml.Controls.RowDefinition);
                            rowDefinition.Parent = this;
                        }
                    } finally {
                        if (Bridge.is($t, System.IDisposable)) {
                            $t.System$IDisposable$Dispose();
                        }
                    }
                }
                var isCSSGrid = document.isGridSupported;
                if (isCSSGrid) {
                    this.RowDefinitions_CollectionChanged_CSSVersion(sender, e);
                } else {
                    this.RowDefinitions_CollectionChanged_NonCSSVersion(sender, e);
                }
            },
            RowDefinitions_CollectionChanged_NonCSSVersion: function (sender, e) {
                if (this._isLoaded) {
                    this.RebuildDomStructure_NonCSSVersion(true);
                    this.LocallyManageChildrenChanged();
                }
            },
            UpdateStructureWhenAddingRow: function (rowDefinition) {

            },
            UpdateStructureWhenRemovingRow: function (rowDefinition) {

            },
            CreateDomElement: function (parentRef, domElementWhereToPlaceChildren) {
                var isCSSGrid = document.isGridSupported;
                if (isCSSGrid) {
                    return this.CreateDomElement_CSSVersion(parentRef, domElementWhereToPlaceChildren);
                } else {
                    var outerDiv = this.CreateDomElement_WorkaroundBridgeInheritanceBug(parentRef, Bridge.ref(this, "_innerDiv"));
                    domElementWhereToPlaceChildren.v = this._innerDiv;
                    return outerDiv;
                }
            },
            INTERNAL_UpdateDomStructureIfNecessary: function () {

                var isCSSGrid = document.isGridSupported;
                if (isCSSGrid) {
                    Windows.UI.Xaml.Controls.Grid_InternalHelpers.RefreshAllColumnsVisibility_CSSVersion(this);
                } else {
                    this.RebuildDomStructure_NonCSSVersion(false);
                }
            },
            RebuildDomStructure_NonCSSVersion: function (detachAndReattachTheChildrenIfNecessary) {
                var $t, $t1, $t2;
                var newCellsStructure = Windows.UI.Xaml.Controls.Grid_InternalHelpers.CalculateNewCellsStructure(this._columnDefinitionsOrNull, this._rowDefinitionsOrNull);

                var childrenToUpdateIfStructureDifferent = Windows.UI.Xaml.Controls.Grid_InternalHelpers.HandleSpansInCellsStructureAndReturnRedirectedElements(newCellsStructure, this.Children);

                if (!this.AreCellsStructuresIdentical(this._currentCellsStructure, newCellsStructure)) {
                    if (detachAndReattachTheChildrenIfNecessary) {
                        $t = Bridge.getEnumerator(this.Children);
                        try {
                            while ($t.moveNext()) {
                                var child = $t.Current;
                                CSHTML5.Internal.INTERNAL_VisualTreeManager.DetachVisualChildIfNotNull(child, this);
                            }
                        } finally {
                            if (Bridge.is($t, System.IDisposable)) {
                                $t.System$IDisposable$Dispose();
                            }
                        }
                    }


                    if (this._currentDomTable != null) {
                        CSHTML5.Internal.INTERNAL_HtmlDomManager.RemoveFromDom(this._currentDomTable);
                        this._currentDomTable = null;
                    }

                    this._currentCellsStructure = newCellsStructure;

                    var normalizedColumnDefinitions = { };
                    var normalizedRowDefinitions = { };
                    this._currentDomTable = Windows.UI.Xaml.Controls.Grid_InternalHelpers.GenerateDomElementsForGrid_NonCSSVersion(this, newCellsStructure, this._columnDefinitionsOrNull, this._rowDefinitionsOrNull, this._innerDiv, normalizedColumnDefinitions, normalizedRowDefinitions);

                    Windows.UI.Xaml.Controls.Grid_InternalHelpers.RefreshAllRowsHeight_NonCSSVersion(this, normalizedRowDefinitions.v);
                    Windows.UI.Xaml.Controls.Grid_InternalHelpers.RefreshAllColumnsWidth_NonCSSVersion(this, normalizedColumnDefinitions.v);

                    $t1 = Bridge.getEnumerator(childrenToUpdateIfStructureDifferent.Keys);
                    try {
                        while ($t1.moveNext()) {
                            var key = $t1.Current;
                            key.INTERNAL_SpanParentCell = childrenToUpdateIfStructureDifferent.getItem(key);
                        }
                    } finally {
                        if (Bridge.is($t1, System.IDisposable)) {
                            $t1.System$IDisposable$Dispose();
                        }
                    }

                    if (detachAndReattachTheChildrenIfNecessary) {
                        $t2 = Bridge.getEnumerator(this.Children);
                        try {
                            while ($t2.moveNext()) {
                                var child1 = $t2.Current;
                                CSHTML5.Internal.INTERNAL_VisualTreeManager.AttachVisualChildIfNotAlreadyAttached(child1, this);
                            }
                        } finally {
                            if (Bridge.is($t2, System.IDisposable)) {
                                $t2.System$IDisposable$Dispose();
                            }
                        }
                    }
                }
            },
            UpdateStructureWhenAddingChild: function (child) {
                var $t;
                var columnSpan = Windows.UI.Xaml.Controls.Grid.GetColumnSpan(child);
                var rowSpan = Windows.UI.Xaml.Controls.Grid.GetRowSpan(child);

                var childColumn = Windows.UI.Xaml.Controls.Grid.GetColumn(child);
                var childRow = Windows.UI.Xaml.Controls.Grid.GetRow(child);
                var maxColumn = (this.ColumnDefinitions.Count - 1) | 0;
                var maxRow = (this.RowDefinitions.Count - 1) | 0;
                if (childRow > maxRow) {
                    childRow = maxRow;
                }
                if (childRow < 0) {
                    childRow = 0;
                }
                if (childColumn > maxColumn) {
                    childColumn = maxColumn;
                }
                if (childColumn < 0) {
                    childColumn = 0;
                }
                var cell = this._currentCellsStructure.getItem(childRow).getItem(childColumn);

                if (cell.IsOverlapped) {
                    child.INTERNAL_SpanParentCell = cell.ParentCell;
                    ($t = child.INTERNAL_SpanParentCell).numberOfChildren = ($t.numberOfChildren + 1) | 0;
                } else {
                    var childLastRow = (((childRow + Windows.UI.Xaml.Controls.Grid.GetRowSpan(child)) | 0) - 1) | 0;
                    var childLastColumn = (((childColumn + Windows.UI.Xaml.Controls.Grid.GetColumnSpan(child)) | 0) - 1) | 0;

                    if (childLastRow > maxRow) {
                        childLastRow = maxRow;
                    }
                    if (childLastRow < childRow) {
                        childLastRow = childRow;
                    }
                    if (childLastColumn > maxColumn) {
                        childLastColumn = maxColumn;
                    }
                    if (childLastColumn < childColumn) {
                        childLastColumn = childColumn;
                    }
                    var parentCell = null;
                    var isFirst = true;
                    for (var row = childRow; row <= childLastRow; row = (row + 1) | 0) {
                        for (var column = childColumn; column <= childLastColumn; column = (column + 1) | 0) {
                            var currentCell = this._currentCellsStructure.getItem(row).getItem(column);
                            if (currentCell.IsOverlapped || (currentCell.IsOccupied && !isFirst)) {
                                if (currentCell.IsOverlapped) {
                                    parentCell = currentCell.ParentCell;
                                } else {
                                    parentCell = currentCell;
                                }
                                child.INTERNAL_SpanParentCell = parentCell;
                                parentCell.numberOfChildren = (parentCell.numberOfChildren + 1) | 0;
                                break;
                            } else {
                            }
                            isFirst = false;
                        }
                        if (parentCell != null) {
                            break;
                        }
                    }
                    if (parentCell == null) {
                        this.MakeCellSpan(childRow, childColumn, ((((childLastRow - childRow) | 0) + 1) | 0), ((((childLastColumn - childColumn) | 0) + 1) | 0));
                        cell.numberOfChildren = (cell.numberOfChildren + 1) | 0;
                    }
                }
            },
            MakeCellSpan: function (cellRow, cellColumn, cellRowSpan, cellColumnSpan) {
                var isFirst = true;
                var parentCell = null;
                var cellLastRow = (((cellRow + cellRowSpan) | 0) - 1) | 0;
                var cellLastColumn = (((cellColumn + cellColumnSpan) | 0) - 1) | 0;
                for (var row = cellRow; row <= cellLastRow; row = (row + 1) | 0) {
                    for (var column = cellColumn; column <= cellLastColumn; column = (column + 1) | 0) {
                        var cell2 = this._currentCellsStructure.getItem(row).getItem(column);
                        if (isFirst) {
                            parentCell = cell2;
                            cell2.IsOccupied = true;
                            cell2.RowSpan = cellRowSpan;
                            cell2.ColumnSpan = cellColumnSpan;
                            if (cell2.ColumnDomElement != null) {
                                var td = cell2.ColumnDomElement;
                                CSHTML5.Internal.INTERNAL_HtmlDomManager.SetDomElementAttribute(td, "rowspan", Bridge.box(cellRowSpan, System.Int32));
                                CSHTML5.Internal.INTERNAL_HtmlDomManager.SetDomElementAttribute(td, "colspan", Bridge.box(cellColumnSpan, System.Int32));

                                var domElementStyleAppliedValue = { v : null };


                                Windows.UI.Xaml.Controls.Grid_InternalHelpers.RefreshCellWithSpanHeight(this, cell2, domElementStyleAppliedValue);

                                if (domElementStyleAppliedValue.v != null) {
                                    var domElementStyle = CSHTML5.Internal.INTERNAL_HtmlDomManager.GetDomElementStyleForModification(cell2.DomElement);
                                    domElementStyle.height = domElementStyleAppliedValue.v;
                                }

                            }
                            isFirst = false;
                        } else {
                            cell2.IsOverlapped = true;
                            cell2.ParentCell = parentCell;
                            if (cell2.ColumnDomElement != null) {
                                var td1 = cell2.ColumnDomElement;
                                CSHTML5.Internal.INTERNAL_HtmlDomManager.RemoveFromDom(td1);
                                cell2.ColumnDomElement = null;
                            }
                        }
                    }
                }
            },
            UpdateStructureWhenRemovingChild: function (child) {
                var cell;
                if (child.INTERNAL_SpanParentCell != null) {
                    cell = child.INTERNAL_SpanParentCell;
                    child.INTERNAL_SpanParentCell = null;
                } else {
                    var childColumn = Windows.UI.Xaml.Controls.Grid.GetColumn(child);
                    var childRow = Windows.UI.Xaml.Controls.Grid.GetRow(child);
                    var maxColumn = (this.ColumnDefinitions.Count - 1) | 0;
                    var maxRow = (this.RowDefinitions.Count - 1) | 0;
                    if (childRow > maxRow) {
                        childRow = maxRow;
                    }
                    if (childRow < 0) {
                        childRow = 0;
                    }
                    if (childColumn > maxColumn) {
                        childColumn = maxColumn;
                    }
                    if (childColumn < 0) {
                        childColumn = 0;
                    }
                    cell = this._currentCellsStructure.getItem(childRow).getItem(childColumn);
                }
                cell.numberOfChildren = (cell.numberOfChildren - 1) | 0;
                if (cell.numberOfChildren === 0) {
                    if (cell.RowSpan !== 1 || cell.ColumnSpan !== 1) {
                        this.RemoveCellSpan(cell);
                    }
                    cell.IsOccupied = false;
                }

            },
            RemoveCellSpan: function (cell) {
                var cellRow = cell.Row;
                var cellColumn = cell.Column;
                var cellRowSpan = cell.RowSpan;
                var cellColumnSpan = cell.ColumnSpan;
                var maxRow = (((cellRow + cell.RowSpan) | 0) - 1) | 0;
                var maxColumn = (((cellColumn + cell.ColumnSpan) | 0) - 1) | 0;
                var isFirst = true;

                var normalizedColumnDefinitions = { };
                var normalizedRowDefinitions = { };
                Windows.UI.Xaml.Controls.Grid_InternalHelpers.NormalizeWidthAndHeightPercentages(this, this._columnDefinitionsOrNull, this._rowDefinitionsOrNull, normalizedColumnDefinitions, normalizedRowDefinitions);
                var isTheOnlyRow = this._rowDefinitionsOrNull.Count <= 1;
                var isTheOnlyColumn = this._columnDefinitionsOrNull.Count <= 1;

                for (var row = cellRow; row <= maxRow; row = (row + 1) | 0) {
                    var columnDomIndex = 0;
                    for (var i = 0; i < cellColumn; i = (i + 1) | 0) {
                        var currentCell = this._currentCellsStructure.getItem(row).getItem(i);
                        if (!currentCell.IsOverlapped) {
                            columnDomIndex = (columnDomIndex + 1) | 0;
                        }
                    }
                    for (var column = cellColumn; column <= maxColumn; column = (column + 1) | 0) {
                        var currentCell1 = this._currentCellsStructure.getItem(row).getItem(column);
                        if (isFirst) {
                            currentCell1.RowSpan = 1;
                            currentCell1.ColumnSpan = 1;
                            currentCell1.IsOccupied = false;
                            if (currentCell1.ColumnDomElement != null) {
                                var td = currentCell1.ColumnDomElement;
                                CSHTML5.Internal.INTERNAL_HtmlDomManager.SetDomElementAttribute(td, "rowspan", Bridge.box(1, System.Int32));
                                CSHTML5.Internal.INTERNAL_HtmlDomManager.SetDomElementAttribute(td, "colspan", Bridge.box(1, System.Int32));

                            }
                            isFirst = false;
                        } else {
                            currentCell1.IsOverlapped = false;
                            currentCell1.ParentCell = null;

                            var td1;
                            if (columnDomIndex === 0) {
                                td1 = CSHTML5.Internal.INTERNAL_HtmlDomManager.CreateDomElementAndInsertIt("td", currentCell1.RowDomElement, this, columnDomIndex, "beforeBegin");
                            } else {

                                td1 = CSHTML5.Internal.INTERNAL_HtmlDomManager.CreateDomElementAndInsertIt("td", currentCell1.RowDomElement, this, ((columnDomIndex - 1) | 0), "afterEnd");
                            }
                            var tdStyle = CSHTML5.Internal.INTERNAL_HtmlDomManager.GetDomElementStyleForModification(td1);
                            var columnDefinition = this.ColumnDefinitions.getItem(currentCell1.Column);
                            if (columnDefinition.Visibility === Windows.UI.Xaml.Visibility.Visible) {
                                tdStyle.display = "table-cell";
                            } else {
                                tdStyle.display = "none";
                            }


                            Windows.UI.Xaml.Controls.Grid_InternalHelpers.ApplyGridLinesValues(this, row, column, tdStyle);

                            var div = CSHTML5.Internal.INTERNAL_HtmlDomManager.CreateDomElementAndAppendIt("div", td1, this);
                            var divStyle = CSHTML5.Internal.INTERNAL_HtmlDomManager.GetDomElementStyleForModification(div);
                            divStyle.position = "relative";


                            currentCell1.DomElement = div;
                            currentCell1.ColumnDomElement = td1;

                            Windows.UI.Xaml.Controls.Grid_InternalHelpers.RefreshCellWidthAndHeight(this, currentCell1, normalizedColumnDefinitions.v.getItem(column), isTheOnlyColumn, normalizedRowDefinitions.v.getItem(row), isTheOnlyRow);
                        }
                        if (!currentCell1.IsOverlapped) {
                            columnDomIndex = (columnDomIndex + 1) | 0;
                        }
                    }
                }
            },
            AreCellsStructuresIdentical: function (cellsStructure1, cellsStructure2) {
                var $t, $t1;
                if (cellsStructure1 == null && cellsStructure2 != null) {
                    return false;
                }
                if (cellsStructure1 != null && cellsStructure2 == null) {
                    return false;
                }
                if (cellsStructure1 == null && cellsStructure2 == null) {
                    return true;
                }

                if (cellsStructure1.Count !== cellsStructure2.Count) {
                    return false;
                }

                var listIndex = 0;
                $t = Bridge.getEnumerator(cellsStructure1);
                try {
                    while ($t.moveNext()) {
                        var list1 = $t.Current;
                        var list2 = cellsStructure2.getItem(listIndex);
                        if (list1.Count !== list2.Count) {
                            return false;
                        }
                        var elementIndex = 0;
                        $t1 = Bridge.getEnumerator(list1);
                        try {
                            while ($t1.moveNext()) {
                                var cellDefinition1 = $t1.Current;
                                if (!this.AreCellsDefinitionsIdentical(cellDefinition1, list2.getItem(elementIndex))) {
                                    return false;
                                }
                                elementIndex = (elementIndex + 1) | 0;
                            }
                        } finally {
                            if (Bridge.is($t1, System.IDisposable)) {
                                $t1.System$IDisposable$Dispose();
                            }
                        }
                        listIndex = (listIndex + 1) | 0;
                    }
                } finally {
                    if (Bridge.is($t, System.IDisposable)) {
                        $t.System$IDisposable$Dispose();
                    }
                }
                return true;
            },
            AreCellsDefinitionsIdentical: function (cellDefinition1, cellDefinition2) {
                if (cellDefinition1.ColumnSpan !== cellDefinition2.ColumnSpan) {
                    return false;
                }
                if (cellDefinition1.IsOverlapped !== cellDefinition2.IsOverlapped) {
                    return false;
                }
                if (cellDefinition1.RowSpan !== cellDefinition2.RowSpan) {
                    return false;
                }
                return true;
            },
            INTERNAL_OnDetachedFromVisualTree: function () {
                Windows.UI.Xaml.Controls.Panel.prototype.INTERNAL_OnDetachedFromVisualTree.call(this);

                this._currentCellsStructure = null;
                this._innerDiv = null;
                this._currentDomTable = null;
            },
            LocallyManageChildrenChanged: function () {
                var $t, $t1;
                var isCSSGrid = document.isGridSupported;
                if (isCSSGrid) {
                    this.LocallyManageChildrenChanged_CSSVersion();
                } else {
                    var thereAreRowDefinitions = (this._rowDefinitionsOrNull != null && this._rowDefinitionsOrNull.Count > 0);
                    var thereAreColumnDefinitions = (this._columnDefinitionsOrNull != null && this._columnDefinitionsOrNull.Count > 0);
                    if (thereAreRowDefinitions || thereAreColumnDefinitions) {
                        var amountOfRows = 1;
                        if (thereAreRowDefinitions) {
                            amountOfRows = this._rowDefinitionsOrNull.Count;
                        }

                        var amountOfColumns = 1;
                        if (thereAreColumnDefinitions) {
                            amountOfColumns = this._columnDefinitionsOrNull.Count;
                        }

                        var lastElements = System.Array.create(null, null, Windows.UI.Xaml.UIElement, amountOfRows, amountOfColumns);
                        $t = Bridge.getEnumerator(this.Children);
                        try {
                            while ($t.moveNext()) {
                                var uiElement = $t.Current;
                                if (CSHTML5.Internal.INTERNAL_VisualTreeManager.IsElementInVisualTree(uiElement)) {
                                    var elementRow;
                                    var elementColumn;
                                    if (uiElement.INTERNAL_SpanParentCell != null) {
                                        var cell = uiElement.INTERNAL_SpanParentCell;
                                        if (cell.ParentCell != null) {
                                            cell = cell.ParentCell;
                                        }
                                        elementRow = cell.Row;
                                        elementColumn = cell.Column;
                                    } else {
                                        elementRow = Windows.UI.Xaml.Controls.Grid.GetRow(uiElement);
                                        if (elementRow >= amountOfRows) {
                                            elementRow = (amountOfRows - 1) | 0;
                                        }
                                        elementColumn = Windows.UI.Xaml.Controls.Grid.GetColumn(uiElement);
                                        if (elementColumn >= amountOfColumns) {
                                            elementColumn = (amountOfColumns - 1) | 0;
                                        }
                                    }
                                    if (lastElements.get([elementRow, elementColumn]) != null) {
                                        var style = CSHTML5.Internal.INTERNAL_HtmlDomManager.GetDomElementStyleForModification(this.INTERNAL_VisualChildrenInformation.getItem(lastElements.get([elementRow, elementColumn])).INTERNAL_OptionalChildWrapper_OuterDomElement);
                                        style.position = "absolute";
                                    }
                                    var style2 = CSHTML5.Internal.INTERNAL_HtmlDomManager.GetDomElementStyleForModification(this.INTERNAL_VisualChildrenInformation.getItem(uiElement).INTERNAL_OptionalChildWrapper_OuterDomElement);
                                    style2.position = "relative";
                                    lastElements.set([elementRow, elementColumn], uiElement);
                                }
                            }
                        } finally {
                            if (Bridge.is($t, System.IDisposable)) {
                                $t.System$IDisposable$Dispose();
                            }
                        }
                    } else {
                        var i = 0;
                        $t1 = Bridge.getEnumerator(this.Children);
                        try {
                            while ($t1.moveNext()) {
                                var uiElement1 = $t1.Current;
                                if (CSHTML5.Internal.INTERNAL_VisualTreeManager.IsElementInVisualTree(uiElement1)) {
                                    var style1 = CSHTML5.Internal.INTERNAL_HtmlDomManager.GetDomElementStyleForModification(this.INTERNAL_VisualChildrenInformation.getItem(uiElement1).INTERNAL_OptionalChildWrapper_OuterDomElement);

                                    if (i < ((this.Children.Count - 1) | 0)) {
                                        style1.position = "absolute";
                                    } else {
                                        style1.position = "relative";
                                    }
                                }
                                i = (i + 1) | 0;
                            }
                        } finally {
                            if (Bridge.is($t1, System.IDisposable)) {
                                $t1.System$IDisposable$Dispose();
                            }
                        }
                    }
                }
            },
            OnChildrenAdded: function (newChild, index) {
                if (!document.isGridSupported) {
                    if (CSHTML5.Internal.INTERNAL_VisualTreeManager.IsElementInVisualTree(this)) {
                        this.UpdateStructureWhenAddingChild(newChild);
                        CSHTML5.Internal.INTERNAL_VisualTreeManager.AttachVisualChildIfNotAlreadyAttached(newChild, this, index);
                        this.LocallyManageChildrenChanged();
                    }
                } else {
                    Windows.UI.Xaml.Controls.Panel.prototype.OnChildrenAdded.call(this, newChild, index);
                    this.LocallyManageChildrenChanged();
                }
            },
            OnChildrenRemoved: function (oldChild, index) {
                if (!document.isGridSupported) {
                    if (CSHTML5.Internal.INTERNAL_VisualTreeManager.IsElementInVisualTree(this)) {
                        this.UpdateStructureWhenRemovingChild(oldChild);
                        CSHTML5.Internal.INTERNAL_VisualTreeManager.DetachVisualChildIfNotNull(oldChild, this);
                        this.LocallyManageChildrenChanged();
                    }
                } else {
                    Windows.UI.Xaml.Controls.Panel.prototype.OnChildrenRemoved.call(this, oldChild, index);
                    this.LocallyManageChildrenChanged();
                }
            },
            OnChildrenReplaced: function (oldChild, newChild, index) {
                if (Bridge.referenceEquals(oldChild, newChild)) {
                    return;
                }

                if (!document.isGridSupported) {
                    this.UpdateStructureWhenRemovingChild(oldChild);
                    CSHTML5.Internal.INTERNAL_VisualTreeManager.DetachVisualChildIfNotNull(oldChild, this);

                    this.UpdateStructureWhenAddingChild(newChild);
                    CSHTML5.Internal.INTERNAL_VisualTreeManager.AttachVisualChildIfNotAlreadyAttached(newChild, this, index);

                    this.LocallyManageChildrenChanged();
                } else {
                    Windows.UI.Xaml.Controls.Panel.prototype.OnChildrenReplaced.call(this, oldChild, newChild, index);
                    this.LocallyManageChildrenChanged();
                }
            },
            OnChildrenReset: function () {
                var $t;
                if (!document.isGridSupported) {
                    if (this.INTERNAL_VisualChildrenInformation != null) {
                        $t = Bridge.getEnumerator(System.Linq.Enumerable.from(System.Linq.Enumerable.from(this.INTERNAL_VisualChildrenInformation).select(function (kp) {
                                    return kp.value;
                                })).ToArray());
                        try {
                            while ($t.moveNext()) {
                                var childInfo = $t.Current;
                                this.UpdateStructureWhenRemovingChild(childInfo.INTERNAL_UIElement);
                                CSHTML5.Internal.INTERNAL_VisualTreeManager.DetachVisualChildIfNotNull(childInfo.INTERNAL_UIElement, this);
                            }
                        } finally {
                            if (Bridge.is($t, System.IDisposable)) {
                                $t.System$IDisposable$Dispose();
                            }
                        }
                    }

                    for (var i = 0; i < this.Children.Count; i = (i + 1) | 0) {
                        this.UpdateStructureWhenAddingChild(this.Children.getItem(i));
                        CSHTML5.Internal.INTERNAL_VisualTreeManager.AttachVisualChildIfNotAlreadyAttached(this.Children.getItem(i), this, i);
                    }

                    this.LocallyManageChildrenChanged();
                } else {
                    Windows.UI.Xaml.Controls.Panel.prototype.OnChildrenReset.call(this);
                    this.LocallyManageChildrenChanged();
                }
            },
            GetDomElementWhereToPlaceChild: function (child) {
                var isCSSGrid = document.isGridSupported;
                if (isCSSGrid) {
                    return Windows.UI.Xaml.Controls.Panel.prototype.GetDomElementWhereToPlaceChild.call(this, child);
                } else {
                    var spanParentCell = child.INTERNAL_SpanParentCell;
                    if (spanParentCell != null) {
                        if (spanParentCell.IsOverlapped) {
                            spanParentCell = spanParentCell.ParentCell;
                        }
                        if (spanParentCell.DomElement != null) {
                            return spanParentCell.DomElement;
                        }
                    }
                    var columnIndex = Windows.UI.Xaml.Controls.Grid.GetColumn(child);
                    var rowIndex = Windows.UI.Xaml.Controls.Grid.GetRow(child);

                    if (rowIndex >= this._currentCellsStructure.Count) {
                        rowIndex = (this._currentCellsStructure.Count - 1) | 0;
                    }

                    var row = this._currentCellsStructure.getItem(rowIndex);

                    if (columnIndex >= row.Count) {
                        columnIndex = (row.Count - 1) | 0;
                    }

                    var cell = row.getItem(columnIndex);

                    return cell.DomElement;
                }
            },
            CreateDomChildWrapper: function (parentRef, domElementWhereToPlaceChild, index) {
                if (index === void 0) { index = -1; }
                var isCSSGrid = document.isGridSupported;
                if (isCSSGrid) {
                    return Windows.UI.Xaml.Controls.Panel.prototype.CreateDomChildWrapper.call(this, parentRef, domElementWhereToPlaceChild);
                } else {

                    var outerDiv = { };
                    var outerDivStyle = CSHTML5.Internal.INTERNAL_HtmlDomManager.CreateDomElementAppendItAndGetStyle("div", parentRef, this, outerDiv);
                    outerDivStyle.position = "absolute";
                    outerDivStyle.height = "100%";
                    outerDivStyle.width = "100%";


                    outerDivStyle.pointerEvents = "none";
                    var innerDiv = { };
                    var innerDivStyle = CSHTML5.Internal.INTERNAL_HtmlDomManager.CreateDomElementAppendItAndGetStyle("div", outerDiv.v, this, innerDiv);
                    innerDivStyle.position = "relative";
                    innerDivStyle.height = "100%";
                    innerDivStyle.width = "100%";


                    domElementWhereToPlaceChild.v = innerDiv.v;
                    return outerDiv.v;
                }
            },
            GetColumnActualWidth: function (columnDefinition) {
                var $t;
                var isCSSGrid = document.isGridSupported;
                if (isCSSGrid) {
                    return this.GetColumnActualWidth_CSSVersion(columnDefinition);
                } else {
                    if (CSHTML5.Internal.INTERNAL_VisualTreeManager.IsElementInVisualTree(this) && this.INTERNAL_OuterDomElement != null && this._columnDefinitionsOrNull != null) {
                        var columnIndex = this._columnDefinitionsOrNull.indexOf(columnDefinition);

                        var cell = this._currentCellsStructure.getItem(0).getItem(columnIndex);
                        $t = Bridge.getEnumerator(this._currentCellsStructure);
                        try {
                            while ($t.moveNext()) {
                                var cells = $t.Current;
                                var currentCell = cells.getItem(columnIndex);
                                if (!currentCell.IsOverlapped && currentCell.ColumnSpan === 1) {
                                    cell = currentCell;
                                }
                            }
                        } finally {
                            if (Bridge.is($t, System.IDisposable)) {
                                $t.System$IDisposable$Dispose();
                            }
                        }
                        var columnDomElement = cell.ColumnDomElement;
                        if (CSharpXamlForHtml5.Environment.IsRunningInJavaScript) {
                            return columnDomElement.offsetWidth;
                        } else {
                            return System.Convert.toDouble(CSHTML5.Internal.INTERNAL_HtmlDomManager.GetDomElementAttribute(columnDomElement, "offsetWidth"));
                        }
                    } else {
                        return Number.NaN;
                    }
                }
            },
            GetRowActualHeight: function (rowDefinition) {
                var isCSSGrid = document.isGridSupported;
                if (isCSSGrid) {
                    return this.GetRowActualHeight_CSSVersion(rowDefinition);
                } else {
                    if (CSHTML5.Internal.INTERNAL_VisualTreeManager.IsElementInVisualTree(this) && this.INTERNAL_OuterDomElement != null && this._rowDefinitionsOrNull != null) {
                        var rowIndex = this._rowDefinitionsOrNull.indexOf(rowDefinition);
                        var rowDomElement = this._currentCellsStructure.getItem(rowIndex).getItem(0).RowDomElement;
                        if (CSharpXamlForHtml5.Environment.IsRunningInJavaScript) {
                            return rowDomElement.offsetHeight;
                        }
                        return 0;
                    } else {
                        return Number.NaN;
                    }
                }
            },
            ColumnDefinitions_CollectionChanged_CSSVersion: function (sender, e) {
                if (CSHTML5.Internal.INTERNAL_VisualTreeManager.IsElementInVisualTree(this)) {
                    if (e.NewItems != null) {
                        Windows.UI.Xaml.Controls.Grid_InternalHelpers.RefreshAllColumnsWidth_CSSVersion(this);
                    }

                    if (this._isLoaded) {
                        Windows.UI.Xaml.Controls.Grid_InternalHelpers.RefreshAllColumnsWidth_CSSVersion(this);

                        this.LocallyManageChildrenChanged();
                    }
                }
            },
            RowDefinitions_CollectionChanged_CSSVersion: function (sender, e) {
                if (this._isLoaded) {
                    Windows.UI.Xaml.Controls.Grid_InternalHelpers.RefreshAllRowsHeight_CSSVersion(this);

                    this.LocallyManageChildrenChanged();
                }
            },
            CreateDomElement_CSSVersion: function (parentRef, domElementWhereToPlaceChildren) {
                var outerDiv = this.CreateDomElement_WorkaroundBridgeInheritanceBug(parentRef, Bridge.ref(this, "_innerDiv"));
                domElementWhereToPlaceChildren.v = this._innerDiv;

                this.INTERNAL_OptionalSpecifyDomElementConcernedByMinMaxHeightAndWidth = this._innerDiv;

                var style = CSHTML5.Internal.INTERNAL_HtmlDomManager.GetDomElementStyleForModification(this._innerDiv);
                style.display = !document.isMSGrid ? (style.display = "grid", "grid") : "-ms-grid";

                var normalizedColumnDefinitions = { v : null };
                var normalizedRowDefinitions = { v : null };
                Windows.UI.Xaml.Controls.Grid_InternalHelpers.NormalizeWidthAndHeightPercentages(this, this._columnDefinitionsOrNull, this._rowDefinitionsOrNull, normalizedColumnDefinitions, normalizedRowDefinitions);

                Windows.UI.Xaml.Controls.Grid_InternalHelpers.RefreshAllRowsHeight_CSSVersion(this, normalizedColumnDefinitions.v, normalizedRowDefinitions.v);
                Windows.UI.Xaml.Controls.Grid_InternalHelpers.RefreshAllColumnsWidth_CSSVersion(this, normalizedColumnDefinitions.v, normalizedRowDefinitions.v);

                return outerDiv;
            },
            LocallyManageChildrenChanged_CSSVersion: function () {
                var $t;
                var maxRow = 0;
                var maxColumn = 0;
                if (this.RowDefinitions != null && this.RowDefinitions.Count > 0) {
                    maxRow = (this.RowDefinitions.Count - 1) | 0;
                }
                if (this.ColumnDefinitions != null && this.ColumnDefinitions.Count > 0) {
                    maxColumn = (this.ColumnDefinitions.Count - 1) | 0;
                }



                $t = Bridge.getEnumerator(this.Children);
                try {
                    while ($t.moveNext()) {
                        var uiElement = $t.Current;
                        if (CSHTML5.Internal.INTERNAL_VisualTreeManager.IsElementInVisualTree(uiElement)) {
                            var elementRow = { v : Windows.UI.Xaml.Controls.Grid.GetRow(uiElement) };
                            var elementColumn = { v : Windows.UI.Xaml.Controls.Grid.GetColumn(uiElement) };
                            Windows.UI.Xaml.Controls.Grid.MakeGridPositionCorrect(elementRow, maxRow);
                            Windows.UI.Xaml.Controls.Grid.MakeGridPositionCorrect(elementColumn, maxColumn);

                            var rowSpan = Windows.UI.Xaml.Controls.Grid.GetRowSpan(uiElement);
                            if (rowSpan < 1) {
                                rowSpan = 1;
                            }
                            var columnSpan = Windows.UI.Xaml.Controls.Grid.GetColumnSpan(uiElement);
                            if (columnSpan < 1) {
                                columnSpan = 1;
                            }
                            var elementLastRow = { v : (((rowSpan + elementRow.v) | 0) - 1) | 0 };
                            var elementLastColumn = { v : (((columnSpan + elementColumn.v) | 0) - 1) | 0 };
                            Windows.UI.Xaml.Controls.Grid.MakeGridPositionCorrect(elementLastRow, maxRow);
                            Windows.UI.Xaml.Controls.Grid.MakeGridPositionCorrect(elementLastColumn, maxColumn);

                            var style = CSHTML5.Internal.INTERNAL_HtmlDomManager.GetFrameworkElementBoxSizingStyleForModification(uiElement);

                            style.zIndex = Bridge.toString(Windows.UI.Xaml.Controls.Canvas.GetZIndex(uiElement));
                            style.position = "relative";

                            var isMsGrid = document.isMSGrid;
                            if (!isMsGrid) {
                                style.gridRowStart = Bridge.toString((((elementRow.v + 1) | 0)));
                                style.gridColumnStart = Bridge.toString((((elementColumn.v + 1) | 0)));
                                style.gridRowEnd = Bridge.toString((((elementLastRow.v + 2) | 0)));
                                style.gridColumnEnd = Bridge.toString((((elementLastColumn.v + 2) | 0)));
                            } else {
                                style.msGridRow = Bridge.toString((((elementRow.v + 1) | 0)));
                                style.msGridColumn = Bridge.toString((((elementColumn.v + 1) | 0)));
                                style.msGridRowSpan = Bridge.toString((rowSpan));
                                style.msGridColumnSpan = Bridge.toString((columnSpan));
                            }

                            style.pointerEvents = "none";


                            if (Windows.UI.Xaml.Controls.Grid_InternalHelpers.IsAllCollapsedColumns(this, elementColumn.v, elementLastColumn.v)) {
                                uiElement.INTERNAL_OuterDomElement.style.overflow = 'hidden';
                                uiElement.INTERNAL_OuterDomElement.setAttribute('data-isCollapsedDueToHiddenColumn', true);
                            } else {

if (                                uiElement.INTERNAL_OuterDomElement.getAttribute('data-isCollapsedDueToHiddenColumn' == true)){
    uiElement.INTERNAL_OuterDomElement.style.overflow = 'visible';
    uiElement.INTERNAL_OuterDomElement.setAttribute('data-isCollapsedDueToHiddenColumn', false);
};
                            }
                        }
                    }
                } finally {
                    if (Bridge.is($t, System.IDisposable)) {
                        $t.System$IDisposable$Dispose();
                    }
                }
            },
            GetColumnActualWidth_CSSVersion: function (columnDefinition) {
                var returnValue = Number.NaN;

                var columnIndex = this._columnDefinitionsOrNull.indexOf(columnDefinition);

                var div1 = this.AddTemporaryDivForRowOrColumnDimensions(columnIndex, 0);

                if (CSharpXamlForHtml5.Environment.IsRunningInJavaScript) {
                    returnValue = div1.offsetWidth;
                } else {
                    returnValue = System.Convert.toDouble(CSHTML5.Internal.INTERNAL_HtmlDomManager.GetDomElementAttribute(div1, "offsetWidth"));
                }

                CSHTML5.Internal.INTERNAL_HtmlDomManager.RemoveFromDom(div1);

                return returnValue;
            },
            GetRowActualHeight_CSSVersion: function (rowDefinition) {
                var returnValue = Number.NaN;

                var rowIndex = this._rowDefinitionsOrNull.indexOf(rowDefinition);

                var div1 = this.AddTemporaryDivForRowOrColumnDimensions(0, rowIndex);

                if (CSharpXamlForHtml5.Environment.IsRunningInJavaScript) {
                    returnValue = div1.offsetHeight;
                } else {
                    returnValue = System.Convert.toDouble(CSHTML5.Internal.INTERNAL_HtmlDomManager.GetDomElementAttribute(div1, "offsetHeight"));
                }

                CSHTML5.Internal.INTERNAL_HtmlDomManager.RemoveFromDom(div1);

                return returnValue;
            },
            AddTemporaryDivForRowOrColumnDimensions: function (columnIndex, rowIndex) {
                var div1 = { };
                var div1style = CSHTML5.Internal.INTERNAL_HtmlDomManager.CreateDomElementAppendItAndGetStyle("div", this._innerDiv, this, div1);
                div1style.width = "100%";
                div1style.height = "100%";
                div1style.opacity = "0";
                div1style.position = "relative";

                var isMsGrid = document.isMSGrid;
                if (!isMsGrid) {
                    div1style.gridColumnStart = Bridge.toString((((columnIndex + 1) | 0)));
                    div1style.gridRowStart = Bridge.toString((((rowIndex + 1) | 0)));
                } else {
                    div1style.msGridColumn = Bridge.toString((((columnIndex + 1) | 0)));

                    div1style.msGridRow = Bridge.toString((((rowIndex + 1) | 0)));

                }

                return div1.v;
            }
        }
    });

    /**
     * Represents the control that redistributes space between columns or rows of a Grid control.
     *
     * @public
     * @class Windows.UI.Xaml.Controls.GridSplitter
     * @augments Windows.UI.Xaml.Controls.Control
     */
    Bridge.define("Windows.UI.Xaml.Controls.GridSplitter", {
        inherits: [Windows.UI.Xaml.Controls.Control],
        statics: {
            fields: {
                GripperCustomCursorDefaultResource: 0,
                ColumnsSplitterCursor: null,
                RowSplitterCursor: null,
                /**
                 * Identifies the {@link } dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Controls.GridSplitter
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                ElementProperty: null,
                /**
                 * Identifies the {@link } dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Controls.GridSplitter
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                ResizeDirectionProperty: null,
                /**
                 * Identifies the {@link } dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Controls.GridSplitter
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                ResizeBehaviorProperty: null,
                /**
                 * Identifies the {@link } dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Controls.GridSplitter
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                GripperForegroundProperty: null,
                /**
                 * Identifies the {@link } dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Controls.GridSplitter
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                ParentLevelProperty: null,
                /**
                 * Identifies the {@link } dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Controls.GridSplitter
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                GripperCursorProperty: null,
                /**
                 * Identifies the {@link } dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Controls.GridSplitter
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                GripperCustomCursorResourceProperty: null,
                /**
                 * Identifies the {@link } dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Controls.GridSplitter
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                CursorBehaviorProperty: null
            },
            ctors: {
                init: function () {
                    var $t;
                    this.GripperCustomCursorDefaultResource = -1;
                    this.ColumnsSplitterCursor = System.Windows.Input.Cursors.SizeWE;
                    this.RowSplitterCursor = System.Windows.Input.Cursors.SizeNS;
                    this.ElementProperty = Windows.UI.Xaml.DependencyProperty.Register("Element", Windows.UI.Xaml.UIElement, Windows.UI.Xaml.Controls.GridSplitter, new Windows.UI.Xaml.PropertyMetadata.$ctor4(null));
                    this.ResizeDirectionProperty = Windows.UI.Xaml.DependencyProperty.Register("ResizeDirection", Windows.UI.Xaml.Controls.GridSplitter.GridResizeDirection, Windows.UI.Xaml.Controls.GridSplitter, new Windows.UI.Xaml.PropertyMetadata.$ctor1(Bridge.box(Windows.UI.Xaml.Controls.GridSplitter.GridResizeDirection.Auto, Windows.UI.Xaml.Controls.GridSplitter.GridResizeDirection, System.Enum.toStringFn(Windows.UI.Xaml.Controls.GridSplitter.GridResizeDirection))));
                    this.ResizeBehaviorProperty = Windows.UI.Xaml.DependencyProperty.Register("ResizeBehavior", Windows.UI.Xaml.Controls.GridSplitter.GridResizeBehavior, Windows.UI.Xaml.Controls.GridSplitter, new Windows.UI.Xaml.PropertyMetadata.$ctor1(Bridge.box(Windows.UI.Xaml.Controls.GridSplitter.GridResizeBehavior.BasedOnAlignment, Windows.UI.Xaml.Controls.GridSplitter.GridResizeBehavior, System.Enum.toStringFn(Windows.UI.Xaml.Controls.GridSplitter.GridResizeBehavior))));
                    this.GripperForegroundProperty = Windows.UI.Xaml.DependencyProperty.Register("GripperForeground", Windows.UI.Xaml.Media.Brush, Windows.UI.Xaml.Controls.GridSplitter, ($t = new Windows.UI.Xaml.PropertyMetadata.$ctor2(null, Windows.UI.Xaml.Controls.GridSplitter.OnGripperForegroundPropertyChanged), $t.CallPropertyChangedWhenLoadedIntoVisualTree = Windows.UI.Xaml.WhenToCallPropertyChangedEnum.IfPropertyIsSet, $t));
                    this.ParentLevelProperty = Windows.UI.Xaml.DependencyProperty.Register("ParentLevel", System.Int32, Windows.UI.Xaml.Controls.GridSplitter, new Windows.UI.Xaml.PropertyMetadata.$ctor1(Bridge.box(0, System.Int32)));
                    this.GripperCursorProperty = Windows.UI.Xaml.DependencyProperty.RegisterAttached("GripperCursor", Windows.UI.Xaml.Controls.GridSplitter.GripperCursorType, Windows.UI.Xaml.Controls.GridSplitter, new Windows.UI.Xaml.PropertyMetadata.$ctor2(Bridge.box(Windows.UI.Xaml.Controls.GridSplitter.GripperCursorType.Default, Windows.UI.Xaml.Controls.GridSplitter.GripperCursorType, System.Enum.toStringFn(Windows.UI.Xaml.Controls.GridSplitter.GripperCursorType)), Windows.UI.Xaml.Controls.GridSplitter.OnGripperCursorPropertyChanged));
                    this.GripperCustomCursorResourceProperty = Windows.UI.Xaml.DependencyProperty.RegisterAttached("GripperCustomCursorResource", System.Int32, Windows.UI.Xaml.Controls.GridSplitter, new Windows.UI.Xaml.PropertyMetadata.$ctor2(Bridge.box(Windows.UI.Xaml.Controls.GridSplitter.GripperCustomCursorDefaultResource, System.Int32), Windows.UI.Xaml.Controls.GridSplitter.GripperCustomCursorResourcePropertyChanged));
                    this.CursorBehaviorProperty = Windows.UI.Xaml.DependencyProperty.RegisterAttached("CursorBehavior", Windows.UI.Xaml.Controls.GridSplitter.SplitterCursorBehavior, Windows.UI.Xaml.Controls.GridSplitter, new Windows.UI.Xaml.PropertyMetadata.$ctor2(Bridge.box(Windows.UI.Xaml.Controls.GridSplitter.SplitterCursorBehavior.ChangeOnSplitterHover, Windows.UI.Xaml.Controls.GridSplitter.SplitterCursorBehavior, System.Enum.toStringFn(Windows.UI.Xaml.Controls.GridSplitter.SplitterCursorBehavior)), Windows.UI.Xaml.Controls.GridSplitter.CursorBehaviorPropertyChanged));
                }
            },
            methods: {
                GetColumnDefinitionToActualWidthDictionary: function (grid) {
                    var $t;
                    var output = new (System.Collections.Generic.Dictionary$2(Windows.UI.Xaml.Controls.ColumnDefinition,System.Double)).ctor();
                    $t = Bridge.getEnumerator(grid.ColumnDefinitions);
                    try {
                        while ($t.moveNext()) {
                            var columnDefinition = $t.Current;
                            output.add(columnDefinition, columnDefinition.ActualWidth);
                        }
                    } finally {
                        if (Bridge.is($t, System.IDisposable)) {
                            $t.System$IDisposable$Dispose();
                        }
                    }
                    return output;
                },
                GetRowDefinitionToActualHeightDictionary: function (grid) {
                    var $t;
                    var output = new (System.Collections.Generic.Dictionary$2(Windows.UI.Xaml.Controls.RowDefinition,System.Double)).ctor();
                    $t = Bridge.getEnumerator(grid.RowDefinitions);
                    try {
                        while ($t.moveNext()) {
                            var rowDefinition = $t.Current;
                            output.add(rowDefinition, rowDefinition.ActualHeight);
                        }
                    } finally {
                        if (Bridge.is($t, System.IDisposable)) {
                            $t.System$IDisposable$Dispose();
                        }
                    }
                    return output;
                },
                IsStarColumn: function (definition) {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(definition.GetValue(Windows.UI.Xaml.Controls.ColumnDefinition.WidthProperty), Windows.UI.Xaml.GridLength), Windows.UI.Xaml.GridLength)).IsStar;
                },
                IsStarRow: function (definition) {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(definition.GetValue(Windows.UI.Xaml.Controls.RowDefinition.HeightProperty), Windows.UI.Xaml.GridLength), Windows.UI.Xaml.GridLength)).IsStar;
                },
                OnGripperForegroundPropertyChanged: function (d, e) {
                    var gridSplitter = Bridge.cast(d, Windows.UI.Xaml.Controls.GridSplitter);
                    var grip = Bridge.as(gridSplitter.Element, Windows.UI.Xaml.Controls.GridSplitterGripper);
                    if (grip != null) {
                        grip.GripperForeground = gridSplitter.GripperForeground;
                    }
                },
                OnGripperCursorPropertyChanged: function (d, e) {
                    var gridSplitter = Bridge.cast(d, Windows.UI.Xaml.Controls.GridSplitter);

                    if (gridSplitter._hoverWrapper == null) {
                        return;
                    }

                },
                GripperCustomCursorResourcePropertyChanged: function (d, e) {
                    var gridSplitter = Bridge.cast(d, Windows.UI.Xaml.Controls.GridSplitter);

                    if (gridSplitter._hoverWrapper == null) {
                        return;
                    }

                    gridSplitter._hoverWrapper.GripperCustomCursorResource = gridSplitter.GripperCustomCursorResource;
                },
                CursorBehaviorPropertyChanged: function (d, e) {
                    var gridSplitter = Bridge.cast(d, Windows.UI.Xaml.Controls.GridSplitter);

                    if (gridSplitter._hoverWrapper != null) {
                        gridSplitter._hoverWrapper.UpdateHoverElement(gridSplitter.CursorBehavior === Windows.UI.Xaml.Controls.GridSplitter.SplitterCursorBehavior.ChangeOnSplitterHover ? gridSplitter : gridSplitter.Element);
                    }
                }
            }
        },
        fields: {
            PreviousCursor: null,
            _resizeDirection: 0,
            _resizeBehavior: 0,
            _hoverWrapper: null,
            _thumb: null
        },
        props: {
            /**
             * Gets the target parent grid from level
             *
             * @instance
             * @private
             * @readonly
             * @memberof Windows.UI.Xaml.Controls.GridSplitter
             * @function TargetControl
             * @type Windows.UI.Xaml.FrameworkElement
             */
            TargetControl: {
                get: function () {
                    if (this.ParentLevel === 0) {
                        return this;
                    }

                    var parent = this.Parent;
                    for (var i = 2; i < this.ParentLevel; i = (i + 1) | 0) {
                        var frameworkElement = Bridge.as(parent, Windows.UI.Xaml.FrameworkElement);
                        if (frameworkElement != null) {
                            parent = frameworkElement.Parent;
                        }
                    }

                    return Bridge.as(parent, Windows.UI.Xaml.FrameworkElement);
                }
            },
            /**
             * Gets GridSplitter Container Grid
             *
             * @instance
             * @private
             * @readonly
             * @memberof Windows.UI.Xaml.Controls.GridSplitter
             * @function Resizable
             * @type Windows.UI.Xaml.Controls.Grid
             */
            Resizable: {
                get: function () {
                    if (this.TargetControl != null) {
                        return Bridge.as(this.TargetControl.Parent, Windows.UI.Xaml.Controls.Grid);
                    } else {
                        return null;
                    }
                }
            },
            /**
             * Gets the current Column definition of the parent Grid
             *
             * @instance
             * @private
             * @readonly
             * @memberof Windows.UI.Xaml.Controls.GridSplitter
             * @function CurrentColumn
             * @type Windows.UI.Xaml.Controls.ColumnDefinition
             */
            CurrentColumn: {
                get: function () {
                    if (this.Resizable == null) {
                        return null;
                    }

                    var gridSplitterTargetedColumnIndex = this.GetTargetedColumn();

                    if ((gridSplitterTargetedColumnIndex >= 0) && (gridSplitterTargetedColumnIndex < this.Resizable.ColumnDefinitions.Count)) {
                        return this.Resizable.ColumnDefinitions.getItem(gridSplitterTargetedColumnIndex);
                    }

                    return null;
                }
            },
            /**
             * Gets the Sibling Column definition of the parent Grid
             *
             * @instance
             * @private
             * @readonly
             * @memberof Windows.UI.Xaml.Controls.GridSplitter
             * @function SiblingColumn
             * @type Windows.UI.Xaml.Controls.ColumnDefinition
             */
            SiblingColumn: {
                get: function () {
                    if (this.Resizable == null) {
                        return null;
                    }

                    var gridSplitterSiblingColumnIndex = this.GetSiblingColumn();

                    if ((gridSplitterSiblingColumnIndex >= 0) && (gridSplitterSiblingColumnIndex < this.Resizable.ColumnDefinitions.Count)) {
                        return this.Resizable.ColumnDefinitions.getItem(gridSplitterSiblingColumnIndex);
                    }

                    return null;
                }
            },
            /**
             * Gets the current Row definition of the parent Grid
             *
             * @instance
             * @private
             * @readonly
             * @memberof Windows.UI.Xaml.Controls.GridSplitter
             * @function CurrentRow
             * @type Windows.UI.Xaml.Controls.RowDefinition
             */
            CurrentRow: {
                get: function () {
                    if (this.Resizable == null) {
                        return null;
                    }

                    var gridSplitterTargetedRowIndex = this.GetTargetedRow();

                    if ((gridSplitterTargetedRowIndex >= 0) && (gridSplitterTargetedRowIndex < this.Resizable.RowDefinitions.Count)) {
                        return this.Resizable.RowDefinitions.getItem(gridSplitterTargetedRowIndex);
                    }

                    return null;
                }
            },
            /**
             * Gets the Sibling Row definition of the parent Grid
             *
             * @instance
             * @private
             * @readonly
             * @memberof Windows.UI.Xaml.Controls.GridSplitter
             * @function SiblingRow
             * @type Windows.UI.Xaml.Controls.RowDefinition
             */
            SiblingRow: {
                get: function () {
                    if (this.Resizable == null) {
                        return null;
                    }

                    var gridSplitterSiblingRowIndex = this.GetSiblingRow();

                    if ((gridSplitterSiblingRowIndex >= 0) && (gridSplitterSiblingRowIndex < this.Resizable.RowDefinitions.Count)) {
                        return this.Resizable.RowDefinitions.getItem(gridSplitterSiblingRowIndex);
                    }

                    return null;
                }
            },
            /**
             * Gets or sets the visual content of this Grid Splitter
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.GridSplitter
             * @function Element
             * @type Windows.UI.Xaml.UIElement
             */
            Element: {
                get: function () {
                    return Bridge.cast(this.GetValue(Windows.UI.Xaml.Controls.GridSplitter.ElementProperty), Windows.UI.Xaml.UIElement);
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Controls.GridSplitter.ElementProperty, value);
                }
            },
            /**
             * Gets or sets whether the Splitter resizes the Columns, Rows, or Both.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.GridSplitter
             * @function ResizeDirection
             * @type number
             */
            ResizeDirection: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.Controls.GridSplitter.ResizeDirectionProperty), Windows.UI.Xaml.Controls.GridSplitter.GridResizeDirection), System.Int32));
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Controls.GridSplitter.ResizeDirectionProperty, Bridge.box(value, Windows.UI.Xaml.Controls.GridSplitter.GridResizeDirection, System.Enum.toStringFn(Windows.UI.Xaml.Controls.GridSplitter.GridResizeDirection)));
                }
            },
            /**
             * Gets or sets which Columns or Rows the Splitter resizes.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.GridSplitter
             * @function ResizeBehavior
             * @type number
             */
            ResizeBehavior: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.Controls.GridSplitter.ResizeBehaviorProperty), Windows.UI.Xaml.Controls.GridSplitter.GridResizeBehavior), System.Int32));
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Controls.GridSplitter.ResizeBehaviorProperty, Bridge.box(value, Windows.UI.Xaml.Controls.GridSplitter.GridResizeBehavior, System.Enum.toStringFn(Windows.UI.Xaml.Controls.GridSplitter.GridResizeBehavior)));
                }
            },
            /**
             * Gets or sets the foreground color of grid splitter grip
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.GridSplitter
             * @function GripperForeground
             * @type Windows.UI.Xaml.Media.Brush
             */
            GripperForeground: {
                get: function () {
                    return Bridge.cast(this.GetValue(Windows.UI.Xaml.Controls.GridSplitter.GripperForegroundProperty), Windows.UI.Xaml.Media.Brush);
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Controls.GridSplitter.GripperForegroundProperty, value);
                }
            },
            /**
             * Gets or sets the level of the parent grid to resize
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.GridSplitter
             * @function ParentLevel
             * @type number
             */
            ParentLevel: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.Controls.GridSplitter.ParentLevelProperty), System.Int32), System.Int32));
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Controls.GridSplitter.ParentLevelProperty, Bridge.box(value, System.Int32));
                }
            },
            /**
             * Gets or sets the gripper Cursor type
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.GridSplitter
             * @function GripperCursor
             * @type number
             */
            GripperCursor: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.Controls.GridSplitter.GripperCursorProperty), Windows.UI.Xaml.Controls.GridSplitter.GripperCursorType), System.Int32));
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Controls.GridSplitter.GripperCursorProperty, Bridge.box(value, Windows.UI.Xaml.Controls.GridSplitter.GripperCursorType, System.Enum.toStringFn(Windows.UI.Xaml.Controls.GridSplitter.GripperCursorType)));
                }
            },
            /**
             * Gets or sets the gripper Custom Cursor resource number
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.GridSplitter
             * @function GripperCustomCursorResource
             * @type number
             */
            GripperCustomCursorResource: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.Controls.GridSplitter.GripperCustomCursorResourceProperty), System.Int32), System.Int32));
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Controls.GridSplitter.GripperCustomCursorResourceProperty, Bridge.box(value, System.Int32));
                }
            },
            /**
             * Gets or sets splitter cursor on hover behavior
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.GridSplitter
             * @function CursorBehavior
             * @type number
             */
            CursorBehavior: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.Controls.GridSplitter.CursorBehaviorProperty), Windows.UI.Xaml.Controls.GridSplitter.SplitterCursorBehavior), System.Int32));
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Controls.GridSplitter.CursorBehaviorProperty, Bridge.box(value, Windows.UI.Xaml.Controls.GridSplitter.SplitterCursorBehavior, System.Enum.toStringFn(Windows.UI.Xaml.Controls.GridSplitter.SplitterCursorBehavior)));
                }
            }
        },
        ctors: {
            /**
             * Initializes a new instance of the {@link } class.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Controls.GridSplitter
             * @memberof Windows.UI.Xaml.Controls.GridSplitter
             * @return  {void}
             */
            ctor: function () {
                this.$initialize();
                Windows.UI.Xaml.Controls.Control.ctor.call(this);
                this.DefaultStyleKey = Windows.UI.Xaml.Controls.GridSplitter;
            }
        },
        methods: {
            /**
             * @instance
             * @protected
             * @override
             * @this Windows.UI.Xaml.Controls.GridSplitter
             * @memberof Windows.UI.Xaml.Controls.GridSplitter
             * @return  {void}
             */
            OnApplyTemplate: function () {
                Windows.UI.Xaml.Controls.Control.prototype.OnApplyTemplate.call(this);

                this.removeLoaded(Bridge.fn.cacheBind(this, this.GridSplitter_Loaded));

                this.addLoaded(Bridge.fn.cacheBind(this, this.GridSplitter_Loaded));

                if (this._hoverWrapper != null) {
                    this._hoverWrapper.UnhookEvents();
                }

                this._thumb = Bridge.as(this.GetTemplateChild("PART_Thumb"), Windows.UI.Xaml.Controls.Primitives.Thumb);
            },
            GridSplitter_Loaded: function (sender, e) {
                this._resizeDirection = this.GetResizeDirection();
                this._resizeBehavior = this.GetResizeBehavior();

                var gripper;

                if (Bridge.referenceEquals(this.Element, null)) {
                    gripper = new Windows.UI.Xaml.Controls.GridSplitterGripper.$ctor1(this._resizeDirection, this.GripperForeground);
                } else {
                    var content = this.Element;
                    this.Element = null;
                    gripper = new Windows.UI.Xaml.Controls.GridSplitterGripper.$ctor2(content, this._resizeDirection);
                }

                this.Element = gripper;


                if (this._resizeDirection === Windows.UI.Xaml.Controls.GridSplitter.GridResizeDirection.Columns) {
                    this.Cursor = Windows.UI.Xaml.Controls.GridSplitter.ColumnsSplitterCursor;
                } else if (this._resizeDirection === Windows.UI.Xaml.Controls.GridSplitter.GridResizeDirection.Rows) {
                    this.Cursor = Windows.UI.Xaml.Controls.GridSplitter.RowSplitterCursor;
                }

                if (this._thumb != null) {
                    this._thumb.addDragDelta(Bridge.fn.cacheBind(this, this.Thumb_DragDelta));
                    this._thumb.addDragStarted(Bridge.fn.cacheBind(this, this.Thumb_DragStarted));
                    this._thumb.addDragCompleted(Bridge.fn.cacheBind(this, this.Thumb_DragCompleted));
                }
            },
            Thumb_DragStarted: function (sender, e) {
                this.PreviousCursor = Windows.UI.Xaml.Window.Current.Cursor;
                this._resizeDirection = this.GetResizeDirection();
                this._resizeBehavior = this.GetResizeBehavior();

                if (this._resizeDirection === Windows.UI.Xaml.Controls.GridSplitter.GridResizeDirection.Columns) {
                    Windows.UI.Xaml.Window.Current.Cursor = Windows.UI.Xaml.Controls.GridSplitter.ColumnsSplitterCursor;
                } else if (this._resizeDirection === Windows.UI.Xaml.Controls.GridSplitter.GridResizeDirection.Rows) {
                    Windows.UI.Xaml.Window.Current.Cursor = Windows.UI.Xaml.Controls.GridSplitter.RowSplitterCursor;
                }
            },
            Thumb_DragCompleted: function (sender, e) {
                Windows.UI.Xaml.Window.Current.Cursor = this.PreviousCursor;
            },
            Thumb_DragDelta: function (sender, e) {
                var horizontalChange = e.HorizontalChange;
                var verticalChange = e.VerticalChange;

                if (this._resizeDirection === Windows.UI.Xaml.Controls.GridSplitter.GridResizeDirection.Columns) {
                    if (this.HorizontalMove(horizontalChange)) {
                        return;
                    }
                } else if (this._resizeDirection === Windows.UI.Xaml.Controls.GridSplitter.GridResizeDirection.Rows) {
                    if (this.VerticalMove(verticalChange)) {
                        return;
                    }
                }
            },
            VerticalMove: function (verticalChange) {
                var $t;
                if (this.CurrentRow == null || this.SiblingRow == null) {
                    return true;
                }

                var rowDefinitionToActualHeightDictionary = Windows.UI.Xaml.Controls.GridSplitter.GetRowDefinitionToActualHeightDictionary(this.Resizable);

                if (!Windows.UI.Xaml.Controls.GridSplitter.IsStarRow(this.CurrentRow)) {
                    this.SetRowHeight(this.CurrentRow, verticalChange, Windows.UI.Xaml.GridUnitType.Pixel, rowDefinitionToActualHeightDictionary);
                } else if (!Windows.UI.Xaml.Controls.GridSplitter.IsStarRow(this.SiblingRow)) {
                    this.SetRowHeight(this.SiblingRow, verticalChange * -1, Windows.UI.Xaml.GridUnitType.Pixel, rowDefinitionToActualHeightDictionary);
                } else {

                    if (!this.IsValidRowHeight(this.CurrentRow, verticalChange, rowDefinitionToActualHeightDictionary) || !this.IsValidRowHeight(this.SiblingRow, verticalChange * -1, rowDefinitionToActualHeightDictionary)) {
                        return true;
                    }

                    $t = Bridge.getEnumerator(this.Resizable.RowDefinitions);
                    try {
                        while ($t.moveNext()) {
                            var rowDefinition = $t.Current;
                            if (Bridge.referenceEquals(rowDefinition, this.CurrentRow)) {
                                this.SetRowHeight(this.CurrentRow, verticalChange, Windows.UI.Xaml.GridUnitType.Star, rowDefinitionToActualHeightDictionary);
                            } else if (Bridge.referenceEquals(rowDefinition, this.SiblingRow)) {
                                this.SetRowHeight(this.SiblingRow, verticalChange * -1, Windows.UI.Xaml.GridUnitType.Star, rowDefinitionToActualHeightDictionary);
                            } else if (Windows.UI.Xaml.Controls.GridSplitter.IsStarRow(rowDefinition)) {
                                rowDefinition.Height = new Windows.UI.Xaml.GridLength.$ctor2(rowDefinitionToActualHeightDictionary.getItem(rowDefinition), Windows.UI.Xaml.GridUnitType.Star);
                            }
                        }
                    } finally {
                        if (Bridge.is($t, System.IDisposable)) {
                            $t.System$IDisposable$Dispose();
                        }
                    }
                }

                return false;
            },
            HorizontalMove: function (horizontalChange) {
                var $t;
                if (this.CurrentColumn == null || this.SiblingColumn == null) {
                    return true;
                }

                var columnDefinitionToActualWidthDictionary = Windows.UI.Xaml.Controls.GridSplitter.GetColumnDefinitionToActualWidthDictionary(this.Resizable);

                if (!Windows.UI.Xaml.Controls.GridSplitter.IsStarColumn(this.CurrentColumn)) {
                    this.SetColumnWidth(this.CurrentColumn, horizontalChange, Windows.UI.Xaml.GridUnitType.Pixel, columnDefinitionToActualWidthDictionary);
                } else if (!Windows.UI.Xaml.Controls.GridSplitter.IsStarColumn(this.SiblingColumn)) {
                    this.SetColumnWidth(this.SiblingColumn, horizontalChange * -1, Windows.UI.Xaml.GridUnitType.Pixel, columnDefinitionToActualWidthDictionary);
                } else {

                    if (!this.IsValidColumnWidth(this.CurrentColumn, horizontalChange, columnDefinitionToActualWidthDictionary) || !this.IsValidColumnWidth(this.SiblingColumn, horizontalChange * -1, columnDefinitionToActualWidthDictionary)) {
                        return true;
                    }

                    $t = Bridge.getEnumerator(this.Resizable.ColumnDefinitions);
                    try {
                        while ($t.moveNext()) {
                            var columnDefinition = $t.Current;
                            if (Bridge.referenceEquals(columnDefinition, this.CurrentColumn)) {
                                this.SetColumnWidth(this.CurrentColumn, horizontalChange, Windows.UI.Xaml.GridUnitType.Star, columnDefinitionToActualWidthDictionary);
                            } else if (Bridge.referenceEquals(columnDefinition, this.SiblingColumn)) {
                                this.SetColumnWidth(this.SiblingColumn, horizontalChange * -1, Windows.UI.Xaml.GridUnitType.Star, columnDefinitionToActualWidthDictionary);
                            } else if (Windows.UI.Xaml.Controls.GridSplitter.IsStarColumn(columnDefinition)) {
                                columnDefinition.Width = new Windows.UI.Xaml.GridLength.$ctor2(columnDefinitionToActualWidthDictionary.getItem(columnDefinition), Windows.UI.Xaml.GridUnitType.Star);
                            }
                        }
                    } finally {
                        if (Bridge.is($t, System.IDisposable)) {
                            $t.System$IDisposable$Dispose();
                        }
                    }
                }

                return false;
            },
            SetColumnWidth: function (columnDefinition, horizontalChange, unitType, columnDefinitionToActualWidthDictionary) {
                var newWidth = columnDefinitionToActualWidthDictionary.getItem(columnDefinition) + horizontalChange;
                if (newWidth > this.ActualWidth) {
                    columnDefinition.Width = new Windows.UI.Xaml.GridLength.$ctor2(newWidth, unitType);
                }
            },
            IsValidColumnWidth: function (columnDefinition, horizontalChange, columnDefinitionToActualWidthDictionary) {
                var newWidth = columnDefinitionToActualWidthDictionary.getItem(columnDefinition) + horizontalChange;
                if (newWidth > this.ActualWidth) {
                    return true;
                }

                return false;
            },
            SetRowHeight: function (rowDefinition, verticalChange, unitType, rowDefinitionToActualHeightDictionary) {
                var newHeight = rowDefinitionToActualHeightDictionary.getItem(rowDefinition) + verticalChange;
                if (newHeight > this.ActualHeight) {
                    rowDefinition.Height = new Windows.UI.Xaml.GridLength.$ctor2(newHeight, unitType);
                }
            },
            IsValidRowHeight: function (rowDefinition, verticalChange, rowDefinitionToActualHeightDictionary) {
                var newHeight = rowDefinitionToActualHeightDictionary.getItem(rowDefinition) + verticalChange;
                if (newHeight > this.ActualHeight) {
                    return true;
                }

                return false;
            },
            GetTargetedColumn: function () {
                var currentIndex = Windows.UI.Xaml.Controls.Grid.GetColumn(this.TargetControl);
                return this.GetTargetIndex(currentIndex);
            },
            GetTargetedRow: function () {
                var currentIndex = Windows.UI.Xaml.Controls.Grid.GetRow(this.TargetControl);
                return this.GetTargetIndex(currentIndex);
            },
            GetSiblingColumn: function () {
                var currentIndex = Windows.UI.Xaml.Controls.Grid.GetColumn(this.TargetControl);
                return this.GetSiblingIndex(currentIndex);
            },
            GetSiblingRow: function () {
                var currentIndex = Windows.UI.Xaml.Controls.Grid.GetRow(this.TargetControl);
                return this.GetSiblingIndex(currentIndex);
            },
            GetTargetIndex: function (currentIndex) {
                switch (this._resizeBehavior) {
                    case Windows.UI.Xaml.Controls.GridSplitter.GridResizeBehavior.CurrentAndNext: 
                        return currentIndex;
                    case Windows.UI.Xaml.Controls.GridSplitter.GridResizeBehavior.PreviousAndNext: 
                        return ((currentIndex - 1) | 0);
                    case Windows.UI.Xaml.Controls.GridSplitter.GridResizeBehavior.PreviousAndCurrent: 
                        return ((currentIndex - 1) | 0);
                    default: 
                        return -1;
                }
            },
            GetSiblingIndex: function (currentIndex) {
                switch (this._resizeBehavior) {
                    case Windows.UI.Xaml.Controls.GridSplitter.GridResizeBehavior.CurrentAndNext: 
                        return ((currentIndex + 1) | 0);
                    case Windows.UI.Xaml.Controls.GridSplitter.GridResizeBehavior.PreviousAndNext: 
                        return ((currentIndex + 1) | 0);
                    case Windows.UI.Xaml.Controls.GridSplitter.GridResizeBehavior.PreviousAndCurrent: 
                        return currentIndex;
                    default: 
                        return -1;
                }
            },
            GetResizeDirection: function () {
                var direction = this.ResizeDirection;

                if (direction === Windows.UI.Xaml.Controls.GridSplitter.GridResizeDirection.Auto) {
                    var actualSize = this.INTERNAL_GetActualWidthAndHeight();
                    if (!isNaN(actualSize.Width) && !isNaN(actualSize.Height) && actualSize.Width > 0 && actualSize.Height > 0) {
                        if (actualSize.Width <= actualSize.Height) {
                            direction = Windows.UI.Xaml.Controls.GridSplitter.GridResizeDirection.Columns;
                        } else {
                            direction = Windows.UI.Xaml.Controls.GridSplitter.GridResizeDirection.Rows;
                        }
                    } else {
                        if (!isNaN(actualSize.Width) && actualSize.Width > 0) {
                            direction = Windows.UI.Xaml.Controls.GridSplitter.GridResizeDirection.Columns;
                        } else if (!isNaN(actualSize.Height) && actualSize.Height > 0) {
                            direction = Windows.UI.Xaml.Controls.GridSplitter.GridResizeDirection.Rows;
                        } else if (this.HorizontalAlignment !== Windows.UI.Xaml.HorizontalAlignment.Stretch) {
                            direction = Windows.UI.Xaml.Controls.GridSplitter.GridResizeDirection.Columns;
                        } else if (this.VerticalAlignment !== Windows.UI.Xaml.VerticalAlignment.Stretch) {
                            direction = Windows.UI.Xaml.Controls.GridSplitter.GridResizeDirection.Rows;
                        }
                    }
                }

                return direction;
            },
            GetResizeBehavior: function () {
                var resizeBehavior = this.ResizeBehavior;

                if (resizeBehavior === Windows.UI.Xaml.Controls.GridSplitter.GridResizeBehavior.BasedOnAlignment) {
                    if (this._resizeDirection === Windows.UI.Xaml.Controls.GridSplitter.GridResizeDirection.Columns) {
                        switch (this.HorizontalAlignment) {
                            case Windows.UI.Xaml.HorizontalAlignment.Left: 
                                resizeBehavior = Windows.UI.Xaml.Controls.GridSplitter.GridResizeBehavior.PreviousAndCurrent;
                                break;
                            case Windows.UI.Xaml.HorizontalAlignment.Right: 
                                resizeBehavior = Windows.UI.Xaml.Controls.GridSplitter.GridResizeBehavior.CurrentAndNext;
                                break;
                            default: 
                                resizeBehavior = Windows.UI.Xaml.Controls.GridSplitter.GridResizeBehavior.PreviousAndNext;
                                break;
                        }
                    } else {
                        switch (this.VerticalAlignment) {
                            case Windows.UI.Xaml.VerticalAlignment.Top: 
                                resizeBehavior = Windows.UI.Xaml.Controls.GridSplitter.GridResizeBehavior.PreviousAndCurrent;
                                break;
                            case Windows.UI.Xaml.VerticalAlignment.Bottom: 
                                resizeBehavior = Windows.UI.Xaml.Controls.GridSplitter.GridResizeBehavior.CurrentAndNext;
                                break;
                            default: 
                                resizeBehavior = Windows.UI.Xaml.Controls.GridSplitter.GridResizeBehavior.PreviousAndNext;
                                break;
                        }
                    }
                }

                return resizeBehavior;
            }
        }
    });

    /**
     * @memberof Windows.UI.Xaml.Controls.Primitives
     * @callback Windows.UI.Xaml.Controls.Primitives.RangeBaseValueChangedEventHandler
     * @param   {System.Object}                                                         sender    
     * @param   {Windows.UI.Xaml.Controls.Primitives.RangeBaseValueChangedEventArgs}    e
     * @return  {void}
     */

    /**
     * Represents an element that has a value within a specific range, such as the ProgressBar, ScrollBar, and Slider controls.
     *
     * @public
     * @class Windows.UI.Xaml.Controls.Primitives.RangeBase
     * @augments Windows.UI.Xaml.Controls.Control
     */
    Bridge.define("Windows.UI.Xaml.Controls.Primitives.RangeBase", {
        inherits: [Windows.UI.Xaml.Controls.Control],
        statics: {
            fields: {
                /**
                 * Identifies the LargeChange dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Controls.Primitives.RangeBase
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                LargeChangeProperty: null,
                /**
                 * Identifies the Maximum dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Controls.Primitives.RangeBase
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                MaximumProperty: null,
                /**
                 * Identifies the Minimum dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Controls.Primitives.RangeBase
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                MinimumProperty: null,
                /**
                 * The identifier for the SmallChange dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Controls.Primitives.RangeBase
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                SmallChangeProperty: null,
                /**
                 * The identifier for the Value dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Controls.Primitives.RangeBase
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                ValueProperty: null
            },
            ctors: {
                init: function () {
                    this.LargeChangeProperty = Windows.UI.Xaml.DependencyProperty.Register("LargeChange", System.Double, Windows.UI.Xaml.Controls.Primitives.RangeBase, new Windows.UI.Xaml.PropertyMetadata.$ctor1(Bridge.box(1.0, System.Double, System.Double.format, System.Double.getHashCode)));
                    this.MaximumProperty = Windows.UI.Xaml.DependencyProperty.Register("Maximum", System.Double, Windows.UI.Xaml.Controls.Primitives.RangeBase, new Windows.UI.Xaml.PropertyMetadata.$ctor3(Bridge.box(1.0, System.Double, System.Double.format, System.Double.getHashCode), Windows.UI.Xaml.Controls.Primitives.RangeBase.Maximum_Changed, Windows.UI.Xaml.Controls.Primitives.RangeBase.CoerceMaximum));
                    this.MinimumProperty = Windows.UI.Xaml.DependencyProperty.Register("Minimum", System.Double, Windows.UI.Xaml.Controls.Primitives.RangeBase, new Windows.UI.Xaml.PropertyMetadata.$ctor2(Bridge.box(0.0, System.Double, System.Double.format, System.Double.getHashCode), Windows.UI.Xaml.Controls.Primitives.RangeBase.Minimum_Changed));
                    this.SmallChangeProperty = Windows.UI.Xaml.DependencyProperty.Register("SmallChange", System.Double, Windows.UI.Xaml.Controls.Primitives.RangeBase, new Windows.UI.Xaml.PropertyMetadata.$ctor1(Bridge.box(0.1, System.Double, System.Double.format, System.Double.getHashCode)));
                    this.ValueProperty = Windows.UI.Xaml.DependencyProperty.Register("Value", System.Double, Windows.UI.Xaml.Controls.Primitives.RangeBase, new Windows.UI.Xaml.PropertyMetadata.$ctor3(Bridge.box(0.0, System.Double, System.Double.format, System.Double.getHashCode), Windows.UI.Xaml.Controls.Primitives.RangeBase.Value_Changed, Windows.UI.Xaml.Controls.Primitives.RangeBase.CoerceValue));
                }
            },
            methods: {
                CoerceMaximum: function (d, baseValue) {
                    var rangeBase = Bridge.cast(d, Windows.UI.Xaml.Controls.Primitives.RangeBase);
                    var returnValue = Bridge.is(baseValue, System.Double) ? System.Nullable.getValue(Bridge.cast(Bridge.unbox(baseValue, System.Double), System.Double)) : Number.NaN;
                    var minimum = rangeBase.Minimum;
                    if (!isNaN(minimum) && returnValue < minimum) {
                        return Bridge.box(minimum, System.Double, System.Double.format, System.Double.getHashCode);
                    }
                    return Bridge.box(returnValue, System.Double, System.Double.format, System.Double.getHashCode);
                },
                Maximum_Changed: function (d, e) {
                    var rangeBase = Bridge.cast(d, Windows.UI.Xaml.Controls.Primitives.RangeBase);
                    var newValue = e.NewValue;

                    var newMax = Bridge.is(newValue, System.Double) ? System.Nullable.getValue(Bridge.cast(Bridge.unbox(newValue, System.Double), System.Double)) : Number.NaN;
                    var oldMax = Bridge.is(e.OldValue, System.Double) ? System.Nullable.getValue(Bridge.cast(Bridge.unbox(e.OldValue, System.Double), System.Double)) : Number.NaN;

                    rangeBase.CoerceValue(Windows.UI.Xaml.Controls.Primitives.RangeBase.ValueProperty);
                    rangeBase.OnMaximumChanged(oldMax, newMax);
                },
                Minimum_Changed: function (d, e) {
                    var rangeBase = Bridge.cast(d, Windows.UI.Xaml.Controls.Primitives.RangeBase);
                    var newValue = e.NewValue;
                    var oldValue = e.OldValue;
                    var oldMin = Bridge.is(oldValue, System.Double) ? System.Nullable.getValue(Bridge.cast(Bridge.unbox(oldValue, System.Double), System.Double)) : Number.NaN;
                    var newMin = Bridge.is(newValue, System.Double) ? System.Nullable.getValue(Bridge.cast(Bridge.unbox(newValue, System.Double), System.Double)) : Number.NaN;

                    rangeBase.CoerceValue(Windows.UI.Xaml.Controls.Primitives.RangeBase.MaximumProperty);
                    rangeBase.CoerceValue(Windows.UI.Xaml.Controls.Primitives.RangeBase.ValueProperty);
                    rangeBase.OnMinimumChanged(oldMin, newMin);
                },
                CoerceValue: function (d, baseValue) {
                    var rangeBase = Bridge.cast(d, Windows.UI.Xaml.Controls.Primitives.RangeBase);
                    var returnValue = Bridge.is(baseValue, System.Double) ? System.Nullable.getValue(Bridge.cast(Bridge.unbox(baseValue, System.Double), System.Double)) : Number.NaN;
                    var minimum = rangeBase.Minimum;
                    if (!isNaN(minimum) && returnValue < minimum) {
                        return Bridge.box(minimum, System.Double, System.Double.format, System.Double.getHashCode);
                    }
                    var maximum = rangeBase.Maximum;
                    if (!isNaN(maximum) && returnValue > maximum) {
                        return Bridge.box(maximum, System.Double, System.Double.format, System.Double.getHashCode);
                    }
                    return Bridge.box(returnValue, System.Double, System.Double.format, System.Double.getHashCode);
                },
                Value_Changed: function (d, e) {
                    var rangeBase = Bridge.cast(d, Windows.UI.Xaml.Controls.Primitives.RangeBase);
                    var newValue = e.NewValue;
                    var oldValue = e.OldValue;
                    rangeBase.OnValueChanged(Bridge.is(newValue, System.Double) ? System.Nullable.getValue(Bridge.cast(Bridge.unbox(newValue, System.Double), System.Double)) : Number.NaN, Bridge.is(oldValue, System.Double) ? System.Nullable.getValue(Bridge.cast(Bridge.unbox(oldValue, System.Double), System.Double)) : Number.NaN);
                }
            }
        },
        events: {
            /**
             * Occurs when the range value changes.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Controls.Primitives.RangeBase
             * @memberof Windows.UI.Xaml.Controls.Primitives.RangeBase
             * @function addValueChanged
             * @param   {Windows.UI.Xaml.Controls.Primitives.RangeBaseValueChangedEventHandler}    value
             * @return  {void}
             */
            /**
             * Occurs when the range value changes.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Controls.Primitives.RangeBase
             * @memberof Windows.UI.Xaml.Controls.Primitives.RangeBase
             * @function removeValueChanged
             * @param   {Windows.UI.Xaml.Controls.Primitives.RangeBaseValueChangedEventHandler}    value
             * @return  {void}
             */
            ValueChanged: null
        },
        props: {
            /**
             * Gets or sets a value to be added to or subtracted from the Value of a RangeBase control. The default is 1.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.Primitives.RangeBase
             * @function LargeChange
             * @type number
             */
            LargeChange: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.Controls.Primitives.RangeBase.LargeChangeProperty), System.Double), System.Double));
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Controls.Primitives.RangeBase.LargeChangeProperty, Bridge.box(value, System.Double, System.Double.format, System.Double.getHashCode));
                }
            },
            /**
             * Gets or sets the highest possible Value of the range element. The default is 1.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.Primitives.RangeBase
             * @function Maximum
             * @type number
             */
            Maximum: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.Controls.Primitives.RangeBase.MaximumProperty), System.Double), System.Double));
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Controls.Primitives.RangeBase.MaximumProperty, Bridge.box(value, System.Double, System.Double.format, System.Double.getHashCode));
                }
            },
            /**
             * Gets or sets the Minimum possible Value of the range element. The default is 0.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.Primitives.RangeBase
             * @function Minimum
             * @type number
             */
            Minimum: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.Controls.Primitives.RangeBase.MinimumProperty), System.Double), System.Double));
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Controls.Primitives.RangeBase.MinimumProperty, Bridge.box(value, System.Double, System.Double.format, System.Double.getHashCode));
                }
            },
            /**
             * Gets or sets a Value to be added to or subtracted from the Value of a RangeBase control. The default is 0.1.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.Primitives.RangeBase
             * @function SmallChange
             * @type number
             */
            SmallChange: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.Controls.Primitives.RangeBase.SmallChangeProperty), System.Double), System.Double));
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Controls.Primitives.RangeBase.SmallChangeProperty, Bridge.box(value, System.Double, System.Double.format, System.Double.getHashCode));
                }
            },
            /**
             * Gets or sets the current setting of the range control, which may be coerced. The default is 0.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.Primitives.RangeBase
             * @function Value
             * @type number
             */
            Value: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.Controls.Primitives.RangeBase.ValueProperty), System.Double), System.Double));
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Controls.Primitives.RangeBase.ValueProperty, Bridge.box(value, System.Double, System.Double.format, System.Double.getHashCode));
                }
            }
        },
        ctors: {
            /**
             * Provides base class initialization behavior for RangeBase-derived classes.
             *
             * @instance
             * @protected
             * @this Windows.UI.Xaml.Controls.Primitives.RangeBase
             * @memberof Windows.UI.Xaml.Controls.Primitives.RangeBase
             * @return  {void}
             */
            ctor: function () {
                this.$initialize();
                Windows.UI.Xaml.Controls.Control.ctor.call(this);
            }
        },
        methods: {
            /**
             * Called when the Maximum property changes.
             *
             * @instance
             * @protected
             * @this Windows.UI.Xaml.Controls.Primitives.RangeBase
             * @memberof Windows.UI.Xaml.Controls.Primitives.RangeBase
             * @param   {number}    oldMaximum    Old value of the Maximum property.
             * @param   {number}    newMaximum    New value of the Maximum property.
             * @return  {void}
             */
            OnMaximumChanged: function (oldMaximum, newMaximum) { },
            /**
             * Called when the Minimum property changes.
             *
             * @instance
             * @protected
             * @this Windows.UI.Xaml.Controls.Primitives.RangeBase
             * @memberof Windows.UI.Xaml.Controls.Primitives.RangeBase
             * @param   {number}    oldMinimum    Old value of the Minimum property.
             * @param   {number}    newMinimum    New value of the Minimum property.
             * @return  {void}
             */
            OnMinimumChanged: function (oldMinimum, newMinimum) { },
            /**
             * Fires the ValueChanged routed event.
             *
             * @instance
             * @protected
             * @this Windows.UI.Xaml.Controls.Primitives.RangeBase
             * @memberof Windows.UI.Xaml.Controls.Primitives.RangeBase
             * @param   {number}    oldValue    Old value of the Value property.
             * @param   {number}    newValue    New value of the Value property.
             * @return  {void}
             */
            OnValueChanged: function (oldValue, newValue) {
                if (!Bridge.staticEquals(this.ValueChanged, null)) {
                    this.ValueChanged(this, new Windows.UI.Xaml.Controls.Primitives.RangeBaseValueChangedEventArgs(oldValue, newValue));
                }
            }
        }
    });

    /**
     * Provides the base class for defining a new control that encapsulates related
     existing controls and provides its own logic.
     *
     * @public
     * @class Windows.UI.Xaml.Controls.UserControl
     * @augments Windows.UI.Xaml.Controls.Control
     * @implements  Windows.UI.Xaml.Markup.INameScope
     */
    Bridge.define("Windows.UI.Xaml.Controls.UserControl", {
        inherits: [Windows.UI.Xaml.Controls.Control,Windows.UI.Xaml.Markup.INameScope],
        statics: {
            fields: {
                /**
                 * Identifies the Content dependency property
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Controls.UserControl
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                ContentProperty: null
            },
            ctors: {
                init: function () {
                    var $t;
                    this.ContentProperty = Windows.UI.Xaml.DependencyProperty.Register("Content", Windows.UI.Xaml.UIElement, Windows.UI.Xaml.Controls.UserControl, ($t = new Windows.UI.Xaml.PropertyMetadata.$ctor2(null, Windows.UI.Xaml.Controls.UserControl.Content_Changed), $t.CallPropertyChangedWhenLoadedIntoVisualTree = Windows.UI.Xaml.WhenToCallPropertyChangedEnum.IfPropertyIsSet, $t));
                }
            },
            methods: {
                Content_Changed: function (d, e) {
                    var parent = Bridge.cast(d, Windows.UI.Xaml.UIElement);
                    var oldChild = Bridge.cast(e.OldValue, Windows.UI.Xaml.UIElement);
                    var newChild = Bridge.cast(e.NewValue, Windows.UI.Xaml.UIElement);
                    CSHTML5.Internal.INTERNAL_VisualTreeManager.DetachVisualChildIfNotNull(oldChild, parent);
                    CSHTML5.Internal.INTERNAL_VisualTreeManager.AttachVisualChildIfNotAlreadyAttached(newChild, parent);
                }
            }
        },
        fields: {
            _nameScopeDictionary$1: null
        },
        props: {
            /**
             * Gets or sets the content that is contained within a user control.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.UserControl
             * @function Content
             * @type Windows.UI.Xaml.UIElement
             */
            Content: {
                get: function () {
                    return Bridge.cast(this.GetValue(Windows.UI.Xaml.Controls.UserControl.ContentProperty), Windows.UI.Xaml.UIElement);
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Controls.UserControl.ContentProperty, value);
                }
            }
        },
        alias: [
            "FindName$1", "Windows$UI$Xaml$Markup$INameScope$FindName",
            "RegisterName$1", "Windows$UI$Xaml$Markup$INameScope$RegisterName",
            "UnregisterName$1", "Windows$UI$Xaml$Markup$INameScope$UnregisterName"
        ],
        ctors: {
            init: function () {
                this._nameScopeDictionary$1 = new (System.Collections.Generic.Dictionary$2(System.String,System.Object)).ctor();
            },
            ctor: function () {
                this.$initialize();
                Windows.UI.Xaml.Controls.Control.ctor.call(this);
                this.IsTabStop = false;
            }
        },
        methods: {
            /**
             * Finds the UIElement with the specified name.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Controls.UserControl
             * @memberof Windows.UI.Xaml.Controls.UserControl
             * @param   {string}           name    The name to look for.
             * @return  {System.Object}            The object with the specified name if any; otherwise null.
             */
            FindName$1: function (name) {
                if (this._nameScopeDictionary$1.containsKey(name)) {
                    return this._nameScopeDictionary$1.getItem(name);
                } else {
                    return null;
                }
            },
            RegisterName$1: function (name, scopedElement) {
                if (this._nameScopeDictionary$1.containsKey(name) && !Bridge.referenceEquals(this._nameScopeDictionary$1.getItem(name), scopedElement)) {
                    throw new System.ArgumentException.$ctor1(System.String.format("Cannot register duplicate name '{0}' in this scope.", [name]));
                }

                this._nameScopeDictionary$1.setItem(name, scopedElement);
            },
            UnregisterName$1: function (name) {
                if (!this._nameScopeDictionary$1.containsKey(name)) {
                    throw new System.ArgumentException.$ctor1(System.String.format("Name '{0}' was not found.", [name]));
                }

                this._nameScopeDictionary$1.remove(name);
            }
        }
    });

    /**
     * Represents a control for entering passwords.
     *
     * @public
     * @class Windows.UI.Xaml.Controls.PasswordBox
     * @augments Windows.UI.Xaml.Controls.Control
     */
    Bridge.define("Windows.UI.Xaml.Controls.PasswordBox", {
        inherits: [Windows.UI.Xaml.Controls.Control],
        statics: {
            fields: {
                /**
                 * Identifies the MaxLength dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Controls.PasswordBox
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                MaxLengthProperty: null,
                /**
                 * Identifies the Password dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Controls.PasswordBox
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                PasswordProperty: null
            },
            ctors: {
                init: function () {
                    var $t;
                    this.MaxLengthProperty = Windows.UI.Xaml.DependencyProperty.Register("MaxLength", System.Int32, Windows.UI.Xaml.Controls.PasswordBox, ($t = new Windows.UI.Xaml.PropertyMetadata.$ctor4(null), $t.MethodToUpdateDom = Windows.UI.Xaml.Controls.PasswordBox.MaxLength_MethodToUpdateDom, $t));
                    this.PasswordProperty = Windows.UI.Xaml.DependencyProperty.Register("Password", System.String, Windows.UI.Xaml.Controls.PasswordBox, ($t = new Windows.UI.Xaml.PropertyMetadata.$ctor2("", Windows.UI.Xaml.Controls.PasswordBox.Password_Changed), $t.CallPropertyChangedWhenLoadedIntoVisualTree = Windows.UI.Xaml.WhenToCallPropertyChangedEnum.IfPropertyIsSet, $t));
                }
            },
            methods: {
                MaxLength_MethodToUpdateDom: function (d, newValue) {
                    var passwordBox = Bridge.cast(d, Windows.UI.Xaml.Controls.PasswordBox);
                    var newValueInt = System.Nullable.getValue(Bridge.cast(Bridge.unbox(newValue, System.Int32), System.Int32));
                    if (CSHTML5.Internal.INTERNAL_VisualTreeManager.IsElementInVisualTree(passwordBox)) {
                        CSHTML5.Internal.INTERNAL_HtmlDomManager.SetDomElementProperty(passwordBox.INTERNAL_InnerDomElement, "maxLength", Bridge.box(newValueInt, System.Int32));
                    }
                },
                Password_Changed: function (d, e) {
                    var $t;
                    var passwordBox = Bridge.cast(d, Windows.UI.Xaml.Controls.PasswordBox);
                    if (!passwordBox._isUserChangingPassword) {
                        passwordBox._isCodeProgrammaticallyChangingPassword = true;
                        var newPassword = ($t = Bridge.as(e.NewValue, System.String), $t != null ? $t : "");
                        if (CSHTML5.Internal.INTERNAL_VisualTreeManager.IsElementInVisualTree(passwordBox)) {
                            CSHTML5.Internal.INTERNAL_HtmlDomManager.SetUIElementContentString(passwordBox, newPassword);
                        }
                        passwordBox._isCodeProgrammaticallyChangingPassword = false;
                    }
                }
            }
        },
        fields: {
            _isUserChangingPassword: false,
            _isCodeProgrammaticallyChangingPassword: false,
            TextAreaContainer: null,
            _passwordInputField: null,
            TextAreaContainerNames: null,
            previousInnerText: null
        },
        events: {
            /**
             * Occurs when the value of the Password property changes.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Controls.PasswordBox
             * @memberof Windows.UI.Xaml.Controls.PasswordBox
             * @function addPasswordChanged
             * @param   {Windows.UI.Xaml.RoutedEventHandler}    value
             * @return  {void}
             */
            /**
             * Occurs when the value of the Password property changes.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Controls.PasswordBox
             * @memberof Windows.UI.Xaml.Controls.PasswordBox
             * @function removePasswordChanged
             * @param   {Windows.UI.Xaml.RoutedEventHandler}    value
             * @return  {void}
             */
            PasswordChanged: null
        },
        props: {
            INTERNAL_GetFocusInBrowser: {
                get: function () {
                    return true;
                }
            },
            /**
             * Gets or sets the maximum length for passwords to be handled by this PasswordBox.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.PasswordBox
             * @function MaxLength
             * @type number
             */
            MaxLength: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.Controls.PasswordBox.MaxLengthProperty), System.Int32), System.Int32));
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Controls.PasswordBox.MaxLengthProperty, Bridge.box(value, System.Int32));
                }
            },
            /**
             * Gets or sets the password currently held by the PasswordBox.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.PasswordBox
             * @function Password
             * @type string
             */
            Password: {
                get: function () {
                    return Bridge.cast(this.GetValue(Windows.UI.Xaml.Controls.PasswordBox.PasswordProperty), System.String);
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Controls.PasswordBox.PasswordProperty, value);
                }
            }
        },
        ctors: {
            init: function () {
                this._isUserChangingPassword = false;
                this._isCodeProgrammaticallyChangingPassword = false;
                this.TextAreaContainerNames = System.Array.init(["ContentElement", "PART_ContentHost"], System.String);
            },
            ctor: function () {
                this.$initialize();
                Windows.UI.Xaml.Controls.Control.ctor.call(this);
                this.UseSystemFocusVisuals = true;
            }
        },
        methods: {
            /**
             * Raises the PasswordChanged event
             *
             * @instance
             * @protected
             * @this Windows.UI.Xaml.Controls.PasswordBox
             * @memberof Windows.UI.Xaml.Controls.PasswordBox
             * @param   {Windows.UI.Xaml.RoutedEventArgs}    eventArgs
             * @return  {void}
             */
            OnPasswordChanged: function (eventArgs) {
                if (!Bridge.staticEquals(this.PasswordChanged, null)) {
                    this.PasswordChanged(this, eventArgs);
                }
            },
            PasswordAreaValueChanged: function () {
                var $t;
                if (!this._isCodeProgrammaticallyChangingPassword) {
                    if (CSHTML5.Internal.INTERNAL_VisualTreeManager.IsElementInVisualTree(this)) {
                        var text = System.Convert.toString(this.INTERNAL_InnerDomElement['value'] || '');
                        this._isUserChangingPassword = true;
                        this.SetCurrentValue(Windows.UI.Xaml.Controls.PasswordBox.PasswordProperty, text);
                        this._isUserChangingPassword = false;
                        this.OnPasswordChanged(($t = new Windows.UI.Xaml.RoutedEventArgs(), $t.OriginalSource = this, $t));
                    }
                }
            },
            CreateDomElement: function (parentRef, domElementWhereToPlaceChildren) {
                return this.AddPasswordInputDomElement(parentRef, domElementWhereToPlaceChildren, false);
            },
            AddPasswordInputDomElement: function (parentRef, domElementWhereToPlaceChildren, isTemplated) {
                var passwordField = { };
                var passwordFieldStyle = CSHTML5.Internal.INTERNAL_HtmlDomManager.CreateDomElementAppendItAndGetStyle("input", parentRef, this, passwordField);

                this._passwordInputField = passwordField.v;

                domElementWhereToPlaceChildren.v = passwordField.v;

                if (isTemplated) {
                    passwordFieldStyle.border = "transparent";
                    passwordFieldStyle.outline = "solid transparent";
                    passwordFieldStyle.backgroundColor = "transparent";
                    passwordFieldStyle.fontSize = "inherit";
                }

                passwordFieldStyle.width = "100%";
                passwordFieldStyle.height = "100%";

                CSHTML5.Internal.INTERNAL_HtmlDomManager.SetDomElementAttribute(passwordField.v, "type", "password", true);

                if (window.IE_VERSION) {
                    this.addGotFocus(Bridge.fn.cacheBind(this, this.InternetExplorer_GotFocus));
                    this.addLostFocus(Bridge.fn.cacheBind(this, this.InternetExplorer_LostFocus));
                    CSHTML5.Internal.INTERNAL_EventsHelper.AttachToDomEvents("textinput", passwordField.v, Bridge.fn.bind(this, function (e) {
                        this.InternetExplorer_RaisePasswordChangedIfNecessary();
                    }));
                    CSHTML5.Internal.INTERNAL_EventsHelper.AttachToDomEvents("paste", passwordField.v, Bridge.fn.bind(this, function (e) {
                        this.InternetExplorer_RaisePasswordChangedIfNecessary();
                    }));
                    CSHTML5.Internal.INTERNAL_EventsHelper.AttachToDomEvents("cut", passwordField.v, Bridge.fn.bind(this, function (e) {
                        this.InternetExplorer_RaisePasswordChangedIfNecessary();
                    }));
                    CSHTML5.Internal.INTERNAL_EventsHelper.AttachToDomEvents("keyup", passwordField.v, Bridge.fn.bind(this, function (e) {
                        this.InternetExplorer_RaisePasswordChangedIfNecessary();
                    }));
                    CSHTML5.Internal.INTERNAL_EventsHelper.AttachToDomEvents("delete", passwordField.v, Bridge.fn.bind(this, function (e) {
                        this.InternetExplorer_RaisePasswordChangedIfNecessary();
                    }));
                    CSHTML5.Internal.INTERNAL_EventsHelper.AttachToDomEvents("mouseup", passwordField.v, Bridge.fn.bind(this, function (e) {
                        this.InternetExplorer_RaisePasswordChangedIfNecessary();
                    }));
                } else {
                    CSHTML5.Internal.INTERNAL_EventsHelper.AttachToDomEvents("input", passwordField.v, Bridge.fn.bind(this, function (e) {
                        this.PasswordAreaValueChanged();
                    }));
                }


                if (isTemplated) {
                    this.OnAfterApplyHorizontalAlignmentAndWidth();
                    this.OnAfterApplyVerticalAlignmentAndWidth();

                    this.INTERNAL_OuterDomElement.tabIndex = 32767; this.INTERNAL_OuterDomElement.addEventListener('focusin', Bridge.fn.cacheBind(this, this.PasswordBox_GotFocus));
                    if ((window.IE_VERSION)) {
                        this.INTERNAL_OuterDomElement.addEventListener('click', Bridge.fn.cacheBind(this, this.PasswordBox_GotFocus));
                    }
                }

                return passwordField.v;
            },
            PasswordBox_GotFocus: function (e) {
                if (this._passwordInputField != null) {

if(                    e.target != this._passwordInputField) {
this._passwordInputField.focus()
};
                }
            },
            /**
             * Builds the visual tree for the
             {@link } control when a new
             template is applied.
             *
             * @instance
             * @protected
             * @override
             * @this Windows.UI.Xaml.Controls.PasswordBox
             * @memberof Windows.UI.Xaml.Controls.PasswordBox
             * @return  {void}
             */
            OnApplyTemplate: function () {
                Windows.UI.Xaml.Controls.Control.prototype.OnApplyTemplate.call(this);

                this.TextAreaContainer = null;

                var i = 0;
                while (this.TextAreaContainer == null && i < this.TextAreaContainerNames.length) {
                    this.TextAreaContainer = Bridge.as(this.GetTemplateChild(this.TextAreaContainerNames[System.Array.index(i, this.TextAreaContainerNames)]), Windows.UI.Xaml.Controls.Control);
                    i = (i + 1) | 0;
                }
                if (this.TextAreaContainer != null) {
                    var domElementWheretoPlaceChildren = { };
                    this.AddPasswordInputDomElement(this.TextAreaContainer.INTERNAL_InnerDomElement, domElementWheretoPlaceChildren, true);
                    this.INTERNAL_InnerDomElement = this._passwordInputField;
                    var text = this.Password;
                    if (!System.String.isNullOrEmpty(text)) {
                        Windows.UI.Xaml.Controls.PasswordBox.Password_Changed(this, new Windows.UI.Xaml.DependencyPropertyChangedEventArgs.$ctor1(text, text, Windows.UI.Xaml.Controls.PasswordBox.PasswordProperty));
                    }
                }
            },
            InternetExplorer_GotFocus: function (sender, e) {
                this.previousInnerText = System.Convert.toString(this.INTERNAL_InnerDomElement['value'] || '');
            },
            InternetExplorer_LostFocus: function (sender, e) {
                this.InternetExplorer_RaisePasswordChangedIfNecessary();
            },
            InternetExplorer_RaisePasswordChangedIfNecessary: function () {
                var newInnerText = System.Convert.toString(this.INTERNAL_InnerDomElement['value'] || '');
                if (!Bridge.referenceEquals(newInnerText, this.previousInnerText)) {
                    this.PasswordAreaValueChanged();
                    this.previousInnerText = newInnerText;
                }
            },
            /**
             * Selects all the character in the PasswordBox.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Controls.PasswordBox
             * @memberof Windows.UI.Xaml.Controls.PasswordBox
             * @return  {void}
             */
            SelectAll: function () {
                this.INTERNAL_InnerDomElement.setSelectionRange(0, this.INTERNAL_InnerDomElement.value.length);
            }
        }
    });

    /**
     * @memberof Windows.UI.Xaml.Controls.Primitives
     * @callback Windows.UI.Xaml.Controls.Primitives.DragStartedEventHandler
     * @param   {System.Object}                                               sender    
     * @param   {Windows.UI.Xaml.Controls.Primitives.DragStartedEventArgs}    e
     * @return  {void}
     */

    /**
     * @memberof Windows.UI.Xaml.Controls.Primitives
     * @callback Windows.UI.Xaml.Controls.Primitives.DragDeltaEventHandler
     * @param   {System.Object}                                             sender    
     * @param   {Windows.UI.Xaml.Controls.Primitives.DragDeltaEventArgs}    e
     * @return  {void}
     */

    /**
     * @memberof Windows.UI.Xaml.Controls.Primitives
     * @callback Windows.UI.Xaml.Controls.Primitives.DragCompletedEventHandler
     * @param   {System.Object}                                                 sender    
     * @param   {Windows.UI.Xaml.Controls.Primitives.DragCompletedEventArgs}    e
     * @return  {void}
     */

    /**
     * Represents a control that can be dragged by the user.
     *
     * @public
     * @class Windows.UI.Xaml.Controls.Primitives.Thumb
     * @augments Windows.UI.Xaml.Controls.Control
     */
    Bridge.define("Windows.UI.Xaml.Controls.Primitives.Thumb", {
        inherits: [Windows.UI.Xaml.Controls.Control],
        statics: {
            fields: {
                /**
                 * Identifies the IsDragging dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Controls.Primitives.Thumb
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                IsDraggingProperty: null
            },
            ctors: {
                init: function () {
                    this.IsDraggingProperty = Windows.UI.Xaml.DependencyProperty.Register("IsDragging", System.Boolean, Windows.UI.Xaml.Controls.Primitives.Thumb, new Windows.UI.Xaml.PropertyMetadata.$ctor1(Bridge.box(false, System.Boolean, System.Boolean.toString)));
                }
            }
        },
        fields: {
            _isPointerCaptured: false,
            _capturedPointer: null,
            _pointerX: 0,
            _pointerY: 0
        },
        events: {
            /**
             * Fires when the Thumb control loses mouse capture.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Controls.Primitives.Thumb
             * @memberof Windows.UI.Xaml.Controls.Primitives.Thumb
             * @function addDragCompleted
             * @param   {Windows.UI.Xaml.Controls.Primitives.DragCompletedEventHandler}    value
             * @return  {void}
             */
            /**
             * Fires when the Thumb control loses mouse capture.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Controls.Primitives.Thumb
             * @memberof Windows.UI.Xaml.Controls.Primitives.Thumb
             * @function removeDragCompleted
             * @param   {Windows.UI.Xaml.Controls.Primitives.DragCompletedEventHandler}    value
             * @return  {void}
             */
            DragCompleted: null,
            /**
             * Fires one or more times as the mouse pointer is moved when a Thumb control has logical focus and mouse capture.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Controls.Primitives.Thumb
             * @memberof Windows.UI.Xaml.Controls.Primitives.Thumb
             * @function addDragDelta
             * @param   {Windows.UI.Xaml.Controls.Primitives.DragDeltaEventHandler}    value
             * @return  {void}
             */
            /**
             * Fires one or more times as the mouse pointer is moved when a Thumb control has logical focus and mouse capture.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Controls.Primitives.Thumb
             * @memberof Windows.UI.Xaml.Controls.Primitives.Thumb
             * @function removeDragDelta
             * @param   {Windows.UI.Xaml.Controls.Primitives.DragDeltaEventHandler}    value
             * @return  {void}
             */
            DragDelta: null,
            /**
             * Fires when a Thumb control receives logical focus and mouse capture.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Controls.Primitives.Thumb
             * @memberof Windows.UI.Xaml.Controls.Primitives.Thumb
             * @function addDragStarted
             * @param   {Windows.UI.Xaml.Controls.Primitives.DragStartedEventHandler}    value
             * @return  {void}
             */
            /**
             * Fires when a Thumb control receives logical focus and mouse capture.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Controls.Primitives.Thumb
             * @memberof Windows.UI.Xaml.Controls.Primitives.Thumb
             * @function removeDragStarted
             * @param   {Windows.UI.Xaml.Controls.Primitives.DragStartedEventHandler}    value
             * @return  {void}
             */
            DragStarted: null
        },
        props: {
            /**
             * Gets whether the Thumb control has focus and mouse capture. The default is false.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.Primitives.Thumb
             * @function IsDragging
             * @type boolean
             */
            IsDragging: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.Controls.Primitives.Thumb.IsDraggingProperty), System.Boolean), System.Boolean));
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Controls.Primitives.Thumb.IsDraggingProperty, Bridge.box(value, System.Boolean, System.Boolean.toString));
                }
            }
        },
        ctors: {
            /**
             * Initializes a new instance of the Thumb class.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Controls.Primitives.Thumb
             * @memberof Windows.UI.Xaml.Controls.Primitives.Thumb
             * @return  {void}
             */
            ctor: function () {
                this.$initialize();
                Windows.UI.Xaml.Controls.Control.ctor.call(this);
                this.DefaultStyleKey = Windows.UI.Xaml.Controls.Primitives.Thumb;

                this.addPointerPressed(Bridge.fn.cacheBind(this, this.Thumb_PointerPressed));
                this.addPointerMoved(Bridge.fn.cacheBind(this, this.Thumb_PointerMoved));
                this.addPointerReleased(Bridge.fn.cacheBind(this, this.Thumb_PointerReleased));
            }
        },
        methods: {
            Thumb_PointerPressed: function (sender, e) {

                this._pointerX = e.GetCurrentPoint(null).Position.X;
                this._pointerY = e.GetCurrentPoint(null).Position.Y;

                this.CapturePointer(e.Pointer);

                this._isPointerCaptured = true;
                this._capturedPointer = e.Pointer;

                this.IsDragging = true;

                if (!Bridge.staticEquals(this.DragStarted, null)) {
                    this.DragStarted(this, new Windows.UI.Xaml.Controls.Primitives.DragStartedEventArgs(this._pointerX, this._pointerY));
                }
            },
            Thumb_PointerReleased: function (sender, e) {
                if (this._isPointerCaptured && Bridge.referenceEquals(e.Pointer, this._capturedPointer)) {
                    this.CancelDrag();
                }
            },
            Thumb_PointerMoved: function (sender, e) {
                if (this._isPointerCaptured) {

                    var horizontalChange = e.GetCurrentPoint(null).Position.X - this._pointerX;
                    var verticalChange = e.GetCurrentPoint(null).Position.Y - this._pointerY;
                    if (!Bridge.staticEquals(this.DragDelta, null)) {
                        this.DragDelta(this, new Windows.UI.Xaml.Controls.Primitives.DragDeltaEventArgs(horizontalChange, verticalChange));
                    }

                    this._pointerX = e.GetCurrentPoint(null).Position.X;
                    this._pointerY = e.GetCurrentPoint(null).Position.Y;
                }
            },
            /**
             * Cancels a drag operation for the Thumb.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Controls.Primitives.Thumb
             * @memberof Windows.UI.Xaml.Controls.Primitives.Thumb
             * @return  {void}
             */
            CancelDrag: function () {
                if (this._isPointerCaptured) {
                    this._isPointerCaptured = false;
                    this.ReleasePointerCapture(this._capturedPointer);

                    this.IsDragging = false;

                    if (!Bridge.staticEquals(this.DragCompleted, null)) {
                        this.DragCompleted(this, new Windows.UI.Xaml.Controls.Primitives.DragCompletedEventArgs(0.0, 0.0, false));
                    }
                }
            },
            INTERNAL_OnAttachedToVisualTree: function () {
                Windows.UI.Xaml.Controls.Control.prototype.INTERNAL_OnAttachedToVisualTree.call(this);

                if (!CSHTML5.Interop.IsRunningInTheSimulator) {
                    this.INTERNAL_OuterDomElement.onselectstart = function() { return false; };
                }
            }
        }
    });

    /**
     * @memberof Windows.UI.Xaml.Controls
     * @callback Windows.UI.Xaml.Controls.TextChangedEventHandler
     * @param   {System.Object}                                    sender    
     * @param   {Windows.UI.Xaml.Controls.TextChangedEventArgs}    e
     * @return  {void}
     */

    /**
     * Represents a control that can be used to display single-format, multi-line
     text.
     *
     * @public
     * @class Windows.UI.Xaml.Controls.TextBox
     * @augments Windows.UI.Xaml.Controls.Control
     * @example
     *
     *
     *     TextBoxName.Text = "Some text";
     * 
     *
     *
     */
    Bridge.define("Windows.UI.Xaml.Controls.TextBox", {
        inherits: [Windows.UI.Xaml.Controls.Control],
        statics: {
            fields: {
                /**
                 * Identifies the AcceptsReturn dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Controls.TextBox
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                AcceptsReturnProperty: null,
                /**
                 * Identifies the AcceptsReturn dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Controls.TextBox
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                AcceptsTabProperty: null,
                /**
                 * Identifies the PlaceholderText dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Controls.TextBox
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                PlaceholderTextProperty: null,
                /**
                 * Identifies the Text dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Controls.TextBox
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                TextProperty: null,
                /**
                 * Identifies the TextAlignment dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Controls.TextBox
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                TextAlignmentProperty: null,
                /**
                 * Identify the CaretBrush dependency property
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Controls.TextBox
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                CaretBrushProperty: null,
                /**
                 * Identifies the TextWrapping dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Controls.TextBox
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                TextWrappingProperty: null,
                /**
                 * Identifies the HorizontalScrollBarVisibility dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Controls.TextBox
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                HorizontalScrollBarVisibilityProperty: null,
                /**
                 * Identifies the VerticalScrollBarVisibility dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Controls.TextBox
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                VerticalScrollBarVisibilityProperty: null,
                /**
                 * Identifies the MaxLength dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Controls.TextBox
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                MaxLengthProperty: null,
                /**
                 * Identifies the TextDecorations dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Controls.TextBox
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                TextDecorationsProperty: null,
                IsReadOnlyProperty: null,
                SelectionForegroundProperty: null
            },
            ctors: {
                init: function () {
                    var $t;
                    this.AcceptsReturnProperty = Windows.UI.Xaml.DependencyProperty.Register("AcceptsReturn", System.Boolean, Windows.UI.Xaml.Controls.TextBox, ($t = new Windows.UI.Xaml.PropertyMetadata.$ctor2(Bridge.box(false, System.Boolean, System.Boolean.toString), Windows.UI.Xaml.Controls.TextBox.AcceptsReturn_Changed), $t.CallPropertyChangedWhenLoadedIntoVisualTree = Windows.UI.Xaml.WhenToCallPropertyChangedEnum.IfPropertyIsSet, $t));
                    this.AcceptsTabProperty = Windows.UI.Xaml.DependencyProperty.Register("AcceptsTab", System.Boolean, Windows.UI.Xaml.Controls.TextBox, new Windows.UI.Xaml.PropertyMetadata.$ctor1(Bridge.box(false, System.Boolean, System.Boolean.toString)));
                    this.PlaceholderTextProperty = Windows.UI.Xaml.DependencyProperty.Register("PlaceholderText", System.String, Windows.UI.Xaml.Controls.TextBox, new Windows.UI.Xaml.PropertyMetadata.$ctor1(""));
                    this.TextProperty = Windows.UI.Xaml.DependencyProperty.Register("Text", System.String, Windows.UI.Xaml.Controls.TextBox, ($t = new Windows.UI.Xaml.PropertyMetadata.$ctor3("", Windows.UI.Xaml.Controls.TextBox.Text_Changed, Windows.UI.Xaml.Controls.TextBox.CoerceText), $t.MethodToUpdateDom = Windows.UI.Xaml.Controls.TextBox.UpdateDomText, $t));
                    this.TextAlignmentProperty = Windows.UI.Xaml.DependencyProperty.Register("TextAlignment", Windows.UI.Xaml.TextAlignment, Windows.UI.Xaml.Controls.TextBox, ($t = new Windows.UI.Xaml.PropertyMetadata.$ctor1(Bridge.box(Windows.UI.Xaml.TextAlignment.Left, Windows.UI.Xaml.TextAlignment, System.Enum.toStringFn(Windows.UI.Xaml.TextAlignment))), $t.MethodToUpdateDom = Windows.UI.Xaml.Controls.TextBox.TextAlignment_MethodToUpdateDom, $t));
                    this.CaretBrushProperty = Windows.UI.Xaml.DependencyProperty.Register("CaretBrush", Windows.UI.Xaml.Media.Brush, Windows.UI.Xaml.Controls.TextBox, ($t = new Windows.UI.Xaml.PropertyMetadata.$ctor1(new Windows.UI.Xaml.Media.SolidColorBrush.$ctor1(Windows.UI.Colors.Black.$clone())), $t.GetCSSEquivalent = function (instance) {
                        var $t1;
                        return ($t1 = new Windows.UI.Xaml.CSSEquivalent(), $t1.Name = function (_o1) {
                                _o1.add("caretColor");
                                _o1.add("colorAlpha");
                                return _o1;
                            }(new (System.Collections.Generic.List$1(System.String)).ctor()), $t1.ApplyAlsoWhenThereIsAControlTemplate = false, $t1);
                    }, $t));
                    this.TextWrappingProperty = Windows.UI.Xaml.DependencyProperty.Register("TextWrapping", Windows.UI.Xaml.TextWrapping, Windows.UI.Xaml.Controls.TextBox, ($t = new Windows.UI.Xaml.PropertyMetadata.$ctor1(Bridge.box(Windows.UI.Xaml.TextWrapping.Wrap, Windows.UI.Xaml.TextWrapping, System.Enum.toStringFn(Windows.UI.Xaml.TextWrapping))), $t.MethodToUpdateDom = Windows.UI.Xaml.Controls.TextBox.TextWrapping_MethodToUpdateDom, $t));
                    this.HorizontalScrollBarVisibilityProperty = Windows.UI.Xaml.DependencyProperty.Register("HorizontalScrollBarVisibility", Windows.UI.Xaml.Controls.ScrollBarVisibility, Windows.UI.Xaml.Controls.TextBox, ($t = new Windows.UI.Xaml.PropertyMetadata.$ctor1(Bridge.box(Windows.UI.Xaml.Controls.ScrollBarVisibility.Hidden, Windows.UI.Xaml.Controls.ScrollBarVisibility, System.Enum.toStringFn(Windows.UI.Xaml.Controls.ScrollBarVisibility))), $t.MethodToUpdateDom = Windows.UI.Xaml.Controls.TextBox.HorizontalScrollBarVisibility_MethodToUpdateDom, $t));
                    this.VerticalScrollBarVisibilityProperty = Windows.UI.Xaml.DependencyProperty.Register("VerticalScrollBarVisibility", Windows.UI.Xaml.Controls.ScrollBarVisibility, Windows.UI.Xaml.Controls.TextBox, ($t = new Windows.UI.Xaml.PropertyMetadata.$ctor1(Bridge.box(Windows.UI.Xaml.Controls.ScrollBarVisibility.Hidden, Windows.UI.Xaml.Controls.ScrollBarVisibility, System.Enum.toStringFn(Windows.UI.Xaml.Controls.ScrollBarVisibility))), $t.MethodToUpdateDom = Windows.UI.Xaml.Controls.TextBox.VerticalScrollBarVisibility_MethodToUpdateDom, $t));
                    this.MaxLengthProperty = Windows.UI.Xaml.DependencyProperty.Register("MaxLength", System.Int32, Windows.UI.Xaml.Controls.TextBox, ($t = new Windows.UI.Xaml.PropertyMetadata.$ctor2(Bridge.box(0, System.Int32), Windows.UI.Xaml.Controls.TextBox.MaxLength_Changed), $t.CallPropertyChangedWhenLoadedIntoVisualTree = Windows.UI.Xaml.WhenToCallPropertyChangedEnum.IfPropertyIsSet, $t));
                    this.TextDecorationsProperty = Windows.UI.Xaml.DependencyProperty.Register("TextDecorations", System.Nullable$1(Windows.UI.Text.TextDecorations), Windows.UI.Xaml.Controls.TextBox, ($t = new Windows.UI.Xaml.PropertyMetadata.$ctor4(null), $t.MethodToUpdateDom = Windows.UI.Xaml.Controls.TextBox.TextDecorations_MethodToUpdateDom, $t));
                    this.IsReadOnlyProperty = Windows.UI.Xaml.DependencyProperty.Register("IsReadOnly", System.Boolean, Windows.UI.Xaml.Controls.TextBox, ($t = new Windows.UI.Xaml.PropertyMetadata.$ctor2(Bridge.box(false, System.Boolean, System.Boolean.toString), Windows.UI.Xaml.Controls.TextBox.IsReadOnly_changed), $t.CallPropertyChangedWhenLoadedIntoVisualTree = Windows.UI.Xaml.WhenToCallPropertyChangedEnum.IfPropertyIsSet, $t));
                    this.SelectionForegroundProperty = Windows.UI.Xaml.DependencyProperty.Register("SelectiongoreGround", Windows.UI.Xaml.Media.Brush, Windows.UI.Xaml.Controls.TextBox, null);
                }
            },
            methods: {
                AcceptsReturn_Changed: function (d, e) {
                    var textBox = Bridge.cast(d, Windows.UI.Xaml.Controls.TextBox);
                    if (CSHTML5.Internal.INTERNAL_VisualTreeManager.IsElementInVisualTree(textBox) && CSHTML5.Internal.INTERNAL_HtmlDomManager.IsNotUndefinedOrNull(textBox._contentEditableDiv)) {
                        if (!Bridge.referenceEquals(e.NewValue, e.OldValue) && Bridge.is(e.NewValue, System.Boolean)) {
                            var acceptsReturn = System.Nullable.getValue(Bridge.cast(Bridge.unbox(e.NewValue, System.Boolean), System.Boolean));

                            if (!true) {
                                CSHTML5.Internal.INTERNAL_HtmlDomManager.ExecuteJavaScript(System.String.format("\r\nvar element = document.getElementById(\"{0}\");\r\nelement.setAttribute(\"data-acceptsreturn\", \"{1}\");\r\n", textBox._contentEditableDiv.UniqueIdentifier, System.Boolean.toString(acceptsReturn).toLowerCase()));
                            }
                        }
                    }
                },
                Text_Changed: function (d, e) {
                    var $t;
                    var textBox = Bridge.cast(d, Windows.UI.Xaml.Controls.TextBox);
                    if (!textBox._isUserChangingText) {
                        textBox.OnTextChanged(($t = new Windows.UI.Xaml.Controls.TextChangedEventArgs(), $t.OriginalSource = textBox, $t));
                    }
                },
                UpdateDomText: function (d, newValue) {
                    var $t;
                    var textBox = Bridge.cast(d, Windows.UI.Xaml.Controls.TextBox);
                    if (!textBox._isUserChangingText) {
                        var displayedText = CSHTML5.Internal.INTERNAL_HtmlDomManager.GetTextBoxText(textBox.INTERNAL_InnerDomElement);
                        var text = ($t = textBox.Text, $t != null ? $t : "");
                        if (!Bridge.referenceEquals(displayedText, text)) {
                            if (textBox._contentEditableDiv != null || textBox.Template == null) {
                                textBox._isCodeProgrammaticallyChangingText = true;
                                CSHTML5.Internal.INTERNAL_HtmlDomManager.SetContentString(textBox, text);
                                textBox._isCodeProgrammaticallyChangingText = false;
                            }
                        }
                    }
                },
                CoerceText: function (d, value) {
                    if (value == null) {
                        return "";
                    }
                    return value;
                },
                TextAlignment_MethodToUpdateDom: function (d, newValue) {
                    var textBlock = Bridge.cast(d, Windows.UI.Xaml.Controls.TextBox);
                    var newTextAlignment = new Windows.UI.Xaml.TextAlignment();
                    if (Bridge.is(newValue, System.Int32)) {
                        newTextAlignment = System.Nullable.getValue(Bridge.cast(Bridge.unbox(newValue, Windows.UI.Xaml.TextAlignment), System.Int32));
                    } else {
                        newTextAlignment = Windows.UI.Xaml.TextAlignment.Left;
                    }
                    switch (newTextAlignment) {
                        case Windows.UI.Xaml.TextAlignment.Center: 
                            CSHTML5.Internal.INTERNAL_HtmlDomManager.GetFrameworkElementOuterStyleForModification(textBlock).textAlign = "center";
                            break;
                        case Windows.UI.Xaml.TextAlignment.Left: 
                            CSHTML5.Internal.INTERNAL_HtmlDomManager.GetFrameworkElementOuterStyleForModification(textBlock).textAlign = "left";
                            break;
                        case Windows.UI.Xaml.TextAlignment.Right: 
                            CSHTML5.Internal.INTERNAL_HtmlDomManager.GetFrameworkElementOuterStyleForModification(textBlock).textAlign = "right";
                            break;
                        case Windows.UI.Xaml.TextAlignment.Justify: 
                            CSHTML5.Internal.INTERNAL_HtmlDomManager.GetFrameworkElementOuterStyleForModification(textBlock).textAlign = "justify";
                            break;
                        default: 
                            break;
                    }
                },
                TextWrapping_MethodToUpdateDom: function (d, newValue) {
                    var textBox = Bridge.cast(d, Windows.UI.Xaml.Controls.TextBox);
                    var newTextWrapping = new Windows.UI.Xaml.TextWrapping();
                    if (Bridge.is(newValue, System.Int32)) {
                        newTextWrapping = System.Nullable.getValue(Bridge.cast(Bridge.unbox(newValue, Windows.UI.Xaml.TextWrapping), System.Int32));
                    } else {
                        newTextWrapping = System.Nullable.getValue(Bridge.cast(Bridge.unbox(Windows.UI.Xaml.Controls.TextBox.TextWrappingProperty.GetTypeMetaData(Windows.UI.Xaml.Controls.TextBox).DefaultValue, Windows.UI.Xaml.TextWrapping), System.Int32));
                    }
                    if (CSHTML5.Internal.INTERNAL_HtmlDomManager.IsNotUndefinedOrNull(textBox._contentEditableDiv)) {
                        switch (newTextWrapping) {
                            case Windows.UI.Xaml.TextWrapping.NoWrap: 
                                CSHTML5.Internal.INTERNAL_HtmlDomManager.GetDomElementStyleForModification(textBox._contentEditableDiv).whiteSpace = "nowrap";
                                break;
                            case Windows.UI.Xaml.TextWrapping.Wrap: 
                                CSHTML5.Internal.INTERNAL_HtmlDomManager.GetDomElementStyleForModification(textBox._contentEditableDiv).whiteSpace = "pre-wrap";
                                break;
                            default: 
                                break;
                        }
                    }
                },
                HorizontalScrollBarVisibility_MethodToUpdateDom: function (d, newValue) {
                    var textBox = Bridge.cast(d, Windows.UI.Xaml.Controls.TextBox);
                    var newVisibility = System.Nullable.getValue(Bridge.cast(Bridge.unbox(newValue, Windows.UI.Xaml.Controls.ScrollBarVisibility), System.Int32));

                    if (CSHTML5.Internal.INTERNAL_HtmlDomManager.IsNotUndefinedOrNull(textBox._contentEditableDiv)) {
                        var style = CSHTML5.Internal.INTERNAL_HtmlDomManager.GetDomElementStyleForModification(textBox._contentEditableDiv);

                        switch (newVisibility) {
                            case Windows.UI.Xaml.Controls.ScrollBarVisibility.Disabled: 
                                style.overflowX = "hidden";
                                break;
                            case Windows.UI.Xaml.Controls.ScrollBarVisibility.Auto: 
                                style.overflowX = "auto";
                                break;
                            case Windows.UI.Xaml.Controls.ScrollBarVisibility.Hidden: 
                                style.overflowX = "hidden";
                                break;
                            case Windows.UI.Xaml.Controls.ScrollBarVisibility.Visible: 
                                style.overflowX = "scroll";
                                break;
                            default: 
                                break;
                        }
                    }
                },
                VerticalScrollBarVisibility_MethodToUpdateDom: function (d, newValue) {
                    var textBox = Bridge.cast(d, Windows.UI.Xaml.Controls.TextBox);
                    var newVisibility = System.Nullable.getValue(Bridge.cast(Bridge.unbox(newValue, Windows.UI.Xaml.Controls.ScrollBarVisibility), System.Int32));

                    if (CSHTML5.Internal.INTERNAL_HtmlDomManager.IsNotUndefinedOrNull(textBox._contentEditableDiv)) {
                        var style = CSHTML5.Internal.INTERNAL_HtmlDomManager.GetDomElementStyleForModification(textBox._contentEditableDiv);

                        switch (newVisibility) {
                            case Windows.UI.Xaml.Controls.ScrollBarVisibility.Disabled: 
                                style.overflowY = "hidden";
                                break;
                            case Windows.UI.Xaml.Controls.ScrollBarVisibility.Auto: 
                                style.overflowY = "auto";
                                break;
                            case Windows.UI.Xaml.Controls.ScrollBarVisibility.Hidden: 
                                style.overflowY = "hidden";
                                break;
                            case Windows.UI.Xaml.Controls.ScrollBarVisibility.Visible: 
                                style.overflowY = "scroll";
                                break;
                            default: 
                                break;
                        }
                    }
                },
                MaxLength_Changed: function (d, e) {
                    var textBox = Bridge.cast(d, Windows.UI.Xaml.Controls.TextBox);
                    if (CSHTML5.Internal.INTERNAL_VisualTreeManager.IsElementInVisualTree(textBox) && CSHTML5.Internal.INTERNAL_HtmlDomManager.IsNotUndefinedOrNull(textBox._contentEditableDiv)) {
                        if (!Bridge.referenceEquals(e.NewValue, e.OldValue) && Bridge.is(e.NewValue, System.Int32)) {
                            var maxlength = System.Nullable.getValue(Bridge.cast(Bridge.unbox(e.NewValue, System.Int32), System.Int32));

                            if (!true) {
                                CSHTML5.Internal.INTERNAL_HtmlDomManager.ExecuteJavaScript(System.String.format("\r\nvar element = document.getElementById(\"{0}\");\r\nelement.setAttribute(\"data-maxlength\", \"{1}\");\r\n", textBox._contentEditableDiv.UniqueIdentifier, Bridge.box(maxlength, System.Int32)));
                            }
                        }
                    }
                },
                TextDecorations_MethodToUpdateDom: function (d, newValue) {
                    var textBox = Bridge.cast(d, Windows.UI.Xaml.Controls.TextBox);
                    var newTextDecorations = System.Nullable.getValue(Bridge.cast(Bridge.unbox(newValue, Windows.UI.Text.TextDecorations), System.Int32, true));

                    var cssValue;
                    if (System.Nullable.hasValue(newTextDecorations)) {
                        switch (System.Nullable.getValue(newTextDecorations)) {
                            case Windows.UI.Text.TextDecorations.OverLine: 
                                cssValue = "overline";
                                break;
                            case Windows.UI.Text.TextDecorations.Strikethrough: 
                                cssValue = "line-through";
                                break;
                            case Windows.UI.Text.TextDecorations.Underline: 
                                cssValue = "underline";
                                break;
                            case Windows.UI.Text.TextDecorations.None: 
                            default: 
                                cssValue = "";
                                break;
                        }
                    } else {
                        cssValue = "";
                    }
                    CSHTML5.Internal.INTERNAL_HtmlDomManager.GetDomElementStyleForModification(textBox.INTERNAL_OptionalSpecifyDomElementConcernedByFocus).textDecoration = cssValue;
                },
                IsReadOnly_changed: function (d, e) {
                    var textBox = Bridge.cast(d, Windows.UI.Xaml.Controls.TextBox);
                    if (CSHTML5.Internal.INTERNAL_VisualTreeManager.IsElementInVisualTree(textBox) && CSHTML5.Internal.INTERNAL_HtmlDomManager.IsNotUndefinedOrNull(textBox._contentEditableDiv)) {
                        if (!Bridge.referenceEquals(e.NewValue, e.OldValue) && Bridge.is(e.NewValue, System.Boolean)) {
                            var isReadOnly = System.Nullable.getValue(Bridge.cast(Bridge.unbox(e.NewValue, System.Boolean), System.Boolean));
                            var canChangeBackground = textBox.Template == null;

                            textBox._contentEditableDiv.setAttribute("contentEditable", System.Boolean.toString((!isReadOnly)).toLowerCase());
if(System.Boolean.toString(canChangeBackground).toLowerCase())
    textBox._contentEditableDiv.style.backgroundColor=System.Boolean.toString(canChangeBackground).toLowerCase();


                            if (!true) {
                                CSHTML5.Internal.INTERNAL_HtmlDomManager.ExecuteJavaScript(System.String.format("\r\nvar element = document.getElementById(\"{0}\");\r\nelement.setAttribute(\"data-isreadonly\",\"{1}\");\r\n", textBox._contentEditableDiv.UniqueIdentifier, System.Boolean.toString(isReadOnly).toLowerCase()));
                            }
                        }
                    }
                }
            }
        },
        fields: {
            _contentEditableDiv: null,
            TextAreaContainer: null,
            /**
             * The name of the ExpanderButton template part.
             *
             * @instance
             * @private
             * @readonly
             * @memberof Windows.UI.Xaml.Controls.TextBox
             * @type Array.<string>
             */
            TextAreaContainerNames: null,
            _isCodeProgrammaticallyChangingText: false,
            _isUserChangingText: false,
            previousInnerText: null
        },
        events: {
            /**
             * Occurs when the text is changed.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Controls.TextBox
             * @memberof Windows.UI.Xaml.Controls.TextBox
             * @function addTextChanged
             * @param   {Windows.UI.Xaml.Controls.TextChangedEventHandler}    value
             * @return  {void}
             */
            /**
             * Occurs when the text is changed.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Controls.TextBox
             * @memberof Windows.UI.Xaml.Controls.TextBox
             * @function removeTextChanged
             * @param   {Windows.UI.Xaml.Controls.TextChangedEventHandler}    value
             * @return  {void}
             */
            TextChanged: null,
            SelectionChanged: null
        },
        props: {
            INTERNAL_GetFocusInBrowser: {
                get: function () {
                    return true;
                }
            },
            /**
             * Gets or sets the value that determines whether the text box allows and displays
             the newline or return characters.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.TextBox
             * @function AcceptsReturn
             * @type boolean
             */
            AcceptsReturn: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.Controls.TextBox.AcceptsReturnProperty), System.Boolean), System.Boolean));
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Controls.TextBox.AcceptsReturnProperty, Bridge.box(value, System.Boolean, System.Boolean.toString));
                }
            },
            /**
             * Gets or sets the value that determines whether pressing tab while the TextBox has the focus will add a tabulation in the text or set the focus to the next element.
             True to add a tabulation, false to set the focus to the next element.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.TextBox
             * @function AcceptsTab
             * @type boolean
             */
            AcceptsTab: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.Controls.TextBox.AcceptsTabProperty), System.Boolean), System.Boolean));
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Controls.TextBox.AcceptsTabProperty, Bridge.box(value, System.Boolean, System.Boolean.toString));
                }
            },
            /**
             * Gets or sets the text that is displayed in the control until the value is changed by a user action or some other operation.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.TextBox
             * @function PlaceholderText
             * @type string
             */
            PlaceholderText: {
                get: function () {
                    return Bridge.cast(this.GetValue(Windows.UI.Xaml.Controls.TextBox.PlaceholderTextProperty), System.String);
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Controls.TextBox.PlaceholderTextProperty, value);
                }
            },
            /**
             * Gets or sets the text displayed in the TextBox.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.TextBox
             * @function Text
             * @type string
             */
            Text: {
                get: function () {
                    return Bridge.cast(this.GetValue(Windows.UI.Xaml.Controls.TextBox.TextProperty), System.String);
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Controls.TextBox.TextProperty, value);
                }
            },
            /**
             * Gets or sets how the text should be aligned in the text box.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.TextBox
             * @function TextAlignment
             * @type Windows.UI.Xaml.TextAlignment
             */
            TextAlignment: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.Controls.TextBox.TextAlignmentProperty), Windows.UI.Xaml.TextAlignment), System.Int32));
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Controls.TextBox.TextAlignmentProperty, Bridge.box(value, Windows.UI.Xaml.TextAlignment, System.Enum.toStringFn(Windows.UI.Xaml.TextAlignment)));
                }
            },
            /**
             * Gets or sets the brush that is used to render the vertical bar that indicates
             the insertion point.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.TextBox
             * @function CaretBrush
             * @type Windows.UI.Xaml.Media.Brush
             */
            CaretBrush: {
                get: function () {
                    return Bridge.cast(this.GetValue(Windows.UI.Xaml.Controls.TextBox.CaretBrushProperty), Windows.UI.Xaml.Media.Brush);
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Controls.TextBox.CaretBrushProperty, value);
                }
            },
            SelectionStart: {
                get: function () {
                    var selectionStartIndex = { };
                    var selectionLength = { };
                    this.NEW_GET_SELECTION(selectionStartIndex, selectionLength);
                    return selectionStartIndex.v;
                },
                set: function (value) {
                    if (value < 0) {
                        throw new System.ArgumentOutOfRangeException.$ctor1("SelectionStart cannot be lower than 0");
                    }
                    this.RefreshSelection(value, this.SelectionLength);
                }
            },
            SelectionLength: {
                get: function () {
                    var selectionStartIndex = { };
                    var selectionLength = { };
                    this.NEW_GET_SELECTION(selectionStartIndex, selectionLength);
                    return selectionLength.v;
                },
                set: function (value) {
                    if (value < 0) {
                        throw new System.ArgumentOutOfRangeException.$ctor1("SelectionLength cannot be lower than 0");
                    }
                    this.RefreshSelection(this.SelectionStart, value);
                }
            },
            /**
             * Gets or sets how the TextBow wraps text.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.TextBox
             * @function TextWrapping
             * @type Windows.UI.Xaml.TextWrapping
             */
            TextWrapping: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.Controls.TextBox.TextWrappingProperty), Windows.UI.Xaml.TextWrapping), System.Int32));
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Controls.TextBox.TextWrappingProperty, Bridge.box(value, Windows.UI.Xaml.TextWrapping, System.Enum.toStringFn(Windows.UI.Xaml.TextWrapping)));
                }
            },
            /**
             * Gets or sets a value that indicates whether a horizontal ScrollBar should
             be displayed.
             Returns a ScrollBarVisibility value that indicates whether a horizontal ScrollBar
             should be displayed. The default value is Hidden.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.TextBox
             * @function HorizontalScrollBarVisibility
             * @type Windows.UI.Xaml.Controls.ScrollBarVisibility
             */
            HorizontalScrollBarVisibility: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.Controls.TextBox.HorizontalScrollBarVisibilityProperty), Windows.UI.Xaml.Controls.ScrollBarVisibility), System.Int32));
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Controls.TextBox.HorizontalScrollBarVisibilityProperty, Bridge.box(value, Windows.UI.Xaml.Controls.ScrollBarVisibility, System.Enum.toStringFn(Windows.UI.Xaml.Controls.ScrollBarVisibility)));
                }
            },
            /**
             * Gets or sets a value that indicates whether a vertical ScrollBar should be displayed.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.TextBox
             * @function VerticalScrollBarVisibility
             * @type Windows.UI.Xaml.Controls.ScrollBarVisibility
             */
            VerticalScrollBarVisibility: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.Controls.TextBox.VerticalScrollBarVisibilityProperty), Windows.UI.Xaml.Controls.ScrollBarVisibility), System.Int32));
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Controls.TextBox.VerticalScrollBarVisibilityProperty, Bridge.box(value, Windows.UI.Xaml.Controls.ScrollBarVisibility, System.Enum.toStringFn(Windows.UI.Xaml.Controls.ScrollBarVisibility)));
                }
            },
            /**
             * Gets or sets the value that determines the maximum number of characters allowed
             for user input.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.TextBox
             * @function MaxLength
             * @type number
             */
            MaxLength: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.Controls.TextBox.MaxLengthProperty), System.Int32), System.Int32));
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Controls.TextBox.MaxLengthProperty, Bridge.box(value, System.Int32));
                }
            },
            /**
             * Gets or sets the text decorations (underline, strikethrough...).
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.TextBox
             * @function TextDecorations$1
             * @type ?Windows.UI.Text.TextDecorations
             */
            TextDecorations$1: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.Controls.TextBox.TextDecorationsProperty), Windows.UI.Text.TextDecorations), System.Int32, true));
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Controls.TextBox.TextDecorationsProperty, Bridge.box(value, Windows.UI.Text.TextDecorations, System.Nullable.toStringFn(System.Enum.toStringFn(Windows.UI.Text.TextDecorations)), System.Nullable.getHashCode));
                }
            },
            IsReadOnly: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.Controls.TextBox.IsReadOnlyProperty), System.Boolean), System.Boolean));
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Controls.TextBox.IsReadOnlyProperty, Bridge.box(value, System.Boolean, System.Boolean.toString));
                }
            },
            SelectionForeground: {
                get: function () {
                    return Bridge.cast(this.GetValue(Windows.UI.Xaml.Controls.TextBox.SelectionForegroundProperty), Windows.UI.Xaml.Media.Brush);
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Controls.TextBox.SelectionForegroundProperty, value);
                }
            }
        },
        ctors: {
            init: function () {
                this.TextAreaContainerNames = System.Array.init(["ContentElement", "PART_ContentHost"], System.String);
                this._isCodeProgrammaticallyChangingText = false;
                this._isUserChangingText = false;
            },
            /**
             * @instance
             * @public
             * @this Windows.UI.Xaml.Controls.TextBox
             * @memberof Windows.UI.Xaml.Controls.TextBox
             * @return  {void}
             */
            ctor: function () {
                this.$initialize();
                Windows.UI.Xaml.Controls.Control.ctor.call(this);
                this.UseSystemFocusVisuals = true;
            }
        },
        methods: {
            GetDomElementToSetContentString: function () {
                if (this._contentEditableDiv != null) {
                    return this._contentEditableDiv;
                } else {
                    return Windows.UI.Xaml.Controls.Control.prototype.GetDomElementToSetContentString.call(this);
                }
            },
            /**
             * Raises the TextChanged event
             *
             * @instance
             * @protected
             * @this Windows.UI.Xaml.Controls.TextBox
             * @memberof Windows.UI.Xaml.Controls.TextBox
             * @param   {Windows.UI.Xaml.Controls.TextChangedEventArgs}    eventArgs    The arguments for the event.
             * @return  {void}
             */
            OnTextChanged: function (eventArgs) {
                if (!Bridge.staticEquals(this.TextChanged, null)) {
                    this.TextChanged(this, eventArgs);
                }
            },
            INTERNAL_AttachToDomEvents: function () {
                Windows.UI.Xaml.Controls.Control.prototype.INTERNAL_AttachToDomEvents.call(this);
            },
            TextAreaValueChanged: function () {
                var $t;
                if (!this._isCodeProgrammaticallyChangingText) {
                    if (CSHTML5.Internal.INTERNAL_VisualTreeManager.IsElementInVisualTree(this)) {
                        var text = CSHTML5.Internal.INTERNAL_HtmlDomManager.GetTextBoxText(this.INTERNAL_InnerDomElement);
                        if (!this.AcceptsReturn) {
                            text = System.String.replaceAll(System.String.replaceAll(text, "\n", ""), "\r", "");
                        }
                        this._isUserChangingText = true;
                        this.SetCurrentValue(Windows.UI.Xaml.Controls.TextBox.TextProperty, text);
                        this._isUserChangingText = false;
                        this.OnTextChanged(($t = new Windows.UI.Xaml.Controls.TextChangedEventArgs(), $t.OriginalSource = this, $t));
                    }
                }
            },
            CreateDomElement: function (parentRef, domElementWhereToPlaceChildren) {
                return this.AddContentEditableDomElement(parentRef, domElementWhereToPlaceChildren, false);
            },
            AddContentEditableDomElement: function (parentRef, domElementWhereToPlaceChildren, isTemplated) {
                var isReadOnly = this.IsReadOnly;
                var outerDiv = { };
                var outerDivStyle = CSHTML5.Internal.INTERNAL_HtmlDomManager.CreateDomElementAppendItAndGetStyle("div", parentRef, this, outerDiv);
                var backgroundColor = "transparent";
                if (!isTemplated) {
                    outerDivStyle.borderColor = "#ABADB3";
                    outerDivStyle.borderStyle = "solid";
                    outerDivStyle.borderWidth = "1px";
                    outerDivStyle.boxSizing = "border-box";
                    backgroundColor = isReadOnly ? "#DDDDDD" : "White";
                } else {
                    outerDivStyle.borderWidth = "0px";
                    outerDivStyle.width = "100%";
                    outerDivStyle.height = "100%";
                }
                outerDivStyle.backgroundColor = backgroundColor;
                var middleDiv = { };
                var middleDivStyle = CSHTML5.Internal.INTERNAL_HtmlDomManager.CreateDomElementAppendItAndGetStyle("div", outerDiv.v, this, middleDiv);
                middleDivStyle.width = "100%";
                middleDivStyle.height = "100%";

                var contentEditableDiv = { };
                var contentEditableDivStyle = CSHTML5.Internal.INTERNAL_HtmlDomManager.CreateDomElementAppendItAndGetStyle("div", middleDiv.v, this, contentEditableDiv);
                this._contentEditableDiv = contentEditableDiv.v;
                if ((window.IE_VERSION)) {
                    contentEditableDiv.v.classList.add("ie_set_p_margins_to_zero");
                }

                contentEditableDivStyle.width = "100%";
                contentEditableDivStyle.height = "100%";
                contentEditableDivStyle.whiteSpace = "pre-wrap";
                contentEditableDivStyle.overflowX = "hidden";
                contentEditableDivStyle.overflowY = "hidden";
                if (isTemplated) {
                    contentEditableDivStyle.outline = "solid transparent";
                    contentEditableDivStyle.background = "solid transparent";
                }
                contentEditableDivStyle.cursor = "text";

                var isContentEditable = System.Boolean.toString((!isReadOnly)).toLowerCase();
                CSHTML5.Internal.INTERNAL_HtmlDomManager.SetDomElementAttribute(contentEditableDiv.v, "contentEditable", isContentEditable);
                this.INTERNAL_OptionalSpecifyDomElementConcernedByFocus = contentEditableDiv.v;
                this.INTERNAL_OptionalSpecifyDomElementConcernedByMinMaxHeightAndWidth = contentEditableDiv.v;

                contentEditableDivStyle.minWidth = "14px";
                contentEditableDivStyle.minHeight = (System.Double.format((Math.floor(this.FontSize * 1.5 * 1000) / 1000)) || "") + "px";

                if ((window.IE_VERSION)) {
                    var divToImproveIELineBreaks = { };
                    var divToImproveIELineBreaksStyle = CSHTML5.Internal.INTERNAL_HtmlDomManager.CreateDomElementAppendItAndGetStyle("div", contentEditableDiv.v, this, divToImproveIELineBreaks);
                }

                domElementWhereToPlaceChildren.v = contentEditableDiv.v;

                CSHTML5.Internal.INTERNAL_HtmlDomManager.SetDomElementProperty(outerDiv.v, "data-absorb-events", Bridge.box(true, System.Boolean, System.Boolean.toString));


                if (window.IE_VERSION) {
                    this.addGotFocus(Bridge.fn.cacheBind(this, this.InternetExplorer_GotFocus));
                    this.addLostFocus(Bridge.fn.cacheBind(this, this.InternetExplorer_LostFocus));
                    CSHTML5.Internal.INTERNAL_EventsHelper.AttachToDomEvents("textinput", contentEditableDiv.v, Bridge.fn.bind(this, function (e) {
                        this.InternetExplorer_RaiseTextChangedIfNecessary();
                    }));
                    CSHTML5.Internal.INTERNAL_EventsHelper.AttachToDomEvents("paste", contentEditableDiv.v, Bridge.fn.bind(this, function (e) {
                        this.InternetExplorer_RaiseTextChangedIfNecessary();
                    }));
                    CSHTML5.Internal.INTERNAL_EventsHelper.AttachToDomEvents("cut", contentEditableDiv.v, Bridge.fn.bind(this, function (e) {
                        this.InternetExplorer_RaiseTextChangedIfNecessary();
                    }));
                    CSHTML5.Internal.INTERNAL_EventsHelper.AttachToDomEvents("keyup", contentEditableDiv.v, Bridge.fn.bind(this, function (e) {
                        this.InternetExplorer_RaiseTextChangedIfNecessary();
                    }));
                    CSHTML5.Internal.INTERNAL_EventsHelper.AttachToDomEvents("delete", contentEditableDiv.v, Bridge.fn.bind(this, function (e) {
                        this.InternetExplorer_RaiseTextChangedIfNecessary();
                    }));
                    CSHTML5.Internal.INTERNAL_EventsHelper.AttachToDomEvents("mouseup", contentEditableDiv.v, Bridge.fn.bind(this, function (e) {
                        this.InternetExplorer_RaiseTextChangedIfNecessary();
                    }));
                } else {
                    CSHTML5.Internal.INTERNAL_EventsHelper.AttachToDomEvents("input", contentEditableDiv.v, Bridge.fn.bind(this, function (e) {
                        this.TextAreaValueChanged();
                    }));
                }


                if (true) {

var instance =                     this;
Bridge.unbox(contentEditableDiv.v).addEventListener('keydown', function(e) {
    if (e.keyCode == 13 && instance.AcceptsReturn !== true)
    {
        e.preventDefault();
        return false;
    }

    var isAddingTabulation = e.keyCode == 9 && instance.AcceptsTab == true;
    var isRemovingTabulation = isAddingTabulation && e.shiftKey;
    if(isRemovingTabulation)
    {
        isAddingTabulation = false
    }

    if((isAddingTabulation || e.keyCode == 13 || e.keyCode == 32 || e.keyCode > 47) && instance.MaxLength != 0)
    {
        var textLength = instance.GetTextLengthIncludingNewLineCompensation();

        if (e.keyCode == 13)
        {
            textLength += 1; //because adding a new line takes 2 characters instead of 1.
        }
        if(textLength >= instance.MaxLength)
        {
            e.preventDefault();
            return false;
        }
    }

    if(isAddingTabulation)
    {
        //we need to add '\t' where the cursor is, prevent the event (which would change the focus) and dispatch the event for the text changed:
        var sel, range;
        if (window.getSelection) {
            sel = window.getSelection();
            if (sel.rangeCount) {
                range = sel.getRangeAt(0);
                range.deleteContents();
                range.insertNode(document.createTextNode('\t'));
                sel.collapseToEnd();
                range.collapse(false); //for IE
            }
        } else if (document.selection && document.selection.createRange) {
            range = document.selection.createRange();
            range.text = '\t';
            document.selection.collapseToEnd();
        }
        if (window.IS_EDGE)
        {
            sel.removeAllRanges();
            sel.addRange(range);
        }

        instance.TextAreaValueChanged(); //todo: test this.
        e.preventDefault();
            return false;
    }
    if (isRemovingTabulation)
    {
        var sel, range;
        if (window.getSelection) {
            sel = window.getSelection();
            if (sel.rangeCount) {
                range = sel.getRangeAt(0);
                if(range.collapsed)
                {
                    //if the previous character is a '\t', we want to remove it:
                    var rangeStartContainer = range.startContainer;
                    var rangeStartOffset = range.startOffset;

                    //we get the node that contains the text that needs changing and the index of the character to remove:
                    var textNodeToModify = undefined;
                    var indexToRemove = -1;
                    if(rangeStartContainer.nodeType == Node.TEXT_NODE && rangeStartOffset > 0)
                    {
        
                        textNodeToModify = rangeStartContainer;
                        indexToRemove = rangeStartOffset - 1;
                    }
                    else //The potential tab to remove is in the node that is right before the selection(caret) so we look at the last character of the 'previous sibling':
                    {
                        var previousSibling = undefined;

                        // we get the previous node:
                        if(rangeStartContainer.nodeType == Node.TEXT_NODE) //the caret was right before the first element of a textNode. I am not sure if we can arrive here since this case seems to be considered as 'the caret is between the nodes' instead of 'the caret is in the node but before the first character'
                        {
                            previousSibling = rangeStartContainer.previousSibling;
                        }
                        else //the caret is considered between nodes so the range container is the element that contains the text nodes.
                        {
                            if(rangeStartOffset > 0) //otherwise, we are at the first character inside a new div (or p), which means at the start of a new line so nothing to remove.
                            {
                                previousSibling = rangeStartContainer.childNodes[rangeStartOffset -1];
                            }
                        }

                        //We update the node and index to modify:
                        if(previousSibling != undefined && previousSibling.nodeType == Node.TEXT_NODE)
                        {
                            textNodeToModify = previousSibling;
                            indexToRemove = previousSibling.textContent.length - 1;
                        } //else, the previous node was not a text node so there has to be a new line between the caret and the previous text so nothing to remove here.
                    }
        
                    if(textNodeToModify != undefined && indexToRemove != -1)
                    {
                        //range.startContainer.textContent = range.startContainer.textContent.substring(rangeStartOffset)
                        var textContent = textNodeToModify.textContent;
                        if(textContent[indexToRemove] == '\t')
                        {
                            var resultingString = textContent.substring(0, indexToRemove); //note: text.substring(0,-1) returns "
                            resultingString += textContent.substring(indexToRemove + 1); //note: 'aaa'.substring(25) returns "
                            textNodeToModify.textContent = resultingString;
                        }
                    }
    
                }
            }
        }
        //todo: else.
        instance.TextAreaValueChanged(); //todo: test this.
        e.preventDefault();
            return false;
    }

}, false);
                }

                if (true) {
                    var acceptsReturn = this.AcceptsReturn;

                    Bridge.unbox(contentEditableDiv.v).addEventListener('paste', function(e) {
    var isReadOnly= isReadOnly;
    if(!isReadOnly)
    {
        var isSingleLine = (acceptsReturn !== true); // This is the current value at the time when the event is raised.
        // Chrome supports setting ContentEditable to PlainText-Only, so we try this first:
        Bridge.unbox(contentEditableDiv.v).setAttribute('contenteditable', 'PLAINTEXT-ONLY');
        if (Bridge.unbox(contentEditableDiv.v).contentEditable === 'plaintext-only') // If setting the attribute worked, we can read it back (in lowercase)
        {
          // --- CHROME: ---
          // Nothing else to do about rich text conversion to plain text.
          // However we still need to remove line breaks if AcceptsReturn is false:
          if (isSingleLine){
            e.preventDefault();
            var content = (e.originalEvent || e).clipboardData.getData('text/plain');
            content = content.replace(/\n/g, '').replace(/\r/g, '');
            document.execCommand('insertText', false, content);
          }
    }
    else
        {
          Bridge.unbox(contentEditableDiv.v).setAttribute('contenteditable', 'true');
          if (e.clipboardData){

            // --- FIREFOX: ---
            e.preventDefault();
            var content = (e.originalEvent || e).clipboardData.getData('text/plain');
            if (isSingleLine){
              content = content.replace(/\n/g, '').replace(/\r/g, '');
            }
            document.execCommand('insertText', false, content);
          }
          else if (window.clipboardData){

            // --- INTERNET EXPLORER: ---
            e.preventDefault();
            var content = window.clipboardData.getData('Text');
            if (window.getSelection)
            {
              var newDiv = document.createElement('div');
              if (isSingleLine){
                content = content.replace(/\n/g, '').replace(/\r/g, '');
              }
              content = content.replace(/\r\n/g, '<br />');
              newDiv.innerHTML = content.replace(/\n/g, '<br />');
              var range = window.getSelection().getRangeAt(0);
              range.deleteContents()
              range.insertNode( newDiv );
              //window.getSelection().getRangeAt(0).insertNode( document.createTextNode(content) );
            }
          }
      }
            }
  
}, false);
                }

                if (isTemplated) {
                    this.OnAfterApplyHorizontalAlignmentAndWidth();
                    this.OnAfterApplyVerticalAlignmentAndWidth();

                    this.INTERNAL_OuterDomElement.tabIndex = 32767; this.INTERNAL_OuterDomElement.addEventListener('focusin', Bridge.fn.cacheBind(this, this.TextBox_GotFocus));
                    if ((window.IE_VERSION)) {
                        this.INTERNAL_OuterDomElement.addEventListener('click', Bridge.fn.cacheBind(this, this.TextBox_GotFocus));
                    }
                }
                return outerDiv.v;
            },
            GetTextLengthIncludingNewLineCompensation: function () {
                var text = CSHTML5.Internal.INTERNAL_HtmlDomManager.GetTextBoxText(this.INTERNAL_InnerDomElement);
                if (!this.AcceptsReturn) {
                    text = System.String.replaceAll(System.String.replaceAll(text, "\n", ""), "\r", "");
                }

                return text.length;
            },
            TextBox_GotFocus: function (e) {
                if (this._contentEditableDiv != null) {

if(                    e.target != this._contentEditableDiv) {
this._contentEditableDiv.focus()
var range,selection;
    if(document.createRange)//Firefox, Chrome, Opera, Safari, IE 9+
    {
        range = document.createRange();//Create a range (a range is a like the selection but invisible)
        range.selectNodeContents(this._contentEditableDiv);//Select the entire contents of the element with the range
        range.collapse(false);//collapse the range to the end point. false means collapse to end rather than the start
        selection = window.getSelection();//get the selection object (allows you to change selection)
        selection.removeAllRanges();//remove any selections already made
        selection.addRange(range);//make the range you have just created the visible selection
    }
    else if(document.selection)//IE 8 and lower
    { 
        range = document.body.createTextRange();//Create a range (a range is a like the selection but invisible)
        range.moveToElementText(this._contentEditableDiv);//Select the entire contents of the element with the range
        range.collapse(false);//collapse the range to the end point. false means collapse to end rather than the start
        range.select();//Select the range (make it the visible selection
    }
};
                }
            },
            /**
             * Builds the visual tree for the
             {@link } control when a new
             template is applied.
             *
             * @instance
             * @protected
             * @override
             * @this Windows.UI.Xaml.Controls.TextBox
             * @memberof Windows.UI.Xaml.Controls.TextBox
             * @return  {void}
             */
            OnApplyTemplate: function () {
                Windows.UI.Xaml.Controls.Control.prototype.OnApplyTemplate.call(this);

                this.TextAreaContainer = null;

                var i = 0;
                while (this.TextAreaContainer == null && i < this.TextAreaContainerNames.length) {
                    this.TextAreaContainer = Bridge.as(this.GetTemplateChild(this.TextAreaContainerNames[System.Array.index(i, this.TextAreaContainerNames)]), Windows.UI.Xaml.Controls.Control);
                    i = (i + 1) | 0;
                }
                if (this.TextAreaContainer != null) {
                    var domElementWheretoPlaceChildren = { };
                    this.AddContentEditableDomElement(this.TextAreaContainer.INTERNAL_InnerDomElement, domElementWheretoPlaceChildren, true);
                    this.INTERNAL_InnerDomElement = this._contentEditableDiv;
                    var text = this.Text;
                    if (!System.String.isNullOrEmpty(text)) {
                        Windows.UI.Xaml.Controls.TextBox.Text_Changed(this, new Windows.UI.Xaml.DependencyPropertyChangedEventArgs.$ctor1(text, text, Windows.UI.Xaml.Controls.TextBox.TextProperty));
                    }
                    Windows.UI.Xaml.Controls.TextBox.UpdateDomText(this, text);
                }
            },
            InternetExplorer_GotFocus: function (sender, e) {
                this.previousInnerText = getTextAreaInnerText(this.INTERNAL_OptionalSpecifyDomElementConcernedByFocus);
            },
            InternetExplorer_LostFocus: function (sender, e) {
                this.InternetExplorer_RaiseTextChangedIfNecessary();
            },
            InternetExplorer_RaiseTextChangedIfNecessary: function () {
                var newInnerText = getTextAreaInnerText(this.INTERNAL_OptionalSpecifyDomElementConcernedByFocus);
                if (!Bridge.referenceEquals(newInnerText, this.previousInnerText)) {
                    this.TextAreaValueChanged();
                    this.previousInnerText = newInnerText;
                }
            },
            RefreshSelection: function (selectionStart, selectionLength) {
                this.NEW_SET_SELECTION(selectionStart, ((selectionStart + selectionLength) | 0));
            },
            NEW_SET_SELECTION: function (startIndex, endIndex) {

var range = document.createRange();
var sel = window.getSelection();
var nodesAndOffsets = {}; //this will hold the nodes and offsets useful to set the range's start and end.
document.getRangeStartAndEnd(                this._contentEditableDiv, true, 0, Bridge.box(startIndex, System.Int32), Bridge.box(endIndex, System.Int32), nodesAndOffsets, false, false)
range.setStart(nodesAndOffsets['startParent'], nodesAndOffsets['startOffset']);
range.setEnd(nodesAndOffsets['endParent'], nodesAndOffsets['endOffset']);
sel.removeAllRanges();
sel.addRange(range);

            },
            NEW_GET_SELECTION: function (selectionStartIndex, selectionLength) {

                var globalIndexes = 
(function(domElement){
var sel = window.getSelection();
var globalIndexes = {}; //this will hold indexes of start and end and booleans that specify if each has been found.
if(sel.rangeCount == 0)
{
    globalIndexes.startIndex = 0; //apparently, we get 0 and not -1 when nothing is selected.
    globalIndexes.endIndex = 0; //apparently, we get 0 and not -1 when nothing is selected.
}
else
{
    var range = sel.getRangeAt(0);
    document.getRangeGlobalStartAndEndIndexes(domElement, true, 0, range, globalIndexes);
}
return globalIndexes;
}(this._contentEditableDiv))
;

                selectionStartIndex.v = (globalIndexes.isStartFound ? globalIndexes.startIndex : 0);
                selectionLength.v = (globalIndexes.isEndFound ? globalIndexes.endIndex : (globalIndexes.isStartFound ? globalIndexes.startIndex : 0));
            },
            /**
             * Selects all text in the text box.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Controls.TextBox
             * @memberof Windows.UI.Xaml.Controls.TextBox
             * @return  {void}
             */
            SelectAll: function () {
                this.SelectionStart = 0;
                this.SelectionLength = this.Text.length;
            },
            OnAfterApplyHorizontalAlignmentAndWidth: function () {
                if (this._contentEditableDiv != null) {
                    var outerDomStyle = CSHTML5.Internal.INTERNAL_HtmlDomManager.GetFrameworkElementOuterStyleForModification(this);
                    var contentEditableStyle = CSHTML5.Internal.INTERNAL_HtmlDomManager.GetDomElementStyleForModification(this._contentEditableDiv);
                    if (this.TextAreaContainer != null && this.HorizontalContentAlignment === Windows.UI.Xaml.HorizontalAlignment.Stretch) {
                        contentEditableStyle.width = "100%";
                        contentEditableStyle.maxWidth = outerDomStyle.maxWidth;
                    } else {
                        var maxWidth = this.Width;
                        if (isNaN(this.Width)) {
                            if (!isNaN(this.MaxWidth)) {
                                contentEditableStyle.maxWidth = outerDomStyle.maxWidth;
                            }
                        } else {
                            contentEditableStyle.maxWidth = outerDomStyle.width;
                        }
                    }
                }
            },
            OnAfterApplyVerticalAlignmentAndWidth: function () {
                if (this._contentEditableDiv != null) {
                    var outerDomStyle = CSHTML5.Internal.INTERNAL_HtmlDomManager.GetFrameworkElementOuterStyleForModification(this);
                    var contentEditableStyle = CSHTML5.Internal.INTERNAL_HtmlDomManager.GetDomElementStyleForModification(this._contentEditableDiv);
                    if (this.TextAreaContainer != null && this.VerticalContentAlignment === Windows.UI.Xaml.VerticalAlignment.Stretch) {
                        contentEditableStyle.height = "100%";
                        contentEditableStyle.maxHeight = outerDomStyle.maxHeight;
                    } else {
                        var maxHeight = this.Height;
                        if (isNaN(this.Height)) {
                            if (!isNaN(this.MaxHeight)) {
                                contentEditableStyle.maxHeight = outerDomStyle.maxHeight;
                            }
                        } else {
                            contentEditableStyle.maxHeight = outerDomStyle.height;
                        }
                    }
                }
            }
        }
    });

    /**
     * Arranges child elements into a single line that can be oriented horizontally
     or vertically.
     *
     * @public
     * @class Windows.UI.Xaml.Controls.StackPanel
     * @augments Windows.UI.Xaml.Controls.Panel
     * @example You can add a StackPanel with a Horizontal orientation in the XAML as follows:
 
     Or in C#:
     *
     * 
     *     
     * 
     * 
     *
     * StackPanel stackPanel = new StackPanel();
     * stackPanel.Orientation = Orientation.Horizontal;
     * 
     *
     *
     */
    Bridge.define("Windows.UI.Xaml.Controls.StackPanel", {
        inherits: [Windows.UI.Xaml.Controls.Panel],
        statics: {
            fields: {
                /**
                 * Identifies the Orientation dependency property
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Controls.StackPanel
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                OrientationProperty: null
            },
            ctors: {
                init: function () {
                    var $t;
                    this.OrientationProperty = Windows.UI.Xaml.DependencyProperty.Register("Orientation", Windows.UI.Xaml.Controls.Orientation, Windows.UI.Xaml.Controls.StackPanel, ($t = new Windows.UI.Xaml.PropertyMetadata.$ctor2(Bridge.box(Windows.UI.Xaml.Controls.Orientation.Vertical, Windows.UI.Xaml.Controls.Orientation, System.Enum.toStringFn(Windows.UI.Xaml.Controls.Orientation)), Windows.UI.Xaml.Controls.StackPanel.Orientation_Changed), $t.CallPropertyChangedWhenLoadedIntoVisualTree = Windows.UI.Xaml.WhenToCallPropertyChangedEnum.IfPropertyIsSet, $t));
                }
            },
            methods: {
                Orientation_Changed: function (d, e) {
                    var stackPanel = Bridge.cast(d, Windows.UI.Xaml.Controls.StackPanel);
                    var newValue = System.Nullable.getValue(Bridge.cast(Bridge.unbox(e.NewValue, Windows.UI.Xaml.Controls.Orientation), System.Int32));

                    if (CSHTML5.Internal.INTERNAL_VisualTreeManager.IsElementInVisualTree(stackPanel) && System.Nullable.hasValue(stackPanel._renderedOrientation) && System.Nullable.getValue(stackPanel._renderedOrientation) !== newValue) {

                        throw new System.NotSupportedException.$ctor1("Changing the orientation of a StackPanel while it is in the visual tree is not yet supported.");
                    }
                }
            }
        },
        fields: {
            _renderedOrientation: null
        },
        props: {
            /**
             * Gets or sets the dimension by which child elements are stacked.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.StackPanel
             * @function Orientation
             * @type Windows.UI.Xaml.Controls.Orientation
             */
            Orientation: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.Controls.StackPanel.OrientationProperty), Windows.UI.Xaml.Controls.Orientation), System.Int32));
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Controls.StackPanel.OrientationProperty, Bridge.box(value, Windows.UI.Xaml.Controls.Orientation, System.Enum.toStringFn(Windows.UI.Xaml.Controls.Orientation)));
                }
            }
        },
        methods: {
            CreateDomElement: function (parentRef, domElementWhereToPlaceChildren) {

                this._renderedOrientation = this.Orientation;

                if (System.Nullable.eq(this._renderedOrientation, Windows.UI.Xaml.Controls.Orientation.Horizontal)) {









                    var outerDiv = { };
                    var outerDivStyle = CSHTML5.Internal.INTERNAL_HtmlDomManager.CreateDomElementAppendItAndGetStyle("div", parentRef, this, outerDiv);

                    if ((window.IE_VERSION) && !(window.IS_EDGE)) {
                        outerDivStyle.display = !document.isMSGrid ? (outerDivStyle.display = "grid", "grid") : "-ms-grid";
                    } else {
                        outerDivStyle.display = "table";
                    }
                    var innerDiv = { };
                    var innerDivStyle = CSHTML5.Internal.INTERNAL_HtmlDomManager.CreateDomElementAppendItAndGetStyle("div", outerDiv.v, this, innerDiv);

                    innerDivStyle.marginLeft = "0px";
                    innerDivStyle.marginRight = "auto";
                    innerDivStyle.height = "100%";
                    innerDivStyle.display = "table";

                    domElementWhereToPlaceChildren.v = innerDiv.v;

                    return outerDiv.v;
                } else {
                    return this.CreateDomElement_WorkaroundBridgeInheritanceBug(parentRef, domElementWhereToPlaceChildren);
                }
            },
            CreateDomChildWrapper: function (parentRef, domElementWhereToPlaceChild, index) {
                if (this.Orientation === Windows.UI.Xaml.Controls.Orientation.Horizontal) {













                    var div = CSHTML5.Internal.INTERNAL_HtmlDomManager.CreateDomElementAndAppendIt("div", parentRef, this, index);
                    var divStyle = CSHTML5.Internal.INTERNAL_HtmlDomManager.GetDomElementStyleForModification(div);
                    divStyle.position = "relative";
                    divStyle.display = "table-cell";
                    divStyle.height = "100%";
                    divStyle.verticalAlign = "middle";

                    domElementWhereToPlaceChild.v = div;


                    return div;

                } else if (this.Orientation === Windows.UI.Xaml.Controls.Orientation.Vertical) {



                    var div1 = CSHTML5.Internal.INTERNAL_HtmlDomManager.CreateDomElementAndAppendIt("div", parentRef, this, index);
                    var divStyle1 = CSHTML5.Internal.INTERNAL_HtmlDomManager.GetDomElementStyleForModification(div1);
                    divStyle1.position = "relative";
                    divStyle1.width = "100%";

                    domElementWhereToPlaceChild.v = div1;
                    return div1;
                } else {
                    throw new System.NotSupportedException.ctor();
                }
            },
            INTERNAL_OnDetachedFromVisualTree: function () {
                this._renderedOrientation = null;

                Windows.UI.Xaml.Controls.Panel.prototype.INTERNAL_OnDetachedFromVisualTree.call(this);
            }
        }
    });

    /**
     * Positions child elements in sequential position from left to right, breaking
     content to the next line at the edge of the containing box. Subsequent ordering
     happens sequentially from top to bottom or from right to left, depending
     on the value of the System.Windows.Controls.WrapPanel.Orientation property.
     *
     * @public
     * @class Windows.UI.Xaml.Controls.WrapPanel
     * @augments Windows.UI.Xaml.Controls.Panel
     * @example You can add a WrapPanel to the XAML as follows:
 
     Or in C#:
     *
     * 
     *     
     *     
     *     
     * 
     * 
     *
     * WrapPanel myWrapPanel = new WrapPanel();
     * myWrapPanel.Width = 100;
     * myWrapPanel.Height = 100;
     * myWrapPanel.Background = new SolidColorBrush(Windows.UI.Colors.Blue);
     * myWrapPanel.HorizontalAlignment=HorizontalAlignment.Left;
     * //Then you can add the children element using myWrapPanel.Children.Add(...).
     * //Do not forget to add the WrapPanel itself to the visual tree.
     * 
     *
     *
     */
    Bridge.define("Windows.UI.Xaml.Controls.WrapPanel", {
        inherits: [Windows.UI.Xaml.Controls.Panel],
        statics: {
            fields: {
                /**
                 * Identifies the Orientation dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Controls.WrapPanel
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                OrientationProperty: null
            },
            ctors: {
                init: function () {
                    var $t;
                    this.OrientationProperty = Windows.UI.Xaml.DependencyProperty.Register("Orientation", Windows.UI.Xaml.Controls.Orientation, Windows.UI.Xaml.Controls.WrapPanel, ($t = new Windows.UI.Xaml.PropertyMetadata.$ctor2(Bridge.box(Windows.UI.Xaml.Controls.Orientation.Horizontal, Windows.UI.Xaml.Controls.Orientation, System.Enum.toStringFn(Windows.UI.Xaml.Controls.Orientation)), Windows.UI.Xaml.Controls.WrapPanel.Orientation_Changed), $t.CallPropertyChangedWhenLoadedIntoVisualTree = Windows.UI.Xaml.WhenToCallPropertyChangedEnum.IfPropertyIsSet, $t));
                }
            },
            methods: {
                Orientation_Changed: function (d, e) {
                    var wrapPanel = Bridge.cast(d, Windows.UI.Xaml.Controls.WrapPanel);
                    var newValue = System.Nullable.getValue(Bridge.cast(Bridge.unbox(e.NewValue, Windows.UI.Xaml.Controls.Orientation), System.Int32));
                    if (CSHTML5.Internal.INTERNAL_VisualTreeManager.IsElementInVisualTree(wrapPanel)) {
                        if (newValue === Windows.UI.Xaml.Controls.Orientation.Vertical) {
                            throw new System.NotSupportedException.$ctor1("Vertical orientation is not yet supported for the WrapPanel");
                        } else {
                            CSHTML5.Internal.INTERNAL_HtmlDomManager.GetFrameworkElementOuterStyleForModification(wrapPanel).textAlign = "left";
                        }
                    }
                }
            }
        },
        props: {
            /**
             * Gets or sets a value that specifies the dimension in which child content
             is arranged.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.WrapPanel
             * @function Orientation
             * @type Windows.UI.Xaml.Controls.Orientation
             */
            Orientation: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.Controls.WrapPanel.OrientationProperty), Windows.UI.Xaml.Controls.Orientation), System.Int32));
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Controls.WrapPanel.OrientationProperty, Bridge.box(value, Windows.UI.Xaml.Controls.Orientation, System.Enum.toStringFn(Windows.UI.Xaml.Controls.Orientation)));
                }
            }
        },
        ctors: {
            /**
             * Initializes a new instance of the System.Windows.Controls.WrapPanel class.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Controls.WrapPanel
             * @memberof Windows.UI.Xaml.Controls.WrapPanel
             * @return  {void}
             */
            ctor: function () {
                this.$initialize();
                Windows.UI.Xaml.Controls.Panel.ctor.call(this);
            }
        },
        methods: {
            CreateDomElement: function (parentRef, domElementWhereToPlaceChildren) {
                var div = CSHTML5.Internal.INTERNAL_HtmlDomManager.CreateDomElementAndAppendIt("div", parentRef, this);
                var divStyle = CSHTML5.Internal.INTERNAL_HtmlDomManager.GetDomElementStyleForModification(div);
                divStyle.lineHeight = "0px";
                divStyle.whiteSpace = "normal";
                domElementWhereToPlaceChildren.v = div;
                return div;
            },
            CreateDomChildWrapper: function (parentRef, domElementWhereToPlaceChild, index) {
                if (index === void 0) { index = -1; }
                if (this.Orientation === Windows.UI.Xaml.Controls.Orientation.Horizontal) {
                    var div = CSHTML5.Internal.INTERNAL_HtmlDomManager.CreateDomElementAndAppendIt("div", parentRef, this, index);
                    var divStyle = CSHTML5.Internal.INTERNAL_HtmlDomManager.GetDomElementStyleForModification(div);
                    divStyle.display = "inline-block";
                    divStyle.lineHeight = "initial";
                    domElementWhereToPlaceChild.v = div;
                    return div;
                } else {
                    domElementWhereToPlaceChild.v = null;
                    return null;
                }
            }
        }
    });

    /**
     * Provides a lightweight control for displaying small amounts of text.
     *
     * @public
     * @class Windows.UI.Xaml.Controls.TextBlock
     * @augments Windows.UI.Xaml.Controls.Control
     * @example
     *
     *
     * TextBlockName.Text = "Some text.";
     * 
     *
     *
     */
    Bridge.define("Windows.UI.Xaml.Controls.TextBlock", {
        inherits: [Windows.UI.Xaml.Controls.Control],
        statics: {
            fields: {
                /**
                 * Identifies the Text dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Controls.TextBlock
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                TextProperty: null,
                /**
                 * Identifies the TextAlignment dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Controls.TextBlock
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                TextAlignmentProperty: null,
                /**
                 * Identifies the TextWrapping dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Controls.TextBlock
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                TextWrappingProperty: null
            },
            ctors: {
                init: function () {
                    var $t;
                    this.TextProperty = Windows.UI.Xaml.DependencyProperty.Register("Text", System.String, Windows.UI.Xaml.Controls.TextBlock, new Windows.UI.Xaml.PropertyMetadata.$ctor2("", Windows.UI.Xaml.Controls.TextBlock.OnTextPropertyChanged));
                    this.TextAlignmentProperty = Windows.UI.Xaml.DependencyProperty.Register("TextAlignment", Windows.UI.Xaml.TextAlignment, Windows.UI.Xaml.Controls.TextBlock, ($t = new Windows.UI.Xaml.PropertyMetadata.$ctor1(Bridge.box(Windows.UI.Xaml.TextAlignment.Left, Windows.UI.Xaml.TextAlignment, System.Enum.toStringFn(Windows.UI.Xaml.TextAlignment))), $t.MethodToUpdateDom = Windows.UI.Xaml.Controls.TextBlock.TextAlignment_MethodToUpdateDom, $t));
                    this.TextWrappingProperty = Windows.UI.Xaml.DependencyProperty.Register("TextWrapping", Windows.UI.Xaml.TextWrapping, Windows.UI.Xaml.Controls.TextBlock, ($t = new Windows.UI.Xaml.PropertyMetadata.$ctor1(Bridge.box(Windows.UI.Xaml.TextWrapping.NoWrap, Windows.UI.Xaml.TextWrapping, System.Enum.toStringFn(Windows.UI.Xaml.TextWrapping))), $t.GetCSSEquivalent = function (instance) {
                        var $t1;
                        return ($t1 = new Windows.UI.Xaml.CSSEquivalent(), $t1.Value = function (inst, value) {
                            var newTextWrapping = System.Nullable.getValue(Bridge.cast(Bridge.unbox(value, Windows.UI.Xaml.TextWrapping), System.Int32));
                            switch (newTextWrapping) {
                                case Windows.UI.Xaml.TextWrapping.Wrap: 
                                    return "pre-wrap";
                                case Windows.UI.Xaml.TextWrapping.NoWrap: 
                                default: 
                                    return "pre";
                            }
                        }, $t1.Name = function (_o1) {
                                _o1.add("whiteSpace");
                                return _o1;
                            }(new (System.Collections.Generic.List$1(System.String)).ctor()), $t1);
                    }, $t));
                }
            },
            methods: {
                OnTextPropertyChanged: function (d, e) {
                    var $t;
                    var textBlock = Bridge.cast(d, Windows.UI.Xaml.Controls.TextBlock);
                    if (!textBlock._isTextChanging) {
                        textBlock._isTextChanging = true;
                        textBlock.Inlines.clear();
                        textBlock.Inlines.add(($t = new Windows.UI.Xaml.Documents.Run(), $t.Text = Bridge.cast(e.NewValue, System.String), $t));
                        textBlock._isTextChanging = false;
                    }
                },
                TextAlignment_MethodToUpdateDom: function (d, newValue) {
                    var textBlock = Bridge.cast(d, Windows.UI.Xaml.Controls.TextBlock);
                    switch (System.Nullable.getValue(Bridge.cast(Bridge.unbox(newValue, Windows.UI.Xaml.TextAlignment), System.Int32))) {
                        case Windows.UI.Xaml.TextAlignment.Center: 
                            CSHTML5.Internal.INTERNAL_HtmlDomManager.GetFrameworkElementOuterStyleForModification(textBlock).textAlign = "center";
                            break;
                        case Windows.UI.Xaml.TextAlignment.Left: 
                            CSHTML5.Internal.INTERNAL_HtmlDomManager.GetFrameworkElementOuterStyleForModification(textBlock).textAlign = "left";
                            break;
                        case Windows.UI.Xaml.TextAlignment.Right: 
                            CSHTML5.Internal.INTERNAL_HtmlDomManager.GetFrameworkElementOuterStyleForModification(textBlock).textAlign = "right";
                            break;
                        case Windows.UI.Xaml.TextAlignment.Justify: 
                            CSHTML5.Internal.INTERNAL_HtmlDomManager.GetFrameworkElementOuterStyleForModification(textBlock).textAlign = "justify";
                            break;
                        default: 
                            break;
                    }
                }
            }
        },
        fields: {
            _isTextChanging: false,
            Inlines: null
        },
        props: {
            /**
             * Get or Set the Text property
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.TextBlock
             * @function Text
             * @type string
             */
            Text: {
                get: function () {
                    return Bridge.cast(this.GetValue(Windows.UI.Xaml.Controls.TextBlock.TextProperty), System.String);
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Controls.TextBlock.TextProperty, value);
                }
            },
            /**
             * Gets or sets how the text should be aligned in the TextBlock.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.TextBlock
             * @function TextAlignment
             * @type Windows.UI.Xaml.TextAlignment
             */
            TextAlignment: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.Controls.TextBlock.TextAlignmentProperty), Windows.UI.Xaml.TextAlignment), System.Int32));
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Controls.TextBlock.TextAlignmentProperty, Bridge.box(value, Windows.UI.Xaml.TextAlignment, System.Enum.toStringFn(Windows.UI.Xaml.TextAlignment)));
                }
            },
            /**
             * Gets or sets how the TextBlock wraps text.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.TextBlock
             * @function TextWrapping
             * @type Windows.UI.Xaml.TextWrapping
             */
            TextWrapping: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.Controls.TextBlock.TextWrappingProperty), Windows.UI.Xaml.TextWrapping), System.Int32));
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Controls.TextBlock.TextWrappingProperty, Bridge.box(value, Windows.UI.Xaml.TextWrapping, System.Enum.toStringFn(Windows.UI.Xaml.TextWrapping)));
                }
            }
        },
        ctors: {
            ctor: function () {
                this.$initialize();
                Windows.UI.Xaml.Controls.Control.ctor.call(this);
                this.IsTabStop = false;
                this.Inlines = new Windows.UI.Xaml.Documents.InlineCollection(this);
            }
        },
        methods: {
            CreateDomElement: function (parentRef, domElementWhereToPlaceChildren) {
                var div = CSHTML5.Internal.INTERNAL_HtmlDomManager.CreateDomElementAndAppendIt("div", parentRef, this);
                var divStyle = CSHTML5.Internal.INTERNAL_HtmlDomManager.GetDomElementStyleForModification(div);
                divStyle.whiteSpace = this.TextWrapping === Windows.UI.Xaml.TextWrapping.NoWrap ? "pre" : "pre-wrap";
                divStyle.overflow = "hidden";
                divStyle.textAlign = "left";
                domElementWhereToPlaceChildren.v = div;
                return div;
            },
            SetTextPropertyNoCallBack: function (text) {
                if (!this._isTextChanging) {
                    this._isTextChanging = true;
                    this.SetCurrentValue(Windows.UI.Xaml.Controls.TextBlock.TextProperty, text);
                    this._isTextChanging = false;
                }
            },
            INTERNAL_OnAttachedToVisualTree: function () {
                var $t;
                Windows.UI.Xaml.Controls.Control.prototype.INTERNAL_OnAttachedToVisualTree.call(this);

                $t = Bridge.getEnumerator(this.Inlines);
                try {
                    while ($t.moveNext()) {
                        var child = $t.Current;
                        CSHTML5.Internal.INTERNAL_VisualTreeManager.AttachVisualChildIfNotAlreadyAttached(child, this);
                    }
                } finally {
                    if (Bridge.is($t, System.IDisposable)) {
                        $t.System$IDisposable$Dispose();
                    }
                }
            },
            UpdateTabIndex: function (isTabStop, tabIndex) {
                return;
            }
        }
    });

    Bridge.define("Windows.UI.Xaml.Controls.TileView", {
        inherits: [Windows.UI.Xaml.Controls.Control],
        fields: {
            _items: null,
            _maximizedTile: null,
            _contentGrid: null
        },
        props: {
            Items: {
                get: function () {
                    if (this._items == null) {
                        this._items = new (System.Collections.ObjectModel.ObservableCollection$1(Windows.UI.Xaml.Controls.TileViewItem)).ctor();
                        this._items.addCollectionChanged(Bridge.fn.cacheBind(this, this.items_CollectionChanged));
                    }
                    return this._items;
                }
            }
        },
        ctors: {
            ctor: function () {
                this.$initialize();
                Windows.UI.Xaml.Controls.Control.ctor.call(this);
                this.DefaultStyleKey = Windows.UI.Xaml.Controls.TileView;
            }
        },
        methods: {
            items_CollectionChanged: function (sender, e) { },
            AddItemsToVisualTree: function (oldItems, newItems) {
                var $t, $t1, $t2;
                if (this._contentGrid != null) {
                    if (this._contentGrid.ColumnDefinitions == null || this._contentGrid.ColumnDefinitions.Count !== 2) {
                        this._contentGrid.ColumnDefinitions.clear();
                        this._contentGrid.ColumnDefinitions.add(($t = new Windows.UI.Xaml.Controls.ColumnDefinition(), $t.Width = new Windows.UI.Xaml.GridLength.$ctor2(1, Windows.UI.Xaml.GridUnitType.Star), $t));
                        this._contentGrid.ColumnDefinitions.add(($t = new Windows.UI.Xaml.Controls.ColumnDefinition(), $t.Width = Windows.UI.Xaml.GridLength.Auto.$clone(), $t));
                    }
                    if (this._contentGrid.RowDefinitions == null || this._contentGrid.RowDefinitions.Count !== 0) {
                        this._contentGrid.RowDefinitions.clear();
                    }
                    if (!Bridge.referenceEquals(oldItems, newItems)) {
                        if (oldItems != null) {
                            if (this._contentGrid != null) {
                                $t = Bridge.getEnumerator(oldItems);
                                try {
                                    while ($t.moveNext()) {
                                        var oldItem = $t.Current;
                                        this._contentGrid.Children.remove(oldItem);
                                    }
                                } finally {
                                    if (Bridge.is($t, System.IDisposable)) {
                                        $t.System$IDisposable$Dispose();
                                    }
                                }
                            }
                        }
                        if (newItems != null) {
                            var isFirst = true;
                            var n = 0;
                            $t1 = Bridge.getEnumerator(newItems);
                            try {
                                while ($t1.moveNext()) {
                                    var item = $t1.Current;
                                    if (!isFirst) {
                                        Windows.UI.Xaml.Controls.Grid.SetColumn(item, 1);
                                        Windows.UI.Xaml.Controls.Grid.SetRow(item, n);
                                        n = (n + 1) | 0;
                                    } else {
                                        Windows.UI.Xaml.Controls.Grid.SetRowSpan(item, 2147483647);
                                        item.Maximize();
                                        this._maximizedTile = item;
                                    }
                                    this._contentGrid.Children.add(item);
                                    item._TileViewParent = this;
                                    isFirst = false;
                                }
                            } finally {
                                if (Bridge.is($t1, System.IDisposable)) {
                                    $t1.System$IDisposable$Dispose();
                                }
                            }

                            var rowsToAdd = oldItems != null ? ((newItems.Count - oldItems.Count) | 0) : newItems.Count;
                            if (rowsToAdd > 0) {
                                for (var i = 0; i < rowsToAdd; i = (i + 1) | 0) {
                                    this._contentGrid.RowDefinitions.add(($t2 = new Windows.UI.Xaml.Controls.RowDefinition(), $t2.Height = new Windows.UI.Xaml.GridLength.$ctor2(1, Windows.UI.Xaml.GridUnitType.Star), $t2));
                                }
                            } else {
                                for (var i1 = 0; i1 > rowsToAdd; i1 = (i1 - 1) | 0) {
                                    this._contentGrid.RowDefinitions.removeAt(this._contentGrid.RowDefinitions.Count);
                                }
                            }
                        }
                    }
                }
            },
            MaximizeTile: function (tileToMaximize) {
                if (!Bridge.referenceEquals(tileToMaximize, this._maximizedTile)) {
                    if (this._maximizedTile != null) {
                        var maximizedTileRow = Windows.UI.Xaml.Controls.Grid.GetRow(this._maximizedTile);
                        var maximizedTileColumn = Windows.UI.Xaml.Controls.Grid.GetColumn(this._maximizedTile);
                        Windows.UI.Xaml.Controls.Grid.SetRow(this._maximizedTile, Windows.UI.Xaml.Controls.Grid.GetRow(tileToMaximize));
                        Windows.UI.Xaml.Controls.Grid.SetColumn(this._maximizedTile, Windows.UI.Xaml.Controls.Grid.GetColumn(tileToMaximize));
                        Windows.UI.Xaml.Controls.Grid.SetRowSpan(tileToMaximize, Windows.UI.Xaml.Controls.Grid.GetRowSpan(this._maximizedTile));
                        Windows.UI.Xaml.Controls.Grid.SetRow(tileToMaximize, maximizedTileRow);
                        Windows.UI.Xaml.Controls.Grid.SetColumn(tileToMaximize, maximizedTileColumn);
                        Windows.UI.Xaml.Controls.Grid.SetRowSpan(this._maximizedTile, 1);
                    } else {
                        Windows.UI.Xaml.Controls.Grid.SetRow(tileToMaximize, 0);
                        Windows.UI.Xaml.Controls.Grid.SetColumn(tileToMaximize, 0);
                        Windows.UI.Xaml.Controls.Grid.SetRowSpan(tileToMaximize, 2147483647);
                    }
                    this._maximizedTile.Minimize();
                    tileToMaximize.Maximize();
                    this._maximizedTile = tileToMaximize;
                }
            },
            OnApplyTemplate: function () {
                Windows.UI.Xaml.Controls.Control.prototype.OnApplyTemplate.call(this);
                this._contentGrid = Bridge.as(this.GetTemplateChild("PART_ItemsDisplayGrid"), Windows.UI.Xaml.Controls.Grid);
                this.AddItemsToVisualTree(null, this.Items);
            }
        }
    });

    /** @namespace Windows.UI.Xaml.Documents */

    /**
     * An abstract class used as the base class for the also-abstract Block and Inline classes. 
     TextElement supports common API for classes involved in the XAML text object model,
     such as properties that control text size, font families and so on.
     *
     * @abstract
     * @public
     * @class Windows.UI.Xaml.Documents.TextElement
     * @augments Windows.UI.Xaml.Controls.Control
     */
    Bridge.define("Windows.UI.Xaml.Documents.TextElement", {
        inherits: [Windows.UI.Xaml.Controls.Control],
        methods: {
            UpdateTabIndex: function (isTabStop, tabIndex) {
                return;
            }
        }
    });

    /**
     * Draws an ellipse.
     *
     * @public
     * @class Windows.UI.Xaml.Shapes.Ellipse
     * @augments Windows.UI.Xaml.Shapes.Shape
     */
    Bridge.define("Windows.UI.Xaml.Shapes.Ellipse", {
        inherits: [Windows.UI.Xaml.Shapes.Shape],
        statics: {
            ctors: {
                ctor: function () {
                    var $t;
                    Windows.UI.Xaml.Shapes.Shape.StretchProperty.OverrideMetadata(Windows.UI.Xaml.Shapes.Ellipse, ($t = new Windows.UI.Xaml.PropertyMetadata.$ctor2(Bridge.box(Windows.UI.Xaml.Media.Stretch.Fill, Windows.UI.Xaml.Media.Stretch, System.Enum.toStringFn(Windows.UI.Xaml.Media.Stretch)), Windows.UI.Xaml.Shapes.Shape.Stretch_Changed), $t.CallPropertyChangedWhenLoadedIntoVisualTree = Windows.UI.Xaml.WhenToCallPropertyChangedEnum.IfPropertyIsSet, $t));
                }
            }
        },
        methods: {
            CreateDomElement: function (parentRef, domElementWhereToPlaceChildren) {
                return CSHTML5.Internal.INTERNAL_ShapesDrawHelpers.CreateDomElementForPathAndSimilar(this, parentRef, Bridge.ref(this, "_canvasDomElement"), domElementWhereToPlaceChildren);

            },
            Redraw: function () {
                if (CSHTML5.Internal.INTERNAL_VisualTreeManager.IsElementInVisualTree(this)) {
                    var xOffsetToApplyBeforeMultiplication = { };
                    var yOffsetToApplyBeforeMultiplication = { };
                    var xOffsetToApplyAfterMultiplication = { };
                    var yOffsetToApplyAfterMultiplication = { };
                    var sizeX = { };
                    var sizeY = { };
                    var horizontalMultiplicator = { };
                    var verticalMultiplicator = { };
                    var shapeActualSize = { v : new Windows.Foundation.Size() };
                    Windows.UI.Xaml.Shapes.Shape.GetShapeInfos(this, xOffsetToApplyBeforeMultiplication, yOffsetToApplyBeforeMultiplication, xOffsetToApplyAfterMultiplication, yOffsetToApplyAfterMultiplication, sizeX, sizeY, horizontalMultiplicator, verticalMultiplicator, shapeActualSize);

                    this.ApplyMarginToFixNegativeCoordinates(new Windows.Foundation.Point.ctor());
                    if (this.Stretch === Windows.UI.Xaml.Media.Stretch.None) {
                        this.ApplyMarginToFixNegativeCoordinates(this._marginOffsets.$clone());
                    }

                    CSHTML5.Internal.INTERNAL_ShapesDrawHelpers.PrepareEllipse(this._canvasDomElement, sizeX.v, sizeY.v, sizeX.v / 2 + xOffsetToApplyBeforeMultiplication.v + xOffsetToApplyAfterMultiplication.v, sizeY.v / 2 + yOffsetToApplyBeforeMultiplication.v + yOffsetToApplyAfterMultiplication.v);

                    Windows.UI.Xaml.Shapes.Shape.DrawFillAndStroke(this, "evenodd", xOffsetToApplyAfterMultiplication.v, yOffsetToApplyAfterMultiplication.v, xOffsetToApplyAfterMultiplication.v + sizeX.v, yOffsetToApplyAfterMultiplication.v + sizeY.v, horizontalMultiplicator.v, verticalMultiplicator.v, xOffsetToApplyBeforeMultiplication.v, yOffsetToApplyBeforeMultiplication.v, shapeActualSize.v.$clone());


                }
            }
        }
    });

    /**
     * Draws a straight line between two points.
     *
     * @public
     * @class Windows.UI.Xaml.Shapes.Line
     * @augments Windows.UI.Xaml.Shapes.Shape
     */
    Bridge.define("Windows.UI.Xaml.Shapes.Line", {
        inherits: [Windows.UI.Xaml.Shapes.Shape],
        statics: {
            fields: {
                /**
                 * Identifies the X1 dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Shapes.Line
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                X1Property: null,
                /**
                 * Identifies the X2 dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Shapes.Line
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                X2Property: null,
                /**
                 * Identifies the Y1 dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Shapes.Line
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                Y1Property: null,
                /**
                 * Identifies the Y2 dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Shapes.Line
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                Y2Property: null
            },
            ctors: {
                init: function () {
                    this.X1Property = Windows.UI.Xaml.DependencyProperty.Register("X1", System.Double, Windows.UI.Xaml.Shapes.Line, new Windows.UI.Xaml.PropertyMetadata.$ctor2(Bridge.box(0.0, System.Double, System.Double.format, System.Double.getHashCode), Windows.UI.Xaml.Shapes.Line.X1_Changed));
                    this.X2Property = Windows.UI.Xaml.DependencyProperty.Register("X2", System.Double, Windows.UI.Xaml.Shapes.Line, new Windows.UI.Xaml.PropertyMetadata.$ctor2(Bridge.box(0.0, System.Double, System.Double.format, System.Double.getHashCode), Windows.UI.Xaml.Shapes.Line.X2_Changed));
                    this.Y1Property = Windows.UI.Xaml.DependencyProperty.Register("Y1", System.Double, Windows.UI.Xaml.Shapes.Line, new Windows.UI.Xaml.PropertyMetadata.$ctor2(Bridge.box(0.0, System.Double, System.Double.format, System.Double.getHashCode), Windows.UI.Xaml.Shapes.Line.Y1_Changed));
                    this.Y2Property = Windows.UI.Xaml.DependencyProperty.Register("Y2", System.Double, Windows.UI.Xaml.Shapes.Line, new Windows.UI.Xaml.PropertyMetadata.$ctor2(Bridge.box(0.0, System.Double, System.Double.format, System.Double.getHashCode), Windows.UI.Xaml.Shapes.Line.Y2_Changed));
                }
            },
            methods: {
                X1_Changed: function (d, e) {
                    var line = Bridge.cast(d, Windows.UI.Xaml.Shapes.Line);
                    line.ScheduleRedraw();
                },
                X2_Changed: function (d, e) {
                    var line = Bridge.cast(d, Windows.UI.Xaml.Shapes.Line);
                    line.ScheduleRedraw();
                },
                Y1_Changed: function (d, e) {
                    var line = Bridge.cast(d, Windows.UI.Xaml.Shapes.Line);
                    line.ScheduleRedraw();
                },
                Y2_Changed: function (d, e) {
                    var line = Bridge.cast(d, Windows.UI.Xaml.Shapes.Line);
                    line.ScheduleRedraw();
                }
            }
        },
        props: {
            /**
             * Gets or sets the x-coordinate of the Line start point.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Shapes.Line
             * @function X1
             * @type number
             */
            X1: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.Shapes.Line.X1Property), System.Double), System.Double));
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Shapes.Line.X1Property, Bridge.box(value, System.Double, System.Double.format, System.Double.getHashCode));
                }
            },
            /**
             * Gets or sets the x-coordinate of the Line end point.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Shapes.Line
             * @function X2
             * @type number
             */
            X2: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.Shapes.Line.X2Property), System.Double), System.Double));
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Shapes.Line.X2Property, Bridge.box(value, System.Double, System.Double.format, System.Double.getHashCode));
                }
            },
            /**
             * Gets or sets the y-coordinate of the Line start point.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Shapes.Line
             * @function Y1
             * @type number
             */
            Y1: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.Shapes.Line.Y1Property), System.Double), System.Double));
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Shapes.Line.Y1Property, Bridge.box(value, System.Double, System.Double.format, System.Double.getHashCode));
                }
            },
            /**
             * Gets or sets the y-coordinate of the Line end point.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Shapes.Line
             * @function Y2
             * @type number
             */
            Y2: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.Shapes.Line.Y2Property), System.Double), System.Double));
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Shapes.Line.Y2Property, Bridge.box(value, System.Double, System.Double.format, System.Double.getHashCode));
                }
            }
        },
        methods: {
            CreateDomElement: function (parentRef, domElementWhereToPlaceChildren) {
                return CSHTML5.Internal.INTERNAL_ShapesDrawHelpers.CreateDomElementForPathAndSimilar(this, parentRef, Bridge.ref(this, "_canvasDomElement"), domElementWhereToPlaceChildren);

            },
            GetMinMaxXY: function (minX, maxX, minY, maxY) {
                var maxAbs = this.X1 > this.X2 ? this.X1 : this.X2;
                var minAbs = this.X1 < this.X2 ? this.X1 : this.X2;
                var minOrd = this.Y1 < this.Y2 ? this.Y1 : this.Y2;
                var maxOrd = this.Y1 > this.Y2 ? this.Y1 : this.Y2;
                if (maxX.v < maxAbs) {
                    maxX.v = maxAbs;
                }
                if (maxY.v < maxOrd) {
                    maxY.v = maxOrd;
                }
                if (minX.v > minAbs) {
                    minX.v = minAbs;
                }
                if (minY.v > minOrd) {
                    minY.v = minOrd;
                }
            },
            Redraw: function () {
                if (CSHTML5.Internal.INTERNAL_VisualTreeManager.IsElementInVisualTree(this)) {
                    var minX = this.X1;
                    var minY = this.Y1;
                    var maxX = this.X2;
                    var maxY = this.Y2;
                    if (this.X1 > this.X2) {
                        minX = this.X2;
                        maxX = this.X1;
                    }
                    if (this.Y1 > this.Y2) {
                        minY = this.Y2;
                        maxY = this.Y1;
                    }

                    var shapeActualSize = { v : new Windows.Foundation.Size() };
                    CSHTML5.Internal.INTERNAL_ShapesDrawHelpers.PrepareStretch(this, this._canvasDomElement, minX, maxX, minY, maxY, this.Stretch, shapeActualSize);

                    var horizontalMultiplicator = { };
                    var verticalMultiplicator = { };
                    var xOffsetToApplyBeforeMultiplication = { };
                    var yOffsetToApplyBeforeMultiplication = { };
                    var xOffsetToApplyAfterMultiplication = { };
                    var yOffsetToApplyAfterMultiplication = { };
                    CSHTML5.Internal.INTERNAL_ShapesDrawHelpers.GetMultiplicatorsAndOffsetForStretch(this, this.StrokeThickness, minX, maxX, minY, maxY, this.Stretch, shapeActualSize.v.$clone(), horizontalMultiplicator, verticalMultiplicator, xOffsetToApplyBeforeMultiplication, yOffsetToApplyBeforeMultiplication, xOffsetToApplyAfterMultiplication, yOffsetToApplyAfterMultiplication, Bridge.ref(this, "_marginOffsets"));

                    this.ApplyMarginToFixNegativeCoordinates(new Windows.Foundation.Point.ctor());

                    if (this.Stretch === Windows.UI.Xaml.Media.Stretch.None) {
                        this.ApplyMarginToFixNegativeCoordinates(this._marginOffsets.$clone());
                    }

                    var context = this._canvasDomElement.getContext('2d');

                    context.clearRect(0,0, Bridge.box(shapeActualSize.v.Width, System.Double, System.Double.format, System.Double.getHashCode), Bridge.box(shapeActualSize.v.Height, System.Double, System.Double.format, System.Double.getHashCode));


                    var preparedX1 = this.X1 * horizontalMultiplicator.v + xOffsetToApplyBeforeMultiplication.v * horizontalMultiplicator.v + xOffsetToApplyAfterMultiplication.v;
                    var preparedX2 = this.X2 * horizontalMultiplicator.v + xOffsetToApplyBeforeMultiplication.v * horizontalMultiplicator.v + xOffsetToApplyAfterMultiplication.v;
                    var preparedY1 = this.Y1 * verticalMultiplicator.v + yOffsetToApplyBeforeMultiplication.v * verticalMultiplicator.v + yOffsetToApplyAfterMultiplication.v;
                    var preparedY2 = this.Y2 * verticalMultiplicator.v + yOffsetToApplyBeforeMultiplication.v * verticalMultiplicator.v + yOffsetToApplyAfterMultiplication.v;

                    var opacity = this.Stroke == null ? 1 : this.Stroke.Opacity;
                    var strokeValue = Windows.UI.Xaml.Shapes.Shape.GetHtmlBrush(this, this.Stroke, opacity, minX, minY, maxX, maxY, horizontalMultiplicator.v, verticalMultiplicator.v, xOffsetToApplyBeforeMultiplication.v, yOffsetToApplyBeforeMultiplication.v, shapeActualSize.v.$clone());

                    if (strokeValue != null && this.StrokeThickness > 0) {
                        var thickness = this.StrokeThickness;

                        context.strokeStyle = strokeValue;

                        context.lineWidth = Bridge.box(this.StrokeThickness, System.Double, System.Double.format, System.Double.getHashCode);
                        if (this.StrokeDashArray != null) {
                            if (CSHTML5.Interop.IsRunningInTheSimulator) {
                            } else {
                                var options = new Array();
                                for (var i = 0; i < this.StrokeDashArray.Count; i = (i + 1) | 0) {

                                    options[Bridge.box(i, System.Int32)] = Bridge.box(this.StrokeDashArray.getItem(i) * thickness, System.Double, System.Double.format, System.Double.getHashCode);

                                }


if (                                context.setLineDash)
    context.setLineDash(options);
                            }
                        }
                    }


                    CSHTML5.Internal.INTERNAL_ShapesDrawHelpers.PrepareLine(this._canvasDomElement, new Windows.Foundation.Point.$ctor1(preparedX1, preparedY1), new Windows.Foundation.Point.$ctor1(preparedX2, preparedY2));

                    if (strokeValue != null) {
                        context.strokeStyle = strokeValue;
                    }

                    context.lineWidth= System.Double.format(this.StrokeThickness);
                    if (this.Stroke != null && this.StrokeThickness > 0) {
                        context.stroke();
                    }
                }
            }
        }
    });

    /**
     * Draws a series of connected lines and curves. The line and curve dimensions
     are declared through the Data property, and can be specified either with
     a path-specific mini-language, or with an object model.
     *
     * @public
     * @class Windows.UI.Xaml.Shapes.Path
     * @augments Windows.UI.Xaml.Shapes.Shape
     */
    Bridge.define("Windows.UI.Xaml.Shapes.Path", {
        inherits: [Windows.UI.Xaml.Shapes.Shape],
        statics: {
            fields: {
                /**
                 * Identifies the Data dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Shapes.Path
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                DataProperty: null
            },
            ctors: {
                init: function () {
                    this.DataProperty = Windows.UI.Xaml.DependencyProperty.Register("Data", Windows.UI.Xaml.Media.Geometry, Windows.UI.Xaml.Shapes.Path, new Windows.UI.Xaml.PropertyMetadata.$ctor2(null, Windows.UI.Xaml.Shapes.Path.Data_Changed));
                }
            },
            methods: {
                Data_Changed: function (d, e) {
                    var path = Bridge.cast(d, Windows.UI.Xaml.Shapes.Path);
                    if (null != e.OldValue) {
                        Bridge.cast(e.OldValue, Windows.UI.Xaml.Media.Geometry).SetParentPath(null);
                    }
                    if (null != e.NewValue) {
                        Bridge.cast(e.NewValue, Windows.UI.Xaml.Media.Geometry).SetParentPath(path);
                    }
                    path.ScheduleRedraw();
                }
            }
        },
        props: {
            /**
             * Gets or sets a Geometry that specifies the shape to be drawn.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Shapes.Path
             * @function Data
             * @type Windows.UI.Xaml.Media.Geometry
             */
            Data: {
                get: function () {
                    return Bridge.cast(this.GetValue(Windows.UI.Xaml.Shapes.Path.DataProperty), Windows.UI.Xaml.Media.Geometry);
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Shapes.Path.DataProperty, value);
                }
            }
        },
        methods: {
            CreateDomElement: function (parentRef, domElementWhereToPlaceChildren) {
                return CSHTML5.Internal.INTERNAL_ShapesDrawHelpers.CreateDomElementForPathAndSimilar(this, parentRef, Bridge.ref(this, "_canvasDomElement"), domElementWhereToPlaceChildren);
            },
            Redraw: function () {
                if (CSHTML5.Internal.INTERNAL_VisualTreeManager.IsElementInVisualTree(this)) {
                    if (this.Data != null) {
                        var minX = { v : System.Double.max };
                        var minY = { v : System.Double.max };
                        var maxX = { v : System.Double.min };
                        var maxY = { v : System.Double.min };
                        var pathGeometry;
                        if (((pathGeometry = Bridge.as(this.Data, Windows.UI.Xaml.Media.PathGeometry))) != null) {
                            if (pathGeometry.Figures == null || pathGeometry.Figures.Count === 0) {
                                return;
                            }
                        }
                        this.Data.GetMinMaxXY(minX, maxX, minY, maxY);


                        var originalTransform = this.Data.Transform;
                        var actualTransform = { v : originalTransform };
                        var int32FactorX = { v : 1.0 };
                        var int32FactorY = { v : 1.0 };
                        CSHTML5.Internal.INTERNAL_ShapesDrawHelpers.FixCoordinatesGreaterThanInt32MaxValue(minX.v, minY.v, maxX.v, maxY.v, actualTransform, int32FactorX, int32FactorY);

                        if (originalTransform != null) {

                            var tmp1 = originalTransform.Transform(new Windows.Foundation.Point.$ctor1(minX.v, minY.v));
                            var tmp2 = originalTransform.Transform(new Windows.Foundation.Point.$ctor1(maxX.v, maxY.v));
                            minX.v = tmp1._x;
                            minY.v = tmp1._y;
                            maxX.v = tmp2._x;
                            maxY.v = tmp2._y;
                        }

                        var shapeActualSize = { v : new Windows.Foundation.Size() };
                        CSHTML5.Internal.INTERNAL_ShapesDrawHelpers.PrepareStretch(this, this._canvasDomElement, minX.v, maxX.v, minY.v, maxY.v, this.Stretch, shapeActualSize);

                        var horizontalMultiplicator = { };
                        var verticalMultiplicator = { };
                        var xOffsetToApplyBeforeMultiplication = { };
                        var yOffsetToApplyBeforeMultiplication = { };
                        var xOffsetToApplyAfterMultiplication = { };
                        var yOffsetToApplyAfterMultiplication = { };
                        var strokeThickness = this.Stroke == null ? 0.0 : this.StrokeThickness;
                        CSHTML5.Internal.INTERNAL_ShapesDrawHelpers.GetMultiplicatorsAndOffsetForStretch(this, strokeThickness, minX.v, maxX.v, minY.v, maxY.v, this.Stretch, shapeActualSize.v.$clone(), horizontalMultiplicator, verticalMultiplicator, xOffsetToApplyBeforeMultiplication, yOffsetToApplyBeforeMultiplication, xOffsetToApplyAfterMultiplication, yOffsetToApplyAfterMultiplication, Bridge.ref(this, "_marginOffsets"));

                        this.ApplyMarginToFixNegativeCoordinates(new Windows.Foundation.Point.ctor());
                        if (this.Stretch === Windows.UI.Xaml.Media.Stretch.None) {
                            this.ApplyMarginToFixNegativeCoordinates(this._marginOffsets.$clone());
                        }

                        this._canvasDomElement.getContext('2d').beginPath();

                        var context = CSHTML5.Internal.INTERNAL_HtmlDomManager.Get2dCanvasContext(this._canvasDomElement);

                        context.save();

                        CSHTML5.Internal.INTERNAL_ShapesDrawHelpers.ApplyTransformToCanvas(actualTransform.v, this._canvasDomElement);

                        this.Data.DefineInCanvas(this, this._canvasDomElement, horizontalMultiplicator.v * int32FactorX.v, verticalMultiplicator.v * int32FactorY.v, xOffsetToApplyBeforeMultiplication.v, yOffsetToApplyBeforeMultiplication.v, xOffsetToApplyAfterMultiplication.v, yOffsetToApplyAfterMultiplication.v, shapeActualSize.v.$clone());

                        context.restore();

                        Windows.UI.Xaml.Shapes.Shape.DrawFillAndStroke(this, this.Data.GetFillRuleAsString(), minX.v, minY.v, maxX.v, maxY.v, horizontalMultiplicator.v, verticalMultiplicator.v, xOffsetToApplyBeforeMultiplication.v, yOffsetToApplyBeforeMultiplication.v, shapeActualSize.v.$clone());
                    }
                }
            },
            RefreshOverride: function () {
                if (this.Data != null) {
                    this.Data.SetParentPath(this);
                }
                Windows.UI.Xaml.Shapes.Shape.prototype.RefreshOverride.call(this);
            }
        }
    });

    /**
     * Draws a rectangle shape, which can have a stroke and a fill.
     *
     * @public
     * @class Windows.UI.Xaml.Shapes.Rectangle
     * @augments Windows.UI.Xaml.Shapes.Shape
     * @example
     *
     *
     * Rectangle rect = new Rectangle() { Width = 60, Height = 30, Fill = new SolidColorBrush(Windows.UI.Colors.Blue) };
     * MyStackPanel.Children.Add(rect);
     * 
     *
     *
     */
    Bridge.define("Windows.UI.Xaml.Shapes.Rectangle", {
        inherits: [Windows.UI.Xaml.Shapes.Shape],
        statics: {
            ctors: {
                ctor: function () {
                    var $t;
                    Windows.UI.Xaml.Shapes.Shape.StretchProperty.OverrideMetadata(Windows.UI.Xaml.Shapes.Rectangle, ($t = new Windows.UI.Xaml.PropertyMetadata.$ctor2(Bridge.box(Windows.UI.Xaml.Media.Stretch.Fill, Windows.UI.Xaml.Media.Stretch, System.Enum.toStringFn(Windows.UI.Xaml.Media.Stretch)), Windows.UI.Xaml.Shapes.Shape.Stretch_Changed), $t.CallPropertyChangedWhenLoadedIntoVisualTree = Windows.UI.Xaml.WhenToCallPropertyChangedEnum.IfPropertyIsSet, $t));
                }
            }
        },
        methods: {
            /**
             * // Initializes a new instance of the Rectangle class.
            //
             *
             * @instance
             * @public
             * @override
             * @this Windows.UI.Xaml.Shapes.Rectangle
             * @memberof Windows.UI.Xaml.Shapes.Rectangle
             * @param   {System.Object}    parentRef                         
             * @param   {System.Object}    domElementWhereToPlaceChildren
             * @return  {System.Object}
             */
            CreateDomElement: function (parentRef, domElementWhereToPlaceChildren) {
                return CSHTML5.Internal.INTERNAL_ShapesDrawHelpers.CreateDomElementForPathAndSimilar(this, parentRef, Bridge.ref(this, "_canvasDomElement"), domElementWhereToPlaceChildren);

            },
            Redraw: function () {
                if (CSHTML5.Internal.INTERNAL_VisualTreeManager.IsElementInVisualTree(this)) {
                    var xOffsetToApplyBeforeMultiplication = { };
                    var yOffsetToApplyBeforeMultiplication = { };
                    var xOffsetToApplyAfterMultiplication = { };
                    var yOffsetToApplyAfterMultiplication = { };
                    var sizeX = { };
                    var sizeY = { };
                    var horizontalMultiplicator = { };
                    var verticalMultiplicator = { };
                    var shapeActualSize = { v : new Windows.Foundation.Size() };
                    Windows.UI.Xaml.Shapes.Shape.GetShapeInfos(this, xOffsetToApplyBeforeMultiplication, yOffsetToApplyBeforeMultiplication, xOffsetToApplyAfterMultiplication, yOffsetToApplyAfterMultiplication, sizeX, sizeY, horizontalMultiplicator, verticalMultiplicator, shapeActualSize);

                    this.ApplyMarginToFixNegativeCoordinates(new Windows.Foundation.Point.ctor());
                    if (this.Stretch === Windows.UI.Xaml.Media.Stretch.None) {
                        this.ApplyMarginToFixNegativeCoordinates(this._marginOffsets.$clone());
                    }

                    var context = CSHTML5.Internal.INTERNAL_HtmlDomManager.Get2dCanvasContext(this._canvasDomElement);
                    context.rect(xOffsetToApplyBeforeMultiplication.v + xOffsetToApplyAfterMultiplication.v, yOffsetToApplyBeforeMultiplication.v + yOffsetToApplyAfterMultiplication.v, sizeX.v, sizeY.v);

                    Windows.UI.Xaml.Shapes.Shape.DrawFillAndStroke(this, "evenodd", xOffsetToApplyAfterMultiplication.v, yOffsetToApplyAfterMultiplication.v, xOffsetToApplyAfterMultiplication.v + sizeX.v, yOffsetToApplyAfterMultiplication.v + sizeY.v, horizontalMultiplicator.v, verticalMultiplicator.v, xOffsetToApplyBeforeMultiplication.v, yOffsetToApplyBeforeMultiplication.v, shapeActualSize.v.$clone());

                }
            }
        }
    });

    /**
     * Represents a control that allows a user to select an item from a collection
     of items.
     *
     * @public
     * @class Windows.UI.Xaml.Controls.Primitives.Selector
     * @augments Windows.UI.Xaml.Controls.ItemsControl
     */
    Bridge.define("Windows.UI.Xaml.Controls.Primitives.Selector", {
        inherits: [Windows.UI.Xaml.Controls.ItemsControl],
        statics: {
            fields: {
                /**
                 * Identifies the SelectedItem dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Controls.Primitives.Selector
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                SelectedItemProperty: null,
                /**
                 * Identifies the SelectedIndex dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Controls.Primitives.Selector
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                SelectedIndexProperty: null,
                /**
                 * Identifies the SelectedValue dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Controls.Primitives.Selector
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                SelectedValueProperty: null,
                /**
                 * Identifies the SelectedValuePath dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Controls.Primitives.Selector
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                SelectedValuePathProperty: null,
                /**
                 * Identifies the SelectedItemBackground dependency property
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Controls.Primitives.Selector
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                SelectedItemBackgroundProperty: null,
                /**
                 * Identifies the SelectedItemForeground dependency property
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Controls.Primitives.Selector
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                SelectedItemForegroundProperty: null,
                /**
                 * Identifies the RowBackground dependency property
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Controls.Primitives.Selector
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                RowBackgroundProperty: null,
                /**
                 * Identifies the UnselectedItemForeground dependency property
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Controls.Primitives.Selector
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                UnselectedItemForegroundProperty: null
            },
            ctors: {
                init: function () {
                    this.SelectedItemProperty = Windows.UI.Xaml.DependencyProperty.Register("SelectedItem", System.Object, Windows.UI.Xaml.Controls.Primitives.Selector, new Windows.UI.Xaml.PropertyMetadata.$ctor3(null, Windows.UI.Xaml.Controls.Primitives.Selector.OnSelectedItemChanged, Windows.UI.Xaml.Controls.Primitives.Selector.CoerceSelectedItem));
                    this.SelectedIndexProperty = Windows.UI.Xaml.DependencyProperty.Register("SelectedIndex", System.Int32, Windows.UI.Xaml.Controls.Primitives.Selector, new Windows.UI.Xaml.PropertyMetadata.$ctor3(Bridge.box(-1, System.Int32), Windows.UI.Xaml.Controls.Primitives.Selector.OnSelectedIndexChanged, Windows.UI.Xaml.Controls.Primitives.Selector.CoerceSelectedIndex));
                    this.SelectedValueProperty = Windows.UI.Xaml.DependencyProperty.Register("SelectedValue", System.Object, Windows.UI.Xaml.Controls.Primitives.Selector, new Windows.UI.Xaml.PropertyMetadata.$ctor3(null, Windows.UI.Xaml.Controls.Primitives.Selector.OnSelectedValueChanged, Windows.UI.Xaml.Controls.Primitives.Selector.CoerceSelectedValue));
                    this.SelectedValuePathProperty = Windows.UI.Xaml.DependencyProperty.Register("SelectedValuePath", System.String, Windows.UI.Xaml.Controls.Primitives.Selector, new Windows.UI.Xaml.PropertyMetadata.$ctor2("", Windows.UI.Xaml.Controls.Primitives.Selector.OnSelectedValuePathChanged));
                    this.SelectedItemBackgroundProperty = Windows.UI.Xaml.DependencyProperty.Register("SelectedItemBackground", Windows.UI.Xaml.Media.Brush, Windows.UI.Xaml.Controls.Primitives.Selector, new Windows.UI.Xaml.PropertyMetadata.$ctor1(new Windows.UI.Xaml.Media.SolidColorBrush.$ctor1(Windows.UI.Colors.Blue.$clone())));
                    this.SelectedItemForegroundProperty = Windows.UI.Xaml.DependencyProperty.Register("SelectedItemForeground", Windows.UI.Xaml.Media.Brush, Windows.UI.Xaml.Controls.Primitives.Selector, new Windows.UI.Xaml.PropertyMetadata.$ctor1(new Windows.UI.Xaml.Media.SolidColorBrush.$ctor1(Windows.UI.Colors.White.$clone())));
                    this.RowBackgroundProperty = Windows.UI.Xaml.DependencyProperty.Register("RowBackground", Windows.UI.Xaml.Media.Brush, Windows.UI.Xaml.Controls.Primitives.Selector, new Windows.UI.Xaml.PropertyMetadata.$ctor1(new Windows.UI.Xaml.Media.SolidColorBrush.$ctor1(Windows.UI.Colors.White.$clone())));
                    this.UnselectedItemForegroundProperty = Windows.UI.Xaml.DependencyProperty.Register("UnselectedItemForeground", Windows.UI.Xaml.Media.Brush, Windows.UI.Xaml.Controls.Primitives.Selector, new Windows.UI.Xaml.PropertyMetadata.$ctor1(null));
                }
            },
            methods: {
                OnSelectedItemChanged: function (d, e) {
                    var s = Bridge.cast(d, Windows.UI.Xaml.Controls.Primitives.Selector);
                    var newValue = e.NewValue;

                    if (!s._selectionChangeIsOnValue && !s._selectionChangeIsOnIndex) {
                        s._selectionChangeIsOnItem = true;

                        try {
                            if (newValue == null) {
                                s.SetCurrentValue(Windows.UI.Xaml.Controls.Primitives.Selector.SelectedValueProperty, null);
                                s.SetCurrentValue(Windows.UI.Xaml.Controls.Primitives.Selector.SelectedIndexProperty, Bridge.box(-1, System.Int32));
                            } else {
                                s.SetCurrentValue(Windows.UI.Xaml.Controls.Primitives.Selector.SelectedValueProperty, Windows.UI.Xaml.PropertyPathHelper.AccessValueByApplyingPropertyPathIfAny(newValue, s.SelectedValuePath));
                                s.SetCurrentValue(Windows.UI.Xaml.Controls.Primitives.Selector.SelectedIndexProperty, Bridge.box(s.Items.indexOf(newValue), System.Int32));
                            }
                        } finally {
                            s._selectionChangeIsOnItem = false;
                        }
                    }

                    var removedItems = new (System.Collections.Generic.List$1(System.Object)).ctor();
                    removedItems.add(e.OldValue);
                    var addedItems = new (System.Collections.Generic.List$1(System.Object)).ctor();
                    addedItems.add(e.NewValue);
                    var args = new Windows.UI.Xaml.Controls.SelectionChangedEventArgs(removedItems, addedItems);

                    s.OnSelectionChanged(args);
                },
                CoerceSelectedItem: function (d, baseValue) {
                    var s = Bridge.cast(d, Windows.UI.Xaml.Controls.Primitives.Selector);

                    if (baseValue == null || s.SkipCoerceSelectedItemCheck) {
                        return baseValue;
                    }

                    var selectedIndex = s.SelectedIndex;

                    if ((selectedIndex > -1 && selectedIndex < s.Items.Count && Bridge.referenceEquals(s.Items.getItem(selectedIndex), baseValue)) || s.Items.contains(baseValue)) {
                        return baseValue;
                    }

                    return Windows.UI.Xaml.DependencyProperty.UnsetValue;
                },
                OnSelectedIndexChanged: function (d, e) {
                    var s = Bridge.cast(d, Windows.UI.Xaml.Controls.Primitives.Selector);

                    var newValue = System.Nullable.getValue(Bridge.cast(Bridge.unbox(e.NewValue, System.Int32), System.Int32));
                    if (!s._selectionChangeIsOnValue && !s._selectionChangeIsOnItem) {
                        s._selectionChangeIsOnIndex = true;

                        try {
                            if (newValue === -1) {
                                s.SetCurrentValue(Windows.UI.Xaml.Controls.Primitives.Selector.SelectedValueProperty, null);

                                s.SkipCoerceSelectedItemCheck = true;

                                try {
                                    s.SetCurrentValue(Windows.UI.Xaml.Controls.Primitives.Selector.SelectedItemProperty, null);
                                } finally {
                                    s.SkipCoerceSelectedItemCheck = false;
                                }
                            } else {
                                var item = s.Items.getItem(newValue);

                                s.SetCurrentValue(Windows.UI.Xaml.Controls.Primitives.Selector.SelectedValueProperty, Windows.UI.Xaml.PropertyPathHelper.AccessValueByApplyingPropertyPathIfAny(item, s.SelectedValuePath));

                                s.SkipCoerceSelectedItemCheck = true;

                                try {
                                    s.SetCurrentValue(Windows.UI.Xaml.Controls.Primitives.Selector.SelectedItemProperty, item);
                                } finally {
                                    s.SkipCoerceSelectedItemCheck = false;
                                }
                            }
                        } finally {
                            s._selectionChangeIsOnIndex = false;
                        }
                    }
                    s.ApplySelectedIndex(newValue);
                    s.ManageSelectedIndex_Changed(e);
                },
                CoerceSelectedIndex: function (d, baseValue) {
                    var s = Bridge.cast(d, Windows.UI.Xaml.Controls.Primitives.Selector);

                    var index = System.Nullable.getValue(Bridge.cast(Bridge.unbox(baseValue, System.Int32), System.Int32));
                    if (index < 0) {
                        return Bridge.box(-1, System.Int32);
                    } else if (index >= s.Items.Count) {
                        return Bridge.box(((s.Items.Count - 1) | 0), System.Int32);
                    } else {
                        return Bridge.box(index, System.Int32);
                    }
                },
                OnSelectedValueChanged: function (d, e) {
                    var s = Bridge.cast(d, Windows.UI.Xaml.Controls.Primitives.Selector);

                    if (!s._selectionChangeIsOnItem && !s._selectionChangeIsOnIndex) {
                        s._selectionChangeIsOnValue = true;

                        try {
                            s.SetCurrentValue(Windows.UI.Xaml.Controls.Primitives.Selector.SelectedIndexProperty, Bridge.box(s._selectionInfo.Index, System.Int32));

                            s.SetCurrentValue(Windows.UI.Xaml.Controls.Primitives.Selector.SelectedItemProperty, s._selectionInfo.Item);
                        } finally {
                            s._selectionChangeIsOnValue = false;
                            s._selectionInfo = null;
                        }
                    }
                },
                CoerceSelectedValue: function (d, baseValue) {
                    var s = Bridge.cast(d, Windows.UI.Xaml.Controls.Primitives.Selector);

                    if (s._selectionChangeIsOnIndex || s._selectionChangeIsOnItem) {
                        return baseValue;
                    } else {
                        var index = { };
                        var item = s.FindItemWithValue(baseValue, index);

                        if (Bridge.referenceEquals(item, Windows.UI.Xaml.DependencyProperty.UnsetValue) && s.Items.Count > 0) {
                            baseValue = null;
                        }

                        s._selectionInfo = new Windows.UI.Xaml.Controls.Primitives.Selector.SelectionInfo(Bridge.referenceEquals(item, Windows.UI.Xaml.DependencyProperty.UnsetValue) ? null : item, index.v);
                    }

                    return baseValue;
                },
                OnSelectedValuePathChanged: function (d, e) {
                    var s = Bridge.cast(d, Windows.UI.Xaml.Controls.Primitives.Selector);

                    s._selectionChangeIsOnItem = true;
                    s._selectionChangeIsOnIndex = true;

                    try {
                        var item = null;
                        if (s.SelectedItem != null) {
                            item = Windows.UI.Xaml.PropertyPathHelper.AccessValueByApplyingPropertyPathIfAny(s.SelectedItem, Bridge.cast(e.NewValue, System.String));
                        }
                        s.SetCurrentValue(Windows.UI.Xaml.Controls.Primitives.Selector.SelectedValueProperty, item);
                    } finally {
                        s._selectionChangeIsOnItem = false;
                        s._selectionChangeIsOnIndex = false;
                    }
                }
            }
        },
        fields: {
            _selectionChangeIsOnIndex: false,
            _selectionChangeIsOnItem: false,
            _selectionChangeIsOnValue: false,
            SkipCoerceSelectedItemCheck: false,
            _selectionInfo: null,
            ChangingSelectionProgrammatically: false
        },
        events: {
            /**
             * Occurs when the selection is changed.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Controls.Primitives.Selector
             * @memberof Windows.UI.Xaml.Controls.Primitives.Selector
             * @function addSelectionChanged
             * @param   {Windows.UI.Xaml.Controls.SelectionChangedEventHandler}    value
             * @return  {void}
             */
            /**
             * Occurs when the selection is changed.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Controls.Primitives.Selector
             * @memberof Windows.UI.Xaml.Controls.Primitives.Selector
             * @function removeSelectionChanged
             * @param   {Windows.UI.Xaml.Controls.SelectionChangedEventHandler}    value
             * @return  {void}
             */
            SelectionChanged: null
        },
        props: {
            /**
             * Gets or sets the selected item.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.Primitives.Selector
             * @function SelectedItem
             * @type System.Object
             */
            SelectedItem: {
                get: function () {
                    return this.GetValue(Windows.UI.Xaml.Controls.Primitives.Selector.SelectedItemProperty);
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Controls.Primitives.Selector.SelectedItemProperty, value);
                }
            },
            /**
             * Gets or sets the index of the selected item.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.Primitives.Selector
             * @function SelectedIndex
             * @type number
             */
            SelectedIndex: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.Controls.Primitives.Selector.SelectedIndexProperty), System.Int32), System.Int32));
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Controls.Primitives.Selector.SelectedIndexProperty, Bridge.box(value, System.Int32));
                }
            },
            /**
             * Gets or sets the value of the selected item, obtained by using the SelectedValuePath.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.Primitives.Selector
             * @function SelectedValue
             * @type System.Object
             */
            SelectedValue: {
                get: function () {
                    return this.GetValue(Windows.UI.Xaml.Controls.Primitives.Selector.SelectedValueProperty);
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Controls.Primitives.Selector.SelectedValueProperty, value);
                }
            },
            /**
             * Gets or sets the property path that is used to get the SelectedValue property
             of the SelectedItem property.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.Primitives.Selector
             * @function SelectedValuePath
             * @type string
             */
            SelectedValuePath: {
                get: function () {
                    return Bridge.cast(this.GetValue(Windows.UI.Xaml.Controls.Primitives.Selector.SelectedValuePathProperty), System.String);
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Controls.Primitives.Selector.SelectedValuePathProperty, value);
                }
            },
            /**
             * Gets or sets the bakground color of the selected Items.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.Primitives.Selector
             * @function SelectedItemBackground
             * @type Windows.UI.Xaml.Media.Brush
             */
            SelectedItemBackground: {
                get: function () {
                    return Bridge.cast(this.GetValue(Windows.UI.Xaml.Controls.Primitives.Selector.SelectedItemBackgroundProperty), Windows.UI.Xaml.Media.Brush);
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Controls.Primitives.Selector.SelectedItemBackgroundProperty, value);
                }
            },
            /**
             * Gets or sets the foreground color of the selected Items.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.Primitives.Selector
             * @function SelectedItemForeground
             * @type Windows.UI.Xaml.Media.Brush
             */
            SelectedItemForeground: {
                get: function () {
                    return Bridge.cast(this.GetValue(Windows.UI.Xaml.Controls.Primitives.Selector.SelectedItemForegroundProperty), Windows.UI.Xaml.Media.Brush);
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Controls.Primitives.Selector.SelectedItemForegroundProperty, value);
                }
            },
            /**
             * Gets or sets the bakground color of the Items that are not selected.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.Primitives.Selector
             * @function RowBackground
             * @type Windows.UI.Xaml.Media.Brush
             */
            RowBackground: {
                get: function () {
                    return Bridge.cast(this.GetValue(Windows.UI.Xaml.Controls.Primitives.Selector.RowBackgroundProperty), Windows.UI.Xaml.Media.Brush);
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Controls.Primitives.Selector.RowBackgroundProperty, value);
                }
            },
            /**
             * Gets or sets the foreground color of the Items that are not selected.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.Primitives.Selector
             * @function UnselectedItemForeground
             * @type Windows.UI.Xaml.Media.Brush
             */
            UnselectedItemForeground: {
                get: function () {
                    return Bridge.cast(this.GetValue(Windows.UI.Xaml.Controls.Primitives.Selector.UnselectedItemForegroundProperty), Windows.UI.Xaml.Media.Brush);
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Controls.Primitives.Selector.UnselectedItemForegroundProperty, value);
                }
            },
            /**
             * Gets or sets the bakground color of the Items that are not selected.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.Primitives.Selector
             * @function UnselectedItemBackground
             * @type Windows.UI.Xaml.Media.Brush
             */
            UnselectedItemBackground: {
                get: function () {
                    return this.RowBackground;
                },
                set: function (value) {
                    this.RowBackground = value;
                }
            }
        },
        ctors: {
            init: function () {
                this._selectionChangeIsOnIndex = false;
                this._selectionChangeIsOnItem = false;
                this._selectionChangeIsOnValue = false;
            },
            ctor: function () {
                this.$initialize();
                Windows.UI.Xaml.Controls.ItemsControl.ctor.call(this);

            }
        },
        methods: {
            /**
             * Raises the SelectionChanged event
             *
             * @instance
             * @protected
             * @this Windows.UI.Xaml.Controls.Primitives.Selector
             * @memberof Windows.UI.Xaml.Controls.Primitives.Selector
             * @param   {Windows.UI.Xaml.Controls.SelectionChangedEventArgs}    e    The arguments for the event.
             * @return  {void}
             */
            OnSelectionChanged: function (e) {
                if (!Bridge.staticEquals(this.SelectionChanged, null)) {
                    this.SelectionChanged(this, e);
                }
            },
            OnItemsChanged: function (e) {
                Windows.UI.Xaml.Controls.ItemsControl.prototype.OnItemsChanged.call(this, e);

                var selectedIndex = this.SelectedIndex;

                switch (e.Action) {
                    case System.Collections.Specialized.NotifyCollectionChangedAction.Reset: 
                        if (!this.Items.contains(this.SelectedItem)) {
                            selectedIndex = -1;
                        }
                        break;
                    case System.Collections.Specialized.NotifyCollectionChangedAction.Add: 
                        if (e.NewStartingIndex <= this.SelectedIndex) {
                            selectedIndex = (selectedIndex + 1) | 0;
                        }
                        break;
                    case System.Collections.Specialized.NotifyCollectionChangedAction.Remove: 
                        if (Bridge.referenceEquals(System.Array.getItem(e.OldItems, 0), this.SelectedItem)) {
                            selectedIndex = -1;
                        } else if (e.OldStartingIndex <= this.SelectedIndex) {
                            selectedIndex = (selectedIndex - 1) | 0;
                        }
                        break;
                    case System.Collections.Specialized.NotifyCollectionChangedAction.Replace: 
                        if (Bridge.referenceEquals(System.Array.getItem(e.OldItems, 0), this.SelectedItem)) {
                            selectedIndex = -1;
                        }
                        break;
                    default: 
                        throw new System.NotSupportedException.$ctor1(System.String.format("Unexpected collection change action '{0}'.", [Bridge.box(e.Action, System.Collections.Specialized.NotifyCollectionChangedAction, System.Enum.toStringFn(System.Collections.Specialized.NotifyCollectionChangedAction))]));
                }

                if (selectedIndex !== this.SelectedIndex) {
                    this.SetCurrentValue(Windows.UI.Xaml.Controls.Primitives.Selector.SelectedIndexProperty, Bridge.box(selectedIndex, System.Int32));
                }
            },
            OnItemsSourceChanged_BeforeVisualUpdate: function (oldValue, newValue) {
                this.SetCurrentValue(Windows.UI.Xaml.Controls.Primitives.Selector.SelectedIndexProperty, Bridge.box(-1, System.Int32));
            },
            ApplySelectedIndex: function (index) {

            },
            ManageSelectedIndex_Changed: function (e) {

            },
            FindItemWithValue: function (value, index) {
                var $t;
                index.v = -1;

                if (this.Items.Count === 0) {
                    return Windows.UI.Xaml.DependencyProperty.UnsetValue;
                }

                var selectedValuePath = this.SelectedValuePath;

                if (System.String.isNullOrEmpty(selectedValuePath)) {
                    index.v = this.Items.indexOf(value);
                    if (index.v >= 0) {
                        return value;
                    } else {
                        return Windows.UI.Xaml.DependencyProperty.UnsetValue;
                    }
                }

                index.v = 0;
                $t = Bridge.getEnumerator(this.Items);
                try {
                    while ($t.moveNext()) {
                        var item = $t.Current;
                        var displayedItem = Windows.UI.Xaml.PropertyPathHelper.AccessValueByApplyingPropertyPathIfAny(item, selectedValuePath);
                        if (Windows.UI.Xaml.Controls.ItemsControl.EqualsEx(displayedItem, value)) {
                            return item;
                        }
                        index.v = (index.v + 1) | 0;
                    }
                } finally {
                    if (Bridge.is($t, System.IDisposable)) {
                        $t.System$IDisposable$Dispose();
                    }
                }

                index.v = -1;
                return Windows.UI.Xaml.DependencyProperty.UnsetValue;
            }
        }
    });

    Bridge.define("Windows.UI.Xaml.Controls.BusyIndicator", {
        inherits: [Windows.UI.Xaml.Controls.ContentControl],
        statics: {
            fields: {
                IsBusyProperty: null,
                BusyContentProperty: null,
                BusyContentTemplateProperty: null,
                DisplayAfterProperty: null,
                OverlayStyleProperty: null,
                ProgressBarStyleProperty: null
            },
            ctors: {
                init: function () {
                    this.IsBusyProperty = Windows.UI.Xaml.DependencyProperty.Register("IsBusy", System.Boolean, Windows.UI.Xaml.Controls.BusyIndicator, new Windows.UI.Xaml.PropertyMetadata.$ctor2(Bridge.box(false, System.Boolean, System.Boolean.toString), Windows.UI.Xaml.Controls.BusyIndicator.OnIsBusyChanged));
                    this.BusyContentProperty = Windows.UI.Xaml.DependencyProperty.Register("BusyContent", System.Object, Windows.UI.Xaml.Controls.BusyIndicator, new Windows.UI.Xaml.PropertyMetadata.$ctor4(null));
                    this.BusyContentTemplateProperty = Windows.UI.Xaml.DependencyProperty.Register("BusyContentTemplate", Windows.UI.Xaml.DataTemplate, Windows.UI.Xaml.Controls.BusyIndicator, new Windows.UI.Xaml.PropertyMetadata.$ctor4(null));
                    this.DisplayAfterProperty = Windows.UI.Xaml.DependencyProperty.Register("DisplayAfter", System.TimeSpan, Windows.UI.Xaml.Controls.BusyIndicator, new Windows.UI.Xaml.PropertyMetadata.$ctor1(System.TimeSpan.fromSeconds(0.1)));
                    this.OverlayStyleProperty = Windows.UI.Xaml.DependencyProperty.Register("OverlayStyle", Windows.UI.Xaml.Style, Windows.UI.Xaml.Controls.BusyIndicator, new Windows.UI.Xaml.PropertyMetadata.$ctor4(null));
                    this.ProgressBarStyleProperty = Windows.UI.Xaml.DependencyProperty.Register("ProgressBarStyle", Windows.UI.Xaml.Style, Windows.UI.Xaml.Controls.BusyIndicator, new Windows.UI.Xaml.PropertyMetadata.$ctor4(null));
                }
            },
            methods: {
                OnIsBusyChanged: function (d, e) {
                    Bridge.cast(d, Windows.UI.Xaml.Controls.BusyIndicator).OnIsBusyChanged(e);
                }
            }
        },
        fields: {
            /**
             * Timer used to delay the initial display and avoid flickering.
             *
             * @instance
             * @private
             * @memberof Windows.UI.Xaml.Controls.BusyIndicator
             * @type Windows.UI.Xaml.DispatcherTimer
             */
            _displayAfterTimer: null,
            /**
             * Gets or sets a value indicating whether the BusyContent is visible.
             *
             * @instance
             * @protected
             * @memberof Windows.UI.Xaml.Controls.BusyIndicator
             * @function IsContentVisible
             * @type boolean
             */
            IsContentVisible: false
        },
        props: {
            IsBusy: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.Controls.BusyIndicator.IsBusyProperty), System.Boolean), System.Boolean));
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Controls.BusyIndicator.IsBusyProperty, Bridge.box(value, System.Boolean, System.Boolean.toString));
                }
            },
            BusyContent: {
                get: function () {
                    return this.GetValue(Windows.UI.Xaml.Controls.BusyIndicator.BusyContentProperty);
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Controls.BusyIndicator.BusyContentProperty, value);
                }
            },
            BusyContentTemplate: {
                get: function () {
                    return Bridge.cast(this.GetValue(Windows.UI.Xaml.Controls.BusyIndicator.BusyContentTemplateProperty), Windows.UI.Xaml.DataTemplate);
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Controls.BusyIndicator.BusyContentTemplateProperty, value);
                }
            },
            DisplayAfter: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.Controls.BusyIndicator.DisplayAfterProperty), System.TimeSpan), System.TimeSpan));
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Controls.BusyIndicator.DisplayAfterProperty, value);
                }
            },
            OverlayStyle: {
                get: function () {
                    return Bridge.cast(this.GetValue(Windows.UI.Xaml.Controls.BusyIndicator.OverlayStyleProperty), Windows.UI.Xaml.Style);
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Controls.BusyIndicator.OverlayStyleProperty, value);
                }
            },
            ProgressBarStyle: {
                get: function () {
                    return Bridge.cast(this.GetValue(Windows.UI.Xaml.Controls.BusyIndicator.ProgressBarStyleProperty), Windows.UI.Xaml.Style);
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Controls.BusyIndicator.ProgressBarStyleProperty, value);
                }
            }
        },
        ctors: {
            ctor: function () {
                this.$initialize();
                Windows.UI.Xaml.Controls.ContentControl.ctor.call(this);
                this.DefaultStyleKey = Windows.UI.Xaml.Controls.BusyIndicator;
                this._displayAfterTimer = new Windows.UI.Xaml.DispatcherTimer();
                this.addLoaded(Bridge.fn.bind(this, function () {
                    this._displayAfterTimer.addTick(Bridge.fn.cacheBind(this, this.DisplayAfterTimerElapsed));
                }));
                this.addUnloaded(Bridge.fn.bind(this, function () {
                    this._displayAfterTimer.addTick(Bridge.fn.cacheBind(this, this.DisplayAfterTimerElapsed));
                    this._displayAfterTimer.Stop();
                }));
            }
        },
        methods: {
            /**
             * Overrides the OnApplyTemplate method.
             *
             * @instance
             * @protected
             * @override
             * @this Windows.UI.Xaml.Controls.BusyIndicator
             * @memberof Windows.UI.Xaml.Controls.BusyIndicator
             * @return  {void}
             */
            OnApplyTemplate: function () {
                Windows.UI.Xaml.Controls.ContentControl.prototype.OnApplyTemplate.call(this);
                this.ChangeVisualState(false);
            },
            /**
             * Changes the control's visual state(s).
             *
             * @instance
             * @protected
             * @this Windows.UI.Xaml.Controls.BusyIndicator
             * @memberof Windows.UI.Xaml.Controls.BusyIndicator
             * @param   {boolean}    useTransitions    True if state transitions should be used.
             * @return  {void}
             */
            ChangeVisualState: function (useTransitions) {
                Windows.UI.Xaml.VisualStateManager.GoToState(this, this.IsBusy ? "Busy" : "Idle", useTransitions);
                Windows.UI.Xaml.VisualStateManager.GoToState(this, this.IsContentVisible ? "Visible" : "Hidden", useTransitions);
            },
            /**
             * IsBusyProperty property changed handler.
             *
             * @instance
             * @protected
             * @this Windows.UI.Xaml.Controls.BusyIndicator
             * @memberof Windows.UI.Xaml.Controls.BusyIndicator
             * @param   {Windows.UI.Xaml.DependencyPropertyChangedEventArgs}    e    Event arguments.
             * @return  {void}
             */
            OnIsBusyChanged: function (e) {
                if (this.IsBusy) {
                    if (this.DisplayAfter.equalsT(System.TimeSpan.zero)) {
                        this.IsContentVisible = true;
                    } else {
                        this._displayAfterTimer.Interval = this.DisplayAfter;
                        this._displayAfterTimer.Start();
                    }
                } else {
                    this._displayAfterTimer.Stop();
                    this.IsContentVisible = false;
                }
                this.ChangeVisualState(true);
            },
            /**
             * Handler for the DisplayAfterTimer.
             *
             * @instance
             * @private
             * @this Windows.UI.Xaml.Controls.BusyIndicator
             * @memberof Windows.UI.Xaml.Controls.BusyIndicator
             * @param   {System.Object}    sender    Event sender.
             * @param   {System.Object}    e         Event arguments.
             * @return  {void}
             */
            DisplayAfterTimerElapsed: function (sender, e) {
                this._displayAfterTimer.Stop();
                this.IsContentVisible = true;
                this.ChangeVisualState(true);
            }
        }
    });

    /**
     * Represents the base class for all button controls, such as Button, RepeatButton,
     and HyperlinkButton.
     *
     * @public
     * @class Windows.UI.Xaml.Controls.Primitives.ButtonBase
     * @augments Windows.UI.Xaml.Controls.ContentControl
     */
    Bridge.define("Windows.UI.Xaml.Controls.Primitives.ButtonBase", {
        inherits: [Windows.UI.Xaml.Controls.ContentControl],
        statics: {
            fields: {
                /**
                 * Identifies the CommandParameter dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Controls.Primitives.ButtonBase
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                CommandParameterProperty: null,
                /**
                 * Identifies the Command dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Controls.Primitives.ButtonBase
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                CommandProperty: null,
                /**
                 * Identifies the IsPressed dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Controls.Primitives.ButtonBase
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                IsPressedProperty: null,
                /**
                 * Identifies the ClickMode dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Controls.Primitives.ButtonBase
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                ClickModeProperty: null
            },
            ctors: {
                init: function () {
                    var $t;
                    this.CommandParameterProperty = Windows.UI.Xaml.DependencyProperty.Register("CommandParameter", System.Object, Windows.UI.Xaml.Controls.Primitives.ButtonBase, new Windows.UI.Xaml.PropertyMetadata.$ctor4(null));
                    this.CommandProperty = Windows.UI.Xaml.DependencyProperty.Register("Command", System.Windows.Input.ICommand, Windows.UI.Xaml.Controls.Primitives.ButtonBase, ($t = new Windows.UI.Xaml.PropertyMetadata.$ctor2(null, Windows.UI.Xaml.Controls.Primitives.ButtonBase.Command_Changed), $t.CallPropertyChangedWhenLoadedIntoVisualTree = Windows.UI.Xaml.WhenToCallPropertyChangedEnum.IfPropertyIsSet, $t));
                    this.IsPressedProperty = Windows.UI.Xaml.DependencyProperty.Register("IsPressed", System.Boolean, Windows.UI.Xaml.Controls.Primitives.ButtonBase, new Windows.UI.Xaml.PropertyMetadata.$ctor1(Bridge.box(false, System.Boolean, System.Boolean.toString)));
                    this.ClickModeProperty = Windows.UI.Xaml.DependencyProperty.Register("ClickMode", Windows.UI.Xaml.Controls.ClickMode, Windows.UI.Xaml.Controls.Primitives.ButtonBase, new Windows.UI.Xaml.PropertyMetadata.$ctor1(Bridge.box(Windows.UI.Xaml.Controls.ClickMode.Release, Windows.UI.Xaml.Controls.ClickMode, System.Enum.toStringFn(Windows.UI.Xaml.Controls.ClickMode))));
                }
            },
            methods: {
                Command_Changed: function (d, e) {
                    var buttonBase = Bridge.cast(d, Windows.UI.Xaml.Controls.Primitives.ButtonBase);
                    if (e.OldValue != null && e.NewValue == null) {
                        buttonBase.removeClick(Windows.UI.Xaml.Controls.Primitives.ButtonBase.ExecuteCommand);
                    }
                    if (e.OldValue == null && e.NewValue != null) {
                        buttonBase.addClick(Windows.UI.Xaml.Controls.Primitives.ButtonBase.ExecuteCommand);
                    }
                },
                ExecuteCommand: function (sender, e) {
                    var buttonBase = Bridge.cast(sender, Windows.UI.Xaml.Controls.Primitives.ButtonBase);
                    if (buttonBase.Command.System$Windows$Input$ICommand$CanExecute(buttonBase.CommandParameter)) {
                        buttonBase.Command.System$Windows$Input$ICommand$Execute(buttonBase.CommandParameter);
                    }
                }
            }
        },
        fields: {
            _timerToReleaseCaptureAutomaticallyIfNoMouseUpEvent: null
        },
        events: {
            /**
             * Occurs when a button control is clicked.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Controls.Primitives.ButtonBase
             * @memberof Windows.UI.Xaml.Controls.Primitives.ButtonBase
             * @function addClick
             * @param   {Windows.UI.Xaml.RoutedEventHandler}    value
             * @return  {void}
             */
            /**
             * Occurs when a button control is clicked.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Controls.Primitives.ButtonBase
             * @memberof Windows.UI.Xaml.Controls.Primitives.ButtonBase
             * @function removeClick
             * @param   {Windows.UI.Xaml.RoutedEventHandler}    value
             * @return  {void}
             */
            Click: null
        },
        props: {
            /**
             * Gets or sets the parameter to pass to the {@link } property.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.Primitives.ButtonBase
             * @function CommandParameter
             * @type System.Object
             */
            CommandParameter: {
                get: function () {
                    return this.GetValue(Windows.UI.Xaml.Controls.Primitives.ButtonBase.CommandParameterProperty);
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Controls.Primitives.ButtonBase.CommandParameterProperty, value);
                }
            },
            /**
             * Gets or sets the command to invoke when this button is pressed.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.Primitives.ButtonBase
             * @function Command
             * @type System.Windows.Input.ICommand
             */
            Command: {
                get: function () {
                    return Bridge.cast(this.GetValue(Windows.UI.Xaml.Controls.Primitives.ButtonBase.CommandProperty), System.Windows.Input.ICommand);
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Controls.Primitives.ButtonBase.CommandProperty, value);
                }
            },
            /**
             * Gets a value that indicates whether a ButtonBase is currently in a pressed state.
             The default is false.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.Primitives.ButtonBase
             * @function IsPressed
             * @type boolean
             */
            IsPressed: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.Controls.Primitives.ButtonBase.IsPressedProperty), System.Boolean), System.Boolean));
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Controls.Primitives.ButtonBase.IsPressedProperty, Bridge.box(value, System.Boolean, System.Boolean.toString));
                }
            },
            /**
             * Gets or sets when the Click event occurs. The default value is ClickMode.Release.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.Primitives.ButtonBase
             * @function ClickMode
             * @type Windows.UI.Xaml.Controls.ClickMode
             */
            ClickMode: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.Controls.Primitives.ButtonBase.ClickModeProperty), Windows.UI.Xaml.Controls.ClickMode), System.Int32));
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Controls.Primitives.ButtonBase.ClickModeProperty, Bridge.box(value, Windows.UI.Xaml.Controls.ClickMode, System.Enum.toStringFn(Windows.UI.Xaml.Controls.ClickMode)));
                }
            }
        },
        ctors: {
            init: function () {
                this._timerToReleaseCaptureAutomaticallyIfNoMouseUpEvent = new Windows.UI.Xaml.DispatcherTimer();
            },
            ctor: function () {
                this.$initialize();
                Windows.UI.Xaml.Controls.ContentControl.ctor.call(this);
                this._reactsToKeyboardEventsWhenFocused = true;
                this.UseSystemFocusVisuals = true;

                this._timerToReleaseCaptureAutomaticallyIfNoMouseUpEvent.Interval = new System.TimeSpan(0, 0, 5);
                this._timerToReleaseCaptureAutomaticallyIfNoMouseUpEvent.addTick(Bridge.fn.cacheBind(this, this.TimerToReleaseCaptureAutomaticallyIfNoMouseUpEvent_Tick));

                this.addPointerPressed(function (s, e) { });
                this.addPointerReleased(function (s, e) { });
            }
        },
        methods: {
            /**
             * Raises the Click event
             *
             * @instance
             * @protected
             * @this Windows.UI.Xaml.Controls.Primitives.ButtonBase
             * @memberof Windows.UI.Xaml.Controls.Primitives.ButtonBase
             * @return  {void}
             */
            OnClick: function () {
                var $t;
                if (!Bridge.staticEquals(this.Click, null)) {
                    this.Click(this, ($t = new Windows.UI.Xaml.RoutedEventArgs(), $t.OriginalSource = this, $t));
                }
            },
            OnPointerPressed: function (eventArgs) {
                Windows.UI.Xaml.Controls.ContentControl.prototype.OnPointerPressed.call(this, eventArgs);
                eventArgs.Handled = true;

                this.CapturePointer();
                this._timerToReleaseCaptureAutomaticallyIfNoMouseUpEvent.Stop();
                this._timerToReleaseCaptureAutomaticallyIfNoMouseUpEvent.Start();

                this.IsPressed = true;

                if (this.ClickMode === Windows.UI.Xaml.Controls.ClickMode.Press) {
                    this.OnClick();
                }
            },
            OnPointerReleased: function (eventArgs) {

                if (this.IsPointerCaptured) {

                    Windows.UI.Xaml.Controls.ContentControl.prototype.OnPointerReleased.call(this, eventArgs);
                    eventArgs.Handled = true;

                    this.StopPointerCapture();

                    if (this.ClickMode === Windows.UI.Xaml.Controls.ClickMode.Release) {
                        this.OnClick();
                    }
                }

                this.IsPressed = false;
            },
            StopPointerCapture: function () {
                this._timerToReleaseCaptureAutomaticallyIfNoMouseUpEvent.Stop();
                this.ReleasePointerCapture();
            },
            TimerToReleaseCaptureAutomaticallyIfNoMouseUpEvent_Tick: function (sender, e) {
                this.StopPointerCapture();
            },
            IsPointerOverThisControl: function (e) {
                var actualSize = this.INTERNAL_GetActualWidthAndHeight();
                var actualWidth = actualSize.Width;
                var actualHeight = actualSize.Height;
                if (!isNaN(actualWidth) && !isNaN(actualHeight)) {
                    var position = e.GetCurrentPoint(this).Position.$clone();
                    return (position.X > 0 && position.Y > 0 && position.X < actualWidth && position.Y < actualHeight);
                } else {
                    return false;
                }
            },
            OnKeyDownWhenFocused: function (sender, e) {

                if ((e.Key === Windows.System.VirtualKey.Space) || (e.Key === Windows.System.VirtualKey.Enter)) {
                    this.OnClick();
                }
            }
        }
    });

    /**
     * Provides a window that can be displayed over a parent window and blocks
     interaction with the parent window.
     *
     * @public
     * @class Windows.UI.Xaml.Controls.ChildWindow
     * @augments Windows.UI.Xaml.Controls.ContentControl
     */
    Bridge.define("Windows.UI.Xaml.Controls.ChildWindow", {
        inherits: [Windows.UI.Xaml.Controls.ContentControl],
        statics: {
            fields: {
                /**
                 * The name of the Chrome template part.
                 *
                 * @static
                 * @private
                 * @memberof Windows.UI.Xaml.Controls.ChildWindow
                 * @constant
                 * @default "Chrome"
                 * @type string
                 */
                PART_Chrome: null,
                /**
                 * The name of the CloseButton template part.
                 *
                 * @static
                 * @private
                 * @memberof Windows.UI.Xaml.Controls.ChildWindow
                 * @constant
                 * @default "CloseButton"
                 * @type string
                 */
                PART_CloseButton: null,
                /**
                 * The name of the ContentPresenter template part.
                 *
                 * @static
                 * @private
                 * @memberof Windows.UI.Xaml.Controls.ChildWindow
                 * @constant
                 * @default "ContentPresenter"
                 * @type string
                 */
                PART_ContentPresenter: null,
                /**
                 * The name of the ContentRoot template part.
                 *
                 * @static
                 * @private
                 * @memberof Windows.UI.Xaml.Controls.ChildWindow
                 * @constant
                 * @default "ContentRoot"
                 * @type string
                 */
                PART_ContentRoot: null,
                /**
                 * The name of the Overlay template part.
                 *
                 * @static
                 * @private
                 * @memberof Windows.UI.Xaml.Controls.ChildWindow
                 * @constant
                 * @default "Overlay"
                 * @type string
                 */
                PART_Overlay: null,
                /**
                 * The name of the Root template part.
                 *
                 * @static
                 * @private
                 * @memberof Windows.UI.Xaml.Controls.ChildWindow
                 * @constant
                 * @default "Root"
                 * @type string
                 */
                PART_Root: null,
                /**
                 * The name of the WindowStates VSM group.
                 *
                 * @static
                 * @private
                 * @memberof Windows.UI.Xaml.Controls.ChildWindow
                 * @constant
                 * @default "WindowStates"
                 * @type string
                 */
                VSMGROUP_Window: null,
                /**
                 * The name of the Closing VSM state.
                 *
                 * @static
                 * @private
                 * @memberof Windows.UI.Xaml.Controls.ChildWindow
                 * @constant
                 * @default "Closed"
                 * @type string
                 */
                VSMSTATE_StateClosed: null,
                /**
                 * The name of the Opening VSM state.
                 *
                 * @static
                 * @private
                 * @memberof Windows.UI.Xaml.Controls.ChildWindow
                 * @constant
                 * @default "Open"
                 * @type string
                 */
                VSMSTATE_StateOpen: null,
                /**
                 * Identifies the HasCloseButton
                 dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Controls.ChildWindow
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                HasCloseButtonProperty: null,
                /**
                 * Identifies the OverlayBrush
                 dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Controls.ChildWindow
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                OverlayBrushProperty: null,
                /**
                 * Identifies the OverlayOpacity
                 dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Controls.ChildWindow
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                OverlayOpacityProperty: null,
                /**
                 * Identifies the ChildWindow.Title" /&gt;
                 dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Controls.ChildWindow
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                TitleProperty: null
            },
            props: {
                /**
                 * Gets the root visual element.
                 *
                 * @static
                 * @private
                 * @readonly
                 * @memberof Windows.UI.Xaml.Controls.ChildWindow
                 * @function RootVisual
                 * @type Windows.UI.Xaml.Controls.Control
                 */
                RootVisual: {
                    get: function () {
                        return Windows.UI.Xaml.Application.Current == null ? null : (Bridge.as(Windows.UI.Xaml.Application.Current.RootVisual, Windows.UI.Xaml.Controls.Control));
                    }
                }
            },
            ctors: {
                init: function () {
                    var $t;
                    this.PART_Chrome = "Chrome";
                    this.PART_CloseButton = "CloseButton";
                    this.PART_ContentPresenter = "ContentPresenter";
                    this.PART_ContentRoot = "ContentRoot";
                    this.PART_Overlay = "Overlay";
                    this.PART_Root = "Root";
                    this.VSMGROUP_Window = "WindowStates";
                    this.VSMSTATE_StateClosed = "Closed";
                    this.VSMSTATE_StateOpen = "Open";
                    this.HasCloseButtonProperty = Windows.UI.Xaml.DependencyProperty.Register("HasCloseButton", System.Boolean, Windows.UI.Xaml.Controls.ChildWindow, ($t = new Windows.UI.Xaml.PropertyMetadata.$ctor2(Bridge.box(true, System.Boolean, System.Boolean.toString), Windows.UI.Xaml.Controls.ChildWindow.OnHasCloseButtonPropertyChanged), $t.CallPropertyChangedWhenLoadedIntoVisualTree = Windows.UI.Xaml.WhenToCallPropertyChangedEnum.IfPropertyIsSet, $t));
                    this.OverlayBrushProperty = Windows.UI.Xaml.DependencyProperty.Register("OverlayBrush", Windows.UI.Xaml.Media.Brush, Windows.UI.Xaml.Controls.ChildWindow, ($t = new Windows.UI.Xaml.PropertyMetadata.$ctor4(Windows.UI.Xaml.Controls.ChildWindow.OnOverlayBrushPropertyChanged), $t.CallPropertyChangedWhenLoadedIntoVisualTree = Windows.UI.Xaml.WhenToCallPropertyChangedEnum.IfPropertyIsSet, $t));
                    this.OverlayOpacityProperty = Windows.UI.Xaml.DependencyProperty.Register("OverlayOpacity", System.Double, Windows.UI.Xaml.Controls.ChildWindow, ($t = new Windows.UI.Xaml.PropertyMetadata.$ctor4(Windows.UI.Xaml.Controls.ChildWindow.OnOverlayOpacityPropertyChanged), $t.CallPropertyChangedWhenLoadedIntoVisualTree = Windows.UI.Xaml.WhenToCallPropertyChangedEnum.IfPropertyIsSet, $t));
                    this.TitleProperty = Windows.UI.Xaml.DependencyProperty.Register("Title", System.Object, Windows.UI.Xaml.Controls.ChildWindow, null);
                }
            },
            methods: {
                /**
                 * HasCloseButtonProperty PropertyChangedCallback call back static function.
                 *
                 * @static
                 * @private
                 * @this Windows.UI.Xaml.Controls.ChildWindow
                 * @memberof Windows.UI.Xaml.Controls.ChildWindow
                 * @param   {Windows.UI.Xaml.DependencyObject}                      d    ChildWindow object whose HasCloseButton property is changed.
                 * @param   {Windows.UI.Xaml.DependencyPropertyChangedEventArgs}    e    DependencyPropertyChangedEventArgs which contains the old and new values.
                 * @return  {void}
                 */
                OnHasCloseButtonPropertyChanged: function (d, e) {
                    var cw = Bridge.cast(d, Windows.UI.Xaml.Controls.ChildWindow);

                    if (cw.CloseButton != null) {
                        if (System.Nullable.getValue(Bridge.cast(Bridge.unbox(e.NewValue, System.Boolean), System.Boolean))) {
                            cw.CloseButton.Visibility = Windows.UI.Xaml.Visibility.Visible;
                        } else {
                            cw.CloseButton.Visibility = Windows.UI.Xaml.Visibility.Collapsed;
                        }
                    }
                },
                /**
                 * OverlayBrushProperty PropertyChangedCallback call back static function.
                 *
                 * @static
                 * @private
                 * @this Windows.UI.Xaml.Controls.ChildWindow
                 * @memberof Windows.UI.Xaml.Controls.ChildWindow
                 * @param   {Windows.UI.Xaml.DependencyObject}                      d    ChildWindow object whose OverlayBrush property is changed.
                 * @param   {Windows.UI.Xaml.DependencyPropertyChangedEventArgs}    e    DependencyPropertyChangedEventArgs which contains the old and new values.
                 * @return  {void}
                 */
                OnOverlayBrushPropertyChanged: function (d, e) {
                    var cw = Bridge.cast(d, Windows.UI.Xaml.Controls.ChildWindow);

                    if (cw.Overlay != null) {
                        cw.Overlay.Background = Bridge.cast(e.NewValue, Windows.UI.Xaml.Media.Brush);
                    }
                },
                /**
                 * OverlayOpacityProperty PropertyChangedCallback call back static function.
                 *
                 * @static
                 * @private
                 * @this Windows.UI.Xaml.Controls.ChildWindow
                 * @memberof Windows.UI.Xaml.Controls.ChildWindow
                 * @param   {Windows.UI.Xaml.DependencyObject}                      d    ChildWindow object whose OverlayOpacity property is changed.
                 * @param   {Windows.UI.Xaml.DependencyPropertyChangedEventArgs}    e    DependencyPropertyChangedEventArgs which contains the old and new values.
                 * @return  {void}
                 */
                OnOverlayOpacityPropertyChanged: function (d, e) {
                    var cw = Bridge.cast(d, Windows.UI.Xaml.Controls.ChildWindow);

                    if (cw.Overlay != null) {
                        cw.Overlay.Opacity = System.Nullable.getValue(Bridge.cast(Bridge.unbox(e.NewValue, System.Double), System.Double));
                    }
                }
            }
        },
        fields: {
            /**
             * Private accessor for the Chrome.
             *
             * @instance
             * @private
             * @memberof Windows.UI.Xaml.Controls.ChildWindow
             * @type Windows.UI.Xaml.FrameworkElement
             */
            _chrome: null,
            /**
             * Private accessor for the click point on the chrome.
             *
             * @instance
             * @private
             * @memberof Windows.UI.Xaml.Controls.ChildWindow
             * @type Windows.Foundation.Point
             */
            _clickPoint: null,
            /**
             * Private accessor for the Closing storyboard.
             *
             * @instance
             * @private
             * @memberof Windows.UI.Xaml.Controls.ChildWindow
             * @type Windows.UI.Xaml.Media.Animation.Storyboard
             */
            _closed: null,
            /**
             * Private accessor for the ContentPresenter.
             *
             * @instance
             * @private
             * @memberof Windows.UI.Xaml.Controls.ChildWindow
             * @type Windows.UI.Xaml.FrameworkElement
             */
            _contentPresenter: null,
            /**
             * Private accessor for the translate transform that needs to be applied on to the ContentRoot.
             *
             * @instance
             * @private
             * @memberof Windows.UI.Xaml.Controls.ChildWindow
             * @type Windows.UI.Xaml.Media.TranslateTransform
             */
            _contentRootTransform: null,
            /**
             * Content area desired width.
             *
             * @instance
             * @private
             * @memberof Windows.UI.Xaml.Controls.ChildWindow
             * @type number
             */
            _desiredContentWidth: 0,
            /**
             * Content area desired height.
             *
             * @instance
             * @private
             * @memberof Windows.UI.Xaml.Controls.ChildWindow
             * @type number
             */
            _desiredContentHeight: 0,
            /**
             * The width that was specified when the ChildWindow was created.
             *
             * @instance
             * @private
             * @memberof Windows.UI.Xaml.Controls.ChildWindow
             * @type number
             */
            _widthThatWasInitiallySpecified: 0,
            /**
             * The height that was specified when the ChildWindow was created.
             *
             * @instance
             * @private
             * @memberof Windows.UI.Xaml.Controls.ChildWindow
             * @type number
             */
            _heightThatWasInitiallySpecified: 0,
            /**
             * Desired margin for the window.
             *
             * @instance
             * @private
             * @memberof Windows.UI.Xaml.Controls.ChildWindow
             * @type Windows.UI.Xaml.Thickness
             */
            _desiredMargin: null,
            /**
             * Private accessor for the Dialog Result property.
             *
             * @instance
             * @private
             * @memberof Windows.UI.Xaml.Controls.ChildWindow
             * @type ?boolean
             */
            _dialogresult: null,
            /**
             * Boolean value that specifies whether the application is exit or not.
             *
             * @instance
             * @private
             * @memberof Windows.UI.Xaml.Controls.ChildWindow
             * @type boolean
             */
            _isAppExit: false,
            /**
             * Boolean value that specifies whether the window is in closing state or not.
             *
             * @instance
             * @private
             * @memberof Windows.UI.Xaml.Controls.ChildWindow
             * @type boolean
             */
            _isClosing: false,
            /**
             * Boolean value that specifies whether the window is opened.
             *
             * @instance
             * @private
             * @memberof Windows.UI.Xaml.Controls.ChildWindow
             * @type boolean
             */
            _isOpen: false,
            /**
             * Private accessor for the Opening storyboard.
             *
             * @instance
             * @private
             * @memberof Windows.UI.Xaml.Controls.ChildWindow
             * @type Windows.UI.Xaml.Media.Animation.Storyboard
             */
            _opened: null,
            /**
             * Boolean value that specifies whether the mouse is captured or not.
             *
             * @instance
             * @private
             * @memberof Windows.UI.Xaml.Controls.ChildWindow
             * @type boolean
             */
            _isMouseCaptured: false,
            /**
             * Private accessor for the Root of the window.
             *
             * @instance
             * @private
             * @memberof Windows.UI.Xaml.Controls.ChildWindow
             * @type Windows.UI.Xaml.FrameworkElement
             */
            _root: null,
            /**
             * Private accessor for the position of the window with respect to RootVisual.
             *
             * @instance
             * @private
             * @memberof Windows.UI.Xaml.Controls.ChildWindow
             * @type Windows.Foundation.Point
             */
            _windowPosition: null,
            /**
             * Gets the internal accessor for the ContentRoot of the window.
             *
             * @instance
             * @memberof Windows.UI.Xaml.Controls.ChildWindow
             * @function ContentRoot
             * @type Windows.UI.Xaml.FrameworkElement
             */
            ContentRoot: null,
            /**
             * Gets the internal accessor for the PopUp of the window.
             *
             * @instance
             * @memberof Windows.UI.Xaml.Controls.ChildWindow
             * @function ChildWindowPopup
             * @type Windows.UI.Xaml.Controls.Primitives.Popup
             */
            ChildWindowPopup: null,
            /**
             * Gets the internal accessor for the close button of the window.
             *
             * @instance
             * @memberof Windows.UI.Xaml.Controls.ChildWindow
             * @function CloseButton
             * @type Windows.UI.Xaml.Controls.Primitives.ButtonBase
             */
            CloseButton: null,
            /**
             * Gets the internal accessor for the overlay of the window.
             *
             * @instance
             * @memberof Windows.UI.Xaml.Controls.ChildWindow
             * @function Overlay
             * @type Windows.UI.Xaml.Controls.Panel
             */
            Overlay: null
        },
        events: {
            /**
             * Occurs when the ChildWindow
             is closed.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Controls.ChildWindow
             * @memberof Windows.UI.Xaml.Controls.ChildWindow
             * @function addClosed
             * @param   {System.EventHandler}    value
             * @return  {void}
             */
            /**
             * Occurs when the ChildWindow
             is closed.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Controls.ChildWindow
             * @memberof Windows.UI.Xaml.Controls.ChildWindow
             * @function removeClosed
             * @param   {System.EventHandler}    value
             * @return  {void}
             */
            Closed: null,
            /**
             * Occurs when the ChildWindow
             is closing.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Controls.ChildWindow
             * @memberof Windows.UI.Xaml.Controls.ChildWindow
             * @function addClosing
             * @param   {System.EventHandler}    value
             * @return  {void}
             */
            /**
             * Occurs when the ChildWindow
             is closing.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Controls.ChildWindow
             * @memberof Windows.UI.Xaml.Controls.ChildWindow
             * @function removeClosing
             * @param   {System.EventHandler}    value
             * @return  {void}
             */
            Closing: null
        },
        props: {
            /**
             * Gets or sets a value indicating whether the
             ChildWindow has a close
             button.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.ChildWindow
             * @function HasCloseButton
             * @type boolean
             */
            HasCloseButton: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.Controls.ChildWindow.HasCloseButtonProperty), System.Boolean), System.Boolean));
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Controls.ChildWindow.HasCloseButtonProperty, Bridge.box(value, System.Boolean, System.Boolean.toString));
                }
            },
            /**
             * Gets or sets the visual brush that is used to cover the parent
             window when the child window is open.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.ChildWindow
             * @function OverlayBrush
             * @type Windows.UI.Xaml.Media.Brush
             */
            OverlayBrush: {
                get: function () {
                    return Bridge.cast(this.GetValue(Windows.UI.Xaml.Controls.ChildWindow.OverlayBrushProperty), Windows.UI.Xaml.Media.Brush);
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Controls.ChildWindow.OverlayBrushProperty, value);
                }
            },
            /**
             * Gets or sets the opacity of the overlay brush that is used to cover
             the parent window when the child window is open.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.ChildWindow
             * @function OverlayOpacity
             * @type number
             */
            OverlayOpacity: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.Controls.ChildWindow.OverlayOpacityProperty), System.Double), System.Double));
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Controls.ChildWindow.OverlayOpacityProperty, Bridge.box(value, System.Double, System.Double.format, System.Double.getHashCode));
                }
            },
            /**
             * Gets or sets the title that is displayed in the frame of the
             ChildWindow.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.ChildWindow
             * @function Title
             * @type System.Object
             */
            Title: {
                get: function () {
                    return this.GetValue(Windows.UI.Xaml.Controls.ChildWindow.TitleProperty);
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Controls.ChildWindow.TitleProperty, value);
                }
            },
            /**
             * Gets or sets a value indicating whether the
             ChildWindow was accepted or
             canceled.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.ChildWindow
             * @function DialogResult
             * @type ?boolean
             */
            DialogResult: {
                get: function () {
                    return this._dialogresult;
                },
                set: function (value) {
                    if (System.Nullable.neq(this._dialogresult, value)) {
                        this._dialogresult = value;
                        this.Close();
                    }
                }
            },
            /**
             * Gets a value indicating whether the PopUp is open or not.
             *
             * @instance
             * @private
             * @readonly
             * @memberof Windows.UI.Xaml.Controls.ChildWindow
             * @function IsOpen
             * @type boolean
             */
            IsOpen: {
                get: function () {
                    return (this.ChildWindowPopup != null && this.ChildWindowPopup.IsOpen);
                }
            }
        },
        ctors: {
            init: function () {
                this._clickPoint = new Windows.Foundation.Point();
                this._desiredMargin = new Windows.UI.Xaml.Thickness();
                this._windowPosition = new Windows.Foundation.Point();
            },
            /**
             * Initializes a new instance of the
             ChildWindow class.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Controls.ChildWindow
             * @memberof Windows.UI.Xaml.Controls.ChildWindow
             * @return  {void}
             */
            ctor: function () {
                this.$initialize();
                Windows.UI.Xaml.Controls.ContentControl.ctor.call(this);
                this.DefaultStyleKey = Windows.UI.Xaml.Controls.ChildWindow;
            }
        },
        methods: {
            /**
             * Executed when mouse moves on the chrome.
             *
             * @instance
             * @private
             * @this Windows.UI.Xaml.Controls.ChildWindow
             * @memberof Windows.UI.Xaml.Controls.ChildWindow
             * @param   {System.Object}                                   sender    Sender object.
             * @param   {Windows.UI.Xaml.Input.PointerRoutedEventArgs}    e         Mouse event args.
             * @return  {void}
             */
            Chrome_MouseMove: function (sender, e) {
                if (this._isMouseCaptured) {
                    var posNow = new Windows.Foundation.Point.$ctor1(e._pointerAbsoluteX, e._pointerAbsoluteY);

                    var delta = new Windows.Foundation.Point.$ctor1(posNow.X - this._clickPoint.X, posNow.Y - this._clickPoint.Y);

                    if (this._contentRootTransform == null) {
                        this._contentRootTransform = new Windows.UI.Xaml.Media.TranslateTransform();
                        this._contentRootTransform.X = delta.X;
                        this._contentRootTransform.Y = delta.Y;
                    } else {
                        this._contentRootTransform.X += delta.X;
                        this._contentRootTransform.Y += delta.Y;
                    }

                    this.ContentRoot.RenderTransform = this._contentRootTransform;

                    this._clickPoint = posNow.$clone();
                }
            },
            Chrome_PointerPressed: function (sender, e) {
                this._clickPoint = new Windows.Foundation.Point.$ctor1(e._pointerAbsoluteX, e._pointerAbsoluteY);


                this._chrome.CapturePointer();

                this._isMouseCaptured = true;
            },
            Chrome_PointerReleased: function (sender, e) {
                this._clickPoint = new Windows.Foundation.Point.$ctor1(0, 0);

                this._chrome.ReleasePointerCapture();

                this._isMouseCaptured = false;
            },
            RegisterEvents: function () {
                if (this._chrome != null) {
                    this._chrome.removePointerMoved(Bridge.fn.cacheBind(this, this.Chrome_MouseMove));
                    this._chrome.addPointerMoved(Bridge.fn.cacheBind(this, this.Chrome_MouseMove));
                    this._chrome.removePointerReleased(Bridge.fn.cacheBind(this, this.Chrome_PointerReleased));
                    this._chrome.addPointerReleased(Bridge.fn.cacheBind(this, this.Chrome_PointerReleased));
                    this._chrome.removePointerPressed(Bridge.fn.cacheBind(this, this.Chrome_PointerPressed));
                    this._chrome.addPointerPressed(Bridge.fn.cacheBind(this, this.Chrome_PointerPressed));
                }
            },
            /**
             * Executed when the application is exited.
             *
             * @instance
             * @this Windows.UI.Xaml.Controls.ChildWindow
             * @memberof Windows.UI.Xaml.Controls.ChildWindow
             * @param   {System.Object}    sender    The sender.
             * @param   {System.Object}    e         Event args.
             * @return  {void}
             */
            Application_Exit: function (sender, e) {
                if (this.IsOpen) {
                    this._isAppExit = true;
                    try {
                        this.Close();
                    } finally {
                        this._isAppExit = false;
                    }
                }
            },
            /**
             * Changes the visual state of the ChildWindow.
             *
             * @instance
             * @private
             * @this Windows.UI.Xaml.Controls.ChildWindow
             * @memberof Windows.UI.Xaml.Controls.ChildWindow
             * @return  {void}
             */
            ChangeVisualState: function () {
                if (this._isClosing) {
                    Windows.UI.Xaml.VisualStateManager.GoToState(this, Windows.UI.Xaml.Controls.ChildWindow.VSMSTATE_StateClosed, true);
                } else {
                    Windows.UI.Xaml.VisualStateManager.GoToState(this, Windows.UI.Xaml.Controls.ChildWindow.VSMSTATE_StateOpen, true);
                }
            },
            /**
             * Executed when ChildWindow size is changed.
             *
             * @instance
             * @private
             * @this Windows.UI.Xaml.Controls.ChildWindow
             * @memberof Windows.UI.Xaml.Controls.ChildWindow
             * @param   {System.Object}                           sender    Sender object.
             * @param   {Windows.UI.Xaml.SizeChangedEventArgs}    e         Size changed event args.
             * @return  {void}
             */
            ChildWindow_SizeChanged: function (sender, e) {
                if (this.Overlay != null) {
                    if (e.NewSize.Height !== this.Overlay.Height) {
                        this._desiredContentHeight = e.NewSize.Height;
                    }

                    if (e.NewSize.Width !== this.Overlay.Width) {
                        this._desiredContentWidth = e.NewSize.Width;
                    }
                }

                if (this.IsOpen) {
                    this.UpdateOverlaySize();
                }
            },
            /**
             * Closes a ChildWindow.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Controls.ChildWindow
             * @memberof Windows.UI.Xaml.Controls.ChildWindow
             * @return  {void}
             */
            Close: function () {
                var e = new System.ComponentModel.CancelEventArgs.ctor();
                this.OnClosing(e);

                if (!e.Cancel || this._isAppExit) {
                    if (Windows.UI.Xaml.Controls.ChildWindow.RootVisual != null) {
                        Windows.UI.Xaml.Controls.ChildWindow.RootVisual.IsEnabled = true;
                    }

                    if (this.IsOpen) {
                        if (this._closed != null) {
                            this._isClosing = true;
                            try {
                                this.ChangeVisualState();
                            } finally {
                                this._isClosing = false;
                            }
                        } else {
                            this.ChildWindowPopup.IsOpen = false;
                        }

                        if (!System.Nullable.hasValue(this._dialogresult)) {
                            this._dialogresult = false;
                        }

                        this.OnClosed({ });
                        this.UnSubscribeFromEvents();
                        this.UnsubscribeFromTemplatePartEvents();

                        if (Windows.UI.Xaml.Application.Current.RootVisual != null) {
                            Windows.UI.Xaml.Application.Current.RootVisual.removeGotFocus(Bridge.fn.cacheBind(this, this.RootVisual_GotFocus));
                        }
                    }
                } else {
                    this._dialogresult = null;
                }
            },
            /**
             * Executed when the CloseButton is clicked.
             *
             * @instance
             * @this Windows.UI.Xaml.Controls.ChildWindow
             * @memberof Windows.UI.Xaml.Controls.ChildWindow
             * @param   {System.Object}                      sender    Sender object.
             * @param   {Windows.UI.Xaml.RoutedEventArgs}    e         Routed event args.
             * @return  {void}
             */
            CloseButton_Click: function (sender, e) {
                this.Close();
            },
            /**
             * Executed when the Closing storyboard ends.
             *
             * @instance
             * @private
             * @this Windows.UI.Xaml.Controls.ChildWindow
             * @memberof Windows.UI.Xaml.Controls.ChildWindow
             * @param   {System.Object}    sender    Sender object.
             * @param   {System.Object}    e         Event args.
             * @return  {void}
             */
            Closing_Completed: function (sender, e) {
                if (this.ChildWindowPopup != null) {
                    this.ChildWindowPopup.IsOpen = false;
                }


                if (this._closed != null) {
                }
            },
            /**
             * Executed when the window loses focus.
             *
             * @instance
             * @private
             * @this Windows.UI.Xaml.Controls.ChildWindow
             * @memberof Windows.UI.Xaml.Controls.ChildWindow
             * @param   {System.Object}                      sender    Sender object.
             * @param   {Windows.UI.Xaml.RoutedEventArgs}    e         Routed event args.
             * @return  {void}
             */
            ChildWindow_LostFocus: function (sender, e) {
                if (this.IsOpen && Windows.UI.Xaml.Application.Current != null && Windows.UI.Xaml.Application.Current.RootVisual != null) {
                    Windows.UI.Xaml.Application.Current.RootVisual.addGotFocus(Bridge.fn.cacheBind(this, this.RootVisual_GotFocus));
                }
            },
            /**
             * Builds the visual tree for the
             ChildWindow control when a
             new template is applied.
             *
             * @instance
             * @protected
             * @override
             * @this Windows.UI.Xaml.Controls.ChildWindow
             * @memberof Windows.UI.Xaml.Controls.ChildWindow
             * @return  {void}
             */
            OnApplyTemplate: function () {
                this.UnsubscribeFromTemplatePartEvents();

                Windows.UI.Xaml.Controls.ContentControl.prototype.OnApplyTemplate.call(this);

                this.CloseButton = Bridge.as(this.GetTemplateChild(Windows.UI.Xaml.Controls.ChildWindow.PART_CloseButton), Windows.UI.Xaml.Controls.Primitives.ButtonBase);

                if (this.CloseButton != null) {
                    if (this.HasCloseButton) {
                        this.CloseButton.Visibility = Windows.UI.Xaml.Visibility.Visible;
                    } else {
                        this.CloseButton.Visibility = Windows.UI.Xaml.Visibility.Collapsed;
                    }
                }

                if (this._closed != null) {
                }

                if (this._opened != null) {
                }

                this._root = Bridge.as(this.GetTemplateChild(Windows.UI.Xaml.Controls.ChildWindow.PART_Root), Windows.UI.Xaml.FrameworkElement);

                if (this._root != null) {
                }

                this.ContentRoot = Bridge.as(this.GetTemplateChild(Windows.UI.Xaml.Controls.ChildWindow.PART_ContentRoot), Windows.UI.Xaml.FrameworkElement);

                this._chrome = Bridge.as(this.GetTemplateChild(Windows.UI.Xaml.Controls.ChildWindow.PART_Chrome), Windows.UI.Xaml.FrameworkElement);

                this.Overlay = Bridge.as(this.GetTemplateChild(Windows.UI.Xaml.Controls.ChildWindow.PART_Overlay), Windows.UI.Xaml.Controls.Panel);

                this._contentPresenter = Bridge.as(this.GetTemplateChild(Windows.UI.Xaml.Controls.ChildWindow.PART_ContentPresenter), Windows.UI.Xaml.FrameworkElement);

                this.SubscribeToTemplatePartEvents();
                this.SubscribeToStoryBoardEvents();
                this._desiredMargin = this.Margin.$clone();
                this.Margin = new Windows.UI.Xaml.Thickness.$ctor1(0);


                if (this.IsOpen) {
                    this._desiredContentHeight = this.Height;
                    this._desiredContentWidth = this.Width;
                    this._heightThatWasInitiallySpecified = this.Height;
                    this._widthThatWasInitiallySpecified = this.Width;
                    this.UpdateOverlaySize();
                    this.UpdateRenderTransform();
                    this.ChangeVisualState();
                }

                this.RegisterEvents();
            },
            /**
             * Raises the ChildWindow.Closed event.
             *
             * @instance
             * @protected
             * @this Windows.UI.Xaml.Controls.ChildWindow
             * @memberof Windows.UI.Xaml.Controls.ChildWindow
             * @param   {System.Object}    e    The event data.
             * @return  {void}
             */
            OnClosed: function (e) {
                var handler = this.Closed;

                if (!Bridge.staticEquals(null, handler)) {
                    handler(this, e);
                }

                this._isOpen = false;
            },
            /**
             * Raises the ChildWindow.Closing event.
             *
             * @instance
             * @protected
             * @this Windows.UI.Xaml.Controls.ChildWindow
             * @memberof Windows.UI.Xaml.Controls.ChildWindow
             * @param   {System.ComponentModel.CancelEventArgs}    e    The event data.
             * @return  {void}
             */
            OnClosing: function (e) {
                var handler = this.Closing;

                if (!Bridge.staticEquals(null, handler)) {
                    handler(this, e);
                }
            },
            /**
             * This method is called every time a
             ChildWindow is displayed.
             *
             * @instance
             * @protected
             * @this Windows.UI.Xaml.Controls.ChildWindow
             * @memberof Windows.UI.Xaml.Controls.ChildWindow
             * @return  {void}
             */
            OnOpened: function () {
                this.UpdatePosition();
                this._isOpen = true;

                if (this.Overlay != null) {
                    this.Overlay.Opacity = this.OverlayOpacity;
                    this.Overlay.Background = this.OverlayBrush;
                }

            },
            /**
             * Executed when the page resizes.
             *
             * @instance
             * @private
             * @this Windows.UI.Xaml.Controls.ChildWindow
             * @memberof Windows.UI.Xaml.Controls.ChildWindow
             * @param   {System.Object}                                 sender    Sender object.
             * @param   {Windows.UI.Core.WindowSizeChangedEventArgs}    e         Event args.
             * @return  {void}
             */
            Page_Resized: function (sender, e) {
                if (this.ChildWindowPopup != null) {
                    this.UpdateOverlaySize();
                }
            },
            /**
             * Executed when the root visual gets focus.
             *
             * @instance
             * @private
             * @this Windows.UI.Xaml.Controls.ChildWindow
             * @memberof Windows.UI.Xaml.Controls.ChildWindow
             * @param   {System.Object}                      sender    Sender object.
             * @param   {Windows.UI.Xaml.RoutedEventArgs}    e         Routed event args.
             * @return  {void}
             */
            RootVisual_GotFocus: function (sender, e) {
                this.Focus();
            },
            /**
             * Opens a ChildWindow and
             returns without waiting for the
             ChildWindow to close.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Controls.ChildWindow
             * @memberof Windows.UI.Xaml.Controls.ChildWindow
             * @throws The child window is already in the visual tree.
             * @return  {void}
             */
            Show: function () {

                this.SubscribeToEvents();
                this.SubscribeToTemplatePartEvents();
                this.SubscribeToStoryBoardEvents();

                if (this.ChildWindowPopup == null) {
                    this.ChildWindowPopup = new Windows.UI.Xaml.Controls.Primitives.Popup();

                    this.ChildWindowPopup.HorizontalContentAlignment = Windows.UI.Xaml.HorizontalAlignment.Stretch;
                    this.ChildWindowPopup.VerticalContentAlignment = Windows.UI.Xaml.VerticalAlignment.Stretch;
                    try {
                        this.ChildWindowPopup.Child = this;
                    } catch ($e1) {
                        $e1 = System.Exception.create($e1);
                        if (Bridge.is($e1, System.ArgumentException)) {
                            throw $e1;
                        } else {
                            throw $e1;
                        }
                    }
                }


                if (this.ChildWindowPopup != null) {
                    this.ChildWindowPopup.IsOpen = true;

                    this._dialogresult = null;
                }

                if (Windows.UI.Xaml.Controls.ChildWindow.RootVisual != null) {
                    Windows.UI.Xaml.Controls.ChildWindow.RootVisual.IsEnabled = false;
                }

                if (this.ContentRoot != null) {
                    this.ChangeVisualState();
                }
            },
            /**
             * Opens a ChildWindow and waits for the ChildWindow to close.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Controls.ChildWindow
             * @memberof Windows.UI.Xaml.Controls.ChildWindow
             * @throws The child window is already in the visual tree.
             * @return  {System.Threading.Tasks.Task}
             */
            ShowAndWait: function () {
                var taskCompletionSource = new System.Threading.Tasks.TaskCompletionSource();
                this.addClosed(function (s, e) {
                    if (taskCompletionSource != null) {
                        taskCompletionSource.setResult(null);
                        taskCompletionSource = null;
                    }
                });
                this.Show();
                return taskCompletionSource.task;
            },
            /**
             * Subscribes to events when the ChildWindow is opened.
             *
             * @instance
             * @private
             * @this Windows.UI.Xaml.Controls.ChildWindow
             * @memberof Windows.UI.Xaml.Controls.ChildWindow
             * @return  {void}
             */
            SubscribeToEvents: function () {
                if (Windows.UI.Xaml.Application.Current != null && Windows.UI.Xaml.Application.Current.Host != null && Windows.UI.Xaml.Application.Current.Host.Content != null) {
                    Windows.UI.Xaml.Application.Current.addExit(Bridge.fn.cacheBind(this, this.Application_Exit));
                    Windows.UI.Xaml.Window.Current.addSizeChanged$1(Bridge.fn.cacheBind(this, this.Page_Resized));
                }

                this.addSizeChanged(Bridge.fn.cacheBind(this, this.ChildWindow_SizeChanged));
            },
            /**
             * Subscribes to events that are on the storyboards. 
             Unsubscribing from these events happen in the event handlers individually.
             *
             * @instance
             * @private
             * @this Windows.UI.Xaml.Controls.ChildWindow
             * @memberof Windows.UI.Xaml.Controls.ChildWindow
             * @return  {void}
             */
            SubscribeToStoryBoardEvents: function () {
                if (this._closed != null) {
                }

                if (this._opened != null) {
                }
            },
            /**
             * Subscribes to events on the template parts.
             *
             * @instance
             * @private
             * @this Windows.UI.Xaml.Controls.ChildWindow
             * @memberof Windows.UI.Xaml.Controls.ChildWindow
             * @return  {void}
             */
            SubscribeToTemplatePartEvents: function () {
                if (this.CloseButton != null) {
                    this.CloseButton.addClick(Bridge.fn.cacheBind(this, this.CloseButton_Click));
                }

                if (this._chrome != null) {
                }

                if (this._contentPresenter != null) {
                }
            },
            /**
             * Unsubscribe from events when the ChildWindow is closed.
             *
             * @instance
             * @private
             * @this Windows.UI.Xaml.Controls.ChildWindow
             * @memberof Windows.UI.Xaml.Controls.ChildWindow
             * @return  {void}
             */
            UnSubscribeFromEvents: function () {
                if (Windows.UI.Xaml.Application.Current != null && Windows.UI.Xaml.Application.Current.Host != null && Windows.UI.Xaml.Application.Current.Host.Content != null) {
                    Windows.UI.Xaml.Application.Current.removeExit(Bridge.fn.cacheBind(this, this.Application_Exit));
                    Windows.UI.Xaml.Window.Current.removeSizeChanged$1(Bridge.fn.cacheBind(this, this.Page_Resized));
                }

                this.removeSizeChanged(Bridge.fn.cacheBind(this, this.ChildWindow_SizeChanged));
            },
            /**
             * Unsubscribe from the events that are subscribed on the template part elements.
             *
             * @instance
             * @private
             * @this Windows.UI.Xaml.Controls.ChildWindow
             * @memberof Windows.UI.Xaml.Controls.ChildWindow
             * @return  {void}
             */
            UnsubscribeFromTemplatePartEvents: function () {
                if (this.CloseButton != null) {
                    this.CloseButton.removeClick(Bridge.fn.cacheBind(this, this.CloseButton_Click));
                }

                if (this._chrome != null) {
                }

                if (this._contentPresenter != null) {
                }
            },
            /**
             * Updates the size of the overlay of the window.
             *
             * @instance
             * @private
             * @this Windows.UI.Xaml.Controls.ChildWindow
             * @memberof Windows.UI.Xaml.Controls.ChildWindow
             * @return  {void}
             */
            UpdateOverlaySize: function () {
                if (this.Overlay != null && Windows.UI.Xaml.Application.Current != null && Windows.UI.Xaml.Application.Current.Host != null && Windows.UI.Xaml.Application.Current.Host.Content != null) {
                    var windowBounds = Windows.UI.Xaml.Window.Current.Bounds.$clone();
                    var windowHeight = windowBounds.Height;
                    var windowWidth = windowBounds.Width;
                    this.Height = windowBounds.Height;
                    this.Width = windowBounds.Width;
                    this.Overlay.Height = this.Height;
                    this.Overlay.Width = this.Width;

                    if (this.ContentRoot != null) {
                        this.ContentRoot.Width = this._widthThatWasInitiallySpecified;
                        this.ContentRoot.Height = this._heightThatWasInitiallySpecified;

                        var childWindowContentIfAny = (Bridge.is(this._contentPresenter, Windows.UI.Xaml.Controls.ContentPresenter) ? Bridge.as(Bridge.cast(this._contentPresenter, Windows.UI.Xaml.Controls.ContentPresenter).Content, Windows.UI.Xaml.FrameworkElement) : null);

                        var chromeHeight = { };
                        if (this.TryCalculateChromeHeight(chromeHeight)) {
                            var isHeightAuto = isNaN(this._heightThatWasInitiallySpecified);
                            if (isHeightAuto) {
                                var maxHeight = windowHeight - chromeHeight.v;
                                this._contentPresenter.MaxHeight = maxHeight;
                                if (childWindowContentIfAny != null) {
                                    childWindowContentIfAny.MaxHeight = maxHeight;
                                }
                            } else {
                                var maxHeight1 = this._heightThatWasInitiallySpecified - chromeHeight.v;
                                this._contentPresenter.MaxHeight = maxHeight1;
                                if (childWindowContentIfAny != null) {
                                    childWindowContentIfAny.MaxHeight = maxHeight1;
                                }
                            }
                        }
                        var chromeWidth = { };
                        if (this.TryCalculateChromeWidth(chromeWidth)) {
                            var isWidthAuto = isNaN(this._widthThatWasInitiallySpecified);
                            if (isWidthAuto) {
                                var maxWidth = windowWidth - chromeWidth.v;
                                this._contentPresenter.MaxWidth = maxWidth;
                                if (childWindowContentIfAny != null) {
                                    childWindowContentIfAny.MaxWidth = maxWidth;
                                }
                            } else {
                                var maxWidth1 = this._widthThatWasInitiallySpecified - chromeWidth.v;
                                this._contentPresenter.MaxWidth = maxWidth1;
                                if (childWindowContentIfAny != null) {
                                    childWindowContentIfAny.MaxWidth = maxWidth1;
                                }
                            }
                        }
                        this.ContentRoot.Margin = this._desiredMargin.$clone();
                    }
                }
            },
            /**
             * Determines the chrome size (ie. size of the header bar and margins, that is the difference between the size of the ContentPresenter and the ContentRoot)
             *
             * @instance
             * @private
             * @this Windows.UI.Xaml.Controls.ChildWindow
             * @memberof Windows.UI.Xaml.Controls.ChildWindow
             * @param   {System.Double}    chromeHeight
             * @return  {boolean}
             */
            TryCalculateChromeHeight: function (chromeHeight) {
                if (this.ContentRoot != null && this._contentPresenter != null) {
                    var actualContentRootHeight = this.ContentRoot.ActualHeight;
                    var actualContentPresenterHeight = this._contentPresenter.ActualHeight;
                    if (!isNaN(actualContentRootHeight) && !isNaN(actualContentPresenterHeight)) {
                        chromeHeight.v = actualContentRootHeight - actualContentPresenterHeight;
                        return true;
                    }
                }
                chromeHeight.v = 0.0;
                return false;
            },
            /**
             * Determines the chrome size (ie. size of the header bar and margins, that is the difference between the size of the ContentPresenter and the ContentRoot)
             *
             * @instance
             * @private
             * @this Windows.UI.Xaml.Controls.ChildWindow
             * @memberof Windows.UI.Xaml.Controls.ChildWindow
             * @param   {System.Double}    chromeWidth
             * @return  {boolean}
             */
            TryCalculateChromeWidth: function (chromeWidth) {
                if (this.ContentRoot != null && this._contentPresenter != null) {
                    var actualContentRootWidth = this.ContentRoot.ActualWidth;
                    var actualContentPresenterWidth = this._contentPresenter.ActualWidth;
                    if (!isNaN(actualContentRootWidth) && !isNaN(actualContentPresenterWidth)) {
                        chromeWidth.v = actualContentRootWidth - actualContentPresenterWidth;
                        return true;
                    }
                }
                chromeWidth.v = 0.0;
                return false;
            },
            /**
             * Updates the position of the window in case the size of the content changes.
             This allows ChildWindow only scale from right and bottom.
             *
             * @instance
             * @private
             * @this Windows.UI.Xaml.Controls.ChildWindow
             * @memberof Windows.UI.Xaml.Controls.ChildWindow
             * @return  {void}
             */
            UpdatePosition: function () {
                if (this.ContentRoot != null && Windows.UI.Xaml.Application.Current != null && Windows.UI.Xaml.Application.Current.RootVisual != null) {
                    var gt = this.ContentRoot.TransformToVisual(Windows.UI.Xaml.Application.Current.RootVisual);

                    if (gt != null) {
                        this._windowPosition = gt.Transform(new Windows.Foundation.Point.$ctor1(0, 0));
                    }
                }
            },
            /**
             * Updates the render transform applied on the overlay.
             *
             * @instance
             * @private
             * @this Windows.UI.Xaml.Controls.ChildWindow
             * @memberof Windows.UI.Xaml.Controls.ChildWindow
             * @return  {void}
             */
            UpdateRenderTransform: function () { },
            /**
             * Updates the ContentRootTranslateTransform.
             *
             * @instance
             * @private
             * @this Windows.UI.Xaml.Controls.ChildWindow
             * @memberof Windows.UI.Xaml.Controls.ChildWindow
             * @param   {number}    X    X coordinate of the transform.
             * @param   {number}    Y    Y coordinate of the transform.
             * @return  {void}
             */
            UpdateContentRootTransform: function (X, Y) { }
        }
    });

    /**
     * Displays the content of a ContentControl.
     *
     * @public
     * @class Windows.UI.Xaml.Controls.ContentPresenter
     * @augments Windows.UI.Xaml.Controls.ContentControl
     */
    Bridge.define("Windows.UI.Xaml.Controls.ContentPresenter", {
        inherits: [Windows.UI.Xaml.Controls.ContentControl],
        methods: {
            UpdateTabIndex: function (isTabStop, tabIndex) {
                return;
            }
        }
    });

    /**
     * Represents a control that defines choices for users to select.
     *
     * @abstract
     * @public
     * @class Windows.UI.Xaml.Controls.Primitives.MenuBase
     * @augments Windows.UI.Xaml.Controls.ItemsControl
     */
    Bridge.define("Windows.UI.Xaml.Controls.Primitives.MenuBase", {
        inherits: [Windows.UI.Xaml.Controls.ItemsControl],
        ctors: {
            /**
             * Initializes a new instance of the System.Windows.Controls.Primitives.MenuBase class.
             *
             * @instance
             * @protected
             * @this Windows.UI.Xaml.Controls.Primitives.MenuBase
             * @memberof Windows.UI.Xaml.Controls.Primitives.MenuBase
             * @return  {void}
             */
            ctor: function () {
                this.$initialize();
                Windows.UI.Xaml.Controls.ItemsControl.ctor.call(this);
            }
        },
        methods: {
            IsItemItsOwnContainerOverride: function (item) {
                return Bridge.is(item, Windows.UI.Xaml.Controls.MenuItem);
            },
            GetContainerForItemOverride: function () {
                return new Windows.UI.Xaml.Controls.MenuItem();
            }
        }
    });

    Bridge.define("Windows.UI.Xaml.Controls.DatePicker", {
        inherits: [Windows.UI.Xaml.Controls.INTERNAL_DateTimePickerBase],
        statics: {
            fields: {
                /**
                 * Identifies the dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Controls.DatePicker
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                SelectedDateProperty: null
            },
            ctors: {
                init: function () {
                    var $t;
                    this.SelectedDateProperty = Windows.UI.Xaml.DependencyProperty.Register("SelectedDate", System.Nullable$1(System.DateTime), Windows.UI.Xaml.Controls.DatePicker, ($t = new Windows.UI.Xaml.PropertyMetadata.$ctor4(Windows.UI.Xaml.Controls.DatePicker.OnSelectedDateChanged), $t.CallPropertyChangedWhenLoadedIntoVisualTree = Windows.UI.Xaml.WhenToCallPropertyChangedEnum.IfPropertyIsSet, $t));
                }
            },
            methods: {
                /**
                 * SelectedDateProperty property changed handler.
                 *
                 * @static
                 * @private
                 * @this Windows.UI.Xaml.Controls.DatePicker
                 * @memberof Windows.UI.Xaml.Controls.DatePicker
                 * @param   {Windows.UI.Xaml.DependencyObject}                      d    TemporalPicker that changed its SelectedDate.
                 * @param   {Windows.UI.Xaml.DependencyPropertyChangedEventArgs}    e    The DependencyPropertyChangedEventArgs.
                 * @return  {void}
                 */
                OnSelectedDateChanged: function (d, e) {
                    Bridge.cast(d, Windows.UI.Xaml.Controls.DatePicker).INTERNAL_SelectedDate = Bridge.cast(Bridge.unbox(e.NewValue, System.DateTime), System.DateTime, true);
                }
            }
        },
        props: {
            /**
             * Gets or sets the currently selected date.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.DatePicker
             * @function SelectedDate
             * @type ?System.DateTime
             */
            SelectedDate: {
                get: function () {
                    return Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.Controls.DatePicker.SelectedDateProperty), System.DateTime), System.DateTime, true);
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Controls.DatePicker.SelectedDateProperty, Bridge.box(value, System.DateTime, System.Nullable.toStringFn(System.DateTime.format), System.Nullable.getHashCode));
                }
            }
        },
        ctors: {
            ctor: function () {
                this.$initialize();
                Windows.UI.Xaml.Controls.INTERNAL_DateTimePickerBase.ctor.call(this);
                this._defaultText = "";

                this.DefaultStyleKey = Windows.UI.Xaml.Controls.DatePicker;
            }
        },
        methods: {
            GenerateCalendarOrClock: function () {
                return new Windows.UI.Xaml.Controls.Calendar();
            },
            SetTextFromDate: function (newDate) {
                var split = System.String.split(newDate, [32].map(function (i) {{ return String.fromCharCode(i); }}));
                var date = System.String.split(split[System.Array.index(0, split)], [47].map(function (i) {{ return String.fromCharCode(i); }}));
                return (date[System.Array.index(0, date)] || "") + "-" + (date[System.Array.index(1, date)] || "") + "-" + (date[System.Array.index(2, date)] || "");
            },
            OnSelectionChanged: function (newSelectedDate) {
                if (System.Nullable.hasValue(newSelectedDate) && System.Nullable.hasValue(this.SelectedDate)) {
                    if (!Bridge.equals(this.SelectedDate, newSelectedDate)) {
                        this.SelectedDate = newSelectedDate;
                    }
                } else if (System.Nullable.hasValue(newSelectedDate) && !System.Nullable.hasValue(this.SelectedDate)) {
                    this.SelectedDate = newSelectedDate;
                } else {
                    if (!Bridge.equals(this.SelectedDate, null)) {
                        this.SelectedDate = null;
                    }
                }

                this.IsDropDownOpen = false;
            }
        }
    });

    /**
     * @memberof System.Windows
     * @callback System.Windows.DragEventHandler
     * @param   {System.Object}                   sender    
     * @param   {System.Windows.DragEventArgs}    e
     * @return  {void}
     */

    /**
     * A control that enabled drag and drop operations on an ItemsControl.
     *
     * @abstract
     * @public
     * @class Windows.UI.Xaml.Controls.DragDropTarget$2
     * @augments Windows.UI.Xaml.Controls.ContentControl
     * @param   {Function}    [name]    The type of the items control.
     * @param   {Function}    [name]    The type of the item container.
     */
    Bridge.define("Windows.UI.Xaml.Controls.DragDropTarget$2", function (TItemsControlType, TItemContainerType) { return {
        inherits: [Windows.UI.Xaml.Controls.ContentControl],
        statics: {
            methods: {
                /**
                 * This method returns null if no DragDropTarget is under the pointer, else it returns the DragDropTarget under it (the first Parent found)
                 *
                 * @static
                 * @private
                 * @this Windows.UI.Xaml.Controls.DragDropTarget$2
                 * @memberof Windows.UI.Xaml.Controls.DragDropTarget$2
                 * @param   {number}                                       x                            
                 * @param   {number}                                       y                            
                 * @param   {TItemContainerType}                           itemContainerUnderPointer    The ListBoxItem or other container that is being moved. If there is no container, it is the item directly.
                 * @return  {Windows.UI.Xaml.Controls.DragDropTarget$2}                                 Null if no DragDropTarget is under the pointer, else it returns the DragDropTarget under it (the first Parent found)
                 */
                GetDragDropTargetUnderPointer: function (x, y, itemContainerUnderPointer) {
                    var element = Windows.UI.Xaml.Media.VisualTreeHelper.FindElementInHostCoordinates(new Windows.Foundation.Point.$ctor1(x, y));
                    var ElementsBetweenClickedElementAndDragDropTarget = new (System.Collections.Generic.List$1(System.Object)).ctor();
                    var dragDropTargetUnder = null;
                    while (element != null) {
                        ElementsBetweenClickedElementAndDragDropTarget.add(element);
                        if (Bridge.is(element, Windows.UI.Xaml.Controls.DragDropTarget$2(TItemsControlType,TItemContainerType))) {
                            dragDropTargetUnder = Bridge.cast(element, Windows.UI.Xaml.Controls.DragDropTarget$2(TItemsControlType,TItemContainerType));
                            break;
                        }

                        element = Bridge.cast(element.INTERNAL_VisualParent, Windows.UI.Xaml.UIElement);
                    }

                    if (dragDropTargetUnder != null) {
                        var indexOfLastElementInList = (ElementsBetweenClickedElementAndDragDropTarget.Count - 1) | 0;
                        var amoutOfElementsBetweenItemsRootAndDragDropTarget = dragDropTargetUnder.INTERNAL_GetNumberOfElementsBetweenItemsRootAndDragDropTarget();
                        if (indexOfLastElementInList < amoutOfElementsBetweenItemsRootAndDragDropTarget) {
                            itemContainerUnderPointer.v = null;
                            return dragDropTargetUnder;
                        }
                        var elementToMove = System.Linq.Enumerable.from(ElementsBetweenClickedElementAndDragDropTarget).elementAt(((indexOfLastElementInList - amoutOfElementsBetweenItemsRootAndDragDropTarget) | 0));
                        itemContainerUnderPointer.v = Bridge.cast(elementToMove, TItemContainerType);
                        return dragDropTargetUnder;
                    } else {
                        itemContainerUnderPointer.v = null;
                        return null;
                    }
                },
                GeneratePopupContent: function (sourceItemContainer, iconStop, iconArrow) {
                    var $t;
                    var stackPanelInPopUp = new Windows.UI.Xaml.Controls.StackPanel();
                    iconStop.v = ($t = new Windows.UI.Xaml.Shapes.Path(), $t.Data = Bridge.cast(DotNetForHtml5.Core.TypeFromStringConverters.ConvertFromInvariantString(Windows.UI.Xaml.Media.Geometry, "M13.4005,19.7645 L13.3231,19.8886 C11.5316,22.8369 10.5,26.298 10.5,30 C10.5,40.7696 19.2305,49.5 30,49.5 C33.702,49.5 37.1631,48.4684 40.1115,46.6769 L40.2355,46.5995 z M30,10.5 C26.298,10.5 22.8369,11.5316 19.8886,13.3231 L19.7645,13.4005 L46.5995,40.2355 L46.6769,40.1115 C48.4684,37.1631 49.5,33.702 49.5,30 C49.5,19.2305 40.7696,10.5 30,10.5 z M30,0.5 C46.2924,0.5 59.5,13.7076 59.5,30 C59.5,46.2924 46.2924,59.5 30,59.5 C13.7076,59.5 0.5,46.2924 0.5,30 C0.5,13.7076 13.7076,0.5 30,0.5 z"), Windows.UI.Xaml.Media.Geometry), $t.Fill = new Windows.UI.Xaml.Media.SolidColorBrush.$ctor1(Windows.UI.Colors.Red.$clone()), $t.Height = 10, $t.Width = 10, $t.Stretch = Windows.UI.Xaml.Media.Stretch.Fill, $t.HorizontalAlignment = Windows.UI.Xaml.HorizontalAlignment.Right, $t.VerticalAlignment = Windows.UI.Xaml.VerticalAlignment.Bottom, $t);
                    iconArrow.v = ($t = new Windows.UI.Xaml.Shapes.Path(), $t.Data = Bridge.cast(DotNetForHtml5.Core.TypeFromStringConverters.ConvertFromInvariantString(Windows.UI.Xaml.Media.Geometry, "M120,60 L120,80 L160,80 L160,90 L180,70 L160,50 L160,60 z"), Windows.UI.Xaml.Media.Geometry), $t.Fill = new Windows.UI.Xaml.Media.SolidColorBrush.$ctor1(Windows.UI.Colors.Blue.$clone()), $t.Height = 7.057, $t.Width = 10.5, $t.Stretch = Windows.UI.Xaml.Media.Stretch.Fill, $t.HorizontalAlignment = Windows.UI.Xaml.HorizontalAlignment.Right, $t.VerticalAlignment = Windows.UI.Xaml.VerticalAlignment.Bottom, $t);
                    iconArrow.v.Visibility = Windows.UI.Xaml.Visibility.Collapsed;
                    stackPanelInPopUp.Orientation = Windows.UI.Xaml.Controls.Orientation.Horizontal;
                    stackPanelInPopUp.Children.add(sourceItemContainer);
                    stackPanelInPopUp.Children.add(iconStop.v);
                    stackPanelInPopUp.Children.add(iconArrow.v);
                    return stackPanelInPopUp;
                }
            }
        },
        fields: {
            _isPointerCaptured: false,
            _capturedPointer: null,
            _pointerX: 0,
            _pointerY: 0,
            _sourceItemsControl: Bridge.getDefaultValue(TItemsControlType),
            _sourceItemContainer: Bridge.getDefaultValue(TItemContainerType),
            _sourcePlaceholder: null,
            _popup: null,
            _indexOfSourceContainerWithinItemsControl: 0,
            _previousdragDropTargetUnderPointer: null,
            _isDragCancelled: false,
            _iconStop: null,
            _iconArrow: null
        },
        events: {
            /**
             * An event raised when a drag operation is starting on an item.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Controls.DragDropTarget$2
             * @memberof Windows.UI.Xaml.Controls.DragDropTarget$2
             * @function addItemDragStarting
             * @param   {System.EventHandler}    value
             * @return  {void}
             */
            /**
             * An event raised when a drag operation is starting on an item.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Controls.DragDropTarget$2
             * @memberof Windows.UI.Xaml.Controls.DragDropTarget$2
             * @function removeItemDragStarting
             * @param   {System.EventHandler}    value
             * @return  {void}
             */
            ItemDragStarting: null,
            /**
             * Fires when the DragDropTarget control loses mouse capture.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Controls.DragDropTarget$2
             * @memberof Windows.UI.Xaml.Controls.DragDropTarget$2
             * @function addItemDragCompleted
             * @param   {System.EventHandler}    value
             * @return  {void}
             */
            /**
             * Fires when the DragDropTarget control loses mouse capture.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Controls.DragDropTarget$2
             * @memberof Windows.UI.Xaml.Controls.DragDropTarget$2
             * @function removeItemDragCompleted
             * @param   {System.EventHandler}    value
             * @return  {void}
             */
            ItemDragCompleted: null,
            /**
             * Raises the DragEnter event.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Controls.DragDropTarget$2
             * @memberof Windows.UI.Xaml.Controls.DragDropTarget$2
             * @function addDragEnter
             * @param   {System.Windows.DragEventHandler}    value
             * @return  {void}
             */
            /**
             * Raises the DragEnter event.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Controls.DragDropTarget$2
             * @memberof Windows.UI.Xaml.Controls.DragDropTarget$2
             * @function removeDragEnter
             * @param   {System.Windows.DragEventHandler}    value
             * @return  {void}
             */
            DragEnter: null,
            /**
             * Raises the DragLeave event.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Controls.DragDropTarget$2
             * @memberof Windows.UI.Xaml.Controls.DragDropTarget$2
             * @function addDragLeave
             * @param   {System.Windows.DragEventHandler}    value
             * @return  {void}
             */
            /**
             * Raises the DragLeave event.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Controls.DragDropTarget$2
             * @memberof Windows.UI.Xaml.Controls.DragDropTarget$2
             * @function removeDragLeave
             * @param   {System.Windows.DragEventHandler}    value
             * @return  {void}
             */
            DragLeave: null,
            /**
             * Raises the DragOver event.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Controls.DragDropTarget$2
             * @memberof Windows.UI.Xaml.Controls.DragDropTarget$2
             * @function addDragOver
             * @param   {System.Windows.DragEventHandler}    value
             * @return  {void}
             */
            /**
             * Raises the DragOver event.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Controls.DragDropTarget$2
             * @memberof Windows.UI.Xaml.Controls.DragDropTarget$2
             * @function removeDragOver
             * @param   {System.Windows.DragEventHandler}    value
             * @return  {void}
             */
            DragOver: null,
            /**
             * Raises the Drop event
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Controls.DragDropTarget$2
             * @memberof Windows.UI.Xaml.Controls.DragDropTarget$2
             * @function addDrop
             * @param   {System.Windows.DragEventHandler}    value
             * @return  {void}
             */
            /**
             * Raises the Drop event
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Controls.DragDropTarget$2
             * @memberof Windows.UI.Xaml.Controls.DragDropTarget$2
             * @function removeDrop
             * @param   {System.Windows.DragEventHandler}    value
             * @return  {void}
             */
            Drop: null,
            /**
             * Raises the ItemDroppedOnSource event
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Controls.DragDropTarget$2
             * @memberof Windows.UI.Xaml.Controls.DragDropTarget$2
             * @function addItemDroppedOnSource
             * @param   {System.Windows.DragEventHandler}    value
             * @return  {void}
             */
            /**
             * Raises the ItemDroppedOnSource event
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Controls.DragDropTarget$2
             * @memberof Windows.UI.Xaml.Controls.DragDropTarget$2
             * @function removeItemDroppedOnSource
             * @param   {System.Windows.DragEventHandler}    value
             * @return  {void}
             */
            ItemDroppedOnSource: null
        },
        ctors: {
            /**
             * Initializes a new instance of the DragDropTarget class.
             *
             * @instance
             * @protected
             * @this Windows.UI.Xaml.Controls.DragDropTarget$2
             * @memberof Windows.UI.Xaml.Controls.DragDropTarget$2
             * @return  {void}
             */
            ctor: function () {
                this.$initialize();
                Windows.UI.Xaml.Controls.ContentControl.ctor.call(this);
                this.addPointerPressed(Bridge.fn.cacheBind(this, this.DragDropTarget_PointerPressed));
                this.addPointerMoved(Bridge.fn.cacheBind(this, this.DragDropTarget_PointerMoved));
                this.addPointerReleased(Bridge.fn.cacheBind(this, this.DragDropTarget_PointerReleased));
            }
        },
        methods: {
            DragDropTarget_PointerPressed: function (sender, e) {
                var $t;

                e.Handled = true;

                if (!this._isPointerCaptured) {
                    this._isDragCancelled = false;
                    this._previousdragDropTargetUnderPointer = null;

                    this._pointerX = e.GetCurrentPoint(null).Position.X;
                    this._pointerY = e.GetCurrentPoint(null).Position.Y;
                    var sourceDragDropTarget = Windows.UI.Xaml.Controls.DragDropTarget$2(TItemsControlType,TItemContainerType).GetDragDropTargetUnderPointer(this._pointerX, this._pointerY, Bridge.ref(this, "_sourceItemContainer"));
                    if (!Bridge.referenceEquals(sourceDragDropTarget, this)) {
                        throw new System.Exception("The DragDropTarget is not supposed to support dragging an outer DragDropTarget in case of nested DragDropTargets.");
                    }

                    if (this._sourceItemContainer != null) {
                        this._sourceItemsControl = Bridge.cast(this.Content, TItemsControlType);

                        this.CapturePointer(e.Pointer);
                        this._isPointerCaptured = true;
                        this._capturedPointer = e.Pointer;

                        var height;
                        var width;
                        if (Bridge.is(this._sourceItemContainer, Windows.UI.Xaml.FrameworkElement)) {
                            var actualSize = (Bridge.as(this._sourceItemContainer, Windows.UI.Xaml.FrameworkElement)).INTERNAL_GetActualWidthAndHeight();
                            height = actualSize.Height;
                            width = actualSize.Width;
                        } else {
                            height = Number.NaN;
                            width = Number.NaN;
                        }

                        var selection = new Windows.UI.Xaml.Controls.Selection.$ctor1(this._sourceItemContainer);
                        var selectionCollection = Windows.UI.Xaml.Controls.SelectionCollection.ToSelectionCollection(selection);
                        var itemDragStartingEventArgs = new Windows.UI.Xaml.Controls.ItemDragEventArgs(selectionCollection);

                        if (!Bridge.staticEquals(this.ItemDragStarting, null)) {
                            this.ItemDragStarting(this, itemDragStartingEventArgs);
                        }

                        if (itemDragStartingEventArgs.Handled && itemDragStartingEventArgs.Cancel) {

                            if (this._isPointerCaptured) {
                                this._isPointerCaptured = false;

                                this.ReleasePointerCapture(this._capturedPointer);
                            }
                        } else {

                            this.RemoveSourceAndPutTransparentPlaceholderInPlace(height, width);

                            var stackPanelInPopUp = Windows.UI.Xaml.Controls.DragDropTarget$2(TItemsControlType,TItemContainerType).GeneratePopupContent(this._sourceItemContainer, Bridge.ref(this, "_iconStop"), Bridge.ref(this, "_iconArrow"));
                            this._popup = ($t = new Windows.UI.Xaml.Controls.Primitives.Popup(), $t.Child = stackPanelInPopUp, $t.HorizontalAlignment = Windows.UI.Xaml.HorizontalAlignment.Left, $t.VerticalAlignment = Windows.UI.Xaml.VerticalAlignment.Top, $t.HorizontalContentAlignment = Windows.UI.Xaml.HorizontalAlignment.Left, $t.VerticalContentAlignment = Windows.UI.Xaml.VerticalAlignment.Top, $t.IsHitTestVisible = false, $t);

                            this._popup.HorizontalOffset = this._pointerX;
                            this._popup.VerticalOffset = this._pointerY;

                            this._popup.IsOpen = true;
                        }
                    }
                }
            },
            DragDropTarget_PointerMoved: function (sender, e) {

                if (this._isPointerCaptured) {

                    var horizontalChange = e.GetCurrentPoint(null).Position.X - this._pointerX;
                    var verticalChange = e.GetCurrentPoint(null).Position.Y - this._pointerY;

                    this._pointerX = e.GetCurrentPoint(null).Position.X;
                    this._pointerY = e.GetCurrentPoint(null).Position.Y;

                    this.MovePopupAndRaiseEvents(horizontalChange, verticalChange);
                }
            },
            MovePopupAndRaiseEvents: function (horizontalChange, verticalChange) {

                if (this._popup != null) {
                    this._popup.HorizontalOffset += horizontalChange;
                    this._popup.VerticalOffset += verticalChange;
                }

                var selection = new Windows.UI.Xaml.Controls.Selection.$ctor1(this._sourceItemContainer);
                var selectionCollection = Windows.UI.Xaml.Controls.SelectionCollection.ToSelectionCollection(selection);
                var dataObject = new System.Windows.DataObject();
                dataObject.SetData$1("ItemDragEventArgs", new Windows.UI.Xaml.Controls.ItemDragEventArgs(selectionCollection));
                var dragOverEventArgs = new System.Windows.DragEventArgs.ctor(dataObject);

                var targetItemContainer = { };
                var dragDropTargetUnderPointer = Windows.UI.Xaml.Controls.DragDropTarget$2(TItemsControlType,TItemContainerType).GetDragDropTargetUnderPointer(this._pointerX, this._pointerY, targetItemContainer);

                if (!Bridge.referenceEquals(this._previousdragDropTargetUnderPointer, dragDropTargetUnderPointer)) {

                    if (this._previousdragDropTargetUnderPointer != null && !Bridge.staticEquals(this._previousdragDropTargetUnderPointer.DragLeave, null)) {
                        this._previousdragDropTargetUnderPointer.DragLeave(this._previousdragDropTargetUnderPointer, new System.Windows.DragEventArgs.ctor(dataObject));

                        this._isDragCancelled = false;
                    }
                    this._previousdragDropTargetUnderPointer = dragDropTargetUnderPointer;

                    if (dragDropTargetUnderPointer != null && !Bridge.staticEquals(dragDropTargetUnderPointer.DragEnter, null)) {
                        dragDropTargetUnderPointer.DragEnter(dragDropTargetUnderPointer, new System.Windows.DragEventArgs.ctor(dataObject));
                    }
                }

                if (dragDropTargetUnderPointer != null) {

                    dragDropTargetUnderPointer.OnDragOver(dragOverEventArgs);

                    if (dragOverEventArgs.Handled$1 && dragOverEventArgs.Effects === System.Windows.DragDropEffects.None) {
                        this._isDragCancelled = true;
                    }
                }

                if (dragDropTargetUnderPointer != null && dragDropTargetUnderPointer.AllowDrop && !this._isDragCancelled) {
                    this._iconArrow.Visibility = Windows.UI.Xaml.Visibility.Visible;
                    this._iconStop.Visibility = Windows.UI.Xaml.Visibility.Collapsed;
                } else {
                    this._iconArrow.Visibility = Windows.UI.Xaml.Visibility.Collapsed;
                    this._iconStop.Visibility = Windows.UI.Xaml.Visibility.Visible;
                }
            },
            DragDropTarget_PointerReleased: function (sender, e) {

                this._pointerX = e.GetCurrentPoint(null).Position.X;
                this._pointerY = e.GetCurrentPoint(null).Position.Y;

                if (this._isPointerCaptured && Bridge.referenceEquals(e.Pointer, this._capturedPointer)) {
                    this.MovePopupAndRaiseEvents(0, 0);

                    this._isPointerCaptured = false;

                    this.ReleasePointerCapture(this._capturedPointer);
                    this.OnDropped(e);
                }
            },
            OnDropped: function (e) {

                this._popup.IsOpen = false;

                this._popup.Child = null;
                this._popup = null;

                var selection = new Windows.UI.Xaml.Controls.Selection.$ctor1(this._sourceItemContainer);
                var selectionCollection = Windows.UI.Xaml.Controls.SelectionCollection.ToSelectionCollection(selection);

                var targetItemContainer = { };
                var dragDropTargetUnderPointer = Windows.UI.Xaml.Controls.DragDropTarget$2(TItemsControlType,TItemContainerType).GetDragDropTargetUnderPointer(this._pointerX, this._pointerY, targetItemContainer);

                if (dragDropTargetUnderPointer != null) {

                    if (dragDropTargetUnderPointer.AllowDrop && !this._isDragCancelled) {
                        var targetContainer = Bridge.cast(dragDropTargetUnderPointer.Content, TItemsControlType);

                        if (Bridge.referenceEquals(this.ContainerFromIndex(targetContainer, this._indexOfSourceContainerWithinItemsControl), this._sourcePlaceholder)) {

                            this.PutSourceBackToOriginalPlace();

                            if (!Bridge.staticEquals(dragDropTargetUnderPointer.ItemDroppedOnSource, null)) {
                                var dataObject = new System.Windows.DataObject();
                                dataObject.SetData$1("ItemDragEventArgs", new Windows.UI.Xaml.Controls.ItemDragEventArgs(selectionCollection));

                                dragDropTargetUnderPointer.ItemDroppedOnSource(dragDropTargetUnderPointer, new System.Windows.DragEventArgs.$ctor1(dataObject, e));
                            }
                        } else {

                            this.RemoveItemAtIndex(this._sourceItemsControl, this._indexOfSourceContainerWithinItemsControl);

                            if (!Bridge.staticEquals(dragDropTargetUnderPointer.Drop, null)) {
                                var dataObject1 = new System.Windows.DataObject();
                                dataObject1.SetData$1("ItemDragEventArgs", new Windows.UI.Xaml.Controls.ItemDragEventArgs(selectionCollection));

                                dragDropTargetUnderPointer.Drop(dragDropTargetUnderPointer, new System.Windows.DragEventArgs.$ctor1(dataObject1, e));
                            }

                            dragDropTargetUnderPointer.AddItem(Bridge.cast(dragDropTargetUnderPointer.Content, TItemsControlType), this._sourceItemContainer);
                        }
                    } else {
                        this.PutSourceBackToOriginalPlace();
                    }
                } else {
                    this.PutSourceBackToOriginalPlace();
                }

                if (!Bridge.staticEquals(this.ItemDragCompleted, null)) {
                    this.ItemDragCompleted(this, new Windows.UI.Xaml.Controls.ItemDragEventArgs(selectionCollection));
                }
            },
            PutSourceBackToOriginalPlace: function () {
                this.RemoveItemAtIndex(this._sourceItemsControl, this._indexOfSourceContainerWithinItemsControl);

                this.InsertItem(this._sourceItemsControl, this._indexOfSourceContainerWithinItemsControl, this._sourceItemContainer);
            },
            RemoveSourceAndPutTransparentPlaceholderInPlace: function (sourceHeight, sourceWidth) {
                var $t;
                this._indexOfSourceContainerWithinItemsControl = System.Nullable.getValue(this.IndexFromContainer(this._sourceItemsControl, this._sourceItemContainer));

                this.RemoveItem(this._sourceItemsControl, this._sourceItemContainer);

                this._sourcePlaceholder = ($t = new Windows.UI.Xaml.Controls.Border(), $t.Background = new Windows.UI.Xaml.Media.SolidColorBrush.$ctor1(Windows.UI.Colors.Transparent.$clone()), $t);

                if (!isNaN(sourceHeight) && !isNaN(sourceWidth)) {
                    this._sourcePlaceholder.Height = sourceHeight;
                    this._sourcePlaceholder.Width = sourceWidth;
                }

                this.InsertItem(this._sourceItemsControl, this._indexOfSourceContainerWithinItemsControl, this._sourcePlaceholder);
            },
            /**
             * Raises the DragOver event.
             *
             * @instance
             * @protected
             * @this Windows.UI.Xaml.Controls.DragDropTarget$2
             * @memberof Windows.UI.Xaml.Controls.DragDropTarget$2
             * @param   {System.Windows.DragEventArgs}    dragOverEventArgs    Information about the event.
             * @return  {void}
             */
            OnDragOver: function (dragOverEventArgs) {
                if (!Bridge.staticEquals(this.DragOver, null)) {
                    this.DragOver(this, dragOverEventArgs);
                }

            },
            INTERNAL_OnAttachedToVisualTree: function () {
                Windows.UI.Xaml.Controls.ContentControl.prototype.INTERNAL_OnAttachedToVisualTree.call(this);

                if (!CSHTML5.Interop.IsRunningInTheSimulator) {
                    this.INTERNAL_OuterDomElement.onselectstart = function() { return false; };
                }
            }
        }
    }; });

    /**
     * Provides the base implementation for controls that contain a single
     content element and a header.
     *
     * @public
     * @class Windows.UI.Xaml.Controls.HeaderedContentControl
     * @augments Windows.UI.Xaml.Controls.ContentControl
     */
    Bridge.define("Windows.UI.Xaml.Controls.HeaderedContentControl", {
        inherits: [Windows.UI.Xaml.Controls.ContentControl],
        statics: {
            fields: {
                /**
                 * Identifies the
                 {@link }
                 dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Controls.HeaderedContentControl
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                HeaderProperty: null,
                /**
                 * Identifies the
                 {@link }
                 dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Controls.HeaderedContentControl
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                HeaderTemplateProperty: null
            },
            ctors: {
                init: function () {
                    var $t;
                    this.HeaderProperty = Windows.UI.Xaml.DependencyProperty.Register("Header", System.Object, Windows.UI.Xaml.Controls.HeaderedContentControl, ($t = new Windows.UI.Xaml.PropertyMetadata.$ctor2(null, Windows.UI.Xaml.Controls.HeaderedContentControl.OnHeaderPropertyChanged), $t.CallPropertyChangedWhenLoadedIntoVisualTree = Windows.UI.Xaml.WhenToCallPropertyChangedEnum.IfPropertyIsSet, $t));
                    this.HeaderTemplateProperty = Windows.UI.Xaml.DependencyProperty.Register("HeaderTemplate", Windows.UI.Xaml.DataTemplate, Windows.UI.Xaml.Controls.HeaderedContentControl, ($t = new Windows.UI.Xaml.PropertyMetadata.$ctor2(null, Windows.UI.Xaml.Controls.HeaderedContentControl.OnHeaderTemplatePropertyChanged), $t.CallPropertyChangedWhenLoadedIntoVisualTree = Windows.UI.Xaml.WhenToCallPropertyChangedEnum.IfPropertyIsSet, $t));
                }
            },
            methods: {
                /**
                 * HeaderProperty property changed handler.
                 *
                 * @static
                 * @private
                 * @this Windows.UI.Xaml.Controls.HeaderedContentControl
                 * @memberof Windows.UI.Xaml.Controls.HeaderedContentControl
                 * @param   {Windows.UI.Xaml.DependencyObject}                      d    HeaderedContentControl whose Header property is changed.
                 * @param   {Windows.UI.Xaml.DependencyPropertyChangedEventArgs}    e    DependencyPropertyChangedEventArgs, which contains the old and new value.
                 * @return  {void}
                 */
                OnHeaderPropertyChanged: function (d, e) {
                    var ctrl = Bridge.cast(d, Windows.UI.Xaml.Controls.HeaderedContentControl);
                    ctrl.OnHeaderChanged(e.OldValue, e.NewValue);
                },
                /**
                 * HeaderTemplateProperty property changed handler.
                 *
                 * @static
                 * @private
                 * @this Windows.UI.Xaml.Controls.HeaderedContentControl
                 * @memberof Windows.UI.Xaml.Controls.HeaderedContentControl
                 * @param   {Windows.UI.Xaml.DependencyObject}                      d    HeaderedContentControl whose HeaderTemplate property is changed.
                 * @param   {Windows.UI.Xaml.DependencyPropertyChangedEventArgs}    e    DependencyPropertyChangedEventArgs, which contains the old and new value.
                 * @return  {void}
                 */
                OnHeaderTemplatePropertyChanged: function (d, e) {
                    var ctrl = Bridge.cast(d, Windows.UI.Xaml.Controls.HeaderedContentControl);
                    ctrl.OnHeaderTemplateChanged(Bridge.cast(e.OldValue, Windows.UI.Xaml.DataTemplate), Bridge.cast(e.NewValue, Windows.UI.Xaml.DataTemplate));
                }
            }
        },
        props: {
            /**
             * Gets or sets the content for the header of the control.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.HeaderedContentControl
             * @function Header
             * @type System.Object
             */
            Header: {
                get: function () {
                    return this.GetValue(Windows.UI.Xaml.Controls.HeaderedContentControl.HeaderProperty);
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Controls.HeaderedContentControl.HeaderProperty, value);
                }
            },
            /**
             * Gets or sets the template that is used to display the content of the
             control's header.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.HeaderedContentControl
             * @function HeaderTemplate
             * @type Windows.UI.Xaml.DataTemplate
             */
            HeaderTemplate: {
                get: function () {
                    return Bridge.cast(this.GetValue(Windows.UI.Xaml.Controls.HeaderedContentControl.HeaderTemplateProperty), Windows.UI.Xaml.DataTemplate);
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Controls.HeaderedContentControl.HeaderTemplateProperty, value);
                }
            }
        },
        ctors: {
            /**
             * Initializes a new instance of the
             {@link }
             class.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Controls.HeaderedContentControl
             * @memberof Windows.UI.Xaml.Controls.HeaderedContentControl
             * @return  {void}
             */
            ctor: function () {
                this.$initialize();
                Windows.UI.Xaml.Controls.ContentControl.ctor.call(this);
            }
        },
        methods: {
            /**
             * Called when the value of the
             {@link }
             property changes.
             *
             * @instance
             * @protected
             * @this Windows.UI.Xaml.Controls.HeaderedContentControl
             * @memberof Windows.UI.Xaml.Controls.HeaderedContentControl
             * @param   {System.Object}    oldHeader    The old value of the
             {@link }
             property.
             * @param   {System.Object}    newHeader    The new value of the
             {@link }
             property.
             * @return  {void}
             */
            OnHeaderChanged: function (oldHeader, newHeader) { },
            /**
             * Called when the value of the
             {@link }
             property changes.
             *
             * @instance
             * @protected
             * @this Windows.UI.Xaml.Controls.HeaderedContentControl
             * @memberof Windows.UI.Xaml.Controls.HeaderedContentControl
             * @param   {Windows.UI.Xaml.DataTemplate}    oldHeaderTemplate    The old value of the
             {@link }
             property.
             * @param   {Windows.UI.Xaml.DataTemplate}    newHeaderTemplate    The new value of the
             {@link }
             property.
             * @return  {void}
             */
            OnHeaderTemplateChanged: function (oldHeaderTemplate, newHeaderTemplate) { }
        }
    });

    /**
     * @memberof Windows.UI.Xaml.Navigation
     * @callback Windows.UI.Xaml.Navigation.NavigationFailedEventHandler
     * @param   {System.Object}                                           sender    
     * @param   {Windows.UI.Xaml.Navigation.NavigationFailedEventArgs}    e
     * @return  {void}
     */

    /**
     * @memberof Windows.UI.Xaml.Navigation
     * @callback Windows.UI.Xaml.Navigation.NavigatedEventHandler
     * @param   {System.Object}                                     sender    
     * @param   {Windows.UI.Xaml.Navigation.NavigationEventArgs}    e
     * @return  {void}
     */

    /**
     * Represents a control that supports navigation to and from pages.
     *
     * @public
     * @class Windows.UI.Xaml.Controls.Frame
     * @augments Windows.UI.Xaml.Controls.ContentControl
     * @implements  Windows.UI.Xaml.Controls.INavigate
     */
    Bridge.define("Windows.UI.Xaml.Controls.Frame", {
        inherits: [Windows.UI.Xaml.Controls.ContentControl,Windows.UI.Xaml.Controls.INavigate],
        statics: {
            fields: {
                FIRST_AND_LAST_PAGE_STATE_NAME: null,
                FIRST_PAGE_STATE_NAME: null,
                LAST_PAGE_STATE_NAME: null,
                INTERMEDIARY_PAGE_STATE_NAME: null,
                /**
                 * Identifies the System.Windows.Controls.Frame.CanGoBack dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Controls.Frame
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                CanGoBackProperty: null,
                /**
                 * Identifies the System.Windows.Controls.Frame.CanGoForward dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Controls.Frame
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                CanGoForwardProperty: null,
                /**
                 * Identifies the System.Windows.Controls.Frame.Source dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Controls.Frame
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                SourceProperty: null,
                /**
                 * Identifies the System.Windows.Controls.Frame.JournalOwnership dependency
                 property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Controls.Frame
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                JournalOwnershipProperty: null,
                /**
                 * Identifies the System.Windows.Controls.Frame.UriMapper dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Controls.Frame
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                UriMapperProperty: null
            },
            ctors: {
                init: function () {
                    var $t;
                    this.FIRST_AND_LAST_PAGE_STATE_NAME = "firstLast";
                    this.FIRST_PAGE_STATE_NAME = "first";
                    this.LAST_PAGE_STATE_NAME = "last";
                    this.INTERMEDIARY_PAGE_STATE_NAME = "inter";
                    this.CanGoBackProperty = Windows.UI.Xaml.DependencyProperty.Register("CanGoBack", System.Boolean, Windows.UI.Xaml.Controls.Frame, new Windows.UI.Xaml.PropertyMetadata.$ctor1(Bridge.box(false, System.Boolean, System.Boolean.toString)));
                    this.CanGoForwardProperty = Windows.UI.Xaml.DependencyProperty.Register("CanGoForward", System.Boolean, Windows.UI.Xaml.Controls.Frame, new Windows.UI.Xaml.PropertyMetadata.$ctor1(Bridge.box(false, System.Boolean, System.Boolean.toString)));
                    this.SourceProperty = Windows.UI.Xaml.DependencyProperty.Register("Source", System.Uri, Windows.UI.Xaml.Controls.Frame, ($t = new Windows.UI.Xaml.PropertyMetadata.$ctor2(null, Windows.UI.Xaml.Controls.Frame.Source_Changed), $t.CallPropertyChangedWhenLoadedIntoVisualTree = Windows.UI.Xaml.WhenToCallPropertyChangedEnum.IfPropertyIsSet, $t));
                    this.JournalOwnershipProperty = Windows.UI.Xaml.DependencyProperty.Register("JournalOwnership", Windows.UI.Xaml.Navigation.JournalOwnership, Windows.UI.Xaml.Controls.Frame, ($t = new Windows.UI.Xaml.PropertyMetadata.$ctor2(Bridge.box(Windows.UI.Xaml.Navigation.JournalOwnership.Automatic, Windows.UI.Xaml.Navigation.JournalOwnership, System.Enum.toStringFn(Windows.UI.Xaml.Navigation.JournalOwnership)), Windows.UI.Xaml.Controls.Frame.JournalOwnership_Changed), $t.CallPropertyChangedWhenLoadedIntoVisualTree = Windows.UI.Xaml.WhenToCallPropertyChangedEnum.IfPropertyIsSet, $t));
                    this.UriMapperProperty = Windows.UI.Xaml.DependencyProperty.Register("UriMapper", Windows.UI.Xaml.Navigation.UriMapperBase, Windows.UI.Xaml.Controls.Frame, new Windows.UI.Xaml.PropertyMetadata.$ctor4(null));
                }
            },
            methods: {
                Source_Changed: function (d, e) {
                    if (!Bridge.referenceEquals(e.NewValue, e.OldValue)) {
                        var source = Bridge.cast(e.NewValue, System.Uri);
                        var frame = Bridge.cast(d, Windows.UI.Xaml.Controls.Frame);
                        if (frame._isBrowserJournal) {
                            if (!frame._isNavigationDone) {
                                frame.Navigate_BrowserJournalVersion(source, true);
                            }
                        } else {
                            if (!frame._isGoingBackOrForward) {
                                frame._isSourceChanging = true;
                                frame.Navigate(Bridge.cast(e.NewValue, System.Uri));
                                frame._isSourceChanging = false;
                            }
                        }
                    }
                },
                JournalOwnership_Changed: function (d, e) {
                    var frame = Bridge.cast(d, Windows.UI.Xaml.Controls.Frame);
                    frame.UpdateFieldsBasedOnJournalOwnership();
                },
                FindPage: function (uri) {
                    var $t;
                    var assemblyName = { };
                    var instancierName = { };
                    Windows.UI.Xaml.Controls.Frame.GetXamlFileAssociatedClassInstancierName(uri, assemblyName, instancierName);
                    $t = Bridge.getEnumerator(System.AppDomain.getAssemblies());
                    try {
                        while ($t.moveNext()) {
                            var assembly = $t.Current;
                            if (Bridge.referenceEquals(INTERNAL_BridgeWorkarounds.GetAssemblyNameWithoutCallingGetNameMethod(assembly), assemblyName.v)) {
                                var type = Bridge.Reflection.getType(instancierName.v, assembly);
                                if (type != null) {
                                    var methodInfo = Bridge.Reflection.getMembers(type, 8, 284, "Instantiate");
                                    return Bridge.Reflection.midel(methodInfo, null).apply(null, null);
                                }
                                break;
                            }
                        }
                    } finally {
                        if (Bridge.is($t, System.IDisposable)) {
                            $t.System$IDisposable$Dispose();
                        }
                    }
                    return null;
                },
                IsNullOrUndefined: function (jsObject) {
                    if (CSHTML5.Interop.IsRunningInTheSimulator) {
                        if (jsObject == null) {
                            return true;
                        }
                        if (!(Bridge.is(jsObject, DotNetBrowser.JSValue))) {
                            return false;
                        }
                        var value = Bridge.cast(jsObject, DotNetBrowser.JSValue);
                        return value.IsNull() || value.IsUndefined();
                    } else {
                        return System.Convert.toBoolean((typeof jsObject === 'undefined' || jsObject === null));
                    }
                },
                GetXamlFileAssociatedClassInstancierName: function (uri, assemblyName, instancierName) {
                    var uriAsString = uri.getOriginalString();
                    var absoluteSourceUri = "";
                    assemblyName.v = "";
                    var splittedUri = System.String.split(uriAsString, [59].map(function (i) {{ return String.fromCharCode(i); }}));
                    if (splittedUri.length === 1) {
                        assemblyName.v = CSHTML5.Internal.StartupAssemblyInfo.StartupAssemblyShortName;
                        if (System.String.startsWith(uriAsString, "/")) {
                            absoluteSourceUri = "/" + (assemblyName.v || "") + ";component" + (uriAsString || "");
                        } else {
                            absoluteSourceUri = "/" + (assemblyName.v || "") + ";component/" + (uriAsString || "");
                        }
                    } else if (splittedUri.length === 2) {
                        assemblyName.v = splittedUri[System.Array.index(0, splittedUri)];
                        if (System.String.startsWith(assemblyName.v, "/")) {
                            assemblyName.v = assemblyName.v.substr(1);
                        }
                        absoluteSourceUri = uriAsString;
                    } else {
                        throw new System.Exception("Class name badly formatted.");
                    }
                    instancierName.v = Windows.UI.Xaml.Controls.Frame.GenerateClassNameFromAbsoluteUri_ForRuntimeAccess(absoluteSourceUri);
                },
                GenerateClassNameFromAbsoluteUri_ForRuntimeAccess: function (absoluteSourceUri) {
                    var className = Windows.UI.Xaml.Controls.Frame.MakeTitleCase(absoluteSourceUri);

                    className = System.Text.RegularExpressions.Regex.replace(className, "\\W", "\u01c0\u01c0");

                    if (System.Char.isDigit((className).charCodeAt(0))) {
                        className = System.String.insert(0, className, "_");
                    }

                    className = System.String.replaceAll(className, " ", "");

                    className = (className || "") + "\u01c0\u01c0Factory";

                    return className;
                },
                MakeTitleCase: function (str) {
                    var result = "";
                    var lowerStr = str.toLowerCase();
                    var length = str.length;
                    var makeUpper = true;
                    var lastCopiedIndex = -1;

                    for (var i = 0; i < length; i = (i + 1) | 0) {
                        var ch = lowerStr.charCodeAt(i);
                        if (ch >= 97 && ch <= 122 || ch >= 48 && ch <= 48) {
                            if (makeUpper && ch >= 97 && ch <= 122) {
                                if (!(lastCopiedIndex === -1 && i === 0)) {
                                    result = (result || "") + ((lowerStr.substr(((lastCopiedIndex + 1) | 0), ((((i - lastCopiedIndex) | 0) - 1) | 0))) || "");
                                }
                                result = (result || "") + String.fromCharCode((((((ch - 32) | 0)) & 65535)));
                                lastCopiedIndex = i;
                            }
                            makeUpper = false;
                        } else {
                            makeUpper = true;
                        }
                    }
                    if (lastCopiedIndex < ((length - 1) | 0)) {
                        result = (result || "") + ((str.substr(((lastCopiedIndex + 1) | 0))) || "");
                    }
                    return result;


                }
            }
        },
        fields: {
            _isBrowserJournal: false,
            _isSourceChanging: false,
            _isGoingBackOrForward: false,
            _isTopLevelFrame: false,
            _cache: null,
            _subscribedToHashChangeEvent: false,
            _isNavigationDone: false
        },
        events: {
            /**
             * Occurs when the content that is being navigated to has been found and is
             available.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Controls.Frame
             * @memberof Windows.UI.Xaml.Controls.Frame
             * @function addNavigated
             * @param   {Windows.UI.Xaml.Navigation.NavigatedEventHandler}    value
             * @return  {void}
             */
            /**
             * Occurs when the content that is being navigated to has been found and is
             available.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Controls.Frame
             * @memberof Windows.UI.Xaml.Controls.Frame
             * @function removeNavigated
             * @param   {Windows.UI.Xaml.Navigation.NavigatedEventHandler}    value
             * @return  {void}
             */
            Navigated: null,
            /**
             * Occurs when an error is encountered while navigating to the requested content.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Controls.Frame
             * @memberof Windows.UI.Xaml.Controls.Frame
             * @function addNavigationFailed
             * @param   {Windows.UI.Xaml.Navigation.NavigationFailedEventHandler}    value
             * @return  {void}
             */
            /**
             * Occurs when an error is encountered while navigating to the requested content.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Controls.Frame
             * @memberof Windows.UI.Xaml.Controls.Frame
             * @function removeNavigationFailed
             * @param   {Windows.UI.Xaml.Navigation.NavigationFailedEventHandler}    value
             * @return  {void}
             */
            NavigationFailed: null
        },
        props: {
            /**
             * Gets a value that indicates whether there is at least one entry in the back
             navigation history.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.Frame
             * @function CanGoBack
             * @type boolean
             */
            CanGoBack: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.Controls.Frame.CanGoBackProperty), System.Boolean), System.Boolean));
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Controls.Frame.CanGoBackProperty, Bridge.box(value, System.Boolean, System.Boolean.toString));
                }
            },
            /**
             * Gets a value that indicates whether there is at least one entry in the forward
             navigation history.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.Frame
             * @function CanGoForward
             * @type boolean
             */
            CanGoForward: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.Controls.Frame.CanGoForwardProperty), System.Boolean), System.Boolean));
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Controls.Frame.CanGoForwardProperty, Bridge.box(value, System.Boolean, System.Boolean.toString));
                }
            },
            /**
             * Gets or sets the uniform resource identifier (URI) of the current content
             or the content that is being navigated to.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.Frame
             * @function Source
             * @type System.Uri
             */
            Source: {
                get: function () {
                    return Bridge.cast(this.GetValue(Windows.UI.Xaml.Controls.Frame.SourceProperty), System.Uri);
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Controls.Frame.SourceProperty, value);
                }
            },
            /**
             * Gets or sets whether a frame is responsible for managing its own navigation
             history, or whether it integrates with the Web browser journal.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.Frame
             * @function JournalOwnership
             * @type Windows.UI.Xaml.Navigation.JournalOwnership
             */
            JournalOwnership: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.Controls.Frame.JournalOwnershipProperty), Windows.UI.Xaml.Navigation.JournalOwnership), System.Int32));
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Controls.Frame.JournalOwnershipProperty, Bridge.box(value, Windows.UI.Xaml.Navigation.JournalOwnership, System.Enum.toStringFn(Windows.UI.Xaml.Navigation.JournalOwnership)));
                }
            },
            /**
             * Gets or sets the object to manage converting a uniform resource identifier
             (URI) to another URI for this frame.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.Frame
             * @function UriMapper
             * @type Windows.UI.Xaml.Navigation.UriMapperBase
             */
            UriMapper: {
                get: function () {
                    return Bridge.cast(this.GetValue(Windows.UI.Xaml.Controls.Frame.UriMapperProperty), Windows.UI.Xaml.Navigation.UriMapperBase);
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Controls.Frame.UriMapperProperty, value);
                }
            }
        },
        alias: ["Navigate", "Windows$UI$Xaml$Controls$INavigate$Navigate"],
        ctors: {
            init: function () {
                this._isBrowserJournal = true;
                this._isSourceChanging = false;
                this._isGoingBackOrForward = false;
                this._isTopLevelFrame = true;
                this._cache = new Windows.UI.Xaml.Controls.Frame.FrameCache();
                this._subscribedToHashChangeEvent = false;
                this._isNavigationDone = false;
            },
            /**
             * Initializes a new instance of the System.Windows.Controls.Frame class.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Controls.Frame
             * @memberof Windows.UI.Xaml.Controls.Frame
             * @return  {void}
             */
            ctor: function () {
                this.$initialize();
                Windows.UI.Xaml.Controls.ContentControl.ctor.call(this);
                this.addLoaded(Bridge.fn.cacheBind(this, this.Frame_Loaded));
                this.addUnloaded(Bridge.fn.cacheBind(this, this.Frame_Unloaded));
            }
        },
        methods: {
            UpdateCanGoBack: function () {
                if (this._isBrowserJournal) {
                    this.CanGoBack = this.CanGoBack_BrowserJournalVersion();
                } else {
                    this.CanGoBack = this._cache.CanGoBack;
                }
            },
            UpdateCanGoForward: function () {
                if (this._isBrowserJournal) {
                    this.CanGoForward = this.CanGoForward_BrowserJournalVersion();
                } else {
                    this.CanGoForward = this._cache.CanGoForward;
                }
            },
            UpdateFieldsBasedOnJournalOwnership: function () {
                switch (this.JournalOwnership) {
                    case Windows.UI.Xaml.Navigation.JournalOwnership.Automatic: 
                        if (this._isTopLevelFrame) {
                            if (!this._isBrowserJournal) {
                                this._cache.Clear();
                                this._isBrowserJournal = true;
                                this.SubscribeToHashChanged();
                            }
                        } else {
                            if (this._isBrowserJournal) {
                                this._isBrowserJournal = false;
                                this.UnSubscribeFromHashChanged();
                            }
                        }
                        break;
                    case Windows.UI.Xaml.Navigation.JournalOwnership.OwnsJournal: 
                        if (this._isBrowserJournal) {
                            this._isBrowserJournal = false;
                            this.UnSubscribeFromHashChanged();
                        }
                        break;
                    default: 
                        if (this._isTopLevelFrame) {
                            if (!this._isBrowserJournal) {
                                this._cache.Clear();
                                this._isBrowserJournal = true;
                                this.SubscribeToHashChanged();
                            }
                        } else {
                            throw new System.InvalidOperationException.$ctor1("A frame cannot use the browser's journal it it is not a top-level frame.");
                        }
                        break;
                }
            },
            /**
             * Navigates to the most recent entry in the back navigation history, or throws
             an exception if no entry exists in back navigation.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Controls.Frame
             * @memberof Windows.UI.Xaml.Controls.Frame
             * @return  {void}
             */
            GoBack: function () {
                if (this._isBrowserJournal) {
                    this.GoBack_BrowserJournalVersion();
                } else {
                    this._isGoingBackOrForward = true;
                    this.Navigate(this._cache.GoBack()._uri);
                    this._isGoingBackOrForward = false;
                }
            },
            /**
             * Navigates to the most recent entry in the forward navigation history, or
             throws an exception if no entry exists in forward navigation.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Controls.Frame
             * @memberof Windows.UI.Xaml.Controls.Frame
             * @return  {void}
             */
            GoForward: function () {
                if (this._isBrowserJournal) {
                    this.GoForward_BrowserJournalVersion();
                } else {
                    this._isGoingBackOrForward = true;
                    this.Navigate(this._cache.GoForward()._uri);
                    this._isGoingBackOrForward = false;
                }
            },
            /**
             * Navigates to the content specified by the uniform resource identifier (URI).
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Controls.Frame
             * @memberof Windows.UI.Xaml.Controls.Frame
             * @param   {System.Uri}    source    The URI representing a page to display in the frame.
             * @return  {boolean}                 true if the navigation started successfully; otherwise, false.
             */
            Navigate: function (source) {
                if (this._isBrowserJournal) {
                    return this.Navigate_BrowserJournalVersion(source, true);
                } else {
                    return this.Navigate_OwnJournalVersion(source);
                }
            },
            Navigate_OwnJournalVersion: function (source) {
                try {
                    if (this._isGoingBackOrForward) {
                        this.Content = this._cache._currentItem._value;
                    }
                    if (this._isSourceChanging) {
                        var page = this.GetPageFromUri(source);
                        if (page != null) {
                            this._cache.Add(page, source);
                            this.Content = page;

                            this.UpdateCanGoBack();
                            this.UpdateCanGoForward();
                            this.RaiseNavigatedEvent();
                        } else {
                            this.RaiseNavigationFailedEvent(source);
                        }
                    } else if (System.Uri.notEquals(source, this.Source)) {
                        this.Source = source;
                    }
                    return true;
                } catch ($e1) {
                    $e1 = System.Exception.create($e1);
                    return false;
                }
            },
            Navigate_BrowserJournalVersion: function (source, updateHistoryState) {
                var state = null;
                var stateAfterNavigating = Windows.UI.Xaml.Controls.Frame.LAST_PAGE_STATE_NAME;
                if (updateHistoryState) {
                    state = System.Convert.toString(history.state);
                    var stateBeforeNavigating = null;
                    if (System.String.isNullOrEmpty(state)) {
                        stateBeforeNavigating = null;
                        stateAfterNavigating = Windows.UI.Xaml.Controls.Frame.FIRST_AND_LAST_PAGE_STATE_NAME;
                    } else if (Bridge.referenceEquals(state, Windows.UI.Xaml.Controls.Frame.FIRST_AND_LAST_PAGE_STATE_NAME)) {
                        stateBeforeNavigating = Windows.UI.Xaml.Controls.Frame.FIRST_PAGE_STATE_NAME;
                        stateAfterNavigating = Windows.UI.Xaml.Controls.Frame.LAST_PAGE_STATE_NAME;
                    } else if (Bridge.referenceEquals(state, Windows.UI.Xaml.Controls.Frame.FIRST_PAGE_STATE_NAME)) {
                        stateAfterNavigating = Windows.UI.Xaml.Controls.Frame.LAST_PAGE_STATE_NAME;
                    } else if (Bridge.referenceEquals(state, Windows.UI.Xaml.Controls.Frame.LAST_PAGE_STATE_NAME)) {
                        stateBeforeNavigating = Windows.UI.Xaml.Controls.Frame.INTERMEDIARY_PAGE_STATE_NAME;
                        stateAfterNavigating = Windows.UI.Xaml.Controls.Frame.LAST_PAGE_STATE_NAME;
                    } else {
                        stateAfterNavigating = Windows.UI.Xaml.Controls.Frame.LAST_PAGE_STATE_NAME;
                    }

                    if (stateBeforeNavigating != null) {
history.replaceState(                        stateBeforeNavigating, '');
                    }
                }

                if (this.TryGoToPage(source)) {
                    this._isNavigationDone = true;
                    this.Source = source;
                    this._isNavigationDone = false;

                    if (updateHistoryState) {
location.hash =                         System.Uri.notEquals(source, null) ? source.getOriginalString() : "";
history.replaceState(                        stateAfterNavigating, '');
                    }

                    this.UpdateCanGoBack();
                    this.UpdateCanGoForward();
                    this.RaiseNavigatedEvent();
                    return true;
                } else {
                    this.RaiseNavigationFailedEvent(source);
                    return false;
                }
            },
            CanGoBack_BrowserJournalVersion: function () {
                var state = System.Convert.toString(history.state);
                if (Bridge.referenceEquals(state, "") || Bridge.referenceEquals(state, Windows.UI.Xaml.Controls.Frame.FIRST_PAGE_STATE_NAME) || Bridge.referenceEquals(state, Windows.UI.Xaml.Controls.Frame.FIRST_AND_LAST_PAGE_STATE_NAME)) {
                    return false;
                } else {
                    return true;
                }
            },
            CanGoForward_BrowserJournalVersion: function () {
                var state = System.Convert.toString(history.state);
                if (Bridge.referenceEquals(state, "") || Bridge.referenceEquals(state, Windows.UI.Xaml.Controls.Frame.LAST_PAGE_STATE_NAME) || Bridge.referenceEquals(state, Windows.UI.Xaml.Controls.Frame.FIRST_AND_LAST_PAGE_STATE_NAME)) {
                    return false;
                } else {
                    return true;
                }
            },
            GoBack_BrowserJournalVersion: function () {
                history.back();
            },
            GoForward_BrowserJournalVersion: function () {
                history.forward();
            },
            LocationHashChanged: function () {
                var newHash = System.Convert.toString(location.hash);

                if (!System.String.isNullOrWhiteSpace(newHash)) {
                    if (System.String.startsWith(newHash, "#")) {
                        newHash = newHash.substr(1);
                    }

                    if (System.Uri.equals(this.Source, null) || !Bridge.referenceEquals(newHash, Bridge.toString(this.Source))) {
                        this.Navigate_BrowserJournalVersion(new System.Uri(newHash, System.UriKind.RelativeOrAbsolute), false);
                    }
                } else {
                    if (System.Uri.notEquals(this.Source, null) && !System.String.isNullOrWhiteSpace(Bridge.toString(this.Source))) {
                        this.Navigate_BrowserJournalVersion(this.Source, false);
                    }
                }
            },
            SubscribeToHashChanged: function () {
                if (!this._subscribedToHashChangeEvent) {
window.addEventListener('hashchange',                     Bridge.fn.cacheBind(this, this.LocationHashChanged), false);
                    this._subscribedToHashChangeEvent = true;
                }
            },
            UnSubscribeFromHashChanged: function () {
                if (this._subscribedToHashChangeEvent) {
window.removeEventListener('hashchange',                     Bridge.fn.cacheBind(this, this.LocationHashChanged), false);
                    this._subscribedToHashChangeEvent = false;
                }
            },
            GetPageFromUri: function (source) {
                var page = null;
                if (System.Uri.notEquals(source, null) && !System.String.isNullOrWhiteSpace(source.getOriginalString())) {
                    var mappedSource = source;
                    if (this.UriMapper != null) {
                        mappedSource = this.UriMapper.MapUri(source);
                    }

                    page = Windows.UI.Xaml.Controls.Frame.FindPage(mappedSource);
                }
                return page;
            },
            TryGoToPage: function (source) {
                var page = this.GetPageFromUri(source);
                if (page != null) {
                    this.Content = page;
                    return true;
                } else {
                    return false;
                }
            },
            RaiseNavigatedEvent: function () {
                if (!Bridge.staticEquals(this.Navigated, null)) {
                    var content = (this._isBrowserJournal ? this.Content : (this._cache._currentItem != null ? this._cache._currentItem._value : null));
                    var uri = (this._isBrowserJournal ? this.Source : (this._cache._currentItem != null ? this._cache._currentItem._uri : null));

                    this.Navigated(this, new Windows.UI.Xaml.Navigation.NavigationEventArgs(content, uri));
                }
            },
            RaiseNavigationFailedEvent: function (uri) {
                var exception = new System.ArgumentException.$ctor3("Content for the URI cannot be loaded. The URI may be invalid.", "uri");

                /* 
                if (NavigationFailed != null)
                {
                   var navigationFailedEventArgs = new NavigationFailedEventArgs(exception, false, uri);
                   NavigationFailed(this, navigationFailedEventArgs);

                   if (!navigationFailedEventArgs.Handled)
                       throw exception;
                }
                else
                {
                   throw exception;
                }
                */

                if (!Bridge.staticEquals(this.NavigationFailed, null)) {
                    var navigationFailedEventArgs = new Windows.UI.Xaml.Navigation.NavigationFailedEventArgs(exception, true, uri);
                    this.NavigationFailed(this, navigationFailedEventArgs);

                    if (!navigationFailedEventArgs.Handled) {
                        throw exception;
                    }
                }
            },
            Frame_Loaded: function (sender, e) {
                this._isTopLevelFrame = this.CheckIfThisIsATopLevelFrame();

                this.UpdateFieldsBasedOnJournalOwnership();
                if (this._isBrowserJournal) {
                    this.SubscribeToHashChanged();

                    this.Dispatcher.BeginInvoke(Bridge.fn.bind(this, function () {
                        this.LocationHashChanged();
                    }));
                }
            },
            CheckIfThisIsATopLevelFrame: function () {
                var currentElement = this.INTERNAL_VisualParent;
                while (currentElement != null) {
                    if (Bridge.is(currentElement, Windows.UI.Xaml.Controls.Frame)) {
                        return false;
                    }
                    currentElement = Bridge.cast(currentElement, Windows.UI.Xaml.UIElement).INTERNAL_VisualParent;
                }
                return true;
            },
            Frame_Unloaded: function (sender, e) {
                if (this._isBrowserJournal) {
                    this.UnSubscribeFromHashChanged();
                }
            }
        }
    });

    Bridge.define("Windows.UI.Xaml.Controls.GridSplitterGripper", {
        inherits: [Windows.UI.Xaml.Controls.ContentControl],
        statics: {
            fields: {
                GripperBarVertical: null,
                GripperBarHorizontal: null,
                GripperDisplayFont: null
            },
            ctors: {
                init: function () {
                    this.GripperBarVertical = "";
                    this.GripperBarHorizontal = "";
                    this.GripperDisplayFont = "Segoe MDL2 Assets";
                }
            }
        },
        fields: {
            _gripperDisplay: null,
            ResizeDirection: 0
        },
        props: {
            GripperForeground: {
                set: function (value) {
                    if (this._gripperDisplay == null) {
                        return;
                    }

                    this._gripperDisplay.Foreground = value;
                }
            }
        },
        ctors: {
            ctor: function (gridSplitterDirection) {
                this.$initialize();
                Windows.UI.Xaml.Controls.ContentControl.ctor.call(this);
                this.ResizeDirection = gridSplitterDirection;
                this.HorizontalContentAlignment = Windows.UI.Xaml.HorizontalAlignment.Stretch;
                this.VerticalContentAlignment = Windows.UI.Xaml.VerticalAlignment.Stretch;
            },
            $ctor1: function (gridSplitterDirection, gripForeground) {
                var $t;
                Windows.UI.Xaml.Controls.GridSplitterGripper.ctor.call(this, gridSplitterDirection);
                this._gripperDisplay = ($t = new Windows.UI.Xaml.Controls.TextBlock(), $t.FontFamily = new Windows.UI.Xaml.Media.FontFamily(Windows.UI.Xaml.Controls.GridSplitterGripper.GripperDisplayFont), $t.HorizontalAlignment = Windows.UI.Xaml.HorizontalAlignment.Center, $t.VerticalAlignment = Windows.UI.Xaml.VerticalAlignment.Center, $t.Foreground = gripForeground, $t);

                if (gridSplitterDirection === Windows.UI.Xaml.Controls.GridSplitter.GridResizeDirection.Columns) {
                    this._gripperDisplay.Text = Windows.UI.Xaml.Controls.GridSplitterGripper.GripperBarVertical;
                } else if (gridSplitterDirection === Windows.UI.Xaml.Controls.GridSplitter.GridResizeDirection.Rows) {
                    this._gripperDisplay.Text = Windows.UI.Xaml.Controls.GridSplitterGripper.GripperBarHorizontal;
                }

                this.Content = this._gripperDisplay;
            },
            $ctor2: function (content, gridSplitterDirection) {
                Windows.UI.Xaml.Controls.GridSplitterGripper.ctor.call(this, gridSplitterDirection);
                this.Content = content;
            }
        }
    });

    /**
     * Represents the text label for a control and provides support for access keys.
     *
     * @public
     * @class Windows.UI.Xaml.Controls.Label
     * @augments Windows.UI.Xaml.Controls.ContentControl
     */
    Bridge.define("Windows.UI.Xaml.Controls.Label", {
        inherits: [Windows.UI.Xaml.Controls.ContentControl],
        ctors: {
            ctor: function () {
                this.$initialize();
                Windows.UI.Xaml.Controls.ContentControl.ctor.call(this);
                this.DefaultStyleKey = Windows.UI.Xaml.Controls.Label;
            }
        }
    });

    /**
     * NumericUpDown control - for representing values that can be
     entered with keyboard,
     using increment/decrement buttons
     as well as swiping over the control.
     *
     * @public
     * @class Windows.UI.Xaml.Controls.NumericUpDown
     * @augments Windows.UI.Xaml.Controls.Primitives.RangeBase
     */
    Bridge.define("Windows.UI.Xaml.Controls.NumericUpDown", {
        inherits: [Windows.UI.Xaml.Controls.Primitives.RangeBase],
        statics: {
            fields: {
                DecrementButtonName: null,
                IncrementButtonName: null,
                DragOverlayName: null,
                ValueTextBoxName: null,
                ValueBarName: null,
                MinMouseDragDelta: 0,
                Epsilon: 0,
                /**
                 * ValueBarVisibility Dependency Property
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Controls.NumericUpDown
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                ValueBarVisibilityProperty: null,
                /**
                 * IsReadOnly Dependency Property
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Controls.NumericUpDown
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                IsReadOnlyProperty: null,
                /**
                 * DragSpeed Dependency Property
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Controls.NumericUpDown
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                DragSpeedProperty: null,
                /**
                 * The identifier for the Increment dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Controls.NumericUpDown
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                IncrementProperty: null
            },
            ctors: {
                init: function () {
                    var $t;
                    this.DecrementButtonName = "PART_DecrementButton";
                    this.IncrementButtonName = "PART_IncrementButton";
                    this.DragOverlayName = "PART_DragOverlay";
                    this.ValueTextBoxName = "PART_ValueTextBox";
                    this.ValueBarName = "PART_ValueBar";
                    this.MinMouseDragDelta = 2;
                    this.Epsilon = 1E-05;
                    this.ValueBarVisibilityProperty = Windows.UI.Xaml.DependencyProperty.Register("ValueBarVisibility", Windows.UI.Xaml.Controls.NumericUpDownValueBarVisibility, Windows.UI.Xaml.Controls.NumericUpDown, ($t = new Windows.UI.Xaml.PropertyMetadata.$ctor2(Bridge.box(Windows.UI.Xaml.Controls.NumericUpDownValueBarVisibility.Visible, Windows.UI.Xaml.Controls.NumericUpDownValueBarVisibility, System.Enum.toStringFn(Windows.UI.Xaml.Controls.NumericUpDownValueBarVisibility)), Windows.UI.Xaml.Controls.NumericUpDown.OnValueBarVisibilityChanged), $t.CallPropertyChangedWhenLoadedIntoVisualTree = Windows.UI.Xaml.WhenToCallPropertyChangedEnum.IfPropertyIsSet, $t));
                    this.IsReadOnlyProperty = Windows.UI.Xaml.DependencyProperty.Register("IsReadOnly", System.Boolean, Windows.UI.Xaml.Controls.NumericUpDown, ($t = new Windows.UI.Xaml.PropertyMetadata.$ctor2(Bridge.box(false, System.Boolean, System.Boolean.toString), Windows.UI.Xaml.Controls.NumericUpDown.OnIsReadOnlyChanged), $t.CallPropertyChangedWhenLoadedIntoVisualTree = Windows.UI.Xaml.WhenToCallPropertyChangedEnum.IfPropertyIsSet, $t));
                    this.DragSpeedProperty = Windows.UI.Xaml.DependencyProperty.Register("DragSpeed", System.Double, Windows.UI.Xaml.Controls.NumericUpDown, new Windows.UI.Xaml.PropertyMetadata.$ctor1(Bridge.box(Number.NaN, System.Double, System.Double.format, System.Double.getHashCode)));
                    this.IncrementProperty = Windows.UI.Xaml.DependencyProperty.Register("Increment", System.Double, Windows.UI.Xaml.Controls.NumericUpDown, new Windows.UI.Xaml.PropertyMetadata.$ctor1(Bridge.box(1.0, System.Double, System.Double.format, System.Double.getHashCode)));
                }
            },
            methods: {
                /**
                 * Handles changes to the ValueBarVisibility property.
                 *
                 * @static
                 * @private
                 * @this Windows.UI.Xaml.Controls.NumericUpDown
                 * @memberof Windows.UI.Xaml.Controls.NumericUpDown
                 * @param   {Windows.UI.Xaml.DependencyObject}                      d    The {@link } on which
                 the property has changed value.
                 * @param   {Windows.UI.Xaml.DependencyPropertyChangedEventArgs}    e    Event data that is issued by any event that
                 tracks changes to the effective value of this property.
                 * @return  {void}
                 */
                OnValueBarVisibilityChanged: function (d, e) {
                    var target = Bridge.cast(d, Windows.UI.Xaml.Controls.NumericUpDown);
                    target.OnValueBarVisibilityChanged();
                },
                /**
                 * Handles changes to the IsReadOnly property.
                 *
                 * @static
                 * @private
                 * @this Windows.UI.Xaml.Controls.NumericUpDown
                 * @memberof Windows.UI.Xaml.Controls.NumericUpDown
                 * @param   {Windows.UI.Xaml.DependencyObject}                      d    The {@link } on which
                 the property has changed value.
                 * @param   {Windows.UI.Xaml.DependencyPropertyChangedEventArgs}    e    Event data that is issued by any event that
                 tracks changes to the effective value of this property.
                 * @return  {void}
                 */
                OnIsReadOnlyChanged: function (d, e) {
                    var target = Bridge.cast(d, Windows.UI.Xaml.Controls.NumericUpDown);
                    target.OnIsReadOnlyChanged();
                }
            }
        },
        fields: {
            _dragOverlay: null,
            _valueTextBox: null,
            _decrementButton: null,
            _incrementButton: null,
            _valueBar: null,
            _isDragUpdated: false,
            _isChangingTextWithCode: false,
            _isChangingValueWithCode: false,
            _unusedManipulationDelta: 0,
            _isDraggingWithMouse: false,
            _totalDeltaX: 0,
            _totalDeltaY: 0,
            _hasFocus: false
        },
        props: {
            /**
             * Gets or sets the ValueBarVisibility property. This dependency property 
             indicates whether the value bar should be shown.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.NumericUpDown
             * @function ValueBarVisibility
             * @type Windows.UI.Xaml.Controls.NumericUpDownValueBarVisibility
             */
            ValueBarVisibility: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.Controls.NumericUpDown.ValueBarVisibilityProperty), Windows.UI.Xaml.Controls.NumericUpDownValueBarVisibility), System.Int32));
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Controls.NumericUpDown.ValueBarVisibilityProperty, Bridge.box(value, Windows.UI.Xaml.Controls.NumericUpDownValueBarVisibility, System.Enum.toStringFn(Windows.UI.Xaml.Controls.NumericUpDownValueBarVisibility)));
                }
            },
            /**
             * Gets or sets the IsReadOnly property. This dependency property 
             indicates whether the box should only allow to read the values by copying and pasting them.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.NumericUpDown
             * @function IsReadOnly
             * @type boolean
             */
            IsReadOnly: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.Controls.NumericUpDown.IsReadOnlyProperty), System.Boolean), System.Boolean));
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Controls.NumericUpDown.IsReadOnlyProperty, Bridge.box(value, System.Boolean, System.Boolean.toString));
                }
            },
            /**
             * Gets or sets the DragSpeed property. This dependency property 
             indicates the speed with which the value changes when manipulated with dragging.
             The default value of double.NaN indicates that the value will change by (Maximum - Minimum),
             when the control is dragged a full screen length.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.NumericUpDown
             * @function DragSpeed
             * @type number
             */
            DragSpeed: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.Controls.NumericUpDown.DragSpeedProperty), System.Double), System.Double));
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Controls.NumericUpDown.DragSpeedProperty, Bridge.box(value, System.Double, System.Double.format, System.Double.getHashCode));
                }
            },
            /**
             * Gets or sets a value added or subtracted from the value property. The default values is one.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.NumericUpDown
             * @function Increment
             * @type number
             */
            Increment: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.Controls.NumericUpDown.IncrementProperty), System.Double), System.Double));
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Controls.NumericUpDown.IncrementProperty, Bridge.box(value, System.Double, System.Double.format, System.Double.getHashCode));
                }
            }
        },
        ctors: {
            /**
             * Initializes a new instance of the {@link } class.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Controls.NumericUpDown
             * @memberof Windows.UI.Xaml.Controls.NumericUpDown
             * @return  {void}
             */
            ctor: function () {
                this.$initialize();
                Windows.UI.Xaml.Controls.Primitives.RangeBase.ctor.call(this);
                this.DefaultStyleKey = Windows.UI.Xaml.Controls.NumericUpDown;
                this.addLoaded(Bridge.fn.cacheBind(this, this.OnLoaded));
                this.addUnloaded(Bridge.fn.cacheBind(this, this.OnUnloaded));
            }
        },
        methods: {
            /**
             * Provides derived classes an opportunity to handle changes
             to the ValueBarVisibility property.
             *
             * @instance
             * @private
             * @this Windows.UI.Xaml.Controls.NumericUpDown
             * @memberof Windows.UI.Xaml.Controls.NumericUpDown
             * @return  {void}
             */
            OnValueBarVisibilityChanged: function () {
                this.UpdateValueBar();
            },
            /**
             * Provides derived classes an opportunity to handle changes
             to the IsReadOnly property.
             *
             * @instance
             * @private
             * @this Windows.UI.Xaml.Controls.NumericUpDown
             * @memberof Windows.UI.Xaml.Controls.NumericUpDown
             * @return  {void}
             */
            OnIsReadOnlyChanged: function () {
                this.UpdateIsReadOnlyDependants();
            },
            OnLoaded: function (sender, e) {
                if (this._dragOverlay != null) {
                    Windows.UI.Xaml.Window.Current.addPointerReleased(Bridge.fn.cacheBind(this, this.CoreWindowOnPointerReleased));
                }
            },
            OnUnloaded: function (sender, e) {
                Windows.UI.Xaml.Window.Current.removePointerReleased(Bridge.fn.cacheBind(this, this.CoreWindowOnPointerReleased));
            },
            /**
             * Invoked whenever application code or internal processes (such as a rebuilding layout pass) call ApplyTemplate. In simplest terms, this means the method is called just before a UI element displays in your app. Override this method to influence the default post-template logic of a class.
             *
             * @instance
             * @protected
             * @override
             * @this Windows.UI.Xaml.Controls.NumericUpDown
             * @memberof Windows.UI.Xaml.Controls.NumericUpDown
             * @return  {void}
             */
            OnApplyTemplate: function () {
                Windows.UI.Xaml.Controls.Primitives.RangeBase.prototype.OnApplyTemplate.call(this);


                this.addGotFocus(Bridge.fn.cacheBind(this, this.OnGotFocus$1));
                this.addLostFocus(Bridge.fn.cacheBind(this, this.OnLostFocus$1));
                this._valueTextBox = Bridge.as(this.GetTemplateChild(Windows.UI.Xaml.Controls.NumericUpDown.ValueTextBoxName), Windows.UI.Xaml.Controls.Primitives.UpDownTextBox);
                this._dragOverlay = Bridge.as(this.GetTemplateChild(Windows.UI.Xaml.Controls.NumericUpDown.DragOverlayName), Windows.UI.Xaml.UIElement);
                this._decrementButton = Bridge.as(this.GetTemplateChild(Windows.UI.Xaml.Controls.NumericUpDown.DecrementButtonName), Windows.UI.Xaml.Controls.Primitives.RepeatButton);
                this._incrementButton = Bridge.as(this.GetTemplateChild(Windows.UI.Xaml.Controls.NumericUpDown.IncrementButtonName), Windows.UI.Xaml.Controls.Primitives.RepeatButton);
                this._valueBar = Bridge.as(this.GetTemplateChild(Windows.UI.Xaml.Controls.NumericUpDown.ValueBarName), Windows.UI.Xaml.FrameworkElement);

                if (this._valueTextBox != null) {
                    this._valueTextBox.addLostFocus(Bridge.fn.cacheBind(this, this.OnValueTextBoxLostFocus));
                    this._valueTextBox.addGotFocus(Bridge.fn.cacheBind(this, this.OnValueTextBoxGotFocus));
                    this._valueTextBox.Text = System.Double.format(this.Value);
                    this._valueTextBox.addTextChanged(Bridge.fn.cacheBind(this, this.OnValueTextBoxTextChanged));
                    this._valueTextBox.addKeyDown(Bridge.fn.cacheBind(this, this.OnValueTextBoxKeyDown));
                    this._valueTextBox.addUpPressed(Bridge.fn.bind(this, function (s, e) {
                        this.DoIncrement();
                    }));
                    this._valueTextBox.addDownPressed(Bridge.fn.bind(this, function (s, e) {
                        this.DoDecrement();
                    }));
                }

                if (this._dragOverlay != null) {
                    this._dragOverlay.addPointerPressed(Bridge.fn.cacheBind(this, this.OnDragOverlayPointerPressed));
                    this._dragOverlay.addPointerReleased(Bridge.fn.cacheBind(this, this.OnDragOverlayPointerReleased));
                    this._dragOverlay.addPointerCaptureLost(Bridge.fn.cacheBind(this, this.OnDragOverlayPointerCaptureLost));
                }

                if (this._decrementButton != null) {
                    this._decrementButton.addClick(Bridge.fn.cacheBind(this, this.OnDecrementButtonClick));
                }

                if (this._incrementButton != null) {
                    this._incrementButton.addClick(Bridge.fn.cacheBind(this, this.OnIncrementButtonClick));
                }

                if (this._valueBar != null) {
                    this._valueBar.addSizeChanged(Bridge.fn.cacheBind(this, this.OnValueBarSizeChanged));

                    this.UpdateValueBar();
                }

                this.UpdateIsReadOnlyDependants();
                this.SetValidIncrementDirection();
            },
            OnLostFocus$1: function (sender, routedEventArgs) {
                this._hasFocus = false;
            },
            OnGotFocus$1: function (sender, routedEventArgs) {
                this._hasFocus = true;
            },
            OnValueTextBoxTextChanged: function (sender, textChangedEventArgs) {
                this.UpdateValueFromText();
            },
            OnValueTextBoxKeyDown: function (sender, e) {
                if (e.Key === Windows.System.VirtualKey.Enter) {
                    if (this.UpdateValueFromText()) {
                        this.UpdateValueText();
                        this._valueTextBox.SelectAll();
                        e.Handled = true;
                    }
                }
            },
            UpdateValueFromText: function () {
                if (this._isChangingTextWithCode) {
                    return false;
                }

                var val = { };

                if (System.Double.tryParse(this._valueTextBox.Text, null, val)) {
                    this._isChangingValueWithCode = true;
                    this.SetValueAndUpdateValidDirections(val.v);
                    this._isChangingValueWithCode = false;

                    return true;
                }

                return false;
            },
            OnDecrementButtonClick: function (sender, routedEventArgs) {
                this.DoDecrement();
            },
            OnIncrementButtonClick: function (sender, routedEventArgs) {
                this.DoIncrement();
            },
            /**
             * Decrements the value by Increment.
             *
             * @instance
             * @protected
             * @this Windows.UI.Xaml.Controls.NumericUpDown
             * @memberof Windows.UI.Xaml.Controls.NumericUpDown
             * @return  {boolean}        <pre><code>true</code></pre> if the value was decremented by exactly <pre><code>Increment</code></pre>; <pre><code>false</code></pre> if it was constrained.
             */
            DoDecrement: function () {
                return this.SetValueAndUpdateValidDirections(Bridge.Math.round(this.Value - this.Increment, 5, 6));
            },
            DoIncrement: function () {
                return this.SetValueAndUpdateValidDirections(Bridge.Math.round(this.Value + this.Increment, 5, 6));
            },
            OnValueTextBoxGotFocus: function (sender, routedEventArgs) {
                if (this._dragOverlay != null) {
                    this._dragOverlay.IsHitTestVisible = false;
                }

                this._valueTextBox.SelectAll();
            },
            OnValueTextBoxLostFocus: function (sender, routedEventArgs) {
                var val = { };
                if (this._valueTextBox != null && (!System.Double.tryParse(this._valueTextBox.Text, null, val) || val.v !== this.Value)) {
                    this.UpdateValueText();
                }
            },
            OnDragOverlayPointerPressed: function (sender, e) {
                this._totalDeltaX = 0;
                this._totalDeltaY = 0;

            },
            CoreWindowOnPointerReleased: function (sender, args) {
                if (this._isDragUpdated) {
                    args.Handled = true;
                    this.ResumeValueTextBoxTabStopAsync();
                }
            },
            ResumeValueTextBoxTabStopAsync: function () {
                var $step = 0,
                    $task1, 
                    $jumpFromFinally, 
                    $asyncBody = Bridge.fn.bind(this, function () {
                        for (;;) {
                            $step = System.Array.min([0,1], $step);
                            switch ($step) {
                                case 0: {
                                    $task1 = System.Threading.Tasks.Task.delay(100);
                                    $step = 1;
                                    if ($task1.isCompleted()) {
                                        continue;
                                    }
                                    $task1.continue($asyncBody);
                                    return;
                                }
                                case 1: {
                                    $task1.getAwaitedResult();
                                    return;
                                }
                                default: {
                                    return;
                                }
                            }
                        }
                    }, arguments);

                $asyncBody();
            },
            OnDragOverlayPointerReleased: function (sender, args) {
                this.EndDragging(args);
            },
            OnDragOverlayPointerCaptureLost: function (sender, args) {
                this.EndDragging(args);
            },
            EndDragging: function (args) {
                if (this._isDragUpdated) {
                    if (args != null) {
                        args.Handled = true;
                    }

                    this.ResumeValueTextBoxTabStopAsync();
                }
            },
            ApplyManipulationDelta: function (delta) {
                if (Bridge.Int.sign(delta) === Bridge.Int.sign(this._unusedManipulationDelta)) {
                    this._unusedManipulationDelta += delta;
                } else {
                    this._unusedManipulationDelta = delta;
                }

                if (this._unusedManipulationDelta <= 0 && this.Value === this.Minimum) {
                    this._unusedManipulationDelta = 0;
                    return;
                }

                if (this._unusedManipulationDelta >= 0 && this.Value === this.Maximum) {
                    this._unusedManipulationDelta = 0;
                    return;
                }

                var smallerScreenDimension;

                if (Windows.UI.Xaml.Window.Current != null) {
                    smallerScreenDimension = Math.min(Windows.UI.Xaml.Window.Current.Bounds.Width, Windows.UI.Xaml.Window.Current.Bounds.Height);
                } else {
                    smallerScreenDimension = 768;
                }

                var speed = this.DragSpeed;

                if (isNaN(speed) || (Math.abs(speed) === Number.POSITIVE_INFINITY)) {
                    speed = this.Maximum - this.Minimum;
                }

                if (isNaN(speed) || (Math.abs(speed) === Number.POSITIVE_INFINITY)) {
                    speed = System.Double.max;
                }

                var screenAdjustedDelta = speed * this._unusedManipulationDelta / smallerScreenDimension;
                this.SetValueAndUpdateValidDirections(this.Value + screenAdjustedDelta);
                this._unusedManipulationDelta = 0;
            },
            /**
             * Fires the ValueChanged routed event.
             *
             * @instance
             * @protected
             * @override
             * @this Windows.UI.Xaml.Controls.NumericUpDown
             * @memberof Windows.UI.Xaml.Controls.NumericUpDown
             * @param   {number}    oldValue    Old value of the Value property.
             * @param   {number}    newValue    New value of the Value property.
             * @return  {void}
             */
            OnValueChanged: function (oldValue, newValue) {
                Windows.UI.Xaml.Controls.Primitives.RangeBase.prototype.OnValueChanged.call(this, oldValue, newValue);

                this.UpdateValueBar();

                if (!this._isChangingValueWithCode) {
                    this.UpdateValueText();
                }
            },
            UpdateValueBar: function () {
                if (this._valueBar == null) {
                    return;
                }

                var effectiveValueBarVisibility = this.ValueBarVisibility;

                if (effectiveValueBarVisibility === Windows.UI.Xaml.Controls.NumericUpDownValueBarVisibility.Collapsed) {
                    this._valueBar.Visibility = Windows.UI.Xaml.Visibility.Collapsed;

                    return;
                }


            },
            OnValueBarSizeChanged: function (sender, sizeChangedEventArgs) {
                this.UpdateValueBar();
            },
            UpdateValueText: function () {
                if (this._valueTextBox != null) {
                    this._isChangingTextWithCode = true;
                    this._valueTextBox.Text = System.Double.format(this.Value);
                    this._isChangingTextWithCode = false;
                }
            },
            UpdateIsReadOnlyDependants: function () {
                if (this._decrementButton != null) {
                    this._decrementButton.Visibility = this.IsReadOnly ? Windows.UI.Xaml.Visibility.Collapsed : Windows.UI.Xaml.Visibility.Visible;
                }

                if (this._incrementButton != null) {
                    this._incrementButton.Visibility = this.IsReadOnly ? Windows.UI.Xaml.Visibility.Collapsed : Windows.UI.Xaml.Visibility.Visible;
                }
            },
            SetValueAndUpdateValidDirections: function (value) {
                var oldValue = this.Value;
                this.Value = value;
                this.SetValidIncrementDirection();

                return Math.abs(this.Value - oldValue) > Windows.UI.Xaml.Controls.NumericUpDown.Epsilon;
            },
            SetValidIncrementDirection: function () {
                Windows.UI.Xaml.VisualStateManager.GoToState(this, this.Value < this.Maximum ? "IncrementEnabled" : "IncrementDisabled", true);
                Windows.UI.Xaml.VisualStateManager.GoToState(this, this.Value > this.Minimum ? "DecrementEnabled" : "DecrementDisabled", true);
            }
        }
    });

    /**
     * Encapsulates a page of content that can be navigated to.
     *
     * @public
     * @class Windows.UI.Xaml.Controls.Page
     * @augments Windows.UI.Xaml.Controls.UserControl
     */
    Bridge.define("Windows.UI.Xaml.Controls.Page", {
        inherits: [Windows.UI.Xaml.Controls.UserControl],
        methods: {
            /**
             * Invoked when the Page is loaded and becomes the current source of a parent
             Frame.
             *
             * @instance
             * @protected
             * @this Windows.UI.Xaml.Controls.Page
             * @memberof Windows.UI.Xaml.Controls.Page
             * @param   {Windows.UI.Xaml.Navigation.NavigationEventArgs}    e    Event data that can be examined by overriding code. The event data is representative
             of the pending navigation that will load the current Page. Usually the most
             relevant property to examine is Parameter.
             * @return  {void}
             */
            OnNavigatedTo: function (e) { },
            CreateDomElement: function (parentRef, domElementWhereToPlaceChildren) {
                var div = CSHTML5.Internal.INTERNAL_HtmlDomManager.CreateDomElementAndAppendIt("div", parentRef, this);
                var divStyle = CSHTML5.Internal.INTERNAL_HtmlDomManager.GetDomElementStyleForModification(div);
                domElementWhereToPlaceChildren.v = div;

                divStyle.width = "100%";
                divStyle.height = "100%";

                return div;
            }
        }
    });

    /**
     * @memberof Windows.UI.Xaml.Controls.Primitives
     * @callback Windows.UI.Xaml.Controls.Primitives.ScrollEventHandler
     * @param   {System.Object}                                          sender    
     * @param   {Windows.UI.Xaml.Controls.Primitives.ScrollEventArgs}    e
     * @return  {void}
     */

    /**
     * Represents a control that provides a scroll bar that has a sliding Thumb whose position corresponds to a value.
     *
     * @public
     * @class Windows.UI.Xaml.Controls.Primitives.ScrollBar
     * @augments Windows.UI.Xaml.Controls.Primitives.RangeBase
     */
    Bridge.define("Windows.UI.Xaml.Controls.Primitives.ScrollBar", {
        inherits: [Windows.UI.Xaml.Controls.Primitives.RangeBase],
        statics: {
            fields: {
                MINIMUM_THUMB_SIZE: 0,
                /**
                 * Identifies the Orientation dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Controls.Primitives.ScrollBar
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                OrientationProperty: null,
                /**
                 * Identifies the ViewportSize dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Controls.Primitives.ScrollBar
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                ViewportSizeProperty: null
            },
            ctors: {
                init: function () {
                    this.MINIMUM_THUMB_SIZE = 16.0;
                    this.OrientationProperty = Windows.UI.Xaml.DependencyProperty.Register("Orientation", Windows.UI.Xaml.Controls.Orientation, Windows.UI.Xaml.Controls.Primitives.ScrollBar, new Windows.UI.Xaml.PropertyMetadata.$ctor2(Bridge.box(Windows.UI.Xaml.Controls.Orientation.Vertical, Windows.UI.Xaml.Controls.Orientation, System.Enum.toStringFn(Windows.UI.Xaml.Controls.Orientation)), Windows.UI.Xaml.Controls.Primitives.ScrollBar.Orientation_Changed));
                    this.ViewportSizeProperty = Windows.UI.Xaml.DependencyProperty.Register("ViewportSize", System.Double, Windows.UI.Xaml.Controls.Primitives.ScrollBar, new Windows.UI.Xaml.PropertyMetadata.$ctor2(Bridge.box(0.0, System.Double, System.Double.format, System.Double.getHashCode), Windows.UI.Xaml.Controls.Primitives.ScrollBar.ViewportSize_Changed));
                }
            },
            methods: {
                Orientation_Changed: function (d, e) {
                    var scrollBar = Bridge.cast(d, Windows.UI.Xaml.Controls.Primitives.ScrollBar);
                    scrollBar._controlWasProperlyDrawn = scrollBar.TryUpdateSizeAndPositionOfUIElements();
                },
                ViewportSize_Changed: function (d, e) {
                    var scrollBar = Bridge.cast(d, Windows.UI.Xaml.Controls.Primitives.ScrollBar);
                    scrollBar._controlWasProperlyDrawn = scrollBar.TryUpdateSizeAndPositionOfUIElements();
                }
            }
        },
        fields: {
            _smallDecreaseButtonSize: 0,
            _smallIncreaseButtonSize: 0,
            _controlWasProperlyDrawn: false,
            _valueWithoutCoercion: 0,
            _horizontalRoot: null,
            _horizontalThumb: null,
            _horizontalSmallDecrease: null,
            _horizontalSmallIncrease: null,
            _horizontalLargeDecrease: null,
            _horizontalLargeIncrease: null,
            _verticalRoot: null,
            _verticalThumb: null,
            _verticalSmallDecrease: null,
            _verticalSmallIncrease: null,
            _verticalLargeDecrease: null,
            _verticalLargeIncrease: null
        },
        events: {
            /**
             * Occurs one or more times as content scrolls in a ScrollBar when the user moves the Thumb by using the mouse.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Controls.Primitives.ScrollBar
             * @memberof Windows.UI.Xaml.Controls.Primitives.ScrollBar
             * @function addScroll
             * @param   {Windows.UI.Xaml.Controls.Primitives.ScrollEventHandler}    value
             * @return  {void}
             */
            /**
             * Occurs one or more times as content scrolls in a ScrollBar when the user moves the Thumb by using the mouse.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Controls.Primitives.ScrollBar
             * @memberof Windows.UI.Xaml.Controls.Primitives.ScrollBar
             * @function removeScroll
             * @param   {Windows.UI.Xaml.Controls.Primitives.ScrollEventHandler}    value
             * @return  {void}
             */
            Scroll: null
        },
        props: {
            /**
             * Gets or sets a value that indicates whether the ScrollBar is displayed horizontally or vertically.
             The default is Horizontal.  Specific control templates might change this value, which would cause the templated value to be the apparent runtime default.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.Primitives.ScrollBar
             * @function Orientation
             * @type Windows.UI.Xaml.Controls.Orientation
             */
            Orientation: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.Controls.Primitives.ScrollBar.OrientationProperty), Windows.UI.Xaml.Controls.Orientation), System.Int32));
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Controls.Primitives.ScrollBar.OrientationProperty, Bridge.box(value, Windows.UI.Xaml.Controls.Orientation, System.Enum.toStringFn(Windows.UI.Xaml.Controls.Orientation)));
                }
            },
            /**
             * Gets or sets the amount of the scrollable content that is currently visible. The default is 0.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.Primitives.ScrollBar
             * @function ViewportSize
             * @type number
             */
            ViewportSize: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.Controls.Primitives.ScrollBar.ViewportSizeProperty), System.Double), System.Double));
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Controls.Primitives.ScrollBar.ViewportSizeProperty, Bridge.box(value, System.Double, System.Double.format, System.Double.getHashCode));
                }
            }
        },
        ctors: {
            /**
             * Initializes a new instance of the ScrollBar class.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Controls.Primitives.ScrollBar
             * @memberof Windows.UI.Xaml.Controls.Primitives.ScrollBar
             * @return  {void}
             */
            ctor: function () {
                this.$initialize();
                Windows.UI.Xaml.Controls.Primitives.RangeBase.ctor.call(this);
                this.DefaultStyleKey = Windows.UI.Xaml.Controls.Primitives.ScrollBar;

                this.addUnloaded(Bridge.fn.cacheBind(this, this.ScrollBar_Unloaded));
                this.addSizeChanged(Bridge.fn.cacheBind(this, this.ScrollBar_SizeChanged));
            }
        },
        methods: {
            ScrollBar_SizeChanged: function (sender, e) {

                if (this.TryUpdateSizeAndPositionOfUIElements()) {
                    this._controlWasProperlyDrawn = true;
                }
            },
            ScrollBar_Unloaded: function (sender, e) {
                this._controlWasProperlyDrawn = false;
            },
            OnApplyTemplate: function () {
                Windows.UI.Xaml.Controls.Primitives.RangeBase.prototype.OnApplyTemplate.call(this);


                this._horizontalRoot = Bridge.as(this.GetTemplateChild("HorizontalRoot"), Windows.UI.Xaml.Controls.Canvas);
                this._horizontalThumb = Bridge.as(this.GetTemplateChild("HorizontalThumb"), Windows.UI.Xaml.Controls.Primitives.Thumb);
                this._horizontalSmallDecrease = Bridge.as(this.GetTemplateChild("HorizontalSmallDecrease"), Windows.UI.Xaml.Controls.Button);
                this._horizontalSmallIncrease = Bridge.as(this.GetTemplateChild("HorizontalSmallIncrease"), Windows.UI.Xaml.Controls.Button);
                this._horizontalLargeDecrease = Bridge.as(this.GetTemplateChild("HorizontalLargeDecrease"), Windows.UI.Xaml.Controls.Button);
                this._horizontalLargeIncrease = Bridge.as(this.GetTemplateChild("HorizontalLargeIncrease"), Windows.UI.Xaml.Controls.Button);

                this._verticalRoot = Bridge.as(this.GetTemplateChild("VerticalRoot"), Windows.UI.Xaml.Controls.Canvas);
                this._verticalThumb = Bridge.as(this.GetTemplateChild("VerticalThumb"), Windows.UI.Xaml.Controls.Primitives.Thumb);
                this._verticalSmallDecrease = Bridge.as(this.GetTemplateChild("VerticalSmallDecrease"), Windows.UI.Xaml.Controls.Button);
                this._verticalSmallIncrease = Bridge.as(this.GetTemplateChild("VerticalSmallIncrease"), Windows.UI.Xaml.Controls.Button);
                this._verticalLargeDecrease = Bridge.as(this.GetTemplateChild("VerticalLargeDecrease"), Windows.UI.Xaml.Controls.Button);
                this._verticalLargeIncrease = Bridge.as(this.GetTemplateChild("VerticalLargeIncrease"), Windows.UI.Xaml.Controls.Button);


                if (this._horizontalThumb != null) {
                    this._horizontalThumb.addDragStarted(Bridge.fn.cacheBind(this, this.HorizontalThumb_DragStarted));
                    this._horizontalThumb.addDragDelta(Bridge.fn.cacheBind(this, this.HorizontalThumb_DragDelta));
                    this._horizontalThumb.addDragCompleted(Bridge.fn.cacheBind(this, this.HorizontalThumb_DragCompleted));
                }
                if (this._verticalThumb != null) {
                    this._verticalThumb.addDragStarted(Bridge.fn.cacheBind(this, this.VerticalThumb_DragStarted));
                    this._verticalThumb.addDragDelta(Bridge.fn.cacheBind(this, this.VerticalThumb_DragDelta));
                    this._verticalThumb.addDragCompleted(Bridge.fn.cacheBind(this, this.VerticalThumb_DragCompleted));
                }
                if (this._horizontalSmallDecrease != null) {
                    this._horizontalSmallDecrease.addClick(Bridge.fn.cacheBind(this, this.SmallDecrease_Click));
                }
                if (this._verticalSmallDecrease != null) {
                    this._verticalSmallDecrease.addClick(Bridge.fn.cacheBind(this, this.SmallDecrease_Click));
                }
                if (this._horizontalLargeDecrease != null) {
                    this._horizontalLargeDecrease.addClick(Bridge.fn.cacheBind(this, this.LargeDecrease_Click));
                }
                if (this._verticalLargeDecrease != null) {
                    this._verticalLargeDecrease.addClick(Bridge.fn.cacheBind(this, this.LargeDecrease_Click));
                }
                if (this._horizontalSmallIncrease != null) {
                    this._horizontalSmallIncrease.addClick(Bridge.fn.cacheBind(this, this.SmallIncrease_Click));
                }
                if (this._verticalSmallIncrease != null) {
                    this._verticalSmallIncrease.addClick(Bridge.fn.cacheBind(this, this.SmallIncrease_Click));
                }
                if (this._horizontalLargeIncrease != null) {
                    this._horizontalLargeIncrease.addClick(Bridge.fn.cacheBind(this, this.LargeIncrease_Click));
                }
                if (this._verticalLargeIncrease != null) {
                    this._verticalLargeIncrease.addClick(Bridge.fn.cacheBind(this, this.LargeIncrease_Click));
                }



                if (this._verticalRoot != null) {
                    this._verticalRoot.Visibility = (this.Orientation === Windows.UI.Xaml.Controls.Orientation.Vertical ? Windows.UI.Xaml.Visibility.Visible : Windows.UI.Xaml.Visibility.Collapsed);
                }
                if (this._horizontalRoot != null) {
                    this._horizontalRoot.Visibility = (this.Orientation === Windows.UI.Xaml.Controls.Orientation.Horizontal ? Windows.UI.Xaml.Visibility.Visible : Windows.UI.Xaml.Visibility.Collapsed);
                }
            },
            OnValueChanged: function (oldValue, newValue) {
                Windows.UI.Xaml.Controls.Primitives.RangeBase.prototype.OnValueChanged.call(this, oldValue, newValue);

                this.UpdateThumbPositionAndSize();
            },
            OnMaximumChanged: function (oldMaximum, newMaximum) {
                Windows.UI.Xaml.Controls.Primitives.RangeBase.prototype.OnMaximumChanged.call(this, oldMaximum, newMaximum);

                this.UpdateThumbPositionAndSize();
            },
            OnMinimumChanged: function (oldMinimum, newMinimum) {
                Windows.UI.Xaml.Controls.Primitives.RangeBase.prototype.OnMinimumChanged.call(this, oldMinimum, newMinimum);

                this.UpdateThumbPositionAndSize();
            },
            LargeIncrease_Click: function (sender, e) {
                this.OnClickOnSmallOrLargeDecreaseIncreaseButtons(this.LargeChange, Windows.UI.Xaml.Controls.Primitives.ScrollEventType.LargeIncrement);
            },
            SmallIncrease_Click: function (sender, e) {
                this.OnClickOnSmallOrLargeDecreaseIncreaseButtons(this.SmallChange, Windows.UI.Xaml.Controls.Primitives.ScrollEventType.SmallIncrement);
            },
            LargeDecrease_Click: function (sender, e) {
                this.OnClickOnSmallOrLargeDecreaseIncreaseButtons(-this.LargeChange, Windows.UI.Xaml.Controls.Primitives.ScrollEventType.LargeDecrement);
            },
            SmallDecrease_Click: function (sender, e) {
                this.OnClickOnSmallOrLargeDecreaseIncreaseButtons(-this.SmallChange, Windows.UI.Xaml.Controls.Primitives.ScrollEventType.SmallDecrement);
            },
            VerticalThumb_DragCompleted: function (sender, e) {
                this.OnDragCompleted();
            },
            HorizontalThumb_DragCompleted: function (sender, e) {
                this.OnDragCompleted();
            },
            VerticalThumb_DragStarted: function (sender, e) {
                this.OnDragStarted();
            },
            HorizontalThumb_DragStarted: function (sender, e) {
                this.OnDragStarted();
            },
            VerticalThumb_DragDelta: function (sender, e) {
                this.ChangeValueBasedOnPointerMovement(e.VerticalChange);
            },
            HorizontalThumb_DragDelta: function (sender, e) {
                this.ChangeValueBasedOnPointerMovement(e.HorizontalChange);
            },
            OnClickOnSmallOrLargeDecreaseIncreaseButtons: function (amountByWhichToIncreaseOrDecreaseTheValue, scrollEventType) {
                var newValue = this.CoerceValue$1(this.Value + amountByWhichToIncreaseOrDecreaseTheValue);
                if (newValue !== this.Value) {
                    this.SetCurrentValue(Windows.UI.Xaml.Controls.Primitives.RangeBase.ValueProperty, Bridge.box(newValue, System.Double, System.Double.format, System.Double.getHashCode));
                    if (!Bridge.staticEquals(this.Scroll, null)) {
                        this.Scroll(this, new Windows.UI.Xaml.Controls.Primitives.ScrollEventArgs(newValue, scrollEventType));
                    }
                }
            },
            OnDragStarted: function () {
                this._valueWithoutCoercion = this.Value;

                if (this._verticalLargeDecrease != null) {
                    this._verticalLargeDecrease.IsHitTestVisible = false;
                }
                if (this._verticalLargeIncrease != null) {
                    this._verticalLargeIncrease.IsHitTestVisible = false;
                }
                if (this._horizontalLargeDecrease != null) {
                    this._horizontalLargeDecrease.IsHitTestVisible = false;
                }
                if (this._horizontalLargeIncrease != null) {
                    this._horizontalLargeIncrease.IsHitTestVisible = false;
                }
            },
            OnDragCompleted: function () {
                if (this._verticalLargeDecrease != null) {
                    this._verticalLargeDecrease.IsHitTestVisible = true;
                }
                if (this._verticalLargeIncrease != null) {
                    this._verticalLargeIncrease.IsHitTestVisible = true;
                }
                if (this._horizontalLargeDecrease != null) {
                    this._horizontalLargeDecrease.IsHitTestVisible = true;
                }
                if (this._horizontalLargeIncrease != null) {
                    this._horizontalLargeIncrease.IsHitTestVisible = true;
                }

                if (!Bridge.staticEquals(this.Scroll, null)) {
                    this.Scroll(this, new Windows.UI.Xaml.Controls.Primitives.ScrollEventArgs(this.Value, Windows.UI.Xaml.Controls.Primitives.ScrollEventType.EndScroll));
                }
            },
            ChangeValueBasedOnPointerMovement: function (pointerMovementInPixels) {

                var totalControlSize = { v : new Windows.Foundation.Size() };
                if (this.CheckIfControlWasRenderedProperlyAndGetCurrentControlSize(totalControlSize)) {
                    var totalControlSizeInMainDirection = (this.Orientation === Windows.UI.Xaml.Controls.Orientation.Vertical ? totalControlSize.v.Height : totalControlSize.v.Width);
                    var trackSize = { };
                    var thumbSize = { };
                    var scrollableSize = { };
                    var minValue = { };
                    var maxValue = { };
                    this.CalculateThumbTrackAndScrollableSizes(totalControlSizeInMainDirection, trackSize, thumbSize, scrollableSize, minValue, maxValue);
                    var maxMinusMin = maxValue.v - minValue.v;

                    var valueDelta;
                    if (scrollableSize.v > 0 && maxMinusMin > 0) {
                        valueDelta = (pointerMovementInPixels * maxMinusMin) / scrollableSize.v;
                    } else {
                        valueDelta = 0;
                    }

                    this._valueWithoutCoercion += valueDelta;
                    var newValue = this.CoerceValue$1(this._valueWithoutCoercion);

                    if (newValue !== this.Value) {
                        this.SetCurrentValue(Windows.UI.Xaml.Controls.Primitives.RangeBase.ValueProperty, Bridge.box(newValue, System.Double, System.Double.format, System.Double.getHashCode));

                        this.UpdateThumbPositionAndSize$1(totalControlSizeInMainDirection);

                        if (!Bridge.staticEquals(this.Scroll, null)) {
                            this.Scroll(this, new Windows.UI.Xaml.Controls.Primitives.ScrollEventArgs(newValue, Windows.UI.Xaml.Controls.Primitives.ScrollEventType.ThumbTrack));
                        }
                    }
                }
            },
            CoerceValue$1: function (value) {

                var minValue = this.Minimum;
                var maxValue = this.Maximum;
                if (value <= minValue) {
                    value = minValue;
                }
                if (value >= maxValue) {
                    value = maxValue;
                }
                return value;
            },
            TryUpdateSizeAndPositionOfUIElements: function () {
                var $t, $t1, $t2, $t3, $t4, $t5, $t6, $t7;

                var totalControlSize = { v : new Windows.Foundation.Size() };
                if (this.IsLoaded && this.TryGetCurrentControlSize(totalControlSize)) {
                    if (this.Orientation === Windows.UI.Xaml.Controls.Orientation.Vertical) {

                        if (this._verticalRoot != null && this._verticalThumb != null && this._verticalSmallDecrease != null && this._verticalSmallIncrease != null && this._verticalLargeDecrease != null && this._verticalLargeIncrease != null) {
                            this._verticalThumb.Width = ($t = ($t1 = ($t2 = ($t3 = totalControlSize.v.Width, this._verticalLargeIncrease.Width = $t3, $t3), this._verticalLargeDecrease.Width = $t2, $t2), this._verticalSmallIncrease.Width = $t1, $t1), this._verticalSmallDecrease.Width = $t, $t);

                            var buttonsHeight = totalControlSize.v.Width;

                            this._smallDecreaseButtonSize = buttonsHeight;
                            this._smallIncreaseButtonSize = buttonsHeight;

                            this._verticalSmallDecrease.Height = this._smallDecreaseButtonSize;
                            this._verticalSmallIncrease.Height = this._smallIncreaseButtonSize;

                            Windows.UI.Xaml.Controls.Canvas.SetTop(this._verticalSmallDecrease, 0.0);
                            Windows.UI.Xaml.Controls.Canvas.SetTop(this._verticalSmallIncrease, totalControlSize.v.Height - buttonsHeight);

                            this.UpdateThumbPositionAndSize$1(totalControlSize.v.Height);

                            return true;
                        }
                    } else {

                        if (this._horizontalRoot != null && this._horizontalThumb != null && this._horizontalSmallDecrease != null && this._horizontalSmallIncrease != null && this._horizontalLargeDecrease != null && this._horizontalLargeIncrease != null) {
                            this._horizontalThumb.Height = ($t4 = ($t5 = ($t6 = ($t7 = totalControlSize.v.Height, this._horizontalLargeIncrease.Height = $t7, $t7), this._horizontalLargeDecrease.Height = $t6, $t6), this._horizontalSmallIncrease.Height = $t5, $t5), this._horizontalSmallDecrease.Height = $t4, $t4);

                            var buttonsWidth = totalControlSize.v.Height;

                            this._smallDecreaseButtonSize = buttonsWidth;
                            this._smallIncreaseButtonSize = buttonsWidth;

                            this._horizontalSmallDecrease.Width = this._smallDecreaseButtonSize;
                            this._horizontalSmallIncrease.Width = this._smallIncreaseButtonSize;

                            Windows.UI.Xaml.Controls.Canvas.SetLeft(this._horizontalSmallDecrease, 0.0);
                            Windows.UI.Xaml.Controls.Canvas.SetLeft(this._horizontalSmallIncrease, totalControlSize.v.Width - buttonsWidth);

                            this.UpdateThumbPositionAndSize$1(totalControlSize.v.Width);

                            return true;
                        }
                    }
                }

                return false;
            },
            CheckIfControlWasRenderedProperlyAndGetCurrentControlSize: function (totalControlSize) {
                if (this._controlWasProperlyDrawn) {
                    return this.TryGetCurrentControlSize(totalControlSize);
                } else {
                    totalControlSize.v = new Windows.Foundation.Size.$ctor1(0.0, 0.0);
                    return false;
                }
            },
            TryGetCurrentControlSize: function (totalControlSize) {
                totalControlSize.v = new Windows.Foundation.Size.$ctor1(0.0, 0.0);
                if (this._verticalRoot != null && this._horizontalRoot != null) {
                    var actualSize = new Windows.Foundation.Size();
                    if (this.Orientation === Windows.UI.Xaml.Controls.Orientation.Vertical) {
                        actualSize = this._verticalRoot.INTERNAL_GetActualWidthAndHeight();
                    } else {
                        actualSize = this._horizontalRoot.INTERNAL_GetActualWidthAndHeight();
                    }
                    var width = !isNaN(actualSize.Width) ? actualSize.Width : this.Width;
                    var height = !isNaN(actualSize.Height) ? actualSize.Height : this.Height;

                    if (!isNaN(width) && !isNaN(height)) {
                        totalControlSize.v = new Windows.Foundation.Size.$ctor1(width, height);
                        return true;
                    }
                }
                return false;
            },
            CalculateThumbTrackAndScrollableSizes: function (totalControlSize, trackSize, thumbSize, scrollableSize, minValue, maxValue) {
                minValue.v = this.Minimum;
                maxValue.v = this.Maximum;
                var maxMinusMin = maxValue.v - minValue.v;

                trackSize.v = Math.max(0.0, totalControlSize - this._smallDecreaseButtonSize - this._smallIncreaseButtonSize);
                thumbSize.v = maxMinusMin > 0 ? (trackSize.v * this.ViewportSize / (maxMinusMin * 2)) : trackSize.v;
                thumbSize.v = Math.min(trackSize.v, Math.max(Windows.UI.Xaml.Controls.Primitives.ScrollBar.MINIMUM_THUMB_SIZE, thumbSize.v));
                scrollableSize.v = trackSize.v - thumbSize.v;
            },
            UpdateThumbPositionAndSize: function () {
                var totalControlSize = { v : new Windows.Foundation.Size() };
                if (this.CheckIfControlWasRenderedProperlyAndGetCurrentControlSize(totalControlSize)) {
                    var totalControlSizeInMainDirection = (this.Orientation === Windows.UI.Xaml.Controls.Orientation.Vertical ? totalControlSize.v.Height : totalControlSize.v.Width);
                    this.UpdateThumbPositionAndSize$1(totalControlSizeInMainDirection);
                }
            },
            UpdateThumbPositionAndSize$1: function (totalControlSize) {

                var value = this.Value;
                var trackSize = { };
                var thumbSize = { };
                var scrollableSize = { };
                var minValue = { };
                var maxValue = { };
                this.CalculateThumbTrackAndScrollableSizes(totalControlSize, trackSize, thumbSize, scrollableSize, minValue, maxValue);
                var maxMinusMin = maxValue.v - minValue.v;

                var thumbCenterPosition = this._smallDecreaseButtonSize + (maxMinusMin > 0 ? (thumbSize.v / 2) + (scrollableSize.v * (value - this.Minimum) / maxMinusMin) : trackSize.v / 2);
                var thumbTopLeftPosition = thumbCenterPosition - (thumbSize.v / 2);
                var thumbBottomRightPosition = thumbCenterPosition + (thumbSize.v / 2);

                if (this.Orientation === Windows.UI.Xaml.Controls.Orientation.Vertical) {
                    if (this._verticalThumb != null && this._verticalLargeDecrease != null && this._verticalLargeIncrease != null) {
                        this._verticalThumb.Height = thumbSize.v;
                        Windows.UI.Xaml.Controls.Canvas.SetTop(this._verticalThumb, thumbTopLeftPosition);

                        this._verticalLargeDecrease.Height = Math.max(1.0, thumbTopLeftPosition - this._smallDecreaseButtonSize);
                        Windows.UI.Xaml.Controls.Canvas.SetTop(this._verticalLargeDecrease, this._smallDecreaseButtonSize);

                        this._verticalLargeIncrease.Height = Math.max(1.0, totalControlSize - this._smallIncreaseButtonSize - thumbBottomRightPosition);
                        Windows.UI.Xaml.Controls.Canvas.SetTop(this._verticalLargeIncrease, thumbBottomRightPosition);
                    }
                } else {
                    if (this._horizontalThumb != null && this._horizontalLargeDecrease != null && this._horizontalLargeIncrease != null) {
                        this._horizontalThumb.Width = thumbSize.v;
                        Windows.UI.Xaml.Controls.Canvas.SetLeft(this._horizontalThumb, thumbTopLeftPosition);

                        this._horizontalLargeDecrease.Width = Math.max(1.0, thumbTopLeftPosition - this._smallDecreaseButtonSize);
                        Windows.UI.Xaml.Controls.Canvas.SetLeft(this._horizontalLargeDecrease, this._smallDecreaseButtonSize);

                        this._horizontalLargeIncrease.Width = Math.max(1.0, totalControlSize - this._smallIncreaseButtonSize - thumbBottomRightPosition);
                        Windows.UI.Xaml.Controls.Canvas.SetLeft(this._horizontalLargeIncrease, thumbBottomRightPosition);
                    }
                }
            }
        }
    });

    /**
     * Primitive control - TextBox with UpPressed and DownPressed events for use in a NumericUpDown
     to make up and down keys work to increment and decrement the values.
     *
     * @public
     * @class Windows.UI.Xaml.Controls.Primitives.UpDownTextBox
     * @augments Windows.UI.Xaml.Controls.TextBox
     */
    Bridge.define("Windows.UI.Xaml.Controls.Primitives.UpDownTextBox", {
        inherits: [Windows.UI.Xaml.Controls.TextBox],
        events: {
            /**
             * UpPressed event property.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Controls.Primitives.UpDownTextBox
             * @memberof Windows.UI.Xaml.Controls.Primitives.UpDownTextBox
             * @function addUpPressed
             * @param   {System.EventHandler}    value
             * @return  {void}
             */
            /**
             * UpPressed event property.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Controls.Primitives.UpDownTextBox
             * @memberof Windows.UI.Xaml.Controls.Primitives.UpDownTextBox
             * @function removeUpPressed
             * @param   {System.EventHandler}    value
             * @return  {void}
             */
            UpPressed: null,
            /**
             * DownPressed event property.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Controls.Primitives.UpDownTextBox
             * @memberof Windows.UI.Xaml.Controls.Primitives.UpDownTextBox
             * @function addDownPressed
             * @param   {System.EventHandler}    value
             * @return  {void}
             */
            /**
             * DownPressed event property.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Controls.Primitives.UpDownTextBox
             * @memberof Windows.UI.Xaml.Controls.Primitives.UpDownTextBox
             * @function removeDownPressed
             * @param   {System.EventHandler}    value
             * @return  {void}
             */
            DownPressed: null
        },
        methods: {
            /**
             * Raises UpPressed event.
             *
             * @instance
             * @private
             * @this Windows.UI.Xaml.Controls.Primitives.UpDownTextBox
             * @memberof Windows.UI.Xaml.Controls.Primitives.UpDownTextBox
             * @return  {void}
             */
            RaiseUpPressed: function () {
                var handler = this.UpPressed;

                if (!Bridge.staticEquals(handler, null)) {
                    var args = { };
                    handler(this, args);
                }
            },
            /**
             * Raises DownPressed event.
             *
             * @instance
             * @private
             * @this Windows.UI.Xaml.Controls.Primitives.UpDownTextBox
             * @memberof Windows.UI.Xaml.Controls.Primitives.UpDownTextBox
             * @return  {void}
             */
            RaiseDownPressed: function () {
                var handler = this.DownPressed;

                if (!Bridge.staticEquals(handler, null)) {
                    var args = { };
                    handler(this, args);
                }
            },
            /**
             * Called before the KeyDown event occurs.
             *
             * @instance
             * @protected
             * @override
             * @this Windows.UI.Xaml.Controls.Primitives.UpDownTextBox
             * @memberof Windows.UI.Xaml.Controls.Primitives.UpDownTextBox
             * @param   {Windows.UI.Xaml.Input.KeyRoutedEventArgs}    e    The data for the event.
             * @return  {void}
             */
            OnKeyDown: function (e) {
                if (e.Key === Windows.System.VirtualKey.Up) {
                    this.RaiseUpPressed();
                    e.Handled = true;
                    return;
                }

                if (e.Key === Windows.System.VirtualKey.Down) {
                    this.RaiseDownPressed();
                    e.Handled = true;
                    return;
                }

                Windows.UI.Xaml.Controls.TextBox.prototype.OnKeyDown.call(this, e);
            }
        }
    });

    /**
     * Represents a scrollable area that can contain other visible elements.
     *
     * @public
     * @class Windows.UI.Xaml.Controls.ScrollViewer
     * @augments Windows.UI.Xaml.Controls.ContentControl
     */
    Bridge.define("Windows.UI.Xaml.Controls.ScrollViewer", {
        inherits: [Windows.UI.Xaml.Controls.ContentControl],
        statics: {
            fields: {
                /**
                 * Identifies the HorizontalScrollBarVisibility dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Controls.ScrollViewer
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                HorizontalScrollBarVisibilityProperty: null,
                /**
                 * Identifies the VerticalScrollBarVisibility dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Controls.ScrollViewer
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                VerticalScrollBarVisibilityProperty: null
            },
            ctors: {
                init: function () {
                    var $t;
                    this.HorizontalScrollBarVisibilityProperty = Windows.UI.Xaml.DependencyProperty.Register("HorizontalScrollBarVisibility", Windows.UI.Xaml.Controls.ScrollBarVisibility, Windows.UI.Xaml.Controls.ScrollViewer, ($t = new Windows.UI.Xaml.PropertyMetadata.$ctor2(Bridge.box(Windows.UI.Xaml.Controls.ScrollBarVisibility.Disabled, Windows.UI.Xaml.Controls.ScrollBarVisibility, System.Enum.toStringFn(Windows.UI.Xaml.Controls.ScrollBarVisibility)), Windows.UI.Xaml.Controls.ScrollViewer.HorizontalScrollBarVisibility_Changed), $t.CallPropertyChangedWhenLoadedIntoVisualTree = Windows.UI.Xaml.WhenToCallPropertyChangedEnum.IfPropertyIsSet, $t));
                    this.VerticalScrollBarVisibilityProperty = Windows.UI.Xaml.DependencyProperty.Register("VerticalScrollBarVisibility", Windows.UI.Xaml.Controls.ScrollBarVisibility, Windows.UI.Xaml.Controls.ScrollViewer, ($t = new Windows.UI.Xaml.PropertyMetadata.$ctor2(Bridge.box(Windows.UI.Xaml.Controls.ScrollBarVisibility.Visible, Windows.UI.Xaml.Controls.ScrollBarVisibility, System.Enum.toStringFn(Windows.UI.Xaml.Controls.ScrollBarVisibility)), Windows.UI.Xaml.Controls.ScrollViewer.VerticalScrollBarVisibility_Changed), $t.CallPropertyChangedWhenLoadedIntoVisualTree = Windows.UI.Xaml.WhenToCallPropertyChangedEnum.IfPropertyIsSet, $t));
                }
            },
            methods: {
                HorizontalScrollBarVisibility_Changed: function (d, e) {
                    if (Bridge.is(d, Windows.UI.Xaml.Controls.ScrollViewer)) {
                        var scrollViewer = Bridge.cast(d, Windows.UI.Xaml.Controls.ScrollViewer);
                        var newValue = System.Nullable.getValue(Bridge.cast(Bridge.unbox(e.NewValue, Windows.UI.Xaml.Controls.ScrollBarVisibility), System.Int32));

                        if (CSHTML5.Internal.INTERNAL_VisualTreeManager.IsElementInVisualTree(scrollViewer)) {

                            var outerDiv = scrollViewer.INTERNAL_OuterDomElement;
                            var outerDivStyle = CSHTML5.Internal.INTERNAL_HtmlDomManager.GetDomElementStyleForModification(outerDiv);
                            var innerDiv = outerDiv.firstChild;
                            var innerDivStyle = CSHTML5.Internal.INTERNAL_HtmlDomManager.GetDomElementStyleForModification(innerDiv);


                            scrollViewer.INTERNAL_ApplyHorizontalSettings(newValue, outerDivStyle, innerDivStyle);
                        }
                    }
                },
                VerticalScrollBarVisibility_Changed: function (d, e) {
                    if (Bridge.is(d, Windows.UI.Xaml.Controls.ScrollViewer)) {
                        var scrollViewer = Bridge.cast(d, Windows.UI.Xaml.Controls.ScrollViewer);
                        var newValue = System.Nullable.getValue(Bridge.cast(Bridge.unbox(e.NewValue, Windows.UI.Xaml.Controls.ScrollBarVisibility), System.Int32));

                        if (CSHTML5.Internal.INTERNAL_VisualTreeManager.IsElementInVisualTree(scrollViewer)) {

                            var outerDiv = scrollViewer.INTERNAL_OuterDomElement;
                            var outerDivStyle = CSHTML5.Internal.INTERNAL_HtmlDomManager.GetDomElementStyleForModification(outerDiv);
                            var innerDiv = outerDiv.firstChild;
                            var innerDivStyle = CSHTML5.Internal.INTERNAL_HtmlDomManager.GetDomElementStyleForModification(innerDiv);


                            scrollViewer.INTERNAL_ApplyVerticalSettings(newValue, outerDivStyle, innerDivStyle);
                        }
                    }
                }
            }
        },
        fields: {
            _verticalOffset: 0,
            _horizontalOffset: 0
        },
        props: {
            HorizontalOffset: {
                get: function () {
                    if (CSHTML5.Internal.INTERNAL_VisualTreeManager.IsElementInVisualTree(this)) {
                        this._horizontalOffset = Bridge.cast(CSHTML5.Internal.INTERNAL_HtmlDomManager.GetDomElementAttribute(this.INTERNAL_OuterDomElement, "scrollLeft"), System.Double);
                    }
                    return this._horizontalOffset;
                }
            },
            /**
             * Gets or sets a value that indicates whether a horizontal ScrollBar should
             be displayed.
             Returns a ScrollBarVisibility value that indicates whether a horizontal ScrollBar
             should be displayed. The default value is Hidden.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.ScrollViewer
             * @function HorizontalScrollBarVisibility
             * @type Windows.UI.Xaml.Controls.ScrollBarVisibility
             */
            HorizontalScrollBarVisibility: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.Controls.ScrollViewer.HorizontalScrollBarVisibilityProperty), Windows.UI.Xaml.Controls.ScrollBarVisibility), System.Int32));
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Controls.ScrollViewer.HorizontalScrollBarVisibilityProperty, Bridge.box(value, Windows.UI.Xaml.Controls.ScrollBarVisibility, System.Enum.toStringFn(Windows.UI.Xaml.Controls.ScrollBarVisibility)));
                }
            },
            /**
             * // Gets or sets a value that determines how manipulation input influences scrolling
            // behavior on the horizontal axis.
            // 
            // Returns a value of the enumeration. The typical default (as set through the default
            // template, not class initialization) is Rails.
            //
             * // Identifies the HorizontalScrollMode dependency property.
            //
             * Gets a value that indicates the vertical offset of the scrolled content.
             *
             * @instance
             * @public
             * @readonly
             * @memberof Windows.UI.Xaml.Controls.ScrollViewer
             * @function VerticalOffset
             * @type number
             */
            VerticalOffset: {
                get: function () {
                    if (CSHTML5.Internal.INTERNAL_VisualTreeManager.IsElementInVisualTree(this)) {
                        this._verticalOffset = Bridge.cast(CSHTML5.Internal.INTERNAL_HtmlDomManager.GetDomElementAttribute(this.INTERNAL_OuterDomElement, "scrollTop"), System.Double);
                    }
                    return this._verticalOffset;
                }
            },
            /**
             * Gets or sets a value that indicates whether a vertical ScrollBar should be displayed.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.ScrollViewer
             * @function VerticalScrollBarVisibility
             * @type Windows.UI.Xaml.Controls.ScrollBarVisibility
             */
            VerticalScrollBarVisibility: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.Controls.ScrollViewer.VerticalScrollBarVisibilityProperty), Windows.UI.Xaml.Controls.ScrollBarVisibility), System.Int32));
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Controls.ScrollViewer.VerticalScrollBarVisibilityProperty, Bridge.box(value, Windows.UI.Xaml.Controls.ScrollBarVisibility, System.Enum.toStringFn(Windows.UI.Xaml.Controls.ScrollBarVisibility)));
                }
            }
        },
        ctors: {
            init: function () {
                this._verticalOffset = 0;
                this._horizontalOffset = 0;
            },
            /**
             * Initializes a new instance of the ScrollViewer class.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Controls.ScrollViewer
             * @memberof Windows.UI.Xaml.Controls.ScrollViewer
             * @return  {void}
             */
            ctor: function () {
                this.$initialize();
                Windows.UI.Xaml.Controls.ContentControl.ctor.call(this);
            }
        },
        methods: {
            /**
             * // Gets or sets a value that determines how manipulation input influences scrolling behavior on the vertical axis.
            //
             * // Identifies the VerticalScrollMode dependency property.
            //
             *
             * @instance
             * @public
             * @override
             * @this Windows.UI.Xaml.Controls.ScrollViewer
             * @memberof Windows.UI.Xaml.Controls.ScrollViewer
             * @param   {System.Object}    parentRef                         
             * @param   {System.Object}    domElementWhereToPlaceChildren
             * @return  {System.Object}
             */
            CreateDomElement: function (parentRef, domElementWhereToPlaceChildren) {
                var outerDiv = { };
                var outerDivStyle = CSHTML5.Internal.INTERNAL_HtmlDomManager.CreateDomElementAppendItAndGetStyle("div", parentRef, this, outerDiv);
                outerDivStyle.height = "100%";
                outerDivStyle.width = "100%";
                outerDivStyle.overflowX = "scroll";
                outerDivStyle.overflowY = "scroll";
                if (this._verticalOffset !== 0) {
                    CSHTML5.Internal.INTERNAL_HtmlDomManager.SetDomElementProperty(outerDiv.v, "scrollTop", Bridge.box(this._verticalOffset, System.Double, System.Double.format, System.Double.getHashCode));
                }
                if (this._horizontalOffset !== 0) {
                    CSHTML5.Internal.INTERNAL_HtmlDomManager.SetDomElementProperty(outerDiv.v, "scrollLeft", Bridge.box(this._horizontalOffset, System.Double, System.Double.format, System.Double.getHashCode));
                }


                var innerDiv = { };
                var innerDivStyle = CSHTML5.Internal.INTERNAL_HtmlDomManager.CreateDomElementAppendItAndGetStyle("div", outerDiv.v, this, innerDiv);
                innerDivStyle.position = "relative";


                this.INTERNAL_ApplyHorizontalSettings(this.HorizontalScrollBarVisibility, outerDivStyle, innerDivStyle);
                this.INTERNAL_ApplyVerticalSettings(this.VerticalScrollBarVisibility, outerDivStyle, innerDivStyle);

                domElementWhereToPlaceChildren.v = innerDiv.v;
                return outerDiv.v;
            },
            INTERNAL_ApplyHorizontalSettings: function (horizontalScrollBarVisibility, outerDivStyle, innerDivStyle) {
                if (CSHTML5.Internal.INTERNAL_VisualTreeManager.IsElementInVisualTree(this)) {

                    switch (horizontalScrollBarVisibility) {
                        case Windows.UI.Xaml.Controls.ScrollBarVisibility.Disabled: 
                            outerDivStyle.overflowX = "hidden";
                            break;
                        case Windows.UI.Xaml.Controls.ScrollBarVisibility.Auto: 
                            outerDivStyle.overflowX = "auto";
                            break;
                        case Windows.UI.Xaml.Controls.ScrollBarVisibility.Hidden: 
                            outerDivStyle.overflowX = "hidden";
                            break;
                        case Windows.UI.Xaml.Controls.ScrollBarVisibility.Visible: 
                            outerDivStyle.overflowX = "scroll";
                            try {
                                outerDivStyle.WebkitOverflowScrolling = "touch";
                            } catch ($e1) {
                                $e1 = System.Exception.create($e1);
                            }
                            break;
                        default: 
                            break;
                    }


                    if ((horizontalScrollBarVisibility === Windows.UI.Xaml.Controls.ScrollBarVisibility.Visible || horizontalScrollBarVisibility === Windows.UI.Xaml.Controls.ScrollBarVisibility.Auto) && isNaN(this.Width)) {
                        if (Bridge.is(this.INTERNAL_VisualParent, Windows.UI.Xaml.Controls.Grid) && !document.isGridSupported) {
                            innerDivStyle.width = "0px";
                            innerDivStyle.overflowX = "visible";
                        } else {
                            innerDivStyle.width = "auto";
                            innerDivStyle.minWidth = "100%";
                        }
                    } else {
                        innerDivStyle.width = "100%";
                    }
                }
            },
            INTERNAL_ApplyVerticalSettings: function (verticalScrollBarVisibility, outerDivStyle, innerDivStyle) {
                if (CSHTML5.Internal.INTERNAL_VisualTreeManager.IsElementInVisualTree(this)) {

                    switch (verticalScrollBarVisibility) {
                        case Windows.UI.Xaml.Controls.ScrollBarVisibility.Disabled: 
                            outerDivStyle.overflowY = "hidden";
                            break;
                        case Windows.UI.Xaml.Controls.ScrollBarVisibility.Auto: 
                            outerDivStyle.overflowY = "auto";
                            break;
                        case Windows.UI.Xaml.Controls.ScrollBarVisibility.Hidden: 
                            outerDivStyle.overflowY = "hidden";
                            break;
                        case Windows.UI.Xaml.Controls.ScrollBarVisibility.Visible: 
                            outerDivStyle.overflowY = "scroll";
                            try {
                                outerDivStyle.WebkitOverflowScrolling = "touch";
                            } catch ($e1) {
                                $e1 = System.Exception.create($e1);
                            }
                            break;
                        default: 
                            break;
                    }


                    if ((verticalScrollBarVisibility === Windows.UI.Xaml.Controls.ScrollBarVisibility.Visible || verticalScrollBarVisibility === Windows.UI.Xaml.Controls.ScrollBarVisibility.Auto) && isNaN(this.Height)) {
                        if (Bridge.is(this.INTERNAL_VisualParent, Windows.UI.Xaml.Controls.Grid) && !document.isGridSupported) {
                            innerDivStyle.height = "0px";
                            innerDivStyle.overflowY = "visible";
                        } else {
                            innerDivStyle.height = "auto";
                            innerDivStyle.minHeight = "100%";
                        }
                    } else {
                        innerDivStyle.height = "100%";
                    }
                }
            },
            /**
             * Scrolls the content that is within the ScrollViewer to the specified horizontal offset position.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Controls.ScrollViewer
             * @memberof Windows.UI.Xaml.Controls.ScrollViewer
             * @param   {number}    offset    The position that the content scrolls to.
             * @return  {void}
             */
            ScrollToHorizontalOffset: function (offset) {
                this._horizontalOffset = offset;
                if (CSHTML5.Internal.INTERNAL_VisualTreeManager.IsElementInVisualTree(this)) {
                    CSHTML5.Internal.INTERNAL_HtmlDomManager.SetDomElementProperty(this.INTERNAL_OuterDomElement, "scrollLeft", Bridge.box(offset, System.Double, System.Double.format, System.Double.getHashCode));
                }
            },
            /**
             * Scrolls the content that is within the ScrollViewer to the specified vertical offset position.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Controls.ScrollViewer
             * @memberof Windows.UI.Xaml.Controls.ScrollViewer
             * @param   {number}    offset    The position that the content scrolls to.
             * @return  {void}
             */
            ScrollToVerticalOffset: function (offset) {
                this._verticalOffset = offset;
                if (CSHTML5.Internal.INTERNAL_VisualTreeManager.IsElementInVisualTree(this)) {
                    CSHTML5.Internal.INTERNAL_HtmlDomManager.SetDomElementProperty(this.INTERNAL_OuterDomElement, "scrollTop", Bridge.box(offset, System.Double, System.Double.format, System.Double.getHashCode));
                }
            }
        }
    });

    Bridge.define("Windows.UI.Xaml.Controls.Slider", {
        inherits: [Windows.UI.Xaml.Controls.Primitives.RangeBase],
        statics: {
            fields: {
                MINIMUM_THUMB_SIZE: 0,
                /**
                 * Identifies the Orientation dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Controls.Slider
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                OrientationProperty: null,
                /**
                 * Identifies the ViewportSize dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Controls.Slider
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                ViewportSizeProperty: null
            },
            ctors: {
                init: function () {
                    this.MINIMUM_THUMB_SIZE = 9.0;
                    this.OrientationProperty = Windows.UI.Xaml.DependencyProperty.Register("Orientation", Windows.UI.Xaml.Controls.Orientation, Windows.UI.Xaml.Controls.Slider, new Windows.UI.Xaml.PropertyMetadata.$ctor2(Bridge.box(Windows.UI.Xaml.Controls.Orientation.Vertical, Windows.UI.Xaml.Controls.Orientation, System.Enum.toStringFn(Windows.UI.Xaml.Controls.Orientation)), Windows.UI.Xaml.Controls.Slider.Orientation_Changed));
                    this.ViewportSizeProperty = Windows.UI.Xaml.DependencyProperty.Register("ViewportSize", System.Double, Windows.UI.Xaml.Controls.Slider, new Windows.UI.Xaml.PropertyMetadata.$ctor2(Bridge.box(0.0, System.Double, System.Double.format, System.Double.getHashCode), Windows.UI.Xaml.Controls.Slider.ViewportSize_Changed));
                }
            },
            methods: {
                Orientation_Changed: function (d, e) {
                    var slider = Bridge.cast(d, Windows.UI.Xaml.Controls.Slider);
                    slider._controlWasProperlyDrawn = slider.TryUpdateSizeAndPositionOfUIElements();
                },
                ViewportSize_Changed: function (d, e) {
                    var slider = Bridge.cast(d, Windows.UI.Xaml.Controls.Slider);
                    slider._controlWasProperlyDrawn = slider.TryUpdateSizeAndPositionOfUIElements();
                }
            }
        },
        fields: {
            _controlWasProperlyDrawn: false,
            _valueWithoutCoercion: 0,
            _horizontalRoot: null,
            _horizontalThumb: null,
            _horizontalRail: null,
            _horizontalLargeDecrease: null,
            _horizontalLargeIncrease: null,
            _verticalRoot: null,
            _verticalThumb: null,
            _verticalRail: null,
            _verticalLargeDecrease: null,
            _verticalLargeIncrease: null
        },
        events: {
            /**
             * Occurs one or more times as content scrolls in a Slider when the user moves the Thumb by using the mouse.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Controls.Slider
             * @memberof Windows.UI.Xaml.Controls.Slider
             * @function addScroll
             * @param   {Windows.UI.Xaml.Controls.Primitives.ScrollEventHandler}    value
             * @return  {void}
             */
            /**
             * Occurs one or more times as content scrolls in a Slider when the user moves the Thumb by using the mouse.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Controls.Slider
             * @memberof Windows.UI.Xaml.Controls.Slider
             * @function removeScroll
             * @param   {Windows.UI.Xaml.Controls.Primitives.ScrollEventHandler}    value
             * @return  {void}
             */
            Scroll: null
        },
        props: {
            /**
             * Gets or sets a value that indicates whether the Slider is displayed horizontally or vertically.
             The default is Horizontal.  Specific control templates might change this value, which would cause the templated value to be the apparent runtime default.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.Slider
             * @function Orientation
             * @type Windows.UI.Xaml.Controls.Orientation
             */
            Orientation: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.Controls.Slider.OrientationProperty), Windows.UI.Xaml.Controls.Orientation), System.Int32));
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Controls.Slider.OrientationProperty, Bridge.box(value, Windows.UI.Xaml.Controls.Orientation, System.Enum.toStringFn(Windows.UI.Xaml.Controls.Orientation)));
                }
            },
            /**
             * Gets or sets the amount of the scrollable content that is currently visible. The default is 0.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.Slider
             * @function ViewportSize
             * @type number
             */
            ViewportSize: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.Controls.Slider.ViewportSizeProperty), System.Double), System.Double));
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Controls.Slider.ViewportSizeProperty, Bridge.box(value, System.Double, System.Double.format, System.Double.getHashCode));
                }
            }
        },
        ctors: {
            /**
             * Initializes a new instance of the Slider class.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Controls.Slider
             * @memberof Windows.UI.Xaml.Controls.Slider
             * @return  {void}
             */
            ctor: function () {
                this.$initialize();
                Windows.UI.Xaml.Controls.Primitives.RangeBase.ctor.call(this);
                this.DefaultStyleKey = Windows.UI.Xaml.Controls.Slider;

                this.addUnloaded(Bridge.fn.cacheBind(this, this.Slider_Unloaded));
                this.addSizeChanged(Bridge.fn.cacheBind(this, this.Slider_SizeChanged));
            }
        },
        methods: {
            Slider_SizeChanged: function (sender, e) {

                if (this.TryUpdateSizeAndPositionOfUIElements()) {
                    this._controlWasProperlyDrawn = true;
                }
            },
            Slider_Unloaded: function (sender, e) {
                this._controlWasProperlyDrawn = false;
            },
            OnApplyTemplate: function () {
                Windows.UI.Xaml.Controls.Primitives.RangeBase.prototype.OnApplyTemplate.call(this);


                this._horizontalRoot = Bridge.as(this.GetTemplateChild("HorizontalRoot"), Windows.UI.Xaml.Controls.Canvas);
                this._horizontalThumb = Bridge.as(this.GetTemplateChild("HorizontalThumb"), Windows.UI.Xaml.Controls.Primitives.Thumb);
                this._horizontalRail = Bridge.as(this.GetTemplateChild("HorizontalRail"), Windows.UI.Xaml.Shapes.Rectangle);
                this._horizontalLargeDecrease = Bridge.as(this.GetTemplateChild("HorizontalLargeDecrease"), Windows.UI.Xaml.Controls.Primitives.ButtonBase);
                this._horizontalLargeIncrease = Bridge.as(this.GetTemplateChild("HorizontalLargeIncrease"), Windows.UI.Xaml.Controls.Primitives.ButtonBase);

                this._verticalRoot = Bridge.as(this.GetTemplateChild("VerticalRoot"), Windows.UI.Xaml.Controls.Canvas);
                this._verticalThumb = Bridge.as(this.GetTemplateChild("VerticalThumb"), Windows.UI.Xaml.Controls.Primitives.Thumb);
                this._verticalRail = Bridge.as(this.GetTemplateChild("VerticalRail"), Windows.UI.Xaml.Shapes.Rectangle);
                this._verticalLargeDecrease = Bridge.as(this.GetTemplateChild("VerticalLargeDecrease"), Windows.UI.Xaml.Controls.Primitives.ButtonBase);
                this._verticalLargeIncrease = Bridge.as(this.GetTemplateChild("VerticalLargeIncrease"), Windows.UI.Xaml.Controls.Primitives.ButtonBase);

                if (this._horizontalThumb != null) {
                    this._horizontalThumb.addDragStarted(Bridge.fn.cacheBind(this, this.HorizontalThumb_DragStarted));
                    this._horizontalThumb.addDragDelta(Bridge.fn.cacheBind(this, this.HorizontalThumb_DragDelta));
                    this._horizontalThumb.addDragCompleted(Bridge.fn.cacheBind(this, this.HorizontalThumb_DragCompleted));
                    this._horizontalThumb.addGotFocus(Bridge.fn.cacheBind(this, this.HorizontalThumb_GotFocus));
                    this._horizontalThumb.addLostFocus(Bridge.fn.cacheBind(this, this.HorizontalThumb_LostFocus));
                }
                if (this._verticalThumb != null) {
                    this._verticalThumb.addDragStarted(Bridge.fn.cacheBind(this, this.VerticalThumb_DragStarted));
                    this._verticalThumb.addDragDelta(Bridge.fn.cacheBind(this, this.VerticalThumb_DragDelta));
                    this._verticalThumb.addDragCompleted(Bridge.fn.cacheBind(this, this.VerticalThumb_DragCompleted));
                    this._verticalThumb.addGotFocus(Bridge.fn.cacheBind(this, this.VerticalThumb_GotFocus));
                    this._verticalThumb.addLostFocus(Bridge.fn.cacheBind(this, this.VerticalThumb_LostFocus));
                }

                if (this._horizontalLargeDecrease != null) {
                    this._horizontalLargeDecrease.addClick(Bridge.fn.cacheBind(this, this.LargeDecrease_Click));
                }
                if (this._verticalLargeDecrease != null) {
                    this._verticalLargeDecrease.addClick(Bridge.fn.cacheBind(this, this.LargeDecrease_Click));
                }
                if (this._horizontalLargeIncrease != null) {
                    this._horizontalLargeIncrease.addClick(Bridge.fn.cacheBind(this, this.LargeIncrease_Click));
                }
                if (this._verticalLargeIncrease != null) {
                    this._verticalLargeIncrease.addClick(Bridge.fn.cacheBind(this, this.LargeIncrease_Click));
                }


                if (this._verticalRoot != null) {
                    this._verticalRoot.Visibility = (this.Orientation === Windows.UI.Xaml.Controls.Orientation.Vertical ? Windows.UI.Xaml.Visibility.Visible : Windows.UI.Xaml.Visibility.Collapsed);
                }
                if (this._horizontalRoot != null) {
                    this._horizontalRoot.Visibility = (this.Orientation === Windows.UI.Xaml.Controls.Orientation.Horizontal ? Windows.UI.Xaml.Visibility.Visible : Windows.UI.Xaml.Visibility.Collapsed);
                }
            },
            VerticalThumb_LostFocus: function (sender, e) {
                this._verticalThumb.removeKeyDown(Bridge.fn.cacheBind(this, this.Thumb_KeyDown));
            },
            VerticalThumb_GotFocus: function (sender, e) {
                this._verticalThumb.removeKeyDown(Bridge.fn.cacheBind(this, this.Thumb_KeyDown));
                this._verticalThumb.addKeyDown(Bridge.fn.cacheBind(this, this.Thumb_KeyDown));
            },
            HorizontalThumb_LostFocus: function (sender, e) {
                this._horizontalThumb.removeKeyDown(Bridge.fn.cacheBind(this, this.Thumb_KeyDown));
            },
            HorizontalThumb_GotFocus: function (sender, e) {
                this._horizontalThumb.removeKeyDown(Bridge.fn.cacheBind(this, this.Thumb_KeyDown));
                this._horizontalThumb.addKeyDown(Bridge.fn.cacheBind(this, this.Thumb_KeyDown));
            },
            Thumb_KeyDown: function (sender, e) {
                if (e.Key === Windows.System.VirtualKey.Right || e.Key === Windows.System.VirtualKey.Up) {
                    this.OnClickOnSmallOrLargeDecreaseIncreaseButtons(this.SmallChange, Windows.UI.Xaml.Controls.Primitives.ScrollEventType.SmallIncrement);
                } else if (e.Key === Windows.System.VirtualKey.Left || e.Key === Windows.System.VirtualKey.Down) {
                    this.OnClickOnSmallOrLargeDecreaseIncreaseButtons(-this.SmallChange, Windows.UI.Xaml.Controls.Primitives.ScrollEventType.SmallDecrement);
                }
            },
            OnValueChanged: function (oldValue, newValue) {
                Windows.UI.Xaml.Controls.Primitives.RangeBase.prototype.OnValueChanged.call(this, oldValue, newValue);

                this.UpdateThumbPositionAndSize();
            },
            OnMaximumChanged: function (oldMaximum, newMaximum) {
                Windows.UI.Xaml.Controls.Primitives.RangeBase.prototype.OnMaximumChanged.call(this, oldMaximum, newMaximum);

                this.UpdateThumbPositionAndSize();
            },
            OnMinimumChanged: function (oldMinimum, newMinimum) {
                Windows.UI.Xaml.Controls.Primitives.RangeBase.prototype.OnMinimumChanged.call(this, oldMinimum, newMinimum);

                this.UpdateThumbPositionAndSize();
            },
            LargeIncrease_Click: function (sender, e) {
                this.OnClickOnSmallOrLargeDecreaseIncreaseButtons(this.LargeChange, Windows.UI.Xaml.Controls.Primitives.ScrollEventType.LargeIncrement);
            },
            LargeDecrease_Click: function (sender, e) {
                this.OnClickOnSmallOrLargeDecreaseIncreaseButtons(-this.LargeChange, Windows.UI.Xaml.Controls.Primitives.ScrollEventType.LargeDecrement);
            },
            OnClickOnSmallOrLargeDecreaseIncreaseButtons: function (amountByWhichToIncreaseOrDecreaseTheValue, scrollEventType) {
                var newValue = this.CoerceValue$1(this.Value + amountByWhichToIncreaseOrDecreaseTheValue);
                if (newValue !== this.Value) {
                    this.SetCurrentValue(Windows.UI.Xaml.Controls.Primitives.RangeBase.ValueProperty, Bridge.box(newValue, System.Double, System.Double.format, System.Double.getHashCode));
                    if (!Bridge.staticEquals(this.Scroll, null)) {
                        this.Scroll(this, new Windows.UI.Xaml.Controls.Primitives.ScrollEventArgs(newValue, scrollEventType));
                    }
                }

                if (this.Orientation === Windows.UI.Xaml.Controls.Orientation.Horizontal) {
                    this._horizontalThumb.Focus();
                } else {
                    this._verticalThumb.Focus();
                }
            },
            VerticalThumb_DragCompleted: function (sender, e) {
                this.OnDragCompleted();
            },
            HorizontalThumb_DragCompleted: function (sender, e) {
                this.OnDragCompleted();
            },
            VerticalThumb_DragStarted: function (sender, e) {
                this.OnDragStarted();
            },
            HorizontalThumb_DragStarted: function (sender, e) {
                this.OnDragStarted();
            },
            VerticalThumb_DragDelta: function (sender, e) {
                this.ChangeValueBasedOnPointerMovement(-e.VerticalChange);
            },
            HorizontalThumb_DragDelta: function (sender, e) {
                this.ChangeValueBasedOnPointerMovement(e.HorizontalChange);
            },
            OnDragStarted: function () {
                this._valueWithoutCoercion = this.Value;

                if (this._horizontalLargeDecrease != null) {
                    this._horizontalLargeDecrease.IsHitTestVisible = false;
                }
                if (this._horizontalLargeIncrease != null) {
                    this._horizontalLargeIncrease.IsHitTestVisible = false;
                }
            },
            OnDragCompleted: function () {
                if (!Bridge.staticEquals(this.Scroll, null)) {
                    this.Scroll(this, new Windows.UI.Xaml.Controls.Primitives.ScrollEventArgs(this.Value, Windows.UI.Xaml.Controls.Primitives.ScrollEventType.EndScroll));
                }

                if (this._horizontalLargeDecrease != null) {
                    this._horizontalLargeDecrease.IsHitTestVisible = true;
                }
                if (this._horizontalLargeIncrease != null) {
                    this._horizontalLargeIncrease.IsHitTestVisible = true;
                }
            },
            ChangeValueBasedOnPointerMovement: function (pointerMovementInPixels) {

                var totalControlSize = { v : new Windows.Foundation.Size() };
                if (this.CheckIfControlWasRenderedProperlyAndGetCurrentControlSize(totalControlSize)) {
                    var totalControlSizeInMainDirection = (this.Orientation === Windows.UI.Xaml.Controls.Orientation.Vertical ? totalControlSize.v.Height : totalControlSize.v.Width);
                    var trackSize = { };
                    var thumbSize = { };
                    var scrollableSize = { };
                    var minValue = { };
                    var maxValue = { };
                    this.CalculateThumbTrackAndScrollableSizes(totalControlSizeInMainDirection, trackSize, thumbSize, scrollableSize, minValue, maxValue);
                    var maxMinusMin = maxValue.v - minValue.v;

                    var valueDelta;
                    if (scrollableSize.v > 0 && maxMinusMin > 0) {
                        valueDelta = (pointerMovementInPixels * maxMinusMin) / scrollableSize.v;
                    } else {
                        valueDelta = 0;
                    }

                    this._valueWithoutCoercion += valueDelta;
                    var newValue = this.CoerceValue$1(this._valueWithoutCoercion);

                    if (newValue !== this.Value) {
                        this.SetCurrentValue(Windows.UI.Xaml.Controls.Primitives.RangeBase.ValueProperty, Bridge.box(newValue, System.Double, System.Double.format, System.Double.getHashCode));

                        this.UpdateThumbPositionAndSize$1(totalControlSizeInMainDirection);

                        if (!Bridge.staticEquals(this.Scroll, null)) {
                            this.Scroll(this, new Windows.UI.Xaml.Controls.Primitives.ScrollEventArgs(newValue, Windows.UI.Xaml.Controls.Primitives.ScrollEventType.ThumbTrack));
                        }
                    }
                }
            },
            CoerceValue$1: function (value) {

                var minValue = this.Minimum;
                var maxValue = this.Maximum;
                if (value <= minValue) {
                    value = minValue;
                }
                if (value >= maxValue) {
                    value = maxValue;
                }
                return value;
            },
            TryUpdateSizeAndPositionOfUIElements: function () {
                var $t, $t1, $t2, $t3;

                var totalControlSize = { v : new Windows.Foundation.Size() };
                if (this.IsLoaded && this.TryGetCurrentControlSize(totalControlSize)) {
                    if (this.Orientation === Windows.UI.Xaml.Controls.Orientation.Vertical) {

                        if (this._verticalRoot != null && this._verticalThumb != null && this._verticalRail != null && this._verticalLargeIncrease != null && this._verticalLargeDecrease != null) {
                            this._verticalThumb.Width = ($t = ($t1 = totalControlSize.v.Width, this._verticalLargeIncrease.Height = $t1, $t1), this._verticalLargeDecrease.Height = $t, $t);


                            this.UpdateThumbPositionAndSize$1(totalControlSize.v.Height);

                            var thumbLeftSidePosition = (totalControlSize.v.Width - this._verticalThumb.ActualWidth) / 2;
                            Windows.UI.Xaml.Controls.Canvas.SetLeft(this._verticalRail, (totalControlSize.v.Width - this._verticalRail.ActualWidth) / 2);
                            Windows.UI.Xaml.Controls.Canvas.SetLeft(this._verticalThumb, thumbLeftSidePosition);
                            Windows.UI.Xaml.Controls.Canvas.SetLeft(this._verticalLargeDecrease, thumbLeftSidePosition);
                            Windows.UI.Xaml.Controls.Canvas.SetLeft(this._verticalLargeIncrease, thumbLeftSidePosition);

                            var thumbWidth = this._verticalThumb.ActualWidth;
                            this._verticalLargeDecrease.Width = thumbWidth;
                            this._verticalLargeIncrease.Width = thumbWidth;

                            return true;
                        }
                    } else {

                        if (this._horizontalRoot != null && this._horizontalThumb != null && this._horizontalRail != null && this._horizontalLargeIncrease != null && this._horizontalLargeDecrease != null) {
                            this._horizontalThumb.Height = ($t2 = ($t3 = totalControlSize.v.Height, this._horizontalLargeIncrease.Height = $t3, $t3), this._horizontalLargeDecrease.Height = $t2, $t2);

                            this.UpdateThumbPositionAndSize$1(totalControlSize.v.Width);

                            var thumbTopSidePosition = (totalControlSize.v.Height - this._horizontalThumb.ActualHeight) / 2;
                            Windows.UI.Xaml.Controls.Canvas.SetTop(this._horizontalRail, (totalControlSize.v.Height - this._horizontalRail.ActualHeight) / 2);
                            Windows.UI.Xaml.Controls.Canvas.SetTop(this._horizontalThumb, thumbTopSidePosition);
                            Windows.UI.Xaml.Controls.Canvas.SetTop(this._horizontalLargeDecrease, thumbTopSidePosition);
                            Windows.UI.Xaml.Controls.Canvas.SetTop(this._horizontalLargeIncrease, thumbTopSidePosition);

                            var thumbHeight = this._horizontalThumb.ActualHeight;
                            this._horizontalLargeDecrease.Height = thumbHeight;
                            this._horizontalLargeIncrease.Height = thumbHeight;

                            return true;
                        }
                    }
                }

                return false;
            },
            CheckIfControlWasRenderedProperlyAndGetCurrentControlSize: function (totalControlSize) {
                if (this._controlWasProperlyDrawn) {
                    return this.TryGetCurrentControlSize(totalControlSize);
                } else {
                    totalControlSize.v = new Windows.Foundation.Size.$ctor1(0.0, 0.0);
                    return false;
                }
            },
            TryGetCurrentControlSize: function (totalControlSize) {
                totalControlSize.v = new Windows.Foundation.Size.$ctor1(0.0, 0.0);
                if (this._verticalRoot != null && this._horizontalRoot != null) {
                    var actualSize = new Windows.Foundation.Size();
                    if (this.Orientation === Windows.UI.Xaml.Controls.Orientation.Vertical) {
                        actualSize = this._verticalRoot.INTERNAL_GetActualWidthAndHeight();
                    } else {
                        actualSize = this._horizontalRoot.INTERNAL_GetActualWidthAndHeight();
                    }
                    var width = !isNaN(actualSize.Width) ? actualSize.Width : this.Width;
                    var height = !isNaN(actualSize.Height) ? actualSize.Height : this.Height;

                    if (!isNaN(width) && !isNaN(height)) {
                        totalControlSize.v = new Windows.Foundation.Size.$ctor1(width, height);
                        return true;
                    }
                }
                return false;
            },
            CalculateThumbTrackAndScrollableSizes: function (totalControlSize, trackSize, thumbSize, scrollableSize, minValue, maxValue) {
                minValue.v = this.Minimum;
                maxValue.v = this.Maximum;
                var maxMinusMin = maxValue.v - minValue.v;

                trackSize.v = Math.max(0.0, totalControlSize);
                thumbSize.v = maxMinusMin > 0 ? (trackSize.v * this.ViewportSize / (maxMinusMin * 2)) : trackSize.v;
                thumbSize.v = Math.min(trackSize.v, Math.max(Windows.UI.Xaml.Controls.Slider.MINIMUM_THUMB_SIZE, thumbSize.v));
                scrollableSize.v = trackSize.v - thumbSize.v;
            },
            UpdateThumbPositionAndSize: function () {
                var totalControlSize = { v : new Windows.Foundation.Size() };
                if (this.CheckIfControlWasRenderedProperlyAndGetCurrentControlSize(totalControlSize)) {
                    var totalControlSizeInMainDirection = (this.Orientation === Windows.UI.Xaml.Controls.Orientation.Vertical ? totalControlSize.v.Height : totalControlSize.v.Width);
                    this.UpdateThumbPositionAndSize$1(totalControlSizeInMainDirection);
                }
            },
            UpdateThumbPositionAndSize$1: function (totalControlSize) {

                var value = this.Value;
                var trackSize = { };
                var thumbSize = { };
                var scrollableSize = { };
                var minValue = { };
                var maxValue = { };
                this.CalculateThumbTrackAndScrollableSizes(totalControlSize, trackSize, thumbSize, scrollableSize, minValue, maxValue);
                var maxMinusMin = maxValue.v - minValue.v;

                var thumbCenterPosition = (maxMinusMin > 0 ? (thumbSize.v / 2) + (scrollableSize.v * (value - this.Minimum) / maxMinusMin) : trackSize.v / 2);
                if (this.Orientation === Windows.UI.Xaml.Controls.Orientation.Vertical) {
                    thumbCenterPosition = trackSize.v - thumbCenterPosition;
                }
                var thumbTopLeftPosition = thumbCenterPosition - (thumbSize.v / 2);
                var thumbBottomRightPosition = thumbCenterPosition + (thumbSize.v / 2);

                if (this.Orientation === Windows.UI.Xaml.Controls.Orientation.Vertical) {
                    if (this._verticalThumb != null && this._verticalLargeDecrease != null && this._verticalLargeIncrease != null) {
                        this._verticalThumb.Height = thumbSize.v;
                        Windows.UI.Xaml.Controls.Canvas.SetTop(this._verticalThumb, thumbTopLeftPosition);

                        this._verticalLargeDecrease.Height = Math.max(1.0, totalControlSize - thumbBottomRightPosition);
                        Windows.UI.Xaml.Controls.Canvas.SetTop(this._verticalLargeDecrease, thumbBottomRightPosition);

                        this._verticalLargeIncrease.Height = Math.max(1.0, thumbTopLeftPosition);
                        Windows.UI.Xaml.Controls.Canvas.SetTop(this._verticalLargeIncrease, 0.0);
                    }
                } else {
                    if (this._horizontalThumb != null && this._horizontalLargeDecrease != null && this._horizontalLargeIncrease != null) {
                        this._horizontalThumb.Width = thumbSize.v;
                        Windows.UI.Xaml.Controls.Canvas.SetLeft(this._horizontalThumb, thumbTopLeftPosition);

                        this._horizontalLargeDecrease.Width = Math.max(1.0, thumbTopLeftPosition);
                        Windows.UI.Xaml.Controls.Canvas.SetLeft(this._horizontalLargeDecrease, 0.0);

                        this._horizontalLargeIncrease.Width = Math.max(1.0, totalControlSize - thumbBottomRightPosition);
                        Windows.UI.Xaml.Controls.Canvas.SetLeft(this._horizontalLargeIncrease, thumbBottomRightPosition);
                    }
                }
            }
        }
    });

    /**
     * Represents a control that contains multiple items that share the same
     space on the screen.
     *
     * @public
     * @class Windows.UI.Xaml.Controls.TabControl
     * @augments Windows.UI.Xaml.Controls.ItemsControl
     */
    Bridge.define("Windows.UI.Xaml.Controls.TabControl", {
        inherits: [Windows.UI.Xaml.Controls.ItemsControl],
        statics: {
            fields: {
                /**
                 * Inherited code: Requires comment.
                 *
                 * @static
                 * @memberof Windows.UI.Xaml.Controls.TabControl
                 * @constant
                 * @default "TemplateTop"
                 * @type string
                 */
                ElementTemplateTopName: null,
                /**
                 * Inherited code: Requires comment.
                 *
                 * @static
                 * @memberof Windows.UI.Xaml.Controls.TabControl
                 * @constant
                 * @default "TabPanelTop"
                 * @type string
                 */
                ElementTabPanelTopName: null,
                /**
                 * Inherited code: Requires comment.
                 *
                 * @static
                 * @memberof Windows.UI.Xaml.Controls.TabControl
                 * @constant
                 * @default "ContentTop"
                 * @type string
                 */
                ElementContentTopName: null,
                /**
                 * Identifies the TabControl.SelectedItem dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Controls.TabControl
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                SelectedItemProperty: null,
                /**
                 * Identifies the TabControl.SelectedIndex dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Controls.TabControl
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                SelectedIndexProperty: null,
                /**
                 * Identifies the TabControl.SelectedContent dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Controls.TabControl
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                SelectedContentProperty: null
            },
            ctors: {
                init: function () {
                    this.ElementTemplateTopName = "TemplateTop";
                    this.ElementTabPanelTopName = "TabPanelTop";
                    this.ElementContentTopName = "ContentTop";
                    this.SelectedItemProperty = Windows.UI.Xaml.DependencyProperty.Register("SelectedItem", System.Object, Windows.UI.Xaml.Controls.TabControl, new Windows.UI.Xaml.PropertyMetadata.$ctor2(null, Windows.UI.Xaml.Controls.TabControl.OnSelectedItemChanged));
                    this.SelectedIndexProperty = Windows.UI.Xaml.DependencyProperty.Register("SelectedIndex", System.Int32, Windows.UI.Xaml.Controls.TabControl, new Windows.UI.Xaml.PropertyMetadata.$ctor2(Bridge.box(-1, System.Int32), Windows.UI.Xaml.Controls.TabControl.OnSelectedIndexChanged));
                    this.SelectedContentProperty = Windows.UI.Xaml.DependencyProperty.Register("SelectedContent", System.Object, Windows.UI.Xaml.Controls.TabControl, new Windows.UI.Xaml.PropertyMetadata.$ctor2(null, Windows.UI.Xaml.Controls.TabControl.OnSelectedContentChanged));
                }
            },
            methods: {
                /**
                 * SelectedItem property changed handler.
                 *
                 * @static
                 * @private
                 * @this Windows.UI.Xaml.Controls.TabControl
                 * @memberof Windows.UI.Xaml.Controls.TabControl
                 * @param   {Windows.UI.Xaml.DependencyObject}                      d    TabControl that changed its SelectedItem.
                 * @param   {Windows.UI.Xaml.DependencyPropertyChangedEventArgs}    e    The DependencyPropertyChangedEventArgs.
                 * @return  {void}
                 */
                OnSelectedItemChanged: function (d, e) {
                    var tc = Bridge.cast(d, Windows.UI.Xaml.Controls.TabControl);

                    var oldItem = Bridge.as(e.OldValue, Windows.UI.Xaml.Controls.TabItem);
                    var tabItem = Bridge.as(e.NewValue, Windows.UI.Xaml.Controls.TabItem);

                    var index = (tabItem == null ? -1 : tc.Items.indexOf(tabItem));
                    if (tabItem != null && index === -1) {
                        tc.SelectedItem = oldItem;
                        return;
                    } else {
                        tc.SelectedIndex = index;
                        tc.SelectItem(oldItem, tabItem);
                    }
                },
                /**
                 * SelectedIndex property changed handler.
                 *
                 * @static
                 * @private
                 * @this Windows.UI.Xaml.Controls.TabControl
                 * @memberof Windows.UI.Xaml.Controls.TabControl
                 * @param   {Windows.UI.Xaml.DependencyObject}                      d    TabControl that changed its SelectedIndex.
                 * @param   {Windows.UI.Xaml.DependencyPropertyChangedEventArgs}    e    The DependencyPropertyChangedEventArgs.
                 * @return  {void}
                 */
                OnSelectedIndexChanged: function (d, e) {
                    var tc = Bridge.cast(d, Windows.UI.Xaml.Controls.TabControl);

                    var newIndex = System.Nullable.getValue(Bridge.cast(Bridge.unbox(e.NewValue, System.Int32), System.Int32));
                    var oldIndex = System.Nullable.getValue(Bridge.cast(Bridge.unbox(e.OldValue, System.Int32), System.Int32));

                    if (newIndex < -1) {
                        throw new System.ArgumentException.$ctor1(System.String.format("'{0}' is not a valid value for property 'SelectedIndex'", [Bridge.toString(newIndex)]));
                    }

                    if (tc._updateIndex) {
                        tc._desiredIndex = newIndex;
                    } else if (!tc._updateIndex) {
                        tc._updateIndex = true;
                    }
                    if (newIndex >= tc.Items.Count) {
                        tc._updateIndex = false;
                        tc.SelectedIndex = oldIndex;
                        return;
                    }

                    var item = tc.GetItemAtIndex(newIndex);
                    if (!Bridge.referenceEquals(tc.SelectedItem, item)) {
                        tc.SelectedItem = item;
                    }
                },
                /**
                 * SelectedContent property changed handler.
                 *
                 * @static
                 * @private
                 * @this Windows.UI.Xaml.Controls.TabControl
                 * @memberof Windows.UI.Xaml.Controls.TabControl
                 * @param   {Windows.UI.Xaml.DependencyObject}                      d    TabControl that changed its SelectedContent.
                 * @param   {Windows.UI.Xaml.DependencyPropertyChangedEventArgs}    e    The DependencyPropertyChangedEventArgs.
                 * @return  {void}
                 */
                OnSelectedContentChanged: function (d, e) {
                    var tc = Bridge.cast(d, Windows.UI.Xaml.Controls.TabControl);

                    tc.UpdateSelectedContent(e.NewValue);
                }
            }
        },
        fields: {
            /**
             * Gets or sets the TabStripPlacement Top template.
             *
             * @instance
             * @memberof Windows.UI.Xaml.Controls.TabControl
             * @function ElementTemplateTop
             * @type Windows.UI.Xaml.FrameworkElement
             */
            ElementTemplateTop: null,
            /**
             * Gets or sets the TabPanel of the TabStripPlacement Top template.
             *
             * @instance
             * @memberof Windows.UI.Xaml.Controls.TabControl
             * @function ElementTabPanelTop
             * @type Windows.UI.Xaml.Controls.TabPanel
             */
            ElementTabPanelTop: null,
            /**
             * Gets or sets the ContentHost of the TabStripPlacement Top template.
             *
             * @instance
             * @memberof Windows.UI.Xaml.Controls.TabControl
             * @function ElementContentTop
             * @type Windows.UI.Xaml.Controls.ContentPresenter
             */
            ElementContentTop: null,
            /**
             * Inherited code: Requires comment.
             *
             * @instance
             * @private
             * @memberof Windows.UI.Xaml.Controls.TabControl
             * @type number
             */
            _desiredIndex: 0,
            /**
             * Inherited code: Requires comment.
             *
             * @instance
             * @private
             * @memberof Windows.UI.Xaml.Controls.TabControl
             * @default true
             * @type boolean
             */
            _updateIndex: false
        },
        events: {
            /**
             * Occurs when the selected
             TabItem changes.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Controls.TabControl
             * @memberof Windows.UI.Xaml.Controls.TabControl
             * @function addSelectionChanged
             * @param   {Windows.UI.Xaml.Controls.SelectionChangedEventHandler}    value
             * @return  {void}
             */
            /**
             * Occurs when the selected
             TabItem changes.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Controls.TabControl
             * @memberof Windows.UI.Xaml.Controls.TabControl
             * @function removeSelectionChanged
             * @param   {Windows.UI.Xaml.Controls.SelectionChangedEventHandler}    value
             * @return  {void}
             */
            SelectionChanged: null
        },
        props: {
            /**
             * Gets or sets the currently selected TabItem.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.TabControl
             * @function SelectedItem
             * @type System.Object
             */
            SelectedItem: {
                get: function () {
                    return this.GetValue(Windows.UI.Xaml.Controls.TabControl.SelectedItemProperty);
                },
                set: function (value) {
                    if (value == null || this.Items.contains(value)) {
                        this.SetValue(Windows.UI.Xaml.Controls.TabControl.SelectedItemProperty, value);
                    }
                }
            },
            /**
             * Gets or sets the index of the currently selected TabItem.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.TabControl
             * @function SelectedIndex
             * @type number
             */
            SelectedIndex: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.Controls.TabControl.SelectedIndexProperty), System.Int32), System.Int32));
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Controls.TabControl.SelectedIndexProperty, Bridge.box(value, System.Int32));
                }
            },
            /**
             * Gets the content of the currently selected TabItem.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.TabControl
             * @function SelectedContent
             * @type System.Object
             */
            SelectedContent: {
                get: function () {
                    return this.GetValue(Windows.UI.Xaml.Controls.TabControl.SelectedContentProperty);
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Controls.TabControl.SelectedContentProperty, value);
                }
            }
        },
        ctors: {
            init: function () {
                this._updateIndex = true;
            },
            /**
             * Initializes a new instance of the
             TabControl class.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Controls.TabControl
             * @memberof Windows.UI.Xaml.Controls.TabControl
             * @return  {void}
             */
            ctor: function () {
                this.$initialize();
                Windows.UI.Xaml.Controls.ItemsControl.ctor.call(this);
                this.SelectedIndex = -1;
                this.addKeyDown(function (sender, e) { /* OnKeyDown(e);*/
                });
                this.addSelectionChanged(Bridge.fn.bind(this, function (sender, e) {
                    this.OnSelectionChanged(e);
                }));
                this.addIsEnabledChanged(Bridge.fn.cacheBind(this, this.OnIsEnabledChanged));
                this.DefaultStyleKey = Windows.UI.Xaml.Controls.TabControl;
            }
        },
        methods: {
            /**
             * Builds the visual tree for the TabControl when a new template is applied.
             *
             * @instance
             * @protected
             * @override
             * @this Windows.UI.Xaml.Controls.TabControl
             * @memberof Windows.UI.Xaml.Controls.TabControl
             * @return  {void}
             */
            OnApplyTemplate: function () {
                var $t;
                Windows.UI.Xaml.Controls.ItemsControl.prototype.OnApplyTemplate.call(this);

                if (this.ElementTabPanelTop != null) {
                    this.ElementTabPanelTop.Children.clear();
                }

                var contentHost = this.ElementContentTop;
                if (contentHost != null) {
                    contentHost.Content = null;
                }

                this.ElementTemplateTop = Bridge.as(this.GetTemplateChild(Windows.UI.Xaml.Controls.TabControl.ElementTemplateTopName), Windows.UI.Xaml.FrameworkElement);
                this.ElementTabPanelTop = Bridge.as(this.GetTemplateChild(Windows.UI.Xaml.Controls.TabControl.ElementTabPanelTopName), Windows.UI.Xaml.Controls.TabPanel);
                this.ElementContentTop = Bridge.as(this.GetTemplateChild(Windows.UI.Xaml.Controls.TabControl.ElementContentTopName), Windows.UI.Xaml.Controls.ContentPresenter);

                $t = Bridge.getEnumerator(this.Items);
                try {
                    while ($t.moveNext()) {
                        var item = $t.Current;
                        var tabItem = Bridge.as(item, Windows.UI.Xaml.Controls.TabItem);
                        if (tabItem == null) {
                            throw new System.ArgumentException.$ctor1(System.String.format("Unable to cast object of type '{0}' to type '{1}'.", Bridge.getTypeName(Bridge.getType(item)), Windows.UI.Xaml.Controls.TabItem));
                        }
                        this.AddToTabPanel(tabItem);
                    }
                } finally {
                    if (Bridge.is($t, System.IDisposable)) {
                        $t.System$IDisposable$Dispose();
                    }
                }
                if (this.SelectedIndex < 0) {
                    this.SelectedIndex = 0;
                } else {
                    this.UpdateSelectedContent(this.SelectedContent);
                }

                this.UpdateTabPanelLayout();
                this.ChangeVisualState(false);
            },
            /**
             * Given the TabItem in the list of Items, we will set that item as the
             currently selected item, and un-select the rest of the items.
             *
             * @instance
             * @private
             * @this Windows.UI.Xaml.Controls.TabControl
             * @memberof Windows.UI.Xaml.Controls.TabControl
             * @param   {Windows.UI.Xaml.Controls.TabItem}    oldItem    Previous TabItem that was unselected.
             * @param   {Windows.UI.Xaml.Controls.TabItem}    newItem    New TabItem to set as the SelectedItem.
             * @return  {void}
             */
            SelectItem: function (oldItem, newItem) {
                var $t;
                if (newItem == null) {
                    var contentHost = this.ElementContentTop;
                    if (contentHost != null) {
                        contentHost.Content = null;
                    }
                    this.SetValue(Windows.UI.Xaml.Controls.TabControl.SelectedContentProperty, null);
                }

                $t = Bridge.getEnumerator(this.Items);
                try {
                    while ($t.moveNext()) {
                        var item = $t.Current;
                        var tabItem = Bridge.as(item, Windows.UI.Xaml.Controls.TabItem);
                        if (tabItem == null) {
                            throw new System.ArgumentException.$ctor1(System.String.format("Unable to cast object of type '{0}' to type '{1}'.", Bridge.getTypeName(Bridge.getType(item)), Windows.UI.Xaml.Controls.TabItem));
                        }
                        if (!Bridge.referenceEquals(tabItem, newItem) && tabItem.IsSelected) {
                            tabItem.IsSelected = false;
                        } else if (Bridge.referenceEquals(tabItem, newItem)) {
                            tabItem.IsSelected = true;
                            this.SetValue(Windows.UI.Xaml.Controls.TabControl.SelectedContentProperty, tabItem.Content);
                        }
                    }
                } finally {
                    if (Bridge.is($t, System.IDisposable)) {
                        $t.System$IDisposable$Dispose();
                    }
                }

                var handler = this.SelectionChanged;
                if (!Bridge.staticEquals(handler, null)) {
                    var args = new Windows.UI.Xaml.Controls.SelectionChangedEventArgs((oldItem == null ? new (System.Collections.Generic.List$1(Windows.UI.Xaml.Controls.TabItem)).ctor() : function (_o1) {
                            _o1.add(oldItem);
                            return _o1;
                        }(new (System.Collections.Generic.List$1(Windows.UI.Xaml.Controls.TabItem)).ctor())), (newItem == null ? new (System.Collections.Generic.List$1(Windows.UI.Xaml.Controls.TabItem)).ctor() : function (_o2) {
                            _o2.add(newItem);
                            return _o2;
                        }(new (System.Collections.Generic.List$1(Windows.UI.Xaml.Controls.TabItem)).ctor())));
                    handler(this, args);
                }
            },
            /**
             * Called when the IsEnabled property changes.
             *
             * @instance
             * @private
             * @this Windows.UI.Xaml.Controls.TabControl
             * @memberof Windows.UI.Xaml.Controls.TabControl
             * @param   {System.Object}                                         sender    Control that triggers this property change.
             * @param   {Windows.UI.Xaml.DependencyPropertyChangedEventArgs}    e         Property changed args.
             * @return  {void}
             */
            OnIsEnabledChanged: function (sender, e) {
                this.UpdateVisualState();
            },
            /**
             * This will hide the templates for the TabStripPlacements that are not
             being displayed, and show the template for the TabStripPlacement
             that is currently selected.
             *
             * @instance
             * @private
             * @this Windows.UI.Xaml.Controls.TabControl
             * @memberof Windows.UI.Xaml.Controls.TabControl
             * @return  {void}
             */
            UpdateTabPanelLayout: function () {
                var $t;
                var newTemplate = this.ElementTemplateTop;
                var newPanel = this.ElementTabPanelTop;
                var newContentHost = this.ElementContentTop;

                if (newPanel != null) {
                    $t = Bridge.getEnumerator(this.Items);
                    try {
                        while ($t.moveNext()) {
                            var item = $t.Current;
                            var tabItem = Bridge.as(item, Windows.UI.Xaml.Controls.TabItem);
                            if (tabItem == null) {
                                throw new System.ArgumentException.$ctor1(System.String.format("Unable to cast object of type '{0}' to type '{1}'.", Bridge.getTypeName(Bridge.getType(item)), Windows.UI.Xaml.Controls.TabItem));
                            }
                            this.AddToTabPanel(tabItem);
                        }
                    } finally {
                        if (Bridge.is($t, System.IDisposable)) {
                            $t.System$IDisposable$Dispose();
                        }
                    }
                }
                if (newContentHost != null) {
                    newContentHost.Content = this.SelectedContent;
                }
                if (newTemplate != null) {
                    newTemplate.Visibility = Windows.UI.Xaml.Visibility.Visible;
                }
            },
            /**
             * Update the current visual state of the TabControl.
             *
             * @instance
             * @this Windows.UI.Xaml.Controls.TabControl
             * @memberof Windows.UI.Xaml.Controls.TabControl
             * @return  {void}
             */
            UpdateVisualState: function () {
                this.ChangeVisualState(true);
            },
            /**
             * Change to the correct visual state for the TabControl.
             *
             * @instance
             * @this Windows.UI.Xaml.Controls.TabControl
             * @memberof Windows.UI.Xaml.Controls.TabControl
             * @param   {boolean}    useTransitions    True to use transitions when updating the visual state, false to
             snap directly to the new visual state.
             * @return  {void}
             */
            ChangeVisualState: function (useTransitions) {
                if (!this.IsEnabled) {
                    Windows.UI.Xaml.Controls.VisualStates.GoToState(this, useTransitions, [Windows.UI.Xaml.Controls.VisualStates.StateDisabled, Windows.UI.Xaml.Controls.VisualStates.StateNormal]);
                } else {
                    Windows.UI.Xaml.Controls.VisualStates.GoToState(this, useTransitions, [Windows.UI.Xaml.Controls.VisualStates.StateNormal]);
                }
            },
            /**
             * Updates the current selection when Items has changed.
             *
             * @instance
             * @protected
             * @override
             * @this Windows.UI.Xaml.Controls.TabControl
             * @memberof Windows.UI.Xaml.Controls.TabControl
             * @param   {System.Collections.Specialized.NotifyCollectionChangedEventArgs}    e    Data used by the event.
             * @return  {void}
             */
            OnItemsChanged: function (e) {
                var $t, $t1, $t2, $t3;
                switch (e.Action) {
                    case System.Collections.Specialized.NotifyCollectionChangedAction.Add: 
                        var newSelectedIndex = -1;
                        $t = Bridge.getEnumerator(e.NewItems);
                        try {
                            while ($t.moveNext()) {
                                var o = $t.Current;
                                var tabItem = Bridge.as(o, Windows.UI.Xaml.Controls.TabItem);
                                if (tabItem == null) {
                                    throw new System.ArgumentException.$ctor1(System.String.format("Unable to cast object of type '{0}' to type '{1}'.", Bridge.getTypeName(Bridge.getType(o)), Windows.UI.Xaml.Controls.TabItem));
                                }
                                var index = this.Items.indexOf(tabItem);
                                this.InsertIntoTabPanel(index, tabItem);

                                if (tabItem.IsSelected) {
                                    newSelectedIndex = index;
                                } else if (!Bridge.referenceEquals(this.SelectedItem, this.GetItemAtIndex(this.SelectedIndex))) {
                                    newSelectedIndex = this.Items.indexOf(this.SelectedItem);
                                } else if ((this._desiredIndex < this.Items.Count) && (this._desiredIndex >= 0)) {
                                    newSelectedIndex = this._desiredIndex;
                                }

                                tabItem.UpdateVisualState();
                            }
                        } finally {
                            if (Bridge.is($t, System.IDisposable)) {
                                $t.System$IDisposable$Dispose();
                            }
                        }
                        if (newSelectedIndex === -1) {
                            $t1 = Bridge.getEnumerator(this.Items);
                            try {
                                while ($t1.moveNext()) {
                                    var item = $t1.Current;
                                    var tabItem1 = Bridge.as(item, Windows.UI.Xaml.Controls.TabItem);
                                    if (tabItem1 == null) {
                                        throw new System.ArgumentException.$ctor1(System.String.format("Unable to cast object of type '{0}' to type '{1}'.", Bridge.getTypeName(Bridge.getType(item)), Windows.UI.Xaml.Controls.TabItem));
                                    }

                                    if (tabItem1.IsSelected) {
                                        return;
                                    }
                                }
                            } finally {
                                if (Bridge.is($t1, System.IDisposable)) {
                                    $t1.System$IDisposable$Dispose();
                                }
                            }

                            if (this.Items.Count > 1 || System.Nullable.neq((Bridge.as((Bridge.as(this.Items.getItem(0), Windows.UI.Xaml.Controls.TabItem)).ReadLocalValue(Windows.UI.Xaml.Controls.TabItem.IsSelectedProperty), System.Boolean, true)), false)) {
                                newSelectedIndex = 0;
                            }
                        }
                        this.SelectedItem = this.GetItemAtIndex(newSelectedIndex);
                        this.SelectedIndex = newSelectedIndex;
                        break;
                    case System.Collections.Specialized.NotifyCollectionChangedAction.Remove: 
                        $t2 = Bridge.getEnumerator(e.OldItems);
                        try {
                            while ($t2.moveNext()) {
                                var o1 = $t2.Current;
                                var tabItem2 = Bridge.as(o1, Windows.UI.Xaml.Controls.TabItem);
                                this.RemoveFromTabPanel(tabItem2);

                                if (this.Items.Count === 0) {
                                    this.SelectedIndex = -1;
                                } else if (this.Items.Count <= this.SelectedIndex) {
                                    this.SelectedIndex = (this.Items.Count - 1) | 0;
                                } else {
                                    this.SelectedItem = this.GetItemAtIndex(this.SelectedIndex);
                                }
                            }
                        } finally {
                            if (Bridge.is($t2, System.IDisposable)) {
                                $t2.System$IDisposable$Dispose();
                            }
                        }
                        break;
                    case System.Collections.Specialized.NotifyCollectionChangedAction.Reset: 
                        this.ClearTabPanel();
                        this.SelectedIndex = -1;
                        $t3 = Bridge.getEnumerator(this.Items);
                        try {
                            while ($t3.moveNext()) {
                                var item1 = $t3.Current;
                                var tabItem3 = Bridge.as(item1, Windows.UI.Xaml.Controls.TabItem);
                                if (tabItem3 == null) {
                                    throw new System.ArgumentException.$ctor1(System.String.format("Unable to cast object of type '{0}' to type '{1}'.", Bridge.getTypeName(Bridge.getType(item1)), Windows.UI.Xaml.Controls.TabItem));
                                }
                                this.AddToTabPanel(tabItem3);
                                if (tabItem3.IsSelected) {
                                    this.SelectedItem = tabItem3;
                                }
                            }
                        } finally {
                            if (Bridge.is($t3, System.IDisposable)) {
                                $t3.System$IDisposable$Dispose();
                            }
                        }
                        if (this.SelectedIndex === -1 && this.Items.Count > 0) {
                            this.SelectedIndex = 0;
                        }
                        break;
                    case System.Collections.Specialized.NotifyCollectionChangedAction.Replace: 
                        break;
                }
            },
            /**
             * Raises the TabControl.SelectionChanged event.
             *
             * @instance
             * @protected
             * @this Windows.UI.Xaml.Controls.TabControl
             * @memberof Windows.UI.Xaml.Controls.TabControl
             * @param   {Windows.UI.Xaml.Controls.SelectionChangedEventArgs}    args    Provides data for the TabControl.SelectionChanged event.
             * @return  {void}
             */
            OnSelectionChanged: function (args) { },
            /**
             * This is the method that responds to the KeyDown event.
             *
             * @instance
             * @protected
             * @override
             * @this Windows.UI.Xaml.Controls.TabControl
             * @memberof Windows.UI.Xaml.Controls.TabControl
             * @param   {Windows.UI.Xaml.Input.KeyRoutedEventArgs}    e    Data used by the event.
             * @return  {void}
             */
            OnKeyDown: function (e) {
                Windows.UI.Xaml.Controls.ItemsControl.prototype.OnKeyDown.call(this, e);
                if (e.Handled) {
                    return;
                }
                var nextTabItem = null;

                var direction = 0;
                var startIndex = -1;
                switch (e.Key) {
                    case Windows.System.VirtualKey.Home: 
                        direction = 1;
                        startIndex = -1;
                        break;
                    case Windows.System.VirtualKey.End: 
                        direction = -1;
                        startIndex = this.Items.Count;
                        break;
                    default: 
                        return;
                }

                nextTabItem = this.FindNextTabItem(startIndex, direction);

                if (nextTabItem != null && !Bridge.referenceEquals(nextTabItem, this.SelectedItem)) {
                    e.Handled = true;
                    this.SelectedItem = nextTabItem;
                    nextTabItem.Focus();
                }
            },
            /**
             * Inherited code: Requires comment.
             *
             * @instance
             * @this Windows.UI.Xaml.Controls.TabControl
             * @memberof Windows.UI.Xaml.Controls.TabControl
             * @param   {number}                              startIndex    Inherited code: Requires comment 1.
             * @param   {number}                              direction     Inherited code: Requires comment 2.
             * @return  {Windows.UI.Xaml.Controls.TabItem}                  Inherited code: Requires comment 3.
             */
            FindNextTabItem: function (startIndex, direction) {
                var nextTabItem = null;

                var index = startIndex;
                for (var i = 0; i < this.Items.Count; i = (i + 1) | 0) {
                    index = (index + direction) | 0;
                    if (index >= this.Items.Count) {
                        index = 0;
                    } else if (index < 0) {
                        index = (this.Items.Count - 1) | 0;
                    }

                    var tabItem = this.GetItemAtIndex(index);
                    if (tabItem != null && tabItem.IsEnabled && tabItem.Visibility === Windows.UI.Xaml.Visibility.Visible) {
                        nextTabItem = tabItem;
                        break;
                    }
                }
                return nextTabItem;
            },
            /**
             * Inherited code: Requires comment.
             *
             * @instance
             * @private
             * @this Windows.UI.Xaml.Controls.TabControl
             * @memberof Windows.UI.Xaml.Controls.TabControl
             * @param   {number}                              index    Inherited code: Requires comment 1.
             * @return  {Windows.UI.Xaml.Controls.TabItem}             Inherited code: Requires comment 2.
             */
            GetItemAtIndex: function (index) {
                if (index < 0 || index >= this.Items.Count) {
                    return null;
                } else {
                    return Bridge.as(this.Items.getItem(index), Windows.UI.Xaml.Controls.TabItem);
                }
            },
            /**
             * Inherited code: Requires comment.
             *
             * @instance
             * @private
             * @this Windows.UI.Xaml.Controls.TabControl
             * @memberof Windows.UI.Xaml.Controls.TabControl
             * @param   {System.Object}    content    Inherited code: Requires comment 1.
             * @return  {void}
             */
            UpdateSelectedContent: function (content) {
                var tabItem = Bridge.as(this.SelectedItem, Windows.UI.Xaml.Controls.TabItem);
                if (tabItem != null) {
                    var contentHost = this.ElementContentTop;
                    if (contentHost != null) {
                        contentHost.HorizontalAlignment = tabItem.HorizontalContentAlignment;
                        contentHost.VerticalAlignment = tabItem.VerticalContentAlignment;
                        contentHost.ContentTemplate = tabItem.ContentTemplate;
                        contentHost.Content = content;
                    }
                }
            },
            /**
             * Inherited code: Requires comment.
             *
             * @instance
             * @private
             * @this Windows.UI.Xaml.Controls.TabControl
             * @memberof Windows.UI.Xaml.Controls.TabControl
             * @param   {Windows.UI.Xaml.Controls.TabItem}    tabItem    Inherited code: Requires comment 1.
             * @return  {void}
             */
            AddToTabPanel: function (tabItem) {
                var panel = this.ElementTabPanelTop;
                if (panel != null && !panel.Children.contains(tabItem)) {
                    panel.Children.add(tabItem);
                }
            },
            /**
             * Inherited code: Requires comment.
             *
             * @instance
             * @private
             * @this Windows.UI.Xaml.Controls.TabControl
             * @memberof Windows.UI.Xaml.Controls.TabControl
             * @param   {number}                              index      Inherited code: Requires comment 1.
             * @param   {Windows.UI.Xaml.Controls.TabItem}    tabItem    Inherited code: Requires comment 2.
             * @return  {void}
             */
            InsertIntoTabPanel: function (index, tabItem) {
                var panel = this.ElementTabPanelTop;
                if (panel != null && !panel.Children.contains(tabItem)) {
                    panel.Children.insert(index, tabItem);
                }
            },
            /**
             * Inherited code: Requires comment.
             *
             * @instance
             * @private
             * @this Windows.UI.Xaml.Controls.TabControl
             * @memberof Windows.UI.Xaml.Controls.TabControl
             * @param   {Windows.UI.Xaml.Controls.TabItem}    tabItem    Inherited code: Requires comment 1.
             * @return  {void}
             */
            RemoveFromTabPanel: function (tabItem) {
                var panel = this.ElementTabPanelTop;
                if (panel != null && panel.Children.contains(tabItem)) {
                    panel.Children.remove(tabItem);
                }
            },
            /**
             * Inherited code: Requires comment.
             *
             * @instance
             * @private
             * @this Windows.UI.Xaml.Controls.TabControl
             * @memberof Windows.UI.Xaml.Controls.TabControl
             * @return  {void}
             */
            ClearTabPanel: function () {
                var panel = this.ElementTabPanelTop;
                if (panel != null) {
                    panel.Children.clear();
                }
            }
        }
    });

    /**
     * Represents a selectable item in a
     {@link }.
     *
     * @public
     * @class Windows.UI.Xaml.Controls.TabItem
     * @augments Windows.UI.Xaml.Controls.ContentControl
     */
    Bridge.define("Windows.UI.Xaml.Controls.TabItem", {
        inherits: [Windows.UI.Xaml.Controls.ContentControl],
        statics: {
            fields: {
                /**
                 * Inherited code: Requires comment.
                 *
                 * @static
                 * @memberof Windows.UI.Xaml.Controls.TabItem
                 * @constant
                 * @default "TemplateTopSelected"
                 * @type string
                 */
                ElementTemplateTopSelectedName: null,
                /**
                 * Inherited code: Requires comment.
                 *
                 * @static
                 * @memberof Windows.UI.Xaml.Controls.TabItem
                 * @constant
                 * @default "TemplateTopUnselected"
                 * @type string
                 */
                ElementTemplateTopUnselectedName: null,
                /**
                 * Inherited code: Requires comment.
                 *
                 * @static
                 * @memberof Windows.UI.Xaml.Controls.TabItem
                 * @constant
                 * @default "HeaderTopSelected"
                 * @type string
                 */
                ElementHeaderTopSelectedName: null,
                /**
                 * Inherited code: Requires comment.
                 *
                 * @static
                 * @memberof Windows.UI.Xaml.Controls.TabItem
                 * @constant
                 * @default "HeaderTopUnselected"
                 * @type string
                 */
                ElementHeaderTopUnselectedName: null,
                /**
                 * Identifies the
                 {@link }
                 dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Controls.TabItem
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                HeaderProperty: null,
                /**
                 * Identifies the
                 {@link }
                 dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Controls.TabItem
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                HasHeaderProperty: null,
                /**
                 * Identifies the
                 {@link }
                 dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Controls.TabItem
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                HeaderTemplateProperty: null,
                /**
                 * Identifies the
                 {@link }
                 dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Controls.TabItem
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                IsSelectedProperty: null,
                /**
                 * Identifies the
                 {@link }
                 dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Controls.TabItem
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                IsFocusedProperty: null,
                /**
                 * Identifies the SelectedBackground dependency property
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Controls.TabItem
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                SelectedBackgroundProperty: null,
                /**
                 * Identifies the SelectedForeground dependency property
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Controls.TabItem
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                SelectedForegroundProperty: null,
                /**
                 * Identifies the SelectedAccent dependency property
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Controls.TabItem
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                SelectedAccentProperty: null
            },
            ctors: {
                init: function () {
                    this.ElementTemplateTopSelectedName = "TemplateTopSelected";
                    this.ElementTemplateTopUnselectedName = "TemplateTopUnselected";
                    this.ElementHeaderTopSelectedName = "HeaderTopSelected";
                    this.ElementHeaderTopUnselectedName = "HeaderTopUnselected";
                    this.HeaderProperty = Windows.UI.Xaml.DependencyProperty.Register("Header", System.Object, Windows.UI.Xaml.Controls.TabItem, new Windows.UI.Xaml.PropertyMetadata.$ctor2(null, Windows.UI.Xaml.Controls.TabItem.OnHeaderChanged));
                    this.HasHeaderProperty = Windows.UI.Xaml.DependencyProperty.Register("HasHeader", System.Boolean, Windows.UI.Xaml.Controls.TabItem, new Windows.UI.Xaml.PropertyMetadata.$ctor1(Bridge.box(false, System.Boolean, System.Boolean.toString)));
                    this.HeaderTemplateProperty = Windows.UI.Xaml.DependencyProperty.Register("HeaderTemplate", Windows.UI.Xaml.DataTemplate, Windows.UI.Xaml.Controls.TabItem, new Windows.UI.Xaml.PropertyMetadata.$ctor2(null, Windows.UI.Xaml.Controls.TabItem.OnHeaderTemplateChanged));
                    this.IsSelectedProperty = Windows.UI.Xaml.DependencyProperty.Register("IsSelected", System.Boolean, Windows.UI.Xaml.Controls.TabItem, new Windows.UI.Xaml.PropertyMetadata.$ctor2(Bridge.box(false, System.Boolean, System.Boolean.toString), Windows.UI.Xaml.Controls.TabItem.OnIsSelectedChanged));
                    this.IsFocusedProperty = Windows.UI.Xaml.DependencyProperty.Register("IsFocused", System.Boolean, Windows.UI.Xaml.Controls.TabItem, new Windows.UI.Xaml.PropertyMetadata.$ctor2(Bridge.box(false, System.Boolean, System.Boolean.toString), Windows.UI.Xaml.Controls.TabItem.OnIsFocusedPropertyChanged));
                    this.SelectedBackgroundProperty = Windows.UI.Xaml.DependencyProperty.Register("SelectedBackground", Windows.UI.Xaml.Media.Brush, Windows.UI.Xaml.Controls.TabItem, new Windows.UI.Xaml.PropertyMetadata.$ctor1(new Windows.UI.Xaml.Media.SolidColorBrush.$ctor1(Windows.UI.Colors.White.$clone())));
                    this.SelectedForegroundProperty = Windows.UI.Xaml.DependencyProperty.Register("SelectedForeground", Windows.UI.Xaml.Media.Brush, Windows.UI.Xaml.Controls.TabItem, new Windows.UI.Xaml.PropertyMetadata.$ctor1(new Windows.UI.Xaml.Media.SolidColorBrush.$ctor1(Windows.UI.Colors.Black.$clone())));
                    this.SelectedAccentProperty = Windows.UI.Xaml.DependencyProperty.Register("SelectedAccent", Windows.UI.Xaml.Media.Brush, Windows.UI.Xaml.Controls.TabItem, new Windows.UI.Xaml.PropertyMetadata.$ctor1(new Windows.UI.Xaml.Media.SolidColorBrush.$ctor1(Windows.UI.Colors.Blue.$clone())));
                }
            },
            methods: {
                /**
                 * Header property changed handler.
                 *
                 * @static
                 * @private
                 * @this Windows.UI.Xaml.Controls.TabItem
                 * @memberof Windows.UI.Xaml.Controls.TabItem
                 * @param   {Windows.UI.Xaml.DependencyObject}                      d    TabItem that changed its Header.
                 * @param   {Windows.UI.Xaml.DependencyPropertyChangedEventArgs}    e    The DependencyPropertyChangedEventArgs.
                 * @return  {void}
                 */
                OnHeaderChanged: function (d, e) {
                    var ctrl = Bridge.cast(d, Windows.UI.Xaml.Controls.TabItem);

                    ctrl.HasHeader = (e.NewValue != null) ? true : false;
                    ctrl.OnHeaderChanged(e.OldValue, e.NewValue);
                },
                /**
                 * HeaderTemplate property changed handler.
                 *
                 * @static
                 * @private
                 * @this Windows.UI.Xaml.Controls.TabItem
                 * @memberof Windows.UI.Xaml.Controls.TabItem
                 * @param   {Windows.UI.Xaml.DependencyObject}                      d    TabItem that changed its HeaderTemplate.
                 * @param   {Windows.UI.Xaml.DependencyPropertyChangedEventArgs}    e    The DependencyPropertyChangedEventArgs.
                 * @return  {void}
                 */
                OnHeaderTemplateChanged: function (d, e) {
                    var ctrl = Bridge.cast(d, Windows.UI.Xaml.Controls.TabItem);
                    ctrl.OnHeaderTemplateChanged(Bridge.cast(e.OldValue, Windows.UI.Xaml.DataTemplate), Bridge.cast(e.NewValue, Windows.UI.Xaml.DataTemplate));
                },
                /**
                 * IsSelected changed handler.
                 *
                 * @static
                 * @private
                 * @this Windows.UI.Xaml.Controls.TabItem
                 * @memberof Windows.UI.Xaml.Controls.TabItem
                 * @param   {Windows.UI.Xaml.DependencyObject}                      d    TabItem that changed IsSelected.
                 * @param   {Windows.UI.Xaml.DependencyPropertyChangedEventArgs}    e    The DependencyPropertyChangedEventArgs.
                 * @return  {void}
                 */
                OnIsSelectedChanged: function (d, e) {
                    var tabItem = Bridge.as(d, Windows.UI.Xaml.Controls.TabItem);

                    var isSelected = System.Nullable.getValue(Bridge.cast(Bridge.unbox(e.NewValue, System.Boolean), System.Boolean));

                    var args = new Windows.UI.Xaml.RoutedEventArgs();

                    if (isSelected) {
                        tabItem.OnSelected(args);
                    } else {
                        tabItem.OnUnselected(args);
                    }
                },
                /**
                 * IsFocusedProperty property changed handler.
                 *
                 * @static
                 * @private
                 * @this Windows.UI.Xaml.Controls.TabItem
                 * @memberof Windows.UI.Xaml.Controls.TabItem
                 * @param   {Windows.UI.Xaml.DependencyObject}                      d    TabItem that changed IsFocused.
                 * @param   {Windows.UI.Xaml.DependencyPropertyChangedEventArgs}    e    The DependencyPropertyChangedEventArgs.
                 * @return  {void}
                 */
                OnIsFocusedPropertyChanged: function (d, e) {
                    var ti = Bridge.as(d, Windows.UI.Xaml.Controls.TabItem);

                    ti.OnIsFocusChanged(e);
                }
            }
        },
        fields: {
            /**
             * Gets or sets the TabStripPlacement Top Selected template.
             *
             * @instance
             * @memberof Windows.UI.Xaml.Controls.TabItem
             * @function ElementTemplateTopSelected
             * @type Windows.UI.Xaml.FrameworkElement
             */
            ElementTemplateTopSelected: null,
            /**
             * Gets or sets the TabStripPlacement Top Unselected template.
             *
             * @instance
             * @memberof Windows.UI.Xaml.Controls.TabItem
             * @function ElementTemplateTopUnselected
             * @type Windows.UI.Xaml.FrameworkElement
             */
            ElementTemplateTopUnselected: null,
            /**
             * Gets or sets the Header of the TabStripPlacement Top Selected
             template.
             *
             * @instance
             * @memberof Windows.UI.Xaml.Controls.TabItem
             * @function ElementHeaderTopSelected
             * @type Windows.UI.Xaml.Controls.ContentControl
             */
            ElementHeaderTopSelected: null,
            /**
             * Gets or sets the Header of the TabStripPlacement Top Unselected
             template.
             *
             * @instance
             * @memberof Windows.UI.Xaml.Controls.TabItem
             * @function ElementHeaderTopUnselected
             * @type Windows.UI.Xaml.Controls.ContentControl
             */
            ElementHeaderTopUnselected: null,
            /**
             * Gets or sets a value indicating whether Inherited code: Requires comment.
             *
             * @instance
             * @private
             * @memberof Windows.UI.Xaml.Controls.TabItem
             * @function _isMouseOver
             * @type boolean
             */
            _isMouseOver: false,
            /**
             * Inherited code: Requires comment.
             *
             * @instance
             * @private
             * @memberof Windows.UI.Xaml.Controls.TabItem
             * @type Windows.UI.Xaml.FrameworkElement
             */
            _previousTemplate: null,
            /**
             * Inherited code: Requires comment.
             *
             * @instance
             * @private
             * @memberof Windows.UI.Xaml.Controls.TabItem
             * @type Windows.UI.Xaml.Controls.ContentControl
             */
            _previousHeader: null
        },
        props: {
            /**
             * Gets or sets the header of the
             {@link }.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.TabItem
             * @function Header
             * @type System.Object
             */
            Header: {
                get: function () {
                    return this.GetValue(Windows.UI.Xaml.Controls.TabItem.HeaderProperty);
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Controls.TabItem.HeaderProperty, value);
                }
            },
            /**
             * Gets a value indicating whether the
             {@link } has a header.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.TabItem
             * @function HasHeader
             * @type boolean
             */
            HasHeader: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.Controls.TabItem.HasHeaderProperty), System.Boolean), System.Boolean));
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Controls.TabItem.HasHeaderProperty, Bridge.box(value, System.Boolean, System.Boolean.toString));
                }
            },
            /**
             * Gets or sets the template that is used to display the content of the
             {@link } header.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.TabItem
             * @function HeaderTemplate
             * @type Windows.UI.Xaml.DataTemplate
             */
            HeaderTemplate: {
                get: function () {
                    return Bridge.cast(this.GetValue(Windows.UI.Xaml.Controls.TabItem.HeaderTemplateProperty), Windows.UI.Xaml.DataTemplate);
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Controls.TabItem.HeaderTemplateProperty, value);
                }
            },
            /**
             * Gets or sets a value indicating whether the
             {@link } is currently
             selected.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.TabItem
             * @function IsSelected
             * @type boolean
             */
            IsSelected: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.Controls.TabItem.IsSelectedProperty), System.Boolean), System.Boolean));
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Controls.TabItem.IsSelectedProperty, Bridge.box(value, System.Boolean, System.Boolean.toString));
                }
            },
            /**
             * Gets a value indicating whether this element has logical focus.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.TabItem
             * @function IsFocused
             * @type boolean
             */
            IsFocused: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.Controls.TabItem.IsFocusedProperty), System.Boolean), System.Boolean));
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Controls.TabItem.IsFocusedProperty, Bridge.box(value, System.Boolean, System.Boolean.toString));
                }
            },
            /**
             * Gets a reference to the TabControl that holds this TabItem.  It will
             step up the UI tree to find the TabControl that contains this
             TabItem.
             *
             * @instance
             * @private
             * @readonly
             * @memberof Windows.UI.Xaml.Controls.TabItem
             * @function TabControlParent
             * @type Windows.UI.Xaml.Controls.TabControl
             */
            TabControlParent: {
                get: function () {
                    var tabCtrl = Bridge.as(this.INTERNAL_VisualParent, Windows.UI.Xaml.Controls.TabControl);
                    if (tabCtrl != null) {
                        return tabCtrl;
                    }

                    var obj = Bridge.as(this, Windows.UI.Xaml.DependencyObject);
                    while (obj != null) {
                        var tc = Bridge.as(obj, Windows.UI.Xaml.Controls.TabControl);
                        if (tc != null) {
                            return tc;
                        }
                        obj = Bridge.cast(obj, Windows.UI.Xaml.UIElement).INTERNAL_VisualParent;
                    }
                    return null;
                }
            },
            /**
             * Gets or sets the bakground color of the selected INTERNAL_CorrespondingItem.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.TabItem
             * @function SelectedBackground
             * @type Windows.UI.Xaml.Media.Brush
             */
            SelectedBackground: {
                get: function () {
                    return Bridge.cast(this.GetValue(Windows.UI.Xaml.Controls.TabItem.SelectedBackgroundProperty), Windows.UI.Xaml.Media.Brush);
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Controls.TabItem.SelectedBackgroundProperty, value);
                }
            },
            /**
             * Gets or sets the foreground color of the selected INTERNAL_CorrespondingItem.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.TabItem
             * @function SelectedForeground
             * @type Windows.UI.Xaml.Media.Brush
             */
            SelectedForeground: {
                get: function () {
                    return Bridge.cast(this.GetValue(Windows.UI.Xaml.Controls.TabItem.SelectedForegroundProperty), Windows.UI.Xaml.Media.Brush);
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Controls.TabItem.SelectedForegroundProperty, value);
                }
            },
            /**
             * Gets or sets the accent color of the selected INTERNAL_CorrespondingItem.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.TabItem
             * @function SelectedAccent
             * @type Windows.UI.Xaml.Media.Brush
             */
            SelectedAccent: {
                get: function () {
                    return Bridge.cast(this.GetValue(Windows.UI.Xaml.Controls.TabItem.SelectedAccentProperty), Windows.UI.Xaml.Media.Brush);
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Controls.TabItem.SelectedAccentProperty, value);
                }
            }
        },
        ctors: {
            /**
             * Initializes a new instance of the
             {@link } class.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Controls.TabItem
             * @memberof Windows.UI.Xaml.Controls.TabItem
             * @return  {void}
             */
            ctor: function () {
                this.$initialize();
                Windows.UI.Xaml.Controls.ContentControl.ctor.call(this);
                this.addPointerPressed(Bridge.fn.cacheBind(this, this.OnPointerPressed$1));
                this.addPointerEntered(Bridge.fn.cacheBind(this, this.OnPointerEntered$1));
                this.addPointerExited(Bridge.fn.cacheBind(this, this.OnPointerExited$1));
                this.addGotFocus(Bridge.fn.bind(this, function () {
                    this.IsFocused = true;
                }));
                this.addLostFocus(Bridge.fn.bind(this, function () {
                    this.IsFocused = false;
                }));
                this.addIsEnabledChanged(Bridge.fn.cacheBind(this, this.OnIsEnabledChanged));
                this.DefaultStyleKey = Windows.UI.Xaml.Controls.TabItem;
            }
        },
        methods: {
            /**
             * Builds the visual tree for the
             {@link } when a new template
             is applied.
             *
             * @instance
             * @protected
             * @override
             * @this Windows.UI.Xaml.Controls.TabItem
             * @memberof Windows.UI.Xaml.Controls.TabItem
             * @return  {void}
             */
            OnApplyTemplate: function () {
                Windows.UI.Xaml.Controls.ContentControl.prototype.OnApplyTemplate.call(this);

                var cc = (this.IsSelected ? this.ElementHeaderTopSelected : this.ElementHeaderTopUnselected);
                if (cc != null) {
                    cc.Content = null;
                }

                this.ElementTemplateTopSelected = Bridge.as(this.GetTemplateChild(Windows.UI.Xaml.Controls.TabItem.ElementTemplateTopSelectedName), Windows.UI.Xaml.FrameworkElement);
                this.ElementTemplateTopUnselected = Bridge.as(this.GetTemplateChild(Windows.UI.Xaml.Controls.TabItem.ElementTemplateTopUnselectedName), Windows.UI.Xaml.FrameworkElement);
                this.ElementHeaderTopSelected = Bridge.as(this.GetTemplateChild(Windows.UI.Xaml.Controls.TabItem.ElementHeaderTopSelectedName), Windows.UI.Xaml.Controls.ContentControl);
                this.ElementHeaderTopUnselected = Bridge.as(this.GetTemplateChild(Windows.UI.Xaml.Controls.TabItem.ElementHeaderTopUnselectedName), Windows.UI.Xaml.Controls.ContentControl);

                this.UpdateHeaderVisuals();

                this.ChangeVisualState(false);
            },
            /**
             * Called when the
             {@link } property
             changes.
             *
             * @instance
             * @protected
             * @this Windows.UI.Xaml.Controls.TabItem
             * @memberof Windows.UI.Xaml.Controls.TabItem
             * @param   {System.Object}    oldHeader    The previous value of the
             {@link }
             property.
             * @param   {System.Object}    newHeader    The new value of the
             {@link }
             property.
             * @return  {void}
             */
            OnHeaderChanged: function (oldHeader, newHeader) {
                this.UpdateHeaderVisuals();
            },
            /**
             * Inherited code: Requires comment.
             *
             * @instance
             * @private
             * @this Windows.UI.Xaml.Controls.TabItem
             * @memberof Windows.UI.Xaml.Controls.TabItem
             * @return  {void}
             */
            UpdateHeaderVisuals: function () {
                var header = (this.IsSelected ? this.ElementHeaderTopSelected : this.ElementHeaderTopUnselected);
                if (header != null) {
                    header.ContentTemplate = this.HeaderTemplate;
                    header.Content = this.Header;
                }
            },
            /**
             * Called when the
             {@link }
             property changes.
             *
             * @instance
             * @protected
             * @this Windows.UI.Xaml.Controls.TabItem
             * @memberof Windows.UI.Xaml.Controls.TabItem
             * @param   {Windows.UI.Xaml.DataTemplate}    oldHeaderTemplate    The previous value of the
             {@link }
             property.
             * @param   {Windows.UI.Xaml.DataTemplate}    newHeaderTemplate    The new value of the
             {@link }
             property.
             * @return  {void}
             */
            OnHeaderTemplateChanged: function (oldHeaderTemplate, newHeaderTemplate) {
                this.UpdateHeaderVisuals();
            },
            /**
             * Called to indicate that the
             {@link } property
             has changed to true.
             *
             * @instance
             * @protected
             * @this Windows.UI.Xaml.Controls.TabItem
             * @memberof Windows.UI.Xaml.Controls.TabItem
             * @param   {Windows.UI.Xaml.RoutedEventArgs}    e    A {@link } that contains the
             event data.
             * @return  {void}
             */
            OnSelected: function (e) {
                if (this.TabControlParent != null && !Bridge.referenceEquals(this.TabControlParent.SelectedItem, this)) {
                    this.TabControlParent.SelectedItem = this;
                    this.UpdateTabItemVisuals();
                }

            },
            /**
             * Called to indicate that the
             {@link } property
             has changed to false.
             *
             * @instance
             * @protected
             * @this Windows.UI.Xaml.Controls.TabItem
             * @memberof Windows.UI.Xaml.Controls.TabItem
             * @param   {Windows.UI.Xaml.RoutedEventArgs}    e    A {@link } that contains the
             event data.
             * @return  {void}
             */
            OnUnselected: function (e) {
                if (this.TabControlParent != null && Bridge.referenceEquals(this.TabControlParent.SelectedItem, this)) {
                    this.TabControlParent.SelectedIndex = -1;
                }
                this.UpdateTabItemVisuals();
            },
            /**
             * This method is invoked when the Content property changes.
             *
             * @instance
             * @protected
             * @override
             * @this Windows.UI.Xaml.Controls.TabItem
             * @memberof Windows.UI.Xaml.Controls.TabItem
             * @param   {System.Object}    oldContent    The previous {@link }
             content.
             * @param   {System.Object}    newContent    The new {@link } content.
             * @return  {void}
             */
            OnContentChanged: function (oldContent, newContent) {
                Windows.UI.Xaml.Controls.ContentControl.prototype.OnContentChanged.call(this, oldContent, newContent);

                var tabControl = this.TabControlParent;
                if (tabControl != null) {
                    if (this.IsSelected) {
                        tabControl.SelectedContent = newContent;
                    }
                }
            },
            /**
             * Called when the IsEnabled property changes.
             *
             * @instance
             * @private
             * @this Windows.UI.Xaml.Controls.TabItem
             * @memberof Windows.UI.Xaml.Controls.TabItem
             * @param   {System.Object}                                         sender    Control that triggers this property change.
             * @param   {Windows.UI.Xaml.DependencyPropertyChangedEventArgs}    e         Property changed args.
             * @return  {void}
             */
            OnIsEnabledChanged: function (sender, e) {
                var isEnabled = System.Nullable.getValue(Bridge.cast(Bridge.unbox(e.NewValue, System.Boolean), System.Boolean));
                var header = (this.IsSelected ? this.ElementHeaderTopSelected : this.ElementHeaderTopUnselected);
                if (header != null) {
                    if (!isEnabled) {
                        this._isMouseOver = false;
                    }

                    this.UpdateVisualState();
                }
            },
            /**
             * Called when the IsFocused property changes.
             *
             * @instance
             * @protected
             * @this Windows.UI.Xaml.Controls.TabItem
             * @memberof Windows.UI.Xaml.Controls.TabItem
             * @param   {Windows.UI.Xaml.DependencyPropertyChangedEventArgs}    e    A {@link }
             that contains the event data.
             * @return  {void}
             */
            OnIsFocusChanged: function (e) {
                this.UpdateVisualState();
            },
            /**
             * Change to the correct visual state for the TabItem.
             *
             * @instance
             * @this Windows.UI.Xaml.Controls.TabItem
             * @memberof Windows.UI.Xaml.Controls.TabItem
             * @return  {void}
             */
            UpdateVisualState: function () {
                this.ChangeVisualState(true);
            },
            /**
             * Change to the correct visual state for the TabItem.
             *
             * @instance
             * @private
             * @this Windows.UI.Xaml.Controls.TabItem
             * @memberof Windows.UI.Xaml.Controls.TabItem
             * @param   {boolean}    useTransitions    True to use transitions when updating the visual state, false to
             snap directly to the new visual state.
             * @return  {void}
             */
            ChangeVisualState: function (useTransitions) {
                this.UpdateTabItemVisuals();

                if (!this.IsEnabled || (this.TabControlParent != null && !this.TabControlParent.IsEnabled)) {
                    Windows.UI.Xaml.Controls.VisualStates.GoToState(this, useTransitions, [Windows.UI.Xaml.Controls.VisualStates.StateDisabled, Windows.UI.Xaml.Controls.VisualStates.StateNormal]);
                } else if (this._isMouseOver && !this.IsSelected) {
                    Windows.UI.Xaml.Controls.VisualStates.GoToState(this, useTransitions, [Windows.UI.Xaml.Controls.VisualStates.StateMouseOver, Windows.UI.Xaml.Controls.VisualStates.StateNormal]);
                } else {
                    Windows.UI.Xaml.Controls.VisualStates.GoToState(this, useTransitions, [Windows.UI.Xaml.Controls.VisualStates.StateNormal]);
                }

                if (this.IsSelected) {
                    Windows.UI.Xaml.Controls.VisualStates.GoToState(this, useTransitions, [Windows.UI.Xaml.Controls.VisualStates.StateSelected, Windows.UI.Xaml.Controls.VisualStates.StateUnselected]);
                } else {
                    Windows.UI.Xaml.Controls.VisualStates.GoToState(this, useTransitions, [Windows.UI.Xaml.Controls.VisualStates.StateUnselected]);
                }

                if (this.IsFocused && this.IsEnabled) {
                    Windows.UI.Xaml.Controls.VisualStates.GoToState(this, useTransitions, [Windows.UI.Xaml.Controls.VisualStates.StateFocused, Windows.UI.Xaml.Controls.VisualStates.StateUnfocused]);
                } else {
                    Windows.UI.Xaml.Controls.VisualStates.GoToState(this, useTransitions, [Windows.UI.Xaml.Controls.VisualStates.StateUnfocused]);
                }
            },
            /**
             * Inherited code: Requires comment.
             *
             * @instance
             * @private
             * @this Windows.UI.Xaml.Controls.TabItem
             * @memberof Windows.UI.Xaml.Controls.TabItem
             * @return  {void}
             */
            UpdateTabItemVisuals: function () {
                var currentTemplate = (this.IsSelected ? this.ElementTemplateTopSelected : this.ElementTemplateTopUnselected);

                if (this._previousTemplate != null && !Bridge.referenceEquals(this._previousTemplate, currentTemplate)) {
                    this._previousTemplate.Visibility = Windows.UI.Xaml.Visibility.Collapsed;
                }
                this._previousTemplate = currentTemplate;
                if (currentTemplate != null) {
                    currentTemplate.Visibility = Windows.UI.Xaml.Visibility.Visible;
                }

                var currentHeader = (this.IsSelected ? this.ElementHeaderTopSelected : this.ElementHeaderTopUnselected);

                if (this._previousHeader != null && !Bridge.referenceEquals(this._previousHeader, currentHeader)) {
                    this._previousHeader.Content = null;
                }
                this._previousHeader = currentHeader;
                this.UpdateHeaderVisuals();
            },
            /**
             * Handles when the mouse leaves the control.
             *
             * @instance
             * @private
             * @this Windows.UI.Xaml.Controls.TabItem
             * @memberof Windows.UI.Xaml.Controls.TabItem
             * @param   {System.Object}                                   sender    The source of the event.
             * @param   {Windows.UI.Xaml.Input.PointerRoutedEventArgs}    e         The MouseEventArgs.
             * @return  {void}
             */
            OnPointerExited$1: function (sender, e) {
                this._isMouseOver = false;
                this.UpdateVisualState();
            },
            /**
             * Handles when the mouse enters the control.
             *
             * @instance
             * @private
             * @this Windows.UI.Xaml.Controls.TabItem
             * @memberof Windows.UI.Xaml.Controls.TabItem
             * @param   {System.Object}                                   sender    The source of the event.
             * @param   {Windows.UI.Xaml.Input.PointerRoutedEventArgs}    e         The MouseEventArgs.
             * @return  {void}
             */
            OnPointerEntered$1: function (sender, e) {
                this._isMouseOver = true;
                this.UpdateVisualState();
            },
            /**
             * Handles the mouse left button down.
             *
             * @instance
             * @private
             * @this Windows.UI.Xaml.Controls.TabItem
             * @memberof Windows.UI.Xaml.Controls.TabItem
             * @param   {System.Object}                                   sender    The source of the event.
             * @param   {Windows.UI.Xaml.Input.PointerRoutedEventArgs}    e         The MouseButtonEventArgs.
             * @return  {void}
             */
            OnPointerPressed$1: function (sender, e) {
                if (this.IsEnabled && this.TabControlParent != null && !this.IsSelected && !e.Handled) {
                    this.TabControlParent.SelectedIndex = this.TabControlParent.Items.indexOf(this);
                }
            }
        }
    });

    /**
     * Handles the layout of {@link }
     objects on a {@link }.
     *
     * @public
     * @class Windows.UI.Xaml.Controls.TabPanel
     * @augments Windows.UI.Xaml.Controls.WrapPanel
     */
    Bridge.define("Windows.UI.Xaml.Controls.TabPanel", {
        inherits: [Windows.UI.Xaml.Controls.WrapPanel],
        ctors: {
            ctor: function () {
                this.$initialize();
                Windows.UI.Xaml.Controls.WrapPanel.ctor.call(this);
                this.Orientation = Windows.UI.Xaml.Controls.Orientation.Horizontal;
            }
        }
    });

    Bridge.define("Windows.UI.Xaml.Controls.TileViewItem", {
        inherits: [Windows.UI.Xaml.Controls.ContentControl],
        statics: {
            fields: {
                HeaderProperty: null
            },
            ctors: {
                init: function () {
                    this.HeaderProperty = Windows.UI.Xaml.DependencyProperty.Register("Header", System.Object, Windows.UI.Xaml.Controls.TileViewItem, new Windows.UI.Xaml.PropertyMetadata.$ctor4(null));
                }
            }
        },
        fields: {
            _maximizeButton: null,
            _TileViewParent: null
        },
        events: {
            OnMaximize: null,
            OnMinimize: null
        },
        props: {
            Header: {
                get: function () {
                    return this.GetValue(Windows.UI.Xaml.Controls.TileViewItem.HeaderProperty);
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Controls.TileViewItem.HeaderProperty, value);
                }
            }
        },
        ctors: {
            ctor: function () {
                this.$initialize();
                Windows.UI.Xaml.Controls.ContentControl.ctor.call(this);
                this.DefaultStyleKey = Windows.UI.Xaml.Controls.TileViewItem;
            }
        },
        methods: {
            OnApplyTemplate: function () {
                Windows.UI.Xaml.Controls.ContentControl.prototype.OnApplyTemplate.call(this);
                this._maximizeButton = Bridge.as(this.GetTemplateChild("PART_MaximizeButton"), Windows.UI.Xaml.Controls.Button);

                this._maximizeButton.addClick(Bridge.fn.cacheBind(this, this.MaximizeButton_Click));
            },
            MaximizeButton_Click: function (sender, e) {
                if (this._TileViewParent != null) {
                    this._TileViewParent.MaximizeTile(this);
                }
            },
            Maximize: function () {
                Windows.UI.Xaml.VisualStateManager.GoToState(this, "Maximized", false);
                if (!Bridge.staticEquals(this.OnMaximize, null)) {
                    this.OnMaximize(this, null);
                }
            },
            Minimize: function () {
                Windows.UI.Xaml.VisualStateManager.GoToState(this, "Minimized", false);
                if (!Bridge.staticEquals(this.OnMinimize, null)) {
                    this.OnMinimize(this, null);
                }
            }
        }
    });

    Bridge.define("Windows.UI.Xaml.Controls.TimePicker", {
        inherits: [Windows.UI.Xaml.Controls.INTERNAL_DateTimePickerBase],
        statics: {
            fields: {
                /**
                 * Identifies the dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Controls.TimePicker
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                ValueProperty: null
            },
            ctors: {
                init: function () {
                    var $t;
                    this.ValueProperty = Windows.UI.Xaml.DependencyProperty.Register("Value", System.Nullable$1(System.DateTime), Windows.UI.Xaml.Controls.TimePicker, ($t = new Windows.UI.Xaml.PropertyMetadata.$ctor4(Windows.UI.Xaml.Controls.TimePicker.OnValueChanged), $t.CallPropertyChangedWhenLoadedIntoVisualTree = Windows.UI.Xaml.WhenToCallPropertyChangedEnum.IfPropertyIsSet, $t));
                }
            },
            methods: {
                /**
                 * SelectedDateProperty property changed handler.
                 *
                 * @static
                 * @private
                 * @this Windows.UI.Xaml.Controls.TimePicker
                 * @memberof Windows.UI.Xaml.Controls.TimePicker
                 * @param   {Windows.UI.Xaml.DependencyObject}                      d    TemporalPicker that changed its SelectedDate.
                 * @param   {Windows.UI.Xaml.DependencyPropertyChangedEventArgs}    e    The DependencyPropertyChangedEventArgs.
                 * @return  {void}
                 */
                OnValueChanged: function (d, e) {
                    Bridge.cast(d, Windows.UI.Xaml.Controls.TimePicker).INTERNAL_SelectedDate = Bridge.cast(Bridge.unbox(e.NewValue, System.DateTime), System.DateTime, true);
                }
            }
        },
        props: {
            /**
             * Gets or sets the currently selected date.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.TimePicker
             * @function Value
             * @type ?System.DateTime
             */
            Value: {
                get: function () {
                    return Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.Controls.TimePicker.ValueProperty), System.DateTime), System.DateTime, true);
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Controls.TimePicker.ValueProperty, Bridge.box(value, System.DateTime, System.Nullable.toStringFn(System.DateTime.format), System.Nullable.getHashCode));
                }
            }
        },
        ctors: {
            ctor: function () {
                this.$initialize();
                Windows.UI.Xaml.Controls.INTERNAL_DateTimePickerBase.ctor.call(this);
                this._defaultText = "";

                this.DefaultStyleKey = Windows.UI.Xaml.Controls.TimePicker;
            }
        },
        methods: {
            GenerateCalendarOrClock: function () {
                return new Windows.UI.Xaml.Controls.Clock();
            },
            SetTextFromDate: function (newDate) {
                var split = System.String.split(newDate, [32].map(function (i) {{ return String.fromCharCode(i); }}));
                var time = System.String.split(split[System.Array.index(1, split)], [58].map(function (i) {{ return String.fromCharCode(i); }}));
                return (time[System.Array.index(0, time)] || "") + " : " + (time[System.Array.index(1, time)] || "");
            },
            OnSelectionChanged: function (newSelectedDate) {
                if (System.Nullable.hasValue(newSelectedDate) && System.Nullable.hasValue(this.Value)) {
                    if (!Bridge.equals(this.Value, newSelectedDate)) {
                        this.Value = newSelectedDate;
                    }
                } else if (System.Nullable.hasValue(newSelectedDate) && !System.Nullable.hasValue(this.Value)) {
                    this.Value = newSelectedDate;
                } else {
                    if (!Bridge.equals(this.Value, null)) {
                        this.Value = null;
                    }
                }

            }
        }
    });

    /**
     * Represents a control that creates a pop-up window that displays information for an element in the UI.
     *
     * @public
     * @class Windows.UI.Xaml.Controls.ToolTip
     * @augments Windows.UI.Xaml.Controls.ContentControl
     */
    Bridge.define("Windows.UI.Xaml.Controls.ToolTip", {
        inherits: [Windows.UI.Xaml.Controls.ContentControl],
        statics: {
            fields: {
                /**
                 * Identifies the IsOpen dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Controls.ToolTip
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                IsOpenProperty: null,
                /**
                 * Identifies the HorizontalOffset dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Controls.ToolTip
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                HorizontalOffsetProperty: null,
                /**
                 * Identifies the VerticalOffset dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Controls.ToolTip
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                VerticalOffsetProperty: null
            },
            ctors: {
                init: function () {
                    var $t;
                    this.IsOpenProperty = Windows.UI.Xaml.DependencyProperty.Register("IsOpen", System.Boolean, Windows.UI.Xaml.Controls.ToolTip, ($t = new Windows.UI.Xaml.PropertyMetadata.$ctor2(Bridge.box(false, System.Boolean, System.Boolean.toString), Windows.UI.Xaml.Controls.ToolTip.IsOpen_Changed), $t.CallPropertyChangedWhenLoadedIntoVisualTree = Windows.UI.Xaml.WhenToCallPropertyChangedEnum.IfPropertyIsSet, $t));
                    this.HorizontalOffsetProperty = Windows.UI.Xaml.DependencyProperty.Register("HorizontalOffset", System.Double, Windows.UI.Xaml.Controls.ToolTip, new Windows.UI.Xaml.PropertyMetadata.$ctor1(Bridge.box(0.0, System.Double, System.Double.format, System.Double.getHashCode)));
                    this.VerticalOffsetProperty = Windows.UI.Xaml.DependencyProperty.Register("VerticalOffset", System.Double, Windows.UI.Xaml.Controls.ToolTip, new Windows.UI.Xaml.PropertyMetadata.$ctor1(Bridge.box(0.0, System.Double, System.Double.format, System.Double.getHashCode)));
                }
            },
            methods: {
                IsOpen_Changed: function (d, e) {
                    var $t;
                    var toolTip = Bridge.cast(d, Windows.UI.Xaml.Controls.ToolTip);
                    if (Bridge.is(e.NewValue, System.Boolean)) {
                        var isOpen = System.Nullable.getValue(Bridge.cast(Bridge.unbox(e.NewValue, System.Boolean), System.Boolean));
                        if (isOpen) {
                            var wasPopupAlreadyOpen = (toolTip._parentPopup != null && toolTip._parentPopup.IsOpen === true);
                            if (!wasPopupAlreadyOpen) {
                                if (Bridge.is(toolTip.INTERNAL_ElementToWhichThisToolTipIsAssigned, Windows.UI.Xaml.FrameworkElement)) {
                                    toolTip.DataContext = Bridge.cast(toolTip.INTERNAL_ElementToWhichThisToolTipIsAssigned, Windows.UI.Xaml.FrameworkElement).DataContext;
                                }

                                toolTip.IsHitTestVisible = false;

                                toolTip.HorizontalAlignment = Windows.UI.Xaml.HorizontalAlignment.Left;
                                toolTip.VerticalAlignment = Windows.UI.Xaml.VerticalAlignment.Top;

                                if (toolTip._parentPopup == null) {
                                    toolTip._parentPopup = ($t = new Windows.UI.Xaml.Controls.Primitives.Popup(), $t.Child = toolTip, $t.HorizontalAlignment = Windows.UI.Xaml.HorizontalAlignment.Left, $t.VerticalAlignment = Windows.UI.Xaml.VerticalAlignment.Top, $t.HorizontalContentAlignment = Windows.UI.Xaml.HorizontalAlignment.Left, $t.VerticalContentAlignment = Windows.UI.Xaml.VerticalAlignment.Top, $t);

                                    if (toolTip.INTERNAL_ElementToWhichThisToolTipIsAssigned != null) {
                                        toolTip._parentPopup.INTERNAL_ParentWindow = toolTip.INTERNAL_ElementToWhichThisToolTipIsAssigned.INTERNAL_ParentWindow;
                                    }
                                }

                                var popupAbsolutePosition = new Windows.Foundation.Point();
                                if (System.Nullable.hasValue(toolTip._forceSpecifyAbsoluteCoordinates)) {
                                    popupAbsolutePosition = System.Nullable.getValue(toolTip._forceSpecifyAbsoluteCoordinates).$clone();
                                } else {
                                    popupAbsolutePosition = DotNetForHtml5.Core.INTERNAL_PopupsManager.CalculatePopupAbsolutePositionBasedOnElementPosition(toolTip.INTERNAL_ElementToWhichThisToolTipIsAssigned, toolTip.HorizontalOffset, toolTip.VerticalOffset);
                                }

                                toolTip._parentPopup.HorizontalOffset = popupAbsolutePosition.X;
                                toolTip._parentPopup.VerticalOffset = popupAbsolutePosition.Y;

                                toolTip._parentPopup.removeLoaded(Bridge.fn.cacheBind(toolTip, toolTip._parentPopup_Loaded));
                                toolTip._parentPopup.addLoaded(Bridge.fn.cacheBind(toolTip, toolTip._parentPopup_Loaded));

                                toolTip._parentPopup.IsOpen = true;

                                toolTip._timerForClosingTooltipAfter5Seconds.removeTick(Bridge.fn.cacheBind(toolTip, toolTip._timerForClosingTooltipAfter5Seconds_Tick));
                                toolTip._timerForClosingTooltipAfter5Seconds.addTick(Bridge.fn.cacheBind(toolTip, toolTip._timerForClosingTooltipAfter5Seconds_Tick));
                                toolTip._timerForClosingTooltipAfter5Seconds.Start();

                                if (!Bridge.staticEquals(toolTip.Opened, null)) {
                                    toolTip.Opened(toolTip, new Windows.UI.Xaml.RoutedEventArgs());
                                }
                            }
                        } else {
                            toolTip._timerForClosingTooltipAfter5Seconds.Stop();

                            if (toolTip._parentPopup != null && toolTip._parentPopup.IsOpen === true) {
                                toolTip._parentPopup.IsOpen = false;

                                if (!Bridge.staticEquals(toolTip.Closed, null)) {
                                    toolTip.Closed(toolTip, new Windows.UI.Xaml.RoutedEventArgs());
                                }
                            }
                        }
                    }
                }
            }
        },
        fields: {
            _parentPopup: null,
            _forceSpecifyAbsoluteCoordinates: null,
            INTERNAL_ElementToWhichThisToolTipIsAssigned: null,
            INTERNAL_HtmlCanvasElementToWhichThisToolTipIsAssigned: null,
            _timerForClosingTooltipAfter5Seconds: null
        },
        events: {
            /**
             * Occurs when a ToolTip is closed and is no longer visible.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Controls.ToolTip
             * @memberof Windows.UI.Xaml.Controls.ToolTip
             * @function addClosed
             * @param   {Windows.UI.Xaml.RoutedEventHandler}    value
             * @return  {void}
             */
            /**
             * Occurs when a ToolTip is closed and is no longer visible.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Controls.ToolTip
             * @memberof Windows.UI.Xaml.Controls.ToolTip
             * @function removeClosed
             * @param   {Windows.UI.Xaml.RoutedEventHandler}    value
             * @return  {void}
             */
            Closed: null,
            /**
             * Occurs when a ToolTip becomes visible.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Controls.ToolTip
             * @memberof Windows.UI.Xaml.Controls.ToolTip
             * @function addOpened
             * @param   {Windows.UI.Xaml.RoutedEventHandler}    value
             * @return  {void}
             */
            /**
             * Occurs when a ToolTip becomes visible.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Controls.ToolTip
             * @memberof Windows.UI.Xaml.Controls.ToolTip
             * @function removeOpened
             * @param   {Windows.UI.Xaml.RoutedEventHandler}    value
             * @return  {void}
             */
            Opened: null
        },
        props: {
            /**
             * Gets or sets a value that indicates whether the ToolTip is visible. True if the ToolTip is visible; otherwise, false. The default is false.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.ToolTip
             * @function IsOpen
             * @type boolean
             */
            IsOpen: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.Controls.ToolTip.IsOpenProperty), System.Boolean), System.Boolean));
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Controls.ToolTip.IsOpenProperty, Bridge.box(value, System.Boolean, System.Boolean.toString));
                }
            },
            /**
             * Gets or sets the horizontal distance between the target origin and the pop-up alignment point. The default is 0.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.ToolTip
             * @function HorizontalOffset
             * @type number
             */
            HorizontalOffset: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.Controls.ToolTip.HorizontalOffsetProperty), System.Double), System.Double));
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Controls.ToolTip.HorizontalOffsetProperty, Bridge.box(value, System.Double, System.Double.format, System.Double.getHashCode));
                }
            },
            /**
             * Gets or sets the vertical distance between the target origin and the pop-up alignment point. The default is 0.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.ToolTip
             * @function VerticalOffset
             * @type number
             */
            VerticalOffset: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.Controls.ToolTip.VerticalOffsetProperty), System.Double), System.Double));
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Controls.ToolTip.VerticalOffsetProperty, Bridge.box(value, System.Double, System.Double.format, System.Double.getHashCode));
                }
            }
        },
        ctors: {
            init: function () {
                var $t;
                this._timerForClosingTooltipAfter5Seconds = ($t = new Windows.UI.Xaml.DispatcherTimer(), $t.Interval = new System.TimeSpan(0, 0, 5), $t);
            },
            /**
             * Initializes a new instance of the ToolTip class.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Controls.ToolTip
             * @memberof Windows.UI.Xaml.Controls.ToolTip
             * @return  {void}
             */
            ctor: function () {
                this.$initialize();
                Windows.UI.Xaml.Controls.ContentControl.ctor.call(this);
                this.DefaultStyleKey = Windows.UI.Xaml.Controls.ToolTip;
            }
        },
        methods: {
            _parentPopup_Loaded: function (sender, e) {
                DotNetForHtml5.Core.INTERNAL_PopupsManager.EnsurePopupStaysWithinScreenBounds(this._parentPopup);
            },
            _timerForClosingTooltipAfter5Seconds_Tick: function (sender, e) {
                this._timerForClosingTooltipAfter5Seconds.Stop();
                if (this.IsOpen) {
                    this.IsOpen = false;
                }
            },
            INTERNAL_OpenAtCoordinates: function (absoluteCoordinates) {
                this._forceSpecifyAbsoluteCoordinates = absoluteCoordinates.$clone();

                this.IsOpen = true;
            }
        }
    });

    /**
     * Defines a content decorator that can stretch and scale a single child to
     fill the available space.
     * // XAML markup used to define the write-once Viewbox template.
    //
     * // Measures the child element of a Viewbox to prepare for arranging
    // it during the ArrangeOverride pass.
    //
     * // Arranges the content of a Viewbox element.
    // Viewbox always sets the child to its desired size.  It then computes and applies a transformation
    // from that size to the space available: Viewbox's own input size less child margin.
    //
     *
     * @public
     * @class Windows.UI.Xaml.Controls.Viewbox
     * @augments Windows.UI.Xaml.Controls.ContentControl
     * @return  {[type]}        // The Size that represents the arranged size of this Viewbox element
    // and its child.
    //
     */
    Bridge.define("Windows.UI.Xaml.Controls.Viewbox", {
        inherits: [Windows.UI.Xaml.Controls.ContentControl],
        statics: {
            fields: {
                /**
                 * Identifies the Viewbox.Stretch dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Controls.Viewbox
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                StretchProperty: null,
                /**
                 * Identifies the Viewbox.StretchDirection dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Controls.Viewbox
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                StretchDirectionProperty: null
            },
            ctors: {
                init: function () {
                    this.StretchProperty = Windows.UI.Xaml.DependencyProperty.Register("Stretch", Windows.UI.Xaml.Media.Stretch, Windows.UI.Xaml.Controls.Viewbox, new Windows.UI.Xaml.PropertyMetadata.$ctor1(Bridge.box(Windows.UI.Xaml.Media.Stretch.Uniform, Windows.UI.Xaml.Media.Stretch, System.Enum.toStringFn(Windows.UI.Xaml.Media.Stretch))));
                    this.StretchDirectionProperty = Windows.UI.Xaml.DependencyProperty.Register("StretchDirection", Windows.UI.Xaml.Controls.StretchDirection, Windows.UI.Xaml.Controls.Viewbox, new Windows.UI.Xaml.PropertyMetadata.$ctor2(Bridge.box(Windows.UI.Xaml.Controls.StretchDirection.Both, Windows.UI.Xaml.Controls.StretchDirection, System.Enum.toStringFn(Windows.UI.Xaml.Controls.StretchDirection)), Windows.UI.Xaml.Controls.Viewbox.OnStretchDirectionPropertyChanged));
                }
            },
            methods: {
                /**
                 * StretchDirectionProperty property changed handler.
                 *
                 * @static
                 * @private
                 * @this Windows.UI.Xaml.Controls.Viewbox
                 * @memberof Windows.UI.Xaml.Controls.Viewbox
                 * @param   {Windows.UI.Xaml.DependencyObject}                      d    Viewbox that changed its StretchDirection.
                 * @param   {Windows.UI.Xaml.DependencyPropertyChangedEventArgs}    e    Event arguments.
                 * @return  {void}
                 */
                OnStretchDirectionPropertyChanged: function (d, e) {
                    var viewBox = Bridge.cast(d, Windows.UI.Xaml.Controls.Viewbox);
                    if (!Bridge.referenceEquals(e.NewValue, e.OldValue)) {
                        viewBox.InvalidateMeasure();
                    }
                }
            }
        },
        fields: {
            /**
             * Gets or sets the element of the Viewbox that will render the child.
             *
             * @instance
             * @private
             * @memberof Windows.UI.Xaml.Controls.Viewbox
             * @function ChildElement
             * @type Windows.UI.Xaml.Controls.ContentPresenter
             */
            ChildElement: null,
            /**
             * Gets or sets the Canvas element so that the child does not force its size on the parent.
             *
             * @instance
             * @private
             * @memberof Windows.UI.Xaml.Controls.Viewbox
             * @function RootCanvas
             * @type Windows.UI.Xaml.Controls.Canvas
             */
            RootCanvas: null,
            /**
             * Gets or sets the transformation on the ChildElement used to scale the
             Child content.
             *
             * @instance
             * @private
             * @memberof Windows.UI.Xaml.Controls.Viewbox
             * @function Scale
             * @type Windows.UI.Xaml.Media.ScaleTransform
             */
            Scale: null
        },
        props: {
            /**
             * Gets or sets the single child element of a Viewbox element.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.Viewbox
             * @function Child$1
             * @type Windows.UI.Xaml.UIElement
             */
            Child$1: {
                get: function () {
                    return Bridge.as(this.Content, Windows.UI.Xaml.UIElement);
                },
                set: function (value) {
                    this.Content = value;
                }
            },
            /**
             * Gets or sets the Stretch mode, which determines how content
             fits into the available space. The default is Stretch.Uniform.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.Viewbox
             * @function Stretch
             * @type Windows.UI.Xaml.Media.Stretch
             */
            Stretch: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.Controls.Viewbox.StretchProperty), Windows.UI.Xaml.Media.Stretch), System.Int32));
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Controls.Viewbox.StretchProperty, Bridge.box(value, Windows.UI.Xaml.Media.Stretch, System.Enum.toStringFn(Windows.UI.Xaml.Media.Stretch)));
                }
            },
            /**
             * Gets or sets the StretchDirection, which determines how scaling
             is applied to the contents of a Viewbox. The default is
             StretchDirection.Both.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.Viewbox
             * @function StretchDirection
             * @type Windows.UI.Xaml.Controls.StretchDirection
             */
            StretchDirection: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.Controls.Viewbox.StretchDirectionProperty), Windows.UI.Xaml.Controls.StretchDirection), System.Int32));
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Controls.Viewbox.StretchDirectionProperty, Bridge.box(value, Windows.UI.Xaml.Controls.StretchDirection, System.Enum.toStringFn(Windows.UI.Xaml.Controls.StretchDirection)));
                }
            }
        },
        ctors: {
            /**
             * Initializes a new instance of the
             {@link } class.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Controls.Viewbox
             * @memberof Windows.UI.Xaml.Controls.Viewbox
             * @return  {void}
             */
            ctor: function () {
                this.$initialize();
                Windows.UI.Xaml.Controls.ContentControl.ctor.call(this);
                this.DefaultStyleKey = Windows.UI.Xaml.Controls.Viewbox;

                this.addLoaded(Bridge.fn.cacheBind(this, this.OnLoaded));
                this.addSizeChanged(Bridge.fn.cacheBind(this, this.OnSizeChanged));

            }
        },
        methods: {
            /**
             * Builds the visual tree for the Viewbox control when a new
             template is applied.
             *
             * @instance
             * @protected
             * @override
             * @this Windows.UI.Xaml.Controls.Viewbox
             * @memberof Windows.UI.Xaml.Controls.Viewbox
             * @return  {void}
             */
            OnApplyTemplate: function () {
                var $t;
                this.ChildElement = Bridge.as(this.GetTemplateChild("Child"), Windows.UI.Xaml.Controls.ContentPresenter);
                System.Diagnostics.Debug.Assert$1(this.ChildElement != null, "The required template part Child was not found!");

                this.RootCanvas = Bridge.as(this.GetTemplateChild("RootCanvas"), Windows.UI.Xaml.Controls.Canvas);
                System.Diagnostics.Debug.Assert$1(this.RootCanvas != null, "The required template part RootCanvas was not found!");

                this.ChildElement.RenderTransform = ($t = new Windows.UI.Xaml.Media.ScaleTransform(), this.Scale = $t, $t);
            },
            OnLoaded: function (sender, e) {
                this.Dispatcher.BeginInvoke(Bridge.fn.bind(this, function () {
                    if (Bridge.is(this.Content, Windows.UI.Xaml.FrameworkElement)) {
                        this.UpdateScale(Bridge.as(this.Content, Windows.UI.Xaml.FrameworkElement));
                    }
                }));
            },
            OnContentChanged: function (oldContent, newContent) {
                if (this.IsLoaded && Bridge.is(newContent, Windows.UI.Xaml.FrameworkElement)) {
                    this.UpdateScale(Bridge.as(newContent, Windows.UI.Xaml.FrameworkElement));
                }
                Windows.UI.Xaml.Controls.ContentControl.prototype.OnContentChanged.call(this, oldContent, newContent);
            },
            OnSizeChanged: function (sender, e) {
                if (this.IsLoaded && Bridge.is(this.Content, Windows.UI.Xaml.FrameworkElement)) {
                    this.UpdateScale(Bridge.as(this.Content, Windows.UI.Xaml.FrameworkElement));
                }
            },
            InvalidateMeasure: function () {
                if (this.IsLoaded && Bridge.is(this.Content, Windows.UI.Xaml.FrameworkElement)) {
                    this.UpdateScale(Bridge.as(this.Content, Windows.UI.Xaml.FrameworkElement));
                }
            },
            UpdateScale: function (content) {
                if (this.Scale != null && this.RootCanvas != null) {
                    var contentSize = content.INTERNAL_GetActualWidthAndHeight();

                    var actualSize = this.INTERNAL_GetActualWidthAndHeight();
                    var availableSizeX = actualSize.Width;
                    var availableSizeY = actualSize.Height;

                    if (availableSizeX === 0) {
                        availableSizeX = Number.POSITIVE_INFINITY;
                    }
                    if (availableSizeY === 0) {
                        availableSizeY = Number.POSITIVE_INFINITY;
                    }

                    var isConstrainedWidth = availableSizeX !== Number.POSITIVE_INFINITY;
                    var isConstrainedHeight = availableSizeY !== Number.POSITIVE_INFINITY;

                    var availableSize = new Windows.Foundation.Size.$ctor1(availableSizeX, availableSizeY);
                    var scale = this.ComputeScaleFactor(availableSize.$clone(), contentSize.$clone());

                    this.Scale.ScaleX = scale.Width;
                    this.Scale.ScaleY = scale.Height;

                    if (isConstrainedWidth) {
                        this.RootCanvas.Width = Number.NaN;
                    } else {
                        this.RootCanvas.Width = contentSize.Width * scale.Width;
                    }

                    if (isConstrainedHeight) {
                        this.RootCanvas.Height = Number.NaN;
                    } else {
                        this.RootCanvas.Height = contentSize.Height * scale.Height;
                    }
                }
            },
            /**
             * Compute the scale factor of the Child content.
             *
             * @instance
             * @private
             * @this Windows.UI.Xaml.Controls.Viewbox
             * @memberof Windows.UI.Xaml.Controls.Viewbox
             * @param   {Windows.Foundation.Size}    availableSize    Available size to fill with content.
             * @param   {Windows.Foundation.Size}    contentSize      Desired size of the content.
             * @return  {Windows.Foundation.Size}                     Width and Height scale factors.
             */
            ComputeScaleFactor: function (availableSize, contentSize) {
                var scaleX = 1.0;
                var scaleY = 1.0;

                var isConstrainedWidth = availableSize.Width !== Number.POSITIVE_INFINITY;
                var isConstrainedHeight = availableSize.Height !== Number.POSITIVE_INFINITY;


                var stretch = this.Stretch;

                if ((stretch !== Windows.UI.Xaml.Media.Stretch.None) && (isConstrainedWidth || !isConstrainedHeight)) {
                    scaleX = contentSize.Width === 0.0 ? 0.0 : (availableSize.Width / contentSize.Width);
                    scaleY = contentSize.Height === 0.0 ? 0.0 : (availableSize.Height / contentSize.Height);

                    if (!isConstrainedWidth) {
                        scaleX = scaleY;
                    } else if (!isConstrainedHeight) {
                        scaleY = scaleX;
                    } else {
                        switch (stretch) {
                            case Windows.UI.Xaml.Media.Stretch.Uniform: 
                                scaleX = (scaleY = Math.min(scaleX, scaleY));
                                break;
                            case Windows.UI.Xaml.Media.Stretch.UniformToFill: 
                                scaleX = (scaleY = Math.max(scaleX, scaleY));
                                break;
                            case Windows.UI.Xaml.Media.Stretch.Fill: 
                            default: 
                                break;
                        }
                    }

                    switch (this.StretchDirection) {
                        case Windows.UI.Xaml.Controls.StretchDirection.UpOnly: 
                            scaleX = Math.max(1.0, scaleX);
                            scaleY = Math.max(1.0, scaleY);
                            break;
                        case Windows.UI.Xaml.Controls.StretchDirection.DownOnly: 
                            scaleX = Math.min(1.0, scaleX);
                            scaleY = Math.min(1.0, scaleY);
                            break;
                        case Windows.UI.Xaml.Controls.StretchDirection.Both: 
                        default: 
                            break;
                    }
                }

                return new Windows.Foundation.Size.$ctor1(scaleX, scaleY);
            }
        }
    });

    /**
     * Provides a base class for inline text elements, such as Span and Run.
     *
     * @abstract
     * @public
     * @class Windows.UI.Xaml.Documents.Inline
     * @augments Windows.UI.Xaml.Documents.TextElement
     */
    Bridge.define("Windows.UI.Xaml.Documents.Inline", {
        inherits: [Windows.UI.Xaml.Documents.TextElement],
        statics: {
            fields: {
                /**
                 * Identifies the TextDecorations dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Documents.Inline
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                TextDecorationsProperty: null
            },
            ctors: {
                init: function () {
                    var $t;
                    this.TextDecorationsProperty = Windows.UI.Xaml.DependencyProperty.Register("TextDecorations", System.Nullable$1(Windows.UI.Text.TextDecorations), Windows.UI.Xaml.Documents.Inline, ($t = new Windows.UI.Xaml.PropertyMetadata.$ctor4(null), $t.GetCSSEquivalent = Windows.UI.Xaml.Controls.Control.INTERNAL_GetCSSEquivalentForTextDecorations, $t));
                }
            },
            methods: {
                op_Implicit: function (s) {
                    var $t;
                    return ($t = new Windows.UI.Xaml.Documents.Run(), $t.Text = s, $t);
                }
            }
        },
        props: {
            /**
             * Gets or sets the text decorations (underline, strikethrough...).
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Documents.Inline
             * @function TextDecorations$1
             * @type ?Windows.UI.Text.TextDecorations
             */
            TextDecorations$1: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.Documents.Inline.TextDecorationsProperty), Windows.UI.Text.TextDecorations), System.Int32, true));
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Documents.Inline.TextDecorationsProperty, Bridge.box(value, Windows.UI.Text.TextDecorations, System.Nullable.toStringFn(System.Enum.toStringFn(Windows.UI.Text.TextDecorations)), System.Nullable.getHashCode));
                }
            }
        },
        methods: {
            OnAfterApplyHorizontalAlignmentAndWidth: function () {
                var style = CSHTML5.Internal.INTERNAL_HtmlDomManager.GetDomElementStyleForModification(this.INTERNAL_OuterDomElement);
                style.display = "inline";
            },
            OnAfterApplyVerticalAlignmentAndWidth: function () {
                var style = CSHTML5.Internal.INTERNAL_HtmlDomManager.GetDomElementStyleForModification(this.INTERNAL_OuterDomElement);
                style.display = "inline";
            }
        }
    });

    Bridge.define("Windows.UI.Xaml.Documents.TextElementCollection$1", function (T) { return {
        inherits: [System.Collections.IList,System.Collections.Generic.ICollection$1(T)],
        fields: {
            _owner: null,
            _textContainer: null,
            _collection: null
        },
        props: {
            Count: {
                get: function () {
                    return this._collection.Count;
                }
            },
            IsReadOnly: {
                get: function () {
                    return false;
                }
            },
            System$Collections$IList$IsFixedSize: {
                get: function () {
                    return false;
                }
            },
            System$Collections$ICollection$SyncRoot: {
                get: function () {
                    return this;
                }
            },
            System$Collections$ICollection$IsSynchronized: {
                get: function () {
                    return false;
                }
            },
            TextContainer: {
                get: function () {
                    return this._textContainer;
                }
            }
        },
        alias: [
            "Count", "System$Collections$Generic$ICollection$1$" + Bridge.getTypeAlias(T) + "$Count",
            "Count", "System$Collections$ICollection$Count",
            "IsReadOnly", "System$Collections$Generic$ICollection$1$" + Bridge.getTypeAlias(T) + "$IsReadOnly",
            "IsReadOnly", "System$Collections$IList$IsReadOnly",
            "getItem", "System$Collections$IList$getItem",
            "setItem", "System$Collections$IList$setItem",
            "add", "System$Collections$Generic$ICollection$1$" + Bridge.getTypeAlias(T) + "$add",
            "clear", "System$Collections$Generic$ICollection$1$" + Bridge.getTypeAlias(T) + "$clear",
            "clear", "System$Collections$IList$clear",
            "contains", "System$Collections$Generic$ICollection$1$" + Bridge.getTypeAlias(T) + "$contains",
            "copyTo", "System$Collections$Generic$ICollection$1$" + Bridge.getTypeAlias(T) + "$copyTo",
            "GetEnumerator", ["System$Collections$Generic$IEnumerable$1$" + Bridge.getTypeAlias(T) + "$GetEnumerator", "System$Collections$Generic$IEnumerable$1$GetEnumerator"],
            "remove", "System$Collections$Generic$ICollection$1$" + Bridge.getTypeAlias(T) + "$remove"
        ],
        ctors: {
            ctor: function (owner) {
                this.$initialize();
                this._owner = owner;
                this._collection = new (System.Collections.Generic.List$1(T)).ctor();
                this._textContainer = Windows.UI.Xaml.Documents.INTERNAL_TextContainerHelper.FromOwner(owner);
            }
        },
        methods: {
            getItem: function (index) {
                return this._collection.getItem(index);
            },
            setItem: function (index, value) {
                var item = Bridge.as(value, T);
                if (item == null) {
                    throw new System.ArgumentNullException.$ctor1("Value cannot be null.");
                }
                this._collection.setItem(index, item);
            },
            add: function (item) {
                this.AddInternal(this.Count, item);
            },
            System$Collections$IList$add: function (value) {
                var item = Bridge.as(value, T);
                var index = this.Count;
                this.AddInternal(index, item);
                return index;
            },
            clear: function () {
                this.TextContainer.Windows$UI$Xaml$Documents$INTERNAL_ITextContainer$BeginChange();
                try {
                    for (var i = (this._collection.Count - 1) | 0; i > -1; i = (i - 1) | 0) {
                        this.TextContainer.Windows$UI$Xaml$Documents$INTERNAL_ITextContainer$OnTextRemoved(this._collection.getItem(i));
                        this._collection.removeAt(i);
                    }
                } finally {
                    this.TextContainer.Windows$UI$Xaml$Documents$INTERNAL_ITextContainer$EndChange();
                }
            },
            contains: function (item) {
                return this._collection.contains(item);
            },
            System$Collections$IList$contains: function (value) {
                return this.contains(Bridge.as(value, T));
            },
            copyTo: function (array, arrayIndex) {
                this._collection.copyTo(array, arrayIndex);
            },
            System$Collections$ICollection$copyTo: function (array, arrayIndex) {
                System.Array.copyTo(Bridge.cast(this._collection, System.Collections.ICollection), array, arrayIndex);
            },
            GetEnumerator: function () {
                return this._collection.GetEnumerator().$clone();
            },
            System$Collections$IEnumerable$GetEnumerator: function () {
                return this.GetEnumerator();
            },
            System$Collections$IList$indexOf: function (value) {
                return this._collection.indexOf(Bridge.as(value, T));
            },
            System$Collections$IList$insert: function (index, value) {
                var item = Bridge.as(value, T);
                this.AddInternal(index, item);
            },
            remove: function (item) {
                return this.RemoveInternal(item);
            },
            System$Collections$IList$remove: function (value) {
                this.remove(Bridge.as(value, T));
            },
            System$Collections$IList$removeAt: function (index) {
                var item = this._collection.getItem(index);
                this._collection.removeAt(index);
                this.OnRemove(item);
            },
            OnAdd: function (item) {
                this.TextContainer.Windows$UI$Xaml$Documents$INTERNAL_ITextContainer$BeginChange();
                try {
                    this.TextContainer.Windows$UI$Xaml$Documents$INTERNAL_ITextContainer$OnTextAdded(item);
                } finally {
                    this.TextContainer.Windows$UI$Xaml$Documents$INTERNAL_ITextContainer$EndChange();
                }
            },
            OnRemove: function (item) {
                this.TextContainer.Windows$UI$Xaml$Documents$INTERNAL_ITextContainer$BeginChange();
                try {
                    this.TextContainer.Windows$UI$Xaml$Documents$INTERNAL_ITextContainer$OnTextRemoved(item);
                } finally {
                    this.TextContainer.Windows$UI$Xaml$Documents$INTERNAL_ITextContainer$EndChange();
                }
            },
            AddInternal: function (index, item) {
                if (item == null) {
                    throw new System.ArgumentNullException.$ctor1("item");
                }
                this._collection.insert(index, item);
                this.OnAdd(item);
            },
            RemoveInternal: function (item) {
                var success;
                if ((success = this._collection.remove(item))) {
                    this.OnRemove(item);
                }
                return success;
            }
        }
    }; });

    /**
     * @memberof Windows.UI.Xaml.Controls
     * @callback Windows.UI.Xaml.Controls.AutoCompleteFilterPredicate
     * @param   {string}           search    
     * @param   {System.Object}    item
     * @return  {boolean}
     */

    Bridge.define("Windows.UI.Xaml.Controls.AutoCompleteBox", {
        inherits: [Windows.UI.Xaml.Controls.Primitives.Selector],
        statics: {
            fields: {
                /**
                 * Identifies the IsDropDownOpen dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Controls.AutoCompleteBox
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                IsDropDownOpenProperty: null,
                IsArrowVisibleProperty: null,
                MinimumPopulateDelayProperty: null,
                MinimumPrefixLengthProperty: null,
                TextProperty: null,
                /**
                 * Identifies the MaxDropDownHeight dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Controls.AutoCompleteBox
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                MaxDropDownHeightProperty: null
            },
            ctors: {
                init: function () {
                    this.IsDropDownOpenProperty = Windows.UI.Xaml.DependencyProperty.Register("IsDropDownOpen", System.Boolean, Windows.UI.Xaml.Controls.AutoCompleteBox, new Windows.UI.Xaml.PropertyMetadata.$ctor2(Bridge.box(false, System.Boolean, System.Boolean.toString), Windows.UI.Xaml.Controls.AutoCompleteBox.OnIsDropDownOpenChanged));
                    this.IsArrowVisibleProperty = Windows.UI.Xaml.DependencyProperty.Register("IsArrowVisible", System.Boolean, Windows.UI.Xaml.Controls.AutoCompleteBox, new Windows.UI.Xaml.PropertyMetadata.$ctor2(Bridge.box(false, System.Boolean, System.Boolean.toString), Windows.UI.Xaml.Controls.AutoCompleteBox.OnIsArrowVisibleChanged));
                    this.MinimumPopulateDelayProperty = Windows.UI.Xaml.DependencyProperty.Register("MinimumPopulateDelay", System.Int32, Windows.UI.Xaml.Controls.AutoCompleteBox, new Windows.UI.Xaml.PropertyMetadata.$ctor2(Bridge.box(0, System.Int32), Windows.UI.Xaml.Controls.AutoCompleteBox.OnMinimumPopulateDelayChanged));
                    this.MinimumPrefixLengthProperty = Windows.UI.Xaml.DependencyProperty.Register("MinimumPrefixLength", System.Int32, Windows.UI.Xaml.Controls.AutoCompleteBox, new Windows.UI.Xaml.PropertyMetadata.$ctor1(Bridge.box(1, System.Int32)));
                    this.TextProperty = Windows.UI.Xaml.DependencyProperty.Register("Text", System.String, Windows.UI.Xaml.Controls.AutoCompleteBox, new Windows.UI.Xaml.PropertyMetadata.$ctor2("", Windows.UI.Xaml.Controls.AutoCompleteBox.OnTextChanged));
                    this.MaxDropDownHeightProperty = Windows.UI.Xaml.DependencyProperty.Register("MaxDropDownHeight", System.Double, Windows.UI.Xaml.Controls.AutoCompleteBox, new Windows.UI.Xaml.PropertyMetadata.$ctor1(Bridge.box(200.0, System.Double, System.Double.format, System.Double.getHashCode)));
                }
            },
            methods: {
                OnIsDropDownOpenChanged: function (d, e) {
                    var $t;
                    var autoCompleteBox = Bridge.cast(d, Windows.UI.Xaml.Controls.AutoCompleteBox);
                    if (CSHTML5.Internal.INTERNAL_VisualTreeManager.IsElementInVisualTree(autoCompleteBox)) {
                        var isDropDownOpen = System.Nullable.getValue(Bridge.cast(Bridge.unbox(e.NewValue, System.Boolean), System.Boolean));

                        if (isDropDownOpen) {

                            if (autoCompleteBox._popup != null) {
                                autoCompleteBox._popup.IsOpen = true;

                                var actualWidth = autoCompleteBox._popup.ActualWidth;
                                if (!isNaN(actualWidth) && Bridge.is(autoCompleteBox._popup.Child, Windows.UI.Xaml.FrameworkElement)) {
                                    Bridge.cast(autoCompleteBox._popup.Child, Windows.UI.Xaml.FrameworkElement).Width = actualWidth;
                                }
                            }

                            if (autoCompleteBox._dropDownToggle != null) {
                                if (autoCompleteBox._popup.IsOpen) {
                                    if (System.Nullable.eq(autoCompleteBox._dropDownToggle.IsChecked, false)) {
                                        autoCompleteBox._dropDownToggle.IsChecked = true;
                                    }
                                } else {
                                    if (System.Nullable.eq(autoCompleteBox._dropDownToggle.IsChecked, true)) {
                                        autoCompleteBox._dropDownToggle.IsChecked = false;
                                    }
                                }
                            }

                            if (autoCompleteBox._popup.IsOpen) {
                                autoCompleteBox.OnDropDownOpened(new Windows.UI.Xaml.RoutedEventArgs());
                            }
                        } else {

                            if (autoCompleteBox._popup != null) {
                                autoCompleteBox._popup.IsOpen = false;
                            }

                            if (autoCompleteBox._dropDownToggle != null && System.Nullable.eq(autoCompleteBox._dropDownToggle.IsChecked, true)) {
                                autoCompleteBox._dropDownToggle.IsChecked = false;
                            }

                            if (autoCompleteBox._textBox != null) {
                                autoCompleteBox._updatingTextOnSelection = true;

                                try {
                                    autoCompleteBox._textBox.Text = Bridge.toString((($t = autoCompleteBox.SelectedItem, $t != null ? $t : "")));
                                } finally {
                                    autoCompleteBox._updatingTextOnSelection = false;
                                }
                            }

                            autoCompleteBox.OnDropDownClosed(new Windows.UI.Xaml.RoutedEventArgs());
                        }
                    }
                },
                OnIsArrowVisibleChanged: function (d, e) {
                    Bridge.cast(d, Windows.UI.Xaml.Controls.AutoCompleteBox).UpdateToggleButton(System.Nullable.getValue(Bridge.cast(Bridge.unbox(e.NewValue, System.Boolean), System.Boolean)));
                },
                OnMinimumPopulateDelayChanged: function (d, e) {
                    var autoCompleteBox = Bridge.cast(d, Windows.UI.Xaml.Controls.AutoCompleteBox);
                    autoCompleteBox._timer.Interval = new System.TimeSpan(0, 0, 0, 0, System.Nullable.getValue(Bridge.cast(Bridge.unbox(e.NewValue, System.Int32), System.Int32)));
                },
                OnTextChanged: function (d, e) {
                    var autoCompleteBox = Bridge.cast(d, Windows.UI.Xaml.Controls.AutoCompleteBox);

                    autoCompleteBox.OnTextChanged(new Windows.UI.Xaml.RoutedEventArgs());
                },
                None: function (search, item) {
                    return true;
                },
                StartsWith: function (search, item) {
                    if (item != null && search != null) {
                        return System.String.startsWith(Bridge.toString(item), search, 1);
                    } else {
                        return false;
                    }
                },
                StartsWithCaseSensitive: function (search, item) {
                    if (item != null && search != null) {
                        return System.String.startsWith(Bridge.toString(item), search, 0);
                    } else {
                        return false;
                    }
                },
                StartsWithOrdinal: function (search, item) {
                    if (item != null && search != null) {
                        return System.String.startsWith(Bridge.toString(item), search, 5);
                    } else {
                        return false;
                    }
                },
                StartsWithOrdinalCaseSensitive: function (search, item) {
                    if (item != null && search != null) {
                        return System.String.startsWith(Bridge.toString(item), search, 4);
                    } else {
                        return false;
                    }
                },
                Contains: function (search, item) {
                    if (item != null && search != null) {
                        return System.String.contains(Bridge.toString(item).toLowerCase(),search.toLowerCase());
                    } else {
                        return false;
                    }
                },
                ContainsCaseSensitive: function (search, item) {
                    if (item != null && search != null) {
                        return System.String.contains(Bridge.toString(item),search);
                    } else {
                        return false;
                    }
                },
                ContainsOrdinal: function (search, item) {
                    return Windows.UI.Xaml.Controls.AutoCompleteBox.Contains(search, item);
                },
                ContainsOrdinalCaseSensitive: function (search, item) {
                    return Windows.UI.Xaml.Controls.AutoCompleteBox.ContainsCaseSensitive(search, item);
                },
                Equals: function (search, item) {
                    if (item != null && search != null) {
                        return System.String.equals(Bridge.toString(item), search, 1);
                    } else {
                        return false;
                    }
                },
                EqualsCaseSensitive: function (search, item) {
                    if (item != null && search != null) {
                        return System.String.equals(Bridge.toString(item), search, 0);
                    } else {
                        return false;
                    }
                },
                EqualsOrdinal: function (search, item) {
                    if (item != null && search != null) {
                        return System.String.equals(Bridge.toString(item), search, 5);
                    } else {
                        return false;
                    }
                },
                EqualsOrdinalCaseSensitive: function (search, item) {
                    if (item != null && search != null) {
                        return System.String.equals(Bridge.toString(item), search, 4);
                    } else {
                        return false;
                    }
                }
            }
        },
        fields: {
            _popup: null,
            _dropDownToggle: null,
            _textBox: null,
            _timer: null,
            _filterMode: 0,
            _filter: null,
            _itemFilter: null,
            _updatingTextOnSelection: false
        },
        events: {
            /**
             * Occurs when the text in the text box portion of the AutoCompleteBox changes.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Controls.AutoCompleteBox
             * @memberof Windows.UI.Xaml.Controls.AutoCompleteBox
             * @function addTextChanged
             * @param   {Windows.UI.Xaml.RoutedEventHandler}    value
             * @return  {void}
             */
            /**
             * Occurs when the text in the text box portion of the AutoCompleteBox changes.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Controls.AutoCompleteBox
             * @memberof Windows.UI.Xaml.Controls.AutoCompleteBox
             * @function removeTextChanged
             * @param   {Windows.UI.Xaml.RoutedEventHandler}    value
             * @return  {void}
             */
            TextChanged: null,
            /**
             * Occurs when the drop-down portion of the ComboBox closes.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Controls.AutoCompleteBox
             * @memberof Windows.UI.Xaml.Controls.AutoCompleteBox
             * @function addDropDownClosed
             * @param   {Windows.UI.Xaml.RoutedEventHandler}    value
             * @return  {void}
             */
            /**
             * Occurs when the drop-down portion of the ComboBox closes.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Controls.AutoCompleteBox
             * @memberof Windows.UI.Xaml.Controls.AutoCompleteBox
             * @function removeDropDownClosed
             * @param   {Windows.UI.Xaml.RoutedEventHandler}    value
             * @return  {void}
             */
            DropDownClosed: null,
            /**
             * Occurs when the drop-down portion of the ComboBox opens.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Controls.AutoCompleteBox
             * @memberof Windows.UI.Xaml.Controls.AutoCompleteBox
             * @function addDropDownOpened
             * @param   {Windows.UI.Xaml.RoutedEventHandler}    value
             * @return  {void}
             */
            /**
             * Occurs when the drop-down portion of the ComboBox opens.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Controls.AutoCompleteBox
             * @memberof Windows.UI.Xaml.Controls.AutoCompleteBox
             * @function removeDropDownOpened
             * @param   {Windows.UI.Xaml.RoutedEventHandler}    value
             * @return  {void}
             */
            DropDownOpened: null
        },
        props: {
            /**
             * Gets or sets a value that indicates whether the drop-down portion of the
             ComboBox is currently open.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.AutoCompleteBox
             * @function IsDropDownOpen
             * @type boolean
             */
            IsDropDownOpen: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.Controls.AutoCompleteBox.IsDropDownOpenProperty), System.Boolean), System.Boolean));
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Controls.AutoCompleteBox.IsDropDownOpenProperty, Bridge.box(value, System.Boolean, System.Boolean.toString));
                }
            },
            /**
             * The IsArrowVisible property is true when it's an AutoCompleteComboBox and false if it is an AutoCompleteBox
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.AutoCompleteBox
             * @function IsArrowVisible
             * @type boolean
             */
            IsArrowVisible: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.Controls.AutoCompleteBox.IsArrowVisibleProperty), System.Boolean), System.Boolean));
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Controls.AutoCompleteBox.IsArrowVisibleProperty, Bridge.box(value, System.Boolean, System.Boolean.toString));
                }
            },
            /**
             * Gets or sets the minimum delay, in milliseconds, after text is typed in the text box before the AutoCompleteBox control populates the list of possible matches in the drop-down.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.AutoCompleteBox
             * @function MinimumPopulateDelay
             * @type number
             */
            MinimumPopulateDelay: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.Controls.AutoCompleteBox.MinimumPopulateDelayProperty), System.Int32), System.Int32));
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Controls.AutoCompleteBox.MinimumPopulateDelayProperty, Bridge.box(value, System.Int32));
                }
            },
            /**
             * Gets or sets the minimum number of characters required to be entered in the text box before the AutoCompleteBox displays possible matches.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.AutoCompleteBox
             * @function MinimumPrefixLength
             * @type number
             */
            MinimumPrefixLength: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.Controls.AutoCompleteBox.MinimumPrefixLengthProperty), System.Int32), System.Int32));
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Controls.AutoCompleteBox.MinimumPrefixLengthProperty, Bridge.box(value, System.Int32));
                }
            },
            /**
             * Gets the text that is used to filter items in the ItemsSource item collection.
             *
             * @instance
             * @public
             * @readonly
             * @memberof Windows.UI.Xaml.Controls.AutoCompleteBox
             * @function SearchText
             * @type string
             */
            SearchText: {
                get: function () {
                    return this._textBox.Text;
                }
            },
            /**
             * Gets or sets the text in the text box portion of the AutoCompleteBox control.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.AutoCompleteBox
             * @function Text
             * @type string
             */
            Text: {
                get: function () {
                    return Bridge.cast(this.GetValue(Windows.UI.Xaml.Controls.AutoCompleteBox.TextProperty), System.String);
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Controls.AutoCompleteBox.TextProperty, value);
                }
            },
            /**
             * Gets or sets how the text in the text box is used to filter items specified
             by the AutoCompleteBox.ItemsSource property for display
             in the drop-down.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.AutoCompleteBox
             * @function FilterMode
             * @type Windows.UI.Xaml.Controls.AutoCompleteFilterMode
             */
            FilterMode: {
                get: function () {
                    return this._filterMode;
                },
                set: function (value) {
                    this.SelectFilter(value);
                }
            },
            /**
             * The custom method that uses the user-entered text to filter the items specified by the ItemsSource property.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.AutoCompleteBox
             * @function ItemFilter
             * @type Windows.UI.Xaml.Controls.AutoCompleteFilterPredicate
             */
            ItemFilter: {
                get: function () {
                    return this._itemFilter;
                },
                set: function (value) {
                    if (!Bridge.staticEquals(this._itemFilter, value)) {
                        this._itemFilter = value;
                        if (!Bridge.staticEquals(value, null)) {
                            this.FilterMode = Windows.UI.Xaml.Controls.AutoCompleteFilterMode.Custom;
                        } else {
                            this.FilterMode = Windows.UI.Xaml.Controls.AutoCompleteFilterMode.None;
                        }
                    }
                }
            },
            /**
             * Gets or sets the maximum height for a combo box drop-down.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.AutoCompleteBox
             * @function MaxDropDownHeight
             * @type number
             */
            MaxDropDownHeight: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.Controls.AutoCompleteBox.MaxDropDownHeightProperty), System.Double), System.Double));
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Controls.AutoCompleteBox.MaxDropDownHeightProperty, Bridge.box(value, System.Double, System.Double.format, System.Double.getHashCode));
                }
            }
        },
        ctors: {
            /**
             * Initializes a new instance of the AutoCompleteBox class.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Controls.AutoCompleteBox
             * @memberof Windows.UI.Xaml.Controls.AutoCompleteBox
             * @return  {void}
             */
            ctor: function () {
                this.$initialize();
                Windows.UI.Xaml.Controls.Primitives.Selector.ctor.call(this);
                this.DefaultStyleKey = Windows.UI.Xaml.Controls.AutoCompleteBox;
                this.FilterMode = Windows.UI.Xaml.Controls.AutoCompleteFilterMode.StartsWith;
                this._timer = new Windows.UI.Xaml.DispatcherTimer();
                this._timer.addTick(Bridge.fn.bind(this, function (o, e) {
                    this._timer.Stop();
                    this.UpdateItemsVisibilityOnTextChanged();
                }));
            }
        },
        methods: {
            OnTextBoxTextChanged: function (sender, e) {
                if (this._updatingTextOnSelection) {
                    return;
                }

                if (this._textBox == null || this._textBox.Text.length < this.MinimumPrefixLength) {
                    this._timer.Stop();
                    this.SetCurrentValue(Windows.UI.Xaml.Controls.Primitives.Selector.SelectedIndexProperty, Bridge.box(-1, System.Int32));
                    this.IsDropDownOpen = false;

                    return;
                }

                if (this._timer.IsEnabled) {
                    this._timer.Stop();
                }
                this._timer.Start();
            },
            UpdateItemsVisibilityOnTextChanged: function () {
                if (this.ItemsHost != null) {
                    for (var i = 0; i < this.ItemsHost.Children.Count; i = (i + 1) | 0) {
                        var child = Bridge.as(this.ItemsHost.Children.getItem(i), Windows.UI.Xaml.Controls.ComboBoxItem);
                        if (this._filter(this._textBox.Text, child.Content)) {
                            child.Visibility = Windows.UI.Xaml.Visibility.Visible;
                        } else {
                            child.Visibility = Windows.UI.Xaml.Visibility.Collapsed;
                            if (this.SelectedIndex === i) {
                                this.SetCurrentValue(Windows.UI.Xaml.Controls.Primitives.Selector.SelectedIndexProperty, Bridge.box(-1, System.Int32));
                            }
                        }
                    }

                    this.IsDropDownOpen = true;
                }
            },
            OnApplyTemplate: function () {
                Windows.UI.Xaml.Controls.Primitives.Selector.prototype.OnApplyTemplate.call(this);

                this._popup = Bridge.as(this.GetTemplateChild("Popup"), Windows.UI.Xaml.Controls.Primitives.Popup);
                this._dropDownToggle = Bridge.as(this.GetTemplateChild("DropDownToggle"), Windows.UI.Xaml.Controls.Primitives.ToggleButton);
                this._textBox = Bridge.as(this.GetTemplateChild("PART_TextBox"), Windows.UI.Xaml.Controls.TextBox);

                if (this._dropDownToggle != null) {
                    this._dropDownToggle.addChecked(Bridge.fn.cacheBind(this, this.DropDownToggle_Checked));
                    this._dropDownToggle.addUnchecked(Bridge.fn.cacheBind(this, this.DropDownToggle_Unchecked));
                }

                if (this._textBox != null) {
                    this._textBox.addTextChanged(Bridge.fn.cacheBind(this, this.OnTextBoxTextChanged));
                }

                this._textBox.addPointerPressed(Bridge.fn.cacheBind(this, this.TextBox_PointerPressed));

                this.UpdateToggleButton(this.IsArrowVisible);

                this.ApplySelectedIndex(this.SelectedIndex);
            },
            DropDownToggle_Checked: function (sender, e) {
                this.IsDropDownOpen = true;
            },
            DropDownToggle_Unchecked: function (sender, e) {
                this.IsDropDownOpen = false;
            },
            UpdateToggleButton: function (isVisible) {
                if (this._textBox != null && this._dropDownToggle != null) {
                    if (isVisible) {
                        this._textBox.Margin = new Windows.UI.Xaml.Thickness.$ctor2(0, 0, 30, 0);
                        this._dropDownToggle.Visibility = Windows.UI.Xaml.Visibility.Visible;
                    } else {
                        this._textBox.Margin = new Windows.UI.Xaml.Thickness.$ctor2(0, 0, 0, 0);
                        this._dropDownToggle.Visibility = Windows.UI.Xaml.Visibility.Collapsed;
                    }
                }
            },
            OnTextChanged: function (e) {
                if (!Bridge.staticEquals(this.TextChanged, null)) {
                    this.TextChanged(this, e);
                }
            },
            SelectFilter: function (mode) {
                switch (mode) {
                    case Windows.UI.Xaml.Controls.AutoCompleteFilterMode.None: 
                        this._filterMode = Windows.UI.Xaml.Controls.AutoCompleteFilterMode.None;
                        this._filter = Windows.UI.Xaml.Controls.AutoCompleteBox.None;
                        break;
                    case Windows.UI.Xaml.Controls.AutoCompleteFilterMode.StartsWith: 
                        this._filterMode = Windows.UI.Xaml.Controls.AutoCompleteFilterMode.StartsWith;
                        this._filter = Windows.UI.Xaml.Controls.AutoCompleteBox.StartsWith;
                        break;
                    case Windows.UI.Xaml.Controls.AutoCompleteFilterMode.StartsWithCaseSensitive: 
                        this._filterMode = Windows.UI.Xaml.Controls.AutoCompleteFilterMode.StartsWithCaseSensitive;
                        this._filter = Windows.UI.Xaml.Controls.AutoCompleteBox.StartsWithCaseSensitive;
                        break;
                    case Windows.UI.Xaml.Controls.AutoCompleteFilterMode.StartsWithOrdinal: 
                        this._filterMode = Windows.UI.Xaml.Controls.AutoCompleteFilterMode.StartsWithOrdinal;
                        this._filter = Windows.UI.Xaml.Controls.AutoCompleteBox.StartsWithOrdinal;
                        break;
                    case Windows.UI.Xaml.Controls.AutoCompleteFilterMode.StartsWithOrdinalCaseSensitive: 
                        this._filterMode = Windows.UI.Xaml.Controls.AutoCompleteFilterMode.StartsWithOrdinalCaseSensitive;
                        this._filter = Windows.UI.Xaml.Controls.AutoCompleteBox.StartsWithOrdinalCaseSensitive;
                        break;
                    case Windows.UI.Xaml.Controls.AutoCompleteFilterMode.Contains: 
                        this._filterMode = Windows.UI.Xaml.Controls.AutoCompleteFilterMode.Contains;
                        this._filter = Windows.UI.Xaml.Controls.AutoCompleteBox.Contains;
                        break;
                    case Windows.UI.Xaml.Controls.AutoCompleteFilterMode.ContainsCaseSensitive: 
                        this._filterMode = Windows.UI.Xaml.Controls.AutoCompleteFilterMode.ContainsCaseSensitive;
                        this._filter = Windows.UI.Xaml.Controls.AutoCompleteBox.ContainsCaseSensitive;
                        break;
                    case Windows.UI.Xaml.Controls.AutoCompleteFilterMode.ContainsOrdinal: 
                        this._filterMode = Windows.UI.Xaml.Controls.AutoCompleteFilterMode.ContainsOrdinal;
                        this._filter = Windows.UI.Xaml.Controls.AutoCompleteBox.ContainsOrdinal;
                        break;
                    case Windows.UI.Xaml.Controls.AutoCompleteFilterMode.ContainsOrdinalCaseSensitive: 
                        this._filterMode = Windows.UI.Xaml.Controls.AutoCompleteFilterMode.ContainsOrdinalCaseSensitive;
                        this._filter = Windows.UI.Xaml.Controls.AutoCompleteBox.ContainsOrdinalCaseSensitive;
                        break;
                    case Windows.UI.Xaml.Controls.AutoCompleteFilterMode.Equals: 
                        this._filterMode = Windows.UI.Xaml.Controls.AutoCompleteFilterMode.Equals;
                        this._filter = Windows.UI.Xaml.Controls.AutoCompleteBox.Equals;
                        break;
                    case Windows.UI.Xaml.Controls.AutoCompleteFilterMode.EqualsCaseSensitive: 
                        this._filterMode = Windows.UI.Xaml.Controls.AutoCompleteFilterMode.EqualsCaseSensitive;
                        this._filter = Windows.UI.Xaml.Controls.AutoCompleteBox.EqualsCaseSensitive;
                        break;
                    case Windows.UI.Xaml.Controls.AutoCompleteFilterMode.EqualsOrdinal: 
                        this._filterMode = Windows.UI.Xaml.Controls.AutoCompleteFilterMode.EqualsOrdinal;
                        this._filter = Windows.UI.Xaml.Controls.AutoCompleteBox.EqualsOrdinal;
                        break;
                    case Windows.UI.Xaml.Controls.AutoCompleteFilterMode.EqualsOrdinalCaseSensitive: 
                        this._filterMode = Windows.UI.Xaml.Controls.AutoCompleteFilterMode.EqualsOrdinalCaseSensitive;
                        this._filter = Windows.UI.Xaml.Controls.AutoCompleteBox.EqualsOrdinalCaseSensitive;
                        break;
                    case Windows.UI.Xaml.Controls.AutoCompleteFilterMode.Custom: 
                        this._filterMode = Windows.UI.Xaml.Controls.AutoCompleteFilterMode.Custom;
                        this._filter = Bridge.fn.cacheBind(this, this.Custom);
                        break;
                    default: 
                        throw new System.ArgumentException.$ctor1("Filters can't be combined.");
                }
            },
            Custom: function (search, item) {
                if (item != null && search != null) {
                    return this._itemFilter(search, item);
                } else {
                    return false;
                }
            },
            PrepareContainerForItemOverride: function (element, item) {
                Windows.UI.Xaml.Controls.Primitives.Selector.prototype.PrepareContainerForItemOverride.call(this, element, item);

                var container = Bridge.as(element, Windows.UI.Xaml.Controls.ComboBoxItem);
                if (container != null) {
                    container.INTERNAL_CorrespondingItem = item;
                    container.INTERNAL_ParentSelectorControl = this;
                    container.addClick(Bridge.fn.cacheBind(this, this.ComboBoxItem_Click));
                }
            },
            GetContainerForItemOverride: function () {
                return new Windows.UI.Xaml.Controls.ComboBoxItem();
            },
            IsItemItsOwnContainerOverride: function (item) {
                return (Bridge.is(item, Windows.UI.Xaml.Controls.ComboBoxItem));
            },
            INTERNAL_GenerateContainer: function (item) {
                return Bridge.cast(this.GetContainerFromItem(item), Windows.UI.Xaml.Controls.Primitives.SelectorItem);
            },
            GetContainerFromItem: function (item) {
                var comboBoxItem = Bridge.as(item, Windows.UI.Xaml.Controls.ComboBoxItem) || new Windows.UI.Xaml.Controls.ComboBoxItem();
                comboBoxItem.INTERNAL_CorrespondingItem = item;
                comboBoxItem.INTERNAL_ParentSelectorControl = this;
                comboBoxItem.addClick(Bridge.fn.cacheBind(this, this.ComboBoxItem_Click));
                return comboBoxItem;
            },
            ComboBoxItem_Click: function (sender, e) {
                var selectedContainer = Bridge.cast(sender, Windows.UI.Xaml.Controls.Primitives.SelectorItem);

                this.SelectedItem = selectedContainer.INTERNAL_CorrespondingItem;

                if (this._dropDownToggle != null) {
                    this._dropDownToggle.IsChecked = false;
                }
            },
            ApplySelectedIndex: function (index) {
                Windows.UI.Xaml.Controls.Primitives.Selector.prototype.ApplySelectedIndex.call(this, index);

                if (this.ItemsHost == null) {
                    return;
                }

                var newSelectedContent;

                if (index === -1) {
                    newSelectedContent = null;
                } else if (index < this.ItemsHost.Children.Count) {
                    var container = Bridge.as(this.ItemsHost.Children.getItem(index), Windows.UI.Xaml.Controls.ComboBoxItem);
                    newSelectedContent = container;
                } else {
                    throw new System.IndexOutOfRangeException.ctor();
                }
            },
            TextBox_PointerPressed: function (sender, e) {


                if (this.IsDropDownOpen) {
                    this.IsDropDownOpen = false;
                }
            },
            /**
             * Invoked when the DropDownClosed event is raised.
             *
             * @instance
             * @protected
             * @this Windows.UI.Xaml.Controls.AutoCompleteBox
             * @memberof Windows.UI.Xaml.Controls.AutoCompleteBox
             * @param   {Windows.UI.Xaml.RoutedEventArgs}    e    Event data for the event.
             * @return  {void}
             */
            OnDropDownClosed: function (e) {
                if (!Bridge.staticEquals(this.DropDownClosed, null)) {
                    this.DropDownClosed(this, e);
                }
            },
            /**
             * Invoked when the DropDownOpened event is raised.
             *
             * @instance
             * @protected
             * @this Windows.UI.Xaml.Controls.AutoCompleteBox
             * @memberof Windows.UI.Xaml.Controls.AutoCompleteBox
             * @param   {Windows.UI.Xaml.RoutedEventArgs}    e    Event data for the event.
             * @return  {void}
             */
            OnDropDownOpened: function (e) {
                if (!Bridge.staticEquals(this.DropDownOpened, null)) {
                    this.DropDownOpened(this, e);
                }
            }
        }
    });

    /**
     * Represents a templated button control that interprets a Click user interaction.
     *
     * @public
     * @class Windows.UI.Xaml.Controls.Button
     * @augments Windows.UI.Xaml.Controls.Primitives.ButtonBase
     * @example
     *
     *
     * void MyButton_Click(object sender, RoutedEventArgs e)
     * {
     *     MessageBox.Show("You clicked me.");
     *     Window.Current.IsEnabled = false;
     * }
     * 
     *
     *
     */
    Bridge.define("Windows.UI.Xaml.Controls.Button", {
        inherits: [Windows.UI.Xaml.Controls.Primitives.ButtonBase],
        ctors: {
            ctor: function () {
                this.$initialize();
                Windows.UI.Xaml.Controls.Primitives.ButtonBase.ctor.call(this);
                this.DefaultStyleKey = Windows.UI.Xaml.Controls.Button;
            }
        }
    });

    /**
     * Base class for controls that can switch states, such as CheckBox and RadioButton.
     *
     * @public
     * @class Windows.UI.Xaml.Controls.Primitives.ToggleButton
     * @augments Windows.UI.Xaml.Controls.Primitives.ButtonBase
     */
    Bridge.define("Windows.UI.Xaml.Controls.Primitives.ToggleButton", {
        inherits: [Windows.UI.Xaml.Controls.Primitives.ButtonBase],
        statics: {
            fields: {
                /**
                 * Identifies the IsChecked dependency property
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Controls.Primitives.ToggleButton
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                IsCheckedProperty: null,
                /**
                 * Identifies the ToggleButton.IsThreeState dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Controls.Primitives.ToggleButton
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                IsThreeStateProperty: null
            },
            ctors: {
                init: function () {
                    var $t;
                    this.IsCheckedProperty = Windows.UI.Xaml.DependencyProperty.Register("IsChecked", System.Nullable$1(System.Boolean), Windows.UI.Xaml.Controls.Primitives.ToggleButton, ($t = new Windows.UI.Xaml.PropertyMetadata.$ctor2(Bridge.box(false, System.Boolean, System.Boolean.toString), Windows.UI.Xaml.Controls.Primitives.ToggleButton.IsChecked_Changed), $t.CallPropertyChangedWhenLoadedIntoVisualTree = Windows.UI.Xaml.WhenToCallPropertyChangedEnum.IfPropertyIsSet, $t));
                    this.IsThreeStateProperty = Windows.UI.Xaml.DependencyProperty.Register("IsThreeState", System.Boolean, Windows.UI.Xaml.Controls.Primitives.ToggleButton, new Windows.UI.Xaml.PropertyMetadata.$ctor1(Bridge.box(false, System.Boolean, System.Boolean.toString)));
                }
            },
            methods: {
                IsChecked_Changed: function (d, e) {
                    var toggleButton = Bridge.cast(d, Windows.UI.Xaml.Controls.Primitives.ToggleButton);

                    var isChecked = Bridge.cast(Bridge.unbox(e.NewValue, System.Boolean), System.Boolean, true);

                    if (CSHTML5.Internal.INTERNAL_VisualTreeManager.IsElementInVisualTree(toggleButton) && !(toggleButton.HasTemplate)) {
                        toggleButton.UpdateDomBasedOnCheckedState(isChecked);
                    }

                    var hasValueRemainedTheSame = (e.NewValue != null && Bridge.equals(e.NewValue, e.OldValue)) || (e.NewValue == null && e.OldValue == null);
                    if (!hasValueRemainedTheSame) {
                        if (isChecked == null) {
                            toggleButton.OnIndeterminate();
                        } else if (System.Nullable.eq(isChecked, true)) {
                            toggleButton.OnChecked();
                        } else {
                            toggleButton.OnUnchecked();
                        }
                    }
                }
            }
        },
        fields: {
            INTERNAL_IsCodeProgrammaticallyChangingIsChecked: false,
            _isChecked: null,
            _isDisabled$1: false,
            _isPointerOver$1: false,
            _isPressed$1: false
        },
        events: {
            /**
             * Occurs when a ToggleButton is checked.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Controls.Primitives.ToggleButton
             * @memberof Windows.UI.Xaml.Controls.Primitives.ToggleButton
             * @function addChecked
             * @param   {Windows.UI.Xaml.RoutedEventHandler}    value
             * @return  {void}
             */
            /**
             * Occurs when a ToggleButton is checked.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Controls.Primitives.ToggleButton
             * @memberof Windows.UI.Xaml.Controls.Primitives.ToggleButton
             * @function removeChecked
             * @param   {Windows.UI.Xaml.RoutedEventHandler}    value
             * @return  {void}
             */
            Checked: null,
            /**
             * Occurs when the state of a ToggleButton is switched to the indeterminate state.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Controls.Primitives.ToggleButton
             * @memberof Windows.UI.Xaml.Controls.Primitives.ToggleButton
             * @function addIndeterminate
             * @param   {Windows.UI.Xaml.RoutedEventHandler}    value
             * @return  {void}
             */
            /**
             * Occurs when the state of a ToggleButton is switched to the indeterminate state.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Controls.Primitives.ToggleButton
             * @memberof Windows.UI.Xaml.Controls.Primitives.ToggleButton
             * @function removeIndeterminate
             * @param   {Windows.UI.Xaml.RoutedEventHandler}    value
             * @return  {void}
             */
            Indeterminate: null,
            /**
             * Occurs when a ToggleButton is unchecked.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Controls.Primitives.ToggleButton
             * @memberof Windows.UI.Xaml.Controls.Primitives.ToggleButton
             * @function addUnchecked
             * @param   {Windows.UI.Xaml.RoutedEventHandler}    value
             * @return  {void}
             */
            /**
             * Occurs when a ToggleButton is unchecked.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Controls.Primitives.ToggleButton
             * @memberof Windows.UI.Xaml.Controls.Primitives.ToggleButton
             * @function removeUnchecked
             * @param   {Windows.UI.Xaml.RoutedEventHandler}    value
             * @return  {void}
             */
            Unchecked: null
        },
        props: {
            /**
             * Gets or sets whether the ToggleButton is checked.
             Returns:
             True if the ToggleButton is checked; false if the ToggleButton is unchecked;
             otherwise null. The default is false.  If you are programming using C# or
             Visual Basic, the type of this property is projected as bool? (a nullable
             Boolean).
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.Primitives.ToggleButton
             * @function IsChecked
             * @type ?boolean
             */
            IsChecked: {
                get: function () {
                    return Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.Controls.Primitives.ToggleButton.IsCheckedProperty), System.Boolean), System.Boolean, true);
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Controls.Primitives.ToggleButton.IsCheckedProperty, Bridge.box(value, System.Boolean, System.Nullable.toStringFn(System.Boolean.toString), System.Nullable.getHashCode));
                }
            },
            /**
             * Determines whether the control supports two or three states.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.Primitives.ToggleButton
             * @function IsThreeState
             * @type boolean
             */
            IsThreeState: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.Controls.Primitives.ToggleButton.IsThreeStateProperty), System.Boolean), System.Boolean));
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Controls.Primitives.ToggleButton.IsThreeStateProperty, Bridge.box(value, System.Boolean, System.Boolean.toString));
                }
            }
        },
        ctors: {
            init: function () {
                this.INTERNAL_IsCodeProgrammaticallyChangingIsChecked = false;
                this._isChecked = false;
                this._isPointerOver$1 = false;
                this._isPressed$1 = false;
            },
            ctor: function () {
                this.$initialize();
                Windows.UI.Xaml.Controls.Primitives.ButtonBase.ctor.call(this);
                this._reactsToKeyboardEventsWhenFocused = true;


                this.DisableBaseControlHandlingOfVisualStates = true;

                this.SetDefaultStyle();
            }
        },
        methods: {
            SetDefaultStyle: function () {
                this.DefaultStyleKey = Windows.UI.Xaml.Controls.Primitives.ToggleButton;
            },
            OnKeyDownWhenFocused: function (sender, e) {
                if ((e.Key === Windows.System.VirtualKey.Space) || (e.Key === Windows.System.VirtualKey.Enter)) {
                    this.ToggleButton_Click(this, null);
                }
            },
            OnApplyTemplate: function () {
                Windows.UI.Xaml.Controls.Primitives.ButtonBase.prototype.OnApplyTemplate.call(this);

                this.addClick(Bridge.fn.cacheBind(this, this.ToggleButton_Click));
                var visualStateGroups = this.INTERNAL_GetVisualStateGroups();

                this.UpdateVisualStates$1();

                if (visualStateGroups != null && (visualStateGroups.ContainsVisualState("PointerOver") || visualStateGroups.ContainsVisualState("CheckedPointerOver") || visualStateGroups.ContainsVisualState("IndeterminatePointerOver"))) {
                    this.removePointerEntered(Bridge.fn.cacheBind(this, this.Control_PointerEntered$1));
                    this.addPointerEntered(Bridge.fn.cacheBind(this, this.Control_PointerEntered$1));
                    this.removePointerExited(Bridge.fn.cacheBind(this, this.Control_PointerExited$1));
                    this.addPointerExited(Bridge.fn.cacheBind(this, this.Control_PointerExited$1));
                    this.removePointerCaptureLost(Bridge.fn.cacheBind(this, this.Control_PointerCaptureLost));
                    this.addPointerCaptureLost(Bridge.fn.cacheBind(this, this.Control_PointerCaptureLost));
                }

                if (visualStateGroups != null && (visualStateGroups.ContainsVisualState("Pressed") || visualStateGroups.ContainsVisualState("CheckedPressed") || visualStateGroups.ContainsVisualState("IndeterminatePressed"))) {
                    this.removePointerPressed(Bridge.fn.cacheBind(this, this.Control_PointerPressed$1));
                    this.addPointerPressed(Bridge.fn.cacheBind(this, this.Control_PointerPressed$1));
                    this.removePointerReleased(Bridge.fn.cacheBind(this, this.Control_PointerReleased$1));
                    this.addPointerReleased(Bridge.fn.cacheBind(this, this.Control_PointerReleased$1));
                }
            },
            Control_PointerCaptureLost: function (sender, e) {
                this._isPressed$1 = false;
                this.UpdateVisualStates$1();
            },
            Control_PointerEntered$1: function (sender, e) {
                this._isPointerOver$1 = true;
                this.UpdateVisualStates$1();
            },
            Control_PointerExited$1: function (sender, e) {
                this._isPointerOver$1 = false;
                this.UpdateVisualStates$1();
            },
            Control_PointerPressed$1: function (sender, e) {
                this._isPressed$1 = true;
                this.UpdateVisualStates$1();
            },
            Control_PointerReleased$1: function (sender, e) {
                this._isPressed$1 = false;
                this.UpdateVisualStates$1();
            },
            UpdateVisualStates$1: function () {
                if (this.INTERNAL_IsLegacyVisualStates) {
                    if (this._isDisabled$1) {
                        Windows.UI.Xaml.VisualStateManager.GoToState(this, Windows.UI.Xaml.Controls.VisualStates.StateDisabled, true);
                    } else if (this._isPressed$1) {
                        Windows.UI.Xaml.VisualStateManager.GoToState(this, Windows.UI.Xaml.Controls.VisualStates.StatePressed, true);
                    } else if (this._isPointerOver$1) {
                        Windows.UI.Xaml.VisualStateManager.GoToState(this, "PointerOver", true);
                    } else {
                        Windows.UI.Xaml.VisualStateManager.GoToState(this, Windows.UI.Xaml.Controls.VisualStates.StateNormal, true);
                    }

                    if (System.Nullable.eq(this._isChecked, true)) {
                        Windows.UI.Xaml.VisualStateManager.GoToState(this, "Checked", true);
                    } else if (System.Nullable.eq(this._isChecked, false)) {
                        Windows.UI.Xaml.VisualStateManager.GoToState(this, "Unchecked", true);
                    } else {
                        Windows.UI.Xaml.VisualStateManager.GoToState(this, "Indeterminate", true);
                    }
                } else {
                    if (System.Nullable.eq(this._isChecked, false)) {
                        if (this._isDisabled$1) {
                            Windows.UI.Xaml.VisualStateManager.GoToState(this, Windows.UI.Xaml.Controls.VisualStates.StateDisabled, true);
                        } else if (this._isPressed$1) {
                            Windows.UI.Xaml.VisualStateManager.GoToState(this, Windows.UI.Xaml.Controls.VisualStates.StatePressed, true);
                        } else if (this._isPointerOver$1) {
                            Windows.UI.Xaml.VisualStateManager.GoToState(this, "PointerOver", true);
                        } else {
                            Windows.UI.Xaml.VisualStateManager.GoToState(this, Windows.UI.Xaml.Controls.VisualStates.StateNormal, true);
                        }
                    } else if (System.Nullable.eq(this._isChecked, true)) {
                        if (this._isDisabled$1) {
                            Windows.UI.Xaml.VisualStateManager.GoToState(this, "CheckedDisabled", true);
                        } else if (this._isPressed$1) {
                            Windows.UI.Xaml.VisualStateManager.GoToState(this, "CheckedPressed", true);
                        } else if (this._isPointerOver$1) {
                            Windows.UI.Xaml.VisualStateManager.GoToState(this, "CheckedPointerOver", true);
                        } else {
                            Windows.UI.Xaml.VisualStateManager.GoToState(this, "Checked", true);
                        }
                    } else if (this._isChecked == null) {
                        if (this._isDisabled$1) {
                            Windows.UI.Xaml.VisualStateManager.GoToState(this, "IndeterminateDisabled", true);
                        } else if (this._isPressed$1) {
                            Windows.UI.Xaml.VisualStateManager.GoToState(this, "IndeterminatePressed", true);
                        } else if (this._isPointerOver$1) {
                            Windows.UI.Xaml.VisualStateManager.GoToState(this, "IndeterminatePointerOver", true);
                        } else {
                            Windows.UI.Xaml.VisualStateManager.GoToState(this, "Indeterminate", true);
                        }
                    }
                }
            },
            ToggleButton_Click: function (sender, e) {
                if (System.Nullable.eq(this.IsChecked, true)) {
                    if (this.IsThreeState) {
                        this.SetCurrentValue(Windows.UI.Xaml.Controls.Primitives.ToggleButton.IsCheckedProperty, null);
                    } else {
                        this.SetCurrentValue(Windows.UI.Xaml.Controls.Primitives.ToggleButton.IsCheckedProperty, Bridge.box(false, System.Boolean, System.Boolean.toString));
                    }
                } else if (this.IsChecked == null) {
                    this.SetCurrentValue(Windows.UI.Xaml.Controls.Primitives.ToggleButton.IsCheckedProperty, Bridge.box(false, System.Boolean, System.Boolean.toString));
                } else {
                    this.SetCurrentValue(Windows.UI.Xaml.Controls.Primitives.ToggleButton.IsCheckedProperty, Bridge.box(true, System.Boolean, System.Boolean.toString));
                }
            },
            UnregisterFromDefaultClickEvent: function () {
                this.removeClick(Bridge.fn.cacheBind(this, this.ToggleButton_Click));
            },
            UpdateDomBasedOnCheckedState: function (isChecked) { },
            /**
             * // Gets or sets a value that indicates whether the control supports three states.
            // Returns true if the control supports three states; otherwise, false. The default is false.
            //
             * This method is used to subscribe to the click event on the child container so that clicking on it toggles it.
             *
             * @instance
             * @this Windows.UI.Xaml.Controls.Primitives.ToggleButton
             * @memberof Windows.UI.Xaml.Controls.Primitives.ToggleButton
             * @param   {object}    divWhereToPlaceChild    
             * @param   {object}    toggleDomElement
             * @return  {void}
             */
            SubscribeToClickEventForChildContainerDiv: function (divWhereToPlaceChild, toggleDomElement) { },
            /**
             * Raises the Checked event.
             *
             * @instance
             * @protected
             * @this Windows.UI.Xaml.Controls.Primitives.ToggleButton
             * @memberof Windows.UI.Xaml.Controls.Primitives.ToggleButton
             * @return  {void}
             */
            OnChecked: function () {
                var $t;
                if (!Bridge.staticEquals(this.Checked, null)) {
                    this.Checked(this, ($t = new Windows.UI.Xaml.RoutedEventArgs(), $t.OriginalSource = this, $t));
                }
                this._isChecked = true;
                this.UpdateVisualStates$1();
            },
            /**
             * Raises the Indeterminate event.
             *
             * @instance
             * @protected
             * @this Windows.UI.Xaml.Controls.Primitives.ToggleButton
             * @memberof Windows.UI.Xaml.Controls.Primitives.ToggleButton
             * @return  {void}
             */
            OnIndeterminate: function () {
                var $t;
                if (!Bridge.staticEquals(this.Indeterminate, null)) {
                    this.Indeterminate(this, ($t = new Windows.UI.Xaml.RoutedEventArgs(), $t.OriginalSource = this, $t));
                }
                this._isChecked = null;
                this.UpdateVisualStates$1();
            },
            /**
             * Raises the Unchecked event.
             *
             * @instance
             * @protected
             * @this Windows.UI.Xaml.Controls.Primitives.ToggleButton
             * @memberof Windows.UI.Xaml.Controls.Primitives.ToggleButton
             * @return  {void}
             */
            OnUnchecked: function () {
                var $t;
                if (!Bridge.staticEquals(this.Unchecked, null)) {
                    this.Unchecked(this, ($t = new Windows.UI.Xaml.RoutedEventArgs(), $t.OriginalSource = this, $t));
                }
                this._isChecked = false;
                this.UpdateVisualStates$1();

            },
            ManageIsEnabled: function (isEnabled) {
                Windows.UI.Xaml.Controls.Primitives.ButtonBase.prototype.ManageIsEnabled.call(this, isEnabled);

                this._isDisabled$1 = !isEnabled;
                this.UpdateVisualStates$1();
            }
        }
    });

    /**
     * Represents a selection control that combines a non-editable text box and
     a drop-down list box that allows users to select an item from a list.
     * // Gets a value that indicates whether the SelectionBoxItem component is highlighted.
    //
     *
     * @public
     * @class Windows.UI.Xaml.Controls.ComboBox
     * @augments Windows.UI.Xaml.Controls.Primitives.Selector
     * @example
     *
     *
     * ComboBox1.ItemsSource = Planet.GetListOfPlanets();
     * public partial class Planet
     *{
     *    public string Name { get; set; }
     *    public string ImagePath { get; set; }
     *    public static ObservableCollection<Planet> GetListOfPlanets()
     *    {
     *        return new ObservableCollection<Planet>()
     *        {
     *            new Planet() { Name = "Mercury", ImagePath = "ms-appx:/Planets/Mercury.png" },
     *            new Planet() { Name = "Venus", ImagePath = "ms-appx:/Planets/Venus.png" },
     *            new Planet() { Name = "Earth", ImagePath = "ms-appx:/Planets/Earth.png" },
     *            new Planet() { Name = "Mars", ImagePath = "ms-appx:/Planets/Mars.png" },
     *            new Planet() { Name = "Jupiter", ImagePath = "ms-appx:/Planets/Jupiter.png" },
     *            new Planet() { Name = "Saturn", ImagePath = "ms-appx:/Planets/Saturn.png" },
     *            new Planet() { Name = "Uranus", ImagePath = "ms-appx:/Planets/Uranus.png" },
     *            new Planet() { Name = "Neptune", ImagePath = "ms-appx:/Planets/Neptune.png" },
     *            new Planet() { Name = "Pluto", ImagePath = "ms-appx:/Planets/Pluto.png" }
     *        };
     *    }
     *}
     * 
     *
     *
     */
    Bridge.define("Windows.UI.Xaml.Controls.ComboBox", {
        inherits: [Windows.UI.Xaml.Controls.Primitives.Selector],
        statics: {
            fields: {
                /**
                 * Identifies the IsDropDownOpen dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Controls.ComboBox
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                IsDropDownOpenProperty: null,
                /**
                 * Identifies the MaxDropDownHeight dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Controls.ComboBox
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                MaxDropDownHeightProperty: null
            },
            ctors: {
                init: function () {
                    var $t;
                    this.IsDropDownOpenProperty = Windows.UI.Xaml.DependencyProperty.Register("IsDropDownOpen", System.Boolean, Windows.UI.Xaml.Controls.ComboBox, ($t = new Windows.UI.Xaml.PropertyMetadata.$ctor2(Bridge.box(false, System.Boolean, System.Boolean.toString), Windows.UI.Xaml.Controls.ComboBox.IsDropDownOpen_Changed), $t.CallPropertyChangedWhenLoadedIntoVisualTree = Windows.UI.Xaml.WhenToCallPropertyChangedEnum.IfPropertyIsSet, $t));
                    this.MaxDropDownHeightProperty = Windows.UI.Xaml.DependencyProperty.Register("MaxDropDownHeight", System.Double, Windows.UI.Xaml.Controls.ComboBox, new Windows.UI.Xaml.PropertyMetadata.$ctor1(Bridge.box(200.0, System.Double, System.Double.format, System.Double.getHashCode)));
                }
            },
            methods: {
                IsDropDownOpen_Changed: function (d, e) {
                    var comboBox = Bridge.cast(d, Windows.UI.Xaml.Controls.ComboBox);

                    if (Bridge.is(e.NewValue, System.Boolean)) {
                        var isDropDownOpen = System.Nullable.getValue(Bridge.cast(Bridge.unbox(e.NewValue, System.Boolean), System.Boolean));

                        if (isDropDownOpen) {

                            if (comboBox._contentPresenter != null) {
                                comboBox._contentPresenter.Content = null;
                            }

                            if (comboBox._popup != null) {

                                comboBox._popup.IsOpen = true;

                                var actualWidth = comboBox._popup.ActualWidth;
                                if (!isNaN(actualWidth) && Bridge.is(comboBox._popup.Child, Windows.UI.Xaml.FrameworkElement)) {
                                    Bridge.cast(comboBox._popup.Child, Windows.UI.Xaml.FrameworkElement).Width = actualWidth;
                                }
                            }

                            if (comboBox._dropDownToggle != null && System.Nullable.eq(comboBox._dropDownToggle.IsChecked, false)) {
                                comboBox._dropDownToggle.IsChecked = true;
                            }

                            comboBox.OnDropDownOpened(new Windows.UI.Xaml.RoutedEventArgs());

                            comboBox.EnsurePopupIsWithinBoundaries();
                        } else {

                            if (comboBox._popup != null) {
                                comboBox._popup.IsOpen = false;
                            }

                            if (comboBox._contentPresenter != null && comboBox._contentPresenter.Content == null) {
                                comboBox._contentPresenter.Content = comboBox._selectedContent;
                            }

                            if (comboBox._dropDownToggle != null && System.Nullable.eq(comboBox._dropDownToggle.IsChecked, true)) {
                                comboBox._dropDownToggle.IsChecked = false;
                            }

                            comboBox.OnDropDownClosed(new Windows.UI.Xaml.RoutedEventArgs());
                        }
                    }
                }
            }
        },
        fields: {
            _popup: null,
            _dropDownToggle: null,
            _contentPresenter: null,
            _selectedContent: null
        },
        events: {
            /**
             * Occurs when the drop-down portion of the ComboBox closes.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Controls.ComboBox
             * @memberof Windows.UI.Xaml.Controls.ComboBox
             * @function addDropDownClosed
             * @param   {Windows.UI.Xaml.RoutedEventHandler}    value
             * @return  {void}
             */
            /**
             * Occurs when the drop-down portion of the ComboBox closes.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Controls.ComboBox
             * @memberof Windows.UI.Xaml.Controls.ComboBox
             * @function removeDropDownClosed
             * @param   {Windows.UI.Xaml.RoutedEventHandler}    value
             * @return  {void}
             */
            DropDownClosed: null,
            /**
             * Occurs when the drop-down portion of the ComboBox opens.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Controls.ComboBox
             * @memberof Windows.UI.Xaml.Controls.ComboBox
             * @function addDropDownOpened
             * @param   {Windows.UI.Xaml.RoutedEventHandler}    value
             * @return  {void}
             */
            /**
             * Occurs when the drop-down portion of the ComboBox opens.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Controls.ComboBox
             * @memberof Windows.UI.Xaml.Controls.ComboBox
             * @function removeDropDownOpened
             * @param   {Windows.UI.Xaml.RoutedEventHandler}    value
             * @return  {void}
             */
            DropDownOpened: null
        },
        props: {
            UseNativeComboBox: {
                get: function () {
                    return false;
                },
                set: function (value) { }
            },
            /**
             * Gets or sets a value that indicates whether the drop-down portion of the
             ComboBox is currently open.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.ComboBox
             * @function IsDropDownOpen
             * @type boolean
             */
            IsDropDownOpen: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.Controls.ComboBox.IsDropDownOpenProperty), System.Boolean), System.Boolean));
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Controls.ComboBox.IsDropDownOpenProperty, Bridge.box(value, System.Boolean, System.Boolean.toString));
                }
            },
            /**
             * Gets or sets the maximum height for a combo box drop-down.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.ComboBox
             * @function MaxDropDownHeight
             * @type number
             */
            MaxDropDownHeight: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.Controls.ComboBox.MaxDropDownHeightProperty), System.Double), System.Double));
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Controls.ComboBox.MaxDropDownHeightProperty, Bridge.box(value, System.Double, System.Double.format, System.Double.getHashCode));
                }
            }
        },
        ctors: {
            /**
             * Initializes a new instance of the ComboBox class.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Controls.ComboBox
             * @memberof Windows.UI.Xaml.Controls.ComboBox
             * @return  {void}
             */
            ctor: function () {
                this.$initialize();
                Windows.UI.Xaml.Controls.Primitives.Selector.ctor.call(this);
                this.DefaultStyleKey = Windows.UI.Xaml.Controls.ComboBox;
            }
        },
        methods: {
            PrepareContainerForItemOverride: function (element, item) {
                Windows.UI.Xaml.Controls.Primitives.Selector.prototype.PrepareContainerForItemOverride.call(this, element, item);

                var container = Bridge.as(element, Windows.UI.Xaml.Controls.ComboBoxItem);
                if (container != null) {
                    container.INTERNAL_CorrespondingItem = item;
                    container.INTERNAL_ParentSelectorControl = this;
                    container.addClick(Bridge.fn.cacheBind(this, this.ComboBoxItem_Click));
                }
            },
            GetContainerForItemOverride: function () {
                return new Windows.UI.Xaml.Controls.ComboBoxItem();
            },
            IsItemItsOwnContainerOverride: function (item) {
                return (Bridge.is(item, Windows.UI.Xaml.Controls.ComboBoxItem));
            },
            INTERNAL_GenerateContainer: function (item) {
                return Bridge.cast(this.GetContainerFromItem(item), Windows.UI.Xaml.Controls.Primitives.SelectorItem);
            },
            GetContainerFromItem: function (item) {
                var comboBoxItem = Bridge.as(item, Windows.UI.Xaml.Controls.ComboBoxItem) || new Windows.UI.Xaml.Controls.ComboBoxItem();
                comboBoxItem.INTERNAL_CorrespondingItem = item;
                comboBoxItem.INTERNAL_ParentSelectorControl = this;
                comboBoxItem.addClick(Bridge.fn.cacheBind(this, this.ComboBoxItem_Click));
                return comboBoxItem;
            },
            ComboBoxItem_Click: function (sender, e) {
                var selectedContainer = Bridge.cast(sender, Windows.UI.Xaml.Controls.Primitives.SelectorItem);
                this._selectedContent = Bridge.as(sender, Windows.UI.Xaml.UIElement);

                this.SelectedItem = selectedContainer.INTERNAL_CorrespondingItem;

                if (this._dropDownToggle != null) {
                    this._dropDownToggle.IsChecked = false;
                }
            },
            ApplySelectedIndex: function (index) {
                Windows.UI.Xaml.Controls.Primitives.Selector.prototype.ApplySelectedIndex.call(this, index);

                if (this.ItemsHost == null) {
                    return;
                }

                var newSelectedContent;

                if (index === -1) {
                    if (this._contentPresenter != null) {
                        this._contentPresenter.Content = null;
                    }

                    newSelectedContent = null;
                } else if (index < this.ItemsHost.Children.Count) {
                    var container = Bridge.as(this.ItemsHost.Children.getItem(index), Windows.UI.Xaml.Controls.ComboBoxItem);
                    newSelectedContent = container;
                } else {
                    throw new System.IndexOutOfRangeException.ctor();
                }

                this._selectedContent = newSelectedContent;

                if (!this.IsDropDownOpen) {
                    if (this._contentPresenter != null) {
                        this._contentPresenter.Content = this._selectedContent;
                    }
                }
            },
            OnApplyTemplate: function () {
                Windows.UI.Xaml.Controls.Primitives.Selector.prototype.OnApplyTemplate.call(this);

                if (this._popup != null) {
                    this._popup.removeClosedDueToOutsideClick(Bridge.fn.cacheBind(this, this.Popup_ClosedDueToOutsideClick));
                }

                this._popup = Bridge.as(this.GetTemplateChild("Popup"), Windows.UI.Xaml.Controls.Primitives.Popup);
                this._dropDownToggle = Bridge.as(this.GetTemplateChild("DropDownToggle"), Windows.UI.Xaml.Controls.Primitives.ToggleButton);
                this._contentPresenter = Bridge.as(this.GetTemplateChild("ContentPresenter"), Windows.UI.Xaml.Controls.ContentPresenter);

                if (this._dropDownToggle != null) {
                    this._dropDownToggle.addChecked(Bridge.fn.cacheBind(this, this.DropDownToggle_Checked));
                    this._dropDownToggle.addUnchecked(Bridge.fn.cacheBind(this, this.DropDownToggle_Unchecked));
                }

                if (this._popup != null) {
                    this._popup.StayOpen = false;
                    this._popup.addClosedDueToOutsideClick(Bridge.fn.cacheBind(this, this.Popup_ClosedDueToOutsideClick));
                }

                this.ApplySelectedIndex(this.SelectedIndex);

                if (this._contentPresenter != null) {
                    this._contentPresenter.Content = this._selectedContent;
                }
            },
            DropDownToggle_Checked: function (sender, e) {
                this.IsDropDownOpen = true;
            },
            DropDownToggle_Unchecked: function (sender, e) {
                this.IsDropDownOpen = false;
            },
            /**
             * Invoked when the DropDownClosed event is raised.
             *
             * @instance
             * @protected
             * @this Windows.UI.Xaml.Controls.ComboBox
             * @memberof Windows.UI.Xaml.Controls.ComboBox
             * @param   {Windows.UI.Xaml.RoutedEventArgs}    e    Event data for the event.
             * @return  {void}
             */
            OnDropDownClosed: function (e) {
                if (!Bridge.staticEquals(this.DropDownClosed, null)) {
                    this.DropDownClosed(this, e);
                }
            },
            /**
             * Invoked when the DropDownOpened event is raised.
             *
             * @instance
             * @protected
             * @this Windows.UI.Xaml.Controls.ComboBox
             * @memberof Windows.UI.Xaml.Controls.ComboBox
             * @param   {Windows.UI.Xaml.RoutedEventArgs}    e    Event data for the event.
             * @return  {void}
             */
            OnDropDownOpened: function (e) {
                if (!Bridge.staticEquals(this.DropDownOpened, null)) {
                    this.DropDownOpened(this, e);
                }
            },
            EnsurePopupIsWithinBoundaries: function () {
                var $step = 0,
                    $task1, 
                    $jumpFromFinally, 
                    $asyncBody = Bridge.fn.bind(this, function () {
                        for (;;) {
                            $step = System.Array.min([0,1], $step);
                            switch ($step) {
                                case 0: {
                                    $task1 = System.Threading.Tasks.Task.delay(System.TimeSpan.fromSeconds(0.5));
                                    $step = 1;
                                    if ($task1.isCompleted()) {
                                        continue;
                                    }
                                    $task1.continue($asyncBody);
                                    return;
                                }
                                case 1: {
                                    $task1.getAwaitedResult();
                                    DotNetForHtml5.Core.INTERNAL_PopupsManager.EnsurePopupStaysWithinScreenBounds(this._popup);
                                    return;
                                }
                                default: {
                                    return;
                                }
                            }
                        }
                    }, arguments);

                $asyncBody();
            },
            Popup_ClosedDueToOutsideClick: function (sender, e) {

                if (this._dropDownToggle != null) {
                    if (this._dropDownToggle.IsPointerCaptured) {
                        this._dropDownToggle.ReleasePointerCapture();
                    }
                    this._dropDownToggle.IsChecked = false;
                }
            }
        }
    });

    /**
     * Provides a base class for ListBoxItem, ComboBoxItem, or potentially for other item types.
     *
     * @public
     * @class Windows.UI.Xaml.Controls.Primitives.SelectorItem
     * @augments Windows.UI.Xaml.Controls.Primitives.ButtonBase
     */
    Bridge.define("Windows.UI.Xaml.Controls.Primitives.SelectorItem", {
        inherits: [Windows.UI.Xaml.Controls.Primitives.ButtonBase],
        statics: {
            fields: {
                /**
                 * Identifies the IsSelected dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Controls.Primitives.SelectorItem
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                IsSelectedProperty: null
            },
            ctors: {
                init: function () {
                    var $t;
                    this.IsSelectedProperty = Windows.UI.Xaml.DependencyProperty.Register("IsSelected", System.Boolean, Windows.UI.Xaml.Controls.Primitives.SelectorItem, ($t = new Windows.UI.Xaml.PropertyMetadata.$ctor2(Bridge.box(false, System.Boolean, System.Boolean.toString), Windows.UI.Xaml.Controls.Primitives.SelectorItem.IsSelected_Changed), $t.CallPropertyChangedWhenLoadedIntoVisualTree = Windows.UI.Xaml.WhenToCallPropertyChangedEnum.IfPropertyIsSet, $t));
                }
            },
            methods: {
                IsSelected_Changed: function (d, e) {
                    var selectorItem = Bridge.cast(d, Windows.UI.Xaml.Controls.Primitives.SelectorItem);

                    if (CSHTML5.Internal.INTERNAL_VisualTreeManager.IsElementInVisualTree(selectorItem)) {
                        if (selectorItem.INTERNAL_ParentSelectorControl != null) {
                            var selector = selectorItem.INTERNAL_ParentSelectorControl;

                            if (selector.ItemContainerStyle == null) {
                                var newValue = System.Nullable.getValue(Bridge.cast(Bridge.unbox(e.NewValue, System.Boolean), System.Boolean));
                                if (newValue !== System.Nullable.getValue(Bridge.cast(Bridge.unbox(e.OldValue, System.Boolean), System.Boolean))) {
                                    if (newValue) {
                                        selectorItem._backgroundBeforeBeingSelected = selectorItem.Background;
                                        selectorItem._foregroundBeforeBeingSelected = selectorItem.Foreground;
                                        selectorItem._hasEverBeenSelected = true;

                                        selectorItem.Background = selector.SelectedItemBackground;
                                        selectorItem.Foreground = selector.SelectedItemForeground;
                                    } else {
                                        if (selectorItem._hasEverBeenSelected) {
                                            selectorItem.Background = selectorItem._backgroundBeforeBeingSelected;
                                            selectorItem.Foreground = selectorItem._foregroundBeforeBeingSelected;
                                        }
                                    }
                                }
                            }
                        }
                    }
                    if (System.Nullable.getValue(Bridge.cast(Bridge.unbox(e.NewValue, System.Boolean), System.Boolean))) {
                        Windows.UI.Xaml.VisualStateManager.GoToState(selectorItem, "Selected", false);
                    } else {
                        Windows.UI.Xaml.VisualStateManager.GoToState(selectorItem, "Unselected", false);
                    }
                }
            }
        },
        fields: {
            /**
             * The business object that the SelectorItem (ListBoxItem, ComboBoxItem, etc.) represents. For example, if the ItemsSource is a collection of business objects, each of them will be stored in this variable.
             *
             * @instance
             * @memberof Windows.UI.Xaml.Controls.Primitives.SelectorItem
             * @type System.Object
             */
            INTERNAL_CorrespondingItem: null,
            INTERNAL_ParentSelectorControl: null,
            _backgroundBeforeBeingSelected: null,
            _foregroundBeforeBeingSelected: null,
            _hasEverBeenSelected: false
        },
        props: {
            /**
             * Gets or sets a value that indicates whether the item is selected in a selector.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.Primitives.SelectorItem
             * @function IsSelected
             * @type boolean
             */
            IsSelected: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.Controls.Primitives.SelectorItem.IsSelectedProperty), System.Boolean), System.Boolean));
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Controls.Primitives.SelectorItem.IsSelectedProperty, Bridge.box(value, System.Boolean, System.Boolean.toString));
                }
            }
        },
        ctors: {
            init: function () {
                this._hasEverBeenSelected = false;
            },
            /**
             * Provides base class initialization behavior for SelectorItem-derived classes.
             *
             * @instance
             * @protected
             * @this Windows.UI.Xaml.Controls.Primitives.SelectorItem
             * @memberof Windows.UI.Xaml.Controls.Primitives.SelectorItem
             * @return  {void}
             */
            ctor: function () {
                this.$initialize();
                Windows.UI.Xaml.Controls.Primitives.ButtonBase.ctor.call(this);
            }
        },
        methods: {
            /**
             * Returns a string that represents the current object.
             *
             * @instance
             * @public
             * @override
             * @this Windows.UI.Xaml.Controls.Primitives.SelectorItem
             * @memberof Windows.UI.Xaml.Controls.Primitives.SelectorItem
             * @return  {string}        A string that represents the current object.
             */
            toString: function () {
                var $t;
                return Bridge.toString((($t = this.Content, $t != null ? $t : "")));
            }
        }
    });

    /**
     * Represents a pop-up menu that enables a control to expose functionality that is specific to the context of the control.
     *
     * @public
     * @class Windows.UI.Xaml.Controls.ContextMenu
     * @augments Windows.UI.Xaml.Controls.Primitives.MenuBase
     */
    Bridge.define("Windows.UI.Xaml.Controls.ContextMenu", {
        inherits: [Windows.UI.Xaml.Controls.Primitives.MenuBase],
        statics: {
            fields: {
                /**
                 * Identifies the IsOpen dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Controls.ContextMenu
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                IsOpenProperty: null,
                /**
                 * Identifies the HorizontalOffset dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Controls.ContextMenu
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                HorizontalOffsetProperty: null,
                /**
                 * Identifies the VerticalOffset dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Controls.ContextMenu
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                VerticalOffsetProperty: null
            },
            ctors: {
                init: function () {
                    var $t;
                    this.IsOpenProperty = Windows.UI.Xaml.DependencyProperty.Register("IsOpen", System.Boolean, Windows.UI.Xaml.Controls.ContextMenu, ($t = new Windows.UI.Xaml.PropertyMetadata.$ctor2(Bridge.box(false, System.Boolean, System.Boolean.toString), Windows.UI.Xaml.Controls.ContextMenu.IsOpen_Changed), $t.CallPropertyChangedWhenLoadedIntoVisualTree = Windows.UI.Xaml.WhenToCallPropertyChangedEnum.IfPropertyIsSet, $t));
                    this.HorizontalOffsetProperty = Windows.UI.Xaml.DependencyProperty.Register("HorizontalOffset", System.Double, Windows.UI.Xaml.Controls.ContextMenu, new Windows.UI.Xaml.PropertyMetadata.$ctor1(Bridge.box(0.0, System.Double, System.Double.format, System.Double.getHashCode)));
                    this.VerticalOffsetProperty = Windows.UI.Xaml.DependencyProperty.Register("VerticalOffset", System.Double, Windows.UI.Xaml.Controls.ContextMenu, new Windows.UI.Xaml.PropertyMetadata.$ctor1(Bridge.box(0.0, System.Double, System.Double.format, System.Double.getHashCode)));
                }
            },
            methods: {
                IsOpen_Changed: function (d, e) {
                    var $t;
                    var contextMenu = Bridge.cast(d, Windows.UI.Xaml.Controls.ContextMenu);
                    if (Bridge.is(e.NewValue, System.Boolean)) {
                        var isOpen = System.Nullable.getValue(Bridge.cast(Bridge.unbox(e.NewValue, System.Boolean), System.Boolean));
                        if (isOpen) {
                            var wasPopupAlreadyOpen = (contextMenu._parentPopup != null && contextMenu._parentPopup.IsOpen === true);
                            if (!wasPopupAlreadyOpen) {
                                contextMenu._isClickingInsidePopup = false;

                                if (Bridge.is(contextMenu.INTERNAL_ElementToWhichThisMenuIsAssigned, Windows.UI.Xaml.FrameworkElement)) {
                                    contextMenu.DataContext = Bridge.cast(contextMenu.INTERNAL_ElementToWhichThisMenuIsAssigned, Windows.UI.Xaml.FrameworkElement).DataContext;
                                }

                                var menuAbsolutePosition = new Windows.Foundation.Point();
                                if (System.Nullable.hasValue(contextMenu._forceSpecifyAbsoluteCoordinates)) {
                                    menuAbsolutePosition = System.Nullable.getValue(contextMenu._forceSpecifyAbsoluteCoordinates).$clone();
                                } else {
                                    menuAbsolutePosition = DotNetForHtml5.Core.INTERNAL_PopupsManager.CalculatePopupAbsolutePositionBasedOnElementPosition(contextMenu.INTERNAL_ElementToWhichThisMenuIsAssigned, contextMenu.HorizontalOffset, contextMenu.VerticalOffset);
                                }

                                contextMenu.HorizontalAlignment = Windows.UI.Xaml.HorizontalAlignment.Left;
                                contextMenu.VerticalAlignment = Windows.UI.Xaml.VerticalAlignment.Top;
                                Windows.UI.Xaml.Controls.Canvas.SetLeft(contextMenu, menuAbsolutePosition.X);
                                Windows.UI.Xaml.Controls.Canvas.SetTop(contextMenu, menuAbsolutePosition.Y);

                                if (contextMenu._parentPopup == null) {
                                    var canvas = ($t = new Windows.UI.Xaml.Controls.Canvas(), $t.Background = new Windows.UI.Xaml.Media.SolidColorBrush.$ctor1(Windows.UI.Colors.Transparent.$clone()), $t.HorizontalAlignment = Windows.UI.Xaml.HorizontalAlignment.Stretch, $t.VerticalAlignment = Windows.UI.Xaml.VerticalAlignment.Stretch, $t);
                                    canvas.Children.add(contextMenu);

                                    contextMenu._parentPopup = ($t = new Windows.UI.Xaml.Controls.Primitives.Popup(), $t.Child = canvas, $t.HorizontalAlignment = Windows.UI.Xaml.HorizontalAlignment.Stretch, $t.VerticalAlignment = Windows.UI.Xaml.VerticalAlignment.Stretch, $t.HorizontalContentAlignment = Windows.UI.Xaml.HorizontalAlignment.Stretch, $t.VerticalContentAlignment = Windows.UI.Xaml.VerticalAlignment.Stretch, $t);

                                    canvas.addPointerReleased(function (s2, e2) {
                                        if (!contextMenu._isClickingInsidePopup) {
                                            contextMenu.IsOpen = false;
                                        }
                                    });

                                    contextMenu.addPointerPressed(function (s2, e2) {
                                        contextMenu._isClickingInsidePopup = true;
                                    });
                                    contextMenu.addPointerReleased(function (s2, e2) {
                                        contextMenu.Dispatcher.BeginInvoke(function () {
                                            var $step = 0,
                                                $task1, 
                                                $jumpFromFinally, 
                                                $asyncBody = Bridge.fn.bind(this, function () {
                                                    for (;;) {
                                                        $step = System.Array.min([0,1], $step);
                                                        switch ($step) {
                                                            case 0: {
                                                                $task1 = System.Threading.Tasks.Task.delay(200);
                                                                $step = 1;
                                                                if ($task1.isCompleted()) {
                                                                    continue;
                                                                }
                                                                $task1.continue($asyncBody);
                                                                return;
                                                            }
                                                            case 1: {
                                                                $task1.getAwaitedResult();
                                                                contextMenu._isClickingInsidePopup = false;
                                                                contextMenu.IsOpen = false;
                                                                return;
                                                            }
                                                            default: {
                                                                return;
                                                            }
                                                        }
                                                    }
                                                }, arguments);

                                            $asyncBody();
                                        });
                                    });
                                }

                                contextMenu._parentPopup.IsOpen = true;

                                contextMenu.OnOpened(new Windows.UI.Xaml.RoutedEventArgs());
                            }
                        } else {
                            if (contextMenu._parentPopup != null && contextMenu._parentPopup.IsOpen === true) {
                                contextMenu._parentPopup.IsOpen = false;

                                contextMenu.OnClosed(new Windows.UI.Xaml.RoutedEventArgs());
                            }
                        }
                    }
                }
            }
        },
        fields: {
            _parentPopup: null,
            _forceSpecifyAbsoluteCoordinates: null,
            _isClickingInsidePopup: false,
            INTERNAL_ElementToWhichThisMenuIsAssigned: null,
            INTERNAL_HtmlCanvasElementToWhichThisMenuIsAssigned: null
        },
        events: {
            /**
             * Occurs when a particular instance of a System.Windows.Controls.ContextMenu closes.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Controls.ContextMenu
             * @memberof Windows.UI.Xaml.Controls.ContextMenu
             * @function addClosed
             * @param   {Windows.UI.Xaml.RoutedEventHandler}    value
             * @return  {void}
             */
            /**
             * Occurs when a particular instance of a System.Windows.Controls.ContextMenu closes.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Controls.ContextMenu
             * @memberof Windows.UI.Xaml.Controls.ContextMenu
             * @function removeClosed
             * @param   {Windows.UI.Xaml.RoutedEventHandler}    value
             * @return  {void}
             */
            Closed: null,
            /**
             * Occurs when a particular instance of a context menu opens.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Controls.ContextMenu
             * @memberof Windows.UI.Xaml.Controls.ContextMenu
             * @function addOpened
             * @param   {Windows.UI.Xaml.RoutedEventHandler}    value
             * @return  {void}
             */
            /**
             * Occurs when a particular instance of a context menu opens.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Controls.ContextMenu
             * @memberof Windows.UI.Xaml.Controls.ContextMenu
             * @function removeOpened
             * @param   {Windows.UI.Xaml.RoutedEventHandler}    value
             * @return  {void}
             */
            Opened: null
        },
        props: {
            /**
             * Gets or sets a value that indicates whether the ContextMenu is visible. True if the ContextMenu is visible; otherwise, false. The default is false.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.ContextMenu
             * @function IsOpen
             * @type boolean
             */
            IsOpen: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.Controls.ContextMenu.IsOpenProperty), System.Boolean), System.Boolean));
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Controls.ContextMenu.IsOpenProperty, Bridge.box(value, System.Boolean, System.Boolean.toString));
                }
            },
            /**
             * Gets or sets the horizontal distance between the target origin and the pop-up alignment point. The default is 0.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.ContextMenu
             * @function HorizontalOffset
             * @type number
             */
            HorizontalOffset: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.Controls.ContextMenu.HorizontalOffsetProperty), System.Double), System.Double));
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Controls.ContextMenu.HorizontalOffsetProperty, Bridge.box(value, System.Double, System.Double.format, System.Double.getHashCode));
                }
            },
            /**
             * Gets or sets the vertical distance between the target origin and the pop-up alignment point. The default is 0.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.ContextMenu
             * @function VerticalOffset
             * @type number
             */
            VerticalOffset: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.Controls.ContextMenu.VerticalOffsetProperty), System.Double), System.Double));
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Controls.ContextMenu.VerticalOffsetProperty, Bridge.box(value, System.Double, System.Double.format, System.Double.getHashCode));
                }
            }
        },
        ctors: {
            /**
             * Initializes a new instance of the System.Windows.Controls.ContextMenu class.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Controls.ContextMenu
             * @memberof Windows.UI.Xaml.Controls.ContextMenu
             * @return  {void}
             */
            ctor: function () {
                this.$initialize();
                Windows.UI.Xaml.Controls.Primitives.MenuBase.ctor.call(this);
            }
        },
        methods: {
            INTERNAL_OpenAtCoordinates: function (absoluteCoordinates) {
                this._forceSpecifyAbsoluteCoordinates = absoluteCoordinates.$clone();

                this.IsOpen = true;
            },
            /**
             * Called when the System.Windows.Controls.ContextMenu.Closed event occurs.
             *
             * @instance
             * @protected
             * @this Windows.UI.Xaml.Controls.ContextMenu
             * @memberof Windows.UI.Xaml.Controls.ContextMenu
             * @param   {Windows.UI.Xaml.RoutedEventArgs}    e    The event data for the System.Windows.Controls.ContextMenu.Closed event.
             * @return  {void}
             */
            OnClosed: function (e) {
                if (!Bridge.staticEquals(this.Closed, null)) {
                    this.Closed(this, new Windows.UI.Xaml.RoutedEventArgs());
                }
            },
            /**
             * Called when the System.Windows.Controls.ContextMenu.Opened event occurs.
             *
             * @instance
             * @protected
             * @this Windows.UI.Xaml.Controls.ContextMenu
             * @memberof Windows.UI.Xaml.Controls.ContextMenu
             * @param   {Windows.UI.Xaml.RoutedEventArgs}    e    The event data for the System.Windows.Controls.ContextMenu.Opened event.
             * @return  {void}
             */
            OnOpened: function (e) {
                if (!Bridge.staticEquals(this.Opened, null)) {
                    this.Opened(this, new Windows.UI.Xaml.RoutedEventArgs());
                }
            }
        }
    });

    /**
     * Provides an abstract class for controls that allow multiple items to be selected.
     *
     * @abstract
     * @public
     * @class Windows.UI.Xaml.Controls.Primitives.MultiSelector
     * @augments Windows.UI.Xaml.Controls.Primitives.Selector
     */
    Bridge.define("Windows.UI.Xaml.Controls.Primitives.MultiSelector", {
        inherits: [Windows.UI.Xaml.Controls.Primitives.Selector],
        statics: {
            fields: {
                /**
                 * Identifies the SelectedItems dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Controls.Primitives.MultiSelector
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                SelectedItemsProperty: null
            },
            ctors: {
                init: function () {
                    this.SelectedItemsProperty = Windows.UI.Xaml.DependencyProperty.Register("SelectedItems", System.Collections.IList, Windows.UI.Xaml.Controls.Primitives.MultiSelector, new Windows.UI.Xaml.PropertyMetadata.$ctor4(null));
                }
            }
        },
        props: {
            /**
             * Gets the items in the System.Windows.Controls.Primitives.MultiSelector that
             are selected.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.Primitives.MultiSelector
             * @function SelectedItems
             * @type System.Collections.IList
             */
            SelectedItems: {
                get: function () {
                    return Bridge.cast(this.GetValue(Windows.UI.Xaml.Controls.Primitives.MultiSelector.SelectedItemsProperty), System.Collections.IList);
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Controls.Primitives.MultiSelector.SelectedItemsProperty, value);
                }
            }
        },
        ctors: {
            /**
             * Initializes a new instance of the System.Windows.Controls.Primitives.MultiSelector
             class.
             *
             * @instance
             * @protected
             * @this Windows.UI.Xaml.Controls.Primitives.MultiSelector
             * @memberof Windows.UI.Xaml.Controls.Primitives.MultiSelector
             * @return  {void}
             */
            ctor: function () {
                this.$initialize();
                Windows.UI.Xaml.Controls.Primitives.Selector.ctor.call(this);
                this.SelectedItems = new (System.Collections.ObjectModel.ObservableCollection$1(System.Object)).ctor();
                Bridge.cast(this.SelectedItems, System.Collections.ObjectModel.ObservableCollection$1(System.Object)).addCollectionChanged(Bridge.fn.cacheBind(this, this.SelectedItems_CollectionChanged));
            }
        },
        methods: {
            SelectedItems_CollectionChanged: function (sender, e) {
                this.RefreshSelectedItems(e);
            },
            RefreshSelectedItems: function (e) {
                var $t, $t1;
                if (e.Action === System.Collections.Specialized.NotifyCollectionChangedAction.Reset) {
                    this.UnselectAllItems();
                }
                if (e.OldItems != null) {
                    var oldItems = CSHTML5.Internal.INTERNAL_ListsHelper.ConvertToListOfObjectsOrNull(e.OldItems);
                    $t = Bridge.getEnumerator(oldItems);
                    try {
                        while ($t.moveNext()) {
                            var item = $t.Current;
                            this.SetItemVisualSelectionState(item, false);
                        }
                    } finally {
                        if (Bridge.is($t, System.IDisposable)) {
                            $t.System$IDisposable$Dispose();
                        }
                    }
                }
                if (e.NewItems != null) {
                    var newItems = CSHTML5.Internal.INTERNAL_ListsHelper.ConvertToListOfObjectsOrNull(e.NewItems);
                    $t1 = Bridge.getEnumerator(newItems);
                    try {
                        while ($t1.moveNext()) {
                            var item1 = $t1.Current;
                            this.SetItemVisualSelectionState(item1, true);
                        }
                    } finally {
                        if (Bridge.is($t1, System.IDisposable)) {
                            $t1.System$IDisposable$Dispose();
                        }
                    }
                }
            },
            OnItemsSourceChanged_BeforeVisualUpdate: function (oldValue, newValue) {
                Windows.UI.Xaml.Controls.Primitives.Selector.prototype.OnItemsSourceChanged_BeforeVisualUpdate.call(this, oldValue, newValue);
                if (this.SelectedItems != null && System.Array.getCount(this.SelectedItems) !== 0) {
                    CSHTML5.Internal.INTERNAL_WorkaroundObservableCollectionBugWithJSIL.Clear(this.SelectedItems);
                }
            }
        }
    });

    /**
     * Represents a cell of a System.Windows.Controls.DataGrid control.
     *
     * @public
     * @class Windows.UI.Xaml.Controls.DataGridCell
     * @augments Windows.UI.Xaml.Controls.Primitives.ButtonBase
     */
    Bridge.define("Windows.UI.Xaml.Controls.DataGridCell", {
        inherits: [Windows.UI.Xaml.Controls.Primitives.ButtonBase],
        statics: {
            fields: {
                /**
                 * Identifies the System.Windows.Controls.DataGridCell.Column dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Controls.DataGridCell
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                ColumnProperty: null,
                /**
                 * Identifies the System.Windows.Controls.DataGridCell.IsEditing dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Controls.DataGridCell
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                IsEditingProperty: null,
                /**
                 * Identifies the System.Windows.Controls.DataGridCell.IsSelected dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Controls.DataGridCell
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                IsSelectedProperty: null
            },
            ctors: {
                init: function () {
                    var $t;
                    this.ColumnProperty = Windows.UI.Xaml.DependencyProperty.Register("Column", Windows.UI.Xaml.Controls.DataGridColumn, Windows.UI.Xaml.Controls.DataGridCell, new Windows.UI.Xaml.PropertyMetadata.$ctor4(null));
                    this.IsEditingProperty = Windows.UI.Xaml.DependencyProperty.Register("IsEditing", System.Boolean, Windows.UI.Xaml.Controls.DataGridCell, ($t = new Windows.UI.Xaml.PropertyMetadata.$ctor2(Bridge.box(false, System.Boolean, System.Boolean.toString), Windows.UI.Xaml.Controls.DataGridCell.IsEditing_Changed), $t.CallPropertyChangedWhenLoadedIntoVisualTree = Windows.UI.Xaml.WhenToCallPropertyChangedEnum.IfPropertyIsSet, $t));
                    this.IsSelectedProperty = Windows.UI.Xaml.DependencyProperty.Register("IsSelected", System.Boolean, Windows.UI.Xaml.Controls.DataGridCell, ($t = new Windows.UI.Xaml.PropertyMetadata.$ctor2(Bridge.box(false, System.Boolean, System.Boolean.toString), Windows.UI.Xaml.Controls.DataGridCell.IsSelected_Changed), $t.CallPropertyChangedWhenLoadedIntoVisualTree = Windows.UI.Xaml.WhenToCallPropertyChangedEnum.IfPropertyIsSet, $t));
                }
            },
            methods: {
                IsEditing_Changed: function (d, e) {
                    var cell = Bridge.cast(d, Windows.UI.Xaml.Controls.DataGridCell);
                    Bridge.cast(cell.Content, Windows.UI.Xaml.FrameworkElement).IsEnabled = System.Nullable.getValue(Bridge.cast(Bridge.unbox(e.NewValue, System.Boolean), System.Boolean));
                    cell.ManageIsEditingChange(e);
                },
                IsSelected_Changed: function (d, e) {
                    var dataGridCell = Bridge.cast(d, Windows.UI.Xaml.Controls.DataGridCell);

                    if (CSHTML5.Internal.INTERNAL_VisualTreeManager.IsElementInVisualTree(dataGridCell)) {
                        var newValue = System.Nullable.getValue(Bridge.cast(Bridge.unbox(e.NewValue, System.Boolean), System.Boolean));
                        if (newValue) {
                            dataGridCell.Background = dataGridCell.Column._parent.SelectedItemBackground;
                            dataGridCell.Foreground = dataGridCell.Column._parent.SelectedItemForeground;
                        } else {
                            var RowIndex = Windows.UI.Xaml.Controls.Grid.GetRow(dataGridCell);

                            if (RowIndex % 2 === 0 && RowIndex !== 0) {
                                dataGridCell.Background = dataGridCell.Column._parent.AlternatingRowBackground || dataGridCell.Column._parent.Background;
                            } else {
                                dataGridCell.Background = dataGridCell.Column._parent.RowBackground || dataGridCell.Column._parent.Background;
                            }
                            dataGridCell.Foreground = dataGridCell.Column._parent.UnselectedItemForeground || dataGridCell.Column._parent.Foreground;

                        }
                    }
                }
            }
        },
        fields: {
            Item: null
        },
        props: {
            /**
             * Gets the column that the cell is in.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.DataGridCell
             * @function Column
             * @type Windows.UI.Xaml.Controls.DataGridColumn
             */
            Column: {
                get: function () {
                    return Bridge.cast(this.GetValue(Windows.UI.Xaml.Controls.DataGridCell.ColumnProperty), Windows.UI.Xaml.Controls.DataGridColumn);
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Controls.DataGridCell.ColumnProperty, value);
                }
            },
            /**
             * Gets or sets a value that indicates whether the cell is in edit mode.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.DataGridCell
             * @function IsEditing
             * @type boolean
             */
            IsEditing: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.Controls.DataGridCell.IsEditingProperty), System.Boolean), System.Boolean));
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Controls.DataGridCell.IsEditingProperty, Bridge.box(value, System.Boolean, System.Boolean.toString));
                }
            },
            /**
             * Gets or sets a value that indicates whether the cell is selected.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.DataGridCell
             * @function IsSelected
             * @type boolean
             */
            IsSelected: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.Controls.DataGridCell.IsSelectedProperty), System.Boolean), System.Boolean));
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Controls.DataGridCell.IsSelectedProperty, Bridge.box(value, System.Boolean, System.Boolean.toString));
                }
            }
        },
        methods: {
            ManageIsEditingChange: function (e) {
                if (System.Nullable.getValue(Bridge.cast(Bridge.unbox(e.NewValue, System.Boolean), System.Boolean))) {
                    this.Column.EnterEditionMode(this);
                } else {
                    this.Column.LeaveEditionMode(this);
                }
            },
            OnSelectedFromContent: function (sender, e) {
                this.Column._parent.SelectedItem = this.Item;
            },
            OnClickOnContent: function (sender, e) {
                this.OnClick();
            },
            RegisterToContentPressEvent: function () {
                if (this.Content != null) {
                    if (Bridge.is(this.Content, Windows.UI.Xaml.Controls.Control)) {
                        var control = Bridge.cast(this.Content, Windows.UI.Xaml.Controls.Control);

                        control.addPointerReleased(Bridge.fn.cacheBind(this, this.OnClickOnContent));
                    }
                }
            },
            CreateDomElement: function (parentRef, domElementWhereToPlaceChildren) {
                var div = Windows.UI.Xaml.Controls.Primitives.ButtonBase.prototype.CreateDomElement.call(this, parentRef, domElementWhereToPlaceChildren);
                var style = CSHTML5.Internal.INTERNAL_HtmlDomManager.GetDomElementStyleForModification(div);
                style.overflow = "hidden";
                style.boxSizing = "border-box";
                return div;
            }
        }
    });

    Bridge.define("Windows.UI.Xaml.Controls.Expander", {
        inherits: [Windows.UI.Xaml.Controls.HeaderedContentControl],
        statics: {
            fields: {
                /**
                 * The name of the ExpanderButton template part.
                 *
                 * @static
                 * @private
                 * @memberof Windows.UI.Xaml.Controls.Expander
                 * @constant
                 * @default "ExpanderButton"
                 * @type string
                 */
                ElementExpanderButtonName: null,
                /**
                 * Identifies the
                 {@link }
                 dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Controls.Expander
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                IsExpandedProperty: null
            },
            ctors: {
                init: function () {
                    var $t;
                    this.ElementExpanderButtonName = "ExpanderButton";
                    this.IsExpandedProperty = Windows.UI.Xaml.DependencyProperty.Register("IsExpanded", System.Boolean, Windows.UI.Xaml.Controls.Expander, ($t = new Windows.UI.Xaml.PropertyMetadata.$ctor2(Bridge.box(false, System.Boolean, System.Boolean.toString), Windows.UI.Xaml.Controls.Expander.OnIsExpandedPropertyChanged), $t.CallPropertyChangedWhenLoadedIntoVisualTree = Windows.UI.Xaml.WhenToCallPropertyChangedEnum.IfPropertyIsSet, $t));
                }
            },
            methods: {
                /**
                 * ExpandedProperty PropertyChangedCallback static function.
                 *
                 * @static
                 * @private
                 * @this Windows.UI.Xaml.Controls.Expander
                 * @memberof Windows.UI.Xaml.Controls.Expander
                 * @param   {Windows.UI.Xaml.DependencyObject}                      d    Expander object whose Expanded property is changed.
                 * @param   {Windows.UI.Xaml.DependencyPropertyChangedEventArgs}    e    DependencyPropertyChangedEventArgs which contains the old and new values.
                 * @return  {void}
                 */
                OnIsExpandedPropertyChanged: function (d, e) {
                    var ctrl = Bridge.cast(d, Windows.UI.Xaml.Controls.Expander);
                    var isExpanded = System.Nullable.getValue(Bridge.cast(Bridge.unbox(e.NewValue, System.Boolean), System.Boolean));


                    if (isExpanded) {
                        ctrl.OnExpanded();
                    } else {
                        ctrl.OnCollapsed();
                    }
                }
            }
        },
        fields: {
            _isInToggleButtonClickHandler: false,
            /**
             * The ExpanderButton template part is a templated ToggleButton that's used 
             to expand and collapse the ExpandSite, which hosts the content.
             *
             * @instance
             * @private
             * @memberof Windows.UI.Xaml.Controls.Expander
             * @type Windows.UI.Xaml.Controls.Primitives.ToggleButton
             */
            _expanderButton: null
        },
        events: {
            /**
             * Occurs when the content window of an
             {@link } control opens to
             display both its header and content.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Controls.Expander
             * @memberof Windows.UI.Xaml.Controls.Expander
             * @function addExpanded
             * @param   {Windows.UI.Xaml.RoutedEventHandler}    value
             * @return  {void}
             */
            /**
             * Occurs when the content window of an
             {@link } control opens to
             display both its header and content.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Controls.Expander
             * @memberof Windows.UI.Xaml.Controls.Expander
             * @function removeExpanded
             * @param   {Windows.UI.Xaml.RoutedEventHandler}    value
             * @return  {void}
             */
            Expanded: null,
            /**
             * Occurs when the content window of an
             {@link } control closes and
             only the
             {@link }
             is visible.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Controls.Expander
             * @memberof Windows.UI.Xaml.Controls.Expander
             * @function addCollapsed
             * @param   {Windows.UI.Xaml.RoutedEventHandler}    value
             * @return  {void}
             */
            /**
             * Occurs when the content window of an
             {@link } control closes and
             only the
             {@link }
             is visible.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Controls.Expander
             * @memberof Windows.UI.Xaml.Controls.Expander
             * @function removeCollapsed
             * @param   {Windows.UI.Xaml.RoutedEventHandler}    value
             * @return  {void}
             */
            Collapsed: null
        },
        props: {
            /**
             * Gets or sets a value indicating whether the
             {@link } content window is
             visible.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.Expander
             * @function IsExpanded
             * @type boolean
             */
            IsExpanded: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.Controls.Expander.IsExpandedProperty), System.Boolean), System.Boolean));
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Controls.Expander.IsExpandedProperty, Bridge.box(value, System.Boolean, System.Boolean.toString));
                }
            },
            /**
             * Gets or sets the ExpanderButton template part.
             *
             * @instance
             * @private
             * @memberof Windows.UI.Xaml.Controls.Expander
             * @function ExpanderButton
             * @type Windows.UI.Xaml.Controls.Primitives.ToggleButton
             */
            ExpanderButton: {
                get: function () {
                    return this._expanderButton;
                },
                set: function (value) {
                    if (this._expanderButton != null) {
                        this._expanderButton.removeClick(Bridge.fn.cacheBind(this, this.OnExpanderButtonClicked));
                    }

                    this._expanderButton = value;

                    if (this._expanderButton != null) {
                        this._expanderButton.IsChecked = this.IsExpanded;
                        this._expanderButton.addClick(Bridge.fn.cacheBind(this, this.OnExpanderButtonClicked));
                    }
                }
            }
        },
        ctors: {
            init: function () {
                this._isInToggleButtonClickHandler = false;
            },
            /**
             * Initializes a new instance of the
             {@link } class.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Controls.Expander
             * @memberof Windows.UI.Xaml.Controls.Expander
             * @return  {void}
             */
            ctor: function () {
                this.$initialize();
                Windows.UI.Xaml.Controls.HeaderedContentControl.ctor.call(this);

                this.DefaultStyleKey = Windows.UI.Xaml.Controls.Expander;
            }
        },
        methods: {
            /**
             * Raises the
             {@link } event
             when the
             {@link }
             property changes from false to true.
             *
             * @instance
             * @protected
             * @this Windows.UI.Xaml.Controls.Expander
             * @memberof Windows.UI.Xaml.Controls.Expander
             * @return  {void}
             */
            OnExpanded: function () {
                this.ToggleExpanded(this.Expanded, new Windows.UI.Xaml.RoutedEventArgs());
            },
            /**
             * Raises the
             {@link } event
             when the
             {@link }
             property changes from true to false.
             *
             * @instance
             * @protected
             * @this Windows.UI.Xaml.Controls.Expander
             * @memberof Windows.UI.Xaml.Controls.Expander
             * @return  {void}
             */
            OnCollapsed: function () {
                this.ToggleExpanded(this.Collapsed, new Windows.UI.Xaml.RoutedEventArgs());
            },
            /**
             * Handle changes to the IsExpanded property.
             *
             * @instance
             * @private
             * @this Windows.UI.Xaml.Controls.Expander
             * @memberof Windows.UI.Xaml.Controls.Expander
             * @param   {Windows.UI.Xaml.RoutedEventHandler}    handler    Event handler.
             * @param   {Windows.UI.Xaml.RoutedEventArgs}       args       Event arguments.
             * @return  {void}
             */
            ToggleExpanded: function (handler, args) {
                var expander = this.ExpanderButton;
                if (!this._isInToggleButtonClickHandler && expander != null) {
                    expander.IsChecked = this.IsExpanded;
                }

                this.UpdateVisualState(true);
                this.RaiseEvent(handler, args);
            },
            /**
             * Raise a RoutedEvent.
             *
             * @instance
             * @private
             * @this Windows.UI.Xaml.Controls.Expander
             * @memberof Windows.UI.Xaml.Controls.Expander
             * @param   {Windows.UI.Xaml.RoutedEventHandler}    handler    Event handler.
             * @param   {Windows.UI.Xaml.RoutedEventArgs}       args       Event arguments.
             * @return  {void}
             */
            RaiseEvent: function (handler, args) {
                if (!Bridge.staticEquals(handler, null)) {
                    handler(this, args);
                }
            },
            /**
             * Handle ExpanderButton's click event.
             *
             * @instance
             * @private
             * @this Windows.UI.Xaml.Controls.Expander
             * @memberof Windows.UI.Xaml.Controls.Expander
             * @param   {System.Object}                      sender    The ExpanderButton in template.
             * @param   {Windows.UI.Xaml.RoutedEventArgs}    e         Routed event arg.
             * @return  {void}
             */
            OnExpanderButtonClicked: function (sender, e) {
                this._isInToggleButtonClickHandler = true;
                this.IsExpanded = !this.IsExpanded;
                this._isInToggleButtonClickHandler = false;
            },
            /**
             * // Update the visual state of the control.
            //
             * Update the current visual state of the button.
             *
             * @instance
             * @this Windows.UI.Xaml.Controls.Expander
             * @memberof Windows.UI.Xaml.Controls.Expander
             * @param   {boolean}    useTransitions    True to use transitions when updating the visual state, false to
             snap directly to the new visual state.
             * @return  {void}
             */
            UpdateVisualState: function (useTransitions) {
                if (this.IsExpanded) {
                    Windows.UI.Xaml.VisualStateManager.GoToState(this, "Expanded", false);
                } else {
                    Windows.UI.Xaml.VisualStateManager.GoToState(this, "Collapsed", false);
                }

            },
            /**
             * Builds the visual tree for the
             {@link } control when a new
             template is applied.
             *
             * @instance
             * @protected
             * @override
             * @this Windows.UI.Xaml.Controls.Expander
             * @memberof Windows.UI.Xaml.Controls.Expander
             * @return  {void}
             */
            OnApplyTemplate: function () {
                Windows.UI.Xaml.Controls.HeaderedContentControl.prototype.OnApplyTemplate.call(this);
                this.ExpanderButton = Bridge.as(this.GetTemplateChild(Windows.UI.Xaml.Controls.Expander.ElementExpanderButtonName), Windows.UI.Xaml.Controls.Primitives.ToggleButton);
            }
        }
    });

    /**
     * Represents a button control that displays a hyperlink.
     *
     * @public
     * @class Windows.UI.Xaml.Controls.HyperlinkButton
     * @augments Windows.UI.Xaml.Controls.Primitives.ButtonBase
     * @example
     *
     *
     * HyperlinkButton hyperlinkButton = new HyperlinkButton() { Content = "here", NavigateUri = new Uri("http://www.myaddress.com"), Foreground = new SolidColorBrush(Windows.UI.Colors.Blue) };
     * MyStackPanel.Children.Add(hyperlinkButton);
     * 
     *
     *
     */
    Bridge.define("Windows.UI.Xaml.Controls.HyperlinkButton", {
        inherits: [Windows.UI.Xaml.Controls.Primitives.ButtonBase],
        statics: {
            fields: {
                /**
                 * Identifies the NavigateUri dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Controls.HyperlinkButton
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                NavigateUriProperty: null,
                /**
                 * Identifies the TargetName dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Controls.HyperlinkButton
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                TargetNameProperty: null
            },
            ctors: {
                init: function () {
                    this.NavigateUriProperty = Windows.UI.Xaml.DependencyProperty.Register("NavigateUri", System.Uri, Windows.UI.Xaml.Controls.HyperlinkButton, new Windows.UI.Xaml.PropertyMetadata.$ctor4(null));
                    this.TargetNameProperty = Windows.UI.Xaml.DependencyProperty.Register("TargetName", System.String, Windows.UI.Xaml.Controls.HyperlinkButton, new Windows.UI.Xaml.PropertyMetadata.$ctor1(""));
                }
            }
        },
        props: {
            /**
             * Gets or sets the Uniform Resource Identifier (URI) to navigate to when the
             HyperlinkButton is clicked.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.HyperlinkButton
             * @function NavigateUri
             * @type System.Uri
             */
            NavigateUri: {
                get: function () {
                    return Bridge.cast(this.GetValue(Windows.UI.Xaml.Controls.HyperlinkButton.NavigateUriProperty), System.Uri);
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Controls.HyperlinkButton.NavigateUriProperty, value);
                }
            },
            /**
             * Gets or sets the name of the target window or frame that the Web page should
             open in, or the name of the object within the application to navigate to.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.HyperlinkButton
             * @function TargetName
             * @type string
             */
            TargetName: {
                get: function () {
                    return Bridge.cast(this.GetValue(Windows.UI.Xaml.Controls.HyperlinkButton.TargetNameProperty), System.String);
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Controls.HyperlinkButton.TargetNameProperty, value);
                }
            }
        },
        ctors: {
            /**
             * Initializes a new instance of the HyperlinkButton class.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Controls.HyperlinkButton
             * @memberof Windows.UI.Xaml.Controls.HyperlinkButton
             * @return  {void}
             */
            ctor: function () {
                this.$initialize();
                Windows.UI.Xaml.Controls.Primitives.ButtonBase.ctor.call(this);
                this.DefaultStyleKey = Windows.UI.Xaml.Controls.HyperlinkButton;

                this.addClick(Bridge.fn.cacheBind(this, this.HyperlinkButton_Click));
            }
        },
        methods: {
            HyperlinkButton_Click: function (sender, e) {
                if (System.Uri.notEquals(this.NavigateUri, null)) {
                    var targetName = this.TargetName;
                    var targetElement = null;

                    if (!System.String.isNullOrEmpty(targetName) && Bridge.is(((targetElement = this.FindName(targetName))), Windows.UI.Xaml.Controls.INavigate)) {

                        Bridge.cast(targetElement, Windows.UI.Xaml.Controls.INavigate).Windows$UI$Xaml$Controls$INavigate$Navigate(this.NavigateUri);
                    } else {

                        if (System.String.isNullOrEmpty(targetName)) {
                            targetName = "_blank";
                        }
                        System.Windows.Browser.HtmlPage.Window.Navigate(this.NavigateUri, targetName);
                    }
                }
            }
        }
    });

    /**
     * A control that enabled drag and drop operations on an TItemsControl.
     * // Gets or sets the duration to use to animate an item into view.
    //
     * // Identifies the ScrollItemIntoViewAnimationDuration dependency property.
    //
     * // Returns a value indicating whether an item can be added to the
    // items control.
    //
     * // Retrieves the number of items in an items control.
    //
     * // Retrieves the items host for a given items control.
    //
     * // Returns a value indicating whether an item can be removed from the
    // items control.
    //
     * // Returns a value indicating whether a container belongs to an items 
    // control.
    //
     * // Gets the item from an item container.
    //
     * // Returns a value indicating whether a given items control
    // can scroll.
    //
     * // Scrolls a given item container into the view.
    //
     *
     * @public
     * @class Windows.UI.Xaml.Controls.ItemsControlDragDropTarget
     * @augments Windows.UI.Xaml.Controls.DragDropTarget$2
     * @param   {Function}    [name]    The type of the control that contains 
     the items that can be dragged.
     * @param   {Function}    [name]    The type of the item container.
     * @return  {[type]}                The value indicating whether the given items control
    // can scroll.
     */
    Bridge.define("Windows.UI.Xaml.Controls.ItemsControlDragDropTarget", {
        inherits: [Windows.UI.Xaml.Controls.DragDropTarget$2(Windows.UI.Xaml.Controls.ItemsControl,Windows.UI.Xaml.UIElement)],
        methods: {
            /**
             * Adds an item to an items control.
             *
             * @instance
             * @protected
             * @override
             * @this Windows.UI.Xaml.Controls.ItemsControlDragDropTarget
             * @memberof Windows.UI.Xaml.Controls.ItemsControlDragDropTarget
             * @param   {Windows.UI.Xaml.Controls.ItemsControl}    itemsControl    The items control.
             * @param   {System.Object}                            data            The data to be inserted.
             * @return  {void}
             */
            AddItem: function (itemsControl, data) {
                var dataAsFrameworkElement = Bridge.as(data, Windows.UI.Xaml.FrameworkElement);
                if (dataAsFrameworkElement != null) {
                    if (dataAsFrameworkElement.DataContext != null) {
                        Windows.UI.Xaml.Controls.ItemsControlExtensions.AddItem(itemsControl, dataAsFrameworkElement.DataContext);
                    }
                } else {
                    Windows.UI.Xaml.Controls.ItemsControlExtensions.AddItem(itemsControl, data);
                }
            },
            /**
             * Retrieves the item container at a given index.
             *
             * @instance
             * @protected
             * @override
             * @this Windows.UI.Xaml.Controls.ItemsControlDragDropTarget
             * @memberof Windows.UI.Xaml.Controls.ItemsControlDragDropTarget
             * @param   {Windows.UI.Xaml.Controls.ItemsControl}    itemsControl    The items control.
             * @param   {number}                                   index           The index at which to retrieve the container.
             * @return  {Windows.UI.Xaml.UIElement}                                The item container at a given index.
             */
            ContainerFromIndex: function (itemsControl, index) {
                return Bridge.as(itemsControl.ItemContainerGenerator.ContainerFromIndex(index), Windows.UI.Xaml.UIElement);
            },
            /**
             * Retrieves the index of an item container.
             *
             * @instance
             * @protected
             * @override
             * @this Windows.UI.Xaml.Controls.ItemsControlDragDropTarget
             * @memberof Windows.UI.Xaml.Controls.ItemsControlDragDropTarget
             * @param   {Windows.UI.Xaml.Controls.ItemsControl}    itemsControl     The items control.
             * @param   {Windows.UI.Xaml.UIElement}                itemContainer    The item container.
             * @return  {?number}                                                   The index of an item container.
             */
            IndexFromContainer: function (itemsControl, itemContainer) {
                var index = itemsControl.ItemContainerGenerator.IndexFromContainer(itemContainer);
                return (index === -1) ? null : index;
            },
            /**
             * Inserts an item into an items control.
             *
             * @instance
             * @protected
             * @override
             * @this Windows.UI.Xaml.Controls.ItemsControlDragDropTarget
             * @memberof Windows.UI.Xaml.Controls.ItemsControlDragDropTarget
             * @param   {Windows.UI.Xaml.Controls.ItemsControl}    itemsControl    The items control.
             * @param   {number}                                   index           The index at which to insert the item.
             * @param   {System.Object}                            data            The data to be inserted.
             * @return  {void}
             */
            InsertItem: function (itemsControl, index, data) {
                var dataAsFrameworkElement = Bridge.as(data, Windows.UI.Xaml.FrameworkElement);
                if (dataAsFrameworkElement != null) {
                    if (dataAsFrameworkElement.DataContext != null) {
                        Windows.UI.Xaml.Controls.ItemsControlExtensions.InsertItem(itemsControl, index, dataAsFrameworkElement.DataContext);
                    }
                } else {
                    Windows.UI.Xaml.Controls.ItemsControlExtensions.InsertItem(itemsControl, index, data);
                }
            },
            /**
             * Removes an item from an items control.
             *
             * @instance
             * @protected
             * @override
             * @this Windows.UI.Xaml.Controls.ItemsControlDragDropTarget
             * @memberof Windows.UI.Xaml.Controls.ItemsControlDragDropTarget
             * @param   {Windows.UI.Xaml.Controls.ItemsControl}    itemsControl    The items control.
             * @param   {System.Object}                            data            The data to be removed.
             * @return  {void}
             */
            RemoveItem: function (itemsControl, data) {
                var dataAsFrameworkElement = Bridge.as(data, Windows.UI.Xaml.FrameworkElement);
                if (dataAsFrameworkElement != null) {
                    Windows.UI.Xaml.Controls.ItemsControlExtensions.RemoveItem(itemsControl, dataAsFrameworkElement.DataContext);
                } else {
                    Windows.UI.Xaml.Controls.ItemsControlExtensions.RemoveItem(itemsControl, data);
                }
            },
            /**
             * Removes an item from an items control by index.
             *
             * @instance
             * @protected
             * @override
             * @this Windows.UI.Xaml.Controls.ItemsControlDragDropTarget
             * @memberof Windows.UI.Xaml.Controls.ItemsControlDragDropTarget
             * @param   {Windows.UI.Xaml.Controls.ItemsControl}    itemsControl    The items control.
             * @param   {number}                                   index           The index of the item to be removed.
             * @return  {void}
             */
            RemoveItemAtIndex: function (itemsControl, index) { },
            /**
             * Create a new TItemsControlType
             *
             * @instance
             * @protected
             * @override
             * @this Windows.UI.Xaml.Controls.ItemsControlDragDropTarget
             * @memberof Windows.UI.Xaml.Controls.ItemsControlDragDropTarget
             * @return  {Windows.UI.Xaml.Controls.ItemsControl}        A new TItemsControlType
             */
            INTERNAL_ReturnNewTItemsControl: function () {
                return new Windows.UI.Xaml.Controls.ItemsControl();
            },
            /**
             * Confirms that the Content property is set to an object
             of type Panel.
             *
             * @instance
             * @protected
             * @override
             * @this Windows.UI.Xaml.Controls.ItemsControlDragDropTarget
             * @memberof Windows.UI.Xaml.Controls.ItemsControlDragDropTarget
             * @param   {System.Object}    oldContent    The old content value.
             * @param   {System.Object}    newContent    The new content value.
             * @return  {void}
             */
            OnContentChanged: function (oldContent, newContent) {
                if (newContent != null && !(Bridge.is(newContent, Windows.UI.Xaml.Controls.ItemsControl))) {
                    throw new System.ArgumentException.$ctor1("Content must be an ItemsControl.");
                }

                Windows.UI.Xaml.Controls.DragDropTarget$2(Windows.UI.Xaml.Controls.ItemsControl,Windows.UI.Xaml.UIElement).prototype.OnContentChanged.call(this, oldContent, newContent);
            },
            INTERNAL_GetNumberOfElementsBetweenItemsRootAndDragDropTarget: function () {
                return 4;
            }
        }
    });

    Bridge.define("Windows.UI.Xaml.Controls.PanelDragDropTarget", {
        inherits: [Windows.UI.Xaml.Controls.DragDropTarget$2(Windows.UI.Xaml.Controls.Panel,Windows.UI.Xaml.UIElement)],
        ctors: {
            /**
             * Initializes a new instance of the PanelDragDropTarget class.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Controls.PanelDragDropTarget
             * @memberof Windows.UI.Xaml.Controls.PanelDragDropTarget
             * @return  {void}
             */
            ctor: function () {
                this.$initialize();
                Windows.UI.Xaml.Controls.DragDropTarget$2(Windows.UI.Xaml.Controls.Panel,Windows.UI.Xaml.UIElement).ctor.call(this);

            }
        },
        methods: {
            /**
             * Adds an item to an items control.
             *
             * @instance
             * @protected
             * @override
             * @this Windows.UI.Xaml.Controls.PanelDragDropTarget
             * @memberof Windows.UI.Xaml.Controls.PanelDragDropTarget
             * @param   {Windows.UI.Xaml.Controls.Panel}    itemsControl    The items control.
             * @param   {System.Object}                     data            The data to be inserted.
             * @return  {void}
             */
            AddItem: function (itemsControl, data) {
                itemsControl.Children.add(Bridge.cast(data, Windows.UI.Xaml.UIElement));
            },
            /**
             * Retrieves the item container at a given index.
             *
             * @instance
             * @protected
             * @override
             * @this Windows.UI.Xaml.Controls.PanelDragDropTarget
             * @memberof Windows.UI.Xaml.Controls.PanelDragDropTarget
             * @param   {Windows.UI.Xaml.Controls.Panel}    itemsControl    The items control.
             * @param   {number}                            index           The index at which to retrieve the container.
             * @return  {Windows.UI.Xaml.UIElement}                         The item container at a given index.
             */
            ContainerFromIndex: function (itemsControl, index) {
                if (itemsControl.Children.Count > index) {
                    return itemsControl.Children.getItem(index);
                }
                return null;
            },
            /**
             * Inserts an item into an items control.
             *
             * @instance
             * @protected
             * @override
             * @this Windows.UI.Xaml.Controls.PanelDragDropTarget
             * @memberof Windows.UI.Xaml.Controls.PanelDragDropTarget
             * @param   {Windows.UI.Xaml.Controls.Panel}    itemsControl    The items control.
             * @param   {number}                            index           The index at which to insert the item.
             * @param   {System.Object}                     data            The data to be inserted.
             * @return  {void}
             */
            InsertItem: function (itemsControl, index, data) {
                itemsControl.Children.insert(index, Bridge.cast(data, Windows.UI.Xaml.UIElement));
            },
            /**
             * Retrieves the index of an item container.
             *
             * @instance
             * @protected
             * @override
             * @this Windows.UI.Xaml.Controls.PanelDragDropTarget
             * @memberof Windows.UI.Xaml.Controls.PanelDragDropTarget
             * @param   {Windows.UI.Xaml.Controls.Panel}    itemsControl     The items control.
             * @param   {Windows.UI.Xaml.UIElement}         itemContainer    The item container.
             * @return  {?number}                                            The index of an item container.
             */
            IndexFromContainer: function (itemsControl, itemContainer) {
                var index = itemsControl.Children.indexOf(itemContainer);
                return (index !== -1) ? index : null;
            },
            /**
             * Removes an item from an items control.
             *
             * @instance
             * @protected
             * @override
             * @this Windows.UI.Xaml.Controls.PanelDragDropTarget
             * @memberof Windows.UI.Xaml.Controls.PanelDragDropTarget
             * @param   {Windows.UI.Xaml.Controls.Panel}    itemsControl    The items control.
             * @param   {System.Object}                     data            The data to be removed.
             * @return  {void}
             */
            RemoveItem: function (itemsControl, data) {
                itemsControl.Children.remove(Bridge.cast(data, Windows.UI.Xaml.UIElement));
            },
            /**
             * Removes data from an ItemsControl.
             *
             * @instance
             * @protected
             * @override
             * @this Windows.UI.Xaml.Controls.PanelDragDropTarget
             * @memberof Windows.UI.Xaml.Controls.PanelDragDropTarget
             * @param   {Windows.UI.Xaml.Controls.Panel}    itemsControl    The items control.
             * @param   {number}                            index           The index at which to remove an item.
             * @return  {void}
             */
            RemoveItemAtIndex: function (itemsControl, index) {
                itemsControl.Children.removeAt(index);
            },
            /**
             * Confirms that the Content property is set to an object
             of type Panel.
             *
             * @instance
             * @protected
             * @override
             * @this Windows.UI.Xaml.Controls.PanelDragDropTarget
             * @memberof Windows.UI.Xaml.Controls.PanelDragDropTarget
             * @param   {System.Object}    oldContent    The old content value.
             * @param   {System.Object}    newContent    The new content value.
             * @return  {void}
             */
            OnContentChanged: function (oldContent, newContent) {
                if (newContent != null && !(Bridge.is(newContent, Windows.UI.Xaml.Controls.Panel))) {
                    throw new System.ArgumentException.$ctor1("Content must be a Panel.");
                }

                Windows.UI.Xaml.Controls.DragDropTarget$2(Windows.UI.Xaml.Controls.Panel,Windows.UI.Xaml.UIElement).prototype.OnContentChanged.call(this, oldContent, newContent);
            },
            /**
             * Create a new TItemsControlType
             *
             * @instance
             * @protected
             * @override
             * @this Windows.UI.Xaml.Controls.PanelDragDropTarget
             * @memberof Windows.UI.Xaml.Controls.PanelDragDropTarget
             * @return  {Windows.UI.Xaml.Controls.Panel}        A new TItemsControlType
             */
            INTERNAL_ReturnNewTItemsControl: function () {
                var $t;
                return ($t = new Windows.UI.Xaml.Controls.StackPanel(), $t.Orientation = Windows.UI.Xaml.Controls.Orientation.Horizontal, $t);
            },
            INTERNAL_GetNumberOfElementsBetweenItemsRootAndDragDropTarget: function () {
                return 2;
            }
        }
    });

    /**
     * Represents an individual System.Windows.Controls.DataGrid column header.
     *
     * @public
     * @class Windows.UI.Xaml.Controls.Primitives.DataGridColumnHeader
     * @augments Windows.UI.Xaml.Controls.Primitives.ButtonBase
     */
    Bridge.define("Windows.UI.Xaml.Controls.Primitives.DataGridColumnHeader", {
        inherits: [Windows.UI.Xaml.Controls.Primitives.ButtonBase],
        fields: {
            /**
             * Gets the System.Windows.Controls.DataGridColumn associated with this column
             header.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.Primitives.DataGridColumnHeader
             * @function Column
             * @type Windows.UI.Xaml.Controls.DataGridColumn
             */
            Column: null
        },
        ctors: {
            /**
             * Initializes a new instance of the System.Windows.Controls.Primitives.DataGridColumnHeader
             class.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Controls.Primitives.DataGridColumnHeader
             * @memberof Windows.UI.Xaml.Controls.Primitives.DataGridColumnHeader
             * @return  {void}
             */
            ctor: function () {
                this.$initialize();
                Windows.UI.Xaml.Controls.Primitives.ButtonBase.ctor.call(this);
                this.DefaultStyleKey = Windows.UI.Xaml.Controls.Primitives.DataGridColumnHeader;
                this.CanBeInheritanceContext = false;
                this.IsInheritanceContextSealed = true;
            }
        },
        methods: {
            CreateDomElement: function (parentRef, domElementWhereToPlaceChildren) {
                var div = Windows.UI.Xaml.Controls.Primitives.ButtonBase.prototype.CreateDomElement.call(this, parentRef, domElementWhereToPlaceChildren);
                var style = CSHTML5.Internal.INTERNAL_HtmlDomManager.GetDomElementStyleForModification(div);
                style.boxSizing = "border-box";
                return div;
            }
        }
    });

    /**
     * Represents a control that raises its Click event repeatedly from the time it is pressed until it is released.
     * // This is the method that responds to the KeyDown event.
    //
     * // This is the method that responds to the KeyUp event.
    //
     *
     * @public
     * @class Windows.UI.Xaml.Controls.Primitives.RepeatButton
     * @augments Windows.UI.Xaml.Controls.Primitives.ButtonBase
     */
    Bridge.define("Windows.UI.Xaml.Controls.Primitives.RepeatButton", {
        inherits: [Windows.UI.Xaml.Controls.Primitives.ButtonBase],
        statics: {
            fields: {
                /**
                 * Gets or sets the time, in milliseconds, the RepeatButton
                 waits when it is pressed before it starts repeating the click action.
                 The default is 250.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Controls.Primitives.RepeatButton
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                DelayProperty: null,
                /**
                 * Gets or sets the time, in milliseconds, between repetitions of the click
                 action, as soon as repeating starts. The default is 250.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Controls.Primitives.RepeatButton
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                IntervalProperty: null
            },
            ctors: {
                init: function () {
                    this.DelayProperty = Windows.UI.Xaml.DependencyProperty.Register("Delay", System.Int32, Windows.UI.Xaml.Controls.Primitives.RepeatButton, new Windows.UI.Xaml.PropertyMetadata.$ctor1(Bridge.box(250, System.Int32)));
                    this.IntervalProperty = Windows.UI.Xaml.DependencyProperty.Register("Interval", System.Int32, Windows.UI.Xaml.Controls.Primitives.RepeatButton, new Windows.UI.Xaml.PropertyMetadata.$ctor1(Bridge.box(250, System.Int32)));
                },
                ctor: function () {
                    Windows.UI.Xaml.Controls.Primitives.ButtonBase.ClickModeProperty.OverrideMetadata(Windows.UI.Xaml.Controls.Primitives.RepeatButton, new Windows.UI.Xaml.PropertyMetadata.$ctor1(Bridge.box(Windows.UI.Xaml.Controls.ClickMode.Press, Windows.UI.Xaml.Controls.ClickMode, System.Enum.toStringFn(Windows.UI.Xaml.Controls.ClickMode))));
                }
            }
        },
        fields: {
            _timer: null
        },
        props: {
            /**
             * Gets or sets the time, in milliseconds, the RepeatButton
             waits when it is pressed before it starts repeating the click action.
             The default is 250.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.Primitives.RepeatButton
             * @function Delay
             * @type number
             */
            Delay: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.Controls.Primitives.RepeatButton.DelayProperty), System.Int32), System.Int32));
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Controls.Primitives.RepeatButton.DelayProperty, Bridge.box(value, System.Int32));
                }
            },
            /**
             * Gets or sets the time, in milliseconds, between repetitions of the click
             action, as soon as repeating starts. The default is 250.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.Primitives.RepeatButton
             * @function Interval
             * @type number
             */
            Interval: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.Controls.Primitives.RepeatButton.IntervalProperty), System.Int32), System.Int32));
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Controls.Primitives.RepeatButton.IntervalProperty, Bridge.box(value, System.Int32));
                }
            }
        },
        ctors: {
            /**
             * Initializes a new instance of the RepeatButton class.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Controls.Primitives.RepeatButton
             * @memberof Windows.UI.Xaml.Controls.Primitives.RepeatButton
             * @return  {void}
             */
            ctor: function () {
                this.$initialize();
                Windows.UI.Xaml.Controls.Primitives.ButtonBase.ctor.call(this);
                this.DefaultStyleKey = Windows.UI.Xaml.Controls.Primitives.RepeatButton;

            }
        },
        methods: {
            StartTimer: function () {
                if (this._timer == null) {
                    this._timer = new Windows.UI.Xaml.DispatcherTimer();
                    this._timer.addTick(Bridge.fn.cacheBind(this, this.OnTimeout));
                } else if (this._timer.IsEnabled) {
                    return;
                }

                this._timer.Interval = System.TimeSpan.fromMilliseconds(this.Delay);
                this._timer.Start();
            },
            StopTimer: function () {
                if (this._timer != null) {
                    this._timer.Stop();
                }
            },
            OnTimeout: function (sender, e) {
                var interval = System.TimeSpan.fromMilliseconds(this.Interval);
                if (System.TimeSpan.neq(this._timer.Interval, interval)) {
                    this._timer.Interval = interval;
                }

                if (this.IsPressed) {
                    this.OnClick();
                }
            },
            /**
             * Raises InvokedAutomationEvent and call the base method to raise the Click event
             *
             * @instance
             * @protected
             * @override
             * @this Windows.UI.Xaml.Controls.Primitives.RepeatButton
             * @memberof Windows.UI.Xaml.Controls.Primitives.RepeatButton
             * @return  {void}
             */
            OnClick: function () {
                Windows.UI.Xaml.Controls.Primitives.ButtonBase.prototype.OnClick.call(this);
            },
            OnPointerPressed: function (eventArgs) {
                this.StartTimer();


                Windows.UI.Xaml.Controls.Primitives.ButtonBase.prototype.OnPointerPressed.call(this, eventArgs);
            },
            OnPointerReleased: function (eventArgs) {
                Windows.UI.Xaml.Controls.Primitives.ButtonBase.prototype.OnPointerReleased.call(this, eventArgs);

                this.StopTimer();
            },
            /**
             * Called when this element loses mouse capture.
             *
             * @instance
             * @protected
             * @override
             * @this Windows.UI.Xaml.Controls.Primitives.RepeatButton
             * @memberof Windows.UI.Xaml.Controls.Primitives.RepeatButton
             * @param   {Windows.UI.Xaml.Input.PointerRoutedEventArgs}    e
             * @return  {void}
             */
            OnPointerCaptureLost: function (e) {
                Windows.UI.Xaml.Controls.Primitives.ButtonBase.prototype.OnPointerCaptureLost.call(this, e);
                this.StopTimer();
            }
        }
    });

    /**
     * @public
     * @class Windows.UI.Xaml.Controls.UIElementCollection
     * @augments System.Collections.ObjectModel.ObservableCollection$1
     */
    Bridge.define("Windows.UI.Xaml.Controls.UIElementCollection", {
        inherits: [System.Collections.ObjectModel.ObservableCollection$1(Windows.UI.Xaml.UIElement)],
        ctors: {
            ctor: function () {
                this.$initialize();
                System.Collections.ObjectModel.ObservableCollection$1(Windows.UI.Xaml.UIElement).ctor.call(this);

            }
        }
    });

    /**
     * Groups other Inline flow content elements.
     *
     * @public
     * @class Windows.UI.Xaml.Documents.Span
     * @augments Windows.UI.Xaml.Documents.Inline
     */
    Bridge.define("Windows.UI.Xaml.Documents.Span", {
        inherits: [Windows.UI.Xaml.Documents.Inline],
        fields: {
            /**
             * Gets an InlineCollection containing the top-level inline elements that include the contents of Span.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Documents.Span
             * @function Inlines
             * @type Windows.UI.Xaml.Documents.InlineCollection
             */
            Inlines: null
        },
        ctors: {
            /**
             * Initializes a new instance of the Span class.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Documents.Span
             * @memberof Windows.UI.Xaml.Documents.Span
             * @return  {void}
             */
            ctor: function () {
                this.$initialize();
                Windows.UI.Xaml.Documents.Inline.ctor.call(this);
                this.Inlines = new Windows.UI.Xaml.Documents.InlineCollection(this);
            }
        },
        methods: {
            CreateDomElement: function (parentRef, domElementWhereToPlaceChildren) {
                var span = CSHTML5.Internal.INTERNAL_HtmlDomManager.CreateDomElementAndAppendIt("span", parentRef, this);
                domElementWhereToPlaceChildren.v = span;
                return span;
            },
            INTERNAL_OnAttachedToVisualTree: function () {
                var $t;
                Windows.UI.Xaml.Documents.Inline.prototype.INTERNAL_OnAttachedToVisualTree.call(this);

                $t = Bridge.getEnumerator(this.Inlines);
                try {
                    while ($t.moveNext()) {
                        var child = $t.Current;
                        CSHTML5.Internal.INTERNAL_VisualTreeManager.AttachVisualChildIfNotAlreadyAttached(child, this);
                    }
                } finally {
                    if (Bridge.is($t, System.IDisposable)) {
                        $t.System$IDisposable$Dispose();
                    }
                }
            }
        }
    });

    Bridge.define("Windows.UI.Xaml.Documents.InlineCollection", {
        inherits: [Windows.UI.Xaml.Documents.TextElementCollection$1(Windows.UI.Xaml.Documents.Inline),System.Collections.IList],
        props: {
            FirstElement: {
                get: function () {
                    if (this.Count === 0) {
                        return null;
                    }
                    return Bridge.cast(this.getItem(0), Windows.UI.Xaml.Documents.Inline);
                }
            }
        },
        alias: [
            "System$Collections$IEnumerable$GetEnumerator", "System$Collections$IEnumerable$GetEnumerator",
            "System$Collections$ICollection$copyTo", "System$Collections$ICollection$copyTo",
            "Count", "System$Collections$ICollection$Count",
            "System$Collections$ICollection$SyncRoot", "System$Collections$ICollection$SyncRoot",
            "System$Collections$ICollection$IsSynchronized", "System$Collections$ICollection$IsSynchronized",
            "System$Collections$IList$add", "System$Collections$IList$add",
            "clear", "System$Collections$IList$clear",
            "System$Collections$IList$contains", "System$Collections$IList$contains",
            "System$Collections$IList$indexOf", "System$Collections$IList$indexOf",
            "System$Collections$IList$insert", "System$Collections$IList$insert",
            "System$Collections$IList$remove", "System$Collections$IList$remove",
            "System$Collections$IList$removeAt", "System$Collections$IList$removeAt",
            "getItem", "System$Collections$IList$getItem",
            "setItem", "System$Collections$IList$setItem",
            "IsReadOnly", "System$Collections$IList$IsReadOnly",
            "System$Collections$IList$IsFixedSize", "System$Collections$IList$IsFixedSize"
        ],
        ctors: {
            ctor: function (owner) {
                this.$initialize();
                Windows.UI.Xaml.Documents.TextElementCollection$1(Windows.UI.Xaml.Documents.Inline).ctor.call(this, owner);

            }
        },
        methods: {
            Add: function (text) {
                this.AddInternal$1(text);
            },
            AddInternal$1: function (text) {
                var $t;
                if (text == null) {
                    throw new System.ArgumentNullException.$ctor1("text");
                }
                var run = ($t = new Windows.UI.Xaml.Documents.Run(), $t.Text = text, $t);
                this.add(run);
            }
        }
    });

    /**
     * Represents an inline element that causes a new line to begin in content when rendered in a text container.
     *
     * @public
     * @class Windows.UI.Xaml.Documents.LineBreak
     * @augments Windows.UI.Xaml.Documents.Inline
     */
    Bridge.define("Windows.UI.Xaml.Documents.LineBreak", {
        inherits: [Windows.UI.Xaml.Documents.Inline],
        methods: {
            CreateDomElement: function (parentRef, domElementWhereToPlaceChildren) {
                var linebreak = CSHTML5.Internal.INTERNAL_HtmlDomManager.CreateDomElementAndAppendIt("br", parentRef, this);
                domElementWhereToPlaceChildren.v = linebreak;
                return linebreak;
            }
        }
    });

    /**
     * Represents a discrete section of formatted or unformatted text.
     *
     * @public
     * @class Windows.UI.Xaml.Documents.Run
     * @augments Windows.UI.Xaml.Documents.Inline
     */
    Bridge.define("Windows.UI.Xaml.Documents.Run", {
        inherits: [Windows.UI.Xaml.Documents.Inline],
        statics: {
            fields: {
                defaultText: null,
                /**
                 * Identifies the Text dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Documents.Run
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                TextProperty: null
            },
            ctors: {
                init: function () {
                    this.defaultText = "\u00a0";
                    this.TextProperty = Windows.UI.Xaml.DependencyProperty.Register("Text", System.String, Windows.UI.Xaml.Documents.Run, new Windows.UI.Xaml.PropertyMetadata.$ctor3("", Windows.UI.Xaml.Documents.Run.OnTextPropertyChanged, Windows.UI.Xaml.Documents.Run.CoerceTextProperty));
                }
            },
            methods: {
                OnTextPropertyChanged: function (d, e) {
                    var run = Bridge.cast(d, Windows.UI.Xaml.Documents.Run);
                    if (CSHTML5.Internal.INTERNAL_VisualTreeManager.IsElementInVisualTree(run)) {
                        CSHTML5.Internal.INTERNAL_HtmlDomManager.SetContentString(run, Bridge.cast(e.NewValue, System.String));
                    }
                },
                CoerceTextProperty: function (d, baseValue) {
                    if (System.String.isNullOrEmpty(Bridge.cast(baseValue, System.String))) {
                        return Windows.UI.Xaml.Documents.Run.defaultText;
                    }
                    return baseValue;
                }
            }
        },
        props: {
            /**
             * Get or Set the Text property
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Documents.Run
             * @function Text
             * @type string
             */
            Text: {
                get: function () {
                    return Bridge.cast(this.GetValue(Windows.UI.Xaml.Documents.Run.TextProperty), System.String);
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Documents.Run.TextProperty, value);
                }
            }
        },
        methods: {
            CreateDomElement: function (parentRef, domElementWhereToPlaceChildren) {
                var span = CSHTML5.Internal.INTERNAL_HtmlDomManager.CreateDomElementAndAppendIt("span", parentRef, this);
                domElementWhereToPlaceChildren.v = span;
                return span;
            },
            INTERNAL_OnAttachedToVisualTree: function () {
                Windows.UI.Xaml.Documents.Inline.prototype.INTERNAL_OnAttachedToVisualTree.call(this);

                CSHTML5.Internal.INTERNAL_HtmlDomManager.SetContentString(this, Bridge.cast(Windows.UI.Xaml.Documents.Run.CoerceTextProperty(this, this.Text), System.String));
            }
        }
    });

    /**
     * Represents a control that a user can select (check) or clear (uncheck). A
     CheckBox can also report its value as indeterminate.
     *
     * @public
     * @class Windows.UI.Xaml.Controls.CheckBox
     * @augments Windows.UI.Xaml.Controls.Primitives.ToggleButton
     * @example
     *
     *
     * void CheckBox_Checked(object sender, RoutedEventArgs e)
     * {
     *     MessageBox.Show("You checked me.");
     * }
     * void CheckBox_Unchecked(object sender, RoutedEventArgs e)
     * {
     *     MessageBox.Show("You unchecked me.");
     * }
     * 
     *
     *
     */
    Bridge.define("Windows.UI.Xaml.Controls.CheckBox", {
        inherits: [Windows.UI.Xaml.Controls.Primitives.ToggleButton],
        props: {
            INTERNAL_GetFocusInBrowser: {
                get: function () {
                    return true;
                }
            }
        },
        ctors: {
            /**
             * Initializes a new instance of the CheckBox class.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Controls.CheckBox
             * @memberof Windows.UI.Xaml.Controls.CheckBox
             * @return  {void}
             */
            ctor: function () {
                this.$initialize();
                Windows.UI.Xaml.Controls.Primitives.ToggleButton.ctor.call(this);
                if (!CSHTML5.Interop.IsRunningInTheSimulator) {
                    this._reactsToKeyboardEventsWhenFocused = false;
                }
            }
        },
        methods: {
            OnKeyDownWhenFocused: function (sender, e) {
                if ((e.Key === Windows.System.VirtualKey.Space)) {
                    this.ToggleButton_Click(this, null);
                }
            },
            SetDefaultStyle: function () { },
            CreateDomElement: function (parentRef, domElementWhereToPlaceChildren) {
                return Windows.UI.Xaml.Controls.INTERNAL_CheckBoxAndRadioButtonHelpers.CreateDomElement(this, "checkbox", parentRef, domElementWhereToPlaceChildren);
            },
            UpdateDomBasedOnCheckedState: function (isChecked) {
                Windows.UI.Xaml.Controls.INTERNAL_CheckBoxAndRadioButtonHelpers.UpdateDomBasedOnCheckedState(this, isChecked);
            },
            SubscribeToClickEventForChildContainerDiv: function (divWhereToPlaceChild, checkBoxDomElement) {
                if (true) {

                    divWhereToPlaceChild.addEventListener('click', function(e) {
    if(checkBoxDomElement.checked === true)
    {
        checkBoxDomElement.checked = false;
    }
    else
    {
        checkBoxDomElement.checked = true;
    }
    var evt = document.createEvent('Event');
    evt.initEvent('change', false, false);
    checkBoxDomElement.dispatchEvent(evt);
}, false);

                }
            }
        }
    });

    /**
     * Represents the container for an item in a ListBox control.
     *
     * @public
     * @class Windows.UI.Xaml.Controls.ComboBoxItem
     * @augments Windows.UI.Xaml.Controls.Primitives.SelectorItem
     */
    Bridge.define("Windows.UI.Xaml.Controls.ComboBoxItem", {
        inherits: [Windows.UI.Xaml.Controls.Primitives.SelectorItem]
    });

    /**
     * Represents a control that displays data in a customizable grid.
     *
     * @public
     * @class Windows.UI.Xaml.Controls.DataGrid
     * @augments Windows.UI.Xaml.Controls.Primitives.MultiSelector
     * @example
     *
     *
     * DataGrid1.ItemsSource = Planet.GetListOfPlanets();
     * public partial class Planet
     * {
     *    public string Name { get; set; }
     *    public string ImagePath { get; set; }
     *    public static ObservableCollection<Planet> GetListOfPlanets()
     *    {
     *        return new ObservableCollection<Planet>()
     *        {
     *            new Planet() { Name = "Mercury", ImagePath = "ms-appx:/Planets/Mercury.png" },
     *            new Planet() { Name = "Venus", ImagePath = "ms-appx:/Planets/Venus.png" },
     *            new Planet() { Name = "Earth", ImagePath = "ms-appx:/Planets/Earth.png" },
     *            new Planet() { Name = "Mars", ImagePath = "ms-appx:/Planets/Mars.png" },
     *            new Planet() { Name = "Jupiter", ImagePath = "ms-appx:/Planets/Jupiter.png" },
     *            new Planet() { Name = "Saturn", ImagePath = "ms-appx:/Planets/Saturn.png" },
     *            new Planet() { Name = "Uranus", ImagePath = "ms-appx:/Planets/Uranus.png" },
     *            new Planet() { Name = "Neptune", ImagePath = "ms-appx:/Planets/Neptune.png" }
     *            new Planet() { Name = "Pluto", ImagePath = "ms-appx:/Planets/Pluto.png" }
     *        };
     *    }
     *} 
     * 
     *
     *
     */
    Bridge.define("Windows.UI.Xaml.Controls.DataGrid", {
        inherits: [Windows.UI.Xaml.Controls.Primitives.MultiSelector],
        statics: {
            fields: {
                /**
                 * Identifies the AlternatingRowBackground dependency property
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Controls.DataGrid
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                AlternatingRowBackgroundProperty: null,
                /**
                 * Identifies the VerticalCellPadding dependency property
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Controls.DataGrid
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                VerticalCellPaddingProperty: null,
                /**
                 * Identifies the System.Windows.Controls.DataGrid.AutoGenerateColumns dependency
                 property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Controls.DataGrid
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                AutoGenerateColumnsProperty: null,
                /**
                 * Identifies the System.Windows.Controls.DataGrid.ColumnHeaderStyle dependency
                 property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Controls.DataGrid
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                ColumnHeaderStyleProperty: null,
                ColumnHeaderHeightProperty: null,
                /**
                 * Identifies the System.Windows.Controls.DataGrid.CellStyle dependency
                 property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Controls.DataGrid
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                CellStyleProperty: null,
                /**
                 * Identifies the System.Windows.Controls.DataGrid.Columns dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Controls.DataGrid
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                ColumnsProperty: null,
                /**
                 * Identifies the System.Windows.Controls.DataGrid.IsReadOnly dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Controls.DataGrid
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                IsReadOnlyProperty: null,
                EnableTwoStepsEditModeProperty: null,
                /**
                 * Identifies the System.Windows.Controls.DataGrid.SelectionMode dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Controls.DataGrid
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                SelectionModeProperty: null,
                /**
                 * Identifies the System.Windows.Controls.DataGrid.HorizontalGridLinesBrush dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Controls.DataGrid
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                HorizontalGridLinesBrushProperty: null,
                /**
                 * Identifies the System.Windows.Controls.DataGrid.RowHeaderTemplate dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Controls.DataGrid
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                RowHeaderTemplateProperty: null,
                /**
                 * Identifies the System.Windows.Controls.DataGrid.VerticalGridLinesBrush dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Controls.DataGrid
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                VerticalGridLinesBrushProperty: null,
                _itemsPanelTemplate: null
            },
            props: {
                ItemsPanelInternal: {
                    get: function () {
                        var $t;
                        if (Windows.UI.Xaml.Controls.DataGrid._itemsPanelTemplate == null) {
                            Windows.UI.Xaml.Controls.DataGrid._itemsPanelTemplate = ($t = new Windows.UI.Xaml.Controls.ItemsPanelTemplate(), $t._methodToInstantiateFrameworkTemplate = function (templateOwner) {
                                var $t1;
                                var grid = new Windows.UI.Xaml.Controls.Grid();

                                var row1 = new Windows.UI.Xaml.Controls.RowDefinition();
                                row1.Height = new Windows.UI.Xaml.GridLength.$ctor2(1.0, Windows.UI.Xaml.GridUnitType.Star);

                                var row2 = new Windows.UI.Xaml.Controls.RowDefinition();
                                row2.Height = new Windows.UI.Xaml.GridLength.$ctor2(1.0, Windows.UI.Xaml.GridUnitType.Auto);

                                grid.RowDefinitions.add(row1);
                                grid.RowDefinitions.add(row2);

                                return ($t1 = new Windows.UI.Xaml.TemplateInstance(), $t1.TemplateContent = grid, $t1);
                            }, $t);

                            Windows.UI.Xaml.Controls.DataGrid._itemsPanelTemplate.Seal();
                        }
                        return Windows.UI.Xaml.Controls.DataGrid._itemsPanelTemplate;
                    }
                }
            },
            ctors: {
                init: function () {
                    var $t;
                    this.AlternatingRowBackgroundProperty = Windows.UI.Xaml.DependencyProperty.Register("AlternatingRowBackground", Windows.UI.Xaml.Media.Brush, Windows.UI.Xaml.Controls.DataGrid, new Windows.UI.Xaml.PropertyMetadata.$ctor1(new Windows.UI.Xaml.Media.SolidColorBrush.$ctor1(System.Nullable.getValue(Bridge.cast(Bridge.unbox(Windows.UI.Color.INTERNAL_ConvertFromString("#f0f0e9"), Windows.UI.Color), Windows.UI.Color)))));
                    this.VerticalCellPaddingProperty = Windows.UI.Xaml.DependencyProperty.Register("VerticalCellPadding", System.Double, Windows.UI.Xaml.Controls.DataGrid, new Windows.UI.Xaml.PropertyMetadata.$ctor1(Bridge.box(3.0, System.Double, System.Double.format, System.Double.getHashCode)));
                    this.AutoGenerateColumnsProperty = Windows.UI.Xaml.DependencyProperty.Register("AutoGenerateColumns", System.Boolean, Windows.UI.Xaml.Controls.DataGrid, new Windows.UI.Xaml.PropertyMetadata.$ctor1(Bridge.box(true, System.Boolean, System.Boolean.toString)));
                    this.ColumnHeaderStyleProperty = Windows.UI.Xaml.DependencyProperty.Register("ColumnHeaderStyle", Windows.UI.Xaml.Style, Windows.UI.Xaml.Controls.DataGrid, ($t = new Windows.UI.Xaml.PropertyMetadata.$ctor2(null, Windows.UI.Xaml.Controls.DataGrid.ColumnHeaderStyle_Changed), $t.CallPropertyChangedWhenLoadedIntoVisualTree = Windows.UI.Xaml.WhenToCallPropertyChangedEnum.IfPropertyIsSet, $t));
                    this.ColumnHeaderHeightProperty = Windows.UI.Xaml.DependencyProperty.Register("ColumnHeaderHeight", System.Double, Windows.UI.Xaml.Controls.DataGrid, new Windows.UI.Xaml.PropertyMetadata.$ctor1(Bridge.box(Number.NaN, System.Double, System.Double.format, System.Double.getHashCode)));
                    this.CellStyleProperty = Windows.UI.Xaml.DependencyProperty.Register("CellStyle", Windows.UI.Xaml.Style, Windows.UI.Xaml.Controls.DataGrid, ($t = new Windows.UI.Xaml.PropertyMetadata.$ctor2(null, Windows.UI.Xaml.Controls.DataGrid.CellStyle_Changed), $t.CallPropertyChangedWhenLoadedIntoVisualTree = Windows.UI.Xaml.WhenToCallPropertyChangedEnum.IfPropertyIsSet, $t));
                    this.ColumnsProperty = Windows.UI.Xaml.DependencyProperty.Register("Columns", System.Collections.ObjectModel.ObservableCollection$1(Windows.UI.Xaml.Controls.DataGridColumn), Windows.UI.Xaml.Controls.DataGrid, ($t = new Windows.UI.Xaml.PropertyMetadata.$ctor2(null, Windows.UI.Xaml.Controls.DataGrid.Columns_Changed), $t.CallPropertyChangedWhenLoadedIntoVisualTree = Windows.UI.Xaml.WhenToCallPropertyChangedEnum.IfPropertyIsSet, $t));
                    this.IsReadOnlyProperty = Windows.UI.Xaml.DependencyProperty.Register("IsReadOnly", System.Boolean, Windows.UI.Xaml.Controls.DataGrid, ($t = new Windows.UI.Xaml.PropertyMetadata.$ctor2(Bridge.box(false, System.Boolean, System.Boolean.toString), Windows.UI.Xaml.Controls.DataGrid.IsReadonly_Changed), $t.CallPropertyChangedWhenLoadedIntoVisualTree = Windows.UI.Xaml.WhenToCallPropertyChangedEnum.IfPropertyIsSet, $t));
                    this.EnableTwoStepsEditModeProperty = Windows.UI.Xaml.DependencyProperty.Register("EnableTwoStepsEditMode", System.Boolean, Windows.UI.Xaml.Controls.DataGrid, new Windows.UI.Xaml.PropertyMetadata.$ctor1(Bridge.box(false, System.Boolean, System.Boolean.toString)));
                    this.SelectionModeProperty = Windows.UI.Xaml.DependencyProperty.Register("SelectionMode", Windows.UI.Xaml.Controls.DataGridSelectionMode, Windows.UI.Xaml.Controls.DataGrid, ($t = new Windows.UI.Xaml.PropertyMetadata.$ctor2(Bridge.box(Windows.UI.Xaml.Controls.DataGridSelectionMode.Single, Windows.UI.Xaml.Controls.DataGridSelectionMode, System.Enum.toStringFn(Windows.UI.Xaml.Controls.DataGridSelectionMode)), Windows.UI.Xaml.Controls.DataGrid.SelectionMode_Changed), $t.CallPropertyChangedWhenLoadedIntoVisualTree = Windows.UI.Xaml.WhenToCallPropertyChangedEnum.IfPropertyIsSet, $t));
                    this.HorizontalGridLinesBrushProperty = Windows.UI.Xaml.DependencyProperty.Register("HorizontalGridLinesBrush", Windows.UI.Xaml.Media.Brush, Windows.UI.Xaml.Controls.DataGrid, ($t = new Windows.UI.Xaml.PropertyMetadata.$ctor2(new Windows.UI.Xaml.Media.SolidColorBrush.$ctor1(Windows.UI.Colors.Gray.$clone()), Windows.UI.Xaml.Controls.DataGrid.HorizontalGridLinesBrush_Changed), $t.CallPropertyChangedWhenLoadedIntoVisualTree = Windows.UI.Xaml.WhenToCallPropertyChangedEnum.IfPropertyIsSet, $t));
                    this.RowHeaderTemplateProperty = Windows.UI.Xaml.DependencyProperty.Register("RowHeaderTemplate", Windows.UI.Xaml.DataTemplate, Windows.UI.Xaml.Controls.DataGrid, ($t = new Windows.UI.Xaml.PropertyMetadata.$ctor2(null, Windows.UI.Xaml.Controls.DataGrid.RowHeaderTemplate_Changed), $t.CallPropertyChangedWhenLoadedIntoVisualTree = Windows.UI.Xaml.WhenToCallPropertyChangedEnum.IfPropertyIsSet, $t));
                    this.VerticalGridLinesBrushProperty = Windows.UI.Xaml.DependencyProperty.Register("VerticalGridLinesBrush", Windows.UI.Xaml.Media.Brush, Windows.UI.Xaml.Controls.DataGrid, ($t = new Windows.UI.Xaml.PropertyMetadata.$ctor2(new Windows.UI.Xaml.Media.SolidColorBrush.$ctor1(Windows.UI.Colors.Gray.$clone()), Windows.UI.Xaml.Controls.DataGrid.VerticalGridLinesBrush_Changed), $t.CallPropertyChangedWhenLoadedIntoVisualTree = Windows.UI.Xaml.WhenToCallPropertyChangedEnum.IfPropertyIsSet, $t));
                }
            },
            methods: {
                ColumnHeaderStyle_Changed: function (d, e) {
                    var $t;
                    if (Bridge.is(d, Windows.UI.Xaml.Controls.DataGrid)) {
                        var dataGrid = Bridge.cast(d, Windows.UI.Xaml.Controls.DataGrid);
                        if (CSHTML5.Internal.INTERNAL_VisualTreeManager.IsElementInVisualTree(dataGrid)) {
                            $t = Bridge.getEnumerator(dataGrid.Columns);
                            try {
                                while ($t.moveNext()) {
                                    var column = $t.Current;
                                    column.SetHeaderStyleIfColumnsStyleNotSet(Bridge.cast(e.NewValue, Windows.UI.Xaml.Style));
                                }
                            } finally {
                                if (Bridge.is($t, System.IDisposable)) {
                                    $t.System$IDisposable$Dispose();
                                }
                            }
                        }
                    }
                },
                CellStyle_Changed: function (d, e) {
                    var $t;
                    if (Bridge.is(d, Windows.UI.Xaml.Controls.DataGrid)) {
                        var dataGrid = Bridge.cast(d, Windows.UI.Xaml.Controls.DataGrid);
                        if (CSHTML5.Internal.INTERNAL_VisualTreeManager.IsElementInVisualTree(dataGrid)) {
                            $t = Bridge.getEnumerator(dataGrid._grid.Children);
                            try {
                                while ($t.moveNext()) {
                                    var cell = Bridge.cast($t.Current, Windows.UI.Xaml.FrameworkElement);
                                    if (Bridge.is(cell, Windows.UI.Xaml.Controls.DataGridCell)) {
                                        var cellAsDataGridCell = Bridge.cast(cell, Windows.UI.Xaml.Controls.DataGridCell);
                                        var column = cellAsDataGridCell.Column;
                                        if (column.CellStyle != null) {
                                            cellAsDataGridCell.Style = column.CellStyle;
                                        } else if (dataGrid.CellStyle != null) {
                                            cellAsDataGridCell.Style = dataGrid.CellStyle;
                                        }
                                    }
                                }
                            } finally {
                                if (Bridge.is($t, System.IDisposable)) {
                                    $t.System$IDisposable$Dispose();
                                }
                            }
                        }
                    }
                },
                Columns_Changed: function (d, e) { },
                IsReadonly_Changed: function (d, e) { },
                SelectionMode_Changed: function (d, e) {
                    var $t;
                    if (!Bridge.referenceEquals(e.NewValue, e.OldValue)) {
                        var dataGrid = Bridge.cast(d, Windows.UI.Xaml.Controls.DataGrid);
                        if (CSHTML5.Internal.INTERNAL_VisualTreeManager.IsElementInVisualTree(dataGrid)) {
                            $t = Bridge.getEnumerator(INTERNAL_BridgeWorkarounds.GetDictionaryValues_SimulatorCompatible(System.Object, Windows.UI.Xaml.Controls.DataGridRow, dataGrid._objectsToDisplay), Windows.UI.Xaml.Controls.DataGridRow);
                            try {
                                while ($t.moveNext()) {
                                    var dataGridRow = $t.Current;
                                    dataGridRow.ChangeIntoSelectionMode(System.Nullable.getValue(Bridge.cast(Bridge.unbox((e.NewValue), Windows.UI.Xaml.Controls.DataGridSelectionMode), System.Int32)));
                                }
                            } finally {
                                if (Bridge.is($t, System.IDisposable)) {
                                    $t.System$IDisposable$Dispose();
                                }
                            }
                            dataGrid.UpdateChildrenInVisualTree(dataGrid.Items, dataGrid.Items, true);
                            dataGrid.UpdateUIStructure();
                        }
                    }
                },
                HorizontalGridLinesBrush_Changed: function (sender, e) {
                    var dataGrid = Bridge.cast(sender, Windows.UI.Xaml.Controls.DataGrid);
                    if (CSHTML5.Internal.INTERNAL_VisualTreeManager.IsElementInVisualTree(dataGrid)) {
                        dataGrid._grid.INTERNAL_StringToSetHorizontalGridLinesInCss = dataGrid.GenerateStringForGridLines(Bridge.cast(e.NewValue, Windows.UI.Xaml.Media.Brush));
                        Windows.UI.Xaml.Controls.Grid_InternalHelpers.RefreshGridLines(dataGrid._grid, dataGrid._grid._currentCellsStructure, dataGrid._grid.ColumnDefinitions, dataGrid._grid.RowDefinitions);
                    }
                },
                RowHeaderTemplate_Changed: function (d, e) {
                    var $t;
                    var dataGrid = Bridge.cast(d, Windows.UI.Xaml.Controls.DataGrid);

                    var newValue = dataGrid.GetActiveRowHeaderTemplate();

                    $t = Bridge.getEnumerator(INTERNAL_BridgeWorkarounds.GetDictionaryValues_SimulatorCompatible(System.Object, Windows.UI.Xaml.Controls.DataGridRow, dataGrid._objectsToDisplay), Windows.UI.Xaml.Controls.DataGridRow);
                    try {
                        while ($t.moveNext()) {
                            var row = $t.Current;
                            row.HeaderTemplate = newValue;
                        }
                    } finally {
                        if (Bridge.is($t, System.IDisposable)) {
                            $t.System$IDisposable$Dispose();
                        }
                    }
                },
                VerticalGridLinesBrush_Changed: function (sender, e) {
                    var dataGrid = Bridge.cast(sender, Windows.UI.Xaml.Controls.DataGrid);
                    if (CSHTML5.Internal.INTERNAL_VisualTreeManager.IsElementInVisualTree(dataGrid)) {
                        dataGrid._grid.INTERNAL_StringToSetVerticalGridLinesInCss = dataGrid.GenerateStringForGridLines(Bridge.cast(e.NewValue, Windows.UI.Xaml.Media.Brush));
                        Windows.UI.Xaml.Controls.Grid_InternalHelpers.RefreshGridLines(dataGrid._grid, dataGrid._grid._currentCellsStructure, dataGrid._grid.ColumnDefinitions, dataGrid._grid.RowDefinitions);
                    }
                }
            }
        },
        fields: {
            _objectsToDisplay: null,
            _grid: null,
            _areItemsInVisualTree: false,
            _currentEditionElement: null,
            _currentCell: null,
            _pagedView: null,
            _pagerUI: null,
            _oldOldValue: null,
            AutoGeneratedColumns: null
        },
        events: {
            /**
             * Event that is fired just before a row is prepared.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Controls.DataGrid
             * @memberof Windows.UI.Xaml.Controls.DataGrid
             * @function addLoadingRow
             * @param   {System.EventHandler}    value
             * @return  {void}
             */
            /**
             * Event that is fired just before a row is prepared.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Controls.DataGrid
             * @memberof Windows.UI.Xaml.Controls.DataGrid
             * @function removeLoadingRow
             * @param   {System.EventHandler}    value
             * @return  {void}
             */
            LoadingRow: null,
            /**
             * Event that is fired just before a row is cleared.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Controls.DataGrid
             * @memberof Windows.UI.Xaml.Controls.DataGrid
             * @function addUnloadingRow
             * @param   {System.EventHandler}    value
             * @return  {void}
             */
            /**
             * Event that is fired just before a row is cleared.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Controls.DataGrid
             * @memberof Windows.UI.Xaml.Controls.DataGrid
             * @function removeUnloadingRow
             * @param   {System.EventHandler}    value
             * @return  {void}
             */
            UnloadingRow: null
        },
        props: {
            UnselectedItemAlternateBackground: {
                get: function () {
                    return this.AlternatingRowBackground;
                },
                set: function (value) {
                    this.AlternatingRowBackground = value;
                }
            },
            /**
             * Gets or sets the alternative bakground color of the Items that are not selected.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.DataGrid
             * @function AlternatingRowBackground
             * @type Windows.UI.Xaml.Media.Brush
             */
            AlternatingRowBackground: {
                get: function () {
                    return Bridge.cast(this.GetValue(Windows.UI.Xaml.Controls.DataGrid.AlternatingRowBackgroundProperty), Windows.UI.Xaml.Media.Brush);
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Controls.DataGrid.AlternatingRowBackgroundProperty, value);
                }
            },
            /**
             * Gets or sets the vertical padding in all cells
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.DataGrid
             * @function VerticalCellPadding
             * @type number
             */
            VerticalCellPadding: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.Controls.DataGrid.VerticalCellPaddingProperty), System.Double), System.Double));
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Controls.DataGrid.VerticalCellPaddingProperty, Bridge.box(value, System.Double, System.Double.format, System.Double.getHashCode));
                }
            },
            /**
             * Gets or sets a value that indicates whether the columns are created automatically.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.DataGrid
             * @function AutoGenerateColumns
             * @type boolean
             */
            AutoGenerateColumns: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.Controls.DataGrid.AutoGenerateColumnsProperty), System.Boolean), System.Boolean));
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Controls.DataGrid.AutoGenerateColumnsProperty, Bridge.box(value, System.Boolean, System.Boolean.toString));
                }
            },
            /**
             * Gets or sets the style applied to all column headers in the System.Windows.Controls.DataGrid.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.DataGrid
             * @function ColumnHeaderStyle
             * @type Windows.UI.Xaml.Style
             */
            ColumnHeaderStyle: {
                get: function () {
                    return Bridge.cast(this.GetValue(Windows.UI.Xaml.Controls.DataGrid.ColumnHeaderStyleProperty), Windows.UI.Xaml.Style);
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Controls.DataGrid.ColumnHeaderStyleProperty, value);
                }
            },
            ColumnHeaderHeight: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.Controls.DataGrid.ColumnHeaderHeightProperty), System.Double), System.Double));
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Controls.DataGrid.ColumnHeaderHeightProperty, Bridge.box(value, System.Double, System.Double.format, System.Double.getHashCode));
                }
            },
            /**
             * Gets or sets the style applied to all cells in the System.Windows.Controls.DataGrid.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.DataGrid
             * @function CellStyle
             * @type Windows.UI.Xaml.Style
             */
            CellStyle: {
                get: function () {
                    return Bridge.cast(this.GetValue(Windows.UI.Xaml.Controls.DataGrid.CellStyleProperty), Windows.UI.Xaml.Style);
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Controls.DataGrid.CellStyleProperty, value);
                }
            },
            /**
             * Gets a collection that contains all the columns in the System.Windows.Controls.DataGrid.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.DataGrid
             * @function Columns
             * @type System.Collections.ObjectModel.ObservableCollection$1
             */
            Columns: {
                get: function () {
                    return Bridge.cast(this.GetValue(Windows.UI.Xaml.Controls.DataGrid.ColumnsProperty), System.Collections.ObjectModel.ObservableCollection$1(Windows.UI.Xaml.Controls.DataGridColumn));
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Controls.DataGrid.ColumnsProperty, value);
                }
            },
            /**
             * Gets or sets the maximum number of items to display on a page.
             -1 means infinity. Default is 20.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.DataGrid
             * @function PageSize
             * @type number
             */
            PageSize: {
                get: function () {
                    if (this._pagerUI != null) {
                        return this._pagerUI.PageSize;
                    } else {
                        return -1;
                    }
                },
                set: function (value) {
                    if (this._pagerUI != null) {
                        this._pagerUI.PageSize = value;
                    }
                }
            },
            /**
             * Gets or sets a value that indicates whether the user can edit values in the
             System.Windows.Controls.DataGrid.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.DataGrid
             * @function IsReadOnly
             * @type boolean
             */
            IsReadOnly: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.Controls.DataGrid.IsReadOnlyProperty), System.Boolean), System.Boolean));
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Controls.DataGrid.IsReadOnlyProperty, Bridge.box(value, System.Boolean, System.Boolean.toString));
                }
            },
            /**
             * Gets or sets a value that indicates whether entering Edit Mode requires two steps
             (like in Silverlight) rather than one. 'True' means that the user must first select
             a cell and then click again to enter Edit Mode. 'False' means that the DataGrid
             enters Edit Mode immediately when a cell is clicked, even if the cell is not
             selected yet (it will get selected too). Default is false.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.DataGrid
             * @function EnableTwoStepsEditMode
             * @type boolean
             */
            EnableTwoStepsEditMode: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.Controls.DataGrid.EnableTwoStepsEditModeProperty), System.Boolean), System.Boolean));
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Controls.DataGrid.EnableTwoStepsEditModeProperty, Bridge.box(value, System.Boolean, System.Boolean.toString));
                }
            },
            /**
             * Gets or sets a value that indicates how rows and cells are selected in the
             System.Windows.Controls.DataGrid.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.DataGrid
             * @function SelectionMode
             * @type Windows.UI.Xaml.Controls.DataGridSelectionMode
             */
            SelectionMode: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.Controls.DataGrid.SelectionModeProperty), Windows.UI.Xaml.Controls.DataGridSelectionMode), System.Int32));
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Controls.DataGrid.SelectionModeProperty, Bridge.box(value, Windows.UI.Xaml.Controls.DataGridSelectionMode, System.Enum.toStringFn(Windows.UI.Xaml.Controls.DataGridSelectionMode)));
                }
            },
            /**
             * // Gets or sets a value that indicates whether rows, cells, or both can be selected
            // in the System.Windows.Controls.DataGrid.
            //
             * Temporary solution to allow the choice of the DataGrid's horizontal lines colors
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.DataGrid
             * @function HorizontalGridLinesBrush
             * @type Windows.UI.Xaml.Media.Brush
             */
            HorizontalGridLinesBrush: {
                get: function () {
                    return Bridge.cast(this.GetValue(Windows.UI.Xaml.Controls.DataGrid.HorizontalGridLinesBrushProperty), Windows.UI.Xaml.Media.Brush);
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Controls.DataGrid.HorizontalGridLinesBrushProperty, value);
                }
            },
            /**
             * Gets or set the template for the row headers.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.DataGrid
             * @function RowHeaderTemplate
             * @type Windows.UI.Xaml.DataTemplate
             */
            RowHeaderTemplate: {
                get: function () {
                    return Bridge.cast(this.GetValue(Windows.UI.Xaml.Controls.DataGrid.RowHeaderTemplateProperty), Windows.UI.Xaml.DataTemplate);
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Controls.DataGrid.RowHeaderTemplateProperty, value);
                }
            },
            /**
             * Temporary solution to allow the choice of the DataGrid's vertical lines colors
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.DataGrid
             * @function VerticalGridLinesBrush
             * @type Windows.UI.Xaml.Media.Brush
             */
            VerticalGridLinesBrush: {
                get: function () {
                    return Bridge.cast(this.GetValue(Windows.UI.Xaml.Controls.DataGrid.VerticalGridLinesBrushProperty), Windows.UI.Xaml.Media.Brush);
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Controls.DataGrid.VerticalGridLinesBrushProperty, value);
                }
            }
        },
        ctors: {
            init: function () {
                this._objectsToDisplay = new (System.Collections.Generic.Dictionary$2(System.Object,Windows.UI.Xaml.Controls.DataGridRow)).ctor();
                this._areItemsInVisualTree = false;
            },
            /**
             * Initializes a new instance of the System.Windows.Controls.DataGrid class.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Controls.DataGrid
             * @memberof Windows.UI.Xaml.Controls.DataGrid
             * @return  {void}
             */
            ctor: function () {
                var $t;
                this.$initialize();
                Windows.UI.Xaml.Controls.Primitives.MultiSelector.ctor.call(this);
                this.CanBeInheritanceContext = false;
                this.IsInheritanceContextSealed = true;

                this.Columns = new (System.Collections.ObjectModel.ObservableCollection$1(Windows.UI.Xaml.Controls.DataGridColumn)).ctor();

                this._pagedView = new Windows.UI.Xaml.Data.INTERNAL_PagedCollectionView(null);
                this._pagedView.PageSize = 20;

                this._grid = new Windows.UI.Xaml.Controls.Grid();
                this._grid.RowDefinitions.add(($t = new Windows.UI.Xaml.Controls.RowDefinition(), $t.Height = Windows.UI.Xaml.GridLength.Auto.$clone(), $t));
                this._grid.INTERNAL_StringToSetVerticalGridLinesInCss = "thin solid black";
                this._grid.INTERNAL_StringToSetHorizontalGridLinesInCss = "thin solid black";
                Windows.UI.Xaml.Controls.Grid.SetRow(this._grid, 0);

                this._pagerUI = new Windows.UI.Xaml.Controls.DataPager();
                this._pagerUI.Source = this._pagedView;
                this._pagerUI.PageSize = 20;
                this._pagerUI.DisplayMode = Windows.UI.Xaml.Controls.DataPager_DisplayMode.FirstLastPreviousNext;
                Windows.UI.Xaml.Controls.Grid.SetRow(this._pagerUI, 1);
            }
        },
        methods: {
            OnLoadingRow: function (e) {
                if (!Bridge.staticEquals(this.LoadingRow, null)) {
                    this.LoadingRow(this, e);
                }
            },
            /**
             * Updates the reference to this DataGrid in the list of columns.
             *
             * @instance
             * @this Windows.UI.Xaml.Controls.DataGrid
             * @memberof Windows.UI.Xaml.Controls.DataGrid
             * @return  {void}
             */
            UpdateDataGridReference: function () {
                var $t;
                $t = Bridge.getEnumerator(this.Columns);
                try {
                    while ($t.moveNext()) {
                        var column = $t.Current;
                        column._parent = this;
                    }
                } finally {
                    if (Bridge.is($t, System.IDisposable)) {
                        $t.System$IDisposable$Dispose();
                    }
                }
            },
            ManageSelectedIndex_Changed: function (e) {
                Windows.UI.Xaml.Controls.Primitives.MultiSelector.prototype.ManageSelectedIndex_Changed.call(this, e);
                if ((e.OldValue != null && System.Nullable.getValue(Bridge.cast(Bridge.unbox(e.OldValue, System.Int32), System.Int32)) >= 0) && this.SelectionMode === Windows.UI.Xaml.Controls.DataGridSelectionMode.Single) {
                    CSHTML5.Internal.INTERNAL_WorkaroundObservableCollectionBugWithJSIL.Clear(this.SelectedItems);
                }
                if (e.NewValue != null && (System.Nullable.getValue(Bridge.cast(Bridge.unbox(e.NewValue, System.Int32), System.Int32)) >= 0)) {
                    if (this.SelectionMode === Windows.UI.Xaml.Controls.DataGridSelectionMode.Single) {
                        CSHTML5.Internal.INTERNAL_WorkaroundObservableCollectionBugWithJSIL.Clear(this.SelectedItems);
                        System.Array.add(this.SelectedItems, this.Items.getItem(System.Nullable.getValue(Bridge.cast(Bridge.unbox(e.NewValue, System.Int32), System.Int32))), Object);
                    } else if (CSHTML5.Internal.INTERNAL_WorkaroundObservableCollectionBugWithJSIL.Contains(this.SelectedItems, e.NewValue)) {
                        System.Array.add(this.SelectedItems, this.Items.getItem(System.Nullable.getValue(Bridge.cast(Bridge.unbox(e.NewValue, System.Int32), System.Int32))), Object);
                    }
                }
            },
            GetActiveRowHeaderTemplate: function () {
                if (this.RowHeaderTemplate == null) {
                    if (this.SelectionMode === Windows.UI.Xaml.Controls.DataGridSelectionMode.Extended) {
                        return Windows.UI.Xaml.Controls.DataGridRow.DefaultTemplateForExtendedSelectionMode;
                    } else {
                        return null;
                    }
                }
                return this.RowHeaderTemplate;
            },
            GenerateStringForGridLines: function (brush) {
                var str = "thin ";
                if (Bridge.is(brush, Windows.UI.Xaml.Media.SolidColorBrush)) {
                    str = (str || "") + "solid ";
                    str = (str || "") + ((Bridge.cast(brush, Windows.UI.Xaml.Media.SolidColorBrush).INTERNAL_ToHtmlString()) || "");
                } else {
                    str = (str || "") + "solid black";
                }
                return str;
            },
            /**
             * @instance
             * @override
             * @this Windows.UI.Xaml.Controls.DataGrid
             * @memberof Windows.UI.Xaml.Controls.DataGrid
             * @return  {void}
             */
            INTERNAL_OnAttachedToVisualTree: function () {
                Windows.UI.Xaml.Controls.Primitives.MultiSelector.prototype.INTERNAL_OnAttachedToVisualTree.call(this);

                this.UpdateUIStructure();

                this.AttachGridToVisualTree();
                if (this.ItemsHost == null && this.ItemsPanel != null) {
                    this.UpdateItemsPanel(this.ItemsPanel);
                } else if (this.ItemsHost != null) {
                    if (this._objectsToDisplay != null && this._objectsToDisplay.Count > 0) {
                        this._areItemsInVisualTree = true;
                        this.UpdateChildrenInVisualTree(this._objectsToDisplay.Keys, this.Items, true);
                    } else {
                        this.UpdateChildrenInVisualTree(null, this.Items);
                    }
                }
            },
            INTERNAL_OnDetachedFromVisualTree: function () {
                Windows.UI.Xaml.Controls.Primitives.MultiSelector.prototype.INTERNAL_OnDetachedFromVisualTree.call(this);

                this._areItemsInVisualTree = false;
            },
            UpdateUIStructure: function () {
                if (this._grid != null) {
                    if (this.Columns != null) {
                        this.RemoveAllHeaders();

                        this._grid.ColumnDefinitions.clear();

                        if (this.AutoGenerateColumns) {
                            this.GenerateColumns();
                        }
                        this.GenerateFakeColumnToTheEnd();

                        this.AddColumns(this.Columns);
                    }

                } else {
                    throw new System.Exception("Problem while changing DataGrid.ItemsSource, value cannot be null. Please report this issue to support@cshtml5.com.");
                }
            },
            UpdateItemsPanel: function (newTemplate) {
                newTemplate = Windows.UI.Xaml.Controls.DataGrid.ItemsPanelInternal;
                Windows.UI.Xaml.Controls.Primitives.MultiSelector.prototype.UpdateItemsPanel.call(this, newTemplate);
            },
            AddElementToGrid: function (elementToAdd) {
                this._grid.Children.add(elementToAdd);
            },
            RemoveElementFromGrid: function (elementToRemove) {
                this._grid.Children.remove(elementToRemove);
            },
            RemoveAllHeaders: function () {
                for (var i = (this._grid.Children.Count - 1) | 0; i > -1; i = (i - 1) | 0) {
                    var header = Bridge.as(this._grid.Children.getItem(i), Windows.UI.Xaml.Controls.Primitives.DataGridColumnHeader);
                    if (header != null) {
                        this._grid.Children.removeAt(i);
                    }
                }
            },
            AddColumns: function (columnsToAdd) {
                var $t, $t1;
                if (columnsToAdd != null) {
                    if (this.ItemsHost != null) {

                        this.ItemsHost.Children.clear();
                    }

                    var columnDefinitionForHeader = new Windows.UI.Xaml.Controls.ColumnDefinition();
                    columnDefinitionForHeader.Width = Windows.UI.Xaml.GridLength.Auto.$clone();
                    if (this.SelectionMode === Windows.UI.Xaml.Controls.DataGridSelectionMode.Single) {
                        columnDefinitionForHeader.Width = new Windows.UI.Xaml.GridLength.$ctor1(0);
                    }
                    this._grid.ColumnDefinitions.add(columnDefinitionForHeader);


                    var headers = new (System.Collections.Generic.List$1(Windows.UI.Xaml.Controls.Primitives.DataGridColumnHeader)).ctor();


                    var isCSSGrid = document.isGridSupported;

                    var currentColumnIndex = 1;
                    $t = Bridge.getEnumerator(columnsToAdd);
                    try {
                        while ($t.moveNext()) {
                            var column = $t.Current;
                            column._parent = this;
                            var columnDefinition = new Windows.UI.Xaml.Controls.ColumnDefinition();
                            column._gridColumn = columnDefinition;
                            if (column.Visibility === Windows.UI.Xaml.Visibility.Collapsed) {
                                column._gridColumn.Visibility = Windows.UI.Xaml.Visibility.Collapsed;
                            }
                            Windows.UI.Xaml.Controls.DataGridColumn.UpdateGridColumnWidth(column);
                            this._grid.ColumnDefinitions.add(columnDefinition);
                            var columnHeader = column.GetHeader();

                            columnHeader.HorizontalContentAlignment = Windows.UI.Xaml.HorizontalAlignment.Center;
                            columnHeader.VerticalContentAlignment = Windows.UI.Xaml.VerticalAlignment.Center;
                            columnHeader.FontWeight = Windows.UI.Text.FontWeights.Bold.$clone();
                            if (!isNaN(this.ColumnHeaderHeight)) {
                                columnHeader.Height = this.ColumnHeaderHeight;
                            }


                            if (column.HeaderStyle != null) {
                                columnHeader.Style = column.HeaderStyle;
                            } else if (this.ColumnHeaderStyle != null) {
                                columnHeader.Style = this.ColumnHeaderStyle;
                            }

                            Windows.UI.Xaml.Controls.Grid.SetColumn(columnHeader, currentColumnIndex);
                            headers.add(columnHeader);

                            if (Bridge.is(column, Windows.UI.Xaml.Controls.DataGridBoundColumn)) {
                                var b = Bridge.as(Bridge.cast(column, Windows.UI.Xaml.Controls.DataGridBoundColumn).Binding, Windows.UI.Xaml.Data.Binding);
                                if (b != null) {
                                    if (b.Mode === Windows.UI.Xaml.Data.BindingMode.OneWay && b.INTERNAL_WasModeSetByUserRatherThanDefaultValue()) {
                                        column.IsReadOnly = true;
                                    }
                                }
                            }
                            currentColumnIndex = (currentColumnIndex + 1) | 0;
                        }
                    } finally {
                        if (Bridge.is($t, System.IDisposable)) {
                            $t.System$IDisposable$Dispose();
                        }
                    }

                    $t1 = Bridge.getEnumerator(headers);
                    try {
                        while ($t1.moveNext()) {
                            var header = $t1.Current;
                            if (isCSSGrid) {
                                var brush = this.HorizontalGridLinesBrush;
                                var HorizontalLinesThickness = this.HorizontalGridLinesBrush.Opacity > 0 ? 1 : 0;
                                var VerticalLinesThickness = this.VerticalGridLinesBrush.Opacity > 0 ? 1 : 0;
                                if (HorizontalLinesThickness === 0) {
                                    brush = this.VerticalGridLinesBrush;
                                }
                                header.BorderBrush = brush;
                                header.BorderThickness = new Windows.UI.Xaml.Thickness.$ctor2((Windows.UI.Xaml.Controls.Grid.GetColumn(header) === 1 ? VerticalLinesThickness : 0), HorizontalLinesThickness, VerticalLinesThickness, HorizontalLinesThickness);
                            }
                            var u = Windows.UI.Xaml.Controls.Grid.GetColumn(header);
                            this._grid.Children.add(header);
                        }
                    } finally {
                        if (Bridge.is($t1, System.IDisposable)) {
                            $t1.System$IDisposable$Dispose();
                        }
                    }

                    this.AttachGridToVisualTree();

                }
            },
            ManageCollectionChanged: function (e) {
                switch (e.Action) {
                    case System.Collections.Specialized.NotifyCollectionChangedAction.Add: 
                        if (this.ItemsHost != null) {
                            var isItemsSourceEmptyOrNull = (this.ItemsSource == null);
                            if (!isItemsSourceEmptyOrNull) {
                                isItemsSourceEmptyOrNull = !Bridge.getEnumerator(this.ItemsSource).System$Collections$IEnumerator$moveNext();
                            }

                            if (this.AutoGenerateColumns && isItemsSourceEmptyOrNull) {
                                if (this.Items.Count === 1) {
                                    this.UpdateUIStructure();
                                }
                            }

                            var oldChildren = CSHTML5.Internal.INTERNAL_ListsHelper.ConvertToListOfObjectsOrNull(this.Items);
                            var i = (System.Array.getCount(e.NewItems) - 1) | 0;
                            while (i >= 0) {
                                oldChildren.removeAt(((e.NewStartingIndex + i) | 0));
                                i = (i - 1) | 0;
                            }
                            this.UpdateChildrenInVisualTree(oldChildren, this.Items, true);
                        }
                        break;
                    case System.Collections.Specialized.NotifyCollectionChangedAction.Remove: 
                        if (this.ItemsHost != null) {
                            var oldChildren2 = CSHTML5.Internal.INTERNAL_ListsHelper.ConvertToListOfObjectsOrNull(this.Items);
                            var oldItems = CSHTML5.Internal.INTERNAL_ListsHelper.ConvertToListOfObjectsOrNull(e.OldItems);
                            for (var i1 = 0; i1 < oldItems.Count; i1 = (i1 + 1) | 0) {
                                var str = Bridge.toString(e.OldStartingIndex);
                                if (e.OldStartingIndex >= 0) {
                                    oldChildren2.insert(((e.OldStartingIndex + i1) | 0), oldItems.getItem(i1));
                                } else {
                                    oldChildren2.add(oldItems.getItem(i1));
                                }
                            }

                            this.UpdateChildrenInVisualTree(oldChildren2, this.Items, true);
                        }
                        break;
                    case System.Collections.Specialized.NotifyCollectionChangedAction.Replace: 
                        break;
                    case System.Collections.Specialized.NotifyCollectionChangedAction.Move: 
                        break;
                    case System.Collections.Specialized.NotifyCollectionChangedAction.Reset: 
                        if (this.ItemsHost != null) {
                            this.RemoveAllChildren();
                            if (this.Items.Count > 0) {
                                this.UpdateChildrenInVisualTree(System.Linq.Enumerable.empty(), this.Items, true);
                            }
                        }
                        break;
                    default: 
                        break;
                }
            },
            /**
             * Adds a row to the DataGrid. NOTE: The element can only be added at the end of the list.
             *
             * @instance
             * @private
             * @this Windows.UI.Xaml.Controls.DataGrid
             * @memberof Windows.UI.Xaml.Controls.DataGrid
             * @param   {System.Object}    childData
             * @return  {boolean}
             */
            AddChild: function (childData) {
                var $t;
                if (!this._objectsToDisplay.containsKey(childData)) {
                    var rowDefinition = new Windows.UI.Xaml.Controls.RowDefinition();
                    rowDefinition.Height = Windows.UI.Xaml.GridLength.Auto.$clone();
                    this._grid.RowDefinitions.add(rowDefinition);

                    var objectRow = new Windows.UI.Xaml.Controls.DataGridRow();
                    objectRow._datagrid = this;
                    objectRow._representationInRow.RowDefinition = rowDefinition;
                    objectRow._rowIndex = (this._grid.RowDefinitions.Count - 1) | 0;
                    objectRow.HeaderTemplate = this.GetActiveRowHeaderTemplate();

                    var isCSSGrid = document.isGridSupported;

                    var currentColumnIndex = 1;
                    $t = Bridge.getEnumerator(this.Columns);
                    try {
                        while ($t.moveNext()) {
                            var column = $t.Current;
                            var cell = new Windows.UI.Xaml.Controls.DataGridCell();
                            cell.Item = childData;
                            cell.DataContext = childData;
                            cell.Column = column;
                            var f = column.GenerateElement(childData);
                            Windows.UI.Xaml.Controls.Grid.SetRow(cell, ((this._grid.RowDefinitions.Count - 1) | 0));
                            Windows.UI.Xaml.Controls.Grid.SetColumn(cell, currentColumnIndex);
                            cell.Content = f;
                            cell.RegisterToContentPressEvent();
                            objectRow._representationInRow.ElementsInRow.add(cell);
                            cell.addClick(Bridge.fn.cacheBind(this, this.CellElement_Click));
                            this._grid.Children.add(cell);
                            if (isCSSGrid) {
                                var brush = this.HorizontalGridLinesBrush;
                                var HorizontalLinesThickness = this.HorizontalGridLinesBrush.Opacity > 0 ? 1 : 0;
                                var VerticalLinesThickness = this.VerticalGridLinesBrush.Opacity > 0 ? 1 : 0;
                                if (HorizontalLinesThickness === 0) {
                                    brush = this.VerticalGridLinesBrush;
                                }
                                cell.BorderBrush = brush;
                                cell.BorderThickness = new Windows.UI.Xaml.Thickness.$ctor2((currentColumnIndex === 1 ? VerticalLinesThickness : 0), (this._grid.RowDefinitions.Count === 1 ? HorizontalLinesThickness : 0), VerticalLinesThickness, HorizontalLinesThickness);
                                cell.Padding = new Windows.UI.Xaml.Thickness.$ctor2(4, this.VerticalCellPadding, 4, this.VerticalCellPadding);
                            }

                            var elementRow = (this._grid.RowDefinitions.Count - 1) | 0;

                            if (elementRow % 2 === 0 && elementRow !== 0) {
                                cell.Background = this.AlternatingRowBackground;
                            } else {
                                cell.Background = this.RowBackground;
                            }
                            cell.Foreground = this.UnselectedItemForeground;

                            if (column.CellStyle != null) {
                                cell.Style = column.CellStyle;
                            } else if (this.CellStyle != null) {
                                cell.Style = this.CellStyle;
                            }
                            currentColumnIndex = (currentColumnIndex + 1) | 0;
                        }
                    } finally {
                        if (Bridge.is($t, System.IDisposable)) {
                            $t.System$IDisposable$Dispose();
                        }
                    }

                    objectRow.INTERNAL_SetRowEvents();

                    this._objectsToDisplay.add(childData, objectRow);

                    objectRow.INTERNAL_SetDataContext();

                    this.OnLoadingRow(new Windows.UI.Xaml.Controls.DataGridRowEventArgs(objectRow));

                    return true;
                } else {
                    return false;
                }
            },
            CellElement_Click: function (sender, e) {
                var cell = Bridge.cast(sender, Windows.UI.Xaml.Controls.DataGridCell);
                if (!Bridge.referenceEquals(this._currentCell, cell)) {

                    if (this.SelectionMode === Windows.UI.Xaml.Controls.DataGridSelectionMode.Single && this.SelectedItem != null) {
                        this.UnselectItem(this.SelectedItem);
                    }
                    if (this._currentCell != null) {
                        this.LeaveCellEditionMode();
                    }
                    this.SelectItem(cell.Item);
                    this._currentCell = cell;

                    if (!this.EnableTwoStepsEditMode) {
                        this.EnterCellEditionMode(cell);
                    }
                } else {

                    this.EnterCellEditionMode(cell);
                }
            },
            RemoveChild: function (childData) {
                var $t;
                var child = this._objectsToDisplay.getItem(childData);
                $t = Bridge.getEnumerator(child._representationInRow.ElementsInRow);
                try {
                    while ($t.moveNext()) {
                        var element = Bridge.cast($t.Current, Windows.UI.Xaml.UIElement);
                        this._grid.Children.remove(element);
                    }
                } finally {
                    if (Bridge.is($t, System.IDisposable)) {
                        $t.System$IDisposable$Dispose();
                    }
                }
                this._grid.RowDefinitions.remove(child._representationInRow.RowDefinition);
                this._objectsToDisplay.remove(childData);
            },
            RemoveAllChildren: function () {
                var $t, $t1;
                $t = Bridge.getEnumerator(INTERNAL_BridgeWorkarounds.GetDictionaryValues_SimulatorCompatible(System.Object, Windows.UI.Xaml.Controls.DataGridRow, this._objectsToDisplay), Windows.UI.Xaml.Controls.DataGridRow);
                try {
                    while ($t.moveNext()) {
                        var child = $t.Current;
                        $t1 = Bridge.getEnumerator(child._representationInRow.ElementsInRow);
                        try {
                            while ($t1.moveNext()) {
                                var element = Bridge.cast($t1.Current, Windows.UI.Xaml.UIElement);
                                this._grid.Children.remove(element);
                            }
                        } finally {
                            if (Bridge.is($t1, System.IDisposable)) {
                                $t1.System$IDisposable$Dispose();
                            }
                        }
                        this._grid.RowDefinitions.remove(child._representationInRow.RowDefinition);
                    }
                } finally {
                    if (Bridge.is($t, System.IDisposable)) {
                        $t.System$IDisposable$Dispose();
                    }
                }
                this._objectsToDisplay.clear();
            },
            OnItemsSourceChanged: function (oldValue, newValue) {
                if (newValue == null) {

                    Windows.UI.Xaml.Controls.Primitives.MultiSelector.prototype.OnItemsSourceChanged.call(this, oldValue, newValue);
                } else {


                    if (!Bridge.referenceEquals(this.ItemsSource, this._pagedView)) {

                        this._oldOldValue = oldValue;

                        this._pagedView.SetIEnumerableAsCollectionSource(newValue);

                        this.SetCurrentValue(Windows.UI.Xaml.Controls.ItemsControl.ItemsSourceProperty, this._pagedView);
                    } else {

                        Windows.UI.Xaml.Controls.Primitives.MultiSelector.prototype.OnItemsSourceChanged.call(this, this._oldOldValue, newValue);
                    }
                }
            },
            /**
             * Prepares the DataGrid for the update of its display (removes the autogenerated Columns if needed and registers to the ItemsSource.CollectionChanged event.
             *
             * @instance
             * @protected
             * @override
             * @this Windows.UI.Xaml.Controls.DataGrid
             * @memberof Windows.UI.Xaml.Controls.DataGrid
             * @param   {System.Collections.IEnumerable}    oldValue    
             * @param   {System.Collections.IEnumerable}    newValue
             * @return  {void}
             */
            OnItemsSourceChanged_BeforeVisualUpdate: function (oldValue, newValue) {
                var $t, $t1, $t2;
                if (!Bridge.referenceEquals(newValue, oldValue) || !this._areItemsInVisualTree) {
                    if (CSHTML5.Internal.INTERNAL_VisualTreeManager.IsElementInVisualTree(this)) {
                        var oldType = null;
                        if (oldValue != null) {
                            $t = Bridge.getEnumerator(oldValue);
                            try {
                                while ($t.moveNext()) {
                                    var oldItem = $t.Current;
                                    oldType = Bridge.getType(oldItem);
                                    break;
                                }
                            } finally {
                                if (Bridge.is($t, System.IDisposable)) {
                                    $t.System$IDisposable$Dispose();
                                }
                            }
                        }
                        var newType = null;
                        if (newValue != null) {
                            $t1 = Bridge.getEnumerator(newValue);
                            try {
                                while ($t1.moveNext()) {
                                    var newItem = $t1.Current;
                                    newType = Bridge.getType(newItem);
                                    break;
                                }
                            } finally {
                                if (Bridge.is($t1, System.IDisposable)) {
                                    $t1.System$IDisposable$Dispose();
                                }
                            }
                        }

                        if (!Bridge.referenceEquals(oldType, newType)) {
                            if (this.AutoGeneratedColumns != null) {
                                $t2 = Bridge.getEnumerator(this.AutoGeneratedColumns);
                                try {
                                    while ($t2.moveNext()) {
                                        var column = $t2.Current;
                                        this.Columns.remove(column);
                                    }
                                } finally {
                                    if (Bridge.is($t2, System.IDisposable)) {
                                        $t2.System$IDisposable$Dispose();
                                    }
                                }
                                this.AutoGeneratedColumns.clear();
                            }
                            if (newType != null) {
                                this.UpdateUIStructure();
                            }
                        }
                    }

                    this.UpdateDataGridReference();
                }
            },
            EnterCellEditionMode: function (cell) {
                if (!this.IsReadOnly && this.IsEnabled && !cell.Column.IsReadOnly) {
                    var item = cell.Item;
                    var cellColumn = Windows.UI.Xaml.Controls.Grid.GetColumn(cell);
                    var column = System.Linq.Enumerable.from(this.Columns).elementAt(((cellColumn - 1) | 0));
                    var editableVersionOfCell = column.GenerateEditingElement(item);
                    if (editableVersionOfCell != null) {
                        var editableVersionOfCellContainer = new Windows.UI.Xaml.Controls.DataGridCell();
                        editableVersionOfCellContainer.Column = column;
                        editableVersionOfCellContainer.IsSelected = true;
                        if (column.CellStyle != null) {
                            editableVersionOfCellContainer.Style = column.CellStyle;
                        } else if (column._parent.CellStyle != null) {
                            editableVersionOfCellContainer.Style = column._parent.CellStyle;
                        }
                        editableVersionOfCellContainer.Content = editableVersionOfCell;
                        editableVersionOfCell = editableVersionOfCellContainer;



                        cell.Visibility = Windows.UI.Xaml.Visibility.Collapsed;

                        Windows.UI.Xaml.Controls.Grid.SetRow(editableVersionOfCell, Windows.UI.Xaml.Controls.Grid.GetRow(cell));
                        Windows.UI.Xaml.Controls.Grid.SetColumn(editableVersionOfCell, cellColumn);
                        editableVersionOfCell.addLostFocus(Bridge.fn.cacheBind(this, this.CurrentEditionElement_LostFocus));

                        this._grid.Children.add(editableVersionOfCell);
                        if (Bridge.is(column, Windows.UI.Xaml.Controls.DataGridTextColumn)) {
                            editableVersionOfCellContainer.Foreground = new Windows.UI.Xaml.Media.SolidColorBrush.$ctor1(Windows.UI.Colors.Black.$clone());
                        }
                    } else {
                        Bridge.cast(cell.Content, Windows.UI.Xaml.FrameworkElement).removeLostFocus(Bridge.fn.cacheBind(this, this.CurrentEditionElement_LostFocus));
                        Bridge.cast(cell.Content, Windows.UI.Xaml.FrameworkElement).addLostFocus(Bridge.fn.cacheBind(this, this.CurrentEditionElement_LostFocus));
                    }
                    this._currentCell = cell;
                    cell.IsEditing = true;
                    this._currentEditionElement = editableVersionOfCell;
                }
            },
            CurrentEditionElement_LostFocus: function (sender, e) {
                Bridge.cast(sender, Windows.UI.Xaml.FrameworkElement).removeLostFocus(Bridge.fn.cacheBind(this, this.CurrentEditionElement_LostFocus));
                this.LeaveCellEditionMode();
            },
            LeaveCellEditionMode: function () {
                if (this._currentCell != null) {
                    if (!this.IsReadOnly && this.IsEnabled && !this._currentCell.Column.IsReadOnly) {
                        var temp = this._currentCell.DataContext;
                        this._currentCell.DataContext = null;
                        this._currentCell.DataContext = temp;
                        this._currentCell.Visibility = Windows.UI.Xaml.Visibility.Visible;
                        if (!(Bridge.is(this._currentCell.Column, Windows.UI.Xaml.Controls.DataGridTemplateColumn))) {
                            this._currentCell.IsEditing = false;
                        }

                        if (this._currentEditionElement != null) {
                            Bridge.cast(this._currentEditionElement, Windows.UI.Xaml.FrameworkElement).DataContext = null;
                            this._grid.Children.remove(this._currentEditionElement);
                        }
                    }
                    this._currentCell = null;
                    this._currentEditionElement = null;
                }
            },
            GenerateColumns: function () {
                var $t, $t1, $t2;
                if (this.AutoGeneratedColumns != null) {
                    $t = Bridge.getEnumerator(this.AutoGeneratedColumns);
                    try {
                        while ($t.moveNext()) {
                            var column = $t.Current;
                            this.Columns.remove(column);
                        }
                    } finally {
                        if (Bridge.is($t, System.IDisposable)) {
                            $t.System$IDisposable$Dispose();
                        }
                    }
                    this.AutoGeneratedColumns.clear();
                }
                if (this.Items != null || this.ItemsSource != null) {
                    var type = null;
                    if (this.ItemsSource != null) {
                        var type1 = Bridge.getType(this.ItemsSource);

                        if (Bridge.is(this.ItemsSource, Windows.UI.Xaml.Data.INTERNAL_PagedCollectionView)) {
                            type1 = Bridge.getType(Bridge.cast(this.ItemsSource, Windows.UI.Xaml.Data.INTERNAL_PagedCollectionView)._originalDataSourceWithoutCopy);
                        }

                        if (Bridge.Reflection.getGenericArguments(type1).length > 0) {
                            type = ($t1 = Bridge.Reflection.getGenericArguments(type1))[System.Array.index(0, $t1)];
                        }
                    }
                    if (type == null) {
                        if (this.Items != null && this.Items.Count > 0) {
                            type = Bridge.getType(this.Items.getItem(0));
                        }
                    }
                    if (type != null) {
                        if (this.AutoGeneratedColumns == null) {
                            this.AutoGeneratedColumns = new (System.Collections.ObjectModel.ObservableCollection$1(Windows.UI.Xaml.Controls.DataGridColumn)).ctor();
                        }
                        $t2 = Bridge.getEnumerator(Bridge.Reflection.getMembers(type, 16, 28));
                        try {
                            while ($t2.moveNext()) {
                                var property = $t2.Current;
                                var propertyName = property.n;
                                var propertyType = property.rt;
                                var column1;
                                var binding = new Windows.UI.Xaml.Data.Binding.$ctor1(propertyName);

                                if (Bridge.Reflection.isAssignableFrom(System.String, propertyType)) {
                                    column1 = new Windows.UI.Xaml.Controls.DataGridTextColumn();
                                } else if (Bridge.Reflection.isAssignableFrom(System.Boolean, propertyType)) {
                                    column1 = new Windows.UI.Xaml.Controls.DataGridCheckBoxColumn();
                                } else if (Bridge.Reflection.isAssignableFrom(System.Uri, propertyType)) {
                                    column1 = new Windows.UI.Xaml.Controls.DataGridHyperlinkColumn();
                                } else if (Bridge.Reflection.isAssignableFrom(System.Enum, propertyType)) {
                                    column1 = new Windows.UI.Xaml.Controls.DataGridComboBoxColumn();
                                } else {
                                    column1 = new Windows.UI.Xaml.Controls.DataGridTextColumn();
                                    column1.IsReadOnly = false;
                                }
                                column1.IsAutoGenerated = true;
                                column1.Header = propertyName;
                                column1.Binding = binding;
                                this.AutoGeneratedColumns.add(column1);
                                this.Columns.add(column1);
                            }
                        } finally {
                            if (Bridge.is($t2, System.IDisposable)) {
                                $t2.System$IDisposable$Dispose();
                            }
                        }
                    }
                }
            },
            GenerateFakeColumnToTheEnd: function () {
                var $t, $t1;

                if (this.AutoGeneratedColumns == null) {
                    this.AutoGeneratedColumns = new (System.Collections.ObjectModel.ObservableCollection$1(Windows.UI.Xaml.Controls.DataGridColumn)).ctor();
                }

                var atLeastOneColumnHasStarSize = false;
                $t = Bridge.getEnumerator(this.Columns);
                try {
                    while ($t.moveNext()) {
                        var column = $t.Current;
                        if (column.Width.IsStar) {
                            atLeastOneColumnHasStarSize = true;
                        }
                    }
                } finally {
                    if (Bridge.is($t, System.IDisposable)) {
                        $t.System$IDisposable$Dispose();
                    }
                }

                if (!atLeastOneColumnHasStarSize) {
                    var fakeColumn = ($t1 = new Windows.UI.Xaml.Controls.DataGridTextColumn(), $t1.Width = new Windows.UI.Xaml.Controls.DataGridLength.$ctor2(1, Windows.UI.Xaml.Controls.DataGridLengthUnitType.Star), $t1);
                    fakeColumn.Header = "";
                    var binding = new Windows.UI.Xaml.Data.Binding.$ctor1("PROPERTY_THAT_DOES_NOT_EXIST");
                    fakeColumn.Binding = binding;

                    this.AutoGeneratedColumns.add(fakeColumn);
                    this.Columns.add(fakeColumn);
                }
            },
            UpdateChildrenInVisualTree: function (oldChildrenEnumerable, newChildrenEnumerable, forceUpdateAllChildren) {
                var $t, $t1, $t2;
                if (forceUpdateAllChildren === void 0) { forceUpdateAllChildren = false; }
                if (CSHTML5.Internal.INTERNAL_VisualTreeManager.IsElementInVisualTree(this) && this.ItemsHost != null) {
                    this.ItemsHost.Children.clear();

                    var oldChildren = CSHTML5.Internal.INTERNAL_ListsHelper.ConvertToListOfObjectsOrNull(oldChildrenEnumerable);
                    var newChildren = CSHTML5.Internal.INTERNAL_ListsHelper.ConvertToListOfObjectsOrNull(newChildrenEnumerable);


                    if (!this._areItemsInVisualTree && oldChildren != null) {
                        if (this._objectsToDisplay != null) {
                            this._objectsToDisplay.clear();
                        }
                        oldChildren.clear();
                    }

                    if (forceUpdateAllChildren) {
                        if (oldChildren != null) {
                            $t = Bridge.getEnumerator(oldChildren);
                            try {
                                while ($t.moveNext()) {
                                    var oldChild = $t.Current;
                                    this.RemoveChild(oldChild);
                                }
                            } finally {
                                if (Bridge.is($t, System.IDisposable)) {
                                    $t.System$IDisposable$Dispose();
                                }
                            }
                        }
                        if (newChildren != null) {
                            $t1 = Bridge.getEnumerator(newChildren);
                            try {
                                while ($t1.moveNext()) {
                                    var newChild = $t1.Current;
                                    this.AddChild(newChild);
                                }
                            } finally {
                                if (Bridge.is($t1, System.IDisposable)) {
                                    $t1.System$IDisposable$Dispose();
                                }
                            }
                        }
                    } else {
                        var currentIndexInOldChildren = 0;
                        if (newChildren != null) {
                            var currentIndexOfChild = 1;
                            $t2 = Bridge.getEnumerator(newChildren);
                            try {
                                while ($t2.moveNext()) {
                                    var newChild1 = $t2.Current;
                                    var newChildSuccessfullyAdded = true;
                                    if (oldChildren != null) {
                                        while (currentIndexInOldChildren < oldChildren.Count && !Bridge.referenceEquals(newChild1, oldChildren.getItem(currentIndexInOldChildren))) {
                                            this.RemoveChild(oldChildren.getItem(currentIndexInOldChildren));
                                            currentIndexInOldChildren = (currentIndexInOldChildren + 1) | 0;
                                        }
                                    }
                                    if (oldChildren != null && currentIndexInOldChildren < oldChildren.Count) {
                                        var currentDataGridRow = this._objectsToDisplay.getItem(newChild1);
                                        currentDataGridRow.INTERNAL_SetRowIndex(currentIndexOfChild);
                                        currentIndexInOldChildren = (currentIndexInOldChildren + 1) | 0;
                                    } else {
                                        newChildSuccessfullyAdded = this.AddChild(newChild1);
                                    }

                                    if (newChildSuccessfullyAdded) {
                                        currentIndexOfChild = (currentIndexOfChild + 1) | 0;
                                    }
                                }
                            } finally {
                                if (Bridge.is($t2, System.IDisposable)) {
                                    $t2.System$IDisposable$Dispose();
                                }
                            }
                        }
                        if (oldChildren != null) {
                            while (currentIndexInOldChildren < oldChildren.Count) {
                                this.RemoveChild(oldChildren.getItem(currentIndexInOldChildren));
                                currentIndexInOldChildren = (currentIndexInOldChildren + 1) | 0;
                            }
                        }
                    }
                    this._areItemsInVisualTree = true;

                    this.AttachGridToVisualTree();
                } else {
                    this._areItemsInVisualTree = false;
                }
            },
            AttachGridToVisualTree: function () {
                if (!CSHTML5.Internal.INTERNAL_VisualTreeManager.IsElementInVisualTree(this._grid)) {
                    CSHTML5.Internal.INTERNAL_VisualTreeManager.AttachVisualChildIfNotAlreadyAttached(this._grid, this);

                    if (CSHTML5.Internal.INTERNAL_VisualTreeManager.IsElementInVisualTree(this._pagerUI)) {
                        throw new System.Exception("The DataPager is already attached and cannot be attached twice.");
                    }

                    CSHTML5.Internal.INTERNAL_VisualTreeManager.AttachVisualChildIfNotAlreadyAttached(this._pagerUI, this);
                }

                this.VerticalAlignment = Windows.UI.Xaml.VerticalAlignment.Top;
            },
            UnselectAllItems: function () {
                var $t;
                $t = Bridge.getEnumerator(INTERNAL_BridgeWorkarounds.GetDictionaryValues_SimulatorCompatible(System.Object, Windows.UI.Xaml.Controls.DataGridRow, this._objectsToDisplay), Windows.UI.Xaml.Controls.DataGridRow);
                try {
                    while ($t.moveNext()) {
                        var row = $t.Current;
                        row.IsSelected = false;
                        row.VisuallyRefreshItemSelection(false);
                    }
                } finally {
                    if (Bridge.is($t, System.IDisposable)) {
                        $t.System$IDisposable$Dispose();
                    }
                }
            },
            SetItemVisualSelectionState: function (item, newState) {
                if (item != null && this._objectsToDisplay.containsKey(item)) {
                    var itemRow = this._objectsToDisplay.getItem(item);
                    itemRow.IsSelected = newState;
                    itemRow.VisuallyRefreshItemSelection(newState);
                    if (newState) {
                        this.SelectedItem = item;
                    }
                }
            },
            SelectItem: function (item) {
                if (this.SelectionMode === Windows.UI.Xaml.Controls.DataGridSelectionMode.Single) {
                    System.Array.add(this.SelectedItems, item, Object);
                } else {
                    if (!CSHTML5.Internal.INTERNAL_WorkaroundObservableCollectionBugWithJSIL.Contains(this.SelectedItems, item)) {
                        System.Array.add(this.SelectedItems, item, Object);
                    }
                }
            },
            UnselectItem: function (item) {
                if (this.SelectionMode === Windows.UI.Xaml.Controls.DataGridSelectionMode.Single) {
                    CSHTML5.Internal.INTERNAL_WorkaroundObservableCollectionBugWithJSIL.Clear(this.SelectedItems);
                } else {
                    if (CSHTML5.Internal.INTERNAL_WorkaroundObservableCollectionBugWithJSIL.Contains(this.SelectedItems, item)) {
                        CSHTML5.Internal.INTERNAL_WorkaroundObservableCollectionBugWithJSIL.Remove(this.SelectedItems, item);
                    }
                }

            }
        }
    });

    /**
     * Contains a list of selectable items.
     *
     * @public
     * @class Windows.UI.Xaml.Controls.ListBox
     * @augments Windows.UI.Xaml.Controls.Primitives.MultiSelector
     * @example
     *
     *
     * ListBox1.ItemsSource = Planet.GetListOfPlanets();
     * public partial class Planet
     * {
     *    public string Name { get; set; }
     *    public string ImagePath { get; set; }
     *    public static ObservableCollection<Planet> GetListOfPlanets()
     *    {
     *        return new ObservableCollection<Planet>()
     *        {
     *            new Planet() { Name = "Mercury", ImagePath = "ms-appx:/Planets/Mercury.png" },
     *            new Planet() { Name = "Venus", ImagePath = "ms-appx:/Planets/Venus.png" },
     *            new Planet() { Name = "Earth", ImagePath = "ms-appx:/Planets/Earth.png" },
     *            new Planet() { Name = "Mars", ImagePath = "ms-appx:/Planets/Mars.png" },
     *            new Planet() { Name = "Jupiter", ImagePath = "ms-appx:/Planets/Jupiter.png" },
     *            new Planet() { Name = "Saturn", ImagePath = "ms-appx:/Planets/Saturn.png" },
     *            new Planet() { Name = "Uranus", ImagePath = "ms-appx:/Planets/Uranus.png" },
     *            new Planet() { Name = "Neptune", ImagePath = "ms-appx:/Planets/Neptune.png" }
     *            new Planet() { Name = "Pluto", ImagePath = "ms-appx:/Planets/Pluto.png" }
     *        };
     *    }
     *} 
     * 
     *
     *
     */
    Bridge.define("Windows.UI.Xaml.Controls.ListBox", {
        inherits: [Windows.UI.Xaml.Controls.Primitives.MultiSelector],
        statics: {
            fields: {
                /**
                 * Identifies the SelectionMode dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Controls.ListBox
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                SelectionModeProperty: null
            },
            ctors: {
                init: function () {
                    this.SelectionModeProperty = Windows.UI.Xaml.DependencyProperty.Register("SelectionMode", Windows.UI.Xaml.Controls.SelectionMode, Windows.UI.Xaml.Controls.ListBox, new Windows.UI.Xaml.PropertyMetadata.$ctor1(Bridge.box(Windows.UI.Xaml.Controls.SelectionMode.Single, Windows.UI.Xaml.Controls.SelectionMode, System.Enum.toStringFn(Windows.UI.Xaml.Controls.SelectionMode))));
                }
            }
        },
        fields: {
            _indexOfLastClickedItemWithoutShiftKey: 0
        },
        props: {
            /**
             * Gets or sets the selection behavior for the ListBox control.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.ListBox
             * @function SelectionMode
             * @type Windows.UI.Xaml.Controls.SelectionMode
             */
            SelectionMode: {
                get: function () {
                    return System.Nullable.getValue(Bridge.cast(Bridge.unbox(this.GetValue(Windows.UI.Xaml.Controls.ListBox.SelectionModeProperty), Windows.UI.Xaml.Controls.SelectionMode), System.Int32));
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Controls.ListBox.SelectionModeProperty, Bridge.box(value, Windows.UI.Xaml.Controls.SelectionMode, System.Enum.toStringFn(Windows.UI.Xaml.Controls.SelectionMode)));
                }
            },
            /**
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.ListBox
             * @function SelectedItemBackgroundBrush
             * @type Windows.UI.Xaml.Media.Brush
             */
            SelectedItemBackgroundBrush: {
                get: function () {
                    return this.SelectedItemBackground;
                },
                set: function (value) {
                    this.SelectedItemBackground = value;
                }
            },
            SelectedItemForegroundBrush: {
                get: function () {
                    return this.SelectedItemForeground;
                },
                set: function (value) {
                    this.SelectedItemForeground = value;
                }
            },
            UnselectedItemBackgroundBrush: {
                get: function () {
                    return this.RowBackground;
                },
                set: function (value) {
                    this.RowBackground = value;
                }
            },
            UnselectedItemForegroundBrush: {
                get: function () {
                    return this.UnselectedItemForeground;
                },
                set: function (value) {
                    this.UnselectedItemForeground = value;
                }
            }
        },
        ctors: {
            init: function () {
                this._indexOfLastClickedItemWithoutShiftKey = 0;
            },
            /**
             * Initializes a new instance of the ListBox class.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Controls.ListBox
             * @memberof Windows.UI.Xaml.Controls.ListBox
             * @return  {void}
             */
            ctor: function () {
                this.$initialize();
                Windows.UI.Xaml.Controls.Primitives.MultiSelector.ctor.call(this);
            }
        },
        methods: {
            PrepareContainerForItemOverride: function (element, item) {
                Windows.UI.Xaml.Controls.Primitives.MultiSelector.prototype.PrepareContainerForItemOverride.call(this, element, item);

                var container = Bridge.as(element, Windows.UI.Xaml.Controls.ListBoxItem);
                if (container != null) {
                    container.INTERNAL_CorrespondingItem = item;
                    container.INTERNAL_ParentSelectorControl = this;
                    if (System.Array.contains(this.SelectedItems, item)) {

                        container.IsSelected = true;
                    }
                    container.addClick(Bridge.fn.cacheBind(this, this.listBoxItem_Click));
                }
            },
            GetContainerForItemOverride: function () {
                return new Windows.UI.Xaml.Controls.ListBoxItem();
            },
            IsItemItsOwnContainerOverride: function (item) {
                return (Bridge.is(item, Windows.UI.Xaml.Controls.ListBoxItem));
            },
            INTERNAL_GenerateContainer: function (item) {
                return Bridge.cast(this.GetContainerFromItem(item), Windows.UI.Xaml.Controls.Primitives.SelectorItem);
            },
            GetContainerFromItem: function (item) {
                var listBoxItem = Bridge.as(item, Windows.UI.Xaml.Controls.ListBoxItem) || new Windows.UI.Xaml.Controls.ListBoxItem();
                listBoxItem.INTERNAL_CorrespondingItem = item;
                listBoxItem.INTERNAL_ParentSelectorControl = this;
                if (System.Array.contains(this.SelectedItems, item)) {

                    listBoxItem.IsSelected = true;
                }
                listBoxItem.addClick(Bridge.fn.cacheBind(this, this.listBoxItem_Click));
                return listBoxItem;
            },
            ManageSelectedIndex_Changed: function (e) {

                Windows.UI.Xaml.Controls.Primitives.MultiSelector.prototype.ManageSelectedIndex_Changed.call(this, e);
                if (e.OldValue != null && Bridge.is(e.OldValue, System.Int32) && System.Convert.toInt32(e.OldValue) >= 0 && this.SelectionMode === Windows.UI.Xaml.Controls.SelectionMode.Single) {
                    CSHTML5.Internal.INTERNAL_WorkaroundObservableCollectionBugWithJSIL.Clear(this.SelectedItems);
                }
                if (e.NewValue != null && Bridge.is(e.NewValue, System.Int32) && System.Convert.toInt32(e.NewValue) >= 0) {
                    var newValue = System.Convert.toInt32(e.NewValue);
                    var item = this.Items.getItem(newValue);
                    if (this.SelectionMode === Windows.UI.Xaml.Controls.SelectionMode.Single) {
                        CSHTML5.Internal.INTERNAL_WorkaroundObservableCollectionBugWithJSIL.Clear(this.SelectedItems);
                        System.Array.add(this.SelectedItems, item, Object);
                    } else if (!CSHTML5.Internal.INTERNAL_WorkaroundObservableCollectionBugWithJSIL.Contains(this.SelectedItems, item)) {
                        System.Array.add(this.SelectedItems, item, Object);
                    }
                }
            },
            listBoxItem_Click: function (sender, e) {
                var $t;
                var modifiersKey = (0);
                modifiersKey = System.Windows.Input.Keyboard.Modifiers;
                var isControl = System.Enum.hasFlag(modifiersKey, Bridge.box(Windows.System.VirtualKeyModifiers.Control, Windows.System.VirtualKeyModifiers, System.Enum.toStringFn(Windows.System.VirtualKeyModifiers)));
                var isShift = System.Enum.hasFlag(modifiersKey, Bridge.box(Windows.System.VirtualKeyModifiers.Shift, Windows.System.VirtualKeyModifiers, System.Enum.toStringFn(Windows.System.VirtualKeyModifiers)));

                var selectorItem = Bridge.cast(sender, Windows.UI.Xaml.Controls.Primitives.SelectorItem);
                if (this.SelectionMode === Windows.UI.Xaml.Controls.SelectionMode.Single) {
                    CSHTML5.Internal.INTERNAL_WorkaroundObservableCollectionBugWithJSIL.Clear(this.SelectedItems);
                    System.Array.add(this.SelectedItems, selectorItem.INTERNAL_CorrespondingItem, Object);
                } else if (this.SelectionMode === Windows.UI.Xaml.Controls.SelectionMode.Multiple) {
                    if (CSHTML5.Internal.INTERNAL_WorkaroundObservableCollectionBugWithJSIL.Contains(this.SelectedItems, selectorItem.INTERNAL_CorrespondingItem)) {
                        CSHTML5.Internal.INTERNAL_WorkaroundObservableCollectionBugWithJSIL.Remove(this.SelectedItems, selectorItem.INTERNAL_CorrespondingItem);
                    } else {
                        System.Array.add(this.SelectedItems, selectorItem.INTERNAL_CorrespondingItem, Object);
                    }
                } else if (this.SelectionMode === Windows.UI.Xaml.Controls.SelectionMode.Extended) {

                    var indexOfClickedItem = CSHTML5.Internal.INTERNAL_ListsHelper.ConvertToListOfObjectsOrNull(this.Items).indexOf(selectorItem.INTERNAL_CorrespondingItem);

                    if (isShift) {


                        var indexStart = this._indexOfLastClickedItemWithoutShiftKey;
                        var indexEnd = indexOfClickedItem;
                        var change;
                        if (indexStart > indexEnd) {
                            change = indexEnd;
                            indexEnd = indexStart;
                            indexStart = change;
                        }
                        var index = 0;
                        CSHTML5.Internal.INTERNAL_WorkaroundObservableCollectionBugWithJSIL.Clear(this.SelectedItems);
                        $t = Bridge.getEnumerator(this.Items);
                        try {
                            while ($t.moveNext()) {
                                var item = $t.Current;
                                if (indexStart <= index && index <= indexEnd) {
                                    if (!CSHTML5.Internal.INTERNAL_WorkaroundObservableCollectionBugWithJSIL.Contains(this.SelectedItems, item)) {
                                        System.Array.add(this.SelectedItems, item, Object);
                                    }
                                }
                                index = (index + 1) | 0;
                            }
                        } finally {
                            if (Bridge.is($t, System.IDisposable)) {
                                $t.System$IDisposable$Dispose();
                            }
                        }
                    } else if (isControl) {
                        if (CSHTML5.Internal.INTERNAL_WorkaroundObservableCollectionBugWithJSIL.Contains(this.SelectedItems, selectorItem.INTERNAL_CorrespondingItem)) {
                            CSHTML5.Internal.INTERNAL_WorkaroundObservableCollectionBugWithJSIL.Remove(this.SelectedItems, selectorItem.INTERNAL_CorrespondingItem);

                        } else {
                            System.Array.add(this.SelectedItems, selectorItem.INTERNAL_CorrespondingItem, Object);
                        }
                    } else {
                        CSHTML5.Internal.INTERNAL_WorkaroundObservableCollectionBugWithJSIL.Clear(this.SelectedItems);
                        System.Array.add(this.SelectedItems, selectorItem.INTERNAL_CorrespondingItem, Object);
                    }

                    if (!isShift) {
                        this._indexOfLastClickedItemWithoutShiftKey = indexOfClickedItem;
                    }
                } else {
                    throw new System.NotSupportedException.$ctor1("SelectionMode is not Single, Multiple or Extended.");
                }
            },
            UnselectAllItems: function () {
                var $t;
                $t = Bridge.getEnumerator(this.ItemContainerGenerator.INTERNAL_AllContainers, System.Object);
                try {
                    while ($t.moveNext()) {
                        var selectorItem = Bridge.cast($t.Current, Windows.UI.Xaml.Controls.Primitives.SelectorItem);
                        selectorItem.IsSelected = false;
                    }
                } finally {
                    if (Bridge.is($t, System.IDisposable)) {
                        $t.System$IDisposable$Dispose();
                    }
                }
            },
            SetItemVisualSelectionState: function (item, newState) {
                var listBoxItem = Bridge.cast(this.ItemContainerGenerator.ContainerFromItem(item), Windows.UI.Xaml.Controls.ListBoxItem);
                if (listBoxItem != null) {
                    listBoxItem.IsSelected = newState;
                    if (newState) {
                        this.SelectedItem = item;
                    } else {
                        var oldItems = new (System.Collections.Generic.List$1(System.Object)).ctor();
                        oldItems.add(item);
                        var newItems = new (System.Collections.Generic.List$1(System.Object)).ctor();
                        this.OnSelectionChanged(new Windows.UI.Xaml.Controls.SelectionChangedEventArgs(oldItems, newItems));
                    }
                }
            },
            /**
             * @instance
             * @public
             * @this Windows.UI.Xaml.Controls.ListBox
             * @memberof Windows.UI.Xaml.Controls.ListBox
             * @return  {void}
             */
            SelectAll: function () {
                var $t;
                if (this.SelectionMode === Windows.UI.Xaml.Controls.SelectionMode.Single) {
                    throw new System.NotSupportedException.ctor();
                } else {
                    $t = Bridge.getEnumerator(this.ItemContainerGenerator.INTERNAL_AllContainers, System.Object);
                    try {
                        while ($t.moveNext()) {
                            var selectorItem = Bridge.cast($t.Current, Windows.UI.Xaml.Controls.Primitives.SelectorItem);
                            selectorItem.IsSelected = true;
                        }
                    } finally {
                        if (Bridge.is($t, System.IDisposable)) {
                            $t.System$IDisposable$Dispose();
                        }
                    }
                }
            },
            OnApplyTemplate: function () {
                var $t, $t1;
                Windows.UI.Xaml.Controls.Primitives.MultiSelector.prototype.OnApplyTemplate.call(this);
                var l = new (System.Collections.Generic.List$1(System.Object)).ctor();
                $t = Bridge.getEnumerator(this.SelectedItems);
                try {
                    while ($t.moveNext()) {
                        var item = $t.Current;
                        l.add(item);
                    }
                } finally {
                    if (Bridge.is($t, System.IDisposable)) {
                        $t.System$IDisposable$Dispose();
                    }
                }
                $t1 = Bridge.getEnumerator(l);
                try {
                    while ($t1.moveNext()) {
                        var item1 = $t1.Current;
                        this.SetItemVisualSelectionState(item1, true);
                    }
                } finally {
                    if (Bridge.is($t1, System.IDisposable)) {
                        $t1.System$IDisposable$Dispose();
                    }
                }
            }
        }
    });

    /**
     * Represents the container for an item in a ListBox control.
     *
     * @public
     * @class Windows.UI.Xaml.Controls.ListBoxItem
     * @augments Windows.UI.Xaml.Controls.Primitives.SelectorItem
     */
    Bridge.define("Windows.UI.Xaml.Controls.ListBoxItem", {
        inherits: [Windows.UI.Xaml.Controls.Primitives.SelectorItem]
    });

    /**
     * Represents a selectable item inside a Menu.
     *
     * @public
     * @class Windows.UI.Xaml.Controls.MenuItem
     * @augments Windows.UI.Xaml.Controls.Button
     */
    Bridge.define("Windows.UI.Xaml.Controls.MenuItem", {
        inherits: [Windows.UI.Xaml.Controls.Button],
        statics: {
            fields: {
                /**
                 * Identifies the System.Windows.Controls.MenuItem.Icon dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Controls.MenuItem
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                IconProperty: null,
                /**
                 * Identifies the System.Windows.Controls.HeaderedItemsControl.Header dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Controls.MenuItem
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                HeaderProperty: null
            },
            ctors: {
                init: function () {
                    var $t;
                    this.IconProperty = Windows.UI.Xaml.DependencyProperty.Register("Icon", System.Object, Windows.UI.Xaml.Controls.MenuItem, new Windows.UI.Xaml.PropertyMetadata.$ctor4(null));
                    this.HeaderProperty = Windows.UI.Xaml.DependencyProperty.Register("Header", System.Object, Windows.UI.Xaml.Controls.MenuItem, ($t = new Windows.UI.Xaml.PropertyMetadata.$ctor2(null, Windows.UI.Xaml.Controls.MenuItem.Header_Changed), $t.CallPropertyChangedWhenLoadedIntoVisualTree = Windows.UI.Xaml.WhenToCallPropertyChangedEnum.IfPropertyIsSet, $t));
                }
            },
            methods: {
                Header_Changed: function (d, e) {
                    var menuItem = Bridge.cast(d, Windows.UI.Xaml.Controls.MenuItem);
                    var newHeader = e.NewValue;
                    menuItem.Content = newHeader;
                }
            }
        },
        props: {
            /**
             * Gets or sets the icon that appears in a MenuItem.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.MenuItem
             * @function Icon
             * @type System.Object
             */
            Icon: {
                get: function () {
                    return this.GetValue(Windows.UI.Xaml.Controls.MenuItem.IconProperty);
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Controls.MenuItem.IconProperty, value);
                }
            },
            /**
             * Gets or sets the item that labels the control.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.MenuItem
             * @function Header
             * @type System.Object
             */
            Header: {
                get: function () {
                    return this.GetValue(Windows.UI.Xaml.Controls.MenuItem.HeaderProperty);
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Controls.MenuItem.HeaderProperty, value);
                }
            }
        },
        ctors: {
            /**
             * Initializes a new instance of the MenuItem class.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Controls.MenuItem
             * @memberof Windows.UI.Xaml.Controls.MenuItem
             * @return  {void}
             */
            ctor: function () {
                this.$initialize();
                Windows.UI.Xaml.Controls.Button.ctor.call(this);
                this.DefaultStyleKey = Windows.UI.Xaml.Controls.MenuItem;
            }
        }
    });

    /**
     * Represents a button that allows a user to select a single option from a group
      of options.
     *
     * @public
     * @class Windows.UI.Xaml.Controls.RadioButton
     * @augments Windows.UI.Xaml.Controls.Primitives.ToggleButton
     */
    Bridge.define("Windows.UI.Xaml.Controls.RadioButton", {
        inherits: [Windows.UI.Xaml.Controls.Primitives.ToggleButton],
        statics: {
            fields: {
                /**
                 * Identifies the GroupName dependency property.
                 *
                 * @static
                 * @public
                 * @readonly
                 * @memberof Windows.UI.Xaml.Controls.RadioButton
                 * @type Windows.UI.Xaml.DependencyProperty
                 */
                GroupNameProperty: null
            },
            ctors: {
                init: function () {
                    var $t;
                    this.GroupNameProperty = Windows.UI.Xaml.DependencyProperty.Register("GroupName", System.String, Windows.UI.Xaml.Controls.RadioButton, ($t = new Windows.UI.Xaml.PropertyMetadata.$ctor2(null, Windows.UI.Xaml.Controls.RadioButton.GroupName_Changed), $t.CallPropertyChangedWhenLoadedIntoVisualTree = Windows.UI.Xaml.WhenToCallPropertyChangedEnum.IfPropertyIsSet, $t));
                }
            },
            methods: {
                GroupName_Changed: function (d, e) {
                    var radio = Bridge.cast(d, Windows.UI.Xaml.Controls.RadioButton);
                    if (CSHTML5.Internal.INTERNAL_VisualTreeManager.IsElementInVisualTree(radio)) {
                        var isUsingNativeHtml5RadioButtonRatherThanAControlTemplate = (radio.INTERNAL_OptionalSpecifyDomElementConcernedByFocus != null);
                        if (isUsingNativeHtml5RadioButtonRatherThanAControlTemplate) {
                            if (!System.String.isNullOrWhiteSpace(Bridge.cast(e.NewValue, System.String))) {
                                CSHTML5.Internal.INTERNAL_HtmlDomManager.SetDomElementAttribute(radio.INTERNAL_OptionalSpecifyDomElementConcernedByFocus, "name", radio.GroupName, true);
                            } else {
                                CSHTML5.Internal.INTERNAL_HtmlDomManager.SetDomElementAttribute(radio.INTERNAL_OptionalSpecifyDomElementConcernedByFocus, "name", Bridge.cast(radio.INTERNAL_VisualParent, Windows.UI.Xaml.UIElement).INTERNAL_ChildrenRadioButtonDefaultName, true);
                            }
                        }
                    }
                }
            }
        },
        props: {
            /**
             * Gets or sets the name that specifies which RadioButton controls are mutually
             exclusive.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Controls.RadioButton
             * @function GroupName
             * @type string
             */
            GroupName: {
                get: function () {
                    return Bridge.cast(this.GetValue(Windows.UI.Xaml.Controls.RadioButton.GroupNameProperty), System.String);
                },
                set: function (value) {
                    this.SetValue(Windows.UI.Xaml.Controls.RadioButton.GroupNameProperty, value);
                }
            }
        },
        ctors: {
            /**
             * Initializes a new instance of the RadioButton class.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Controls.RadioButton
             * @memberof Windows.UI.Xaml.Controls.RadioButton
             * @return  {void}
             */
            ctor: function () {
                this.$initialize();
                Windows.UI.Xaml.Controls.Primitives.ToggleButton.ctor.call(this);
                this._reactsToKeyboardEventsWhenFocused = false;
            }
        },
        methods: {
            SetDefaultStyle: function () { },
            CreateDomElement: function (parentRef, domElementWhereToPlaceChildren) {
                return Windows.UI.Xaml.Controls.INTERNAL_CheckBoxAndRadioButtonHelpers.CreateDomElement(this, "radio", parentRef, domElementWhereToPlaceChildren);
            },
            SubscribeToClickEventForChildContainerDiv: function (divWhereToPlaceChild, checkBoxDomElement) {
                if (true) {

                    divWhereToPlaceChild.addEventListener('click', function(e) {
    if(checkBoxDomElement.checked === true)
    {
        
    }
    else
    {
        checkBoxDomElement.checked = true;
        var evt = document.createEvent('Event');
        evt.initEvent('change', false, false);
        checkBoxDomElement.dispatchEvent(evt);
    }
}, false);
                }
            },
            ChangedEventReceived: function () {
                var newIsChecked = System.Convert.toBoolean(this.INTERNAL_OptionalSpecifyDomElementConcernedByFocus.checked);
                var oldIsChecked = this.IsChecked == null ? false : System.Nullable.getValue(this.IsChecked);
                if (oldIsChecked !== newIsChecked) {
                    this.IsChecked = newIsChecked;
                }
                if (newIsChecked === true) {
                    this.UnselectOtherRadioButtonsFromSameGroup();
                } else {
                    if (true) {
                        this.INTERNAL_OptionalSpecifyDomElementConcernedByFocus.isLastChecked = undefined;
                    } else {
                        var javaScriptCodeToExecute = System.String.format("document.getElementById(\"{0}\").isLastChecked = undefined;", [this.INTERNAL_OptionalSpecifyDomElementConcernedByFocus.UniqueIdentifier]);
                        CSHTML5.Internal.INTERNAL_HtmlDomManager.ExecuteJavaScriptWithResult(javaScriptCodeToExecute);
                    }
                }
            },
            UnselectOtherRadioButtonsFromSameGroup: function () {
                var groupName = this.GroupName;
                if (System.String.isNullOrWhiteSpace(groupName)) {
                    groupName = Bridge.cast(this.INTERNAL_VisualParent, Windows.UI.Xaml.UIElement).INTERNAL_ChildrenRadioButtonDefaultName;
                }

                if (true) {


var radios = document.getElementsByName(                     groupName );
                        for(var i = 0; i < radios.length; i++ ) {
                            if( radios[i].isLastChecked === true && radios[i] !== this.INTERNAL_OptionalSpecifyDomElementConcernedByFocus) {
                                radios[i].checked = false;
                                radios[i].isLastChecked = undefined;
                                var evt = document.createEvent('Event');
                                evt.initEvent('change', false, false);
                                radios[i].dispatchEvent(evt);
                            }
                        }
                        this.INTERNAL_OptionalSpecifyDomElementConcernedByFocus.isLastChecked = true;
                    
                } else {
                    var javaScriptCodeToExecute = System.String.format("\r\n                        var radios = document.getElementsByName( \"{0}\" );\r\n                        var newlySelectedRadio = document.getElementById(\"{1}\");\r\n                            for(var i = 0; i < radios.length; i++ ) {{\r\n                                if( radios[i].isLastChecked === true  && radios[i] !== newlySelectedRadio) {{\r\n                                    radios[i].checked = false;\r\n                                    radios[i].isLastChecked = undefined;\r\n                                    var evt = document.createEvent('Event');\r\n                                    evt.initEvent('change', false, false);\r\n                                    radios[i].dispatchEvent(evt);\r\n                                }}\r\n                            }}\r\n                            newlySelectedRadio.isLastChecked = true;\r\n                        ", groupName, this.INTERNAL_OptionalSpecifyDomElementConcernedByFocus.UniqueIdentifier);
                    CSHTML5.Internal.INTERNAL_HtmlDomManager.ExecuteJavaScriptWithResult(javaScriptCodeToExecute);
                }
            },
            UpdateDomBasedOnCheckedState: function (isChecked) {
                Windows.UI.Xaml.Controls.INTERNAL_CheckBoxAndRadioButtonHelpers.UpdateDomBasedOnCheckedState(this, isChecked);
var evt = document.createEvent('Event');
                                    evt.initEvent('change', false, false);
                                                    this.INTERNAL_OptionalSpecifyDomElementConcernedByFocus.dispatchEvent(evt);
            },
            INTERNAL_OnAttachedToVisualTree: function () {
                Windows.UI.Xaml.Controls.Primitives.ToggleButton.prototype.INTERNAL_OnAttachedToVisualTree.call(this);

                if (this.HasTemplate) {
                    this.INTERNAL_OptionalSpecifyDomElementConcernedByFocus = this.INTERNAL_OuterDomElement;
                    CSHTML5.Internal.INTERNAL_HtmlDomManager.SetDomElementAttribute(this.INTERNAL_OptionalSpecifyDomElementConcernedByFocus, "checked", Bridge.box(this.IsChecked, System.Boolean, System.Nullable.toStringFn(System.Boolean.toString), System.Nullable.getHashCode), true);
                    Windows.UI.Xaml.Controls.INTERNAL_CheckBoxAndRadioButtonHelpers.SubscribeToBasicEventsForRadioButton(this, this.INTERNAL_OptionalSpecifyDomElementConcernedByFocus, this.INTERNAL_OptionalSpecifyDomElementConcernedByFocus);
                    this.UpdateDomBasedOnCheckedState(this.IsChecked);
                }

                if (System.String.isNullOrWhiteSpace(this.GroupName)) {
                    CSHTML5.Internal.INTERNAL_HtmlDomManager.SetDomElementAttribute(this.INTERNAL_OptionalSpecifyDomElementConcernedByFocus, "name", Bridge.cast(this.INTERNAL_VisualParent, Windows.UI.Xaml.UIElement).INTERNAL_ChildrenRadioButtonDefaultName, true);
                }
            },
            OnApplyTemplate: function () {
                Windows.UI.Xaml.Controls.Primitives.ToggleButton.prototype.OnApplyTemplate.call(this);
                this.UnregisterFromDefaultClickEvent();
            }
        }
    });

    /**
     * Provides an inline-level content element that causes content to render with a bold font weight.
     *
     * @public
     * @class Windows.UI.Xaml.Documents.Bold
     * @augments Windows.UI.Xaml.Documents.Span
     */
    Bridge.define("Windows.UI.Xaml.Documents.Bold", {
        inherits: [Windows.UI.Xaml.Documents.Span],
        ctors: {
            /**
             * Initializes a new instance of the Bold class.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Documents.Bold
             * @memberof Windows.UI.Xaml.Documents.Bold
             * @return  {void}
             */
            ctor: function () {
                this.$initialize();
                Windows.UI.Xaml.Documents.Span.ctor.call(this);
                this.FontWeight = Windows.UI.Text.FontWeights.Bold.$clone();
            }
        }
    });

    /**
     * Provides an inline-level content element that provides facilities for hosting hyperlinks.
     *
     * @public
     * @class Windows.UI.Xaml.Documents.Hyperlink
     * @augments Windows.UI.Xaml.Documents.Span
     */
    Bridge.define("Windows.UI.Xaml.Documents.Hyperlink", {
        inherits: [Windows.UI.Xaml.Documents.Span],
        fields: {
            _uri: null
        },
        events: {
            /**
             * Occurs when the left mouse button is clicked on a Hyperlink.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Documents.Hyperlink
             * @memberof Windows.UI.Xaml.Documents.Hyperlink
             * @function addClick
             * @param   {Windows.UI.Xaml.RoutedEventHandler}    value
             * @return  {void}
             */
            /**
             * Occurs when the left mouse button is clicked on a Hyperlink.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Documents.Hyperlink
             * @memberof Windows.UI.Xaml.Documents.Hyperlink
             * @function removeClick
             * @param   {Windows.UI.Xaml.RoutedEventHandler}    value
             * @return  {void}
             */
            Click: null
        },
        props: {
            /**
             * Gets or sets a URI to navigate to when the Hyperlink
             is activated. The default is null.
             *
             * @instance
             * @public
             * @memberof Windows.UI.Xaml.Documents.Hyperlink
             * @function NavigateUri
             * @type System.Uri
             */
            NavigateUri: {
                get: function () {
                    return this._uri;
                },
                set: function (value) {
                    this._uri = value;
                }
            }
        },
        ctors: {
            /**
             * Initializes a new instance of the Hyperlink class.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Documents.Hyperlink
             * @memberof Windows.UI.Xaml.Documents.Hyperlink
             * @return  {void}
             */
            ctor: function () {
                this.$initialize();
                Windows.UI.Xaml.Documents.Span.ctor.call(this);
                this.Foreground = new Windows.UI.Xaml.Media.SolidColorBrush.$ctor1(Windows.UI.Color.FromArgb(245, 26, 13, 171));
                this.Cursor = System.Windows.Input.Cursors.Hand;

                this.addPointerPressed(Bridge.fn.cacheBind(this, this.Hyperlink_PointerPressed));
                this.TextDecorations$1 = Windows.UI.Text.TextDecorations.Underline;
            }
        },
        methods: {
            Hyperlink_PointerPressed: function (sender, e) {
                if (!Bridge.staticEquals(this.Click, null)) {
                    this.Click(this, new Windows.UI.Xaml.RoutedEventArgs());
                }

                if (System.Uri.notEquals(this._uri, null)) {
                    System.Windows.Browser.HtmlPage.Window.Navigate(this._uri, "_blank");
                }
            },
            CreateDomElement: function (parentRef, domElementWhereToPlaceChildren) {
                var a = CSHTML5.Internal.INTERNAL_HtmlDomManager.CreateDomElementAndAppendIt("a", parentRef, this);
                domElementWhereToPlaceChildren.v = a;
                return a;
            }
        }
    });

    /**
     * Provides an inline-level content element that causes content to render with a bold font weight.
     *
     * @public
     * @class Windows.UI.Xaml.Documents.Italic
     * @augments Windows.UI.Xaml.Documents.Span
     */
    Bridge.define("Windows.UI.Xaml.Documents.Italic", {
        inherits: [Windows.UI.Xaml.Documents.Span],
        ctors: {
            /**
             * Initializes a new instance of the Italic class.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Documents.Italic
             * @memberof Windows.UI.Xaml.Documents.Italic
             * @return  {void}
             */
            ctor: function () {
                this.$initialize();
                Windows.UI.Xaml.Documents.Span.ctor.call(this);
                this.FontStyle = Windows.UI.Text.FontStyle.Italic;
            }
        }
    });

    /**
     * Provides an inline-level content element that causes content to render with a bold font weight.
     *
     * @public
     * @class Windows.UI.Xaml.Documents.Underline
     * @augments Windows.UI.Xaml.Documents.Span
     */
    Bridge.define("Windows.UI.Xaml.Documents.Underline", {
        inherits: [Windows.UI.Xaml.Documents.Span],
        ctors: {
            /**
             * Initializes a new instance of the Underline class.
             *
             * @instance
             * @public
             * @this Windows.UI.Xaml.Documents.Underline
             * @memberof Windows.UI.Xaml.Documents.Underline
             * @return  {void}
             */
            ctor: function () {
                this.$initialize();
                Windows.UI.Xaml.Documents.Span.ctor.call(this);
                this.TextDecorations$1 = Windows.UI.Text.TextDecorations.Underline;
            }
        }
    });
});
